
##############################################################################
#
# Copyright (c) 2003-2018 by The University of Queensland
# http://www.uq.edu.au
#
# Primary Business: Queensland, Australia
# Licensed under the Apache License, version 2.0
# http://www.apache.org/licenses/LICENSE-2.0
#
# Development until 2012 by Earth Systems Science Computational Center (ESSCC)
# Development 2012-2013 by School of Earth Sciences
# Development from 2014 by Centre for Geoscience Computing (GeoComp)
#
##############################################################################

from __future__ import print_function, division

__copyright__="""Copyright (c) 2003-2018 by The University of Queensland
http://www.uq.edu.au
Primary Business: Queensland, Australia"""
__license__="""Licensed under the Apache License, version 2.0
http://www.apache.org/licenses/LICENSE-2.0"""
__url__="https://launchpad.net/escript-finley"

"""
test for util operations for slicing operations with tagged data

:remark: use see `test_util`
:var __author__: name of author
:var __copyright__: copyrights
:var __license__: licence agreement
:var __url__: url entry point on documentation
:var __version__: version
:var __date__: date of the version
"""

__author__="Lutz Gross, l.gross@uq.edu.au"

import esys.escriptcore.utestselect as unittest
import numpy
from esys.escript import *
from test_util_base import Test_util_base

class Test_util_slicing_with_tagged_data(Test_util_base):
   """
   test for slicing operations for tagged data are tested.
   """
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank1_U(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([-0.0, -1.0, -2.0]))
      res=arg[:2]
      ref=Data(numpy.array([0.0, 1.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-0.0, -1.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank1_L(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([-0.0, -1.0, -2.0]))
      res=arg[2:]
      ref=Data(numpy.array([2.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-2.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank1_R(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([-0.0, -1.0, -2.0]))
      res=arg[1:2]
      ref=Data(numpy.array([1.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-1.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank1_P(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([-0.0, -1.0, -2.0]))
      res=arg[1]
      ref=Data(1.0,self.functionspace)
      ref.setTaggedValue(1,-1.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank1_C(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([-0.0, -1.0, -2.0]))
      res=arg[:]
      ref=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-0.0, -1.0, -2.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_CP(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:,2]
      ref=Data(numpy.array([2.0, 6.0, 10.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-2.0, -6.0, -10.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_CU(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:,:3]
      ref=Data(numpy.array([[0.0, 1.0, 2.0], [4.0, 5.0, 6.0], [8.0, 9.0, 10.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0], [-4.0, -5.0, -6.0], [-8.0, -9.0, -10.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_UR(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:2,1:3]
      ref=Data(numpy.array([[1.0, 2.0], [5.0, 6.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-1.0, -2.0], [-5.0, -6.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_RR(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[1:2,1:3]
      ref=Data(numpy.array([[5.0, 6.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-5.0, -6.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_PU(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[1,:3]
      ref=Data(numpy.array([4.0, 5.0, 6.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-4.0, -5.0, -6.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_UL(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:2,2:]
      ref=Data(numpy.array([[2.0, 3.0], [6.0, 7.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-2.0, -3.0], [-6.0, -7.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_CL(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:,2:]
      ref=Data(numpy.array([[2.0, 3.0], [6.0, 7.0], [10.0, 11.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-2.0, -3.0], [-6.0, -7.0], [-10.0, -11.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_LP(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[2:,2]
      ref=Data(numpy.array([10.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-10.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_LR(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[2:,1:3]
      ref=Data(numpy.array([[9.0, 10.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-9.0, -10.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_UN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:2]
      ref=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_UU(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:2,:3]
      ref=Data(numpy.array([[0.0, 1.0, 2.0], [4.0, 5.0, 6.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0], [-4.0, -5.0, -6.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_RP(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[1:2,2]
      ref=Data(numpy.array([6.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-6.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_CC(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:,:]
      ref=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_RC(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[1:2,:]
      ref=Data(numpy.array([[4.0, 5.0, 6.0, 7.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-4.0, -5.0, -6.0, -7.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_LL(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[2:,2:]
      ref=Data(numpy.array([[10.0, 11.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-10.0, -11.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_RN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[1:2]
      ref=Data(numpy.array([[4.0, 5.0, 6.0, 7.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-4.0, -5.0, -6.0, -7.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_CN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[:]
      ref=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_PN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[1]
      ref=Data(numpy.array([4.0, 5.0, 6.0, 7.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-4.0, -5.0, -6.0, -7.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_PP(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[1,2]
      ref=Data(6.0,self.functionspace)
      ref.setTaggedValue(1,-6.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank2_LN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]]))
      res=arg[2:]
      ref=Data(numpy.array([[8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-8.0, -9.0, -10.0, -11.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_URP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,1:3,1]
      ref=Data(numpy.array([[4.0, 7.0], [16.0, 19.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-4.0, -7.0], [-16.0, -19.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,:]
      ref=Data(numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, 
-35.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LCR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,:,1:2]
      ref=Data(numpy.array([[[25.0], [28.0], [31.0], [34.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-25.0], [-28.0], [-31.0], [-34.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_URU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,1:3,:2]
      ref=Data(numpy.array([[[3.0, 4.0], [6.0, 7.0]], [[15.0, 16.0], [18.0, 19.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-3.0, -4.0], [-6.0, -7.0]], [[-15.0, -16.0], [-18.0, -19.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RCR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,:,1:2]
      ref=Data(numpy.array([[[13.0], [16.0], [19.0], [22.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-13.0], [-16.0], [-19.0], [-22.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_UCP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,:,1]
      ref=Data(numpy.array([[1.0, 4.0, 7.0, 10.0], [13.0, 16.0, 19.0, 22.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-1.0, -4.0, -7.0, -10.0], [-13.0, -16.0, -19.0, -22.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LLN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,2:]
      ref=Data(numpy.array([[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RPP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,2,1]
      ref=Data(numpy.array([19.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-19.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LPL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,2,2:]
      ref=Data(numpy.array([[32.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-32.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CCR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,:,1:2]
      ref=Data(numpy.array([[[1.0], [4.0], [7.0], [10.0]], [[13.0], [16.0], [19.0], [22.0]], [[25.0], [28.0], [31.0], 
[34.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-1.0], [-4.0], [-7.0], [-10.0]], [[-13.0], [-16.0], [-19.0], [-22.0]], [[-25.0], 
[-28.0], [-31.0], [-34.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LUL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,:3,2:]
      ref=Data(numpy.array([[[26.0], [29.0], [32.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-26.0], [-29.0], [-32.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PPU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,2,:2]
      ref=Data(numpy.array([18.0, 19.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-18.0, -19.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LCP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,:,1]
      ref=Data(numpy.array([[25.0, 28.0, 31.0, 34.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-25.0, -28.0, -31.0, -34.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RUR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,:3,1:2]
      ref=Data(numpy.array([[[13.0], [16.0], [19.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-13.0], [-16.0], [-19.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CUC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,:3,:]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-12.0, -13.0, -14.0], 
[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]], [[-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LRU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,1:3,:2]
      ref=Data(numpy.array([[[27.0, 28.0], [30.0, 31.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-27.0, -28.0], [-30.0, -31.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LRR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,1:3,1:2]
      ref=Data(numpy.array([[[28.0], [31.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-28.0], [-31.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_UUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,:3]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-12.0, -13.0, -14.0], 
[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CUL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,:3,2:]
      ref=Data(numpy.array([[[2.0], [5.0], [8.0]], [[14.0], [17.0], [20.0]], [[26.0], [29.0], [32.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-2.0], [-5.0], [-8.0]], [[-14.0], [-17.0], [-20.0]], [[-26.0], [-29.0], [-32.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LUP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,:3,1]
      ref=Data(numpy.array([[25.0, 28.0, 31.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-25.0, -28.0, -31.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CCL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,:,2:]
      ref=Data(numpy.array([[[2.0], [5.0], [8.0], [11.0]], [[14.0], [17.0], [20.0], [23.0]], [[26.0], [29.0], [32.0], 
[35.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-2.0], [-5.0], [-8.0], [-11.0]], [[-14.0], [-17.0], [-20.0], [-23.0]], [[-26.0], 
[-29.0], [-32.0], [-35.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_UCR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,:,1:2]
      ref=Data(numpy.array([[[1.0], [4.0], [7.0], [10.0]], [[13.0], [16.0], [19.0], [22.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-1.0], [-4.0], [-7.0], [-10.0]], [[-13.0], [-16.0], [-19.0], [-22.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LCU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,:,:2]
      ref=Data(numpy.array([[[24.0, 25.0], [27.0, 28.0], [30.0, 31.0], [33.0, 34.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-24.0, -25.0], [-27.0, -28.0], [-30.0, -31.0], [-33.0, -34.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CCP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,:,1]
      ref=Data(numpy.array([[1.0, 4.0, 7.0, 10.0], [13.0, 16.0, 19.0, 22.0], [25.0, 28.0, 31.0, 34.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-1.0, -4.0, -7.0, -10.0], [-13.0, -16.0, -19.0, -22.0], [-25.0, -28.0, -31.0, -34.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LPP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,2,1]
      ref=Data(numpy.array([31.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-31.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_ULN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,2:]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,2]
      ref=Data(numpy.array([18.0, 19.0, 20.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-18.0, -19.0, -20.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,:]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RLP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,2:,1]
      ref=Data(numpy.array([[19.0, 22.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-19.0, -22.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PLU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,2:,:2]
      ref=Data(numpy.array([[18.0, 19.0], [21.0, 22.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-18.0, -19.0], [-21.0, -22.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RUC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,:3,:]
      ref=Data(numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PPP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,2,1]
      ref=Data(19.0,self.functionspace)
      ref.setTaggedValue(1,-19.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CUU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,:3,:2]
      ref=Data(numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]], [[24.0, 25.0], 
[27.0, 28.0], [30.0, 31.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-0.0, -1.0], [-3.0, -4.0], [-6.0, -7.0]], [[-12.0, -13.0], [-15.0, -16.0], [-18.0, 
-19.0]], [[-24.0, -25.0], [-27.0, -28.0], [-30.0, -31.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,:3]
      ref=Data(numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LLC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,2:,:]
      ref=Data(numpy.array([[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_UUL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,:3,2:]
      ref=Data(numpy.array([[[2.0], [5.0], [8.0]], [[14.0], [17.0], [20.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-2.0], [-5.0], [-8.0]], [[-14.0], [-17.0], [-20.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,2]
      ref=Data(numpy.array([[30.0, 31.0, 32.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-30.0, -31.0, -32.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,2]
      ref=Data(numpy.array([[18.0, 19.0, 20.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-18.0, -19.0, -20.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RCP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,:,1]
      ref=Data(numpy.array([[13.0, 16.0, 19.0, 22.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-13.0, -16.0, -19.0, -22.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_UCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,:]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LRL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,1:3,2:]
      ref=Data(numpy.array([[[29.0], [32.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-29.0], [-32.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PUR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,:3,1:2]
      ref=Data(numpy.array([[13.0], [16.0], [19.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-13.0], [-16.0], [-19.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RLC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,2:,:]
      ref=Data(numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PRC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,1:3,:]
      ref=Data(numpy.array([[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,2]
      ref=Data(numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0], [30.0, 31.0, 32.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-6.0, -7.0, -8.0], [-18.0, -19.0, -20.0], [-30.0, -31.0, -32.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CLN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,2:]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[30.0, 31.0, 
32.0], [33.0, 34.0, 35.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RRN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,1:3]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,:3]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-12.0, -13.0, -14.0], 
[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]], [[-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_UUP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,:3,1]
      ref=Data(numpy.array([[1.0, 4.0, 7.0], [13.0, 16.0, 19.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-1.0, -4.0, -7.0], [-13.0, -16.0, -19.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_URN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,1:3]
      ref=Data(numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, 
-20.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RRP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,1:3,1]
      ref=Data(numpy.array([[16.0, 19.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-16.0, -19.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PRN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,1:3]
      ref=Data(numpy.array([[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RLN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,2:]
      ref=Data(numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PRL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,1:3,2:]
      ref=Data(numpy.array([[17.0], [20.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-17.0], [-20.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PLC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,2:,:]
      ref=Data(numpy.array([[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_UPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,2]
      ref=Data(numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-6.0, -7.0, -8.0], [-18.0, -19.0, -20.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_ULR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:2,2:,1:2]
      ref=Data(numpy.array([[[7.0], [10.0]], [[19.0], [22.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-7.0], [-10.0]], [[-19.0], [-22.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PUC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,:3,:]
      ref=Data(numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CPP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,2,1]
      ref=Data(numpy.array([7.0, 19.0, 31.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-7.0, -19.0, -31.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_LRN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[2:,1:3]
      ref=Data(numpy.array([[[27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,:]
      ref=Data(numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,:]
      ref=Data(numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 
23.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_RUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1:2,:3]
      ref=Data(numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_CRN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[:,1:3]
      ref=Data(numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]], [[27.0, 28.0, 29.0], 
[30.0, 31.0, 32.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, 
-20.0]], [[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PLN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,2:]
      ref=Data(numpy.array([[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank3_PUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]]))
      res=arg[1,:3]
      ref=Data(numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UCCL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:,:,2:]
      ref=Data(numpy.array([[[[2.0], [5.0], [8.0], [11.0], [14.0]], [[17.0], [20.0], [23.0], [26.0], [29.0]], [[32.0], [35.0], 
[38.0], [41.0], [44.0]]], [[[47.0], [50.0], [53.0], [56.0], [59.0]], [[62.0], [65.0], [68.0], [71.0], [74.0]], [[77.0], [80.0], 
[83.0], [86.0], [89.0]]], [[[92.0], [95.0], [98.0], [101.0], [104.0]], [[107.0], [110.0], [113.0], [116.0], [119.0]], [[122.0], 
[125.0], [128.0], [131.0], [134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-2.0], [-5.0], [-8.0], [-11.0], [-14.0]], [[-17.0], [-20.0], [-23.0], [-26.0], 
[-29.0]], [[-32.0], [-35.0], [-38.0], [-41.0], [-44.0]]], [[[-47.0], [-50.0], [-53.0], [-56.0], [-59.0]], [[-62.0], [-65.0], 
[-68.0], [-71.0], [-74.0]], [[-77.0], [-80.0], [-83.0], [-86.0], [-89.0]]], [[[-92.0], [-95.0], [-98.0], [-101.0], [-104.0]], 
[[-107.0], [-110.0], [-113.0], [-116.0], [-119.0]], [[-122.0], [-125.0], [-128.0], [-131.0], [-134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LPCU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1,:,:2]
      ref=Data(numpy.array([[[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 118.0]], [[150.0, 151.0], 
[153.0, 154.0], [156.0, 157.0], [159.0, 160.0], [162.0, 163.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0], [-117.0, 
-118.0]], [[-150.0, -151.0], [-153.0, -154.0], [-156.0, -157.0], [-159.0, -160.0], [-162.0, -163.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_URCR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1:2,:,1:2]
      ref=Data(numpy.array([[[[16.0], [19.0], [22.0], [25.0], [28.0]]], [[[61.0], [64.0], [67.0], [70.0], [73.0]]], [[[106.0], 
[109.0], [112.0], [115.0], [118.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-16.0], [-19.0], [-22.0], [-25.0], [-28.0]]], [[[-61.0], [-64.0], [-67.0], [-70.0], 
[-73.0]]], [[[-106.0], [-109.0], [-112.0], [-115.0], [-118.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,:4,:2]
      ref=Data(numpy.array([[[[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]], [[[150.0, 151.0], [153.0, 
154.0], [156.0, 157.0], [159.0, 160.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]]], [[[-150.0, 
-151.0], [-153.0, -154.0], [-156.0, -157.0], [-159.0, -160.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CUUL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:2,:4,2:]
      ref=Data(numpy.array([[[[2.0], [5.0], [8.0], [11.0]], [[17.0], [20.0], [23.0], [26.0]]], [[[47.0], [50.0], [53.0], 
[56.0]], [[62.0], [65.0], [68.0], [71.0]]], [[[92.0], [95.0], [98.0], [101.0]], [[107.0], [110.0], [113.0], [116.0]]], 
[[[137.0], [140.0], [143.0], [146.0]], [[152.0], [155.0], [158.0], [161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-2.0], [-5.0], [-8.0], [-11.0]], [[-17.0], [-20.0], [-23.0], [-26.0]]], [[[-47.0], 
[-50.0], [-53.0], [-56.0]], [[-62.0], [-65.0], [-68.0], [-71.0]]], [[[-92.0], [-95.0], [-98.0], [-101.0]], [[-107.0], [-110.0], 
[-113.0], [-116.0]]], [[[-137.0], [-140.0], [-143.0], [-146.0]], [[-152.0], [-155.0], [-158.0], [-161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCPC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,2,:]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], 
[81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 
158.0], [171.0, 172.0, 173.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-6.0, -7.0, -8.0], [-21.0, -22.0, -23.0], [-36.0, -37.0, -38.0]], [[-51.0, -52.0, 
-53.0], [-66.0, -67.0, -68.0], [-81.0, -82.0, -83.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, 
-128.0]], [[-141.0, -142.0, -143.0], [-156.0, -157.0, -158.0], [-171.0, -172.0, -173.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RLCP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,2:,:,1]
      ref=Data(numpy.array([[[76.0, 79.0, 82.0, 85.0, 88.0]], [[121.0, 124.0, 127.0, 130.0, 133.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-76.0, -79.0, -82.0, -85.0, -88.0]], [[-121.0, -124.0, -127.0, -130.0, -133.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRRC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,1:4,:]
      ref=Data(numpy.array([[[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], 
[[[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLCP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,:,1]
      ref=Data(numpy.array([[[121.0, 124.0, 127.0, 130.0, 133.0]], [[166.0, 169.0, 172.0, 175.0, 178.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-121.0, -124.0, -127.0, -130.0, -133.0]], [[-166.0, -169.0, -172.0, -175.0, 
-178.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PRCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1:2,:]
      ref=Data(numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RPRC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1,1:4,:]
      ref=Data(numpy.array([[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_ULCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,2:,:]
      ref=Data(numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 
44.0]]], [[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, 
-41.0], [-42.0, -43.0, -44.0]]], [[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], 
[-87.0, -88.0, -89.0]]], [[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0], [-132.0, -133.0, -134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CLRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,2:,1:4]
      ref=Data(numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 170.0], 
[171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-78.0, 
-79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]], [[[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRRL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,1:4,2:]
      ref=Data(numpy.array([[[[65.0], [68.0], [71.0]]], [[[110.0], [113.0], [116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-65.0], [-68.0], [-71.0]]], [[[-110.0], [-113.0], [-116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLPL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,2,2:]
      ref=Data(numpy.array([[128.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-128.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,2]
      ref=Data(numpy.array([[[126.0, 127.0, 128.0]], [[171.0, 172.0, 173.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-126.0, -127.0, -128.0]], [[-171.0, -172.0, -173.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PCRC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:,1:4,:]
      ref=Data(numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CPRP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1,1:4,1]
      ref=Data(numpy.array([[19.0, 22.0, 25.0], [64.0, 67.0, 70.0], [109.0, 112.0, 115.0], [154.0, 157.0, 
160.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-19.0, -22.0, -25.0], [-64.0, -67.0, -70.0], [-109.0, -112.0, -115.0], [-154.0, 
-157.0, -160.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CUPU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:2,2,:2]
      ref=Data(numpy.array([[[6.0, 7.0], [21.0, 22.0]], [[51.0, 52.0], [66.0, 67.0]], [[96.0, 97.0], [111.0, 112.0]], [[141.0, 
142.0], [156.0, 157.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-6.0, -7.0], [-21.0, -22.0]], [[-51.0, -52.0], [-66.0, -67.0]], [[-96.0, -97.0], 
[-111.0, -112.0]], [[-141.0, -142.0], [-156.0, -157.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UUUL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,:4,2:]
      ref=Data(numpy.array([[[[2.0], [5.0], [8.0], [11.0]], [[17.0], [20.0], [23.0], [26.0]]], [[[47.0], [50.0], [53.0], 
[56.0]], [[62.0], [65.0], [68.0], [71.0]]], [[[92.0], [95.0], [98.0], [101.0]], [[107.0], [110.0], [113.0], 
[116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-2.0], [-5.0], [-8.0], [-11.0]], [[-17.0], [-20.0], [-23.0], [-26.0]]], [[[-47.0], 
[-50.0], [-53.0], [-56.0]], [[-62.0], [-65.0], [-68.0], [-71.0]]], [[[-92.0], [-95.0], [-98.0], [-101.0]], [[-107.0], [-110.0], 
[-113.0], [-116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RUUC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,:4,:]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 
98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, 
-110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCLL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,2:,2:]
      ref=Data(numpy.array([[[[8.0], [11.0], [14.0]], [[23.0], [26.0], [29.0]], [[38.0], [41.0], [44.0]]], [[[53.0], [56.0], 
[59.0]], [[68.0], [71.0], [74.0]], [[83.0], [86.0], [89.0]]], [[[98.0], [101.0], [104.0]], [[113.0], [116.0], [119.0]], 
[[128.0], [131.0], [134.0]]], [[[143.0], [146.0], [149.0]], [[158.0], [161.0], [164.0]], [[173.0], [176.0], 
[179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-8.0], [-11.0], [-14.0]], [[-23.0], [-26.0], [-29.0]], [[-38.0], [-41.0], [-44.0]]], 
[[[-53.0], [-56.0], [-59.0]], [[-68.0], [-71.0], [-74.0]], [[-83.0], [-86.0], [-89.0]]], [[[-98.0], [-101.0], [-104.0]], 
[[-113.0], [-116.0], [-119.0]], [[-128.0], [-131.0], [-134.0]]], [[[-143.0], [-146.0], [-149.0]], [[-158.0], [-161.0], 
[-164.0]], [[-173.0], [-176.0], [-179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRLC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,2:,:]
      ref=Data(numpy.array([[[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]], 
[[[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,2:]
      ref=Data(numpy.array([[[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,:4,:2]
      ref=Data(numpy.array([[[120.0, 121.0], [123.0, 124.0], [126.0, 127.0], [129.0, 130.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-120.0, -121.0], [-123.0, -124.0], [-126.0, -127.0], [-129.0, -130.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPUR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,:4,1:2]
      ref=Data(numpy.array([[106.0], [109.0], [112.0], [115.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-106.0], [-109.0], [-112.0], [-115.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CRCU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1:2,:,:2]
      ref=Data(numpy.array([[[[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 25.0], [27.0, 28.0]]], [[[60.0, 61.0], [63.0, 
64.0], [66.0, 67.0], [69.0, 70.0], [72.0, 73.0]]], [[[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 
118.0]]], [[[150.0, 151.0], [153.0, 154.0], [156.0, 157.0], [159.0, 160.0], [162.0, 163.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-15.0, -16.0], [-18.0, -19.0], [-21.0, -22.0], [-24.0, -25.0], [-27.0, -28.0]]], 
[[[-60.0, -61.0], [-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0], [-72.0, -73.0]]], [[[-105.0, -106.0], [-108.0, -109.0], 
[-111.0, -112.0], [-114.0, -115.0], [-117.0, -118.0]]], [[[-150.0, -151.0], [-153.0, -154.0], [-156.0, -157.0], [-159.0, 
-160.0], [-162.0, -163.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RUPL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,2,2:]
      ref=Data(numpy.array([[[53.0], [68.0]], [[98.0], [113.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-53.0], [-68.0]], [[-98.0], [-113.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLPU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,2,:2]
      ref=Data(numpy.array([[126.0, 127.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-126.0, -127.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLRL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,1:4,2:]
      ref=Data(numpy.array([[[125.0], [128.0], [131.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-125.0], [-128.0], [-131.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1,2]
      ref=Data(numpy.array([[66.0, 67.0, 68.0], [111.0, 112.0, 113.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-66.0, -67.0, -68.0], [-111.0, -112.0, -113.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1,:]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,:4]
      ref=Data(numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], 
[[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]], [[[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, 
-176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPLC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,2:,:]
      ref=Data(numpy.array([[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PRRP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1:2,1:4,1]
      ref=Data(numpy.array([[109.0, 112.0, 115.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-109.0, -112.0, -115.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPPC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,2,:]
      ref=Data(numpy.array([111.0, 112.0, 113.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-111.0, -112.0, -113.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RURL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,1:4,2:]
      ref=Data(numpy.array([[[[50.0], [53.0], [56.0]], [[65.0], [68.0], [71.0]]], [[[95.0], [98.0], [101.0]], [[110.0], 
[113.0], [116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-50.0], [-53.0], [-56.0]], [[-65.0], [-68.0], [-71.0]]], [[[-95.0], [-98.0], 
[-101.0]], [[-110.0], [-113.0], [-116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PUUR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:2,:4,1:2]
      ref=Data(numpy.array([[[91.0], [94.0], [97.0], [100.0]], [[106.0], [109.0], [112.0], [115.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-91.0], [-94.0], [-97.0], [-100.0]], [[-106.0], [-109.0], [-112.0], [-115.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RUCL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,:,2:]
      ref=Data(numpy.array([[[[47.0], [50.0], [53.0], [56.0], [59.0]], [[62.0], [65.0], [68.0], [71.0], [74.0]]], [[[92.0], 
[95.0], [98.0], [101.0], [104.0]], [[107.0], [110.0], [113.0], [116.0], [119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-47.0], [-50.0], [-53.0], [-56.0], [-59.0]], [[-62.0], [-65.0], [-68.0], [-71.0], 
[-74.0]]], [[[-92.0], [-95.0], [-98.0], [-101.0], [-104.0]], [[-107.0], [-110.0], [-113.0], [-116.0], [-119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCPP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,2,1]
      ref=Data(numpy.array([[7.0, 22.0, 37.0], [52.0, 67.0, 82.0], [97.0, 112.0, 127.0], [142.0, 157.0, 
172.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-7.0, -22.0, -37.0], [-52.0, -67.0, -82.0], [-97.0, -112.0, -127.0], [-142.0, -157.0, 
-172.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRPL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,2,2:]
      ref=Data(numpy.array([[[68.0]], [[113.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-68.0]], [[-113.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LULP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:2,2:,1]
      ref=Data(numpy.array([[[97.0, 100.0, 103.0], [112.0, 115.0, 118.0]], [[142.0, 145.0, 148.0], [157.0, 160.0, 
163.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-97.0, -100.0, -103.0], [-112.0, -115.0, -118.0]], [[-142.0, -145.0, -148.0], 
[-157.0, -160.0, -163.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLLL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,2:,2:]
      ref=Data(numpy.array([[[128.0], [131.0], [134.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-128.0], [-131.0], [-134.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPPR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,2,1:2]
      ref=Data(numpy.array([112.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-112.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,:]
      ref=Data(numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCUC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,:4,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 
145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-30.0, -31.0, -32.0], [-33.0, 
-34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, 
-53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], 
[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-90.0, -91.0, -92.0], [-93.0, 
-94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, 
-146.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]], [[-165.0, 
-166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRLL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,2:,2:]
      ref=Data(numpy.array([[[[113.0], [116.0], [119.0]]], [[[158.0], [161.0], [164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-113.0], [-116.0], [-119.0]]], [[[-158.0], [-161.0], [-164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CRRR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1:2,1:4,1:2]
      ref=Data(numpy.array([[[[19.0], [22.0], [25.0]]], [[[64.0], [67.0], [70.0]]], [[[109.0], [112.0], [115.0]]], [[[154.0], 
[157.0], [160.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-19.0], [-22.0], [-25.0]]], [[[-64.0], [-67.0], [-70.0]]], [[[-109.0], [-112.0], 
[-115.0]]], [[[-154.0], [-157.0], [-160.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CRCR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1:2,:,1:2]
      ref=Data(numpy.array([[[[16.0], [19.0], [22.0], [25.0], [28.0]]], [[[61.0], [64.0], [67.0], [70.0], [73.0]]], [[[106.0], 
[109.0], [112.0], [115.0], [118.0]]], [[[151.0], [154.0], [157.0], [160.0], [163.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-16.0], [-19.0], [-22.0], [-25.0], [-28.0]]], [[[-61.0], [-64.0], [-67.0], [-70.0], 
[-73.0]]], [[[-106.0], [-109.0], [-112.0], [-115.0], [-118.0]]], [[[-151.0], [-154.0], [-157.0], [-160.0], [-163.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:,1:4]
      ref=Data(numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 
98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-63.0, -64.0, 
-65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], 
[[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PUCP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:2,:,1]
      ref=Data(numpy.array([[91.0, 94.0, 97.0, 100.0, 103.0], [106.0, 109.0, 112.0, 115.0, 118.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-91.0, -94.0, -97.0, -100.0, -103.0], [-106.0, -109.0, -112.0, -115.0, -118.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:2,:4]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], 
[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-45.0, -46.0, -47.0], [-48.0, 
-49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-135.0, 
-136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-150.0, -151.0, -152.0], 
[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1,:]
      ref=Data(numpy.array([[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 
74.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, 
-71.0], [-72.0, -73.0, -74.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LPRP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1,1:4,1]
      ref=Data(numpy.array([[109.0, 112.0, 115.0], [154.0, 157.0, 160.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-109.0, -112.0, -115.0], [-154.0, -157.0, -160.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,:4]
      ref=Data(numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:,2]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], 
[81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-6.0, -7.0, -8.0], [-21.0, -22.0, -23.0], [-36.0, -37.0, -38.0]], [[-51.0, -52.0, 
-53.0], [-66.0, -67.0, -68.0], [-81.0, -82.0, -83.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, 
-128.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_URCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1:2,:]
      ref=Data(numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]]], [[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]]], [[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]]], [[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PCCL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:,:,2:]
      ref=Data(numpy.array([[[92.0], [95.0], [98.0], [101.0], [104.0]], [[107.0], [110.0], [113.0], [116.0], [119.0]], 
[[122.0], [125.0], [128.0], [131.0], [134.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-92.0], [-95.0], [-98.0], [-101.0], [-104.0]], [[-107.0], [-110.0], [-113.0], 
[-116.0], [-119.0]], [[-122.0], [-125.0], [-128.0], [-131.0], [-134.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,:4]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 
98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, 
-110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLRU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,1:4,:2]
      ref=Data(numpy.array([[[[123.0, 124.0], [126.0, 127.0], [129.0, 130.0]]], [[[168.0, 169.0], [171.0, 172.0], [174.0, 
175.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-123.0, -124.0], [-126.0, -127.0], [-129.0, -130.0]]], [[[-168.0, -169.0], [-171.0, 
-172.0], [-174.0, -175.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_URUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1:2,:4,:2]
      ref=Data(numpy.array([[[[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 25.0]]], [[[60.0, 61.0], [63.0, 64.0], [66.0, 
67.0], [69.0, 70.0]]], [[[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-15.0, -16.0], [-18.0, -19.0], [-21.0, -22.0], [-24.0, -25.0]]], [[[-60.0, -61.0], 
[-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0]]], [[[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,:4]
      ref=Data(numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UUUP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,:4,1]
      ref=Data(numpy.array([[[1.0, 4.0, 7.0, 10.0], [16.0, 19.0, 22.0, 25.0]], [[46.0, 49.0, 52.0, 55.0], [61.0, 64.0, 67.0, 
70.0]], [[91.0, 94.0, 97.0, 100.0], [106.0, 109.0, 112.0, 115.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-1.0, -4.0, -7.0, -10.0], [-16.0, -19.0, -22.0, -25.0]], [[-46.0, -49.0, -52.0, 
-55.0], [-61.0, -64.0, -67.0, -70.0]], [[-91.0, -94.0, -97.0, -100.0], [-106.0, -109.0, -112.0, -115.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1,:4]
      ref=Data(numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,2]
      ref=Data(numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 
113.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-51.0, -52.0, -53.0], [-66.0, -67.0, -68.0]], [[-96.0, -97.0, -98.0], [-111.0, 
-112.0, -113.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:,2]
      ref=Data(numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], [81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 
113.0], [126.0, 127.0, 128.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-51.0, -52.0, -53.0], [-66.0, -67.0, -68.0], [-81.0, -82.0, -83.0]], [[-96.0, -97.0, 
-98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, -128.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRRR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,1:4,1:2]
      ref=Data(numpy.array([[[[64.0], [67.0], [70.0]]], [[[109.0], [112.0], [115.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-64.0], [-67.0], [-70.0]]], [[[-109.0], [-112.0], [-115.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_ULUL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,2:,:4,2:]
      ref=Data(numpy.array([[[[32.0], [35.0], [38.0], [41.0]]], [[[77.0], [80.0], [83.0], [86.0]]], [[[122.0], [125.0], 
[128.0], [131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-32.0], [-35.0], [-38.0], [-41.0]]], [[[-77.0], [-80.0], [-83.0], [-86.0]]], 
[[[-122.0], [-125.0], [-128.0], [-131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_ULCL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,2:,:,2:]
      ref=Data(numpy.array([[[[32.0], [35.0], [38.0], [41.0], [44.0]]], [[[77.0], [80.0], [83.0], [86.0], [89.0]]], [[[122.0], 
[125.0], [128.0], [131.0], [134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-32.0], [-35.0], [-38.0], [-41.0], [-44.0]]], [[[-77.0], [-80.0], [-83.0], [-86.0], 
[-89.0]]], [[[-122.0], [-125.0], [-128.0], [-131.0], [-134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,:]
      ref=Data(numpy.array([[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 
74.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, 
-71.0], [-72.0, -73.0, -74.0]]], [[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCLC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,2:,:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], 
[57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]], [[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], 
[[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, 
-73.0, -74.0]], [[-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-96.0, -97.0, -98.0], [-99.0, 
-100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], 
[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-141.0, -142.0, -143.0], [-144.0, -145.0, 
-146.0], [-147.0, -148.0, -149.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]], [[-171.0, 
-172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:2,2]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 
98.0], [111.0, 112.0, 113.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 158.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-6.0, -7.0, -8.0], [-21.0, -22.0, -23.0]], [[-51.0, -52.0, -53.0], [-66.0, -67.0, 
-68.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0]], [[-141.0, -142.0, -143.0], [-156.0, -157.0, -158.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_ULLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,2:,2:]
      ref=Data(numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], [[[-81.0, 
-82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, 
-133.0, -134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], 
[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], 
[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], 
[99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, 
-59.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], 
[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], 
[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLLC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,2:,:]
      ref=Data(numpy.array([[[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1,2:]
      ref=Data(numpy.array([[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PUCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:2,:,:]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:,1:4]
      ref=Data(numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, 
-20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], 
[[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, 
-70.0, -71.0]], [[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, 
-98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, 
-124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CLRC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,2:,1:4,:]
      ref=Data(numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 170.0], 
[171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-78.0, 
-79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]], [[[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:2,2]
      ref=Data(numpy.array([[[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 
158.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0]], [[-141.0, -142.0, -143.0], [-156.0, 
-157.0, -158.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPPU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,2,:2]
      ref=Data(numpy.array([111.0, 112.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-111.0, -112.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PRRU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1:2,1:4,:2]
      ref=Data(numpy.array([[[108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_ULUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,2:,:4]
      ref=Data(numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, 
-41.0]]], [[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-120.0, -121.0, 
-122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,1:4]
      ref=Data(numpy.array([[[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 170.0], 
[171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], 
[[[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RPCU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1,:,:2]
      ref=Data(numpy.array([[[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0], [72.0, 73.0]], [[105.0, 106.0], [108.0, 
109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 118.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-60.0, -61.0], [-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0], [-72.0, -73.0]], 
[[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0], [-117.0, -118.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:2,2]
      ref=Data(numpy.array([[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,2:]
      ref=Data(numpy.array([[[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]], 
[[[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:2,:]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], 
[[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 
151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], 
[-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UCUC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:,:4,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-30.0, -31.0, -32.0], [-33.0, 
-34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, 
-53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], 
[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-90.0, -91.0, -92.0], [-93.0, 
-94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:2,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], 
[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], 
[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], 
[99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 
146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], 
[162.0, 163.0, 164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, 
-59.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], 
[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], 
[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, 
-148.0, -149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], 
[-162.0, -163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,2:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], 
[57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]], [[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], 
[[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, 
-73.0, -74.0]], [[-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-96.0, -97.0, -98.0], [-99.0, 
-100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], 
[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-141.0, -142.0, -143.0], [-144.0, -145.0, 
-146.0], [-147.0, -148.0, -149.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]], [[-171.0, 
-172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,2:]
      ref=Data(numpy.array([[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRLL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,2:,2:]
      ref=Data(numpy.array([[[[68.0], [71.0], [74.0]]], [[[113.0], [116.0], [119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-68.0], [-71.0], [-74.0]]], [[[-113.0], [-116.0], [-119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UUUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,:4,:2]
      ref=Data(numpy.array([[[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], 
[24.0, 25.0]]], [[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 
70.0]]], [[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 
115.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0], [-3.0, -4.0], [-6.0, -7.0], [-9.0, -10.0]], [[-15.0, -16.0], [-18.0, 
-19.0], [-21.0, -22.0], [-24.0, -25.0]]], [[[-45.0, -46.0], [-48.0, -49.0], [-51.0, -52.0], [-54.0, -55.0]], [[-60.0, -61.0], 
[-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0]]], [[[-90.0, -91.0], [-93.0, -94.0], [-96.0, -97.0], [-99.0, -100.0]], [[-105.0, 
-106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1,:4]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:,:4]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 
125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], 
[144.0, 145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 
166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], [[[-135.0, -136.0, -137.0], 
[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, 
-155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, 
-172.0, -173.0], [-174.0, -175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UPPU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1,2,:2]
      ref=Data(numpy.array([[21.0, 22.0], [66.0, 67.0], [111.0, 112.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-21.0, -22.0], [-66.0, -67.0], [-111.0, -112.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLUL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,:4,2:]
      ref=Data(numpy.array([[[[122.0], [125.0], [128.0], [131.0]]], [[[167.0], [170.0], [173.0], [176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-122.0], [-125.0], [-128.0], [-131.0]]], [[[-167.0], [-170.0], [-173.0], 
[-176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RLLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,2:,2:]
      ref=Data(numpy.array([[[[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 
130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-126.0, 
-127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CLUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,2:,:4]
      ref=Data(numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]], [[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 
176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, 
-41.0]]], [[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-120.0, -121.0, 
-122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], [[[-165.0, -166.0, -167.0], [-168.0, 
-169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:,2]
      ref=Data(numpy.array([[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, -128.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,:]
      ref=Data(numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], 
[132.0, 133.0, 134.0]]], [[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 
178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, 
-173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UUUC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,:4,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], 
[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-45.0, -46.0, -47.0], [-48.0, 
-49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRRP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,1:4,1]
      ref=Data(numpy.array([[[64.0, 67.0, 70.0]], [[109.0, 112.0, 115.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-64.0, -67.0, -70.0]], [[-109.0, -112.0, -115.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:2,1:4]
      ref=Data(numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], 
[-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,:4]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], 
[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-45.0, -46.0, -47.0], [-48.0, 
-49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,:]
      ref=Data(numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], 
[132.0, 133.0, 134.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0], [-132.0, -133.0, -134.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:,:]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 
59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 
95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 
128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0], [-57.0, -58.0, -59.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]], [[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, 
-88.0, -89.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, 
-103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], 
[-117.0, -118.0, -119.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0], [-132.0, -133.0, -134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:2,:4]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-135.0, 
-136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-150.0, -151.0, -152.0], 
[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RLRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,2:,1:4]
      ref=Data(numpy.array([[[[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-123.0, 
-124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1,:4]
      ref=Data(numpy.array([[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, 
-71.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UURR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,1:4,1:2]
      ref=Data(numpy.array([[[[4.0], [7.0], [10.0]], [[19.0], [22.0], [25.0]]], [[[49.0], [52.0], [55.0]], [[64.0], [67.0], 
[70.0]]], [[[94.0], [97.0], [100.0]], [[109.0], [112.0], [115.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-4.0], [-7.0], [-10.0]], [[-19.0], [-22.0], [-25.0]]], [[[-49.0], [-52.0], [-55.0]], 
[[-64.0], [-67.0], [-70.0]]], [[[-94.0], [-97.0], [-100.0]], [[-109.0], [-112.0], [-115.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,1:4]
      ref=Data(numpy.array([[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PRUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1:2,:4]
      ref=Data(numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_ULRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,2:,1:4]
      ref=Data(numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-78.0, 
-79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:,2:]
      ref=Data(numpy.array([[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], 
[-132.0, -133.0, -134.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,:4]
      ref=Data(numpy.array([[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, 
-71.0]]], [[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LPPP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1,2,1]
      ref=Data(numpy.array([112.0, 157.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-112.0, -157.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,:4]
      ref=Data(numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], 
[[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0]]], [[[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, 
-161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRUR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,:4,1:2]
      ref=Data(numpy.array([[[[61.0], [64.0], [67.0], [70.0]]], [[[106.0], [109.0], [112.0], [115.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-61.0], [-64.0], [-67.0], [-70.0]]], [[[-106.0], [-109.0], [-112.0], [-115.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1,1:4]
      ref=Data(numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-63.0, -64.0, 
-65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RLUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,2:,:4]
      ref=Data(numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, 
-86.0]]], [[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CLCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,2:,:]
      ref=Data(numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 
44.0]]], [[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, 
-41.0], [-42.0, -43.0, -44.0]]], [[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], 
[-87.0, -88.0, -89.0]]], [[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0], [-132.0, -133.0, -134.0]]], [[[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, 
-175.0, -176.0], [-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1,2:]
      ref=Data(numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]], [[-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,2:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]]], [[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], 
[-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,1:4]
      ref=Data(numpy.array([[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:,2]
      ref=Data(numpy.array([[[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], [156.0, 
157.0, 158.0], [171.0, 172.0, 173.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, -128.0]], [[-141.0, 
-142.0, -143.0], [-156.0, -157.0, -158.0], [-171.0, -172.0, -173.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:2,2:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 
158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]]], [[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], 
[-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-141.0, -142.0, 
-143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PRLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1:2,2:]
      ref=Data(numpy.array([[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1,2]
      ref=Data(numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 113.0], [156.0, 157.0, 
158.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-21.0, -22.0, -23.0], [-66.0, -67.0, -68.0], [-111.0, -112.0, -113.0], [-156.0, 
-157.0, -158.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:,:4]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 
125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-75.0, -76.0, -77.0], 
[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, 
-97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PLPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,2:,2]
      ref=Data(numpy.array([[126.0, 127.0, 128.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-126.0, -127.0, -128.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:,2:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], 
[57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]], [[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], 
[[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, 
-73.0, -74.0]], [[-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-96.0, -97.0, -98.0], [-99.0, 
-100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], 
[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:,1:4]
      ref=Data(numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,2]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], 
[81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 
158.0], [171.0, 172.0, 173.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-6.0, -7.0, -8.0], [-21.0, -22.0, -23.0], [-36.0, -37.0, -38.0]], [[-51.0, -52.0, 
-53.0], [-66.0, -67.0, -68.0], [-81.0, -82.0, -83.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, 
-128.0]], [[-141.0, -142.0, -143.0], [-156.0, -157.0, -158.0], [-171.0, -172.0, -173.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_URLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1:2,2:]
      ref=Data(numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]]], [[[-66.0, 
-67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, 
-118.0, -119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,1:4]
      ref=Data(numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-63.0, -64.0, 
-65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RLCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,2:,:]
      ref=Data(numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, 
-86.0], [-87.0, -88.0, -89.0]]], [[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0], [-132.0, -133.0, -134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,:4]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 
145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-30.0, -31.0, -32.0], [-33.0, 
-34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, 
-53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], 
[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-90.0, -91.0, -92.0], [-93.0, 
-94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, 
-146.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]], [[-165.0, 
-166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CLLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,2:,2:]
      ref=Data(numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], [[[-81.0, 
-82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, 
-133.0, -134.0]]], [[[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:2,1:4]
      ref=Data(numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, 
-20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], 
[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, 
-100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-138.0, -139.0, -140.0], 
[-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, 
-161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PRPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1:2,2]
      ref=Data(numpy.array([[111.0, 112.0, 113.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-111.0, -112.0, -113.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_URRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1:2,1:4]
      ref=Data(numpy.array([[[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-63.0, 
-64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,1:4]
      ref=Data(numpy.array([[[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], 
[[[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_URUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1:2,:4]
      ref=Data(numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0]]], [[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,2:]
      ref=Data(numpy.array([[[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1,1:4]
      ref=Data(numpy.array([[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], 
[[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1,:]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], 
[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, 
-160.0, -161.0], [-162.0, -163.0, -164.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CRCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1:2,:]
      ref=Data(numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]]], [[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]]], [[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]]], [[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]], [[[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, 
-160.0, -161.0], [-162.0, -163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1,2:]
      ref=Data(numpy.array([[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], 
[[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:,2:]
      ref=Data(numpy.array([[[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 
101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], 
[129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], 
[[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,:,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:2,2:]
      ref=Data(numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 
158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], 
[-147.0, -148.0, -149.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:,:4]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LRPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1:2,2]
      ref=Data(numpy.array([[[111.0, 112.0, 113.0]], [[156.0, 157.0, 158.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-111.0, -112.0, -113.0]], [[-156.0, -157.0, -158.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,:]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 
59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 
92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 
109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0], [-57.0, -58.0, -59.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], 
[-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CRUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1:2,:4]
      ref=Data(numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0]]], [[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-150.0, -151.0, -152.0], [-153.0, 
-154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1,:]
      ref=Data(numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], 
[-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1,2]
      ref=Data(numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 113.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-21.0, -22.0, -23.0], [-66.0, -67.0, -68.0], [-111.0, -112.0, -113.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,1:4]
      ref=Data(numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, 
-20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], 
[[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, 
-70.0, -71.0]], [[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, 
-98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, 
-124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], [[[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], 
[-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]], [[-168.0, -169.0, 
-170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:,:]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,2]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 
98.0], [111.0, 112.0, 113.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-6.0, -7.0, -8.0], [-21.0, -22.0, -23.0]], [[-51.0, -52.0, -53.0], [-66.0, -67.0, 
-68.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CCCR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,:,:,1:2]
      ref=Data(numpy.array([[[[1.0], [4.0], [7.0], [10.0], [13.0]], [[16.0], [19.0], [22.0], [25.0], [28.0]], [[31.0], [34.0], 
[37.0], [40.0], [43.0]]], [[[46.0], [49.0], [52.0], [55.0], [58.0]], [[61.0], [64.0], [67.0], [70.0], [73.0]], [[76.0], [79.0], 
[82.0], [85.0], [88.0]]], [[[91.0], [94.0], [97.0], [100.0], [103.0]], [[106.0], [109.0], [112.0], [115.0], [118.0]], [[121.0], 
[124.0], [127.0], [130.0], [133.0]]], [[[136.0], [139.0], [142.0], [145.0], [148.0]], [[151.0], [154.0], [157.0], [160.0], 
[163.0]], [[166.0], [169.0], [172.0], [175.0], [178.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-1.0], [-4.0], [-7.0], [-10.0], [-13.0]], [[-16.0], [-19.0], [-22.0], [-25.0], 
[-28.0]], [[-31.0], [-34.0], [-37.0], [-40.0], [-43.0]]], [[[-46.0], [-49.0], [-52.0], [-55.0], [-58.0]], [[-61.0], [-64.0], 
[-67.0], [-70.0], [-73.0]], [[-76.0], [-79.0], [-82.0], [-85.0], [-88.0]]], [[[-91.0], [-94.0], [-97.0], [-100.0], [-103.0]], 
[[-106.0], [-109.0], [-112.0], [-115.0], [-118.0]], [[-121.0], [-124.0], [-127.0], [-130.0], [-133.0]]], [[[-136.0], [-139.0], 
[-142.0], [-145.0], [-148.0]], [[-151.0], [-154.0], [-157.0], [-160.0], [-163.0]], [[-166.0], [-169.0], [-172.0], [-175.0], 
[-178.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1,2:]
      ref=Data(numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]], [[-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPPP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,2,1]
      ref=Data(112.0,self.functionspace)
      ref.setTaggedValue(1,-112.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1,1:4]
      ref=Data(numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-63.0, -64.0, 
-65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:2,:]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_ULPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,2:,2]
      ref=Data(numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-36.0, -37.0, -38.0]], [[-81.0, -82.0, -83.0]], [[-126.0, -127.0, -128.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,2:]
      ref=Data(numpy.array([[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CLPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,2:,2]
      ref=Data(numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]], [[171.0, 172.0, 
173.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-36.0, -37.0, -38.0]], [[-81.0, -82.0, -83.0]], [[-126.0, -127.0, -128.0]], [[-171.0, 
-172.0, -173.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:2,1:4]
      ref=Data(numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LLLL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,2:,2:,2:]
      ref=Data(numpy.array([[[[128.0], [131.0], [134.0]]], [[[173.0], [176.0], [179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-128.0], [-131.0], [-134.0]]], [[[-173.0], [-176.0], [-179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,:]
      ref=Data(numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 
118.0, 119.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RLPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,2:,2]
      ref=Data(numpy.array([[[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-81.0, -82.0, -83.0]], [[-126.0, -127.0, -128.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CRPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1:2,2]
      ref=Data(numpy.array([[[21.0, 22.0, 23.0]], [[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]], [[156.0, 157.0, 
158.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-21.0, -22.0, -23.0]], [[-66.0, -67.0, -68.0]], [[-111.0, -112.0, -113.0]], [[-156.0, 
-157.0, -158.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CRLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1:2,2:]
      ref=Data(numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]]], [[[-66.0, 
-67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, 
-118.0, -119.0]]], [[[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1,1:4]
      ref=Data(numpy.array([[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,1,2]
      ref=Data(numpy.array([[111.0, 112.0, 113.0], [156.0, 157.0, 158.0]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[-111.0, -112.0, -113.0], [-156.0, -157.0, -158.0]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CRRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1:2,1:4]
      ref=Data(numpy.array([[[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-63.0, 
-64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]]], [[[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PRRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1:2,1:4]
      ref=Data(numpy.array([[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:,:4]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-30.0, -31.0, -32.0], [-33.0, 
-34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, 
-53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], 
[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-90.0, -91.0, -92.0], [-93.0, 
-94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,1:4]
      ref=Data(numpy.array([[[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:,2:]
      ref=Data(numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], 
[-132.0, -133.0, -134.0]]], [[[-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]], [[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, 
-178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,1,2]
      ref=Data(numpy.array([111.0, 112.0, 113.0]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([-111.0, -112.0, -113.0]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_URPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,1:2,2]
      ref=Data(numpy.array([[[21.0, 22.0, 23.0]], [[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-21.0, -22.0, -23.0]], [[-66.0, -67.0, -68.0]], [[-111.0, -112.0, -113.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_CPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:,1,:4]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-150.0, -151.0, -152.0], [-153.0, 
-154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RRPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,1:2,2]
      ref=Data(numpy.array([[[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-66.0, -67.0, -68.0]], [[-111.0, -112.0, -113.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:,:]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, 
-143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, 
-157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], 
[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:2,2:]
      ref=Data(numpy.array([[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_PUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2,:2,:4]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_RULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[1:3,:2,2:]
      ref=Data(numpy.array([[[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, 
-104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_LCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[2:,:,1:4]
      ref=Data(numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]], [[[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-153.0, -154.0, 
-155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]], [[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, 
-175.0, -176.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_taggedData_rank4_UURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]]))
      res=arg[:3,:2,1:4]
      ref=Data(numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      ref.setTaggedValue(1,numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, 
-20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], 
[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, 
-100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]]))
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_taggedData_L(self):
      arg=Data(numpy.array([8.0, 7.0, 4.0]),self.functionspace)
      val=Data(numpy.array([8.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([2.0]))
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([8.0, 7.0, 8.0])+(1.-msk_ref)*numpy.array([8.0, 7.0, 2.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_taggedData_C(self):
      arg=Data(numpy.array([1.0, 5.0, 5.0]),self.functionspace)
      val=Data(numpy.array([8.0, 7.0, 3.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 3.0, 4.0]))
      arg[:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([8.0, 7.0, 3.0])+(1.-msk_ref)*numpy.array([3.0, 3.0, 4.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_taggedData_U(self):
      arg=Data(numpy.array([2.0, 3.0, 4.0]),self.functionspace)
      val=Data(numpy.array([6.0, 3.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 7.0]))
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([6.0, 3.0, 4.0])+(1.-msk_ref)*numpy.array([3.0, 7.0, 4.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_taggedData_P(self):
      arg=Data(numpy.array([3.0, 6.0, 6.0]),self.functionspace)
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[1]=val
      ref=Data(numpy.array([3.0, 1.0, 6.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_taggedData_R(self):
      arg=Data(numpy.array([5.0, 7.0, 2.0]),self.functionspace)
      val=Data(numpy.array([7.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([6.0]))
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([5.0, 7.0, 2.0])+(1.-msk_ref)*numpy.array([5.0, 6.0, 2.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_UU(self):
      arg=Data(numpy.array([[1.0, 6.0, 1.0, 8.0], [4.0, 4.0, 5.0, 6.0], [4.0, 7.0, 5.0, 7.0]]),self.functionspace)
      val=Data(numpy.array([[7.0, 7.0, 5.0], [6.0, 3.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 4.0, 1.0], [5.0, 2.0, 6.0]]))
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 7.0, 5.0, 8.0], [6.0, 3.0, 4.0, 6.0], [4.0, 7.0, 5.0, 
7.0]])+(1.-msk_ref)*numpy.array([[4.0, 4.0, 1.0, 8.0], [5.0, 2.0, 6.0, 6.0], [4.0, 7.0, 5.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_LC(self):
      arg=Data(numpy.array([[3.0, 8.0, 1.0, 4.0], [6.0, 1.0, 1.0, 2.0], [4.0, 2.0, 1.0, 7.0]]),self.functionspace)
      val=Data(numpy.array([[8.0, 2.0, 5.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 4.0, 7.0, 6.0]]))
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 8.0, 1.0, 4.0], [6.0, 1.0, 1.0, 2.0], [8.0, 2.0, 5.0, 
4.0]])+(1.-msk_ref)*numpy.array([[3.0, 8.0, 1.0, 4.0], [6.0, 1.0, 1.0, 2.0], [4.0, 4.0, 7.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_LP(self):
      arg=Data(numpy.array([[2.0, 2.0, 3.0, 4.0], [1.0, 6.0, 8.0, 8.0], [4.0, 3.0, 1.0, 1.0]]),self.functionspace)
      val=Data(numpy.array([5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([7.0]))
      arg[2:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 2.0, 3.0, 4.0], [1.0, 6.0, 8.0, 8.0], [4.0, 3.0, 5.0, 
1.0]])+(1.-msk_ref)*numpy.array([[2.0, 2.0, 3.0, 4.0], [1.0, 6.0, 8.0, 8.0], [4.0, 3.0, 7.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_RU(self):
      arg=Data(numpy.array([[3.0, 5.0, 3.0, 3.0], [8.0, 1.0, 8.0, 5.0], [1.0, 4.0, 3.0, 1.0]]),self.functionspace)
      val=Data(numpy.array([[6.0, 8.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 7.0, 4.0]]))
      arg[1:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 5.0, 3.0, 3.0], [6.0, 8.0, 6.0, 5.0], [1.0, 4.0, 3.0, 
1.0]])+(1.-msk_ref)*numpy.array([[3.0, 5.0, 3.0, 3.0], [7.0, 7.0, 4.0, 5.0], [1.0, 4.0, 3.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_PN(self):
      arg=Data(numpy.array([[3.0, 7.0, 3.0, 8.0], [5.0, 2.0, 2.0, 6.0], [5.0, 8.0, 1.0, 6.0]]),self.functionspace)
      val=Data(numpy.array([5.0, 1.0, 7.0, 4.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 5.0, 1.0, 8.0]))
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 7.0, 3.0, 8.0], [5.0, 1.0, 7.0, 4.0], [5.0, 8.0, 1.0, 
6.0]])+(1.-msk_ref)*numpy.array([[3.0, 7.0, 3.0, 8.0], [3.0, 5.0, 1.0, 8.0], [5.0, 8.0, 1.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_UC(self):
      arg=Data(numpy.array([[4.0, 2.0, 6.0, 2.0], [8.0, 3.0, 1.0, 4.0], [3.0, 3.0, 5.0, 1.0]]),self.functionspace)
      val=Data(numpy.array([[3.0, 3.0, 6.0, 2.0], [7.0, 8.0, 4.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 7.0, 4.0, 4.0], [3.0, 1.0, 6.0, 3.0]]))
      arg[:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 3.0, 6.0, 2.0], [7.0, 8.0, 4.0, 5.0], [3.0, 3.0, 5.0, 
1.0]])+(1.-msk_ref)*numpy.array([[6.0, 7.0, 4.0, 4.0], [3.0, 1.0, 6.0, 3.0], [3.0, 3.0, 5.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_CN(self):
      arg=Data(numpy.array([[3.0, 8.0, 8.0, 6.0], [4.0, 4.0, 5.0, 2.0], [3.0, 3.0, 7.0, 1.0]]),self.functionspace)
      val=Data(numpy.array([[4.0, 6.0, 4.0, 4.0], [6.0, 4.0, 7.0, 5.0], [1.0, 5.0, 7.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 5.0, 2.0, 3.0], [6.0, 7.0, 6.0, 3.0], [1.0, 4.0, 8.0, 4.0]]))
      arg[:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 6.0, 4.0, 4.0], [6.0, 4.0, 7.0, 5.0], [1.0, 5.0, 7.0, 
3.0]])+(1.-msk_ref)*numpy.array([[7.0, 5.0, 2.0, 3.0], [6.0, 7.0, 6.0, 3.0], [1.0, 4.0, 8.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_RR(self):
      arg=Data(numpy.array([[1.0, 8.0, 2.0, 1.0], [4.0, 4.0, 8.0, 4.0], [7.0, 7.0, 8.0, 4.0]]),self.functionspace)
      val=Data(numpy.array([[4.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 8.0]]))
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 8.0, 2.0, 1.0], [4.0, 4.0, 2.0, 4.0], [7.0, 7.0, 8.0, 
4.0]])+(1.-msk_ref)*numpy.array([[1.0, 8.0, 2.0, 1.0], [4.0, 7.0, 8.0, 4.0], [7.0, 7.0, 8.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_LR(self):
      arg=Data(numpy.array([[5.0, 4.0, 8.0, 2.0], [5.0, 1.0, 2.0, 4.0], [4.0, 7.0, 6.0, 8.0]]),self.functionspace)
      val=Data(numpy.array([[1.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 4.0]]))
      arg[2:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 4.0, 8.0, 2.0], [5.0, 1.0, 2.0, 4.0], [4.0, 1.0, 4.0, 
8.0]])+(1.-msk_ref)*numpy.array([[5.0, 4.0, 8.0, 2.0], [5.0, 1.0, 2.0, 4.0], [4.0, 3.0, 4.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_PP(self):
      arg=Data(numpy.array([[4.0, 1.0, 4.0, 6.0], [3.0, 2.0, 2.0, 3.0], [4.0, 1.0, 6.0, 7.0]]),self.functionspace)
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[1,2]=val
      ref=Data(numpy.array([[4.0, 1.0, 4.0, 6.0], [3.0, 2.0, 1.0, 3.0], [4.0, 1.0, 6.0, 7.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_LL(self):
      arg=Data(numpy.array([[6.0, 6.0, 1.0, 7.0], [4.0, 6.0, 3.0, 2.0], [4.0, 7.0, 4.0, 5.0]]),self.functionspace)
      val=Data(numpy.array([[1.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 5.0]]))
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 6.0, 1.0, 7.0], [4.0, 6.0, 3.0, 2.0], [4.0, 7.0, 1.0, 
3.0]])+(1.-msk_ref)*numpy.array([[6.0, 6.0, 1.0, 7.0], [4.0, 6.0, 3.0, 2.0], [4.0, 7.0, 2.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_PU(self):
      arg=Data(numpy.array([[2.0, 3.0, 1.0, 7.0], [1.0, 3.0, 3.0, 6.0], [7.0, 2.0, 2.0, 1.0]]),self.functionspace)
      val=Data(numpy.array([8.0, 8.0, 4.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0, 8.0, 1.0]))
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 3.0, 1.0, 7.0], [8.0, 8.0, 4.0, 6.0], [7.0, 2.0, 2.0, 
1.0]])+(1.-msk_ref)*numpy.array([[2.0, 3.0, 1.0, 7.0], [8.0, 8.0, 1.0, 6.0], [7.0, 2.0, 2.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_RC(self):
      arg=Data(numpy.array([[8.0, 5.0, 8.0, 4.0], [5.0, 1.0, 6.0, 5.0], [7.0, 7.0, 1.0, 3.0]]),self.functionspace)
      val=Data(numpy.array([[8.0, 8.0, 7.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 3.0, 7.0, 7.0]]))
      arg[1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 5.0, 8.0, 4.0], [8.0, 8.0, 7.0, 8.0], [7.0, 7.0, 1.0, 
3.0]])+(1.-msk_ref)*numpy.array([[8.0, 5.0, 8.0, 4.0], [6.0, 3.0, 7.0, 7.0], [7.0, 7.0, 1.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_CP(self):
      arg=Data(numpy.array([[7.0, 2.0, 7.0, 4.0], [6.0, 3.0, 5.0, 8.0], [8.0, 8.0, 3.0, 2.0]]),self.functionspace)
      val=Data(numpy.array([4.0, 8.0, 5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 4.0, 6.0]))
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 2.0, 4.0, 4.0], [6.0, 3.0, 8.0, 8.0], [8.0, 8.0, 5.0, 
2.0]])+(1.-msk_ref)*numpy.array([[7.0, 2.0, 3.0, 4.0], [6.0, 3.0, 4.0, 8.0], [8.0, 8.0, 6.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_LN(self):
      arg=Data(numpy.array([[8.0, 7.0, 8.0, 3.0], [2.0, 6.0, 1.0, 8.0], [2.0, 1.0, 7.0, 8.0]]),self.functionspace)
      val=Data(numpy.array([[8.0, 8.0, 5.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 2.0, 4.0, 3.0]]))
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 7.0, 8.0, 3.0], [2.0, 6.0, 1.0, 8.0], [8.0, 8.0, 5.0, 
7.0]])+(1.-msk_ref)*numpy.array([[8.0, 7.0, 8.0, 3.0], [2.0, 6.0, 1.0, 8.0], [8.0, 2.0, 4.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_PR(self):
      arg=Data(numpy.array([[6.0, 3.0, 3.0, 2.0], [5.0, 3.0, 6.0, 5.0], [1.0, 1.0, 4.0, 5.0]]),self.functionspace)
      val=Data(numpy.array([7.0, 2.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([4.0, 7.0]))
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 3.0, 3.0, 2.0], [5.0, 7.0, 2.0, 5.0], [1.0, 1.0, 4.0, 
5.0]])+(1.-msk_ref)*numpy.array([[6.0, 3.0, 3.0, 2.0], [5.0, 4.0, 7.0, 5.0], [1.0, 1.0, 4.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_UL(self):
      arg=Data(numpy.array([[2.0, 3.0, 4.0, 8.0], [6.0, 6.0, 5.0, 4.0], [8.0, 4.0, 2.0, 8.0]]),self.functionspace)
      val=Data(numpy.array([[8.0, 8.0], [6.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 1.0], [7.0, 1.0]]))
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 3.0, 8.0, 8.0], [6.0, 6.0, 6.0, 8.0], [8.0, 4.0, 2.0, 
8.0]])+(1.-msk_ref)*numpy.array([[2.0, 3.0, 8.0, 1.0], [6.0, 6.0, 7.0, 1.0], [8.0, 4.0, 2.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_CC(self):
      arg=Data(numpy.array([[2.0, 4.0, 8.0, 3.0], [8.0, 8.0, 6.0, 6.0], [1.0, 3.0, 5.0, 3.0]]),self.functionspace)
      val=Data(numpy.array([[5.0, 8.0, 5.0, 3.0], [7.0, 8.0, 5.0, 2.0], [1.0, 3.0, 3.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 3.0, 6.0, 1.0], [8.0, 5.0, 2.0, 3.0], [4.0, 4.0, 3.0, 2.0]]))
      arg[:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 8.0, 5.0, 3.0], [7.0, 8.0, 5.0, 2.0], [1.0, 3.0, 3.0, 
7.0]])+(1.-msk_ref)*numpy.array([[4.0, 3.0, 6.0, 1.0], [8.0, 5.0, 2.0, 3.0], [4.0, 4.0, 3.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_RN(self):
      arg=Data(numpy.array([[7.0, 1.0, 1.0, 4.0], [6.0, 3.0, 7.0, 4.0], [4.0, 5.0, 7.0, 1.0]]),self.functionspace)
      val=Data(numpy.array([[6.0, 5.0, 3.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 1.0, 2.0, 5.0]]))
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 1.0, 1.0, 4.0], [6.0, 5.0, 3.0, 2.0], [4.0, 5.0, 7.0, 
1.0]])+(1.-msk_ref)*numpy.array([[7.0, 1.0, 1.0, 4.0], [8.0, 1.0, 2.0, 5.0], [4.0, 5.0, 7.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_taggedData_UN(self):
      arg=Data(numpy.array([[6.0, 8.0, 7.0, 5.0], [2.0, 3.0, 8.0, 7.0], [1.0, 7.0, 6.0, 8.0]]),self.functionspace)
      val=Data(numpy.array([[3.0, 8.0, 7.0, 7.0], [8.0, 6.0, 6.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 3.0, 3.0, 4.0], [4.0, 7.0, 2.0, 2.0]]))
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 8.0, 7.0, 7.0], [8.0, 6.0, 6.0, 5.0], [1.0, 7.0, 6.0, 
8.0]])+(1.-msk_ref)*numpy.array([[4.0, 3.0, 3.0, 4.0], [4.0, 7.0, 2.0, 2.0], [1.0, 7.0, 6.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LLL(self):
      arg=Data(numpy.array([[[8.0, 8.0, 2.0], [5.0, 7.0, 2.0], [6.0, 6.0, 3.0], [8.0, 6.0, 1.0]], [[2.0, 1.0, 3.0], [8.0, 7.0, 
2.0], [3.0, 8.0, 3.0], [6.0, 5.0, 6.0]], [[1.0, 2.0, 6.0], [2.0, 8.0, 1.0], [3.0, 4.0, 4.0], [5.0, 5.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[5.0], [8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0], [3.0]]]))
      arg[2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 8.0, 2.0], [5.0, 7.0, 2.0], [6.0, 6.0, 3.0], [8.0, 6.0, 1.0]], [[2.0, 1.0, 3.0], [8.0, 
7.0, 2.0], [3.0, 8.0, 3.0], [6.0, 5.0, 6.0]], [[1.0, 2.0, 6.0], [2.0, 8.0, 1.0], [3.0, 4.0, 5.0], [5.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 2.0], [5.0, 7.0, 2.0], [6.0, 6.0, 3.0], [8.0, 6.0, 1.0]], [[2.0, 1.0, 3.0], [8.0, 
7.0, 2.0], [3.0, 8.0, 3.0], [6.0, 5.0, 6.0]], [[1.0, 2.0, 6.0], [2.0, 8.0, 1.0], [3.0, 4.0, 7.0], [5.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RCC(self):
      arg=Data(numpy.array([[[3.0, 6.0, 2.0], [2.0, 5.0, 3.0], [5.0, 3.0, 3.0], [2.0, 2.0, 6.0]], [[1.0, 1.0, 7.0], [6.0, 2.0, 
5.0], [3.0, 6.0, 2.0], [3.0, 1.0, 6.0]], [[7.0, 4.0, 3.0], [3.0, 5.0, 7.0], [2.0, 4.0, 6.0], [6.0, 5.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 7.0, 4.0], [6.0, 6.0, 4.0], [6.0, 2.0, 5.0], [1.0, 2.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 2.0, 3.0], [4.0, 1.0, 8.0], [3.0, 8.0, 5.0], [3.0, 8.0, 2.0]]]))
      arg[1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 2.0], [2.0, 5.0, 3.0], [5.0, 3.0, 3.0], [2.0, 2.0, 6.0]], [[8.0, 7.0, 4.0], [6.0, 
6.0, 4.0], [6.0, 2.0, 5.0], [1.0, 2.0, 2.0]], [[7.0, 4.0, 3.0], [3.0, 5.0, 7.0], [2.0, 4.0, 6.0], [6.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 6.0, 2.0], [2.0, 5.0, 3.0], [5.0, 3.0, 3.0], [2.0, 2.0, 6.0]], [[2.0, 2.0, 3.0], [4.0, 
1.0, 8.0], [3.0, 8.0, 5.0], [3.0, 8.0, 2.0]], [[7.0, 4.0, 3.0], [3.0, 5.0, 7.0], [2.0, 4.0, 6.0], [6.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PPU(self):
      arg=Data(numpy.array([[[1.0, 2.0, 6.0], [2.0, 5.0, 8.0], [2.0, 2.0, 6.0], [4.0, 7.0, 6.0]], [[1.0, 2.0, 3.0], [6.0, 8.0, 
5.0], [2.0, 4.0, 8.0], [8.0, 5.0, 1.0]], [[2.0, 1.0, 1.0], [6.0, 6.0, 8.0], [1.0, 2.0, 6.0], [1.0, 1.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([6.0, 6.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([2.0, 6.0]))
      arg[1,2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 6.0], [2.0, 5.0, 8.0], [2.0, 2.0, 6.0], [4.0, 7.0, 6.0]], [[1.0, 2.0, 3.0], [6.0, 
8.0, 5.0], [6.0, 6.0, 8.0], [8.0, 5.0, 1.0]], [[2.0, 1.0, 1.0], [6.0, 6.0, 8.0], [1.0, 2.0, 6.0], [1.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 6.0], [2.0, 5.0, 8.0], [2.0, 2.0, 6.0], [4.0, 7.0, 6.0]], [[1.0, 2.0, 3.0], [6.0, 
8.0, 5.0], [2.0, 6.0, 8.0], [8.0, 5.0, 1.0]], [[2.0, 1.0, 1.0], [6.0, 6.0, 8.0], [1.0, 2.0, 6.0], [1.0, 1.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PLU(self):
      arg=Data(numpy.array([[[6.0, 7.0, 3.0], [7.0, 7.0, 8.0], [6.0, 7.0, 6.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 1.0], [1.0, 6.0, 
7.0], [4.0, 3.0, 1.0], [4.0, 3.0, 5.0]], [[8.0, 3.0, 5.0], [3.0, 8.0, 7.0], [5.0, 8.0, 5.0], [1.0, 8.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 4.0], [8.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 8.0], [5.0, 3.0]]))
      arg[1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 3.0], [7.0, 7.0, 8.0], [6.0, 7.0, 6.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 1.0], [1.0, 
6.0, 7.0], [7.0, 4.0, 1.0], [8.0, 1.0, 5.0]], [[8.0, 3.0, 5.0], [3.0, 8.0, 7.0], [5.0, 8.0, 5.0], [1.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 7.0, 3.0], [7.0, 7.0, 8.0], [6.0, 7.0, 6.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 1.0], [1.0, 
6.0, 7.0], [4.0, 8.0, 1.0], [5.0, 3.0, 5.0]], [[8.0, 3.0, 5.0], [3.0, 8.0, 7.0], [5.0, 8.0, 5.0], [1.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_URN(self):
      arg=Data(numpy.array([[[2.0, 2.0, 5.0], [7.0, 3.0, 8.0], [8.0, 6.0, 2.0], [2.0, 2.0, 6.0]], [[6.0, 2.0, 6.0], [6.0, 5.0, 
3.0], [8.0, 1.0, 7.0], [5.0, 1.0, 5.0]], [[3.0, 7.0, 6.0], [4.0, 2.0, 6.0], [3.0, 1.0, 4.0], [3.0, 5.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 6.0], [4.0, 4.0, 2.0]], [[7.0, 5.0, 7.0], [7.0, 1.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 5.0, 5.0], [5.0, 5.0, 6.0]], [[4.0, 8.0, 6.0], [7.0, 3.0, 5.0]]]))
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 5.0], [4.0, 7.0, 6.0], [4.0, 4.0, 2.0], [2.0, 2.0, 6.0]], [[6.0, 2.0, 6.0], [7.0, 
5.0, 7.0], [7.0, 1.0, 3.0], [5.0, 1.0, 5.0]], [[3.0, 7.0, 6.0], [4.0, 2.0, 6.0], [3.0, 1.0, 4.0], [3.0, 5.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 5.0], [1.0, 5.0, 5.0], [5.0, 5.0, 6.0], [2.0, 2.0, 6.0]], [[6.0, 2.0, 6.0], [4.0, 
8.0, 6.0], [7.0, 3.0, 5.0], [5.0, 1.0, 5.0]], [[3.0, 7.0, 6.0], [4.0, 2.0, 6.0], [3.0, 1.0, 4.0], [3.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PLR(self):
      arg=Data(numpy.array([[[2.0, 7.0, 5.0], [4.0, 6.0, 8.0], [4.0, 8.0, 3.0], [7.0, 5.0, 5.0]], [[8.0, 1.0, 4.0], [3.0, 6.0, 
3.0], [4.0, 3.0, 2.0], [3.0, 4.0, 6.0]], [[8.0, 6.0, 8.0], [5.0, 7.0, 4.0], [1.0, 1.0, 4.0], [3.0, 5.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[3.0], [3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0], [6.0]]))
      arg[1,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 5.0], [4.0, 6.0, 8.0], [4.0, 8.0, 3.0], [7.0, 5.0, 5.0]], [[8.0, 1.0, 4.0], [3.0, 
6.0, 3.0], [4.0, 3.0, 2.0], [3.0, 3.0, 6.0]], [[8.0, 6.0, 8.0], [5.0, 7.0, 4.0], [1.0, 1.0, 4.0], [3.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 5.0], [4.0, 6.0, 8.0], [4.0, 8.0, 3.0], [7.0, 5.0, 5.0]], [[8.0, 1.0, 4.0], [3.0, 
6.0, 3.0], [4.0, 7.0, 2.0], [3.0, 6.0, 6.0]], [[8.0, 6.0, 8.0], [5.0, 7.0, 4.0], [1.0, 1.0, 4.0], [3.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UCL(self):
      arg=Data(numpy.array([[[4.0, 4.0, 8.0], [7.0, 6.0, 8.0], [8.0, 1.0, 8.0], [6.0, 1.0, 5.0]], [[4.0, 4.0, 5.0], [2.0, 6.0, 
5.0], [3.0, 3.0, 1.0], [4.0, 3.0, 5.0]], [[1.0, 1.0, 4.0], [7.0, 1.0, 6.0], [2.0, 6.0, 6.0], [3.0, 7.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[8.0], [7.0], [8.0], [1.0]], [[7.0], [8.0], [8.0], [1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0], [2.0], [1.0], [4.0]], [[1.0], [8.0], [6.0], [5.0]]]))
      arg[:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 8.0], [7.0, 6.0, 7.0], [8.0, 1.0, 8.0], [6.0, 1.0, 1.0]], [[4.0, 4.0, 7.0], [2.0, 
6.0, 8.0], [3.0, 3.0, 8.0], [4.0, 3.0, 1.0]], [[1.0, 1.0, 4.0], [7.0, 1.0, 6.0], [2.0, 6.0, 6.0], [3.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 3.0], [7.0, 6.0, 2.0], [8.0, 1.0, 1.0], [6.0, 1.0, 4.0]], [[4.0, 4.0, 1.0], [2.0, 
6.0, 8.0], [3.0, 3.0, 6.0], [4.0, 3.0, 5.0]], [[1.0, 1.0, 4.0], [7.0, 1.0, 6.0], [2.0, 6.0, 6.0], [3.0, 7.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PUU(self):
      arg=Data(numpy.array([[[4.0, 2.0, 5.0], [2.0, 4.0, 1.0], [6.0, 5.0, 4.0], [3.0, 8.0, 3.0]], [[2.0, 1.0, 3.0], [8.0, 5.0, 
8.0], [8.0, 4.0, 6.0], [6.0, 7.0, 2.0]], [[3.0, 1.0, 7.0], [8.0, 6.0, 4.0], [6.0, 8.0, 3.0], [1.0, 3.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[5.0, 2.0], [1.0, 7.0], [7.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 7.0], [6.0, 2.0], [8.0, 1.0]]))
      arg[1,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 5.0], [2.0, 4.0, 1.0], [6.0, 5.0, 4.0], [3.0, 8.0, 3.0]], [[5.0, 2.0, 3.0], [1.0, 
7.0, 8.0], [7.0, 3.0, 6.0], [6.0, 7.0, 2.0]], [[3.0, 1.0, 7.0], [8.0, 6.0, 4.0], [6.0, 8.0, 3.0], [1.0, 3.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 5.0], [2.0, 4.0, 1.0], [6.0, 5.0, 4.0], [3.0, 8.0, 3.0]], [[5.0, 7.0, 3.0], [6.0, 
2.0, 8.0], [8.0, 1.0, 6.0], [6.0, 7.0, 2.0]], [[3.0, 1.0, 7.0], [8.0, 6.0, 4.0], [6.0, 8.0, 3.0], [1.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_ULR(self):
      arg=Data(numpy.array([[[8.0, 4.0, 1.0], [1.0, 5.0, 6.0], [7.0, 6.0, 4.0], [6.0, 6.0, 2.0]], [[3.0, 4.0, 2.0], [4.0, 4.0, 
2.0], [3.0, 4.0, 6.0], [4.0, 1.0, 4.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 5.0], [7.0, 1.0, 3.0], [2.0, 3.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[2.0], [7.0]], [[4.0], [8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0], [5.0]], [[4.0], [7.0]]]))
      arg[:2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 1.0], [1.0, 5.0, 6.0], [7.0, 2.0, 4.0], [6.0, 7.0, 2.0]], [[3.0, 4.0, 2.0], [4.0, 
4.0, 2.0], [3.0, 4.0, 6.0], [4.0, 8.0, 4.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 5.0], [7.0, 1.0, 3.0], [2.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 1.0], [1.0, 5.0, 6.0], [7.0, 6.0, 4.0], [6.0, 5.0, 2.0]], [[3.0, 4.0, 2.0], [4.0, 
4.0, 2.0], [3.0, 4.0, 6.0], [4.0, 7.0, 4.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 5.0], [7.0, 1.0, 3.0], [2.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RUU(self):
      arg=Data(numpy.array([[[7.0, 8.0, 2.0], [8.0, 6.0, 1.0], [1.0, 7.0, 8.0], [1.0, 4.0, 2.0]], [[8.0, 5.0, 8.0], [4.0, 8.0, 
5.0], [6.0, 1.0, 4.0], [7.0, 8.0, 6.0]], [[7.0, 8.0, 1.0], [6.0, 2.0, 8.0], [4.0, 8.0, 1.0], [1.0, 3.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 2.0], [7.0, 3.0], [5.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 4.0], [6.0, 8.0], [5.0, 7.0]]]))
      arg[1:2,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 2.0], [8.0, 6.0, 1.0], [1.0, 7.0, 8.0], [1.0, 4.0, 2.0]], [[2.0, 2.0, 8.0], [7.0, 
3.0, 5.0], [5.0, 4.0, 4.0], [7.0, 8.0, 6.0]], [[7.0, 8.0, 1.0], [6.0, 2.0, 8.0], [4.0, 8.0, 1.0], [1.0, 3.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 2.0], [8.0, 6.0, 1.0], [1.0, 7.0, 8.0], [1.0, 4.0, 2.0]], [[4.0, 4.0, 8.0], [6.0, 
8.0, 5.0], [5.0, 7.0, 4.0], [7.0, 8.0, 6.0]], [[7.0, 8.0, 1.0], [6.0, 2.0, 8.0], [4.0, 8.0, 1.0], [1.0, 3.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CRR(self):
      arg=Data(numpy.array([[[6.0, 4.0, 3.0], [4.0, 5.0, 4.0], [3.0, 3.0, 7.0], [1.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [4.0, 4.0, 
2.0], [8.0, 7.0, 1.0], [8.0, 6.0, 3.0]], [[2.0, 1.0, 7.0], [4.0, 4.0, 2.0], [1.0, 8.0, 5.0], [7.0, 6.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[1.0], [1.0]], [[8.0], [7.0]], [[4.0], [2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0], [3.0]], [[1.0], [6.0]], [[6.0], [2.0]]]))
      arg[:,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 3.0], [4.0, 1.0, 4.0], [3.0, 1.0, 7.0], [1.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [4.0, 
8.0, 2.0], [8.0, 7.0, 1.0], [8.0, 6.0, 3.0]], [[2.0, 1.0, 7.0], [4.0, 4.0, 2.0], [1.0, 2.0, 5.0], [7.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 3.0], [4.0, 3.0, 4.0], [3.0, 3.0, 7.0], [1.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [4.0, 
1.0, 2.0], [8.0, 6.0, 1.0], [8.0, 6.0, 3.0]], [[2.0, 1.0, 7.0], [4.0, 6.0, 2.0], [1.0, 2.0, 5.0], [7.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LRN(self):
      arg=Data(numpy.array([[[2.0, 1.0, 6.0], [7.0, 2.0, 7.0], [5.0, 2.0, 4.0], [2.0, 6.0, 5.0]], [[1.0, 5.0, 8.0], [6.0, 1.0, 
4.0], [5.0, 6.0, 3.0], [1.0, 3.0, 7.0]], [[5.0, 7.0, 5.0], [4.0, 3.0, 2.0], [5.0, 6.0, 6.0], [6.0, 1.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 8.0, 4.0], [1.0, 2.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 6.0, 4.0], [3.0, 6.0, 3.0]]]))
      arg[2:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 6.0], [7.0, 2.0, 7.0], [5.0, 2.0, 4.0], [2.0, 6.0, 5.0]], [[1.0, 5.0, 8.0], [6.0, 
1.0, 4.0], [5.0, 6.0, 3.0], [1.0, 3.0, 7.0]], [[5.0, 7.0, 5.0], [4.0, 8.0, 4.0], [1.0, 2.0, 5.0], [6.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 6.0], [7.0, 2.0, 7.0], [5.0, 2.0, 4.0], [2.0, 6.0, 5.0]], [[1.0, 5.0, 8.0], [6.0, 
1.0, 4.0], [5.0, 6.0, 3.0], [1.0, 3.0, 7.0]], [[5.0, 7.0, 5.0], [6.0, 6.0, 4.0], [3.0, 6.0, 3.0], [6.0, 1.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CRU(self):
      arg=Data(numpy.array([[[4.0, 2.0, 1.0], [5.0, 8.0, 6.0], [3.0, 8.0, 7.0], [8.0, 6.0, 5.0]], [[3.0, 2.0, 1.0], [4.0, 1.0, 
8.0], [6.0, 8.0, 6.0], [8.0, 3.0, 7.0]], [[8.0, 4.0, 6.0], [4.0, 5.0, 7.0], [8.0, 8.0, 1.0], [8.0, 6.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 2.0], [2.0, 8.0]], [[1.0, 8.0], [7.0, 1.0]], [[8.0, 2.0], [4.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 4.0], [8.0, 2.0]], [[2.0, 6.0], [3.0, 7.0]], [[2.0, 6.0], [7.0, 6.0]]]))
      arg[:,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 1.0], [5.0, 2.0, 6.0], [2.0, 8.0, 7.0], [8.0, 6.0, 5.0]], [[3.0, 2.0, 1.0], [1.0, 
8.0, 8.0], [7.0, 1.0, 6.0], [8.0, 3.0, 7.0]], [[8.0, 4.0, 6.0], [8.0, 2.0, 7.0], [4.0, 1.0, 1.0], [8.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 1.0], [2.0, 4.0, 6.0], [8.0, 2.0, 7.0], [8.0, 6.0, 5.0]], [[3.0, 2.0, 1.0], [2.0, 
6.0, 8.0], [3.0, 7.0, 6.0], [8.0, 3.0, 7.0]], [[8.0, 4.0, 6.0], [2.0, 6.0, 7.0], [7.0, 6.0, 1.0], [8.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UCC(self):
      arg=Data(numpy.array([[[8.0, 5.0, 8.0], [8.0, 4.0, 3.0], [6.0, 8.0, 3.0], [3.0, 3.0, 1.0]], [[2.0, 4.0, 2.0], [7.0, 5.0, 
3.0], [5.0, 5.0, 4.0], [5.0, 8.0, 2.0]], [[4.0, 6.0, 4.0], [3.0, 1.0, 7.0], [5.0, 6.0, 7.0], [3.0, 3.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 4.0, 8.0], [6.0, 3.0, 5.0], [4.0, 2.0, 1.0], [8.0, 4.0, 2.0]], [[6.0, 3.0, 5.0], [7.0, 1.0, 
5.0], [4.0, 8.0, 6.0], [1.0, 3.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 3.0, 7.0], [4.0, 6.0, 5.0], [1.0, 5.0, 6.0], [8.0, 3.0, 3.0]], [[1.0, 5.0, 3.0], 
[7.0, 7.0, 7.0], [4.0, 5.0, 1.0], [4.0, 3.0, 6.0]]]))
      arg[:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 8.0], [6.0, 3.0, 5.0], [4.0, 2.0, 1.0], [8.0, 4.0, 2.0]], [[6.0, 3.0, 5.0], [7.0, 
1.0, 5.0], [4.0, 8.0, 6.0], [1.0, 3.0, 2.0]], [[4.0, 6.0, 4.0], [3.0, 1.0, 7.0], [5.0, 6.0, 7.0], [3.0, 3.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 7.0], [4.0, 6.0, 5.0], [1.0, 5.0, 6.0], [8.0, 3.0, 3.0]], [[1.0, 5.0, 3.0], [7.0, 
7.0, 7.0], [4.0, 5.0, 1.0], [4.0, 3.0, 6.0]], [[4.0, 6.0, 4.0], [3.0, 1.0, 7.0], [5.0, 6.0, 7.0], [3.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CPN(self):
      arg=Data(numpy.array([[[4.0, 6.0, 4.0], [1.0, 7.0, 1.0], [7.0, 1.0, 8.0], [5.0, 2.0, 8.0]], [[2.0, 4.0, 2.0], [4.0, 4.0, 
1.0], [1.0, 4.0, 7.0], [4.0, 7.0, 2.0]], [[2.0, 4.0, 1.0], [4.0, 1.0, 1.0], [4.0, 8.0, 5.0], [7.0, 7.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[4.0, 6.0, 7.0], [3.0, 2.0, 8.0], [1.0, 2.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 2.0, 2.0], [8.0, 4.0, 2.0], [4.0, 6.0, 8.0]]))
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 4.0], [1.0, 7.0, 1.0], [4.0, 6.0, 7.0], [5.0, 2.0, 8.0]], [[2.0, 4.0, 2.0], [4.0, 
4.0, 1.0], [3.0, 2.0, 8.0], [4.0, 7.0, 2.0]], [[2.0, 4.0, 1.0], [4.0, 1.0, 1.0], [1.0, 2.0, 3.0], [7.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 4.0], [1.0, 7.0, 1.0], [6.0, 2.0, 2.0], [5.0, 2.0, 8.0]], [[2.0, 4.0, 2.0], [4.0, 
4.0, 1.0], [8.0, 4.0, 2.0], [4.0, 7.0, 2.0]], [[2.0, 4.0, 1.0], [4.0, 1.0, 1.0], [4.0, 6.0, 8.0], [7.0, 7.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PCL(self):
      arg=Data(numpy.array([[[7.0, 1.0, 6.0], [7.0, 6.0, 4.0], [1.0, 7.0, 8.0], [4.0, 7.0, 1.0]], [[7.0, 4.0, 6.0], [7.0, 5.0, 
2.0], [4.0, 5.0, 1.0], [2.0, 4.0, 2.0]], [[8.0, 1.0, 6.0], [2.0, 3.0, 4.0], [5.0, 2.0, 2.0], [1.0, 4.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[6.0], [5.0], [6.0], [4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0], [7.0], [8.0], [4.0]]))
      arg[1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 1.0, 6.0], [7.0, 6.0, 4.0], [1.0, 7.0, 8.0], [4.0, 7.0, 1.0]], [[7.0, 4.0, 6.0], [7.0, 
5.0, 5.0], [4.0, 5.0, 6.0], [2.0, 4.0, 4.0]], [[8.0, 1.0, 6.0], [2.0, 3.0, 4.0], [5.0, 2.0, 2.0], [1.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 6.0], [7.0, 6.0, 4.0], [1.0, 7.0, 8.0], [4.0, 7.0, 1.0]], [[7.0, 4.0, 4.0], [7.0, 
5.0, 7.0], [4.0, 5.0, 8.0], [2.0, 4.0, 4.0]], [[8.0, 1.0, 6.0], [2.0, 3.0, 4.0], [5.0, 2.0, 2.0], [1.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RCL(self):
      arg=Data(numpy.array([[[7.0, 7.0, 4.0], [2.0, 4.0, 3.0], [2.0, 4.0, 7.0], [7.0, 3.0, 5.0]], [[6.0, 3.0, 3.0], [2.0, 4.0, 
4.0], [2.0, 4.0, 3.0], [2.0, 4.0, 5.0]], [[4.0, 6.0, 1.0], [6.0, 7.0, 2.0], [3.0, 2.0, 6.0], [5.0, 1.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0], [6.0], [8.0], [7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0], [7.0], [4.0], [8.0]]]))
      arg[1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 7.0, 4.0], [2.0, 4.0, 3.0], [2.0, 4.0, 7.0], [7.0, 3.0, 5.0]], [[6.0, 3.0, 3.0], [2.0, 
4.0, 6.0], [2.0, 4.0, 8.0], [2.0, 4.0, 7.0]], [[4.0, 6.0, 1.0], [6.0, 7.0, 2.0], [3.0, 2.0, 6.0], [5.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 7.0, 4.0], [2.0, 4.0, 3.0], [2.0, 4.0, 7.0], [7.0, 3.0, 5.0]], [[6.0, 3.0, 1.0], [2.0, 
4.0, 7.0], [2.0, 4.0, 4.0], [2.0, 4.0, 8.0]], [[4.0, 6.0, 1.0], [6.0, 7.0, 2.0], [3.0, 2.0, 6.0], [5.0, 1.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PPN(self):
      arg=Data(numpy.array([[[6.0, 6.0, 1.0], [1.0, 1.0, 6.0], [2.0, 1.0, 5.0], [1.0, 7.0, 5.0]], [[8.0, 5.0, 3.0], [5.0, 1.0, 
8.0], [4.0, 6.0, 4.0], [4.0, 4.0, 1.0]], [[8.0, 3.0, 6.0], [8.0, 4.0, 2.0], [8.0, 6.0, 7.0], [5.0, 5.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([2.0, 4.0, 2.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0, 8.0, 1.0]))
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 1.0], [1.0, 1.0, 6.0], [2.0, 1.0, 5.0], [1.0, 7.0, 5.0]], [[8.0, 5.0, 3.0], [5.0, 
1.0, 8.0], [2.0, 4.0, 2.0], [4.0, 4.0, 1.0]], [[8.0, 3.0, 6.0], [8.0, 4.0, 2.0], [8.0, 6.0, 7.0], [5.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 6.0, 1.0], [1.0, 1.0, 6.0], [2.0, 1.0, 5.0], [1.0, 7.0, 5.0]], [[8.0, 5.0, 3.0], [5.0, 
1.0, 8.0], [8.0, 8.0, 1.0], [4.0, 4.0, 1.0]], [[8.0, 3.0, 6.0], [8.0, 4.0, 2.0], [8.0, 6.0, 7.0], [5.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_ULL(self):
      arg=Data(numpy.array([[[2.0, 7.0, 3.0], [4.0, 8.0, 2.0], [8.0, 2.0, 5.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 2.0], [3.0, 7.0, 
6.0], [1.0, 8.0, 3.0], [5.0, 5.0, 2.0]], [[7.0, 3.0, 6.0], [8.0, 7.0, 4.0], [2.0, 3.0, 2.0], [5.0, 5.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0], [1.0]], [[5.0], [8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0], [8.0]], [[7.0], [7.0]]]))
      arg[:2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 3.0], [4.0, 8.0, 2.0], [8.0, 2.0, 4.0], [3.0, 6.0, 1.0]], [[1.0, 2.0, 2.0], [3.0, 
7.0, 6.0], [1.0, 8.0, 5.0], [5.0, 5.0, 8.0]], [[7.0, 3.0, 6.0], [8.0, 7.0, 4.0], [2.0, 3.0, 2.0], [5.0, 5.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 3.0], [4.0, 8.0, 2.0], [8.0, 2.0, 3.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 2.0], [3.0, 
7.0, 6.0], [1.0, 8.0, 7.0], [5.0, 5.0, 7.0]], [[7.0, 3.0, 6.0], [8.0, 7.0, 4.0], [2.0, 3.0, 2.0], [5.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UUC(self):
      arg=Data(numpy.array([[[1.0, 7.0, 7.0], [4.0, 2.0, 4.0], [4.0, 1.0, 5.0], [3.0, 8.0, 2.0]], [[1.0, 1.0, 6.0], [4.0, 8.0, 
1.0], [5.0, 1.0, 2.0], [3.0, 7.0, 5.0]], [[4.0, 5.0, 8.0], [8.0, 2.0, 8.0], [6.0, 5.0, 3.0], [2.0, 3.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 6.0, 6.0], [8.0, 2.0, 2.0], [7.0, 3.0, 3.0]], [[5.0, 2.0, 1.0], [5.0, 1.0, 3.0], [8.0, 7.0, 
8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 8.0, 1.0], [7.0, 6.0, 5.0], [4.0, 3.0, 4.0]], [[7.0, 7.0, 8.0], [2.0, 3.0, 1.0], 
[1.0, 1.0, 2.0]]]))
      arg[:2,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 6.0], [8.0, 2.0, 2.0], [7.0, 3.0, 3.0], [3.0, 8.0, 2.0]], [[5.0, 2.0, 1.0], [5.0, 
1.0, 3.0], [8.0, 7.0, 8.0], [3.0, 7.0, 5.0]], [[4.0, 5.0, 8.0], [8.0, 2.0, 8.0], [6.0, 5.0, 3.0], [2.0, 3.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 1.0], [7.0, 6.0, 5.0], [4.0, 3.0, 4.0], [3.0, 8.0, 2.0]], [[7.0, 7.0, 8.0], [2.0, 
3.0, 1.0], [1.0, 1.0, 2.0], [3.0, 7.0, 5.0]], [[4.0, 5.0, 8.0], [8.0, 2.0, 8.0], [6.0, 5.0, 3.0], [2.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PRC(self):
      arg=Data(numpy.array([[[1.0, 2.0, 2.0], [3.0, 7.0, 6.0], [8.0, 6.0, 1.0], [2.0, 6.0, 5.0]], [[3.0, 6.0, 1.0], [6.0, 7.0, 
5.0], [1.0, 8.0, 1.0], [3.0, 4.0, 1.0]], [[3.0, 4.0, 8.0], [7.0, 7.0, 1.0], [4.0, 3.0, 8.0], [7.0, 8.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 3.0, 1.0], [8.0, 5.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 6.0, 1.0], [4.0, 2.0, 7.0]]))
      arg[1,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 2.0], [3.0, 7.0, 6.0], [8.0, 6.0, 1.0], [2.0, 6.0, 5.0]], [[3.0, 6.0, 1.0], [7.0, 
3.0, 1.0], [8.0, 5.0, 1.0], [3.0, 4.0, 1.0]], [[3.0, 4.0, 8.0], [7.0, 7.0, 1.0], [4.0, 3.0, 8.0], [7.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 2.0], [3.0, 7.0, 6.0], [8.0, 6.0, 1.0], [2.0, 6.0, 5.0]], [[3.0, 6.0, 1.0], [1.0, 
6.0, 1.0], [4.0, 2.0, 7.0], [3.0, 4.0, 1.0]], [[3.0, 4.0, 8.0], [7.0, 7.0, 1.0], [4.0, 3.0, 8.0], [7.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PPL(self):
      arg=Data(numpy.array([[[6.0, 5.0, 6.0], [8.0, 2.0, 8.0], [1.0, 2.0, 5.0], [3.0, 5.0, 3.0]], [[4.0, 8.0, 5.0], [4.0, 5.0, 
1.0], [6.0, 6.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 1.0, 3.0], [7.0, 2.0, 7.0], [6.0, 8.0, 1.0], [6.0, 3.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([3.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([7.0]))
      arg[1,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 6.0], [8.0, 2.0, 8.0], [1.0, 2.0, 5.0], [3.0, 5.0, 3.0]], [[4.0, 8.0, 5.0], [4.0, 
5.0, 1.0], [6.0, 6.0, 3.0], [3.0, 8.0, 8.0]], [[5.0, 1.0, 3.0], [7.0, 2.0, 7.0], [6.0, 8.0, 1.0], [6.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 6.0], [8.0, 2.0, 8.0], [1.0, 2.0, 5.0], [3.0, 5.0, 3.0]], [[4.0, 8.0, 5.0], [4.0, 
5.0, 1.0], [6.0, 6.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 1.0, 3.0], [7.0, 2.0, 7.0], [6.0, 8.0, 1.0], [6.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RLR(self):
      arg=Data(numpy.array([[[3.0, 3.0, 1.0], [4.0, 5.0, 8.0], [1.0, 7.0, 2.0], [7.0, 4.0, 8.0]], [[4.0, 5.0, 4.0], [1.0, 8.0, 
8.0], [8.0, 2.0, 2.0], [5.0, 4.0, 1.0]], [[2.0, 6.0, 7.0], [6.0, 6.0, 4.0], [8.0, 2.0, 8.0], [3.0, 4.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0], [2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0], [6.0]]]))
      arg[1:2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 1.0], [4.0, 5.0, 8.0], [1.0, 7.0, 2.0], [7.0, 4.0, 8.0]], [[4.0, 5.0, 4.0], [1.0, 
8.0, 8.0], [8.0, 4.0, 2.0], [5.0, 2.0, 1.0]], [[2.0, 6.0, 7.0], [6.0, 6.0, 4.0], [8.0, 2.0, 8.0], [3.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 1.0], [4.0, 5.0, 8.0], [1.0, 7.0, 2.0], [7.0, 4.0, 8.0]], [[4.0, 5.0, 4.0], [1.0, 
8.0, 8.0], [8.0, 5.0, 2.0], [5.0, 6.0, 1.0]], [[2.0, 6.0, 7.0], [6.0, 6.0, 4.0], [8.0, 2.0, 8.0], [3.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_URR(self):
      arg=Data(numpy.array([[[1.0, 2.0, 8.0], [2.0, 8.0, 4.0], [1.0, 5.0, 8.0], [3.0, 2.0, 5.0]], [[7.0, 2.0, 6.0], [6.0, 6.0, 
1.0], [6.0, 7.0, 1.0], [4.0, 2.0, 5.0]], [[1.0, 7.0, 6.0], [8.0, 6.0, 6.0], [8.0, 1.0, 4.0], [3.0, 4.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0], [3.0]], [[7.0], [8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0], [6.0]], [[3.0], [8.0]]]))
      arg[:2,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 8.0], [2.0, 6.0, 4.0], [1.0, 3.0, 8.0], [3.0, 2.0, 5.0]], [[7.0, 2.0, 6.0], [6.0, 
7.0, 1.0], [6.0, 8.0, 1.0], [4.0, 2.0, 5.0]], [[1.0, 7.0, 6.0], [8.0, 6.0, 6.0], [8.0, 1.0, 4.0], [3.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 8.0], [2.0, 3.0, 4.0], [1.0, 6.0, 8.0], [3.0, 2.0, 5.0]], [[7.0, 2.0, 6.0], [6.0, 
3.0, 1.0], [6.0, 8.0, 1.0], [4.0, 2.0, 5.0]], [[1.0, 7.0, 6.0], [8.0, 6.0, 6.0], [8.0, 1.0, 4.0], [3.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PRP(self):
      arg=Data(numpy.array([[[6.0, 8.0, 8.0], [7.0, 2.0, 7.0], [1.0, 7.0, 5.0], [4.0, 4.0, 4.0]], [[7.0, 8.0, 4.0], [1.0, 8.0, 
2.0], [8.0, 2.0, 7.0], [3.0, 4.0, 2.0]], [[1.0, 3.0, 2.0], [1.0, 6.0, 5.0], [6.0, 7.0, 8.0], [2.0, 8.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([8.0, 5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([4.0, 2.0]))
      arg[1,1:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 8.0], [7.0, 2.0, 7.0], [1.0, 7.0, 5.0], [4.0, 4.0, 4.0]], [[7.0, 8.0, 4.0], [1.0, 
8.0, 2.0], [8.0, 5.0, 7.0], [3.0, 4.0, 2.0]], [[1.0, 3.0, 2.0], [1.0, 6.0, 5.0], [6.0, 7.0, 8.0], [2.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 8.0], [7.0, 2.0, 7.0], [1.0, 7.0, 5.0], [4.0, 4.0, 4.0]], [[7.0, 8.0, 4.0], [1.0, 
4.0, 2.0], [8.0, 2.0, 7.0], [3.0, 4.0, 2.0]], [[1.0, 3.0, 2.0], [1.0, 6.0, 5.0], [6.0, 7.0, 8.0], [2.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UUU(self):
      arg=Data(numpy.array([[[4.0, 4.0, 3.0], [2.0, 3.0, 4.0], [1.0, 7.0, 7.0], [2.0, 8.0, 4.0]], [[7.0, 2.0, 2.0], [1.0, 6.0, 
3.0], [8.0, 5.0, 4.0], [5.0, 6.0, 5.0]], [[2.0, 6.0, 2.0], [6.0, 5.0, 3.0], [8.0, 8.0, 8.0], [4.0, 7.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 8.0], [7.0, 3.0], [5.0, 6.0]], [[3.0, 2.0], [1.0, 2.0], [7.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 4.0], [6.0, 8.0], [2.0, 7.0]], [[8.0, 5.0], [5.0, 1.0], [2.0, 4.0]]]))
      arg[:2,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 3.0], [7.0, 3.0, 4.0], [5.0, 6.0, 7.0], [2.0, 8.0, 4.0]], [[3.0, 2.0, 2.0], [1.0, 
2.0, 3.0], [7.0, 3.0, 4.0], [5.0, 6.0, 5.0]], [[2.0, 6.0, 2.0], [6.0, 5.0, 3.0], [8.0, 8.0, 8.0], [4.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 4.0, 3.0], [6.0, 8.0, 4.0], [2.0, 7.0, 7.0], [2.0, 8.0, 4.0]], [[8.0, 5.0, 2.0], [5.0, 
1.0, 3.0], [2.0, 4.0, 4.0], [5.0, 6.0, 5.0]], [[2.0, 6.0, 2.0], [6.0, 5.0, 3.0], [8.0, 8.0, 8.0], [4.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PRU(self):
      arg=Data(numpy.array([[[3.0, 1.0, 5.0], [3.0, 2.0, 2.0], [2.0, 5.0, 3.0], [7.0, 4.0, 6.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 
3.0], [8.0, 6.0, 7.0], [2.0, 6.0, 8.0]], [[6.0, 3.0, 7.0], [4.0, 4.0, 7.0], [3.0, 8.0, 6.0], [2.0, 8.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 6.0], [2.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 5.0], [1.0, 6.0]]))
      arg[1,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 5.0], [3.0, 2.0, 2.0], [2.0, 5.0, 3.0], [7.0, 4.0, 6.0]], [[7.0, 2.0, 3.0], [8.0, 
6.0, 3.0], [2.0, 7.0, 7.0], [2.0, 6.0, 8.0]], [[6.0, 3.0, 7.0], [4.0, 4.0, 7.0], [3.0, 8.0, 6.0], [2.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 5.0], [3.0, 2.0, 2.0], [2.0, 5.0, 3.0], [7.0, 4.0, 6.0]], [[7.0, 2.0, 3.0], [2.0, 
5.0, 3.0], [1.0, 6.0, 7.0], [2.0, 6.0, 8.0]], [[6.0, 3.0, 7.0], [4.0, 4.0, 7.0], [3.0, 8.0, 6.0], [2.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LPC(self):
      arg=Data(numpy.array([[[5.0, 8.0, 7.0], [4.0, 2.0, 2.0], [8.0, 8.0, 4.0], [2.0, 7.0, 8.0]], [[1.0, 8.0, 2.0], [2.0, 3.0, 
3.0], [2.0, 5.0, 7.0], [7.0, 2.0, 3.0]], [[4.0, 4.0, 3.0], [5.0, 5.0, 6.0], [4.0, 3.0, 7.0], [6.0, 6.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 4.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 3.0, 7.0]]))
      arg[2:,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 7.0], [4.0, 2.0, 2.0], [8.0, 8.0, 4.0], [2.0, 7.0, 8.0]], [[1.0, 8.0, 2.0], [2.0, 
3.0, 3.0], [2.0, 5.0, 7.0], [7.0, 2.0, 3.0]], [[4.0, 4.0, 3.0], [5.0, 5.0, 6.0], [7.0, 4.0, 2.0], [6.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 7.0], [4.0, 2.0, 2.0], [8.0, 8.0, 4.0], [2.0, 7.0, 8.0]], [[1.0, 8.0, 2.0], [2.0, 
3.0, 3.0], [2.0, 5.0, 7.0], [7.0, 2.0, 3.0]], [[4.0, 4.0, 3.0], [5.0, 5.0, 6.0], [8.0, 3.0, 7.0], [6.0, 6.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UCN(self):
      arg=Data(numpy.array([[[4.0, 3.0, 4.0], [4.0, 5.0, 7.0], [1.0, 2.0, 6.0], [5.0, 6.0, 6.0]], [[2.0, 1.0, 2.0], [1.0, 1.0, 
3.0], [6.0, 4.0, 1.0], [2.0, 1.0, 3.0]], [[4.0, 7.0, 1.0], [6.0, 2.0, 4.0], [1.0, 8.0, 4.0], [7.0, 5.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 5.0, 1.0], [5.0, 5.0, 3.0], [3.0, 7.0, 3.0], [8.0, 2.0, 4.0]], [[4.0, 3.0, 2.0], [6.0, 2.0, 
6.0], [7.0, 4.0, 1.0], [4.0, 5.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0, 3.0], [5.0, 6.0, 7.0], [7.0, 7.0, 8.0], [8.0, 1.0, 2.0]], [[1.0, 5.0, 6.0], 
[8.0, 7.0, 7.0], [8.0, 5.0, 6.0], [2.0, 7.0, 7.0]]]))
      arg[:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 1.0], [5.0, 5.0, 3.0], [3.0, 7.0, 3.0], [8.0, 2.0, 4.0]], [[4.0, 3.0, 2.0], [6.0, 
2.0, 6.0], [7.0, 4.0, 1.0], [4.0, 5.0, 4.0]], [[4.0, 7.0, 1.0], [6.0, 2.0, 4.0], [1.0, 8.0, 4.0], [7.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 3.0], [5.0, 6.0, 7.0], [7.0, 7.0, 8.0], [8.0, 1.0, 2.0]], [[1.0, 5.0, 6.0], [8.0, 
7.0, 7.0], [8.0, 5.0, 6.0], [2.0, 7.0, 7.0]], [[4.0, 7.0, 1.0], [6.0, 2.0, 4.0], [1.0, 8.0, 4.0], [7.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RUL(self):
      arg=Data(numpy.array([[[6.0, 1.0, 2.0], [3.0, 3.0, 2.0], [5.0, 3.0, 8.0], [5.0, 8.0, 7.0]], [[7.0, 3.0, 2.0], [3.0, 5.0, 
1.0], [7.0, 7.0, 4.0], [5.0, 1.0, 1.0]], [[8.0, 8.0, 6.0], [1.0, 7.0, 7.0], [8.0, 5.0, 1.0], [3.0, 7.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0], [4.0], [5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0], [1.0], [3.0]]]))
      arg[1:2,:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 2.0], [3.0, 3.0, 2.0], [5.0, 3.0, 8.0], [5.0, 8.0, 7.0]], [[7.0, 3.0, 3.0], [3.0, 
5.0, 4.0], [7.0, 7.0, 5.0], [5.0, 1.0, 1.0]], [[8.0, 8.0, 6.0], [1.0, 7.0, 7.0], [8.0, 5.0, 1.0], [3.0, 7.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 2.0], [3.0, 3.0, 2.0], [5.0, 3.0, 8.0], [5.0, 8.0, 7.0]], [[7.0, 3.0, 4.0], [3.0, 
5.0, 1.0], [7.0, 7.0, 3.0], [5.0, 1.0, 1.0]], [[8.0, 8.0, 6.0], [1.0, 7.0, 7.0], [8.0, 5.0, 1.0], [3.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_ULU(self):
      arg=Data(numpy.array([[[7.0, 1.0, 3.0], [5.0, 3.0, 3.0], [7.0, 6.0, 2.0], [3.0, 4.0, 8.0]], [[2.0, 3.0, 6.0], [3.0, 6.0, 
7.0], [1.0, 5.0, 6.0], [6.0, 5.0, 6.0]], [[4.0, 5.0, 4.0], [5.0, 1.0, 7.0], [8.0, 5.0, 3.0], [7.0, 4.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 4.0], [2.0, 1.0]], [[7.0, 5.0], [7.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 2.0], [1.0, 5.0]], [[3.0, 5.0], [7.0, 3.0]]]))
      arg[:2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 1.0, 3.0], [5.0, 3.0, 3.0], [2.0, 4.0, 2.0], [2.0, 1.0, 8.0]], [[2.0, 3.0, 6.0], [3.0, 
6.0, 7.0], [7.0, 5.0, 6.0], [7.0, 8.0, 6.0]], [[4.0, 5.0, 4.0], [5.0, 1.0, 7.0], [8.0, 5.0, 3.0], [7.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 3.0], [5.0, 3.0, 3.0], [6.0, 2.0, 2.0], [1.0, 5.0, 8.0]], [[2.0, 3.0, 6.0], [3.0, 
6.0, 7.0], [3.0, 5.0, 6.0], [7.0, 3.0, 6.0]], [[4.0, 5.0, 4.0], [5.0, 1.0, 7.0], [8.0, 5.0, 3.0], [7.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RRP(self):
      arg=Data(numpy.array([[[7.0, 3.0, 6.0], [7.0, 6.0, 6.0], [6.0, 6.0, 5.0], [5.0, 3.0, 8.0]], [[3.0, 4.0, 6.0], [1.0, 2.0, 
2.0], [1.0, 8.0, 5.0], [7.0, 3.0, 8.0]], [[1.0, 6.0, 1.0], [4.0, 6.0, 1.0], [3.0, 1.0, 7.0], [4.0, 4.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[4.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 3.0]]))
      arg[1:2,1:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 6.0], [7.0, 6.0, 6.0], [6.0, 6.0, 5.0], [5.0, 3.0, 8.0]], [[3.0, 4.0, 6.0], [1.0, 
4.0, 2.0], [1.0, 2.0, 5.0], [7.0, 3.0, 8.0]], [[1.0, 6.0, 1.0], [4.0, 6.0, 1.0], [3.0, 1.0, 7.0], [4.0, 4.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 6.0], [7.0, 6.0, 6.0], [6.0, 6.0, 5.0], [5.0, 3.0, 8.0]], [[3.0, 4.0, 6.0], [1.0, 
8.0, 2.0], [1.0, 3.0, 5.0], [7.0, 3.0, 8.0]], [[1.0, 6.0, 1.0], [4.0, 6.0, 1.0], [3.0, 1.0, 7.0], [4.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RPN(self):
      arg=Data(numpy.array([[[8.0, 5.0, 4.0], [1.0, 7.0, 5.0], [2.0, 1.0, 6.0], [4.0, 3.0, 1.0]], [[8.0, 5.0, 6.0], [6.0, 1.0, 
2.0], [6.0, 2.0, 8.0], [4.0, 2.0, 1.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 8.0], [4.0, 2.0, 1.0], [2.0, 8.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 6.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 5.0, 6.0]]))
      arg[1:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 4.0], [1.0, 7.0, 5.0], [2.0, 1.0, 6.0], [4.0, 3.0, 1.0]], [[8.0, 5.0, 6.0], [6.0, 
1.0, 2.0], [1.0, 6.0, 2.0], [4.0, 2.0, 1.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 8.0], [4.0, 2.0, 1.0], [2.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 4.0], [1.0, 7.0, 5.0], [2.0, 1.0, 6.0], [4.0, 3.0, 1.0]], [[8.0, 5.0, 6.0], [6.0, 
1.0, 2.0], [8.0, 5.0, 6.0], [4.0, 2.0, 1.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 8.0], [4.0, 2.0, 1.0], [2.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PRN(self):
      arg=Data(numpy.array([[[1.0, 6.0, 7.0], [8.0, 2.0, 5.0], [3.0, 1.0, 3.0], [1.0, 7.0, 2.0]], [[1.0, 8.0, 5.0], [4.0, 4.0, 
5.0], [4.0, 6.0, 7.0], [5.0, 6.0, 4.0]], [[4.0, 1.0, 8.0], [7.0, 4.0, 6.0], [7.0, 4.0, 5.0], [8.0, 1.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 8.0, 4.0], [1.0, 2.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 5.0, 7.0], [1.0, 4.0, 2.0]]))
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 7.0], [8.0, 2.0, 5.0], [3.0, 1.0, 3.0], [1.0, 7.0, 2.0]], [[1.0, 8.0, 5.0], [1.0, 
8.0, 4.0], [1.0, 2.0, 4.0], [5.0, 6.0, 4.0]], [[4.0, 1.0, 8.0], [7.0, 4.0, 6.0], [7.0, 4.0, 5.0], [8.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 7.0], [8.0, 2.0, 5.0], [3.0, 1.0, 3.0], [1.0, 7.0, 2.0]], [[1.0, 8.0, 5.0], [6.0, 
5.0, 7.0], [1.0, 4.0, 2.0], [5.0, 6.0, 4.0]], [[4.0, 1.0, 8.0], [7.0, 4.0, 6.0], [7.0, 4.0, 5.0], [8.0, 1.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LLR(self):
      arg=Data(numpy.array([[[3.0, 6.0, 1.0], [7.0, 8.0, 2.0], [5.0, 1.0, 8.0], [2.0, 7.0, 1.0]], [[6.0, 7.0, 2.0], [2.0, 8.0, 
2.0], [3.0, 1.0, 4.0], [8.0, 5.0, 2.0]], [[7.0, 4.0, 3.0], [4.0, 7.0, 7.0], [2.0, 4.0, 2.0], [7.0, 3.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[2.0], [3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0], [7.0]]]))
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 1.0], [7.0, 8.0, 2.0], [5.0, 1.0, 8.0], [2.0, 7.0, 1.0]], [[6.0, 7.0, 2.0], [2.0, 
8.0, 2.0], [3.0, 1.0, 4.0], [8.0, 5.0, 2.0]], [[7.0, 4.0, 3.0], [4.0, 7.0, 7.0], [2.0, 2.0, 2.0], [7.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 6.0, 1.0], [7.0, 8.0, 2.0], [5.0, 1.0, 8.0], [2.0, 7.0, 1.0]], [[6.0, 7.0, 2.0], [2.0, 
8.0, 2.0], [3.0, 1.0, 4.0], [8.0, 5.0, 2.0]], [[7.0, 4.0, 3.0], [4.0, 7.0, 7.0], [2.0, 4.0, 2.0], [7.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CCP(self):
      arg=Data(numpy.array([[[2.0, 1.0, 2.0], [2.0, 7.0, 6.0], [1.0, 4.0, 3.0], [4.0, 1.0, 4.0]], [[2.0, 1.0, 3.0], [8.0, 3.0, 
8.0], [5.0, 1.0, 5.0], [6.0, 8.0, 4.0]], [[1.0, 2.0, 4.0], [5.0, 5.0, 3.0], [6.0, 7.0, 6.0], [8.0, 4.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[5.0, 5.0, 2.0, 3.0], [2.0, 6.0, 6.0, 1.0], [3.0, 2.0, 6.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 4.0, 4.0, 3.0], [7.0, 4.0, 5.0, 4.0], [5.0, 2.0, 4.0, 7.0]]))
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 2.0], [2.0, 5.0, 6.0], [1.0, 2.0, 3.0], [4.0, 3.0, 4.0]], [[2.0, 2.0, 3.0], [8.0, 
6.0, 8.0], [5.0, 6.0, 5.0], [6.0, 1.0, 4.0]], [[1.0, 3.0, 4.0], [5.0, 2.0, 3.0], [6.0, 6.0, 6.0], [8.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 2.0], [2.0, 4.0, 6.0], [1.0, 4.0, 3.0], [4.0, 3.0, 4.0]], [[2.0, 7.0, 3.0], [8.0, 
4.0, 8.0], [5.0, 5.0, 5.0], [6.0, 4.0, 4.0]], [[1.0, 5.0, 4.0], [5.0, 2.0, 3.0], [6.0, 4.0, 6.0], [8.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UPR(self):
      arg=Data(numpy.array([[[6.0, 8.0, 7.0], [4.0, 3.0, 5.0], [1.0, 5.0, 5.0], [4.0, 6.0, 4.0]], [[5.0, 7.0, 3.0], [5.0, 3.0, 
3.0], [7.0, 2.0, 6.0], [3.0, 1.0, 8.0]], [[1.0, 1.0, 4.0], [3.0, 4.0, 4.0], [1.0, 7.0, 7.0], [6.0, 3.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[4.0], [1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0], [3.0]]))
      arg[:2,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 7.0], [4.0, 3.0, 5.0], [1.0, 4.0, 5.0], [4.0, 6.0, 4.0]], [[5.0, 7.0, 3.0], [5.0, 
3.0, 3.0], [7.0, 1.0, 6.0], [3.0, 1.0, 8.0]], [[1.0, 1.0, 4.0], [3.0, 4.0, 4.0], [1.0, 7.0, 7.0], [6.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 7.0], [4.0, 3.0, 5.0], [1.0, 2.0, 5.0], [4.0, 6.0, 4.0]], [[5.0, 7.0, 3.0], [5.0, 
3.0, 3.0], [7.0, 3.0, 6.0], [3.0, 1.0, 8.0]], [[1.0, 1.0, 4.0], [3.0, 4.0, 4.0], [1.0, 7.0, 7.0], [6.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PCN(self):
      arg=Data(numpy.array([[[1.0, 7.0, 7.0], [7.0, 8.0, 6.0], [3.0, 8.0, 5.0], [5.0, 1.0, 5.0]], [[3.0, 8.0, 6.0], [5.0, 6.0, 
1.0], [3.0, 7.0, 1.0], [4.0, 2.0, 1.0]], [[2.0, 1.0, 7.0], [8.0, 5.0, 5.0], [1.0, 7.0, 4.0], [5.0, 2.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 5.0, 4.0], [4.0, 6.0, 5.0], [2.0, 5.0, 4.0], [4.0, 5.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 7.0, 6.0], [8.0, 3.0, 5.0], [7.0, 8.0, 2.0], [6.0, 2.0, 3.0]]))
      arg[1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 7.0], [7.0, 8.0, 6.0], [3.0, 8.0, 5.0], [5.0, 1.0, 5.0]], [[7.0, 5.0, 4.0], [4.0, 
6.0, 5.0], [2.0, 5.0, 4.0], [4.0, 5.0, 5.0]], [[2.0, 1.0, 7.0], [8.0, 5.0, 5.0], [1.0, 7.0, 4.0], [5.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 7.0], [7.0, 8.0, 6.0], [3.0, 8.0, 5.0], [5.0, 1.0, 5.0]], [[6.0, 7.0, 6.0], [8.0, 
3.0, 5.0], [7.0, 8.0, 2.0], [6.0, 2.0, 3.0]], [[2.0, 1.0, 7.0], [8.0, 5.0, 5.0], [1.0, 7.0, 4.0], [5.0, 2.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PPC(self):
      arg=Data(numpy.array([[[3.0, 7.0, 2.0], [8.0, 5.0, 1.0], [3.0, 5.0, 3.0], [8.0, 4.0, 4.0]], [[5.0, 6.0, 7.0], [2.0, 2.0, 
7.0], [3.0, 5.0, 5.0], [1.0, 4.0, 3.0]], [[1.0, 7.0, 2.0], [2.0, 7.0, 6.0], [8.0, 1.0, 3.0], [5.0, 3.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([5.0, 2.0, 4.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([5.0, 2.0, 2.0]))
      arg[1,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 2.0], [8.0, 5.0, 1.0], [3.0, 5.0, 3.0], [8.0, 4.0, 4.0]], [[5.0, 6.0, 7.0], [2.0, 
2.0, 7.0], [5.0, 2.0, 4.0], [1.0, 4.0, 3.0]], [[1.0, 7.0, 2.0], [2.0, 7.0, 6.0], [8.0, 1.0, 3.0], [5.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 7.0, 2.0], [8.0, 5.0, 1.0], [3.0, 5.0, 3.0], [8.0, 4.0, 4.0]], [[5.0, 6.0, 7.0], [2.0, 
2.0, 7.0], [5.0, 2.0, 2.0], [1.0, 4.0, 3.0]], [[1.0, 7.0, 2.0], [2.0, 7.0, 6.0], [8.0, 1.0, 3.0], [5.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CPU(self):
      arg=Data(numpy.array([[[5.0, 3.0, 2.0], [6.0, 2.0, 7.0], [8.0, 6.0, 1.0], [6.0, 8.0, 7.0]], [[3.0, 8.0, 4.0], [2.0, 4.0, 
3.0], [8.0, 1.0, 7.0], [2.0, 8.0, 4.0]], [[2.0, 8.0, 8.0], [8.0, 8.0, 4.0], [7.0, 2.0, 1.0], [2.0, 8.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[4.0, 6.0], [5.0, 5.0], [3.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 1.0], [6.0, 4.0], [2.0, 2.0]]))
      arg[:,2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 2.0], [6.0, 2.0, 7.0], [4.0, 6.0, 1.0], [6.0, 8.0, 7.0]], [[3.0, 8.0, 4.0], [2.0, 
4.0, 3.0], [5.0, 5.0, 7.0], [2.0, 8.0, 4.0]], [[2.0, 8.0, 8.0], [8.0, 8.0, 4.0], [3.0, 1.0, 1.0], [2.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 2.0], [6.0, 2.0, 7.0], [8.0, 1.0, 1.0], [6.0, 8.0, 7.0]], [[3.0, 8.0, 4.0], [2.0, 
4.0, 3.0], [6.0, 4.0, 7.0], [2.0, 8.0, 4.0]], [[2.0, 8.0, 8.0], [8.0, 8.0, 4.0], [2.0, 2.0, 1.0], [2.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CCL(self):
      arg=Data(numpy.array([[[5.0, 3.0, 7.0], [6.0, 5.0, 6.0], [1.0, 5.0, 2.0], [8.0, 5.0, 1.0]], [[6.0, 3.0, 1.0], [7.0, 4.0, 
4.0], [3.0, 3.0, 5.0], [4.0, 4.0, 8.0]], [[1.0, 1.0, 2.0], [4.0, 5.0, 6.0], [4.0, 7.0, 2.0], [5.0, 3.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0], [7.0], [1.0], [1.0]], [[5.0], [5.0], [2.0], [7.0]], [[5.0], [8.0], [5.0], 
[8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0], [7.0], [2.0], [3.0]], [[6.0], [7.0], [7.0], [8.0]], [[8.0], [2.0], [1.0], 
[8.0]]]))
      arg[:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 3.0], [6.0, 5.0, 7.0], [1.0, 5.0, 1.0], [8.0, 5.0, 1.0]], [[6.0, 3.0, 5.0], [7.0, 
4.0, 5.0], [3.0, 3.0, 2.0], [4.0, 4.0, 7.0]], [[1.0, 1.0, 5.0], [4.0, 5.0, 8.0], [4.0, 7.0, 5.0], [5.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 6.0], [6.0, 5.0, 7.0], [1.0, 5.0, 2.0], [8.0, 5.0, 3.0]], [[6.0, 3.0, 6.0], [7.0, 
4.0, 7.0], [3.0, 3.0, 7.0], [4.0, 4.0, 8.0]], [[1.0, 1.0, 8.0], [4.0, 5.0, 2.0], [4.0, 7.0, 1.0], [5.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LCN(self):
      arg=Data(numpy.array([[[6.0, 6.0, 6.0], [4.0, 4.0, 8.0], [6.0, 5.0, 7.0], [6.0, 7.0, 7.0]], [[6.0, 6.0, 6.0], [8.0, 7.0, 
1.0], [5.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[7.0, 4.0, 2.0], [8.0, 4.0, 1.0], [3.0, 1.0, 7.0], [6.0, 3.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 3.0, 5.0], [1.0, 4.0, 5.0], [2.0, 6.0, 6.0], [3.0, 2.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 4.0, 2.0], [4.0, 8.0, 7.0], [8.0, 1.0, 3.0], [3.0, 7.0, 2.0]]]))
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 6.0], [4.0, 4.0, 8.0], [6.0, 5.0, 7.0], [6.0, 7.0, 7.0]], [[6.0, 6.0, 6.0], [8.0, 
7.0, 1.0], [5.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[8.0, 3.0, 5.0], [1.0, 4.0, 5.0], [2.0, 6.0, 6.0], [3.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 6.0, 6.0], [4.0, 4.0, 8.0], [6.0, 5.0, 7.0], [6.0, 7.0, 7.0]], [[6.0, 6.0, 6.0], [8.0, 
7.0, 1.0], [5.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[8.0, 4.0, 2.0], [4.0, 8.0, 7.0], [8.0, 1.0, 3.0], [3.0, 7.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CPR(self):
      arg=Data(numpy.array([[[7.0, 6.0, 3.0], [6.0, 2.0, 2.0], [7.0, 7.0, 6.0], [4.0, 4.0, 5.0]], [[8.0, 2.0, 4.0], [5.0, 3.0, 
1.0], [5.0, 3.0, 2.0], [4.0, 6.0, 2.0]], [[4.0, 1.0, 6.0], [4.0, 2.0, 6.0], [5.0, 1.0, 3.0], [1.0, 2.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[3.0], [4.0], [6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0], [2.0], [6.0]]))
      arg[:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 3.0], [6.0, 2.0, 2.0], [7.0, 3.0, 6.0], [4.0, 4.0, 5.0]], [[8.0, 2.0, 4.0], [5.0, 
3.0, 1.0], [5.0, 4.0, 2.0], [4.0, 6.0, 2.0]], [[4.0, 1.0, 6.0], [4.0, 2.0, 6.0], [5.0, 6.0, 3.0], [1.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 3.0], [6.0, 2.0, 2.0], [7.0, 5.0, 6.0], [4.0, 4.0, 5.0]], [[8.0, 2.0, 4.0], [5.0, 
3.0, 1.0], [5.0, 2.0, 2.0], [4.0, 6.0, 2.0]], [[4.0, 1.0, 6.0], [4.0, 2.0, 6.0], [5.0, 6.0, 3.0], [1.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UPN(self):
      arg=Data(numpy.array([[[6.0, 8.0, 2.0], [1.0, 8.0, 6.0], [4.0, 4.0, 2.0], [8.0, 8.0, 8.0]], [[4.0, 8.0, 8.0], [7.0, 5.0, 
1.0], [3.0, 1.0, 8.0], [2.0, 2.0, 1.0]], [[4.0, 3.0, 1.0], [6.0, 8.0, 8.0], [7.0, 4.0, 1.0], [7.0, 3.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 1.0, 3.0], [7.0, 5.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 2.0, 4.0], [8.0, 2.0, 2.0]]))
      arg[:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 2.0], [1.0, 8.0, 6.0], [8.0, 1.0, 3.0], [8.0, 8.0, 8.0]], [[4.0, 8.0, 8.0], [7.0, 
5.0, 1.0], [7.0, 5.0, 6.0], [2.0, 2.0, 1.0]], [[4.0, 3.0, 1.0], [6.0, 8.0, 8.0], [7.0, 4.0, 1.0], [7.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 2.0], [1.0, 8.0, 6.0], [3.0, 2.0, 4.0], [8.0, 8.0, 8.0]], [[4.0, 8.0, 8.0], [7.0, 
5.0, 1.0], [8.0, 2.0, 2.0], [2.0, 2.0, 1.0]], [[4.0, 3.0, 1.0], [6.0, 8.0, 8.0], [7.0, 4.0, 1.0], [7.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LUC(self):
      arg=Data(numpy.array([[[8.0, 1.0, 7.0], [7.0, 7.0, 8.0], [5.0, 2.0, 7.0], [2.0, 1.0, 7.0]], [[6.0, 1.0, 3.0], [8.0, 6.0, 
8.0], [1.0, 8.0, 4.0], [3.0, 2.0, 6.0]], [[7.0, 1.0, 5.0], [6.0, 1.0, 6.0], [4.0, 4.0, 2.0], [7.0, 2.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 1.0, 7.0], [1.0, 6.0, 5.0], [8.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 7.0, 7.0], [5.0, 4.0, 5.0], [3.0, 3.0, 1.0]]]))
      arg[2:,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 1.0, 7.0], [7.0, 7.0, 8.0], [5.0, 2.0, 7.0], [2.0, 1.0, 7.0]], [[6.0, 1.0, 3.0], [8.0, 
6.0, 8.0], [1.0, 8.0, 4.0], [3.0, 2.0, 6.0]], [[4.0, 1.0, 7.0], [1.0, 6.0, 5.0], [8.0, 3.0, 1.0], [7.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 1.0, 7.0], [7.0, 7.0, 8.0], [5.0, 2.0, 7.0], [2.0, 1.0, 7.0]], [[6.0, 1.0, 3.0], [8.0, 
6.0, 8.0], [1.0, 8.0, 4.0], [3.0, 2.0, 6.0]], [[1.0, 7.0, 7.0], [5.0, 4.0, 5.0], [3.0, 3.0, 1.0], [7.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LLN(self):
      arg=Data(numpy.array([[[8.0, 8.0, 4.0], [3.0, 6.0, 4.0], [8.0, 2.0, 2.0], [7.0, 3.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 1.0, 
5.0], [6.0, 6.0, 7.0], [3.0, 8.0, 1.0]], [[6.0, 2.0, 8.0], [1.0, 4.0, 5.0], [4.0, 6.0, 6.0], [6.0, 2.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 5.0, 8.0], [2.0, 2.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 2.0, 6.0], [6.0, 2.0, 1.0]]]))
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 8.0, 4.0], [3.0, 6.0, 4.0], [8.0, 2.0, 2.0], [7.0, 3.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 
1.0, 5.0], [6.0, 6.0, 7.0], [3.0, 8.0, 1.0]], [[6.0, 2.0, 8.0], [1.0, 4.0, 5.0], [4.0, 5.0, 8.0], [2.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 4.0], [3.0, 6.0, 4.0], [8.0, 2.0, 2.0], [7.0, 3.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 
1.0, 5.0], [6.0, 6.0, 7.0], [3.0, 8.0, 1.0]], [[6.0, 2.0, 8.0], [1.0, 4.0, 5.0], [8.0, 2.0, 6.0], [6.0, 2.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RPL(self):
      arg=Data(numpy.array([[[4.0, 2.0, 6.0], [1.0, 8.0, 7.0], [5.0, 3.0, 5.0], [1.0, 2.0, 6.0]], [[4.0, 4.0, 3.0], [6.0, 5.0, 
5.0], [6.0, 1.0, 7.0], [4.0, 7.0, 3.0]], [[1.0, 4.0, 6.0], [8.0, 3.0, 7.0], [8.0, 8.0, 6.0], [5.0, 5.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0]]))
      arg[1:2,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 6.0], [1.0, 8.0, 7.0], [5.0, 3.0, 5.0], [1.0, 2.0, 6.0]], [[4.0, 4.0, 3.0], [6.0, 
5.0, 5.0], [6.0, 1.0, 3.0], [4.0, 7.0, 3.0]], [[1.0, 4.0, 6.0], [8.0, 3.0, 7.0], [8.0, 8.0, 6.0], [5.0, 5.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 6.0], [1.0, 8.0, 7.0], [5.0, 3.0, 5.0], [1.0, 2.0, 6.0]], [[4.0, 4.0, 3.0], [6.0, 
5.0, 5.0], [6.0, 1.0, 5.0], [4.0, 7.0, 3.0]], [[1.0, 4.0, 6.0], [8.0, 3.0, 7.0], [8.0, 8.0, 6.0], [5.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CLN(self):
      arg=Data(numpy.array([[[3.0, 2.0, 7.0], [8.0, 3.0, 1.0], [2.0, 4.0, 8.0], [5.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [1.0, 4.0, 
7.0], [8.0, 8.0, 6.0], [7.0, 6.0, 2.0]], [[7.0, 8.0, 6.0], [8.0, 4.0, 3.0], [3.0, 4.0, 4.0], [1.0, 4.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 8.0, 6.0], [2.0, 1.0, 3.0]], [[7.0, 8.0, 1.0], [3.0, 8.0, 2.0]], [[5.0, 7.0, 5.0], [4.0, 
3.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 8.0, 4.0], [3.0, 2.0, 5.0]], [[4.0, 7.0, 5.0], [2.0, 5.0, 5.0]], [[7.0, 4.0, 
8.0], [5.0, 3.0, 4.0]]]))
      arg[:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 7.0], [8.0, 3.0, 1.0], [3.0, 8.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 8.0, 4.0], [1.0, 
4.0, 7.0], [7.0, 8.0, 1.0], [3.0, 8.0, 2.0]], [[7.0, 8.0, 6.0], [8.0, 4.0, 3.0], [5.0, 7.0, 5.0], [4.0, 3.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 7.0], [8.0, 3.0, 1.0], [7.0, 8.0, 4.0], [3.0, 2.0, 5.0]], [[4.0, 8.0, 4.0], [1.0, 
4.0, 7.0], [4.0, 7.0, 5.0], [2.0, 5.0, 5.0]], [[7.0, 8.0, 6.0], [8.0, 4.0, 3.0], [7.0, 4.0, 8.0], [5.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UCR(self):
      arg=Data(numpy.array([[[2.0, 5.0, 1.0], [2.0, 1.0, 5.0], [8.0, 2.0, 5.0], [5.0, 7.0, 7.0]], [[2.0, 1.0, 6.0], [6.0, 5.0, 
1.0], [1.0, 7.0, 6.0], [3.0, 8.0, 6.0]], [[5.0, 5.0, 7.0], [2.0, 3.0, 5.0], [5.0, 6.0, 3.0], [2.0, 5.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0], [5.0], [1.0], [6.0]], [[8.0], [8.0], [2.0], [2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0], [8.0], [8.0], [1.0]], [[3.0], [8.0], [7.0], [6.0]]]))
      arg[:2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 1.0], [2.0, 5.0, 5.0], [8.0, 1.0, 5.0], [5.0, 6.0, 7.0]], [[2.0, 8.0, 6.0], [6.0, 
8.0, 1.0], [1.0, 2.0, 6.0], [3.0, 2.0, 6.0]], [[5.0, 5.0, 7.0], [2.0, 3.0, 5.0], [5.0, 6.0, 3.0], [2.0, 5.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 1.0], [2.0, 8.0, 5.0], [8.0, 8.0, 5.0], [5.0, 1.0, 7.0]], [[2.0, 3.0, 6.0], [6.0, 
8.0, 1.0], [1.0, 7.0, 6.0], [3.0, 6.0, 6.0]], [[5.0, 5.0, 7.0], [2.0, 3.0, 5.0], [5.0, 6.0, 3.0], [2.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_ULN(self):
      arg=Data(numpy.array([[[3.0, 8.0, 4.0], [7.0, 7.0, 6.0], [6.0, 4.0, 6.0], [1.0, 5.0, 8.0]], [[5.0, 4.0, 7.0], [8.0, 4.0, 
3.0], [7.0, 2.0, 4.0], [5.0, 3.0, 4.0]], [[6.0, 7.0, 5.0], [8.0, 2.0, 6.0], [8.0, 1.0, 1.0], [7.0, 1.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[3.0, 7.0, 3.0], [8.0, 6.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 7.0, 8.0], [5.0, 3.0, 2.0]], [[2.0, 8.0, 2.0], [5.0, 4.0, 8.0]]]))
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 4.0], [7.0, 7.0, 6.0], [6.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[5.0, 4.0, 7.0], [8.0, 
4.0, 3.0], [3.0, 7.0, 3.0], [8.0, 6.0, 4.0]], [[6.0, 7.0, 5.0], [8.0, 2.0, 6.0], [8.0, 1.0, 1.0], [7.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 4.0], [7.0, 7.0, 6.0], [5.0, 7.0, 8.0], [5.0, 3.0, 2.0]], [[5.0, 4.0, 7.0], [8.0, 
4.0, 3.0], [2.0, 8.0, 2.0], [5.0, 4.0, 8.0]], [[6.0, 7.0, 5.0], [8.0, 2.0, 6.0], [8.0, 1.0, 1.0], [7.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PUN(self):
      arg=Data(numpy.array([[[2.0, 5.0, 4.0], [6.0, 7.0, 3.0], [1.0, 8.0, 4.0], [3.0, 7.0, 7.0]], [[3.0, 3.0, 5.0], [3.0, 2.0, 
8.0], [3.0, 2.0, 8.0], [5.0, 8.0, 3.0]], [[5.0, 5.0, 6.0], [4.0, 8.0, 7.0], [2.0, 2.0, 8.0], [6.0, 5.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 3.0, 3.0], [7.0, 6.0, 4.0], [2.0, 5.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 7.0, 2.0], [1.0, 3.0, 5.0], [6.0, 8.0, 3.0]]))
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 4.0], [6.0, 7.0, 3.0], [1.0, 8.0, 4.0], [3.0, 7.0, 7.0]], [[3.0, 3.0, 3.0], [7.0, 
6.0, 4.0], [2.0, 5.0, 1.0], [5.0, 8.0, 3.0]], [[5.0, 5.0, 6.0], [4.0, 8.0, 7.0], [2.0, 2.0, 8.0], [6.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 4.0], [6.0, 7.0, 3.0], [1.0, 8.0, 4.0], [3.0, 7.0, 7.0]], [[5.0, 7.0, 2.0], [1.0, 
3.0, 5.0], [6.0, 8.0, 3.0], [5.0, 8.0, 3.0]], [[5.0, 5.0, 6.0], [4.0, 8.0, 7.0], [2.0, 2.0, 8.0], [6.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RCN(self):
      arg=Data(numpy.array([[[7.0, 3.0, 7.0], [5.0, 3.0, 3.0], [3.0, 7.0, 8.0], [7.0, 2.0, 3.0]], [[6.0, 7.0, 6.0], [2.0, 6.0, 
1.0], [4.0, 8.0, 3.0], [6.0, 8.0, 8.0]], [[3.0, 6.0, 3.0], [5.0, 6.0, 6.0], [4.0, 2.0, 6.0], [4.0, 1.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 8.0, 2.0], [6.0, 6.0, 4.0], [3.0, 1.0, 2.0], [1.0, 6.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 5.0, 5.0], [4.0, 3.0, 2.0], [8.0, 7.0, 5.0], [4.0, 3.0, 6.0]]]))
      arg[1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 7.0], [5.0, 3.0, 3.0], [3.0, 7.0, 8.0], [7.0, 2.0, 3.0]], [[7.0, 8.0, 2.0], [6.0, 
6.0, 4.0], [3.0, 1.0, 2.0], [1.0, 6.0, 2.0]], [[3.0, 6.0, 3.0], [5.0, 6.0, 6.0], [4.0, 2.0, 6.0], [4.0, 1.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 7.0], [5.0, 3.0, 3.0], [3.0, 7.0, 8.0], [7.0, 2.0, 3.0]], [[7.0, 5.0, 5.0], [4.0, 
3.0, 2.0], [8.0, 7.0, 5.0], [4.0, 3.0, 6.0]], [[3.0, 6.0, 3.0], [5.0, 6.0, 6.0], [4.0, 2.0, 6.0], [4.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CRN(self):
      arg=Data(numpy.array([[[5.0, 3.0, 6.0], [2.0, 3.0, 8.0], [2.0, 6.0, 8.0], [8.0, 8.0, 4.0]], [[7.0, 2.0, 4.0], [7.0, 6.0, 
3.0], [2.0, 8.0, 3.0], [4.0, 1.0, 1.0]], [[5.0, 3.0, 4.0], [4.0, 3.0, 1.0], [1.0, 1.0, 1.0], [7.0, 5.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 1.0, 8.0], [5.0, 3.0, 5.0]], [[5.0, 6.0, 1.0], [5.0, 8.0, 3.0]], [[5.0, 8.0, 5.0], [5.0, 
3.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 2.0, 8.0], [5.0, 7.0, 3.0]], [[4.0, 1.0, 1.0], [7.0, 1.0, 3.0]], [[5.0, 8.0, 
3.0], [7.0, 4.0, 7.0]]]))
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 6.0], [6.0, 1.0, 8.0], [5.0, 3.0, 5.0], [8.0, 8.0, 4.0]], [[7.0, 2.0, 4.0], [5.0, 
6.0, 1.0], [5.0, 8.0, 3.0], [4.0, 1.0, 1.0]], [[5.0, 3.0, 4.0], [5.0, 8.0, 5.0], [5.0, 3.0, 7.0], [7.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 6.0], [6.0, 2.0, 8.0], [5.0, 7.0, 3.0], [8.0, 8.0, 4.0]], [[7.0, 2.0, 4.0], [4.0, 
1.0, 1.0], [7.0, 1.0, 3.0], [4.0, 1.0, 1.0]], [[5.0, 3.0, 4.0], [5.0, 8.0, 3.0], [7.0, 4.0, 7.0], [7.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UUN(self):
      arg=Data(numpy.array([[[8.0, 4.0, 8.0], [2.0, 3.0, 3.0], [2.0, 1.0, 4.0], [3.0, 1.0, 1.0]], [[2.0, 3.0, 6.0], [3.0, 7.0, 
3.0], [4.0, 7.0, 2.0], [6.0, 2.0, 3.0]], [[1.0, 3.0, 6.0], [2.0, 4.0, 4.0], [8.0, 2.0, 7.0], [2.0, 8.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 8.0, 4.0], [4.0, 4.0, 5.0], [1.0, 2.0, 3.0]], [[7.0, 5.0, 3.0], [8.0, 2.0, 6.0], [4.0, 6.0, 
5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 3.0, 3.0], [3.0, 3.0, 1.0], [1.0, 4.0, 1.0]], [[7.0, 5.0, 6.0], [6.0, 5.0, 2.0], 
[6.0, 7.0, 2.0]]]))
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 4.0], [4.0, 4.0, 5.0], [1.0, 2.0, 3.0], [3.0, 1.0, 1.0]], [[7.0, 5.0, 3.0], [8.0, 
2.0, 6.0], [4.0, 6.0, 5.0], [6.0, 2.0, 3.0]], [[1.0, 3.0, 6.0], [2.0, 4.0, 4.0], [8.0, 2.0, 7.0], [2.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 3.0], [3.0, 3.0, 1.0], [1.0, 4.0, 1.0], [3.0, 1.0, 1.0]], [[7.0, 5.0, 6.0], [6.0, 
5.0, 2.0], [6.0, 7.0, 2.0], [6.0, 2.0, 3.0]], [[1.0, 3.0, 6.0], [2.0, 4.0, 4.0], [8.0, 2.0, 7.0], [2.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UCP(self):
      arg=Data(numpy.array([[[5.0, 6.0, 4.0], [6.0, 4.0, 3.0], [7.0, 1.0, 2.0], [8.0, 2.0, 6.0]], [[6.0, 4.0, 1.0], [8.0, 1.0, 
4.0], [7.0, 4.0, 2.0], [3.0, 6.0, 8.0]], [[1.0, 1.0, 4.0], [8.0, 6.0, 3.0], [6.0, 3.0, 4.0], [6.0, 2.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 4.0, 2.0, 7.0], [1.0, 3.0, 1.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 7.0, 1.0, 5.0], [7.0, 3.0, 7.0, 7.0]]))
      arg[:2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 4.0], [6.0, 4.0, 3.0], [7.0, 2.0, 2.0], [8.0, 7.0, 6.0]], [[6.0, 1.0, 1.0], [8.0, 
3.0, 4.0], [7.0, 1.0, 2.0], [3.0, 6.0, 8.0]], [[1.0, 1.0, 4.0], [8.0, 6.0, 3.0], [6.0, 3.0, 4.0], [6.0, 2.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 4.0], [6.0, 7.0, 3.0], [7.0, 1.0, 2.0], [8.0, 5.0, 6.0]], [[6.0, 7.0, 1.0], [8.0, 
3.0, 4.0], [7.0, 7.0, 2.0], [3.0, 7.0, 8.0]], [[1.0, 1.0, 4.0], [8.0, 6.0, 3.0], [6.0, 3.0, 4.0], [6.0, 2.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RUN(self):
      arg=Data(numpy.array([[[1.0, 5.0, 2.0], [6.0, 7.0, 5.0], [2.0, 8.0, 7.0], [2.0, 4.0, 2.0]], [[3.0, 6.0, 8.0], [2.0, 8.0, 
7.0], [8.0, 4.0, 5.0], [2.0, 1.0, 7.0]], [[4.0, 4.0, 4.0], [4.0, 1.0, 3.0], [8.0, 8.0, 8.0], [7.0, 2.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 3.0, 8.0], [3.0, 3.0, 1.0], [4.0, 2.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 6.0, 3.0], [8.0, 1.0, 7.0], [1.0, 4.0, 3.0]]]))
      arg[1:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 2.0], [6.0, 7.0, 5.0], [2.0, 8.0, 7.0], [2.0, 4.0, 2.0]], [[3.0, 3.0, 8.0], [3.0, 
3.0, 1.0], [4.0, 2.0, 4.0], [2.0, 1.0, 7.0]], [[4.0, 4.0, 4.0], [4.0, 1.0, 3.0], [8.0, 8.0, 8.0], [7.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 2.0], [6.0, 7.0, 5.0], [2.0, 8.0, 7.0], [2.0, 4.0, 2.0]], [[5.0, 6.0, 3.0], [8.0, 
1.0, 7.0], [1.0, 4.0, 3.0], [2.0, 1.0, 7.0]], [[4.0, 4.0, 4.0], [4.0, 1.0, 3.0], [8.0, 8.0, 8.0], [7.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CUN(self):
      arg=Data(numpy.array([[[5.0, 4.0, 2.0], [8.0, 8.0, 6.0], [3.0, 2.0, 3.0], [8.0, 2.0, 4.0]], [[7.0, 1.0, 6.0], [3.0, 1.0, 
4.0], [6.0, 6.0, 4.0], [8.0, 8.0, 4.0]], [[4.0, 2.0, 6.0], [5.0, 5.0, 5.0], [3.0, 4.0, 1.0], [2.0, 6.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 4.0, 1.0], [2.0, 6.0, 8.0], [7.0, 7.0, 4.0]], [[2.0, 1.0, 5.0], [7.0, 7.0, 1.0], [6.0, 6.0, 
6.0]], [[4.0, 7.0, 4.0], [3.0, 4.0, 5.0], [2.0, 4.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 5.0, 1.0], [4.0, 1.0, 3.0], [3.0, 2.0, 5.0]], [[2.0, 8.0, 7.0], [6.0, 4.0, 1.0], 
[8.0, 4.0, 6.0]], [[6.0, 1.0, 3.0], [2.0, 4.0, 1.0], [6.0, 3.0, 2.0]]]))
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 1.0], [2.0, 6.0, 8.0], [7.0, 7.0, 4.0], [8.0, 2.0, 4.0]], [[2.0, 1.0, 5.0], [7.0, 
7.0, 1.0], [6.0, 6.0, 6.0], [8.0, 8.0, 4.0]], [[4.0, 7.0, 4.0], [3.0, 4.0, 5.0], [2.0, 4.0, 1.0], [2.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 1.0], [4.0, 1.0, 3.0], [3.0, 2.0, 5.0], [8.0, 2.0, 4.0]], [[2.0, 8.0, 7.0], [6.0, 
4.0, 1.0], [8.0, 4.0, 6.0], [8.0, 8.0, 4.0]], [[6.0, 1.0, 3.0], [2.0, 4.0, 1.0], [6.0, 3.0, 2.0], [2.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LLC(self):
      arg=Data(numpy.array([[[2.0, 6.0, 6.0], [2.0, 6.0, 5.0], [1.0, 6.0, 8.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 8.0], [5.0, 2.0, 
5.0], [2.0, 5.0, 4.0], [5.0, 1.0, 1.0]], [[3.0, 5.0, 8.0], [3.0, 3.0, 4.0], [7.0, 7.0, 3.0], [8.0, 5.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 4.0, 6.0], [8.0, 5.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 4.0, 5.0], [3.0, 6.0, 7.0]]]))
      arg[2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 6.0, 6.0], [2.0, 6.0, 5.0], [1.0, 6.0, 8.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 8.0], [5.0, 
2.0, 5.0], [2.0, 5.0, 4.0], [5.0, 1.0, 1.0]], [[3.0, 5.0, 8.0], [3.0, 3.0, 4.0], [5.0, 4.0, 6.0], [8.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 6.0], [2.0, 6.0, 5.0], [1.0, 6.0, 8.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 8.0], [5.0, 
2.0, 5.0], [2.0, 5.0, 4.0], [5.0, 1.0, 1.0]], [[3.0, 5.0, 8.0], [3.0, 3.0, 4.0], [2.0, 4.0, 5.0], [3.0, 6.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RRN(self):
      arg=Data(numpy.array([[[4.0, 3.0, 1.0], [2.0, 2.0, 6.0], [7.0, 8.0, 7.0], [5.0, 4.0, 3.0]], [[5.0, 6.0, 8.0], [3.0, 3.0, 
8.0], [1.0, 4.0, 1.0], [3.0, 8.0, 1.0]], [[4.0, 8.0, 4.0], [2.0, 5.0, 7.0], [7.0, 5.0, 1.0], [5.0, 3.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 7.0, 3.0], [7.0, 7.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 4.0, 5.0], [7.0, 8.0, 2.0]]]))
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 1.0], [2.0, 2.0, 6.0], [7.0, 8.0, 7.0], [5.0, 4.0, 3.0]], [[5.0, 6.0, 8.0], [5.0, 
7.0, 3.0], [7.0, 7.0, 4.0], [3.0, 8.0, 1.0]], [[4.0, 8.0, 4.0], [2.0, 5.0, 7.0], [7.0, 5.0, 1.0], [5.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 1.0], [2.0, 2.0, 6.0], [7.0, 8.0, 7.0], [5.0, 4.0, 3.0]], [[5.0, 6.0, 8.0], [1.0, 
4.0, 5.0], [7.0, 8.0, 2.0], [3.0, 8.0, 1.0]], [[4.0, 8.0, 4.0], [2.0, 5.0, 7.0], [7.0, 5.0, 1.0], [5.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LUN(self):
      arg=Data(numpy.array([[[7.0, 3.0, 4.0], [3.0, 4.0, 3.0], [8.0, 7.0, 6.0], [8.0, 4.0, 7.0]], [[3.0, 8.0, 8.0], [5.0, 2.0, 
4.0], [3.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[3.0, 2.0, 7.0], [7.0, 5.0, 1.0], [6.0, 1.0, 1.0], [3.0, 3.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 3.0, 8.0], [6.0, 6.0, 3.0], [6.0, 6.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 2.0, 3.0], [1.0, 4.0, 8.0], [2.0, 3.0, 7.0]]]))
      arg[2:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 4.0], [3.0, 4.0, 3.0], [8.0, 7.0, 6.0], [8.0, 4.0, 7.0]], [[3.0, 8.0, 8.0], [5.0, 
2.0, 4.0], [3.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[3.0, 3.0, 8.0], [6.0, 6.0, 3.0], [6.0, 6.0, 7.0], [3.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 4.0], [3.0, 4.0, 3.0], [8.0, 7.0, 6.0], [8.0, 4.0, 7.0]], [[3.0, 8.0, 8.0], [5.0, 
2.0, 4.0], [3.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[5.0, 2.0, 3.0], [1.0, 4.0, 8.0], [2.0, 3.0, 7.0], [3.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_RLN(self):
      arg=Data(numpy.array([[[5.0, 8.0, 4.0], [6.0, 2.0, 6.0], [1.0, 6.0, 6.0], [1.0, 7.0, 2.0]], [[6.0, 3.0, 3.0], [3.0, 1.0, 
5.0], [1.0, 7.0, 4.0], [8.0, 2.0, 8.0]], [[1.0, 1.0, 4.0], [3.0, 2.0, 6.0], [8.0, 7.0, 8.0], [2.0, 1.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 8.0, 8.0], [7.0, 4.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 4.0, 3.0], [8.0, 4.0, 4.0]]]))
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 4.0], [6.0, 2.0, 6.0], [1.0, 6.0, 6.0], [1.0, 7.0, 2.0]], [[6.0, 3.0, 3.0], [3.0, 
1.0, 5.0], [6.0, 8.0, 8.0], [7.0, 4.0, 4.0]], [[1.0, 1.0, 4.0], [3.0, 2.0, 6.0], [8.0, 7.0, 8.0], [2.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 4.0], [6.0, 2.0, 6.0], [1.0, 6.0, 6.0], [1.0, 7.0, 2.0]], [[6.0, 3.0, 3.0], [3.0, 
1.0, 5.0], [3.0, 4.0, 3.0], [8.0, 4.0, 4.0]], [[1.0, 1.0, 4.0], [3.0, 2.0, 6.0], [8.0, 7.0, 8.0], [2.0, 1.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PLL(self):
      arg=Data(numpy.array([[[6.0, 1.0, 2.0], [6.0, 3.0, 5.0], [6.0, 7.0, 1.0], [3.0, 3.0, 5.0]], [[6.0, 2.0, 5.0], [5.0, 8.0, 
6.0], [6.0, 4.0, 1.0], [3.0, 2.0, 5.0]], [[8.0, 8.0, 7.0], [7.0, 1.0, 1.0], [7.0, 5.0, 1.0], [1.0, 1.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[7.0], [8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0], [2.0]]))
      arg[1,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 2.0], [6.0, 3.0, 5.0], [6.0, 7.0, 1.0], [3.0, 3.0, 5.0]], [[6.0, 2.0, 5.0], [5.0, 
8.0, 6.0], [6.0, 4.0, 7.0], [3.0, 2.0, 8.0]], [[8.0, 8.0, 7.0], [7.0, 1.0, 1.0], [7.0, 5.0, 1.0], [1.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 2.0], [6.0, 3.0, 5.0], [6.0, 7.0, 1.0], [3.0, 3.0, 5.0]], [[6.0, 2.0, 5.0], [5.0, 
8.0, 6.0], [6.0, 4.0, 3.0], [3.0, 2.0, 2.0]], [[8.0, 8.0, 7.0], [7.0, 1.0, 1.0], [7.0, 5.0, 1.0], [1.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_PLN(self):
      arg=Data(numpy.array([[[3.0, 2.0, 6.0], [4.0, 8.0, 3.0], [4.0, 2.0, 1.0], [8.0, 3.0, 1.0]], [[4.0, 2.0, 3.0], [7.0, 7.0, 
6.0], [5.0, 5.0, 4.0], [8.0, 7.0, 8.0]], [[7.0, 1.0, 1.0], [4.0, 7.0, 2.0], [3.0, 7.0, 2.0], [8.0, 8.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 5.0, 4.0], [7.0, 3.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 2.0, 6.0], [7.0, 4.0, 2.0]]))
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 6.0], [4.0, 8.0, 3.0], [4.0, 2.0, 1.0], [8.0, 3.0, 1.0]], [[4.0, 2.0, 3.0], [7.0, 
7.0, 6.0], [1.0, 5.0, 4.0], [7.0, 3.0, 3.0]], [[7.0, 1.0, 1.0], [4.0, 7.0, 2.0], [3.0, 7.0, 2.0], [8.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 6.0], [4.0, 8.0, 3.0], [4.0, 2.0, 1.0], [8.0, 3.0, 1.0]], [[4.0, 2.0, 3.0], [7.0, 
7.0, 6.0], [3.0, 2.0, 6.0], [7.0, 4.0, 2.0]], [[7.0, 1.0, 1.0], [4.0, 7.0, 2.0], [3.0, 7.0, 2.0], [8.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LPN(self):
      arg=Data(numpy.array([[[7.0, 5.0, 8.0], [5.0, 8.0, 2.0], [5.0, 7.0, 8.0], [2.0, 6.0, 4.0]], [[8.0, 1.0, 2.0], [7.0, 5.0, 
3.0], [6.0, 5.0, 3.0], [1.0, 8.0, 3.0]], [[8.0, 5.0, 1.0], [2.0, 1.0, 2.0], [8.0, 3.0, 6.0], [8.0, 4.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 1.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 4.0, 7.0]]))
      arg[2:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 8.0], [5.0, 8.0, 2.0], [5.0, 7.0, 8.0], [2.0, 6.0, 4.0]], [[8.0, 1.0, 2.0], [7.0, 
5.0, 3.0], [6.0, 5.0, 3.0], [1.0, 8.0, 3.0]], [[8.0, 5.0, 1.0], [2.0, 1.0, 2.0], [7.0, 1.0, 3.0], [8.0, 4.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 8.0], [5.0, 8.0, 2.0], [5.0, 7.0, 8.0], [2.0, 6.0, 4.0]], [[8.0, 1.0, 2.0], [7.0, 
5.0, 3.0], [6.0, 5.0, 3.0], [1.0, 8.0, 3.0]], [[8.0, 5.0, 1.0], [2.0, 1.0, 2.0], [8.0, 4.0, 7.0], [8.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CCC(self):
      arg=Data(numpy.array([[[6.0, 3.0, 8.0], [1.0, 6.0, 1.0], [5.0, 7.0, 7.0], [2.0, 8.0, 2.0]], [[8.0, 8.0, 5.0], [8.0, 7.0, 
2.0], [1.0, 6.0, 4.0], [8.0, 2.0, 8.0]], [[2.0, 6.0, 8.0], [2.0, 4.0, 4.0], [6.0, 5.0, 1.0], [7.0, 5.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 2.0, 2.0], [8.0, 1.0, 5.0], [2.0, 4.0, 1.0], [7.0, 6.0, 6.0]], [[1.0, 7.0, 5.0], [3.0, 4.0, 
2.0], [6.0, 7.0, 5.0], [1.0, 8.0, 6.0]], [[2.0, 4.0, 5.0], [3.0, 1.0, 5.0], [5.0, 1.0, 6.0], [2.0, 6.0, 
4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 3.0, 4.0], [8.0, 2.0, 1.0], [5.0, 1.0, 3.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 1.0], 
[1.0, 6.0, 5.0], [8.0, 3.0, 6.0], [5.0, 3.0, 6.0]], [[6.0, 3.0, 3.0], [1.0, 5.0, 5.0], [5.0, 5.0, 1.0], [6.0, 4.0, 8.0]]]))
      arg[:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 2.0], [8.0, 1.0, 5.0], [2.0, 4.0, 1.0], [7.0, 6.0, 6.0]], [[1.0, 7.0, 5.0], [3.0, 
4.0, 2.0], [6.0, 7.0, 5.0], [1.0, 8.0, 6.0]], [[2.0, 4.0, 5.0], [3.0, 1.0, 5.0], [5.0, 1.0, 6.0], [2.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 4.0], [8.0, 2.0, 1.0], [5.0, 1.0, 3.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 1.0], [1.0, 
6.0, 5.0], [8.0, 3.0, 6.0], [5.0, 3.0, 6.0]], [[6.0, 3.0, 3.0], [1.0, 5.0, 5.0], [5.0, 5.0, 1.0], [6.0, 4.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_LRC(self):
      arg=Data(numpy.array([[[8.0, 3.0, 4.0], [5.0, 8.0, 4.0], [5.0, 2.0, 7.0], [7.0, 6.0, 7.0]], [[2.0, 2.0, 1.0], [1.0, 3.0, 
2.0], [2.0, 3.0, 3.0], [4.0, 4.0, 8.0]], [[4.0, 4.0, 1.0], [2.0, 6.0, 1.0], [5.0, 5.0, 2.0], [5.0, 6.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 2.0, 8.0], [3.0, 2.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 1.0, 8.0], [5.0, 3.0, 7.0]]]))
      arg[2:,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 4.0], [5.0, 8.0, 4.0], [5.0, 2.0, 7.0], [7.0, 6.0, 7.0]], [[2.0, 2.0, 1.0], [1.0, 
3.0, 2.0], [2.0, 3.0, 3.0], [4.0, 4.0, 8.0]], [[4.0, 4.0, 1.0], [4.0, 2.0, 8.0], [3.0, 2.0, 6.0], [5.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 4.0], [5.0, 8.0, 4.0], [5.0, 2.0, 7.0], [7.0, 6.0, 7.0]], [[2.0, 2.0, 1.0], [1.0, 
3.0, 2.0], [2.0, 3.0, 3.0], [4.0, 4.0, 8.0]], [[4.0, 4.0, 1.0], [2.0, 1.0, 8.0], [5.0, 3.0, 7.0], [5.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_CCN(self):
      arg=Data(numpy.array([[[2.0, 2.0, 4.0], [4.0, 3.0, 3.0], [2.0, 8.0, 5.0], [4.0, 5.0, 2.0]], [[2.0, 5.0, 8.0], [7.0, 1.0, 
1.0], [4.0, 6.0, 7.0], [6.0, 6.0, 2.0]], [[7.0, 4.0, 4.0], [1.0, 8.0, 8.0], [8.0, 1.0, 7.0], [5.0, 7.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 7.0, 8.0], [6.0, 8.0, 6.0], [8.0, 3.0, 6.0], [7.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], [8.0, 7.0, 
8.0], [4.0, 1.0, 2.0], [1.0, 8.0, 7.0]], [[2.0, 4.0, 8.0], [1.0, 7.0, 7.0], [7.0, 4.0, 4.0], [8.0, 5.0, 
1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 7.0, 7.0], [3.0, 4.0, 5.0], [4.0, 6.0, 6.0], [4.0, 1.0, 3.0]], [[7.0, 5.0, 8.0], 
[6.0, 8.0, 5.0], [5.0, 2.0, 2.0], [7.0, 6.0, 5.0]], [[2.0, 8.0, 8.0], [6.0, 2.0, 4.0], [7.0, 6.0, 1.0], [4.0, 5.0, 6.0]]]))
      arg[:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 8.0], [6.0, 8.0, 6.0], [8.0, 3.0, 6.0], [7.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], [8.0, 
7.0, 8.0], [4.0, 1.0, 2.0], [1.0, 8.0, 7.0]], [[2.0, 4.0, 8.0], [1.0, 7.0, 7.0], [7.0, 4.0, 4.0], [8.0, 5.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 7.0, 7.0], [3.0, 4.0, 5.0], [4.0, 6.0, 6.0], [4.0, 1.0, 3.0]], [[7.0, 5.0, 8.0], [6.0, 
8.0, 5.0], [5.0, 2.0, 2.0], [7.0, 6.0, 5.0]], [[2.0, 8.0, 8.0], [6.0, 2.0, 4.0], [7.0, 6.0, 1.0], [4.0, 5.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_taggedData_UPL(self):
      arg=Data(numpy.array([[[7.0, 2.0, 5.0], [2.0, 7.0, 3.0], [2.0, 7.0, 4.0], [1.0, 3.0, 1.0]], [[5.0, 8.0, 8.0], [2.0, 2.0, 
1.0], [4.0, 5.0, 5.0], [6.0, 2.0, 3.0]], [[4.0, 3.0, 4.0], [2.0, 4.0, 4.0], [2.0, 1.0, 7.0], [7.0, 8.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[3.0], [6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0], [6.0]]))
      arg[:2,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 5.0], [2.0, 7.0, 3.0], [2.0, 7.0, 3.0], [1.0, 3.0, 1.0]], [[5.0, 8.0, 8.0], [2.0, 
2.0, 1.0], [4.0, 5.0, 6.0], [6.0, 2.0, 3.0]], [[4.0, 3.0, 4.0], [2.0, 4.0, 4.0], [2.0, 1.0, 7.0], [7.0, 8.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 5.0], [2.0, 7.0, 3.0], [2.0, 7.0, 5.0], [1.0, 3.0, 1.0]], [[5.0, 8.0, 8.0], [2.0, 
2.0, 1.0], [4.0, 5.0, 6.0], [6.0, 2.0, 3.0]], [[4.0, 3.0, 4.0], [2.0, 4.0, 4.0], [2.0, 1.0, 7.0], [7.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLCP(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 6.0], [6.0, 4.0, 2.0], [5.0, 7.0, 8.0]], [[5.0, 7.0, 3.0], [1.0, 8.0, 6.0], [1.0, 3.0, 
5.0]], [[8.0, 3.0, 3.0], [2.0, 6.0, 5.0], [3.0, 2.0, 6.0]]], [[[5.0, 3.0, 3.0], [2.0, 6.0, 1.0], [3.0, 6.0, 7.0]], [[8.0, 8.0, 
2.0], [5.0, 4.0, 4.0], [3.0, 3.0, 4.0]], [[3.0, 3.0, 8.0], [5.0, 2.0, 8.0], [8.0, 1.0, 5.0]]], [[[3.0, 6.0, 7.0], [6.0, 7.0, 
1.0], [2.0, 1.0, 2.0]], [[1.0, 6.0, 3.0], [2.0, 5.0, 8.0], [6.0, 1.0, 6.0]], [[6.0, 3.0, 5.0], [4.0, 6.0, 4.0], [1.0, 2.0, 
4.0]]], [[[5.0, 8.0, 6.0], [8.0, 2.0, 6.0], [1.0, 2.0, 3.0]], [[6.0, 8.0, 5.0], [1.0, 3.0, 8.0], [6.0, 8.0, 3.0]], [[5.0, 4.0, 
6.0], [1.0, 5.0, 4.0], [6.0, 6.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 3.0, 8.0]], [[6.0, 2.0, 1.0]], [[7.0, 4.0, 8.0]], [[2.0, 2.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 3.0, 8.0]], [[8.0, 8.0, 4.0]], [[7.0, 4.0, 3.0]], [[6.0, 1.0, 2.0]]]))
      arg[:,2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 6.0], [6.0, 4.0, 2.0], [5.0, 7.0, 8.0]], [[5.0, 7.0, 3.0], [1.0, 8.0, 6.0], [1.0, 
3.0, 5.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 5.0], [3.0, 8.0, 6.0]]], [[[5.0, 3.0, 3.0], [2.0, 6.0, 1.0], [3.0, 6.0, 7.0]], [[8.0, 
8.0, 2.0], [5.0, 4.0, 4.0], [3.0, 3.0, 4.0]], [[3.0, 6.0, 8.0], [5.0, 2.0, 8.0], [8.0, 1.0, 5.0]]], [[[3.0, 6.0, 7.0], [6.0, 
7.0, 1.0], [2.0, 1.0, 2.0]], [[1.0, 6.0, 3.0], [2.0, 5.0, 8.0], [6.0, 1.0, 6.0]], [[6.0, 7.0, 5.0], [4.0, 4.0, 4.0], [1.0, 8.0, 
4.0]]], [[[5.0, 8.0, 6.0], [8.0, 2.0, 6.0], [1.0, 2.0, 3.0]], [[6.0, 8.0, 5.0], [1.0, 3.0, 8.0], [6.0, 8.0, 3.0]], [[5.0, 2.0, 
6.0], [1.0, 2.0, 4.0], [6.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 6.0], [6.0, 4.0, 2.0], [5.0, 7.0, 8.0]], 
[[5.0, 7.0, 3.0], [1.0, 8.0, 6.0], [1.0, 3.0, 5.0]], [[8.0, 6.0, 3.0], [2.0, 3.0, 5.0], [3.0, 8.0, 6.0]]], [[[5.0, 3.0, 3.0], 
[2.0, 6.0, 1.0], [3.0, 6.0, 7.0]], [[8.0, 8.0, 2.0], [5.0, 4.0, 4.0], [3.0, 3.0, 4.0]], [[3.0, 8.0, 8.0], [5.0, 8.0, 8.0], 
[8.0, 4.0, 5.0]]], [[[3.0, 6.0, 7.0], [6.0, 7.0, 1.0], [2.0, 1.0, 2.0]], [[1.0, 6.0, 3.0], [2.0, 5.0, 8.0], [6.0, 1.0, 6.0]], 
[[6.0, 7.0, 5.0], [4.0, 4.0, 4.0], [1.0, 3.0, 4.0]]], [[[5.0, 8.0, 6.0], [8.0, 2.0, 6.0], [1.0, 2.0, 3.0]], [[6.0, 8.0, 5.0], 
[1.0, 3.0, 8.0], [6.0, 8.0, 3.0]], [[5.0, 6.0, 6.0], [1.0, 1.0, 4.0], [6.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RUCP(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 5.0], [1.0, 3.0, 5.0], [8.0, 6.0, 5.0]], [[5.0, 4.0, 3.0], [4.0, 3.0, 5.0], [4.0, 5.0, 
4.0]], [[3.0, 2.0, 5.0], [4.0, 8.0, 2.0], [8.0, 2.0, 1.0]]], [[[1.0, 4.0, 5.0], [7.0, 8.0, 1.0], [7.0, 2.0, 2.0]], [[6.0, 1.0, 
6.0], [8.0, 3.0, 4.0], [3.0, 5.0, 5.0]], [[5.0, 2.0, 7.0], [3.0, 7.0, 5.0], [7.0, 7.0, 8.0]]], [[[2.0, 7.0, 6.0], [2.0, 8.0, 
8.0], [8.0, 4.0, 5.0]], [[5.0, 6.0, 7.0], [3.0, 2.0, 3.0], [1.0, 5.0, 3.0]], [[8.0, 7.0, 3.0], [7.0, 7.0, 2.0], [8.0, 5.0, 
1.0]]], [[[6.0, 7.0, 8.0], [1.0, 2.0, 2.0], [5.0, 2.0, 7.0]], [[3.0, 2.0, 8.0], [6.0, 7.0, 2.0], [6.0, 5.0, 1.0]], [[4.0, 4.0, 
7.0], [6.0, 8.0, 3.0], [7.0, 1.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 8.0, 8.0], [1.0, 8.0, 4.0]], [[6.0, 6.0, 6.0], [5.0, 6.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 1.0, 3.0], [3.0, 3.0, 4.0]], [[6.0, 6.0, 7.0], [4.0, 6.0, 3.0]]]))
      arg[1:3,:2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 5.0], [1.0, 3.0, 5.0], [8.0, 6.0, 5.0]], [[5.0, 4.0, 3.0], [4.0, 3.0, 5.0], [4.0, 
5.0, 4.0]], [[3.0, 2.0, 5.0], [4.0, 8.0, 2.0], [8.0, 2.0, 1.0]]], [[[1.0, 7.0, 5.0], [7.0, 8.0, 1.0], [7.0, 8.0, 2.0]], [[6.0, 
1.0, 6.0], [8.0, 8.0, 4.0], [3.0, 4.0, 5.0]], [[5.0, 2.0, 7.0], [3.0, 7.0, 5.0], [7.0, 7.0, 8.0]]], [[[2.0, 6.0, 6.0], [2.0, 
6.0, 8.0], [8.0, 6.0, 5.0]], [[5.0, 5.0, 7.0], [3.0, 6.0, 3.0], [1.0, 6.0, 3.0]], [[8.0, 7.0, 3.0], [7.0, 7.0, 2.0], [8.0, 5.0, 
1.0]]], [[[6.0, 7.0, 8.0], [1.0, 2.0, 2.0], [5.0, 2.0, 7.0]], [[3.0, 2.0, 8.0], [6.0, 7.0, 2.0], [6.0, 5.0, 1.0]], [[4.0, 4.0, 
7.0], [6.0, 8.0, 3.0], [7.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 5.0], [1.0, 3.0, 5.0], [8.0, 6.0, 5.0]], 
[[5.0, 4.0, 3.0], [4.0, 3.0, 5.0], [4.0, 5.0, 4.0]], [[3.0, 2.0, 5.0], [4.0, 8.0, 2.0], [8.0, 2.0, 1.0]]], [[[1.0, 8.0, 5.0], 
[7.0, 1.0, 1.0], [7.0, 3.0, 2.0]], [[6.0, 3.0, 6.0], [8.0, 3.0, 4.0], [3.0, 4.0, 5.0]], [[5.0, 2.0, 7.0], [3.0, 7.0, 5.0], 
[7.0, 7.0, 8.0]]], [[[2.0, 6.0, 6.0], [2.0, 6.0, 8.0], [8.0, 7.0, 5.0]], [[5.0, 4.0, 7.0], [3.0, 6.0, 3.0], [1.0, 3.0, 3.0]], 
[[8.0, 7.0, 3.0], [7.0, 7.0, 2.0], [8.0, 5.0, 1.0]]], [[[6.0, 7.0, 8.0], [1.0, 2.0, 2.0], [5.0, 2.0, 7.0]], [[3.0, 2.0, 8.0], 
[6.0, 7.0, 2.0], [6.0, 5.0, 1.0]], [[4.0, 4.0, 7.0], [6.0, 8.0, 3.0], [7.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CRRR(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 1.0], [8.0, 3.0, 7.0], [3.0, 2.0, 3.0]], [[7.0, 5.0, 7.0], [2.0, 2.0, 7.0], [2.0, 5.0, 
6.0]], [[8.0, 7.0, 4.0], [3.0, 6.0, 6.0], [3.0, 2.0, 7.0]]], [[[3.0, 5.0, 3.0], [2.0, 2.0, 1.0], [7.0, 8.0, 6.0]], [[1.0, 8.0, 
5.0], [2.0, 5.0, 6.0], [7.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [5.0, 3.0, 5.0], [6.0, 2.0, 8.0]]], [[[1.0, 7.0, 6.0], [8.0, 6.0, 
2.0], [3.0, 4.0, 1.0]], [[3.0, 1.0, 6.0], [7.0, 7.0, 6.0], [4.0, 1.0, 2.0]], [[5.0, 4.0, 5.0], [4.0, 4.0, 6.0], [8.0, 5.0, 
4.0]]], [[[2.0, 5.0, 5.0], [5.0, 3.0, 4.0], [6.0, 6.0, 5.0]], [[2.0, 3.0, 3.0], [3.0, 8.0, 2.0], [7.0, 4.0, 7.0]], [[8.0, 6.0, 
4.0], [5.0, 8.0, 5.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0]]], [[[1.0]]], [[[8.0]]], [[[7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0]]], [[[8.0]]], [[[1.0]]], [[[5.0]]]]))
      arg[:,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 1.0], [8.0, 3.0, 7.0], [3.0, 2.0, 3.0]], [[7.0, 5.0, 7.0], [2.0, 3.0, 7.0], [2.0, 
5.0, 6.0]], [[8.0, 7.0, 4.0], [3.0, 6.0, 6.0], [3.0, 2.0, 7.0]]], [[[3.0, 5.0, 3.0], [2.0, 2.0, 1.0], [7.0, 8.0, 6.0]], [[1.0, 
8.0, 5.0], [2.0, 1.0, 6.0], [7.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [5.0, 3.0, 5.0], [6.0, 2.0, 8.0]]], [[[1.0, 7.0, 6.0], [8.0, 
6.0, 2.0], [3.0, 4.0, 1.0]], [[3.0, 1.0, 6.0], [7.0, 8.0, 6.0], [4.0, 1.0, 2.0]], [[5.0, 4.0, 5.0], [4.0, 4.0, 6.0], [8.0, 5.0, 
4.0]]], [[[2.0, 5.0, 5.0], [5.0, 3.0, 4.0], [6.0, 6.0, 5.0]], [[2.0, 3.0, 3.0], [3.0, 7.0, 2.0], [7.0, 4.0, 7.0]], [[8.0, 6.0, 
4.0], [5.0, 8.0, 5.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 1.0], [8.0, 3.0, 7.0], [3.0, 2.0, 3.0]], 
[[7.0, 5.0, 7.0], [2.0, 8.0, 7.0], [2.0, 5.0, 6.0]], [[8.0, 7.0, 4.0], [3.0, 6.0, 6.0], [3.0, 2.0, 7.0]]], [[[3.0, 5.0, 3.0], 
[2.0, 2.0, 1.0], [7.0, 8.0, 6.0]], [[1.0, 8.0, 5.0], [2.0, 8.0, 6.0], [7.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [5.0, 3.0, 5.0], 
[6.0, 2.0, 8.0]]], [[[1.0, 7.0, 6.0], [8.0, 6.0, 2.0], [3.0, 4.0, 1.0]], [[3.0, 1.0, 6.0], [7.0, 1.0, 6.0], [4.0, 1.0, 2.0]], 
[[5.0, 4.0, 5.0], [4.0, 4.0, 6.0], [8.0, 5.0, 4.0]]], [[[2.0, 5.0, 5.0], [5.0, 3.0, 4.0], [6.0, 6.0, 5.0]], [[2.0, 3.0, 3.0], 
[3.0, 5.0, 2.0], [7.0, 4.0, 7.0]], [[8.0, 6.0, 4.0], [5.0, 8.0, 5.0], [7.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PCRR(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 1.0], [2.0, 2.0, 5.0], [5.0, 8.0, 8.0]], [[7.0, 7.0, 8.0], [3.0, 8.0, 3.0], [3.0, 5.0, 
2.0]], [[7.0, 2.0, 6.0], [3.0, 4.0, 2.0], [6.0, 3.0, 2.0]]], [[[7.0, 3.0, 7.0], [5.0, 6.0, 6.0], [6.0, 6.0, 8.0]], [[5.0, 4.0, 
3.0], [4.0, 3.0, 4.0], [4.0, 5.0, 3.0]], [[4.0, 7.0, 2.0], [7.0, 2.0, 5.0], [4.0, 3.0, 6.0]]], [[[7.0, 4.0, 4.0], [8.0, 7.0, 
4.0], [7.0, 7.0, 8.0]], [[8.0, 2.0, 8.0], [3.0, 5.0, 8.0], [4.0, 2.0, 5.0]], [[1.0, 4.0, 2.0], [5.0, 4.0, 3.0], [3.0, 5.0, 
5.0]]], [[[6.0, 1.0, 7.0], [7.0, 5.0, 5.0], [3.0, 2.0, 8.0]], [[2.0, 3.0, 2.0], [3.0, 6.0, 5.0], [6.0, 7.0, 6.0]], [[1.0, 3.0, 
1.0], [8.0, 7.0, 7.0], [1.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0]], [[4.0]], [[6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0]], [[7.0]], [[7.0]]]))
      arg[2,:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 1.0], [2.0, 2.0, 5.0], [5.0, 8.0, 8.0]], [[7.0, 7.0, 8.0], [3.0, 8.0, 3.0], [3.0, 
5.0, 2.0]], [[7.0, 2.0, 6.0], [3.0, 4.0, 2.0], [6.0, 3.0, 2.0]]], [[[7.0, 3.0, 7.0], [5.0, 6.0, 6.0], [6.0, 6.0, 8.0]], [[5.0, 
4.0, 3.0], [4.0, 3.0, 4.0], [4.0, 5.0, 3.0]], [[4.0, 7.0, 2.0], [7.0, 2.0, 5.0], [4.0, 3.0, 6.0]]], [[[7.0, 4.0, 4.0], [8.0, 
5.0, 4.0], [7.0, 7.0, 8.0]], [[8.0, 2.0, 8.0], [3.0, 4.0, 8.0], [4.0, 2.0, 5.0]], [[1.0, 4.0, 2.0], [5.0, 6.0, 3.0], [3.0, 5.0, 
5.0]]], [[[6.0, 1.0, 7.0], [7.0, 5.0, 5.0], [3.0, 2.0, 8.0]], [[2.0, 3.0, 2.0], [3.0, 6.0, 5.0], [6.0, 7.0, 6.0]], [[1.0, 3.0, 
1.0], [8.0, 7.0, 7.0], [1.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 1.0], [2.0, 2.0, 5.0], [5.0, 8.0, 8.0]], 
[[7.0, 7.0, 8.0], [3.0, 8.0, 3.0], [3.0, 5.0, 2.0]], [[7.0, 2.0, 6.0], [3.0, 4.0, 2.0], [6.0, 3.0, 2.0]]], [[[7.0, 3.0, 7.0], 
[5.0, 6.0, 6.0], [6.0, 6.0, 8.0]], [[5.0, 4.0, 3.0], [4.0, 3.0, 4.0], [4.0, 5.0, 3.0]], [[4.0, 7.0, 2.0], [7.0, 2.0, 5.0], 
[4.0, 3.0, 6.0]]], [[[7.0, 4.0, 4.0], [8.0, 7.0, 4.0], [7.0, 7.0, 8.0]], [[8.0, 2.0, 8.0], [3.0, 7.0, 8.0], [4.0, 2.0, 5.0]], 
[[1.0, 4.0, 2.0], [5.0, 7.0, 3.0], [3.0, 5.0, 5.0]]], [[[6.0, 1.0, 7.0], [7.0, 5.0, 5.0], [3.0, 2.0, 8.0]], [[2.0, 3.0, 2.0], 
[3.0, 6.0, 5.0], [6.0, 7.0, 6.0]], [[1.0, 3.0, 1.0], [8.0, 7.0, 7.0], [1.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PURN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 6.0], [6.0, 3.0, 7.0], [7.0, 4.0, 3.0]], [[5.0, 7.0, 6.0], [6.0, 4.0, 4.0], [5.0, 1.0, 
5.0]], [[1.0, 7.0, 7.0], [6.0, 7.0, 5.0], [3.0, 7.0, 4.0]]], [[[1.0, 3.0, 7.0], [3.0, 5.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 7.0, 
7.0], [7.0, 4.0, 4.0], [3.0, 5.0, 5.0]], [[6.0, 4.0, 1.0], [8.0, 5.0, 2.0], [8.0, 3.0, 7.0]]], [[[2.0, 1.0, 5.0], [6.0, 6.0, 
3.0], [7.0, 5.0, 6.0]], [[3.0, 1.0, 8.0], [1.0, 5.0, 4.0], [4.0, 6.0, 3.0]], [[1.0, 7.0, 6.0], [6.0, 6.0, 7.0], [6.0, 8.0, 
1.0]]], [[[1.0, 4.0, 6.0], [7.0, 2.0, 6.0], [5.0, 5.0, 7.0]], [[4.0, 1.0, 2.0], [8.0, 6.0, 5.0], [8.0, 8.0, 7.0]], [[1.0, 3.0, 
1.0], [7.0, 4.0, 1.0], [8.0, 7.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 1.0, 4.0]], [[5.0, 4.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 1.0, 3.0]], [[8.0, 1.0, 5.0]]]))
      arg[2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 6.0], [6.0, 3.0, 7.0], [7.0, 4.0, 3.0]], [[5.0, 7.0, 6.0], [6.0, 4.0, 4.0], [5.0, 
1.0, 5.0]], [[1.0, 7.0, 7.0], [6.0, 7.0, 5.0], [3.0, 7.0, 4.0]]], [[[1.0, 3.0, 7.0], [3.0, 5.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 
7.0, 7.0], [7.0, 4.0, 4.0], [3.0, 5.0, 5.0]], [[6.0, 4.0, 1.0], [8.0, 5.0, 2.0], [8.0, 3.0, 7.0]]], [[[2.0, 1.0, 5.0], [6.0, 
1.0, 4.0], [7.0, 5.0, 6.0]], [[3.0, 1.0, 8.0], [5.0, 4.0, 6.0], [4.0, 6.0, 3.0]], [[1.0, 7.0, 6.0], [6.0, 6.0, 7.0], [6.0, 8.0, 
1.0]]], [[[1.0, 4.0, 6.0], [7.0, 2.0, 6.0], [5.0, 5.0, 7.0]], [[4.0, 1.0, 2.0], [8.0, 6.0, 5.0], [8.0, 8.0, 7.0]], [[1.0, 3.0, 
1.0], [7.0, 4.0, 1.0], [8.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 6.0], [6.0, 3.0, 7.0], [7.0, 4.0, 3.0]], 
[[5.0, 7.0, 6.0], [6.0, 4.0, 4.0], [5.0, 1.0, 5.0]], [[1.0, 7.0, 7.0], [6.0, 7.0, 5.0], [3.0, 7.0, 4.0]]], [[[1.0, 3.0, 7.0], 
[3.0, 5.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 7.0, 7.0], [7.0, 4.0, 4.0], [3.0, 5.0, 5.0]], [[6.0, 4.0, 1.0], [8.0, 5.0, 2.0], 
[8.0, 3.0, 7.0]]], [[[2.0, 1.0, 5.0], [2.0, 1.0, 3.0], [7.0, 5.0, 6.0]], [[3.0, 1.0, 8.0], [8.0, 1.0, 5.0], [4.0, 6.0, 3.0]], 
[[1.0, 7.0, 6.0], [6.0, 6.0, 7.0], [6.0, 8.0, 1.0]]], [[[1.0, 4.0, 6.0], [7.0, 2.0, 6.0], [5.0, 5.0, 7.0]], [[4.0, 1.0, 2.0], 
[8.0, 6.0, 5.0], [8.0, 8.0, 7.0]], [[1.0, 3.0, 1.0], [7.0, 4.0, 1.0], [8.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LURL(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 6.0], [1.0, 2.0, 5.0], [4.0, 6.0, 1.0]], [[7.0, 5.0, 2.0], [4.0, 3.0, 4.0], [4.0, 8.0, 
7.0]], [[5.0, 7.0, 4.0], [3.0, 3.0, 3.0], [7.0, 4.0, 6.0]]], [[[6.0, 4.0, 1.0], [8.0, 2.0, 2.0], [2.0, 2.0, 7.0]], [[6.0, 2.0, 
4.0], [6.0, 3.0, 4.0], [2.0, 6.0, 5.0]], [[8.0, 5.0, 8.0], [8.0, 8.0, 6.0], [8.0, 7.0, 4.0]]], [[[3.0, 7.0, 7.0], [2.0, 2.0, 
2.0], [6.0, 5.0, 2.0]], [[1.0, 1.0, 3.0], [7.0, 6.0, 1.0], [3.0, 8.0, 3.0]], [[4.0, 8.0, 3.0], [1.0, 3.0, 3.0], [8.0, 2.0, 
5.0]]], [[[6.0, 3.0, 7.0], [7.0, 5.0, 6.0], [3.0, 7.0, 5.0]], [[7.0, 5.0, 6.0], [3.0, 3.0, 5.0], [3.0, 4.0, 5.0]], [[7.0, 4.0, 
4.0], [3.0, 6.0, 7.0], [8.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0]], [[5.0]]], [[[1.0]], [[4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0]], [[4.0]]], [[[1.0]], [[1.0]]]]))
      arg[2:,:2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 6.0], [1.0, 2.0, 5.0], [4.0, 6.0, 1.0]], [[7.0, 5.0, 2.0], [4.0, 3.0, 4.0], [4.0, 
8.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 3.0, 3.0], [7.0, 4.0, 6.0]]], [[[6.0, 4.0, 1.0], [8.0, 2.0, 2.0], [2.0, 2.0, 7.0]], [[6.0, 
2.0, 4.0], [6.0, 3.0, 4.0], [2.0, 6.0, 5.0]], [[8.0, 5.0, 8.0], [8.0, 8.0, 6.0], [8.0, 7.0, 4.0]]], [[[3.0, 7.0, 7.0], [2.0, 
2.0, 8.0], [6.0, 5.0, 2.0]], [[1.0, 1.0, 3.0], [7.0, 6.0, 5.0], [3.0, 8.0, 3.0]], [[4.0, 8.0, 3.0], [1.0, 3.0, 3.0], [8.0, 2.0, 
5.0]]], [[[6.0, 3.0, 7.0], [7.0, 5.0, 1.0], [3.0, 7.0, 5.0]], [[7.0, 5.0, 6.0], [3.0, 3.0, 4.0], [3.0, 4.0, 5.0]], [[7.0, 4.0, 
4.0], [3.0, 6.0, 7.0], [8.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 6.0], [1.0, 2.0, 5.0], [4.0, 6.0, 1.0]], 
[[7.0, 5.0, 2.0], [4.0, 3.0, 4.0], [4.0, 8.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 3.0, 3.0], [7.0, 4.0, 6.0]]], [[[6.0, 4.0, 1.0], 
[8.0, 2.0, 2.0], [2.0, 2.0, 7.0]], [[6.0, 2.0, 4.0], [6.0, 3.0, 4.0], [2.0, 6.0, 5.0]], [[8.0, 5.0, 8.0], [8.0, 8.0, 6.0], 
[8.0, 7.0, 4.0]]], [[[3.0, 7.0, 7.0], [2.0, 2.0, 4.0], [6.0, 5.0, 2.0]], [[1.0, 1.0, 3.0], [7.0, 6.0, 4.0], [3.0, 8.0, 3.0]], 
[[4.0, 8.0, 3.0], [1.0, 3.0, 3.0], [8.0, 2.0, 5.0]]], [[[6.0, 3.0, 7.0], [7.0, 5.0, 1.0], [3.0, 7.0, 5.0]], [[7.0, 5.0, 6.0], 
[3.0, 3.0, 1.0], [3.0, 4.0, 5.0]], [[7.0, 4.0, 4.0], [3.0, 6.0, 7.0], [8.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CPPC(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 1.0], [2.0, 1.0, 4.0], [3.0, 7.0, 8.0]], [[4.0, 8.0, 3.0], [1.0, 1.0, 6.0], [6.0, 7.0, 
5.0]], [[3.0, 7.0, 5.0], [4.0, 6.0, 7.0], [3.0, 5.0, 8.0]]], [[[4.0, 4.0, 6.0], [6.0, 1.0, 2.0], [7.0, 3.0, 2.0]], [[1.0, 8.0, 
1.0], [3.0, 4.0, 5.0], [1.0, 4.0, 3.0]], [[1.0, 8.0, 4.0], [3.0, 5.0, 5.0], [2.0, 2.0, 6.0]]], [[[7.0, 1.0, 6.0], [6.0, 3.0, 
8.0], [4.0, 6.0, 2.0]], [[5.0, 8.0, 2.0], [6.0, 3.0, 5.0], [4.0, 4.0, 2.0]], [[6.0, 3.0, 3.0], [3.0, 2.0, 7.0], [4.0, 7.0, 
1.0]]], [[[5.0, 5.0, 8.0], [4.0, 6.0, 5.0], [2.0, 1.0, 7.0]], [[3.0, 8.0, 8.0], [3.0, 5.0, 1.0], [8.0, 1.0, 6.0]], [[8.0, 2.0, 
6.0], [3.0, 6.0, 2.0], [4.0, 3.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 1.0, 7.0], [8.0, 8.0, 4.0], [1.0, 2.0, 8.0], [3.0, 5.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 7.0, 4.0], [7.0, 3.0, 1.0], [3.0, 1.0, 8.0], [5.0, 5.0, 8.0]]))
      arg[:,1,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 1.0], [2.0, 1.0, 4.0], [3.0, 7.0, 8.0]], [[4.0, 8.0, 3.0], [7.0, 1.0, 7.0], [6.0, 
7.0, 5.0]], [[3.0, 7.0, 5.0], [4.0, 6.0, 7.0], [3.0, 5.0, 8.0]]], [[[4.0, 4.0, 6.0], [6.0, 1.0, 2.0], [7.0, 3.0, 2.0]], [[1.0, 
8.0, 1.0], [8.0, 8.0, 4.0], [1.0, 4.0, 3.0]], [[1.0, 8.0, 4.0], [3.0, 5.0, 5.0], [2.0, 2.0, 6.0]]], [[[7.0, 1.0, 6.0], [6.0, 
3.0, 8.0], [4.0, 6.0, 2.0]], [[5.0, 8.0, 2.0], [1.0, 2.0, 8.0], [4.0, 4.0, 2.0]], [[6.0, 3.0, 3.0], [3.0, 2.0, 7.0], [4.0, 7.0, 
1.0]]], [[[5.0, 5.0, 8.0], [4.0, 6.0, 5.0], [2.0, 1.0, 7.0]], [[3.0, 8.0, 8.0], [3.0, 5.0, 3.0], [8.0, 1.0, 6.0]], [[8.0, 2.0, 
6.0], [3.0, 6.0, 2.0], [4.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 1.0], [2.0, 1.0, 4.0], [3.0, 7.0, 8.0]], 
[[4.0, 8.0, 3.0], [5.0, 7.0, 4.0], [6.0, 7.0, 5.0]], [[3.0, 7.0, 5.0], [4.0, 6.0, 7.0], [3.0, 5.0, 8.0]]], [[[4.0, 4.0, 6.0], 
[6.0, 1.0, 2.0], [7.0, 3.0, 2.0]], [[1.0, 8.0, 1.0], [7.0, 3.0, 1.0], [1.0, 4.0, 3.0]], [[1.0, 8.0, 4.0], [3.0, 5.0, 5.0], 
[2.0, 2.0, 6.0]]], [[[7.0, 1.0, 6.0], [6.0, 3.0, 8.0], [4.0, 6.0, 2.0]], [[5.0, 8.0, 2.0], [3.0, 1.0, 8.0], [4.0, 4.0, 2.0]], 
[[6.0, 3.0, 3.0], [3.0, 2.0, 7.0], [4.0, 7.0, 1.0]]], [[[5.0, 5.0, 8.0], [4.0, 6.0, 5.0], [2.0, 1.0, 7.0]], [[3.0, 8.0, 8.0], 
[5.0, 5.0, 8.0], [8.0, 1.0, 6.0]], [[8.0, 2.0, 6.0], [3.0, 6.0, 2.0], [4.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLRR(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 4.0], [5.0, 4.0, 3.0], [5.0, 6.0, 4.0]], [[2.0, 8.0, 3.0], [6.0, 3.0, 7.0], [1.0, 6.0, 
1.0]], [[5.0, 6.0, 2.0], [6.0, 5.0, 8.0], [7.0, 3.0, 1.0]]], [[[3.0, 2.0, 7.0], [3.0, 4.0, 8.0], [8.0, 1.0, 1.0]], [[3.0, 3.0, 
6.0], [2.0, 4.0, 3.0], [7.0, 4.0, 2.0]], [[2.0, 4.0, 4.0], [1.0, 4.0, 6.0], [3.0, 1.0, 8.0]]], [[[8.0, 5.0, 7.0], [1.0, 6.0, 
8.0], [4.0, 7.0, 7.0]], [[7.0, 7.0, 2.0], [3.0, 2.0, 1.0], [1.0, 5.0, 6.0]], [[4.0, 8.0, 6.0], [8.0, 6.0, 6.0], [7.0, 5.0, 
5.0]]], [[[7.0, 5.0, 5.0], [7.0, 3.0, 6.0], [3.0, 1.0, 3.0]], [[3.0, 7.0, 7.0], [1.0, 3.0, 7.0], [6.0, 4.0, 6.0]], [[8.0, 2.0, 
3.0], [2.0, 3.0, 6.0], [6.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0]]]))
      arg[2,2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 4.0], [5.0, 4.0, 3.0], [5.0, 6.0, 4.0]], [[2.0, 8.0, 3.0], [6.0, 3.0, 7.0], [1.0, 
6.0, 1.0]], [[5.0, 6.0, 2.0], [6.0, 5.0, 8.0], [7.0, 3.0, 1.0]]], [[[3.0, 2.0, 7.0], [3.0, 4.0, 8.0], [8.0, 1.0, 1.0]], [[3.0, 
3.0, 6.0], [2.0, 4.0, 3.0], [7.0, 4.0, 2.0]], [[2.0, 4.0, 4.0], [1.0, 4.0, 6.0], [3.0, 1.0, 8.0]]], [[[8.0, 5.0, 7.0], [1.0, 
6.0, 8.0], [4.0, 7.0, 7.0]], [[7.0, 7.0, 2.0], [3.0, 2.0, 1.0], [1.0, 5.0, 6.0]], [[4.0, 8.0, 6.0], [8.0, 8.0, 6.0], [7.0, 5.0, 
5.0]]], [[[7.0, 5.0, 5.0], [7.0, 3.0, 6.0], [3.0, 1.0, 3.0]], [[3.0, 7.0, 7.0], [1.0, 3.0, 7.0], [6.0, 4.0, 6.0]], [[8.0, 2.0, 
3.0], [2.0, 3.0, 6.0], [6.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 4.0], [5.0, 4.0, 3.0], [5.0, 6.0, 4.0]], 
[[2.0, 8.0, 3.0], [6.0, 3.0, 7.0], [1.0, 6.0, 1.0]], [[5.0, 6.0, 2.0], [6.0, 5.0, 8.0], [7.0, 3.0, 1.0]]], [[[3.0, 2.0, 7.0], 
[3.0, 4.0, 8.0], [8.0, 1.0, 1.0]], [[3.0, 3.0, 6.0], [2.0, 4.0, 3.0], [7.0, 4.0, 2.0]], [[2.0, 4.0, 4.0], [1.0, 4.0, 6.0], 
[3.0, 1.0, 8.0]]], [[[8.0, 5.0, 7.0], [1.0, 6.0, 8.0], [4.0, 7.0, 7.0]], [[7.0, 7.0, 2.0], [3.0, 2.0, 1.0], [1.0, 5.0, 6.0]], 
[[4.0, 8.0, 6.0], [8.0, 7.0, 6.0], [7.0, 5.0, 5.0]]], [[[7.0, 5.0, 5.0], [7.0, 3.0, 6.0], [3.0, 1.0, 3.0]], [[3.0, 7.0, 7.0], 
[1.0, 3.0, 7.0], [6.0, 4.0, 6.0]], [[8.0, 2.0, 3.0], [2.0, 3.0, 6.0], [6.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LCPP(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 7.0], [2.0, 7.0, 2.0], [8.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], [2.0, 2.0, 2.0], [5.0, 8.0, 
3.0]], [[7.0, 1.0, 4.0], [4.0, 2.0, 4.0], [5.0, 7.0, 6.0]]], [[[3.0, 3.0, 7.0], [8.0, 1.0, 5.0], [7.0, 3.0, 4.0]], [[7.0, 2.0, 
8.0], [6.0, 1.0, 2.0], [2.0, 6.0, 4.0]], [[1.0, 1.0, 8.0], [6.0, 1.0, 2.0], [1.0, 8.0, 5.0]]], [[[3.0, 8.0, 4.0], [5.0, 8.0, 
5.0], [3.0, 5.0, 7.0]], [[6.0, 3.0, 7.0], [1.0, 4.0, 1.0], [3.0, 1.0, 2.0]], [[3.0, 7.0, 7.0], [2.0, 8.0, 8.0], [7.0, 8.0, 
8.0]]], [[[7.0, 3.0, 8.0], [5.0, 6.0, 5.0], [2.0, 6.0, 2.0]], [[2.0, 5.0, 7.0], [6.0, 5.0, 6.0], [2.0, 5.0, 5.0]], [[3.0, 7.0, 
3.0], [5.0, 6.0, 6.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 4.0, 7.0], [6.0, 6.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 2.0, 3.0], [4.0, 5.0, 7.0]]))
      arg[2:,:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 7.0], [2.0, 7.0, 2.0], [8.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], [2.0, 2.0, 2.0], [5.0, 
8.0, 3.0]], [[7.0, 1.0, 4.0], [4.0, 2.0, 4.0], [5.0, 7.0, 6.0]]], [[[3.0, 3.0, 7.0], [8.0, 1.0, 5.0], [7.0, 3.0, 4.0]], [[7.0, 
2.0, 8.0], [6.0, 1.0, 2.0], [2.0, 6.0, 4.0]], [[1.0, 1.0, 8.0], [6.0, 1.0, 2.0], [1.0, 8.0, 5.0]]], [[[3.0, 8.0, 4.0], [5.0, 
2.0, 5.0], [3.0, 5.0, 7.0]], [[6.0, 3.0, 7.0], [1.0, 4.0, 1.0], [3.0, 1.0, 2.0]], [[3.0, 7.0, 7.0], [2.0, 7.0, 8.0], [7.0, 8.0, 
8.0]]], [[[7.0, 3.0, 8.0], [5.0, 6.0, 5.0], [2.0, 6.0, 2.0]], [[2.0, 5.0, 7.0], [6.0, 6.0, 6.0], [2.0, 5.0, 5.0]], [[3.0, 7.0, 
3.0], [5.0, 1.0, 6.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 7.0], [2.0, 7.0, 2.0], [8.0, 5.0, 4.0]], 
[[1.0, 3.0, 8.0], [2.0, 2.0, 2.0], [5.0, 8.0, 3.0]], [[7.0, 1.0, 4.0], [4.0, 2.0, 4.0], [5.0, 7.0, 6.0]]], [[[3.0, 3.0, 7.0], 
[8.0, 1.0, 5.0], [7.0, 3.0, 4.0]], [[7.0, 2.0, 8.0], [6.0, 1.0, 2.0], [2.0, 6.0, 4.0]], [[1.0, 1.0, 8.0], [6.0, 1.0, 2.0], 
[1.0, 8.0, 5.0]]], [[[3.0, 8.0, 4.0], [5.0, 3.0, 5.0], [3.0, 5.0, 7.0]], [[6.0, 3.0, 7.0], [1.0, 2.0, 1.0], [3.0, 1.0, 2.0]], 
[[3.0, 7.0, 7.0], [2.0, 3.0, 8.0], [7.0, 8.0, 8.0]]], [[[7.0, 3.0, 8.0], [5.0, 4.0, 5.0], [2.0, 6.0, 2.0]], [[2.0, 5.0, 7.0], 
[6.0, 5.0, 6.0], [2.0, 5.0, 5.0]], [[3.0, 7.0, 3.0], [5.0, 7.0, 6.0], [7.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UURP(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 8.0], [6.0, 6.0, 8.0], [4.0, 7.0, 8.0]], [[8.0, 4.0, 3.0], [5.0, 3.0, 1.0], [4.0, 1.0, 
2.0]], [[3.0, 4.0, 7.0], [4.0, 5.0, 8.0], [2.0, 6.0, 2.0]]], [[[5.0, 1.0, 6.0], [6.0, 1.0, 3.0], [1.0, 3.0, 7.0]], [[2.0, 8.0, 
4.0], [5.0, 7.0, 5.0], [5.0, 6.0, 1.0]], [[7.0, 4.0, 4.0], [3.0, 2.0, 1.0], [6.0, 6.0, 1.0]]], [[[7.0, 8.0, 6.0], [1.0, 6.0, 
6.0], [4.0, 8.0, 2.0]], [[3.0, 1.0, 7.0], [2.0, 8.0, 7.0], [8.0, 5.0, 3.0]], [[6.0, 2.0, 3.0], [4.0, 6.0, 3.0], [2.0, 5.0, 
7.0]]], [[[1.0, 3.0, 8.0], [6.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[1.0, 4.0, 5.0], [3.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[6.0, 7.0, 
3.0], [8.0, 7.0, 7.0], [6.0, 1.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0], [7.0]], [[5.0], [4.0]], [[7.0], [4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0], [5.0]], [[4.0], [8.0]], [[7.0], [7.0]]]))
      arg[:3,:2,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 8.0], [6.0, 4.0, 8.0], [4.0, 7.0, 8.0]], [[8.0, 4.0, 3.0], [5.0, 7.0, 1.0], [4.0, 
1.0, 2.0]], [[3.0, 4.0, 7.0], [4.0, 5.0, 8.0], [2.0, 6.0, 2.0]]], [[[5.0, 1.0, 6.0], [6.0, 5.0, 3.0], [1.0, 3.0, 7.0]], [[2.0, 
8.0, 4.0], [5.0, 4.0, 5.0], [5.0, 6.0, 1.0]], [[7.0, 4.0, 4.0], [3.0, 2.0, 1.0], [6.0, 6.0, 1.0]]], [[[7.0, 8.0, 6.0], [1.0, 
7.0, 6.0], [4.0, 8.0, 2.0]], [[3.0, 1.0, 7.0], [2.0, 4.0, 7.0], [8.0, 5.0, 3.0]], [[6.0, 2.0, 3.0], [4.0, 6.0, 3.0], [2.0, 5.0, 
7.0]]], [[[1.0, 3.0, 8.0], [6.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[1.0, 4.0, 5.0], [3.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[6.0, 7.0, 
3.0], [8.0, 7.0, 7.0], [6.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 8.0], [6.0, 3.0, 8.0], [4.0, 7.0, 8.0]], 
[[8.0, 4.0, 3.0], [5.0, 5.0, 1.0], [4.0, 1.0, 2.0]], [[3.0, 4.0, 7.0], [4.0, 5.0, 8.0], [2.0, 6.0, 2.0]]], [[[5.0, 1.0, 6.0], 
[6.0, 4.0, 3.0], [1.0, 3.0, 7.0]], [[2.0, 8.0, 4.0], [5.0, 8.0, 5.0], [5.0, 6.0, 1.0]], [[7.0, 4.0, 4.0], [3.0, 2.0, 1.0], 
[6.0, 6.0, 1.0]]], [[[7.0, 8.0, 6.0], [1.0, 7.0, 6.0], [4.0, 8.0, 2.0]], [[3.0, 1.0, 7.0], [2.0, 7.0, 7.0], [8.0, 5.0, 3.0]], 
[[6.0, 2.0, 3.0], [4.0, 6.0, 3.0], [2.0, 5.0, 7.0]]], [[[1.0, 3.0, 8.0], [6.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[1.0, 4.0, 5.0], 
[3.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[6.0, 7.0, 3.0], [8.0, 7.0, 7.0], [6.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLRC(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 6.0], [7.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[3.0, 3.0, 3.0], [3.0, 4.0, 3.0], [6.0, 8.0, 
4.0]], [[3.0, 5.0, 2.0], [4.0, 5.0, 8.0], [8.0, 8.0, 3.0]]], [[[2.0, 8.0, 2.0], [4.0, 2.0, 3.0], [8.0, 3.0, 6.0]], [[5.0, 4.0, 
6.0], [5.0, 5.0, 3.0], [5.0, 5.0, 2.0]], [[8.0, 7.0, 7.0], [5.0, 1.0, 6.0], [1.0, 1.0, 6.0]]], [[[2.0, 3.0, 2.0], [2.0, 7.0, 
7.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 4.0], [4.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[3.0, 3.0, 3.0], [6.0, 8.0, 8.0], [4.0, 5.0, 
4.0]]], [[[2.0, 8.0, 8.0], [4.0, 1.0, 6.0], [4.0, 7.0, 5.0]], [[2.0, 1.0, 2.0], [8.0, 5.0, 6.0], [7.0, 8.0, 6.0]], [[4.0, 2.0, 
8.0], [4.0, 8.0, 1.0], [3.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 6.0, 4.0]]], [[[6.0, 2.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 7.0]]], [[[3.0, 1.0, 3.0]]]]))
      arg[1:3,2:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 6.0], [7.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[3.0, 3.0, 3.0], [3.0, 4.0, 3.0], [6.0, 
8.0, 4.0]], [[3.0, 5.0, 2.0], [4.0, 5.0, 8.0], [8.0, 8.0, 3.0]]], [[[2.0, 8.0, 2.0], [4.0, 2.0, 3.0], [8.0, 3.0, 6.0]], [[5.0, 
4.0, 6.0], [5.0, 5.0, 3.0], [5.0, 5.0, 2.0]], [[8.0, 7.0, 7.0], [7.0, 6.0, 4.0], [1.0, 1.0, 6.0]]], [[[2.0, 3.0, 2.0], [2.0, 
7.0, 7.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 4.0], [4.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[3.0, 3.0, 3.0], [6.0, 2.0, 1.0], [4.0, 5.0, 
4.0]]], [[[2.0, 8.0, 8.0], [4.0, 1.0, 6.0], [4.0, 7.0, 5.0]], [[2.0, 1.0, 2.0], [8.0, 5.0, 6.0], [7.0, 8.0, 6.0]], [[4.0, 2.0, 
8.0], [4.0, 8.0, 1.0], [3.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 6.0], [7.0, 8.0, 2.0], [2.0, 1.0, 8.0]], 
[[3.0, 3.0, 3.0], [3.0, 4.0, 3.0], [6.0, 8.0, 4.0]], [[3.0, 5.0, 2.0], [4.0, 5.0, 8.0], [8.0, 8.0, 3.0]]], [[[2.0, 8.0, 2.0], 
[4.0, 2.0, 3.0], [8.0, 3.0, 6.0]], [[5.0, 4.0, 6.0], [5.0, 5.0, 3.0], [5.0, 5.0, 2.0]], [[8.0, 7.0, 7.0], [5.0, 2.0, 7.0], 
[1.0, 1.0, 6.0]]], [[[2.0, 3.0, 2.0], [2.0, 7.0, 7.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 4.0], [4.0, 2.0, 1.0], [3.0, 7.0, 8.0]], 
[[3.0, 3.0, 3.0], [3.0, 1.0, 3.0], [4.0, 5.0, 4.0]]], [[[2.0, 8.0, 8.0], [4.0, 1.0, 6.0], [4.0, 7.0, 5.0]], [[2.0, 1.0, 2.0], 
[8.0, 5.0, 6.0], [7.0, 8.0, 6.0]], [[4.0, 2.0, 8.0], [4.0, 8.0, 1.0], [3.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CRPP(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 4.0], [2.0, 6.0, 3.0], [5.0, 1.0, 3.0]], [[1.0, 5.0, 4.0], [7.0, 6.0, 5.0], [1.0, 5.0, 
8.0]], [[3.0, 2.0, 7.0], [7.0, 7.0, 1.0], [3.0, 3.0, 4.0]]], [[[2.0, 4.0, 2.0], [1.0, 5.0, 1.0], [8.0, 1.0, 1.0]], [[4.0, 3.0, 
6.0], [6.0, 2.0, 2.0], [4.0, 5.0, 5.0]], [[1.0, 3.0, 5.0], [3.0, 8.0, 8.0], [1.0, 7.0, 5.0]]], [[[5.0, 5.0, 4.0], [1.0, 1.0, 
1.0], [5.0, 7.0, 3.0]], [[5.0, 7.0, 5.0], [3.0, 1.0, 5.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 3.0], [2.0, 3.0, 6.0], [6.0, 7.0, 
2.0]]], [[[6.0, 6.0, 2.0], [8.0, 7.0, 5.0], [5.0, 3.0, 2.0]], [[1.0, 4.0, 8.0], [8.0, 1.0, 3.0], [1.0, 4.0, 1.0]], [[3.0, 3.0, 
7.0], [5.0, 5.0, 8.0], [5.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0], [3.0], [8.0], [8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0], [2.0], [3.0], [4.0]]))
      arg[:,1:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 4.0], [2.0, 6.0, 3.0], [5.0, 1.0, 3.0]], [[1.0, 5.0, 4.0], [7.0, 2.0, 5.0], [1.0, 
5.0, 8.0]], [[3.0, 2.0, 7.0], [7.0, 7.0, 1.0], [3.0, 3.0, 4.0]]], [[[2.0, 4.0, 2.0], [1.0, 5.0, 1.0], [8.0, 1.0, 1.0]], [[4.0, 
3.0, 6.0], [6.0, 3.0, 2.0], [4.0, 5.0, 5.0]], [[1.0, 3.0, 5.0], [3.0, 8.0, 8.0], [1.0, 7.0, 5.0]]], [[[5.0, 5.0, 4.0], [1.0, 
1.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 7.0, 5.0], [3.0, 8.0, 5.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 3.0], [2.0, 3.0, 6.0], [6.0, 7.0, 
2.0]]], [[[6.0, 6.0, 2.0], [8.0, 7.0, 5.0], [5.0, 3.0, 2.0]], [[1.0, 4.0, 8.0], [8.0, 8.0, 3.0], [1.0, 4.0, 1.0]], [[3.0, 3.0, 
7.0], [5.0, 5.0, 8.0], [5.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 4.0], [2.0, 6.0, 3.0], [5.0, 1.0, 3.0]], 
[[1.0, 5.0, 4.0], [7.0, 7.0, 5.0], [1.0, 5.0, 8.0]], [[3.0, 2.0, 7.0], [7.0, 7.0, 1.0], [3.0, 3.0, 4.0]]], [[[2.0, 4.0, 2.0], 
[1.0, 5.0, 1.0], [8.0, 1.0, 1.0]], [[4.0, 3.0, 6.0], [6.0, 2.0, 2.0], [4.0, 5.0, 5.0]], [[1.0, 3.0, 5.0], [3.0, 8.0, 8.0], 
[1.0, 7.0, 5.0]]], [[[5.0, 5.0, 4.0], [1.0, 1.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 7.0, 5.0], [3.0, 3.0, 5.0], [3.0, 7.0, 7.0]], 
[[8.0, 6.0, 3.0], [2.0, 3.0, 6.0], [6.0, 7.0, 2.0]]], [[[6.0, 6.0, 2.0], [8.0, 7.0, 5.0], [5.0, 3.0, 2.0]], [[1.0, 4.0, 8.0], 
[8.0, 4.0, 3.0], [1.0, 4.0, 1.0]], [[3.0, 3.0, 7.0], [5.0, 5.0, 8.0], [5.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPUU(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 4.0], [8.0, 7.0, 5.0], [2.0, 6.0, 6.0]], [[7.0, 3.0, 7.0], [8.0, 2.0, 8.0], [6.0, 5.0, 
2.0]], [[2.0, 5.0, 5.0], [6.0, 4.0, 4.0], [1.0, 8.0, 8.0]]], [[[6.0, 4.0, 7.0], [6.0, 5.0, 3.0], [7.0, 5.0, 2.0]], [[1.0, 5.0, 
8.0], [1.0, 8.0, 3.0], [2.0, 5.0, 6.0]], [[6.0, 4.0, 5.0], [6.0, 1.0, 2.0], [4.0, 7.0, 4.0]]], [[[4.0, 2.0, 5.0], [7.0, 6.0, 
6.0], [5.0, 6.0, 5.0]], [[7.0, 3.0, 7.0], [5.0, 6.0, 3.0], [5.0, 1.0, 7.0]], [[7.0, 8.0, 3.0], [5.0, 5.0, 8.0], [7.0, 6.0, 
1.0]]], [[[2.0, 6.0, 8.0], [4.0, 4.0, 8.0], [1.0, 3.0, 4.0]], [[7.0, 4.0, 5.0], [8.0, 5.0, 7.0], [6.0, 8.0, 1.0]], [[6.0, 4.0, 
3.0], [1.0, 4.0, 8.0], [1.0, 6.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 7.0], [5.0, 3.0]], [[5.0, 6.0], [4.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 4.0], [8.0, 1.0]], [[7.0, 6.0], [5.0, 4.0]]]))
      arg[2:,1,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 4.0], [8.0, 7.0, 5.0], [2.0, 6.0, 6.0]], [[7.0, 3.0, 7.0], [8.0, 2.0, 8.0], [6.0, 
5.0, 2.0]], [[2.0, 5.0, 5.0], [6.0, 4.0, 4.0], [1.0, 8.0, 8.0]]], [[[6.0, 4.0, 7.0], [6.0, 5.0, 3.0], [7.0, 5.0, 2.0]], [[1.0, 
5.0, 8.0], [1.0, 8.0, 3.0], [2.0, 5.0, 6.0]], [[6.0, 4.0, 5.0], [6.0, 1.0, 2.0], [4.0, 7.0, 4.0]]], [[[4.0, 2.0, 5.0], [7.0, 
6.0, 6.0], [5.0, 6.0, 5.0]], [[8.0, 7.0, 7.0], [5.0, 3.0, 3.0], [5.0, 1.0, 7.0]], [[7.0, 8.0, 3.0], [5.0, 5.0, 8.0], [7.0, 6.0, 
1.0]]], [[[2.0, 6.0, 8.0], [4.0, 4.0, 8.0], [1.0, 3.0, 4.0]], [[5.0, 6.0, 5.0], [4.0, 2.0, 7.0], [6.0, 8.0, 1.0]], [[6.0, 4.0, 
3.0], [1.0, 4.0, 8.0], [1.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 4.0], [8.0, 7.0, 5.0], [2.0, 6.0, 6.0]], 
[[7.0, 3.0, 7.0], [8.0, 2.0, 8.0], [6.0, 5.0, 2.0]], [[2.0, 5.0, 5.0], [6.0, 4.0, 4.0], [1.0, 8.0, 8.0]]], [[[6.0, 4.0, 7.0], 
[6.0, 5.0, 3.0], [7.0, 5.0, 2.0]], [[1.0, 5.0, 8.0], [1.0, 8.0, 3.0], [2.0, 5.0, 6.0]], [[6.0, 4.0, 5.0], [6.0, 1.0, 2.0], 
[4.0, 7.0, 4.0]]], [[[4.0, 2.0, 5.0], [7.0, 6.0, 6.0], [5.0, 6.0, 5.0]], [[3.0, 4.0, 7.0], [8.0, 1.0, 3.0], [5.0, 1.0, 7.0]], 
[[7.0, 8.0, 3.0], [5.0, 5.0, 8.0], [7.0, 6.0, 1.0]]], [[[2.0, 6.0, 8.0], [4.0, 4.0, 8.0], [1.0, 3.0, 4.0]], [[7.0, 6.0, 5.0], 
[5.0, 4.0, 7.0], [6.0, 8.0, 1.0]], [[6.0, 4.0, 3.0], [1.0, 4.0, 8.0], [1.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPUN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 6.0], [5.0, 1.0, 2.0], [3.0, 3.0, 1.0]], [[6.0, 4.0, 1.0], [3.0, 4.0, 7.0], [8.0, 6.0, 
7.0]], [[1.0, 4.0, 2.0], [1.0, 7.0, 1.0], [3.0, 4.0, 4.0]]], [[[3.0, 6.0, 1.0], [3.0, 8.0, 2.0], [2.0, 8.0, 2.0]], [[1.0, 3.0, 
8.0], [1.0, 6.0, 3.0], [3.0, 2.0, 8.0]], [[4.0, 1.0, 5.0], [7.0, 1.0, 8.0], [8.0, 3.0, 2.0]]], [[[2.0, 1.0, 4.0], [6.0, 3.0, 
7.0], [6.0, 5.0, 5.0]], [[7.0, 7.0, 5.0], [5.0, 3.0, 5.0], [3.0, 4.0, 4.0]], [[1.0, 5.0, 2.0], [2.0, 7.0, 8.0], [8.0, 2.0, 
3.0]]], [[[1.0, 4.0, 3.0], [6.0, 4.0, 3.0], [4.0, 1.0, 2.0]], [[4.0, 6.0, 6.0], [3.0, 6.0, 1.0], [6.0, 1.0, 7.0]], [[2.0, 7.0, 
6.0], [2.0, 2.0, 4.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 3.0, 6.0], [4.0, 1.0, 8.0]], [[7.0, 3.0, 4.0], [8.0, 1.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 6.0, 5.0], [3.0, 4.0, 2.0]], [[4.0, 4.0, 6.0], [4.0, 4.0, 6.0]]]))
      arg[2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 6.0], [5.0, 1.0, 2.0], [3.0, 3.0, 1.0]], [[6.0, 4.0, 1.0], [3.0, 4.0, 7.0], [8.0, 
6.0, 7.0]], [[1.0, 4.0, 2.0], [1.0, 7.0, 1.0], [3.0, 4.0, 4.0]]], [[[3.0, 6.0, 1.0], [3.0, 8.0, 2.0], [2.0, 8.0, 2.0]], [[1.0, 
3.0, 8.0], [1.0, 6.0, 3.0], [3.0, 2.0, 8.0]], [[4.0, 1.0, 5.0], [7.0, 1.0, 8.0], [8.0, 3.0, 2.0]]], [[[2.0, 1.0, 4.0], [6.0, 
3.0, 7.0], [6.0, 5.0, 5.0]], [[6.0, 3.0, 6.0], [4.0, 1.0, 8.0], [3.0, 4.0, 4.0]], [[1.0, 5.0, 2.0], [2.0, 7.0, 8.0], [8.0, 2.0, 
3.0]]], [[[1.0, 4.0, 3.0], [6.0, 4.0, 3.0], [4.0, 1.0, 2.0]], [[7.0, 3.0, 4.0], [8.0, 1.0, 2.0], [6.0, 1.0, 7.0]], [[2.0, 7.0, 
6.0], [2.0, 2.0, 4.0], [7.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 6.0], [5.0, 1.0, 2.0], [3.0, 3.0, 1.0]], 
[[6.0, 4.0, 1.0], [3.0, 4.0, 7.0], [8.0, 6.0, 7.0]], [[1.0, 4.0, 2.0], [1.0, 7.0, 1.0], [3.0, 4.0, 4.0]]], [[[3.0, 6.0, 1.0], 
[3.0, 8.0, 2.0], [2.0, 8.0, 2.0]], [[1.0, 3.0, 8.0], [1.0, 6.0, 3.0], [3.0, 2.0, 8.0]], [[4.0, 1.0, 5.0], [7.0, 1.0, 8.0], 
[8.0, 3.0, 2.0]]], [[[2.0, 1.0, 4.0], [6.0, 3.0, 7.0], [6.0, 5.0, 5.0]], [[6.0, 6.0, 5.0], [3.0, 4.0, 2.0], [3.0, 4.0, 4.0]], 
[[1.0, 5.0, 2.0], [2.0, 7.0, 8.0], [8.0, 2.0, 3.0]]], [[[1.0, 4.0, 3.0], [6.0, 4.0, 3.0], [4.0, 1.0, 2.0]], [[4.0, 4.0, 6.0], 
[4.0, 4.0, 6.0], [6.0, 1.0, 7.0]], [[2.0, 7.0, 6.0], [2.0, 2.0, 4.0], [7.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CUPN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 6.0], [4.0, 3.0, 3.0], [1.0, 3.0, 4.0]], [[4.0, 8.0, 3.0], [6.0, 1.0, 7.0], [3.0, 4.0, 
1.0]], [[8.0, 4.0, 3.0], [4.0, 4.0, 5.0], [6.0, 7.0, 2.0]]], [[[2.0, 6.0, 8.0], [5.0, 8.0, 6.0], [8.0, 5.0, 4.0]], [[4.0, 5.0, 
2.0], [3.0, 7.0, 8.0], [2.0, 2.0, 6.0]], [[5.0, 7.0, 2.0], [5.0, 5.0, 5.0], [5.0, 7.0, 6.0]]], [[[6.0, 2.0, 5.0], [6.0, 4.0, 
8.0], [7.0, 8.0, 6.0]], [[4.0, 3.0, 5.0], [7.0, 7.0, 8.0], [6.0, 4.0, 4.0]], [[8.0, 5.0, 7.0], [5.0, 7.0, 6.0], [7.0, 6.0, 
3.0]]], [[[5.0, 5.0, 4.0], [3.0, 7.0, 6.0], [3.0, 1.0, 4.0]], [[4.0, 1.0, 7.0], [4.0, 3.0, 3.0], [4.0, 1.0, 3.0]], [[8.0, 2.0, 
8.0], [3.0, 2.0, 8.0], [7.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 8.0, 7.0], [3.0, 5.0, 3.0]], [[5.0, 2.0, 5.0], [4.0, 6.0, 3.0]], [[2.0, 5.0, 2.0], [5.0, 
1.0, 8.0]], [[7.0, 7.0, 2.0], [4.0, 8.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 4.0, 4.0], [5.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [8.0, 5.0, 7.0]], [[8.0, 5.0, 
5.0], [1.0, 1.0, 2.0]], [[1.0, 3.0, 7.0], [6.0, 7.0, 3.0]]]))
      arg[:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 6.0], [5.0, 8.0, 7.0], [1.0, 3.0, 4.0]], [[4.0, 8.0, 3.0], [3.0, 5.0, 3.0], [3.0, 
4.0, 1.0]], [[8.0, 4.0, 3.0], [4.0, 4.0, 5.0], [6.0, 7.0, 2.0]]], [[[2.0, 6.0, 8.0], [5.0, 2.0, 5.0], [8.0, 5.0, 4.0]], [[4.0, 
5.0, 2.0], [4.0, 6.0, 3.0], [2.0, 2.0, 6.0]], [[5.0, 7.0, 2.0], [5.0, 5.0, 5.0], [5.0, 7.0, 6.0]]], [[[6.0, 2.0, 5.0], [2.0, 
5.0, 2.0], [7.0, 8.0, 6.0]], [[4.0, 3.0, 5.0], [5.0, 1.0, 8.0], [6.0, 4.0, 4.0]], [[8.0, 5.0, 7.0], [5.0, 7.0, 6.0], [7.0, 6.0, 
3.0]]], [[[5.0, 5.0, 4.0], [7.0, 7.0, 2.0], [3.0, 1.0, 4.0]], [[4.0, 1.0, 7.0], [4.0, 8.0, 8.0], [4.0, 1.0, 3.0]], [[8.0, 2.0, 
8.0], [3.0, 2.0, 8.0], [7.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 6.0], [8.0, 4.0, 4.0], [1.0, 3.0, 4.0]], 
[[4.0, 8.0, 3.0], [5.0, 2.0, 3.0], [3.0, 4.0, 1.0]], [[8.0, 4.0, 3.0], [4.0, 4.0, 5.0], [6.0, 7.0, 2.0]]], [[[2.0, 6.0, 8.0], 
[2.0, 1.0, 4.0], [8.0, 5.0, 4.0]], [[4.0, 5.0, 2.0], [8.0, 5.0, 7.0], [2.0, 2.0, 6.0]], [[5.0, 7.0, 2.0], [5.0, 5.0, 5.0], 
[5.0, 7.0, 6.0]]], [[[6.0, 2.0, 5.0], [8.0, 5.0, 5.0], [7.0, 8.0, 6.0]], [[4.0, 3.0, 5.0], [1.0, 1.0, 2.0], [6.0, 4.0, 4.0]], 
[[8.0, 5.0, 7.0], [5.0, 7.0, 6.0], [7.0, 6.0, 3.0]]], [[[5.0, 5.0, 4.0], [1.0, 3.0, 7.0], [3.0, 1.0, 4.0]], [[4.0, 1.0, 7.0], 
[6.0, 7.0, 3.0], [4.0, 1.0, 3.0]], [[8.0, 2.0, 8.0], [3.0, 2.0, 8.0], [7.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPLP(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 5.0], [1.0, 4.0, 4.0], [3.0, 7.0, 8.0]], [[5.0, 1.0, 2.0], [7.0, 1.0, 2.0], [4.0, 4.0, 
3.0]], [[1.0, 5.0, 4.0], [8.0, 6.0, 3.0], [4.0, 1.0, 4.0]]], [[[7.0, 2.0, 7.0], [6.0, 3.0, 3.0], [5.0, 8.0, 1.0]], [[7.0, 1.0, 
7.0], [1.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[1.0, 6.0, 4.0], [8.0, 1.0, 7.0], [6.0, 3.0, 8.0]]], [[[2.0, 2.0, 5.0], [5.0, 8.0, 
5.0], [5.0, 8.0, 2.0]], [[1.0, 4.0, 7.0], [8.0, 4.0, 2.0], [2.0, 4.0, 8.0]], [[5.0, 5.0, 6.0], [8.0, 5.0, 6.0], [1.0, 6.0, 
6.0]]], [[[8.0, 5.0, 1.0], [7.0, 3.0, 5.0], [3.0, 1.0, 3.0]], [[4.0, 1.0, 3.0], [5.0, 1.0, 6.0], [1.0, 2.0, 3.0]], [[4.0, 2.0, 
7.0], [4.0, 1.0, 3.0], [7.0, 7.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[5.0], [1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0], [1.0]]))
      arg[2:,1,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 5.0], [1.0, 4.0, 4.0], [3.0, 7.0, 8.0]], [[5.0, 1.0, 2.0], [7.0, 1.0, 2.0], [4.0, 
4.0, 3.0]], [[1.0, 5.0, 4.0], [8.0, 6.0, 3.0], [4.0, 1.0, 4.0]]], [[[7.0, 2.0, 7.0], [6.0, 3.0, 3.0], [5.0, 8.0, 1.0]], [[7.0, 
1.0, 7.0], [1.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[1.0, 6.0, 4.0], [8.0, 1.0, 7.0], [6.0, 3.0, 8.0]]], [[[2.0, 2.0, 5.0], [5.0, 
8.0, 5.0], [5.0, 8.0, 2.0]], [[1.0, 4.0, 7.0], [8.0, 4.0, 2.0], [2.0, 5.0, 8.0]], [[5.0, 5.0, 6.0], [8.0, 5.0, 6.0], [1.0, 6.0, 
6.0]]], [[[8.0, 5.0, 1.0], [7.0, 3.0, 5.0], [3.0, 1.0, 3.0]], [[4.0, 1.0, 3.0], [5.0, 1.0, 6.0], [1.0, 1.0, 3.0]], [[4.0, 2.0, 
7.0], [4.0, 1.0, 3.0], [7.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 5.0], [1.0, 4.0, 4.0], [3.0, 7.0, 8.0]], 
[[5.0, 1.0, 2.0], [7.0, 1.0, 2.0], [4.0, 4.0, 3.0]], [[1.0, 5.0, 4.0], [8.0, 6.0, 3.0], [4.0, 1.0, 4.0]]], [[[7.0, 2.0, 7.0], 
[6.0, 3.0, 3.0], [5.0, 8.0, 1.0]], [[7.0, 1.0, 7.0], [1.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[1.0, 6.0, 4.0], [8.0, 1.0, 7.0], 
[6.0, 3.0, 8.0]]], [[[2.0, 2.0, 5.0], [5.0, 8.0, 5.0], [5.0, 8.0, 2.0]], [[1.0, 4.0, 7.0], [8.0, 4.0, 2.0], [2.0, 7.0, 8.0]], 
[[5.0, 5.0, 6.0], [8.0, 5.0, 6.0], [1.0, 6.0, 6.0]]], [[[8.0, 5.0, 1.0], [7.0, 3.0, 5.0], [3.0, 1.0, 3.0]], [[4.0, 1.0, 3.0], 
[5.0, 1.0, 6.0], [1.0, 1.0, 3.0]], [[4.0, 2.0, 7.0], [4.0, 1.0, 3.0], [7.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PRUP(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 6.0], [7.0, 5.0, 6.0], [4.0, 4.0, 6.0]], [[8.0, 2.0, 4.0], [4.0, 3.0, 8.0], [2.0, 4.0, 
6.0]], [[6.0, 6.0, 1.0], [5.0, 3.0, 6.0], [4.0, 8.0, 4.0]]], [[[8.0, 8.0, 8.0], [3.0, 5.0, 3.0], [8.0, 7.0, 5.0]], [[5.0, 8.0, 
7.0], [5.0, 8.0, 2.0], [6.0, 5.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 8.0, 8.0], [7.0, 4.0, 8.0]]], [[[6.0, 5.0, 4.0], [6.0, 1.0, 
8.0], [1.0, 8.0, 3.0]], [[6.0, 2.0, 4.0], [2.0, 7.0, 6.0], [4.0, 2.0, 1.0]], [[1.0, 6.0, 1.0], [5.0, 2.0, 4.0], [8.0, 4.0, 
2.0]]], [[[2.0, 1.0, 1.0], [1.0, 4.0, 3.0], [4.0, 3.0, 4.0]], [[6.0, 4.0, 2.0], [3.0, 6.0, 8.0], [3.0, 1.0, 2.0]], [[1.0, 4.0, 
5.0], [1.0, 2.0, 2.0], [6.0, 3.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[4.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 8.0]]))
      arg[2,1:2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 6.0], [7.0, 5.0, 6.0], [4.0, 4.0, 6.0]], [[8.0, 2.0, 4.0], [4.0, 3.0, 8.0], [2.0, 
4.0, 6.0]], [[6.0, 6.0, 1.0], [5.0, 3.0, 6.0], [4.0, 8.0, 4.0]]], [[[8.0, 8.0, 8.0], [3.0, 5.0, 3.0], [8.0, 7.0, 5.0]], [[5.0, 
8.0, 7.0], [5.0, 8.0, 2.0], [6.0, 5.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 8.0, 8.0], [7.0, 4.0, 8.0]]], [[[6.0, 5.0, 4.0], [6.0, 
1.0, 8.0], [1.0, 8.0, 3.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 6.0], [4.0, 2.0, 1.0]], [[1.0, 6.0, 1.0], [5.0, 2.0, 4.0], [8.0, 4.0, 
2.0]]], [[[2.0, 1.0, 1.0], [1.0, 4.0, 3.0], [4.0, 3.0, 4.0]], [[6.0, 4.0, 2.0], [3.0, 6.0, 8.0], [3.0, 1.0, 2.0]], [[1.0, 4.0, 
5.0], [1.0, 2.0, 2.0], [6.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 6.0], [7.0, 5.0, 6.0], [4.0, 4.0, 6.0]], 
[[8.0, 2.0, 4.0], [4.0, 3.0, 8.0], [2.0, 4.0, 6.0]], [[6.0, 6.0, 1.0], [5.0, 3.0, 6.0], [4.0, 8.0, 4.0]]], [[[8.0, 8.0, 8.0], 
[3.0, 5.0, 3.0], [8.0, 7.0, 5.0]], [[5.0, 8.0, 7.0], [5.0, 8.0, 2.0], [6.0, 5.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 8.0, 8.0], 
[7.0, 4.0, 8.0]]], [[[6.0, 5.0, 4.0], [6.0, 1.0, 8.0], [1.0, 8.0, 3.0]], [[6.0, 1.0, 4.0], [2.0, 8.0, 6.0], [4.0, 2.0, 1.0]], 
[[1.0, 6.0, 1.0], [5.0, 2.0, 4.0], [8.0, 4.0, 2.0]]], [[[2.0, 1.0, 1.0], [1.0, 4.0, 3.0], [4.0, 3.0, 4.0]], [[6.0, 4.0, 2.0], 
[3.0, 6.0, 8.0], [3.0, 1.0, 2.0]], [[1.0, 4.0, 5.0], [1.0, 2.0, 2.0], [6.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLUC(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 6.0], [6.0, 6.0, 7.0], [3.0, 1.0, 4.0]], [[8.0, 5.0, 7.0], [4.0, 6.0, 6.0], [8.0, 2.0, 
1.0]], [[3.0, 1.0, 3.0], [3.0, 7.0, 8.0], [1.0, 1.0, 2.0]]], [[[2.0, 1.0, 1.0], [2.0, 7.0, 8.0], [5.0, 1.0, 5.0]], [[8.0, 5.0, 
3.0], [5.0, 3.0, 6.0], [5.0, 8.0, 6.0]], [[3.0, 5.0, 8.0], [1.0, 8.0, 6.0], [4.0, 2.0, 5.0]]], [[[1.0, 6.0, 4.0], [7.0, 6.0, 
1.0], [2.0, 7.0, 6.0]], [[4.0, 7.0, 5.0], [2.0, 6.0, 3.0], [5.0, 7.0, 4.0]], [[5.0, 6.0, 8.0], [3.0, 1.0, 1.0], [1.0, 5.0, 
8.0]]], [[[7.0, 2.0, 6.0], [4.0, 6.0, 4.0], [3.0, 7.0, 2.0]], [[7.0, 2.0, 8.0], [1.0, 2.0, 4.0], [5.0, 4.0, 1.0]], [[5.0, 2.0, 
2.0], [1.0, 7.0, 5.0], [5.0, 4.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 5.0, 7.0], [3.0, 3.0, 2.0]]], [[[8.0, 5.0, 5.0], [6.0, 5.0, 1.0]]], [[[1.0, 8.0, 4.0], 
[7.0, 3.0, 6.0]]], [[[5.0, 8.0, 1.0], [3.0, 4.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 7.0], [8.0, 5.0, 7.0]]], [[[4.0, 1.0, 1.0], [6.0, 2.0, 8.0]]], [[[4.0, 
3.0, 2.0], [1.0, 5.0, 7.0]]], [[[7.0, 5.0, 2.0], [4.0, 2.0, 6.0]]]]))
      arg[:,2:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 6.0], [6.0, 6.0, 7.0], [3.0, 1.0, 4.0]], [[8.0, 5.0, 7.0], [4.0, 6.0, 6.0], [8.0, 
2.0, 1.0]], [[5.0, 5.0, 7.0], [3.0, 3.0, 2.0], [1.0, 1.0, 2.0]]], [[[2.0, 1.0, 1.0], [2.0, 7.0, 8.0], [5.0, 1.0, 5.0]], [[8.0, 
5.0, 3.0], [5.0, 3.0, 6.0], [5.0, 8.0, 6.0]], [[8.0, 5.0, 5.0], [6.0, 5.0, 1.0], [4.0, 2.0, 5.0]]], [[[1.0, 6.0, 4.0], [7.0, 
6.0, 1.0], [2.0, 7.0, 6.0]], [[4.0, 7.0, 5.0], [2.0, 6.0, 3.0], [5.0, 7.0, 4.0]], [[1.0, 8.0, 4.0], [7.0, 3.0, 6.0], [1.0, 5.0, 
8.0]]], [[[7.0, 2.0, 6.0], [4.0, 6.0, 4.0], [3.0, 7.0, 2.0]], [[7.0, 2.0, 8.0], [1.0, 2.0, 4.0], [5.0, 4.0, 1.0]], [[5.0, 8.0, 
1.0], [3.0, 4.0, 1.0], [5.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 6.0], [6.0, 6.0, 7.0], [3.0, 1.0, 4.0]], 
[[8.0, 5.0, 7.0], [4.0, 6.0, 6.0], [8.0, 2.0, 1.0]], [[8.0, 1.0, 7.0], [8.0, 5.0, 7.0], [1.0, 1.0, 2.0]]], [[[2.0, 1.0, 1.0], 
[2.0, 7.0, 8.0], [5.0, 1.0, 5.0]], [[8.0, 5.0, 3.0], [5.0, 3.0, 6.0], [5.0, 8.0, 6.0]], [[4.0, 1.0, 1.0], [6.0, 2.0, 8.0], 
[4.0, 2.0, 5.0]]], [[[1.0, 6.0, 4.0], [7.0, 6.0, 1.0], [2.0, 7.0, 6.0]], [[4.0, 7.0, 5.0], [2.0, 6.0, 3.0], [5.0, 7.0, 4.0]], 
[[4.0, 3.0, 2.0], [1.0, 5.0, 7.0], [1.0, 5.0, 8.0]]], [[[7.0, 2.0, 6.0], [4.0, 6.0, 4.0], [3.0, 7.0, 2.0]], [[7.0, 2.0, 8.0], 
[1.0, 2.0, 4.0], [5.0, 4.0, 1.0]], [[7.0, 5.0, 2.0], [4.0, 2.0, 6.0], [5.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLUP(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 8.0], [1.0, 3.0, 2.0], [5.0, 1.0, 3.0]], [[6.0, 4.0, 8.0], [4.0, 5.0, 1.0], [4.0, 2.0, 
1.0]], [[7.0, 3.0, 6.0], [2.0, 7.0, 7.0], [1.0, 5.0, 3.0]]], [[[6.0, 5.0, 7.0], [3.0, 2.0, 1.0], [8.0, 8.0, 6.0]], [[4.0, 5.0, 
2.0], [8.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[4.0, 1.0, 2.0], [3.0, 8.0, 7.0], [1.0, 6.0, 7.0]]], [[[6.0, 7.0, 1.0], [5.0, 2.0, 
8.0], [5.0, 8.0, 3.0]], [[1.0, 2.0, 8.0], [1.0, 8.0, 1.0], [1.0, 4.0, 2.0]], [[1.0, 7.0, 6.0], [3.0, 5.0, 6.0], [4.0, 2.0, 
8.0]]], [[[7.0, 3.0, 2.0], [2.0, 8.0, 7.0], [4.0, 3.0, 7.0]], [[4.0, 5.0, 4.0], [6.0, 7.0, 1.0], [3.0, 2.0, 3.0]], [[5.0, 7.0, 
4.0], [3.0, 6.0, 6.0], [6.0, 5.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 4.0]]))
      arg[2,2:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 8.0], [1.0, 3.0, 2.0], [5.0, 1.0, 3.0]], [[6.0, 4.0, 8.0], [4.0, 5.0, 1.0], [4.0, 
2.0, 1.0]], [[7.0, 3.0, 6.0], [2.0, 7.0, 7.0], [1.0, 5.0, 3.0]]], [[[6.0, 5.0, 7.0], [3.0, 2.0, 1.0], [8.0, 8.0, 6.0]], [[4.0, 
5.0, 2.0], [8.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[4.0, 1.0, 2.0], [3.0, 8.0, 7.0], [1.0, 6.0, 7.0]]], [[[6.0, 7.0, 1.0], [5.0, 
2.0, 8.0], [5.0, 8.0, 3.0]], [[1.0, 2.0, 8.0], [1.0, 8.0, 1.0], [1.0, 4.0, 2.0]], [[1.0, 1.0, 6.0], [3.0, 6.0, 6.0], [4.0, 2.0, 
8.0]]], [[[7.0, 3.0, 2.0], [2.0, 8.0, 7.0], [4.0, 3.0, 7.0]], [[4.0, 5.0, 4.0], [6.0, 7.0, 1.0], [3.0, 2.0, 3.0]], [[5.0, 7.0, 
4.0], [3.0, 6.0, 6.0], [6.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 8.0], [1.0, 3.0, 2.0], [5.0, 1.0, 3.0]], 
[[6.0, 4.0, 8.0], [4.0, 5.0, 1.0], [4.0, 2.0, 1.0]], [[7.0, 3.0, 6.0], [2.0, 7.0, 7.0], [1.0, 5.0, 3.0]]], [[[6.0, 5.0, 7.0], 
[3.0, 2.0, 1.0], [8.0, 8.0, 6.0]], [[4.0, 5.0, 2.0], [8.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[4.0, 1.0, 2.0], [3.0, 8.0, 7.0], 
[1.0, 6.0, 7.0]]], [[[6.0, 7.0, 1.0], [5.0, 2.0, 8.0], [5.0, 8.0, 3.0]], [[1.0, 2.0, 8.0], [1.0, 8.0, 1.0], [1.0, 4.0, 2.0]], 
[[1.0, 4.0, 6.0], [3.0, 4.0, 6.0], [4.0, 2.0, 8.0]]], [[[7.0, 3.0, 2.0], [2.0, 8.0, 7.0], [4.0, 3.0, 7.0]], [[4.0, 5.0, 4.0], 
[6.0, 7.0, 1.0], [3.0, 2.0, 3.0]], [[5.0, 7.0, 4.0], [3.0, 6.0, 6.0], [6.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UUUU(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 7.0], [2.0, 3.0, 3.0], [3.0, 5.0, 6.0]], [[2.0, 5.0, 6.0], [7.0, 2.0, 6.0], [6.0, 4.0, 
2.0]], [[4.0, 6.0, 3.0], [3.0, 5.0, 3.0], [4.0, 4.0, 1.0]]], [[[7.0, 4.0, 7.0], [7.0, 8.0, 2.0], [8.0, 6.0, 7.0]], [[4.0, 1.0, 
4.0], [7.0, 2.0, 7.0], [6.0, 8.0, 8.0]], [[7.0, 3.0, 5.0], [4.0, 1.0, 7.0], [5.0, 2.0, 5.0]]], [[[7.0, 3.0, 1.0], [8.0, 2.0, 
1.0], [7.0, 7.0, 4.0]], [[2.0, 7.0, 6.0], [8.0, 2.0, 5.0], [2.0, 5.0, 6.0]], [[7.0, 4.0, 3.0], [2.0, 3.0, 2.0], [1.0, 7.0, 
4.0]]], [[[2.0, 5.0, 3.0], [7.0, 7.0, 7.0], [6.0, 5.0, 6.0]], [[3.0, 4.0, 5.0], [1.0, 3.0, 7.0], [2.0, 8.0, 4.0]], [[4.0, 7.0, 
2.0], [3.0, 3.0, 3.0], [3.0, 7.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 7.0], [5.0, 5.0]], [[6.0, 2.0], [4.0, 2.0]]], [[[1.0, 6.0], [7.0, 6.0]], [[6.0, 4.0], [3.0, 
6.0]]], [[[1.0, 1.0], [2.0, 5.0]], [[8.0, 4.0], [7.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 2.0], [5.0, 5.0]], [[6.0, 3.0], [8.0, 4.0]]], [[[2.0, 5.0], [8.0, 5.0]], [[6.0, 
7.0], [2.0, 3.0]]], [[[8.0, 4.0], [5.0, 1.0]], [[2.0, 4.0], [8.0, 1.0]]]]))
      arg[:3,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 7.0], [5.0, 5.0, 3.0], [3.0, 5.0, 6.0]], [[6.0, 2.0, 6.0], [4.0, 2.0, 6.0], [6.0, 
4.0, 2.0]], [[4.0, 6.0, 3.0], [3.0, 5.0, 3.0], [4.0, 4.0, 1.0]]], [[[1.0, 6.0, 7.0], [7.0, 6.0, 2.0], [8.0, 6.0, 7.0]], [[6.0, 
4.0, 4.0], [3.0, 6.0, 7.0], [6.0, 8.0, 8.0]], [[7.0, 3.0, 5.0], [4.0, 1.0, 7.0], [5.0, 2.0, 5.0]]], [[[1.0, 1.0, 1.0], [2.0, 
5.0, 1.0], [7.0, 7.0, 4.0]], [[8.0, 4.0, 6.0], [7.0, 8.0, 5.0], [2.0, 5.0, 6.0]], [[7.0, 4.0, 3.0], [2.0, 3.0, 2.0], [1.0, 7.0, 
4.0]]], [[[2.0, 5.0, 3.0], [7.0, 7.0, 7.0], [6.0, 5.0, 6.0]], [[3.0, 4.0, 5.0], [1.0, 3.0, 7.0], [2.0, 8.0, 4.0]], [[4.0, 7.0, 
2.0], [3.0, 3.0, 3.0], [3.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 7.0], [5.0, 5.0, 3.0], [3.0, 5.0, 6.0]], 
[[6.0, 3.0, 6.0], [8.0, 4.0, 6.0], [6.0, 4.0, 2.0]], [[4.0, 6.0, 3.0], [3.0, 5.0, 3.0], [4.0, 4.0, 1.0]]], [[[2.0, 5.0, 7.0], 
[8.0, 5.0, 2.0], [8.0, 6.0, 7.0]], [[6.0, 7.0, 4.0], [2.0, 3.0, 7.0], [6.0, 8.0, 8.0]], [[7.0, 3.0, 5.0], [4.0, 1.0, 7.0], 
[5.0, 2.0, 5.0]]], [[[8.0, 4.0, 1.0], [5.0, 1.0, 1.0], [7.0, 7.0, 4.0]], [[2.0, 4.0, 6.0], [8.0, 1.0, 5.0], [2.0, 5.0, 6.0]], 
[[7.0, 4.0, 3.0], [2.0, 3.0, 2.0], [1.0, 7.0, 4.0]]], [[[2.0, 5.0, 3.0], [7.0, 7.0, 7.0], [6.0, 5.0, 6.0]], [[3.0, 4.0, 5.0], 
[1.0, 3.0, 7.0], [2.0, 8.0, 4.0]], [[4.0, 7.0, 2.0], [3.0, 3.0, 3.0], [3.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCRC(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 1.0], [4.0, 1.0, 6.0], [3.0, 7.0, 7.0]], [[3.0, 8.0, 2.0], [3.0, 5.0, 8.0], [3.0, 5.0, 
1.0]], [[5.0, 4.0, 7.0], [3.0, 5.0, 6.0], [8.0, 4.0, 3.0]]], [[[2.0, 8.0, 2.0], [2.0, 5.0, 2.0], [5.0, 3.0, 4.0]], [[2.0, 6.0, 
1.0], [5.0, 4.0, 2.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 6.0], [3.0, 4.0, 7.0], [5.0, 3.0, 6.0]]], [[[3.0, 4.0, 1.0], [3.0, 8.0, 
3.0], [8.0, 8.0, 7.0]], [[6.0, 4.0, 7.0], [8.0, 4.0, 8.0], [1.0, 1.0, 5.0]], [[5.0, 6.0, 1.0], [4.0, 2.0, 7.0], [3.0, 4.0, 
8.0]]], [[[8.0, 6.0, 7.0], [5.0, 6.0, 6.0], [7.0, 5.0, 3.0]], [[3.0, 4.0, 6.0], [6.0, 6.0, 4.0], [8.0, 1.0, 3.0]], [[3.0, 6.0, 
7.0], [4.0, 1.0, 3.0], [5.0, 4.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 3.0, 4.0]], [[1.0, 3.0, 5.0]], [[2.0, 8.0, 3.0]]], [[[3.0, 5.0, 8.0]], [[7.0, 3.0, 4.0]], 
[[1.0, 1.0, 2.0]]], [[[1.0, 7.0, 1.0]], [[7.0, 2.0, 6.0]], [[5.0, 1.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 3.0]], [[4.0, 1.0, 7.0]], [[8.0, 5.0, 5.0]]], [[[7.0, 1.0, 8.0]], [[6.0, 
2.0, 8.0]], [[4.0, 8.0, 7.0]]], [[[5.0, 2.0, 6.0]], [[7.0, 6.0, 2.0]], [[7.0, 3.0, 6.0]]]]))
      arg[:3,:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 1.0], [7.0, 3.0, 4.0], [3.0, 7.0, 7.0]], [[3.0, 8.0, 2.0], [1.0, 3.0, 5.0], [3.0, 
5.0, 1.0]], [[5.0, 4.0, 7.0], [2.0, 8.0, 3.0], [8.0, 4.0, 3.0]]], [[[2.0, 8.0, 2.0], [3.0, 5.0, 8.0], [5.0, 3.0, 4.0]], [[2.0, 
6.0, 1.0], [7.0, 3.0, 4.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 6.0], [1.0, 1.0, 2.0], [5.0, 3.0, 6.0]]], [[[3.0, 4.0, 1.0], [1.0, 
7.0, 1.0], [8.0, 8.0, 7.0]], [[6.0, 4.0, 7.0], [7.0, 2.0, 6.0], [1.0, 1.0, 5.0]], [[5.0, 6.0, 1.0], [5.0, 1.0, 2.0], [3.0, 4.0, 
8.0]]], [[[8.0, 6.0, 7.0], [5.0, 6.0, 6.0], [7.0, 5.0, 3.0]], [[3.0, 4.0, 6.0], [6.0, 6.0, 4.0], [8.0, 1.0, 3.0]], [[3.0, 6.0, 
7.0], [4.0, 1.0, 3.0], [5.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 1.0], [7.0, 8.0, 3.0], [3.0, 7.0, 7.0]], 
[[3.0, 8.0, 2.0], [4.0, 1.0, 7.0], [3.0, 5.0, 1.0]], [[5.0, 4.0, 7.0], [8.0, 5.0, 5.0], [8.0, 4.0, 3.0]]], [[[2.0, 8.0, 2.0], 
[7.0, 1.0, 8.0], [5.0, 3.0, 4.0]], [[2.0, 6.0, 1.0], [6.0, 2.0, 8.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 6.0], [4.0, 8.0, 7.0], 
[5.0, 3.0, 6.0]]], [[[3.0, 4.0, 1.0], [5.0, 2.0, 6.0], [8.0, 8.0, 7.0]], [[6.0, 4.0, 7.0], [7.0, 6.0, 2.0], [1.0, 1.0, 5.0]], 
[[5.0, 6.0, 1.0], [7.0, 3.0, 6.0], [3.0, 4.0, 8.0]]], [[[8.0, 6.0, 7.0], [5.0, 6.0, 6.0], [7.0, 5.0, 3.0]], [[3.0, 4.0, 6.0], 
[6.0, 6.0, 4.0], [8.0, 1.0, 3.0]], [[3.0, 6.0, 7.0], [4.0, 1.0, 3.0], [5.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LCLN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 1.0], [4.0, 7.0, 1.0], [4.0, 1.0, 8.0]], [[2.0, 2.0, 6.0], [7.0, 4.0, 6.0], [2.0, 4.0, 
2.0]], [[1.0, 6.0, 6.0], [6.0, 7.0, 6.0], [5.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], [6.0, 4.0, 4.0], [3.0, 7.0, 8.0]], [[1.0, 2.0, 
1.0], [5.0, 4.0, 7.0], [4.0, 5.0, 3.0]], [[7.0, 8.0, 8.0], [7.0, 6.0, 5.0], [5.0, 2.0, 3.0]]], [[[6.0, 5.0, 4.0], [7.0, 6.0, 
6.0], [5.0, 2.0, 2.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 8.0], [4.0, 7.0, 8.0]], [[8.0, 6.0, 4.0], [1.0, 1.0, 3.0], [6.0, 3.0, 
2.0]]], [[[8.0, 3.0, 2.0], [4.0, 6.0, 3.0], [8.0, 4.0, 3.0]], [[1.0, 4.0, 5.0], [8.0, 8.0, 6.0], [6.0, 1.0, 2.0]], [[1.0, 7.0, 
2.0], [8.0, 3.0, 6.0], [5.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 7.0, 4.0]], [[7.0, 3.0, 3.0]], [[8.0, 4.0, 4.0]]], [[[2.0, 6.0, 7.0]], [[7.0, 4.0, 3.0]], 
[[3.0, 8.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 2.0]], [[1.0, 8.0, 5.0]], [[3.0, 8.0, 8.0]]], [[[4.0, 2.0, 8.0]], [[3.0, 
8.0, 7.0]], [[6.0, 6.0, 2.0]]]]))
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 1.0], [4.0, 7.0, 1.0], [4.0, 1.0, 8.0]], [[2.0, 2.0, 6.0], [7.0, 4.0, 6.0], [2.0, 
4.0, 2.0]], [[1.0, 6.0, 6.0], [6.0, 7.0, 6.0], [5.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], [6.0, 4.0, 4.0], [3.0, 7.0, 8.0]], [[1.0, 
2.0, 1.0], [5.0, 4.0, 7.0], [4.0, 5.0, 3.0]], [[7.0, 8.0, 8.0], [7.0, 6.0, 5.0], [5.0, 2.0, 3.0]]], [[[6.0, 5.0, 4.0], [7.0, 
6.0, 6.0], [5.0, 7.0, 4.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 8.0], [7.0, 3.0, 3.0]], [[8.0, 6.0, 4.0], [1.0, 1.0, 3.0], [8.0, 4.0, 
4.0]]], [[[8.0, 3.0, 2.0], [4.0, 6.0, 3.0], [2.0, 6.0, 7.0]], [[1.0, 4.0, 5.0], [8.0, 8.0, 6.0], [7.0, 4.0, 3.0]], [[1.0, 7.0, 
2.0], [8.0, 3.0, 6.0], [3.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 1.0], [4.0, 7.0, 1.0], [4.0, 1.0, 8.0]], 
[[2.0, 2.0, 6.0], [7.0, 4.0, 6.0], [2.0, 4.0, 2.0]], [[1.0, 6.0, 6.0], [6.0, 7.0, 6.0], [5.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], 
[6.0, 4.0, 4.0], [3.0, 7.0, 8.0]], [[1.0, 2.0, 1.0], [5.0, 4.0, 7.0], [4.0, 5.0, 3.0]], [[7.0, 8.0, 8.0], [7.0, 6.0, 5.0], 
[5.0, 2.0, 3.0]]], [[[6.0, 5.0, 4.0], [7.0, 6.0, 6.0], [2.0, 4.0, 2.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 8.0], [1.0, 8.0, 5.0]], 
[[8.0, 6.0, 4.0], [1.0, 1.0, 3.0], [3.0, 8.0, 8.0]]], [[[8.0, 3.0, 2.0], [4.0, 6.0, 3.0], [4.0, 2.0, 8.0]], [[1.0, 4.0, 5.0], 
[8.0, 8.0, 6.0], [3.0, 8.0, 7.0]], [[1.0, 7.0, 2.0], [8.0, 3.0, 6.0], [6.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULLL(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 7.0], [4.0, 4.0, 2.0], [3.0, 4.0, 1.0]], [[5.0, 4.0, 5.0], [3.0, 2.0, 3.0], [7.0, 2.0, 
4.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 3.0], [7.0, 7.0, 6.0]]], [[[5.0, 1.0, 7.0], [1.0, 6.0, 4.0], [4.0, 8.0, 8.0]], [[7.0, 7.0, 
6.0], [7.0, 3.0, 4.0], [6.0, 3.0, 1.0]], [[6.0, 7.0, 4.0], [1.0, 8.0, 4.0], [3.0, 8.0, 4.0]]], [[[5.0, 8.0, 6.0], [2.0, 5.0, 
1.0], [5.0, 1.0, 7.0]], [[5.0, 2.0, 3.0], [2.0, 5.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 1.0, 4.0], [7.0, 4.0, 8.0], [5.0, 8.0, 
8.0]]], [[[5.0, 7.0, 5.0], [4.0, 5.0, 8.0], [1.0, 1.0, 2.0]], [[2.0, 8.0, 2.0], [6.0, 3.0, 1.0], [6.0, 7.0, 2.0]], [[1.0, 2.0, 
5.0], [4.0, 4.0, 5.0], [7.0, 7.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0]]], [[[7.0]]], [[[4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0]]], [[[7.0]]], [[[1.0]]]]))
      arg[:3,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 7.0], [4.0, 4.0, 2.0], [3.0, 4.0, 1.0]], [[5.0, 4.0, 5.0], [3.0, 2.0, 3.0], [7.0, 
2.0, 4.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 3.0], [7.0, 7.0, 1.0]]], [[[5.0, 1.0, 7.0], [1.0, 6.0, 4.0], [4.0, 8.0, 8.0]], [[7.0, 
7.0, 6.0], [7.0, 3.0, 4.0], [6.0, 3.0, 1.0]], [[6.0, 7.0, 4.0], [1.0, 8.0, 4.0], [3.0, 8.0, 7.0]]], [[[5.0, 8.0, 6.0], [2.0, 
5.0, 1.0], [5.0, 1.0, 7.0]], [[5.0, 2.0, 3.0], [2.0, 5.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 1.0, 4.0], [7.0, 4.0, 8.0], [5.0, 8.0, 
4.0]]], [[[5.0, 7.0, 5.0], [4.0, 5.0, 8.0], [1.0, 1.0, 2.0]], [[2.0, 8.0, 2.0], [6.0, 3.0, 1.0], [6.0, 7.0, 2.0]], [[1.0, 2.0, 
5.0], [4.0, 4.0, 5.0], [7.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 7.0], [4.0, 4.0, 2.0], [3.0, 4.0, 1.0]], 
[[5.0, 4.0, 5.0], [3.0, 2.0, 3.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 3.0], [7.0, 7.0, 6.0]]], [[[5.0, 1.0, 7.0], 
[1.0, 6.0, 4.0], [4.0, 8.0, 8.0]], [[7.0, 7.0, 6.0], [7.0, 3.0, 4.0], [6.0, 3.0, 1.0]], [[6.0, 7.0, 4.0], [1.0, 8.0, 4.0], 
[3.0, 8.0, 7.0]]], [[[5.0, 8.0, 6.0], [2.0, 5.0, 1.0], [5.0, 1.0, 7.0]], [[5.0, 2.0, 3.0], [2.0, 5.0, 4.0], [3.0, 8.0, 6.0]], 
[[6.0, 1.0, 4.0], [7.0, 4.0, 8.0], [5.0, 8.0, 1.0]]], [[[5.0, 7.0, 5.0], [4.0, 5.0, 8.0], [1.0, 1.0, 2.0]], [[2.0, 8.0, 2.0], 
[6.0, 3.0, 1.0], [6.0, 7.0, 2.0]], [[1.0, 2.0, 5.0], [4.0, 4.0, 5.0], [7.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLRU(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 3.0], [8.0, 8.0, 5.0], [3.0, 2.0, 6.0]], [[4.0, 2.0, 4.0], [8.0, 2.0, 4.0], [8.0, 6.0, 
2.0]], [[3.0, 5.0, 8.0], [6.0, 4.0, 8.0], [1.0, 4.0, 7.0]]], [[[8.0, 6.0, 1.0], [3.0, 7.0, 1.0], [1.0, 6.0, 6.0]], [[4.0, 6.0, 
3.0], [8.0, 7.0, 6.0], [2.0, 3.0, 6.0]], [[4.0, 1.0, 7.0], [4.0, 4.0, 2.0], [8.0, 6.0, 4.0]]], [[[7.0, 1.0, 7.0], [6.0, 4.0, 
3.0], [5.0, 6.0, 3.0]], [[8.0, 1.0, 7.0], [6.0, 7.0, 6.0], [8.0, 6.0, 6.0]], [[6.0, 4.0, 1.0], [8.0, 4.0, 6.0], [4.0, 6.0, 
1.0]]], [[[3.0, 3.0, 3.0], [2.0, 5.0, 4.0], [2.0, 1.0, 6.0]], [[5.0, 2.0, 2.0], [8.0, 3.0, 5.0], [6.0, 1.0, 7.0]], [[7.0, 3.0, 
6.0], [2.0, 2.0, 3.0], [7.0, 1.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 1.0]]], [[[7.0, 7.0]]], [[[4.0, 8.0]]], [[[4.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 6.0]]], [[[1.0, 3.0]]], [[[6.0, 3.0]]], [[[4.0, 3.0]]]]))
      arg[:,2:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 3.0], [8.0, 8.0, 5.0], [3.0, 2.0, 6.0]], [[4.0, 2.0, 4.0], [8.0, 2.0, 4.0], [8.0, 
6.0, 2.0]], [[3.0, 5.0, 8.0], [5.0, 1.0, 8.0], [1.0, 4.0, 7.0]]], [[[8.0, 6.0, 1.0], [3.0, 7.0, 1.0], [1.0, 6.0, 6.0]], [[4.0, 
6.0, 3.0], [8.0, 7.0, 6.0], [2.0, 3.0, 6.0]], [[4.0, 1.0, 7.0], [7.0, 7.0, 2.0], [8.0, 6.0, 4.0]]], [[[7.0, 1.0, 7.0], [6.0, 
4.0, 3.0], [5.0, 6.0, 3.0]], [[8.0, 1.0, 7.0], [6.0, 7.0, 6.0], [8.0, 6.0, 6.0]], [[6.0, 4.0, 1.0], [4.0, 8.0, 6.0], [4.0, 6.0, 
1.0]]], [[[3.0, 3.0, 3.0], [2.0, 5.0, 4.0], [2.0, 1.0, 6.0]], [[5.0, 2.0, 2.0], [8.0, 3.0, 5.0], [6.0, 1.0, 7.0]], [[7.0, 3.0, 
6.0], [4.0, 5.0, 3.0], [7.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 3.0], [8.0, 8.0, 5.0], [3.0, 2.0, 6.0]], 
[[4.0, 2.0, 4.0], [8.0, 2.0, 4.0], [8.0, 6.0, 2.0]], [[3.0, 5.0, 8.0], [5.0, 6.0, 8.0], [1.0, 4.0, 7.0]]], [[[8.0, 6.0, 1.0], 
[3.0, 7.0, 1.0], [1.0, 6.0, 6.0]], [[4.0, 6.0, 3.0], [8.0, 7.0, 6.0], [2.0, 3.0, 6.0]], [[4.0, 1.0, 7.0], [1.0, 3.0, 2.0], 
[8.0, 6.0, 4.0]]], [[[7.0, 1.0, 7.0], [6.0, 4.0, 3.0], [5.0, 6.0, 3.0]], [[8.0, 1.0, 7.0], [6.0, 7.0, 6.0], [8.0, 6.0, 6.0]], 
[[6.0, 4.0, 1.0], [6.0, 3.0, 6.0], [4.0, 6.0, 1.0]]], [[[3.0, 3.0, 3.0], [2.0, 5.0, 4.0], [2.0, 1.0, 6.0]], [[5.0, 2.0, 2.0], 
[8.0, 3.0, 5.0], [6.0, 1.0, 7.0]], [[7.0, 3.0, 6.0], [4.0, 3.0, 3.0], [7.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULRR(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 2.0], [6.0, 1.0, 4.0], [4.0, 8.0, 7.0]], [[3.0, 7.0, 3.0], [3.0, 5.0, 1.0], [5.0, 7.0, 
6.0]], [[8.0, 1.0, 4.0], [8.0, 8.0, 7.0], [4.0, 6.0, 7.0]]], [[[5.0, 7.0, 3.0], [4.0, 8.0, 6.0], [2.0, 1.0, 5.0]], [[4.0, 3.0, 
2.0], [4.0, 4.0, 4.0], [5.0, 4.0, 3.0]], [[2.0, 8.0, 1.0], [5.0, 7.0, 7.0], [6.0, 7.0, 8.0]]], [[[5.0, 4.0, 3.0], [8.0, 1.0, 
5.0], [2.0, 4.0, 6.0]], [[7.0, 4.0, 8.0], [8.0, 8.0, 5.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 2.0], [7.0, 6.0, 5.0], [6.0, 4.0, 
5.0]]], [[[3.0, 3.0, 8.0], [3.0, 2.0, 1.0], [7.0, 2.0, 6.0]], [[7.0, 1.0, 4.0], [7.0, 5.0, 8.0], [8.0, 2.0, 8.0]], [[7.0, 4.0, 
8.0], [6.0, 3.0, 3.0], [8.0, 8.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0]]], [[[3.0]]], [[[3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0]]], [[[6.0]]], [[[1.0]]]]))
      arg[:3,2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 2.0], [6.0, 1.0, 4.0], [4.0, 8.0, 7.0]], [[3.0, 7.0, 3.0], [3.0, 5.0, 1.0], [5.0, 
7.0, 6.0]], [[8.0, 1.0, 4.0], [8.0, 2.0, 7.0], [4.0, 6.0, 7.0]]], [[[5.0, 7.0, 3.0], [4.0, 8.0, 6.0], [2.0, 1.0, 5.0]], [[4.0, 
3.0, 2.0], [4.0, 4.0, 4.0], [5.0, 4.0, 3.0]], [[2.0, 8.0, 1.0], [5.0, 3.0, 7.0], [6.0, 7.0, 8.0]]], [[[5.0, 4.0, 3.0], [8.0, 
1.0, 5.0], [2.0, 4.0, 6.0]], [[7.0, 4.0, 8.0], [8.0, 8.0, 5.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 2.0], [7.0, 3.0, 5.0], [6.0, 4.0, 
5.0]]], [[[3.0, 3.0, 8.0], [3.0, 2.0, 1.0], [7.0, 2.0, 6.0]], [[7.0, 1.0, 4.0], [7.0, 5.0, 8.0], [8.0, 2.0, 8.0]], [[7.0, 4.0, 
8.0], [6.0, 3.0, 3.0], [8.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 2.0], [6.0, 1.0, 4.0], [4.0, 8.0, 7.0]], 
[[3.0, 7.0, 3.0], [3.0, 5.0, 1.0], [5.0, 7.0, 6.0]], [[8.0, 1.0, 4.0], [8.0, 5.0, 7.0], [4.0, 6.0, 7.0]]], [[[5.0, 7.0, 3.0], 
[4.0, 8.0, 6.0], [2.0, 1.0, 5.0]], [[4.0, 3.0, 2.0], [4.0, 4.0, 4.0], [5.0, 4.0, 3.0]], [[2.0, 8.0, 1.0], [5.0, 6.0, 7.0], 
[6.0, 7.0, 8.0]]], [[[5.0, 4.0, 3.0], [8.0, 1.0, 5.0], [2.0, 4.0, 6.0]], [[7.0, 4.0, 8.0], [8.0, 8.0, 5.0], [5.0, 5.0, 1.0]], 
[[5.0, 1.0, 2.0], [7.0, 1.0, 5.0], [6.0, 4.0, 5.0]]], [[[3.0, 3.0, 8.0], [3.0, 2.0, 1.0], [7.0, 2.0, 6.0]], [[7.0, 1.0, 4.0], 
[7.0, 5.0, 8.0], [8.0, 2.0, 8.0]], [[7.0, 4.0, 8.0], [6.0, 3.0, 3.0], [8.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RPPP(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 3.0], [4.0, 2.0, 8.0], [4.0, 8.0, 8.0]], [[2.0, 7.0, 7.0], [6.0, 4.0, 6.0], [3.0, 6.0, 
5.0]], [[4.0, 6.0, 7.0], [8.0, 7.0, 3.0], [1.0, 4.0, 4.0]]], [[[2.0, 1.0, 2.0], [1.0, 7.0, 7.0], [4.0, 7.0, 3.0]], [[2.0, 4.0, 
1.0], [3.0, 4.0, 2.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 3.0], [2.0, 2.0, 6.0], [4.0, 2.0, 2.0]]], [[[7.0, 4.0, 7.0], [7.0, 5.0, 
5.0], [7.0, 4.0, 3.0]], [[1.0, 7.0, 5.0], [5.0, 5.0, 5.0], [7.0, 3.0, 6.0]], [[4.0, 5.0, 4.0], [2.0, 7.0, 1.0], [6.0, 4.0, 
2.0]]], [[[7.0, 8.0, 2.0], [3.0, 7.0, 5.0], [3.0, 3.0, 1.0]], [[3.0, 6.0, 4.0], [3.0, 8.0, 5.0], [8.0, 7.0, 6.0]], [[7.0, 2.0, 
6.0], [1.0, 8.0, 8.0], [7.0, 1.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([2.0, 4.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([7.0, 6.0]))
      arg[1:3,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 3.0], [4.0, 2.0, 8.0], [4.0, 8.0, 8.0]], [[2.0, 7.0, 7.0], [6.0, 4.0, 6.0], [3.0, 
6.0, 5.0]], [[4.0, 6.0, 7.0], [8.0, 7.0, 3.0], [1.0, 4.0, 4.0]]], [[[2.0, 1.0, 2.0], [1.0, 7.0, 7.0], [4.0, 7.0, 3.0]], [[2.0, 
4.0, 1.0], [3.0, 2.0, 2.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 3.0], [2.0, 2.0, 6.0], [4.0, 2.0, 2.0]]], [[[7.0, 4.0, 7.0], [7.0, 
5.0, 5.0], [7.0, 4.0, 3.0]], [[1.0, 7.0, 5.0], [5.0, 4.0, 5.0], [7.0, 3.0, 6.0]], [[4.0, 5.0, 4.0], [2.0, 7.0, 1.0], [6.0, 4.0, 
2.0]]], [[[7.0, 8.0, 2.0], [3.0, 7.0, 5.0], [3.0, 3.0, 1.0]], [[3.0, 6.0, 4.0], [3.0, 8.0, 5.0], [8.0, 7.0, 6.0]], [[7.0, 2.0, 
6.0], [1.0, 8.0, 8.0], [7.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 3.0], [4.0, 2.0, 8.0], [4.0, 8.0, 8.0]], 
[[2.0, 7.0, 7.0], [6.0, 4.0, 6.0], [3.0, 6.0, 5.0]], [[4.0, 6.0, 7.0], [8.0, 7.0, 3.0], [1.0, 4.0, 4.0]]], [[[2.0, 1.0, 2.0], 
[1.0, 7.0, 7.0], [4.0, 7.0, 3.0]], [[2.0, 4.0, 1.0], [3.0, 7.0, 2.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 3.0], [2.0, 2.0, 6.0], 
[4.0, 2.0, 2.0]]], [[[7.0, 4.0, 7.0], [7.0, 5.0, 5.0], [7.0, 4.0, 3.0]], [[1.0, 7.0, 5.0], [5.0, 6.0, 5.0], [7.0, 3.0, 6.0]], 
[[4.0, 5.0, 4.0], [2.0, 7.0, 1.0], [6.0, 4.0, 2.0]]], [[[7.0, 8.0, 2.0], [3.0, 7.0, 5.0], [3.0, 3.0, 1.0]], [[3.0, 6.0, 4.0], 
[3.0, 8.0, 5.0], [8.0, 7.0, 6.0]], [[7.0, 2.0, 6.0], [1.0, 8.0, 8.0], [7.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LRUU(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 6.0], [3.0, 1.0, 4.0], [5.0, 4.0, 8.0]], [[3.0, 4.0, 3.0], [2.0, 8.0, 5.0], [7.0, 6.0, 
7.0]], [[7.0, 5.0, 4.0], [7.0, 3.0, 6.0], [4.0, 5.0, 3.0]]], [[[1.0, 2.0, 8.0], [5.0, 2.0, 5.0], [4.0, 4.0, 2.0]], [[8.0, 3.0, 
1.0], [3.0, 3.0, 5.0], [3.0, 1.0, 3.0]], [[2.0, 5.0, 7.0], [3.0, 1.0, 8.0], [6.0, 7.0, 5.0]]], [[[5.0, 8.0, 2.0], [8.0, 4.0, 
3.0], [3.0, 1.0, 1.0]], [[7.0, 8.0, 5.0], [8.0, 2.0, 2.0], [7.0, 5.0, 6.0]], [[3.0, 7.0, 8.0], [1.0, 6.0, 6.0], [1.0, 4.0, 
1.0]]], [[[3.0, 7.0, 1.0], [3.0, 4.0, 2.0], [3.0, 5.0, 6.0]], [[1.0, 8.0, 1.0], [5.0, 1.0, 7.0], [7.0, 6.0, 2.0]], [[7.0, 6.0, 
3.0], [7.0, 8.0, 7.0], [7.0, 7.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 1.0], [3.0, 8.0]]], [[[4.0, 7.0], [7.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 6.0], [2.0, 5.0]]], [[[1.0, 2.0], [1.0, 5.0]]]]))
      arg[2:,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 6.0], [3.0, 1.0, 4.0], [5.0, 4.0, 8.0]], [[3.0, 4.0, 3.0], [2.0, 8.0, 5.0], [7.0, 
6.0, 7.0]], [[7.0, 5.0, 4.0], [7.0, 3.0, 6.0], [4.0, 5.0, 3.0]]], [[[1.0, 2.0, 8.0], [5.0, 2.0, 5.0], [4.0, 4.0, 2.0]], [[8.0, 
3.0, 1.0], [3.0, 3.0, 5.0], [3.0, 1.0, 3.0]], [[2.0, 5.0, 7.0], [3.0, 1.0, 8.0], [6.0, 7.0, 5.0]]], [[[5.0, 8.0, 2.0], [8.0, 
4.0, 3.0], [3.0, 1.0, 1.0]], [[6.0, 1.0, 5.0], [3.0, 8.0, 2.0], [7.0, 5.0, 6.0]], [[3.0, 7.0, 8.0], [1.0, 6.0, 6.0], [1.0, 4.0, 
1.0]]], [[[3.0, 7.0, 1.0], [3.0, 4.0, 2.0], [3.0, 5.0, 6.0]], [[4.0, 7.0, 1.0], [7.0, 7.0, 7.0], [7.0, 6.0, 2.0]], [[7.0, 6.0, 
3.0], [7.0, 8.0, 7.0], [7.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 6.0], [3.0, 1.0, 4.0], [5.0, 4.0, 8.0]], 
[[3.0, 4.0, 3.0], [2.0, 8.0, 5.0], [7.0, 6.0, 7.0]], [[7.0, 5.0, 4.0], [7.0, 3.0, 6.0], [4.0, 5.0, 3.0]]], [[[1.0, 2.0, 8.0], 
[5.0, 2.0, 5.0], [4.0, 4.0, 2.0]], [[8.0, 3.0, 1.0], [3.0, 3.0, 5.0], [3.0, 1.0, 3.0]], [[2.0, 5.0, 7.0], [3.0, 1.0, 8.0], 
[6.0, 7.0, 5.0]]], [[[5.0, 8.0, 2.0], [8.0, 4.0, 3.0], [3.0, 1.0, 1.0]], [[8.0, 6.0, 5.0], [2.0, 5.0, 2.0], [7.0, 5.0, 6.0]], 
[[3.0, 7.0, 8.0], [1.0, 6.0, 6.0], [1.0, 4.0, 1.0]]], [[[3.0, 7.0, 1.0], [3.0, 4.0, 2.0], [3.0, 5.0, 6.0]], [[1.0, 2.0, 1.0], 
[1.0, 5.0, 7.0], [7.0, 6.0, 2.0]], [[7.0, 6.0, 3.0], [7.0, 8.0, 7.0], [7.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPLU(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 8.0], [8.0, 1.0, 8.0], [5.0, 2.0, 4.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 2.0], [7.0, 3.0, 
8.0]], [[8.0, 8.0, 5.0], [4.0, 5.0, 7.0], [6.0, 8.0, 2.0]]], [[[5.0, 1.0, 3.0], [3.0, 1.0, 6.0], [7.0, 3.0, 7.0]], [[4.0, 6.0, 
4.0], [6.0, 5.0, 7.0], [8.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [5.0, 2.0, 7.0], [3.0, 2.0, 2.0]]], [[[7.0, 5.0, 8.0], [1.0, 7.0, 
1.0], [2.0, 3.0, 1.0]], [[1.0, 2.0, 6.0], [8.0, 4.0, 5.0], [4.0, 3.0, 4.0]], [[2.0, 2.0, 8.0], [8.0, 1.0, 3.0], [4.0, 7.0, 
8.0]]], [[[6.0, 4.0, 5.0], [4.0, 6.0, 1.0], [4.0, 7.0, 6.0]], [[5.0, 7.0, 5.0], [6.0, 4.0, 6.0], [7.0, 8.0, 8.0]], [[2.0, 6.0, 
1.0], [8.0, 7.0, 2.0], [6.0, 6.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 1.0]], [[4.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 3.0]], [[7.0, 4.0]]]))
      arg[2:,1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 1.0, 8.0], [5.0, 2.0, 4.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 2.0], [7.0, 
3.0, 8.0]], [[8.0, 8.0, 5.0], [4.0, 5.0, 7.0], [6.0, 8.0, 2.0]]], [[[5.0, 1.0, 3.0], [3.0, 1.0, 6.0], [7.0, 3.0, 7.0]], [[4.0, 
6.0, 4.0], [6.0, 5.0, 7.0], [8.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [5.0, 2.0, 7.0], [3.0, 2.0, 2.0]]], [[[7.0, 5.0, 8.0], [1.0, 
7.0, 1.0], [2.0, 3.0, 1.0]], [[1.0, 2.0, 6.0], [8.0, 4.0, 5.0], [8.0, 1.0, 4.0]], [[2.0, 2.0, 8.0], [8.0, 1.0, 3.0], [4.0, 7.0, 
8.0]]], [[[6.0, 4.0, 5.0], [4.0, 6.0, 1.0], [4.0, 7.0, 6.0]], [[5.0, 7.0, 5.0], [6.0, 4.0, 6.0], [4.0, 3.0, 8.0]], [[2.0, 6.0, 
1.0], [8.0, 7.0, 2.0], [6.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 1.0, 8.0], [5.0, 2.0, 4.0]], 
[[7.0, 2.0, 8.0], [3.0, 4.0, 2.0], [7.0, 3.0, 8.0]], [[8.0, 8.0, 5.0], [4.0, 5.0, 7.0], [6.0, 8.0, 2.0]]], [[[5.0, 1.0, 3.0], 
[3.0, 1.0, 6.0], [7.0, 3.0, 7.0]], [[4.0, 6.0, 4.0], [6.0, 5.0, 7.0], [8.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [5.0, 2.0, 7.0], 
[3.0, 2.0, 2.0]]], [[[7.0, 5.0, 8.0], [1.0, 7.0, 1.0], [2.0, 3.0, 1.0]], [[1.0, 2.0, 6.0], [8.0, 4.0, 5.0], [7.0, 3.0, 4.0]], 
[[2.0, 2.0, 8.0], [8.0, 1.0, 3.0], [4.0, 7.0, 8.0]]], [[[6.0, 4.0, 5.0], [4.0, 6.0, 1.0], [4.0, 7.0, 6.0]], [[5.0, 7.0, 5.0], 
[6.0, 4.0, 6.0], [7.0, 4.0, 8.0]], [[2.0, 6.0, 1.0], [8.0, 7.0, 2.0], [6.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LRCN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 7.0], [1.0, 7.0, 2.0], [8.0, 8.0, 4.0]], [[8.0, 7.0, 4.0], [2.0, 1.0, 2.0], [5.0, 7.0, 
6.0]], [[7.0, 1.0, 7.0], [3.0, 3.0, 4.0], [2.0, 7.0, 3.0]]], [[[5.0, 6.0, 4.0], [8.0, 6.0, 4.0], [5.0, 1.0, 4.0]], [[5.0, 5.0, 
6.0], [5.0, 4.0, 8.0], [2.0, 8.0, 4.0]], [[2.0, 4.0, 3.0], [7.0, 2.0, 2.0], [5.0, 6.0, 5.0]]], [[[7.0, 4.0, 8.0], [3.0, 1.0, 
6.0], [7.0, 6.0, 2.0]], [[6.0, 2.0, 4.0], [8.0, 8.0, 7.0], [2.0, 4.0, 5.0]], [[2.0, 5.0, 3.0], [8.0, 4.0, 1.0], [3.0, 7.0, 
6.0]]], [[[8.0, 4.0, 5.0], [4.0, 6.0, 1.0], [3.0, 7.0, 1.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 7.0], [2.0, 1.0, 5.0]], [[6.0, 7.0, 
7.0], [8.0, 4.0, 4.0], [3.0, 3.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 7.0, 5.0], [7.0, 7.0, 1.0], [5.0, 6.0, 1.0]]], [[[2.0, 7.0, 7.0], [2.0, 7.0, 4.0], [6.0, 
1.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 2.0], [8.0, 7.0, 8.0], [6.0, 1.0, 2.0]]], [[[4.0, 1.0, 8.0], [1.0, 8.0, 
2.0], [2.0, 3.0, 8.0]]]]))
      arg[2:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 7.0], [1.0, 7.0, 2.0], [8.0, 8.0, 4.0]], [[8.0, 7.0, 4.0], [2.0, 1.0, 2.0], [5.0, 
7.0, 6.0]], [[7.0, 1.0, 7.0], [3.0, 3.0, 4.0], [2.0, 7.0, 3.0]]], [[[5.0, 6.0, 4.0], [8.0, 6.0, 4.0], [5.0, 1.0, 4.0]], [[5.0, 
5.0, 6.0], [5.0, 4.0, 8.0], [2.0, 8.0, 4.0]], [[2.0, 4.0, 3.0], [7.0, 2.0, 2.0], [5.0, 6.0, 5.0]]], [[[7.0, 4.0, 8.0], [3.0, 
1.0, 6.0], [7.0, 6.0, 2.0]], [[3.0, 7.0, 5.0], [7.0, 7.0, 1.0], [5.0, 6.0, 1.0]], [[2.0, 5.0, 3.0], [8.0, 4.0, 1.0], [3.0, 7.0, 
6.0]]], [[[8.0, 4.0, 5.0], [4.0, 6.0, 1.0], [3.0, 7.0, 1.0]], [[2.0, 7.0, 7.0], [2.0, 7.0, 4.0], [6.0, 1.0, 2.0]], [[6.0, 7.0, 
7.0], [8.0, 4.0, 4.0], [3.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 7.0], [1.0, 7.0, 2.0], [8.0, 8.0, 4.0]], 
[[8.0, 7.0, 4.0], [2.0, 1.0, 2.0], [5.0, 7.0, 6.0]], [[7.0, 1.0, 7.0], [3.0, 3.0, 4.0], [2.0, 7.0, 3.0]]], [[[5.0, 6.0, 4.0], 
[8.0, 6.0, 4.0], [5.0, 1.0, 4.0]], [[5.0, 5.0, 6.0], [5.0, 4.0, 8.0], [2.0, 8.0, 4.0]], [[2.0, 4.0, 3.0], [7.0, 2.0, 2.0], 
[5.0, 6.0, 5.0]]], [[[7.0, 4.0, 8.0], [3.0, 1.0, 6.0], [7.0, 6.0, 2.0]], [[1.0, 8.0, 2.0], [8.0, 7.0, 8.0], [6.0, 1.0, 2.0]], 
[[2.0, 5.0, 3.0], [8.0, 4.0, 1.0], [3.0, 7.0, 6.0]]], [[[8.0, 4.0, 5.0], [4.0, 6.0, 1.0], [3.0, 7.0, 1.0]], [[4.0, 1.0, 8.0], 
[1.0, 8.0, 2.0], [2.0, 3.0, 8.0]], [[6.0, 7.0, 7.0], [8.0, 4.0, 4.0], [3.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULCL(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 4.0], [7.0, 5.0, 7.0], [7.0, 1.0, 2.0]], [[6.0, 3.0, 5.0], [5.0, 8.0, 2.0], [2.0, 5.0, 
5.0]], [[4.0, 4.0, 4.0], [8.0, 2.0, 7.0], [8.0, 3.0, 7.0]]], [[[4.0, 8.0, 4.0], [1.0, 1.0, 1.0], [1.0, 2.0, 4.0]], [[3.0, 4.0, 
4.0], [1.0, 4.0, 7.0], [1.0, 2.0, 3.0]], [[3.0, 8.0, 2.0], [5.0, 1.0, 8.0], [5.0, 6.0, 6.0]]], [[[7.0, 5.0, 4.0], [7.0, 3.0, 
5.0], [5.0, 5.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 6.0, 8.0], [2.0, 5.0, 2.0]], [[2.0, 2.0, 1.0], [4.0, 6.0, 6.0], [3.0, 8.0, 
6.0]]], [[[2.0, 5.0, 2.0], [6.0, 3.0, 5.0], [4.0, 2.0, 5.0]], [[3.0, 8.0, 6.0], [2.0, 2.0, 4.0], [3.0, 7.0, 6.0]], [[2.0, 7.0, 
4.0], [1.0, 8.0, 5.0], [1.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0], [5.0], [7.0]]], [[[1.0], [2.0], [3.0]]], [[[7.0], [6.0], [4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0], [4.0], [6.0]]], [[[1.0], [5.0], [7.0]]], [[[8.0], [7.0], [4.0]]]]))
      arg[:3,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 4.0], [7.0, 5.0, 7.0], [7.0, 1.0, 2.0]], [[6.0, 3.0, 5.0], [5.0, 8.0, 2.0], [2.0, 
5.0, 5.0]], [[4.0, 4.0, 7.0], [8.0, 2.0, 5.0], [8.0, 3.0, 7.0]]], [[[4.0, 8.0, 4.0], [1.0, 1.0, 1.0], [1.0, 2.0, 4.0]], [[3.0, 
4.0, 4.0], [1.0, 4.0, 7.0], [1.0, 2.0, 3.0]], [[3.0, 8.0, 1.0], [5.0, 1.0, 2.0], [5.0, 6.0, 3.0]]], [[[7.0, 5.0, 4.0], [7.0, 
3.0, 5.0], [5.0, 5.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 6.0, 8.0], [2.0, 5.0, 2.0]], [[2.0, 2.0, 7.0], [4.0, 6.0, 6.0], [3.0, 8.0, 
4.0]]], [[[2.0, 5.0, 2.0], [6.0, 3.0, 5.0], [4.0, 2.0, 5.0]], [[3.0, 8.0, 6.0], [2.0, 2.0, 4.0], [3.0, 7.0, 6.0]], [[2.0, 7.0, 
4.0], [1.0, 8.0, 5.0], [1.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 4.0], [7.0, 5.0, 7.0], [7.0, 1.0, 2.0]], 
[[6.0, 3.0, 5.0], [5.0, 8.0, 2.0], [2.0, 5.0, 5.0]], [[4.0, 4.0, 1.0], [8.0, 2.0, 4.0], [8.0, 3.0, 6.0]]], [[[4.0, 8.0, 4.0], 
[1.0, 1.0, 1.0], [1.0, 2.0, 4.0]], [[3.0, 4.0, 4.0], [1.0, 4.0, 7.0], [1.0, 2.0, 3.0]], [[3.0, 8.0, 1.0], [5.0, 1.0, 5.0], 
[5.0, 6.0, 7.0]]], [[[7.0, 5.0, 4.0], [7.0, 3.0, 5.0], [5.0, 5.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 6.0, 8.0], [2.0, 5.0, 2.0]], 
[[2.0, 2.0, 8.0], [4.0, 6.0, 7.0], [3.0, 8.0, 4.0]]], [[[2.0, 5.0, 2.0], [6.0, 3.0, 5.0], [4.0, 2.0, 5.0]], [[3.0, 8.0, 6.0], 
[2.0, 2.0, 4.0], [3.0, 7.0, 6.0]], [[2.0, 7.0, 4.0], [1.0, 8.0, 5.0], [1.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RRPP(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 8.0], [1.0, 1.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 2.0, 8.0], [8.0, 5.0, 5.0], [4.0, 6.0, 
3.0]], [[2.0, 2.0, 1.0], [5.0, 3.0, 6.0], [8.0, 1.0, 4.0]]], [[[8.0, 5.0, 5.0], [5.0, 8.0, 6.0], [2.0, 8.0, 1.0]], [[6.0, 5.0, 
3.0], [3.0, 5.0, 7.0], [1.0, 1.0, 7.0]], [[2.0, 5.0, 5.0], [3.0, 5.0, 2.0], [6.0, 4.0, 4.0]]], [[[7.0, 2.0, 3.0], [5.0, 7.0, 
3.0], [7.0, 4.0, 8.0]], [[8.0, 3.0, 8.0], [6.0, 3.0, 5.0], [6.0, 3.0, 6.0]], [[8.0, 5.0, 1.0], [6.0, 3.0, 2.0], [5.0, 4.0, 
1.0]]], [[[5.0, 5.0, 2.0], [5.0, 7.0, 8.0], [8.0, 7.0, 7.0]], [[8.0, 3.0, 5.0], [2.0, 3.0, 3.0], [3.0, 3.0, 7.0]], [[4.0, 3.0, 
1.0], [1.0, 3.0, 6.0], [6.0, 8.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[1.0], [4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0], [7.0]]))
      arg[1:3,1:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 8.0], [1.0, 1.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 2.0, 8.0], [8.0, 5.0, 5.0], [4.0, 
6.0, 3.0]], [[2.0, 2.0, 1.0], [5.0, 3.0, 6.0], [8.0, 1.0, 4.0]]], [[[8.0, 5.0, 5.0], [5.0, 8.0, 6.0], [2.0, 8.0, 1.0]], [[6.0, 
5.0, 3.0], [3.0, 1.0, 7.0], [1.0, 1.0, 7.0]], [[2.0, 5.0, 5.0], [3.0, 5.0, 2.0], [6.0, 4.0, 4.0]]], [[[7.0, 2.0, 3.0], [5.0, 
7.0, 3.0], [7.0, 4.0, 8.0]], [[8.0, 3.0, 8.0], [6.0, 4.0, 5.0], [6.0, 3.0, 6.0]], [[8.0, 5.0, 1.0], [6.0, 3.0, 2.0], [5.0, 4.0, 
1.0]]], [[[5.0, 5.0, 2.0], [5.0, 7.0, 8.0], [8.0, 7.0, 7.0]], [[8.0, 3.0, 5.0], [2.0, 3.0, 3.0], [3.0, 3.0, 7.0]], [[4.0, 3.0, 
1.0], [1.0, 3.0, 6.0], [6.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 8.0], [1.0, 1.0, 7.0], [3.0, 8.0, 8.0]], 
[[5.0, 2.0, 8.0], [8.0, 5.0, 5.0], [4.0, 6.0, 3.0]], [[2.0, 2.0, 1.0], [5.0, 3.0, 6.0], [8.0, 1.0, 4.0]]], [[[8.0, 5.0, 5.0], 
[5.0, 8.0, 6.0], [2.0, 8.0, 1.0]], [[6.0, 5.0, 3.0], [3.0, 4.0, 7.0], [1.0, 1.0, 7.0]], [[2.0, 5.0, 5.0], [3.0, 5.0, 2.0], 
[6.0, 4.0, 4.0]]], [[[7.0, 2.0, 3.0], [5.0, 7.0, 3.0], [7.0, 4.0, 8.0]], [[8.0, 3.0, 8.0], [6.0, 7.0, 5.0], [6.0, 3.0, 6.0]], 
[[8.0, 5.0, 1.0], [6.0, 3.0, 2.0], [5.0, 4.0, 1.0]]], [[[5.0, 5.0, 2.0], [5.0, 7.0, 8.0], [8.0, 7.0, 7.0]], [[8.0, 3.0, 5.0], 
[2.0, 3.0, 3.0], [3.0, 3.0, 7.0]], [[4.0, 3.0, 1.0], [1.0, 3.0, 6.0], [6.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLLP(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 5.0], [4.0, 2.0, 5.0], [8.0, 8.0, 1.0]], [[2.0, 3.0, 2.0], [2.0, 4.0, 6.0], [2.0, 1.0, 
4.0]], [[6.0, 5.0, 7.0], [7.0, 8.0, 8.0], [2.0, 8.0, 4.0]]], [[[3.0, 3.0, 8.0], [4.0, 8.0, 8.0], [5.0, 8.0, 1.0]], [[8.0, 1.0, 
6.0], [2.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 4.0, 4.0], [7.0, 7.0, 4.0], [1.0, 3.0, 6.0]]], [[[6.0, 1.0, 6.0], [5.0, 2.0, 
3.0], [6.0, 6.0, 4.0]], [[5.0, 6.0, 4.0], [5.0, 4.0, 6.0], [5.0, 2.0, 4.0]], [[3.0, 1.0, 6.0], [6.0, 8.0, 6.0], [7.0, 5.0, 
6.0]]], [[[7.0, 7.0, 8.0], [5.0, 4.0, 7.0], [3.0, 3.0, 3.0]], [[4.0, 4.0, 3.0], [6.0, 7.0, 7.0], [5.0, 6.0, 6.0]], [[5.0, 5.0, 
5.0], [5.0, 3.0, 8.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0]], [[5.0]], [[2.0]], [[1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0]], [[1.0]], [[8.0]], [[2.0]]]))
      arg[:,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 5.0], [4.0, 2.0, 5.0], [8.0, 8.0, 1.0]], [[2.0, 3.0, 2.0], [2.0, 4.0, 6.0], [2.0, 
1.0, 4.0]], [[6.0, 5.0, 7.0], [7.0, 8.0, 8.0], [2.0, 7.0, 4.0]]], [[[3.0, 3.0, 8.0], [4.0, 8.0, 8.0], [5.0, 8.0, 1.0]], [[8.0, 
1.0, 6.0], [2.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 4.0, 4.0], [7.0, 7.0, 4.0], [1.0, 5.0, 6.0]]], [[[6.0, 1.0, 6.0], [5.0, 
2.0, 3.0], [6.0, 6.0, 4.0]], [[5.0, 6.0, 4.0], [5.0, 4.0, 6.0], [5.0, 2.0, 4.0]], [[3.0, 1.0, 6.0], [6.0, 8.0, 6.0], [7.0, 2.0, 
6.0]]], [[[7.0, 7.0, 8.0], [5.0, 4.0, 7.0], [3.0, 3.0, 3.0]], [[4.0, 4.0, 3.0], [6.0, 7.0, 7.0], [5.0, 6.0, 6.0]], [[5.0, 5.0, 
5.0], [5.0, 3.0, 8.0], [2.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 5.0], [4.0, 2.0, 5.0], [8.0, 8.0, 1.0]], 
[[2.0, 3.0, 2.0], [2.0, 4.0, 6.0], [2.0, 1.0, 4.0]], [[6.0, 5.0, 7.0], [7.0, 8.0, 8.0], [2.0, 6.0, 4.0]]], [[[3.0, 3.0, 8.0], 
[4.0, 8.0, 8.0], [5.0, 8.0, 1.0]], [[8.0, 1.0, 6.0], [2.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 4.0, 4.0], [7.0, 7.0, 4.0], 
[1.0, 1.0, 6.0]]], [[[6.0, 1.0, 6.0], [5.0, 2.0, 3.0], [6.0, 6.0, 4.0]], [[5.0, 6.0, 4.0], [5.0, 4.0, 6.0], [5.0, 2.0, 4.0]], 
[[3.0, 1.0, 6.0], [6.0, 8.0, 6.0], [7.0, 8.0, 6.0]]], [[[7.0, 7.0, 8.0], [5.0, 4.0, 7.0], [3.0, 3.0, 3.0]], [[4.0, 4.0, 3.0], 
[6.0, 7.0, 7.0], [5.0, 6.0, 6.0]], [[5.0, 5.0, 5.0], [5.0, 3.0, 8.0], [2.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_URRN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 4.0], [3.0, 2.0, 2.0], [1.0, 7.0, 3.0]], [[4.0, 6.0, 5.0], [5.0, 2.0, 6.0], [6.0, 8.0, 
5.0]], [[3.0, 3.0, 3.0], [3.0, 8.0, 5.0], [8.0, 1.0, 1.0]]], [[[3.0, 1.0, 8.0], [3.0, 7.0, 3.0], [4.0, 2.0, 5.0]], [[8.0, 1.0, 
4.0], [8.0, 3.0, 5.0], [1.0, 1.0, 3.0]], [[6.0, 7.0, 1.0], [8.0, 5.0, 7.0], [3.0, 8.0, 7.0]]], [[[1.0, 2.0, 2.0], [5.0, 1.0, 
6.0], [2.0, 8.0, 6.0]], [[3.0, 6.0, 8.0], [2.0, 2.0, 2.0], [6.0, 5.0, 5.0]], [[2.0, 3.0, 3.0], [6.0, 2.0, 8.0], [2.0, 5.0, 
7.0]]], [[[6.0, 3.0, 1.0], [7.0, 2.0, 8.0], [6.0, 3.0, 6.0]], [[2.0, 7.0, 6.0], [1.0, 5.0, 6.0], [3.0, 2.0, 6.0]], [[6.0, 7.0, 
3.0], [3.0, 1.0, 2.0], [1.0, 3.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 8.0, 8.0]]], [[[1.0, 4.0, 7.0]]], [[[7.0, 5.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 2.0]]], [[[1.0, 7.0, 2.0]]], [[[3.0, 2.0, 3.0]]]]))
      arg[:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 4.0], [3.0, 2.0, 2.0], [1.0, 7.0, 3.0]], [[4.0, 6.0, 5.0], [5.0, 8.0, 8.0], [6.0, 
8.0, 5.0]], [[3.0, 3.0, 3.0], [3.0, 8.0, 5.0], [8.0, 1.0, 1.0]]], [[[3.0, 1.0, 8.0], [3.0, 7.0, 3.0], [4.0, 2.0, 5.0]], [[8.0, 
1.0, 4.0], [1.0, 4.0, 7.0], [1.0, 1.0, 3.0]], [[6.0, 7.0, 1.0], [8.0, 5.0, 7.0], [3.0, 8.0, 7.0]]], [[[1.0, 2.0, 2.0], [5.0, 
1.0, 6.0], [2.0, 8.0, 6.0]], [[3.0, 6.0, 8.0], [7.0, 5.0, 3.0], [6.0, 5.0, 5.0]], [[2.0, 3.0, 3.0], [6.0, 2.0, 8.0], [2.0, 5.0, 
7.0]]], [[[6.0, 3.0, 1.0], [7.0, 2.0, 8.0], [6.0, 3.0, 6.0]], [[2.0, 7.0, 6.0], [1.0, 5.0, 6.0], [3.0, 2.0, 6.0]], [[6.0, 7.0, 
3.0], [3.0, 1.0, 2.0], [1.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 4.0], [3.0, 2.0, 2.0], [1.0, 7.0, 3.0]], 
[[4.0, 6.0, 5.0], [7.0, 3.0, 2.0], [6.0, 8.0, 5.0]], [[3.0, 3.0, 3.0], [3.0, 8.0, 5.0], [8.0, 1.0, 1.0]]], [[[3.0, 1.0, 8.0], 
[3.0, 7.0, 3.0], [4.0, 2.0, 5.0]], [[8.0, 1.0, 4.0], [1.0, 7.0, 2.0], [1.0, 1.0, 3.0]], [[6.0, 7.0, 1.0], [8.0, 5.0, 7.0], 
[3.0, 8.0, 7.0]]], [[[1.0, 2.0, 2.0], [5.0, 1.0, 6.0], [2.0, 8.0, 6.0]], [[3.0, 6.0, 8.0], [3.0, 2.0, 3.0], [6.0, 5.0, 5.0]], 
[[2.0, 3.0, 3.0], [6.0, 2.0, 8.0], [2.0, 5.0, 7.0]]], [[[6.0, 3.0, 1.0], [7.0, 2.0, 8.0], [6.0, 3.0, 6.0]], [[2.0, 7.0, 6.0], 
[1.0, 5.0, 6.0], [3.0, 2.0, 6.0]], [[6.0, 7.0, 3.0], [3.0, 1.0, 2.0], [1.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RRLN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 6.0], [1.0, 6.0, 6.0], [5.0, 7.0, 8.0]], [[8.0, 2.0, 4.0], [4.0, 1.0, 3.0], [3.0, 8.0, 
5.0]], [[2.0, 7.0, 2.0], [1.0, 8.0, 6.0], [8.0, 3.0, 4.0]]], [[[1.0, 1.0, 1.0], [3.0, 3.0, 4.0], [8.0, 8.0, 4.0]], [[7.0, 1.0, 
6.0], [5.0, 2.0, 8.0], [7.0, 4.0, 3.0]], [[1.0, 2.0, 2.0], [1.0, 4.0, 1.0], [1.0, 4.0, 7.0]]], [[[1.0, 5.0, 1.0], [3.0, 6.0, 
8.0], [1.0, 7.0, 7.0]], [[2.0, 4.0, 7.0], [8.0, 3.0, 6.0], [7.0, 8.0, 6.0]], [[8.0, 7.0, 4.0], [7.0, 4.0, 5.0], [8.0, 5.0, 
4.0]]], [[[2.0, 2.0, 5.0], [1.0, 5.0, 4.0], [7.0, 8.0, 7.0]], [[8.0, 8.0, 5.0], [7.0, 1.0, 5.0], [1.0, 5.0, 3.0]], [[6.0, 6.0, 
8.0], [4.0, 6.0, 2.0], [4.0, 5.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 2.0, 1.0]]], [[[3.0, 2.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 8.0]]], [[[8.0, 5.0, 3.0]]]]))
      arg[1:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 6.0], [1.0, 6.0, 6.0], [5.0, 7.0, 8.0]], [[8.0, 2.0, 4.0], [4.0, 1.0, 3.0], [3.0, 
8.0, 5.0]], [[2.0, 7.0, 2.0], [1.0, 8.0, 6.0], [8.0, 3.0, 4.0]]], [[[1.0, 1.0, 1.0], [3.0, 3.0, 4.0], [8.0, 8.0, 4.0]], [[7.0, 
1.0, 6.0], [5.0, 2.0, 8.0], [7.0, 2.0, 1.0]], [[1.0, 2.0, 2.0], [1.0, 4.0, 1.0], [1.0, 4.0, 7.0]]], [[[1.0, 5.0, 1.0], [3.0, 
6.0, 8.0], [1.0, 7.0, 7.0]], [[2.0, 4.0, 7.0], [8.0, 3.0, 6.0], [3.0, 2.0, 4.0]], [[8.0, 7.0, 4.0], [7.0, 4.0, 5.0], [8.0, 5.0, 
4.0]]], [[[2.0, 2.0, 5.0], [1.0, 5.0, 4.0], [7.0, 8.0, 7.0]], [[8.0, 8.0, 5.0], [7.0, 1.0, 5.0], [1.0, 5.0, 3.0]], [[6.0, 6.0, 
8.0], [4.0, 6.0, 2.0], [4.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 6.0], [1.0, 6.0, 6.0], [5.0, 7.0, 8.0]], 
[[8.0, 2.0, 4.0], [4.0, 1.0, 3.0], [3.0, 8.0, 5.0]], [[2.0, 7.0, 2.0], [1.0, 8.0, 6.0], [8.0, 3.0, 4.0]]], [[[1.0, 1.0, 1.0], 
[3.0, 3.0, 4.0], [8.0, 8.0, 4.0]], [[7.0, 1.0, 6.0], [5.0, 2.0, 8.0], [8.0, 8.0, 8.0]], [[1.0, 2.0, 2.0], [1.0, 4.0, 1.0], 
[1.0, 4.0, 7.0]]], [[[1.0, 5.0, 1.0], [3.0, 6.0, 8.0], [1.0, 7.0, 7.0]], [[2.0, 4.0, 7.0], [8.0, 3.0, 6.0], [8.0, 5.0, 3.0]], 
[[8.0, 7.0, 4.0], [7.0, 4.0, 5.0], [8.0, 5.0, 4.0]]], [[[2.0, 2.0, 5.0], [1.0, 5.0, 4.0], [7.0, 8.0, 7.0]], [[8.0, 8.0, 5.0], 
[7.0, 1.0, 5.0], [1.0, 5.0, 3.0]], [[6.0, 6.0, 8.0], [4.0, 6.0, 2.0], [4.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULUN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 1.0], [4.0, 6.0, 4.0], [6.0, 7.0, 2.0]], [[3.0, 3.0, 3.0], [8.0, 4.0, 8.0], [1.0, 6.0, 
8.0]], [[3.0, 4.0, 8.0], [3.0, 1.0, 7.0], [7.0, 4.0, 2.0]]], [[[6.0, 6.0, 2.0], [2.0, 1.0, 2.0], [2.0, 8.0, 3.0]], [[5.0, 3.0, 
6.0], [2.0, 1.0, 1.0], [4.0, 3.0, 7.0]], [[6.0, 6.0, 6.0], [1.0, 1.0, 3.0], [2.0, 4.0, 1.0]]], [[[5.0, 4.0, 1.0], [3.0, 7.0, 
4.0], [7.0, 7.0, 2.0]], [[1.0, 3.0, 7.0], [7.0, 3.0, 6.0], [4.0, 4.0, 7.0]], [[7.0, 6.0, 3.0], [1.0, 7.0, 1.0], [8.0, 7.0, 
4.0]]], [[[5.0, 3.0, 2.0], [3.0, 2.0, 2.0], [7.0, 5.0, 4.0]], [[2.0, 8.0, 3.0], [8.0, 6.0, 4.0], [1.0, 5.0, 8.0]], [[2.0, 5.0, 
3.0], [1.0, 5.0, 7.0], [3.0, 5.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 6.0, 2.0], [5.0, 2.0, 5.0]]], [[[5.0, 8.0, 4.0], [8.0, 4.0, 4.0]]], [[[4.0, 7.0, 4.0], 
[7.0, 7.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 7.0], [6.0, 3.0, 5.0]]], [[[6.0, 8.0, 7.0], [4.0, 6.0, 8.0]]], [[[4.0, 
5.0, 4.0], [8.0, 4.0, 8.0]]]]))
      arg[:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 1.0], [4.0, 6.0, 4.0], [6.0, 7.0, 2.0]], [[3.0, 3.0, 3.0], [8.0, 4.0, 8.0], [1.0, 
6.0, 8.0]], [[2.0, 6.0, 2.0], [5.0, 2.0, 5.0], [7.0, 4.0, 2.0]]], [[[6.0, 6.0, 2.0], [2.0, 1.0, 2.0], [2.0, 8.0, 3.0]], [[5.0, 
3.0, 6.0], [2.0, 1.0, 1.0], [4.0, 3.0, 7.0]], [[5.0, 8.0, 4.0], [8.0, 4.0, 4.0], [2.0, 4.0, 1.0]]], [[[5.0, 4.0, 1.0], [3.0, 
7.0, 4.0], [7.0, 7.0, 2.0]], [[1.0, 3.0, 7.0], [7.0, 3.0, 6.0], [4.0, 4.0, 7.0]], [[4.0, 7.0, 4.0], [7.0, 7.0, 7.0], [8.0, 7.0, 
4.0]]], [[[5.0, 3.0, 2.0], [3.0, 2.0, 2.0], [7.0, 5.0, 4.0]], [[2.0, 8.0, 3.0], [8.0, 6.0, 4.0], [1.0, 5.0, 8.0]], [[2.0, 5.0, 
3.0], [1.0, 5.0, 7.0], [3.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 1.0], [4.0, 6.0, 4.0], [6.0, 7.0, 2.0]], 
[[3.0, 3.0, 3.0], [8.0, 4.0, 8.0], [1.0, 6.0, 8.0]], [[6.0, 3.0, 7.0], [6.0, 3.0, 5.0], [7.0, 4.0, 2.0]]], [[[6.0, 6.0, 2.0], 
[2.0, 1.0, 2.0], [2.0, 8.0, 3.0]], [[5.0, 3.0, 6.0], [2.0, 1.0, 1.0], [4.0, 3.0, 7.0]], [[6.0, 8.0, 7.0], [4.0, 6.0, 8.0], 
[2.0, 4.0, 1.0]]], [[[5.0, 4.0, 1.0], [3.0, 7.0, 4.0], [7.0, 7.0, 2.0]], [[1.0, 3.0, 7.0], [7.0, 3.0, 6.0], [4.0, 4.0, 7.0]], 
[[4.0, 5.0, 4.0], [8.0, 4.0, 8.0], [8.0, 7.0, 4.0]]], [[[5.0, 3.0, 2.0], [3.0, 2.0, 2.0], [7.0, 5.0, 4.0]], [[2.0, 8.0, 3.0], 
[8.0, 6.0, 4.0], [1.0, 5.0, 8.0]], [[2.0, 5.0, 3.0], [1.0, 5.0, 7.0], [3.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CRRN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 3.0], [1.0, 3.0, 3.0], [4.0, 7.0, 4.0]], [[2.0, 3.0, 8.0], [2.0, 6.0, 6.0], [4.0, 3.0, 
8.0]], [[6.0, 7.0, 8.0], [7.0, 8.0, 1.0], [8.0, 8.0, 1.0]]], [[[3.0, 1.0, 2.0], [4.0, 8.0, 7.0], [6.0, 1.0, 7.0]], [[6.0, 3.0, 
6.0], [1.0, 6.0, 5.0], [8.0, 5.0, 7.0]], [[6.0, 3.0, 2.0], [4.0, 1.0, 4.0], [6.0, 4.0, 8.0]]], [[[4.0, 1.0, 8.0], [4.0, 5.0, 
7.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 3.0], [3.0, 7.0, 2.0], [8.0, 5.0, 7.0]], [[4.0, 4.0, 2.0], [3.0, 7.0, 2.0], [1.0, 7.0, 
6.0]]], [[[1.0, 3.0, 3.0], [6.0, 6.0, 6.0], [2.0, 7.0, 2.0]], [[4.0, 7.0, 8.0], [6.0, 5.0, 7.0], [8.0, 8.0, 4.0]], [[3.0, 8.0, 
3.0], [8.0, 7.0, 8.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 1.0, 6.0]]], [[[1.0, 3.0, 8.0]]], [[[4.0, 3.0, 3.0]]], [[[1.0, 1.0, 
7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 1.0]]], [[[7.0, 8.0, 2.0]]], [[[6.0, 4.0, 6.0]]], [[[7.0, 3.0, 2.0]]]]))
      arg[:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 3.0], [1.0, 3.0, 3.0], [4.0, 7.0, 4.0]], [[2.0, 3.0, 8.0], [2.0, 1.0, 6.0], [4.0, 
3.0, 8.0]], [[6.0, 7.0, 8.0], [7.0, 8.0, 1.0], [8.0, 8.0, 1.0]]], [[[3.0, 1.0, 2.0], [4.0, 8.0, 7.0], [6.0, 1.0, 7.0]], [[6.0, 
3.0, 6.0], [1.0, 3.0, 8.0], [8.0, 5.0, 7.0]], [[6.0, 3.0, 2.0], [4.0, 1.0, 4.0], [6.0, 4.0, 8.0]]], [[[4.0, 1.0, 8.0], [4.0, 
5.0, 7.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 3.0], [4.0, 3.0, 3.0], [8.0, 5.0, 7.0]], [[4.0, 4.0, 2.0], [3.0, 7.0, 2.0], [1.0, 7.0, 
6.0]]], [[[1.0, 3.0, 3.0], [6.0, 6.0, 6.0], [2.0, 7.0, 2.0]], [[4.0, 7.0, 8.0], [1.0, 1.0, 7.0], [8.0, 8.0, 4.0]], [[3.0, 8.0, 
3.0], [8.0, 7.0, 8.0], [6.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 3.0], [1.0, 3.0, 3.0], [4.0, 7.0, 4.0]], 
[[2.0, 3.0, 8.0], [3.0, 6.0, 1.0], [4.0, 3.0, 8.0]], [[6.0, 7.0, 8.0], [7.0, 8.0, 1.0], [8.0, 8.0, 1.0]]], [[[3.0, 1.0, 2.0], 
[4.0, 8.0, 7.0], [6.0, 1.0, 7.0]], [[6.0, 3.0, 6.0], [7.0, 8.0, 2.0], [8.0, 5.0, 7.0]], [[6.0, 3.0, 2.0], [4.0, 1.0, 4.0], 
[6.0, 4.0, 8.0]]], [[[4.0, 1.0, 8.0], [4.0, 5.0, 7.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 3.0], [6.0, 4.0, 6.0], [8.0, 5.0, 7.0]], 
[[4.0, 4.0, 2.0], [3.0, 7.0, 2.0], [1.0, 7.0, 6.0]]], [[[1.0, 3.0, 3.0], [6.0, 6.0, 6.0], [2.0, 7.0, 2.0]], [[4.0, 7.0, 8.0], 
[7.0, 3.0, 2.0], [8.0, 8.0, 4.0]], [[3.0, 8.0, 3.0], [8.0, 7.0, 8.0], [6.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PUPN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 6.0], [4.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[5.0, 7.0, 6.0], [3.0, 6.0, 1.0], [8.0, 1.0, 
1.0]], [[5.0, 7.0, 2.0], [3.0, 3.0, 4.0], [6.0, 7.0, 5.0]]], [[[8.0, 1.0, 8.0], [4.0, 2.0, 5.0], [4.0, 3.0, 8.0]], [[5.0, 4.0, 
5.0], [2.0, 2.0, 7.0], [5.0, 4.0, 1.0]], [[4.0, 6.0, 2.0], [3.0, 5.0, 7.0], [2.0, 7.0, 5.0]]], [[[6.0, 3.0, 8.0], [7.0, 3.0, 
8.0], [8.0, 5.0, 2.0]], [[3.0, 8.0, 4.0], [2.0, 2.0, 2.0], [1.0, 8.0, 3.0]], [[8.0, 8.0, 5.0], [3.0, 1.0, 6.0], [8.0, 5.0, 
6.0]]], [[[2.0, 2.0, 1.0], [1.0, 2.0, 3.0], [3.0, 7.0, 5.0]], [[1.0, 6.0, 1.0], [8.0, 3.0, 1.0], [8.0, 5.0, 7.0]], [[7.0, 7.0, 
2.0], [6.0, 1.0, 7.0], [3.0, 1.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 7.0, 7.0], [3.0, 1.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 4.0, 6.0], [6.0, 3.0, 6.0]]))
      arg[2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 6.0], [4.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[5.0, 7.0, 6.0], [3.0, 6.0, 1.0], [8.0, 
1.0, 1.0]], [[5.0, 7.0, 2.0], [3.0, 3.0, 4.0], [6.0, 7.0, 5.0]]], [[[8.0, 1.0, 8.0], [4.0, 2.0, 5.0], [4.0, 3.0, 8.0]], [[5.0, 
4.0, 5.0], [2.0, 2.0, 7.0], [5.0, 4.0, 1.0]], [[4.0, 6.0, 2.0], [3.0, 5.0, 7.0], [2.0, 7.0, 5.0]]], [[[6.0, 3.0, 8.0], [2.0, 
7.0, 7.0], [8.0, 5.0, 2.0]], [[3.0, 8.0, 4.0], [3.0, 1.0, 1.0], [1.0, 8.0, 3.0]], [[8.0, 8.0, 5.0], [3.0, 1.0, 6.0], [8.0, 5.0, 
6.0]]], [[[2.0, 2.0, 1.0], [1.0, 2.0, 3.0], [3.0, 7.0, 5.0]], [[1.0, 6.0, 1.0], [8.0, 3.0, 1.0], [8.0, 5.0, 7.0]], [[7.0, 7.0, 
2.0], [6.0, 1.0, 7.0], [3.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 6.0], [4.0, 7.0, 1.0], [3.0, 3.0, 3.0]], 
[[5.0, 7.0, 6.0], [3.0, 6.0, 1.0], [8.0, 1.0, 1.0]], [[5.0, 7.0, 2.0], [3.0, 3.0, 4.0], [6.0, 7.0, 5.0]]], [[[8.0, 1.0, 8.0], 
[4.0, 2.0, 5.0], [4.0, 3.0, 8.0]], [[5.0, 4.0, 5.0], [2.0, 2.0, 7.0], [5.0, 4.0, 1.0]], [[4.0, 6.0, 2.0], [3.0, 5.0, 7.0], 
[2.0, 7.0, 5.0]]], [[[6.0, 3.0, 8.0], [6.0, 4.0, 6.0], [8.0, 5.0, 2.0]], [[3.0, 8.0, 4.0], [6.0, 3.0, 6.0], [1.0, 8.0, 3.0]], 
[[8.0, 8.0, 5.0], [3.0, 1.0, 6.0], [8.0, 5.0, 6.0]]], [[[2.0, 2.0, 1.0], [1.0, 2.0, 3.0], [3.0, 7.0, 5.0]], [[1.0, 6.0, 1.0], 
[8.0, 3.0, 1.0], [8.0, 5.0, 7.0]], [[7.0, 7.0, 2.0], [6.0, 1.0, 7.0], [3.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UURR(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 1.0], [6.0, 7.0, 6.0], [4.0, 1.0, 5.0]], [[3.0, 2.0, 7.0], [2.0, 4.0, 8.0], [3.0, 2.0, 
2.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 4.0], [6.0, 1.0, 6.0]]], [[[8.0, 6.0, 4.0], [6.0, 3.0, 1.0], [1.0, 5.0, 6.0]], [[2.0, 6.0, 
4.0], [3.0, 3.0, 1.0], [3.0, 5.0, 2.0]], [[2.0, 3.0, 7.0], [5.0, 2.0, 2.0], [7.0, 1.0, 6.0]]], [[[7.0, 6.0, 5.0], [7.0, 1.0, 
2.0], [4.0, 7.0, 6.0]], [[5.0, 8.0, 8.0], [2.0, 3.0, 7.0], [1.0, 4.0, 8.0]], [[1.0, 2.0, 6.0], [1.0, 1.0, 3.0], [5.0, 1.0, 
7.0]]], [[[2.0, 1.0, 4.0], [1.0, 8.0, 4.0], [6.0, 8.0, 6.0]], [[5.0, 7.0, 2.0], [3.0, 5.0, 8.0], [5.0, 6.0, 1.0]], [[7.0, 8.0, 
4.0], [8.0, 5.0, 8.0], [3.0, 5.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0]], [[1.0]]], [[[1.0]], [[2.0]]], [[[1.0]], [[7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0]], [[6.0]]], [[[5.0]], [[1.0]]], [[[4.0]], [[1.0]]]]))
      arg[:3,:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 1.0], [6.0, 6.0, 6.0], [4.0, 1.0, 5.0]], [[3.0, 2.0, 7.0], [2.0, 1.0, 8.0], [3.0, 
2.0, 2.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 4.0], [6.0, 1.0, 6.0]]], [[[8.0, 6.0, 4.0], [6.0, 1.0, 1.0], [1.0, 5.0, 6.0]], [[2.0, 
6.0, 4.0], [3.0, 2.0, 1.0], [3.0, 5.0, 2.0]], [[2.0, 3.0, 7.0], [5.0, 2.0, 2.0], [7.0, 1.0, 6.0]]], [[[7.0, 6.0, 5.0], [7.0, 
1.0, 2.0], [4.0, 7.0, 6.0]], [[5.0, 8.0, 8.0], [2.0, 7.0, 7.0], [1.0, 4.0, 8.0]], [[1.0, 2.0, 6.0], [1.0, 1.0, 3.0], [5.0, 1.0, 
7.0]]], [[[2.0, 1.0, 4.0], [1.0, 8.0, 4.0], [6.0, 8.0, 6.0]], [[5.0, 7.0, 2.0], [3.0, 5.0, 8.0], [5.0, 6.0, 1.0]], [[7.0, 8.0, 
4.0], [8.0, 5.0, 8.0], [3.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 1.0], [6.0, 6.0, 6.0], [4.0, 1.0, 5.0]], 
[[3.0, 2.0, 7.0], [2.0, 6.0, 8.0], [3.0, 2.0, 2.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 4.0], [6.0, 1.0, 6.0]]], [[[8.0, 6.0, 4.0], 
[6.0, 5.0, 1.0], [1.0, 5.0, 6.0]], [[2.0, 6.0, 4.0], [3.0, 1.0, 1.0], [3.0, 5.0, 2.0]], [[2.0, 3.0, 7.0], [5.0, 2.0, 2.0], 
[7.0, 1.0, 6.0]]], [[[7.0, 6.0, 5.0], [7.0, 4.0, 2.0], [4.0, 7.0, 6.0]], [[5.0, 8.0, 8.0], [2.0, 1.0, 7.0], [1.0, 4.0, 8.0]], 
[[1.0, 2.0, 6.0], [1.0, 1.0, 3.0], [5.0, 1.0, 7.0]]], [[[2.0, 1.0, 4.0], [1.0, 8.0, 4.0], [6.0, 8.0, 6.0]], [[5.0, 7.0, 2.0], 
[3.0, 5.0, 8.0], [5.0, 6.0, 1.0]], [[7.0, 8.0, 4.0], [8.0, 5.0, 8.0], [3.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLUN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 7.0], [4.0, 3.0, 5.0], [4.0, 4.0, 7.0]], [[5.0, 7.0, 7.0], [3.0, 3.0, 4.0], [7.0, 1.0, 
5.0]], [[3.0, 5.0, 3.0], [1.0, 5.0, 2.0], [2.0, 4.0, 4.0]]], [[[8.0, 1.0, 5.0], [2.0, 7.0, 1.0], [6.0, 7.0, 2.0]], [[8.0, 7.0, 
2.0], [1.0, 8.0, 7.0], [2.0, 5.0, 4.0]], [[7.0, 7.0, 4.0], [3.0, 1.0, 8.0], [3.0, 5.0, 3.0]]], [[[5.0, 6.0, 2.0], [1.0, 7.0, 
5.0], [4.0, 5.0, 4.0]], [[2.0, 6.0, 1.0], [7.0, 8.0, 7.0], [3.0, 2.0, 6.0]], [[6.0, 4.0, 2.0], [5.0, 7.0, 3.0], [4.0, 6.0, 
7.0]]], [[[6.0, 2.0, 3.0], [6.0, 7.0, 1.0], [2.0, 1.0, 8.0]], [[5.0, 3.0, 6.0], [1.0, 6.0, 4.0], [3.0, 5.0, 2.0]], [[6.0, 1.0, 
5.0], [1.0, 4.0, 3.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 4.0, 8.0], [6.0, 7.0, 5.0]]], [[[5.0, 8.0, 4.0], [1.0, 4.0, 6.0]]], [[[2.0, 7.0, 8.0], 
[1.0, 8.0, 2.0]]], [[[2.0, 3.0, 3.0], [5.0, 7.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 1.0], [7.0, 1.0, 5.0]]], [[[5.0, 8.0, 8.0], [2.0, 3.0, 5.0]]], [[[6.0, 
7.0, 6.0], [3.0, 7.0, 2.0]]], [[[6.0, 8.0, 6.0], [2.0, 1.0, 3.0]]]]))
      arg[:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 7.0], [4.0, 3.0, 5.0], [4.0, 4.0, 7.0]], [[5.0, 7.0, 7.0], [3.0, 3.0, 4.0], [7.0, 
1.0, 5.0]], [[3.0, 4.0, 8.0], [6.0, 7.0, 5.0], [2.0, 4.0, 4.0]]], [[[8.0, 1.0, 5.0], [2.0, 7.0, 1.0], [6.0, 7.0, 2.0]], [[8.0, 
7.0, 2.0], [1.0, 8.0, 7.0], [2.0, 5.0, 4.0]], [[5.0, 8.0, 4.0], [1.0, 4.0, 6.0], [3.0, 5.0, 3.0]]], [[[5.0, 6.0, 2.0], [1.0, 
7.0, 5.0], [4.0, 5.0, 4.0]], [[2.0, 6.0, 1.0], [7.0, 8.0, 7.0], [3.0, 2.0, 6.0]], [[2.0, 7.0, 8.0], [1.0, 8.0, 2.0], [4.0, 6.0, 
7.0]]], [[[6.0, 2.0, 3.0], [6.0, 7.0, 1.0], [2.0, 1.0, 8.0]], [[5.0, 3.0, 6.0], [1.0, 6.0, 4.0], [3.0, 5.0, 2.0]], [[2.0, 3.0, 
3.0], [5.0, 7.0, 4.0], [3.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 7.0], [4.0, 3.0, 5.0], [4.0, 4.0, 7.0]], 
[[5.0, 7.0, 7.0], [3.0, 3.0, 4.0], [7.0, 1.0, 5.0]], [[5.0, 5.0, 1.0], [7.0, 1.0, 5.0], [2.0, 4.0, 4.0]]], [[[8.0, 1.0, 5.0], 
[2.0, 7.0, 1.0], [6.0, 7.0, 2.0]], [[8.0, 7.0, 2.0], [1.0, 8.0, 7.0], [2.0, 5.0, 4.0]], [[5.0, 8.0, 8.0], [2.0, 3.0, 5.0], 
[3.0, 5.0, 3.0]]], [[[5.0, 6.0, 2.0], [1.0, 7.0, 5.0], [4.0, 5.0, 4.0]], [[2.0, 6.0, 1.0], [7.0, 8.0, 7.0], [3.0, 2.0, 6.0]], 
[[6.0, 7.0, 6.0], [3.0, 7.0, 2.0], [4.0, 6.0, 7.0]]], [[[6.0, 2.0, 3.0], [6.0, 7.0, 1.0], [2.0, 1.0, 8.0]], [[5.0, 3.0, 6.0], 
[1.0, 6.0, 4.0], [3.0, 5.0, 2.0]], [[6.0, 8.0, 6.0], [2.0, 1.0, 3.0], [3.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RUCR(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 8.0], [1.0, 4.0, 6.0], [7.0, 2.0, 8.0]], [[8.0, 2.0, 6.0], [7.0, 7.0, 1.0], [6.0, 6.0, 
6.0]], [[4.0, 4.0, 7.0], [1.0, 7.0, 6.0], [4.0, 1.0, 3.0]]], [[[1.0, 2.0, 7.0], [2.0, 4.0, 6.0], [4.0, 1.0, 8.0]], [[2.0, 3.0, 
7.0], [5.0, 8.0, 5.0], [2.0, 5.0, 6.0]], [[7.0, 6.0, 7.0], [4.0, 5.0, 7.0], [1.0, 4.0, 7.0]]], [[[6.0, 6.0, 4.0], [7.0, 8.0, 
1.0], [2.0, 4.0, 8.0]], [[1.0, 6.0, 3.0], [3.0, 4.0, 3.0], [2.0, 2.0, 8.0]], [[1.0, 2.0, 4.0], [1.0, 3.0, 7.0], [5.0, 5.0, 
7.0]]], [[[2.0, 3.0, 6.0], [2.0, 3.0, 7.0], [4.0, 4.0, 7.0]], [[5.0, 1.0, 3.0], [4.0, 2.0, 3.0], [1.0, 8.0, 7.0]], [[7.0, 8.0, 
2.0], [2.0, 2.0, 6.0], [5.0, 8.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0], [7.0], [7.0]], [[8.0], [8.0], [3.0]]], [[[5.0], [7.0], [7.0]], [[3.0], [2.0], 
[2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0], [3.0], [6.0]], [[5.0], [3.0], [6.0]]], [[[8.0], [5.0], [5.0]], [[6.0], [4.0], 
[2.0]]]]))
      arg[1:3,:2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 8.0], [1.0, 4.0, 6.0], [7.0, 2.0, 8.0]], [[8.0, 2.0, 6.0], [7.0, 7.0, 1.0], [6.0, 
6.0, 6.0]], [[4.0, 4.0, 7.0], [1.0, 7.0, 6.0], [4.0, 1.0, 3.0]]], [[[1.0, 3.0, 7.0], [2.0, 7.0, 6.0], [4.0, 7.0, 8.0]], [[2.0, 
8.0, 7.0], [5.0, 8.0, 5.0], [2.0, 3.0, 6.0]], [[7.0, 6.0, 7.0], [4.0, 5.0, 7.0], [1.0, 4.0, 7.0]]], [[[6.0, 5.0, 4.0], [7.0, 
7.0, 1.0], [2.0, 7.0, 8.0]], [[1.0, 3.0, 3.0], [3.0, 2.0, 3.0], [2.0, 2.0, 8.0]], [[1.0, 2.0, 4.0], [1.0, 3.0, 7.0], [5.0, 5.0, 
7.0]]], [[[2.0, 3.0, 6.0], [2.0, 3.0, 7.0], [4.0, 4.0, 7.0]], [[5.0, 1.0, 3.0], [4.0, 2.0, 3.0], [1.0, 8.0, 7.0]], [[7.0, 8.0, 
2.0], [2.0, 2.0, 6.0], [5.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 8.0], [1.0, 4.0, 6.0], [7.0, 2.0, 8.0]], 
[[8.0, 2.0, 6.0], [7.0, 7.0, 1.0], [6.0, 6.0, 6.0]], [[4.0, 4.0, 7.0], [1.0, 7.0, 6.0], [4.0, 1.0, 3.0]]], [[[1.0, 7.0, 7.0], 
[2.0, 3.0, 6.0], [4.0, 6.0, 8.0]], [[2.0, 5.0, 7.0], [5.0, 3.0, 5.0], [2.0, 6.0, 6.0]], [[7.0, 6.0, 7.0], [4.0, 5.0, 7.0], 
[1.0, 4.0, 7.0]]], [[[6.0, 8.0, 4.0], [7.0, 5.0, 1.0], [2.0, 5.0, 8.0]], [[1.0, 6.0, 3.0], [3.0, 4.0, 3.0], [2.0, 2.0, 8.0]], 
[[1.0, 2.0, 4.0], [1.0, 3.0, 7.0], [5.0, 5.0, 7.0]]], [[[2.0, 3.0, 6.0], [2.0, 3.0, 7.0], [4.0, 4.0, 7.0]], [[5.0, 1.0, 3.0], 
[4.0, 2.0, 3.0], [1.0, 8.0, 7.0]], [[7.0, 8.0, 2.0], [2.0, 2.0, 6.0], [5.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CURU(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 1.0], [1.0, 8.0, 1.0], [4.0, 8.0, 8.0]], [[3.0, 8.0, 6.0], [6.0, 8.0, 3.0], [5.0, 7.0, 
5.0]], [[3.0, 2.0, 8.0], [1.0, 3.0, 6.0], [6.0, 3.0, 1.0]]], [[[3.0, 5.0, 1.0], [5.0, 4.0, 2.0], [7.0, 3.0, 6.0]], [[3.0, 4.0, 
7.0], [2.0, 7.0, 2.0], [3.0, 7.0, 1.0]], [[8.0, 8.0, 5.0], [3.0, 6.0, 8.0], [2.0, 6.0, 7.0]]], [[[8.0, 6.0, 5.0], [5.0, 3.0, 
5.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 2.0], [2.0, 3.0, 3.0], [1.0, 4.0, 8.0]], [[7.0, 5.0, 5.0], [7.0, 6.0, 1.0], [3.0, 1.0, 
5.0]]], [[[4.0, 4.0, 1.0], [7.0, 5.0, 7.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 6.0], [3.0, 4.0, 6.0], [2.0, 5.0, 1.0]], [[3.0, 6.0, 
4.0], [3.0, 4.0, 1.0], [5.0, 3.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 1.0]], [[2.0, 7.0]]], [[[7.0, 5.0]], [[4.0, 8.0]]], [[[3.0, 2.0]], [[5.0, 2.0]]], [[[5.0, 
3.0]], [[2.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 6.0]], [[1.0, 8.0]]], [[[1.0, 8.0]], [[4.0, 6.0]]], [[[5.0, 8.0]], [[2.0, 
2.0]]], [[[7.0, 2.0]], [[1.0, 2.0]]]]))
      arg[:,:2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 1.0], [7.0, 1.0, 1.0], [4.0, 8.0, 8.0]], [[3.0, 8.0, 6.0], [2.0, 7.0, 3.0], [5.0, 
7.0, 5.0]], [[3.0, 2.0, 8.0], [1.0, 3.0, 6.0], [6.0, 3.0, 1.0]]], [[[3.0, 5.0, 1.0], [7.0, 5.0, 2.0], [7.0, 3.0, 6.0]], [[3.0, 
4.0, 7.0], [4.0, 8.0, 2.0], [3.0, 7.0, 1.0]], [[8.0, 8.0, 5.0], [3.0, 6.0, 8.0], [2.0, 6.0, 7.0]]], [[[8.0, 6.0, 5.0], [3.0, 
2.0, 5.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 2.0], [5.0, 2.0, 3.0], [1.0, 4.0, 8.0]], [[7.0, 5.0, 5.0], [7.0, 6.0, 1.0], [3.0, 1.0, 
5.0]]], [[[4.0, 4.0, 1.0], [5.0, 3.0, 7.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 6.0], [2.0, 7.0, 6.0], [2.0, 5.0, 1.0]], [[3.0, 6.0, 
4.0], [3.0, 4.0, 1.0], [5.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 1.0], [4.0, 6.0, 1.0], [4.0, 8.0, 8.0]], 
[[3.0, 8.0, 6.0], [1.0, 8.0, 3.0], [5.0, 7.0, 5.0]], [[3.0, 2.0, 8.0], [1.0, 3.0, 6.0], [6.0, 3.0, 1.0]]], [[[3.0, 5.0, 1.0], 
[1.0, 8.0, 2.0], [7.0, 3.0, 6.0]], [[3.0, 4.0, 7.0], [4.0, 6.0, 2.0], [3.0, 7.0, 1.0]], [[8.0, 8.0, 5.0], [3.0, 6.0, 8.0], 
[2.0, 6.0, 7.0]]], [[[8.0, 6.0, 5.0], [5.0, 8.0, 5.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 2.0], [2.0, 2.0, 3.0], [1.0, 4.0, 8.0]], 
[[7.0, 5.0, 5.0], [7.0, 6.0, 1.0], [3.0, 1.0, 5.0]]], [[[4.0, 4.0, 1.0], [7.0, 2.0, 7.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 6.0], 
[1.0, 2.0, 6.0], [2.0, 5.0, 1.0]], [[3.0, 6.0, 4.0], [3.0, 4.0, 1.0], [5.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LRPN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 6.0], [6.0, 3.0, 7.0], [5.0, 8.0, 6.0]], [[2.0, 1.0, 1.0], [4.0, 4.0, 6.0], [6.0, 4.0, 
7.0]], [[3.0, 8.0, 1.0], [7.0, 5.0, 8.0], [4.0, 3.0, 8.0]]], [[[8.0, 7.0, 3.0], [1.0, 8.0, 5.0], [5.0, 2.0, 1.0]], [[8.0, 6.0, 
1.0], [2.0, 5.0, 3.0], [2.0, 3.0, 4.0]], [[1.0, 6.0, 6.0], [4.0, 2.0, 7.0], [3.0, 5.0, 8.0]]], [[[8.0, 2.0, 1.0], [2.0, 6.0, 
6.0], [7.0, 5.0, 4.0]], [[8.0, 4.0, 3.0], [1.0, 8.0, 6.0], [2.0, 6.0, 4.0]], [[4.0, 2.0, 3.0], [8.0, 5.0, 2.0], [5.0, 8.0, 
6.0]]], [[[8.0, 6.0, 6.0], [1.0, 6.0, 4.0], [5.0, 7.0, 1.0]], [[6.0, 4.0, 1.0], [6.0, 5.0, 1.0], [7.0, 6.0, 4.0]], [[3.0, 3.0, 
7.0], [8.0, 1.0, 7.0], [1.0, 3.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 3.0]], [[5.0, 6.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 1.0, 4.0]], [[3.0, 4.0, 6.0]]]))
      arg[2:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 6.0], [6.0, 3.0, 7.0], [5.0, 8.0, 6.0]], [[2.0, 1.0, 1.0], [4.0, 4.0, 6.0], [6.0, 
4.0, 7.0]], [[3.0, 8.0, 1.0], [7.0, 5.0, 8.0], [4.0, 3.0, 8.0]]], [[[8.0, 7.0, 3.0], [1.0, 8.0, 5.0], [5.0, 2.0, 1.0]], [[8.0, 
6.0, 1.0], [2.0, 5.0, 3.0], [2.0, 3.0, 4.0]], [[1.0, 6.0, 6.0], [4.0, 2.0, 7.0], [3.0, 5.0, 8.0]]], [[[8.0, 2.0, 1.0], [2.0, 
6.0, 6.0], [7.0, 5.0, 4.0]], [[8.0, 4.0, 3.0], [4.0, 7.0, 3.0], [2.0, 6.0, 4.0]], [[4.0, 2.0, 3.0], [8.0, 5.0, 2.0], [5.0, 8.0, 
6.0]]], [[[8.0, 6.0, 6.0], [1.0, 6.0, 4.0], [5.0, 7.0, 1.0]], [[6.0, 4.0, 1.0], [5.0, 6.0, 8.0], [7.0, 6.0, 4.0]], [[3.0, 3.0, 
7.0], [8.0, 1.0, 7.0], [1.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 6.0], [6.0, 3.0, 7.0], [5.0, 8.0, 6.0]], 
[[2.0, 1.0, 1.0], [4.0, 4.0, 6.0], [6.0, 4.0, 7.0]], [[3.0, 8.0, 1.0], [7.0, 5.0, 8.0], [4.0, 3.0, 8.0]]], [[[8.0, 7.0, 3.0], 
[1.0, 8.0, 5.0], [5.0, 2.0, 1.0]], [[8.0, 6.0, 1.0], [2.0, 5.0, 3.0], [2.0, 3.0, 4.0]], [[1.0, 6.0, 6.0], [4.0, 2.0, 7.0], 
[3.0, 5.0, 8.0]]], [[[8.0, 2.0, 1.0], [2.0, 6.0, 6.0], [7.0, 5.0, 4.0]], [[8.0, 4.0, 3.0], [3.0, 1.0, 4.0], [2.0, 6.0, 4.0]], 
[[4.0, 2.0, 3.0], [8.0, 5.0, 2.0], [5.0, 8.0, 6.0]]], [[[8.0, 6.0, 6.0], [1.0, 6.0, 4.0], [5.0, 7.0, 1.0]], [[6.0, 4.0, 1.0], 
[3.0, 4.0, 6.0], [7.0, 6.0, 4.0]], [[3.0, 3.0, 7.0], [8.0, 1.0, 7.0], [1.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PCRN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 7.0], [8.0, 5.0, 2.0], [1.0, 3.0, 8.0]], [[6.0, 6.0, 7.0], [1.0, 4.0, 4.0], [7.0, 1.0, 
8.0]], [[4.0, 3.0, 8.0], [6.0, 2.0, 1.0], [2.0, 2.0, 7.0]]], [[[1.0, 1.0, 1.0], [1.0, 6.0, 5.0], [2.0, 2.0, 6.0]], [[6.0, 3.0, 
4.0], [4.0, 1.0, 2.0], [3.0, 5.0, 5.0]], [[2.0, 2.0, 6.0], [2.0, 5.0, 5.0], [6.0, 2.0, 1.0]]], [[[1.0, 5.0, 5.0], [2.0, 1.0, 
3.0], [8.0, 3.0, 1.0]], [[6.0, 4.0, 4.0], [5.0, 5.0, 6.0], [4.0, 4.0, 7.0]], [[7.0, 7.0, 5.0], [4.0, 6.0, 4.0], [1.0, 1.0, 
6.0]]], [[[4.0, 1.0, 7.0], [8.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[3.0, 6.0, 1.0], [6.0, 1.0, 7.0], [1.0, 1.0, 4.0]], [[5.0, 6.0, 
3.0], [2.0, 2.0, 2.0], [1.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 3.0, 2.0]], [[5.0, 2.0, 4.0]], [[1.0, 8.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 3.0, 6.0]], [[7.0, 2.0, 7.0]], [[5.0, 6.0, 2.0]]]))
      arg[2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 7.0], [8.0, 5.0, 2.0], [1.0, 3.0, 8.0]], [[6.0, 6.0, 7.0], [1.0, 4.0, 4.0], [7.0, 
1.0, 8.0]], [[4.0, 3.0, 8.0], [6.0, 2.0, 1.0], [2.0, 2.0, 7.0]]], [[[1.0, 1.0, 1.0], [1.0, 6.0, 5.0], [2.0, 2.0, 6.0]], [[6.0, 
3.0, 4.0], [4.0, 1.0, 2.0], [3.0, 5.0, 5.0]], [[2.0, 2.0, 6.0], [2.0, 5.0, 5.0], [6.0, 2.0, 1.0]]], [[[1.0, 5.0, 5.0], [1.0, 
3.0, 2.0], [8.0, 3.0, 1.0]], [[6.0, 4.0, 4.0], [5.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[7.0, 7.0, 5.0], [1.0, 8.0, 4.0], [1.0, 1.0, 
6.0]]], [[[4.0, 1.0, 7.0], [8.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[3.0, 6.0, 1.0], [6.0, 1.0, 7.0], [1.0, 1.0, 4.0]], [[5.0, 6.0, 
3.0], [2.0, 2.0, 2.0], [1.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 7.0], [8.0, 5.0, 2.0], [1.0, 3.0, 8.0]], 
[[6.0, 6.0, 7.0], [1.0, 4.0, 4.0], [7.0, 1.0, 8.0]], [[4.0, 3.0, 8.0], [6.0, 2.0, 1.0], [2.0, 2.0, 7.0]]], [[[1.0, 1.0, 1.0], 
[1.0, 6.0, 5.0], [2.0, 2.0, 6.0]], [[6.0, 3.0, 4.0], [4.0, 1.0, 2.0], [3.0, 5.0, 5.0]], [[2.0, 2.0, 6.0], [2.0, 5.0, 5.0], 
[6.0, 2.0, 1.0]]], [[[1.0, 5.0, 5.0], [1.0, 3.0, 6.0], [8.0, 3.0, 1.0]], [[6.0, 4.0, 4.0], [7.0, 2.0, 7.0], [4.0, 4.0, 7.0]], 
[[7.0, 7.0, 5.0], [5.0, 6.0, 2.0], [1.0, 1.0, 6.0]]], [[[4.0, 1.0, 7.0], [8.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[3.0, 6.0, 1.0], 
[6.0, 1.0, 7.0], [1.0, 1.0, 4.0]], [[5.0, 6.0, 3.0], [2.0, 2.0, 2.0], [1.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UPPP(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 4.0], [2.0, 8.0, 2.0], [1.0, 4.0, 8.0]], [[6.0, 2.0, 1.0], [5.0, 3.0, 4.0], [6.0, 6.0, 
2.0]], [[4.0, 8.0, 8.0], [1.0, 5.0, 6.0], [7.0, 8.0, 6.0]]], [[[1.0, 1.0, 6.0], [7.0, 7.0, 5.0], [3.0, 2.0, 1.0]], [[6.0, 4.0, 
6.0], [8.0, 6.0, 8.0], [5.0, 2.0, 6.0]], [[6.0, 7.0, 1.0], [6.0, 7.0, 7.0], [7.0, 5.0, 8.0]]], [[[8.0, 1.0, 3.0], [1.0, 4.0, 
8.0], [2.0, 8.0, 5.0]], [[4.0, 4.0, 7.0], [6.0, 3.0, 2.0], [3.0, 6.0, 8.0]], [[7.0, 6.0, 6.0], [8.0, 6.0, 6.0], [6.0, 6.0, 
6.0]]], [[[3.0, 6.0, 4.0], [4.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[5.0, 5.0, 8.0], [2.0, 5.0, 5.0], [5.0, 8.0, 3.0]], [[1.0, 6.0, 
8.0], [6.0, 6.0, 1.0], [5.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([6.0, 4.0, 5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([4.0, 3.0, 1.0]))
      arg[:3,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 4.0], [2.0, 8.0, 2.0], [1.0, 4.0, 8.0]], [[6.0, 2.0, 1.0], [5.0, 6.0, 4.0], [6.0, 
6.0, 2.0]], [[4.0, 8.0, 8.0], [1.0, 5.0, 6.0], [7.0, 8.0, 6.0]]], [[[1.0, 1.0, 6.0], [7.0, 7.0, 5.0], [3.0, 2.0, 1.0]], [[6.0, 
4.0, 6.0], [8.0, 4.0, 8.0], [5.0, 2.0, 6.0]], [[6.0, 7.0, 1.0], [6.0, 7.0, 7.0], [7.0, 5.0, 8.0]]], [[[8.0, 1.0, 3.0], [1.0, 
4.0, 8.0], [2.0, 8.0, 5.0]], [[4.0, 4.0, 7.0], [6.0, 5.0, 2.0], [3.0, 6.0, 8.0]], [[7.0, 6.0, 6.0], [8.0, 6.0, 6.0], [6.0, 6.0, 
6.0]]], [[[3.0, 6.0, 4.0], [4.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[5.0, 5.0, 8.0], [2.0, 5.0, 5.0], [5.0, 8.0, 3.0]], [[1.0, 6.0, 
8.0], [6.0, 6.0, 1.0], [5.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 4.0], [2.0, 8.0, 2.0], [1.0, 4.0, 8.0]], 
[[6.0, 2.0, 1.0], [5.0, 4.0, 4.0], [6.0, 6.0, 2.0]], [[4.0, 8.0, 8.0], [1.0, 5.0, 6.0], [7.0, 8.0, 6.0]]], [[[1.0, 1.0, 6.0], 
[7.0, 7.0, 5.0], [3.0, 2.0, 1.0]], [[6.0, 4.0, 6.0], [8.0, 3.0, 8.0], [5.0, 2.0, 6.0]], [[6.0, 7.0, 1.0], [6.0, 7.0, 7.0], 
[7.0, 5.0, 8.0]]], [[[8.0, 1.0, 3.0], [1.0, 4.0, 8.0], [2.0, 8.0, 5.0]], [[4.0, 4.0, 7.0], [6.0, 1.0, 2.0], [3.0, 6.0, 8.0]], 
[[7.0, 6.0, 6.0], [8.0, 6.0, 6.0], [6.0, 6.0, 6.0]]], [[[3.0, 6.0, 4.0], [4.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[5.0, 5.0, 8.0], 
[2.0, 5.0, 5.0], [5.0, 8.0, 3.0]], [[1.0, 6.0, 8.0], [6.0, 6.0, 1.0], [5.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLPL(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 3.0], [8.0, 6.0, 8.0], [5.0, 7.0, 6.0]], [[7.0, 6.0, 1.0], [7.0, 1.0, 2.0], [2.0, 4.0, 
1.0]], [[4.0, 3.0, 7.0], [5.0, 6.0, 5.0], [5.0, 4.0, 1.0]]], [[[7.0, 4.0, 1.0], [3.0, 8.0, 5.0], [3.0, 4.0, 2.0]], [[1.0, 4.0, 
1.0], [7.0, 4.0, 1.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 4.0], [8.0, 5.0, 7.0], [7.0, 7.0, 5.0]]], [[[7.0, 6.0, 5.0], [8.0, 1.0, 
5.0], [6.0, 2.0, 7.0]], [[4.0, 7.0, 4.0], [8.0, 3.0, 4.0], [4.0, 6.0, 6.0]], [[6.0, 7.0, 7.0], [4.0, 1.0, 2.0], [6.0, 3.0, 
6.0]]], [[[3.0, 3.0, 6.0], [2.0, 8.0, 2.0], [5.0, 1.0, 7.0]], [[8.0, 4.0, 5.0], [3.0, 1.0, 2.0], [3.0, 4.0, 6.0]], [[8.0, 4.0, 
8.0], [1.0, 4.0, 2.0], [1.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0]], [[6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0]], [[3.0]]]))
      arg[1:3,2:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [8.0, 6.0, 8.0], [5.0, 7.0, 6.0]], [[7.0, 6.0, 1.0], [7.0, 1.0, 2.0], [2.0, 
4.0, 1.0]], [[4.0, 3.0, 7.0], [5.0, 6.0, 5.0], [5.0, 4.0, 1.0]]], [[[7.0, 4.0, 1.0], [3.0, 8.0, 5.0], [3.0, 4.0, 2.0]], [[1.0, 
4.0, 1.0], [7.0, 4.0, 1.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 4.0], [8.0, 5.0, 5.0], [7.0, 7.0, 5.0]]], [[[7.0, 6.0, 5.0], [8.0, 
1.0, 5.0], [6.0, 2.0, 7.0]], [[4.0, 7.0, 4.0], [8.0, 3.0, 4.0], [4.0, 6.0, 6.0]], [[6.0, 7.0, 7.0], [4.0, 1.0, 6.0], [6.0, 3.0, 
6.0]]], [[[3.0, 3.0, 6.0], [2.0, 8.0, 2.0], [5.0, 1.0, 7.0]], [[8.0, 4.0, 5.0], [3.0, 1.0, 2.0], [3.0, 4.0, 6.0]], [[8.0, 4.0, 
8.0], [1.0, 4.0, 2.0], [1.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 3.0], [8.0, 6.0, 8.0], [5.0, 7.0, 6.0]], 
[[7.0, 6.0, 1.0], [7.0, 1.0, 2.0], [2.0, 4.0, 1.0]], [[4.0, 3.0, 7.0], [5.0, 6.0, 5.0], [5.0, 4.0, 1.0]]], [[[7.0, 4.0, 1.0], 
[3.0, 8.0, 5.0], [3.0, 4.0, 2.0]], [[1.0, 4.0, 1.0], [7.0, 4.0, 1.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 4.0], [8.0, 5.0, 1.0], 
[7.0, 7.0, 5.0]]], [[[7.0, 6.0, 5.0], [8.0, 1.0, 5.0], [6.0, 2.0, 7.0]], [[4.0, 7.0, 4.0], [8.0, 3.0, 4.0], [4.0, 6.0, 6.0]], 
[[6.0, 7.0, 7.0], [4.0, 1.0, 3.0], [6.0, 3.0, 6.0]]], [[[3.0, 3.0, 6.0], [2.0, 8.0, 2.0], [5.0, 1.0, 7.0]], [[8.0, 4.0, 5.0], 
[3.0, 1.0, 2.0], [3.0, 4.0, 6.0]], [[8.0, 4.0, 8.0], [1.0, 4.0, 2.0], [1.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCPP(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 3.0], [1.0, 1.0, 6.0], [1.0, 8.0, 4.0]], [[6.0, 3.0, 5.0], [5.0, 3.0, 3.0], [5.0, 8.0, 
6.0]], [[3.0, 8.0, 8.0], [1.0, 6.0, 1.0], [3.0, 4.0, 8.0]]], [[[6.0, 7.0, 2.0], [8.0, 4.0, 6.0], [4.0, 8.0, 3.0]], [[8.0, 8.0, 
2.0], [4.0, 7.0, 8.0], [1.0, 6.0, 7.0]], [[2.0, 7.0, 2.0], [4.0, 6.0, 4.0], [8.0, 1.0, 7.0]]], [[[2.0, 6.0, 5.0], [1.0, 2.0, 
8.0], [2.0, 8.0, 1.0]], [[4.0, 7.0, 1.0], [6.0, 5.0, 4.0], [8.0, 2.0, 8.0]], [[6.0, 8.0, 3.0], [5.0, 5.0, 5.0], [2.0, 8.0, 
5.0]]], [[[5.0, 1.0, 8.0], [8.0, 7.0, 5.0], [7.0, 2.0, 3.0]], [[6.0, 8.0, 1.0], [7.0, 5.0, 7.0], [3.0, 5.0, 3.0]], [[8.0, 2.0, 
1.0], [3.0, 4.0, 5.0], [2.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 2.0, 3.0], [1.0, 2.0, 8.0], [3.0, 6.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 5.0, 4.0], [7.0, 2.0, 5.0], [2.0, 4.0, 4.0]]))
      arg[:3,:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 3.0], [1.0, 7.0, 6.0], [1.0, 8.0, 4.0]], [[6.0, 3.0, 5.0], [5.0, 2.0, 3.0], [5.0, 
8.0, 6.0]], [[3.0, 8.0, 8.0], [1.0, 3.0, 1.0], [3.0, 4.0, 8.0]]], [[[6.0, 7.0, 2.0], [8.0, 1.0, 6.0], [4.0, 8.0, 3.0]], [[8.0, 
8.0, 2.0], [4.0, 2.0, 8.0], [1.0, 6.0, 7.0]], [[2.0, 7.0, 2.0], [4.0, 8.0, 4.0], [8.0, 1.0, 7.0]]], [[[2.0, 6.0, 5.0], [1.0, 
3.0, 8.0], [2.0, 8.0, 1.0]], [[4.0, 7.0, 1.0], [6.0, 6.0, 4.0], [8.0, 2.0, 8.0]], [[6.0, 8.0, 3.0], [5.0, 8.0, 5.0], [2.0, 8.0, 
5.0]]], [[[5.0, 1.0, 8.0], [8.0, 7.0, 5.0], [7.0, 2.0, 3.0]], [[6.0, 8.0, 1.0], [7.0, 5.0, 7.0], [3.0, 5.0, 3.0]], [[8.0, 2.0, 
1.0], [3.0, 4.0, 5.0], [2.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 3.0], [1.0, 4.0, 6.0], [1.0, 8.0, 4.0]], 
[[6.0, 3.0, 5.0], [5.0, 5.0, 3.0], [5.0, 8.0, 6.0]], [[3.0, 8.0, 8.0], [1.0, 4.0, 1.0], [3.0, 4.0, 8.0]]], [[[6.0, 7.0, 2.0], 
[8.0, 7.0, 6.0], [4.0, 8.0, 3.0]], [[8.0, 8.0, 2.0], [4.0, 2.0, 8.0], [1.0, 6.0, 7.0]], [[2.0, 7.0, 2.0], [4.0, 5.0, 4.0], 
[8.0, 1.0, 7.0]]], [[[2.0, 6.0, 5.0], [1.0, 2.0, 8.0], [2.0, 8.0, 1.0]], [[4.0, 7.0, 1.0], [6.0, 4.0, 4.0], [8.0, 2.0, 8.0]], 
[[6.0, 8.0, 3.0], [5.0, 4.0, 5.0], [2.0, 8.0, 5.0]]], [[[5.0, 1.0, 8.0], [8.0, 7.0, 5.0], [7.0, 2.0, 3.0]], [[6.0, 8.0, 1.0], 
[7.0, 5.0, 7.0], [3.0, 5.0, 3.0]], [[8.0, 2.0, 1.0], [3.0, 4.0, 5.0], [2.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLCN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 8.0], [4.0, 7.0, 4.0], [6.0, 2.0, 7.0]], [[2.0, 5.0, 2.0], [4.0, 2.0, 2.0], [5.0, 5.0, 
8.0]], [[6.0, 2.0, 1.0], [2.0, 6.0, 7.0], [6.0, 1.0, 1.0]]], [[[6.0, 2.0, 3.0], [2.0, 3.0, 1.0], [2.0, 4.0, 4.0]], [[2.0, 1.0, 
6.0], [3.0, 6.0, 6.0], [4.0, 6.0, 3.0]], [[2.0, 6.0, 7.0], [4.0, 7.0, 3.0], [3.0, 5.0, 6.0]]], [[[1.0, 5.0, 1.0], [6.0, 2.0, 
6.0], [3.0, 3.0, 2.0]], [[2.0, 4.0, 5.0], [7.0, 6.0, 4.0], [7.0, 5.0, 3.0]], [[5.0, 1.0, 7.0], [4.0, 5.0, 7.0], [1.0, 1.0, 
2.0]]], [[[3.0, 4.0, 3.0], [4.0, 2.0, 8.0], [1.0, 4.0, 3.0]], [[2.0, 1.0, 1.0], [1.0, 3.0, 2.0], [8.0, 6.0, 1.0]], [[5.0, 2.0, 
8.0], [3.0, 1.0, 4.0], [2.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 5.0, 5.0], [4.0, 7.0, 3.0], [4.0, 4.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 8.0, 8.0], [8.0, 2.0, 6.0], [4.0, 1.0, 7.0]]]))
      arg[2,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 8.0], [4.0, 7.0, 4.0], [6.0, 2.0, 7.0]], [[2.0, 5.0, 2.0], [4.0, 2.0, 2.0], [5.0, 
5.0, 8.0]], [[6.0, 2.0, 1.0], [2.0, 6.0, 7.0], [6.0, 1.0, 1.0]]], [[[6.0, 2.0, 3.0], [2.0, 3.0, 1.0], [2.0, 4.0, 4.0]], [[2.0, 
1.0, 6.0], [3.0, 6.0, 6.0], [4.0, 6.0, 3.0]], [[2.0, 6.0, 7.0], [4.0, 7.0, 3.0], [3.0, 5.0, 6.0]]], [[[1.0, 5.0, 1.0], [6.0, 
2.0, 6.0], [3.0, 3.0, 2.0]], [[2.0, 4.0, 5.0], [7.0, 6.0, 4.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 5.0], [4.0, 7.0, 3.0], [4.0, 4.0, 
4.0]]], [[[3.0, 4.0, 3.0], [4.0, 2.0, 8.0], [1.0, 4.0, 3.0]], [[2.0, 1.0, 1.0], [1.0, 3.0, 2.0], [8.0, 6.0, 1.0]], [[5.0, 2.0, 
8.0], [3.0, 1.0, 4.0], [2.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 8.0], [4.0, 7.0, 4.0], [6.0, 2.0, 7.0]], 
[[2.0, 5.0, 2.0], [4.0, 2.0, 2.0], [5.0, 5.0, 8.0]], [[6.0, 2.0, 1.0], [2.0, 6.0, 7.0], [6.0, 1.0, 1.0]]], [[[6.0, 2.0, 3.0], 
[2.0, 3.0, 1.0], [2.0, 4.0, 4.0]], [[2.0, 1.0, 6.0], [3.0, 6.0, 6.0], [4.0, 6.0, 3.0]], [[2.0, 6.0, 7.0], [4.0, 7.0, 3.0], 
[3.0, 5.0, 6.0]]], [[[1.0, 5.0, 1.0], [6.0, 2.0, 6.0], [3.0, 3.0, 2.0]], [[2.0, 4.0, 5.0], [7.0, 6.0, 4.0], [7.0, 5.0, 3.0]], 
[[7.0, 8.0, 8.0], [8.0, 2.0, 6.0], [4.0, 1.0, 7.0]]], [[[3.0, 4.0, 3.0], [4.0, 2.0, 8.0], [1.0, 4.0, 3.0]], [[2.0, 1.0, 1.0], 
[1.0, 3.0, 2.0], [8.0, 6.0, 1.0]], [[5.0, 2.0, 8.0], [3.0, 1.0, 4.0], [2.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPCU(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 6.0], [2.0, 5.0, 1.0], [6.0, 7.0, 5.0]], [[4.0, 5.0, 3.0], [3.0, 5.0, 1.0], [6.0, 7.0, 
1.0]], [[3.0, 1.0, 8.0], [1.0, 4.0, 2.0], [2.0, 8.0, 2.0]]], [[[1.0, 8.0, 8.0], [1.0, 1.0, 7.0], [2.0, 1.0, 5.0]], [[3.0, 4.0, 
7.0], [2.0, 5.0, 8.0], [2.0, 3.0, 5.0]], [[7.0, 6.0, 5.0], [5.0, 3.0, 3.0], [3.0, 7.0, 1.0]]], [[[3.0, 7.0, 4.0], [7.0, 3.0, 
4.0], [7.0, 7.0, 5.0]], [[8.0, 6.0, 5.0], [1.0, 8.0, 7.0], [5.0, 2.0, 6.0]], [[4.0, 7.0, 8.0], [6.0, 2.0, 1.0], [5.0, 3.0, 
4.0]]], [[[5.0, 4.0, 6.0], [7.0, 6.0, 4.0], [7.0, 3.0, 3.0]], [[3.0, 6.0, 6.0], [6.0, 3.0, 2.0], [1.0, 7.0, 5.0]], [[5.0, 8.0, 
4.0], [7.0, 4.0, 6.0], [1.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 7.0], [1.0, 3.0], [1.0, 8.0]], [[7.0, 5.0], [7.0, 7.0], [7.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 8.0], [3.0, 7.0], [2.0, 4.0]], [[8.0, 5.0], [5.0, 4.0], [1.0, 5.0]]]))
      arg[2:,1,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 6.0], [2.0, 5.0, 1.0], [6.0, 7.0, 5.0]], [[4.0, 5.0, 3.0], [3.0, 5.0, 1.0], [6.0, 
7.0, 1.0]], [[3.0, 1.0, 8.0], [1.0, 4.0, 2.0], [2.0, 8.0, 2.0]]], [[[1.0, 8.0, 8.0], [1.0, 1.0, 7.0], [2.0, 1.0, 5.0]], [[3.0, 
4.0, 7.0], [2.0, 5.0, 8.0], [2.0, 3.0, 5.0]], [[7.0, 6.0, 5.0], [5.0, 3.0, 3.0], [3.0, 7.0, 1.0]]], [[[3.0, 7.0, 4.0], [7.0, 
3.0, 4.0], [7.0, 7.0, 5.0]], [[3.0, 7.0, 5.0], [1.0, 3.0, 7.0], [1.0, 8.0, 6.0]], [[4.0, 7.0, 8.0], [6.0, 2.0, 1.0], [5.0, 3.0, 
4.0]]], [[[5.0, 4.0, 6.0], [7.0, 6.0, 4.0], [7.0, 3.0, 3.0]], [[7.0, 5.0, 6.0], [7.0, 7.0, 2.0], [7.0, 7.0, 5.0]], [[5.0, 8.0, 
4.0], [7.0, 4.0, 6.0], [1.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 6.0], [2.0, 5.0, 1.0], [6.0, 7.0, 5.0]], 
[[4.0, 5.0, 3.0], [3.0, 5.0, 1.0], [6.0, 7.0, 1.0]], [[3.0, 1.0, 8.0], [1.0, 4.0, 2.0], [2.0, 8.0, 2.0]]], [[[1.0, 8.0, 8.0], 
[1.0, 1.0, 7.0], [2.0, 1.0, 5.0]], [[3.0, 4.0, 7.0], [2.0, 5.0, 8.0], [2.0, 3.0, 5.0]], [[7.0, 6.0, 5.0], [5.0, 3.0, 3.0], 
[3.0, 7.0, 1.0]]], [[[3.0, 7.0, 4.0], [7.0, 3.0, 4.0], [7.0, 7.0, 5.0]], [[7.0, 8.0, 5.0], [3.0, 7.0, 7.0], [2.0, 4.0, 6.0]], 
[[4.0, 7.0, 8.0], [6.0, 2.0, 1.0], [5.0, 3.0, 4.0]]], [[[5.0, 4.0, 6.0], [7.0, 6.0, 4.0], [7.0, 3.0, 3.0]], [[8.0, 5.0, 6.0], 
[5.0, 4.0, 2.0], [1.0, 5.0, 5.0]], [[5.0, 8.0, 4.0], [7.0, 4.0, 6.0], [1.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCCP(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 5.0], [1.0, 1.0, 2.0], [7.0, 4.0, 6.0]], [[6.0, 5.0, 7.0], [3.0, 5.0, 8.0], [4.0, 2.0, 
4.0]], [[4.0, 6.0, 1.0], [7.0, 6.0, 3.0], [7.0, 2.0, 4.0]]], [[[6.0, 1.0, 1.0], [7.0, 2.0, 5.0], [4.0, 2.0, 2.0]], [[3.0, 8.0, 
6.0], [7.0, 7.0, 2.0], [3.0, 4.0, 6.0]], [[2.0, 8.0, 1.0], [3.0, 1.0, 1.0], [3.0, 4.0, 5.0]]], [[[5.0, 1.0, 6.0], [5.0, 6.0, 
1.0], [6.0, 2.0, 2.0]], [[6.0, 7.0, 5.0], [2.0, 1.0, 6.0], [7.0, 8.0, 7.0]], [[3.0, 8.0, 4.0], [1.0, 2.0, 8.0], [1.0, 4.0, 
4.0]]], [[[6.0, 7.0, 4.0], [8.0, 2.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 6.0, 3.0], [5.0, 2.0, 7.0], [5.0, 8.0, 8.0]], [[6.0, 6.0, 
7.0], [3.0, 2.0, 4.0], [6.0, 4.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 5.0], [2.0, 7.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 3.0, 2.0], [8.0, 3.0, 3.0], [1.0, 6.0, 
8.0]], [[8.0, 3.0, 7.0], [3.0, 4.0, 3.0], [2.0, 5.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 5.0, 1.0], [1.0, 7.0, 1.0], [1.0, 6.0, 3.0]], [[7.0, 7.0, 3.0], [7.0, 4.0, 5.0], 
[2.0, 7.0, 6.0]], [[2.0, 6.0, 2.0], [8.0, 7.0, 8.0], [8.0, 8.0, 1.0]]]))
      arg[:3,:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 5.0], [1.0, 7.0, 2.0], [7.0, 5.0, 6.0]], [[6.0, 2.0, 7.0], [3.0, 7.0, 8.0], [4.0, 
2.0, 4.0]], [[4.0, 7.0, 1.0], [7.0, 6.0, 3.0], [7.0, 8.0, 4.0]]], [[[6.0, 3.0, 1.0], [7.0, 3.0, 5.0], [4.0, 2.0, 2.0]], [[3.0, 
8.0, 6.0], [7.0, 3.0, 2.0], [3.0, 3.0, 6.0]], [[2.0, 1.0, 1.0], [3.0, 6.0, 1.0], [3.0, 8.0, 5.0]]], [[[5.0, 8.0, 6.0], [5.0, 
3.0, 1.0], [6.0, 7.0, 2.0]], [[6.0, 3.0, 5.0], [2.0, 4.0, 6.0], [7.0, 3.0, 7.0]], [[3.0, 2.0, 4.0], [1.0, 5.0, 8.0], [1.0, 3.0, 
4.0]]], [[[6.0, 7.0, 4.0], [8.0, 2.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 6.0, 3.0], [5.0, 2.0, 7.0], [5.0, 8.0, 8.0]], [[6.0, 6.0, 
7.0], [3.0, 2.0, 4.0], [6.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 5.0], [1.0, 5.0, 2.0], [7.0, 1.0, 6.0]], 
[[6.0, 1.0, 7.0], [3.0, 7.0, 8.0], [4.0, 1.0, 4.0]], [[4.0, 1.0, 1.0], [7.0, 6.0, 3.0], [7.0, 3.0, 4.0]]], [[[6.0, 7.0, 1.0], 
[7.0, 7.0, 5.0], [4.0, 3.0, 2.0]], [[3.0, 7.0, 6.0], [7.0, 4.0, 2.0], [3.0, 5.0, 6.0]], [[2.0, 2.0, 1.0], [3.0, 7.0, 1.0], 
[3.0, 6.0, 5.0]]], [[[5.0, 2.0, 6.0], [5.0, 6.0, 1.0], [6.0, 2.0, 2.0]], [[6.0, 8.0, 5.0], [2.0, 7.0, 6.0], [7.0, 8.0, 7.0]], 
[[3.0, 8.0, 4.0], [1.0, 8.0, 8.0], [1.0, 1.0, 4.0]]], [[[6.0, 7.0, 4.0], [8.0, 2.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 6.0, 3.0], 
[5.0, 2.0, 7.0], [5.0, 8.0, 8.0]], [[6.0, 6.0, 7.0], [3.0, 2.0, 4.0], [6.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLRR(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 8.0], [4.0, 4.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 2.0], [1.0, 8.0, 1.0], [4.0, 6.0, 
2.0]], [[2.0, 2.0, 7.0], [7.0, 7.0, 8.0], [4.0, 7.0, 1.0]]], [[[7.0, 2.0, 3.0], [1.0, 4.0, 1.0], [7.0, 8.0, 2.0]], [[4.0, 3.0, 
1.0], [8.0, 6.0, 3.0], [5.0, 8.0, 1.0]], [[5.0, 1.0, 5.0], [6.0, 1.0, 6.0], [1.0, 3.0, 7.0]]], [[[4.0, 6.0, 5.0], [2.0, 6.0, 
1.0], [1.0, 3.0, 6.0]], [[4.0, 8.0, 7.0], [5.0, 7.0, 4.0], [5.0, 4.0, 7.0]], [[5.0, 2.0, 2.0], [7.0, 1.0, 7.0], [3.0, 5.0, 
5.0]]], [[[3.0, 1.0, 2.0], [7.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[5.0, 2.0, 2.0], [8.0, 1.0, 4.0], [3.0, 2.0, 1.0]], [[3.0, 2.0, 
8.0], [5.0, 8.0, 7.0], [3.0, 3.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0]]], [[[7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0]]], [[[8.0]]]]))
      arg[1:3,2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 8.0], [4.0, 4.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 2.0], [1.0, 8.0, 1.0], [4.0, 
6.0, 2.0]], [[2.0, 2.0, 7.0], [7.0, 7.0, 8.0], [4.0, 7.0, 1.0]]], [[[7.0, 2.0, 3.0], [1.0, 4.0, 1.0], [7.0, 8.0, 2.0]], [[4.0, 
3.0, 1.0], [8.0, 6.0, 3.0], [5.0, 8.0, 1.0]], [[5.0, 1.0, 5.0], [6.0, 7.0, 6.0], [1.0, 3.0, 7.0]]], [[[4.0, 6.0, 5.0], [2.0, 
6.0, 1.0], [1.0, 3.0, 6.0]], [[4.0, 8.0, 7.0], [5.0, 7.0, 4.0], [5.0, 4.0, 7.0]], [[5.0, 2.0, 2.0], [7.0, 7.0, 7.0], [3.0, 5.0, 
5.0]]], [[[3.0, 1.0, 2.0], [7.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[5.0, 2.0, 2.0], [8.0, 1.0, 4.0], [3.0, 2.0, 1.0]], [[3.0, 2.0, 
8.0], [5.0, 8.0, 7.0], [3.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 8.0], [4.0, 4.0, 3.0], [2.0, 5.0, 5.0]], 
[[6.0, 5.0, 2.0], [1.0, 8.0, 1.0], [4.0, 6.0, 2.0]], [[2.0, 2.0, 7.0], [7.0, 7.0, 8.0], [4.0, 7.0, 1.0]]], [[[7.0, 2.0, 3.0], 
[1.0, 4.0, 1.0], [7.0, 8.0, 2.0]], [[4.0, 3.0, 1.0], [8.0, 6.0, 3.0], [5.0, 8.0, 1.0]], [[5.0, 1.0, 5.0], [6.0, 2.0, 6.0], 
[1.0, 3.0, 7.0]]], [[[4.0, 6.0, 5.0], [2.0, 6.0, 1.0], [1.0, 3.0, 6.0]], [[4.0, 8.0, 7.0], [5.0, 7.0, 4.0], [5.0, 4.0, 7.0]], 
[[5.0, 2.0, 2.0], [7.0, 8.0, 7.0], [3.0, 5.0, 5.0]]], [[[3.0, 1.0, 2.0], [7.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[5.0, 2.0, 2.0], 
[8.0, 1.0, 4.0], [3.0, 2.0, 1.0]], [[3.0, 2.0, 8.0], [5.0, 8.0, 7.0], [3.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_URRP(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 3.0], [5.0, 1.0, 7.0], [5.0, 7.0, 8.0]], [[6.0, 3.0, 8.0], [4.0, 8.0, 2.0], [5.0, 3.0, 
4.0]], [[2.0, 2.0, 3.0], [5.0, 1.0, 1.0], [3.0, 2.0, 3.0]]], [[[5.0, 2.0, 1.0], [3.0, 8.0, 3.0], [4.0, 8.0, 4.0]], [[3.0, 2.0, 
5.0], [4.0, 3.0, 5.0], [7.0, 1.0, 5.0]], [[8.0, 8.0, 7.0], [4.0, 8.0, 8.0], [2.0, 7.0, 8.0]]], [[[1.0, 8.0, 7.0], [5.0, 4.0, 
3.0], [5.0, 7.0, 5.0]], [[8.0, 8.0, 5.0], [3.0, 3.0, 3.0], [7.0, 7.0, 7.0]], [[2.0, 5.0, 8.0], [6.0, 3.0, 8.0], [1.0, 1.0, 
3.0]]], [[[4.0, 2.0, 5.0], [3.0, 5.0, 4.0], [3.0, 7.0, 5.0]], [[5.0, 5.0, 4.0], [1.0, 3.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 7.0, 
8.0], [1.0, 1.0, 4.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0]], [[4.0]], [[7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0]], [[7.0]], [[2.0]]]))
      arg[:3,1:2,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 3.0], [5.0, 1.0, 7.0], [5.0, 7.0, 8.0]], [[6.0, 3.0, 8.0], [4.0, 4.0, 2.0], [5.0, 
3.0, 4.0]], [[2.0, 2.0, 3.0], [5.0, 1.0, 1.0], [3.0, 2.0, 3.0]]], [[[5.0, 2.0, 1.0], [3.0, 8.0, 3.0], [4.0, 8.0, 4.0]], [[3.0, 
2.0, 5.0], [4.0, 4.0, 5.0], [7.0, 1.0, 5.0]], [[8.0, 8.0, 7.0], [4.0, 8.0, 8.0], [2.0, 7.0, 8.0]]], [[[1.0, 8.0, 7.0], [5.0, 
4.0, 3.0], [5.0, 7.0, 5.0]], [[8.0, 8.0, 5.0], [3.0, 7.0, 3.0], [7.0, 7.0, 7.0]], [[2.0, 5.0, 8.0], [6.0, 3.0, 8.0], [1.0, 1.0, 
3.0]]], [[[4.0, 2.0, 5.0], [3.0, 5.0, 4.0], [3.0, 7.0, 5.0]], [[5.0, 5.0, 4.0], [1.0, 3.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 7.0, 
8.0], [1.0, 1.0, 4.0], [3.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 3.0], [5.0, 1.0, 7.0], [5.0, 7.0, 8.0]], 
[[6.0, 3.0, 8.0], [4.0, 5.0, 2.0], [5.0, 3.0, 4.0]], [[2.0, 2.0, 3.0], [5.0, 1.0, 1.0], [3.0, 2.0, 3.0]]], [[[5.0, 2.0, 1.0], 
[3.0, 8.0, 3.0], [4.0, 8.0, 4.0]], [[3.0, 2.0, 5.0], [4.0, 7.0, 5.0], [7.0, 1.0, 5.0]], [[8.0, 8.0, 7.0], [4.0, 8.0, 8.0], 
[2.0, 7.0, 8.0]]], [[[1.0, 8.0, 7.0], [5.0, 4.0, 3.0], [5.0, 7.0, 5.0]], [[8.0, 8.0, 5.0], [3.0, 2.0, 3.0], [7.0, 7.0, 7.0]], 
[[2.0, 5.0, 8.0], [6.0, 3.0, 8.0], [1.0, 1.0, 3.0]]], [[[4.0, 2.0, 5.0], [3.0, 5.0, 4.0], [3.0, 7.0, 5.0]], [[5.0, 5.0, 4.0], 
[1.0, 3.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 7.0, 8.0], [1.0, 1.0, 4.0], [3.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UUCU(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 1.0], [1.0, 5.0, 5.0], [1.0, 7.0, 1.0]], [[5.0, 4.0, 3.0], [2.0, 1.0, 2.0], [1.0, 2.0, 
2.0]], [[8.0, 4.0, 5.0], [8.0, 5.0, 5.0], [8.0, 5.0, 4.0]]], [[[2.0, 3.0, 2.0], [5.0, 6.0, 3.0], [5.0, 4.0, 6.0]], [[5.0, 1.0, 
2.0], [5.0, 5.0, 7.0], [8.0, 8.0, 1.0]], [[8.0, 7.0, 5.0], [2.0, 2.0, 2.0], [7.0, 2.0, 4.0]]], [[[4.0, 3.0, 1.0], [2.0, 1.0, 
5.0], [1.0, 2.0, 2.0]], [[7.0, 7.0, 6.0], [2.0, 4.0, 6.0], [6.0, 5.0, 2.0]], [[7.0, 8.0, 1.0], [5.0, 5.0, 7.0], [3.0, 3.0, 
6.0]]], [[[7.0, 6.0, 5.0], [8.0, 1.0, 5.0], [7.0, 4.0, 4.0]], [[4.0, 5.0, 8.0], [5.0, 3.0, 5.0], [3.0, 2.0, 8.0]], [[3.0, 7.0, 
2.0], [4.0, 8.0, 8.0], [7.0, 2.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 6.0], [8.0, 5.0], [1.0, 5.0]], [[4.0, 5.0], [4.0, 5.0], [6.0, 2.0]]], [[[3.0, 8.0], [1.0, 
4.0], [8.0, 7.0]], [[5.0, 6.0], [6.0, 6.0], [7.0, 7.0]]], [[[3.0, 3.0], [6.0, 8.0], [7.0, 6.0]], [[6.0, 3.0], [7.0, 7.0], [7.0, 
8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 1.0], [3.0, 6.0], [6.0, 4.0]], [[2.0, 4.0], [2.0, 8.0], [1.0, 2.0]]], [[[8.0, 
3.0], [8.0, 7.0], [4.0, 8.0]], [[5.0, 3.0], [1.0, 5.0], [3.0, 6.0]]], [[[2.0, 4.0], [1.0, 7.0], [4.0, 8.0]], [[5.0, 6.0], [8.0, 
3.0], [4.0, 3.0]]]]))
      arg[:3,:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 1.0], [8.0, 5.0, 5.0], [1.0, 5.0, 1.0]], [[4.0, 5.0, 3.0], [4.0, 5.0, 2.0], [6.0, 
2.0, 2.0]], [[8.0, 4.0, 5.0], [8.0, 5.0, 5.0], [8.0, 5.0, 4.0]]], [[[3.0, 8.0, 2.0], [1.0, 4.0, 3.0], [8.0, 7.0, 6.0]], [[5.0, 
6.0, 2.0], [6.0, 6.0, 7.0], [7.0, 7.0, 1.0]], [[8.0, 7.0, 5.0], [2.0, 2.0, 2.0], [7.0, 2.0, 4.0]]], [[[3.0, 3.0, 1.0], [6.0, 
8.0, 5.0], [7.0, 6.0, 2.0]], [[6.0, 3.0, 6.0], [7.0, 7.0, 6.0], [7.0, 8.0, 2.0]], [[7.0, 8.0, 1.0], [5.0, 5.0, 7.0], [3.0, 3.0, 
6.0]]], [[[7.0, 6.0, 5.0], [8.0, 1.0, 5.0], [7.0, 4.0, 4.0]], [[4.0, 5.0, 8.0], [5.0, 3.0, 5.0], [3.0, 2.0, 8.0]], [[3.0, 7.0, 
2.0], [4.0, 8.0, 8.0], [7.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 1.0], [3.0, 6.0, 5.0], [6.0, 4.0, 1.0]], 
[[2.0, 4.0, 3.0], [2.0, 8.0, 2.0], [1.0, 2.0, 2.0]], [[8.0, 4.0, 5.0], [8.0, 5.0, 5.0], [8.0, 5.0, 4.0]]], [[[8.0, 3.0, 2.0], 
[8.0, 7.0, 3.0], [4.0, 8.0, 6.0]], [[5.0, 3.0, 2.0], [1.0, 5.0, 7.0], [3.0, 6.0, 1.0]], [[8.0, 7.0, 5.0], [2.0, 2.0, 2.0], 
[7.0, 2.0, 4.0]]], [[[2.0, 4.0, 1.0], [1.0, 7.0, 5.0], [4.0, 8.0, 2.0]], [[5.0, 6.0, 6.0], [8.0, 3.0, 6.0], [4.0, 3.0, 2.0]], 
[[7.0, 8.0, 1.0], [5.0, 5.0, 7.0], [3.0, 3.0, 6.0]]], [[[7.0, 6.0, 5.0], [8.0, 1.0, 5.0], [7.0, 4.0, 4.0]], [[4.0, 5.0, 8.0], 
[5.0, 3.0, 5.0], [3.0, 2.0, 8.0]], [[3.0, 7.0, 2.0], [4.0, 8.0, 8.0], [7.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CPRL(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 4.0], [6.0, 3.0, 2.0], [2.0, 1.0, 5.0]], [[2.0, 7.0, 4.0], [4.0, 6.0, 2.0], [8.0, 3.0, 
2.0]], [[1.0, 7.0, 6.0], [3.0, 1.0, 4.0], [6.0, 7.0, 8.0]]], [[[1.0, 1.0, 8.0], [2.0, 1.0, 4.0], [7.0, 1.0, 2.0]], [[2.0, 3.0, 
1.0], [7.0, 4.0, 8.0], [3.0, 5.0, 3.0]], [[3.0, 2.0, 2.0], [8.0, 3.0, 8.0], [6.0, 4.0, 7.0]]], [[[6.0, 6.0, 1.0], [2.0, 2.0, 
8.0], [7.0, 5.0, 4.0]], [[4.0, 7.0, 8.0], [5.0, 4.0, 2.0], [1.0, 3.0, 7.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 3.0], [1.0, 8.0, 
8.0]]], [[[6.0, 5.0, 3.0], [5.0, 3.0, 3.0], [2.0, 1.0, 8.0]], [[1.0, 8.0, 6.0], [7.0, 1.0, 4.0], [2.0, 3.0, 3.0]], [[3.0, 4.0, 
3.0], [5.0, 4.0, 5.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0]], [[8.0]], [[7.0]], [[3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0]], [[7.0]], [[4.0]], [[4.0]]]))
      arg[:,1,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 4.0], [6.0, 3.0, 2.0], [2.0, 1.0, 5.0]], [[2.0, 7.0, 4.0], [4.0, 6.0, 5.0], [8.0, 
3.0, 2.0]], [[1.0, 7.0, 6.0], [3.0, 1.0, 4.0], [6.0, 7.0, 8.0]]], [[[1.0, 1.0, 8.0], [2.0, 1.0, 4.0], [7.0, 1.0, 2.0]], [[2.0, 
3.0, 1.0], [7.0, 4.0, 8.0], [3.0, 5.0, 3.0]], [[3.0, 2.0, 2.0], [8.0, 3.0, 8.0], [6.0, 4.0, 7.0]]], [[[6.0, 6.0, 1.0], [2.0, 
2.0, 8.0], [7.0, 5.0, 4.0]], [[4.0, 7.0, 8.0], [5.0, 4.0, 7.0], [1.0, 3.0, 7.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 3.0], [1.0, 8.0, 
8.0]]], [[[6.0, 5.0, 3.0], [5.0, 3.0, 3.0], [2.0, 1.0, 8.0]], [[1.0, 8.0, 6.0], [7.0, 1.0, 3.0], [2.0, 3.0, 3.0]], [[3.0, 4.0, 
3.0], [5.0, 4.0, 5.0], [3.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 4.0], [6.0, 3.0, 2.0], [2.0, 1.0, 5.0]], 
[[2.0, 7.0, 4.0], [4.0, 6.0, 2.0], [8.0, 3.0, 2.0]], [[1.0, 7.0, 6.0], [3.0, 1.0, 4.0], [6.0, 7.0, 8.0]]], [[[1.0, 1.0, 8.0], 
[2.0, 1.0, 4.0], [7.0, 1.0, 2.0]], [[2.0, 3.0, 1.0], [7.0, 4.0, 7.0], [3.0, 5.0, 3.0]], [[3.0, 2.0, 2.0], [8.0, 3.0, 8.0], 
[6.0, 4.0, 7.0]]], [[[6.0, 6.0, 1.0], [2.0, 2.0, 8.0], [7.0, 5.0, 4.0]], [[4.0, 7.0, 8.0], [5.0, 4.0, 4.0], [1.0, 3.0, 7.0]], 
[[7.0, 2.0, 3.0], [8.0, 4.0, 3.0], [1.0, 8.0, 8.0]]], [[[6.0, 5.0, 3.0], [5.0, 3.0, 3.0], [2.0, 1.0, 8.0]], [[1.0, 8.0, 6.0], 
[7.0, 1.0, 4.0], [2.0, 3.0, 3.0]], [[3.0, 4.0, 3.0], [5.0, 4.0, 5.0], [3.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_URUN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 5.0], [6.0, 8.0, 1.0], [7.0, 5.0, 2.0]], [[8.0, 6.0, 2.0], [1.0, 8.0, 3.0], [5.0, 7.0, 
6.0]], [[4.0, 5.0, 5.0], [3.0, 5.0, 5.0], [3.0, 7.0, 7.0]]], [[[8.0, 5.0, 2.0], [7.0, 3.0, 2.0], [1.0, 3.0, 4.0]], [[6.0, 8.0, 
3.0], [3.0, 4.0, 3.0], [2.0, 5.0, 8.0]], [[7.0, 1.0, 1.0], [5.0, 1.0, 6.0], [3.0, 2.0, 6.0]]], [[[6.0, 3.0, 8.0], [3.0, 7.0, 
2.0], [8.0, 3.0, 6.0]], [[2.0, 7.0, 4.0], [3.0, 4.0, 8.0], [4.0, 6.0, 8.0]], [[6.0, 3.0, 1.0], [1.0, 5.0, 4.0], [1.0, 8.0, 
4.0]]], [[[7.0, 2.0, 8.0], [4.0, 5.0, 5.0], [1.0, 6.0, 4.0]], [[4.0, 2.0, 3.0], [7.0, 4.0, 3.0], [8.0, 2.0, 6.0]], [[3.0, 4.0, 
4.0], [2.0, 7.0, 8.0], [8.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 8.0, 1.0], [6.0, 4.0, 8.0]]], [[[6.0, 1.0, 3.0], [5.0, 2.0, 7.0]]], [[[4.0, 3.0, 5.0], 
[5.0, 7.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 7.0], [2.0, 6.0, 6.0]]], [[[3.0, 4.0, 1.0], [6.0, 7.0, 5.0]]], [[[6.0, 
5.0, 8.0], [5.0, 5.0, 8.0]]]]))
      arg[:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 5.0], [6.0, 8.0, 1.0], [7.0, 5.0, 2.0]], [[6.0, 8.0, 1.0], [6.0, 4.0, 8.0], [5.0, 
7.0, 6.0]], [[4.0, 5.0, 5.0], [3.0, 5.0, 5.0], [3.0, 7.0, 7.0]]], [[[8.0, 5.0, 2.0], [7.0, 3.0, 2.0], [1.0, 3.0, 4.0]], [[6.0, 
1.0, 3.0], [5.0, 2.0, 7.0], [2.0, 5.0, 8.0]], [[7.0, 1.0, 1.0], [5.0, 1.0, 6.0], [3.0, 2.0, 6.0]]], [[[6.0, 3.0, 8.0], [3.0, 
7.0, 2.0], [8.0, 3.0, 6.0]], [[4.0, 3.0, 5.0], [5.0, 7.0, 2.0], [4.0, 6.0, 8.0]], [[6.0, 3.0, 1.0], [1.0, 5.0, 4.0], [1.0, 8.0, 
4.0]]], [[[7.0, 2.0, 8.0], [4.0, 5.0, 5.0], [1.0, 6.0, 4.0]], [[4.0, 2.0, 3.0], [7.0, 4.0, 3.0], [8.0, 2.0, 6.0]], [[3.0, 4.0, 
4.0], [2.0, 7.0, 8.0], [8.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 5.0], [6.0, 8.0, 1.0], [7.0, 5.0, 2.0]], 
[[4.0, 5.0, 7.0], [2.0, 6.0, 6.0], [5.0, 7.0, 6.0]], [[4.0, 5.0, 5.0], [3.0, 5.0, 5.0], [3.0, 7.0, 7.0]]], [[[8.0, 5.0, 2.0], 
[7.0, 3.0, 2.0], [1.0, 3.0, 4.0]], [[3.0, 4.0, 1.0], [6.0, 7.0, 5.0], [2.0, 5.0, 8.0]], [[7.0, 1.0, 1.0], [5.0, 1.0, 6.0], 
[3.0, 2.0, 6.0]]], [[[6.0, 3.0, 8.0], [3.0, 7.0, 2.0], [8.0, 3.0, 6.0]], [[6.0, 5.0, 8.0], [5.0, 5.0, 8.0], [4.0, 6.0, 8.0]], 
[[6.0, 3.0, 1.0], [1.0, 5.0, 4.0], [1.0, 8.0, 4.0]]], [[[7.0, 2.0, 8.0], [4.0, 5.0, 5.0], [1.0, 6.0, 4.0]], [[4.0, 2.0, 3.0], 
[7.0, 4.0, 3.0], [8.0, 2.0, 6.0]], [[3.0, 4.0, 4.0], [2.0, 7.0, 8.0], [8.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PUCN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 5.0], [2.0, 7.0, 3.0], [3.0, 5.0, 2.0]], [[3.0, 1.0, 5.0], [7.0, 1.0, 3.0], [8.0, 3.0, 
1.0]], [[4.0, 7.0, 2.0], [2.0, 1.0, 4.0], [3.0, 8.0, 5.0]]], [[[8.0, 5.0, 4.0], [1.0, 4.0, 5.0], [8.0, 1.0, 8.0]], [[4.0, 6.0, 
4.0], [1.0, 4.0, 7.0], [3.0, 7.0, 5.0]], [[8.0, 5.0, 2.0], [3.0, 3.0, 4.0], [1.0, 4.0, 1.0]]], [[[6.0, 8.0, 5.0], [8.0, 6.0, 
7.0], [7.0, 7.0, 3.0]], [[8.0, 2.0, 5.0], [5.0, 2.0, 3.0], [4.0, 2.0, 6.0]], [[7.0, 6.0, 3.0], [2.0, 7.0, 7.0], [1.0, 8.0, 
3.0]]], [[[4.0, 7.0, 5.0], [1.0, 7.0, 6.0], [5.0, 3.0, 7.0]], [[6.0, 3.0, 4.0], [6.0, 3.0, 5.0], [2.0, 6.0, 1.0]], [[5.0, 5.0, 
6.0], [8.0, 7.0, 6.0], [2.0, 8.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 1.0, 5.0], [1.0, 3.0, 5.0], [3.0, 2.0, 7.0]], [[4.0, 3.0, 7.0], [2.0, 4.0, 2.0], [5.0, 1.0, 
6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 1.0, 3.0], [5.0, 1.0, 3.0], [3.0, 7.0, 7.0]], [[2.0, 1.0, 6.0], [4.0, 5.0, 3.0], 
[2.0, 6.0, 2.0]]]))
      arg[2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 5.0], [2.0, 7.0, 3.0], [3.0, 5.0, 2.0]], [[3.0, 1.0, 5.0], [7.0, 1.0, 3.0], [8.0, 
3.0, 1.0]], [[4.0, 7.0, 2.0], [2.0, 1.0, 4.0], [3.0, 8.0, 5.0]]], [[[8.0, 5.0, 4.0], [1.0, 4.0, 5.0], [8.0, 1.0, 8.0]], [[4.0, 
6.0, 4.0], [1.0, 4.0, 7.0], [3.0, 7.0, 5.0]], [[8.0, 5.0, 2.0], [3.0, 3.0, 4.0], [1.0, 4.0, 1.0]]], [[[4.0, 1.0, 5.0], [1.0, 
3.0, 5.0], [3.0, 2.0, 7.0]], [[4.0, 3.0, 7.0], [2.0, 4.0, 2.0], [5.0, 1.0, 6.0]], [[7.0, 6.0, 3.0], [2.0, 7.0, 7.0], [1.0, 8.0, 
3.0]]], [[[4.0, 7.0, 5.0], [1.0, 7.0, 6.0], [5.0, 3.0, 7.0]], [[6.0, 3.0, 4.0], [6.0, 3.0, 5.0], [2.0, 6.0, 1.0]], [[5.0, 5.0, 
6.0], [8.0, 7.0, 6.0], [2.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 5.0], [2.0, 7.0, 3.0], [3.0, 5.0, 2.0]], 
[[3.0, 1.0, 5.0], [7.0, 1.0, 3.0], [8.0, 3.0, 1.0]], [[4.0, 7.0, 2.0], [2.0, 1.0, 4.0], [3.0, 8.0, 5.0]]], [[[8.0, 5.0, 4.0], 
[1.0, 4.0, 5.0], [8.0, 1.0, 8.0]], [[4.0, 6.0, 4.0], [1.0, 4.0, 7.0], [3.0, 7.0, 5.0]], [[8.0, 5.0, 2.0], [3.0, 3.0, 4.0], 
[1.0, 4.0, 1.0]]], [[[7.0, 1.0, 3.0], [5.0, 1.0, 3.0], [3.0, 7.0, 7.0]], [[2.0, 1.0, 6.0], [4.0, 5.0, 3.0], [2.0, 6.0, 2.0]], 
[[7.0, 6.0, 3.0], [2.0, 7.0, 7.0], [1.0, 8.0, 3.0]]], [[[4.0, 7.0, 5.0], [1.0, 7.0, 6.0], [5.0, 3.0, 7.0]], [[6.0, 3.0, 4.0], 
[6.0, 3.0, 5.0], [2.0, 6.0, 1.0]], [[5.0, 5.0, 6.0], [8.0, 7.0, 6.0], [2.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULRP(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 3.0], [4.0, 4.0, 1.0], [4.0, 6.0, 1.0]], [[6.0, 8.0, 1.0], [3.0, 2.0, 7.0], [2.0, 6.0, 
3.0]], [[2.0, 8.0, 7.0], [5.0, 8.0, 4.0], [7.0, 5.0, 5.0]]], [[[3.0, 8.0, 7.0], [7.0, 4.0, 2.0], [4.0, 6.0, 2.0]], [[3.0, 5.0, 
1.0], [3.0, 7.0, 4.0], [5.0, 2.0, 8.0]], [[6.0, 4.0, 2.0], [3.0, 8.0, 1.0], [1.0, 5.0, 8.0]]], [[[2.0, 6.0, 5.0], [6.0, 5.0, 
3.0], [1.0, 7.0, 3.0]], [[3.0, 4.0, 2.0], [6.0, 4.0, 6.0], [1.0, 3.0, 2.0]], [[5.0, 5.0, 4.0], [6.0, 3.0, 6.0], [8.0, 3.0, 
5.0]]], [[[4.0, 1.0, 6.0], [4.0, 2.0, 1.0], [8.0, 3.0, 8.0]], [[3.0, 6.0, 1.0], [1.0, 3.0, 3.0], [7.0, 2.0, 7.0]], [[6.0, 2.0, 
7.0], [7.0, 2.0, 7.0], [4.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0]], [[3.0]], [[2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0]], [[5.0]], [[2.0]]]))
      arg[:3,2:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 3.0], [4.0, 4.0, 1.0], [4.0, 6.0, 1.0]], [[6.0, 8.0, 1.0], [3.0, 2.0, 7.0], [2.0, 
6.0, 3.0]], [[2.0, 8.0, 7.0], [5.0, 8.0, 4.0], [7.0, 5.0, 5.0]]], [[[3.0, 8.0, 7.0], [7.0, 4.0, 2.0], [4.0, 6.0, 2.0]], [[3.0, 
5.0, 1.0], [3.0, 7.0, 4.0], [5.0, 2.0, 8.0]], [[6.0, 4.0, 2.0], [3.0, 3.0, 1.0], [1.0, 5.0, 8.0]]], [[[2.0, 6.0, 5.0], [6.0, 
5.0, 3.0], [1.0, 7.0, 3.0]], [[3.0, 4.0, 2.0], [6.0, 4.0, 6.0], [1.0, 3.0, 2.0]], [[5.0, 5.0, 4.0], [6.0, 2.0, 6.0], [8.0, 3.0, 
5.0]]], [[[4.0, 1.0, 6.0], [4.0, 2.0, 1.0], [8.0, 3.0, 8.0]], [[3.0, 6.0, 1.0], [1.0, 3.0, 3.0], [7.0, 2.0, 7.0]], [[6.0, 2.0, 
7.0], [7.0, 2.0, 7.0], [4.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 3.0], [4.0, 4.0, 1.0], [4.0, 6.0, 1.0]], 
[[6.0, 8.0, 1.0], [3.0, 2.0, 7.0], [2.0, 6.0, 3.0]], [[2.0, 8.0, 7.0], [5.0, 6.0, 4.0], [7.0, 5.0, 5.0]]], [[[3.0, 8.0, 7.0], 
[7.0, 4.0, 2.0], [4.0, 6.0, 2.0]], [[3.0, 5.0, 1.0], [3.0, 7.0, 4.0], [5.0, 2.0, 8.0]], [[6.0, 4.0, 2.0], [3.0, 5.0, 1.0], 
[1.0, 5.0, 8.0]]], [[[2.0, 6.0, 5.0], [6.0, 5.0, 3.0], [1.0, 7.0, 3.0]], [[3.0, 4.0, 2.0], [6.0, 4.0, 6.0], [1.0, 3.0, 2.0]], 
[[5.0, 5.0, 4.0], [6.0, 2.0, 6.0], [8.0, 3.0, 5.0]]], [[[4.0, 1.0, 6.0], [4.0, 2.0, 1.0], [8.0, 3.0, 8.0]], [[3.0, 6.0, 1.0], 
[1.0, 3.0, 3.0], [7.0, 2.0, 7.0]], [[6.0, 2.0, 7.0], [7.0, 2.0, 7.0], [4.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RPUN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 6.0], [4.0, 2.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 4.0, 8.0], [8.0, 4.0, 1.0], [1.0, 3.0, 
1.0]], [[4.0, 7.0, 5.0], [1.0, 7.0, 8.0], [2.0, 1.0, 5.0]]], [[[5.0, 2.0, 7.0], [4.0, 7.0, 7.0], [4.0, 4.0, 5.0]], [[3.0, 8.0, 
6.0], [5.0, 2.0, 8.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 2.0], [1.0, 7.0, 4.0], [4.0, 6.0, 3.0]]], [[[6.0, 7.0, 2.0], [1.0, 5.0, 
2.0], [5.0, 2.0, 4.0]], [[2.0, 6.0, 1.0], [5.0, 5.0, 2.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 8.0], [4.0, 3.0, 4.0], [7.0, 1.0, 
3.0]]], [[[1.0, 1.0, 6.0], [8.0, 5.0, 5.0], [3.0, 2.0, 8.0]], [[4.0, 3.0, 3.0], [6.0, 4.0, 2.0], [1.0, 5.0, 6.0]], [[5.0, 1.0, 
4.0], [4.0, 5.0, 8.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 8.0, 3.0], [6.0, 6.0, 4.0]], [[2.0, 8.0, 3.0], [4.0, 7.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 6.0, 5.0], [6.0, 7.0, 3.0]], [[1.0, 5.0, 3.0], [3.0, 6.0, 2.0]]]))
      arg[1:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 6.0], [4.0, 2.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 4.0, 8.0], [8.0, 4.0, 1.0], [1.0, 
3.0, 1.0]], [[4.0, 7.0, 5.0], [1.0, 7.0, 8.0], [2.0, 1.0, 5.0]]], [[[5.0, 2.0, 7.0], [4.0, 7.0, 7.0], [4.0, 4.0, 5.0]], [[6.0, 
8.0, 3.0], [6.0, 6.0, 4.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 2.0], [1.0, 7.0, 4.0], [4.0, 6.0, 3.0]]], [[[6.0, 7.0, 2.0], [1.0, 
5.0, 2.0], [5.0, 2.0, 4.0]], [[2.0, 8.0, 3.0], [4.0, 7.0, 4.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 8.0], [4.0, 3.0, 4.0], [7.0, 1.0, 
3.0]]], [[[1.0, 1.0, 6.0], [8.0, 5.0, 5.0], [3.0, 2.0, 8.0]], [[4.0, 3.0, 3.0], [6.0, 4.0, 2.0], [1.0, 5.0, 6.0]], [[5.0, 1.0, 
4.0], [4.0, 5.0, 8.0], [1.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 6.0], [4.0, 2.0, 1.0], [3.0, 3.0, 2.0]], 
[[7.0, 4.0, 8.0], [8.0, 4.0, 1.0], [1.0, 3.0, 1.0]], [[4.0, 7.0, 5.0], [1.0, 7.0, 8.0], [2.0, 1.0, 5.0]]], [[[5.0, 2.0, 7.0], 
[4.0, 7.0, 7.0], [4.0, 4.0, 5.0]], [[6.0, 6.0, 5.0], [6.0, 7.0, 3.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 2.0], [1.0, 7.0, 4.0], 
[4.0, 6.0, 3.0]]], [[[6.0, 7.0, 2.0], [1.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[1.0, 5.0, 3.0], [3.0, 6.0, 2.0], [7.0, 5.0, 3.0]], 
[[3.0, 5.0, 8.0], [4.0, 3.0, 4.0], [7.0, 1.0, 3.0]]], [[[1.0, 1.0, 6.0], [8.0, 5.0, 5.0], [3.0, 2.0, 8.0]], [[4.0, 3.0, 3.0], 
[6.0, 4.0, 2.0], [1.0, 5.0, 6.0]], [[5.0, 1.0, 4.0], [4.0, 5.0, 8.0], [1.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RPPN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 2.0], [2.0, 6.0, 8.0], [4.0, 6.0, 4.0]], [[1.0, 6.0, 5.0], [6.0, 4.0, 5.0], [6.0, 1.0, 
8.0]], [[7.0, 2.0, 3.0], [7.0, 4.0, 4.0], [5.0, 7.0, 3.0]]], [[[8.0, 4.0, 6.0], [7.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[4.0, 2.0, 
6.0], [5.0, 3.0, 2.0], [6.0, 4.0, 7.0]], [[6.0, 3.0, 5.0], [2.0, 6.0, 4.0], [1.0, 5.0, 2.0]]], [[[1.0, 6.0, 8.0], [6.0, 8.0, 
3.0], [1.0, 7.0, 6.0]], [[3.0, 8.0, 5.0], [2.0, 2.0, 6.0], [2.0, 8.0, 3.0]], [[4.0, 7.0, 4.0], [5.0, 4.0, 2.0], [1.0, 3.0, 
8.0]]], [[[2.0, 3.0, 7.0], [2.0, 3.0, 7.0], [7.0, 8.0, 3.0]], [[1.0, 2.0, 5.0], [3.0, 3.0, 8.0], [4.0, 4.0, 8.0]], [[5.0, 5.0, 
2.0], [2.0, 6.0, 4.0], [1.0, 8.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 2.0, 8.0], [7.0, 1.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 2.0, 1.0], [8.0, 6.0, 3.0]]))
      arg[1:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 2.0], [2.0, 6.0, 8.0], [4.0, 6.0, 4.0]], [[1.0, 6.0, 5.0], [6.0, 4.0, 5.0], [6.0, 
1.0, 8.0]], [[7.0, 2.0, 3.0], [7.0, 4.0, 4.0], [5.0, 7.0, 3.0]]], [[[8.0, 4.0, 6.0], [7.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[4.0, 
2.0, 6.0], [2.0, 2.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 3.0, 5.0], [2.0, 6.0, 4.0], [1.0, 5.0, 2.0]]], [[[1.0, 6.0, 8.0], [6.0, 
8.0, 3.0], [1.0, 7.0, 6.0]], [[3.0, 8.0, 5.0], [7.0, 1.0, 2.0], [2.0, 8.0, 3.0]], [[4.0, 7.0, 4.0], [5.0, 4.0, 2.0], [1.0, 3.0, 
8.0]]], [[[2.0, 3.0, 7.0], [2.0, 3.0, 7.0], [7.0, 8.0, 3.0]], [[1.0, 2.0, 5.0], [3.0, 3.0, 8.0], [4.0, 4.0, 8.0]], [[5.0, 5.0, 
2.0], [2.0, 6.0, 4.0], [1.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 2.0], [2.0, 6.0, 8.0], [4.0, 6.0, 4.0]], 
[[1.0, 6.0, 5.0], [6.0, 4.0, 5.0], [6.0, 1.0, 8.0]], [[7.0, 2.0, 3.0], [7.0, 4.0, 4.0], [5.0, 7.0, 3.0]]], [[[8.0, 4.0, 6.0], 
[7.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[4.0, 2.0, 6.0], [4.0, 2.0, 1.0], [6.0, 4.0, 7.0]], [[6.0, 3.0, 5.0], [2.0, 6.0, 4.0], 
[1.0, 5.0, 2.0]]], [[[1.0, 6.0, 8.0], [6.0, 8.0, 3.0], [1.0, 7.0, 6.0]], [[3.0, 8.0, 5.0], [8.0, 6.0, 3.0], [2.0, 8.0, 3.0]], 
[[4.0, 7.0, 4.0], [5.0, 4.0, 2.0], [1.0, 3.0, 8.0]]], [[[2.0, 3.0, 7.0], [2.0, 3.0, 7.0], [7.0, 8.0, 3.0]], [[1.0, 2.0, 5.0], 
[3.0, 3.0, 8.0], [4.0, 4.0, 8.0]], [[5.0, 5.0, 2.0], [2.0, 6.0, 4.0], [1.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PPPC(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 4.0], [4.0, 4.0, 6.0], [7.0, 3.0, 3.0]], [[2.0, 2.0, 1.0], [2.0, 7.0, 3.0], [8.0, 2.0, 
4.0]], [[3.0, 8.0, 5.0], [5.0, 5.0, 3.0], [3.0, 3.0, 1.0]]], [[[6.0, 4.0, 7.0], [1.0, 4.0, 1.0], [8.0, 6.0, 3.0]], [[3.0, 1.0, 
5.0], [5.0, 2.0, 6.0], [2.0, 2.0, 1.0]], [[6.0, 2.0, 4.0], [4.0, 4.0, 7.0], [1.0, 3.0, 8.0]]], [[[8.0, 3.0, 2.0], [7.0, 6.0, 
1.0], [6.0, 5.0, 1.0]], [[5.0, 3.0, 3.0], [4.0, 7.0, 3.0], [5.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], [6.0, 2.0, 2.0], [7.0, 5.0, 
8.0]]], [[[5.0, 1.0, 7.0], [4.0, 6.0, 8.0], [1.0, 2.0, 2.0]], [[4.0, 8.0, 2.0], [2.0, 5.0, 5.0], [1.0, 3.0, 8.0]], [[4.0, 6.0, 
2.0], [2.0, 6.0, 2.0], [6.0, 2.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([2.0, 8.0, 6.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([6.0, 1.0, 6.0]))
      arg[2,1,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 4.0], [4.0, 4.0, 6.0], [7.0, 3.0, 3.0]], [[2.0, 2.0, 1.0], [2.0, 7.0, 3.0], [8.0, 
2.0, 4.0]], [[3.0, 8.0, 5.0], [5.0, 5.0, 3.0], [3.0, 3.0, 1.0]]], [[[6.0, 4.0, 7.0], [1.0, 4.0, 1.0], [8.0, 6.0, 3.0]], [[3.0, 
1.0, 5.0], [5.0, 2.0, 6.0], [2.0, 2.0, 1.0]], [[6.0, 2.0, 4.0], [4.0, 4.0, 7.0], [1.0, 3.0, 8.0]]], [[[8.0, 3.0, 2.0], [7.0, 
6.0, 1.0], [6.0, 5.0, 1.0]], [[5.0, 3.0, 3.0], [2.0, 8.0, 6.0], [5.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], [6.0, 2.0, 2.0], [7.0, 5.0, 
8.0]]], [[[5.0, 1.0, 7.0], [4.0, 6.0, 8.0], [1.0, 2.0, 2.0]], [[4.0, 8.0, 2.0], [2.0, 5.0, 5.0], [1.0, 3.0, 8.0]], [[4.0, 6.0, 
2.0], [2.0, 6.0, 2.0], [6.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 4.0], [4.0, 4.0, 6.0], [7.0, 3.0, 3.0]], 
[[2.0, 2.0, 1.0], [2.0, 7.0, 3.0], [8.0, 2.0, 4.0]], [[3.0, 8.0, 5.0], [5.0, 5.0, 3.0], [3.0, 3.0, 1.0]]], [[[6.0, 4.0, 7.0], 
[1.0, 4.0, 1.0], [8.0, 6.0, 3.0]], [[3.0, 1.0, 5.0], [5.0, 2.0, 6.0], [2.0, 2.0, 1.0]], [[6.0, 2.0, 4.0], [4.0, 4.0, 7.0], 
[1.0, 3.0, 8.0]]], [[[8.0, 3.0, 2.0], [7.0, 6.0, 1.0], [6.0, 5.0, 1.0]], [[5.0, 3.0, 3.0], [6.0, 1.0, 6.0], [5.0, 4.0, 1.0]], 
[[3.0, 4.0, 6.0], [6.0, 2.0, 2.0], [7.0, 5.0, 8.0]]], [[[5.0, 1.0, 7.0], [4.0, 6.0, 8.0], [1.0, 2.0, 2.0]], [[4.0, 8.0, 2.0], 
[2.0, 5.0, 5.0], [1.0, 3.0, 8.0]], [[4.0, 6.0, 2.0], [2.0, 6.0, 2.0], [6.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLPR(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 4.0], [3.0, 7.0, 2.0], [3.0, 8.0, 5.0]], [[8.0, 1.0, 2.0], [5.0, 4.0, 6.0], [5.0, 4.0, 
3.0]], [[8.0, 3.0, 2.0], [3.0, 4.0, 3.0], [1.0, 4.0, 3.0]]], [[[5.0, 5.0, 2.0], [3.0, 2.0, 2.0], [8.0, 2.0, 1.0]], [[1.0, 1.0, 
4.0], [3.0, 2.0, 4.0], [5.0, 2.0, 1.0]], [[2.0, 7.0, 4.0], [3.0, 2.0, 6.0], [3.0, 5.0, 5.0]]], [[[6.0, 3.0, 1.0], [5.0, 7.0, 
5.0], [7.0, 1.0, 6.0]], [[5.0, 4.0, 5.0], [1.0, 6.0, 7.0], [6.0, 2.0, 7.0]], [[8.0, 2.0, 3.0], [3.0, 4.0, 2.0], [1.0, 6.0, 
1.0]]], [[[3.0, 3.0, 7.0], [1.0, 3.0, 2.0], [8.0, 4.0, 6.0]], [[1.0, 3.0, 2.0], [6.0, 2.0, 8.0], [8.0, 6.0, 2.0]], [[7.0, 1.0, 
3.0], [4.0, 3.0, 6.0], [2.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0]]))
      arg[2,2:,1,1:2]=val
      ref=Data(numpy.array([[[[6.0, 5.0, 4.0], [3.0, 7.0, 2.0], [3.0, 8.0, 5.0]], [[8.0, 1.0, 2.0], [5.0, 4.0, 6.0], [5.0, 4.0, 
3.0]], [[8.0, 3.0, 2.0], [3.0, 4.0, 3.0], [1.0, 4.0, 3.0]]], [[[5.0, 5.0, 2.0], [3.0, 2.0, 2.0], [8.0, 2.0, 1.0]], [[1.0, 1.0, 
4.0], [3.0, 2.0, 4.0], [5.0, 2.0, 1.0]], [[2.0, 7.0, 4.0], [3.0, 2.0, 6.0], [3.0, 5.0, 5.0]]], [[[6.0, 3.0, 1.0], [5.0, 7.0, 
5.0], [7.0, 1.0, 6.0]], [[5.0, 4.0, 5.0], [1.0, 6.0, 7.0], [6.0, 2.0, 7.0]], [[8.0, 2.0, 3.0], [3.0, 8.0, 2.0], [1.0, 6.0, 
1.0]]], [[[3.0, 3.0, 7.0], [1.0, 3.0, 2.0], [8.0, 4.0, 6.0]], [[1.0, 3.0, 2.0], [6.0, 2.0, 8.0], [8.0, 6.0, 2.0]], [[7.0, 1.0, 
3.0], [4.0, 3.0, 6.0], [2.0, 1.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UUUN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 2.0], [5.0, 8.0, 8.0], [5.0, 7.0, 7.0]], [[7.0, 1.0, 6.0], [5.0, 6.0, 2.0], [5.0, 7.0, 
4.0]], [[7.0, 4.0, 6.0], [8.0, 5.0, 5.0], [7.0, 8.0, 5.0]]], [[[3.0, 4.0, 6.0], [1.0, 5.0, 3.0], [2.0, 2.0, 1.0]], [[4.0, 7.0, 
3.0], [5.0, 4.0, 4.0], [4.0, 3.0, 1.0]], [[2.0, 7.0, 7.0], [1.0, 5.0, 3.0], [5.0, 7.0, 7.0]]], [[[5.0, 2.0, 5.0], [6.0, 4.0, 
7.0], [1.0, 4.0, 4.0]], [[4.0, 7.0, 7.0], [3.0, 3.0, 5.0], [3.0, 3.0, 2.0]], [[7.0, 4.0, 5.0], [7.0, 5.0, 8.0], [1.0, 3.0, 
2.0]]], [[[6.0, 2.0, 4.0], [1.0, 5.0, 7.0], [1.0, 8.0, 4.0]], [[5.0, 6.0, 8.0], [1.0, 1.0, 2.0], [2.0, 3.0, 3.0]], [[7.0, 1.0, 
8.0], [1.0, 2.0, 2.0], [1.0, 3.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 4.0, 6.0], [2.0, 3.0, 3.0]], [[6.0, 6.0, 4.0], [8.0, 8.0, 3.0]]], [[[2.0, 4.0, 1.0], [3.0, 
2.0, 6.0]], [[3.0, 1.0, 6.0], [2.0, 6.0, 4.0]]], [[[3.0, 3.0, 4.0], [2.0, 1.0, 6.0]], [[1.0, 6.0, 7.0], [6.0, 5.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 4.0], [1.0, 3.0, 3.0]], [[5.0, 5.0, 4.0], [2.0, 8.0, 4.0]]], [[[6.0, 8.0, 
2.0], [4.0, 1.0, 1.0]], [[7.0, 5.0, 5.0], [1.0, 1.0, 4.0]]], [[[1.0, 7.0, 3.0], [7.0, 8.0, 7.0]], [[2.0, 1.0, 1.0], [2.0, 3.0, 
1.0]]]]))
      arg[:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 6.0], [2.0, 3.0, 3.0], [5.0, 7.0, 7.0]], [[6.0, 6.0, 4.0], [8.0, 8.0, 3.0], [5.0, 
7.0, 4.0]], [[7.0, 4.0, 6.0], [8.0, 5.0, 5.0], [7.0, 8.0, 5.0]]], [[[2.0, 4.0, 1.0], [3.0, 2.0, 6.0], [2.0, 2.0, 1.0]], [[3.0, 
1.0, 6.0], [2.0, 6.0, 4.0], [4.0, 3.0, 1.0]], [[2.0, 7.0, 7.0], [1.0, 5.0, 3.0], [5.0, 7.0, 7.0]]], [[[3.0, 3.0, 4.0], [2.0, 
1.0, 6.0], [1.0, 4.0, 4.0]], [[1.0, 6.0, 7.0], [6.0, 5.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 4.0, 5.0], [7.0, 5.0, 8.0], [1.0, 3.0, 
2.0]]], [[[6.0, 2.0, 4.0], [1.0, 5.0, 7.0], [1.0, 8.0, 4.0]], [[5.0, 6.0, 8.0], [1.0, 1.0, 2.0], [2.0, 3.0, 3.0]], [[7.0, 1.0, 
8.0], [1.0, 2.0, 2.0], [1.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [1.0, 3.0, 3.0], [5.0, 7.0, 7.0]], 
[[5.0, 5.0, 4.0], [2.0, 8.0, 4.0], [5.0, 7.0, 4.0]], [[7.0, 4.0, 6.0], [8.0, 5.0, 5.0], [7.0, 8.0, 5.0]]], [[[6.0, 8.0, 2.0], 
[4.0, 1.0, 1.0], [2.0, 2.0, 1.0]], [[7.0, 5.0, 5.0], [1.0, 1.0, 4.0], [4.0, 3.0, 1.0]], [[2.0, 7.0, 7.0], [1.0, 5.0, 3.0], 
[5.0, 7.0, 7.0]]], [[[1.0, 7.0, 3.0], [7.0, 8.0, 7.0], [1.0, 4.0, 4.0]], [[2.0, 1.0, 1.0], [2.0, 3.0, 1.0], [3.0, 3.0, 2.0]], 
[[7.0, 4.0, 5.0], [7.0, 5.0, 8.0], [1.0, 3.0, 2.0]]], [[[6.0, 2.0, 4.0], [1.0, 5.0, 7.0], [1.0, 8.0, 4.0]], [[5.0, 6.0, 8.0], 
[1.0, 1.0, 2.0], [2.0, 3.0, 3.0]], [[7.0, 1.0, 8.0], [1.0, 2.0, 2.0], [1.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UURN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 5.0], [8.0, 6.0, 3.0], [8.0, 8.0, 5.0]], [[3.0, 8.0, 5.0], [4.0, 3.0, 5.0], [6.0, 8.0, 
5.0]], [[3.0, 8.0, 5.0], [2.0, 4.0, 5.0], [7.0, 6.0, 5.0]]], [[[6.0, 4.0, 5.0], [5.0, 5.0, 1.0], [5.0, 7.0, 3.0]], [[1.0, 6.0, 
4.0], [1.0, 1.0, 4.0], [8.0, 4.0, 1.0]], [[5.0, 1.0, 4.0], [8.0, 6.0, 8.0], [6.0, 4.0, 4.0]]], [[[8.0, 5.0, 5.0], [5.0, 4.0, 
4.0], [6.0, 4.0, 2.0]], [[2.0, 1.0, 8.0], [1.0, 7.0, 2.0], [3.0, 5.0, 7.0]], [[3.0, 1.0, 8.0], [8.0, 5.0, 8.0], [4.0, 6.0, 
3.0]]], [[[3.0, 5.0, 8.0], [7.0, 1.0, 4.0], [1.0, 2.0, 3.0]], [[4.0, 2.0, 4.0], [5.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[2.0, 1.0, 
1.0], [5.0, 3.0, 4.0], [4.0, 6.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 6.0, 7.0]], [[6.0, 6.0, 3.0]]], [[[2.0, 1.0, 2.0]], [[2.0, 4.0, 3.0]]], [[[2.0, 8.0, 2.0]], 
[[1.0, 2.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 2.0]], [[5.0, 8.0, 3.0]]], [[[3.0, 4.0, 4.0]], [[6.0, 8.0, 5.0]]], [[[7.0, 
6.0, 1.0]], [[8.0, 1.0, 3.0]]]]))
      arg[:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 5.0], [8.0, 6.0, 7.0], [8.0, 8.0, 5.0]], [[3.0, 8.0, 5.0], [6.0, 6.0, 3.0], [6.0, 
8.0, 5.0]], [[3.0, 8.0, 5.0], [2.0, 4.0, 5.0], [7.0, 6.0, 5.0]]], [[[6.0, 4.0, 5.0], [2.0, 1.0, 2.0], [5.0, 7.0, 3.0]], [[1.0, 
6.0, 4.0], [2.0, 4.0, 3.0], [8.0, 4.0, 1.0]], [[5.0, 1.0, 4.0], [8.0, 6.0, 8.0], [6.0, 4.0, 4.0]]], [[[8.0, 5.0, 5.0], [2.0, 
8.0, 2.0], [6.0, 4.0, 2.0]], [[2.0, 1.0, 8.0], [1.0, 2.0, 6.0], [3.0, 5.0, 7.0]], [[3.0, 1.0, 8.0], [8.0, 5.0, 8.0], [4.0, 6.0, 
3.0]]], [[[3.0, 5.0, 8.0], [7.0, 1.0, 4.0], [1.0, 2.0, 3.0]], [[4.0, 2.0, 4.0], [5.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[2.0, 1.0, 
1.0], [5.0, 3.0, 4.0], [4.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 5.0], [8.0, 2.0, 2.0], [8.0, 8.0, 5.0]], 
[[3.0, 8.0, 5.0], [5.0, 8.0, 3.0], [6.0, 8.0, 5.0]], [[3.0, 8.0, 5.0], [2.0, 4.0, 5.0], [7.0, 6.0, 5.0]]], [[[6.0, 4.0, 5.0], 
[3.0, 4.0, 4.0], [5.0, 7.0, 3.0]], [[1.0, 6.0, 4.0], [6.0, 8.0, 5.0], [8.0, 4.0, 1.0]], [[5.0, 1.0, 4.0], [8.0, 6.0, 8.0], 
[6.0, 4.0, 4.0]]], [[[8.0, 5.0, 5.0], [7.0, 6.0, 1.0], [6.0, 4.0, 2.0]], [[2.0, 1.0, 8.0], [8.0, 1.0, 3.0], [3.0, 5.0, 7.0]], 
[[3.0, 1.0, 8.0], [8.0, 5.0, 8.0], [4.0, 6.0, 3.0]]], [[[3.0, 5.0, 8.0], [7.0, 1.0, 4.0], [1.0, 2.0, 3.0]], [[4.0, 2.0, 4.0], 
[5.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[2.0, 1.0, 1.0], [5.0, 3.0, 4.0], [4.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LCRC(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 8.0], [1.0, 6.0, 2.0], [1.0, 6.0, 1.0]], [[5.0, 4.0, 3.0], [2.0, 5.0, 2.0], [4.0, 4.0, 
3.0]], [[4.0, 6.0, 2.0], [4.0, 1.0, 3.0], [8.0, 6.0, 2.0]]], [[[1.0, 5.0, 8.0], [4.0, 5.0, 5.0], [1.0, 6.0, 1.0]], [[3.0, 1.0, 
8.0], [7.0, 6.0, 6.0], [1.0, 1.0, 6.0]], [[5.0, 8.0, 8.0], [1.0, 7.0, 1.0], [1.0, 8.0, 6.0]]], [[[1.0, 7.0, 2.0], [2.0, 2.0, 
2.0], [4.0, 2.0, 1.0]], [[6.0, 2.0, 5.0], [1.0, 5.0, 4.0], [5.0, 2.0, 2.0]], [[3.0, 4.0, 7.0], [6.0, 5.0, 3.0], [3.0, 7.0, 
2.0]]], [[[8.0, 4.0, 3.0], [3.0, 3.0, 8.0], [4.0, 2.0, 4.0]], [[6.0, 4.0, 8.0], [5.0, 3.0, 6.0], [5.0, 4.0, 1.0]], [[7.0, 6.0, 
2.0], [4.0, 8.0, 1.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 6.0, 1.0]], [[6.0, 2.0, 7.0]], [[1.0, 5.0, 8.0]]], [[[4.0, 4.0, 5.0]], [[3.0, 3.0, 4.0]], 
[[8.0, 5.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 1.0]], [[5.0, 3.0, 6.0]], [[8.0, 1.0, 5.0]]], [[[8.0, 3.0, 1.0]], [[5.0, 
7.0, 3.0]], [[1.0, 1.0, 6.0]]]]))
      arg[2:,:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [1.0, 6.0, 2.0], [1.0, 6.0, 1.0]], [[5.0, 4.0, 3.0], [2.0, 5.0, 2.0], [4.0, 
4.0, 3.0]], [[4.0, 6.0, 2.0], [4.0, 1.0, 3.0], [8.0, 6.0, 2.0]]], [[[1.0, 5.0, 8.0], [4.0, 5.0, 5.0], [1.0, 6.0, 1.0]], [[3.0, 
1.0, 8.0], [7.0, 6.0, 6.0], [1.0, 1.0, 6.0]], [[5.0, 8.0, 8.0], [1.0, 7.0, 1.0], [1.0, 8.0, 6.0]]], [[[1.0, 7.0, 2.0], [3.0, 
6.0, 1.0], [4.0, 2.0, 1.0]], [[6.0, 2.0, 5.0], [6.0, 2.0, 7.0], [5.0, 2.0, 2.0]], [[3.0, 4.0, 7.0], [1.0, 5.0, 8.0], [3.0, 7.0, 
2.0]]], [[[8.0, 4.0, 3.0], [4.0, 4.0, 5.0], [4.0, 2.0, 4.0]], [[6.0, 4.0, 8.0], [3.0, 3.0, 4.0], [5.0, 4.0, 1.0]], [[7.0, 6.0, 
2.0], [8.0, 5.0, 8.0], [3.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 8.0], [1.0, 6.0, 2.0], [1.0, 6.0, 1.0]], 
[[5.0, 4.0, 3.0], [2.0, 5.0, 2.0], [4.0, 4.0, 3.0]], [[4.0, 6.0, 2.0], [4.0, 1.0, 3.0], [8.0, 6.0, 2.0]]], [[[1.0, 5.0, 8.0], 
[4.0, 5.0, 5.0], [1.0, 6.0, 1.0]], [[3.0, 1.0, 8.0], [7.0, 6.0, 6.0], [1.0, 1.0, 6.0]], [[5.0, 8.0, 8.0], [1.0, 7.0, 1.0], 
[1.0, 8.0, 6.0]]], [[[1.0, 7.0, 2.0], [6.0, 6.0, 1.0], [4.0, 2.0, 1.0]], [[6.0, 2.0, 5.0], [5.0, 3.0, 6.0], [5.0, 2.0, 2.0]], 
[[3.0, 4.0, 7.0], [8.0, 1.0, 5.0], [3.0, 7.0, 2.0]]], [[[8.0, 4.0, 3.0], [8.0, 3.0, 1.0], [4.0, 2.0, 4.0]], [[6.0, 4.0, 8.0], 
[5.0, 7.0, 3.0], [5.0, 4.0, 1.0]], [[7.0, 6.0, 2.0], [1.0, 1.0, 6.0], [3.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULPN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 7.0], [5.0, 7.0, 5.0], [2.0, 2.0, 3.0]], [[4.0, 6.0, 8.0], [6.0, 3.0, 8.0], [8.0, 4.0, 
1.0]], [[1.0, 7.0, 8.0], [6.0, 5.0, 1.0], [1.0, 3.0, 2.0]]], [[[5.0, 7.0, 7.0], [7.0, 3.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 1.0, 
5.0], [7.0, 1.0, 2.0], [5.0, 2.0, 1.0]], [[8.0, 5.0, 2.0], [2.0, 6.0, 5.0], [1.0, 3.0, 7.0]]], [[[6.0, 8.0, 3.0], [1.0, 5.0, 
5.0], [5.0, 8.0, 1.0]], [[1.0, 7.0, 1.0], [8.0, 5.0, 3.0], [6.0, 7.0, 5.0]], [[7.0, 6.0, 6.0], [6.0, 6.0, 8.0], [8.0, 2.0, 
2.0]]], [[[5.0, 1.0, 5.0], [3.0, 7.0, 6.0], [1.0, 7.0, 6.0]], [[7.0, 5.0, 5.0], [1.0, 6.0, 6.0], [4.0, 1.0, 5.0]], [[6.0, 3.0, 
3.0], [6.0, 2.0, 1.0], [7.0, 2.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 8.0, 7.0]], [[4.0, 1.0, 2.0]], [[7.0, 2.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 4.0, 3.0]], [[8.0, 1.0, 4.0]], [[8.0, 8.0, 8.0]]]))
      arg[:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 7.0], [5.0, 7.0, 5.0], [2.0, 2.0, 3.0]], [[4.0, 6.0, 8.0], [6.0, 3.0, 8.0], [8.0, 
4.0, 1.0]], [[1.0, 7.0, 8.0], [7.0, 8.0, 7.0], [1.0, 3.0, 2.0]]], [[[5.0, 7.0, 7.0], [7.0, 3.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 
1.0, 5.0], [7.0, 1.0, 2.0], [5.0, 2.0, 1.0]], [[8.0, 5.0, 2.0], [4.0, 1.0, 2.0], [1.0, 3.0, 7.0]]], [[[6.0, 8.0, 3.0], [1.0, 
5.0, 5.0], [5.0, 8.0, 1.0]], [[1.0, 7.0, 1.0], [8.0, 5.0, 3.0], [6.0, 7.0, 5.0]], [[7.0, 6.0, 6.0], [7.0, 2.0, 3.0], [8.0, 2.0, 
2.0]]], [[[5.0, 1.0, 5.0], [3.0, 7.0, 6.0], [1.0, 7.0, 6.0]], [[7.0, 5.0, 5.0], [1.0, 6.0, 6.0], [4.0, 1.0, 5.0]], [[6.0, 3.0, 
3.0], [6.0, 2.0, 1.0], [7.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 7.0], [5.0, 7.0, 5.0], [2.0, 2.0, 3.0]], 
[[4.0, 6.0, 8.0], [6.0, 3.0, 8.0], [8.0, 4.0, 1.0]], [[1.0, 7.0, 8.0], [3.0, 4.0, 3.0], [1.0, 3.0, 2.0]]], [[[5.0, 7.0, 7.0], 
[7.0, 3.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 1.0, 5.0], [7.0, 1.0, 2.0], [5.0, 2.0, 1.0]], [[8.0, 5.0, 2.0], [8.0, 1.0, 4.0], 
[1.0, 3.0, 7.0]]], [[[6.0, 8.0, 3.0], [1.0, 5.0, 5.0], [5.0, 8.0, 1.0]], [[1.0, 7.0, 1.0], [8.0, 5.0, 3.0], [6.0, 7.0, 5.0]], 
[[7.0, 6.0, 6.0], [8.0, 8.0, 8.0], [8.0, 2.0, 2.0]]], [[[5.0, 1.0, 5.0], [3.0, 7.0, 6.0], [1.0, 7.0, 6.0]], [[7.0, 5.0, 5.0], 
[1.0, 6.0, 6.0], [4.0, 1.0, 5.0]], [[6.0, 3.0, 3.0], [6.0, 2.0, 1.0], [7.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULCU(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 8.0], [8.0, 5.0, 4.0], [2.0, 1.0, 8.0]], [[1.0, 6.0, 6.0], [8.0, 4.0, 2.0], [5.0, 3.0, 
7.0]], [[7.0, 4.0, 6.0], [2.0, 8.0, 5.0], [4.0, 5.0, 3.0]]], [[[4.0, 7.0, 8.0], [4.0, 1.0, 4.0], [8.0, 8.0, 6.0]], [[4.0, 3.0, 
3.0], [6.0, 8.0, 3.0], [8.0, 4.0, 7.0]], [[2.0, 5.0, 8.0], [5.0, 5.0, 6.0], [1.0, 6.0, 2.0]]], [[[8.0, 3.0, 3.0], [3.0, 5.0, 
5.0], [3.0, 2.0, 2.0]], [[1.0, 7.0, 8.0], [8.0, 1.0, 7.0], [8.0, 6.0, 6.0]], [[7.0, 3.0, 2.0], [3.0, 3.0, 3.0], [7.0, 6.0, 
1.0]]], [[[1.0, 6.0, 8.0], [7.0, 3.0, 7.0], [2.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], [4.0, 4.0, 3.0], [1.0, 5.0, 6.0]], [[3.0, 3.0, 
5.0], [7.0, 4.0, 3.0], [6.0, 6.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 1.0], [4.0, 4.0], [6.0, 5.0]]], [[[6.0, 1.0], [3.0, 6.0], [8.0, 5.0]]], [[[7.0, 4.0], [1.0, 
2.0], [5.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 2.0], [1.0, 8.0], [6.0, 2.0]]], [[[6.0, 6.0], [4.0, 3.0], [7.0, 6.0]]], [[[3.0, 
5.0], [2.0, 2.0], [7.0, 1.0]]]]))
      arg[:3,2:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 8.0], [8.0, 5.0, 4.0], [2.0, 1.0, 8.0]], [[1.0, 6.0, 6.0], [8.0, 4.0, 2.0], [5.0, 
3.0, 7.0]], [[5.0, 1.0, 6.0], [4.0, 4.0, 5.0], [6.0, 5.0, 3.0]]], [[[4.0, 7.0, 8.0], [4.0, 1.0, 4.0], [8.0, 8.0, 6.0]], [[4.0, 
3.0, 3.0], [6.0, 8.0, 3.0], [8.0, 4.0, 7.0]], [[6.0, 1.0, 8.0], [3.0, 6.0, 6.0], [8.0, 5.0, 2.0]]], [[[8.0, 3.0, 3.0], [3.0, 
5.0, 5.0], [3.0, 2.0, 2.0]], [[1.0, 7.0, 8.0], [8.0, 1.0, 7.0], [8.0, 6.0, 6.0]], [[7.0, 4.0, 2.0], [1.0, 2.0, 3.0], [5.0, 5.0, 
1.0]]], [[[1.0, 6.0, 8.0], [7.0, 3.0, 7.0], [2.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], [4.0, 4.0, 3.0], [1.0, 5.0, 6.0]], [[3.0, 3.0, 
5.0], [7.0, 4.0, 3.0], [6.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 8.0], [8.0, 5.0, 4.0], [2.0, 1.0, 8.0]], 
[[1.0, 6.0, 6.0], [8.0, 4.0, 2.0], [5.0, 3.0, 7.0]], [[5.0, 2.0, 6.0], [1.0, 8.0, 5.0], [6.0, 2.0, 3.0]]], [[[4.0, 7.0, 8.0], 
[4.0, 1.0, 4.0], [8.0, 8.0, 6.0]], [[4.0, 3.0, 3.0], [6.0, 8.0, 3.0], [8.0, 4.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 3.0, 6.0], 
[7.0, 6.0, 2.0]]], [[[8.0, 3.0, 3.0], [3.0, 5.0, 5.0], [3.0, 2.0, 2.0]], [[1.0, 7.0, 8.0], [8.0, 1.0, 7.0], [8.0, 6.0, 6.0]], 
[[3.0, 5.0, 2.0], [2.0, 2.0, 3.0], [7.0, 1.0, 1.0]]], [[[1.0, 6.0, 8.0], [7.0, 3.0, 7.0], [2.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], 
[4.0, 4.0, 3.0], [1.0, 5.0, 6.0]], [[3.0, 3.0, 5.0], [7.0, 4.0, 3.0], [6.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCLC(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 4.0], [7.0, 5.0, 3.0], [2.0, 7.0, 4.0]], [[2.0, 5.0, 2.0], [4.0, 8.0, 8.0], [2.0, 5.0, 
8.0]], [[5.0, 5.0, 4.0], [6.0, 8.0, 2.0], [6.0, 5.0, 7.0]]], [[[1.0, 3.0, 4.0], [7.0, 1.0, 8.0], [3.0, 4.0, 5.0]], [[2.0, 7.0, 
2.0], [5.0, 4.0, 2.0], [2.0, 2.0, 6.0]], [[8.0, 1.0, 1.0], [1.0, 8.0, 8.0], [1.0, 6.0, 6.0]]], [[[6.0, 2.0, 7.0], [8.0, 4.0, 
5.0], [7.0, 6.0, 1.0]], [[6.0, 5.0, 8.0], [4.0, 5.0, 7.0], [6.0, 4.0, 2.0]], [[5.0, 2.0, 6.0], [7.0, 6.0, 5.0], [5.0, 7.0, 
1.0]]], [[[5.0, 2.0, 7.0], [4.0, 3.0, 6.0], [1.0, 3.0, 5.0]], [[5.0, 7.0, 3.0], [7.0, 1.0, 4.0], [8.0, 5.0, 8.0]], [[5.0, 7.0, 
2.0], [7.0, 2.0, 3.0], [4.0, 4.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 3.0, 6.0]], [[2.0, 2.0, 4.0]], [[2.0, 6.0, 5.0]]], [[[7.0, 4.0, 8.0]], [[6.0, 2.0, 1.0]], 
[[2.0, 1.0, 5.0]]], [[[5.0, 7.0, 7.0]], [[7.0, 1.0, 7.0]], [[4.0, 2.0, 8.0]]], [[[3.0, 4.0, 8.0]], [[1.0, 6.0, 8.0]], [[1.0, 
6.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 4.0]], [[3.0, 2.0, 2.0]], [[3.0, 3.0, 2.0]]], [[[4.0, 6.0, 7.0]], [[6.0, 
4.0, 7.0]], [[8.0, 6.0, 3.0]]], [[[7.0, 8.0, 3.0]], [[2.0, 7.0, 2.0]], [[1.0, 4.0, 8.0]]], [[[7.0, 5.0, 6.0]], [[3.0, 8.0, 
7.0]], [[4.0, 1.0, 8.0]]]]))
      arg[:,:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 4.0], [7.0, 5.0, 3.0], [4.0, 3.0, 6.0]], [[2.0, 5.0, 2.0], [4.0, 8.0, 8.0], [2.0, 
2.0, 4.0]], [[5.0, 5.0, 4.0], [6.0, 8.0, 2.0], [2.0, 6.0, 5.0]]], [[[1.0, 3.0, 4.0], [7.0, 1.0, 8.0], [7.0, 4.0, 8.0]], [[2.0, 
7.0, 2.0], [5.0, 4.0, 2.0], [6.0, 2.0, 1.0]], [[8.0, 1.0, 1.0], [1.0, 8.0, 8.0], [2.0, 1.0, 5.0]]], [[[6.0, 2.0, 7.0], [8.0, 
4.0, 5.0], [5.0, 7.0, 7.0]], [[6.0, 5.0, 8.0], [4.0, 5.0, 7.0], [7.0, 1.0, 7.0]], [[5.0, 2.0, 6.0], [7.0, 6.0, 5.0], [4.0, 2.0, 
8.0]]], [[[5.0, 2.0, 7.0], [4.0, 3.0, 6.0], [3.0, 4.0, 8.0]], [[5.0, 7.0, 3.0], [7.0, 1.0, 4.0], [1.0, 6.0, 8.0]], [[5.0, 7.0, 
2.0], [7.0, 2.0, 3.0], [1.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 4.0], [7.0, 5.0, 3.0], [8.0, 7.0, 4.0]], 
[[2.0, 5.0, 2.0], [4.0, 8.0, 8.0], [3.0, 2.0, 2.0]], [[5.0, 5.0, 4.0], [6.0, 8.0, 2.0], [3.0, 3.0, 2.0]]], [[[1.0, 3.0, 4.0], 
[7.0, 1.0, 8.0], [4.0, 6.0, 7.0]], [[2.0, 7.0, 2.0], [5.0, 4.0, 2.0], [6.0, 4.0, 7.0]], [[8.0, 1.0, 1.0], [1.0, 8.0, 8.0], 
[8.0, 6.0, 3.0]]], [[[6.0, 2.0, 7.0], [8.0, 4.0, 5.0], [7.0, 8.0, 3.0]], [[6.0, 5.0, 8.0], [4.0, 5.0, 7.0], [2.0, 7.0, 2.0]], 
[[5.0, 2.0, 6.0], [7.0, 6.0, 5.0], [1.0, 4.0, 8.0]]], [[[5.0, 2.0, 7.0], [4.0, 3.0, 6.0], [7.0, 5.0, 6.0]], [[5.0, 7.0, 3.0], 
[7.0, 1.0, 4.0], [3.0, 8.0, 7.0]], [[5.0, 7.0, 2.0], [7.0, 2.0, 3.0], [4.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCLL(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 1.0], [4.0, 5.0, 6.0], [5.0, 7.0, 8.0]], [[6.0, 7.0, 4.0], [7.0, 2.0, 7.0], [2.0, 1.0, 
3.0]], [[4.0, 1.0, 4.0], [8.0, 4.0, 7.0], [4.0, 6.0, 2.0]]], [[[2.0, 5.0, 7.0], [5.0, 2.0, 2.0], [2.0, 4.0, 5.0]], [[3.0, 2.0, 
6.0], [5.0, 5.0, 8.0], [6.0, 3.0, 3.0]], [[7.0, 5.0, 5.0], [8.0, 1.0, 6.0], [6.0, 6.0, 5.0]]], [[[8.0, 5.0, 6.0], [8.0, 6.0, 
2.0], [6.0, 5.0, 5.0]], [[7.0, 8.0, 3.0], [6.0, 1.0, 5.0], [7.0, 3.0, 6.0]], [[6.0, 4.0, 4.0], [7.0, 1.0, 6.0], [4.0, 1.0, 
8.0]]], [[[8.0, 1.0, 3.0], [5.0, 7.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 6.0, 1.0], [1.0, 5.0, 7.0], [8.0, 4.0, 4.0]], [[4.0, 3.0, 
5.0], [3.0, 3.0, 7.0], [1.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0]], [[6.0]], [[1.0]]], [[[2.0]], [[1.0]], [[6.0]]], [[[8.0]], [[2.0]], [[7.0]]], [[[1.0]], 
[[4.0]], [[5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0]], [[6.0]], [[8.0]]], [[[4.0]], [[3.0]], [[3.0]]], [[[6.0]], [[3.0]], [[5.0]]], 
[[[4.0]], [[5.0]], [[1.0]]]]))
      arg[:,:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 1.0], [4.0, 5.0, 6.0], [5.0, 7.0, 7.0]], [[6.0, 7.0, 4.0], [7.0, 2.0, 7.0], [2.0, 
1.0, 6.0]], [[4.0, 1.0, 4.0], [8.0, 4.0, 7.0], [4.0, 6.0, 1.0]]], [[[2.0, 5.0, 7.0], [5.0, 2.0, 2.0], [2.0, 4.0, 2.0]], [[3.0, 
2.0, 6.0], [5.0, 5.0, 8.0], [6.0, 3.0, 1.0]], [[7.0, 5.0, 5.0], [8.0, 1.0, 6.0], [6.0, 6.0, 6.0]]], [[[8.0, 5.0, 6.0], [8.0, 
6.0, 2.0], [6.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [6.0, 1.0, 5.0], [7.0, 3.0, 2.0]], [[6.0, 4.0, 4.0], [7.0, 1.0, 6.0], [4.0, 1.0, 
7.0]]], [[[8.0, 1.0, 3.0], [5.0, 7.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 6.0, 1.0], [1.0, 5.0, 7.0], [8.0, 4.0, 4.0]], [[4.0, 3.0, 
5.0], [3.0, 3.0, 7.0], [1.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 1.0], [4.0, 5.0, 6.0], [5.0, 7.0, 5.0]], 
[[6.0, 7.0, 4.0], [7.0, 2.0, 7.0], [2.0, 1.0, 6.0]], [[4.0, 1.0, 4.0], [8.0, 4.0, 7.0], [4.0, 6.0, 8.0]]], [[[2.0, 5.0, 7.0], 
[5.0, 2.0, 2.0], [2.0, 4.0, 4.0]], [[3.0, 2.0, 6.0], [5.0, 5.0, 8.0], [6.0, 3.0, 3.0]], [[7.0, 5.0, 5.0], [8.0, 1.0, 6.0], 
[6.0, 6.0, 3.0]]], [[[8.0, 5.0, 6.0], [8.0, 6.0, 2.0], [6.0, 5.0, 6.0]], [[7.0, 8.0, 3.0], [6.0, 1.0, 5.0], [7.0, 3.0, 3.0]], 
[[6.0, 4.0, 4.0], [7.0, 1.0, 6.0], [4.0, 1.0, 5.0]]], [[[8.0, 1.0, 3.0], [5.0, 7.0, 3.0], [2.0, 4.0, 4.0]], [[3.0, 6.0, 1.0], 
[1.0, 5.0, 7.0], [8.0, 4.0, 5.0]], [[4.0, 3.0, 5.0], [3.0, 3.0, 7.0], [1.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLLC(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 1.0], [2.0, 4.0, 5.0], [7.0, 6.0, 2.0]], [[1.0, 7.0, 1.0], [6.0, 8.0, 1.0], [8.0, 6.0, 
2.0]], [[7.0, 8.0, 6.0], [6.0, 6.0, 3.0], [5.0, 4.0, 6.0]]], [[[1.0, 8.0, 1.0], [7.0, 4.0, 1.0], [5.0, 3.0, 8.0]], [[5.0, 1.0, 
4.0], [3.0, 2.0, 7.0], [5.0, 8.0, 3.0]], [[4.0, 1.0, 1.0], [7.0, 7.0, 3.0], [7.0, 4.0, 5.0]]], [[[2.0, 1.0, 2.0], [5.0, 2.0, 
4.0], [4.0, 1.0, 3.0]], [[1.0, 1.0, 7.0], [3.0, 1.0, 2.0], [6.0, 1.0, 1.0]], [[4.0, 4.0, 5.0], [5.0, 5.0, 2.0], [1.0, 8.0, 
8.0]]], [[[5.0, 6.0, 5.0], [7.0, 2.0, 1.0], [8.0, 6.0, 6.0]], [[2.0, 1.0, 6.0], [4.0, 8.0, 3.0], [2.0, 1.0, 4.0]], [[5.0, 1.0, 
3.0], [3.0, 8.0, 1.0], [3.0, 4.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 2.0, 1.0]]], [[[3.0, 1.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 4.0]]], [[[1.0, 4.0, 5.0]]]]))
      arg[1:3,2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 1.0], [2.0, 4.0, 5.0], [7.0, 6.0, 2.0]], [[1.0, 7.0, 1.0], [6.0, 8.0, 1.0], [8.0, 
6.0, 2.0]], [[7.0, 8.0, 6.0], [6.0, 6.0, 3.0], [5.0, 4.0, 6.0]]], [[[1.0, 8.0, 1.0], [7.0, 4.0, 1.0], [5.0, 3.0, 8.0]], [[5.0, 
1.0, 4.0], [3.0, 2.0, 7.0], [5.0, 8.0, 3.0]], [[4.0, 1.0, 1.0], [7.0, 7.0, 3.0], [6.0, 2.0, 1.0]]], [[[2.0, 1.0, 2.0], [5.0, 
2.0, 4.0], [4.0, 1.0, 3.0]], [[1.0, 1.0, 7.0], [3.0, 1.0, 2.0], [6.0, 1.0, 1.0]], [[4.0, 4.0, 5.0], [5.0, 5.0, 2.0], [3.0, 1.0, 
1.0]]], [[[5.0, 6.0, 5.0], [7.0, 2.0, 1.0], [8.0, 6.0, 6.0]], [[2.0, 1.0, 6.0], [4.0, 8.0, 3.0], [2.0, 1.0, 4.0]], [[5.0, 1.0, 
3.0], [3.0, 8.0, 1.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 1.0], [2.0, 4.0, 5.0], [7.0, 6.0, 2.0]], 
[[1.0, 7.0, 1.0], [6.0, 8.0, 1.0], [8.0, 6.0, 2.0]], [[7.0, 8.0, 6.0], [6.0, 6.0, 3.0], [5.0, 4.0, 6.0]]], [[[1.0, 8.0, 1.0], 
[7.0, 4.0, 1.0], [5.0, 3.0, 8.0]], [[5.0, 1.0, 4.0], [3.0, 2.0, 7.0], [5.0, 8.0, 3.0]], [[4.0, 1.0, 1.0], [7.0, 7.0, 3.0], 
[7.0, 1.0, 4.0]]], [[[2.0, 1.0, 2.0], [5.0, 2.0, 4.0], [4.0, 1.0, 3.0]], [[1.0, 1.0, 7.0], [3.0, 1.0, 2.0], [6.0, 1.0, 1.0]], 
[[4.0, 4.0, 5.0], [5.0, 5.0, 2.0], [1.0, 4.0, 5.0]]], [[[5.0, 6.0, 5.0], [7.0, 2.0, 1.0], [8.0, 6.0, 6.0]], [[2.0, 1.0, 6.0], 
[4.0, 8.0, 3.0], [2.0, 1.0, 4.0]], [[5.0, 1.0, 3.0], [3.0, 8.0, 1.0], [3.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LRLL(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 7.0], [7.0, 5.0, 1.0], [3.0, 1.0, 3.0]], [[6.0, 5.0, 2.0], [8.0, 8.0, 2.0], [5.0, 3.0, 
2.0]], [[5.0, 6.0, 1.0], [1.0, 4.0, 4.0], [3.0, 3.0, 2.0]]], [[[4.0, 5.0, 3.0], [8.0, 6.0, 1.0], [6.0, 7.0, 5.0]], [[1.0, 5.0, 
7.0], [7.0, 7.0, 6.0], [1.0, 5.0, 7.0]], [[8.0, 3.0, 1.0], [5.0, 4.0, 5.0], [6.0, 4.0, 4.0]]], [[[2.0, 5.0, 6.0], [8.0, 1.0, 
4.0], [4.0, 7.0, 2.0]], [[1.0, 2.0, 2.0], [1.0, 5.0, 7.0], [6.0, 4.0, 3.0]], [[6.0, 5.0, 5.0], [3.0, 7.0, 4.0], [2.0, 8.0, 
1.0]]], [[[7.0, 6.0, 6.0], [4.0, 5.0, 8.0], [8.0, 6.0, 7.0]], [[7.0, 1.0, 6.0], [7.0, 4.0, 5.0], [4.0, 7.0, 3.0]], [[8.0, 1.0, 
5.0], [2.0, 4.0, 3.0], [2.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0]]], [[[1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0]]], [[[6.0]]]]))
      arg[2:,1:2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 7.0], [7.0, 5.0, 1.0], [3.0, 1.0, 3.0]], [[6.0, 5.0, 2.0], [8.0, 8.0, 2.0], [5.0, 
3.0, 2.0]], [[5.0, 6.0, 1.0], [1.0, 4.0, 4.0], [3.0, 3.0, 2.0]]], [[[4.0, 5.0, 3.0], [8.0, 6.0, 1.0], [6.0, 7.0, 5.0]], [[1.0, 
5.0, 7.0], [7.0, 7.0, 6.0], [1.0, 5.0, 7.0]], [[8.0, 3.0, 1.0], [5.0, 4.0, 5.0], [6.0, 4.0, 4.0]]], [[[2.0, 5.0, 6.0], [8.0, 
1.0, 4.0], [4.0, 7.0, 2.0]], [[1.0, 2.0, 2.0], [1.0, 5.0, 7.0], [6.0, 4.0, 3.0]], [[6.0, 5.0, 5.0], [3.0, 7.0, 4.0], [2.0, 8.0, 
1.0]]], [[[7.0, 6.0, 6.0], [4.0, 5.0, 8.0], [8.0, 6.0, 7.0]], [[7.0, 1.0, 6.0], [7.0, 4.0, 5.0], [4.0, 7.0, 1.0]], [[8.0, 1.0, 
5.0], [2.0, 4.0, 3.0], [2.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 7.0], [7.0, 5.0, 1.0], [3.0, 1.0, 3.0]], 
[[6.0, 5.0, 2.0], [8.0, 8.0, 2.0], [5.0, 3.0, 2.0]], [[5.0, 6.0, 1.0], [1.0, 4.0, 4.0], [3.0, 3.0, 2.0]]], [[[4.0, 5.0, 3.0], 
[8.0, 6.0, 1.0], [6.0, 7.0, 5.0]], [[1.0, 5.0, 7.0], [7.0, 7.0, 6.0], [1.0, 5.0, 7.0]], [[8.0, 3.0, 1.0], [5.0, 4.0, 5.0], 
[6.0, 4.0, 4.0]]], [[[2.0, 5.0, 6.0], [8.0, 1.0, 4.0], [4.0, 7.0, 2.0]], [[1.0, 2.0, 2.0], [1.0, 5.0, 7.0], [6.0, 4.0, 4.0]], 
[[6.0, 5.0, 5.0], [3.0, 7.0, 4.0], [2.0, 8.0, 1.0]]], [[[7.0, 6.0, 6.0], [4.0, 5.0, 8.0], [8.0, 6.0, 7.0]], [[7.0, 1.0, 6.0], 
[7.0, 4.0, 5.0], [4.0, 7.0, 6.0]], [[8.0, 1.0, 5.0], [2.0, 4.0, 3.0], [2.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 2.0], [5.0, 7.0, 6.0], [7.0, 5.0, 3.0]], [[2.0, 4.0, 6.0], [1.0, 5.0, 8.0], [6.0, 3.0, 
4.0]], [[1.0, 7.0, 1.0], [3.0, 3.0, 4.0], [3.0, 5.0, 8.0]]], [[[1.0, 5.0, 8.0], [6.0, 7.0, 8.0], [8.0, 3.0, 1.0]], [[2.0, 7.0, 
6.0], [4.0, 7.0, 8.0], [6.0, 7.0, 5.0]], [[8.0, 3.0, 2.0], [7.0, 7.0, 1.0], [2.0, 7.0, 1.0]]], [[[6.0, 1.0, 3.0], [6.0, 2.0, 
2.0], [1.0, 3.0, 5.0]], [[4.0, 3.0, 1.0], [6.0, 2.0, 7.0], [4.0, 4.0, 8.0]], [[7.0, 7.0, 8.0], [5.0, 4.0, 6.0], [6.0, 3.0, 
4.0]]], [[[5.0, 3.0, 6.0], [6.0, 1.0, 6.0], [7.0, 7.0, 4.0]], [[2.0, 8.0, 8.0], [4.0, 4.0, 7.0], [7.0, 5.0, 3.0]], [[2.0, 8.0, 
1.0], [2.0, 6.0, 5.0], [3.0, 8.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 8.0, 8.0], [2.0, 6.0, 5.0], [1.0, 7.0, 5.0]], [[5.0, 5.0, 1.0], [7.0, 6.0, 5.0], [7.0, 3.0, 
2.0]], [[3.0, 3.0, 7.0], [4.0, 7.0, 3.0], [4.0, 4.0, 6.0]]], [[[6.0, 2.0, 5.0], [8.0, 7.0, 2.0], [6.0, 7.0, 6.0]], [[7.0, 3.0, 
2.0], [6.0, 8.0, 3.0], [4.0, 1.0, 3.0]], [[3.0, 4.0, 6.0], [7.0, 4.0, 3.0], [5.0, 2.0, 7.0]]], [[[8.0, 6.0, 1.0], [3.0, 6.0, 
1.0], [7.0, 7.0, 3.0]], [[4.0, 6.0, 8.0], [4.0, 1.0, 7.0], [8.0, 3.0, 4.0]], [[1.0, 4.0, 6.0], [7.0, 6.0, 5.0], [4.0, 5.0, 
4.0]]], [[[3.0, 2.0, 4.0], [2.0, 7.0, 4.0], [6.0, 2.0, 5.0]], [[4.0, 8.0, 4.0], [3.0, 8.0, 3.0], [6.0, 5.0, 7.0]], [[7.0, 6.0, 
3.0], [1.0, 1.0, 3.0], [2.0, 6.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 5.0], [6.0, 4.0, 7.0], [7.0, 8.0, 4.0]], [[8.0, 7.0, 7.0], [5.0, 2.0, 
4.0], [1.0, 3.0, 4.0]], [[5.0, 1.0, 2.0], [1.0, 1.0, 2.0], [4.0, 2.0, 6.0]]], [[[3.0, 3.0, 6.0], [8.0, 5.0, 3.0], [6.0, 1.0, 
7.0]], [[8.0, 6.0, 7.0], [1.0, 2.0, 2.0], [4.0, 6.0, 7.0]], [[3.0, 3.0, 8.0], [7.0, 3.0, 5.0], [1.0, 8.0, 8.0]]], [[[4.0, 5.0, 
5.0], [4.0, 5.0, 2.0], [5.0, 8.0, 4.0]], [[1.0, 6.0, 8.0], [3.0, 7.0, 3.0], [2.0, 5.0, 8.0]], [[4.0, 4.0, 4.0], [8.0, 7.0, 
8.0], [4.0, 5.0, 2.0]]], [[[6.0, 4.0, 5.0], [6.0, 2.0, 2.0], [6.0, 1.0, 6.0]], [[2.0, 5.0, 7.0], [1.0, 2.0, 8.0], [7.0, 2.0, 
3.0]], [[6.0, 5.0, 6.0], [7.0, 1.0, 4.0], [7.0, 3.0, 6.0]]]]))
      arg[:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 8.0], [2.0, 6.0, 5.0], [1.0, 7.0, 5.0]], [[5.0, 5.0, 1.0], [7.0, 6.0, 5.0], [7.0, 
3.0, 2.0]], [[3.0, 3.0, 7.0], [4.0, 7.0, 3.0], [4.0, 4.0, 6.0]]], [[[6.0, 2.0, 5.0], [8.0, 7.0, 2.0], [6.0, 7.0, 6.0]], [[7.0, 
3.0, 2.0], [6.0, 8.0, 3.0], [4.0, 1.0, 3.0]], [[3.0, 4.0, 6.0], [7.0, 4.0, 3.0], [5.0, 2.0, 7.0]]], [[[8.0, 6.0, 1.0], [3.0, 
6.0, 1.0], [7.0, 7.0, 3.0]], [[4.0, 6.0, 8.0], [4.0, 1.0, 7.0], [8.0, 3.0, 4.0]], [[1.0, 4.0, 6.0], [7.0, 6.0, 5.0], [4.0, 5.0, 
4.0]]], [[[3.0, 2.0, 4.0], [2.0, 7.0, 4.0], [6.0, 2.0, 5.0]], [[4.0, 8.0, 4.0], [3.0, 8.0, 3.0], [6.0, 5.0, 7.0]], [[7.0, 6.0, 
3.0], [1.0, 1.0, 3.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 5.0], [6.0, 4.0, 7.0], [7.0, 8.0, 4.0]], 
[[8.0, 7.0, 7.0], [5.0, 2.0, 4.0], [1.0, 3.0, 4.0]], [[5.0, 1.0, 2.0], [1.0, 1.0, 2.0], [4.0, 2.0, 6.0]]], [[[3.0, 3.0, 6.0], 
[8.0, 5.0, 3.0], [6.0, 1.0, 7.0]], [[8.0, 6.0, 7.0], [1.0, 2.0, 2.0], [4.0, 6.0, 7.0]], [[3.0, 3.0, 8.0], [7.0, 3.0, 5.0], 
[1.0, 8.0, 8.0]]], [[[4.0, 5.0, 5.0], [4.0, 5.0, 2.0], [5.0, 8.0, 4.0]], [[1.0, 6.0, 8.0], [3.0, 7.0, 3.0], [2.0, 5.0, 8.0]], 
[[4.0, 4.0, 4.0], [8.0, 7.0, 8.0], [4.0, 5.0, 2.0]]], [[[6.0, 4.0, 5.0], [6.0, 2.0, 2.0], [6.0, 1.0, 6.0]], [[2.0, 5.0, 7.0], 
[1.0, 2.0, 8.0], [7.0, 2.0, 3.0]], [[6.0, 5.0, 6.0], [7.0, 1.0, 4.0], [7.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RUCN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 7.0], [2.0, 5.0, 2.0], [6.0, 8.0, 1.0]], [[3.0, 5.0, 5.0], [7.0, 6.0, 8.0], [2.0, 8.0, 
7.0]], [[7.0, 4.0, 2.0], [3.0, 8.0, 8.0], [3.0, 2.0, 3.0]]], [[[5.0, 4.0, 3.0], [4.0, 1.0, 3.0], [5.0, 5.0, 5.0]], [[5.0, 2.0, 
3.0], [1.0, 5.0, 8.0], [8.0, 4.0, 2.0]], [[4.0, 3.0, 1.0], [7.0, 3.0, 7.0], [3.0, 7.0, 6.0]]], [[[7.0, 8.0, 6.0], [1.0, 2.0, 
5.0], [5.0, 4.0, 4.0]], [[7.0, 6.0, 6.0], [5.0, 7.0, 3.0], [4.0, 1.0, 5.0]], [[4.0, 8.0, 2.0], [4.0, 8.0, 6.0], [1.0, 7.0, 
7.0]]], [[[7.0, 6.0, 8.0], [6.0, 6.0, 8.0], [1.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], [6.0, 5.0, 8.0], [2.0, 8.0, 8.0]], [[8.0, 6.0, 
6.0], [4.0, 7.0, 2.0], [2.0, 1.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 3.0, 4.0], [6.0, 1.0, 8.0], [1.0, 8.0, 6.0]], [[1.0, 5.0, 8.0], [3.0, 5.0, 8.0], [4.0, 5.0, 
7.0]]], [[[2.0, 1.0, 7.0], [8.0, 4.0, 8.0], [6.0, 1.0, 5.0]], [[7.0, 2.0, 7.0], [5.0, 5.0, 7.0], [6.0, 2.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 5.0], [1.0, 3.0, 3.0], [8.0, 3.0, 6.0]], [[6.0, 5.0, 5.0], [2.0, 3.0, 
3.0], [5.0, 1.0, 4.0]]], [[[1.0, 7.0, 1.0], [1.0, 3.0, 6.0], [3.0, 3.0, 4.0]], [[6.0, 8.0, 5.0], [8.0, 1.0, 8.0], [2.0, 3.0, 
6.0]]]]))
      arg[1:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 7.0], [2.0, 5.0, 2.0], [6.0, 8.0, 1.0]], [[3.0, 5.0, 5.0], [7.0, 6.0, 8.0], [2.0, 
8.0, 7.0]], [[7.0, 4.0, 2.0], [3.0, 8.0, 8.0], [3.0, 2.0, 3.0]]], [[[5.0, 3.0, 4.0], [6.0, 1.0, 8.0], [1.0, 8.0, 6.0]], [[1.0, 
5.0, 8.0], [3.0, 5.0, 8.0], [4.0, 5.0, 7.0]], [[4.0, 3.0, 1.0], [7.0, 3.0, 7.0], [3.0, 7.0, 6.0]]], [[[2.0, 1.0, 7.0], [8.0, 
4.0, 8.0], [6.0, 1.0, 5.0]], [[7.0, 2.0, 7.0], [5.0, 5.0, 7.0], [6.0, 2.0, 2.0]], [[4.0, 8.0, 2.0], [4.0, 8.0, 6.0], [1.0, 7.0, 
7.0]]], [[[7.0, 6.0, 8.0], [6.0, 6.0, 8.0], [1.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], [6.0, 5.0, 8.0], [2.0, 8.0, 8.0]], [[8.0, 6.0, 
6.0], [4.0, 7.0, 2.0], [2.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 7.0], [2.0, 5.0, 2.0], [6.0, 8.0, 1.0]], 
[[3.0, 5.0, 5.0], [7.0, 6.0, 8.0], [2.0, 8.0, 7.0]], [[7.0, 4.0, 2.0], [3.0, 8.0, 8.0], [3.0, 2.0, 3.0]]], [[[4.0, 6.0, 5.0], 
[1.0, 3.0, 3.0], [8.0, 3.0, 6.0]], [[6.0, 5.0, 5.0], [2.0, 3.0, 3.0], [5.0, 1.0, 4.0]], [[4.0, 3.0, 1.0], [7.0, 3.0, 7.0], 
[3.0, 7.0, 6.0]]], [[[1.0, 7.0, 1.0], [1.0, 3.0, 6.0], [3.0, 3.0, 4.0]], [[6.0, 8.0, 5.0], [8.0, 1.0, 8.0], [2.0, 3.0, 6.0]], 
[[4.0, 8.0, 2.0], [4.0, 8.0, 6.0], [1.0, 7.0, 7.0]]], [[[7.0, 6.0, 8.0], [6.0, 6.0, 8.0], [1.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], 
[6.0, 5.0, 8.0], [2.0, 8.0, 8.0]], [[8.0, 6.0, 6.0], [4.0, 7.0, 2.0], [2.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PRUN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 6.0], [6.0, 4.0, 7.0], [4.0, 2.0, 4.0]], [[4.0, 1.0, 8.0], [8.0, 1.0, 2.0], [6.0, 3.0, 
6.0]], [[3.0, 2.0, 4.0], [5.0, 6.0, 6.0], [4.0, 2.0, 8.0]]], [[[1.0, 5.0, 4.0], [2.0, 1.0, 2.0], [4.0, 2.0, 7.0]], [[1.0, 7.0, 
6.0], [4.0, 2.0, 3.0], [4.0, 8.0, 4.0]], [[2.0, 6.0, 3.0], [4.0, 8.0, 6.0], [2.0, 4.0, 2.0]]], [[[4.0, 2.0, 2.0], [7.0, 7.0, 
5.0], [2.0, 2.0, 7.0]], [[3.0, 3.0, 5.0], [7.0, 7.0, 7.0], [5.0, 1.0, 7.0]], [[6.0, 2.0, 3.0], [7.0, 6.0, 2.0], [8.0, 6.0, 
5.0]]], [[[8.0, 5.0, 6.0], [6.0, 5.0, 7.0], [5.0, 4.0, 4.0]], [[6.0, 6.0, 4.0], [8.0, 2.0, 1.0], [7.0, 8.0, 5.0]], [[3.0, 2.0, 
7.0], [4.0, 5.0, 4.0], [2.0, 5.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 7.0, 2.0], [6.0, 8.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 1.0, 1.0], [8.0, 1.0, 1.0]]]))
      arg[2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 6.0], [6.0, 4.0, 7.0], [4.0, 2.0, 4.0]], [[4.0, 1.0, 8.0], [8.0, 1.0, 2.0], [6.0, 
3.0, 6.0]], [[3.0, 2.0, 4.0], [5.0, 6.0, 6.0], [4.0, 2.0, 8.0]]], [[[1.0, 5.0, 4.0], [2.0, 1.0, 2.0], [4.0, 2.0, 7.0]], [[1.0, 
7.0, 6.0], [4.0, 2.0, 3.0], [4.0, 8.0, 4.0]], [[2.0, 6.0, 3.0], [4.0, 8.0, 6.0], [2.0, 4.0, 2.0]]], [[[4.0, 2.0, 2.0], [7.0, 
7.0, 5.0], [2.0, 2.0, 7.0]], [[3.0, 7.0, 2.0], [6.0, 8.0, 2.0], [5.0, 1.0, 7.0]], [[6.0, 2.0, 3.0], [7.0, 6.0, 2.0], [8.0, 6.0, 
5.0]]], [[[8.0, 5.0, 6.0], [6.0, 5.0, 7.0], [5.0, 4.0, 4.0]], [[6.0, 6.0, 4.0], [8.0, 2.0, 1.0], [7.0, 8.0, 5.0]], [[3.0, 2.0, 
7.0], [4.0, 5.0, 4.0], [2.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 6.0], [6.0, 4.0, 7.0], [4.0, 2.0, 4.0]], 
[[4.0, 1.0, 8.0], [8.0, 1.0, 2.0], [6.0, 3.0, 6.0]], [[3.0, 2.0, 4.0], [5.0, 6.0, 6.0], [4.0, 2.0, 8.0]]], [[[1.0, 5.0, 4.0], 
[2.0, 1.0, 2.0], [4.0, 2.0, 7.0]], [[1.0, 7.0, 6.0], [4.0, 2.0, 3.0], [4.0, 8.0, 4.0]], [[2.0, 6.0, 3.0], [4.0, 8.0, 6.0], 
[2.0, 4.0, 2.0]]], [[[4.0, 2.0, 2.0], [7.0, 7.0, 5.0], [2.0, 2.0, 7.0]], [[6.0, 1.0, 1.0], [8.0, 1.0, 1.0], [5.0, 1.0, 7.0]], 
[[6.0, 2.0, 3.0], [7.0, 6.0, 2.0], [8.0, 6.0, 5.0]]], [[[8.0, 5.0, 6.0], [6.0, 5.0, 7.0], [5.0, 4.0, 4.0]], [[6.0, 6.0, 4.0], 
[8.0, 2.0, 1.0], [7.0, 8.0, 5.0]], [[3.0, 2.0, 7.0], [4.0, 5.0, 4.0], [2.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LULU(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 5.0], [2.0, 8.0, 5.0], [2.0, 5.0, 5.0]], [[6.0, 6.0, 7.0], [6.0, 3.0, 3.0], [3.0, 3.0, 
7.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 7.0], [3.0, 5.0, 4.0]]], [[[8.0, 7.0, 4.0], [2.0, 4.0, 1.0], [6.0, 1.0, 6.0]], [[6.0, 6.0, 
4.0], [4.0, 4.0, 2.0], [4.0, 2.0, 4.0]], [[5.0, 6.0, 4.0], [1.0, 5.0, 7.0], [6.0, 4.0, 8.0]]], [[[1.0, 5.0, 4.0], [8.0, 4.0, 
6.0], [8.0, 7.0, 8.0]], [[6.0, 8.0, 2.0], [2.0, 4.0, 1.0], [4.0, 5.0, 6.0]], [[8.0, 2.0, 8.0], [8.0, 1.0, 7.0], [3.0, 1.0, 
3.0]]], [[[7.0, 5.0, 1.0], [6.0, 6.0, 7.0], [7.0, 1.0, 5.0]], [[5.0, 7.0, 8.0], [8.0, 3.0, 4.0], [8.0, 2.0, 4.0]], [[7.0, 5.0, 
3.0], [4.0, 2.0, 1.0], [6.0, 3.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 4.0]], [[2.0, 6.0]]], [[[2.0, 3.0]], [[5.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 3.0]], [[6.0, 8.0]]], [[[4.0, 5.0]], [[3.0, 7.0]]]]))
      arg[2:,:2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 5.0], [2.0, 8.0, 5.0], [2.0, 5.0, 5.0]], [[6.0, 6.0, 7.0], [6.0, 3.0, 3.0], [3.0, 
3.0, 7.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 7.0], [3.0, 5.0, 4.0]]], [[[8.0, 7.0, 4.0], [2.0, 4.0, 1.0], [6.0, 1.0, 6.0]], [[6.0, 
6.0, 4.0], [4.0, 4.0, 2.0], [4.0, 2.0, 4.0]], [[5.0, 6.0, 4.0], [1.0, 5.0, 7.0], [6.0, 4.0, 8.0]]], [[[1.0, 5.0, 4.0], [8.0, 
4.0, 6.0], [8.0, 4.0, 8.0]], [[6.0, 8.0, 2.0], [2.0, 4.0, 1.0], [2.0, 6.0, 6.0]], [[8.0, 2.0, 8.0], [8.0, 1.0, 7.0], [3.0, 1.0, 
3.0]]], [[[7.0, 5.0, 1.0], [6.0, 6.0, 7.0], [2.0, 3.0, 5.0]], [[5.0, 7.0, 8.0], [8.0, 3.0, 4.0], [5.0, 3.0, 4.0]], [[7.0, 5.0, 
3.0], [4.0, 2.0, 1.0], [6.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 5.0], [2.0, 8.0, 5.0], [2.0, 5.0, 5.0]], 
[[6.0, 6.0, 7.0], [6.0, 3.0, 3.0], [3.0, 3.0, 7.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 7.0], [3.0, 5.0, 4.0]]], [[[8.0, 7.0, 4.0], 
[2.0, 4.0, 1.0], [6.0, 1.0, 6.0]], [[6.0, 6.0, 4.0], [4.0, 4.0, 2.0], [4.0, 2.0, 4.0]], [[5.0, 6.0, 4.0], [1.0, 5.0, 7.0], 
[6.0, 4.0, 8.0]]], [[[1.0, 5.0, 4.0], [8.0, 4.0, 6.0], [8.0, 3.0, 8.0]], [[6.0, 8.0, 2.0], [2.0, 4.0, 1.0], [6.0, 8.0, 6.0]], 
[[8.0, 2.0, 8.0], [8.0, 1.0, 7.0], [3.0, 1.0, 3.0]]], [[[7.0, 5.0, 1.0], [6.0, 6.0, 7.0], [4.0, 5.0, 5.0]], [[5.0, 7.0, 8.0], 
[8.0, 3.0, 4.0], [3.0, 7.0, 4.0]], [[7.0, 5.0, 3.0], [4.0, 2.0, 1.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCPU(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 8.0], [4.0, 5.0, 2.0], [8.0, 1.0, 7.0]], [[2.0, 6.0, 7.0], [2.0, 2.0, 1.0], [2.0, 8.0, 
7.0]], [[8.0, 8.0, 6.0], [3.0, 8.0, 5.0], [5.0, 8.0, 8.0]]], [[[4.0, 5.0, 5.0], [7.0, 4.0, 2.0], [2.0, 4.0, 5.0]], [[8.0, 1.0, 
3.0], [1.0, 3.0, 2.0], [8.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [5.0, 5.0, 5.0], [4.0, 3.0, 6.0]]], [[[5.0, 6.0, 2.0], [2.0, 5.0, 
1.0], [6.0, 4.0, 2.0]], [[8.0, 7.0, 1.0], [7.0, 3.0, 7.0], [2.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [8.0, 5.0, 8.0], [2.0, 3.0, 
2.0]]], [[[1.0, 1.0, 1.0], [6.0, 3.0, 3.0], [1.0, 5.0, 1.0]], [[1.0, 4.0, 1.0], [2.0, 2.0, 2.0], [7.0, 7.0, 1.0]], [[8.0, 3.0, 
7.0], [2.0, 7.0, 8.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 5.0], [8.0, 1.0], [5.0, 7.0]], [[5.0, 3.0], [7.0, 2.0], [7.0, 8.0]], [[2.0, 5.0], [4.0, 
2.0], [4.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 5.0], [7.0, 6.0], [6.0, 3.0]], [[5.0, 3.0], [1.0, 7.0], [1.0, 6.0]], [[7.0, 
2.0], [2.0, 7.0], [7.0, 5.0]]]))
      arg[:3,:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 8.0], [7.0, 5.0, 2.0], [8.0, 1.0, 7.0]], [[2.0, 6.0, 7.0], [8.0, 1.0, 1.0], [2.0, 
8.0, 7.0]], [[8.0, 8.0, 6.0], [5.0, 7.0, 5.0], [5.0, 8.0, 8.0]]], [[[4.0, 5.0, 5.0], [5.0, 3.0, 2.0], [2.0, 4.0, 5.0]], [[8.0, 
1.0, 3.0], [7.0, 2.0, 2.0], [8.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [7.0, 8.0, 5.0], [4.0, 3.0, 6.0]]], [[[5.0, 6.0, 2.0], [2.0, 
5.0, 1.0], [6.0, 4.0, 2.0]], [[8.0, 7.0, 1.0], [4.0, 2.0, 7.0], [2.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [4.0, 3.0, 8.0], [2.0, 3.0, 
2.0]]], [[[1.0, 1.0, 1.0], [6.0, 3.0, 3.0], [1.0, 5.0, 1.0]], [[1.0, 4.0, 1.0], [2.0, 2.0, 2.0], [7.0, 7.0, 1.0]], [[8.0, 3.0, 
7.0], [2.0, 7.0, 8.0], [3.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 8.0], [5.0, 5.0, 2.0], [8.0, 1.0, 7.0]], 
[[2.0, 6.0, 7.0], [7.0, 6.0, 1.0], [2.0, 8.0, 7.0]], [[8.0, 8.0, 6.0], [6.0, 3.0, 5.0], [5.0, 8.0, 8.0]]], [[[4.0, 5.0, 5.0], 
[5.0, 3.0, 2.0], [2.0, 4.0, 5.0]], [[8.0, 1.0, 3.0], [1.0, 7.0, 2.0], [8.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [1.0, 6.0, 5.0], 
[4.0, 3.0, 6.0]]], [[[5.0, 6.0, 2.0], [7.0, 2.0, 1.0], [6.0, 4.0, 2.0]], [[8.0, 7.0, 1.0], [2.0, 7.0, 7.0], [2.0, 5.0, 4.0]], 
[[3.0, 8.0, 5.0], [7.0, 5.0, 8.0], [2.0, 3.0, 2.0]]], [[[1.0, 1.0, 1.0], [6.0, 3.0, 3.0], [1.0, 5.0, 1.0]], [[1.0, 4.0, 1.0], 
[2.0, 2.0, 2.0], [7.0, 7.0, 1.0]], [[8.0, 3.0, 7.0], [2.0, 7.0, 8.0], [3.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLRL(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 1.0], [5.0, 7.0, 2.0], [6.0, 5.0, 2.0]], [[1.0, 4.0, 8.0], [3.0, 5.0, 2.0], [1.0, 4.0, 
4.0]], [[1.0, 3.0, 6.0], [3.0, 6.0, 7.0], [7.0, 8.0, 7.0]]], [[[4.0, 8.0, 1.0], [1.0, 5.0, 1.0], [4.0, 6.0, 2.0]], [[4.0, 5.0, 
6.0], [4.0, 7.0, 1.0], [7.0, 6.0, 8.0]], [[2.0, 4.0, 2.0], [5.0, 2.0, 8.0], [8.0, 7.0, 2.0]]], [[[1.0, 5.0, 3.0], [7.0, 3.0, 
3.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 2.0], [6.0, 5.0, 2.0], [5.0, 3.0, 6.0]], [[1.0, 2.0, 5.0], [6.0, 3.0, 5.0], [5.0, 1.0, 
2.0]]], [[[7.0, 6.0, 6.0], [8.0, 4.0, 2.0], [2.0, 6.0, 7.0]], [[2.0, 6.0, 2.0], [7.0, 2.0, 1.0], [7.0, 1.0, 3.0]], [[1.0, 5.0, 
2.0], [7.0, 2.0, 3.0], [4.0, 2.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0]]], [[[1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0]]], [[[2.0]]]]))
      arg[1:3,2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 1.0], [5.0, 7.0, 2.0], [6.0, 5.0, 2.0]], [[1.0, 4.0, 8.0], [3.0, 5.0, 2.0], [1.0, 
4.0, 4.0]], [[1.0, 3.0, 6.0], [3.0, 6.0, 7.0], [7.0, 8.0, 7.0]]], [[[4.0, 8.0, 1.0], [1.0, 5.0, 1.0], [4.0, 6.0, 2.0]], [[4.0, 
5.0, 6.0], [4.0, 7.0, 1.0], [7.0, 6.0, 8.0]], [[2.0, 4.0, 2.0], [5.0, 2.0, 2.0], [8.0, 7.0, 2.0]]], [[[1.0, 5.0, 3.0], [7.0, 
3.0, 3.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 2.0], [6.0, 5.0, 2.0], [5.0, 3.0, 6.0]], [[1.0, 2.0, 5.0], [6.0, 3.0, 1.0], [5.0, 1.0, 
2.0]]], [[[7.0, 6.0, 6.0], [8.0, 4.0, 2.0], [2.0, 6.0, 7.0]], [[2.0, 6.0, 2.0], [7.0, 2.0, 1.0], [7.0, 1.0, 3.0]], [[1.0, 5.0, 
2.0], [7.0, 2.0, 3.0], [4.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 1.0], [5.0, 7.0, 2.0], [6.0, 5.0, 2.0]], 
[[1.0, 4.0, 8.0], [3.0, 5.0, 2.0], [1.0, 4.0, 4.0]], [[1.0, 3.0, 6.0], [3.0, 6.0, 7.0], [7.0, 8.0, 7.0]]], [[[4.0, 8.0, 1.0], 
[1.0, 5.0, 1.0], [4.0, 6.0, 2.0]], [[4.0, 5.0, 6.0], [4.0, 7.0, 1.0], [7.0, 6.0, 8.0]], [[2.0, 4.0, 2.0], [5.0, 2.0, 1.0], 
[8.0, 7.0, 2.0]]], [[[1.0, 5.0, 3.0], [7.0, 3.0, 3.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 2.0], [6.0, 5.0, 2.0], [5.0, 3.0, 6.0]], 
[[1.0, 2.0, 5.0], [6.0, 3.0, 2.0], [5.0, 1.0, 2.0]]], [[[7.0, 6.0, 6.0], [8.0, 4.0, 2.0], [2.0, 6.0, 7.0]], [[2.0, 6.0, 2.0], 
[7.0, 2.0, 1.0], [7.0, 1.0, 3.0]], [[1.0, 5.0, 2.0], [7.0, 2.0, 3.0], [4.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PCLN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 6.0], [6.0, 1.0, 5.0], [8.0, 3.0, 6.0]], [[2.0, 5.0, 8.0], [7.0, 2.0, 2.0], [7.0, 6.0, 
7.0]], [[7.0, 5.0, 8.0], [4.0, 2.0, 8.0], [6.0, 3.0, 7.0]]], [[[6.0, 1.0, 4.0], [2.0, 4.0, 2.0], [2.0, 8.0, 6.0]], [[7.0, 5.0, 
6.0], [6.0, 4.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [6.0, 7.0, 2.0], [4.0, 1.0, 2.0]]], [[[4.0, 7.0, 1.0], [2.0, 3.0, 
6.0], [6.0, 2.0, 3.0]], [[6.0, 6.0, 3.0], [2.0, 3.0, 6.0], [3.0, 7.0, 7.0]], [[6.0, 3.0, 8.0], [1.0, 3.0, 6.0], [4.0, 5.0, 
7.0]]], [[[4.0, 4.0, 7.0], [1.0, 2.0, 3.0], [2.0, 7.0, 2.0]], [[2.0, 3.0, 5.0], [2.0, 1.0, 7.0], [6.0, 8.0, 2.0]], [[1.0, 5.0, 
6.0], [7.0, 5.0, 5.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 3.0, 2.0]], [[8.0, 4.0, 7.0]], [[7.0, 2.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 8.0, 8.0]], [[5.0, 4.0, 2.0]], [[4.0, 2.0, 6.0]]]))
      arg[2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 6.0], [6.0, 1.0, 5.0], [8.0, 3.0, 6.0]], [[2.0, 5.0, 8.0], [7.0, 2.0, 2.0], [7.0, 
6.0, 7.0]], [[7.0, 5.0, 8.0], [4.0, 2.0, 8.0], [6.0, 3.0, 7.0]]], [[[6.0, 1.0, 4.0], [2.0, 4.0, 2.0], [2.0, 8.0, 6.0]], [[7.0, 
5.0, 6.0], [6.0, 4.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [6.0, 7.0, 2.0], [4.0, 1.0, 2.0]]], [[[4.0, 7.0, 1.0], [2.0, 
3.0, 6.0], [8.0, 3.0, 2.0]], [[6.0, 6.0, 3.0], [2.0, 3.0, 6.0], [8.0, 4.0, 7.0]], [[6.0, 3.0, 8.0], [1.0, 3.0, 6.0], [7.0, 2.0, 
3.0]]], [[[4.0, 4.0, 7.0], [1.0, 2.0, 3.0], [2.0, 7.0, 2.0]], [[2.0, 3.0, 5.0], [2.0, 1.0, 7.0], [6.0, 8.0, 2.0]], [[1.0, 5.0, 
6.0], [7.0, 5.0, 5.0], [6.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 6.0], [6.0, 1.0, 5.0], [8.0, 3.0, 6.0]], 
[[2.0, 5.0, 8.0], [7.0, 2.0, 2.0], [7.0, 6.0, 7.0]], [[7.0, 5.0, 8.0], [4.0, 2.0, 8.0], [6.0, 3.0, 7.0]]], [[[6.0, 1.0, 4.0], 
[2.0, 4.0, 2.0], [2.0, 8.0, 6.0]], [[7.0, 5.0, 6.0], [6.0, 4.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [6.0, 7.0, 2.0], 
[4.0, 1.0, 2.0]]], [[[4.0, 7.0, 1.0], [2.0, 3.0, 6.0], [2.0, 8.0, 8.0]], [[6.0, 6.0, 3.0], [2.0, 3.0, 6.0], [5.0, 4.0, 2.0]], 
[[6.0, 3.0, 8.0], [1.0, 3.0, 6.0], [4.0, 2.0, 6.0]]], [[[4.0, 4.0, 7.0], [1.0, 2.0, 3.0], [2.0, 7.0, 2.0]], [[2.0, 3.0, 5.0], 
[2.0, 1.0, 7.0], [6.0, 8.0, 2.0]], [[1.0, 5.0, 6.0], [7.0, 5.0, 5.0], [6.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCPN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 1.0], [8.0, 1.0, 5.0], [2.0, 4.0, 1.0]], [[6.0, 4.0, 3.0], [6.0, 7.0, 7.0], [2.0, 8.0, 
8.0]], [[4.0, 4.0, 6.0], [8.0, 2.0, 6.0], [3.0, 2.0, 7.0]]], [[[1.0, 1.0, 1.0], [8.0, 8.0, 2.0], [1.0, 2.0, 1.0]], [[3.0, 3.0, 
8.0], [4.0, 2.0, 3.0], [8.0, 6.0, 6.0]], [[6.0, 6.0, 6.0], [6.0, 2.0, 4.0], [3.0, 8.0, 1.0]]], [[[6.0, 4.0, 7.0], [1.0, 6.0, 
4.0], [8.0, 8.0, 3.0]], [[2.0, 7.0, 1.0], [3.0, 3.0, 5.0], [5.0, 3.0, 7.0]], [[8.0, 2.0, 4.0], [4.0, 8.0, 1.0], [7.0, 7.0, 
8.0]]], [[[7.0, 1.0, 3.0], [8.0, 5.0, 8.0], [2.0, 1.0, 3.0]], [[3.0, 1.0, 6.0], [4.0, 8.0, 3.0], [6.0, 2.0, 6.0]], [[4.0, 2.0, 
2.0], [2.0, 1.0, 8.0], [2.0, 1.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 8.0, 6.0], [8.0, 8.0, 8.0], [4.0, 4.0, 7.0]], [[5.0, 7.0, 7.0], [6.0, 7.0, 6.0], [7.0, 6.0, 
5.0]], [[7.0, 2.0, 8.0], [5.0, 3.0, 6.0], [6.0, 3.0, 6.0]], [[7.0, 7.0, 2.0], [7.0, 3.0, 8.0], [3.0, 7.0, 
8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 8.0, 3.0], [1.0, 1.0, 6.0], [5.0, 1.0, 2.0]], [[7.0, 2.0, 8.0], [8.0, 7.0, 6.0], 
[1.0, 1.0, 2.0]], [[4.0, 3.0, 3.0], [1.0, 1.0, 6.0], [8.0, 8.0, 5.0]], [[3.0, 4.0, 1.0], [7.0, 4.0, 4.0], [4.0, 3.0, 3.0]]]))
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 1.0], [7.0, 8.0, 6.0], [2.0, 4.0, 1.0]], [[6.0, 4.0, 3.0], [8.0, 8.0, 8.0], [2.0, 
8.0, 8.0]], [[4.0, 4.0, 6.0], [4.0, 4.0, 7.0], [3.0, 2.0, 7.0]]], [[[1.0, 1.0, 1.0], [5.0, 7.0, 7.0], [1.0, 2.0, 1.0]], [[3.0, 
3.0, 8.0], [6.0, 7.0, 6.0], [8.0, 6.0, 6.0]], [[6.0, 6.0, 6.0], [7.0, 6.0, 5.0], [3.0, 8.0, 1.0]]], [[[6.0, 4.0, 7.0], [7.0, 
2.0, 8.0], [8.0, 8.0, 3.0]], [[2.0, 7.0, 1.0], [5.0, 3.0, 6.0], [5.0, 3.0, 7.0]], [[8.0, 2.0, 4.0], [6.0, 3.0, 6.0], [7.0, 7.0, 
8.0]]], [[[7.0, 1.0, 3.0], [7.0, 7.0, 2.0], [2.0, 1.0, 3.0]], [[3.0, 1.0, 6.0], [7.0, 3.0, 8.0], [6.0, 2.0, 6.0]], [[4.0, 2.0, 
2.0], [3.0, 7.0, 8.0], [2.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 1.0], [5.0, 8.0, 3.0], [2.0, 4.0, 1.0]], 
[[6.0, 4.0, 3.0], [1.0, 1.0, 6.0], [2.0, 8.0, 8.0]], [[4.0, 4.0, 6.0], [5.0, 1.0, 2.0], [3.0, 2.0, 7.0]]], [[[1.0, 1.0, 1.0], 
[7.0, 2.0, 8.0], [1.0, 2.0, 1.0]], [[3.0, 3.0, 8.0], [8.0, 7.0, 6.0], [8.0, 6.0, 6.0]], [[6.0, 6.0, 6.0], [1.0, 1.0, 2.0], 
[3.0, 8.0, 1.0]]], [[[6.0, 4.0, 7.0], [4.0, 3.0, 3.0], [8.0, 8.0, 3.0]], [[2.0, 7.0, 1.0], [1.0, 1.0, 6.0], [5.0, 3.0, 7.0]], 
[[8.0, 2.0, 4.0], [8.0, 8.0, 5.0], [7.0, 7.0, 8.0]]], [[[7.0, 1.0, 3.0], [3.0, 4.0, 1.0], [2.0, 1.0, 3.0]], [[3.0, 1.0, 6.0], 
[7.0, 4.0, 4.0], [6.0, 2.0, 6.0]], [[4.0, 2.0, 2.0], [4.0, 3.0, 3.0], [2.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULCN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 7.0], [8.0, 2.0, 1.0], [3.0, 2.0, 5.0]], [[7.0, 3.0, 8.0], [2.0, 6.0, 7.0], [6.0, 3.0, 
3.0]], [[7.0, 5.0, 4.0], [2.0, 2.0, 1.0], [4.0, 1.0, 8.0]]], [[[5.0, 3.0, 7.0], [7.0, 5.0, 4.0], [5.0, 3.0, 1.0]], [[6.0, 2.0, 
4.0], [1.0, 4.0, 1.0], [3.0, 4.0, 5.0]], [[1.0, 1.0, 3.0], [7.0, 2.0, 2.0], [4.0, 5.0, 3.0]]], [[[4.0, 4.0, 4.0], [2.0, 3.0, 
5.0], [2.0, 6.0, 5.0]], [[2.0, 5.0, 1.0], [2.0, 8.0, 8.0], [5.0, 1.0, 6.0]], [[1.0, 5.0, 4.0], [3.0, 3.0, 4.0], [3.0, 5.0, 
8.0]]], [[[3.0, 2.0, 7.0], [1.0, 3.0, 8.0], [8.0, 2.0, 8.0]], [[8.0, 7.0, 7.0], [2.0, 2.0, 2.0], [1.0, 4.0, 1.0]], [[2.0, 3.0, 
2.0], [4.0, 1.0, 1.0], [3.0, 7.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 5.0, 8.0], [1.0, 7.0, 4.0], [4.0, 2.0, 5.0]]], [[[8.0, 5.0, 5.0], [7.0, 8.0, 4.0], [3.0, 
4.0, 4.0]]], [[[8.0, 7.0, 1.0], [6.0, 1.0, 5.0], [6.0, 5.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 4.0], [8.0, 3.0, 5.0], [4.0, 5.0, 4.0]]], [[[4.0, 6.0, 8.0], [7.0, 8.0, 
8.0], [8.0, 7.0, 1.0]]], [[[1.0, 3.0, 4.0], [7.0, 4.0, 3.0], [4.0, 5.0, 7.0]]]]))
      arg[:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 7.0], [8.0, 2.0, 1.0], [3.0, 2.0, 5.0]], [[7.0, 3.0, 8.0], [2.0, 6.0, 7.0], [6.0, 
3.0, 3.0]], [[6.0, 5.0, 8.0], [1.0, 7.0, 4.0], [4.0, 2.0, 5.0]]], [[[5.0, 3.0, 7.0], [7.0, 5.0, 4.0], [5.0, 3.0, 1.0]], [[6.0, 
2.0, 4.0], [1.0, 4.0, 1.0], [3.0, 4.0, 5.0]], [[8.0, 5.0, 5.0], [7.0, 8.0, 4.0], [3.0, 4.0, 4.0]]], [[[4.0, 4.0, 4.0], [2.0, 
3.0, 5.0], [2.0, 6.0, 5.0]], [[2.0, 5.0, 1.0], [2.0, 8.0, 8.0], [5.0, 1.0, 6.0]], [[8.0, 7.0, 1.0], [6.0, 1.0, 5.0], [6.0, 5.0, 
4.0]]], [[[3.0, 2.0, 7.0], [1.0, 3.0, 8.0], [8.0, 2.0, 8.0]], [[8.0, 7.0, 7.0], [2.0, 2.0, 2.0], [1.0, 4.0, 1.0]], [[2.0, 3.0, 
2.0], [4.0, 1.0, 1.0], [3.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 7.0], [8.0, 2.0, 1.0], [3.0, 2.0, 5.0]], 
[[7.0, 3.0, 8.0], [2.0, 6.0, 7.0], [6.0, 3.0, 3.0]], [[3.0, 2.0, 4.0], [8.0, 3.0, 5.0], [4.0, 5.0, 4.0]]], [[[5.0, 3.0, 7.0], 
[7.0, 5.0, 4.0], [5.0, 3.0, 1.0]], [[6.0, 2.0, 4.0], [1.0, 4.0, 1.0], [3.0, 4.0, 5.0]], [[4.0, 6.0, 8.0], [7.0, 8.0, 8.0], 
[8.0, 7.0, 1.0]]], [[[4.0, 4.0, 4.0], [2.0, 3.0, 5.0], [2.0, 6.0, 5.0]], [[2.0, 5.0, 1.0], [2.0, 8.0, 8.0], [5.0, 1.0, 6.0]], 
[[1.0, 3.0, 4.0], [7.0, 4.0, 3.0], [4.0, 5.0, 7.0]]], [[[3.0, 2.0, 7.0], [1.0, 3.0, 8.0], [8.0, 2.0, 8.0]], [[8.0, 7.0, 7.0], 
[2.0, 2.0, 2.0], [1.0, 4.0, 1.0]], [[2.0, 3.0, 2.0], [4.0, 1.0, 1.0], [3.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UUPP(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 7.0], [1.0, 4.0, 2.0], [5.0, 6.0, 8.0]], [[1.0, 2.0, 4.0], [3.0, 1.0, 5.0], [3.0, 1.0, 
3.0]], [[6.0, 2.0, 7.0], [1.0, 2.0, 4.0], [8.0, 2.0, 7.0]]], [[[7.0, 2.0, 7.0], [2.0, 4.0, 2.0], [5.0, 2.0, 2.0]], [[1.0, 4.0, 
5.0], [3.0, 4.0, 7.0], [7.0, 7.0, 4.0]], [[1.0, 6.0, 7.0], [2.0, 6.0, 8.0], [5.0, 2.0, 3.0]]], [[[6.0, 4.0, 3.0], [6.0, 1.0, 
4.0], [3.0, 2.0, 8.0]], [[4.0, 8.0, 4.0], [8.0, 5.0, 1.0], [8.0, 8.0, 2.0]], [[7.0, 1.0, 6.0], [4.0, 2.0, 8.0], [5.0, 2.0, 
5.0]]], [[[4.0, 8.0, 7.0], [4.0, 6.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 4.0, 5.0], [4.0, 7.0, 1.0], [8.0, 5.0, 2.0]], [[3.0, 3.0, 
8.0], [1.0, 2.0, 3.0], [1.0, 5.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 6.0], [5.0, 6.0], [7.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 6.0], [3.0, 3.0], [4.0, 5.0]]))
      arg[:3,:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 7.0], [1.0, 3.0, 2.0], [5.0, 6.0, 8.0]], [[1.0, 2.0, 4.0], [3.0, 6.0, 5.0], [3.0, 
1.0, 3.0]], [[6.0, 2.0, 7.0], [1.0, 2.0, 4.0], [8.0, 2.0, 7.0]]], [[[7.0, 2.0, 7.0], [2.0, 5.0, 2.0], [5.0, 2.0, 2.0]], [[1.0, 
4.0, 5.0], [3.0, 6.0, 7.0], [7.0, 7.0, 4.0]], [[1.0, 6.0, 7.0], [2.0, 6.0, 8.0], [5.0, 2.0, 3.0]]], [[[6.0, 4.0, 3.0], [6.0, 
7.0, 4.0], [3.0, 2.0, 8.0]], [[4.0, 8.0, 4.0], [8.0, 1.0, 1.0], [8.0, 8.0, 2.0]], [[7.0, 1.0, 6.0], [4.0, 2.0, 8.0], [5.0, 2.0, 
5.0]]], [[[4.0, 8.0, 7.0], [4.0, 6.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 4.0, 5.0], [4.0, 7.0, 1.0], [8.0, 5.0, 2.0]], [[3.0, 3.0, 
8.0], [1.0, 2.0, 3.0], [1.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 7.0], [1.0, 6.0, 2.0], [5.0, 6.0, 8.0]], 
[[1.0, 2.0, 4.0], [3.0, 6.0, 5.0], [3.0, 1.0, 3.0]], [[6.0, 2.0, 7.0], [1.0, 2.0, 4.0], [8.0, 2.0, 7.0]]], [[[7.0, 2.0, 7.0], 
[2.0, 3.0, 2.0], [5.0, 2.0, 2.0]], [[1.0, 4.0, 5.0], [3.0, 3.0, 7.0], [7.0, 7.0, 4.0]], [[1.0, 6.0, 7.0], [2.0, 6.0, 8.0], 
[5.0, 2.0, 3.0]]], [[[6.0, 4.0, 3.0], [6.0, 4.0, 4.0], [3.0, 2.0, 8.0]], [[4.0, 8.0, 4.0], [8.0, 5.0, 1.0], [8.0, 8.0, 2.0]], 
[[7.0, 1.0, 6.0], [4.0, 2.0, 8.0], [5.0, 2.0, 5.0]]], [[[4.0, 8.0, 7.0], [4.0, 6.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 4.0, 5.0], 
[4.0, 7.0, 1.0], [8.0, 5.0, 2.0]], [[3.0, 3.0, 8.0], [1.0, 2.0, 3.0], [1.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLPN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 1.0], [2.0, 5.0, 8.0], [8.0, 6.0, 4.0]], [[5.0, 6.0, 2.0], [3.0, 1.0, 8.0], [2.0, 5.0, 
3.0]], [[2.0, 2.0, 5.0], [3.0, 8.0, 1.0], [5.0, 4.0, 7.0]]], [[[5.0, 7.0, 2.0], [6.0, 6.0, 4.0], [3.0, 3.0, 3.0]], [[1.0, 1.0, 
1.0], [6.0, 6.0, 2.0], [1.0, 2.0, 2.0]], [[7.0, 2.0, 5.0], [5.0, 5.0, 3.0], [3.0, 8.0, 8.0]]], [[[6.0, 1.0, 4.0], [3.0, 8.0, 
1.0], [1.0, 1.0, 5.0]], [[7.0, 6.0, 3.0], [8.0, 2.0, 7.0], [6.0, 5.0, 7.0]], [[2.0, 2.0, 1.0], [1.0, 5.0, 4.0], [5.0, 6.0, 
3.0]]], [[[5.0, 2.0, 5.0], [5.0, 5.0, 6.0], [4.0, 7.0, 7.0]], [[2.0, 7.0, 5.0], [6.0, 8.0, 2.0], [4.0, 7.0, 3.0]], [[1.0, 4.0, 
6.0], [7.0, 5.0, 2.0], [7.0, 2.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 8.0, 1.0]], [[5.0, 8.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 7.0, 1.0]], [[5.0, 2.0, 4.0]]]))
      arg[1:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 1.0], [2.0, 5.0, 8.0], [8.0, 6.0, 4.0]], [[5.0, 6.0, 2.0], [3.0, 1.0, 8.0], [2.0, 
5.0, 3.0]], [[2.0, 2.0, 5.0], [3.0, 8.0, 1.0], [5.0, 4.0, 7.0]]], [[[5.0, 7.0, 2.0], [6.0, 6.0, 4.0], [3.0, 3.0, 3.0]], [[1.0, 
1.0, 1.0], [6.0, 6.0, 2.0], [1.0, 2.0, 2.0]], [[7.0, 2.0, 5.0], [4.0, 8.0, 1.0], [3.0, 8.0, 8.0]]], [[[6.0, 1.0, 4.0], [3.0, 
8.0, 1.0], [1.0, 1.0, 5.0]], [[7.0, 6.0, 3.0], [8.0, 2.0, 7.0], [6.0, 5.0, 7.0]], [[2.0, 2.0, 1.0], [5.0, 8.0, 1.0], [5.0, 6.0, 
3.0]]], [[[5.0, 2.0, 5.0], [5.0, 5.0, 6.0], [4.0, 7.0, 7.0]], [[2.0, 7.0, 5.0], [6.0, 8.0, 2.0], [4.0, 7.0, 3.0]], [[1.0, 4.0, 
6.0], [7.0, 5.0, 2.0], [7.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 1.0], [2.0, 5.0, 8.0], [8.0, 6.0, 4.0]], 
[[5.0, 6.0, 2.0], [3.0, 1.0, 8.0], [2.0, 5.0, 3.0]], [[2.0, 2.0, 5.0], [3.0, 8.0, 1.0], [5.0, 4.0, 7.0]]], [[[5.0, 7.0, 2.0], 
[6.0, 6.0, 4.0], [3.0, 3.0, 3.0]], [[1.0, 1.0, 1.0], [6.0, 6.0, 2.0], [1.0, 2.0, 2.0]], [[7.0, 2.0, 5.0], [4.0, 7.0, 1.0], 
[3.0, 8.0, 8.0]]], [[[6.0, 1.0, 4.0], [3.0, 8.0, 1.0], [1.0, 1.0, 5.0]], [[7.0, 6.0, 3.0], [8.0, 2.0, 7.0], [6.0, 5.0, 7.0]], 
[[2.0, 2.0, 1.0], [5.0, 2.0, 4.0], [5.0, 6.0, 3.0]]], [[[5.0, 2.0, 5.0], [5.0, 5.0, 6.0], [4.0, 7.0, 7.0]], [[2.0, 7.0, 5.0], 
[6.0, 8.0, 2.0], [4.0, 7.0, 3.0]], [[1.0, 4.0, 6.0], [7.0, 5.0, 2.0], [7.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCUN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 7.0], [1.0, 5.0, 3.0], [5.0, 1.0, 1.0]], [[6.0, 1.0, 5.0], [1.0, 1.0, 3.0], [6.0, 8.0, 
3.0]], [[5.0, 7.0, 3.0], [4.0, 8.0, 2.0], [2.0, 1.0, 3.0]]], [[[4.0, 2.0, 7.0], [1.0, 5.0, 4.0], [1.0, 6.0, 6.0]], [[3.0, 3.0, 
3.0], [1.0, 8.0, 6.0], [6.0, 5.0, 3.0]], [[3.0, 5.0, 2.0], [7.0, 8.0, 3.0], [8.0, 4.0, 2.0]]], [[[4.0, 1.0, 3.0], [6.0, 1.0, 
1.0], [3.0, 3.0, 8.0]], [[7.0, 3.0, 4.0], [3.0, 6.0, 5.0], [2.0, 1.0, 6.0]], [[5.0, 2.0, 2.0], [3.0, 7.0, 5.0], [6.0, 4.0, 
2.0]]], [[[7.0, 5.0, 8.0], [7.0, 1.0, 7.0], [4.0, 2.0, 2.0]], [[8.0, 3.0, 5.0], [7.0, 1.0, 7.0], [5.0, 7.0, 2.0]], [[1.0, 6.0, 
8.0], [1.0, 6.0, 3.0], [5.0, 2.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 5.0, 3.0], [1.0, 4.0, 1.0]], [[5.0, 1.0, 8.0], [4.0, 3.0, 5.0]], [[2.0, 1.0, 5.0], [4.0, 
6.0, 7.0]]], [[[4.0, 6.0, 4.0], [5.0, 7.0, 7.0]], [[8.0, 3.0, 1.0], [4.0, 8.0, 1.0]], [[6.0, 7.0, 8.0], [3.0, 1.0, 2.0]]], 
[[[2.0, 4.0, 4.0], [7.0, 2.0, 7.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 5.0]], [[5.0, 7.0, 3.0], [3.0, 2.0, 3.0]]], [[[3.0, 8.0, 
4.0], [4.0, 7.0, 7.0]], [[3.0, 7.0, 1.0], [3.0, 4.0, 2.0]], [[4.0, 7.0, 4.0], [6.0, 1.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 1.0], [3.0, 7.0, 5.0]], [[4.0, 7.0, 5.0], [8.0, 1.0, 3.0]], [[4.0, 3.0, 
3.0], [7.0, 1.0, 8.0]]], [[[3.0, 1.0, 2.0], [5.0, 7.0, 2.0]], [[4.0, 3.0, 8.0], [3.0, 3.0, 4.0]], [[1.0, 7.0, 4.0], [3.0, 8.0, 
5.0]]], [[[3.0, 6.0, 8.0], [8.0, 2.0, 2.0]], [[7.0, 4.0, 4.0], [6.0, 8.0, 7.0]], [[8.0, 7.0, 5.0], [4.0, 8.0, 2.0]]], [[[8.0, 
8.0, 1.0], [8.0, 5.0, 8.0]], [[1.0, 4.0, 8.0], [3.0, 2.0, 2.0]], [[1.0, 4.0, 6.0], [5.0, 1.0, 3.0]]]]))
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 3.0], [1.0, 4.0, 1.0], [5.0, 1.0, 1.0]], [[5.0, 1.0, 8.0], [4.0, 3.0, 5.0], [6.0, 
8.0, 3.0]], [[2.0, 1.0, 5.0], [4.0, 6.0, 7.0], [2.0, 1.0, 3.0]]], [[[4.0, 6.0, 4.0], [5.0, 7.0, 7.0], [1.0, 6.0, 6.0]], [[8.0, 
3.0, 1.0], [4.0, 8.0, 1.0], [6.0, 5.0, 3.0]], [[6.0, 7.0, 8.0], [3.0, 1.0, 2.0], [8.0, 4.0, 2.0]]], [[[2.0, 4.0, 4.0], [7.0, 
2.0, 7.0], [3.0, 3.0, 8.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 5.0], [2.0, 1.0, 6.0]], [[5.0, 7.0, 3.0], [3.0, 2.0, 3.0], [6.0, 4.0, 
2.0]]], [[[3.0, 8.0, 4.0], [4.0, 7.0, 7.0], [4.0, 2.0, 2.0]], [[3.0, 7.0, 1.0], [3.0, 4.0, 2.0], [5.0, 7.0, 2.0]], [[4.0, 7.0, 
4.0], [6.0, 1.0, 4.0], [5.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 1.0], [3.0, 7.0, 5.0], [5.0, 1.0, 1.0]], 
[[4.0, 7.0, 5.0], [8.0, 1.0, 3.0], [6.0, 8.0, 3.0]], [[4.0, 3.0, 3.0], [7.0, 1.0, 8.0], [2.0, 1.0, 3.0]]], [[[3.0, 1.0, 2.0], 
[5.0, 7.0, 2.0], [1.0, 6.0, 6.0]], [[4.0, 3.0, 8.0], [3.0, 3.0, 4.0], [6.0, 5.0, 3.0]], [[1.0, 7.0, 4.0], [3.0, 8.0, 5.0], 
[8.0, 4.0, 2.0]]], [[[3.0, 6.0, 8.0], [8.0, 2.0, 2.0], [3.0, 3.0, 8.0]], [[7.0, 4.0, 4.0], [6.0, 8.0, 7.0], [2.0, 1.0, 6.0]], 
[[8.0, 7.0, 5.0], [4.0, 8.0, 2.0], [6.0, 4.0, 2.0]]], [[[8.0, 8.0, 1.0], [8.0, 5.0, 8.0], [4.0, 2.0, 2.0]], [[1.0, 4.0, 8.0], 
[3.0, 2.0, 2.0], [5.0, 7.0, 2.0]], [[1.0, 4.0, 6.0], [5.0, 1.0, 3.0], [5.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CUCN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 3.0], [7.0, 4.0, 8.0], [8.0, 8.0, 7.0]], [[4.0, 1.0, 4.0], [5.0, 8.0, 2.0], [1.0, 8.0, 
5.0]], [[1.0, 2.0, 1.0], [5.0, 3.0, 3.0], [5.0, 1.0, 4.0]]], [[[8.0, 4.0, 3.0], [7.0, 5.0, 7.0], [8.0, 4.0, 2.0]], [[6.0, 5.0, 
5.0], [1.0, 8.0, 7.0], [8.0, 6.0, 1.0]], [[5.0, 1.0, 7.0], [4.0, 7.0, 7.0], [2.0, 6.0, 3.0]]], [[[7.0, 6.0, 7.0], [5.0, 5.0, 
5.0], [7.0, 2.0, 4.0]], [[5.0, 6.0, 2.0], [4.0, 7.0, 8.0], [7.0, 8.0, 8.0]], [[5.0, 8.0, 7.0], [6.0, 6.0, 4.0], [4.0, 2.0, 
8.0]]], [[[6.0, 5.0, 8.0], [6.0, 2.0, 4.0], [4.0, 2.0, 6.0]], [[6.0, 8.0, 2.0], [7.0, 6.0, 2.0], [3.0, 1.0, 3.0]], [[5.0, 8.0, 
8.0], [6.0, 1.0, 3.0], [5.0, 8.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 5.0, 6.0], [8.0, 4.0, 6.0], [6.0, 3.0, 2.0]], [[8.0, 8.0, 1.0], [6.0, 8.0, 3.0], [8.0, 6.0, 
3.0]]], [[[2.0, 7.0, 2.0], [3.0, 8.0, 5.0], [8.0, 5.0, 1.0]], [[4.0, 3.0, 3.0], [2.0, 4.0, 6.0], [1.0, 4.0, 4.0]]], [[[3.0, 
4.0, 4.0], [8.0, 6.0, 8.0], [5.0, 5.0, 2.0]], [[5.0, 7.0, 1.0], [7.0, 5.0, 3.0], [7.0, 4.0, 4.0]]], [[[7.0, 2.0, 6.0], [5.0, 
2.0, 7.0], [6.0, 8.0, 6.0]], [[1.0, 6.0, 8.0], [3.0, 2.0, 7.0], [1.0, 4.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 1.0], [6.0, 1.0, 7.0], [4.0, 7.0, 7.0]], [[8.0, 2.0, 4.0], [4.0, 7.0, 
2.0], [3.0, 4.0, 2.0]]], [[[3.0, 1.0, 2.0], [5.0, 7.0, 6.0], [6.0, 7.0, 6.0]], [[7.0, 7.0, 4.0], [8.0, 8.0, 6.0], [7.0, 4.0, 
5.0]]], [[[5.0, 1.0, 6.0], [7.0, 7.0, 8.0], [6.0, 6.0, 7.0]], [[4.0, 2.0, 5.0], [7.0, 5.0, 7.0], [8.0, 8.0, 8.0]]], [[[5.0, 
3.0, 7.0], [5.0, 8.0, 5.0], [4.0, 1.0, 8.0]], [[1.0, 7.0, 2.0], [4.0, 1.0, 6.0], [8.0, 3.0, 3.0]]]]))
      arg[:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 6.0], [8.0, 4.0, 6.0], [6.0, 3.0, 2.0]], [[8.0, 8.0, 1.0], [6.0, 8.0, 3.0], [8.0, 
6.0, 3.0]], [[1.0, 2.0, 1.0], [5.0, 3.0, 3.0], [5.0, 1.0, 4.0]]], [[[2.0, 7.0, 2.0], [3.0, 8.0, 5.0], [8.0, 5.0, 1.0]], [[4.0, 
3.0, 3.0], [2.0, 4.0, 6.0], [1.0, 4.0, 4.0]], [[5.0, 1.0, 7.0], [4.0, 7.0, 7.0], [2.0, 6.0, 3.0]]], [[[3.0, 4.0, 4.0], [8.0, 
6.0, 8.0], [5.0, 5.0, 2.0]], [[5.0, 7.0, 1.0], [7.0, 5.0, 3.0], [7.0, 4.0, 4.0]], [[5.0, 8.0, 7.0], [6.0, 6.0, 4.0], [4.0, 2.0, 
8.0]]], [[[7.0, 2.0, 6.0], [5.0, 2.0, 7.0], [6.0, 8.0, 6.0]], [[1.0, 6.0, 8.0], [3.0, 2.0, 7.0], [1.0, 4.0, 5.0]], [[5.0, 8.0, 
8.0], [6.0, 1.0, 3.0], [5.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 1.0], [6.0, 1.0, 7.0], [4.0, 7.0, 7.0]], 
[[8.0, 2.0, 4.0], [4.0, 7.0, 2.0], [3.0, 4.0, 2.0]], [[1.0, 2.0, 1.0], [5.0, 3.0, 3.0], [5.0, 1.0, 4.0]]], [[[3.0, 1.0, 2.0], 
[5.0, 7.0, 6.0], [6.0, 7.0, 6.0]], [[7.0, 7.0, 4.0], [8.0, 8.0, 6.0], [7.0, 4.0, 5.0]], [[5.0, 1.0, 7.0], [4.0, 7.0, 7.0], 
[2.0, 6.0, 3.0]]], [[[5.0, 1.0, 6.0], [7.0, 7.0, 8.0], [6.0, 6.0, 7.0]], [[4.0, 2.0, 5.0], [7.0, 5.0, 7.0], [8.0, 8.0, 8.0]], 
[[5.0, 8.0, 7.0], [6.0, 6.0, 4.0], [4.0, 2.0, 8.0]]], [[[5.0, 3.0, 7.0], [5.0, 8.0, 5.0], [4.0, 1.0, 8.0]], [[1.0, 7.0, 2.0], 
[4.0, 1.0, 6.0], [8.0, 3.0, 3.0]], [[5.0, 8.0, 8.0], [6.0, 1.0, 3.0], [5.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCCP(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 5.0], [4.0, 6.0, 1.0], [5.0, 5.0, 5.0]], [[5.0, 1.0, 6.0], [5.0, 3.0, 4.0], [2.0, 5.0, 
3.0]], [[5.0, 2.0, 5.0], [2.0, 6.0, 6.0], [5.0, 2.0, 1.0]]], [[[2.0, 5.0, 2.0], [3.0, 2.0, 2.0], [6.0, 8.0, 5.0]], [[8.0, 5.0, 
8.0], [4.0, 8.0, 2.0], [8.0, 4.0, 6.0]], [[6.0, 8.0, 2.0], [7.0, 7.0, 7.0], [4.0, 6.0, 3.0]]], [[[5.0, 1.0, 7.0], [3.0, 8.0, 
7.0], [1.0, 2.0, 2.0]], [[1.0, 1.0, 5.0], [5.0, 7.0, 8.0], [3.0, 6.0, 4.0]], [[1.0, 6.0, 7.0], [3.0, 7.0, 5.0], [6.0, 7.0, 
6.0]]], [[[3.0, 1.0, 4.0], [5.0, 1.0, 2.0], [3.0, 3.0, 4.0]], [[1.0, 3.0, 4.0], [8.0, 8.0, 1.0], [8.0, 7.0, 4.0]], [[1.0, 7.0, 
8.0], [4.0, 7.0, 7.0], [6.0, 2.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 3.0, 2.0], [1.0, 7.0, 2.0], [2.0, 2.0, 8.0]], [[4.0, 1.0, 7.0], [1.0, 2.0, 7.0], [1.0, 4.0, 
5.0]], [[4.0, 5.0, 7.0], [1.0, 2.0, 4.0], [5.0, 6.0, 7.0]], [[7.0, 1.0, 7.0], [5.0, 1.0, 7.0], [7.0, 8.0, 
2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 4.0, 1.0], [6.0, 1.0, 1.0], [6.0, 6.0, 5.0]], [[4.0, 3.0, 3.0], [1.0, 3.0, 5.0], 
[5.0, 6.0, 2.0]], [[6.0, 3.0, 6.0], [5.0, 5.0, 4.0], [1.0, 5.0, 6.0]], [[7.0, 3.0, 7.0], [4.0, 3.0, 5.0], [3.0, 3.0, 6.0]]]))
      arg[:,:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 5.0], [4.0, 3.0, 1.0], [5.0, 2.0, 5.0]], [[5.0, 1.0, 6.0], [5.0, 7.0, 4.0], [2.0, 
2.0, 3.0]], [[5.0, 2.0, 5.0], [2.0, 2.0, 6.0], [5.0, 8.0, 1.0]]], [[[2.0, 4.0, 2.0], [3.0, 1.0, 2.0], [6.0, 7.0, 5.0]], [[8.0, 
1.0, 8.0], [4.0, 2.0, 2.0], [8.0, 7.0, 6.0]], [[6.0, 1.0, 2.0], [7.0, 4.0, 7.0], [4.0, 5.0, 3.0]]], [[[5.0, 4.0, 7.0], [3.0, 
5.0, 7.0], [1.0, 7.0, 2.0]], [[1.0, 1.0, 5.0], [5.0, 2.0, 8.0], [3.0, 4.0, 4.0]], [[1.0, 5.0, 7.0], [3.0, 6.0, 5.0], [6.0, 7.0, 
6.0]]], [[[3.0, 7.0, 4.0], [5.0, 1.0, 2.0], [3.0, 7.0, 4.0]], [[1.0, 5.0, 4.0], [8.0, 1.0, 1.0], [8.0, 7.0, 4.0]], [[1.0, 7.0, 
8.0], [4.0, 8.0, 7.0], [6.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 5.0], [4.0, 4.0, 1.0], [5.0, 1.0, 5.0]], 
[[5.0, 6.0, 6.0], [5.0, 1.0, 4.0], [2.0, 1.0, 3.0]], [[5.0, 6.0, 5.0], [2.0, 6.0, 6.0], [5.0, 5.0, 1.0]]], [[[2.0, 4.0, 2.0], 
[3.0, 3.0, 2.0], [6.0, 3.0, 5.0]], [[8.0, 1.0, 8.0], [4.0, 3.0, 2.0], [8.0, 5.0, 6.0]], [[6.0, 5.0, 2.0], [7.0, 6.0, 7.0], 
[4.0, 2.0, 3.0]]], [[[5.0, 6.0, 7.0], [3.0, 3.0, 7.0], [1.0, 6.0, 2.0]], [[1.0, 5.0, 5.0], [5.0, 5.0, 8.0], [3.0, 4.0, 4.0]], 
[[1.0, 1.0, 7.0], [3.0, 5.0, 5.0], [6.0, 6.0, 6.0]]], [[[3.0, 7.0, 4.0], [5.0, 3.0, 2.0], [3.0, 7.0, 4.0]], [[1.0, 4.0, 4.0], 
[8.0, 3.0, 1.0], [8.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], [4.0, 3.0, 7.0], [6.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCCN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 1.0], [8.0, 2.0, 3.0], [4.0, 7.0, 1.0]], [[6.0, 5.0, 6.0], [2.0, 7.0, 7.0], [4.0, 5.0, 
3.0]], [[6.0, 2.0, 4.0], [3.0, 8.0, 2.0], [6.0, 7.0, 1.0]]], [[[5.0, 8.0, 8.0], [4.0, 1.0, 5.0], [3.0, 2.0, 4.0]], [[8.0, 6.0, 
1.0], [1.0, 4.0, 1.0], [5.0, 6.0, 1.0]], [[2.0, 4.0, 2.0], [8.0, 7.0, 8.0], [5.0, 5.0, 4.0]]], [[[6.0, 3.0, 8.0], [1.0, 8.0, 
6.0], [6.0, 4.0, 5.0]], [[6.0, 1.0, 2.0], [6.0, 4.0, 3.0], [7.0, 6.0, 7.0]], [[5.0, 8.0, 1.0], [5.0, 1.0, 2.0], [6.0, 1.0, 
8.0]]], [[[5.0, 4.0, 1.0], [6.0, 3.0, 4.0], [1.0, 1.0, 1.0]], [[2.0, 7.0, 3.0], [6.0, 5.0, 3.0], [5.0, 5.0, 7.0]], [[5.0, 5.0, 
1.0], [2.0, 2.0, 8.0], [5.0, 1.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 6.0, 7.0], [1.0, 4.0, 1.0], [3.0, 2.0, 3.0]], [[6.0, 2.0, 8.0], [7.0, 4.0, 1.0], [7.0, 7.0, 
2.0]], [[7.0, 5.0, 2.0], [4.0, 4.0, 4.0], [3.0, 7.0, 3.0]]], [[[4.0, 7.0, 3.0], [8.0, 7.0, 7.0], [7.0, 5.0, 5.0]], [[3.0, 3.0, 
6.0], [8.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[7.0, 2.0, 1.0], [6.0, 3.0, 6.0], [1.0, 2.0, 4.0]]], [[[8.0, 3.0, 6.0], [3.0, 1.0, 
7.0], [7.0, 8.0, 3.0]], [[5.0, 3.0, 6.0], [7.0, 8.0, 5.0], [3.0, 5.0, 7.0]], [[8.0, 3.0, 1.0], [7.0, 4.0, 1.0], [6.0, 6.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 4.0], [3.0, 8.0, 2.0], [3.0, 8.0, 5.0]], [[4.0, 4.0, 3.0], [4.0, 6.0, 
3.0], [7.0, 3.0, 4.0]], [[1.0, 3.0, 5.0], [4.0, 5.0, 8.0], [8.0, 6.0, 4.0]]], [[[3.0, 4.0, 4.0], [1.0, 6.0, 5.0], [8.0, 7.0, 
6.0]], [[4.0, 3.0, 3.0], [3.0, 8.0, 3.0], [3.0, 2.0, 7.0]], [[8.0, 3.0, 2.0], [3.0, 1.0, 7.0], [8.0, 3.0, 5.0]]], [[[5.0, 2.0, 
1.0], [2.0, 3.0, 7.0], [3.0, 7.0, 4.0]], [[6.0, 4.0, 7.0], [4.0, 3.0, 8.0], [3.0, 7.0, 4.0]], [[3.0, 2.0, 3.0], [2.0, 5.0, 
1.0], [5.0, 6.0, 3.0]]]]))
      arg[:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 7.0], [1.0, 4.0, 1.0], [3.0, 2.0, 3.0]], [[6.0, 2.0, 8.0], [7.0, 4.0, 1.0], [7.0, 
7.0, 2.0]], [[7.0, 5.0, 2.0], [4.0, 4.0, 4.0], [3.0, 7.0, 3.0]]], [[[4.0, 7.0, 3.0], [8.0, 7.0, 7.0], [7.0, 5.0, 5.0]], [[3.0, 
3.0, 6.0], [8.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[7.0, 2.0, 1.0], [6.0, 3.0, 6.0], [1.0, 2.0, 4.0]]], [[[8.0, 3.0, 6.0], [3.0, 
1.0, 7.0], [7.0, 8.0, 3.0]], [[5.0, 3.0, 6.0], [7.0, 8.0, 5.0], [3.0, 5.0, 7.0]], [[8.0, 3.0, 1.0], [7.0, 4.0, 1.0], [6.0, 6.0, 
1.0]]], [[[5.0, 4.0, 1.0], [6.0, 3.0, 4.0], [1.0, 1.0, 1.0]], [[2.0, 7.0, 3.0], [6.0, 5.0, 3.0], [5.0, 5.0, 7.0]], [[5.0, 5.0, 
1.0], [2.0, 2.0, 8.0], [5.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 4.0], [3.0, 8.0, 2.0], [3.0, 8.0, 5.0]], 
[[4.0, 4.0, 3.0], [4.0, 6.0, 3.0], [7.0, 3.0, 4.0]], [[1.0, 3.0, 5.0], [4.0, 5.0, 8.0], [8.0, 6.0, 4.0]]], [[[3.0, 4.0, 4.0], 
[1.0, 6.0, 5.0], [8.0, 7.0, 6.0]], [[4.0, 3.0, 3.0], [3.0, 8.0, 3.0], [3.0, 2.0, 7.0]], [[8.0, 3.0, 2.0], [3.0, 1.0, 7.0], 
[8.0, 3.0, 5.0]]], [[[5.0, 2.0, 1.0], [2.0, 3.0, 7.0], [3.0, 7.0, 4.0]], [[6.0, 4.0, 7.0], [4.0, 3.0, 8.0], [3.0, 7.0, 4.0]], 
[[3.0, 2.0, 3.0], [2.0, 5.0, 1.0], [5.0, 6.0, 3.0]]], [[[5.0, 4.0, 1.0], [6.0, 3.0, 4.0], [1.0, 1.0, 1.0]], [[2.0, 7.0, 3.0], 
[6.0, 5.0, 3.0], [5.0, 5.0, 7.0]], [[5.0, 5.0, 1.0], [2.0, 2.0, 8.0], [5.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CRCC(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 7.0], [7.0, 4.0, 5.0], [3.0, 6.0, 2.0]], [[2.0, 4.0, 6.0], [5.0, 1.0, 6.0], [2.0, 8.0, 
2.0]], [[6.0, 7.0, 6.0], [4.0, 7.0, 2.0], [6.0, 8.0, 8.0]]], [[[4.0, 1.0, 3.0], [5.0, 3.0, 7.0], [1.0, 2.0, 4.0]], [[4.0, 4.0, 
7.0], [8.0, 3.0, 4.0], [1.0, 7.0, 1.0]], [[5.0, 4.0, 8.0], [8.0, 8.0, 2.0], [5.0, 1.0, 4.0]]], [[[5.0, 2.0, 6.0], [3.0, 2.0, 
3.0], [6.0, 5.0, 2.0]], [[2.0, 7.0, 6.0], [1.0, 5.0, 3.0], [8.0, 7.0, 3.0]], [[7.0, 5.0, 6.0], [7.0, 8.0, 8.0], [3.0, 8.0, 
4.0]]], [[[1.0, 2.0, 6.0], [1.0, 3.0, 2.0], [7.0, 1.0, 6.0]], [[5.0, 4.0, 1.0], [7.0, 3.0, 3.0], [1.0, 1.0, 3.0]], [[4.0, 3.0, 
6.0], [6.0, 3.0, 3.0], [3.0, 3.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 8.0, 3.0], [2.0, 5.0, 1.0], [3.0, 7.0, 1.0]]], [[[6.0, 2.0, 6.0], [1.0, 3.0, 8.0], [3.0, 
2.0, 3.0]]], [[[4.0, 6.0, 6.0], [2.0, 1.0, 5.0], [2.0, 2.0, 6.0]]], [[[8.0, 8.0, 3.0], [6.0, 2.0, 3.0], [3.0, 7.0, 
8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 2.0], [8.0, 1.0, 1.0], [4.0, 3.0, 7.0]]], [[[7.0, 5.0, 2.0], [2.0, 3.0, 
5.0], [8.0, 3.0, 3.0]]], [[[5.0, 6.0, 7.0], [1.0, 4.0, 8.0], [6.0, 7.0, 1.0]]], [[[8.0, 4.0, 4.0], [7.0, 1.0, 7.0], [3.0, 4.0, 
5.0]]]]))
      arg[:,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 7.0], [7.0, 4.0, 5.0], [3.0, 6.0, 2.0]], [[6.0, 8.0, 3.0], [2.0, 5.0, 1.0], [3.0, 
7.0, 1.0]], [[6.0, 7.0, 6.0], [4.0, 7.0, 2.0], [6.0, 8.0, 8.0]]], [[[4.0, 1.0, 3.0], [5.0, 3.0, 7.0], [1.0, 2.0, 4.0]], [[6.0, 
2.0, 6.0], [1.0, 3.0, 8.0], [3.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [8.0, 8.0, 2.0], [5.0, 1.0, 4.0]]], [[[5.0, 2.0, 6.0], [3.0, 
2.0, 3.0], [6.0, 5.0, 2.0]], [[4.0, 6.0, 6.0], [2.0, 1.0, 5.0], [2.0, 2.0, 6.0]], [[7.0, 5.0, 6.0], [7.0, 8.0, 8.0], [3.0, 8.0, 
4.0]]], [[[1.0, 2.0, 6.0], [1.0, 3.0, 2.0], [7.0, 1.0, 6.0]], [[8.0, 8.0, 3.0], [6.0, 2.0, 3.0], [3.0, 7.0, 8.0]], [[4.0, 3.0, 
6.0], [6.0, 3.0, 3.0], [3.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 7.0], [7.0, 4.0, 5.0], [3.0, 6.0, 2.0]], 
[[6.0, 4.0, 2.0], [8.0, 1.0, 1.0], [4.0, 3.0, 7.0]], [[6.0, 7.0, 6.0], [4.0, 7.0, 2.0], [6.0, 8.0, 8.0]]], [[[4.0, 1.0, 3.0], 
[5.0, 3.0, 7.0], [1.0, 2.0, 4.0]], [[7.0, 5.0, 2.0], [2.0, 3.0, 5.0], [8.0, 3.0, 3.0]], [[5.0, 4.0, 8.0], [8.0, 8.0, 2.0], 
[5.0, 1.0, 4.0]]], [[[5.0, 2.0, 6.0], [3.0, 2.0, 3.0], [6.0, 5.0, 2.0]], [[5.0, 6.0, 7.0], [1.0, 4.0, 8.0], [6.0, 7.0, 1.0]], 
[[7.0, 5.0, 6.0], [7.0, 8.0, 8.0], [3.0, 8.0, 4.0]]], [[[1.0, 2.0, 6.0], [1.0, 3.0, 2.0], [7.0, 1.0, 6.0]], [[8.0, 4.0, 4.0], 
[7.0, 1.0, 7.0], [3.0, 4.0, 5.0]], [[4.0, 3.0, 6.0], [6.0, 3.0, 3.0], [3.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UURU(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [5.0, 5.0, 8.0], [4.0, 7.0, 8.0]], [[4.0, 8.0, 8.0], [5.0, 4.0, 2.0], [3.0, 8.0, 
1.0]], [[7.0, 4.0, 4.0], [5.0, 4.0, 7.0], [1.0, 8.0, 2.0]]], [[[7.0, 4.0, 1.0], [4.0, 1.0, 1.0], [4.0, 3.0, 7.0]], [[8.0, 5.0, 
7.0], [7.0, 7.0, 5.0], [4.0, 8.0, 5.0]], [[3.0, 4.0, 7.0], [4.0, 8.0, 2.0], [1.0, 6.0, 4.0]]], [[[4.0, 8.0, 3.0], [5.0, 1.0, 
1.0], [2.0, 5.0, 7.0]], [[1.0, 1.0, 6.0], [8.0, 5.0, 7.0], [6.0, 1.0, 2.0]], [[7.0, 1.0, 5.0], [3.0, 8.0, 5.0], [8.0, 6.0, 
7.0]]], [[[2.0, 6.0, 5.0], [2.0, 6.0, 8.0], [7.0, 2.0, 2.0]], [[5.0, 6.0, 8.0], [7.0, 5.0, 7.0], [3.0, 6.0, 6.0]], [[3.0, 1.0, 
5.0], [8.0, 8.0, 6.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 3.0]], [[6.0, 8.0]]], [[[7.0, 7.0]], [[8.0, 7.0]]], [[[3.0, 1.0]], [[3.0, 
8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 4.0]], [[8.0, 5.0]]], [[[8.0, 6.0]], [[1.0, 8.0]]], [[[3.0, 7.0]], [[8.0, 
7.0]]]]))
      arg[:3,:2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [5.0, 3.0, 8.0], [4.0, 7.0, 8.0]], [[4.0, 8.0, 8.0], [6.0, 8.0, 2.0], [3.0, 
8.0, 1.0]], [[7.0, 4.0, 4.0], [5.0, 4.0, 7.0], [1.0, 8.0, 2.0]]], [[[7.0, 4.0, 1.0], [7.0, 7.0, 1.0], [4.0, 3.0, 7.0]], [[8.0, 
5.0, 7.0], [8.0, 7.0, 5.0], [4.0, 8.0, 5.0]], [[3.0, 4.0, 7.0], [4.0, 8.0, 2.0], [1.0, 6.0, 4.0]]], [[[4.0, 8.0, 3.0], [3.0, 
1.0, 1.0], [2.0, 5.0, 7.0]], [[1.0, 1.0, 6.0], [3.0, 8.0, 7.0], [6.0, 1.0, 2.0]], [[7.0, 1.0, 5.0], [3.0, 8.0, 5.0], [8.0, 6.0, 
7.0]]], [[[2.0, 6.0, 5.0], [2.0, 6.0, 8.0], [7.0, 2.0, 2.0]], [[5.0, 6.0, 8.0], [7.0, 5.0, 7.0], [3.0, 6.0, 6.0]], [[3.0, 1.0, 
5.0], [8.0, 8.0, 6.0], [6.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 7.0], [1.0, 4.0, 8.0], [4.0, 7.0, 8.0]], 
[[4.0, 8.0, 8.0], [8.0, 5.0, 2.0], [3.0, 8.0, 1.0]], [[7.0, 4.0, 4.0], [5.0, 4.0, 7.0], [1.0, 8.0, 2.0]]], [[[7.0, 4.0, 1.0], 
[8.0, 6.0, 1.0], [4.0, 3.0, 7.0]], [[8.0, 5.0, 7.0], [1.0, 8.0, 5.0], [4.0, 8.0, 5.0]], [[3.0, 4.0, 7.0], [4.0, 8.0, 2.0], 
[1.0, 6.0, 4.0]]], [[[4.0, 8.0, 3.0], [3.0, 7.0, 1.0], [2.0, 5.0, 7.0]], [[1.0, 1.0, 6.0], [8.0, 7.0, 7.0], [6.0, 1.0, 2.0]], 
[[7.0, 1.0, 5.0], [3.0, 8.0, 5.0], [8.0, 6.0, 7.0]]], [[[2.0, 6.0, 5.0], [2.0, 6.0, 8.0], [7.0, 2.0, 2.0]], [[5.0, 6.0, 8.0], 
[7.0, 5.0, 7.0], [3.0, 6.0, 6.0]], [[3.0, 1.0, 5.0], [8.0, 8.0, 6.0], [6.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PRCC(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 1.0], [5.0, 7.0, 3.0], [4.0, 5.0, 2.0]], [[7.0, 1.0, 5.0], [7.0, 4.0, 6.0], [2.0, 4.0, 
2.0]], [[2.0, 5.0, 5.0], [3.0, 2.0, 4.0], [8.0, 4.0, 8.0]]], [[[6.0, 7.0, 8.0], [7.0, 1.0, 4.0], [7.0, 2.0, 5.0]], [[5.0, 3.0, 
3.0], [3.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 1.0], [1.0, 3.0, 2.0], [1.0, 3.0, 8.0]]], [[[4.0, 1.0, 8.0], [1.0, 6.0, 
7.0], [3.0, 6.0, 8.0]], [[2.0, 4.0, 2.0], [2.0, 6.0, 8.0], [5.0, 5.0, 3.0]], [[4.0, 1.0, 2.0], [2.0, 4.0, 5.0], [6.0, 5.0, 
2.0]]], [[[1.0, 4.0, 6.0], [6.0, 3.0, 3.0], [4.0, 5.0, 7.0]], [[8.0, 6.0, 5.0], [1.0, 7.0, 8.0], [3.0, 2.0, 8.0]], [[7.0, 8.0, 
1.0], [3.0, 3.0, 8.0], [7.0, 3.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 6.0, 2.0], [3.0, 6.0, 3.0], [1.0, 1.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 2.0, 7.0], [5.0, 7.0, 1.0], [6.0, 2.0, 3.0]]]))
      arg[2,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 1.0], [5.0, 7.0, 3.0], [4.0, 5.0, 2.0]], [[7.0, 1.0, 5.0], [7.0, 4.0, 6.0], [2.0, 
4.0, 2.0]], [[2.0, 5.0, 5.0], [3.0, 2.0, 4.0], [8.0, 4.0, 8.0]]], [[[6.0, 7.0, 8.0], [7.0, 1.0, 4.0], [7.0, 2.0, 5.0]], [[5.0, 
3.0, 3.0], [3.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 1.0], [1.0, 3.0, 2.0], [1.0, 3.0, 8.0]]], [[[4.0, 1.0, 8.0], [1.0, 
6.0, 7.0], [3.0, 6.0, 8.0]], [[2.0, 6.0, 2.0], [3.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[4.0, 1.0, 2.0], [2.0, 4.0, 5.0], [6.0, 5.0, 
2.0]]], [[[1.0, 4.0, 6.0], [6.0, 3.0, 3.0], [4.0, 5.0, 7.0]], [[8.0, 6.0, 5.0], [1.0, 7.0, 8.0], [3.0, 2.0, 8.0]], [[7.0, 8.0, 
1.0], [3.0, 3.0, 8.0], [7.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 1.0], [5.0, 7.0, 3.0], [4.0, 5.0, 2.0]], 
[[7.0, 1.0, 5.0], [7.0, 4.0, 6.0], [2.0, 4.0, 2.0]], [[2.0, 5.0, 5.0], [3.0, 2.0, 4.0], [8.0, 4.0, 8.0]]], [[[6.0, 7.0, 8.0], 
[7.0, 1.0, 4.0], [7.0, 2.0, 5.0]], [[5.0, 3.0, 3.0], [3.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 1.0], [1.0, 3.0, 2.0], 
[1.0, 3.0, 8.0]]], [[[4.0, 1.0, 8.0], [1.0, 6.0, 7.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 7.0], [5.0, 7.0, 1.0], [6.0, 2.0, 3.0]], 
[[4.0, 1.0, 2.0], [2.0, 4.0, 5.0], [6.0, 5.0, 2.0]]], [[[1.0, 4.0, 6.0], [6.0, 3.0, 3.0], [4.0, 5.0, 7.0]], [[8.0, 6.0, 5.0], 
[1.0, 7.0, 8.0], [3.0, 2.0, 8.0]], [[7.0, 8.0, 1.0], [3.0, 3.0, 8.0], [7.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLLL(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 2.0], [4.0, 7.0, 3.0], [5.0, 4.0, 4.0]], [[7.0, 3.0, 5.0], [4.0, 1.0, 7.0], [6.0, 7.0, 
1.0]], [[3.0, 2.0, 8.0], [5.0, 8.0, 3.0], [7.0, 2.0, 5.0]]], [[[4.0, 4.0, 1.0], [8.0, 6.0, 4.0], [7.0, 4.0, 3.0]], [[4.0, 7.0, 
3.0], [3.0, 3.0, 5.0], [6.0, 3.0, 3.0]], [[4.0, 6.0, 1.0], [6.0, 1.0, 4.0], [4.0, 6.0, 8.0]]], [[[3.0, 6.0, 6.0], [4.0, 4.0, 
2.0], [4.0, 2.0, 3.0]], [[8.0, 7.0, 6.0], [1.0, 5.0, 8.0], [3.0, 8.0, 7.0]], [[5.0, 8.0, 4.0], [8.0, 7.0, 3.0], [4.0, 2.0, 
8.0]]], [[[2.0, 5.0, 3.0], [4.0, 1.0, 2.0], [7.0, 5.0, 3.0]], [[1.0, 3.0, 8.0], [2.0, 6.0, 4.0], [8.0, 3.0, 5.0]], [[4.0, 2.0, 
3.0], [6.0, 4.0, 8.0], [4.0, 6.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0]]]))
      arg[2,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 2.0], [4.0, 7.0, 3.0], [5.0, 4.0, 4.0]], [[7.0, 3.0, 5.0], [4.0, 1.0, 7.0], [6.0, 
7.0, 1.0]], [[3.0, 2.0, 8.0], [5.0, 8.0, 3.0], [7.0, 2.0, 5.0]]], [[[4.0, 4.0, 1.0], [8.0, 6.0, 4.0], [7.0, 4.0, 3.0]], [[4.0, 
7.0, 3.0], [3.0, 3.0, 5.0], [6.0, 3.0, 3.0]], [[4.0, 6.0, 1.0], [6.0, 1.0, 4.0], [4.0, 6.0, 8.0]]], [[[3.0, 6.0, 6.0], [4.0, 
4.0, 2.0], [4.0, 2.0, 3.0]], [[8.0, 7.0, 6.0], [1.0, 5.0, 8.0], [3.0, 8.0, 7.0]], [[5.0, 8.0, 4.0], [8.0, 7.0, 3.0], [4.0, 2.0, 
4.0]]], [[[2.0, 5.0, 3.0], [4.0, 1.0, 2.0], [7.0, 5.0, 3.0]], [[1.0, 3.0, 8.0], [2.0, 6.0, 4.0], [8.0, 3.0, 5.0]], [[4.0, 2.0, 
3.0], [6.0, 4.0, 8.0], [4.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 2.0], [4.0, 7.0, 3.0], [5.0, 4.0, 4.0]], 
[[7.0, 3.0, 5.0], [4.0, 1.0, 7.0], [6.0, 7.0, 1.0]], [[3.0, 2.0, 8.0], [5.0, 8.0, 3.0], [7.0, 2.0, 5.0]]], [[[4.0, 4.0, 1.0], 
[8.0, 6.0, 4.0], [7.0, 4.0, 3.0]], [[4.0, 7.0, 3.0], [3.0, 3.0, 5.0], [6.0, 3.0, 3.0]], [[4.0, 6.0, 1.0], [6.0, 1.0, 4.0], 
[4.0, 6.0, 8.0]]], [[[3.0, 6.0, 6.0], [4.0, 4.0, 2.0], [4.0, 2.0, 3.0]], [[8.0, 7.0, 6.0], [1.0, 5.0, 8.0], [3.0, 8.0, 7.0]], 
[[5.0, 8.0, 4.0], [8.0, 7.0, 3.0], [4.0, 2.0, 6.0]]], [[[2.0, 5.0, 3.0], [4.0, 1.0, 2.0], [7.0, 5.0, 3.0]], [[1.0, 3.0, 8.0], 
[2.0, 6.0, 4.0], [8.0, 3.0, 5.0]], [[4.0, 2.0, 3.0], [6.0, 4.0, 8.0], [4.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LUCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 3.0], [8.0, 4.0, 8.0], [2.0, 1.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 7.0, 5.0], [1.0, 2.0, 
4.0]], [[1.0, 4.0, 4.0], [3.0, 8.0, 7.0], [6.0, 2.0, 5.0]]], [[[5.0, 5.0, 2.0], [4.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[2.0, 4.0, 
6.0], [6.0, 4.0, 8.0], [3.0, 1.0, 1.0]], [[1.0, 6.0, 1.0], [2.0, 1.0, 7.0], [1.0, 2.0, 4.0]]], [[[5.0, 8.0, 5.0], [8.0, 6.0, 
6.0], [7.0, 5.0, 8.0]], [[6.0, 3.0, 1.0], [2.0, 5.0, 4.0], [4.0, 8.0, 2.0]], [[8.0, 5.0, 5.0], [8.0, 5.0, 3.0], [8.0, 3.0, 
7.0]]], [[[1.0, 5.0, 7.0], [7.0, 3.0, 5.0], [4.0, 2.0, 7.0]], [[7.0, 8.0, 6.0], [8.0, 4.0, 4.0], [5.0, 8.0, 6.0]], [[7.0, 8.0, 
7.0], [6.0, 8.0, 7.0], [6.0, 4.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 7.0, 3.0], [1.0, 3.0, 6.0], [6.0, 8.0, 5.0]], [[4.0, 5.0, 7.0], [4.0, 4.0, 2.0], [5.0, 2.0, 
3.0]]], [[[2.0, 7.0, 2.0], [5.0, 4.0, 1.0], [2.0, 4.0, 2.0]], [[7.0, 2.0, 5.0], [6.0, 6.0, 8.0], [1.0, 8.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 8.0], [3.0, 6.0, 8.0], [1.0, 1.0, 4.0]], [[4.0, 1.0, 3.0], [4.0, 5.0, 
3.0], [7.0, 2.0, 6.0]]], [[[6.0, 8.0, 6.0], [2.0, 3.0, 7.0], [4.0, 3.0, 2.0]], [[6.0, 5.0, 7.0], [7.0, 4.0, 1.0], [4.0, 1.0, 
3.0]]]]))
      arg[2:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 3.0], [8.0, 4.0, 8.0], [2.0, 1.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 7.0, 5.0], [1.0, 
2.0, 4.0]], [[1.0, 4.0, 4.0], [3.0, 8.0, 7.0], [6.0, 2.0, 5.0]]], [[[5.0, 5.0, 2.0], [4.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[2.0, 
4.0, 6.0], [6.0, 4.0, 8.0], [3.0, 1.0, 1.0]], [[1.0, 6.0, 1.0], [2.0, 1.0, 7.0], [1.0, 2.0, 4.0]]], [[[6.0, 7.0, 3.0], [1.0, 
3.0, 6.0], [6.0, 8.0, 5.0]], [[4.0, 5.0, 7.0], [4.0, 4.0, 2.0], [5.0, 2.0, 3.0]], [[8.0, 5.0, 5.0], [8.0, 5.0, 3.0], [8.0, 3.0, 
7.0]]], [[[2.0, 7.0, 2.0], [5.0, 4.0, 1.0], [2.0, 4.0, 2.0]], [[7.0, 2.0, 5.0], [6.0, 6.0, 8.0], [1.0, 8.0, 1.0]], [[7.0, 8.0, 
7.0], [6.0, 8.0, 7.0], [6.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 3.0], [8.0, 4.0, 8.0], [2.0, 1.0, 6.0]], 
[[2.0, 1.0, 5.0], [2.0, 7.0, 5.0], [1.0, 2.0, 4.0]], [[1.0, 4.0, 4.0], [3.0, 8.0, 7.0], [6.0, 2.0, 5.0]]], [[[5.0, 5.0, 2.0], 
[4.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[2.0, 4.0, 6.0], [6.0, 4.0, 8.0], [3.0, 1.0, 1.0]], [[1.0, 6.0, 1.0], [2.0, 1.0, 7.0], 
[1.0, 2.0, 4.0]]], [[[3.0, 5.0, 8.0], [3.0, 6.0, 8.0], [1.0, 1.0, 4.0]], [[4.0, 1.0, 3.0], [4.0, 5.0, 3.0], [7.0, 2.0, 6.0]], 
[[8.0, 5.0, 5.0], [8.0, 5.0, 3.0], [8.0, 3.0, 7.0]]], [[[6.0, 8.0, 6.0], [2.0, 3.0, 7.0], [4.0, 3.0, 2.0]], [[6.0, 5.0, 7.0], 
[7.0, 4.0, 1.0], [4.0, 1.0, 3.0]], [[7.0, 8.0, 7.0], [6.0, 8.0, 7.0], [6.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RCPN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 8.0], [3.0, 6.0, 6.0], [8.0, 2.0, 8.0]], [[8.0, 4.0, 5.0], [8.0, 1.0, 1.0], [1.0, 3.0, 
1.0]], [[5.0, 3.0, 5.0], [8.0, 8.0, 1.0], [8.0, 7.0, 4.0]]], [[[8.0, 6.0, 7.0], [4.0, 6.0, 3.0], [4.0, 4.0, 1.0]], [[1.0, 1.0, 
3.0], [6.0, 8.0, 3.0], [2.0, 5.0, 3.0]], [[5.0, 3.0, 8.0], [1.0, 8.0, 4.0], [4.0, 8.0, 4.0]]], [[[7.0, 7.0, 6.0], [7.0, 5.0, 
8.0], [8.0, 8.0, 5.0]], [[3.0, 5.0, 7.0], [4.0, 5.0, 5.0], [6.0, 5.0, 6.0]], [[7.0, 6.0, 3.0], [5.0, 5.0, 5.0], [4.0, 5.0, 
3.0]]], [[[8.0, 1.0, 6.0], [4.0, 8.0, 5.0], [6.0, 2.0, 5.0]], [[2.0, 4.0, 2.0], [7.0, 4.0, 6.0], [5.0, 5.0, 8.0]], [[5.0, 5.0, 
5.0], [5.0, 3.0, 6.0], [2.0, 2.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 8.0, 3.0], [3.0, 5.0, 4.0], [1.0, 1.0, 2.0]], [[5.0, 3.0, 2.0], [5.0, 6.0, 5.0], [5.0, 3.0, 
6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 1.0, 8.0], [5.0, 1.0, 8.0], [8.0, 6.0, 6.0]], [[7.0, 8.0, 4.0], [1.0, 3.0, 1.0], 
[8.0, 2.0, 3.0]]]))
      arg[1:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 8.0], [3.0, 6.0, 6.0], [8.0, 2.0, 8.0]], [[8.0, 4.0, 5.0], [8.0, 1.0, 1.0], [1.0, 
3.0, 1.0]], [[5.0, 3.0, 5.0], [8.0, 8.0, 1.0], [8.0, 7.0, 4.0]]], [[[8.0, 6.0, 7.0], [3.0, 8.0, 3.0], [4.0, 4.0, 1.0]], [[1.0, 
1.0, 3.0], [3.0, 5.0, 4.0], [2.0, 5.0, 3.0]], [[5.0, 3.0, 8.0], [1.0, 1.0, 2.0], [4.0, 8.0, 4.0]]], [[[7.0, 7.0, 6.0], [5.0, 
3.0, 2.0], [8.0, 8.0, 5.0]], [[3.0, 5.0, 7.0], [5.0, 6.0, 5.0], [6.0, 5.0, 6.0]], [[7.0, 6.0, 3.0], [5.0, 3.0, 6.0], [4.0, 5.0, 
3.0]]], [[[8.0, 1.0, 6.0], [4.0, 8.0, 5.0], [6.0, 2.0, 5.0]], [[2.0, 4.0, 2.0], [7.0, 4.0, 6.0], [5.0, 5.0, 8.0]], [[5.0, 5.0, 
5.0], [5.0, 3.0, 6.0], [2.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 8.0], [3.0, 6.0, 6.0], [8.0, 2.0, 8.0]], 
[[8.0, 4.0, 5.0], [8.0, 1.0, 1.0], [1.0, 3.0, 1.0]], [[5.0, 3.0, 5.0], [8.0, 8.0, 1.0], [8.0, 7.0, 4.0]]], [[[8.0, 6.0, 7.0], 
[5.0, 1.0, 8.0], [4.0, 4.0, 1.0]], [[1.0, 1.0, 3.0], [5.0, 1.0, 8.0], [2.0, 5.0, 3.0]], [[5.0, 3.0, 8.0], [8.0, 6.0, 6.0], 
[4.0, 8.0, 4.0]]], [[[7.0, 7.0, 6.0], [7.0, 8.0, 4.0], [8.0, 8.0, 5.0]], [[3.0, 5.0, 7.0], [1.0, 3.0, 1.0], [6.0, 5.0, 6.0]], 
[[7.0, 6.0, 3.0], [8.0, 2.0, 3.0], [4.0, 5.0, 3.0]]], [[[8.0, 1.0, 6.0], [4.0, 8.0, 5.0], [6.0, 2.0, 5.0]], [[2.0, 4.0, 2.0], 
[7.0, 4.0, 6.0], [5.0, 5.0, 8.0]], [[5.0, 5.0, 5.0], [5.0, 3.0, 6.0], [2.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_URLN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 6.0], [1.0, 6.0, 1.0], [2.0, 4.0, 8.0]], [[8.0, 5.0, 2.0], [6.0, 5.0, 5.0], [3.0, 7.0, 
6.0]], [[2.0, 1.0, 7.0], [1.0, 2.0, 6.0], [3.0, 5.0, 3.0]]], [[[7.0, 5.0, 4.0], [3.0, 1.0, 1.0], [3.0, 1.0, 3.0]], [[8.0, 6.0, 
7.0], [3.0, 5.0, 8.0], [4.0, 5.0, 6.0]], [[8.0, 8.0, 7.0], [7.0, 8.0, 3.0], [7.0, 3.0, 2.0]]], [[[1.0, 4.0, 1.0], [6.0, 4.0, 
3.0], [2.0, 2.0, 2.0]], [[1.0, 2.0, 6.0], [6.0, 1.0, 5.0], [4.0, 6.0, 5.0]], [[6.0, 4.0, 5.0], [5.0, 1.0, 6.0], [2.0, 1.0, 
3.0]]], [[[5.0, 2.0, 3.0], [7.0, 6.0, 6.0], [7.0, 8.0, 4.0]], [[2.0, 3.0, 8.0], [1.0, 3.0, 7.0], [5.0, 1.0, 4.0]], [[3.0, 2.0, 
6.0], [7.0, 2.0, 5.0], [3.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 6.0, 2.0]]], [[[3.0, 5.0, 1.0]]], [[[8.0, 1.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 2.0]]], [[[4.0, 2.0, 8.0]]], [[[3.0, 5.0, 6.0]]]]))
      arg[:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 6.0], [1.0, 6.0, 1.0], [2.0, 4.0, 8.0]], [[8.0, 5.0, 2.0], [6.0, 5.0, 5.0], [1.0, 
6.0, 2.0]], [[2.0, 1.0, 7.0], [1.0, 2.0, 6.0], [3.0, 5.0, 3.0]]], [[[7.0, 5.0, 4.0], [3.0, 1.0, 1.0], [3.0, 1.0, 3.0]], [[8.0, 
6.0, 7.0], [3.0, 5.0, 8.0], [3.0, 5.0, 1.0]], [[8.0, 8.0, 7.0], [7.0, 8.0, 3.0], [7.0, 3.0, 2.0]]], [[[1.0, 4.0, 1.0], [6.0, 
4.0, 3.0], [2.0, 2.0, 2.0]], [[1.0, 2.0, 6.0], [6.0, 1.0, 5.0], [8.0, 1.0, 2.0]], [[6.0, 4.0, 5.0], [5.0, 1.0, 6.0], [2.0, 1.0, 
3.0]]], [[[5.0, 2.0, 3.0], [7.0, 6.0, 6.0], [7.0, 8.0, 4.0]], [[2.0, 3.0, 8.0], [1.0, 3.0, 7.0], [5.0, 1.0, 4.0]], [[3.0, 2.0, 
6.0], [7.0, 2.0, 5.0], [3.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 6.0], [1.0, 6.0, 1.0], [2.0, 4.0, 8.0]], 
[[8.0, 5.0, 2.0], [6.0, 5.0, 5.0], [4.0, 1.0, 2.0]], [[2.0, 1.0, 7.0], [1.0, 2.0, 6.0], [3.0, 5.0, 3.0]]], [[[7.0, 5.0, 4.0], 
[3.0, 1.0, 1.0], [3.0, 1.0, 3.0]], [[8.0, 6.0, 7.0], [3.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[8.0, 8.0, 7.0], [7.0, 8.0, 3.0], 
[7.0, 3.0, 2.0]]], [[[1.0, 4.0, 1.0], [6.0, 4.0, 3.0], [2.0, 2.0, 2.0]], [[1.0, 2.0, 6.0], [6.0, 1.0, 5.0], [3.0, 5.0, 6.0]], 
[[6.0, 4.0, 5.0], [5.0, 1.0, 6.0], [2.0, 1.0, 3.0]]], [[[5.0, 2.0, 3.0], [7.0, 6.0, 6.0], [7.0, 8.0, 4.0]], [[2.0, 3.0, 8.0], 
[1.0, 3.0, 7.0], [5.0, 1.0, 4.0]], [[3.0, 2.0, 6.0], [7.0, 2.0, 5.0], [3.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPLN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 1.0], [6.0, 7.0, 1.0], [5.0, 5.0, 3.0]], [[2.0, 5.0, 1.0], [3.0, 7.0, 7.0], [4.0, 8.0, 
6.0]], [[4.0, 6.0, 3.0], [8.0, 8.0, 1.0], [4.0, 6.0, 5.0]]], [[[4.0, 7.0, 3.0], [6.0, 8.0, 6.0], [8.0, 1.0, 1.0]], [[4.0, 2.0, 
3.0], [2.0, 6.0, 5.0], [5.0, 8.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 4.0], [1.0, 7.0, 6.0]]], [[[4.0, 6.0, 5.0], [1.0, 7.0, 
7.0], [1.0, 5.0, 5.0]], [[2.0, 8.0, 3.0], [5.0, 4.0, 7.0], [5.0, 6.0, 4.0]], [[2.0, 3.0, 4.0], [8.0, 7.0, 4.0], [6.0, 3.0, 
8.0]]], [[[3.0, 2.0, 1.0], [3.0, 7.0, 4.0], [6.0, 7.0, 2.0]], [[7.0, 7.0, 4.0], [7.0, 5.0, 2.0], [7.0, 1.0, 5.0]], [[1.0, 8.0, 
6.0], [8.0, 4.0, 2.0], [6.0, 1.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 8.0, 4.0]], [[1.0, 5.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 1.0, 1.0]], [[3.0, 5.0, 8.0]]]))
      arg[2:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 1.0], [6.0, 7.0, 1.0], [5.0, 5.0, 3.0]], [[2.0, 5.0, 1.0], [3.0, 7.0, 7.0], [4.0, 
8.0, 6.0]], [[4.0, 6.0, 3.0], [8.0, 8.0, 1.0], [4.0, 6.0, 5.0]]], [[[4.0, 7.0, 3.0], [6.0, 8.0, 6.0], [8.0, 1.0, 1.0]], [[4.0, 
2.0, 3.0], [2.0, 6.0, 5.0], [5.0, 8.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 4.0], [1.0, 7.0, 6.0]]], [[[4.0, 6.0, 5.0], [1.0, 
7.0, 7.0], [1.0, 5.0, 5.0]], [[2.0, 8.0, 3.0], [5.0, 4.0, 7.0], [8.0, 8.0, 4.0]], [[2.0, 3.0, 4.0], [8.0, 7.0, 4.0], [6.0, 3.0, 
8.0]]], [[[3.0, 2.0, 1.0], [3.0, 7.0, 4.0], [6.0, 7.0, 2.0]], [[7.0, 7.0, 4.0], [7.0, 5.0, 2.0], [1.0, 5.0, 6.0]], [[1.0, 8.0, 
6.0], [8.0, 4.0, 2.0], [6.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 1.0], [6.0, 7.0, 1.0], [5.0, 5.0, 3.0]], 
[[2.0, 5.0, 1.0], [3.0, 7.0, 7.0], [4.0, 8.0, 6.0]], [[4.0, 6.0, 3.0], [8.0, 8.0, 1.0], [4.0, 6.0, 5.0]]], [[[4.0, 7.0, 3.0], 
[6.0, 8.0, 6.0], [8.0, 1.0, 1.0]], [[4.0, 2.0, 3.0], [2.0, 6.0, 5.0], [5.0, 8.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 4.0], 
[1.0, 7.0, 6.0]]], [[[4.0, 6.0, 5.0], [1.0, 7.0, 7.0], [1.0, 5.0, 5.0]], [[2.0, 8.0, 3.0], [5.0, 4.0, 7.0], [1.0, 1.0, 1.0]], 
[[2.0, 3.0, 4.0], [8.0, 7.0, 4.0], [6.0, 3.0, 8.0]]], [[[3.0, 2.0, 1.0], [3.0, 7.0, 4.0], [6.0, 7.0, 2.0]], [[7.0, 7.0, 4.0], 
[7.0, 5.0, 2.0], [3.0, 5.0, 8.0]], [[1.0, 8.0, 6.0], [8.0, 4.0, 2.0], [6.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PPCN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 2.0], [1.0, 8.0, 1.0], [7.0, 5.0, 4.0]], [[5.0, 1.0, 8.0], [2.0, 1.0, 1.0], [8.0, 4.0, 
3.0]], [[4.0, 5.0, 7.0], [4.0, 1.0, 8.0], [1.0, 6.0, 6.0]]], [[[8.0, 1.0, 3.0], [6.0, 1.0, 6.0], [7.0, 3.0, 6.0]], [[8.0, 6.0, 
7.0], [3.0, 4.0, 7.0], [5.0, 3.0, 2.0]], [[7.0, 4.0, 7.0], [2.0, 3.0, 8.0], [7.0, 6.0, 7.0]]], [[[5.0, 3.0, 6.0], [1.0, 7.0, 
6.0], [5.0, 1.0, 8.0]], [[6.0, 8.0, 8.0], [7.0, 8.0, 1.0], [5.0, 6.0, 7.0]], [[4.0, 3.0, 7.0], [4.0, 7.0, 1.0], [1.0, 6.0, 
4.0]]], [[[8.0, 7.0, 6.0], [7.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[1.0, 5.0, 4.0], [4.0, 5.0, 2.0], [8.0, 5.0, 6.0]], [[4.0, 4.0, 
4.0], [4.0, 3.0, 1.0], [1.0, 5.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 4.0, 2.0], [5.0, 3.0, 3.0], [5.0, 1.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 4.0, 4.0], [3.0, 6.0, 4.0], [7.0, 4.0, 1.0]]))
      arg[2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [1.0, 8.0, 1.0], [7.0, 5.0, 4.0]], [[5.0, 1.0, 8.0], [2.0, 1.0, 1.0], [8.0, 
4.0, 3.0]], [[4.0, 5.0, 7.0], [4.0, 1.0, 8.0], [1.0, 6.0, 6.0]]], [[[8.0, 1.0, 3.0], [6.0, 1.0, 6.0], [7.0, 3.0, 6.0]], [[8.0, 
6.0, 7.0], [3.0, 4.0, 7.0], [5.0, 3.0, 2.0]], [[7.0, 4.0, 7.0], [2.0, 3.0, 8.0], [7.0, 6.0, 7.0]]], [[[5.0, 3.0, 6.0], [1.0, 
7.0, 6.0], [5.0, 1.0, 8.0]], [[3.0, 4.0, 2.0], [5.0, 3.0, 3.0], [5.0, 1.0, 3.0]], [[4.0, 3.0, 7.0], [4.0, 7.0, 1.0], [1.0, 6.0, 
4.0]]], [[[8.0, 7.0, 6.0], [7.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[1.0, 5.0, 4.0], [4.0, 5.0, 2.0], [8.0, 5.0, 6.0]], [[4.0, 4.0, 
4.0], [4.0, 3.0, 1.0], [1.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 2.0], [1.0, 8.0, 1.0], [7.0, 5.0, 4.0]], 
[[5.0, 1.0, 8.0], [2.0, 1.0, 1.0], [8.0, 4.0, 3.0]], [[4.0, 5.0, 7.0], [4.0, 1.0, 8.0], [1.0, 6.0, 6.0]]], [[[8.0, 1.0, 3.0], 
[6.0, 1.0, 6.0], [7.0, 3.0, 6.0]], [[8.0, 6.0, 7.0], [3.0, 4.0, 7.0], [5.0, 3.0, 2.0]], [[7.0, 4.0, 7.0], [2.0, 3.0, 8.0], 
[7.0, 6.0, 7.0]]], [[[5.0, 3.0, 6.0], [1.0, 7.0, 6.0], [5.0, 1.0, 8.0]], [[1.0, 4.0, 4.0], [3.0, 6.0, 4.0], [7.0, 4.0, 1.0]], 
[[4.0, 3.0, 7.0], [4.0, 7.0, 1.0], [1.0, 6.0, 4.0]]], [[[8.0, 7.0, 6.0], [7.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[1.0, 5.0, 4.0], 
[4.0, 5.0, 2.0], [8.0, 5.0, 6.0]], [[4.0, 4.0, 4.0], [4.0, 3.0, 1.0], [1.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCUN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 3.0], [6.0, 4.0, 1.0], [7.0, 8.0, 6.0]], [[7.0, 4.0, 7.0], [7.0, 5.0, 3.0], [3.0, 7.0, 
5.0]], [[3.0, 4.0, 8.0], [6.0, 5.0, 4.0], [7.0, 2.0, 3.0]]], [[[8.0, 8.0, 7.0], [1.0, 3.0, 3.0], [2.0, 1.0, 3.0]], [[3.0, 1.0, 
3.0], [4.0, 6.0, 7.0], [5.0, 3.0, 7.0]], [[5.0, 3.0, 4.0], [4.0, 6.0, 6.0], [7.0, 6.0, 8.0]]], [[[1.0, 5.0, 7.0], [2.0, 8.0, 
7.0], [6.0, 5.0, 5.0]], [[7.0, 2.0, 6.0], [5.0, 2.0, 1.0], [8.0, 8.0, 7.0]], [[2.0, 1.0, 7.0], [6.0, 4.0, 4.0], [4.0, 6.0, 
1.0]]], [[[7.0, 3.0, 7.0], [6.0, 7.0, 5.0], [1.0, 6.0, 4.0]], [[8.0, 4.0, 1.0], [2.0, 1.0, 6.0], [1.0, 1.0, 1.0]], [[2.0, 6.0, 
8.0], [5.0, 1.0, 1.0], [1.0, 5.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 8.0, 2.0], [7.0, 1.0, 1.0]], [[6.0, 7.0, 3.0], [5.0, 5.0, 2.0]], [[5.0, 7.0, 6.0], [1.0, 
2.0, 3.0]]], [[[4.0, 1.0, 2.0], [8.0, 7.0, 2.0]], [[2.0, 2.0, 4.0], [7.0, 6.0, 1.0]], [[1.0, 3.0, 3.0], [5.0, 3.0, 8.0]]], 
[[[3.0, 8.0, 6.0], [8.0, 2.0, 1.0]], [[5.0, 2.0, 3.0], [2.0, 1.0, 5.0]], [[5.0, 8.0, 5.0], [8.0, 8.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 5.0], [3.0, 6.0, 8.0]], [[3.0, 3.0, 6.0], [8.0, 6.0, 8.0]], [[1.0, 5.0, 
3.0], [1.0, 6.0, 5.0]]], [[[7.0, 3.0, 2.0], [2.0, 4.0, 2.0]], [[5.0, 3.0, 8.0], [5.0, 7.0, 6.0]], [[1.0, 7.0, 3.0], [5.0, 4.0, 
4.0]]], [[[3.0, 8.0, 5.0], [2.0, 8.0, 5.0]], [[1.0, 4.0, 1.0], [2.0, 7.0, 5.0]], [[5.0, 2.0, 4.0], [6.0, 2.0, 6.0]]]]))
      arg[:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 2.0], [7.0, 1.0, 1.0], [7.0, 8.0, 6.0]], [[6.0, 7.0, 3.0], [5.0, 5.0, 2.0], [3.0, 
7.0, 5.0]], [[5.0, 7.0, 6.0], [1.0, 2.0, 3.0], [7.0, 2.0, 3.0]]], [[[4.0, 1.0, 2.0], [8.0, 7.0, 2.0], [2.0, 1.0, 3.0]], [[2.0, 
2.0, 4.0], [7.0, 6.0, 1.0], [5.0, 3.0, 7.0]], [[1.0, 3.0, 3.0], [5.0, 3.0, 8.0], [7.0, 6.0, 8.0]]], [[[3.0, 8.0, 6.0], [8.0, 
2.0, 1.0], [6.0, 5.0, 5.0]], [[5.0, 2.0, 3.0], [2.0, 1.0, 5.0], [8.0, 8.0, 7.0]], [[5.0, 8.0, 5.0], [8.0, 8.0, 2.0], [4.0, 6.0, 
1.0]]], [[[7.0, 3.0, 7.0], [6.0, 7.0, 5.0], [1.0, 6.0, 4.0]], [[8.0, 4.0, 1.0], [2.0, 1.0, 6.0], [1.0, 1.0, 1.0]], [[2.0, 6.0, 
8.0], [5.0, 1.0, 1.0], [1.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 5.0], [3.0, 6.0, 8.0], [7.0, 8.0, 6.0]], 
[[3.0, 3.0, 6.0], [8.0, 6.0, 8.0], [3.0, 7.0, 5.0]], [[1.0, 5.0, 3.0], [1.0, 6.0, 5.0], [7.0, 2.0, 3.0]]], [[[7.0, 3.0, 2.0], 
[2.0, 4.0, 2.0], [2.0, 1.0, 3.0]], [[5.0, 3.0, 8.0], [5.0, 7.0, 6.0], [5.0, 3.0, 7.0]], [[1.0, 7.0, 3.0], [5.0, 4.0, 4.0], 
[7.0, 6.0, 8.0]]], [[[3.0, 8.0, 5.0], [2.0, 8.0, 5.0], [6.0, 5.0, 5.0]], [[1.0, 4.0, 1.0], [2.0, 7.0, 5.0], [8.0, 8.0, 7.0]], 
[[5.0, 2.0, 4.0], [6.0, 2.0, 6.0], [4.0, 6.0, 1.0]]], [[[7.0, 3.0, 7.0], [6.0, 7.0, 5.0], [1.0, 6.0, 4.0]], [[8.0, 4.0, 1.0], 
[2.0, 1.0, 6.0], [1.0, 1.0, 1.0]], [[2.0, 6.0, 8.0], [5.0, 1.0, 1.0], [1.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UUCN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 6.0], [7.0, 3.0, 1.0], [1.0, 6.0, 2.0]], [[8.0, 2.0, 6.0], [1.0, 6.0, 5.0], [3.0, 6.0, 
8.0]], [[3.0, 7.0, 3.0], [7.0, 1.0, 1.0], [7.0, 4.0, 1.0]]], [[[5.0, 8.0, 8.0], [5.0, 7.0, 3.0], [2.0, 4.0, 4.0]], [[6.0, 8.0, 
1.0], [6.0, 7.0, 4.0], [3.0, 7.0, 1.0]], [[2.0, 2.0, 3.0], [6.0, 1.0, 4.0], [8.0, 7.0, 6.0]]], [[[2.0, 2.0, 5.0], [7.0, 4.0, 
1.0], [7.0, 3.0, 7.0]], [[6.0, 1.0, 2.0], [1.0, 2.0, 1.0], [3.0, 5.0, 8.0]], [[6.0, 6.0, 7.0], [4.0, 1.0, 2.0], [2.0, 2.0, 
8.0]]], [[[3.0, 7.0, 5.0], [2.0, 7.0, 4.0], [2.0, 7.0, 6.0]], [[5.0, 1.0, 6.0], [3.0, 8.0, 8.0], [1.0, 2.0, 7.0]], [[1.0, 2.0, 
3.0], [5.0, 4.0, 4.0], [8.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 4.0, 3.0], [5.0, 5.0, 2.0], [5.0, 6.0, 1.0]], [[6.0, 3.0, 3.0], [7.0, 2.0, 7.0], [1.0, 7.0, 
8.0]]], [[[6.0, 3.0, 7.0], [3.0, 5.0, 1.0], [3.0, 3.0, 1.0]], [[4.0, 4.0, 2.0], [3.0, 8.0, 8.0], [4.0, 4.0, 5.0]]], [[[4.0, 
4.0, 6.0], [2.0, 3.0, 6.0], [2.0, 4.0, 5.0]], [[7.0, 6.0, 3.0], [1.0, 6.0, 2.0], [2.0, 4.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 6.0], [6.0, 3.0, 1.0], [6.0, 8.0, 5.0]], [[1.0, 7.0, 4.0], [1.0, 4.0, 
7.0], [1.0, 4.0, 5.0]]], [[[5.0, 3.0, 6.0], [4.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[6.0, 6.0, 5.0], [6.0, 4.0, 6.0], [4.0, 8.0, 
2.0]]], [[[6.0, 2.0, 2.0], [3.0, 1.0, 3.0], [6.0, 5.0, 1.0]], [[6.0, 7.0, 6.0], [5.0, 7.0, 5.0], [3.0, 6.0, 3.0]]]]))
      arg[:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 3.0], [5.0, 5.0, 2.0], [5.0, 6.0, 1.0]], [[6.0, 3.0, 3.0], [7.0, 2.0, 7.0], [1.0, 
7.0, 8.0]], [[3.0, 7.0, 3.0], [7.0, 1.0, 1.0], [7.0, 4.0, 1.0]]], [[[6.0, 3.0, 7.0], [3.0, 5.0, 1.0], [3.0, 3.0, 1.0]], [[4.0, 
4.0, 2.0], [3.0, 8.0, 8.0], [4.0, 4.0, 5.0]], [[2.0, 2.0, 3.0], [6.0, 1.0, 4.0], [8.0, 7.0, 6.0]]], [[[4.0, 4.0, 6.0], [2.0, 
3.0, 6.0], [2.0, 4.0, 5.0]], [[7.0, 6.0, 3.0], [1.0, 6.0, 2.0], [2.0, 4.0, 2.0]], [[6.0, 6.0, 7.0], [4.0, 1.0, 2.0], [2.0, 2.0, 
8.0]]], [[[3.0, 7.0, 5.0], [2.0, 7.0, 4.0], [2.0, 7.0, 6.0]], [[5.0, 1.0, 6.0], [3.0, 8.0, 8.0], [1.0, 2.0, 7.0]], [[1.0, 2.0, 
3.0], [5.0, 4.0, 4.0], [8.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 6.0], [6.0, 3.0, 1.0], [6.0, 8.0, 5.0]], 
[[1.0, 7.0, 4.0], [1.0, 4.0, 7.0], [1.0, 4.0, 5.0]], [[3.0, 7.0, 3.0], [7.0, 1.0, 1.0], [7.0, 4.0, 1.0]]], [[[5.0, 3.0, 6.0], 
[4.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[6.0, 6.0, 5.0], [6.0, 4.0, 6.0], [4.0, 8.0, 2.0]], [[2.0, 2.0, 3.0], [6.0, 1.0, 4.0], 
[8.0, 7.0, 6.0]]], [[[6.0, 2.0, 2.0], [3.0, 1.0, 3.0], [6.0, 5.0, 1.0]], [[6.0, 7.0, 6.0], [5.0, 7.0, 5.0], [3.0, 6.0, 3.0]], 
[[6.0, 6.0, 7.0], [4.0, 1.0, 2.0], [2.0, 2.0, 8.0]]], [[[3.0, 7.0, 5.0], [2.0, 7.0, 4.0], [2.0, 7.0, 6.0]], [[5.0, 1.0, 6.0], 
[3.0, 8.0, 8.0], [1.0, 2.0, 7.0]], [[1.0, 2.0, 3.0], [5.0, 4.0, 4.0], [8.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCUC(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 1.0], [4.0, 4.0, 8.0], [2.0, 4.0, 5.0]], [[2.0, 1.0, 2.0], [3.0, 2.0, 1.0], [7.0, 4.0, 
3.0]], [[4.0, 4.0, 3.0], [8.0, 8.0, 5.0], [4.0, 1.0, 8.0]]], [[[5.0, 4.0, 1.0], [2.0, 7.0, 8.0], [4.0, 4.0, 3.0]], [[5.0, 8.0, 
3.0], [6.0, 4.0, 3.0], [7.0, 6.0, 3.0]], [[5.0, 8.0, 6.0], [1.0, 3.0, 6.0], [5.0, 6.0, 2.0]]], [[[3.0, 4.0, 5.0], [6.0, 4.0, 
5.0], [1.0, 7.0, 2.0]], [[6.0, 3.0, 6.0], [1.0, 7.0, 1.0], [7.0, 4.0, 7.0]], [[8.0, 3.0, 7.0], [3.0, 1.0, 1.0], [4.0, 7.0, 
5.0]]], [[[5.0, 1.0, 7.0], [2.0, 7.0, 3.0], [1.0, 2.0, 7.0]], [[5.0, 3.0, 8.0], [5.0, 1.0, 2.0], [8.0, 1.0, 8.0]], [[6.0, 2.0, 
4.0], [2.0, 7.0, 4.0], [8.0, 4.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 8.0, 1.0], [6.0, 8.0, 4.0]], [[5.0, 6.0, 6.0], [7.0, 5.0, 8.0]], [[8.0, 7.0, 8.0], [5.0, 
8.0, 2.0]]], [[[2.0, 3.0, 3.0], [8.0, 2.0, 6.0]], [[8.0, 1.0, 3.0], [4.0, 6.0, 6.0]], [[4.0, 1.0, 4.0], [7.0, 7.0, 7.0]]], 
[[[2.0, 5.0, 1.0], [6.0, 6.0, 5.0]], [[2.0, 4.0, 7.0], [1.0, 3.0, 4.0]], [[1.0, 1.0, 7.0], [4.0, 2.0, 5.0]]], [[[5.0, 5.0, 
1.0], [6.0, 6.0, 1.0]], [[6.0, 2.0, 6.0], [2.0, 5.0, 4.0]], [[5.0, 6.0, 1.0], [8.0, 3.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 5.0], [1.0, 3.0, 1.0]], [[6.0, 5.0, 3.0], [3.0, 8.0, 3.0]], [[2.0, 6.0, 
8.0], [3.0, 3.0, 7.0]]], [[[8.0, 6.0, 3.0], [7.0, 7.0, 7.0]], [[6.0, 6.0, 8.0], [8.0, 1.0, 6.0]], [[6.0, 6.0, 1.0], [7.0, 5.0, 
2.0]]], [[[6.0, 3.0, 8.0], [7.0, 3.0, 4.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 1.0]], [[2.0, 7.0, 1.0], [6.0, 7.0, 2.0]]], [[[4.0, 
5.0, 5.0], [7.0, 3.0, 2.0]], [[2.0, 3.0, 7.0], [1.0, 2.0, 1.0]], [[5.0, 7.0, 3.0], [1.0, 7.0, 3.0]]]]))
      arg[:,:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 1.0], [6.0, 8.0, 4.0], [2.0, 4.0, 5.0]], [[5.0, 6.0, 6.0], [7.0, 5.0, 8.0], [7.0, 
4.0, 3.0]], [[8.0, 7.0, 8.0], [5.0, 8.0, 2.0], [4.0, 1.0, 8.0]]], [[[2.0, 3.0, 3.0], [8.0, 2.0, 6.0], [4.0, 4.0, 3.0]], [[8.0, 
1.0, 3.0], [4.0, 6.0, 6.0], [7.0, 6.0, 3.0]], [[4.0, 1.0, 4.0], [7.0, 7.0, 7.0], [5.0, 6.0, 2.0]]], [[[2.0, 5.0, 1.0], [6.0, 
6.0, 5.0], [1.0, 7.0, 2.0]], [[2.0, 4.0, 7.0], [1.0, 3.0, 4.0], [7.0, 4.0, 7.0]], [[1.0, 1.0, 7.0], [4.0, 2.0, 5.0], [4.0, 7.0, 
5.0]]], [[[5.0, 5.0, 1.0], [6.0, 6.0, 1.0], [1.0, 2.0, 7.0]], [[6.0, 2.0, 6.0], [2.0, 5.0, 4.0], [8.0, 1.0, 8.0]], [[5.0, 6.0, 
1.0], [8.0, 3.0, 2.0], [8.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 5.0], [1.0, 3.0, 1.0], [2.0, 4.0, 5.0]], 
[[6.0, 5.0, 3.0], [3.0, 8.0, 3.0], [7.0, 4.0, 3.0]], [[2.0, 6.0, 8.0], [3.0, 3.0, 7.0], [4.0, 1.0, 8.0]]], [[[8.0, 6.0, 3.0], 
[7.0, 7.0, 7.0], [4.0, 4.0, 3.0]], [[6.0, 6.0, 8.0], [8.0, 1.0, 6.0], [7.0, 6.0, 3.0]], [[6.0, 6.0, 1.0], [7.0, 5.0, 2.0], 
[5.0, 6.0, 2.0]]], [[[6.0, 3.0, 8.0], [7.0, 3.0, 4.0], [1.0, 7.0, 2.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 1.0], [7.0, 4.0, 7.0]], 
[[2.0, 7.0, 1.0], [6.0, 7.0, 2.0], [4.0, 7.0, 5.0]]], [[[4.0, 5.0, 5.0], [7.0, 3.0, 2.0], [1.0, 2.0, 7.0]], [[2.0, 3.0, 7.0], 
[1.0, 2.0, 1.0], [8.0, 1.0, 8.0]], [[5.0, 7.0, 3.0], [1.0, 7.0, 3.0], [8.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LRUN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 2.0], [6.0, 5.0, 6.0], [4.0, 5.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 4.0, 3.0], [4.0, 8.0, 
5.0]], [[8.0, 2.0, 6.0], [4.0, 2.0, 6.0], [6.0, 3.0, 3.0]]], [[[4.0, 7.0, 6.0], [1.0, 3.0, 1.0], [8.0, 6.0, 7.0]], [[3.0, 5.0, 
3.0], [1.0, 8.0, 5.0], [6.0, 6.0, 4.0]], [[1.0, 8.0, 8.0], [5.0, 8.0, 5.0], [1.0, 2.0, 3.0]]], [[[8.0, 3.0, 1.0], [6.0, 6.0, 
8.0], [8.0, 2.0, 7.0]], [[7.0, 1.0, 6.0], [8.0, 1.0, 3.0], [8.0, 3.0, 4.0]], [[7.0, 7.0, 1.0], [4.0, 3.0, 5.0], [3.0, 7.0, 
7.0]]], [[[6.0, 4.0, 5.0], [6.0, 3.0, 7.0], [4.0, 6.0, 2.0]], [[5.0, 5.0, 1.0], [6.0, 5.0, 1.0], [5.0, 1.0, 4.0]], [[4.0, 4.0, 
8.0], [4.0, 5.0, 4.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 8.0, 3.0], [4.0, 6.0, 5.0]]], [[[6.0, 6.0, 7.0], [5.0, 5.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 5.0], [4.0, 2.0, 4.0]]], [[[7.0, 4.0, 6.0], [8.0, 1.0, 2.0]]]]))
      arg[2:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 2.0], [6.0, 5.0, 6.0], [4.0, 5.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 4.0, 3.0], [4.0, 
8.0, 5.0]], [[8.0, 2.0, 6.0], [4.0, 2.0, 6.0], [6.0, 3.0, 3.0]]], [[[4.0, 7.0, 6.0], [1.0, 3.0, 1.0], [8.0, 6.0, 7.0]], [[3.0, 
5.0, 3.0], [1.0, 8.0, 5.0], [6.0, 6.0, 4.0]], [[1.0, 8.0, 8.0], [5.0, 8.0, 5.0], [1.0, 2.0, 3.0]]], [[[8.0, 3.0, 1.0], [6.0, 
6.0, 8.0], [8.0, 2.0, 7.0]], [[5.0, 8.0, 3.0], [4.0, 6.0, 5.0], [8.0, 3.0, 4.0]], [[7.0, 7.0, 1.0], [4.0, 3.0, 5.0], [3.0, 7.0, 
7.0]]], [[[6.0, 4.0, 5.0], [6.0, 3.0, 7.0], [4.0, 6.0, 2.0]], [[6.0, 6.0, 7.0], [5.0, 5.0, 4.0], [5.0, 1.0, 4.0]], [[4.0, 4.0, 
8.0], [4.0, 5.0, 4.0], [7.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 2.0], [6.0, 5.0, 6.0], [4.0, 5.0, 6.0]], 
[[7.0, 8.0, 5.0], [4.0, 4.0, 3.0], [4.0, 8.0, 5.0]], [[8.0, 2.0, 6.0], [4.0, 2.0, 6.0], [6.0, 3.0, 3.0]]], [[[4.0, 7.0, 6.0], 
[1.0, 3.0, 1.0], [8.0, 6.0, 7.0]], [[3.0, 5.0, 3.0], [1.0, 8.0, 5.0], [6.0, 6.0, 4.0]], [[1.0, 8.0, 8.0], [5.0, 8.0, 5.0], 
[1.0, 2.0, 3.0]]], [[[8.0, 3.0, 1.0], [6.0, 6.0, 8.0], [8.0, 2.0, 7.0]], [[6.0, 2.0, 5.0], [4.0, 2.0, 4.0], [8.0, 3.0, 4.0]], 
[[7.0, 7.0, 1.0], [4.0, 3.0, 5.0], [3.0, 7.0, 7.0]]], [[[6.0, 4.0, 5.0], [6.0, 3.0, 7.0], [4.0, 6.0, 2.0]], [[7.0, 4.0, 6.0], 
[8.0, 1.0, 2.0], [5.0, 1.0, 4.0]], [[4.0, 4.0, 8.0], [4.0, 5.0, 4.0], [7.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RUPN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 8.0], [4.0, 3.0, 1.0], [3.0, 3.0, 5.0]], [[4.0, 3.0, 2.0], [5.0, 4.0, 6.0], [1.0, 2.0, 
6.0]], [[5.0, 4.0, 1.0], [3.0, 8.0, 5.0], [5.0, 1.0, 7.0]]], [[[2.0, 5.0, 5.0], [8.0, 4.0, 2.0], [1.0, 1.0, 8.0]], [[4.0, 8.0, 
1.0], [5.0, 7.0, 7.0], [7.0, 1.0, 8.0]], [[5.0, 8.0, 2.0], [7.0, 8.0, 5.0], [2.0, 6.0, 4.0]]], [[[2.0, 7.0, 7.0], [8.0, 1.0, 
1.0], [6.0, 2.0, 8.0]], [[3.0, 3.0, 8.0], [6.0, 5.0, 1.0], [4.0, 7.0, 4.0]], [[5.0, 6.0, 6.0], [6.0, 2.0, 3.0], [8.0, 4.0, 
6.0]]], [[[8.0, 5.0, 4.0], [1.0, 5.0, 2.0], [1.0, 3.0, 6.0]], [[8.0, 6.0, 4.0], [3.0, 8.0, 3.0], [7.0, 1.0, 3.0]], [[4.0, 2.0, 
7.0], [3.0, 6.0, 1.0], [6.0, 7.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 1.0, 8.0], [3.0, 2.0, 2.0]], [[5.0, 6.0, 4.0], [2.0, 8.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 3.0, 2.0], [1.0, 5.0, 8.0]], [[8.0, 7.0, 4.0], [7.0, 7.0, 1.0]]]))
      arg[1:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 8.0], [4.0, 3.0, 1.0], [3.0, 3.0, 5.0]], [[4.0, 3.0, 2.0], [5.0, 4.0, 6.0], [1.0, 
2.0, 6.0]], [[5.0, 4.0, 1.0], [3.0, 8.0, 5.0], [5.0, 1.0, 7.0]]], [[[2.0, 5.0, 5.0], [3.0, 1.0, 8.0], [1.0, 1.0, 8.0]], [[4.0, 
8.0, 1.0], [3.0, 2.0, 2.0], [7.0, 1.0, 8.0]], [[5.0, 8.0, 2.0], [7.0, 8.0, 5.0], [2.0, 6.0, 4.0]]], [[[2.0, 7.0, 7.0], [5.0, 
6.0, 4.0], [6.0, 2.0, 8.0]], [[3.0, 3.0, 8.0], [2.0, 8.0, 6.0], [4.0, 7.0, 4.0]], [[5.0, 6.0, 6.0], [6.0, 2.0, 3.0], [8.0, 4.0, 
6.0]]], [[[8.0, 5.0, 4.0], [1.0, 5.0, 2.0], [1.0, 3.0, 6.0]], [[8.0, 6.0, 4.0], [3.0, 8.0, 3.0], [7.0, 1.0, 3.0]], [[4.0, 2.0, 
7.0], [3.0, 6.0, 1.0], [6.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 8.0], [4.0, 3.0, 1.0], [3.0, 3.0, 5.0]], 
[[4.0, 3.0, 2.0], [5.0, 4.0, 6.0], [1.0, 2.0, 6.0]], [[5.0, 4.0, 1.0], [3.0, 8.0, 5.0], [5.0, 1.0, 7.0]]], [[[2.0, 5.0, 5.0], 
[3.0, 3.0, 2.0], [1.0, 1.0, 8.0]], [[4.0, 8.0, 1.0], [1.0, 5.0, 8.0], [7.0, 1.0, 8.0]], [[5.0, 8.0, 2.0], [7.0, 8.0, 5.0], 
[2.0, 6.0, 4.0]]], [[[2.0, 7.0, 7.0], [8.0, 7.0, 4.0], [6.0, 2.0, 8.0]], [[3.0, 3.0, 8.0], [7.0, 7.0, 1.0], [4.0, 7.0, 4.0]], 
[[5.0, 6.0, 6.0], [6.0, 2.0, 3.0], [8.0, 4.0, 6.0]]], [[[8.0, 5.0, 4.0], [1.0, 5.0, 2.0], [1.0, 3.0, 6.0]], [[8.0, 6.0, 4.0], 
[3.0, 8.0, 3.0], [7.0, 1.0, 3.0]], [[4.0, 2.0, 7.0], [3.0, 6.0, 1.0], [6.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RRUR(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 7.0], [8.0, 3.0, 6.0], [7.0, 7.0, 1.0]], [[1.0, 5.0, 6.0], [7.0, 7.0, 8.0], [2.0, 7.0, 
4.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 3.0], [4.0, 1.0, 6.0]]], [[[1.0, 7.0, 5.0], [3.0, 8.0, 5.0], [2.0, 4.0, 4.0]], [[6.0, 2.0, 
8.0], [5.0, 6.0, 6.0], [3.0, 8.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 5.0, 4.0], [1.0, 8.0, 2.0]]], [[[5.0, 1.0, 4.0], [8.0, 1.0, 
3.0], [8.0, 7.0, 2.0]], [[7.0, 8.0, 2.0], [6.0, 3.0, 7.0], [5.0, 3.0, 5.0]], [[1.0, 2.0, 5.0], [4.0, 4.0, 3.0], [4.0, 4.0, 
6.0]]], [[[8.0, 8.0, 6.0], [5.0, 8.0, 2.0], [6.0, 5.0, 2.0]], [[8.0, 7.0, 7.0], [8.0, 2.0, 5.0], [3.0, 8.0, 2.0]], [[8.0, 6.0, 
1.0], [7.0, 2.0, 8.0], [4.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0], [3.0]]], [[[2.0], [1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0], [4.0]]], [[[1.0], [6.0]]]]))
      arg[1:3,1:2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 7.0], [8.0, 3.0, 6.0], [7.0, 7.0, 1.0]], [[1.0, 5.0, 6.0], [7.0, 7.0, 8.0], [2.0, 
7.0, 4.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 3.0], [4.0, 1.0, 6.0]]], [[[1.0, 7.0, 5.0], [3.0, 8.0, 5.0], [2.0, 4.0, 4.0]], [[6.0, 
6.0, 8.0], [5.0, 3.0, 6.0], [3.0, 8.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 5.0, 4.0], [1.0, 8.0, 2.0]]], [[[5.0, 1.0, 4.0], [8.0, 
1.0, 3.0], [8.0, 7.0, 2.0]], [[7.0, 2.0, 2.0], [6.0, 1.0, 7.0], [5.0, 3.0, 5.0]], [[1.0, 2.0, 5.0], [4.0, 4.0, 3.0], [4.0, 4.0, 
6.0]]], [[[8.0, 8.0, 6.0], [5.0, 8.0, 2.0], [6.0, 5.0, 2.0]], [[8.0, 7.0, 7.0], [8.0, 2.0, 5.0], [3.0, 8.0, 2.0]], [[8.0, 6.0, 
1.0], [7.0, 2.0, 8.0], [4.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 7.0], [8.0, 3.0, 6.0], [7.0, 7.0, 1.0]], 
[[1.0, 5.0, 6.0], [7.0, 7.0, 8.0], [2.0, 7.0, 4.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 3.0], [4.0, 1.0, 6.0]]], [[[1.0, 7.0, 5.0], 
[3.0, 8.0, 5.0], [2.0, 4.0, 4.0]], [[6.0, 5.0, 8.0], [5.0, 4.0, 6.0], [3.0, 8.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 5.0, 4.0], 
[1.0, 8.0, 2.0]]], [[[5.0, 1.0, 4.0], [8.0, 1.0, 3.0], [8.0, 7.0, 2.0]], [[7.0, 1.0, 2.0], [6.0, 6.0, 7.0], [5.0, 3.0, 5.0]], 
[[1.0, 2.0, 5.0], [4.0, 4.0, 3.0], [4.0, 4.0, 6.0]]], [[[8.0, 8.0, 6.0], [5.0, 8.0, 2.0], [6.0, 5.0, 2.0]], [[8.0, 7.0, 7.0], 
[8.0, 2.0, 5.0], [3.0, 8.0, 2.0]], [[8.0, 6.0, 1.0], [7.0, 2.0, 8.0], [4.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PPRN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 2.0], [7.0, 5.0, 8.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 6.0], [6.0, 6.0, 3.0], [6.0, 7.0, 
5.0]], [[2.0, 6.0, 3.0], [3.0, 1.0, 7.0], [2.0, 5.0, 1.0]]], [[[5.0, 5.0, 7.0], [5.0, 5.0, 1.0], [4.0, 2.0, 7.0]], [[1.0, 1.0, 
5.0], [6.0, 5.0, 4.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 7.0], [4.0, 7.0, 7.0], [5.0, 2.0, 3.0]]], [[[4.0, 5.0, 7.0], [2.0, 1.0, 
2.0], [6.0, 1.0, 1.0]], [[1.0, 2.0, 1.0], [7.0, 6.0, 2.0], [3.0, 3.0, 8.0]], [[7.0, 8.0, 1.0], [3.0, 5.0, 8.0], [4.0, 6.0, 
3.0]]], [[[3.0, 1.0, 5.0], [7.0, 3.0, 4.0], [8.0, 4.0, 1.0]], [[4.0, 1.0, 8.0], [3.0, 1.0, 7.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 
7.0], [5.0, 1.0, 2.0], [5.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 5.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 3.0, 1.0]]))
      arg[2,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 2.0], [7.0, 5.0, 8.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 6.0], [6.0, 6.0, 3.0], [6.0, 
7.0, 5.0]], [[2.0, 6.0, 3.0], [3.0, 1.0, 7.0], [2.0, 5.0, 1.0]]], [[[5.0, 5.0, 7.0], [5.0, 5.0, 1.0], [4.0, 2.0, 7.0]], [[1.0, 
1.0, 5.0], [6.0, 5.0, 4.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 7.0], [4.0, 7.0, 7.0], [5.0, 2.0, 3.0]]], [[[4.0, 5.0, 7.0], [2.0, 
1.0, 2.0], [6.0, 1.0, 1.0]], [[1.0, 2.0, 1.0], [3.0, 5.0, 4.0], [3.0, 3.0, 8.0]], [[7.0, 8.0, 1.0], [3.0, 5.0, 8.0], [4.0, 6.0, 
3.0]]], [[[3.0, 1.0, 5.0], [7.0, 3.0, 4.0], [8.0, 4.0, 1.0]], [[4.0, 1.0, 8.0], [3.0, 1.0, 7.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 
7.0], [5.0, 1.0, 2.0], [5.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 2.0], [7.0, 5.0, 8.0], [3.0, 1.0, 1.0]], 
[[5.0, 8.0, 6.0], [6.0, 6.0, 3.0], [6.0, 7.0, 5.0]], [[2.0, 6.0, 3.0], [3.0, 1.0, 7.0], [2.0, 5.0, 1.0]]], [[[5.0, 5.0, 7.0], 
[5.0, 5.0, 1.0], [4.0, 2.0, 7.0]], [[1.0, 1.0, 5.0], [6.0, 5.0, 4.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 7.0], [4.0, 7.0, 7.0], 
[5.0, 2.0, 3.0]]], [[[4.0, 5.0, 7.0], [2.0, 1.0, 2.0], [6.0, 1.0, 1.0]], [[1.0, 2.0, 1.0], [1.0, 3.0, 1.0], [3.0, 3.0, 8.0]], 
[[7.0, 8.0, 1.0], [3.0, 5.0, 8.0], [4.0, 6.0, 3.0]]], [[[3.0, 1.0, 5.0], [7.0, 3.0, 4.0], [8.0, 4.0, 1.0]], [[4.0, 1.0, 8.0], 
[3.0, 1.0, 7.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 7.0], [5.0, 1.0, 2.0], [5.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RCRC(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 4.0], [5.0, 4.0, 7.0], [1.0, 7.0, 8.0]], [[1.0, 2.0, 1.0], [2.0, 4.0, 3.0], [5.0, 8.0, 
2.0]], [[4.0, 8.0, 4.0], [5.0, 7.0, 3.0], [2.0, 4.0, 4.0]]], [[[7.0, 8.0, 2.0], [2.0, 4.0, 3.0], [1.0, 3.0, 6.0]], [[6.0, 1.0, 
8.0], [6.0, 1.0, 8.0], [8.0, 5.0, 8.0]], [[2.0, 8.0, 2.0], [5.0, 6.0, 6.0], [2.0, 5.0, 3.0]]], [[[2.0, 3.0, 2.0], [7.0, 8.0, 
8.0], [8.0, 3.0, 2.0]], [[4.0, 3.0, 8.0], [4.0, 1.0, 6.0], [3.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [6.0, 6.0, 3.0], [8.0, 6.0, 
2.0]]], [[[2.0, 1.0, 3.0], [7.0, 4.0, 1.0], [2.0, 1.0, 1.0]], [[3.0, 4.0, 5.0], [2.0, 6.0, 2.0], [4.0, 3.0, 2.0]], [[8.0, 5.0, 
5.0], [2.0, 8.0, 5.0], [2.0, 7.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 8.0, 4.0]], [[6.0, 5.0, 2.0]], [[5.0, 3.0, 8.0]]], [[[5.0, 6.0, 8.0]], [[1.0, 8.0, 5.0]], 
[[8.0, 1.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 8.0]], [[7.0, 3.0, 4.0]], [[3.0, 5.0, 2.0]]], [[[2.0, 7.0, 7.0]], [[2.0, 
3.0, 7.0]], [[8.0, 2.0, 6.0]]]]))
      arg[1:3,:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 4.0], [5.0, 4.0, 7.0], [1.0, 7.0, 8.0]], [[1.0, 2.0, 1.0], [2.0, 4.0, 3.0], [5.0, 
8.0, 2.0]], [[4.0, 8.0, 4.0], [5.0, 7.0, 3.0], [2.0, 4.0, 4.0]]], [[[7.0, 8.0, 2.0], [1.0, 8.0, 4.0], [1.0, 3.0, 6.0]], [[6.0, 
1.0, 8.0], [6.0, 5.0, 2.0], [8.0, 5.0, 8.0]], [[2.0, 8.0, 2.0], [5.0, 3.0, 8.0], [2.0, 5.0, 3.0]]], [[[2.0, 3.0, 2.0], [5.0, 
6.0, 8.0], [8.0, 3.0, 2.0]], [[4.0, 3.0, 8.0], [1.0, 8.0, 5.0], [3.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [8.0, 1.0, 3.0], [8.0, 6.0, 
2.0]]], [[[2.0, 1.0, 3.0], [7.0, 4.0, 1.0], [2.0, 1.0, 1.0]], [[3.0, 4.0, 5.0], [2.0, 6.0, 2.0], [4.0, 3.0, 2.0]], [[8.0, 5.0, 
5.0], [2.0, 8.0, 5.0], [2.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 4.0], [5.0, 4.0, 7.0], [1.0, 7.0, 8.0]], 
[[1.0, 2.0, 1.0], [2.0, 4.0, 3.0], [5.0, 8.0, 2.0]], [[4.0, 8.0, 4.0], [5.0, 7.0, 3.0], [2.0, 4.0, 4.0]]], [[[7.0, 8.0, 2.0], 
[1.0, 8.0, 8.0], [1.0, 3.0, 6.0]], [[6.0, 1.0, 8.0], [7.0, 3.0, 4.0], [8.0, 5.0, 8.0]], [[2.0, 8.0, 2.0], [3.0, 5.0, 2.0], 
[2.0, 5.0, 3.0]]], [[[2.0, 3.0, 2.0], [2.0, 7.0, 7.0], [8.0, 3.0, 2.0]], [[4.0, 3.0, 8.0], [2.0, 3.0, 7.0], [3.0, 6.0, 3.0]], 
[[7.0, 3.0, 5.0], [8.0, 2.0, 6.0], [8.0, 6.0, 2.0]]], [[[2.0, 1.0, 3.0], [7.0, 4.0, 1.0], [2.0, 1.0, 1.0]], [[3.0, 4.0, 5.0], 
[2.0, 6.0, 2.0], [4.0, 3.0, 2.0]], [[8.0, 5.0, 5.0], [2.0, 8.0, 5.0], [2.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UPCN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 1.0], [1.0, 8.0, 7.0], [3.0, 7.0, 8.0]], [[1.0, 7.0, 5.0], [8.0, 1.0, 8.0], [5.0, 1.0, 
8.0]], [[2.0, 1.0, 6.0], [6.0, 3.0, 7.0], [7.0, 4.0, 5.0]]], [[[6.0, 7.0, 5.0], [8.0, 3.0, 5.0], [1.0, 2.0, 2.0]], [[5.0, 4.0, 
7.0], [2.0, 1.0, 3.0], [6.0, 3.0, 1.0]], [[4.0, 3.0, 2.0], [1.0, 7.0, 1.0], [3.0, 2.0, 2.0]]], [[[2.0, 5.0, 5.0], [6.0, 2.0, 
5.0], [8.0, 4.0, 4.0]], [[4.0, 7.0, 7.0], [4.0, 1.0, 5.0], [3.0, 1.0, 7.0]], [[5.0, 1.0, 6.0], [4.0, 2.0, 1.0], [2.0, 2.0, 
5.0]]], [[[5.0, 7.0, 6.0], [3.0, 4.0, 4.0], [1.0, 2.0, 2.0]], [[4.0, 4.0, 1.0], [2.0, 6.0, 3.0], [1.0, 2.0, 4.0]], [[8.0, 4.0, 
1.0], [3.0, 2.0, 4.0], [1.0, 8.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 2.0, 6.0], [7.0, 3.0, 5.0], [8.0, 1.0, 5.0]], [[4.0, 7.0, 6.0], [8.0, 4.0, 4.0], [4.0, 5.0, 
8.0]], [[1.0, 6.0, 5.0], [8.0, 5.0, 3.0], [6.0, 6.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 8.0, 2.0], [2.0, 4.0, 8.0], [7.0, 6.0, 8.0]], [[5.0, 5.0, 4.0], [4.0, 8.0, 1.0], 
[3.0, 4.0, 6.0]], [[4.0, 3.0, 7.0], [3.0, 4.0, 4.0], [2.0, 7.0, 8.0]]]))
      arg[:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 1.0], [1.0, 8.0, 7.0], [3.0, 7.0, 8.0]], [[2.0, 2.0, 6.0], [7.0, 3.0, 5.0], [8.0, 
1.0, 5.0]], [[2.0, 1.0, 6.0], [6.0, 3.0, 7.0], [7.0, 4.0, 5.0]]], [[[6.0, 7.0, 5.0], [8.0, 3.0, 5.0], [1.0, 2.0, 2.0]], [[4.0, 
7.0, 6.0], [8.0, 4.0, 4.0], [4.0, 5.0, 8.0]], [[4.0, 3.0, 2.0], [1.0, 7.0, 1.0], [3.0, 2.0, 2.0]]], [[[2.0, 5.0, 5.0], [6.0, 
2.0, 5.0], [8.0, 4.0, 4.0]], [[1.0, 6.0, 5.0], [8.0, 5.0, 3.0], [6.0, 6.0, 7.0]], [[5.0, 1.0, 6.0], [4.0, 2.0, 1.0], [2.0, 2.0, 
5.0]]], [[[5.0, 7.0, 6.0], [3.0, 4.0, 4.0], [1.0, 2.0, 2.0]], [[4.0, 4.0, 1.0], [2.0, 6.0, 3.0], [1.0, 2.0, 4.0]], [[8.0, 4.0, 
1.0], [3.0, 2.0, 4.0], [1.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 1.0], [1.0, 8.0, 7.0], [3.0, 7.0, 8.0]], 
[[1.0, 8.0, 2.0], [2.0, 4.0, 8.0], [7.0, 6.0, 8.0]], [[2.0, 1.0, 6.0], [6.0, 3.0, 7.0], [7.0, 4.0, 5.0]]], [[[6.0, 7.0, 5.0], 
[8.0, 3.0, 5.0], [1.0, 2.0, 2.0]], [[5.0, 5.0, 4.0], [4.0, 8.0, 1.0], [3.0, 4.0, 6.0]], [[4.0, 3.0, 2.0], [1.0, 7.0, 1.0], 
[3.0, 2.0, 2.0]]], [[[2.0, 5.0, 5.0], [6.0, 2.0, 5.0], [8.0, 4.0, 4.0]], [[4.0, 3.0, 7.0], [3.0, 4.0, 4.0], [2.0, 7.0, 8.0]], 
[[5.0, 1.0, 6.0], [4.0, 2.0, 1.0], [2.0, 2.0, 5.0]]], [[[5.0, 7.0, 6.0], [3.0, 4.0, 4.0], [1.0, 2.0, 2.0]], [[4.0, 4.0, 1.0], 
[2.0, 6.0, 3.0], [1.0, 2.0, 4.0]], [[8.0, 4.0, 1.0], [3.0, 2.0, 4.0], [1.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UPRN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 5.0], [2.0, 2.0, 7.0], [8.0, 3.0, 3.0]], [[4.0, 1.0, 5.0], [3.0, 6.0, 8.0], [5.0, 4.0, 
2.0]], [[3.0, 8.0, 1.0], [5.0, 7.0, 3.0], [3.0, 1.0, 6.0]]], [[[5.0, 8.0, 7.0], [7.0, 1.0, 3.0], [7.0, 5.0, 3.0]], [[7.0, 7.0, 
2.0], [8.0, 3.0, 7.0], [4.0, 1.0, 1.0]], [[2.0, 4.0, 6.0], [6.0, 5.0, 1.0], [7.0, 5.0, 8.0]]], [[[1.0, 8.0, 8.0], [1.0, 8.0, 
6.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 5.0], [5.0, 5.0, 4.0], [8.0, 2.0, 8.0]], [[3.0, 1.0, 4.0], [5.0, 4.0, 7.0], [1.0, 4.0, 
8.0]]], [[[5.0, 2.0, 4.0], [4.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[4.0, 6.0, 5.0], [1.0, 2.0, 4.0], [5.0, 6.0, 3.0]], [[3.0, 8.0, 
5.0], [2.0, 6.0, 2.0], [7.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 3.0, 4.0]], [[2.0, 1.0, 7.0]], [[4.0, 8.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 4.0, 5.0]], [[7.0, 8.0, 6.0]], [[5.0, 2.0, 6.0]]]))
      arg[:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 5.0], [2.0, 2.0, 7.0], [8.0, 3.0, 3.0]], [[4.0, 1.0, 5.0], [5.0, 3.0, 4.0], [5.0, 
4.0, 2.0]], [[3.0, 8.0, 1.0], [5.0, 7.0, 3.0], [3.0, 1.0, 6.0]]], [[[5.0, 8.0, 7.0], [7.0, 1.0, 3.0], [7.0, 5.0, 3.0]], [[7.0, 
7.0, 2.0], [2.0, 1.0, 7.0], [4.0, 1.0, 1.0]], [[2.0, 4.0, 6.0], [6.0, 5.0, 1.0], [7.0, 5.0, 8.0]]], [[[1.0, 8.0, 8.0], [1.0, 
8.0, 6.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 5.0], [4.0, 8.0, 8.0], [8.0, 2.0, 8.0]], [[3.0, 1.0, 4.0], [5.0, 4.0, 7.0], [1.0, 4.0, 
8.0]]], [[[5.0, 2.0, 4.0], [4.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[4.0, 6.0, 5.0], [1.0, 2.0, 4.0], [5.0, 6.0, 3.0]], [[3.0, 8.0, 
5.0], [2.0, 6.0, 2.0], [7.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 5.0], [2.0, 2.0, 7.0], [8.0, 3.0, 3.0]], 
[[4.0, 1.0, 5.0], [6.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[3.0, 8.0, 1.0], [5.0, 7.0, 3.0], [3.0, 1.0, 6.0]]], [[[5.0, 8.0, 7.0], 
[7.0, 1.0, 3.0], [7.0, 5.0, 3.0]], [[7.0, 7.0, 2.0], [7.0, 8.0, 6.0], [4.0, 1.0, 1.0]], [[2.0, 4.0, 6.0], [6.0, 5.0, 1.0], 
[7.0, 5.0, 8.0]]], [[[1.0, 8.0, 8.0], [1.0, 8.0, 6.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 5.0], [5.0, 2.0, 6.0], [8.0, 2.0, 8.0]], 
[[3.0, 1.0, 4.0], [5.0, 4.0, 7.0], [1.0, 4.0, 8.0]]], [[[5.0, 2.0, 4.0], [4.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[4.0, 6.0, 5.0], 
[1.0, 2.0, 4.0], [5.0, 6.0, 3.0]], [[3.0, 8.0, 5.0], [2.0, 6.0, 2.0], [7.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CULN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 2.0], [7.0, 7.0, 3.0], [1.0, 5.0, 6.0]], [[8.0, 4.0, 2.0], [1.0, 3.0, 1.0], [8.0, 6.0, 
6.0]], [[6.0, 1.0, 1.0], [6.0, 3.0, 7.0], [4.0, 4.0, 8.0]]], [[[3.0, 3.0, 5.0], [8.0, 1.0, 1.0], [8.0, 5.0, 5.0]], [[1.0, 4.0, 
8.0], [7.0, 8.0, 8.0], [3.0, 2.0, 2.0]], [[4.0, 5.0, 2.0], [1.0, 1.0, 6.0], [7.0, 7.0, 6.0]]], [[[4.0, 7.0, 7.0], [4.0, 3.0, 
6.0], [8.0, 6.0, 8.0]], [[5.0, 2.0, 3.0], [7.0, 5.0, 7.0], [3.0, 2.0, 8.0]], [[7.0, 6.0, 3.0], [4.0, 2.0, 3.0], [5.0, 2.0, 
8.0]]], [[[4.0, 1.0, 2.0], [3.0, 6.0, 8.0], [7.0, 4.0, 5.0]], [[1.0, 1.0, 2.0], [2.0, 7.0, 1.0], [8.0, 4.0, 7.0]], [[5.0, 8.0, 
8.0], [5.0, 3.0, 3.0], [7.0, 6.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 5.0, 4.0]], [[8.0, 7.0, 2.0]]], [[[6.0, 5.0, 3.0]], [[1.0, 4.0, 3.0]]], [[[3.0, 6.0, 6.0]], 
[[6.0, 3.0, 2.0]]], [[[5.0, 8.0, 8.0]], [[3.0, 3.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 8.0]], [[6.0, 2.0, 1.0]]], [[[2.0, 1.0, 6.0]], [[7.0, 5.0, 2.0]]], [[[1.0, 
5.0, 5.0]], [[8.0, 4.0, 2.0]]], [[[7.0, 7.0, 7.0]], [[2.0, 1.0, 6.0]]]]))
      arg[:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 2.0], [7.0, 7.0, 3.0], [1.0, 5.0, 4.0]], [[8.0, 4.0, 2.0], [1.0, 3.0, 1.0], [8.0, 
7.0, 2.0]], [[6.0, 1.0, 1.0], [6.0, 3.0, 7.0], [4.0, 4.0, 8.0]]], [[[3.0, 3.0, 5.0], [8.0, 1.0, 1.0], [6.0, 5.0, 3.0]], [[1.0, 
4.0, 8.0], [7.0, 8.0, 8.0], [1.0, 4.0, 3.0]], [[4.0, 5.0, 2.0], [1.0, 1.0, 6.0], [7.0, 7.0, 6.0]]], [[[4.0, 7.0, 7.0], [4.0, 
3.0, 6.0], [3.0, 6.0, 6.0]], [[5.0, 2.0, 3.0], [7.0, 5.0, 7.0], [6.0, 3.0, 2.0]], [[7.0, 6.0, 3.0], [4.0, 2.0, 3.0], [5.0, 2.0, 
8.0]]], [[[4.0, 1.0, 2.0], [3.0, 6.0, 8.0], [5.0, 8.0, 8.0]], [[1.0, 1.0, 2.0], [2.0, 7.0, 1.0], [3.0, 3.0, 6.0]], [[5.0, 8.0, 
8.0], [5.0, 3.0, 3.0], [7.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 2.0], [7.0, 7.0, 3.0], [1.0, 1.0, 8.0]], 
[[8.0, 4.0, 2.0], [1.0, 3.0, 1.0], [6.0, 2.0, 1.0]], [[6.0, 1.0, 1.0], [6.0, 3.0, 7.0], [4.0, 4.0, 8.0]]], [[[3.0, 3.0, 5.0], 
[8.0, 1.0, 1.0], [2.0, 1.0, 6.0]], [[1.0, 4.0, 8.0], [7.0, 8.0, 8.0], [7.0, 5.0, 2.0]], [[4.0, 5.0, 2.0], [1.0, 1.0, 6.0], 
[7.0, 7.0, 6.0]]], [[[4.0, 7.0, 7.0], [4.0, 3.0, 6.0], [1.0, 5.0, 5.0]], [[5.0, 2.0, 3.0], [7.0, 5.0, 7.0], [8.0, 4.0, 2.0]], 
[[7.0, 6.0, 3.0], [4.0, 2.0, 3.0], [5.0, 2.0, 8.0]]], [[[4.0, 1.0, 2.0], [3.0, 6.0, 8.0], [7.0, 7.0, 7.0]], [[1.0, 1.0, 2.0], 
[2.0, 7.0, 1.0], [2.0, 1.0, 6.0]], [[5.0, 8.0, 8.0], [5.0, 3.0, 3.0], [7.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLRN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 8.0], [6.0, 7.0, 3.0], [4.0, 7.0, 1.0]], [[1.0, 8.0, 8.0], [4.0, 6.0, 2.0], [4.0, 2.0, 
2.0]], [[5.0, 2.0, 2.0], [5.0, 8.0, 7.0], [3.0, 6.0, 2.0]]], [[[8.0, 4.0, 7.0], [5.0, 7.0, 3.0], [2.0, 1.0, 2.0]], [[1.0, 5.0, 
4.0], [2.0, 5.0, 1.0], [4.0, 2.0, 6.0]], [[8.0, 1.0, 2.0], [7.0, 2.0, 7.0], [4.0, 8.0, 2.0]]], [[[8.0, 5.0, 3.0], [8.0, 1.0, 
2.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 8.0], [6.0, 4.0, 8.0], [8.0, 7.0, 8.0]], [[5.0, 1.0, 7.0], [4.0, 8.0, 8.0], [5.0, 4.0, 
4.0]]], [[[4.0, 3.0, 3.0], [1.0, 6.0, 3.0], [1.0, 8.0, 3.0]], [[4.0, 4.0, 2.0], [4.0, 5.0, 1.0], [2.0, 8.0, 6.0]], [[4.0, 8.0, 
3.0], [1.0, 6.0, 6.0], [4.0, 6.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 3.0, 3.0]]], [[[7.0, 1.0, 5.0]]], [[[4.0, 5.0, 4.0]]], [[[2.0, 7.0, 
8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 7.0]]], [[[3.0, 2.0, 5.0]]], [[[7.0, 7.0, 1.0]]], [[[6.0, 4.0, 6.0]]]]))
      arg[:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 8.0], [6.0, 7.0, 3.0], [4.0, 7.0, 1.0]], [[1.0, 8.0, 8.0], [4.0, 6.0, 2.0], [4.0, 
2.0, 2.0]], [[5.0, 2.0, 2.0], [2.0, 3.0, 3.0], [3.0, 6.0, 2.0]]], [[[8.0, 4.0, 7.0], [5.0, 7.0, 3.0], [2.0, 1.0, 2.0]], [[1.0, 
5.0, 4.0], [2.0, 5.0, 1.0], [4.0, 2.0, 6.0]], [[8.0, 1.0, 2.0], [7.0, 1.0, 5.0], [4.0, 8.0, 2.0]]], [[[8.0, 5.0, 3.0], [8.0, 
1.0, 2.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 8.0], [6.0, 4.0, 8.0], [8.0, 7.0, 8.0]], [[5.0, 1.0, 7.0], [4.0, 5.0, 4.0], [5.0, 4.0, 
4.0]]], [[[4.0, 3.0, 3.0], [1.0, 6.0, 3.0], [1.0, 8.0, 3.0]], [[4.0, 4.0, 2.0], [4.0, 5.0, 1.0], [2.0, 8.0, 6.0]], [[4.0, 8.0, 
3.0], [2.0, 7.0, 8.0], [4.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 8.0], [6.0, 7.0, 3.0], [4.0, 7.0, 1.0]], 
[[1.0, 8.0, 8.0], [4.0, 6.0, 2.0], [4.0, 2.0, 2.0]], [[5.0, 2.0, 2.0], [8.0, 2.0, 7.0], [3.0, 6.0, 2.0]]], [[[8.0, 4.0, 7.0], 
[5.0, 7.0, 3.0], [2.0, 1.0, 2.0]], [[1.0, 5.0, 4.0], [2.0, 5.0, 1.0], [4.0, 2.0, 6.0]], [[8.0, 1.0, 2.0], [3.0, 2.0, 5.0], 
[4.0, 8.0, 2.0]]], [[[8.0, 5.0, 3.0], [8.0, 1.0, 2.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 8.0], [6.0, 4.0, 8.0], [8.0, 7.0, 8.0]], 
[[5.0, 1.0, 7.0], [7.0, 7.0, 1.0], [5.0, 4.0, 4.0]]], [[[4.0, 3.0, 3.0], [1.0, 6.0, 3.0], [1.0, 8.0, 3.0]], [[4.0, 4.0, 2.0], 
[4.0, 5.0, 1.0], [2.0, 8.0, 6.0]], [[4.0, 8.0, 3.0], [6.0, 4.0, 6.0], [4.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLLL(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 4.0], [4.0, 7.0, 6.0], [7.0, 2.0, 4.0]], [[8.0, 8.0, 2.0], [3.0, 5.0, 2.0], [1.0, 1.0, 
5.0]], [[6.0, 7.0, 7.0], [2.0, 4.0, 5.0], [4.0, 7.0, 5.0]]], [[[5.0, 3.0, 2.0], [2.0, 8.0, 4.0], [1.0, 3.0, 7.0]], [[3.0, 6.0, 
4.0], [1.0, 6.0, 4.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 5.0], [1.0, 7.0, 8.0], [7.0, 5.0, 1.0]]], [[[7.0, 1.0, 7.0], [3.0, 6.0, 
4.0], [7.0, 8.0, 5.0]], [[3.0, 3.0, 6.0], [7.0, 2.0, 5.0], [6.0, 4.0, 4.0]], [[7.0, 7.0, 1.0], [3.0, 4.0, 1.0], [6.0, 7.0, 
8.0]]], [[[4.0, 3.0, 1.0], [2.0, 8.0, 6.0], [8.0, 8.0, 8.0]], [[2.0, 1.0, 6.0], [3.0, 5.0, 3.0], [8.0, 4.0, 7.0]], [[4.0, 6.0, 
7.0], [5.0, 4.0, 3.0], [8.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0]]], [[[6.0]]], [[[2.0]]], [[[5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0]]], [[[2.0]]], [[[5.0]]], [[[2.0]]]]))
      arg[:,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 4.0], [4.0, 7.0, 6.0], [7.0, 2.0, 4.0]], [[8.0, 8.0, 2.0], [3.0, 5.0, 2.0], [1.0, 
1.0, 5.0]], [[6.0, 7.0, 7.0], [2.0, 4.0, 5.0], [4.0, 7.0, 3.0]]], [[[5.0, 3.0, 2.0], [2.0, 8.0, 4.0], [1.0, 3.0, 7.0]], [[3.0, 
6.0, 4.0], [1.0, 6.0, 4.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 5.0], [1.0, 7.0, 8.0], [7.0, 5.0, 6.0]]], [[[7.0, 1.0, 7.0], [3.0, 
6.0, 4.0], [7.0, 8.0, 5.0]], [[3.0, 3.0, 6.0], [7.0, 2.0, 5.0], [6.0, 4.0, 4.0]], [[7.0, 7.0, 1.0], [3.0, 4.0, 1.0], [6.0, 7.0, 
2.0]]], [[[4.0, 3.0, 1.0], [2.0, 8.0, 6.0], [8.0, 8.0, 8.0]], [[2.0, 1.0, 6.0], [3.0, 5.0, 3.0], [8.0, 4.0, 7.0]], [[4.0, 6.0, 
7.0], [5.0, 4.0, 3.0], [8.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 4.0], [4.0, 7.0, 6.0], [7.0, 2.0, 4.0]], 
[[8.0, 8.0, 2.0], [3.0, 5.0, 2.0], [1.0, 1.0, 5.0]], [[6.0, 7.0, 7.0], [2.0, 4.0, 5.0], [4.0, 7.0, 5.0]]], [[[5.0, 3.0, 2.0], 
[2.0, 8.0, 4.0], [1.0, 3.0, 7.0]], [[3.0, 6.0, 4.0], [1.0, 6.0, 4.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 5.0], [1.0, 7.0, 8.0], 
[7.0, 5.0, 2.0]]], [[[7.0, 1.0, 7.0], [3.0, 6.0, 4.0], [7.0, 8.0, 5.0]], [[3.0, 3.0, 6.0], [7.0, 2.0, 5.0], [6.0, 4.0, 4.0]], 
[[7.0, 7.0, 1.0], [3.0, 4.0, 1.0], [6.0, 7.0, 5.0]]], [[[4.0, 3.0, 1.0], [2.0, 8.0, 6.0], [8.0, 8.0, 8.0]], [[2.0, 1.0, 6.0], 
[3.0, 5.0, 3.0], [8.0, 4.0, 7.0]], [[4.0, 6.0, 7.0], [5.0, 4.0, 3.0], [8.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RCLN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 5.0], [2.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[8.0, 1.0, 6.0], [1.0, 4.0, 5.0], [6.0, 6.0, 
7.0]], [[4.0, 3.0, 1.0], [2.0, 1.0, 1.0], [4.0, 6.0, 2.0]]], [[[4.0, 3.0, 7.0], [2.0, 5.0, 2.0], [1.0, 3.0, 3.0]], [[6.0, 5.0, 
5.0], [6.0, 4.0, 6.0], [2.0, 5.0, 6.0]], [[8.0, 1.0, 8.0], [6.0, 1.0, 2.0], [6.0, 5.0, 6.0]]], [[[5.0, 4.0, 1.0], [1.0, 4.0, 
5.0], [4.0, 2.0, 5.0]], [[4.0, 7.0, 7.0], [8.0, 6.0, 6.0], [2.0, 7.0, 1.0]], [[1.0, 3.0, 6.0], [4.0, 6.0, 4.0], [4.0, 4.0, 
4.0]]], [[[8.0, 2.0, 3.0], [4.0, 2.0, 1.0], [4.0, 5.0, 3.0]], [[7.0, 1.0, 1.0], [5.0, 4.0, 7.0], [3.0, 1.0, 1.0]], [[3.0, 2.0, 
8.0], [4.0, 1.0, 5.0], [2.0, 2.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 5.0, 5.0]], [[5.0, 4.0, 7.0]], [[5.0, 2.0, 6.0]]], [[[5.0, 8.0, 7.0]], [[8.0, 4.0, 5.0]], 
[[4.0, 5.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 5.0]], [[6.0, 6.0, 8.0]], [[1.0, 7.0, 2.0]]], [[[8.0, 4.0, 6.0]], [[3.0, 
5.0, 6.0]], [[5.0, 2.0, 3.0]]]]))
      arg[1:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 5.0], [2.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[8.0, 1.0, 6.0], [1.0, 4.0, 5.0], [6.0, 
6.0, 7.0]], [[4.0, 3.0, 1.0], [2.0, 1.0, 1.0], [4.0, 6.0, 2.0]]], [[[4.0, 3.0, 7.0], [2.0, 5.0, 2.0], [4.0, 5.0, 5.0]], [[6.0, 
5.0, 5.0], [6.0, 4.0, 6.0], [5.0, 4.0, 7.0]], [[8.0, 1.0, 8.0], [6.0, 1.0, 2.0], [5.0, 2.0, 6.0]]], [[[5.0, 4.0, 1.0], [1.0, 
4.0, 5.0], [5.0, 8.0, 7.0]], [[4.0, 7.0, 7.0], [8.0, 6.0, 6.0], [8.0, 4.0, 5.0]], [[1.0, 3.0, 6.0], [4.0, 6.0, 4.0], [4.0, 5.0, 
6.0]]], [[[8.0, 2.0, 3.0], [4.0, 2.0, 1.0], [4.0, 5.0, 3.0]], [[7.0, 1.0, 1.0], [5.0, 4.0, 7.0], [3.0, 1.0, 1.0]], [[3.0, 2.0, 
8.0], [4.0, 1.0, 5.0], [2.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 5.0], [2.0, 1.0, 4.0], [7.0, 8.0, 2.0]], 
[[8.0, 1.0, 6.0], [1.0, 4.0, 5.0], [6.0, 6.0, 7.0]], [[4.0, 3.0, 1.0], [2.0, 1.0, 1.0], [4.0, 6.0, 2.0]]], [[[4.0, 3.0, 7.0], 
[2.0, 5.0, 2.0], [7.0, 4.0, 5.0]], [[6.0, 5.0, 5.0], [6.0, 4.0, 6.0], [6.0, 6.0, 8.0]], [[8.0, 1.0, 8.0], [6.0, 1.0, 2.0], 
[1.0, 7.0, 2.0]]], [[[5.0, 4.0, 1.0], [1.0, 4.0, 5.0], [8.0, 4.0, 6.0]], [[4.0, 7.0, 7.0], [8.0, 6.0, 6.0], [3.0, 5.0, 6.0]], 
[[1.0, 3.0, 6.0], [4.0, 6.0, 4.0], [5.0, 2.0, 3.0]]], [[[8.0, 2.0, 3.0], [4.0, 2.0, 1.0], [4.0, 5.0, 3.0]], [[7.0, 1.0, 1.0], 
[5.0, 4.0, 7.0], [3.0, 1.0, 1.0]], [[3.0, 2.0, 8.0], [4.0, 1.0, 5.0], [2.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RRRN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 6.0], [2.0, 7.0, 6.0], [2.0, 1.0, 7.0]], [[6.0, 1.0, 6.0], [1.0, 8.0, 8.0], [1.0, 4.0, 
8.0]], [[3.0, 6.0, 7.0], [1.0, 7.0, 2.0], [5.0, 7.0, 6.0]]], [[[1.0, 6.0, 4.0], [7.0, 1.0, 3.0], [7.0, 2.0, 1.0]], [[2.0, 6.0, 
7.0], [8.0, 5.0, 7.0], [1.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [4.0, 2.0, 5.0], [4.0, 3.0, 6.0]]], [[[7.0, 5.0, 7.0], [1.0, 4.0, 
5.0], [8.0, 8.0, 6.0]], [[1.0, 1.0, 5.0], [5.0, 2.0, 6.0], [7.0, 3.0, 8.0]], [[6.0, 7.0, 3.0], [3.0, 1.0, 8.0], [1.0, 4.0, 
8.0]]], [[[1.0, 7.0, 6.0], [3.0, 1.0, 6.0], [3.0, 6.0, 4.0]], [[1.0, 4.0, 4.0], [7.0, 6.0, 3.0], [8.0, 5.0, 1.0]], [[1.0, 4.0, 
7.0], [1.0, 1.0, 4.0], [3.0, 7.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 1.0, 6.0]]], [[[2.0, 6.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 3.0]]], [[[1.0, 4.0, 1.0]]]]))
      arg[1:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 6.0], [2.0, 7.0, 6.0], [2.0, 1.0, 7.0]], [[6.0, 1.0, 6.0], [1.0, 8.0, 8.0], [1.0, 
4.0, 8.0]], [[3.0, 6.0, 7.0], [1.0, 7.0, 2.0], [5.0, 7.0, 6.0]]], [[[1.0, 6.0, 4.0], [7.0, 1.0, 3.0], [7.0, 2.0, 1.0]], [[2.0, 
6.0, 7.0], [8.0, 1.0, 6.0], [1.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [4.0, 2.0, 5.0], [4.0, 3.0, 6.0]]], [[[7.0, 5.0, 7.0], [1.0, 
4.0, 5.0], [8.0, 8.0, 6.0]], [[1.0, 1.0, 5.0], [2.0, 6.0, 2.0], [7.0, 3.0, 8.0]], [[6.0, 7.0, 3.0], [3.0, 1.0, 8.0], [1.0, 4.0, 
8.0]]], [[[1.0, 7.0, 6.0], [3.0, 1.0, 6.0], [3.0, 6.0, 4.0]], [[1.0, 4.0, 4.0], [7.0, 6.0, 3.0], [8.0, 5.0, 1.0]], [[1.0, 4.0, 
7.0], [1.0, 1.0, 4.0], [3.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 6.0], [2.0, 7.0, 6.0], [2.0, 1.0, 7.0]], 
[[6.0, 1.0, 6.0], [1.0, 8.0, 8.0], [1.0, 4.0, 8.0]], [[3.0, 6.0, 7.0], [1.0, 7.0, 2.0], [5.0, 7.0, 6.0]]], [[[1.0, 6.0, 4.0], 
[7.0, 1.0, 3.0], [7.0, 2.0, 1.0]], [[2.0, 6.0, 7.0], [2.0, 8.0, 3.0], [1.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [4.0, 2.0, 5.0], 
[4.0, 3.0, 6.0]]], [[[7.0, 5.0, 7.0], [1.0, 4.0, 5.0], [8.0, 8.0, 6.0]], [[1.0, 1.0, 5.0], [1.0, 4.0, 1.0], [7.0, 3.0, 8.0]], 
[[6.0, 7.0, 3.0], [3.0, 1.0, 8.0], [1.0, 4.0, 8.0]]], [[[1.0, 7.0, 6.0], [3.0, 1.0, 6.0], [3.0, 6.0, 4.0]], [[1.0, 4.0, 4.0], 
[7.0, 6.0, 3.0], [8.0, 5.0, 1.0]], [[1.0, 4.0, 7.0], [1.0, 1.0, 4.0], [3.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CPRN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 5.0], [4.0, 2.0, 6.0], [4.0, 3.0, 5.0]], [[6.0, 1.0, 1.0], [6.0, 6.0, 2.0], [3.0, 8.0, 
3.0]], [[6.0, 5.0, 1.0], [2.0, 1.0, 7.0], [4.0, 4.0, 4.0]]], [[[2.0, 7.0, 1.0], [8.0, 6.0, 2.0], [4.0, 2.0, 2.0]], [[1.0, 3.0, 
1.0], [7.0, 2.0, 2.0], [4.0, 3.0, 2.0]], [[4.0, 1.0, 7.0], [5.0, 1.0, 2.0], [5.0, 5.0, 8.0]]], [[[8.0, 8.0, 8.0], [4.0, 4.0, 
7.0], [1.0, 6.0, 8.0]], [[2.0, 2.0, 6.0], [5.0, 3.0, 5.0], [2.0, 5.0, 4.0]], [[4.0, 6.0, 6.0], [7.0, 2.0, 7.0], [1.0, 1.0, 
8.0]]], [[[8.0, 8.0, 8.0], [3.0, 3.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 5.0, 6.0], [8.0, 4.0, 3.0], [8.0, 3.0, 3.0]], [[5.0, 6.0, 
8.0], [7.0, 3.0, 6.0], [1.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 5.0, 5.0]], [[4.0, 7.0, 5.0]], [[2.0, 6.0, 6.0]], [[5.0, 6.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 6.0, 6.0]], [[4.0, 2.0, 2.0]], [[5.0, 5.0, 7.0]], [[3.0, 4.0, 5.0]]]))
      arg[:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 5.0], [4.0, 2.0, 6.0], [4.0, 3.0, 5.0]], [[6.0, 1.0, 1.0], [3.0, 5.0, 5.0], [3.0, 
8.0, 3.0]], [[6.0, 5.0, 1.0], [2.0, 1.0, 7.0], [4.0, 4.0, 4.0]]], [[[2.0, 7.0, 1.0], [8.0, 6.0, 2.0], [4.0, 2.0, 2.0]], [[1.0, 
3.0, 1.0], [4.0, 7.0, 5.0], [4.0, 3.0, 2.0]], [[4.0, 1.0, 7.0], [5.0, 1.0, 2.0], [5.0, 5.0, 8.0]]], [[[8.0, 8.0, 8.0], [4.0, 
4.0, 7.0], [1.0, 6.0, 8.0]], [[2.0, 2.0, 6.0], [2.0, 6.0, 6.0], [2.0, 5.0, 4.0]], [[4.0, 6.0, 6.0], [7.0, 2.0, 7.0], [1.0, 1.0, 
8.0]]], [[[8.0, 8.0, 8.0], [3.0, 3.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 5.0, 6.0], [5.0, 6.0, 1.0], [8.0, 3.0, 3.0]], [[5.0, 6.0, 
8.0], [7.0, 3.0, 6.0], [1.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 5.0], [4.0, 2.0, 6.0], [4.0, 3.0, 5.0]], 
[[6.0, 1.0, 1.0], [4.0, 6.0, 6.0], [3.0, 8.0, 3.0]], [[6.0, 5.0, 1.0], [2.0, 1.0, 7.0], [4.0, 4.0, 4.0]]], [[[2.0, 7.0, 1.0], 
[8.0, 6.0, 2.0], [4.0, 2.0, 2.0]], [[1.0, 3.0, 1.0], [4.0, 2.0, 2.0], [4.0, 3.0, 2.0]], [[4.0, 1.0, 7.0], [5.0, 1.0, 2.0], 
[5.0, 5.0, 8.0]]], [[[8.0, 8.0, 8.0], [4.0, 4.0, 7.0], [1.0, 6.0, 8.0]], [[2.0, 2.0, 6.0], [5.0, 5.0, 7.0], [2.0, 5.0, 4.0]], 
[[4.0, 6.0, 6.0], [7.0, 2.0, 7.0], [1.0, 1.0, 8.0]]], [[[8.0, 8.0, 8.0], [3.0, 3.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 5.0, 6.0], 
[3.0, 4.0, 5.0], [8.0, 3.0, 3.0]], [[5.0, 6.0, 8.0], [7.0, 3.0, 6.0], [1.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CPUN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 3.0], [8.0, 5.0, 4.0], [3.0, 6.0, 1.0]], [[7.0, 6.0, 6.0], [2.0, 7.0, 2.0], [1.0, 7.0, 
4.0]], [[3.0, 3.0, 1.0], [6.0, 5.0, 7.0], [5.0, 8.0, 3.0]]], [[[2.0, 8.0, 1.0], [1.0, 3.0, 2.0], [2.0, 8.0, 7.0]], [[1.0, 4.0, 
1.0], [1.0, 4.0, 8.0], [4.0, 5.0, 1.0]], [[3.0, 2.0, 5.0], [5.0, 4.0, 5.0], [8.0, 8.0, 1.0]]], [[[6.0, 7.0, 7.0], [6.0, 1.0, 
5.0], [4.0, 5.0, 5.0]], [[7.0, 2.0, 8.0], [3.0, 6.0, 7.0], [6.0, 6.0, 2.0]], [[2.0, 3.0, 1.0], [3.0, 4.0, 4.0], [5.0, 2.0, 
5.0]]], [[[5.0, 1.0, 4.0], [3.0, 1.0, 3.0], [7.0, 6.0, 3.0]], [[6.0, 1.0, 2.0], [7.0, 8.0, 6.0], [4.0, 5.0, 1.0]], [[4.0, 4.0, 
6.0], [6.0, 3.0, 8.0], [1.0, 2.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 8.0], [5.0, 1.0, 4.0]], [[3.0, 5.0, 4.0], [7.0, 2.0, 1.0]], [[8.0, 8.0, 5.0], [6.0, 
3.0, 5.0]], [[8.0, 5.0, 6.0], [7.0, 5.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 7.0, 1.0], [4.0, 2.0, 7.0]], [[8.0, 7.0, 1.0], [4.0, 6.0, 1.0]], [[5.0, 8.0, 
6.0], [8.0, 4.0, 7.0]], [[7.0, 5.0, 2.0], [4.0, 8.0, 8.0]]]))
      arg[:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 3.0], [8.0, 5.0, 4.0], [3.0, 6.0, 1.0]], [[4.0, 7.0, 8.0], [5.0, 1.0, 4.0], [1.0, 
7.0, 4.0]], [[3.0, 3.0, 1.0], [6.0, 5.0, 7.0], [5.0, 8.0, 3.0]]], [[[2.0, 8.0, 1.0], [1.0, 3.0, 2.0], [2.0, 8.0, 7.0]], [[3.0, 
5.0, 4.0], [7.0, 2.0, 1.0], [4.0, 5.0, 1.0]], [[3.0, 2.0, 5.0], [5.0, 4.0, 5.0], [8.0, 8.0, 1.0]]], [[[6.0, 7.0, 7.0], [6.0, 
1.0, 5.0], [4.0, 5.0, 5.0]], [[8.0, 8.0, 5.0], [6.0, 3.0, 5.0], [6.0, 6.0, 2.0]], [[2.0, 3.0, 1.0], [3.0, 4.0, 4.0], [5.0, 2.0, 
5.0]]], [[[5.0, 1.0, 4.0], [3.0, 1.0, 3.0], [7.0, 6.0, 3.0]], [[8.0, 5.0, 6.0], [7.0, 5.0, 1.0], [4.0, 5.0, 1.0]], [[4.0, 4.0, 
6.0], [6.0, 3.0, 8.0], [1.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 3.0], [8.0, 5.0, 4.0], [3.0, 6.0, 1.0]], 
[[7.0, 7.0, 1.0], [4.0, 2.0, 7.0], [1.0, 7.0, 4.0]], [[3.0, 3.0, 1.0], [6.0, 5.0, 7.0], [5.0, 8.0, 3.0]]], [[[2.0, 8.0, 1.0], 
[1.0, 3.0, 2.0], [2.0, 8.0, 7.0]], [[8.0, 7.0, 1.0], [4.0, 6.0, 1.0], [4.0, 5.0, 1.0]], [[3.0, 2.0, 5.0], [5.0, 4.0, 5.0], 
[8.0, 8.0, 1.0]]], [[[6.0, 7.0, 7.0], [6.0, 1.0, 5.0], [4.0, 5.0, 5.0]], [[5.0, 8.0, 6.0], [8.0, 4.0, 7.0], [6.0, 6.0, 2.0]], 
[[2.0, 3.0, 1.0], [3.0, 4.0, 4.0], [5.0, 2.0, 5.0]]], [[[5.0, 1.0, 4.0], [3.0, 1.0, 3.0], [7.0, 6.0, 3.0]], [[7.0, 5.0, 2.0], 
[4.0, 8.0, 8.0], [4.0, 5.0, 1.0]], [[4.0, 4.0, 6.0], [6.0, 3.0, 8.0], [1.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPRN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [5.0, 8.0, 6.0], [8.0, 1.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 8.0, 6.0], [3.0, 8.0, 
3.0]], [[5.0, 1.0, 6.0], [3.0, 7.0, 7.0], [2.0, 1.0, 7.0]]], [[[1.0, 2.0, 1.0], [7.0, 8.0, 5.0], [5.0, 6.0, 6.0]], [[1.0, 7.0, 
5.0], [8.0, 3.0, 8.0], [2.0, 5.0, 8.0]], [[3.0, 6.0, 2.0], [8.0, 3.0, 2.0], [4.0, 2.0, 5.0]]], [[[2.0, 8.0, 1.0], [1.0, 8.0, 
6.0], [5.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [5.0, 5.0, 6.0], [4.0, 1.0, 1.0]], [[1.0, 7.0, 1.0], [5.0, 6.0, 5.0], [7.0, 4.0, 
3.0]]], [[[6.0, 5.0, 7.0], [6.0, 5.0, 4.0], [6.0, 1.0, 5.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 7.0], [5.0, 7.0, 4.0]], [[5.0, 2.0, 
3.0], [1.0, 8.0, 2.0], [6.0, 1.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 5.0, 3.0]], [[5.0, 8.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 2.0, 3.0]], [[8.0, 5.0, 7.0]]]))
      arg[2:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 6.0], [5.0, 8.0, 6.0], [8.0, 1.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 8.0, 6.0], [3.0, 
8.0, 3.0]], [[5.0, 1.0, 6.0], [3.0, 7.0, 7.0], [2.0, 1.0, 7.0]]], [[[1.0, 2.0, 1.0], [7.0, 8.0, 5.0], [5.0, 6.0, 6.0]], [[1.0, 
7.0, 5.0], [8.0, 3.0, 8.0], [2.0, 5.0, 8.0]], [[3.0, 6.0, 2.0], [8.0, 3.0, 2.0], [4.0, 2.0, 5.0]]], [[[2.0, 8.0, 1.0], [1.0, 
8.0, 6.0], [5.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [6.0, 5.0, 3.0], [4.0, 1.0, 1.0]], [[1.0, 7.0, 1.0], [5.0, 6.0, 5.0], [7.0, 4.0, 
3.0]]], [[[6.0, 5.0, 7.0], [6.0, 5.0, 4.0], [6.0, 1.0, 5.0]], [[6.0, 3.0, 3.0], [5.0, 8.0, 8.0], [5.0, 7.0, 4.0]], [[5.0, 2.0, 
3.0], [1.0, 8.0, 2.0], [6.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 6.0], [5.0, 8.0, 6.0], [8.0, 1.0, 7.0]], 
[[8.0, 8.0, 3.0], [6.0, 8.0, 6.0], [3.0, 8.0, 3.0]], [[5.0, 1.0, 6.0], [3.0, 7.0, 7.0], [2.0, 1.0, 7.0]]], [[[1.0, 2.0, 1.0], 
[7.0, 8.0, 5.0], [5.0, 6.0, 6.0]], [[1.0, 7.0, 5.0], [8.0, 3.0, 8.0], [2.0, 5.0, 8.0]], [[3.0, 6.0, 2.0], [8.0, 3.0, 2.0], 
[4.0, 2.0, 5.0]]], [[[2.0, 8.0, 1.0], [1.0, 8.0, 6.0], [5.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [8.0, 2.0, 3.0], [4.0, 1.0, 1.0]], 
[[1.0, 7.0, 1.0], [5.0, 6.0, 5.0], [7.0, 4.0, 3.0]]], [[[6.0, 5.0, 7.0], [6.0, 5.0, 4.0], [6.0, 1.0, 5.0]], [[6.0, 3.0, 3.0], 
[8.0, 5.0, 7.0], [5.0, 7.0, 4.0]], [[5.0, 2.0, 3.0], [1.0, 8.0, 2.0], [6.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LCRN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 5.0], [8.0, 5.0, 3.0], [3.0, 2.0, 2.0]], [[7.0, 2.0, 3.0], [1.0, 4.0, 5.0], [8.0, 2.0, 
1.0]], [[8.0, 1.0, 5.0], [2.0, 3.0, 4.0], [4.0, 4.0, 3.0]]], [[[8.0, 6.0, 1.0], [8.0, 6.0, 6.0], [2.0, 2.0, 6.0]], [[6.0, 6.0, 
5.0], [2.0, 8.0, 6.0], [5.0, 2.0, 2.0]], [[4.0, 7.0, 3.0], [3.0, 3.0, 3.0], [6.0, 3.0, 7.0]]], [[[1.0, 7.0, 1.0], [5.0, 6.0, 
7.0], [2.0, 2.0, 4.0]], [[3.0, 6.0, 7.0], [1.0, 3.0, 2.0], [7.0, 8.0, 1.0]], [[3.0, 7.0, 6.0], [7.0, 5.0, 6.0], [2.0, 5.0, 
4.0]]], [[[3.0, 4.0, 5.0], [6.0, 7.0, 7.0], [5.0, 8.0, 5.0]], [[4.0, 4.0, 8.0], [7.0, 5.0, 7.0], [3.0, 8.0, 6.0]], [[7.0, 7.0, 
7.0], [6.0, 4.0, 7.0], [4.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 5.0, 5.0]], [[8.0, 1.0, 2.0]], [[1.0, 8.0, 8.0]]], [[[5.0, 5.0, 2.0]], [[4.0, 8.0, 5.0]], 
[[8.0, 4.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 3.0]], [[5.0, 5.0, 1.0]], [[8.0, 1.0, 3.0]]], [[[1.0, 4.0, 4.0]], [[4.0, 
4.0, 1.0]], [[4.0, 5.0, 2.0]]]]))
      arg[2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 5.0], [8.0, 5.0, 3.0], [3.0, 2.0, 2.0]], [[7.0, 2.0, 3.0], [1.0, 4.0, 5.0], [8.0, 
2.0, 1.0]], [[8.0, 1.0, 5.0], [2.0, 3.0, 4.0], [4.0, 4.0, 3.0]]], [[[8.0, 6.0, 1.0], [8.0, 6.0, 6.0], [2.0, 2.0, 6.0]], [[6.0, 
6.0, 5.0], [2.0, 8.0, 6.0], [5.0, 2.0, 2.0]], [[4.0, 7.0, 3.0], [3.0, 3.0, 3.0], [6.0, 3.0, 7.0]]], [[[1.0, 7.0, 1.0], [5.0, 
5.0, 5.0], [2.0, 2.0, 4.0]], [[3.0, 6.0, 7.0], [8.0, 1.0, 2.0], [7.0, 8.0, 1.0]], [[3.0, 7.0, 6.0], [1.0, 8.0, 8.0], [2.0, 5.0, 
4.0]]], [[[3.0, 4.0, 5.0], [5.0, 5.0, 2.0], [5.0, 8.0, 5.0]], [[4.0, 4.0, 8.0], [4.0, 8.0, 5.0], [3.0, 8.0, 6.0]], [[7.0, 7.0, 
7.0], [8.0, 4.0, 4.0], [4.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 5.0], [8.0, 5.0, 3.0], [3.0, 2.0, 2.0]], 
[[7.0, 2.0, 3.0], [1.0, 4.0, 5.0], [8.0, 2.0, 1.0]], [[8.0, 1.0, 5.0], [2.0, 3.0, 4.0], [4.0, 4.0, 3.0]]], [[[8.0, 6.0, 1.0], 
[8.0, 6.0, 6.0], [2.0, 2.0, 6.0]], [[6.0, 6.0, 5.0], [2.0, 8.0, 6.0], [5.0, 2.0, 2.0]], [[4.0, 7.0, 3.0], [3.0, 3.0, 3.0], 
[6.0, 3.0, 7.0]]], [[[1.0, 7.0, 1.0], [7.0, 2.0, 3.0], [2.0, 2.0, 4.0]], [[3.0, 6.0, 7.0], [5.0, 5.0, 1.0], [7.0, 8.0, 1.0]], 
[[3.0, 7.0, 6.0], [8.0, 1.0, 3.0], [2.0, 5.0, 4.0]]], [[[3.0, 4.0, 5.0], [1.0, 4.0, 4.0], [5.0, 8.0, 5.0]], [[4.0, 4.0, 8.0], 
[4.0, 4.0, 1.0], [3.0, 8.0, 6.0]], [[7.0, 7.0, 7.0], [4.0, 5.0, 2.0], [4.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPCN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 7.0], [3.0, 8.0, 6.0], [2.0, 4.0, 5.0]], [[1.0, 6.0, 3.0], [7.0, 7.0, 8.0], [4.0, 5.0, 
3.0]], [[3.0, 5.0, 8.0], [5.0, 8.0, 7.0], [6.0, 3.0, 7.0]]], [[[8.0, 1.0, 3.0], [4.0, 8.0, 2.0], [2.0, 8.0, 7.0]], [[5.0, 8.0, 
6.0], [7.0, 1.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 7.0, 3.0], [2.0, 4.0, 3.0], [4.0, 3.0, 3.0]]], [[[5.0, 7.0, 4.0], [7.0, 8.0, 
6.0], [4.0, 1.0, 8.0]], [[1.0, 3.0, 5.0], [2.0, 1.0, 8.0], [8.0, 2.0, 5.0]], [[6.0, 3.0, 6.0], [1.0, 3.0, 1.0], [7.0, 7.0, 
7.0]]], [[[7.0, 2.0, 8.0], [2.0, 6.0, 1.0], [5.0, 3.0, 6.0]], [[8.0, 8.0, 5.0], [6.0, 2.0, 8.0], [8.0, 6.0, 1.0]], [[7.0, 4.0, 
7.0], [8.0, 8.0, 1.0], [8.0, 8.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 3.0, 4.0], [7.0, 5.0, 3.0], [5.0, 7.0, 3.0]], [[5.0, 3.0, 3.0], [6.0, 8.0, 7.0], [6.0, 3.0, 
6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 8.0, 6.0], [5.0, 6.0, 2.0], [5.0, 3.0, 8.0]], [[5.0, 4.0, 6.0], [7.0, 4.0, 7.0], 
[2.0, 4.0, 3.0]]]))
      arg[2:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 7.0], [3.0, 8.0, 6.0], [2.0, 4.0, 5.0]], [[1.0, 6.0, 3.0], [7.0, 7.0, 8.0], [4.0, 
5.0, 3.0]], [[3.0, 5.0, 8.0], [5.0, 8.0, 7.0], [6.0, 3.0, 7.0]]], [[[8.0, 1.0, 3.0], [4.0, 8.0, 2.0], [2.0, 8.0, 7.0]], [[5.0, 
8.0, 6.0], [7.0, 1.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 7.0, 3.0], [2.0, 4.0, 3.0], [4.0, 3.0, 3.0]]], [[[5.0, 7.0, 4.0], [7.0, 
8.0, 6.0], [4.0, 1.0, 8.0]], [[3.0, 3.0, 4.0], [7.0, 5.0, 3.0], [5.0, 7.0, 3.0]], [[6.0, 3.0, 6.0], [1.0, 3.0, 1.0], [7.0, 7.0, 
7.0]]], [[[7.0, 2.0, 8.0], [2.0, 6.0, 1.0], [5.0, 3.0, 6.0]], [[5.0, 3.0, 3.0], [6.0, 8.0, 7.0], [6.0, 3.0, 6.0]], [[7.0, 4.0, 
7.0], [8.0, 8.0, 1.0], [8.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 7.0], [3.0, 8.0, 6.0], [2.0, 4.0, 5.0]], 
[[1.0, 6.0, 3.0], [7.0, 7.0, 8.0], [4.0, 5.0, 3.0]], [[3.0, 5.0, 8.0], [5.0, 8.0, 7.0], [6.0, 3.0, 7.0]]], [[[8.0, 1.0, 3.0], 
[4.0, 8.0, 2.0], [2.0, 8.0, 7.0]], [[5.0, 8.0, 6.0], [7.0, 1.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 7.0, 3.0], [2.0, 4.0, 3.0], 
[4.0, 3.0, 3.0]]], [[[5.0, 7.0, 4.0], [7.0, 8.0, 6.0], [4.0, 1.0, 8.0]], [[3.0, 8.0, 6.0], [5.0, 6.0, 2.0], [5.0, 3.0, 8.0]], 
[[6.0, 3.0, 6.0], [1.0, 3.0, 1.0], [7.0, 7.0, 7.0]]], [[[7.0, 2.0, 8.0], [2.0, 6.0, 1.0], [5.0, 3.0, 6.0]], [[5.0, 4.0, 6.0], 
[7.0, 4.0, 7.0], [2.0, 4.0, 3.0]], [[7.0, 4.0, 7.0], [8.0, 8.0, 1.0], [8.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LCUN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 1.0], [3.0, 7.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 4.0, 1.0], [6.0, 2.0, 4.0], [3.0, 6.0, 
3.0]], [[2.0, 1.0, 6.0], [1.0, 7.0, 7.0], [4.0, 7.0, 2.0]]], [[[1.0, 1.0, 7.0], [1.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[7.0, 3.0, 
4.0], [7.0, 4.0, 8.0], [4.0, 2.0, 7.0]], [[6.0, 7.0, 2.0], [5.0, 3.0, 2.0], [6.0, 2.0, 1.0]]], [[[6.0, 3.0, 4.0], [2.0, 4.0, 
1.0], [1.0, 1.0, 7.0]], [[3.0, 7.0, 7.0], [7.0, 5.0, 7.0], [2.0, 5.0, 7.0]], [[1.0, 3.0, 1.0], [6.0, 1.0, 4.0], [7.0, 4.0, 
4.0]]], [[[2.0, 8.0, 4.0], [2.0, 8.0, 3.0], [3.0, 2.0, 3.0]], [[4.0, 6.0, 3.0], [6.0, 7.0, 4.0], [3.0, 3.0, 5.0]], [[7.0, 2.0, 
8.0], [5.0, 6.0, 2.0], [6.0, 3.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 8.0, 4.0], [3.0, 7.0, 8.0]], [[4.0, 2.0, 3.0], [8.0, 1.0, 1.0]], [[7.0, 2.0, 4.0], [8.0, 
8.0, 2.0]]], [[[5.0, 5.0, 4.0], [1.0, 6.0, 6.0]], [[5.0, 4.0, 7.0], [4.0, 5.0, 3.0]], [[1.0, 6.0, 3.0], [4.0, 7.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 6.0], [1.0, 2.0, 3.0]], [[4.0, 2.0, 8.0], [2.0, 8.0, 3.0]], [[1.0, 7.0, 
3.0], [7.0, 3.0, 1.0]]], [[[5.0, 7.0, 1.0], [2.0, 4.0, 4.0]], [[7.0, 3.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], [7.0, 5.0, 
1.0]]]]))
      arg[2:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 1.0], [3.0, 7.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 4.0, 1.0], [6.0, 2.0, 4.0], [3.0, 
6.0, 3.0]], [[2.0, 1.0, 6.0], [1.0, 7.0, 7.0], [4.0, 7.0, 2.0]]], [[[1.0, 1.0, 7.0], [1.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[7.0, 
3.0, 4.0], [7.0, 4.0, 8.0], [4.0, 2.0, 7.0]], [[6.0, 7.0, 2.0], [5.0, 3.0, 2.0], [6.0, 2.0, 1.0]]], [[[7.0, 8.0, 4.0], [3.0, 
7.0, 8.0], [1.0, 1.0, 7.0]], [[4.0, 2.0, 3.0], [8.0, 1.0, 1.0], [2.0, 5.0, 7.0]], [[7.0, 2.0, 4.0], [8.0, 8.0, 2.0], [7.0, 4.0, 
4.0]]], [[[5.0, 5.0, 4.0], [1.0, 6.0, 6.0], [3.0, 2.0, 3.0]], [[5.0, 4.0, 7.0], [4.0, 5.0, 3.0], [3.0, 3.0, 5.0]], [[1.0, 6.0, 
3.0], [4.0, 7.0, 2.0], [6.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 1.0], [3.0, 7.0, 1.0], [1.0, 8.0, 8.0]], 
[[1.0, 4.0, 1.0], [6.0, 2.0, 4.0], [3.0, 6.0, 3.0]], [[2.0, 1.0, 6.0], [1.0, 7.0, 7.0], [4.0, 7.0, 2.0]]], [[[1.0, 1.0, 7.0], 
[1.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[7.0, 3.0, 4.0], [7.0, 4.0, 8.0], [4.0, 2.0, 7.0]], [[6.0, 7.0, 2.0], [5.0, 3.0, 2.0], 
[6.0, 2.0, 1.0]]], [[[3.0, 5.0, 6.0], [1.0, 2.0, 3.0], [1.0, 1.0, 7.0]], [[4.0, 2.0, 8.0], [2.0, 8.0, 3.0], [2.0, 5.0, 7.0]], 
[[1.0, 7.0, 3.0], [7.0, 3.0, 1.0], [7.0, 4.0, 4.0]]], [[[5.0, 7.0, 1.0], [2.0, 4.0, 4.0], [3.0, 2.0, 3.0]], [[7.0, 3.0, 2.0], 
[2.0, 2.0, 2.0], [3.0, 3.0, 5.0]], [[4.0, 4.0, 4.0], [7.0, 5.0, 1.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LPPN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 2.0], [3.0, 1.0, 1.0], [4.0, 7.0, 2.0]], [[7.0, 4.0, 3.0], [5.0, 4.0, 7.0], [7.0, 6.0, 
7.0]], [[1.0, 1.0, 8.0], [3.0, 3.0, 5.0], [2.0, 2.0, 5.0]]], [[[5.0, 2.0, 1.0], [8.0, 2.0, 2.0], [2.0, 2.0, 8.0]], [[6.0, 7.0, 
1.0], [7.0, 4.0, 2.0], [6.0, 8.0, 5.0]], [[1.0, 4.0, 6.0], [3.0, 3.0, 1.0], [6.0, 4.0, 4.0]]], [[[2.0, 3.0, 8.0], [5.0, 4.0, 
3.0], [5.0, 1.0, 5.0]], [[1.0, 8.0, 8.0], [4.0, 4.0, 8.0], [5.0, 2.0, 4.0]], [[3.0, 8.0, 5.0], [8.0, 3.0, 8.0], [6.0, 8.0, 
3.0]]], [[[5.0, 8.0, 8.0], [8.0, 4.0, 7.0], [3.0, 5.0, 6.0]], [[6.0, 3.0, 6.0], [7.0, 5.0, 8.0], [1.0, 7.0, 4.0]], [[6.0, 8.0, 
2.0], [3.0, 2.0, 1.0], [2.0, 1.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[6.0, 5.0, 2.0], [4.0, 3.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 7.0, 7.0], [5.0, 1.0, 6.0]]))
      arg[2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 2.0], [3.0, 1.0, 1.0], [4.0, 7.0, 2.0]], [[7.0, 4.0, 3.0], [5.0, 4.0, 7.0], [7.0, 
6.0, 7.0]], [[1.0, 1.0, 8.0], [3.0, 3.0, 5.0], [2.0, 2.0, 5.0]]], [[[5.0, 2.0, 1.0], [8.0, 2.0, 2.0], [2.0, 2.0, 8.0]], [[6.0, 
7.0, 1.0], [7.0, 4.0, 2.0], [6.0, 8.0, 5.0]], [[1.0, 4.0, 6.0], [3.0, 3.0, 1.0], [6.0, 4.0, 4.0]]], [[[2.0, 3.0, 8.0], [5.0, 
4.0, 3.0], [5.0, 1.0, 5.0]], [[1.0, 8.0, 8.0], [6.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[3.0, 8.0, 5.0], [8.0, 3.0, 8.0], [6.0, 8.0, 
3.0]]], [[[5.0, 8.0, 8.0], [8.0, 4.0, 7.0], [3.0, 5.0, 6.0]], [[6.0, 3.0, 6.0], [4.0, 3.0, 1.0], [1.0, 7.0, 4.0]], [[6.0, 8.0, 
2.0], [3.0, 2.0, 1.0], [2.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 2.0], [3.0, 1.0, 1.0], [4.0, 7.0, 2.0]], 
[[7.0, 4.0, 3.0], [5.0, 4.0, 7.0], [7.0, 6.0, 7.0]], [[1.0, 1.0, 8.0], [3.0, 3.0, 5.0], [2.0, 2.0, 5.0]]], [[[5.0, 2.0, 1.0], 
[8.0, 2.0, 2.0], [2.0, 2.0, 8.0]], [[6.0, 7.0, 1.0], [7.0, 4.0, 2.0], [6.0, 8.0, 5.0]], [[1.0, 4.0, 6.0], [3.0, 3.0, 1.0], 
[6.0, 4.0, 4.0]]], [[[2.0, 3.0, 8.0], [5.0, 4.0, 3.0], [5.0, 1.0, 5.0]], [[1.0, 8.0, 8.0], [2.0, 7.0, 7.0], [5.0, 2.0, 4.0]], 
[[3.0, 8.0, 5.0], [8.0, 3.0, 8.0], [6.0, 8.0, 3.0]]], [[[5.0, 8.0, 8.0], [8.0, 4.0, 7.0], [3.0, 5.0, 6.0]], [[6.0, 3.0, 6.0], 
[5.0, 1.0, 6.0], [1.0, 7.0, 4.0]], [[6.0, 8.0, 2.0], [3.0, 2.0, 1.0], [2.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLLN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 5.0], [3.0, 3.0, 1.0], [3.0, 5.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 6.0, 7.0], [7.0, 4.0, 
6.0]], [[8.0, 1.0, 7.0], [8.0, 7.0, 5.0], [4.0, 3.0, 3.0]]], [[[4.0, 8.0, 6.0], [3.0, 7.0, 1.0], [4.0, 5.0, 1.0]], [[1.0, 2.0, 
3.0], [2.0, 5.0, 6.0], [1.0, 4.0, 8.0]], [[4.0, 2.0, 8.0], [8.0, 6.0, 6.0], [2.0, 4.0, 7.0]]], [[[2.0, 2.0, 4.0], [2.0, 8.0, 
5.0], [6.0, 2.0, 5.0]], [[3.0, 3.0, 3.0], [6.0, 2.0, 6.0], [7.0, 7.0, 5.0]], [[5.0, 3.0, 7.0], [1.0, 1.0, 8.0], [7.0, 2.0, 
1.0]]], [[[1.0, 6.0, 2.0], [8.0, 4.0, 7.0], [2.0, 8.0, 6.0]], [[2.0, 5.0, 2.0], [4.0, 1.0, 7.0], [5.0, 2.0, 1.0]], [[7.0, 7.0, 
6.0], [4.0, 5.0, 8.0], [2.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 4.0, 7.0]]], [[[4.0, 2.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 5.0]]], [[[4.0, 5.0, 1.0]]]]))
      arg[1:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 5.0], [3.0, 3.0, 1.0], [3.0, 5.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 6.0, 7.0], [7.0, 
4.0, 6.0]], [[8.0, 1.0, 7.0], [8.0, 7.0, 5.0], [4.0, 3.0, 3.0]]], [[[4.0, 8.0, 6.0], [3.0, 7.0, 1.0], [4.0, 5.0, 1.0]], [[1.0, 
2.0, 3.0], [2.0, 5.0, 6.0], [1.0, 4.0, 8.0]], [[4.0, 2.0, 8.0], [8.0, 6.0, 6.0], [4.0, 4.0, 7.0]]], [[[2.0, 2.0, 4.0], [2.0, 
8.0, 5.0], [6.0, 2.0, 5.0]], [[3.0, 3.0, 3.0], [6.0, 2.0, 6.0], [7.0, 7.0, 5.0]], [[5.0, 3.0, 7.0], [1.0, 1.0, 8.0], [4.0, 2.0, 
3.0]]], [[[1.0, 6.0, 2.0], [8.0, 4.0, 7.0], [2.0, 8.0, 6.0]], [[2.0, 5.0, 2.0], [4.0, 1.0, 7.0], [5.0, 2.0, 1.0]], [[7.0, 7.0, 
6.0], [4.0, 5.0, 8.0], [2.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 5.0], [3.0, 3.0, 1.0], [3.0, 5.0, 6.0]], 
[[7.0, 8.0, 5.0], [4.0, 6.0, 7.0], [7.0, 4.0, 6.0]], [[8.0, 1.0, 7.0], [8.0, 7.0, 5.0], [4.0, 3.0, 3.0]]], [[[4.0, 8.0, 6.0], 
[3.0, 7.0, 1.0], [4.0, 5.0, 1.0]], [[1.0, 2.0, 3.0], [2.0, 5.0, 6.0], [1.0, 4.0, 8.0]], [[4.0, 2.0, 8.0], [8.0, 6.0, 6.0], 
[6.0, 2.0, 5.0]]], [[[2.0, 2.0, 4.0], [2.0, 8.0, 5.0], [6.0, 2.0, 5.0]], [[3.0, 3.0, 3.0], [6.0, 2.0, 6.0], [7.0, 7.0, 5.0]], 
[[5.0, 3.0, 7.0], [1.0, 1.0, 8.0], [4.0, 5.0, 1.0]]], [[[1.0, 6.0, 2.0], [8.0, 4.0, 7.0], [2.0, 8.0, 6.0]], [[2.0, 5.0, 2.0], 
[4.0, 1.0, 7.0], [5.0, 2.0, 1.0]], [[7.0, 7.0, 6.0], [4.0, 5.0, 8.0], [2.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RULN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 3.0], [2.0, 3.0, 5.0], [8.0, 4.0, 2.0]], [[3.0, 7.0, 4.0], [2.0, 3.0, 3.0], [2.0, 1.0, 
6.0]], [[7.0, 4.0, 1.0], [8.0, 4.0, 1.0], [1.0, 6.0, 1.0]]], [[[2.0, 8.0, 2.0], [4.0, 5.0, 2.0], [6.0, 5.0, 3.0]], [[6.0, 6.0, 
7.0], [1.0, 1.0, 8.0], [4.0, 1.0, 5.0]], [[8.0, 2.0, 6.0], [6.0, 3.0, 6.0], [2.0, 7.0, 7.0]]], [[[3.0, 8.0, 6.0], [1.0, 8.0, 
2.0], [1.0, 7.0, 6.0]], [[2.0, 4.0, 3.0], [5.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[4.0, 2.0, 6.0], [3.0, 4.0, 6.0], [3.0, 4.0, 
5.0]]], [[[3.0, 8.0, 6.0], [8.0, 2.0, 8.0], [1.0, 3.0, 7.0]], [[7.0, 5.0, 4.0], [4.0, 4.0, 8.0], [1.0, 5.0, 6.0]], [[8.0, 3.0, 
3.0], [2.0, 6.0, 1.0], [3.0, 7.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 4.0, 4.0]], [[2.0, 8.0, 2.0]]], [[[1.0, 4.0, 1.0]], [[7.0, 6.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 5.0]], [[7.0, 2.0, 3.0]]], [[[7.0, 5.0, 2.0]], [[6.0, 7.0, 6.0]]]]))
      arg[1:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 3.0], [2.0, 3.0, 5.0], [8.0, 4.0, 2.0]], [[3.0, 7.0, 4.0], [2.0, 3.0, 3.0], [2.0, 
1.0, 6.0]], [[7.0, 4.0, 1.0], [8.0, 4.0, 1.0], [1.0, 6.0, 1.0]]], [[[2.0, 8.0, 2.0], [4.0, 5.0, 2.0], [7.0, 4.0, 4.0]], [[6.0, 
6.0, 7.0], [1.0, 1.0, 8.0], [2.0, 8.0, 2.0]], [[8.0, 2.0, 6.0], [6.0, 3.0, 6.0], [2.0, 7.0, 7.0]]], [[[3.0, 8.0, 6.0], [1.0, 
8.0, 2.0], [1.0, 4.0, 1.0]], [[2.0, 4.0, 3.0], [5.0, 4.0, 6.0], [7.0, 6.0, 8.0]], [[4.0, 2.0, 6.0], [3.0, 4.0, 6.0], [3.0, 4.0, 
5.0]]], [[[3.0, 8.0, 6.0], [8.0, 2.0, 8.0], [1.0, 3.0, 7.0]], [[7.0, 5.0, 4.0], [4.0, 4.0, 8.0], [1.0, 5.0, 6.0]], [[8.0, 3.0, 
3.0], [2.0, 6.0, 1.0], [3.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 3.0], [2.0, 3.0, 5.0], [8.0, 4.0, 2.0]], 
[[3.0, 7.0, 4.0], [2.0, 3.0, 3.0], [2.0, 1.0, 6.0]], [[7.0, 4.0, 1.0], [8.0, 4.0, 1.0], [1.0, 6.0, 1.0]]], [[[2.0, 8.0, 2.0], 
[4.0, 5.0, 2.0], [4.0, 6.0, 5.0]], [[6.0, 6.0, 7.0], [1.0, 1.0, 8.0], [7.0, 2.0, 3.0]], [[8.0, 2.0, 6.0], [6.0, 3.0, 6.0], 
[2.0, 7.0, 7.0]]], [[[3.0, 8.0, 6.0], [1.0, 8.0, 2.0], [7.0, 5.0, 2.0]], [[2.0, 4.0, 3.0], [5.0, 4.0, 6.0], [6.0, 7.0, 6.0]], 
[[4.0, 2.0, 6.0], [3.0, 4.0, 6.0], [3.0, 4.0, 5.0]]], [[[3.0, 8.0, 6.0], [8.0, 2.0, 8.0], [1.0, 3.0, 7.0]], [[7.0, 5.0, 4.0], 
[4.0, 4.0, 8.0], [1.0, 5.0, 6.0]], [[8.0, 3.0, 3.0], [2.0, 6.0, 1.0], [3.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CRUN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 2.0], [8.0, 8.0, 3.0], [5.0, 1.0, 7.0]], [[5.0, 7.0, 5.0], [7.0, 7.0, 8.0], [7.0, 6.0, 
6.0]], [[6.0, 6.0, 1.0], [7.0, 8.0, 6.0], [4.0, 4.0, 5.0]]], [[[7.0, 2.0, 6.0], [3.0, 7.0, 2.0], [1.0, 8.0, 5.0]], [[7.0, 8.0, 
8.0], [8.0, 4.0, 7.0], [7.0, 4.0, 6.0]], [[8.0, 2.0, 6.0], [2.0, 7.0, 5.0], [5.0, 6.0, 7.0]]], [[[6.0, 6.0, 3.0], [6.0, 6.0, 
8.0], [2.0, 8.0, 2.0]], [[5.0, 7.0, 7.0], [5.0, 3.0, 2.0], [6.0, 8.0, 5.0]], [[5.0, 6.0, 5.0], [3.0, 4.0, 1.0], [1.0, 2.0, 
5.0]]], [[[1.0, 7.0, 1.0], [7.0, 7.0, 6.0], [1.0, 1.0, 6.0]], [[3.0, 2.0, 6.0], [1.0, 3.0, 6.0], [2.0, 8.0, 6.0]], [[5.0, 5.0, 
5.0], [1.0, 6.0, 2.0], [1.0, 6.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 1.0, 6.0], [2.0, 2.0, 3.0]]], [[[2.0, 2.0, 5.0], [1.0, 8.0, 1.0]]], [[[5.0, 8.0, 4.0], 
[2.0, 8.0, 6.0]]], [[[5.0, 4.0, 5.0], [1.0, 5.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 7.0], [5.0, 5.0, 7.0]]], [[[2.0, 7.0, 8.0], [2.0, 1.0, 3.0]]], [[[7.0, 
6.0, 6.0], [4.0, 1.0, 4.0]]], [[[8.0, 3.0, 8.0], [3.0, 4.0, 6.0]]]]))
      arg[:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 2.0], [8.0, 8.0, 3.0], [5.0, 1.0, 7.0]], [[2.0, 1.0, 6.0], [2.0, 2.0, 3.0], [7.0, 
6.0, 6.0]], [[6.0, 6.0, 1.0], [7.0, 8.0, 6.0], [4.0, 4.0, 5.0]]], [[[7.0, 2.0, 6.0], [3.0, 7.0, 2.0], [1.0, 8.0, 5.0]], [[2.0, 
2.0, 5.0], [1.0, 8.0, 1.0], [7.0, 4.0, 6.0]], [[8.0, 2.0, 6.0], [2.0, 7.0, 5.0], [5.0, 6.0, 7.0]]], [[[6.0, 6.0, 3.0], [6.0, 
6.0, 8.0], [2.0, 8.0, 2.0]], [[5.0, 8.0, 4.0], [2.0, 8.0, 6.0], [6.0, 8.0, 5.0]], [[5.0, 6.0, 5.0], [3.0, 4.0, 1.0], [1.0, 2.0, 
5.0]]], [[[1.0, 7.0, 1.0], [7.0, 7.0, 6.0], [1.0, 1.0, 6.0]], [[5.0, 4.0, 5.0], [1.0, 5.0, 7.0], [2.0, 8.0, 6.0]], [[5.0, 5.0, 
5.0], [1.0, 6.0, 2.0], [1.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 2.0], [8.0, 8.0, 3.0], [5.0, 1.0, 7.0]], 
[[1.0, 2.0, 7.0], [5.0, 5.0, 7.0], [7.0, 6.0, 6.0]], [[6.0, 6.0, 1.0], [7.0, 8.0, 6.0], [4.0, 4.0, 5.0]]], [[[7.0, 2.0, 6.0], 
[3.0, 7.0, 2.0], [1.0, 8.0, 5.0]], [[2.0, 7.0, 8.0], [2.0, 1.0, 3.0], [7.0, 4.0, 6.0]], [[8.0, 2.0, 6.0], [2.0, 7.0, 5.0], 
[5.0, 6.0, 7.0]]], [[[6.0, 6.0, 3.0], [6.0, 6.0, 8.0], [2.0, 8.0, 2.0]], [[7.0, 6.0, 6.0], [4.0, 1.0, 4.0], [6.0, 8.0, 5.0]], 
[[5.0, 6.0, 5.0], [3.0, 4.0, 1.0], [1.0, 2.0, 5.0]]], [[[1.0, 7.0, 1.0], [7.0, 7.0, 6.0], [1.0, 1.0, 6.0]], [[8.0, 3.0, 8.0], 
[3.0, 4.0, 6.0], [2.0, 8.0, 6.0]], [[5.0, 5.0, 5.0], [1.0, 6.0, 2.0], [1.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PULN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 2.0], [6.0, 6.0, 5.0], [6.0, 1.0, 8.0]], [[8.0, 4.0, 7.0], [4.0, 8.0, 7.0], [4.0, 3.0, 
3.0]], [[6.0, 5.0, 6.0], [1.0, 8.0, 2.0], [1.0, 2.0, 3.0]]], [[[4.0, 5.0, 3.0], [6.0, 4.0, 5.0], [6.0, 3.0, 1.0]], [[2.0, 3.0, 
3.0], [5.0, 7.0, 1.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 8.0], [2.0, 2.0, 8.0], [5.0, 7.0, 3.0]]], [[[4.0, 8.0, 3.0], [8.0, 7.0, 
8.0], [2.0, 6.0, 4.0]], [[6.0, 5.0, 8.0], [6.0, 5.0, 5.0], [2.0, 3.0, 4.0]], [[8.0, 5.0, 2.0], [5.0, 2.0, 4.0], [4.0, 8.0, 
8.0]]], [[[3.0, 1.0, 5.0], [6.0, 7.0, 6.0], [7.0, 1.0, 1.0]], [[7.0, 5.0, 3.0], [1.0, 4.0, 1.0], [8.0, 6.0, 3.0]], [[6.0, 7.0, 
3.0], [4.0, 6.0, 5.0], [7.0, 6.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 7.0, 3.0]], [[7.0, 4.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 4.0, 6.0]], [[7.0, 7.0, 7.0]]]))
      arg[2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 2.0], [6.0, 6.0, 5.0], [6.0, 1.0, 8.0]], [[8.0, 4.0, 7.0], [4.0, 8.0, 7.0], [4.0, 
3.0, 3.0]], [[6.0, 5.0, 6.0], [1.0, 8.0, 2.0], [1.0, 2.0, 3.0]]], [[[4.0, 5.0, 3.0], [6.0, 4.0, 5.0], [6.0, 3.0, 1.0]], [[2.0, 
3.0, 3.0], [5.0, 7.0, 1.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 8.0], [2.0, 2.0, 8.0], [5.0, 7.0, 3.0]]], [[[4.0, 8.0, 3.0], [8.0, 
7.0, 8.0], [1.0, 7.0, 3.0]], [[6.0, 5.0, 8.0], [6.0, 5.0, 5.0], [7.0, 4.0, 3.0]], [[8.0, 5.0, 2.0], [5.0, 2.0, 4.0], [4.0, 8.0, 
8.0]]], [[[3.0, 1.0, 5.0], [6.0, 7.0, 6.0], [7.0, 1.0, 1.0]], [[7.0, 5.0, 3.0], [1.0, 4.0, 1.0], [8.0, 6.0, 3.0]], [[6.0, 7.0, 
3.0], [4.0, 6.0, 5.0], [7.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 2.0], [6.0, 6.0, 5.0], [6.0, 1.0, 8.0]], 
[[8.0, 4.0, 7.0], [4.0, 8.0, 7.0], [4.0, 3.0, 3.0]], [[6.0, 5.0, 6.0], [1.0, 8.0, 2.0], [1.0, 2.0, 3.0]]], [[[4.0, 5.0, 3.0], 
[6.0, 4.0, 5.0], [6.0, 3.0, 1.0]], [[2.0, 3.0, 3.0], [5.0, 7.0, 1.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 8.0], [2.0, 2.0, 8.0], 
[5.0, 7.0, 3.0]]], [[[4.0, 8.0, 3.0], [8.0, 7.0, 8.0], [6.0, 4.0, 6.0]], [[6.0, 5.0, 8.0], [6.0, 5.0, 5.0], [7.0, 7.0, 7.0]], 
[[8.0, 5.0, 2.0], [5.0, 2.0, 4.0], [4.0, 8.0, 8.0]]], [[[3.0, 1.0, 5.0], [6.0, 7.0, 6.0], [7.0, 1.0, 1.0]], [[7.0, 5.0, 3.0], 
[1.0, 4.0, 1.0], [8.0, 6.0, 3.0]], [[6.0, 7.0, 3.0], [4.0, 6.0, 5.0], [7.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RCCN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 5.0], [3.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[3.0, 2.0, 6.0], [1.0, 7.0, 7.0], [8.0, 1.0, 
3.0]], [[5.0, 7.0, 3.0], [2.0, 2.0, 7.0], [3.0, 5.0, 5.0]]], [[[5.0, 5.0, 3.0], [4.0, 3.0, 1.0], [8.0, 7.0, 4.0]], [[3.0, 4.0, 
6.0], [8.0, 1.0, 4.0], [3.0, 2.0, 3.0]], [[3.0, 5.0, 7.0], [8.0, 1.0, 4.0], [2.0, 1.0, 5.0]]], [[[5.0, 8.0, 3.0], [2.0, 1.0, 
3.0], [5.0, 4.0, 6.0]], [[8.0, 4.0, 1.0], [4.0, 6.0, 5.0], [7.0, 6.0, 2.0]], [[1.0, 4.0, 2.0], [7.0, 1.0, 4.0], [7.0, 5.0, 
7.0]]], [[[2.0, 5.0, 2.0], [1.0, 8.0, 4.0], [1.0, 6.0, 8.0]], [[2.0, 2.0, 1.0], [2.0, 2.0, 4.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 
5.0], [5.0, 7.0, 4.0], [5.0, 6.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 2.0, 3.0], [5.0, 5.0, 3.0], [2.0, 6.0, 1.0]], [[3.0, 1.0, 7.0], [4.0, 4.0, 4.0], [2.0, 3.0, 
4.0]], [[3.0, 1.0, 8.0], [2.0, 4.0, 1.0], [2.0, 6.0, 4.0]]], [[[5.0, 4.0, 5.0], [1.0, 7.0, 4.0], [1.0, 7.0, 4.0]], [[7.0, 1.0, 
6.0], [3.0, 3.0, 2.0], [1.0, 2.0, 1.0]], [[7.0, 1.0, 2.0], [7.0, 8.0, 5.0], [5.0, 7.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 5.0], [8.0, 1.0, 8.0], [3.0, 3.0, 4.0]], [[7.0, 4.0, 3.0], [1.0, 5.0, 
3.0], [2.0, 2.0, 5.0]], [[1.0, 6.0, 6.0], [7.0, 8.0, 8.0], [2.0, 3.0, 7.0]]], [[[6.0, 4.0, 5.0], [7.0, 1.0, 6.0], [1.0, 5.0, 
8.0]], [[8.0, 1.0, 7.0], [4.0, 4.0, 1.0], [8.0, 6.0, 5.0]], [[7.0, 6.0, 4.0], [2.0, 4.0, 2.0], [1.0, 6.0, 3.0]]]]))
      arg[1:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 5.0], [3.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[3.0, 2.0, 6.0], [1.0, 7.0, 7.0], [8.0, 
1.0, 3.0]], [[5.0, 7.0, 3.0], [2.0, 2.0, 7.0], [3.0, 5.0, 5.0]]], [[[8.0, 2.0, 3.0], [5.0, 5.0, 3.0], [2.0, 6.0, 1.0]], [[3.0, 
1.0, 7.0], [4.0, 4.0, 4.0], [2.0, 3.0, 4.0]], [[3.0, 1.0, 8.0], [2.0, 4.0, 1.0], [2.0, 6.0, 4.0]]], [[[5.0, 4.0, 5.0], [1.0, 
7.0, 4.0], [1.0, 7.0, 4.0]], [[7.0, 1.0, 6.0], [3.0, 3.0, 2.0], [1.0, 2.0, 1.0]], [[7.0, 1.0, 2.0], [7.0, 8.0, 5.0], [5.0, 7.0, 
3.0]]], [[[2.0, 5.0, 2.0], [1.0, 8.0, 4.0], [1.0, 6.0, 8.0]], [[2.0, 2.0, 1.0], [2.0, 2.0, 4.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 
5.0], [5.0, 7.0, 4.0], [5.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 5.0], [3.0, 3.0, 7.0], [7.0, 1.0, 7.0]], 
[[3.0, 2.0, 6.0], [1.0, 7.0, 7.0], [8.0, 1.0, 3.0]], [[5.0, 7.0, 3.0], [2.0, 2.0, 7.0], [3.0, 5.0, 5.0]]], [[[3.0, 6.0, 5.0], 
[8.0, 1.0, 8.0], [3.0, 3.0, 4.0]], [[7.0, 4.0, 3.0], [1.0, 5.0, 3.0], [2.0, 2.0, 5.0]], [[1.0, 6.0, 6.0], [7.0, 8.0, 8.0], 
[2.0, 3.0, 7.0]]], [[[6.0, 4.0, 5.0], [7.0, 1.0, 6.0], [1.0, 5.0, 8.0]], [[8.0, 1.0, 7.0], [4.0, 4.0, 1.0], [8.0, 6.0, 5.0]], 
[[7.0, 6.0, 4.0], [2.0, 4.0, 2.0], [1.0, 6.0, 3.0]]], [[[2.0, 5.0, 2.0], [1.0, 8.0, 4.0], [1.0, 6.0, 8.0]], [[2.0, 2.0, 1.0], 
[2.0, 2.0, 4.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 5.0], [5.0, 7.0, 4.0], [5.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CURN(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 5.0], [5.0, 8.0, 7.0], [3.0, 5.0, 4.0]], [[8.0, 4.0, 4.0], [5.0, 2.0, 5.0], [7.0, 6.0, 
1.0]], [[6.0, 6.0, 6.0], [4.0, 1.0, 7.0], [2.0, 3.0, 1.0]]], [[[2.0, 7.0, 1.0], [6.0, 5.0, 4.0], [7.0, 4.0, 3.0]], [[7.0, 6.0, 
2.0], [2.0, 2.0, 2.0], [5.0, 8.0, 1.0]], [[5.0, 1.0, 7.0], [1.0, 7.0, 6.0], [7.0, 7.0, 3.0]]], [[[2.0, 5.0, 8.0], [3.0, 1.0, 
8.0], [2.0, 2.0, 3.0]], [[4.0, 1.0, 1.0], [5.0, 7.0, 6.0], [1.0, 4.0, 2.0]], [[3.0, 3.0, 8.0], [4.0, 8.0, 7.0], [2.0, 7.0, 
4.0]]], [[[6.0, 2.0, 6.0], [2.0, 2.0, 2.0], [7.0, 4.0, 1.0]], [[4.0, 4.0, 7.0], [3.0, 5.0, 6.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 
2.0], [7.0, 6.0, 2.0], [3.0, 2.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 1.0, 4.0]], [[8.0, 3.0, 8.0]]], [[[5.0, 4.0, 5.0]], [[8.0, 6.0, 6.0]]], [[[3.0, 4.0, 2.0]], 
[[3.0, 8.0, 2.0]]], [[[6.0, 2.0, 4.0]], [[3.0, 5.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 2.0]], [[4.0, 2.0, 1.0]]], [[[7.0, 1.0, 8.0]], [[4.0, 6.0, 4.0]]], [[[5.0, 
3.0, 7.0]], [[8.0, 3.0, 5.0]]], [[[7.0, 1.0, 2.0]], [[4.0, 3.0, 1.0]]]]))
      arg[:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 5.0], [3.0, 1.0, 4.0], [3.0, 5.0, 4.0]], [[8.0, 4.0, 4.0], [8.0, 3.0, 8.0], [7.0, 
6.0, 1.0]], [[6.0, 6.0, 6.0], [4.0, 1.0, 7.0], [2.0, 3.0, 1.0]]], [[[2.0, 7.0, 1.0], [5.0, 4.0, 5.0], [7.0, 4.0, 3.0]], [[7.0, 
6.0, 2.0], [8.0, 6.0, 6.0], [5.0, 8.0, 1.0]], [[5.0, 1.0, 7.0], [1.0, 7.0, 6.0], [7.0, 7.0, 3.0]]], [[[2.0, 5.0, 8.0], [3.0, 
4.0, 2.0], [2.0, 2.0, 3.0]], [[4.0, 1.0, 1.0], [3.0, 8.0, 2.0], [1.0, 4.0, 2.0]], [[3.0, 3.0, 8.0], [4.0, 8.0, 7.0], [2.0, 7.0, 
4.0]]], [[[6.0, 2.0, 6.0], [6.0, 2.0, 4.0], [7.0, 4.0, 1.0]], [[4.0, 4.0, 7.0], [3.0, 5.0, 3.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 
2.0], [7.0, 6.0, 2.0], [3.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 5.0], [7.0, 2.0, 2.0], [3.0, 5.0, 4.0]], 
[[8.0, 4.0, 4.0], [4.0, 2.0, 1.0], [7.0, 6.0, 1.0]], [[6.0, 6.0, 6.0], [4.0, 1.0, 7.0], [2.0, 3.0, 1.0]]], [[[2.0, 7.0, 1.0], 
[7.0, 1.0, 8.0], [7.0, 4.0, 3.0]], [[7.0, 6.0, 2.0], [4.0, 6.0, 4.0], [5.0, 8.0, 1.0]], [[5.0, 1.0, 7.0], [1.0, 7.0, 6.0], 
[7.0, 7.0, 3.0]]], [[[2.0, 5.0, 8.0], [5.0, 3.0, 7.0], [2.0, 2.0, 3.0]], [[4.0, 1.0, 1.0], [8.0, 3.0, 5.0], [1.0, 4.0, 2.0]], 
[[3.0, 3.0, 8.0], [4.0, 8.0, 7.0], [2.0, 7.0, 4.0]]], [[[6.0, 2.0, 6.0], [7.0, 1.0, 2.0], [7.0, 4.0, 1.0]], [[4.0, 4.0, 7.0], 
[4.0, 3.0, 1.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 2.0], [7.0, 6.0, 2.0], [3.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UUUP(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 1.0], [1.0, 5.0, 2.0], [6.0, 6.0, 8.0]], [[6.0, 5.0, 1.0], [8.0, 7.0, 6.0], [4.0, 2.0, 
2.0]], [[2.0, 1.0, 4.0], [2.0, 1.0, 8.0], [6.0, 1.0, 6.0]]], [[[5.0, 1.0, 7.0], [6.0, 8.0, 2.0], [1.0, 2.0, 1.0]], [[4.0, 5.0, 
4.0], [4.0, 8.0, 2.0], [4.0, 2.0, 6.0]], [[2.0, 4.0, 4.0], [4.0, 6.0, 8.0], [6.0, 3.0, 6.0]]], [[[1.0, 7.0, 8.0], [5.0, 6.0, 
2.0], [5.0, 5.0, 2.0]], [[3.0, 3.0, 6.0], [1.0, 3.0, 7.0], [8.0, 1.0, 1.0]], [[2.0, 7.0, 4.0], [7.0, 5.0, 3.0], [4.0, 5.0, 
1.0]]], [[[1.0, 4.0, 2.0], [6.0, 4.0, 4.0], [1.0, 7.0, 7.0]], [[4.0, 7.0, 7.0], [6.0, 5.0, 2.0], [6.0, 1.0, 2.0]], [[2.0, 1.0, 
8.0], [1.0, 5.0, 8.0], [2.0, 4.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 5.0], [1.0, 7.0]], [[5.0, 4.0], [5.0, 5.0]], [[4.0, 4.0], [5.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 6.0], [4.0, 3.0]], [[5.0, 6.0], [2.0, 5.0]], [[4.0, 8.0], [3.0, 7.0]]]))
      arg[:3,:2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 1.0], [1.0, 5.0, 2.0], [6.0, 6.0, 8.0]], [[6.0, 1.0, 1.0], [8.0, 7.0, 6.0], [4.0, 
2.0, 2.0]], [[2.0, 1.0, 4.0], [2.0, 1.0, 8.0], [6.0, 1.0, 6.0]]], [[[5.0, 5.0, 7.0], [6.0, 4.0, 2.0], [1.0, 2.0, 1.0]], [[4.0, 
5.0, 4.0], [4.0, 5.0, 2.0], [4.0, 2.0, 6.0]], [[2.0, 4.0, 4.0], [4.0, 6.0, 8.0], [6.0, 3.0, 6.0]]], [[[1.0, 4.0, 8.0], [5.0, 
4.0, 2.0], [5.0, 5.0, 2.0]], [[3.0, 5.0, 6.0], [1.0, 2.0, 7.0], [8.0, 1.0, 1.0]], [[2.0, 7.0, 4.0], [7.0, 5.0, 3.0], [4.0, 5.0, 
1.0]]], [[[1.0, 4.0, 2.0], [6.0, 4.0, 4.0], [1.0, 7.0, 7.0]], [[4.0, 7.0, 7.0], [6.0, 5.0, 2.0], [6.0, 1.0, 2.0]], [[2.0, 1.0, 
8.0], [1.0, 5.0, 8.0], [2.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 1.0], [1.0, 6.0, 2.0], [6.0, 6.0, 8.0]], 
[[6.0, 4.0, 1.0], [8.0, 3.0, 6.0], [4.0, 2.0, 2.0]], [[2.0, 1.0, 4.0], [2.0, 1.0, 8.0], [6.0, 1.0, 6.0]]], [[[5.0, 5.0, 7.0], 
[6.0, 6.0, 2.0], [1.0, 2.0, 1.0]], [[4.0, 2.0, 4.0], [4.0, 5.0, 2.0], [4.0, 2.0, 6.0]], [[2.0, 4.0, 4.0], [4.0, 6.0, 8.0], 
[6.0, 3.0, 6.0]]], [[[1.0, 4.0, 8.0], [5.0, 8.0, 2.0], [5.0, 5.0, 2.0]], [[3.0, 3.0, 6.0], [1.0, 7.0, 7.0], [8.0, 1.0, 1.0]], 
[[2.0, 7.0, 4.0], [7.0, 5.0, 3.0], [4.0, 5.0, 1.0]]], [[[1.0, 4.0, 2.0], [6.0, 4.0, 4.0], [1.0, 7.0, 7.0]], [[4.0, 7.0, 7.0], 
[6.0, 5.0, 2.0], [6.0, 1.0, 2.0]], [[2.0, 1.0, 8.0], [1.0, 5.0, 8.0], [2.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLPN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 7.0], [2.0, 3.0, 5.0], [2.0, 5.0, 2.0]], [[2.0, 8.0, 7.0], [8.0, 7.0, 1.0], [2.0, 3.0, 
2.0]], [[7.0, 7.0, 2.0], [6.0, 8.0, 4.0], [8.0, 2.0, 4.0]]], [[[4.0, 5.0, 8.0], [2.0, 1.0, 5.0], [4.0, 5.0, 4.0]], [[7.0, 6.0, 
3.0], [4.0, 1.0, 2.0], [3.0, 6.0, 7.0]], [[3.0, 7.0, 8.0], [8.0, 5.0, 1.0], [6.0, 6.0, 8.0]]], [[[3.0, 5.0, 2.0], [5.0, 1.0, 
1.0], [6.0, 4.0, 3.0]], [[5.0, 4.0, 1.0], [2.0, 3.0, 7.0], [5.0, 2.0, 6.0]], [[5.0, 1.0, 8.0], [5.0, 6.0, 5.0], [6.0, 6.0, 
3.0]]], [[[4.0, 8.0, 5.0], [7.0, 7.0, 3.0], [6.0, 1.0, 3.0]], [[5.0, 8.0, 1.0], [6.0, 3.0, 8.0], [2.0, 6.0, 4.0]], [[6.0, 8.0, 
2.0], [5.0, 4.0, 5.0], [1.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 4.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 7.0, 7.0]]))
      arg[2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 7.0], [2.0, 3.0, 5.0], [2.0, 5.0, 2.0]], [[2.0, 8.0, 7.0], [8.0, 7.0, 1.0], [2.0, 
3.0, 2.0]], [[7.0, 7.0, 2.0], [6.0, 8.0, 4.0], [8.0, 2.0, 4.0]]], [[[4.0, 5.0, 8.0], [2.0, 1.0, 5.0], [4.0, 5.0, 4.0]], [[7.0, 
6.0, 3.0], [4.0, 1.0, 2.0], [3.0, 6.0, 7.0]], [[3.0, 7.0, 8.0], [8.0, 5.0, 1.0], [6.0, 6.0, 8.0]]], [[[3.0, 5.0, 2.0], [5.0, 
1.0, 1.0], [6.0, 4.0, 3.0]], [[5.0, 4.0, 1.0], [2.0, 3.0, 7.0], [5.0, 2.0, 6.0]], [[5.0, 1.0, 8.0], [1.0, 4.0, 2.0], [6.0, 6.0, 
3.0]]], [[[4.0, 8.0, 5.0], [7.0, 7.0, 3.0], [6.0, 1.0, 3.0]], [[5.0, 8.0, 1.0], [6.0, 3.0, 8.0], [2.0, 6.0, 4.0]], [[6.0, 8.0, 
2.0], [5.0, 4.0, 5.0], [1.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 7.0], [2.0, 3.0, 5.0], [2.0, 5.0, 2.0]], 
[[2.0, 8.0, 7.0], [8.0, 7.0, 1.0], [2.0, 3.0, 2.0]], [[7.0, 7.0, 2.0], [6.0, 8.0, 4.0], [8.0, 2.0, 4.0]]], [[[4.0, 5.0, 8.0], 
[2.0, 1.0, 5.0], [4.0, 5.0, 4.0]], [[7.0, 6.0, 3.0], [4.0, 1.0, 2.0], [3.0, 6.0, 7.0]], [[3.0, 7.0, 8.0], [8.0, 5.0, 1.0], 
[6.0, 6.0, 8.0]]], [[[3.0, 5.0, 2.0], [5.0, 1.0, 1.0], [6.0, 4.0, 3.0]], [[5.0, 4.0, 1.0], [2.0, 3.0, 7.0], [5.0, 2.0, 6.0]], 
[[5.0, 1.0, 8.0], [5.0, 7.0, 7.0], [6.0, 6.0, 3.0]]], [[[4.0, 8.0, 5.0], [7.0, 7.0, 3.0], [6.0, 1.0, 3.0]], [[5.0, 8.0, 1.0], 
[6.0, 3.0, 8.0], [2.0, 6.0, 4.0]], [[6.0, 8.0, 2.0], [5.0, 4.0, 5.0], [1.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LCCN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 1.0], [4.0, 8.0, 3.0], [4.0, 4.0, 1.0]], [[3.0, 5.0, 2.0], [6.0, 2.0, 7.0], [5.0, 6.0, 
1.0]], [[8.0, 8.0, 8.0], [4.0, 4.0, 4.0], [6.0, 8.0, 1.0]]], [[[1.0, 2.0, 4.0], [6.0, 1.0, 8.0], [5.0, 2.0, 7.0]], [[8.0, 7.0, 
6.0], [7.0, 4.0, 3.0], [6.0, 8.0, 1.0]], [[3.0, 5.0, 4.0], [2.0, 7.0, 7.0], [2.0, 4.0, 5.0]]], [[[8.0, 3.0, 3.0], [1.0, 3.0, 
2.0], [2.0, 2.0, 6.0]], [[6.0, 2.0, 1.0], [4.0, 6.0, 2.0], [2.0, 4.0, 3.0]], [[8.0, 5.0, 6.0], [1.0, 7.0, 4.0], [3.0, 8.0, 
6.0]]], [[[5.0, 8.0, 7.0], [7.0, 2.0, 1.0], [3.0, 5.0, 4.0]], [[1.0, 5.0, 1.0], [6.0, 1.0, 4.0], [1.0, 2.0, 2.0]], [[4.0, 5.0, 
1.0], [2.0, 2.0, 8.0], [4.0, 7.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 7.0, 1.0], [6.0, 7.0, 7.0], [7.0, 2.0, 1.0]], [[2.0, 5.0, 8.0], [4.0, 8.0, 7.0], [4.0, 3.0, 
6.0]], [[8.0, 3.0, 8.0], [5.0, 6.0, 6.0], [1.0, 1.0, 8.0]]], [[[4.0, 6.0, 2.0], [6.0, 8.0, 5.0], [3.0, 8.0, 2.0]], [[4.0, 7.0, 
2.0], [7.0, 8.0, 2.0], [4.0, 3.0, 6.0]], [[3.0, 5.0, 6.0], [5.0, 8.0, 5.0], [5.0, 3.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 6.0], [1.0, 7.0, 7.0], [4.0, 5.0, 1.0]], [[5.0, 1.0, 5.0], [2.0, 2.0, 
7.0], [3.0, 5.0, 8.0]], [[5.0, 5.0, 1.0], [3.0, 7.0, 3.0], [2.0, 4.0, 8.0]]], [[[7.0, 8.0, 4.0], [1.0, 3.0, 3.0], [4.0, 6.0, 
6.0]], [[1.0, 2.0, 6.0], [7.0, 1.0, 5.0], [3.0, 1.0, 5.0]], [[8.0, 3.0, 6.0], [6.0, 8.0, 7.0], [6.0, 3.0, 1.0]]]]))
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 1.0], [4.0, 8.0, 3.0], [4.0, 4.0, 1.0]], [[3.0, 5.0, 2.0], [6.0, 2.0, 7.0], [5.0, 
6.0, 1.0]], [[8.0, 8.0, 8.0], [4.0, 4.0, 4.0], [6.0, 8.0, 1.0]]], [[[1.0, 2.0, 4.0], [6.0, 1.0, 8.0], [5.0, 2.0, 7.0]], [[8.0, 
7.0, 6.0], [7.0, 4.0, 3.0], [6.0, 8.0, 1.0]], [[3.0, 5.0, 4.0], [2.0, 7.0, 7.0], [2.0, 4.0, 5.0]]], [[[6.0, 7.0, 1.0], [6.0, 
7.0, 7.0], [7.0, 2.0, 1.0]], [[2.0, 5.0, 8.0], [4.0, 8.0, 7.0], [4.0, 3.0, 6.0]], [[8.0, 3.0, 8.0], [5.0, 6.0, 6.0], [1.0, 1.0, 
8.0]]], [[[4.0, 6.0, 2.0], [6.0, 8.0, 5.0], [3.0, 8.0, 2.0]], [[4.0, 7.0, 2.0], [7.0, 8.0, 2.0], [4.0, 3.0, 6.0]], [[3.0, 5.0, 
6.0], [5.0, 8.0, 5.0], [5.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 1.0], [4.0, 8.0, 3.0], [4.0, 4.0, 1.0]], 
[[3.0, 5.0, 2.0], [6.0, 2.0, 7.0], [5.0, 6.0, 1.0]], [[8.0, 8.0, 8.0], [4.0, 4.0, 4.0], [6.0, 8.0, 1.0]]], [[[1.0, 2.0, 4.0], 
[6.0, 1.0, 8.0], [5.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], [7.0, 4.0, 3.0], [6.0, 8.0, 1.0]], [[3.0, 5.0, 4.0], [2.0, 7.0, 7.0], 
[2.0, 4.0, 5.0]]], [[[4.0, 2.0, 6.0], [1.0, 7.0, 7.0], [4.0, 5.0, 1.0]], [[5.0, 1.0, 5.0], [2.0, 2.0, 7.0], [3.0, 5.0, 8.0]], 
[[5.0, 5.0, 1.0], [3.0, 7.0, 3.0], [2.0, 4.0, 8.0]]], [[[7.0, 8.0, 4.0], [1.0, 3.0, 3.0], [4.0, 6.0, 6.0]], [[1.0, 2.0, 6.0], 
[7.0, 1.0, 5.0], [3.0, 1.0, 5.0]], [[8.0, 3.0, 6.0], [6.0, 8.0, 7.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULRN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 5.0], [2.0, 2.0, 2.0], [5.0, 7.0, 7.0]], [[1.0, 6.0, 5.0], [3.0, 7.0, 2.0], [4.0, 4.0, 
4.0]], [[8.0, 7.0, 7.0], [7.0, 1.0, 4.0], [4.0, 3.0, 2.0]]], [[[5.0, 8.0, 2.0], [8.0, 8.0, 3.0], [7.0, 5.0, 8.0]], [[1.0, 1.0, 
5.0], [8.0, 2.0, 4.0], [3.0, 5.0, 8.0]], [[8.0, 2.0, 2.0], [5.0, 4.0, 4.0], [7.0, 6.0, 8.0]]], [[[2.0, 5.0, 1.0], [6.0, 2.0, 
4.0], [7.0, 7.0, 5.0]], [[6.0, 4.0, 5.0], [2.0, 5.0, 2.0], [3.0, 4.0, 4.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 3.0], [6.0, 8.0, 
6.0]]], [[[1.0, 4.0, 1.0], [8.0, 3.0, 6.0], [3.0, 7.0, 5.0]], [[6.0, 6.0, 2.0], [8.0, 6.0, 2.0], [8.0, 2.0, 1.0]], [[7.0, 5.0, 
4.0], [4.0, 3.0, 6.0], [2.0, 2.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 5.0, 5.0]]], [[[5.0, 4.0, 2.0]]], [[[1.0, 4.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 2.0]]], [[[6.0, 7.0, 5.0]]], [[[1.0, 7.0, 6.0]]]]))
      arg[:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 5.0], [2.0, 2.0, 2.0], [5.0, 7.0, 7.0]], [[1.0, 6.0, 5.0], [3.0, 7.0, 2.0], [4.0, 
4.0, 4.0]], [[8.0, 7.0, 7.0], [4.0, 5.0, 5.0], [4.0, 3.0, 2.0]]], [[[5.0, 8.0, 2.0], [8.0, 8.0, 3.0], [7.0, 5.0, 8.0]], [[1.0, 
1.0, 5.0], [8.0, 2.0, 4.0], [3.0, 5.0, 8.0]], [[8.0, 2.0, 2.0], [5.0, 4.0, 2.0], [7.0, 6.0, 8.0]]], [[[2.0, 5.0, 1.0], [6.0, 
2.0, 4.0], [7.0, 7.0, 5.0]], [[6.0, 4.0, 5.0], [2.0, 5.0, 2.0], [3.0, 4.0, 4.0]], [[6.0, 8.0, 6.0], [1.0, 4.0, 4.0], [6.0, 8.0, 
6.0]]], [[[1.0, 4.0, 1.0], [8.0, 3.0, 6.0], [3.0, 7.0, 5.0]], [[6.0, 6.0, 2.0], [8.0, 6.0, 2.0], [8.0, 2.0, 1.0]], [[7.0, 5.0, 
4.0], [4.0, 3.0, 6.0], [2.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 5.0], [2.0, 2.0, 2.0], [5.0, 7.0, 7.0]], 
[[1.0, 6.0, 5.0], [3.0, 7.0, 2.0], [4.0, 4.0, 4.0]], [[8.0, 7.0, 7.0], [3.0, 2.0, 2.0], [4.0, 3.0, 2.0]]], [[[5.0, 8.0, 2.0], 
[8.0, 8.0, 3.0], [7.0, 5.0, 8.0]], [[1.0, 1.0, 5.0], [8.0, 2.0, 4.0], [3.0, 5.0, 8.0]], [[8.0, 2.0, 2.0], [6.0, 7.0, 5.0], 
[7.0, 6.0, 8.0]]], [[[2.0, 5.0, 1.0], [6.0, 2.0, 4.0], [7.0, 7.0, 5.0]], [[6.0, 4.0, 5.0], [2.0, 5.0, 2.0], [3.0, 4.0, 4.0]], 
[[6.0, 8.0, 6.0], [1.0, 7.0, 6.0], [6.0, 8.0, 6.0]]], [[[1.0, 4.0, 1.0], [8.0, 3.0, 6.0], [3.0, 7.0, 5.0]], [[6.0, 6.0, 2.0], 
[8.0, 6.0, 2.0], [8.0, 2.0, 1.0]], [[7.0, 5.0, 4.0], [4.0, 3.0, 6.0], [2.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UUPN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 6.0], [7.0, 5.0, 2.0], [7.0, 8.0, 3.0]], [[2.0, 3.0, 3.0], [1.0, 7.0, 7.0], [5.0, 3.0, 
6.0]], [[5.0, 4.0, 5.0], [7.0, 7.0, 6.0], [5.0, 7.0, 3.0]]], [[[3.0, 3.0, 6.0], [3.0, 8.0, 4.0], [6.0, 7.0, 4.0]], [[7.0, 5.0, 
7.0], [2.0, 6.0, 4.0], [5.0, 3.0, 6.0]], [[8.0, 7.0, 2.0], [2.0, 6.0, 2.0], [3.0, 2.0, 2.0]]], [[[7.0, 2.0, 5.0], [7.0, 1.0, 
2.0], [3.0, 6.0, 4.0]], [[7.0, 8.0, 8.0], [8.0, 1.0, 6.0], [2.0, 2.0, 3.0]], [[1.0, 8.0, 5.0], [1.0, 2.0, 1.0], [2.0, 6.0, 
3.0]]], [[[6.0, 2.0, 7.0], [6.0, 4.0, 8.0], [1.0, 2.0, 3.0]], [[1.0, 7.0, 1.0], [4.0, 1.0, 8.0], [3.0, 5.0, 7.0]], [[3.0, 5.0, 
7.0], [1.0, 5.0, 7.0], [4.0, 2.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 7.0, 8.0], [2.0, 2.0, 3.0]], [[5.0, 8.0, 5.0], [3.0, 8.0, 3.0]], [[5.0, 1.0, 4.0], [1.0, 
5.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 1.0, 2.0], [2.0, 8.0, 7.0]], [[2.0, 3.0, 5.0], [7.0, 1.0, 6.0]], [[5.0, 2.0, 
1.0], [5.0, 2.0, 3.0]]]))
      arg[:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 6.0], [2.0, 7.0, 8.0], [7.0, 8.0, 3.0]], [[2.0, 3.0, 3.0], [2.0, 2.0, 3.0], [5.0, 
3.0, 6.0]], [[5.0, 4.0, 5.0], [7.0, 7.0, 6.0], [5.0, 7.0, 3.0]]], [[[3.0, 3.0, 6.0], [5.0, 8.0, 5.0], [6.0, 7.0, 4.0]], [[7.0, 
5.0, 7.0], [3.0, 8.0, 3.0], [5.0, 3.0, 6.0]], [[8.0, 7.0, 2.0], [2.0, 6.0, 2.0], [3.0, 2.0, 2.0]]], [[[7.0, 2.0, 5.0], [5.0, 
1.0, 4.0], [3.0, 6.0, 4.0]], [[7.0, 8.0, 8.0], [1.0, 5.0, 7.0], [2.0, 2.0, 3.0]], [[1.0, 8.0, 5.0], [1.0, 2.0, 1.0], [2.0, 6.0, 
3.0]]], [[[6.0, 2.0, 7.0], [6.0, 4.0, 8.0], [1.0, 2.0, 3.0]], [[1.0, 7.0, 1.0], [4.0, 1.0, 8.0], [3.0, 5.0, 7.0]], [[3.0, 5.0, 
7.0], [1.0, 5.0, 7.0], [4.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 6.0], [4.0, 1.0, 2.0], [7.0, 8.0, 3.0]], 
[[2.0, 3.0, 3.0], [2.0, 8.0, 7.0], [5.0, 3.0, 6.0]], [[5.0, 4.0, 5.0], [7.0, 7.0, 6.0], [5.0, 7.0, 3.0]]], [[[3.0, 3.0, 6.0], 
[2.0, 3.0, 5.0], [6.0, 7.0, 4.0]], [[7.0, 5.0, 7.0], [7.0, 1.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 7.0, 2.0], [2.0, 6.0, 2.0], 
[3.0, 2.0, 2.0]]], [[[7.0, 2.0, 5.0], [5.0, 2.0, 1.0], [3.0, 6.0, 4.0]], [[7.0, 8.0, 8.0], [5.0, 2.0, 3.0], [2.0, 2.0, 3.0]], 
[[1.0, 8.0, 5.0], [1.0, 2.0, 1.0], [2.0, 6.0, 3.0]]], [[[6.0, 2.0, 7.0], [6.0, 4.0, 8.0], [1.0, 2.0, 3.0]], [[1.0, 7.0, 1.0], 
[4.0, 1.0, 8.0], [3.0, 5.0, 7.0]], [[3.0, 5.0, 7.0], [1.0, 5.0, 7.0], [4.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RCRN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 2.0], [5.0, 2.0, 8.0], [6.0, 5.0, 2.0]], [[3.0, 5.0, 3.0], [6.0, 6.0, 4.0], [3.0, 6.0, 
3.0]], [[1.0, 4.0, 8.0], [5.0, 2.0, 8.0], [8.0, 7.0, 8.0]]], [[[6.0, 3.0, 7.0], [2.0, 3.0, 5.0], [8.0, 2.0, 7.0]], [[1.0, 6.0, 
3.0], [7.0, 7.0, 4.0], [5.0, 2.0, 2.0]], [[5.0, 1.0, 4.0], [2.0, 3.0, 6.0], [8.0, 4.0, 4.0]]], [[[6.0, 6.0, 4.0], [7.0, 8.0, 
3.0], [6.0, 1.0, 1.0]], [[4.0, 6.0, 3.0], [7.0, 3.0, 3.0], [8.0, 7.0, 4.0]], [[1.0, 2.0, 3.0], [6.0, 2.0, 3.0], [4.0, 8.0, 
1.0]]], [[[8.0, 3.0, 3.0], [1.0, 2.0, 7.0], [1.0, 1.0, 3.0]], [[4.0, 2.0, 4.0], [4.0, 5.0, 8.0], [5.0, 6.0, 1.0]], [[1.0, 8.0, 
7.0], [2.0, 8.0, 7.0], [1.0, 8.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 7.0, 3.0]], [[8.0, 2.0, 3.0]], [[3.0, 7.0, 6.0]]], [[[2.0, 6.0, 7.0]], [[3.0, 7.0, 6.0]], 
[[3.0, 7.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 5.0]], [[1.0, 7.0, 2.0]], [[5.0, 3.0, 4.0]]], [[[6.0, 8.0, 1.0]], [[6.0, 
8.0, 1.0]], [[7.0, 8.0, 3.0]]]]))
      arg[1:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 2.0], [5.0, 2.0, 8.0], [6.0, 5.0, 2.0]], [[3.0, 5.0, 3.0], [6.0, 6.0, 4.0], [3.0, 
6.0, 3.0]], [[1.0, 4.0, 8.0], [5.0, 2.0, 8.0], [8.0, 7.0, 8.0]]], [[[6.0, 3.0, 7.0], [3.0, 7.0, 3.0], [8.0, 2.0, 7.0]], [[1.0, 
6.0, 3.0], [8.0, 2.0, 3.0], [5.0, 2.0, 2.0]], [[5.0, 1.0, 4.0], [3.0, 7.0, 6.0], [8.0, 4.0, 4.0]]], [[[6.0, 6.0, 4.0], [2.0, 
6.0, 7.0], [6.0, 1.0, 1.0]], [[4.0, 6.0, 3.0], [3.0, 7.0, 6.0], [8.0, 7.0, 4.0]], [[1.0, 2.0, 3.0], [3.0, 7.0, 7.0], [4.0, 8.0, 
1.0]]], [[[8.0, 3.0, 3.0], [1.0, 2.0, 7.0], [1.0, 1.0, 3.0]], [[4.0, 2.0, 4.0], [4.0, 5.0, 8.0], [5.0, 6.0, 1.0]], [[1.0, 8.0, 
7.0], [2.0, 8.0, 7.0], [1.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 2.0], [5.0, 2.0, 8.0], [6.0, 5.0, 2.0]], 
[[3.0, 5.0, 3.0], [6.0, 6.0, 4.0], [3.0, 6.0, 3.0]], [[1.0, 4.0, 8.0], [5.0, 2.0, 8.0], [8.0, 7.0, 8.0]]], [[[6.0, 3.0, 7.0], 
[8.0, 4.0, 5.0], [8.0, 2.0, 7.0]], [[1.0, 6.0, 3.0], [1.0, 7.0, 2.0], [5.0, 2.0, 2.0]], [[5.0, 1.0, 4.0], [5.0, 3.0, 4.0], 
[8.0, 4.0, 4.0]]], [[[6.0, 6.0, 4.0], [6.0, 8.0, 1.0], [6.0, 1.0, 1.0]], [[4.0, 6.0, 3.0], [6.0, 8.0, 1.0], [8.0, 7.0, 4.0]], 
[[1.0, 2.0, 3.0], [7.0, 8.0, 3.0], [4.0, 8.0, 1.0]]], [[[8.0, 3.0, 3.0], [1.0, 2.0, 7.0], [1.0, 1.0, 3.0]], [[4.0, 2.0, 4.0], 
[4.0, 5.0, 8.0], [5.0, 6.0, 1.0]], [[1.0, 8.0, 7.0], [2.0, 8.0, 7.0], [1.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PUUN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 7.0], [7.0, 8.0, 1.0], [7.0, 7.0, 2.0]], [[3.0, 2.0, 5.0], [2.0, 3.0, 1.0], [6.0, 6.0, 
7.0]], [[8.0, 5.0, 6.0], [3.0, 7.0, 5.0], [4.0, 8.0, 5.0]]], [[[8.0, 6.0, 3.0], [2.0, 2.0, 1.0], [5.0, 6.0, 7.0]], [[5.0, 4.0, 
7.0], [4.0, 1.0, 1.0], [4.0, 8.0, 4.0]], [[2.0, 2.0, 5.0], [4.0, 3.0, 5.0], [4.0, 5.0, 1.0]]], [[[1.0, 2.0, 1.0], [7.0, 5.0, 
6.0], [7.0, 8.0, 8.0]], [[7.0, 3.0, 3.0], [2.0, 2.0, 8.0], [4.0, 8.0, 8.0]], [[5.0, 2.0, 3.0], [8.0, 1.0, 5.0], [2.0, 3.0, 
5.0]]], [[[5.0, 4.0, 8.0], [4.0, 7.0, 4.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 3.0], [5.0, 4.0, 2.0], [2.0, 2.0, 6.0]], [[6.0, 2.0, 
2.0], [7.0, 6.0, 7.0], [5.0, 6.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 5.0, 4.0], [6.0, 6.0, 5.0]], [[4.0, 2.0, 5.0], [7.0, 1.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 8.0, 6.0], [5.0, 7.0, 6.0]], [[5.0, 7.0, 7.0], [3.0, 4.0, 7.0]]]))
      arg[2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 7.0], [7.0, 8.0, 1.0], [7.0, 7.0, 2.0]], [[3.0, 2.0, 5.0], [2.0, 3.0, 1.0], [6.0, 
6.0, 7.0]], [[8.0, 5.0, 6.0], [3.0, 7.0, 5.0], [4.0, 8.0, 5.0]]], [[[8.0, 6.0, 3.0], [2.0, 2.0, 1.0], [5.0, 6.0, 7.0]], [[5.0, 
4.0, 7.0], [4.0, 1.0, 1.0], [4.0, 8.0, 4.0]], [[2.0, 2.0, 5.0], [4.0, 3.0, 5.0], [4.0, 5.0, 1.0]]], [[[8.0, 5.0, 4.0], [6.0, 
6.0, 5.0], [7.0, 8.0, 8.0]], [[4.0, 2.0, 5.0], [7.0, 1.0, 4.0], [4.0, 8.0, 8.0]], [[5.0, 2.0, 3.0], [8.0, 1.0, 5.0], [2.0, 3.0, 
5.0]]], [[[5.0, 4.0, 8.0], [4.0, 7.0, 4.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 3.0], [5.0, 4.0, 2.0], [2.0, 2.0, 6.0]], [[6.0, 2.0, 
2.0], [7.0, 6.0, 7.0], [5.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 7.0], [7.0, 8.0, 1.0], [7.0, 7.0, 2.0]], 
[[3.0, 2.0, 5.0], [2.0, 3.0, 1.0], [6.0, 6.0, 7.0]], [[8.0, 5.0, 6.0], [3.0, 7.0, 5.0], [4.0, 8.0, 5.0]]], [[[8.0, 6.0, 3.0], 
[2.0, 2.0, 1.0], [5.0, 6.0, 7.0]], [[5.0, 4.0, 7.0], [4.0, 1.0, 1.0], [4.0, 8.0, 4.0]], [[2.0, 2.0, 5.0], [4.0, 3.0, 5.0], 
[4.0, 5.0, 1.0]]], [[[5.0, 8.0, 6.0], [5.0, 7.0, 6.0], [7.0, 8.0, 8.0]], [[5.0, 7.0, 7.0], [3.0, 4.0, 7.0], [4.0, 8.0, 8.0]], 
[[5.0, 2.0, 3.0], [8.0, 1.0, 5.0], [2.0, 3.0, 5.0]]], [[[5.0, 4.0, 8.0], [4.0, 7.0, 4.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 3.0], 
[5.0, 4.0, 2.0], [2.0, 2.0, 6.0]], [[6.0, 2.0, 2.0], [7.0, 6.0, 7.0], [5.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RRRR(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 2.0], [5.0, 4.0, 7.0], [5.0, 4.0, 4.0]], [[3.0, 8.0, 4.0], [6.0, 3.0, 3.0], [6.0, 4.0, 
3.0]], [[3.0, 3.0, 8.0], [2.0, 3.0, 5.0], [7.0, 3.0, 4.0]]], [[[8.0, 5.0, 8.0], [7.0, 2.0, 6.0], [5.0, 6.0, 5.0]], [[4.0, 4.0, 
6.0], [4.0, 3.0, 1.0], [7.0, 1.0, 3.0]], [[4.0, 7.0, 6.0], [3.0, 2.0, 1.0], [1.0, 8.0, 3.0]]], [[[8.0, 3.0, 2.0], [1.0, 1.0, 
2.0], [5.0, 6.0, 1.0]], [[3.0, 5.0, 3.0], [3.0, 5.0, 3.0], [5.0, 2.0, 4.0]], [[6.0, 2.0, 8.0], [5.0, 6.0, 7.0], [5.0, 8.0, 
8.0]]], [[[2.0, 6.0, 3.0], [4.0, 7.0, 5.0], [3.0, 7.0, 3.0]], [[8.0, 4.0, 7.0], [4.0, 5.0, 1.0], [5.0, 1.0, 6.0]], [[3.0, 4.0, 
2.0], [4.0, 6.0, 5.0], [6.0, 3.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0]]], [[[5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0]]], [[[2.0]]]]))
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 2.0], [5.0, 4.0, 7.0], [5.0, 4.0, 4.0]], [[3.0, 8.0, 4.0], [6.0, 3.0, 3.0], [6.0, 
4.0, 3.0]], [[3.0, 3.0, 8.0], [2.0, 3.0, 5.0], [7.0, 3.0, 4.0]]], [[[8.0, 5.0, 8.0], [7.0, 2.0, 6.0], [5.0, 6.0, 5.0]], [[4.0, 
4.0, 6.0], [4.0, 6.0, 1.0], [7.0, 1.0, 3.0]], [[4.0, 7.0, 6.0], [3.0, 2.0, 1.0], [1.0, 8.0, 3.0]]], [[[8.0, 3.0, 2.0], [1.0, 
1.0, 2.0], [5.0, 6.0, 1.0]], [[3.0, 5.0, 3.0], [3.0, 5.0, 3.0], [5.0, 2.0, 4.0]], [[6.0, 2.0, 8.0], [5.0, 6.0, 7.0], [5.0, 8.0, 
8.0]]], [[[2.0, 6.0, 3.0], [4.0, 7.0, 5.0], [3.0, 7.0, 3.0]], [[8.0, 4.0, 7.0], [4.0, 5.0, 1.0], [5.0, 1.0, 6.0]], [[3.0, 4.0, 
2.0], [4.0, 6.0, 5.0], [6.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 2.0], [5.0, 4.0, 7.0], [5.0, 4.0, 4.0]], 
[[3.0, 8.0, 4.0], [6.0, 3.0, 3.0], [6.0, 4.0, 3.0]], [[3.0, 3.0, 8.0], [2.0, 3.0, 5.0], [7.0, 3.0, 4.0]]], [[[8.0, 5.0, 8.0], 
[7.0, 2.0, 6.0], [5.0, 6.0, 5.0]], [[4.0, 4.0, 6.0], [4.0, 4.0, 1.0], [7.0, 1.0, 3.0]], [[4.0, 7.0, 6.0], [3.0, 2.0, 1.0], 
[1.0, 8.0, 3.0]]], [[[8.0, 3.0, 2.0], [1.0, 1.0, 2.0], [5.0, 6.0, 1.0]], [[3.0, 5.0, 3.0], [3.0, 2.0, 3.0], [5.0, 2.0, 4.0]], 
[[6.0, 2.0, 8.0], [5.0, 6.0, 7.0], [5.0, 8.0, 8.0]]], [[[2.0, 6.0, 3.0], [4.0, 7.0, 5.0], [3.0, 7.0, 3.0]], [[8.0, 4.0, 7.0], 
[4.0, 5.0, 1.0], [5.0, 1.0, 6.0]], [[3.0, 4.0, 2.0], [4.0, 6.0, 5.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LLUN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 5.0], [7.0, 4.0, 6.0], [4.0, 1.0, 8.0]], [[7.0, 4.0, 5.0], [7.0, 1.0, 3.0], [5.0, 4.0, 
1.0]], [[8.0, 1.0, 5.0], [3.0, 6.0, 2.0], [1.0, 5.0, 7.0]]], [[[3.0, 5.0, 5.0], [5.0, 5.0, 7.0], [4.0, 7.0, 5.0]], [[4.0, 7.0, 
5.0], [1.0, 3.0, 1.0], [1.0, 7.0, 4.0]], [[5.0, 7.0, 1.0], [1.0, 6.0, 1.0], [2.0, 2.0, 5.0]]], [[[3.0, 3.0, 3.0], [3.0, 3.0, 
8.0], [5.0, 8.0, 4.0]], [[1.0, 2.0, 1.0], [8.0, 2.0, 1.0], [5.0, 2.0, 8.0]], [[2.0, 4.0, 8.0], [1.0, 4.0, 2.0], [6.0, 2.0, 
7.0]]], [[[5.0, 1.0, 7.0], [7.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[2.0, 7.0, 3.0], [8.0, 7.0, 5.0], [6.0, 4.0, 3.0]], [[2.0, 3.0, 
7.0], [3.0, 2.0, 5.0], [3.0, 6.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 6.0, 7.0], [5.0, 4.0, 8.0]]], [[[5.0, 6.0, 3.0], [7.0, 2.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 7.0], [6.0, 8.0, 7.0]]], [[[3.0, 2.0, 6.0], [8.0, 4.0, 6.0]]]]))
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 5.0], [7.0, 4.0, 6.0], [4.0, 1.0, 8.0]], [[7.0, 4.0, 5.0], [7.0, 1.0, 3.0], [5.0, 
4.0, 1.0]], [[8.0, 1.0, 5.0], [3.0, 6.0, 2.0], [1.0, 5.0, 7.0]]], [[[3.0, 5.0, 5.0], [5.0, 5.0, 7.0], [4.0, 7.0, 5.0]], [[4.0, 
7.0, 5.0], [1.0, 3.0, 1.0], [1.0, 7.0, 4.0]], [[5.0, 7.0, 1.0], [1.0, 6.0, 1.0], [2.0, 2.0, 5.0]]], [[[3.0, 3.0, 3.0], [3.0, 
3.0, 8.0], [5.0, 8.0, 4.0]], [[1.0, 2.0, 1.0], [8.0, 2.0, 1.0], [5.0, 2.0, 8.0]], [[6.0, 6.0, 7.0], [5.0, 4.0, 8.0], [6.0, 2.0, 
7.0]]], [[[5.0, 1.0, 7.0], [7.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[2.0, 7.0, 3.0], [8.0, 7.0, 5.0], [6.0, 4.0, 3.0]], [[5.0, 6.0, 
3.0], [7.0, 2.0, 5.0], [3.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 5.0], [7.0, 4.0, 6.0], [4.0, 1.0, 8.0]], 
[[7.0, 4.0, 5.0], [7.0, 1.0, 3.0], [5.0, 4.0, 1.0]], [[8.0, 1.0, 5.0], [3.0, 6.0, 2.0], [1.0, 5.0, 7.0]]], [[[3.0, 5.0, 5.0], 
[5.0, 5.0, 7.0], [4.0, 7.0, 5.0]], [[4.0, 7.0, 5.0], [1.0, 3.0, 1.0], [1.0, 7.0, 4.0]], [[5.0, 7.0, 1.0], [1.0, 6.0, 1.0], 
[2.0, 2.0, 5.0]]], [[[3.0, 3.0, 3.0], [3.0, 3.0, 8.0], [5.0, 8.0, 4.0]], [[1.0, 2.0, 1.0], [8.0, 2.0, 1.0], [5.0, 2.0, 8.0]], 
[[2.0, 4.0, 7.0], [6.0, 8.0, 7.0], [6.0, 2.0, 7.0]]], [[[5.0, 1.0, 7.0], [7.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[2.0, 7.0, 3.0], 
[8.0, 7.0, 5.0], [6.0, 4.0, 3.0]], [[3.0, 2.0, 6.0], [8.0, 4.0, 6.0], [3.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PRPN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 5.0], [2.0, 6.0, 1.0], [1.0, 1.0, 3.0]], [[7.0, 2.0, 6.0], [4.0, 4.0, 8.0], [7.0, 2.0, 
7.0]], [[5.0, 2.0, 8.0], [4.0, 3.0, 5.0], [6.0, 4.0, 8.0]]], [[[8.0, 2.0, 6.0], [5.0, 7.0, 5.0], [5.0, 1.0, 4.0]], [[3.0, 7.0, 
5.0], [6.0, 5.0, 4.0], [2.0, 5.0, 3.0]], [[4.0, 3.0, 6.0], [1.0, 3.0, 1.0], [7.0, 5.0, 1.0]]], [[[5.0, 7.0, 6.0], [5.0, 7.0, 
1.0], [3.0, 8.0, 6.0]], [[2.0, 1.0, 8.0], [6.0, 1.0, 4.0], [7.0, 6.0, 6.0]], [[8.0, 3.0, 8.0], [4.0, 5.0, 6.0], [8.0, 6.0, 
5.0]]], [[[1.0, 1.0, 3.0], [2.0, 6.0, 1.0], [8.0, 4.0, 7.0]], [[5.0, 6.0, 6.0], [2.0, 8.0, 5.0], [1.0, 7.0, 8.0]], [[1.0, 4.0, 
5.0], [3.0, 7.0, 4.0], [1.0, 2.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 4.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 5.0, 5.0]]))
      arg[2,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 5.0], [2.0, 6.0, 1.0], [1.0, 1.0, 3.0]], [[7.0, 2.0, 6.0], [4.0, 4.0, 8.0], [7.0, 
2.0, 7.0]], [[5.0, 2.0, 8.0], [4.0, 3.0, 5.0], [6.0, 4.0, 8.0]]], [[[8.0, 2.0, 6.0], [5.0, 7.0, 5.0], [5.0, 1.0, 4.0]], [[3.0, 
7.0, 5.0], [6.0, 5.0, 4.0], [2.0, 5.0, 3.0]], [[4.0, 3.0, 6.0], [1.0, 3.0, 1.0], [7.0, 5.0, 1.0]]], [[[5.0, 7.0, 6.0], [5.0, 
7.0, 1.0], [3.0, 8.0, 6.0]], [[2.0, 1.0, 8.0], [3.0, 4.0, 1.0], [7.0, 6.0, 6.0]], [[8.0, 3.0, 8.0], [4.0, 5.0, 6.0], [8.0, 6.0, 
5.0]]], [[[1.0, 1.0, 3.0], [2.0, 6.0, 1.0], [8.0, 4.0, 7.0]], [[5.0, 6.0, 6.0], [2.0, 8.0, 5.0], [1.0, 7.0, 8.0]], [[1.0, 4.0, 
5.0], [3.0, 7.0, 4.0], [1.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 5.0], [2.0, 6.0, 1.0], [1.0, 1.0, 3.0]], 
[[7.0, 2.0, 6.0], [4.0, 4.0, 8.0], [7.0, 2.0, 7.0]], [[5.0, 2.0, 8.0], [4.0, 3.0, 5.0], [6.0, 4.0, 8.0]]], [[[8.0, 2.0, 6.0], 
[5.0, 7.0, 5.0], [5.0, 1.0, 4.0]], [[3.0, 7.0, 5.0], [6.0, 5.0, 4.0], [2.0, 5.0, 3.0]], [[4.0, 3.0, 6.0], [1.0, 3.0, 1.0], 
[7.0, 5.0, 1.0]]], [[[5.0, 7.0, 6.0], [5.0, 7.0, 1.0], [3.0, 8.0, 6.0]], [[2.0, 1.0, 8.0], [8.0, 5.0, 5.0], [7.0, 6.0, 6.0]], 
[[8.0, 3.0, 8.0], [4.0, 5.0, 6.0], [8.0, 6.0, 5.0]]], [[[1.0, 1.0, 3.0], [2.0, 6.0, 1.0], [8.0, 4.0, 7.0]], [[5.0, 6.0, 6.0], 
[2.0, 8.0, 5.0], [1.0, 7.0, 8.0]], [[1.0, 4.0, 5.0], [3.0, 7.0, 4.0], [1.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PPLP(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 5.0], [2.0, 3.0, 4.0], [8.0, 4.0, 3.0]], [[4.0, 5.0, 5.0], [4.0, 2.0, 2.0], [7.0, 6.0, 
5.0]], [[6.0, 1.0, 2.0], [4.0, 7.0, 2.0], [3.0, 1.0, 5.0]]], [[[6.0, 2.0, 6.0], [7.0, 3.0, 1.0], [3.0, 6.0, 7.0]], [[7.0, 2.0, 
4.0], [2.0, 1.0, 6.0], [8.0, 7.0, 2.0]], [[5.0, 2.0, 1.0], [2.0, 1.0, 2.0], [6.0, 7.0, 4.0]]], [[[5.0, 3.0, 7.0], [3.0, 8.0, 
8.0], [5.0, 5.0, 1.0]], [[4.0, 5.0, 1.0], [6.0, 4.0, 5.0], [5.0, 7.0, 3.0]], [[6.0, 7.0, 8.0], [5.0, 8.0, 5.0], [7.0, 8.0, 
8.0]]], [[[2.0, 6.0, 5.0], [4.0, 6.0, 3.0], [8.0, 1.0, 7.0]], [[4.0, 4.0, 2.0], [8.0, 1.0, 7.0], [2.0, 8.0, 3.0]], [[2.0, 7.0, 
8.0], [3.0, 2.0, 2.0], [2.0, 4.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([7.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([1.0]))
      arg[2,1,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 5.0], [2.0, 3.0, 4.0], [8.0, 4.0, 3.0]], [[4.0, 5.0, 5.0], [4.0, 2.0, 2.0], [7.0, 
6.0, 5.0]], [[6.0, 1.0, 2.0], [4.0, 7.0, 2.0], [3.0, 1.0, 5.0]]], [[[6.0, 2.0, 6.0], [7.0, 3.0, 1.0], [3.0, 6.0, 7.0]], [[7.0, 
2.0, 4.0], [2.0, 1.0, 6.0], [8.0, 7.0, 2.0]], [[5.0, 2.0, 1.0], [2.0, 1.0, 2.0], [6.0, 7.0, 4.0]]], [[[5.0, 3.0, 7.0], [3.0, 
8.0, 8.0], [5.0, 5.0, 1.0]], [[4.0, 5.0, 1.0], [6.0, 4.0, 5.0], [5.0, 7.0, 3.0]], [[6.0, 7.0, 8.0], [5.0, 8.0, 5.0], [7.0, 8.0, 
8.0]]], [[[2.0, 6.0, 5.0], [4.0, 6.0, 3.0], [8.0, 1.0, 7.0]], [[4.0, 4.0, 2.0], [8.0, 1.0, 7.0], [2.0, 8.0, 3.0]], [[2.0, 7.0, 
8.0], [3.0, 2.0, 2.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 5.0], [2.0, 3.0, 4.0], [8.0, 4.0, 3.0]], 
[[4.0, 5.0, 5.0], [4.0, 2.0, 2.0], [7.0, 6.0, 5.0]], [[6.0, 1.0, 2.0], [4.0, 7.0, 2.0], [3.0, 1.0, 5.0]]], [[[6.0, 2.0, 6.0], 
[7.0, 3.0, 1.0], [3.0, 6.0, 7.0]], [[7.0, 2.0, 4.0], [2.0, 1.0, 6.0], [8.0, 7.0, 2.0]], [[5.0, 2.0, 1.0], [2.0, 1.0, 2.0], 
[6.0, 7.0, 4.0]]], [[[5.0, 3.0, 7.0], [3.0, 8.0, 8.0], [5.0, 5.0, 1.0]], [[4.0, 5.0, 1.0], [6.0, 4.0, 5.0], [5.0, 1.0, 3.0]], 
[[6.0, 7.0, 8.0], [5.0, 8.0, 5.0], [7.0, 8.0, 8.0]]], [[[2.0, 6.0, 5.0], [4.0, 6.0, 3.0], [8.0, 1.0, 7.0]], [[4.0, 4.0, 2.0], 
[8.0, 1.0, 7.0], [2.0, 8.0, 3.0]], [[2.0, 7.0, 8.0], [3.0, 2.0, 2.0], [2.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLUN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 3.0], [5.0, 4.0, 2.0], [1.0, 7.0, 3.0]], [[8.0, 8.0, 4.0], [5.0, 3.0, 8.0], [4.0, 6.0, 
3.0]], [[6.0, 5.0, 3.0], [8.0, 5.0, 4.0], [5.0, 3.0, 3.0]]], [[[4.0, 4.0, 7.0], [2.0, 1.0, 7.0], [8.0, 5.0, 3.0]], [[2.0, 2.0, 
4.0], [5.0, 7.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 2.0, 8.0], [4.0, 2.0, 6.0], [6.0, 4.0, 5.0]]], [[[3.0, 4.0, 8.0], [6.0, 3.0, 
5.0], [2.0, 5.0, 2.0]], [[8.0, 1.0, 2.0], [8.0, 8.0, 6.0], [8.0, 6.0, 5.0]], [[1.0, 7.0, 6.0], [7.0, 1.0, 5.0], [3.0, 5.0, 
1.0]]], [[[4.0, 5.0, 4.0], [6.0, 6.0, 1.0], [1.0, 6.0, 6.0]], [[4.0, 1.0, 4.0], [5.0, 8.0, 4.0], [1.0, 8.0, 5.0]], [[4.0, 4.0, 
7.0], [7.0, 3.0, 3.0], [3.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 3.0, 7.0], [5.0, 6.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0, 8.0], [6.0, 6.0, 7.0]]]))
      arg[2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 3.0], [5.0, 4.0, 2.0], [1.0, 7.0, 3.0]], [[8.0, 8.0, 4.0], [5.0, 3.0, 8.0], [4.0, 
6.0, 3.0]], [[6.0, 5.0, 3.0], [8.0, 5.0, 4.0], [5.0, 3.0, 3.0]]], [[[4.0, 4.0, 7.0], [2.0, 1.0, 7.0], [8.0, 5.0, 3.0]], [[2.0, 
2.0, 4.0], [5.0, 7.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 2.0, 8.0], [4.0, 2.0, 6.0], [6.0, 4.0, 5.0]]], [[[3.0, 4.0, 8.0], [6.0, 
3.0, 5.0], [2.0, 5.0, 2.0]], [[8.0, 1.0, 2.0], [8.0, 8.0, 6.0], [8.0, 6.0, 5.0]], [[6.0, 3.0, 7.0], [5.0, 6.0, 7.0], [3.0, 5.0, 
1.0]]], [[[4.0, 5.0, 4.0], [6.0, 6.0, 1.0], [1.0, 6.0, 6.0]], [[4.0, 1.0, 4.0], [5.0, 8.0, 4.0], [1.0, 8.0, 5.0]], [[4.0, 4.0, 
7.0], [7.0, 3.0, 3.0], [3.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 3.0], [5.0, 4.0, 2.0], [1.0, 7.0, 3.0]], 
[[8.0, 8.0, 4.0], [5.0, 3.0, 8.0], [4.0, 6.0, 3.0]], [[6.0, 5.0, 3.0], [8.0, 5.0, 4.0], [5.0, 3.0, 3.0]]], [[[4.0, 4.0, 7.0], 
[2.0, 1.0, 7.0], [8.0, 5.0, 3.0]], [[2.0, 2.0, 4.0], [5.0, 7.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 2.0, 8.0], [4.0, 2.0, 6.0], 
[6.0, 4.0, 5.0]]], [[[3.0, 4.0, 8.0], [6.0, 3.0, 5.0], [2.0, 5.0, 2.0]], [[8.0, 1.0, 2.0], [8.0, 8.0, 6.0], [8.0, 6.0, 5.0]], 
[[3.0, 5.0, 8.0], [6.0, 6.0, 7.0], [3.0, 5.0, 1.0]]], [[[4.0, 5.0, 4.0], [6.0, 6.0, 1.0], [1.0, 6.0, 6.0]], [[4.0, 1.0, 4.0], 
[5.0, 8.0, 4.0], [1.0, 8.0, 5.0]], [[4.0, 4.0, 7.0], [7.0, 3.0, 3.0], [3.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CPCN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 4.0], [5.0, 5.0, 8.0], [1.0, 7.0, 2.0]], [[2.0, 8.0, 3.0], [1.0, 6.0, 5.0], [4.0, 7.0, 
6.0]], [[1.0, 5.0, 5.0], [1.0, 5.0, 2.0], [4.0, 2.0, 1.0]]], [[[3.0, 3.0, 1.0], [3.0, 2.0, 2.0], [8.0, 5.0, 3.0]], [[2.0, 8.0, 
7.0], [6.0, 5.0, 2.0], [3.0, 7.0, 5.0]], [[8.0, 8.0, 6.0], [4.0, 8.0, 6.0], [6.0, 3.0, 3.0]]], [[[8.0, 5.0, 7.0], [5.0, 1.0, 
6.0], [1.0, 3.0, 5.0]], [[1.0, 6.0, 2.0], [3.0, 4.0, 3.0], [3.0, 4.0, 1.0]], [[1.0, 7.0, 1.0], [6.0, 1.0, 5.0], [7.0, 2.0, 
7.0]]], [[[6.0, 1.0, 6.0], [3.0, 3.0, 5.0], [5.0, 2.0, 3.0]], [[2.0, 3.0, 8.0], [1.0, 2.0, 5.0], [7.0, 8.0, 5.0]], [[5.0, 8.0, 
6.0], [8.0, 5.0, 8.0], [1.0, 7.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 3.0, 1.0], [1.0, 3.0, 1.0], [8.0, 1.0, 2.0]], [[1.0, 6.0, 3.0], [1.0, 7.0, 8.0], [1.0, 8.0, 
2.0]], [[2.0, 4.0, 7.0], [1.0, 5.0, 1.0], [8.0, 7.0, 6.0]], [[5.0, 3.0, 1.0], [6.0, 3.0, 4.0], [3.0, 1.0, 
1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 3.0, 8.0], [1.0, 7.0, 5.0], [2.0, 7.0, 8.0]], [[1.0, 6.0, 5.0], [6.0, 4.0, 6.0], 
[3.0, 4.0, 6.0]], [[5.0, 3.0, 5.0], [1.0, 6.0, 5.0], [4.0, 2.0, 7.0]], [[5.0, 6.0, 4.0], [6.0, 1.0, 2.0], [4.0, 2.0, 8.0]]]))
      arg[:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 4.0], [5.0, 5.0, 8.0], [1.0, 7.0, 2.0]], [[4.0, 3.0, 1.0], [1.0, 3.0, 1.0], [8.0, 
1.0, 2.0]], [[1.0, 5.0, 5.0], [1.0, 5.0, 2.0], [4.0, 2.0, 1.0]]], [[[3.0, 3.0, 1.0], [3.0, 2.0, 2.0], [8.0, 5.0, 3.0]], [[1.0, 
6.0, 3.0], [1.0, 7.0, 8.0], [1.0, 8.0, 2.0]], [[8.0, 8.0, 6.0], [4.0, 8.0, 6.0], [6.0, 3.0, 3.0]]], [[[8.0, 5.0, 7.0], [5.0, 
1.0, 6.0], [1.0, 3.0, 5.0]], [[2.0, 4.0, 7.0], [1.0, 5.0, 1.0], [8.0, 7.0, 6.0]], [[1.0, 7.0, 1.0], [6.0, 1.0, 5.0], [7.0, 2.0, 
7.0]]], [[[6.0, 1.0, 6.0], [3.0, 3.0, 5.0], [5.0, 2.0, 3.0]], [[5.0, 3.0, 1.0], [6.0, 3.0, 4.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 
6.0], [8.0, 5.0, 8.0], [1.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 4.0], [5.0, 5.0, 8.0], [1.0, 7.0, 2.0]], 
[[6.0, 3.0, 8.0], [1.0, 7.0, 5.0], [2.0, 7.0, 8.0]], [[1.0, 5.0, 5.0], [1.0, 5.0, 2.0], [4.0, 2.0, 1.0]]], [[[3.0, 3.0, 1.0], 
[3.0, 2.0, 2.0], [8.0, 5.0, 3.0]], [[1.0, 6.0, 5.0], [6.0, 4.0, 6.0], [3.0, 4.0, 6.0]], [[8.0, 8.0, 6.0], [4.0, 8.0, 6.0], 
[6.0, 3.0, 3.0]]], [[[8.0, 5.0, 7.0], [5.0, 1.0, 6.0], [1.0, 3.0, 5.0]], [[5.0, 3.0, 5.0], [1.0, 6.0, 5.0], [4.0, 2.0, 7.0]], 
[[1.0, 7.0, 1.0], [6.0, 1.0, 5.0], [7.0, 2.0, 7.0]]], [[[6.0, 1.0, 6.0], [3.0, 3.0, 5.0], [5.0, 2.0, 3.0]], [[5.0, 6.0, 4.0], 
[6.0, 1.0, 2.0], [4.0, 2.0, 8.0]], [[5.0, 8.0, 6.0], [8.0, 5.0, 8.0], [1.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LUUN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 2.0], [3.0, 8.0, 3.0], [5.0, 5.0, 2.0]], [[5.0, 8.0, 6.0], [6.0, 5.0, 4.0], [5.0, 8.0, 
8.0]], [[2.0, 2.0, 2.0], [4.0, 3.0, 4.0], [8.0, 6.0, 6.0]]], [[[2.0, 8.0, 2.0], [1.0, 1.0, 4.0], [3.0, 7.0, 6.0]], [[8.0, 3.0, 
4.0], [7.0, 3.0, 5.0], [6.0, 7.0, 4.0]], [[7.0, 1.0, 6.0], [8.0, 6.0, 7.0], [1.0, 2.0, 7.0]]], [[[6.0, 1.0, 4.0], [3.0, 8.0, 
8.0], [4.0, 5.0, 5.0]], [[6.0, 8.0, 8.0], [3.0, 3.0, 5.0], [8.0, 7.0, 8.0]], [[2.0, 7.0, 1.0], [7.0, 6.0, 6.0], [6.0, 7.0, 
8.0]]], [[[1.0, 8.0, 1.0], [5.0, 3.0, 1.0], [3.0, 1.0, 4.0]], [[5.0, 7.0, 8.0], [1.0, 6.0, 5.0], [1.0, 4.0, 8.0]], [[6.0, 8.0, 
4.0], [7.0, 4.0, 4.0], [2.0, 2.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 4.0, 3.0], [1.0, 6.0, 5.0]], [[5.0, 7.0, 6.0], [6.0, 7.0, 2.0]]], [[[7.0, 5.0, 4.0], [3.0, 
5.0, 2.0]], [[6.0, 1.0, 3.0], [1.0, 7.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 7.0], [1.0, 5.0, 2.0]], [[8.0, 5.0, 2.0], [4.0, 8.0, 2.0]]], [[[8.0, 3.0, 
3.0], [1.0, 4.0, 2.0]], [[4.0, 4.0, 3.0], [6.0, 8.0, 2.0]]]]))
      arg[2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 2.0], [3.0, 8.0, 3.0], [5.0, 5.0, 2.0]], [[5.0, 8.0, 6.0], [6.0, 5.0, 4.0], [5.0, 
8.0, 8.0]], [[2.0, 2.0, 2.0], [4.0, 3.0, 4.0], [8.0, 6.0, 6.0]]], [[[2.0, 8.0, 2.0], [1.0, 1.0, 4.0], [3.0, 7.0, 6.0]], [[8.0, 
3.0, 4.0], [7.0, 3.0, 5.0], [6.0, 7.0, 4.0]], [[7.0, 1.0, 6.0], [8.0, 6.0, 7.0], [1.0, 2.0, 7.0]]], [[[4.0, 4.0, 3.0], [1.0, 
6.0, 5.0], [4.0, 5.0, 5.0]], [[5.0, 7.0, 6.0], [6.0, 7.0, 2.0], [8.0, 7.0, 8.0]], [[2.0, 7.0, 1.0], [7.0, 6.0, 6.0], [6.0, 7.0, 
8.0]]], [[[7.0, 5.0, 4.0], [3.0, 5.0, 2.0], [3.0, 1.0, 4.0]], [[6.0, 1.0, 3.0], [1.0, 7.0, 5.0], [1.0, 4.0, 8.0]], [[6.0, 8.0, 
4.0], [7.0, 4.0, 4.0], [2.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 2.0], [3.0, 8.0, 3.0], [5.0, 5.0, 2.0]], 
[[5.0, 8.0, 6.0], [6.0, 5.0, 4.0], [5.0, 8.0, 8.0]], [[2.0, 2.0, 2.0], [4.0, 3.0, 4.0], [8.0, 6.0, 6.0]]], [[[2.0, 8.0, 2.0], 
[1.0, 1.0, 4.0], [3.0, 7.0, 6.0]], [[8.0, 3.0, 4.0], [7.0, 3.0, 5.0], [6.0, 7.0, 4.0]], [[7.0, 1.0, 6.0], [8.0, 6.0, 7.0], 
[1.0, 2.0, 7.0]]], [[[7.0, 3.0, 7.0], [1.0, 5.0, 2.0], [4.0, 5.0, 5.0]], [[8.0, 5.0, 2.0], [4.0, 8.0, 2.0], [8.0, 7.0, 8.0]], 
[[2.0, 7.0, 1.0], [7.0, 6.0, 6.0], [6.0, 7.0, 8.0]]], [[[8.0, 3.0, 3.0], [1.0, 4.0, 2.0], [3.0, 1.0, 4.0]], [[4.0, 4.0, 3.0], 
[6.0, 8.0, 2.0], [1.0, 4.0, 8.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 4.0], [2.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PPPN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 5.0], [2.0, 4.0, 2.0], [5.0, 1.0, 7.0]], [[8.0, 5.0, 3.0], [3.0, 7.0, 7.0], [3.0, 6.0, 
3.0]], [[4.0, 8.0, 7.0], [4.0, 3.0, 6.0], [5.0, 7.0, 8.0]]], [[[2.0, 4.0, 2.0], [3.0, 4.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 8.0, 
7.0], [6.0, 5.0, 5.0], [8.0, 8.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 4.0, 6.0], [4.0, 6.0, 4.0]]], [[[1.0, 6.0, 5.0], [5.0, 5.0, 
6.0], [2.0, 7.0, 2.0]], [[7.0, 4.0, 2.0], [2.0, 7.0, 2.0], [2.0, 4.0, 2.0]], [[7.0, 3.0, 2.0], [7.0, 7.0, 8.0], [1.0, 8.0, 
1.0]]], [[[5.0, 7.0, 1.0], [3.0, 8.0, 4.0], [6.0, 2.0, 2.0]], [[3.0, 6.0, 2.0], [1.0, 6.0, 4.0], [2.0, 8.0, 8.0]], [[7.0, 3.0, 
2.0], [2.0, 8.0, 1.0], [8.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([5.0, 2.0, 6.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0, 1.0, 5.0]))
      arg[2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 5.0], [2.0, 4.0, 2.0], [5.0, 1.0, 7.0]], [[8.0, 5.0, 3.0], [3.0, 7.0, 7.0], [3.0, 
6.0, 3.0]], [[4.0, 8.0, 7.0], [4.0, 3.0, 6.0], [5.0, 7.0, 8.0]]], [[[2.0, 4.0, 2.0], [3.0, 4.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 
8.0, 7.0], [6.0, 5.0, 5.0], [8.0, 8.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 4.0, 6.0], [4.0, 6.0, 4.0]]], [[[1.0, 6.0, 5.0], [5.0, 
5.0, 6.0], [2.0, 7.0, 2.0]], [[7.0, 4.0, 2.0], [5.0, 2.0, 6.0], [2.0, 4.0, 2.0]], [[7.0, 3.0, 2.0], [7.0, 7.0, 8.0], [1.0, 8.0, 
1.0]]], [[[5.0, 7.0, 1.0], [3.0, 8.0, 4.0], [6.0, 2.0, 2.0]], [[3.0, 6.0, 2.0], [1.0, 6.0, 4.0], [2.0, 8.0, 8.0]], [[7.0, 3.0, 
2.0], [2.0, 8.0, 1.0], [8.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 5.0], [2.0, 4.0, 2.0], [5.0, 1.0, 7.0]], 
[[8.0, 5.0, 3.0], [3.0, 7.0, 7.0], [3.0, 6.0, 3.0]], [[4.0, 8.0, 7.0], [4.0, 3.0, 6.0], [5.0, 7.0, 8.0]]], [[[2.0, 4.0, 2.0], 
[3.0, 4.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 8.0, 7.0], [6.0, 5.0, 5.0], [8.0, 8.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 4.0, 6.0], 
[4.0, 6.0, 4.0]]], [[[1.0, 6.0, 5.0], [5.0, 5.0, 6.0], [2.0, 7.0, 2.0]], [[7.0, 4.0, 2.0], [8.0, 1.0, 5.0], [2.0, 4.0, 2.0]], 
[[7.0, 3.0, 2.0], [7.0, 7.0, 8.0], [1.0, 8.0, 1.0]]], [[[5.0, 7.0, 1.0], [3.0, 8.0, 4.0], [6.0, 2.0, 2.0]], [[3.0, 6.0, 2.0], 
[1.0, 6.0, 4.0], [2.0, 8.0, 8.0]], [[7.0, 3.0, 2.0], [2.0, 8.0, 1.0], [8.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LLLL(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 2.0], [2.0, 7.0, 1.0], [7.0, 7.0, 6.0]], [[5.0, 3.0, 5.0], [8.0, 7.0, 1.0], [8.0, 8.0, 
7.0]], [[6.0, 3.0, 7.0], [5.0, 2.0, 1.0], [4.0, 3.0, 4.0]]], [[[7.0, 8.0, 2.0], [4.0, 1.0, 5.0], [1.0, 3.0, 6.0]], [[1.0, 5.0, 
1.0], [8.0, 2.0, 7.0], [3.0, 1.0, 5.0]], [[5.0, 7.0, 3.0], [7.0, 2.0, 6.0], [2.0, 4.0, 2.0]]], [[[8.0, 2.0, 2.0], [5.0, 1.0, 
8.0], [3.0, 8.0, 1.0]], [[3.0, 5.0, 5.0], [1.0, 5.0, 2.0], [2.0, 6.0, 2.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 3.0], [7.0, 1.0, 
3.0]]], [[[6.0, 1.0, 3.0], [4.0, 2.0, 4.0], [5.0, 5.0, 1.0]], [[1.0, 8.0, 2.0], [1.0, 1.0, 2.0], [1.0, 3.0, 8.0]], [[8.0, 1.0, 
1.0], [2.0, 7.0, 1.0], [3.0, 6.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0]]], [[[1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0]]], [[[1.0]]]]))
      arg[2:,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 2.0], [2.0, 7.0, 1.0], [7.0, 7.0, 6.0]], [[5.0, 3.0, 5.0], [8.0, 7.0, 1.0], [8.0, 
8.0, 7.0]], [[6.0, 3.0, 7.0], [5.0, 2.0, 1.0], [4.0, 3.0, 4.0]]], [[[7.0, 8.0, 2.0], [4.0, 1.0, 5.0], [1.0, 3.0, 6.0]], [[1.0, 
5.0, 1.0], [8.0, 2.0, 7.0], [3.0, 1.0, 5.0]], [[5.0, 7.0, 3.0], [7.0, 2.0, 6.0], [2.0, 4.0, 2.0]]], [[[8.0, 2.0, 2.0], [5.0, 
1.0, 8.0], [3.0, 8.0, 1.0]], [[3.0, 5.0, 5.0], [1.0, 5.0, 2.0], [2.0, 6.0, 2.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 3.0], [7.0, 1.0, 
2.0]]], [[[6.0, 1.0, 3.0], [4.0, 2.0, 4.0], [5.0, 5.0, 1.0]], [[1.0, 8.0, 2.0], [1.0, 1.0, 2.0], [1.0, 3.0, 8.0]], [[8.0, 1.0, 
1.0], [2.0, 7.0, 1.0], [3.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 2.0], [2.0, 7.0, 1.0], [7.0, 7.0, 6.0]], 
[[5.0, 3.0, 5.0], [8.0, 7.0, 1.0], [8.0, 8.0, 7.0]], [[6.0, 3.0, 7.0], [5.0, 2.0, 1.0], [4.0, 3.0, 4.0]]], [[[7.0, 8.0, 2.0], 
[4.0, 1.0, 5.0], [1.0, 3.0, 6.0]], [[1.0, 5.0, 1.0], [8.0, 2.0, 7.0], [3.0, 1.0, 5.0]], [[5.0, 7.0, 3.0], [7.0, 2.0, 6.0], 
[2.0, 4.0, 2.0]]], [[[8.0, 2.0, 2.0], [5.0, 1.0, 8.0], [3.0, 8.0, 1.0]], [[3.0, 5.0, 5.0], [1.0, 5.0, 2.0], [2.0, 6.0, 2.0]], 
[[3.0, 3.0, 3.0], [6.0, 5.0, 3.0], [7.0, 1.0, 4.0]]], [[[6.0, 1.0, 3.0], [4.0, 2.0, 4.0], [5.0, 5.0, 1.0]], [[1.0, 8.0, 2.0], 
[1.0, 1.0, 2.0], [1.0, 3.0, 8.0]], [[8.0, 1.0, 1.0], [2.0, 7.0, 1.0], [3.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RPLN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 4.0], [3.0, 6.0, 2.0], [8.0, 4.0, 2.0]], [[2.0, 6.0, 8.0], [5.0, 3.0, 5.0], [4.0, 6.0, 
7.0]], [[7.0, 7.0, 1.0], [1.0, 8.0, 1.0], [4.0, 1.0, 5.0]]], [[[3.0, 8.0, 4.0], [5.0, 2.0, 2.0], [3.0, 3.0, 2.0]], [[6.0, 6.0, 
6.0], [1.0, 2.0, 7.0], [1.0, 1.0, 5.0]], [[5.0, 6.0, 4.0], [1.0, 6.0, 5.0], [1.0, 2.0, 5.0]]], [[[7.0, 7.0, 8.0], [3.0, 7.0, 
8.0], [2.0, 1.0, 5.0]], [[2.0, 5.0, 4.0], [1.0, 5.0, 6.0], [8.0, 6.0, 1.0]], [[5.0, 8.0, 4.0], [4.0, 2.0, 7.0], [3.0, 7.0, 
8.0]]], [[[5.0, 1.0, 3.0], [8.0, 7.0, 5.0], [4.0, 5.0, 3.0]], [[2.0, 5.0, 8.0], [1.0, 4.0, 7.0], [8.0, 5.0, 1.0]], [[4.0, 6.0, 
4.0], [1.0, 6.0, 7.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 7.0, 5.0]], [[6.0, 4.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 1.0, 8.0]], [[4.0, 8.0, 4.0]]]))
      arg[1:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 4.0], [3.0, 6.0, 2.0], [8.0, 4.0, 2.0]], [[2.0, 6.0, 8.0], [5.0, 3.0, 5.0], [4.0, 
6.0, 7.0]], [[7.0, 7.0, 1.0], [1.0, 8.0, 1.0], [4.0, 1.0, 5.0]]], [[[3.0, 8.0, 4.0], [5.0, 2.0, 2.0], [3.0, 3.0, 2.0]], [[6.0, 
6.0, 6.0], [1.0, 2.0, 7.0], [3.0, 7.0, 5.0]], [[5.0, 6.0, 4.0], [1.0, 6.0, 5.0], [1.0, 2.0, 5.0]]], [[[7.0, 7.0, 8.0], [3.0, 
7.0, 8.0], [2.0, 1.0, 5.0]], [[2.0, 5.0, 4.0], [1.0, 5.0, 6.0], [6.0, 4.0, 1.0]], [[5.0, 8.0, 4.0], [4.0, 2.0, 7.0], [3.0, 7.0, 
8.0]]], [[[5.0, 1.0, 3.0], [8.0, 7.0, 5.0], [4.0, 5.0, 3.0]], [[2.0, 5.0, 8.0], [1.0, 4.0, 7.0], [8.0, 5.0, 1.0]], [[4.0, 6.0, 
4.0], [1.0, 6.0, 7.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 4.0], [3.0, 6.0, 2.0], [8.0, 4.0, 2.0]], 
[[2.0, 6.0, 8.0], [5.0, 3.0, 5.0], [4.0, 6.0, 7.0]], [[7.0, 7.0, 1.0], [1.0, 8.0, 1.0], [4.0, 1.0, 5.0]]], [[[3.0, 8.0, 4.0], 
[5.0, 2.0, 2.0], [3.0, 3.0, 2.0]], [[6.0, 6.0, 6.0], [1.0, 2.0, 7.0], [6.0, 1.0, 8.0]], [[5.0, 6.0, 4.0], [1.0, 6.0, 5.0], 
[1.0, 2.0, 5.0]]], [[[7.0, 7.0, 8.0], [3.0, 7.0, 8.0], [2.0, 1.0, 5.0]], [[2.0, 5.0, 4.0], [1.0, 5.0, 6.0], [4.0, 8.0, 4.0]], 
[[5.0, 8.0, 4.0], [4.0, 2.0, 7.0], [3.0, 7.0, 8.0]]], [[[5.0, 1.0, 3.0], [8.0, 7.0, 5.0], [4.0, 5.0, 3.0]], [[2.0, 5.0, 8.0], 
[1.0, 4.0, 7.0], [8.0, 5.0, 1.0]], [[4.0, 6.0, 4.0], [1.0, 6.0, 7.0], [3.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCPN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 8.0], [5.0, 7.0, 4.0], [5.0, 5.0, 5.0]], [[6.0, 1.0, 3.0], [2.0, 5.0, 8.0], [5.0, 1.0, 
5.0]], [[7.0, 5.0, 5.0], [4.0, 2.0, 4.0], [2.0, 7.0, 3.0]]], [[[7.0, 6.0, 6.0], [5.0, 2.0, 4.0], [7.0, 5.0, 1.0]], [[7.0, 6.0, 
7.0], [7.0, 6.0, 1.0], [3.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [6.0, 6.0, 1.0], [6.0, 6.0, 2.0]]], [[[4.0, 8.0, 7.0], [1.0, 3.0, 
4.0], [4.0, 6.0, 8.0]], [[7.0, 4.0, 5.0], [8.0, 4.0, 7.0], [5.0, 7.0, 2.0]], [[7.0, 5.0, 2.0], [4.0, 6.0, 6.0], [2.0, 2.0, 
7.0]]], [[[1.0, 1.0, 4.0], [7.0, 5.0, 6.0], [4.0, 7.0, 5.0]], [[6.0, 2.0, 1.0], [2.0, 1.0, 1.0], [4.0, 5.0, 3.0]], [[4.0, 1.0, 
6.0], [7.0, 6.0, 3.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 5.0, 8.0], [8.0, 3.0, 6.0], [1.0, 8.0, 7.0]], [[7.0, 2.0, 4.0], [5.0, 8.0, 6.0], [5.0, 6.0, 
1.0]], [[3.0, 4.0, 1.0], [1.0, 6.0, 6.0], [4.0, 4.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 3.0, 7.0], [6.0, 5.0, 2.0], [4.0, 7.0, 8.0]], [[6.0, 2.0, 6.0], [2.0, 8.0, 3.0], 
[6.0, 4.0, 3.0]], [[2.0, 2.0, 3.0], [8.0, 6.0, 2.0], [2.0, 4.0, 7.0]]]))
      arg[:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 8.0], [2.0, 5.0, 8.0], [5.0, 5.0, 5.0]], [[6.0, 1.0, 3.0], [8.0, 3.0, 6.0], [5.0, 
1.0, 5.0]], [[7.0, 5.0, 5.0], [1.0, 8.0, 7.0], [2.0, 7.0, 3.0]]], [[[7.0, 6.0, 6.0], [7.0, 2.0, 4.0], [7.0, 5.0, 1.0]], [[7.0, 
6.0, 7.0], [5.0, 8.0, 6.0], [3.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [5.0, 6.0, 1.0], [6.0, 6.0, 2.0]]], [[[4.0, 8.0, 7.0], [3.0, 
4.0, 1.0], [4.0, 6.0, 8.0]], [[7.0, 4.0, 5.0], [1.0, 6.0, 6.0], [5.0, 7.0, 2.0]], [[7.0, 5.0, 2.0], [4.0, 4.0, 2.0], [2.0, 2.0, 
7.0]]], [[[1.0, 1.0, 4.0], [7.0, 5.0, 6.0], [4.0, 7.0, 5.0]], [[6.0, 2.0, 1.0], [2.0, 1.0, 1.0], [4.0, 5.0, 3.0]], [[4.0, 1.0, 
6.0], [7.0, 6.0, 3.0], [5.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 8.0], [3.0, 3.0, 7.0], [5.0, 5.0, 5.0]], 
[[6.0, 1.0, 3.0], [6.0, 5.0, 2.0], [5.0, 1.0, 5.0]], [[7.0, 5.0, 5.0], [4.0, 7.0, 8.0], [2.0, 7.0, 3.0]]], [[[7.0, 6.0, 6.0], 
[6.0, 2.0, 6.0], [7.0, 5.0, 1.0]], [[7.0, 6.0, 7.0], [2.0, 8.0, 3.0], [3.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [6.0, 4.0, 3.0], 
[6.0, 6.0, 2.0]]], [[[4.0, 8.0, 7.0], [2.0, 2.0, 3.0], [4.0, 6.0, 8.0]], [[7.0, 4.0, 5.0], [8.0, 6.0, 2.0], [5.0, 7.0, 2.0]], 
[[7.0, 5.0, 2.0], [2.0, 4.0, 7.0], [2.0, 2.0, 7.0]]], [[[1.0, 1.0, 4.0], [7.0, 5.0, 6.0], [4.0, 7.0, 5.0]], [[6.0, 2.0, 1.0], 
[2.0, 1.0, 1.0], [4.0, 5.0, 3.0]], [[4.0, 1.0, 6.0], [7.0, 6.0, 3.0], [5.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLPN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 6.0], [8.0, 4.0, 7.0], [7.0, 4.0, 4.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 8.0], [1.0, 3.0, 
7.0]], [[6.0, 6.0, 3.0], [5.0, 4.0, 4.0], [1.0, 4.0, 6.0]]], [[[2.0, 4.0, 3.0], [5.0, 4.0, 2.0], [4.0, 7.0, 2.0]], [[6.0, 8.0, 
1.0], [7.0, 4.0, 7.0], [5.0, 8.0, 8.0]], [[3.0, 1.0, 6.0], [6.0, 5.0, 5.0], [8.0, 2.0, 3.0]]], [[[8.0, 3.0, 8.0], [6.0, 2.0, 
1.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 8.0], [2.0, 5.0, 2.0], [2.0, 2.0, 3.0]], [[7.0, 7.0, 8.0], [1.0, 2.0, 5.0], [8.0, 2.0, 
1.0]]], [[[8.0, 2.0, 2.0], [5.0, 2.0, 4.0], [8.0, 4.0, 7.0]], [[8.0, 3.0, 6.0], [1.0, 6.0, 4.0], [2.0, 1.0, 8.0]], [[8.0, 7.0, 
1.0], [3.0, 5.0, 1.0], [2.0, 3.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 8.0, 4.0]], [[4.0, 8.0, 7.0]], [[2.0, 3.0, 8.0]], [[1.0, 7.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 4.0, 5.0]], [[7.0, 3.0, 5.0]], [[1.0, 2.0, 2.0]], [[1.0, 8.0, 3.0]]]))
      arg[:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 6.0], [8.0, 4.0, 7.0], [7.0, 4.0, 4.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 8.0], [1.0, 
3.0, 7.0]], [[6.0, 6.0, 3.0], [2.0, 8.0, 4.0], [1.0, 4.0, 6.0]]], [[[2.0, 4.0, 3.0], [5.0, 4.0, 2.0], [4.0, 7.0, 2.0]], [[6.0, 
8.0, 1.0], [7.0, 4.0, 7.0], [5.0, 8.0, 8.0]], [[3.0, 1.0, 6.0], [4.0, 8.0, 7.0], [8.0, 2.0, 3.0]]], [[[8.0, 3.0, 8.0], [6.0, 
2.0, 1.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 8.0], [2.0, 5.0, 2.0], [2.0, 2.0, 3.0]], [[7.0, 7.0, 8.0], [2.0, 3.0, 8.0], [8.0, 2.0, 
1.0]]], [[[8.0, 2.0, 2.0], [5.0, 2.0, 4.0], [8.0, 4.0, 7.0]], [[8.0, 3.0, 6.0], [1.0, 6.0, 4.0], [2.0, 1.0, 8.0]], [[8.0, 7.0, 
1.0], [1.0, 7.0, 3.0], [2.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 6.0], [8.0, 4.0, 7.0], [7.0, 4.0, 4.0]], 
[[1.0, 4.0, 3.0], [2.0, 1.0, 8.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 3.0], [1.0, 4.0, 5.0], [1.0, 4.0, 6.0]]], [[[2.0, 4.0, 3.0], 
[5.0, 4.0, 2.0], [4.0, 7.0, 2.0]], [[6.0, 8.0, 1.0], [7.0, 4.0, 7.0], [5.0, 8.0, 8.0]], [[3.0, 1.0, 6.0], [7.0, 3.0, 5.0], 
[8.0, 2.0, 3.0]]], [[[8.0, 3.0, 8.0], [6.0, 2.0, 1.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 8.0], [2.0, 5.0, 2.0], [2.0, 2.0, 3.0]], 
[[7.0, 7.0, 8.0], [1.0, 2.0, 2.0], [8.0, 2.0, 1.0]]], [[[8.0, 2.0, 2.0], [5.0, 2.0, 4.0], [8.0, 4.0, 7.0]], [[8.0, 3.0, 6.0], 
[1.0, 6.0, 4.0], [2.0, 1.0, 8.0]], [[8.0, 7.0, 1.0], [1.0, 8.0, 3.0], [2.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UULU(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 6.0], [4.0, 5.0, 1.0], [8.0, 1.0, 6.0]], [[3.0, 7.0, 6.0], [5.0, 8.0, 6.0], [4.0, 6.0, 
1.0]], [[8.0, 7.0, 1.0], [5.0, 4.0, 4.0], [4.0, 8.0, 1.0]]], [[[4.0, 6.0, 7.0], [5.0, 2.0, 2.0], [3.0, 1.0, 3.0]], [[1.0, 3.0, 
6.0], [7.0, 5.0, 7.0], [3.0, 4.0, 2.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 1.0], [2.0, 4.0, 4.0]]], [[[4.0, 8.0, 5.0], [3.0, 8.0, 
7.0], [6.0, 3.0, 3.0]], [[2.0, 7.0, 5.0], [7.0, 7.0, 3.0], [5.0, 2.0, 6.0]], [[1.0, 6.0, 4.0], [7.0, 4.0, 1.0], [1.0, 4.0, 
2.0]]], [[[6.0, 7.0, 7.0], [7.0, 4.0, 6.0], [3.0, 2.0, 6.0]], [[5.0, 2.0, 7.0], [1.0, 1.0, 4.0], [3.0, 4.0, 5.0]], [[7.0, 1.0, 
7.0], [2.0, 4.0, 5.0], [5.0, 8.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 8.0]], [[8.0, 8.0]]], [[[4.0, 7.0]], [[6.0, 1.0]]], [[[3.0, 7.0]], [[3.0, 
5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 4.0]], [[4.0, 2.0]]], [[[5.0, 7.0]], [[6.0, 1.0]]], [[[6.0, 3.0]], [[6.0, 
8.0]]]]))
      arg[:3,:2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 6.0], [4.0, 5.0, 1.0], [2.0, 8.0, 6.0]], [[3.0, 7.0, 6.0], [5.0, 8.0, 6.0], [8.0, 
8.0, 1.0]], [[8.0, 7.0, 1.0], [5.0, 4.0, 4.0], [4.0, 8.0, 1.0]]], [[[4.0, 6.0, 7.0], [5.0, 2.0, 2.0], [4.0, 7.0, 3.0]], [[1.0, 
3.0, 6.0], [7.0, 5.0, 7.0], [6.0, 1.0, 2.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 1.0], [2.0, 4.0, 4.0]]], [[[4.0, 8.0, 5.0], [3.0, 
8.0, 7.0], [3.0, 7.0, 3.0]], [[2.0, 7.0, 5.0], [7.0, 7.0, 3.0], [3.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [7.0, 4.0, 1.0], [1.0, 4.0, 
2.0]]], [[[6.0, 7.0, 7.0], [7.0, 4.0, 6.0], [3.0, 2.0, 6.0]], [[5.0, 2.0, 7.0], [1.0, 1.0, 4.0], [3.0, 4.0, 5.0]], [[7.0, 1.0, 
7.0], [2.0, 4.0, 5.0], [5.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 6.0], [4.0, 5.0, 1.0], [1.0, 4.0, 6.0]], 
[[3.0, 7.0, 6.0], [5.0, 8.0, 6.0], [4.0, 2.0, 1.0]], [[8.0, 7.0, 1.0], [5.0, 4.0, 4.0], [4.0, 8.0, 1.0]]], [[[4.0, 6.0, 7.0], 
[5.0, 2.0, 2.0], [5.0, 7.0, 3.0]], [[1.0, 3.0, 6.0], [7.0, 5.0, 7.0], [6.0, 1.0, 2.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 1.0], 
[2.0, 4.0, 4.0]]], [[[4.0, 8.0, 5.0], [3.0, 8.0, 7.0], [6.0, 3.0, 3.0]], [[2.0, 7.0, 5.0], [7.0, 7.0, 3.0], [6.0, 8.0, 6.0]], 
[[1.0, 6.0, 4.0], [7.0, 4.0, 1.0], [1.0, 4.0, 2.0]]], [[[6.0, 7.0, 7.0], [7.0, 4.0, 6.0], [3.0, 2.0, 6.0]], [[5.0, 2.0, 7.0], 
[1.0, 1.0, 4.0], [3.0, 4.0, 5.0]], [[7.0, 1.0, 7.0], [2.0, 4.0, 5.0], [5.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PRCN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 7.0], [1.0, 8.0, 5.0], [8.0, 4.0, 6.0]], [[3.0, 8.0, 4.0], [2.0, 7.0, 5.0], [1.0, 3.0, 
1.0]], [[6.0, 4.0, 5.0], [1.0, 2.0, 3.0], [7.0, 6.0, 7.0]]], [[[1.0, 2.0, 3.0], [5.0, 8.0, 1.0], [6.0, 7.0, 1.0]], [[3.0, 5.0, 
1.0], [2.0, 1.0, 2.0], [5.0, 8.0, 7.0]], [[2.0, 7.0, 5.0], [3.0, 7.0, 8.0], [8.0, 1.0, 1.0]]], [[[7.0, 5.0, 1.0], [5.0, 5.0, 
2.0], [7.0, 1.0, 2.0]], [[1.0, 7.0, 2.0], [8.0, 2.0, 1.0], [4.0, 3.0, 1.0]], [[2.0, 3.0, 8.0], [1.0, 7.0, 4.0], [4.0, 8.0, 
3.0]]], [[[8.0, 7.0, 1.0], [6.0, 5.0, 7.0], [2.0, 6.0, 3.0]], [[4.0, 3.0, 7.0], [1.0, 1.0, 7.0], [3.0, 6.0, 3.0]], [[1.0, 2.0, 
8.0], [3.0, 5.0, 8.0], [8.0, 6.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 1.0, 4.0], [4.0, 3.0, 1.0], [3.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 8.0, 8.0], [3.0, 5.0, 7.0], [2.0, 3.0, 8.0]]]))
      arg[2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 7.0], [1.0, 8.0, 5.0], [8.0, 4.0, 6.0]], [[3.0, 8.0, 4.0], [2.0, 7.0, 5.0], [1.0, 
3.0, 1.0]], [[6.0, 4.0, 5.0], [1.0, 2.0, 3.0], [7.0, 6.0, 7.0]]], [[[1.0, 2.0, 3.0], [5.0, 8.0, 1.0], [6.0, 7.0, 1.0]], [[3.0, 
5.0, 1.0], [2.0, 1.0, 2.0], [5.0, 8.0, 7.0]], [[2.0, 7.0, 5.0], [3.0, 7.0, 8.0], [8.0, 1.0, 1.0]]], [[[7.0, 5.0, 1.0], [5.0, 
5.0, 2.0], [7.0, 1.0, 2.0]], [[8.0, 1.0, 4.0], [4.0, 3.0, 1.0], [3.0, 3.0, 1.0]], [[2.0, 3.0, 8.0], [1.0, 7.0, 4.0], [4.0, 8.0, 
3.0]]], [[[8.0, 7.0, 1.0], [6.0, 5.0, 7.0], [2.0, 6.0, 3.0]], [[4.0, 3.0, 7.0], [1.0, 1.0, 7.0], [3.0, 6.0, 3.0]], [[1.0, 2.0, 
8.0], [3.0, 5.0, 8.0], [8.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 7.0], [1.0, 8.0, 5.0], [8.0, 4.0, 6.0]], 
[[3.0, 8.0, 4.0], [2.0, 7.0, 5.0], [1.0, 3.0, 1.0]], [[6.0, 4.0, 5.0], [1.0, 2.0, 3.0], [7.0, 6.0, 7.0]]], [[[1.0, 2.0, 3.0], 
[5.0, 8.0, 1.0], [6.0, 7.0, 1.0]], [[3.0, 5.0, 1.0], [2.0, 1.0, 2.0], [5.0, 8.0, 7.0]], [[2.0, 7.0, 5.0], [3.0, 7.0, 8.0], 
[8.0, 1.0, 1.0]]], [[[7.0, 5.0, 1.0], [5.0, 5.0, 2.0], [7.0, 1.0, 2.0]], [[4.0, 8.0, 8.0], [3.0, 5.0, 7.0], [2.0, 3.0, 8.0]], 
[[2.0, 3.0, 8.0], [1.0, 7.0, 4.0], [4.0, 8.0, 3.0]]], [[[8.0, 7.0, 1.0], [6.0, 5.0, 7.0], [2.0, 6.0, 3.0]], [[4.0, 3.0, 7.0], 
[1.0, 1.0, 7.0], [3.0, 6.0, 3.0]], [[1.0, 2.0, 8.0], [3.0, 5.0, 8.0], [8.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PRRN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 7.0], [8.0, 2.0, 8.0], [6.0, 6.0, 7.0]], [[2.0, 8.0, 3.0], [4.0, 1.0, 6.0], [7.0, 5.0, 
4.0]], [[7.0, 2.0, 7.0], [4.0, 2.0, 4.0], [8.0, 3.0, 2.0]]], [[[5.0, 1.0, 8.0], [5.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[2.0, 6.0, 
3.0], [7.0, 4.0, 4.0], [5.0, 7.0, 2.0]], [[6.0, 3.0, 2.0], [1.0, 8.0, 6.0], [1.0, 7.0, 4.0]]], [[[4.0, 4.0, 4.0], [5.0, 2.0, 
2.0], [5.0, 8.0, 8.0]], [[1.0, 1.0, 8.0], [4.0, 1.0, 1.0], [7.0, 1.0, 7.0]], [[8.0, 3.0, 2.0], [5.0, 6.0, 2.0], [2.0, 4.0, 
1.0]]], [[[1.0, 5.0, 8.0], [8.0, 3.0, 3.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 4.0], [8.0, 8.0, 7.0], [4.0, 1.0, 8.0]], [[5.0, 2.0, 
5.0], [4.0, 7.0, 8.0], [7.0, 8.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 7.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 2.0, 7.0]]]))
      arg[2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 7.0], [8.0, 2.0, 8.0], [6.0, 6.0, 7.0]], [[2.0, 8.0, 3.0], [4.0, 1.0, 6.0], [7.0, 
5.0, 4.0]], [[7.0, 2.0, 7.0], [4.0, 2.0, 4.0], [8.0, 3.0, 2.0]]], [[[5.0, 1.0, 8.0], [5.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[2.0, 
6.0, 3.0], [7.0, 4.0, 4.0], [5.0, 7.0, 2.0]], [[6.0, 3.0, 2.0], [1.0, 8.0, 6.0], [1.0, 7.0, 4.0]]], [[[4.0, 4.0, 4.0], [5.0, 
2.0, 2.0], [5.0, 8.0, 8.0]], [[1.0, 1.0, 8.0], [6.0, 7.0, 7.0], [7.0, 1.0, 7.0]], [[8.0, 3.0, 2.0], [5.0, 6.0, 2.0], [2.0, 4.0, 
1.0]]], [[[1.0, 5.0, 8.0], [8.0, 3.0, 3.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 4.0], [8.0, 8.0, 7.0], [4.0, 1.0, 8.0]], [[5.0, 2.0, 
5.0], [4.0, 7.0, 8.0], [7.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 7.0], [8.0, 2.0, 8.0], [6.0, 6.0, 7.0]], 
[[2.0, 8.0, 3.0], [4.0, 1.0, 6.0], [7.0, 5.0, 4.0]], [[7.0, 2.0, 7.0], [4.0, 2.0, 4.0], [8.0, 3.0, 2.0]]], [[[5.0, 1.0, 8.0], 
[5.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[2.0, 6.0, 3.0], [7.0, 4.0, 4.0], [5.0, 7.0, 2.0]], [[6.0, 3.0, 2.0], [1.0, 8.0, 6.0], 
[1.0, 7.0, 4.0]]], [[[4.0, 4.0, 4.0], [5.0, 2.0, 2.0], [5.0, 8.0, 8.0]], [[1.0, 1.0, 8.0], [1.0, 2.0, 7.0], [7.0, 1.0, 7.0]], 
[[8.0, 3.0, 2.0], [5.0, 6.0, 2.0], [2.0, 4.0, 1.0]]], [[[1.0, 5.0, 8.0], [8.0, 3.0, 3.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 4.0], 
[8.0, 8.0, 7.0], [4.0, 1.0, 8.0]], [[5.0, 2.0, 5.0], [4.0, 7.0, 8.0], [7.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UPPN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 4.0], [1.0, 4.0, 1.0], [8.0, 4.0, 1.0]], [[6.0, 1.0, 3.0], [5.0, 1.0, 8.0], [8.0, 3.0, 
5.0]], [[1.0, 1.0, 4.0], [1.0, 8.0, 5.0], [6.0, 5.0, 7.0]]], [[[4.0, 5.0, 2.0], [8.0, 3.0, 1.0], [2.0, 4.0, 5.0]], [[4.0, 1.0, 
4.0], [5.0, 4.0, 5.0], [5.0, 3.0, 1.0]], [[5.0, 2.0, 7.0], [2.0, 5.0, 8.0], [3.0, 8.0, 2.0]]], [[[6.0, 6.0, 7.0], [8.0, 3.0, 
4.0], [2.0, 1.0, 7.0]], [[2.0, 3.0, 1.0], [2.0, 1.0, 7.0], [1.0, 5.0, 8.0]], [[3.0, 8.0, 7.0], [4.0, 5.0, 3.0], [1.0, 1.0, 
3.0]]], [[[5.0, 2.0, 8.0], [3.0, 7.0, 2.0], [6.0, 7.0, 6.0]], [[2.0, 2.0, 2.0], [3.0, 3.0, 3.0], [3.0, 7.0, 2.0]], [[7.0, 1.0, 
7.0], [7.0, 4.0, 8.0], [1.0, 6.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 4.0, 4.0], [5.0, 2.0, 3.0], [1.0, 3.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 5.0, 1.0], [1.0, 6.0, 7.0], [6.0, 5.0, 2.0]]))
      arg[:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 4.0], [1.0, 4.0, 1.0], [8.0, 4.0, 1.0]], [[6.0, 1.0, 3.0], [1.0, 4.0, 4.0], [8.0, 
3.0, 5.0]], [[1.0, 1.0, 4.0], [1.0, 8.0, 5.0], [6.0, 5.0, 7.0]]], [[[4.0, 5.0, 2.0], [8.0, 3.0, 1.0], [2.0, 4.0, 5.0]], [[4.0, 
1.0, 4.0], [5.0, 2.0, 3.0], [5.0, 3.0, 1.0]], [[5.0, 2.0, 7.0], [2.0, 5.0, 8.0], [3.0, 8.0, 2.0]]], [[[6.0, 6.0, 7.0], [8.0, 
3.0, 4.0], [2.0, 1.0, 7.0]], [[2.0, 3.0, 1.0], [1.0, 3.0, 2.0], [1.0, 5.0, 8.0]], [[3.0, 8.0, 7.0], [4.0, 5.0, 3.0], [1.0, 1.0, 
3.0]]], [[[5.0, 2.0, 8.0], [3.0, 7.0, 2.0], [6.0, 7.0, 6.0]], [[2.0, 2.0, 2.0], [3.0, 3.0, 3.0], [3.0, 7.0, 2.0]], [[7.0, 1.0, 
7.0], [7.0, 4.0, 8.0], [1.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 4.0], [1.0, 4.0, 1.0], [8.0, 4.0, 1.0]], 
[[6.0, 1.0, 3.0], [4.0, 5.0, 1.0], [8.0, 3.0, 5.0]], [[1.0, 1.0, 4.0], [1.0, 8.0, 5.0], [6.0, 5.0, 7.0]]], [[[4.0, 5.0, 2.0], 
[8.0, 3.0, 1.0], [2.0, 4.0, 5.0]], [[4.0, 1.0, 4.0], [1.0, 6.0, 7.0], [5.0, 3.0, 1.0]], [[5.0, 2.0, 7.0], [2.0, 5.0, 8.0], 
[3.0, 8.0, 2.0]]], [[[6.0, 6.0, 7.0], [8.0, 3.0, 4.0], [2.0, 1.0, 7.0]], [[2.0, 3.0, 1.0], [6.0, 5.0, 2.0], [1.0, 5.0, 8.0]], 
[[3.0, 8.0, 7.0], [4.0, 5.0, 3.0], [1.0, 1.0, 3.0]]], [[[5.0, 2.0, 8.0], [3.0, 7.0, 2.0], [6.0, 7.0, 6.0]], [[2.0, 2.0, 2.0], 
[3.0, 3.0, 3.0], [3.0, 7.0, 2.0]], [[7.0, 1.0, 7.0], [7.0, 4.0, 8.0], [1.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RRCN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 1.0], [3.0, 6.0, 6.0], [3.0, 4.0, 7.0]], [[7.0, 6.0, 8.0], [3.0, 6.0, 4.0], [2.0, 1.0, 
3.0]], [[8.0, 6.0, 8.0], [7.0, 1.0, 5.0], [1.0, 2.0, 2.0]]], [[[1.0, 7.0, 3.0], [7.0, 6.0, 2.0], [7.0, 4.0, 4.0]], [[2.0, 6.0, 
5.0], [1.0, 7.0, 2.0], [8.0, 4.0, 1.0]], [[2.0, 7.0, 5.0], [8.0, 8.0, 7.0], [1.0, 6.0, 8.0]]], [[[3.0, 4.0, 8.0], [3.0, 8.0, 
5.0], [1.0, 7.0, 5.0]], [[6.0, 1.0, 2.0], [2.0, 4.0, 1.0], [6.0, 2.0, 5.0]], [[2.0, 7.0, 7.0], [4.0, 5.0, 7.0], [2.0, 3.0, 
3.0]]], [[[8.0, 3.0, 6.0], [3.0, 1.0, 4.0], [4.0, 3.0, 5.0]], [[3.0, 3.0, 8.0], [3.0, 1.0, 8.0], [5.0, 2.0, 3.0]], [[4.0, 7.0, 
4.0], [1.0, 3.0, 2.0], [5.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 6.0, 7.0], [4.0, 6.0, 6.0], [2.0, 6.0, 1.0]]], [[[3.0, 5.0, 3.0], [8.0, 2.0, 3.0], [3.0, 
5.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 3.0], [6.0, 7.0, 6.0], [7.0, 6.0, 3.0]]], [[[7.0, 7.0, 3.0], [3.0, 5.0, 
3.0], [6.0, 5.0, 2.0]]]]))
      arg[1:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 1.0], [3.0, 6.0, 6.0], [3.0, 4.0, 7.0]], [[7.0, 6.0, 8.0], [3.0, 6.0, 4.0], [2.0, 
1.0, 3.0]], [[8.0, 6.0, 8.0], [7.0, 1.0, 5.0], [1.0, 2.0, 2.0]]], [[[1.0, 7.0, 3.0], [7.0, 6.0, 2.0], [7.0, 4.0, 4.0]], [[3.0, 
6.0, 7.0], [4.0, 6.0, 6.0], [2.0, 6.0, 1.0]], [[2.0, 7.0, 5.0], [8.0, 8.0, 7.0], [1.0, 6.0, 8.0]]], [[[3.0, 4.0, 8.0], [3.0, 
8.0, 5.0], [1.0, 7.0, 5.0]], [[3.0, 5.0, 3.0], [8.0, 2.0, 3.0], [3.0, 5.0, 2.0]], [[2.0, 7.0, 7.0], [4.0, 5.0, 7.0], [2.0, 3.0, 
3.0]]], [[[8.0, 3.0, 6.0], [3.0, 1.0, 4.0], [4.0, 3.0, 5.0]], [[3.0, 3.0, 8.0], [3.0, 1.0, 8.0], [5.0, 2.0, 3.0]], [[4.0, 7.0, 
4.0], [1.0, 3.0, 2.0], [5.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 1.0], [3.0, 6.0, 6.0], [3.0, 4.0, 7.0]], 
[[7.0, 6.0, 8.0], [3.0, 6.0, 4.0], [2.0, 1.0, 3.0]], [[8.0, 6.0, 8.0], [7.0, 1.0, 5.0], [1.0, 2.0, 2.0]]], [[[1.0, 7.0, 3.0], 
[7.0, 6.0, 2.0], [7.0, 4.0, 4.0]], [[4.0, 7.0, 3.0], [6.0, 7.0, 6.0], [7.0, 6.0, 3.0]], [[2.0, 7.0, 5.0], [8.0, 8.0, 7.0], 
[1.0, 6.0, 8.0]]], [[[3.0, 4.0, 8.0], [3.0, 8.0, 5.0], [1.0, 7.0, 5.0]], [[7.0, 7.0, 3.0], [3.0, 5.0, 3.0], [6.0, 5.0, 2.0]], 
[[2.0, 7.0, 7.0], [4.0, 5.0, 7.0], [2.0, 3.0, 3.0]]], [[[8.0, 3.0, 6.0], [3.0, 1.0, 4.0], [4.0, 3.0, 5.0]], [[3.0, 3.0, 8.0], 
[3.0, 1.0, 8.0], [5.0, 2.0, 3.0]], [[4.0, 7.0, 4.0], [1.0, 3.0, 2.0], [5.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PPUN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 4.0], [5.0, 7.0, 3.0], [2.0, 8.0, 2.0]], [[5.0, 5.0, 6.0], [4.0, 1.0, 7.0], [4.0, 8.0, 
8.0]], [[5.0, 8.0, 3.0], [6.0, 3.0, 6.0], [1.0, 1.0, 2.0]]], [[[8.0, 7.0, 4.0], [4.0, 2.0, 4.0], [3.0, 8.0, 1.0]], [[5.0, 6.0, 
5.0], [6.0, 5.0, 4.0], [8.0, 8.0, 8.0]], [[2.0, 4.0, 3.0], [4.0, 8.0, 7.0], [1.0, 3.0, 4.0]]], [[[1.0, 5.0, 4.0], [5.0, 8.0, 
4.0], [6.0, 2.0, 8.0]], [[6.0, 1.0, 5.0], [2.0, 1.0, 6.0], [2.0, 6.0, 3.0]], [[5.0, 8.0, 7.0], [6.0, 6.0, 8.0], [2.0, 5.0, 
1.0]]], [[[3.0, 3.0, 3.0], [1.0, 8.0, 7.0], [7.0, 1.0, 8.0]], [[3.0, 4.0, 5.0], [2.0, 1.0, 6.0], [4.0, 1.0, 3.0]], [[7.0, 4.0, 
6.0], [8.0, 8.0, 7.0], [4.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 5.0, 6.0], [1.0, 7.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 1.0, 2.0], [8.0, 1.0, 3.0]]))
      arg[2,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 4.0], [5.0, 7.0, 3.0], [2.0, 8.0, 2.0]], [[5.0, 5.0, 6.0], [4.0, 1.0, 7.0], [4.0, 
8.0, 8.0]], [[5.0, 8.0, 3.0], [6.0, 3.0, 6.0], [1.0, 1.0, 2.0]]], [[[8.0, 7.0, 4.0], [4.0, 2.0, 4.0], [3.0, 8.0, 1.0]], [[5.0, 
6.0, 5.0], [6.0, 5.0, 4.0], [8.0, 8.0, 8.0]], [[2.0, 4.0, 3.0], [4.0, 8.0, 7.0], [1.0, 3.0, 4.0]]], [[[1.0, 5.0, 4.0], [5.0, 
8.0, 4.0], [6.0, 2.0, 8.0]], [[2.0, 5.0, 6.0], [1.0, 7.0, 4.0], [2.0, 6.0, 3.0]], [[5.0, 8.0, 7.0], [6.0, 6.0, 8.0], [2.0, 5.0, 
1.0]]], [[[3.0, 3.0, 3.0], [1.0, 8.0, 7.0], [7.0, 1.0, 8.0]], [[3.0, 4.0, 5.0], [2.0, 1.0, 6.0], [4.0, 1.0, 3.0]], [[7.0, 4.0, 
6.0], [8.0, 8.0, 7.0], [4.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 4.0], [5.0, 7.0, 3.0], [2.0, 8.0, 2.0]], 
[[5.0, 5.0, 6.0], [4.0, 1.0, 7.0], [4.0, 8.0, 8.0]], [[5.0, 8.0, 3.0], [6.0, 3.0, 6.0], [1.0, 1.0, 2.0]]], [[[8.0, 7.0, 4.0], 
[4.0, 2.0, 4.0], [3.0, 8.0, 1.0]], [[5.0, 6.0, 5.0], [6.0, 5.0, 4.0], [8.0, 8.0, 8.0]], [[2.0, 4.0, 3.0], [4.0, 8.0, 7.0], 
[1.0, 3.0, 4.0]]], [[[1.0, 5.0, 4.0], [5.0, 8.0, 4.0], [6.0, 2.0, 8.0]], [[1.0, 1.0, 2.0], [8.0, 1.0, 3.0], [2.0, 6.0, 3.0]], 
[[5.0, 8.0, 7.0], [6.0, 6.0, 8.0], [2.0, 5.0, 1.0]]], [[[3.0, 3.0, 3.0], [1.0, 8.0, 7.0], [7.0, 1.0, 8.0]], [[3.0, 4.0, 5.0], 
[2.0, 1.0, 6.0], [4.0, 1.0, 3.0]], [[7.0, 4.0, 6.0], [8.0, 8.0, 7.0], [4.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LLCN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 6.0], [1.0, 3.0, 5.0], [6.0, 6.0, 4.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 6.0], [5.0, 3.0, 
6.0]], [[4.0, 3.0, 3.0], [7.0, 1.0, 8.0], [4.0, 1.0, 1.0]]], [[[7.0, 3.0, 5.0], [8.0, 7.0, 8.0], [4.0, 7.0, 3.0]], [[6.0, 7.0, 
8.0], [3.0, 7.0, 4.0], [1.0, 7.0, 5.0]], [[4.0, 6.0, 1.0], [7.0, 5.0, 4.0], [7.0, 7.0, 3.0]]], [[[2.0, 7.0, 5.0], [4.0, 7.0, 
1.0], [5.0, 1.0, 5.0]], [[8.0, 7.0, 2.0], [2.0, 4.0, 2.0], [2.0, 7.0, 2.0]], [[8.0, 7.0, 2.0], [5.0, 8.0, 4.0], [4.0, 5.0, 
3.0]]], [[[8.0, 8.0, 2.0], [2.0, 3.0, 1.0], [5.0, 1.0, 7.0]], [[8.0, 7.0, 5.0], [5.0, 4.0, 2.0], [7.0, 4.0, 1.0]], [[5.0, 6.0, 
2.0], [7.0, 4.0, 5.0], [7.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 2.0, 6.0], [5.0, 5.0, 5.0], [3.0, 1.0, 1.0]]], [[[2.0, 1.0, 1.0], [3.0, 1.0, 4.0], [4.0, 
7.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 5.0], [1.0, 5.0, 8.0], [5.0, 3.0, 8.0]]], [[[3.0, 1.0, 1.0], [1.0, 7.0, 
6.0], [4.0, 8.0, 5.0]]]]))
      arg[2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 6.0], [1.0, 3.0, 5.0], [6.0, 6.0, 4.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 6.0], [5.0, 
3.0, 6.0]], [[4.0, 3.0, 3.0], [7.0, 1.0, 8.0], [4.0, 1.0, 1.0]]], [[[7.0, 3.0, 5.0], [8.0, 7.0, 8.0], [4.0, 7.0, 3.0]], [[6.0, 
7.0, 8.0], [3.0, 7.0, 4.0], [1.0, 7.0, 5.0]], [[4.0, 6.0, 1.0], [7.0, 5.0, 4.0], [7.0, 7.0, 3.0]]], [[[2.0, 7.0, 5.0], [4.0, 
7.0, 1.0], [5.0, 1.0, 5.0]], [[8.0, 7.0, 2.0], [2.0, 4.0, 2.0], [2.0, 7.0, 2.0]], [[1.0, 2.0, 6.0], [5.0, 5.0, 5.0], [3.0, 1.0, 
1.0]]], [[[8.0, 8.0, 2.0], [2.0, 3.0, 1.0], [5.0, 1.0, 7.0]], [[8.0, 7.0, 5.0], [5.0, 4.0, 2.0], [7.0, 4.0, 1.0]], [[2.0, 1.0, 
1.0], [3.0, 1.0, 4.0], [4.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 6.0], [1.0, 3.0, 5.0], [6.0, 6.0, 4.0]], 
[[7.0, 7.0, 8.0], [1.0, 3.0, 6.0], [5.0, 3.0, 6.0]], [[4.0, 3.0, 3.0], [7.0, 1.0, 8.0], [4.0, 1.0, 1.0]]], [[[7.0, 3.0, 5.0], 
[8.0, 7.0, 8.0], [4.0, 7.0, 3.0]], [[6.0, 7.0, 8.0], [3.0, 7.0, 4.0], [1.0, 7.0, 5.0]], [[4.0, 6.0, 1.0], [7.0, 5.0, 4.0], 
[7.0, 7.0, 3.0]]], [[[2.0, 7.0, 5.0], [4.0, 7.0, 1.0], [5.0, 1.0, 5.0]], [[8.0, 7.0, 2.0], [2.0, 4.0, 2.0], [2.0, 7.0, 2.0]], 
[[8.0, 7.0, 5.0], [1.0, 5.0, 8.0], [5.0, 3.0, 8.0]]], [[[8.0, 8.0, 2.0], [2.0, 3.0, 1.0], [5.0, 1.0, 7.0]], [[8.0, 7.0, 5.0], 
[5.0, 4.0, 2.0], [7.0, 4.0, 1.0]], [[3.0, 1.0, 1.0], [1.0, 7.0, 6.0], [4.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCRN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 7.0], [1.0, 1.0, 1.0], [8.0, 3.0, 7.0]], [[7.0, 2.0, 7.0], [6.0, 8.0, 1.0], [8.0, 5.0, 
2.0]], [[8.0, 3.0, 4.0], [1.0, 5.0, 2.0], [1.0, 1.0, 6.0]]], [[[8.0, 7.0, 1.0], [7.0, 8.0, 5.0], [6.0, 8.0, 1.0]], [[4.0, 1.0, 
2.0], [8.0, 1.0, 5.0], [7.0, 8.0, 4.0]], [[1.0, 5.0, 3.0], [5.0, 7.0, 5.0], [4.0, 7.0, 4.0]]], [[[5.0, 5.0, 7.0], [2.0, 2.0, 
7.0], [5.0, 8.0, 1.0]], [[1.0, 2.0, 2.0], [6.0, 4.0, 4.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 2.0], [4.0, 5.0, 3.0], [6.0, 5.0, 
2.0]]], [[[4.0, 5.0, 8.0], [4.0, 8.0, 2.0], [3.0, 1.0, 5.0]], [[2.0, 6.0, 2.0], [2.0, 6.0, 2.0], [5.0, 7.0, 8.0]], [[5.0, 6.0, 
5.0], [4.0, 8.0, 1.0], [4.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 8.0, 6.0]], [[3.0, 3.0, 6.0]], [[4.0, 4.0, 3.0]]], [[[8.0, 7.0, 1.0]], [[6.0, 3.0, 7.0]], 
[[6.0, 4.0, 2.0]]], [[[7.0, 8.0, 8.0]], [[4.0, 2.0, 8.0]], [[7.0, 4.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 1.0]], [[7.0, 4.0, 5.0]], [[8.0, 1.0, 8.0]]], [[[5.0, 4.0, 8.0]], [[6.0, 
2.0, 4.0]], [[4.0, 7.0, 8.0]]], [[[7.0, 2.0, 6.0]], [[2.0, 1.0, 8.0]], [[3.0, 4.0, 2.0]]]]))
      arg[:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 7.0], [5.0, 8.0, 6.0], [8.0, 3.0, 7.0]], [[7.0, 2.0, 7.0], [3.0, 3.0, 6.0], [8.0, 
5.0, 2.0]], [[8.0, 3.0, 4.0], [4.0, 4.0, 3.0], [1.0, 1.0, 6.0]]], [[[8.0, 7.0, 1.0], [8.0, 7.0, 1.0], [6.0, 8.0, 1.0]], [[4.0, 
1.0, 2.0], [6.0, 3.0, 7.0], [7.0, 8.0, 4.0]], [[1.0, 5.0, 3.0], [6.0, 4.0, 2.0], [4.0, 7.0, 4.0]]], [[[5.0, 5.0, 7.0], [7.0, 
8.0, 8.0], [5.0, 8.0, 1.0]], [[1.0, 2.0, 2.0], [4.0, 2.0, 8.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 2.0], [7.0, 4.0, 5.0], [6.0, 5.0, 
2.0]]], [[[4.0, 5.0, 8.0], [4.0, 8.0, 2.0], [3.0, 1.0, 5.0]], [[2.0, 6.0, 2.0], [2.0, 6.0, 2.0], [5.0, 7.0, 8.0]], [[5.0, 6.0, 
5.0], [4.0, 8.0, 1.0], [4.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 7.0], [4.0, 5.0, 1.0], [8.0, 3.0, 7.0]], 
[[7.0, 2.0, 7.0], [7.0, 4.0, 5.0], [8.0, 5.0, 2.0]], [[8.0, 3.0, 4.0], [8.0, 1.0, 8.0], [1.0, 1.0, 6.0]]], [[[8.0, 7.0, 1.0], 
[5.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 1.0, 2.0], [6.0, 2.0, 4.0], [7.0, 8.0, 4.0]], [[1.0, 5.0, 3.0], [4.0, 7.0, 8.0], 
[4.0, 7.0, 4.0]]], [[[5.0, 5.0, 7.0], [7.0, 2.0, 6.0], [5.0, 8.0, 1.0]], [[1.0, 2.0, 2.0], [2.0, 1.0, 8.0], [5.0, 2.0, 5.0]], 
[[6.0, 4.0, 2.0], [3.0, 4.0, 2.0], [6.0, 5.0, 2.0]]], [[[4.0, 5.0, 8.0], [4.0, 8.0, 2.0], [3.0, 1.0, 5.0]], [[2.0, 6.0, 2.0], 
[2.0, 6.0, 2.0], [5.0, 7.0, 8.0]], [[5.0, 6.0, 5.0], [4.0, 8.0, 1.0], [4.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UPLN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 5.0], [6.0, 6.0, 2.0], [6.0, 7.0, 6.0]], [[8.0, 6.0, 6.0], [6.0, 6.0, 8.0], [5.0, 7.0, 
1.0]], [[1.0, 6.0, 6.0], [5.0, 2.0, 2.0], [3.0, 8.0, 7.0]]], [[[6.0, 5.0, 3.0], [5.0, 7.0, 1.0], [5.0, 5.0, 3.0]], [[4.0, 1.0, 
3.0], [5.0, 1.0, 8.0], [7.0, 7.0, 6.0]], [[6.0, 2.0, 7.0], [7.0, 7.0, 3.0], [8.0, 3.0, 1.0]]], [[[4.0, 6.0, 6.0], [1.0, 4.0, 
6.0], [3.0, 6.0, 4.0]], [[2.0, 7.0, 7.0], [2.0, 7.0, 3.0], [2.0, 1.0, 6.0]], [[2.0, 1.0, 3.0], [3.0, 4.0, 7.0], [3.0, 1.0, 
2.0]]], [[[6.0, 5.0, 1.0], [8.0, 6.0, 2.0], [1.0, 6.0, 8.0]], [[6.0, 8.0, 2.0], [5.0, 8.0, 5.0], [3.0, 2.0, 4.0]], [[6.0, 4.0, 
3.0], [8.0, 7.0, 5.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 2.0, 4.0]], [[8.0, 2.0, 7.0]], [[3.0, 5.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 3.0, 7.0]], [[6.0, 5.0, 4.0]], [[3.0, 8.0, 4.0]]]))
      arg[:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 5.0], [6.0, 6.0, 2.0], [6.0, 7.0, 6.0]], [[8.0, 6.0, 6.0], [6.0, 6.0, 8.0], [7.0, 
2.0, 4.0]], [[1.0, 6.0, 6.0], [5.0, 2.0, 2.0], [3.0, 8.0, 7.0]]], [[[6.0, 5.0, 3.0], [5.0, 7.0, 1.0], [5.0, 5.0, 3.0]], [[4.0, 
1.0, 3.0], [5.0, 1.0, 8.0], [8.0, 2.0, 7.0]], [[6.0, 2.0, 7.0], [7.0, 7.0, 3.0], [8.0, 3.0, 1.0]]], [[[4.0, 6.0, 6.0], [1.0, 
4.0, 6.0], [3.0, 6.0, 4.0]], [[2.0, 7.0, 7.0], [2.0, 7.0, 3.0], [3.0, 5.0, 3.0]], [[2.0, 1.0, 3.0], [3.0, 4.0, 7.0], [3.0, 1.0, 
2.0]]], [[[6.0, 5.0, 1.0], [8.0, 6.0, 2.0], [1.0, 6.0, 8.0]], [[6.0, 8.0, 2.0], [5.0, 8.0, 5.0], [3.0, 2.0, 4.0]], [[6.0, 4.0, 
3.0], [8.0, 7.0, 5.0], [7.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 5.0], [6.0, 6.0, 2.0], [6.0, 7.0, 6.0]], 
[[8.0, 6.0, 6.0], [6.0, 6.0, 8.0], [1.0, 3.0, 7.0]], [[1.0, 6.0, 6.0], [5.0, 2.0, 2.0], [3.0, 8.0, 7.0]]], [[[6.0, 5.0, 3.0], 
[5.0, 7.0, 1.0], [5.0, 5.0, 3.0]], [[4.0, 1.0, 3.0], [5.0, 1.0, 8.0], [6.0, 5.0, 4.0]], [[6.0, 2.0, 7.0], [7.0, 7.0, 3.0], 
[8.0, 3.0, 1.0]]], [[[4.0, 6.0, 6.0], [1.0, 4.0, 6.0], [3.0, 6.0, 4.0]], [[2.0, 7.0, 7.0], [2.0, 7.0, 3.0], [3.0, 8.0, 4.0]], 
[[2.0, 1.0, 3.0], [3.0, 4.0, 7.0], [3.0, 1.0, 2.0]]], [[[6.0, 5.0, 1.0], [8.0, 6.0, 2.0], [1.0, 6.0, 8.0]], [[6.0, 8.0, 2.0], 
[5.0, 8.0, 5.0], [3.0, 2.0, 4.0]], [[6.0, 4.0, 3.0], [8.0, 7.0, 5.0], [7.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PRRR(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 3.0], [2.0, 2.0, 3.0], [1.0, 3.0, 4.0]], [[2.0, 3.0, 1.0], [6.0, 1.0, 7.0], [4.0, 5.0, 
3.0]], [[3.0, 7.0, 2.0], [4.0, 5.0, 4.0], [6.0, 4.0, 6.0]]], [[[4.0, 3.0, 8.0], [7.0, 1.0, 4.0], [1.0, 6.0, 1.0]], [[5.0, 2.0, 
4.0], [8.0, 5.0, 7.0], [1.0, 4.0, 6.0]], [[2.0, 6.0, 5.0], [2.0, 2.0, 1.0], [3.0, 8.0, 1.0]]], [[[6.0, 7.0, 3.0], [6.0, 3.0, 
7.0], [8.0, 5.0, 6.0]], [[5.0, 5.0, 4.0], [4.0, 7.0, 1.0], [1.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 1.0], [2.0, 6.0, 
8.0]]], [[[6.0, 8.0, 6.0], [5.0, 3.0, 7.0], [7.0, 5.0, 1.0]], [[4.0, 7.0, 4.0], [5.0, 4.0, 8.0], [6.0, 8.0, 8.0]], [[5.0, 4.0, 
7.0], [7.0, 8.0, 7.0], [8.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0]]]))
      arg[2,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 3.0], [2.0, 2.0, 3.0], [1.0, 3.0, 4.0]], [[2.0, 3.0, 1.0], [6.0, 1.0, 7.0], [4.0, 
5.0, 3.0]], [[3.0, 7.0, 2.0], [4.0, 5.0, 4.0], [6.0, 4.0, 6.0]]], [[[4.0, 3.0, 8.0], [7.0, 1.0, 4.0], [1.0, 6.0, 1.0]], [[5.0, 
2.0, 4.0], [8.0, 5.0, 7.0], [1.0, 4.0, 6.0]], [[2.0, 6.0, 5.0], [2.0, 2.0, 1.0], [3.0, 8.0, 1.0]]], [[[6.0, 7.0, 3.0], [6.0, 
3.0, 7.0], [8.0, 5.0, 6.0]], [[5.0, 5.0, 4.0], [4.0, 5.0, 1.0], [1.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 1.0], [2.0, 6.0, 
8.0]]], [[[6.0, 8.0, 6.0], [5.0, 3.0, 7.0], [7.0, 5.0, 1.0]], [[4.0, 7.0, 4.0], [5.0, 4.0, 8.0], [6.0, 8.0, 8.0]], [[5.0, 4.0, 
7.0], [7.0, 8.0, 7.0], [8.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 3.0], [2.0, 2.0, 3.0], [1.0, 3.0, 4.0]], 
[[2.0, 3.0, 1.0], [6.0, 1.0, 7.0], [4.0, 5.0, 3.0]], [[3.0, 7.0, 2.0], [4.0, 5.0, 4.0], [6.0, 4.0, 6.0]]], [[[4.0, 3.0, 8.0], 
[7.0, 1.0, 4.0], [1.0, 6.0, 1.0]], [[5.0, 2.0, 4.0], [8.0, 5.0, 7.0], [1.0, 4.0, 6.0]], [[2.0, 6.0, 5.0], [2.0, 2.0, 1.0], 
[3.0, 8.0, 1.0]]], [[[6.0, 7.0, 3.0], [6.0, 3.0, 7.0], [8.0, 5.0, 6.0]], [[5.0, 5.0, 4.0], [4.0, 4.0, 1.0], [1.0, 1.0, 2.0]], 
[[7.0, 8.0, 7.0], [3.0, 8.0, 1.0], [2.0, 6.0, 8.0]]], [[[6.0, 8.0, 6.0], [5.0, 3.0, 7.0], [7.0, 5.0, 1.0]], [[4.0, 7.0, 4.0], 
[5.0, 4.0, 8.0], [6.0, 8.0, 8.0]], [[5.0, 4.0, 7.0], [7.0, 8.0, 7.0], [8.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LCPN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 6.0], [3.0, 6.0, 5.0], [2.0, 1.0, 2.0]], [[2.0, 2.0, 4.0], [7.0, 8.0, 3.0], [2.0, 3.0, 
6.0]], [[2.0, 1.0, 6.0], [6.0, 8.0, 3.0], [4.0, 8.0, 1.0]]], [[[8.0, 6.0, 3.0], [1.0, 5.0, 6.0], [5.0, 6.0, 6.0]], [[5.0, 7.0, 
5.0], [6.0, 3.0, 2.0], [7.0, 8.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 8.0, 8.0], [3.0, 8.0, 5.0]]], [[[2.0, 7.0, 3.0], [7.0, 8.0, 
6.0], [3.0, 6.0, 3.0]], [[7.0, 3.0, 3.0], [8.0, 6.0, 5.0], [1.0, 7.0, 1.0]], [[5.0, 2.0, 7.0], [5.0, 8.0, 1.0], [5.0, 8.0, 
2.0]]], [[[5.0, 6.0, 2.0], [4.0, 3.0, 2.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 5.0], [1.0, 7.0, 5.0], [4.0, 7.0, 2.0]], [[7.0, 7.0, 
6.0], [8.0, 6.0, 1.0], [4.0, 7.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 2.0, 2.0], [5.0, 8.0, 5.0], [2.0, 3.0, 2.0]], [[7.0, 1.0, 2.0], [4.0, 4.0, 2.0], [7.0, 6.0, 
6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 1.0, 1.0], [3.0, 8.0, 5.0], [6.0, 2.0, 3.0]], [[2.0, 1.0, 3.0], [1.0, 3.0, 5.0], 
[5.0, 7.0, 7.0]]]))
      arg[2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 6.0], [3.0, 6.0, 5.0], [2.0, 1.0, 2.0]], [[2.0, 2.0, 4.0], [7.0, 8.0, 3.0], [2.0, 
3.0, 6.0]], [[2.0, 1.0, 6.0], [6.0, 8.0, 3.0], [4.0, 8.0, 1.0]]], [[[8.0, 6.0, 3.0], [1.0, 5.0, 6.0], [5.0, 6.0, 6.0]], [[5.0, 
7.0, 5.0], [6.0, 3.0, 2.0], [7.0, 8.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 8.0, 8.0], [3.0, 8.0, 5.0]]], [[[2.0, 7.0, 3.0], [4.0, 
2.0, 2.0], [3.0, 6.0, 3.0]], [[7.0, 3.0, 3.0], [5.0, 8.0, 5.0], [1.0, 7.0, 1.0]], [[5.0, 2.0, 7.0], [2.0, 3.0, 2.0], [5.0, 8.0, 
2.0]]], [[[5.0, 6.0, 2.0], [7.0, 1.0, 2.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 5.0], [4.0, 4.0, 2.0], [4.0, 7.0, 2.0]], [[7.0, 7.0, 
6.0], [7.0, 6.0, 6.0], [4.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 6.0], [3.0, 6.0, 5.0], [2.0, 1.0, 2.0]], 
[[2.0, 2.0, 4.0], [7.0, 8.0, 3.0], [2.0, 3.0, 6.0]], [[2.0, 1.0, 6.0], [6.0, 8.0, 3.0], [4.0, 8.0, 1.0]]], [[[8.0, 6.0, 3.0], 
[1.0, 5.0, 6.0], [5.0, 6.0, 6.0]], [[5.0, 7.0, 5.0], [6.0, 3.0, 2.0], [7.0, 8.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 8.0, 8.0], 
[3.0, 8.0, 5.0]]], [[[2.0, 7.0, 3.0], [8.0, 1.0, 1.0], [3.0, 6.0, 3.0]], [[7.0, 3.0, 3.0], [3.0, 8.0, 5.0], [1.0, 7.0, 1.0]], 
[[5.0, 2.0, 7.0], [6.0, 2.0, 3.0], [5.0, 8.0, 2.0]]], [[[5.0, 6.0, 2.0], [2.0, 1.0, 3.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 5.0], 
[1.0, 3.0, 5.0], [4.0, 7.0, 2.0]], [[7.0, 7.0, 6.0], [5.0, 7.0, 7.0], [4.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LRRN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 2.0], [4.0, 5.0, 2.0], [3.0, 3.0, 8.0]], [[7.0, 5.0, 1.0], [4.0, 2.0, 2.0], [2.0, 4.0, 
5.0]], [[4.0, 7.0, 6.0], [7.0, 4.0, 1.0], [8.0, 2.0, 7.0]]], [[[4.0, 6.0, 3.0], [8.0, 5.0, 8.0], [8.0, 5.0, 5.0]], [[4.0, 4.0, 
1.0], [3.0, 1.0, 4.0], [2.0, 4.0, 2.0]], [[1.0, 2.0, 7.0], [2.0, 8.0, 7.0], [2.0, 2.0, 8.0]]], [[[4.0, 2.0, 8.0], [4.0, 7.0, 
2.0], [8.0, 1.0, 7.0]], [[4.0, 4.0, 1.0], [1.0, 5.0, 6.0], [3.0, 2.0, 8.0]], [[2.0, 5.0, 3.0], [8.0, 8.0, 8.0], [3.0, 2.0, 
3.0]]], [[[8.0, 1.0, 4.0], [8.0, 2.0, 6.0], [6.0, 6.0, 8.0]], [[7.0, 3.0, 6.0], [1.0, 6.0, 7.0], [2.0, 2.0, 3.0]], [[2.0, 6.0, 
3.0], [3.0, 2.0, 5.0], [1.0, 6.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 7.0, 5.0]]], [[[8.0, 2.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 8.0]]], [[[1.0, 2.0, 3.0]]]]))
      arg[2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 2.0], [4.0, 5.0, 2.0], [3.0, 3.0, 8.0]], [[7.0, 5.0, 1.0], [4.0, 2.0, 2.0], [2.0, 
4.0, 5.0]], [[4.0, 7.0, 6.0], [7.0, 4.0, 1.0], [8.0, 2.0, 7.0]]], [[[4.0, 6.0, 3.0], [8.0, 5.0, 8.0], [8.0, 5.0, 5.0]], [[4.0, 
4.0, 1.0], [3.0, 1.0, 4.0], [2.0, 4.0, 2.0]], [[1.0, 2.0, 7.0], [2.0, 8.0, 7.0], [2.0, 2.0, 8.0]]], [[[4.0, 2.0, 8.0], [4.0, 
7.0, 2.0], [8.0, 1.0, 7.0]], [[4.0, 4.0, 1.0], [4.0, 7.0, 5.0], [3.0, 2.0, 8.0]], [[2.0, 5.0, 3.0], [8.0, 8.0, 8.0], [3.0, 2.0, 
3.0]]], [[[8.0, 1.0, 4.0], [8.0, 2.0, 6.0], [6.0, 6.0, 8.0]], [[7.0, 3.0, 6.0], [8.0, 2.0, 1.0], [2.0, 2.0, 3.0]], [[2.0, 6.0, 
3.0], [3.0, 2.0, 5.0], [1.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 2.0], [4.0, 5.0, 2.0], [3.0, 3.0, 8.0]], 
[[7.0, 5.0, 1.0], [4.0, 2.0, 2.0], [2.0, 4.0, 5.0]], [[4.0, 7.0, 6.0], [7.0, 4.0, 1.0], [8.0, 2.0, 7.0]]], [[[4.0, 6.0, 3.0], 
[8.0, 5.0, 8.0], [8.0, 5.0, 5.0]], [[4.0, 4.0, 1.0], [3.0, 1.0, 4.0], [2.0, 4.0, 2.0]], [[1.0, 2.0, 7.0], [2.0, 8.0, 7.0], 
[2.0, 2.0, 8.0]]], [[[4.0, 2.0, 8.0], [4.0, 7.0, 2.0], [8.0, 1.0, 7.0]], [[4.0, 4.0, 1.0], [2.0, 8.0, 8.0], [3.0, 2.0, 8.0]], 
[[2.0, 5.0, 3.0], [8.0, 8.0, 8.0], [3.0, 2.0, 3.0]]], [[[8.0, 1.0, 4.0], [8.0, 2.0, 6.0], [6.0, 6.0, 8.0]], [[7.0, 3.0, 6.0], 
[1.0, 2.0, 3.0], [2.0, 2.0, 3.0]], [[2.0, 6.0, 3.0], [3.0, 2.0, 5.0], [1.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CUCU(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 3.0], [3.0, 7.0, 4.0], [5.0, 8.0, 3.0]], [[8.0, 6.0, 2.0], [2.0, 8.0, 8.0], [3.0, 1.0, 
1.0]], [[7.0, 5.0, 8.0], [2.0, 1.0, 4.0], [1.0, 8.0, 4.0]]], [[[6.0, 5.0, 5.0], [2.0, 6.0, 6.0], [1.0, 6.0, 6.0]], [[2.0, 2.0, 
4.0], [5.0, 6.0, 2.0], [7.0, 1.0, 3.0]], [[6.0, 8.0, 5.0], [4.0, 3.0, 3.0], [3.0, 8.0, 2.0]]], [[[1.0, 6.0, 5.0], [5.0, 2.0, 
2.0], [3.0, 2.0, 8.0]], [[7.0, 5.0, 3.0], [5.0, 5.0, 8.0], [4.0, 5.0, 5.0]], [[6.0, 6.0, 6.0], [2.0, 8.0, 8.0], [2.0, 4.0, 
5.0]]], [[[6.0, 5.0, 7.0], [3.0, 2.0, 1.0], [2.0, 5.0, 2.0]], [[1.0, 7.0, 5.0], [8.0, 8.0, 1.0], [7.0, 5.0, 3.0]], [[8.0, 1.0, 
3.0], [3.0, 8.0, 6.0], [7.0, 4.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 2.0], [2.0, 3.0], [2.0, 8.0]], [[7.0, 3.0], [5.0, 6.0], [2.0, 1.0]]], [[[1.0, 2.0], [4.0, 
1.0], [7.0, 4.0]], [[5.0, 5.0], [4.0, 5.0], [3.0, 7.0]]], [[[7.0, 7.0], [1.0, 6.0], [6.0, 2.0]], [[1.0, 4.0], [8.0, 1.0], [5.0, 
7.0]]], [[[4.0, 7.0], [5.0, 1.0], [3.0, 3.0]], [[1.0, 4.0], [7.0, 4.0], [4.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 2.0], [3.0, 3.0], [3.0, 5.0]], [[3.0, 2.0], [1.0, 1.0], [7.0, 7.0]]], [[[5.0, 
6.0], [6.0, 1.0], [5.0, 4.0]], [[7.0, 2.0], [6.0, 4.0], [3.0, 4.0]]], [[[8.0, 7.0], [8.0, 8.0], [6.0, 1.0]], [[8.0, 2.0], [3.0, 
7.0], [4.0, 7.0]]], [[[4.0, 3.0], [3.0, 6.0], [5.0, 6.0]], [[4.0, 6.0], [7.0, 6.0], [2.0, 1.0]]]]))
      arg[:,:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 3.0], [2.0, 3.0, 4.0], [2.0, 8.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 6.0, 8.0], [2.0, 
1.0, 1.0]], [[7.0, 5.0, 8.0], [2.0, 1.0, 4.0], [1.0, 8.0, 4.0]]], [[[1.0, 2.0, 5.0], [4.0, 1.0, 6.0], [7.0, 4.0, 6.0]], [[5.0, 
5.0, 4.0], [4.0, 5.0, 2.0], [3.0, 7.0, 3.0]], [[6.0, 8.0, 5.0], [4.0, 3.0, 3.0], [3.0, 8.0, 2.0]]], [[[7.0, 7.0, 5.0], [1.0, 
6.0, 2.0], [6.0, 2.0, 8.0]], [[1.0, 4.0, 3.0], [8.0, 1.0, 8.0], [5.0, 7.0, 5.0]], [[6.0, 6.0, 6.0], [2.0, 8.0, 8.0], [2.0, 4.0, 
5.0]]], [[[4.0, 7.0, 7.0], [5.0, 1.0, 1.0], [3.0, 3.0, 2.0]], [[1.0, 4.0, 5.0], [7.0, 4.0, 1.0], [4.0, 7.0, 3.0]], [[8.0, 1.0, 
3.0], [3.0, 8.0, 6.0], [7.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 3.0], [3.0, 3.0, 4.0], [3.0, 5.0, 3.0]], 
[[3.0, 2.0, 2.0], [1.0, 1.0, 8.0], [7.0, 7.0, 1.0]], [[7.0, 5.0, 8.0], [2.0, 1.0, 4.0], [1.0, 8.0, 4.0]]], [[[5.0, 6.0, 5.0], 
[6.0, 1.0, 6.0], [5.0, 4.0, 6.0]], [[7.0, 2.0, 4.0], [6.0, 4.0, 2.0], [3.0, 4.0, 3.0]], [[6.0, 8.0, 5.0], [4.0, 3.0, 3.0], 
[3.0, 8.0, 2.0]]], [[[8.0, 7.0, 5.0], [8.0, 8.0, 2.0], [6.0, 1.0, 8.0]], [[8.0, 2.0, 3.0], [3.0, 7.0, 8.0], [4.0, 7.0, 5.0]], 
[[6.0, 6.0, 6.0], [2.0, 8.0, 8.0], [2.0, 4.0, 5.0]]], [[[4.0, 3.0, 7.0], [3.0, 6.0, 1.0], [5.0, 6.0, 2.0]], [[4.0, 6.0, 5.0], 
[7.0, 6.0, 1.0], [2.0, 1.0, 3.0]], [[8.0, 1.0, 3.0], [3.0, 8.0, 6.0], [7.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCRN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 5.0], [3.0, 1.0, 8.0], [1.0, 8.0, 6.0]], [[8.0, 5.0, 7.0], [5.0, 6.0, 2.0], [8.0, 8.0, 
7.0]], [[7.0, 4.0, 5.0], [6.0, 5.0, 4.0], [5.0, 1.0, 5.0]]], [[[3.0, 5.0, 2.0], [1.0, 3.0, 3.0], [3.0, 3.0, 2.0]], [[1.0, 7.0, 
5.0], [4.0, 7.0, 1.0], [3.0, 5.0, 8.0]], [[5.0, 7.0, 5.0], [3.0, 4.0, 7.0], [1.0, 7.0, 6.0]]], [[[4.0, 4.0, 6.0], [1.0, 8.0, 
7.0], [6.0, 4.0, 8.0]], [[5.0, 4.0, 8.0], [2.0, 8.0, 2.0], [3.0, 3.0, 8.0]], [[4.0, 3.0, 8.0], [1.0, 6.0, 6.0], [8.0, 1.0, 
2.0]]], [[[7.0, 4.0, 5.0], [3.0, 8.0, 4.0], [8.0, 8.0, 5.0]], [[1.0, 8.0, 6.0], [4.0, 3.0, 2.0], [6.0, 4.0, 5.0]], [[8.0, 8.0, 
8.0], [2.0, 2.0, 7.0], [4.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 4.0, 3.0]], [[8.0, 3.0, 3.0]], [[2.0, 7.0, 3.0]]], [[[6.0, 6.0, 7.0]], [[7.0, 5.0, 6.0]], 
[[6.0, 1.0, 3.0]]], [[[6.0, 1.0, 1.0]], [[6.0, 8.0, 6.0]], [[4.0, 7.0, 6.0]]], [[[5.0, 3.0, 1.0]], [[3.0, 2.0, 4.0]], [[4.0, 
8.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 2.0]], [[8.0, 4.0, 4.0]], [[2.0, 5.0, 4.0]]], [[[5.0, 6.0, 2.0]], [[4.0, 
1.0, 4.0]], [[2.0, 4.0, 8.0]]], [[[2.0, 3.0, 7.0]], [[2.0, 8.0, 8.0]], [[4.0, 3.0, 6.0]]], [[[2.0, 2.0, 5.0]], [[4.0, 1.0, 
4.0]], [[8.0, 4.0, 1.0]]]]))
      arg[:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 5.0], [7.0, 4.0, 3.0], [1.0, 8.0, 6.0]], [[8.0, 5.0, 7.0], [8.0, 3.0, 3.0], [8.0, 
8.0, 7.0]], [[7.0, 4.0, 5.0], [2.0, 7.0, 3.0], [5.0, 1.0, 5.0]]], [[[3.0, 5.0, 2.0], [6.0, 6.0, 7.0], [3.0, 3.0, 2.0]], [[1.0, 
7.0, 5.0], [7.0, 5.0, 6.0], [3.0, 5.0, 8.0]], [[5.0, 7.0, 5.0], [6.0, 1.0, 3.0], [1.0, 7.0, 6.0]]], [[[4.0, 4.0, 6.0], [6.0, 
1.0, 1.0], [6.0, 4.0, 8.0]], [[5.0, 4.0, 8.0], [6.0, 8.0, 6.0], [3.0, 3.0, 8.0]], [[4.0, 3.0, 8.0], [4.0, 7.0, 6.0], [8.0, 1.0, 
2.0]]], [[[7.0, 4.0, 5.0], [5.0, 3.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 8.0, 6.0], [3.0, 2.0, 4.0], [6.0, 4.0, 5.0]], [[8.0, 8.0, 
8.0], [4.0, 8.0, 8.0], [4.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 5.0], [3.0, 5.0, 2.0], [1.0, 8.0, 6.0]], 
[[8.0, 5.0, 7.0], [8.0, 4.0, 4.0], [8.0, 8.0, 7.0]], [[7.0, 4.0, 5.0], [2.0, 5.0, 4.0], [5.0, 1.0, 5.0]]], [[[3.0, 5.0, 2.0], 
[5.0, 6.0, 2.0], [3.0, 3.0, 2.0]], [[1.0, 7.0, 5.0], [4.0, 1.0, 4.0], [3.0, 5.0, 8.0]], [[5.0, 7.0, 5.0], [2.0, 4.0, 8.0], 
[1.0, 7.0, 6.0]]], [[[4.0, 4.0, 6.0], [2.0, 3.0, 7.0], [6.0, 4.0, 8.0]], [[5.0, 4.0, 8.0], [2.0, 8.0, 8.0], [3.0, 3.0, 8.0]], 
[[4.0, 3.0, 8.0], [4.0, 3.0, 6.0], [8.0, 1.0, 2.0]]], [[[7.0, 4.0, 5.0], [2.0, 2.0, 5.0], [8.0, 8.0, 5.0]], [[1.0, 8.0, 6.0], 
[4.0, 1.0, 4.0], [6.0, 4.0, 5.0]], [[8.0, 8.0, 8.0], [8.0, 4.0, 1.0], [4.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PRLN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 7.0], [8.0, 8.0, 2.0], [2.0, 6.0, 2.0]], [[6.0, 6.0, 8.0], [6.0, 7.0, 4.0], [2.0, 3.0, 
7.0]], [[2.0, 5.0, 5.0], [3.0, 7.0, 7.0], [8.0, 4.0, 8.0]]], [[[5.0, 5.0, 6.0], [3.0, 2.0, 5.0], [7.0, 4.0, 2.0]], [[3.0, 1.0, 
5.0], [6.0, 2.0, 7.0], [2.0, 3.0, 3.0]], [[2.0, 1.0, 1.0], [7.0, 3.0, 2.0], [5.0, 8.0, 2.0]]], [[[2.0, 1.0, 1.0], [7.0, 6.0, 
5.0], [6.0, 6.0, 3.0]], [[6.0, 7.0, 6.0], [2.0, 2.0, 8.0], [2.0, 1.0, 1.0]], [[3.0, 8.0, 3.0], [1.0, 7.0, 1.0], [4.0, 4.0, 
2.0]]], [[[8.0, 4.0, 7.0], [5.0, 1.0, 5.0], [5.0, 1.0, 4.0]], [[4.0, 3.0, 6.0], [7.0, 2.0, 4.0], [1.0, 2.0, 2.0]], [[3.0, 5.0, 
3.0], [3.0, 7.0, 2.0], [7.0, 3.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 1.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 7.0, 1.0]]]))
      arg[2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 7.0], [8.0, 8.0, 2.0], [2.0, 6.0, 2.0]], [[6.0, 6.0, 8.0], [6.0, 7.0, 4.0], [2.0, 
3.0, 7.0]], [[2.0, 5.0, 5.0], [3.0, 7.0, 7.0], [8.0, 4.0, 8.0]]], [[[5.0, 5.0, 6.0], [3.0, 2.0, 5.0], [7.0, 4.0, 2.0]], [[3.0, 
1.0, 5.0], [6.0, 2.0, 7.0], [2.0, 3.0, 3.0]], [[2.0, 1.0, 1.0], [7.0, 3.0, 2.0], [5.0, 8.0, 2.0]]], [[[2.0, 1.0, 1.0], [7.0, 
6.0, 5.0], [6.0, 6.0, 3.0]], [[6.0, 7.0, 6.0], [2.0, 2.0, 8.0], [5.0, 1.0, 8.0]], [[3.0, 8.0, 3.0], [1.0, 7.0, 1.0], [4.0, 4.0, 
2.0]]], [[[8.0, 4.0, 7.0], [5.0, 1.0, 5.0], [5.0, 1.0, 4.0]], [[4.0, 3.0, 6.0], [7.0, 2.0, 4.0], [1.0, 2.0, 2.0]], [[3.0, 5.0, 
3.0], [3.0, 7.0, 2.0], [7.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 7.0], [8.0, 8.0, 2.0], [2.0, 6.0, 2.0]], 
[[6.0, 6.0, 8.0], [6.0, 7.0, 4.0], [2.0, 3.0, 7.0]], [[2.0, 5.0, 5.0], [3.0, 7.0, 7.0], [8.0, 4.0, 8.0]]], [[[5.0, 5.0, 6.0], 
[3.0, 2.0, 5.0], [7.0, 4.0, 2.0]], [[3.0, 1.0, 5.0], [6.0, 2.0, 7.0], [2.0, 3.0, 3.0]], [[2.0, 1.0, 1.0], [7.0, 3.0, 2.0], 
[5.0, 8.0, 2.0]]], [[[2.0, 1.0, 1.0], [7.0, 6.0, 5.0], [6.0, 6.0, 3.0]], [[6.0, 7.0, 6.0], [2.0, 2.0, 8.0], [6.0, 7.0, 1.0]], 
[[3.0, 8.0, 3.0], [1.0, 7.0, 1.0], [4.0, 4.0, 2.0]]], [[[8.0, 4.0, 7.0], [5.0, 1.0, 5.0], [5.0, 1.0, 4.0]], [[4.0, 3.0, 6.0], 
[7.0, 2.0, 4.0], [1.0, 2.0, 2.0]], [[3.0, 5.0, 3.0], [3.0, 7.0, 2.0], [7.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UCLN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 5.0], [4.0, 7.0, 8.0], [6.0, 2.0, 1.0]], [[4.0, 4.0, 2.0], [1.0, 6.0, 7.0], [4.0, 4.0, 
1.0]], [[4.0, 2.0, 1.0], [7.0, 8.0, 8.0], [5.0, 3.0, 1.0]]], [[[3.0, 4.0, 4.0], [2.0, 4.0, 3.0], [8.0, 2.0, 5.0]], [[6.0, 3.0, 
7.0], [1.0, 1.0, 2.0], [1.0, 1.0, 1.0]], [[3.0, 2.0, 6.0], [8.0, 1.0, 7.0], [6.0, 1.0, 7.0]]], [[[8.0, 3.0, 5.0], [7.0, 5.0, 
3.0], [8.0, 5.0, 1.0]], [[8.0, 5.0, 7.0], [8.0, 3.0, 1.0], [1.0, 5.0, 1.0]], [[1.0, 7.0, 5.0], [3.0, 4.0, 4.0], [3.0, 5.0, 
5.0]]], [[[1.0, 3.0, 8.0], [7.0, 7.0, 6.0], [1.0, 6.0, 3.0]], [[6.0, 4.0, 3.0], [8.0, 7.0, 3.0], [8.0, 3.0, 2.0]], [[2.0, 2.0, 
5.0], [1.0, 6.0, 6.0], [5.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 4.0, 1.0]], [[2.0, 2.0, 5.0]], [[4.0, 7.0, 3.0]]], [[[3.0, 3.0, 1.0]], [[1.0, 5.0, 5.0]], 
[[3.0, 4.0, 4.0]]], [[[6.0, 4.0, 3.0]], [[6.0, 7.0, 5.0]], [[5.0, 5.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 4.0]], [[5.0, 2.0, 2.0]], [[7.0, 4.0, 3.0]]], [[[7.0, 8.0, 7.0]], [[6.0, 
2.0, 4.0]], [[5.0, 4.0, 5.0]]], [[[7.0, 4.0, 1.0]], [[1.0, 6.0, 3.0]], [[7.0, 3.0, 5.0]]]]))
      arg[:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 5.0], [4.0, 7.0, 8.0], [6.0, 4.0, 1.0]], [[4.0, 4.0, 2.0], [1.0, 6.0, 7.0], [2.0, 
2.0, 5.0]], [[4.0, 2.0, 1.0], [7.0, 8.0, 8.0], [4.0, 7.0, 3.0]]], [[[3.0, 4.0, 4.0], [2.0, 4.0, 3.0], [3.0, 3.0, 1.0]], [[6.0, 
3.0, 7.0], [1.0, 1.0, 2.0], [1.0, 5.0, 5.0]], [[3.0, 2.0, 6.0], [8.0, 1.0, 7.0], [3.0, 4.0, 4.0]]], [[[8.0, 3.0, 5.0], [7.0, 
5.0, 3.0], [6.0, 4.0, 3.0]], [[8.0, 5.0, 7.0], [8.0, 3.0, 1.0], [6.0, 7.0, 5.0]], [[1.0, 7.0, 5.0], [3.0, 4.0, 4.0], [5.0, 5.0, 
8.0]]], [[[1.0, 3.0, 8.0], [7.0, 7.0, 6.0], [1.0, 6.0, 3.0]], [[6.0, 4.0, 3.0], [8.0, 7.0, 3.0], [8.0, 3.0, 2.0]], [[2.0, 2.0, 
5.0], [1.0, 6.0, 6.0], [5.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 5.0], [4.0, 7.0, 8.0], [1.0, 5.0, 4.0]], 
[[4.0, 4.0, 2.0], [1.0, 6.0, 7.0], [5.0, 2.0, 2.0]], [[4.0, 2.0, 1.0], [7.0, 8.0, 8.0], [7.0, 4.0, 3.0]]], [[[3.0, 4.0, 4.0], 
[2.0, 4.0, 3.0], [7.0, 8.0, 7.0]], [[6.0, 3.0, 7.0], [1.0, 1.0, 2.0], [6.0, 2.0, 4.0]], [[3.0, 2.0, 6.0], [8.0, 1.0, 7.0], 
[5.0, 4.0, 5.0]]], [[[8.0, 3.0, 5.0], [7.0, 5.0, 3.0], [7.0, 4.0, 1.0]], [[8.0, 5.0, 7.0], [8.0, 3.0, 1.0], [1.0, 6.0, 3.0]], 
[[1.0, 7.0, 5.0], [3.0, 4.0, 4.0], [7.0, 3.0, 5.0]]], [[[1.0, 3.0, 8.0], [7.0, 7.0, 6.0], [1.0, 6.0, 3.0]], [[6.0, 4.0, 3.0], 
[8.0, 7.0, 3.0], [8.0, 3.0, 2.0]], [[2.0, 2.0, 5.0], [1.0, 6.0, 6.0], [5.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLRN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 2.0], [4.0, 2.0, 6.0], [4.0, 3.0, 3.0]], [[3.0, 4.0, 6.0], [3.0, 7.0, 6.0], [4.0, 1.0, 
1.0]], [[8.0, 2.0, 5.0], [6.0, 7.0, 2.0], [3.0, 6.0, 5.0]]], [[[2.0, 7.0, 2.0], [3.0, 3.0, 7.0], [3.0, 7.0, 1.0]], [[1.0, 7.0, 
1.0], [7.0, 4.0, 5.0], [7.0, 2.0, 4.0]], [[2.0, 4.0, 3.0], [2.0, 5.0, 3.0], [6.0, 8.0, 1.0]]], [[[8.0, 4.0, 3.0], [2.0, 2.0, 
1.0], [7.0, 7.0, 3.0]], [[4.0, 7.0, 2.0], [8.0, 5.0, 4.0], [5.0, 4.0, 8.0]], [[7.0, 5.0, 8.0], [8.0, 2.0, 4.0], [8.0, 8.0, 
5.0]]], [[[2.0, 4.0, 3.0], [8.0, 6.0, 4.0], [8.0, 3.0, 5.0]], [[3.0, 4.0, 1.0], [7.0, 5.0, 3.0], [3.0, 7.0, 8.0]], [[2.0, 1.0, 
3.0], [3.0, 4.0, 8.0], [7.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 4.0, 2.0]]], [[[1.0, 1.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 3.0]]], [[[2.0, 7.0, 3.0]]]]))
      arg[1:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 2.0], [4.0, 2.0, 6.0], [4.0, 3.0, 3.0]], [[3.0, 4.0, 6.0], [3.0, 7.0, 6.0], [4.0, 
1.0, 1.0]], [[8.0, 2.0, 5.0], [6.0, 7.0, 2.0], [3.0, 6.0, 5.0]]], [[[2.0, 7.0, 2.0], [3.0, 3.0, 7.0], [3.0, 7.0, 1.0]], [[1.0, 
7.0, 1.0], [7.0, 4.0, 5.0], [7.0, 2.0, 4.0]], [[2.0, 4.0, 3.0], [6.0, 4.0, 2.0], [6.0, 8.0, 1.0]]], [[[8.0, 4.0, 3.0], [2.0, 
2.0, 1.0], [7.0, 7.0, 3.0]], [[4.0, 7.0, 2.0], [8.0, 5.0, 4.0], [5.0, 4.0, 8.0]], [[7.0, 5.0, 8.0], [1.0, 1.0, 1.0], [8.0, 8.0, 
5.0]]], [[[2.0, 4.0, 3.0], [8.0, 6.0, 4.0], [8.0, 3.0, 5.0]], [[3.0, 4.0, 1.0], [7.0, 5.0, 3.0], [3.0, 7.0, 8.0]], [[2.0, 1.0, 
3.0], [3.0, 4.0, 8.0], [7.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 2.0], [4.0, 2.0, 6.0], [4.0, 3.0, 3.0]], 
[[3.0, 4.0, 6.0], [3.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[8.0, 2.0, 5.0], [6.0, 7.0, 2.0], [3.0, 6.0, 5.0]]], [[[2.0, 7.0, 2.0], 
[3.0, 3.0, 7.0], [3.0, 7.0, 1.0]], [[1.0, 7.0, 1.0], [7.0, 4.0, 5.0], [7.0, 2.0, 4.0]], [[2.0, 4.0, 3.0], [5.0, 2.0, 3.0], 
[6.0, 8.0, 1.0]]], [[[8.0, 4.0, 3.0], [2.0, 2.0, 1.0], [7.0, 7.0, 3.0]], [[4.0, 7.0, 2.0], [8.0, 5.0, 4.0], [5.0, 4.0, 8.0]], 
[[7.0, 5.0, 8.0], [2.0, 7.0, 3.0], [8.0, 8.0, 5.0]]], [[[2.0, 4.0, 3.0], [8.0, 6.0, 4.0], [8.0, 3.0, 5.0]], [[3.0, 4.0, 1.0], 
[7.0, 5.0, 3.0], [3.0, 7.0, 8.0]], [[2.0, 1.0, 3.0], [3.0, 4.0, 8.0], [7.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RPRN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 8.0], [1.0, 4.0, 3.0], [5.0, 8.0, 1.0]], [[1.0, 3.0, 6.0], [7.0, 8.0, 4.0], [3.0, 3.0, 
2.0]], [[5.0, 2.0, 7.0], [5.0, 5.0, 1.0], [4.0, 7.0, 1.0]]], [[[6.0, 7.0, 1.0], [4.0, 8.0, 6.0], [5.0, 3.0, 3.0]], [[6.0, 8.0, 
3.0], [2.0, 7.0, 6.0], [7.0, 2.0, 6.0]], [[4.0, 7.0, 3.0], [1.0, 1.0, 4.0], [3.0, 8.0, 8.0]]], [[[4.0, 8.0, 6.0], [8.0, 2.0, 
8.0], [2.0, 1.0, 6.0]], [[2.0, 1.0, 1.0], [1.0, 5.0, 6.0], [4.0, 5.0, 3.0]], [[5.0, 7.0, 1.0], [3.0, 8.0, 3.0], [4.0, 7.0, 
8.0]]], [[[1.0, 6.0, 5.0], [8.0, 5.0, 7.0], [2.0, 8.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 2.0, 4.0], [4.0, 5.0, 8.0]], [[1.0, 1.0, 
3.0], [1.0, 8.0, 7.0], [5.0, 6.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 2.0, 5.0]], [[7.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 7.0, 2.0]], [[3.0, 4.0, 5.0]]]))
      arg[1:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 8.0], [1.0, 4.0, 3.0], [5.0, 8.0, 1.0]], [[1.0, 3.0, 6.0], [7.0, 8.0, 4.0], [3.0, 
3.0, 2.0]], [[5.0, 2.0, 7.0], [5.0, 5.0, 1.0], [4.0, 7.0, 1.0]]], [[[6.0, 7.0, 1.0], [4.0, 8.0, 6.0], [5.0, 3.0, 3.0]], [[6.0, 
8.0, 3.0], [8.0, 2.0, 5.0], [7.0, 2.0, 6.0]], [[4.0, 7.0, 3.0], [1.0, 1.0, 4.0], [3.0, 8.0, 8.0]]], [[[4.0, 8.0, 6.0], [8.0, 
2.0, 8.0], [2.0, 1.0, 6.0]], [[2.0, 1.0, 1.0], [7.0, 3.0, 1.0], [4.0, 5.0, 3.0]], [[5.0, 7.0, 1.0], [3.0, 8.0, 3.0], [4.0, 7.0, 
8.0]]], [[[1.0, 6.0, 5.0], [8.0, 5.0, 7.0], [2.0, 8.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 2.0, 4.0], [4.0, 5.0, 8.0]], [[1.0, 1.0, 
3.0], [1.0, 8.0, 7.0], [5.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 8.0], [1.0, 4.0, 3.0], [5.0, 8.0, 1.0]], 
[[1.0, 3.0, 6.0], [7.0, 8.0, 4.0], [3.0, 3.0, 2.0]], [[5.0, 2.0, 7.0], [5.0, 5.0, 1.0], [4.0, 7.0, 1.0]]], [[[6.0, 7.0, 1.0], 
[4.0, 8.0, 6.0], [5.0, 3.0, 3.0]], [[6.0, 8.0, 3.0], [1.0, 7.0, 2.0], [7.0, 2.0, 6.0]], [[4.0, 7.0, 3.0], [1.0, 1.0, 4.0], 
[3.0, 8.0, 8.0]]], [[[4.0, 8.0, 6.0], [8.0, 2.0, 8.0], [2.0, 1.0, 6.0]], [[2.0, 1.0, 1.0], [3.0, 4.0, 5.0], [4.0, 5.0, 3.0]], 
[[5.0, 7.0, 1.0], [3.0, 8.0, 3.0], [4.0, 7.0, 8.0]]], [[[1.0, 6.0, 5.0], [8.0, 5.0, 7.0], [2.0, 8.0, 4.0]], [[3.0, 1.0, 6.0], 
[5.0, 2.0, 4.0], [4.0, 5.0, 8.0]], [[1.0, 1.0, 3.0], [1.0, 8.0, 7.0], [5.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RUUN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 8.0], [4.0, 7.0, 8.0], [4.0, 7.0, 5.0]], [[4.0, 4.0, 4.0], [1.0, 4.0, 1.0], [4.0, 3.0, 
6.0]], [[2.0, 2.0, 4.0], [5.0, 5.0, 6.0], [6.0, 3.0, 5.0]]], [[[4.0, 7.0, 8.0], [6.0, 6.0, 7.0], [4.0, 5.0, 7.0]], [[8.0, 5.0, 
2.0], [3.0, 5.0, 5.0], [3.0, 5.0, 8.0]], [[6.0, 8.0, 2.0], [1.0, 1.0, 1.0], [4.0, 6.0, 7.0]]], [[[7.0, 4.0, 5.0], [7.0, 3.0, 
8.0], [8.0, 6.0, 8.0]], [[8.0, 2.0, 8.0], [7.0, 7.0, 2.0], [8.0, 7.0, 5.0]], [[8.0, 7.0, 3.0], [4.0, 8.0, 5.0], [7.0, 3.0, 
3.0]]], [[[1.0, 3.0, 3.0], [8.0, 7.0, 5.0], [6.0, 2.0, 7.0]], [[3.0, 4.0, 5.0], [8.0, 6.0, 8.0], [5.0, 8.0, 4.0]], [[2.0, 6.0, 
8.0], [8.0, 3.0, 2.0], [6.0, 7.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 7.0, 5.0], [8.0, 8.0, 4.0]], [[3.0, 8.0, 7.0], [3.0, 8.0, 8.0]]], [[[7.0, 5.0, 3.0], [4.0, 
8.0, 3.0]], [[7.0, 2.0, 2.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 4.0], [4.0, 1.0, 7.0]], [[7.0, 5.0, 3.0], [1.0, 5.0, 2.0]]], [[[2.0, 3.0, 
5.0], [7.0, 7.0, 1.0]], [[8.0, 3.0, 6.0], [1.0, 6.0, 6.0]]]]))
      arg[1:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 8.0], [4.0, 7.0, 8.0], [4.0, 7.0, 5.0]], [[4.0, 4.0, 4.0], [1.0, 4.0, 1.0], [4.0, 
3.0, 6.0]], [[2.0, 2.0, 4.0], [5.0, 5.0, 6.0], [6.0, 3.0, 5.0]]], [[[3.0, 7.0, 5.0], [8.0, 8.0, 4.0], [4.0, 5.0, 7.0]], [[3.0, 
8.0, 7.0], [3.0, 8.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 8.0, 2.0], [1.0, 1.0, 1.0], [4.0, 6.0, 7.0]]], [[[7.0, 5.0, 3.0], [4.0, 
8.0, 3.0], [8.0, 6.0, 8.0]], [[7.0, 2.0, 2.0], [4.0, 8.0, 8.0], [8.0, 7.0, 5.0]], [[8.0, 7.0, 3.0], [4.0, 8.0, 5.0], [7.0, 3.0, 
3.0]]], [[[1.0, 3.0, 3.0], [8.0, 7.0, 5.0], [6.0, 2.0, 7.0]], [[3.0, 4.0, 5.0], [8.0, 6.0, 8.0], [5.0, 8.0, 4.0]], [[2.0, 6.0, 
8.0], [8.0, 3.0, 2.0], [6.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 8.0], [4.0, 7.0, 8.0], [4.0, 7.0, 5.0]], 
[[4.0, 4.0, 4.0], [1.0, 4.0, 1.0], [4.0, 3.0, 6.0]], [[2.0, 2.0, 4.0], [5.0, 5.0, 6.0], [6.0, 3.0, 5.0]]], [[[6.0, 6.0, 4.0], 
[4.0, 1.0, 7.0], [4.0, 5.0, 7.0]], [[7.0, 5.0, 3.0], [1.0, 5.0, 2.0], [3.0, 5.0, 8.0]], [[6.0, 8.0, 2.0], [1.0, 1.0, 1.0], 
[4.0, 6.0, 7.0]]], [[[2.0, 3.0, 5.0], [7.0, 7.0, 1.0], [8.0, 6.0, 8.0]], [[8.0, 3.0, 6.0], [1.0, 6.0, 6.0], [8.0, 7.0, 5.0]], 
[[8.0, 7.0, 3.0], [4.0, 8.0, 5.0], [7.0, 3.0, 3.0]]], [[[1.0, 3.0, 3.0], [8.0, 7.0, 5.0], [6.0, 2.0, 7.0]], [[3.0, 4.0, 5.0], 
[8.0, 6.0, 8.0], [5.0, 8.0, 4.0]], [[2.0, 6.0, 8.0], [8.0, 3.0, 2.0], [6.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PPPP(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 6.0], [8.0, 2.0, 4.0], [3.0, 6.0, 3.0]], [[7.0, 2.0, 1.0], [4.0, 4.0, 8.0], [6.0, 7.0, 
2.0]], [[6.0, 3.0, 5.0], [6.0, 6.0, 6.0], [1.0, 3.0, 5.0]]], [[[2.0, 4.0, 3.0], [5.0, 1.0, 6.0], [4.0, 7.0, 5.0]], [[8.0, 8.0, 
5.0], [5.0, 3.0, 3.0], [3.0, 2.0, 3.0]], [[2.0, 3.0, 4.0], [8.0, 7.0, 4.0], [1.0, 3.0, 7.0]]], [[[4.0, 2.0, 3.0], [4.0, 3.0, 
2.0], [8.0, 3.0, 2.0]], [[8.0, 5.0, 1.0], [3.0, 7.0, 6.0], [4.0, 5.0, 1.0]], [[6.0, 6.0, 4.0], [6.0, 5.0, 1.0], [6.0, 2.0, 
1.0]]], [[[8.0, 2.0, 3.0], [1.0, 3.0, 3.0], [7.0, 2.0, 1.0]], [[6.0, 4.0, 4.0], [4.0, 1.0, 3.0], [3.0, 6.0, 4.0]], [[2.0, 8.0, 
1.0], [7.0, 5.0, 2.0], [4.0, 2.0, 4.0]]]]),self.functionspace)
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[2,1,1,1]=val
      ref=Data(numpy.array([[[[3.0, 3.0, 6.0], [8.0, 2.0, 4.0], [3.0, 6.0, 3.0]], [[7.0, 2.0, 1.0], [4.0, 4.0, 8.0], [6.0, 7.0, 
2.0]], [[6.0, 3.0, 5.0], [6.0, 6.0, 6.0], [1.0, 3.0, 5.0]]], [[[2.0, 4.0, 3.0], [5.0, 1.0, 6.0], [4.0, 7.0, 5.0]], [[8.0, 8.0, 
5.0], [5.0, 3.0, 3.0], [3.0, 2.0, 3.0]], [[2.0, 3.0, 4.0], [8.0, 7.0, 4.0], [1.0, 3.0, 7.0]]], [[[4.0, 2.0, 3.0], [4.0, 3.0, 
2.0], [8.0, 3.0, 2.0]], [[8.0, 5.0, 1.0], [3.0, 1.0, 6.0], [4.0, 5.0, 1.0]], [[6.0, 6.0, 4.0], [6.0, 5.0, 1.0], [6.0, 2.0, 
1.0]]], [[[8.0, 2.0, 3.0], [1.0, 3.0, 3.0], [7.0, 2.0, 1.0]], [[6.0, 4.0, 4.0], [4.0, 1.0, 3.0], [3.0, 6.0, 4.0]], [[2.0, 8.0, 
1.0], [7.0, 5.0, 2.0], [4.0, 2.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PPLN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 5.0], [6.0, 7.0, 7.0], [1.0, 1.0, 7.0]], [[3.0, 8.0, 2.0], [5.0, 1.0, 2.0], [3.0, 1.0, 
8.0]], [[5.0, 7.0, 4.0], [6.0, 5.0, 2.0], [8.0, 5.0, 2.0]]], [[[1.0, 7.0, 4.0], [5.0, 4.0, 7.0], [1.0, 2.0, 8.0]], [[4.0, 2.0, 
7.0], [4.0, 5.0, 4.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 5.0], [8.0, 4.0, 1.0], [3.0, 5.0, 3.0]]], [[[1.0, 6.0, 4.0], [2.0, 4.0, 
3.0], [3.0, 2.0, 7.0]], [[3.0, 3.0, 1.0], [5.0, 8.0, 7.0], [6.0, 5.0, 2.0]], [[3.0, 5.0, 2.0], [3.0, 6.0, 7.0], [7.0, 6.0, 
5.0]]], [[[1.0, 3.0, 3.0], [5.0, 3.0, 4.0], [2.0, 1.0, 2.0]], [[7.0, 1.0, 6.0], [3.0, 5.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 1.0, 
2.0], [3.0, 8.0, 8.0], [5.0, 1.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 4.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 8.0, 8.0]]))
      arg[2,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 5.0], [6.0, 7.0, 7.0], [1.0, 1.0, 7.0]], [[3.0, 8.0, 2.0], [5.0, 1.0, 2.0], [3.0, 
1.0, 8.0]], [[5.0, 7.0, 4.0], [6.0, 5.0, 2.0], [8.0, 5.0, 2.0]]], [[[1.0, 7.0, 4.0], [5.0, 4.0, 7.0], [1.0, 2.0, 8.0]], [[4.0, 
2.0, 7.0], [4.0, 5.0, 4.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 5.0], [8.0, 4.0, 1.0], [3.0, 5.0, 3.0]]], [[[1.0, 6.0, 4.0], [2.0, 
4.0, 3.0], [3.0, 2.0, 7.0]], [[3.0, 3.0, 1.0], [5.0, 8.0, 7.0], [3.0, 4.0, 7.0]], [[3.0, 5.0, 2.0], [3.0, 6.0, 7.0], [7.0, 6.0, 
5.0]]], [[[1.0, 3.0, 3.0], [5.0, 3.0, 4.0], [2.0, 1.0, 2.0]], [[7.0, 1.0, 6.0], [3.0, 5.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 1.0, 
2.0], [3.0, 8.0, 8.0], [5.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 5.0], [6.0, 7.0, 7.0], [1.0, 1.0, 7.0]], 
[[3.0, 8.0, 2.0], [5.0, 1.0, 2.0], [3.0, 1.0, 8.0]], [[5.0, 7.0, 4.0], [6.0, 5.0, 2.0], [8.0, 5.0, 2.0]]], [[[1.0, 7.0, 4.0], 
[5.0, 4.0, 7.0], [1.0, 2.0, 8.0]], [[4.0, 2.0, 7.0], [4.0, 5.0, 4.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 5.0], [8.0, 4.0, 1.0], 
[3.0, 5.0, 3.0]]], [[[1.0, 6.0, 4.0], [2.0, 4.0, 3.0], [3.0, 2.0, 7.0]], [[3.0, 3.0, 1.0], [5.0, 8.0, 7.0], [8.0, 8.0, 8.0]], 
[[3.0, 5.0, 2.0], [3.0, 6.0, 7.0], [7.0, 6.0, 5.0]]], [[[1.0, 3.0, 3.0], [5.0, 3.0, 4.0], [2.0, 1.0, 2.0]], [[7.0, 1.0, 6.0], 
[3.0, 5.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 1.0, 2.0], [3.0, 8.0, 8.0], [5.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RRPN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 7.0], [2.0, 7.0, 1.0], [4.0, 2.0, 4.0]], [[6.0, 6.0, 8.0], [6.0, 4.0, 8.0], [5.0, 6.0, 
2.0]], [[4.0, 6.0, 7.0], [2.0, 3.0, 5.0], [3.0, 6.0, 7.0]]], [[[6.0, 8.0, 2.0], [1.0, 6.0, 3.0], [5.0, 1.0, 2.0]], [[5.0, 5.0, 
7.0], [1.0, 7.0, 7.0], [3.0, 8.0, 4.0]], [[1.0, 1.0, 5.0], [1.0, 7.0, 5.0], [2.0, 2.0, 4.0]]], [[[6.0, 6.0, 4.0], [7.0, 7.0, 
6.0], [3.0, 8.0, 1.0]], [[3.0, 8.0, 6.0], [8.0, 6.0, 6.0], [3.0, 3.0, 3.0]], [[6.0, 4.0, 6.0], [6.0, 2.0, 3.0], [5.0, 1.0, 
1.0]]], [[[1.0, 3.0, 7.0], [5.0, 5.0, 3.0], [4.0, 5.0, 1.0]], [[4.0, 2.0, 5.0], [3.0, 4.0, 3.0], [1.0, 3.0, 8.0]], [[3.0, 7.0, 
3.0], [2.0, 7.0, 3.0], [5.0, 2.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 1.0, 5.0]], [[8.0, 3.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 3.0, 8.0]], [[2.0, 2.0, 2.0]]]))
      arg[1:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 7.0], [2.0, 7.0, 1.0], [4.0, 2.0, 4.0]], [[6.0, 6.0, 8.0], [6.0, 4.0, 8.0], [5.0, 
6.0, 2.0]], [[4.0, 6.0, 7.0], [2.0, 3.0, 5.0], [3.0, 6.0, 7.0]]], [[[6.0, 8.0, 2.0], [1.0, 6.0, 3.0], [5.0, 1.0, 2.0]], [[5.0, 
5.0, 7.0], [4.0, 1.0, 5.0], [3.0, 8.0, 4.0]], [[1.0, 1.0, 5.0], [1.0, 7.0, 5.0], [2.0, 2.0, 4.0]]], [[[6.0, 6.0, 4.0], [7.0, 
7.0, 6.0], [3.0, 8.0, 1.0]], [[3.0, 8.0, 6.0], [8.0, 3.0, 2.0], [3.0, 3.0, 3.0]], [[6.0, 4.0, 6.0], [6.0, 2.0, 3.0], [5.0, 1.0, 
1.0]]], [[[1.0, 3.0, 7.0], [5.0, 5.0, 3.0], [4.0, 5.0, 1.0]], [[4.0, 2.0, 5.0], [3.0, 4.0, 3.0], [1.0, 3.0, 8.0]], [[3.0, 7.0, 
3.0], [2.0, 7.0, 3.0], [5.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 7.0], [2.0, 7.0, 1.0], [4.0, 2.0, 4.0]], 
[[6.0, 6.0, 8.0], [6.0, 4.0, 8.0], [5.0, 6.0, 2.0]], [[4.0, 6.0, 7.0], [2.0, 3.0, 5.0], [3.0, 6.0, 7.0]]], [[[6.0, 8.0, 2.0], 
[1.0, 6.0, 3.0], [5.0, 1.0, 2.0]], [[5.0, 5.0, 7.0], [3.0, 3.0, 8.0], [3.0, 8.0, 4.0]], [[1.0, 1.0, 5.0], [1.0, 7.0, 5.0], 
[2.0, 2.0, 4.0]]], [[[6.0, 6.0, 4.0], [7.0, 7.0, 6.0], [3.0, 8.0, 1.0]], [[3.0, 8.0, 6.0], [2.0, 2.0, 2.0], [3.0, 3.0, 3.0]], 
[[6.0, 4.0, 6.0], [6.0, 2.0, 3.0], [5.0, 1.0, 1.0]]], [[[1.0, 3.0, 7.0], [5.0, 5.0, 3.0], [4.0, 5.0, 1.0]], [[4.0, 2.0, 5.0], 
[3.0, 4.0, 3.0], [1.0, 3.0, 8.0]], [[3.0, 7.0, 3.0], [2.0, 7.0, 3.0], [5.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LLPN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 4.0], [8.0, 8.0, 8.0], [3.0, 4.0, 1.0]], [[6.0, 3.0, 6.0], [5.0, 6.0, 8.0], [1.0, 6.0, 
2.0]], [[2.0, 8.0, 2.0], [1.0, 1.0, 7.0], [1.0, 8.0, 5.0]]], [[[6.0, 1.0, 5.0], [7.0, 2.0, 7.0], [2.0, 2.0, 1.0]], [[4.0, 6.0, 
5.0], [5.0, 8.0, 2.0], [2.0, 4.0, 5.0]], [[4.0, 1.0, 5.0], [3.0, 6.0, 6.0], [6.0, 8.0, 8.0]]], [[[4.0, 1.0, 7.0], [1.0, 5.0, 
8.0], [7.0, 1.0, 6.0]], [[1.0, 2.0, 3.0], [3.0, 2.0, 5.0], [5.0, 2.0, 4.0]], [[7.0, 3.0, 4.0], [2.0, 4.0, 4.0], [8.0, 4.0, 
6.0]]], [[[5.0, 3.0, 2.0], [3.0, 8.0, 8.0], [3.0, 2.0, 3.0]], [[4.0, 5.0, 3.0], [8.0, 2.0, 7.0], [2.0, 1.0, 5.0]], [[6.0, 1.0, 
7.0], [7.0, 5.0, 4.0], [8.0, 4.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 1.0, 5.0]], [[8.0, 5.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 4.0, 2.0]], [[4.0, 2.0, 3.0]]]))
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 4.0], [8.0, 8.0, 8.0], [3.0, 4.0, 1.0]], [[6.0, 3.0, 6.0], [5.0, 6.0, 8.0], [1.0, 
6.0, 2.0]], [[2.0, 8.0, 2.0], [1.0, 1.0, 7.0], [1.0, 8.0, 5.0]]], [[[6.0, 1.0, 5.0], [7.0, 2.0, 7.0], [2.0, 2.0, 1.0]], [[4.0, 
6.0, 5.0], [5.0, 8.0, 2.0], [2.0, 4.0, 5.0]], [[4.0, 1.0, 5.0], [3.0, 6.0, 6.0], [6.0, 8.0, 8.0]]], [[[4.0, 1.0, 7.0], [1.0, 
5.0, 8.0], [7.0, 1.0, 6.0]], [[1.0, 2.0, 3.0], [3.0, 2.0, 5.0], [5.0, 2.0, 4.0]], [[7.0, 3.0, 4.0], [6.0, 1.0, 5.0], [8.0, 4.0, 
6.0]]], [[[5.0, 3.0, 2.0], [3.0, 8.0, 8.0], [3.0, 2.0, 3.0]], [[4.0, 5.0, 3.0], [8.0, 2.0, 7.0], [2.0, 1.0, 5.0]], [[6.0, 1.0, 
7.0], [8.0, 5.0, 6.0], [8.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 4.0], [8.0, 8.0, 8.0], [3.0, 4.0, 1.0]], 
[[6.0, 3.0, 6.0], [5.0, 6.0, 8.0], [1.0, 6.0, 2.0]], [[2.0, 8.0, 2.0], [1.0, 1.0, 7.0], [1.0, 8.0, 5.0]]], [[[6.0, 1.0, 5.0], 
[7.0, 2.0, 7.0], [2.0, 2.0, 1.0]], [[4.0, 6.0, 5.0], [5.0, 8.0, 2.0], [2.0, 4.0, 5.0]], [[4.0, 1.0, 5.0], [3.0, 6.0, 6.0], 
[6.0, 8.0, 8.0]]], [[[4.0, 1.0, 7.0], [1.0, 5.0, 8.0], [7.0, 1.0, 6.0]], [[1.0, 2.0, 3.0], [3.0, 2.0, 5.0], [5.0, 2.0, 4.0]], 
[[7.0, 3.0, 4.0], [4.0, 4.0, 2.0], [8.0, 4.0, 6.0]]], [[[5.0, 3.0, 2.0], [3.0, 8.0, 8.0], [3.0, 2.0, 3.0]], [[4.0, 5.0, 3.0], 
[8.0, 2.0, 7.0], [2.0, 1.0, 5.0]], [[6.0, 1.0, 7.0], [4.0, 2.0, 3.0], [8.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LULN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 3.0], [7.0, 5.0, 7.0], [2.0, 1.0, 7.0]], [[3.0, 7.0, 4.0], [6.0, 6.0, 6.0], [6.0, 3.0, 
3.0]], [[6.0, 2.0, 2.0], [6.0, 7.0, 5.0], [5.0, 3.0, 1.0]]], [[[6.0, 3.0, 6.0], [1.0, 2.0, 1.0], [2.0, 8.0, 6.0]], [[4.0, 6.0, 
8.0], [5.0, 8.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 1.0, 7.0], [2.0, 1.0, 7.0], [2.0, 2.0, 1.0]]], [[[2.0, 2.0, 5.0], [5.0, 7.0, 
4.0], [3.0, 3.0, 5.0]], [[3.0, 7.0, 1.0], [8.0, 7.0, 2.0], [7.0, 5.0, 7.0]], [[2.0, 5.0, 7.0], [3.0, 6.0, 4.0], [5.0, 4.0, 
5.0]]], [[[4.0, 4.0, 4.0], [7.0, 4.0, 1.0], [8.0, 7.0, 6.0]], [[8.0, 2.0, 3.0], [1.0, 1.0, 7.0], [3.0, 5.0, 2.0]], [[1.0, 4.0, 
5.0], [5.0, 1.0, 3.0], [3.0, 5.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 8.0, 4.0]], [[5.0, 7.0, 8.0]]], [[[5.0, 8.0, 3.0]], [[4.0, 2.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 8.0]], [[7.0, 2.0, 3.0]]], [[[3.0, 7.0, 2.0]], [[7.0, 2.0, 7.0]]]]))
      arg[2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 3.0], [7.0, 5.0, 7.0], [2.0, 1.0, 7.0]], [[3.0, 7.0, 4.0], [6.0, 6.0, 6.0], [6.0, 
3.0, 3.0]], [[6.0, 2.0, 2.0], [6.0, 7.0, 5.0], [5.0, 3.0, 1.0]]], [[[6.0, 3.0, 6.0], [1.0, 2.0, 1.0], [2.0, 8.0, 6.0]], [[4.0, 
6.0, 8.0], [5.0, 8.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 1.0, 7.0], [2.0, 1.0, 7.0], [2.0, 2.0, 1.0]]], [[[2.0, 2.0, 5.0], [5.0, 
7.0, 4.0], [6.0, 8.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 7.0, 2.0], [5.0, 7.0, 8.0]], [[2.0, 5.0, 7.0], [3.0, 6.0, 4.0], [5.0, 4.0, 
5.0]]], [[[4.0, 4.0, 4.0], [7.0, 4.0, 1.0], [5.0, 8.0, 3.0]], [[8.0, 2.0, 3.0], [1.0, 1.0, 7.0], [4.0, 2.0, 7.0]], [[1.0, 4.0, 
5.0], [5.0, 1.0, 3.0], [3.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 3.0], [7.0, 5.0, 7.0], [2.0, 1.0, 7.0]], 
[[3.0, 7.0, 4.0], [6.0, 6.0, 6.0], [6.0, 3.0, 3.0]], [[6.0, 2.0, 2.0], [6.0, 7.0, 5.0], [5.0, 3.0, 1.0]]], [[[6.0, 3.0, 6.0], 
[1.0, 2.0, 1.0], [2.0, 8.0, 6.0]], [[4.0, 6.0, 8.0], [5.0, 8.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 1.0, 7.0], [2.0, 1.0, 7.0], 
[2.0, 2.0, 1.0]]], [[[2.0, 2.0, 5.0], [5.0, 7.0, 4.0], [5.0, 3.0, 8.0]], [[3.0, 7.0, 1.0], [8.0, 7.0, 2.0], [7.0, 2.0, 3.0]], 
[[2.0, 5.0, 7.0], [3.0, 6.0, 4.0], [5.0, 4.0, 5.0]]], [[[4.0, 4.0, 4.0], [7.0, 4.0, 1.0], [3.0, 7.0, 2.0]], [[8.0, 2.0, 3.0], 
[1.0, 1.0, 7.0], [7.0, 2.0, 7.0]], [[1.0, 4.0, 5.0], [5.0, 1.0, 3.0], [3.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CRPN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 7.0], [5.0, 8.0, 7.0], [6.0, 4.0, 3.0]], [[7.0, 2.0, 5.0], [7.0, 2.0, 2.0], [7.0, 8.0, 
7.0]], [[1.0, 5.0, 1.0], [7.0, 4.0, 8.0], [3.0, 4.0, 4.0]]], [[[7.0, 6.0, 5.0], [2.0, 2.0, 4.0], [8.0, 7.0, 5.0]], [[5.0, 1.0, 
4.0], [4.0, 4.0, 7.0], [7.0, 7.0, 6.0]], [[2.0, 2.0, 4.0], [1.0, 4.0, 2.0], [3.0, 5.0, 1.0]]], [[[4.0, 5.0, 6.0], [8.0, 8.0, 
5.0], [1.0, 6.0, 2.0]], [[8.0, 8.0, 7.0], [2.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[5.0, 2.0, 7.0], [7.0, 4.0, 7.0], [8.0, 4.0, 
8.0]]], [[[4.0, 5.0, 8.0], [7.0, 6.0, 4.0], [1.0, 7.0, 5.0]], [[4.0, 5.0, 2.0], [6.0, 3.0, 4.0], [4.0, 1.0, 5.0]], [[5.0, 6.0, 
4.0], [3.0, 2.0, 4.0], [4.0, 6.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 5.0, 3.0]], [[2.0, 1.0, 7.0]], [[5.0, 1.0, 6.0]], [[4.0, 4.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 4.0, 8.0]], [[3.0, 2.0, 5.0]], [[5.0, 7.0, 1.0]], [[4.0, 2.0, 4.0]]]))
      arg[:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 7.0], [5.0, 8.0, 7.0], [6.0, 4.0, 3.0]], [[7.0, 2.0, 5.0], [3.0, 5.0, 3.0], [7.0, 
8.0, 7.0]], [[1.0, 5.0, 1.0], [7.0, 4.0, 8.0], [3.0, 4.0, 4.0]]], [[[7.0, 6.0, 5.0], [2.0, 2.0, 4.0], [8.0, 7.0, 5.0]], [[5.0, 
1.0, 4.0], [2.0, 1.0, 7.0], [7.0, 7.0, 6.0]], [[2.0, 2.0, 4.0], [1.0, 4.0, 2.0], [3.0, 5.0, 1.0]]], [[[4.0, 5.0, 6.0], [8.0, 
8.0, 5.0], [1.0, 6.0, 2.0]], [[8.0, 8.0, 7.0], [5.0, 1.0, 6.0], [5.0, 6.0, 4.0]], [[5.0, 2.0, 7.0], [7.0, 4.0, 7.0], [8.0, 4.0, 
8.0]]], [[[4.0, 5.0, 8.0], [7.0, 6.0, 4.0], [1.0, 7.0, 5.0]], [[4.0, 5.0, 2.0], [4.0, 4.0, 1.0], [4.0, 1.0, 5.0]], [[5.0, 6.0, 
4.0], [3.0, 2.0, 4.0], [4.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 7.0], [5.0, 8.0, 7.0], [6.0, 4.0, 3.0]], 
[[7.0, 2.0, 5.0], [8.0, 4.0, 8.0], [7.0, 8.0, 7.0]], [[1.0, 5.0, 1.0], [7.0, 4.0, 8.0], [3.0, 4.0, 4.0]]], [[[7.0, 6.0, 5.0], 
[2.0, 2.0, 4.0], [8.0, 7.0, 5.0]], [[5.0, 1.0, 4.0], [3.0, 2.0, 5.0], [7.0, 7.0, 6.0]], [[2.0, 2.0, 4.0], [1.0, 4.0, 2.0], 
[3.0, 5.0, 1.0]]], [[[4.0, 5.0, 6.0], [8.0, 8.0, 5.0], [1.0, 6.0, 2.0]], [[8.0, 8.0, 7.0], [5.0, 7.0, 1.0], [5.0, 6.0, 4.0]], 
[[5.0, 2.0, 7.0], [7.0, 4.0, 7.0], [8.0, 4.0, 8.0]]], [[[4.0, 5.0, 8.0], [7.0, 6.0, 4.0], [1.0, 7.0, 5.0]], [[4.0, 5.0, 2.0], 
[4.0, 2.0, 4.0], [4.0, 1.0, 5.0]], [[5.0, 6.0, 4.0], [3.0, 2.0, 4.0], [4.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LLLN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 6.0], [4.0, 8.0, 2.0], [3.0, 7.0, 8.0]], [[8.0, 5.0, 2.0], [3.0, 8.0, 5.0], [5.0, 6.0, 
5.0]], [[8.0, 4.0, 5.0], [5.0, 2.0, 5.0], [1.0, 5.0, 8.0]]], [[[8.0, 2.0, 6.0], [2.0, 1.0, 1.0], [3.0, 7.0, 7.0]], [[8.0, 7.0, 
5.0], [1.0, 4.0, 2.0], [2.0, 6.0, 1.0]], [[1.0, 5.0, 4.0], [3.0, 5.0, 1.0], [8.0, 5.0, 5.0]]], [[[6.0, 6.0, 5.0], [1.0, 7.0, 
6.0], [3.0, 4.0, 8.0]], [[8.0, 1.0, 5.0], [5.0, 1.0, 1.0], [8.0, 4.0, 3.0]], [[8.0, 3.0, 4.0], [1.0, 4.0, 1.0], [3.0, 8.0, 
3.0]]], [[[8.0, 3.0, 2.0], [8.0, 4.0, 2.0], [5.0, 7.0, 3.0]], [[1.0, 2.0, 3.0], [7.0, 4.0, 5.0], [7.0, 3.0, 2.0]], [[5.0, 4.0, 
3.0], [8.0, 1.0, 5.0], [7.0, 1.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 7.0, 6.0]]], [[[5.0, 7.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 7.0]]], [[[1.0, 1.0, 8.0]]]]))
      arg[2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 6.0], [4.0, 8.0, 2.0], [3.0, 7.0, 8.0]], [[8.0, 5.0, 2.0], [3.0, 8.0, 5.0], [5.0, 
6.0, 5.0]], [[8.0, 4.0, 5.0], [5.0, 2.0, 5.0], [1.0, 5.0, 8.0]]], [[[8.0, 2.0, 6.0], [2.0, 1.0, 1.0], [3.0, 7.0, 7.0]], [[8.0, 
7.0, 5.0], [1.0, 4.0, 2.0], [2.0, 6.0, 1.0]], [[1.0, 5.0, 4.0], [3.0, 5.0, 1.0], [8.0, 5.0, 5.0]]], [[[6.0, 6.0, 5.0], [1.0, 
7.0, 6.0], [3.0, 4.0, 8.0]], [[8.0, 1.0, 5.0], [5.0, 1.0, 1.0], [8.0, 4.0, 3.0]], [[8.0, 3.0, 4.0], [1.0, 4.0, 1.0], [7.0, 7.0, 
6.0]]], [[[8.0, 3.0, 2.0], [8.0, 4.0, 2.0], [5.0, 7.0, 3.0]], [[1.0, 2.0, 3.0], [7.0, 4.0, 5.0], [7.0, 3.0, 2.0]], [[5.0, 4.0, 
3.0], [8.0, 1.0, 5.0], [5.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 6.0], [4.0, 8.0, 2.0], [3.0, 7.0, 8.0]], 
[[8.0, 5.0, 2.0], [3.0, 8.0, 5.0], [5.0, 6.0, 5.0]], [[8.0, 4.0, 5.0], [5.0, 2.0, 5.0], [1.0, 5.0, 8.0]]], [[[8.0, 2.0, 6.0], 
[2.0, 1.0, 1.0], [3.0, 7.0, 7.0]], [[8.0, 7.0, 5.0], [1.0, 4.0, 2.0], [2.0, 6.0, 1.0]], [[1.0, 5.0, 4.0], [3.0, 5.0, 1.0], 
[8.0, 5.0, 5.0]]], [[[6.0, 6.0, 5.0], [1.0, 7.0, 6.0], [3.0, 4.0, 8.0]], [[8.0, 1.0, 5.0], [5.0, 1.0, 1.0], [8.0, 4.0, 3.0]], 
[[8.0, 3.0, 4.0], [1.0, 4.0, 1.0], [6.0, 5.0, 7.0]]], [[[8.0, 3.0, 2.0], [8.0, 4.0, 2.0], [5.0, 7.0, 3.0]], [[1.0, 2.0, 3.0], 
[7.0, 4.0, 5.0], [7.0, 3.0, 2.0]], [[5.0, 4.0, 3.0], [8.0, 1.0, 5.0], [1.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLLN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 3.0], [4.0, 4.0, 8.0], [4.0, 7.0, 5.0]], [[6.0, 2.0, 1.0], [3.0, 3.0, 8.0], [8.0, 2.0, 
3.0]], [[6.0, 6.0, 3.0], [2.0, 4.0, 3.0], [5.0, 6.0, 3.0]]], [[[1.0, 7.0, 8.0], [6.0, 7.0, 2.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 
2.0], [1.0, 8.0, 7.0], [4.0, 7.0, 7.0]], [[6.0, 1.0, 4.0], [8.0, 5.0, 8.0], [4.0, 7.0, 3.0]]], [[[5.0, 5.0, 1.0], [4.0, 2.0, 
3.0], [8.0, 7.0, 7.0]], [[8.0, 6.0, 7.0], [4.0, 2.0, 2.0], [6.0, 7.0, 7.0]], [[7.0, 3.0, 3.0], [8.0, 3.0, 7.0], [1.0, 5.0, 
8.0]]], [[[4.0, 3.0, 8.0], [6.0, 5.0, 4.0], [1.0, 8.0, 7.0]], [[7.0, 2.0, 8.0], [2.0, 7.0, 6.0], [5.0, 8.0, 8.0]], [[2.0, 6.0, 
6.0], [5.0, 3.0, 1.0], [8.0, 3.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 5.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 2.0, 4.0]]]))
      arg[2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 3.0], [4.0, 4.0, 8.0], [4.0, 7.0, 5.0]], [[6.0, 2.0, 1.0], [3.0, 3.0, 8.0], [8.0, 
2.0, 3.0]], [[6.0, 6.0, 3.0], [2.0, 4.0, 3.0], [5.0, 6.0, 3.0]]], [[[1.0, 7.0, 8.0], [6.0, 7.0, 2.0], [1.0, 1.0, 8.0]], [[5.0, 
5.0, 2.0], [1.0, 8.0, 7.0], [4.0, 7.0, 7.0]], [[6.0, 1.0, 4.0], [8.0, 5.0, 8.0], [4.0, 7.0, 3.0]]], [[[5.0, 5.0, 1.0], [4.0, 
2.0, 3.0], [8.0, 7.0, 7.0]], [[8.0, 6.0, 7.0], [4.0, 2.0, 2.0], [6.0, 7.0, 7.0]], [[7.0, 3.0, 3.0], [8.0, 3.0, 7.0], [2.0, 5.0, 
3.0]]], [[[4.0, 3.0, 8.0], [6.0, 5.0, 4.0], [1.0, 8.0, 7.0]], [[7.0, 2.0, 8.0], [2.0, 7.0, 6.0], [5.0, 8.0, 8.0]], [[2.0, 6.0, 
6.0], [5.0, 3.0, 1.0], [8.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 3.0], [4.0, 4.0, 8.0], [4.0, 7.0, 5.0]], 
[[6.0, 2.0, 1.0], [3.0, 3.0, 8.0], [8.0, 2.0, 3.0]], [[6.0, 6.0, 3.0], [2.0, 4.0, 3.0], [5.0, 6.0, 3.0]]], [[[1.0, 7.0, 8.0], 
[6.0, 7.0, 2.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 2.0], [1.0, 8.0, 7.0], [4.0, 7.0, 7.0]], [[6.0, 1.0, 4.0], [8.0, 5.0, 8.0], 
[4.0, 7.0, 3.0]]], [[[5.0, 5.0, 1.0], [4.0, 2.0, 3.0], [8.0, 7.0, 7.0]], [[8.0, 6.0, 7.0], [4.0, 2.0, 2.0], [6.0, 7.0, 7.0]], 
[[7.0, 3.0, 3.0], [8.0, 3.0, 7.0], [7.0, 2.0, 4.0]]], [[[4.0, 3.0, 8.0], [6.0, 5.0, 4.0], [1.0, 8.0, 7.0]], [[7.0, 2.0, 8.0], 
[2.0, 7.0, 6.0], [5.0, 8.0, 8.0]], [[2.0, 6.0, 6.0], [5.0, 3.0, 1.0], [8.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RCUN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 6.0], [4.0, 7.0, 6.0], [8.0, 1.0, 4.0]], [[7.0, 1.0, 7.0], [8.0, 3.0, 4.0], [3.0, 1.0, 
2.0]], [[1.0, 6.0, 4.0], [6.0, 3.0, 7.0], [2.0, 6.0, 1.0]]], [[[3.0, 8.0, 6.0], [2.0, 5.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 3.0, 
8.0], [1.0, 8.0, 1.0], [2.0, 1.0, 1.0]], [[2.0, 5.0, 5.0], [7.0, 1.0, 8.0], [1.0, 5.0, 6.0]]], [[[8.0, 5.0, 7.0], [5.0, 7.0, 
4.0], [6.0, 7.0, 7.0]], [[7.0, 5.0, 7.0], [2.0, 6.0, 7.0], [2.0, 6.0, 2.0]], [[3.0, 2.0, 3.0], [7.0, 4.0, 3.0], [6.0, 3.0, 
7.0]]], [[[5.0, 8.0, 3.0], [7.0, 1.0, 3.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 4.0], [8.0, 6.0, 3.0], [5.0, 7.0, 2.0]], [[1.0, 3.0, 
4.0], [5.0, 7.0, 1.0], [2.0, 6.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 5.0, 6.0], [7.0, 5.0, 8.0]], [[2.0, 8.0, 6.0], [7.0, 5.0, 7.0]], [[4.0, 1.0, 6.0], [8.0, 
8.0, 3.0]]], [[[1.0, 2.0, 4.0], [1.0, 5.0, 6.0]], [[1.0, 3.0, 4.0], [4.0, 8.0, 1.0]], [[1.0, 1.0, 2.0], [1.0, 3.0, 
7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 4.0], [8.0, 4.0, 3.0]], [[1.0, 3.0, 3.0], [7.0, 1.0, 5.0]], [[1.0, 7.0, 
5.0], [5.0, 2.0, 6.0]]], [[[1.0, 6.0, 7.0], [6.0, 5.0, 7.0]], [[4.0, 1.0, 3.0], [4.0, 2.0, 2.0]], [[6.0, 3.0, 6.0], [5.0, 6.0, 
3.0]]]]))
      arg[1:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 6.0], [4.0, 7.0, 6.0], [8.0, 1.0, 4.0]], [[7.0, 1.0, 7.0], [8.0, 3.0, 4.0], [3.0, 
1.0, 2.0]], [[1.0, 6.0, 4.0], [6.0, 3.0, 7.0], [2.0, 6.0, 1.0]]], [[[2.0, 5.0, 6.0], [7.0, 5.0, 8.0], [5.0, 5.0, 4.0]], [[2.0, 
8.0, 6.0], [7.0, 5.0, 7.0], [2.0, 1.0, 1.0]], [[4.0, 1.0, 6.0], [8.0, 8.0, 3.0], [1.0, 5.0, 6.0]]], [[[1.0, 2.0, 4.0], [1.0, 
5.0, 6.0], [6.0, 7.0, 7.0]], [[1.0, 3.0, 4.0], [4.0, 8.0, 1.0], [2.0, 6.0, 2.0]], [[1.0, 1.0, 2.0], [1.0, 3.0, 7.0], [6.0, 3.0, 
7.0]]], [[[5.0, 8.0, 3.0], [7.0, 1.0, 3.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 4.0], [8.0, 6.0, 3.0], [5.0, 7.0, 2.0]], [[1.0, 3.0, 
4.0], [5.0, 7.0, 1.0], [2.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 6.0], [4.0, 7.0, 6.0], [8.0, 1.0, 4.0]], 
[[7.0, 1.0, 7.0], [8.0, 3.0, 4.0], [3.0, 1.0, 2.0]], [[1.0, 6.0, 4.0], [6.0, 3.0, 7.0], [2.0, 6.0, 1.0]]], [[[6.0, 1.0, 4.0], 
[8.0, 4.0, 3.0], [5.0, 5.0, 4.0]], [[1.0, 3.0, 3.0], [7.0, 1.0, 5.0], [2.0, 1.0, 1.0]], [[1.0, 7.0, 5.0], [5.0, 2.0, 6.0], 
[1.0, 5.0, 6.0]]], [[[1.0, 6.0, 7.0], [6.0, 5.0, 7.0], [6.0, 7.0, 7.0]], [[4.0, 1.0, 3.0], [4.0, 2.0, 2.0], [2.0, 6.0, 2.0]], 
[[6.0, 3.0, 6.0], [5.0, 6.0, 3.0], [6.0, 3.0, 7.0]]], [[[5.0, 8.0, 3.0], [7.0, 1.0, 3.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 4.0], 
[8.0, 6.0, 3.0], [5.0, 7.0, 2.0]], [[1.0, 3.0, 4.0], [5.0, 7.0, 1.0], [2.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CUUN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 2.0], [8.0, 6.0, 4.0], [4.0, 8.0, 4.0]], [[1.0, 5.0, 8.0], [4.0, 6.0, 5.0], [7.0, 6.0, 
5.0]], [[1.0, 1.0, 1.0], [4.0, 6.0, 5.0], [7.0, 7.0, 1.0]]], [[[8.0, 1.0, 6.0], [6.0, 5.0, 5.0], [3.0, 8.0, 6.0]], [[1.0, 5.0, 
8.0], [5.0, 2.0, 2.0], [8.0, 2.0, 7.0]], [[8.0, 1.0, 3.0], [1.0, 3.0, 6.0], [5.0, 3.0, 2.0]]], [[[5.0, 3.0, 1.0], [8.0, 5.0, 
8.0], [2.0, 1.0, 5.0]], [[8.0, 2.0, 6.0], [6.0, 5.0, 3.0], [4.0, 2.0, 6.0]], [[4.0, 1.0, 1.0], [3.0, 6.0, 3.0], [4.0, 4.0, 
5.0]]], [[[2.0, 3.0, 5.0], [5.0, 1.0, 4.0], [7.0, 2.0, 5.0]], [[6.0, 2.0, 6.0], [3.0, 4.0, 7.0], [2.0, 2.0, 3.0]], [[8.0, 6.0, 
1.0], [7.0, 5.0, 2.0], [4.0, 2.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 7.0, 6.0], [2.0, 4.0, 3.0]], [[5.0, 2.0, 6.0], [6.0, 1.0, 8.0]]], [[[6.0, 6.0, 5.0], [5.0, 
8.0, 3.0]], [[1.0, 8.0, 2.0], [6.0, 1.0, 7.0]]], [[[1.0, 6.0, 1.0], [4.0, 7.0, 4.0]], [[8.0, 5.0, 7.0], [7.0, 3.0, 8.0]]], 
[[[2.0, 4.0, 6.0], [2.0, 1.0, 6.0]], [[2.0, 2.0, 2.0], [7.0, 5.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 2.0], [6.0, 8.0, 7.0]], [[6.0, 3.0, 4.0], [1.0, 6.0, 7.0]]], [[[8.0, 6.0, 
7.0], [8.0, 2.0, 7.0]], [[1.0, 2.0, 4.0], [2.0, 8.0, 2.0]]], [[[6.0, 1.0, 5.0], [2.0, 4.0, 7.0]], [[5.0, 8.0, 8.0], [3.0, 3.0, 
7.0]]], [[[3.0, 2.0, 6.0], [2.0, 5.0, 7.0]], [[2.0, 8.0, 5.0], [4.0, 7.0, 3.0]]]]))
      arg[:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 6.0], [2.0, 4.0, 3.0], [4.0, 8.0, 4.0]], [[5.0, 2.0, 6.0], [6.0, 1.0, 8.0], [7.0, 
6.0, 5.0]], [[1.0, 1.0, 1.0], [4.0, 6.0, 5.0], [7.0, 7.0, 1.0]]], [[[6.0, 6.0, 5.0], [5.0, 8.0, 3.0], [3.0, 8.0, 6.0]], [[1.0, 
8.0, 2.0], [6.0, 1.0, 7.0], [8.0, 2.0, 7.0]], [[8.0, 1.0, 3.0], [1.0, 3.0, 6.0], [5.0, 3.0, 2.0]]], [[[1.0, 6.0, 1.0], [4.0, 
7.0, 4.0], [2.0, 1.0, 5.0]], [[8.0, 5.0, 7.0], [7.0, 3.0, 8.0], [4.0, 2.0, 6.0]], [[4.0, 1.0, 1.0], [3.0, 6.0, 3.0], [4.0, 4.0, 
5.0]]], [[[2.0, 4.0, 6.0], [2.0, 1.0, 6.0], [7.0, 2.0, 5.0]], [[2.0, 2.0, 2.0], [7.0, 5.0, 1.0], [2.0, 2.0, 3.0]], [[8.0, 6.0, 
1.0], [7.0, 5.0, 2.0], [4.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 2.0], [6.0, 8.0, 7.0], [4.0, 8.0, 4.0]], 
[[6.0, 3.0, 4.0], [1.0, 6.0, 7.0], [7.0, 6.0, 5.0]], [[1.0, 1.0, 1.0], [4.0, 6.0, 5.0], [7.0, 7.0, 1.0]]], [[[8.0, 6.0, 7.0], 
[8.0, 2.0, 7.0], [3.0, 8.0, 6.0]], [[1.0, 2.0, 4.0], [2.0, 8.0, 2.0], [8.0, 2.0, 7.0]], [[8.0, 1.0, 3.0], [1.0, 3.0, 6.0], 
[5.0, 3.0, 2.0]]], [[[6.0, 1.0, 5.0], [2.0, 4.0, 7.0], [2.0, 1.0, 5.0]], [[5.0, 8.0, 8.0], [3.0, 3.0, 7.0], [4.0, 2.0, 6.0]], 
[[4.0, 1.0, 1.0], [3.0, 6.0, 3.0], [4.0, 4.0, 5.0]]], [[[3.0, 2.0, 6.0], [2.0, 5.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 8.0, 5.0], 
[4.0, 7.0, 3.0], [2.0, 2.0, 3.0]], [[8.0, 6.0, 1.0], [7.0, 5.0, 2.0], [4.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_URPN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 3.0], [5.0, 4.0, 3.0], [6.0, 2.0, 1.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 6.0], [7.0, 8.0, 
6.0]], [[7.0, 6.0, 5.0], [7.0, 2.0, 4.0], [7.0, 5.0, 7.0]]], [[[8.0, 1.0, 7.0], [4.0, 5.0, 2.0], [3.0, 3.0, 6.0]], [[5.0, 4.0, 
6.0], [7.0, 4.0, 7.0], [6.0, 3.0, 5.0]], [[5.0, 4.0, 8.0], [8.0, 8.0, 7.0], [6.0, 5.0, 7.0]]], [[[3.0, 8.0, 1.0], [7.0, 6.0, 
5.0], [6.0, 1.0, 8.0]], [[5.0, 5.0, 3.0], [6.0, 8.0, 3.0], [8.0, 4.0, 6.0]], [[8.0, 5.0, 7.0], [7.0, 7.0, 4.0], [5.0, 1.0, 
5.0]]], [[[7.0, 1.0, 3.0], [6.0, 5.0, 8.0], [4.0, 5.0, 6.0]], [[2.0, 2.0, 8.0], [5.0, 3.0, 6.0], [7.0, 3.0, 4.0]], [[7.0, 4.0, 
8.0], [2.0, 4.0, 5.0], [3.0, 1.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 6.0, 7.0]], [[6.0, 1.0, 7.0]], [[1.0, 5.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 4.0, 4.0]], [[4.0, 3.0, 4.0]], [[4.0, 1.0, 8.0]]]))
      arg[:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [5.0, 4.0, 3.0], [6.0, 2.0, 1.0]], [[7.0, 8.0, 7.0], [7.0, 6.0, 7.0], [7.0, 
8.0, 6.0]], [[7.0, 6.0, 5.0], [7.0, 2.0, 4.0], [7.0, 5.0, 7.0]]], [[[8.0, 1.0, 7.0], [4.0, 5.0, 2.0], [3.0, 3.0, 6.0]], [[5.0, 
4.0, 6.0], [6.0, 1.0, 7.0], [6.0, 3.0, 5.0]], [[5.0, 4.0, 8.0], [8.0, 8.0, 7.0], [6.0, 5.0, 7.0]]], [[[3.0, 8.0, 1.0], [7.0, 
6.0, 5.0], [6.0, 1.0, 8.0]], [[5.0, 5.0, 3.0], [1.0, 5.0, 6.0], [8.0, 4.0, 6.0]], [[8.0, 5.0, 7.0], [7.0, 7.0, 4.0], [5.0, 1.0, 
5.0]]], [[[7.0, 1.0, 3.0], [6.0, 5.0, 8.0], [4.0, 5.0, 6.0]], [[2.0, 2.0, 8.0], [5.0, 3.0, 6.0], [7.0, 3.0, 4.0]], [[7.0, 4.0, 
8.0], [2.0, 4.0, 5.0], [3.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 3.0], [5.0, 4.0, 3.0], [6.0, 2.0, 1.0]], 
[[7.0, 8.0, 7.0], [1.0, 4.0, 4.0], [7.0, 8.0, 6.0]], [[7.0, 6.0, 5.0], [7.0, 2.0, 4.0], [7.0, 5.0, 7.0]]], [[[8.0, 1.0, 7.0], 
[4.0, 5.0, 2.0], [3.0, 3.0, 6.0]], [[5.0, 4.0, 6.0], [4.0, 3.0, 4.0], [6.0, 3.0, 5.0]], [[5.0, 4.0, 8.0], [8.0, 8.0, 7.0], 
[6.0, 5.0, 7.0]]], [[[3.0, 8.0, 1.0], [7.0, 6.0, 5.0], [6.0, 1.0, 8.0]], [[5.0, 5.0, 3.0], [4.0, 1.0, 8.0], [8.0, 4.0, 6.0]], 
[[8.0, 5.0, 7.0], [7.0, 7.0, 4.0], [5.0, 1.0, 5.0]]], [[[7.0, 1.0, 3.0], [6.0, 5.0, 8.0], [4.0, 5.0, 6.0]], [[2.0, 2.0, 8.0], 
[5.0, 3.0, 6.0], [7.0, 3.0, 4.0]], [[7.0, 4.0, 8.0], [2.0, 4.0, 5.0], [3.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCCC(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 4.0], [3.0, 8.0, 5.0], [2.0, 1.0, 8.0]], [[3.0, 5.0, 6.0], [7.0, 7.0, 2.0], [6.0, 4.0, 
2.0]], [[5.0, 1.0, 7.0], [7.0, 5.0, 3.0], [4.0, 8.0, 6.0]]], [[[4.0, 8.0, 3.0], [5.0, 6.0, 1.0], [6.0, 3.0, 1.0]], [[7.0, 2.0, 
4.0], [6.0, 8.0, 2.0], [8.0, 7.0, 1.0]], [[2.0, 3.0, 2.0], [6.0, 6.0, 4.0], [1.0, 3.0, 4.0]]], [[[8.0, 4.0, 6.0], [7.0, 5.0, 
7.0], [1.0, 6.0, 4.0]], [[5.0, 5.0, 5.0], [5.0, 6.0, 5.0], [2.0, 7.0, 8.0]], [[4.0, 3.0, 8.0], [6.0, 2.0, 7.0], [5.0, 4.0, 
7.0]]], [[[4.0, 8.0, 4.0], [4.0, 4.0, 8.0], [2.0, 1.0, 5.0]], [[2.0, 7.0, 3.0], [1.0, 8.0, 8.0], [8.0, 4.0, 4.0]], [[4.0, 6.0, 
7.0], [5.0, 6.0, 4.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 5.0, 2.0], [1.0, 6.0, 7.0], [8.0, 5.0, 5.0]], [[2.0, 8.0, 4.0], [7.0, 7.0, 1.0], [3.0, 2.0, 
3.0]], [[8.0, 5.0, 8.0], [5.0, 8.0, 4.0], [3.0, 8.0, 6.0]]], [[[6.0, 1.0, 6.0], [5.0, 5.0, 2.0], [6.0, 2.0, 1.0]], [[8.0, 3.0, 
3.0], [3.0, 2.0, 7.0], [2.0, 3.0, 1.0]], [[8.0, 3.0, 1.0], [2.0, 5.0, 6.0], [2.0, 3.0, 3.0]]], [[[6.0, 8.0, 1.0], [7.0, 3.0, 
4.0], [3.0, 7.0, 4.0]], [[6.0, 3.0, 4.0], [3.0, 4.0, 3.0], [5.0, 2.0, 5.0]], [[2.0, 2.0, 2.0], [6.0, 4.0, 6.0], [3.0, 2.0, 
7.0]]], [[[5.0, 6.0, 5.0], [5.0, 3.0, 1.0], [2.0, 4.0, 4.0]], [[5.0, 8.0, 6.0], [5.0, 8.0, 7.0], [8.0, 2.0, 7.0]], [[4.0, 3.0, 
1.0], [8.0, 7.0, 5.0], [4.0, 6.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 5.0], [5.0, 3.0, 1.0], [6.0, 3.0, 3.0]], [[2.0, 6.0, 3.0], [3.0, 1.0, 
2.0], [6.0, 1.0, 3.0]], [[2.0, 8.0, 7.0], [7.0, 4.0, 5.0], [6.0, 2.0, 3.0]]], [[[2.0, 4.0, 2.0], [1.0, 6.0, 6.0], [3.0, 4.0, 
6.0]], [[8.0, 5.0, 6.0], [5.0, 1.0, 1.0], [6.0, 5.0, 7.0]], [[1.0, 1.0, 6.0], [6.0, 6.0, 2.0], [1.0, 8.0, 2.0]]], [[[3.0, 3.0, 
7.0], [1.0, 5.0, 6.0], [8.0, 7.0, 3.0]], [[8.0, 3.0, 4.0], [4.0, 4.0, 5.0], [1.0, 7.0, 3.0]], [[3.0, 2.0, 2.0], [1.0, 5.0, 
8.0], [3.0, 4.0, 6.0]]], [[[7.0, 2.0, 7.0], [5.0, 5.0, 4.0], [5.0, 1.0, 3.0]], [[3.0, 5.0, 3.0], [6.0, 7.0, 6.0], [8.0, 8.0, 
6.0]], [[6.0, 3.0, 2.0], [2.0, 3.0, 3.0], [7.0, 1.0, 1.0]]]]))
      arg[:,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 2.0], [1.0, 6.0, 7.0], [8.0, 5.0, 5.0]], [[2.0, 8.0, 4.0], [7.0, 7.0, 1.0], [3.0, 
2.0, 3.0]], [[8.0, 5.0, 8.0], [5.0, 8.0, 4.0], [3.0, 8.0, 6.0]]], [[[6.0, 1.0, 6.0], [5.0, 5.0, 2.0], [6.0, 2.0, 1.0]], [[8.0, 
3.0, 3.0], [3.0, 2.0, 7.0], [2.0, 3.0, 1.0]], [[8.0, 3.0, 1.0], [2.0, 5.0, 6.0], [2.0, 3.0, 3.0]]], [[[6.0, 8.0, 1.0], [7.0, 
3.0, 4.0], [3.0, 7.0, 4.0]], [[6.0, 3.0, 4.0], [3.0, 4.0, 3.0], [5.0, 2.0, 5.0]], [[2.0, 2.0, 2.0], [6.0, 4.0, 6.0], [3.0, 2.0, 
7.0]]], [[[5.0, 6.0, 5.0], [5.0, 3.0, 1.0], [2.0, 4.0, 4.0]], [[5.0, 8.0, 6.0], [5.0, 8.0, 7.0], [8.0, 2.0, 7.0]], [[4.0, 3.0, 
1.0], [8.0, 7.0, 5.0], [4.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 5.0], [5.0, 3.0, 1.0], [6.0, 3.0, 3.0]], 
[[2.0, 6.0, 3.0], [3.0, 1.0, 2.0], [6.0, 1.0, 3.0]], [[2.0, 8.0, 7.0], [7.0, 4.0, 5.0], [6.0, 2.0, 3.0]]], [[[2.0, 4.0, 2.0], 
[1.0, 6.0, 6.0], [3.0, 4.0, 6.0]], [[8.0, 5.0, 6.0], [5.0, 1.0, 1.0], [6.0, 5.0, 7.0]], [[1.0, 1.0, 6.0], [6.0, 6.0, 2.0], 
[1.0, 8.0, 2.0]]], [[[3.0, 3.0, 7.0], [1.0, 5.0, 6.0], [8.0, 7.0, 3.0]], [[8.0, 3.0, 4.0], [4.0, 4.0, 5.0], [1.0, 7.0, 3.0]], 
[[3.0, 2.0, 2.0], [1.0, 5.0, 8.0], [3.0, 4.0, 6.0]]], [[[7.0, 2.0, 7.0], [5.0, 5.0, 4.0], [5.0, 1.0, 3.0]], [[3.0, 5.0, 3.0], 
[6.0, 7.0, 6.0], [8.0, 8.0, 6.0]], [[6.0, 3.0, 2.0], [2.0, 3.0, 3.0], [7.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UULN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 2.0], [3.0, 8.0, 8.0], [6.0, 6.0, 5.0]], [[1.0, 5.0, 5.0], [2.0, 5.0, 2.0], [3.0, 2.0, 
6.0]], [[3.0, 3.0, 4.0], [2.0, 8.0, 8.0], [6.0, 7.0, 7.0]]], [[[4.0, 2.0, 1.0], [7.0, 1.0, 4.0], [7.0, 7.0, 8.0]], [[6.0, 4.0, 
8.0], [8.0, 4.0, 7.0], [5.0, 1.0, 7.0]], [[1.0, 2.0, 1.0], [6.0, 2.0, 1.0], [5.0, 7.0, 4.0]]], [[[3.0, 3.0, 1.0], [8.0, 6.0, 
8.0], [5.0, 1.0, 3.0]], [[3.0, 8.0, 7.0], [7.0, 2.0, 4.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 4.0], [7.0, 6.0, 5.0], [5.0, 5.0, 
5.0]]], [[[2.0, 5.0, 2.0], [5.0, 2.0, 1.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 5.0], [2.0, 2.0, 5.0], [5.0, 5.0, 6.0]], [[7.0, 7.0, 
3.0], [5.0, 3.0, 8.0], [3.0, 8.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 4.0, 3.0]], [[8.0, 8.0, 8.0]]], [[[5.0, 3.0, 8.0]], [[8.0, 6.0, 6.0]]], [[[1.0, 8.0, 8.0]], 
[[8.0, 2.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 5.0]], [[5.0, 2.0, 6.0]]], [[[2.0, 6.0, 1.0]], [[7.0, 2.0, 5.0]]], [[[5.0, 
5.0, 5.0]], [[3.0, 3.0, 7.0]]]]))
      arg[:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 2.0], [3.0, 8.0, 8.0], [5.0, 4.0, 3.0]], [[1.0, 5.0, 5.0], [2.0, 5.0, 2.0], [8.0, 
8.0, 8.0]], [[3.0, 3.0, 4.0], [2.0, 8.0, 8.0], [6.0, 7.0, 7.0]]], [[[4.0, 2.0, 1.0], [7.0, 1.0, 4.0], [5.0, 3.0, 8.0]], [[6.0, 
4.0, 8.0], [8.0, 4.0, 7.0], [8.0, 6.0, 6.0]], [[1.0, 2.0, 1.0], [6.0, 2.0, 1.0], [5.0, 7.0, 4.0]]], [[[3.0, 3.0, 1.0], [8.0, 
6.0, 8.0], [1.0, 8.0, 8.0]], [[3.0, 8.0, 7.0], [7.0, 2.0, 4.0], [8.0, 2.0, 6.0]], [[8.0, 1.0, 4.0], [7.0, 6.0, 5.0], [5.0, 5.0, 
5.0]]], [[[2.0, 5.0, 2.0], [5.0, 2.0, 1.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 5.0], [2.0, 2.0, 5.0], [5.0, 5.0, 6.0]], [[7.0, 7.0, 
3.0], [5.0, 3.0, 8.0], [3.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 2.0], [3.0, 8.0, 8.0], [6.0, 8.0, 5.0]], 
[[1.0, 5.0, 5.0], [2.0, 5.0, 2.0], [5.0, 2.0, 6.0]], [[3.0, 3.0, 4.0], [2.0, 8.0, 8.0], [6.0, 7.0, 7.0]]], [[[4.0, 2.0, 1.0], 
[7.0, 1.0, 4.0], [2.0, 6.0, 1.0]], [[6.0, 4.0, 8.0], [8.0, 4.0, 7.0], [7.0, 2.0, 5.0]], [[1.0, 2.0, 1.0], [6.0, 2.0, 1.0], 
[5.0, 7.0, 4.0]]], [[[3.0, 3.0, 1.0], [8.0, 6.0, 8.0], [5.0, 5.0, 5.0]], [[3.0, 8.0, 7.0], [7.0, 2.0, 4.0], [3.0, 3.0, 7.0]], 
[[8.0, 1.0, 4.0], [7.0, 6.0, 5.0], [5.0, 5.0, 5.0]]], [[[2.0, 5.0, 2.0], [5.0, 2.0, 1.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 5.0], 
[2.0, 2.0, 5.0], [5.0, 5.0, 6.0]], [[7.0, 7.0, 3.0], [5.0, 3.0, 8.0], [3.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CPLN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 6.0], [4.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[3.0, 2.0, 5.0], [6.0, 5.0, 7.0], [8.0, 1.0, 
5.0]], [[7.0, 8.0, 3.0], [7.0, 8.0, 3.0], [7.0, 2.0, 7.0]]], [[[8.0, 8.0, 7.0], [7.0, 3.0, 4.0], [8.0, 6.0, 1.0]], [[2.0, 7.0, 
5.0], [2.0, 4.0, 7.0], [2.0, 8.0, 4.0]], [[1.0, 5.0, 8.0], [3.0, 2.0, 7.0], [3.0, 6.0, 1.0]]], [[[7.0, 1.0, 5.0], [1.0, 1.0, 
5.0], [5.0, 6.0, 2.0]], [[2.0, 8.0, 7.0], [4.0, 5.0, 2.0], [1.0, 7.0, 7.0]], [[7.0, 6.0, 7.0], [7.0, 5.0, 2.0], [5.0, 8.0, 
4.0]]], [[[3.0, 6.0, 3.0], [2.0, 3.0, 2.0], [2.0, 1.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 1.0], [3.0, 8.0, 6.0]], [[1.0, 3.0, 
3.0], [2.0, 7.0, 1.0], [1.0, 3.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 8.0, 3.0]], [[8.0, 5.0, 7.0]], [[6.0, 1.0, 3.0]], [[8.0, 8.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 8.0, 6.0]], [[8.0, 7.0, 7.0]], [[8.0, 8.0, 3.0]], [[6.0, 4.0, 3.0]]]))
      arg[:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[3.0, 2.0, 5.0], [6.0, 5.0, 7.0], [7.0, 
8.0, 3.0]], [[7.0, 8.0, 3.0], [7.0, 8.0, 3.0], [7.0, 2.0, 7.0]]], [[[8.0, 8.0, 7.0], [7.0, 3.0, 4.0], [8.0, 6.0, 1.0]], [[2.0, 
7.0, 5.0], [2.0, 4.0, 7.0], [8.0, 5.0, 7.0]], [[1.0, 5.0, 8.0], [3.0, 2.0, 7.0], [3.0, 6.0, 1.0]]], [[[7.0, 1.0, 5.0], [1.0, 
1.0, 5.0], [5.0, 6.0, 2.0]], [[2.0, 8.0, 7.0], [4.0, 5.0, 2.0], [6.0, 1.0, 3.0]], [[7.0, 6.0, 7.0], [7.0, 5.0, 2.0], [5.0, 8.0, 
4.0]]], [[[3.0, 6.0, 3.0], [2.0, 3.0, 2.0], [2.0, 1.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 3.0, 
3.0], [2.0, 7.0, 1.0], [1.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 8.0, 1.0], [3.0, 8.0, 1.0]], 
[[3.0, 2.0, 5.0], [6.0, 5.0, 7.0], [5.0, 8.0, 6.0]], [[7.0, 8.0, 3.0], [7.0, 8.0, 3.0], [7.0, 2.0, 7.0]]], [[[8.0, 8.0, 7.0], 
[7.0, 3.0, 4.0], [8.0, 6.0, 1.0]], [[2.0, 7.0, 5.0], [2.0, 4.0, 7.0], [8.0, 7.0, 7.0]], [[1.0, 5.0, 8.0], [3.0, 2.0, 7.0], 
[3.0, 6.0, 1.0]]], [[[7.0, 1.0, 5.0], [1.0, 1.0, 5.0], [5.0, 6.0, 2.0]], [[2.0, 8.0, 7.0], [4.0, 5.0, 2.0], [8.0, 8.0, 3.0]], 
[[7.0, 6.0, 7.0], [7.0, 5.0, 2.0], [5.0, 8.0, 4.0]]], [[[3.0, 6.0, 3.0], [2.0, 3.0, 2.0], [2.0, 1.0, 6.0]], [[3.0, 2.0, 1.0], 
[1.0, 7.0, 1.0], [6.0, 4.0, 3.0]], [[1.0, 3.0, 3.0], [2.0, 7.0, 1.0], [1.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LLRN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 7.0], [2.0, 2.0, 6.0], [4.0, 8.0, 7.0]], [[6.0, 8.0, 2.0], [7.0, 4.0, 6.0], [1.0, 5.0, 
7.0]], [[1.0, 2.0, 2.0], [2.0, 5.0, 4.0], [2.0, 8.0, 2.0]]], [[[5.0, 1.0, 7.0], [4.0, 5.0, 4.0], [3.0, 8.0, 1.0]], [[2.0, 3.0, 
4.0], [7.0, 6.0, 5.0], [4.0, 8.0, 6.0]], [[4.0, 7.0, 4.0], [7.0, 8.0, 5.0], [4.0, 6.0, 2.0]]], [[[1.0, 4.0, 3.0], [7.0, 4.0, 
7.0], [8.0, 2.0, 2.0]], [[4.0, 5.0, 5.0], [2.0, 2.0, 5.0], [2.0, 2.0, 6.0]], [[5.0, 1.0, 3.0], [3.0, 4.0, 4.0], [7.0, 2.0, 
4.0]]], [[[4.0, 4.0, 5.0], [2.0, 7.0, 6.0], [5.0, 7.0, 8.0]], [[6.0, 1.0, 1.0], [7.0, 3.0, 3.0], [2.0, 3.0, 1.0]], [[2.0, 6.0, 
3.0], [5.0, 2.0, 3.0], [6.0, 8.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 7.0, 4.0]]], [[[3.0, 6.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 4.0]]], [[[2.0, 8.0, 4.0]]]]))
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 7.0], [2.0, 2.0, 6.0], [4.0, 8.0, 7.0]], [[6.0, 8.0, 2.0], [7.0, 4.0, 6.0], [1.0, 
5.0, 7.0]], [[1.0, 2.0, 2.0], [2.0, 5.0, 4.0], [2.0, 8.0, 2.0]]], [[[5.0, 1.0, 7.0], [4.0, 5.0, 4.0], [3.0, 8.0, 1.0]], [[2.0, 
3.0, 4.0], [7.0, 6.0, 5.0], [4.0, 8.0, 6.0]], [[4.0, 7.0, 4.0], [7.0, 8.0, 5.0], [4.0, 6.0, 2.0]]], [[[1.0, 4.0, 3.0], [7.0, 
4.0, 7.0], [8.0, 2.0, 2.0]], [[4.0, 5.0, 5.0], [2.0, 2.0, 5.0], [2.0, 2.0, 6.0]], [[5.0, 1.0, 3.0], [3.0, 7.0, 4.0], [7.0, 2.0, 
4.0]]], [[[4.0, 4.0, 5.0], [2.0, 7.0, 6.0], [5.0, 7.0, 8.0]], [[6.0, 1.0, 1.0], [7.0, 3.0, 3.0], [2.0, 3.0, 1.0]], [[2.0, 6.0, 
3.0], [3.0, 6.0, 2.0], [6.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 7.0], [2.0, 2.0, 6.0], [4.0, 8.0, 7.0]], 
[[6.0, 8.0, 2.0], [7.0, 4.0, 6.0], [1.0, 5.0, 7.0]], [[1.0, 2.0, 2.0], [2.0, 5.0, 4.0], [2.0, 8.0, 2.0]]], [[[5.0, 1.0, 7.0], 
[4.0, 5.0, 4.0], [3.0, 8.0, 1.0]], [[2.0, 3.0, 4.0], [7.0, 6.0, 5.0], [4.0, 8.0, 6.0]], [[4.0, 7.0, 4.0], [7.0, 8.0, 5.0], 
[4.0, 6.0, 2.0]]], [[[1.0, 4.0, 3.0], [7.0, 4.0, 7.0], [8.0, 2.0, 2.0]], [[4.0, 5.0, 5.0], [2.0, 2.0, 5.0], [2.0, 2.0, 6.0]], 
[[5.0, 1.0, 3.0], [1.0, 5.0, 4.0], [7.0, 2.0, 4.0]]], [[[4.0, 4.0, 5.0], [2.0, 7.0, 6.0], [5.0, 7.0, 8.0]], [[6.0, 1.0, 1.0], 
[7.0, 3.0, 3.0], [2.0, 3.0, 1.0]], [[2.0, 6.0, 3.0], [2.0, 8.0, 4.0], [6.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PCCN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 3.0], [8.0, 3.0, 8.0], [8.0, 4.0, 7.0]], [[6.0, 2.0, 4.0], [8.0, 8.0, 2.0], [7.0, 5.0, 
7.0]], [[5.0, 5.0, 6.0], [1.0, 1.0, 7.0], [3.0, 2.0, 8.0]]], [[[6.0, 8.0, 8.0], [6.0, 7.0, 1.0], [3.0, 4.0, 4.0]], [[2.0, 7.0, 
6.0], [6.0, 2.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 1.0, 7.0], [5.0, 4.0, 4.0], [7.0, 1.0, 3.0]]], [[[5.0, 7.0, 6.0], [4.0, 3.0, 
1.0], [1.0, 3.0, 4.0]], [[2.0, 7.0, 2.0], [1.0, 7.0, 1.0], [3.0, 1.0, 8.0]], [[7.0, 3.0, 2.0], [7.0, 8.0, 4.0], [7.0, 6.0, 
6.0]]], [[[5.0, 1.0, 1.0], [6.0, 5.0, 1.0], [7.0, 5.0, 3.0]], [[3.0, 4.0, 2.0], [2.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 
8.0], [8.0, 8.0, 6.0], [6.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 5.0, 1.0], [4.0, 1.0, 5.0], [5.0, 8.0, 5.0]], [[4.0, 1.0, 1.0], [7.0, 7.0, 7.0], [4.0, 2.0, 
7.0]], [[8.0, 6.0, 7.0], [7.0, 8.0, 2.0], [7.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 7.0, 1.0], [3.0, 8.0, 2.0], [6.0, 6.0, 1.0]], [[7.0, 2.0, 3.0], [7.0, 6.0, 1.0], 
[4.0, 4.0, 3.0]], [[8.0, 8.0, 1.0], [7.0, 1.0, 6.0], [4.0, 2.0, 5.0]]]))
      arg[2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 3.0], [8.0, 3.0, 8.0], [8.0, 4.0, 7.0]], [[6.0, 2.0, 4.0], [8.0, 8.0, 2.0], [7.0, 
5.0, 7.0]], [[5.0, 5.0, 6.0], [1.0, 1.0, 7.0], [3.0, 2.0, 8.0]]], [[[6.0, 8.0, 8.0], [6.0, 7.0, 1.0], [3.0, 4.0, 4.0]], [[2.0, 
7.0, 6.0], [6.0, 2.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 1.0, 7.0], [5.0, 4.0, 4.0], [7.0, 1.0, 3.0]]], [[[6.0, 5.0, 1.0], [4.0, 
1.0, 5.0], [5.0, 8.0, 5.0]], [[4.0, 1.0, 1.0], [7.0, 7.0, 7.0], [4.0, 2.0, 7.0]], [[8.0, 6.0, 7.0], [7.0, 8.0, 2.0], [7.0, 3.0, 
1.0]]], [[[5.0, 1.0, 1.0], [6.0, 5.0, 1.0], [7.0, 5.0, 3.0]], [[3.0, 4.0, 2.0], [2.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 
8.0], [8.0, 8.0, 6.0], [6.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 3.0], [8.0, 3.0, 8.0], [8.0, 4.0, 7.0]], 
[[6.0, 2.0, 4.0], [8.0, 8.0, 2.0], [7.0, 5.0, 7.0]], [[5.0, 5.0, 6.0], [1.0, 1.0, 7.0], [3.0, 2.0, 8.0]]], [[[6.0, 8.0, 8.0], 
[6.0, 7.0, 1.0], [3.0, 4.0, 4.0]], [[2.0, 7.0, 6.0], [6.0, 2.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 1.0, 7.0], [5.0, 4.0, 4.0], 
[7.0, 1.0, 3.0]]], [[[3.0, 7.0, 1.0], [3.0, 8.0, 2.0], [6.0, 6.0, 1.0]], [[7.0, 2.0, 3.0], [7.0, 6.0, 1.0], [4.0, 4.0, 3.0]], 
[[8.0, 8.0, 1.0], [7.0, 1.0, 6.0], [4.0, 2.0, 5.0]]], [[[5.0, 1.0, 1.0], [6.0, 5.0, 1.0], [7.0, 5.0, 3.0]], [[3.0, 4.0, 2.0], 
[2.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 8.0], [8.0, 8.0, 6.0], [6.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RPCN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 1.0], [5.0, 4.0, 3.0], [5.0, 3.0, 3.0]], [[7.0, 6.0, 1.0], [4.0, 1.0, 8.0], [4.0, 6.0, 
2.0]], [[7.0, 8.0, 3.0], [1.0, 7.0, 2.0], [4.0, 7.0, 1.0]]], [[[4.0, 2.0, 8.0], [7.0, 4.0, 4.0], [2.0, 7.0, 5.0]], [[1.0, 7.0, 
1.0], [8.0, 2.0, 7.0], [4.0, 5.0, 3.0]], [[1.0, 8.0, 2.0], [7.0, 8.0, 6.0], [1.0, 2.0, 4.0]]], [[[4.0, 2.0, 6.0], [8.0, 7.0, 
8.0], [4.0, 5.0, 8.0]], [[4.0, 7.0, 3.0], [6.0, 6.0, 1.0], [8.0, 8.0, 2.0]], [[2.0, 6.0, 5.0], [1.0, 8.0, 6.0], [3.0, 3.0, 
2.0]]], [[[5.0, 4.0, 5.0], [2.0, 2.0, 8.0], [7.0, 1.0, 8.0]], [[3.0, 5.0, 2.0], [4.0, 1.0, 6.0], [3.0, 8.0, 3.0]], [[7.0, 1.0, 
4.0], [4.0, 8.0, 1.0], [4.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 2.0, 7.0], [3.0, 4.0, 6.0], [1.0, 2.0, 5.0]], [[5.0, 6.0, 3.0], [2.0, 4.0, 5.0], [8.0, 3.0, 
3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 5.0, 3.0], [4.0, 3.0, 2.0], [2.0, 5.0, 3.0]], [[2.0, 5.0, 8.0], [4.0, 2.0, 1.0], 
[4.0, 6.0, 2.0]]]))
      arg[1:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 1.0], [5.0, 4.0, 3.0], [5.0, 3.0, 3.0]], [[7.0, 6.0, 1.0], [4.0, 1.0, 8.0], [4.0, 
6.0, 2.0]], [[7.0, 8.0, 3.0], [1.0, 7.0, 2.0], [4.0, 7.0, 1.0]]], [[[4.0, 2.0, 8.0], [7.0, 4.0, 4.0], [2.0, 7.0, 5.0]], [[7.0, 
2.0, 7.0], [3.0, 4.0, 6.0], [1.0, 2.0, 5.0]], [[1.0, 8.0, 2.0], [7.0, 8.0, 6.0], [1.0, 2.0, 4.0]]], [[[4.0, 2.0, 6.0], [8.0, 
7.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 6.0, 3.0], [2.0, 4.0, 5.0], [8.0, 3.0, 3.0]], [[2.0, 6.0, 5.0], [1.0, 8.0, 6.0], [3.0, 3.0, 
2.0]]], [[[5.0, 4.0, 5.0], [2.0, 2.0, 8.0], [7.0, 1.0, 8.0]], [[3.0, 5.0, 2.0], [4.0, 1.0, 6.0], [3.0, 8.0, 3.0]], [[7.0, 1.0, 
4.0], [4.0, 8.0, 1.0], [4.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 1.0], [5.0, 4.0, 3.0], [5.0, 3.0, 3.0]], 
[[7.0, 6.0, 1.0], [4.0, 1.0, 8.0], [4.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], [1.0, 7.0, 2.0], [4.0, 7.0, 1.0]]], [[[4.0, 2.0, 8.0], 
[7.0, 4.0, 4.0], [2.0, 7.0, 5.0]], [[2.0, 5.0, 3.0], [4.0, 3.0, 2.0], [2.0, 5.0, 3.0]], [[1.0, 8.0, 2.0], [7.0, 8.0, 6.0], 
[1.0, 2.0, 4.0]]], [[[4.0, 2.0, 6.0], [8.0, 7.0, 8.0], [4.0, 5.0, 8.0]], [[2.0, 5.0, 8.0], [4.0, 2.0, 1.0], [4.0, 6.0, 2.0]], 
[[2.0, 6.0, 5.0], [1.0, 8.0, 6.0], [3.0, 3.0, 2.0]]], [[[5.0, 4.0, 5.0], [2.0, 2.0, 8.0], [7.0, 1.0, 8.0]], [[3.0, 5.0, 2.0], 
[4.0, 1.0, 6.0], [3.0, 8.0, 3.0]], [[7.0, 1.0, 4.0], [4.0, 8.0, 1.0], [4.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CPPN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 8.0], [6.0, 2.0, 4.0], [3.0, 3.0, 2.0]], [[1.0, 7.0, 3.0], [6.0, 1.0, 1.0], [3.0, 3.0, 
5.0]], [[2.0, 3.0, 2.0], [2.0, 7.0, 8.0], [6.0, 2.0, 3.0]]], [[[7.0, 5.0, 2.0], [7.0, 2.0, 4.0], [5.0, 5.0, 2.0]], [[2.0, 1.0, 
1.0], [3.0, 6.0, 1.0], [4.0, 2.0, 4.0]], [[5.0, 8.0, 3.0], [1.0, 8.0, 1.0], [4.0, 2.0, 4.0]]], [[[1.0, 1.0, 8.0], [7.0, 4.0, 
4.0], [4.0, 8.0, 5.0]], [[2.0, 5.0, 3.0], [1.0, 3.0, 4.0], [2.0, 5.0, 5.0]], [[2.0, 5.0, 2.0], [6.0, 3.0, 4.0], [8.0, 7.0, 
4.0]]], [[[3.0, 8.0, 4.0], [6.0, 2.0, 5.0], [2.0, 6.0, 5.0]], [[6.0, 4.0, 1.0], [2.0, 7.0, 4.0], [5.0, 6.0, 5.0]], [[8.0, 3.0, 
1.0], [3.0, 8.0, 5.0], [4.0, 7.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 3.0, 5.0], [6.0, 8.0, 4.0], [7.0, 7.0, 5.0], [6.0, 6.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 1.0, 2.0], [8.0, 5.0, 3.0], [2.0, 3.0, 1.0], [2.0, 3.0, 2.0]]))
      arg[:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 8.0], [6.0, 2.0, 4.0], [3.0, 3.0, 2.0]], [[1.0, 7.0, 3.0], [8.0, 3.0, 5.0], [3.0, 
3.0, 5.0]], [[2.0, 3.0, 2.0], [2.0, 7.0, 8.0], [6.0, 2.0, 3.0]]], [[[7.0, 5.0, 2.0], [7.0, 2.0, 4.0], [5.0, 5.0, 2.0]], [[2.0, 
1.0, 1.0], [6.0, 8.0, 4.0], [4.0, 2.0, 4.0]], [[5.0, 8.0, 3.0], [1.0, 8.0, 1.0], [4.0, 2.0, 4.0]]], [[[1.0, 1.0, 8.0], [7.0, 
4.0, 4.0], [4.0, 8.0, 5.0]], [[2.0, 5.0, 3.0], [7.0, 7.0, 5.0], [2.0, 5.0, 5.0]], [[2.0, 5.0, 2.0], [6.0, 3.0, 4.0], [8.0, 7.0, 
4.0]]], [[[3.0, 8.0, 4.0], [6.0, 2.0, 5.0], [2.0, 6.0, 5.0]], [[6.0, 4.0, 1.0], [6.0, 6.0, 1.0], [5.0, 6.0, 5.0]], [[8.0, 3.0, 
1.0], [3.0, 8.0, 5.0], [4.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 8.0], [6.0, 2.0, 4.0], [3.0, 3.0, 2.0]], 
[[1.0, 7.0, 3.0], [5.0, 1.0, 2.0], [3.0, 3.0, 5.0]], [[2.0, 3.0, 2.0], [2.0, 7.0, 8.0], [6.0, 2.0, 3.0]]], [[[7.0, 5.0, 2.0], 
[7.0, 2.0, 4.0], [5.0, 5.0, 2.0]], [[2.0, 1.0, 1.0], [8.0, 5.0, 3.0], [4.0, 2.0, 4.0]], [[5.0, 8.0, 3.0], [1.0, 8.0, 1.0], 
[4.0, 2.0, 4.0]]], [[[1.0, 1.0, 8.0], [7.0, 4.0, 4.0], [4.0, 8.0, 5.0]], [[2.0, 5.0, 3.0], [2.0, 3.0, 1.0], [2.0, 5.0, 5.0]], 
[[2.0, 5.0, 2.0], [6.0, 3.0, 4.0], [8.0, 7.0, 4.0]]], [[[3.0, 8.0, 4.0], [6.0, 2.0, 5.0], [2.0, 6.0, 5.0]], [[6.0, 4.0, 1.0], 
[2.0, 3.0, 2.0], [5.0, 6.0, 5.0]], [[8.0, 3.0, 1.0], [3.0, 8.0, 5.0], [4.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_URCN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 7.0], [4.0, 7.0, 6.0], [3.0, 3.0, 6.0]], [[6.0, 3.0, 4.0], [5.0, 3.0, 3.0], [6.0, 5.0, 
7.0]], [[3.0, 4.0, 1.0], [7.0, 4.0, 1.0], [7.0, 2.0, 6.0]]], [[[2.0, 4.0, 3.0], [3.0, 4.0, 7.0], [7.0, 8.0, 1.0]], [[5.0, 3.0, 
2.0], [2.0, 7.0, 3.0], [6.0, 2.0, 2.0]], [[1.0, 8.0, 2.0], [8.0, 6.0, 5.0], [2.0, 4.0, 4.0]]], [[[7.0, 7.0, 3.0], [3.0, 7.0, 
8.0], [8.0, 2.0, 6.0]], [[1.0, 6.0, 6.0], [7.0, 8.0, 1.0], [4.0, 4.0, 1.0]], [[2.0, 6.0, 3.0], [8.0, 2.0, 2.0], [6.0, 4.0, 
2.0]]], [[[5.0, 6.0, 3.0], [5.0, 3.0, 3.0], [6.0, 2.0, 1.0]], [[2.0, 8.0, 2.0], [8.0, 4.0, 8.0], [6.0, 4.0, 3.0]], [[2.0, 3.0, 
7.0], [4.0, 2.0, 4.0], [8.0, 2.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 3.0, 3.0], [3.0, 1.0, 1.0], [7.0, 6.0, 1.0]]], [[[6.0, 8.0, 3.0], [5.0, 7.0, 6.0], [4.0, 
3.0, 7.0]]], [[[8.0, 3.0, 3.0], [2.0, 6.0, 5.0], [8.0, 3.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 2.0], [1.0, 8.0, 8.0], [8.0, 1.0, 1.0]]], [[[3.0, 4.0, 4.0], [6.0, 3.0, 
5.0], [5.0, 7.0, 3.0]]], [[[5.0, 8.0, 2.0], [2.0, 3.0, 1.0], [6.0, 8.0, 8.0]]]]))
      arg[:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 7.0], [4.0, 7.0, 6.0], [3.0, 3.0, 6.0]], [[5.0, 3.0, 3.0], [3.0, 1.0, 1.0], [7.0, 
6.0, 1.0]], [[3.0, 4.0, 1.0], [7.0, 4.0, 1.0], [7.0, 2.0, 6.0]]], [[[2.0, 4.0, 3.0], [3.0, 4.0, 7.0], [7.0, 8.0, 1.0]], [[6.0, 
8.0, 3.0], [5.0, 7.0, 6.0], [4.0, 3.0, 7.0]], [[1.0, 8.0, 2.0], [8.0, 6.0, 5.0], [2.0, 4.0, 4.0]]], [[[7.0, 7.0, 3.0], [3.0, 
7.0, 8.0], [8.0, 2.0, 6.0]], [[8.0, 3.0, 3.0], [2.0, 6.0, 5.0], [8.0, 3.0, 2.0]], [[2.0, 6.0, 3.0], [8.0, 2.0, 2.0], [6.0, 4.0, 
2.0]]], [[[5.0, 6.0, 3.0], [5.0, 3.0, 3.0], [6.0, 2.0, 1.0]], [[2.0, 8.0, 2.0], [8.0, 4.0, 8.0], [6.0, 4.0, 3.0]], [[2.0, 3.0, 
7.0], [4.0, 2.0, 4.0], [8.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 7.0], [4.0, 7.0, 6.0], [3.0, 3.0, 6.0]], 
[[3.0, 4.0, 2.0], [1.0, 8.0, 8.0], [8.0, 1.0, 1.0]], [[3.0, 4.0, 1.0], [7.0, 4.0, 1.0], [7.0, 2.0, 6.0]]], [[[2.0, 4.0, 3.0], 
[3.0, 4.0, 7.0], [7.0, 8.0, 1.0]], [[3.0, 4.0, 4.0], [6.0, 3.0, 5.0], [5.0, 7.0, 3.0]], [[1.0, 8.0, 2.0], [8.0, 6.0, 5.0], 
[2.0, 4.0, 4.0]]], [[[7.0, 7.0, 3.0], [3.0, 7.0, 8.0], [8.0, 2.0, 6.0]], [[5.0, 8.0, 2.0], [2.0, 3.0, 1.0], [6.0, 8.0, 8.0]], 
[[2.0, 6.0, 3.0], [8.0, 2.0, 2.0], [6.0, 4.0, 2.0]]], [[[5.0, 6.0, 3.0], [5.0, 3.0, 3.0], [6.0, 2.0, 1.0]], [[2.0, 8.0, 2.0], 
[8.0, 4.0, 8.0], [6.0, 4.0, 3.0]], [[2.0, 3.0, 7.0], [4.0, 2.0, 4.0], [8.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PLRN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 4.0], [2.0, 3.0, 8.0], [2.0, 5.0, 7.0]], [[7.0, 4.0, 2.0], [7.0, 3.0, 1.0], [2.0, 1.0, 
1.0]], [[5.0, 7.0, 7.0], [2.0, 5.0, 4.0], [4.0, 8.0, 7.0]]], [[[6.0, 6.0, 6.0], [7.0, 3.0, 7.0], [8.0, 8.0, 1.0]], [[7.0, 7.0, 
2.0], [4.0, 4.0, 5.0], [2.0, 4.0, 6.0]], [[5.0, 6.0, 3.0], [3.0, 2.0, 6.0], [1.0, 5.0, 8.0]]], [[[1.0, 3.0, 5.0], [2.0, 5.0, 
3.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 1.0], [3.0, 5.0, 7.0], [2.0, 6.0, 3.0]], [[5.0, 1.0, 2.0], [2.0, 4.0, 7.0], [2.0, 1.0, 
7.0]]], [[[7.0, 1.0, 2.0], [6.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[2.0, 4.0, 4.0], [7.0, 6.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 8.0, 
4.0], [3.0, 7.0, 3.0], [3.0, 1.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 8.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 2.0, 8.0]]]))
      arg[2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 4.0], [2.0, 3.0, 8.0], [2.0, 5.0, 7.0]], [[7.0, 4.0, 2.0], [7.0, 3.0, 1.0], [2.0, 
1.0, 1.0]], [[5.0, 7.0, 7.0], [2.0, 5.0, 4.0], [4.0, 8.0, 7.0]]], [[[6.0, 6.0, 6.0], [7.0, 3.0, 7.0], [8.0, 8.0, 1.0]], [[7.0, 
7.0, 2.0], [4.0, 4.0, 5.0], [2.0, 4.0, 6.0]], [[5.0, 6.0, 3.0], [3.0, 2.0, 6.0], [1.0, 5.0, 8.0]]], [[[1.0, 3.0, 5.0], [2.0, 
5.0, 3.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 1.0], [3.0, 5.0, 7.0], [2.0, 6.0, 3.0]], [[5.0, 1.0, 2.0], [8.0, 8.0, 1.0], [2.0, 1.0, 
7.0]]], [[[7.0, 1.0, 2.0], [6.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[2.0, 4.0, 4.0], [7.0, 6.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 8.0, 
4.0], [3.0, 7.0, 3.0], [3.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 4.0], [2.0, 3.0, 8.0], [2.0, 5.0, 7.0]], 
[[7.0, 4.0, 2.0], [7.0, 3.0, 1.0], [2.0, 1.0, 1.0]], [[5.0, 7.0, 7.0], [2.0, 5.0, 4.0], [4.0, 8.0, 7.0]]], [[[6.0, 6.0, 6.0], 
[7.0, 3.0, 7.0], [8.0, 8.0, 1.0]], [[7.0, 7.0, 2.0], [4.0, 4.0, 5.0], [2.0, 4.0, 6.0]], [[5.0, 6.0, 3.0], [3.0, 2.0, 6.0], 
[1.0, 5.0, 8.0]]], [[[1.0, 3.0, 5.0], [2.0, 5.0, 3.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 1.0], [3.0, 5.0, 7.0], [2.0, 6.0, 3.0]], 
[[5.0, 1.0, 2.0], [6.0, 2.0, 8.0], [2.0, 1.0, 7.0]]], [[[7.0, 1.0, 2.0], [6.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[2.0, 4.0, 4.0], 
[7.0, 6.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 8.0, 4.0], [3.0, 7.0, 3.0], [3.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLUN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 5.0], [6.0, 3.0, 2.0], [3.0, 2.0, 3.0]], [[4.0, 6.0, 5.0], [1.0, 2.0, 8.0], [5.0, 4.0, 
1.0]], [[1.0, 8.0, 3.0], [7.0, 5.0, 1.0], [3.0, 1.0, 8.0]]], [[[3.0, 2.0, 5.0], [7.0, 5.0, 1.0], [3.0, 7.0, 8.0]], [[4.0, 6.0, 
4.0], [6.0, 7.0, 1.0], [4.0, 7.0, 6.0]], [[6.0, 7.0, 4.0], [6.0, 8.0, 7.0], [8.0, 8.0, 5.0]]], [[[7.0, 7.0, 7.0], [6.0, 5.0, 
3.0], [1.0, 4.0, 5.0]], [[4.0, 1.0, 8.0], [3.0, 4.0, 4.0], [8.0, 3.0, 7.0]], [[3.0, 7.0, 6.0], [1.0, 1.0, 4.0], [5.0, 5.0, 
1.0]]], [[[8.0, 7.0, 3.0], [4.0, 5.0, 7.0], [6.0, 5.0, 7.0]], [[5.0, 2.0, 5.0], [2.0, 2.0, 2.0], [2.0, 6.0, 8.0]], [[7.0, 2.0, 
6.0], [5.0, 1.0, 2.0], [2.0, 2.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 1.0, 1.0], [5.0, 5.0, 3.0]]], [[[4.0, 6.0, 6.0], [3.0, 2.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 3.0], [7.0, 6.0, 1.0]]], [[[8.0, 5.0, 7.0], [7.0, 5.0, 3.0]]]]))
      arg[1:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 5.0], [6.0, 3.0, 2.0], [3.0, 2.0, 3.0]], [[4.0, 6.0, 5.0], [1.0, 2.0, 8.0], [5.0, 
4.0, 1.0]], [[1.0, 8.0, 3.0], [7.0, 5.0, 1.0], [3.0, 1.0, 8.0]]], [[[3.0, 2.0, 5.0], [7.0, 5.0, 1.0], [3.0, 7.0, 8.0]], [[4.0, 
6.0, 4.0], [6.0, 7.0, 1.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 1.0], [5.0, 5.0, 3.0], [8.0, 8.0, 5.0]]], [[[7.0, 7.0, 7.0], [6.0, 
5.0, 3.0], [1.0, 4.0, 5.0]], [[4.0, 1.0, 8.0], [3.0, 4.0, 4.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 6.0], [3.0, 2.0, 3.0], [5.0, 5.0, 
1.0]]], [[[8.0, 7.0, 3.0], [4.0, 5.0, 7.0], [6.0, 5.0, 7.0]], [[5.0, 2.0, 5.0], [2.0, 2.0, 2.0], [2.0, 6.0, 8.0]], [[7.0, 2.0, 
6.0], [5.0, 1.0, 2.0], [2.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 5.0], [6.0, 3.0, 2.0], [3.0, 2.0, 3.0]], 
[[4.0, 6.0, 5.0], [1.0, 2.0, 8.0], [5.0, 4.0, 1.0]], [[1.0, 8.0, 3.0], [7.0, 5.0, 1.0], [3.0, 1.0, 8.0]]], [[[3.0, 2.0, 5.0], 
[7.0, 5.0, 1.0], [3.0, 7.0, 8.0]], [[4.0, 6.0, 4.0], [6.0, 7.0, 1.0], [4.0, 7.0, 6.0]], [[2.0, 4.0, 3.0], [7.0, 6.0, 1.0], 
[8.0, 8.0, 5.0]]], [[[7.0, 7.0, 7.0], [6.0, 5.0, 3.0], [1.0, 4.0, 5.0]], [[4.0, 1.0, 8.0], [3.0, 4.0, 4.0], [8.0, 3.0, 7.0]], 
[[8.0, 5.0, 7.0], [7.0, 5.0, 3.0], [5.0, 5.0, 1.0]]], [[[8.0, 7.0, 3.0], [4.0, 5.0, 7.0], [6.0, 5.0, 7.0]], [[5.0, 2.0, 5.0], 
[2.0, 2.0, 2.0], [2.0, 6.0, 8.0]], [[7.0, 2.0, 6.0], [5.0, 1.0, 2.0], [2.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LUPN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 7.0], [5.0, 8.0, 2.0], [2.0, 7.0, 7.0]], [[5.0, 4.0, 1.0], [6.0, 1.0, 2.0], [8.0, 7.0, 
5.0]], [[2.0, 7.0, 8.0], [6.0, 7.0, 2.0], [7.0, 7.0, 1.0]]], [[[3.0, 2.0, 3.0], [3.0, 2.0, 7.0], [6.0, 3.0, 5.0]], [[4.0, 1.0, 
1.0], [1.0, 1.0, 2.0], [5.0, 1.0, 1.0]], [[5.0, 6.0, 7.0], [8.0, 1.0, 7.0], [3.0, 3.0, 1.0]]], [[[3.0, 1.0, 6.0], [4.0, 4.0, 
7.0], [8.0, 7.0, 7.0]], [[5.0, 2.0, 1.0], [3.0, 2.0, 4.0], [6.0, 5.0, 6.0]], [[7.0, 8.0, 8.0], [7.0, 1.0, 3.0], [4.0, 7.0, 
4.0]]], [[[2.0, 5.0, 1.0], [1.0, 6.0, 5.0], [1.0, 6.0, 1.0]], [[5.0, 7.0, 1.0], [4.0, 5.0, 5.0], [6.0, 4.0, 8.0]], [[6.0, 8.0, 
7.0], [1.0, 4.0, 8.0], [1.0, 8.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 4.0, 8.0], [2.0, 7.0, 1.0]], [[6.0, 4.0, 6.0], [2.0, 2.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 3.0, 7.0], [4.0, 3.0, 3.0]], [[1.0, 2.0, 1.0], [1.0, 4.0, 3.0]]]))
      arg[2:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 7.0], [5.0, 8.0, 2.0], [2.0, 7.0, 7.0]], [[5.0, 4.0, 1.0], [6.0, 1.0, 2.0], [8.0, 
7.0, 5.0]], [[2.0, 7.0, 8.0], [6.0, 7.0, 2.0], [7.0, 7.0, 1.0]]], [[[3.0, 2.0, 3.0], [3.0, 2.0, 7.0], [6.0, 3.0, 5.0]], [[4.0, 
1.0, 1.0], [1.0, 1.0, 2.0], [5.0, 1.0, 1.0]], [[5.0, 6.0, 7.0], [8.0, 1.0, 7.0], [3.0, 3.0, 1.0]]], [[[3.0, 1.0, 6.0], [3.0, 
4.0, 8.0], [8.0, 7.0, 7.0]], [[5.0, 2.0, 1.0], [2.0, 7.0, 1.0], [6.0, 5.0, 6.0]], [[7.0, 8.0, 8.0], [7.0, 1.0, 3.0], [4.0, 7.0, 
4.0]]], [[[2.0, 5.0, 1.0], [6.0, 4.0, 6.0], [1.0, 6.0, 1.0]], [[5.0, 7.0, 1.0], [2.0, 2.0, 8.0], [6.0, 4.0, 8.0]], [[6.0, 8.0, 
7.0], [1.0, 4.0, 8.0], [1.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 7.0], [5.0, 8.0, 2.0], [2.0, 7.0, 7.0]], 
[[5.0, 4.0, 1.0], [6.0, 1.0, 2.0], [8.0, 7.0, 5.0]], [[2.0, 7.0, 8.0], [6.0, 7.0, 2.0], [7.0, 7.0, 1.0]]], [[[3.0, 2.0, 3.0], 
[3.0, 2.0, 7.0], [6.0, 3.0, 5.0]], [[4.0, 1.0, 1.0], [1.0, 1.0, 2.0], [5.0, 1.0, 1.0]], [[5.0, 6.0, 7.0], [8.0, 1.0, 7.0], 
[3.0, 3.0, 1.0]]], [[[3.0, 1.0, 6.0], [4.0, 3.0, 7.0], [8.0, 7.0, 7.0]], [[5.0, 2.0, 1.0], [4.0, 3.0, 3.0], [6.0, 5.0, 6.0]], 
[[7.0, 8.0, 8.0], [7.0, 1.0, 3.0], [4.0, 7.0, 4.0]]], [[[2.0, 5.0, 1.0], [1.0, 2.0, 1.0], [1.0, 6.0, 1.0]], [[5.0, 7.0, 1.0], 
[1.0, 4.0, 3.0], [6.0, 4.0, 8.0]], [[6.0, 8.0, 7.0], [1.0, 4.0, 8.0], [1.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PCPN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 3.0], [3.0, 3.0, 1.0], [7.0, 5.0, 5.0]], [[2.0, 7.0, 8.0], [7.0, 2.0, 4.0], [5.0, 6.0, 
3.0]], [[6.0, 2.0, 5.0], [5.0, 3.0, 8.0], [8.0, 5.0, 3.0]]], [[[8.0, 6.0, 1.0], [4.0, 5.0, 1.0], [5.0, 8.0, 7.0]], [[4.0, 8.0, 
4.0], [5.0, 1.0, 4.0], [8.0, 5.0, 6.0]], [[8.0, 7.0, 5.0], [7.0, 8.0, 6.0], [4.0, 3.0, 6.0]]], [[[2.0, 2.0, 5.0], [3.0, 4.0, 
1.0], [7.0, 7.0, 4.0]], [[3.0, 6.0, 3.0], [4.0, 5.0, 7.0], [2.0, 8.0, 1.0]], [[5.0, 1.0, 2.0], [8.0, 7.0, 4.0], [6.0, 7.0, 
2.0]]], [[[7.0, 5.0, 3.0], [7.0, 2.0, 3.0], [7.0, 4.0, 2.0]], [[4.0, 1.0, 1.0], [3.0, 8.0, 7.0], [2.0, 3.0, 3.0]], [[3.0, 2.0, 
1.0], [4.0, 8.0, 2.0], [8.0, 7.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 1.0, 5.0], [2.0, 3.0, 5.0], [4.0, 4.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 8.0, 4.0], [5.0, 4.0, 2.0], [1.0, 2.0, 2.0]]))
      arg[2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [3.0, 3.0, 1.0], [7.0, 5.0, 5.0]], [[2.0, 7.0, 8.0], [7.0, 2.0, 4.0], [5.0, 
6.0, 3.0]], [[6.0, 2.0, 5.0], [5.0, 3.0, 8.0], [8.0, 5.0, 3.0]]], [[[8.0, 6.0, 1.0], [4.0, 5.0, 1.0], [5.0, 8.0, 7.0]], [[4.0, 
8.0, 4.0], [5.0, 1.0, 4.0], [8.0, 5.0, 6.0]], [[8.0, 7.0, 5.0], [7.0, 8.0, 6.0], [4.0, 3.0, 6.0]]], [[[2.0, 2.0, 5.0], [3.0, 
1.0, 5.0], [7.0, 7.0, 4.0]], [[3.0, 6.0, 3.0], [2.0, 3.0, 5.0], [2.0, 8.0, 1.0]], [[5.0, 1.0, 2.0], [4.0, 4.0, 8.0], [6.0, 7.0, 
2.0]]], [[[7.0, 5.0, 3.0], [7.0, 2.0, 3.0], [7.0, 4.0, 2.0]], [[4.0, 1.0, 1.0], [3.0, 8.0, 7.0], [2.0, 3.0, 3.0]], [[3.0, 2.0, 
1.0], [4.0, 8.0, 2.0], [8.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 3.0], [3.0, 3.0, 1.0], [7.0, 5.0, 5.0]], 
[[2.0, 7.0, 8.0], [7.0, 2.0, 4.0], [5.0, 6.0, 3.0]], [[6.0, 2.0, 5.0], [5.0, 3.0, 8.0], [8.0, 5.0, 3.0]]], [[[8.0, 6.0, 1.0], 
[4.0, 5.0, 1.0], [5.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [5.0, 1.0, 4.0], [8.0, 5.0, 6.0]], [[8.0, 7.0, 5.0], [7.0, 8.0, 6.0], 
[4.0, 3.0, 6.0]]], [[[2.0, 2.0, 5.0], [8.0, 8.0, 4.0], [7.0, 7.0, 4.0]], [[3.0, 6.0, 3.0], [5.0, 4.0, 2.0], [2.0, 8.0, 1.0]], 
[[5.0, 1.0, 2.0], [1.0, 2.0, 2.0], [6.0, 7.0, 2.0]]], [[[7.0, 5.0, 3.0], [7.0, 2.0, 3.0], [7.0, 4.0, 2.0]], [[4.0, 1.0, 1.0], 
[3.0, 8.0, 7.0], [2.0, 3.0, 3.0]], [[3.0, 2.0, 1.0], [4.0, 8.0, 2.0], [8.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_UPUN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 2.0], [4.0, 4.0, 2.0], [2.0, 8.0, 2.0]], [[1.0, 1.0, 1.0], [3.0, 5.0, 3.0], [5.0, 3.0, 
7.0]], [[7.0, 5.0, 5.0], [5.0, 4.0, 1.0], [1.0, 3.0, 8.0]]], [[[3.0, 8.0, 2.0], [4.0, 1.0, 8.0], [8.0, 8.0, 3.0]], [[4.0, 2.0, 
5.0], [2.0, 8.0, 2.0], [4.0, 5.0, 2.0]], [[2.0, 5.0, 6.0], [7.0, 1.0, 2.0], [3.0, 6.0, 7.0]]], [[[1.0, 6.0, 4.0], [6.0, 3.0, 
3.0], [6.0, 3.0, 1.0]], [[1.0, 8.0, 6.0], [4.0, 4.0, 4.0], [4.0, 5.0, 6.0]], [[1.0, 5.0, 8.0], [5.0, 6.0, 4.0], [4.0, 6.0, 
3.0]]], [[[2.0, 5.0, 3.0], [7.0, 5.0, 6.0], [6.0, 1.0, 4.0]], [[8.0, 2.0, 6.0], [2.0, 1.0, 6.0], [3.0, 2.0, 8.0]], [[4.0, 6.0, 
8.0], [2.0, 4.0, 4.0], [8.0, 5.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 4.0, 1.0], [8.0, 5.0, 7.0]], [[8.0, 5.0, 6.0], [3.0, 3.0, 2.0]], [[7.0, 7.0, 8.0], [5.0, 
8.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 3.0, 8.0], [1.0, 5.0, 4.0]], [[5.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[1.0, 2.0, 
4.0], [5.0, 8.0, 2.0]]]))
      arg[:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 2.0], [4.0, 4.0, 2.0], [2.0, 8.0, 2.0]], [[6.0, 4.0, 1.0], [8.0, 5.0, 7.0], [5.0, 
3.0, 7.0]], [[7.0, 5.0, 5.0], [5.0, 4.0, 1.0], [1.0, 3.0, 8.0]]], [[[3.0, 8.0, 2.0], [4.0, 1.0, 8.0], [8.0, 8.0, 3.0]], [[8.0, 
5.0, 6.0], [3.0, 3.0, 2.0], [4.0, 5.0, 2.0]], [[2.0, 5.0, 6.0], [7.0, 1.0, 2.0], [3.0, 6.0, 7.0]]], [[[1.0, 6.0, 4.0], [6.0, 
3.0, 3.0], [6.0, 3.0, 1.0]], [[7.0, 7.0, 8.0], [5.0, 8.0, 7.0], [4.0, 5.0, 6.0]], [[1.0, 5.0, 8.0], [5.0, 6.0, 4.0], [4.0, 6.0, 
3.0]]], [[[2.0, 5.0, 3.0], [7.0, 5.0, 6.0], [6.0, 1.0, 4.0]], [[8.0, 2.0, 6.0], [2.0, 1.0, 6.0], [3.0, 2.0, 8.0]], [[4.0, 6.0, 
8.0], [2.0, 4.0, 4.0], [8.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 2.0], [4.0, 4.0, 2.0], [2.0, 8.0, 2.0]], 
[[8.0, 3.0, 8.0], [1.0, 5.0, 4.0], [5.0, 3.0, 7.0]], [[7.0, 5.0, 5.0], [5.0, 4.0, 1.0], [1.0, 3.0, 8.0]]], [[[3.0, 8.0, 2.0], 
[4.0, 1.0, 8.0], [8.0, 8.0, 3.0]], [[5.0, 5.0, 2.0], [5.0, 2.0, 4.0], [4.0, 5.0, 2.0]], [[2.0, 5.0, 6.0], [7.0, 1.0, 2.0], 
[3.0, 6.0, 7.0]]], [[[1.0, 6.0, 4.0], [6.0, 3.0, 3.0], [6.0, 3.0, 1.0]], [[1.0, 2.0, 4.0], [5.0, 8.0, 2.0], [4.0, 5.0, 6.0]], 
[[1.0, 5.0, 8.0], [5.0, 6.0, 4.0], [4.0, 6.0, 3.0]]], [[[2.0, 5.0, 3.0], [7.0, 5.0, 6.0], [6.0, 1.0, 4.0]], [[8.0, 2.0, 6.0], 
[2.0, 1.0, 6.0], [3.0, 2.0, 8.0]], [[4.0, 6.0, 8.0], [2.0, 4.0, 4.0], [8.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LURN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 2.0], [7.0, 5.0, 1.0], [8.0, 5.0, 1.0]], [[7.0, 8.0, 5.0], [1.0, 2.0, 6.0], [1.0, 4.0, 
5.0]], [[7.0, 4.0, 1.0], [3.0, 3.0, 4.0], [8.0, 8.0, 3.0]]], [[[7.0, 6.0, 2.0], [6.0, 2.0, 6.0], [6.0, 1.0, 8.0]], [[3.0, 5.0, 
8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 5.0]], [[1.0, 4.0, 3.0], [6.0, 4.0, 2.0], [1.0, 1.0, 3.0]]], [[[1.0, 5.0, 4.0], [7.0, 7.0, 
7.0], [6.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], [7.0, 2.0, 7.0], [5.0, 3.0, 8.0]], [[8.0, 5.0, 6.0], [3.0, 8.0, 6.0], [1.0, 8.0, 
1.0]]], [[[1.0, 2.0, 7.0], [5.0, 8.0, 1.0], [1.0, 8.0, 2.0]], [[6.0, 5.0, 2.0], [4.0, 8.0, 3.0], [4.0, 3.0, 2.0]], [[6.0, 7.0, 
1.0], [3.0, 8.0, 3.0], [4.0, 5.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 7.0, 6.0]], [[5.0, 2.0, 7.0]]], [[[6.0, 7.0, 2.0]], [[8.0, 8.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 3.0]], [[4.0, 5.0, 4.0]]], [[[3.0, 1.0, 1.0]], [[5.0, 4.0, 3.0]]]]))
      arg[2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 2.0], [7.0, 5.0, 1.0], [8.0, 5.0, 1.0]], [[7.0, 8.0, 5.0], [1.0, 2.0, 6.0], [1.0, 
4.0, 5.0]], [[7.0, 4.0, 1.0], [3.0, 3.0, 4.0], [8.0, 8.0, 3.0]]], [[[7.0, 6.0, 2.0], [6.0, 2.0, 6.0], [6.0, 1.0, 8.0]], [[3.0, 
5.0, 8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 5.0]], [[1.0, 4.0, 3.0], [6.0, 4.0, 2.0], [1.0, 1.0, 3.0]]], [[[1.0, 5.0, 4.0], [4.0, 
7.0, 6.0], [6.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], [5.0, 2.0, 7.0], [5.0, 3.0, 8.0]], [[8.0, 5.0, 6.0], [3.0, 8.0, 6.0], [1.0, 8.0, 
1.0]]], [[[1.0, 2.0, 7.0], [6.0, 7.0, 2.0], [1.0, 8.0, 2.0]], [[6.0, 5.0, 2.0], [8.0, 8.0, 7.0], [4.0, 3.0, 2.0]], [[6.0, 7.0, 
1.0], [3.0, 8.0, 3.0], [4.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 2.0], [7.0, 5.0, 1.0], [8.0, 5.0, 1.0]], 
[[7.0, 8.0, 5.0], [1.0, 2.0, 6.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 1.0], [3.0, 3.0, 4.0], [8.0, 8.0, 3.0]]], [[[7.0, 6.0, 2.0], 
[6.0, 2.0, 6.0], [6.0, 1.0, 8.0]], [[3.0, 5.0, 8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 5.0]], [[1.0, 4.0, 3.0], [6.0, 4.0, 2.0], 
[1.0, 1.0, 3.0]]], [[[1.0, 5.0, 4.0], [2.0, 8.0, 3.0], [6.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], [4.0, 5.0, 4.0], [5.0, 3.0, 8.0]], 
[[8.0, 5.0, 6.0], [3.0, 8.0, 6.0], [1.0, 8.0, 1.0]]], [[[1.0, 2.0, 7.0], [3.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[6.0, 5.0, 2.0], 
[5.0, 4.0, 3.0], [4.0, 3.0, 2.0]], [[6.0, 7.0, 1.0], [3.0, 8.0, 3.0], [4.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLLN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 6.0], [6.0, 2.0, 8.0], [7.0, 2.0, 6.0]], [[5.0, 7.0, 1.0], [2.0, 2.0, 6.0], [3.0, 2.0, 
8.0]], [[3.0, 5.0, 5.0], [1.0, 2.0, 5.0], [8.0, 4.0, 3.0]]], [[[3.0, 3.0, 4.0], [8.0, 1.0, 7.0], [3.0, 3.0, 2.0]], [[3.0, 1.0, 
8.0], [6.0, 1.0, 4.0], [8.0, 2.0, 7.0]], [[4.0, 2.0, 4.0], [3.0, 1.0, 8.0], [2.0, 3.0, 8.0]]], [[[5.0, 8.0, 7.0], [2.0, 5.0, 
7.0], [2.0, 4.0, 7.0]], [[3.0, 8.0, 4.0], [8.0, 1.0, 1.0], [1.0, 8.0, 5.0]], [[1.0, 7.0, 6.0], [7.0, 6.0, 4.0], [8.0, 7.0, 
1.0]]], [[[3.0, 3.0, 5.0], [5.0, 1.0, 6.0], [3.0, 6.0, 4.0]], [[2.0, 4.0, 8.0], [5.0, 4.0, 1.0], [5.0, 8.0, 2.0]], [[2.0, 2.0, 
6.0], [7.0, 7.0, 7.0], [4.0, 1.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 6.0, 5.0]]], [[[7.0, 1.0, 8.0]]], [[[1.0, 7.0, 2.0]]], [[[2.0, 1.0, 
7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 2.0]]], [[[8.0, 8.0, 2.0]]], [[[7.0, 3.0, 4.0]]], [[[7.0, 5.0, 4.0]]]]))
      arg[:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 6.0], [6.0, 2.0, 8.0], [7.0, 2.0, 6.0]], [[5.0, 7.0, 1.0], [2.0, 2.0, 6.0], [3.0, 
2.0, 8.0]], [[3.0, 5.0, 5.0], [1.0, 2.0, 5.0], [1.0, 6.0, 5.0]]], [[[3.0, 3.0, 4.0], [8.0, 1.0, 7.0], [3.0, 3.0, 2.0]], [[3.0, 
1.0, 8.0], [6.0, 1.0, 4.0], [8.0, 2.0, 7.0]], [[4.0, 2.0, 4.0], [3.0, 1.0, 8.0], [7.0, 1.0, 8.0]]], [[[5.0, 8.0, 7.0], [2.0, 
5.0, 7.0], [2.0, 4.0, 7.0]], [[3.0, 8.0, 4.0], [8.0, 1.0, 1.0], [1.0, 8.0, 5.0]], [[1.0, 7.0, 6.0], [7.0, 6.0, 4.0], [1.0, 7.0, 
2.0]]], [[[3.0, 3.0, 5.0], [5.0, 1.0, 6.0], [3.0, 6.0, 4.0]], [[2.0, 4.0, 8.0], [5.0, 4.0, 1.0], [5.0, 8.0, 2.0]], [[2.0, 2.0, 
6.0], [7.0, 7.0, 7.0], [2.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 6.0], [6.0, 2.0, 8.0], [7.0, 2.0, 6.0]], 
[[5.0, 7.0, 1.0], [2.0, 2.0, 6.0], [3.0, 2.0, 8.0]], [[3.0, 5.0, 5.0], [1.0, 2.0, 5.0], [5.0, 6.0, 2.0]]], [[[3.0, 3.0, 4.0], 
[8.0, 1.0, 7.0], [3.0, 3.0, 2.0]], [[3.0, 1.0, 8.0], [6.0, 1.0, 4.0], [8.0, 2.0, 7.0]], [[4.0, 2.0, 4.0], [3.0, 1.0, 8.0], 
[8.0, 8.0, 2.0]]], [[[5.0, 8.0, 7.0], [2.0, 5.0, 7.0], [2.0, 4.0, 7.0]], [[3.0, 8.0, 4.0], [8.0, 1.0, 1.0], [1.0, 8.0, 5.0]], 
[[1.0, 7.0, 6.0], [7.0, 6.0, 4.0], [7.0, 3.0, 4.0]]], [[[3.0, 3.0, 5.0], [5.0, 1.0, 6.0], [3.0, 6.0, 4.0]], [[2.0, 4.0, 8.0], 
[5.0, 4.0, 1.0], [5.0, 8.0, 2.0]], [[2.0, 2.0, 6.0], [7.0, 7.0, 7.0], [7.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CCLN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 5.0], [3.0, 3.0, 6.0], [8.0, 6.0, 6.0]], [[1.0, 5.0, 4.0], [3.0, 3.0, 3.0], [5.0, 8.0, 
4.0]], [[1.0, 4.0, 6.0], [1.0, 6.0, 4.0], [8.0, 1.0, 4.0]]], [[[6.0, 4.0, 3.0], [8.0, 6.0, 4.0], [4.0, 6.0, 7.0]], [[5.0, 3.0, 
4.0], [5.0, 3.0, 2.0], [3.0, 5.0, 1.0]], [[1.0, 6.0, 3.0], [6.0, 6.0, 4.0], [6.0, 4.0, 4.0]]], [[[7.0, 5.0, 1.0], [6.0, 7.0, 
2.0], [8.0, 8.0, 4.0]], [[2.0, 4.0, 8.0], [4.0, 7.0, 3.0], [8.0, 8.0, 5.0]], [[3.0, 1.0, 6.0], [2.0, 1.0, 7.0], [2.0, 6.0, 
5.0]]], [[[2.0, 7.0, 2.0], [8.0, 3.0, 5.0], [8.0, 7.0, 1.0]], [[7.0, 6.0, 1.0], [7.0, 6.0, 5.0], [8.0, 8.0, 4.0]], [[5.0, 6.0, 
6.0], [2.0, 3.0, 7.0], [4.0, 6.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 3.0, 4.0]], [[7.0, 1.0, 5.0]], [[6.0, 8.0, 6.0]]], [[[8.0, 2.0, 5.0]], [[2.0, 8.0, 3.0]], 
[[2.0, 2.0, 4.0]]], [[[5.0, 8.0, 7.0]], [[3.0, 7.0, 7.0]], [[5.0, 7.0, 7.0]]], [[[4.0, 3.0, 1.0]], [[1.0, 4.0, 2.0]], [[8.0, 
7.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 5.0]], [[4.0, 4.0, 4.0]], [[3.0, 3.0, 6.0]]], [[[8.0, 4.0, 5.0]], [[2.0, 
1.0, 5.0]], [[8.0, 1.0, 3.0]]], [[[1.0, 2.0, 8.0]], [[2.0, 7.0, 5.0]], [[7.0, 2.0, 2.0]]], [[[4.0, 5.0, 1.0]], [[4.0, 5.0, 
2.0]], [[2.0, 7.0, 6.0]]]]))
      arg[:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 5.0], [3.0, 3.0, 6.0], [1.0, 3.0, 4.0]], [[1.0, 5.0, 4.0], [3.0, 3.0, 3.0], [7.0, 
1.0, 5.0]], [[1.0, 4.0, 6.0], [1.0, 6.0, 4.0], [6.0, 8.0, 6.0]]], [[[6.0, 4.0, 3.0], [8.0, 6.0, 4.0], [8.0, 2.0, 5.0]], [[5.0, 
3.0, 4.0], [5.0, 3.0, 2.0], [2.0, 8.0, 3.0]], [[1.0, 6.0, 3.0], [6.0, 6.0, 4.0], [2.0, 2.0, 4.0]]], [[[7.0, 5.0, 1.0], [6.0, 
7.0, 2.0], [5.0, 8.0, 7.0]], [[2.0, 4.0, 8.0], [4.0, 7.0, 3.0], [3.0, 7.0, 7.0]], [[3.0, 1.0, 6.0], [2.0, 1.0, 7.0], [5.0, 7.0, 
7.0]]], [[[2.0, 7.0, 2.0], [8.0, 3.0, 5.0], [4.0, 3.0, 1.0]], [[7.0, 6.0, 1.0], [7.0, 6.0, 5.0], [1.0, 4.0, 2.0]], [[5.0, 6.0, 
6.0], [2.0, 3.0, 7.0], [8.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 5.0], [3.0, 3.0, 6.0], [6.0, 1.0, 5.0]], 
[[1.0, 5.0, 4.0], [3.0, 3.0, 3.0], [4.0, 4.0, 4.0]], [[1.0, 4.0, 6.0], [1.0, 6.0, 4.0], [3.0, 3.0, 6.0]]], [[[6.0, 4.0, 3.0], 
[8.0, 6.0, 4.0], [8.0, 4.0, 5.0]], [[5.0, 3.0, 4.0], [5.0, 3.0, 2.0], [2.0, 1.0, 5.0]], [[1.0, 6.0, 3.0], [6.0, 6.0, 4.0], 
[8.0, 1.0, 3.0]]], [[[7.0, 5.0, 1.0], [6.0, 7.0, 2.0], [1.0, 2.0, 8.0]], [[2.0, 4.0, 8.0], [4.0, 7.0, 3.0], [2.0, 7.0, 5.0]], 
[[3.0, 1.0, 6.0], [2.0, 1.0, 7.0], [7.0, 2.0, 2.0]]], [[[2.0, 7.0, 2.0], [8.0, 3.0, 5.0], [4.0, 5.0, 1.0]], [[7.0, 6.0, 1.0], 
[7.0, 6.0, 5.0], [4.0, 5.0, 2.0]], [[5.0, 6.0, 6.0], [2.0, 3.0, 7.0], [2.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_LRLN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 6.0], [3.0, 1.0, 4.0], [7.0, 3.0, 8.0]], [[8.0, 6.0, 2.0], [3.0, 4.0, 7.0], [4.0, 5.0, 
5.0]], [[6.0, 2.0, 4.0], [1.0, 7.0, 3.0], [7.0, 3.0, 3.0]]], [[[8.0, 3.0, 6.0], [4.0, 8.0, 5.0], [1.0, 6.0, 6.0]], [[8.0, 7.0, 
6.0], [5.0, 7.0, 6.0], [2.0, 7.0, 7.0]], [[3.0, 3.0, 7.0], [5.0, 5.0, 8.0], [7.0, 1.0, 6.0]]], [[[6.0, 4.0, 2.0], [7.0, 4.0, 
5.0], [2.0, 8.0, 4.0]], [[4.0, 7.0, 5.0], [8.0, 1.0, 5.0], [2.0, 4.0, 8.0]], [[3.0, 4.0, 8.0], [8.0, 1.0, 2.0], [2.0, 1.0, 
7.0]]], [[[4.0, 5.0, 3.0], [6.0, 7.0, 3.0], [8.0, 6.0, 8.0]], [[4.0, 8.0, 1.0], [5.0, 1.0, 2.0], [8.0, 6.0, 2.0]], [[3.0, 6.0, 
4.0], [2.0, 6.0, 7.0], [2.0, 6.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 2.0, 4.0]]], [[[7.0, 6.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 8.0]]], [[[2.0, 8.0, 4.0]]]]))
      arg[2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 6.0], [3.0, 1.0, 4.0], [7.0, 3.0, 8.0]], [[8.0, 6.0, 2.0], [3.0, 4.0, 7.0], [4.0, 
5.0, 5.0]], [[6.0, 2.0, 4.0], [1.0, 7.0, 3.0], [7.0, 3.0, 3.0]]], [[[8.0, 3.0, 6.0], [4.0, 8.0, 5.0], [1.0, 6.0, 6.0]], [[8.0, 
7.0, 6.0], [5.0, 7.0, 6.0], [2.0, 7.0, 7.0]], [[3.0, 3.0, 7.0], [5.0, 5.0, 8.0], [7.0, 1.0, 6.0]]], [[[6.0, 4.0, 2.0], [7.0, 
4.0, 5.0], [2.0, 8.0, 4.0]], [[4.0, 7.0, 5.0], [8.0, 1.0, 5.0], [4.0, 2.0, 4.0]], [[3.0, 4.0, 8.0], [8.0, 1.0, 2.0], [2.0, 1.0, 
7.0]]], [[[4.0, 5.0, 3.0], [6.0, 7.0, 3.0], [8.0, 6.0, 8.0]], [[4.0, 8.0, 1.0], [5.0, 1.0, 2.0], [7.0, 6.0, 2.0]], [[3.0, 6.0, 
4.0], [2.0, 6.0, 7.0], [2.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 6.0], [3.0, 1.0, 4.0], [7.0, 3.0, 8.0]], 
[[8.0, 6.0, 2.0], [3.0, 4.0, 7.0], [4.0, 5.0, 5.0]], [[6.0, 2.0, 4.0], [1.0, 7.0, 3.0], [7.0, 3.0, 3.0]]], [[[8.0, 3.0, 6.0], 
[4.0, 8.0, 5.0], [1.0, 6.0, 6.0]], [[8.0, 7.0, 6.0], [5.0, 7.0, 6.0], [2.0, 7.0, 7.0]], [[3.0, 3.0, 7.0], [5.0, 5.0, 8.0], 
[7.0, 1.0, 6.0]]], [[[6.0, 4.0, 2.0], [7.0, 4.0, 5.0], [2.0, 8.0, 4.0]], [[4.0, 7.0, 5.0], [8.0, 1.0, 5.0], [4.0, 5.0, 8.0]], 
[[3.0, 4.0, 8.0], [8.0, 1.0, 2.0], [2.0, 1.0, 7.0]]], [[[4.0, 5.0, 3.0], [6.0, 7.0, 3.0], [8.0, 6.0, 8.0]], [[4.0, 8.0, 1.0], 
[5.0, 1.0, 2.0], [2.0, 8.0, 4.0]], [[3.0, 6.0, 4.0], [2.0, 6.0, 7.0], [2.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RURN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 3.0], [3.0, 5.0, 6.0], [6.0, 3.0, 6.0]], [[7.0, 8.0, 5.0], [2.0, 1.0, 6.0], [4.0, 1.0, 
2.0]], [[3.0, 6.0, 8.0], [2.0, 1.0, 5.0], [7.0, 2.0, 3.0]]], [[[8.0, 6.0, 7.0], [7.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[3.0, 8.0, 
7.0], [3.0, 7.0, 3.0], [6.0, 1.0, 4.0]], [[5.0, 2.0, 8.0], [4.0, 5.0, 3.0], [2.0, 7.0, 6.0]]], [[[2.0, 4.0, 1.0], [7.0, 3.0, 
7.0], [8.0, 7.0, 2.0]], [[4.0, 2.0, 5.0], [8.0, 3.0, 3.0], [4.0, 7.0, 5.0]], [[3.0, 5.0, 2.0], [1.0, 3.0, 2.0], [5.0, 8.0, 
5.0]]], [[[1.0, 7.0, 4.0], [8.0, 1.0, 5.0], [7.0, 4.0, 2.0]], [[2.0, 5.0, 8.0], [4.0, 8.0, 6.0], [1.0, 8.0, 8.0]], [[2.0, 8.0, 
3.0], [3.0, 3.0, 6.0], [8.0, 5.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 1.0, 4.0]], [[8.0, 3.0, 3.0]]], [[[7.0, 2.0, 1.0]], [[7.0, 5.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 4.0]], [[8.0, 4.0, 3.0]]], [[[2.0, 2.0, 5.0]], [[3.0, 1.0, 3.0]]]]))
      arg[1:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 3.0], [3.0, 5.0, 6.0], [6.0, 3.0, 6.0]], [[7.0, 8.0, 5.0], [2.0, 1.0, 6.0], [4.0, 
1.0, 2.0]], [[3.0, 6.0, 8.0], [2.0, 1.0, 5.0], [7.0, 2.0, 3.0]]], [[[8.0, 6.0, 7.0], [8.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[3.0, 
8.0, 7.0], [8.0, 3.0, 3.0], [6.0, 1.0, 4.0]], [[5.0, 2.0, 8.0], [4.0, 5.0, 3.0], [2.0, 7.0, 6.0]]], [[[2.0, 4.0, 1.0], [7.0, 
2.0, 1.0], [8.0, 7.0, 2.0]], [[4.0, 2.0, 5.0], [7.0, 5.0, 1.0], [4.0, 7.0, 5.0]], [[3.0, 5.0, 2.0], [1.0, 3.0, 2.0], [5.0, 8.0, 
5.0]]], [[[1.0, 7.0, 4.0], [8.0, 1.0, 5.0], [7.0, 4.0, 2.0]], [[2.0, 5.0, 8.0], [4.0, 8.0, 6.0], [1.0, 8.0, 8.0]], [[2.0, 8.0, 
3.0], [3.0, 3.0, 6.0], [8.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 3.0], [3.0, 5.0, 6.0], [6.0, 3.0, 6.0]], 
[[7.0, 8.0, 5.0], [2.0, 1.0, 6.0], [4.0, 1.0, 2.0]], [[3.0, 6.0, 8.0], [2.0, 1.0, 5.0], [7.0, 2.0, 3.0]]], [[[8.0, 6.0, 7.0], 
[5.0, 2.0, 4.0], [1.0, 3.0, 4.0]], [[3.0, 8.0, 7.0], [8.0, 4.0, 3.0], [6.0, 1.0, 4.0]], [[5.0, 2.0, 8.0], [4.0, 5.0, 3.0], 
[2.0, 7.0, 6.0]]], [[[2.0, 4.0, 1.0], [2.0, 2.0, 5.0], [8.0, 7.0, 2.0]], [[4.0, 2.0, 5.0], [3.0, 1.0, 3.0], [4.0, 7.0, 5.0]], 
[[3.0, 5.0, 2.0], [1.0, 3.0, 2.0], [5.0, 8.0, 5.0]]], [[[1.0, 7.0, 4.0], [8.0, 1.0, 5.0], [7.0, 4.0, 2.0]], [[2.0, 5.0, 8.0], 
[4.0, 8.0, 6.0], [1.0, 8.0, 8.0]], [[2.0, 8.0, 3.0], [3.0, 3.0, 6.0], [8.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RLCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 2.0], [2.0, 6.0, 8.0], [3.0, 1.0, 7.0]], [[1.0, 4.0, 2.0], [8.0, 2.0, 3.0], [4.0, 7.0, 
8.0]], [[6.0, 1.0, 7.0], [8.0, 6.0, 8.0], [5.0, 6.0, 2.0]]], [[[2.0, 7.0, 3.0], [6.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[1.0, 6.0, 
4.0], [4.0, 3.0, 3.0], [8.0, 8.0, 7.0]], [[2.0, 7.0, 6.0], [6.0, 3.0, 8.0], [1.0, 1.0, 8.0]]], [[[3.0, 1.0, 2.0], [7.0, 2.0, 
8.0], [2.0, 3.0, 5.0]], [[4.0, 5.0, 6.0], [7.0, 2.0, 7.0], [3.0, 5.0, 7.0]], [[4.0, 3.0, 7.0], [2.0, 5.0, 4.0], [8.0, 5.0, 
3.0]]], [[[2.0, 2.0, 5.0], [2.0, 1.0, 4.0], [6.0, 7.0, 1.0]], [[8.0, 1.0, 6.0], [7.0, 4.0, 7.0], [7.0, 3.0, 6.0]], [[3.0, 7.0, 
6.0], [1.0, 6.0, 2.0], [2.0, 4.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 5.0, 4.0], [5.0, 6.0, 7.0], [5.0, 5.0, 5.0]]], [[[3.0, 7.0, 4.0], [8.0, 7.0, 8.0], [2.0, 
8.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 5.0], [8.0, 1.0, 6.0], [8.0, 3.0, 2.0]]], [[[4.0, 7.0, 2.0], [6.0, 2.0, 
6.0], [3.0, 3.0, 1.0]]]]))
      arg[1:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 2.0], [2.0, 6.0, 8.0], [3.0, 1.0, 7.0]], [[1.0, 4.0, 2.0], [8.0, 2.0, 3.0], [4.0, 
7.0, 8.0]], [[6.0, 1.0, 7.0], [8.0, 6.0, 8.0], [5.0, 6.0, 2.0]]], [[[2.0, 7.0, 3.0], [6.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[1.0, 
6.0, 4.0], [4.0, 3.0, 3.0], [8.0, 8.0, 7.0]], [[2.0, 5.0, 4.0], [5.0, 6.0, 7.0], [5.0, 5.0, 5.0]]], [[[3.0, 1.0, 2.0], [7.0, 
2.0, 8.0], [2.0, 3.0, 5.0]], [[4.0, 5.0, 6.0], [7.0, 2.0, 7.0], [3.0, 5.0, 7.0]], [[3.0, 7.0, 4.0], [8.0, 7.0, 8.0], [2.0, 8.0, 
3.0]]], [[[2.0, 2.0, 5.0], [2.0, 1.0, 4.0], [6.0, 7.0, 1.0]], [[8.0, 1.0, 6.0], [7.0, 4.0, 7.0], [7.0, 3.0, 6.0]], [[3.0, 7.0, 
6.0], [1.0, 6.0, 2.0], [2.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 2.0], [2.0, 6.0, 8.0], [3.0, 1.0, 7.0]], 
[[1.0, 4.0, 2.0], [8.0, 2.0, 3.0], [4.0, 7.0, 8.0]], [[6.0, 1.0, 7.0], [8.0, 6.0, 8.0], [5.0, 6.0, 2.0]]], [[[2.0, 7.0, 3.0], 
[6.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[1.0, 6.0, 4.0], [4.0, 3.0, 3.0], [8.0, 8.0, 7.0]], [[3.0, 8.0, 5.0], [8.0, 1.0, 6.0], 
[8.0, 3.0, 2.0]]], [[[3.0, 1.0, 2.0], [7.0, 2.0, 8.0], [2.0, 3.0, 5.0]], [[4.0, 5.0, 6.0], [7.0, 2.0, 7.0], [3.0, 5.0, 7.0]], 
[[4.0, 7.0, 2.0], [6.0, 2.0, 6.0], [3.0, 3.0, 1.0]]], [[[2.0, 2.0, 5.0], [2.0, 1.0, 4.0], [6.0, 7.0, 1.0]], [[8.0, 1.0, 6.0], 
[7.0, 4.0, 7.0], [7.0, 3.0, 6.0]], [[3.0, 7.0, 6.0], [1.0, 6.0, 2.0], [2.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_RRUN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 4.0], [8.0, 3.0, 1.0], [5.0, 5.0, 5.0]], [[5.0, 1.0, 7.0], [2.0, 7.0, 1.0], [1.0, 3.0, 
5.0]], [[7.0, 3.0, 2.0], [1.0, 2.0, 5.0], [6.0, 2.0, 8.0]]], [[[2.0, 4.0, 3.0], [6.0, 5.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 5.0, 
6.0], [8.0, 5.0, 3.0], [4.0, 1.0, 4.0]], [[6.0, 7.0, 8.0], [5.0, 3.0, 3.0], [5.0, 7.0, 4.0]]], [[[3.0, 6.0, 5.0], [2.0, 7.0, 
7.0], [1.0, 6.0, 1.0]], [[3.0, 5.0, 4.0], [3.0, 2.0, 8.0], [6.0, 6.0, 5.0]], [[7.0, 6.0, 3.0], [6.0, 3.0, 7.0], [8.0, 5.0, 
4.0]]], [[[8.0, 5.0, 4.0], [5.0, 1.0, 2.0], [4.0, 8.0, 7.0]], [[8.0, 5.0, 4.0], [3.0, 5.0, 5.0], [7.0, 6.0, 5.0]], [[1.0, 7.0, 
3.0], [5.0, 1.0, 3.0], [6.0, 3.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 3.0, 7.0], [8.0, 5.0, 3.0]]], [[[4.0, 8.0, 7.0], [7.0, 6.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 3.0], [2.0, 6.0, 8.0]]], [[[3.0, 4.0, 5.0], [1.0, 6.0, 4.0]]]]))
      arg[1:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 4.0], [8.0, 3.0, 1.0], [5.0, 5.0, 5.0]], [[5.0, 1.0, 7.0], [2.0, 7.0, 1.0], [1.0, 
3.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 2.0, 5.0], [6.0, 2.0, 8.0]]], [[[2.0, 4.0, 3.0], [6.0, 5.0, 3.0], [2.0, 5.0, 6.0]], [[1.0, 
3.0, 7.0], [8.0, 5.0, 3.0], [4.0, 1.0, 4.0]], [[6.0, 7.0, 8.0], [5.0, 3.0, 3.0], [5.0, 7.0, 4.0]]], [[[3.0, 6.0, 5.0], [2.0, 
7.0, 7.0], [1.0, 6.0, 1.0]], [[4.0, 8.0, 7.0], [7.0, 6.0, 2.0], [6.0, 6.0, 5.0]], [[7.0, 6.0, 3.0], [6.0, 3.0, 7.0], [8.0, 5.0, 
4.0]]], [[[8.0, 5.0, 4.0], [5.0, 1.0, 2.0], [4.0, 8.0, 7.0]], [[8.0, 5.0, 4.0], [3.0, 5.0, 5.0], [7.0, 6.0, 5.0]], [[1.0, 7.0, 
3.0], [5.0, 1.0, 3.0], [6.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 4.0], [8.0, 3.0, 1.0], [5.0, 5.0, 5.0]], 
[[5.0, 1.0, 7.0], [2.0, 7.0, 1.0], [1.0, 3.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 2.0, 5.0], [6.0, 2.0, 8.0]]], [[[2.0, 4.0, 3.0], 
[6.0, 5.0, 3.0], [2.0, 5.0, 6.0]], [[8.0, 2.0, 3.0], [2.0, 6.0, 8.0], [4.0, 1.0, 4.0]], [[6.0, 7.0, 8.0], [5.0, 3.0, 3.0], 
[5.0, 7.0, 4.0]]], [[[3.0, 6.0, 5.0], [2.0, 7.0, 7.0], [1.0, 6.0, 1.0]], [[3.0, 4.0, 5.0], [1.0, 6.0, 4.0], [6.0, 6.0, 5.0]], 
[[7.0, 6.0, 3.0], [6.0, 3.0, 7.0], [8.0, 5.0, 4.0]]], [[[8.0, 5.0, 4.0], [5.0, 1.0, 2.0], [4.0, 8.0, 7.0]], [[8.0, 5.0, 4.0], 
[3.0, 5.0, 5.0], [7.0, 6.0, 5.0]], [[1.0, 7.0, 3.0], [5.0, 1.0, 3.0], [6.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_PCUN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 3.0], [7.0, 2.0, 8.0], [4.0, 7.0, 5.0]], [[3.0, 2.0, 8.0], [7.0, 4.0, 3.0], [1.0, 7.0, 
1.0]], [[7.0, 6.0, 7.0], [2.0, 5.0, 1.0], [8.0, 7.0, 1.0]]], [[[7.0, 3.0, 3.0], [1.0, 4.0, 6.0], [5.0, 3.0, 3.0]], [[3.0, 1.0, 
4.0], [3.0, 6.0, 8.0], [6.0, 5.0, 7.0]], [[4.0, 4.0, 5.0], [4.0, 1.0, 5.0], [3.0, 6.0, 2.0]]], [[[4.0, 4.0, 8.0], [8.0, 5.0, 
1.0], [8.0, 6.0, 6.0]], [[5.0, 8.0, 8.0], [8.0, 2.0, 2.0], [8.0, 4.0, 1.0]], [[8.0, 8.0, 2.0], [4.0, 6.0, 7.0], [2.0, 4.0, 
6.0]]], [[[6.0, 8.0, 7.0], [5.0, 3.0, 6.0], [1.0, 2.0, 4.0]], [[2.0, 8.0, 2.0], [2.0, 2.0, 8.0], [5.0, 5.0, 5.0]], [[4.0, 2.0, 
7.0], [2.0, 5.0, 8.0], [3.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 3.0, 1.0], [7.0, 3.0, 5.0]], [[6.0, 5.0, 3.0], [6.0, 1.0, 3.0]], [[1.0, 3.0, 8.0], [1.0, 
5.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 1.0, 6.0], [8.0, 4.0, 7.0]], [[5.0, 6.0, 4.0], [7.0, 6.0, 5.0]], [[8.0, 5.0, 
2.0], [5.0, 1.0, 8.0]]]))
      arg[2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 3.0], [7.0, 2.0, 8.0], [4.0, 7.0, 5.0]], [[3.0, 2.0, 8.0], [7.0, 4.0, 3.0], [1.0, 
7.0, 1.0]], [[7.0, 6.0, 7.0], [2.0, 5.0, 1.0], [8.0, 7.0, 1.0]]], [[[7.0, 3.0, 3.0], [1.0, 4.0, 6.0], [5.0, 3.0, 3.0]], [[3.0, 
1.0, 4.0], [3.0, 6.0, 8.0], [6.0, 5.0, 7.0]], [[4.0, 4.0, 5.0], [4.0, 1.0, 5.0], [3.0, 6.0, 2.0]]], [[[7.0, 3.0, 1.0], [7.0, 
3.0, 5.0], [8.0, 6.0, 6.0]], [[6.0, 5.0, 3.0], [6.0, 1.0, 3.0], [8.0, 4.0, 1.0]], [[1.0, 3.0, 8.0], [1.0, 5.0, 7.0], [2.0, 4.0, 
6.0]]], [[[6.0, 8.0, 7.0], [5.0, 3.0, 6.0], [1.0, 2.0, 4.0]], [[2.0, 8.0, 2.0], [2.0, 2.0, 8.0], [5.0, 5.0, 5.0]], [[4.0, 2.0, 
7.0], [2.0, 5.0, 8.0], [3.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 3.0], [7.0, 2.0, 8.0], [4.0, 7.0, 5.0]], 
[[3.0, 2.0, 8.0], [7.0, 4.0, 3.0], [1.0, 7.0, 1.0]], [[7.0, 6.0, 7.0], [2.0, 5.0, 1.0], [8.0, 7.0, 1.0]]], [[[7.0, 3.0, 3.0], 
[1.0, 4.0, 6.0], [5.0, 3.0, 3.0]], [[3.0, 1.0, 4.0], [3.0, 6.0, 8.0], [6.0, 5.0, 7.0]], [[4.0, 4.0, 5.0], [4.0, 1.0, 5.0], 
[3.0, 6.0, 2.0]]], [[[2.0, 1.0, 6.0], [8.0, 4.0, 7.0], [8.0, 6.0, 6.0]], [[5.0, 6.0, 4.0], [7.0, 6.0, 5.0], [8.0, 4.0, 1.0]], 
[[8.0, 5.0, 2.0], [5.0, 1.0, 8.0], [2.0, 4.0, 6.0]]], [[[6.0, 8.0, 7.0], [5.0, 3.0, 6.0], [1.0, 2.0, 4.0]], [[2.0, 8.0, 2.0], 
[2.0, 2.0, 8.0], [5.0, 5.0, 5.0]], [[4.0, 2.0, 7.0], [2.0, 5.0, 8.0], [3.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CRLN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 2.0], [7.0, 6.0, 4.0], [8.0, 2.0, 4.0]], [[5.0, 2.0, 8.0], [6.0, 1.0, 2.0], [3.0, 3.0, 
8.0]], [[5.0, 4.0, 6.0], [4.0, 5.0, 6.0], [5.0, 2.0, 7.0]]], [[[5.0, 8.0, 4.0], [1.0, 7.0, 7.0], [5.0, 2.0, 8.0]], [[3.0, 1.0, 
6.0], [4.0, 4.0, 6.0], [6.0, 1.0, 5.0]], [[3.0, 5.0, 7.0], [6.0, 2.0, 5.0], [8.0, 3.0, 7.0]]], [[[3.0, 2.0, 7.0], [5.0, 4.0, 
7.0], [5.0, 5.0, 4.0]], [[8.0, 2.0, 3.0], [2.0, 7.0, 1.0], [6.0, 3.0, 5.0]], [[1.0, 6.0, 7.0], [8.0, 6.0, 7.0], [8.0, 1.0, 
1.0]]], [[[4.0, 3.0, 2.0], [7.0, 6.0, 3.0], [2.0, 1.0, 5.0]], [[4.0, 4.0, 1.0], [8.0, 1.0, 7.0], [6.0, 8.0, 4.0]], [[3.0, 1.0, 
1.0], [1.0, 7.0, 5.0], [3.0, 4.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 2.0, 6.0]]], [[[7.0, 4.0, 2.0]]], [[[2.0, 6.0, 6.0]]], [[[6.0, 1.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 1.0]]], [[[8.0, 6.0, 5.0]]], [[[2.0, 6.0, 6.0]]], [[[3.0, 3.0, 3.0]]]]))
      arg[:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 2.0], [7.0, 6.0, 4.0], [8.0, 2.0, 4.0]], [[5.0, 2.0, 8.0], [6.0, 1.0, 2.0], [5.0, 
2.0, 6.0]], [[5.0, 4.0, 6.0], [4.0, 5.0, 6.0], [5.0, 2.0, 7.0]]], [[[5.0, 8.0, 4.0], [1.0, 7.0, 7.0], [5.0, 2.0, 8.0]], [[3.0, 
1.0, 6.0], [4.0, 4.0, 6.0], [7.0, 4.0, 2.0]], [[3.0, 5.0, 7.0], [6.0, 2.0, 5.0], [8.0, 3.0, 7.0]]], [[[3.0, 2.0, 7.0], [5.0, 
4.0, 7.0], [5.0, 5.0, 4.0]], [[8.0, 2.0, 3.0], [2.0, 7.0, 1.0], [2.0, 6.0, 6.0]], [[1.0, 6.0, 7.0], [8.0, 6.0, 7.0], [8.0, 1.0, 
1.0]]], [[[4.0, 3.0, 2.0], [7.0, 6.0, 3.0], [2.0, 1.0, 5.0]], [[4.0, 4.0, 1.0], [8.0, 1.0, 7.0], [6.0, 1.0, 1.0]], [[3.0, 1.0, 
1.0], [1.0, 7.0, 5.0], [3.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 2.0], [7.0, 6.0, 4.0], [8.0, 2.0, 4.0]], 
[[5.0, 2.0, 8.0], [6.0, 1.0, 2.0], [5.0, 5.0, 1.0]], [[5.0, 4.0, 6.0], [4.0, 5.0, 6.0], [5.0, 2.0, 7.0]]], [[[5.0, 8.0, 4.0], 
[1.0, 7.0, 7.0], [5.0, 2.0, 8.0]], [[3.0, 1.0, 6.0], [4.0, 4.0, 6.0], [8.0, 6.0, 5.0]], [[3.0, 5.0, 7.0], [6.0, 2.0, 5.0], 
[8.0, 3.0, 7.0]]], [[[3.0, 2.0, 7.0], [5.0, 4.0, 7.0], [5.0, 5.0, 4.0]], [[8.0, 2.0, 3.0], [2.0, 7.0, 1.0], [2.0, 6.0, 6.0]], 
[[1.0, 6.0, 7.0], [8.0, 6.0, 7.0], [8.0, 1.0, 1.0]]], [[[4.0, 3.0, 2.0], [7.0, 6.0, 3.0], [2.0, 1.0, 5.0]], [[4.0, 4.0, 1.0], 
[8.0, 1.0, 7.0], [3.0, 3.0, 3.0]], [[3.0, 1.0, 1.0], [1.0, 7.0, 5.0], [3.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CRCN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 5.0], [8.0, 7.0, 4.0], [2.0, 5.0, 2.0]], [[1.0, 4.0, 8.0], [7.0, 6.0, 6.0], [1.0, 7.0, 
3.0]], [[6.0, 2.0, 8.0], [8.0, 1.0, 8.0], [2.0, 2.0, 5.0]]], [[[4.0, 3.0, 2.0], [6.0, 7.0, 4.0], [7.0, 8.0, 5.0]], [[5.0, 7.0, 
1.0], [1.0, 3.0, 2.0], [8.0, 3.0, 4.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 3.0], [1.0, 7.0, 3.0]]], [[[2.0, 1.0, 7.0], [5.0, 4.0, 
8.0], [6.0, 4.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 6.0, 3.0], [1.0, 4.0, 7.0]], [[2.0, 2.0, 3.0], [3.0, 8.0, 5.0], [5.0, 7.0, 
7.0]]], [[[8.0, 6.0, 7.0], [3.0, 4.0, 8.0], [7.0, 7.0, 5.0]], [[8.0, 3.0, 8.0], [8.0, 2.0, 4.0], [4.0, 2.0, 1.0]], [[5.0, 2.0, 
3.0], [1.0, 3.0, 6.0], [6.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 6.0, 6.0], [4.0, 5.0, 6.0], [6.0, 7.0, 2.0]]], [[[6.0, 1.0, 6.0], [7.0, 4.0, 3.0], [6.0, 
6.0, 6.0]]], [[[1.0, 6.0, 5.0], [2.0, 3.0, 4.0], [8.0, 6.0, 1.0]]], [[[6.0, 6.0, 3.0], [1.0, 3.0, 7.0], [3.0, 7.0, 
4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 5.0], [6.0, 2.0, 6.0], [3.0, 1.0, 6.0]]], [[[2.0, 1.0, 1.0], [3.0, 6.0, 
7.0], [1.0, 1.0, 7.0]]], [[[7.0, 4.0, 8.0], [2.0, 4.0, 3.0], [2.0, 5.0, 2.0]]], [[[1.0, 7.0, 6.0], [6.0, 3.0, 8.0], [6.0, 2.0, 
4.0]]]]))
      arg[:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 5.0], [8.0, 7.0, 4.0], [2.0, 5.0, 2.0]], [[7.0, 6.0, 6.0], [4.0, 5.0, 6.0], [6.0, 
7.0, 2.0]], [[6.0, 2.0, 8.0], [8.0, 1.0, 8.0], [2.0, 2.0, 5.0]]], [[[4.0, 3.0, 2.0], [6.0, 7.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 
1.0, 6.0], [7.0, 4.0, 3.0], [6.0, 6.0, 6.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 3.0], [1.0, 7.0, 3.0]]], [[[2.0, 1.0, 7.0], [5.0, 
4.0, 8.0], [6.0, 4.0, 4.0]], [[1.0, 6.0, 5.0], [2.0, 3.0, 4.0], [8.0, 6.0, 1.0]], [[2.0, 2.0, 3.0], [3.0, 8.0, 5.0], [5.0, 7.0, 
7.0]]], [[[8.0, 6.0, 7.0], [3.0, 4.0, 8.0], [7.0, 7.0, 5.0]], [[6.0, 6.0, 3.0], [1.0, 3.0, 7.0], [3.0, 7.0, 4.0]], [[5.0, 2.0, 
3.0], [1.0, 3.0, 6.0], [6.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 5.0], [8.0, 7.0, 4.0], [2.0, 5.0, 2.0]], 
[[8.0, 2.0, 5.0], [6.0, 2.0, 6.0], [3.0, 1.0, 6.0]], [[6.0, 2.0, 8.0], [8.0, 1.0, 8.0], [2.0, 2.0, 5.0]]], [[[4.0, 3.0, 2.0], 
[6.0, 7.0, 4.0], [7.0, 8.0, 5.0]], [[2.0, 1.0, 1.0], [3.0, 6.0, 7.0], [1.0, 1.0, 7.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 3.0], 
[1.0, 7.0, 3.0]]], [[[2.0, 1.0, 7.0], [5.0, 4.0, 8.0], [6.0, 4.0, 4.0]], [[7.0, 4.0, 8.0], [2.0, 4.0, 3.0], [2.0, 5.0, 2.0]], 
[[2.0, 2.0, 3.0], [3.0, 8.0, 5.0], [5.0, 7.0, 7.0]]], [[[8.0, 6.0, 7.0], [3.0, 4.0, 8.0], [7.0, 7.0, 5.0]], [[1.0, 7.0, 6.0], 
[6.0, 3.0, 8.0], [6.0, 2.0, 4.0]], [[5.0, 2.0, 3.0], [1.0, 3.0, 6.0], [6.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_ULLN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 8.0], [5.0, 6.0, 1.0], [7.0, 2.0, 5.0]], [[4.0, 6.0, 8.0], [2.0, 7.0, 4.0], [6.0, 3.0, 
5.0]], [[5.0, 8.0, 1.0], [6.0, 2.0, 8.0], [4.0, 5.0, 5.0]]], [[[8.0, 6.0, 2.0], [8.0, 1.0, 4.0], [5.0, 8.0, 7.0]], [[5.0, 5.0, 
3.0], [1.0, 8.0, 7.0], [1.0, 2.0, 8.0]], [[3.0, 1.0, 3.0], [4.0, 4.0, 8.0], [3.0, 7.0, 7.0]]], [[[1.0, 6.0, 2.0], [4.0, 8.0, 
2.0], [2.0, 5.0, 5.0]], [[8.0, 5.0, 5.0], [1.0, 8.0, 7.0], [2.0, 7.0, 4.0]], [[3.0, 6.0, 6.0], [8.0, 4.0, 7.0], [4.0, 1.0, 
7.0]]], [[[3.0, 2.0, 4.0], [3.0, 6.0, 1.0], [3.0, 4.0, 1.0]], [[5.0, 7.0, 6.0], [8.0, 2.0, 3.0], [4.0, 6.0, 6.0]], [[7.0, 5.0, 
6.0], [3.0, 1.0, 3.0], [1.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 1.0, 7.0]]], [[[3.0, 5.0, 2.0]]], [[[8.0, 2.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 5.0]]], [[[5.0, 6.0, 1.0]]], [[[5.0, 5.0, 6.0]]]]))
      arg[:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 8.0], [5.0, 6.0, 1.0], [7.0, 2.0, 5.0]], [[4.0, 6.0, 8.0], [2.0, 7.0, 4.0], [6.0, 
3.0, 5.0]], [[5.0, 8.0, 1.0], [6.0, 2.0, 8.0], [1.0, 1.0, 7.0]]], [[[8.0, 6.0, 2.0], [8.0, 1.0, 4.0], [5.0, 8.0, 7.0]], [[5.0, 
5.0, 3.0], [1.0, 8.0, 7.0], [1.0, 2.0, 8.0]], [[3.0, 1.0, 3.0], [4.0, 4.0, 8.0], [3.0, 5.0, 2.0]]], [[[1.0, 6.0, 2.0], [4.0, 
8.0, 2.0], [2.0, 5.0, 5.0]], [[8.0, 5.0, 5.0], [1.0, 8.0, 7.0], [2.0, 7.0, 4.0]], [[3.0, 6.0, 6.0], [8.0, 4.0, 7.0], [8.0, 2.0, 
6.0]]], [[[3.0, 2.0, 4.0], [3.0, 6.0, 1.0], [3.0, 4.0, 1.0]], [[5.0, 7.0, 6.0], [8.0, 2.0, 3.0], [4.0, 6.0, 6.0]], [[7.0, 5.0, 
6.0], [3.0, 1.0, 3.0], [1.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 8.0], [5.0, 6.0, 1.0], [7.0, 2.0, 5.0]], 
[[4.0, 6.0, 8.0], [2.0, 7.0, 4.0], [6.0, 3.0, 5.0]], [[5.0, 8.0, 1.0], [6.0, 2.0, 8.0], [5.0, 7.0, 5.0]]], [[[8.0, 6.0, 2.0], 
[8.0, 1.0, 4.0], [5.0, 8.0, 7.0]], [[5.0, 5.0, 3.0], [1.0, 8.0, 7.0], [1.0, 2.0, 8.0]], [[3.0, 1.0, 3.0], [4.0, 4.0, 8.0], 
[5.0, 6.0, 1.0]]], [[[1.0, 6.0, 2.0], [4.0, 8.0, 2.0], [2.0, 5.0, 5.0]], [[8.0, 5.0, 5.0], [1.0, 8.0, 7.0], [2.0, 7.0, 4.0]], 
[[3.0, 6.0, 6.0], [8.0, 4.0, 7.0], [5.0, 5.0, 6.0]]], [[[3.0, 2.0, 4.0], [3.0, 6.0, 1.0], [3.0, 4.0, 1.0]], [[5.0, 7.0, 6.0], 
[8.0, 2.0, 3.0], [4.0, 6.0, 6.0]], [[7.0, 5.0, 6.0], [3.0, 1.0, 3.0], [1.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_taggedData_CLCN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 6.0], [1.0, 4.0, 2.0], [4.0, 6.0, 3.0]], [[1.0, 5.0, 8.0], [7.0, 7.0, 5.0], [4.0, 4.0, 
3.0]], [[4.0, 1.0, 7.0], [3.0, 8.0, 7.0], [5.0, 6.0, 2.0]]], [[[2.0, 3.0, 7.0], [1.0, 3.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 4.0, 
4.0], [3.0, 3.0, 2.0], [5.0, 1.0, 8.0]], [[4.0, 6.0, 4.0], [2.0, 4.0, 6.0], [3.0, 7.0, 7.0]]], [[[7.0, 1.0, 6.0], [3.0, 6.0, 
2.0], [7.0, 7.0, 2.0]], [[4.0, 5.0, 1.0], [7.0, 2.0, 8.0], [4.0, 1.0, 6.0]], [[3.0, 7.0, 3.0], [3.0, 7.0, 2.0], [1.0, 8.0, 
2.0]]], [[[3.0, 7.0, 5.0], [8.0, 1.0, 4.0], [2.0, 7.0, 6.0]], [[3.0, 3.0, 6.0], [4.0, 7.0, 5.0], [5.0, 8.0, 4.0]], [[6.0, 6.0, 
1.0], [4.0, 3.0, 7.0], [3.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 7.0, 2.0], [4.0, 4.0, 1.0], [2.0, 3.0, 2.0]]], [[[6.0, 4.0, 5.0], [4.0, 3.0, 1.0], [1.0, 
7.0, 5.0]]], [[[7.0, 4.0, 7.0], [8.0, 4.0, 3.0], [1.0, 7.0, 2.0]]], [[[1.0, 6.0, 1.0], [3.0, 2.0, 2.0], [3.0, 4.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 7.0], [5.0, 4.0, 8.0], [3.0, 8.0, 3.0]]], [[[8.0, 6.0, 5.0], [6.0, 1.0, 
1.0], [4.0, 6.0, 3.0]]], [[[8.0, 2.0, 2.0], [3.0, 3.0, 1.0], [7.0, 3.0, 6.0]]], [[[3.0, 6.0, 7.0], [1.0, 7.0, 8.0], [7.0, 6.0, 
4.0]]]]))
      arg[:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 6.0], [1.0, 4.0, 2.0], [4.0, 6.0, 3.0]], [[1.0, 5.0, 8.0], [7.0, 7.0, 5.0], [4.0, 
4.0, 3.0]], [[3.0, 7.0, 2.0], [4.0, 4.0, 1.0], [2.0, 3.0, 2.0]]], [[[2.0, 3.0, 7.0], [1.0, 3.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 
4.0, 4.0], [3.0, 3.0, 2.0], [5.0, 1.0, 8.0]], [[6.0, 4.0, 5.0], [4.0, 3.0, 1.0], [1.0, 7.0, 5.0]]], [[[7.0, 1.0, 6.0], [3.0, 
6.0, 2.0], [7.0, 7.0, 2.0]], [[4.0, 5.0, 1.0], [7.0, 2.0, 8.0], [4.0, 1.0, 6.0]], [[7.0, 4.0, 7.0], [8.0, 4.0, 3.0], [1.0, 7.0, 
2.0]]], [[[3.0, 7.0, 5.0], [8.0, 1.0, 4.0], [2.0, 7.0, 6.0]], [[3.0, 3.0, 6.0], [4.0, 7.0, 5.0], [5.0, 8.0, 4.0]], [[1.0, 6.0, 
1.0], [3.0, 2.0, 2.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 6.0], [1.0, 4.0, 2.0], [4.0, 6.0, 3.0]], 
[[1.0, 5.0, 8.0], [7.0, 7.0, 5.0], [4.0, 4.0, 3.0]], [[7.0, 5.0, 7.0], [5.0, 4.0, 8.0], [3.0, 8.0, 3.0]]], [[[2.0, 3.0, 7.0], 
[1.0, 3.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 4.0, 4.0], [3.0, 3.0, 2.0], [5.0, 1.0, 8.0]], [[8.0, 6.0, 5.0], [6.0, 1.0, 1.0], 
[4.0, 6.0, 3.0]]], [[[7.0, 1.0, 6.0], [3.0, 6.0, 2.0], [7.0, 7.0, 2.0]], [[4.0, 5.0, 1.0], [7.0, 2.0, 8.0], [4.0, 1.0, 6.0]], 
[[8.0, 2.0, 2.0], [3.0, 3.0, 1.0], [7.0, 3.0, 6.0]]], [[[3.0, 7.0, 5.0], [8.0, 1.0, 4.0], [2.0, 7.0, 6.0]], [[3.0, 3.0, 6.0], 
[4.0, 7.0, 5.0], [5.0, 8.0, 4.0]], [[3.0, 6.0, 7.0], [1.0, 7.0, 8.0], [7.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_array_L(self):
      arg=Data(numpy.array([7.0, 4.0, 8.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([3.0, 6.0, 7.0]))
      val=numpy.array([5.0])
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([7.0, 4.0, 5.0])+(1.-msk_ref)*numpy.array([3.0, 6.0, 5.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_constData_L(self):
      arg=Data(numpy.array([2.0, 4.0, 3.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([3.0, 2.0, 7.0]))
      val=Data(numpy.array([3.0]),self.functionspace)
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([2.0, 4.0, 3.0])+(1.-msk_ref)*numpy.array([3.0, 2.0, 3.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_taggedData_L(self):
      arg=Data(numpy.array([6.0, 7.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([6.0, 3.0, 1.0]))
      val=Data(numpy.array([1.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0]))
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([6.0, 7.0, 1.0])+(1.-msk_ref)*numpy.array([6.0, 3.0, 3.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_expandedData_L(self):
      arg=Data(numpy.array([3.0, 6.0, 8.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([7.0, 8.0, 6.0]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([1.0])+(1.-msk_val)*numpy.array([7.0])
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 6.0, 1.0])+(1.-msk_ref)*numpy.array([7.0, 8.0, 7.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_array_U(self):
      arg=Data(numpy.array([6.0, 4.0, 5.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([7.0, 5.0, 4.0]))
      val=numpy.array([8.0, 4.0])
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([8.0, 4.0, 5.0])+(1.-msk_ref)*numpy.array([8.0, 4.0, 4.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_constData_U(self):
      arg=Data(numpy.array([1.0, 6.0, 6.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([2.0, 5.0, 3.0]))
      val=Data(numpy.array([8.0, 4.0]),self.functionspace)
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([8.0, 4.0, 6.0])+(1.-msk_ref)*numpy.array([8.0, 4.0, 3.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_taggedData_U(self):
      arg=Data(numpy.array([5.0, 8.0, 8.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([3.0, 4.0, 3.0]))
      val=Data(numpy.array([5.0, 4.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([2.0, 7.0]))
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([5.0, 4.0, 8.0])+(1.-msk_ref)*numpy.array([2.0, 7.0, 3.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_expandedData_U(self):
      arg=Data(numpy.array([5.0, 2.0, 3.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([2.0, 7.0, 5.0]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([5.0, 7.0])+(1.-msk_val)*numpy.array([4.0, 2.0])
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([5.0, 7.0, 3.0])+(1.-msk_ref)*numpy.array([4.0, 2.0, 5.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_array_C(self):
      arg=Data(numpy.array([8.0, 2.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([5.0, 4.0, 4.0]))
      val=numpy.array([7.0, 7.0, 1.0])
      arg[:]=val
      ref=Data(numpy.array([7.0, 7.0, 1.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_constData_C(self):
      arg=Data(numpy.array([7.0, 5.0, 4.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([4.0, 1.0, 5.0]))
      val=Data(numpy.array([8.0, 6.0, 6.0]),self.functionspace)
      arg[:]=val
      ref=Data(numpy.array([8.0, 6.0, 6.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_taggedData_C(self):
      arg=Data(numpy.array([3.0, 3.0, 1.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([8.0, 2.0, 4.0]))
      val=Data(numpy.array([5.0, 6.0, 6.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0, 1.0, 7.0]))
      arg[:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([5.0, 6.0, 6.0])+(1.-msk_ref)*numpy.array([8.0, 1.0, 7.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_expandedData_C(self):
      arg=Data(numpy.array([4.0, 8.0, 3.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([7.0, 7.0, 4.0]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([6.0, 6.0, 7.0])+(1.-msk_val)*numpy.array([7.0, 7.0, 5.0])
      arg[:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([6.0, 6.0, 7.0])+(1.-msk_ref)*numpy.array([7.0, 7.0, 5.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_array_R(self):
      arg=Data(numpy.array([7.0, 4.0, 6.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([3.0, 8.0, 4.0]))
      val=numpy.array([3.0])
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([7.0, 3.0, 6.0])+(1.-msk_ref)*numpy.array([3.0, 3.0, 4.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_constData_R(self):
      arg=Data(numpy.array([2.0, 1.0, 5.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([3.0, 7.0, 3.0]))
      val=Data(numpy.array([5.0]),self.functionspace)
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([2.0, 5.0, 5.0])+(1.-msk_ref)*numpy.array([3.0, 5.0, 3.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_taggedData_R(self):
      arg=Data(numpy.array([5.0, 6.0, 7.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([3.0, 5.0, 6.0]))
      val=Data(numpy.array([7.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([4.0]))
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([5.0, 7.0, 7.0])+(1.-msk_ref)*numpy.array([3.0, 4.0, 6.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_expandedData_R(self):
      arg=Data(numpy.array([3.0, 5.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([2.0, 4.0, 1.0]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([1.0])+(1.-msk_val)*numpy.array([2.0])
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 1.0, 2.0])+(1.-msk_ref)*numpy.array([2.0, 2.0, 1.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_array_P(self):
      arg=Data(numpy.array([5.0, 2.0, 4.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([2.0, 3.0, 7.0]))
      val=numpy.array(1.0)
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([5.0, 1.0, 4.0])+(1.-msk_ref)*numpy.array([2.0, 1.0, 7.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_constData_P(self):
      arg=Data(numpy.array([3.0, 2.0, 7.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([2.0, 8.0, 2.0]))
      val=Data(1.0,self.functionspace)
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 1.0, 7.0])+(1.-msk_ref)*numpy.array([2.0, 1.0, 2.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_taggedData_P(self):
      arg=Data(numpy.array([4.0, 2.0, 5.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([4.0, 7.0, 4.0]))
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([4.0, 1.0, 5.0])+(1.-msk_ref)*numpy.array([4.0, 1.0, 4.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank1_expandedData_P(self):
      arg=Data(numpy.array([3.0, 7.0, 2.0]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([2.0, 2.0, 4.0]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=(1.-msk_val)*(1.0)+msk_val*(1.0)
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 1.0, 2.0])+(1.-msk_ref)*numpy.array([2.0, 1.0, 4.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_UR(self):
      arg=Data(numpy.array([[2.0, 5.0, 4.0, 4.0], [6.0, 4.0, 3.0, 1.0], [1.0, 2.0, 6.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 1.0, 8.0, 8.0], [4.0, 3.0, 4.0, 1.0], [5.0, 1.0, 1.0, 1.0]]))
      val=numpy.array([[4.0, 2.0], [4.0, 6.0]])
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 4.0, 2.0, 4.0], [6.0, 4.0, 6.0, 1.0], [1.0, 2.0, 6.0, 
4.0]])+(1.-msk_ref)*numpy.array([[2.0, 4.0, 2.0, 8.0], [4.0, 4.0, 6.0, 1.0], [5.0, 1.0, 1.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_UR(self):
      arg=Data(numpy.array([[4.0, 1.0, 4.0, 6.0], [2.0, 3.0, 4.0, 2.0], [2.0, 8.0, 4.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 3.0, 2.0, 7.0], [3.0, 2.0, 7.0, 8.0], [8.0, 3.0, 2.0, 3.0]]))
      val=Data(numpy.array([[5.0, 6.0], [3.0, 6.0]]),self.functionspace)
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 5.0, 6.0, 6.0], [2.0, 3.0, 6.0, 2.0], [2.0, 8.0, 4.0, 
7.0]])+(1.-msk_ref)*numpy.array([[7.0, 5.0, 6.0, 7.0], [3.0, 3.0, 6.0, 8.0], [8.0, 3.0, 2.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_UR(self):
      arg=Data(numpy.array([[4.0, 3.0, 4.0, 1.0], [5.0, 6.0, 3.0, 5.0], [6.0, 5.0, 5.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 1.0, 4.0, 2.0], [7.0, 5.0, 8.0, 7.0], [6.0, 8.0, 4.0, 7.0]]))
      val=Data(numpy.array([[1.0, 5.0], [6.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 4.0], [7.0, 1.0]]))
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 1.0, 5.0, 1.0], [5.0, 6.0, 3.0, 5.0], [6.0, 5.0, 5.0, 
8.0]])+(1.-msk_ref)*numpy.array([[3.0, 8.0, 4.0, 2.0], [7.0, 7.0, 1.0, 7.0], [6.0, 8.0, 4.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_UR(self):
      arg=Data(numpy.array([[3.0, 6.0, 7.0, 3.0], [2.0, 2.0, 1.0, 8.0], [1.0, 2.0, 7.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[8.0, 8.0, 1.0, 4.0], [5.0, 4.0, 3.0, 6.0], [8.0, 2.0, 4.0, 7.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[1.0, 7.0], [6.0, 4.0]])+(1.-msk_val)*numpy.array([[6.0, 8.0], [8.0, 8.0]])
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 1.0, 7.0, 3.0], [2.0, 6.0, 4.0, 8.0], [1.0, 2.0, 7.0, 
4.0]])+(1.-msk_ref)*numpy.array([[8.0, 6.0, 8.0, 4.0], [5.0, 8.0, 8.0, 6.0], [8.0, 2.0, 4.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_PU(self):
      arg=Data(numpy.array([[7.0, 3.0, 2.0, 6.0], [4.0, 1.0, 3.0, 7.0], [4.0, 2.0, 1.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 6.0, 8.0, 8.0], [4.0, 4.0, 8.0, 1.0], [6.0, 8.0, 3.0, 3.0]]))
      val=numpy.array([4.0, 4.0, 7.0])
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 3.0, 2.0, 6.0], [4.0, 4.0, 7.0, 7.0], [4.0, 2.0, 1.0, 
2.0]])+(1.-msk_ref)*numpy.array([[2.0, 6.0, 8.0, 8.0], [4.0, 4.0, 7.0, 1.0], [6.0, 8.0, 3.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_PU(self):
      arg=Data(numpy.array([[1.0, 8.0, 2.0, 3.0], [1.0, 4.0, 1.0, 4.0], [1.0, 8.0, 6.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 7.0, 2.0, 4.0], [7.0, 1.0, 7.0, 1.0], [8.0, 7.0, 5.0, 6.0]]))
      val=Data(numpy.array([1.0, 5.0, 7.0]),self.functionspace)
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 8.0, 2.0, 3.0], [1.0, 5.0, 7.0, 4.0], [1.0, 8.0, 6.0, 
5.0]])+(1.-msk_ref)*numpy.array([[2.0, 7.0, 2.0, 4.0], [1.0, 5.0, 7.0, 1.0], [8.0, 7.0, 5.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_PU(self):
      arg=Data(numpy.array([[1.0, 1.0, 4.0, 2.0], [1.0, 8.0, 8.0, 4.0], [1.0, 1.0, 2.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[4.0, 1.0, 6.0, 2.0], [1.0, 8.0, 7.0, 5.0], [3.0, 7.0, 8.0, 1.0]]))
      val=Data(numpy.array([4.0, 2.0, 8.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([4.0, 8.0, 8.0]))
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 1.0, 4.0, 2.0], [4.0, 2.0, 8.0, 4.0], [1.0, 1.0, 2.0, 
2.0]])+(1.-msk_ref)*numpy.array([[4.0, 1.0, 6.0, 2.0], [4.0, 8.0, 8.0, 5.0], [3.0, 7.0, 8.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_PU(self):
      arg=Data(numpy.array([[3.0, 8.0, 2.0, 4.0], [1.0, 7.0, 1.0, 6.0], [1.0, 2.0, 5.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 2.0, 6.0, 6.0], [6.0, 2.0, 2.0, 8.0], [3.0, 6.0, 8.0, 2.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([8.0, 2.0, 7.0])+(1.-msk_val)*numpy.array([7.0, 4.0, 2.0])
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 8.0, 2.0, 4.0], [8.0, 2.0, 7.0, 6.0], [1.0, 2.0, 5.0, 
3.0]])+(1.-msk_ref)*numpy.array([[2.0, 2.0, 6.0, 6.0], [7.0, 4.0, 2.0, 8.0], [3.0, 6.0, 8.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_UU(self):
      arg=Data(numpy.array([[3.0, 3.0, 8.0, 4.0], [1.0, 8.0, 1.0, 8.0], [3.0, 5.0, 2.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 4.0, 7.0, 7.0], [5.0, 2.0, 6.0, 3.0], [4.0, 5.0, 3.0, 2.0]]))
      val=numpy.array([[6.0, 7.0, 7.0], [3.0, 6.0, 6.0]])
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 7.0, 7.0, 4.0], [3.0, 6.0, 6.0, 8.0], [3.0, 5.0, 2.0, 
2.0]])+(1.-msk_ref)*numpy.array([[6.0, 7.0, 7.0, 7.0], [3.0, 6.0, 6.0, 3.0], [4.0, 5.0, 3.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_UU(self):
      arg=Data(numpy.array([[7.0, 5.0, 4.0, 8.0], [5.0, 8.0, 3.0, 5.0], [8.0, 3.0, 1.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 3.0, 5.0, 3.0], [7.0, 1.0, 2.0, 6.0], [1.0, 5.0, 2.0, 6.0]]))
      val=Data(numpy.array([[8.0, 6.0, 6.0], [7.0, 2.0, 3.0]]),self.functionspace)
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 6.0, 6.0, 8.0], [7.0, 2.0, 3.0, 5.0], [8.0, 3.0, 1.0, 
3.0]])+(1.-msk_ref)*numpy.array([[8.0, 6.0, 6.0, 3.0], [7.0, 2.0, 3.0, 6.0], [1.0, 5.0, 2.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_UU(self):
      arg=Data(numpy.array([[2.0, 3.0, 7.0, 2.0], [1.0, 2.0, 1.0, 6.0], [6.0, 4.0, 5.0, 6.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[8.0, 2.0, 3.0, 2.0], [3.0, 6.0, 7.0, 1.0], [4.0, 3.0, 2.0, 2.0]]))
      val=Data(numpy.array([[4.0, 4.0, 6.0], [4.0, 3.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 1.0, 3.0], [2.0, 4.0, 6.0]]))
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 4.0, 6.0, 2.0], [4.0, 3.0, 3.0, 6.0], [6.0, 4.0, 5.0, 
6.0]])+(1.-msk_ref)*numpy.array([[2.0, 1.0, 3.0, 2.0], [2.0, 4.0, 6.0, 1.0], [4.0, 3.0, 2.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_UU(self):
      arg=Data(numpy.array([[3.0, 5.0, 2.0, 8.0], [1.0, 8.0, 6.0, 8.0], [7.0, 4.0, 4.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 3.0, 4.0, 1.0], [2.0, 8.0, 2.0, 4.0], [4.0, 1.0, 2.0, 8.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[8.0, 2.0, 1.0], [5.0, 1.0, 5.0]])+(1.-msk_val)*numpy.array([[8.0, 5.0, 6.0], [2.0, 5.0, 6.0]])
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 2.0, 1.0, 8.0], [5.0, 1.0, 5.0, 8.0], [7.0, 4.0, 4.0, 
4.0]])+(1.-msk_ref)*numpy.array([[8.0, 5.0, 6.0, 1.0], [2.0, 5.0, 6.0, 4.0], [4.0, 1.0, 2.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_RL(self):
      arg=Data(numpy.array([[5.0, 7.0, 2.0, 5.0], [7.0, 4.0, 6.0, 8.0], [3.0, 7.0, 5.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[1.0, 7.0, 3.0, 7.0], [5.0, 1.0, 1.0, 5.0], [6.0, 6.0, 8.0, 3.0]]))
      val=numpy.array([[5.0, 2.0]])
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 7.0, 2.0, 5.0], [7.0, 4.0, 5.0, 2.0], [3.0, 7.0, 5.0, 
5.0]])+(1.-msk_ref)*numpy.array([[1.0, 7.0, 3.0, 7.0], [5.0, 1.0, 5.0, 2.0], [6.0, 6.0, 8.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_RL(self):
      arg=Data(numpy.array([[5.0, 2.0, 6.0, 1.0], [3.0, 4.0, 3.0, 7.0], [3.0, 1.0, 6.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[8.0, 5.0, 8.0, 5.0], [4.0, 1.0, 2.0, 2.0], [4.0, 3.0, 5.0, 4.0]]))
      val=Data(numpy.array([[1.0, 2.0]]),self.functionspace)
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 2.0, 6.0, 1.0], [3.0, 4.0, 1.0, 2.0], [3.0, 1.0, 6.0, 
4.0]])+(1.-msk_ref)*numpy.array([[8.0, 5.0, 8.0, 5.0], [4.0, 1.0, 1.0, 2.0], [4.0, 3.0, 5.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_RL(self):
      arg=Data(numpy.array([[8.0, 8.0, 3.0, 7.0], [4.0, 3.0, 8.0, 4.0], [1.0, 7.0, 5.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 7.0, 1.0, 7.0], [5.0, 1.0, 4.0, 3.0], [7.0, 5.0, 8.0, 6.0]]))
      val=Data(numpy.array([[2.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 7.0]]))
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 8.0, 3.0, 7.0], [4.0, 3.0, 2.0, 5.0], [1.0, 7.0, 5.0, 
2.0]])+(1.-msk_ref)*numpy.array([[3.0, 7.0, 1.0, 7.0], [5.0, 1.0, 4.0, 7.0], [7.0, 5.0, 8.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_RL(self):
      arg=Data(numpy.array([[7.0, 1.0, 4.0, 1.0], [7.0, 8.0, 7.0, 3.0], [5.0, 6.0, 8.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[1.0, 6.0, 6.0, 5.0], [5.0, 5.0, 5.0, 1.0], [1.0, 4.0, 1.0, 8.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 6.0]])+(1.-msk_val)*numpy.array([[5.0, 1.0]])
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 1.0, 4.0, 1.0], [7.0, 8.0, 6.0, 6.0], [5.0, 6.0, 8.0, 
3.0]])+(1.-msk_ref)*numpy.array([[1.0, 6.0, 6.0, 5.0], [5.0, 5.0, 5.0, 1.0], [1.0, 4.0, 1.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_PL(self):
      arg=Data(numpy.array([[7.0, 5.0, 5.0, 7.0], [7.0, 5.0, 3.0, 3.0], [7.0, 4.0, 4.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 6.0, 8.0, 8.0], [2.0, 4.0, 6.0, 7.0], [8.0, 5.0, 8.0, 7.0]]))
      val=numpy.array([2.0, 8.0])
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 5.0, 5.0, 7.0], [7.0, 5.0, 2.0, 8.0], [7.0, 4.0, 4.0, 
5.0]])+(1.-msk_ref)*numpy.array([[6.0, 6.0, 8.0, 8.0], [2.0, 4.0, 2.0, 8.0], [8.0, 5.0, 8.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_PL(self):
      arg=Data(numpy.array([[8.0, 2.0, 2.0, 8.0], [8.0, 7.0, 1.0, 3.0], [7.0, 1.0, 6.0, 1.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 1.0, 3.0, 3.0], [1.0, 1.0, 7.0, 2.0], [5.0, 4.0, 4.0, 4.0]]))
      val=Data(numpy.array([5.0, 3.0]),self.functionspace)
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 2.0, 2.0, 8.0], [8.0, 7.0, 5.0, 3.0], [7.0, 1.0, 6.0, 
1.0]])+(1.-msk_ref)*numpy.array([[7.0, 1.0, 3.0, 3.0], [1.0, 1.0, 5.0, 3.0], [5.0, 4.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_PL(self):
      arg=Data(numpy.array([[8.0, 5.0, 4.0, 6.0], [4.0, 3.0, 2.0, 5.0], [3.0, 3.0, 7.0, 1.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 6.0, 1.0, 1.0], [7.0, 3.0, 1.0, 3.0], [2.0, 4.0, 4.0, 7.0]]))
      val=Data(numpy.array([1.0, 3.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0, 5.0]))
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 5.0, 4.0, 6.0], [4.0, 3.0, 1.0, 3.0], [3.0, 3.0, 7.0, 
1.0]])+(1.-msk_ref)*numpy.array([[7.0, 6.0, 1.0, 1.0], [7.0, 3.0, 8.0, 5.0], [2.0, 4.0, 4.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_PL(self):
      arg=Data(numpy.array([[2.0, 6.0, 6.0, 8.0], [4.0, 8.0, 1.0, 7.0], [3.0, 8.0, 7.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 7.0, 4.0, 3.0], [5.0, 5.0, 3.0, 4.0], [1.0, 2.0, 6.0, 8.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([8.0, 6.0])+(1.-msk_val)*numpy.array([5.0, 1.0])
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 6.0, 6.0, 8.0], [4.0, 8.0, 8.0, 6.0], [3.0, 8.0, 7.0, 
3.0]])+(1.-msk_ref)*numpy.array([[7.0, 7.0, 4.0, 3.0], [5.0, 5.0, 5.0, 1.0], [1.0, 2.0, 6.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_CP(self):
      arg=Data(numpy.array([[8.0, 8.0, 8.0, 6.0], [6.0, 4.0, 8.0, 8.0], [2.0, 1.0, 7.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 1.0, 2.0, 5.0], [2.0, 2.0, 5.0, 8.0], [7.0, 5.0, 3.0, 2.0]]))
      val=numpy.array([4.0, 3.0, 8.0])
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 8.0, 4.0, 6.0], [6.0, 4.0, 3.0, 8.0], [2.0, 1.0, 8.0, 
8.0]])+(1.-msk_ref)*numpy.array([[3.0, 1.0, 4.0, 5.0], [2.0, 2.0, 3.0, 8.0], [7.0, 5.0, 8.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_CP(self):
      arg=Data(numpy.array([[5.0, 8.0, 5.0, 2.0], [2.0, 8.0, 3.0, 2.0], [3.0, 7.0, 5.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 4.0, 1.0, 7.0], [4.0, 2.0, 6.0, 7.0], [5.0, 2.0, 6.0, 1.0]]))
      val=Data(numpy.array([5.0, 5.0, 2.0]),self.functionspace)
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 8.0, 5.0, 2.0], [2.0, 8.0, 5.0, 2.0], [3.0, 7.0, 2.0, 
7.0]])+(1.-msk_ref)*numpy.array([[5.0, 4.0, 5.0, 7.0], [4.0, 2.0, 5.0, 7.0], [5.0, 2.0, 2.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_CP(self):
      arg=Data(numpy.array([[4.0, 2.0, 7.0, 5.0], [6.0, 3.0, 4.0, 6.0], [8.0, 8.0, 8.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[8.0, 7.0, 8.0, 5.0], [6.0, 3.0, 7.0, 7.0], [1.0, 8.0, 4.0, 1.0]]))
      val=Data(numpy.array([6.0, 4.0, 2.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([5.0, 5.0, 3.0]))
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 2.0, 6.0, 5.0], [6.0, 3.0, 4.0, 6.0], [8.0, 8.0, 2.0, 
3.0]])+(1.-msk_ref)*numpy.array([[8.0, 7.0, 5.0, 5.0], [6.0, 3.0, 5.0, 7.0], [1.0, 8.0, 3.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_CP(self):
      arg=Data(numpy.array([[4.0, 1.0, 2.0, 3.0], [7.0, 5.0, 5.0, 6.0], [4.0, 6.0, 6.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 3.0, 1.0, 1.0], [6.0, 1.0, 3.0, 3.0], [4.0, 3.0, 6.0, 3.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([4.0, 5.0, 4.0])+(1.-msk_val)*numpy.array([1.0, 6.0, 6.0])
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 1.0, 4.0, 3.0], [7.0, 5.0, 5.0, 6.0], [4.0, 6.0, 4.0, 
3.0]])+(1.-msk_ref)*numpy.array([[7.0, 3.0, 1.0, 1.0], [6.0, 1.0, 6.0, 3.0], [4.0, 3.0, 6.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_CC(self):
      arg=Data(numpy.array([[3.0, 5.0, 1.0, 7.0], [4.0, 5.0, 4.0, 8.0], [6.0, 2.0, 2.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 3.0, 2.0, 3.0], [4.0, 3.0, 6.0, 2.0], [2.0, 6.0, 2.0, 3.0]]))
      val=numpy.array([[1.0, 6.0, 8.0, 5.0], [4.0, 2.0, 6.0, 7.0], [4.0, 6.0, 4.0, 3.0]])
      arg[:,:]=val
      ref=Data(numpy.array([[1.0, 6.0, 8.0, 5.0], [4.0, 2.0, 6.0, 7.0], [4.0, 6.0, 4.0, 3.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_CC(self):
      arg=Data(numpy.array([[1.0, 8.0, 4.0, 1.0], [4.0, 7.0, 6.0, 5.0], [5.0, 6.0, 8.0, 1.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 5.0, 8.0, 5.0], [1.0, 6.0, 8.0, 1.0], [7.0, 1.0, 2.0, 6.0]]))
      val=Data(numpy.array([[4.0, 8.0, 7.0, 7.0], [6.0, 2.0, 4.0, 5.0], [6.0, 7.0, 3.0, 4.0]]),self.functionspace)
      arg[:,:]=val
      ref=Data(numpy.array([[4.0, 8.0, 7.0, 7.0], [6.0, 2.0, 4.0, 5.0], [6.0, 7.0, 3.0, 4.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_CC(self):
      arg=Data(numpy.array([[5.0, 8.0, 2.0, 2.0], [4.0, 7.0, 6.0, 6.0], [7.0, 8.0, 1.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[1.0, 2.0, 4.0, 1.0], [6.0, 5.0, 5.0, 8.0], [1.0, 8.0, 2.0, 5.0]]))
      val=Data(numpy.array([[4.0, 4.0, 3.0, 4.0], [4.0, 4.0, 2.0, 3.0], [8.0, 1.0, 7.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 6.0, 5.0, 2.0], [2.0, 4.0, 4.0, 6.0], [8.0, 4.0, 5.0, 6.0]]))
      arg[:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 4.0, 3.0, 4.0], [4.0, 4.0, 2.0, 3.0], [8.0, 1.0, 7.0, 
2.0]])+(1.-msk_ref)*numpy.array([[1.0, 6.0, 5.0, 2.0], [2.0, 4.0, 4.0, 6.0], [8.0, 4.0, 5.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_CC(self):
      arg=Data(numpy.array([[7.0, 5.0, 4.0, 4.0], [8.0, 6.0, 5.0, 5.0], [6.0, 7.0, 3.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[4.0, 1.0, 1.0, 1.0], [8.0, 3.0, 6.0, 6.0], [2.0, 2.0, 4.0, 1.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[8.0, 3.0, 4.0, 7.0], [4.0, 5.0, 1.0, 4.0], [4.0, 7.0, 8.0, 
4.0]])+(1.-msk_val)*numpy.array([[5.0, 5.0, 7.0, 2.0], [4.0, 6.0, 1.0, 5.0], [5.0, 4.0, 6.0, 7.0]])
      arg[:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 3.0, 4.0, 7.0], [4.0, 5.0, 1.0, 4.0], [4.0, 7.0, 8.0, 
4.0]])+(1.-msk_ref)*numpy.array([[5.0, 5.0, 7.0, 2.0], [4.0, 6.0, 1.0, 5.0], [5.0, 4.0, 6.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_RR(self):
      arg=Data(numpy.array([[1.0, 8.0, 2.0, 3.0], [1.0, 6.0, 4.0, 3.0], [6.0, 1.0, 2.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 6.0, 2.0, 1.0], [6.0, 6.0, 2.0, 1.0], [2.0, 2.0, 7.0, 5.0]]))
      val=numpy.array([[5.0, 2.0]])
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 8.0, 2.0, 3.0], [1.0, 5.0, 2.0, 3.0], [6.0, 1.0, 2.0, 
2.0]])+(1.-msk_ref)*numpy.array([[6.0, 6.0, 2.0, 1.0], [6.0, 5.0, 2.0, 1.0], [2.0, 2.0, 7.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_RR(self):
      arg=Data(numpy.array([[5.0, 6.0, 6.0, 7.0], [4.0, 1.0, 8.0, 1.0], [7.0, 5.0, 2.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 8.0, 2.0, 1.0], [2.0, 3.0, 7.0, 2.0], [3.0, 4.0, 1.0, 7.0]]))
      val=Data(numpy.array([[6.0, 8.0]]),self.functionspace)
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 6.0, 6.0, 7.0], [4.0, 6.0, 8.0, 1.0], [7.0, 5.0, 2.0, 
5.0]])+(1.-msk_ref)*numpy.array([[6.0, 8.0, 2.0, 1.0], [2.0, 6.0, 8.0, 2.0], [3.0, 4.0, 1.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_RR(self):
      arg=Data(numpy.array([[2.0, 6.0, 5.0, 7.0], [4.0, 1.0, 1.0, 7.0], [3.0, 5.0, 3.0, 1.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 5.0, 3.0, 5.0], [2.0, 8.0, 6.0, 4.0], [2.0, 8.0, 6.0, 7.0]]))
      val=Data(numpy.array([[3.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 2.0]]))
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 6.0, 5.0, 7.0], [4.0, 3.0, 6.0, 7.0], [3.0, 5.0, 3.0, 
1.0]])+(1.-msk_ref)*numpy.array([[5.0, 5.0, 3.0, 5.0], [2.0, 3.0, 2.0, 4.0], [2.0, 8.0, 6.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_RR(self):
      arg=Data(numpy.array([[3.0, 1.0, 5.0, 7.0], [5.0, 1.0, 1.0, 1.0], [4.0, 1.0, 5.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 2.0, 5.0, 6.0], [4.0, 8.0, 8.0, 4.0], [6.0, 1.0, 1.0, 8.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[4.0, 3.0]])+(1.-msk_val)*numpy.array([[7.0, 3.0]])
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 1.0, 5.0, 7.0], [5.0, 4.0, 3.0, 1.0], [4.0, 1.0, 5.0, 
3.0]])+(1.-msk_ref)*numpy.array([[5.0, 2.0, 5.0, 6.0], [4.0, 7.0, 3.0, 4.0], [6.0, 1.0, 1.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_CN(self):
      arg=Data(numpy.array([[3.0, 7.0, 2.0, 7.0], [4.0, 1.0, 5.0, 3.0], [1.0, 6.0, 4.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[1.0, 1.0, 2.0, 3.0], [1.0, 6.0, 2.0, 3.0], [6.0, 6.0, 7.0, 4.0]]))
      val=numpy.array([[6.0, 4.0, 7.0, 1.0], [2.0, 3.0, 1.0, 7.0], [4.0, 3.0, 8.0, 4.0]])
      arg[:]=val
      ref=Data(numpy.array([[6.0, 4.0, 7.0, 1.0], [2.0, 3.0, 1.0, 7.0], [4.0, 3.0, 8.0, 4.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_CN(self):
      arg=Data(numpy.array([[4.0, 2.0, 2.0, 8.0], [5.0, 8.0, 3.0, 6.0], [4.0, 1.0, 6.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[4.0, 1.0, 5.0, 7.0], [7.0, 7.0, 1.0, 7.0], [6.0, 2.0, 1.0, 1.0]]))
      val=Data(numpy.array([[7.0, 4.0, 8.0, 4.0], [3.0, 1.0, 8.0, 7.0], [3.0, 5.0, 3.0, 1.0]]),self.functionspace)
      arg[:]=val
      ref=Data(numpy.array([[7.0, 4.0, 8.0, 4.0], [3.0, 1.0, 8.0, 7.0], [3.0, 5.0, 3.0, 1.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_CN(self):
      arg=Data(numpy.array([[4.0, 4.0, 4.0, 6.0], [8.0, 4.0, 7.0, 4.0], [8.0, 5.0, 8.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 8.0, 3.0, 2.0], [2.0, 8.0, 6.0, 1.0], [2.0, 6.0, 2.0, 2.0]]))
      val=Data(numpy.array([[4.0, 6.0, 5.0, 5.0], [3.0, 4.0, 7.0, 3.0], [5.0, 5.0, 7.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 3.0, 1.0, 8.0], [5.0, 6.0, 2.0, 6.0], [5.0, 5.0, 2.0, 6.0]]))
      arg[:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 6.0, 5.0, 5.0], [3.0, 4.0, 7.0, 3.0], [5.0, 5.0, 7.0, 
7.0]])+(1.-msk_ref)*numpy.array([[2.0, 3.0, 1.0, 8.0], [5.0, 6.0, 2.0, 6.0], [5.0, 5.0, 2.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_CN(self):
      arg=Data(numpy.array([[2.0, 5.0, 2.0, 5.0], [4.0, 7.0, 4.0, 5.0], [5.0, 4.0, 1.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[8.0, 5.0, 8.0, 3.0], [1.0, 7.0, 1.0, 1.0], [7.0, 2.0, 6.0, 2.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[7.0, 4.0, 4.0, 6.0], [2.0, 8.0, 5.0, 8.0], [2.0, 3.0, 8.0, 
1.0]])+(1.-msk_val)*numpy.array([[1.0, 2.0, 5.0, 1.0], [3.0, 3.0, 8.0, 3.0], [5.0, 1.0, 8.0, 1.0]])
      arg[:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 4.0, 4.0, 6.0], [2.0, 8.0, 5.0, 8.0], [2.0, 3.0, 8.0, 
1.0]])+(1.-msk_ref)*numpy.array([[1.0, 2.0, 5.0, 1.0], [3.0, 3.0, 8.0, 3.0], [5.0, 1.0, 8.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_PR(self):
      arg=Data(numpy.array([[4.0, 6.0, 2.0, 4.0], [5.0, 7.0, 2.0, 2.0], [4.0, 2.0, 3.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[1.0, 3.0, 1.0, 7.0], [8.0, 4.0, 3.0, 7.0], [3.0, 4.0, 6.0, 1.0]]))
      val=numpy.array([7.0, 7.0])
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 6.0, 2.0, 4.0], [5.0, 7.0, 7.0, 2.0], [4.0, 2.0, 3.0, 
7.0]])+(1.-msk_ref)*numpy.array([[1.0, 3.0, 1.0, 7.0], [8.0, 7.0, 7.0, 7.0], [3.0, 4.0, 6.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_PR(self):
      arg=Data(numpy.array([[4.0, 3.0, 8.0, 7.0], [6.0, 2.0, 7.0, 2.0], [2.0, 2.0, 6.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 3.0, 4.0, 8.0], [4.0, 6.0, 6.0, 7.0], [5.0, 7.0, 5.0, 1.0]]))
      val=Data(numpy.array([3.0, 6.0]),self.functionspace)
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 3.0, 8.0, 7.0], [6.0, 3.0, 6.0, 2.0], [2.0, 2.0, 6.0, 
2.0]])+(1.-msk_ref)*numpy.array([[7.0, 3.0, 4.0, 8.0], [4.0, 3.0, 6.0, 7.0], [5.0, 7.0, 5.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_PR(self):
      arg=Data(numpy.array([[3.0, 1.0, 6.0, 1.0], [3.0, 2.0, 4.0, 8.0], [2.0, 3.0, 4.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[8.0, 6.0, 1.0, 1.0], [2.0, 8.0, 1.0, 8.0], [1.0, 3.0, 7.0, 1.0]]))
      val=Data(numpy.array([2.0, 7.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([6.0, 6.0]))
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 1.0, 6.0, 1.0], [3.0, 2.0, 7.0, 8.0], [2.0, 3.0, 4.0, 
7.0]])+(1.-msk_ref)*numpy.array([[8.0, 6.0, 1.0, 1.0], [2.0, 6.0, 6.0, 8.0], [1.0, 3.0, 7.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_PR(self):
      arg=Data(numpy.array([[6.0, 3.0, 1.0, 1.0], [2.0, 5.0, 3.0, 8.0], [7.0, 6.0, 1.0, 6.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 2.0, 7.0, 2.0], [3.0, 2.0, 6.0, 7.0], [5.0, 7.0, 1.0, 2.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([7.0, 8.0])+(1.-msk_val)*numpy.array([1.0, 5.0])
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 3.0, 1.0, 1.0], [2.0, 7.0, 8.0, 8.0], [7.0, 6.0, 1.0, 
6.0]])+(1.-msk_ref)*numpy.array([[5.0, 2.0, 7.0, 2.0], [3.0, 1.0, 5.0, 7.0], [5.0, 7.0, 1.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_UL(self):
      arg=Data(numpy.array([[4.0, 4.0, 8.0, 5.0], [8.0, 4.0, 6.0, 8.0], [1.0, 4.0, 8.0, 1.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 1.0, 6.0, 6.0], [6.0, 3.0, 8.0, 2.0], [5.0, 6.0, 3.0, 8.0]]))
      val=numpy.array([[2.0, 1.0], [7.0, 2.0]])
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 4.0, 2.0, 1.0], [8.0, 4.0, 7.0, 2.0], [1.0, 4.0, 8.0, 
1.0]])+(1.-msk_ref)*numpy.array([[3.0, 1.0, 2.0, 1.0], [6.0, 3.0, 7.0, 2.0], [5.0, 6.0, 3.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_UL(self):
      arg=Data(numpy.array([[5.0, 8.0, 7.0, 1.0], [3.0, 2.0, 1.0, 6.0], [2.0, 3.0, 1.0, 6.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 4.0, 7.0, 2.0], [6.0, 5.0, 8.0, 7.0], [4.0, 2.0, 4.0, 8.0]]))
      val=Data(numpy.array([[2.0, 6.0], [4.0, 7.0]]),self.functionspace)
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 8.0, 2.0, 6.0], [3.0, 2.0, 4.0, 7.0], [2.0, 3.0, 1.0, 
6.0]])+(1.-msk_ref)*numpy.array([[7.0, 4.0, 2.0, 6.0], [6.0, 5.0, 4.0, 7.0], [4.0, 2.0, 4.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_UL(self):
      arg=Data(numpy.array([[5.0, 4.0, 5.0, 1.0], [6.0, 5.0, 5.0, 1.0], [3.0, 2.0, 1.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 6.0, 4.0, 1.0], [1.0, 8.0, 4.0, 8.0], [4.0, 5.0, 1.0, 4.0]]))
      val=Data(numpy.array([[2.0, 2.0], [3.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 2.0], [2.0, 7.0]]))
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 4.0, 2.0, 2.0], [6.0, 5.0, 3.0, 3.0], [3.0, 2.0, 1.0, 
5.0]])+(1.-msk_ref)*numpy.array([[6.0, 6.0, 6.0, 2.0], [1.0, 8.0, 2.0, 7.0], [4.0, 5.0, 1.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_UL(self):
      arg=Data(numpy.array([[2.0, 5.0, 7.0, 2.0], [7.0, 7.0, 4.0, 2.0], [2.0, 7.0, 4.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 4.0, 1.0, 4.0], [7.0, 4.0, 4.0, 6.0], [7.0, 8.0, 4.0, 4.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[7.0, 6.0], [4.0, 4.0]])+(1.-msk_val)*numpy.array([[3.0, 2.0], [2.0, 6.0]])
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 5.0, 7.0, 6.0], [7.0, 7.0, 4.0, 4.0], [2.0, 7.0, 4.0, 
8.0]])+(1.-msk_ref)*numpy.array([[2.0, 4.0, 3.0, 2.0], [7.0, 4.0, 2.0, 6.0], [7.0, 8.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_LN(self):
      arg=Data(numpy.array([[1.0, 6.0, 6.0, 2.0], [8.0, 7.0, 7.0, 1.0], [8.0, 7.0, 7.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 6.0, 5.0, 8.0], [8.0, 1.0, 7.0, 4.0], [5.0, 4.0, 8.0, 2.0]]))
      val=numpy.array([[5.0, 5.0, 1.0, 4.0]])
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 6.0, 6.0, 2.0], [8.0, 7.0, 7.0, 1.0], [5.0, 5.0, 1.0, 
4.0]])+(1.-msk_ref)*numpy.array([[7.0, 6.0, 5.0, 8.0], [8.0, 1.0, 7.0, 4.0], [5.0, 5.0, 1.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_LN(self):
      arg=Data(numpy.array([[8.0, 6.0, 5.0, 7.0], [5.0, 1.0, 5.0, 8.0], [8.0, 8.0, 8.0, 6.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 5.0, 6.0, 7.0], [5.0, 4.0, 2.0, 4.0], [3.0, 5.0, 4.0, 3.0]]))
      val=Data(numpy.array([[8.0, 2.0, 2.0, 4.0]]),self.functionspace)
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 6.0, 5.0, 7.0], [5.0, 1.0, 5.0, 8.0], [8.0, 2.0, 2.0, 
4.0]])+(1.-msk_ref)*numpy.array([[2.0, 5.0, 6.0, 7.0], [5.0, 4.0, 2.0, 4.0], [8.0, 2.0, 2.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_LN(self):
      arg=Data(numpy.array([[6.0, 5.0, 2.0, 4.0], [5.0, 2.0, 2.0, 1.0], [5.0, 6.0, 8.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 4.0, 7.0, 5.0], [6.0, 1.0, 4.0, 3.0], [5.0, 1.0, 1.0, 8.0]]))
      val=Data(numpy.array([[1.0, 2.0, 3.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 5.0, 2.0, 6.0]]))
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 5.0, 2.0, 4.0], [5.0, 2.0, 2.0, 1.0], [1.0, 2.0, 3.0, 
8.0]])+(1.-msk_ref)*numpy.array([[6.0, 4.0, 7.0, 5.0], [6.0, 1.0, 4.0, 3.0], [5.0, 5.0, 2.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_LN(self):
      arg=Data(numpy.array([[3.0, 7.0, 7.0, 5.0], [1.0, 4.0, 8.0, 8.0], [4.0, 4.0, 5.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 6.0, 6.0, 6.0], [2.0, 1.0, 7.0, 5.0], [5.0, 4.0, 6.0, 4.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[8.0, 3.0, 2.0, 2.0]])+(1.-msk_val)*numpy.array([[7.0, 4.0, 4.0, 7.0]])
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 7.0, 7.0, 5.0], [1.0, 4.0, 8.0, 8.0], [8.0, 3.0, 2.0, 
2.0]])+(1.-msk_ref)*numpy.array([[6.0, 6.0, 6.0, 6.0], [2.0, 1.0, 7.0, 5.0], [7.0, 4.0, 4.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_PN(self):
      arg=Data(numpy.array([[5.0, 4.0, 5.0, 4.0], [5.0, 3.0, 5.0, 2.0], [7.0, 5.0, 7.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 4.0, 3.0, 8.0], [8.0, 7.0, 8.0, 3.0], [2.0, 5.0, 7.0, 5.0]]))
      val=numpy.array([6.0, 6.0, 2.0, 5.0])
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 4.0, 5.0, 4.0], [6.0, 6.0, 2.0, 5.0], [7.0, 5.0, 7.0, 
4.0]])+(1.-msk_ref)*numpy.array([[3.0, 4.0, 3.0, 8.0], [6.0, 6.0, 2.0, 5.0], [2.0, 5.0, 7.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_PN(self):
      arg=Data(numpy.array([[6.0, 8.0, 7.0, 3.0], [3.0, 7.0, 7.0, 2.0], [6.0, 3.0, 5.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[4.0, 7.0, 4.0, 2.0], [1.0, 6.0, 6.0, 5.0], [1.0, 2.0, 4.0, 3.0]]))
      val=Data(numpy.array([1.0, 1.0, 4.0, 5.0]),self.functionspace)
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 8.0, 7.0, 3.0], [1.0, 1.0, 4.0, 5.0], [6.0, 3.0, 5.0, 
7.0]])+(1.-msk_ref)*numpy.array([[4.0, 7.0, 4.0, 2.0], [1.0, 1.0, 4.0, 5.0], [1.0, 2.0, 4.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_PN(self):
      arg=Data(numpy.array([[8.0, 8.0, 3.0, 4.0], [6.0, 7.0, 1.0, 3.0], [5.0, 7.0, 5.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 7.0, 4.0, 4.0], [5.0, 5.0, 8.0, 8.0], [6.0, 5.0, 4.0, 7.0]]))
      val=Data(numpy.array([1.0, 8.0, 4.0, 6.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([2.0, 5.0, 3.0, 5.0]))
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 8.0, 3.0, 4.0], [1.0, 8.0, 4.0, 6.0], [5.0, 7.0, 5.0, 
2.0]])+(1.-msk_ref)*numpy.array([[6.0, 7.0, 4.0, 4.0], [2.0, 5.0, 3.0, 5.0], [6.0, 5.0, 4.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_PN(self):
      arg=Data(numpy.array([[1.0, 3.0, 3.0, 5.0], [2.0, 3.0, 8.0, 6.0], [1.0, 2.0, 2.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 5.0, 1.0, 2.0], [5.0, 8.0, 2.0, 8.0], [8.0, 7.0, 6.0, 8.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([7.0, 4.0, 1.0, 5.0])+(1.-msk_val)*numpy.array([8.0, 8.0, 7.0, 6.0])
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 3.0, 3.0, 5.0], [7.0, 4.0, 1.0, 5.0], [1.0, 2.0, 2.0, 
3.0]])+(1.-msk_ref)*numpy.array([[3.0, 5.0, 1.0, 2.0], [8.0, 8.0, 7.0, 6.0], [8.0, 7.0, 6.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_CR(self):
      arg=Data(numpy.array([[5.0, 2.0, 4.0, 4.0], [3.0, 3.0, 5.0, 4.0], [7.0, 5.0, 4.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 8.0, 7.0, 4.0], [2.0, 8.0, 2.0, 8.0], [7.0, 8.0, 7.0, 4.0]]))
      val=numpy.array([[4.0, 2.0], [4.0, 6.0], [1.0, 4.0]])
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 4.0, 2.0, 4.0], [3.0, 4.0, 6.0, 4.0], [7.0, 1.0, 4.0, 
5.0]])+(1.-msk_ref)*numpy.array([[6.0, 4.0, 2.0, 4.0], [2.0, 4.0, 6.0, 8.0], [7.0, 1.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_CR(self):
      arg=Data(numpy.array([[5.0, 6.0, 8.0, 1.0], [3.0, 5.0, 8.0, 3.0], [7.0, 3.0, 2.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[4.0, 5.0, 3.0, 7.0], [6.0, 4.0, 3.0, 5.0], [5.0, 8.0, 7.0, 3.0]]))
      val=Data(numpy.array([[4.0, 1.0], [6.0, 1.0], [3.0, 4.0]]),self.functionspace)
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 4.0, 1.0, 1.0], [3.0, 6.0, 1.0, 3.0], [7.0, 3.0, 4.0, 
8.0]])+(1.-msk_ref)*numpy.array([[4.0, 4.0, 1.0, 7.0], [6.0, 6.0, 1.0, 5.0], [5.0, 3.0, 4.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_CR(self):
      arg=Data(numpy.array([[3.0, 8.0, 3.0, 7.0], [5.0, 1.0, 8.0, 2.0], [4.0, 7.0, 2.0, 6.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 6.0, 5.0, 4.0], [6.0, 3.0, 8.0, 7.0], [5.0, 8.0, 7.0, 2.0]]))
      val=Data(numpy.array([[2.0, 3.0], [6.0, 6.0], [6.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 1.0], [6.0, 6.0], [1.0, 7.0]]))
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 2.0, 3.0, 7.0], [5.0, 6.0, 6.0, 2.0], [4.0, 6.0, 1.0, 
6.0]])+(1.-msk_ref)*numpy.array([[5.0, 5.0, 1.0, 4.0], [6.0, 6.0, 6.0, 7.0], [5.0, 1.0, 7.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_CR(self):
      arg=Data(numpy.array([[4.0, 1.0, 5.0, 6.0], [6.0, 2.0, 6.0, 7.0], [3.0, 1.0, 2.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 1.0, 6.0, 2.0], [1.0, 3.0, 8.0, 8.0], [1.0, 6.0, 8.0, 1.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[2.0, 3.0], [3.0, 3.0], [2.0, 4.0]])+(1.-msk_val)*numpy.array([[8.0, 3.0], [7.0, 7.0], [6.0, 
8.0]])
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 2.0, 3.0, 6.0], [6.0, 3.0, 3.0, 7.0], [3.0, 2.0, 4.0, 
5.0]])+(1.-msk_ref)*numpy.array([[6.0, 8.0, 3.0, 2.0], [1.0, 7.0, 7.0, 8.0], [1.0, 6.0, 8.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_CU(self):
      arg=Data(numpy.array([[8.0, 5.0, 4.0, 7.0], [3.0, 8.0, 7.0, 3.0], [5.0, 3.0, 5.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 8.0, 3.0, 8.0], [6.0, 1.0, 4.0, 3.0], [3.0, 5.0, 2.0, 1.0]]))
      val=numpy.array([[3.0, 1.0, 1.0], [1.0, 1.0, 5.0], [3.0, 2.0, 3.0]])
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 1.0, 1.0, 7.0], [1.0, 1.0, 5.0, 3.0], [3.0, 2.0, 3.0, 
7.0]])+(1.-msk_ref)*numpy.array([[3.0, 1.0, 1.0, 8.0], [1.0, 1.0, 5.0, 3.0], [3.0, 2.0, 3.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_CU(self):
      arg=Data(numpy.array([[1.0, 1.0, 7.0, 4.0], [4.0, 7.0, 5.0, 3.0], [3.0, 4.0, 3.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 7.0, 5.0, 1.0], [8.0, 8.0, 3.0, 7.0], [4.0, 2.0, 5.0, 6.0]]))
      val=Data(numpy.array([[2.0, 3.0, 1.0], [7.0, 7.0, 6.0], [3.0, 8.0, 2.0]]),self.functionspace)
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 3.0, 1.0, 4.0], [7.0, 7.0, 6.0, 3.0], [3.0, 8.0, 2.0, 
3.0]])+(1.-msk_ref)*numpy.array([[2.0, 3.0, 1.0, 1.0], [7.0, 7.0, 6.0, 7.0], [3.0, 8.0, 2.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_CU(self):
      arg=Data(numpy.array([[5.0, 8.0, 7.0, 1.0], [2.0, 6.0, 7.0, 8.0], [1.0, 8.0, 5.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 6.0, 1.0, 5.0], [1.0, 4.0, 6.0, 4.0], [1.0, 7.0, 8.0, 1.0]]))
      val=Data(numpy.array([[6.0, 4.0, 4.0], [7.0, 4.0, 5.0], [2.0, 3.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 1.0, 8.0], [7.0, 3.0, 4.0], [8.0, 6.0, 7.0]]))
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 4.0, 4.0, 1.0], [7.0, 4.0, 5.0, 8.0], [2.0, 3.0, 3.0, 
4.0]])+(1.-msk_ref)*numpy.array([[7.0, 1.0, 8.0, 5.0], [7.0, 3.0, 4.0, 4.0], [8.0, 6.0, 7.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_CU(self):
      arg=Data(numpy.array([[1.0, 6.0, 8.0, 1.0], [8.0, 3.0, 6.0, 2.0], [7.0, 6.0, 5.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 5.0, 3.0, 6.0], [8.0, 6.0, 5.0, 2.0], [5.0, 6.0, 6.0, 8.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 8.0, 3.0], [4.0, 4.0, 1.0], [3.0, 3.0, 1.0]])+(1.-msk_val)*numpy.array([[3.0, 2.0, 8.0], 
[1.0, 6.0, 1.0], [3.0, 8.0, 3.0]])
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 8.0, 3.0, 1.0], [4.0, 4.0, 1.0, 2.0], [3.0, 3.0, 1.0, 
5.0]])+(1.-msk_ref)*numpy.array([[3.0, 2.0, 8.0, 6.0], [1.0, 6.0, 1.0, 2.0], [3.0, 8.0, 3.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_LC(self):
      arg=Data(numpy.array([[6.0, 2.0, 5.0, 2.0], [6.0, 6.0, 8.0, 2.0], [2.0, 5.0, 7.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 4.0, 2.0, 8.0], [4.0, 2.0, 7.0, 4.0], [4.0, 2.0, 2.0, 6.0]]))
      val=numpy.array([[4.0, 2.0, 1.0, 6.0]])
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 2.0, 5.0, 2.0], [6.0, 6.0, 8.0, 2.0], [4.0, 2.0, 1.0, 
6.0]])+(1.-msk_ref)*numpy.array([[6.0, 4.0, 2.0, 8.0], [4.0, 2.0, 7.0, 4.0], [4.0, 2.0, 1.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_LC(self):
      arg=Data(numpy.array([[6.0, 7.0, 7.0, 2.0], [2.0, 5.0, 6.0, 3.0], [8.0, 8.0, 5.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 4.0, 4.0, 3.0], [4.0, 4.0, 4.0, 1.0], [6.0, 1.0, 3.0, 2.0]]))
      val=Data(numpy.array([[3.0, 1.0, 8.0, 5.0]]),self.functionspace)
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 7.0, 7.0, 2.0], [2.0, 5.0, 6.0, 3.0], [3.0, 1.0, 8.0, 
5.0]])+(1.-msk_ref)*numpy.array([[3.0, 4.0, 4.0, 3.0], [4.0, 4.0, 4.0, 1.0], [3.0, 1.0, 8.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_LC(self):
      arg=Data(numpy.array([[4.0, 4.0, 1.0, 4.0], [5.0, 5.0, 7.0, 3.0], [3.0, 3.0, 1.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 4.0, 2.0, 4.0], [7.0, 1.0, 2.0, 6.0], [3.0, 5.0, 3.0, 2.0]]))
      val=Data(numpy.array([[8.0, 5.0, 8.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 6.0, 8.0, 5.0]]))
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 4.0, 1.0, 4.0], [5.0, 5.0, 7.0, 3.0], [8.0, 5.0, 8.0, 
7.0]])+(1.-msk_ref)*numpy.array([[6.0, 4.0, 2.0, 4.0], [7.0, 1.0, 2.0, 6.0], [1.0, 6.0, 8.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_LC(self):
      arg=Data(numpy.array([[7.0, 1.0, 6.0, 3.0], [8.0, 6.0, 6.0, 6.0], [5.0, 3.0, 5.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 3.0, 5.0, 8.0], [5.0, 2.0, 5.0, 7.0], [7.0, 7.0, 5.0, 3.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[3.0, 3.0, 7.0, 6.0]])+(1.-msk_val)*numpy.array([[3.0, 1.0, 2.0, 1.0]])
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 1.0, 6.0, 3.0], [8.0, 6.0, 6.0, 6.0], [3.0, 3.0, 7.0, 
6.0]])+(1.-msk_ref)*numpy.array([[5.0, 3.0, 5.0, 8.0], [5.0, 2.0, 5.0, 7.0], [3.0, 1.0, 2.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_UN(self):
      arg=Data(numpy.array([[6.0, 7.0, 4.0, 3.0], [2.0, 1.0, 3.0, 5.0], [5.0, 8.0, 5.0, 2.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[4.0, 6.0, 1.0, 1.0], [4.0, 8.0, 4.0, 4.0], [6.0, 3.0, 7.0, 6.0]]))
      val=numpy.array([[4.0, 2.0, 8.0, 3.0], [5.0, 1.0, 2.0, 2.0]])
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 2.0, 8.0, 3.0], [5.0, 1.0, 2.0, 2.0], [5.0, 8.0, 5.0, 
2.0]])+(1.-msk_ref)*numpy.array([[4.0, 2.0, 8.0, 3.0], [5.0, 1.0, 2.0, 2.0], [6.0, 3.0, 7.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_UN(self):
      arg=Data(numpy.array([[2.0, 1.0, 1.0, 3.0], [6.0, 7.0, 5.0, 4.0], [4.0, 7.0, 7.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 6.0, 4.0, 3.0], [5.0, 1.0, 2.0, 5.0], [3.0, 7.0, 1.0, 5.0]]))
      val=Data(numpy.array([[2.0, 5.0, 3.0, 3.0], [6.0, 3.0, 5.0, 4.0]]),self.functionspace)
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 5.0, 3.0, 3.0], [6.0, 3.0, 5.0, 4.0], [4.0, 7.0, 7.0, 
8.0]])+(1.-msk_ref)*numpy.array([[2.0, 5.0, 3.0, 3.0], [6.0, 3.0, 5.0, 4.0], [3.0, 7.0, 1.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_UN(self):
      arg=Data(numpy.array([[3.0, 7.0, 3.0, 7.0], [1.0, 8.0, 6.0, 5.0], [2.0, 8.0, 6.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[1.0, 1.0, 8.0, 3.0], [6.0, 4.0, 8.0, 8.0], [7.0, 1.0, 3.0, 4.0]]))
      val=Data(numpy.array([[2.0, 7.0, 6.0, 1.0], [5.0, 6.0, 4.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 2.0, 8.0, 8.0], [8.0, 1.0, 2.0, 3.0]]))
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 7.0, 6.0, 1.0], [5.0, 6.0, 4.0, 3.0], [2.0, 8.0, 6.0, 
7.0]])+(1.-msk_ref)*numpy.array([[3.0, 2.0, 8.0, 8.0], [8.0, 1.0, 2.0, 3.0], [7.0, 1.0, 3.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_UN(self):
      arg=Data(numpy.array([[3.0, 7.0, 7.0, 8.0], [7.0, 5.0, 6.0, 7.0], [3.0, 4.0, 7.0, 3.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 1.0, 6.0, 4.0], [4.0, 1.0, 6.0, 5.0], [7.0, 2.0, 4.0, 7.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 5.0, 6.0, 1.0], [4.0, 7.0, 7.0, 2.0]])+(1.-msk_val)*numpy.array([[1.0, 2.0, 1.0, 2.0], 
[5.0, 8.0, 4.0, 8.0]])
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 5.0, 6.0, 1.0], [4.0, 7.0, 7.0, 2.0], [3.0, 4.0, 7.0, 
3.0]])+(1.-msk_ref)*numpy.array([[1.0, 2.0, 1.0, 2.0], [5.0, 8.0, 4.0, 8.0], [7.0, 2.0, 4.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_LL(self):
      arg=Data(numpy.array([[7.0, 7.0, 6.0, 6.0], [2.0, 7.0, 1.0, 8.0], [2.0, 6.0, 7.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 4.0, 4.0, 8.0], [8.0, 6.0, 6.0, 1.0], [2.0, 4.0, 2.0, 2.0]]))
      val=numpy.array([[6.0, 2.0]])
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 7.0, 6.0, 6.0], [2.0, 7.0, 1.0, 8.0], [2.0, 6.0, 6.0, 
2.0]])+(1.-msk_ref)*numpy.array([[7.0, 4.0, 4.0, 8.0], [8.0, 6.0, 6.0, 1.0], [2.0, 4.0, 6.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_LL(self):
      arg=Data(numpy.array([[3.0, 7.0, 2.0, 5.0], [1.0, 4.0, 6.0, 3.0], [6.0, 6.0, 8.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[4.0, 4.0, 7.0, 5.0], [4.0, 7.0, 8.0, 5.0], [8.0, 1.0, 8.0, 7.0]]))
      val=Data(numpy.array([[1.0, 8.0]]),self.functionspace)
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 7.0, 2.0, 5.0], [1.0, 4.0, 6.0, 3.0], [6.0, 6.0, 1.0, 
8.0]])+(1.-msk_ref)*numpy.array([[4.0, 4.0, 7.0, 5.0], [4.0, 7.0, 8.0, 5.0], [8.0, 1.0, 1.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_LL(self):
      arg=Data(numpy.array([[7.0, 3.0, 1.0, 8.0], [3.0, 3.0, 5.0, 5.0], [1.0, 4.0, 5.0, 5.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[1.0, 8.0, 3.0, 5.0], [8.0, 4.0, 5.0, 7.0], [3.0, 4.0, 8.0, 3.0]]))
      val=Data(numpy.array([[4.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 8.0]]))
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 3.0, 1.0, 8.0], [3.0, 3.0, 5.0, 5.0], [1.0, 4.0, 4.0, 
8.0]])+(1.-msk_ref)*numpy.array([[1.0, 8.0, 3.0, 5.0], [8.0, 4.0, 5.0, 7.0], [3.0, 4.0, 5.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_LL(self):
      arg=Data(numpy.array([[6.0, 3.0, 8.0, 2.0], [1.0, 8.0, 2.0, 2.0], [8.0, 8.0, 1.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 4.0, 7.0, 3.0], [4.0, 8.0, 5.0, 7.0], [7.0, 6.0, 2.0, 7.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[2.0, 2.0]])+(1.-msk_val)*numpy.array([[4.0, 1.0]])
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 3.0, 8.0, 2.0], [1.0, 8.0, 2.0, 2.0], [8.0, 8.0, 2.0, 
2.0]])+(1.-msk_ref)*numpy.array([[5.0, 4.0, 7.0, 3.0], [4.0, 8.0, 5.0, 7.0], [7.0, 6.0, 4.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_RN(self):
      arg=Data(numpy.array([[5.0, 7.0, 7.0, 5.0], [2.0, 4.0, 1.0, 1.0], [3.0, 4.0, 4.0, 7.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[2.0, 3.0, 4.0, 2.0], [5.0, 3.0, 3.0, 5.0], [1.0, 4.0, 4.0, 2.0]]))
      val=numpy.array([[1.0, 4.0, 7.0, 3.0]])
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 7.0, 7.0, 5.0], [1.0, 4.0, 7.0, 3.0], [3.0, 4.0, 4.0, 
7.0]])+(1.-msk_ref)*numpy.array([[2.0, 3.0, 4.0, 2.0], [1.0, 4.0, 7.0, 3.0], [1.0, 4.0, 4.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_RN(self):
      arg=Data(numpy.array([[4.0, 2.0, 7.0, 2.0], [4.0, 4.0, 8.0, 7.0], [8.0, 6.0, 7.0, 6.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[4.0, 4.0, 2.0, 4.0], [4.0, 4.0, 4.0, 8.0], [6.0, 6.0, 6.0, 5.0]]))
      val=Data(numpy.array([[4.0, 2.0, 6.0, 4.0]]),self.functionspace)
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 2.0, 7.0, 2.0], [4.0, 2.0, 6.0, 4.0], [8.0, 6.0, 7.0, 
6.0]])+(1.-msk_ref)*numpy.array([[4.0, 4.0, 2.0, 4.0], [4.0, 2.0, 6.0, 4.0], [6.0, 6.0, 6.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_RN(self):
      arg=Data(numpy.array([[5.0, 1.0, 3.0, 7.0], [4.0, 5.0, 3.0, 3.0], [3.0, 7.0, 1.0, 1.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 1.0, 1.0, 8.0], [7.0, 3.0, 4.0, 5.0], [7.0, 8.0, 2.0, 4.0]]))
      val=Data(numpy.array([[4.0, 1.0, 7.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 1.0, 2.0, 5.0]]))
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 1.0, 3.0, 7.0], [4.0, 1.0, 7.0, 8.0], [3.0, 7.0, 1.0, 
1.0]])+(1.-msk_ref)*numpy.array([[5.0, 1.0, 1.0, 8.0], [5.0, 1.0, 2.0, 5.0], [7.0, 8.0, 2.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_RN(self):
      arg=Data(numpy.array([[8.0, 6.0, 2.0, 1.0], [3.0, 6.0, 8.0, 5.0], [2.0, 1.0, 7.0, 6.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[1.0, 4.0, 5.0, 2.0], [1.0, 6.0, 6.0, 5.0], [2.0, 6.0, 7.0, 2.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[7.0, 3.0, 5.0, 3.0]])+(1.-msk_val)*numpy.array([[7.0, 8.0, 8.0, 2.0]])
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 6.0, 2.0, 1.0], [7.0, 3.0, 5.0, 3.0], [2.0, 1.0, 7.0, 
6.0]])+(1.-msk_ref)*numpy.array([[1.0, 4.0, 5.0, 2.0], [7.0, 8.0, 8.0, 2.0], [2.0, 6.0, 7.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_array_PP(self):
      arg=Data(numpy.array([[5.0, 1.0, 5.0, 8.0], [4.0, 1.0, 8.0, 8.0], [4.0, 1.0, 7.0, 8.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[7.0, 7.0, 2.0, 7.0], [7.0, 6.0, 7.0, 1.0], [7.0, 5.0, 3.0, 8.0]]))
      val=numpy.array(1.0)
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 1.0, 5.0, 8.0], [4.0, 1.0, 1.0, 8.0], [4.0, 1.0, 7.0, 
8.0]])+(1.-msk_ref)*numpy.array([[7.0, 7.0, 2.0, 7.0], [7.0, 6.0, 1.0, 1.0], [7.0, 5.0, 3.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_constData_PP(self):
      arg=Data(numpy.array([[8.0, 6.0, 8.0, 5.0], [6.0, 3.0, 6.0, 1.0], [5.0, 2.0, 3.0, 1.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[6.0, 7.0, 6.0, 2.0], [6.0, 2.0, 5.0, 5.0], [8.0, 5.0, 1.0, 3.0]]))
      val=Data(1.0,self.functionspace)
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 6.0, 8.0, 5.0], [6.0, 3.0, 1.0, 1.0], [5.0, 2.0, 3.0, 
1.0]])+(1.-msk_ref)*numpy.array([[6.0, 7.0, 6.0, 2.0], [6.0, 2.0, 1.0, 5.0], [8.0, 5.0, 1.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_taggedData_PP(self):
      arg=Data(numpy.array([[6.0, 6.0, 1.0, 7.0], [7.0, 6.0, 1.0, 8.0], [6.0, 3.0, 8.0, 4.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[5.0, 5.0, 6.0, 7.0], [5.0, 8.0, 8.0, 8.0], [1.0, 1.0, 1.0, 3.0]]))
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 6.0, 1.0, 7.0], [7.0, 6.0, 1.0, 8.0], [6.0, 3.0, 8.0, 
4.0]])+(1.-msk_ref)*numpy.array([[5.0, 5.0, 6.0, 7.0], [5.0, 8.0, 1.0, 8.0], [1.0, 1.0, 1.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank2_expandedData_PP(self):
      arg=Data(numpy.array([[2.0, 5.0, 5.0, 2.0], [5.0, 2.0, 7.0, 2.0], [3.0, 2.0, 5.0, 6.0]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[3.0, 1.0, 1.0, 6.0], [3.0, 8.0, 3.0, 1.0], [8.0, 7.0, 6.0, 7.0]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=(1.-msk_val)*(1.0)+msk_val*(1.0)
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 5.0, 5.0, 2.0], [5.0, 2.0, 1.0, 2.0], [3.0, 2.0, 5.0, 
6.0]])+(1.-msk_ref)*numpy.array([[3.0, 1.0, 1.0, 6.0], [3.0, 8.0, 1.0, 1.0], [8.0, 7.0, 6.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PCN(self):
      arg=Data(numpy.array([[[2.0, 1.0, 6.0], [4.0, 1.0, 3.0], [8.0, 3.0, 8.0], [5.0, 5.0, 5.0]], [[4.0, 4.0, 2.0], [8.0, 1.0, 
6.0], [3.0, 4.0, 7.0], [8.0, 8.0, 3.0]], [[1.0, 1.0, 6.0], [4.0, 5.0, 3.0], [7.0, 7.0, 8.0], [7.0, 1.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 2.0, 1.0], [4.0, 4.0, 4.0], [6.0, 1.0, 6.0], [3.0, 5.0, 1.0]], [[6.0, 8.0, 7.0], 
[2.0, 2.0, 2.0], [2.0, 7.0, 6.0], [6.0, 4.0, 6.0]], [[4.0, 5.0, 4.0], [4.0, 3.0, 2.0], [7.0, 3.0, 5.0], [5.0, 1.0, 8.0]]]))
      val=numpy.array([[4.0, 5.0, 2.0], [4.0, 4.0, 2.0], [6.0, 7.0, 8.0], [5.0, 2.0, 5.0]])
      arg[1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 6.0], [4.0, 1.0, 3.0], [8.0, 3.0, 8.0], [5.0, 5.0, 5.0]], [[4.0, 5.0, 2.0], [4.0, 
4.0, 2.0], [6.0, 7.0, 8.0], [5.0, 2.0, 5.0]], [[1.0, 1.0, 6.0], [4.0, 5.0, 3.0], [7.0, 7.0, 8.0], [7.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 1.0], [4.0, 4.0, 4.0], [6.0, 1.0, 6.0], [3.0, 5.0, 1.0]], [[4.0, 5.0, 2.0], [4.0, 
4.0, 2.0], [6.0, 7.0, 8.0], [5.0, 2.0, 5.0]], [[4.0, 5.0, 4.0], [4.0, 3.0, 2.0], [7.0, 3.0, 5.0], [5.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PCN(self):
      arg=Data(numpy.array([[[3.0, 3.0, 7.0], [8.0, 4.0, 4.0], [5.0, 4.0, 6.0], [7.0, 6.0, 2.0]], [[2.0, 4.0, 2.0], [8.0, 7.0, 
4.0], [7.0, 3.0, 6.0], [7.0, 2.0, 3.0]], [[1.0, 5.0, 1.0], [3.0, 8.0, 7.0], [7.0, 2.0, 8.0], [7.0, 4.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 8.0, 4.0], [3.0, 5.0, 3.0], [3.0, 8.0, 5.0], [5.0, 6.0, 1.0]], [[3.0, 8.0, 8.0], 
[2.0, 8.0, 6.0], [2.0, 5.0, 1.0], [5.0, 3.0, 8.0]], [[3.0, 1.0, 1.0], [4.0, 4.0, 8.0], [8.0, 6.0, 7.0], [3.0, 5.0, 4.0]]]))
      val=Data(numpy.array([[6.0, 1.0, 8.0], [3.0, 6.0, 7.0], [3.0, 4.0, 4.0], [1.0, 2.0, 6.0]]),self.functionspace)
      arg[1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 7.0], [8.0, 4.0, 4.0], [5.0, 4.0, 6.0], [7.0, 6.0, 2.0]], [[6.0, 1.0, 8.0], [3.0, 
6.0, 7.0], [3.0, 4.0, 4.0], [1.0, 2.0, 6.0]], [[1.0, 5.0, 1.0], [3.0, 8.0, 7.0], [7.0, 2.0, 8.0], [7.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 4.0], [3.0, 5.0, 3.0], [3.0, 8.0, 5.0], [5.0, 6.0, 1.0]], [[6.0, 1.0, 8.0], [3.0, 
6.0, 7.0], [3.0, 4.0, 4.0], [1.0, 2.0, 6.0]], [[3.0, 1.0, 1.0], [4.0, 4.0, 8.0], [8.0, 6.0, 7.0], [3.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PCN(self):
      arg=Data(numpy.array([[[2.0, 8.0, 4.0], [6.0, 8.0, 7.0], [8.0, 5.0, 8.0], [1.0, 5.0, 7.0]], [[3.0, 3.0, 4.0], [8.0, 1.0, 
5.0], [5.0, 3.0, 3.0], [6.0, 5.0, 7.0]], [[8.0, 1.0, 4.0], [5.0, 8.0, 6.0], [1.0, 3.0, 2.0], [5.0, 8.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 3.0, 2.0], [6.0, 2.0, 3.0], [4.0, 2.0, 6.0], [6.0, 7.0, 2.0]], [[3.0, 7.0, 7.0], 
[4.0, 7.0, 7.0], [6.0, 3.0, 1.0], [3.0, 3.0, 7.0]], [[6.0, 2.0, 4.0], [4.0, 1.0, 4.0], [2.0, 6.0, 5.0], [8.0, 6.0, 7.0]]]))
      val=Data(numpy.array([[6.0, 2.0, 1.0], [5.0, 2.0, 4.0], [7.0, 5.0, 5.0], [2.0, 4.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 5.0, 1.0], [2.0, 8.0, 1.0], [5.0, 2.0, 5.0], [5.0, 6.0, 6.0]]))
      arg[1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 8.0, 4.0], [6.0, 8.0, 7.0], [8.0, 5.0, 8.0], [1.0, 5.0, 7.0]], [[6.0, 2.0, 1.0], [5.0, 
2.0, 4.0], [7.0, 5.0, 5.0], [2.0, 4.0, 8.0]], [[8.0, 1.0, 4.0], [5.0, 8.0, 6.0], [1.0, 3.0, 2.0], [5.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 2.0], [6.0, 2.0, 3.0], [4.0, 2.0, 6.0], [6.0, 7.0, 2.0]], [[1.0, 5.0, 1.0], [2.0, 
8.0, 1.0], [5.0, 2.0, 5.0], [5.0, 6.0, 6.0]], [[6.0, 2.0, 4.0], [4.0, 1.0, 4.0], [2.0, 6.0, 5.0], [8.0, 6.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PCN(self):
      arg=Data(numpy.array([[[3.0, 7.0, 5.0], [6.0, 2.0, 4.0], [5.0, 5.0, 6.0], [7.0, 2.0, 3.0]], [[5.0, 3.0, 6.0], [6.0, 5.0, 
7.0], [5.0, 2.0, 5.0], [6.0, 8.0, 5.0]], [[2.0, 4.0, 8.0], [7.0, 2.0, 2.0], [7.0, 5.0, 1.0], [7.0, 6.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 6.0, 1.0], [5.0, 8.0, 2.0], [1.0, 7.0, 7.0], [3.0, 5.0, 5.0]], [[8.0, 7.0, 6.0], 
[5.0, 5.0, 2.0], [7.0, 5.0, 6.0], [6.0, 1.0, 4.0]], [[5.0, 3.0, 4.0], [6.0, 7.0, 8.0], [7.0, 2.0, 5.0], [6.0, 4.0, 6.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[5.0, 4.0, 5.0], [7.0, 2.0, 8.0], [1.0, 2.0, 1.0], [8.0, 5.0, 
5.0]])+(1.-msk_val)*numpy.array([[2.0, 6.0, 5.0], [4.0, 7.0, 2.0], [3.0, 2.0, 6.0], [6.0, 5.0, 8.0]])
      arg[1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 5.0], [6.0, 2.0, 4.0], [5.0, 5.0, 6.0], [7.0, 2.0, 3.0]], [[5.0, 4.0, 5.0], [7.0, 
2.0, 8.0], [1.0, 2.0, 1.0], [8.0, 5.0, 5.0]], [[2.0, 4.0, 8.0], [7.0, 2.0, 2.0], [7.0, 5.0, 1.0], [7.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 1.0], [5.0, 8.0, 2.0], [1.0, 7.0, 7.0], [3.0, 5.0, 5.0]], [[2.0, 6.0, 5.0], [4.0, 
7.0, 2.0], [3.0, 2.0, 6.0], [6.0, 5.0, 8.0]], [[5.0, 3.0, 4.0], [6.0, 7.0, 8.0], [7.0, 2.0, 5.0], [6.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UCL(self):
      arg=Data(numpy.array([[[8.0, 7.0, 5.0], [8.0, 2.0, 3.0], [6.0, 4.0, 8.0], [3.0, 2.0, 7.0]], [[1.0, 4.0, 5.0], [7.0, 6.0, 
6.0], [1.0, 3.0, 6.0], [1.0, 5.0, 1.0]], [[6.0, 7.0, 1.0], [7.0, 2.0, 1.0], [3.0, 2.0, 6.0], [6.0, 7.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 6.0, 5.0], [1.0, 6.0, 5.0], [6.0, 8.0, 1.0], [7.0, 5.0, 8.0]], [[5.0, 1.0, 3.0], 
[7.0, 2.0, 5.0], [5.0, 6.0, 7.0], [2.0, 3.0, 3.0]], [[5.0, 3.0, 5.0], [8.0, 5.0, 7.0], [2.0, 3.0, 4.0], [4.0, 3.0, 1.0]]]))
      val=numpy.array([[[6.0], [4.0], [8.0], [3.0]], [[1.0], [4.0], [2.0], [8.0]]])
      arg[:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 7.0, 6.0], [8.0, 2.0, 4.0], [6.0, 4.0, 8.0], [3.0, 2.0, 3.0]], [[1.0, 4.0, 1.0], [7.0, 
6.0, 4.0], [1.0, 3.0, 2.0], [1.0, 5.0, 8.0]], [[6.0, 7.0, 1.0], [7.0, 2.0, 1.0], [3.0, 2.0, 6.0], [6.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 6.0], [1.0, 6.0, 4.0], [6.0, 8.0, 8.0], [7.0, 5.0, 3.0]], [[5.0, 1.0, 1.0], [7.0, 
2.0, 4.0], [5.0, 6.0, 2.0], [2.0, 3.0, 8.0]], [[5.0, 3.0, 5.0], [8.0, 5.0, 7.0], [2.0, 3.0, 4.0], [4.0, 3.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UCL(self):
      arg=Data(numpy.array([[[8.0, 4.0, 6.0], [5.0, 8.0, 6.0], [4.0, 1.0, 7.0], [2.0, 8.0, 7.0]], [[1.0, 8.0, 8.0], [5.0, 3.0, 
1.0], [2.0, 6.0, 1.0], [4.0, 2.0, 6.0]], [[8.0, 4.0, 8.0], [7.0, 2.0, 2.0], [6.0, 3.0, 2.0], [5.0, 8.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 2.0, 2.0], [2.0, 1.0, 6.0], [6.0, 8.0, 1.0], [6.0, 1.0, 1.0]], [[2.0, 5.0, 1.0], 
[5.0, 6.0, 6.0], [8.0, 5.0, 6.0], [2.0, 2.0, 4.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 8.0], [4.0, 7.0, 7.0], [5.0, 8.0, 8.0]]]))
      val=Data(numpy.array([[[7.0], [6.0], [7.0], [4.0]], [[4.0], [7.0], [7.0], [3.0]]]),self.functionspace)
      arg[:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 7.0], [5.0, 8.0, 6.0], [4.0, 1.0, 7.0], [2.0, 8.0, 4.0]], [[1.0, 8.0, 4.0], [5.0, 
3.0, 7.0], [2.0, 6.0, 7.0], [4.0, 2.0, 3.0]], [[8.0, 4.0, 8.0], [7.0, 2.0, 2.0], [6.0, 3.0, 2.0], [5.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 7.0], [2.0, 1.0, 6.0], [6.0, 8.0, 7.0], [6.0, 1.0, 4.0]], [[2.0, 5.0, 4.0], [5.0, 
6.0, 7.0], [8.0, 5.0, 7.0], [2.0, 2.0, 3.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 8.0], [4.0, 7.0, 7.0], [5.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UCL(self):
      arg=Data(numpy.array([[[3.0, 4.0, 5.0], [6.0, 5.0, 5.0], [3.0, 5.0, 5.0], [1.0, 1.0, 2.0]], [[3.0, 3.0, 4.0], [7.0, 8.0, 
7.0], [5.0, 3.0, 3.0], [2.0, 1.0, 5.0]], [[5.0, 8.0, 4.0], [8.0, 1.0, 8.0], [7.0, 3.0, 1.0], [2.0, 3.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 6.0, 4.0], [4.0, 2.0, 3.0], [8.0, 8.0, 1.0], [7.0, 2.0, 1.0]], [[1.0, 2.0, 2.0], 
[7.0, 4.0, 4.0], [7.0, 3.0, 6.0], [2.0, 5.0, 5.0]], [[7.0, 3.0, 7.0], [1.0, 2.0, 4.0], [3.0, 6.0, 7.0], [1.0, 1.0, 8.0]]]))
      val=Data(numpy.array([[[5.0], [1.0], [8.0], [2.0]], [[3.0], [1.0], [7.0], [4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0], [3.0], [8.0], [3.0]], [[3.0], [1.0], [1.0], [5.0]]]))
      arg[:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 5.0], [6.0, 5.0, 1.0], [3.0, 5.0, 8.0], [1.0, 1.0, 2.0]], [[3.0, 3.0, 3.0], [7.0, 
8.0, 1.0], [5.0, 3.0, 7.0], [2.0, 1.0, 4.0]], [[5.0, 8.0, 4.0], [8.0, 1.0, 8.0], [7.0, 3.0, 1.0], [2.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 4.0], [4.0, 2.0, 3.0], [8.0, 8.0, 8.0], [7.0, 2.0, 3.0]], [[1.0, 2.0, 3.0], [7.0, 
4.0, 1.0], [7.0, 3.0, 1.0], [2.0, 5.0, 5.0]], [[7.0, 3.0, 7.0], [1.0, 2.0, 4.0], [3.0, 6.0, 7.0], [1.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UCL(self):
      arg=Data(numpy.array([[[4.0, 3.0, 2.0], [2.0, 8.0, 3.0], [6.0, 8.0, 1.0], [6.0, 6.0, 2.0]], [[2.0, 7.0, 3.0], [6.0, 4.0, 
8.0], [4.0, 6.0, 5.0], [3.0, 7.0, 4.0]], [[7.0, 4.0, 4.0], [7.0, 8.0, 5.0], [7.0, 7.0, 3.0], [8.0, 2.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 4.0, 1.0], [6.0, 1.0, 5.0], [5.0, 8.0, 6.0], [5.0, 2.0, 2.0]], [[3.0, 8.0, 1.0], 
[8.0, 7.0, 8.0], [6.0, 8.0, 5.0], [7.0, 1.0, 3.0]], [[3.0, 7.0, 2.0], [4.0, 6.0, 4.0], [5.0, 7.0, 8.0], [8.0, 5.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0], [6.0], [5.0], [2.0]], [[1.0], [8.0], [3.0], [5.0]]])+(1.-msk_val)*numpy.array([[[2.0], 
[7.0], [8.0], [3.0]], [[2.0], [8.0], [7.0], [3.0]]])
      arg[:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 1.0], [2.0, 8.0, 6.0], [6.0, 8.0, 5.0], [6.0, 6.0, 2.0]], [[2.0, 7.0, 1.0], [6.0, 
4.0, 8.0], [4.0, 6.0, 3.0], [3.0, 7.0, 5.0]], [[7.0, 4.0, 4.0], [7.0, 8.0, 5.0], [7.0, 7.0, 3.0], [8.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 4.0, 2.0], [6.0, 1.0, 7.0], [5.0, 8.0, 8.0], [5.0, 2.0, 3.0]], [[3.0, 8.0, 2.0], [8.0, 
7.0, 8.0], [6.0, 8.0, 7.0], [7.0, 1.0, 3.0]], [[3.0, 7.0, 2.0], [4.0, 6.0, 4.0], [5.0, 7.0, 8.0], [8.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LCL(self):
      arg=Data(numpy.array([[[8.0, 6.0, 8.0], [3.0, 1.0, 4.0], [4.0, 1.0, 6.0], [5.0, 1.0, 6.0]], [[1.0, 6.0, 3.0], [7.0, 5.0, 
6.0], [6.0, 1.0, 6.0], [2.0, 4.0, 7.0]], [[6.0, 7.0, 6.0], [6.0, 3.0, 3.0], [7.0, 8.0, 7.0], [4.0, 8.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 5.0], [7.0, 3.0, 2.0], [8.0, 5.0, 5.0], [2.0, 3.0, 7.0]], [[5.0, 1.0, 8.0], 
[8.0, 5.0, 7.0], [4.0, 8.0, 3.0], [4.0, 1.0, 3.0]], [[8.0, 8.0, 8.0], [4.0, 4.0, 3.0], [6.0, 8.0, 8.0], [2.0, 7.0, 1.0]]]))
      val=numpy.array([[[7.0], [8.0], [5.0], [4.0]]])
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 8.0], [3.0, 1.0, 4.0], [4.0, 1.0, 6.0], [5.0, 1.0, 6.0]], [[1.0, 6.0, 3.0], [7.0, 
5.0, 6.0], [6.0, 1.0, 6.0], [2.0, 4.0, 7.0]], [[6.0, 7.0, 7.0], [6.0, 3.0, 8.0], [7.0, 8.0, 5.0], [4.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 5.0], [7.0, 3.0, 2.0], [8.0, 5.0, 5.0], [2.0, 3.0, 7.0]], [[5.0, 1.0, 8.0], [8.0, 
5.0, 7.0], [4.0, 8.0, 3.0], [4.0, 1.0, 3.0]], [[8.0, 8.0, 7.0], [4.0, 4.0, 8.0], [6.0, 8.0, 5.0], [2.0, 7.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LCL(self):
      arg=Data(numpy.array([[[1.0, 7.0, 5.0], [6.0, 8.0, 1.0], [4.0, 8.0, 5.0], [4.0, 1.0, 1.0]], [[1.0, 8.0, 7.0], [4.0, 2.0, 
7.0], [7.0, 4.0, 6.0], [8.0, 3.0, 6.0]], [[4.0, 4.0, 4.0], [8.0, 5.0, 3.0], [3.0, 1.0, 4.0], [3.0, 6.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 4.0, 7.0], [2.0, 5.0, 7.0], [1.0, 3.0, 5.0], [3.0, 1.0, 5.0]], [[3.0, 8.0, 1.0], 
[3.0, 2.0, 6.0], [5.0, 4.0, 1.0], [1.0, 5.0, 1.0]], [[7.0, 1.0, 8.0], [4.0, 4.0, 2.0], [7.0, 4.0, 5.0], [2.0, 3.0, 8.0]]]))
      val=Data(numpy.array([[[5.0], [4.0], [5.0], [4.0]]]),self.functionspace)
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 5.0], [6.0, 8.0, 1.0], [4.0, 8.0, 5.0], [4.0, 1.0, 1.0]], [[1.0, 8.0, 7.0], [4.0, 
2.0, 7.0], [7.0, 4.0, 6.0], [8.0, 3.0, 6.0]], [[4.0, 4.0, 5.0], [8.0, 5.0, 4.0], [3.0, 1.0, 5.0], [3.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 7.0], [2.0, 5.0, 7.0], [1.0, 3.0, 5.0], [3.0, 1.0, 5.0]], [[3.0, 8.0, 1.0], [3.0, 
2.0, 6.0], [5.0, 4.0, 1.0], [1.0, 5.0, 1.0]], [[7.0, 1.0, 5.0], [4.0, 4.0, 4.0], [7.0, 4.0, 5.0], [2.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LCL(self):
      arg=Data(numpy.array([[[5.0, 4.0, 2.0], [5.0, 5.0, 3.0], [1.0, 1.0, 3.0], [8.0, 7.0, 3.0]], [[6.0, 3.0, 7.0], [6.0, 7.0, 
3.0], [5.0, 2.0, 8.0], [4.0, 4.0, 8.0]], [[7.0, 7.0, 1.0], [7.0, 3.0, 2.0], [6.0, 2.0, 1.0], [3.0, 4.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 6.0, 6.0], [6.0, 8.0, 7.0], [5.0, 2.0, 5.0], [2.0, 7.0, 8.0]], [[1.0, 2.0, 8.0], 
[2.0, 4.0, 1.0], [6.0, 8.0, 4.0], [4.0, 4.0, 2.0]], [[6.0, 6.0, 4.0], [4.0, 1.0, 7.0], [5.0, 4.0, 7.0], [5.0, 5.0, 3.0]]]))
      val=Data(numpy.array([[[8.0], [7.0], [1.0], [1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0], [2.0], [7.0], [5.0]]]))
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 4.0, 2.0], [5.0, 5.0, 3.0], [1.0, 1.0, 3.0], [8.0, 7.0, 3.0]], [[6.0, 3.0, 7.0], [6.0, 
7.0, 3.0], [5.0, 2.0, 8.0], [4.0, 4.0, 8.0]], [[7.0, 7.0, 8.0], [7.0, 3.0, 7.0], [6.0, 2.0, 1.0], [3.0, 4.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 6.0], [6.0, 8.0, 7.0], [5.0, 2.0, 5.0], [2.0, 7.0, 8.0]], [[1.0, 2.0, 8.0], [2.0, 
4.0, 1.0], [6.0, 8.0, 4.0], [4.0, 4.0, 2.0]], [[6.0, 6.0, 8.0], [4.0, 1.0, 2.0], [5.0, 4.0, 7.0], [5.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LCL(self):
      arg=Data(numpy.array([[[1.0, 1.0, 5.0], [6.0, 1.0, 6.0], [8.0, 3.0, 1.0], [8.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [2.0, 4.0, 
4.0], [7.0, 3.0, 7.0], [3.0, 7.0, 3.0]], [[3.0, 3.0, 3.0], [4.0, 5.0, 6.0], [2.0, 6.0, 7.0], [3.0, 8.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 3.0, 4.0], [4.0, 6.0, 1.0], [5.0, 7.0, 4.0], [4.0, 1.0, 3.0]], [[3.0, 3.0, 7.0], 
[7.0, 1.0, 8.0], [3.0, 6.0, 2.0], [4.0, 7.0, 3.0]], [[7.0, 5.0, 3.0], [5.0, 5.0, 5.0], [8.0, 1.0, 5.0], [4.0, 7.0, 7.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0], [2.0], [6.0], [2.0]]])+(1.-msk_val)*numpy.array([[[7.0], [3.0], [4.0], [6.0]]])
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 5.0], [6.0, 1.0, 6.0], [8.0, 3.0, 1.0], [8.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [2.0, 
4.0, 4.0], [7.0, 3.0, 7.0], [3.0, 7.0, 3.0]], [[3.0, 3.0, 8.0], [4.0, 5.0, 2.0], [2.0, 6.0, 6.0], [3.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 4.0], [4.0, 6.0, 1.0], [5.0, 7.0, 4.0], [4.0, 1.0, 3.0]], [[3.0, 3.0, 7.0], [7.0, 
1.0, 8.0], [3.0, 6.0, 2.0], [4.0, 7.0, 3.0]], [[7.0, 5.0, 7.0], [5.0, 5.0, 3.0], [8.0, 1.0, 4.0], [4.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LUN(self):
      arg=Data(numpy.array([[[2.0, 5.0, 7.0], [6.0, 1.0, 8.0], [4.0, 7.0, 4.0], [1.0, 7.0, 2.0]], [[3.0, 4.0, 5.0], [6.0, 1.0, 
5.0], [8.0, 1.0, 5.0], [2.0, 7.0, 2.0]], [[4.0, 2.0, 1.0], [4.0, 5.0, 4.0], [6.0, 3.0, 8.0], [6.0, 5.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 8.0, 7.0], [6.0, 7.0, 3.0], [6.0, 7.0, 7.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 6.0], 
[4.0, 7.0, 3.0], [4.0, 3.0, 3.0], [2.0, 6.0, 3.0]], [[5.0, 5.0, 1.0], [2.0, 1.0, 6.0], [2.0, 3.0, 2.0], [2.0, 6.0, 2.0]]]))
      val=numpy.array([[[1.0, 3.0, 4.0], [2.0, 5.0, 4.0], [8.0, 7.0, 2.0]]])
      arg[2:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 7.0], [6.0, 1.0, 8.0], [4.0, 7.0, 4.0], [1.0, 7.0, 2.0]], [[3.0, 4.0, 5.0], [6.0, 
1.0, 5.0], [8.0, 1.0, 5.0], [2.0, 7.0, 2.0]], [[1.0, 3.0, 4.0], [2.0, 5.0, 4.0], [8.0, 7.0, 2.0], [6.0, 5.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 7.0], [6.0, 7.0, 3.0], [6.0, 7.0, 7.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 6.0], [4.0, 
7.0, 3.0], [4.0, 3.0, 3.0], [2.0, 6.0, 3.0]], [[1.0, 3.0, 4.0], [2.0, 5.0, 4.0], [8.0, 7.0, 2.0], [2.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LUN(self):
      arg=Data(numpy.array([[[4.0, 3.0, 6.0], [8.0, 4.0, 5.0], [4.0, 3.0, 5.0], [8.0, 8.0, 2.0]], [[8.0, 5.0, 8.0], [7.0, 2.0, 
1.0], [2.0, 7.0, 3.0], [6.0, 1.0, 5.0]], [[7.0, 8.0, 2.0], [7.0, 3.0, 7.0], [2.0, 7.0, 4.0], [8.0, 6.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 6.0, 7.0], [1.0, 4.0, 8.0], [3.0, 8.0, 4.0], [2.0, 5.0, 6.0]], [[4.0, 3.0, 3.0], 
[6.0, 4.0, 2.0], [4.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[4.0, 3.0, 6.0], [2.0, 3.0, 8.0], [8.0, 5.0, 3.0], [8.0, 3.0, 3.0]]]))
      val=Data(numpy.array([[[3.0, 3.0, 3.0], [7.0, 3.0, 5.0], [2.0, 1.0, 5.0]]]),self.functionspace)
      arg[2:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 6.0], [8.0, 4.0, 5.0], [4.0, 3.0, 5.0], [8.0, 8.0, 2.0]], [[8.0, 5.0, 8.0], [7.0, 
2.0, 1.0], [2.0, 7.0, 3.0], [6.0, 1.0, 5.0]], [[3.0, 3.0, 3.0], [7.0, 3.0, 5.0], [2.0, 1.0, 5.0], [8.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 7.0], [1.0, 4.0, 8.0], [3.0, 8.0, 4.0], [2.0, 5.0, 6.0]], [[4.0, 3.0, 3.0], [6.0, 
4.0, 2.0], [4.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[3.0, 3.0, 3.0], [7.0, 3.0, 5.0], [2.0, 1.0, 5.0], [8.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LUN(self):
      arg=Data(numpy.array([[[3.0, 7.0, 4.0], [8.0, 4.0, 3.0], [5.0, 2.0, 8.0], [4.0, 4.0, 4.0]], [[1.0, 2.0, 4.0], [1.0, 2.0, 
4.0], [1.0, 3.0, 1.0], [3.0, 7.0, 6.0]], [[7.0, 7.0, 2.0], [7.0, 6.0, 6.0], [4.0, 4.0, 3.0], [4.0, 4.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 3.0, 5.0], [7.0, 8.0, 7.0], [2.0, 5.0, 6.0], [5.0, 2.0, 3.0]], [[2.0, 8.0, 6.0], 
[3.0, 6.0, 7.0], [8.0, 5.0, 5.0], [4.0, 5.0, 6.0]], [[1.0, 7.0, 5.0], [4.0, 5.0, 8.0], [5.0, 4.0, 5.0], [4.0, 2.0, 8.0]]]))
      val=Data(numpy.array([[[8.0, 6.0, 2.0], [2.0, 5.0, 3.0], [5.0, 2.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 4.0, 8.0], [8.0, 3.0, 6.0], [1.0, 3.0, 1.0]]]))
      arg[2:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 4.0], [8.0, 4.0, 3.0], [5.0, 2.0, 8.0], [4.0, 4.0, 4.0]], [[1.0, 2.0, 4.0], [1.0, 
2.0, 4.0], [1.0, 3.0, 1.0], [3.0, 7.0, 6.0]], [[8.0, 6.0, 2.0], [2.0, 5.0, 3.0], [5.0, 2.0, 4.0], [4.0, 4.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 5.0], [7.0, 8.0, 7.0], [2.0, 5.0, 6.0], [5.0, 2.0, 3.0]], [[2.0, 8.0, 6.0], [3.0, 
6.0, 7.0], [8.0, 5.0, 5.0], [4.0, 5.0, 6.0]], [[3.0, 4.0, 8.0], [8.0, 3.0, 6.0], [1.0, 3.0, 1.0], [4.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LUN(self):
      arg=Data(numpy.array([[[8.0, 6.0, 3.0], [8.0, 3.0, 5.0], [4.0, 3.0, 2.0], [8.0, 6.0, 8.0]], [[1.0, 4.0, 1.0], [2.0, 3.0, 
3.0], [3.0, 5.0, 6.0], [3.0, 3.0, 5.0]], [[1.0, 4.0, 7.0], [6.0, 1.0, 7.0], [7.0, 1.0, 5.0], [5.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 2.0, 2.0], [6.0, 5.0, 8.0], [4.0, 7.0, 7.0], [8.0, 2.0, 2.0]], [[7.0, 5.0, 4.0], 
[5.0, 2.0, 8.0], [3.0, 2.0, 8.0], [4.0, 2.0, 6.0]], [[6.0, 7.0, 3.0], [3.0, 3.0, 6.0], [6.0, 3.0, 1.0], [3.0, 8.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 6.0, 5.0], [5.0, 1.0, 8.0], [1.0, 3.0, 5.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 
7.0], [5.0, 7.0, 3.0], [3.0, 4.0, 3.0]]])
      arg[2:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 3.0], [8.0, 3.0, 5.0], [4.0, 3.0, 2.0], [8.0, 6.0, 8.0]], [[1.0, 4.0, 1.0], [2.0, 
3.0, 3.0], [3.0, 5.0, 6.0], [3.0, 3.0, 5.0]], [[5.0, 6.0, 5.0], [5.0, 1.0, 8.0], [1.0, 3.0, 5.0], [5.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 2.0], [6.0, 5.0, 8.0], [4.0, 7.0, 7.0], [8.0, 2.0, 2.0]], [[7.0, 5.0, 4.0], [5.0, 
2.0, 8.0], [3.0, 2.0, 8.0], [4.0, 2.0, 6.0]], [[4.0, 6.0, 7.0], [5.0, 7.0, 3.0], [3.0, 4.0, 3.0], [3.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UUR(self):
      arg=Data(numpy.array([[[6.0, 4.0, 4.0], [1.0, 8.0, 4.0], [3.0, 7.0, 3.0], [8.0, 3.0, 6.0]], [[4.0, 5.0, 1.0], [6.0, 1.0, 
6.0], [2.0, 2.0, 7.0], [2.0, 1.0, 4.0]], [[2.0, 3.0, 3.0], [8.0, 6.0, 2.0], [6.0, 4.0, 4.0], [6.0, 7.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 5.0, 4.0], [4.0, 5.0, 4.0], [8.0, 5.0, 7.0], [2.0, 7.0, 5.0]], [[3.0, 5.0, 3.0], 
[8.0, 4.0, 3.0], [5.0, 7.0, 6.0], [1.0, 3.0, 8.0]], [[8.0, 5.0, 7.0], [2.0, 3.0, 4.0], [6.0, 6.0, 5.0], [4.0, 4.0, 7.0]]]))
      val=numpy.array([[[5.0], [1.0], [5.0]], [[7.0], [1.0], [4.0]]])
      arg[:2,:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 4.0], [1.0, 1.0, 4.0], [3.0, 5.0, 3.0], [8.0, 3.0, 6.0]], [[4.0, 7.0, 1.0], [6.0, 
1.0, 6.0], [2.0, 4.0, 7.0], [2.0, 1.0, 4.0]], [[2.0, 3.0, 3.0], [8.0, 6.0, 2.0], [6.0, 4.0, 4.0], [6.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 4.0], [4.0, 1.0, 4.0], [8.0, 5.0, 7.0], [2.0, 7.0, 5.0]], [[3.0, 7.0, 3.0], [8.0, 
1.0, 3.0], [5.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[8.0, 5.0, 7.0], [2.0, 3.0, 4.0], [6.0, 6.0, 5.0], [4.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UUR(self):
      arg=Data(numpy.array([[[8.0, 4.0, 5.0], [2.0, 2.0, 4.0], [3.0, 2.0, 7.0], [5.0, 6.0, 3.0]], [[4.0, 6.0, 5.0], [4.0, 1.0, 
2.0], [6.0, 2.0, 1.0], [2.0, 2.0, 7.0]], [[6.0, 1.0, 2.0], [7.0, 6.0, 1.0], [8.0, 4.0, 5.0], [6.0, 4.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 8.0, 1.0], [7.0, 1.0, 1.0], [2.0, 1.0, 7.0], [1.0, 1.0, 4.0]], [[7.0, 6.0, 5.0], 
[6.0, 1.0, 8.0], [2.0, 8.0, 4.0], [3.0, 1.0, 6.0]], [[2.0, 5.0, 4.0], [5.0, 4.0, 7.0], [4.0, 6.0, 7.0], [6.0, 4.0, 2.0]]]))
      val=Data(numpy.array([[[6.0], [8.0], [4.0]], [[5.0], [5.0], [2.0]]]),self.functionspace)
      arg[:2,:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 5.0], [2.0, 8.0, 4.0], [3.0, 4.0, 7.0], [5.0, 6.0, 3.0]], [[4.0, 5.0, 5.0], [4.0, 
5.0, 2.0], [6.0, 2.0, 1.0], [2.0, 2.0, 7.0]], [[6.0, 1.0, 2.0], [7.0, 6.0, 1.0], [8.0, 4.0, 5.0], [6.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 1.0], [7.0, 8.0, 1.0], [2.0, 4.0, 7.0], [1.0, 1.0, 4.0]], [[7.0, 5.0, 5.0], [6.0, 
5.0, 8.0], [2.0, 2.0, 4.0], [3.0, 1.0, 6.0]], [[2.0, 5.0, 4.0], [5.0, 4.0, 7.0], [4.0, 6.0, 7.0], [6.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UUR(self):
      arg=Data(numpy.array([[[2.0, 2.0, 4.0], [3.0, 1.0, 1.0], [2.0, 4.0, 1.0], [1.0, 3.0, 6.0]], [[7.0, 8.0, 7.0], [7.0, 1.0, 
8.0], [6.0, 7.0, 8.0], [4.0, 3.0, 6.0]], [[3.0, 7.0, 3.0], [1.0, 6.0, 1.0], [6.0, 5.0, 3.0], [4.0, 7.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 4.0, 1.0], [6.0, 6.0, 5.0], [8.0, 4.0, 4.0], [3.0, 4.0, 3.0]], [[2.0, 8.0, 4.0], 
[2.0, 2.0, 6.0], [3.0, 8.0, 8.0], [6.0, 7.0, 7.0]], [[3.0, 2.0, 1.0], [1.0, 6.0, 2.0], [1.0, 4.0, 5.0], [7.0, 2.0, 4.0]]]))
      val=Data(numpy.array([[[2.0], [5.0], [6.0]], [[1.0], [6.0], [2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0], [5.0], [3.0]], [[6.0], [6.0], [7.0]]]))
      arg[:2,:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 4.0], [3.0, 5.0, 1.0], [2.0, 6.0, 1.0], [1.0, 3.0, 6.0]], [[7.0, 1.0, 7.0], [7.0, 
6.0, 8.0], [6.0, 2.0, 8.0], [4.0, 3.0, 6.0]], [[3.0, 7.0, 3.0], [1.0, 6.0, 1.0], [6.0, 5.0, 3.0], [4.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 1.0], [6.0, 5.0, 5.0], [8.0, 3.0, 4.0], [3.0, 4.0, 3.0]], [[2.0, 6.0, 4.0], [2.0, 
6.0, 6.0], [3.0, 7.0, 8.0], [6.0, 7.0, 7.0]], [[3.0, 2.0, 1.0], [1.0, 6.0, 2.0], [1.0, 4.0, 5.0], [7.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UUR(self):
      arg=Data(numpy.array([[[4.0, 8.0, 2.0], [7.0, 6.0, 7.0], [4.0, 8.0, 6.0], [6.0, 6.0, 4.0]], [[3.0, 8.0, 7.0], [4.0, 5.0, 
1.0], [6.0, 5.0, 1.0], [7.0, 7.0, 2.0]], [[8.0, 5.0, 4.0], [2.0, 8.0, 5.0], [1.0, 1.0, 6.0], [6.0, 7.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 4.0, 2.0], [8.0, 1.0, 1.0], [7.0, 3.0, 1.0], [3.0, 5.0, 5.0]], [[7.0, 2.0, 8.0], 
[4.0, 5.0, 3.0], [3.0, 6.0, 5.0], [3.0, 3.0, 4.0]], [[2.0, 6.0, 8.0], [3.0, 3.0, 1.0], [6.0, 8.0, 8.0], [2.0, 8.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0], [7.0], [3.0]], [[6.0], [4.0], [8.0]]])+(1.-msk_val)*numpy.array([[[5.0], [6.0], [6.0]], 
[[4.0], [1.0], [6.0]]])
      arg[:2,:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 2.0], [7.0, 7.0, 7.0], [4.0, 3.0, 6.0], [6.0, 6.0, 4.0]], [[3.0, 6.0, 7.0], [4.0, 
4.0, 1.0], [6.0, 8.0, 1.0], [7.0, 7.0, 2.0]], [[8.0, 5.0, 4.0], [2.0, 8.0, 5.0], [1.0, 1.0, 6.0], [6.0, 7.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 2.0], [8.0, 6.0, 1.0], [7.0, 6.0, 1.0], [3.0, 5.0, 5.0]], [[7.0, 4.0, 8.0], [4.0, 
1.0, 3.0], [3.0, 6.0, 5.0], [3.0, 3.0, 4.0]], [[2.0, 6.0, 8.0], [3.0, 3.0, 1.0], [6.0, 8.0, 8.0], [2.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PRC(self):
      arg=Data(numpy.array([[[8.0, 5.0, 4.0], [4.0, 3.0, 7.0], [5.0, 8.0, 8.0], [5.0, 5.0, 8.0]], [[6.0, 8.0, 8.0], [4.0, 7.0, 
8.0], [4.0, 3.0, 6.0], [3.0, 2.0, 4.0]], [[3.0, 3.0, 8.0], [1.0, 8.0, 2.0], [2.0, 1.0, 4.0], [4.0, 6.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 4.0, 7.0], [5.0, 2.0, 3.0], [6.0, 6.0, 2.0], [1.0, 3.0, 8.0]], [[2.0, 4.0, 5.0], 
[1.0, 3.0, 8.0], [4.0, 3.0, 3.0], [5.0, 4.0, 4.0]], [[4.0, 5.0, 6.0], [8.0, 3.0, 7.0], [5.0, 1.0, 6.0], [2.0, 4.0, 4.0]]]))
      val=numpy.array([[7.0, 8.0, 8.0], [4.0, 3.0, 8.0]])
      arg[1,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 4.0], [4.0, 3.0, 7.0], [5.0, 8.0, 8.0], [5.0, 5.0, 8.0]], [[6.0, 8.0, 8.0], [7.0, 
8.0, 8.0], [4.0, 3.0, 8.0], [3.0, 2.0, 4.0]], [[3.0, 3.0, 8.0], [1.0, 8.0, 2.0], [2.0, 1.0, 4.0], [4.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 7.0], [5.0, 2.0, 3.0], [6.0, 6.0, 2.0], [1.0, 3.0, 8.0]], [[2.0, 4.0, 5.0], [7.0, 
8.0, 8.0], [4.0, 3.0, 8.0], [5.0, 4.0, 4.0]], [[4.0, 5.0, 6.0], [8.0, 3.0, 7.0], [5.0, 1.0, 6.0], [2.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PRC(self):
      arg=Data(numpy.array([[[7.0, 6.0, 5.0], [3.0, 2.0, 3.0], [1.0, 4.0, 8.0], [6.0, 3.0, 6.0]], [[2.0, 1.0, 3.0], [4.0, 5.0, 
8.0], [2.0, 1.0, 4.0], [6.0, 3.0, 6.0]], [[6.0, 4.0, 8.0], [5.0, 5.0, 5.0], [1.0, 5.0, 8.0], [7.0, 1.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 4.0, 8.0], [4.0, 7.0, 8.0], [4.0, 3.0, 8.0], [5.0, 1.0, 6.0]], [[2.0, 6.0, 4.0], 
[8.0, 2.0, 5.0], [4.0, 5.0, 7.0], [5.0, 4.0, 1.0]], [[4.0, 8.0, 7.0], [8.0, 1.0, 3.0], [3.0, 7.0, 8.0], [1.0, 6.0, 5.0]]]))
      val=Data(numpy.array([[2.0, 8.0, 5.0], [2.0, 4.0, 8.0]]),self.functionspace)
      arg[1,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 5.0], [3.0, 2.0, 3.0], [1.0, 4.0, 8.0], [6.0, 3.0, 6.0]], [[2.0, 1.0, 3.0], [2.0, 
8.0, 5.0], [2.0, 4.0, 8.0], [6.0, 3.0, 6.0]], [[6.0, 4.0, 8.0], [5.0, 5.0, 5.0], [1.0, 5.0, 8.0], [7.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 8.0], [4.0, 7.0, 8.0], [4.0, 3.0, 8.0], [5.0, 1.0, 6.0]], [[2.0, 6.0, 4.0], [2.0, 
8.0, 5.0], [2.0, 4.0, 8.0], [5.0, 4.0, 1.0]], [[4.0, 8.0, 7.0], [8.0, 1.0, 3.0], [3.0, 7.0, 8.0], [1.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PRC(self):
      arg=Data(numpy.array([[[1.0, 5.0, 5.0], [8.0, 8.0, 8.0], [1.0, 8.0, 1.0], [7.0, 3.0, 4.0]], [[6.0, 3.0, 4.0], [6.0, 5.0, 
8.0], [8.0, 3.0, 2.0], [5.0, 4.0, 5.0]], [[6.0, 8.0, 3.0], [6.0, 5.0, 8.0], [6.0, 8.0, 1.0], [2.0, 1.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 7.0, 7.0], [3.0, 8.0, 8.0], [7.0, 6.0, 8.0], [3.0, 4.0, 2.0]], [[2.0, 6.0, 2.0], 
[7.0, 4.0, 1.0], [4.0, 4.0, 7.0], [3.0, 5.0, 5.0]], [[6.0, 3.0, 6.0], [5.0, 4.0, 3.0], [2.0, 4.0, 8.0], [7.0, 5.0, 3.0]]]))
      val=Data(numpy.array([[8.0, 1.0, 5.0], [7.0, 1.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 7.0, 1.0], [3.0, 6.0, 8.0]]))
      arg[1,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 5.0], [8.0, 8.0, 8.0], [1.0, 8.0, 1.0], [7.0, 3.0, 4.0]], [[6.0, 3.0, 4.0], [8.0, 
1.0, 5.0], [7.0, 1.0, 8.0], [5.0, 4.0, 5.0]], [[6.0, 8.0, 3.0], [6.0, 5.0, 8.0], [6.0, 8.0, 1.0], [2.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 7.0], [3.0, 8.0, 8.0], [7.0, 6.0, 8.0], [3.0, 4.0, 2.0]], [[2.0, 6.0, 2.0], [1.0, 
7.0, 1.0], [3.0, 6.0, 8.0], [3.0, 5.0, 5.0]], [[6.0, 3.0, 6.0], [5.0, 4.0, 3.0], [2.0, 4.0, 8.0], [7.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PRC(self):
      arg=Data(numpy.array([[[8.0, 2.0, 2.0], [3.0, 7.0, 8.0], [7.0, 3.0, 8.0], [8.0, 3.0, 4.0]], [[5.0, 7.0, 4.0], [4.0, 1.0, 
3.0], [5.0, 2.0, 2.0], [5.0, 4.0, 1.0]], [[6.0, 3.0, 7.0], [8.0, 4.0, 4.0], [2.0, 3.0, 3.0], [1.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 4.0, 4.0], [5.0, 1.0, 4.0], [5.0, 8.0, 3.0], [1.0, 3.0, 3.0]], [[2.0, 7.0, 1.0], 
[6.0, 2.0, 2.0], [2.0, 6.0, 1.0], [1.0, 4.0, 1.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 8.0], [8.0, 1.0, 6.0], [8.0, 7.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[1.0, 6.0, 3.0], [4.0, 5.0, 3.0]])+(1.-msk_val)*numpy.array([[1.0, 4.0, 1.0], [3.0, 4.0, 8.0]])
      arg[1,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 2.0], [3.0, 7.0, 8.0], [7.0, 3.0, 8.0], [8.0, 3.0, 4.0]], [[5.0, 7.0, 4.0], [1.0, 
6.0, 3.0], [4.0, 5.0, 3.0], [5.0, 4.0, 1.0]], [[6.0, 3.0, 7.0], [8.0, 4.0, 4.0], [2.0, 3.0, 3.0], [1.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 4.0], [5.0, 1.0, 4.0], [5.0, 8.0, 3.0], [1.0, 3.0, 3.0]], [[2.0, 7.0, 1.0], [1.0, 
4.0, 1.0], [3.0, 4.0, 8.0], [1.0, 4.0, 1.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 8.0], [8.0, 1.0, 6.0], [8.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UUN(self):
      arg=Data(numpy.array([[[7.0, 7.0, 4.0], [2.0, 5.0, 1.0], [8.0, 5.0, 5.0], [8.0, 2.0, 1.0]], [[4.0, 3.0, 7.0], [5.0, 4.0, 
4.0], [4.0, 6.0, 1.0], [7.0, 7.0, 3.0]], [[2.0, 6.0, 6.0], [6.0, 6.0, 8.0], [3.0, 6.0, 6.0], [4.0, 7.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 6.0, 3.0], [3.0, 1.0, 5.0], [5.0, 3.0, 3.0], [6.0, 3.0, 5.0]], [[3.0, 5.0, 1.0], 
[1.0, 3.0, 5.0], [2.0, 7.0, 7.0], [7.0, 4.0, 8.0]], [[8.0, 8.0, 8.0], [7.0, 4.0, 2.0], [7.0, 7.0, 5.0], [6.0, 2.0, 3.0]]]))
      val=numpy.array([[[5.0, 2.0, 8.0], [7.0, 6.0, 2.0], [1.0, 1.0, 6.0]], [[6.0, 7.0, 7.0], [4.0, 3.0, 4.0], [2.0, 1.0, 
8.0]]])
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 8.0], [7.0, 6.0, 2.0], [1.0, 1.0, 6.0], [8.0, 2.0, 1.0]], [[6.0, 7.0, 7.0], [4.0, 
3.0, 4.0], [2.0, 1.0, 8.0], [7.0, 7.0, 3.0]], [[2.0, 6.0, 6.0], [6.0, 6.0, 8.0], [3.0, 6.0, 6.0], [4.0, 7.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 8.0], [7.0, 6.0, 2.0], [1.0, 1.0, 6.0], [6.0, 3.0, 5.0]], [[6.0, 7.0, 7.0], [4.0, 
3.0, 4.0], [2.0, 1.0, 8.0], [7.0, 4.0, 8.0]], [[8.0, 8.0, 8.0], [7.0, 4.0, 2.0], [7.0, 7.0, 5.0], [6.0, 2.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UUN(self):
      arg=Data(numpy.array([[[8.0, 1.0, 6.0], [1.0, 5.0, 2.0], [1.0, 1.0, 1.0], [3.0, 1.0, 2.0]], [[5.0, 7.0, 7.0], [7.0, 2.0, 
7.0], [7.0, 3.0, 8.0], [4.0, 4.0, 6.0]], [[5.0, 4.0, 8.0], [4.0, 1.0, 6.0], [8.0, 4.0, 5.0], [2.0, 8.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 1.0, 3.0], [7.0, 5.0, 2.0], [8.0, 2.0, 4.0], [1.0, 6.0, 5.0]], [[2.0, 4.0, 5.0], 
[7.0, 5.0, 5.0], [3.0, 7.0, 8.0], [7.0, 8.0, 4.0]], [[3.0, 1.0, 6.0], [4.0, 3.0, 2.0], [2.0, 4.0, 8.0], [7.0, 4.0, 4.0]]]))
      val=Data(numpy.array([[[5.0, 3.0, 6.0], [5.0, 4.0, 8.0], [6.0, 7.0, 7.0]], [[5.0, 6.0, 4.0], [8.0, 1.0, 5.0], [4.0, 4.0, 
8.0]]]),self.functionspace)
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 6.0], [5.0, 4.0, 8.0], [6.0, 7.0, 7.0], [3.0, 1.0, 2.0]], [[5.0, 6.0, 4.0], [8.0, 
1.0, 5.0], [4.0, 4.0, 8.0], [4.0, 4.0, 6.0]], [[5.0, 4.0, 8.0], [4.0, 1.0, 6.0], [8.0, 4.0, 5.0], [2.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 6.0], [5.0, 4.0, 8.0], [6.0, 7.0, 7.0], [1.0, 6.0, 5.0]], [[5.0, 6.0, 4.0], [8.0, 
1.0, 5.0], [4.0, 4.0, 8.0], [7.0, 8.0, 4.0]], [[3.0, 1.0, 6.0], [4.0, 3.0, 2.0], [2.0, 4.0, 8.0], [7.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UUN(self):
      arg=Data(numpy.array([[[8.0, 8.0, 1.0], [7.0, 1.0, 2.0], [3.0, 2.0, 3.0], [8.0, 6.0, 3.0]], [[2.0, 2.0, 6.0], [7.0, 8.0, 
5.0], [7.0, 6.0, 2.0], [1.0, 1.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 8.0, 6.0], [4.0, 5.0, 7.0], [5.0, 8.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 7.0, 3.0], [5.0, 5.0, 5.0], [8.0, 6.0, 7.0], [1.0, 8.0, 5.0]], [[8.0, 2.0, 3.0], 
[8.0, 5.0, 2.0], [5.0, 4.0, 7.0], [2.0, 8.0, 3.0]], [[5.0, 4.0, 8.0], [6.0, 4.0, 2.0], [1.0, 4.0, 5.0], [6.0, 3.0, 6.0]]]))
      val=Data(numpy.array([[[5.0, 7.0, 1.0], [3.0, 2.0, 3.0], [3.0, 3.0, 8.0]], [[8.0, 5.0, 4.0], [4.0, 7.0, 5.0], [6.0, 8.0, 
7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 1.0, 8.0], [7.0, 7.0, 4.0], [6.0, 4.0, 5.0]], [[4.0, 6.0, 2.0], [5.0, 6.0, 4.0], 
[2.0, 4.0, 1.0]]]))
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 1.0], [3.0, 2.0, 3.0], [3.0, 3.0, 8.0], [8.0, 6.0, 3.0]], [[8.0, 5.0, 4.0], [4.0, 
7.0, 5.0], [6.0, 8.0, 7.0], [1.0, 1.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 8.0, 6.0], [4.0, 5.0, 7.0], [5.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 8.0], [7.0, 7.0, 4.0], [6.0, 4.0, 5.0], [1.0, 8.0, 5.0]], [[4.0, 6.0, 2.0], [5.0, 
6.0, 4.0], [2.0, 4.0, 1.0], [2.0, 8.0, 3.0]], [[5.0, 4.0, 8.0], [6.0, 4.0, 2.0], [1.0, 4.0, 5.0], [6.0, 3.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UUN(self):
      arg=Data(numpy.array([[[4.0, 2.0, 4.0], [8.0, 6.0, 2.0], [7.0, 7.0, 6.0], [2.0, 8.0, 1.0]], [[4.0, 1.0, 4.0], [1.0, 6.0, 
3.0], [6.0, 2.0, 3.0], [7.0, 3.0, 3.0]], [[7.0, 8.0, 3.0], [3.0, 1.0, 7.0], [7.0, 6.0, 5.0], [1.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 6.0, 3.0], [6.0, 3.0, 2.0], [3.0, 7.0, 6.0], [2.0, 4.0, 3.0]], [[7.0, 3.0, 4.0], 
[8.0, 1.0, 4.0], [6.0, 8.0, 7.0], [7.0, 2.0, 2.0]], [[7.0, 6.0, 7.0], [1.0, 8.0, 7.0], [8.0, 8.0, 1.0], [2.0, 3.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 6.0, 7.0], [7.0, 8.0, 1.0], [1.0, 5.0, 3.0]], [[4.0, 3.0, 7.0], [7.0, 2.0, 7.0], [7.0, 
1.0, 1.0]]])+(1.-msk_val)*numpy.array([[[7.0, 3.0, 2.0], [5.0, 4.0, 6.0], [4.0, 6.0, 6.0]], [[1.0, 2.0, 3.0], [5.0, 7.0, 5.0], 
[4.0, 8.0, 7.0]]])
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 7.0], [7.0, 8.0, 1.0], [1.0, 5.0, 3.0], [2.0, 8.0, 1.0]], [[4.0, 3.0, 7.0], [7.0, 
2.0, 7.0], [7.0, 1.0, 1.0], [7.0, 3.0, 3.0]], [[7.0, 8.0, 3.0], [3.0, 1.0, 7.0], [7.0, 6.0, 5.0], [1.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 2.0], [5.0, 4.0, 6.0], [4.0, 6.0, 6.0], [2.0, 4.0, 3.0]], [[1.0, 2.0, 3.0], [5.0, 
7.0, 5.0], [4.0, 8.0, 7.0], [7.0, 2.0, 2.0]], [[7.0, 6.0, 7.0], [1.0, 8.0, 7.0], [8.0, 8.0, 1.0], [2.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LCC(self):
      arg=Data(numpy.array([[[8.0, 6.0, 8.0], [6.0, 4.0, 7.0], [3.0, 3.0, 8.0], [6.0, 5.0, 8.0]], [[4.0, 3.0, 5.0], [3.0, 5.0, 
2.0], [7.0, 2.0, 2.0], [6.0, 4.0, 4.0]], [[5.0, 8.0, 8.0], [5.0, 4.0, 5.0], [1.0, 7.0, 8.0], [7.0, 2.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 3.0, 4.0], [5.0, 4.0, 6.0], [2.0, 6.0, 4.0], [3.0, 2.0, 2.0]], [[8.0, 4.0, 3.0], 
[3.0, 4.0, 4.0], [4.0, 8.0, 8.0], [8.0, 6.0, 1.0]], [[6.0, 8.0, 7.0], [6.0, 7.0, 6.0], [2.0, 5.0, 3.0], [2.0, 7.0, 8.0]]]))
      val=numpy.array([[[3.0, 4.0, 8.0], [8.0, 6.0, 5.0], [6.0, 5.0, 8.0], [4.0, 3.0, 1.0]]])
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 8.0], [6.0, 4.0, 7.0], [3.0, 3.0, 8.0], [6.0, 5.0, 8.0]], [[4.0, 3.0, 5.0], [3.0, 
5.0, 2.0], [7.0, 2.0, 2.0], [6.0, 4.0, 4.0]], [[3.0, 4.0, 8.0], [8.0, 6.0, 5.0], [6.0, 5.0, 8.0], [4.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 4.0], [5.0, 4.0, 6.0], [2.0, 6.0, 4.0], [3.0, 2.0, 2.0]], [[8.0, 4.0, 3.0], [3.0, 
4.0, 4.0], [4.0, 8.0, 8.0], [8.0, 6.0, 1.0]], [[3.0, 4.0, 8.0], [8.0, 6.0, 5.0], [6.0, 5.0, 8.0], [4.0, 3.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LCC(self):
      arg=Data(numpy.array([[[6.0, 2.0, 8.0], [7.0, 7.0, 7.0], [6.0, 8.0, 7.0], [7.0, 1.0, 3.0]], [[8.0, 6.0, 2.0], [7.0, 1.0, 
5.0], [2.0, 2.0, 7.0], [6.0, 7.0, 6.0]], [[7.0, 6.0, 3.0], [5.0, 4.0, 7.0], [3.0, 3.0, 5.0], [6.0, 2.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 4.0, 1.0], [3.0, 8.0, 6.0], [8.0, 6.0, 2.0], [2.0, 8.0, 8.0]], [[8.0, 5.0, 4.0], 
[7.0, 1.0, 2.0], [8.0, 2.0, 5.0], [5.0, 8.0, 1.0]], [[5.0, 4.0, 5.0], [2.0, 2.0, 3.0], [6.0, 6.0, 5.0], [2.0, 5.0, 1.0]]]))
      val=Data(numpy.array([[[6.0, 1.0, 7.0], [7.0, 1.0, 2.0], [5.0, 3.0, 8.0], [7.0, 7.0, 7.0]]]),self.functionspace)
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 8.0], [7.0, 7.0, 7.0], [6.0, 8.0, 7.0], [7.0, 1.0, 3.0]], [[8.0, 6.0, 2.0], [7.0, 
1.0, 5.0], [2.0, 2.0, 7.0], [6.0, 7.0, 6.0]], [[6.0, 1.0, 7.0], [7.0, 1.0, 2.0], [5.0, 3.0, 8.0], [7.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 1.0], [3.0, 8.0, 6.0], [8.0, 6.0, 2.0], [2.0, 8.0, 8.0]], [[8.0, 5.0, 4.0], [7.0, 
1.0, 2.0], [8.0, 2.0, 5.0], [5.0, 8.0, 1.0]], [[6.0, 1.0, 7.0], [7.0, 1.0, 2.0], [5.0, 3.0, 8.0], [7.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LCC(self):
      arg=Data(numpy.array([[[2.0, 4.0, 6.0], [8.0, 3.0, 3.0], [2.0, 3.0, 6.0], [6.0, 1.0, 7.0]], [[2.0, 2.0, 4.0], [5.0, 3.0, 
7.0], [3.0, 1.0, 3.0], [4.0, 1.0, 4.0]], [[5.0, 4.0, 8.0], [4.0, 3.0, 8.0], [2.0, 8.0, 4.0], [7.0, 7.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 1.0, 2.0], [8.0, 6.0, 7.0], [5.0, 1.0, 8.0], [3.0, 4.0, 4.0]], [[6.0, 1.0, 7.0], 
[7.0, 3.0, 2.0], [6.0, 4.0, 6.0], [3.0, 7.0, 7.0]], [[3.0, 8.0, 8.0], [6.0, 8.0, 1.0], [2.0, 2.0, 2.0], [8.0, 8.0, 1.0]]]))
      val=Data(numpy.array([[[3.0, 5.0, 8.0], [5.0, 4.0, 5.0], [4.0, 4.0, 7.0], [3.0, 3.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 3.0, 4.0], [6.0, 4.0, 1.0], [5.0, 6.0, 7.0], [3.0, 1.0, 7.0]]]))
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 6.0], [8.0, 3.0, 3.0], [2.0, 3.0, 6.0], [6.0, 1.0, 7.0]], [[2.0, 2.0, 4.0], [5.0, 
3.0, 7.0], [3.0, 1.0, 3.0], [4.0, 1.0, 4.0]], [[3.0, 5.0, 8.0], [5.0, 4.0, 5.0], [4.0, 4.0, 7.0], [3.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 2.0], [8.0, 6.0, 7.0], [5.0, 1.0, 8.0], [3.0, 4.0, 4.0]], [[6.0, 1.0, 7.0], [7.0, 
3.0, 2.0], [6.0, 4.0, 6.0], [3.0, 7.0, 7.0]], [[5.0, 3.0, 4.0], [6.0, 4.0, 1.0], [5.0, 6.0, 7.0], [3.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LCC(self):
      arg=Data(numpy.array([[[7.0, 6.0, 3.0], [3.0, 5.0, 8.0], [5.0, 4.0, 2.0], [5.0, 4.0, 4.0]], [[5.0, 8.0, 6.0], [4.0, 1.0, 
1.0], [7.0, 8.0, 8.0], [5.0, 5.0, 2.0]], [[1.0, 1.0, 5.0], [1.0, 3.0, 1.0], [2.0, 5.0, 8.0], [1.0, 8.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 3.0, 8.0], [4.0, 4.0, 1.0], [2.0, 4.0, 4.0], [2.0, 1.0, 1.0]], [[6.0, 5.0, 8.0], 
[6.0, 7.0, 4.0], [6.0, 6.0, 4.0], [1.0, 6.0, 6.0]], [[6.0, 6.0, 1.0], [4.0, 1.0, 4.0], [2.0, 1.0, 4.0], [3.0, 4.0, 6.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 6.0, 2.0], [3.0, 3.0, 4.0], [4.0, 8.0, 8.0], [8.0, 1.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[4.0, 7.0, 6.0], [4.0, 4.0, 7.0], [3.0, 1.0, 2.0], [1.0, 5.0, 3.0]]])
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 3.0], [3.0, 5.0, 8.0], [5.0, 4.0, 2.0], [5.0, 4.0, 4.0]], [[5.0, 8.0, 6.0], [4.0, 
1.0, 1.0], [7.0, 8.0, 8.0], [5.0, 5.0, 2.0]], [[3.0, 6.0, 2.0], [3.0, 3.0, 4.0], [4.0, 8.0, 8.0], [8.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 8.0], [4.0, 4.0, 1.0], [2.0, 4.0, 4.0], [2.0, 1.0, 1.0]], [[6.0, 5.0, 8.0], [6.0, 
7.0, 4.0], [6.0, 6.0, 4.0], [1.0, 6.0, 6.0]], [[4.0, 7.0, 6.0], [4.0, 4.0, 7.0], [3.0, 1.0, 2.0], [1.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LPR(self):
      arg=Data(numpy.array([[[7.0, 4.0, 5.0], [7.0, 5.0, 7.0], [8.0, 5.0, 7.0], [1.0, 6.0, 4.0]], [[4.0, 2.0, 5.0], [5.0, 3.0, 
4.0], [3.0, 6.0, 3.0], [5.0, 4.0, 3.0]], [[2.0, 7.0, 2.0], [6.0, 7.0, 3.0], [4.0, 6.0, 1.0], [7.0, 1.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 3.0, 1.0], [1.0, 2.0, 5.0], [6.0, 6.0, 1.0], [6.0, 1.0, 4.0]], [[2.0, 8.0, 2.0], 
[3.0, 7.0, 2.0], [3.0, 1.0, 7.0], [1.0, 4.0, 3.0]], [[6.0, 1.0, 4.0], [7.0, 7.0, 1.0], [4.0, 4.0, 1.0], [5.0, 4.0, 6.0]]]))
      val=numpy.array([[7.0]])
      arg[2:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 5.0], [7.0, 5.0, 7.0], [8.0, 5.0, 7.0], [1.0, 6.0, 4.0]], [[4.0, 2.0, 5.0], [5.0, 
3.0, 4.0], [3.0, 6.0, 3.0], [5.0, 4.0, 3.0]], [[2.0, 7.0, 2.0], [6.0, 7.0, 3.0], [4.0, 7.0, 1.0], [7.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 1.0], [1.0, 2.0, 5.0], [6.0, 6.0, 1.0], [6.0, 1.0, 4.0]], [[2.0, 8.0, 2.0], [3.0, 
7.0, 2.0], [3.0, 1.0, 7.0], [1.0, 4.0, 3.0]], [[6.0, 1.0, 4.0], [7.0, 7.0, 1.0], [4.0, 7.0, 1.0], [5.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LPR(self):
      arg=Data(numpy.array([[[1.0, 8.0, 4.0], [4.0, 4.0, 7.0], [8.0, 8.0, 5.0], [2.0, 7.0, 5.0]], [[5.0, 6.0, 6.0], [4.0, 2.0, 
2.0], [6.0, 5.0, 4.0], [6.0, 5.0, 5.0]], [[1.0, 4.0, 1.0], [7.0, 7.0, 5.0], [2.0, 3.0, 4.0], [6.0, 6.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 7.0, 1.0], [4.0, 2.0, 2.0], [7.0, 6.0, 1.0], [3.0, 1.0, 2.0]], [[4.0, 4.0, 3.0], 
[4.0, 6.0, 4.0], [8.0, 1.0, 4.0], [5.0, 2.0, 8.0]], [[2.0, 3.0, 8.0], [1.0, 3.0, 7.0], [6.0, 4.0, 4.0], [2.0, 5.0, 2.0]]]))
      val=Data(numpy.array([[8.0]]),self.functionspace)
      arg[2:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 4.0], [4.0, 4.0, 7.0], [8.0, 8.0, 5.0], [2.0, 7.0, 5.0]], [[5.0, 6.0, 6.0], [4.0, 
2.0, 2.0], [6.0, 5.0, 4.0], [6.0, 5.0, 5.0]], [[1.0, 4.0, 1.0], [7.0, 7.0, 5.0], [2.0, 8.0, 4.0], [6.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 1.0], [4.0, 2.0, 2.0], [7.0, 6.0, 1.0], [3.0, 1.0, 2.0]], [[4.0, 4.0, 3.0], [4.0, 
6.0, 4.0], [8.0, 1.0, 4.0], [5.0, 2.0, 8.0]], [[2.0, 3.0, 8.0], [1.0, 3.0, 7.0], [6.0, 8.0, 4.0], [2.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LPR(self):
      arg=Data(numpy.array([[[1.0, 2.0, 1.0], [4.0, 6.0, 7.0], [1.0, 3.0, 4.0], [7.0, 7.0, 1.0]], [[4.0, 4.0, 7.0], [5.0, 8.0, 
5.0], [3.0, 2.0, 2.0], [8.0, 5.0, 6.0]], [[7.0, 4.0, 8.0], [2.0, 3.0, 5.0], [3.0, 2.0, 1.0], [8.0, 4.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 2.0, 4.0], [8.0, 7.0, 1.0], [5.0, 1.0, 5.0], [8.0, 6.0, 2.0]], [[5.0, 6.0, 3.0], 
[3.0, 6.0, 1.0], [1.0, 1.0, 2.0], [2.0, 1.0, 7.0]], [[5.0, 8.0, 7.0], [3.0, 3.0, 5.0], [3.0, 7.0, 8.0], [3.0, 5.0, 1.0]]]))
      val=Data(numpy.array([[3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0]]))
      arg[2:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 1.0], [4.0, 6.0, 7.0], [1.0, 3.0, 4.0], [7.0, 7.0, 1.0]], [[4.0, 4.0, 7.0], [5.0, 
8.0, 5.0], [3.0, 2.0, 2.0], [8.0, 5.0, 6.0]], [[7.0, 4.0, 8.0], [2.0, 3.0, 5.0], [3.0, 3.0, 1.0], [8.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 4.0], [8.0, 7.0, 1.0], [5.0, 1.0, 5.0], [8.0, 6.0, 2.0]], [[5.0, 6.0, 3.0], [3.0, 
6.0, 1.0], [1.0, 1.0, 2.0], [2.0, 1.0, 7.0]], [[5.0, 8.0, 7.0], [3.0, 3.0, 5.0], [3.0, 6.0, 8.0], [3.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LPR(self):
      arg=Data(numpy.array([[[6.0, 6.0, 5.0], [3.0, 7.0, 8.0], [2.0, 5.0, 1.0], [6.0, 8.0, 8.0]], [[5.0, 3.0, 3.0], [8.0, 8.0, 
6.0], [1.0, 5.0, 1.0], [8.0, 5.0, 8.0]], [[2.0, 5.0, 6.0], [2.0, 4.0, 2.0], [8.0, 2.0, 4.0], [8.0, 6.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 1.0, 7.0], [6.0, 4.0, 4.0], [2.0, 5.0, 4.0], [4.0, 7.0, 8.0]], [[6.0, 5.0, 8.0], 
[1.0, 1.0, 5.0], [1.0, 4.0, 2.0], [8.0, 3.0, 8.0]], [[5.0, 8.0, 3.0], [1.0, 4.0, 4.0], [5.0, 8.0, 6.0], [1.0, 7.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[2.0]])+(1.-msk_val)*numpy.array([[7.0]])
      arg[2:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 5.0], [3.0, 7.0, 8.0], [2.0, 5.0, 1.0], [6.0, 8.0, 8.0]], [[5.0, 3.0, 3.0], [8.0, 
8.0, 6.0], [1.0, 5.0, 1.0], [8.0, 5.0, 8.0]], [[2.0, 5.0, 6.0], [2.0, 4.0, 2.0], [8.0, 2.0, 4.0], [8.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 1.0, 7.0], [6.0, 4.0, 4.0], [2.0, 5.0, 4.0], [4.0, 7.0, 8.0]], [[6.0, 5.0, 8.0], [1.0, 
1.0, 5.0], [1.0, 4.0, 2.0], [8.0, 3.0, 8.0]], [[5.0, 8.0, 3.0], [1.0, 4.0, 4.0], [5.0, 7.0, 6.0], [1.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RRN(self):
      arg=Data(numpy.array([[[6.0, 4.0, 2.0], [8.0, 3.0, 3.0], [3.0, 8.0, 1.0], [2.0, 8.0, 2.0]], [[7.0, 8.0, 5.0], [7.0, 8.0, 
5.0], [3.0, 8.0, 6.0], [8.0, 1.0, 7.0]], [[3.0, 2.0, 7.0], [1.0, 5.0, 2.0], [3.0, 2.0, 1.0], [6.0, 5.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 6.0, 7.0], [3.0, 7.0, 4.0], [8.0, 5.0, 8.0], [3.0, 7.0, 2.0]], [[5.0, 8.0, 7.0], 
[6.0, 2.0, 1.0], [8.0, 7.0, 8.0], [6.0, 2.0, 3.0]], [[6.0, 4.0, 2.0], [2.0, 3.0, 7.0], [4.0, 7.0, 3.0], [6.0, 4.0, 4.0]]]))
      val=numpy.array([[[7.0, 7.0, 1.0], [1.0, 3.0, 6.0]]])
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 2.0], [8.0, 3.0, 3.0], [3.0, 8.0, 1.0], [2.0, 8.0, 2.0]], [[7.0, 8.0, 5.0], [7.0, 
7.0, 1.0], [1.0, 3.0, 6.0], [8.0, 1.0, 7.0]], [[3.0, 2.0, 7.0], [1.0, 5.0, 2.0], [3.0, 2.0, 1.0], [6.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 6.0, 7.0], [3.0, 7.0, 4.0], [8.0, 5.0, 8.0], [3.0, 7.0, 2.0]], [[5.0, 8.0, 7.0], [7.0, 
7.0, 1.0], [1.0, 3.0, 6.0], [6.0, 2.0, 3.0]], [[6.0, 4.0, 2.0], [2.0, 3.0, 7.0], [4.0, 7.0, 3.0], [6.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RRN(self):
      arg=Data(numpy.array([[[7.0, 3.0, 7.0], [7.0, 5.0, 3.0], [5.0, 4.0, 7.0], [4.0, 8.0, 6.0]], [[1.0, 2.0, 1.0], [1.0, 6.0, 
5.0], [4.0, 2.0, 8.0], [1.0, 5.0, 1.0]], [[8.0, 4.0, 7.0], [4.0, 7.0, 6.0], [1.0, 8.0, 7.0], [1.0, 4.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 7.0, 4.0], [7.0, 3.0, 1.0], [8.0, 7.0, 2.0], [7.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], 
[2.0, 8.0, 4.0], [4.0, 7.0, 6.0], [3.0, 1.0, 4.0]], [[6.0, 2.0, 7.0], [7.0, 3.0, 4.0], [6.0, 4.0, 6.0], [4.0, 5.0, 2.0]]]))
      val=Data(numpy.array([[[3.0, 2.0, 8.0], [1.0, 1.0, 8.0]]]),self.functionspace)
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 7.0], [7.0, 5.0, 3.0], [5.0, 4.0, 7.0], [4.0, 8.0, 6.0]], [[1.0, 2.0, 1.0], [3.0, 
2.0, 8.0], [1.0, 1.0, 8.0], [1.0, 5.0, 1.0]], [[8.0, 4.0, 7.0], [4.0, 7.0, 6.0], [1.0, 8.0, 7.0], [1.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 4.0], [7.0, 3.0, 1.0], [8.0, 7.0, 2.0], [7.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], [3.0, 
2.0, 8.0], [1.0, 1.0, 8.0], [3.0, 1.0, 4.0]], [[6.0, 2.0, 7.0], [7.0, 3.0, 4.0], [6.0, 4.0, 6.0], [4.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RRN(self):
      arg=Data(numpy.array([[[6.0, 4.0, 3.0], [2.0, 1.0, 7.0], [8.0, 7.0, 6.0], [7.0, 3.0, 2.0]], [[6.0, 2.0, 8.0], [3.0, 4.0, 
4.0], [2.0, 4.0, 1.0], [4.0, 5.0, 5.0]], [[6.0, 3.0, 6.0], [1.0, 4.0, 6.0], [5.0, 6.0, 3.0], [8.0, 8.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 4.0, 8.0], [5.0, 2.0, 1.0], [1.0, 2.0, 1.0], [3.0, 1.0, 7.0]], [[4.0, 1.0, 5.0], 
[3.0, 1.0, 3.0], [5.0, 6.0, 3.0], [2.0, 3.0, 8.0]], [[2.0, 1.0, 2.0], [4.0, 7.0, 1.0], [5.0, 4.0, 3.0], [2.0, 1.0, 1.0]]]))
      val=Data(numpy.array([[[7.0, 3.0, 2.0], [5.0, 7.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 4.0, 6.0], [1.0, 7.0, 8.0]]]))
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 3.0], [2.0, 1.0, 7.0], [8.0, 7.0, 6.0], [7.0, 3.0, 2.0]], [[6.0, 2.0, 8.0], [7.0, 
3.0, 2.0], [5.0, 7.0, 7.0], [4.0, 5.0, 5.0]], [[6.0, 3.0, 6.0], [1.0, 4.0, 6.0], [5.0, 6.0, 3.0], [8.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 8.0], [5.0, 2.0, 1.0], [1.0, 2.0, 1.0], [3.0, 1.0, 7.0]], [[4.0, 1.0, 5.0], [8.0, 
4.0, 6.0], [1.0, 7.0, 8.0], [2.0, 3.0, 8.0]], [[2.0, 1.0, 2.0], [4.0, 7.0, 1.0], [5.0, 4.0, 3.0], [2.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RRN(self):
      arg=Data(numpy.array([[[7.0, 7.0, 7.0], [4.0, 5.0, 6.0], [6.0, 4.0, 4.0], [5.0, 8.0, 1.0]], [[3.0, 2.0, 2.0], [4.0, 7.0, 
5.0], [8.0, 3.0, 5.0], [6.0, 5.0, 6.0]], [[5.0, 8.0, 4.0], [4.0, 5.0, 5.0], [1.0, 4.0, 4.0], [1.0, 1.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 7.0, 7.0], [8.0, 4.0, 3.0], [1.0, 7.0, 7.0], [3.0, 7.0, 7.0]], [[6.0, 6.0, 5.0], 
[8.0, 5.0, 2.0], [3.0, 4.0, 1.0], [1.0, 2.0, 4.0]], [[3.0, 6.0, 8.0], [5.0, 8.0, 7.0], [6.0, 4.0, 1.0], [4.0, 5.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[6.0, 4.0, 5.0], [1.0, 1.0, 7.0]]])+(1.-msk_val)*numpy.array([[[2.0, 4.0, 6.0], [8.0, 6.0, 
2.0]]])
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 7.0, 7.0], [4.0, 5.0, 6.0], [6.0, 4.0, 4.0], [5.0, 8.0, 1.0]], [[3.0, 2.0, 2.0], [6.0, 
4.0, 5.0], [1.0, 1.0, 7.0], [6.0, 5.0, 6.0]], [[5.0, 8.0, 4.0], [4.0, 5.0, 5.0], [1.0, 4.0, 4.0], [1.0, 1.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 7.0, 7.0], [8.0, 4.0, 3.0], [1.0, 7.0, 7.0], [3.0, 7.0, 7.0]], [[6.0, 6.0, 5.0], [2.0, 
4.0, 6.0], [8.0, 6.0, 2.0], [1.0, 2.0, 4.0]], [[3.0, 6.0, 8.0], [5.0, 8.0, 7.0], [6.0, 4.0, 1.0], [4.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_URC(self):
      arg=Data(numpy.array([[[4.0, 3.0, 8.0], [6.0, 7.0, 7.0], [4.0, 3.0, 6.0], [4.0, 8.0, 1.0]], [[2.0, 4.0, 2.0], [3.0, 3.0, 
7.0], [6.0, 1.0, 7.0], [5.0, 3.0, 5.0]], [[8.0, 4.0, 4.0], [5.0, 3.0, 1.0], [5.0, 3.0, 8.0], [5.0, 8.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 3.0, 5.0], [2.0, 4.0, 1.0], [8.0, 8.0, 6.0], [8.0, 5.0, 7.0]], [[1.0, 7.0, 6.0], 
[6.0, 3.0, 2.0], [8.0, 5.0, 7.0], [6.0, 6.0, 4.0]], [[2.0, 4.0, 7.0], [2.0, 1.0, 8.0], [2.0, 7.0, 8.0], [3.0, 2.0, 7.0]]]))
      val=numpy.array([[[7.0, 3.0, 8.0], [2.0, 1.0, 3.0]], [[4.0, 4.0, 6.0], [8.0, 6.0, 2.0]]])
      arg[:2,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 8.0], [7.0, 3.0, 8.0], [2.0, 1.0, 3.0], [4.0, 8.0, 1.0]], [[2.0, 4.0, 2.0], [4.0, 
4.0, 6.0], [8.0, 6.0, 2.0], [5.0, 3.0, 5.0]], [[8.0, 4.0, 4.0], [5.0, 3.0, 1.0], [5.0, 3.0, 8.0], [5.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 5.0], [7.0, 3.0, 8.0], [2.0, 1.0, 3.0], [8.0, 5.0, 7.0]], [[1.0, 7.0, 6.0], [4.0, 
4.0, 6.0], [8.0, 6.0, 2.0], [6.0, 6.0, 4.0]], [[2.0, 4.0, 7.0], [2.0, 1.0, 8.0], [2.0, 7.0, 8.0], [3.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_URC(self):
      arg=Data(numpy.array([[[6.0, 1.0, 6.0], [2.0, 6.0, 2.0], [4.0, 8.0, 1.0], [8.0, 1.0, 7.0]], [[4.0, 8.0, 1.0], [3.0, 6.0, 
7.0], [6.0, 8.0, 4.0], [6.0, 6.0, 1.0]], [[6.0, 8.0, 4.0], [3.0, 3.0, 4.0], [7.0, 5.0, 3.0], [5.0, 1.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 4.0, 6.0], [4.0, 4.0, 6.0], [6.0, 8.0, 4.0], [6.0, 1.0, 8.0]], [[3.0, 5.0, 6.0], 
[3.0, 8.0, 2.0], [2.0, 2.0, 7.0], [6.0, 7.0, 2.0]], [[6.0, 5.0, 1.0], [2.0, 1.0, 5.0], [7.0, 8.0, 3.0], [5.0, 4.0, 5.0]]]))
      val=Data(numpy.array([[[8.0, 1.0, 6.0], [3.0, 6.0, 4.0]], [[3.0, 5.0, 4.0], [8.0, 2.0, 8.0]]]),self.functionspace)
      arg[:2,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 6.0], [8.0, 1.0, 6.0], [3.0, 6.0, 4.0], [8.0, 1.0, 7.0]], [[4.0, 8.0, 1.0], [3.0, 
5.0, 4.0], [8.0, 2.0, 8.0], [6.0, 6.0, 1.0]], [[6.0, 8.0, 4.0], [3.0, 3.0, 4.0], [7.0, 5.0, 3.0], [5.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 6.0], [8.0, 1.0, 6.0], [3.0, 6.0, 4.0], [6.0, 1.0, 8.0]], [[3.0, 5.0, 6.0], [3.0, 
5.0, 4.0], [8.0, 2.0, 8.0], [6.0, 7.0, 2.0]], [[6.0, 5.0, 1.0], [2.0, 1.0, 5.0], [7.0, 8.0, 3.0], [5.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_URC(self):
      arg=Data(numpy.array([[[6.0, 1.0, 2.0], [5.0, 4.0, 8.0], [2.0, 2.0, 2.0], [7.0, 2.0, 7.0]], [[2.0, 8.0, 1.0], [5.0, 4.0, 
4.0], [4.0, 7.0, 1.0], [2.0, 4.0, 6.0]], [[7.0, 5.0, 4.0], [5.0, 5.0, 1.0], [5.0, 4.0, 5.0], [6.0, 5.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 8.0, 2.0], [2.0, 1.0, 5.0], [6.0, 3.0, 8.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 6.0], 
[5.0, 3.0, 5.0], [1.0, 3.0, 8.0], [4.0, 4.0, 5.0]], [[2.0, 4.0, 4.0], [2.0, 5.0, 6.0], [8.0, 7.0, 6.0], [8.0, 2.0, 5.0]]]))
      val=Data(numpy.array([[[2.0, 5.0, 6.0], [4.0, 1.0, 4.0]], [[4.0, 5.0, 6.0], [4.0, 6.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 8.0, 3.0], [4.0, 3.0, 8.0]], [[4.0, 7.0, 3.0], [5.0, 6.0, 3.0]]]))
      arg[:2,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 2.0], [2.0, 5.0, 6.0], [4.0, 1.0, 4.0], [7.0, 2.0, 7.0]], [[2.0, 8.0, 1.0], [4.0, 
5.0, 6.0], [4.0, 6.0, 8.0], [2.0, 4.0, 6.0]], [[7.0, 5.0, 4.0], [5.0, 5.0, 1.0], [5.0, 4.0, 5.0], [6.0, 5.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 2.0], [1.0, 8.0, 3.0], [4.0, 3.0, 8.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 6.0], [4.0, 
7.0, 3.0], [5.0, 6.0, 3.0], [4.0, 4.0, 5.0]], [[2.0, 4.0, 4.0], [2.0, 5.0, 6.0], [8.0, 7.0, 6.0], [8.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_URC(self):
      arg=Data(numpy.array([[[1.0, 1.0, 8.0], [3.0, 7.0, 3.0], [5.0, 2.0, 1.0], [4.0, 6.0, 3.0]], [[6.0, 8.0, 2.0], [1.0, 4.0, 
1.0], [6.0, 2.0, 1.0], [3.0, 5.0, 4.0]], [[1.0, 4.0, 8.0], [1.0, 8.0, 6.0], [4.0, 2.0, 6.0], [6.0, 7.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 3.0, 2.0], [6.0, 7.0, 5.0], [4.0, 7.0, 8.0], [5.0, 3.0, 3.0]], [[1.0, 8.0, 2.0], 
[4.0, 1.0, 1.0], [1.0, 6.0, 6.0], [5.0, 5.0, 2.0]], [[2.0, 5.0, 8.0], [8.0, 1.0, 3.0], [3.0, 4.0, 6.0], [7.0, 6.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[6.0, 6.0, 5.0], [2.0, 8.0, 6.0]], [[3.0, 8.0, 8.0], [4.0, 3.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[3.0, 7.0, 8.0], [6.0, 5.0, 2.0]], [[8.0, 8.0, 8.0], [4.0, 2.0, 2.0]]])
      arg[:2,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 8.0], [6.0, 6.0, 5.0], [2.0, 8.0, 6.0], [4.0, 6.0, 3.0]], [[6.0, 8.0, 2.0], [3.0, 
8.0, 8.0], [4.0, 3.0, 4.0], [3.0, 5.0, 4.0]], [[1.0, 4.0, 8.0], [1.0, 8.0, 6.0], [4.0, 2.0, 6.0], [6.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 2.0], [3.0, 7.0, 8.0], [6.0, 5.0, 2.0], [5.0, 3.0, 3.0]], [[1.0, 8.0, 2.0], [8.0, 
8.0, 8.0], [4.0, 2.0, 2.0], [5.0, 5.0, 2.0]], [[2.0, 5.0, 8.0], [8.0, 1.0, 3.0], [3.0, 4.0, 6.0], [7.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RCL(self):
      arg=Data(numpy.array([[[6.0, 5.0, 2.0], [6.0, 5.0, 5.0], [7.0, 5.0, 6.0], [1.0, 4.0, 7.0]], [[5.0, 4.0, 6.0], [4.0, 2.0, 
1.0], [6.0, 5.0, 1.0], [2.0, 4.0, 3.0]], [[2.0, 2.0, 5.0], [5.0, 1.0, 7.0], [4.0, 4.0, 4.0], [3.0, 2.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 2.0, 3.0], [4.0, 6.0, 8.0], [6.0, 3.0, 5.0], [6.0, 6.0, 4.0]], [[3.0, 2.0, 8.0], 
[6.0, 1.0, 3.0], [3.0, 4.0, 3.0], [5.0, 4.0, 2.0]], [[1.0, 5.0, 4.0], [8.0, 3.0, 1.0], [1.0, 3.0, 4.0], [1.0, 8.0, 7.0]]]))
      val=numpy.array([[[4.0], [2.0], [8.0], [8.0]]])
      arg[1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 2.0], [6.0, 5.0, 5.0], [7.0, 5.0, 6.0], [1.0, 4.0, 7.0]], [[5.0, 4.0, 4.0], [4.0, 
2.0, 2.0], [6.0, 5.0, 8.0], [2.0, 4.0, 8.0]], [[2.0, 2.0, 5.0], [5.0, 1.0, 7.0], [4.0, 4.0, 4.0], [3.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 3.0], [4.0, 6.0, 8.0], [6.0, 3.0, 5.0], [6.0, 6.0, 4.0]], [[3.0, 2.0, 4.0], [6.0, 
1.0, 2.0], [3.0, 4.0, 8.0], [5.0, 4.0, 8.0]], [[1.0, 5.0, 4.0], [8.0, 3.0, 1.0], [1.0, 3.0, 4.0], [1.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RCL(self):
      arg=Data(numpy.array([[[1.0, 4.0, 1.0], [5.0, 1.0, 4.0], [7.0, 1.0, 1.0], [8.0, 1.0, 4.0]], [[5.0, 8.0, 6.0], [7.0, 1.0, 
2.0], [4.0, 7.0, 1.0], [3.0, 8.0, 5.0]], [[6.0, 5.0, 4.0], [4.0, 6.0, 5.0], [4.0, 4.0, 2.0], [3.0, 2.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 1.0, 2.0], [4.0, 7.0, 5.0], [3.0, 1.0, 3.0], [7.0, 1.0, 4.0]], [[5.0, 7.0, 3.0], 
[5.0, 4.0, 4.0], [6.0, 1.0, 6.0], [2.0, 6.0, 2.0]], [[2.0, 1.0, 6.0], [3.0, 6.0, 3.0], [5.0, 8.0, 5.0], [6.0, 4.0, 2.0]]]))
      val=Data(numpy.array([[[2.0], [2.0], [3.0], [4.0]]]),self.functionspace)
      arg[1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 4.0, 1.0], [5.0, 1.0, 4.0], [7.0, 1.0, 1.0], [8.0, 1.0, 4.0]], [[5.0, 8.0, 2.0], [7.0, 
1.0, 2.0], [4.0, 7.0, 3.0], [3.0, 8.0, 4.0]], [[6.0, 5.0, 4.0], [4.0, 6.0, 5.0], [4.0, 4.0, 2.0], [3.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 2.0], [4.0, 7.0, 5.0], [3.0, 1.0, 3.0], [7.0, 1.0, 4.0]], [[5.0, 7.0, 2.0], [5.0, 
4.0, 2.0], [6.0, 1.0, 3.0], [2.0, 6.0, 4.0]], [[2.0, 1.0, 6.0], [3.0, 6.0, 3.0], [5.0, 8.0, 5.0], [6.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RCL(self):
      arg=Data(numpy.array([[[8.0, 6.0, 5.0], [7.0, 6.0, 1.0], [7.0, 7.0, 8.0], [7.0, 4.0, 7.0]], [[6.0, 6.0, 3.0], [8.0, 5.0, 
7.0], [4.0, 8.0, 1.0], [8.0, 3.0, 2.0]], [[7.0, 8.0, 6.0], [5.0, 7.0, 3.0], [8.0, 6.0, 4.0], [4.0, 7.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 6.0, 3.0], [7.0, 2.0, 2.0], [8.0, 6.0, 7.0], [8.0, 5.0, 4.0]], [[3.0, 7.0, 5.0], 
[3.0, 2.0, 1.0], [8.0, 2.0, 6.0], [8.0, 7.0, 1.0]], [[6.0, 3.0, 8.0], [8.0, 4.0, 7.0], [4.0, 6.0, 2.0], [7.0, 2.0, 5.0]]]))
      val=Data(numpy.array([[[5.0], [4.0], [4.0], [4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0], [4.0], [6.0], [7.0]]]))
      arg[1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 5.0], [7.0, 6.0, 1.0], [7.0, 7.0, 8.0], [7.0, 4.0, 7.0]], [[6.0, 6.0, 5.0], [8.0, 
5.0, 4.0], [4.0, 8.0, 4.0], [8.0, 3.0, 4.0]], [[7.0, 8.0, 6.0], [5.0, 7.0, 3.0], [8.0, 6.0, 4.0], [4.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 6.0, 3.0], [7.0, 2.0, 2.0], [8.0, 6.0, 7.0], [8.0, 5.0, 4.0]], [[3.0, 7.0, 1.0], [3.0, 
2.0, 4.0], [8.0, 2.0, 6.0], [8.0, 7.0, 7.0]], [[6.0, 3.0, 8.0], [8.0, 4.0, 7.0], [4.0, 6.0, 2.0], [7.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RCL(self):
      arg=Data(numpy.array([[[5.0, 7.0, 3.0], [7.0, 3.0, 2.0], [6.0, 7.0, 8.0], [3.0, 2.0, 7.0]], [[1.0, 1.0, 1.0], [5.0, 2.0, 
1.0], [2.0, 3.0, 1.0], [4.0, 1.0, 1.0]], [[2.0, 2.0, 2.0], [5.0, 8.0, 3.0], [6.0, 6.0, 8.0], [6.0, 7.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 3.0], [1.0, 4.0, 5.0], [7.0, 6.0, 6.0], [6.0, 3.0, 7.0]], [[5.0, 2.0, 8.0], 
[7.0, 7.0, 1.0], [8.0, 3.0, 1.0], [3.0, 7.0, 4.0]], [[3.0, 5.0, 4.0], [3.0, 6.0, 8.0], [4.0, 8.0, 1.0], [3.0, 3.0, 7.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0], [6.0], [6.0], [1.0]]])+(1.-msk_val)*numpy.array([[[7.0], [1.0], [6.0], [1.0]]])
      arg[1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 3.0], [7.0, 3.0, 2.0], [6.0, 7.0, 8.0], [3.0, 2.0, 7.0]], [[1.0, 1.0, 3.0], [5.0, 
2.0, 6.0], [2.0, 3.0, 6.0], [4.0, 1.0, 1.0]], [[2.0, 2.0, 2.0], [5.0, 8.0, 3.0], [6.0, 6.0, 8.0], [6.0, 7.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 3.0], [1.0, 4.0, 5.0], [7.0, 6.0, 6.0], [6.0, 3.0, 7.0]], [[5.0, 2.0, 7.0], [7.0, 
7.0, 1.0], [8.0, 3.0, 6.0], [3.0, 7.0, 1.0]], [[3.0, 5.0, 4.0], [3.0, 6.0, 8.0], [4.0, 8.0, 1.0], [3.0, 3.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RCC(self):
      arg=Data(numpy.array([[[6.0, 2.0, 3.0], [7.0, 3.0, 2.0], [2.0, 2.0, 4.0], [1.0, 2.0, 8.0]], [[2.0, 7.0, 8.0], [6.0, 5.0, 
8.0], [3.0, 4.0, 4.0], [4.0, 2.0, 7.0]], [[1.0, 8.0, 1.0], [8.0, 7.0, 4.0], [8.0, 4.0, 4.0], [8.0, 5.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 1.0, 7.0], [7.0, 1.0, 7.0], [1.0, 1.0, 4.0], [4.0, 2.0, 2.0]], [[6.0, 4.0, 6.0], 
[8.0, 6.0, 8.0], [5.0, 3.0, 8.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 5.0], [7.0, 1.0, 3.0], [7.0, 6.0, 1.0], [5.0, 1.0, 4.0]]]))
      val=numpy.array([[[3.0, 4.0, 7.0], [5.0, 2.0, 5.0], [6.0, 4.0, 4.0], [5.0, 5.0, 4.0]]])
      arg[1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 3.0], [7.0, 3.0, 2.0], [2.0, 2.0, 4.0], [1.0, 2.0, 8.0]], [[3.0, 4.0, 7.0], [5.0, 
2.0, 5.0], [6.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[1.0, 8.0, 1.0], [8.0, 7.0, 4.0], [8.0, 4.0, 4.0], [8.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 7.0], [7.0, 1.0, 7.0], [1.0, 1.0, 4.0], [4.0, 2.0, 2.0]], [[3.0, 4.0, 7.0], [5.0, 
2.0, 5.0], [6.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [7.0, 1.0, 3.0], [7.0, 6.0, 1.0], [5.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RCC(self):
      arg=Data(numpy.array([[[5.0, 6.0, 5.0], [2.0, 2.0, 4.0], [6.0, 2.0, 2.0], [8.0, 7.0, 3.0]], [[2.0, 3.0, 1.0], [5.0, 3.0, 
2.0], [7.0, 4.0, 4.0], [4.0, 2.0, 1.0]], [[2.0, 3.0, 4.0], [2.0, 7.0, 2.0], [3.0, 3.0, 6.0], [4.0, 7.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 4.0, 2.0], [8.0, 8.0, 2.0], [6.0, 1.0, 6.0], [6.0, 7.0, 3.0]], [[6.0, 2.0, 2.0], 
[4.0, 5.0, 3.0], [5.0, 5.0, 7.0], [5.0, 7.0, 6.0]], [[6.0, 2.0, 2.0], [4.0, 5.0, 8.0], [1.0, 5.0, 8.0], [7.0, 4.0, 6.0]]]))
      val=Data(numpy.array([[[3.0, 8.0, 5.0], [2.0, 4.0, 5.0], [6.0, 5.0, 7.0], [5.0, 4.0, 1.0]]]),self.functionspace)
      arg[1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 5.0], [2.0, 2.0, 4.0], [6.0, 2.0, 2.0], [8.0, 7.0, 3.0]], [[3.0, 8.0, 5.0], [2.0, 
4.0, 5.0], [6.0, 5.0, 7.0], [5.0, 4.0, 1.0]], [[2.0, 3.0, 4.0], [2.0, 7.0, 2.0], [3.0, 3.0, 6.0], [4.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 2.0], [8.0, 8.0, 2.0], [6.0, 1.0, 6.0], [6.0, 7.0, 3.0]], [[3.0, 8.0, 5.0], [2.0, 
4.0, 5.0], [6.0, 5.0, 7.0], [5.0, 4.0, 1.0]], [[6.0, 2.0, 2.0], [4.0, 5.0, 8.0], [1.0, 5.0, 8.0], [7.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RCC(self):
      arg=Data(numpy.array([[[3.0, 8.0, 8.0], [6.0, 4.0, 7.0], [8.0, 8.0, 8.0], [6.0, 3.0, 8.0]], [[3.0, 1.0, 5.0], [4.0, 4.0, 
8.0], [6.0, 1.0, 1.0], [3.0, 5.0, 7.0]], [[5.0, 1.0, 1.0], [8.0, 3.0, 3.0], [1.0, 4.0, 7.0], [5.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 7.0, 6.0], [3.0, 2.0, 8.0], [1.0, 6.0, 8.0], [1.0, 1.0, 6.0]], [[5.0, 7.0, 5.0], 
[7.0, 2.0, 2.0], [8.0, 2.0, 8.0], [5.0, 7.0, 5.0]], [[2.0, 5.0, 4.0], [8.0, 3.0, 5.0], [6.0, 5.0, 6.0], [8.0, 3.0, 4.0]]]))
      val=Data(numpy.array([[[4.0, 4.0, 4.0], [5.0, 3.0, 8.0], [2.0, 6.0, 7.0], [8.0, 2.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 4.0, 7.0], [8.0, 4.0, 5.0], [1.0, 1.0, 7.0], [4.0, 3.0, 5.0]]]))
      arg[1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 8.0], [6.0, 4.0, 7.0], [8.0, 8.0, 8.0], [6.0, 3.0, 8.0]], [[4.0, 4.0, 4.0], [5.0, 
3.0, 8.0], [2.0, 6.0, 7.0], [8.0, 2.0, 8.0]], [[5.0, 1.0, 1.0], [8.0, 3.0, 3.0], [1.0, 4.0, 7.0], [5.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 7.0, 6.0], [3.0, 2.0, 8.0], [1.0, 6.0, 8.0], [1.0, 1.0, 6.0]], [[5.0, 4.0, 7.0], [8.0, 
4.0, 5.0], [1.0, 1.0, 7.0], [4.0, 3.0, 5.0]], [[2.0, 5.0, 4.0], [8.0, 3.0, 5.0], [6.0, 5.0, 6.0], [8.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RCC(self):
      arg=Data(numpy.array([[[3.0, 2.0, 3.0], [2.0, 3.0, 6.0], [1.0, 1.0, 1.0], [6.0, 5.0, 6.0]], [[2.0, 4.0, 5.0], [3.0, 4.0, 
1.0], [3.0, 8.0, 3.0], [3.0, 4.0, 2.0]], [[5.0, 5.0, 5.0], [4.0, 2.0, 1.0], [7.0, 2.0, 1.0], [3.0, 6.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 2.0, 5.0], [2.0, 1.0, 8.0], [4.0, 7.0, 4.0], [3.0, 2.0, 5.0]], [[5.0, 4.0, 7.0], 
[2.0, 2.0, 2.0], [7.0, 2.0, 3.0], [2.0, 4.0, 2.0]], [[8.0, 6.0, 1.0], [2.0, 4.0, 4.0], [5.0, 1.0, 8.0], [7.0, 6.0, 5.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 4.0, 1.0], [5.0, 7.0, 6.0], [7.0, 1.0, 8.0], [7.0, 1.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[5.0, 6.0, 3.0], [8.0, 4.0, 3.0], [4.0, 6.0, 7.0], [3.0, 4.0, 2.0]]])
      arg[1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 3.0], [2.0, 3.0, 6.0], [1.0, 1.0, 1.0], [6.0, 5.0, 6.0]], [[3.0, 4.0, 1.0], [5.0, 
7.0, 6.0], [7.0, 1.0, 8.0], [7.0, 1.0, 4.0]], [[5.0, 5.0, 5.0], [4.0, 2.0, 1.0], [7.0, 2.0, 1.0], [3.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 5.0], [2.0, 1.0, 8.0], [4.0, 7.0, 4.0], [3.0, 2.0, 5.0]], [[5.0, 6.0, 3.0], [8.0, 
4.0, 3.0], [4.0, 6.0, 7.0], [3.0, 4.0, 2.0]], [[8.0, 6.0, 1.0], [2.0, 4.0, 4.0], [5.0, 1.0, 8.0], [7.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_URN(self):
      arg=Data(numpy.array([[[1.0, 7.0, 2.0], [3.0, 3.0, 5.0], [5.0, 1.0, 3.0], [4.0, 1.0, 8.0]], [[8.0, 5.0, 3.0], [3.0, 1.0, 
3.0], [2.0, 4.0, 3.0], [2.0, 2.0, 8.0]], [[2.0, 8.0, 5.0], [7.0, 6.0, 8.0], [2.0, 5.0, 6.0], [7.0, 2.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 1.0, 8.0], [7.0, 3.0, 7.0], [6.0, 1.0, 4.0], [2.0, 5.0, 4.0]], [[8.0, 4.0, 1.0], 
[8.0, 1.0, 8.0], [7.0, 1.0, 2.0], [2.0, 4.0, 4.0]], [[6.0, 6.0, 7.0], [6.0, 6.0, 7.0], [8.0, 7.0, 4.0], [7.0, 5.0, 4.0]]]))
      val=numpy.array([[[5.0, 4.0, 7.0], [5.0, 4.0, 6.0]], [[8.0, 1.0, 2.0], [7.0, 4.0, 1.0]]])
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 2.0], [5.0, 4.0, 7.0], [5.0, 4.0, 6.0], [4.0, 1.0, 8.0]], [[8.0, 5.0, 3.0], [8.0, 
1.0, 2.0], [7.0, 4.0, 1.0], [2.0, 2.0, 8.0]], [[2.0, 8.0, 5.0], [7.0, 6.0, 8.0], [2.0, 5.0, 6.0], [7.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 8.0], [5.0, 4.0, 7.0], [5.0, 4.0, 6.0], [2.0, 5.0, 4.0]], [[8.0, 4.0, 1.0], [8.0, 
1.0, 2.0], [7.0, 4.0, 1.0], [2.0, 4.0, 4.0]], [[6.0, 6.0, 7.0], [6.0, 6.0, 7.0], [8.0, 7.0, 4.0], [7.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_URN(self):
      arg=Data(numpy.array([[[3.0, 6.0, 1.0], [8.0, 3.0, 3.0], [6.0, 6.0, 1.0], [1.0, 8.0, 2.0]], [[2.0, 2.0, 3.0], [3.0, 3.0, 
6.0], [2.0, 3.0, 4.0], [8.0, 7.0, 7.0]], [[1.0, 7.0, 3.0], [7.0, 5.0, 6.0], [1.0, 4.0, 7.0], [4.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 1.0, 5.0], [2.0, 4.0, 2.0], [8.0, 7.0, 3.0], [1.0, 2.0, 1.0]], [[1.0, 7.0, 8.0], 
[7.0, 4.0, 7.0], [7.0, 1.0, 7.0], [7.0, 7.0, 3.0]], [[2.0, 7.0, 6.0], [8.0, 1.0, 8.0], [2.0, 6.0, 7.0], [1.0, 5.0, 3.0]]]))
      val=Data(numpy.array([[[5.0, 1.0, 1.0], [1.0, 2.0, 4.0]], [[1.0, 8.0, 6.0], [3.0, 7.0, 5.0]]]),self.functionspace)
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 1.0], [5.0, 1.0, 1.0], [1.0, 2.0, 4.0], [1.0, 8.0, 2.0]], [[2.0, 2.0, 3.0], [1.0, 
8.0, 6.0], [3.0, 7.0, 5.0], [8.0, 7.0, 7.0]], [[1.0, 7.0, 3.0], [7.0, 5.0, 6.0], [1.0, 4.0, 7.0], [4.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 5.0], [5.0, 1.0, 1.0], [1.0, 2.0, 4.0], [1.0, 2.0, 1.0]], [[1.0, 7.0, 8.0], [1.0, 
8.0, 6.0], [3.0, 7.0, 5.0], [7.0, 7.0, 3.0]], [[2.0, 7.0, 6.0], [8.0, 1.0, 8.0], [2.0, 6.0, 7.0], [1.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_URN(self):
      arg=Data(numpy.array([[[6.0, 4.0, 7.0], [4.0, 6.0, 4.0], [2.0, 6.0, 6.0], [1.0, 2.0, 4.0]], [[4.0, 7.0, 5.0], [3.0, 6.0, 
7.0], [7.0, 3.0, 7.0], [8.0, 2.0, 4.0]], [[1.0, 8.0, 5.0], [3.0, 8.0, 3.0], [4.0, 5.0, 5.0], [5.0, 4.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 2.0], [6.0, 5.0, 7.0], [7.0, 2.0, 5.0], [1.0, 7.0, 1.0]], [[2.0, 4.0, 8.0], 
[7.0, 5.0, 1.0], [5.0, 6.0, 5.0], [8.0, 8.0, 8.0]], [[4.0, 1.0, 8.0], [1.0, 7.0, 6.0], [2.0, 6.0, 5.0], [6.0, 2.0, 7.0]]]))
      val=Data(numpy.array([[[6.0, 6.0, 5.0], [4.0, 6.0, 4.0]], [[7.0, 1.0, 1.0], [3.0, 5.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 6.0, 1.0], [4.0, 6.0, 4.0]], [[8.0, 4.0, 1.0], [8.0, 7.0, 1.0]]]))
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 7.0], [6.0, 6.0, 5.0], [4.0, 6.0, 4.0], [1.0, 2.0, 4.0]], [[4.0, 7.0, 5.0], [7.0, 
1.0, 1.0], [3.0, 5.0, 1.0], [8.0, 2.0, 4.0]], [[1.0, 8.0, 5.0], [3.0, 8.0, 3.0], [4.0, 5.0, 5.0], [5.0, 4.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 2.0], [8.0, 6.0, 1.0], [4.0, 6.0, 4.0], [1.0, 7.0, 1.0]], [[2.0, 4.0, 8.0], [8.0, 
4.0, 1.0], [8.0, 7.0, 1.0], [8.0, 8.0, 8.0]], [[4.0, 1.0, 8.0], [1.0, 7.0, 6.0], [2.0, 6.0, 5.0], [6.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_URN(self):
      arg=Data(numpy.array([[[6.0, 8.0, 5.0], [5.0, 7.0, 3.0], [4.0, 2.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 3.0, 2.0], [8.0, 3.0, 
2.0], [7.0, 3.0, 3.0], [5.0, 7.0, 5.0]], [[2.0, 8.0, 6.0], [1.0, 3.0, 7.0], [2.0, 7.0, 5.0], [1.0, 6.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 8.0], [1.0, 3.0, 8.0], [3.0, 7.0, 1.0], [3.0, 4.0, 3.0]], [[3.0, 5.0, 1.0], 
[8.0, 6.0, 5.0], [3.0, 6.0, 5.0], [6.0, 3.0, 1.0]], [[5.0, 6.0, 7.0], [2.0, 6.0, 1.0], [7.0, 6.0, 4.0], [8.0, 6.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 2.0, 6.0], [1.0, 7.0, 4.0]], [[8.0, 5.0, 3.0], [5.0, 5.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[5.0, 5.0, 5.0], [1.0, 3.0, 7.0]], [[2.0, 8.0, 2.0], [3.0, 1.0, 4.0]]])
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 5.0], [3.0, 2.0, 6.0], [1.0, 7.0, 4.0], [5.0, 7.0, 3.0]], [[5.0, 3.0, 2.0], [8.0, 
5.0, 3.0], [5.0, 5.0, 4.0], [5.0, 7.0, 5.0]], [[2.0, 8.0, 6.0], [1.0, 3.0, 7.0], [2.0, 7.0, 5.0], [1.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 8.0], [5.0, 5.0, 5.0], [1.0, 3.0, 7.0], [3.0, 4.0, 3.0]], [[3.0, 5.0, 1.0], [2.0, 
8.0, 2.0], [3.0, 1.0, 4.0], [6.0, 3.0, 1.0]], [[5.0, 6.0, 7.0], [2.0, 6.0, 1.0], [7.0, 6.0, 4.0], [8.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CUN(self):
      arg=Data(numpy.array([[[4.0, 7.0, 3.0], [4.0, 6.0, 2.0], [4.0, 6.0, 5.0], [7.0, 7.0, 3.0]], [[2.0, 2.0, 8.0], [8.0, 5.0, 
1.0], [8.0, 4.0, 8.0], [1.0, 7.0, 6.0]], [[7.0, 5.0, 6.0], [7.0, 6.0, 1.0], [4.0, 4.0, 3.0], [1.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 6.0, 8.0], [5.0, 3.0, 1.0], [2.0, 4.0, 8.0], [7.0, 2.0, 5.0]], [[7.0, 8.0, 4.0], 
[5.0, 8.0, 3.0], [8.0, 4.0, 4.0], [2.0, 4.0, 7.0]], [[8.0, 6.0, 7.0], [4.0, 6.0, 8.0], [1.0, 2.0, 2.0], [8.0, 8.0, 6.0]]]))
      val=numpy.array([[[2.0, 2.0, 2.0], [5.0, 3.0, 2.0], [5.0, 4.0, 5.0]], [[4.0, 4.0, 5.0], [7.0, 6.0, 6.0], [6.0, 2.0, 
4.0]], [[3.0, 2.0, 5.0], [1.0, 5.0, 6.0], [4.0, 7.0, 1.0]]])
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 2.0], [5.0, 3.0, 2.0], [5.0, 4.0, 5.0], [7.0, 7.0, 3.0]], [[4.0, 4.0, 5.0], [7.0, 
6.0, 6.0], [6.0, 2.0, 4.0], [1.0, 7.0, 6.0]], [[3.0, 2.0, 5.0], [1.0, 5.0, 6.0], [4.0, 7.0, 1.0], [1.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 2.0], [5.0, 3.0, 2.0], [5.0, 4.0, 5.0], [7.0, 2.0, 5.0]], [[4.0, 4.0, 5.0], [7.0, 
6.0, 6.0], [6.0, 2.0, 4.0], [2.0, 4.0, 7.0]], [[3.0, 2.0, 5.0], [1.0, 5.0, 6.0], [4.0, 7.0, 1.0], [8.0, 8.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CUN(self):
      arg=Data(numpy.array([[[7.0, 4.0, 8.0], [3.0, 1.0, 1.0], [8.0, 3.0, 7.0], [4.0, 7.0, 4.0]], [[3.0, 6.0, 6.0], [1.0, 1.0, 
6.0], [3.0, 3.0, 3.0], [3.0, 7.0, 1.0]], [[4.0, 5.0, 6.0], [1.0, 5.0, 5.0], [2.0, 8.0, 1.0], [2.0, 7.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 6.0, 1.0], [3.0, 5.0, 4.0], [4.0, 3.0, 6.0], [2.0, 3.0, 2.0]], [[3.0, 6.0, 1.0], 
[8.0, 7.0, 6.0], [7.0, 1.0, 2.0], [7.0, 2.0, 2.0]], [[5.0, 3.0, 8.0], [7.0, 6.0, 3.0], [4.0, 2.0, 8.0], [2.0, 7.0, 7.0]]]))
      val=Data(numpy.array([[[1.0, 7.0, 3.0], [6.0, 3.0, 6.0], [8.0, 2.0, 5.0]], [[2.0, 7.0, 6.0], [2.0, 1.0, 8.0], [5.0, 5.0, 
1.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 1.0], [2.0, 4.0, 3.0]]]),self.functionspace)
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 3.0], [6.0, 3.0, 6.0], [8.0, 2.0, 5.0], [4.0, 7.0, 4.0]], [[2.0, 7.0, 6.0], [2.0, 
1.0, 8.0], [5.0, 5.0, 1.0], [3.0, 7.0, 1.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 1.0], [2.0, 4.0, 3.0], [2.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 3.0], [6.0, 3.0, 6.0], [8.0, 2.0, 5.0], [2.0, 3.0, 2.0]], [[2.0, 7.0, 6.0], [2.0, 
1.0, 8.0], [5.0, 5.0, 1.0], [7.0, 2.0, 2.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 1.0], [2.0, 4.0, 3.0], [2.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CUN(self):
      arg=Data(numpy.array([[[2.0, 2.0, 1.0], [3.0, 3.0, 1.0], [2.0, 7.0, 7.0], [4.0, 5.0, 4.0]], [[6.0, 8.0, 5.0], [4.0, 6.0, 
5.0], [7.0, 1.0, 5.0], [8.0, 5.0, 2.0]], [[1.0, 2.0, 6.0], [1.0, 2.0, 3.0], [7.0, 5.0, 7.0], [4.0, 5.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 7.0, 1.0], [7.0, 6.0, 1.0], [1.0, 1.0, 1.0], [7.0, 6.0, 6.0]], [[5.0, 3.0, 2.0], 
[4.0, 7.0, 3.0], [3.0, 1.0, 4.0], [7.0, 5.0, 8.0]], [[1.0, 5.0, 5.0], [8.0, 4.0, 7.0], [5.0, 2.0, 2.0], [2.0, 5.0, 2.0]]]))
      val=Data(numpy.array([[[2.0, 7.0, 3.0], [3.0, 7.0, 7.0], [6.0, 5.0, 1.0]], [[8.0, 2.0, 1.0], [2.0, 7.0, 2.0], [1.0, 8.0, 
6.0]], [[7.0, 5.0, 1.0], [3.0, 6.0, 5.0], [2.0, 6.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 8.0, 6.0], [8.0, 4.0, 3.0], [8.0, 6.0, 5.0]], [[1.0, 1.0, 8.0], [6.0, 2.0, 1.0], 
[5.0, 1.0, 6.0]], [[8.0, 7.0, 4.0], [2.0, 1.0, 6.0], [6.0, 3.0, 3.0]]]))
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 3.0], [3.0, 7.0, 7.0], [6.0, 5.0, 1.0], [4.0, 5.0, 4.0]], [[8.0, 2.0, 1.0], [2.0, 
7.0, 2.0], [1.0, 8.0, 6.0], [8.0, 5.0, 2.0]], [[7.0, 5.0, 1.0], [3.0, 6.0, 5.0], [2.0, 6.0, 8.0], [4.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 6.0], [8.0, 4.0, 3.0], [8.0, 6.0, 5.0], [7.0, 6.0, 6.0]], [[1.0, 1.0, 8.0], [6.0, 
2.0, 1.0], [5.0, 1.0, 6.0], [7.0, 5.0, 8.0]], [[8.0, 7.0, 4.0], [2.0, 1.0, 6.0], [6.0, 3.0, 3.0], [2.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CUN(self):
      arg=Data(numpy.array([[[8.0, 7.0, 8.0], [2.0, 6.0, 2.0], [5.0, 2.0, 5.0], [6.0, 2.0, 1.0]], [[8.0, 1.0, 3.0], [4.0, 3.0, 
6.0], [6.0, 1.0, 6.0], [3.0, 1.0, 7.0]], [[7.0, 4.0, 4.0], [6.0, 1.0, 6.0], [2.0, 5.0, 6.0], [2.0, 6.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 5.0, 8.0], [1.0, 8.0, 4.0], [3.0, 5.0, 5.0], [1.0, 5.0, 7.0]], [[2.0, 6.0, 8.0], 
[4.0, 5.0, 5.0], [3.0, 4.0, 8.0], [2.0, 2.0, 7.0]], [[3.0, 6.0, 1.0], [1.0, 8.0, 4.0], [1.0, 6.0, 5.0], [7.0, 1.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 3.0, 5.0], [5.0, 4.0, 5.0], [7.0, 8.0, 8.0]], [[8.0, 4.0, 8.0], [5.0, 8.0, 4.0], [2.0, 
6.0, 3.0]], [[1.0, 4.0, 7.0], [2.0, 8.0, 5.0], [3.0, 4.0, 4.0]]])+(1.-msk_val)*numpy.array([[[4.0, 7.0, 2.0], [2.0, 7.0, 1.0], 
[7.0, 7.0, 5.0]], [[1.0, 3.0, 3.0], [6.0, 1.0, 2.0], [4.0, 4.0, 7.0]], [[3.0, 8.0, 6.0], [6.0, 7.0, 7.0], [3.0, 8.0, 7.0]]])
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 5.0], [5.0, 4.0, 5.0], [7.0, 8.0, 8.0], [6.0, 2.0, 1.0]], [[8.0, 4.0, 8.0], [5.0, 
8.0, 4.0], [2.0, 6.0, 3.0], [3.0, 1.0, 7.0]], [[1.0, 4.0, 7.0], [2.0, 8.0, 5.0], [3.0, 4.0, 4.0], [2.0, 6.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 7.0, 2.0], [2.0, 7.0, 1.0], [7.0, 7.0, 5.0], [1.0, 5.0, 7.0]], [[1.0, 3.0, 3.0], [6.0, 
1.0, 2.0], [4.0, 4.0, 7.0], [2.0, 2.0, 7.0]], [[3.0, 8.0, 6.0], [6.0, 7.0, 7.0], [3.0, 8.0, 7.0], [7.0, 1.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RRU(self):
      arg=Data(numpy.array([[[5.0, 2.0, 8.0], [8.0, 1.0, 8.0], [1.0, 4.0, 1.0], [2.0, 8.0, 3.0]], [[5.0, 8.0, 3.0], [8.0, 2.0, 
5.0], [2.0, 8.0, 2.0], [5.0, 6.0, 2.0]], [[3.0, 3.0, 7.0], [1.0, 1.0, 4.0], [7.0, 3.0, 5.0], [3.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 8.0, 7.0], [3.0, 8.0, 7.0], [3.0, 8.0, 8.0], [2.0, 8.0, 3.0]], [[5.0, 6.0, 5.0], 
[8.0, 1.0, 2.0], [5.0, 3.0, 4.0], [4.0, 7.0, 5.0]], [[6.0, 2.0, 2.0], [4.0, 1.0, 7.0], [7.0, 1.0, 6.0], [6.0, 7.0, 8.0]]]))
      val=numpy.array([[[1.0, 6.0], [5.0, 6.0]]])
      arg[1:2,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 8.0], [8.0, 1.0, 8.0], [1.0, 4.0, 1.0], [2.0, 8.0, 3.0]], [[5.0, 8.0, 3.0], [1.0, 
6.0, 5.0], [5.0, 6.0, 2.0], [5.0, 6.0, 2.0]], [[3.0, 3.0, 7.0], [1.0, 1.0, 4.0], [7.0, 3.0, 5.0], [3.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 7.0], [3.0, 8.0, 7.0], [3.0, 8.0, 8.0], [2.0, 8.0, 3.0]], [[5.0, 6.0, 5.0], [1.0, 
6.0, 2.0], [5.0, 6.0, 4.0], [4.0, 7.0, 5.0]], [[6.0, 2.0, 2.0], [4.0, 1.0, 7.0], [7.0, 1.0, 6.0], [6.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RRU(self):
      arg=Data(numpy.array([[[5.0, 6.0, 4.0], [3.0, 8.0, 1.0], [1.0, 1.0, 8.0], [4.0, 2.0, 3.0]], [[6.0, 4.0, 1.0], [3.0, 6.0, 
7.0], [4.0, 7.0, 2.0], [1.0, 4.0, 5.0]], [[5.0, 5.0, 3.0], [6.0, 6.0, 6.0], [5.0, 3.0, 8.0], [3.0, 5.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 2.0, 2.0], [8.0, 2.0, 3.0], [3.0, 1.0, 1.0], [4.0, 5.0, 4.0]], [[1.0, 1.0, 7.0], 
[4.0, 2.0, 4.0], [1.0, 6.0, 2.0], [5.0, 1.0, 3.0]], [[8.0, 2.0, 1.0], [2.0, 4.0, 6.0], [7.0, 6.0, 6.0], [1.0, 5.0, 6.0]]]))
      val=Data(numpy.array([[[8.0, 8.0], [6.0, 5.0]]]),self.functionspace)
      arg[1:2,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 4.0], [3.0, 8.0, 1.0], [1.0, 1.0, 8.0], [4.0, 2.0, 3.0]], [[6.0, 4.0, 1.0], [8.0, 
8.0, 7.0], [6.0, 5.0, 2.0], [1.0, 4.0, 5.0]], [[5.0, 5.0, 3.0], [6.0, 6.0, 6.0], [5.0, 3.0, 8.0], [3.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 2.0], [8.0, 2.0, 3.0], [3.0, 1.0, 1.0], [4.0, 5.0, 4.0]], [[1.0, 1.0, 7.0], [8.0, 
8.0, 4.0], [6.0, 5.0, 2.0], [5.0, 1.0, 3.0]], [[8.0, 2.0, 1.0], [2.0, 4.0, 6.0], [7.0, 6.0, 6.0], [1.0, 5.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RRU(self):
      arg=Data(numpy.array([[[2.0, 5.0, 5.0], [6.0, 4.0, 3.0], [8.0, 4.0, 2.0], [5.0, 4.0, 4.0]], [[8.0, 8.0, 6.0], [5.0, 1.0, 
2.0], [8.0, 3.0, 3.0], [6.0, 3.0, 1.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 3.0], [7.0, 5.0, 4.0], [4.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 8.0, 1.0], [2.0, 1.0, 7.0], [7.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[5.0, 8.0, 7.0], 
[2.0, 3.0, 6.0], [3.0, 2.0, 6.0], [7.0, 4.0, 7.0]], [[6.0, 5.0, 7.0], [2.0, 1.0, 2.0], [8.0, 2.0, 8.0], [7.0, 3.0, 2.0]]]))
      val=Data(numpy.array([[[2.0, 4.0], [2.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 8.0], [7.0, 3.0]]]))
      arg[1:2,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 5.0], [6.0, 4.0, 3.0], [8.0, 4.0, 2.0], [5.0, 4.0, 4.0]], [[8.0, 8.0, 6.0], [2.0, 
4.0, 2.0], [2.0, 3.0, 3.0], [6.0, 3.0, 1.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 3.0], [7.0, 5.0, 4.0], [4.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 1.0], [2.0, 1.0, 7.0], [7.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[5.0, 8.0, 7.0], [6.0, 
8.0, 6.0], [7.0, 3.0, 6.0], [7.0, 4.0, 7.0]], [[6.0, 5.0, 7.0], [2.0, 1.0, 2.0], [8.0, 2.0, 8.0], [7.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RRU(self):
      arg=Data(numpy.array([[[7.0, 3.0, 6.0], [7.0, 6.0, 6.0], [7.0, 4.0, 2.0], [1.0, 7.0, 8.0]], [[2.0, 2.0, 6.0], [3.0, 8.0, 
2.0], [8.0, 6.0, 4.0], [2.0, 8.0, 7.0]], [[4.0, 3.0, 3.0], [4.0, 5.0, 2.0], [7.0, 3.0, 4.0], [3.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 3.0, 5.0], [2.0, 6.0, 7.0], [4.0, 6.0, 1.0], [8.0, 2.0, 6.0]], [[3.0, 5.0, 1.0], 
[6.0, 4.0, 7.0], [1.0, 6.0, 2.0], [1.0, 3.0, 8.0]], [[6.0, 1.0, 4.0], [3.0, 4.0, 6.0], [6.0, 3.0, 3.0], [7.0, 2.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[6.0, 2.0], [8.0, 2.0]]])+(1.-msk_val)*numpy.array([[[7.0, 4.0], [6.0, 2.0]]])
      arg[1:2,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 6.0], [7.0, 6.0, 6.0], [7.0, 4.0, 2.0], [1.0, 7.0, 8.0]], [[2.0, 2.0, 6.0], [6.0, 
2.0, 2.0], [8.0, 2.0, 4.0], [2.0, 8.0, 7.0]], [[4.0, 3.0, 3.0], [4.0, 5.0, 2.0], [7.0, 3.0, 4.0], [3.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 5.0], [2.0, 6.0, 7.0], [4.0, 6.0, 1.0], [8.0, 2.0, 6.0]], [[3.0, 5.0, 1.0], [7.0, 
4.0, 7.0], [6.0, 2.0, 2.0], [1.0, 3.0, 8.0]], [[6.0, 1.0, 4.0], [3.0, 4.0, 6.0], [6.0, 3.0, 3.0], [7.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RUL(self):
      arg=Data(numpy.array([[[3.0, 3.0, 5.0], [2.0, 2.0, 7.0], [7.0, 7.0, 3.0], [7.0, 1.0, 3.0]], [[2.0, 6.0, 8.0], [8.0, 5.0, 
5.0], [7.0, 2.0, 3.0], [2.0, 5.0, 7.0]], [[4.0, 8.0, 8.0], [1.0, 7.0, 2.0], [2.0, 2.0, 4.0], [4.0, 3.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 5.0, 2.0], [1.0, 8.0, 2.0], [8.0, 4.0, 3.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 7.0], 
[8.0, 6.0, 1.0], [8.0, 1.0, 8.0], [2.0, 4.0, 4.0]], [[3.0, 2.0, 6.0], [1.0, 6.0, 8.0], [3.0, 2.0, 5.0], [6.0, 6.0, 1.0]]]))
      val=numpy.array([[[7.0], [4.0], [8.0]]])
      arg[1:2,:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 5.0], [2.0, 2.0, 7.0], [7.0, 7.0, 3.0], [7.0, 1.0, 3.0]], [[2.0, 6.0, 7.0], [8.0, 
5.0, 4.0], [7.0, 2.0, 8.0], [2.0, 5.0, 7.0]], [[4.0, 8.0, 8.0], [1.0, 7.0, 2.0], [2.0, 2.0, 4.0], [4.0, 3.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 2.0], [1.0, 8.0, 2.0], [8.0, 4.0, 3.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 7.0], [8.0, 
6.0, 4.0], [8.0, 1.0, 8.0], [2.0, 4.0, 4.0]], [[3.0, 2.0, 6.0], [1.0, 6.0, 8.0], [3.0, 2.0, 5.0], [6.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RUL(self):
      arg=Data(numpy.array([[[6.0, 4.0, 8.0], [5.0, 6.0, 5.0], [7.0, 3.0, 6.0], [5.0, 4.0, 7.0]], [[8.0, 1.0, 1.0], [5.0, 8.0, 
4.0], [1.0, 3.0, 4.0], [4.0, 5.0, 5.0]], [[5.0, 6.0, 1.0], [2.0, 7.0, 2.0], [7.0, 3.0, 5.0], [3.0, 1.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 4.0, 1.0], [7.0, 8.0, 3.0], [7.0, 7.0, 1.0], [1.0, 2.0, 8.0]], [[6.0, 4.0, 5.0], 
[4.0, 4.0, 3.0], [6.0, 7.0, 3.0], [6.0, 7.0, 4.0]], [[2.0, 1.0, 7.0], [2.0, 3.0, 7.0], [6.0, 1.0, 2.0], [6.0, 1.0, 1.0]]]))
      val=Data(numpy.array([[[1.0], [1.0], [5.0]]]),self.functionspace)
      arg[1:2,:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 8.0], [5.0, 6.0, 5.0], [7.0, 3.0, 6.0], [5.0, 4.0, 7.0]], [[8.0, 1.0, 1.0], [5.0, 
8.0, 1.0], [1.0, 3.0, 5.0], [4.0, 5.0, 5.0]], [[5.0, 6.0, 1.0], [2.0, 7.0, 2.0], [7.0, 3.0, 5.0], [3.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 1.0], [7.0, 8.0, 3.0], [7.0, 7.0, 1.0], [1.0, 2.0, 8.0]], [[6.0, 4.0, 1.0], [4.0, 
4.0, 1.0], [6.0, 7.0, 5.0], [6.0, 7.0, 4.0]], [[2.0, 1.0, 7.0], [2.0, 3.0, 7.0], [6.0, 1.0, 2.0], [6.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RUL(self):
      arg=Data(numpy.array([[[4.0, 3.0, 1.0], [7.0, 5.0, 7.0], [1.0, 5.0, 7.0], [4.0, 3.0, 3.0]], [[1.0, 4.0, 4.0], [2.0, 2.0, 
5.0], [5.0, 8.0, 3.0], [3.0, 1.0, 2.0]], [[6.0, 3.0, 7.0], [8.0, 1.0, 2.0], [8.0, 3.0, 3.0], [3.0, 4.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 1.0, 8.0], [1.0, 8.0, 1.0], [6.0, 6.0, 7.0], [2.0, 3.0, 4.0]], [[8.0, 1.0, 6.0], 
[2.0, 4.0, 3.0], [6.0, 5.0, 7.0], [2.0, 1.0, 3.0]], [[2.0, 3.0, 5.0], [2.0, 7.0, 7.0], [3.0, 6.0, 4.0], [5.0, 7.0, 3.0]]]))
      val=Data(numpy.array([[[5.0], [6.0], [2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0], [4.0], [8.0]]]))
      arg[1:2,:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 1.0], [7.0, 5.0, 7.0], [1.0, 5.0, 7.0], [4.0, 3.0, 3.0]], [[1.0, 4.0, 5.0], [2.0, 
2.0, 6.0], [5.0, 8.0, 2.0], [3.0, 1.0, 2.0]], [[6.0, 3.0, 7.0], [8.0, 1.0, 2.0], [8.0, 3.0, 3.0], [3.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 8.0], [1.0, 8.0, 1.0], [6.0, 6.0, 7.0], [2.0, 3.0, 4.0]], [[8.0, 1.0, 4.0], [2.0, 
4.0, 4.0], [6.0, 5.0, 8.0], [2.0, 1.0, 3.0]], [[2.0, 3.0, 5.0], [2.0, 7.0, 7.0], [3.0, 6.0, 4.0], [5.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RUL(self):
      arg=Data(numpy.array([[[8.0, 6.0, 6.0], [3.0, 6.0, 4.0], [3.0, 3.0, 7.0], [1.0, 3.0, 4.0]], [[5.0, 8.0, 4.0], [5.0, 6.0, 
6.0], [8.0, 2.0, 2.0], [7.0, 8.0, 7.0]], [[4.0, 6.0, 3.0], [2.0, 3.0, 4.0], [5.0, 8.0, 8.0], [4.0, 4.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 7.0, 6.0], [3.0, 5.0, 5.0], [1.0, 2.0, 2.0], [4.0, 6.0, 2.0]], [[2.0, 8.0, 8.0], 
[2.0, 4.0, 3.0], [2.0, 8.0, 1.0], [5.0, 6.0, 8.0]], [[2.0, 3.0, 7.0], [5.0, 7.0, 3.0], [8.0, 8.0, 6.0], [7.0, 4.0, 7.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0], [2.0], [6.0]]])+(1.-msk_val)*numpy.array([[[6.0], [2.0], [8.0]]])
      arg[1:2,:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 6.0], [3.0, 6.0, 4.0], [3.0, 3.0, 7.0], [1.0, 3.0, 4.0]], [[5.0, 8.0, 4.0], [5.0, 
6.0, 2.0], [8.0, 2.0, 6.0], [7.0, 8.0, 7.0]], [[4.0, 6.0, 3.0], [2.0, 3.0, 4.0], [5.0, 8.0, 8.0], [4.0, 4.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 6.0], [3.0, 5.0, 5.0], [1.0, 2.0, 2.0], [4.0, 6.0, 2.0]], [[2.0, 8.0, 6.0], [2.0, 
4.0, 2.0], [2.0, 8.0, 8.0], [5.0, 6.0, 8.0]], [[2.0, 3.0, 7.0], [5.0, 7.0, 3.0], [8.0, 8.0, 6.0], [7.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RRR(self):
      arg=Data(numpy.array([[[4.0, 6.0, 8.0], [7.0, 3.0, 8.0], [6.0, 7.0, 7.0], [1.0, 4.0, 3.0]], [[5.0, 6.0, 1.0], [5.0, 3.0, 
2.0], [8.0, 8.0, 3.0], [8.0, 6.0, 7.0]], [[5.0, 4.0, 3.0], [2.0, 2.0, 3.0], [1.0, 3.0, 3.0], [3.0, 7.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 2.0], [8.0, 5.0, 5.0], [8.0, 7.0, 7.0], [5.0, 7.0, 8.0]], [[3.0, 2.0, 1.0], 
[4.0, 2.0, 8.0], [1.0, 2.0, 1.0], [1.0, 7.0, 5.0]], [[3.0, 3.0, 4.0], [2.0, 1.0, 4.0], [2.0, 8.0, 7.0], [1.0, 5.0, 2.0]]]))
      val=numpy.array([[[8.0], [3.0]]])
      arg[1:2,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 8.0], [7.0, 3.0, 8.0], [6.0, 7.0, 7.0], [1.0, 4.0, 3.0]], [[5.0, 6.0, 1.0], [5.0, 
8.0, 2.0], [8.0, 3.0, 3.0], [8.0, 6.0, 7.0]], [[5.0, 4.0, 3.0], [2.0, 2.0, 3.0], [1.0, 3.0, 3.0], [3.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 2.0], [8.0, 5.0, 5.0], [8.0, 7.0, 7.0], [5.0, 7.0, 8.0]], [[3.0, 2.0, 1.0], [4.0, 
8.0, 8.0], [1.0, 3.0, 1.0], [1.0, 7.0, 5.0]], [[3.0, 3.0, 4.0], [2.0, 1.0, 4.0], [2.0, 8.0, 7.0], [1.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RRR(self):
      arg=Data(numpy.array([[[4.0, 2.0, 6.0], [1.0, 7.0, 7.0], [3.0, 2.0, 5.0], [1.0, 6.0, 2.0]], [[7.0, 6.0, 7.0], [5.0, 5.0, 
3.0], [7.0, 5.0, 2.0], [6.0, 3.0, 1.0]], [[7.0, 2.0, 6.0], [2.0, 8.0, 6.0], [6.0, 2.0, 7.0], [5.0, 7.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 2.0, 6.0], [4.0, 2.0, 2.0], [8.0, 7.0, 1.0], [6.0, 8.0, 2.0]], [[1.0, 8.0, 1.0], 
[8.0, 4.0, 2.0], [4.0, 6.0, 2.0], [3.0, 2.0, 3.0]], [[3.0, 3.0, 7.0], [2.0, 5.0, 6.0], [2.0, 5.0, 1.0], [7.0, 2.0, 5.0]]]))
      val=Data(numpy.array([[[5.0], [4.0]]]),self.functionspace)
      arg[1:2,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 6.0], [1.0, 7.0, 7.0], [3.0, 2.0, 5.0], [1.0, 6.0, 2.0]], [[7.0, 6.0, 7.0], [5.0, 
5.0, 3.0], [7.0, 4.0, 2.0], [6.0, 3.0, 1.0]], [[7.0, 2.0, 6.0], [2.0, 8.0, 6.0], [6.0, 2.0, 7.0], [5.0, 7.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 6.0], [4.0, 2.0, 2.0], [8.0, 7.0, 1.0], [6.0, 8.0, 2.0]], [[1.0, 8.0, 1.0], [8.0, 
5.0, 2.0], [4.0, 4.0, 2.0], [3.0, 2.0, 3.0]], [[3.0, 3.0, 7.0], [2.0, 5.0, 6.0], [2.0, 5.0, 1.0], [7.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RRR(self):
      arg=Data(numpy.array([[[3.0, 8.0, 2.0], [3.0, 8.0, 4.0], [6.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[3.0, 8.0, 5.0], [6.0, 2.0, 
8.0], [4.0, 3.0, 1.0], [7.0, 2.0, 5.0]], [[8.0, 8.0, 8.0], [5.0, 4.0, 5.0], [8.0, 2.0, 2.0], [5.0, 7.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 8.0, 6.0], [3.0, 2.0, 8.0], [5.0, 6.0, 6.0], [2.0, 7.0, 2.0]], [[7.0, 4.0, 5.0], 
[4.0, 1.0, 8.0], [3.0, 6.0, 6.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 5.0], [3.0, 2.0, 6.0], [2.0, 6.0, 5.0], [1.0, 6.0, 8.0]]]))
      val=Data(numpy.array([[[5.0], [7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0], [3.0]]]))
      arg[1:2,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 2.0], [3.0, 8.0, 4.0], [6.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[3.0, 8.0, 5.0], [6.0, 
5.0, 8.0], [4.0, 7.0, 1.0], [7.0, 2.0, 5.0]], [[8.0, 8.0, 8.0], [5.0, 4.0, 5.0], [8.0, 2.0, 2.0], [5.0, 7.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 6.0], [3.0, 2.0, 8.0], [5.0, 6.0, 6.0], [2.0, 7.0, 2.0]], [[7.0, 4.0, 5.0], [4.0, 
2.0, 8.0], [3.0, 3.0, 6.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 5.0], [3.0, 2.0, 6.0], [2.0, 6.0, 5.0], [1.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RRR(self):
      arg=Data(numpy.array([[[5.0, 8.0, 7.0], [1.0, 2.0, 2.0], [3.0, 2.0, 5.0], [2.0, 7.0, 2.0]], [[2.0, 7.0, 4.0], [1.0, 1.0, 
8.0], [6.0, 2.0, 6.0], [8.0, 1.0, 8.0]], [[2.0, 4.0, 8.0], [4.0, 5.0, 6.0], [3.0, 6.0, 6.0], [1.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 2.0, 6.0], [7.0, 6.0, 4.0], [7.0, 1.0, 7.0], [5.0, 7.0, 8.0]], [[6.0, 8.0, 5.0], 
[5.0, 8.0, 8.0], [7.0, 6.0, 8.0], [2.0, 3.0, 5.0]], [[5.0, 1.0, 8.0], [1.0, 4.0, 2.0], [8.0, 7.0, 4.0], [6.0, 8.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0], [4.0]]])+(1.-msk_val)*numpy.array([[[1.0], [6.0]]])
      arg[1:2,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 7.0], [1.0, 2.0, 2.0], [3.0, 2.0, 5.0], [2.0, 7.0, 2.0]], [[2.0, 7.0, 4.0], [1.0, 
7.0, 8.0], [6.0, 4.0, 6.0], [8.0, 1.0, 8.0]], [[2.0, 4.0, 8.0], [4.0, 5.0, 6.0], [3.0, 6.0, 6.0], [1.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 6.0], [7.0, 6.0, 4.0], [7.0, 1.0, 7.0], [5.0, 7.0, 8.0]], [[6.0, 8.0, 5.0], [5.0, 
1.0, 8.0], [7.0, 6.0, 8.0], [2.0, 3.0, 5.0]], [[5.0, 1.0, 8.0], [1.0, 4.0, 2.0], [8.0, 7.0, 4.0], [6.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PPP(self):
      arg=Data(numpy.array([[[7.0, 8.0, 5.0], [4.0, 8.0, 5.0], [1.0, 6.0, 6.0], [6.0, 6.0, 7.0]], [[1.0, 3.0, 3.0], [8.0, 8.0, 
4.0], [2.0, 4.0, 7.0], [1.0, 3.0, 1.0]], [[2.0, 3.0, 8.0], [2.0, 7.0, 4.0], [6.0, 7.0, 2.0], [7.0, 3.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 1.0, 8.0], [7.0, 3.0, 5.0], [6.0, 6.0, 2.0], [4.0, 6.0, 2.0]], [[1.0, 4.0, 8.0], 
[5.0, 4.0, 1.0], [7.0, 6.0, 6.0], [1.0, 7.0, 4.0]], [[4.0, 1.0, 7.0], [1.0, 7.0, 1.0], [6.0, 5.0, 5.0], [6.0, 8.0, 4.0]]]))
      val=numpy.array(1.0)
      arg[1,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 5.0], [4.0, 8.0, 5.0], [1.0, 6.0, 6.0], [6.0, 6.0, 7.0]], [[1.0, 3.0, 3.0], [8.0, 
8.0, 4.0], [2.0, 1.0, 7.0], [1.0, 3.0, 1.0]], [[2.0, 3.0, 8.0], [2.0, 7.0, 4.0], [6.0, 7.0, 2.0], [7.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 8.0], [7.0, 3.0, 5.0], [6.0, 6.0, 2.0], [4.0, 6.0, 2.0]], [[1.0, 4.0, 8.0], [5.0, 
4.0, 1.0], [7.0, 1.0, 6.0], [1.0, 7.0, 4.0]], [[4.0, 1.0, 7.0], [1.0, 7.0, 1.0], [6.0, 5.0, 5.0], [6.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PPP(self):
      arg=Data(numpy.array([[[2.0, 6.0, 7.0], [7.0, 4.0, 8.0], [4.0, 1.0, 4.0], [3.0, 1.0, 7.0]], [[3.0, 1.0, 3.0], [8.0, 5.0, 
4.0], [6.0, 8.0, 4.0], [7.0, 8.0, 2.0]], [[7.0, 7.0, 7.0], [8.0, 8.0, 2.0], [1.0, 4.0, 1.0], [8.0, 4.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 5.0, 4.0], [8.0, 6.0, 8.0], [4.0, 3.0, 6.0], [7.0, 6.0, 3.0]], [[4.0, 5.0, 4.0], 
[3.0, 2.0, 4.0], [5.0, 1.0, 1.0], [6.0, 6.0, 4.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 6.0], [8.0, 8.0, 6.0], [3.0, 1.0, 6.0]]]))
      val=Data(1.0,self.functionspace)
      arg[1,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 6.0, 7.0], [7.0, 4.0, 8.0], [4.0, 1.0, 4.0], [3.0, 1.0, 7.0]], [[3.0, 1.0, 3.0], [8.0, 
5.0, 4.0], [6.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[7.0, 7.0, 7.0], [8.0, 8.0, 2.0], [1.0, 4.0, 1.0], [8.0, 4.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 5.0, 4.0], [8.0, 6.0, 8.0], [4.0, 3.0, 6.0], [7.0, 6.0, 3.0]], [[4.0, 5.0, 4.0], [3.0, 
2.0, 4.0], [5.0, 1.0, 1.0], [6.0, 6.0, 4.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 6.0], [8.0, 8.0, 6.0], [3.0, 1.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PPP(self):
      arg=Data(numpy.array([[[8.0, 7.0, 7.0], [8.0, 3.0, 7.0], [2.0, 2.0, 3.0], [1.0, 8.0, 5.0]], [[1.0, 6.0, 3.0], [5.0, 1.0, 
4.0], [7.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[1.0, 2.0, 5.0], [7.0, 7.0, 4.0], [3.0, 8.0, 5.0], [2.0, 5.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 6.0, 1.0], [2.0, 1.0, 4.0], [6.0, 2.0, 5.0], [8.0, 5.0, 8.0]], [[7.0, 4.0, 5.0], 
[6.0, 2.0, 5.0], [4.0, 8.0, 2.0], [6.0, 8.0, 8.0]], [[8.0, 4.0, 6.0], [5.0, 3.0, 7.0], [5.0, 6.0, 1.0], [8.0, 6.0, 6.0]]]))
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[1,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 7.0, 7.0], [8.0, 3.0, 7.0], [2.0, 2.0, 3.0], [1.0, 8.0, 5.0]], [[1.0, 6.0, 3.0], [5.0, 
1.0, 4.0], [7.0, 1.0, 7.0], [4.0, 4.0, 7.0]], [[1.0, 2.0, 5.0], [7.0, 7.0, 4.0], [3.0, 8.0, 5.0], [2.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 1.0], [2.0, 1.0, 4.0], [6.0, 2.0, 5.0], [8.0, 5.0, 8.0]], [[7.0, 4.0, 5.0], [6.0, 
2.0, 5.0], [4.0, 1.0, 2.0], [6.0, 8.0, 8.0]], [[8.0, 4.0, 6.0], [5.0, 3.0, 7.0], [5.0, 6.0, 1.0], [8.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PPP(self):
      arg=Data(numpy.array([[[3.0, 2.0, 3.0], [8.0, 3.0, 1.0], [6.0, 8.0, 3.0], [2.0, 2.0, 3.0]], [[4.0, 1.0, 3.0], [7.0, 5.0, 
8.0], [5.0, 6.0, 8.0], [3.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], [4.0, 7.0, 1.0], [6.0, 6.0, 1.0], [5.0, 4.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 7.0, 3.0], [7.0, 7.0, 7.0], [8.0, 4.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 3.0, 5.0], 
[3.0, 6.0, 8.0], [4.0, 3.0, 4.0], [7.0, 3.0, 1.0]], [[1.0, 6.0, 8.0], [2.0, 2.0, 5.0], [7.0, 5.0, 6.0], [5.0, 3.0, 5.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=(1.-msk_val)*(1.0)+msk_val*(1.0)
      arg[1,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 3.0], [8.0, 3.0, 1.0], [6.0, 8.0, 3.0], [2.0, 2.0, 3.0]], [[4.0, 1.0, 3.0], [7.0, 
5.0, 8.0], [5.0, 1.0, 8.0], [3.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], [4.0, 7.0, 1.0], [6.0, 6.0, 1.0], [5.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 3.0], [7.0, 7.0, 7.0], [8.0, 4.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 3.0, 5.0], [3.0, 
6.0, 8.0], [4.0, 1.0, 4.0], [7.0, 3.0, 1.0]], [[1.0, 6.0, 8.0], [2.0, 2.0, 5.0], [7.0, 5.0, 6.0], [5.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LUU(self):
      arg=Data(numpy.array([[[4.0, 4.0, 1.0], [2.0, 1.0, 5.0], [1.0, 8.0, 5.0], [2.0, 4.0, 2.0]], [[7.0, 5.0, 7.0], [1.0, 4.0, 
1.0], [3.0, 4.0, 2.0], [1.0, 4.0, 2.0]], [[8.0, 4.0, 2.0], [6.0, 3.0, 8.0], [1.0, 5.0, 5.0], [7.0, 5.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 3.0, 6.0], [6.0, 4.0, 4.0], [3.0, 5.0, 7.0], [7.0, 7.0, 1.0]], [[5.0, 8.0, 4.0], 
[1.0, 6.0, 8.0], [3.0, 3.0, 6.0], [4.0, 1.0, 3.0]], [[5.0, 3.0, 8.0], [3.0, 1.0, 8.0], [4.0, 7.0, 8.0], [6.0, 8.0, 5.0]]]))
      val=numpy.array([[[1.0, 4.0], [7.0, 1.0], [1.0, 3.0]]])
      arg[2:,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 1.0], [2.0, 1.0, 5.0], [1.0, 8.0, 5.0], [2.0, 4.0, 2.0]], [[7.0, 5.0, 7.0], [1.0, 
4.0, 1.0], [3.0, 4.0, 2.0], [1.0, 4.0, 2.0]], [[1.0, 4.0, 2.0], [7.0, 1.0, 8.0], [1.0, 3.0, 5.0], [7.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 6.0], [6.0, 4.0, 4.0], [3.0, 5.0, 7.0], [7.0, 7.0, 1.0]], [[5.0, 8.0, 4.0], [1.0, 
6.0, 8.0], [3.0, 3.0, 6.0], [4.0, 1.0, 3.0]], [[1.0, 4.0, 8.0], [7.0, 1.0, 8.0], [1.0, 3.0, 8.0], [6.0, 8.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LUU(self):
      arg=Data(numpy.array([[[3.0, 3.0, 7.0], [4.0, 6.0, 8.0], [5.0, 2.0, 2.0], [1.0, 3.0, 7.0]], [[8.0, 3.0, 1.0], [2.0, 6.0, 
3.0], [7.0, 1.0, 3.0], [8.0, 7.0, 6.0]], [[8.0, 1.0, 8.0], [2.0, 8.0, 1.0], [2.0, 8.0, 1.0], [6.0, 7.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 2.0, 1.0], [3.0, 7.0, 7.0], [4.0, 6.0, 2.0], [1.0, 4.0, 5.0]], [[4.0, 4.0, 6.0], 
[3.0, 1.0, 4.0], [6.0, 5.0, 5.0], [6.0, 2.0, 8.0]], [[5.0, 5.0, 2.0], [3.0, 4.0, 8.0], [6.0, 4.0, 1.0], [8.0, 4.0, 7.0]]]))
      val=Data(numpy.array([[[2.0, 5.0], [2.0, 2.0], [3.0, 3.0]]]),self.functionspace)
      arg[2:,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 7.0], [4.0, 6.0, 8.0], [5.0, 2.0, 2.0], [1.0, 3.0, 7.0]], [[8.0, 3.0, 1.0], [2.0, 
6.0, 3.0], [7.0, 1.0, 3.0], [8.0, 7.0, 6.0]], [[2.0, 5.0, 8.0], [2.0, 2.0, 1.0], [3.0, 3.0, 1.0], [6.0, 7.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 1.0], [3.0, 7.0, 7.0], [4.0, 6.0, 2.0], [1.0, 4.0, 5.0]], [[4.0, 4.0, 6.0], [3.0, 
1.0, 4.0], [6.0, 5.0, 5.0], [6.0, 2.0, 8.0]], [[2.0, 5.0, 2.0], [2.0, 2.0, 8.0], [3.0, 3.0, 1.0], [8.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LUU(self):
      arg=Data(numpy.array([[[7.0, 2.0, 4.0], [5.0, 3.0, 8.0], [8.0, 4.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 6.0, 4.0], [4.0, 2.0, 
5.0], [6.0, 4.0, 4.0], [1.0, 2.0, 2.0]], [[1.0, 5.0, 8.0], [6.0, 2.0, 4.0], [1.0, 2.0, 1.0], [1.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 8.0, 7.0], [4.0, 7.0, 1.0], [3.0, 5.0, 3.0], [3.0, 3.0, 7.0]], [[1.0, 4.0, 5.0], 
[7.0, 7.0, 6.0], [7.0, 7.0, 1.0], [3.0, 6.0, 8.0]], [[5.0, 8.0, 1.0], [1.0, 2.0, 2.0], [8.0, 7.0, 8.0], [4.0, 4.0, 6.0]]]))
      val=Data(numpy.array([[[1.0, 4.0], [2.0, 4.0], [2.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 3.0], [6.0, 1.0], [7.0, 2.0]]]))
      arg[2:,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 4.0], [5.0, 3.0, 8.0], [8.0, 4.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 6.0, 4.0], [4.0, 
2.0, 5.0], [6.0, 4.0, 4.0], [1.0, 2.0, 2.0]], [[1.0, 4.0, 8.0], [2.0, 4.0, 4.0], [2.0, 1.0, 1.0], [1.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 8.0, 7.0], [4.0, 7.0, 1.0], [3.0, 5.0, 3.0], [3.0, 3.0, 7.0]], [[1.0, 4.0, 5.0], [7.0, 
7.0, 6.0], [7.0, 7.0, 1.0], [3.0, 6.0, 8.0]], [[1.0, 3.0, 1.0], [6.0, 1.0, 2.0], [7.0, 2.0, 8.0], [4.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LUU(self):
      arg=Data(numpy.array([[[8.0, 4.0, 1.0], [4.0, 7.0, 6.0], [8.0, 7.0, 3.0], [3.0, 1.0, 5.0]], [[7.0, 2.0, 7.0], [1.0, 1.0, 
3.0], [8.0, 6.0, 8.0], [5.0, 8.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 4.0, 7.0], [2.0, 8.0, 4.0], [7.0, 7.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 1.0, 7.0], [5.0, 1.0, 1.0], [1.0, 3.0, 7.0], [8.0, 6.0, 3.0]], [[3.0, 1.0, 2.0], 
[4.0, 3.0, 8.0], [5.0, 4.0, 8.0], [5.0, 2.0, 2.0]], [[2.0, 7.0, 1.0], [8.0, 8.0, 8.0], [3.0, 1.0, 6.0], [5.0, 6.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 1.0], [6.0, 6.0], [8.0, 6.0]]])+(1.-msk_val)*numpy.array([[[7.0, 6.0], [6.0, 1.0], [2.0, 
1.0]]])
      arg[2:,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 1.0], [4.0, 7.0, 6.0], [8.0, 7.0, 3.0], [3.0, 1.0, 5.0]], [[7.0, 2.0, 7.0], [1.0, 
1.0, 3.0], [8.0, 6.0, 8.0], [5.0, 8.0, 3.0]], [[5.0, 1.0, 3.0], [6.0, 6.0, 7.0], [8.0, 6.0, 4.0], [7.0, 7.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 7.0], [5.0, 1.0, 1.0], [1.0, 3.0, 7.0], [8.0, 6.0, 3.0]], [[3.0, 1.0, 2.0], [4.0, 
3.0, 8.0], [5.0, 4.0, 8.0], [5.0, 2.0, 2.0]], [[7.0, 6.0, 1.0], [6.0, 1.0, 8.0], [2.0, 1.0, 6.0], [5.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UPR(self):
      arg=Data(numpy.array([[[6.0, 3.0, 8.0], [5.0, 4.0, 5.0], [6.0, 8.0, 2.0], [6.0, 8.0, 2.0]], [[4.0, 8.0, 1.0], [5.0, 6.0, 
8.0], [1.0, 5.0, 5.0], [6.0, 3.0, 3.0]], [[3.0, 5.0, 7.0], [4.0, 4.0, 1.0], [3.0, 3.0, 2.0], [2.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 7.0, 3.0], [6.0, 7.0, 7.0], [5.0, 7.0, 8.0], [7.0, 7.0, 8.0]], [[7.0, 3.0, 1.0], 
[1.0, 1.0, 8.0], [2.0, 8.0, 4.0], [4.0, 4.0, 6.0]], [[8.0, 6.0, 7.0], [6.0, 3.0, 6.0], [8.0, 2.0, 6.0], [2.0, 3.0, 7.0]]]))
      val=numpy.array([[1.0], [8.0]])
      arg[:2,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 8.0], [5.0, 4.0, 5.0], [6.0, 1.0, 2.0], [6.0, 8.0, 2.0]], [[4.0, 8.0, 1.0], [5.0, 
6.0, 8.0], [1.0, 8.0, 5.0], [6.0, 3.0, 3.0]], [[3.0, 5.0, 7.0], [4.0, 4.0, 1.0], [3.0, 3.0, 2.0], [2.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 3.0], [6.0, 7.0, 7.0], [5.0, 1.0, 8.0], [7.0, 7.0, 8.0]], [[7.0, 3.0, 1.0], [1.0, 
1.0, 8.0], [2.0, 8.0, 4.0], [4.0, 4.0, 6.0]], [[8.0, 6.0, 7.0], [6.0, 3.0, 6.0], [8.0, 2.0, 6.0], [2.0, 3.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UPR(self):
      arg=Data(numpy.array([[[7.0, 3.0, 2.0], [1.0, 6.0, 3.0], [2.0, 7.0, 1.0], [2.0, 3.0, 4.0]], [[2.0, 3.0, 4.0], [4.0, 6.0, 
4.0], [2.0, 6.0, 6.0], [7.0, 1.0, 2.0]], [[4.0, 6.0, 6.0], [8.0, 6.0, 1.0], [6.0, 4.0, 7.0], [2.0, 4.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 3.0, 5.0], [1.0, 6.0, 8.0], [5.0, 5.0, 2.0], [4.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], 
[8.0, 8.0, 7.0], [2.0, 7.0, 2.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 8.0], [6.0, 8.0, 5.0], [5.0, 8.0, 7.0], [4.0, 1.0, 7.0]]]))
      val=Data(numpy.array([[2.0], [1.0]]),self.functionspace)
      arg[:2,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 2.0], [1.0, 6.0, 3.0], [2.0, 2.0, 1.0], [2.0, 3.0, 4.0]], [[2.0, 3.0, 4.0], [4.0, 
6.0, 4.0], [2.0, 1.0, 6.0], [7.0, 1.0, 2.0]], [[4.0, 6.0, 6.0], [8.0, 6.0, 1.0], [6.0, 4.0, 7.0], [2.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 5.0], [1.0, 6.0, 8.0], [5.0, 2.0, 2.0], [4.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], [8.0, 
8.0, 7.0], [2.0, 1.0, 2.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 8.0], [6.0, 8.0, 5.0], [5.0, 8.0, 7.0], [4.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UPR(self):
      arg=Data(numpy.array([[[1.0, 1.0, 4.0], [3.0, 8.0, 3.0], [8.0, 4.0, 4.0], [8.0, 2.0, 2.0]], [[4.0, 3.0, 3.0], [1.0, 8.0, 
3.0], [6.0, 2.0, 6.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 5.0], [2.0, 1.0, 1.0], [2.0, 5.0, 6.0], [7.0, 4.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 6.0], [2.0, 1.0, 2.0], [6.0, 7.0, 4.0], [6.0, 8.0, 3.0]], [[4.0, 4.0, 1.0], 
[4.0, 4.0, 5.0], [2.0, 3.0, 6.0], [5.0, 5.0, 2.0]], [[2.0, 3.0, 5.0], [5.0, 2.0, 8.0], [2.0, 4.0, 5.0], [1.0, 5.0, 3.0]]]))
      val=Data(numpy.array([[8.0], [7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0], [8.0]]))
      arg[:2,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 4.0], [3.0, 8.0, 3.0], [8.0, 8.0, 4.0], [8.0, 2.0, 2.0]], [[4.0, 3.0, 3.0], [1.0, 
8.0, 3.0], [6.0, 7.0, 6.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 5.0], [2.0, 1.0, 1.0], [2.0, 5.0, 6.0], [7.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 6.0], [2.0, 1.0, 2.0], [6.0, 4.0, 4.0], [6.0, 8.0, 3.0]], [[4.0, 4.0, 1.0], [4.0, 
4.0, 5.0], [2.0, 8.0, 6.0], [5.0, 5.0, 2.0]], [[2.0, 3.0, 5.0], [5.0, 2.0, 8.0], [2.0, 4.0, 5.0], [1.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UPR(self):
      arg=Data(numpy.array([[[4.0, 8.0, 7.0], [4.0, 5.0, 7.0], [8.0, 4.0, 3.0], [5.0, 1.0, 5.0]], [[7.0, 5.0, 2.0], [5.0, 6.0, 
4.0], [1.0, 7.0, 2.0], [8.0, 1.0, 7.0]], [[7.0, 1.0, 2.0], [4.0, 7.0, 1.0], [6.0, 6.0, 6.0], [7.0, 7.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 3.0, 1.0], [2.0, 4.0, 5.0], [1.0, 5.0, 6.0], [3.0, 4.0, 7.0]], [[5.0, 5.0, 1.0], 
[6.0, 7.0, 6.0], [6.0, 6.0, 7.0], [6.0, 4.0, 2.0]], [[3.0, 5.0, 8.0], [2.0, 2.0, 7.0], [7.0, 4.0, 3.0], [3.0, 1.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[7.0], [3.0]])+(1.-msk_val)*numpy.array([[1.0], [4.0]])
      arg[:2,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 7.0], [4.0, 5.0, 7.0], [8.0, 7.0, 3.0], [5.0, 1.0, 5.0]], [[7.0, 5.0, 2.0], [5.0, 
6.0, 4.0], [1.0, 3.0, 2.0], [8.0, 1.0, 7.0]], [[7.0, 1.0, 2.0], [4.0, 7.0, 1.0], [6.0, 6.0, 6.0], [7.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 1.0], [2.0, 4.0, 5.0], [1.0, 1.0, 6.0], [3.0, 4.0, 7.0]], [[5.0, 5.0, 1.0], [6.0, 
7.0, 6.0], [6.0, 4.0, 7.0], [6.0, 4.0, 2.0]], [[3.0, 5.0, 8.0], [2.0, 2.0, 7.0], [7.0, 4.0, 3.0], [3.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CCU(self):
      arg=Data(numpy.array([[[6.0, 3.0, 6.0], [5.0, 4.0, 2.0], [1.0, 2.0, 7.0], [5.0, 1.0, 7.0]], [[6.0, 1.0, 1.0], [8.0, 7.0, 
7.0], [8.0, 8.0, 2.0], [7.0, 8.0, 7.0]], [[5.0, 3.0, 3.0], [6.0, 2.0, 5.0], [5.0, 8.0, 4.0], [6.0, 7.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 4.0, 6.0], [2.0, 8.0, 4.0], [8.0, 7.0, 5.0], [8.0, 1.0, 3.0]], [[1.0, 7.0, 4.0], 
[2.0, 7.0, 4.0], [3.0, 6.0, 3.0], [1.0, 2.0, 8.0]], [[2.0, 2.0, 4.0], [6.0, 5.0, 6.0], [6.0, 7.0, 2.0], [2.0, 2.0, 2.0]]]))
      val=numpy.array([[[7.0, 6.0], [1.0, 3.0], [7.0, 8.0], [7.0, 5.0]], [[3.0, 3.0], [4.0, 6.0], [1.0, 8.0], [1.0, 4.0]], 
[[4.0, 4.0], [4.0, 8.0], [2.0, 2.0], [8.0, 2.0]]])
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 6.0], [1.0, 3.0, 2.0], [7.0, 8.0, 7.0], [7.0, 5.0, 7.0]], [[3.0, 3.0, 1.0], [4.0, 
6.0, 7.0], [1.0, 8.0, 2.0], [1.0, 4.0, 7.0]], [[4.0, 4.0, 3.0], [4.0, 8.0, 5.0], [2.0, 2.0, 4.0], [8.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 6.0], [1.0, 3.0, 4.0], [7.0, 8.0, 5.0], [7.0, 5.0, 3.0]], [[3.0, 3.0, 4.0], [4.0, 
6.0, 4.0], [1.0, 8.0, 3.0], [1.0, 4.0, 8.0]], [[4.0, 4.0, 4.0], [4.0, 8.0, 6.0], [2.0, 2.0, 2.0], [8.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CCU(self):
      arg=Data(numpy.array([[[3.0, 1.0, 8.0], [2.0, 4.0, 5.0], [7.0, 6.0, 7.0], [7.0, 5.0, 4.0]], [[4.0, 2.0, 1.0], [6.0, 2.0, 
6.0], [1.0, 8.0, 4.0], [5.0, 4.0, 4.0]], [[7.0, 5.0, 8.0], [1.0, 5.0, 6.0], [2.0, 3.0, 6.0], [4.0, 1.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 6.0, 1.0], [2.0, 2.0, 5.0], [5.0, 4.0, 8.0], [8.0, 2.0, 1.0]], [[1.0, 6.0, 4.0], 
[7.0, 5.0, 5.0], [5.0, 6.0, 2.0], [8.0, 3.0, 6.0]], [[8.0, 8.0, 3.0], [4.0, 7.0, 5.0], [7.0, 8.0, 4.0], [5.0, 4.0, 4.0]]]))
      val=Data(numpy.array([[[4.0, 3.0], [1.0, 5.0], [8.0, 7.0], [1.0, 5.0]], [[5.0, 3.0], [3.0, 1.0], [8.0, 3.0], [2.0, 3.0]], 
[[3.0, 8.0], [4.0, 7.0], [3.0, 8.0], [6.0, 8.0]]]),self.functionspace)
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 8.0], [1.0, 5.0, 5.0], [8.0, 7.0, 7.0], [1.0, 5.0, 4.0]], [[5.0, 3.0, 1.0], [3.0, 
1.0, 6.0], [8.0, 3.0, 4.0], [2.0, 3.0, 4.0]], [[3.0, 8.0, 8.0], [4.0, 7.0, 6.0], [3.0, 8.0, 6.0], [6.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 1.0], [1.0, 5.0, 5.0], [8.0, 7.0, 8.0], [1.0, 5.0, 1.0]], [[5.0, 3.0, 4.0], [3.0, 
1.0, 5.0], [8.0, 3.0, 2.0], [2.0, 3.0, 6.0]], [[3.0, 8.0, 3.0], [4.0, 7.0, 5.0], [3.0, 8.0, 4.0], [6.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CCU(self):
      arg=Data(numpy.array([[[6.0, 6.0, 6.0], [5.0, 8.0, 6.0], [7.0, 4.0, 2.0], [4.0, 6.0, 8.0]], [[8.0, 7.0, 7.0], [3.0, 5.0, 
1.0], [3.0, 6.0, 1.0], [3.0, 8.0, 1.0]], [[6.0, 1.0, 4.0], [4.0, 4.0, 3.0], [4.0, 1.0, 1.0], [8.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 6.0, 7.0], [2.0, 2.0, 6.0], [4.0, 4.0, 8.0], [7.0, 1.0, 7.0]], [[4.0, 2.0, 7.0], 
[5.0, 7.0, 6.0], [1.0, 8.0, 7.0], [4.0, 2.0, 7.0]], [[6.0, 3.0, 4.0], [3.0, 5.0, 8.0], [6.0, 6.0, 5.0], [2.0, 2.0, 2.0]]]))
      val=Data(numpy.array([[[1.0, 5.0], [4.0, 7.0], [6.0, 7.0], [4.0, 1.0]], [[4.0, 7.0], [5.0, 1.0], [5.0, 2.0], [7.0, 2.0]], 
[[5.0, 8.0], [3.0, 1.0], [6.0, 5.0], [3.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 5.0], [6.0, 6.0], [8.0, 6.0], [4.0, 5.0]], [[4.0, 6.0], [2.0, 5.0], [4.0, 5.0], 
[1.0, 7.0]], [[3.0, 4.0], [6.0, 3.0], [5.0, 1.0], [4.0, 2.0]]]))
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 6.0], [4.0, 7.0, 6.0], [6.0, 7.0, 2.0], [4.0, 1.0, 8.0]], [[4.0, 7.0, 7.0], [5.0, 
1.0, 1.0], [5.0, 2.0, 1.0], [7.0, 2.0, 1.0]], [[5.0, 8.0, 4.0], [3.0, 1.0, 3.0], [6.0, 5.0, 1.0], [3.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 7.0], [6.0, 6.0, 6.0], [8.0, 6.0, 8.0], [4.0, 5.0, 7.0]], [[4.0, 6.0, 7.0], [2.0, 
5.0, 6.0], [4.0, 5.0, 7.0], [1.0, 7.0, 7.0]], [[3.0, 4.0, 4.0], [6.0, 3.0, 8.0], [5.0, 1.0, 5.0], [4.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CCU(self):
      arg=Data(numpy.array([[[2.0, 8.0, 3.0], [8.0, 6.0, 5.0], [8.0, 7.0, 8.0], [7.0, 8.0, 3.0]], [[4.0, 8.0, 2.0], [8.0, 3.0, 
5.0], [1.0, 6.0, 2.0], [5.0, 7.0, 2.0]], [[5.0, 4.0, 5.0], [7.0, 5.0, 1.0], [7.0, 2.0, 1.0], [7.0, 8.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 6.0, 7.0], [6.0, 3.0, 4.0], [4.0, 2.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 3.0, 2.0], 
[8.0, 2.0, 1.0], [2.0, 7.0, 8.0], [7.0, 7.0, 1.0]], [[7.0, 1.0, 7.0], [6.0, 1.0, 7.0], [1.0, 6.0, 6.0], [2.0, 6.0, 5.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 2.0], [5.0, 2.0], [7.0, 6.0], [1.0, 5.0]], [[1.0, 5.0], [7.0, 1.0], [1.0, 6.0], [2.0, 
7.0]], [[6.0, 8.0], [8.0, 3.0], [2.0, 5.0], [3.0, 4.0]]])+(1.-msk_val)*numpy.array([[[6.0, 2.0], [4.0, 5.0], [7.0, 3.0], [5.0, 
8.0]], [[6.0, 3.0], [8.0, 6.0], [3.0, 6.0], [2.0, 6.0]], [[7.0, 8.0], [3.0, 1.0], [7.0, 6.0], [3.0, 8.0]]])
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 3.0], [5.0, 2.0, 5.0], [7.0, 6.0, 8.0], [1.0, 5.0, 3.0]], [[1.0, 5.0, 2.0], [7.0, 
1.0, 5.0], [1.0, 6.0, 2.0], [2.0, 7.0, 2.0]], [[6.0, 8.0, 5.0], [8.0, 3.0, 1.0], [2.0, 5.0, 1.0], [3.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 7.0], [4.0, 5.0, 4.0], [7.0, 3.0, 1.0], [5.0, 8.0, 1.0]], [[6.0, 3.0, 2.0], [8.0, 
6.0, 1.0], [3.0, 6.0, 8.0], [2.0, 6.0, 1.0]], [[7.0, 8.0, 7.0], [3.0, 1.0, 7.0], [7.0, 6.0, 6.0], [3.0, 8.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PCU(self):
      arg=Data(numpy.array([[[2.0, 1.0, 7.0], [7.0, 7.0, 5.0], [8.0, 4.0, 5.0], [4.0, 6.0, 5.0]], [[8.0, 6.0, 8.0], [6.0, 7.0, 
7.0], [1.0, 6.0, 3.0], [2.0, 8.0, 8.0]], [[8.0, 5.0, 8.0], [4.0, 7.0, 3.0], [2.0, 3.0, 2.0], [8.0, 3.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 7.0, 4.0], [3.0, 2.0, 2.0], [4.0, 5.0, 7.0], [7.0, 8.0, 8.0]], [[3.0, 5.0, 7.0], 
[3.0, 4.0, 1.0], [4.0, 5.0, 7.0], [3.0, 1.0, 4.0]], [[6.0, 6.0, 8.0], [1.0, 3.0, 3.0], [2.0, 8.0, 2.0], [3.0, 2.0, 1.0]]]))
      val=numpy.array([[7.0, 7.0], [3.0, 7.0], [1.0, 3.0], [4.0, 1.0]])
      arg[1,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 7.0], [7.0, 7.0, 5.0], [8.0, 4.0, 5.0], [4.0, 6.0, 5.0]], [[7.0, 7.0, 8.0], [3.0, 
7.0, 7.0], [1.0, 3.0, 3.0], [4.0, 1.0, 8.0]], [[8.0, 5.0, 8.0], [4.0, 7.0, 3.0], [2.0, 3.0, 2.0], [8.0, 3.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 4.0], [3.0, 2.0, 2.0], [4.0, 5.0, 7.0], [7.0, 8.0, 8.0]], [[7.0, 7.0, 7.0], [3.0, 
7.0, 1.0], [1.0, 3.0, 7.0], [4.0, 1.0, 4.0]], [[6.0, 6.0, 8.0], [1.0, 3.0, 3.0], [2.0, 8.0, 2.0], [3.0, 2.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PCU(self):
      arg=Data(numpy.array([[[4.0, 5.0, 2.0], [3.0, 8.0, 3.0], [3.0, 3.0, 2.0], [2.0, 2.0, 3.0]], [[7.0, 2.0, 3.0], [6.0, 4.0, 
4.0], [2.0, 7.0, 4.0], [6.0, 7.0, 7.0]], [[2.0, 2.0, 3.0], [5.0, 4.0, 5.0], [6.0, 2.0, 1.0], [6.0, 7.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 5.0, 2.0], [7.0, 6.0, 5.0], [6.0, 6.0, 6.0], [4.0, 2.0, 5.0]], [[3.0, 2.0, 6.0], 
[7.0, 1.0, 7.0], [6.0, 6.0, 5.0], [3.0, 2.0, 1.0]], [[2.0, 7.0, 4.0], [6.0, 8.0, 6.0], [8.0, 8.0, 1.0], [1.0, 2.0, 3.0]]]))
      val=Data(numpy.array([[6.0, 6.0], [5.0, 2.0], [5.0, 2.0], [4.0, 6.0]]),self.functionspace)
      arg[1,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 5.0, 2.0], [3.0, 8.0, 3.0], [3.0, 3.0, 2.0], [2.0, 2.0, 3.0]], [[6.0, 6.0, 3.0], [5.0, 
2.0, 4.0], [5.0, 2.0, 4.0], [4.0, 6.0, 7.0]], [[2.0, 2.0, 3.0], [5.0, 4.0, 5.0], [6.0, 2.0, 1.0], [6.0, 7.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 2.0], [7.0, 6.0, 5.0], [6.0, 6.0, 6.0], [4.0, 2.0, 5.0]], [[6.0, 6.0, 6.0], [5.0, 
2.0, 7.0], [5.0, 2.0, 5.0], [4.0, 6.0, 1.0]], [[2.0, 7.0, 4.0], [6.0, 8.0, 6.0], [8.0, 8.0, 1.0], [1.0, 2.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PCU(self):
      arg=Data(numpy.array([[[7.0, 1.0, 8.0], [4.0, 4.0, 4.0], [5.0, 8.0, 2.0], [2.0, 5.0, 4.0]], [[5.0, 5.0, 5.0], [7.0, 4.0, 
8.0], [5.0, 2.0, 1.0], [1.0, 7.0, 1.0]], [[5.0, 6.0, 4.0], [3.0, 8.0, 2.0], [5.0, 7.0, 7.0], [5.0, 1.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 8.0, 4.0], [7.0, 7.0, 8.0], [6.0, 2.0, 5.0], [6.0, 3.0, 2.0]], [[8.0, 1.0, 1.0], 
[5.0, 8.0, 4.0], [4.0, 8.0, 6.0], [6.0, 2.0, 8.0]], [[7.0, 3.0, 4.0], [7.0, 6.0, 8.0], [4.0, 4.0, 7.0], [8.0, 4.0, 4.0]]]))
      val=Data(numpy.array([[3.0, 8.0], [5.0, 2.0], [8.0, 2.0], [2.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 4.0], [2.0, 8.0], [4.0, 6.0], [3.0, 4.0]]))
      arg[1,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 1.0, 8.0], [4.0, 4.0, 4.0], [5.0, 8.0, 2.0], [2.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [5.0, 
2.0, 8.0], [8.0, 2.0, 1.0], [2.0, 2.0, 1.0]], [[5.0, 6.0, 4.0], [3.0, 8.0, 2.0], [5.0, 7.0, 7.0], [5.0, 1.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 4.0], [7.0, 7.0, 8.0], [6.0, 2.0, 5.0], [6.0, 3.0, 2.0]], [[2.0, 4.0, 1.0], [2.0, 
8.0, 4.0], [4.0, 6.0, 6.0], [3.0, 4.0, 8.0]], [[7.0, 3.0, 4.0], [7.0, 6.0, 8.0], [4.0, 4.0, 7.0], [8.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PCU(self):
      arg=Data(numpy.array([[[8.0, 4.0, 1.0], [7.0, 4.0, 5.0], [2.0, 8.0, 8.0], [5.0, 1.0, 6.0]], [[6.0, 5.0, 5.0], [2.0, 2.0, 
3.0], [7.0, 4.0, 8.0], [3.0, 4.0, 6.0]], [[8.0, 6.0, 5.0], [5.0, 7.0, 8.0], [6.0, 4.0, 4.0], [6.0, 6.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 8.0, 7.0], [2.0, 1.0, 3.0], [7.0, 7.0, 3.0], [5.0, 8.0, 6.0]], [[5.0, 1.0, 8.0], 
[4.0, 1.0, 7.0], [1.0, 5.0, 7.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], [1.0, 7.0, 3.0], [6.0, 7.0, 6.0], [7.0, 2.0, 6.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[1.0, 2.0], [3.0, 1.0], [7.0, 2.0], [2.0, 5.0]])+(1.-msk_val)*numpy.array([[1.0, 3.0], [8.0, 
4.0], [1.0, 3.0], [5.0, 4.0]])
      arg[1,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 1.0], [7.0, 4.0, 5.0], [2.0, 8.0, 8.0], [5.0, 1.0, 6.0]], [[1.0, 2.0, 5.0], [3.0, 
1.0, 3.0], [7.0, 2.0, 8.0], [2.0, 5.0, 6.0]], [[8.0, 6.0, 5.0], [5.0, 7.0, 8.0], [6.0, 4.0, 4.0], [6.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 7.0], [2.0, 1.0, 3.0], [7.0, 7.0, 3.0], [5.0, 8.0, 6.0]], [[1.0, 3.0, 8.0], [8.0, 
4.0, 7.0], [1.0, 3.0, 7.0], [5.0, 4.0, 5.0]], [[8.0, 1.0, 1.0], [1.0, 7.0, 3.0], [6.0, 7.0, 6.0], [7.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PPL(self):
      arg=Data(numpy.array([[[1.0, 8.0, 1.0], [1.0, 3.0, 6.0], [8.0, 4.0, 5.0], [8.0, 6.0, 4.0]], [[2.0, 4.0, 1.0], [7.0, 2.0, 
6.0], [4.0, 4.0, 3.0], [3.0, 2.0, 6.0]], [[7.0, 7.0, 2.0], [6.0, 4.0, 5.0], [7.0, 5.0, 6.0], [2.0, 4.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 6.0, 1.0], [1.0, 8.0, 4.0], [7.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[1.0, 5.0, 7.0], 
[4.0, 1.0, 2.0], [3.0, 1.0, 4.0], [8.0, 3.0, 8.0]], [[2.0, 5.0, 6.0], [8.0, 2.0, 6.0], [2.0, 5.0, 4.0], [2.0, 1.0, 1.0]]]))
      val=numpy.array([1.0])
      arg[1,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 1.0], [1.0, 3.0, 6.0], [8.0, 4.0, 5.0], [8.0, 6.0, 4.0]], [[2.0, 4.0, 1.0], [7.0, 
2.0, 6.0], [4.0, 4.0, 1.0], [3.0, 2.0, 6.0]], [[7.0, 7.0, 2.0], [6.0, 4.0, 5.0], [7.0, 5.0, 6.0], [2.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 6.0, 1.0], [1.0, 8.0, 4.0], [7.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[1.0, 5.0, 7.0], [4.0, 
1.0, 2.0], [3.0, 1.0, 1.0], [8.0, 3.0, 8.0]], [[2.0, 5.0, 6.0], [8.0, 2.0, 6.0], [2.0, 5.0, 4.0], [2.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PPL(self):
      arg=Data(numpy.array([[[7.0, 7.0, 6.0], [8.0, 2.0, 2.0], [1.0, 7.0, 4.0], [6.0, 6.0, 6.0]], [[4.0, 5.0, 6.0], [7.0, 3.0, 
2.0], [1.0, 8.0, 3.0], [1.0, 4.0, 1.0]], [[6.0, 1.0, 1.0], [5.0, 6.0, 8.0], [2.0, 1.0, 5.0], [3.0, 5.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 6.0, 5.0], [4.0, 6.0, 6.0], [4.0, 7.0, 6.0], [7.0, 6.0, 4.0]], [[3.0, 2.0, 6.0], 
[3.0, 8.0, 5.0], [4.0, 3.0, 4.0], [2.0, 6.0, 3.0]], [[8.0, 7.0, 7.0], [1.0, 3.0, 4.0], [1.0, 2.0, 5.0], [5.0, 4.0, 7.0]]]))
      val=Data(numpy.array([5.0]),self.functionspace)
      arg[1,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 7.0, 6.0], [8.0, 2.0, 2.0], [1.0, 7.0, 4.0], [6.0, 6.0, 6.0]], [[4.0, 5.0, 6.0], [7.0, 
3.0, 2.0], [1.0, 8.0, 5.0], [1.0, 4.0, 1.0]], [[6.0, 1.0, 1.0], [5.0, 6.0, 8.0], [2.0, 1.0, 5.0], [3.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 5.0], [4.0, 6.0, 6.0], [4.0, 7.0, 6.0], [7.0, 6.0, 4.0]], [[3.0, 2.0, 6.0], [3.0, 
8.0, 5.0], [4.0, 3.0, 5.0], [2.0, 6.0, 3.0]], [[8.0, 7.0, 7.0], [1.0, 3.0, 4.0], [1.0, 2.0, 5.0], [5.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PPL(self):
      arg=Data(numpy.array([[[5.0, 2.0, 7.0], [5.0, 4.0, 4.0], [8.0, 1.0, 4.0], [8.0, 2.0, 3.0]], [[7.0, 2.0, 4.0], [6.0, 4.0, 
7.0], [8.0, 1.0, 8.0], [7.0, 5.0, 5.0]], [[2.0, 1.0, 8.0], [6.0, 5.0, 1.0], [2.0, 7.0, 2.0], [4.0, 7.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 3.0, 5.0], [4.0, 1.0, 5.0], [7.0, 4.0, 7.0], [5.0, 1.0, 8.0]], [[5.0, 2.0, 4.0], 
[8.0, 2.0, 7.0], [5.0, 6.0, 5.0], [8.0, 1.0, 6.0]], [[7.0, 7.0, 7.0], [3.0, 2.0, 3.0], [8.0, 7.0, 4.0], [3.0, 3.0, 8.0]]]))
      val=Data(numpy.array([5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([2.0]))
      arg[1,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 7.0], [5.0, 4.0, 4.0], [8.0, 1.0, 4.0], [8.0, 2.0, 3.0]], [[7.0, 2.0, 4.0], [6.0, 
4.0, 7.0], [8.0, 1.0, 5.0], [7.0, 5.0, 5.0]], [[2.0, 1.0, 8.0], [6.0, 5.0, 1.0], [2.0, 7.0, 2.0], [4.0, 7.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 5.0], [4.0, 1.0, 5.0], [7.0, 4.0, 7.0], [5.0, 1.0, 8.0]], [[5.0, 2.0, 4.0], [8.0, 
2.0, 7.0], [5.0, 6.0, 2.0], [8.0, 1.0, 6.0]], [[7.0, 7.0, 7.0], [3.0, 2.0, 3.0], [8.0, 7.0, 4.0], [3.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PPL(self):
      arg=Data(numpy.array([[[3.0, 2.0, 1.0], [6.0, 1.0, 2.0], [4.0, 6.0, 7.0], [8.0, 8.0, 5.0]], [[3.0, 6.0, 2.0], [1.0, 1.0, 
5.0], [8.0, 7.0, 5.0], [4.0, 7.0, 8.0]], [[1.0, 5.0, 4.0], [2.0, 2.0, 7.0], [5.0, 7.0, 5.0], [2.0, 6.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 8.0, 4.0], [5.0, 8.0, 1.0], [4.0, 3.0, 3.0], [8.0, 3.0, 5.0]], [[6.0, 8.0, 6.0], 
[2.0, 5.0, 5.0], [6.0, 2.0, 1.0], [5.0, 7.0, 6.0]], [[1.0, 5.0, 1.0], [7.0, 2.0, 6.0], [4.0, 6.0, 8.0], [6.0, 4.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([4.0])+(1.-msk_val)*numpy.array([7.0])
      arg[1,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 1.0], [6.0, 1.0, 2.0], [4.0, 6.0, 7.0], [8.0, 8.0, 5.0]], [[3.0, 6.0, 2.0], [1.0, 
1.0, 5.0], [8.0, 7.0, 4.0], [4.0, 7.0, 8.0]], [[1.0, 5.0, 4.0], [2.0, 2.0, 7.0], [5.0, 7.0, 5.0], [2.0, 6.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 4.0], [5.0, 8.0, 1.0], [4.0, 3.0, 3.0], [8.0, 3.0, 5.0]], [[6.0, 8.0, 6.0], [2.0, 
5.0, 5.0], [6.0, 2.0, 7.0], [5.0, 7.0, 6.0]], [[1.0, 5.0, 1.0], [7.0, 2.0, 6.0], [4.0, 6.0, 8.0], [6.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CPR(self):
      arg=Data(numpy.array([[[8.0, 7.0, 6.0], [5.0, 7.0, 1.0], [2.0, 2.0, 1.0], [3.0, 4.0, 6.0]], [[8.0, 2.0, 6.0], [7.0, 7.0, 
2.0], [5.0, 4.0, 2.0], [7.0, 1.0, 2.0]], [[4.0, 1.0, 8.0], [1.0, 5.0, 5.0], [2.0, 5.0, 4.0], [7.0, 6.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 7.0, 4.0], [1.0, 6.0, 2.0], [7.0, 7.0, 2.0], [3.0, 3.0, 7.0]], [[1.0, 2.0, 4.0], 
[2.0, 6.0, 6.0], [2.0, 5.0, 4.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 6.0], [5.0, 7.0, 4.0], [6.0, 7.0, 3.0], [4.0, 4.0, 4.0]]]))
      val=numpy.array([[8.0], [4.0], [8.0]])
      arg[:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 7.0, 6.0], [5.0, 7.0, 1.0], [2.0, 8.0, 1.0], [3.0, 4.0, 6.0]], [[8.0, 2.0, 6.0], [7.0, 
7.0, 2.0], [5.0, 4.0, 2.0], [7.0, 1.0, 2.0]], [[4.0, 1.0, 8.0], [1.0, 5.0, 5.0], [2.0, 8.0, 4.0], [7.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 4.0], [1.0, 6.0, 2.0], [7.0, 8.0, 2.0], [3.0, 3.0, 7.0]], [[1.0, 2.0, 4.0], [2.0, 
6.0, 6.0], [2.0, 4.0, 4.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 6.0], [5.0, 7.0, 4.0], [6.0, 8.0, 3.0], [4.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CPR(self):
      arg=Data(numpy.array([[[3.0, 3.0, 5.0], [5.0, 8.0, 2.0], [1.0, 1.0, 4.0], [3.0, 4.0, 4.0]], [[5.0, 2.0, 5.0], [2.0, 4.0, 
8.0], [7.0, 5.0, 7.0], [1.0, 4.0, 6.0]], [[7.0, 7.0, 5.0], [8.0, 6.0, 4.0], [6.0, 1.0, 8.0], [8.0, 7.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 3.0, 2.0], [2.0, 3.0, 8.0], [6.0, 7.0, 8.0], [5.0, 7.0, 7.0]], [[6.0, 6.0, 6.0], 
[5.0, 5.0, 3.0], [2.0, 6.0, 3.0], [7.0, 8.0, 6.0]], [[3.0, 1.0, 1.0], [6.0, 3.0, 8.0], [1.0, 6.0, 5.0], [3.0, 2.0, 4.0]]]))
      val=Data(numpy.array([[4.0], [7.0], [6.0]]),self.functionspace)
      arg[:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 5.0], [5.0, 8.0, 2.0], [1.0, 4.0, 4.0], [3.0, 4.0, 4.0]], [[5.0, 2.0, 5.0], [2.0, 
4.0, 8.0], [7.0, 7.0, 7.0], [1.0, 4.0, 6.0]], [[7.0, 7.0, 5.0], [8.0, 6.0, 4.0], [6.0, 6.0, 8.0], [8.0, 7.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 2.0], [2.0, 3.0, 8.0], [6.0, 4.0, 8.0], [5.0, 7.0, 7.0]], [[6.0, 6.0, 6.0], [5.0, 
5.0, 3.0], [2.0, 7.0, 3.0], [7.0, 8.0, 6.0]], [[3.0, 1.0, 1.0], [6.0, 3.0, 8.0], [1.0, 6.0, 5.0], [3.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CPR(self):
      arg=Data(numpy.array([[[6.0, 6.0, 6.0], [2.0, 3.0, 7.0], [6.0, 4.0, 1.0], [1.0, 8.0, 8.0]], [[7.0, 8.0, 6.0], [2.0, 1.0, 
8.0], [5.0, 5.0, 7.0], [5.0, 4.0, 3.0]], [[2.0, 8.0, 8.0], [8.0, 8.0, 6.0], [6.0, 4.0, 3.0], [1.0, 7.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 8.0, 7.0], [5.0, 5.0, 7.0], [6.0, 5.0, 2.0], [3.0, 4.0, 2.0]], [[4.0, 6.0, 8.0], 
[6.0, 4.0, 5.0], [2.0, 5.0, 7.0], [5.0, 1.0, 6.0]], [[4.0, 6.0, 5.0], [8.0, 8.0, 6.0], [6.0, 6.0, 6.0], [4.0, 7.0, 5.0]]]))
      val=Data(numpy.array([[3.0], [7.0], [4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0], [2.0], [1.0]]))
      arg[:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 6.0], [2.0, 3.0, 7.0], [6.0, 3.0, 1.0], [1.0, 8.0, 8.0]], [[7.0, 8.0, 6.0], [2.0, 
1.0, 8.0], [5.0, 7.0, 7.0], [5.0, 4.0, 3.0]], [[2.0, 8.0, 8.0], [8.0, 8.0, 6.0], [6.0, 4.0, 3.0], [1.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 7.0], [5.0, 5.0, 7.0], [6.0, 8.0, 2.0], [3.0, 4.0, 2.0]], [[4.0, 6.0, 8.0], [6.0, 
4.0, 5.0], [2.0, 2.0, 7.0], [5.0, 1.0, 6.0]], [[4.0, 6.0, 5.0], [8.0, 8.0, 6.0], [6.0, 1.0, 6.0], [4.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CPR(self):
      arg=Data(numpy.array([[[7.0, 5.0, 4.0], [1.0, 4.0, 2.0], [1.0, 4.0, 7.0], [4.0, 7.0, 1.0]], [[7.0, 8.0, 6.0], [7.0, 7.0, 
6.0], [4.0, 8.0, 6.0], [3.0, 3.0, 5.0]], [[2.0, 2.0, 7.0], [3.0, 8.0, 8.0], [6.0, 6.0, 1.0], [6.0, 1.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 6.0, 5.0], [4.0, 3.0, 8.0], [2.0, 8.0, 8.0], [2.0, 5.0, 8.0]], [[2.0, 1.0, 8.0], 
[3.0, 3.0, 1.0], [8.0, 7.0, 7.0], [5.0, 4.0, 6.0]], [[4.0, 7.0, 1.0], [5.0, 4.0, 8.0], [2.0, 3.0, 2.0], [3.0, 6.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[5.0], [7.0], [4.0]])+(1.-msk_val)*numpy.array([[8.0], [2.0], [1.0]])
      arg[:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 4.0], [1.0, 4.0, 2.0], [1.0, 5.0, 7.0], [4.0, 7.0, 1.0]], [[7.0, 8.0, 6.0], [7.0, 
7.0, 6.0], [4.0, 7.0, 6.0], [3.0, 3.0, 5.0]], [[2.0, 2.0, 7.0], [3.0, 8.0, 8.0], [6.0, 4.0, 1.0], [6.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 5.0], [4.0, 3.0, 8.0], [2.0, 8.0, 8.0], [2.0, 5.0, 8.0]], [[2.0, 1.0, 8.0], [3.0, 
3.0, 1.0], [8.0, 2.0, 7.0], [5.0, 4.0, 6.0]], [[4.0, 7.0, 1.0], [5.0, 4.0, 8.0], [2.0, 1.0, 2.0], [3.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PCL(self):
      arg=Data(numpy.array([[[1.0, 5.0, 6.0], [6.0, 1.0, 6.0], [1.0, 4.0, 5.0], [3.0, 3.0, 1.0]], [[2.0, 2.0, 1.0], [5.0, 8.0, 
3.0], [4.0, 7.0, 4.0], [6.0, 3.0, 5.0]], [[5.0, 5.0, 6.0], [4.0, 1.0, 4.0], [3.0, 5.0, 7.0], [4.0, 4.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 3.0, 2.0], [3.0, 7.0, 5.0], [6.0, 5.0, 5.0], [3.0, 5.0, 7.0]], [[6.0, 7.0, 8.0], 
[8.0, 3.0, 6.0], [6.0, 2.0, 2.0], [3.0, 8.0, 3.0]], [[7.0, 3.0, 2.0], [6.0, 6.0, 3.0], [4.0, 4.0, 6.0], [2.0, 5.0, 7.0]]]))
      val=numpy.array([[2.0], [4.0], [4.0], [5.0]])
      arg[1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 6.0], [6.0, 1.0, 6.0], [1.0, 4.0, 5.0], [3.0, 3.0, 1.0]], [[2.0, 2.0, 2.0], [5.0, 
8.0, 4.0], [4.0, 7.0, 4.0], [6.0, 3.0, 5.0]], [[5.0, 5.0, 6.0], [4.0, 1.0, 4.0], [3.0, 5.0, 7.0], [4.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 2.0], [3.0, 7.0, 5.0], [6.0, 5.0, 5.0], [3.0, 5.0, 7.0]], [[6.0, 7.0, 2.0], [8.0, 
3.0, 4.0], [6.0, 2.0, 4.0], [3.0, 8.0, 5.0]], [[7.0, 3.0, 2.0], [6.0, 6.0, 3.0], [4.0, 4.0, 6.0], [2.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PCL(self):
      arg=Data(numpy.array([[[7.0, 5.0, 4.0], [8.0, 8.0, 7.0], [4.0, 5.0, 4.0], [3.0, 7.0, 8.0]], [[8.0, 4.0, 6.0], [3.0, 3.0, 
4.0], [5.0, 8.0, 3.0], [4.0, 4.0, 8.0]], [[7.0, 1.0, 3.0], [7.0, 1.0, 8.0], [2.0, 6.0, 6.0], [6.0, 1.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 3.0, 8.0], [1.0, 5.0, 6.0], [6.0, 1.0, 1.0], [2.0, 7.0, 8.0]], [[7.0, 1.0, 5.0], 
[8.0, 2.0, 3.0], [2.0, 4.0, 3.0], [2.0, 7.0, 7.0]], [[8.0, 6.0, 3.0], [5.0, 3.0, 5.0], [2.0, 6.0, 4.0], [7.0, 3.0, 6.0]]]))
      val=Data(numpy.array([[7.0], [5.0], [2.0], [4.0]]),self.functionspace)
      arg[1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 4.0], [8.0, 8.0, 7.0], [4.0, 5.0, 4.0], [3.0, 7.0, 8.0]], [[8.0, 4.0, 7.0], [3.0, 
3.0, 5.0], [5.0, 8.0, 2.0], [4.0, 4.0, 4.0]], [[7.0, 1.0, 3.0], [7.0, 1.0, 8.0], [2.0, 6.0, 6.0], [6.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 8.0], [1.0, 5.0, 6.0], [6.0, 1.0, 1.0], [2.0, 7.0, 8.0]], [[7.0, 1.0, 7.0], [8.0, 
2.0, 5.0], [2.0, 4.0, 2.0], [2.0, 7.0, 4.0]], [[8.0, 6.0, 3.0], [5.0, 3.0, 5.0], [2.0, 6.0, 4.0], [7.0, 3.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PCL(self):
      arg=Data(numpy.array([[[4.0, 8.0, 7.0], [7.0, 8.0, 2.0], [6.0, 8.0, 1.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 7.0], [4.0, 7.0, 
7.0], [7.0, 8.0, 4.0], [5.0, 7.0, 4.0]], [[6.0, 8.0, 6.0], [4.0, 7.0, 3.0], [4.0, 1.0, 8.0], [6.0, 1.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 5.0, 4.0], [6.0, 4.0, 6.0], [2.0, 8.0, 2.0], [2.0, 6.0, 2.0]], [[3.0, 8.0, 8.0], 
[7.0, 2.0, 7.0], [1.0, 7.0, 1.0], [1.0, 4.0, 5.0]], [[8.0, 7.0, 1.0], [3.0, 1.0, 6.0], [5.0, 5.0, 5.0], [7.0, 6.0, 6.0]]]))
      val=Data(numpy.array([[6.0], [1.0], [7.0], [7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0], [8.0], [6.0], [8.0]]))
      arg[1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 7.0], [7.0, 8.0, 2.0], [6.0, 8.0, 1.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 6.0], [4.0, 
7.0, 1.0], [7.0, 8.0, 7.0], [5.0, 7.0, 7.0]], [[6.0, 8.0, 6.0], [4.0, 7.0, 3.0], [4.0, 1.0, 8.0], [6.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 4.0], [6.0, 4.0, 6.0], [2.0, 8.0, 2.0], [2.0, 6.0, 2.0]], [[3.0, 8.0, 6.0], [7.0, 
2.0, 8.0], [1.0, 7.0, 6.0], [1.0, 4.0, 8.0]], [[8.0, 7.0, 1.0], [3.0, 1.0, 6.0], [5.0, 5.0, 5.0], [7.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PCL(self):
      arg=Data(numpy.array([[[7.0, 4.0, 2.0], [2.0, 5.0, 3.0], [7.0, 3.0, 2.0], [6.0, 3.0, 7.0]], [[5.0, 3.0, 7.0], [5.0, 2.0, 
4.0], [2.0, 5.0, 5.0], [7.0, 7.0, 7.0]], [[1.0, 4.0, 4.0], [2.0, 2.0, 6.0], [2.0, 4.0, 2.0], [3.0, 2.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 8.0, 1.0], [4.0, 8.0, 3.0], [4.0, 8.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 4.0, 2.0], 
[1.0, 5.0, 7.0], [7.0, 6.0, 8.0], [5.0, 3.0, 6.0]], [[3.0, 3.0, 3.0], [5.0, 6.0, 6.0], [6.0, 1.0, 2.0], [8.0, 7.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[7.0], [4.0], [7.0], [1.0]])+(1.-msk_val)*numpy.array([[5.0], [1.0], [2.0], [2.0]])
      arg[1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 2.0], [2.0, 5.0, 3.0], [7.0, 3.0, 2.0], [6.0, 3.0, 7.0]], [[5.0, 3.0, 7.0], [5.0, 
2.0, 4.0], [2.0, 5.0, 7.0], [7.0, 7.0, 1.0]], [[1.0, 4.0, 4.0], [2.0, 2.0, 6.0], [2.0, 4.0, 2.0], [3.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 1.0], [4.0, 8.0, 3.0], [4.0, 8.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 4.0, 5.0], [1.0, 
5.0, 1.0], [7.0, 6.0, 2.0], [5.0, 3.0, 2.0]], [[3.0, 3.0, 3.0], [5.0, 6.0, 6.0], [6.0, 1.0, 2.0], [8.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LLU(self):
      arg=Data(numpy.array([[[2.0, 4.0, 8.0], [5.0, 1.0, 5.0], [1.0, 8.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 6.0, 5.0], [6.0, 2.0, 
4.0], [5.0, 6.0, 5.0], [5.0, 7.0, 6.0]], [[1.0, 4.0, 5.0], [8.0, 2.0, 6.0], [3.0, 2.0, 1.0], [4.0, 1.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 6.0, 7.0], [4.0, 5.0, 1.0], [7.0, 3.0, 4.0], [3.0, 8.0, 2.0]], [[7.0, 3.0, 4.0], 
[3.0, 8.0, 7.0], [1.0, 3.0, 4.0], [2.0, 7.0, 4.0]], [[1.0, 8.0, 5.0], [4.0, 7.0, 1.0], [1.0, 5.0, 5.0], [6.0, 5.0, 8.0]]]))
      val=numpy.array([[[5.0, 3.0], [6.0, 6.0]]])
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 8.0], [5.0, 1.0, 5.0], [1.0, 8.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 6.0, 5.0], [6.0, 
2.0, 4.0], [5.0, 6.0, 5.0], [5.0, 7.0, 6.0]], [[1.0, 4.0, 5.0], [8.0, 2.0, 6.0], [5.0, 3.0, 1.0], [6.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 6.0, 7.0], [4.0, 5.0, 1.0], [7.0, 3.0, 4.0], [3.0, 8.0, 2.0]], [[7.0, 3.0, 4.0], [3.0, 
8.0, 7.0], [1.0, 3.0, 4.0], [2.0, 7.0, 4.0]], [[1.0, 8.0, 5.0], [4.0, 7.0, 1.0], [5.0, 3.0, 5.0], [6.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LLU(self):
      arg=Data(numpy.array([[[8.0, 3.0, 7.0], [5.0, 6.0, 5.0], [1.0, 7.0, 2.0], [2.0, 5.0, 5.0]], [[6.0, 3.0, 7.0], [6.0, 7.0, 
2.0], [3.0, 7.0, 2.0], [7.0, 6.0, 5.0]], [[8.0, 2.0, 7.0], [2.0, 7.0, 3.0], [1.0, 1.0, 6.0], [2.0, 4.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 5.0, 5.0], [8.0, 3.0, 5.0], [2.0, 4.0, 2.0], [2.0, 7.0, 8.0]], [[2.0, 5.0, 6.0], 
[3.0, 4.0, 2.0], [3.0, 2.0, 1.0], [8.0, 8.0, 6.0]], [[2.0, 3.0, 8.0], [3.0, 8.0, 4.0], [3.0, 4.0, 2.0], [7.0, 8.0, 2.0]]]))
      val=Data(numpy.array([[[2.0, 4.0], [7.0, 3.0]]]),self.functionspace)
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 7.0], [5.0, 6.0, 5.0], [1.0, 7.0, 2.0], [2.0, 5.0, 5.0]], [[6.0, 3.0, 7.0], [6.0, 
7.0, 2.0], [3.0, 7.0, 2.0], [7.0, 6.0, 5.0]], [[8.0, 2.0, 7.0], [2.0, 7.0, 3.0], [2.0, 4.0, 6.0], [7.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 5.0], [8.0, 3.0, 5.0], [2.0, 4.0, 2.0], [2.0, 7.0, 8.0]], [[2.0, 5.0, 6.0], [3.0, 
4.0, 2.0], [3.0, 2.0, 1.0], [8.0, 8.0, 6.0]], [[2.0, 3.0, 8.0], [3.0, 8.0, 4.0], [2.0, 4.0, 2.0], [7.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LLU(self):
      arg=Data(numpy.array([[[2.0, 2.0, 5.0], [4.0, 7.0, 4.0], [8.0, 3.0, 4.0], [4.0, 6.0, 6.0]], [[4.0, 4.0, 4.0], [6.0, 6.0, 
5.0], [2.0, 8.0, 8.0], [1.0, 3.0, 6.0]], [[4.0, 6.0, 8.0], [5.0, 6.0, 1.0], [3.0, 4.0, 6.0], [7.0, 4.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 8.0, 3.0], [8.0, 6.0, 3.0], [2.0, 2.0, 1.0], [2.0, 2.0, 3.0]], [[8.0, 6.0, 3.0], 
[2.0, 4.0, 1.0], [4.0, 1.0, 7.0], [8.0, 2.0, 3.0]], [[4.0, 1.0, 6.0], [3.0, 2.0, 8.0], [5.0, 6.0, 2.0], [7.0, 7.0, 6.0]]]))
      val=Data(numpy.array([[[4.0, 1.0], [4.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 1.0], [6.0, 3.0]]]))
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 5.0], [4.0, 7.0, 4.0], [8.0, 3.0, 4.0], [4.0, 6.0, 6.0]], [[4.0, 4.0, 4.0], [6.0, 
6.0, 5.0], [2.0, 8.0, 8.0], [1.0, 3.0, 6.0]], [[4.0, 6.0, 8.0], [5.0, 6.0, 1.0], [4.0, 1.0, 6.0], [4.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 3.0], [8.0, 6.0, 3.0], [2.0, 2.0, 1.0], [2.0, 2.0, 3.0]], [[8.0, 6.0, 3.0], [2.0, 
4.0, 1.0], [4.0, 1.0, 7.0], [8.0, 2.0, 3.0]], [[4.0, 1.0, 6.0], [3.0, 2.0, 8.0], [5.0, 1.0, 2.0], [6.0, 3.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LLU(self):
      arg=Data(numpy.array([[[3.0, 8.0, 5.0], [1.0, 3.0, 6.0], [2.0, 3.0, 7.0], [6.0, 7.0, 5.0]], [[4.0, 2.0, 7.0], [5.0, 1.0, 
1.0], [1.0, 5.0, 2.0], [5.0, 4.0, 3.0]], [[8.0, 6.0, 7.0], [8.0, 7.0, 6.0], [2.0, 4.0, 3.0], [7.0, 4.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 3.0, 1.0], [3.0, 4.0, 3.0], [4.0, 8.0, 4.0], [1.0, 1.0, 1.0]], [[5.0, 2.0, 4.0], 
[8.0, 8.0, 8.0], [8.0, 5.0, 3.0], [1.0, 8.0, 1.0]], [[1.0, 3.0, 8.0], [7.0, 1.0, 8.0], [8.0, 4.0, 7.0], [4.0, 6.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 7.0], [7.0, 5.0]]])+(1.-msk_val)*numpy.array([[[7.0, 1.0], [6.0, 6.0]]])
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 5.0], [1.0, 3.0, 6.0], [2.0, 3.0, 7.0], [6.0, 7.0, 5.0]], [[4.0, 2.0, 7.0], [5.0, 
1.0, 1.0], [1.0, 5.0, 2.0], [5.0, 4.0, 3.0]], [[8.0, 6.0, 7.0], [8.0, 7.0, 6.0], [5.0, 7.0, 3.0], [7.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 1.0], [3.0, 4.0, 3.0], [4.0, 8.0, 4.0], [1.0, 1.0, 1.0]], [[5.0, 2.0, 4.0], [8.0, 
8.0, 8.0], [8.0, 5.0, 3.0], [1.0, 8.0, 1.0]], [[1.0, 3.0, 8.0], [7.0, 1.0, 8.0], [7.0, 1.0, 7.0], [6.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CCC(self):
      arg=Data(numpy.array([[[5.0, 5.0, 2.0], [1.0, 1.0, 1.0], [6.0, 2.0, 7.0], [2.0, 8.0, 4.0]], [[4.0, 5.0, 6.0], [2.0, 1.0, 
8.0], [1.0, 1.0, 2.0], [1.0, 5.0, 7.0]], [[7.0, 3.0, 5.0], [8.0, 8.0, 8.0], [6.0, 5.0, 1.0], [1.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 1.0, 8.0], [4.0, 1.0, 8.0], [1.0, 6.0, 2.0], [2.0, 3.0, 3.0]], [[3.0, 3.0, 2.0], 
[1.0, 4.0, 7.0], [7.0, 2.0, 8.0], [4.0, 5.0, 6.0]], [[3.0, 2.0, 1.0], [2.0, 2.0, 8.0], [7.0, 3.0, 8.0], [7.0, 2.0, 3.0]]]))
      val=numpy.array([[[2.0, 5.0, 2.0], [8.0, 4.0, 7.0], [1.0, 6.0, 4.0], [8.0, 8.0, 6.0]], [[4.0, 1.0, 1.0], [5.0, 6.0, 8.0], 
[3.0, 1.0, 5.0], [8.0, 8.0, 8.0]], [[6.0, 5.0, 6.0], [1.0, 8.0, 6.0], [2.0, 6.0, 2.0], [1.0, 8.0, 2.0]]])
      arg[:,:,:]=val
      ref=Data(numpy.array([[[2.0, 5.0, 2.0], [8.0, 4.0, 7.0], [1.0, 6.0, 4.0], [8.0, 8.0, 6.0]], [[4.0, 1.0, 1.0], [5.0, 6.0, 
8.0], [3.0, 1.0, 5.0], [8.0, 8.0, 8.0]], [[6.0, 5.0, 6.0], [1.0, 8.0, 6.0], [2.0, 6.0, 2.0], [1.0, 8.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CCC(self):
      arg=Data(numpy.array([[[4.0, 7.0, 3.0], [4.0, 6.0, 7.0], [4.0, 8.0, 1.0], [2.0, 5.0, 1.0]], [[2.0, 8.0, 3.0], [1.0, 1.0, 
3.0], [5.0, 8.0, 3.0], [4.0, 6.0, 7.0]], [[7.0, 6.0, 6.0], [1.0, 8.0, 4.0], [5.0, 8.0, 6.0], [2.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 7.0, 5.0], [1.0, 8.0, 1.0], [3.0, 1.0, 2.0], [8.0, 6.0, 4.0]], [[2.0, 5.0, 2.0], 
[4.0, 1.0, 4.0], [7.0, 5.0, 2.0], [8.0, 7.0, 7.0]], [[2.0, 5.0, 5.0], [3.0, 3.0, 4.0], [4.0, 7.0, 1.0], [3.0, 3.0, 4.0]]]))
      val=Data(numpy.array([[[6.0, 3.0, 7.0], [1.0, 1.0, 7.0], [5.0, 5.0, 8.0], [2.0, 1.0, 1.0]], [[2.0, 8.0, 8.0], [6.0, 6.0, 
7.0], [8.0, 3.0, 4.0], [2.0, 4.0, 6.0]], [[6.0, 4.0, 5.0], [6.0, 3.0, 8.0], [2.0, 4.0, 1.0], [5.0, 6.0, 
3.0]]]),self.functionspace)
      arg[:,:,:]=val
      ref=Data(numpy.array([[[6.0, 3.0, 7.0], [1.0, 1.0, 7.0], [5.0, 5.0, 8.0], [2.0, 1.0, 1.0]], [[2.0, 8.0, 8.0], [6.0, 6.0, 
7.0], [8.0, 3.0, 4.0], [2.0, 4.0, 6.0]], [[6.0, 4.0, 5.0], [6.0, 3.0, 8.0], [2.0, 4.0, 1.0], [5.0, 6.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CCC(self):
      arg=Data(numpy.array([[[4.0, 3.0, 1.0], [7.0, 6.0, 6.0], [1.0, 3.0, 8.0], [4.0, 1.0, 8.0]], [[4.0, 4.0, 7.0], [7.0, 3.0, 
6.0], [4.0, 7.0, 1.0], [6.0, 6.0, 4.0]], [[8.0, 6.0, 6.0], [5.0, 4.0, 8.0], [6.0, 1.0, 1.0], [7.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 3.0], [6.0, 7.0, 6.0], [4.0, 3.0, 1.0], [4.0, 6.0, 7.0]], [[8.0, 6.0, 2.0], 
[8.0, 3.0, 8.0], [5.0, 7.0, 8.0], [2.0, 6.0, 1.0]], [[7.0, 4.0, 5.0], [4.0, 7.0, 4.0], [1.0, 1.0, 8.0], [6.0, 1.0, 3.0]]]))
      val=Data(numpy.array([[[6.0, 6.0, 3.0], [3.0, 3.0, 6.0], [6.0, 2.0, 6.0], [5.0, 7.0, 5.0]], [[1.0, 7.0, 3.0], [6.0, 5.0, 
5.0], [1.0, 8.0, 7.0], [5.0, 5.0, 1.0]], [[8.0, 7.0, 5.0], [3.0, 2.0, 1.0], [5.0, 8.0, 3.0], [8.0, 6.0, 
8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 7.0, 5.0], [6.0, 4.0, 3.0], [4.0, 3.0, 7.0], [4.0, 1.0, 6.0]], [[4.0, 6.0, 3.0], 
[4.0, 2.0, 6.0], [1.0, 2.0, 4.0], [2.0, 8.0, 7.0]], [[5.0, 6.0, 3.0], [5.0, 6.0, 2.0], [5.0, 5.0, 4.0], [2.0, 8.0, 7.0]]]))
      arg[:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 3.0], [3.0, 3.0, 6.0], [6.0, 2.0, 6.0], [5.0, 7.0, 5.0]], [[1.0, 7.0, 3.0], [6.0, 
5.0, 5.0], [1.0, 8.0, 7.0], [5.0, 5.0, 1.0]], [[8.0, 7.0, 5.0], [3.0, 2.0, 1.0], [5.0, 8.0, 3.0], [8.0, 6.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 7.0, 5.0], [6.0, 4.0, 3.0], [4.0, 3.0, 7.0], [4.0, 1.0, 6.0]], [[4.0, 6.0, 3.0], [4.0, 
2.0, 6.0], [1.0, 2.0, 4.0], [2.0, 8.0, 7.0]], [[5.0, 6.0, 3.0], [5.0, 6.0, 2.0], [5.0, 5.0, 4.0], [2.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CCC(self):
      arg=Data(numpy.array([[[1.0, 8.0, 2.0], [6.0, 7.0, 7.0], [2.0, 7.0, 5.0], [4.0, 5.0, 2.0]], [[5.0, 7.0, 1.0], [1.0, 2.0, 
6.0], [2.0, 5.0, 3.0], [6.0, 6.0, 4.0]], [[7.0, 2.0, 3.0], [6.0, 3.0, 5.0], [1.0, 8.0, 4.0], [5.0, 6.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 8.0, 6.0], [3.0, 8.0, 8.0], [2.0, 2.0, 1.0], [3.0, 4.0, 4.0]], [[8.0, 5.0, 8.0], 
[4.0, 4.0, 1.0], [7.0, 1.0, 8.0], [4.0, 4.0, 3.0]], [[6.0, 8.0, 1.0], [8.0, 2.0, 5.0], [1.0, 6.0, 3.0], [3.0, 7.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 7.0, 1.0], [6.0, 5.0, 5.0], [8.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[6.0, 4.0, 7.0], [5.0, 
7.0, 7.0], [4.0, 5.0, 5.0], [6.0, 2.0, 2.0]], [[7.0, 4.0, 2.0], [2.0, 6.0, 1.0], [1.0, 3.0, 1.0], [3.0, 2.0, 
5.0]]])+(1.-msk_val)*numpy.array([[[3.0, 8.0, 4.0], [2.0, 3.0, 3.0], [1.0, 3.0, 5.0], [6.0, 3.0, 4.0]], [[7.0, 5.0, 8.0], [1.0, 
7.0, 3.0], [1.0, 6.0, 2.0], [1.0, 2.0, 6.0]], [[3.0, 1.0, 7.0], [2.0, 8.0, 3.0], [2.0, 7.0, 7.0], [8.0, 5.0, 8.0]]])
      arg[:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 7.0, 1.0], [6.0, 5.0, 5.0], [8.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[6.0, 4.0, 7.0], [5.0, 
7.0, 7.0], [4.0, 5.0, 5.0], [6.0, 2.0, 2.0]], [[7.0, 4.0, 2.0], [2.0, 6.0, 1.0], [1.0, 3.0, 1.0], [3.0, 2.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 4.0], [2.0, 3.0, 3.0], [1.0, 3.0, 5.0], [6.0, 3.0, 4.0]], [[7.0, 5.0, 8.0], [1.0, 
7.0, 3.0], [1.0, 6.0, 2.0], [1.0, 2.0, 6.0]], [[3.0, 1.0, 7.0], [2.0, 8.0, 3.0], [2.0, 7.0, 7.0], [8.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PLU(self):
      arg=Data(numpy.array([[[7.0, 1.0, 7.0], [3.0, 4.0, 1.0], [8.0, 6.0, 2.0], [6.0, 8.0, 6.0]], [[6.0, 8.0, 7.0], [1.0, 4.0, 
5.0], [1.0, 1.0, 1.0], [1.0, 5.0, 6.0]], [[8.0, 2.0, 8.0], [3.0, 6.0, 6.0], [1.0, 8.0, 7.0], [3.0, 7.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 5.0, 7.0], [1.0, 1.0, 3.0], [7.0, 3.0, 8.0], [8.0, 1.0, 8.0]], [[7.0, 4.0, 8.0], 
[8.0, 4.0, 3.0], [2.0, 4.0, 3.0], [3.0, 7.0, 4.0]], [[7.0, 5.0, 3.0], [3.0, 3.0, 8.0], [2.0, 4.0, 4.0], [7.0, 6.0, 1.0]]]))
      val=numpy.array([[2.0, 3.0], [5.0, 2.0]])
      arg[1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 1.0, 7.0], [3.0, 4.0, 1.0], [8.0, 6.0, 2.0], [6.0, 8.0, 6.0]], [[6.0, 8.0, 7.0], [1.0, 
4.0, 5.0], [2.0, 3.0, 1.0], [5.0, 2.0, 6.0]], [[8.0, 2.0, 8.0], [3.0, 6.0, 6.0], [1.0, 8.0, 7.0], [3.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 7.0], [1.0, 1.0, 3.0], [7.0, 3.0, 8.0], [8.0, 1.0, 8.0]], [[7.0, 4.0, 8.0], [8.0, 
4.0, 3.0], [2.0, 3.0, 3.0], [5.0, 2.0, 4.0]], [[7.0, 5.0, 3.0], [3.0, 3.0, 8.0], [2.0, 4.0, 4.0], [7.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PLU(self):
      arg=Data(numpy.array([[[5.0, 1.0, 3.0], [5.0, 4.0, 1.0], [4.0, 6.0, 1.0], [6.0, 1.0, 6.0]], [[2.0, 7.0, 7.0], [4.0, 8.0, 
2.0], [7.0, 7.0, 7.0], [1.0, 5.0, 5.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 8.0], [1.0, 3.0, 3.0], [1.0, 7.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 8.0, 1.0], [6.0, 7.0, 5.0], [2.0, 7.0, 8.0], [3.0, 1.0, 5.0]], [[2.0, 4.0, 7.0], 
[8.0, 2.0, 1.0], [6.0, 2.0, 5.0], [7.0, 6.0, 8.0]], [[4.0, 7.0, 1.0], [6.0, 4.0, 1.0], [5.0, 5.0, 6.0], [7.0, 1.0, 4.0]]]))
      val=Data(numpy.array([[7.0, 5.0], [3.0, 5.0]]),self.functionspace)
      arg[1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 3.0], [5.0, 4.0, 1.0], [4.0, 6.0, 1.0], [6.0, 1.0, 6.0]], [[2.0, 7.0, 7.0], [4.0, 
8.0, 2.0], [7.0, 5.0, 7.0], [3.0, 5.0, 5.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 8.0], [1.0, 3.0, 3.0], [1.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 1.0], [6.0, 7.0, 5.0], [2.0, 7.0, 8.0], [3.0, 1.0, 5.0]], [[2.0, 4.0, 7.0], [8.0, 
2.0, 1.0], [7.0, 5.0, 5.0], [3.0, 5.0, 8.0]], [[4.0, 7.0, 1.0], [6.0, 4.0, 1.0], [5.0, 5.0, 6.0], [7.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PLU(self):
      arg=Data(numpy.array([[[2.0, 4.0, 4.0], [8.0, 6.0, 2.0], [4.0, 7.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 2.0, 1.0], [8.0, 2.0, 
8.0], [1.0, 8.0, 2.0], [4.0, 8.0, 3.0]], [[5.0, 1.0, 6.0], [1.0, 5.0, 4.0], [3.0, 2.0, 2.0], [2.0, 4.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 7.0], [4.0, 2.0, 3.0], [2.0, 4.0, 4.0], [5.0, 8.0, 6.0]], [[6.0, 4.0, 3.0], 
[2.0, 1.0, 3.0], [4.0, 8.0, 8.0], [7.0, 8.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 2.0, 5.0], [6.0, 6.0, 4.0], [1.0, 6.0, 6.0]]]))
      val=Data(numpy.array([[6.0, 5.0], [7.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 1.0], [3.0, 5.0]]))
      arg[1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 4.0], [8.0, 6.0, 2.0], [4.0, 7.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 2.0, 1.0], [8.0, 
2.0, 8.0], [6.0, 5.0, 2.0], [7.0, 5.0, 3.0]], [[5.0, 1.0, 6.0], [1.0, 5.0, 4.0], [3.0, 2.0, 2.0], [2.0, 4.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 7.0], [4.0, 2.0, 3.0], [2.0, 4.0, 4.0], [5.0, 8.0, 6.0]], [[6.0, 4.0, 3.0], [2.0, 
1.0, 3.0], [6.0, 1.0, 8.0], [3.0, 5.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 2.0, 5.0], [6.0, 6.0, 4.0], [1.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PLU(self):
      arg=Data(numpy.array([[[2.0, 2.0, 6.0], [2.0, 7.0, 8.0], [1.0, 4.0, 3.0], [7.0, 5.0, 1.0]], [[8.0, 6.0, 1.0], [1.0, 1.0, 
7.0], [6.0, 5.0, 1.0], [4.0, 7.0, 4.0]], [[6.0, 5.0, 6.0], [4.0, 1.0, 6.0], [2.0, 2.0, 2.0], [8.0, 2.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 3.0, 5.0], [1.0, 6.0, 1.0], [4.0, 8.0, 2.0], [1.0, 5.0, 7.0]], [[7.0, 3.0, 1.0], 
[8.0, 8.0, 4.0], [4.0, 3.0, 7.0], [3.0, 4.0, 1.0]], [[7.0, 1.0, 5.0], [3.0, 5.0, 8.0], [6.0, 7.0, 3.0], [2.0, 8.0, 5.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[8.0, 6.0], [6.0, 5.0]])+(1.-msk_val)*numpy.array([[1.0, 2.0], [2.0, 3.0]])
      arg[1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 6.0], [2.0, 7.0, 8.0], [1.0, 4.0, 3.0], [7.0, 5.0, 1.0]], [[8.0, 6.0, 1.0], [1.0, 
1.0, 7.0], [8.0, 6.0, 1.0], [6.0, 5.0, 4.0]], [[6.0, 5.0, 6.0], [4.0, 1.0, 6.0], [2.0, 2.0, 2.0], [8.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 5.0], [1.0, 6.0, 1.0], [4.0, 8.0, 2.0], [1.0, 5.0, 7.0]], [[7.0, 3.0, 1.0], [8.0, 
8.0, 4.0], [1.0, 2.0, 7.0], [2.0, 3.0, 1.0]], [[7.0, 1.0, 5.0], [3.0, 5.0, 8.0], [6.0, 7.0, 3.0], [2.0, 8.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CLN(self):
      arg=Data(numpy.array([[[5.0, 7.0, 3.0], [6.0, 3.0, 6.0], [5.0, 3.0, 3.0], [3.0, 2.0, 8.0]], [[1.0, 2.0, 3.0], [6.0, 5.0, 
3.0], [1.0, 2.0, 3.0], [8.0, 3.0, 4.0]], [[2.0, 5.0, 8.0], [1.0, 8.0, 6.0], [5.0, 7.0, 3.0], [3.0, 3.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 3.0, 5.0], [7.0, 5.0, 6.0], [5.0, 1.0, 8.0], [2.0, 7.0, 7.0]], [[3.0, 2.0, 5.0], 
[3.0, 2.0, 4.0], [4.0, 1.0, 2.0], [8.0, 5.0, 1.0]], [[3.0, 6.0, 4.0], [5.0, 3.0, 2.0], [8.0, 2.0, 8.0], [3.0, 4.0, 1.0]]]))
      val=numpy.array([[[4.0, 6.0, 8.0], [5.0, 7.0, 3.0]], [[7.0, 1.0, 5.0], [2.0, 8.0, 8.0]], [[7.0, 2.0, 5.0], [5.0, 6.0, 
6.0]]])
      arg[:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 3.0], [6.0, 3.0, 6.0], [4.0, 6.0, 8.0], [5.0, 7.0, 3.0]], [[1.0, 2.0, 3.0], [6.0, 
5.0, 3.0], [7.0, 1.0, 5.0], [2.0, 8.0, 8.0]], [[2.0, 5.0, 8.0], [1.0, 8.0, 6.0], [7.0, 2.0, 5.0], [5.0, 6.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 5.0], [7.0, 5.0, 6.0], [4.0, 6.0, 8.0], [5.0, 7.0, 3.0]], [[3.0, 2.0, 5.0], [3.0, 
2.0, 4.0], [7.0, 1.0, 5.0], [2.0, 8.0, 8.0]], [[3.0, 6.0, 4.0], [5.0, 3.0, 2.0], [7.0, 2.0, 5.0], [5.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CLN(self):
      arg=Data(numpy.array([[[3.0, 6.0, 2.0], [2.0, 4.0, 6.0], [3.0, 3.0, 7.0], [7.0, 7.0, 2.0]], [[1.0, 5.0, 3.0], [7.0, 1.0, 
4.0], [6.0, 6.0, 4.0], [2.0, 6.0, 8.0]], [[6.0, 1.0, 8.0], [2.0, 7.0, 8.0], [3.0, 4.0, 4.0], [5.0, 8.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 7.0, 1.0], [1.0, 8.0, 2.0], [4.0, 6.0, 4.0], [4.0, 5.0, 1.0]], [[6.0, 1.0, 3.0], 
[4.0, 5.0, 5.0], [1.0, 8.0, 5.0], [1.0, 1.0, 2.0]], [[8.0, 2.0, 2.0], [5.0, 5.0, 5.0], [2.0, 6.0, 8.0], [2.0, 7.0, 6.0]]]))
      val=Data(numpy.array([[[3.0, 1.0, 4.0], [8.0, 4.0, 2.0]], [[4.0, 1.0, 8.0], [6.0, 8.0, 7.0]], [[8.0, 6.0, 3.0], [4.0, 
8.0, 1.0]]]),self.functionspace)
      arg[:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 2.0], [2.0, 4.0, 6.0], [3.0, 1.0, 4.0], [8.0, 4.0, 2.0]], [[1.0, 5.0, 3.0], [7.0, 
1.0, 4.0], [4.0, 1.0, 8.0], [6.0, 8.0, 7.0]], [[6.0, 1.0, 8.0], [2.0, 7.0, 8.0], [8.0, 6.0, 3.0], [4.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 7.0, 1.0], [1.0, 8.0, 2.0], [3.0, 1.0, 4.0], [8.0, 4.0, 2.0]], [[6.0, 1.0, 3.0], [4.0, 
5.0, 5.0], [4.0, 1.0, 8.0], [6.0, 8.0, 7.0]], [[8.0, 2.0, 2.0], [5.0, 5.0, 5.0], [8.0, 6.0, 3.0], [4.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CLN(self):
      arg=Data(numpy.array([[[8.0, 5.0, 4.0], [3.0, 3.0, 6.0], [6.0, 2.0, 8.0], [4.0, 7.0, 2.0]], [[6.0, 3.0, 8.0], [7.0, 4.0, 
4.0], [1.0, 6.0, 3.0], [4.0, 5.0, 3.0]], [[4.0, 1.0, 3.0], [4.0, 5.0, 3.0], [2.0, 2.0, 2.0], [5.0, 2.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 7.0, 4.0], [4.0, 5.0, 8.0], [1.0, 1.0, 1.0], [6.0, 4.0, 1.0]], [[2.0, 7.0, 6.0], 
[7.0, 3.0, 8.0], [1.0, 3.0, 6.0], [6.0, 4.0, 6.0]], [[1.0, 2.0, 2.0], [4.0, 8.0, 8.0], [3.0, 5.0, 8.0], [3.0, 6.0, 8.0]]]))
      val=Data(numpy.array([[[2.0, 2.0, 3.0], [7.0, 3.0, 8.0]], [[5.0, 7.0, 1.0], [7.0, 4.0, 6.0]], [[6.0, 2.0, 7.0], [3.0, 
1.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0, 6.0], [4.0, 5.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 1.0, 7.0]], [[7.0, 8.0, 
2.0], [6.0, 7.0, 2.0]]]))
      arg[:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 4.0], [3.0, 3.0, 6.0], [2.0, 2.0, 3.0], [7.0, 3.0, 8.0]], [[6.0, 3.0, 8.0], [7.0, 
4.0, 4.0], [5.0, 7.0, 1.0], [7.0, 4.0, 6.0]], [[4.0, 1.0, 3.0], [4.0, 5.0, 3.0], [6.0, 2.0, 7.0], [3.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 4.0], [4.0, 5.0, 8.0], [3.0, 5.0, 6.0], [4.0, 5.0, 8.0]], [[2.0, 7.0, 6.0], [7.0, 
3.0, 8.0], [1.0, 5.0, 8.0], [5.0, 1.0, 7.0]], [[1.0, 2.0, 2.0], [4.0, 8.0, 8.0], [7.0, 8.0, 2.0], [6.0, 7.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CLN(self):
      arg=Data(numpy.array([[[4.0, 3.0, 8.0], [5.0, 8.0, 8.0], [3.0, 5.0, 4.0], [2.0, 3.0, 6.0]], [[6.0, 7.0, 7.0], [2.0, 5.0, 
4.0], [7.0, 8.0, 4.0], [4.0, 2.0, 1.0]], [[8.0, 5.0, 4.0], [6.0, 3.0, 1.0], [6.0, 6.0, 3.0], [2.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 4.0, 2.0], [1.0, 6.0, 1.0], [2.0, 7.0, 6.0], [1.0, 1.0, 7.0]], [[1.0, 1.0, 3.0], 
[1.0, 3.0, 6.0], [6.0, 2.0, 5.0], [7.0, 2.0, 1.0]], [[7.0, 8.0, 2.0], [6.0, 6.0, 5.0], [8.0, 6.0, 4.0], [8.0, 2.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 1.0, 6.0], [2.0, 8.0, 8.0]], [[2.0, 6.0, 6.0], [4.0, 8.0, 2.0]], [[2.0, 7.0, 3.0], [5.0, 
3.0, 8.0]]])+(1.-msk_val)*numpy.array([[[1.0, 4.0, 3.0], [5.0, 8.0, 4.0]], [[2.0, 1.0, 1.0], [4.0, 4.0, 4.0]], [[1.0, 6.0, 
8.0], [2.0, 6.0, 4.0]]])
      arg[:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 8.0], [5.0, 8.0, 8.0], [5.0, 1.0, 6.0], [2.0, 8.0, 8.0]], [[6.0, 7.0, 7.0], [2.0, 
5.0, 4.0], [2.0, 6.0, 6.0], [4.0, 8.0, 2.0]], [[8.0, 5.0, 4.0], [6.0, 3.0, 1.0], [2.0, 7.0, 3.0], [5.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 2.0], [1.0, 6.0, 1.0], [1.0, 4.0, 3.0], [5.0, 8.0, 4.0]], [[1.0, 1.0, 3.0], [1.0, 
3.0, 6.0], [2.0, 1.0, 1.0], [4.0, 4.0, 4.0]], [[7.0, 8.0, 2.0], [6.0, 6.0, 5.0], [1.0, 6.0, 8.0], [2.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LRU(self):
      arg=Data(numpy.array([[[2.0, 3.0, 5.0], [7.0, 6.0, 6.0], [8.0, 3.0, 4.0], [3.0, 6.0, 6.0]], [[5.0, 2.0, 7.0], [6.0, 4.0, 
5.0], [5.0, 5.0, 2.0], [6.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], [5.0, 6.0, 5.0], [4.0, 8.0, 8.0], [6.0, 5.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 4.0, 2.0], [1.0, 8.0, 2.0], [1.0, 1.0, 4.0], [7.0, 8.0, 3.0]], [[1.0, 6.0, 7.0], 
[7.0, 3.0, 1.0], [6.0, 5.0, 4.0], [7.0, 2.0, 7.0]], [[8.0, 4.0, 3.0], [3.0, 6.0, 8.0], [4.0, 4.0, 7.0], [1.0, 8.0, 4.0]]]))
      val=numpy.array([[[7.0, 7.0], [4.0, 4.0]]])
      arg[2:,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 3.0, 5.0], [7.0, 6.0, 6.0], [8.0, 3.0, 4.0], [3.0, 6.0, 6.0]], [[5.0, 2.0, 7.0], [6.0, 
4.0, 5.0], [5.0, 5.0, 2.0], [6.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], [7.0, 7.0, 5.0], [4.0, 4.0, 8.0], [6.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 2.0], [1.0, 8.0, 2.0], [1.0, 1.0, 4.0], [7.0, 8.0, 3.0]], [[1.0, 6.0, 7.0], [7.0, 
3.0, 1.0], [6.0, 5.0, 4.0], [7.0, 2.0, 7.0]], [[8.0, 4.0, 3.0], [7.0, 7.0, 8.0], [4.0, 4.0, 7.0], [1.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LRU(self):
      arg=Data(numpy.array([[[6.0, 7.0, 4.0], [7.0, 8.0, 1.0], [8.0, 7.0, 5.0], [1.0, 1.0, 5.0]], [[4.0, 6.0, 1.0], [7.0, 6.0, 
4.0], [4.0, 7.0, 7.0], [5.0, 1.0, 4.0]], [[7.0, 1.0, 1.0], [5.0, 2.0, 1.0], [4.0, 2.0, 1.0], [3.0, 6.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 5.0], [6.0, 7.0, 1.0], [6.0, 1.0, 8.0], [3.0, 7.0, 1.0]], [[2.0, 4.0, 1.0], 
[5.0, 5.0, 4.0], [1.0, 7.0, 5.0], [3.0, 4.0, 8.0]], [[5.0, 5.0, 5.0], [6.0, 4.0, 2.0], [5.0, 1.0, 8.0], [6.0, 6.0, 2.0]]]))
      val=Data(numpy.array([[[8.0, 2.0], [7.0, 4.0]]]),self.functionspace)
      arg[2:,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 4.0], [7.0, 8.0, 1.0], [8.0, 7.0, 5.0], [1.0, 1.0, 5.0]], [[4.0, 6.0, 1.0], [7.0, 
6.0, 4.0], [4.0, 7.0, 7.0], [5.0, 1.0, 4.0]], [[7.0, 1.0, 1.0], [8.0, 2.0, 1.0], [7.0, 4.0, 1.0], [3.0, 6.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 5.0], [6.0, 7.0, 1.0], [6.0, 1.0, 8.0], [3.0, 7.0, 1.0]], [[2.0, 4.0, 1.0], [5.0, 
5.0, 4.0], [1.0, 7.0, 5.0], [3.0, 4.0, 8.0]], [[5.0, 5.0, 5.0], [8.0, 2.0, 2.0], [7.0, 4.0, 8.0], [6.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LRU(self):
      arg=Data(numpy.array([[[2.0, 8.0, 6.0], [2.0, 8.0, 6.0], [3.0, 4.0, 7.0], [1.0, 4.0, 7.0]], [[4.0, 6.0, 6.0], [2.0, 8.0, 
2.0], [1.0, 4.0, 3.0], [7.0, 7.0, 7.0]], [[6.0, 1.0, 6.0], [1.0, 5.0, 7.0], [1.0, 6.0, 3.0], [8.0, 5.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 8.0, 1.0], [6.0, 8.0, 5.0], [5.0, 3.0, 4.0], [8.0, 2.0, 5.0]], [[3.0, 4.0, 8.0], 
[1.0, 3.0, 7.0], [7.0, 6.0, 7.0], [6.0, 6.0, 5.0]], [[5.0, 7.0, 8.0], [7.0, 8.0, 6.0], [1.0, 3.0, 7.0], [5.0, 8.0, 1.0]]]))
      val=Data(numpy.array([[[2.0, 3.0], [2.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 7.0], [4.0, 8.0]]]))
      arg[2:,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 8.0, 6.0], [2.0, 8.0, 6.0], [3.0, 4.0, 7.0], [1.0, 4.0, 7.0]], [[4.0, 6.0, 6.0], [2.0, 
8.0, 2.0], [1.0, 4.0, 3.0], [7.0, 7.0, 7.0]], [[6.0, 1.0, 6.0], [2.0, 3.0, 7.0], [2.0, 7.0, 3.0], [8.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 1.0], [6.0, 8.0, 5.0], [5.0, 3.0, 4.0], [8.0, 2.0, 5.0]], [[3.0, 4.0, 8.0], [1.0, 
3.0, 7.0], [7.0, 6.0, 7.0], [6.0, 6.0, 5.0]], [[5.0, 7.0, 8.0], [3.0, 7.0, 6.0], [4.0, 8.0, 7.0], [5.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LRU(self):
      arg=Data(numpy.array([[[8.0, 5.0, 1.0], [5.0, 2.0, 3.0], [2.0, 8.0, 1.0], [2.0, 2.0, 3.0]], [[5.0, 7.0, 5.0], [4.0, 4.0, 
8.0], [5.0, 6.0, 8.0], [3.0, 4.0, 8.0]], [[7.0, 5.0, 3.0], [5.0, 4.0, 8.0], [5.0, 7.0, 2.0], [6.0, 6.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 1.0], [4.0, 5.0, 2.0], [4.0, 2.0, 1.0], [7.0, 2.0, 4.0]], [[3.0, 2.0, 5.0], 
[7.0, 3.0, 1.0], [8.0, 1.0, 4.0], [2.0, 4.0, 5.0]], [[2.0, 2.0, 1.0], [3.0, 2.0, 2.0], [5.0, 8.0, 6.0], [7.0, 8.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 1.0], [8.0, 1.0]]])+(1.-msk_val)*numpy.array([[[5.0, 1.0], [3.0, 2.0]]])
      arg[2:,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 1.0], [5.0, 2.0, 3.0], [2.0, 8.0, 1.0], [2.0, 2.0, 3.0]], [[5.0, 7.0, 5.0], [4.0, 
4.0, 8.0], [5.0, 6.0, 8.0], [3.0, 4.0, 8.0]], [[7.0, 5.0, 3.0], [7.0, 1.0, 8.0], [8.0, 1.0, 2.0], [6.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 1.0], [4.0, 5.0, 2.0], [4.0, 2.0, 1.0], [7.0, 2.0, 4.0]], [[3.0, 2.0, 5.0], [7.0, 
3.0, 1.0], [8.0, 1.0, 4.0], [2.0, 4.0, 5.0]], [[2.0, 2.0, 1.0], [5.0, 1.0, 2.0], [3.0, 2.0, 6.0], [7.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LLP(self):
      arg=Data(numpy.array([[[6.0, 5.0, 7.0], [4.0, 3.0, 4.0], [4.0, 7.0, 2.0], [5.0, 1.0, 8.0]], [[4.0, 1.0, 1.0], [2.0, 8.0, 
8.0], [7.0, 1.0, 1.0], [6.0, 2.0, 4.0]], [[6.0, 3.0, 7.0], [5.0, 6.0, 3.0], [7.0, 6.0, 8.0], [4.0, 6.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 1.0, 7.0], [6.0, 1.0, 5.0], [3.0, 1.0, 2.0], [5.0, 3.0, 5.0]], [[2.0, 7.0, 8.0], 
[6.0, 6.0, 5.0], [1.0, 4.0, 5.0], [6.0, 8.0, 5.0]], [[7.0, 1.0, 5.0], [4.0, 6.0, 4.0], [1.0, 6.0, 1.0], [2.0, 7.0, 1.0]]]))
      val=numpy.array([[5.0, 2.0]])
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 7.0], [4.0, 3.0, 4.0], [4.0, 7.0, 2.0], [5.0, 1.0, 8.0]], [[4.0, 1.0, 1.0], [2.0, 
8.0, 8.0], [7.0, 1.0, 1.0], [6.0, 2.0, 4.0]], [[6.0, 3.0, 7.0], [5.0, 6.0, 3.0], [7.0, 5.0, 8.0], [4.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 7.0], [6.0, 1.0, 5.0], [3.0, 1.0, 2.0], [5.0, 3.0, 5.0]], [[2.0, 7.0, 8.0], [6.0, 
6.0, 5.0], [1.0, 4.0, 5.0], [6.0, 8.0, 5.0]], [[7.0, 1.0, 5.0], [4.0, 6.0, 4.0], [1.0, 5.0, 1.0], [2.0, 2.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LLP(self):
      arg=Data(numpy.array([[[7.0, 7.0, 1.0], [2.0, 1.0, 6.0], [4.0, 6.0, 4.0], [5.0, 4.0, 4.0]], [[5.0, 7.0, 1.0], [3.0, 4.0, 
5.0], [6.0, 8.0, 6.0], [1.0, 1.0, 4.0]], [[7.0, 6.0, 2.0], [6.0, 6.0, 2.0], [4.0, 1.0, 6.0], [7.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 4.0, 8.0], [3.0, 3.0, 2.0], [2.0, 3.0, 2.0], [5.0, 8.0, 6.0]], [[3.0, 2.0, 3.0], 
[7.0, 1.0, 8.0], [5.0, 1.0, 3.0], [4.0, 6.0, 5.0]], [[4.0, 3.0, 2.0], [4.0, 4.0, 3.0], [7.0, 4.0, 7.0], [8.0, 3.0, 1.0]]]))
      val=Data(numpy.array([[1.0, 8.0]]),self.functionspace)
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 7.0, 1.0], [2.0, 1.0, 6.0], [4.0, 6.0, 4.0], [5.0, 4.0, 4.0]], [[5.0, 7.0, 1.0], [3.0, 
4.0, 5.0], [6.0, 8.0, 6.0], [1.0, 1.0, 4.0]], [[7.0, 6.0, 2.0], [6.0, 6.0, 2.0], [4.0, 1.0, 6.0], [7.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 8.0], [3.0, 3.0, 2.0], [2.0, 3.0, 2.0], [5.0, 8.0, 6.0]], [[3.0, 2.0, 3.0], [7.0, 
1.0, 8.0], [5.0, 1.0, 3.0], [4.0, 6.0, 5.0]], [[4.0, 3.0, 2.0], [4.0, 4.0, 3.0], [7.0, 1.0, 7.0], [8.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LLP(self):
      arg=Data(numpy.array([[[8.0, 8.0, 1.0], [3.0, 4.0, 7.0], [7.0, 2.0, 5.0], [8.0, 6.0, 5.0]], [[5.0, 6.0, 8.0], [4.0, 1.0, 
8.0], [6.0, 5.0, 8.0], [4.0, 2.0, 6.0]], [[5.0, 5.0, 3.0], [4.0, 8.0, 3.0], [5.0, 6.0, 6.0], [3.0, 7.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 6.0, 8.0], [2.0, 1.0, 6.0], [7.0, 2.0, 6.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 2.0], 
[1.0, 5.0, 2.0], [4.0, 8.0, 2.0], [8.0, 3.0, 8.0]], [[7.0, 2.0, 2.0], [4.0, 6.0, 4.0], [1.0, 3.0, 5.0], [5.0, 6.0, 2.0]]]))
      val=Data(numpy.array([[3.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 3.0]]))
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 8.0, 1.0], [3.0, 4.0, 7.0], [7.0, 2.0, 5.0], [8.0, 6.0, 5.0]], [[5.0, 6.0, 8.0], [4.0, 
1.0, 8.0], [6.0, 5.0, 8.0], [4.0, 2.0, 6.0]], [[5.0, 5.0, 3.0], [4.0, 8.0, 3.0], [5.0, 3.0, 6.0], [3.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 8.0], [2.0, 1.0, 6.0], [7.0, 2.0, 6.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 2.0], [1.0, 
5.0, 2.0], [4.0, 8.0, 2.0], [8.0, 3.0, 8.0]], [[7.0, 2.0, 2.0], [4.0, 6.0, 4.0], [1.0, 2.0, 5.0], [5.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LLP(self):
      arg=Data(numpy.array([[[7.0, 4.0, 2.0], [7.0, 8.0, 7.0], [8.0, 3.0, 2.0], [1.0, 5.0, 7.0]], [[6.0, 7.0, 2.0], [5.0, 7.0, 
3.0], [2.0, 1.0, 6.0], [4.0, 8.0, 5.0]], [[5.0, 8.0, 2.0], [6.0, 3.0, 4.0], [1.0, 5.0, 2.0], [4.0, 4.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 4.0, 8.0], [3.0, 4.0, 7.0], [4.0, 8.0, 1.0], [6.0, 3.0, 7.0]], [[2.0, 7.0, 1.0], 
[5.0, 4.0, 7.0], [1.0, 8.0, 6.0], [7.0, 7.0, 7.0]], [[2.0, 8.0, 7.0], [1.0, 4.0, 5.0], [5.0, 3.0, 5.0], [8.0, 7.0, 6.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[1.0, 2.0]])+(1.-msk_val)*numpy.array([[3.0, 6.0]])
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 2.0], [7.0, 8.0, 7.0], [8.0, 3.0, 2.0], [1.0, 5.0, 7.0]], [[6.0, 7.0, 2.0], [5.0, 
7.0, 3.0], [2.0, 1.0, 6.0], [4.0, 8.0, 5.0]], [[5.0, 8.0, 2.0], [6.0, 3.0, 4.0], [1.0, 1.0, 2.0], [4.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 8.0], [3.0, 4.0, 7.0], [4.0, 8.0, 1.0], [6.0, 3.0, 7.0]], [[2.0, 7.0, 1.0], [5.0, 
4.0, 7.0], [1.0, 8.0, 6.0], [7.0, 7.0, 7.0]], [[2.0, 8.0, 7.0], [1.0, 4.0, 5.0], [5.0, 3.0, 5.0], [8.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PUU(self):
      arg=Data(numpy.array([[[1.0, 8.0, 1.0], [8.0, 3.0, 8.0], [8.0, 6.0, 6.0], [7.0, 6.0, 2.0]], [[1.0, 1.0, 2.0], [6.0, 2.0, 
3.0], [8.0, 3.0, 4.0], [5.0, 6.0, 7.0]], [[5.0, 1.0, 1.0], [1.0, 4.0, 7.0], [1.0, 7.0, 7.0], [7.0, 1.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 6.0, 1.0], [5.0, 7.0, 6.0], [3.0, 7.0, 5.0], [3.0, 6.0, 5.0]], [[7.0, 7.0, 3.0], 
[3.0, 1.0, 7.0], [1.0, 3.0, 6.0], [5.0, 8.0, 2.0]], [[5.0, 6.0, 5.0], [7.0, 7.0, 8.0], [8.0, 6.0, 3.0], [3.0, 7.0, 8.0]]]))
      val=numpy.array([[7.0, 4.0], [6.0, 3.0], [2.0, 8.0]])
      arg[1,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 1.0], [8.0, 3.0, 8.0], [8.0, 6.0, 6.0], [7.0, 6.0, 2.0]], [[7.0, 4.0, 2.0], [6.0, 
3.0, 3.0], [2.0, 8.0, 4.0], [5.0, 6.0, 7.0]], [[5.0, 1.0, 1.0], [1.0, 4.0, 7.0], [1.0, 7.0, 7.0], [7.0, 1.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 6.0, 1.0], [5.0, 7.0, 6.0], [3.0, 7.0, 5.0], [3.0, 6.0, 5.0]], [[7.0, 4.0, 3.0], [6.0, 
3.0, 7.0], [2.0, 8.0, 6.0], [5.0, 8.0, 2.0]], [[5.0, 6.0, 5.0], [7.0, 7.0, 8.0], [8.0, 6.0, 3.0], [3.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PUU(self):
      arg=Data(numpy.array([[[1.0, 8.0, 2.0], [1.0, 6.0, 7.0], [1.0, 7.0, 2.0], [4.0, 1.0, 2.0]], [[6.0, 8.0, 8.0], [2.0, 1.0, 
4.0], [1.0, 4.0, 7.0], [5.0, 4.0, 6.0]], [[5.0, 2.0, 7.0], [4.0, 4.0, 5.0], [4.0, 8.0, 7.0], [5.0, 7.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 4.0, 8.0], [6.0, 1.0, 3.0], [4.0, 1.0, 4.0], [2.0, 4.0, 4.0]], [[7.0, 7.0, 4.0], 
[8.0, 8.0, 8.0], [1.0, 3.0, 5.0], [1.0, 8.0, 6.0]], [[1.0, 6.0, 2.0], [6.0, 2.0, 1.0], [8.0, 8.0, 5.0], [6.0, 2.0, 4.0]]]))
      val=Data(numpy.array([[8.0, 3.0], [2.0, 3.0], [1.0, 7.0]]),self.functionspace)
      arg[1,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 2.0], [1.0, 6.0, 7.0], [1.0, 7.0, 2.0], [4.0, 1.0, 2.0]], [[8.0, 3.0, 8.0], [2.0, 
3.0, 4.0], [1.0, 7.0, 7.0], [5.0, 4.0, 6.0]], [[5.0, 2.0, 7.0], [4.0, 4.0, 5.0], [4.0, 8.0, 7.0], [5.0, 7.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 8.0], [6.0, 1.0, 3.0], [4.0, 1.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 3.0, 4.0], [2.0, 
3.0, 8.0], [1.0, 7.0, 5.0], [1.0, 8.0, 6.0]], [[1.0, 6.0, 2.0], [6.0, 2.0, 1.0], [8.0, 8.0, 5.0], [6.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PUU(self):
      arg=Data(numpy.array([[[6.0, 3.0, 6.0], [3.0, 8.0, 2.0], [7.0, 5.0, 5.0], [6.0, 8.0, 4.0]], [[8.0, 8.0, 1.0], [4.0, 4.0, 
2.0], [7.0, 7.0, 7.0], [4.0, 4.0, 2.0]], [[3.0, 3.0, 4.0], [3.0, 8.0, 2.0], [7.0, 7.0, 6.0], [8.0, 3.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 5.0, 3.0], [4.0, 7.0, 7.0], [6.0, 1.0, 3.0], [8.0, 3.0, 8.0]], [[1.0, 7.0, 6.0], 
[3.0, 1.0, 7.0], [5.0, 7.0, 1.0], [1.0, 3.0, 1.0]], [[6.0, 2.0, 1.0], [1.0, 1.0, 1.0], [4.0, 6.0, 1.0], [6.0, 1.0, 6.0]]]))
      val=Data(numpy.array([[5.0, 3.0], [6.0, 5.0], [2.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 3.0], [1.0, 6.0], [7.0, 3.0]]))
      arg[1,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 6.0], [3.0, 8.0, 2.0], [7.0, 5.0, 5.0], [6.0, 8.0, 4.0]], [[5.0, 3.0, 1.0], [6.0, 
5.0, 2.0], [2.0, 6.0, 7.0], [4.0, 4.0, 2.0]], [[3.0, 3.0, 4.0], [3.0, 8.0, 2.0], [7.0, 7.0, 6.0], [8.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 3.0], [4.0, 7.0, 7.0], [6.0, 1.0, 3.0], [8.0, 3.0, 8.0]], [[3.0, 3.0, 6.0], [1.0, 
6.0, 7.0], [7.0, 3.0, 1.0], [1.0, 3.0, 1.0]], [[6.0, 2.0, 1.0], [1.0, 1.0, 1.0], [4.0, 6.0, 1.0], [6.0, 1.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PUU(self):
      arg=Data(numpy.array([[[1.0, 6.0, 8.0], [4.0, 6.0, 1.0], [1.0, 1.0, 3.0], [2.0, 7.0, 4.0]], [[8.0, 7.0, 1.0], [3.0, 1.0, 
1.0], [1.0, 3.0, 2.0], [8.0, 3.0, 2.0]], [[8.0, 6.0, 3.0], [5.0, 1.0, 4.0], [8.0, 6.0, 2.0], [4.0, 7.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 7.0, 5.0], [4.0, 5.0, 4.0], [4.0, 4.0, 7.0], [6.0, 6.0, 5.0]], [[5.0, 4.0, 3.0], 
[2.0, 5.0, 2.0], [1.0, 7.0, 1.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 8.0], [1.0, 8.0, 3.0], [3.0, 1.0, 4.0], [4.0, 8.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[3.0, 6.0], [3.0, 7.0], [7.0, 4.0]])+(1.-msk_val)*numpy.array([[4.0, 2.0], [3.0, 7.0], [2.0, 
8.0]])
      arg[1,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 8.0], [4.0, 6.0, 1.0], [1.0, 1.0, 3.0], [2.0, 7.0, 4.0]], [[3.0, 6.0, 1.0], [3.0, 
7.0, 1.0], [7.0, 4.0, 2.0], [8.0, 3.0, 2.0]], [[8.0, 6.0, 3.0], [5.0, 1.0, 4.0], [8.0, 6.0, 2.0], [4.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 7.0, 5.0], [4.0, 5.0, 4.0], [4.0, 4.0, 7.0], [6.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], [3.0, 
7.0, 2.0], [2.0, 8.0, 1.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 8.0], [1.0, 8.0, 3.0], [3.0, 1.0, 4.0], [4.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RPP(self):
      arg=Data(numpy.array([[[1.0, 2.0, 8.0], [4.0, 7.0, 6.0], [7.0, 8.0, 2.0], [3.0, 2.0, 3.0]], [[3.0, 7.0, 6.0], [7.0, 5.0, 
2.0], [8.0, 3.0, 6.0], [2.0, 5.0, 3.0]], [[8.0, 5.0, 7.0], [7.0, 8.0, 1.0], [7.0, 6.0, 3.0], [8.0, 7.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 8.0, 5.0], [8.0, 1.0, 1.0], [8.0, 3.0, 8.0], [2.0, 7.0, 6.0]], [[8.0, 2.0, 2.0], 
[1.0, 4.0, 7.0], [8.0, 5.0, 3.0], [3.0, 5.0, 3.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 7.0], [7.0, 4.0, 3.0], [5.0, 6.0, 6.0]]]))
      val=numpy.array([8.0])
      arg[1:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 8.0], [4.0, 7.0, 6.0], [7.0, 8.0, 2.0], [3.0, 2.0, 3.0]], [[3.0, 7.0, 6.0], [7.0, 
5.0, 2.0], [8.0, 8.0, 6.0], [2.0, 5.0, 3.0]], [[8.0, 5.0, 7.0], [7.0, 8.0, 1.0], [7.0, 6.0, 3.0], [8.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 5.0], [8.0, 1.0, 1.0], [8.0, 3.0, 8.0], [2.0, 7.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 
4.0, 7.0], [8.0, 8.0, 3.0], [3.0, 5.0, 3.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 7.0], [7.0, 4.0, 3.0], [5.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RPP(self):
      arg=Data(numpy.array([[[4.0, 2.0, 7.0], [6.0, 8.0, 1.0], [6.0, 4.0, 2.0], [6.0, 1.0, 7.0]], [[6.0, 1.0, 2.0], [5.0, 8.0, 
7.0], [6.0, 4.0, 2.0], [6.0, 3.0, 6.0]], [[4.0, 8.0, 6.0], [3.0, 4.0, 7.0], [4.0, 6.0, 6.0], [6.0, 2.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 7.0, 5.0], [3.0, 2.0, 8.0], [8.0, 1.0, 4.0], [1.0, 8.0, 8.0]], [[5.0, 5.0, 2.0], 
[4.0, 5.0, 7.0], [5.0, 1.0, 6.0], [3.0, 1.0, 8.0]], [[5.0, 3.0, 4.0], [5.0, 1.0, 8.0], [4.0, 4.0, 4.0], [7.0, 6.0, 8.0]]]))
      val=Data(numpy.array([4.0]),self.functionspace)
      arg[1:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 7.0], [6.0, 8.0, 1.0], [6.0, 4.0, 2.0], [6.0, 1.0, 7.0]], [[6.0, 1.0, 2.0], [5.0, 
8.0, 7.0], [6.0, 4.0, 2.0], [6.0, 3.0, 6.0]], [[4.0, 8.0, 6.0], [3.0, 4.0, 7.0], [4.0, 6.0, 6.0], [6.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 5.0], [3.0, 2.0, 8.0], [8.0, 1.0, 4.0], [1.0, 8.0, 8.0]], [[5.0, 5.0, 2.0], [4.0, 
5.0, 7.0], [5.0, 4.0, 6.0], [3.0, 1.0, 8.0]], [[5.0, 3.0, 4.0], [5.0, 1.0, 8.0], [4.0, 4.0, 4.0], [7.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RPP(self):
      arg=Data(numpy.array([[[1.0, 1.0, 2.0], [7.0, 4.0, 6.0], [3.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[8.0, 8.0, 7.0], [8.0, 3.0, 
2.0], [4.0, 1.0, 3.0], [1.0, 2.0, 6.0]], [[6.0, 3.0, 1.0], [5.0, 7.0, 8.0], [7.0, 3.0, 4.0], [2.0, 8.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 6.0, 3.0], [4.0, 8.0, 7.0], [2.0, 6.0, 1.0], [3.0, 3.0, 6.0]], [[7.0, 6.0, 4.0], 
[2.0, 8.0, 8.0], [8.0, 5.0, 4.0], [6.0, 4.0, 2.0]], [[1.0, 4.0, 2.0], [6.0, 4.0, 1.0], [6.0, 7.0, 3.0], [7.0, 2.0, 8.0]]]))
      val=Data(numpy.array([1.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0]))
      arg[1:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 2.0], [7.0, 4.0, 6.0], [3.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[8.0, 8.0, 7.0], [8.0, 
3.0, 2.0], [4.0, 1.0, 3.0], [1.0, 2.0, 6.0]], [[6.0, 3.0, 1.0], [5.0, 7.0, 8.0], [7.0, 3.0, 4.0], [2.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 6.0, 3.0], [4.0, 8.0, 7.0], [2.0, 6.0, 1.0], [3.0, 3.0, 6.0]], [[7.0, 6.0, 4.0], [2.0, 
8.0, 8.0], [8.0, 8.0, 4.0], [6.0, 4.0, 2.0]], [[1.0, 4.0, 2.0], [6.0, 4.0, 1.0], [6.0, 7.0, 3.0], [7.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RPP(self):
      arg=Data(numpy.array([[[5.0, 5.0, 4.0], [8.0, 1.0, 7.0], [5.0, 2.0, 2.0], [2.0, 2.0, 8.0]], [[3.0, 8.0, 4.0], [7.0, 8.0, 
4.0], [5.0, 2.0, 5.0], [4.0, 6.0, 3.0]], [[4.0, 4.0, 2.0], [6.0, 3.0, 5.0], [5.0, 4.0, 6.0], [7.0, 7.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 1.0, 8.0], [8.0, 4.0, 1.0], [1.0, 6.0, 5.0], [1.0, 5.0, 2.0]], [[2.0, 5.0, 5.0], 
[8.0, 8.0, 6.0], [1.0, 4.0, 4.0], [3.0, 7.0, 1.0]], [[6.0, 3.0, 6.0], [6.0, 1.0, 6.0], [3.0, 6.0, 4.0], [8.0, 2.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([1.0])+(1.-msk_val)*numpy.array([3.0])
      arg[1:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 5.0, 4.0], [8.0, 1.0, 7.0], [5.0, 2.0, 2.0], [2.0, 2.0, 8.0]], [[3.0, 8.0, 4.0], [7.0, 
8.0, 4.0], [5.0, 1.0, 5.0], [4.0, 6.0, 3.0]], [[4.0, 4.0, 2.0], [6.0, 3.0, 5.0], [5.0, 4.0, 6.0], [7.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 8.0], [8.0, 4.0, 1.0], [1.0, 6.0, 5.0], [1.0, 5.0, 2.0]], [[2.0, 5.0, 5.0], [8.0, 
8.0, 6.0], [1.0, 3.0, 4.0], [3.0, 7.0, 1.0]], [[6.0, 3.0, 6.0], [6.0, 1.0, 6.0], [3.0, 6.0, 4.0], [8.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RUP(self):
      arg=Data(numpy.array([[[3.0, 7.0, 1.0], [1.0, 6.0, 5.0], [5.0, 8.0, 1.0], [1.0, 7.0, 1.0]], [[5.0, 8.0, 3.0], [6.0, 2.0, 
7.0], [3.0, 1.0, 5.0], [2.0, 5.0, 3.0]], [[6.0, 4.0, 5.0], [1.0, 8.0, 6.0], [3.0, 8.0, 1.0], [7.0, 4.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 8.0, 1.0], [5.0, 2.0, 6.0], [3.0, 3.0, 6.0], [1.0, 3.0, 2.0]], [[5.0, 2.0, 1.0], 
[2.0, 1.0, 7.0], [5.0, 6.0, 6.0], [1.0, 1.0, 8.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 7.0], [2.0, 7.0, 5.0], [7.0, 4.0, 6.0]]]))
      val=numpy.array([[1.0, 5.0, 2.0]])
      arg[1:2,:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 1.0], [1.0, 6.0, 5.0], [5.0, 8.0, 1.0], [1.0, 7.0, 1.0]], [[5.0, 1.0, 3.0], [6.0, 
5.0, 7.0], [3.0, 2.0, 5.0], [2.0, 5.0, 3.0]], [[6.0, 4.0, 5.0], [1.0, 8.0, 6.0], [3.0, 8.0, 1.0], [7.0, 4.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 1.0], [5.0, 2.0, 6.0], [3.0, 3.0, 6.0], [1.0, 3.0, 2.0]], [[5.0, 1.0, 1.0], [2.0, 
5.0, 7.0], [5.0, 2.0, 6.0], [1.0, 1.0, 8.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 7.0], [2.0, 7.0, 5.0], [7.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RUP(self):
      arg=Data(numpy.array([[[4.0, 7.0, 8.0], [3.0, 7.0, 2.0], [3.0, 8.0, 8.0], [2.0, 1.0, 4.0]], [[3.0, 5.0, 7.0], [4.0, 3.0, 
4.0], [7.0, 7.0, 8.0], [3.0, 7.0, 8.0]], [[1.0, 1.0, 6.0], [6.0, 5.0, 3.0], [3.0, 7.0, 6.0], [4.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 2.0, 5.0], [4.0, 1.0, 4.0], [1.0, 4.0, 6.0], [5.0, 7.0, 6.0]], [[4.0, 3.0, 7.0], 
[5.0, 2.0, 5.0], [1.0, 8.0, 1.0], [4.0, 6.0, 3.0]], [[1.0, 2.0, 8.0], [3.0, 2.0, 1.0], [5.0, 6.0, 8.0], [6.0, 4.0, 2.0]]]))
      val=Data(numpy.array([[8.0, 6.0, 8.0]]),self.functionspace)
      arg[1:2,:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 7.0, 8.0], [3.0, 7.0, 2.0], [3.0, 8.0, 8.0], [2.0, 1.0, 4.0]], [[3.0, 8.0, 7.0], [4.0, 
6.0, 4.0], [7.0, 8.0, 8.0], [3.0, 7.0, 8.0]], [[1.0, 1.0, 6.0], [6.0, 5.0, 3.0], [3.0, 7.0, 6.0], [4.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 5.0], [4.0, 1.0, 4.0], [1.0, 4.0, 6.0], [5.0, 7.0, 6.0]], [[4.0, 8.0, 7.0], [5.0, 
6.0, 5.0], [1.0, 8.0, 1.0], [4.0, 6.0, 3.0]], [[1.0, 2.0, 8.0], [3.0, 2.0, 1.0], [5.0, 6.0, 8.0], [6.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RUP(self):
      arg=Data(numpy.array([[[7.0, 8.0, 3.0], [2.0, 4.0, 5.0], [5.0, 2.0, 8.0], [3.0, 4.0, 5.0]], [[1.0, 3.0, 6.0], [2.0, 2.0, 
6.0], [2.0, 4.0, 6.0], [2.0, 3.0, 6.0]], [[6.0, 6.0, 3.0], [5.0, 4.0, 4.0], [5.0, 3.0, 4.0], [4.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 4.0, 6.0], [8.0, 4.0, 1.0], [7.0, 5.0, 5.0], [8.0, 3.0, 8.0]], [[8.0, 2.0, 8.0], 
[7.0, 3.0, 3.0], [4.0, 5.0, 7.0], [5.0, 5.0, 4.0]], [[5.0, 6.0, 6.0], [2.0, 4.0, 6.0], [1.0, 5.0, 5.0], [8.0, 1.0, 4.0]]]))
      val=Data(numpy.array([[6.0, 5.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 6.0, 6.0]]))
      arg[1:2,:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 3.0], [2.0, 4.0, 5.0], [5.0, 2.0, 8.0], [3.0, 4.0, 5.0]], [[1.0, 6.0, 6.0], [2.0, 
5.0, 6.0], [2.0, 2.0, 6.0], [2.0, 3.0, 6.0]], [[6.0, 6.0, 3.0], [5.0, 4.0, 4.0], [5.0, 3.0, 4.0], [4.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 6.0], [8.0, 4.0, 1.0], [7.0, 5.0, 5.0], [8.0, 3.0, 8.0]], [[8.0, 2.0, 8.0], [7.0, 
6.0, 3.0], [4.0, 6.0, 7.0], [5.0, 5.0, 4.0]], [[5.0, 6.0, 6.0], [2.0, 4.0, 6.0], [1.0, 5.0, 5.0], [8.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RUP(self):
      arg=Data(numpy.array([[[8.0, 5.0, 5.0], [5.0, 4.0, 4.0], [1.0, 8.0, 5.0], [4.0, 1.0, 6.0]], [[3.0, 8.0, 6.0], [1.0, 6.0, 
5.0], [4.0, 7.0, 7.0], [5.0, 5.0, 7.0]], [[1.0, 1.0, 8.0], [2.0, 1.0, 3.0], [6.0, 2.0, 2.0], [1.0, 4.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 8.0, 8.0], [5.0, 6.0, 6.0], [1.0, 5.0, 4.0], [2.0, 6.0, 2.0]], [[5.0, 7.0, 4.0], 
[2.0, 3.0, 6.0], [4.0, 8.0, 3.0], [2.0, 8.0, 3.0]], [[7.0, 1.0, 2.0], [3.0, 2.0, 3.0], [3.0, 7.0, 1.0], [4.0, 6.0, 6.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 3.0, 5.0]])+(1.-msk_val)*numpy.array([[1.0, 2.0, 1.0]])
      arg[1:2,:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 5.0], [5.0, 4.0, 4.0], [1.0, 8.0, 5.0], [4.0, 1.0, 6.0]], [[3.0, 6.0, 6.0], [1.0, 
3.0, 5.0], [4.0, 5.0, 7.0], [5.0, 5.0, 7.0]], [[1.0, 1.0, 8.0], [2.0, 1.0, 3.0], [6.0, 2.0, 2.0], [1.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 8.0], [5.0, 6.0, 6.0], [1.0, 5.0, 4.0], [2.0, 6.0, 2.0]], [[5.0, 1.0, 4.0], [2.0, 
2.0, 6.0], [4.0, 1.0, 3.0], [2.0, 8.0, 3.0]], [[7.0, 1.0, 2.0], [3.0, 2.0, 3.0], [3.0, 7.0, 1.0], [4.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UPP(self):
      arg=Data(numpy.array([[[2.0, 2.0, 2.0], [2.0, 3.0, 4.0], [5.0, 8.0, 3.0], [3.0, 3.0, 3.0]], [[2.0, 1.0, 8.0], [5.0, 5.0, 
8.0], [1.0, 3.0, 2.0], [4.0, 7.0, 5.0]], [[6.0, 8.0, 5.0], [2.0, 2.0, 6.0], [5.0, 8.0, 1.0], [2.0, 1.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 8.0, 1.0], [1.0, 7.0, 4.0], [8.0, 4.0, 4.0], [6.0, 3.0, 5.0]], [[5.0, 5.0, 6.0], 
[8.0, 3.0, 8.0], [1.0, 3.0, 6.0], [4.0, 7.0, 2.0]], [[4.0, 1.0, 8.0], [2.0, 2.0, 3.0], [6.0, 4.0, 2.0], [1.0, 4.0, 7.0]]]))
      val=numpy.array([5.0, 2.0])
      arg[:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 2.0], [2.0, 3.0, 4.0], [5.0, 5.0, 3.0], [3.0, 3.0, 3.0]], [[2.0, 1.0, 8.0], [5.0, 
5.0, 8.0], [1.0, 2.0, 2.0], [4.0, 7.0, 5.0]], [[6.0, 8.0, 5.0], [2.0, 2.0, 6.0], [5.0, 8.0, 1.0], [2.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 1.0], [1.0, 7.0, 4.0], [8.0, 5.0, 4.0], [6.0, 3.0, 5.0]], [[5.0, 5.0, 6.0], [8.0, 
3.0, 8.0], [1.0, 2.0, 6.0], [4.0, 7.0, 2.0]], [[4.0, 1.0, 8.0], [2.0, 2.0, 3.0], [6.0, 4.0, 2.0], [1.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UPP(self):
      arg=Data(numpy.array([[[2.0, 7.0, 2.0], [8.0, 4.0, 8.0], [7.0, 2.0, 3.0], [8.0, 4.0, 4.0]], [[2.0, 4.0, 7.0], [1.0, 8.0, 
5.0], [5.0, 1.0, 5.0], [6.0, 4.0, 4.0]], [[5.0, 4.0, 2.0], [8.0, 6.0, 4.0], [3.0, 1.0, 6.0], [6.0, 4.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 2.0, 5.0], [5.0, 1.0, 5.0], [7.0, 4.0, 4.0], [7.0, 3.0, 3.0]], [[2.0, 6.0, 4.0], 
[7.0, 1.0, 2.0], [6.0, 4.0, 6.0], [7.0, 6.0, 6.0]], [[2.0, 4.0, 8.0], [7.0, 8.0, 4.0], [6.0, 1.0, 4.0], [5.0, 1.0, 1.0]]]))
      val=Data(numpy.array([3.0, 3.0]),self.functionspace)
      arg[:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 2.0], [8.0, 4.0, 8.0], [7.0, 3.0, 3.0], [8.0, 4.0, 4.0]], [[2.0, 4.0, 7.0], [1.0, 
8.0, 5.0], [5.0, 3.0, 5.0], [6.0, 4.0, 4.0]], [[5.0, 4.0, 2.0], [8.0, 6.0, 4.0], [3.0, 1.0, 6.0], [6.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 5.0], [5.0, 1.0, 5.0], [7.0, 3.0, 4.0], [7.0, 3.0, 3.0]], [[2.0, 6.0, 4.0], [7.0, 
1.0, 2.0], [6.0, 3.0, 6.0], [7.0, 6.0, 6.0]], [[2.0, 4.0, 8.0], [7.0, 8.0, 4.0], [6.0, 1.0, 4.0], [5.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UPP(self):
      arg=Data(numpy.array([[[7.0, 8.0, 2.0], [2.0, 3.0, 1.0], [3.0, 6.0, 6.0], [1.0, 7.0, 4.0]], [[4.0, 7.0, 6.0], [8.0, 4.0, 
4.0], [1.0, 3.0, 7.0], [7.0, 7.0, 3.0]], [[4.0, 8.0, 1.0], [1.0, 7.0, 5.0], [6.0, 3.0, 5.0], [4.0, 1.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 8.0, 3.0], [3.0, 2.0, 2.0], [6.0, 4.0, 4.0], [8.0, 4.0, 1.0]], [[1.0, 2.0, 4.0], 
[8.0, 5.0, 6.0], [5.0, 5.0, 4.0], [2.0, 5.0, 3.0]], [[1.0, 7.0, 3.0], [1.0, 2.0, 6.0], [6.0, 3.0, 8.0], [8.0, 7.0, 1.0]]]))
      val=Data(numpy.array([5.0, 5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([7.0, 5.0]))
      arg[:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 2.0], [2.0, 3.0, 1.0], [3.0, 5.0, 6.0], [1.0, 7.0, 4.0]], [[4.0, 7.0, 6.0], [8.0, 
4.0, 4.0], [1.0, 5.0, 7.0], [7.0, 7.0, 3.0]], [[4.0, 8.0, 1.0], [1.0, 7.0, 5.0], [6.0, 3.0, 5.0], [4.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 3.0], [3.0, 2.0, 2.0], [6.0, 7.0, 4.0], [8.0, 4.0, 1.0]], [[1.0, 2.0, 4.0], [8.0, 
5.0, 6.0], [5.0, 5.0, 4.0], [2.0, 5.0, 3.0]], [[1.0, 7.0, 3.0], [1.0, 2.0, 6.0], [6.0, 3.0, 8.0], [8.0, 7.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UPP(self):
      arg=Data(numpy.array([[[1.0, 8.0, 3.0], [6.0, 6.0, 8.0], [3.0, 2.0, 6.0], [3.0, 6.0, 6.0]], [[6.0, 8.0, 6.0], [5.0, 5.0, 
2.0], [2.0, 4.0, 5.0], [4.0, 8.0, 7.0]], [[3.0, 2.0, 7.0], [4.0, 3.0, 4.0], [8.0, 2.0, 6.0], [4.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 5.0, 5.0], [5.0, 6.0, 2.0], [7.0, 8.0, 3.0], [8.0, 8.0, 8.0]], [[4.0, 3.0, 1.0], 
[2.0, 7.0, 2.0], [8.0, 8.0, 8.0], [2.0, 8.0, 5.0]], [[7.0, 2.0, 4.0], [3.0, 4.0, 8.0], [4.0, 2.0, 4.0], [1.0, 2.0, 1.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([8.0, 4.0])+(1.-msk_val)*numpy.array([1.0, 4.0])
      arg[:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 3.0], [6.0, 6.0, 8.0], [3.0, 8.0, 6.0], [3.0, 6.0, 6.0]], [[6.0, 8.0, 6.0], [5.0, 
5.0, 2.0], [2.0, 4.0, 5.0], [4.0, 8.0, 7.0]], [[3.0, 2.0, 7.0], [4.0, 3.0, 4.0], [8.0, 2.0, 6.0], [4.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 5.0], [5.0, 6.0, 2.0], [7.0, 1.0, 3.0], [8.0, 8.0, 8.0]], [[4.0, 3.0, 1.0], [2.0, 
7.0, 2.0], [8.0, 4.0, 8.0], [2.0, 8.0, 5.0]], [[7.0, 2.0, 4.0], [3.0, 4.0, 8.0], [4.0, 2.0, 4.0], [1.0, 2.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LRC(self):
      arg=Data(numpy.array([[[6.0, 2.0, 7.0], [5.0, 3.0, 3.0], [3.0, 1.0, 4.0], [8.0, 5.0, 5.0]], [[7.0, 1.0, 3.0], [2.0, 8.0, 
1.0], [1.0, 5.0, 7.0], [1.0, 2.0, 7.0]], [[4.0, 1.0, 2.0], [2.0, 6.0, 7.0], [5.0, 3.0, 2.0], [3.0, 3.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 8.0], [6.0, 2.0, 5.0], [8.0, 1.0, 2.0], [2.0, 8.0, 6.0]], [[6.0, 7.0, 5.0], 
[4.0, 7.0, 4.0], [7.0, 1.0, 8.0], [6.0, 8.0, 3.0]], [[3.0, 3.0, 6.0], [3.0, 8.0, 2.0], [5.0, 4.0, 3.0], [5.0, 1.0, 3.0]]]))
      val=numpy.array([[[1.0, 7.0, 7.0], [1.0, 5.0, 7.0]]])
      arg[2:,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 7.0], [5.0, 3.0, 3.0], [3.0, 1.0, 4.0], [8.0, 5.0, 5.0]], [[7.0, 1.0, 3.0], [2.0, 
8.0, 1.0], [1.0, 5.0, 7.0], [1.0, 2.0, 7.0]], [[4.0, 1.0, 2.0], [1.0, 7.0, 7.0], [1.0, 5.0, 7.0], [3.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 8.0], [6.0, 2.0, 5.0], [8.0, 1.0, 2.0], [2.0, 8.0, 6.0]], [[6.0, 7.0, 5.0], [4.0, 
7.0, 4.0], [7.0, 1.0, 8.0], [6.0, 8.0, 3.0]], [[3.0, 3.0, 6.0], [1.0, 7.0, 7.0], [1.0, 5.0, 7.0], [5.0, 1.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LRC(self):
      arg=Data(numpy.array([[[5.0, 7.0, 8.0], [5.0, 6.0, 6.0], [1.0, 7.0, 3.0], [5.0, 2.0, 7.0]], [[4.0, 2.0, 4.0], [5.0, 7.0, 
1.0], [5.0, 4.0, 3.0], [7.0, 4.0, 5.0]], [[5.0, 2.0, 6.0], [3.0, 1.0, 8.0], [5.0, 5.0, 7.0], [5.0, 4.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 8.0], [7.0, 5.0, 6.0], [5.0, 4.0, 1.0], [5.0, 4.0, 4.0]], [[4.0, 1.0, 2.0], 
[1.0, 4.0, 8.0], [2.0, 6.0, 5.0], [5.0, 6.0, 3.0]], [[8.0, 4.0, 7.0], [8.0, 8.0, 3.0], [5.0, 8.0, 8.0], [6.0, 6.0, 8.0]]]))
      val=Data(numpy.array([[[3.0, 3.0, 2.0], [2.0, 4.0, 5.0]]]),self.functionspace)
      arg[2:,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 8.0], [5.0, 6.0, 6.0], [1.0, 7.0, 3.0], [5.0, 2.0, 7.0]], [[4.0, 2.0, 4.0], [5.0, 
7.0, 1.0], [5.0, 4.0, 3.0], [7.0, 4.0, 5.0]], [[5.0, 2.0, 6.0], [3.0, 3.0, 2.0], [2.0, 4.0, 5.0], [5.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 8.0], [7.0, 5.0, 6.0], [5.0, 4.0, 1.0], [5.0, 4.0, 4.0]], [[4.0, 1.0, 2.0], [1.0, 
4.0, 8.0], [2.0, 6.0, 5.0], [5.0, 6.0, 3.0]], [[8.0, 4.0, 7.0], [3.0, 3.0, 2.0], [2.0, 4.0, 5.0], [6.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LRC(self):
      arg=Data(numpy.array([[[3.0, 8.0, 6.0], [2.0, 3.0, 4.0], [1.0, 5.0, 7.0], [8.0, 1.0, 3.0]], [[8.0, 3.0, 4.0], [4.0, 8.0, 
2.0], [7.0, 1.0, 8.0], [8.0, 5.0, 3.0]], [[4.0, 5.0, 8.0], [8.0, 2.0, 8.0], [6.0, 4.0, 5.0], [4.0, 5.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 6.0, 4.0], [5.0, 6.0, 5.0], [7.0, 2.0, 5.0], [5.0, 2.0, 8.0]], [[1.0, 6.0, 4.0], 
[4.0, 3.0, 4.0], [7.0, 2.0, 7.0], [6.0, 7.0, 3.0]], [[5.0, 2.0, 1.0], [2.0, 4.0, 4.0], [7.0, 1.0, 4.0], [4.0, 4.0, 5.0]]]))
      val=Data(numpy.array([[[1.0, 7.0, 5.0], [4.0, 4.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 6.0, 8.0], [7.0, 2.0, 1.0]]]))
      arg[2:,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 6.0], [2.0, 3.0, 4.0], [1.0, 5.0, 7.0], [8.0, 1.0, 3.0]], [[8.0, 3.0, 4.0], [4.0, 
8.0, 2.0], [7.0, 1.0, 8.0], [8.0, 5.0, 3.0]], [[4.0, 5.0, 8.0], [1.0, 7.0, 5.0], [4.0, 4.0, 5.0], [4.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 6.0, 4.0], [5.0, 6.0, 5.0], [7.0, 2.0, 5.0], [5.0, 2.0, 8.0]], [[1.0, 6.0, 4.0], [4.0, 
3.0, 4.0], [7.0, 2.0, 7.0], [6.0, 7.0, 3.0]], [[5.0, 2.0, 1.0], [8.0, 6.0, 8.0], [7.0, 2.0, 1.0], [4.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LRC(self):
      arg=Data(numpy.array([[[2.0, 2.0, 8.0], [6.0, 7.0, 3.0], [7.0, 6.0, 5.0], [2.0, 8.0, 2.0]], [[8.0, 1.0, 2.0], [3.0, 2.0, 
1.0], [7.0, 3.0, 1.0], [5.0, 6.0, 2.0]], [[6.0, 4.0, 4.0], [8.0, 4.0, 4.0], [5.0, 8.0, 2.0], [3.0, 2.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 2.0, 7.0], [8.0, 7.0, 1.0], [2.0, 4.0, 4.0], [1.0, 5.0, 1.0]], [[6.0, 4.0, 2.0], 
[7.0, 7.0, 6.0], [6.0, 6.0, 1.0], [4.0, 1.0, 7.0]], [[1.0, 5.0, 4.0], [6.0, 2.0, 8.0], [4.0, 6.0, 7.0], [7.0, 5.0, 1.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 6.0, 6.0], [6.0, 2.0, 7.0]]])+(1.-msk_val)*numpy.array([[[3.0, 6.0, 8.0], [6.0, 3.0, 
1.0]]])
      arg[2:,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 8.0], [6.0, 7.0, 3.0], [7.0, 6.0, 5.0], [2.0, 8.0, 2.0]], [[8.0, 1.0, 2.0], [3.0, 
2.0, 1.0], [7.0, 3.0, 1.0], [5.0, 6.0, 2.0]], [[6.0, 4.0, 4.0], [3.0, 6.0, 6.0], [6.0, 2.0, 7.0], [3.0, 2.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 7.0], [8.0, 7.0, 1.0], [2.0, 4.0, 4.0], [1.0, 5.0, 1.0]], [[6.0, 4.0, 2.0], [7.0, 
7.0, 6.0], [6.0, 6.0, 1.0], [4.0, 1.0, 7.0]], [[1.0, 5.0, 4.0], [3.0, 6.0, 8.0], [6.0, 3.0, 1.0], [7.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UPL(self):
      arg=Data(numpy.array([[[6.0, 6.0, 3.0], [2.0, 1.0, 7.0], [6.0, 5.0, 5.0], [7.0, 2.0, 1.0]], [[3.0, 2.0, 7.0], [3.0, 1.0, 
3.0], [2.0, 1.0, 4.0], [4.0, 4.0, 7.0]], [[6.0, 8.0, 1.0], [5.0, 7.0, 3.0], [2.0, 5.0, 3.0], [8.0, 8.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 5.0, 3.0], [2.0, 5.0, 7.0], [1.0, 6.0, 1.0], [4.0, 6.0, 1.0]], [[1.0, 3.0, 5.0], 
[8.0, 3.0, 4.0], [4.0, 5.0, 1.0], [8.0, 1.0, 3.0]], [[5.0, 3.0, 8.0], [6.0, 7.0, 8.0], [4.0, 4.0, 8.0], [2.0, 2.0, 7.0]]]))
      val=numpy.array([[5.0], [4.0]])
      arg[:2,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 3.0], [2.0, 1.0, 7.0], [6.0, 5.0, 5.0], [7.0, 2.0, 1.0]], [[3.0, 2.0, 7.0], [3.0, 
1.0, 3.0], [2.0, 1.0, 4.0], [4.0, 4.0, 7.0]], [[6.0, 8.0, 1.0], [5.0, 7.0, 3.0], [2.0, 5.0, 3.0], [8.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 3.0], [2.0, 5.0, 7.0], [1.0, 6.0, 5.0], [4.0, 6.0, 1.0]], [[1.0, 3.0, 5.0], [8.0, 
3.0, 4.0], [4.0, 5.0, 4.0], [8.0, 1.0, 3.0]], [[5.0, 3.0, 8.0], [6.0, 7.0, 8.0], [4.0, 4.0, 8.0], [2.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UPL(self):
      arg=Data(numpy.array([[[7.0, 6.0, 7.0], [7.0, 4.0, 6.0], [7.0, 4.0, 4.0], [7.0, 5.0, 5.0]], [[5.0, 1.0, 3.0], [8.0, 8.0, 
6.0], [1.0, 3.0, 3.0], [4.0, 5.0, 3.0]], [[5.0, 2.0, 4.0], [1.0, 3.0, 7.0], [6.0, 4.0, 2.0], [5.0, 8.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 8.0, 7.0], [4.0, 1.0, 7.0], [4.0, 6.0, 5.0], [2.0, 8.0, 5.0]], [[2.0, 1.0, 3.0], 
[8.0, 7.0, 6.0], [7.0, 3.0, 6.0], [8.0, 7.0, 6.0]], [[2.0, 3.0, 2.0], [1.0, 5.0, 5.0], [3.0, 7.0, 2.0], [4.0, 3.0, 7.0]]]))
      val=Data(numpy.array([[2.0], [2.0]]),self.functionspace)
      arg[:2,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 7.0], [7.0, 4.0, 6.0], [7.0, 4.0, 2.0], [7.0, 5.0, 5.0]], [[5.0, 1.0, 3.0], [8.0, 
8.0, 6.0], [1.0, 3.0, 2.0], [4.0, 5.0, 3.0]], [[5.0, 2.0, 4.0], [1.0, 3.0, 7.0], [6.0, 4.0, 2.0], [5.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 7.0], [4.0, 1.0, 7.0], [4.0, 6.0, 2.0], [2.0, 8.0, 5.0]], [[2.0, 1.0, 3.0], [8.0, 
7.0, 6.0], [7.0, 3.0, 2.0], [8.0, 7.0, 6.0]], [[2.0, 3.0, 2.0], [1.0, 5.0, 5.0], [3.0, 7.0, 2.0], [4.0, 3.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UPL(self):
      arg=Data(numpy.array([[[4.0, 6.0, 4.0], [7.0, 1.0, 8.0], [2.0, 6.0, 6.0], [1.0, 7.0, 7.0]], [[2.0, 8.0, 1.0], [6.0, 8.0, 
7.0], [3.0, 2.0, 8.0], [5.0, 5.0, 6.0]], [[6.0, 6.0, 6.0], [6.0, 1.0, 6.0], [6.0, 2.0, 6.0], [5.0, 6.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 8.0, 2.0], [2.0, 8.0, 2.0], [7.0, 1.0, 3.0], [4.0, 8.0, 5.0]], [[8.0, 2.0, 1.0], 
[4.0, 5.0, 2.0], [2.0, 2.0, 6.0], [6.0, 2.0, 6.0]], [[1.0, 7.0, 3.0], [3.0, 8.0, 7.0], [1.0, 8.0, 3.0], [1.0, 3.0, 4.0]]]))
      val=Data(numpy.array([[5.0], [7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0], [7.0]]))
      arg[:2,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 4.0], [7.0, 1.0, 8.0], [2.0, 6.0, 5.0], [1.0, 7.0, 7.0]], [[2.0, 8.0, 1.0], [6.0, 
8.0, 7.0], [3.0, 2.0, 7.0], [5.0, 5.0, 6.0]], [[6.0, 6.0, 6.0], [6.0, 1.0, 6.0], [6.0, 2.0, 6.0], [5.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 2.0], [2.0, 8.0, 2.0], [7.0, 1.0, 7.0], [4.0, 8.0, 5.0]], [[8.0, 2.0, 1.0], [4.0, 
5.0, 2.0], [2.0, 2.0, 7.0], [6.0, 2.0, 6.0]], [[1.0, 7.0, 3.0], [3.0, 8.0, 7.0], [1.0, 8.0, 3.0], [1.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UPL(self):
      arg=Data(numpy.array([[[3.0, 4.0, 1.0], [7.0, 7.0, 3.0], [1.0, 1.0, 4.0], [8.0, 3.0, 7.0]], [[8.0, 7.0, 1.0], [3.0, 4.0, 
4.0], [4.0, 2.0, 7.0], [4.0, 4.0, 2.0]], [[3.0, 3.0, 1.0], [4.0, 7.0, 2.0], [5.0, 1.0, 7.0], [4.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 5.0, 7.0], [5.0, 1.0, 8.0], [6.0, 3.0, 6.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 2.0], 
[6.0, 1.0, 5.0], [4.0, 3.0, 3.0], [2.0, 7.0, 7.0]], [[2.0, 5.0, 8.0], [5.0, 7.0, 2.0], [1.0, 3.0, 4.0], [7.0, 5.0, 6.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[1.0], [7.0]])+(1.-msk_val)*numpy.array([[6.0], [4.0]])
      arg[:2,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 1.0], [7.0, 7.0, 3.0], [1.0, 1.0, 1.0], [8.0, 3.0, 7.0]], [[8.0, 7.0, 1.0], [3.0, 
4.0, 4.0], [4.0, 2.0, 7.0], [4.0, 4.0, 2.0]], [[3.0, 3.0, 1.0], [4.0, 7.0, 2.0], [5.0, 1.0, 7.0], [4.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 7.0], [5.0, 1.0, 8.0], [6.0, 3.0, 6.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 2.0], [6.0, 
1.0, 5.0], [4.0, 3.0, 4.0], [2.0, 7.0, 7.0]], [[2.0, 5.0, 8.0], [5.0, 7.0, 2.0], [1.0, 3.0, 4.0], [7.0, 5.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LLN(self):
      arg=Data(numpy.array([[[4.0, 8.0, 7.0], [3.0, 4.0, 8.0], [2.0, 1.0, 8.0], [3.0, 3.0, 5.0]], [[3.0, 1.0, 5.0], [6.0, 5.0, 
8.0], [7.0, 6.0, 4.0], [4.0, 7.0, 3.0]], [[4.0, 5.0, 8.0], [6.0, 1.0, 6.0], [8.0, 5.0, 8.0], [1.0, 3.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 8.0, 1.0], [4.0, 8.0, 6.0], [1.0, 3.0, 3.0], [8.0, 3.0, 4.0]], [[1.0, 1.0, 8.0], 
[5.0, 2.0, 3.0], [2.0, 6.0, 6.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 4.0], [3.0, 4.0, 7.0], [7.0, 6.0, 5.0], [1.0, 3.0, 6.0]]]))
      val=numpy.array([[[7.0, 4.0, 3.0], [4.0, 6.0, 2.0]]])
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 7.0], [3.0, 4.0, 8.0], [2.0, 1.0, 8.0], [3.0, 3.0, 5.0]], [[3.0, 1.0, 5.0], [6.0, 
5.0, 8.0], [7.0, 6.0, 4.0], [4.0, 7.0, 3.0]], [[4.0, 5.0, 8.0], [6.0, 1.0, 6.0], [7.0, 4.0, 3.0], [4.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 1.0], [4.0, 8.0, 6.0], [1.0, 3.0, 3.0], [8.0, 3.0, 4.0]], [[1.0, 1.0, 8.0], [5.0, 
2.0, 3.0], [2.0, 6.0, 6.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 4.0], [3.0, 4.0, 7.0], [7.0, 4.0, 3.0], [4.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LLN(self):
      arg=Data(numpy.array([[[4.0, 6.0, 5.0], [2.0, 6.0, 6.0], [2.0, 3.0, 7.0], [2.0, 4.0, 5.0]], [[6.0, 5.0, 6.0], [4.0, 1.0, 
5.0], [1.0, 1.0, 1.0], [5.0, 7.0, 3.0]], [[3.0, 1.0, 4.0], [4.0, 4.0, 3.0], [5.0, 2.0, 1.0], [3.0, 1.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 3.0, 2.0], [5.0, 8.0, 7.0], [2.0, 5.0, 8.0], [5.0, 8.0, 4.0]], [[7.0, 7.0, 6.0], 
[2.0, 7.0, 1.0], [2.0, 8.0, 3.0], [1.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [2.0, 8.0, 3.0], [1.0, 8.0, 8.0], [3.0, 6.0, 1.0]]]))
      val=Data(numpy.array([[[3.0, 3.0, 8.0], [8.0, 5.0, 4.0]]]),self.functionspace)
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 5.0], [2.0, 6.0, 6.0], [2.0, 3.0, 7.0], [2.0, 4.0, 5.0]], [[6.0, 5.0, 6.0], [4.0, 
1.0, 5.0], [1.0, 1.0, 1.0], [5.0, 7.0, 3.0]], [[3.0, 1.0, 4.0], [4.0, 4.0, 3.0], [3.0, 3.0, 8.0], [8.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 2.0], [5.0, 8.0, 7.0], [2.0, 5.0, 8.0], [5.0, 8.0, 4.0]], [[7.0, 7.0, 6.0], [2.0, 
7.0, 1.0], [2.0, 8.0, 3.0], [1.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [2.0, 8.0, 3.0], [3.0, 3.0, 8.0], [8.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LLN(self):
      arg=Data(numpy.array([[[3.0, 7.0, 7.0], [2.0, 1.0, 2.0], [7.0, 7.0, 2.0], [7.0, 6.0, 7.0]], [[7.0, 2.0, 5.0], [1.0, 3.0, 
5.0], [4.0, 8.0, 6.0], [3.0, 8.0, 6.0]], [[7.0, 2.0, 2.0], [3.0, 6.0, 2.0], [2.0, 5.0, 1.0], [5.0, 1.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 6.0, 8.0], [5.0, 6.0, 1.0], [2.0, 1.0, 8.0], [7.0, 3.0, 3.0]], [[3.0, 1.0, 8.0], 
[2.0, 7.0, 3.0], [2.0, 4.0, 1.0], [7.0, 1.0, 6.0]], [[3.0, 5.0, 1.0], [6.0, 7.0, 8.0], [7.0, 8.0, 6.0], [6.0, 4.0, 8.0]]]))
      val=Data(numpy.array([[[4.0, 4.0, 2.0], [5.0, 2.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 1.0, 8.0], [1.0, 7.0, 6.0]]]))
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 7.0], [2.0, 1.0, 2.0], [7.0, 7.0, 2.0], [7.0, 6.0, 7.0]], [[7.0, 2.0, 5.0], [1.0, 
3.0, 5.0], [4.0, 8.0, 6.0], [3.0, 8.0, 6.0]], [[7.0, 2.0, 2.0], [3.0, 6.0, 2.0], [4.0, 4.0, 2.0], [5.0, 2.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 8.0], [5.0, 6.0, 1.0], [2.0, 1.0, 8.0], [7.0, 3.0, 3.0]], [[3.0, 1.0, 8.0], [2.0, 
7.0, 3.0], [2.0, 4.0, 1.0], [7.0, 1.0, 6.0]], [[3.0, 5.0, 1.0], [6.0, 7.0, 8.0], [8.0, 1.0, 8.0], [1.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LLN(self):
      arg=Data(numpy.array([[[5.0, 4.0, 7.0], [8.0, 5.0, 7.0], [5.0, 2.0, 5.0], [5.0, 6.0, 7.0]], [[4.0, 1.0, 7.0], [5.0, 7.0, 
8.0], [4.0, 2.0, 8.0], [3.0, 1.0, 4.0]], [[5.0, 5.0, 2.0], [2.0, 4.0, 5.0], [2.0, 6.0, 5.0], [3.0, 1.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 4.0, 5.0], [7.0, 7.0, 4.0], [5.0, 6.0, 2.0], [5.0, 7.0, 3.0]], [[4.0, 8.0, 7.0], 
[7.0, 8.0, 4.0], [8.0, 8.0, 1.0], [3.0, 6.0, 1.0]], [[5.0, 8.0, 3.0], [5.0, 3.0, 8.0], [4.0, 6.0, 1.0], [5.0, 2.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[6.0, 7.0, 5.0], [3.0, 2.0, 3.0]]])+(1.-msk_val)*numpy.array([[[3.0, 7.0, 7.0], [6.0, 8.0, 
2.0]]])
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 4.0, 7.0], [8.0, 5.0, 7.0], [5.0, 2.0, 5.0], [5.0, 6.0, 7.0]], [[4.0, 1.0, 7.0], [5.0, 
7.0, 8.0], [4.0, 2.0, 8.0], [3.0, 1.0, 4.0]], [[5.0, 5.0, 2.0], [2.0, 4.0, 5.0], [6.0, 7.0, 5.0], [3.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 5.0], [7.0, 7.0, 4.0], [5.0, 6.0, 2.0], [5.0, 7.0, 3.0]], [[4.0, 8.0, 7.0], [7.0, 
8.0, 4.0], [8.0, 8.0, 1.0], [3.0, 6.0, 1.0]], [[5.0, 8.0, 3.0], [5.0, 3.0, 8.0], [3.0, 7.0, 7.0], [6.0, 8.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PRR(self):
      arg=Data(numpy.array([[[5.0, 5.0, 3.0], [1.0, 2.0, 7.0], [1.0, 6.0, 1.0], [7.0, 3.0, 1.0]], [[6.0, 5.0, 8.0], [8.0, 2.0, 
3.0], [4.0, 5.0, 2.0], [5.0, 4.0, 6.0]], [[5.0, 6.0, 5.0], [2.0, 1.0, 5.0], [6.0, 5.0, 8.0], [1.0, 2.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 6.0], [6.0, 7.0, 7.0], [5.0, 4.0, 2.0], [2.0, 4.0, 2.0]], [[5.0, 3.0, 1.0], 
[2.0, 8.0, 4.0], [6.0, 8.0, 1.0], [4.0, 5.0, 1.0]], [[7.0, 5.0, 1.0], [7.0, 4.0, 8.0], [7.0, 3.0, 5.0], [5.0, 8.0, 3.0]]]))
      val=numpy.array([[4.0], [4.0]])
      arg[1,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 5.0, 3.0], [1.0, 2.0, 7.0], [1.0, 6.0, 1.0], [7.0, 3.0, 1.0]], [[6.0, 5.0, 8.0], [8.0, 
4.0, 3.0], [4.0, 4.0, 2.0], [5.0, 4.0, 6.0]], [[5.0, 6.0, 5.0], [2.0, 1.0, 5.0], [6.0, 5.0, 8.0], [1.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 6.0], [6.0, 7.0, 7.0], [5.0, 4.0, 2.0], [2.0, 4.0, 2.0]], [[5.0, 3.0, 1.0], [2.0, 
4.0, 4.0], [6.0, 4.0, 1.0], [4.0, 5.0, 1.0]], [[7.0, 5.0, 1.0], [7.0, 4.0, 8.0], [7.0, 3.0, 5.0], [5.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PRR(self):
      arg=Data(numpy.array([[[5.0, 2.0, 8.0], [1.0, 4.0, 2.0], [5.0, 7.0, 8.0], [7.0, 1.0, 5.0]], [[2.0, 8.0, 6.0], [6.0, 8.0, 
6.0], [6.0, 5.0, 1.0], [3.0, 8.0, 6.0]], [[6.0, 8.0, 6.0], [5.0, 2.0, 4.0], [7.0, 6.0, 5.0], [7.0, 4.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 8.0, 1.0], [4.0, 1.0, 1.0], [2.0, 5.0, 2.0], [5.0, 1.0, 2.0]], [[8.0, 7.0, 5.0], 
[7.0, 3.0, 7.0], [6.0, 6.0, 8.0], [2.0, 7.0, 8.0]], [[5.0, 8.0, 2.0], [6.0, 4.0, 2.0], [5.0, 3.0, 2.0], [3.0, 3.0, 5.0]]]))
      val=Data(numpy.array([[3.0], [1.0]]),self.functionspace)
      arg[1,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 8.0], [1.0, 4.0, 2.0], [5.0, 7.0, 8.0], [7.0, 1.0, 5.0]], [[2.0, 8.0, 6.0], [6.0, 
3.0, 6.0], [6.0, 1.0, 1.0], [3.0, 8.0, 6.0]], [[6.0, 8.0, 6.0], [5.0, 2.0, 4.0], [7.0, 6.0, 5.0], [7.0, 4.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 1.0], [4.0, 1.0, 1.0], [2.0, 5.0, 2.0], [5.0, 1.0, 2.0]], [[8.0, 7.0, 5.0], [7.0, 
3.0, 7.0], [6.0, 1.0, 8.0], [2.0, 7.0, 8.0]], [[5.0, 8.0, 2.0], [6.0, 4.0, 2.0], [5.0, 3.0, 2.0], [3.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PRR(self):
      arg=Data(numpy.array([[[1.0, 6.0, 3.0], [3.0, 3.0, 3.0], [1.0, 3.0, 4.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [4.0, 1.0, 
6.0], [3.0, 6.0, 2.0], [8.0, 8.0, 4.0]], [[1.0, 8.0, 2.0], [8.0, 7.0, 3.0], [2.0, 7.0, 3.0], [7.0, 5.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 1.0, 5.0], [3.0, 8.0, 8.0], [1.0, 7.0, 3.0], [6.0, 7.0, 3.0]], [[4.0, 3.0, 8.0], 
[7.0, 4.0, 8.0], [2.0, 5.0, 6.0], [6.0, 3.0, 5.0]], [[6.0, 5.0, 6.0], [6.0, 3.0, 1.0], [8.0, 8.0, 6.0], [1.0, 5.0, 3.0]]]))
      val=Data(numpy.array([[3.0], [6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0], [6.0]]))
      arg[1,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 3.0], [3.0, 3.0, 3.0], [1.0, 3.0, 4.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [4.0, 
3.0, 6.0], [3.0, 6.0, 2.0], [8.0, 8.0, 4.0]], [[1.0, 8.0, 2.0], [8.0, 7.0, 3.0], [2.0, 7.0, 3.0], [7.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 1.0, 5.0], [3.0, 8.0, 8.0], [1.0, 7.0, 3.0], [6.0, 7.0, 3.0]], [[4.0, 3.0, 8.0], [7.0, 
1.0, 8.0], [2.0, 6.0, 6.0], [6.0, 3.0, 5.0]], [[6.0, 5.0, 6.0], [6.0, 3.0, 1.0], [8.0, 8.0, 6.0], [1.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PRR(self):
      arg=Data(numpy.array([[[4.0, 8.0, 7.0], [2.0, 7.0, 5.0], [4.0, 2.0, 7.0], [3.0, 6.0, 4.0]], [[8.0, 6.0, 2.0], [6.0, 6.0, 
6.0], [2.0, 2.0, 2.0], [8.0, 7.0, 8.0]], [[3.0, 5.0, 4.0], [7.0, 3.0, 5.0], [4.0, 2.0, 7.0], [6.0, 6.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 1.0, 1.0], [1.0, 6.0, 5.0], [1.0, 1.0, 4.0], [4.0, 7.0, 4.0]], [[7.0, 5.0, 4.0], 
[3.0, 3.0, 2.0], [5.0, 6.0, 4.0], [3.0, 3.0, 4.0]], [[5.0, 6.0, 1.0], [3.0, 3.0, 8.0], [8.0, 7.0, 2.0], [6.0, 8.0, 1.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[3.0], [6.0]])+(1.-msk_val)*numpy.array([[2.0], [3.0]])
      arg[1,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 7.0], [2.0, 7.0, 5.0], [4.0, 2.0, 7.0], [3.0, 6.0, 4.0]], [[8.0, 6.0, 2.0], [6.0, 
3.0, 6.0], [2.0, 6.0, 2.0], [8.0, 7.0, 8.0]], [[3.0, 5.0, 4.0], [7.0, 3.0, 5.0], [4.0, 2.0, 7.0], [6.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 1.0], [1.0, 6.0, 5.0], [1.0, 1.0, 4.0], [4.0, 7.0, 4.0]], [[7.0, 5.0, 4.0], [3.0, 
2.0, 2.0], [5.0, 3.0, 4.0], [3.0, 3.0, 4.0]], [[5.0, 6.0, 1.0], [3.0, 3.0, 8.0], [8.0, 7.0, 2.0], [6.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CRN(self):
      arg=Data(numpy.array([[[4.0, 4.0, 3.0], [2.0, 7.0, 7.0], [3.0, 4.0, 7.0], [6.0, 6.0, 4.0]], [[8.0, 7.0, 1.0], [5.0, 7.0, 
4.0], [1.0, 6.0, 6.0], [4.0, 2.0, 2.0]], [[4.0, 5.0, 6.0], [7.0, 6.0, 3.0], [6.0, 1.0, 2.0], [6.0, 8.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 2.0, 5.0], [4.0, 6.0, 8.0], [4.0, 6.0, 3.0], [2.0, 1.0, 7.0]], [[7.0, 5.0, 2.0], 
[8.0, 5.0, 4.0], [8.0, 8.0, 5.0], [6.0, 3.0, 8.0]], [[5.0, 1.0, 3.0], [5.0, 7.0, 2.0], [6.0, 6.0, 1.0], [2.0, 6.0, 8.0]]]))
      val=numpy.array([[[7.0, 8.0, 2.0], [2.0, 2.0, 5.0]], [[3.0, 8.0, 3.0], [1.0, 7.0, 8.0]], [[7.0, 4.0, 3.0], [4.0, 1.0, 
5.0]]])
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 3.0], [7.0, 8.0, 2.0], [2.0, 2.0, 5.0], [6.0, 6.0, 4.0]], [[8.0, 7.0, 1.0], [3.0, 
8.0, 3.0], [1.0, 7.0, 8.0], [4.0, 2.0, 2.0]], [[4.0, 5.0, 6.0], [7.0, 4.0, 3.0], [4.0, 1.0, 5.0], [6.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 5.0], [7.0, 8.0, 2.0], [2.0, 2.0, 5.0], [2.0, 1.0, 7.0]], [[7.0, 5.0, 2.0], [3.0, 
8.0, 3.0], [1.0, 7.0, 8.0], [6.0, 3.0, 8.0]], [[5.0, 1.0, 3.0], [7.0, 4.0, 3.0], [4.0, 1.0, 5.0], [2.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CRN(self):
      arg=Data(numpy.array([[[3.0, 5.0, 6.0], [3.0, 4.0, 2.0], [8.0, 3.0, 6.0], [1.0, 6.0, 3.0]], [[1.0, 1.0, 2.0], [4.0, 3.0, 
4.0], [8.0, 1.0, 1.0], [8.0, 6.0, 8.0]], [[7.0, 3.0, 2.0], [1.0, 6.0, 4.0], [6.0, 3.0, 4.0], [5.0, 8.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 4.0, 2.0], [6.0, 6.0, 4.0], [8.0, 3.0, 2.0], [5.0, 7.0, 8.0]], [[1.0, 4.0, 6.0], 
[8.0, 2.0, 1.0], [2.0, 3.0, 3.0], [5.0, 6.0, 4.0]], [[8.0, 7.0, 8.0], [1.0, 4.0, 2.0], [3.0, 2.0, 3.0], [1.0, 6.0, 7.0]]]))
      val=Data(numpy.array([[[5.0, 2.0, 4.0], [8.0, 6.0, 8.0]], [[5.0, 5.0, 8.0], [7.0, 7.0, 6.0]], [[2.0, 4.0, 2.0], [8.0, 
5.0, 6.0]]]),self.functionspace)
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 6.0], [5.0, 2.0, 4.0], [8.0, 6.0, 8.0], [1.0, 6.0, 3.0]], [[1.0, 1.0, 2.0], [5.0, 
5.0, 8.0], [7.0, 7.0, 6.0], [8.0, 6.0, 8.0]], [[7.0, 3.0, 2.0], [2.0, 4.0, 2.0], [8.0, 5.0, 6.0], [5.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 2.0], [5.0, 2.0, 4.0], [8.0, 6.0, 8.0], [5.0, 7.0, 8.0]], [[1.0, 4.0, 6.0], [5.0, 
5.0, 8.0], [7.0, 7.0, 6.0], [5.0, 6.0, 4.0]], [[8.0, 7.0, 8.0], [2.0, 4.0, 2.0], [8.0, 5.0, 6.0], [1.0, 6.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CRN(self):
      arg=Data(numpy.array([[[7.0, 4.0, 6.0], [4.0, 4.0, 6.0], [7.0, 4.0, 8.0], [6.0, 3.0, 1.0]], [[7.0, 1.0, 3.0], [4.0, 2.0, 
3.0], [1.0, 8.0, 6.0], [1.0, 1.0, 4.0]], [[2.0, 6.0, 3.0], [4.0, 2.0, 7.0], [5.0, 3.0, 3.0], [8.0, 8.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 7.0, 8.0], [2.0, 6.0, 8.0], [5.0, 6.0, 8.0], [7.0, 2.0, 1.0]], [[7.0, 7.0, 6.0], 
[3.0, 5.0, 4.0], [6.0, 4.0, 2.0], [2.0, 3.0, 4.0]], [[3.0, 6.0, 4.0], [4.0, 7.0, 4.0], [6.0, 2.0, 3.0], [1.0, 7.0, 2.0]]]))
      val=Data(numpy.array([[[5.0, 5.0, 5.0], [6.0, 2.0, 7.0]], [[5.0, 3.0, 1.0], [3.0, 6.0, 2.0]], [[1.0, 7.0, 3.0], [3.0, 
7.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 5.0, 1.0], [3.0, 5.0, 3.0]], [[1.0, 1.0, 1.0], [7.0, 2.0, 8.0]], [[7.0, 1.0, 
6.0], [7.0, 6.0, 8.0]]]))
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 6.0], [5.0, 5.0, 5.0], [6.0, 2.0, 7.0], [6.0, 3.0, 1.0]], [[7.0, 1.0, 3.0], [5.0, 
3.0, 1.0], [3.0, 6.0, 2.0], [1.0, 1.0, 4.0]], [[2.0, 6.0, 3.0], [1.0, 7.0, 3.0], [3.0, 7.0, 3.0], [8.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 7.0, 8.0], [7.0, 5.0, 1.0], [3.0, 5.0, 3.0], [7.0, 2.0, 1.0]], [[7.0, 7.0, 6.0], [1.0, 
1.0, 1.0], [7.0, 2.0, 8.0], [2.0, 3.0, 4.0]], [[3.0, 6.0, 4.0], [7.0, 1.0, 6.0], [7.0, 6.0, 8.0], [1.0, 7.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CRN(self):
      arg=Data(numpy.array([[[8.0, 3.0, 2.0], [2.0, 5.0, 7.0], [8.0, 5.0, 8.0], [2.0, 5.0, 1.0]], [[4.0, 7.0, 7.0], [3.0, 1.0, 
6.0], [5.0, 3.0, 1.0], [4.0, 1.0, 2.0]], [[6.0, 8.0, 3.0], [6.0, 2.0, 6.0], [2.0, 8.0, 1.0], [4.0, 3.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 5.0, 6.0], [8.0, 1.0, 1.0], [1.0, 7.0, 3.0], [7.0, 1.0, 1.0]], [[3.0, 7.0, 1.0], 
[6.0, 4.0, 8.0], [3.0, 8.0, 2.0], [2.0, 7.0, 4.0]], [[6.0, 6.0, 1.0], [5.0, 5.0, 6.0], [1.0, 7.0, 2.0], [3.0, 6.0, 5.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 5.0, 7.0], [1.0, 7.0, 4.0]], [[1.0, 5.0, 3.0], [3.0, 2.0, 4.0]], [[2.0, 7.0, 1.0], [7.0, 
4.0, 1.0]]])+(1.-msk_val)*numpy.array([[[3.0, 5.0, 8.0], [2.0, 5.0, 2.0]], [[8.0, 4.0, 5.0], [1.0, 1.0, 5.0]], [[5.0, 8.0, 
3.0], [8.0, 7.0, 5.0]]])
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 2.0], [8.0, 5.0, 7.0], [1.0, 7.0, 4.0], [2.0, 5.0, 1.0]], [[4.0, 7.0, 7.0], [1.0, 
5.0, 3.0], [3.0, 2.0, 4.0], [4.0, 1.0, 2.0]], [[6.0, 8.0, 3.0], [2.0, 7.0, 1.0], [7.0, 4.0, 1.0], [4.0, 3.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 6.0], [3.0, 5.0, 8.0], [2.0, 5.0, 2.0], [7.0, 1.0, 1.0]], [[3.0, 7.0, 1.0], [8.0, 
4.0, 5.0], [1.0, 1.0, 5.0], [2.0, 7.0, 4.0]], [[6.0, 6.0, 1.0], [5.0, 8.0, 3.0], [8.0, 7.0, 5.0], [3.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UPC(self):
      arg=Data(numpy.array([[[2.0, 6.0, 7.0], [5.0, 6.0, 2.0], [4.0, 6.0, 5.0], [2.0, 2.0, 8.0]], [[4.0, 5.0, 6.0], [5.0, 5.0, 
6.0], [4.0, 2.0, 4.0], [6.0, 1.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 8.0, 1.0], [4.0, 4.0, 2.0], [8.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 8.0, 5.0], [3.0, 3.0, 8.0], [2.0, 6.0, 8.0], [2.0, 7.0, 3.0]], [[8.0, 5.0, 1.0], 
[7.0, 7.0, 2.0], [5.0, 7.0, 2.0], [1.0, 6.0, 8.0]], [[3.0, 1.0, 5.0], [1.0, 5.0, 7.0], [5.0, 3.0, 4.0], [3.0, 8.0, 2.0]]]))
      val=numpy.array([[2.0, 5.0, 5.0], [5.0, 4.0, 5.0]])
      arg[:2,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 6.0, 7.0], [5.0, 6.0, 2.0], [2.0, 5.0, 5.0], [2.0, 2.0, 8.0]], [[4.0, 5.0, 6.0], [5.0, 
5.0, 6.0], [5.0, 4.0, 5.0], [6.0, 1.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 8.0, 1.0], [4.0, 4.0, 2.0], [8.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 5.0], [3.0, 3.0, 8.0], [2.0, 5.0, 5.0], [2.0, 7.0, 3.0]], [[8.0, 5.0, 1.0], [7.0, 
7.0, 2.0], [5.0, 4.0, 5.0], [1.0, 6.0, 8.0]], [[3.0, 1.0, 5.0], [1.0, 5.0, 7.0], [5.0, 3.0, 4.0], [3.0, 8.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UPC(self):
      arg=Data(numpy.array([[[2.0, 1.0, 6.0], [5.0, 8.0, 4.0], [2.0, 3.0, 4.0], [5.0, 1.0, 5.0]], [[2.0, 2.0, 1.0], [4.0, 6.0, 
2.0], [7.0, 4.0, 5.0], [8.0, 7.0, 4.0]], [[1.0, 5.0, 7.0], [4.0, 8.0, 8.0], [1.0, 8.0, 1.0], [7.0, 4.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 7.0, 8.0], [3.0, 1.0, 2.0], [7.0, 6.0, 6.0], [8.0, 6.0, 1.0]], [[3.0, 2.0, 5.0], 
[7.0, 5.0, 3.0], [6.0, 5.0, 5.0], [3.0, 8.0, 1.0]], [[7.0, 8.0, 4.0], [2.0, 8.0, 8.0], [6.0, 6.0, 1.0], [7.0, 8.0, 5.0]]]))
      val=Data(numpy.array([[7.0, 1.0, 4.0], [7.0, 3.0, 3.0]]),self.functionspace)
      arg[:2,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 6.0], [5.0, 8.0, 4.0], [7.0, 1.0, 4.0], [5.0, 1.0, 5.0]], [[2.0, 2.0, 1.0], [4.0, 
6.0, 2.0], [7.0, 3.0, 3.0], [8.0, 7.0, 4.0]], [[1.0, 5.0, 7.0], [4.0, 8.0, 8.0], [1.0, 8.0, 1.0], [7.0, 4.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 8.0], [3.0, 1.0, 2.0], [7.0, 1.0, 4.0], [8.0, 6.0, 1.0]], [[3.0, 2.0, 5.0], [7.0, 
5.0, 3.0], [7.0, 3.0, 3.0], [3.0, 8.0, 1.0]], [[7.0, 8.0, 4.0], [2.0, 8.0, 8.0], [6.0, 6.0, 1.0], [7.0, 8.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UPC(self):
      arg=Data(numpy.array([[[8.0, 4.0, 5.0], [6.0, 2.0, 3.0], [2.0, 4.0, 5.0], [4.0, 5.0, 2.0]], [[2.0, 6.0, 6.0], [4.0, 2.0, 
6.0], [1.0, 6.0, 1.0], [5.0, 3.0, 8.0]], [[3.0, 3.0, 5.0], [6.0, 5.0, 2.0], [6.0, 8.0, 6.0], [2.0, 5.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 6.0, 5.0], [8.0, 1.0, 7.0], [5.0, 7.0, 3.0], [8.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], 
[1.0, 2.0, 4.0], [8.0, 6.0, 4.0], [3.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], [3.0, 1.0, 2.0], [6.0, 6.0, 8.0], [2.0, 6.0, 2.0]]]))
      val=Data(numpy.array([[5.0, 1.0, 8.0], [1.0, 7.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 8.0, 3.0], [5.0, 2.0, 6.0]]))
      arg[:2,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 5.0], [6.0, 2.0, 3.0], [5.0, 1.0, 8.0], [4.0, 5.0, 2.0]], [[2.0, 6.0, 6.0], [4.0, 
2.0, 6.0], [1.0, 7.0, 6.0], [5.0, 3.0, 8.0]], [[3.0, 3.0, 5.0], [6.0, 5.0, 2.0], [6.0, 8.0, 6.0], [2.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 5.0], [8.0, 1.0, 7.0], [8.0, 8.0, 3.0], [8.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], [1.0, 
2.0, 4.0], [5.0, 2.0, 6.0], [3.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], [3.0, 1.0, 2.0], [6.0, 6.0, 8.0], [2.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UPC(self):
      arg=Data(numpy.array([[[8.0, 2.0, 1.0], [3.0, 2.0, 8.0], [7.0, 4.0, 3.0], [8.0, 4.0, 5.0]], [[1.0, 1.0, 1.0], [4.0, 1.0, 
2.0], [4.0, 1.0, 2.0], [2.0, 5.0, 7.0]], [[8.0, 7.0, 4.0], [8.0, 3.0, 8.0], [1.0, 7.0, 2.0], [8.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 3.0, 1.0], [3.0, 1.0, 1.0], [4.0, 4.0, 6.0], [1.0, 6.0, 3.0]], [[2.0, 6.0, 3.0], 
[2.0, 6.0, 4.0], [5.0, 8.0, 6.0], [6.0, 3.0, 3.0]], [[5.0, 7.0, 4.0], [3.0, 6.0, 8.0], [5.0, 6.0, 3.0], [4.0, 3.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[4.0, 5.0, 8.0], [3.0, 6.0, 3.0]])+(1.-msk_val)*numpy.array([[5.0, 6.0, 2.0], [7.0, 1.0, 7.0]])
      arg[:2,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 1.0], [3.0, 2.0, 8.0], [4.0, 5.0, 8.0], [8.0, 4.0, 5.0]], [[1.0, 1.0, 1.0], [4.0, 
1.0, 2.0], [3.0, 6.0, 3.0], [2.0, 5.0, 7.0]], [[8.0, 7.0, 4.0], [8.0, 3.0, 8.0], [1.0, 7.0, 2.0], [8.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 1.0], [3.0, 1.0, 1.0], [5.0, 6.0, 2.0], [1.0, 6.0, 3.0]], [[2.0, 6.0, 3.0], [2.0, 
6.0, 4.0], [7.0, 1.0, 7.0], [6.0, 3.0, 3.0]], [[5.0, 7.0, 4.0], [3.0, 6.0, 8.0], [5.0, 6.0, 3.0], [4.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UUC(self):
      arg=Data(numpy.array([[[3.0, 2.0, 8.0], [7.0, 1.0, 8.0], [8.0, 6.0, 1.0], [8.0, 4.0, 8.0]], [[7.0, 3.0, 3.0], [8.0, 7.0, 
1.0], [8.0, 1.0, 5.0], [2.0, 5.0, 8.0]], [[7.0, 4.0, 7.0], [4.0, 1.0, 2.0], [2.0, 8.0, 3.0], [4.0, 8.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 1.0, 2.0], [3.0, 2.0, 5.0], [7.0, 7.0, 2.0], [7.0, 1.0, 2.0]], [[5.0, 2.0, 2.0], 
[2.0, 5.0, 7.0], [1.0, 1.0, 6.0], [8.0, 7.0, 3.0]], [[3.0, 5.0, 2.0], [1.0, 7.0, 8.0], [2.0, 4.0, 7.0], [5.0, 1.0, 6.0]]]))
      val=numpy.array([[[8.0, 1.0, 2.0], [2.0, 7.0, 1.0], [6.0, 7.0, 7.0]], [[3.0, 4.0, 2.0], [3.0, 3.0, 8.0], [3.0, 1.0, 
4.0]]])
      arg[:2,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 1.0, 2.0], [2.0, 7.0, 1.0], [6.0, 7.0, 7.0], [8.0, 4.0, 8.0]], [[3.0, 4.0, 2.0], [3.0, 
3.0, 8.0], [3.0, 1.0, 4.0], [2.0, 5.0, 8.0]], [[7.0, 4.0, 7.0], [4.0, 1.0, 2.0], [2.0, 8.0, 3.0], [4.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 1.0, 2.0], [2.0, 7.0, 1.0], [6.0, 7.0, 7.0], [7.0, 1.0, 2.0]], [[3.0, 4.0, 2.0], [3.0, 
3.0, 8.0], [3.0, 1.0, 4.0], [8.0, 7.0, 3.0]], [[3.0, 5.0, 2.0], [1.0, 7.0, 8.0], [2.0, 4.0, 7.0], [5.0, 1.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UUC(self):
      arg=Data(numpy.array([[[7.0, 7.0, 6.0], [7.0, 1.0, 5.0], [3.0, 1.0, 5.0], [6.0, 1.0, 1.0]], [[8.0, 1.0, 3.0], [3.0, 6.0, 
4.0], [4.0, 7.0, 6.0], [8.0, 2.0, 2.0]], [[7.0, 7.0, 7.0], [3.0, 1.0, 6.0], [4.0, 6.0, 7.0], [5.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 4.0, 6.0], [2.0, 2.0, 7.0], [8.0, 2.0, 7.0], [5.0, 7.0, 4.0]], [[6.0, 2.0, 6.0], 
[1.0, 5.0, 7.0], [2.0, 5.0, 4.0], [3.0, 3.0, 6.0]], [[4.0, 2.0, 2.0], [1.0, 6.0, 1.0], [7.0, 5.0, 2.0], [5.0, 5.0, 4.0]]]))
      val=Data(numpy.array([[[1.0, 6.0, 7.0], [1.0, 8.0, 5.0], [1.0, 2.0, 7.0]], [[6.0, 5.0, 2.0], [7.0, 1.0, 1.0], [8.0, 5.0, 
2.0]]]),self.functionspace)
      arg[:2,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 7.0], [1.0, 8.0, 5.0], [1.0, 2.0, 7.0], [6.0, 1.0, 1.0]], [[6.0, 5.0, 2.0], [7.0, 
1.0, 1.0], [8.0, 5.0, 2.0], [8.0, 2.0, 2.0]], [[7.0, 7.0, 7.0], [3.0, 1.0, 6.0], [4.0, 6.0, 7.0], [5.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 7.0], [1.0, 8.0, 5.0], [1.0, 2.0, 7.0], [5.0, 7.0, 4.0]], [[6.0, 5.0, 2.0], [7.0, 
1.0, 1.0], [8.0, 5.0, 2.0], [3.0, 3.0, 6.0]], [[4.0, 2.0, 2.0], [1.0, 6.0, 1.0], [7.0, 5.0, 2.0], [5.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UUC(self):
      arg=Data(numpy.array([[[3.0, 5.0, 8.0], [3.0, 1.0, 5.0], [2.0, 8.0, 8.0], [4.0, 4.0, 6.0]], [[1.0, 6.0, 7.0], [3.0, 8.0, 
7.0], [1.0, 1.0, 3.0], [1.0, 2.0, 4.0]], [[1.0, 3.0, 3.0], [3.0, 8.0, 8.0], [1.0, 5.0, 6.0], [2.0, 4.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 6.0, 7.0], [2.0, 7.0, 4.0], [8.0, 7.0, 8.0], [5.0, 8.0, 1.0]], [[7.0, 8.0, 6.0], 
[5.0, 1.0, 7.0], [4.0, 2.0, 2.0], [5.0, 8.0, 2.0]], [[4.0, 2.0, 3.0], [1.0, 8.0, 5.0], [4.0, 4.0, 4.0], [1.0, 2.0, 4.0]]]))
      val=Data(numpy.array([[[1.0, 5.0, 3.0], [3.0, 1.0, 6.0], [5.0, 5.0, 2.0]], [[6.0, 6.0, 3.0], [4.0, 2.0, 3.0], [4.0, 3.0, 
1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 1.0, 4.0], [1.0, 4.0, 3.0], [1.0, 4.0, 2.0]], [[2.0, 8.0, 4.0], [5.0, 2.0, 3.0], 
[2.0, 6.0, 8.0]]]))
      arg[:2,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 3.0], [3.0, 1.0, 6.0], [5.0, 5.0, 2.0], [4.0, 4.0, 6.0]], [[6.0, 6.0, 3.0], [4.0, 
2.0, 3.0], [4.0, 3.0, 1.0], [1.0, 2.0, 4.0]], [[1.0, 3.0, 3.0], [3.0, 8.0, 8.0], [1.0, 5.0, 6.0], [2.0, 4.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 4.0], [1.0, 4.0, 3.0], [1.0, 4.0, 2.0], [5.0, 8.0, 1.0]], [[2.0, 8.0, 4.0], [5.0, 
2.0, 3.0], [2.0, 6.0, 8.0], [5.0, 8.0, 2.0]], [[4.0, 2.0, 3.0], [1.0, 8.0, 5.0], [4.0, 4.0, 4.0], [1.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UUC(self):
      arg=Data(numpy.array([[[2.0, 2.0, 2.0], [3.0, 1.0, 5.0], [6.0, 2.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 7.0, 6.0], [1.0, 6.0, 
8.0], [1.0, 6.0, 4.0], [3.0, 1.0, 1.0]], [[4.0, 3.0, 7.0], [5.0, 2.0, 7.0], [2.0, 6.0, 1.0], [8.0, 5.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 4.0, 6.0], [5.0, 7.0, 5.0], [1.0, 4.0, 1.0], [4.0, 2.0, 1.0]], [[4.0, 7.0, 4.0], 
[6.0, 2.0, 8.0], [7.0, 4.0, 7.0], [1.0, 1.0, 8.0]], [[2.0, 1.0, 4.0], [7.0, 4.0, 5.0], [7.0, 3.0, 3.0], [1.0, 3.0, 5.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 8.0, 4.0], [5.0, 7.0, 1.0], [1.0, 6.0, 6.0]], [[5.0, 2.0, 1.0], [6.0, 6.0, 8.0], [3.0, 
8.0, 6.0]]])+(1.-msk_val)*numpy.array([[[2.0, 8.0, 6.0], [8.0, 2.0, 4.0], [1.0, 4.0, 8.0]], [[1.0, 7.0, 4.0], [1.0, 5.0, 1.0], 
[2.0, 5.0, 8.0]]])
      arg[:2,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 4.0], [5.0, 7.0, 1.0], [1.0, 6.0, 6.0], [8.0, 5.0, 5.0]], [[5.0, 2.0, 1.0], [6.0, 
6.0, 8.0], [3.0, 8.0, 6.0], [3.0, 1.0, 1.0]], [[4.0, 3.0, 7.0], [5.0, 2.0, 7.0], [2.0, 6.0, 1.0], [8.0, 5.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 6.0], [8.0, 2.0, 4.0], [1.0, 4.0, 8.0], [4.0, 2.0, 1.0]], [[1.0, 7.0, 4.0], [1.0, 
5.0, 1.0], [2.0, 5.0, 8.0], [1.0, 1.0, 8.0]], [[2.0, 1.0, 4.0], [7.0, 4.0, 5.0], [7.0, 3.0, 3.0], [1.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LLR(self):
      arg=Data(numpy.array([[[4.0, 1.0, 7.0], [6.0, 4.0, 8.0], [7.0, 2.0, 7.0], [7.0, 5.0, 2.0]], [[5.0, 6.0, 8.0], [3.0, 2.0, 
8.0], [6.0, 7.0, 1.0], [2.0, 5.0, 7.0]], [[6.0, 2.0, 1.0], [5.0, 7.0, 3.0], [6.0, 1.0, 6.0], [6.0, 3.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 7.0], [8.0, 6.0, 7.0], [5.0, 5.0, 3.0], [3.0, 5.0, 5.0]], [[8.0, 7.0, 4.0], 
[2.0, 6.0, 1.0], [7.0, 7.0, 5.0], [6.0, 2.0, 3.0]], [[6.0, 8.0, 6.0], [7.0, 1.0, 1.0], [8.0, 2.0, 8.0], [5.0, 6.0, 8.0]]]))
      val=numpy.array([[[3.0], [5.0]]])
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 1.0, 7.0], [6.0, 4.0, 8.0], [7.0, 2.0, 7.0], [7.0, 5.0, 2.0]], [[5.0, 6.0, 8.0], [3.0, 
2.0, 8.0], [6.0, 7.0, 1.0], [2.0, 5.0, 7.0]], [[6.0, 2.0, 1.0], [5.0, 7.0, 3.0], [6.0, 3.0, 6.0], [6.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 7.0], [8.0, 6.0, 7.0], [5.0, 5.0, 3.0], [3.0, 5.0, 5.0]], [[8.0, 7.0, 4.0], [2.0, 
6.0, 1.0], [7.0, 7.0, 5.0], [6.0, 2.0, 3.0]], [[6.0, 8.0, 6.0], [7.0, 1.0, 1.0], [8.0, 3.0, 8.0], [5.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LLR(self):
      arg=Data(numpy.array([[[6.0, 5.0, 3.0], [2.0, 1.0, 8.0], [3.0, 4.0, 2.0], [5.0, 7.0, 4.0]], [[3.0, 8.0, 5.0], [2.0, 6.0, 
3.0], [3.0, 8.0, 7.0], [3.0, 1.0, 3.0]], [[6.0, 5.0, 3.0], [1.0, 4.0, 3.0], [3.0, 3.0, 6.0], [4.0, 7.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 3.0, 4.0], [7.0, 2.0, 7.0], [1.0, 8.0, 4.0], [8.0, 4.0, 3.0]], [[3.0, 6.0, 6.0], 
[1.0, 3.0, 8.0], [7.0, 2.0, 7.0], [2.0, 7.0, 6.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 2.0], [3.0, 8.0, 5.0], [8.0, 3.0, 5.0]]]))
      val=Data(numpy.array([[[7.0], [5.0]]]),self.functionspace)
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 3.0], [2.0, 1.0, 8.0], [3.0, 4.0, 2.0], [5.0, 7.0, 4.0]], [[3.0, 8.0, 5.0], [2.0, 
6.0, 3.0], [3.0, 8.0, 7.0], [3.0, 1.0, 3.0]], [[6.0, 5.0, 3.0], [1.0, 4.0, 3.0], [3.0, 7.0, 6.0], [4.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 4.0], [7.0, 2.0, 7.0], [1.0, 8.0, 4.0], [8.0, 4.0, 3.0]], [[3.0, 6.0, 6.0], [1.0, 
3.0, 8.0], [7.0, 2.0, 7.0], [2.0, 7.0, 6.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 2.0], [3.0, 7.0, 5.0], [8.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LLR(self):
      arg=Data(numpy.array([[[2.0, 6.0, 6.0], [3.0, 7.0, 6.0], [3.0, 6.0, 5.0], [4.0, 5.0, 7.0]], [[6.0, 4.0, 8.0], [8.0, 5.0, 
5.0], [6.0, 4.0, 7.0], [4.0, 3.0, 4.0]], [[4.0, 6.0, 3.0], [8.0, 2.0, 1.0], [5.0, 7.0, 2.0], [5.0, 1.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 4.0, 3.0], [7.0, 1.0, 2.0], [7.0, 7.0, 1.0], [6.0, 4.0, 1.0]], [[2.0, 2.0, 8.0], 
[7.0, 3.0, 1.0], [4.0, 3.0, 5.0], [6.0, 5.0, 3.0]], [[7.0, 3.0, 8.0], [3.0, 4.0, 1.0], [6.0, 7.0, 8.0], [3.0, 3.0, 5.0]]]))
      val=Data(numpy.array([[[4.0], [1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0], [8.0]]]))
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 6.0, 6.0], [3.0, 7.0, 6.0], [3.0, 6.0, 5.0], [4.0, 5.0, 7.0]], [[6.0, 4.0, 8.0], [8.0, 
5.0, 5.0], [6.0, 4.0, 7.0], [4.0, 3.0, 4.0]], [[4.0, 6.0, 3.0], [8.0, 2.0, 1.0], [5.0, 4.0, 2.0], [5.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 3.0], [7.0, 1.0, 2.0], [7.0, 7.0, 1.0], [6.0, 4.0, 1.0]], [[2.0, 2.0, 8.0], [7.0, 
3.0, 1.0], [4.0, 3.0, 5.0], [6.0, 5.0, 3.0]], [[7.0, 3.0, 8.0], [3.0, 4.0, 1.0], [6.0, 6.0, 8.0], [3.0, 8.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LLR(self):
      arg=Data(numpy.array([[[1.0, 2.0, 1.0], [7.0, 4.0, 7.0], [1.0, 7.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 5.0, 7.0], [5.0, 4.0, 
3.0], [6.0, 1.0, 3.0], [2.0, 6.0, 3.0]], [[1.0, 3.0, 7.0], [2.0, 4.0, 4.0], [8.0, 4.0, 1.0], [6.0, 3.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 8.0, 8.0], [8.0, 8.0, 8.0], [8.0, 1.0, 3.0], [1.0, 6.0, 2.0]], [[2.0, 2.0, 5.0], 
[7.0, 2.0, 3.0], [4.0, 8.0, 1.0], [6.0, 7.0, 6.0]], [[4.0, 1.0, 7.0], [5.0, 1.0, 8.0], [5.0, 7.0, 2.0], [7.0, 3.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0], [1.0]]])+(1.-msk_val)*numpy.array([[[6.0], [7.0]]])
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 1.0], [7.0, 4.0, 7.0], [1.0, 7.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 5.0, 7.0], [5.0, 
4.0, 3.0], [6.0, 1.0, 3.0], [2.0, 6.0, 3.0]], [[1.0, 3.0, 7.0], [2.0, 4.0, 4.0], [8.0, 7.0, 1.0], [6.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 8.0], [8.0, 8.0, 8.0], [8.0, 1.0, 3.0], [1.0, 6.0, 2.0]], [[2.0, 2.0, 5.0], [7.0, 
2.0, 3.0], [4.0, 8.0, 1.0], [6.0, 7.0, 6.0]], [[4.0, 1.0, 7.0], [5.0, 1.0, 8.0], [5.0, 6.0, 2.0], [7.0, 7.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CPN(self):
      arg=Data(numpy.array([[[5.0, 6.0, 2.0], [8.0, 7.0, 5.0], [8.0, 7.0, 8.0], [7.0, 6.0, 6.0]], [[6.0, 2.0, 5.0], [6.0, 8.0, 
4.0], [4.0, 6.0, 7.0], [6.0, 1.0, 7.0]], [[6.0, 7.0, 5.0], [7.0, 7.0, 8.0], [3.0, 8.0, 8.0], [6.0, 3.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 7.0, 3.0], [3.0, 5.0, 7.0], [6.0, 2.0, 1.0], [6.0, 8.0, 3.0]], [[5.0, 5.0, 5.0], 
[3.0, 6.0, 5.0], [2.0, 1.0, 4.0], [8.0, 8.0, 4.0]], [[3.0, 6.0, 5.0], [1.0, 1.0, 2.0], [8.0, 5.0, 8.0], [2.0, 5.0, 7.0]]]))
      val=numpy.array([[2.0, 8.0, 6.0], [2.0, 1.0, 2.0], [3.0, 8.0, 7.0]])
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 2.0], [8.0, 7.0, 5.0], [2.0, 8.0, 6.0], [7.0, 6.0, 6.0]], [[6.0, 2.0, 5.0], [6.0, 
8.0, 4.0], [2.0, 1.0, 2.0], [6.0, 1.0, 7.0]], [[6.0, 7.0, 5.0], [7.0, 7.0, 8.0], [3.0, 8.0, 7.0], [6.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 7.0, 3.0], [3.0, 5.0, 7.0], [2.0, 8.0, 6.0], [6.0, 8.0, 3.0]], [[5.0, 5.0, 5.0], [3.0, 
6.0, 5.0], [2.0, 1.0, 2.0], [8.0, 8.0, 4.0]], [[3.0, 6.0, 5.0], [1.0, 1.0, 2.0], [3.0, 8.0, 7.0], [2.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CPN(self):
      arg=Data(numpy.array([[[2.0, 2.0, 4.0], [8.0, 2.0, 5.0], [8.0, 8.0, 1.0], [5.0, 8.0, 4.0]], [[6.0, 4.0, 5.0], [4.0, 2.0, 
5.0], [4.0, 6.0, 7.0], [3.0, 2.0, 6.0]], [[2.0, 2.0, 8.0], [5.0, 5.0, 5.0], [3.0, 8.0, 8.0], [8.0, 7.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 2.0, 6.0], [2.0, 7.0, 3.0], [2.0, 3.0, 7.0], [2.0, 3.0, 7.0]], [[8.0, 8.0, 5.0], 
[1.0, 2.0, 6.0], [7.0, 4.0, 3.0], [1.0, 4.0, 6.0]], [[2.0, 5.0, 8.0], [6.0, 7.0, 6.0], [8.0, 6.0, 6.0], [5.0, 7.0, 6.0]]]))
      val=Data(numpy.array([[1.0, 3.0, 2.0], [2.0, 6.0, 5.0], [8.0, 4.0, 8.0]]),self.functionspace)
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 4.0], [8.0, 2.0, 5.0], [1.0, 3.0, 2.0], [5.0, 8.0, 4.0]], [[6.0, 4.0, 5.0], [4.0, 
2.0, 5.0], [2.0, 6.0, 5.0], [3.0, 2.0, 6.0]], [[2.0, 2.0, 8.0], [5.0, 5.0, 5.0], [8.0, 4.0, 8.0], [8.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 6.0], [2.0, 7.0, 3.0], [1.0, 3.0, 2.0], [2.0, 3.0, 7.0]], [[8.0, 8.0, 5.0], [1.0, 
2.0, 6.0], [2.0, 6.0, 5.0], [1.0, 4.0, 6.0]], [[2.0, 5.0, 8.0], [6.0, 7.0, 6.0], [8.0, 4.0, 8.0], [5.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CPN(self):
      arg=Data(numpy.array([[[8.0, 8.0, 8.0], [4.0, 8.0, 7.0], [8.0, 4.0, 5.0], [1.0, 5.0, 7.0]], [[3.0, 8.0, 6.0], [7.0, 1.0, 
6.0], [1.0, 7.0, 7.0], [6.0, 8.0, 7.0]], [[6.0, 7.0, 8.0], [8.0, 2.0, 1.0], [2.0, 7.0, 7.0], [8.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 2.0, 2.0], [8.0, 1.0, 2.0], [7.0, 5.0, 5.0], [8.0, 2.0, 1.0]], [[6.0, 4.0, 1.0], 
[3.0, 5.0, 8.0], [2.0, 6.0, 6.0], [5.0, 2.0, 1.0]], [[8.0, 1.0, 6.0], [3.0, 1.0, 6.0], [4.0, 7.0, 2.0], [6.0, 3.0, 4.0]]]))
      val=Data(numpy.array([[8.0, 4.0, 7.0], [8.0, 8.0, 7.0], [7.0, 8.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 4.0, 2.0], [3.0, 8.0, 1.0], [1.0, 2.0, 1.0]]))
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 8.0, 8.0], [4.0, 8.0, 7.0], [8.0, 4.0, 7.0], [1.0, 5.0, 7.0]], [[3.0, 8.0, 6.0], [7.0, 
1.0, 6.0], [8.0, 8.0, 7.0], [6.0, 8.0, 7.0]], [[6.0, 7.0, 8.0], [8.0, 2.0, 1.0], [7.0, 8.0, 7.0], [8.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 2.0], [8.0, 1.0, 2.0], [6.0, 4.0, 2.0], [8.0, 2.0, 1.0]], [[6.0, 4.0, 1.0], [3.0, 
5.0, 8.0], [3.0, 8.0, 1.0], [5.0, 2.0, 1.0]], [[8.0, 1.0, 6.0], [3.0, 1.0, 6.0], [1.0, 2.0, 1.0], [6.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CPN(self):
      arg=Data(numpy.array([[[8.0, 5.0, 2.0], [8.0, 7.0, 2.0], [4.0, 2.0, 5.0], [6.0, 4.0, 6.0]], [[6.0, 7.0, 1.0], [1.0, 5.0, 
5.0], [3.0, 7.0, 8.0], [7.0, 7.0, 3.0]], [[8.0, 6.0, 1.0], [4.0, 3.0, 5.0], [6.0, 4.0, 2.0], [7.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 2.0, 4.0], [7.0, 8.0, 2.0], [2.0, 8.0, 5.0], [3.0, 7.0, 6.0]], [[3.0, 5.0, 6.0], 
[1.0, 2.0, 4.0], [7.0, 1.0, 2.0], [5.0, 4.0, 1.0]], [[2.0, 5.0, 7.0], [2.0, 6.0, 8.0], [4.0, 2.0, 3.0], [7.0, 7.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[5.0, 6.0, 3.0], [8.0, 8.0, 4.0], [5.0, 1.0, 3.0]])+(1.-msk_val)*numpy.array([[8.0, 5.0, 7.0], 
[7.0, 2.0, 5.0], [4.0, 5.0, 5.0]])
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 2.0], [8.0, 7.0, 2.0], [5.0, 6.0, 3.0], [6.0, 4.0, 6.0]], [[6.0, 7.0, 1.0], [1.0, 
5.0, 5.0], [8.0, 8.0, 4.0], [7.0, 7.0, 3.0]], [[8.0, 6.0, 1.0], [4.0, 3.0, 5.0], [5.0, 1.0, 3.0], [7.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 4.0], [7.0, 8.0, 2.0], [8.0, 5.0, 7.0], [3.0, 7.0, 6.0]], [[3.0, 5.0, 6.0], [1.0, 
2.0, 4.0], [7.0, 2.0, 5.0], [5.0, 4.0, 1.0]], [[2.0, 5.0, 7.0], [2.0, 6.0, 8.0], [4.0, 5.0, 5.0], [7.0, 7.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PPN(self):
      arg=Data(numpy.array([[[1.0, 1.0, 5.0], [5.0, 1.0, 5.0], [5.0, 8.0, 2.0], [3.0, 6.0, 7.0]], [[7.0, 1.0, 3.0], [3.0, 1.0, 
8.0], [6.0, 6.0, 7.0], [2.0, 4.0, 1.0]], [[1.0, 4.0, 7.0], [6.0, 7.0, 1.0], [4.0, 3.0, 6.0], [7.0, 8.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 7.0, 6.0], [6.0, 1.0, 1.0], [1.0, 7.0, 6.0], [4.0, 8.0, 4.0]], [[2.0, 1.0, 4.0], 
[3.0, 3.0, 5.0], [4.0, 4.0, 4.0], [8.0, 7.0, 5.0]], [[2.0, 2.0, 5.0], [6.0, 1.0, 2.0], [1.0, 8.0, 4.0], [1.0, 4.0, 5.0]]]))
      val=numpy.array([6.0, 4.0, 4.0])
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 5.0], [5.0, 1.0, 5.0], [5.0, 8.0, 2.0], [3.0, 6.0, 7.0]], [[7.0, 1.0, 3.0], [3.0, 
1.0, 8.0], [6.0, 4.0, 4.0], [2.0, 4.0, 1.0]], [[1.0, 4.0, 7.0], [6.0, 7.0, 1.0], [4.0, 3.0, 6.0], [7.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 6.0], [6.0, 1.0, 1.0], [1.0, 7.0, 6.0], [4.0, 8.0, 4.0]], [[2.0, 1.0, 4.0], [3.0, 
3.0, 5.0], [6.0, 4.0, 4.0], [8.0, 7.0, 5.0]], [[2.0, 2.0, 5.0], [6.0, 1.0, 2.0], [1.0, 8.0, 4.0], [1.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PPN(self):
      arg=Data(numpy.array([[[4.0, 8.0, 8.0], [8.0, 5.0, 1.0], [8.0, 6.0, 8.0], [8.0, 6.0, 5.0]], [[2.0, 5.0, 1.0], [1.0, 1.0, 
4.0], [2.0, 3.0, 7.0], [6.0, 2.0, 4.0]], [[5.0, 5.0, 1.0], [4.0, 8.0, 5.0], [3.0, 4.0, 4.0], [6.0, 5.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 5.0, 8.0], [8.0, 6.0, 5.0], [8.0, 1.0, 2.0], [6.0, 1.0, 2.0]], [[5.0, 1.0, 5.0], 
[2.0, 8.0, 7.0], [4.0, 5.0, 4.0], [1.0, 2.0, 5.0]], [[5.0, 6.0, 4.0], [8.0, 2.0, 4.0], [5.0, 7.0, 4.0], [6.0, 4.0, 2.0]]]))
      val=Data(numpy.array([8.0, 8.0, 7.0]),self.functionspace)
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 8.0], [8.0, 5.0, 1.0], [8.0, 6.0, 8.0], [8.0, 6.0, 5.0]], [[2.0, 5.0, 1.0], [1.0, 
1.0, 4.0], [8.0, 8.0, 7.0], [6.0, 2.0, 4.0]], [[5.0, 5.0, 1.0], [4.0, 8.0, 5.0], [3.0, 4.0, 4.0], [6.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 8.0], [8.0, 6.0, 5.0], [8.0, 1.0, 2.0], [6.0, 1.0, 2.0]], [[5.0, 1.0, 5.0], [2.0, 
8.0, 7.0], [8.0, 8.0, 7.0], [1.0, 2.0, 5.0]], [[5.0, 6.0, 4.0], [8.0, 2.0, 4.0], [5.0, 7.0, 4.0], [6.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PPN(self):
      arg=Data(numpy.array([[[4.0, 5.0, 5.0], [8.0, 8.0, 5.0], [2.0, 7.0, 8.0], [7.0, 8.0, 2.0]], [[2.0, 1.0, 7.0], [6.0, 3.0, 
6.0], [4.0, 1.0, 7.0], [1.0, 5.0, 3.0]], [[3.0, 8.0, 7.0], [4.0, 5.0, 8.0], [8.0, 1.0, 2.0], [3.0, 1.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 7.0, 6.0], [2.0, 2.0, 6.0], [5.0, 1.0, 7.0], [6.0, 5.0, 3.0]], [[8.0, 6.0, 4.0], 
[5.0, 6.0, 5.0], [5.0, 5.0, 3.0], [3.0, 6.0, 7.0]], [[8.0, 6.0, 1.0], [4.0, 4.0, 8.0], [7.0, 5.0, 8.0], [7.0, 1.0, 1.0]]]))
      val=Data(numpy.array([2.0, 2.0, 1.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 8.0, 8.0]))
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 5.0, 5.0], [8.0, 8.0, 5.0], [2.0, 7.0, 8.0], [7.0, 8.0, 2.0]], [[2.0, 1.0, 7.0], [6.0, 
3.0, 6.0], [2.0, 2.0, 1.0], [1.0, 5.0, 3.0]], [[3.0, 8.0, 7.0], [4.0, 5.0, 8.0], [8.0, 1.0, 2.0], [3.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 6.0], [2.0, 2.0, 6.0], [5.0, 1.0, 7.0], [6.0, 5.0, 3.0]], [[8.0, 6.0, 4.0], [5.0, 
6.0, 5.0], [3.0, 8.0, 8.0], [3.0, 6.0, 7.0]], [[8.0, 6.0, 1.0], [4.0, 4.0, 8.0], [7.0, 5.0, 8.0], [7.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PPN(self):
      arg=Data(numpy.array([[[8.0, 2.0, 1.0], [7.0, 6.0, 1.0], [3.0, 7.0, 3.0], [5.0, 1.0, 2.0]], [[4.0, 3.0, 1.0], [6.0, 3.0, 
5.0], [8.0, 3.0, 2.0], [4.0, 6.0, 4.0]], [[2.0, 5.0, 5.0], [8.0, 7.0, 8.0], [3.0, 1.0, 5.0], [6.0, 3.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 5.0, 8.0], [3.0, 4.0, 2.0], [6.0, 4.0, 5.0], [8.0, 3.0, 7.0]], [[1.0, 2.0, 7.0], 
[8.0, 4.0, 4.0], [6.0, 6.0, 2.0], [6.0, 8.0, 7.0]], [[6.0, 1.0, 8.0], [5.0, 1.0, 6.0], [5.0, 5.0, 3.0], [3.0, 5.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([3.0, 6.0, 2.0])+(1.-msk_val)*numpy.array([8.0, 7.0, 2.0])
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 1.0], [7.0, 6.0, 1.0], [3.0, 7.0, 3.0], [5.0, 1.0, 2.0]], [[4.0, 3.0, 1.0], [6.0, 
3.0, 5.0], [3.0, 6.0, 2.0], [4.0, 6.0, 4.0]], [[2.0, 5.0, 5.0], [8.0, 7.0, 8.0], [3.0, 1.0, 5.0], [6.0, 3.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 8.0], [3.0, 4.0, 2.0], [6.0, 4.0, 5.0], [8.0, 3.0, 7.0]], [[1.0, 2.0, 7.0], [8.0, 
4.0, 4.0], [8.0, 7.0, 2.0], [6.0, 8.0, 7.0]], [[6.0, 1.0, 8.0], [5.0, 1.0, 6.0], [5.0, 5.0, 3.0], [3.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CCN(self):
      arg=Data(numpy.array([[[6.0, 8.0, 7.0], [4.0, 6.0, 4.0], [6.0, 2.0, 5.0], [4.0, 7.0, 1.0]], [[8.0, 3.0, 7.0], [2.0, 1.0, 
5.0], [1.0, 7.0, 4.0], [6.0, 8.0, 3.0]], [[7.0, 4.0, 1.0], [8.0, 3.0, 8.0], [7.0, 2.0, 8.0], [7.0, 4.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 8.0, 5.0], [4.0, 6.0, 5.0], [1.0, 3.0, 1.0], [4.0, 8.0, 3.0]], [[1.0, 5.0, 1.0], 
[1.0, 2.0, 7.0], [1.0, 8.0, 5.0], [6.0, 1.0, 4.0]], [[2.0, 4.0, 6.0], [1.0, 1.0, 3.0], [4.0, 8.0, 4.0], [3.0, 5.0, 3.0]]]))
      val=numpy.array([[[7.0, 1.0, 5.0], [6.0, 6.0, 1.0], [6.0, 7.0, 8.0], [3.0, 3.0, 2.0]], [[3.0, 6.0, 5.0], [4.0, 6.0, 3.0], 
[7.0, 7.0, 5.0], [4.0, 4.0, 5.0]], [[7.0, 6.0, 5.0], [7.0, 4.0, 4.0], [3.0, 4.0, 8.0], [7.0, 7.0, 2.0]]])
      arg[:,:]=val
      ref=Data(numpy.array([[[7.0, 1.0, 5.0], [6.0, 6.0, 1.0], [6.0, 7.0, 8.0], [3.0, 3.0, 2.0]], [[3.0, 6.0, 5.0], [4.0, 6.0, 
3.0], [7.0, 7.0, 5.0], [4.0, 4.0, 5.0]], [[7.0, 6.0, 5.0], [7.0, 4.0, 4.0], [3.0, 4.0, 8.0], [7.0, 7.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CCN(self):
      arg=Data(numpy.array([[[6.0, 1.0, 3.0], [6.0, 2.0, 5.0], [4.0, 4.0, 8.0], [2.0, 6.0, 1.0]], [[5.0, 3.0, 8.0], [5.0, 8.0, 
2.0], [3.0, 3.0, 2.0], [2.0, 1.0, 1.0]], [[6.0, 4.0, 8.0], [2.0, 8.0, 2.0], [8.0, 8.0, 7.0], [6.0, 6.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 8.0, 3.0], [2.0, 3.0, 5.0], [1.0, 4.0, 8.0], [6.0, 2.0, 6.0]], [[7.0, 2.0, 3.0], 
[7.0, 4.0, 4.0], [5.0, 4.0, 1.0], [2.0, 7.0, 3.0]], [[3.0, 2.0, 6.0], [5.0, 2.0, 8.0], [7.0, 1.0, 3.0], [3.0, 4.0, 8.0]]]))
      val=Data(numpy.array([[[1.0, 4.0, 8.0], [6.0, 5.0, 4.0], [8.0, 7.0, 1.0], [4.0, 5.0, 1.0]], [[6.0, 7.0, 3.0], [6.0, 5.0, 
2.0], [3.0, 5.0, 8.0], [6.0, 5.0, 1.0]], [[2.0, 6.0, 8.0], [3.0, 3.0, 1.0], [3.0, 2.0, 4.0], [7.0, 3.0, 
2.0]]]),self.functionspace)
      arg[:,:]=val
      ref=Data(numpy.array([[[1.0, 4.0, 8.0], [6.0, 5.0, 4.0], [8.0, 7.0, 1.0], [4.0, 5.0, 1.0]], [[6.0, 7.0, 3.0], [6.0, 5.0, 
2.0], [3.0, 5.0, 8.0], [6.0, 5.0, 1.0]], [[2.0, 6.0, 8.0], [3.0, 3.0, 1.0], [3.0, 2.0, 4.0], [7.0, 3.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CCN(self):
      arg=Data(numpy.array([[[6.0, 2.0, 1.0], [3.0, 8.0, 1.0], [1.0, 6.0, 6.0], [3.0, 4.0, 3.0]], [[2.0, 1.0, 4.0], [3.0, 4.0, 
7.0], [1.0, 2.0, 1.0], [7.0, 8.0, 7.0]], [[5.0, 8.0, 3.0], [3.0, 1.0, 7.0], [8.0, 5.0, 6.0], [6.0, 2.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 6.0], [2.0, 2.0, 7.0], [5.0, 7.0, 3.0], [7.0, 2.0, 7.0]], [[8.0, 7.0, 8.0], 
[8.0, 1.0, 2.0], [5.0, 5.0, 1.0], [7.0, 1.0, 6.0]], [[8.0, 1.0, 3.0], [2.0, 2.0, 8.0], [6.0, 8.0, 8.0], [4.0, 8.0, 3.0]]]))
      val=Data(numpy.array([[[6.0, 6.0, 8.0], [6.0, 4.0, 5.0], [3.0, 1.0, 2.0], [7.0, 8.0, 5.0]], [[6.0, 3.0, 4.0], [7.0, 1.0, 
7.0], [1.0, 6.0, 2.0], [2.0, 7.0, 5.0]], [[2.0, 4.0, 4.0], [1.0, 5.0, 5.0], [1.0, 8.0, 5.0], [6.0, 8.0, 
6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 4.0, 2.0], [7.0, 4.0, 6.0], [2.0, 5.0, 8.0], [1.0, 8.0, 5.0]], [[5.0, 3.0, 5.0], 
[3.0, 2.0, 7.0], [5.0, 7.0, 7.0], [7.0, 4.0, 2.0]], [[6.0, 2.0, 7.0], [3.0, 7.0, 6.0], [2.0, 6.0, 3.0], [6.0, 6.0, 1.0]]]))
      arg[:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 8.0], [6.0, 4.0, 5.0], [3.0, 1.0, 2.0], [7.0, 8.0, 5.0]], [[6.0, 3.0, 4.0], [7.0, 
1.0, 7.0], [1.0, 6.0, 2.0], [2.0, 7.0, 5.0]], [[2.0, 4.0, 4.0], [1.0, 5.0, 5.0], [1.0, 8.0, 5.0], [6.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 2.0], [7.0, 4.0, 6.0], [2.0, 5.0, 8.0], [1.0, 8.0, 5.0]], [[5.0, 3.0, 5.0], [3.0, 
2.0, 7.0], [5.0, 7.0, 7.0], [7.0, 4.0, 2.0]], [[6.0, 2.0, 7.0], [3.0, 7.0, 6.0], [2.0, 6.0, 3.0], [6.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CCN(self):
      arg=Data(numpy.array([[[7.0, 2.0, 4.0], [1.0, 2.0, 1.0], [1.0, 6.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 2.0, 6.0], [6.0, 8.0, 
5.0], [2.0, 1.0, 2.0], [4.0, 3.0, 3.0]], [[7.0, 3.0, 2.0], [7.0, 8.0, 8.0], [5.0, 1.0, 2.0], [1.0, 4.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 4.0, 2.0], [1.0, 3.0, 2.0], [7.0, 7.0, 5.0], [1.0, 5.0, 6.0]], [[2.0, 4.0, 3.0], 
[4.0, 4.0, 4.0], [4.0, 2.0, 4.0], [3.0, 7.0, 2.0]], [[5.0, 8.0, 6.0], [7.0, 8.0, 3.0], [1.0, 2.0, 2.0], [5.0, 5.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 6.0, 3.0], [1.0, 2.0, 5.0], [8.0, 4.0, 7.0], [2.0, 8.0, 2.0]], [[6.0, 1.0, 1.0], [4.0, 
4.0, 8.0], [8.0, 6.0, 8.0], [7.0, 7.0, 8.0]], [[1.0, 6.0, 2.0], [8.0, 7.0, 8.0], [4.0, 8.0, 5.0], [6.0, 8.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[1.0, 2.0, 1.0], [4.0, 5.0, 8.0], [2.0, 4.0, 2.0], [6.0, 5.0, 8.0]], [[6.0, 6.0, 7.0], [4.0, 
2.0, 6.0], [3.0, 5.0, 5.0], [8.0, 2.0, 6.0]], [[6.0, 2.0, 4.0], [4.0, 3.0, 8.0], [4.0, 6.0, 4.0], [3.0, 6.0, 3.0]]])
      arg[:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 3.0], [1.0, 2.0, 5.0], [8.0, 4.0, 7.0], [2.0, 8.0, 2.0]], [[6.0, 1.0, 1.0], [4.0, 
4.0, 8.0], [8.0, 6.0, 8.0], [7.0, 7.0, 8.0]], [[1.0, 6.0, 2.0], [8.0, 7.0, 8.0], [4.0, 8.0, 5.0], [6.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 1.0], [4.0, 5.0, 8.0], [2.0, 4.0, 2.0], [6.0, 5.0, 8.0]], [[6.0, 6.0, 7.0], [4.0, 
2.0, 6.0], [3.0, 5.0, 5.0], [8.0, 2.0, 6.0]], [[6.0, 2.0, 4.0], [4.0, 3.0, 8.0], [4.0, 6.0, 4.0], [3.0, 6.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PCP(self):
      arg=Data(numpy.array([[[3.0, 6.0, 2.0], [6.0, 8.0, 3.0], [4.0, 4.0, 2.0], [3.0, 4.0, 3.0]], [[2.0, 6.0, 5.0], [6.0, 8.0, 
2.0], [3.0, 3.0, 1.0], [7.0, 2.0, 4.0]], [[4.0, 7.0, 7.0], [8.0, 7.0, 5.0], [7.0, 1.0, 5.0], [1.0, 7.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 8.0, 8.0], [8.0, 8.0, 8.0], [6.0, 2.0, 8.0], [3.0, 2.0, 2.0]], [[4.0, 7.0, 2.0], 
[5.0, 6.0, 7.0], [8.0, 2.0, 8.0], [6.0, 7.0, 2.0]], [[8.0, 2.0, 8.0], [8.0, 2.0, 2.0], [7.0, 6.0, 6.0], [5.0, 2.0, 8.0]]]))
      val=numpy.array([8.0, 6.0, 6.0, 2.0])
      arg[1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 2.0], [6.0, 8.0, 3.0], [4.0, 4.0, 2.0], [3.0, 4.0, 3.0]], [[2.0, 8.0, 5.0], [6.0, 
6.0, 2.0], [3.0, 6.0, 1.0], [7.0, 2.0, 4.0]], [[4.0, 7.0, 7.0], [8.0, 7.0, 5.0], [7.0, 1.0, 5.0], [1.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 8.0, 8.0], [8.0, 8.0, 8.0], [6.0, 2.0, 8.0], [3.0, 2.0, 2.0]], [[4.0, 8.0, 2.0], [5.0, 
6.0, 7.0], [8.0, 6.0, 8.0], [6.0, 2.0, 2.0]], [[8.0, 2.0, 8.0], [8.0, 2.0, 2.0], [7.0, 6.0, 6.0], [5.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PCP(self):
      arg=Data(numpy.array([[[1.0, 8.0, 6.0], [8.0, 3.0, 8.0], [4.0, 8.0, 4.0], [4.0, 5.0, 2.0]], [[5.0, 6.0, 1.0], [4.0, 4.0, 
1.0], [5.0, 7.0, 4.0], [7.0, 2.0, 4.0]], [[5.0, 2.0, 5.0], [1.0, 7.0, 3.0], [7.0, 1.0, 5.0], [8.0, 4.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 1.0, 1.0], [1.0, 7.0, 1.0], [5.0, 8.0, 7.0], [6.0, 6.0, 6.0]], [[3.0, 8.0, 7.0], 
[5.0, 2.0, 2.0], [7.0, 2.0, 3.0], [1.0, 1.0, 5.0]], [[7.0, 5.0, 3.0], [2.0, 4.0, 7.0], [1.0, 3.0, 8.0], [4.0, 8.0, 7.0]]]))
      val=Data(numpy.array([4.0, 2.0, 2.0, 8.0]),self.functionspace)
      arg[1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 6.0], [8.0, 3.0, 8.0], [4.0, 8.0, 4.0], [4.0, 5.0, 2.0]], [[5.0, 4.0, 1.0], [4.0, 
2.0, 1.0], [5.0, 2.0, 4.0], [7.0, 8.0, 4.0]], [[5.0, 2.0, 5.0], [1.0, 7.0, 3.0], [7.0, 1.0, 5.0], [8.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 1.0], [1.0, 7.0, 1.0], [5.0, 8.0, 7.0], [6.0, 6.0, 6.0]], [[3.0, 4.0, 7.0], [5.0, 
2.0, 2.0], [7.0, 2.0, 3.0], [1.0, 8.0, 5.0]], [[7.0, 5.0, 3.0], [2.0, 4.0, 7.0], [1.0, 3.0, 8.0], [4.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PCP(self):
      arg=Data(numpy.array([[[2.0, 6.0, 8.0], [5.0, 7.0, 6.0], [7.0, 4.0, 6.0], [4.0, 5.0, 3.0]], [[6.0, 8.0, 5.0], [1.0, 2.0, 
2.0], [7.0, 3.0, 4.0], [4.0, 3.0, 5.0]], [[1.0, 3.0, 5.0], [1.0, 8.0, 4.0], [4.0, 2.0, 8.0], [2.0, 2.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 7.0, 6.0], [4.0, 2.0, 4.0], [8.0, 3.0, 2.0], [7.0, 2.0, 8.0]], [[2.0, 7.0, 1.0], 
[6.0, 4.0, 4.0], [8.0, 8.0, 7.0], [7.0, 1.0, 5.0]], [[6.0, 7.0, 4.0], [8.0, 5.0, 2.0], [4.0, 3.0, 4.0], [3.0, 1.0, 4.0]]]))
      val=Data(numpy.array([4.0, 5.0, 3.0, 7.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([6.0, 5.0, 3.0, 5.0]))
      arg[1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 6.0, 8.0], [5.0, 7.0, 6.0], [7.0, 4.0, 6.0], [4.0, 5.0, 3.0]], [[6.0, 4.0, 5.0], [1.0, 
5.0, 2.0], [7.0, 3.0, 4.0], [4.0, 7.0, 5.0]], [[1.0, 3.0, 5.0], [1.0, 8.0, 4.0], [4.0, 2.0, 8.0], [2.0, 2.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 6.0], [4.0, 2.0, 4.0], [8.0, 3.0, 2.0], [7.0, 2.0, 8.0]], [[2.0, 6.0, 1.0], [6.0, 
5.0, 4.0], [8.0, 3.0, 7.0], [7.0, 5.0, 5.0]], [[6.0, 7.0, 4.0], [8.0, 5.0, 2.0], [4.0, 3.0, 4.0], [3.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PCP(self):
      arg=Data(numpy.array([[[1.0, 4.0, 5.0], [6.0, 8.0, 1.0], [2.0, 3.0, 5.0], [7.0, 4.0, 8.0]], [[5.0, 5.0, 3.0], [1.0, 6.0, 
8.0], [2.0, 4.0, 7.0], [7.0, 1.0, 6.0]], [[8.0, 3.0, 2.0], [2.0, 2.0, 4.0], [6.0, 5.0, 6.0], [5.0, 6.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 4.0, 7.0], [2.0, 6.0, 3.0], [8.0, 4.0, 1.0], [8.0, 3.0, 4.0]], [[2.0, 8.0, 6.0], 
[6.0, 4.0, 5.0], [7.0, 8.0, 5.0], [4.0, 6.0, 5.0]], [[7.0, 1.0, 3.0], [5.0, 5.0, 4.0], [5.0, 1.0, 2.0], [5.0, 5.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([1.0, 1.0, 3.0, 2.0])+(1.-msk_val)*numpy.array([1.0, 3.0, 3.0, 6.0])
      arg[1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 4.0, 5.0], [6.0, 8.0, 1.0], [2.0, 3.0, 5.0], [7.0, 4.0, 8.0]], [[5.0, 1.0, 3.0], [1.0, 
1.0, 8.0], [2.0, 3.0, 7.0], [7.0, 2.0, 6.0]], [[8.0, 3.0, 2.0], [2.0, 2.0, 4.0], [6.0, 5.0, 6.0], [5.0, 6.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 7.0], [2.0, 6.0, 3.0], [8.0, 4.0, 1.0], [8.0, 3.0, 4.0]], [[2.0, 1.0, 6.0], [6.0, 
3.0, 5.0], [7.0, 3.0, 5.0], [4.0, 6.0, 5.0]], [[7.0, 1.0, 3.0], [5.0, 5.0, 4.0], [5.0, 1.0, 2.0], [5.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_ULN(self):
      arg=Data(numpy.array([[[3.0, 8.0, 8.0], [6.0, 7.0, 4.0], [3.0, 8.0, 7.0], [3.0, 1.0, 6.0]], [[8.0, 1.0, 6.0], [1.0, 7.0, 
5.0], [6.0, 5.0, 1.0], [6.0, 3.0, 4.0]], [[7.0, 4.0, 8.0], [7.0, 2.0, 5.0], [4.0, 2.0, 1.0], [6.0, 3.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 8.0, 7.0], [2.0, 6.0, 7.0], [2.0, 7.0, 7.0], [8.0, 8.0, 5.0]], [[4.0, 1.0, 6.0], 
[3.0, 6.0, 8.0], [7.0, 6.0, 3.0], [6.0, 4.0, 2.0]], [[2.0, 1.0, 4.0], [4.0, 4.0, 6.0], [1.0, 3.0, 4.0], [8.0, 3.0, 3.0]]]))
      val=numpy.array([[[5.0, 3.0, 3.0], [7.0, 2.0, 4.0]], [[1.0, 4.0, 8.0], [1.0, 8.0, 3.0]]])
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 8.0], [6.0, 7.0, 4.0], [5.0, 3.0, 3.0], [7.0, 2.0, 4.0]], [[8.0, 1.0, 6.0], [1.0, 
7.0, 5.0], [1.0, 4.0, 8.0], [1.0, 8.0, 3.0]], [[7.0, 4.0, 8.0], [7.0, 2.0, 5.0], [4.0, 2.0, 1.0], [6.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 7.0], [2.0, 6.0, 7.0], [5.0, 3.0, 3.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 6.0], [3.0, 
6.0, 8.0], [1.0, 4.0, 8.0], [1.0, 8.0, 3.0]], [[2.0, 1.0, 4.0], [4.0, 4.0, 6.0], [1.0, 3.0, 4.0], [8.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_ULN(self):
      arg=Data(numpy.array([[[8.0, 2.0, 8.0], [5.0, 7.0, 7.0], [2.0, 1.0, 1.0], [7.0, 2.0, 1.0]], [[3.0, 5.0, 7.0], [2.0, 5.0, 
4.0], [8.0, 7.0, 3.0], [1.0, 7.0, 4.0]], [[3.0, 2.0, 2.0], [6.0, 6.0, 4.0], [2.0, 2.0, 2.0], [6.0, 3.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 7.0], [4.0, 7.0, 1.0], [2.0, 5.0, 4.0], [6.0, 7.0, 5.0]], [[4.0, 1.0, 5.0], 
[4.0, 5.0, 2.0], [7.0, 4.0, 7.0], [2.0, 3.0, 7.0]], [[4.0, 8.0, 5.0], [5.0, 2.0, 5.0], [1.0, 2.0, 3.0], [3.0, 7.0, 7.0]]]))
      val=Data(numpy.array([[[6.0, 8.0, 1.0], [8.0, 4.0, 5.0]], [[4.0, 2.0, 8.0], [8.0, 4.0, 6.0]]]),self.functionspace)
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 8.0], [5.0, 7.0, 7.0], [6.0, 8.0, 1.0], [8.0, 4.0, 5.0]], [[3.0, 5.0, 7.0], [2.0, 
5.0, 4.0], [4.0, 2.0, 8.0], [8.0, 4.0, 6.0]], [[3.0, 2.0, 2.0], [6.0, 6.0, 4.0], [2.0, 2.0, 2.0], [6.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 7.0], [4.0, 7.0, 1.0], [6.0, 8.0, 1.0], [8.0, 4.0, 5.0]], [[4.0, 1.0, 5.0], [4.0, 
5.0, 2.0], [4.0, 2.0, 8.0], [8.0, 4.0, 6.0]], [[4.0, 8.0, 5.0], [5.0, 2.0, 5.0], [1.0, 2.0, 3.0], [3.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_ULN(self):
      arg=Data(numpy.array([[[5.0, 2.0, 6.0], [1.0, 7.0, 4.0], [7.0, 6.0, 7.0], [8.0, 6.0, 3.0]], [[7.0, 2.0, 2.0], [1.0, 4.0, 
6.0], [2.0, 8.0, 4.0], [4.0, 6.0, 2.0]], [[7.0, 1.0, 1.0], [2.0, 4.0, 8.0], [7.0, 5.0, 7.0], [2.0, 6.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 1.0, 4.0], [5.0, 7.0, 7.0], [7.0, 1.0, 2.0], [3.0, 6.0, 2.0]], [[2.0, 1.0, 8.0], 
[8.0, 3.0, 6.0], [3.0, 6.0, 1.0], [2.0, 3.0, 1.0]], [[7.0, 8.0, 4.0], [5.0, 8.0, 5.0], [7.0, 8.0, 7.0], [2.0, 6.0, 7.0]]]))
      val=Data(numpy.array([[[1.0, 3.0, 8.0], [4.0, 3.0, 4.0]], [[1.0, 7.0, 4.0], [5.0, 4.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 4.0, 6.0], [8.0, 6.0, 1.0]], [[2.0, 2.0, 8.0], [6.0, 8.0, 1.0]]]))
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 6.0], [1.0, 7.0, 4.0], [1.0, 3.0, 8.0], [4.0, 3.0, 4.0]], [[7.0, 2.0, 2.0], [1.0, 
4.0, 6.0], [1.0, 7.0, 4.0], [5.0, 4.0, 8.0]], [[7.0, 1.0, 1.0], [2.0, 4.0, 8.0], [7.0, 5.0, 7.0], [2.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 4.0], [5.0, 7.0, 7.0], [2.0, 4.0, 6.0], [8.0, 6.0, 1.0]], [[2.0, 1.0, 8.0], [8.0, 
3.0, 6.0], [2.0, 2.0, 8.0], [6.0, 8.0, 1.0]], [[7.0, 8.0, 4.0], [5.0, 8.0, 5.0], [7.0, 8.0, 7.0], [2.0, 6.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_ULN(self):
      arg=Data(numpy.array([[[3.0, 3.0, 5.0], [6.0, 7.0, 5.0], [4.0, 8.0, 7.0], [8.0, 7.0, 8.0]], [[3.0, 1.0, 5.0], [7.0, 3.0, 
2.0], [3.0, 2.0, 2.0], [4.0, 1.0, 7.0]], [[1.0, 8.0, 4.0], [8.0, 1.0, 4.0], [7.0, 2.0, 6.0], [7.0, 3.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 2.0, 4.0], [3.0, 2.0, 4.0], [6.0, 1.0, 8.0], [1.0, 6.0, 4.0]], [[1.0, 3.0, 4.0], 
[4.0, 5.0, 1.0], [3.0, 7.0, 5.0], [4.0, 2.0, 8.0]], [[3.0, 7.0, 4.0], [1.0, 8.0, 4.0], [8.0, 5.0, 8.0], [3.0, 4.0, 7.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 5.0, 8.0], [7.0, 5.0, 4.0]], [[7.0, 6.0, 2.0], [1.0, 2.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[4.0, 8.0, 5.0], [4.0, 7.0, 6.0]], [[5.0, 2.0, 8.0], [4.0, 1.0, 5.0]]])
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 5.0], [6.0, 7.0, 5.0], [5.0, 5.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 1.0, 5.0], [7.0, 
3.0, 2.0], [7.0, 6.0, 2.0], [1.0, 2.0, 4.0]], [[1.0, 8.0, 4.0], [8.0, 1.0, 4.0], [7.0, 2.0, 6.0], [7.0, 3.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 4.0], [3.0, 2.0, 4.0], [4.0, 8.0, 5.0], [4.0, 7.0, 6.0]], [[1.0, 3.0, 4.0], [4.0, 
5.0, 1.0], [5.0, 2.0, 8.0], [4.0, 1.0, 5.0]], [[3.0, 7.0, 4.0], [1.0, 8.0, 4.0], [8.0, 5.0, 8.0], [3.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UPN(self):
      arg=Data(numpy.array([[[6.0, 4.0, 6.0], [3.0, 6.0, 5.0], [4.0, 4.0, 1.0], [7.0, 2.0, 1.0]], [[8.0, 5.0, 4.0], [6.0, 5.0, 
5.0], [3.0, 5.0, 8.0], [1.0, 8.0, 2.0]], [[1.0, 4.0, 8.0], [1.0, 1.0, 2.0], [7.0, 8.0, 3.0], [6.0, 1.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 2.0, 3.0], [3.0, 5.0, 6.0], [3.0, 1.0, 2.0], [7.0, 8.0, 2.0]], [[2.0, 3.0, 8.0], 
[4.0, 8.0, 5.0], [1.0, 8.0, 7.0], [7.0, 6.0, 4.0]], [[4.0, 7.0, 3.0], [8.0, 4.0, 5.0], [1.0, 4.0, 6.0], [4.0, 3.0, 8.0]]]))
      val=numpy.array([[8.0, 5.0, 1.0], [1.0, 4.0, 6.0]])
      arg[:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 6.0], [3.0, 6.0, 5.0], [8.0, 5.0, 1.0], [7.0, 2.0, 1.0]], [[8.0, 5.0, 4.0], [6.0, 
5.0, 5.0], [1.0, 4.0, 6.0], [1.0, 8.0, 2.0]], [[1.0, 4.0, 8.0], [1.0, 1.0, 2.0], [7.0, 8.0, 3.0], [6.0, 1.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 3.0], [3.0, 5.0, 6.0], [8.0, 5.0, 1.0], [7.0, 8.0, 2.0]], [[2.0, 3.0, 8.0], [4.0, 
8.0, 5.0], [1.0, 4.0, 6.0], [7.0, 6.0, 4.0]], [[4.0, 7.0, 3.0], [8.0, 4.0, 5.0], [1.0, 4.0, 6.0], [4.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UPN(self):
      arg=Data(numpy.array([[[2.0, 7.0, 3.0], [1.0, 3.0, 2.0], [5.0, 6.0, 4.0], [3.0, 2.0, 5.0]], [[7.0, 1.0, 1.0], [6.0, 3.0, 
6.0], [7.0, 1.0, 7.0], [4.0, 6.0, 8.0]], [[7.0, 6.0, 7.0], [8.0, 1.0, 3.0], [2.0, 7.0, 8.0], [7.0, 3.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 8.0, 3.0], [7.0, 1.0, 2.0], [6.0, 6.0, 4.0], [1.0, 5.0, 4.0]], [[3.0, 1.0, 4.0], 
[8.0, 6.0, 7.0], [5.0, 4.0, 3.0], [2.0, 6.0, 8.0]], [[2.0, 5.0, 5.0], [5.0, 2.0, 2.0], [2.0, 4.0, 4.0], [8.0, 8.0, 8.0]]]))
      val=Data(numpy.array([[2.0, 5.0, 5.0], [2.0, 4.0, 4.0]]),self.functionspace)
      arg[:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 3.0], [1.0, 3.0, 2.0], [2.0, 5.0, 5.0], [3.0, 2.0, 5.0]], [[7.0, 1.0, 1.0], [6.0, 
3.0, 6.0], [2.0, 4.0, 4.0], [4.0, 6.0, 8.0]], [[7.0, 6.0, 7.0], [8.0, 1.0, 3.0], [2.0, 7.0, 8.0], [7.0, 3.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 3.0], [7.0, 1.0, 2.0], [2.0, 5.0, 5.0], [1.0, 5.0, 4.0]], [[3.0, 1.0, 4.0], [8.0, 
6.0, 7.0], [2.0, 4.0, 4.0], [2.0, 6.0, 8.0]], [[2.0, 5.0, 5.0], [5.0, 2.0, 2.0], [2.0, 4.0, 4.0], [8.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UPN(self):
      arg=Data(numpy.array([[[4.0, 6.0, 3.0], [8.0, 7.0, 7.0], [6.0, 3.0, 5.0], [2.0, 4.0, 2.0]], [[7.0, 7.0, 8.0], [2.0, 8.0, 
4.0], [8.0, 6.0, 3.0], [6.0, 8.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 3.0, 8.0], [8.0, 1.0, 3.0], [4.0, 6.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 5.0, 1.0], [7.0, 5.0, 4.0], [2.0, 3.0, 1.0], [2.0, 2.0, 7.0]], [[3.0, 2.0, 7.0], 
[6.0, 3.0, 8.0], [5.0, 5.0, 1.0], [6.0, 8.0, 7.0]], [[5.0, 2.0, 1.0], [8.0, 2.0, 6.0], [4.0, 5.0, 1.0], [7.0, 4.0, 5.0]]]))
      val=Data(numpy.array([[7.0, 1.0, 4.0], [5.0, 2.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 3.0, 3.0], [3.0, 3.0, 1.0]]))
      arg[:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 3.0], [8.0, 7.0, 7.0], [7.0, 1.0, 4.0], [2.0, 4.0, 2.0]], [[7.0, 7.0, 8.0], [2.0, 
8.0, 4.0], [5.0, 2.0, 1.0], [6.0, 8.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 3.0, 8.0], [8.0, 1.0, 3.0], [4.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 1.0], [7.0, 5.0, 4.0], [5.0, 3.0, 3.0], [2.0, 2.0, 7.0]], [[3.0, 2.0, 7.0], [6.0, 
3.0, 8.0], [3.0, 3.0, 1.0], [6.0, 8.0, 7.0]], [[5.0, 2.0, 1.0], [8.0, 2.0, 6.0], [4.0, 5.0, 1.0], [7.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UPN(self):
      arg=Data(numpy.array([[[1.0, 7.0, 5.0], [7.0, 1.0, 5.0], [7.0, 5.0, 2.0], [4.0, 7.0, 7.0]], [[3.0, 2.0, 7.0], [4.0, 3.0, 
4.0], [7.0, 8.0, 5.0], [1.0, 2.0, 4.0]], [[1.0, 1.0, 2.0], [3.0, 1.0, 6.0], [7.0, 2.0, 3.0], [8.0, 5.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 8.0, 2.0], [8.0, 3.0, 4.0], [7.0, 1.0, 3.0], [5.0, 6.0, 3.0]], [[6.0, 1.0, 1.0], 
[1.0, 3.0, 5.0], [1.0, 3.0, 7.0], [8.0, 3.0, 3.0]], [[8.0, 5.0, 3.0], [1.0, 1.0, 3.0], [5.0, 2.0, 3.0], [2.0, 6.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[3.0, 7.0, 5.0], [6.0, 1.0, 2.0]])+(1.-msk_val)*numpy.array([[3.0, 1.0, 1.0], [1.0, 6.0, 7.0]])
      arg[:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 5.0], [7.0, 1.0, 5.0], [3.0, 7.0, 5.0], [4.0, 7.0, 7.0]], [[3.0, 2.0, 7.0], [4.0, 
3.0, 4.0], [6.0, 1.0, 2.0], [1.0, 2.0, 4.0]], [[1.0, 1.0, 2.0], [3.0, 1.0, 6.0], [7.0, 2.0, 3.0], [8.0, 5.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 2.0], [8.0, 3.0, 4.0], [3.0, 1.0, 1.0], [5.0, 6.0, 3.0]], [[6.0, 1.0, 1.0], [1.0, 
3.0, 5.0], [1.0, 6.0, 7.0], [8.0, 3.0, 3.0]], [[8.0, 5.0, 3.0], [1.0, 1.0, 3.0], [5.0, 2.0, 3.0], [2.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RRC(self):
      arg=Data(numpy.array([[[6.0, 5.0, 8.0], [4.0, 2.0, 2.0], [5.0, 6.0, 3.0], [8.0, 3.0, 4.0]], [[5.0, 4.0, 8.0], [8.0, 2.0, 
1.0], [7.0, 4.0, 7.0], [1.0, 6.0, 4.0]], [[3.0, 3.0, 3.0], [5.0, 8.0, 8.0], [5.0, 8.0, 5.0], [3.0, 7.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 8.0, 4.0], [1.0, 4.0, 6.0], [7.0, 2.0, 2.0], [1.0, 6.0, 7.0]], [[4.0, 7.0, 6.0], 
[8.0, 8.0, 8.0], [6.0, 4.0, 5.0], [5.0, 8.0, 6.0]], [[7.0, 6.0, 4.0], [5.0, 7.0, 5.0], [2.0, 6.0, 5.0], [1.0, 3.0, 3.0]]]))
      val=numpy.array([[[3.0, 8.0, 8.0], [7.0, 2.0, 2.0]]])
      arg[1:2,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 8.0], [4.0, 2.0, 2.0], [5.0, 6.0, 3.0], [8.0, 3.0, 4.0]], [[5.0, 4.0, 8.0], [3.0, 
8.0, 8.0], [7.0, 2.0, 2.0], [1.0, 6.0, 4.0]], [[3.0, 3.0, 3.0], [5.0, 8.0, 8.0], [5.0, 8.0, 5.0], [3.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 4.0], [1.0, 4.0, 6.0], [7.0, 2.0, 2.0], [1.0, 6.0, 7.0]], [[4.0, 7.0, 6.0], [3.0, 
8.0, 8.0], [7.0, 2.0, 2.0], [5.0, 8.0, 6.0]], [[7.0, 6.0, 4.0], [5.0, 7.0, 5.0], [2.0, 6.0, 5.0], [1.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RRC(self):
      arg=Data(numpy.array([[[1.0, 2.0, 4.0], [1.0, 5.0, 7.0], [6.0, 4.0, 4.0], [3.0, 6.0, 2.0]], [[3.0, 2.0, 8.0], [7.0, 6.0, 
8.0], [5.0, 2.0, 6.0], [1.0, 6.0, 7.0]], [[3.0, 3.0, 8.0], [3.0, 1.0, 8.0], [2.0, 2.0, 2.0], [4.0, 8.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 4.0, 4.0], [5.0, 4.0, 3.0], [7.0, 6.0, 3.0], [2.0, 7.0, 6.0]], [[1.0, 3.0, 7.0], 
[7.0, 7.0, 6.0], [8.0, 7.0, 7.0], [8.0, 8.0, 7.0]], [[2.0, 6.0, 8.0], [5.0, 1.0, 7.0], [3.0, 4.0, 5.0], [5.0, 8.0, 2.0]]]))
      val=Data(numpy.array([[[5.0, 5.0, 5.0], [1.0, 1.0, 1.0]]]),self.functionspace)
      arg[1:2,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 4.0], [1.0, 5.0, 7.0], [6.0, 4.0, 4.0], [3.0, 6.0, 2.0]], [[3.0, 2.0, 8.0], [5.0, 
5.0, 5.0], [1.0, 1.0, 1.0], [1.0, 6.0, 7.0]], [[3.0, 3.0, 8.0], [3.0, 1.0, 8.0], [2.0, 2.0, 2.0], [4.0, 8.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 4.0], [5.0, 4.0, 3.0], [7.0, 6.0, 3.0], [2.0, 7.0, 6.0]], [[1.0, 3.0, 7.0], [5.0, 
5.0, 5.0], [1.0, 1.0, 1.0], [8.0, 8.0, 7.0]], [[2.0, 6.0, 8.0], [5.0, 1.0, 7.0], [3.0, 4.0, 5.0], [5.0, 8.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RRC(self):
      arg=Data(numpy.array([[[8.0, 3.0, 1.0], [7.0, 5.0, 4.0], [7.0, 2.0, 5.0], [4.0, 6.0, 2.0]], [[1.0, 4.0, 2.0], [1.0, 7.0, 
1.0], [8.0, 3.0, 1.0], [2.0, 1.0, 2.0]], [[1.0, 7.0, 7.0], [5.0, 7.0, 1.0], [1.0, 4.0, 7.0], [4.0, 4.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 1.0, 7.0], [5.0, 2.0, 2.0], [2.0, 7.0, 2.0], [1.0, 5.0, 2.0]], [[4.0, 1.0, 4.0], 
[6.0, 4.0, 5.0], [6.0, 4.0, 1.0], [1.0, 2.0, 8.0]], [[4.0, 1.0, 4.0], [6.0, 6.0, 7.0], [1.0, 4.0, 1.0], [4.0, 3.0, 4.0]]]))
      val=Data(numpy.array([[[6.0, 7.0, 4.0], [4.0, 1.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 7.0, 4.0], [2.0, 8.0, 1.0]]]))
      arg[1:2,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 1.0], [7.0, 5.0, 4.0], [7.0, 2.0, 5.0], [4.0, 6.0, 2.0]], [[1.0, 4.0, 2.0], [6.0, 
7.0, 4.0], [4.0, 1.0, 2.0], [2.0, 1.0, 2.0]], [[1.0, 7.0, 7.0], [5.0, 7.0, 1.0], [1.0, 4.0, 7.0], [4.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 7.0], [5.0, 2.0, 2.0], [2.0, 7.0, 2.0], [1.0, 5.0, 2.0]], [[4.0, 1.0, 4.0], [1.0, 
7.0, 4.0], [2.0, 8.0, 1.0], [1.0, 2.0, 8.0]], [[4.0, 1.0, 4.0], [6.0, 6.0, 7.0], [1.0, 4.0, 1.0], [4.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RRC(self):
      arg=Data(numpy.array([[[7.0, 3.0, 3.0], [8.0, 5.0, 7.0], [5.0, 5.0, 7.0], [8.0, 2.0, 5.0]], [[3.0, 1.0, 1.0], [6.0, 4.0, 
3.0], [7.0, 5.0, 6.0], [8.0, 4.0, 4.0]], [[1.0, 4.0, 1.0], [5.0, 7.0, 4.0], [8.0, 4.0, 6.0], [5.0, 6.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 4.0, 1.0], [7.0, 2.0, 8.0], [8.0, 8.0, 1.0], [6.0, 4.0, 7.0]], [[7.0, 3.0, 6.0], 
[4.0, 8.0, 4.0], [3.0, 4.0, 1.0], [5.0, 5.0, 3.0]], [[2.0, 1.0, 8.0], [6.0, 6.0, 4.0], [7.0, 4.0, 1.0], [5.0, 8.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 1.0, 8.0], [2.0, 4.0, 3.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0, 1.0], [5.0, 2.0, 
4.0]]])
      arg[1:2,1:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 3.0], [8.0, 5.0, 7.0], [5.0, 5.0, 7.0], [8.0, 2.0, 5.0]], [[3.0, 1.0, 1.0], [4.0, 
1.0, 8.0], [2.0, 4.0, 3.0], [8.0, 4.0, 4.0]], [[1.0, 4.0, 1.0], [5.0, 7.0, 4.0], [8.0, 4.0, 6.0], [5.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 4.0, 1.0], [7.0, 2.0, 8.0], [8.0, 8.0, 1.0], [6.0, 4.0, 7.0]], [[7.0, 3.0, 6.0], [8.0, 
1.0, 1.0], [5.0, 2.0, 4.0], [5.0, 5.0, 3.0]], [[2.0, 1.0, 8.0], [6.0, 6.0, 4.0], [7.0, 4.0, 1.0], [5.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LUC(self):
      arg=Data(numpy.array([[[7.0, 6.0, 2.0], [6.0, 7.0, 4.0], [4.0, 4.0, 8.0], [4.0, 1.0, 4.0]], [[6.0, 5.0, 1.0], [6.0, 1.0, 
8.0], [7.0, 2.0, 8.0], [1.0, 1.0, 4.0]], [[1.0, 8.0, 7.0], [2.0, 1.0, 5.0], [3.0, 6.0, 5.0], [4.0, 4.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 7.0, 5.0], [4.0, 1.0, 2.0], [8.0, 3.0, 4.0], [1.0, 3.0, 3.0]], [[2.0, 5.0, 7.0], 
[7.0, 6.0, 5.0], [1.0, 8.0, 2.0], [8.0, 1.0, 2.0]], [[2.0, 6.0, 7.0], [6.0, 8.0, 6.0], [2.0, 8.0, 2.0], [3.0, 1.0, 1.0]]]))
      val=numpy.array([[[5.0, 7.0, 6.0], [6.0, 4.0, 2.0], [1.0, 7.0, 1.0]]])
      arg[2:,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 2.0], [6.0, 7.0, 4.0], [4.0, 4.0, 8.0], [4.0, 1.0, 4.0]], [[6.0, 5.0, 1.0], [6.0, 
1.0, 8.0], [7.0, 2.0, 8.0], [1.0, 1.0, 4.0]], [[5.0, 7.0, 6.0], [6.0, 4.0, 2.0], [1.0, 7.0, 1.0], [4.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 5.0], [4.0, 1.0, 2.0], [8.0, 3.0, 4.0], [1.0, 3.0, 3.0]], [[2.0, 5.0, 7.0], [7.0, 
6.0, 5.0], [1.0, 8.0, 2.0], [8.0, 1.0, 2.0]], [[5.0, 7.0, 6.0], [6.0, 4.0, 2.0], [1.0, 7.0, 1.0], [3.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LUC(self):
      arg=Data(numpy.array([[[6.0, 1.0, 2.0], [8.0, 7.0, 1.0], [5.0, 8.0, 4.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 5.0], [5.0, 6.0, 
7.0], [8.0, 2.0, 5.0], [5.0, 8.0, 6.0]], [[5.0, 5.0, 8.0], [3.0, 6.0, 7.0], [7.0, 1.0, 3.0], [6.0, 6.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 6.0, 2.0], [4.0, 7.0, 2.0], [1.0, 6.0, 5.0], [4.0, 2.0, 4.0]], [[6.0, 3.0, 5.0], 
[3.0, 5.0, 1.0], [2.0, 4.0, 2.0], [5.0, 7.0, 7.0]], [[3.0, 1.0, 4.0], [5.0, 4.0, 4.0], [6.0, 5.0, 7.0], [3.0, 6.0, 2.0]]]))
      val=Data(numpy.array([[[5.0, 4.0, 4.0], [2.0, 2.0, 6.0], [4.0, 1.0, 6.0]]]),self.functionspace)
      arg[2:,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 2.0], [8.0, 7.0, 1.0], [5.0, 8.0, 4.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 5.0], [5.0, 
6.0, 7.0], [8.0, 2.0, 5.0], [5.0, 8.0, 6.0]], [[5.0, 4.0, 4.0], [2.0, 2.0, 6.0], [4.0, 1.0, 6.0], [6.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 6.0, 2.0], [4.0, 7.0, 2.0], [1.0, 6.0, 5.0], [4.0, 2.0, 4.0]], [[6.0, 3.0, 5.0], [3.0, 
5.0, 1.0], [2.0, 4.0, 2.0], [5.0, 7.0, 7.0]], [[5.0, 4.0, 4.0], [2.0, 2.0, 6.0], [4.0, 1.0, 6.0], [3.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LUC(self):
      arg=Data(numpy.array([[[4.0, 7.0, 6.0], [7.0, 5.0, 7.0], [6.0, 5.0, 5.0], [5.0, 2.0, 6.0]], [[3.0, 1.0, 7.0], [8.0, 1.0, 
3.0], [3.0, 4.0, 2.0], [7.0, 2.0, 6.0]], [[8.0, 8.0, 4.0], [8.0, 3.0, 2.0], [6.0, 8.0, 8.0], [4.0, 3.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 2.0, 7.0], [4.0, 6.0, 5.0], [2.0, 3.0, 4.0], [8.0, 5.0, 2.0]], [[2.0, 7.0, 3.0], 
[5.0, 4.0, 2.0], [2.0, 3.0, 5.0], [5.0, 5.0, 1.0]], [[2.0, 8.0, 8.0], [2.0, 2.0, 5.0], [4.0, 6.0, 5.0], [1.0, 6.0, 4.0]]]))
      val=Data(numpy.array([[[4.0, 4.0, 5.0], [7.0, 5.0, 5.0], [7.0, 4.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 8.0, 3.0], [6.0, 8.0, 6.0], [2.0, 8.0, 4.0]]]))
      arg[2:,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 7.0, 6.0], [7.0, 5.0, 7.0], [6.0, 5.0, 5.0], [5.0, 2.0, 6.0]], [[3.0, 1.0, 7.0], [8.0, 
1.0, 3.0], [3.0, 4.0, 2.0], [7.0, 2.0, 6.0]], [[4.0, 4.0, 5.0], [7.0, 5.0, 5.0], [7.0, 4.0, 7.0], [4.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 7.0], [4.0, 6.0, 5.0], [2.0, 3.0, 4.0], [8.0, 5.0, 2.0]], [[2.0, 7.0, 3.0], [5.0, 
4.0, 2.0], [2.0, 3.0, 5.0], [5.0, 5.0, 1.0]], [[5.0, 8.0, 3.0], [6.0, 8.0, 6.0], [2.0, 8.0, 4.0], [1.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LUC(self):
      arg=Data(numpy.array([[[2.0, 5.0, 6.0], [3.0, 5.0, 7.0], [7.0, 1.0, 3.0], [7.0, 6.0, 4.0]], [[8.0, 4.0, 6.0], [7.0, 1.0, 
6.0], [4.0, 5.0, 8.0], [4.0, 2.0, 5.0]], [[8.0, 8.0, 2.0], [5.0, 5.0, 1.0], [7.0, 4.0, 5.0], [5.0, 1.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 2.0, 7.0], [5.0, 3.0, 7.0], [4.0, 4.0, 3.0], [4.0, 4.0, 7.0]], [[2.0, 6.0, 1.0], 
[4.0, 7.0, 1.0], [8.0, 1.0, 1.0], [5.0, 2.0, 6.0]], [[4.0, 4.0, 3.0], [8.0, 8.0, 5.0], [7.0, 5.0, 6.0], [8.0, 1.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 2.0, 3.0], [8.0, 1.0, 1.0], [7.0, 2.0, 7.0]]])+(1.-msk_val)*numpy.array([[[5.0, 2.0, 
5.0], [4.0, 3.0, 4.0], [4.0, 8.0, 8.0]]])
      arg[2:,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 6.0], [3.0, 5.0, 7.0], [7.0, 1.0, 3.0], [7.0, 6.0, 4.0]], [[8.0, 4.0, 6.0], [7.0, 
1.0, 6.0], [4.0, 5.0, 8.0], [4.0, 2.0, 5.0]], [[8.0, 2.0, 3.0], [8.0, 1.0, 1.0], [7.0, 2.0, 7.0], [5.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 7.0], [5.0, 3.0, 7.0], [4.0, 4.0, 3.0], [4.0, 4.0, 7.0]], [[2.0, 6.0, 1.0], [4.0, 
7.0, 1.0], [8.0, 1.0, 1.0], [5.0, 2.0, 6.0]], [[5.0, 2.0, 5.0], [4.0, 3.0, 4.0], [4.0, 8.0, 8.0], [8.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RCN(self):
      arg=Data(numpy.array([[[6.0, 3.0, 4.0], [6.0, 1.0, 7.0], [8.0, 6.0, 2.0], [6.0, 5.0, 7.0]], [[8.0, 4.0, 1.0], [7.0, 1.0, 
6.0], [4.0, 6.0, 7.0], [1.0, 7.0, 5.0]], [[7.0, 7.0, 7.0], [8.0, 3.0, 7.0], [4.0, 4.0, 8.0], [6.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 1.0, 6.0], [7.0, 5.0, 1.0], [7.0, 2.0, 7.0], [4.0, 6.0, 3.0]], [[7.0, 5.0, 6.0], 
[6.0, 7.0, 2.0], [2.0, 6.0, 5.0], [2.0, 4.0, 2.0]], [[7.0, 4.0, 2.0], [7.0, 7.0, 6.0], [4.0, 3.0, 6.0], [7.0, 7.0, 5.0]]]))
      val=numpy.array([[[1.0, 7.0, 8.0], [5.0, 6.0, 4.0], [7.0, 2.0, 1.0], [8.0, 4.0, 2.0]]])
      arg[1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 4.0], [6.0, 1.0, 7.0], [8.0, 6.0, 2.0], [6.0, 5.0, 7.0]], [[1.0, 7.0, 8.0], [5.0, 
6.0, 4.0], [7.0, 2.0, 1.0], [8.0, 4.0, 2.0]], [[7.0, 7.0, 7.0], [8.0, 3.0, 7.0], [4.0, 4.0, 8.0], [6.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 6.0], [7.0, 5.0, 1.0], [7.0, 2.0, 7.0], [4.0, 6.0, 3.0]], [[1.0, 7.0, 8.0], [5.0, 
6.0, 4.0], [7.0, 2.0, 1.0], [8.0, 4.0, 2.0]], [[7.0, 4.0, 2.0], [7.0, 7.0, 6.0], [4.0, 3.0, 6.0], [7.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RCN(self):
      arg=Data(numpy.array([[[6.0, 8.0, 6.0], [8.0, 3.0, 8.0], [7.0, 1.0, 4.0], [5.0, 2.0, 5.0]], [[7.0, 7.0, 5.0], [8.0, 1.0, 
1.0], [3.0, 5.0, 2.0], [5.0, 8.0, 1.0]], [[6.0, 2.0, 5.0], [4.0, 7.0, 8.0], [6.0, 7.0, 8.0], [3.0, 2.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 4.0, 4.0], [3.0, 7.0, 2.0], [5.0, 2.0, 5.0], [1.0, 8.0, 1.0]], [[6.0, 7.0, 4.0], 
[8.0, 6.0, 5.0], [4.0, 8.0, 3.0], [6.0, 5.0, 8.0]], [[3.0, 6.0, 6.0], [1.0, 1.0, 6.0], [7.0, 7.0, 5.0], [1.0, 2.0, 6.0]]]))
      val=Data(numpy.array([[[3.0, 4.0, 5.0], [1.0, 6.0, 6.0], [7.0, 2.0, 4.0], [5.0, 3.0, 1.0]]]),self.functionspace)
      arg[1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 6.0], [8.0, 3.0, 8.0], [7.0, 1.0, 4.0], [5.0, 2.0, 5.0]], [[3.0, 4.0, 5.0], [1.0, 
6.0, 6.0], [7.0, 2.0, 4.0], [5.0, 3.0, 1.0]], [[6.0, 2.0, 5.0], [4.0, 7.0, 8.0], [6.0, 7.0, 8.0], [3.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 4.0], [3.0, 7.0, 2.0], [5.0, 2.0, 5.0], [1.0, 8.0, 1.0]], [[3.0, 4.0, 5.0], [1.0, 
6.0, 6.0], [7.0, 2.0, 4.0], [5.0, 3.0, 1.0]], [[3.0, 6.0, 6.0], [1.0, 1.0, 6.0], [7.0, 7.0, 5.0], [1.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RCN(self):
      arg=Data(numpy.array([[[3.0, 6.0, 2.0], [4.0, 4.0, 2.0], [5.0, 2.0, 4.0], [8.0, 2.0, 7.0]], [[5.0, 7.0, 3.0], [2.0, 5.0, 
1.0], [8.0, 3.0, 5.0], [2.0, 5.0, 7.0]], [[4.0, 6.0, 8.0], [5.0, 6.0, 7.0], [5.0, 2.0, 2.0], [2.0, 1.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 8.0, 5.0], [2.0, 8.0, 6.0], [7.0, 1.0, 1.0], [3.0, 2.0, 2.0]], [[7.0, 6.0, 7.0], 
[3.0, 4.0, 4.0], [4.0, 7.0, 5.0], [2.0, 7.0, 3.0]], [[2.0, 4.0, 8.0], [6.0, 6.0, 2.0], [6.0, 3.0, 8.0], [4.0, 4.0, 3.0]]]))
      val=Data(numpy.array([[[4.0, 3.0, 5.0], [7.0, 5.0, 7.0], [8.0, 3.0, 5.0], [7.0, 5.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 7.0, 1.0], [6.0, 2.0, 1.0], [2.0, 3.0, 6.0], [8.0, 2.0, 2.0]]]))
      arg[1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 2.0], [4.0, 4.0, 2.0], [5.0, 2.0, 4.0], [8.0, 2.0, 7.0]], [[4.0, 3.0, 5.0], [7.0, 
5.0, 7.0], [8.0, 3.0, 5.0], [7.0, 5.0, 2.0]], [[4.0, 6.0, 8.0], [5.0, 6.0, 7.0], [5.0, 2.0, 2.0], [2.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 5.0], [2.0, 8.0, 6.0], [7.0, 1.0, 1.0], [3.0, 2.0, 2.0]], [[3.0, 7.0, 1.0], [6.0, 
2.0, 1.0], [2.0, 3.0, 6.0], [8.0, 2.0, 2.0]], [[2.0, 4.0, 8.0], [6.0, 6.0, 2.0], [6.0, 3.0, 8.0], [4.0, 4.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RCN(self):
      arg=Data(numpy.array([[[3.0, 4.0, 7.0], [1.0, 7.0, 8.0], [1.0, 8.0, 5.0], [1.0, 2.0, 7.0]], [[3.0, 7.0, 1.0], [3.0, 6.0, 
1.0], [4.0, 3.0, 5.0], [3.0, 4.0, 3.0]], [[8.0, 8.0, 2.0], [5.0, 2.0, 8.0], [1.0, 5.0, 5.0], [5.0, 8.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 3.0, 2.0], [7.0, 8.0, 8.0], [5.0, 8.0, 7.0], [3.0, 2.0, 4.0]], [[8.0, 5.0, 2.0], 
[6.0, 8.0, 5.0], [3.0, 4.0, 3.0], [1.0, 7.0, 7.0]], [[2.0, 1.0, 4.0], [7.0, 2.0, 6.0], [7.0, 8.0, 2.0], [8.0, 5.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 6.0, 1.0], [4.0, 3.0, 4.0], [3.0, 6.0, 1.0], [7.0, 6.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[5.0, 4.0, 2.0], [2.0, 1.0, 8.0], [2.0, 8.0, 1.0], [5.0, 3.0, 8.0]]])
      arg[1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 7.0], [1.0, 7.0, 8.0], [1.0, 8.0, 5.0], [1.0, 2.0, 7.0]], [[3.0, 6.0, 1.0], [4.0, 
3.0, 4.0], [3.0, 6.0, 1.0], [7.0, 6.0, 6.0]], [[8.0, 8.0, 2.0], [5.0, 2.0, 8.0], [1.0, 5.0, 5.0], [5.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 2.0], [7.0, 8.0, 8.0], [5.0, 8.0, 7.0], [3.0, 2.0, 4.0]], [[5.0, 4.0, 2.0], [2.0, 
1.0, 8.0], [2.0, 8.0, 1.0], [5.0, 3.0, 8.0]], [[2.0, 1.0, 4.0], [7.0, 2.0, 6.0], [7.0, 8.0, 2.0], [8.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UCN(self):
      arg=Data(numpy.array([[[7.0, 4.0, 5.0], [2.0, 6.0, 2.0], [5.0, 7.0, 3.0], [5.0, 8.0, 7.0]], [[3.0, 8.0, 5.0], [1.0, 2.0, 
5.0], [3.0, 5.0, 8.0], [5.0, 5.0, 2.0]], [[2.0, 1.0, 2.0], [2.0, 4.0, 5.0], [8.0, 6.0, 2.0], [8.0, 8.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 8.0, 5.0], [4.0, 3.0, 8.0], [4.0, 5.0, 3.0], [5.0, 1.0, 8.0]], [[6.0, 6.0, 6.0], 
[8.0, 5.0, 7.0], [5.0, 7.0, 4.0], [7.0, 5.0, 6.0]], [[1.0, 5.0, 6.0], [5.0, 1.0, 5.0], [6.0, 8.0, 5.0], [6.0, 3.0, 5.0]]]))
      val=numpy.array([[[2.0, 5.0, 5.0], [3.0, 5.0, 4.0], [2.0, 4.0, 8.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 6.0], [8.0, 3.0, 5.0], 
[2.0, 3.0, 8.0], [6.0, 1.0, 8.0]]])
      arg[:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 5.0], [3.0, 5.0, 4.0], [2.0, 4.0, 8.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 6.0], [8.0, 
3.0, 5.0], [2.0, 3.0, 8.0], [6.0, 1.0, 8.0]], [[2.0, 1.0, 2.0], [2.0, 4.0, 5.0], [8.0, 6.0, 2.0], [8.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 5.0], [3.0, 5.0, 4.0], [2.0, 4.0, 8.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 6.0], [8.0, 
3.0, 5.0], [2.0, 3.0, 8.0], [6.0, 1.0, 8.0]], [[1.0, 5.0, 6.0], [5.0, 1.0, 5.0], [6.0, 8.0, 5.0], [6.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UCN(self):
      arg=Data(numpy.array([[[1.0, 5.0, 8.0], [4.0, 5.0, 7.0], [4.0, 7.0, 6.0], [2.0, 3.0, 8.0]], [[4.0, 2.0, 6.0], [6.0, 3.0, 
1.0], [4.0, 1.0, 2.0], [2.0, 7.0, 8.0]], [[6.0, 2.0, 1.0], [7.0, 2.0, 7.0], [4.0, 7.0, 5.0], [4.0, 8.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 3.0, 1.0], [5.0, 7.0, 4.0], [2.0, 6.0, 4.0], [8.0, 7.0, 4.0]], [[5.0, 6.0, 5.0], 
[1.0, 7.0, 3.0], [8.0, 7.0, 7.0], [3.0, 3.0, 3.0]], [[1.0, 4.0, 3.0], [8.0, 1.0, 4.0], [2.0, 7.0, 2.0], [1.0, 3.0, 4.0]]]))
      val=Data(numpy.array([[[2.0, 7.0, 7.0], [3.0, 2.0, 4.0], [5.0, 4.0, 4.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 5.0], [4.0, 2.0, 
2.0], [5.0, 2.0, 1.0], [5.0, 6.0, 4.0]]]),self.functionspace)
      arg[:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 7.0], [3.0, 2.0, 4.0], [5.0, 4.0, 4.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 5.0], [4.0, 
2.0, 2.0], [5.0, 2.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 2.0, 1.0], [7.0, 2.0, 7.0], [4.0, 7.0, 5.0], [4.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 7.0], [3.0, 2.0, 4.0], [5.0, 4.0, 4.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 5.0], [4.0, 
2.0, 2.0], [5.0, 2.0, 1.0], [5.0, 6.0, 4.0]], [[1.0, 4.0, 3.0], [8.0, 1.0, 4.0], [2.0, 7.0, 2.0], [1.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UCN(self):
      arg=Data(numpy.array([[[8.0, 4.0, 4.0], [3.0, 5.0, 3.0], [5.0, 5.0, 7.0], [5.0, 8.0, 4.0]], [[8.0, 8.0, 2.0], [6.0, 1.0, 
7.0], [2.0, 3.0, 3.0], [6.0, 3.0, 4.0]], [[8.0, 6.0, 4.0], [3.0, 7.0, 8.0], [3.0, 4.0, 6.0], [2.0, 6.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 4.0, 8.0], [3.0, 7.0, 4.0], [2.0, 6.0, 8.0], [2.0, 7.0, 8.0]], [[5.0, 5.0, 3.0], 
[5.0, 8.0, 8.0], [2.0, 8.0, 3.0], [7.0, 4.0, 1.0]], [[6.0, 4.0, 7.0], [8.0, 4.0, 2.0], [2.0, 2.0, 3.0], [6.0, 8.0, 8.0]]]))
      val=Data(numpy.array([[[5.0, 2.0, 4.0], [6.0, 6.0, 6.0], [5.0, 2.0, 3.0], [3.0, 1.0, 8.0]], [[2.0, 8.0, 3.0], [2.0, 4.0, 
4.0], [8.0, 6.0, 8.0], [7.0, 7.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 1.0, 7.0], [1.0, 6.0, 4.0], [2.0, 7.0, 2.0], [1.0, 5.0, 6.0]], [[4.0, 1.0, 2.0], 
[7.0, 8.0, 6.0], [4.0, 5.0, 1.0], [4.0, 2.0, 6.0]]]))
      arg[:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 4.0], [6.0, 6.0, 6.0], [5.0, 2.0, 3.0], [3.0, 1.0, 8.0]], [[2.0, 8.0, 3.0], [2.0, 
4.0, 4.0], [8.0, 6.0, 8.0], [7.0, 7.0, 4.0]], [[8.0, 6.0, 4.0], [3.0, 7.0, 8.0], [3.0, 4.0, 6.0], [2.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 7.0], [1.0, 6.0, 4.0], [2.0, 7.0, 2.0], [1.0, 5.0, 6.0]], [[4.0, 1.0, 2.0], [7.0, 
8.0, 6.0], [4.0, 5.0, 1.0], [4.0, 2.0, 6.0]], [[6.0, 4.0, 7.0], [8.0, 4.0, 2.0], [2.0, 2.0, 3.0], [6.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UCN(self):
      arg=Data(numpy.array([[[6.0, 6.0, 8.0], [5.0, 1.0, 8.0], [5.0, 8.0, 3.0], [8.0, 3.0, 5.0]], [[4.0, 7.0, 4.0], [6.0, 3.0, 
7.0], [6.0, 8.0, 7.0], [6.0, 1.0, 8.0]], [[5.0, 3.0, 5.0], [6.0, 2.0, 8.0], [6.0, 3.0, 6.0], [8.0, 6.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 5.0, 5.0], [7.0, 5.0, 5.0], [4.0, 1.0, 8.0], [6.0, 2.0, 2.0]], [[6.0, 1.0, 5.0], 
[3.0, 2.0, 4.0], [2.0, 3.0, 1.0], [6.0, 3.0, 3.0]], [[6.0, 7.0, 4.0], [2.0, 1.0, 6.0], [7.0, 1.0, 5.0], [5.0, 2.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 7.0, 4.0], [5.0, 8.0, 4.0], [6.0, 2.0, 2.0], [5.0, 6.0, 4.0]], [[7.0, 2.0, 2.0], [2.0, 
8.0, 7.0], [2.0, 4.0, 5.0], [1.0, 3.0, 8.0]]])+(1.-msk_val)*numpy.array([[[2.0, 1.0, 3.0], [2.0, 7.0, 3.0], [4.0, 1.0, 5.0], 
[1.0, 4.0, 5.0]], [[5.0, 7.0, 6.0], [4.0, 2.0, 5.0], [1.0, 7.0, 7.0], [4.0, 7.0, 5.0]]])
      arg[:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 4.0], [5.0, 8.0, 4.0], [6.0, 2.0, 2.0], [5.0, 6.0, 4.0]], [[7.0, 2.0, 2.0], [2.0, 
8.0, 7.0], [2.0, 4.0, 5.0], [1.0, 3.0, 8.0]], [[5.0, 3.0, 5.0], [6.0, 2.0, 8.0], [6.0, 3.0, 6.0], [8.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 3.0], [2.0, 7.0, 3.0], [4.0, 1.0, 5.0], [1.0, 4.0, 5.0]], [[5.0, 7.0, 6.0], [4.0, 
2.0, 5.0], [1.0, 7.0, 7.0], [4.0, 7.0, 5.0]], [[6.0, 7.0, 4.0], [2.0, 1.0, 6.0], [7.0, 1.0, 5.0], [5.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RPN(self):
      arg=Data(numpy.array([[[4.0, 4.0, 1.0], [5.0, 5.0, 3.0], [3.0, 8.0, 4.0], [6.0, 3.0, 6.0]], [[7.0, 5.0, 8.0], [7.0, 1.0, 
5.0], [2.0, 6.0, 3.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 6.0], [6.0, 3.0, 7.0], [2.0, 5.0, 5.0], [6.0, 5.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 4.0, 2.0], [8.0, 4.0, 5.0], [1.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[8.0, 1.0, 2.0], 
[5.0, 5.0, 2.0], [2.0, 2.0, 8.0], [7.0, 2.0, 6.0]], [[6.0, 6.0, 3.0], [5.0, 8.0, 6.0], [5.0, 5.0, 3.0], [2.0, 3.0, 2.0]]]))
      val=numpy.array([[2.0, 8.0, 1.0]])
      arg[1:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 1.0], [5.0, 5.0, 3.0], [3.0, 8.0, 4.0], [6.0, 3.0, 6.0]], [[7.0, 5.0, 8.0], [7.0, 
1.0, 5.0], [2.0, 8.0, 1.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 6.0], [6.0, 3.0, 7.0], [2.0, 5.0, 5.0], [6.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 2.0], [8.0, 4.0, 5.0], [1.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[8.0, 1.0, 2.0], [5.0, 
5.0, 2.0], [2.0, 8.0, 1.0], [7.0, 2.0, 6.0]], [[6.0, 6.0, 3.0], [5.0, 8.0, 6.0], [5.0, 5.0, 3.0], [2.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RPN(self):
      arg=Data(numpy.array([[[2.0, 7.0, 2.0], [7.0, 6.0, 2.0], [1.0, 1.0, 1.0], [2.0, 6.0, 8.0]], [[2.0, 6.0, 6.0], [6.0, 3.0, 
3.0], [4.0, 1.0, 6.0], [4.0, 8.0, 5.0]], [[1.0, 3.0, 1.0], [2.0, 6.0, 8.0], [6.0, 8.0, 7.0], [5.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 4.0, 4.0], [3.0, 1.0, 2.0], [1.0, 3.0, 6.0], [7.0, 8.0, 8.0]], [[8.0, 4.0, 8.0], 
[3.0, 6.0, 5.0], [2.0, 3.0, 5.0], [8.0, 5.0, 3.0]], [[4.0, 7.0, 1.0], [8.0, 1.0, 4.0], [7.0, 3.0, 5.0], [2.0, 1.0, 7.0]]]))
      val=Data(numpy.array([[8.0, 6.0, 7.0]]),self.functionspace)
      arg[1:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 2.0], [7.0, 6.0, 2.0], [1.0, 1.0, 1.0], [2.0, 6.0, 8.0]], [[2.0, 6.0, 6.0], [6.0, 
3.0, 3.0], [8.0, 6.0, 7.0], [4.0, 8.0, 5.0]], [[1.0, 3.0, 1.0], [2.0, 6.0, 8.0], [6.0, 8.0, 7.0], [5.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 4.0, 4.0], [3.0, 1.0, 2.0], [1.0, 3.0, 6.0], [7.0, 8.0, 8.0]], [[8.0, 4.0, 8.0], [3.0, 
6.0, 5.0], [8.0, 6.0, 7.0], [8.0, 5.0, 3.0]], [[4.0, 7.0, 1.0], [8.0, 1.0, 4.0], [7.0, 3.0, 5.0], [2.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RPN(self):
      arg=Data(numpy.array([[[5.0, 2.0, 6.0], [6.0, 4.0, 7.0], [5.0, 5.0, 6.0], [2.0, 3.0, 1.0]], [[6.0, 7.0, 6.0], [7.0, 5.0, 
8.0], [6.0, 3.0, 2.0], [1.0, 4.0, 7.0]], [[5.0, 6.0, 4.0], [2.0, 3.0, 1.0], [4.0, 8.0, 3.0], [2.0, 6.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 2.0, 6.0], [5.0, 7.0, 7.0], [1.0, 8.0, 1.0], [6.0, 4.0, 8.0]], [[2.0, 2.0, 2.0], 
[4.0, 2.0, 3.0], [6.0, 8.0, 3.0], [4.0, 5.0, 2.0]], [[5.0, 7.0, 2.0], [4.0, 7.0, 5.0], [8.0, 8.0, 4.0], [8.0, 1.0, 1.0]]]))
      val=Data(numpy.array([[5.0, 3.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 8.0, 8.0]]))
      arg[1:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 6.0], [6.0, 4.0, 7.0], [5.0, 5.0, 6.0], [2.0, 3.0, 1.0]], [[6.0, 7.0, 6.0], [7.0, 
5.0, 8.0], [5.0, 3.0, 2.0], [1.0, 4.0, 7.0]], [[5.0, 6.0, 4.0], [2.0, 3.0, 1.0], [4.0, 8.0, 3.0], [2.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 6.0], [5.0, 7.0, 7.0], [1.0, 8.0, 1.0], [6.0, 4.0, 8.0]], [[2.0, 2.0, 2.0], [4.0, 
2.0, 3.0], [7.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[5.0, 7.0, 2.0], [4.0, 7.0, 5.0], [8.0, 8.0, 4.0], [8.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RPN(self):
      arg=Data(numpy.array([[[6.0, 8.0, 6.0], [3.0, 6.0, 3.0], [1.0, 3.0, 6.0], [1.0, 2.0, 3.0]], [[4.0, 4.0, 7.0], [1.0, 3.0, 
4.0], [1.0, 6.0, 1.0], [8.0, 6.0, 2.0]], [[1.0, 7.0, 2.0], [7.0, 4.0, 2.0], [6.0, 2.0, 5.0], [8.0, 6.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 7.0, 4.0], [6.0, 3.0, 8.0], [1.0, 7.0, 4.0], [3.0, 6.0, 3.0]], [[3.0, 8.0, 2.0], 
[6.0, 7.0, 4.0], [8.0, 2.0, 1.0], [8.0, 1.0, 5.0]], [[8.0, 7.0, 5.0], [3.0, 6.0, 4.0], [7.0, 1.0, 7.0], [2.0, 7.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[3.0, 2.0, 7.0]])+(1.-msk_val)*numpy.array([[7.0, 1.0, 2.0]])
      arg[1:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 6.0], [3.0, 6.0, 3.0], [1.0, 3.0, 6.0], [1.0, 2.0, 3.0]], [[4.0, 4.0, 7.0], [1.0, 
3.0, 4.0], [3.0, 2.0, 7.0], [8.0, 6.0, 2.0]], [[1.0, 7.0, 2.0], [7.0, 4.0, 2.0], [6.0, 2.0, 5.0], [8.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 4.0], [6.0, 3.0, 8.0], [1.0, 7.0, 4.0], [3.0, 6.0, 3.0]], [[3.0, 8.0, 2.0], [6.0, 
7.0, 4.0], [7.0, 1.0, 2.0], [8.0, 1.0, 5.0]], [[8.0, 7.0, 5.0], [3.0, 6.0, 4.0], [7.0, 1.0, 7.0], [2.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PLN(self):
      arg=Data(numpy.array([[[2.0, 7.0, 3.0], [5.0, 2.0, 2.0], [2.0, 8.0, 8.0], [8.0, 2.0, 2.0]], [[5.0, 2.0, 7.0], [6.0, 6.0, 
3.0], [2.0, 8.0, 1.0], [7.0, 2.0, 3.0]], [[3.0, 4.0, 6.0], [5.0, 3.0, 3.0], [8.0, 1.0, 6.0], [3.0, 1.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 3.0, 5.0], [1.0, 6.0, 4.0], [7.0, 4.0, 8.0], [8.0, 7.0, 4.0]], [[4.0, 5.0, 2.0], 
[7.0, 1.0, 2.0], [5.0, 4.0, 5.0], [5.0, 5.0, 8.0]], [[4.0, 8.0, 7.0], [5.0, 8.0, 3.0], [5.0, 5.0, 6.0], [5.0, 1.0, 4.0]]]))
      val=numpy.array([[5.0, 1.0, 7.0], [2.0, 2.0, 3.0]])
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 3.0], [5.0, 2.0, 2.0], [2.0, 8.0, 8.0], [8.0, 2.0, 2.0]], [[5.0, 2.0, 7.0], [6.0, 
6.0, 3.0], [5.0, 1.0, 7.0], [2.0, 2.0, 3.0]], [[3.0, 4.0, 6.0], [5.0, 3.0, 3.0], [8.0, 1.0, 6.0], [3.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 5.0], [1.0, 6.0, 4.0], [7.0, 4.0, 8.0], [8.0, 7.0, 4.0]], [[4.0, 5.0, 2.0], [7.0, 
1.0, 2.0], [5.0, 1.0, 7.0], [2.0, 2.0, 3.0]], [[4.0, 8.0, 7.0], [5.0, 8.0, 3.0], [5.0, 5.0, 6.0], [5.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PLN(self):
      arg=Data(numpy.array([[[7.0, 3.0, 2.0], [3.0, 3.0, 6.0], [6.0, 2.0, 5.0], [6.0, 6.0, 4.0]], [[5.0, 7.0, 3.0], [4.0, 2.0, 
4.0], [2.0, 2.0, 7.0], [8.0, 3.0, 4.0]], [[1.0, 2.0, 8.0], [7.0, 6.0, 8.0], [3.0, 6.0, 6.0], [4.0, 1.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 5.0, 5.0], [4.0, 4.0, 2.0], [7.0, 4.0, 6.0], [7.0, 5.0, 8.0]], [[7.0, 7.0, 4.0], 
[2.0, 7.0, 5.0], [7.0, 1.0, 4.0], [6.0, 7.0, 1.0]], [[2.0, 1.0, 2.0], [8.0, 6.0, 4.0], [2.0, 1.0, 2.0], [7.0, 3.0, 8.0]]]))
      val=Data(numpy.array([[6.0, 1.0, 5.0], [7.0, 4.0, 8.0]]),self.functionspace)
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 2.0], [3.0, 3.0, 6.0], [6.0, 2.0, 5.0], [6.0, 6.0, 4.0]], [[5.0, 7.0, 3.0], [4.0, 
2.0, 4.0], [6.0, 1.0, 5.0], [7.0, 4.0, 8.0]], [[1.0, 2.0, 8.0], [7.0, 6.0, 8.0], [3.0, 6.0, 6.0], [4.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 5.0], [4.0, 4.0, 2.0], [7.0, 4.0, 6.0], [7.0, 5.0, 8.0]], [[7.0, 7.0, 4.0], [2.0, 
7.0, 5.0], [6.0, 1.0, 5.0], [7.0, 4.0, 8.0]], [[2.0, 1.0, 2.0], [8.0, 6.0, 4.0], [2.0, 1.0, 2.0], [7.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PLN(self):
      arg=Data(numpy.array([[[8.0, 7.0, 8.0], [8.0, 7.0, 4.0], [6.0, 2.0, 3.0], [3.0, 2.0, 3.0]], [[6.0, 3.0, 5.0], [5.0, 3.0, 
4.0], [5.0, 7.0, 1.0], [4.0, 3.0, 1.0]], [[7.0, 2.0, 2.0], [4.0, 6.0, 3.0], [2.0, 2.0, 5.0], [7.0, 4.0, 
8.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 8.0, 3.0], [5.0, 2.0, 3.0], [7.0, 1.0, 1.0], [2.0, 3.0, 6.0]], [[1.0, 3.0, 1.0], 
[5.0, 8.0, 6.0], [3.0, 2.0, 8.0], [8.0, 5.0, 6.0]], [[1.0, 1.0, 2.0], [1.0, 8.0, 3.0], [4.0, 2.0, 1.0], [2.0, 4.0, 6.0]]]))
      val=Data(numpy.array([[1.0, 4.0, 8.0], [4.0, 5.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 5.0, 6.0], [8.0, 1.0, 5.0]]))
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 7.0, 8.0], [8.0, 7.0, 4.0], [6.0, 2.0, 3.0], [3.0, 2.0, 3.0]], [[6.0, 3.0, 5.0], [5.0, 
3.0, 4.0], [1.0, 4.0, 8.0], [4.0, 5.0, 1.0]], [[7.0, 2.0, 2.0], [4.0, 6.0, 3.0], [2.0, 2.0, 5.0], [7.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 3.0], [5.0, 2.0, 3.0], [7.0, 1.0, 1.0], [2.0, 3.0, 6.0]], [[1.0, 3.0, 1.0], [5.0, 
8.0, 6.0], [1.0, 5.0, 6.0], [8.0, 1.0, 5.0]], [[1.0, 1.0, 2.0], [1.0, 8.0, 3.0], [4.0, 2.0, 1.0], [2.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PLN(self):
      arg=Data(numpy.array([[[1.0, 6.0, 8.0], [2.0, 5.0, 8.0], [2.0, 3.0, 2.0], [2.0, 7.0, 7.0]], [[8.0, 3.0, 5.0], [7.0, 1.0, 
1.0], [7.0, 3.0, 8.0], [5.0, 6.0, 2.0]], [[3.0, 1.0, 2.0], [5.0, 2.0, 5.0], [2.0, 5.0, 2.0], [1.0, 6.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 5.0], [8.0, 6.0, 6.0], [8.0, 1.0, 6.0], [3.0, 1.0, 7.0]], [[5.0, 4.0, 6.0], 
[3.0, 8.0, 1.0], [7.0, 3.0, 5.0], [8.0, 5.0, 6.0]], [[7.0, 1.0, 5.0], [4.0, 2.0, 4.0], [7.0, 7.0, 7.0], [2.0, 5.0, 1.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 8.0, 2.0], [2.0, 1.0, 8.0]])+(1.-msk_val)*numpy.array([[1.0, 5.0, 8.0], [7.0, 3.0, 6.0]])
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 8.0], [2.0, 5.0, 8.0], [2.0, 3.0, 2.0], [2.0, 7.0, 7.0]], [[8.0, 3.0, 5.0], [7.0, 
1.0, 1.0], [6.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[3.0, 1.0, 2.0], [5.0, 2.0, 5.0], [2.0, 5.0, 2.0], [1.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 5.0], [8.0, 6.0, 6.0], [8.0, 1.0, 6.0], [3.0, 1.0, 7.0]], [[5.0, 4.0, 6.0], [3.0, 
8.0, 1.0], [1.0, 5.0, 8.0], [7.0, 3.0, 6.0]], [[7.0, 1.0, 5.0], [4.0, 2.0, 4.0], [7.0, 7.0, 7.0], [2.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CUR(self):
      arg=Data(numpy.array([[[8.0, 8.0, 8.0], [7.0, 3.0, 4.0], [8.0, 5.0, 2.0], [3.0, 3.0, 3.0]], [[4.0, 2.0, 1.0], [3.0, 1.0, 
4.0], [2.0, 1.0, 5.0], [2.0, 1.0, 2.0]], [[7.0, 5.0, 7.0], [2.0, 7.0, 7.0], [1.0, 4.0, 2.0], [1.0, 1.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 2.0, 7.0], [2.0, 2.0, 4.0], [7.0, 7.0, 6.0], [5.0, 7.0, 4.0]], [[7.0, 6.0, 8.0], 
[3.0, 4.0, 1.0], [4.0, 6.0, 2.0], [6.0, 3.0, 6.0]], [[6.0, 4.0, 4.0], [1.0, 8.0, 5.0], [4.0, 4.0, 4.0], [1.0, 2.0, 4.0]]]))
      val=numpy.array([[[4.0], [8.0], [7.0]], [[1.0], [5.0], [8.0]], [[1.0], [5.0], [1.0]]])
      arg[:,:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 8.0], [7.0, 8.0, 4.0], [8.0, 7.0, 2.0], [3.0, 3.0, 3.0]], [[4.0, 1.0, 1.0], [3.0, 
5.0, 4.0], [2.0, 8.0, 5.0], [2.0, 1.0, 2.0]], [[7.0, 1.0, 7.0], [2.0, 5.0, 7.0], [1.0, 1.0, 2.0], [1.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 7.0], [2.0, 8.0, 4.0], [7.0, 7.0, 6.0], [5.0, 7.0, 4.0]], [[7.0, 1.0, 8.0], [3.0, 
5.0, 1.0], [4.0, 8.0, 2.0], [6.0, 3.0, 6.0]], [[6.0, 1.0, 4.0], [1.0, 5.0, 5.0], [4.0, 1.0, 4.0], [1.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CUR(self):
      arg=Data(numpy.array([[[4.0, 3.0, 3.0], [4.0, 1.0, 7.0], [1.0, 1.0, 4.0], [8.0, 6.0, 7.0]], [[4.0, 6.0, 1.0], [5.0, 1.0, 
1.0], [1.0, 3.0, 6.0], [7.0, 2.0, 2.0]], [[1.0, 3.0, 6.0], [5.0, 4.0, 8.0], [1.0, 7.0, 2.0], [4.0, 5.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 8.0, 7.0], [8.0, 7.0, 3.0], [1.0, 8.0, 3.0], [8.0, 2.0, 1.0]], [[3.0, 4.0, 4.0], 
[4.0, 2.0, 6.0], [7.0, 4.0, 5.0], [4.0, 7.0, 5.0]], [[1.0, 8.0, 4.0], [5.0, 1.0, 3.0], [7.0, 4.0, 2.0], [1.0, 2.0, 8.0]]]))
      val=Data(numpy.array([[[1.0], [5.0], [1.0]], [[7.0], [5.0], [4.0]], [[6.0], [8.0], [5.0]]]),self.functionspace)
      arg[:,:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 1.0, 3.0], [4.0, 5.0, 7.0], [1.0, 1.0, 4.0], [8.0, 6.0, 7.0]], [[4.0, 7.0, 1.0], [5.0, 
5.0, 1.0], [1.0, 4.0, 6.0], [7.0, 2.0, 2.0]], [[1.0, 6.0, 6.0], [5.0, 8.0, 8.0], [1.0, 5.0, 2.0], [4.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 7.0], [8.0, 5.0, 3.0], [1.0, 1.0, 3.0], [8.0, 2.0, 1.0]], [[3.0, 7.0, 4.0], [4.0, 
5.0, 6.0], [7.0, 4.0, 5.0], [4.0, 7.0, 5.0]], [[1.0, 6.0, 4.0], [5.0, 8.0, 3.0], [7.0, 5.0, 2.0], [1.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CUR(self):
      arg=Data(numpy.array([[[1.0, 6.0, 2.0], [8.0, 8.0, 2.0], [3.0, 8.0, 5.0], [4.0, 4.0, 1.0]], [[6.0, 5.0, 2.0], [3.0, 1.0, 
4.0], [6.0, 4.0, 1.0], [8.0, 8.0, 3.0]], [[6.0, 7.0, 1.0], [8.0, 3.0, 6.0], [1.0, 5.0, 5.0], [2.0, 8.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 4.0, 2.0], [3.0, 2.0, 2.0], [6.0, 4.0, 3.0], [8.0, 2.0, 5.0]], [[5.0, 5.0, 4.0], 
[2.0, 7.0, 4.0], [2.0, 8.0, 1.0], [8.0, 6.0, 4.0]], [[1.0, 2.0, 5.0], [6.0, 6.0, 3.0], [2.0, 4.0, 7.0], [8.0, 7.0, 6.0]]]))
      val=Data(numpy.array([[[1.0], [3.0], [6.0]], [[4.0], [6.0], [2.0]], [[6.0], [3.0], [8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0], [6.0], [6.0]], [[2.0], [4.0], [5.0]], [[4.0], [1.0], [1.0]]]))
      arg[:,:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 2.0], [8.0, 3.0, 2.0], [3.0, 6.0, 5.0], [4.0, 4.0, 1.0]], [[6.0, 4.0, 2.0], [3.0, 
6.0, 4.0], [6.0, 2.0, 1.0], [8.0, 8.0, 3.0]], [[6.0, 6.0, 1.0], [8.0, 3.0, 6.0], [1.0, 8.0, 5.0], [2.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 2.0], [3.0, 6.0, 2.0], [6.0, 6.0, 3.0], [8.0, 2.0, 5.0]], [[5.0, 2.0, 4.0], [2.0, 
4.0, 4.0], [2.0, 5.0, 1.0], [8.0, 6.0, 4.0]], [[1.0, 4.0, 5.0], [6.0, 1.0, 3.0], [2.0, 1.0, 7.0], [8.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CUR(self):
      arg=Data(numpy.array([[[7.0, 1.0, 3.0], [7.0, 8.0, 8.0], [5.0, 7.0, 8.0], [7.0, 5.0, 8.0]], [[1.0, 3.0, 5.0], [7.0, 1.0, 
3.0], [8.0, 3.0, 1.0], [6.0, 6.0, 3.0]], [[5.0, 2.0, 1.0], [1.0, 3.0, 1.0], [2.0, 3.0, 8.0], [8.0, 5.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 1.0, 4.0], [6.0, 5.0, 7.0], [5.0, 7.0, 2.0], [2.0, 8.0, 7.0]], [[4.0, 4.0, 6.0], 
[1.0, 6.0, 5.0], [6.0, 2.0, 1.0], [7.0, 2.0, 7.0]], [[4.0, 7.0, 1.0], [8.0, 5.0, 1.0], [3.0, 6.0, 3.0], [4.0, 3.0, 3.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0], [1.0], [1.0]], [[7.0], [7.0], [4.0]], [[7.0], [8.0], 
[1.0]]])+(1.-msk_val)*numpy.array([[[5.0], [2.0], [8.0]], [[2.0], [2.0], [3.0]], [[4.0], [2.0], [1.0]]])
      arg[:,:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 3.0], [7.0, 1.0, 8.0], [5.0, 1.0, 8.0], [7.0, 5.0, 8.0]], [[1.0, 7.0, 5.0], [7.0, 
7.0, 3.0], [8.0, 4.0, 1.0], [6.0, 6.0, 3.0]], [[5.0, 7.0, 1.0], [1.0, 8.0, 1.0], [2.0, 1.0, 8.0], [8.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 4.0], [6.0, 2.0, 7.0], [5.0, 8.0, 2.0], [2.0, 8.0, 7.0]], [[4.0, 2.0, 6.0], [1.0, 
2.0, 5.0], [6.0, 3.0, 1.0], [7.0, 2.0, 7.0]], [[4.0, 4.0, 1.0], [8.0, 2.0, 1.0], [3.0, 1.0, 3.0], [4.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_CCP(self):
      arg=Data(numpy.array([[[1.0, 4.0, 4.0], [5.0, 6.0, 4.0], [5.0, 6.0, 4.0], [5.0, 5.0, 6.0]], [[3.0, 5.0, 1.0], [8.0, 7.0, 
7.0], [7.0, 4.0, 5.0], [5.0, 5.0, 4.0]], [[5.0, 8.0, 8.0], [1.0, 1.0, 3.0], [1.0, 2.0, 3.0], [8.0, 5.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 8.0, 7.0], [8.0, 3.0, 2.0], [5.0, 1.0, 5.0], [4.0, 5.0, 8.0]], [[4.0, 6.0, 8.0], 
[5.0, 1.0, 8.0], [3.0, 5.0, 3.0], [6.0, 4.0, 2.0]], [[8.0, 6.0, 2.0], [3.0, 3.0, 4.0], [8.0, 3.0, 8.0], [5.0, 2.0, 7.0]]]))
      val=numpy.array([[6.0, 2.0, 5.0, 2.0], [2.0, 2.0, 7.0, 6.0], [3.0, 8.0, 1.0, 7.0]])
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 4.0], [5.0, 2.0, 4.0], [5.0, 5.0, 4.0], [5.0, 2.0, 6.0]], [[3.0, 2.0, 1.0], [8.0, 
2.0, 7.0], [7.0, 7.0, 5.0], [5.0, 6.0, 4.0]], [[5.0, 3.0, 8.0], [1.0, 8.0, 3.0], [1.0, 1.0, 3.0], [8.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 6.0, 7.0], [8.0, 2.0, 2.0], [5.0, 5.0, 5.0], [4.0, 2.0, 8.0]], [[4.0, 2.0, 8.0], [5.0, 
2.0, 8.0], [3.0, 7.0, 3.0], [6.0, 6.0, 2.0]], [[8.0, 3.0, 2.0], [3.0, 8.0, 4.0], [8.0, 1.0, 8.0], [5.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_CCP(self):
      arg=Data(numpy.array([[[5.0, 1.0, 3.0], [8.0, 5.0, 8.0], [3.0, 6.0, 4.0], [8.0, 8.0, 5.0]], [[5.0, 6.0, 4.0], [7.0, 1.0, 
7.0], [1.0, 3.0, 4.0], [6.0, 4.0, 8.0]], [[8.0, 2.0, 3.0], [5.0, 2.0, 4.0], [1.0, 1.0, 1.0], [1.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 4.0, 1.0], [2.0, 6.0, 5.0], [1.0, 6.0, 1.0], [1.0, 1.0, 4.0]], [[8.0, 7.0, 1.0], 
[6.0, 1.0, 3.0], [6.0, 3.0, 8.0], [2.0, 7.0, 7.0]], [[4.0, 4.0, 1.0], [8.0, 4.0, 5.0], [6.0, 5.0, 2.0], [4.0, 6.0, 4.0]]]))
      val=Data(numpy.array([[5.0, 3.0, 8.0, 8.0], [8.0, 6.0, 6.0, 3.0], [4.0, 3.0, 3.0, 4.0]]),self.functionspace)
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 5.0, 3.0], [8.0, 3.0, 8.0], [3.0, 8.0, 4.0], [8.0, 8.0, 5.0]], [[5.0, 8.0, 4.0], [7.0, 
6.0, 7.0], [1.0, 6.0, 4.0], [6.0, 3.0, 8.0]], [[8.0, 4.0, 3.0], [5.0, 3.0, 4.0], [1.0, 3.0, 1.0], [1.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 1.0], [2.0, 3.0, 5.0], [1.0, 8.0, 1.0], [1.0, 8.0, 4.0]], [[8.0, 8.0, 1.0], [6.0, 
6.0, 3.0], [6.0, 6.0, 8.0], [2.0, 3.0, 7.0]], [[4.0, 4.0, 1.0], [8.0, 3.0, 5.0], [6.0, 3.0, 2.0], [4.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_CCP(self):
      arg=Data(numpy.array([[[1.0, 8.0, 6.0], [3.0, 8.0, 6.0], [4.0, 4.0, 2.0], [1.0, 6.0, 2.0]], [[3.0, 1.0, 1.0], [6.0, 5.0, 
6.0], [5.0, 7.0, 4.0], [4.0, 1.0, 4.0]], [[2.0, 2.0, 8.0], [6.0, 5.0, 7.0], [4.0, 4.0, 8.0], [6.0, 5.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 2.0, 5.0], [2.0, 1.0, 6.0], [7.0, 8.0, 4.0], [4.0, 1.0, 6.0]], [[8.0, 6.0, 3.0], 
[2.0, 1.0, 1.0], [1.0, 8.0, 6.0], [6.0, 1.0, 8.0]], [[3.0, 2.0, 7.0], [3.0, 3.0, 8.0], [7.0, 4.0, 5.0], [7.0, 4.0, 1.0]]]))
      val=Data(numpy.array([[3.0, 6.0, 4.0, 6.0], [6.0, 3.0, 5.0, 1.0], [4.0, 3.0, 3.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 8.0, 7.0, 1.0], [5.0, 5.0, 6.0, 6.0], [8.0, 8.0, 1.0, 6.0]]))
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 6.0], [3.0, 6.0, 6.0], [4.0, 4.0, 2.0], [1.0, 6.0, 2.0]], [[3.0, 6.0, 1.0], [6.0, 
3.0, 6.0], [5.0, 5.0, 4.0], [4.0, 1.0, 4.0]], [[2.0, 4.0, 8.0], [6.0, 3.0, 7.0], [4.0, 3.0, 8.0], [6.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 4.0, 5.0], [2.0, 8.0, 6.0], [7.0, 7.0, 4.0], [4.0, 1.0, 6.0]], [[8.0, 5.0, 3.0], [2.0, 
5.0, 1.0], [1.0, 6.0, 6.0], [6.0, 6.0, 8.0]], [[3.0, 8.0, 7.0], [3.0, 8.0, 8.0], [7.0, 1.0, 5.0], [7.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_CCP(self):
      arg=Data(numpy.array([[[6.0, 5.0, 3.0], [6.0, 3.0, 8.0], [4.0, 5.0, 7.0], [5.0, 4.0, 8.0]], [[7.0, 8.0, 4.0], [1.0, 7.0, 
7.0], [3.0, 3.0, 7.0], [8.0, 4.0, 2.0]], [[4.0, 2.0, 4.0], [2.0, 6.0, 5.0], [6.0, 5.0, 8.0], [2.0, 4.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 5.0, 3.0], [7.0, 1.0, 2.0], [5.0, 1.0, 5.0], [5.0, 4.0, 1.0]], [[6.0, 1.0, 4.0], 
[1.0, 2.0, 2.0], [2.0, 5.0, 1.0], [2.0, 2.0, 3.0]], [[1.0, 1.0, 6.0], [5.0, 7.0, 7.0], [2.0, 6.0, 4.0], [7.0, 3.0, 7.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[8.0, 4.0, 2.0, 6.0], [1.0, 2.0, 6.0, 3.0], [8.0, 3.0, 1.0, 
4.0]])+(1.-msk_val)*numpy.array([[8.0, 2.0, 5.0, 6.0], [8.0, 1.0, 8.0, 4.0], [7.0, 5.0, 7.0, 1.0]])
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 3.0], [6.0, 4.0, 8.0], [4.0, 2.0, 7.0], [5.0, 6.0, 8.0]], [[7.0, 1.0, 4.0], [1.0, 
2.0, 7.0], [3.0, 6.0, 7.0], [8.0, 3.0, 2.0]], [[4.0, 8.0, 4.0], [2.0, 3.0, 5.0], [6.0, 1.0, 8.0], [2.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 3.0], [7.0, 2.0, 2.0], [5.0, 5.0, 5.0], [5.0, 6.0, 1.0]], [[6.0, 8.0, 4.0], [1.0, 
1.0, 2.0], [2.0, 8.0, 1.0], [2.0, 4.0, 3.0]], [[1.0, 7.0, 6.0], [5.0, 5.0, 7.0], [2.0, 7.0, 4.0], [7.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RLN(self):
      arg=Data(numpy.array([[[6.0, 4.0, 6.0], [8.0, 3.0, 5.0], [4.0, 6.0, 5.0], [6.0, 1.0, 8.0]], [[3.0, 7.0, 4.0], [3.0, 4.0, 
1.0], [3.0, 5.0, 7.0], [3.0, 2.0, 7.0]], [[5.0, 1.0, 8.0], [7.0, 2.0, 2.0], [8.0, 3.0, 5.0], [7.0, 2.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 2.0, 1.0], [4.0, 3.0, 1.0], [5.0, 4.0, 1.0], [7.0, 7.0, 2.0]], [[6.0, 5.0, 8.0], 
[4.0, 1.0, 6.0], [7.0, 3.0, 8.0], [8.0, 6.0, 2.0]], [[8.0, 6.0, 5.0], [5.0, 5.0, 3.0], [5.0, 5.0, 7.0], [2.0, 2.0, 4.0]]]))
      val=numpy.array([[[7.0, 1.0, 6.0], [3.0, 7.0, 2.0]]])
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 6.0], [8.0, 3.0, 5.0], [4.0, 6.0, 5.0], [6.0, 1.0, 8.0]], [[3.0, 7.0, 4.0], [3.0, 
4.0, 1.0], [7.0, 1.0, 6.0], [3.0, 7.0, 2.0]], [[5.0, 1.0, 8.0], [7.0, 2.0, 2.0], [8.0, 3.0, 5.0], [7.0, 2.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 1.0], [4.0, 3.0, 1.0], [5.0, 4.0, 1.0], [7.0, 7.0, 2.0]], [[6.0, 5.0, 8.0], [4.0, 
1.0, 6.0], [7.0, 1.0, 6.0], [3.0, 7.0, 2.0]], [[8.0, 6.0, 5.0], [5.0, 5.0, 3.0], [5.0, 5.0, 7.0], [2.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RLN(self):
      arg=Data(numpy.array([[[4.0, 1.0, 2.0], [3.0, 3.0, 8.0], [6.0, 6.0, 1.0], [8.0, 7.0, 3.0]], [[6.0, 3.0, 2.0], [3.0, 3.0, 
6.0], [3.0, 6.0, 2.0], [7.0, 1.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 4.0, 6.0], [6.0, 5.0, 1.0], [2.0, 1.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 5.0, 4.0], [2.0, 4.0, 4.0], [7.0, 4.0, 3.0], [1.0, 7.0, 8.0]], [[5.0, 3.0, 7.0], 
[7.0, 6.0, 8.0], [4.0, 5.0, 4.0], [7.0, 2.0, 1.0]], [[4.0, 4.0, 6.0], [1.0, 4.0, 6.0], [8.0, 3.0, 6.0], [4.0, 8.0, 8.0]]]))
      val=Data(numpy.array([[[7.0, 3.0, 7.0], [6.0, 6.0, 2.0]]]),self.functionspace)
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 1.0, 2.0], [3.0, 3.0, 8.0], [6.0, 6.0, 1.0], [8.0, 7.0, 3.0]], [[6.0, 3.0, 2.0], [3.0, 
3.0, 6.0], [7.0, 3.0, 7.0], [6.0, 6.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 4.0, 6.0], [6.0, 5.0, 1.0], [2.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 4.0], [2.0, 4.0, 4.0], [7.0, 4.0, 3.0], [1.0, 7.0, 8.0]], [[5.0, 3.0, 7.0], [7.0, 
6.0, 8.0], [7.0, 3.0, 7.0], [6.0, 6.0, 2.0]], [[4.0, 4.0, 6.0], [1.0, 4.0, 6.0], [8.0, 3.0, 6.0], [4.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RLN(self):
      arg=Data(numpy.array([[[8.0, 1.0, 5.0], [8.0, 7.0, 2.0], [4.0, 4.0, 7.0], [5.0, 8.0, 2.0]], [[7.0, 8.0, 2.0], [8.0, 4.0, 
8.0], [4.0, 7.0, 4.0], [7.0, 4.0, 1.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 2.0], [6.0, 4.0, 3.0], [3.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 5.0, 2.0], [3.0, 3.0, 5.0], [6.0, 1.0, 2.0], [2.0, 2.0, 6.0]], [[5.0, 4.0, 5.0], 
[8.0, 8.0, 3.0], [8.0, 4.0, 8.0], [3.0, 7.0, 8.0]], [[3.0, 8.0, 3.0], [2.0, 3.0, 4.0], [6.0, 7.0, 5.0], [5.0, 8.0, 4.0]]]))
      val=Data(numpy.array([[[6.0, 4.0, 8.0], [1.0, 2.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 3.0, 3.0], [3.0, 1.0, 6.0]]]))
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 1.0, 5.0], [8.0, 7.0, 2.0], [4.0, 4.0, 7.0], [5.0, 8.0, 2.0]], [[7.0, 8.0, 2.0], [8.0, 
4.0, 8.0], [6.0, 4.0, 8.0], [1.0, 2.0, 5.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 2.0], [6.0, 4.0, 3.0], [3.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 2.0], [3.0, 3.0, 5.0], [6.0, 1.0, 2.0], [2.0, 2.0, 6.0]], [[5.0, 4.0, 5.0], [8.0, 
8.0, 3.0], [5.0, 3.0, 3.0], [3.0, 1.0, 6.0]], [[3.0, 8.0, 3.0], [2.0, 3.0, 4.0], [6.0, 7.0, 5.0], [5.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RLN(self):
      arg=Data(numpy.array([[[2.0, 6.0, 3.0], [7.0, 8.0, 2.0], [5.0, 8.0, 4.0], [8.0, 8.0, 8.0]], [[4.0, 3.0, 1.0], [1.0, 2.0, 
5.0], [6.0, 6.0, 6.0], [2.0, 3.0, 7.0]], [[5.0, 7.0, 7.0], [2.0, 7.0, 8.0], [1.0, 5.0, 2.0], [5.0, 1.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 1.0, 4.0], [4.0, 2.0, 5.0], [2.0, 1.0, 8.0], [4.0, 7.0, 3.0]], [[1.0, 1.0, 3.0], 
[8.0, 3.0, 4.0], [2.0, 1.0, 5.0], [5.0, 2.0, 4.0]], [[2.0, 4.0, 4.0], [3.0, 6.0, 2.0], [1.0, 3.0, 6.0], [2.0, 8.0, 1.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 3.0, 4.0], [6.0, 4.0, 8.0]]])+(1.-msk_val)*numpy.array([[[6.0, 3.0, 6.0], [5.0, 1.0, 
6.0]]])
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 6.0, 3.0], [7.0, 8.0, 2.0], [5.0, 8.0, 4.0], [8.0, 8.0, 8.0]], [[4.0, 3.0, 1.0], [1.0, 
2.0, 5.0], [4.0, 3.0, 4.0], [6.0, 4.0, 8.0]], [[5.0, 7.0, 7.0], [2.0, 7.0, 8.0], [1.0, 5.0, 2.0], [5.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 4.0], [4.0, 2.0, 5.0], [2.0, 1.0, 8.0], [4.0, 7.0, 3.0]], [[1.0, 1.0, 3.0], [8.0, 
3.0, 4.0], [6.0, 3.0, 6.0], [5.0, 1.0, 6.0]], [[2.0, 4.0, 4.0], [3.0, 6.0, 2.0], [1.0, 3.0, 6.0], [2.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RLP(self):
      arg=Data(numpy.array([[[4.0, 3.0, 3.0], [8.0, 1.0, 6.0], [4.0, 3.0, 4.0], [5.0, 5.0, 3.0]], [[1.0, 3.0, 1.0], [8.0, 1.0, 
3.0], [6.0, 2.0, 2.0], [8.0, 4.0, 5.0]], [[4.0, 1.0, 4.0], [5.0, 5.0, 4.0], [4.0, 5.0, 2.0], [7.0, 2.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 2.0, 5.0], [3.0, 8.0, 6.0], [6.0, 5.0, 2.0], [3.0, 6.0, 6.0]], [[5.0, 1.0, 4.0], 
[5.0, 8.0, 7.0], [8.0, 4.0, 3.0], [2.0, 5.0, 2.0]], [[3.0, 5.0, 1.0], [6.0, 8.0, 7.0], [7.0, 6.0, 8.0], [2.0, 1.0, 7.0]]]))
      val=numpy.array([[6.0, 6.0]])
      arg[1:2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 3.0], [8.0, 1.0, 6.0], [4.0, 3.0, 4.0], [5.0, 5.0, 3.0]], [[1.0, 3.0, 1.0], [8.0, 
1.0, 3.0], [6.0, 6.0, 2.0], [8.0, 6.0, 5.0]], [[4.0, 1.0, 4.0], [5.0, 5.0, 4.0], [4.0, 5.0, 2.0], [7.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 5.0], [3.0, 8.0, 6.0], [6.0, 5.0, 2.0], [3.0, 6.0, 6.0]], [[5.0, 1.0, 4.0], [5.0, 
8.0, 7.0], [8.0, 6.0, 3.0], [2.0, 6.0, 2.0]], [[3.0, 5.0, 1.0], [6.0, 8.0, 7.0], [7.0, 6.0, 8.0], [2.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RLP(self):
      arg=Data(numpy.array([[[3.0, 8.0, 7.0], [2.0, 6.0, 1.0], [1.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[3.0, 8.0, 3.0], [8.0, 4.0, 
3.0], [8.0, 7.0, 8.0], [6.0, 3.0, 4.0]], [[5.0, 2.0, 6.0], [2.0, 2.0, 5.0], [1.0, 2.0, 2.0], [6.0, 3.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 7.0, 1.0], [3.0, 5.0, 2.0], [5.0, 8.0, 3.0], [1.0, 7.0, 5.0]], [[4.0, 3.0, 1.0], 
[5.0, 5.0, 7.0], [6.0, 4.0, 2.0], [6.0, 4.0, 3.0]], [[6.0, 2.0, 5.0], [4.0, 1.0, 2.0], [1.0, 3.0, 4.0], [1.0, 1.0, 1.0]]]))
      val=Data(numpy.array([[7.0, 7.0]]),self.functionspace)
      arg[1:2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 7.0], [2.0, 6.0, 1.0], [1.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[3.0, 8.0, 3.0], [8.0, 
4.0, 3.0], [8.0, 7.0, 8.0], [6.0, 7.0, 4.0]], [[5.0, 2.0, 6.0], [2.0, 2.0, 5.0], [1.0, 2.0, 2.0], [6.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 7.0, 1.0], [3.0, 5.0, 2.0], [5.0, 8.0, 3.0], [1.0, 7.0, 5.0]], [[4.0, 3.0, 1.0], [5.0, 
5.0, 7.0], [6.0, 7.0, 2.0], [6.0, 7.0, 3.0]], [[6.0, 2.0, 5.0], [4.0, 1.0, 2.0], [1.0, 3.0, 4.0], [1.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RLP(self):
      arg=Data(numpy.array([[[3.0, 3.0, 8.0], [2.0, 4.0, 6.0], [2.0, 2.0, 6.0], [3.0, 5.0, 8.0]], [[8.0, 5.0, 7.0], [3.0, 6.0, 
4.0], [2.0, 3.0, 5.0], [1.0, 7.0, 1.0]], [[2.0, 1.0, 5.0], [8.0, 3.0, 4.0], [7.0, 8.0, 8.0], [6.0, 8.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 1.0, 5.0], [3.0, 3.0, 6.0], [1.0, 5.0, 3.0], [8.0, 5.0, 3.0]], [[6.0, 8.0, 8.0], 
[1.0, 8.0, 5.0], [7.0, 1.0, 1.0], [4.0, 3.0, 3.0]], [[7.0, 4.0, 3.0], [1.0, 5.0, 6.0], [4.0, 5.0, 1.0], [1.0, 1.0, 5.0]]]))
      val=Data(numpy.array([[2.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 1.0]]))
      arg[1:2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 8.0], [2.0, 4.0, 6.0], [2.0, 2.0, 6.0], [3.0, 5.0, 8.0]], [[8.0, 5.0, 7.0], [3.0, 
6.0, 4.0], [2.0, 2.0, 5.0], [1.0, 7.0, 1.0]], [[2.0, 1.0, 5.0], [8.0, 3.0, 4.0], [7.0, 8.0, 8.0], [6.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 5.0], [3.0, 3.0, 6.0], [1.0, 5.0, 3.0], [8.0, 5.0, 3.0]], [[6.0, 8.0, 8.0], [1.0, 
8.0, 5.0], [7.0, 5.0, 1.0], [4.0, 1.0, 3.0]], [[7.0, 4.0, 3.0], [1.0, 5.0, 6.0], [4.0, 5.0, 1.0], [1.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RLP(self):
      arg=Data(numpy.array([[[8.0, 2.0, 5.0], [1.0, 1.0, 8.0], [3.0, 2.0, 2.0], [7.0, 6.0, 2.0]], [[7.0, 3.0, 3.0], [3.0, 4.0, 
3.0], [3.0, 1.0, 5.0], [6.0, 8.0, 6.0]], [[4.0, 6.0, 6.0], [3.0, 6.0, 5.0], [4.0, 3.0, 7.0], [8.0, 6.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 5.0, 7.0], [7.0, 2.0, 3.0], [7.0, 4.0, 3.0], [2.0, 6.0, 1.0]], [[2.0, 5.0, 2.0], 
[2.0, 2.0, 5.0], [1.0, 3.0, 7.0], [7.0, 5.0, 6.0]], [[4.0, 2.0, 7.0], [8.0, 5.0, 3.0], [2.0, 4.0, 5.0], [2.0, 5.0, 5.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[4.0, 8.0]])+(1.-msk_val)*numpy.array([[3.0, 5.0]])
      arg[1:2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 5.0], [1.0, 1.0, 8.0], [3.0, 2.0, 2.0], [7.0, 6.0, 2.0]], [[7.0, 3.0, 3.0], [3.0, 
4.0, 3.0], [3.0, 4.0, 5.0], [6.0, 8.0, 6.0]], [[4.0, 6.0, 6.0], [3.0, 6.0, 5.0], [4.0, 3.0, 7.0], [8.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 7.0], [7.0, 2.0, 3.0], [7.0, 4.0, 3.0], [2.0, 6.0, 1.0]], [[2.0, 5.0, 2.0], [2.0, 
2.0, 5.0], [1.0, 3.0, 7.0], [7.0, 5.0, 6.0]], [[4.0, 2.0, 7.0], [8.0, 5.0, 3.0], [2.0, 4.0, 5.0], [2.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LCN(self):
      arg=Data(numpy.array([[[4.0, 7.0, 6.0], [2.0, 6.0, 4.0], [1.0, 6.0, 5.0], [3.0, 4.0, 8.0]], [[7.0, 3.0, 4.0], [5.0, 1.0, 
1.0], [2.0, 1.0, 8.0], [6.0, 2.0, 7.0]], [[1.0, 2.0, 7.0], [7.0, 8.0, 7.0], [2.0, 3.0, 4.0], [8.0, 1.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 8.0, 2.0], [5.0, 8.0, 1.0], [2.0, 2.0, 4.0], [1.0, 1.0, 2.0]], [[2.0, 7.0, 6.0], 
[6.0, 7.0, 3.0], [8.0, 5.0, 3.0], [8.0, 5.0, 5.0]], [[8.0, 7.0, 3.0], [2.0, 2.0, 2.0], [8.0, 5.0, 8.0], [8.0, 6.0, 6.0]]]))
      val=numpy.array([[[2.0, 1.0, 6.0], [1.0, 2.0, 4.0], [3.0, 5.0, 3.0], [4.0, 6.0, 1.0]]])
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 7.0, 6.0], [2.0, 6.0, 4.0], [1.0, 6.0, 5.0], [3.0, 4.0, 8.0]], [[7.0, 3.0, 4.0], [5.0, 
1.0, 1.0], [2.0, 1.0, 8.0], [6.0, 2.0, 7.0]], [[2.0, 1.0, 6.0], [1.0, 2.0, 4.0], [3.0, 5.0, 3.0], [4.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 2.0], [5.0, 8.0, 1.0], [2.0, 2.0, 4.0], [1.0, 1.0, 2.0]], [[2.0, 7.0, 6.0], [6.0, 
7.0, 3.0], [8.0, 5.0, 3.0], [8.0, 5.0, 5.0]], [[2.0, 1.0, 6.0], [1.0, 2.0, 4.0], [3.0, 5.0, 3.0], [4.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LCN(self):
      arg=Data(numpy.array([[[3.0, 7.0, 4.0], [6.0, 3.0, 7.0], [1.0, 2.0, 6.0], [4.0, 2.0, 4.0]], [[2.0, 4.0, 4.0], [3.0, 4.0, 
3.0], [3.0, 1.0, 2.0], [2.0, 3.0, 1.0]], [[1.0, 8.0, 7.0], [8.0, 2.0, 4.0], [5.0, 5.0, 8.0], [1.0, 8.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 3.0, 8.0], [6.0, 6.0, 3.0], [2.0, 3.0, 5.0], [2.0, 2.0, 6.0]], [[8.0, 2.0, 3.0], 
[2.0, 7.0, 7.0], [4.0, 7.0, 1.0], [5.0, 6.0, 3.0]], [[5.0, 1.0, 4.0], [7.0, 6.0, 5.0], [4.0, 6.0, 3.0], [6.0, 8.0, 7.0]]]))
      val=Data(numpy.array([[[7.0, 1.0, 1.0], [5.0, 2.0, 6.0], [6.0, 5.0, 3.0], [5.0, 6.0, 2.0]]]),self.functionspace)
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 4.0], [6.0, 3.0, 7.0], [1.0, 2.0, 6.0], [4.0, 2.0, 4.0]], [[2.0, 4.0, 4.0], [3.0, 
4.0, 3.0], [3.0, 1.0, 2.0], [2.0, 3.0, 1.0]], [[7.0, 1.0, 1.0], [5.0, 2.0, 6.0], [6.0, 5.0, 3.0], [5.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 8.0], [6.0, 6.0, 3.0], [2.0, 3.0, 5.0], [2.0, 2.0, 6.0]], [[8.0, 2.0, 3.0], [2.0, 
7.0, 7.0], [4.0, 7.0, 1.0], [5.0, 6.0, 3.0]], [[7.0, 1.0, 1.0], [5.0, 2.0, 6.0], [6.0, 5.0, 3.0], [5.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LCN(self):
      arg=Data(numpy.array([[[7.0, 4.0, 1.0], [8.0, 2.0, 5.0], [8.0, 6.0, 4.0], [6.0, 4.0, 6.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 
6.0], [3.0, 6.0, 5.0], [1.0, 5.0, 2.0]], [[2.0, 5.0, 8.0], [1.0, 1.0, 8.0], [1.0, 2.0, 1.0], [7.0, 5.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 4.0, 8.0], [3.0, 8.0, 2.0], [7.0, 5.0, 2.0], [2.0, 7.0, 2.0]], [[4.0, 8.0, 7.0], 
[1.0, 8.0, 8.0], [3.0, 6.0, 8.0], [6.0, 7.0, 7.0]], [[2.0, 2.0, 5.0], [7.0, 3.0, 4.0], [2.0, 3.0, 6.0], [7.0, 5.0, 1.0]]]))
      val=Data(numpy.array([[[6.0, 3.0, 2.0], [5.0, 7.0, 6.0], [2.0, 2.0, 4.0], [1.0, 3.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 7.0, 3.0], [7.0, 5.0, 2.0], [6.0, 7.0, 5.0], [7.0, 5.0, 3.0]]]))
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 1.0], [8.0, 2.0, 5.0], [8.0, 6.0, 4.0], [6.0, 4.0, 6.0]], [[4.0, 5.0, 7.0], [2.0, 
6.0, 6.0], [3.0, 6.0, 5.0], [1.0, 5.0, 2.0]], [[6.0, 3.0, 2.0], [5.0, 7.0, 6.0], [2.0, 2.0, 4.0], [1.0, 3.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 8.0], [3.0, 8.0, 2.0], [7.0, 5.0, 2.0], [2.0, 7.0, 2.0]], [[4.0, 8.0, 7.0], [1.0, 
8.0, 8.0], [3.0, 6.0, 8.0], [6.0, 7.0, 7.0]], [[5.0, 7.0, 3.0], [7.0, 5.0, 2.0], [6.0, 7.0, 5.0], [7.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LCN(self):
      arg=Data(numpy.array([[[5.0, 6.0, 1.0], [2.0, 5.0, 6.0], [1.0, 1.0, 5.0], [5.0, 7.0, 2.0]], [[6.0, 8.0, 1.0], [3.0, 3.0, 
1.0], [5.0, 3.0, 5.0], [5.0, 7.0, 5.0]], [[4.0, 5.0, 1.0], [2.0, 3.0, 1.0], [8.0, 5.0, 7.0], [2.0, 7.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 7.0, 3.0], [1.0, 2.0, 6.0], [4.0, 2.0, 5.0], [7.0, 4.0, 7.0]], [[6.0, 2.0, 6.0], 
[1.0, 1.0, 6.0], [5.0, 3.0, 3.0], [3.0, 3.0, 8.0]], [[6.0, 4.0, 6.0], [1.0, 7.0, 8.0], [2.0, 8.0, 6.0], [1.0, 3.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 6.0, 5.0], [8.0, 2.0, 6.0], [2.0, 2.0, 2.0], [8.0, 6.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[4.0, 2.0, 6.0], [1.0, 6.0, 4.0], [4.0, 4.0, 2.0], [1.0, 3.0, 6.0]]])
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 1.0], [2.0, 5.0, 6.0], [1.0, 1.0, 5.0], [5.0, 7.0, 2.0]], [[6.0, 8.0, 1.0], [3.0, 
3.0, 1.0], [5.0, 3.0, 5.0], [5.0, 7.0, 5.0]], [[5.0, 6.0, 5.0], [8.0, 2.0, 6.0], [2.0, 2.0, 2.0], [8.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 7.0, 3.0], [1.0, 2.0, 6.0], [4.0, 2.0, 5.0], [7.0, 4.0, 7.0]], [[6.0, 2.0, 6.0], [1.0, 
1.0, 6.0], [5.0, 3.0, 3.0], [3.0, 3.0, 8.0]], [[4.0, 2.0, 6.0], [1.0, 6.0, 4.0], [4.0, 4.0, 2.0], [1.0, 3.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LPN(self):
      arg=Data(numpy.array([[[5.0, 8.0, 3.0], [1.0, 8.0, 3.0], [2.0, 4.0, 8.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 1.0], [3.0, 2.0, 
7.0], [1.0, 2.0, 1.0], [8.0, 3.0, 8.0]], [[8.0, 4.0, 6.0], [5.0, 3.0, 3.0], [5.0, 3.0, 3.0], [4.0, 5.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 2.0, 3.0], [8.0, 5.0, 2.0], [1.0, 1.0, 8.0], [6.0, 2.0, 6.0]], [[6.0, 8.0, 1.0], 
[5.0, 7.0, 5.0], [7.0, 2.0, 2.0], [5.0, 6.0, 2.0]], [[1.0, 2.0, 6.0], [3.0, 6.0, 5.0], [3.0, 8.0, 6.0], [6.0, 4.0, 1.0]]]))
      val=numpy.array([[2.0, 5.0, 4.0]])
      arg[2:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 3.0], [1.0, 8.0, 3.0], [2.0, 4.0, 8.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 1.0], [3.0, 
2.0, 7.0], [1.0, 2.0, 1.0], [8.0, 3.0, 8.0]], [[8.0, 4.0, 6.0], [5.0, 3.0, 3.0], [2.0, 5.0, 4.0], [4.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 3.0], [8.0, 5.0, 2.0], [1.0, 1.0, 8.0], [6.0, 2.0, 6.0]], [[6.0, 8.0, 1.0], [5.0, 
7.0, 5.0], [7.0, 2.0, 2.0], [5.0, 6.0, 2.0]], [[1.0, 2.0, 6.0], [3.0, 6.0, 5.0], [2.0, 5.0, 4.0], [6.0, 4.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LPN(self):
      arg=Data(numpy.array([[[2.0, 1.0, 1.0], [7.0, 8.0, 1.0], [8.0, 8.0, 5.0], [7.0, 5.0, 5.0]], [[4.0, 5.0, 6.0], [8.0, 2.0, 
1.0], [3.0, 7.0, 2.0], [7.0, 4.0, 8.0]], [[1.0, 1.0, 7.0], [6.0, 2.0, 1.0], [4.0, 1.0, 6.0], [8.0, 8.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 1.0, 7.0], [1.0, 3.0, 1.0], [1.0, 4.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 5.0, 1.0], 
[2.0, 3.0, 8.0], [7.0, 4.0, 8.0], [6.0, 5.0, 4.0]], [[5.0, 4.0, 1.0], [6.0, 8.0, 3.0], [2.0, 5.0, 6.0], [8.0, 8.0, 1.0]]]))
      val=Data(numpy.array([[8.0, 8.0, 1.0]]),self.functionspace)
      arg[2:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 1.0], [7.0, 8.0, 1.0], [8.0, 8.0, 5.0], [7.0, 5.0, 5.0]], [[4.0, 5.0, 6.0], [8.0, 
2.0, 1.0], [3.0, 7.0, 2.0], [7.0, 4.0, 8.0]], [[1.0, 1.0, 7.0], [6.0, 2.0, 1.0], [8.0, 8.0, 1.0], [8.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 1.0, 7.0], [1.0, 3.0, 1.0], [1.0, 4.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 5.0, 1.0], [2.0, 
3.0, 8.0], [7.0, 4.0, 8.0], [6.0, 5.0, 4.0]], [[5.0, 4.0, 1.0], [6.0, 8.0, 3.0], [8.0, 8.0, 1.0], [8.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LPN(self):
      arg=Data(numpy.array([[[8.0, 3.0, 4.0], [6.0, 3.0, 7.0], [3.0, 6.0, 4.0], [1.0, 4.0, 1.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 
6.0], [2.0, 6.0, 1.0], [5.0, 8.0, 1.0]], [[7.0, 8.0, 1.0], [5.0, 4.0, 8.0], [3.0, 8.0, 4.0], [5.0, 8.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 8.0, 5.0], [8.0, 1.0, 2.0], [1.0, 6.0, 7.0], [1.0, 5.0, 5.0]], [[6.0, 1.0, 5.0], 
[4.0, 8.0, 5.0], [4.0, 5.0, 5.0], [8.0, 2.0, 1.0]], [[7.0, 7.0, 8.0], [6.0, 6.0, 2.0], [1.0, 8.0, 1.0], [7.0, 7.0, 8.0]]]))
      val=Data(numpy.array([[8.0, 2.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 8.0, 1.0]]))
      arg[2:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 4.0], [6.0, 3.0, 7.0], [3.0, 6.0, 4.0], [1.0, 4.0, 1.0]], [[6.0, 4.0, 4.0], [2.0, 
5.0, 6.0], [2.0, 6.0, 1.0], [5.0, 8.0, 1.0]], [[7.0, 8.0, 1.0], [5.0, 4.0, 8.0], [8.0, 2.0, 8.0], [5.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 5.0], [8.0, 1.0, 2.0], [1.0, 6.0, 7.0], [1.0, 5.0, 5.0]], [[6.0, 1.0, 5.0], [4.0, 
8.0, 5.0], [4.0, 5.0, 5.0], [8.0, 2.0, 1.0]], [[7.0, 7.0, 8.0], [6.0, 6.0, 2.0], [6.0, 8.0, 1.0], [7.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LPN(self):
      arg=Data(numpy.array([[[7.0, 3.0, 3.0], [6.0, 5.0, 7.0], [7.0, 3.0, 5.0], [2.0, 3.0, 8.0]], [[6.0, 8.0, 6.0], [8.0, 4.0, 
4.0], [3.0, 7.0, 3.0], [2.0, 6.0, 2.0]], [[5.0, 3.0, 7.0], [7.0, 8.0, 1.0], [4.0, 3.0, 7.0], [6.0, 6.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 2.0, 4.0], [4.0, 3.0, 6.0], [1.0, 2.0, 7.0], [4.0, 2.0, 1.0]], [[2.0, 8.0, 8.0], 
[1.0, 8.0, 3.0], [4.0, 8.0, 5.0], [2.0, 8.0, 4.0]], [[5.0, 1.0, 7.0], [8.0, 3.0, 4.0], [6.0, 2.0, 6.0], [3.0, 6.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[5.0, 1.0, 5.0]])+(1.-msk_val)*numpy.array([[2.0, 4.0, 6.0]])
      arg[2:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 3.0], [6.0, 5.0, 7.0], [7.0, 3.0, 5.0], [2.0, 3.0, 8.0]], [[6.0, 8.0, 6.0], [8.0, 
4.0, 4.0], [3.0, 7.0, 3.0], [2.0, 6.0, 2.0]], [[5.0, 3.0, 7.0], [7.0, 8.0, 1.0], [5.0, 1.0, 5.0], [6.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 4.0], [4.0, 3.0, 6.0], [1.0, 2.0, 7.0], [4.0, 2.0, 1.0]], [[2.0, 8.0, 8.0], [1.0, 
8.0, 3.0], [4.0, 8.0, 5.0], [2.0, 8.0, 4.0]], [[5.0, 1.0, 7.0], [8.0, 3.0, 4.0], [2.0, 4.0, 6.0], [3.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LPC(self):
      arg=Data(numpy.array([[[8.0, 6.0, 8.0], [2.0, 8.0, 1.0], [8.0, 2.0, 8.0], [3.0, 8.0, 2.0]], [[8.0, 2.0, 4.0], [5.0, 4.0, 
8.0], [1.0, 5.0, 7.0], [3.0, 5.0, 6.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 7.0], [4.0, 5.0, 5.0], [5.0, 8.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 7.0, 5.0], [2.0, 3.0, 5.0], [1.0, 7.0, 3.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 2.0], 
[1.0, 7.0, 8.0], [5.0, 1.0, 5.0], [5.0, 7.0, 8.0]], [[4.0, 6.0, 2.0], [6.0, 8.0, 8.0], [2.0, 6.0, 1.0], [6.0, 2.0, 2.0]]]))
      val=numpy.array([[7.0, 4.0, 3.0]])
      arg[2:,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 8.0], [2.0, 8.0, 1.0], [8.0, 2.0, 8.0], [3.0, 8.0, 2.0]], [[8.0, 2.0, 4.0], [5.0, 
4.0, 8.0], [1.0, 5.0, 7.0], [3.0, 5.0, 6.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 7.0], [7.0, 4.0, 3.0], [5.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 5.0], [2.0, 3.0, 5.0], [1.0, 7.0, 3.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 2.0], [1.0, 
7.0, 8.0], [5.0, 1.0, 5.0], [5.0, 7.0, 8.0]], [[4.0, 6.0, 2.0], [6.0, 8.0, 8.0], [7.0, 4.0, 3.0], [6.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LPC(self):
      arg=Data(numpy.array([[[5.0, 3.0, 1.0], [4.0, 2.0, 8.0], [8.0, 7.0, 2.0], [1.0, 7.0, 3.0]], [[4.0, 3.0, 4.0], [4.0, 7.0, 
6.0], [1.0, 7.0, 7.0], [1.0, 8.0, 2.0]], [[3.0, 2.0, 3.0], [8.0, 2.0, 1.0], [8.0, 2.0, 8.0], [2.0, 4.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 5.0, 1.0], [1.0, 5.0, 5.0], [4.0, 4.0, 8.0], [6.0, 1.0, 4.0]], [[4.0, 1.0, 5.0], 
[8.0, 3.0, 6.0], [7.0, 3.0, 4.0], [8.0, 5.0, 1.0]], [[4.0, 7.0, 1.0], [1.0, 7.0, 4.0], [1.0, 2.0, 4.0], [2.0, 1.0, 5.0]]]))
      val=Data(numpy.array([[8.0, 8.0, 2.0]]),self.functionspace)
      arg[2:,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 1.0], [4.0, 2.0, 8.0], [8.0, 7.0, 2.0], [1.0, 7.0, 3.0]], [[4.0, 3.0, 4.0], [4.0, 
7.0, 6.0], [1.0, 7.0, 7.0], [1.0, 8.0, 2.0]], [[3.0, 2.0, 3.0], [8.0, 2.0, 1.0], [8.0, 8.0, 2.0], [2.0, 4.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 1.0], [1.0, 5.0, 5.0], [4.0, 4.0, 8.0], [6.0, 1.0, 4.0]], [[4.0, 1.0, 5.0], [8.0, 
3.0, 6.0], [7.0, 3.0, 4.0], [8.0, 5.0, 1.0]], [[4.0, 7.0, 1.0], [1.0, 7.0, 4.0], [8.0, 8.0, 2.0], [2.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LPC(self):
      arg=Data(numpy.array([[[6.0, 5.0, 6.0], [7.0, 2.0, 2.0], [3.0, 1.0, 7.0], [2.0, 4.0, 7.0]], [[1.0, 6.0, 8.0], [7.0, 8.0, 
6.0], [2.0, 3.0, 4.0], [7.0, 5.0, 8.0]], [[4.0, 1.0, 4.0], [1.0, 2.0, 7.0], [6.0, 2.0, 7.0], [1.0, 5.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 6.0, 2.0], [7.0, 5.0, 6.0], [8.0, 8.0, 4.0], [6.0, 3.0, 7.0]], [[5.0, 5.0, 8.0], 
[7.0, 4.0, 4.0], [8.0, 7.0, 5.0], [8.0, 2.0, 5.0]], [[2.0, 4.0, 8.0], [1.0, 6.0, 2.0], [5.0, 7.0, 2.0], [8.0, 6.0, 6.0]]]))
      val=Data(numpy.array([[6.0, 3.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 7.0, 3.0]]))
      arg[2:,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 6.0], [7.0, 2.0, 2.0], [3.0, 1.0, 7.0], [2.0, 4.0, 7.0]], [[1.0, 6.0, 8.0], [7.0, 
8.0, 6.0], [2.0, 3.0, 4.0], [7.0, 5.0, 8.0]], [[4.0, 1.0, 4.0], [1.0, 2.0, 7.0], [6.0, 3.0, 5.0], [1.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 2.0], [7.0, 5.0, 6.0], [8.0, 8.0, 4.0], [6.0, 3.0, 7.0]], [[5.0, 5.0, 8.0], [7.0, 
4.0, 4.0], [8.0, 7.0, 5.0], [8.0, 2.0, 5.0]], [[2.0, 4.0, 8.0], [1.0, 6.0, 2.0], [3.0, 7.0, 3.0], [8.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LPC(self):
      arg=Data(numpy.array([[[4.0, 2.0, 5.0], [4.0, 7.0, 5.0], [3.0, 7.0, 6.0], [7.0, 1.0, 4.0]], [[7.0, 3.0, 7.0], [5.0, 1.0, 
8.0], [2.0, 4.0, 3.0], [7.0, 8.0, 8.0]], [[2.0, 7.0, 4.0], [6.0, 3.0, 7.0], [7.0, 2.0, 7.0], [8.0, 4.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 2.0, 6.0], [4.0, 4.0, 3.0], [4.0, 7.0, 1.0], [4.0, 7.0, 8.0]], [[2.0, 7.0, 2.0], 
[3.0, 6.0, 1.0], [3.0, 4.0, 6.0], [1.0, 6.0, 2.0]], [[4.0, 3.0, 1.0], [3.0, 5.0, 3.0], [5.0, 6.0, 7.0], [3.0, 5.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[5.0, 8.0, 7.0]])+(1.-msk_val)*numpy.array([[6.0, 8.0, 5.0]])
      arg[2:,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 5.0], [4.0, 7.0, 5.0], [3.0, 7.0, 6.0], [7.0, 1.0, 4.0]], [[7.0, 3.0, 7.0], [5.0, 
1.0, 8.0], [2.0, 4.0, 3.0], [7.0, 8.0, 8.0]], [[2.0, 7.0, 4.0], [6.0, 3.0, 7.0], [5.0, 8.0, 7.0], [8.0, 4.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 6.0], [4.0, 4.0, 3.0], [4.0, 7.0, 1.0], [4.0, 7.0, 8.0]], [[2.0, 7.0, 2.0], [3.0, 
6.0, 1.0], [3.0, 4.0, 6.0], [1.0, 6.0, 2.0]], [[4.0, 3.0, 1.0], [3.0, 5.0, 3.0], [6.0, 8.0, 5.0], [3.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RRL(self):
      arg=Data(numpy.array([[[7.0, 4.0, 6.0], [1.0, 3.0, 1.0], [2.0, 2.0, 5.0], [4.0, 1.0, 1.0]], [[7.0, 4.0, 3.0], [4.0, 3.0, 
3.0], [5.0, 1.0, 8.0], [3.0, 3.0, 5.0]], [[5.0, 8.0, 8.0], [3.0, 8.0, 7.0], [5.0, 1.0, 5.0], [4.0, 2.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[7.0, 5.0, 3.0], [2.0, 2.0, 3.0], [5.0, 2.0, 3.0], [1.0, 5.0, 1.0]], [[8.0, 2.0, 5.0], 
[6.0, 8.0, 1.0], [6.0, 4.0, 6.0], [3.0, 1.0, 2.0]], [[3.0, 8.0, 1.0], [5.0, 1.0, 4.0], [1.0, 6.0, 3.0], [5.0, 5.0, 8.0]]]))
      val=numpy.array([[[6.0], [1.0]]])
      arg[1:2,1:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 6.0], [1.0, 3.0, 1.0], [2.0, 2.0, 5.0], [4.0, 1.0, 1.0]], [[7.0, 4.0, 3.0], [4.0, 
3.0, 6.0], [5.0, 1.0, 1.0], [3.0, 3.0, 5.0]], [[5.0, 8.0, 8.0], [3.0, 8.0, 7.0], [5.0, 1.0, 5.0], [4.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 3.0], [2.0, 2.0, 3.0], [5.0, 2.0, 3.0], [1.0, 5.0, 1.0]], [[8.0, 2.0, 5.0], [6.0, 
8.0, 6.0], [6.0, 4.0, 1.0], [3.0, 1.0, 2.0]], [[3.0, 8.0, 1.0], [5.0, 1.0, 4.0], [1.0, 6.0, 3.0], [5.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RRL(self):
      arg=Data(numpy.array([[[3.0, 4.0, 5.0], [3.0, 8.0, 8.0], [6.0, 6.0, 1.0], [5.0, 5.0, 5.0]], [[6.0, 8.0, 2.0], [6.0, 5.0, 
8.0], [2.0, 5.0, 8.0], [2.0, 2.0, 3.0]], [[8.0, 1.0, 8.0], [8.0, 8.0, 2.0], [6.0, 3.0, 4.0], [4.0, 2.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 5.0, 6.0], [1.0, 4.0, 4.0], [7.0, 5.0, 7.0], [8.0, 3.0, 2.0]], [[3.0, 3.0, 3.0], 
[2.0, 4.0, 2.0], [7.0, 8.0, 5.0], [4.0, 6.0, 2.0]], [[4.0, 3.0, 8.0], [5.0, 4.0, 2.0], [6.0, 8.0, 2.0], [6.0, 5.0, 8.0]]]))
      val=Data(numpy.array([[[8.0], [3.0]]]),self.functionspace)
      arg[1:2,1:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 5.0], [3.0, 8.0, 8.0], [6.0, 6.0, 1.0], [5.0, 5.0, 5.0]], [[6.0, 8.0, 2.0], [6.0, 
5.0, 8.0], [2.0, 5.0, 3.0], [2.0, 2.0, 3.0]], [[8.0, 1.0, 8.0], [8.0, 8.0, 2.0], [6.0, 3.0, 4.0], [4.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 5.0, 6.0], [1.0, 4.0, 4.0], [7.0, 5.0, 7.0], [8.0, 3.0, 2.0]], [[3.0, 3.0, 3.0], [2.0, 
4.0, 8.0], [7.0, 8.0, 3.0], [4.0, 6.0, 2.0]], [[4.0, 3.0, 8.0], [5.0, 4.0, 2.0], [6.0, 8.0, 2.0], [6.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RRL(self):
      arg=Data(numpy.array([[[4.0, 4.0, 7.0], [7.0, 1.0, 5.0], [3.0, 1.0, 7.0], [2.0, 6.0, 7.0]], [[1.0, 7.0, 8.0], [5.0, 8.0, 
3.0], [4.0, 1.0, 3.0], [3.0, 2.0, 4.0]], [[6.0, 7.0, 1.0], [7.0, 1.0, 7.0], [7.0, 1.0, 6.0], [3.0, 2.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 8.0, 7.0], [7.0, 5.0, 5.0], [3.0, 4.0, 3.0], [3.0, 5.0, 7.0]], [[4.0, 4.0, 8.0], 
[2.0, 2.0, 8.0], [4.0, 7.0, 1.0], [1.0, 6.0, 8.0]], [[2.0, 6.0, 1.0], [7.0, 3.0, 2.0], [6.0, 2.0, 8.0], [3.0, 6.0, 8.0]]]))
      val=Data(numpy.array([[[4.0], [4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0], [3.0]]]))
      arg[1:2,1:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 7.0], [7.0, 1.0, 5.0], [3.0, 1.0, 7.0], [2.0, 6.0, 7.0]], [[1.0, 7.0, 8.0], [5.0, 
8.0, 4.0], [4.0, 1.0, 4.0], [3.0, 2.0, 4.0]], [[6.0, 7.0, 1.0], [7.0, 1.0, 7.0], [7.0, 1.0, 6.0], [3.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 7.0], [7.0, 5.0, 5.0], [3.0, 4.0, 3.0], [3.0, 5.0, 7.0]], [[4.0, 4.0, 8.0], [2.0, 
2.0, 8.0], [4.0, 7.0, 3.0], [1.0, 6.0, 8.0]], [[2.0, 6.0, 1.0], [7.0, 3.0, 2.0], [6.0, 2.0, 8.0], [3.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RRL(self):
      arg=Data(numpy.array([[[7.0, 5.0, 3.0], [8.0, 6.0, 7.0], [7.0, 7.0, 6.0], [5.0, 5.0, 8.0]], [[5.0, 8.0, 3.0], [7.0, 6.0, 
5.0], [2.0, 2.0, 3.0], [1.0, 2.0, 3.0]], [[6.0, 3.0, 5.0], [1.0, 1.0, 6.0], [1.0, 2.0, 2.0], [1.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 5.0, 4.0], [1.0, 8.0, 4.0], [3.0, 2.0, 7.0], [6.0, 6.0, 6.0]], [[2.0, 3.0, 6.0], 
[3.0, 1.0, 4.0], [4.0, 4.0, 2.0], [6.0, 4.0, 6.0]], [[3.0, 5.0, 8.0], [1.0, 3.0, 8.0], [8.0, 7.0, 4.0], [5.0, 2.0, 2.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0], [5.0]]])+(1.-msk_val)*numpy.array([[[8.0], [6.0]]])
      arg[1:2,1:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 3.0], [8.0, 6.0, 7.0], [7.0, 7.0, 6.0], [5.0, 5.0, 8.0]], [[5.0, 8.0, 3.0], [7.0, 
6.0, 8.0], [2.0, 2.0, 5.0], [1.0, 2.0, 3.0]], [[6.0, 3.0, 5.0], [1.0, 1.0, 6.0], [1.0, 2.0, 2.0], [1.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 4.0], [1.0, 8.0, 4.0], [3.0, 2.0, 7.0], [6.0, 6.0, 6.0]], [[2.0, 3.0, 6.0], [3.0, 
1.0, 8.0], [4.0, 4.0, 6.0], [6.0, 4.0, 6.0]], [[3.0, 5.0, 8.0], [1.0, 3.0, 8.0], [8.0, 7.0, 4.0], [5.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PUN(self):
      arg=Data(numpy.array([[[7.0, 2.0, 8.0], [3.0, 6.0, 4.0], [8.0, 7.0, 7.0], [4.0, 6.0, 8.0]], [[6.0, 6.0, 8.0], [4.0, 3.0, 
4.0], [1.0, 4.0, 3.0], [2.0, 6.0, 1.0]], [[5.0, 1.0, 7.0], [4.0, 1.0, 4.0], [1.0, 3.0, 3.0], [1.0, 7.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 1.0, 6.0], [3.0, 2.0, 8.0], [3.0, 8.0, 8.0], [3.0, 5.0, 8.0]], [[3.0, 2.0, 5.0], 
[1.0, 7.0, 3.0], [7.0, 1.0, 4.0], [8.0, 8.0, 7.0]], [[2.0, 7.0, 3.0], [3.0, 2.0, 3.0], [6.0, 4.0, 8.0], [2.0, 6.0, 8.0]]]))
      val=numpy.array([[6.0, 1.0, 5.0], [3.0, 7.0, 7.0], [8.0, 8.0, 2.0]])
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 8.0], [3.0, 6.0, 4.0], [8.0, 7.0, 7.0], [4.0, 6.0, 8.0]], [[6.0, 1.0, 5.0], [3.0, 
7.0, 7.0], [8.0, 8.0, 2.0], [2.0, 6.0, 1.0]], [[5.0, 1.0, 7.0], [4.0, 1.0, 4.0], [1.0, 3.0, 3.0], [1.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 6.0], [3.0, 2.0, 8.0], [3.0, 8.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 1.0, 5.0], [3.0, 
7.0, 7.0], [8.0, 8.0, 2.0], [8.0, 8.0, 7.0]], [[2.0, 7.0, 3.0], [3.0, 2.0, 3.0], [6.0, 4.0, 8.0], [2.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PUN(self):
      arg=Data(numpy.array([[[1.0, 7.0, 6.0], [2.0, 5.0, 4.0], [6.0, 6.0, 3.0], [7.0, 1.0, 3.0]], [[7.0, 6.0, 4.0], [4.0, 2.0, 
4.0], [4.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[5.0, 7.0, 6.0], [7.0, 4.0, 2.0], [8.0, 7.0, 1.0], [6.0, 6.0, 
4.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 2.0, 1.0], [7.0, 5.0, 2.0], [3.0, 7.0, 3.0], [3.0, 6.0, 7.0]], [[7.0, 8.0, 4.0], 
[8.0, 7.0, 6.0], [8.0, 5.0, 1.0], [5.0, 6.0, 3.0]], [[8.0, 6.0, 4.0], [8.0, 3.0, 7.0], [8.0, 3.0, 6.0], [4.0, 5.0, 4.0]]]))
      val=Data(numpy.array([[6.0, 3.0, 4.0], [8.0, 4.0, 1.0], [4.0, 8.0, 5.0]]),self.functionspace)
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 6.0], [2.0, 5.0, 4.0], [6.0, 6.0, 3.0], [7.0, 1.0, 3.0]], [[6.0, 3.0, 4.0], [8.0, 
4.0, 1.0], [4.0, 8.0, 5.0], [1.0, 8.0, 7.0]], [[5.0, 7.0, 6.0], [7.0, 4.0, 2.0], [8.0, 7.0, 1.0], [6.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 1.0], [7.0, 5.0, 2.0], [3.0, 7.0, 3.0], [3.0, 6.0, 7.0]], [[6.0, 3.0, 4.0], [8.0, 
4.0, 1.0], [4.0, 8.0, 5.0], [5.0, 6.0, 3.0]], [[8.0, 6.0, 4.0], [8.0, 3.0, 7.0], [8.0, 3.0, 6.0], [4.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PUN(self):
      arg=Data(numpy.array([[[2.0, 7.0, 5.0], [4.0, 8.0, 8.0], [5.0, 3.0, 3.0], [3.0, 6.0, 5.0]], [[7.0, 3.0, 7.0], [7.0, 5.0, 
1.0], [6.0, 1.0, 8.0], [1.0, 2.0, 7.0]], [[8.0, 2.0, 5.0], [7.0, 6.0, 7.0], [7.0, 6.0, 8.0], [8.0, 2.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 5.0, 4.0], [1.0, 7.0, 4.0], [2.0, 4.0, 8.0], [2.0, 5.0, 4.0]], [[2.0, 2.0, 7.0], 
[3.0, 2.0, 6.0], [8.0, 4.0, 2.0], [6.0, 8.0, 7.0]], [[6.0, 1.0, 2.0], [4.0, 1.0, 8.0], [3.0, 8.0, 3.0], [1.0, 7.0, 5.0]]]))
      val=Data(numpy.array([[5.0, 2.0, 5.0], [6.0, 3.0, 4.0], [1.0, 5.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 7.0, 2.0], [8.0, 5.0, 8.0], [7.0, 4.0, 6.0]]))
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 5.0], [4.0, 8.0, 8.0], [5.0, 3.0, 3.0], [3.0, 6.0, 5.0]], [[5.0, 2.0, 5.0], [6.0, 
3.0, 4.0], [1.0, 5.0, 6.0], [1.0, 2.0, 7.0]], [[8.0, 2.0, 5.0], [7.0, 6.0, 7.0], [7.0, 6.0, 8.0], [8.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 5.0, 4.0], [1.0, 7.0, 4.0], [2.0, 4.0, 8.0], [2.0, 5.0, 4.0]], [[3.0, 7.0, 2.0], [8.0, 
5.0, 8.0], [7.0, 4.0, 6.0], [6.0, 8.0, 7.0]], [[6.0, 1.0, 2.0], [4.0, 1.0, 8.0], [3.0, 8.0, 3.0], [1.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PUN(self):
      arg=Data(numpy.array([[[6.0, 7.0, 4.0], [2.0, 8.0, 3.0], [8.0, 6.0, 3.0], [8.0, 4.0, 6.0]], [[5.0, 6.0, 7.0], [4.0, 8.0, 
3.0], [3.0, 1.0, 4.0], [4.0, 6.0, 7.0]], [[3.0, 1.0, 7.0], [8.0, 3.0, 7.0], [7.0, 6.0, 1.0], [8.0, 3.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 5.0, 7.0], [4.0, 4.0, 4.0], [5.0, 5.0, 8.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 1.0], 
[4.0, 8.0, 6.0], [8.0, 3.0, 2.0], [2.0, 4.0, 3.0]], [[3.0, 8.0, 4.0], [7.0, 5.0, 2.0], [3.0, 8.0, 2.0], [7.0, 1.0, 8.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 8.0, 4.0], [7.0, 6.0, 7.0], [8.0, 8.0, 6.0]])+(1.-msk_val)*numpy.array([[5.0, 2.0, 1.0], 
[6.0, 1.0, 5.0], [4.0, 1.0, 3.0]])
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 4.0], [2.0, 8.0, 3.0], [8.0, 6.0, 3.0], [8.0, 4.0, 6.0]], [[6.0, 8.0, 4.0], [7.0, 
6.0, 7.0], [8.0, 8.0, 6.0], [4.0, 6.0, 7.0]], [[3.0, 1.0, 7.0], [8.0, 3.0, 7.0], [7.0, 6.0, 1.0], [8.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 7.0], [4.0, 4.0, 4.0], [5.0, 5.0, 8.0], [4.0, 8.0, 5.0]], [[5.0, 2.0, 1.0], [6.0, 
1.0, 5.0], [4.0, 1.0, 3.0], [2.0, 4.0, 3.0]], [[3.0, 8.0, 4.0], [7.0, 5.0, 2.0], [3.0, 8.0, 2.0], [7.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_RUN(self):
      arg=Data(numpy.array([[[6.0, 2.0, 4.0], [6.0, 7.0, 5.0], [4.0, 1.0, 8.0], [7.0, 4.0, 7.0]], [[4.0, 8.0, 8.0], [8.0, 1.0, 
5.0], [4.0, 8.0, 4.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 6.0], [6.0, 3.0, 5.0], [3.0, 5.0, 2.0], [4.0, 7.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 3.0, 2.0], [7.0, 3.0, 2.0], [6.0, 1.0, 3.0], [7.0, 7.0, 3.0]], [[1.0, 4.0, 3.0], 
[8.0, 1.0, 7.0], [5.0, 3.0, 3.0], [8.0, 6.0, 1.0]], [[6.0, 5.0, 8.0], [6.0, 4.0, 6.0], [3.0, 5.0, 8.0], [2.0, 6.0, 5.0]]]))
      val=numpy.array([[[8.0, 8.0, 4.0], [5.0, 3.0, 2.0], [6.0, 8.0, 1.0]]])
      arg[1:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 4.0], [6.0, 7.0, 5.0], [4.0, 1.0, 8.0], [7.0, 4.0, 7.0]], [[8.0, 8.0, 4.0], [5.0, 
3.0, 2.0], [6.0, 8.0, 1.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 6.0], [6.0, 3.0, 5.0], [3.0, 5.0, 2.0], [4.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 2.0], [7.0, 3.0, 2.0], [6.0, 1.0, 3.0], [7.0, 7.0, 3.0]], [[8.0, 8.0, 4.0], [5.0, 
3.0, 2.0], [6.0, 8.0, 1.0], [8.0, 6.0, 1.0]], [[6.0, 5.0, 8.0], [6.0, 4.0, 6.0], [3.0, 5.0, 8.0], [2.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_RUN(self):
      arg=Data(numpy.array([[[1.0, 6.0, 7.0], [6.0, 6.0, 2.0], [1.0, 8.0, 5.0], [8.0, 6.0, 5.0]], [[8.0, 7.0, 6.0], [6.0, 7.0, 
7.0], [5.0, 8.0, 2.0], [4.0, 4.0, 8.0]], [[3.0, 5.0, 7.0], [8.0, 8.0, 8.0], [1.0, 7.0, 6.0], [2.0, 3.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 6.0, 7.0], [7.0, 3.0, 1.0], [2.0, 4.0, 3.0], [7.0, 4.0, 1.0]], [[8.0, 4.0, 7.0], 
[3.0, 3.0, 8.0], [5.0, 7.0, 5.0], [7.0, 3.0, 4.0]], [[8.0, 3.0, 3.0], [3.0, 4.0, 3.0], [1.0, 2.0, 3.0], [2.0, 7.0, 8.0]]]))
      val=Data(numpy.array([[[5.0, 7.0, 5.0], [3.0, 5.0, 7.0], [7.0, 2.0, 3.0]]]),self.functionspace)
      arg[1:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 7.0], [6.0, 6.0, 2.0], [1.0, 8.0, 5.0], [8.0, 6.0, 5.0]], [[5.0, 7.0, 5.0], [3.0, 
5.0, 7.0], [7.0, 2.0, 3.0], [4.0, 4.0, 8.0]], [[3.0, 5.0, 7.0], [8.0, 8.0, 8.0], [1.0, 7.0, 6.0], [2.0, 3.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 6.0, 7.0], [7.0, 3.0, 1.0], [2.0, 4.0, 3.0], [7.0, 4.0, 1.0]], [[5.0, 7.0, 5.0], [3.0, 
5.0, 7.0], [7.0, 2.0, 3.0], [7.0, 3.0, 4.0]], [[8.0, 3.0, 3.0], [3.0, 4.0, 3.0], [1.0, 2.0, 3.0], [2.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_RUN(self):
      arg=Data(numpy.array([[[1.0, 3.0, 5.0], [2.0, 8.0, 4.0], [1.0, 6.0, 8.0], [5.0, 1.0, 5.0]], [[3.0, 1.0, 6.0], [2.0, 7.0, 
8.0], [4.0, 3.0, 5.0], [2.0, 5.0, 8.0]], [[5.0, 3.0, 7.0], [8.0, 6.0, 3.0], [1.0, 6.0, 5.0], [5.0, 1.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 7.0, 5.0], [7.0, 7.0, 7.0], [2.0, 4.0, 4.0], [8.0, 2.0, 7.0]], [[5.0, 2.0, 8.0], 
[1.0, 2.0, 7.0], [4.0, 6.0, 7.0], [1.0, 8.0, 1.0]], [[5.0, 4.0, 8.0], [4.0, 7.0, 7.0], [2.0, 8.0, 5.0], [1.0, 1.0, 7.0]]]))
      val=Data(numpy.array([[[5.0, 6.0, 8.0], [2.0, 3.0, 7.0], [6.0, 7.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 3.0, 1.0], [3.0, 5.0, 5.0], [7.0, 4.0, 6.0]]]))
      arg[1:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 5.0], [2.0, 8.0, 4.0], [1.0, 6.0, 8.0], [5.0, 1.0, 5.0]], [[5.0, 6.0, 8.0], [2.0, 
3.0, 7.0], [6.0, 7.0, 2.0], [2.0, 5.0, 8.0]], [[5.0, 3.0, 7.0], [8.0, 6.0, 3.0], [1.0, 6.0, 5.0], [5.0, 1.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 5.0], [7.0, 7.0, 7.0], [2.0, 4.0, 4.0], [8.0, 2.0, 7.0]], [[6.0, 3.0, 1.0], [3.0, 
5.0, 5.0], [7.0, 4.0, 6.0], [1.0, 8.0, 1.0]], [[5.0, 4.0, 8.0], [4.0, 7.0, 7.0], [2.0, 8.0, 5.0], [1.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_RUN(self):
      arg=Data(numpy.array([[[3.0, 8.0, 2.0], [6.0, 6.0, 7.0], [8.0, 2.0, 8.0], [3.0, 6.0, 4.0]], [[1.0, 4.0, 8.0], [3.0, 7.0, 
8.0], [2.0, 3.0, 4.0], [5.0, 6.0, 8.0]], [[8.0, 4.0, 1.0], [4.0, 1.0, 1.0], [5.0, 8.0, 5.0], [6.0, 2.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 1.0, 3.0], [2.0, 4.0, 4.0], [7.0, 2.0, 4.0], [3.0, 7.0, 7.0]], [[2.0, 8.0, 1.0], 
[6.0, 3.0, 1.0], [5.0, 8.0, 6.0], [4.0, 7.0, 7.0]], [[2.0, 4.0, 1.0], [1.0, 1.0, 7.0], [3.0, 5.0, 8.0], [7.0, 3.0, 5.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 5.0, 5.0], [5.0, 1.0, 8.0], [1.0, 2.0, 7.0]]])+(1.-msk_val)*numpy.array([[[2.0, 4.0, 
5.0], [6.0, 2.0, 7.0], [2.0, 5.0, 3.0]]])
      arg[1:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 2.0], [6.0, 6.0, 7.0], [8.0, 2.0, 8.0], [3.0, 6.0, 4.0]], [[7.0, 5.0, 5.0], [5.0, 
1.0, 8.0], [1.0, 2.0, 7.0], [5.0, 6.0, 8.0]], [[8.0, 4.0, 1.0], [4.0, 1.0, 1.0], [5.0, 8.0, 5.0], [6.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 1.0, 3.0], [2.0, 4.0, 4.0], [7.0, 2.0, 4.0], [3.0, 7.0, 7.0]], [[2.0, 4.0, 5.0], [6.0, 
2.0, 7.0], [2.0, 5.0, 3.0], [4.0, 7.0, 7.0]], [[2.0, 4.0, 1.0], [1.0, 1.0, 7.0], [3.0, 5.0, 8.0], [7.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_PRN(self):
      arg=Data(numpy.array([[[6.0, 2.0, 3.0], [8.0, 7.0, 6.0], [7.0, 7.0, 2.0], [5.0, 8.0, 7.0]], [[1.0, 7.0, 7.0], [1.0, 2.0, 
4.0], [5.0, 2.0, 6.0], [4.0, 1.0, 5.0]], [[5.0, 7.0, 3.0], [8.0, 4.0, 3.0], [5.0, 2.0, 7.0], [7.0, 7.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[4.0, 1.0, 8.0], [2.0, 4.0, 7.0], [7.0, 4.0, 2.0], [1.0, 3.0, 1.0]], [[6.0, 8.0, 3.0], 
[6.0, 5.0, 1.0], [1.0, 7.0, 4.0], [3.0, 4.0, 6.0]], [[1.0, 6.0, 8.0], [2.0, 3.0, 7.0], [5.0, 1.0, 5.0], [5.0, 3.0, 2.0]]]))
      val=numpy.array([[2.0, 2.0, 5.0], [6.0, 4.0, 6.0]])
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 3.0], [8.0, 7.0, 6.0], [7.0, 7.0, 2.0], [5.0, 8.0, 7.0]], [[1.0, 7.0, 7.0], [2.0, 
2.0, 5.0], [6.0, 4.0, 6.0], [4.0, 1.0, 5.0]], [[5.0, 7.0, 3.0], [8.0, 4.0, 3.0], [5.0, 2.0, 7.0], [7.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 1.0, 8.0], [2.0, 4.0, 7.0], [7.0, 4.0, 2.0], [1.0, 3.0, 1.0]], [[6.0, 8.0, 3.0], [2.0, 
2.0, 5.0], [6.0, 4.0, 6.0], [3.0, 4.0, 6.0]], [[1.0, 6.0, 8.0], [2.0, 3.0, 7.0], [5.0, 1.0, 5.0], [5.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_PRN(self):
      arg=Data(numpy.array([[[3.0, 2.0, 1.0], [2.0, 6.0, 3.0], [7.0, 8.0, 2.0], [6.0, 5.0, 1.0]], [[8.0, 5.0, 6.0], [1.0, 2.0, 
2.0], [8.0, 5.0, 6.0], [6.0, 1.0, 5.0]], [[5.0, 8.0, 2.0], [3.0, 8.0, 1.0], [3.0, 3.0, 1.0], [6.0, 2.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 2.0, 8.0], [6.0, 1.0, 5.0], [5.0, 6.0, 6.0], [6.0, 1.0, 6.0]], [[6.0, 8.0, 8.0], 
[8.0, 7.0, 6.0], [2.0, 8.0, 4.0], [5.0, 1.0, 1.0]], [[4.0, 1.0, 4.0], [8.0, 8.0, 2.0], [4.0, 8.0, 1.0], [4.0, 7.0, 8.0]]]))
      val=Data(numpy.array([[5.0, 7.0, 2.0], [8.0, 4.0, 3.0]]),self.functionspace)
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 1.0], [2.0, 6.0, 3.0], [7.0, 8.0, 2.0], [6.0, 5.0, 1.0]], [[8.0, 5.0, 6.0], [5.0, 
7.0, 2.0], [8.0, 4.0, 3.0], [6.0, 1.0, 5.0]], [[5.0, 8.0, 2.0], [3.0, 8.0, 1.0], [3.0, 3.0, 1.0], [6.0, 2.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 8.0], [6.0, 1.0, 5.0], [5.0, 6.0, 6.0], [6.0, 1.0, 6.0]], [[6.0, 8.0, 8.0], [5.0, 
7.0, 2.0], [8.0, 4.0, 3.0], [5.0, 1.0, 1.0]], [[4.0, 1.0, 4.0], [8.0, 8.0, 2.0], [4.0, 8.0, 1.0], [4.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_PRN(self):
      arg=Data(numpy.array([[[3.0, 7.0, 2.0], [8.0, 1.0, 6.0], [2.0, 6.0, 3.0], [6.0, 2.0, 7.0]], [[8.0, 2.0, 6.0], [3.0, 8.0, 
2.0], [4.0, 5.0, 3.0], [4.0, 8.0, 3.0]], [[4.0, 5.0, 6.0], [7.0, 8.0, 7.0], [6.0, 2.0, 5.0], [6.0, 1.0, 
3.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 7.0, 8.0], [6.0, 3.0, 3.0], [6.0, 7.0, 4.0], [7.0, 7.0, 1.0]], [[5.0, 6.0, 1.0], 
[6.0, 2.0, 4.0], [8.0, 7.0, 2.0], [4.0, 4.0, 3.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 8.0], [2.0, 1.0, 7.0], [7.0, 8.0, 4.0]]]))
      val=Data(numpy.array([[5.0, 5.0, 6.0], [6.0, 5.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 4.0, 8.0], [6.0, 8.0, 7.0]]))
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 2.0], [8.0, 1.0, 6.0], [2.0, 6.0, 3.0], [6.0, 2.0, 7.0]], [[8.0, 2.0, 6.0], [5.0, 
5.0, 6.0], [6.0, 5.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 5.0, 6.0], [7.0, 8.0, 7.0], [6.0, 2.0, 5.0], [6.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 8.0], [6.0, 3.0, 3.0], [6.0, 7.0, 4.0], [7.0, 7.0, 1.0]], [[5.0, 6.0, 1.0], [6.0, 
4.0, 8.0], [6.0, 8.0, 7.0], [4.0, 4.0, 3.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 8.0], [2.0, 1.0, 7.0], [7.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_PRN(self):
      arg=Data(numpy.array([[[2.0, 2.0, 1.0], [7.0, 2.0, 4.0], [6.0, 5.0, 3.0], [4.0, 7.0, 3.0]], [[7.0, 5.0, 6.0], [5.0, 7.0, 
6.0], [2.0, 8.0, 2.0], [1.0, 8.0, 5.0]], [[4.0, 4.0, 5.0], [1.0, 5.0, 7.0], [5.0, 7.0, 1.0], [1.0, 1.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 3.0, 4.0], [8.0, 6.0, 5.0], [7.0, 6.0, 6.0], [1.0, 8.0, 7.0]], [[1.0, 6.0, 6.0], 
[3.0, 3.0, 6.0], [7.0, 6.0, 7.0], [4.0, 5.0, 5.0]], [[2.0, 4.0, 5.0], [8.0, 6.0, 5.0], [8.0, 5.0, 5.0], [3.0, 6.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[2.0, 7.0, 5.0], [3.0, 2.0, 4.0]])+(1.-msk_val)*numpy.array([[2.0, 6.0, 2.0], [1.0, 7.0, 8.0]])
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 1.0], [7.0, 2.0, 4.0], [6.0, 5.0, 3.0], [4.0, 7.0, 3.0]], [[7.0, 5.0, 6.0], [2.0, 
7.0, 5.0], [3.0, 2.0, 4.0], [1.0, 8.0, 5.0]], [[4.0, 4.0, 5.0], [1.0, 5.0, 7.0], [5.0, 7.0, 1.0], [1.0, 1.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 4.0], [8.0, 6.0, 5.0], [7.0, 6.0, 6.0], [1.0, 8.0, 7.0]], [[1.0, 6.0, 6.0], [2.0, 
6.0, 2.0], [1.0, 7.0, 8.0], [4.0, 5.0, 5.0]], [[2.0, 4.0, 5.0], [8.0, 6.0, 5.0], [8.0, 5.0, 5.0], [3.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_LRN(self):
      arg=Data(numpy.array([[[5.0, 4.0, 6.0], [6.0, 8.0, 4.0], [7.0, 6.0, 2.0], [6.0, 3.0, 2.0]], [[3.0, 2.0, 8.0], [6.0, 2.0, 
5.0], [6.0, 4.0, 4.0], [4.0, 3.0, 1.0]], [[3.0, 2.0, 3.0], [3.0, 3.0, 4.0], [6.0, 7.0, 7.0], [3.0, 2.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[8.0, 8.0, 1.0], [8.0, 1.0, 8.0], [2.0, 4.0, 2.0], [1.0, 5.0, 8.0]], [[5.0, 4.0, 6.0], 
[4.0, 1.0, 2.0], [1.0, 4.0, 4.0], [4.0, 6.0, 5.0]], [[3.0, 2.0, 4.0], [7.0, 8.0, 7.0], [5.0, 7.0, 4.0], [5.0, 3.0, 2.0]]]))
      val=numpy.array([[[5.0, 7.0, 6.0], [4.0, 4.0, 2.0]]])
      arg[2:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 4.0, 6.0], [6.0, 8.0, 4.0], [7.0, 6.0, 2.0], [6.0, 3.0, 2.0]], [[3.0, 2.0, 8.0], [6.0, 
2.0, 5.0], [6.0, 4.0, 4.0], [4.0, 3.0, 1.0]], [[3.0, 2.0, 3.0], [5.0, 7.0, 6.0], [4.0, 4.0, 2.0], [3.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 1.0], [8.0, 1.0, 8.0], [2.0, 4.0, 2.0], [1.0, 5.0, 8.0]], [[5.0, 4.0, 6.0], [4.0, 
1.0, 2.0], [1.0, 4.0, 4.0], [4.0, 6.0, 5.0]], [[3.0, 2.0, 4.0], [5.0, 7.0, 6.0], [4.0, 4.0, 2.0], [5.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_LRN(self):
      arg=Data(numpy.array([[[2.0, 4.0, 3.0], [4.0, 2.0, 4.0], [3.0, 2.0, 4.0], [3.0, 5.0, 7.0]], [[6.0, 3.0, 8.0], [4.0, 3.0, 
5.0], [3.0, 4.0, 2.0], [1.0, 1.0, 4.0]], [[3.0, 5.0, 2.0], [1.0, 8.0, 1.0], [8.0, 2.0, 8.0], [2.0, 4.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[3.0, 1.0, 2.0], [5.0, 6.0, 4.0], [5.0, 4.0, 1.0], [5.0, 2.0, 6.0]], [[4.0, 5.0, 3.0], 
[2.0, 8.0, 7.0], [1.0, 1.0, 8.0], [2.0, 3.0, 4.0]], [[7.0, 7.0, 8.0], [5.0, 6.0, 1.0], [2.0, 6.0, 8.0], [4.0, 5.0, 4.0]]]))
      val=Data(numpy.array([[[2.0, 5.0, 8.0], [6.0, 7.0, 5.0]]]),self.functionspace)
      arg[2:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 3.0], [4.0, 2.0, 4.0], [3.0, 2.0, 4.0], [3.0, 5.0, 7.0]], [[6.0, 3.0, 8.0], [4.0, 
3.0, 5.0], [3.0, 4.0, 2.0], [1.0, 1.0, 4.0]], [[3.0, 5.0, 2.0], [2.0, 5.0, 8.0], [6.0, 7.0, 5.0], [2.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 2.0], [5.0, 6.0, 4.0], [5.0, 4.0, 1.0], [5.0, 2.0, 6.0]], [[4.0, 5.0, 3.0], [2.0, 
8.0, 7.0], [1.0, 1.0, 8.0], [2.0, 3.0, 4.0]], [[7.0, 7.0, 8.0], [2.0, 5.0, 8.0], [6.0, 7.0, 5.0], [4.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_LRN(self):
      arg=Data(numpy.array([[[7.0, 2.0, 5.0], [2.0, 6.0, 4.0], [7.0, 2.0, 8.0], [3.0, 1.0, 5.0]], [[1.0, 8.0, 7.0], [3.0, 5.0, 
2.0], [2.0, 8.0, 4.0], [8.0, 6.0, 7.0]], [[2.0, 2.0, 8.0], [5.0, 4.0, 7.0], [7.0, 5.0, 1.0], [7.0, 5.0, 
5.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 7.0, 7.0], [2.0, 7.0, 3.0], [8.0, 3.0, 3.0], [7.0, 5.0, 6.0]], [[5.0, 6.0, 1.0], 
[4.0, 7.0, 3.0], [2.0, 1.0, 4.0], [3.0, 7.0, 7.0]], [[1.0, 5.0, 6.0], [5.0, 2.0, 6.0], [3.0, 5.0, 8.0], [3.0, 5.0, 1.0]]]))
      val=Data(numpy.array([[[6.0, 6.0, 3.0], [7.0, 4.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 2.0, 3.0], [3.0, 2.0, 8.0]]]))
      arg[2:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 5.0], [2.0, 6.0, 4.0], [7.0, 2.0, 8.0], [3.0, 1.0, 5.0]], [[1.0, 8.0, 7.0], [3.0, 
5.0, 2.0], [2.0, 8.0, 4.0], [8.0, 6.0, 7.0]], [[2.0, 2.0, 8.0], [6.0, 6.0, 3.0], [7.0, 4.0, 8.0], [7.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 7.0, 7.0], [2.0, 7.0, 3.0], [8.0, 3.0, 3.0], [7.0, 5.0, 6.0]], [[5.0, 6.0, 1.0], [4.0, 
7.0, 3.0], [2.0, 1.0, 4.0], [3.0, 7.0, 7.0]], [[1.0, 5.0, 6.0], [3.0, 2.0, 3.0], [3.0, 2.0, 8.0], [3.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_LRN(self):
      arg=Data(numpy.array([[[4.0, 3.0, 4.0], [1.0, 7.0, 3.0], [2.0, 4.0, 5.0], [4.0, 7.0, 4.0]], [[3.0, 4.0, 7.0], [8.0, 4.0, 
7.0], [8.0, 3.0, 7.0], [1.0, 4.0, 3.0]], [[6.0, 3.0, 5.0], [8.0, 1.0, 8.0], [5.0, 1.0, 1.0], [3.0, 3.0, 
1.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 2.0, 5.0], [5.0, 2.0, 4.0], [8.0, 7.0, 3.0], [2.0, 8.0, 7.0]], [[7.0, 5.0, 1.0], 
[2.0, 4.0, 8.0], [4.0, 8.0, 3.0], [2.0, 1.0, 7.0]], [[5.0, 6.0, 6.0], [8.0, 8.0, 3.0], [8.0, 7.0, 2.0], [5.0, 3.0, 4.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 1.0, 7.0], [3.0, 4.0, 2.0]]])+(1.-msk_val)*numpy.array([[[4.0, 3.0, 4.0], [2.0, 8.0, 
6.0]]])
      arg[2:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 4.0], [1.0, 7.0, 3.0], [2.0, 4.0, 5.0], [4.0, 7.0, 4.0]], [[3.0, 4.0, 7.0], [8.0, 
4.0, 7.0], [8.0, 3.0, 7.0], [1.0, 4.0, 3.0]], [[6.0, 3.0, 5.0], [5.0, 1.0, 7.0], [3.0, 4.0, 2.0], [3.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 5.0], [5.0, 2.0, 4.0], [8.0, 7.0, 3.0], [2.0, 8.0, 7.0]], [[7.0, 5.0, 1.0], [2.0, 
4.0, 8.0], [4.0, 8.0, 3.0], [2.0, 1.0, 7.0]], [[5.0, 6.0, 6.0], [4.0, 3.0, 4.0], [2.0, 8.0, 6.0], [5.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_array_UUU(self):
      arg=Data(numpy.array([[[3.0, 3.0, 4.0], [6.0, 8.0, 8.0], [4.0, 3.0, 7.0], [4.0, 3.0, 8.0]], [[1.0, 7.0, 5.0], [4.0, 7.0, 
2.0], [1.0, 3.0, 4.0], [7.0, 8.0, 1.0]], [[6.0, 7.0, 7.0], [6.0, 4.0, 6.0], [6.0, 1.0, 8.0], [4.0, 4.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[2.0, 5.0, 5.0], [1.0, 1.0, 4.0], [5.0, 8.0, 2.0], [8.0, 3.0, 7.0]], [[2.0, 2.0, 5.0], 
[4.0, 3.0, 1.0], [5.0, 2.0, 6.0], [4.0, 6.0, 5.0]], [[7.0, 1.0, 8.0], [5.0, 1.0, 7.0], [7.0, 3.0, 1.0], [4.0, 2.0, 6.0]]]))
      val=numpy.array([[[2.0, 2.0], [3.0, 1.0], [4.0, 7.0]], [[8.0, 6.0], [5.0, 6.0], [7.0, 6.0]]])
      arg[:2,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 4.0], [3.0, 1.0, 8.0], [4.0, 7.0, 7.0], [4.0, 3.0, 8.0]], [[8.0, 6.0, 5.0], [5.0, 
6.0, 2.0], [7.0, 6.0, 4.0], [7.0, 8.0, 1.0]], [[6.0, 7.0, 7.0], [6.0, 4.0, 6.0], [6.0, 1.0, 8.0], [4.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 5.0], [3.0, 1.0, 4.0], [4.0, 7.0, 2.0], [8.0, 3.0, 7.0]], [[8.0, 6.0, 5.0], [5.0, 
6.0, 1.0], [7.0, 6.0, 6.0], [4.0, 6.0, 5.0]], [[7.0, 1.0, 8.0], [5.0, 1.0, 7.0], [7.0, 3.0, 1.0], [4.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_constData_UUU(self):
      arg=Data(numpy.array([[[4.0, 6.0, 6.0], [3.0, 7.0, 4.0], [2.0, 4.0, 1.0], [4.0, 8.0, 7.0]], [[4.0, 6.0, 1.0], [7.0, 3.0, 
6.0], [6.0, 7.0, 6.0], [7.0, 5.0, 6.0]], [[3.0, 5.0, 5.0], [1.0, 1.0, 8.0], [4.0, 4.0, 3.0], [2.0, 4.0, 
2.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[5.0, 2.0, 5.0], [7.0, 1.0, 4.0], [4.0, 7.0, 5.0], [6.0, 7.0, 3.0]], [[2.0, 4.0, 8.0], 
[1.0, 2.0, 3.0], [5.0, 8.0, 2.0], [1.0, 5.0, 7.0]], [[6.0, 6.0, 6.0], [2.0, 3.0, 8.0], [4.0, 6.0, 4.0], [8.0, 8.0, 2.0]]]))
      val=Data(numpy.array([[[5.0, 7.0], [7.0, 7.0], [1.0, 7.0]], [[4.0, 4.0], [2.0, 8.0], [3.0, 4.0]]]),self.functionspace)
      arg[:2,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 6.0], [7.0, 7.0, 4.0], [1.0, 7.0, 1.0], [4.0, 8.0, 7.0]], [[4.0, 4.0, 1.0], [2.0, 
8.0, 6.0], [3.0, 4.0, 6.0], [7.0, 5.0, 6.0]], [[3.0, 5.0, 5.0], [1.0, 1.0, 8.0], [4.0, 4.0, 3.0], [2.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 5.0], [7.0, 7.0, 4.0], [1.0, 7.0, 5.0], [6.0, 7.0, 3.0]], [[4.0, 4.0, 8.0], [2.0, 
8.0, 3.0], [3.0, 4.0, 2.0], [1.0, 5.0, 7.0]], [[6.0, 6.0, 6.0], [2.0, 3.0, 8.0], [4.0, 6.0, 4.0], [8.0, 8.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_taggedData_UUU(self):
      arg=Data(numpy.array([[[2.0, 7.0, 2.0], [7.0, 5.0, 4.0], [6.0, 5.0, 6.0], [2.0, 4.0, 7.0]], [[1.0, 5.0, 8.0], [5.0, 3.0, 
1.0], [3.0, 4.0, 3.0], [1.0, 5.0, 8.0]], [[2.0, 1.0, 3.0], [1.0, 4.0, 3.0], [2.0, 4.0, 8.0], [6.0, 1.0, 
6.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[6.0, 4.0, 5.0], [4.0, 7.0, 2.0], [7.0, 1.0, 2.0], [4.0, 3.0, 1.0]], [[7.0, 1.0, 6.0], 
[7.0, 1.0, 1.0], [8.0, 8.0, 7.0], [8.0, 5.0, 2.0]], [[4.0, 2.0, 6.0], [3.0, 6.0, 2.0], [5.0, 2.0, 3.0], [4.0, 5.0, 3.0]]]))
      val=Data(numpy.array([[[3.0, 1.0], [8.0, 8.0], [7.0, 1.0]], [[3.0, 7.0], [2.0, 6.0], [1.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 5.0], [6.0, 8.0], [1.0, 5.0]], [[7.0, 7.0], [6.0, 4.0], [4.0, 5.0]]]))
      arg[:2,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 2.0], [8.0, 8.0, 4.0], [7.0, 1.0, 6.0], [2.0, 4.0, 7.0]], [[3.0, 7.0, 8.0], [2.0, 
6.0, 1.0], [1.0, 3.0, 3.0], [1.0, 5.0, 8.0]], [[2.0, 1.0, 3.0], [1.0, 4.0, 3.0], [2.0, 4.0, 8.0], [6.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 5.0], [6.0, 8.0, 2.0], [1.0, 5.0, 2.0], [4.0, 3.0, 1.0]], [[7.0, 7.0, 6.0], [6.0, 
4.0, 1.0], [4.0, 5.0, 7.0], [8.0, 5.0, 2.0]], [[4.0, 2.0, 6.0], [3.0, 6.0, 2.0], [5.0, 2.0, 3.0], [4.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank3_expandedData_UUU(self):
      arg=Data(numpy.array([[[7.0, 6.0, 7.0], [3.0, 1.0, 6.0], [3.0, 2.0, 4.0], [5.0, 1.0, 3.0]], [[3.0, 4.0, 5.0], [8.0, 8.0, 
2.0], [2.0, 1.0, 7.0], [3.0, 4.0, 3.0]], [[2.0, 6.0, 8.0], [7.0, 7.0, 5.0], [8.0, 5.0, 6.0], [7.0, 3.0, 
7.0]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[1.0, 5.0, 1.0], [1.0, 6.0, 2.0], [5.0, 1.0, 3.0], [4.0, 4.0, 4.0]], [[5.0, 2.0, 1.0], 
[7.0, 4.0, 6.0], [3.0, 7.0, 8.0], [8.0, 2.0, 6.0]], [[3.0, 2.0, 3.0], [3.0, 6.0, 2.0], [5.0, 5.0, 3.0], [6.0, 1.0, 7.0]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 1.0], [2.0, 6.0], [6.0, 2.0]], [[7.0, 4.0], [8.0, 7.0], [6.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[4.0, 5.0], [7.0, 2.0], [7.0, 5.0]], [[6.0, 5.0], [4.0, 8.0], [7.0, 4.0]]])
      arg[:2,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 1.0, 7.0], [2.0, 6.0, 6.0], [6.0, 2.0, 4.0], [5.0, 1.0, 3.0]], [[7.0, 4.0, 5.0], [8.0, 
7.0, 2.0], [6.0, 1.0, 7.0], [3.0, 4.0, 3.0]], [[2.0, 6.0, 8.0], [7.0, 7.0, 5.0], [8.0, 5.0, 6.0], [7.0, 3.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 1.0], [7.0, 2.0, 2.0], [7.0, 5.0, 3.0], [4.0, 4.0, 4.0]], [[6.0, 5.0, 1.0], [4.0, 
8.0, 6.0], [7.0, 4.0, 8.0], [8.0, 2.0, 6.0]], [[3.0, 2.0, 3.0], [3.0, 6.0, 2.0], [5.0, 5.0, 3.0], [6.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPUP(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 4.0], [1.0, 1.0, 7.0], [8.0, 7.0, 8.0]], [[3.0, 1.0, 2.0], [8.0, 2.0, 7.0], [3.0, 6.0, 
5.0]], [[4.0, 2.0, 1.0], [8.0, 8.0, 7.0], [7.0, 3.0, 4.0]]], [[[3.0, 8.0, 8.0], [4.0, 2.0, 7.0], [7.0, 2.0, 8.0]], [[1.0, 6.0, 
1.0], [8.0, 6.0, 7.0], [5.0, 6.0, 4.0]], [[8.0, 5.0, 5.0], [5.0, 4.0, 7.0], [1.0, 2.0, 1.0]]], [[[2.0, 2.0, 8.0], [2.0, 3.0, 
3.0], [7.0, 4.0, 8.0]], [[1.0, 6.0, 4.0], [5.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[4.0, 5.0, 8.0], [4.0, 6.0, 1.0], [8.0, 6.0, 
8.0]]], [[[3.0, 3.0, 6.0], [7.0, 6.0, 6.0], [8.0, 8.0, 5.0]], [[5.0, 1.0, 8.0], [1.0, 5.0, 1.0], [2.0, 4.0, 7.0]], [[3.0, 1.0, 
4.0], [7.0, 2.0, 4.0], [5.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 7.0], [4.0, 2.0, 2.0], [8.0, 5.0, 3.0]], [[2.0, 2.0, 6.0], [4.0, 7.0, 
4.0], [5.0, 3.0, 6.0]], [[4.0, 4.0, 3.0], [3.0, 1.0, 2.0], [8.0, 8.0, 1.0]]], [[[7.0, 8.0, 2.0], [2.0, 3.0, 1.0], [2.0, 6.0, 
8.0]], [[6.0, 2.0, 5.0], [2.0, 5.0, 2.0], [3.0, 7.0, 3.0]], [[1.0, 2.0, 7.0], [4.0, 2.0, 7.0], [2.0, 2.0, 5.0]]], [[[6.0, 7.0, 
1.0], [5.0, 6.0, 5.0], [8.0, 7.0, 8.0]], [[7.0, 4.0, 5.0], [1.0, 3.0, 7.0], [2.0, 6.0, 4.0]], [[5.0, 8.0, 6.0], [3.0, 1.0, 
7.0], [5.0, 4.0, 8.0]]], [[[3.0, 8.0, 5.0], [4.0, 5.0, 1.0], [1.0, 3.0, 4.0]], [[2.0, 1.0, 1.0], [8.0, 1.0, 4.0], [4.0, 6.0, 
8.0]], [[8.0, 4.0, 6.0], [4.0, 5.0, 8.0], [6.0, 7.0, 4.0]]]]))
      val=numpy.array([[7.0, 3.0], [1.0, 8.0], [5.0, 2.0], [6.0, 3.0]])
      arg[:,1,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 4.0], [1.0, 1.0, 7.0], [8.0, 7.0, 8.0]], [[3.0, 7.0, 2.0], [8.0, 3.0, 7.0], [3.0, 
6.0, 5.0]], [[4.0, 2.0, 1.0], [8.0, 8.0, 7.0], [7.0, 3.0, 4.0]]], [[[3.0, 8.0, 8.0], [4.0, 2.0, 7.0], [7.0, 2.0, 8.0]], [[1.0, 
1.0, 1.0], [8.0, 8.0, 7.0], [5.0, 6.0, 4.0]], [[8.0, 5.0, 5.0], [5.0, 4.0, 7.0], [1.0, 2.0, 1.0]]], [[[2.0, 2.0, 8.0], [2.0, 
3.0, 3.0], [7.0, 4.0, 8.0]], [[1.0, 5.0, 4.0], [5.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[4.0, 5.0, 8.0], [4.0, 6.0, 1.0], [8.0, 6.0, 
8.0]]], [[[3.0, 3.0, 6.0], [7.0, 6.0, 6.0], [8.0, 8.0, 5.0]], [[5.0, 6.0, 8.0], [1.0, 3.0, 1.0], [2.0, 4.0, 7.0]], [[3.0, 1.0, 
4.0], [7.0, 2.0, 4.0], [5.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 7.0], [4.0, 2.0, 2.0], [8.0, 5.0, 3.0]], 
[[2.0, 7.0, 6.0], [4.0, 3.0, 4.0], [5.0, 3.0, 6.0]], [[4.0, 4.0, 3.0], [3.0, 1.0, 2.0], [8.0, 8.0, 1.0]]], [[[7.0, 8.0, 2.0], 
[2.0, 3.0, 1.0], [2.0, 6.0, 8.0]], [[6.0, 1.0, 5.0], [2.0, 8.0, 2.0], [3.0, 7.0, 3.0]], [[1.0, 2.0, 7.0], [4.0, 2.0, 7.0], 
[2.0, 2.0, 5.0]]], [[[6.0, 7.0, 1.0], [5.0, 6.0, 5.0], [8.0, 7.0, 8.0]], [[7.0, 5.0, 5.0], [1.0, 2.0, 7.0], [2.0, 6.0, 4.0]], 
[[5.0, 8.0, 6.0], [3.0, 1.0, 7.0], [5.0, 4.0, 8.0]]], [[[3.0, 8.0, 5.0], [4.0, 5.0, 1.0], [1.0, 3.0, 4.0]], [[2.0, 6.0, 1.0], 
[8.0, 3.0, 4.0], [4.0, 6.0, 8.0]], [[8.0, 4.0, 6.0], [4.0, 5.0, 8.0], [6.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPUP(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 2.0], [6.0, 3.0, 8.0], [3.0, 4.0, 1.0]], [[8.0, 5.0, 7.0], [5.0, 5.0, 6.0], [2.0, 7.0, 
5.0]], [[1.0, 6.0, 8.0], [6.0, 7.0, 1.0], [8.0, 2.0, 8.0]]], [[[8.0, 4.0, 6.0], [5.0, 8.0, 7.0], [4.0, 2.0, 2.0]], [[5.0, 3.0, 
1.0], [5.0, 5.0, 7.0], [8.0, 5.0, 6.0]], [[1.0, 4.0, 2.0], [6.0, 1.0, 2.0], [2.0, 5.0, 1.0]]], [[[6.0, 2.0, 4.0], [7.0, 7.0, 
5.0], [1.0, 3.0, 8.0]], [[1.0, 3.0, 1.0], [5.0, 1.0, 4.0], [6.0, 1.0, 3.0]], [[1.0, 7.0, 3.0], [7.0, 3.0, 7.0], [5.0, 8.0, 
8.0]]], [[[1.0, 1.0, 5.0], [3.0, 6.0, 1.0], [3.0, 7.0, 2.0]], [[8.0, 2.0, 7.0], [8.0, 6.0, 4.0], [3.0, 4.0, 6.0]], [[8.0, 3.0, 
4.0], [5.0, 1.0, 3.0], [2.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 5.0], [6.0, 3.0, 5.0], [6.0, 3.0, 1.0]], [[7.0, 5.0, 4.0], [4.0, 8.0, 
2.0], [3.0, 3.0, 2.0]], [[3.0, 8.0, 8.0], [3.0, 7.0, 3.0], [1.0, 8.0, 7.0]]], [[[5.0, 4.0, 3.0], [1.0, 3.0, 4.0], [3.0, 1.0, 
4.0]], [[7.0, 1.0, 5.0], [3.0, 5.0, 4.0], [7.0, 7.0, 5.0]], [[4.0, 2.0, 5.0], [1.0, 8.0, 3.0], [7.0, 4.0, 2.0]]], [[[1.0, 6.0, 
3.0], [4.0, 2.0, 8.0], [5.0, 5.0, 8.0]], [[8.0, 5.0, 8.0], [8.0, 4.0, 6.0], [4.0, 3.0, 7.0]], [[2.0, 2.0, 7.0], [3.0, 7.0, 
3.0], [6.0, 6.0, 6.0]]], [[[4.0, 3.0, 3.0], [8.0, 1.0, 1.0], [7.0, 5.0, 4.0]], [[8.0, 4.0, 7.0], [8.0, 2.0, 4.0], [7.0, 4.0, 
6.0]], [[3.0, 8.0, 7.0], [4.0, 7.0, 8.0], [2.0, 3.0, 8.0]]]]))
      val=Data(numpy.array([[6.0, 1.0], [2.0, 7.0], [7.0, 8.0], [8.0, 7.0]]),self.functionspace)
      arg[:,1,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 2.0], [6.0, 3.0, 8.0], [3.0, 4.0, 1.0]], [[8.0, 6.0, 7.0], [5.0, 1.0, 6.0], [2.0, 
7.0, 5.0]], [[1.0, 6.0, 8.0], [6.0, 7.0, 1.0], [8.0, 2.0, 8.0]]], [[[8.0, 4.0, 6.0], [5.0, 8.0, 7.0], [4.0, 2.0, 2.0]], [[5.0, 
2.0, 1.0], [5.0, 7.0, 7.0], [8.0, 5.0, 6.0]], [[1.0, 4.0, 2.0], [6.0, 1.0, 2.0], [2.0, 5.0, 1.0]]], [[[6.0, 2.0, 4.0], [7.0, 
7.0, 5.0], [1.0, 3.0, 8.0]], [[1.0, 7.0, 1.0], [5.0, 8.0, 4.0], [6.0, 1.0, 3.0]], [[1.0, 7.0, 3.0], [7.0, 3.0, 7.0], [5.0, 8.0, 
8.0]]], [[[1.0, 1.0, 5.0], [3.0, 6.0, 1.0], [3.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [8.0, 7.0, 4.0], [3.0, 4.0, 6.0]], [[8.0, 3.0, 
4.0], [5.0, 1.0, 3.0], [2.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 5.0], [6.0, 3.0, 5.0], [6.0, 3.0, 1.0]], 
[[7.0, 6.0, 4.0], [4.0, 1.0, 2.0], [3.0, 3.0, 2.0]], [[3.0, 8.0, 8.0], [3.0, 7.0, 3.0], [1.0, 8.0, 7.0]]], [[[5.0, 4.0, 3.0], 
[1.0, 3.0, 4.0], [3.0, 1.0, 4.0]], [[7.0, 2.0, 5.0], [3.0, 7.0, 4.0], [7.0, 7.0, 5.0]], [[4.0, 2.0, 5.0], [1.0, 8.0, 3.0], 
[7.0, 4.0, 2.0]]], [[[1.0, 6.0, 3.0], [4.0, 2.0, 8.0], [5.0, 5.0, 8.0]], [[8.0, 7.0, 8.0], [8.0, 8.0, 6.0], [4.0, 3.0, 7.0]], 
[[2.0, 2.0, 7.0], [3.0, 7.0, 3.0], [6.0, 6.0, 6.0]]], [[[4.0, 3.0, 3.0], [8.0, 1.0, 1.0], [7.0, 5.0, 4.0]], [[8.0, 8.0, 7.0], 
[8.0, 7.0, 4.0], [7.0, 4.0, 6.0]], [[3.0, 8.0, 7.0], [4.0, 7.0, 8.0], [2.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPUP(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 6.0], [7.0, 2.0, 8.0], [6.0, 3.0, 6.0]], [[5.0, 3.0, 1.0], [6.0, 3.0, 2.0], [3.0, 1.0, 
1.0]], [[1.0, 2.0, 6.0], [1.0, 3.0, 4.0], [8.0, 8.0, 8.0]]], [[[3.0, 3.0, 4.0], [3.0, 3.0, 6.0], [5.0, 2.0, 7.0]], [[6.0, 7.0, 
3.0], [8.0, 8.0, 5.0], [6.0, 6.0, 3.0]], [[3.0, 2.0, 3.0], [7.0, 2.0, 5.0], [4.0, 7.0, 7.0]]], [[[1.0, 3.0, 7.0], [8.0, 5.0, 
7.0], [1.0, 1.0, 7.0]], [[5.0, 5.0, 8.0], [4.0, 3.0, 5.0], [2.0, 1.0, 7.0]], [[2.0, 2.0, 1.0], [8.0, 4.0, 4.0], [3.0, 8.0, 
2.0]]], [[[4.0, 6.0, 1.0], [7.0, 8.0, 4.0], [5.0, 7.0, 7.0]], [[6.0, 6.0, 7.0], [4.0, 1.0, 1.0], [1.0, 7.0, 4.0]], [[1.0, 7.0, 
1.0], [2.0, 1.0, 8.0], [2.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 1.0], [5.0, 2.0, 1.0], [5.0, 2.0, 6.0]], [[4.0, 4.0, 4.0], [7.0, 7.0, 
6.0], [1.0, 4.0, 6.0]], [[4.0, 7.0, 7.0], [8.0, 7.0, 6.0], [7.0, 5.0, 5.0]]], [[[4.0, 3.0, 3.0], [6.0, 4.0, 8.0], [5.0, 8.0, 
6.0]], [[5.0, 8.0, 4.0], [1.0, 6.0, 8.0], [5.0, 4.0, 5.0]], [[1.0, 7.0, 5.0], [4.0, 3.0, 8.0], [1.0, 6.0, 5.0]]], [[[8.0, 7.0, 
4.0], [6.0, 3.0, 8.0], [6.0, 2.0, 1.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 7.0], [4.0, 8.0, 7.0]], [[8.0, 3.0, 3.0], [2.0, 8.0, 
7.0], [5.0, 6.0, 5.0]]], [[[5.0, 7.0, 2.0], [3.0, 6.0, 6.0], [4.0, 8.0, 4.0]], [[8.0, 5.0, 8.0], [5.0, 3.0, 5.0], [2.0, 4.0, 
4.0]], [[4.0, 6.0, 4.0], [7.0, 5.0, 6.0], [6.0, 4.0, 6.0]]]]))
      val=Data(numpy.array([[6.0, 4.0], [6.0, 7.0], [8.0, 3.0], [3.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 1.0], [7.0, 7.0], [5.0, 6.0], [6.0, 7.0]]))
      arg[:,1,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 6.0], [7.0, 2.0, 8.0], [6.0, 3.0, 6.0]], [[5.0, 6.0, 1.0], [6.0, 4.0, 2.0], [3.0, 
1.0, 1.0]], [[1.0, 2.0, 6.0], [1.0, 3.0, 4.0], [8.0, 8.0, 8.0]]], [[[3.0, 3.0, 4.0], [3.0, 3.0, 6.0], [5.0, 2.0, 7.0]], [[6.0, 
6.0, 3.0], [8.0, 7.0, 5.0], [6.0, 6.0, 3.0]], [[3.0, 2.0, 3.0], [7.0, 2.0, 5.0], [4.0, 7.0, 7.0]]], [[[1.0, 3.0, 7.0], [8.0, 
5.0, 7.0], [1.0, 1.0, 7.0]], [[5.0, 8.0, 8.0], [4.0, 3.0, 5.0], [2.0, 1.0, 7.0]], [[2.0, 2.0, 1.0], [8.0, 4.0, 4.0], [3.0, 8.0, 
2.0]]], [[[4.0, 6.0, 1.0], [7.0, 8.0, 4.0], [5.0, 7.0, 7.0]], [[6.0, 3.0, 7.0], [4.0, 7.0, 1.0], [1.0, 7.0, 4.0]], [[1.0, 7.0, 
1.0], [2.0, 1.0, 8.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 1.0], [5.0, 2.0, 1.0], [5.0, 2.0, 6.0]], 
[[4.0, 3.0, 4.0], [7.0, 1.0, 6.0], [1.0, 4.0, 6.0]], [[4.0, 7.0, 7.0], [8.0, 7.0, 6.0], [7.0, 5.0, 5.0]]], [[[4.0, 3.0, 3.0], 
[6.0, 4.0, 8.0], [5.0, 8.0, 6.0]], [[5.0, 7.0, 4.0], [1.0, 7.0, 8.0], [5.0, 4.0, 5.0]], [[1.0, 7.0, 5.0], [4.0, 3.0, 8.0], 
[1.0, 6.0, 5.0]]], [[[8.0, 7.0, 4.0], [6.0, 3.0, 8.0], [6.0, 2.0, 1.0]], [[7.0, 5.0, 8.0], [1.0, 6.0, 7.0], [4.0, 8.0, 7.0]], 
[[8.0, 3.0, 3.0], [2.0, 8.0, 7.0], [5.0, 6.0, 5.0]]], [[[5.0, 7.0, 2.0], [3.0, 6.0, 6.0], [4.0, 8.0, 4.0]], [[8.0, 6.0, 8.0], 
[5.0, 7.0, 5.0], [2.0, 4.0, 4.0]], [[4.0, 6.0, 4.0], [7.0, 5.0, 6.0], [6.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPUP(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 7.0], [2.0, 5.0, 8.0], [5.0, 6.0, 5.0]], [[2.0, 3.0, 4.0], [5.0, 5.0, 4.0], [2.0, 5.0, 
5.0]], [[8.0, 5.0, 7.0], [4.0, 7.0, 2.0], [6.0, 8.0, 4.0]]], [[[3.0, 8.0, 6.0], [2.0, 3.0, 3.0], [8.0, 5.0, 5.0]], [[5.0, 1.0, 
1.0], [1.0, 3.0, 3.0], [4.0, 6.0, 7.0]], [[2.0, 2.0, 5.0], [1.0, 8.0, 7.0], [5.0, 2.0, 8.0]]], [[[2.0, 5.0, 8.0], [6.0, 6.0, 
1.0], [6.0, 4.0, 1.0]], [[8.0, 2.0, 4.0], [2.0, 3.0, 1.0], [7.0, 3.0, 8.0]], [[1.0, 8.0, 3.0], [1.0, 3.0, 4.0], [5.0, 4.0, 
6.0]]], [[[2.0, 5.0, 4.0], [8.0, 5.0, 1.0], [7.0, 5.0, 4.0]], [[4.0, 4.0, 6.0], [5.0, 1.0, 5.0], [2.0, 4.0, 4.0]], [[4.0, 5.0, 
2.0], [1.0, 3.0, 6.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 7.0], [1.0, 4.0, 6.0], [3.0, 7.0, 1.0]], [[3.0, 2.0, 7.0], [4.0, 6.0, 
1.0], [3.0, 2.0, 6.0]], [[1.0, 5.0, 4.0], [3.0, 6.0, 6.0], [3.0, 8.0, 6.0]]], [[[2.0, 3.0, 6.0], [6.0, 4.0, 1.0], [1.0, 5.0, 
2.0]], [[6.0, 5.0, 2.0], [6.0, 4.0, 5.0], [3.0, 4.0, 6.0]], [[7.0, 3.0, 5.0], [1.0, 5.0, 7.0], [5.0, 2.0, 6.0]]], [[[8.0, 3.0, 
7.0], [5.0, 1.0, 4.0], [7.0, 2.0, 2.0]], [[4.0, 8.0, 8.0], [4.0, 2.0, 7.0], [8.0, 1.0, 5.0]], [[7.0, 1.0, 2.0], [8.0, 3.0, 
7.0], [6.0, 1.0, 7.0]]], [[[5.0, 7.0, 4.0], [8.0, 2.0, 2.0], [1.0, 8.0, 2.0]], [[3.0, 7.0, 7.0], [3.0, 5.0, 7.0], [1.0, 5.0, 
1.0]], [[1.0, 6.0, 6.0], [5.0, 5.0, 3.0], [8.0, 1.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 3.0], [7.0, 7.0], [5.0, 8.0], [2.0, 1.0]])+(1.-msk_val)*numpy.array([[2.0, 5.0], [5.0, 
8.0], [4.0, 7.0], [8.0, 4.0]])
      arg[:,1,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 7.0], [2.0, 5.0, 8.0], [5.0, 6.0, 5.0]], [[2.0, 6.0, 4.0], [5.0, 3.0, 4.0], [2.0, 
5.0, 5.0]], [[8.0, 5.0, 7.0], [4.0, 7.0, 2.0], [6.0, 8.0, 4.0]]], [[[3.0, 8.0, 6.0], [2.0, 3.0, 3.0], [8.0, 5.0, 5.0]], [[5.0, 
7.0, 1.0], [1.0, 7.0, 3.0], [4.0, 6.0, 7.0]], [[2.0, 2.0, 5.0], [1.0, 8.0, 7.0], [5.0, 2.0, 8.0]]], [[[2.0, 5.0, 8.0], [6.0, 
6.0, 1.0], [6.0, 4.0, 1.0]], [[8.0, 5.0, 4.0], [2.0, 8.0, 1.0], [7.0, 3.0, 8.0]], [[1.0, 8.0, 3.0], [1.0, 3.0, 4.0], [5.0, 4.0, 
6.0]]], [[[2.0, 5.0, 4.0], [8.0, 5.0, 1.0], [7.0, 5.0, 4.0]], [[4.0, 2.0, 6.0], [5.0, 1.0, 5.0], [2.0, 4.0, 4.0]], [[4.0, 5.0, 
2.0], [1.0, 3.0, 6.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 7.0], [1.0, 4.0, 6.0], [3.0, 7.0, 1.0]], 
[[3.0, 2.0, 7.0], [4.0, 5.0, 1.0], [3.0, 2.0, 6.0]], [[1.0, 5.0, 4.0], [3.0, 6.0, 6.0], [3.0, 8.0, 6.0]]], [[[2.0, 3.0, 6.0], 
[6.0, 4.0, 1.0], [1.0, 5.0, 2.0]], [[6.0, 5.0, 2.0], [6.0, 8.0, 5.0], [3.0, 4.0, 6.0]], [[7.0, 3.0, 5.0], [1.0, 5.0, 7.0], 
[5.0, 2.0, 6.0]]], [[[8.0, 3.0, 7.0], [5.0, 1.0, 4.0], [7.0, 2.0, 2.0]], [[4.0, 4.0, 8.0], [4.0, 7.0, 7.0], [8.0, 1.0, 5.0]], 
[[7.0, 1.0, 2.0], [8.0, 3.0, 7.0], [6.0, 1.0, 7.0]]], [[[5.0, 7.0, 4.0], [8.0, 2.0, 2.0], [1.0, 8.0, 2.0]], [[3.0, 8.0, 7.0], 
[3.0, 4.0, 7.0], [1.0, 5.0, 1.0]], [[1.0, 6.0, 6.0], [5.0, 5.0, 3.0], [8.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLRR(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 2.0], [6.0, 3.0, 8.0], [8.0, 5.0, 2.0]], [[3.0, 7.0, 4.0], [8.0, 8.0, 1.0], [8.0, 4.0, 
1.0]], [[3.0, 8.0, 2.0], [1.0, 1.0, 7.0], [8.0, 4.0, 8.0]]], [[[3.0, 4.0, 7.0], [4.0, 3.0, 2.0], [7.0, 7.0, 2.0]], [[4.0, 2.0, 
8.0], [8.0, 5.0, 7.0], [5.0, 6.0, 7.0]], [[4.0, 4.0, 3.0], [2.0, 3.0, 3.0], [1.0, 1.0, 4.0]]], [[[7.0, 5.0, 3.0], [2.0, 6.0, 
4.0], [5.0, 3.0, 1.0]], [[4.0, 7.0, 3.0], [2.0, 4.0, 2.0], [2.0, 5.0, 1.0]], [[6.0, 3.0, 4.0], [2.0, 6.0, 2.0], [7.0, 8.0, 
7.0]]], [[[7.0, 1.0, 3.0], [7.0, 2.0, 8.0], [7.0, 8.0, 8.0]], [[6.0, 4.0, 1.0], [6.0, 3.0, 3.0], [2.0, 1.0, 4.0]], [[5.0, 6.0, 
7.0], [3.0, 1.0, 5.0], [8.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 1.0], [3.0, 7.0, 1.0], [7.0, 5.0, 4.0]], [[2.0, 1.0, 7.0], [7.0, 5.0, 
5.0], [2.0, 5.0, 4.0]], [[8.0, 6.0, 8.0], [7.0, 4.0, 2.0], [4.0, 8.0, 4.0]]], [[[3.0, 6.0, 5.0], [3.0, 3.0, 4.0], [2.0, 6.0, 
8.0]], [[8.0, 3.0, 2.0], [1.0, 8.0, 4.0], [3.0, 8.0, 3.0]], [[1.0, 6.0, 6.0], [4.0, 4.0, 4.0], [1.0, 3.0, 3.0]]], [[[4.0, 3.0, 
3.0], [7.0, 5.0, 5.0], [8.0, 4.0, 4.0]], [[7.0, 5.0, 8.0], [1.0, 2.0, 1.0], [4.0, 8.0, 4.0]], [[3.0, 8.0, 6.0], [6.0, 5.0, 
3.0], [7.0, 5.0, 5.0]]], [[[8.0, 7.0, 1.0], [6.0, 1.0, 7.0], [4.0, 3.0, 6.0]], [[2.0, 2.0, 8.0], [4.0, 5.0, 4.0], [2.0, 8.0, 
2.0]], [[3.0, 7.0, 5.0], [7.0, 4.0, 7.0], [3.0, 4.0, 2.0]]]]))
      val=numpy.array([[[[6.0]]], [[[1.0]]]])
      arg[2:,2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 2.0], [6.0, 3.0, 8.0], [8.0, 5.0, 2.0]], [[3.0, 7.0, 4.0], [8.0, 8.0, 1.0], [8.0, 
4.0, 1.0]], [[3.0, 8.0, 2.0], [1.0, 1.0, 7.0], [8.0, 4.0, 8.0]]], [[[3.0, 4.0, 7.0], [4.0, 3.0, 2.0], [7.0, 7.0, 2.0]], [[4.0, 
2.0, 8.0], [8.0, 5.0, 7.0], [5.0, 6.0, 7.0]], [[4.0, 4.0, 3.0], [2.0, 3.0, 3.0], [1.0, 1.0, 4.0]]], [[[7.0, 5.0, 3.0], [2.0, 
6.0, 4.0], [5.0, 3.0, 1.0]], [[4.0, 7.0, 3.0], [2.0, 4.0, 2.0], [2.0, 5.0, 1.0]], [[6.0, 3.0, 4.0], [2.0, 6.0, 2.0], [7.0, 8.0, 
7.0]]], [[[7.0, 1.0, 3.0], [7.0, 2.0, 8.0], [7.0, 8.0, 8.0]], [[6.0, 4.0, 1.0], [6.0, 3.0, 3.0], [2.0, 1.0, 4.0]], [[5.0, 6.0, 
7.0], [3.0, 1.0, 5.0], [8.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 1.0], [3.0, 7.0, 1.0], [7.0, 5.0, 4.0]], 
[[2.0, 1.0, 7.0], [7.0, 5.0, 5.0], [2.0, 5.0, 4.0]], [[8.0, 6.0, 8.0], [7.0, 4.0, 2.0], [4.0, 8.0, 4.0]]], [[[3.0, 6.0, 5.0], 
[3.0, 3.0, 4.0], [2.0, 6.0, 8.0]], [[8.0, 3.0, 2.0], [1.0, 8.0, 4.0], [3.0, 8.0, 3.0]], [[1.0, 6.0, 6.0], [4.0, 4.0, 4.0], 
[1.0, 3.0, 3.0]]], [[[4.0, 3.0, 3.0], [7.0, 5.0, 5.0], [8.0, 4.0, 4.0]], [[7.0, 5.0, 8.0], [1.0, 2.0, 1.0], [4.0, 8.0, 4.0]], 
[[3.0, 8.0, 6.0], [6.0, 6.0, 3.0], [7.0, 5.0, 5.0]]], [[[8.0, 7.0, 1.0], [6.0, 1.0, 7.0], [4.0, 3.0, 6.0]], [[2.0, 2.0, 8.0], 
[4.0, 5.0, 4.0], [2.0, 8.0, 2.0]], [[3.0, 7.0, 5.0], [7.0, 1.0, 7.0], [3.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLRR(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 6.0], [4.0, 8.0, 3.0], [7.0, 1.0, 3.0]], [[4.0, 5.0, 5.0], [5.0, 1.0, 8.0], [1.0, 3.0, 
8.0]], [[8.0, 4.0, 6.0], [8.0, 7.0, 2.0], [6.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], [4.0, 6.0, 1.0], [3.0, 5.0, 7.0]], [[3.0, 2.0, 
7.0], [1.0, 8.0, 1.0], [6.0, 6.0, 8.0]], [[8.0, 2.0, 5.0], [1.0, 1.0, 2.0], [7.0, 4.0, 5.0]]], [[[7.0, 1.0, 5.0], [2.0, 1.0, 
2.0], [2.0, 1.0, 6.0]], [[5.0, 3.0, 7.0], [8.0, 3.0, 1.0], [1.0, 8.0, 2.0]], [[1.0, 1.0, 1.0], [6.0, 7.0, 3.0], [6.0, 7.0, 
3.0]]], [[[5.0, 1.0, 3.0], [4.0, 6.0, 6.0], [3.0, 4.0, 6.0]], [[6.0, 6.0, 5.0], [8.0, 3.0, 3.0], [1.0, 1.0, 6.0]], [[2.0, 3.0, 
3.0], [1.0, 5.0, 1.0], [7.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 3.0], [8.0, 8.0, 5.0], [8.0, 6.0, 2.0]], [[4.0, 1.0, 5.0], [1.0, 8.0, 
4.0], [3.0, 1.0, 6.0]], [[8.0, 1.0, 4.0], [1.0, 7.0, 3.0], [6.0, 1.0, 2.0]]], [[[6.0, 7.0, 5.0], [1.0, 7.0, 7.0], [5.0, 6.0, 
1.0]], [[8.0, 1.0, 7.0], [8.0, 8.0, 2.0], [3.0, 6.0, 4.0]], [[8.0, 6.0, 2.0], [8.0, 1.0, 7.0], [8.0, 3.0, 4.0]]], [[[5.0, 4.0, 
2.0], [3.0, 5.0, 2.0], [5.0, 2.0, 5.0]], [[4.0, 3.0, 6.0], [3.0, 4.0, 3.0], [2.0, 1.0, 1.0]], [[6.0, 8.0, 1.0], [1.0, 5.0, 
1.0], [5.0, 2.0, 1.0]]], [[[8.0, 4.0, 2.0], [2.0, 2.0, 4.0], [5.0, 6.0, 7.0]], [[2.0, 5.0, 8.0], [1.0, 1.0, 5.0], [6.0, 3.0, 
7.0]], [[1.0, 5.0, 8.0], [8.0, 8.0, 3.0], [5.0, 2.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0]]], [[[3.0]]]]),self.functionspace)
      arg[2:,2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 6.0], [4.0, 8.0, 3.0], [7.0, 1.0, 3.0]], [[4.0, 5.0, 5.0], [5.0, 1.0, 8.0], [1.0, 
3.0, 8.0]], [[8.0, 4.0, 6.0], [8.0, 7.0, 2.0], [6.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], [4.0, 6.0, 1.0], [3.0, 5.0, 7.0]], [[3.0, 
2.0, 7.0], [1.0, 8.0, 1.0], [6.0, 6.0, 8.0]], [[8.0, 2.0, 5.0], [1.0, 1.0, 2.0], [7.0, 4.0, 5.0]]], [[[7.0, 1.0, 5.0], [2.0, 
1.0, 2.0], [2.0, 1.0, 6.0]], [[5.0, 3.0, 7.0], [8.0, 3.0, 1.0], [1.0, 8.0, 2.0]], [[1.0, 1.0, 1.0], [6.0, 5.0, 3.0], [6.0, 7.0, 
3.0]]], [[[5.0, 1.0, 3.0], [4.0, 6.0, 6.0], [3.0, 4.0, 6.0]], [[6.0, 6.0, 5.0], [8.0, 3.0, 3.0], [1.0, 1.0, 6.0]], [[2.0, 3.0, 
3.0], [1.0, 3.0, 1.0], [7.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 3.0], [8.0, 8.0, 5.0], [8.0, 6.0, 2.0]], 
[[4.0, 1.0, 5.0], [1.0, 8.0, 4.0], [3.0, 1.0, 6.0]], [[8.0, 1.0, 4.0], [1.0, 7.0, 3.0], [6.0, 1.0, 2.0]]], [[[6.0, 7.0, 5.0], 
[1.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[8.0, 1.0, 7.0], [8.0, 8.0, 2.0], [3.0, 6.0, 4.0]], [[8.0, 6.0, 2.0], [8.0, 1.0, 7.0], 
[8.0, 3.0, 4.0]]], [[[5.0, 4.0, 2.0], [3.0, 5.0, 2.0], [5.0, 2.0, 5.0]], [[4.0, 3.0, 6.0], [3.0, 4.0, 3.0], [2.0, 1.0, 1.0]], 
[[6.0, 8.0, 1.0], [1.0, 5.0, 1.0], [5.0, 2.0, 1.0]]], [[[8.0, 4.0, 2.0], [2.0, 2.0, 4.0], [5.0, 6.0, 7.0]], [[2.0, 5.0, 8.0], 
[1.0, 1.0, 5.0], [6.0, 3.0, 7.0]], [[1.0, 5.0, 8.0], [8.0, 3.0, 3.0], [5.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLRR(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 1.0], [7.0, 2.0, 2.0], [5.0, 8.0, 3.0]], [[7.0, 8.0, 6.0], [8.0, 5.0, 2.0], [5.0, 7.0, 
1.0]], [[3.0, 1.0, 8.0], [5.0, 1.0, 1.0], [5.0, 6.0, 2.0]]], [[[4.0, 1.0, 4.0], [2.0, 8.0, 4.0], [8.0, 8.0, 6.0]], [[2.0, 3.0, 
8.0], [8.0, 1.0, 2.0], [5.0, 5.0, 2.0]], [[8.0, 7.0, 1.0], [1.0, 5.0, 7.0], [1.0, 2.0, 7.0]]], [[[8.0, 6.0, 6.0], [2.0, 7.0, 
4.0], [7.0, 1.0, 8.0]], [[6.0, 5.0, 5.0], [5.0, 6.0, 1.0], [2.0, 4.0, 8.0]], [[1.0, 4.0, 2.0], [2.0, 7.0, 4.0], [6.0, 8.0, 
4.0]]], [[[1.0, 3.0, 6.0], [6.0, 3.0, 4.0], [6.0, 3.0, 1.0]], [[5.0, 4.0, 2.0], [1.0, 3.0, 8.0], [6.0, 3.0, 5.0]], [[7.0, 2.0, 
1.0], [2.0, 8.0, 7.0], [2.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 6.0], [8.0, 6.0, 2.0], [5.0, 6.0, 4.0]], [[1.0, 5.0, 6.0], [1.0, 1.0, 
2.0], [3.0, 1.0, 5.0]], [[8.0, 4.0, 2.0], [5.0, 4.0, 8.0], [7.0, 3.0, 3.0]]], [[[8.0, 2.0, 6.0], [4.0, 4.0, 1.0], [8.0, 5.0, 
4.0]], [[7.0, 6.0, 2.0], [2.0, 1.0, 6.0], [4.0, 5.0, 3.0]], [[3.0, 6.0, 7.0], [2.0, 7.0, 8.0], [3.0, 1.0, 6.0]]], [[[3.0, 5.0, 
8.0], [2.0, 6.0, 5.0], [1.0, 5.0, 1.0]], [[3.0, 3.0, 1.0], [4.0, 7.0, 3.0], [8.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], [3.0, 7.0, 
8.0], [3.0, 4.0, 4.0]]], [[[7.0, 8.0, 5.0], [8.0, 7.0, 6.0], [1.0, 8.0, 8.0]], [[6.0, 7.0, 1.0], [6.0, 7.0, 7.0], [1.0, 7.0, 
4.0]], [[2.0, 3.0, 7.0], [1.0, 4.0, 6.0], [2.0, 2.0, 5.0]]]]))
      val=Data(numpy.array([[[[7.0]]], [[[3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0]]], [[[4.0]]]]))
      arg[2:,2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 1.0], [7.0, 2.0, 2.0], [5.0, 8.0, 3.0]], [[7.0, 8.0, 6.0], [8.0, 5.0, 2.0], [5.0, 
7.0, 1.0]], [[3.0, 1.0, 8.0], [5.0, 1.0, 1.0], [5.0, 6.0, 2.0]]], [[[4.0, 1.0, 4.0], [2.0, 8.0, 4.0], [8.0, 8.0, 6.0]], [[2.0, 
3.0, 8.0], [8.0, 1.0, 2.0], [5.0, 5.0, 2.0]], [[8.0, 7.0, 1.0], [1.0, 5.0, 7.0], [1.0, 2.0, 7.0]]], [[[8.0, 6.0, 6.0], [2.0, 
7.0, 4.0], [7.0, 1.0, 8.0]], [[6.0, 5.0, 5.0], [5.0, 6.0, 1.0], [2.0, 4.0, 8.0]], [[1.0, 4.0, 2.0], [2.0, 7.0, 4.0], [6.0, 8.0, 
4.0]]], [[[1.0, 3.0, 6.0], [6.0, 3.0, 4.0], [6.0, 3.0, 1.0]], [[5.0, 4.0, 2.0], [1.0, 3.0, 8.0], [6.0, 3.0, 5.0]], [[7.0, 2.0, 
1.0], [2.0, 3.0, 7.0], [2.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 6.0], [8.0, 6.0, 2.0], [5.0, 6.0, 4.0]], 
[[1.0, 5.0, 6.0], [1.0, 1.0, 2.0], [3.0, 1.0, 5.0]], [[8.0, 4.0, 2.0], [5.0, 4.0, 8.0], [7.0, 3.0, 3.0]]], [[[8.0, 2.0, 6.0], 
[4.0, 4.0, 1.0], [8.0, 5.0, 4.0]], [[7.0, 6.0, 2.0], [2.0, 1.0, 6.0], [4.0, 5.0, 3.0]], [[3.0, 6.0, 7.0], [2.0, 7.0, 8.0], 
[3.0, 1.0, 6.0]]], [[[3.0, 5.0, 8.0], [2.0, 6.0, 5.0], [1.0, 5.0, 1.0]], [[3.0, 3.0, 1.0], [4.0, 7.0, 3.0], [8.0, 8.0, 8.0]], 
[[7.0, 4.0, 1.0], [3.0, 6.0, 8.0], [3.0, 4.0, 4.0]]], [[[7.0, 8.0, 5.0], [8.0, 7.0, 6.0], [1.0, 8.0, 8.0]], [[6.0, 7.0, 1.0], 
[6.0, 7.0, 7.0], [1.0, 7.0, 4.0]], [[2.0, 3.0, 7.0], [1.0, 4.0, 6.0], [2.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLRR(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 6.0], [7.0, 3.0, 3.0], [4.0, 4.0, 2.0]], [[1.0, 3.0, 1.0], [6.0, 6.0, 1.0], [1.0, 6.0, 
2.0]], [[3.0, 3.0, 8.0], [4.0, 1.0, 3.0], [7.0, 6.0, 1.0]]], [[[2.0, 2.0, 1.0], [1.0, 2.0, 2.0], [8.0, 7.0, 8.0]], [[7.0, 8.0, 
3.0], [7.0, 5.0, 8.0], [1.0, 2.0, 7.0]], [[3.0, 2.0, 4.0], [4.0, 7.0, 7.0], [5.0, 3.0, 3.0]]], [[[3.0, 8.0, 4.0], [7.0, 4.0, 
6.0], [1.0, 6.0, 3.0]], [[1.0, 6.0, 5.0], [2.0, 7.0, 2.0], [5.0, 4.0, 8.0]], [[5.0, 5.0, 3.0], [8.0, 7.0, 3.0], [2.0, 6.0, 
7.0]]], [[[7.0, 3.0, 8.0], [5.0, 1.0, 8.0], [3.0, 4.0, 6.0]], [[2.0, 4.0, 8.0], [1.0, 2.0, 4.0], [4.0, 7.0, 2.0]], [[5.0, 2.0, 
2.0], [5.0, 3.0, 7.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 7.0], [2.0, 6.0, 2.0], [3.0, 1.0, 6.0]], [[2.0, 6.0, 5.0], [7.0, 8.0, 
3.0], [5.0, 7.0, 4.0]], [[4.0, 1.0, 7.0], [3.0, 5.0, 2.0], [7.0, 7.0, 2.0]]], [[[7.0, 8.0, 3.0], [2.0, 7.0, 7.0], [1.0, 4.0, 
3.0]], [[7.0, 8.0, 7.0], [4.0, 8.0, 1.0], [5.0, 2.0, 7.0]], [[2.0, 8.0, 1.0], [1.0, 6.0, 6.0], [7.0, 4.0, 4.0]]], [[[1.0, 1.0, 
2.0], [3.0, 3.0, 3.0], [5.0, 6.0, 7.0]], [[4.0, 8.0, 6.0], [3.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[4.0, 4.0, 5.0], [1.0, 4.0, 
8.0], [7.0, 8.0, 1.0]]], [[[6.0, 3.0, 8.0], [6.0, 8.0, 6.0], [6.0, 6.0, 4.0]], [[6.0, 4.0, 7.0], [5.0, 5.0, 5.0], [5.0, 4.0, 
6.0]], [[1.0, 4.0, 5.0], [5.0, 6.0, 7.0], [7.0, 6.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0]]], [[[5.0]]]])+(1.-msk_val)*numpy.array([[[[8.0]]], [[[8.0]]]])
      arg[2:,2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 6.0], [7.0, 3.0, 3.0], [4.0, 4.0, 2.0]], [[1.0, 3.0, 1.0], [6.0, 6.0, 1.0], [1.0, 
6.0, 2.0]], [[3.0, 3.0, 8.0], [4.0, 1.0, 3.0], [7.0, 6.0, 1.0]]], [[[2.0, 2.0, 1.0], [1.0, 2.0, 2.0], [8.0, 7.0, 8.0]], [[7.0, 
8.0, 3.0], [7.0, 5.0, 8.0], [1.0, 2.0, 7.0]], [[3.0, 2.0, 4.0], [4.0, 7.0, 7.0], [5.0, 3.0, 3.0]]], [[[3.0, 8.0, 4.0], [7.0, 
4.0, 6.0], [1.0, 6.0, 3.0]], [[1.0, 6.0, 5.0], [2.0, 7.0, 2.0], [5.0, 4.0, 8.0]], [[5.0, 5.0, 3.0], [8.0, 2.0, 3.0], [2.0, 6.0, 
7.0]]], [[[7.0, 3.0, 8.0], [5.0, 1.0, 8.0], [3.0, 4.0, 6.0]], [[2.0, 4.0, 8.0], [1.0, 2.0, 4.0], [4.0, 7.0, 2.0]], [[5.0, 2.0, 
2.0], [5.0, 5.0, 7.0], [5.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 7.0], [2.0, 6.0, 2.0], [3.0, 1.0, 6.0]], 
[[2.0, 6.0, 5.0], [7.0, 8.0, 3.0], [5.0, 7.0, 4.0]], [[4.0, 1.0, 7.0], [3.0, 5.0, 2.0], [7.0, 7.0, 2.0]]], [[[7.0, 8.0, 3.0], 
[2.0, 7.0, 7.0], [1.0, 4.0, 3.0]], [[7.0, 8.0, 7.0], [4.0, 8.0, 1.0], [5.0, 2.0, 7.0]], [[2.0, 8.0, 1.0], [1.0, 6.0, 6.0], 
[7.0, 4.0, 4.0]]], [[[1.0, 1.0, 2.0], [3.0, 3.0, 3.0], [5.0, 6.0, 7.0]], [[4.0, 8.0, 6.0], [3.0, 8.0, 2.0], [6.0, 1.0, 5.0]], 
[[4.0, 4.0, 5.0], [1.0, 8.0, 8.0], [7.0, 8.0, 1.0]]], [[[6.0, 3.0, 8.0], [6.0, 8.0, 6.0], [6.0, 6.0, 4.0]], [[6.0, 4.0, 7.0], 
[5.0, 5.0, 5.0], [5.0, 4.0, 6.0]], [[1.0, 4.0, 5.0], [5.0, 8.0, 7.0], [7.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRUU(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 1.0], [7.0, 3.0, 8.0], [7.0, 2.0, 2.0]], [[8.0, 8.0, 5.0], [6.0, 7.0, 1.0], [8.0, 7.0, 
5.0]], [[5.0, 7.0, 5.0], [8.0, 1.0, 3.0], [7.0, 3.0, 8.0]]], [[[3.0, 6.0, 8.0], [1.0, 7.0, 3.0], [6.0, 8.0, 2.0]], [[8.0, 7.0, 
2.0], [5.0, 4.0, 1.0], [6.0, 8.0, 5.0]], [[7.0, 5.0, 8.0], [5.0, 2.0, 5.0], [3.0, 7.0, 1.0]]], [[[1.0, 7.0, 8.0], [7.0, 8.0, 
1.0], [2.0, 8.0, 3.0]], [[1.0, 7.0, 7.0], [5.0, 1.0, 7.0], [5.0, 8.0, 2.0]], [[3.0, 7.0, 4.0], [3.0, 1.0, 3.0], [8.0, 7.0, 
4.0]]], [[[6.0, 3.0, 6.0], [3.0, 6.0, 7.0], [4.0, 8.0, 6.0]], [[7.0, 4.0, 7.0], [5.0, 2.0, 5.0], [3.0, 2.0, 3.0]], [[7.0, 6.0, 
8.0], [1.0, 3.0, 4.0], [6.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 7.0], [8.0, 4.0, 5.0], [2.0, 3.0, 6.0]], [[1.0, 2.0, 8.0], [7.0, 1.0, 
1.0], [1.0, 7.0, 8.0]], [[5.0, 1.0, 3.0], [8.0, 3.0, 6.0], [1.0, 8.0, 6.0]]], [[[3.0, 4.0, 1.0], [6.0, 1.0, 8.0], [6.0, 4.0, 
4.0]], [[1.0, 1.0, 7.0], [4.0, 8.0, 1.0], [4.0, 4.0, 8.0]], [[8.0, 3.0, 8.0], [1.0, 4.0, 3.0], [1.0, 1.0, 2.0]]], [[[8.0, 8.0, 
8.0], [2.0, 2.0, 6.0], [2.0, 2.0, 7.0]], [[6.0, 8.0, 8.0], [7.0, 1.0, 5.0], [3.0, 6.0, 7.0]], [[8.0, 7.0, 7.0], [6.0, 4.0, 
1.0], [1.0, 6.0, 5.0]]], [[[8.0, 6.0, 6.0], [2.0, 8.0, 1.0], [5.0, 6.0, 1.0]], [[4.0, 3.0, 5.0], [7.0, 7.0, 6.0], [2.0, 5.0, 
6.0]], [[6.0, 4.0, 7.0], [3.0, 1.0, 5.0], [5.0, 5.0, 8.0]]]]))
      val=numpy.array([[[[5.0, 8.0], [2.0, 1.0]]], [[[8.0, 4.0], [6.0, 1.0]]]])
      arg[1:3,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [7.0, 3.0, 8.0], [7.0, 2.0, 2.0]], [[8.0, 8.0, 5.0], [6.0, 7.0, 1.0], [8.0, 
7.0, 5.0]], [[5.0, 7.0, 5.0], [8.0, 1.0, 3.0], [7.0, 3.0, 8.0]]], [[[3.0, 6.0, 8.0], [1.0, 7.0, 3.0], [6.0, 8.0, 2.0]], [[5.0, 
8.0, 2.0], [2.0, 1.0, 1.0], [6.0, 8.0, 5.0]], [[7.0, 5.0, 8.0], [5.0, 2.0, 5.0], [3.0, 7.0, 1.0]]], [[[1.0, 7.0, 8.0], [7.0, 
8.0, 1.0], [2.0, 8.0, 3.0]], [[8.0, 4.0, 7.0], [6.0, 1.0, 7.0], [5.0, 8.0, 2.0]], [[3.0, 7.0, 4.0], [3.0, 1.0, 3.0], [8.0, 7.0, 
4.0]]], [[[6.0, 3.0, 6.0], [3.0, 6.0, 7.0], [4.0, 8.0, 6.0]], [[7.0, 4.0, 7.0], [5.0, 2.0, 5.0], [3.0, 2.0, 3.0]], [[7.0, 6.0, 
8.0], [1.0, 3.0, 4.0], [6.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 7.0], [8.0, 4.0, 5.0], [2.0, 3.0, 6.0]], 
[[1.0, 2.0, 8.0], [7.0, 1.0, 1.0], [1.0, 7.0, 8.0]], [[5.0, 1.0, 3.0], [8.0, 3.0, 6.0], [1.0, 8.0, 6.0]]], [[[3.0, 4.0, 1.0], 
[6.0, 1.0, 8.0], [6.0, 4.0, 4.0]], [[5.0, 8.0, 7.0], [2.0, 1.0, 1.0], [4.0, 4.0, 8.0]], [[8.0, 3.0, 8.0], [1.0, 4.0, 3.0], 
[1.0, 1.0, 2.0]]], [[[8.0, 8.0, 8.0], [2.0, 2.0, 6.0], [2.0, 2.0, 7.0]], [[8.0, 4.0, 8.0], [6.0, 1.0, 5.0], [3.0, 6.0, 7.0]], 
[[8.0, 7.0, 7.0], [6.0, 4.0, 1.0], [1.0, 6.0, 5.0]]], [[[8.0, 6.0, 6.0], [2.0, 8.0, 1.0], [5.0, 6.0, 1.0]], [[4.0, 3.0, 5.0], 
[7.0, 7.0, 6.0], [2.0, 5.0, 6.0]], [[6.0, 4.0, 7.0], [3.0, 1.0, 5.0], [5.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRUU(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 5.0], [6.0, 4.0, 6.0], [4.0, 4.0, 8.0]], [[4.0, 1.0, 4.0], [3.0, 5.0, 1.0], [4.0, 4.0, 
4.0]], [[5.0, 3.0, 7.0], [5.0, 4.0, 2.0], [8.0, 1.0, 5.0]]], [[[5.0, 4.0, 3.0], [4.0, 7.0, 4.0], [8.0, 7.0, 8.0]], [[7.0, 8.0, 
4.0], [6.0, 8.0, 4.0], [8.0, 1.0, 1.0]], [[2.0, 2.0, 5.0], [6.0, 4.0, 2.0], [2.0, 3.0, 6.0]]], [[[1.0, 1.0, 1.0], [7.0, 8.0, 
2.0], [4.0, 6.0, 6.0]], [[6.0, 3.0, 3.0], [2.0, 8.0, 8.0], [1.0, 3.0, 1.0]], [[6.0, 2.0, 8.0], [5.0, 5.0, 5.0], [8.0, 5.0, 
4.0]]], [[[5.0, 2.0, 5.0], [1.0, 4.0, 4.0], [8.0, 6.0, 2.0]], [[8.0, 4.0, 6.0], [8.0, 5.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 3.0, 
2.0], [7.0, 8.0, 6.0], [8.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 7.0], [5.0, 5.0, 7.0], [5.0, 8.0, 8.0]], [[4.0, 3.0, 1.0], [5.0, 7.0, 
3.0], [6.0, 2.0, 2.0]], [[3.0, 7.0, 3.0], [4.0, 5.0, 1.0], [5.0, 2.0, 4.0]]], [[[1.0, 3.0, 1.0], [7.0, 7.0, 6.0], [3.0, 2.0, 
3.0]], [[1.0, 1.0, 1.0], [3.0, 4.0, 4.0], [3.0, 1.0, 2.0]], [[3.0, 6.0, 8.0], [6.0, 7.0, 7.0], [6.0, 8.0, 8.0]]], [[[1.0, 2.0, 
7.0], [5.0, 3.0, 4.0], [5.0, 6.0, 1.0]], [[6.0, 3.0, 7.0], [7.0, 5.0, 5.0], [7.0, 8.0, 2.0]], [[8.0, 4.0, 7.0], [4.0, 6.0, 
1.0], [6.0, 5.0, 3.0]]], [[[2.0, 2.0, 3.0], [3.0, 5.0, 1.0], [4.0, 1.0, 7.0]], [[7.0, 3.0, 4.0], [1.0, 6.0, 1.0], [4.0, 5.0, 
8.0]], [[6.0, 3.0, 8.0], [8.0, 5.0, 3.0], [5.0, 4.0, 6.0]]]]))
      val=Data(numpy.array([[[[1.0, 5.0], [5.0, 6.0]]], [[[1.0, 6.0], [6.0, 1.0]]]]),self.functionspace)
      arg[1:3,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 5.0], [6.0, 4.0, 6.0], [4.0, 4.0, 8.0]], [[4.0, 1.0, 4.0], [3.0, 5.0, 1.0], [4.0, 
4.0, 4.0]], [[5.0, 3.0, 7.0], [5.0, 4.0, 2.0], [8.0, 1.0, 5.0]]], [[[5.0, 4.0, 3.0], [4.0, 7.0, 4.0], [8.0, 7.0, 8.0]], [[1.0, 
5.0, 4.0], [5.0, 6.0, 4.0], [8.0, 1.0, 1.0]], [[2.0, 2.0, 5.0], [6.0, 4.0, 2.0], [2.0, 3.0, 6.0]]], [[[1.0, 1.0, 1.0], [7.0, 
8.0, 2.0], [4.0, 6.0, 6.0]], [[1.0, 6.0, 3.0], [6.0, 1.0, 8.0], [1.0, 3.0, 1.0]], [[6.0, 2.0, 8.0], [5.0, 5.0, 5.0], [8.0, 5.0, 
4.0]]], [[[5.0, 2.0, 5.0], [1.0, 4.0, 4.0], [8.0, 6.0, 2.0]], [[8.0, 4.0, 6.0], [8.0, 5.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 3.0, 
2.0], [7.0, 8.0, 6.0], [8.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 7.0], [5.0, 5.0, 7.0], [5.0, 8.0, 8.0]], 
[[4.0, 3.0, 1.0], [5.0, 7.0, 3.0], [6.0, 2.0, 2.0]], [[3.0, 7.0, 3.0], [4.0, 5.0, 1.0], [5.0, 2.0, 4.0]]], [[[1.0, 3.0, 1.0], 
[7.0, 7.0, 6.0], [3.0, 2.0, 3.0]], [[1.0, 5.0, 1.0], [5.0, 6.0, 4.0], [3.0, 1.0, 2.0]], [[3.0, 6.0, 8.0], [6.0, 7.0, 7.0], 
[6.0, 8.0, 8.0]]], [[[1.0, 2.0, 7.0], [5.0, 3.0, 4.0], [5.0, 6.0, 1.0]], [[1.0, 6.0, 7.0], [6.0, 1.0, 5.0], [7.0, 8.0, 2.0]], 
[[8.0, 4.0, 7.0], [4.0, 6.0, 1.0], [6.0, 5.0, 3.0]]], [[[2.0, 2.0, 3.0], [3.0, 5.0, 1.0], [4.0, 1.0, 7.0]], [[7.0, 3.0, 4.0], 
[1.0, 6.0, 1.0], [4.0, 5.0, 8.0]], [[6.0, 3.0, 8.0], [8.0, 5.0, 3.0], [5.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRUU(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 6.0], [3.0, 1.0, 7.0], [1.0, 3.0, 2.0]], [[8.0, 7.0, 6.0], [1.0, 3.0, 1.0], [4.0, 8.0, 
4.0]], [[4.0, 8.0, 7.0], [6.0, 7.0, 5.0], [3.0, 2.0, 6.0]]], [[[1.0, 6.0, 2.0], [4.0, 1.0, 7.0], [7.0, 1.0, 1.0]], [[2.0, 4.0, 
7.0], [8.0, 6.0, 5.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 3.0], [3.0, 3.0, 8.0], [1.0, 2.0, 1.0]]], [[[4.0, 2.0, 2.0], [1.0, 7.0, 
6.0], [3.0, 8.0, 6.0]], [[2.0, 1.0, 8.0], [1.0, 2.0, 1.0], [6.0, 7.0, 8.0]], [[8.0, 8.0, 6.0], [3.0, 5.0, 1.0], [8.0, 7.0, 
3.0]]], [[[8.0, 3.0, 4.0], [4.0, 1.0, 1.0], [3.0, 4.0, 3.0]], [[7.0, 3.0, 3.0], [1.0, 4.0, 1.0], [6.0, 4.0, 2.0]], [[4.0, 1.0, 
1.0], [5.0, 3.0, 1.0], [3.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 6.0], [2.0, 4.0, 8.0], [2.0, 2.0, 4.0]], [[3.0, 8.0, 3.0], [8.0, 4.0, 
2.0], [6.0, 8.0, 6.0]], [[8.0, 4.0, 8.0], [4.0, 5.0, 2.0], [3.0, 6.0, 1.0]]], [[[8.0, 2.0, 2.0], [8.0, 3.0, 1.0], [2.0, 1.0, 
3.0]], [[2.0, 7.0, 3.0], [3.0, 1.0, 3.0], [7.0, 1.0, 7.0]], [[6.0, 3.0, 5.0], [7.0, 5.0, 7.0], [6.0, 2.0, 6.0]]], [[[1.0, 3.0, 
4.0], [7.0, 5.0, 1.0], [3.0, 7.0, 3.0]], [[6.0, 2.0, 8.0], [3.0, 6.0, 4.0], [6.0, 6.0, 5.0]], [[3.0, 5.0, 6.0], [8.0, 7.0, 
8.0], [6.0, 8.0, 5.0]]], [[[7.0, 6.0, 3.0], [7.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[5.0, 5.0, 4.0], [8.0, 5.0, 1.0], [8.0, 2.0, 
1.0]], [[4.0, 1.0, 1.0], [8.0, 1.0, 1.0], [7.0, 6.0, 8.0]]]]))
      val=Data(numpy.array([[[[5.0, 7.0], [7.0, 7.0]]], [[[4.0, 2.0], [3.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 4.0], [6.0, 2.0]]], [[[3.0, 2.0], [2.0, 5.0]]]]))
      arg[1:3,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 6.0], [3.0, 1.0, 7.0], [1.0, 3.0, 2.0]], [[8.0, 7.0, 6.0], [1.0, 3.0, 1.0], [4.0, 
8.0, 4.0]], [[4.0, 8.0, 7.0], [6.0, 7.0, 5.0], [3.0, 2.0, 6.0]]], [[[1.0, 6.0, 2.0], [4.0, 1.0, 7.0], [7.0, 1.0, 1.0]], [[5.0, 
7.0, 7.0], [7.0, 7.0, 5.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 3.0], [3.0, 3.0, 8.0], [1.0, 2.0, 1.0]]], [[[4.0, 2.0, 2.0], [1.0, 
7.0, 6.0], [3.0, 8.0, 6.0]], [[4.0, 2.0, 8.0], [3.0, 5.0, 1.0], [6.0, 7.0, 8.0]], [[8.0, 8.0, 6.0], [3.0, 5.0, 1.0], [8.0, 7.0, 
3.0]]], [[[8.0, 3.0, 4.0], [4.0, 1.0, 1.0], [3.0, 4.0, 3.0]], [[7.0, 3.0, 3.0], [1.0, 4.0, 1.0], [6.0, 4.0, 2.0]], [[4.0, 1.0, 
1.0], [5.0, 3.0, 1.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [2.0, 4.0, 8.0], [2.0, 2.0, 4.0]], 
[[3.0, 8.0, 3.0], [8.0, 4.0, 2.0], [6.0, 8.0, 6.0]], [[8.0, 4.0, 8.0], [4.0, 5.0, 2.0], [3.0, 6.0, 1.0]]], [[[8.0, 2.0, 2.0], 
[8.0, 3.0, 1.0], [2.0, 1.0, 3.0]], [[2.0, 4.0, 3.0], [6.0, 2.0, 3.0], [7.0, 1.0, 7.0]], [[6.0, 3.0, 5.0], [7.0, 5.0, 7.0], 
[6.0, 2.0, 6.0]]], [[[1.0, 3.0, 4.0], [7.0, 5.0, 1.0], [3.0, 7.0, 3.0]], [[3.0, 2.0, 8.0], [2.0, 5.0, 4.0], [6.0, 6.0, 5.0]], 
[[3.0, 5.0, 6.0], [8.0, 7.0, 8.0], [6.0, 8.0, 5.0]]], [[[7.0, 6.0, 3.0], [7.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[5.0, 5.0, 4.0], 
[8.0, 5.0, 1.0], [8.0, 2.0, 1.0]], [[4.0, 1.0, 1.0], [8.0, 1.0, 1.0], [7.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRUU(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 1.0], [6.0, 7.0, 5.0], [3.0, 4.0, 3.0]], [[3.0, 3.0, 8.0], [5.0, 6.0, 1.0], [2.0, 8.0, 
7.0]], [[7.0, 6.0, 7.0], [5.0, 2.0, 3.0], [6.0, 2.0, 1.0]]], [[[6.0, 7.0, 3.0], [5.0, 3.0, 3.0], [3.0, 7.0, 2.0]], [[4.0, 3.0, 
3.0], [2.0, 7.0, 5.0], [4.0, 2.0, 1.0]], [[5.0, 2.0, 6.0], [5.0, 7.0, 4.0], [2.0, 4.0, 7.0]]], [[[7.0, 2.0, 8.0], [3.0, 1.0, 
8.0], [2.0, 4.0, 3.0]], [[1.0, 8.0, 8.0], [8.0, 8.0, 1.0], [3.0, 2.0, 4.0]], [[8.0, 7.0, 2.0], [8.0, 2.0, 3.0], [1.0, 4.0, 
6.0]]], [[[5.0, 3.0, 3.0], [5.0, 7.0, 6.0], [3.0, 1.0, 5.0]], [[8.0, 4.0, 8.0], [1.0, 5.0, 2.0], [3.0, 2.0, 2.0]], [[5.0, 5.0, 
1.0], [6.0, 5.0, 3.0], [2.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 6.0], [8.0, 1.0, 2.0], [4.0, 2.0, 6.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 
2.0], [1.0, 8.0, 3.0]], [[3.0, 2.0, 2.0], [8.0, 6.0, 4.0], [7.0, 4.0, 7.0]]], [[[6.0, 8.0, 6.0], [1.0, 1.0, 5.0], [7.0, 2.0, 
6.0]], [[5.0, 3.0, 5.0], [6.0, 7.0, 2.0], [2.0, 1.0, 4.0]], [[8.0, 4.0, 2.0], [6.0, 5.0, 3.0], [2.0, 3.0, 7.0]]], [[[8.0, 1.0, 
1.0], [4.0, 7.0, 4.0], [5.0, 3.0, 2.0]], [[7.0, 1.0, 2.0], [8.0, 6.0, 8.0], [1.0, 8.0, 6.0]], [[5.0, 5.0, 5.0], [6.0, 2.0, 
5.0], [7.0, 2.0, 4.0]]], [[[3.0, 5.0, 7.0], [8.0, 6.0, 6.0], [8.0, 6.0, 6.0]], [[4.0, 1.0, 4.0], [4.0, 3.0, 3.0], [4.0, 4.0, 
1.0]], [[8.0, 6.0, 8.0], [4.0, 4.0, 7.0], [6.0, 3.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 6.0], [5.0, 6.0]]], [[[2.0, 5.0], [2.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 8.0], 
[2.0, 5.0]]], [[[1.0, 1.0], [7.0, 8.0]]]])
      arg[1:3,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 1.0], [6.0, 7.0, 5.0], [3.0, 4.0, 3.0]], [[3.0, 3.0, 8.0], [5.0, 6.0, 1.0], [2.0, 
8.0, 7.0]], [[7.0, 6.0, 7.0], [5.0, 2.0, 3.0], [6.0, 2.0, 1.0]]], [[[6.0, 7.0, 3.0], [5.0, 3.0, 3.0], [3.0, 7.0, 2.0]], [[5.0, 
6.0, 3.0], [5.0, 6.0, 5.0], [4.0, 2.0, 1.0]], [[5.0, 2.0, 6.0], [5.0, 7.0, 4.0], [2.0, 4.0, 7.0]]], [[[7.0, 2.0, 8.0], [3.0, 
1.0, 8.0], [2.0, 4.0, 3.0]], [[2.0, 5.0, 8.0], [2.0, 3.0, 1.0], [3.0, 2.0, 4.0]], [[8.0, 7.0, 2.0], [8.0, 2.0, 3.0], [1.0, 4.0, 
6.0]]], [[[5.0, 3.0, 3.0], [5.0, 7.0, 6.0], [3.0, 1.0, 5.0]], [[8.0, 4.0, 8.0], [1.0, 5.0, 2.0], [3.0, 2.0, 2.0]], [[5.0, 5.0, 
1.0], [6.0, 5.0, 3.0], [2.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [8.0, 1.0, 2.0], [4.0, 2.0, 6.0]], 
[[6.0, 4.0, 4.0], [2.0, 5.0, 2.0], [1.0, 8.0, 3.0]], [[3.0, 2.0, 2.0], [8.0, 6.0, 4.0], [7.0, 4.0, 7.0]]], [[[6.0, 8.0, 6.0], 
[1.0, 1.0, 5.0], [7.0, 2.0, 6.0]], [[2.0, 8.0, 5.0], [2.0, 5.0, 2.0], [2.0, 1.0, 4.0]], [[8.0, 4.0, 2.0], [6.0, 5.0, 3.0], 
[2.0, 3.0, 7.0]]], [[[8.0, 1.0, 1.0], [4.0, 7.0, 4.0], [5.0, 3.0, 2.0]], [[1.0, 1.0, 2.0], [7.0, 8.0, 8.0], [1.0, 8.0, 6.0]], 
[[5.0, 5.0, 5.0], [6.0, 2.0, 5.0], [7.0, 2.0, 4.0]]], [[[3.0, 5.0, 7.0], [8.0, 6.0, 6.0], [8.0, 6.0, 6.0]], [[4.0, 1.0, 4.0], 
[4.0, 3.0, 3.0], [4.0, 4.0, 1.0]], [[8.0, 6.0, 8.0], [4.0, 4.0, 7.0], [6.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCUP(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 1.0], [4.0, 2.0, 3.0], [8.0, 8.0, 4.0]], [[1.0, 5.0, 2.0], [4.0, 6.0, 2.0], [6.0, 2.0, 
8.0]], [[2.0, 6.0, 8.0], [5.0, 5.0, 2.0], [7.0, 5.0, 7.0]]], [[[1.0, 5.0, 8.0], [1.0, 4.0, 8.0], [4.0, 7.0, 6.0]], [[1.0, 8.0, 
2.0], [1.0, 1.0, 8.0], [2.0, 1.0, 7.0]], [[2.0, 1.0, 2.0], [4.0, 8.0, 8.0], [6.0, 5.0, 5.0]]], [[[3.0, 8.0, 8.0], [2.0, 1.0, 
1.0], [6.0, 1.0, 3.0]], [[3.0, 6.0, 7.0], [1.0, 2.0, 8.0], [5.0, 2.0, 8.0]], [[8.0, 2.0, 3.0], [3.0, 4.0, 8.0], [1.0, 7.0, 
2.0]]], [[[4.0, 8.0, 1.0], [2.0, 5.0, 2.0], [7.0, 2.0, 8.0]], [[1.0, 8.0, 7.0], [2.0, 6.0, 8.0], [3.0, 5.0, 6.0]], [[6.0, 1.0, 
6.0], [3.0, 4.0, 6.0], [6.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 3.0], [5.0, 5.0, 3.0], [8.0, 6.0, 7.0]], [[8.0, 3.0, 4.0], [4.0, 8.0, 
4.0], [2.0, 2.0, 5.0]], [[3.0, 2.0, 2.0], [7.0, 6.0, 3.0], [2.0, 2.0, 5.0]]], [[[3.0, 1.0, 6.0], [5.0, 8.0, 8.0], [7.0, 1.0, 
2.0]], [[7.0, 8.0, 7.0], [3.0, 5.0, 6.0], [6.0, 4.0, 4.0]], [[3.0, 4.0, 1.0], [8.0, 2.0, 2.0], [5.0, 7.0, 1.0]]], [[[1.0, 4.0, 
1.0], [6.0, 7.0, 6.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 1.0], [8.0, 7.0, 6.0], [5.0, 2.0, 6.0]], [[4.0, 6.0, 4.0], [4.0, 6.0, 
5.0], [1.0, 2.0, 5.0]]], [[[7.0, 2.0, 5.0], [5.0, 5.0, 3.0], [4.0, 7.0, 4.0]], [[2.0, 1.0, 7.0], [8.0, 7.0, 7.0], [7.0, 8.0, 
6.0]], [[3.0, 3.0, 2.0], [3.0, 7.0, 4.0], [6.0, 8.0, 2.0]]]]))
      val=numpy.array([[[7.0, 2.0], [1.0, 5.0], [3.0, 3.0]], [[8.0, 3.0], [8.0, 5.0], [1.0, 1.0]], [[3.0, 4.0], [2.0, 2.0], 
[6.0, 7.0]], [[2.0, 1.0], [1.0, 1.0], [1.0, 2.0]]])
      arg[:,:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 1.0], [4.0, 2.0, 3.0], [8.0, 8.0, 4.0]], [[1.0, 1.0, 2.0], [4.0, 5.0, 2.0], [6.0, 
2.0, 8.0]], [[2.0, 3.0, 8.0], [5.0, 3.0, 2.0], [7.0, 5.0, 7.0]]], [[[1.0, 8.0, 8.0], [1.0, 3.0, 8.0], [4.0, 7.0, 6.0]], [[1.0, 
8.0, 2.0], [1.0, 5.0, 8.0], [2.0, 1.0, 7.0]], [[2.0, 1.0, 2.0], [4.0, 1.0, 8.0], [6.0, 5.0, 5.0]]], [[[3.0, 3.0, 8.0], [2.0, 
4.0, 1.0], [6.0, 1.0, 3.0]], [[3.0, 2.0, 7.0], [1.0, 2.0, 8.0], [5.0, 2.0, 8.0]], [[8.0, 6.0, 3.0], [3.0, 7.0, 8.0], [1.0, 7.0, 
2.0]]], [[[4.0, 2.0, 1.0], [2.0, 1.0, 2.0], [7.0, 2.0, 8.0]], [[1.0, 1.0, 7.0], [2.0, 1.0, 8.0], [3.0, 5.0, 6.0]], [[6.0, 1.0, 
6.0], [3.0, 2.0, 6.0], [6.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 3.0], [5.0, 2.0, 3.0], [8.0, 6.0, 7.0]], 
[[8.0, 1.0, 4.0], [4.0, 5.0, 4.0], [2.0, 2.0, 5.0]], [[3.0, 3.0, 2.0], [7.0, 3.0, 3.0], [2.0, 2.0, 5.0]]], [[[3.0, 8.0, 6.0], 
[5.0, 3.0, 8.0], [7.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [3.0, 5.0, 6.0], [6.0, 4.0, 4.0]], [[3.0, 1.0, 1.0], [8.0, 1.0, 2.0], 
[5.0, 7.0, 1.0]]], [[[1.0, 3.0, 1.0], [6.0, 4.0, 6.0], [7.0, 4.0, 3.0]], [[8.0, 2.0, 1.0], [8.0, 2.0, 6.0], [5.0, 2.0, 6.0]], 
[[4.0, 6.0, 4.0], [4.0, 7.0, 5.0], [1.0, 2.0, 5.0]]], [[[7.0, 2.0, 5.0], [5.0, 1.0, 3.0], [4.0, 7.0, 4.0]], [[2.0, 1.0, 7.0], 
[8.0, 1.0, 7.0], [7.0, 8.0, 6.0]], [[3.0, 1.0, 2.0], [3.0, 2.0, 4.0], [6.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCUP(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 1.0], [6.0, 3.0, 3.0], [2.0, 5.0, 8.0]], [[1.0, 3.0, 3.0], [7.0, 6.0, 6.0], [2.0, 6.0, 
1.0]], [[2.0, 2.0, 1.0], [4.0, 8.0, 3.0], [8.0, 2.0, 2.0]]], [[[4.0, 2.0, 4.0], [5.0, 2.0, 6.0], [3.0, 8.0, 8.0]], [[5.0, 7.0, 
2.0], [7.0, 3.0, 7.0], [5.0, 1.0, 3.0]], [[2.0, 2.0, 6.0], [5.0, 5.0, 5.0], [6.0, 6.0, 4.0]]], [[[4.0, 2.0, 8.0], [6.0, 6.0, 
8.0], [2.0, 5.0, 4.0]], [[4.0, 5.0, 6.0], [2.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[4.0, 4.0, 6.0], [8.0, 1.0, 3.0], [1.0, 6.0, 
7.0]]], [[[5.0, 6.0, 2.0], [1.0, 7.0, 6.0], [2.0, 7.0, 6.0]], [[1.0, 8.0, 1.0], [8.0, 2.0, 6.0], [5.0, 1.0, 6.0]], [[6.0, 8.0, 
8.0], [5.0, 7.0, 6.0], [7.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 5.0], [5.0, 2.0, 7.0], [8.0, 5.0, 3.0]], [[6.0, 4.0, 3.0], [3.0, 1.0, 
5.0], [5.0, 4.0, 3.0]], [[6.0, 8.0, 7.0], [6.0, 1.0, 1.0], [8.0, 7.0, 5.0]]], [[[1.0, 7.0, 4.0], [6.0, 2.0, 1.0], [8.0, 2.0, 
1.0]], [[3.0, 2.0, 2.0], [2.0, 1.0, 6.0], [8.0, 1.0, 3.0]], [[1.0, 8.0, 4.0], [1.0, 4.0, 6.0], [3.0, 1.0, 5.0]]], [[[2.0, 5.0, 
3.0], [2.0, 3.0, 3.0], [3.0, 4.0, 2.0]], [[8.0, 8.0, 2.0], [1.0, 2.0, 6.0], [4.0, 4.0, 4.0]], [[7.0, 5.0, 1.0], [8.0, 2.0, 
1.0], [1.0, 4.0, 7.0]]], [[[8.0, 4.0, 2.0], [7.0, 4.0, 5.0], [2.0, 6.0, 8.0]], [[2.0, 8.0, 6.0], [6.0, 6.0, 3.0], [6.0, 8.0, 
4.0]], [[3.0, 5.0, 6.0], [8.0, 2.0, 6.0], [3.0, 8.0, 1.0]]]]))
      val=Data(numpy.array([[[5.0, 4.0], [7.0, 8.0], [6.0, 4.0]], [[4.0, 6.0], [1.0, 5.0], [7.0, 6.0]], [[3.0, 2.0], [3.0, 
6.0], [5.0, 7.0]], [[2.0, 6.0], [4.0, 6.0], [8.0, 1.0]]]),self.functionspace)
      arg[:,:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 1.0], [6.0, 4.0, 3.0], [2.0, 5.0, 8.0]], [[1.0, 7.0, 3.0], [7.0, 8.0, 6.0], [2.0, 
6.0, 1.0]], [[2.0, 6.0, 1.0], [4.0, 4.0, 3.0], [8.0, 2.0, 2.0]]], [[[4.0, 4.0, 4.0], [5.0, 6.0, 6.0], [3.0, 8.0, 8.0]], [[5.0, 
1.0, 2.0], [7.0, 5.0, 7.0], [5.0, 1.0, 3.0]], [[2.0, 7.0, 6.0], [5.0, 6.0, 5.0], [6.0, 6.0, 4.0]]], [[[4.0, 3.0, 8.0], [6.0, 
2.0, 8.0], [2.0, 5.0, 4.0]], [[4.0, 3.0, 6.0], [2.0, 6.0, 4.0], [5.0, 5.0, 4.0]], [[4.0, 5.0, 6.0], [8.0, 7.0, 3.0], [1.0, 6.0, 
7.0]]], [[[5.0, 2.0, 2.0], [1.0, 6.0, 6.0], [2.0, 7.0, 6.0]], [[1.0, 4.0, 1.0], [8.0, 6.0, 6.0], [5.0, 1.0, 6.0]], [[6.0, 8.0, 
8.0], [5.0, 1.0, 6.0], [7.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 5.0], [5.0, 4.0, 7.0], [8.0, 5.0, 3.0]], 
[[6.0, 7.0, 3.0], [3.0, 8.0, 5.0], [5.0, 4.0, 3.0]], [[6.0, 6.0, 7.0], [6.0, 4.0, 1.0], [8.0, 7.0, 5.0]]], [[[1.0, 4.0, 4.0], 
[6.0, 6.0, 1.0], [8.0, 2.0, 1.0]], [[3.0, 1.0, 2.0], [2.0, 5.0, 6.0], [8.0, 1.0, 3.0]], [[1.0, 7.0, 4.0], [1.0, 6.0, 6.0], 
[3.0, 1.0, 5.0]]], [[[2.0, 3.0, 3.0], [2.0, 2.0, 3.0], [3.0, 4.0, 2.0]], [[8.0, 3.0, 2.0], [1.0, 6.0, 6.0], [4.0, 4.0, 4.0]], 
[[7.0, 5.0, 1.0], [8.0, 7.0, 1.0], [1.0, 4.0, 7.0]]], [[[8.0, 2.0, 2.0], [7.0, 6.0, 5.0], [2.0, 6.0, 8.0]], [[2.0, 4.0, 6.0], 
[6.0, 6.0, 3.0], [6.0, 8.0, 4.0]], [[3.0, 8.0, 6.0], [8.0, 1.0, 6.0], [3.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCUP(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 1.0], [4.0, 2.0, 4.0], [1.0, 4.0, 1.0]], [[1.0, 8.0, 6.0], [6.0, 1.0, 3.0], [6.0, 8.0, 
7.0]], [[2.0, 6.0, 6.0], [3.0, 4.0, 3.0], [8.0, 4.0, 1.0]]], [[[3.0, 6.0, 2.0], [5.0, 4.0, 5.0], [2.0, 2.0, 6.0]], [[8.0, 3.0, 
2.0], [7.0, 2.0, 2.0], [7.0, 5.0, 5.0]], [[1.0, 1.0, 1.0], [6.0, 1.0, 1.0], [7.0, 6.0, 5.0]]], [[[7.0, 8.0, 6.0], [3.0, 8.0, 
1.0], [3.0, 1.0, 8.0]], [[1.0, 3.0, 7.0], [1.0, 4.0, 3.0], [7.0, 8.0, 5.0]], [[8.0, 1.0, 8.0], [1.0, 1.0, 7.0], [8.0, 6.0, 
7.0]]], [[[3.0, 4.0, 4.0], [3.0, 5.0, 4.0], [7.0, 4.0, 4.0]], [[6.0, 5.0, 1.0], [5.0, 2.0, 5.0], [4.0, 4.0, 8.0]], [[4.0, 6.0, 
6.0], [6.0, 2.0, 3.0], [5.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 4.0], [4.0, 4.0, 3.0], [2.0, 5.0, 8.0]], [[5.0, 2.0, 3.0], [7.0, 3.0, 
5.0], [5.0, 2.0, 5.0]], [[7.0, 3.0, 3.0], [6.0, 7.0, 2.0], [3.0, 2.0, 8.0]]], [[[1.0, 1.0, 4.0], [7.0, 7.0, 4.0], [7.0, 8.0, 
8.0]], [[3.0, 3.0, 3.0], [7.0, 6.0, 6.0], [6.0, 3.0, 5.0]], [[3.0, 3.0, 7.0], [5.0, 2.0, 5.0], [8.0, 7.0, 5.0]]], [[[3.0, 2.0, 
4.0], [1.0, 4.0, 6.0], [8.0, 6.0, 1.0]], [[6.0, 7.0, 7.0], [2.0, 3.0, 3.0], [4.0, 1.0, 1.0]], [[7.0, 2.0, 5.0], [1.0, 8.0, 
7.0], [2.0, 1.0, 4.0]]], [[[3.0, 4.0, 4.0], [3.0, 7.0, 4.0], [2.0, 1.0, 2.0]], [[7.0, 4.0, 8.0], [1.0, 2.0, 5.0], [3.0, 4.0, 
1.0]], [[5.0, 3.0, 7.0], [4.0, 7.0, 3.0], [6.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([[[2.0, 4.0], [7.0, 8.0], [6.0, 7.0]], [[4.0, 3.0], [5.0, 2.0], [5.0, 7.0]], [[8.0, 8.0], [2.0, 
8.0], [7.0, 6.0]], [[8.0, 1.0], [1.0, 3.0], [2.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 3.0], [5.0, 3.0], [4.0, 6.0]], [[3.0, 5.0], [7.0, 7.0], [1.0, 3.0]], [[4.0, 
1.0], [3.0, 4.0], [5.0, 7.0]], [[4.0, 2.0], [6.0, 8.0], [8.0, 4.0]]]))
      arg[:,:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 1.0], [4.0, 4.0, 4.0], [1.0, 4.0, 1.0]], [[1.0, 7.0, 6.0], [6.0, 8.0, 3.0], [6.0, 
8.0, 7.0]], [[2.0, 6.0, 6.0], [3.0, 7.0, 3.0], [8.0, 4.0, 1.0]]], [[[3.0, 4.0, 2.0], [5.0, 3.0, 5.0], [2.0, 2.0, 6.0]], [[8.0, 
5.0, 2.0], [7.0, 2.0, 2.0], [7.0, 5.0, 5.0]], [[1.0, 5.0, 1.0], [6.0, 7.0, 1.0], [7.0, 6.0, 5.0]]], [[[7.0, 8.0, 6.0], [3.0, 
8.0, 1.0], [3.0, 1.0, 8.0]], [[1.0, 2.0, 7.0], [1.0, 8.0, 3.0], [7.0, 8.0, 5.0]], [[8.0, 7.0, 8.0], [1.0, 6.0, 7.0], [8.0, 6.0, 
7.0]]], [[[3.0, 8.0, 4.0], [3.0, 1.0, 4.0], [7.0, 4.0, 4.0]], [[6.0, 1.0, 1.0], [5.0, 3.0, 5.0], [4.0, 4.0, 8.0]], [[4.0, 2.0, 
6.0], [6.0, 8.0, 3.0], [5.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 4.0], [4.0, 3.0, 3.0], [2.0, 5.0, 8.0]], 
[[5.0, 5.0, 3.0], [7.0, 3.0, 5.0], [5.0, 2.0, 5.0]], [[7.0, 4.0, 3.0], [6.0, 6.0, 2.0], [3.0, 2.0, 8.0]]], [[[1.0, 3.0, 4.0], 
[7.0, 5.0, 4.0], [7.0, 8.0, 8.0]], [[3.0, 7.0, 3.0], [7.0, 7.0, 6.0], [6.0, 3.0, 5.0]], [[3.0, 1.0, 7.0], [5.0, 3.0, 5.0], 
[8.0, 7.0, 5.0]]], [[[3.0, 4.0, 4.0], [1.0, 1.0, 6.0], [8.0, 6.0, 1.0]], [[6.0, 3.0, 7.0], [2.0, 4.0, 3.0], [4.0, 1.0, 1.0]], 
[[7.0, 5.0, 5.0], [1.0, 7.0, 7.0], [2.0, 1.0, 4.0]]], [[[3.0, 4.0, 4.0], [3.0, 2.0, 4.0], [2.0, 1.0, 2.0]], [[7.0, 6.0, 8.0], 
[1.0, 8.0, 5.0], [3.0, 4.0, 1.0]], [[5.0, 8.0, 7.0], [4.0, 4.0, 3.0], [6.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCUP(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 3.0], [7.0, 1.0, 5.0], [1.0, 7.0, 5.0]], [[6.0, 4.0, 4.0], [5.0, 8.0, 2.0], [7.0, 7.0, 
1.0]], [[2.0, 1.0, 2.0], [3.0, 3.0, 5.0], [8.0, 7.0, 2.0]]], [[[1.0, 1.0, 4.0], [5.0, 3.0, 2.0], [8.0, 5.0, 6.0]], [[6.0, 4.0, 
2.0], [7.0, 4.0, 1.0], [2.0, 8.0, 2.0]], [[8.0, 4.0, 3.0], [8.0, 8.0, 8.0], [7.0, 6.0, 3.0]]], [[[8.0, 3.0, 4.0], [8.0, 5.0, 
7.0], [8.0, 5.0, 7.0]], [[5.0, 7.0, 6.0], [1.0, 2.0, 8.0], [7.0, 8.0, 3.0]], [[7.0, 7.0, 8.0], [8.0, 8.0, 8.0], [7.0, 6.0, 
8.0]]], [[[4.0, 7.0, 5.0], [8.0, 2.0, 3.0], [2.0, 2.0, 6.0]], [[1.0, 3.0, 4.0], [3.0, 1.0, 2.0], [5.0, 2.0, 6.0]], [[4.0, 6.0, 
6.0], [1.0, 1.0, 3.0], [5.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 6.0], [6.0, 8.0, 5.0], [4.0, 8.0, 2.0]], [[7.0, 7.0, 5.0], [1.0, 8.0, 
6.0], [1.0, 7.0, 5.0]], [[3.0, 8.0, 1.0], [3.0, 1.0, 5.0], [1.0, 4.0, 6.0]]], [[[2.0, 5.0, 5.0], [4.0, 5.0, 3.0], [1.0, 2.0, 
1.0]], [[7.0, 1.0, 6.0], [4.0, 3.0, 8.0], [3.0, 7.0, 3.0]], [[1.0, 1.0, 3.0], [4.0, 8.0, 5.0], [6.0, 8.0, 7.0]]], [[[8.0, 2.0, 
5.0], [6.0, 7.0, 6.0], [1.0, 1.0, 2.0]], [[3.0, 5.0, 7.0], [1.0, 7.0, 4.0], [1.0, 3.0, 2.0]], [[3.0, 5.0, 8.0], [4.0, 7.0, 
7.0], [8.0, 2.0, 3.0]]], [[[1.0, 6.0, 7.0], [3.0, 5.0, 6.0], [5.0, 2.0, 7.0]], [[8.0, 3.0, 5.0], [2.0, 8.0, 1.0], [6.0, 4.0, 
8.0]], [[1.0, 7.0, 6.0], [8.0, 4.0, 4.0], [6.0, 3.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 1.0], [8.0, 7.0], [3.0, 7.0]], [[4.0, 1.0], [5.0, 8.0], [7.0, 2.0]], [[3.0, 8.0], [7.0, 
2.0], [3.0, 6.0]], [[7.0, 4.0], [6.0, 8.0], [1.0, 2.0]]])+(1.-msk_val)*numpy.array([[[3.0, 5.0], [2.0, 1.0], [6.0, 3.0]], 
[[2.0, 3.0], [1.0, 1.0], [8.0, 6.0]], [[7.0, 8.0], [6.0, 8.0], [1.0, 2.0]], [[6.0, 7.0], [2.0, 3.0], [7.0, 1.0]]])
      arg[:,:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 3.0], [7.0, 1.0, 5.0], [1.0, 7.0, 5.0]], [[6.0, 8.0, 4.0], [5.0, 7.0, 2.0], [7.0, 
7.0, 1.0]], [[2.0, 3.0, 2.0], [3.0, 7.0, 5.0], [8.0, 7.0, 2.0]]], [[[1.0, 4.0, 4.0], [5.0, 1.0, 2.0], [8.0, 5.0, 6.0]], [[6.0, 
5.0, 2.0], [7.0, 8.0, 1.0], [2.0, 8.0, 2.0]], [[8.0, 7.0, 3.0], [8.0, 2.0, 8.0], [7.0, 6.0, 3.0]]], [[[8.0, 3.0, 4.0], [8.0, 
8.0, 7.0], [8.0, 5.0, 7.0]], [[5.0, 7.0, 6.0], [1.0, 2.0, 8.0], [7.0, 8.0, 3.0]], [[7.0, 3.0, 8.0], [8.0, 6.0, 8.0], [7.0, 6.0, 
8.0]]], [[[4.0, 7.0, 5.0], [8.0, 4.0, 3.0], [2.0, 2.0, 6.0]], [[1.0, 6.0, 4.0], [3.0, 8.0, 2.0], [5.0, 2.0, 6.0]], [[4.0, 1.0, 
6.0], [1.0, 2.0, 3.0], [5.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 6.0], [6.0, 5.0, 5.0], [4.0, 8.0, 2.0]], 
[[7.0, 2.0, 5.0], [1.0, 1.0, 6.0], [1.0, 7.0, 5.0]], [[3.0, 6.0, 1.0], [3.0, 3.0, 5.0], [1.0, 4.0, 6.0]]], [[[2.0, 2.0, 5.0], 
[4.0, 3.0, 3.0], [1.0, 2.0, 1.0]], [[7.0, 1.0, 6.0], [4.0, 1.0, 8.0], [3.0, 7.0, 3.0]], [[1.0, 8.0, 3.0], [4.0, 6.0, 5.0], 
[6.0, 8.0, 7.0]]], [[[8.0, 7.0, 5.0], [6.0, 8.0, 6.0], [1.0, 1.0, 2.0]], [[3.0, 6.0, 7.0], [1.0, 8.0, 4.0], [1.0, 3.0, 2.0]], 
[[3.0, 1.0, 8.0], [4.0, 2.0, 7.0], [8.0, 2.0, 3.0]]], [[[1.0, 6.0, 7.0], [3.0, 7.0, 6.0], [5.0, 2.0, 7.0]], [[8.0, 2.0, 5.0], 
[2.0, 3.0, 1.0], [6.0, 4.0, 8.0]], [[1.0, 7.0, 6.0], [8.0, 1.0, 4.0], [6.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RLUN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 5.0], [8.0, 2.0, 2.0], [7.0, 5.0, 6.0]], [[4.0, 5.0, 3.0], [4.0, 6.0, 5.0], [7.0, 4.0, 
7.0]], [[3.0, 8.0, 6.0], [8.0, 5.0, 7.0], [4.0, 5.0, 4.0]]], [[[5.0, 7.0, 7.0], [8.0, 7.0, 5.0], [7.0, 4.0, 7.0]], [[8.0, 2.0, 
8.0], [1.0, 5.0, 4.0], [1.0, 7.0, 3.0]], [[6.0, 7.0, 7.0], [3.0, 7.0, 2.0], [1.0, 6.0, 2.0]]], [[[8.0, 7.0, 7.0], [7.0, 3.0, 
3.0], [4.0, 2.0, 7.0]], [[1.0, 6.0, 8.0], [7.0, 7.0, 4.0], [4.0, 5.0, 3.0]], [[5.0, 3.0, 8.0], [3.0, 5.0, 2.0], [4.0, 6.0, 
1.0]]], [[[2.0, 3.0, 8.0], [1.0, 7.0, 3.0], [6.0, 5.0, 4.0]], [[3.0, 6.0, 3.0], [4.0, 2.0, 2.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 
4.0], [1.0, 4.0, 6.0], [6.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 5.0], [5.0, 5.0, 8.0], [4.0, 2.0, 7.0]], [[4.0, 6.0, 4.0], [7.0, 8.0, 
4.0], [6.0, 2.0, 5.0]], [[1.0, 2.0, 1.0], [3.0, 2.0, 8.0], [3.0, 5.0, 5.0]]], [[[2.0, 7.0, 1.0], [8.0, 3.0, 4.0], [7.0, 5.0, 
7.0]], [[7.0, 4.0, 1.0], [6.0, 8.0, 7.0], [7.0, 4.0, 4.0]], [[8.0, 4.0, 5.0], [7.0, 2.0, 1.0], [2.0, 4.0, 4.0]]], [[[3.0, 3.0, 
4.0], [8.0, 4.0, 7.0], [2.0, 2.0, 4.0]], [[1.0, 6.0, 5.0], [5.0, 6.0, 8.0], [3.0, 5.0, 8.0]], [[1.0, 1.0, 5.0], [2.0, 1.0, 
5.0], [1.0, 3.0, 2.0]]], [[[7.0, 3.0, 2.0], [2.0, 8.0, 8.0], [5.0, 1.0, 3.0]], [[6.0, 4.0, 4.0], [3.0, 6.0, 2.0], [2.0, 6.0, 
5.0]], [[8.0, 7.0, 4.0], [4.0, 3.0, 3.0], [7.0, 3.0, 8.0]]]]))
      val=numpy.array([[[[4.0, 6.0, 2.0], [6.0, 1.0, 7.0]]], [[[5.0, 6.0, 5.0], [1.0, 6.0, 7.0]]]])
      arg[1:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 5.0], [8.0, 2.0, 2.0], [7.0, 5.0, 6.0]], [[4.0, 5.0, 3.0], [4.0, 6.0, 5.0], [7.0, 
4.0, 7.0]], [[3.0, 8.0, 6.0], [8.0, 5.0, 7.0], [4.0, 5.0, 4.0]]], [[[5.0, 7.0, 7.0], [8.0, 7.0, 5.0], [7.0, 4.0, 7.0]], [[8.0, 
2.0, 8.0], [1.0, 5.0, 4.0], [1.0, 7.0, 3.0]], [[4.0, 6.0, 2.0], [6.0, 1.0, 7.0], [1.0, 6.0, 2.0]]], [[[8.0, 7.0, 7.0], [7.0, 
3.0, 3.0], [4.0, 2.0, 7.0]], [[1.0, 6.0, 8.0], [7.0, 7.0, 4.0], [4.0, 5.0, 3.0]], [[5.0, 6.0, 5.0], [1.0, 6.0, 7.0], [4.0, 6.0, 
1.0]]], [[[2.0, 3.0, 8.0], [1.0, 7.0, 3.0], [6.0, 5.0, 4.0]], [[3.0, 6.0, 3.0], [4.0, 2.0, 2.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 
4.0], [1.0, 4.0, 6.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 5.0], [5.0, 5.0, 8.0], [4.0, 2.0, 7.0]], 
[[4.0, 6.0, 4.0], [7.0, 8.0, 4.0], [6.0, 2.0, 5.0]], [[1.0, 2.0, 1.0], [3.0, 2.0, 8.0], [3.0, 5.0, 5.0]]], [[[2.0, 7.0, 1.0], 
[8.0, 3.0, 4.0], [7.0, 5.0, 7.0]], [[7.0, 4.0, 1.0], [6.0, 8.0, 7.0], [7.0, 4.0, 4.0]], [[4.0, 6.0, 2.0], [6.0, 1.0, 7.0], 
[2.0, 4.0, 4.0]]], [[[3.0, 3.0, 4.0], [8.0, 4.0, 7.0], [2.0, 2.0, 4.0]], [[1.0, 6.0, 5.0], [5.0, 6.0, 8.0], [3.0, 5.0, 8.0]], 
[[5.0, 6.0, 5.0], [1.0, 6.0, 7.0], [1.0, 3.0, 2.0]]], [[[7.0, 3.0, 2.0], [2.0, 8.0, 8.0], [5.0, 1.0, 3.0]], [[6.0, 4.0, 4.0], 
[3.0, 6.0, 2.0], [2.0, 6.0, 5.0]], [[8.0, 7.0, 4.0], [4.0, 3.0, 3.0], [7.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RLUN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 5.0], [8.0, 3.0, 3.0], [8.0, 4.0, 5.0]], [[8.0, 6.0, 2.0], [8.0, 5.0, 1.0], [5.0, 7.0, 
3.0]], [[5.0, 5.0, 1.0], [1.0, 8.0, 7.0], [6.0, 8.0, 1.0]]], [[[1.0, 8.0, 3.0], [4.0, 4.0, 3.0], [4.0, 8.0, 4.0]], [[6.0, 6.0, 
7.0], [2.0, 1.0, 5.0], [8.0, 2.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 5.0, 7.0], [1.0, 1.0, 5.0]]], [[[8.0, 7.0, 6.0], [3.0, 8.0, 
2.0], [3.0, 3.0, 1.0]], [[7.0, 5.0, 5.0], [1.0, 5.0, 8.0], [2.0, 5.0, 1.0]], [[3.0, 4.0, 5.0], [1.0, 5.0, 5.0], [2.0, 5.0, 
4.0]]], [[[5.0, 4.0, 3.0], [8.0, 4.0, 4.0], [7.0, 8.0, 8.0]], [[6.0, 1.0, 3.0], [7.0, 6.0, 5.0], [7.0, 1.0, 8.0]], [[2.0, 4.0, 
2.0], [7.0, 5.0, 5.0], [2.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 7.0], [3.0, 2.0, 4.0], [2.0, 5.0, 5.0]], [[6.0, 8.0, 6.0], [6.0, 8.0, 
2.0], [6.0, 4.0, 1.0]], [[3.0, 3.0, 4.0], [2.0, 4.0, 1.0], [1.0, 4.0, 3.0]]], [[[3.0, 1.0, 2.0], [1.0, 5.0, 6.0], [4.0, 7.0, 
2.0]], [[3.0, 7.0, 3.0], [8.0, 8.0, 5.0], [6.0, 2.0, 5.0]], [[6.0, 1.0, 2.0], [6.0, 1.0, 6.0], [5.0, 8.0, 3.0]]], [[[7.0, 4.0, 
8.0], [8.0, 4.0, 6.0], [7.0, 7.0, 4.0]], [[3.0, 7.0, 7.0], [2.0, 8.0, 1.0], [8.0, 2.0, 2.0]], [[4.0, 6.0, 7.0], [3.0, 4.0, 
8.0], [3.0, 2.0, 5.0]]], [[[2.0, 5.0, 3.0], [7.0, 4.0, 3.0], [5.0, 5.0, 3.0]], [[1.0, 7.0, 6.0], [5.0, 6.0, 4.0], [3.0, 5.0, 
7.0]], [[4.0, 4.0, 5.0], [1.0, 2.0, 8.0], [7.0, 3.0, 3.0]]]]))
      val=Data(numpy.array([[[[5.0, 2.0, 6.0], [6.0, 7.0, 3.0]]], [[[4.0, 3.0, 5.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      arg[1:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 5.0], [8.0, 3.0, 3.0], [8.0, 4.0, 5.0]], [[8.0, 6.0, 2.0], [8.0, 5.0, 1.0], [5.0, 
7.0, 3.0]], [[5.0, 5.0, 1.0], [1.0, 8.0, 7.0], [6.0, 8.0, 1.0]]], [[[1.0, 8.0, 3.0], [4.0, 4.0, 3.0], [4.0, 8.0, 4.0]], [[6.0, 
6.0, 7.0], [2.0, 1.0, 5.0], [8.0, 2.0, 7.0]], [[5.0, 2.0, 6.0], [6.0, 7.0, 3.0], [1.0, 1.0, 5.0]]], [[[8.0, 7.0, 6.0], [3.0, 
8.0, 2.0], [3.0, 3.0, 1.0]], [[7.0, 5.0, 5.0], [1.0, 5.0, 8.0], [2.0, 5.0, 1.0]], [[4.0, 3.0, 5.0], [2.0, 3.0, 4.0], [2.0, 5.0, 
4.0]]], [[[5.0, 4.0, 3.0], [8.0, 4.0, 4.0], [7.0, 8.0, 8.0]], [[6.0, 1.0, 3.0], [7.0, 6.0, 5.0], [7.0, 1.0, 8.0]], [[2.0, 4.0, 
2.0], [7.0, 5.0, 5.0], [2.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 7.0], [3.0, 2.0, 4.0], [2.0, 5.0, 5.0]], 
[[6.0, 8.0, 6.0], [6.0, 8.0, 2.0], [6.0, 4.0, 1.0]], [[3.0, 3.0, 4.0], [2.0, 4.0, 1.0], [1.0, 4.0, 3.0]]], [[[3.0, 1.0, 2.0], 
[1.0, 5.0, 6.0], [4.0, 7.0, 2.0]], [[3.0, 7.0, 3.0], [8.0, 8.0, 5.0], [6.0, 2.0, 5.0]], [[5.0, 2.0, 6.0], [6.0, 7.0, 3.0], 
[5.0, 8.0, 3.0]]], [[[7.0, 4.0, 8.0], [8.0, 4.0, 6.0], [7.0, 7.0, 4.0]], [[3.0, 7.0, 7.0], [2.0, 8.0, 1.0], [8.0, 2.0, 2.0]], 
[[4.0, 3.0, 5.0], [2.0, 3.0, 4.0], [3.0, 2.0, 5.0]]], [[[2.0, 5.0, 3.0], [7.0, 4.0, 3.0], [5.0, 5.0, 3.0]], [[1.0, 7.0, 6.0], 
[5.0, 6.0, 4.0], [3.0, 5.0, 7.0]], [[4.0, 4.0, 5.0], [1.0, 2.0, 8.0], [7.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RLUN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 8.0], [1.0, 7.0, 7.0], [3.0, 2.0, 3.0]], [[3.0, 8.0, 8.0], [4.0, 5.0, 8.0], [8.0, 8.0, 
2.0]], [[8.0, 1.0, 8.0], [1.0, 1.0, 5.0], [7.0, 5.0, 3.0]]], [[[4.0, 2.0, 2.0], [2.0, 1.0, 6.0], [3.0, 5.0, 5.0]], [[5.0, 6.0, 
2.0], [1.0, 2.0, 8.0], [1.0, 6.0, 7.0]], [[7.0, 4.0, 5.0], [7.0, 4.0, 3.0], [1.0, 5.0, 2.0]]], [[[7.0, 4.0, 1.0], [2.0, 2.0, 
6.0], [2.0, 2.0, 2.0]], [[5.0, 5.0, 2.0], [2.0, 5.0, 2.0], [5.0, 2.0, 8.0]], [[3.0, 7.0, 2.0], [5.0, 6.0, 6.0], [7.0, 7.0, 
8.0]]], [[[5.0, 7.0, 1.0], [8.0, 4.0, 3.0], [5.0, 8.0, 4.0]], [[1.0, 6.0, 5.0], [6.0, 5.0, 6.0], [6.0, 5.0, 3.0]], [[2.0, 6.0, 
8.0], [4.0, 5.0, 5.0], [5.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 5.0], [3.0, 2.0, 7.0], [6.0, 7.0, 3.0]], [[6.0, 3.0, 8.0], [3.0, 1.0, 
7.0], [2.0, 1.0, 4.0]], [[7.0, 8.0, 5.0], [5.0, 3.0, 7.0], [1.0, 7.0, 2.0]]], [[[5.0, 4.0, 2.0], [8.0, 2.0, 8.0], [3.0, 2.0, 
2.0]], [[7.0, 8.0, 1.0], [3.0, 7.0, 5.0], [8.0, 2.0, 2.0]], [[5.0, 4.0, 8.0], [3.0, 6.0, 6.0], [4.0, 8.0, 1.0]]], [[[3.0, 1.0, 
3.0], [6.0, 6.0, 4.0], [1.0, 4.0, 7.0]], [[8.0, 6.0, 1.0], [5.0, 1.0, 6.0], [8.0, 2.0, 6.0]], [[6.0, 7.0, 4.0], [6.0, 6.0, 
8.0], [3.0, 3.0, 3.0]]], [[[2.0, 1.0, 8.0], [4.0, 8.0, 6.0], [3.0, 4.0, 4.0]], [[6.0, 7.0, 2.0], [1.0, 8.0, 2.0], [1.0, 3.0, 
2.0]], [[1.0, 8.0, 3.0], [8.0, 7.0, 2.0], [2.0, 8.0, 8.0]]]]))
      val=Data(numpy.array([[[[6.0, 3.0, 3.0], [8.0, 7.0, 7.0]]], [[[1.0, 3.0, 4.0], [4.0, 5.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 1.0], [6.0, 1.0, 7.0]]], [[[4.0, 5.0, 2.0], [6.0, 6.0, 7.0]]]]))
      arg[1:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 8.0], [1.0, 7.0, 7.0], [3.0, 2.0, 3.0]], [[3.0, 8.0, 8.0], [4.0, 5.0, 8.0], [8.0, 
8.0, 2.0]], [[8.0, 1.0, 8.0], [1.0, 1.0, 5.0], [7.0, 5.0, 3.0]]], [[[4.0, 2.0, 2.0], [2.0, 1.0, 6.0], [3.0, 5.0, 5.0]], [[5.0, 
6.0, 2.0], [1.0, 2.0, 8.0], [1.0, 6.0, 7.0]], [[6.0, 3.0, 3.0], [8.0, 7.0, 7.0], [1.0, 5.0, 2.0]]], [[[7.0, 4.0, 1.0], [2.0, 
2.0, 6.0], [2.0, 2.0, 2.0]], [[5.0, 5.0, 2.0], [2.0, 5.0, 2.0], [5.0, 2.0, 8.0]], [[1.0, 3.0, 4.0], [4.0, 5.0, 7.0], [7.0, 7.0, 
8.0]]], [[[5.0, 7.0, 1.0], [8.0, 4.0, 3.0], [5.0, 8.0, 4.0]], [[1.0, 6.0, 5.0], [6.0, 5.0, 6.0], [6.0, 5.0, 3.0]], [[2.0, 6.0, 
8.0], [4.0, 5.0, 5.0], [5.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 5.0], [3.0, 2.0, 7.0], [6.0, 7.0, 3.0]], 
[[6.0, 3.0, 8.0], [3.0, 1.0, 7.0], [2.0, 1.0, 4.0]], [[7.0, 8.0, 5.0], [5.0, 3.0, 7.0], [1.0, 7.0, 2.0]]], [[[5.0, 4.0, 2.0], 
[8.0, 2.0, 8.0], [3.0, 2.0, 2.0]], [[7.0, 8.0, 1.0], [3.0, 7.0, 5.0], [8.0, 2.0, 2.0]], [[8.0, 4.0, 1.0], [6.0, 1.0, 7.0], 
[4.0, 8.0, 1.0]]], [[[3.0, 1.0, 3.0], [6.0, 6.0, 4.0], [1.0, 4.0, 7.0]], [[8.0, 6.0, 1.0], [5.0, 1.0, 6.0], [8.0, 2.0, 6.0]], 
[[4.0, 5.0, 2.0], [6.0, 6.0, 7.0], [3.0, 3.0, 3.0]]], [[[2.0, 1.0, 8.0], [4.0, 8.0, 6.0], [3.0, 4.0, 4.0]], [[6.0, 7.0, 2.0], 
[1.0, 8.0, 2.0], [1.0, 3.0, 2.0]], [[1.0, 8.0, 3.0], [8.0, 7.0, 2.0], [2.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RLUN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 5.0], [6.0, 1.0, 2.0], [5.0, 6.0, 5.0]], [[3.0, 6.0, 5.0], [8.0, 1.0, 6.0], [6.0, 3.0, 
4.0]], [[6.0, 2.0, 3.0], [5.0, 5.0, 2.0], [3.0, 1.0, 6.0]]], [[[2.0, 4.0, 3.0], [2.0, 4.0, 7.0], [8.0, 5.0, 4.0]], [[6.0, 3.0, 
5.0], [5.0, 5.0, 5.0], [2.0, 2.0, 2.0]], [[4.0, 2.0, 2.0], [3.0, 5.0, 2.0], [4.0, 1.0, 6.0]]], [[[8.0, 1.0, 5.0], [4.0, 5.0, 
7.0], [5.0, 5.0, 7.0]], [[5.0, 1.0, 8.0], [7.0, 7.0, 6.0], [1.0, 1.0, 7.0]], [[6.0, 7.0, 6.0], [2.0, 6.0, 1.0], [8.0, 7.0, 
2.0]]], [[[8.0, 7.0, 6.0], [7.0, 4.0, 5.0], [1.0, 5.0, 4.0]], [[8.0, 1.0, 3.0], [2.0, 2.0, 6.0], [8.0, 3.0, 3.0]], [[7.0, 4.0, 
6.0], [2.0, 4.0, 2.0], [2.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 7.0], [6.0, 8.0, 8.0], [6.0, 1.0, 3.0]], [[6.0, 3.0, 6.0], [4.0, 6.0, 
6.0], [6.0, 4.0, 2.0]], [[7.0, 3.0, 6.0], [5.0, 2.0, 2.0], [6.0, 3.0, 1.0]]], [[[8.0, 5.0, 1.0], [8.0, 2.0, 6.0], [8.0, 4.0, 
8.0]], [[8.0, 2.0, 4.0], [3.0, 4.0, 2.0], [2.0, 4.0, 4.0]], [[5.0, 6.0, 7.0], [3.0, 8.0, 3.0], [5.0, 5.0, 1.0]]], [[[1.0, 1.0, 
8.0], [6.0, 3.0, 2.0], [2.0, 1.0, 3.0]], [[1.0, 3.0, 5.0], [2.0, 6.0, 2.0], [6.0, 5.0, 2.0]], [[5.0, 2.0, 8.0], [3.0, 5.0, 
2.0], [3.0, 5.0, 4.0]]], [[[7.0, 7.0, 4.0], [2.0, 6.0, 5.0], [5.0, 4.0, 4.0]], [[8.0, 3.0, 8.0], [3.0, 2.0, 2.0], [7.0, 5.0, 
7.0]], [[7.0, 5.0, 5.0], [8.0, 8.0, 1.0], [6.0, 5.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 4.0, 2.0], [6.0, 2.0, 1.0]]], [[[3.0, 7.0, 5.0], [4.0, 3.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 7.0, 7.0], [4.0, 2.0, 1.0]]], [[[6.0, 2.0, 2.0], [2.0, 2.0, 3.0]]]])
      arg[1:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 5.0], [6.0, 1.0, 2.0], [5.0, 6.0, 5.0]], [[3.0, 6.0, 5.0], [8.0, 1.0, 6.0], [6.0, 
3.0, 4.0]], [[6.0, 2.0, 3.0], [5.0, 5.0, 2.0], [3.0, 1.0, 6.0]]], [[[2.0, 4.0, 3.0], [2.0, 4.0, 7.0], [8.0, 5.0, 4.0]], [[6.0, 
3.0, 5.0], [5.0, 5.0, 5.0], [2.0, 2.0, 2.0]], [[5.0, 4.0, 2.0], [6.0, 2.0, 1.0], [4.0, 1.0, 6.0]]], [[[8.0, 1.0, 5.0], [4.0, 
5.0, 7.0], [5.0, 5.0, 7.0]], [[5.0, 1.0, 8.0], [7.0, 7.0, 6.0], [1.0, 1.0, 7.0]], [[3.0, 7.0, 5.0], [4.0, 3.0, 2.0], [8.0, 7.0, 
2.0]]], [[[8.0, 7.0, 6.0], [7.0, 4.0, 5.0], [1.0, 5.0, 4.0]], [[8.0, 1.0, 3.0], [2.0, 2.0, 6.0], [8.0, 3.0, 3.0]], [[7.0, 4.0, 
6.0], [2.0, 4.0, 2.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 7.0], [6.0, 8.0, 8.0], [6.0, 1.0, 3.0]], 
[[6.0, 3.0, 6.0], [4.0, 6.0, 6.0], [6.0, 4.0, 2.0]], [[7.0, 3.0, 6.0], [5.0, 2.0, 2.0], [6.0, 3.0, 1.0]]], [[[8.0, 5.0, 1.0], 
[8.0, 2.0, 6.0], [8.0, 4.0, 8.0]], [[8.0, 2.0, 4.0], [3.0, 4.0, 2.0], [2.0, 4.0, 4.0]], [[4.0, 7.0, 7.0], [4.0, 2.0, 1.0], 
[5.0, 5.0, 1.0]]], [[[1.0, 1.0, 8.0], [6.0, 3.0, 2.0], [2.0, 1.0, 3.0]], [[1.0, 3.0, 5.0], [2.0, 6.0, 2.0], [6.0, 5.0, 2.0]], 
[[6.0, 2.0, 2.0], [2.0, 2.0, 3.0], [3.0, 5.0, 4.0]]], [[[7.0, 7.0, 4.0], [2.0, 6.0, 5.0], [5.0, 4.0, 4.0]], [[8.0, 3.0, 8.0], 
[3.0, 2.0, 2.0], [7.0, 5.0, 7.0]], [[7.0, 5.0, 5.0], [8.0, 8.0, 1.0], [6.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PRRC(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 4.0], [4.0, 8.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 5.0, 8.0], [5.0, 1.0, 7.0], [1.0, 6.0, 
2.0]], [[7.0, 2.0, 5.0], [3.0, 5.0, 7.0], [7.0, 4.0, 4.0]]], [[[4.0, 8.0, 2.0], [8.0, 6.0, 5.0], [7.0, 8.0, 5.0]], [[7.0, 8.0, 
7.0], [5.0, 3.0, 1.0], [6.0, 2.0, 1.0]], [[5.0, 4.0, 5.0], [4.0, 5.0, 2.0], [2.0, 2.0, 2.0]]], [[[8.0, 1.0, 7.0], [7.0, 7.0, 
3.0], [1.0, 4.0, 5.0]], [[5.0, 4.0, 5.0], [3.0, 6.0, 1.0], [1.0, 1.0, 3.0]], [[7.0, 8.0, 2.0], [6.0, 8.0, 8.0], [3.0, 4.0, 
1.0]]], [[[8.0, 8.0, 1.0], [6.0, 7.0, 6.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 7.0], [1.0, 1.0, 3.0], [8.0, 7.0, 4.0]], [[5.0, 5.0, 
6.0], [7.0, 1.0, 8.0], [3.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 3.0], [5.0, 4.0, 6.0], [4.0, 5.0, 5.0]], [[8.0, 1.0, 6.0], [3.0, 1.0, 
8.0], [7.0, 6.0, 3.0]], [[1.0, 4.0, 1.0], [7.0, 8.0, 8.0], [7.0, 8.0, 1.0]]], [[[4.0, 1.0, 6.0], [1.0, 4.0, 2.0], [6.0, 3.0, 
4.0]], [[8.0, 1.0, 7.0], [4.0, 8.0, 1.0], [4.0, 6.0, 7.0]], [[7.0, 8.0, 6.0], [5.0, 8.0, 8.0], [2.0, 6.0, 8.0]]], [[[5.0, 1.0, 
4.0], [5.0, 7.0, 7.0], [5.0, 6.0, 7.0]], [[7.0, 6.0, 2.0], [6.0, 3.0, 4.0], [7.0, 2.0, 7.0]], [[1.0, 5.0, 6.0], [6.0, 4.0, 
4.0], [4.0, 2.0, 6.0]]], [[[1.0, 6.0, 1.0], [3.0, 5.0, 2.0], [5.0, 6.0, 3.0]], [[5.0, 3.0, 6.0], [6.0, 1.0, 6.0], [1.0, 4.0, 
2.0]], [[8.0, 5.0, 2.0], [7.0, 8.0, 2.0], [7.0, 2.0, 4.0]]]]))
      val=numpy.array([[[4.0, 3.0, 6.0]]])
      arg[2,1:2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 4.0], [4.0, 8.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 5.0, 8.0], [5.0, 1.0, 7.0], [1.0, 
6.0, 2.0]], [[7.0, 2.0, 5.0], [3.0, 5.0, 7.0], [7.0, 4.0, 4.0]]], [[[4.0, 8.0, 2.0], [8.0, 6.0, 5.0], [7.0, 8.0, 5.0]], [[7.0, 
8.0, 7.0], [5.0, 3.0, 1.0], [6.0, 2.0, 1.0]], [[5.0, 4.0, 5.0], [4.0, 5.0, 2.0], [2.0, 2.0, 2.0]]], [[[8.0, 1.0, 7.0], [7.0, 
7.0, 3.0], [1.0, 4.0, 5.0]], [[5.0, 4.0, 5.0], [4.0, 3.0, 6.0], [1.0, 1.0, 3.0]], [[7.0, 8.0, 2.0], [6.0, 8.0, 8.0], [3.0, 4.0, 
1.0]]], [[[8.0, 8.0, 1.0], [6.0, 7.0, 6.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 7.0], [1.0, 1.0, 3.0], [8.0, 7.0, 4.0]], [[5.0, 5.0, 
6.0], [7.0, 1.0, 8.0], [3.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 3.0], [5.0, 4.0, 6.0], [4.0, 5.0, 5.0]], 
[[8.0, 1.0, 6.0], [3.0, 1.0, 8.0], [7.0, 6.0, 3.0]], [[1.0, 4.0, 1.0], [7.0, 8.0, 8.0], [7.0, 8.0, 1.0]]], [[[4.0, 1.0, 6.0], 
[1.0, 4.0, 2.0], [6.0, 3.0, 4.0]], [[8.0, 1.0, 7.0], [4.0, 8.0, 1.0], [4.0, 6.0, 7.0]], [[7.0, 8.0, 6.0], [5.0, 8.0, 8.0], 
[2.0, 6.0, 8.0]]], [[[5.0, 1.0, 4.0], [5.0, 7.0, 7.0], [5.0, 6.0, 7.0]], [[7.0, 6.0, 2.0], [4.0, 3.0, 6.0], [7.0, 2.0, 7.0]], 
[[1.0, 5.0, 6.0], [6.0, 4.0, 4.0], [4.0, 2.0, 6.0]]], [[[1.0, 6.0, 1.0], [3.0, 5.0, 2.0], [5.0, 6.0, 3.0]], [[5.0, 3.0, 6.0], 
[6.0, 1.0, 6.0], [1.0, 4.0, 2.0]], [[8.0, 5.0, 2.0], [7.0, 8.0, 2.0], [7.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PRRC(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 8.0], [1.0, 2.0, 6.0], [6.0, 6.0, 4.0]], [[2.0, 8.0, 2.0], [3.0, 3.0, 5.0], [7.0, 6.0, 
6.0]], [[1.0, 3.0, 6.0], [1.0, 1.0, 7.0], [3.0, 2.0, 1.0]]], [[[2.0, 7.0, 7.0], [2.0, 8.0, 3.0], [5.0, 8.0, 2.0]], [[6.0, 2.0, 
7.0], [7.0, 5.0, 8.0], [2.0, 3.0, 8.0]], [[7.0, 7.0, 6.0], [3.0, 4.0, 3.0], [5.0, 2.0, 8.0]]], [[[3.0, 6.0, 2.0], [1.0, 2.0, 
8.0], [1.0, 6.0, 7.0]], [[5.0, 8.0, 2.0], [5.0, 4.0, 5.0], [2.0, 7.0, 4.0]], [[3.0, 4.0, 1.0], [3.0, 4.0, 3.0], [8.0, 5.0, 
8.0]]], [[[8.0, 5.0, 2.0], [6.0, 2.0, 5.0], [5.0, 2.0, 1.0]], [[6.0, 3.0, 4.0], [7.0, 5.0, 6.0], [6.0, 8.0, 5.0]], [[2.0, 2.0, 
8.0], [6.0, 3.0, 8.0], [8.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 4.0], [1.0, 4.0, 7.0], [1.0, 3.0, 2.0]], [[6.0, 1.0, 8.0], [4.0, 8.0, 
4.0], [7.0, 8.0, 4.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 6.0], [8.0, 1.0, 3.0]]], [[[1.0, 1.0, 7.0], [7.0, 2.0, 2.0], [4.0, 8.0, 
1.0]], [[7.0, 4.0, 3.0], [1.0, 5.0, 4.0], [7.0, 7.0, 3.0]], [[6.0, 3.0, 2.0], [3.0, 7.0, 5.0], [7.0, 7.0, 6.0]]], [[[1.0, 1.0, 
4.0], [4.0, 7.0, 8.0], [5.0, 4.0, 3.0]], [[1.0, 5.0, 8.0], [7.0, 8.0, 3.0], [2.0, 8.0, 2.0]], [[7.0, 6.0, 8.0], [1.0, 4.0, 
2.0], [2.0, 5.0, 6.0]]], [[[6.0, 8.0, 3.0], [4.0, 6.0, 6.0], [8.0, 7.0, 3.0]], [[4.0, 3.0, 3.0], [5.0, 3.0, 1.0], [2.0, 6.0, 
7.0]], [[8.0, 4.0, 7.0], [8.0, 8.0, 8.0], [6.0, 7.0, 4.0]]]]))
      val=Data(numpy.array([[[1.0, 6.0, 7.0]]]),self.functionspace)
      arg[2,1:2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 8.0], [1.0, 2.0, 6.0], [6.0, 6.0, 4.0]], [[2.0, 8.0, 2.0], [3.0, 3.0, 5.0], [7.0, 
6.0, 6.0]], [[1.0, 3.0, 6.0], [1.0, 1.0, 7.0], [3.0, 2.0, 1.0]]], [[[2.0, 7.0, 7.0], [2.0, 8.0, 3.0], [5.0, 8.0, 2.0]], [[6.0, 
2.0, 7.0], [7.0, 5.0, 8.0], [2.0, 3.0, 8.0]], [[7.0, 7.0, 6.0], [3.0, 4.0, 3.0], [5.0, 2.0, 8.0]]], [[[3.0, 6.0, 2.0], [1.0, 
2.0, 8.0], [1.0, 6.0, 7.0]], [[5.0, 8.0, 2.0], [1.0, 6.0, 7.0], [2.0, 7.0, 4.0]], [[3.0, 4.0, 1.0], [3.0, 4.0, 3.0], [8.0, 5.0, 
8.0]]], [[[8.0, 5.0, 2.0], [6.0, 2.0, 5.0], [5.0, 2.0, 1.0]], [[6.0, 3.0, 4.0], [7.0, 5.0, 6.0], [6.0, 8.0, 5.0]], [[2.0, 2.0, 
8.0], [6.0, 3.0, 8.0], [8.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 4.0], [1.0, 4.0, 7.0], [1.0, 3.0, 2.0]], 
[[6.0, 1.0, 8.0], [4.0, 8.0, 4.0], [7.0, 8.0, 4.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 6.0], [8.0, 1.0, 3.0]]], [[[1.0, 1.0, 7.0], 
[7.0, 2.0, 2.0], [4.0, 8.0, 1.0]], [[7.0, 4.0, 3.0], [1.0, 5.0, 4.0], [7.0, 7.0, 3.0]], [[6.0, 3.0, 2.0], [3.0, 7.0, 5.0], 
[7.0, 7.0, 6.0]]], [[[1.0, 1.0, 4.0], [4.0, 7.0, 8.0], [5.0, 4.0, 3.0]], [[1.0, 5.0, 8.0], [1.0, 6.0, 7.0], [2.0, 8.0, 2.0]], 
[[7.0, 6.0, 8.0], [1.0, 4.0, 2.0], [2.0, 5.0, 6.0]]], [[[6.0, 8.0, 3.0], [4.0, 6.0, 6.0], [8.0, 7.0, 3.0]], [[4.0, 3.0, 3.0], 
[5.0, 3.0, 1.0], [2.0, 6.0, 7.0]], [[8.0, 4.0, 7.0], [8.0, 8.0, 8.0], [6.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PRRC(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 2.0], [1.0, 5.0, 1.0], [7.0, 8.0, 3.0]], [[3.0, 4.0, 2.0], [2.0, 1.0, 3.0], [1.0, 8.0, 
4.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 4.0], [5.0, 6.0, 4.0]]], [[[8.0, 7.0, 5.0], [3.0, 2.0, 2.0], [1.0, 3.0, 5.0]], [[8.0, 1.0, 
4.0], [5.0, 3.0, 8.0], [6.0, 3.0, 2.0]], [[3.0, 7.0, 4.0], [8.0, 8.0, 7.0], [5.0, 4.0, 2.0]]], [[[4.0, 1.0, 4.0], [4.0, 6.0, 
4.0], [2.0, 5.0, 1.0]], [[1.0, 2.0, 3.0], [3.0, 3.0, 8.0], [8.0, 5.0, 4.0]], [[2.0, 4.0, 8.0], [7.0, 4.0, 5.0], [7.0, 3.0, 
3.0]]], [[[5.0, 6.0, 8.0], [6.0, 4.0, 4.0], [6.0, 3.0, 7.0]], [[4.0, 1.0, 1.0], [5.0, 3.0, 5.0], [4.0, 8.0, 1.0]], [[4.0, 3.0, 
6.0], [5.0, 2.0, 8.0], [6.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 6.0], [4.0, 5.0, 7.0], [8.0, 4.0, 3.0]], [[2.0, 8.0, 1.0], [1.0, 1.0, 
1.0], [5.0, 6.0, 8.0]], [[8.0, 7.0, 3.0], [8.0, 3.0, 3.0], [6.0, 2.0, 5.0]]], [[[8.0, 7.0, 8.0], [3.0, 3.0, 7.0], [6.0, 6.0, 
2.0]], [[5.0, 6.0, 8.0], [7.0, 6.0, 7.0], [5.0, 4.0, 5.0]], [[6.0, 1.0, 8.0], [4.0, 5.0, 2.0], [2.0, 8.0, 3.0]]], [[[8.0, 7.0, 
2.0], [1.0, 2.0, 4.0], [5.0, 5.0, 3.0]], [[5.0, 6.0, 5.0], [5.0, 2.0, 6.0], [2.0, 3.0, 2.0]], [[7.0, 2.0, 7.0], [7.0, 1.0, 
1.0], [8.0, 6.0, 1.0]]], [[[4.0, 7.0, 2.0], [3.0, 1.0, 8.0], [5.0, 2.0, 8.0]], [[6.0, 3.0, 4.0], [7.0, 5.0, 2.0], [6.0, 7.0, 
6.0]], [[1.0, 2.0, 4.0], [1.0, 4.0, 8.0], [7.0, 2.0, 2.0]]]]))
      val=Data(numpy.array([[[4.0, 5.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 6.0, 3.0]]]))
      arg[2,1:2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 2.0], [1.0, 5.0, 1.0], [7.0, 8.0, 3.0]], [[3.0, 4.0, 2.0], [2.0, 1.0, 3.0], [1.0, 
8.0, 4.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 4.0], [5.0, 6.0, 4.0]]], [[[8.0, 7.0, 5.0], [3.0, 2.0, 2.0], [1.0, 3.0, 5.0]], [[8.0, 
1.0, 4.0], [5.0, 3.0, 8.0], [6.0, 3.0, 2.0]], [[3.0, 7.0, 4.0], [8.0, 8.0, 7.0], [5.0, 4.0, 2.0]]], [[[4.0, 1.0, 4.0], [4.0, 
6.0, 4.0], [2.0, 5.0, 1.0]], [[1.0, 2.0, 3.0], [4.0, 5.0, 1.0], [8.0, 5.0, 4.0]], [[2.0, 4.0, 8.0], [7.0, 4.0, 5.0], [7.0, 3.0, 
3.0]]], [[[5.0, 6.0, 8.0], [6.0, 4.0, 4.0], [6.0, 3.0, 7.0]], [[4.0, 1.0, 1.0], [5.0, 3.0, 5.0], [4.0, 8.0, 1.0]], [[4.0, 3.0, 
6.0], [5.0, 2.0, 8.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 5.0, 7.0], [8.0, 4.0, 3.0]], 
[[2.0, 8.0, 1.0], [1.0, 1.0, 1.0], [5.0, 6.0, 8.0]], [[8.0, 7.0, 3.0], [8.0, 3.0, 3.0], [6.0, 2.0, 5.0]]], [[[8.0, 7.0, 8.0], 
[3.0, 3.0, 7.0], [6.0, 6.0, 2.0]], [[5.0, 6.0, 8.0], [7.0, 6.0, 7.0], [5.0, 4.0, 5.0]], [[6.0, 1.0, 8.0], [4.0, 5.0, 2.0], 
[2.0, 8.0, 3.0]]], [[[8.0, 7.0, 2.0], [1.0, 2.0, 4.0], [5.0, 5.0, 3.0]], [[5.0, 6.0, 5.0], [7.0, 6.0, 3.0], [2.0, 3.0, 2.0]], 
[[7.0, 2.0, 7.0], [7.0, 1.0, 1.0], [8.0, 6.0, 1.0]]], [[[4.0, 7.0, 2.0], [3.0, 1.0, 8.0], [5.0, 2.0, 8.0]], [[6.0, 3.0, 4.0], 
[7.0, 5.0, 2.0], [6.0, 7.0, 6.0]], [[1.0, 2.0, 4.0], [1.0, 4.0, 8.0], [7.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PRRC(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 5.0], [4.0, 8.0, 3.0], [3.0, 2.0, 1.0]], [[4.0, 5.0, 3.0], [3.0, 2.0, 8.0], [3.0, 7.0, 
1.0]], [[4.0, 3.0, 7.0], [7.0, 4.0, 3.0], [4.0, 2.0, 4.0]]], [[[6.0, 2.0, 1.0], [3.0, 7.0, 4.0], [7.0, 8.0, 1.0]], [[6.0, 7.0, 
3.0], [8.0, 8.0, 3.0], [8.0, 6.0, 8.0]], [[1.0, 6.0, 5.0], [8.0, 8.0, 5.0], [1.0, 8.0, 8.0]]], [[[2.0, 3.0, 8.0], [7.0, 1.0, 
3.0], [1.0, 5.0, 4.0]], [[5.0, 2.0, 1.0], [4.0, 2.0, 8.0], [1.0, 7.0, 3.0]], [[7.0, 8.0, 2.0], [8.0, 6.0, 1.0], [2.0, 5.0, 
5.0]]], [[[1.0, 5.0, 1.0], [4.0, 8.0, 7.0], [4.0, 3.0, 3.0]], [[1.0, 1.0, 5.0], [2.0, 1.0, 3.0], [7.0, 4.0, 5.0]], [[4.0, 5.0, 
3.0], [6.0, 7.0, 4.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 3.0], [7.0, 3.0, 4.0], [7.0, 7.0, 6.0]], [[1.0, 7.0, 4.0], [3.0, 4.0, 
8.0], [5.0, 1.0, 7.0]], [[4.0, 5.0, 7.0], [5.0, 6.0, 5.0], [5.0, 3.0, 4.0]]], [[[8.0, 1.0, 5.0], [4.0, 1.0, 4.0], [5.0, 7.0, 
3.0]], [[4.0, 6.0, 3.0], [2.0, 7.0, 3.0], [5.0, 1.0, 3.0]], [[8.0, 5.0, 4.0], [6.0, 6.0, 8.0], [7.0, 5.0, 4.0]]], [[[1.0, 2.0, 
1.0], [3.0, 2.0, 8.0], [2.0, 6.0, 7.0]], [[7.0, 1.0, 1.0], [2.0, 5.0, 3.0], [2.0, 8.0, 6.0]], [[4.0, 6.0, 2.0], [2.0, 1.0, 
8.0], [6.0, 4.0, 3.0]]], [[[8.0, 6.0, 5.0], [2.0, 3.0, 4.0], [7.0, 5.0, 2.0]], [[5.0, 3.0, 8.0], [3.0, 4.0, 8.0], [5.0, 4.0, 
1.0]], [[2.0, 4.0, 5.0], [6.0, 1.0, 6.0], [5.0, 2.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0, 4.0, 6.0]]])+(1.-msk_val)*numpy.array([[[7.0, 1.0, 7.0]]])
      arg[2,1:2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 5.0], [4.0, 8.0, 3.0], [3.0, 2.0, 1.0]], [[4.0, 5.0, 3.0], [3.0, 2.0, 8.0], [3.0, 
7.0, 1.0]], [[4.0, 3.0, 7.0], [7.0, 4.0, 3.0], [4.0, 2.0, 4.0]]], [[[6.0, 2.0, 1.0], [3.0, 7.0, 4.0], [7.0, 8.0, 1.0]], [[6.0, 
7.0, 3.0], [8.0, 8.0, 3.0], [8.0, 6.0, 8.0]], [[1.0, 6.0, 5.0], [8.0, 8.0, 5.0], [1.0, 8.0, 8.0]]], [[[2.0, 3.0, 8.0], [7.0, 
1.0, 3.0], [1.0, 5.0, 4.0]], [[5.0, 2.0, 1.0], [1.0, 4.0, 6.0], [1.0, 7.0, 3.0]], [[7.0, 8.0, 2.0], [8.0, 6.0, 1.0], [2.0, 5.0, 
5.0]]], [[[1.0, 5.0, 1.0], [4.0, 8.0, 7.0], [4.0, 3.0, 3.0]], [[1.0, 1.0, 5.0], [2.0, 1.0, 3.0], [7.0, 4.0, 5.0]], [[4.0, 5.0, 
3.0], [6.0, 7.0, 4.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 3.0], [7.0, 3.0, 4.0], [7.0, 7.0, 6.0]], 
[[1.0, 7.0, 4.0], [3.0, 4.0, 8.0], [5.0, 1.0, 7.0]], [[4.0, 5.0, 7.0], [5.0, 6.0, 5.0], [5.0, 3.0, 4.0]]], [[[8.0, 1.0, 5.0], 
[4.0, 1.0, 4.0], [5.0, 7.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 7.0, 3.0], [5.0, 1.0, 3.0]], [[8.0, 5.0, 4.0], [6.0, 6.0, 8.0], 
[7.0, 5.0, 4.0]]], [[[1.0, 2.0, 1.0], [3.0, 2.0, 8.0], [2.0, 6.0, 7.0]], [[7.0, 1.0, 1.0], [7.0, 1.0, 7.0], [2.0, 8.0, 6.0]], 
[[4.0, 6.0, 2.0], [2.0, 1.0, 8.0], [6.0, 4.0, 3.0]]], [[[8.0, 6.0, 5.0], [2.0, 3.0, 4.0], [7.0, 5.0, 2.0]], [[5.0, 3.0, 8.0], 
[3.0, 4.0, 8.0], [5.0, 4.0, 1.0]], [[2.0, 4.0, 5.0], [6.0, 1.0, 6.0], [5.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LUUP(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 4.0], [6.0, 4.0, 6.0], [8.0, 2.0, 3.0]], [[5.0, 5.0, 6.0], [1.0, 3.0, 5.0], [7.0, 8.0, 
1.0]], [[7.0, 4.0, 6.0], [7.0, 5.0, 5.0], [2.0, 5.0, 6.0]]], [[[8.0, 4.0, 2.0], [7.0, 6.0, 5.0], [3.0, 3.0, 8.0]], [[5.0, 6.0, 
1.0], [8.0, 3.0, 1.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 2.0], [6.0, 1.0, 3.0], [4.0, 7.0, 4.0]]], [[[4.0, 7.0, 2.0], [3.0, 2.0, 
8.0], [4.0, 3.0, 2.0]], [[1.0, 7.0, 7.0], [2.0, 4.0, 4.0], [5.0, 8.0, 5.0]], [[5.0, 5.0, 7.0], [4.0, 4.0, 8.0], [3.0, 4.0, 
8.0]]], [[[4.0, 3.0, 5.0], [2.0, 2.0, 5.0], [2.0, 4.0, 4.0]], [[4.0, 2.0, 6.0], [8.0, 2.0, 5.0], [4.0, 2.0, 6.0]], [[2.0, 7.0, 
4.0], [3.0, 8.0, 1.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 5.0], [2.0, 7.0, 3.0], [3.0, 6.0, 5.0]], [[2.0, 6.0, 8.0], [1.0, 4.0, 
2.0], [3.0, 6.0, 5.0]], [[3.0, 1.0, 7.0], [3.0, 6.0, 4.0], [6.0, 6.0, 8.0]]], [[[3.0, 2.0, 7.0], [8.0, 5.0, 5.0], [5.0, 1.0, 
7.0]], [[1.0, 6.0, 2.0], [3.0, 8.0, 7.0], [8.0, 2.0, 7.0]], [[3.0, 7.0, 2.0], [4.0, 7.0, 8.0], [8.0, 5.0, 4.0]]], [[[2.0, 7.0, 
2.0], [7.0, 3.0, 3.0], [7.0, 8.0, 4.0]], [[5.0, 7.0, 1.0], [6.0, 6.0, 7.0], [4.0, 3.0, 6.0]], [[6.0, 1.0, 3.0], [6.0, 7.0, 
7.0], [1.0, 3.0, 2.0]]], [[[4.0, 1.0, 2.0], [7.0, 6.0, 1.0], [5.0, 2.0, 5.0]], [[2.0, 2.0, 4.0], [3.0, 4.0, 8.0], [3.0, 7.0, 
6.0]], [[1.0, 3.0, 3.0], [5.0, 4.0, 6.0], [8.0, 4.0, 3.0]]]]))
      val=numpy.array([[[8.0, 7.0], [8.0, 7.0]], [[7.0, 7.0], [4.0, 1.0]]])
      arg[2:,:2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 4.0], [6.0, 4.0, 6.0], [8.0, 2.0, 3.0]], [[5.0, 5.0, 6.0], [1.0, 3.0, 5.0], [7.0, 
8.0, 1.0]], [[7.0, 4.0, 6.0], [7.0, 5.0, 5.0], [2.0, 5.0, 6.0]]], [[[8.0, 4.0, 2.0], [7.0, 6.0, 5.0], [3.0, 3.0, 8.0]], [[5.0, 
6.0, 1.0], [8.0, 3.0, 1.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 2.0], [6.0, 1.0, 3.0], [4.0, 7.0, 4.0]]], [[[4.0, 8.0, 2.0], [3.0, 
7.0, 8.0], [4.0, 3.0, 2.0]], [[1.0, 8.0, 7.0], [2.0, 7.0, 4.0], [5.0, 8.0, 5.0]], [[5.0, 5.0, 7.0], [4.0, 4.0, 8.0], [3.0, 4.0, 
8.0]]], [[[4.0, 7.0, 5.0], [2.0, 7.0, 5.0], [2.0, 4.0, 4.0]], [[4.0, 4.0, 6.0], [8.0, 1.0, 5.0], [4.0, 2.0, 6.0]], [[2.0, 7.0, 
4.0], [3.0, 8.0, 1.0], [2.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 5.0], [2.0, 7.0, 3.0], [3.0, 6.0, 5.0]], 
[[2.0, 6.0, 8.0], [1.0, 4.0, 2.0], [3.0, 6.0, 5.0]], [[3.0, 1.0, 7.0], [3.0, 6.0, 4.0], [6.0, 6.0, 8.0]]], [[[3.0, 2.0, 7.0], 
[8.0, 5.0, 5.0], [5.0, 1.0, 7.0]], [[1.0, 6.0, 2.0], [3.0, 8.0, 7.0], [8.0, 2.0, 7.0]], [[3.0, 7.0, 2.0], [4.0, 7.0, 8.0], 
[8.0, 5.0, 4.0]]], [[[2.0, 8.0, 2.0], [7.0, 7.0, 3.0], [7.0, 8.0, 4.0]], [[5.0, 8.0, 1.0], [6.0, 7.0, 7.0], [4.0, 3.0, 6.0]], 
[[6.0, 1.0, 3.0], [6.0, 7.0, 7.0], [1.0, 3.0, 2.0]]], [[[4.0, 7.0, 2.0], [7.0, 7.0, 1.0], [5.0, 2.0, 5.0]], [[2.0, 4.0, 4.0], 
[3.0, 1.0, 8.0], [3.0, 7.0, 6.0]], [[1.0, 3.0, 3.0], [5.0, 4.0, 6.0], [8.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LUUP(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 4.0], [3.0, 7.0, 8.0], [3.0, 1.0, 3.0]], [[2.0, 2.0, 1.0], [6.0, 1.0, 5.0], [2.0, 1.0, 
2.0]], [[8.0, 6.0, 2.0], [4.0, 7.0, 1.0], [6.0, 4.0, 2.0]]], [[[8.0, 8.0, 7.0], [6.0, 8.0, 2.0], [6.0, 6.0, 2.0]], [[5.0, 6.0, 
7.0], [6.0, 4.0, 8.0], [7.0, 6.0, 1.0]], [[4.0, 8.0, 6.0], [7.0, 3.0, 6.0], [3.0, 3.0, 4.0]]], [[[1.0, 5.0, 8.0], [3.0, 5.0, 
7.0], [1.0, 6.0, 5.0]], [[3.0, 1.0, 3.0], [1.0, 1.0, 2.0], [1.0, 4.0, 7.0]], [[4.0, 2.0, 4.0], [8.0, 8.0, 7.0], [3.0, 2.0, 
3.0]]], [[[4.0, 3.0, 2.0], [8.0, 7.0, 7.0], [2.0, 4.0, 2.0]], [[2.0, 6.0, 5.0], [7.0, 2.0, 8.0], [2.0, 7.0, 1.0]], [[4.0, 5.0, 
7.0], [8.0, 8.0, 1.0], [6.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 5.0], [8.0, 3.0, 7.0], [5.0, 6.0, 5.0]], [[3.0, 5.0, 5.0], [6.0, 4.0, 
1.0], [4.0, 7.0, 5.0]], [[3.0, 6.0, 3.0], [3.0, 5.0, 5.0], [4.0, 2.0, 8.0]]], [[[8.0, 3.0, 1.0], [1.0, 7.0, 6.0], [7.0, 1.0, 
7.0]], [[7.0, 5.0, 4.0], [2.0, 1.0, 6.0], [3.0, 4.0, 5.0]], [[2.0, 8.0, 5.0], [7.0, 1.0, 3.0], [6.0, 4.0, 1.0]]], [[[3.0, 3.0, 
1.0], [2.0, 3.0, 4.0], [3.0, 8.0, 7.0]], [[8.0, 8.0, 6.0], [8.0, 1.0, 1.0], [8.0, 6.0, 3.0]], [[4.0, 7.0, 7.0], [2.0, 5.0, 
5.0], [5.0, 6.0, 4.0]]], [[[2.0, 3.0, 7.0], [8.0, 8.0, 3.0], [4.0, 3.0, 7.0]], [[3.0, 7.0, 6.0], [6.0, 6.0, 5.0], [6.0, 5.0, 
7.0]], [[8.0, 5.0, 2.0], [7.0, 8.0, 5.0], [6.0, 8.0, 6.0]]]]))
      val=Data(numpy.array([[[4.0, 2.0], [4.0, 1.0]], [[3.0, 4.0], [7.0, 1.0]]]),self.functionspace)
      arg[2:,:2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 4.0], [3.0, 7.0, 8.0], [3.0, 1.0, 3.0]], [[2.0, 2.0, 1.0], [6.0, 1.0, 5.0], [2.0, 
1.0, 2.0]], [[8.0, 6.0, 2.0], [4.0, 7.0, 1.0], [6.0, 4.0, 2.0]]], [[[8.0, 8.0, 7.0], [6.0, 8.0, 2.0], [6.0, 6.0, 2.0]], [[5.0, 
6.0, 7.0], [6.0, 4.0, 8.0], [7.0, 6.0, 1.0]], [[4.0, 8.0, 6.0], [7.0, 3.0, 6.0], [3.0, 3.0, 4.0]]], [[[1.0, 4.0, 8.0], [3.0, 
2.0, 7.0], [1.0, 6.0, 5.0]], [[3.0, 4.0, 3.0], [1.0, 1.0, 2.0], [1.0, 4.0, 7.0]], [[4.0, 2.0, 4.0], [8.0, 8.0, 7.0], [3.0, 2.0, 
3.0]]], [[[4.0, 3.0, 2.0], [8.0, 4.0, 7.0], [2.0, 4.0, 2.0]], [[2.0, 7.0, 5.0], [7.0, 1.0, 8.0], [2.0, 7.0, 1.0]], [[4.0, 5.0, 
7.0], [8.0, 8.0, 1.0], [6.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 5.0], [8.0, 3.0, 7.0], [5.0, 6.0, 5.0]], 
[[3.0, 5.0, 5.0], [6.0, 4.0, 1.0], [4.0, 7.0, 5.0]], [[3.0, 6.0, 3.0], [3.0, 5.0, 5.0], [4.0, 2.0, 8.0]]], [[[8.0, 3.0, 1.0], 
[1.0, 7.0, 6.0], [7.0, 1.0, 7.0]], [[7.0, 5.0, 4.0], [2.0, 1.0, 6.0], [3.0, 4.0, 5.0]], [[2.0, 8.0, 5.0], [7.0, 1.0, 3.0], 
[6.0, 4.0, 1.0]]], [[[3.0, 4.0, 1.0], [2.0, 2.0, 4.0], [3.0, 8.0, 7.0]], [[8.0, 4.0, 6.0], [8.0, 1.0, 1.0], [8.0, 6.0, 3.0]], 
[[4.0, 7.0, 7.0], [2.0, 5.0, 5.0], [5.0, 6.0, 4.0]]], [[[2.0, 3.0, 7.0], [8.0, 4.0, 3.0], [4.0, 3.0, 7.0]], [[3.0, 7.0, 6.0], 
[6.0, 1.0, 5.0], [6.0, 5.0, 7.0]], [[8.0, 5.0, 2.0], [7.0, 8.0, 5.0], [6.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LUUP(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 4.0], [2.0, 4.0, 3.0], [5.0, 7.0, 3.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 2.0], [6.0, 3.0, 
2.0]], [[7.0, 3.0, 5.0], [2.0, 3.0, 1.0], [1.0, 4.0, 6.0]]], [[[6.0, 7.0, 8.0], [8.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[6.0, 4.0, 
5.0], [1.0, 1.0, 3.0], [5.0, 5.0, 8.0]], [[5.0, 3.0, 2.0], [2.0, 6.0, 7.0], [7.0, 4.0, 4.0]]], [[[7.0, 6.0, 3.0], [7.0, 8.0, 
4.0], [8.0, 3.0, 3.0]], [[1.0, 1.0, 5.0], [3.0, 4.0, 6.0], [5.0, 8.0, 7.0]], [[3.0, 8.0, 2.0], [8.0, 3.0, 3.0], [7.0, 5.0, 
3.0]]], [[[6.0, 8.0, 1.0], [2.0, 3.0, 4.0], [2.0, 7.0, 3.0]], [[2.0, 6.0, 5.0], [7.0, 4.0, 2.0], [6.0, 6.0, 7.0]], [[6.0, 5.0, 
2.0], [7.0, 3.0, 3.0], [6.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 4.0], [3.0, 3.0, 2.0], [5.0, 3.0, 8.0]], [[3.0, 1.0, 7.0], [2.0, 4.0, 
6.0], [6.0, 5.0, 2.0]], [[1.0, 8.0, 4.0], [7.0, 5.0, 2.0], [3.0, 6.0, 2.0]]], [[[1.0, 5.0, 7.0], [4.0, 6.0, 1.0], [2.0, 3.0, 
5.0]], [[2.0, 3.0, 5.0], [7.0, 5.0, 4.0], [7.0, 1.0, 3.0]], [[6.0, 2.0, 8.0], [3.0, 3.0, 7.0], [1.0, 3.0, 5.0]]], [[[4.0, 6.0, 
8.0], [3.0, 2.0, 1.0], [7.0, 6.0, 6.0]], [[3.0, 6.0, 8.0], [6.0, 5.0, 8.0], [3.0, 1.0, 3.0]], [[1.0, 5.0, 6.0], [8.0, 8.0, 
7.0], [1.0, 3.0, 5.0]]], [[[1.0, 6.0, 6.0], [2.0, 3.0, 6.0], [1.0, 8.0, 2.0]], [[5.0, 2.0, 8.0], [3.0, 3.0, 1.0], [8.0, 7.0, 
3.0]], [[1.0, 6.0, 3.0], [1.0, 8.0, 4.0], [3.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[[7.0, 7.0], [4.0, 6.0]], [[3.0, 5.0], [1.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 5.0], [5.0, 5.0]], [[6.0, 7.0], [8.0, 7.0]]]))
      arg[2:,:2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 4.0], [2.0, 4.0, 3.0], [5.0, 7.0, 3.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 2.0], [6.0, 
3.0, 2.0]], [[7.0, 3.0, 5.0], [2.0, 3.0, 1.0], [1.0, 4.0, 6.0]]], [[[6.0, 7.0, 8.0], [8.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[6.0, 
4.0, 5.0], [1.0, 1.0, 3.0], [5.0, 5.0, 8.0]], [[5.0, 3.0, 2.0], [2.0, 6.0, 7.0], [7.0, 4.0, 4.0]]], [[[7.0, 7.0, 3.0], [7.0, 
7.0, 4.0], [8.0, 3.0, 3.0]], [[1.0, 4.0, 5.0], [3.0, 6.0, 6.0], [5.0, 8.0, 7.0]], [[3.0, 8.0, 2.0], [8.0, 3.0, 3.0], [7.0, 5.0, 
3.0]]], [[[6.0, 3.0, 1.0], [2.0, 5.0, 4.0], [2.0, 7.0, 3.0]], [[2.0, 1.0, 5.0], [7.0, 8.0, 2.0], [6.0, 6.0, 7.0]], [[6.0, 5.0, 
2.0], [7.0, 3.0, 3.0], [6.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 4.0], [3.0, 3.0, 2.0], [5.0, 3.0, 8.0]], 
[[3.0, 1.0, 7.0], [2.0, 4.0, 6.0], [6.0, 5.0, 2.0]], [[1.0, 8.0, 4.0], [7.0, 5.0, 2.0], [3.0, 6.0, 2.0]]], [[[1.0, 5.0, 7.0], 
[4.0, 6.0, 1.0], [2.0, 3.0, 5.0]], [[2.0, 3.0, 5.0], [7.0, 5.0, 4.0], [7.0, 1.0, 3.0]], [[6.0, 2.0, 8.0], [3.0, 3.0, 7.0], 
[1.0, 3.0, 5.0]]], [[[4.0, 2.0, 8.0], [3.0, 5.0, 1.0], [7.0, 6.0, 6.0]], [[3.0, 5.0, 8.0], [6.0, 5.0, 8.0], [3.0, 1.0, 3.0]], 
[[1.0, 5.0, 6.0], [8.0, 8.0, 7.0], [1.0, 3.0, 5.0]]], [[[1.0, 6.0, 6.0], [2.0, 7.0, 6.0], [1.0, 8.0, 2.0]], [[5.0, 8.0, 8.0], 
[3.0, 7.0, 1.0], [8.0, 7.0, 3.0]], [[1.0, 6.0, 3.0], [1.0, 8.0, 4.0], [3.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LUUP(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 1.0], [5.0, 6.0, 2.0], [2.0, 2.0, 3.0]], [[1.0, 7.0, 8.0], [3.0, 6.0, 3.0], [5.0, 7.0, 
2.0]], [[1.0, 2.0, 2.0], [6.0, 8.0, 3.0], [3.0, 8.0, 6.0]]], [[[7.0, 8.0, 5.0], [7.0, 6.0, 8.0], [3.0, 7.0, 6.0]], [[2.0, 4.0, 
4.0], [2.0, 2.0, 1.0], [8.0, 1.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 4.0, 6.0], [2.0, 3.0, 1.0]]], [[[2.0, 8.0, 2.0], [3.0, 8.0, 
5.0], [8.0, 6.0, 1.0]], [[5.0, 8.0, 1.0], [4.0, 7.0, 2.0], [2.0, 6.0, 5.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 1.0], [6.0, 2.0, 
5.0]]], [[[6.0, 1.0, 3.0], [7.0, 5.0, 8.0], [2.0, 1.0, 6.0]], [[6.0, 2.0, 4.0], [1.0, 1.0, 3.0], [6.0, 6.0, 5.0]], [[6.0, 7.0, 
6.0], [7.0, 7.0, 6.0], [4.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 6.0], [3.0, 7.0, 6.0], [2.0, 1.0, 6.0]], [[3.0, 4.0, 1.0], [8.0, 5.0, 
2.0], [8.0, 8.0, 8.0]], [[8.0, 2.0, 5.0], [6.0, 7.0, 7.0], [4.0, 6.0, 3.0]]], [[[1.0, 8.0, 3.0], [3.0, 3.0, 2.0], [8.0, 6.0, 
7.0]], [[6.0, 4.0, 3.0], [3.0, 6.0, 1.0], [2.0, 8.0, 7.0]], [[5.0, 3.0, 6.0], [5.0, 5.0, 5.0], [7.0, 2.0, 2.0]]], [[[1.0, 6.0, 
1.0], [2.0, 2.0, 6.0], [3.0, 6.0, 3.0]], [[6.0, 7.0, 4.0], [7.0, 2.0, 4.0], [5.0, 7.0, 3.0]], [[8.0, 6.0, 5.0], [5.0, 7.0, 
8.0], [7.0, 8.0, 1.0]]], [[[7.0, 6.0, 1.0], [2.0, 8.0, 8.0], [2.0, 6.0, 1.0]], [[7.0, 4.0, 6.0], [2.0, 4.0, 8.0], [2.0, 4.0, 
1.0]], [[4.0, 5.0, 3.0], [3.0, 3.0, 8.0], [5.0, 5.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 6.0], [7.0, 8.0]], [[6.0, 3.0], [7.0, 5.0]]])+(1.-msk_val)*numpy.array([[[3.0, 4.0], 
[6.0, 4.0]], [[8.0, 4.0], [8.0, 7.0]]])
      arg[2:,:2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 1.0], [5.0, 6.0, 2.0], [2.0, 2.0, 3.0]], [[1.0, 7.0, 8.0], [3.0, 6.0, 3.0], [5.0, 
7.0, 2.0]], [[1.0, 2.0, 2.0], [6.0, 8.0, 3.0], [3.0, 8.0, 6.0]]], [[[7.0, 8.0, 5.0], [7.0, 6.0, 8.0], [3.0, 7.0, 6.0]], [[2.0, 
4.0, 4.0], [2.0, 2.0, 1.0], [8.0, 1.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 4.0, 6.0], [2.0, 3.0, 1.0]]], [[[2.0, 3.0, 2.0], [3.0, 
6.0, 5.0], [8.0, 6.0, 1.0]], [[5.0, 7.0, 1.0], [4.0, 8.0, 2.0], [2.0, 6.0, 5.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 1.0], [6.0, 2.0, 
5.0]]], [[[6.0, 6.0, 3.0], [7.0, 3.0, 8.0], [2.0, 1.0, 6.0]], [[6.0, 7.0, 4.0], [1.0, 5.0, 3.0], [6.0, 6.0, 5.0]], [[6.0, 7.0, 
6.0], [7.0, 7.0, 6.0], [4.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 6.0], [3.0, 7.0, 6.0], [2.0, 1.0, 6.0]], 
[[3.0, 4.0, 1.0], [8.0, 5.0, 2.0], [8.0, 8.0, 8.0]], [[8.0, 2.0, 5.0], [6.0, 7.0, 7.0], [4.0, 6.0, 3.0]]], [[[1.0, 8.0, 3.0], 
[3.0, 3.0, 2.0], [8.0, 6.0, 7.0]], [[6.0, 4.0, 3.0], [3.0, 6.0, 1.0], [2.0, 8.0, 7.0]], [[5.0, 3.0, 6.0], [5.0, 5.0, 5.0], 
[7.0, 2.0, 2.0]]], [[[1.0, 3.0, 1.0], [2.0, 4.0, 6.0], [3.0, 6.0, 3.0]], [[6.0, 6.0, 4.0], [7.0, 4.0, 4.0], [5.0, 7.0, 3.0]], 
[[8.0, 6.0, 5.0], [5.0, 7.0, 8.0], [7.0, 8.0, 1.0]]], [[[7.0, 8.0, 1.0], [2.0, 4.0, 8.0], [2.0, 6.0, 1.0]], [[7.0, 8.0, 6.0], 
[2.0, 7.0, 8.0], [2.0, 4.0, 1.0]], [[4.0, 5.0, 3.0], [3.0, 3.0, 8.0], [5.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPUC(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 4.0], [1.0, 6.0, 4.0], [7.0, 2.0, 8.0]], [[1.0, 2.0, 2.0], [1.0, 3.0, 4.0], [1.0, 8.0, 
4.0]], [[2.0, 4.0, 1.0], [6.0, 6.0, 6.0], [8.0, 8.0, 4.0]]], [[[6.0, 5.0, 4.0], [1.0, 3.0, 8.0], [1.0, 8.0, 4.0]], [[3.0, 2.0, 
2.0], [3.0, 8.0, 3.0], [8.0, 7.0, 1.0]], [[8.0, 6.0, 2.0], [5.0, 6.0, 3.0], [5.0, 4.0, 8.0]]], [[[4.0, 7.0, 2.0], [2.0, 5.0, 
6.0], [2.0, 4.0, 1.0]], [[7.0, 1.0, 5.0], [6.0, 6.0, 7.0], [4.0, 8.0, 2.0]], [[7.0, 1.0, 7.0], [5.0, 8.0, 3.0], [6.0, 6.0, 
4.0]]], [[[6.0, 4.0, 1.0], [1.0, 3.0, 8.0], [2.0, 3.0, 3.0]], [[8.0, 8.0, 7.0], [3.0, 3.0, 1.0], [7.0, 2.0, 8.0]], [[4.0, 5.0, 
2.0], [3.0, 1.0, 3.0], [8.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 3.0], [3.0, 1.0, 6.0], [1.0, 6.0, 1.0]], [[8.0, 8.0, 6.0], [7.0, 8.0, 
1.0], [3.0, 8.0, 3.0]], [[8.0, 5.0, 6.0], [6.0, 6.0, 5.0], [5.0, 4.0, 2.0]]], [[[1.0, 2.0, 8.0], [3.0, 6.0, 7.0], [5.0, 7.0, 
4.0]], [[3.0, 2.0, 8.0], [2.0, 7.0, 6.0], [4.0, 2.0, 2.0]], [[2.0, 3.0, 2.0], [5.0, 5.0, 8.0], [4.0, 8.0, 6.0]]], [[[4.0, 4.0, 
2.0], [5.0, 5.0, 7.0], [8.0, 3.0, 8.0]], [[7.0, 3.0, 4.0], [4.0, 4.0, 3.0], [8.0, 8.0, 5.0]], [[8.0, 2.0, 5.0], [7.0, 8.0, 
8.0], [2.0, 5.0, 5.0]]], [[[2.0, 4.0, 5.0], [1.0, 6.0, 4.0], [6.0, 8.0, 3.0]], [[5.0, 4.0, 1.0], [4.0, 7.0, 1.0], [6.0, 4.0, 
1.0]], [[4.0, 8.0, 5.0], [2.0, 6.0, 8.0], [5.0, 1.0, 7.0]]]]))
      val=numpy.array([[[3.0, 7.0, 6.0], [8.0, 5.0, 2.0]], [[7.0, 5.0, 8.0], [5.0, 1.0, 6.0]]])
      arg[1:3,1,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 4.0], [1.0, 6.0, 4.0], [7.0, 2.0, 8.0]], [[1.0, 2.0, 2.0], [1.0, 3.0, 4.0], [1.0, 
8.0, 4.0]], [[2.0, 4.0, 1.0], [6.0, 6.0, 6.0], [8.0, 8.0, 4.0]]], [[[6.0, 5.0, 4.0], [1.0, 3.0, 8.0], [1.0, 8.0, 4.0]], [[3.0, 
7.0, 6.0], [8.0, 5.0, 2.0], [8.0, 7.0, 1.0]], [[8.0, 6.0, 2.0], [5.0, 6.0, 3.0], [5.0, 4.0, 8.0]]], [[[4.0, 7.0, 2.0], [2.0, 
5.0, 6.0], [2.0, 4.0, 1.0]], [[7.0, 5.0, 8.0], [5.0, 1.0, 6.0], [4.0, 8.0, 2.0]], [[7.0, 1.0, 7.0], [5.0, 8.0, 3.0], [6.0, 6.0, 
4.0]]], [[[6.0, 4.0, 1.0], [1.0, 3.0, 8.0], [2.0, 3.0, 3.0]], [[8.0, 8.0, 7.0], [3.0, 3.0, 1.0], [7.0, 2.0, 8.0]], [[4.0, 5.0, 
2.0], [3.0, 1.0, 3.0], [8.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 3.0], [3.0, 1.0, 6.0], [1.0, 6.0, 1.0]], 
[[8.0, 8.0, 6.0], [7.0, 8.0, 1.0], [3.0, 8.0, 3.0]], [[8.0, 5.0, 6.0], [6.0, 6.0, 5.0], [5.0, 4.0, 2.0]]], [[[1.0, 2.0, 8.0], 
[3.0, 6.0, 7.0], [5.0, 7.0, 4.0]], [[3.0, 7.0, 6.0], [8.0, 5.0, 2.0], [4.0, 2.0, 2.0]], [[2.0, 3.0, 2.0], [5.0, 5.0, 8.0], 
[4.0, 8.0, 6.0]]], [[[4.0, 4.0, 2.0], [5.0, 5.0, 7.0], [8.0, 3.0, 8.0]], [[7.0, 5.0, 8.0], [5.0, 1.0, 6.0], [8.0, 8.0, 5.0]], 
[[8.0, 2.0, 5.0], [7.0, 8.0, 8.0], [2.0, 5.0, 5.0]]], [[[2.0, 4.0, 5.0], [1.0, 6.0, 4.0], [6.0, 8.0, 3.0]], [[5.0, 4.0, 1.0], 
[4.0, 7.0, 1.0], [6.0, 4.0, 1.0]], [[4.0, 8.0, 5.0], [2.0, 6.0, 8.0], [5.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPUC(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 8.0], [6.0, 3.0, 6.0], [2.0, 2.0, 7.0]], [[6.0, 4.0, 8.0], [5.0, 3.0, 7.0], [6.0, 1.0, 
1.0]], [[3.0, 6.0, 4.0], [2.0, 8.0, 1.0], [4.0, 6.0, 2.0]]], [[[7.0, 6.0, 8.0], [8.0, 4.0, 5.0], [2.0, 5.0, 8.0]], [[7.0, 5.0, 
4.0], [8.0, 8.0, 5.0], [7.0, 7.0, 4.0]], [[6.0, 8.0, 4.0], [3.0, 6.0, 1.0], [2.0, 2.0, 5.0]]], [[[8.0, 1.0, 4.0], [3.0, 8.0, 
5.0], [5.0, 5.0, 3.0]], [[8.0, 7.0, 2.0], [3.0, 8.0, 4.0], [6.0, 5.0, 1.0]], [[8.0, 1.0, 3.0], [3.0, 3.0, 4.0], [5.0, 2.0, 
1.0]]], [[[5.0, 1.0, 7.0], [1.0, 5.0, 5.0], [1.0, 7.0, 4.0]], [[4.0, 4.0, 2.0], [1.0, 8.0, 2.0], [3.0, 7.0, 4.0]], [[7.0, 8.0, 
3.0], [8.0, 1.0, 6.0], [4.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 6.0], [8.0, 2.0, 6.0], [7.0, 3.0, 2.0]], [[2.0, 6.0, 7.0], [3.0, 7.0, 
2.0], [1.0, 6.0, 6.0]], [[8.0, 3.0, 7.0], [7.0, 3.0, 8.0], [8.0, 4.0, 7.0]]], [[[8.0, 1.0, 8.0], [5.0, 6.0, 8.0], [3.0, 5.0, 
7.0]], [[8.0, 4.0, 5.0], [5.0, 4.0, 6.0], [1.0, 8.0, 8.0]], [[7.0, 1.0, 1.0], [1.0, 2.0, 8.0], [1.0, 4.0, 8.0]]], [[[3.0, 8.0, 
4.0], [2.0, 6.0, 8.0], [5.0, 7.0, 7.0]], [[8.0, 6.0, 4.0], [6.0, 3.0, 7.0], [3.0, 4.0, 6.0]], [[1.0, 5.0, 7.0], [6.0, 8.0, 
2.0], [1.0, 1.0, 1.0]]], [[[8.0, 6.0, 6.0], [2.0, 8.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 1.0, 8.0], [1.0, 4.0, 3.0], [4.0, 7.0, 
3.0]], [[6.0, 6.0, 4.0], [8.0, 2.0, 8.0], [4.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[[6.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[1.0, 4.0, 4.0], [3.0, 7.0, 8.0]]]),self.functionspace)
      arg[1:3,1,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [6.0, 3.0, 6.0], [2.0, 2.0, 7.0]], [[6.0, 4.0, 8.0], [5.0, 3.0, 7.0], [6.0, 
1.0, 1.0]], [[3.0, 6.0, 4.0], [2.0, 8.0, 1.0], [4.0, 6.0, 2.0]]], [[[7.0, 6.0, 8.0], [8.0, 4.0, 5.0], [2.0, 5.0, 8.0]], [[6.0, 
2.0, 3.0], [8.0, 2.0, 7.0], [7.0, 7.0, 4.0]], [[6.0, 8.0, 4.0], [3.0, 6.0, 1.0], [2.0, 2.0, 5.0]]], [[[8.0, 1.0, 4.0], [3.0, 
8.0, 5.0], [5.0, 5.0, 3.0]], [[1.0, 4.0, 4.0], [3.0, 7.0, 8.0], [6.0, 5.0, 1.0]], [[8.0, 1.0, 3.0], [3.0, 3.0, 4.0], [5.0, 2.0, 
1.0]]], [[[5.0, 1.0, 7.0], [1.0, 5.0, 5.0], [1.0, 7.0, 4.0]], [[4.0, 4.0, 2.0], [1.0, 8.0, 2.0], [3.0, 7.0, 4.0]], [[7.0, 8.0, 
3.0], [8.0, 1.0, 6.0], [4.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 6.0], [8.0, 2.0, 6.0], [7.0, 3.0, 2.0]], 
[[2.0, 6.0, 7.0], [3.0, 7.0, 2.0], [1.0, 6.0, 6.0]], [[8.0, 3.0, 7.0], [7.0, 3.0, 8.0], [8.0, 4.0, 7.0]]], [[[8.0, 1.0, 8.0], 
[5.0, 6.0, 8.0], [3.0, 5.0, 7.0]], [[6.0, 2.0, 3.0], [8.0, 2.0, 7.0], [1.0, 8.0, 8.0]], [[7.0, 1.0, 1.0], [1.0, 2.0, 8.0], 
[1.0, 4.0, 8.0]]], [[[3.0, 8.0, 4.0], [2.0, 6.0, 8.0], [5.0, 7.0, 7.0]], [[1.0, 4.0, 4.0], [3.0, 7.0, 8.0], [3.0, 4.0, 6.0]], 
[[1.0, 5.0, 7.0], [6.0, 8.0, 2.0], [1.0, 1.0, 1.0]]], [[[8.0, 6.0, 6.0], [2.0, 8.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 1.0, 8.0], 
[1.0, 4.0, 3.0], [4.0, 7.0, 3.0]], [[6.0, 6.0, 4.0], [8.0, 2.0, 8.0], [4.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPUC(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 1.0], [3.0, 2.0, 1.0], [6.0, 6.0, 5.0]], [[3.0, 6.0, 7.0], [2.0, 8.0, 6.0], [6.0, 8.0, 
2.0]], [[1.0, 6.0, 2.0], [1.0, 4.0, 4.0], [2.0, 5.0, 8.0]]], [[[3.0, 1.0, 3.0], [4.0, 4.0, 5.0], [3.0, 3.0, 1.0]], [[1.0, 7.0, 
5.0], [4.0, 5.0, 1.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 7.0], [6.0, 5.0, 8.0], [4.0, 2.0, 4.0]]], [[[4.0, 3.0, 2.0], [5.0, 2.0, 
7.0], [6.0, 5.0, 6.0]], [[4.0, 7.0, 2.0], [4.0, 3.0, 1.0], [3.0, 3.0, 5.0]], [[4.0, 8.0, 8.0], [8.0, 7.0, 3.0], [4.0, 5.0, 
1.0]]], [[[3.0, 2.0, 5.0], [4.0, 1.0, 1.0], [2.0, 2.0, 1.0]], [[8.0, 3.0, 2.0], [5.0, 6.0, 3.0], [7.0, 2.0, 5.0]], [[1.0, 4.0, 
8.0], [4.0, 6.0, 4.0], [4.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 7.0], [4.0, 3.0, 6.0], [8.0, 8.0, 6.0]], [[6.0, 2.0, 3.0], [4.0, 4.0, 
4.0], [4.0, 7.0, 4.0]], [[8.0, 2.0, 3.0], [4.0, 2.0, 8.0], [2.0, 6.0, 6.0]]], [[[1.0, 7.0, 8.0], [6.0, 7.0, 6.0], [8.0, 2.0, 
1.0]], [[4.0, 5.0, 1.0], [8.0, 2.0, 3.0], [8.0, 7.0, 6.0]], [[2.0, 6.0, 6.0], [7.0, 7.0, 3.0], [7.0, 7.0, 8.0]]], [[[3.0, 8.0, 
4.0], [2.0, 2.0, 7.0], [4.0, 8.0, 8.0]], [[8.0, 4.0, 3.0], [3.0, 1.0, 4.0], [7.0, 3.0, 1.0]], [[8.0, 1.0, 6.0], [2.0, 5.0, 
7.0], [5.0, 7.0, 8.0]]], [[[5.0, 3.0, 3.0], [5.0, 3.0, 6.0], [5.0, 1.0, 4.0]], [[4.0, 2.0, 5.0], [2.0, 6.0, 8.0], [2.0, 4.0, 
2.0]], [[2.0, 3.0, 5.0], [4.0, 4.0, 1.0], [3.0, 4.0, 4.0]]]]))
      val=Data(numpy.array([[[7.0, 1.0, 8.0], [5.0, 6.0, 5.0]], [[4.0, 2.0, 8.0], [1.0, 6.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 7.0, 1.0], [1.0, 8.0, 2.0]], [[3.0, 5.0, 3.0], [4.0, 2.0, 3.0]]]))
      arg[1:3,1,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 1.0], [3.0, 2.0, 1.0], [6.0, 6.0, 5.0]], [[3.0, 6.0, 7.0], [2.0, 8.0, 6.0], [6.0, 
8.0, 2.0]], [[1.0, 6.0, 2.0], [1.0, 4.0, 4.0], [2.0, 5.0, 8.0]]], [[[3.0, 1.0, 3.0], [4.0, 4.0, 5.0], [3.0, 3.0, 1.0]], [[7.0, 
1.0, 8.0], [5.0, 6.0, 5.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 7.0], [6.0, 5.0, 8.0], [4.0, 2.0, 4.0]]], [[[4.0, 3.0, 2.0], [5.0, 
2.0, 7.0], [6.0, 5.0, 6.0]], [[4.0, 2.0, 8.0], [1.0, 6.0, 4.0], [3.0, 3.0, 5.0]], [[4.0, 8.0, 8.0], [8.0, 7.0, 3.0], [4.0, 5.0, 
1.0]]], [[[3.0, 2.0, 5.0], [4.0, 1.0, 1.0], [2.0, 2.0, 1.0]], [[8.0, 3.0, 2.0], [5.0, 6.0, 3.0], [7.0, 2.0, 5.0]], [[1.0, 4.0, 
8.0], [4.0, 6.0, 4.0], [4.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 7.0], [4.0, 3.0, 6.0], [8.0, 8.0, 6.0]], 
[[6.0, 2.0, 3.0], [4.0, 4.0, 4.0], [4.0, 7.0, 4.0]], [[8.0, 2.0, 3.0], [4.0, 2.0, 8.0], [2.0, 6.0, 6.0]]], [[[1.0, 7.0, 8.0], 
[6.0, 7.0, 6.0], [8.0, 2.0, 1.0]], [[6.0, 7.0, 1.0], [1.0, 8.0, 2.0], [8.0, 7.0, 6.0]], [[2.0, 6.0, 6.0], [7.0, 7.0, 3.0], 
[7.0, 7.0, 8.0]]], [[[3.0, 8.0, 4.0], [2.0, 2.0, 7.0], [4.0, 8.0, 8.0]], [[3.0, 5.0, 3.0], [4.0, 2.0, 3.0], [7.0, 3.0, 1.0]], 
[[8.0, 1.0, 6.0], [2.0, 5.0, 7.0], [5.0, 7.0, 8.0]]], [[[5.0, 3.0, 3.0], [5.0, 3.0, 6.0], [5.0, 1.0, 4.0]], [[4.0, 2.0, 5.0], 
[2.0, 6.0, 8.0], [2.0, 4.0, 2.0]], [[2.0, 3.0, 5.0], [4.0, 4.0, 1.0], [3.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPUC(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 3.0], [4.0, 5.0, 5.0], [4.0, 2.0, 5.0]], [[1.0, 6.0, 1.0], [1.0, 6.0, 2.0], [3.0, 5.0, 
5.0]], [[8.0, 5.0, 5.0], [3.0, 8.0, 6.0], [8.0, 3.0, 6.0]]], [[[8.0, 5.0, 5.0], [7.0, 3.0, 8.0], [1.0, 2.0, 2.0]], [[4.0, 6.0, 
1.0], [8.0, 2.0, 1.0], [8.0, 6.0, 5.0]], [[2.0, 2.0, 8.0], [6.0, 5.0, 4.0], [6.0, 7.0, 4.0]]], [[[6.0, 2.0, 1.0], [7.0, 5.0, 
4.0], [2.0, 2.0, 8.0]], [[7.0, 5.0, 7.0], [6.0, 2.0, 2.0], [2.0, 3.0, 8.0]], [[2.0, 8.0, 6.0], [1.0, 3.0, 2.0], [7.0, 3.0, 
7.0]]], [[[4.0, 7.0, 5.0], [1.0, 5.0, 1.0], [1.0, 3.0, 1.0]], [[8.0, 4.0, 5.0], [5.0, 2.0, 7.0], [8.0, 2.0, 1.0]], [[7.0, 8.0, 
2.0], [7.0, 1.0, 6.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 1.0], [4.0, 3.0, 7.0], [5.0, 7.0, 6.0]], [[5.0, 7.0, 5.0], [5.0, 3.0, 
8.0], [4.0, 4.0, 1.0]], [[5.0, 4.0, 2.0], [3.0, 6.0, 4.0], [5.0, 8.0, 5.0]]], [[[1.0, 2.0, 1.0], [6.0, 6.0, 3.0], [3.0, 6.0, 
8.0]], [[5.0, 5.0, 7.0], [2.0, 8.0, 1.0], [3.0, 3.0, 3.0]], [[3.0, 3.0, 1.0], [8.0, 6.0, 3.0], [2.0, 7.0, 7.0]]], [[[7.0, 2.0, 
6.0], [2.0, 4.0, 3.0], [4.0, 2.0, 4.0]], [[1.0, 5.0, 7.0], [8.0, 6.0, 7.0], [8.0, 3.0, 5.0]], [[8.0, 7.0, 3.0], [1.0, 6.0, 
8.0], [7.0, 2.0, 5.0]]], [[[1.0, 4.0, 8.0], [8.0, 1.0, 5.0], [1.0, 8.0, 4.0]], [[2.0, 5.0, 2.0], [4.0, 3.0, 6.0], [4.0, 1.0, 
3.0]], [[1.0, 6.0, 7.0], [8.0, 5.0, 6.0], [4.0, 1.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 8.0, 2.0], [8.0, 2.0, 8.0]], [[8.0, 3.0, 6.0], [3.0, 5.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[4.0, 8.0, 3.0], [8.0, 8.0, 3.0]], [[8.0, 5.0, 5.0], [6.0, 6.0, 3.0]]])
      arg[1:3,1,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 3.0], [4.0, 5.0, 5.0], [4.0, 2.0, 5.0]], [[1.0, 6.0, 1.0], [1.0, 6.0, 2.0], [3.0, 
5.0, 5.0]], [[8.0, 5.0, 5.0], [3.0, 8.0, 6.0], [8.0, 3.0, 6.0]]], [[[8.0, 5.0, 5.0], [7.0, 3.0, 8.0], [1.0, 2.0, 2.0]], [[7.0, 
8.0, 2.0], [8.0, 2.0, 8.0], [8.0, 6.0, 5.0]], [[2.0, 2.0, 8.0], [6.0, 5.0, 4.0], [6.0, 7.0, 4.0]]], [[[6.0, 2.0, 1.0], [7.0, 
5.0, 4.0], [2.0, 2.0, 8.0]], [[8.0, 3.0, 6.0], [3.0, 5.0, 6.0], [2.0, 3.0, 8.0]], [[2.0, 8.0, 6.0], [1.0, 3.0, 2.0], [7.0, 3.0, 
7.0]]], [[[4.0, 7.0, 5.0], [1.0, 5.0, 1.0], [1.0, 3.0, 1.0]], [[8.0, 4.0, 5.0], [5.0, 2.0, 7.0], [8.0, 2.0, 1.0]], [[7.0, 8.0, 
2.0], [7.0, 1.0, 6.0], [7.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 1.0], [4.0, 3.0, 7.0], [5.0, 7.0, 6.0]], 
[[5.0, 7.0, 5.0], [5.0, 3.0, 8.0], [4.0, 4.0, 1.0]], [[5.0, 4.0, 2.0], [3.0, 6.0, 4.0], [5.0, 8.0, 5.0]]], [[[1.0, 2.0, 1.0], 
[6.0, 6.0, 3.0], [3.0, 6.0, 8.0]], [[4.0, 8.0, 3.0], [8.0, 8.0, 3.0], [3.0, 3.0, 3.0]], [[3.0, 3.0, 1.0], [8.0, 6.0, 3.0], 
[2.0, 7.0, 7.0]]], [[[7.0, 2.0, 6.0], [2.0, 4.0, 3.0], [4.0, 2.0, 4.0]], [[8.0, 5.0, 5.0], [6.0, 6.0, 3.0], [8.0, 3.0, 5.0]], 
[[8.0, 7.0, 3.0], [1.0, 6.0, 8.0], [7.0, 2.0, 5.0]]], [[[1.0, 4.0, 8.0], [8.0, 1.0, 5.0], [1.0, 8.0, 4.0]], [[2.0, 5.0, 2.0], 
[4.0, 3.0, 6.0], [4.0, 1.0, 3.0]], [[1.0, 6.0, 7.0], [8.0, 5.0, 6.0], [4.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLUR(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 8.0], [5.0, 2.0, 4.0], [4.0, 3.0, 3.0]], [[4.0, 7.0, 1.0], [7.0, 5.0, 7.0], [8.0, 2.0, 
6.0]], [[4.0, 4.0, 3.0], [5.0, 7.0, 5.0], [5.0, 2.0, 5.0]]], [[[8.0, 1.0, 4.0], [2.0, 7.0, 3.0], [6.0, 1.0, 1.0]], [[1.0, 8.0, 
7.0], [2.0, 5.0, 6.0], [4.0, 8.0, 8.0]], [[2.0, 2.0, 3.0], [1.0, 2.0, 8.0], [7.0, 6.0, 4.0]]], [[[6.0, 2.0, 4.0], [8.0, 5.0, 
7.0], [1.0, 8.0, 7.0]], [[2.0, 5.0, 1.0], [8.0, 3.0, 4.0], [2.0, 5.0, 8.0]], [[3.0, 5.0, 7.0], [6.0, 2.0, 2.0], [2.0, 4.0, 
7.0]]], [[[7.0, 1.0, 6.0], [4.0, 7.0, 5.0], [3.0, 8.0, 6.0]], [[4.0, 8.0, 2.0], [2.0, 6.0, 5.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 
7.0], [5.0, 6.0, 8.0], [7.0, 2.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 1.0], [5.0, 5.0, 3.0], [8.0, 2.0, 5.0]], [[4.0, 6.0, 5.0], [1.0, 4.0, 
3.0], [4.0, 8.0, 7.0]], [[2.0, 6.0, 8.0], [7.0, 2.0, 2.0], [1.0, 3.0, 3.0]]], [[[1.0, 2.0, 4.0], [1.0, 8.0, 6.0], [7.0, 1.0, 
8.0]], [[5.0, 3.0, 4.0], [5.0, 1.0, 4.0], [3.0, 8.0, 3.0]], [[3.0, 3.0, 2.0], [2.0, 3.0, 7.0], [5.0, 4.0, 1.0]]], [[[8.0, 3.0, 
5.0], [4.0, 4.0, 7.0], [6.0, 5.0, 8.0]], [[3.0, 2.0, 6.0], [3.0, 4.0, 5.0], [7.0, 6.0, 6.0]], [[8.0, 4.0, 4.0], [2.0, 8.0, 
4.0], [3.0, 1.0, 8.0]]], [[[8.0, 4.0, 4.0], [8.0, 4.0, 3.0], [3.0, 4.0, 4.0]], [[1.0, 8.0, 2.0], [2.0, 3.0, 3.0], [2.0, 2.0, 
8.0]], [[4.0, 7.0, 8.0], [3.0, 1.0, 6.0], [6.0, 5.0, 6.0]]]]))
      val=numpy.array([[[4.0], [4.0]]])
      arg[2,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 8.0], [5.0, 2.0, 4.0], [4.0, 3.0, 3.0]], [[4.0, 7.0, 1.0], [7.0, 5.0, 7.0], [8.0, 
2.0, 6.0]], [[4.0, 4.0, 3.0], [5.0, 7.0, 5.0], [5.0, 2.0, 5.0]]], [[[8.0, 1.0, 4.0], [2.0, 7.0, 3.0], [6.0, 1.0, 1.0]], [[1.0, 
8.0, 7.0], [2.0, 5.0, 6.0], [4.0, 8.0, 8.0]], [[2.0, 2.0, 3.0], [1.0, 2.0, 8.0], [7.0, 6.0, 4.0]]], [[[6.0, 2.0, 4.0], [8.0, 
5.0, 7.0], [1.0, 8.0, 7.0]], [[2.0, 5.0, 1.0], [8.0, 3.0, 4.0], [2.0, 5.0, 8.0]], [[3.0, 4.0, 7.0], [6.0, 4.0, 2.0], [2.0, 4.0, 
7.0]]], [[[7.0, 1.0, 6.0], [4.0, 7.0, 5.0], [3.0, 8.0, 6.0]], [[4.0, 8.0, 2.0], [2.0, 6.0, 5.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 
7.0], [5.0, 6.0, 8.0], [7.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 1.0], [5.0, 5.0, 3.0], [8.0, 2.0, 5.0]], 
[[4.0, 6.0, 5.0], [1.0, 4.0, 3.0], [4.0, 8.0, 7.0]], [[2.0, 6.0, 8.0], [7.0, 2.0, 2.0], [1.0, 3.0, 3.0]]], [[[1.0, 2.0, 4.0], 
[1.0, 8.0, 6.0], [7.0, 1.0, 8.0]], [[5.0, 3.0, 4.0], [5.0, 1.0, 4.0], [3.0, 8.0, 3.0]], [[3.0, 3.0, 2.0], [2.0, 3.0, 7.0], 
[5.0, 4.0, 1.0]]], [[[8.0, 3.0, 5.0], [4.0, 4.0, 7.0], [6.0, 5.0, 8.0]], [[3.0, 2.0, 6.0], [3.0, 4.0, 5.0], [7.0, 6.0, 6.0]], 
[[8.0, 4.0, 4.0], [2.0, 4.0, 4.0], [3.0, 1.0, 8.0]]], [[[8.0, 4.0, 4.0], [8.0, 4.0, 3.0], [3.0, 4.0, 4.0]], [[1.0, 8.0, 2.0], 
[2.0, 3.0, 3.0], [2.0, 2.0, 8.0]], [[4.0, 7.0, 8.0], [3.0, 1.0, 6.0], [6.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLUR(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 2.0], [8.0, 3.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 3.0, 7.0], [2.0, 3.0, 6.0], [1.0, 7.0, 
6.0]], [[1.0, 3.0, 6.0], [7.0, 8.0, 8.0], [5.0, 8.0, 1.0]]], [[[3.0, 7.0, 8.0], [5.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[4.0, 6.0, 
3.0], [5.0, 7.0, 4.0], [5.0, 4.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 8.0, 3.0], [5.0, 3.0, 2.0]]], [[[3.0, 1.0, 1.0], [1.0, 7.0, 
7.0], [2.0, 4.0, 8.0]], [[3.0, 6.0, 1.0], [3.0, 7.0, 3.0], [1.0, 5.0, 5.0]], [[2.0, 8.0, 4.0], [4.0, 4.0, 7.0], [2.0, 3.0, 
3.0]]], [[[3.0, 6.0, 3.0], [8.0, 7.0, 3.0], [6.0, 5.0, 5.0]], [[3.0, 7.0, 3.0], [5.0, 5.0, 7.0], [6.0, 8.0, 8.0]], [[7.0, 5.0, 
7.0], [8.0, 3.0, 6.0], [3.0, 5.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 2.0], [5.0, 8.0, 3.0], [3.0, 5.0, 7.0]], [[7.0, 3.0, 4.0], [6.0, 4.0, 
1.0], [1.0, 3.0, 2.0]], [[4.0, 3.0, 3.0], [5.0, 4.0, 8.0], [2.0, 4.0, 6.0]]], [[[4.0, 6.0, 4.0], [7.0, 2.0, 2.0], [6.0, 8.0, 
4.0]], [[1.0, 4.0, 1.0], [5.0, 4.0, 8.0], [4.0, 4.0, 3.0]], [[4.0, 3.0, 1.0], [2.0, 1.0, 7.0], [8.0, 1.0, 8.0]]], [[[1.0, 2.0, 
3.0], [6.0, 3.0, 7.0], [2.0, 8.0, 7.0]], [[3.0, 4.0, 4.0], [5.0, 1.0, 6.0], [8.0, 3.0, 8.0]], [[4.0, 1.0, 1.0], [2.0, 7.0, 
5.0], [6.0, 2.0, 7.0]]], [[[6.0, 6.0, 5.0], [7.0, 5.0, 2.0], [4.0, 5.0, 3.0]], [[8.0, 5.0, 2.0], [6.0, 4.0, 3.0], [8.0, 3.0, 
2.0]], [[8.0, 3.0, 2.0], [3.0, 2.0, 2.0], [5.0, 6.0, 8.0]]]]))
      val=Data(numpy.array([[[5.0], [4.0]]]),self.functionspace)
      arg[2,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 2.0], [8.0, 3.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 3.0, 7.0], [2.0, 3.0, 6.0], [1.0, 
7.0, 6.0]], [[1.0, 3.0, 6.0], [7.0, 8.0, 8.0], [5.0, 8.0, 1.0]]], [[[3.0, 7.0, 8.0], [5.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[4.0, 
6.0, 3.0], [5.0, 7.0, 4.0], [5.0, 4.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 8.0, 3.0], [5.0, 3.0, 2.0]]], [[[3.0, 1.0, 1.0], [1.0, 
7.0, 7.0], [2.0, 4.0, 8.0]], [[3.0, 6.0, 1.0], [3.0, 7.0, 3.0], [1.0, 5.0, 5.0]], [[2.0, 5.0, 4.0], [4.0, 4.0, 7.0], [2.0, 3.0, 
3.0]]], [[[3.0, 6.0, 3.0], [8.0, 7.0, 3.0], [6.0, 5.0, 5.0]], [[3.0, 7.0, 3.0], [5.0, 5.0, 7.0], [6.0, 8.0, 8.0]], [[7.0, 5.0, 
7.0], [8.0, 3.0, 6.0], [3.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 2.0], [5.0, 8.0, 3.0], [3.0, 5.0, 7.0]], 
[[7.0, 3.0, 4.0], [6.0, 4.0, 1.0], [1.0, 3.0, 2.0]], [[4.0, 3.0, 3.0], [5.0, 4.0, 8.0], [2.0, 4.0, 6.0]]], [[[4.0, 6.0, 4.0], 
[7.0, 2.0, 2.0], [6.0, 8.0, 4.0]], [[1.0, 4.0, 1.0], [5.0, 4.0, 8.0], [4.0, 4.0, 3.0]], [[4.0, 3.0, 1.0], [2.0, 1.0, 7.0], 
[8.0, 1.0, 8.0]]], [[[1.0, 2.0, 3.0], [6.0, 3.0, 7.0], [2.0, 8.0, 7.0]], [[3.0, 4.0, 4.0], [5.0, 1.0, 6.0], [8.0, 3.0, 8.0]], 
[[4.0, 5.0, 1.0], [2.0, 4.0, 5.0], [6.0, 2.0, 7.0]]], [[[6.0, 6.0, 5.0], [7.0, 5.0, 2.0], [4.0, 5.0, 3.0]], [[8.0, 5.0, 2.0], 
[6.0, 4.0, 3.0], [8.0, 3.0, 2.0]], [[8.0, 3.0, 2.0], [3.0, 2.0, 2.0], [5.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLUR(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 4.0], [6.0, 7.0, 3.0], [5.0, 8.0, 2.0]], [[1.0, 5.0, 6.0], [6.0, 1.0, 8.0], [8.0, 4.0, 
1.0]], [[3.0, 4.0, 8.0], [7.0, 3.0, 8.0], [8.0, 2.0, 3.0]]], [[[5.0, 8.0, 4.0], [3.0, 6.0, 2.0], [6.0, 6.0, 8.0]], [[7.0, 3.0, 
3.0], [3.0, 4.0, 4.0], [6.0, 4.0, 1.0]], [[5.0, 4.0, 2.0], [3.0, 7.0, 8.0], [7.0, 7.0, 7.0]]], [[[2.0, 8.0, 3.0], [1.0, 3.0, 
1.0], [7.0, 8.0, 8.0]], [[7.0, 6.0, 4.0], [2.0, 5.0, 7.0], [1.0, 2.0, 3.0]], [[5.0, 7.0, 3.0], [8.0, 3.0, 2.0], [2.0, 3.0, 
2.0]]], [[[8.0, 3.0, 5.0], [5.0, 3.0, 4.0], [8.0, 4.0, 6.0]], [[2.0, 1.0, 1.0], [3.0, 3.0, 7.0], [8.0, 4.0, 7.0]], [[7.0, 7.0, 
5.0], [8.0, 6.0, 2.0], [2.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 2.0], [7.0, 6.0, 8.0], [2.0, 6.0, 7.0]], [[5.0, 8.0, 2.0], [3.0, 5.0, 
5.0], [3.0, 2.0, 8.0]], [[2.0, 7.0, 7.0], [8.0, 2.0, 7.0], [3.0, 7.0, 4.0]]], [[[7.0, 6.0, 3.0], [7.0, 8.0, 2.0], [2.0, 5.0, 
4.0]], [[6.0, 1.0, 3.0], [5.0, 8.0, 4.0], [7.0, 7.0, 3.0]], [[1.0, 7.0, 7.0], [5.0, 2.0, 5.0], [3.0, 3.0, 3.0]]], [[[5.0, 5.0, 
4.0], [5.0, 2.0, 4.0], [6.0, 8.0, 7.0]], [[2.0, 2.0, 4.0], [5.0, 5.0, 8.0], [2.0, 3.0, 6.0]], [[1.0, 1.0, 1.0], [8.0, 2.0, 
8.0], [5.0, 8.0, 4.0]]], [[[7.0, 4.0, 3.0], [7.0, 5.0, 5.0], [3.0, 3.0, 7.0]], [[8.0, 7.0, 4.0], [1.0, 4.0, 2.0], [6.0, 5.0, 
2.0]], [[1.0, 2.0, 7.0], [1.0, 8.0, 6.0], [6.0, 7.0, 5.0]]]]))
      val=Data(numpy.array([[[7.0], [4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0], [3.0]]]))
      arg[2,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 4.0], [6.0, 7.0, 3.0], [5.0, 8.0, 2.0]], [[1.0, 5.0, 6.0], [6.0, 1.0, 8.0], [8.0, 
4.0, 1.0]], [[3.0, 4.0, 8.0], [7.0, 3.0, 8.0], [8.0, 2.0, 3.0]]], [[[5.0, 8.0, 4.0], [3.0, 6.0, 2.0], [6.0, 6.0, 8.0]], [[7.0, 
3.0, 3.0], [3.0, 4.0, 4.0], [6.0, 4.0, 1.0]], [[5.0, 4.0, 2.0], [3.0, 7.0, 8.0], [7.0, 7.0, 7.0]]], [[[2.0, 8.0, 3.0], [1.0, 
3.0, 1.0], [7.0, 8.0, 8.0]], [[7.0, 6.0, 4.0], [2.0, 5.0, 7.0], [1.0, 2.0, 3.0]], [[5.0, 7.0, 3.0], [8.0, 4.0, 2.0], [2.0, 3.0, 
2.0]]], [[[8.0, 3.0, 5.0], [5.0, 3.0, 4.0], [8.0, 4.0, 6.0]], [[2.0, 1.0, 1.0], [3.0, 3.0, 7.0], [8.0, 4.0, 7.0]], [[7.0, 7.0, 
5.0], [8.0, 6.0, 2.0], [2.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 2.0], [7.0, 6.0, 8.0], [2.0, 6.0, 7.0]], 
[[5.0, 8.0, 2.0], [3.0, 5.0, 5.0], [3.0, 2.0, 8.0]], [[2.0, 7.0, 7.0], [8.0, 2.0, 7.0], [3.0, 7.0, 4.0]]], [[[7.0, 6.0, 3.0], 
[7.0, 8.0, 2.0], [2.0, 5.0, 4.0]], [[6.0, 1.0, 3.0], [5.0, 8.0, 4.0], [7.0, 7.0, 3.0]], [[1.0, 7.0, 7.0], [5.0, 2.0, 5.0], 
[3.0, 3.0, 3.0]]], [[[5.0, 5.0, 4.0], [5.0, 2.0, 4.0], [6.0, 8.0, 7.0]], [[2.0, 2.0, 4.0], [5.0, 5.0, 8.0], [2.0, 3.0, 6.0]], 
[[1.0, 6.0, 1.0], [8.0, 3.0, 8.0], [5.0, 8.0, 4.0]]], [[[7.0, 4.0, 3.0], [7.0, 5.0, 5.0], [3.0, 3.0, 7.0]], [[8.0, 7.0, 4.0], 
[1.0, 4.0, 2.0], [6.0, 5.0, 2.0]], [[1.0, 2.0, 7.0], [1.0, 8.0, 6.0], [6.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLUR(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 5.0], [6.0, 1.0, 7.0], [3.0, 7.0, 5.0]], [[8.0, 5.0, 6.0], [5.0, 7.0, 8.0], [4.0, 1.0, 
7.0]], [[3.0, 6.0, 4.0], [5.0, 1.0, 2.0], [7.0, 1.0, 5.0]]], [[[1.0, 6.0, 5.0], [2.0, 4.0, 2.0], [4.0, 6.0, 5.0]], [[1.0, 2.0, 
5.0], [3.0, 6.0, 3.0], [2.0, 2.0, 8.0]], [[7.0, 7.0, 1.0], [6.0, 1.0, 1.0], [8.0, 6.0, 3.0]]], [[[8.0, 7.0, 8.0], [8.0, 1.0, 
6.0], [7.0, 6.0, 5.0]], [[8.0, 7.0, 1.0], [6.0, 2.0, 5.0], [8.0, 3.0, 2.0]], [[8.0, 2.0, 2.0], [6.0, 2.0, 5.0], [4.0, 4.0, 
5.0]]], [[[4.0, 8.0, 6.0], [8.0, 5.0, 8.0], [8.0, 3.0, 5.0]], [[3.0, 3.0, 5.0], [4.0, 4.0, 1.0], [7.0, 6.0, 3.0]], [[7.0, 3.0, 
6.0], [8.0, 3.0, 8.0], [5.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 8.0], [3.0, 8.0, 6.0], [5.0, 3.0, 4.0]], [[7.0, 3.0, 8.0], [6.0, 6.0, 
4.0], [5.0, 7.0, 8.0]], [[7.0, 2.0, 4.0], [7.0, 8.0, 2.0], [4.0, 1.0, 2.0]]], [[[4.0, 1.0, 6.0], [2.0, 5.0, 7.0], [6.0, 6.0, 
7.0]], [[2.0, 4.0, 3.0], [5.0, 8.0, 1.0], [8.0, 2.0, 8.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 8.0], [6.0, 2.0, 5.0]]], [[[4.0, 4.0, 
4.0], [5.0, 3.0, 7.0], [8.0, 2.0, 4.0]], [[7.0, 4.0, 1.0], [6.0, 6.0, 2.0], [2.0, 1.0, 1.0]], [[2.0, 7.0, 4.0], [4.0, 4.0, 
2.0], [6.0, 2.0, 5.0]]], [[[2.0, 7.0, 5.0], [7.0, 3.0, 6.0], [4.0, 7.0, 5.0]], [[5.0, 7.0, 4.0], [5.0, 4.0, 5.0], [8.0, 7.0, 
8.0]], [[7.0, 2.0, 6.0], [4.0, 1.0, 8.0], [8.0, 6.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0], [5.0]]])+(1.-msk_val)*numpy.array([[[2.0], [2.0]]])
      arg[2,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 5.0], [6.0, 1.0, 7.0], [3.0, 7.0, 5.0]], [[8.0, 5.0, 6.0], [5.0, 7.0, 8.0], [4.0, 
1.0, 7.0]], [[3.0, 6.0, 4.0], [5.0, 1.0, 2.0], [7.0, 1.0, 5.0]]], [[[1.0, 6.0, 5.0], [2.0, 4.0, 2.0], [4.0, 6.0, 5.0]], [[1.0, 
2.0, 5.0], [3.0, 6.0, 3.0], [2.0, 2.0, 8.0]], [[7.0, 7.0, 1.0], [6.0, 1.0, 1.0], [8.0, 6.0, 3.0]]], [[[8.0, 7.0, 8.0], [8.0, 
1.0, 6.0], [7.0, 6.0, 5.0]], [[8.0, 7.0, 1.0], [6.0, 2.0, 5.0], [8.0, 3.0, 2.0]], [[8.0, 1.0, 2.0], [6.0, 5.0, 5.0], [4.0, 4.0, 
5.0]]], [[[4.0, 8.0, 6.0], [8.0, 5.0, 8.0], [8.0, 3.0, 5.0]], [[3.0, 3.0, 5.0], [4.0, 4.0, 1.0], [7.0, 6.0, 3.0]], [[7.0, 3.0, 
6.0], [8.0, 3.0, 8.0], [5.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 8.0], [3.0, 8.0, 6.0], [5.0, 3.0, 4.0]], 
[[7.0, 3.0, 8.0], [6.0, 6.0, 4.0], [5.0, 7.0, 8.0]], [[7.0, 2.0, 4.0], [7.0, 8.0, 2.0], [4.0, 1.0, 2.0]]], [[[4.0, 1.0, 6.0], 
[2.0, 5.0, 7.0], [6.0, 6.0, 7.0]], [[2.0, 4.0, 3.0], [5.0, 8.0, 1.0], [8.0, 2.0, 8.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 8.0], 
[6.0, 2.0, 5.0]]], [[[4.0, 4.0, 4.0], [5.0, 3.0, 7.0], [8.0, 2.0, 4.0]], [[7.0, 4.0, 1.0], [6.0, 6.0, 2.0], [2.0, 1.0, 1.0]], 
[[2.0, 2.0, 4.0], [4.0, 2.0, 2.0], [6.0, 2.0, 5.0]]], [[[2.0, 7.0, 5.0], [7.0, 3.0, 6.0], [4.0, 7.0, 5.0]], [[5.0, 7.0, 4.0], 
[5.0, 4.0, 5.0], [8.0, 7.0, 8.0]], [[7.0, 2.0, 6.0], [4.0, 1.0, 8.0], [8.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLLP(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 2.0], [7.0, 3.0, 2.0], [7.0, 8.0, 1.0]], [[1.0, 2.0, 8.0], [8.0, 4.0, 1.0], [2.0, 4.0, 
5.0]], [[4.0, 6.0, 2.0], [1.0, 1.0, 2.0], [2.0, 6.0, 1.0]]], [[[4.0, 2.0, 1.0], [5.0, 8.0, 5.0], [5.0, 3.0, 3.0]], [[2.0, 3.0, 
8.0], [4.0, 5.0, 4.0], [3.0, 3.0, 6.0]], [[6.0, 4.0, 6.0], [8.0, 7.0, 8.0], [8.0, 3.0, 3.0]]], [[[8.0, 6.0, 7.0], [8.0, 4.0, 
1.0], [3.0, 1.0, 7.0]], [[2.0, 6.0, 4.0], [4.0, 1.0, 4.0], [3.0, 3.0, 4.0]], [[1.0, 1.0, 4.0], [5.0, 4.0, 4.0], [6.0, 7.0, 
2.0]]], [[[6.0, 6.0, 5.0], [6.0, 4.0, 4.0], [4.0, 1.0, 4.0]], [[1.0, 3.0, 4.0], [6.0, 8.0, 1.0], [2.0, 5.0, 3.0]], [[5.0, 7.0, 
4.0], [2.0, 7.0, 3.0], [6.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 3.0], [6.0, 4.0, 4.0], [5.0, 3.0, 6.0]], [[1.0, 1.0, 4.0], [7.0, 3.0, 
2.0], [3.0, 6.0, 6.0]], [[8.0, 5.0, 5.0], [7.0, 7.0, 1.0], [2.0, 3.0, 1.0]]], [[[6.0, 3.0, 7.0], [4.0, 5.0, 7.0], [4.0, 6.0, 
5.0]], [[6.0, 8.0, 8.0], [7.0, 6.0, 1.0], [7.0, 2.0, 5.0]], [[6.0, 1.0, 5.0], [8.0, 2.0, 7.0], [5.0, 7.0, 6.0]]], [[[8.0, 7.0, 
5.0], [8.0, 8.0, 3.0], [3.0, 4.0, 4.0]], [[8.0, 3.0, 1.0], [2.0, 6.0, 7.0], [6.0, 4.0, 8.0]], [[3.0, 4.0, 4.0], [1.0, 2.0, 
1.0], [6.0, 5.0, 6.0]]], [[[4.0, 1.0, 3.0], [7.0, 3.0, 1.0], [7.0, 5.0, 5.0]], [[1.0, 2.0, 7.0], [2.0, 3.0, 8.0], [4.0, 6.0, 
1.0]], [[6.0, 3.0, 2.0], [8.0, 1.0, 1.0], [6.0, 6.0, 1.0]]]]))
      val=numpy.array([[6.0]])
      arg[2,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 2.0], [7.0, 3.0, 2.0], [7.0, 8.0, 1.0]], [[1.0, 2.0, 8.0], [8.0, 4.0, 1.0], [2.0, 
4.0, 5.0]], [[4.0, 6.0, 2.0], [1.0, 1.0, 2.0], [2.0, 6.0, 1.0]]], [[[4.0, 2.0, 1.0], [5.0, 8.0, 5.0], [5.0, 3.0, 3.0]], [[2.0, 
3.0, 8.0], [4.0, 5.0, 4.0], [3.0, 3.0, 6.0]], [[6.0, 4.0, 6.0], [8.0, 7.0, 8.0], [8.0, 3.0, 3.0]]], [[[8.0, 6.0, 7.0], [8.0, 
4.0, 1.0], [3.0, 1.0, 7.0]], [[2.0, 6.0, 4.0], [4.0, 1.0, 4.0], [3.0, 3.0, 4.0]], [[1.0, 1.0, 4.0], [5.0, 4.0, 4.0], [6.0, 6.0, 
2.0]]], [[[6.0, 6.0, 5.0], [6.0, 4.0, 4.0], [4.0, 1.0, 4.0]], [[1.0, 3.0, 4.0], [6.0, 8.0, 1.0], [2.0, 5.0, 3.0]], [[5.0, 7.0, 
4.0], [2.0, 7.0, 3.0], [6.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 3.0], [6.0, 4.0, 4.0], [5.0, 3.0, 6.0]], 
[[1.0, 1.0, 4.0], [7.0, 3.0, 2.0], [3.0, 6.0, 6.0]], [[8.0, 5.0, 5.0], [7.0, 7.0, 1.0], [2.0, 3.0, 1.0]]], [[[6.0, 3.0, 7.0], 
[4.0, 5.0, 7.0], [4.0, 6.0, 5.0]], [[6.0, 8.0, 8.0], [7.0, 6.0, 1.0], [7.0, 2.0, 5.0]], [[6.0, 1.0, 5.0], [8.0, 2.0, 7.0], 
[5.0, 7.0, 6.0]]], [[[8.0, 7.0, 5.0], [8.0, 8.0, 3.0], [3.0, 4.0, 4.0]], [[8.0, 3.0, 1.0], [2.0, 6.0, 7.0], [6.0, 4.0, 8.0]], 
[[3.0, 4.0, 4.0], [1.0, 2.0, 1.0], [6.0, 6.0, 6.0]]], [[[4.0, 1.0, 3.0], [7.0, 3.0, 1.0], [7.0, 5.0, 5.0]], [[1.0, 2.0, 7.0], 
[2.0, 3.0, 8.0], [4.0, 6.0, 1.0]], [[6.0, 3.0, 2.0], [8.0, 1.0, 1.0], [6.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLLP(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 6.0], [2.0, 2.0, 6.0], [4.0, 2.0, 4.0]], [[1.0, 7.0, 1.0], [7.0, 7.0, 4.0], [6.0, 8.0, 
8.0]], [[4.0, 6.0, 5.0], [4.0, 1.0, 6.0], [3.0, 3.0, 6.0]]], [[[1.0, 3.0, 6.0], [6.0, 7.0, 7.0], [2.0, 2.0, 2.0]], [[7.0, 6.0, 
4.0], [2.0, 4.0, 3.0], [6.0, 6.0, 6.0]], [[7.0, 4.0, 4.0], [8.0, 6.0, 8.0], [8.0, 8.0, 7.0]]], [[[1.0, 7.0, 2.0], [5.0, 7.0, 
5.0], [8.0, 2.0, 7.0]], [[2.0, 5.0, 5.0], [8.0, 3.0, 8.0], [8.0, 4.0, 6.0]], [[1.0, 8.0, 3.0], [5.0, 1.0, 8.0], [7.0, 1.0, 
2.0]]], [[[4.0, 8.0, 1.0], [5.0, 6.0, 8.0], [5.0, 3.0, 3.0]], [[6.0, 4.0, 6.0], [8.0, 4.0, 7.0], [8.0, 7.0, 5.0]], [[5.0, 4.0, 
1.0], [4.0, 7.0, 7.0], [8.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 8.0], [8.0, 1.0, 7.0], [6.0, 6.0, 5.0]], [[2.0, 4.0, 4.0], [7.0, 5.0, 
5.0], [6.0, 6.0, 1.0]], [[1.0, 5.0, 2.0], [4.0, 4.0, 3.0], [1.0, 1.0, 8.0]]], [[[5.0, 6.0, 4.0], [1.0, 6.0, 2.0], [3.0, 3.0, 
4.0]], [[6.0, 4.0, 1.0], [2.0, 8.0, 3.0], [3.0, 5.0, 4.0]], [[3.0, 3.0, 4.0], [7.0, 7.0, 7.0], [1.0, 3.0, 8.0]]], [[[8.0, 6.0, 
1.0], [3.0, 8.0, 7.0], [5.0, 8.0, 7.0]], [[8.0, 6.0, 6.0], [2.0, 3.0, 8.0], [2.0, 2.0, 2.0]], [[4.0, 3.0, 4.0], [6.0, 6.0, 
2.0], [8.0, 5.0, 3.0]]], [[[1.0, 3.0, 5.0], [1.0, 4.0, 4.0], [8.0, 1.0, 4.0]], [[6.0, 7.0, 3.0], [6.0, 1.0, 7.0], [1.0, 4.0, 
3.0]], [[6.0, 7.0, 6.0], [4.0, 3.0, 2.0], [2.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[7.0]]),self.functionspace)
      arg[2,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 6.0], [2.0, 2.0, 6.0], [4.0, 2.0, 4.0]], [[1.0, 7.0, 1.0], [7.0, 7.0, 4.0], [6.0, 
8.0, 8.0]], [[4.0, 6.0, 5.0], [4.0, 1.0, 6.0], [3.0, 3.0, 6.0]]], [[[1.0, 3.0, 6.0], [6.0, 7.0, 7.0], [2.0, 2.0, 2.0]], [[7.0, 
6.0, 4.0], [2.0, 4.0, 3.0], [6.0, 6.0, 6.0]], [[7.0, 4.0, 4.0], [8.0, 6.0, 8.0], [8.0, 8.0, 7.0]]], [[[1.0, 7.0, 2.0], [5.0, 
7.0, 5.0], [8.0, 2.0, 7.0]], [[2.0, 5.0, 5.0], [8.0, 3.0, 8.0], [8.0, 4.0, 6.0]], [[1.0, 8.0, 3.0], [5.0, 1.0, 8.0], [7.0, 7.0, 
2.0]]], [[[4.0, 8.0, 1.0], [5.0, 6.0, 8.0], [5.0, 3.0, 3.0]], [[6.0, 4.0, 6.0], [8.0, 4.0, 7.0], [8.0, 7.0, 5.0]], [[5.0, 4.0, 
1.0], [4.0, 7.0, 7.0], [8.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 8.0], [8.0, 1.0, 7.0], [6.0, 6.0, 5.0]], 
[[2.0, 4.0, 4.0], [7.0, 5.0, 5.0], [6.0, 6.0, 1.0]], [[1.0, 5.0, 2.0], [4.0, 4.0, 3.0], [1.0, 1.0, 8.0]]], [[[5.0, 6.0, 4.0], 
[1.0, 6.0, 2.0], [3.0, 3.0, 4.0]], [[6.0, 4.0, 1.0], [2.0, 8.0, 3.0], [3.0, 5.0, 4.0]], [[3.0, 3.0, 4.0], [7.0, 7.0, 7.0], 
[1.0, 3.0, 8.0]]], [[[8.0, 6.0, 1.0], [3.0, 8.0, 7.0], [5.0, 8.0, 7.0]], [[8.0, 6.0, 6.0], [2.0, 3.0, 8.0], [2.0, 2.0, 2.0]], 
[[4.0, 3.0, 4.0], [6.0, 6.0, 2.0], [8.0, 7.0, 3.0]]], [[[1.0, 3.0, 5.0], [1.0, 4.0, 4.0], [8.0, 1.0, 4.0]], [[6.0, 7.0, 3.0], 
[6.0, 1.0, 7.0], [1.0, 4.0, 3.0]], [[6.0, 7.0, 6.0], [4.0, 3.0, 2.0], [2.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLLP(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 4.0], [7.0, 4.0, 3.0], [7.0, 5.0, 4.0]], [[3.0, 8.0, 4.0], [3.0, 3.0, 2.0], [5.0, 1.0, 
6.0]], [[4.0, 5.0, 5.0], [3.0, 4.0, 6.0], [7.0, 8.0, 5.0]]], [[[5.0, 3.0, 7.0], [4.0, 3.0, 6.0], [7.0, 4.0, 4.0]], [[8.0, 7.0, 
4.0], [5.0, 2.0, 2.0], [1.0, 6.0, 2.0]], [[3.0, 1.0, 1.0], [7.0, 4.0, 1.0], [6.0, 5.0, 2.0]]], [[[4.0, 3.0, 1.0], [5.0, 3.0, 
1.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 7.0], [7.0, 1.0, 1.0], [4.0, 5.0, 5.0]], [[6.0, 4.0, 4.0], [4.0, 7.0, 8.0], [8.0, 2.0, 
2.0]]], [[[3.0, 5.0, 4.0], [4.0, 8.0, 4.0], [2.0, 5.0, 7.0]], [[7.0, 6.0, 5.0], [2.0, 5.0, 7.0], [2.0, 5.0, 6.0]], [[5.0, 2.0, 
8.0], [1.0, 8.0, 7.0], [7.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 1.0], [7.0, 7.0, 6.0], [4.0, 1.0, 2.0]], [[8.0, 7.0, 5.0], [2.0, 5.0, 
3.0], [5.0, 1.0, 4.0]], [[6.0, 4.0, 7.0], [4.0, 6.0, 4.0], [6.0, 2.0, 6.0]]], [[[6.0, 7.0, 6.0], [4.0, 3.0, 6.0], [5.0, 8.0, 
6.0]], [[7.0, 3.0, 1.0], [3.0, 4.0, 4.0], [5.0, 2.0, 4.0]], [[5.0, 6.0, 5.0], [4.0, 1.0, 6.0], [5.0, 2.0, 5.0]]], [[[8.0, 6.0, 
8.0], [3.0, 8.0, 1.0], [4.0, 1.0, 6.0]], [[6.0, 6.0, 8.0], [8.0, 6.0, 7.0], [7.0, 4.0, 2.0]], [[7.0, 1.0, 4.0], [2.0, 6.0, 
3.0], [8.0, 1.0, 8.0]]], [[[1.0, 7.0, 6.0], [5.0, 6.0, 7.0], [7.0, 4.0, 3.0]], [[3.0, 3.0, 2.0], [4.0, 8.0, 2.0], [4.0, 3.0, 
1.0]], [[1.0, 8.0, 1.0], [1.0, 1.0, 5.0], [8.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0]]))
      arg[2,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 4.0], [7.0, 4.0, 3.0], [7.0, 5.0, 4.0]], [[3.0, 8.0, 4.0], [3.0, 3.0, 2.0], [5.0, 
1.0, 6.0]], [[4.0, 5.0, 5.0], [3.0, 4.0, 6.0], [7.0, 8.0, 5.0]]], [[[5.0, 3.0, 7.0], [4.0, 3.0, 6.0], [7.0, 4.0, 4.0]], [[8.0, 
7.0, 4.0], [5.0, 2.0, 2.0], [1.0, 6.0, 2.0]], [[3.0, 1.0, 1.0], [7.0, 4.0, 1.0], [6.0, 5.0, 2.0]]], [[[4.0, 3.0, 1.0], [5.0, 
3.0, 1.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 7.0], [7.0, 1.0, 1.0], [4.0, 5.0, 5.0]], [[6.0, 4.0, 4.0], [4.0, 7.0, 8.0], [8.0, 6.0, 
2.0]]], [[[3.0, 5.0, 4.0], [4.0, 8.0, 4.0], [2.0, 5.0, 7.0]], [[7.0, 6.0, 5.0], [2.0, 5.0, 7.0], [2.0, 5.0, 6.0]], [[5.0, 2.0, 
8.0], [1.0, 8.0, 7.0], [7.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 1.0], [7.0, 7.0, 6.0], [4.0, 1.0, 2.0]], 
[[8.0, 7.0, 5.0], [2.0, 5.0, 3.0], [5.0, 1.0, 4.0]], [[6.0, 4.0, 7.0], [4.0, 6.0, 4.0], [6.0, 2.0, 6.0]]], [[[6.0, 7.0, 6.0], 
[4.0, 3.0, 6.0], [5.0, 8.0, 6.0]], [[7.0, 3.0, 1.0], [3.0, 4.0, 4.0], [5.0, 2.0, 4.0]], [[5.0, 6.0, 5.0], [4.0, 1.0, 6.0], 
[5.0, 2.0, 5.0]]], [[[8.0, 6.0, 8.0], [3.0, 8.0, 1.0], [4.0, 1.0, 6.0]], [[6.0, 6.0, 8.0], [8.0, 6.0, 7.0], [7.0, 4.0, 2.0]], 
[[7.0, 1.0, 4.0], [2.0, 6.0, 3.0], [8.0, 7.0, 8.0]]], [[[1.0, 7.0, 6.0], [5.0, 6.0, 7.0], [7.0, 4.0, 3.0]], [[3.0, 3.0, 2.0], 
[4.0, 8.0, 2.0], [4.0, 3.0, 1.0]], [[1.0, 8.0, 1.0], [1.0, 1.0, 5.0], [8.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLLP(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 2.0], [3.0, 1.0, 6.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 8.0], [8.0, 2.0, 7.0], [4.0, 6.0, 
6.0]], [[6.0, 4.0, 6.0], [3.0, 6.0, 3.0], [7.0, 5.0, 1.0]]], [[[4.0, 8.0, 1.0], [2.0, 1.0, 2.0], [8.0, 7.0, 5.0]], [[5.0, 4.0, 
6.0], [5.0, 6.0, 7.0], [4.0, 3.0, 1.0]], [[3.0, 6.0, 6.0], [5.0, 7.0, 3.0], [6.0, 4.0, 3.0]]], [[[1.0, 5.0, 3.0], [7.0, 5.0, 
4.0], [4.0, 6.0, 3.0]], [[2.0, 3.0, 7.0], [6.0, 3.0, 1.0], [3.0, 2.0, 8.0]], [[3.0, 4.0, 3.0], [2.0, 1.0, 5.0], [5.0, 7.0, 
8.0]]], [[[5.0, 8.0, 2.0], [3.0, 2.0, 8.0], [6.0, 8.0, 7.0]], [[1.0, 5.0, 8.0], [8.0, 8.0, 7.0], [3.0, 7.0, 7.0]], [[4.0, 2.0, 
3.0], [5.0, 1.0, 1.0], [8.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 1.0], [7.0, 6.0, 8.0], [8.0, 4.0, 1.0]], [[1.0, 8.0, 3.0], [2.0, 3.0, 
7.0], [5.0, 4.0, 8.0]], [[6.0, 8.0, 4.0], [3.0, 6.0, 5.0], [4.0, 1.0, 2.0]]], [[[6.0, 2.0, 2.0], [7.0, 6.0, 5.0], [6.0, 7.0, 
1.0]], [[6.0, 3.0, 1.0], [2.0, 1.0, 5.0], [2.0, 6.0, 4.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 2.0], [4.0, 4.0, 3.0]]], [[[1.0, 8.0, 
1.0], [2.0, 4.0, 6.0], [8.0, 5.0, 5.0]], [[2.0, 1.0, 7.0], [7.0, 7.0, 6.0], [7.0, 3.0, 3.0]], [[8.0, 7.0, 2.0], [4.0, 7.0, 
6.0], [7.0, 6.0, 2.0]]], [[[3.0, 5.0, 7.0], [4.0, 3.0, 8.0], [4.0, 4.0, 8.0]], [[2.0, 5.0, 3.0], [2.0, 6.0, 1.0], [1.0, 6.0, 
2.0]], [[1.0, 4.0, 8.0], [7.0, 7.0, 3.0], [2.0, 2.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[7.0]])+(1.-msk_val)*numpy.array([[6.0]])
      arg[2,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 2.0], [3.0, 1.0, 6.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 8.0], [8.0, 2.0, 7.0], [4.0, 
6.0, 6.0]], [[6.0, 4.0, 6.0], [3.0, 6.0, 3.0], [7.0, 5.0, 1.0]]], [[[4.0, 8.0, 1.0], [2.0, 1.0, 2.0], [8.0, 7.0, 5.0]], [[5.0, 
4.0, 6.0], [5.0, 6.0, 7.0], [4.0, 3.0, 1.0]], [[3.0, 6.0, 6.0], [5.0, 7.0, 3.0], [6.0, 4.0, 3.0]]], [[[1.0, 5.0, 3.0], [7.0, 
5.0, 4.0], [4.0, 6.0, 3.0]], [[2.0, 3.0, 7.0], [6.0, 3.0, 1.0], [3.0, 2.0, 8.0]], [[3.0, 4.0, 3.0], [2.0, 1.0, 5.0], [5.0, 7.0, 
8.0]]], [[[5.0, 8.0, 2.0], [3.0, 2.0, 8.0], [6.0, 8.0, 7.0]], [[1.0, 5.0, 8.0], [8.0, 8.0, 7.0], [3.0, 7.0, 7.0]], [[4.0, 2.0, 
3.0], [5.0, 1.0, 1.0], [8.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 1.0], [7.0, 6.0, 8.0], [8.0, 4.0, 1.0]], 
[[1.0, 8.0, 3.0], [2.0, 3.0, 7.0], [5.0, 4.0, 8.0]], [[6.0, 8.0, 4.0], [3.0, 6.0, 5.0], [4.0, 1.0, 2.0]]], [[[6.0, 2.0, 2.0], 
[7.0, 6.0, 5.0], [6.0, 7.0, 1.0]], [[6.0, 3.0, 1.0], [2.0, 1.0, 5.0], [2.0, 6.0, 4.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 2.0], 
[4.0, 4.0, 3.0]]], [[[1.0, 8.0, 1.0], [2.0, 4.0, 6.0], [8.0, 5.0, 5.0]], [[2.0, 1.0, 7.0], [7.0, 7.0, 6.0], [7.0, 3.0, 3.0]], 
[[8.0, 7.0, 2.0], [4.0, 7.0, 6.0], [7.0, 6.0, 2.0]]], [[[3.0, 5.0, 7.0], [4.0, 3.0, 8.0], [4.0, 4.0, 8.0]], [[2.0, 5.0, 3.0], 
[2.0, 6.0, 1.0], [1.0, 6.0, 2.0]], [[1.0, 4.0, 8.0], [7.0, 7.0, 3.0], [2.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UPCL(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 6.0], [7.0, 3.0, 5.0], [2.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [2.0, 5.0, 8.0], [8.0, 1.0, 
5.0]], [[1.0, 1.0, 7.0], [1.0, 6.0, 8.0], [2.0, 1.0, 5.0]]], [[[1.0, 8.0, 5.0], [7.0, 2.0, 3.0], [6.0, 6.0, 1.0]], [[6.0, 4.0, 
8.0], [3.0, 2.0, 3.0], [7.0, 2.0, 8.0]], [[1.0, 7.0, 5.0], [7.0, 8.0, 1.0], [7.0, 1.0, 4.0]]], [[[3.0, 8.0, 2.0], [2.0, 3.0, 
4.0], [7.0, 4.0, 6.0]], [[1.0, 1.0, 6.0], [7.0, 3.0, 3.0], [1.0, 6.0, 7.0]], [[8.0, 6.0, 8.0], [7.0, 2.0, 8.0], [6.0, 2.0, 
5.0]]], [[[7.0, 7.0, 5.0], [4.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[1.0, 8.0, 3.0], [6.0, 3.0, 7.0], [2.0, 5.0, 1.0]], [[7.0, 5.0, 
1.0], [5.0, 4.0, 2.0], [3.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 5.0], [8.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[5.0, 8.0, 3.0], [7.0, 8.0, 
8.0], [3.0, 2.0, 4.0]], [[3.0, 6.0, 4.0], [8.0, 1.0, 3.0], [3.0, 5.0, 1.0]]], [[[2.0, 7.0, 2.0], [8.0, 8.0, 3.0], [8.0, 2.0, 
5.0]], [[6.0, 8.0, 7.0], [5.0, 6.0, 1.0], [3.0, 4.0, 6.0]], [[7.0, 8.0, 2.0], [7.0, 5.0, 4.0], [1.0, 4.0, 4.0]]], [[[3.0, 1.0, 
7.0], [4.0, 8.0, 5.0], [5.0, 8.0, 4.0]], [[1.0, 7.0, 4.0], [1.0, 7.0, 2.0], [3.0, 8.0, 7.0]], [[7.0, 3.0, 5.0], [8.0, 1.0, 
8.0], [4.0, 7.0, 6.0]]], [[[7.0, 5.0, 7.0], [2.0, 7.0, 8.0], [4.0, 8.0, 7.0]], [[5.0, 3.0, 7.0], [4.0, 6.0, 5.0], [7.0, 6.0, 
3.0]], [[8.0, 8.0, 7.0], [1.0, 3.0, 4.0], [3.0, 5.0, 1.0]]]]))
      val=numpy.array([[[6.0], [5.0], [6.0]], [[6.0], [4.0], [4.0]], [[4.0], [5.0], [5.0]]])
      arg[:3,1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 6.0], [7.0, 3.0, 5.0], [2.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [2.0, 5.0, 5.0], [8.0, 
1.0, 6.0]], [[1.0, 1.0, 7.0], [1.0, 6.0, 8.0], [2.0, 1.0, 5.0]]], [[[1.0, 8.0, 5.0], [7.0, 2.0, 3.0], [6.0, 6.0, 1.0]], [[6.0, 
4.0, 6.0], [3.0, 2.0, 4.0], [7.0, 2.0, 4.0]], [[1.0, 7.0, 5.0], [7.0, 8.0, 1.0], [7.0, 1.0, 4.0]]], [[[3.0, 8.0, 2.0], [2.0, 
3.0, 4.0], [7.0, 4.0, 6.0]], [[1.0, 1.0, 4.0], [7.0, 3.0, 5.0], [1.0, 6.0, 5.0]], [[8.0, 6.0, 8.0], [7.0, 2.0, 8.0], [6.0, 2.0, 
5.0]]], [[[7.0, 7.0, 5.0], [4.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[1.0, 8.0, 3.0], [6.0, 3.0, 7.0], [2.0, 5.0, 1.0]], [[7.0, 5.0, 
1.0], [5.0, 4.0, 2.0], [3.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 5.0], [8.0, 2.0, 3.0], [8.0, 2.0, 7.0]], 
[[5.0, 8.0, 6.0], [7.0, 8.0, 5.0], [3.0, 2.0, 6.0]], [[3.0, 6.0, 4.0], [8.0, 1.0, 3.0], [3.0, 5.0, 1.0]]], [[[2.0, 7.0, 2.0], 
[8.0, 8.0, 3.0], [8.0, 2.0, 5.0]], [[6.0, 8.0, 6.0], [5.0, 6.0, 4.0], [3.0, 4.0, 4.0]], [[7.0, 8.0, 2.0], [7.0, 5.0, 4.0], 
[1.0, 4.0, 4.0]]], [[[3.0, 1.0, 7.0], [4.0, 8.0, 5.0], [5.0, 8.0, 4.0]], [[1.0, 7.0, 4.0], [1.0, 7.0, 5.0], [3.0, 8.0, 5.0]], 
[[7.0, 3.0, 5.0], [8.0, 1.0, 8.0], [4.0, 7.0, 6.0]]], [[[7.0, 5.0, 7.0], [2.0, 7.0, 8.0], [4.0, 8.0, 7.0]], [[5.0, 3.0, 7.0], 
[4.0, 6.0, 5.0], [7.0, 6.0, 3.0]], [[8.0, 8.0, 7.0], [1.0, 3.0, 4.0], [3.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UPCL(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 1.0], [3.0, 4.0, 6.0], [2.0, 1.0, 1.0]], [[1.0, 1.0, 5.0], [3.0, 4.0, 1.0], [1.0, 6.0, 
3.0]], [[3.0, 2.0, 3.0], [8.0, 6.0, 8.0], [4.0, 8.0, 2.0]]], [[[1.0, 7.0, 1.0], [1.0, 2.0, 2.0], [8.0, 7.0, 3.0]], [[4.0, 1.0, 
1.0], [2.0, 3.0, 5.0], [4.0, 7.0, 6.0]], [[2.0, 5.0, 4.0], [5.0, 2.0, 1.0], [5.0, 7.0, 6.0]]], [[[1.0, 1.0, 3.0], [4.0, 5.0, 
8.0], [3.0, 5.0, 4.0]], [[3.0, 4.0, 7.0], [1.0, 8.0, 3.0], [4.0, 2.0, 2.0]], [[1.0, 8.0, 5.0], [5.0, 5.0, 5.0], [3.0, 2.0, 
3.0]]], [[[2.0, 2.0, 6.0], [5.0, 4.0, 6.0], [3.0, 6.0, 5.0]], [[8.0, 6.0, 5.0], [7.0, 8.0, 5.0], [3.0, 2.0, 4.0]], [[8.0, 1.0, 
3.0], [7.0, 1.0, 4.0], [2.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 2.0], [4.0, 6.0, 7.0], [4.0, 2.0, 4.0]], [[3.0, 2.0, 1.0], [8.0, 1.0, 
2.0], [8.0, 8.0, 8.0]], [[8.0, 2.0, 4.0], [7.0, 3.0, 4.0], [8.0, 5.0, 7.0]]], [[[4.0, 5.0, 1.0], [4.0, 4.0, 8.0], [3.0, 3.0, 
4.0]], [[1.0, 2.0, 3.0], [6.0, 7.0, 2.0], [4.0, 1.0, 6.0]], [[3.0, 7.0, 4.0], [1.0, 2.0, 2.0], [5.0, 5.0, 1.0]]], [[[6.0, 7.0, 
2.0], [6.0, 2.0, 6.0], [3.0, 3.0, 8.0]], [[4.0, 3.0, 5.0], [8.0, 4.0, 3.0], [7.0, 1.0, 3.0]], [[6.0, 5.0, 5.0], [5.0, 6.0, 
4.0], [4.0, 2.0, 1.0]]], [[[2.0, 6.0, 5.0], [3.0, 4.0, 1.0], [5.0, 3.0, 5.0]], [[2.0, 1.0, 8.0], [2.0, 4.0, 3.0], [2.0, 6.0, 
7.0]], [[8.0, 3.0, 4.0], [7.0, 1.0, 5.0], [4.0, 1.0, 6.0]]]]))
      val=Data(numpy.array([[[7.0], [4.0], [3.0]], [[7.0], [5.0], [1.0]], [[3.0], [2.0], [2.0]]]),self.functionspace)
      arg[:3,1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 1.0], [3.0, 4.0, 6.0], [2.0, 1.0, 1.0]], [[1.0, 1.0, 7.0], [3.0, 4.0, 4.0], [1.0, 
6.0, 3.0]], [[3.0, 2.0, 3.0], [8.0, 6.0, 8.0], [4.0, 8.0, 2.0]]], [[[1.0, 7.0, 1.0], [1.0, 2.0, 2.0], [8.0, 7.0, 3.0]], [[4.0, 
1.0, 7.0], [2.0, 3.0, 5.0], [4.0, 7.0, 1.0]], [[2.0, 5.0, 4.0], [5.0, 2.0, 1.0], [5.0, 7.0, 6.0]]], [[[1.0, 1.0, 3.0], [4.0, 
5.0, 8.0], [3.0, 5.0, 4.0]], [[3.0, 4.0, 3.0], [1.0, 8.0, 2.0], [4.0, 2.0, 2.0]], [[1.0, 8.0, 5.0], [5.0, 5.0, 5.0], [3.0, 2.0, 
3.0]]], [[[2.0, 2.0, 6.0], [5.0, 4.0, 6.0], [3.0, 6.0, 5.0]], [[8.0, 6.0, 5.0], [7.0, 8.0, 5.0], [3.0, 2.0, 4.0]], [[8.0, 1.0, 
3.0], [7.0, 1.0, 4.0], [2.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 2.0], [4.0, 6.0, 7.0], [4.0, 2.0, 4.0]], 
[[3.0, 2.0, 7.0], [8.0, 1.0, 4.0], [8.0, 8.0, 3.0]], [[8.0, 2.0, 4.0], [7.0, 3.0, 4.0], [8.0, 5.0, 7.0]]], [[[4.0, 5.0, 1.0], 
[4.0, 4.0, 8.0], [3.0, 3.0, 4.0]], [[1.0, 2.0, 7.0], [6.0, 7.0, 5.0], [4.0, 1.0, 1.0]], [[3.0, 7.0, 4.0], [1.0, 2.0, 2.0], 
[5.0, 5.0, 1.0]]], [[[6.0, 7.0, 2.0], [6.0, 2.0, 6.0], [3.0, 3.0, 8.0]], [[4.0, 3.0, 3.0], [8.0, 4.0, 2.0], [7.0, 1.0, 2.0]], 
[[6.0, 5.0, 5.0], [5.0, 6.0, 4.0], [4.0, 2.0, 1.0]]], [[[2.0, 6.0, 5.0], [3.0, 4.0, 1.0], [5.0, 3.0, 5.0]], [[2.0, 1.0, 8.0], 
[2.0, 4.0, 3.0], [2.0, 6.0, 7.0]], [[8.0, 3.0, 4.0], [7.0, 1.0, 5.0], [4.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UPCL(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 6.0], [3.0, 8.0, 6.0], [7.0, 4.0, 5.0]], [[8.0, 4.0, 6.0], [7.0, 6.0, 1.0], [7.0, 3.0, 
1.0]], [[4.0, 8.0, 2.0], [7.0, 2.0, 1.0], [8.0, 4.0, 3.0]]], [[[1.0, 1.0, 1.0], [7.0, 7.0, 7.0], [3.0, 8.0, 8.0]], [[7.0, 4.0, 
4.0], [6.0, 2.0, 1.0], [5.0, 3.0, 4.0]], [[6.0, 5.0, 4.0], [8.0, 7.0, 4.0], [5.0, 7.0, 3.0]]], [[[6.0, 6.0, 7.0], [8.0, 8.0, 
3.0], [7.0, 8.0, 2.0]], [[4.0, 7.0, 7.0], [2.0, 8.0, 6.0], [4.0, 8.0, 3.0]], [[5.0, 2.0, 8.0], [4.0, 1.0, 8.0], [2.0, 2.0, 
6.0]]], [[[3.0, 5.0, 5.0], [8.0, 8.0, 1.0], [4.0, 5.0, 6.0]], [[8.0, 2.0, 4.0], [7.0, 2.0, 5.0], [2.0, 4.0, 2.0]], [[2.0, 6.0, 
6.0], [4.0, 7.0, 2.0], [8.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 3.0], [2.0, 1.0, 6.0], [3.0, 4.0, 1.0]], [[6.0, 1.0, 7.0], [8.0, 3.0, 
1.0], [3.0, 8.0, 6.0]], [[3.0, 4.0, 1.0], [7.0, 2.0, 1.0], [6.0, 1.0, 8.0]]], [[[8.0, 4.0, 8.0], [2.0, 7.0, 1.0], [8.0, 4.0, 
4.0]], [[6.0, 3.0, 8.0], [1.0, 7.0, 7.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 7.0], [2.0, 6.0, 3.0], [8.0, 6.0, 8.0]]], [[[7.0, 2.0, 
5.0], [1.0, 7.0, 1.0], [7.0, 7.0, 4.0]], [[2.0, 8.0, 1.0], [5.0, 7.0, 6.0], [5.0, 1.0, 3.0]], [[6.0, 8.0, 8.0], [6.0, 4.0, 
6.0], [5.0, 4.0, 3.0]]], [[[4.0, 4.0, 6.0], [4.0, 3.0, 8.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 7.0], [1.0, 2.0, 5.0], [5.0, 4.0, 
2.0]], [[5.0, 2.0, 5.0], [3.0, 8.0, 3.0], [5.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[[7.0], [6.0], [7.0]], [[3.0], [6.0], [7.0]], [[2.0], [4.0], [3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0], [4.0], [7.0]], [[4.0], [8.0], [7.0]], [[2.0], [5.0], [5.0]]]))
      arg[:3,1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 6.0], [3.0, 8.0, 6.0], [7.0, 4.0, 5.0]], [[8.0, 4.0, 7.0], [7.0, 6.0, 6.0], [7.0, 
3.0, 7.0]], [[4.0, 8.0, 2.0], [7.0, 2.0, 1.0], [8.0, 4.0, 3.0]]], [[[1.0, 1.0, 1.0], [7.0, 7.0, 7.0], [3.0, 8.0, 8.0]], [[7.0, 
4.0, 3.0], [6.0, 2.0, 6.0], [5.0, 3.0, 7.0]], [[6.0, 5.0, 4.0], [8.0, 7.0, 4.0], [5.0, 7.0, 3.0]]], [[[6.0, 6.0, 7.0], [8.0, 
8.0, 3.0], [7.0, 8.0, 2.0]], [[4.0, 7.0, 2.0], [2.0, 8.0, 4.0], [4.0, 8.0, 3.0]], [[5.0, 2.0, 8.0], [4.0, 1.0, 8.0], [2.0, 2.0, 
6.0]]], [[[3.0, 5.0, 5.0], [8.0, 8.0, 1.0], [4.0, 5.0, 6.0]], [[8.0, 2.0, 4.0], [7.0, 2.0, 5.0], [2.0, 4.0, 2.0]], [[2.0, 6.0, 
6.0], [4.0, 7.0, 2.0], [8.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 3.0], [2.0, 1.0, 6.0], [3.0, 4.0, 1.0]], 
[[6.0, 1.0, 1.0], [8.0, 3.0, 4.0], [3.0, 8.0, 7.0]], [[3.0, 4.0, 1.0], [7.0, 2.0, 1.0], [6.0, 1.0, 8.0]]], [[[8.0, 4.0, 8.0], 
[2.0, 7.0, 1.0], [8.0, 4.0, 4.0]], [[6.0, 3.0, 4.0], [1.0, 7.0, 8.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 7.0], [2.0, 6.0, 3.0], 
[8.0, 6.0, 8.0]]], [[[7.0, 2.0, 5.0], [1.0, 7.0, 1.0], [7.0, 7.0, 4.0]], [[2.0, 8.0, 2.0], [5.0, 7.0, 5.0], [5.0, 1.0, 5.0]], 
[[6.0, 8.0, 8.0], [6.0, 4.0, 6.0], [5.0, 4.0, 3.0]]], [[[4.0, 4.0, 6.0], [4.0, 3.0, 8.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 7.0], 
[1.0, 2.0, 5.0], [5.0, 4.0, 2.0]], [[5.0, 2.0, 5.0], [3.0, 8.0, 3.0], [5.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UPCL(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 5.0], [1.0, 7.0, 6.0], [7.0, 5.0, 1.0]], [[7.0, 3.0, 2.0], [8.0, 6.0, 5.0], [4.0, 7.0, 
7.0]], [[4.0, 2.0, 6.0], [2.0, 2.0, 6.0], [1.0, 1.0, 6.0]]], [[[6.0, 8.0, 7.0], [6.0, 4.0, 3.0], [8.0, 2.0, 4.0]], [[2.0, 7.0, 
8.0], [2.0, 5.0, 3.0], [8.0, 2.0, 6.0]], [[4.0, 8.0, 3.0], [8.0, 4.0, 4.0], [4.0, 8.0, 3.0]]], [[[4.0, 6.0, 6.0], [2.0, 8.0, 
8.0], [2.0, 2.0, 1.0]], [[8.0, 8.0, 4.0], [4.0, 5.0, 6.0], [7.0, 3.0, 4.0]], [[3.0, 4.0, 1.0], [6.0, 5.0, 5.0], [6.0, 3.0, 
5.0]]], [[[8.0, 7.0, 3.0], [4.0, 5.0, 8.0], [6.0, 4.0, 4.0]], [[5.0, 2.0, 2.0], [1.0, 6.0, 4.0], [6.0, 3.0, 3.0]], [[7.0, 3.0, 
8.0], [7.0, 4.0, 4.0], [7.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 1.0], [2.0, 5.0, 5.0], [3.0, 5.0, 3.0]], [[3.0, 5.0, 7.0], [1.0, 5.0, 
5.0], [3.0, 8.0, 1.0]], [[3.0, 6.0, 4.0], [8.0, 6.0, 2.0], [7.0, 5.0, 1.0]]], [[[4.0, 6.0, 8.0], [4.0, 8.0, 8.0], [4.0, 4.0, 
1.0]], [[1.0, 7.0, 6.0], [5.0, 4.0, 1.0], [1.0, 2.0, 2.0]], [[6.0, 5.0, 1.0], [6.0, 7.0, 4.0], [5.0, 6.0, 8.0]]], [[[3.0, 6.0, 
2.0], [3.0, 7.0, 8.0], [2.0, 1.0, 2.0]], [[6.0, 2.0, 1.0], [7.0, 8.0, 2.0], [7.0, 8.0, 2.0]], [[7.0, 4.0, 6.0], [4.0, 4.0, 
4.0], [7.0, 7.0, 3.0]]], [[[1.0, 5.0, 2.0], [1.0, 6.0, 7.0], [7.0, 1.0, 5.0]], [[2.0, 6.0, 7.0], [7.0, 3.0, 4.0], [2.0, 2.0, 
5.0]], [[1.0, 5.0, 2.0], [8.0, 2.0, 7.0], [3.0, 2.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0], [5.0], [4.0]], [[3.0], [1.0], [3.0]], [[6.0], [6.0], 
[4.0]]])+(1.-msk_val)*numpy.array([[[8.0], [7.0], [7.0]], [[1.0], [1.0], [4.0]], [[6.0], [5.0], [8.0]]])
      arg[:3,1,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 5.0], [1.0, 7.0, 6.0], [7.0, 5.0, 1.0]], [[7.0, 3.0, 1.0], [8.0, 6.0, 5.0], [4.0, 
7.0, 4.0]], [[4.0, 2.0, 6.0], [2.0, 2.0, 6.0], [1.0, 1.0, 6.0]]], [[[6.0, 8.0, 7.0], [6.0, 4.0, 3.0], [8.0, 2.0, 4.0]], [[2.0, 
7.0, 3.0], [2.0, 5.0, 1.0], [8.0, 2.0, 3.0]], [[4.0, 8.0, 3.0], [8.0, 4.0, 4.0], [4.0, 8.0, 3.0]]], [[[4.0, 6.0, 6.0], [2.0, 
8.0, 8.0], [2.0, 2.0, 1.0]], [[8.0, 8.0, 6.0], [4.0, 5.0, 6.0], [7.0, 3.0, 4.0]], [[3.0, 4.0, 1.0], [6.0, 5.0, 5.0], [6.0, 3.0, 
5.0]]], [[[8.0, 7.0, 3.0], [4.0, 5.0, 8.0], [6.0, 4.0, 4.0]], [[5.0, 2.0, 2.0], [1.0, 6.0, 4.0], [6.0, 3.0, 3.0]], [[7.0, 3.0, 
8.0], [7.0, 4.0, 4.0], [7.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 1.0], [2.0, 5.0, 5.0], [3.0, 5.0, 3.0]], 
[[3.0, 5.0, 8.0], [1.0, 5.0, 7.0], [3.0, 8.0, 7.0]], [[3.0, 6.0, 4.0], [8.0, 6.0, 2.0], [7.0, 5.0, 1.0]]], [[[4.0, 6.0, 8.0], 
[4.0, 8.0, 8.0], [4.0, 4.0, 1.0]], [[1.0, 7.0, 1.0], [5.0, 4.0, 1.0], [1.0, 2.0, 4.0]], [[6.0, 5.0, 1.0], [6.0, 7.0, 4.0], 
[5.0, 6.0, 8.0]]], [[[3.0, 6.0, 2.0], [3.0, 7.0, 8.0], [2.0, 1.0, 2.0]], [[6.0, 2.0, 6.0], [7.0, 8.0, 5.0], [7.0, 8.0, 8.0]], 
[[7.0, 4.0, 6.0], [4.0, 4.0, 4.0], [7.0, 7.0, 3.0]]], [[[1.0, 5.0, 2.0], [1.0, 6.0, 7.0], [7.0, 1.0, 5.0]], [[2.0, 6.0, 7.0], 
[7.0, 3.0, 4.0], [2.0, 2.0, 5.0]], [[1.0, 5.0, 2.0], [8.0, 2.0, 7.0], [3.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRCU(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 8.0], [4.0, 1.0, 2.0], [4.0, 2.0, 2.0]], [[2.0, 1.0, 6.0], [2.0, 7.0, 6.0], [2.0, 3.0, 
7.0]], [[1.0, 3.0, 8.0], [6.0, 1.0, 4.0], [5.0, 3.0, 8.0]]], [[[7.0, 1.0, 6.0], [2.0, 8.0, 2.0], [7.0, 1.0, 5.0]], [[4.0, 4.0, 
7.0], [4.0, 3.0, 7.0], [2.0, 7.0, 5.0]], [[1.0, 7.0, 6.0], [5.0, 8.0, 2.0], [3.0, 7.0, 3.0]]], [[[5.0, 4.0, 6.0], [3.0, 2.0, 
1.0], [4.0, 2.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 4.0, 8.0], [1.0, 3.0, 2.0]], [[1.0, 7.0, 3.0], [6.0, 1.0, 1.0], [3.0, 7.0, 
4.0]]], [[[4.0, 1.0, 5.0], [2.0, 7.0, 5.0], [5.0, 7.0, 8.0]], [[3.0, 2.0, 5.0], [5.0, 2.0, 1.0], [4.0, 5.0, 5.0]], [[5.0, 2.0, 
6.0], [5.0, 4.0, 7.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 1.0], [1.0, 7.0, 4.0], [8.0, 8.0, 1.0]], [[4.0, 3.0, 2.0], [2.0, 5.0, 
1.0], [3.0, 4.0, 4.0]], [[3.0, 6.0, 6.0], [8.0, 1.0, 5.0], [4.0, 3.0, 2.0]]], [[[1.0, 7.0, 5.0], [8.0, 4.0, 8.0], [4.0, 7.0, 
8.0]], [[2.0, 5.0, 5.0], [2.0, 3.0, 6.0], [4.0, 7.0, 2.0]], [[6.0, 3.0, 2.0], [3.0, 8.0, 8.0], [3.0, 1.0, 1.0]]], [[[3.0, 5.0, 
7.0], [6.0, 1.0, 7.0], [5.0, 4.0, 5.0]], [[1.0, 5.0, 2.0], [2.0, 8.0, 8.0], [8.0, 8.0, 5.0]], [[2.0, 3.0, 5.0], [5.0, 5.0, 
8.0], [8.0, 3.0, 8.0]]], [[[7.0, 8.0, 7.0], [5.0, 2.0, 3.0], [3.0, 2.0, 8.0]], [[3.0, 3.0, 7.0], [6.0, 5.0, 4.0], [8.0, 7.0, 
3.0]], [[2.0, 3.0, 7.0], [1.0, 5.0, 6.0], [6.0, 6.0, 5.0]]]]))
      val=numpy.array([[[[8.0, 2.0], [5.0, 6.0], [7.0, 1.0]]], [[[5.0, 6.0], [7.0, 6.0], [5.0, 2.0]]], [[[7.0, 2.0], [8.0, 
2.0], [3.0, 3.0]]], [[[4.0, 4.0], [2.0, 2.0], [6.0, 7.0]]]])
      arg[:,1:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 8.0], [4.0, 1.0, 2.0], [4.0, 2.0, 2.0]], [[8.0, 2.0, 6.0], [5.0, 6.0, 6.0], [7.0, 
1.0, 7.0]], [[1.0, 3.0, 8.0], [6.0, 1.0, 4.0], [5.0, 3.0, 8.0]]], [[[7.0, 1.0, 6.0], [2.0, 8.0, 2.0], [7.0, 1.0, 5.0]], [[5.0, 
6.0, 7.0], [7.0, 6.0, 7.0], [5.0, 2.0, 5.0]], [[1.0, 7.0, 6.0], [5.0, 8.0, 2.0], [3.0, 7.0, 3.0]]], [[[5.0, 4.0, 6.0], [3.0, 
2.0, 1.0], [4.0, 2.0, 2.0]], [[7.0, 2.0, 3.0], [8.0, 2.0, 8.0], [3.0, 3.0, 2.0]], [[1.0, 7.0, 3.0], [6.0, 1.0, 1.0], [3.0, 7.0, 
4.0]]], [[[4.0, 1.0, 5.0], [2.0, 7.0, 5.0], [5.0, 7.0, 8.0]], [[4.0, 4.0, 5.0], [2.0, 2.0, 1.0], [6.0, 7.0, 5.0]], [[5.0, 2.0, 
6.0], [5.0, 4.0, 7.0], [3.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 1.0], [1.0, 7.0, 4.0], [8.0, 8.0, 1.0]], 
[[8.0, 2.0, 2.0], [5.0, 6.0, 1.0], [7.0, 1.0, 4.0]], [[3.0, 6.0, 6.0], [8.0, 1.0, 5.0], [4.0, 3.0, 2.0]]], [[[1.0, 7.0, 5.0], 
[8.0, 4.0, 8.0], [4.0, 7.0, 8.0]], [[5.0, 6.0, 5.0], [7.0, 6.0, 6.0], [5.0, 2.0, 2.0]], [[6.0, 3.0, 2.0], [3.0, 8.0, 8.0], 
[3.0, 1.0, 1.0]]], [[[3.0, 5.0, 7.0], [6.0, 1.0, 7.0], [5.0, 4.0, 5.0]], [[7.0, 2.0, 2.0], [8.0, 2.0, 8.0], [3.0, 3.0, 5.0]], 
[[2.0, 3.0, 5.0], [5.0, 5.0, 8.0], [8.0, 3.0, 8.0]]], [[[7.0, 8.0, 7.0], [5.0, 2.0, 3.0], [3.0, 2.0, 8.0]], [[4.0, 4.0, 7.0], 
[2.0, 2.0, 4.0], [6.0, 7.0, 3.0]], [[2.0, 3.0, 7.0], [1.0, 5.0, 6.0], [6.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRCU(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 5.0], [8.0, 5.0, 8.0], [3.0, 8.0, 1.0]], [[3.0, 8.0, 6.0], [5.0, 5.0, 5.0], [5.0, 3.0, 
5.0]], [[7.0, 2.0, 2.0], [3.0, 8.0, 4.0], [8.0, 8.0, 3.0]]], [[[6.0, 6.0, 2.0], [4.0, 6.0, 3.0], [3.0, 2.0, 8.0]], [[3.0, 8.0, 
1.0], [5.0, 8.0, 1.0], [6.0, 5.0, 1.0]], [[4.0, 5.0, 1.0], [2.0, 3.0, 8.0], [6.0, 2.0, 7.0]]], [[[4.0, 6.0, 4.0], [1.0, 4.0, 
8.0], [8.0, 8.0, 1.0]], [[6.0, 1.0, 7.0], [7.0, 6.0, 2.0], [3.0, 8.0, 8.0]], [[4.0, 2.0, 4.0], [7.0, 8.0, 1.0], [2.0, 2.0, 
8.0]]], [[[4.0, 7.0, 5.0], [4.0, 3.0, 3.0], [6.0, 8.0, 8.0]], [[3.0, 8.0, 5.0], [5.0, 6.0, 6.0], [3.0, 1.0, 3.0]], [[7.0, 2.0, 
1.0], [4.0, 8.0, 7.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 2.0], [8.0, 2.0, 5.0], [3.0, 1.0, 2.0]], [[1.0, 4.0, 5.0], [5.0, 4.0, 
4.0], [7.0, 1.0, 7.0]], [[7.0, 7.0, 5.0], [3.0, 6.0, 3.0], [8.0, 2.0, 4.0]]], [[[4.0, 1.0, 5.0], [8.0, 7.0, 7.0], [5.0, 7.0, 
4.0]], [[6.0, 7.0, 3.0], [5.0, 6.0, 4.0], [3.0, 7.0, 7.0]], [[7.0, 1.0, 4.0], [6.0, 5.0, 6.0], [6.0, 7.0, 6.0]]], [[[3.0, 2.0, 
5.0], [7.0, 2.0, 2.0], [3.0, 5.0, 3.0]], [[7.0, 5.0, 7.0], [4.0, 7.0, 1.0], [2.0, 6.0, 8.0]], [[6.0, 8.0, 2.0], [3.0, 2.0, 
4.0], [1.0, 4.0, 6.0]]], [[[4.0, 7.0, 1.0], [1.0, 2.0, 3.0], [8.0, 8.0, 5.0]], [[8.0, 7.0, 4.0], [4.0, 5.0, 7.0], [5.0, 2.0, 
4.0]], [[3.0, 6.0, 3.0], [2.0, 3.0, 5.0], [4.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[[6.0, 7.0], [3.0, 4.0], [8.0, 2.0]]], [[[3.0, 3.0], [6.0, 3.0], [3.0, 1.0]]], [[[6.0, 5.0], [1.0, 
1.0], [1.0, 1.0]]], [[[5.0, 2.0], [5.0, 1.0], [8.0, 2.0]]]]),self.functionspace)
      arg[:,1:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 5.0], [8.0, 5.0, 8.0], [3.0, 8.0, 1.0]], [[6.0, 7.0, 6.0], [3.0, 4.0, 5.0], [8.0, 
2.0, 5.0]], [[7.0, 2.0, 2.0], [3.0, 8.0, 4.0], [8.0, 8.0, 3.0]]], [[[6.0, 6.0, 2.0], [4.0, 6.0, 3.0], [3.0, 2.0, 8.0]], [[3.0, 
3.0, 1.0], [6.0, 3.0, 1.0], [3.0, 1.0, 1.0]], [[4.0, 5.0, 1.0], [2.0, 3.0, 8.0], [6.0, 2.0, 7.0]]], [[[4.0, 6.0, 4.0], [1.0, 
4.0, 8.0], [8.0, 8.0, 1.0]], [[6.0, 5.0, 7.0], [1.0, 1.0, 2.0], [1.0, 1.0, 8.0]], [[4.0, 2.0, 4.0], [7.0, 8.0, 1.0], [2.0, 2.0, 
8.0]]], [[[4.0, 7.0, 5.0], [4.0, 3.0, 3.0], [6.0, 8.0, 8.0]], [[5.0, 2.0, 5.0], [5.0, 1.0, 6.0], [8.0, 2.0, 3.0]], [[7.0, 2.0, 
1.0], [4.0, 8.0, 7.0], [3.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 2.0], [8.0, 2.0, 5.0], [3.0, 1.0, 2.0]], 
[[6.0, 7.0, 5.0], [3.0, 4.0, 4.0], [8.0, 2.0, 7.0]], [[7.0, 7.0, 5.0], [3.0, 6.0, 3.0], [8.0, 2.0, 4.0]]], [[[4.0, 1.0, 5.0], 
[8.0, 7.0, 7.0], [5.0, 7.0, 4.0]], [[3.0, 3.0, 3.0], [6.0, 3.0, 4.0], [3.0, 1.0, 7.0]], [[7.0, 1.0, 4.0], [6.0, 5.0, 6.0], 
[6.0, 7.0, 6.0]]], [[[3.0, 2.0, 5.0], [7.0, 2.0, 2.0], [3.0, 5.0, 3.0]], [[6.0, 5.0, 7.0], [1.0, 1.0, 1.0], [1.0, 1.0, 8.0]], 
[[6.0, 8.0, 2.0], [3.0, 2.0, 4.0], [1.0, 4.0, 6.0]]], [[[4.0, 7.0, 1.0], [1.0, 2.0, 3.0], [8.0, 8.0, 5.0]], [[5.0, 2.0, 4.0], 
[5.0, 1.0, 7.0], [8.0, 2.0, 4.0]], [[3.0, 6.0, 3.0], [2.0, 3.0, 5.0], [4.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRCU(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 1.0], [2.0, 7.0, 1.0], [8.0, 4.0, 7.0]], [[7.0, 6.0, 7.0], [5.0, 2.0, 7.0], [3.0, 6.0, 
4.0]], [[6.0, 7.0, 6.0], [1.0, 1.0, 3.0], [4.0, 4.0, 6.0]]], [[[7.0, 6.0, 5.0], [4.0, 1.0, 7.0], [3.0, 2.0, 5.0]], [[4.0, 4.0, 
3.0], [2.0, 3.0, 7.0], [5.0, 4.0, 8.0]], [[8.0, 6.0, 5.0], [5.0, 7.0, 1.0], [3.0, 2.0, 3.0]]], [[[8.0, 8.0, 8.0], [6.0, 3.0, 
7.0], [2.0, 5.0, 5.0]], [[4.0, 1.0, 8.0], [8.0, 1.0, 4.0], [7.0, 4.0, 3.0]], [[4.0, 3.0, 7.0], [7.0, 2.0, 7.0], [1.0, 7.0, 
2.0]]], [[[8.0, 1.0, 8.0], [3.0, 3.0, 2.0], [1.0, 8.0, 3.0]], [[5.0, 1.0, 5.0], [8.0, 2.0, 7.0], [7.0, 1.0, 6.0]], [[5.0, 7.0, 
8.0], [3.0, 6.0, 6.0], [4.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 4.0], [3.0, 7.0, 2.0], [1.0, 4.0, 8.0]], [[7.0, 1.0, 7.0], [3.0, 8.0, 
7.0], [8.0, 6.0, 1.0]], [[7.0, 7.0, 6.0], [3.0, 1.0, 1.0], [5.0, 6.0, 3.0]]], [[[1.0, 6.0, 6.0], [1.0, 2.0, 8.0], [6.0, 6.0, 
4.0]], [[2.0, 8.0, 6.0], [7.0, 3.0, 3.0], [7.0, 7.0, 2.0]], [[8.0, 5.0, 6.0], [6.0, 6.0, 6.0], [6.0, 8.0, 8.0]]], [[[8.0, 7.0, 
7.0], [3.0, 1.0, 4.0], [3.0, 8.0, 8.0]], [[7.0, 8.0, 2.0], [5.0, 6.0, 4.0], [1.0, 2.0, 1.0]], [[1.0, 1.0, 5.0], [3.0, 5.0, 
7.0], [4.0, 2.0, 1.0]]], [[[3.0, 4.0, 7.0], [5.0, 2.0, 3.0], [8.0, 5.0, 6.0]], [[1.0, 2.0, 2.0], [5.0, 8.0, 6.0], [4.0, 2.0, 
4.0]], [[1.0, 8.0, 1.0], [4.0, 5.0, 4.0], [4.0, 5.0, 2.0]]]]))
      val=Data(numpy.array([[[[8.0, 5.0], [3.0, 5.0], [1.0, 6.0]]], [[[6.0, 6.0], [2.0, 1.0], [5.0, 5.0]]], [[[4.0, 2.0], [5.0, 
4.0], [3.0, 4.0]]], [[[3.0, 4.0], [6.0, 1.0], [2.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 5.0], [1.0, 2.0], [2.0, 4.0]]], [[[1.0, 4.0], [1.0, 6.0], [1.0, 2.0]]], [[[2.0, 
7.0], [4.0, 1.0], [3.0, 8.0]]], [[[7.0, 1.0], [1.0, 4.0], [1.0, 1.0]]]]))
      arg[:,1:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 1.0], [2.0, 7.0, 1.0], [8.0, 4.0, 7.0]], [[8.0, 5.0, 7.0], [3.0, 5.0, 7.0], [1.0, 
6.0, 4.0]], [[6.0, 7.0, 6.0], [1.0, 1.0, 3.0], [4.0, 4.0, 6.0]]], [[[7.0, 6.0, 5.0], [4.0, 1.0, 7.0], [3.0, 2.0, 5.0]], [[6.0, 
6.0, 3.0], [2.0, 1.0, 7.0], [5.0, 5.0, 8.0]], [[8.0, 6.0, 5.0], [5.0, 7.0, 1.0], [3.0, 2.0, 3.0]]], [[[8.0, 8.0, 8.0], [6.0, 
3.0, 7.0], [2.0, 5.0, 5.0]], [[4.0, 2.0, 8.0], [5.0, 4.0, 4.0], [3.0, 4.0, 3.0]], [[4.0, 3.0, 7.0], [7.0, 2.0, 7.0], [1.0, 7.0, 
2.0]]], [[[8.0, 1.0, 8.0], [3.0, 3.0, 2.0], [1.0, 8.0, 3.0]], [[3.0, 4.0, 5.0], [6.0, 1.0, 7.0], [2.0, 2.0, 6.0]], [[5.0, 7.0, 
8.0], [3.0, 6.0, 6.0], [4.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [3.0, 7.0, 2.0], [1.0, 4.0, 8.0]], 
[[2.0, 5.0, 7.0], [1.0, 2.0, 7.0], [2.0, 4.0, 1.0]], [[7.0, 7.0, 6.0], [3.0, 1.0, 1.0], [5.0, 6.0, 3.0]]], [[[1.0, 6.0, 6.0], 
[1.0, 2.0, 8.0], [6.0, 6.0, 4.0]], [[1.0, 4.0, 6.0], [1.0, 6.0, 3.0], [1.0, 2.0, 2.0]], [[8.0, 5.0, 6.0], [6.0, 6.0, 6.0], 
[6.0, 8.0, 8.0]]], [[[8.0, 7.0, 7.0], [3.0, 1.0, 4.0], [3.0, 8.0, 8.0]], [[2.0, 7.0, 2.0], [4.0, 1.0, 4.0], [3.0, 8.0, 1.0]], 
[[1.0, 1.0, 5.0], [3.0, 5.0, 7.0], [4.0, 2.0, 1.0]]], [[[3.0, 4.0, 7.0], [5.0, 2.0, 3.0], [8.0, 5.0, 6.0]], [[7.0, 1.0, 2.0], 
[1.0, 4.0, 6.0], [1.0, 1.0, 4.0]], [[1.0, 8.0, 1.0], [4.0, 5.0, 4.0], [4.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRCU(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 5.0], [5.0, 4.0, 5.0], [5.0, 2.0, 8.0]], [[2.0, 6.0, 2.0], [5.0, 3.0, 8.0], [2.0, 1.0, 
6.0]], [[8.0, 6.0, 7.0], [2.0, 3.0, 4.0], [3.0, 2.0, 8.0]]], [[[4.0, 4.0, 6.0], [2.0, 3.0, 5.0], [2.0, 4.0, 2.0]], [[8.0, 5.0, 
7.0], [4.0, 6.0, 3.0], [7.0, 2.0, 2.0]], [[7.0, 7.0, 8.0], [8.0, 2.0, 2.0], [4.0, 8.0, 8.0]]], [[[6.0, 8.0, 8.0], [3.0, 7.0, 
1.0], [5.0, 8.0, 4.0]], [[7.0, 4.0, 8.0], [2.0, 3.0, 4.0], [2.0, 3.0, 6.0]], [[4.0, 8.0, 8.0], [5.0, 7.0, 5.0], [6.0, 2.0, 
7.0]]], [[[6.0, 2.0, 6.0], [5.0, 1.0, 2.0], [7.0, 2.0, 1.0]], [[1.0, 1.0, 4.0], [3.0, 1.0, 5.0], [8.0, 5.0, 6.0]], [[5.0, 5.0, 
5.0], [5.0, 6.0, 8.0], [5.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 6.0], [8.0, 2.0, 2.0], [7.0, 8.0, 4.0]], [[6.0, 8.0, 4.0], [6.0, 8.0, 
4.0], [4.0, 3.0, 1.0]], [[4.0, 2.0, 2.0], [6.0, 3.0, 8.0], [4.0, 8.0, 1.0]]], [[[8.0, 1.0, 2.0], [8.0, 4.0, 5.0], [7.0, 5.0, 
4.0]], [[7.0, 3.0, 5.0], [5.0, 8.0, 3.0], [1.0, 2.0, 3.0]], [[7.0, 7.0, 8.0], [1.0, 4.0, 1.0], [4.0, 2.0, 5.0]]], [[[5.0, 6.0, 
7.0], [6.0, 7.0, 8.0], [2.0, 1.0, 4.0]], [[5.0, 7.0, 5.0], [5.0, 7.0, 3.0], [6.0, 6.0, 7.0]], [[2.0, 8.0, 6.0], [3.0, 1.0, 
6.0], [6.0, 7.0, 3.0]]], [[[8.0, 1.0, 4.0], [2.0, 4.0, 2.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 8.0, 6.0], [3.0, 3.0, 
4.0]], [[8.0, 6.0, 6.0], [3.0, 4.0, 4.0], [2.0, 8.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 4.0], [1.0, 8.0], [5.0, 8.0]]], [[[6.0, 8.0], [1.0, 6.0], [6.0, 5.0]]], [[[4.0, 5.0], 
[5.0, 3.0], [4.0, 6.0]]], [[[6.0, 3.0], [1.0, 1.0], [6.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 4.0], [3.0, 3.0], [7.0, 
3.0]]], [[[8.0, 7.0], [3.0, 5.0], [2.0, 3.0]]], [[[3.0, 2.0], [5.0, 1.0], [2.0, 4.0]]], [[[2.0, 4.0], [6.0, 4.0], [7.0, 4.0]]]])
      arg[:,1:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 5.0], [5.0, 4.0, 5.0], [5.0, 2.0, 8.0]], [[8.0, 4.0, 2.0], [1.0, 8.0, 8.0], [5.0, 
8.0, 6.0]], [[8.0, 6.0, 7.0], [2.0, 3.0, 4.0], [3.0, 2.0, 8.0]]], [[[4.0, 4.0, 6.0], [2.0, 3.0, 5.0], [2.0, 4.0, 2.0]], [[6.0, 
8.0, 7.0], [1.0, 6.0, 3.0], [6.0, 5.0, 2.0]], [[7.0, 7.0, 8.0], [8.0, 2.0, 2.0], [4.0, 8.0, 8.0]]], [[[6.0, 8.0, 8.0], [3.0, 
7.0, 1.0], [5.0, 8.0, 4.0]], [[4.0, 5.0, 8.0], [5.0, 3.0, 4.0], [4.0, 6.0, 6.0]], [[4.0, 8.0, 8.0], [5.0, 7.0, 5.0], [6.0, 2.0, 
7.0]]], [[[6.0, 2.0, 6.0], [5.0, 1.0, 2.0], [7.0, 2.0, 1.0]], [[6.0, 3.0, 4.0], [1.0, 1.0, 5.0], [6.0, 6.0, 6.0]], [[5.0, 5.0, 
5.0], [5.0, 6.0, 8.0], [5.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 6.0], [8.0, 2.0, 2.0], [7.0, 8.0, 4.0]], 
[[7.0, 4.0, 4.0], [3.0, 3.0, 4.0], [7.0, 3.0, 1.0]], [[4.0, 2.0, 2.0], [6.0, 3.0, 8.0], [4.0, 8.0, 1.0]]], [[[8.0, 1.0, 2.0], 
[8.0, 4.0, 5.0], [7.0, 5.0, 4.0]], [[8.0, 7.0, 5.0], [3.0, 5.0, 3.0], [2.0, 3.0, 3.0]], [[7.0, 7.0, 8.0], [1.0, 4.0, 1.0], 
[4.0, 2.0, 5.0]]], [[[5.0, 6.0, 7.0], [6.0, 7.0, 8.0], [2.0, 1.0, 4.0]], [[3.0, 2.0, 5.0], [5.0, 1.0, 3.0], [2.0, 4.0, 7.0]], 
[[2.0, 8.0, 6.0], [3.0, 1.0, 6.0], [6.0, 7.0, 3.0]]], [[[8.0, 1.0, 4.0], [2.0, 4.0, 2.0], [1.0, 5.0, 7.0]], [[2.0, 4.0, 4.0], 
[6.0, 4.0, 6.0], [7.0, 4.0, 4.0]], [[8.0, 6.0, 6.0], [3.0, 4.0, 4.0], [2.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CLCL(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 5.0], [6.0, 4.0, 1.0], [6.0, 1.0, 5.0]], [[3.0, 5.0, 2.0], [5.0, 4.0, 4.0], [6.0, 5.0, 
2.0]], [[3.0, 3.0, 2.0], [7.0, 2.0, 6.0], [4.0, 3.0, 1.0]]], [[[1.0, 7.0, 5.0], [6.0, 8.0, 1.0], [2.0, 8.0, 3.0]], [[1.0, 7.0, 
3.0], [5.0, 3.0, 3.0], [8.0, 4.0, 3.0]], [[3.0, 5.0, 4.0], [6.0, 7.0, 3.0], [7.0, 6.0, 2.0]]], [[[7.0, 5.0, 1.0], [3.0, 6.0, 
1.0], [2.0, 2.0, 1.0]], [[4.0, 6.0, 8.0], [3.0, 7.0, 4.0], [5.0, 4.0, 2.0]], [[1.0, 5.0, 7.0], [3.0, 8.0, 2.0], [4.0, 2.0, 
8.0]]], [[[4.0, 2.0, 8.0], [3.0, 2.0, 7.0], [7.0, 3.0, 8.0]], [[5.0, 1.0, 8.0], [8.0, 1.0, 8.0], [3.0, 2.0, 4.0]], [[5.0, 1.0, 
5.0], [7.0, 3.0, 4.0], [5.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 4.0], [4.0, 7.0, 2.0], [1.0, 3.0, 4.0]], [[4.0, 3.0, 3.0], [1.0, 7.0, 
6.0], [5.0, 7.0, 5.0]], [[6.0, 8.0, 7.0], [1.0, 7.0, 4.0], [3.0, 1.0, 4.0]]], [[[1.0, 2.0, 4.0], [1.0, 2.0, 2.0], [6.0, 5.0, 
1.0]], [[4.0, 6.0, 2.0], [7.0, 4.0, 7.0], [1.0, 8.0, 8.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 1.0], [1.0, 8.0, 2.0]]], [[[2.0, 4.0, 
7.0], [6.0, 4.0, 1.0], [6.0, 5.0, 8.0]], [[4.0, 5.0, 3.0], [4.0, 4.0, 4.0], [7.0, 7.0, 1.0]], [[5.0, 1.0, 2.0], [3.0, 1.0, 
6.0], [8.0, 8.0, 8.0]]], [[[3.0, 2.0, 8.0], [5.0, 5.0, 8.0], [1.0, 8.0, 7.0]], [[5.0, 3.0, 1.0], [7.0, 1.0, 3.0], [5.0, 2.0, 
1.0]], [[1.0, 8.0, 3.0], [8.0, 5.0, 1.0], [1.0, 3.0, 6.0]]]]))
      val=numpy.array([[[[1.0], [6.0], [7.0]]], [[[7.0], [1.0], [2.0]]], [[[5.0], [2.0], [3.0]]], [[[6.0], [1.0], [1.0]]]])
      arg[:,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 5.0], [6.0, 4.0, 1.0], [6.0, 1.0, 5.0]], [[3.0, 5.0, 2.0], [5.0, 4.0, 4.0], [6.0, 
5.0, 2.0]], [[3.0, 3.0, 1.0], [7.0, 2.0, 6.0], [4.0, 3.0, 7.0]]], [[[1.0, 7.0, 5.0], [6.0, 8.0, 1.0], [2.0, 8.0, 3.0]], [[1.0, 
7.0, 3.0], [5.0, 3.0, 3.0], [8.0, 4.0, 3.0]], [[3.0, 5.0, 7.0], [6.0, 7.0, 1.0], [7.0, 6.0, 2.0]]], [[[7.0, 5.0, 1.0], [3.0, 
6.0, 1.0], [2.0, 2.0, 1.0]], [[4.0, 6.0, 8.0], [3.0, 7.0, 4.0], [5.0, 4.0, 2.0]], [[1.0, 5.0, 5.0], [3.0, 8.0, 2.0], [4.0, 2.0, 
3.0]]], [[[4.0, 2.0, 8.0], [3.0, 2.0, 7.0], [7.0, 3.0, 8.0]], [[5.0, 1.0, 8.0], [8.0, 1.0, 8.0], [3.0, 2.0, 4.0]], [[5.0, 1.0, 
6.0], [7.0, 3.0, 1.0], [5.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 4.0], [4.0, 7.0, 2.0], [1.0, 3.0, 4.0]], 
[[4.0, 3.0, 3.0], [1.0, 7.0, 6.0], [5.0, 7.0, 5.0]], [[6.0, 8.0, 1.0], [1.0, 7.0, 6.0], [3.0, 1.0, 7.0]]], [[[1.0, 2.0, 4.0], 
[1.0, 2.0, 2.0], [6.0, 5.0, 1.0]], [[4.0, 6.0, 2.0], [7.0, 4.0, 7.0], [1.0, 8.0, 8.0]], [[7.0, 5.0, 7.0], [1.0, 7.0, 1.0], 
[1.0, 8.0, 2.0]]], [[[2.0, 4.0, 7.0], [6.0, 4.0, 1.0], [6.0, 5.0, 8.0]], [[4.0, 5.0, 3.0], [4.0, 4.0, 4.0], [7.0, 7.0, 1.0]], 
[[5.0, 1.0, 5.0], [3.0, 1.0, 2.0], [8.0, 8.0, 3.0]]], [[[3.0, 2.0, 8.0], [5.0, 5.0, 8.0], [1.0, 8.0, 7.0]], [[5.0, 3.0, 1.0], 
[7.0, 1.0, 3.0], [5.0, 2.0, 1.0]], [[1.0, 8.0, 6.0], [8.0, 5.0, 1.0], [1.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CLCL(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 4.0], [3.0, 8.0, 8.0], [4.0, 6.0, 2.0]], [[3.0, 8.0, 1.0], [5.0, 3.0, 4.0], [8.0, 4.0, 
2.0]], [[8.0, 5.0, 3.0], [8.0, 6.0, 6.0], [4.0, 7.0, 4.0]]], [[[3.0, 7.0, 4.0], [7.0, 2.0, 8.0], [8.0, 7.0, 6.0]], [[7.0, 7.0, 
5.0], [7.0, 1.0, 4.0], [1.0, 8.0, 6.0]], [[8.0, 4.0, 2.0], [7.0, 6.0, 8.0], [4.0, 5.0, 8.0]]], [[[1.0, 6.0, 1.0], [7.0, 1.0, 
2.0], [1.0, 8.0, 2.0]], [[3.0, 8.0, 4.0], [3.0, 1.0, 5.0], [1.0, 6.0, 6.0]], [[3.0, 8.0, 5.0], [4.0, 1.0, 3.0], [4.0, 6.0, 
7.0]]], [[[7.0, 5.0, 1.0], [1.0, 7.0, 2.0], [3.0, 6.0, 1.0]], [[5.0, 5.0, 2.0], [1.0, 8.0, 3.0], [2.0, 1.0, 2.0]], [[6.0, 4.0, 
4.0], [1.0, 6.0, 2.0], [6.0, 5.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 6.0], [8.0, 1.0, 5.0], [4.0, 8.0, 3.0]], [[1.0, 5.0, 3.0], [7.0, 7.0, 
7.0], [6.0, 4.0, 1.0]], [[4.0, 1.0, 8.0], [1.0, 6.0, 2.0], [2.0, 4.0, 3.0]]], [[[3.0, 7.0, 5.0], [8.0, 1.0, 4.0], [4.0, 7.0, 
4.0]], [[5.0, 6.0, 5.0], [8.0, 7.0, 6.0], [8.0, 2.0, 3.0]], [[6.0, 8.0, 1.0], [6.0, 5.0, 8.0], [4.0, 7.0, 7.0]]], [[[4.0, 3.0, 
4.0], [4.0, 7.0, 4.0], [5.0, 8.0, 4.0]], [[8.0, 2.0, 8.0], [5.0, 6.0, 7.0], [6.0, 5.0, 1.0]], [[5.0, 3.0, 4.0], [5.0, 5.0, 
1.0], [2.0, 3.0, 2.0]]], [[[4.0, 7.0, 3.0], [7.0, 7.0, 3.0], [2.0, 4.0, 2.0]], [[6.0, 1.0, 2.0], [4.0, 2.0, 6.0], [5.0, 6.0, 
7.0]], [[1.0, 3.0, 4.0], [5.0, 5.0, 6.0], [8.0, 1.0, 1.0]]]]))
      val=Data(numpy.array([[[[1.0], [7.0], [6.0]]], [[[2.0], [8.0], [5.0]]], [[[4.0], [6.0], [4.0]]], [[[6.0], [4.0], 
[6.0]]]]),self.functionspace)
      arg[:,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 4.0], [3.0, 8.0, 8.0], [4.0, 6.0, 2.0]], [[3.0, 8.0, 1.0], [5.0, 3.0, 4.0], [8.0, 
4.0, 2.0]], [[8.0, 5.0, 1.0], [8.0, 6.0, 7.0], [4.0, 7.0, 6.0]]], [[[3.0, 7.0, 4.0], [7.0, 2.0, 8.0], [8.0, 7.0, 6.0]], [[7.0, 
7.0, 5.0], [7.0, 1.0, 4.0], [1.0, 8.0, 6.0]], [[8.0, 4.0, 2.0], [7.0, 6.0, 8.0], [4.0, 5.0, 5.0]]], [[[1.0, 6.0, 1.0], [7.0, 
1.0, 2.0], [1.0, 8.0, 2.0]], [[3.0, 8.0, 4.0], [3.0, 1.0, 5.0], [1.0, 6.0, 6.0]], [[3.0, 8.0, 4.0], [4.0, 1.0, 6.0], [4.0, 6.0, 
4.0]]], [[[7.0, 5.0, 1.0], [1.0, 7.0, 2.0], [3.0, 6.0, 1.0]], [[5.0, 5.0, 2.0], [1.0, 8.0, 3.0], [2.0, 1.0, 2.0]], [[6.0, 4.0, 
6.0], [1.0, 6.0, 4.0], [6.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 6.0], [8.0, 1.0, 5.0], [4.0, 8.0, 3.0]], 
[[1.0, 5.0, 3.0], [7.0, 7.0, 7.0], [6.0, 4.0, 1.0]], [[4.0, 1.0, 1.0], [1.0, 6.0, 7.0], [2.0, 4.0, 6.0]]], [[[3.0, 7.0, 5.0], 
[8.0, 1.0, 4.0], [4.0, 7.0, 4.0]], [[5.0, 6.0, 5.0], [8.0, 7.0, 6.0], [8.0, 2.0, 3.0]], [[6.0, 8.0, 2.0], [6.0, 5.0, 8.0], 
[4.0, 7.0, 5.0]]], [[[4.0, 3.0, 4.0], [4.0, 7.0, 4.0], [5.0, 8.0, 4.0]], [[8.0, 2.0, 8.0], [5.0, 6.0, 7.0], [6.0, 5.0, 1.0]], 
[[5.0, 3.0, 4.0], [5.0, 5.0, 6.0], [2.0, 3.0, 4.0]]], [[[4.0, 7.0, 3.0], [7.0, 7.0, 3.0], [2.0, 4.0, 2.0]], [[6.0, 1.0, 2.0], 
[4.0, 2.0, 6.0], [5.0, 6.0, 7.0]], [[1.0, 3.0, 6.0], [5.0, 5.0, 4.0], [8.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CLCL(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 4.0], [8.0, 3.0, 6.0], [8.0, 5.0, 1.0]], [[4.0, 2.0, 8.0], [7.0, 6.0, 2.0], [4.0, 5.0, 
3.0]], [[6.0, 7.0, 8.0], [3.0, 6.0, 6.0], [1.0, 1.0, 3.0]]], [[[4.0, 6.0, 3.0], [2.0, 5.0, 4.0], [3.0, 7.0, 2.0]], [[1.0, 2.0, 
7.0], [6.0, 3.0, 3.0], [1.0, 1.0, 1.0]], [[4.0, 3.0, 1.0], [7.0, 1.0, 4.0], [6.0, 8.0, 1.0]]], [[[7.0, 8.0, 8.0], [6.0, 1.0, 
1.0], [4.0, 6.0, 2.0]], [[8.0, 8.0, 6.0], [3.0, 4.0, 2.0], [2.0, 5.0, 1.0]], [[3.0, 6.0, 6.0], [7.0, 2.0, 8.0], [2.0, 8.0, 
4.0]]], [[[6.0, 5.0, 3.0], [4.0, 8.0, 6.0], [6.0, 1.0, 7.0]], [[4.0, 5.0, 2.0], [5.0, 3.0, 6.0], [6.0, 5.0, 4.0]], [[1.0, 6.0, 
5.0], [8.0, 8.0, 1.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 1.0], [6.0, 8.0, 2.0], [4.0, 8.0, 2.0]], [[8.0, 1.0, 2.0], [7.0, 3.0, 
4.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 6.0], [4.0, 7.0, 7.0], [6.0, 2.0, 3.0]]], [[[1.0, 7.0, 6.0], [5.0, 1.0, 5.0], [1.0, 1.0, 
8.0]], [[6.0, 4.0, 2.0], [6.0, 3.0, 4.0], [7.0, 8.0, 5.0]], [[2.0, 5.0, 1.0], [3.0, 6.0, 8.0], [6.0, 7.0, 7.0]]], [[[7.0, 3.0, 
2.0], [1.0, 4.0, 8.0], [8.0, 5.0, 7.0]], [[7.0, 8.0, 1.0], [3.0, 2.0, 8.0], [3.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], [2.0, 2.0, 
5.0], [6.0, 6.0, 6.0]]], [[[7.0, 1.0, 1.0], [4.0, 2.0, 7.0], [4.0, 6.0, 1.0]], [[1.0, 2.0, 7.0], [8.0, 2.0, 1.0], [5.0, 2.0, 
4.0]], [[7.0, 7.0, 8.0], [6.0, 8.0, 4.0], [6.0, 6.0, 6.0]]]]))
      val=Data(numpy.array([[[[3.0], [1.0], [1.0]]], [[[4.0], [3.0], [4.0]]], [[[8.0], [1.0], [5.0]]], [[[8.0], [7.0], 
[6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0], [3.0], [8.0]]], [[[2.0], [5.0], [3.0]]], [[[6.0], [6.0], [7.0]]], [[[1.0], 
[6.0], [7.0]]]]))
      arg[:,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 4.0], [8.0, 3.0, 6.0], [8.0, 5.0, 1.0]], [[4.0, 2.0, 8.0], [7.0, 6.0, 2.0], [4.0, 
5.0, 3.0]], [[6.0, 7.0, 3.0], [3.0, 6.0, 1.0], [1.0, 1.0, 1.0]]], [[[4.0, 6.0, 3.0], [2.0, 5.0, 4.0], [3.0, 7.0, 2.0]], [[1.0, 
2.0, 7.0], [6.0, 3.0, 3.0], [1.0, 1.0, 1.0]], [[4.0, 3.0, 4.0], [7.0, 1.0, 3.0], [6.0, 8.0, 4.0]]], [[[7.0, 8.0, 8.0], [6.0, 
1.0, 1.0], [4.0, 6.0, 2.0]], [[8.0, 8.0, 6.0], [3.0, 4.0, 2.0], [2.0, 5.0, 1.0]], [[3.0, 6.0, 8.0], [7.0, 2.0, 1.0], [2.0, 8.0, 
5.0]]], [[[6.0, 5.0, 3.0], [4.0, 8.0, 6.0], [6.0, 1.0, 7.0]], [[4.0, 5.0, 2.0], [5.0, 3.0, 6.0], [6.0, 5.0, 4.0]], [[1.0, 6.0, 
8.0], [8.0, 8.0, 7.0], [7.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 1.0], [6.0, 8.0, 2.0], [4.0, 8.0, 2.0]], 
[[8.0, 1.0, 2.0], [7.0, 3.0, 4.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 8.0], [4.0, 7.0, 3.0], [6.0, 2.0, 8.0]]], [[[1.0, 7.0, 6.0], 
[5.0, 1.0, 5.0], [1.0, 1.0, 8.0]], [[6.0, 4.0, 2.0], [6.0, 3.0, 4.0], [7.0, 8.0, 5.0]], [[2.0, 5.0, 2.0], [3.0, 6.0, 5.0], 
[6.0, 7.0, 3.0]]], [[[7.0, 3.0, 2.0], [1.0, 4.0, 8.0], [8.0, 5.0, 7.0]], [[7.0, 8.0, 1.0], [3.0, 2.0, 8.0], [3.0, 5.0, 5.0]], 
[[1.0, 6.0, 6.0], [2.0, 2.0, 6.0], [6.0, 6.0, 7.0]]], [[[7.0, 1.0, 1.0], [4.0, 2.0, 7.0], [4.0, 6.0, 1.0]], [[1.0, 2.0, 7.0], 
[8.0, 2.0, 1.0], [5.0, 2.0, 4.0]], [[7.0, 7.0, 1.0], [6.0, 8.0, 6.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CLCL(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 5.0], [2.0, 8.0, 6.0], [7.0, 1.0, 8.0]], [[7.0, 1.0, 6.0], [5.0, 8.0, 5.0], [6.0, 4.0, 
1.0]], [[3.0, 8.0, 6.0], [6.0, 3.0, 7.0], [7.0, 7.0, 3.0]]], [[[5.0, 5.0, 3.0], [2.0, 6.0, 5.0], [1.0, 3.0, 1.0]], [[3.0, 8.0, 
4.0], [1.0, 1.0, 2.0], [6.0, 1.0, 5.0]], [[8.0, 2.0, 1.0], [6.0, 2.0, 8.0], [2.0, 3.0, 3.0]]], [[[2.0, 7.0, 5.0], [2.0, 1.0, 
2.0], [5.0, 2.0, 1.0]], [[4.0, 5.0, 7.0], [5.0, 1.0, 4.0], [8.0, 5.0, 1.0]], [[6.0, 3.0, 2.0], [8.0, 6.0, 2.0], [1.0, 1.0, 
2.0]]], [[[1.0, 7.0, 8.0], [1.0, 1.0, 7.0], [5.0, 1.0, 1.0]], [[8.0, 8.0, 6.0], [5.0, 4.0, 7.0], [6.0, 5.0, 8.0]], [[2.0, 6.0, 
4.0], [7.0, 6.0, 8.0], [4.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 5.0], [8.0, 1.0, 5.0], [1.0, 8.0, 1.0]], [[3.0, 6.0, 4.0], [6.0, 2.0, 
4.0], [8.0, 1.0, 5.0]], [[4.0, 8.0, 3.0], [3.0, 1.0, 2.0], [4.0, 3.0, 7.0]]], [[[3.0, 4.0, 5.0], [1.0, 8.0, 3.0], [2.0, 7.0, 
6.0]], [[3.0, 6.0, 8.0], [8.0, 2.0, 8.0], [3.0, 7.0, 1.0]], [[5.0, 7.0, 4.0], [7.0, 1.0, 2.0], [1.0, 1.0, 6.0]]], [[[6.0, 8.0, 
8.0], [6.0, 6.0, 3.0], [1.0, 7.0, 8.0]], [[7.0, 1.0, 5.0], [2.0, 2.0, 5.0], [5.0, 6.0, 4.0]], [[6.0, 2.0, 3.0], [6.0, 7.0, 
8.0], [6.0, 4.0, 2.0]]], [[[1.0, 8.0, 8.0], [8.0, 4.0, 1.0], [1.0, 1.0, 1.0]], [[5.0, 2.0, 5.0], [7.0, 4.0, 7.0], [2.0, 6.0, 
3.0]], [[7.0, 5.0, 6.0], [3.0, 3.0, 2.0], [7.0, 1.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0], [7.0], [7.0]]], [[[7.0], [6.0], [7.0]]], [[[2.0], [6.0], [1.0]]], [[[1.0], [3.0], 
[1.0]]]])+(1.-msk_val)*numpy.array([[[[2.0], [6.0], [2.0]]], [[[8.0], [5.0], [5.0]]], [[[5.0], [5.0], [5.0]]], [[[5.0], [8.0], 
[3.0]]]])
      arg[:,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 5.0], [2.0, 8.0, 6.0], [7.0, 1.0, 8.0]], [[7.0, 1.0, 6.0], [5.0, 8.0, 5.0], [6.0, 
4.0, 1.0]], [[3.0, 8.0, 4.0], [6.0, 3.0, 7.0], [7.0, 7.0, 7.0]]], [[[5.0, 5.0, 3.0], [2.0, 6.0, 5.0], [1.0, 3.0, 1.0]], [[3.0, 
8.0, 4.0], [1.0, 1.0, 2.0], [6.0, 1.0, 5.0]], [[8.0, 2.0, 7.0], [6.0, 2.0, 6.0], [2.0, 3.0, 7.0]]], [[[2.0, 7.0, 5.0], [2.0, 
1.0, 2.0], [5.0, 2.0, 1.0]], [[4.0, 5.0, 7.0], [5.0, 1.0, 4.0], [8.0, 5.0, 1.0]], [[6.0, 3.0, 2.0], [8.0, 6.0, 6.0], [1.0, 1.0, 
1.0]]], [[[1.0, 7.0, 8.0], [1.0, 1.0, 7.0], [5.0, 1.0, 1.0]], [[8.0, 8.0, 6.0], [5.0, 4.0, 7.0], [6.0, 5.0, 8.0]], [[2.0, 6.0, 
1.0], [7.0, 6.0, 3.0], [4.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 5.0], [8.0, 1.0, 5.0], [1.0, 8.0, 1.0]], 
[[3.0, 6.0, 4.0], [6.0, 2.0, 4.0], [8.0, 1.0, 5.0]], [[4.0, 8.0, 2.0], [3.0, 1.0, 6.0], [4.0, 3.0, 2.0]]], [[[3.0, 4.0, 5.0], 
[1.0, 8.0, 3.0], [2.0, 7.0, 6.0]], [[3.0, 6.0, 8.0], [8.0, 2.0, 8.0], [3.0, 7.0, 1.0]], [[5.0, 7.0, 8.0], [7.0, 1.0, 5.0], 
[1.0, 1.0, 5.0]]], [[[6.0, 8.0, 8.0], [6.0, 6.0, 3.0], [1.0, 7.0, 8.0]], [[7.0, 1.0, 5.0], [2.0, 2.0, 5.0], [5.0, 6.0, 4.0]], 
[[6.0, 2.0, 5.0], [6.0, 7.0, 5.0], [6.0, 4.0, 5.0]]], [[[1.0, 8.0, 8.0], [8.0, 4.0, 1.0], [1.0, 1.0, 1.0]], [[5.0, 2.0, 5.0], 
[7.0, 4.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 5.0, 5.0], [3.0, 3.0, 8.0], [7.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULCL(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 7.0], [1.0, 5.0, 6.0], [5.0, 7.0, 2.0]], [[5.0, 6.0, 3.0], [8.0, 5.0, 5.0], [6.0, 4.0, 
4.0]], [[3.0, 4.0, 1.0], [3.0, 7.0, 4.0], [1.0, 3.0, 7.0]]], [[[4.0, 8.0, 5.0], [3.0, 6.0, 7.0], [2.0, 5.0, 5.0]], [[4.0, 3.0, 
2.0], [7.0, 4.0, 6.0], [4.0, 7.0, 8.0]], [[2.0, 5.0, 1.0], [6.0, 2.0, 2.0], [6.0, 4.0, 3.0]]], [[[3.0, 5.0, 1.0], [8.0, 5.0, 
7.0], [8.0, 2.0, 5.0]], [[2.0, 1.0, 1.0], [4.0, 7.0, 4.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 2.0], [7.0, 3.0, 3.0], [3.0, 4.0, 
5.0]]], [[[7.0, 8.0, 5.0], [2.0, 4.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 3.0, 2.0], [5.0, 3.0, 3.0], [1.0, 1.0, 2.0]], [[6.0, 5.0, 
3.0], [8.0, 3.0, 5.0], [7.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 1.0], [3.0, 1.0, 8.0], [3.0, 8.0, 3.0]], [[2.0, 8.0, 1.0], [1.0, 6.0, 
8.0], [7.0, 5.0, 1.0]], [[1.0, 2.0, 4.0], [7.0, 1.0, 2.0], [5.0, 3.0, 4.0]]], [[[2.0, 4.0, 1.0], [3.0, 4.0, 5.0], [6.0, 7.0, 
1.0]], [[6.0, 6.0, 8.0], [2.0, 3.0, 4.0], [8.0, 7.0, 5.0]], [[8.0, 2.0, 3.0], [6.0, 6.0, 6.0], [6.0, 6.0, 1.0]]], [[[5.0, 4.0, 
7.0], [8.0, 5.0, 1.0], [2.0, 8.0, 1.0]], [[2.0, 4.0, 8.0], [5.0, 8.0, 5.0], [1.0, 2.0, 6.0]], [[5.0, 6.0, 2.0], [4.0, 4.0, 
3.0], [7.0, 3.0, 7.0]]], [[[4.0, 4.0, 8.0], [6.0, 8.0, 3.0], [2.0, 4.0, 8.0]], [[3.0, 4.0, 4.0], [7.0, 6.0, 5.0], [7.0, 2.0, 
4.0]], [[4.0, 3.0, 1.0], [1.0, 2.0, 5.0], [1.0, 8.0, 5.0]]]]))
      val=numpy.array([[[[6.0], [8.0], [7.0]]], [[[8.0], [4.0], [1.0]]], [[[7.0], [4.0], [7.0]]]])
      arg[:3,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 7.0], [1.0, 5.0, 6.0], [5.0, 7.0, 2.0]], [[5.0, 6.0, 3.0], [8.0, 5.0, 5.0], [6.0, 
4.0, 4.0]], [[3.0, 4.0, 6.0], [3.0, 7.0, 8.0], [1.0, 3.0, 7.0]]], [[[4.0, 8.0, 5.0], [3.0, 6.0, 7.0], [2.0, 5.0, 5.0]], [[4.0, 
3.0, 2.0], [7.0, 4.0, 6.0], [4.0, 7.0, 8.0]], [[2.0, 5.0, 8.0], [6.0, 2.0, 4.0], [6.0, 4.0, 1.0]]], [[[3.0, 5.0, 1.0], [8.0, 
5.0, 7.0], [8.0, 2.0, 5.0]], [[2.0, 1.0, 1.0], [4.0, 7.0, 4.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 7.0], [7.0, 3.0, 4.0], [3.0, 4.0, 
7.0]]], [[[7.0, 8.0, 5.0], [2.0, 4.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 3.0, 2.0], [5.0, 3.0, 3.0], [1.0, 1.0, 2.0]], [[6.0, 5.0, 
3.0], [8.0, 3.0, 5.0], [7.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 1.0], [3.0, 1.0, 8.0], [3.0, 8.0, 3.0]], 
[[2.0, 8.0, 1.0], [1.0, 6.0, 8.0], [7.0, 5.0, 1.0]], [[1.0, 2.0, 6.0], [7.0, 1.0, 8.0], [5.0, 3.0, 7.0]]], [[[2.0, 4.0, 1.0], 
[3.0, 4.0, 5.0], [6.0, 7.0, 1.0]], [[6.0, 6.0, 8.0], [2.0, 3.0, 4.0], [8.0, 7.0, 5.0]], [[8.0, 2.0, 8.0], [6.0, 6.0, 4.0], 
[6.0, 6.0, 1.0]]], [[[5.0, 4.0, 7.0], [8.0, 5.0, 1.0], [2.0, 8.0, 1.0]], [[2.0, 4.0, 8.0], [5.0, 8.0, 5.0], [1.0, 2.0, 6.0]], 
[[5.0, 6.0, 7.0], [4.0, 4.0, 4.0], [7.0, 3.0, 7.0]]], [[[4.0, 4.0, 8.0], [6.0, 8.0, 3.0], [2.0, 4.0, 8.0]], [[3.0, 4.0, 4.0], 
[7.0, 6.0, 5.0], [7.0, 2.0, 4.0]], [[4.0, 3.0, 1.0], [1.0, 2.0, 5.0], [1.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULCL(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 5.0], [7.0, 5.0, 5.0], [2.0, 7.0, 7.0]], [[4.0, 2.0, 6.0], [8.0, 7.0, 5.0], [6.0, 3.0, 
3.0]], [[1.0, 4.0, 1.0], [2.0, 6.0, 3.0], [6.0, 8.0, 6.0]]], [[[1.0, 4.0, 5.0], [8.0, 5.0, 4.0], [8.0, 3.0, 5.0]], [[3.0, 6.0, 
1.0], [5.0, 3.0, 2.0], [1.0, 3.0, 4.0]], [[2.0, 5.0, 3.0], [7.0, 2.0, 7.0], [2.0, 7.0, 5.0]]], [[[6.0, 2.0, 1.0], [2.0, 5.0, 
7.0], [7.0, 2.0, 5.0]], [[8.0, 6.0, 4.0], [2.0, 6.0, 5.0], [1.0, 4.0, 6.0]], [[1.0, 6.0, 7.0], [8.0, 4.0, 2.0], [7.0, 3.0, 
1.0]]], [[[2.0, 2.0, 6.0], [8.0, 1.0, 7.0], [4.0, 2.0, 2.0]], [[7.0, 7.0, 1.0], [1.0, 5.0, 3.0], [3.0, 5.0, 6.0]], [[2.0, 2.0, 
5.0], [6.0, 1.0, 1.0], [3.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 2.0], [7.0, 6.0, 3.0], [8.0, 6.0, 7.0]], [[3.0, 8.0, 5.0], [3.0, 1.0, 
4.0], [5.0, 6.0, 6.0]], [[3.0, 1.0, 7.0], [8.0, 3.0, 8.0], [8.0, 8.0, 6.0]]], [[[4.0, 3.0, 3.0], [1.0, 1.0, 3.0], [1.0, 1.0, 
6.0]], [[8.0, 3.0, 8.0], [6.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[8.0, 5.0, 8.0], [8.0, 4.0, 5.0], [5.0, 8.0, 3.0]]], [[[4.0, 4.0, 
1.0], [4.0, 3.0, 1.0], [1.0, 6.0, 7.0]], [[7.0, 3.0, 3.0], [2.0, 4.0, 2.0], [1.0, 6.0, 8.0]], [[6.0, 2.0, 8.0], [6.0, 1.0, 
4.0], [5.0, 5.0, 7.0]]], [[[6.0, 7.0, 4.0], [4.0, 3.0, 4.0], [2.0, 8.0, 2.0]], [[1.0, 2.0, 7.0], [3.0, 2.0, 8.0], [8.0, 3.0, 
5.0]], [[4.0, 5.0, 3.0], [6.0, 6.0, 2.0], [5.0, 1.0, 8.0]]]]))
      val=Data(numpy.array([[[[7.0], [1.0], [4.0]]], [[[3.0], [7.0], [7.0]]], [[[3.0], [4.0], [7.0]]]]),self.functionspace)
      arg[:3,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 5.0], [7.0, 5.0, 5.0], [2.0, 7.0, 7.0]], [[4.0, 2.0, 6.0], [8.0, 7.0, 5.0], [6.0, 
3.0, 3.0]], [[1.0, 4.0, 7.0], [2.0, 6.0, 1.0], [6.0, 8.0, 4.0]]], [[[1.0, 4.0, 5.0], [8.0, 5.0, 4.0], [8.0, 3.0, 5.0]], [[3.0, 
6.0, 1.0], [5.0, 3.0, 2.0], [1.0, 3.0, 4.0]], [[2.0, 5.0, 3.0], [7.0, 2.0, 7.0], [2.0, 7.0, 7.0]]], [[[6.0, 2.0, 1.0], [2.0, 
5.0, 7.0], [7.0, 2.0, 5.0]], [[8.0, 6.0, 4.0], [2.0, 6.0, 5.0], [1.0, 4.0, 6.0]], [[1.0, 6.0, 3.0], [8.0, 4.0, 4.0], [7.0, 3.0, 
7.0]]], [[[2.0, 2.0, 6.0], [8.0, 1.0, 7.0], [4.0, 2.0, 2.0]], [[7.0, 7.0, 1.0], [1.0, 5.0, 3.0], [3.0, 5.0, 6.0]], [[2.0, 2.0, 
5.0], [6.0, 1.0, 1.0], [3.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 2.0], [7.0, 6.0, 3.0], [8.0, 6.0, 7.0]], 
[[3.0, 8.0, 5.0], [3.0, 1.0, 4.0], [5.0, 6.0, 6.0]], [[3.0, 1.0, 7.0], [8.0, 3.0, 1.0], [8.0, 8.0, 4.0]]], [[[4.0, 3.0, 3.0], 
[1.0, 1.0, 3.0], [1.0, 1.0, 6.0]], [[8.0, 3.0, 8.0], [6.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[8.0, 5.0, 3.0], [8.0, 4.0, 7.0], 
[5.0, 8.0, 7.0]]], [[[4.0, 4.0, 1.0], [4.0, 3.0, 1.0], [1.0, 6.0, 7.0]], [[7.0, 3.0, 3.0], [2.0, 4.0, 2.0], [1.0, 6.0, 8.0]], 
[[6.0, 2.0, 3.0], [6.0, 1.0, 4.0], [5.0, 5.0, 7.0]]], [[[6.0, 7.0, 4.0], [4.0, 3.0, 4.0], [2.0, 8.0, 2.0]], [[1.0, 2.0, 7.0], 
[3.0, 2.0, 8.0], [8.0, 3.0, 5.0]], [[4.0, 5.0, 3.0], [6.0, 6.0, 2.0], [5.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULCL(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 4.0], [1.0, 6.0, 1.0], [3.0, 3.0, 7.0]], [[6.0, 3.0, 4.0], [6.0, 8.0, 2.0], [1.0, 5.0, 
2.0]], [[1.0, 2.0, 3.0], [6.0, 2.0, 3.0], [3.0, 4.0, 3.0]]], [[[2.0, 5.0, 6.0], [7.0, 7.0, 1.0], [4.0, 4.0, 6.0]], [[7.0, 7.0, 
1.0], [5.0, 1.0, 2.0], [2.0, 5.0, 2.0]], [[7.0, 8.0, 5.0], [3.0, 5.0, 8.0], [7.0, 7.0, 2.0]]], [[[4.0, 4.0, 1.0], [7.0, 2.0, 
2.0], [3.0, 7.0, 8.0]], [[8.0, 2.0, 8.0], [1.0, 8.0, 8.0], [4.0, 8.0, 6.0]], [[6.0, 3.0, 2.0], [1.0, 4.0, 6.0], [7.0, 2.0, 
2.0]]], [[[4.0, 3.0, 3.0], [8.0, 2.0, 6.0], [6.0, 5.0, 6.0]], [[6.0, 5.0, 6.0], [5.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[6.0, 2.0, 
4.0], [3.0, 3.0, 1.0], [5.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 2.0], [2.0, 6.0, 4.0], [1.0, 7.0, 7.0]], [[7.0, 4.0, 5.0], [4.0, 2.0, 
5.0], [1.0, 5.0, 7.0]], [[7.0, 1.0, 4.0], [6.0, 7.0, 8.0], [7.0, 2.0, 4.0]]], [[[8.0, 6.0, 1.0], [2.0, 3.0, 8.0], [5.0, 7.0, 
8.0]], [[8.0, 7.0, 2.0], [2.0, 7.0, 1.0], [4.0, 4.0, 1.0]], [[4.0, 4.0, 4.0], [3.0, 8.0, 2.0], [2.0, 8.0, 2.0]]], [[[6.0, 7.0, 
1.0], [4.0, 5.0, 8.0], [1.0, 5.0, 8.0]], [[4.0, 8.0, 3.0], [1.0, 7.0, 5.0], [7.0, 8.0, 4.0]], [[8.0, 6.0, 4.0], [5.0, 2.0, 
4.0], [2.0, 2.0, 2.0]]], [[[4.0, 8.0, 7.0], [8.0, 4.0, 2.0], [1.0, 3.0, 3.0]], [[2.0, 1.0, 1.0], [5.0, 5.0, 8.0], [5.0, 6.0, 
3.0]], [[1.0, 4.0, 2.0], [1.0, 4.0, 7.0], [3.0, 5.0, 1.0]]]]))
      val=Data(numpy.array([[[[8.0], [7.0], [7.0]]], [[[7.0], [6.0], [1.0]]], [[[5.0], [6.0], [5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0], [2.0], [2.0]]], [[[7.0], [7.0], [7.0]]], [[[3.0], [8.0], [7.0]]]]))
      arg[:3,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 4.0], [1.0, 6.0, 1.0], [3.0, 3.0, 7.0]], [[6.0, 3.0, 4.0], [6.0, 8.0, 2.0], [1.0, 
5.0, 2.0]], [[1.0, 2.0, 8.0], [6.0, 2.0, 7.0], [3.0, 4.0, 7.0]]], [[[2.0, 5.0, 6.0], [7.0, 7.0, 1.0], [4.0, 4.0, 6.0]], [[7.0, 
7.0, 1.0], [5.0, 1.0, 2.0], [2.0, 5.0, 2.0]], [[7.0, 8.0, 7.0], [3.0, 5.0, 6.0], [7.0, 7.0, 1.0]]], [[[4.0, 4.0, 1.0], [7.0, 
2.0, 2.0], [3.0, 7.0, 8.0]], [[8.0, 2.0, 8.0], [1.0, 8.0, 8.0], [4.0, 8.0, 6.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 6.0], [7.0, 2.0, 
5.0]]], [[[4.0, 3.0, 3.0], [8.0, 2.0, 6.0], [6.0, 5.0, 6.0]], [[6.0, 5.0, 6.0], [5.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[6.0, 2.0, 
4.0], [3.0, 3.0, 1.0], [5.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 2.0], [2.0, 6.0, 4.0], [1.0, 7.0, 7.0]], 
[[7.0, 4.0, 5.0], [4.0, 2.0, 5.0], [1.0, 5.0, 7.0]], [[7.0, 1.0, 7.0], [6.0, 7.0, 2.0], [7.0, 2.0, 2.0]]], [[[8.0, 6.0, 1.0], 
[2.0, 3.0, 8.0], [5.0, 7.0, 8.0]], [[8.0, 7.0, 2.0], [2.0, 7.0, 1.0], [4.0, 4.0, 1.0]], [[4.0, 4.0, 7.0], [3.0, 8.0, 7.0], 
[2.0, 8.0, 7.0]]], [[[6.0, 7.0, 1.0], [4.0, 5.0, 8.0], [1.0, 5.0, 8.0]], [[4.0, 8.0, 3.0], [1.0, 7.0, 5.0], [7.0, 8.0, 4.0]], 
[[8.0, 6.0, 3.0], [5.0, 2.0, 8.0], [2.0, 2.0, 7.0]]], [[[4.0, 8.0, 7.0], [8.0, 4.0, 2.0], [1.0, 3.0, 3.0]], [[2.0, 1.0, 1.0], 
[5.0, 5.0, 8.0], [5.0, 6.0, 3.0]], [[1.0, 4.0, 2.0], [1.0, 4.0, 7.0], [3.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULCL(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 5.0], [3.0, 7.0, 1.0], [8.0, 2.0, 8.0]], [[5.0, 4.0, 7.0], [2.0, 1.0, 4.0], [3.0, 6.0, 
4.0]], [[7.0, 8.0, 3.0], [7.0, 5.0, 8.0], [8.0, 4.0, 5.0]]], [[[2.0, 4.0, 6.0], [4.0, 2.0, 2.0], [4.0, 1.0, 1.0]], [[5.0, 5.0, 
2.0], [5.0, 4.0, 8.0], [8.0, 1.0, 5.0]], [[2.0, 1.0, 8.0], [8.0, 4.0, 3.0], [5.0, 8.0, 8.0]]], [[[2.0, 5.0, 7.0], [2.0, 4.0, 
4.0], [2.0, 1.0, 4.0]], [[1.0, 1.0, 4.0], [4.0, 3.0, 4.0], [1.0, 2.0, 8.0]], [[8.0, 1.0, 8.0], [7.0, 2.0, 5.0], [1.0, 6.0, 
4.0]]], [[[8.0, 3.0, 6.0], [3.0, 4.0, 7.0], [1.0, 8.0, 2.0]], [[7.0, 1.0, 7.0], [4.0, 2.0, 6.0], [6.0, 3.0, 2.0]], [[6.0, 3.0, 
8.0], [3.0, 8.0, 4.0], [8.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 4.0], [4.0, 7.0, 1.0], [3.0, 6.0, 2.0]], [[6.0, 7.0, 4.0], [3.0, 2.0, 
7.0], [3.0, 3.0, 1.0]], [[3.0, 6.0, 3.0], [2.0, 8.0, 4.0], [6.0, 3.0, 7.0]]], [[[2.0, 2.0, 5.0], [8.0, 2.0, 4.0], [8.0, 1.0, 
1.0]], [[2.0, 8.0, 5.0], [2.0, 6.0, 6.0], [1.0, 1.0, 6.0]], [[2.0, 2.0, 8.0], [8.0, 8.0, 6.0], [8.0, 5.0, 8.0]]], [[[6.0, 1.0, 
6.0], [2.0, 1.0, 8.0], [3.0, 1.0, 1.0]], [[6.0, 1.0, 6.0], [8.0, 8.0, 1.0], [4.0, 7.0, 2.0]], [[1.0, 7.0, 8.0], [1.0, 3.0, 
4.0], [1.0, 4.0, 8.0]]], [[[8.0, 4.0, 7.0], [1.0, 5.0, 3.0], [6.0, 4.0, 4.0]], [[4.0, 4.0, 2.0], [6.0, 4.0, 3.0], [2.0, 8.0, 
6.0]], [[3.0, 5.0, 1.0], [8.0, 2.0, 2.0], [2.0, 8.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0], [6.0], [2.0]]], [[[8.0], [1.0], [8.0]]], [[[7.0], [2.0], 
[7.0]]]])+(1.-msk_val)*numpy.array([[[[5.0], [6.0], [1.0]]], [[[7.0], [7.0], [2.0]]], [[[3.0], [6.0], [1.0]]]])
      arg[:3,2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 5.0], [3.0, 7.0, 1.0], [8.0, 2.0, 8.0]], [[5.0, 4.0, 7.0], [2.0, 1.0, 4.0], [3.0, 
6.0, 4.0]], [[7.0, 8.0, 3.0], [7.0, 5.0, 6.0], [8.0, 4.0, 2.0]]], [[[2.0, 4.0, 6.0], [4.0, 2.0, 2.0], [4.0, 1.0, 1.0]], [[5.0, 
5.0, 2.0], [5.0, 4.0, 8.0], [8.0, 1.0, 5.0]], [[2.0, 1.0, 8.0], [8.0, 4.0, 1.0], [5.0, 8.0, 8.0]]], [[[2.0, 5.0, 7.0], [2.0, 
4.0, 4.0], [2.0, 1.0, 4.0]], [[1.0, 1.0, 4.0], [4.0, 3.0, 4.0], [1.0, 2.0, 8.0]], [[8.0, 1.0, 7.0], [7.0, 2.0, 2.0], [1.0, 6.0, 
7.0]]], [[[8.0, 3.0, 6.0], [3.0, 4.0, 7.0], [1.0, 8.0, 2.0]], [[7.0, 1.0, 7.0], [4.0, 2.0, 6.0], [6.0, 3.0, 2.0]], [[6.0, 3.0, 
8.0], [3.0, 8.0, 4.0], [8.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 4.0], [4.0, 7.0, 1.0], [3.0, 6.0, 2.0]], 
[[6.0, 7.0, 4.0], [3.0, 2.0, 7.0], [3.0, 3.0, 1.0]], [[3.0, 6.0, 5.0], [2.0, 8.0, 6.0], [6.0, 3.0, 1.0]]], [[[2.0, 2.0, 5.0], 
[8.0, 2.0, 4.0], [8.0, 1.0, 1.0]], [[2.0, 8.0, 5.0], [2.0, 6.0, 6.0], [1.0, 1.0, 6.0]], [[2.0, 2.0, 7.0], [8.0, 8.0, 7.0], 
[8.0, 5.0, 2.0]]], [[[6.0, 1.0, 6.0], [2.0, 1.0, 8.0], [3.0, 1.0, 1.0]], [[6.0, 1.0, 6.0], [8.0, 8.0, 1.0], [4.0, 7.0, 2.0]], 
[[1.0, 7.0, 3.0], [1.0, 3.0, 6.0], [1.0, 4.0, 1.0]]], [[[8.0, 4.0, 7.0], [1.0, 5.0, 3.0], [6.0, 4.0, 4.0]], [[4.0, 4.0, 2.0], 
[6.0, 4.0, 3.0], [2.0, 8.0, 6.0]], [[3.0, 5.0, 1.0], [8.0, 2.0, 2.0], [2.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULRL(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 4.0], [3.0, 3.0, 2.0], [6.0, 3.0, 7.0]], [[5.0, 2.0, 4.0], [6.0, 8.0, 7.0], [8.0, 8.0, 
3.0]], [[1.0, 8.0, 1.0], [3.0, 1.0, 1.0], [3.0, 2.0, 5.0]]], [[[6.0, 2.0, 5.0], [6.0, 5.0, 2.0], [1.0, 4.0, 6.0]], [[8.0, 4.0, 
5.0], [8.0, 4.0, 5.0], [2.0, 6.0, 4.0]], [[5.0, 4.0, 6.0], [5.0, 8.0, 7.0], [2.0, 1.0, 8.0]]], [[[8.0, 5.0, 7.0], [1.0, 3.0, 
6.0], [3.0, 7.0, 4.0]], [[2.0, 2.0, 1.0], [1.0, 7.0, 2.0], [5.0, 6.0, 6.0]], [[4.0, 7.0, 3.0], [2.0, 7.0, 4.0], [3.0, 6.0, 
1.0]]], [[[8.0, 8.0, 4.0], [4.0, 5.0, 5.0], [1.0, 6.0, 6.0]], [[5.0, 2.0, 1.0], [8.0, 5.0, 1.0], [5.0, 1.0, 3.0]], [[7.0, 6.0, 
4.0], [6.0, 1.0, 7.0], [1.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 6.0], [1.0, 4.0, 7.0], [8.0, 6.0, 5.0]], [[2.0, 3.0, 1.0], [4.0, 2.0, 
8.0], [4.0, 3.0, 5.0]], [[4.0, 5.0, 4.0], [7.0, 6.0, 1.0], [2.0, 5.0, 1.0]]], [[[1.0, 2.0, 7.0], [8.0, 1.0, 2.0], [1.0, 4.0, 
6.0]], [[3.0, 7.0, 8.0], [2.0, 3.0, 6.0], [6.0, 1.0, 7.0]], [[4.0, 4.0, 2.0], [6.0, 3.0, 3.0], [8.0, 4.0, 3.0]]], [[[7.0, 2.0, 
3.0], [8.0, 1.0, 2.0], [2.0, 7.0, 2.0]], [[1.0, 7.0, 8.0], [7.0, 1.0, 4.0], [8.0, 6.0, 3.0]], [[7.0, 6.0, 6.0], [2.0, 1.0, 
7.0], [6.0, 6.0, 3.0]]], [[[6.0, 1.0, 6.0], [6.0, 3.0, 6.0], [6.0, 5.0, 2.0]], [[1.0, 8.0, 1.0], [3.0, 1.0, 2.0], [4.0, 5.0, 
6.0]], [[8.0, 6.0, 4.0], [3.0, 7.0, 3.0], [1.0, 3.0, 2.0]]]]))
      val=numpy.array([[[[5.0]]], [[[2.0]]], [[[4.0]]]])
      arg[:3,2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 4.0], [3.0, 3.0, 2.0], [6.0, 3.0, 7.0]], [[5.0, 2.0, 4.0], [6.0, 8.0, 7.0], [8.0, 
8.0, 3.0]], [[1.0, 8.0, 1.0], [3.0, 1.0, 5.0], [3.0, 2.0, 5.0]]], [[[6.0, 2.0, 5.0], [6.0, 5.0, 2.0], [1.0, 4.0, 6.0]], [[8.0, 
4.0, 5.0], [8.0, 4.0, 5.0], [2.0, 6.0, 4.0]], [[5.0, 4.0, 6.0], [5.0, 8.0, 2.0], [2.0, 1.0, 8.0]]], [[[8.0, 5.0, 7.0], [1.0, 
3.0, 6.0], [3.0, 7.0, 4.0]], [[2.0, 2.0, 1.0], [1.0, 7.0, 2.0], [5.0, 6.0, 6.0]], [[4.0, 7.0, 3.0], [2.0, 7.0, 4.0], [3.0, 6.0, 
1.0]]], [[[8.0, 8.0, 4.0], [4.0, 5.0, 5.0], [1.0, 6.0, 6.0]], [[5.0, 2.0, 1.0], [8.0, 5.0, 1.0], [5.0, 1.0, 3.0]], [[7.0, 6.0, 
4.0], [6.0, 1.0, 7.0], [1.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 6.0], [1.0, 4.0, 7.0], [8.0, 6.0, 5.0]], 
[[2.0, 3.0, 1.0], [4.0, 2.0, 8.0], [4.0, 3.0, 5.0]], [[4.0, 5.0, 4.0], [7.0, 6.0, 5.0], [2.0, 5.0, 1.0]]], [[[1.0, 2.0, 7.0], 
[8.0, 1.0, 2.0], [1.0, 4.0, 6.0]], [[3.0, 7.0, 8.0], [2.0, 3.0, 6.0], [6.0, 1.0, 7.0]], [[4.0, 4.0, 2.0], [6.0, 3.0, 2.0], 
[8.0, 4.0, 3.0]]], [[[7.0, 2.0, 3.0], [8.0, 1.0, 2.0], [2.0, 7.0, 2.0]], [[1.0, 7.0, 8.0], [7.0, 1.0, 4.0], [8.0, 6.0, 3.0]], 
[[7.0, 6.0, 6.0], [2.0, 1.0, 4.0], [6.0, 6.0, 3.0]]], [[[6.0, 1.0, 6.0], [6.0, 3.0, 6.0], [6.0, 5.0, 2.0]], [[1.0, 8.0, 1.0], 
[3.0, 1.0, 2.0], [4.0, 5.0, 6.0]], [[8.0, 6.0, 4.0], [3.0, 7.0, 3.0], [1.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULRL(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 8.0], [7.0, 5.0, 4.0], [7.0, 8.0, 4.0]], [[2.0, 3.0, 5.0], [7.0, 7.0, 8.0], [7.0, 4.0, 
2.0]], [[5.0, 3.0, 6.0], [4.0, 7.0, 2.0], [7.0, 5.0, 1.0]]], [[[1.0, 1.0, 6.0], [5.0, 3.0, 5.0], [6.0, 2.0, 8.0]], [[8.0, 4.0, 
8.0], [3.0, 1.0, 5.0], [2.0, 3.0, 4.0]], [[7.0, 8.0, 8.0], [8.0, 2.0, 5.0], [8.0, 1.0, 7.0]]], [[[3.0, 5.0, 1.0], [3.0, 4.0, 
7.0], [7.0, 1.0, 4.0]], [[1.0, 8.0, 7.0], [2.0, 2.0, 1.0], [1.0, 1.0, 8.0]], [[7.0, 8.0, 8.0], [8.0, 7.0, 3.0], [2.0, 2.0, 
7.0]]], [[[8.0, 7.0, 1.0], [4.0, 1.0, 7.0], [8.0, 5.0, 1.0]], [[2.0, 1.0, 2.0], [1.0, 8.0, 1.0], [2.0, 3.0, 2.0]], [[7.0, 1.0, 
1.0], [8.0, 2.0, 2.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 7.0], [4.0, 1.0, 8.0], [8.0, 1.0, 6.0]], [[2.0, 4.0, 4.0], [5.0, 6.0, 
8.0], [5.0, 2.0, 8.0]], [[3.0, 6.0, 4.0], [8.0, 8.0, 2.0], [4.0, 1.0, 6.0]]], [[[5.0, 1.0, 6.0], [3.0, 2.0, 8.0], [1.0, 3.0, 
8.0]], [[3.0, 6.0, 7.0], [7.0, 2.0, 4.0], [3.0, 6.0, 1.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 5.0], [4.0, 7.0, 3.0]]], [[[3.0, 4.0, 
4.0], [7.0, 6.0, 2.0], [2.0, 2.0, 1.0]], [[3.0, 6.0, 8.0], [1.0, 3.0, 1.0], [3.0, 5.0, 6.0]], [[2.0, 1.0, 5.0], [1.0, 3.0, 
3.0], [8.0, 6.0, 8.0]]], [[[4.0, 4.0, 6.0], [1.0, 3.0, 5.0], [1.0, 8.0, 6.0]], [[7.0, 7.0, 1.0], [6.0, 2.0, 3.0], [2.0, 1.0, 
3.0]], [[5.0, 3.0, 7.0], [5.0, 5.0, 7.0], [1.0, 1.0, 7.0]]]]))
      val=Data(numpy.array([[[[5.0]]], [[[3.0]]], [[[8.0]]]]),self.functionspace)
      arg[:3,2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 8.0], [7.0, 5.0, 4.0], [7.0, 8.0, 4.0]], [[2.0, 3.0, 5.0], [7.0, 7.0, 8.0], [7.0, 
4.0, 2.0]], [[5.0, 3.0, 6.0], [4.0, 7.0, 5.0], [7.0, 5.0, 1.0]]], [[[1.0, 1.0, 6.0], [5.0, 3.0, 5.0], [6.0, 2.0, 8.0]], [[8.0, 
4.0, 8.0], [3.0, 1.0, 5.0], [2.0, 3.0, 4.0]], [[7.0, 8.0, 8.0], [8.0, 2.0, 3.0], [8.0, 1.0, 7.0]]], [[[3.0, 5.0, 1.0], [3.0, 
4.0, 7.0], [7.0, 1.0, 4.0]], [[1.0, 8.0, 7.0], [2.0, 2.0, 1.0], [1.0, 1.0, 8.0]], [[7.0, 8.0, 8.0], [8.0, 7.0, 8.0], [2.0, 2.0, 
7.0]]], [[[8.0, 7.0, 1.0], [4.0, 1.0, 7.0], [8.0, 5.0, 1.0]], [[2.0, 1.0, 2.0], [1.0, 8.0, 1.0], [2.0, 3.0, 2.0]], [[7.0, 1.0, 
1.0], [8.0, 2.0, 2.0], [3.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 7.0], [4.0, 1.0, 8.0], [8.0, 1.0, 6.0]], 
[[2.0, 4.0, 4.0], [5.0, 6.0, 8.0], [5.0, 2.0, 8.0]], [[3.0, 6.0, 4.0], [8.0, 8.0, 5.0], [4.0, 1.0, 6.0]]], [[[5.0, 1.0, 6.0], 
[3.0, 2.0, 8.0], [1.0, 3.0, 8.0]], [[3.0, 6.0, 7.0], [7.0, 2.0, 4.0], [3.0, 6.0, 1.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 3.0], 
[4.0, 7.0, 3.0]]], [[[3.0, 4.0, 4.0], [7.0, 6.0, 2.0], [2.0, 2.0, 1.0]], [[3.0, 6.0, 8.0], [1.0, 3.0, 1.0], [3.0, 5.0, 6.0]], 
[[2.0, 1.0, 5.0], [1.0, 3.0, 8.0], [8.0, 6.0, 8.0]]], [[[4.0, 4.0, 6.0], [1.0, 3.0, 5.0], [1.0, 8.0, 6.0]], [[7.0, 7.0, 1.0], 
[6.0, 2.0, 3.0], [2.0, 1.0, 3.0]], [[5.0, 3.0, 7.0], [5.0, 5.0, 7.0], [1.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULRL(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 6.0], [4.0, 5.0, 8.0], [2.0, 4.0, 6.0]], [[8.0, 6.0, 6.0], [4.0, 1.0, 5.0], [7.0, 1.0, 
2.0]], [[5.0, 2.0, 4.0], [8.0, 4.0, 4.0], [6.0, 7.0, 1.0]]], [[[3.0, 4.0, 4.0], [2.0, 8.0, 4.0], [5.0, 6.0, 3.0]], [[5.0, 4.0, 
5.0], [1.0, 2.0, 2.0], [1.0, 2.0, 8.0]], [[3.0, 7.0, 6.0], [1.0, 3.0, 3.0], [6.0, 5.0, 2.0]]], [[[4.0, 1.0, 8.0], [6.0, 8.0, 
2.0], [7.0, 6.0, 6.0]], [[3.0, 6.0, 7.0], [6.0, 8.0, 2.0], [1.0, 3.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 5.0, 5.0], [2.0, 8.0, 
8.0]]], [[[2.0, 7.0, 2.0], [2.0, 3.0, 1.0], [3.0, 7.0, 5.0]], [[6.0, 3.0, 4.0], [8.0, 6.0, 1.0], [4.0, 6.0, 1.0]], [[3.0, 3.0, 
5.0], [3.0, 5.0, 4.0], [6.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 5.0], [8.0, 7.0, 3.0], [6.0, 6.0, 5.0]], [[2.0, 3.0, 1.0], [2.0, 1.0, 
3.0], [1.0, 3.0, 5.0]], [[6.0, 8.0, 5.0], [6.0, 2.0, 5.0], [1.0, 2.0, 8.0]]], [[[6.0, 1.0, 5.0], [1.0, 4.0, 2.0], [1.0, 2.0, 
7.0]], [[4.0, 5.0, 6.0], [5.0, 6.0, 2.0], [8.0, 4.0, 6.0]], [[7.0, 5.0, 7.0], [6.0, 6.0, 2.0], [4.0, 5.0, 3.0]]], [[[5.0, 6.0, 
3.0], [3.0, 2.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 2.0, 3.0], [1.0, 8.0, 4.0], [7.0, 5.0, 5.0]], [[3.0, 6.0, 7.0], [2.0, 5.0, 
5.0], [3.0, 5.0, 8.0]]], [[[3.0, 5.0, 8.0], [4.0, 6.0, 5.0], [2.0, 5.0, 8.0]], [[6.0, 7.0, 1.0], [7.0, 2.0, 4.0], [3.0, 4.0, 
3.0]], [[4.0, 5.0, 2.0], [2.0, 8.0, 7.0], [8.0, 1.0, 1.0]]]]))
      val=Data(numpy.array([[[[8.0]]], [[[5.0]]], [[[5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0]]], [[[4.0]]], [[[6.0]]]]))
      arg[:3,2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 6.0], [4.0, 5.0, 8.0], [2.0, 4.0, 6.0]], [[8.0, 6.0, 6.0], [4.0, 1.0, 5.0], [7.0, 
1.0, 2.0]], [[5.0, 2.0, 4.0], [8.0, 4.0, 8.0], [6.0, 7.0, 1.0]]], [[[3.0, 4.0, 4.0], [2.0, 8.0, 4.0], [5.0, 6.0, 3.0]], [[5.0, 
4.0, 5.0], [1.0, 2.0, 2.0], [1.0, 2.0, 8.0]], [[3.0, 7.0, 6.0], [1.0, 3.0, 5.0], [6.0, 5.0, 2.0]]], [[[4.0, 1.0, 8.0], [6.0, 
8.0, 2.0], [7.0, 6.0, 6.0]], [[3.0, 6.0, 7.0], [6.0, 8.0, 2.0], [1.0, 3.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 5.0, 5.0], [2.0, 8.0, 
8.0]]], [[[2.0, 7.0, 2.0], [2.0, 3.0, 1.0], [3.0, 7.0, 5.0]], [[6.0, 3.0, 4.0], [8.0, 6.0, 1.0], [4.0, 6.0, 1.0]], [[3.0, 3.0, 
5.0], [3.0, 5.0, 4.0], [6.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 5.0], [8.0, 7.0, 3.0], [6.0, 6.0, 5.0]], 
[[2.0, 3.0, 1.0], [2.0, 1.0, 3.0], [1.0, 3.0, 5.0]], [[6.0, 8.0, 5.0], [6.0, 2.0, 4.0], [1.0, 2.0, 8.0]]], [[[6.0, 1.0, 5.0], 
[1.0, 4.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 5.0, 6.0], [5.0, 6.0, 2.0], [8.0, 4.0, 6.0]], [[7.0, 5.0, 7.0], [6.0, 6.0, 4.0], 
[4.0, 5.0, 3.0]]], [[[5.0, 6.0, 3.0], [3.0, 2.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 2.0, 3.0], [1.0, 8.0, 4.0], [7.0, 5.0, 5.0]], 
[[3.0, 6.0, 7.0], [2.0, 5.0, 6.0], [3.0, 5.0, 8.0]]], [[[3.0, 5.0, 8.0], [4.0, 6.0, 5.0], [2.0, 5.0, 8.0]], [[6.0, 7.0, 1.0], 
[7.0, 2.0, 4.0], [3.0, 4.0, 3.0]], [[4.0, 5.0, 2.0], [2.0, 8.0, 7.0], [8.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULRL(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 8.0], [7.0, 5.0, 7.0], [8.0, 6.0, 3.0]], [[8.0, 6.0, 4.0], [7.0, 1.0, 3.0], [5.0, 8.0, 
3.0]], [[5.0, 5.0, 2.0], [7.0, 7.0, 1.0], [4.0, 2.0, 1.0]]], [[[5.0, 6.0, 3.0], [3.0, 6.0, 7.0], [5.0, 8.0, 6.0]], [[6.0, 7.0, 
8.0], [7.0, 2.0, 6.0], [4.0, 2.0, 6.0]], [[4.0, 5.0, 8.0], [4.0, 5.0, 5.0], [3.0, 5.0, 1.0]]], [[[1.0, 8.0, 8.0], [4.0, 5.0, 
7.0], [6.0, 3.0, 6.0]], [[2.0, 7.0, 7.0], [7.0, 5.0, 2.0], [1.0, 4.0, 2.0]], [[2.0, 2.0, 8.0], [5.0, 8.0, 3.0], [6.0, 6.0, 
4.0]]], [[[6.0, 6.0, 2.0], [3.0, 4.0, 7.0], [8.0, 2.0, 1.0]], [[6.0, 7.0, 4.0], [1.0, 6.0, 7.0], [5.0, 7.0, 5.0]], [[3.0, 3.0, 
6.0], [5.0, 8.0, 7.0], [4.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 6.0], [2.0, 7.0, 1.0], [6.0, 2.0, 7.0]], [[8.0, 3.0, 2.0], [7.0, 4.0, 
6.0], [5.0, 5.0, 2.0]], [[6.0, 2.0, 6.0], [6.0, 5.0, 8.0], [5.0, 4.0, 8.0]]], [[[5.0, 8.0, 3.0], [2.0, 6.0, 8.0], [5.0, 4.0, 
6.0]], [[5.0, 4.0, 6.0], [7.0, 5.0, 7.0], [6.0, 3.0, 3.0]], [[6.0, 7.0, 5.0], [5.0, 6.0, 6.0], [4.0, 3.0, 8.0]]], [[[7.0, 2.0, 
2.0], [6.0, 6.0, 1.0], [5.0, 7.0, 6.0]], [[3.0, 8.0, 3.0], [8.0, 3.0, 1.0], [2.0, 1.0, 1.0]], [[1.0, 7.0, 3.0], [7.0, 7.0, 
6.0], [3.0, 2.0, 5.0]]], [[[6.0, 5.0, 1.0], [7.0, 6.0, 5.0], [5.0, 3.0, 4.0]], [[5.0, 4.0, 4.0], [4.0, 6.0, 7.0], [1.0, 3.0, 
2.0]], [[2.0, 3.0, 5.0], [1.0, 2.0, 6.0], [7.0, 3.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0]]], [[[3.0]]], [[[3.0]]]])+(1.-msk_val)*numpy.array([[[[7.0]]], [[[3.0]]], [[[4.0]]]])
      arg[:3,2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 8.0], [7.0, 5.0, 7.0], [8.0, 6.0, 3.0]], [[8.0, 6.0, 4.0], [7.0, 1.0, 3.0], [5.0, 
8.0, 3.0]], [[5.0, 5.0, 2.0], [7.0, 7.0, 8.0], [4.0, 2.0, 1.0]]], [[[5.0, 6.0, 3.0], [3.0, 6.0, 7.0], [5.0, 8.0, 6.0]], [[6.0, 
7.0, 8.0], [7.0, 2.0, 6.0], [4.0, 2.0, 6.0]], [[4.0, 5.0, 8.0], [4.0, 5.0, 3.0], [3.0, 5.0, 1.0]]], [[[1.0, 8.0, 8.0], [4.0, 
5.0, 7.0], [6.0, 3.0, 6.0]], [[2.0, 7.0, 7.0], [7.0, 5.0, 2.0], [1.0, 4.0, 2.0]], [[2.0, 2.0, 8.0], [5.0, 8.0, 3.0], [6.0, 6.0, 
4.0]]], [[[6.0, 6.0, 2.0], [3.0, 4.0, 7.0], [8.0, 2.0, 1.0]], [[6.0, 7.0, 4.0], [1.0, 6.0, 7.0], [5.0, 7.0, 5.0]], [[3.0, 3.0, 
6.0], [5.0, 8.0, 7.0], [4.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 6.0], [2.0, 7.0, 1.0], [6.0, 2.0, 7.0]], 
[[8.0, 3.0, 2.0], [7.0, 4.0, 6.0], [5.0, 5.0, 2.0]], [[6.0, 2.0, 6.0], [6.0, 5.0, 7.0], [5.0, 4.0, 8.0]]], [[[5.0, 8.0, 3.0], 
[2.0, 6.0, 8.0], [5.0, 4.0, 6.0]], [[5.0, 4.0, 6.0], [7.0, 5.0, 7.0], [6.0, 3.0, 3.0]], [[6.0, 7.0, 5.0], [5.0, 6.0, 3.0], 
[4.0, 3.0, 8.0]]], [[[7.0, 2.0, 2.0], [6.0, 6.0, 1.0], [5.0, 7.0, 6.0]], [[3.0, 8.0, 3.0], [8.0, 3.0, 1.0], [2.0, 1.0, 1.0]], 
[[1.0, 7.0, 3.0], [7.0, 7.0, 4.0], [3.0, 2.0, 5.0]]], [[[6.0, 5.0, 1.0], [7.0, 6.0, 5.0], [5.0, 3.0, 4.0]], [[5.0, 4.0, 4.0], 
[4.0, 6.0, 7.0], [1.0, 3.0, 2.0]], [[2.0, 3.0, 5.0], [1.0, 2.0, 6.0], [7.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCLC(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 4.0], [8.0, 4.0, 4.0], [8.0, 5.0, 2.0]], [[6.0, 7.0, 3.0], [6.0, 8.0, 2.0], [3.0, 2.0, 
1.0]], [[4.0, 8.0, 6.0], [3.0, 8.0, 1.0], [2.0, 7.0, 5.0]]], [[[8.0, 1.0, 6.0], [8.0, 5.0, 5.0], [6.0, 1.0, 8.0]], [[7.0, 8.0, 
4.0], [6.0, 3.0, 8.0], [3.0, 1.0, 1.0]], [[1.0, 4.0, 1.0], [6.0, 7.0, 5.0], [7.0, 5.0, 7.0]]], [[[1.0, 3.0, 6.0], [7.0, 6.0, 
3.0], [4.0, 4.0, 3.0]], [[4.0, 5.0, 6.0], [4.0, 6.0, 6.0], [8.0, 3.0, 5.0]], [[5.0, 1.0, 2.0], [2.0, 3.0, 7.0], [2.0, 7.0, 
7.0]]], [[[7.0, 7.0, 4.0], [7.0, 7.0, 3.0], [5.0, 5.0, 4.0]], [[7.0, 3.0, 3.0], [1.0, 8.0, 5.0], [2.0, 3.0, 3.0]], [[2.0, 3.0, 
1.0], [2.0, 4.0, 7.0], [1.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 7.0], [7.0, 3.0, 2.0], [8.0, 7.0, 2.0]], [[4.0, 4.0, 4.0], [8.0, 1.0, 
8.0], [3.0, 3.0, 7.0]], [[8.0, 7.0, 3.0], [5.0, 6.0, 2.0], [3.0, 3.0, 3.0]]], [[[8.0, 8.0, 1.0], [5.0, 6.0, 7.0], [8.0, 7.0, 
3.0]], [[6.0, 4.0, 7.0], [8.0, 4.0, 8.0], [4.0, 5.0, 7.0]], [[7.0, 6.0, 1.0], [2.0, 2.0, 7.0], [6.0, 3.0, 6.0]]], [[[3.0, 1.0, 
3.0], [5.0, 6.0, 3.0], [7.0, 6.0, 6.0]], [[6.0, 1.0, 7.0], [3.0, 1.0, 5.0], [7.0, 5.0, 5.0]], [[7.0, 7.0, 2.0], [5.0, 8.0, 
5.0], [5.0, 5.0, 8.0]]], [[[8.0, 6.0, 7.0], [7.0, 7.0, 2.0], [5.0, 3.0, 4.0]], [[5.0, 8.0, 3.0], [3.0, 6.0, 3.0], [2.0, 3.0, 
4.0]], [[8.0, 1.0, 8.0], [3.0, 1.0, 3.0], [4.0, 1.0, 6.0]]]]))
      val=numpy.array([[[[4.0, 1.0, 2.0]], [[2.0, 4.0, 8.0]], [[7.0, 1.0, 4.0]]], [[[8.0, 3.0, 8.0]], [[2.0, 7.0, 8.0]], [[4.0, 
8.0, 8.0]]], [[[3.0, 7.0, 4.0]], [[6.0, 6.0, 5.0]], [[8.0, 8.0, 8.0]]], [[[1.0, 6.0, 3.0]], [[1.0, 1.0, 2.0]], [[2.0, 7.0, 
6.0]]]])
      arg[:,:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 4.0], [8.0, 4.0, 4.0], [4.0, 1.0, 2.0]], [[6.0, 7.0, 3.0], [6.0, 8.0, 2.0], [2.0, 
4.0, 8.0]], [[4.0, 8.0, 6.0], [3.0, 8.0, 1.0], [7.0, 1.0, 4.0]]], [[[8.0, 1.0, 6.0], [8.0, 5.0, 5.0], [8.0, 3.0, 8.0]], [[7.0, 
8.0, 4.0], [6.0, 3.0, 8.0], [2.0, 7.0, 8.0]], [[1.0, 4.0, 1.0], [6.0, 7.0, 5.0], [4.0, 8.0, 8.0]]], [[[1.0, 3.0, 6.0], [7.0, 
6.0, 3.0], [3.0, 7.0, 4.0]], [[4.0, 5.0, 6.0], [4.0, 6.0, 6.0], [6.0, 6.0, 5.0]], [[5.0, 1.0, 2.0], [2.0, 3.0, 7.0], [8.0, 8.0, 
8.0]]], [[[7.0, 7.0, 4.0], [7.0, 7.0, 3.0], [1.0, 6.0, 3.0]], [[7.0, 3.0, 3.0], [1.0, 8.0, 5.0], [1.0, 1.0, 2.0]], [[2.0, 3.0, 
1.0], [2.0, 4.0, 7.0], [2.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 7.0], [7.0, 3.0, 2.0], [4.0, 1.0, 2.0]], 
[[4.0, 4.0, 4.0], [8.0, 1.0, 8.0], [2.0, 4.0, 8.0]], [[8.0, 7.0, 3.0], [5.0, 6.0, 2.0], [7.0, 1.0, 4.0]]], [[[8.0, 8.0, 1.0], 
[5.0, 6.0, 7.0], [8.0, 3.0, 8.0]], [[6.0, 4.0, 7.0], [8.0, 4.0, 8.0], [2.0, 7.0, 8.0]], [[7.0, 6.0, 1.0], [2.0, 2.0, 7.0], 
[4.0, 8.0, 8.0]]], [[[3.0, 1.0, 3.0], [5.0, 6.0, 3.0], [3.0, 7.0, 4.0]], [[6.0, 1.0, 7.0], [3.0, 1.0, 5.0], [6.0, 6.0, 5.0]], 
[[7.0, 7.0, 2.0], [5.0, 8.0, 5.0], [8.0, 8.0, 8.0]]], [[[8.0, 6.0, 7.0], [7.0, 7.0, 2.0], [1.0, 6.0, 3.0]], [[5.0, 8.0, 3.0], 
[3.0, 6.0, 3.0], [1.0, 1.0, 2.0]], [[8.0, 1.0, 8.0], [3.0, 1.0, 3.0], [2.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCLC(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 2.0], [8.0, 2.0, 3.0], [8.0, 7.0, 8.0]], [[5.0, 3.0, 8.0], [3.0, 7.0, 3.0], [6.0, 2.0, 
3.0]], [[8.0, 6.0, 7.0], [2.0, 2.0, 6.0], [6.0, 8.0, 6.0]]], [[[7.0, 6.0, 6.0], [4.0, 6.0, 3.0], [6.0, 5.0, 2.0]], [[7.0, 1.0, 
4.0], [2.0, 8.0, 8.0], [1.0, 4.0, 3.0]], [[5.0, 3.0, 2.0], [2.0, 8.0, 7.0], [6.0, 6.0, 6.0]]], [[[1.0, 3.0, 7.0], [8.0, 8.0, 
6.0], [6.0, 7.0, 1.0]], [[2.0, 2.0, 4.0], [7.0, 1.0, 6.0], [8.0, 6.0, 8.0]], [[3.0, 1.0, 2.0], [1.0, 8.0, 7.0], [5.0, 2.0, 
6.0]]], [[[1.0, 6.0, 8.0], [2.0, 5.0, 6.0], [3.0, 3.0, 6.0]], [[3.0, 3.0, 4.0], [4.0, 5.0, 5.0], [7.0, 6.0, 1.0]], [[4.0, 1.0, 
5.0], [3.0, 8.0, 2.0], [8.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 4.0], [8.0, 7.0, 6.0], [3.0, 8.0, 1.0]], [[2.0, 1.0, 5.0], [1.0, 8.0, 
2.0], [8.0, 5.0, 1.0]], [[3.0, 4.0, 7.0], [6.0, 1.0, 1.0], [4.0, 2.0, 5.0]]], [[[2.0, 8.0, 7.0], [8.0, 8.0, 6.0], [8.0, 3.0, 
7.0]], [[8.0, 3.0, 8.0], [1.0, 2.0, 1.0], [8.0, 8.0, 1.0]], [[4.0, 5.0, 7.0], [5.0, 7.0, 7.0], [5.0, 5.0, 5.0]]], [[[3.0, 8.0, 
8.0], [1.0, 5.0, 6.0], [3.0, 8.0, 1.0]], [[4.0, 3.0, 4.0], [5.0, 6.0, 8.0], [6.0, 6.0, 8.0]], [[5.0, 1.0, 6.0], [4.0, 7.0, 
4.0], [3.0, 6.0, 5.0]]], [[[8.0, 2.0, 6.0], [4.0, 6.0, 6.0], [6.0, 4.0, 3.0]], [[1.0, 6.0, 8.0], [8.0, 3.0, 6.0], [7.0, 7.0, 
4.0]], [[6.0, 5.0, 7.0], [8.0, 3.0, 7.0], [8.0, 3.0, 8.0]]]]))
      val=Data(numpy.array([[[[7.0, 8.0, 7.0]], [[6.0, 2.0, 5.0]], [[1.0, 4.0, 1.0]]], [[[4.0, 3.0, 3.0]], [[5.0, 6.0, 8.0]], 
[[1.0, 1.0, 3.0]]], [[[5.0, 3.0, 5.0]], [[2.0, 5.0, 1.0]], [[3.0, 1.0, 4.0]]], [[[7.0, 8.0, 7.0]], [[6.0, 4.0, 6.0]], [[5.0, 
1.0, 1.0]]]]),self.functionspace)
      arg[:,:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 2.0], [8.0, 2.0, 3.0], [7.0, 8.0, 7.0]], [[5.0, 3.0, 8.0], [3.0, 7.0, 3.0], [6.0, 
2.0, 5.0]], [[8.0, 6.0, 7.0], [2.0, 2.0, 6.0], [1.0, 4.0, 1.0]]], [[[7.0, 6.0, 6.0], [4.0, 6.0, 3.0], [4.0, 3.0, 3.0]], [[7.0, 
1.0, 4.0], [2.0, 8.0, 8.0], [5.0, 6.0, 8.0]], [[5.0, 3.0, 2.0], [2.0, 8.0, 7.0], [1.0, 1.0, 3.0]]], [[[1.0, 3.0, 7.0], [8.0, 
8.0, 6.0], [5.0, 3.0, 5.0]], [[2.0, 2.0, 4.0], [7.0, 1.0, 6.0], [2.0, 5.0, 1.0]], [[3.0, 1.0, 2.0], [1.0, 8.0, 7.0], [3.0, 1.0, 
4.0]]], [[[1.0, 6.0, 8.0], [2.0, 5.0, 6.0], [7.0, 8.0, 7.0]], [[3.0, 3.0, 4.0], [4.0, 5.0, 5.0], [6.0, 4.0, 6.0]], [[4.0, 1.0, 
5.0], [3.0, 8.0, 2.0], [5.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 4.0], [8.0, 7.0, 6.0], [7.0, 8.0, 7.0]], 
[[2.0, 1.0, 5.0], [1.0, 8.0, 2.0], [6.0, 2.0, 5.0]], [[3.0, 4.0, 7.0], [6.0, 1.0, 1.0], [1.0, 4.0, 1.0]]], [[[2.0, 8.0, 7.0], 
[8.0, 8.0, 6.0], [4.0, 3.0, 3.0]], [[8.0, 3.0, 8.0], [1.0, 2.0, 1.0], [5.0, 6.0, 8.0]], [[4.0, 5.0, 7.0], [5.0, 7.0, 7.0], 
[1.0, 1.0, 3.0]]], [[[3.0, 8.0, 8.0], [1.0, 5.0, 6.0], [5.0, 3.0, 5.0]], [[4.0, 3.0, 4.0], [5.0, 6.0, 8.0], [2.0, 5.0, 1.0]], 
[[5.0, 1.0, 6.0], [4.0, 7.0, 4.0], [3.0, 1.0, 4.0]]], [[[8.0, 2.0, 6.0], [4.0, 6.0, 6.0], [7.0, 8.0, 7.0]], [[1.0, 6.0, 8.0], 
[8.0, 3.0, 6.0], [6.0, 4.0, 6.0]], [[6.0, 5.0, 7.0], [8.0, 3.0, 7.0], [5.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCLC(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 3.0], [4.0, 3.0, 4.0], [3.0, 5.0, 5.0]], [[2.0, 5.0, 3.0], [2.0, 4.0, 4.0], [7.0, 6.0, 
1.0]], [[2.0, 4.0, 8.0], [5.0, 4.0, 1.0], [6.0, 6.0, 1.0]]], [[[7.0, 5.0, 6.0], [4.0, 6.0, 6.0], [3.0, 6.0, 1.0]], [[7.0, 4.0, 
8.0], [6.0, 4.0, 5.0], [6.0, 6.0, 2.0]], [[6.0, 1.0, 2.0], [2.0, 6.0, 3.0], [5.0, 2.0, 1.0]]], [[[5.0, 4.0, 4.0], [7.0, 1.0, 
8.0], [5.0, 4.0, 1.0]], [[1.0, 5.0, 1.0], [5.0, 7.0, 7.0], [3.0, 1.0, 5.0]], [[5.0, 1.0, 2.0], [7.0, 4.0, 5.0], [8.0, 1.0, 
5.0]]], [[[6.0, 6.0, 4.0], [5.0, 3.0, 2.0], [7.0, 5.0, 4.0]], [[2.0, 4.0, 5.0], [8.0, 8.0, 6.0], [6.0, 2.0, 5.0]], [[4.0, 5.0, 
6.0], [6.0, 2.0, 3.0], [3.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 3.0], [8.0, 8.0, 1.0], [7.0, 8.0, 3.0]], [[3.0, 8.0, 5.0], [8.0, 4.0, 
4.0], [2.0, 1.0, 6.0]], [[7.0, 4.0, 4.0], [3.0, 2.0, 6.0], [1.0, 5.0, 7.0]]], [[[2.0, 3.0, 3.0], [7.0, 2.0, 1.0], [1.0, 3.0, 
5.0]], [[3.0, 3.0, 4.0], [1.0, 1.0, 8.0], [8.0, 4.0, 5.0]], [[4.0, 4.0, 5.0], [2.0, 5.0, 8.0], [6.0, 4.0, 5.0]]], [[[5.0, 3.0, 
8.0], [6.0, 4.0, 1.0], [1.0, 1.0, 3.0]], [[4.0, 7.0, 3.0], [1.0, 7.0, 7.0], [2.0, 4.0, 3.0]], [[5.0, 7.0, 6.0], [3.0, 2.0, 
2.0], [7.0, 4.0, 7.0]]], [[[3.0, 4.0, 3.0], [2.0, 8.0, 6.0], [6.0, 6.0, 4.0]], [[8.0, 6.0, 1.0], [1.0, 4.0, 8.0], [6.0, 2.0, 
6.0]], [[1.0, 7.0, 2.0], [8.0, 1.0, 5.0], [4.0, 1.0, 5.0]]]]))
      val=Data(numpy.array([[[[7.0, 6.0, 6.0]], [[3.0, 7.0, 5.0]], [[7.0, 2.0, 1.0]]], [[[1.0, 5.0, 7.0]], [[1.0, 3.0, 4.0]], 
[[2.0, 4.0, 7.0]]], [[[6.0, 4.0, 1.0]], [[7.0, 7.0, 5.0]], [[4.0, 1.0, 2.0]]], [[[7.0, 4.0, 3.0]], [[1.0, 5.0, 6.0]], [[2.0, 
1.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 6.0]], [[1.0, 2.0, 5.0]], [[1.0, 3.0, 7.0]]], [[[6.0, 7.0, 5.0]], [[8.0, 
6.0, 5.0]], [[3.0, 7.0, 6.0]]], [[[2.0, 3.0, 4.0]], [[3.0, 1.0, 4.0]], [[7.0, 5.0, 2.0]]], [[[2.0, 6.0, 7.0]], [[7.0, 7.0, 
6.0]], [[1.0, 8.0, 4.0]]]]))
      arg[:,:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 3.0], [4.0, 3.0, 4.0], [7.0, 6.0, 6.0]], [[2.0, 5.0, 3.0], [2.0, 4.0, 4.0], [3.0, 
7.0, 5.0]], [[2.0, 4.0, 8.0], [5.0, 4.0, 1.0], [7.0, 2.0, 1.0]]], [[[7.0, 5.0, 6.0], [4.0, 6.0, 6.0], [1.0, 5.0, 7.0]], [[7.0, 
4.0, 8.0], [6.0, 4.0, 5.0], [1.0, 3.0, 4.0]], [[6.0, 1.0, 2.0], [2.0, 6.0, 3.0], [2.0, 4.0, 7.0]]], [[[5.0, 4.0, 4.0], [7.0, 
1.0, 8.0], [6.0, 4.0, 1.0]], [[1.0, 5.0, 1.0], [5.0, 7.0, 7.0], [7.0, 7.0, 5.0]], [[5.0, 1.0, 2.0], [7.0, 4.0, 5.0], [4.0, 1.0, 
2.0]]], [[[6.0, 6.0, 4.0], [5.0, 3.0, 2.0], [7.0, 4.0, 3.0]], [[2.0, 4.0, 5.0], [8.0, 8.0, 6.0], [1.0, 5.0, 6.0]], [[4.0, 5.0, 
6.0], [6.0, 2.0, 3.0], [2.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 3.0], [8.0, 8.0, 1.0], [3.0, 1.0, 6.0]], 
[[3.0, 8.0, 5.0], [8.0, 4.0, 4.0], [1.0, 2.0, 5.0]], [[7.0, 4.0, 4.0], [3.0, 2.0, 6.0], [1.0, 3.0, 7.0]]], [[[2.0, 3.0, 3.0], 
[7.0, 2.0, 1.0], [6.0, 7.0, 5.0]], [[3.0, 3.0, 4.0], [1.0, 1.0, 8.0], [8.0, 6.0, 5.0]], [[4.0, 4.0, 5.0], [2.0, 5.0, 8.0], 
[3.0, 7.0, 6.0]]], [[[5.0, 3.0, 8.0], [6.0, 4.0, 1.0], [2.0, 3.0, 4.0]], [[4.0, 7.0, 3.0], [1.0, 7.0, 7.0], [3.0, 1.0, 4.0]], 
[[5.0, 7.0, 6.0], [3.0, 2.0, 2.0], [7.0, 5.0, 2.0]]], [[[3.0, 4.0, 3.0], [2.0, 8.0, 6.0], [2.0, 6.0, 7.0]], [[8.0, 6.0, 1.0], 
[1.0, 4.0, 8.0], [7.0, 7.0, 6.0]], [[1.0, 7.0, 2.0], [8.0, 1.0, 5.0], [1.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCLC(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 1.0], [8.0, 2.0, 7.0], [5.0, 2.0, 5.0]], [[5.0, 3.0, 5.0], [4.0, 8.0, 1.0], [1.0, 2.0, 
6.0]], [[4.0, 8.0, 1.0], [4.0, 5.0, 3.0], [7.0, 6.0, 1.0]]], [[[2.0, 8.0, 1.0], [2.0, 2.0, 1.0], [7.0, 6.0, 6.0]], [[8.0, 8.0, 
5.0], [3.0, 4.0, 7.0], [7.0, 5.0, 3.0]], [[2.0, 5.0, 5.0], [7.0, 3.0, 5.0], [6.0, 1.0, 3.0]]], [[[1.0, 2.0, 6.0], [3.0, 1.0, 
3.0], [6.0, 2.0, 7.0]], [[2.0, 1.0, 5.0], [2.0, 4.0, 8.0], [6.0, 6.0, 1.0]], [[8.0, 5.0, 7.0], [2.0, 4.0, 3.0], [4.0, 8.0, 
5.0]]], [[[6.0, 2.0, 7.0], [5.0, 5.0, 8.0], [6.0, 8.0, 4.0]], [[4.0, 2.0, 6.0], [7.0, 1.0, 2.0], [8.0, 1.0, 1.0]], [[8.0, 7.0, 
5.0], [5.0, 2.0, 5.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 3.0], [5.0, 1.0, 4.0], [2.0, 3.0, 1.0]], [[4.0, 6.0, 5.0], [2.0, 7.0, 
1.0], [3.0, 3.0, 8.0]], [[3.0, 2.0, 4.0], [4.0, 3.0, 7.0], [1.0, 2.0, 4.0]]], [[[1.0, 8.0, 4.0], [1.0, 3.0, 2.0], [8.0, 1.0, 
4.0]], [[8.0, 1.0, 3.0], [3.0, 7.0, 1.0], [2.0, 8.0, 3.0]], [[8.0, 8.0, 1.0], [5.0, 8.0, 5.0], [4.0, 3.0, 4.0]]], [[[2.0, 7.0, 
6.0], [8.0, 2.0, 7.0], [1.0, 2.0, 2.0]], [[5.0, 6.0, 1.0], [2.0, 4.0, 1.0], [6.0, 6.0, 5.0]], [[8.0, 5.0, 2.0], [3.0, 7.0, 
8.0], [2.0, 2.0, 1.0]]], [[[2.0, 8.0, 2.0], [6.0, 2.0, 8.0], [6.0, 2.0, 7.0]], [[7.0, 4.0, 5.0], [2.0, 6.0, 8.0], [3.0, 6.0, 
8.0]], [[7.0, 6.0, 7.0], [1.0, 1.0, 1.0], [7.0, 8.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 5.0, 4.0]], [[4.0, 6.0, 1.0]], [[4.0, 3.0, 1.0]]], [[[6.0, 1.0, 5.0]], [[2.0, 2.0, 
8.0]], [[6.0, 8.0, 4.0]]], [[[4.0, 4.0, 8.0]], [[6.0, 8.0, 1.0]], [[7.0, 2.0, 8.0]]], [[[4.0, 8.0, 1.0]], [[5.0, 6.0, 8.0]], 
[[6.0, 3.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 6.0, 5.0]], [[5.0, 3.0, 5.0]], [[4.0, 6.0, 2.0]]], [[[1.0, 5.0, 4.0]], 
[[2.0, 7.0, 6.0]], [[4.0, 7.0, 2.0]]], [[[3.0, 4.0, 8.0]], [[2.0, 3.0, 4.0]], [[4.0, 6.0, 4.0]]], [[[4.0, 8.0, 6.0]], [[1.0, 
7.0, 4.0]], [[7.0, 5.0, 5.0]]]])
      arg[:,:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 1.0], [8.0, 2.0, 7.0], [1.0, 5.0, 4.0]], [[5.0, 3.0, 5.0], [4.0, 8.0, 1.0], [4.0, 
6.0, 1.0]], [[4.0, 8.0, 1.0], [4.0, 5.0, 3.0], [4.0, 3.0, 1.0]]], [[[2.0, 8.0, 1.0], [2.0, 2.0, 1.0], [6.0, 1.0, 5.0]], [[8.0, 
8.0, 5.0], [3.0, 4.0, 7.0], [2.0, 2.0, 8.0]], [[2.0, 5.0, 5.0], [7.0, 3.0, 5.0], [6.0, 8.0, 4.0]]], [[[1.0, 2.0, 6.0], [3.0, 
1.0, 3.0], [4.0, 4.0, 8.0]], [[2.0, 1.0, 5.0], [2.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[8.0, 5.0, 7.0], [2.0, 4.0, 3.0], [7.0, 2.0, 
8.0]]], [[[6.0, 2.0, 7.0], [5.0, 5.0, 8.0], [4.0, 8.0, 1.0]], [[4.0, 2.0, 6.0], [7.0, 1.0, 2.0], [5.0, 6.0, 8.0]], [[8.0, 7.0, 
5.0], [5.0, 2.0, 5.0], [6.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 3.0], [5.0, 1.0, 4.0], [1.0, 6.0, 5.0]], 
[[4.0, 6.0, 5.0], [2.0, 7.0, 1.0], [5.0, 3.0, 5.0]], [[3.0, 2.0, 4.0], [4.0, 3.0, 7.0], [4.0, 6.0, 2.0]]], [[[1.0, 8.0, 4.0], 
[1.0, 3.0, 2.0], [1.0, 5.0, 4.0]], [[8.0, 1.0, 3.0], [3.0, 7.0, 1.0], [2.0, 7.0, 6.0]], [[8.0, 8.0, 1.0], [5.0, 8.0, 5.0], 
[4.0, 7.0, 2.0]]], [[[2.0, 7.0, 6.0], [8.0, 2.0, 7.0], [3.0, 4.0, 8.0]], [[5.0, 6.0, 1.0], [2.0, 4.0, 1.0], [2.0, 3.0, 4.0]], 
[[8.0, 5.0, 2.0], [3.0, 7.0, 8.0], [4.0, 6.0, 4.0]]], [[[2.0, 8.0, 2.0], [6.0, 2.0, 8.0], [4.0, 8.0, 6.0]], [[7.0, 4.0, 5.0], 
[2.0, 6.0, 8.0], [1.0, 7.0, 4.0]], [[7.0, 6.0, 7.0], [1.0, 1.0, 1.0], [7.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULLU(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 8.0], [4.0, 1.0, 4.0], [6.0, 5.0, 8.0]], [[5.0, 6.0, 2.0], [3.0, 4.0, 4.0], [7.0, 2.0, 
7.0]], [[7.0, 3.0, 2.0], [2.0, 4.0, 2.0], [8.0, 4.0, 2.0]]], [[[7.0, 8.0, 3.0], [5.0, 3.0, 2.0], [6.0, 2.0, 5.0]], [[7.0, 4.0, 
4.0], [3.0, 3.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 1.0, 6.0], [3.0, 2.0, 2.0], [8.0, 5.0, 1.0]]], [[[8.0, 3.0, 2.0], [6.0, 3.0, 
4.0], [8.0, 5.0, 1.0]], [[8.0, 8.0, 1.0], [5.0, 6.0, 5.0], [6.0, 2.0, 6.0]], [[8.0, 2.0, 1.0], [7.0, 6.0, 3.0], [7.0, 5.0, 
1.0]]], [[[8.0, 4.0, 4.0], [4.0, 6.0, 1.0], [6.0, 2.0, 8.0]], [[2.0, 8.0, 5.0], [6.0, 3.0, 8.0], [5.0, 1.0, 8.0]], [[2.0, 4.0, 
5.0], [4.0, 7.0, 4.0], [4.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 5.0], [5.0, 3.0, 2.0], [2.0, 7.0, 3.0]], [[1.0, 3.0, 3.0], [5.0, 6.0, 
3.0], [1.0, 3.0, 6.0]], [[3.0, 1.0, 8.0], [7.0, 5.0, 7.0], [7.0, 5.0, 3.0]]], [[[1.0, 8.0, 3.0], [8.0, 4.0, 4.0], [1.0, 7.0, 
3.0]], [[3.0, 8.0, 6.0], [6.0, 5.0, 1.0], [6.0, 1.0, 3.0]], [[7.0, 3.0, 3.0], [7.0, 4.0, 4.0], [7.0, 4.0, 1.0]]], [[[6.0, 5.0, 
7.0], [5.0, 2.0, 5.0], [5.0, 7.0, 1.0]], [[5.0, 3.0, 8.0], [6.0, 7.0, 2.0], [6.0, 3.0, 3.0]], [[7.0, 1.0, 5.0], [4.0, 7.0, 
2.0], [5.0, 6.0, 5.0]]], [[[7.0, 6.0, 2.0], [3.0, 8.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 1.0, 1.0], [7.0, 3.0, 4.0], [7.0, 1.0, 
3.0]], [[3.0, 7.0, 5.0], [8.0, 7.0, 5.0], [7.0, 1.0, 4.0]]]]))
      val=numpy.array([[[[5.0, 7.0]]], [[[3.0, 3.0]]], [[[5.0, 8.0]]]])
      arg[:3,2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 8.0], [4.0, 1.0, 4.0], [6.0, 5.0, 8.0]], [[5.0, 6.0, 2.0], [3.0, 4.0, 4.0], [7.0, 
2.0, 7.0]], [[7.0, 3.0, 2.0], [2.0, 4.0, 2.0], [5.0, 7.0, 2.0]]], [[[7.0, 8.0, 3.0], [5.0, 3.0, 2.0], [6.0, 2.0, 5.0]], [[7.0, 
4.0, 4.0], [3.0, 3.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 1.0, 6.0], [3.0, 2.0, 2.0], [3.0, 3.0, 1.0]]], [[[8.0, 3.0, 2.0], [6.0, 
3.0, 4.0], [8.0, 5.0, 1.0]], [[8.0, 8.0, 1.0], [5.0, 6.0, 5.0], [6.0, 2.0, 6.0]], [[8.0, 2.0, 1.0], [7.0, 6.0, 3.0], [5.0, 8.0, 
1.0]]], [[[8.0, 4.0, 4.0], [4.0, 6.0, 1.0], [6.0, 2.0, 8.0]], [[2.0, 8.0, 5.0], [6.0, 3.0, 8.0], [5.0, 1.0, 8.0]], [[2.0, 4.0, 
5.0], [4.0, 7.0, 4.0], [4.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 5.0], [5.0, 3.0, 2.0], [2.0, 7.0, 3.0]], 
[[1.0, 3.0, 3.0], [5.0, 6.0, 3.0], [1.0, 3.0, 6.0]], [[3.0, 1.0, 8.0], [7.0, 5.0, 7.0], [5.0, 7.0, 3.0]]], [[[1.0, 8.0, 3.0], 
[8.0, 4.0, 4.0], [1.0, 7.0, 3.0]], [[3.0, 8.0, 6.0], [6.0, 5.0, 1.0], [6.0, 1.0, 3.0]], [[7.0, 3.0, 3.0], [7.0, 4.0, 4.0], 
[3.0, 3.0, 1.0]]], [[[6.0, 5.0, 7.0], [5.0, 2.0, 5.0], [5.0, 7.0, 1.0]], [[5.0, 3.0, 8.0], [6.0, 7.0, 2.0], [6.0, 3.0, 3.0]], 
[[7.0, 1.0, 5.0], [4.0, 7.0, 2.0], [5.0, 8.0, 5.0]]], [[[7.0, 6.0, 2.0], [3.0, 8.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 1.0, 1.0], 
[7.0, 3.0, 4.0], [7.0, 1.0, 3.0]], [[3.0, 7.0, 5.0], [8.0, 7.0, 5.0], [7.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULLU(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 2.0], [7.0, 2.0, 1.0], [7.0, 8.0, 4.0]], [[3.0, 1.0, 3.0], [6.0, 1.0, 8.0], [3.0, 4.0, 
1.0]], [[2.0, 3.0, 4.0], [7.0, 2.0, 1.0], [1.0, 5.0, 7.0]]], [[[7.0, 8.0, 6.0], [5.0, 2.0, 2.0], [2.0, 5.0, 5.0]], [[3.0, 3.0, 
3.0], [7.0, 5.0, 6.0], [3.0, 7.0, 2.0]], [[8.0, 5.0, 3.0], [7.0, 1.0, 2.0], [1.0, 8.0, 6.0]]], [[[4.0, 8.0, 1.0], [2.0, 3.0, 
7.0], [5.0, 8.0, 6.0]], [[1.0, 5.0, 5.0], [1.0, 7.0, 4.0], [6.0, 4.0, 7.0]], [[7.0, 3.0, 5.0], [3.0, 5.0, 6.0], [3.0, 8.0, 
3.0]]], [[[7.0, 7.0, 6.0], [4.0, 1.0, 7.0], [1.0, 3.0, 2.0]], [[3.0, 8.0, 1.0], [8.0, 5.0, 8.0], [8.0, 7.0, 3.0]], [[8.0, 1.0, 
4.0], [8.0, 2.0, 6.0], [5.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 6.0], [1.0, 7.0, 3.0], [1.0, 1.0, 6.0]], [[6.0, 5.0, 1.0], [3.0, 6.0, 
7.0], [3.0, 1.0, 6.0]], [[8.0, 1.0, 3.0], [1.0, 8.0, 3.0], [5.0, 1.0, 8.0]]], [[[7.0, 7.0, 7.0], [7.0, 4.0, 3.0], [3.0, 1.0, 
8.0]], [[6.0, 8.0, 6.0], [6.0, 8.0, 5.0], [3.0, 5.0, 1.0]], [[5.0, 1.0, 5.0], [6.0, 3.0, 1.0], [7.0, 2.0, 5.0]]], [[[4.0, 5.0, 
7.0], [4.0, 4.0, 4.0], [2.0, 2.0, 2.0]], [[1.0, 4.0, 4.0], [7.0, 3.0, 8.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 3.0], [8.0, 4.0, 
1.0], [3.0, 8.0, 8.0]]], [[[8.0, 1.0, 8.0], [6.0, 3.0, 7.0], [1.0, 1.0, 4.0]], [[3.0, 4.0, 2.0], [5.0, 4.0, 7.0], [3.0, 7.0, 
8.0]], [[3.0, 3.0, 1.0], [6.0, 2.0, 3.0], [7.0, 4.0, 5.0]]]]))
      val=Data(numpy.array([[[[6.0, 2.0]]], [[[7.0, 1.0]]], [[[5.0, 8.0]]]]),self.functionspace)
      arg[:3,2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 2.0], [7.0, 2.0, 1.0], [7.0, 8.0, 4.0]], [[3.0, 1.0, 3.0], [6.0, 1.0, 8.0], [3.0, 
4.0, 1.0]], [[2.0, 3.0, 4.0], [7.0, 2.0, 1.0], [6.0, 2.0, 7.0]]], [[[7.0, 8.0, 6.0], [5.0, 2.0, 2.0], [2.0, 5.0, 5.0]], [[3.0, 
3.0, 3.0], [7.0, 5.0, 6.0], [3.0, 7.0, 2.0]], [[8.0, 5.0, 3.0], [7.0, 1.0, 2.0], [7.0, 1.0, 6.0]]], [[[4.0, 8.0, 1.0], [2.0, 
3.0, 7.0], [5.0, 8.0, 6.0]], [[1.0, 5.0, 5.0], [1.0, 7.0, 4.0], [6.0, 4.0, 7.0]], [[7.0, 3.0, 5.0], [3.0, 5.0, 6.0], [5.0, 8.0, 
3.0]]], [[[7.0, 7.0, 6.0], [4.0, 1.0, 7.0], [1.0, 3.0, 2.0]], [[3.0, 8.0, 1.0], [8.0, 5.0, 8.0], [8.0, 7.0, 3.0]], [[8.0, 1.0, 
4.0], [8.0, 2.0, 6.0], [5.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 6.0], [1.0, 7.0, 3.0], [1.0, 1.0, 6.0]], 
[[6.0, 5.0, 1.0], [3.0, 6.0, 7.0], [3.0, 1.0, 6.0]], [[8.0, 1.0, 3.0], [1.0, 8.0, 3.0], [6.0, 2.0, 8.0]]], [[[7.0, 7.0, 7.0], 
[7.0, 4.0, 3.0], [3.0, 1.0, 8.0]], [[6.0, 8.0, 6.0], [6.0, 8.0, 5.0], [3.0, 5.0, 1.0]], [[5.0, 1.0, 5.0], [6.0, 3.0, 1.0], 
[7.0, 1.0, 5.0]]], [[[4.0, 5.0, 7.0], [4.0, 4.0, 4.0], [2.0, 2.0, 2.0]], [[1.0, 4.0, 4.0], [7.0, 3.0, 8.0], [5.0, 8.0, 8.0]], 
[[3.0, 3.0, 3.0], [8.0, 4.0, 1.0], [5.0, 8.0, 8.0]]], [[[8.0, 1.0, 8.0], [6.0, 3.0, 7.0], [1.0, 1.0, 4.0]], [[3.0, 4.0, 2.0], 
[5.0, 4.0, 7.0], [3.0, 7.0, 8.0]], [[3.0, 3.0, 1.0], [6.0, 2.0, 3.0], [7.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULLU(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 4.0], [6.0, 7.0, 5.0], [6.0, 1.0, 3.0]], [[6.0, 6.0, 8.0], [4.0, 6.0, 4.0], [8.0, 1.0, 
5.0]], [[2.0, 1.0, 7.0], [6.0, 8.0, 4.0], [8.0, 7.0, 1.0]]], [[[7.0, 2.0, 4.0], [7.0, 5.0, 4.0], [4.0, 6.0, 8.0]], [[2.0, 3.0, 
1.0], [7.0, 5.0, 4.0], [1.0, 1.0, 6.0]], [[8.0, 1.0, 3.0], [7.0, 7.0, 1.0], [7.0, 4.0, 6.0]]], [[[7.0, 6.0, 2.0], [4.0, 2.0, 
6.0], [8.0, 6.0, 7.0]], [[5.0, 8.0, 7.0], [4.0, 2.0, 5.0], [7.0, 2.0, 1.0]], [[4.0, 4.0, 4.0], [7.0, 2.0, 3.0], [7.0, 4.0, 
6.0]]], [[[6.0, 6.0, 7.0], [4.0, 8.0, 7.0], [3.0, 6.0, 6.0]], [[4.0, 1.0, 6.0], [8.0, 7.0, 4.0], [4.0, 7.0, 5.0]], [[4.0, 5.0, 
6.0], [3.0, 2.0, 2.0], [3.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 5.0], [6.0, 8.0, 4.0], [4.0, 3.0, 5.0]], [[1.0, 6.0, 7.0], [8.0, 5.0, 
5.0], [4.0, 7.0, 7.0]], [[4.0, 2.0, 8.0], [3.0, 7.0, 4.0], [5.0, 1.0, 6.0]]], [[[8.0, 2.0, 2.0], [6.0, 4.0, 3.0], [7.0, 2.0, 
8.0]], [[1.0, 7.0, 3.0], [8.0, 8.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 7.0, 6.0], [3.0, 2.0, 5.0], [8.0, 5.0, 1.0]]], [[[7.0, 8.0, 
1.0], [3.0, 6.0, 6.0], [1.0, 2.0, 2.0]], [[4.0, 1.0, 3.0], [7.0, 2.0, 7.0], [6.0, 1.0, 5.0]], [[7.0, 4.0, 7.0], [1.0, 3.0, 
1.0], [8.0, 2.0, 7.0]]], [[[7.0, 7.0, 3.0], [2.0, 5.0, 4.0], [2.0, 4.0, 3.0]], [[1.0, 5.0, 2.0], [2.0, 5.0, 5.0], [8.0, 5.0, 
2.0]], [[1.0, 1.0, 5.0], [2.0, 3.0, 7.0], [1.0, 7.0, 5.0]]]]))
      val=Data(numpy.array([[[[8.0, 6.0]]], [[[7.0, 3.0]]], [[[6.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 2.0]]], [[[6.0, 3.0]]], [[[4.0, 1.0]]]]))
      arg[:3,2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 4.0], [6.0, 7.0, 5.0], [6.0, 1.0, 3.0]], [[6.0, 6.0, 8.0], [4.0, 6.0, 4.0], [8.0, 
1.0, 5.0]], [[2.0, 1.0, 7.0], [6.0, 8.0, 4.0], [8.0, 6.0, 1.0]]], [[[7.0, 2.0, 4.0], [7.0, 5.0, 4.0], [4.0, 6.0, 8.0]], [[2.0, 
3.0, 1.0], [7.0, 5.0, 4.0], [1.0, 1.0, 6.0]], [[8.0, 1.0, 3.0], [7.0, 7.0, 1.0], [7.0, 3.0, 6.0]]], [[[7.0, 6.0, 2.0], [4.0, 
2.0, 6.0], [8.0, 6.0, 7.0]], [[5.0, 8.0, 7.0], [4.0, 2.0, 5.0], [7.0, 2.0, 1.0]], [[4.0, 4.0, 4.0], [7.0, 2.0, 3.0], [6.0, 3.0, 
6.0]]], [[[6.0, 6.0, 7.0], [4.0, 8.0, 7.0], [3.0, 6.0, 6.0]], [[4.0, 1.0, 6.0], [8.0, 7.0, 4.0], [4.0, 7.0, 5.0]], [[4.0, 5.0, 
6.0], [3.0, 2.0, 2.0], [3.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 5.0], [6.0, 8.0, 4.0], [4.0, 3.0, 5.0]], 
[[1.0, 6.0, 7.0], [8.0, 5.0, 5.0], [4.0, 7.0, 7.0]], [[4.0, 2.0, 8.0], [3.0, 7.0, 4.0], [3.0, 2.0, 6.0]]], [[[8.0, 2.0, 2.0], 
[6.0, 4.0, 3.0], [7.0, 2.0, 8.0]], [[1.0, 7.0, 3.0], [8.0, 8.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 7.0, 6.0], [3.0, 2.0, 5.0], 
[6.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], [3.0, 6.0, 6.0], [1.0, 2.0, 2.0]], [[4.0, 1.0, 3.0], [7.0, 2.0, 7.0], [6.0, 1.0, 5.0]], 
[[7.0, 4.0, 7.0], [1.0, 3.0, 1.0], [4.0, 1.0, 7.0]]], [[[7.0, 7.0, 3.0], [2.0, 5.0, 4.0], [2.0, 4.0, 3.0]], [[1.0, 5.0, 2.0], 
[2.0, 5.0, 5.0], [8.0, 5.0, 2.0]], [[1.0, 1.0, 5.0], [2.0, 3.0, 7.0], [1.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULLU(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 1.0], [6.0, 5.0, 4.0], [6.0, 5.0, 6.0]], [[6.0, 1.0, 5.0], [6.0, 1.0, 5.0], [7.0, 1.0, 
8.0]], [[2.0, 3.0, 2.0], [6.0, 2.0, 2.0], [1.0, 7.0, 8.0]]], [[[3.0, 1.0, 7.0], [5.0, 3.0, 6.0], [4.0, 6.0, 6.0]], [[2.0, 3.0, 
7.0], [5.0, 6.0, 3.0], [5.0, 6.0, 6.0]], [[7.0, 7.0, 1.0], [8.0, 2.0, 1.0], [6.0, 2.0, 6.0]]], [[[4.0, 7.0, 5.0], [5.0, 7.0, 
3.0], [7.0, 1.0, 3.0]], [[2.0, 2.0, 7.0], [2.0, 3.0, 8.0], [4.0, 5.0, 5.0]], [[3.0, 8.0, 2.0], [5.0, 5.0, 6.0], [7.0, 2.0, 
6.0]]], [[[2.0, 7.0, 8.0], [6.0, 4.0, 3.0], [4.0, 8.0, 3.0]], [[3.0, 1.0, 4.0], [3.0, 8.0, 3.0], [1.0, 7.0, 2.0]], [[3.0, 6.0, 
1.0], [4.0, 3.0, 1.0], [2.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 3.0], [3.0, 2.0, 8.0], [2.0, 4.0, 6.0]], [[6.0, 6.0, 1.0], [4.0, 4.0, 
7.0], [6.0, 5.0, 2.0]], [[7.0, 2.0, 8.0], [4.0, 7.0, 4.0], [4.0, 7.0, 1.0]]], [[[8.0, 8.0, 6.0], [7.0, 7.0, 5.0], [2.0, 4.0, 
5.0]], [[4.0, 4.0, 7.0], [5.0, 1.0, 8.0], [6.0, 8.0, 5.0]], [[4.0, 8.0, 6.0], [6.0, 5.0, 3.0], [7.0, 1.0, 1.0]]], [[[4.0, 5.0, 
8.0], [1.0, 2.0, 4.0], [8.0, 4.0, 2.0]], [[7.0, 2.0, 1.0], [5.0, 3.0, 3.0], [5.0, 1.0, 3.0]], [[1.0, 1.0, 3.0], [8.0, 8.0, 
2.0], [3.0, 1.0, 3.0]]], [[[5.0, 1.0, 5.0], [8.0, 5.0, 3.0], [2.0, 3.0, 5.0]], [[4.0, 5.0, 7.0], [1.0, 2.0, 1.0], [5.0, 6.0, 
8.0]], [[5.0, 2.0, 4.0], [6.0, 8.0, 1.0], [1.0, 1.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 3.0]]], [[[5.0, 4.0]]], [[[8.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 6.0]]], 
[[[3.0, 4.0]]], [[[5.0, 6.0]]]])
      arg[:3,2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 1.0], [6.0, 5.0, 4.0], [6.0, 5.0, 6.0]], [[6.0, 1.0, 5.0], [6.0, 1.0, 5.0], [7.0, 
1.0, 8.0]], [[2.0, 3.0, 2.0], [6.0, 2.0, 2.0], [3.0, 3.0, 8.0]]], [[[3.0, 1.0, 7.0], [5.0, 3.0, 6.0], [4.0, 6.0, 6.0]], [[2.0, 
3.0, 7.0], [5.0, 6.0, 3.0], [5.0, 6.0, 6.0]], [[7.0, 7.0, 1.0], [8.0, 2.0, 1.0], [5.0, 4.0, 6.0]]], [[[4.0, 7.0, 5.0], [5.0, 
7.0, 3.0], [7.0, 1.0, 3.0]], [[2.0, 2.0, 7.0], [2.0, 3.0, 8.0], [4.0, 5.0, 5.0]], [[3.0, 8.0, 2.0], [5.0, 5.0, 6.0], [8.0, 7.0, 
6.0]]], [[[2.0, 7.0, 8.0], [6.0, 4.0, 3.0], [4.0, 8.0, 3.0]], [[3.0, 1.0, 4.0], [3.0, 8.0, 3.0], [1.0, 7.0, 2.0]], [[3.0, 6.0, 
1.0], [4.0, 3.0, 1.0], [2.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 3.0], [3.0, 2.0, 8.0], [2.0, 4.0, 6.0]], 
[[6.0, 6.0, 1.0], [4.0, 4.0, 7.0], [6.0, 5.0, 2.0]], [[7.0, 2.0, 8.0], [4.0, 7.0, 4.0], [1.0, 6.0, 1.0]]], [[[8.0, 8.0, 6.0], 
[7.0, 7.0, 5.0], [2.0, 4.0, 5.0]], [[4.0, 4.0, 7.0], [5.0, 1.0, 8.0], [6.0, 8.0, 5.0]], [[4.0, 8.0, 6.0], [6.0, 5.0, 3.0], 
[3.0, 4.0, 1.0]]], [[[4.0, 5.0, 8.0], [1.0, 2.0, 4.0], [8.0, 4.0, 2.0]], [[7.0, 2.0, 1.0], [5.0, 3.0, 3.0], [5.0, 1.0, 3.0]], 
[[1.0, 1.0, 3.0], [8.0, 8.0, 2.0], [5.0, 6.0, 3.0]]], [[[5.0, 1.0, 5.0], [8.0, 5.0, 3.0], [2.0, 3.0, 5.0]], [[4.0, 5.0, 7.0], 
[1.0, 2.0, 1.0], [5.0, 6.0, 8.0]], [[5.0, 2.0, 4.0], [6.0, 8.0, 1.0], [1.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPPR(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 8.0], [7.0, 2.0, 7.0], [3.0, 8.0, 6.0]], [[6.0, 2.0, 3.0], [8.0, 2.0, 1.0], [1.0, 5.0, 
6.0]], [[3.0, 3.0, 2.0], [8.0, 1.0, 3.0], [7.0, 5.0, 6.0]]], [[[1.0, 7.0, 5.0], [8.0, 8.0, 2.0], [7.0, 8.0, 5.0]], [[8.0, 7.0, 
3.0], [7.0, 1.0, 4.0], [5.0, 5.0, 4.0]], [[1.0, 2.0, 6.0], [4.0, 2.0, 5.0], [1.0, 4.0, 8.0]]], [[[2.0, 2.0, 8.0], [1.0, 4.0, 
4.0], [4.0, 4.0, 7.0]], [[7.0, 1.0, 8.0], [6.0, 6.0, 8.0], [3.0, 1.0, 5.0]], [[4.0, 8.0, 4.0], [2.0, 2.0, 5.0], [4.0, 6.0, 
2.0]]], [[[8.0, 8.0, 6.0], [1.0, 8.0, 8.0], [4.0, 1.0, 5.0]], [[1.0, 5.0, 7.0], [7.0, 1.0, 6.0], [4.0, 4.0, 4.0]], [[1.0, 1.0, 
7.0], [7.0, 1.0, 7.0], [7.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 4.0], [8.0, 7.0, 8.0], [1.0, 5.0, 2.0]], [[1.0, 2.0, 6.0], [1.0, 5.0, 
3.0], [6.0, 7.0, 8.0]], [[6.0, 4.0, 8.0], [3.0, 3.0, 6.0], [4.0, 6.0, 6.0]]], [[[8.0, 2.0, 7.0], [3.0, 5.0, 5.0], [1.0, 6.0, 
2.0]], [[7.0, 4.0, 8.0], [3.0, 5.0, 3.0], [8.0, 3.0, 1.0]], [[2.0, 4.0, 6.0], [4.0, 1.0, 6.0], [5.0, 3.0, 6.0]]], [[[4.0, 3.0, 
7.0], [4.0, 7.0, 7.0], [7.0, 2.0, 6.0]], [[6.0, 2.0, 2.0], [8.0, 6.0, 3.0], [4.0, 3.0, 7.0]], [[4.0, 2.0, 3.0], [3.0, 8.0, 
7.0], [4.0, 1.0, 8.0]]], [[[8.0, 2.0, 7.0], [2.0, 6.0, 4.0], [1.0, 7.0, 7.0]], [[7.0, 5.0, 7.0], [3.0, 8.0, 6.0], [5.0, 5.0, 
5.0]], [[7.0, 7.0, 4.0], [5.0, 2.0, 4.0], [5.0, 8.0, 1.0]]]]))
      val=numpy.array([[1.0], [7.0]])
      arg[1:3,1,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 8.0], [7.0, 2.0, 7.0], [3.0, 8.0, 6.0]], [[6.0, 2.0, 3.0], [8.0, 2.0, 1.0], [1.0, 
5.0, 6.0]], [[3.0, 3.0, 2.0], [8.0, 1.0, 3.0], [7.0, 5.0, 6.0]]], [[[1.0, 7.0, 5.0], [8.0, 8.0, 2.0], [7.0, 8.0, 5.0]], [[8.0, 
7.0, 3.0], [7.0, 1.0, 4.0], [5.0, 5.0, 4.0]], [[1.0, 2.0, 6.0], [4.0, 2.0, 5.0], [1.0, 4.0, 8.0]]], [[[2.0, 2.0, 8.0], [1.0, 
4.0, 4.0], [4.0, 4.0, 7.0]], [[7.0, 1.0, 8.0], [6.0, 7.0, 8.0], [3.0, 1.0, 5.0]], [[4.0, 8.0, 4.0], [2.0, 2.0, 5.0], [4.0, 6.0, 
2.0]]], [[[8.0, 8.0, 6.0], [1.0, 8.0, 8.0], [4.0, 1.0, 5.0]], [[1.0, 5.0, 7.0], [7.0, 1.0, 6.0], [4.0, 4.0, 4.0]], [[1.0, 1.0, 
7.0], [7.0, 1.0, 7.0], [7.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 4.0], [8.0, 7.0, 8.0], [1.0, 5.0, 2.0]], 
[[1.0, 2.0, 6.0], [1.0, 5.0, 3.0], [6.0, 7.0, 8.0]], [[6.0, 4.0, 8.0], [3.0, 3.0, 6.0], [4.0, 6.0, 6.0]]], [[[8.0, 2.0, 7.0], 
[3.0, 5.0, 5.0], [1.0, 6.0, 2.0]], [[7.0, 4.0, 8.0], [3.0, 1.0, 3.0], [8.0, 3.0, 1.0]], [[2.0, 4.0, 6.0], [4.0, 1.0, 6.0], 
[5.0, 3.0, 6.0]]], [[[4.0, 3.0, 7.0], [4.0, 7.0, 7.0], [7.0, 2.0, 6.0]], [[6.0, 2.0, 2.0], [8.0, 7.0, 3.0], [4.0, 3.0, 7.0]], 
[[4.0, 2.0, 3.0], [3.0, 8.0, 7.0], [4.0, 1.0, 8.0]]], [[[8.0, 2.0, 7.0], [2.0, 6.0, 4.0], [1.0, 7.0, 7.0]], [[7.0, 5.0, 7.0], 
[3.0, 8.0, 6.0], [5.0, 5.0, 5.0]], [[7.0, 7.0, 4.0], [5.0, 2.0, 4.0], [5.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPPR(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 7.0], [4.0, 1.0, 6.0], [6.0, 4.0, 7.0]], [[5.0, 2.0, 2.0], [2.0, 3.0, 8.0], [1.0, 3.0, 
6.0]], [[5.0, 5.0, 7.0], [6.0, 2.0, 7.0], [7.0, 1.0, 4.0]]], [[[2.0, 5.0, 4.0], [4.0, 6.0, 3.0], [4.0, 3.0, 7.0]], [[5.0, 7.0, 
2.0], [5.0, 4.0, 5.0], [6.0, 2.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 2.0, 7.0], [6.0, 1.0, 7.0]]], [[[5.0, 3.0, 7.0], [2.0, 1.0, 
6.0], [8.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 8.0, 7.0], [6.0, 5.0, 7.0]], [[2.0, 2.0, 4.0], [3.0, 4.0, 7.0], [5.0, 3.0, 
3.0]]], [[[3.0, 4.0, 6.0], [6.0, 8.0, 3.0], [2.0, 2.0, 7.0]], [[5.0, 7.0, 5.0], [5.0, 6.0, 1.0], [6.0, 7.0, 7.0]], [[6.0, 1.0, 
6.0], [8.0, 2.0, 8.0], [6.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 4.0], [7.0, 4.0, 8.0], [7.0, 3.0, 7.0]], [[4.0, 8.0, 7.0], [1.0, 6.0, 
6.0], [4.0, 4.0, 3.0]], [[3.0, 1.0, 7.0], [3.0, 6.0, 1.0], [2.0, 3.0, 6.0]]], [[[8.0, 8.0, 5.0], [4.0, 3.0, 4.0], [2.0, 6.0, 
2.0]], [[2.0, 6.0, 4.0], [6.0, 1.0, 3.0], [3.0, 3.0, 3.0]], [[4.0, 8.0, 2.0], [7.0, 6.0, 8.0], [5.0, 7.0, 1.0]]], [[[8.0, 3.0, 
8.0], [2.0, 3.0, 8.0], [5.0, 1.0, 3.0]], [[7.0, 1.0, 3.0], [5.0, 4.0, 6.0], [6.0, 4.0, 3.0]], [[3.0, 7.0, 3.0], [7.0, 8.0, 
3.0], [6.0, 3.0, 2.0]]], [[[8.0, 5.0, 5.0], [7.0, 6.0, 4.0], [1.0, 8.0, 5.0]], [[4.0, 7.0, 4.0], [1.0, 2.0, 6.0], [5.0, 8.0, 
5.0]], [[2.0, 7.0, 6.0], [1.0, 5.0, 6.0], [2.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[4.0], [8.0]]),self.functionspace)
      arg[1:3,1,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 7.0], [4.0, 1.0, 6.0], [6.0, 4.0, 7.0]], [[5.0, 2.0, 2.0], [2.0, 3.0, 8.0], [1.0, 
3.0, 6.0]], [[5.0, 5.0, 7.0], [6.0, 2.0, 7.0], [7.0, 1.0, 4.0]]], [[[2.0, 5.0, 4.0], [4.0, 6.0, 3.0], [4.0, 3.0, 7.0]], [[5.0, 
7.0, 2.0], [5.0, 4.0, 5.0], [6.0, 2.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 2.0, 7.0], [6.0, 1.0, 7.0]]], [[[5.0, 3.0, 7.0], [2.0, 
1.0, 6.0], [8.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 8.0, 7.0], [6.0, 5.0, 7.0]], [[2.0, 2.0, 4.0], [3.0, 4.0, 7.0], [5.0, 3.0, 
3.0]]], [[[3.0, 4.0, 6.0], [6.0, 8.0, 3.0], [2.0, 2.0, 7.0]], [[5.0, 7.0, 5.0], [5.0, 6.0, 1.0], [6.0, 7.0, 7.0]], [[6.0, 1.0, 
6.0], [8.0, 2.0, 8.0], [6.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 4.0], [7.0, 4.0, 8.0], [7.0, 3.0, 7.0]], 
[[4.0, 8.0, 7.0], [1.0, 6.0, 6.0], [4.0, 4.0, 3.0]], [[3.0, 1.0, 7.0], [3.0, 6.0, 1.0], [2.0, 3.0, 6.0]]], [[[8.0, 8.0, 5.0], 
[4.0, 3.0, 4.0], [2.0, 6.0, 2.0]], [[2.0, 6.0, 4.0], [6.0, 4.0, 3.0], [3.0, 3.0, 3.0]], [[4.0, 8.0, 2.0], [7.0, 6.0, 8.0], 
[5.0, 7.0, 1.0]]], [[[8.0, 3.0, 8.0], [2.0, 3.0, 8.0], [5.0, 1.0, 3.0]], [[7.0, 1.0, 3.0], [5.0, 8.0, 6.0], [6.0, 4.0, 3.0]], 
[[3.0, 7.0, 3.0], [7.0, 8.0, 3.0], [6.0, 3.0, 2.0]]], [[[8.0, 5.0, 5.0], [7.0, 6.0, 4.0], [1.0, 8.0, 5.0]], [[4.0, 7.0, 4.0], 
[1.0, 2.0, 6.0], [5.0, 8.0, 5.0]], [[2.0, 7.0, 6.0], [1.0, 5.0, 6.0], [2.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPPR(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 6.0], [8.0, 7.0, 2.0], [7.0, 5.0, 5.0]], [[2.0, 2.0, 7.0], [4.0, 1.0, 8.0], [8.0, 8.0, 
1.0]], [[3.0, 5.0, 1.0], [5.0, 2.0, 3.0], [3.0, 2.0, 4.0]]], [[[7.0, 7.0, 8.0], [2.0, 2.0, 5.0], [4.0, 5.0, 5.0]], [[6.0, 2.0, 
6.0], [5.0, 7.0, 7.0], [4.0, 6.0, 3.0]], [[8.0, 7.0, 4.0], [4.0, 7.0, 3.0], [3.0, 5.0, 8.0]]], [[[6.0, 5.0, 7.0], [5.0, 6.0, 
8.0], [7.0, 6.0, 3.0]], [[4.0, 5.0, 4.0], [4.0, 5.0, 3.0], [1.0, 7.0, 5.0]], [[7.0, 1.0, 3.0], [8.0, 6.0, 6.0], [7.0, 6.0, 
4.0]]], [[[6.0, 6.0, 5.0], [7.0, 4.0, 8.0], [4.0, 8.0, 6.0]], [[8.0, 7.0, 4.0], [3.0, 6.0, 8.0], [6.0, 7.0, 2.0]], [[2.0, 1.0, 
7.0], [2.0, 5.0, 3.0], [2.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 7.0], [2.0, 3.0, 4.0], [2.0, 3.0, 4.0]], [[4.0, 8.0, 1.0], [6.0, 3.0, 
3.0], [3.0, 3.0, 5.0]], [[4.0, 4.0, 6.0], [1.0, 8.0, 6.0], [4.0, 6.0, 3.0]]], [[[6.0, 1.0, 2.0], [6.0, 8.0, 6.0], [6.0, 2.0, 
3.0]], [[2.0, 3.0, 2.0], [8.0, 5.0, 4.0], [7.0, 8.0, 8.0]], [[6.0, 8.0, 3.0], [8.0, 7.0, 7.0], [7.0, 4.0, 1.0]]], [[[7.0, 5.0, 
4.0], [2.0, 5.0, 4.0], [7.0, 4.0, 1.0]], [[7.0, 7.0, 2.0], [4.0, 2.0, 1.0], [1.0, 3.0, 5.0]], [[8.0, 3.0, 7.0], [8.0, 6.0, 
6.0], [2.0, 6.0, 3.0]]], [[[4.0, 7.0, 8.0], [1.0, 8.0, 2.0], [1.0, 6.0, 5.0]], [[3.0, 1.0, 7.0], [2.0, 4.0, 6.0], [1.0, 6.0, 
1.0]], [[4.0, 1.0, 3.0], [5.0, 2.0, 1.0], [2.0, 4.0, 8.0]]]]))
      val=Data(numpy.array([[4.0], [1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0], [4.0]]))
      arg[1:3,1,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 6.0], [8.0, 7.0, 2.0], [7.0, 5.0, 5.0]], [[2.0, 2.0, 7.0], [4.0, 1.0, 8.0], [8.0, 
8.0, 1.0]], [[3.0, 5.0, 1.0], [5.0, 2.0, 3.0], [3.0, 2.0, 4.0]]], [[[7.0, 7.0, 8.0], [2.0, 2.0, 5.0], [4.0, 5.0, 5.0]], [[6.0, 
2.0, 6.0], [5.0, 4.0, 7.0], [4.0, 6.0, 3.0]], [[8.0, 7.0, 4.0], [4.0, 7.0, 3.0], [3.0, 5.0, 8.0]]], [[[6.0, 5.0, 7.0], [5.0, 
6.0, 8.0], [7.0, 6.0, 3.0]], [[4.0, 5.0, 4.0], [4.0, 1.0, 3.0], [1.0, 7.0, 5.0]], [[7.0, 1.0, 3.0], [8.0, 6.0, 6.0], [7.0, 6.0, 
4.0]]], [[[6.0, 6.0, 5.0], [7.0, 4.0, 8.0], [4.0, 8.0, 6.0]], [[8.0, 7.0, 4.0], [3.0, 6.0, 8.0], [6.0, 7.0, 2.0]], [[2.0, 1.0, 
7.0], [2.0, 5.0, 3.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 7.0], [2.0, 3.0, 4.0], [2.0, 3.0, 4.0]], 
[[4.0, 8.0, 1.0], [6.0, 3.0, 3.0], [3.0, 3.0, 5.0]], [[4.0, 4.0, 6.0], [1.0, 8.0, 6.0], [4.0, 6.0, 3.0]]], [[[6.0, 1.0, 2.0], 
[6.0, 8.0, 6.0], [6.0, 2.0, 3.0]], [[2.0, 3.0, 2.0], [8.0, 6.0, 4.0], [7.0, 8.0, 8.0]], [[6.0, 8.0, 3.0], [8.0, 7.0, 7.0], 
[7.0, 4.0, 1.0]]], [[[7.0, 5.0, 4.0], [2.0, 5.0, 4.0], [7.0, 4.0, 1.0]], [[7.0, 7.0, 2.0], [4.0, 4.0, 1.0], [1.0, 3.0, 5.0]], 
[[8.0, 3.0, 7.0], [8.0, 6.0, 6.0], [2.0, 6.0, 3.0]]], [[[4.0, 7.0, 8.0], [1.0, 8.0, 2.0], [1.0, 6.0, 5.0]], [[3.0, 1.0, 7.0], 
[2.0, 4.0, 6.0], [1.0, 6.0, 1.0]], [[4.0, 1.0, 3.0], [5.0, 2.0, 1.0], [2.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPPR(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 6.0], [1.0, 1.0, 8.0], [6.0, 1.0, 6.0]], [[6.0, 1.0, 1.0], [8.0, 4.0, 2.0], [3.0, 7.0, 
8.0]], [[8.0, 1.0, 5.0], [7.0, 4.0, 6.0], [2.0, 2.0, 4.0]]], [[[5.0, 5.0, 6.0], [8.0, 7.0, 4.0], [6.0, 5.0, 2.0]], [[7.0, 7.0, 
7.0], [4.0, 6.0, 4.0], [4.0, 6.0, 6.0]], [[5.0, 4.0, 7.0], [7.0, 7.0, 6.0], [2.0, 2.0, 8.0]]], [[[5.0, 5.0, 3.0], [2.0, 6.0, 
6.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 8.0], [3.0, 2.0, 5.0], [4.0, 7.0, 5.0]], [[4.0, 2.0, 5.0], [7.0, 1.0, 7.0], [6.0, 6.0, 
2.0]]], [[[3.0, 5.0, 4.0], [3.0, 5.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 5.0, 1.0], [8.0, 8.0, 3.0], [2.0, 4.0, 6.0]], [[5.0, 8.0, 
4.0], [7.0, 3.0, 8.0], [7.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 6.0], [1.0, 5.0, 3.0], [3.0, 5.0, 3.0]], [[6.0, 4.0, 6.0], [1.0, 6.0, 
1.0], [4.0, 8.0, 1.0]], [[6.0, 4.0, 8.0], [4.0, 7.0, 5.0], [1.0, 4.0, 5.0]]], [[[6.0, 1.0, 5.0], [1.0, 1.0, 7.0], [3.0, 5.0, 
4.0]], [[2.0, 2.0, 8.0], [6.0, 5.0, 4.0], [4.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], [7.0, 4.0, 1.0], [1.0, 4.0, 2.0]]], [[[6.0, 7.0, 
5.0], [5.0, 4.0, 3.0], [1.0, 3.0, 4.0]], [[1.0, 5.0, 8.0], [8.0, 4.0, 3.0], [3.0, 7.0, 3.0]], [[3.0, 4.0, 4.0], [2.0, 7.0, 
5.0], [2.0, 2.0, 4.0]]], [[[7.0, 6.0, 4.0], [2.0, 6.0, 6.0], [8.0, 6.0, 7.0]], [[7.0, 1.0, 2.0], [1.0, 5.0, 7.0], [8.0, 5.0, 
1.0]], [[4.0, 4.0, 4.0], [5.0, 5.0, 8.0], [6.0, 3.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[4.0], [8.0]])+(1.-msk_val)*numpy.array([[2.0], [3.0]])
      arg[1:3,1,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 6.0], [1.0, 1.0, 8.0], [6.0, 1.0, 6.0]], [[6.0, 1.0, 1.0], [8.0, 4.0, 2.0], [3.0, 
7.0, 8.0]], [[8.0, 1.0, 5.0], [7.0, 4.0, 6.0], [2.0, 2.0, 4.0]]], [[[5.0, 5.0, 6.0], [8.0, 7.0, 4.0], [6.0, 5.0, 2.0]], [[7.0, 
7.0, 7.0], [4.0, 4.0, 4.0], [4.0, 6.0, 6.0]], [[5.0, 4.0, 7.0], [7.0, 7.0, 6.0], [2.0, 2.0, 8.0]]], [[[5.0, 5.0, 3.0], [2.0, 
6.0, 6.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 8.0], [3.0, 8.0, 5.0], [4.0, 7.0, 5.0]], [[4.0, 2.0, 5.0], [7.0, 1.0, 7.0], [6.0, 6.0, 
2.0]]], [[[3.0, 5.0, 4.0], [3.0, 5.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 5.0, 1.0], [8.0, 8.0, 3.0], [2.0, 4.0, 6.0]], [[5.0, 8.0, 
4.0], [7.0, 3.0, 8.0], [7.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 6.0], [1.0, 5.0, 3.0], [3.0, 5.0, 3.0]], 
[[6.0, 4.0, 6.0], [1.0, 6.0, 1.0], [4.0, 8.0, 1.0]], [[6.0, 4.0, 8.0], [4.0, 7.0, 5.0], [1.0, 4.0, 5.0]]], [[[6.0, 1.0, 5.0], 
[1.0, 1.0, 7.0], [3.0, 5.0, 4.0]], [[2.0, 2.0, 8.0], [6.0, 2.0, 4.0], [4.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], [7.0, 4.0, 1.0], 
[1.0, 4.0, 2.0]]], [[[6.0, 7.0, 5.0], [5.0, 4.0, 3.0], [1.0, 3.0, 4.0]], [[1.0, 5.0, 8.0], [8.0, 3.0, 3.0], [3.0, 7.0, 3.0]], 
[[3.0, 4.0, 4.0], [2.0, 7.0, 5.0], [2.0, 2.0, 4.0]]], [[[7.0, 6.0, 4.0], [2.0, 6.0, 6.0], [8.0, 6.0, 7.0]], [[7.0, 1.0, 2.0], 
[1.0, 5.0, 7.0], [8.0, 5.0, 1.0]], [[4.0, 4.0, 4.0], [5.0, 5.0, 8.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLLC(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 2.0], [7.0, 1.0, 1.0], [5.0, 2.0, 3.0]], [[8.0, 3.0, 7.0], [6.0, 7.0, 2.0], [5.0, 6.0, 
4.0]], [[5.0, 2.0, 5.0], [8.0, 4.0, 5.0], [6.0, 2.0, 1.0]]], [[[1.0, 5.0, 8.0], [4.0, 2.0, 4.0], [1.0, 3.0, 4.0]], [[2.0, 5.0, 
8.0], [7.0, 8.0, 7.0], [2.0, 6.0, 6.0]], [[6.0, 8.0, 5.0], [6.0, 6.0, 4.0], [8.0, 4.0, 2.0]]], [[[4.0, 2.0, 3.0], [3.0, 6.0, 
2.0], [8.0, 2.0, 2.0]], [[4.0, 7.0, 6.0], [4.0, 8.0, 1.0], [8.0, 2.0, 6.0]], [[6.0, 3.0, 4.0], [8.0, 6.0, 8.0], [1.0, 1.0, 
2.0]]], [[[6.0, 7.0, 6.0], [2.0, 5.0, 4.0], [3.0, 8.0, 2.0]], [[8.0, 5.0, 8.0], [6.0, 8.0, 7.0], [6.0, 2.0, 1.0]], [[2.0, 6.0, 
3.0], [3.0, 2.0, 4.0], [8.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 5.0], [2.0, 4.0, 5.0], [7.0, 5.0, 8.0]], [[3.0, 4.0, 6.0], [8.0, 2.0, 
1.0], [2.0, 4.0, 1.0]], [[7.0, 7.0, 5.0], [7.0, 2.0, 5.0], [4.0, 3.0, 8.0]]], [[[2.0, 8.0, 1.0], [6.0, 2.0, 2.0], [2.0, 5.0, 
3.0]], [[5.0, 7.0, 6.0], [3.0, 6.0, 8.0], [5.0, 2.0, 5.0]], [[8.0, 6.0, 1.0], [8.0, 2.0, 2.0], [2.0, 3.0, 8.0]]], [[[4.0, 5.0, 
7.0], [6.0, 5.0, 6.0], [6.0, 8.0, 4.0]], [[2.0, 2.0, 7.0], [8.0, 6.0, 6.0], [1.0, 8.0, 3.0]], [[2.0, 8.0, 6.0], [5.0, 6.0, 
7.0], [4.0, 3.0, 7.0]]], [[[1.0, 2.0, 5.0], [5.0, 7.0, 1.0], [6.0, 6.0, 5.0]], [[6.0, 4.0, 1.0], [4.0, 6.0, 1.0], [6.0, 2.0, 
3.0]], [[7.0, 1.0, 6.0], [3.0, 5.0, 4.0], [8.0, 7.0, 5.0]]]]))
      val=numpy.array([[[[3.0, 8.0, 2.0]]], [[[8.0, 1.0, 5.0]]]])
      arg[2:,2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 2.0], [7.0, 1.0, 1.0], [5.0, 2.0, 3.0]], [[8.0, 3.0, 7.0], [6.0, 7.0, 2.0], [5.0, 
6.0, 4.0]], [[5.0, 2.0, 5.0], [8.0, 4.0, 5.0], [6.0, 2.0, 1.0]]], [[[1.0, 5.0, 8.0], [4.0, 2.0, 4.0], [1.0, 3.0, 4.0]], [[2.0, 
5.0, 8.0], [7.0, 8.0, 7.0], [2.0, 6.0, 6.0]], [[6.0, 8.0, 5.0], [6.0, 6.0, 4.0], [8.0, 4.0, 2.0]]], [[[4.0, 2.0, 3.0], [3.0, 
6.0, 2.0], [8.0, 2.0, 2.0]], [[4.0, 7.0, 6.0], [4.0, 8.0, 1.0], [8.0, 2.0, 6.0]], [[6.0, 3.0, 4.0], [8.0, 6.0, 8.0], [3.0, 8.0, 
2.0]]], [[[6.0, 7.0, 6.0], [2.0, 5.0, 4.0], [3.0, 8.0, 2.0]], [[8.0, 5.0, 8.0], [6.0, 8.0, 7.0], [6.0, 2.0, 1.0]], [[2.0, 6.0, 
3.0], [3.0, 2.0, 4.0], [8.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 5.0], [2.0, 4.0, 5.0], [7.0, 5.0, 8.0]], 
[[3.0, 4.0, 6.0], [8.0, 2.0, 1.0], [2.0, 4.0, 1.0]], [[7.0, 7.0, 5.0], [7.0, 2.0, 5.0], [4.0, 3.0, 8.0]]], [[[2.0, 8.0, 1.0], 
[6.0, 2.0, 2.0], [2.0, 5.0, 3.0]], [[5.0, 7.0, 6.0], [3.0, 6.0, 8.0], [5.0, 2.0, 5.0]], [[8.0, 6.0, 1.0], [8.0, 2.0, 2.0], 
[2.0, 3.0, 8.0]]], [[[4.0, 5.0, 7.0], [6.0, 5.0, 6.0], [6.0, 8.0, 4.0]], [[2.0, 2.0, 7.0], [8.0, 6.0, 6.0], [1.0, 8.0, 3.0]], 
[[2.0, 8.0, 6.0], [5.0, 6.0, 7.0], [3.0, 8.0, 2.0]]], [[[1.0, 2.0, 5.0], [5.0, 7.0, 1.0], [6.0, 6.0, 5.0]], [[6.0, 4.0, 1.0], 
[4.0, 6.0, 1.0], [6.0, 2.0, 3.0]], [[7.0, 1.0, 6.0], [3.0, 5.0, 4.0], [8.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLLC(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 8.0], [7.0, 8.0, 8.0], [2.0, 1.0, 6.0]], [[1.0, 6.0, 3.0], [4.0, 6.0, 7.0], [2.0, 4.0, 
4.0]], [[8.0, 3.0, 4.0], [6.0, 3.0, 1.0], [3.0, 8.0, 3.0]]], [[[1.0, 8.0, 6.0], [7.0, 2.0, 3.0], [6.0, 6.0, 7.0]], [[2.0, 7.0, 
6.0], [5.0, 8.0, 2.0], [2.0, 8.0, 2.0]], [[6.0, 7.0, 4.0], [2.0, 7.0, 3.0], [3.0, 3.0, 5.0]]], [[[2.0, 2.0, 5.0], [2.0, 4.0, 
7.0], [7.0, 3.0, 2.0]], [[8.0, 8.0, 8.0], [5.0, 2.0, 7.0], [3.0, 6.0, 8.0]], [[4.0, 6.0, 6.0], [6.0, 1.0, 3.0], [8.0, 1.0, 
7.0]]], [[[2.0, 4.0, 8.0], [5.0, 8.0, 6.0], [7.0, 4.0, 4.0]], [[8.0, 3.0, 7.0], [2.0, 8.0, 2.0], [3.0, 3.0, 4.0]], [[2.0, 6.0, 
7.0], [3.0, 8.0, 4.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 4.0], [1.0, 3.0, 6.0], [8.0, 8.0, 7.0]], [[7.0, 3.0, 5.0], [4.0, 7.0, 
3.0], [3.0, 4.0, 7.0]], [[8.0, 2.0, 8.0], [7.0, 4.0, 6.0], [1.0, 3.0, 4.0]]], [[[7.0, 4.0, 6.0], [8.0, 2.0, 8.0], [1.0, 5.0, 
8.0]], [[4.0, 5.0, 8.0], [3.0, 8.0, 2.0], [4.0, 5.0, 8.0]], [[6.0, 5.0, 4.0], [1.0, 8.0, 2.0], [4.0, 5.0, 4.0]]], [[[2.0, 8.0, 
7.0], [6.0, 1.0, 1.0], [2.0, 7.0, 8.0]], [[3.0, 5.0, 4.0], [2.0, 6.0, 2.0], [5.0, 6.0, 7.0]], [[8.0, 2.0, 6.0], [3.0, 6.0, 
7.0], [7.0, 8.0, 7.0]]], [[[7.0, 6.0, 7.0], [2.0, 8.0, 3.0], [6.0, 8.0, 5.0]], [[2.0, 4.0, 7.0], [7.0, 5.0, 2.0], [4.0, 5.0, 
3.0]], [[5.0, 2.0, 4.0], [4.0, 1.0, 1.0], [2.0, 4.0, 3.0]]]]))
      val=Data(numpy.array([[[[3.0, 4.0, 5.0]]], [[[2.0, 6.0, 8.0]]]]),self.functionspace)
      arg[2:,2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 8.0], [7.0, 8.0, 8.0], [2.0, 1.0, 6.0]], [[1.0, 6.0, 3.0], [4.0, 6.0, 7.0], [2.0, 
4.0, 4.0]], [[8.0, 3.0, 4.0], [6.0, 3.0, 1.0], [3.0, 8.0, 3.0]]], [[[1.0, 8.0, 6.0], [7.0, 2.0, 3.0], [6.0, 6.0, 7.0]], [[2.0, 
7.0, 6.0], [5.0, 8.0, 2.0], [2.0, 8.0, 2.0]], [[6.0, 7.0, 4.0], [2.0, 7.0, 3.0], [3.0, 3.0, 5.0]]], [[[2.0, 2.0, 5.0], [2.0, 
4.0, 7.0], [7.0, 3.0, 2.0]], [[8.0, 8.0, 8.0], [5.0, 2.0, 7.0], [3.0, 6.0, 8.0]], [[4.0, 6.0, 6.0], [6.0, 1.0, 3.0], [3.0, 4.0, 
5.0]]], [[[2.0, 4.0, 8.0], [5.0, 8.0, 6.0], [7.0, 4.0, 4.0]], [[8.0, 3.0, 7.0], [2.0, 8.0, 2.0], [3.0, 3.0, 4.0]], [[2.0, 6.0, 
7.0], [3.0, 8.0, 4.0], [2.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 4.0], [1.0, 3.0, 6.0], [8.0, 8.0, 7.0]], 
[[7.0, 3.0, 5.0], [4.0, 7.0, 3.0], [3.0, 4.0, 7.0]], [[8.0, 2.0, 8.0], [7.0, 4.0, 6.0], [1.0, 3.0, 4.0]]], [[[7.0, 4.0, 6.0], 
[8.0, 2.0, 8.0], [1.0, 5.0, 8.0]], [[4.0, 5.0, 8.0], [3.0, 8.0, 2.0], [4.0, 5.0, 8.0]], [[6.0, 5.0, 4.0], [1.0, 8.0, 2.0], 
[4.0, 5.0, 4.0]]], [[[2.0, 8.0, 7.0], [6.0, 1.0, 1.0], [2.0, 7.0, 8.0]], [[3.0, 5.0, 4.0], [2.0, 6.0, 2.0], [5.0, 6.0, 7.0]], 
[[8.0, 2.0, 6.0], [3.0, 6.0, 7.0], [3.0, 4.0, 5.0]]], [[[7.0, 6.0, 7.0], [2.0, 8.0, 3.0], [6.0, 8.0, 5.0]], [[2.0, 4.0, 7.0], 
[7.0, 5.0, 2.0], [4.0, 5.0, 3.0]], [[5.0, 2.0, 4.0], [4.0, 1.0, 1.0], [2.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLLC(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 7.0], [6.0, 8.0, 5.0], [5.0, 4.0, 2.0]], [[4.0, 5.0, 5.0], [5.0, 5.0, 8.0], [4.0, 8.0, 
2.0]], [[7.0, 8.0, 1.0], [6.0, 7.0, 8.0], [6.0, 3.0, 2.0]]], [[[1.0, 3.0, 1.0], [7.0, 8.0, 5.0], [6.0, 3.0, 7.0]], [[8.0, 2.0, 
8.0], [4.0, 2.0, 2.0], [3.0, 5.0, 1.0]], [[5.0, 7.0, 6.0], [1.0, 5.0, 1.0], [3.0, 8.0, 6.0]]], [[[4.0, 8.0, 3.0], [7.0, 7.0, 
2.0], [5.0, 8.0, 5.0]], [[7.0, 8.0, 6.0], [1.0, 6.0, 4.0], [3.0, 8.0, 7.0]], [[1.0, 3.0, 5.0], [8.0, 3.0, 8.0], [4.0, 4.0, 
7.0]]], [[[4.0, 7.0, 4.0], [2.0, 2.0, 5.0], [7.0, 7.0, 4.0]], [[7.0, 7.0, 6.0], [4.0, 8.0, 3.0], [6.0, 6.0, 7.0]], [[4.0, 2.0, 
8.0], [3.0, 5.0, 2.0], [8.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 3.0], [3.0, 7.0, 7.0], [4.0, 4.0, 6.0]], [[3.0, 2.0, 6.0], [1.0, 4.0, 
6.0], [5.0, 5.0, 2.0]], [[7.0, 7.0, 4.0], [6.0, 7.0, 8.0], [5.0, 2.0, 4.0]]], [[[6.0, 8.0, 4.0], [4.0, 6.0, 4.0], [1.0, 3.0, 
2.0]], [[6.0, 5.0, 2.0], [7.0, 5.0, 2.0], [4.0, 2.0, 4.0]], [[1.0, 1.0, 6.0], [6.0, 3.0, 4.0], [5.0, 7.0, 7.0]]], [[[1.0, 4.0, 
4.0], [4.0, 2.0, 4.0], [6.0, 7.0, 1.0]], [[4.0, 2.0, 1.0], [7.0, 7.0, 1.0], [1.0, 8.0, 5.0]], [[1.0, 7.0, 1.0], [5.0, 1.0, 
8.0], [6.0, 4.0, 3.0]]], [[[1.0, 6.0, 1.0], [3.0, 1.0, 2.0], [8.0, 6.0, 1.0]], [[8.0, 3.0, 3.0], [1.0, 1.0, 3.0], [8.0, 4.0, 
6.0]], [[3.0, 5.0, 8.0], [6.0, 4.0, 4.0], [1.0, 7.0, 6.0]]]]))
      val=Data(numpy.array([[[[1.0, 4.0, 3.0]]], [[[3.0, 3.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 4.0]]], [[[7.0, 3.0, 5.0]]]]))
      arg[2:,2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 7.0], [6.0, 8.0, 5.0], [5.0, 4.0, 2.0]], [[4.0, 5.0, 5.0], [5.0, 5.0, 8.0], [4.0, 
8.0, 2.0]], [[7.0, 8.0, 1.0], [6.0, 7.0, 8.0], [6.0, 3.0, 2.0]]], [[[1.0, 3.0, 1.0], [7.0, 8.0, 5.0], [6.0, 3.0, 7.0]], [[8.0, 
2.0, 8.0], [4.0, 2.0, 2.0], [3.0, 5.0, 1.0]], [[5.0, 7.0, 6.0], [1.0, 5.0, 1.0], [3.0, 8.0, 6.0]]], [[[4.0, 8.0, 3.0], [7.0, 
7.0, 2.0], [5.0, 8.0, 5.0]], [[7.0, 8.0, 6.0], [1.0, 6.0, 4.0], [3.0, 8.0, 7.0]], [[1.0, 3.0, 5.0], [8.0, 3.0, 8.0], [1.0, 4.0, 
3.0]]], [[[4.0, 7.0, 4.0], [2.0, 2.0, 5.0], [7.0, 7.0, 4.0]], [[7.0, 7.0, 6.0], [4.0, 8.0, 3.0], [6.0, 6.0, 7.0]], [[4.0, 2.0, 
8.0], [3.0, 5.0, 2.0], [3.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 3.0], [3.0, 7.0, 7.0], [4.0, 4.0, 6.0]], 
[[3.0, 2.0, 6.0], [1.0, 4.0, 6.0], [5.0, 5.0, 2.0]], [[7.0, 7.0, 4.0], [6.0, 7.0, 8.0], [5.0, 2.0, 4.0]]], [[[6.0, 8.0, 4.0], 
[4.0, 6.0, 4.0], [1.0, 3.0, 2.0]], [[6.0, 5.0, 2.0], [7.0, 5.0, 2.0], [4.0, 2.0, 4.0]], [[1.0, 1.0, 6.0], [6.0, 3.0, 4.0], 
[5.0, 7.0, 7.0]]], [[[1.0, 4.0, 4.0], [4.0, 2.0, 4.0], [6.0, 7.0, 1.0]], [[4.0, 2.0, 1.0], [7.0, 7.0, 1.0], [1.0, 8.0, 5.0]], 
[[1.0, 7.0, 1.0], [5.0, 1.0, 8.0], [7.0, 8.0, 4.0]]], [[[1.0, 6.0, 1.0], [3.0, 1.0, 2.0], [8.0, 6.0, 1.0]], [[8.0, 3.0, 3.0], 
[1.0, 1.0, 3.0], [8.0, 4.0, 6.0]], [[3.0, 5.0, 8.0], [6.0, 4.0, 4.0], [7.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLLC(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 2.0], [8.0, 2.0, 3.0], [3.0, 2.0, 4.0]], [[7.0, 8.0, 2.0], [6.0, 2.0, 7.0], [4.0, 7.0, 
2.0]], [[5.0, 1.0, 1.0], [6.0, 7.0, 2.0], [8.0, 5.0, 7.0]]], [[[3.0, 5.0, 8.0], [6.0, 3.0, 8.0], [2.0, 8.0, 3.0]], [[5.0, 3.0, 
5.0], [6.0, 4.0, 1.0], [7.0, 7.0, 4.0]], [[8.0, 2.0, 5.0], [3.0, 1.0, 8.0], [3.0, 5.0, 4.0]]], [[[6.0, 3.0, 1.0], [5.0, 8.0, 
7.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], [4.0, 1.0, 6.0], [8.0, 6.0, 8.0]], [[3.0, 4.0, 7.0], [2.0, 6.0, 2.0], [1.0, 8.0, 
5.0]]], [[[1.0, 3.0, 2.0], [8.0, 4.0, 8.0], [1.0, 8.0, 1.0]], [[2.0, 5.0, 8.0], [7.0, 2.0, 8.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 
8.0], [2.0, 1.0, 2.0], [5.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 2.0], [1.0, 3.0, 7.0], [4.0, 6.0, 6.0]], [[5.0, 6.0, 3.0], [6.0, 2.0, 
6.0], [1.0, 5.0, 5.0]], [[7.0, 6.0, 6.0], [3.0, 6.0, 5.0], [5.0, 5.0, 8.0]]], [[[1.0, 3.0, 7.0], [8.0, 6.0, 4.0], [3.0, 5.0, 
4.0]], [[2.0, 2.0, 1.0], [5.0, 7.0, 8.0], [3.0, 8.0, 8.0]], [[4.0, 4.0, 8.0], [8.0, 8.0, 7.0], [5.0, 6.0, 7.0]]], [[[3.0, 6.0, 
8.0], [3.0, 5.0, 8.0], [8.0, 8.0, 4.0]], [[7.0, 5.0, 5.0], [4.0, 2.0, 6.0], [5.0, 8.0, 4.0]], [[2.0, 7.0, 8.0], [6.0, 7.0, 
1.0], [5.0, 3.0, 5.0]]], [[[3.0, 5.0, 3.0], [8.0, 1.0, 8.0], [4.0, 5.0, 6.0]], [[3.0, 5.0, 6.0], [4.0, 5.0, 7.0], [5.0, 5.0, 
3.0]], [[3.0, 4.0, 6.0], [4.0, 4.0, 1.0], [7.0, 8.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 6.0, 4.0]]], [[[5.0, 3.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 5.0, 3.0]]], 
[[[4.0, 1.0, 2.0]]]])
      arg[2:,2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 2.0], [8.0, 2.0, 3.0], [3.0, 2.0, 4.0]], [[7.0, 8.0, 2.0], [6.0, 2.0, 7.0], [4.0, 
7.0, 2.0]], [[5.0, 1.0, 1.0], [6.0, 7.0, 2.0], [8.0, 5.0, 7.0]]], [[[3.0, 5.0, 8.0], [6.0, 3.0, 8.0], [2.0, 8.0, 3.0]], [[5.0, 
3.0, 5.0], [6.0, 4.0, 1.0], [7.0, 7.0, 4.0]], [[8.0, 2.0, 5.0], [3.0, 1.0, 8.0], [3.0, 5.0, 4.0]]], [[[6.0, 3.0, 1.0], [5.0, 
8.0, 7.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], [4.0, 1.0, 6.0], [8.0, 6.0, 8.0]], [[3.0, 4.0, 7.0], [2.0, 6.0, 2.0], [3.0, 6.0, 
4.0]]], [[[1.0, 3.0, 2.0], [8.0, 4.0, 8.0], [1.0, 8.0, 1.0]], [[2.0, 5.0, 8.0], [7.0, 2.0, 8.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 
8.0], [2.0, 1.0, 2.0], [5.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 2.0], [1.0, 3.0, 7.0], [4.0, 6.0, 6.0]], 
[[5.0, 6.0, 3.0], [6.0, 2.0, 6.0], [1.0, 5.0, 5.0]], [[7.0, 6.0, 6.0], [3.0, 6.0, 5.0], [5.0, 5.0, 8.0]]], [[[1.0, 3.0, 7.0], 
[8.0, 6.0, 4.0], [3.0, 5.0, 4.0]], [[2.0, 2.0, 1.0], [5.0, 7.0, 8.0], [3.0, 8.0, 8.0]], [[4.0, 4.0, 8.0], [8.0, 8.0, 7.0], 
[5.0, 6.0, 7.0]]], [[[3.0, 6.0, 8.0], [3.0, 5.0, 8.0], [8.0, 8.0, 4.0]], [[7.0, 5.0, 5.0], [4.0, 2.0, 6.0], [5.0, 8.0, 4.0]], 
[[2.0, 7.0, 8.0], [6.0, 7.0, 1.0], [7.0, 5.0, 3.0]]], [[[3.0, 5.0, 3.0], [8.0, 1.0, 8.0], [4.0, 5.0, 6.0]], [[3.0, 5.0, 6.0], 
[4.0, 5.0, 7.0], [5.0, 5.0, 3.0]], [[3.0, 4.0, 6.0], [4.0, 4.0, 1.0], [4.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPLC(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 5.0], [4.0, 8.0, 5.0], [4.0, 7.0, 3.0]], [[6.0, 6.0, 1.0], [7.0, 8.0, 3.0], [6.0, 7.0, 
7.0]], [[7.0, 1.0, 8.0], [2.0, 4.0, 4.0], [6.0, 7.0, 7.0]]], [[[4.0, 6.0, 1.0], [8.0, 3.0, 3.0], [6.0, 6.0, 5.0]], [[6.0, 8.0, 
4.0], [1.0, 8.0, 7.0], [7.0, 1.0, 7.0]], [[4.0, 5.0, 2.0], [1.0, 2.0, 5.0], [4.0, 2.0, 4.0]]], [[[3.0, 7.0, 3.0], [8.0, 4.0, 
2.0], [7.0, 3.0, 7.0]], [[1.0, 1.0, 1.0], [6.0, 3.0, 2.0], [4.0, 3.0, 8.0]], [[1.0, 3.0, 3.0], [2.0, 2.0, 6.0], [1.0, 8.0, 
8.0]]], [[[1.0, 7.0, 2.0], [1.0, 6.0, 3.0], [4.0, 5.0, 7.0]], [[1.0, 8.0, 8.0], [3.0, 4.0, 5.0], [2.0, 4.0, 3.0]], [[1.0, 1.0, 
4.0], [3.0, 1.0, 6.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 4.0], [8.0, 7.0, 1.0], [5.0, 7.0, 3.0]], [[4.0, 5.0, 3.0], [2.0, 3.0, 
4.0], [8.0, 7.0, 5.0]], [[8.0, 7.0, 6.0], [4.0, 5.0, 8.0], [7.0, 8.0, 3.0]]], [[[8.0, 8.0, 1.0], [1.0, 2.0, 6.0], [2.0, 8.0, 
8.0]], [[5.0, 8.0, 4.0], [7.0, 1.0, 3.0], [3.0, 6.0, 8.0]], [[6.0, 3.0, 7.0], [8.0, 6.0, 5.0], [8.0, 5.0, 2.0]]], [[[5.0, 4.0, 
5.0], [2.0, 3.0, 5.0], [5.0, 7.0, 6.0]], [[7.0, 1.0, 3.0], [5.0, 3.0, 3.0], [6.0, 7.0, 6.0]], [[6.0, 7.0, 7.0], [3.0, 5.0, 
7.0], [5.0, 8.0, 4.0]]], [[[2.0, 4.0, 1.0], [6.0, 4.0, 7.0], [5.0, 7.0, 6.0]], [[2.0, 1.0, 7.0], [2.0, 3.0, 4.0], [7.0, 7.0, 
7.0]], [[2.0, 5.0, 1.0], [1.0, 3.0, 7.0], [8.0, 3.0, 2.0]]]]))
      val=numpy.array([[[7.0, 7.0, 2.0]], [[5.0, 3.0, 3.0]], [[6.0, 3.0, 7.0]], [[5.0, 1.0, 3.0]]])
      arg[:,1,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 5.0], [4.0, 8.0, 5.0], [4.0, 7.0, 3.0]], [[6.0, 6.0, 1.0], [7.0, 8.0, 3.0], [7.0, 
7.0, 2.0]], [[7.0, 1.0, 8.0], [2.0, 4.0, 4.0], [6.0, 7.0, 7.0]]], [[[4.0, 6.0, 1.0], [8.0, 3.0, 3.0], [6.0, 6.0, 5.0]], [[6.0, 
8.0, 4.0], [1.0, 8.0, 7.0], [5.0, 3.0, 3.0]], [[4.0, 5.0, 2.0], [1.0, 2.0, 5.0], [4.0, 2.0, 4.0]]], [[[3.0, 7.0, 3.0], [8.0, 
4.0, 2.0], [7.0, 3.0, 7.0]], [[1.0, 1.0, 1.0], [6.0, 3.0, 2.0], [6.0, 3.0, 7.0]], [[1.0, 3.0, 3.0], [2.0, 2.0, 6.0], [1.0, 8.0, 
8.0]]], [[[1.0, 7.0, 2.0], [1.0, 6.0, 3.0], [4.0, 5.0, 7.0]], [[1.0, 8.0, 8.0], [3.0, 4.0, 5.0], [5.0, 1.0, 3.0]], [[1.0, 1.0, 
4.0], [3.0, 1.0, 6.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 4.0], [8.0, 7.0, 1.0], [5.0, 7.0, 3.0]], 
[[4.0, 5.0, 3.0], [2.0, 3.0, 4.0], [7.0, 7.0, 2.0]], [[8.0, 7.0, 6.0], [4.0, 5.0, 8.0], [7.0, 8.0, 3.0]]], [[[8.0, 8.0, 1.0], 
[1.0, 2.0, 6.0], [2.0, 8.0, 8.0]], [[5.0, 8.0, 4.0], [7.0, 1.0, 3.0], [5.0, 3.0, 3.0]], [[6.0, 3.0, 7.0], [8.0, 6.0, 5.0], 
[8.0, 5.0, 2.0]]], [[[5.0, 4.0, 5.0], [2.0, 3.0, 5.0], [5.0, 7.0, 6.0]], [[7.0, 1.0, 3.0], [5.0, 3.0, 3.0], [6.0, 3.0, 7.0]], 
[[6.0, 7.0, 7.0], [3.0, 5.0, 7.0], [5.0, 8.0, 4.0]]], [[[2.0, 4.0, 1.0], [6.0, 4.0, 7.0], [5.0, 7.0, 6.0]], [[2.0, 1.0, 7.0], 
[2.0, 3.0, 4.0], [5.0, 1.0, 3.0]], [[2.0, 5.0, 1.0], [1.0, 3.0, 7.0], [8.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPLC(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 3.0], [3.0, 2.0, 4.0], [7.0, 6.0, 7.0]], [[3.0, 6.0, 2.0], [3.0, 6.0, 2.0], [2.0, 3.0, 
3.0]], [[6.0, 8.0, 7.0], [1.0, 7.0, 8.0], [6.0, 7.0, 4.0]]], [[[4.0, 4.0, 7.0], [8.0, 2.0, 6.0], [4.0, 5.0, 2.0]], [[8.0, 5.0, 
4.0], [3.0, 8.0, 2.0], [6.0, 8.0, 4.0]], [[7.0, 3.0, 4.0], [4.0, 8.0, 1.0], [5.0, 6.0, 7.0]]], [[[3.0, 5.0, 3.0], [7.0, 5.0, 
8.0], [6.0, 7.0, 8.0]], [[2.0, 8.0, 7.0], [4.0, 1.0, 2.0], [6.0, 8.0, 2.0]], [[8.0, 5.0, 3.0], [1.0, 8.0, 6.0], [4.0, 6.0, 
2.0]]], [[[6.0, 6.0, 1.0], [8.0, 3.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 2.0, 7.0], [5.0, 8.0, 8.0], [6.0, 4.0, 2.0]], [[4.0, 3.0, 
6.0], [7.0, 6.0, 8.0], [6.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 3.0], [5.0, 8.0, 2.0], [5.0, 6.0, 6.0]], [[2.0, 3.0, 5.0], [1.0, 8.0, 
6.0], [2.0, 7.0, 8.0]], [[2.0, 6.0, 7.0], [7.0, 8.0, 1.0], [4.0, 5.0, 1.0]]], [[[6.0, 1.0, 1.0], [6.0, 5.0, 6.0], [4.0, 8.0, 
5.0]], [[4.0, 1.0, 7.0], [1.0, 6.0, 2.0], [5.0, 1.0, 5.0]], [[3.0, 7.0, 7.0], [2.0, 3.0, 3.0], [3.0, 8.0, 3.0]]], [[[1.0, 7.0, 
2.0], [3.0, 5.0, 4.0], [7.0, 2.0, 6.0]], [[2.0, 7.0, 6.0], [3.0, 3.0, 1.0], [3.0, 3.0, 4.0]], [[1.0, 3.0, 8.0], [7.0, 2.0, 
6.0], [2.0, 8.0, 5.0]]], [[[6.0, 8.0, 2.0], [4.0, 3.0, 8.0], [7.0, 1.0, 8.0]], [[8.0, 4.0, 8.0], [1.0, 7.0, 4.0], [1.0, 5.0, 
2.0]], [[4.0, 1.0, 1.0], [6.0, 1.0, 4.0], [7.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[[8.0, 8.0, 7.0]], [[4.0, 8.0, 7.0]], [[5.0, 3.0, 1.0]], [[3.0, 8.0, 5.0]]]),self.functionspace)
      arg[:,1,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 3.0], [3.0, 2.0, 4.0], [7.0, 6.0, 7.0]], [[3.0, 6.0, 2.0], [3.0, 6.0, 2.0], [8.0, 
8.0, 7.0]], [[6.0, 8.0, 7.0], [1.0, 7.0, 8.0], [6.0, 7.0, 4.0]]], [[[4.0, 4.0, 7.0], [8.0, 2.0, 6.0], [4.0, 5.0, 2.0]], [[8.0, 
5.0, 4.0], [3.0, 8.0, 2.0], [4.0, 8.0, 7.0]], [[7.0, 3.0, 4.0], [4.0, 8.0, 1.0], [5.0, 6.0, 7.0]]], [[[3.0, 5.0, 3.0], [7.0, 
5.0, 8.0], [6.0, 7.0, 8.0]], [[2.0, 8.0, 7.0], [4.0, 1.0, 2.0], [5.0, 3.0, 1.0]], [[8.0, 5.0, 3.0], [1.0, 8.0, 6.0], [4.0, 6.0, 
2.0]]], [[[6.0, 6.0, 1.0], [8.0, 3.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 2.0, 7.0], [5.0, 8.0, 8.0], [3.0, 8.0, 5.0]], [[4.0, 3.0, 
6.0], [7.0, 6.0, 8.0], [6.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 3.0], [5.0, 8.0, 2.0], [5.0, 6.0, 6.0]], 
[[2.0, 3.0, 5.0], [1.0, 8.0, 6.0], [8.0, 8.0, 7.0]], [[2.0, 6.0, 7.0], [7.0, 8.0, 1.0], [4.0, 5.0, 1.0]]], [[[6.0, 1.0, 1.0], 
[6.0, 5.0, 6.0], [4.0, 8.0, 5.0]], [[4.0, 1.0, 7.0], [1.0, 6.0, 2.0], [4.0, 8.0, 7.0]], [[3.0, 7.0, 7.0], [2.0, 3.0, 3.0], 
[3.0, 8.0, 3.0]]], [[[1.0, 7.0, 2.0], [3.0, 5.0, 4.0], [7.0, 2.0, 6.0]], [[2.0, 7.0, 6.0], [3.0, 3.0, 1.0], [5.0, 3.0, 1.0]], 
[[1.0, 3.0, 8.0], [7.0, 2.0, 6.0], [2.0, 8.0, 5.0]]], [[[6.0, 8.0, 2.0], [4.0, 3.0, 8.0], [7.0, 1.0, 8.0]], [[8.0, 4.0, 8.0], 
[1.0, 7.0, 4.0], [3.0, 8.0, 5.0]], [[4.0, 1.0, 1.0], [6.0, 1.0, 4.0], [7.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPLC(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 2.0], [8.0, 3.0, 8.0], [6.0, 6.0, 5.0]], [[5.0, 7.0, 5.0], [1.0, 5.0, 1.0], [5.0, 2.0, 
3.0]], [[7.0, 3.0, 5.0], [3.0, 6.0, 6.0], [7.0, 5.0, 2.0]]], [[[7.0, 5.0, 2.0], [2.0, 6.0, 3.0], [8.0, 8.0, 4.0]], [[7.0, 8.0, 
7.0], [4.0, 7.0, 5.0], [2.0, 8.0, 6.0]], [[4.0, 8.0, 5.0], [2.0, 8.0, 7.0], [1.0, 1.0, 3.0]]], [[[1.0, 3.0, 4.0], [7.0, 8.0, 
7.0], [6.0, 5.0, 1.0]], [[6.0, 6.0, 6.0], [3.0, 2.0, 5.0], [8.0, 1.0, 1.0]], [[5.0, 3.0, 4.0], [4.0, 7.0, 3.0], [1.0, 5.0, 
1.0]]], [[[6.0, 5.0, 7.0], [4.0, 2.0, 3.0], [6.0, 5.0, 2.0]], [[4.0, 5.0, 4.0], [1.0, 2.0, 1.0], [1.0, 5.0, 3.0]], [[8.0, 6.0, 
6.0], [6.0, 4.0, 5.0], [3.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 4.0], [7.0, 4.0, 1.0], [5.0, 5.0, 6.0]], [[2.0, 3.0, 5.0], [3.0, 7.0, 
4.0], [6.0, 3.0, 4.0]], [[7.0, 7.0, 3.0], [1.0, 3.0, 6.0], [8.0, 7.0, 5.0]]], [[[8.0, 4.0, 7.0], [8.0, 5.0, 7.0], [8.0, 3.0, 
2.0]], [[2.0, 8.0, 3.0], [1.0, 1.0, 5.0], [7.0, 3.0, 3.0]], [[2.0, 8.0, 6.0], [8.0, 5.0, 3.0], [7.0, 3.0, 5.0]]], [[[6.0, 8.0, 
5.0], [3.0, 5.0, 6.0], [1.0, 2.0, 8.0]], [[5.0, 5.0, 7.0], [6.0, 3.0, 1.0], [1.0, 5.0, 3.0]], [[1.0, 4.0, 5.0], [7.0, 7.0, 
7.0], [6.0, 8.0, 4.0]]], [[[3.0, 1.0, 1.0], [7.0, 3.0, 7.0], [3.0, 3.0, 7.0]], [[8.0, 2.0, 1.0], [5.0, 6.0, 1.0], [1.0, 8.0, 
7.0]], [[6.0, 1.0, 3.0], [3.0, 1.0, 4.0], [6.0, 6.0, 8.0]]]]))
      val=Data(numpy.array([[[5.0, 5.0, 1.0]], [[3.0, 4.0, 6.0]], [[1.0, 8.0, 6.0]], [[2.0, 2.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 2.0, 4.0]], [[2.0, 3.0, 2.0]], [[4.0, 8.0, 2.0]], [[5.0, 5.0, 7.0]]]))
      arg[:,1,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 2.0], [8.0, 3.0, 8.0], [6.0, 6.0, 5.0]], [[5.0, 7.0, 5.0], [1.0, 5.0, 1.0], [5.0, 
5.0, 1.0]], [[7.0, 3.0, 5.0], [3.0, 6.0, 6.0], [7.0, 5.0, 2.0]]], [[[7.0, 5.0, 2.0], [2.0, 6.0, 3.0], [8.0, 8.0, 4.0]], [[7.0, 
8.0, 7.0], [4.0, 7.0, 5.0], [3.0, 4.0, 6.0]], [[4.0, 8.0, 5.0], [2.0, 8.0, 7.0], [1.0, 1.0, 3.0]]], [[[1.0, 3.0, 4.0], [7.0, 
8.0, 7.0], [6.0, 5.0, 1.0]], [[6.0, 6.0, 6.0], [3.0, 2.0, 5.0], [1.0, 8.0, 6.0]], [[5.0, 3.0, 4.0], [4.0, 7.0, 3.0], [1.0, 5.0, 
1.0]]], [[[6.0, 5.0, 7.0], [4.0, 2.0, 3.0], [6.0, 5.0, 2.0]], [[4.0, 5.0, 4.0], [1.0, 2.0, 1.0], [2.0, 2.0, 2.0]], [[8.0, 6.0, 
6.0], [6.0, 4.0, 5.0], [3.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 4.0], [7.0, 4.0, 1.0], [5.0, 5.0, 6.0]], 
[[2.0, 3.0, 5.0], [3.0, 7.0, 4.0], [7.0, 2.0, 4.0]], [[7.0, 7.0, 3.0], [1.0, 3.0, 6.0], [8.0, 7.0, 5.0]]], [[[8.0, 4.0, 7.0], 
[8.0, 5.0, 7.0], [8.0, 3.0, 2.0]], [[2.0, 8.0, 3.0], [1.0, 1.0, 5.0], [2.0, 3.0, 2.0]], [[2.0, 8.0, 6.0], [8.0, 5.0, 3.0], 
[7.0, 3.0, 5.0]]], [[[6.0, 8.0, 5.0], [3.0, 5.0, 6.0], [1.0, 2.0, 8.0]], [[5.0, 5.0, 7.0], [6.0, 3.0, 1.0], [4.0, 8.0, 2.0]], 
[[1.0, 4.0, 5.0], [7.0, 7.0, 7.0], [6.0, 8.0, 4.0]]], [[[3.0, 1.0, 1.0], [7.0, 3.0, 7.0], [3.0, 3.0, 7.0]], [[8.0, 2.0, 1.0], 
[5.0, 6.0, 1.0], [5.0, 5.0, 7.0]], [[6.0, 1.0, 3.0], [3.0, 1.0, 4.0], [6.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPLC(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 4.0], [7.0, 3.0, 5.0], [2.0, 5.0, 1.0]], [[4.0, 3.0, 1.0], [1.0, 5.0, 3.0], [1.0, 5.0, 
3.0]], [[6.0, 8.0, 3.0], [2.0, 5.0, 6.0], [8.0, 5.0, 4.0]]], [[[8.0, 6.0, 4.0], [6.0, 7.0, 6.0], [6.0, 4.0, 6.0]], [[7.0, 8.0, 
7.0], [3.0, 5.0, 7.0], [7.0, 7.0, 2.0]], [[3.0, 3.0, 8.0], [7.0, 4.0, 2.0], [5.0, 8.0, 1.0]]], [[[4.0, 2.0, 3.0], [8.0, 4.0, 
8.0], [7.0, 5.0, 4.0]], [[6.0, 4.0, 6.0], [4.0, 2.0, 2.0], [3.0, 6.0, 3.0]], [[1.0, 7.0, 6.0], [2.0, 6.0, 1.0], [3.0, 4.0, 
6.0]]], [[[4.0, 7.0, 1.0], [8.0, 6.0, 6.0], [3.0, 4.0, 1.0]], [[4.0, 3.0, 6.0], [5.0, 3.0, 1.0], [6.0, 6.0, 2.0]], [[5.0, 1.0, 
1.0], [1.0, 8.0, 5.0], [4.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 3.0], [2.0, 6.0, 3.0], [7.0, 3.0, 2.0]], [[3.0, 4.0, 2.0], [3.0, 6.0, 
8.0], [3.0, 1.0, 7.0]], [[7.0, 6.0, 2.0], [5.0, 3.0, 2.0], [8.0, 7.0, 1.0]]], [[[2.0, 6.0, 7.0], [6.0, 7.0, 6.0], [3.0, 7.0, 
6.0]], [[8.0, 6.0, 7.0], [8.0, 3.0, 4.0], [3.0, 8.0, 4.0]], [[8.0, 2.0, 1.0], [5.0, 4.0, 4.0], [6.0, 1.0, 3.0]]], [[[3.0, 2.0, 
3.0], [4.0, 8.0, 2.0], [1.0, 2.0, 2.0]], [[8.0, 8.0, 6.0], [5.0, 7.0, 1.0], [2.0, 8.0, 7.0]], [[8.0, 2.0, 5.0], [3.0, 8.0, 
2.0], [4.0, 7.0, 6.0]]], [[[4.0, 7.0, 1.0], [6.0, 7.0, 4.0], [2.0, 8.0, 1.0]], [[2.0, 1.0, 8.0], [2.0, 1.0, 1.0], [1.0, 5.0, 
8.0]], [[5.0, 7.0, 5.0], [7.0, 4.0, 2.0], [3.0, 1.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 5.0, 6.0]], [[4.0, 4.0, 3.0]], [[1.0, 1.0, 8.0]], [[8.0, 1.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[7.0, 7.0, 7.0]], [[2.0, 6.0, 3.0]], [[3.0, 3.0, 4.0]], [[4.0, 5.0, 8.0]]])
      arg[:,1,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 4.0], [7.0, 3.0, 5.0], [2.0, 5.0, 1.0]], [[4.0, 3.0, 1.0], [1.0, 5.0, 3.0], [5.0, 
5.0, 6.0]], [[6.0, 8.0, 3.0], [2.0, 5.0, 6.0], [8.0, 5.0, 4.0]]], [[[8.0, 6.0, 4.0], [6.0, 7.0, 6.0], [6.0, 4.0, 6.0]], [[7.0, 
8.0, 7.0], [3.0, 5.0, 7.0], [4.0, 4.0, 3.0]], [[3.0, 3.0, 8.0], [7.0, 4.0, 2.0], [5.0, 8.0, 1.0]]], [[[4.0, 2.0, 3.0], [8.0, 
4.0, 8.0], [7.0, 5.0, 4.0]], [[6.0, 4.0, 6.0], [4.0, 2.0, 2.0], [1.0, 1.0, 8.0]], [[1.0, 7.0, 6.0], [2.0, 6.0, 1.0], [3.0, 4.0, 
6.0]]], [[[4.0, 7.0, 1.0], [8.0, 6.0, 6.0], [3.0, 4.0, 1.0]], [[4.0, 3.0, 6.0], [5.0, 3.0, 1.0], [8.0, 1.0, 6.0]], [[5.0, 1.0, 
1.0], [1.0, 8.0, 5.0], [4.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 3.0], [2.0, 6.0, 3.0], [7.0, 3.0, 2.0]], 
[[3.0, 4.0, 2.0], [3.0, 6.0, 8.0], [7.0, 7.0, 7.0]], [[7.0, 6.0, 2.0], [5.0, 3.0, 2.0], [8.0, 7.0, 1.0]]], [[[2.0, 6.0, 7.0], 
[6.0, 7.0, 6.0], [3.0, 7.0, 6.0]], [[8.0, 6.0, 7.0], [8.0, 3.0, 4.0], [2.0, 6.0, 3.0]], [[8.0, 2.0, 1.0], [5.0, 4.0, 4.0], 
[6.0, 1.0, 3.0]]], [[[3.0, 2.0, 3.0], [4.0, 8.0, 2.0], [1.0, 2.0, 2.0]], [[8.0, 8.0, 6.0], [5.0, 7.0, 1.0], [3.0, 3.0, 4.0]], 
[[8.0, 2.0, 5.0], [3.0, 8.0, 2.0], [4.0, 7.0, 6.0]]], [[[4.0, 7.0, 1.0], [6.0, 7.0, 4.0], [2.0, 8.0, 1.0]], [[2.0, 1.0, 8.0], 
[2.0, 1.0, 1.0], [4.0, 5.0, 8.0]], [[5.0, 7.0, 5.0], [7.0, 4.0, 2.0], [3.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UUCL(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 5.0], [5.0, 8.0, 4.0], [6.0, 8.0, 6.0]], [[7.0, 4.0, 3.0], [4.0, 7.0, 1.0], [8.0, 8.0, 
8.0]], [[7.0, 5.0, 3.0], [8.0, 4.0, 7.0], [2.0, 1.0, 1.0]]], [[[7.0, 4.0, 8.0], [8.0, 4.0, 5.0], [2.0, 2.0, 2.0]], [[4.0, 7.0, 
7.0], [2.0, 2.0, 2.0], [5.0, 7.0, 4.0]], [[4.0, 3.0, 5.0], [8.0, 2.0, 5.0], [8.0, 6.0, 5.0]]], [[[7.0, 1.0, 2.0], [1.0, 8.0, 
6.0], [6.0, 7.0, 1.0]], [[1.0, 4.0, 1.0], [4.0, 5.0, 1.0], [1.0, 5.0, 3.0]], [[7.0, 3.0, 3.0], [6.0, 8.0, 3.0], [5.0, 8.0, 
8.0]]], [[[1.0, 4.0, 4.0], [2.0, 2.0, 7.0], [3.0, 3.0, 7.0]], [[8.0, 6.0, 6.0], [6.0, 3.0, 3.0], [5.0, 8.0, 3.0]], [[2.0, 4.0, 
5.0], [1.0, 4.0, 6.0], [8.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 4.0], [4.0, 8.0, 4.0], [7.0, 3.0, 7.0]], [[1.0, 8.0, 2.0], [8.0, 5.0, 
8.0], [1.0, 8.0, 7.0]], [[5.0, 1.0, 5.0], [1.0, 3.0, 6.0], [4.0, 7.0, 4.0]]], [[[3.0, 8.0, 8.0], [3.0, 3.0, 4.0], [6.0, 8.0, 
2.0]], [[6.0, 3.0, 6.0], [7.0, 5.0, 1.0], [6.0, 5.0, 5.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 5.0], [5.0, 2.0, 8.0]]], [[[3.0, 8.0, 
8.0], [8.0, 1.0, 5.0], [7.0, 8.0, 8.0]], [[6.0, 2.0, 8.0], [5.0, 5.0, 6.0], [5.0, 8.0, 6.0]], [[7.0, 2.0, 5.0], [3.0, 3.0, 
8.0], [4.0, 5.0, 2.0]]], [[[1.0, 4.0, 3.0], [3.0, 1.0, 6.0], [7.0, 4.0, 1.0]], [[4.0, 3.0, 1.0], [5.0, 5.0, 7.0], [2.0, 6.0, 
7.0]], [[3.0, 5.0, 3.0], [4.0, 8.0, 5.0], [6.0, 3.0, 5.0]]]]))
      val=numpy.array([[[[4.0], [1.0], [7.0]], [[8.0], [4.0], [4.0]]], [[[1.0], [4.0], [5.0]], [[3.0], [1.0], [8.0]]], [[[1.0], 
[7.0], [4.0]], [[8.0], [5.0], [6.0]]]])
      arg[:3,:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 4.0], [5.0, 8.0, 1.0], [6.0, 8.0, 7.0]], [[7.0, 4.0, 8.0], [4.0, 7.0, 4.0], [8.0, 
8.0, 4.0]], [[7.0, 5.0, 3.0], [8.0, 4.0, 7.0], [2.0, 1.0, 1.0]]], [[[7.0, 4.0, 1.0], [8.0, 4.0, 4.0], [2.0, 2.0, 5.0]], [[4.0, 
7.0, 3.0], [2.0, 2.0, 1.0], [5.0, 7.0, 8.0]], [[4.0, 3.0, 5.0], [8.0, 2.0, 5.0], [8.0, 6.0, 5.0]]], [[[7.0, 1.0, 1.0], [1.0, 
8.0, 7.0], [6.0, 7.0, 4.0]], [[1.0, 4.0, 8.0], [4.0, 5.0, 5.0], [1.0, 5.0, 6.0]], [[7.0, 3.0, 3.0], [6.0, 8.0, 3.0], [5.0, 8.0, 
8.0]]], [[[1.0, 4.0, 4.0], [2.0, 2.0, 7.0], [3.0, 3.0, 7.0]], [[8.0, 6.0, 6.0], [6.0, 3.0, 3.0], [5.0, 8.0, 3.0]], [[2.0, 4.0, 
5.0], [1.0, 4.0, 6.0], [8.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 4.0], [4.0, 8.0, 1.0], [7.0, 3.0, 7.0]], 
[[1.0, 8.0, 8.0], [8.0, 5.0, 4.0], [1.0, 8.0, 4.0]], [[5.0, 1.0, 5.0], [1.0, 3.0, 6.0], [4.0, 7.0, 4.0]]], [[[3.0, 8.0, 1.0], 
[3.0, 3.0, 4.0], [6.0, 8.0, 5.0]], [[6.0, 3.0, 3.0], [7.0, 5.0, 1.0], [6.0, 5.0, 8.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 5.0], 
[5.0, 2.0, 8.0]]], [[[3.0, 8.0, 1.0], [8.0, 1.0, 7.0], [7.0, 8.0, 4.0]], [[6.0, 2.0, 8.0], [5.0, 5.0, 5.0], [5.0, 8.0, 6.0]], 
[[7.0, 2.0, 5.0], [3.0, 3.0, 8.0], [4.0, 5.0, 2.0]]], [[[1.0, 4.0, 3.0], [3.0, 1.0, 6.0], [7.0, 4.0, 1.0]], [[4.0, 3.0, 1.0], 
[5.0, 5.0, 7.0], [2.0, 6.0, 7.0]], [[3.0, 5.0, 3.0], [4.0, 8.0, 5.0], [6.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UUCL(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 1.0], [1.0, 3.0, 4.0], [8.0, 2.0, 4.0]], [[7.0, 3.0, 6.0], [1.0, 1.0, 1.0], [8.0, 3.0, 
8.0]], [[4.0, 6.0, 7.0], [6.0, 5.0, 5.0], [1.0, 8.0, 6.0]]], [[[7.0, 1.0, 4.0], [4.0, 7.0, 3.0], [1.0, 6.0, 8.0]], [[3.0, 4.0, 
3.0], [2.0, 1.0, 2.0], [1.0, 1.0, 2.0]], [[6.0, 6.0, 7.0], [8.0, 7.0, 7.0], [3.0, 7.0, 4.0]]], [[[7.0, 4.0, 3.0], [6.0, 4.0, 
2.0], [8.0, 2.0, 4.0]], [[5.0, 4.0, 2.0], [8.0, 7.0, 7.0], [8.0, 4.0, 5.0]], [[1.0, 6.0, 4.0], [3.0, 1.0, 8.0], [3.0, 1.0, 
3.0]]], [[[1.0, 4.0, 5.0], [1.0, 1.0, 3.0], [6.0, 5.0, 3.0]], [[2.0, 6.0, 7.0], [2.0, 1.0, 6.0], [7.0, 4.0, 6.0]], [[2.0, 6.0, 
6.0], [6.0, 6.0, 3.0], [7.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 7.0], [1.0, 7.0, 4.0], [7.0, 4.0, 5.0]], [[6.0, 4.0, 1.0], [3.0, 1.0, 
4.0], [2.0, 2.0, 5.0]], [[8.0, 7.0, 8.0], [6.0, 3.0, 8.0], [4.0, 6.0, 1.0]]], [[[5.0, 5.0, 5.0], [1.0, 7.0, 6.0], [8.0, 4.0, 
6.0]], [[4.0, 3.0, 4.0], [6.0, 6.0, 3.0], [6.0, 5.0, 6.0]], [[8.0, 4.0, 1.0], [1.0, 1.0, 1.0], [3.0, 7.0, 5.0]]], [[[1.0, 1.0, 
8.0], [2.0, 4.0, 3.0], [5.0, 6.0, 3.0]], [[8.0, 1.0, 3.0], [1.0, 3.0, 7.0], [3.0, 6.0, 5.0]], [[4.0, 1.0, 7.0], [1.0, 3.0, 
2.0], [1.0, 2.0, 3.0]]], [[[7.0, 4.0, 4.0], [2.0, 4.0, 2.0], [8.0, 1.0, 5.0]], [[5.0, 4.0, 8.0], [1.0, 8.0, 2.0], [6.0, 1.0, 
2.0]], [[1.0, 8.0, 7.0], [1.0, 7.0, 6.0], [4.0, 7.0, 3.0]]]]))
      val=Data(numpy.array([[[[2.0], [7.0], [8.0]], [[2.0], [7.0], [5.0]]], [[[2.0], [5.0], [8.0]], [[4.0], [7.0], [1.0]]], 
[[[1.0], [4.0], [1.0]], [[5.0], [5.0], [6.0]]]]),self.functionspace)
      arg[:3,:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 2.0], [1.0, 3.0, 7.0], [8.0, 2.0, 8.0]], [[7.0, 3.0, 2.0], [1.0, 1.0, 7.0], [8.0, 
3.0, 5.0]], [[4.0, 6.0, 7.0], [6.0, 5.0, 5.0], [1.0, 8.0, 6.0]]], [[[7.0, 1.0, 2.0], [4.0, 7.0, 5.0], [1.0, 6.0, 8.0]], [[3.0, 
4.0, 4.0], [2.0, 1.0, 7.0], [1.0, 1.0, 1.0]], [[6.0, 6.0, 7.0], [8.0, 7.0, 7.0], [3.0, 7.0, 4.0]]], [[[7.0, 4.0, 1.0], [6.0, 
4.0, 4.0], [8.0, 2.0, 1.0]], [[5.0, 4.0, 5.0], [8.0, 7.0, 5.0], [8.0, 4.0, 6.0]], [[1.0, 6.0, 4.0], [3.0, 1.0, 8.0], [3.0, 1.0, 
3.0]]], [[[1.0, 4.0, 5.0], [1.0, 1.0, 3.0], [6.0, 5.0, 3.0]], [[2.0, 6.0, 7.0], [2.0, 1.0, 6.0], [7.0, 4.0, 6.0]], [[2.0, 6.0, 
6.0], [6.0, 6.0, 3.0], [7.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 2.0], [1.0, 7.0, 7.0], [7.0, 4.0, 8.0]], 
[[6.0, 4.0, 2.0], [3.0, 1.0, 7.0], [2.0, 2.0, 5.0]], [[8.0, 7.0, 8.0], [6.0, 3.0, 8.0], [4.0, 6.0, 1.0]]], [[[5.0, 5.0, 2.0], 
[1.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[4.0, 3.0, 4.0], [6.0, 6.0, 7.0], [6.0, 5.0, 1.0]], [[8.0, 4.0, 1.0], [1.0, 1.0, 1.0], 
[3.0, 7.0, 5.0]]], [[[1.0, 1.0, 1.0], [2.0, 4.0, 4.0], [5.0, 6.0, 1.0]], [[8.0, 1.0, 5.0], [1.0, 3.0, 5.0], [3.0, 6.0, 6.0]], 
[[4.0, 1.0, 7.0], [1.0, 3.0, 2.0], [1.0, 2.0, 3.0]]], [[[7.0, 4.0, 4.0], [2.0, 4.0, 2.0], [8.0, 1.0, 5.0]], [[5.0, 4.0, 8.0], 
[1.0, 8.0, 2.0], [6.0, 1.0, 2.0]], [[1.0, 8.0, 7.0], [1.0, 7.0, 6.0], [4.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UUCL(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 8.0], [4.0, 1.0, 5.0], [1.0, 7.0, 7.0]], [[7.0, 4.0, 1.0], [1.0, 1.0, 2.0], [6.0, 7.0, 
8.0]], [[7.0, 6.0, 4.0], [3.0, 3.0, 3.0], [5.0, 2.0, 2.0]]], [[[2.0, 7.0, 4.0], [8.0, 1.0, 6.0], [4.0, 6.0, 2.0]], [[1.0, 4.0, 
6.0], [1.0, 3.0, 5.0], [3.0, 4.0, 1.0]], [[2.0, 2.0, 4.0], [3.0, 5.0, 7.0], [1.0, 3.0, 7.0]]], [[[4.0, 4.0, 8.0], [5.0, 2.0, 
7.0], [5.0, 7.0, 3.0]], [[5.0, 8.0, 3.0], [4.0, 1.0, 3.0], [5.0, 8.0, 8.0]], [[1.0, 3.0, 2.0], [3.0, 5.0, 7.0], [2.0, 5.0, 
3.0]]], [[[5.0, 2.0, 2.0], [8.0, 2.0, 3.0], [2.0, 1.0, 5.0]], [[7.0, 2.0, 5.0], [3.0, 8.0, 3.0], [7.0, 6.0, 5.0]], [[8.0, 6.0, 
7.0], [3.0, 6.0, 5.0], [6.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 2.0], [5.0, 5.0, 2.0], [7.0, 6.0, 2.0]], [[7.0, 4.0, 6.0], [4.0, 1.0, 
7.0], [4.0, 5.0, 6.0]], [[6.0, 3.0, 7.0], [8.0, 1.0, 6.0], [5.0, 6.0, 8.0]]], [[[3.0, 5.0, 8.0], [7.0, 3.0, 2.0], [6.0, 6.0, 
5.0]], [[6.0, 8.0, 3.0], [6.0, 1.0, 4.0], [1.0, 1.0, 6.0]], [[8.0, 1.0, 6.0], [3.0, 3.0, 1.0], [8.0, 2.0, 8.0]]], [[[1.0, 3.0, 
3.0], [5.0, 4.0, 8.0], [1.0, 3.0, 3.0]], [[1.0, 1.0, 2.0], [6.0, 7.0, 8.0], [3.0, 5.0, 5.0]], [[3.0, 3.0, 4.0], [1.0, 7.0, 
5.0], [2.0, 1.0, 1.0]]], [[[3.0, 5.0, 1.0], [3.0, 5.0, 2.0], [4.0, 5.0, 6.0]], [[4.0, 3.0, 4.0], [8.0, 8.0, 7.0], [1.0, 4.0, 
8.0]], [[6.0, 4.0, 7.0], [8.0, 3.0, 7.0], [6.0, 7.0, 6.0]]]]))
      val=Data(numpy.array([[[[5.0], [6.0], [8.0]], [[4.0], [7.0], [3.0]]], [[[3.0], [7.0], [2.0]], [[1.0], [7.0], [4.0]]], 
[[[3.0], [3.0], [5.0]], [[4.0], [1.0], [4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0], [6.0], [4.0]], [[7.0], [2.0], [1.0]]], [[[2.0], [3.0], [5.0]], [[8.0], [2.0], 
[7.0]]], [[[4.0], [4.0], [7.0]], [[1.0], [6.0], [4.0]]]]))
      arg[:3,:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 5.0], [4.0, 1.0, 6.0], [1.0, 7.0, 8.0]], [[7.0, 4.0, 4.0], [1.0, 1.0, 7.0], [6.0, 
7.0, 3.0]], [[7.0, 6.0, 4.0], [3.0, 3.0, 3.0], [5.0, 2.0, 2.0]]], [[[2.0, 7.0, 3.0], [8.0, 1.0, 7.0], [4.0, 6.0, 2.0]], [[1.0, 
4.0, 1.0], [1.0, 3.0, 7.0], [3.0, 4.0, 4.0]], [[2.0, 2.0, 4.0], [3.0, 5.0, 7.0], [1.0, 3.0, 7.0]]], [[[4.0, 4.0, 3.0], [5.0, 
2.0, 3.0], [5.0, 7.0, 5.0]], [[5.0, 8.0, 4.0], [4.0, 1.0, 1.0], [5.0, 8.0, 4.0]], [[1.0, 3.0, 2.0], [3.0, 5.0, 7.0], [2.0, 5.0, 
3.0]]], [[[5.0, 2.0, 2.0], [8.0, 2.0, 3.0], [2.0, 1.0, 5.0]], [[7.0, 2.0, 5.0], [3.0, 8.0, 3.0], [7.0, 6.0, 5.0]], [[8.0, 6.0, 
7.0], [3.0, 6.0, 5.0], [6.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 3.0], [5.0, 5.0, 6.0], [7.0, 6.0, 4.0]], 
[[7.0, 4.0, 7.0], [4.0, 1.0, 2.0], [4.0, 5.0, 1.0]], [[6.0, 3.0, 7.0], [8.0, 1.0, 6.0], [5.0, 6.0, 8.0]]], [[[3.0, 5.0, 2.0], 
[7.0, 3.0, 3.0], [6.0, 6.0, 5.0]], [[6.0, 8.0, 8.0], [6.0, 1.0, 2.0], [1.0, 1.0, 7.0]], [[8.0, 1.0, 6.0], [3.0, 3.0, 1.0], 
[8.0, 2.0, 8.0]]], [[[1.0, 3.0, 4.0], [5.0, 4.0, 4.0], [1.0, 3.0, 7.0]], [[1.0, 1.0, 1.0], [6.0, 7.0, 6.0], [3.0, 5.0, 4.0]], 
[[3.0, 3.0, 4.0], [1.0, 7.0, 5.0], [2.0, 1.0, 1.0]]], [[[3.0, 5.0, 1.0], [3.0, 5.0, 2.0], [4.0, 5.0, 6.0]], [[4.0, 3.0, 4.0], 
[8.0, 8.0, 7.0], [1.0, 4.0, 8.0]], [[6.0, 4.0, 7.0], [8.0, 3.0, 7.0], [6.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UUCL(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 6.0], [6.0, 7.0, 4.0], [4.0, 1.0, 7.0]], [[5.0, 8.0, 8.0], [1.0, 7.0, 3.0], [5.0, 3.0, 
5.0]], [[8.0, 4.0, 8.0], [1.0, 3.0, 5.0], [1.0, 7.0, 7.0]]], [[[1.0, 7.0, 3.0], [8.0, 3.0, 4.0], [1.0, 2.0, 5.0]], [[1.0, 5.0, 
2.0], [3.0, 1.0, 4.0], [1.0, 4.0, 5.0]], [[7.0, 1.0, 8.0], [2.0, 3.0, 3.0], [8.0, 4.0, 4.0]]], [[[7.0, 5.0, 8.0], [1.0, 7.0, 
3.0], [5.0, 7.0, 4.0]], [[4.0, 1.0, 2.0], [6.0, 4.0, 7.0], [5.0, 3.0, 3.0]], [[6.0, 8.0, 6.0], [1.0, 2.0, 5.0], [7.0, 6.0, 
6.0]]], [[[6.0, 3.0, 3.0], [8.0, 1.0, 2.0], [8.0, 8.0, 3.0]], [[3.0, 3.0, 6.0], [2.0, 3.0, 4.0], [8.0, 5.0, 4.0]], [[1.0, 1.0, 
5.0], [1.0, 6.0, 6.0], [6.0, 5.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 6.0], [1.0, 6.0, 1.0], [1.0, 7.0, 2.0]], [[6.0, 7.0, 2.0], [3.0, 3.0, 
2.0], [8.0, 3.0, 1.0]], [[1.0, 8.0, 1.0], [4.0, 5.0, 5.0], [6.0, 8.0, 5.0]]], [[[4.0, 5.0, 6.0], [2.0, 7.0, 5.0], [8.0, 4.0, 
8.0]], [[5.0, 1.0, 3.0], [5.0, 1.0, 2.0], [3.0, 4.0, 1.0]], [[7.0, 6.0, 4.0], [1.0, 2.0, 2.0], [4.0, 4.0, 4.0]]], [[[7.0, 6.0, 
5.0], [1.0, 8.0, 2.0], [1.0, 6.0, 3.0]], [[2.0, 4.0, 5.0], [1.0, 7.0, 8.0], [2.0, 8.0, 4.0]], [[5.0, 1.0, 7.0], [1.0, 6.0, 
2.0], [4.0, 3.0, 2.0]]], [[[2.0, 8.0, 1.0], [3.0, 1.0, 4.0], [5.0, 5.0, 2.0]], [[4.0, 6.0, 4.0], [4.0, 6.0, 4.0], [6.0, 1.0, 
5.0]], [[8.0, 1.0, 5.0], [5.0, 6.0, 2.0], [8.0, 1.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0], [1.0], [5.0]], [[4.0], [1.0], [5.0]]], [[[1.0], [8.0], [1.0]], [[7.0], [4.0], [7.0]]], 
[[[6.0], [6.0], [7.0]], [[7.0], [2.0], [2.0]]]])+(1.-msk_val)*numpy.array([[[[1.0], [6.0], [3.0]], [[6.0], [8.0], [3.0]]], 
[[[1.0], [7.0], [7.0]], [[1.0], [8.0], [2.0]]], [[[2.0], [1.0], [1.0]], [[6.0], [8.0], [1.0]]]])
      arg[:3,:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 4.0], [6.0, 7.0, 1.0], [4.0, 1.0, 5.0]], [[5.0, 8.0, 4.0], [1.0, 7.0, 1.0], [5.0, 
3.0, 5.0]], [[8.0, 4.0, 8.0], [1.0, 3.0, 5.0], [1.0, 7.0, 7.0]]], [[[1.0, 7.0, 1.0], [8.0, 3.0, 8.0], [1.0, 2.0, 1.0]], [[1.0, 
5.0, 7.0], [3.0, 1.0, 4.0], [1.0, 4.0, 7.0]], [[7.0, 1.0, 8.0], [2.0, 3.0, 3.0], [8.0, 4.0, 4.0]]], [[[7.0, 5.0, 6.0], [1.0, 
7.0, 6.0], [5.0, 7.0, 7.0]], [[4.0, 1.0, 7.0], [6.0, 4.0, 2.0], [5.0, 3.0, 2.0]], [[6.0, 8.0, 6.0], [1.0, 2.0, 5.0], [7.0, 6.0, 
6.0]]], [[[6.0, 3.0, 3.0], [8.0, 1.0, 2.0], [8.0, 8.0, 3.0]], [[3.0, 3.0, 6.0], [2.0, 3.0, 4.0], [8.0, 5.0, 4.0]], [[1.0, 1.0, 
5.0], [1.0, 6.0, 6.0], [6.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 1.0], [1.0, 6.0, 6.0], [1.0, 7.0, 3.0]], 
[[6.0, 7.0, 6.0], [3.0, 3.0, 8.0], [8.0, 3.0, 3.0]], [[1.0, 8.0, 1.0], [4.0, 5.0, 5.0], [6.0, 8.0, 5.0]]], [[[4.0, 5.0, 1.0], 
[2.0, 7.0, 7.0], [8.0, 4.0, 7.0]], [[5.0, 1.0, 1.0], [5.0, 1.0, 8.0], [3.0, 4.0, 2.0]], [[7.0, 6.0, 4.0], [1.0, 2.0, 2.0], 
[4.0, 4.0, 4.0]]], [[[7.0, 6.0, 2.0], [1.0, 8.0, 1.0], [1.0, 6.0, 1.0]], [[2.0, 4.0, 6.0], [1.0, 7.0, 8.0], [2.0, 8.0, 1.0]], 
[[5.0, 1.0, 7.0], [1.0, 6.0, 2.0], [4.0, 3.0, 2.0]]], [[[2.0, 8.0, 1.0], [3.0, 1.0, 4.0], [5.0, 5.0, 2.0]], [[4.0, 6.0, 4.0], 
[4.0, 6.0, 4.0], [6.0, 1.0, 5.0]], [[8.0, 1.0, 5.0], [5.0, 6.0, 2.0], [8.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRRR(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 8.0], [8.0, 8.0, 5.0], [6.0, 3.0, 6.0]], [[5.0, 1.0, 1.0], [5.0, 2.0, 3.0], [3.0, 6.0, 
8.0]], [[3.0, 1.0, 6.0], [8.0, 2.0, 8.0], [4.0, 5.0, 5.0]]], [[[5.0, 6.0, 5.0], [8.0, 1.0, 3.0], [8.0, 8.0, 4.0]], [[8.0, 5.0, 
2.0], [6.0, 6.0, 6.0], [7.0, 3.0, 7.0]], [[6.0, 6.0, 7.0], [8.0, 4.0, 1.0], [4.0, 4.0, 3.0]]], [[[1.0, 2.0, 3.0], [6.0, 2.0, 
6.0], [7.0, 3.0, 4.0]], [[5.0, 1.0, 5.0], [6.0, 5.0, 6.0], [7.0, 4.0, 5.0]], [[2.0, 1.0, 2.0], [5.0, 2.0, 7.0], [7.0, 1.0, 
5.0]]], [[[7.0, 6.0, 4.0], [3.0, 2.0, 6.0], [7.0, 8.0, 1.0]], [[7.0, 4.0, 6.0], [8.0, 3.0, 1.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 
2.0], [3.0, 1.0, 2.0], [1.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 4.0], [2.0, 8.0, 2.0], [6.0, 7.0, 5.0]], [[6.0, 4.0, 7.0], [2.0, 4.0, 
4.0], [4.0, 8.0, 6.0]], [[5.0, 8.0, 7.0], [4.0, 5.0, 7.0], [5.0, 7.0, 1.0]]], [[[6.0, 5.0, 3.0], [5.0, 3.0, 6.0], [7.0, 8.0, 
1.0]], [[6.0, 5.0, 7.0], [7.0, 2.0, 7.0], [4.0, 7.0, 2.0]], [[2.0, 2.0, 1.0], [8.0, 2.0, 3.0], [2.0, 6.0, 4.0]]], [[[3.0, 7.0, 
3.0], [8.0, 1.0, 5.0], [4.0, 5.0, 5.0]], [[6.0, 1.0, 5.0], [6.0, 2.0, 5.0], [1.0, 6.0, 6.0]], [[1.0, 5.0, 7.0], [6.0, 5.0, 
8.0], [4.0, 6.0, 8.0]]], [[[8.0, 6.0, 5.0], [3.0, 4.0, 1.0], [5.0, 2.0, 5.0]], [[6.0, 3.0, 8.0], [1.0, 3.0, 5.0], [3.0, 8.0, 
3.0]], [[8.0, 5.0, 1.0], [2.0, 5.0, 2.0], [7.0, 5.0, 3.0]]]]))
      val=numpy.array([[[[7.0]]], [[[1.0]]]])
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 8.0], [8.0, 8.0, 5.0], [6.0, 3.0, 6.0]], [[5.0, 1.0, 1.0], [5.0, 2.0, 3.0], [3.0, 
6.0, 8.0]], [[3.0, 1.0, 6.0], [8.0, 2.0, 8.0], [4.0, 5.0, 5.0]]], [[[5.0, 6.0, 5.0], [8.0, 1.0, 3.0], [8.0, 8.0, 4.0]], [[8.0, 
5.0, 2.0], [6.0, 7.0, 6.0], [7.0, 3.0, 7.0]], [[6.0, 6.0, 7.0], [8.0, 4.0, 1.0], [4.0, 4.0, 3.0]]], [[[1.0, 2.0, 3.0], [6.0, 
2.0, 6.0], [7.0, 3.0, 4.0]], [[5.0, 1.0, 5.0], [6.0, 1.0, 6.0], [7.0, 4.0, 5.0]], [[2.0, 1.0, 2.0], [5.0, 2.0, 7.0], [7.0, 1.0, 
5.0]]], [[[7.0, 6.0, 4.0], [3.0, 2.0, 6.0], [7.0, 8.0, 1.0]], [[7.0, 4.0, 6.0], [8.0, 3.0, 1.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 
2.0], [3.0, 1.0, 2.0], [1.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 4.0], [2.0, 8.0, 2.0], [6.0, 7.0, 5.0]], 
[[6.0, 4.0, 7.0], [2.0, 4.0, 4.0], [4.0, 8.0, 6.0]], [[5.0, 8.0, 7.0], [4.0, 5.0, 7.0], [5.0, 7.0, 1.0]]], [[[6.0, 5.0, 3.0], 
[5.0, 3.0, 6.0], [7.0, 8.0, 1.0]], [[6.0, 5.0, 7.0], [7.0, 7.0, 7.0], [4.0, 7.0, 2.0]], [[2.0, 2.0, 1.0], [8.0, 2.0, 3.0], 
[2.0, 6.0, 4.0]]], [[[3.0, 7.0, 3.0], [8.0, 1.0, 5.0], [4.0, 5.0, 5.0]], [[6.0, 1.0, 5.0], [6.0, 1.0, 5.0], [1.0, 6.0, 6.0]], 
[[1.0, 5.0, 7.0], [6.0, 5.0, 8.0], [4.0, 6.0, 8.0]]], [[[8.0, 6.0, 5.0], [3.0, 4.0, 1.0], [5.0, 2.0, 5.0]], [[6.0, 3.0, 8.0], 
[1.0, 3.0, 5.0], [3.0, 8.0, 3.0]], [[8.0, 5.0, 1.0], [2.0, 5.0, 2.0], [7.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRRR(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 6.0], [4.0, 8.0, 6.0], [3.0, 3.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 5.0, 7.0], [6.0, 5.0, 
1.0]], [[6.0, 5.0, 5.0], [5.0, 7.0, 7.0], [4.0, 1.0, 6.0]]], [[[3.0, 8.0, 6.0], [8.0, 3.0, 5.0], [4.0, 8.0, 5.0]], [[2.0, 3.0, 
2.0], [8.0, 1.0, 2.0], [3.0, 3.0, 4.0]], [[1.0, 6.0, 6.0], [1.0, 2.0, 2.0], [7.0, 6.0, 5.0]]], [[[8.0, 5.0, 2.0], [1.0, 8.0, 
8.0], [5.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [4.0, 2.0, 7.0], [3.0, 5.0, 4.0]], [[5.0, 7.0, 1.0], [8.0, 2.0, 5.0], [3.0, 1.0, 
7.0]]], [[[2.0, 2.0, 7.0], [1.0, 7.0, 3.0], [4.0, 4.0, 8.0]], [[5.0, 5.0, 5.0], [7.0, 2.0, 7.0], [2.0, 3.0, 2.0]], [[3.0, 7.0, 
5.0], [8.0, 5.0, 4.0], [4.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 3.0], [1.0, 7.0, 5.0], [7.0, 4.0, 3.0]], [[4.0, 7.0, 5.0], [1.0, 1.0, 
7.0], [2.0, 3.0, 1.0]], [[1.0, 7.0, 8.0], [5.0, 5.0, 1.0], [4.0, 4.0, 7.0]]], [[[1.0, 2.0, 8.0], [8.0, 3.0, 2.0], [3.0, 2.0, 
7.0]], [[1.0, 4.0, 1.0], [7.0, 6.0, 6.0], [5.0, 4.0, 2.0]], [[5.0, 3.0, 5.0], [7.0, 4.0, 7.0], [1.0, 2.0, 1.0]]], [[[2.0, 7.0, 
1.0], [5.0, 8.0, 2.0], [1.0, 7.0, 8.0]], [[3.0, 6.0, 7.0], [4.0, 4.0, 5.0], [3.0, 7.0, 8.0]], [[8.0, 7.0, 7.0], [1.0, 2.0, 
2.0], [6.0, 4.0, 3.0]]], [[[1.0, 7.0, 1.0], [7.0, 1.0, 7.0], [6.0, 6.0, 7.0]], [[5.0, 3.0, 4.0], [7.0, 4.0, 7.0], [4.0, 3.0, 
7.0]], [[3.0, 1.0, 6.0], [7.0, 4.0, 1.0], [7.0, 7.0, 5.0]]]]))
      val=Data(numpy.array([[[[3.0]]], [[[6.0]]]]),self.functionspace)
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 6.0], [4.0, 8.0, 6.0], [3.0, 3.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 5.0, 7.0], [6.0, 
5.0, 1.0]], [[6.0, 5.0, 5.0], [5.0, 7.0, 7.0], [4.0, 1.0, 6.0]]], [[[3.0, 8.0, 6.0], [8.0, 3.0, 5.0], [4.0, 8.0, 5.0]], [[2.0, 
3.0, 2.0], [8.0, 3.0, 2.0], [3.0, 3.0, 4.0]], [[1.0, 6.0, 6.0], [1.0, 2.0, 2.0], [7.0, 6.0, 5.0]]], [[[8.0, 5.0, 2.0], [1.0, 
8.0, 8.0], [5.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [4.0, 6.0, 7.0], [3.0, 5.0, 4.0]], [[5.0, 7.0, 1.0], [8.0, 2.0, 5.0], [3.0, 1.0, 
7.0]]], [[[2.0, 2.0, 7.0], [1.0, 7.0, 3.0], [4.0, 4.0, 8.0]], [[5.0, 5.0, 5.0], [7.0, 2.0, 7.0], [2.0, 3.0, 2.0]], [[3.0, 7.0, 
5.0], [8.0, 5.0, 4.0], [4.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 3.0], [1.0, 7.0, 5.0], [7.0, 4.0, 3.0]], 
[[4.0, 7.0, 5.0], [1.0, 1.0, 7.0], [2.0, 3.0, 1.0]], [[1.0, 7.0, 8.0], [5.0, 5.0, 1.0], [4.0, 4.0, 7.0]]], [[[1.0, 2.0, 8.0], 
[8.0, 3.0, 2.0], [3.0, 2.0, 7.0]], [[1.0, 4.0, 1.0], [7.0, 3.0, 6.0], [5.0, 4.0, 2.0]], [[5.0, 3.0, 5.0], [7.0, 4.0, 7.0], 
[1.0, 2.0, 1.0]]], [[[2.0, 7.0, 1.0], [5.0, 8.0, 2.0], [1.0, 7.0, 8.0]], [[3.0, 6.0, 7.0], [4.0, 6.0, 5.0], [3.0, 7.0, 8.0]], 
[[8.0, 7.0, 7.0], [1.0, 2.0, 2.0], [6.0, 4.0, 3.0]]], [[[1.0, 7.0, 1.0], [7.0, 1.0, 7.0], [6.0, 6.0, 7.0]], [[5.0, 3.0, 4.0], 
[7.0, 4.0, 7.0], [4.0, 3.0, 7.0]], [[3.0, 1.0, 6.0], [7.0, 4.0, 1.0], [7.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRRR(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 8.0], [6.0, 5.0, 2.0], [2.0, 3.0, 4.0]], [[5.0, 2.0, 8.0], [7.0, 3.0, 4.0], [2.0, 3.0, 
2.0]], [[6.0, 2.0, 2.0], [6.0, 1.0, 8.0], [3.0, 1.0, 8.0]]], [[[1.0, 8.0, 5.0], [7.0, 7.0, 1.0], [3.0, 7.0, 6.0]], [[4.0, 2.0, 
8.0], [4.0, 1.0, 8.0], [3.0, 8.0, 1.0]], [[5.0, 4.0, 2.0], [2.0, 6.0, 6.0], [4.0, 4.0, 6.0]]], [[[6.0, 8.0, 5.0], [6.0, 6.0, 
5.0], [7.0, 8.0, 3.0]], [[3.0, 2.0, 5.0], [2.0, 3.0, 3.0], [1.0, 7.0, 7.0]], [[1.0, 3.0, 7.0], [2.0, 8.0, 3.0], [6.0, 5.0, 
3.0]]], [[[1.0, 8.0, 8.0], [8.0, 3.0, 6.0], [7.0, 4.0, 2.0]], [[7.0, 8.0, 4.0], [5.0, 5.0, 4.0], [8.0, 6.0, 6.0]], [[1.0, 6.0, 
4.0], [5.0, 8.0, 8.0], [4.0, 2.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 6.0], [3.0, 1.0, 4.0], [5.0, 6.0, 1.0]], [[5.0, 6.0, 2.0], [4.0, 8.0, 
1.0], [1.0, 3.0, 1.0]], [[8.0, 3.0, 2.0], [5.0, 8.0, 4.0], [4.0, 2.0, 1.0]]], [[[1.0, 1.0, 6.0], [6.0, 6.0, 1.0], [3.0, 1.0, 
4.0]], [[5.0, 7.0, 4.0], [3.0, 3.0, 3.0], [7.0, 2.0, 6.0]], [[5.0, 7.0, 6.0], [3.0, 1.0, 7.0], [1.0, 4.0, 6.0]]], [[[7.0, 2.0, 
5.0], [4.0, 4.0, 8.0], [8.0, 1.0, 1.0]], [[1.0, 8.0, 8.0], [5.0, 7.0, 4.0], [1.0, 6.0, 2.0]], [[7.0, 5.0, 3.0], [7.0, 6.0, 
8.0], [2.0, 4.0, 4.0]]], [[[7.0, 1.0, 1.0], [1.0, 7.0, 3.0], [4.0, 3.0, 8.0]], [[3.0, 7.0, 6.0], [4.0, 5.0, 3.0], [3.0, 8.0, 
6.0]], [[2.0, 6.0, 6.0], [3.0, 2.0, 6.0], [4.0, 6.0, 3.0]]]]))
      val=Data(numpy.array([[[[7.0]]], [[[7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0]]], [[[3.0]]]]))
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 8.0], [6.0, 5.0, 2.0], [2.0, 3.0, 4.0]], [[5.0, 2.0, 8.0], [7.0, 3.0, 4.0], [2.0, 
3.0, 2.0]], [[6.0, 2.0, 2.0], [6.0, 1.0, 8.0], [3.0, 1.0, 8.0]]], [[[1.0, 8.0, 5.0], [7.0, 7.0, 1.0], [3.0, 7.0, 6.0]], [[4.0, 
2.0, 8.0], [4.0, 7.0, 8.0], [3.0, 8.0, 1.0]], [[5.0, 4.0, 2.0], [2.0, 6.0, 6.0], [4.0, 4.0, 6.0]]], [[[6.0, 8.0, 5.0], [6.0, 
6.0, 5.0], [7.0, 8.0, 3.0]], [[3.0, 2.0, 5.0], [2.0, 7.0, 3.0], [1.0, 7.0, 7.0]], [[1.0, 3.0, 7.0], [2.0, 8.0, 3.0], [6.0, 5.0, 
3.0]]], [[[1.0, 8.0, 8.0], [8.0, 3.0, 6.0], [7.0, 4.0, 2.0]], [[7.0, 8.0, 4.0], [5.0, 5.0, 4.0], [8.0, 6.0, 6.0]], [[1.0, 6.0, 
4.0], [5.0, 8.0, 8.0], [4.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 6.0], [3.0, 1.0, 4.0], [5.0, 6.0, 1.0]], 
[[5.0, 6.0, 2.0], [4.0, 8.0, 1.0], [1.0, 3.0, 1.0]], [[8.0, 3.0, 2.0], [5.0, 8.0, 4.0], [4.0, 2.0, 1.0]]], [[[1.0, 1.0, 6.0], 
[6.0, 6.0, 1.0], [3.0, 1.0, 4.0]], [[5.0, 7.0, 4.0], [3.0, 4.0, 3.0], [7.0, 2.0, 6.0]], [[5.0, 7.0, 6.0], [3.0, 1.0, 7.0], 
[1.0, 4.0, 6.0]]], [[[7.0, 2.0, 5.0], [4.0, 4.0, 8.0], [8.0, 1.0, 1.0]], [[1.0, 8.0, 8.0], [5.0, 3.0, 4.0], [1.0, 6.0, 2.0]], 
[[7.0, 5.0, 3.0], [7.0, 6.0, 8.0], [2.0, 4.0, 4.0]]], [[[7.0, 1.0, 1.0], [1.0, 7.0, 3.0], [4.0, 3.0, 8.0]], [[3.0, 7.0, 6.0], 
[4.0, 5.0, 3.0], [3.0, 8.0, 6.0]], [[2.0, 6.0, 6.0], [3.0, 2.0, 6.0], [4.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRRR(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 4.0], [2.0, 4.0, 3.0], [1.0, 6.0, 8.0]], [[1.0, 3.0, 5.0], [7.0, 6.0, 2.0], [7.0, 4.0, 
7.0]], [[6.0, 2.0, 7.0], [5.0, 7.0, 8.0], [3.0, 7.0, 3.0]]], [[[4.0, 8.0, 4.0], [2.0, 5.0, 5.0], [7.0, 4.0, 5.0]], [[8.0, 1.0, 
3.0], [7.0, 7.0, 6.0], [5.0, 7.0, 7.0]], [[5.0, 8.0, 8.0], [6.0, 1.0, 3.0], [5.0, 4.0, 3.0]]], [[[6.0, 7.0, 3.0], [4.0, 5.0, 
3.0], [4.0, 8.0, 8.0]], [[3.0, 4.0, 6.0], [2.0, 4.0, 4.0], [3.0, 8.0, 3.0]], [[5.0, 6.0, 8.0], [3.0, 8.0, 4.0], [8.0, 6.0, 
7.0]]], [[[7.0, 4.0, 7.0], [7.0, 4.0, 5.0], [8.0, 1.0, 2.0]], [[1.0, 7.0, 5.0], [7.0, 5.0, 8.0], [6.0, 4.0, 1.0]], [[3.0, 7.0, 
8.0], [7.0, 1.0, 1.0], [2.0, 5.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 2.0], [4.0, 3.0, 5.0], [4.0, 8.0, 7.0]], [[7.0, 1.0, 2.0], [4.0, 7.0, 
6.0], [1.0, 1.0, 4.0]], [[4.0, 5.0, 7.0], [3.0, 4.0, 1.0], [4.0, 2.0, 8.0]]], [[[1.0, 1.0, 7.0], [2.0, 1.0, 1.0], [4.0, 1.0, 
6.0]], [[3.0, 8.0, 2.0], [8.0, 6.0, 4.0], [7.0, 3.0, 4.0]], [[3.0, 7.0, 7.0], [5.0, 1.0, 6.0], [6.0, 1.0, 5.0]]], [[[5.0, 6.0, 
1.0], [6.0, 6.0, 7.0], [6.0, 2.0, 8.0]], [[3.0, 2.0, 5.0], [6.0, 4.0, 6.0], [7.0, 6.0, 5.0]], [[1.0, 3.0, 5.0], [7.0, 7.0, 
8.0], [4.0, 7.0, 5.0]]], [[[7.0, 7.0, 1.0], [4.0, 1.0, 7.0], [3.0, 3.0, 7.0]], [[3.0, 7.0, 1.0], [1.0, 2.0, 3.0], [3.0, 5.0, 
4.0]], [[6.0, 7.0, 4.0], [6.0, 5.0, 7.0], [1.0, 8.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0]]], [[[2.0]]]])+(1.-msk_val)*numpy.array([[[[8.0]]], [[[3.0]]]])
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 4.0], [2.0, 4.0, 3.0], [1.0, 6.0, 8.0]], [[1.0, 3.0, 5.0], [7.0, 6.0, 2.0], [7.0, 
4.0, 7.0]], [[6.0, 2.0, 7.0], [5.0, 7.0, 8.0], [3.0, 7.0, 3.0]]], [[[4.0, 8.0, 4.0], [2.0, 5.0, 5.0], [7.0, 4.0, 5.0]], [[8.0, 
1.0, 3.0], [7.0, 7.0, 6.0], [5.0, 7.0, 7.0]], [[5.0, 8.0, 8.0], [6.0, 1.0, 3.0], [5.0, 4.0, 3.0]]], [[[6.0, 7.0, 3.0], [4.0, 
5.0, 3.0], [4.0, 8.0, 8.0]], [[3.0, 4.0, 6.0], [2.0, 2.0, 4.0], [3.0, 8.0, 3.0]], [[5.0, 6.0, 8.0], [3.0, 8.0, 4.0], [8.0, 6.0, 
7.0]]], [[[7.0, 4.0, 7.0], [7.0, 4.0, 5.0], [8.0, 1.0, 2.0]], [[1.0, 7.0, 5.0], [7.0, 5.0, 8.0], [6.0, 4.0, 1.0]], [[3.0, 7.0, 
8.0], [7.0, 1.0, 1.0], [2.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 2.0], [4.0, 3.0, 5.0], [4.0, 8.0, 7.0]], 
[[7.0, 1.0, 2.0], [4.0, 7.0, 6.0], [1.0, 1.0, 4.0]], [[4.0, 5.0, 7.0], [3.0, 4.0, 1.0], [4.0, 2.0, 8.0]]], [[[1.0, 1.0, 7.0], 
[2.0, 1.0, 1.0], [4.0, 1.0, 6.0]], [[3.0, 8.0, 2.0], [8.0, 8.0, 4.0], [7.0, 3.0, 4.0]], [[3.0, 7.0, 7.0], [5.0, 1.0, 6.0], 
[6.0, 1.0, 5.0]]], [[[5.0, 6.0, 1.0], [6.0, 6.0, 7.0], [6.0, 2.0, 8.0]], [[3.0, 2.0, 5.0], [6.0, 3.0, 6.0], [7.0, 6.0, 5.0]], 
[[1.0, 3.0, 5.0], [7.0, 7.0, 8.0], [4.0, 7.0, 5.0]]], [[[7.0, 7.0, 1.0], [4.0, 1.0, 7.0], [3.0, 3.0, 7.0]], [[3.0, 7.0, 1.0], 
[1.0, 2.0, 3.0], [3.0, 5.0, 4.0]], [[6.0, 7.0, 4.0], [6.0, 5.0, 7.0], [1.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPPL(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 4.0], [4.0, 4.0, 8.0], [5.0, 8.0, 1.0]], [[5.0, 7.0, 5.0], [8.0, 2.0, 2.0], [2.0, 5.0, 
4.0]], [[5.0, 1.0, 4.0], [5.0, 4.0, 2.0], [2.0, 4.0, 8.0]]], [[[1.0, 6.0, 1.0], [3.0, 5.0, 5.0], [8.0, 7.0, 5.0]], [[4.0, 1.0, 
8.0], [2.0, 1.0, 4.0], [1.0, 7.0, 3.0]], [[7.0, 2.0, 3.0], [4.0, 1.0, 2.0], [4.0, 7.0, 7.0]]], [[[2.0, 2.0, 5.0], [4.0, 1.0, 
8.0], [3.0, 7.0, 4.0]], [[6.0, 5.0, 4.0], [4.0, 1.0, 1.0], [2.0, 2.0, 3.0]], [[1.0, 2.0, 2.0], [3.0, 1.0, 5.0], [1.0, 2.0, 
2.0]]], [[[8.0, 1.0, 3.0], [7.0, 1.0, 5.0], [7.0, 5.0, 8.0]], [[4.0, 4.0, 6.0], [2.0, 8.0, 8.0], [6.0, 8.0, 1.0]], [[7.0, 1.0, 
5.0], [5.0, 3.0, 4.0], [5.0, 3.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 6.0], [5.0, 7.0, 1.0], [3.0, 5.0, 5.0]], [[1.0, 2.0, 5.0], [1.0, 4.0, 
3.0], [6.0, 3.0, 1.0]], [[3.0, 5.0, 7.0], [2.0, 2.0, 5.0], [7.0, 8.0, 8.0]]], [[[3.0, 5.0, 4.0], [6.0, 5.0, 3.0], [7.0, 7.0, 
2.0]], [[8.0, 8.0, 8.0], [5.0, 8.0, 3.0], [8.0, 1.0, 7.0]], [[1.0, 7.0, 3.0], [7.0, 4.0, 4.0], [2.0, 7.0, 3.0]]], [[[8.0, 3.0, 
5.0], [7.0, 7.0, 8.0], [1.0, 4.0, 3.0]], [[2.0, 6.0, 5.0], [8.0, 4.0, 5.0], [2.0, 3.0, 2.0]], [[8.0, 3.0, 1.0], [6.0, 8.0, 
8.0], [1.0, 8.0, 3.0]]], [[[6.0, 3.0, 8.0], [2.0, 4.0, 2.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 6.0], [5.0, 4.0, 6.0], [1.0, 6.0, 
1.0]], [[5.0, 8.0, 8.0], [2.0, 3.0, 1.0], [6.0, 1.0, 5.0]]]]))
      val=numpy.array([[8.0], [4.0]])
      arg[1:3,1,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 4.0], [4.0, 4.0, 8.0], [5.0, 8.0, 1.0]], [[5.0, 7.0, 5.0], [8.0, 2.0, 2.0], [2.0, 
5.0, 4.0]], [[5.0, 1.0, 4.0], [5.0, 4.0, 2.0], [2.0, 4.0, 8.0]]], [[[1.0, 6.0, 1.0], [3.0, 5.0, 5.0], [8.0, 7.0, 5.0]], [[4.0, 
1.0, 8.0], [2.0, 1.0, 8.0], [1.0, 7.0, 3.0]], [[7.0, 2.0, 3.0], [4.0, 1.0, 2.0], [4.0, 7.0, 7.0]]], [[[2.0, 2.0, 5.0], [4.0, 
1.0, 8.0], [3.0, 7.0, 4.0]], [[6.0, 5.0, 4.0], [4.0, 1.0, 4.0], [2.0, 2.0, 3.0]], [[1.0, 2.0, 2.0], [3.0, 1.0, 5.0], [1.0, 2.0, 
2.0]]], [[[8.0, 1.0, 3.0], [7.0, 1.0, 5.0], [7.0, 5.0, 8.0]], [[4.0, 4.0, 6.0], [2.0, 8.0, 8.0], [6.0, 8.0, 1.0]], [[7.0, 1.0, 
5.0], [5.0, 3.0, 4.0], [5.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 6.0], [5.0, 7.0, 1.0], [3.0, 5.0, 5.0]], 
[[1.0, 2.0, 5.0], [1.0, 4.0, 3.0], [6.0, 3.0, 1.0]], [[3.0, 5.0, 7.0], [2.0, 2.0, 5.0], [7.0, 8.0, 8.0]]], [[[3.0, 5.0, 4.0], 
[6.0, 5.0, 3.0], [7.0, 7.0, 2.0]], [[8.0, 8.0, 8.0], [5.0, 8.0, 8.0], [8.0, 1.0, 7.0]], [[1.0, 7.0, 3.0], [7.0, 4.0, 4.0], 
[2.0, 7.0, 3.0]]], [[[8.0, 3.0, 5.0], [7.0, 7.0, 8.0], [1.0, 4.0, 3.0]], [[2.0, 6.0, 5.0], [8.0, 4.0, 4.0], [2.0, 3.0, 2.0]], 
[[8.0, 3.0, 1.0], [6.0, 8.0, 8.0], [1.0, 8.0, 3.0]]], [[[6.0, 3.0, 8.0], [2.0, 4.0, 2.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 6.0], 
[5.0, 4.0, 6.0], [1.0, 6.0, 1.0]], [[5.0, 8.0, 8.0], [2.0, 3.0, 1.0], [6.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPPL(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 5.0], [1.0, 5.0, 2.0], [5.0, 6.0, 5.0]], [[8.0, 2.0, 4.0], [3.0, 2.0, 4.0], [8.0, 3.0, 
1.0]], [[1.0, 8.0, 4.0], [2.0, 5.0, 1.0], [4.0, 2.0, 8.0]]], [[[4.0, 8.0, 6.0], [6.0, 8.0, 1.0], [8.0, 5.0, 5.0]], [[1.0, 1.0, 
2.0], [2.0, 2.0, 4.0], [2.0, 7.0, 3.0]], [[8.0, 5.0, 8.0], [3.0, 2.0, 3.0], [2.0, 4.0, 2.0]]], [[[7.0, 2.0, 3.0], [1.0, 2.0, 
8.0], [6.0, 3.0, 4.0]], [[2.0, 1.0, 7.0], [1.0, 8.0, 8.0], [2.0, 8.0, 7.0]], [[6.0, 2.0, 7.0], [3.0, 6.0, 4.0], [6.0, 6.0, 
3.0]]], [[[1.0, 5.0, 4.0], [2.0, 1.0, 4.0], [7.0, 7.0, 6.0]], [[7.0, 4.0, 4.0], [2.0, 7.0, 4.0], [2.0, 1.0, 1.0]], [[5.0, 2.0, 
8.0], [1.0, 1.0, 6.0], [5.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 7.0], [6.0, 3.0, 7.0], [6.0, 8.0, 6.0]], [[8.0, 1.0, 3.0], [5.0, 1.0, 
8.0], [7.0, 8.0, 7.0]], [[1.0, 4.0, 5.0], [1.0, 4.0, 1.0], [5.0, 4.0, 5.0]]], [[[3.0, 1.0, 8.0], [8.0, 7.0, 8.0], [7.0, 7.0, 
3.0]], [[4.0, 2.0, 4.0], [5.0, 4.0, 4.0], [7.0, 5.0, 7.0]], [[8.0, 6.0, 4.0], [3.0, 7.0, 8.0], [2.0, 3.0, 2.0]]], [[[8.0, 6.0, 
5.0], [6.0, 3.0, 8.0], [7.0, 8.0, 4.0]], [[4.0, 4.0, 4.0], [4.0, 7.0, 3.0], [1.0, 4.0, 4.0]], [[3.0, 6.0, 3.0], [3.0, 7.0, 
7.0], [1.0, 5.0, 7.0]]], [[[1.0, 8.0, 6.0], [5.0, 2.0, 6.0], [5.0, 8.0, 3.0]], [[1.0, 8.0, 7.0], [3.0, 2.0, 5.0], [7.0, 8.0, 
3.0]], [[2.0, 2.0, 5.0], [1.0, 6.0, 1.0], [1.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[1.0], [6.0]]),self.functionspace)
      arg[1:3,1,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 5.0], [1.0, 5.0, 2.0], [5.0, 6.0, 5.0]], [[8.0, 2.0, 4.0], [3.0, 2.0, 4.0], [8.0, 
3.0, 1.0]], [[1.0, 8.0, 4.0], [2.0, 5.0, 1.0], [4.0, 2.0, 8.0]]], [[[4.0, 8.0, 6.0], [6.0, 8.0, 1.0], [8.0, 5.0, 5.0]], [[1.0, 
1.0, 2.0], [2.0, 2.0, 1.0], [2.0, 7.0, 3.0]], [[8.0, 5.0, 8.0], [3.0, 2.0, 3.0], [2.0, 4.0, 2.0]]], [[[7.0, 2.0, 3.0], [1.0, 
2.0, 8.0], [6.0, 3.0, 4.0]], [[2.0, 1.0, 7.0], [1.0, 8.0, 6.0], [2.0, 8.0, 7.0]], [[6.0, 2.0, 7.0], [3.0, 6.0, 4.0], [6.0, 6.0, 
3.0]]], [[[1.0, 5.0, 4.0], [2.0, 1.0, 4.0], [7.0, 7.0, 6.0]], [[7.0, 4.0, 4.0], [2.0, 7.0, 4.0], [2.0, 1.0, 1.0]], [[5.0, 2.0, 
8.0], [1.0, 1.0, 6.0], [5.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 7.0], [6.0, 3.0, 7.0], [6.0, 8.0, 6.0]], 
[[8.0, 1.0, 3.0], [5.0, 1.0, 8.0], [7.0, 8.0, 7.0]], [[1.0, 4.0, 5.0], [1.0, 4.0, 1.0], [5.0, 4.0, 5.0]]], [[[3.0, 1.0, 8.0], 
[8.0, 7.0, 8.0], [7.0, 7.0, 3.0]], [[4.0, 2.0, 4.0], [5.0, 4.0, 1.0], [7.0, 5.0, 7.0]], [[8.0, 6.0, 4.0], [3.0, 7.0, 8.0], 
[2.0, 3.0, 2.0]]], [[[8.0, 6.0, 5.0], [6.0, 3.0, 8.0], [7.0, 8.0, 4.0]], [[4.0, 4.0, 4.0], [4.0, 7.0, 6.0], [1.0, 4.0, 4.0]], 
[[3.0, 6.0, 3.0], [3.0, 7.0, 7.0], [1.0, 5.0, 7.0]]], [[[1.0, 8.0, 6.0], [5.0, 2.0, 6.0], [5.0, 8.0, 3.0]], [[1.0, 8.0, 7.0], 
[3.0, 2.0, 5.0], [7.0, 8.0, 3.0]], [[2.0, 2.0, 5.0], [1.0, 6.0, 1.0], [1.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPPL(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 1.0], [5.0, 7.0, 6.0], [8.0, 8.0, 8.0]], [[3.0, 5.0, 2.0], [3.0, 3.0, 2.0], [3.0, 7.0, 
1.0]], [[6.0, 3.0, 1.0], [7.0, 7.0, 4.0], [3.0, 4.0, 1.0]]], [[[5.0, 3.0, 1.0], [3.0, 6.0, 4.0], [2.0, 6.0, 5.0]], [[1.0, 2.0, 
6.0], [7.0, 5.0, 7.0], [4.0, 3.0, 1.0]], [[4.0, 7.0, 5.0], [5.0, 8.0, 4.0], [4.0, 7.0, 3.0]]], [[[3.0, 7.0, 3.0], [2.0, 1.0, 
5.0], [6.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], [3.0, 8.0, 7.0], [5.0, 3.0, 4.0]], [[5.0, 3.0, 6.0], [4.0, 6.0, 2.0], [2.0, 6.0, 
8.0]]], [[[2.0, 4.0, 4.0], [8.0, 7.0, 1.0], [7.0, 2.0, 5.0]], [[5.0, 5.0, 6.0], [4.0, 7.0, 5.0], [4.0, 8.0, 2.0]], [[8.0, 2.0, 
8.0], [5.0, 6.0, 7.0], [7.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 2.0], [2.0, 2.0, 3.0], [1.0, 7.0, 7.0]], [[6.0, 1.0, 7.0], [4.0, 7.0, 
2.0], [7.0, 4.0, 5.0]], [[3.0, 6.0, 6.0], [6.0, 5.0, 2.0], [8.0, 2.0, 4.0]]], [[[2.0, 2.0, 5.0], [6.0, 6.0, 6.0], [4.0, 8.0, 
5.0]], [[3.0, 6.0, 6.0], [5.0, 3.0, 6.0], [2.0, 6.0, 4.0]], [[3.0, 7.0, 5.0], [6.0, 8.0, 4.0], [3.0, 8.0, 5.0]]], [[[1.0, 8.0, 
5.0], [2.0, 2.0, 5.0], [2.0, 1.0, 8.0]], [[6.0, 5.0, 4.0], [2.0, 3.0, 7.0], [6.0, 4.0, 4.0]], [[7.0, 6.0, 8.0], [1.0, 3.0, 
8.0], [8.0, 5.0, 6.0]]], [[[1.0, 2.0, 5.0], [3.0, 3.0, 4.0], [3.0, 6.0, 3.0]], [[5.0, 2.0, 5.0], [1.0, 2.0, 8.0], [1.0, 1.0, 
3.0]], [[1.0, 1.0, 7.0], [8.0, 8.0, 1.0], [2.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[4.0], [8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0], [3.0]]))
      arg[1:3,1,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 1.0], [5.0, 7.0, 6.0], [8.0, 8.0, 8.0]], [[3.0, 5.0, 2.0], [3.0, 3.0, 2.0], [3.0, 
7.0, 1.0]], [[6.0, 3.0, 1.0], [7.0, 7.0, 4.0], [3.0, 4.0, 1.0]]], [[[5.0, 3.0, 1.0], [3.0, 6.0, 4.0], [2.0, 6.0, 5.0]], [[1.0, 
2.0, 6.0], [7.0, 5.0, 4.0], [4.0, 3.0, 1.0]], [[4.0, 7.0, 5.0], [5.0, 8.0, 4.0], [4.0, 7.0, 3.0]]], [[[3.0, 7.0, 3.0], [2.0, 
1.0, 5.0], [6.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], [3.0, 8.0, 8.0], [5.0, 3.0, 4.0]], [[5.0, 3.0, 6.0], [4.0, 6.0, 2.0], [2.0, 6.0, 
8.0]]], [[[2.0, 4.0, 4.0], [8.0, 7.0, 1.0], [7.0, 2.0, 5.0]], [[5.0, 5.0, 6.0], [4.0, 7.0, 5.0], [4.0, 8.0, 2.0]], [[8.0, 2.0, 
8.0], [5.0, 6.0, 7.0], [7.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 2.0], [2.0, 2.0, 3.0], [1.0, 7.0, 7.0]], 
[[6.0, 1.0, 7.0], [4.0, 7.0, 2.0], [7.0, 4.0, 5.0]], [[3.0, 6.0, 6.0], [6.0, 5.0, 2.0], [8.0, 2.0, 4.0]]], [[[2.0, 2.0, 5.0], 
[6.0, 6.0, 6.0], [4.0, 8.0, 5.0]], [[3.0, 6.0, 6.0], [5.0, 3.0, 6.0], [2.0, 6.0, 4.0]], [[3.0, 7.0, 5.0], [6.0, 8.0, 4.0], 
[3.0, 8.0, 5.0]]], [[[1.0, 8.0, 5.0], [2.0, 2.0, 5.0], [2.0, 1.0, 8.0]], [[6.0, 5.0, 4.0], [2.0, 3.0, 3.0], [6.0, 4.0, 4.0]], 
[[7.0, 6.0, 8.0], [1.0, 3.0, 8.0], [8.0, 5.0, 6.0]]], [[[1.0, 2.0, 5.0], [3.0, 3.0, 4.0], [3.0, 6.0, 3.0]], [[5.0, 2.0, 5.0], 
[1.0, 2.0, 8.0], [1.0, 1.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 8.0, 1.0], [2.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPPL(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 4.0], [1.0, 1.0, 8.0], [6.0, 6.0, 4.0]], [[8.0, 4.0, 1.0], [4.0, 7.0, 4.0], [7.0, 5.0, 
4.0]], [[2.0, 2.0, 2.0], [8.0, 3.0, 2.0], [1.0, 7.0, 6.0]]], [[[2.0, 8.0, 2.0], [3.0, 1.0, 6.0], [7.0, 5.0, 8.0]], [[5.0, 2.0, 
8.0], [2.0, 6.0, 1.0], [3.0, 5.0, 2.0]], [[2.0, 1.0, 7.0], [3.0, 6.0, 8.0], [1.0, 3.0, 1.0]]], [[[1.0, 4.0, 7.0], [3.0, 7.0, 
1.0], [7.0, 2.0, 7.0]], [[6.0, 1.0, 8.0], [8.0, 2.0, 2.0], [4.0, 5.0, 4.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 4.0], [2.0, 4.0, 
4.0]]], [[[5.0, 1.0, 7.0], [4.0, 4.0, 8.0], [3.0, 7.0, 3.0]], [[4.0, 1.0, 3.0], [8.0, 2.0, 5.0], [8.0, 6.0, 4.0]], [[4.0, 3.0, 
4.0], [5.0, 5.0, 4.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 5.0], [6.0, 3.0, 7.0], [4.0, 4.0, 4.0]], [[6.0, 2.0, 5.0], [3.0, 8.0, 
5.0], [8.0, 6.0, 4.0]], [[8.0, 6.0, 1.0], [6.0, 6.0, 2.0], [6.0, 4.0, 7.0]]], [[[3.0, 1.0, 8.0], [7.0, 2.0, 3.0], [2.0, 6.0, 
8.0]], [[4.0, 6.0, 5.0], [6.0, 2.0, 8.0], [7.0, 5.0, 2.0]], [[7.0, 4.0, 2.0], [4.0, 4.0, 7.0], [8.0, 5.0, 6.0]]], [[[3.0, 4.0, 
4.0], [2.0, 1.0, 5.0], [2.0, 4.0, 6.0]], [[8.0, 5.0, 4.0], [7.0, 6.0, 4.0], [2.0, 7.0, 1.0]], [[7.0, 5.0, 6.0], [7.0, 2.0, 
8.0], [2.0, 5.0, 3.0]]], [[[3.0, 7.0, 1.0], [8.0, 1.0, 4.0], [5.0, 1.0, 8.0]], [[8.0, 2.0, 1.0], [8.0, 8.0, 3.0], [7.0, 3.0, 
4.0]], [[5.0, 1.0, 4.0], [1.0, 8.0, 5.0], [5.0, 7.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[8.0], [3.0]])+(1.-msk_val)*numpy.array([[5.0], [4.0]])
      arg[1:3,1,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 4.0], [1.0, 1.0, 8.0], [6.0, 6.0, 4.0]], [[8.0, 4.0, 1.0], [4.0, 7.0, 4.0], [7.0, 
5.0, 4.0]], [[2.0, 2.0, 2.0], [8.0, 3.0, 2.0], [1.0, 7.0, 6.0]]], [[[2.0, 8.0, 2.0], [3.0, 1.0, 6.0], [7.0, 5.0, 8.0]], [[5.0, 
2.0, 8.0], [2.0, 6.0, 8.0], [3.0, 5.0, 2.0]], [[2.0, 1.0, 7.0], [3.0, 6.0, 8.0], [1.0, 3.0, 1.0]]], [[[1.0, 4.0, 7.0], [3.0, 
7.0, 1.0], [7.0, 2.0, 7.0]], [[6.0, 1.0, 8.0], [8.0, 2.0, 3.0], [4.0, 5.0, 4.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 4.0], [2.0, 4.0, 
4.0]]], [[[5.0, 1.0, 7.0], [4.0, 4.0, 8.0], [3.0, 7.0, 3.0]], [[4.0, 1.0, 3.0], [8.0, 2.0, 5.0], [8.0, 6.0, 4.0]], [[4.0, 3.0, 
4.0], [5.0, 5.0, 4.0], [3.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 5.0], [6.0, 3.0, 7.0], [4.0, 4.0, 4.0]], 
[[6.0, 2.0, 5.0], [3.0, 8.0, 5.0], [8.0, 6.0, 4.0]], [[8.0, 6.0, 1.0], [6.0, 6.0, 2.0], [6.0, 4.0, 7.0]]], [[[3.0, 1.0, 8.0], 
[7.0, 2.0, 3.0], [2.0, 6.0, 8.0]], [[4.0, 6.0, 5.0], [6.0, 2.0, 5.0], [7.0, 5.0, 2.0]], [[7.0, 4.0, 2.0], [4.0, 4.0, 7.0], 
[8.0, 5.0, 6.0]]], [[[3.0, 4.0, 4.0], [2.0, 1.0, 5.0], [2.0, 4.0, 6.0]], [[8.0, 5.0, 4.0], [7.0, 6.0, 4.0], [2.0, 7.0, 1.0]], 
[[7.0, 5.0, 6.0], [7.0, 2.0, 8.0], [2.0, 5.0, 3.0]]], [[[3.0, 7.0, 1.0], [8.0, 1.0, 4.0], [5.0, 1.0, 8.0]], [[8.0, 2.0, 1.0], 
[8.0, 8.0, 3.0], [7.0, 3.0, 4.0]], [[5.0, 1.0, 4.0], [1.0, 8.0, 5.0], [5.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PRRN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 6.0], [8.0, 3.0, 3.0], [2.0, 5.0, 1.0]], [[2.0, 7.0, 7.0], [1.0, 4.0, 7.0], [2.0, 6.0, 
8.0]], [[6.0, 4.0, 3.0], [4.0, 3.0, 2.0], [8.0, 1.0, 2.0]]], [[[6.0, 2.0, 6.0], [4.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[6.0, 1.0, 
8.0], [4.0, 3.0, 4.0], [3.0, 3.0, 5.0]], [[3.0, 6.0, 5.0], [7.0, 4.0, 7.0], [1.0, 7.0, 5.0]]], [[[2.0, 3.0, 4.0], [2.0, 7.0, 
3.0], [5.0, 2.0, 1.0]], [[8.0, 7.0, 2.0], [6.0, 5.0, 3.0], [7.0, 4.0, 8.0]], [[8.0, 5.0, 4.0], [5.0, 4.0, 8.0], [8.0, 8.0, 
2.0]]], [[[3.0, 8.0, 2.0], [4.0, 8.0, 7.0], [4.0, 4.0, 8.0]], [[2.0, 8.0, 7.0], [3.0, 6.0, 8.0], [8.0, 7.0, 3.0]], [[2.0, 4.0, 
5.0], [1.0, 7.0, 6.0], [5.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 8.0], [3.0, 6.0, 8.0], [5.0, 8.0, 2.0]], [[7.0, 7.0, 3.0], [4.0, 6.0, 
5.0], [2.0, 6.0, 4.0]], [[5.0, 2.0, 4.0], [3.0, 3.0, 7.0], [3.0, 1.0, 6.0]]], [[[5.0, 1.0, 5.0], [8.0, 2.0, 6.0], [2.0, 7.0, 
2.0]], [[8.0, 6.0, 1.0], [3.0, 4.0, 3.0], [1.0, 8.0, 6.0]], [[1.0, 5.0, 3.0], [5.0, 3.0, 7.0], [2.0, 6.0, 4.0]]], [[[5.0, 6.0, 
7.0], [8.0, 4.0, 6.0], [2.0, 4.0, 5.0]], [[4.0, 4.0, 2.0], [2.0, 3.0, 7.0], [8.0, 5.0, 2.0]], [[1.0, 8.0, 2.0], [5.0, 8.0, 
3.0], [7.0, 1.0, 2.0]]], [[[8.0, 8.0, 6.0], [6.0, 3.0, 7.0], [5.0, 6.0, 7.0]], [[5.0, 8.0, 2.0], [8.0, 8.0, 2.0], [6.0, 1.0, 
1.0]], [[6.0, 4.0, 3.0], [2.0, 1.0, 2.0], [3.0, 1.0, 3.0]]]]))
      val=numpy.array([[[5.0, 7.0, 7.0]]])
      arg[2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 6.0], [8.0, 3.0, 3.0], [2.0, 5.0, 1.0]], [[2.0, 7.0, 7.0], [1.0, 4.0, 7.0], [2.0, 
6.0, 8.0]], [[6.0, 4.0, 3.0], [4.0, 3.0, 2.0], [8.0, 1.0, 2.0]]], [[[6.0, 2.0, 6.0], [4.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[6.0, 
1.0, 8.0], [4.0, 3.0, 4.0], [3.0, 3.0, 5.0]], [[3.0, 6.0, 5.0], [7.0, 4.0, 7.0], [1.0, 7.0, 5.0]]], [[[2.0, 3.0, 4.0], [2.0, 
7.0, 3.0], [5.0, 2.0, 1.0]], [[8.0, 7.0, 2.0], [5.0, 7.0, 7.0], [7.0, 4.0, 8.0]], [[8.0, 5.0, 4.0], [5.0, 4.0, 8.0], [8.0, 8.0, 
2.0]]], [[[3.0, 8.0, 2.0], [4.0, 8.0, 7.0], [4.0, 4.0, 8.0]], [[2.0, 8.0, 7.0], [3.0, 6.0, 8.0], [8.0, 7.0, 3.0]], [[2.0, 4.0, 
5.0], [1.0, 7.0, 6.0], [5.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 8.0], [3.0, 6.0, 8.0], [5.0, 8.0, 2.0]], 
[[7.0, 7.0, 3.0], [4.0, 6.0, 5.0], [2.0, 6.0, 4.0]], [[5.0, 2.0, 4.0], [3.0, 3.0, 7.0], [3.0, 1.0, 6.0]]], [[[5.0, 1.0, 5.0], 
[8.0, 2.0, 6.0], [2.0, 7.0, 2.0]], [[8.0, 6.0, 1.0], [3.0, 4.0, 3.0], [1.0, 8.0, 6.0]], [[1.0, 5.0, 3.0], [5.0, 3.0, 7.0], 
[2.0, 6.0, 4.0]]], [[[5.0, 6.0, 7.0], [8.0, 4.0, 6.0], [2.0, 4.0, 5.0]], [[4.0, 4.0, 2.0], [5.0, 7.0, 7.0], [8.0, 5.0, 2.0]], 
[[1.0, 8.0, 2.0], [5.0, 8.0, 3.0], [7.0, 1.0, 2.0]]], [[[8.0, 8.0, 6.0], [6.0, 3.0, 7.0], [5.0, 6.0, 7.0]], [[5.0, 8.0, 2.0], 
[8.0, 8.0, 2.0], [6.0, 1.0, 1.0]], [[6.0, 4.0, 3.0], [2.0, 1.0, 2.0], [3.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PRRN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 5.0], [5.0, 7.0, 8.0], [4.0, 8.0, 7.0]], [[3.0, 6.0, 5.0], [6.0, 7.0, 8.0], [8.0, 1.0, 
8.0]], [[2.0, 8.0, 2.0], [3.0, 4.0, 2.0], [8.0, 7.0, 1.0]]], [[[8.0, 3.0, 5.0], [2.0, 3.0, 3.0], [8.0, 4.0, 4.0]], [[2.0, 2.0, 
5.0], [7.0, 2.0, 3.0], [3.0, 8.0, 3.0]], [[2.0, 8.0, 5.0], [8.0, 4.0, 5.0], [6.0, 2.0, 8.0]]], [[[3.0, 3.0, 7.0], [4.0, 4.0, 
6.0], [7.0, 7.0, 8.0]], [[8.0, 6.0, 8.0], [1.0, 3.0, 6.0], [2.0, 8.0, 5.0]], [[7.0, 5.0, 6.0], [4.0, 4.0, 3.0], [7.0, 8.0, 
2.0]]], [[[6.0, 2.0, 8.0], [4.0, 8.0, 6.0], [2.0, 2.0, 8.0]], [[5.0, 4.0, 2.0], [4.0, 1.0, 2.0], [6.0, 4.0, 8.0]], [[4.0, 1.0, 
2.0], [5.0, 3.0, 2.0], [2.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 7.0], [7.0, 8.0, 8.0], [5.0, 8.0, 8.0]], [[3.0, 5.0, 5.0], [2.0, 7.0, 
1.0], [3.0, 4.0, 8.0]], [[7.0, 2.0, 5.0], [7.0, 2.0, 3.0], [5.0, 7.0, 5.0]]], [[[5.0, 7.0, 5.0], [2.0, 7.0, 6.0], [1.0, 6.0, 
4.0]], [[5.0, 7.0, 1.0], [4.0, 3.0, 4.0], [8.0, 3.0, 5.0]], [[8.0, 7.0, 8.0], [3.0, 3.0, 1.0], [6.0, 6.0, 2.0]]], [[[6.0, 3.0, 
7.0], [8.0, 5.0, 3.0], [8.0, 3.0, 5.0]], [[3.0, 5.0, 4.0], [5.0, 2.0, 1.0], [4.0, 8.0, 4.0]], [[7.0, 2.0, 6.0], [3.0, 2.0, 
6.0], [6.0, 5.0, 7.0]]], [[[5.0, 3.0, 6.0], [3.0, 4.0, 1.0], [2.0, 7.0, 8.0]], [[4.0, 5.0, 4.0], [2.0, 2.0, 5.0], [4.0, 6.0, 
1.0]], [[6.0, 6.0, 2.0], [4.0, 5.0, 5.0], [5.0, 8.0, 4.0]]]]))
      val=Data(numpy.array([[[3.0, 2.0, 8.0]]]),self.functionspace)
      arg[2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 5.0], [5.0, 7.0, 8.0], [4.0, 8.0, 7.0]], [[3.0, 6.0, 5.0], [6.0, 7.0, 8.0], [8.0, 
1.0, 8.0]], [[2.0, 8.0, 2.0], [3.0, 4.0, 2.0], [8.0, 7.0, 1.0]]], [[[8.0, 3.0, 5.0], [2.0, 3.0, 3.0], [8.0, 4.0, 4.0]], [[2.0, 
2.0, 5.0], [7.0, 2.0, 3.0], [3.0, 8.0, 3.0]], [[2.0, 8.0, 5.0], [8.0, 4.0, 5.0], [6.0, 2.0, 8.0]]], [[[3.0, 3.0, 7.0], [4.0, 
4.0, 6.0], [7.0, 7.0, 8.0]], [[8.0, 6.0, 8.0], [3.0, 2.0, 8.0], [2.0, 8.0, 5.0]], [[7.0, 5.0, 6.0], [4.0, 4.0, 3.0], [7.0, 8.0, 
2.0]]], [[[6.0, 2.0, 8.0], [4.0, 8.0, 6.0], [2.0, 2.0, 8.0]], [[5.0, 4.0, 2.0], [4.0, 1.0, 2.0], [6.0, 4.0, 8.0]], [[4.0, 1.0, 
2.0], [5.0, 3.0, 2.0], [2.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 7.0], [7.0, 8.0, 8.0], [5.0, 8.0, 8.0]], 
[[3.0, 5.0, 5.0], [2.0, 7.0, 1.0], [3.0, 4.0, 8.0]], [[7.0, 2.0, 5.0], [7.0, 2.0, 3.0], [5.0, 7.0, 5.0]]], [[[5.0, 7.0, 5.0], 
[2.0, 7.0, 6.0], [1.0, 6.0, 4.0]], [[5.0, 7.0, 1.0], [4.0, 3.0, 4.0], [8.0, 3.0, 5.0]], [[8.0, 7.0, 8.0], [3.0, 3.0, 1.0], 
[6.0, 6.0, 2.0]]], [[[6.0, 3.0, 7.0], [8.0, 5.0, 3.0], [8.0, 3.0, 5.0]], [[3.0, 5.0, 4.0], [3.0, 2.0, 8.0], [4.0, 8.0, 4.0]], 
[[7.0, 2.0, 6.0], [3.0, 2.0, 6.0], [6.0, 5.0, 7.0]]], [[[5.0, 3.0, 6.0], [3.0, 4.0, 1.0], [2.0, 7.0, 8.0]], [[4.0, 5.0, 4.0], 
[2.0, 2.0, 5.0], [4.0, 6.0, 1.0]], [[6.0, 6.0, 2.0], [4.0, 5.0, 5.0], [5.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PRRN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 5.0], [5.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[4.0, 2.0, 6.0], [4.0, 5.0, 1.0], [6.0, 6.0, 
1.0]], [[4.0, 4.0, 4.0], [6.0, 7.0, 8.0], [6.0, 6.0, 7.0]]], [[[2.0, 7.0, 6.0], [4.0, 5.0, 7.0], [1.0, 5.0, 4.0]], [[4.0, 2.0, 
6.0], [6.0, 4.0, 7.0], [3.0, 6.0, 1.0]], [[4.0, 4.0, 5.0], [1.0, 3.0, 5.0], [3.0, 4.0, 2.0]]], [[[4.0, 7.0, 4.0], [7.0, 6.0, 
2.0], [2.0, 6.0, 2.0]], [[5.0, 7.0, 7.0], [1.0, 5.0, 1.0], [6.0, 1.0, 7.0]], [[7.0, 2.0, 3.0], [6.0, 8.0, 5.0], [5.0, 6.0, 
4.0]]], [[[5.0, 7.0, 1.0], [1.0, 4.0, 4.0], [8.0, 8.0, 7.0]], [[1.0, 8.0, 5.0], [2.0, 1.0, 2.0], [3.0, 4.0, 2.0]], [[1.0, 6.0, 
6.0], [8.0, 2.0, 7.0], [6.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 8.0], [5.0, 7.0, 3.0], [3.0, 3.0, 5.0]], [[5.0, 3.0, 3.0], [3.0, 7.0, 
4.0], [2.0, 5.0, 6.0]], [[2.0, 1.0, 7.0], [1.0, 1.0, 8.0], [3.0, 6.0, 7.0]]], [[[3.0, 8.0, 1.0], [2.0, 1.0, 3.0], [7.0, 1.0, 
4.0]], [[8.0, 7.0, 6.0], [1.0, 5.0, 3.0], [2.0, 7.0, 3.0]], [[8.0, 7.0, 8.0], [3.0, 5.0, 1.0], [1.0, 7.0, 3.0]]], [[[5.0, 2.0, 
1.0], [5.0, 2.0, 2.0], [2.0, 2.0, 7.0]], [[2.0, 2.0, 2.0], [1.0, 1.0, 5.0], [7.0, 2.0, 6.0]], [[1.0, 2.0, 7.0], [4.0, 7.0, 
5.0], [1.0, 5.0, 3.0]]], [[[3.0, 7.0, 8.0], [6.0, 1.0, 3.0], [4.0, 7.0, 4.0]], [[7.0, 6.0, 8.0], [2.0, 1.0, 3.0], [7.0, 5.0, 
5.0]], [[8.0, 8.0, 6.0], [5.0, 2.0, 5.0], [3.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[6.0, 8.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 3.0, 3.0]]]))
      arg[2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 5.0], [5.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[4.0, 2.0, 6.0], [4.0, 5.0, 1.0], [6.0, 
6.0, 1.0]], [[4.0, 4.0, 4.0], [6.0, 7.0, 8.0], [6.0, 6.0, 7.0]]], [[[2.0, 7.0, 6.0], [4.0, 5.0, 7.0], [1.0, 5.0, 4.0]], [[4.0, 
2.0, 6.0], [6.0, 4.0, 7.0], [3.0, 6.0, 1.0]], [[4.0, 4.0, 5.0], [1.0, 3.0, 5.0], [3.0, 4.0, 2.0]]], [[[4.0, 7.0, 4.0], [7.0, 
6.0, 2.0], [2.0, 6.0, 2.0]], [[5.0, 7.0, 7.0], [6.0, 8.0, 3.0], [6.0, 1.0, 7.0]], [[7.0, 2.0, 3.0], [6.0, 8.0, 5.0], [5.0, 6.0, 
4.0]]], [[[5.0, 7.0, 1.0], [1.0, 4.0, 4.0], [8.0, 8.0, 7.0]], [[1.0, 8.0, 5.0], [2.0, 1.0, 2.0], [3.0, 4.0, 2.0]], [[1.0, 6.0, 
6.0], [8.0, 2.0, 7.0], [6.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 8.0], [5.0, 7.0, 3.0], [3.0, 3.0, 5.0]], 
[[5.0, 3.0, 3.0], [3.0, 7.0, 4.0], [2.0, 5.0, 6.0]], [[2.0, 1.0, 7.0], [1.0, 1.0, 8.0], [3.0, 6.0, 7.0]]], [[[3.0, 8.0, 1.0], 
[2.0, 1.0, 3.0], [7.0, 1.0, 4.0]], [[8.0, 7.0, 6.0], [1.0, 5.0, 3.0], [2.0, 7.0, 3.0]], [[8.0, 7.0, 8.0], [3.0, 5.0, 1.0], 
[1.0, 7.0, 3.0]]], [[[5.0, 2.0, 1.0], [5.0, 2.0, 2.0], [2.0, 2.0, 7.0]], [[2.0, 2.0, 2.0], [3.0, 3.0, 3.0], [7.0, 2.0, 6.0]], 
[[1.0, 2.0, 7.0], [4.0, 7.0, 5.0], [1.0, 5.0, 3.0]]], [[[3.0, 7.0, 8.0], [6.0, 1.0, 3.0], [4.0, 7.0, 4.0]], [[7.0, 6.0, 8.0], 
[2.0, 1.0, 3.0], [7.0, 5.0, 5.0]], [[8.0, 8.0, 6.0], [5.0, 2.0, 5.0], [3.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PRRN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [3.0, 4.0, 2.0], [2.0, 6.0, 4.0]], [[8.0, 7.0, 2.0], [3.0, 2.0, 2.0], [7.0, 6.0, 
1.0]], [[5.0, 5.0, 4.0], [5.0, 4.0, 5.0], [6.0, 4.0, 8.0]]], [[[2.0, 2.0, 6.0], [2.0, 3.0, 1.0], [4.0, 3.0, 3.0]], [[8.0, 2.0, 
2.0], [2.0, 3.0, 3.0], [4.0, 1.0, 6.0]], [[2.0, 2.0, 3.0], [3.0, 3.0, 2.0], [5.0, 7.0, 7.0]]], [[[3.0, 6.0, 2.0], [2.0, 4.0, 
3.0], [1.0, 2.0, 8.0]], [[1.0, 1.0, 5.0], [2.0, 1.0, 8.0], [2.0, 2.0, 5.0]], [[7.0, 5.0, 8.0], [1.0, 3.0, 7.0], [1.0, 4.0, 
4.0]]], [[[5.0, 3.0, 7.0], [5.0, 1.0, 2.0], [5.0, 8.0, 3.0]], [[7.0, 8.0, 6.0], [3.0, 7.0, 5.0], [4.0, 3.0, 8.0]], [[8.0, 4.0, 
5.0], [5.0, 7.0, 4.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 6.0], [7.0, 7.0, 5.0], [8.0, 1.0, 2.0]], [[1.0, 4.0, 4.0], [4.0, 7.0, 
8.0], [3.0, 4.0, 7.0]], [[6.0, 5.0, 1.0], [7.0, 2.0, 7.0], [2.0, 8.0, 8.0]]], [[[5.0, 6.0, 6.0], [1.0, 6.0, 7.0], [2.0, 1.0, 
5.0]], [[6.0, 3.0, 8.0], [4.0, 3.0, 5.0], [1.0, 8.0, 7.0]], [[8.0, 6.0, 5.0], [3.0, 3.0, 8.0], [6.0, 4.0, 8.0]]], [[[4.0, 3.0, 
3.0], [1.0, 5.0, 8.0], [6.0, 8.0, 8.0]], [[8.0, 6.0, 3.0], [2.0, 3.0, 2.0], [1.0, 8.0, 6.0]], [[5.0, 3.0, 7.0], [4.0, 5.0, 
8.0], [8.0, 3.0, 4.0]]], [[[6.0, 2.0, 3.0], [8.0, 2.0, 7.0], [7.0, 1.0, 7.0]], [[7.0, 8.0, 6.0], [5.0, 5.0, 5.0], [8.0, 5.0, 
1.0]], [[7.0, 2.0, 5.0], [6.0, 4.0, 1.0], [6.0, 1.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 1.0, 6.0]]])+(1.-msk_val)*numpy.array([[[8.0, 6.0, 1.0]]])
      arg[2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 7.0], [3.0, 4.0, 2.0], [2.0, 6.0, 4.0]], [[8.0, 7.0, 2.0], [3.0, 2.0, 2.0], [7.0, 
6.0, 1.0]], [[5.0, 5.0, 4.0], [5.0, 4.0, 5.0], [6.0, 4.0, 8.0]]], [[[2.0, 2.0, 6.0], [2.0, 3.0, 1.0], [4.0, 3.0, 3.0]], [[8.0, 
2.0, 2.0], [2.0, 3.0, 3.0], [4.0, 1.0, 6.0]], [[2.0, 2.0, 3.0], [3.0, 3.0, 2.0], [5.0, 7.0, 7.0]]], [[[3.0, 6.0, 2.0], [2.0, 
4.0, 3.0], [1.0, 2.0, 8.0]], [[1.0, 1.0, 5.0], [8.0, 1.0, 6.0], [2.0, 2.0, 5.0]], [[7.0, 5.0, 8.0], [1.0, 3.0, 7.0], [1.0, 4.0, 
4.0]]], [[[5.0, 3.0, 7.0], [5.0, 1.0, 2.0], [5.0, 8.0, 3.0]], [[7.0, 8.0, 6.0], [3.0, 7.0, 5.0], [4.0, 3.0, 8.0]], [[8.0, 4.0, 
5.0], [5.0, 7.0, 4.0], [3.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 6.0], [7.0, 7.0, 5.0], [8.0, 1.0, 2.0]], 
[[1.0, 4.0, 4.0], [4.0, 7.0, 8.0], [3.0, 4.0, 7.0]], [[6.0, 5.0, 1.0], [7.0, 2.0, 7.0], [2.0, 8.0, 8.0]]], [[[5.0, 6.0, 6.0], 
[1.0, 6.0, 7.0], [2.0, 1.0, 5.0]], [[6.0, 3.0, 8.0], [4.0, 3.0, 5.0], [1.0, 8.0, 7.0]], [[8.0, 6.0, 5.0], [3.0, 3.0, 8.0], 
[6.0, 4.0, 8.0]]], [[[4.0, 3.0, 3.0], [1.0, 5.0, 8.0], [6.0, 8.0, 8.0]], [[8.0, 6.0, 3.0], [8.0, 6.0, 1.0], [1.0, 8.0, 6.0]], 
[[5.0, 3.0, 7.0], [4.0, 5.0, 8.0], [8.0, 3.0, 4.0]]], [[[6.0, 2.0, 3.0], [8.0, 2.0, 7.0], [7.0, 1.0, 7.0]], [[7.0, 8.0, 6.0], 
[5.0, 5.0, 5.0], [8.0, 5.0, 1.0]], [[7.0, 2.0, 5.0], [6.0, 4.0, 1.0], [6.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPUU(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 4.0], [5.0, 6.0, 2.0], [2.0, 7.0, 2.0]], [[8.0, 4.0, 2.0], [8.0, 2.0, 1.0], [1.0, 7.0, 
5.0]], [[6.0, 5.0, 1.0], [6.0, 4.0, 7.0], [3.0, 2.0, 7.0]]], [[[3.0, 4.0, 6.0], [3.0, 1.0, 1.0], [2.0, 2.0, 3.0]], [[1.0, 1.0, 
7.0], [7.0, 5.0, 7.0], [6.0, 2.0, 6.0]], [[4.0, 4.0, 3.0], [3.0, 7.0, 5.0], [2.0, 6.0, 1.0]]], [[[6.0, 8.0, 7.0], [3.0, 1.0, 
8.0], [3.0, 4.0, 5.0]], [[4.0, 3.0, 4.0], [7.0, 8.0, 2.0], [7.0, 7.0, 6.0]], [[4.0, 7.0, 2.0], [2.0, 2.0, 2.0], [7.0, 4.0, 
4.0]]], [[[3.0, 4.0, 5.0], [8.0, 2.0, 1.0], [1.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], [5.0, 1.0, 4.0], [1.0, 7.0, 3.0]], [[3.0, 4.0, 
4.0], [6.0, 3.0, 1.0], [8.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 3.0], [7.0, 2.0, 1.0], [2.0, 7.0, 7.0]], [[2.0, 2.0, 8.0], [3.0, 7.0, 
1.0], [8.0, 3.0, 3.0]], [[8.0, 1.0, 8.0], [6.0, 8.0, 2.0], [4.0, 8.0, 4.0]]], [[[4.0, 5.0, 5.0], [8.0, 5.0, 8.0], [7.0, 8.0, 
7.0]], [[2.0, 2.0, 8.0], [3.0, 4.0, 1.0], [8.0, 1.0, 8.0]], [[7.0, 7.0, 6.0], [8.0, 7.0, 5.0], [3.0, 7.0, 8.0]]], [[[7.0, 3.0, 
6.0], [3.0, 2.0, 6.0], [8.0, 4.0, 2.0]], [[5.0, 1.0, 1.0], [3.0, 3.0, 1.0], [6.0, 7.0, 1.0]], [[7.0, 2.0, 5.0], [1.0, 5.0, 
1.0], [2.0, 5.0, 8.0]]], [[[6.0, 3.0, 2.0], [8.0, 7.0, 3.0], [8.0, 4.0, 3.0]], [[5.0, 4.0, 6.0], [7.0, 3.0, 1.0], [1.0, 5.0, 
1.0]], [[8.0, 1.0, 2.0], [7.0, 7.0, 1.0], [1.0, 7.0, 2.0]]]]))
      val=numpy.array([[[1.0, 5.0], [1.0, 8.0]], [[7.0, 7.0], [7.0, 4.0]]])
      arg[1:3,1,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 4.0], [5.0, 6.0, 2.0], [2.0, 7.0, 2.0]], [[8.0, 4.0, 2.0], [8.0, 2.0, 1.0], [1.0, 
7.0, 5.0]], [[6.0, 5.0, 1.0], [6.0, 4.0, 7.0], [3.0, 2.0, 7.0]]], [[[3.0, 4.0, 6.0], [3.0, 1.0, 1.0], [2.0, 2.0, 3.0]], [[1.0, 
5.0, 7.0], [1.0, 8.0, 7.0], [6.0, 2.0, 6.0]], [[4.0, 4.0, 3.0], [3.0, 7.0, 5.0], [2.0, 6.0, 1.0]]], [[[6.0, 8.0, 7.0], [3.0, 
1.0, 8.0], [3.0, 4.0, 5.0]], [[7.0, 7.0, 4.0], [7.0, 4.0, 2.0], [7.0, 7.0, 6.0]], [[4.0, 7.0, 2.0], [2.0, 2.0, 2.0], [7.0, 4.0, 
4.0]]], [[[3.0, 4.0, 5.0], [8.0, 2.0, 1.0], [1.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], [5.0, 1.0, 4.0], [1.0, 7.0, 3.0]], [[3.0, 4.0, 
4.0], [6.0, 3.0, 1.0], [8.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 3.0], [7.0, 2.0, 1.0], [2.0, 7.0, 7.0]], 
[[2.0, 2.0, 8.0], [3.0, 7.0, 1.0], [8.0, 3.0, 3.0]], [[8.0, 1.0, 8.0], [6.0, 8.0, 2.0], [4.0, 8.0, 4.0]]], [[[4.0, 5.0, 5.0], 
[8.0, 5.0, 8.0], [7.0, 8.0, 7.0]], [[1.0, 5.0, 8.0], [1.0, 8.0, 1.0], [8.0, 1.0, 8.0]], [[7.0, 7.0, 6.0], [8.0, 7.0, 5.0], 
[3.0, 7.0, 8.0]]], [[[7.0, 3.0, 6.0], [3.0, 2.0, 6.0], [8.0, 4.0, 2.0]], [[7.0, 7.0, 1.0], [7.0, 4.0, 1.0], [6.0, 7.0, 1.0]], 
[[7.0, 2.0, 5.0], [1.0, 5.0, 1.0], [2.0, 5.0, 8.0]]], [[[6.0, 3.0, 2.0], [8.0, 7.0, 3.0], [8.0, 4.0, 3.0]], [[5.0, 4.0, 6.0], 
[7.0, 3.0, 1.0], [1.0, 5.0, 1.0]], [[8.0, 1.0, 2.0], [7.0, 7.0, 1.0], [1.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPUU(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 6.0], [5.0, 2.0, 7.0], [4.0, 2.0, 3.0]], [[4.0, 8.0, 3.0], [5.0, 4.0, 6.0], [6.0, 7.0, 
6.0]], [[2.0, 2.0, 1.0], [3.0, 3.0, 3.0], [8.0, 7.0, 3.0]]], [[[1.0, 2.0, 6.0], [8.0, 6.0, 4.0], [1.0, 8.0, 5.0]], [[8.0, 5.0, 
1.0], [4.0, 1.0, 7.0], [7.0, 7.0, 6.0]], [[6.0, 2.0, 5.0], [2.0, 1.0, 7.0], [5.0, 7.0, 3.0]]], [[[8.0, 3.0, 1.0], [4.0, 6.0, 
7.0], [6.0, 7.0, 5.0]], [[6.0, 8.0, 4.0], [3.0, 1.0, 2.0], [2.0, 4.0, 1.0]], [[6.0, 7.0, 6.0], [6.0, 1.0, 8.0], [6.0, 4.0, 
3.0]]], [[[5.0, 6.0, 3.0], [8.0, 3.0, 5.0], [5.0, 2.0, 7.0]], [[7.0, 7.0, 1.0], [5.0, 3.0, 5.0], [2.0, 7.0, 2.0]], [[2.0, 3.0, 
4.0], [6.0, 7.0, 4.0], [3.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 4.0], [5.0, 7.0, 6.0], [2.0, 8.0, 8.0]], [[5.0, 5.0, 4.0], [7.0, 1.0, 
4.0], [6.0, 1.0, 5.0]], [[4.0, 5.0, 3.0], [5.0, 8.0, 3.0], [7.0, 7.0, 6.0]]], [[[8.0, 3.0, 4.0], [4.0, 7.0, 5.0], [5.0, 7.0, 
4.0]], [[8.0, 8.0, 8.0], [2.0, 7.0, 6.0], [7.0, 8.0, 6.0]], [[7.0, 1.0, 4.0], [4.0, 7.0, 8.0], [3.0, 6.0, 1.0]]], [[[2.0, 6.0, 
2.0], [7.0, 2.0, 3.0], [5.0, 8.0, 7.0]], [[4.0, 8.0, 2.0], [4.0, 1.0, 8.0], [3.0, 5.0, 7.0]], [[5.0, 8.0, 5.0], [8.0, 5.0, 
5.0], [1.0, 6.0, 6.0]]], [[[7.0, 1.0, 6.0], [1.0, 1.0, 4.0], [3.0, 1.0, 4.0]], [[1.0, 7.0, 6.0], [4.0, 6.0, 2.0], [3.0, 7.0, 
5.0]], [[7.0, 1.0, 3.0], [2.0, 7.0, 4.0], [1.0, 7.0, 4.0]]]]))
      val=Data(numpy.array([[[4.0, 6.0], [7.0, 8.0]], [[5.0, 3.0], [6.0, 6.0]]]),self.functionspace)
      arg[1:3,1,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 6.0], [5.0, 2.0, 7.0], [4.0, 2.0, 3.0]], [[4.0, 8.0, 3.0], [5.0, 4.0, 6.0], [6.0, 
7.0, 6.0]], [[2.0, 2.0, 1.0], [3.0, 3.0, 3.0], [8.0, 7.0, 3.0]]], [[[1.0, 2.0, 6.0], [8.0, 6.0, 4.0], [1.0, 8.0, 5.0]], [[4.0, 
6.0, 1.0], [7.0, 8.0, 7.0], [7.0, 7.0, 6.0]], [[6.0, 2.0, 5.0], [2.0, 1.0, 7.0], [5.0, 7.0, 3.0]]], [[[8.0, 3.0, 1.0], [4.0, 
6.0, 7.0], [6.0, 7.0, 5.0]], [[5.0, 3.0, 4.0], [6.0, 6.0, 2.0], [2.0, 4.0, 1.0]], [[6.0, 7.0, 6.0], [6.0, 1.0, 8.0], [6.0, 4.0, 
3.0]]], [[[5.0, 6.0, 3.0], [8.0, 3.0, 5.0], [5.0, 2.0, 7.0]], [[7.0, 7.0, 1.0], [5.0, 3.0, 5.0], [2.0, 7.0, 2.0]], [[2.0, 3.0, 
4.0], [6.0, 7.0, 4.0], [3.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 4.0], [5.0, 7.0, 6.0], [2.0, 8.0, 8.0]], 
[[5.0, 5.0, 4.0], [7.0, 1.0, 4.0], [6.0, 1.0, 5.0]], [[4.0, 5.0, 3.0], [5.0, 8.0, 3.0], [7.0, 7.0, 6.0]]], [[[8.0, 3.0, 4.0], 
[4.0, 7.0, 5.0], [5.0, 7.0, 4.0]], [[4.0, 6.0, 8.0], [7.0, 8.0, 6.0], [7.0, 8.0, 6.0]], [[7.0, 1.0, 4.0], [4.0, 7.0, 8.0], 
[3.0, 6.0, 1.0]]], [[[2.0, 6.0, 2.0], [7.0, 2.0, 3.0], [5.0, 8.0, 7.0]], [[5.0, 3.0, 2.0], [6.0, 6.0, 8.0], [3.0, 5.0, 7.0]], 
[[5.0, 8.0, 5.0], [8.0, 5.0, 5.0], [1.0, 6.0, 6.0]]], [[[7.0, 1.0, 6.0], [1.0, 1.0, 4.0], [3.0, 1.0, 4.0]], [[1.0, 7.0, 6.0], 
[4.0, 6.0, 2.0], [3.0, 7.0, 5.0]], [[7.0, 1.0, 3.0], [2.0, 7.0, 4.0], [1.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPUU(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 6.0], [1.0, 7.0, 5.0], [8.0, 6.0, 7.0]], [[1.0, 6.0, 6.0], [6.0, 1.0, 6.0], [3.0, 7.0, 
3.0]], [[1.0, 5.0, 8.0], [7.0, 4.0, 8.0], [5.0, 1.0, 8.0]]], [[[6.0, 3.0, 1.0], [6.0, 7.0, 1.0], [3.0, 4.0, 3.0]], [[8.0, 7.0, 
2.0], [5.0, 4.0, 3.0], [2.0, 1.0, 4.0]], [[2.0, 3.0, 1.0], [8.0, 3.0, 1.0], [4.0, 6.0, 8.0]]], [[[6.0, 5.0, 4.0], [2.0, 1.0, 
5.0], [4.0, 3.0, 2.0]], [[5.0, 4.0, 6.0], [7.0, 2.0, 4.0], [8.0, 1.0, 2.0]], [[4.0, 8.0, 3.0], [6.0, 3.0, 2.0], [3.0, 6.0, 
8.0]]], [[[2.0, 8.0, 1.0], [1.0, 2.0, 4.0], [7.0, 8.0, 3.0]], [[4.0, 7.0, 8.0], [4.0, 7.0, 7.0], [5.0, 6.0, 5.0]], [[2.0, 7.0, 
4.0], [3.0, 4.0, 2.0], [2.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 1.0], [5.0, 8.0, 6.0], [3.0, 6.0, 2.0]], [[6.0, 2.0, 5.0], [5.0, 1.0, 
4.0], [2.0, 8.0, 1.0]], [[1.0, 7.0, 6.0], [6.0, 3.0, 8.0], [4.0, 6.0, 6.0]]], [[[1.0, 3.0, 8.0], [4.0, 3.0, 2.0], [2.0, 4.0, 
8.0]], [[1.0, 6.0, 2.0], [2.0, 5.0, 2.0], [5.0, 6.0, 6.0]], [[5.0, 6.0, 3.0], [8.0, 6.0, 3.0], [7.0, 7.0, 2.0]]], [[[2.0, 5.0, 
7.0], [6.0, 8.0, 2.0], [4.0, 2.0, 6.0]], [[8.0, 6.0, 8.0], [3.0, 4.0, 1.0], [5.0, 6.0, 4.0]], [[8.0, 6.0, 2.0], [6.0, 2.0, 
1.0], [3.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [4.0, 7.0, 5.0], [5.0, 5.0, 7.0]], [[1.0, 7.0, 8.0], [8.0, 6.0, 4.0], [8.0, 5.0, 
1.0]], [[2.0, 6.0, 2.0], [2.0, 3.0, 3.0], [5.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[[3.0, 6.0], [7.0, 2.0]], [[6.0, 1.0], [8.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 1.0], [7.0, 7.0]], [[5.0, 4.0], [4.0, 4.0]]]))
      arg[1:3,1,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [1.0, 7.0, 5.0], [8.0, 6.0, 7.0]], [[1.0, 6.0, 6.0], [6.0, 1.0, 6.0], [3.0, 
7.0, 3.0]], [[1.0, 5.0, 8.0], [7.0, 4.0, 8.0], [5.0, 1.0, 8.0]]], [[[6.0, 3.0, 1.0], [6.0, 7.0, 1.0], [3.0, 4.0, 3.0]], [[3.0, 
6.0, 2.0], [7.0, 2.0, 3.0], [2.0, 1.0, 4.0]], [[2.0, 3.0, 1.0], [8.0, 3.0, 1.0], [4.0, 6.0, 8.0]]], [[[6.0, 5.0, 4.0], [2.0, 
1.0, 5.0], [4.0, 3.0, 2.0]], [[6.0, 1.0, 6.0], [8.0, 7.0, 4.0], [8.0, 1.0, 2.0]], [[4.0, 8.0, 3.0], [6.0, 3.0, 2.0], [3.0, 6.0, 
8.0]]], [[[2.0, 8.0, 1.0], [1.0, 2.0, 4.0], [7.0, 8.0, 3.0]], [[4.0, 7.0, 8.0], [4.0, 7.0, 7.0], [5.0, 6.0, 5.0]], [[2.0, 7.0, 
4.0], [3.0, 4.0, 2.0], [2.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 1.0], [5.0, 8.0, 6.0], [3.0, 6.0, 2.0]], 
[[6.0, 2.0, 5.0], [5.0, 1.0, 4.0], [2.0, 8.0, 1.0]], [[1.0, 7.0, 6.0], [6.0, 3.0, 8.0], [4.0, 6.0, 6.0]]], [[[1.0, 3.0, 8.0], 
[4.0, 3.0, 2.0], [2.0, 4.0, 8.0]], [[7.0, 1.0, 2.0], [7.0, 7.0, 2.0], [5.0, 6.0, 6.0]], [[5.0, 6.0, 3.0], [8.0, 6.0, 3.0], 
[7.0, 7.0, 2.0]]], [[[2.0, 5.0, 7.0], [6.0, 8.0, 2.0], [4.0, 2.0, 6.0]], [[5.0, 4.0, 8.0], [4.0, 4.0, 1.0], [5.0, 6.0, 4.0]], 
[[8.0, 6.0, 2.0], [6.0, 2.0, 1.0], [3.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [4.0, 7.0, 5.0], [5.0, 5.0, 7.0]], [[1.0, 7.0, 8.0], 
[8.0, 6.0, 4.0], [8.0, 5.0, 1.0]], [[2.0, 6.0, 2.0], [2.0, 3.0, 3.0], [5.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPUU(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 4.0], [7.0, 7.0, 3.0], [2.0, 2.0, 3.0]], [[4.0, 8.0, 3.0], [4.0, 2.0, 3.0], [8.0, 4.0, 
1.0]], [[5.0, 3.0, 8.0], [2.0, 2.0, 6.0], [1.0, 3.0, 7.0]]], [[[8.0, 1.0, 4.0], [1.0, 1.0, 2.0], [7.0, 8.0, 1.0]], [[1.0, 4.0, 
7.0], [6.0, 3.0, 6.0], [2.0, 6.0, 5.0]], [[6.0, 2.0, 7.0], [1.0, 4.0, 8.0], [6.0, 6.0, 7.0]]], [[[1.0, 2.0, 3.0], [2.0, 1.0, 
5.0], [2.0, 8.0, 7.0]], [[8.0, 5.0, 6.0], [2.0, 1.0, 1.0], [6.0, 4.0, 2.0]], [[4.0, 5.0, 7.0], [4.0, 4.0, 2.0], [6.0, 4.0, 
1.0]]], [[[1.0, 7.0, 3.0], [6.0, 3.0, 3.0], [2.0, 6.0, 8.0]], [[3.0, 1.0, 3.0], [2.0, 4.0, 4.0], [7.0, 3.0, 5.0]], [[6.0, 6.0, 
5.0], [4.0, 6.0, 6.0], [7.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 6.0], [8.0, 1.0, 5.0], [8.0, 6.0, 6.0]], [[7.0, 4.0, 1.0], [3.0, 1.0, 
3.0], [8.0, 6.0, 2.0]], [[1.0, 1.0, 7.0], [2.0, 8.0, 7.0], [3.0, 5.0, 7.0]]], [[[5.0, 8.0, 7.0], [1.0, 7.0, 1.0], [8.0, 2.0, 
2.0]], [[3.0, 4.0, 2.0], [2.0, 7.0, 4.0], [8.0, 2.0, 8.0]], [[1.0, 7.0, 1.0], [1.0, 1.0, 3.0], [5.0, 3.0, 8.0]]], [[[4.0, 6.0, 
3.0], [7.0, 2.0, 8.0], [4.0, 4.0, 8.0]], [[3.0, 4.0, 4.0], [2.0, 3.0, 5.0], [8.0, 7.0, 7.0]], [[1.0, 3.0, 6.0], [3.0, 4.0, 
3.0], [7.0, 4.0, 2.0]]], [[[3.0, 4.0, 8.0], [5.0, 4.0, 2.0], [6.0, 1.0, 1.0]], [[7.0, 1.0, 5.0], [3.0, 6.0, 1.0], [1.0, 4.0, 
7.0]], [[5.0, 4.0, 3.0], [3.0, 8.0, 1.0], [3.0, 4.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[2.0, 8.0], [2.0, 2.0]], [[5.0, 3.0], [5.0, 7.0]]])+(1.-msk_val)*numpy.array([[[8.0, 4.0], 
[6.0, 1.0]], [[6.0, 5.0], [8.0, 5.0]]])
      arg[1:3,1,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 4.0], [7.0, 7.0, 3.0], [2.0, 2.0, 3.0]], [[4.0, 8.0, 3.0], [4.0, 2.0, 3.0], [8.0, 
4.0, 1.0]], [[5.0, 3.0, 8.0], [2.0, 2.0, 6.0], [1.0, 3.0, 7.0]]], [[[8.0, 1.0, 4.0], [1.0, 1.0, 2.0], [7.0, 8.0, 1.0]], [[2.0, 
8.0, 7.0], [2.0, 2.0, 6.0], [2.0, 6.0, 5.0]], [[6.0, 2.0, 7.0], [1.0, 4.0, 8.0], [6.0, 6.0, 7.0]]], [[[1.0, 2.0, 3.0], [2.0, 
1.0, 5.0], [2.0, 8.0, 7.0]], [[5.0, 3.0, 6.0], [5.0, 7.0, 1.0], [6.0, 4.0, 2.0]], [[4.0, 5.0, 7.0], [4.0, 4.0, 2.0], [6.0, 4.0, 
1.0]]], [[[1.0, 7.0, 3.0], [6.0, 3.0, 3.0], [2.0, 6.0, 8.0]], [[3.0, 1.0, 3.0], [2.0, 4.0, 4.0], [7.0, 3.0, 5.0]], [[6.0, 6.0, 
5.0], [4.0, 6.0, 6.0], [7.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 6.0], [8.0, 1.0, 5.0], [8.0, 6.0, 6.0]], 
[[7.0, 4.0, 1.0], [3.0, 1.0, 3.0], [8.0, 6.0, 2.0]], [[1.0, 1.0, 7.0], [2.0, 8.0, 7.0], [3.0, 5.0, 7.0]]], [[[5.0, 8.0, 7.0], 
[1.0, 7.0, 1.0], [8.0, 2.0, 2.0]], [[8.0, 4.0, 2.0], [6.0, 1.0, 4.0], [8.0, 2.0, 8.0]], [[1.0, 7.0, 1.0], [1.0, 1.0, 3.0], 
[5.0, 3.0, 8.0]]], [[[4.0, 6.0, 3.0], [7.0, 2.0, 8.0], [4.0, 4.0, 8.0]], [[6.0, 5.0, 4.0], [8.0, 5.0, 5.0], [8.0, 7.0, 7.0]], 
[[1.0, 3.0, 6.0], [3.0, 4.0, 3.0], [7.0, 4.0, 2.0]]], [[[3.0, 4.0, 8.0], [5.0, 4.0, 2.0], [6.0, 1.0, 1.0]], [[7.0, 1.0, 5.0], 
[3.0, 6.0, 1.0], [1.0, 4.0, 7.0]], [[5.0, 4.0, 3.0], [3.0, 8.0, 1.0], [3.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPLL(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 5.0], [2.0, 5.0, 7.0], [5.0, 6.0, 3.0]], [[7.0, 8.0, 8.0], [1.0, 1.0, 2.0], [1.0, 7.0, 
7.0]], [[5.0, 5.0, 6.0], [8.0, 3.0, 3.0], [5.0, 3.0, 3.0]]], [[[1.0, 1.0, 7.0], [5.0, 4.0, 4.0], [2.0, 4.0, 8.0]], [[2.0, 6.0, 
8.0], [4.0, 3.0, 3.0], [2.0, 6.0, 1.0]], [[4.0, 2.0, 2.0], [2.0, 8.0, 2.0], [2.0, 7.0, 1.0]]], [[[3.0, 1.0, 6.0], [2.0, 8.0, 
5.0], [7.0, 3.0, 5.0]], [[4.0, 2.0, 5.0], [8.0, 3.0, 8.0], [3.0, 3.0, 7.0]], [[4.0, 8.0, 2.0], [7.0, 7.0, 3.0], [8.0, 3.0, 
4.0]]], [[[5.0, 7.0, 4.0], [7.0, 8.0, 1.0], [3.0, 7.0, 1.0]], [[8.0, 2.0, 7.0], [6.0, 6.0, 2.0], [1.0, 2.0, 8.0]], [[1.0, 5.0, 
4.0], [3.0, 6.0, 3.0], [8.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 6.0], [5.0, 8.0, 1.0], [7.0, 1.0, 4.0]], [[3.0, 5.0, 6.0], [2.0, 6.0, 
7.0], [6.0, 5.0, 7.0]], [[7.0, 2.0, 4.0], [3.0, 5.0, 3.0], [6.0, 4.0, 4.0]]], [[[6.0, 4.0, 5.0], [2.0, 3.0, 8.0], [5.0, 7.0, 
2.0]], [[1.0, 1.0, 5.0], [4.0, 8.0, 1.0], [6.0, 2.0, 8.0]], [[4.0, 3.0, 6.0], [2.0, 6.0, 5.0], [2.0, 8.0, 4.0]]], [[[6.0, 2.0, 
5.0], [8.0, 2.0, 5.0], [7.0, 1.0, 4.0]], [[8.0, 6.0, 1.0], [5.0, 6.0, 5.0], [5.0, 5.0, 4.0]], [[4.0, 6.0, 1.0], [1.0, 8.0, 
4.0], [5.0, 3.0, 8.0]]], [[[5.0, 4.0, 7.0], [2.0, 2.0, 3.0], [8.0, 5.0, 5.0]], [[4.0, 6.0, 8.0], [7.0, 5.0, 6.0], [1.0, 8.0, 
6.0]], [[1.0, 5.0, 4.0], [5.0, 5.0, 1.0], [6.0, 5.0, 3.0]]]]))
      val=numpy.array([[[6.0]], [[7.0]]])
      arg[1:3,1,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 5.0], [2.0, 5.0, 7.0], [5.0, 6.0, 3.0]], [[7.0, 8.0, 8.0], [1.0, 1.0, 2.0], [1.0, 
7.0, 7.0]], [[5.0, 5.0, 6.0], [8.0, 3.0, 3.0], [5.0, 3.0, 3.0]]], [[[1.0, 1.0, 7.0], [5.0, 4.0, 4.0], [2.0, 4.0, 8.0]], [[2.0, 
6.0, 8.0], [4.0, 3.0, 3.0], [2.0, 6.0, 6.0]], [[4.0, 2.0, 2.0], [2.0, 8.0, 2.0], [2.0, 7.0, 1.0]]], [[[3.0, 1.0, 6.0], [2.0, 
8.0, 5.0], [7.0, 3.0, 5.0]], [[4.0, 2.0, 5.0], [8.0, 3.0, 8.0], [3.0, 3.0, 7.0]], [[4.0, 8.0, 2.0], [7.0, 7.0, 3.0], [8.0, 3.0, 
4.0]]], [[[5.0, 7.0, 4.0], [7.0, 8.0, 1.0], [3.0, 7.0, 1.0]], [[8.0, 2.0, 7.0], [6.0, 6.0, 2.0], [1.0, 2.0, 8.0]], [[1.0, 5.0, 
4.0], [3.0, 6.0, 3.0], [8.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 6.0], [5.0, 8.0, 1.0], [7.0, 1.0, 4.0]], 
[[3.0, 5.0, 6.0], [2.0, 6.0, 7.0], [6.0, 5.0, 7.0]], [[7.0, 2.0, 4.0], [3.0, 5.0, 3.0], [6.0, 4.0, 4.0]]], [[[6.0, 4.0, 5.0], 
[2.0, 3.0, 8.0], [5.0, 7.0, 2.0]], [[1.0, 1.0, 5.0], [4.0, 8.0, 1.0], [6.0, 2.0, 6.0]], [[4.0, 3.0, 6.0], [2.0, 6.0, 5.0], 
[2.0, 8.0, 4.0]]], [[[6.0, 2.0, 5.0], [8.0, 2.0, 5.0], [7.0, 1.0, 4.0]], [[8.0, 6.0, 1.0], [5.0, 6.0, 5.0], [5.0, 5.0, 7.0]], 
[[4.0, 6.0, 1.0], [1.0, 8.0, 4.0], [5.0, 3.0, 8.0]]], [[[5.0, 4.0, 7.0], [2.0, 2.0, 3.0], [8.0, 5.0, 5.0]], [[4.0, 6.0, 8.0], 
[7.0, 5.0, 6.0], [1.0, 8.0, 6.0]], [[1.0, 5.0, 4.0], [5.0, 5.0, 1.0], [6.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPLL(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 1.0], [4.0, 4.0, 3.0], [4.0, 8.0, 2.0]], [[7.0, 3.0, 6.0], [3.0, 8.0, 8.0], [5.0, 8.0, 
6.0]], [[3.0, 8.0, 8.0], [8.0, 8.0, 3.0], [5.0, 6.0, 5.0]]], [[[3.0, 6.0, 8.0], [4.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 7.0, 
7.0], [8.0, 6.0, 4.0], [2.0, 5.0, 1.0]], [[3.0, 4.0, 4.0], [4.0, 5.0, 8.0], [3.0, 5.0, 8.0]]], [[[6.0, 2.0, 7.0], [1.0, 7.0, 
1.0], [6.0, 3.0, 7.0]], [[5.0, 2.0, 4.0], [2.0, 1.0, 8.0], [3.0, 7.0, 4.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 4.0], [5.0, 7.0, 
6.0]]], [[[2.0, 8.0, 6.0], [7.0, 6.0, 2.0], [4.0, 4.0, 3.0]], [[7.0, 6.0, 2.0], [2.0, 1.0, 5.0], [1.0, 7.0, 8.0]], [[5.0, 4.0, 
3.0], [7.0, 4.0, 8.0], [6.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 1.0], [8.0, 2.0, 6.0], [4.0, 7.0, 7.0]], [[4.0, 3.0, 8.0], [6.0, 6.0, 
7.0], [6.0, 2.0, 5.0]], [[2.0, 8.0, 5.0], [2.0, 2.0, 1.0], [6.0, 3.0, 2.0]]], [[[7.0, 2.0, 5.0], [5.0, 6.0, 7.0], [6.0, 7.0, 
2.0]], [[8.0, 2.0, 5.0], [7.0, 2.0, 4.0], [6.0, 1.0, 1.0]], [[1.0, 6.0, 8.0], [1.0, 4.0, 6.0], [5.0, 5.0, 1.0]]], [[[5.0, 4.0, 
2.0], [6.0, 6.0, 6.0], [8.0, 4.0, 7.0]], [[1.0, 5.0, 7.0], [3.0, 6.0, 6.0], [2.0, 2.0, 3.0]], [[7.0, 8.0, 7.0], [4.0, 1.0, 
5.0], [2.0, 6.0, 4.0]]], [[[7.0, 1.0, 3.0], [4.0, 4.0, 4.0], [1.0, 4.0, 8.0]], [[4.0, 6.0, 8.0], [6.0, 5.0, 2.0], [1.0, 3.0, 
4.0]], [[8.0, 1.0, 4.0], [4.0, 6.0, 5.0], [5.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[[2.0]], [[4.0]]]),self.functionspace)
      arg[1:3,1,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 1.0], [4.0, 4.0, 3.0], [4.0, 8.0, 2.0]], [[7.0, 3.0, 6.0], [3.0, 8.0, 8.0], [5.0, 
8.0, 6.0]], [[3.0, 8.0, 8.0], [8.0, 8.0, 3.0], [5.0, 6.0, 5.0]]], [[[3.0, 6.0, 8.0], [4.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 
7.0, 7.0], [8.0, 6.0, 4.0], [2.0, 5.0, 2.0]], [[3.0, 4.0, 4.0], [4.0, 5.0, 8.0], [3.0, 5.0, 8.0]]], [[[6.0, 2.0, 7.0], [1.0, 
7.0, 1.0], [6.0, 3.0, 7.0]], [[5.0, 2.0, 4.0], [2.0, 1.0, 8.0], [3.0, 7.0, 4.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 4.0], [5.0, 7.0, 
6.0]]], [[[2.0, 8.0, 6.0], [7.0, 6.0, 2.0], [4.0, 4.0, 3.0]], [[7.0, 6.0, 2.0], [2.0, 1.0, 5.0], [1.0, 7.0, 8.0]], [[5.0, 4.0, 
3.0], [7.0, 4.0, 8.0], [6.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 1.0], [8.0, 2.0, 6.0], [4.0, 7.0, 7.0]], 
[[4.0, 3.0, 8.0], [6.0, 6.0, 7.0], [6.0, 2.0, 5.0]], [[2.0, 8.0, 5.0], [2.0, 2.0, 1.0], [6.0, 3.0, 2.0]]], [[[7.0, 2.0, 5.0], 
[5.0, 6.0, 7.0], [6.0, 7.0, 2.0]], [[8.0, 2.0, 5.0], [7.0, 2.0, 4.0], [6.0, 1.0, 2.0]], [[1.0, 6.0, 8.0], [1.0, 4.0, 6.0], 
[5.0, 5.0, 1.0]]], [[[5.0, 4.0, 2.0], [6.0, 6.0, 6.0], [8.0, 4.0, 7.0]], [[1.0, 5.0, 7.0], [3.0, 6.0, 6.0], [2.0, 2.0, 4.0]], 
[[7.0, 8.0, 7.0], [4.0, 1.0, 5.0], [2.0, 6.0, 4.0]]], [[[7.0, 1.0, 3.0], [4.0, 4.0, 4.0], [1.0, 4.0, 8.0]], [[4.0, 6.0, 8.0], 
[6.0, 5.0, 2.0], [1.0, 3.0, 4.0]], [[8.0, 1.0, 4.0], [4.0, 6.0, 5.0], [5.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPLL(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 2.0], [6.0, 7.0, 2.0], [2.0, 6.0, 3.0]], [[3.0, 6.0, 8.0], [5.0, 6.0, 4.0], [6.0, 8.0, 
6.0]], [[2.0, 7.0, 7.0], [5.0, 5.0, 3.0], [8.0, 7.0, 3.0]]], [[[6.0, 6.0, 8.0], [1.0, 3.0, 7.0], [2.0, 6.0, 3.0]], [[4.0, 4.0, 
6.0], [4.0, 4.0, 8.0], [2.0, 8.0, 5.0]], [[3.0, 6.0, 5.0], [6.0, 6.0, 2.0], [7.0, 7.0, 8.0]]], [[[7.0, 8.0, 8.0], [8.0, 1.0, 
1.0], [3.0, 8.0, 3.0]], [[7.0, 2.0, 4.0], [3.0, 4.0, 5.0], [1.0, 8.0, 6.0]], [[1.0, 7.0, 7.0], [8.0, 1.0, 3.0], [1.0, 7.0, 
5.0]]], [[[6.0, 5.0, 7.0], [8.0, 5.0, 7.0], [8.0, 3.0, 3.0]], [[6.0, 6.0, 7.0], [3.0, 8.0, 7.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 
6.0], [3.0, 1.0, 2.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 8.0], [2.0, 7.0, 2.0], [6.0, 3.0, 5.0]], [[8.0, 7.0, 8.0], [4.0, 5.0, 
2.0], [3.0, 6.0, 7.0]], [[2.0, 4.0, 1.0], [4.0, 8.0, 8.0], [4.0, 4.0, 6.0]]], [[[4.0, 5.0, 7.0], [4.0, 3.0, 3.0], [8.0, 1.0, 
1.0]], [[5.0, 8.0, 5.0], [8.0, 1.0, 7.0], [3.0, 1.0, 1.0]], [[7.0, 8.0, 4.0], [1.0, 3.0, 7.0], [8.0, 4.0, 3.0]]], [[[6.0, 7.0, 
6.0], [6.0, 4.0, 6.0], [7.0, 2.0, 3.0]], [[2.0, 1.0, 2.0], [1.0, 8.0, 4.0], [1.0, 6.0, 2.0]], [[4.0, 7.0, 4.0], [8.0, 6.0, 
8.0], [8.0, 1.0, 4.0]]], [[[8.0, 5.0, 4.0], [3.0, 8.0, 6.0], [3.0, 2.0, 6.0]], [[7.0, 7.0, 4.0], [5.0, 7.0, 7.0], [1.0, 8.0, 
3.0]], [[7.0, 1.0, 4.0], [3.0, 5.0, 7.0], [6.0, 1.0, 2.0]]]]))
      val=Data(numpy.array([[[4.0]], [[2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0]], [[4.0]]]))
      arg[1:3,1,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 2.0], [6.0, 7.0, 2.0], [2.0, 6.0, 3.0]], [[3.0, 6.0, 8.0], [5.0, 6.0, 4.0], [6.0, 
8.0, 6.0]], [[2.0, 7.0, 7.0], [5.0, 5.0, 3.0], [8.0, 7.0, 3.0]]], [[[6.0, 6.0, 8.0], [1.0, 3.0, 7.0], [2.0, 6.0, 3.0]], [[4.0, 
4.0, 6.0], [4.0, 4.0, 8.0], [2.0, 8.0, 4.0]], [[3.0, 6.0, 5.0], [6.0, 6.0, 2.0], [7.0, 7.0, 8.0]]], [[[7.0, 8.0, 8.0], [8.0, 
1.0, 1.0], [3.0, 8.0, 3.0]], [[7.0, 2.0, 4.0], [3.0, 4.0, 5.0], [1.0, 8.0, 2.0]], [[1.0, 7.0, 7.0], [8.0, 1.0, 3.0], [1.0, 7.0, 
5.0]]], [[[6.0, 5.0, 7.0], [8.0, 5.0, 7.0], [8.0, 3.0, 3.0]], [[6.0, 6.0, 7.0], [3.0, 8.0, 7.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 
6.0], [3.0, 1.0, 2.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 8.0], [2.0, 7.0, 2.0], [6.0, 3.0, 5.0]], 
[[8.0, 7.0, 8.0], [4.0, 5.0, 2.0], [3.0, 6.0, 7.0]], [[2.0, 4.0, 1.0], [4.0, 8.0, 8.0], [4.0, 4.0, 6.0]]], [[[4.0, 5.0, 7.0], 
[4.0, 3.0, 3.0], [8.0, 1.0, 1.0]], [[5.0, 8.0, 5.0], [8.0, 1.0, 7.0], [3.0, 1.0, 2.0]], [[7.0, 8.0, 4.0], [1.0, 3.0, 7.0], 
[8.0, 4.0, 3.0]]], [[[6.0, 7.0, 6.0], [6.0, 4.0, 6.0], [7.0, 2.0, 3.0]], [[2.0, 1.0, 2.0], [1.0, 8.0, 4.0], [1.0, 6.0, 4.0]], 
[[4.0, 7.0, 4.0], [8.0, 6.0, 8.0], [8.0, 1.0, 4.0]]], [[[8.0, 5.0, 4.0], [3.0, 8.0, 6.0], [3.0, 2.0, 6.0]], [[7.0, 7.0, 4.0], 
[5.0, 7.0, 7.0], [1.0, 8.0, 3.0]], [[7.0, 1.0, 4.0], [3.0, 5.0, 7.0], [6.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPLL(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 5.0], [4.0, 8.0, 1.0], [6.0, 3.0, 7.0]], [[5.0, 6.0, 8.0], [2.0, 1.0, 1.0], [1.0, 4.0, 
3.0]], [[6.0, 3.0, 5.0], [1.0, 2.0, 1.0], [8.0, 6.0, 6.0]]], [[[6.0, 6.0, 8.0], [5.0, 2.0, 5.0], [5.0, 8.0, 3.0]], [[4.0, 3.0, 
3.0], [8.0, 2.0, 5.0], [4.0, 3.0, 8.0]], [[2.0, 2.0, 3.0], [5.0, 1.0, 7.0], [1.0, 8.0, 7.0]]], [[[2.0, 7.0, 3.0], [3.0, 2.0, 
2.0], [7.0, 2.0, 7.0]], [[6.0, 7.0, 8.0], [2.0, 7.0, 6.0], [2.0, 5.0, 7.0]], [[4.0, 7.0, 1.0], [4.0, 1.0, 7.0], [3.0, 4.0, 
4.0]]], [[[7.0, 6.0, 3.0], [7.0, 6.0, 4.0], [5.0, 4.0, 7.0]], [[3.0, 4.0, 1.0], [6.0, 5.0, 7.0], [4.0, 2.0, 8.0]], [[1.0, 2.0, 
4.0], [5.0, 2.0, 4.0], [7.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 7.0], [6.0, 4.0, 3.0], [5.0, 1.0, 5.0]], [[4.0, 8.0, 7.0], [1.0, 8.0, 
4.0], [1.0, 7.0, 6.0]], [[6.0, 7.0, 3.0], [7.0, 1.0, 7.0], [8.0, 2.0, 8.0]]], [[[3.0, 1.0, 4.0], [5.0, 7.0, 2.0], [7.0, 6.0, 
4.0]], [[1.0, 3.0, 1.0], [4.0, 1.0, 3.0], [6.0, 5.0, 4.0]], [[3.0, 4.0, 5.0], [5.0, 8.0, 7.0], [1.0, 7.0, 4.0]]], [[[6.0, 2.0, 
6.0], [2.0, 7.0, 6.0], [2.0, 1.0, 5.0]], [[5.0, 3.0, 8.0], [2.0, 4.0, 3.0], [3.0, 6.0, 4.0]], [[4.0, 7.0, 2.0], [8.0, 1.0, 
8.0], [6.0, 5.0, 6.0]]], [[[6.0, 2.0, 3.0], [1.0, 8.0, 2.0], [2.0, 3.0, 5.0]], [[2.0, 5.0, 8.0], [6.0, 7.0, 2.0], [6.0, 4.0, 
8.0]], [[3.0, 5.0, 1.0], [3.0, 7.0, 7.0], [5.0, 2.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0]], [[3.0]]])+(1.-msk_val)*numpy.array([[[6.0]], [[2.0]]])
      arg[1:3,1,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 5.0], [4.0, 8.0, 1.0], [6.0, 3.0, 7.0]], [[5.0, 6.0, 8.0], [2.0, 1.0, 1.0], [1.0, 
4.0, 3.0]], [[6.0, 3.0, 5.0], [1.0, 2.0, 1.0], [8.0, 6.0, 6.0]]], [[[6.0, 6.0, 8.0], [5.0, 2.0, 5.0], [5.0, 8.0, 3.0]], [[4.0, 
3.0, 3.0], [8.0, 2.0, 5.0], [4.0, 3.0, 5.0]], [[2.0, 2.0, 3.0], [5.0, 1.0, 7.0], [1.0, 8.0, 7.0]]], [[[2.0, 7.0, 3.0], [3.0, 
2.0, 2.0], [7.0, 2.0, 7.0]], [[6.0, 7.0, 8.0], [2.0, 7.0, 6.0], [2.0, 5.0, 3.0]], [[4.0, 7.0, 1.0], [4.0, 1.0, 7.0], [3.0, 4.0, 
4.0]]], [[[7.0, 6.0, 3.0], [7.0, 6.0, 4.0], [5.0, 4.0, 7.0]], [[3.0, 4.0, 1.0], [6.0, 5.0, 7.0], [4.0, 2.0, 8.0]], [[1.0, 2.0, 
4.0], [5.0, 2.0, 4.0], [7.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 7.0], [6.0, 4.0, 3.0], [5.0, 1.0, 5.0]], 
[[4.0, 8.0, 7.0], [1.0, 8.0, 4.0], [1.0, 7.0, 6.0]], [[6.0, 7.0, 3.0], [7.0, 1.0, 7.0], [8.0, 2.0, 8.0]]], [[[3.0, 1.0, 4.0], 
[5.0, 7.0, 2.0], [7.0, 6.0, 4.0]], [[1.0, 3.0, 1.0], [4.0, 1.0, 3.0], [6.0, 5.0, 6.0]], [[3.0, 4.0, 5.0], [5.0, 8.0, 7.0], 
[1.0, 7.0, 4.0]]], [[[6.0, 2.0, 6.0], [2.0, 7.0, 6.0], [2.0, 1.0, 5.0]], [[5.0, 3.0, 8.0], [2.0, 4.0, 3.0], [3.0, 6.0, 2.0]], 
[[4.0, 7.0, 2.0], [8.0, 1.0, 8.0], [6.0, 5.0, 6.0]]], [[[6.0, 2.0, 3.0], [1.0, 8.0, 2.0], [2.0, 3.0, 5.0]], [[2.0, 5.0, 8.0], 
[6.0, 7.0, 2.0], [6.0, 4.0, 8.0]], [[3.0, 5.0, 1.0], [3.0, 7.0, 7.0], [5.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RLCN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 3.0], [4.0, 5.0, 1.0], [4.0, 6.0, 8.0]], [[1.0, 7.0, 7.0], [8.0, 5.0, 4.0], [5.0, 4.0, 
6.0]], [[4.0, 2.0, 5.0], [8.0, 4.0, 2.0], [3.0, 5.0, 7.0]]], [[[2.0, 4.0, 2.0], [6.0, 1.0, 4.0], [7.0, 7.0, 5.0]], [[4.0, 2.0, 
1.0], [8.0, 2.0, 3.0], [4.0, 6.0, 5.0]], [[4.0, 2.0, 1.0], [7.0, 5.0, 2.0], [6.0, 1.0, 5.0]]], [[[4.0, 1.0, 7.0], [1.0, 6.0, 
6.0], [7.0, 2.0, 2.0]], [[7.0, 1.0, 7.0], [4.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[3.0, 5.0, 3.0], [1.0, 5.0, 5.0], [4.0, 3.0, 
1.0]]], [[[8.0, 1.0, 3.0], [1.0, 2.0, 2.0], [5.0, 4.0, 6.0]], [[2.0, 6.0, 8.0], [3.0, 1.0, 4.0], [5.0, 8.0, 2.0]], [[5.0, 1.0, 
7.0], [5.0, 5.0, 2.0], [8.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 3.0], [7.0, 8.0, 2.0], [5.0, 8.0, 3.0]], [[3.0, 3.0, 5.0], [6.0, 6.0, 
8.0], [8.0, 5.0, 3.0]], [[8.0, 6.0, 7.0], [7.0, 3.0, 8.0], [4.0, 5.0, 6.0]]], [[[4.0, 6.0, 6.0], [1.0, 4.0, 8.0], [2.0, 3.0, 
4.0]], [[8.0, 5.0, 5.0], [1.0, 4.0, 1.0], [3.0, 6.0, 4.0]], [[1.0, 7.0, 3.0], [3.0, 7.0, 8.0], [1.0, 3.0, 7.0]]], [[[5.0, 2.0, 
3.0], [8.0, 2.0, 6.0], [4.0, 5.0, 6.0]], [[6.0, 2.0, 8.0], [1.0, 8.0, 2.0], [5.0, 3.0, 7.0]], [[3.0, 3.0, 8.0], [4.0, 7.0, 
5.0], [2.0, 7.0, 3.0]]], [[[6.0, 3.0, 1.0], [3.0, 3.0, 5.0], [8.0, 7.0, 7.0]], [[7.0, 5.0, 8.0], [6.0, 1.0, 8.0], [2.0, 6.0, 
8.0]], [[8.0, 6.0, 8.0], [7.0, 3.0, 4.0], [4.0, 8.0, 1.0]]]]))
      val=numpy.array([[[[4.0, 7.0, 5.0], [5.0, 7.0, 2.0], [3.0, 8.0, 7.0]]], [[[1.0, 7.0, 1.0], [6.0, 6.0, 7.0], [3.0, 8.0, 
6.0]]]])
      arg[1:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 3.0], [4.0, 5.0, 1.0], [4.0, 6.0, 8.0]], [[1.0, 7.0, 7.0], [8.0, 5.0, 4.0], [5.0, 
4.0, 6.0]], [[4.0, 2.0, 5.0], [8.0, 4.0, 2.0], [3.0, 5.0, 7.0]]], [[[2.0, 4.0, 2.0], [6.0, 1.0, 4.0], [7.0, 7.0, 5.0]], [[4.0, 
2.0, 1.0], [8.0, 2.0, 3.0], [4.0, 6.0, 5.0]], [[4.0, 7.0, 5.0], [5.0, 7.0, 2.0], [3.0, 8.0, 7.0]]], [[[4.0, 1.0, 7.0], [1.0, 
6.0, 6.0], [7.0, 2.0, 2.0]], [[7.0, 1.0, 7.0], [4.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[1.0, 7.0, 1.0], [6.0, 6.0, 7.0], [3.0, 8.0, 
6.0]]], [[[8.0, 1.0, 3.0], [1.0, 2.0, 2.0], [5.0, 4.0, 6.0]], [[2.0, 6.0, 8.0], [3.0, 1.0, 4.0], [5.0, 8.0, 2.0]], [[5.0, 1.0, 
7.0], [5.0, 5.0, 2.0], [8.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 3.0], [7.0, 8.0, 2.0], [5.0, 8.0, 3.0]], 
[[3.0, 3.0, 5.0], [6.0, 6.0, 8.0], [8.0, 5.0, 3.0]], [[8.0, 6.0, 7.0], [7.0, 3.0, 8.0], [4.0, 5.0, 6.0]]], [[[4.0, 6.0, 6.0], 
[1.0, 4.0, 8.0], [2.0, 3.0, 4.0]], [[8.0, 5.0, 5.0], [1.0, 4.0, 1.0], [3.0, 6.0, 4.0]], [[4.0, 7.0, 5.0], [5.0, 7.0, 2.0], 
[3.0, 8.0, 7.0]]], [[[5.0, 2.0, 3.0], [8.0, 2.0, 6.0], [4.0, 5.0, 6.0]], [[6.0, 2.0, 8.0], [1.0, 8.0, 2.0], [5.0, 3.0, 7.0]], 
[[1.0, 7.0, 1.0], [6.0, 6.0, 7.0], [3.0, 8.0, 6.0]]], [[[6.0, 3.0, 1.0], [3.0, 3.0, 5.0], [8.0, 7.0, 7.0]], [[7.0, 5.0, 8.0], 
[6.0, 1.0, 8.0], [2.0, 6.0, 8.0]], [[8.0, 6.0, 8.0], [7.0, 3.0, 4.0], [4.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RLCN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 6.0], [4.0, 6.0, 6.0], [3.0, 6.0, 6.0]], [[8.0, 4.0, 3.0], [6.0, 7.0, 4.0], [5.0, 1.0, 
5.0]], [[5.0, 4.0, 1.0], [1.0, 3.0, 4.0], [8.0, 2.0, 6.0]]], [[[4.0, 2.0, 3.0], [7.0, 6.0, 6.0], [5.0, 4.0, 5.0]], [[2.0, 2.0, 
8.0], [4.0, 7.0, 1.0], [8.0, 3.0, 1.0]], [[8.0, 1.0, 6.0], [2.0, 7.0, 6.0], [3.0, 2.0, 8.0]]], [[[5.0, 7.0, 2.0], [5.0, 1.0, 
4.0], [1.0, 5.0, 1.0]], [[5.0, 1.0, 5.0], [2.0, 8.0, 1.0], [1.0, 2.0, 3.0]], [[8.0, 2.0, 7.0], [2.0, 6.0, 8.0], [2.0, 1.0, 
8.0]]], [[[6.0, 1.0, 4.0], [8.0, 7.0, 6.0], [7.0, 4.0, 5.0]], [[3.0, 2.0, 1.0], [5.0, 3.0, 2.0], [5.0, 3.0, 4.0]], [[6.0, 4.0, 
7.0], [3.0, 3.0, 1.0], [6.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 7.0], [4.0, 5.0, 3.0], [8.0, 2.0, 3.0]], [[7.0, 1.0, 5.0], [5.0, 6.0, 
7.0], [7.0, 7.0, 1.0]], [[8.0, 8.0, 5.0], [8.0, 6.0, 7.0], [2.0, 2.0, 4.0]]], [[[7.0, 7.0, 1.0], [4.0, 7.0, 7.0], [3.0, 5.0, 
4.0]], [[5.0, 4.0, 4.0], [4.0, 6.0, 4.0], [2.0, 7.0, 1.0]], [[7.0, 3.0, 6.0], [8.0, 4.0, 3.0], [1.0, 6.0, 4.0]]], [[[8.0, 4.0, 
1.0], [8.0, 3.0, 7.0], [6.0, 6.0, 4.0]], [[4.0, 8.0, 3.0], [8.0, 2.0, 5.0], [2.0, 5.0, 6.0]], [[2.0, 5.0, 3.0], [8.0, 2.0, 
8.0], [8.0, 8.0, 8.0]]], [[[2.0, 3.0, 4.0], [5.0, 3.0, 1.0], [8.0, 3.0, 2.0]], [[5.0, 5.0, 3.0], [6.0, 8.0, 3.0], [3.0, 4.0, 
8.0]], [[2.0, 2.0, 2.0], [7.0, 7.0, 6.0], [1.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[[3.0, 8.0, 8.0], [7.0, 8.0, 8.0], [2.0, 8.0, 4.0]]], [[[6.0, 4.0, 2.0], [1.0, 5.0, 2.0], [1.0, 
4.0, 2.0]]]]),self.functionspace)
      arg[1:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 6.0], [4.0, 6.0, 6.0], [3.0, 6.0, 6.0]], [[8.0, 4.0, 3.0], [6.0, 7.0, 4.0], [5.0, 
1.0, 5.0]], [[5.0, 4.0, 1.0], [1.0, 3.0, 4.0], [8.0, 2.0, 6.0]]], [[[4.0, 2.0, 3.0], [7.0, 6.0, 6.0], [5.0, 4.0, 5.0]], [[2.0, 
2.0, 8.0], [4.0, 7.0, 1.0], [8.0, 3.0, 1.0]], [[3.0, 8.0, 8.0], [7.0, 8.0, 8.0], [2.0, 8.0, 4.0]]], [[[5.0, 7.0, 2.0], [5.0, 
1.0, 4.0], [1.0, 5.0, 1.0]], [[5.0, 1.0, 5.0], [2.0, 8.0, 1.0], [1.0, 2.0, 3.0]], [[6.0, 4.0, 2.0], [1.0, 5.0, 2.0], [1.0, 4.0, 
2.0]]], [[[6.0, 1.0, 4.0], [8.0, 7.0, 6.0], [7.0, 4.0, 5.0]], [[3.0, 2.0, 1.0], [5.0, 3.0, 2.0], [5.0, 3.0, 4.0]], [[6.0, 4.0, 
7.0], [3.0, 3.0, 1.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 7.0], [4.0, 5.0, 3.0], [8.0, 2.0, 3.0]], 
[[7.0, 1.0, 5.0], [5.0, 6.0, 7.0], [7.0, 7.0, 1.0]], [[8.0, 8.0, 5.0], [8.0, 6.0, 7.0], [2.0, 2.0, 4.0]]], [[[7.0, 7.0, 1.0], 
[4.0, 7.0, 7.0], [3.0, 5.0, 4.0]], [[5.0, 4.0, 4.0], [4.0, 6.0, 4.0], [2.0, 7.0, 1.0]], [[3.0, 8.0, 8.0], [7.0, 8.0, 8.0], 
[2.0, 8.0, 4.0]]], [[[8.0, 4.0, 1.0], [8.0, 3.0, 7.0], [6.0, 6.0, 4.0]], [[4.0, 8.0, 3.0], [8.0, 2.0, 5.0], [2.0, 5.0, 6.0]], 
[[6.0, 4.0, 2.0], [1.0, 5.0, 2.0], [1.0, 4.0, 2.0]]], [[[2.0, 3.0, 4.0], [5.0, 3.0, 1.0], [8.0, 3.0, 2.0]], [[5.0, 5.0, 3.0], 
[6.0, 8.0, 3.0], [3.0, 4.0, 8.0]], [[2.0, 2.0, 2.0], [7.0, 7.0, 6.0], [1.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RLCN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 3.0], [1.0, 5.0, 2.0], [8.0, 8.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 6.0, 5.0], [8.0, 8.0, 
5.0]], [[2.0, 5.0, 2.0], [3.0, 8.0, 7.0], [5.0, 6.0, 1.0]]], [[[3.0, 5.0, 4.0], [6.0, 6.0, 7.0], [4.0, 3.0, 8.0]], [[8.0, 4.0, 
2.0], [3.0, 8.0, 4.0], [2.0, 2.0, 7.0]], [[5.0, 3.0, 3.0], [5.0, 6.0, 8.0], [8.0, 4.0, 6.0]]], [[[8.0, 6.0, 6.0], [1.0, 1.0, 
6.0], [4.0, 1.0, 3.0]], [[5.0, 4.0, 5.0], [1.0, 8.0, 3.0], [7.0, 7.0, 4.0]], [[5.0, 2.0, 4.0], [4.0, 2.0, 8.0], [7.0, 7.0, 
2.0]]], [[[2.0, 6.0, 4.0], [6.0, 8.0, 7.0], [1.0, 2.0, 2.0]], [[8.0, 2.0, 6.0], [1.0, 2.0, 2.0], [5.0, 4.0, 7.0]], [[5.0, 3.0, 
3.0], [4.0, 4.0, 8.0], [4.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 5.0], [1.0, 3.0, 3.0], [2.0, 8.0, 3.0]], [[8.0, 1.0, 3.0], [3.0, 2.0, 
3.0], [1.0, 2.0, 5.0]], [[4.0, 4.0, 7.0], [5.0, 7.0, 5.0], [7.0, 6.0, 4.0]]], [[[8.0, 1.0, 2.0], [8.0, 8.0, 8.0], [5.0, 5.0, 
4.0]], [[4.0, 7.0, 1.0], [2.0, 1.0, 6.0], [2.0, 1.0, 5.0]], [[3.0, 1.0, 4.0], [2.0, 5.0, 8.0], [5.0, 7.0, 4.0]]], [[[4.0, 8.0, 
5.0], [4.0, 1.0, 5.0], [4.0, 3.0, 5.0]], [[6.0, 5.0, 8.0], [2.0, 6.0, 7.0], [1.0, 2.0, 7.0]], [[7.0, 1.0, 8.0], [4.0, 8.0, 
3.0], [8.0, 1.0, 1.0]]], [[[6.0, 6.0, 4.0], [1.0, 8.0, 7.0], [1.0, 5.0, 5.0]], [[1.0, 7.0, 6.0], [5.0, 3.0, 3.0], [8.0, 1.0, 
1.0]], [[2.0, 7.0, 3.0], [7.0, 3.0, 7.0], [5.0, 2.0, 6.0]]]]))
      val=Data(numpy.array([[[[8.0, 5.0, 6.0], [2.0, 6.0, 8.0], [3.0, 1.0, 1.0]]], [[[8.0, 1.0, 4.0], [7.0, 3.0, 1.0], [8.0, 
3.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 5.0], [6.0, 4.0, 4.0], [8.0, 8.0, 6.0]]], [[[7.0, 4.0, 6.0], [3.0, 5.0, 
5.0], [4.0, 4.0, 8.0]]]]))
      arg[1:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [1.0, 5.0, 2.0], [8.0, 8.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 6.0, 5.0], [8.0, 
8.0, 5.0]], [[2.0, 5.0, 2.0], [3.0, 8.0, 7.0], [5.0, 6.0, 1.0]]], [[[3.0, 5.0, 4.0], [6.0, 6.0, 7.0], [4.0, 3.0, 8.0]], [[8.0, 
4.0, 2.0], [3.0, 8.0, 4.0], [2.0, 2.0, 7.0]], [[8.0, 5.0, 6.0], [2.0, 6.0, 8.0], [3.0, 1.0, 1.0]]], [[[8.0, 6.0, 6.0], [1.0, 
1.0, 6.0], [4.0, 1.0, 3.0]], [[5.0, 4.0, 5.0], [1.0, 8.0, 3.0], [7.0, 7.0, 4.0]], [[8.0, 1.0, 4.0], [7.0, 3.0, 1.0], [8.0, 3.0, 
5.0]]], [[[2.0, 6.0, 4.0], [6.0, 8.0, 7.0], [1.0, 2.0, 2.0]], [[8.0, 2.0, 6.0], [1.0, 2.0, 2.0], [5.0, 4.0, 7.0]], [[5.0, 3.0, 
3.0], [4.0, 4.0, 8.0], [4.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 5.0], [1.0, 3.0, 3.0], [2.0, 8.0, 3.0]], 
[[8.0, 1.0, 3.0], [3.0, 2.0, 3.0], [1.0, 2.0, 5.0]], [[4.0, 4.0, 7.0], [5.0, 7.0, 5.0], [7.0, 6.0, 4.0]]], [[[8.0, 1.0, 2.0], 
[8.0, 8.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 7.0, 1.0], [2.0, 1.0, 6.0], [2.0, 1.0, 5.0]], [[8.0, 1.0, 5.0], [6.0, 4.0, 4.0], 
[8.0, 8.0, 6.0]]], [[[4.0, 8.0, 5.0], [4.0, 1.0, 5.0], [4.0, 3.0, 5.0]], [[6.0, 5.0, 8.0], [2.0, 6.0, 7.0], [1.0, 2.0, 7.0]], 
[[7.0, 4.0, 6.0], [3.0, 5.0, 5.0], [4.0, 4.0, 8.0]]], [[[6.0, 6.0, 4.0], [1.0, 8.0, 7.0], [1.0, 5.0, 5.0]], [[1.0, 7.0, 6.0], 
[5.0, 3.0, 3.0], [8.0, 1.0, 1.0]], [[2.0, 7.0, 3.0], [7.0, 3.0, 7.0], [5.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RLCN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 2.0], [5.0, 4.0, 3.0], [1.0, 5.0, 5.0]], [[7.0, 5.0, 5.0], [7.0, 7.0, 2.0], [5.0, 6.0, 
8.0]], [[7.0, 6.0, 6.0], [3.0, 2.0, 4.0], [7.0, 3.0, 7.0]]], [[[4.0, 2.0, 4.0], [7.0, 3.0, 1.0], [8.0, 6.0, 8.0]], [[6.0, 7.0, 
4.0], [1.0, 4.0, 1.0], [7.0, 7.0, 4.0]], [[2.0, 1.0, 5.0], [2.0, 4.0, 4.0], [8.0, 1.0, 5.0]]], [[[5.0, 3.0, 2.0], [4.0, 6.0, 
4.0], [5.0, 7.0, 1.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 6.0], [8.0, 1.0, 5.0]], [[3.0, 2.0, 1.0], [5.0, 4.0, 6.0], [4.0, 7.0, 
3.0]]], [[[1.0, 1.0, 3.0], [3.0, 3.0, 6.0], [1.0, 8.0, 7.0]], [[7.0, 8.0, 6.0], [4.0, 7.0, 8.0], [7.0, 1.0, 4.0]], [[2.0, 7.0, 
1.0], [5.0, 8.0, 3.0], [7.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 7.0], [6.0, 3.0, 3.0], [7.0, 1.0, 4.0]], [[7.0, 7.0, 7.0], [1.0, 1.0, 
1.0], [7.0, 5.0, 8.0]], [[3.0, 7.0, 5.0], [3.0, 5.0, 2.0], [7.0, 5.0, 3.0]]], [[[7.0, 7.0, 1.0], [1.0, 7.0, 1.0], [1.0, 1.0, 
4.0]], [[3.0, 5.0, 8.0], [7.0, 2.0, 7.0], [5.0, 7.0, 5.0]], [[8.0, 4.0, 8.0], [4.0, 2.0, 2.0], [2.0, 6.0, 4.0]]], [[[4.0, 2.0, 
1.0], [2.0, 3.0, 6.0], [1.0, 4.0, 6.0]], [[1.0, 8.0, 4.0], [6.0, 7.0, 7.0], [1.0, 6.0, 5.0]], [[4.0, 4.0, 2.0], [5.0, 5.0, 
8.0], [3.0, 2.0, 2.0]]], [[[4.0, 1.0, 6.0], [7.0, 4.0, 2.0], [7.0, 6.0, 5.0]], [[6.0, 1.0, 3.0], [5.0, 8.0, 7.0], [4.0, 6.0, 
6.0]], [[5.0, 2.0, 7.0], [1.0, 1.0, 5.0], [7.0, 4.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 5.0, 2.0], [7.0, 5.0, 6.0], [1.0, 4.0, 4.0]]], [[[4.0, 8.0, 1.0], [7.0, 1.0, 6.0], [7.0, 
8.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 6.0, 4.0], [5.0, 3.0, 2.0], [6.0, 3.0, 2.0]]], [[[6.0, 2.0, 2.0], [6.0, 2.0, 
5.0], [6.0, 2.0, 2.0]]]])
      arg[1:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 2.0], [5.0, 4.0, 3.0], [1.0, 5.0, 5.0]], [[7.0, 5.0, 5.0], [7.0, 7.0, 2.0], [5.0, 
6.0, 8.0]], [[7.0, 6.0, 6.0], [3.0, 2.0, 4.0], [7.0, 3.0, 7.0]]], [[[4.0, 2.0, 4.0], [7.0, 3.0, 1.0], [8.0, 6.0, 8.0]], [[6.0, 
7.0, 4.0], [1.0, 4.0, 1.0], [7.0, 7.0, 4.0]], [[5.0, 5.0, 2.0], [7.0, 5.0, 6.0], [1.0, 4.0, 4.0]]], [[[5.0, 3.0, 2.0], [4.0, 
6.0, 4.0], [5.0, 7.0, 1.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 6.0], [8.0, 1.0, 5.0]], [[4.0, 8.0, 1.0], [7.0, 1.0, 6.0], [7.0, 8.0, 
5.0]]], [[[1.0, 1.0, 3.0], [3.0, 3.0, 6.0], [1.0, 8.0, 7.0]], [[7.0, 8.0, 6.0], [4.0, 7.0, 8.0], [7.0, 1.0, 4.0]], [[2.0, 7.0, 
1.0], [5.0, 8.0, 3.0], [7.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 7.0], [6.0, 3.0, 3.0], [7.0, 1.0, 4.0]], 
[[7.0, 7.0, 7.0], [1.0, 1.0, 1.0], [7.0, 5.0, 8.0]], [[3.0, 7.0, 5.0], [3.0, 5.0, 2.0], [7.0, 5.0, 3.0]]], [[[7.0, 7.0, 1.0], 
[1.0, 7.0, 1.0], [1.0, 1.0, 4.0]], [[3.0, 5.0, 8.0], [7.0, 2.0, 7.0], [5.0, 7.0, 5.0]], [[1.0, 6.0, 4.0], [5.0, 3.0, 2.0], 
[6.0, 3.0, 2.0]]], [[[4.0, 2.0, 1.0], [2.0, 3.0, 6.0], [1.0, 4.0, 6.0]], [[1.0, 8.0, 4.0], [6.0, 7.0, 7.0], [1.0, 6.0, 5.0]], 
[[6.0, 2.0, 2.0], [6.0, 2.0, 5.0], [6.0, 2.0, 2.0]]], [[[4.0, 1.0, 6.0], [7.0, 4.0, 2.0], [7.0, 6.0, 5.0]], [[6.0, 1.0, 3.0], 
[5.0, 8.0, 7.0], [4.0, 6.0, 6.0]], [[5.0, 2.0, 7.0], [1.0, 1.0, 5.0], [7.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UCCU(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 5.0], [8.0, 3.0, 4.0], [3.0, 4.0, 1.0]], [[1.0, 4.0, 1.0], [5.0, 8.0, 4.0], [1.0, 7.0, 
1.0]], [[8.0, 4.0, 7.0], [6.0, 4.0, 5.0], [8.0, 1.0, 5.0]]], [[[5.0, 5.0, 6.0], [5.0, 7.0, 5.0], [7.0, 2.0, 7.0]], [[8.0, 8.0, 
1.0], [1.0, 4.0, 7.0], [1.0, 1.0, 6.0]], [[3.0, 7.0, 2.0], [1.0, 4.0, 5.0], [3.0, 6.0, 1.0]]], [[[1.0, 5.0, 6.0], [8.0, 6.0, 
8.0], [6.0, 1.0, 4.0]], [[3.0, 3.0, 4.0], [8.0, 5.0, 1.0], [3.0, 4.0, 5.0]], [[1.0, 6.0, 3.0], [2.0, 3.0, 8.0], [6.0, 7.0, 
6.0]]], [[[1.0, 8.0, 2.0], [4.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[8.0, 3.0, 6.0], [4.0, 5.0, 8.0], [4.0, 1.0, 3.0]], [[4.0, 7.0, 
6.0], [2.0, 7.0, 5.0], [4.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 3.0], [4.0, 5.0, 6.0], [3.0, 4.0, 5.0]], [[1.0, 8.0, 7.0], [7.0, 4.0, 
8.0], [2.0, 4.0, 2.0]], [[8.0, 7.0, 6.0], [8.0, 1.0, 4.0], [5.0, 3.0, 4.0]]], [[[4.0, 5.0, 2.0], [2.0, 1.0, 1.0], [8.0, 1.0, 
4.0]], [[6.0, 2.0, 8.0], [3.0, 6.0, 3.0], [6.0, 1.0, 1.0]], [[2.0, 8.0, 1.0], [1.0, 6.0, 2.0], [3.0, 4.0, 2.0]]], [[[6.0, 6.0, 
7.0], [7.0, 3.0, 3.0], [8.0, 2.0, 8.0]], [[7.0, 4.0, 1.0], [3.0, 2.0, 8.0], [1.0, 3.0, 3.0]], [[8.0, 2.0, 6.0], [6.0, 7.0, 
5.0], [6.0, 4.0, 1.0]]], [[[8.0, 8.0, 6.0], [5.0, 8.0, 6.0], [5.0, 8.0, 8.0]], [[6.0, 5.0, 2.0], [2.0, 4.0, 5.0], [5.0, 1.0, 
2.0]], [[7.0, 1.0, 8.0], [2.0, 4.0, 6.0], [1.0, 7.0, 5.0]]]]))
      val=numpy.array([[[[7.0, 2.0], [7.0, 5.0], [7.0, 2.0]], [[3.0, 3.0], [5.0, 5.0], [5.0, 4.0]], [[4.0, 3.0], [4.0, 3.0], 
[1.0, 2.0]]], [[[3.0, 2.0], [3.0, 8.0], [1.0, 8.0]], [[1.0, 3.0], [5.0, 1.0], [2.0, 5.0]], [[5.0, 6.0], [8.0, 7.0], [8.0, 
8.0]]], [[[8.0, 1.0], [8.0, 1.0], [8.0, 5.0]], [[2.0, 2.0], [2.0, 3.0], [1.0, 7.0]], [[5.0, 7.0], [5.0, 6.0], [7.0, 1.0]]]])
      arg[:3,:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 5.0], [7.0, 5.0, 4.0], [7.0, 2.0, 1.0]], [[3.0, 3.0, 1.0], [5.0, 5.0, 4.0], [5.0, 
4.0, 1.0]], [[4.0, 3.0, 7.0], [4.0, 3.0, 5.0], [1.0, 2.0, 5.0]]], [[[3.0, 2.0, 6.0], [3.0, 8.0, 5.0], [1.0, 8.0, 7.0]], [[1.0, 
3.0, 1.0], [5.0, 1.0, 7.0], [2.0, 5.0, 6.0]], [[5.0, 6.0, 2.0], [8.0, 7.0, 5.0], [8.0, 8.0, 1.0]]], [[[8.0, 1.0, 6.0], [8.0, 
1.0, 8.0], [8.0, 5.0, 4.0]], [[2.0, 2.0, 4.0], [2.0, 3.0, 1.0], [1.0, 7.0, 5.0]], [[5.0, 7.0, 3.0], [5.0, 6.0, 8.0], [7.0, 1.0, 
6.0]]], [[[1.0, 8.0, 2.0], [4.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[8.0, 3.0, 6.0], [4.0, 5.0, 8.0], [4.0, 1.0, 3.0]], [[4.0, 7.0, 
6.0], [2.0, 7.0, 5.0], [4.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 3.0], [7.0, 5.0, 6.0], [7.0, 2.0, 5.0]], 
[[3.0, 3.0, 7.0], [5.0, 5.0, 8.0], [5.0, 4.0, 2.0]], [[4.0, 3.0, 6.0], [4.0, 3.0, 4.0], [1.0, 2.0, 4.0]]], [[[3.0, 2.0, 2.0], 
[3.0, 8.0, 1.0], [1.0, 8.0, 4.0]], [[1.0, 3.0, 8.0], [5.0, 1.0, 3.0], [2.0, 5.0, 1.0]], [[5.0, 6.0, 1.0], [8.0, 7.0, 2.0], 
[8.0, 8.0, 2.0]]], [[[8.0, 1.0, 7.0], [8.0, 1.0, 3.0], [8.0, 5.0, 8.0]], [[2.0, 2.0, 1.0], [2.0, 3.0, 8.0], [1.0, 7.0, 3.0]], 
[[5.0, 7.0, 6.0], [5.0, 6.0, 5.0], [7.0, 1.0, 1.0]]], [[[8.0, 8.0, 6.0], [5.0, 8.0, 6.0], [5.0, 8.0, 8.0]], [[6.0, 5.0, 2.0], 
[2.0, 4.0, 5.0], [5.0, 1.0, 2.0]], [[7.0, 1.0, 8.0], [2.0, 4.0, 6.0], [1.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UCCU(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 6.0], [8.0, 2.0, 5.0], [2.0, 2.0, 4.0]], [[3.0, 2.0, 5.0], [5.0, 8.0, 3.0], [8.0, 5.0, 
8.0]], [[5.0, 4.0, 6.0], [4.0, 7.0, 8.0], [5.0, 4.0, 8.0]]], [[[3.0, 2.0, 3.0], [5.0, 5.0, 4.0], [5.0, 4.0, 7.0]], [[2.0, 7.0, 
3.0], [8.0, 6.0, 1.0], [8.0, 2.0, 3.0]], [[3.0, 8.0, 6.0], [5.0, 2.0, 6.0], [5.0, 4.0, 7.0]]], [[[2.0, 2.0, 3.0], [3.0, 2.0, 
1.0], [2.0, 7.0, 7.0]], [[8.0, 4.0, 6.0], [5.0, 1.0, 6.0], [4.0, 4.0, 8.0]], [[6.0, 6.0, 2.0], [4.0, 4.0, 2.0], [1.0, 7.0, 
2.0]]], [[[5.0, 1.0, 1.0], [2.0, 4.0, 6.0], [2.0, 3.0, 6.0]], [[7.0, 7.0, 2.0], [4.0, 8.0, 1.0], [4.0, 7.0, 1.0]], [[3.0, 6.0, 
8.0], [1.0, 3.0, 4.0], [1.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 5.0], [4.0, 3.0, 8.0], [7.0, 2.0, 4.0]], [[8.0, 8.0, 6.0], [4.0, 3.0, 
7.0], [7.0, 2.0, 4.0]], [[6.0, 3.0, 2.0], [4.0, 7.0, 2.0], [5.0, 5.0, 5.0]]], [[[7.0, 4.0, 7.0], [6.0, 2.0, 1.0], [2.0, 3.0, 
2.0]], [[2.0, 2.0, 1.0], [8.0, 3.0, 5.0], [6.0, 5.0, 6.0]], [[7.0, 5.0, 2.0], [1.0, 8.0, 1.0], [7.0, 6.0, 7.0]]], [[[1.0, 7.0, 
1.0], [7.0, 7.0, 8.0], [2.0, 8.0, 6.0]], [[7.0, 6.0, 8.0], [6.0, 2.0, 5.0], [5.0, 1.0, 3.0]], [[4.0, 4.0, 5.0], [3.0, 3.0, 
5.0], [4.0, 4.0, 8.0]]], [[[6.0, 7.0, 3.0], [7.0, 7.0, 8.0], [2.0, 2.0, 2.0]], [[3.0, 6.0, 7.0], [3.0, 3.0, 6.0], [6.0, 4.0, 
5.0]], [[7.0, 5.0, 8.0], [5.0, 1.0, 6.0], [3.0, 5.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0, 5.0], [8.0, 6.0], [6.0, 5.0]], [[7.0, 8.0], [1.0, 5.0], [1.0, 7.0]], [[5.0, 2.0], [8.0, 
1.0], [5.0, 7.0]]], [[[2.0, 1.0], [1.0, 2.0], [5.0, 5.0]], [[5.0, 7.0], [3.0, 3.0], [6.0, 2.0]], [[3.0, 7.0], [6.0, 2.0], [4.0, 
1.0]]], [[[4.0, 4.0], [5.0, 6.0], [3.0, 6.0]], [[1.0, 3.0], [8.0, 6.0], [5.0, 4.0]], [[3.0, 3.0], [1.0, 1.0], [8.0, 
6.0]]]]),self.functionspace)
      arg[:3,:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [8.0, 6.0, 5.0], [6.0, 5.0, 4.0]], [[7.0, 8.0, 5.0], [1.0, 5.0, 3.0], [1.0, 
7.0, 8.0]], [[5.0, 2.0, 6.0], [8.0, 1.0, 8.0], [5.0, 7.0, 8.0]]], [[[2.0, 1.0, 3.0], [1.0, 2.0, 4.0], [5.0, 5.0, 7.0]], [[5.0, 
7.0, 3.0], [3.0, 3.0, 1.0], [6.0, 2.0, 3.0]], [[3.0, 7.0, 6.0], [6.0, 2.0, 6.0], [4.0, 1.0, 7.0]]], [[[4.0, 4.0, 3.0], [5.0, 
6.0, 1.0], [3.0, 6.0, 7.0]], [[1.0, 3.0, 6.0], [8.0, 6.0, 6.0], [5.0, 4.0, 8.0]], [[3.0, 3.0, 2.0], [1.0, 1.0, 2.0], [8.0, 6.0, 
2.0]]], [[[5.0, 1.0, 1.0], [2.0, 4.0, 6.0], [2.0, 3.0, 6.0]], [[7.0, 7.0, 2.0], [4.0, 8.0, 1.0], [4.0, 7.0, 1.0]], [[3.0, 6.0, 
8.0], [1.0, 3.0, 4.0], [1.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 5.0], [8.0, 6.0, 8.0], [6.0, 5.0, 4.0]], 
[[7.0, 8.0, 6.0], [1.0, 5.0, 7.0], [1.0, 7.0, 4.0]], [[5.0, 2.0, 2.0], [8.0, 1.0, 2.0], [5.0, 7.0, 5.0]]], [[[2.0, 1.0, 7.0], 
[1.0, 2.0, 1.0], [5.0, 5.0, 2.0]], [[5.0, 7.0, 1.0], [3.0, 3.0, 5.0], [6.0, 2.0, 6.0]], [[3.0, 7.0, 2.0], [6.0, 2.0, 1.0], 
[4.0, 1.0, 7.0]]], [[[4.0, 4.0, 1.0], [5.0, 6.0, 8.0], [3.0, 6.0, 6.0]], [[1.0, 3.0, 8.0], [8.0, 6.0, 5.0], [5.0, 4.0, 3.0]], 
[[3.0, 3.0, 5.0], [1.0, 1.0, 5.0], [8.0, 6.0, 8.0]]], [[[6.0, 7.0, 3.0], [7.0, 7.0, 8.0], [2.0, 2.0, 2.0]], [[3.0, 6.0, 7.0], 
[3.0, 3.0, 6.0], [6.0, 4.0, 5.0]], [[7.0, 5.0, 8.0], [5.0, 1.0, 6.0], [3.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UCCU(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 5.0], [6.0, 6.0, 6.0], [2.0, 4.0, 5.0]], [[3.0, 1.0, 6.0], [7.0, 8.0, 7.0], [6.0, 7.0, 
1.0]], [[1.0, 3.0, 6.0], [6.0, 1.0, 1.0], [8.0, 7.0, 4.0]]], [[[8.0, 4.0, 2.0], [3.0, 7.0, 2.0], [7.0, 1.0, 4.0]], [[7.0, 5.0, 
4.0], [2.0, 4.0, 1.0], [7.0, 6.0, 8.0]], [[1.0, 1.0, 5.0], [5.0, 4.0, 6.0], [7.0, 2.0, 3.0]]], [[[5.0, 7.0, 7.0], [8.0, 7.0, 
8.0], [8.0, 7.0, 5.0]], [[3.0, 3.0, 2.0], [1.0, 4.0, 2.0], [7.0, 2.0, 6.0]], [[8.0, 8.0, 7.0], [2.0, 3.0, 5.0], [2.0, 2.0, 
1.0]]], [[[1.0, 4.0, 7.0], [1.0, 6.0, 4.0], [1.0, 1.0, 7.0]], [[4.0, 4.0, 8.0], [4.0, 4.0, 5.0], [5.0, 7.0, 1.0]], [[8.0, 5.0, 
5.0], [8.0, 8.0, 6.0], [1.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 7.0], [3.0, 3.0, 4.0], [7.0, 1.0, 2.0]], [[7.0, 7.0, 3.0], [7.0, 8.0, 
1.0], [5.0, 4.0, 4.0]], [[7.0, 1.0, 5.0], [2.0, 5.0, 8.0], [5.0, 8.0, 8.0]]], [[[3.0, 8.0, 8.0], [8.0, 2.0, 8.0], [4.0, 1.0, 
4.0]], [[3.0, 1.0, 6.0], [7.0, 5.0, 5.0], [3.0, 1.0, 5.0]], [[3.0, 7.0, 1.0], [5.0, 2.0, 1.0], [2.0, 7.0, 2.0]]], [[[8.0, 8.0, 
5.0], [8.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[2.0, 3.0, 3.0], [4.0, 3.0, 1.0], [3.0, 3.0, 4.0]], [[8.0, 5.0, 2.0], [6.0, 2.0, 
8.0], [2.0, 2.0, 4.0]]], [[[5.0, 7.0, 3.0], [4.0, 7.0, 2.0], [6.0, 7.0, 2.0]], [[7.0, 6.0, 2.0], [7.0, 8.0, 7.0], [1.0, 8.0, 
4.0]], [[1.0, 3.0, 7.0], [7.0, 2.0, 7.0], [4.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[[[6.0, 6.0], [4.0, 1.0], [6.0, 1.0]], [[8.0, 4.0], [6.0, 5.0], [2.0, 6.0]], [[8.0, 6.0], [7.0, 
5.0], [4.0, 5.0]]], [[[7.0, 7.0], [3.0, 3.0], [4.0, 4.0]], [[4.0, 8.0], [2.0, 4.0], [1.0, 5.0]], [[2.0, 2.0], [5.0, 4.0], [4.0, 
8.0]]], [[[2.0, 1.0], [8.0, 8.0], [4.0, 4.0]], [[5.0, 3.0], [5.0, 6.0], [7.0, 8.0]], [[5.0, 2.0], [5.0, 6.0], [6.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 6.0], [5.0, 2.0], [1.0, 6.0]], [[2.0, 6.0], [8.0, 8.0], [7.0, 6.0]], [[7.0, 
7.0], [5.0, 1.0], [6.0, 3.0]]], [[[2.0, 3.0], [8.0, 8.0], [1.0, 8.0]], [[7.0, 5.0], [1.0, 2.0], [8.0, 2.0]], [[5.0, 4.0], [8.0, 
2.0], [7.0, 1.0]]], [[[1.0, 1.0], [3.0, 6.0], [3.0, 2.0]], [[1.0, 5.0], [6.0, 1.0], [1.0, 4.0]], [[4.0, 6.0], [3.0, 7.0], [7.0, 
3.0]]]]))
      arg[:3,:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 5.0], [4.0, 1.0, 6.0], [6.0, 1.0, 5.0]], [[8.0, 4.0, 6.0], [6.0, 5.0, 7.0], [2.0, 
6.0, 1.0]], [[8.0, 6.0, 6.0], [7.0, 5.0, 1.0], [4.0, 5.0, 4.0]]], [[[7.0, 7.0, 2.0], [3.0, 3.0, 2.0], [4.0, 4.0, 4.0]], [[4.0, 
8.0, 4.0], [2.0, 4.0, 1.0], [1.0, 5.0, 8.0]], [[2.0, 2.0, 5.0], [5.0, 4.0, 6.0], [4.0, 8.0, 3.0]]], [[[2.0, 1.0, 7.0], [8.0, 
8.0, 8.0], [4.0, 4.0, 5.0]], [[5.0, 3.0, 2.0], [5.0, 6.0, 2.0], [7.0, 8.0, 6.0]], [[5.0, 2.0, 7.0], [5.0, 6.0, 5.0], [6.0, 1.0, 
1.0]]], [[[1.0, 4.0, 7.0], [1.0, 6.0, 4.0], [1.0, 1.0, 7.0]], [[4.0, 4.0, 8.0], [4.0, 4.0, 5.0], [5.0, 7.0, 1.0]], [[8.0, 5.0, 
5.0], [8.0, 8.0, 6.0], [1.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 7.0], [5.0, 2.0, 4.0], [1.0, 6.0, 2.0]], 
[[2.0, 6.0, 3.0], [8.0, 8.0, 1.0], [7.0, 6.0, 4.0]], [[7.0, 7.0, 5.0], [5.0, 1.0, 8.0], [6.0, 3.0, 8.0]]], [[[2.0, 3.0, 8.0], 
[8.0, 8.0, 8.0], [1.0, 8.0, 4.0]], [[7.0, 5.0, 6.0], [1.0, 2.0, 5.0], [8.0, 2.0, 5.0]], [[5.0, 4.0, 1.0], [8.0, 2.0, 1.0], 
[7.0, 1.0, 2.0]]], [[[1.0, 1.0, 5.0], [3.0, 6.0, 1.0], [3.0, 2.0, 3.0]], [[1.0, 5.0, 3.0], [6.0, 1.0, 1.0], [1.0, 4.0, 4.0]], 
[[4.0, 6.0, 2.0], [3.0, 7.0, 8.0], [7.0, 3.0, 4.0]]], [[[5.0, 7.0, 3.0], [4.0, 7.0, 2.0], [6.0, 7.0, 2.0]], [[7.0, 6.0, 2.0], 
[7.0, 8.0, 7.0], [1.0, 8.0, 4.0]], [[1.0, 3.0, 7.0], [7.0, 2.0, 7.0], [4.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UCCU(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 8.0], [3.0, 1.0, 2.0], [5.0, 8.0, 1.0]], [[1.0, 5.0, 3.0], [6.0, 4.0, 4.0], [7.0, 8.0, 
2.0]], [[8.0, 2.0, 4.0], [3.0, 2.0, 8.0], [7.0, 7.0, 1.0]]], [[[7.0, 3.0, 4.0], [5.0, 4.0, 4.0], [3.0, 5.0, 5.0]], [[7.0, 1.0, 
7.0], [4.0, 3.0, 3.0], [3.0, 2.0, 5.0]], [[7.0, 5.0, 5.0], [7.0, 8.0, 3.0], [7.0, 4.0, 6.0]]], [[[3.0, 7.0, 1.0], [4.0, 3.0, 
2.0], [3.0, 6.0, 5.0]], [[3.0, 5.0, 2.0], [6.0, 4.0, 8.0], [2.0, 2.0, 8.0]], [[4.0, 7.0, 4.0], [5.0, 6.0, 5.0], [5.0, 5.0, 
6.0]]], [[[5.0, 8.0, 5.0], [3.0, 1.0, 3.0], [6.0, 5.0, 7.0]], [[3.0, 1.0, 2.0], [7.0, 2.0, 8.0], [8.0, 7.0, 4.0]], [[8.0, 6.0, 
2.0], [2.0, 6.0, 3.0], [2.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 6.0], [4.0, 8.0, 5.0], [2.0, 4.0, 3.0]], [[5.0, 4.0, 5.0], [4.0, 3.0, 
5.0], [3.0, 4.0, 4.0]], [[7.0, 6.0, 2.0], [1.0, 6.0, 1.0], [7.0, 7.0, 1.0]]], [[[5.0, 5.0, 3.0], [6.0, 7.0, 5.0], [8.0, 2.0, 
7.0]], [[7.0, 1.0, 8.0], [2.0, 2.0, 5.0], [7.0, 3.0, 7.0]], [[4.0, 8.0, 1.0], [5.0, 3.0, 3.0], [2.0, 2.0, 4.0]]], [[[8.0, 7.0, 
8.0], [4.0, 3.0, 5.0], [6.0, 3.0, 8.0]], [[5.0, 5.0, 1.0], [2.0, 3.0, 1.0], [2.0, 1.0, 8.0]], [[8.0, 3.0, 5.0], [5.0, 1.0, 
5.0], [8.0, 6.0, 8.0]]], [[[5.0, 3.0, 4.0], [1.0, 2.0, 7.0], [6.0, 2.0, 7.0]], [[6.0, 8.0, 1.0], [3.0, 5.0, 4.0], [2.0, 7.0, 
5.0]], [[8.0, 3.0, 2.0], [3.0, 7.0, 5.0], [2.0, 8.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0, 6.0], [8.0, 7.0], [7.0, 6.0]], [[3.0, 6.0], [4.0, 1.0], [5.0, 6.0]], [[6.0, 7.0], [6.0, 
3.0], [8.0, 8.0]]], [[[1.0, 3.0], [5.0, 4.0], [4.0, 3.0]], [[7.0, 7.0], [7.0, 2.0], [3.0, 2.0]], [[2.0, 7.0], [6.0, 7.0], [3.0, 
6.0]]], [[[5.0, 4.0], [7.0, 2.0], [6.0, 3.0]], [[4.0, 3.0], [7.0, 7.0], [5.0, 2.0]], [[1.0, 8.0], [6.0, 2.0], [1.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 2.0], [5.0, 4.0], [1.0, 3.0]], [[2.0, 4.0], [7.0, 5.0], [5.0, 5.0]], [[5.0, 8.0], 
[4.0, 2.0], [8.0, 6.0]]], [[[8.0, 6.0], [5.0, 5.0], [2.0, 4.0]], [[2.0, 7.0], [7.0, 8.0], [2.0, 7.0]], [[2.0, 5.0], [4.0, 4.0], 
[4.0, 8.0]]], [[[3.0, 7.0], [7.0, 5.0], [5.0, 4.0]], [[4.0, 8.0], [1.0, 2.0], [8.0, 5.0]], [[8.0, 1.0], [2.0, 1.0], [7.0, 
4.0]]]])
      arg[:3,:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 8.0], [8.0, 7.0, 2.0], [7.0, 6.0, 1.0]], [[3.0, 6.0, 3.0], [4.0, 1.0, 4.0], [5.0, 
6.0, 2.0]], [[6.0, 7.0, 4.0], [6.0, 3.0, 8.0], [8.0, 8.0, 1.0]]], [[[1.0, 3.0, 4.0], [5.0, 4.0, 4.0], [4.0, 3.0, 5.0]], [[7.0, 
7.0, 7.0], [7.0, 2.0, 3.0], [3.0, 2.0, 5.0]], [[2.0, 7.0, 5.0], [6.0, 7.0, 3.0], [3.0, 6.0, 6.0]]], [[[5.0, 4.0, 1.0], [7.0, 
2.0, 2.0], [6.0, 3.0, 5.0]], [[4.0, 3.0, 2.0], [7.0, 7.0, 8.0], [5.0, 2.0, 8.0]], [[1.0, 8.0, 4.0], [6.0, 2.0, 5.0], [1.0, 8.0, 
6.0]]], [[[5.0, 8.0, 5.0], [3.0, 1.0, 3.0], [6.0, 5.0, 7.0]], [[3.0, 1.0, 2.0], [7.0, 2.0, 8.0], [8.0, 7.0, 4.0]], [[8.0, 6.0, 
2.0], [2.0, 6.0, 3.0], [2.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 6.0], [5.0, 4.0, 5.0], [1.0, 3.0, 3.0]], 
[[2.0, 4.0, 5.0], [7.0, 5.0, 5.0], [5.0, 5.0, 4.0]], [[5.0, 8.0, 2.0], [4.0, 2.0, 1.0], [8.0, 6.0, 1.0]]], [[[8.0, 6.0, 3.0], 
[5.0, 5.0, 5.0], [2.0, 4.0, 7.0]], [[2.0, 7.0, 8.0], [7.0, 8.0, 5.0], [2.0, 7.0, 7.0]], [[2.0, 5.0, 1.0], [4.0, 4.0, 3.0], 
[4.0, 8.0, 4.0]]], [[[3.0, 7.0, 8.0], [7.0, 5.0, 5.0], [5.0, 4.0, 8.0]], [[4.0, 8.0, 1.0], [1.0, 2.0, 1.0], [8.0, 5.0, 8.0]], 
[[8.0, 1.0, 5.0], [2.0, 1.0, 5.0], [7.0, 4.0, 8.0]]], [[[5.0, 3.0, 4.0], [1.0, 2.0, 7.0], [6.0, 2.0, 7.0]], [[6.0, 8.0, 1.0], 
[3.0, 5.0, 4.0], [2.0, 7.0, 5.0]], [[8.0, 3.0, 2.0], [3.0, 7.0, 5.0], [2.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_URCN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 6.0], [4.0, 6.0, 1.0], [6.0, 6.0, 4.0]], [[6.0, 8.0, 4.0], [3.0, 3.0, 4.0], [4.0, 3.0, 
2.0]], [[1.0, 7.0, 7.0], [5.0, 3.0, 4.0], [1.0, 4.0, 2.0]]], [[[1.0, 7.0, 1.0], [5.0, 3.0, 4.0], [2.0, 6.0, 4.0]], [[5.0, 3.0, 
5.0], [2.0, 5.0, 8.0], [8.0, 2.0, 7.0]], [[1.0, 4.0, 7.0], [8.0, 3.0, 3.0], [4.0, 8.0, 3.0]]], [[[4.0, 8.0, 3.0], [8.0, 6.0, 
2.0], [8.0, 1.0, 8.0]], [[8.0, 7.0, 7.0], [5.0, 5.0, 8.0], [3.0, 6.0, 8.0]], [[5.0, 8.0, 2.0], [5.0, 1.0, 1.0], [8.0, 4.0, 
7.0]]], [[[7.0, 6.0, 2.0], [6.0, 5.0, 2.0], [5.0, 8.0, 6.0]], [[2.0, 1.0, 7.0], [7.0, 2.0, 5.0], [6.0, 5.0, 6.0]], [[8.0, 3.0, 
2.0], [8.0, 1.0, 8.0], [8.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 2.0], [8.0, 4.0, 2.0], [5.0, 3.0, 5.0]], [[4.0, 7.0, 5.0], [8.0, 4.0, 
1.0], [1.0, 5.0, 4.0]], [[2.0, 5.0, 1.0], [7.0, 2.0, 4.0], [4.0, 5.0, 4.0]]], [[[7.0, 2.0, 7.0], [4.0, 6.0, 5.0], [8.0, 7.0, 
5.0]], [[4.0, 4.0, 3.0], [7.0, 4.0, 5.0], [7.0, 1.0, 3.0]], [[5.0, 1.0, 1.0], [1.0, 7.0, 4.0], [5.0, 4.0, 4.0]]], [[[6.0, 3.0, 
8.0], [4.0, 2.0, 1.0], [1.0, 2.0, 3.0]], [[2.0, 4.0, 2.0], [4.0, 2.0, 5.0], [6.0, 1.0, 4.0]], [[2.0, 7.0, 2.0], [7.0, 2.0, 
2.0], [4.0, 5.0, 4.0]]], [[[3.0, 7.0, 8.0], [7.0, 7.0, 4.0], [6.0, 3.0, 3.0]], [[2.0, 5.0, 2.0], [5.0, 3.0, 1.0], [5.0, 7.0, 
6.0]], [[6.0, 6.0, 2.0], [1.0, 3.0, 7.0], [5.0, 7.0, 2.0]]]]))
      val=numpy.array([[[[6.0, 2.0, 8.0], [3.0, 6.0, 1.0], [2.0, 7.0, 5.0]]], [[[2.0, 8.0, 3.0], [6.0, 2.0, 5.0], [3.0, 2.0, 
2.0]]], [[[2.0, 5.0, 8.0], [2.0, 5.0, 8.0], [1.0, 7.0, 6.0]]]])
      arg[:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 6.0], [4.0, 6.0, 1.0], [6.0, 6.0, 4.0]], [[6.0, 2.0, 8.0], [3.0, 6.0, 1.0], [2.0, 
7.0, 5.0]], [[1.0, 7.0, 7.0], [5.0, 3.0, 4.0], [1.0, 4.0, 2.0]]], [[[1.0, 7.0, 1.0], [5.0, 3.0, 4.0], [2.0, 6.0, 4.0]], [[2.0, 
8.0, 3.0], [6.0, 2.0, 5.0], [3.0, 2.0, 2.0]], [[1.0, 4.0, 7.0], [8.0, 3.0, 3.0], [4.0, 8.0, 3.0]]], [[[4.0, 8.0, 3.0], [8.0, 
6.0, 2.0], [8.0, 1.0, 8.0]], [[2.0, 5.0, 8.0], [2.0, 5.0, 8.0], [1.0, 7.0, 6.0]], [[5.0, 8.0, 2.0], [5.0, 1.0, 1.0], [8.0, 4.0, 
7.0]]], [[[7.0, 6.0, 2.0], [6.0, 5.0, 2.0], [5.0, 8.0, 6.0]], [[2.0, 1.0, 7.0], [7.0, 2.0, 5.0], [6.0, 5.0, 6.0]], [[8.0, 3.0, 
2.0], [8.0, 1.0, 8.0], [8.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 2.0], [8.0, 4.0, 2.0], [5.0, 3.0, 5.0]], 
[[6.0, 2.0, 8.0], [3.0, 6.0, 1.0], [2.0, 7.0, 5.0]], [[2.0, 5.0, 1.0], [7.0, 2.0, 4.0], [4.0, 5.0, 4.0]]], [[[7.0, 2.0, 7.0], 
[4.0, 6.0, 5.0], [8.0, 7.0, 5.0]], [[2.0, 8.0, 3.0], [6.0, 2.0, 5.0], [3.0, 2.0, 2.0]], [[5.0, 1.0, 1.0], [1.0, 7.0, 4.0], 
[5.0, 4.0, 4.0]]], [[[6.0, 3.0, 8.0], [4.0, 2.0, 1.0], [1.0, 2.0, 3.0]], [[2.0, 5.0, 8.0], [2.0, 5.0, 8.0], [1.0, 7.0, 6.0]], 
[[2.0, 7.0, 2.0], [7.0, 2.0, 2.0], [4.0, 5.0, 4.0]]], [[[3.0, 7.0, 8.0], [7.0, 7.0, 4.0], [6.0, 3.0, 3.0]], [[2.0, 5.0, 2.0], 
[5.0, 3.0, 1.0], [5.0, 7.0, 6.0]], [[6.0, 6.0, 2.0], [1.0, 3.0, 7.0], [5.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_URCN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 3.0], [2.0, 2.0, 4.0], [1.0, 7.0, 3.0]], [[6.0, 2.0, 5.0], [3.0, 8.0, 5.0], [3.0, 6.0, 
7.0]], [[1.0, 3.0, 4.0], [8.0, 2.0, 2.0], [5.0, 3.0, 5.0]]], [[[1.0, 7.0, 8.0], [7.0, 6.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 7.0, 
5.0], [6.0, 1.0, 8.0], [7.0, 8.0, 2.0]], [[7.0, 4.0, 6.0], [6.0, 8.0, 6.0], [7.0, 4.0, 4.0]]], [[[1.0, 7.0, 1.0], [8.0, 7.0, 
2.0], [5.0, 8.0, 8.0]], [[1.0, 6.0, 1.0], [8.0, 6.0, 4.0], [4.0, 4.0, 7.0]], [[4.0, 3.0, 8.0], [3.0, 8.0, 7.0], [7.0, 7.0, 
6.0]]], [[[3.0, 8.0, 8.0], [5.0, 3.0, 4.0], [7.0, 4.0, 3.0]], [[7.0, 8.0, 6.0], [2.0, 8.0, 5.0], [6.0, 4.0, 8.0]], [[4.0, 2.0, 
5.0], [8.0, 8.0, 1.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 1.0], [1.0, 2.0, 6.0], [6.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], [3.0, 4.0, 
5.0], [8.0, 4.0, 5.0]], [[4.0, 1.0, 6.0], [7.0, 6.0, 8.0], [7.0, 8.0, 4.0]]], [[[7.0, 7.0, 7.0], [3.0, 5.0, 8.0], [6.0, 4.0, 
6.0]], [[6.0, 3.0, 6.0], [6.0, 8.0, 6.0], [2.0, 3.0, 5.0]], [[8.0, 4.0, 4.0], [8.0, 3.0, 8.0], [5.0, 6.0, 8.0]]], [[[4.0, 8.0, 
6.0], [1.0, 4.0, 8.0], [3.0, 2.0, 8.0]], [[3.0, 8.0, 8.0], [5.0, 7.0, 3.0], [8.0, 1.0, 2.0]], [[4.0, 7.0, 4.0], [7.0, 1.0, 
6.0], [2.0, 1.0, 1.0]]], [[[6.0, 4.0, 1.0], [6.0, 5.0, 7.0], [8.0, 6.0, 6.0]], [[1.0, 4.0, 8.0], [8.0, 5.0, 7.0], [2.0, 5.0, 
5.0]], [[1.0, 3.0, 3.0], [5.0, 7.0, 4.0], [4.0, 6.0, 1.0]]]]))
      val=Data(numpy.array([[[[1.0, 2.0, 2.0], [6.0, 3.0, 5.0], [5.0, 7.0, 1.0]]], [[[2.0, 8.0, 1.0], [4.0, 1.0, 7.0], [4.0, 
7.0, 5.0]]], [[[4.0, 1.0, 8.0], [1.0, 2.0, 7.0], [8.0, 2.0, 5.0]]]]),self.functionspace)
      arg[:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 3.0], [2.0, 2.0, 4.0], [1.0, 7.0, 3.0]], [[1.0, 2.0, 2.0], [6.0, 3.0, 5.0], [5.0, 
7.0, 1.0]], [[1.0, 3.0, 4.0], [8.0, 2.0, 2.0], [5.0, 3.0, 5.0]]], [[[1.0, 7.0, 8.0], [7.0, 6.0, 7.0], [5.0, 2.0, 7.0]], [[2.0, 
8.0, 1.0], [4.0, 1.0, 7.0], [4.0, 7.0, 5.0]], [[7.0, 4.0, 6.0], [6.0, 8.0, 6.0], [7.0, 4.0, 4.0]]], [[[1.0, 7.0, 1.0], [8.0, 
7.0, 2.0], [5.0, 8.0, 8.0]], [[4.0, 1.0, 8.0], [1.0, 2.0, 7.0], [8.0, 2.0, 5.0]], [[4.0, 3.0, 8.0], [3.0, 8.0, 7.0], [7.0, 7.0, 
6.0]]], [[[3.0, 8.0, 8.0], [5.0, 3.0, 4.0], [7.0, 4.0, 3.0]], [[7.0, 8.0, 6.0], [2.0, 8.0, 5.0], [6.0, 4.0, 8.0]], [[4.0, 2.0, 
5.0], [8.0, 8.0, 1.0], [8.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 1.0], [1.0, 2.0, 6.0], [6.0, 5.0, 5.0]], 
[[1.0, 2.0, 2.0], [6.0, 3.0, 5.0], [5.0, 7.0, 1.0]], [[4.0, 1.0, 6.0], [7.0, 6.0, 8.0], [7.0, 8.0, 4.0]]], [[[7.0, 7.0, 7.0], 
[3.0, 5.0, 8.0], [6.0, 4.0, 6.0]], [[2.0, 8.0, 1.0], [4.0, 1.0, 7.0], [4.0, 7.0, 5.0]], [[8.0, 4.0, 4.0], [8.0, 3.0, 8.0], 
[5.0, 6.0, 8.0]]], [[[4.0, 8.0, 6.0], [1.0, 4.0, 8.0], [3.0, 2.0, 8.0]], [[4.0, 1.0, 8.0], [1.0, 2.0, 7.0], [8.0, 2.0, 5.0]], 
[[4.0, 7.0, 4.0], [7.0, 1.0, 6.0], [2.0, 1.0, 1.0]]], [[[6.0, 4.0, 1.0], [6.0, 5.0, 7.0], [8.0, 6.0, 6.0]], [[1.0, 4.0, 8.0], 
[8.0, 5.0, 7.0], [2.0, 5.0, 5.0]], [[1.0, 3.0, 3.0], [5.0, 7.0, 4.0], [4.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_URCN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 5.0], [7.0, 3.0, 3.0], [5.0, 8.0, 2.0]], [[8.0, 8.0, 1.0], [6.0, 1.0, 6.0], [2.0, 8.0, 
1.0]], [[6.0, 5.0, 1.0], [7.0, 4.0, 6.0], [8.0, 8.0, 3.0]]], [[[3.0, 8.0, 6.0], [7.0, 5.0, 1.0], [3.0, 2.0, 5.0]], [[3.0, 4.0, 
8.0], [5.0, 8.0, 4.0], [4.0, 8.0, 6.0]], [[8.0, 7.0, 8.0], [5.0, 3.0, 3.0], [5.0, 3.0, 5.0]]], [[[3.0, 3.0, 8.0], [6.0, 1.0, 
3.0], [1.0, 3.0, 5.0]], [[1.0, 5.0, 5.0], [6.0, 8.0, 2.0], [2.0, 2.0, 8.0]], [[5.0, 2.0, 3.0], [1.0, 3.0, 7.0], [1.0, 8.0, 
7.0]]], [[[3.0, 3.0, 4.0], [4.0, 2.0, 3.0], [6.0, 6.0, 6.0]], [[5.0, 2.0, 7.0], [4.0, 2.0, 7.0], [8.0, 7.0, 3.0]], [[4.0, 5.0, 
5.0], [1.0, 2.0, 2.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 2.0], [8.0, 2.0, 4.0], [3.0, 8.0, 8.0]], [[2.0, 8.0, 6.0], [4.0, 2.0, 
2.0], [3.0, 4.0, 5.0]], [[8.0, 7.0, 2.0], [4.0, 4.0, 2.0], [5.0, 6.0, 7.0]]], [[[6.0, 1.0, 5.0], [6.0, 5.0, 7.0], [1.0, 6.0, 
2.0]], [[2.0, 6.0, 1.0], [1.0, 8.0, 4.0], [2.0, 3.0, 8.0]], [[4.0, 3.0, 8.0], [5.0, 1.0, 7.0], [3.0, 1.0, 3.0]]], [[[4.0, 7.0, 
5.0], [7.0, 4.0, 5.0], [4.0, 7.0, 7.0]], [[3.0, 4.0, 7.0], [6.0, 8.0, 8.0], [4.0, 1.0, 8.0]], [[7.0, 8.0, 7.0], [3.0, 4.0, 
4.0], [1.0, 1.0, 5.0]]], [[[8.0, 5.0, 2.0], [3.0, 4.0, 5.0], [4.0, 5.0, 5.0]], [[7.0, 4.0, 6.0], [8.0, 8.0, 5.0], [5.0, 5.0, 
1.0]], [[8.0, 6.0, 6.0], [5.0, 4.0, 8.0], [3.0, 2.0, 6.0]]]]))
      val=Data(numpy.array([[[[7.0, 7.0, 1.0], [8.0, 7.0, 3.0], [8.0, 7.0, 8.0]]], [[[2.0, 3.0, 5.0], [3.0, 7.0, 2.0], [6.0, 
8.0, 6.0]]], [[[8.0, 7.0, 4.0], [8.0, 8.0, 7.0], [7.0, 1.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 5.0], [3.0, 3.0, 3.0], [3.0, 5.0, 6.0]]], [[[5.0, 1.0, 6.0], [6.0, 7.0, 
7.0], [5.0, 4.0, 7.0]]], [[[1.0, 4.0, 6.0], [2.0, 4.0, 7.0], [2.0, 8.0, 8.0]]]]))
      arg[:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 5.0], [7.0, 3.0, 3.0], [5.0, 8.0, 2.0]], [[7.0, 7.0, 1.0], [8.0, 7.0, 3.0], [8.0, 
7.0, 8.0]], [[6.0, 5.0, 1.0], [7.0, 4.0, 6.0], [8.0, 8.0, 3.0]]], [[[3.0, 8.0, 6.0], [7.0, 5.0, 1.0], [3.0, 2.0, 5.0]], [[2.0, 
3.0, 5.0], [3.0, 7.0, 2.0], [6.0, 8.0, 6.0]], [[8.0, 7.0, 8.0], [5.0, 3.0, 3.0], [5.0, 3.0, 5.0]]], [[[3.0, 3.0, 8.0], [6.0, 
1.0, 3.0], [1.0, 3.0, 5.0]], [[8.0, 7.0, 4.0], [8.0, 8.0, 7.0], [7.0, 1.0, 6.0]], [[5.0, 2.0, 3.0], [1.0, 3.0, 7.0], [1.0, 8.0, 
7.0]]], [[[3.0, 3.0, 4.0], [4.0, 2.0, 3.0], [6.0, 6.0, 6.0]], [[5.0, 2.0, 7.0], [4.0, 2.0, 7.0], [8.0, 7.0, 3.0]], [[4.0, 5.0, 
5.0], [1.0, 2.0, 2.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 2.0], [8.0, 2.0, 4.0], [3.0, 8.0, 8.0]], 
[[7.0, 7.0, 5.0], [3.0, 3.0, 3.0], [3.0, 5.0, 6.0]], [[8.0, 7.0, 2.0], [4.0, 4.0, 2.0], [5.0, 6.0, 7.0]]], [[[6.0, 1.0, 5.0], 
[6.0, 5.0, 7.0], [1.0, 6.0, 2.0]], [[5.0, 1.0, 6.0], [6.0, 7.0, 7.0], [5.0, 4.0, 7.0]], [[4.0, 3.0, 8.0], [5.0, 1.0, 7.0], 
[3.0, 1.0, 3.0]]], [[[4.0, 7.0, 5.0], [7.0, 4.0, 5.0], [4.0, 7.0, 7.0]], [[1.0, 4.0, 6.0], [2.0, 4.0, 7.0], [2.0, 8.0, 8.0]], 
[[7.0, 8.0, 7.0], [3.0, 4.0, 4.0], [1.0, 1.0, 5.0]]], [[[8.0, 5.0, 2.0], [3.0, 4.0, 5.0], [4.0, 5.0, 5.0]], [[7.0, 4.0, 6.0], 
[8.0, 8.0, 5.0], [5.0, 5.0, 1.0]], [[8.0, 6.0, 6.0], [5.0, 4.0, 8.0], [3.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_URCN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [6.0, 2.0, 7.0], [5.0, 8.0, 1.0]], [[3.0, 4.0, 5.0], [6.0, 4.0, 4.0], [1.0, 3.0, 
1.0]], [[3.0, 8.0, 1.0], [8.0, 8.0, 2.0], [5.0, 6.0, 5.0]]], [[[5.0, 5.0, 4.0], [5.0, 7.0, 7.0], [7.0, 4.0, 4.0]], [[4.0, 3.0, 
5.0], [7.0, 1.0, 4.0], [1.0, 5.0, 5.0]], [[8.0, 5.0, 8.0], [5.0, 4.0, 6.0], [1.0, 8.0, 4.0]]], [[[3.0, 8.0, 5.0], [3.0, 5.0, 
3.0], [5.0, 7.0, 5.0]], [[4.0, 6.0, 5.0], [8.0, 6.0, 8.0], [1.0, 4.0, 4.0]], [[8.0, 2.0, 5.0], [3.0, 1.0, 6.0], [3.0, 2.0, 
7.0]]], [[[1.0, 2.0, 5.0], [4.0, 1.0, 5.0], [5.0, 8.0, 2.0]], [[1.0, 3.0, 6.0], [4.0, 3.0, 2.0], [1.0, 7.0, 5.0]], [[7.0, 2.0, 
6.0], [2.0, 7.0, 5.0], [1.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 7.0], [1.0, 6.0, 4.0], [6.0, 5.0, 7.0]], [[6.0, 4.0, 8.0], [1.0, 6.0, 
7.0], [8.0, 4.0, 7.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 7.0], [1.0, 5.0, 2.0]]], [[[8.0, 7.0, 7.0], [6.0, 7.0, 6.0], [4.0, 1.0, 
2.0]], [[2.0, 1.0, 8.0], [7.0, 7.0, 1.0], [6.0, 7.0, 3.0]], [[2.0, 8.0, 3.0], [5.0, 5.0, 7.0], [3.0, 2.0, 5.0]]], [[[5.0, 6.0, 
1.0], [1.0, 1.0, 3.0], [5.0, 1.0, 3.0]], [[8.0, 5.0, 3.0], [7.0, 3.0, 2.0], [2.0, 8.0, 5.0]], [[6.0, 1.0, 1.0], [3.0, 4.0, 
8.0], [5.0, 8.0, 3.0]]], [[[6.0, 1.0, 6.0], [6.0, 6.0, 1.0], [4.0, 4.0, 5.0]], [[7.0, 1.0, 8.0], [7.0, 6.0, 7.0], [7.0, 4.0, 
5.0]], [[7.0, 3.0, 6.0], [4.0, 1.0, 6.0], [6.0, 7.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 8.0, 5.0], [1.0, 5.0, 5.0], [8.0, 8.0, 1.0]]], [[[7.0, 5.0, 2.0], [5.0, 8.0, 7.0], [5.0, 
2.0, 2.0]]], [[[8.0, 6.0, 2.0], [6.0, 4.0, 5.0], [2.0, 8.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 5.0, 6.0], [4.0, 2.0, 
4.0], [8.0, 3.0, 1.0]]], [[[4.0, 1.0, 4.0], [5.0, 2.0, 3.0], [6.0, 3.0, 6.0]]], [[[7.0, 4.0, 6.0], [4.0, 3.0, 5.0], [8.0, 7.0, 
6.0]]]])
      arg[:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 6.0], [6.0, 2.0, 7.0], [5.0, 8.0, 1.0]], [[3.0, 8.0, 5.0], [1.0, 5.0, 5.0], [8.0, 
8.0, 1.0]], [[3.0, 8.0, 1.0], [8.0, 8.0, 2.0], [5.0, 6.0, 5.0]]], [[[5.0, 5.0, 4.0], [5.0, 7.0, 7.0], [7.0, 4.0, 4.0]], [[7.0, 
5.0, 2.0], [5.0, 8.0, 7.0], [5.0, 2.0, 2.0]], [[8.0, 5.0, 8.0], [5.0, 4.0, 6.0], [1.0, 8.0, 4.0]]], [[[3.0, 8.0, 5.0], [3.0, 
5.0, 3.0], [5.0, 7.0, 5.0]], [[8.0, 6.0, 2.0], [6.0, 4.0, 5.0], [2.0, 8.0, 8.0]], [[8.0, 2.0, 5.0], [3.0, 1.0, 6.0], [3.0, 2.0, 
7.0]]], [[[1.0, 2.0, 5.0], [4.0, 1.0, 5.0], [5.0, 8.0, 2.0]], [[1.0, 3.0, 6.0], [4.0, 3.0, 2.0], [1.0, 7.0, 5.0]], [[7.0, 2.0, 
6.0], [2.0, 7.0, 5.0], [1.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 7.0], [1.0, 6.0, 4.0], [6.0, 5.0, 7.0]], 
[[4.0, 5.0, 6.0], [4.0, 2.0, 4.0], [8.0, 3.0, 1.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 7.0], [1.0, 5.0, 2.0]]], [[[8.0, 7.0, 7.0], 
[6.0, 7.0, 6.0], [4.0, 1.0, 2.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 3.0], [6.0, 3.0, 6.0]], [[2.0, 8.0, 3.0], [5.0, 5.0, 7.0], 
[3.0, 2.0, 5.0]]], [[[5.0, 6.0, 1.0], [1.0, 1.0, 3.0], [5.0, 1.0, 3.0]], [[7.0, 4.0, 6.0], [4.0, 3.0, 5.0], [8.0, 7.0, 6.0]], 
[[6.0, 1.0, 1.0], [3.0, 4.0, 8.0], [5.0, 8.0, 3.0]]], [[[6.0, 1.0, 6.0], [6.0, 6.0, 1.0], [4.0, 4.0, 5.0]], [[7.0, 1.0, 8.0], 
[7.0, 6.0, 7.0], [7.0, 4.0, 5.0]], [[7.0, 3.0, 6.0], [4.0, 1.0, 6.0], [6.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RUPN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 4.0], [5.0, 2.0, 5.0], [7.0, 2.0, 6.0]], [[1.0, 8.0, 5.0], [8.0, 1.0, 6.0], [2.0, 4.0, 
8.0]], [[6.0, 1.0, 3.0], [8.0, 3.0, 4.0], [5.0, 8.0, 8.0]]], [[[4.0, 5.0, 3.0], [7.0, 5.0, 2.0], [3.0, 8.0, 8.0]], [[3.0, 4.0, 
3.0], [6.0, 3.0, 4.0], [1.0, 6.0, 2.0]], [[6.0, 3.0, 3.0], [2.0, 7.0, 7.0], [1.0, 6.0, 1.0]]], [[[1.0, 4.0, 5.0], [3.0, 3.0, 
5.0], [4.0, 7.0, 7.0]], [[1.0, 8.0, 1.0], [3.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 5.0, 1.0], [6.0, 2.0, 2.0], [6.0, 3.0, 
6.0]]], [[[7.0, 7.0, 6.0], [7.0, 3.0, 8.0], [8.0, 8.0, 2.0]], [[5.0, 4.0, 1.0], [3.0, 7.0, 8.0], [4.0, 8.0, 2.0]], [[5.0, 2.0, 
3.0], [2.0, 5.0, 4.0], [4.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 7.0], [7.0, 6.0, 1.0], [4.0, 5.0, 2.0]], [[3.0, 8.0, 1.0], [1.0, 5.0, 
3.0], [8.0, 1.0, 7.0]], [[5.0, 2.0, 1.0], [8.0, 5.0, 7.0], [6.0, 2.0, 4.0]]], [[[1.0, 6.0, 2.0], [2.0, 1.0, 8.0], [3.0, 8.0, 
1.0]], [[8.0, 6.0, 8.0], [4.0, 5.0, 1.0], [6.0, 3.0, 3.0]], [[1.0, 8.0, 5.0], [8.0, 7.0, 1.0], [8.0, 5.0, 7.0]]], [[[5.0, 1.0, 
7.0], [6.0, 1.0, 6.0], [2.0, 3.0, 1.0]], [[4.0, 1.0, 2.0], [4.0, 3.0, 6.0], [5.0, 8.0, 7.0]], [[7.0, 7.0, 6.0], [5.0, 1.0, 
8.0], [4.0, 4.0, 6.0]]], [[[5.0, 4.0, 7.0], [1.0, 2.0, 5.0], [1.0, 1.0, 2.0]], [[1.0, 4.0, 1.0], [1.0, 3.0, 6.0], [3.0, 4.0, 
8.0]], [[3.0, 3.0, 7.0], [4.0, 8.0, 1.0], [8.0, 2.0, 4.0]]]]))
      val=numpy.array([[[7.0, 4.0, 4.0], [4.0, 2.0, 3.0]], [[8.0, 2.0, 3.0], [1.0, 1.0, 5.0]]])
      arg[1:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 4.0], [5.0, 2.0, 5.0], [7.0, 2.0, 6.0]], [[1.0, 8.0, 5.0], [8.0, 1.0, 6.0], [2.0, 
4.0, 8.0]], [[6.0, 1.0, 3.0], [8.0, 3.0, 4.0], [5.0, 8.0, 8.0]]], [[[4.0, 5.0, 3.0], [7.0, 4.0, 4.0], [3.0, 8.0, 8.0]], [[3.0, 
4.0, 3.0], [4.0, 2.0, 3.0], [1.0, 6.0, 2.0]], [[6.0, 3.0, 3.0], [2.0, 7.0, 7.0], [1.0, 6.0, 1.0]]], [[[1.0, 4.0, 5.0], [8.0, 
2.0, 3.0], [4.0, 7.0, 7.0]], [[1.0, 8.0, 1.0], [1.0, 1.0, 5.0], [4.0, 2.0, 3.0]], [[4.0, 5.0, 1.0], [6.0, 2.0, 2.0], [6.0, 3.0, 
6.0]]], [[[7.0, 7.0, 6.0], [7.0, 3.0, 8.0], [8.0, 8.0, 2.0]], [[5.0, 4.0, 1.0], [3.0, 7.0, 8.0], [4.0, 8.0, 2.0]], [[5.0, 2.0, 
3.0], [2.0, 5.0, 4.0], [4.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 7.0], [7.0, 6.0, 1.0], [4.0, 5.0, 2.0]], 
[[3.0, 8.0, 1.0], [1.0, 5.0, 3.0], [8.0, 1.0, 7.0]], [[5.0, 2.0, 1.0], [8.0, 5.0, 7.0], [6.0, 2.0, 4.0]]], [[[1.0, 6.0, 2.0], 
[7.0, 4.0, 4.0], [3.0, 8.0, 1.0]], [[8.0, 6.0, 8.0], [4.0, 2.0, 3.0], [6.0, 3.0, 3.0]], [[1.0, 8.0, 5.0], [8.0, 7.0, 1.0], 
[8.0, 5.0, 7.0]]], [[[5.0, 1.0, 7.0], [8.0, 2.0, 3.0], [2.0, 3.0, 1.0]], [[4.0, 1.0, 2.0], [1.0, 1.0, 5.0], [5.0, 8.0, 7.0]], 
[[7.0, 7.0, 6.0], [5.0, 1.0, 8.0], [4.0, 4.0, 6.0]]], [[[5.0, 4.0, 7.0], [1.0, 2.0, 5.0], [1.0, 1.0, 2.0]], [[1.0, 4.0, 1.0], 
[1.0, 3.0, 6.0], [3.0, 4.0, 8.0]], [[3.0, 3.0, 7.0], [4.0, 8.0, 1.0], [8.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RUPN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 3.0], [7.0, 7.0, 5.0], [2.0, 4.0, 2.0]], [[6.0, 4.0, 5.0], [1.0, 3.0, 4.0], [1.0, 2.0, 
7.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 6.0], [6.0, 1.0, 6.0]]], [[[5.0, 2.0, 3.0], [5.0, 8.0, 7.0], [2.0, 1.0, 8.0]], [[7.0, 1.0, 
6.0], [2.0, 4.0, 2.0], [7.0, 8.0, 2.0]], [[7.0, 2.0, 3.0], [6.0, 4.0, 7.0], [2.0, 5.0, 8.0]]], [[[1.0, 2.0, 7.0], [6.0, 4.0, 
1.0], [7.0, 6.0, 7.0]], [[7.0, 6.0, 5.0], [5.0, 7.0, 1.0], [4.0, 7.0, 3.0]], [[2.0, 7.0, 3.0], [4.0, 1.0, 4.0], [2.0, 1.0, 
5.0]]], [[[1.0, 4.0, 2.0], [2.0, 1.0, 1.0], [3.0, 2.0, 6.0]], [[4.0, 8.0, 2.0], [3.0, 8.0, 5.0], [2.0, 4.0, 3.0]], [[5.0, 7.0, 
6.0], [8.0, 3.0, 6.0], [2.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 1.0], [8.0, 7.0, 5.0], [3.0, 3.0, 6.0]], [[5.0, 1.0, 6.0], [1.0, 5.0, 
4.0], [3.0, 7.0, 3.0]], [[5.0, 2.0, 5.0], [1.0, 8.0, 6.0], [4.0, 8.0, 6.0]]], [[[2.0, 8.0, 1.0], [2.0, 8.0, 6.0], [4.0, 7.0, 
5.0]], [[5.0, 3.0, 6.0], [2.0, 2.0, 2.0], [2.0, 1.0, 4.0]], [[4.0, 8.0, 8.0], [1.0, 6.0, 4.0], [3.0, 4.0, 8.0]]], [[[6.0, 1.0, 
6.0], [7.0, 1.0, 4.0], [7.0, 8.0, 8.0]], [[6.0, 5.0, 4.0], [5.0, 5.0, 1.0], [4.0, 6.0, 7.0]], [[8.0, 6.0, 5.0], [6.0, 4.0, 
1.0], [8.0, 1.0, 6.0]]], [[[3.0, 4.0, 2.0], [3.0, 8.0, 3.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 1.0], [5.0, 4.0, 4.0], [3.0, 1.0, 
3.0]], [[3.0, 6.0, 4.0], [4.0, 4.0, 7.0], [4.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[6.0, 7.0, 4.0], [7.0, 4.0, 5.0]], [[6.0, 1.0, 3.0], [5.0, 1.0, 2.0]]]),self.functionspace)
      arg[1:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 3.0], [7.0, 7.0, 5.0], [2.0, 4.0, 2.0]], [[6.0, 4.0, 5.0], [1.0, 3.0, 4.0], [1.0, 
2.0, 7.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 6.0], [6.0, 1.0, 6.0]]], [[[5.0, 2.0, 3.0], [6.0, 7.0, 4.0], [2.0, 1.0, 8.0]], [[7.0, 
1.0, 6.0], [7.0, 4.0, 5.0], [7.0, 8.0, 2.0]], [[7.0, 2.0, 3.0], [6.0, 4.0, 7.0], [2.0, 5.0, 8.0]]], [[[1.0, 2.0, 7.0], [6.0, 
1.0, 3.0], [7.0, 6.0, 7.0]], [[7.0, 6.0, 5.0], [5.0, 1.0, 2.0], [4.0, 7.0, 3.0]], [[2.0, 7.0, 3.0], [4.0, 1.0, 4.0], [2.0, 1.0, 
5.0]]], [[[1.0, 4.0, 2.0], [2.0, 1.0, 1.0], [3.0, 2.0, 6.0]], [[4.0, 8.0, 2.0], [3.0, 8.0, 5.0], [2.0, 4.0, 3.0]], [[5.0, 7.0, 
6.0], [8.0, 3.0, 6.0], [2.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 1.0], [8.0, 7.0, 5.0], [3.0, 3.0, 6.0]], 
[[5.0, 1.0, 6.0], [1.0, 5.0, 4.0], [3.0, 7.0, 3.0]], [[5.0, 2.0, 5.0], [1.0, 8.0, 6.0], [4.0, 8.0, 6.0]]], [[[2.0, 8.0, 1.0], 
[6.0, 7.0, 4.0], [4.0, 7.0, 5.0]], [[5.0, 3.0, 6.0], [7.0, 4.0, 5.0], [2.0, 1.0, 4.0]], [[4.0, 8.0, 8.0], [1.0, 6.0, 4.0], 
[3.0, 4.0, 8.0]]], [[[6.0, 1.0, 6.0], [6.0, 1.0, 3.0], [7.0, 8.0, 8.0]], [[6.0, 5.0, 4.0], [5.0, 1.0, 2.0], [4.0, 6.0, 7.0]], 
[[8.0, 6.0, 5.0], [6.0, 4.0, 1.0], [8.0, 1.0, 6.0]]], [[[3.0, 4.0, 2.0], [3.0, 8.0, 3.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 1.0], 
[5.0, 4.0, 4.0], [3.0, 1.0, 3.0]], [[3.0, 6.0, 4.0], [4.0, 4.0, 7.0], [4.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RUPN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 2.0], [2.0, 1.0, 4.0], [2.0, 6.0, 5.0]], [[5.0, 1.0, 2.0], [8.0, 1.0, 1.0], [4.0, 1.0, 
6.0]], [[8.0, 5.0, 2.0], [8.0, 8.0, 4.0], [7.0, 2.0, 5.0]]], [[[3.0, 2.0, 7.0], [1.0, 8.0, 2.0], [4.0, 2.0, 7.0]], [[4.0, 3.0, 
2.0], [7.0, 5.0, 6.0], [7.0, 3.0, 6.0]], [[6.0, 1.0, 4.0], [1.0, 1.0, 7.0], [1.0, 7.0, 8.0]]], [[[8.0, 5.0, 2.0], [3.0, 5.0, 
2.0], [8.0, 2.0, 2.0]], [[7.0, 6.0, 2.0], [4.0, 7.0, 5.0], [2.0, 8.0, 8.0]], [[7.0, 1.0, 3.0], [2.0, 2.0, 6.0], [4.0, 5.0, 
4.0]]], [[[8.0, 8.0, 1.0], [5.0, 2.0, 6.0], [1.0, 6.0, 3.0]], [[4.0, 2.0, 7.0], [4.0, 4.0, 5.0], [7.0, 2.0, 5.0]], [[4.0, 7.0, 
5.0], [8.0, 4.0, 1.0], [4.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 6.0], [5.0, 6.0, 4.0], [1.0, 5.0, 4.0]], [[8.0, 8.0, 7.0], [4.0, 4.0, 
5.0], [2.0, 5.0, 2.0]], [[6.0, 2.0, 4.0], [1.0, 7.0, 3.0], [3.0, 6.0, 4.0]]], [[[6.0, 2.0, 4.0], [6.0, 5.0, 5.0], [3.0, 1.0, 
5.0]], [[4.0, 4.0, 2.0], [8.0, 8.0, 7.0], [8.0, 7.0, 6.0]], [[5.0, 2.0, 6.0], [6.0, 8.0, 5.0], [8.0, 3.0, 5.0]]], [[[4.0, 6.0, 
2.0], [3.0, 7.0, 1.0], [2.0, 2.0, 5.0]], [[4.0, 4.0, 4.0], [2.0, 5.0, 8.0], [5.0, 6.0, 5.0]], [[6.0, 4.0, 2.0], [3.0, 1.0, 
2.0], [6.0, 7.0, 2.0]]], [[[8.0, 2.0, 7.0], [4.0, 1.0, 1.0], [3.0, 6.0, 5.0]], [[5.0, 6.0, 4.0], [3.0, 5.0, 2.0], [2.0, 4.0, 
4.0]], [[6.0, 4.0, 2.0], [3.0, 8.0, 6.0], [1.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[[8.0, 5.0, 1.0], [3.0, 4.0, 2.0]], [[4.0, 8.0, 5.0], [4.0, 7.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 5.0, 6.0], [7.0, 5.0, 2.0]], [[1.0, 1.0, 2.0], [6.0, 6.0, 8.0]]]))
      arg[1:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 2.0], [2.0, 1.0, 4.0], [2.0, 6.0, 5.0]], [[5.0, 1.0, 2.0], [8.0, 1.0, 1.0], [4.0, 
1.0, 6.0]], [[8.0, 5.0, 2.0], [8.0, 8.0, 4.0], [7.0, 2.0, 5.0]]], [[[3.0, 2.0, 7.0], [8.0, 5.0, 1.0], [4.0, 2.0, 7.0]], [[4.0, 
3.0, 2.0], [3.0, 4.0, 2.0], [7.0, 3.0, 6.0]], [[6.0, 1.0, 4.0], [1.0, 1.0, 7.0], [1.0, 7.0, 8.0]]], [[[8.0, 5.0, 2.0], [4.0, 
8.0, 5.0], [8.0, 2.0, 2.0]], [[7.0, 6.0, 2.0], [4.0, 7.0, 4.0], [2.0, 8.0, 8.0]], [[7.0, 1.0, 3.0], [2.0, 2.0, 6.0], [4.0, 5.0, 
4.0]]], [[[8.0, 8.0, 1.0], [5.0, 2.0, 6.0], [1.0, 6.0, 3.0]], [[4.0, 2.0, 7.0], [4.0, 4.0, 5.0], [7.0, 2.0, 5.0]], [[4.0, 7.0, 
5.0], [8.0, 4.0, 1.0], [4.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 6.0], [5.0, 6.0, 4.0], [1.0, 5.0, 4.0]], 
[[8.0, 8.0, 7.0], [4.0, 4.0, 5.0], [2.0, 5.0, 2.0]], [[6.0, 2.0, 4.0], [1.0, 7.0, 3.0], [3.0, 6.0, 4.0]]], [[[6.0, 2.0, 4.0], 
[7.0, 5.0, 6.0], [3.0, 1.0, 5.0]], [[4.0, 4.0, 2.0], [7.0, 5.0, 2.0], [8.0, 7.0, 6.0]], [[5.0, 2.0, 6.0], [6.0, 8.0, 5.0], 
[8.0, 3.0, 5.0]]], [[[4.0, 6.0, 2.0], [1.0, 1.0, 2.0], [2.0, 2.0, 5.0]], [[4.0, 4.0, 4.0], [6.0, 6.0, 8.0], [5.0, 6.0, 5.0]], 
[[6.0, 4.0, 2.0], [3.0, 1.0, 2.0], [6.0, 7.0, 2.0]]], [[[8.0, 2.0, 7.0], [4.0, 1.0, 1.0], [3.0, 6.0, 5.0]], [[5.0, 6.0, 4.0], 
[3.0, 5.0, 2.0], [2.0, 4.0, 4.0]], [[6.0, 4.0, 2.0], [3.0, 8.0, 6.0], [1.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RUPN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 5.0], [3.0, 1.0, 3.0], [2.0, 4.0, 8.0]], [[8.0, 3.0, 2.0], [2.0, 4.0, 6.0], [8.0, 1.0, 
3.0]], [[3.0, 4.0, 1.0], [3.0, 2.0, 8.0], [1.0, 7.0, 7.0]]], [[[1.0, 5.0, 8.0], [6.0, 3.0, 5.0], [8.0, 5.0, 7.0]], [[5.0, 7.0, 
3.0], [2.0, 8.0, 3.0], [4.0, 6.0, 7.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 8.0], [4.0, 8.0, 8.0]]], [[[6.0, 8.0, 3.0], [6.0, 8.0, 
6.0], [4.0, 7.0, 4.0]], [[6.0, 6.0, 8.0], [5.0, 5.0, 1.0], [2.0, 7.0, 8.0]], [[3.0, 8.0, 2.0], [2.0, 6.0, 8.0], [1.0, 7.0, 
8.0]]], [[[6.0, 8.0, 6.0], [5.0, 5.0, 3.0], [4.0, 1.0, 2.0]], [[4.0, 4.0, 1.0], [1.0, 6.0, 8.0], [2.0, 6.0, 5.0]], [[4.0, 2.0, 
2.0], [6.0, 8.0, 2.0], [5.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 2.0], [2.0, 1.0, 2.0], [3.0, 7.0, 4.0]], [[4.0, 7.0, 4.0], [3.0, 2.0, 
7.0], [1.0, 7.0, 4.0]], [[4.0, 8.0, 1.0], [5.0, 2.0, 7.0], [1.0, 5.0, 5.0]]], [[[2.0, 1.0, 5.0], [2.0, 3.0, 6.0], [3.0, 7.0, 
2.0]], [[1.0, 3.0, 1.0], [7.0, 7.0, 2.0], [2.0, 5.0, 7.0]], [[6.0, 7.0, 7.0], [2.0, 2.0, 3.0], [5.0, 1.0, 8.0]]], [[[5.0, 7.0, 
7.0], [3.0, 4.0, 8.0], [6.0, 3.0, 8.0]], [[2.0, 2.0, 5.0], [3.0, 3.0, 8.0], [7.0, 5.0, 4.0]], [[6.0, 3.0, 4.0], [8.0, 8.0, 
8.0], [2.0, 3.0, 7.0]]], [[[3.0, 4.0, 2.0], [1.0, 3.0, 2.0], [1.0, 4.0, 1.0]], [[7.0, 7.0, 3.0], [5.0, 1.0, 7.0], [4.0, 2.0, 
8.0]], [[5.0, 7.0, 5.0], [4.0, 7.0, 5.0], [3.0, 8.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 3.0, 3.0], [6.0, 8.0, 1.0]], [[5.0, 1.0, 2.0], [8.0, 3.0, 
5.0]]])+(1.-msk_val)*numpy.array([[[6.0, 1.0, 1.0], [4.0, 5.0, 3.0]], [[1.0, 8.0, 1.0], [2.0, 2.0, 8.0]]])
      arg[1:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 5.0], [3.0, 1.0, 3.0], [2.0, 4.0, 8.0]], [[8.0, 3.0, 2.0], [2.0, 4.0, 6.0], [8.0, 
1.0, 3.0]], [[3.0, 4.0, 1.0], [3.0, 2.0, 8.0], [1.0, 7.0, 7.0]]], [[[1.0, 5.0, 8.0], [3.0, 3.0, 3.0], [8.0, 5.0, 7.0]], [[5.0, 
7.0, 3.0], [6.0, 8.0, 1.0], [4.0, 6.0, 7.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 8.0], [4.0, 8.0, 8.0]]], [[[6.0, 8.0, 3.0], [5.0, 
1.0, 2.0], [4.0, 7.0, 4.0]], [[6.0, 6.0, 8.0], [8.0, 3.0, 5.0], [2.0, 7.0, 8.0]], [[3.0, 8.0, 2.0], [2.0, 6.0, 8.0], [1.0, 7.0, 
8.0]]], [[[6.0, 8.0, 6.0], [5.0, 5.0, 3.0], [4.0, 1.0, 2.0]], [[4.0, 4.0, 1.0], [1.0, 6.0, 8.0], [2.0, 6.0, 5.0]], [[4.0, 2.0, 
2.0], [6.0, 8.0, 2.0], [5.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 2.0], [2.0, 1.0, 2.0], [3.0, 7.0, 4.0]], 
[[4.0, 7.0, 4.0], [3.0, 2.0, 7.0], [1.0, 7.0, 4.0]], [[4.0, 8.0, 1.0], [5.0, 2.0, 7.0], [1.0, 5.0, 5.0]]], [[[2.0, 1.0, 5.0], 
[6.0, 1.0, 1.0], [3.0, 7.0, 2.0]], [[1.0, 3.0, 1.0], [4.0, 5.0, 3.0], [2.0, 5.0, 7.0]], [[6.0, 7.0, 7.0], [2.0, 2.0, 3.0], 
[5.0, 1.0, 8.0]]], [[[5.0, 7.0, 7.0], [1.0, 8.0, 1.0], [6.0, 3.0, 8.0]], [[2.0, 2.0, 5.0], [2.0, 2.0, 8.0], [7.0, 5.0, 4.0]], 
[[6.0, 3.0, 4.0], [8.0, 8.0, 8.0], [2.0, 3.0, 7.0]]], [[[3.0, 4.0, 2.0], [1.0, 3.0, 2.0], [1.0, 4.0, 1.0]], [[7.0, 7.0, 3.0], 
[5.0, 1.0, 7.0], [4.0, 2.0, 8.0]], [[5.0, 7.0, 5.0], [4.0, 7.0, 5.0], [3.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRLN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 8.0], [8.0, 4.0, 3.0], [7.0, 7.0, 2.0]], [[1.0, 8.0, 2.0], [6.0, 8.0, 3.0], [6.0, 2.0, 
4.0]], [[4.0, 3.0, 8.0], [4.0, 4.0, 4.0], [2.0, 3.0, 6.0]]], [[[8.0, 8.0, 4.0], [5.0, 2.0, 2.0], [8.0, 1.0, 7.0]], [[2.0, 2.0, 
2.0], [6.0, 8.0, 4.0], [2.0, 1.0, 3.0]], [[4.0, 3.0, 3.0], [7.0, 3.0, 5.0], [8.0, 8.0, 7.0]]], [[[2.0, 6.0, 2.0], [6.0, 7.0, 
2.0], [5.0, 2.0, 8.0]], [[8.0, 1.0, 3.0], [6.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[4.0, 7.0, 3.0], [3.0, 3.0, 3.0], [3.0, 7.0, 
8.0]]], [[[2.0, 3.0, 8.0], [8.0, 1.0, 1.0], [3.0, 8.0, 1.0]], [[6.0, 1.0, 1.0], [7.0, 1.0, 5.0], [7.0, 4.0, 4.0]], [[3.0, 6.0, 
1.0], [8.0, 6.0, 5.0], [5.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 4.0], [7.0, 1.0, 6.0], [7.0, 8.0, 5.0]], [[4.0, 2.0, 3.0], [1.0, 1.0, 
1.0], [5.0, 6.0, 1.0]], [[5.0, 2.0, 6.0], [6.0, 5.0, 4.0], [3.0, 7.0, 7.0]]], [[[5.0, 5.0, 7.0], [1.0, 6.0, 8.0], [4.0, 2.0, 
2.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 3.0], [3.0, 1.0, 5.0]], [[8.0, 2.0, 7.0], [5.0, 8.0, 5.0], [3.0, 6.0, 7.0]]], [[[2.0, 2.0, 
1.0], [2.0, 5.0, 3.0], [8.0, 6.0, 4.0]], [[4.0, 6.0, 7.0], [2.0, 4.0, 7.0], [6.0, 1.0, 6.0]], [[2.0, 5.0, 6.0], [4.0, 2.0, 
7.0], [6.0, 6.0, 4.0]]], [[[6.0, 7.0, 3.0], [7.0, 1.0, 3.0], [1.0, 3.0, 3.0]], [[8.0, 7.0, 8.0], [3.0, 3.0, 5.0], [1.0, 7.0, 
8.0]], [[3.0, 3.0, 5.0], [5.0, 3.0, 8.0], [1.0, 3.0, 7.0]]]]))
      val=numpy.array([[[[1.0, 4.0, 6.0]]], [[[1.0, 1.0, 3.0]]], [[[1.0, 2.0, 1.0]]], [[[2.0, 7.0, 2.0]]]])
      arg[:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 8.0], [8.0, 4.0, 3.0], [7.0, 7.0, 2.0]], [[1.0, 8.0, 2.0], [6.0, 8.0, 3.0], [1.0, 
4.0, 6.0]], [[4.0, 3.0, 8.0], [4.0, 4.0, 4.0], [2.0, 3.0, 6.0]]], [[[8.0, 8.0, 4.0], [5.0, 2.0, 2.0], [8.0, 1.0, 7.0]], [[2.0, 
2.0, 2.0], [6.0, 8.0, 4.0], [1.0, 1.0, 3.0]], [[4.0, 3.0, 3.0], [7.0, 3.0, 5.0], [8.0, 8.0, 7.0]]], [[[2.0, 6.0, 2.0], [6.0, 
7.0, 2.0], [5.0, 2.0, 8.0]], [[8.0, 1.0, 3.0], [6.0, 8.0, 5.0], [1.0, 2.0, 1.0]], [[4.0, 7.0, 3.0], [3.0, 3.0, 3.0], [3.0, 7.0, 
8.0]]], [[[2.0, 3.0, 8.0], [8.0, 1.0, 1.0], [3.0, 8.0, 1.0]], [[6.0, 1.0, 1.0], [7.0, 1.0, 5.0], [2.0, 7.0, 2.0]], [[3.0, 6.0, 
1.0], [8.0, 6.0, 5.0], [5.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 4.0], [7.0, 1.0, 6.0], [7.0, 8.0, 5.0]], 
[[4.0, 2.0, 3.0], [1.0, 1.0, 1.0], [1.0, 4.0, 6.0]], [[5.0, 2.0, 6.0], [6.0, 5.0, 4.0], [3.0, 7.0, 7.0]]], [[[5.0, 5.0, 7.0], 
[1.0, 6.0, 8.0], [4.0, 2.0, 2.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 3.0], [1.0, 1.0, 3.0]], [[8.0, 2.0, 7.0], [5.0, 8.0, 5.0], 
[3.0, 6.0, 7.0]]], [[[2.0, 2.0, 1.0], [2.0, 5.0, 3.0], [8.0, 6.0, 4.0]], [[4.0, 6.0, 7.0], [2.0, 4.0, 7.0], [1.0, 2.0, 1.0]], 
[[2.0, 5.0, 6.0], [4.0, 2.0, 7.0], [6.0, 6.0, 4.0]]], [[[6.0, 7.0, 3.0], [7.0, 1.0, 3.0], [1.0, 3.0, 3.0]], [[8.0, 7.0, 8.0], 
[3.0, 3.0, 5.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 5.0], [5.0, 3.0, 8.0], [1.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRLN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 6.0], [2.0, 4.0, 6.0], [3.0, 4.0, 6.0]], [[1.0, 7.0, 8.0], [8.0, 3.0, 4.0], [7.0, 1.0, 
8.0]], [[5.0, 7.0, 5.0], [1.0, 7.0, 5.0], [8.0, 5.0, 8.0]]], [[[1.0, 4.0, 7.0], [2.0, 6.0, 8.0], [4.0, 7.0, 8.0]], [[4.0, 2.0, 
1.0], [5.0, 4.0, 5.0], [5.0, 3.0, 8.0]], [[2.0, 1.0, 2.0], [6.0, 8.0, 2.0], [3.0, 8.0, 1.0]]], [[[3.0, 2.0, 1.0], [6.0, 8.0, 
6.0], [2.0, 7.0, 3.0]], [[5.0, 8.0, 8.0], [3.0, 7.0, 1.0], [5.0, 8.0, 2.0]], [[4.0, 6.0, 2.0], [3.0, 4.0, 2.0], [2.0, 7.0, 
5.0]]], [[[1.0, 1.0, 1.0], [1.0, 7.0, 3.0], [5.0, 1.0, 5.0]], [[7.0, 7.0, 3.0], [6.0, 1.0, 5.0], [1.0, 2.0, 6.0]], [[2.0, 1.0, 
8.0], [7.0, 6.0, 8.0], [6.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 2.0], [5.0, 6.0, 3.0], [6.0, 2.0, 1.0]], [[1.0, 3.0, 7.0], [5.0, 8.0, 
2.0], [4.0, 1.0, 1.0]], [[2.0, 8.0, 2.0], [2.0, 1.0, 1.0], [8.0, 5.0, 7.0]]], [[[1.0, 2.0, 2.0], [5.0, 5.0, 6.0], [6.0, 3.0, 
2.0]], [[7.0, 6.0, 5.0], [1.0, 2.0, 6.0], [4.0, 8.0, 6.0]], [[8.0, 3.0, 1.0], [7.0, 7.0, 1.0], [4.0, 3.0, 2.0]]], [[[5.0, 4.0, 
4.0], [3.0, 8.0, 6.0], [2.0, 4.0, 1.0]], [[7.0, 7.0, 6.0], [6.0, 4.0, 3.0], [7.0, 1.0, 4.0]], [[4.0, 7.0, 2.0], [3.0, 3.0, 
1.0], [5.0, 4.0, 5.0]]], [[[8.0, 7.0, 2.0], [5.0, 6.0, 1.0], [5.0, 7.0, 3.0]], [[1.0, 7.0, 4.0], [8.0, 7.0, 5.0], [5.0, 3.0, 
8.0]], [[5.0, 6.0, 3.0], [2.0, 1.0, 4.0], [6.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[[1.0, 3.0, 5.0]]], [[[4.0, 3.0, 5.0]]], [[[8.0, 7.0, 1.0]]], [[[6.0, 6.0, 
8.0]]]]),self.functionspace)
      arg[:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 6.0], [2.0, 4.0, 6.0], [3.0, 4.0, 6.0]], [[1.0, 7.0, 8.0], [8.0, 3.0, 4.0], [1.0, 
3.0, 5.0]], [[5.0, 7.0, 5.0], [1.0, 7.0, 5.0], [8.0, 5.0, 8.0]]], [[[1.0, 4.0, 7.0], [2.0, 6.0, 8.0], [4.0, 7.0, 8.0]], [[4.0, 
2.0, 1.0], [5.0, 4.0, 5.0], [4.0, 3.0, 5.0]], [[2.0, 1.0, 2.0], [6.0, 8.0, 2.0], [3.0, 8.0, 1.0]]], [[[3.0, 2.0, 1.0], [6.0, 
8.0, 6.0], [2.0, 7.0, 3.0]], [[5.0, 8.0, 8.0], [3.0, 7.0, 1.0], [8.0, 7.0, 1.0]], [[4.0, 6.0, 2.0], [3.0, 4.0, 2.0], [2.0, 7.0, 
5.0]]], [[[1.0, 1.0, 1.0], [1.0, 7.0, 3.0], [5.0, 1.0, 5.0]], [[7.0, 7.0, 3.0], [6.0, 1.0, 5.0], [6.0, 6.0, 8.0]], [[2.0, 1.0, 
8.0], [7.0, 6.0, 8.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 2.0], [5.0, 6.0, 3.0], [6.0, 2.0, 1.0]], 
[[1.0, 3.0, 7.0], [5.0, 8.0, 2.0], [1.0, 3.0, 5.0]], [[2.0, 8.0, 2.0], [2.0, 1.0, 1.0], [8.0, 5.0, 7.0]]], [[[1.0, 2.0, 2.0], 
[5.0, 5.0, 6.0], [6.0, 3.0, 2.0]], [[7.0, 6.0, 5.0], [1.0, 2.0, 6.0], [4.0, 3.0, 5.0]], [[8.0, 3.0, 1.0], [7.0, 7.0, 1.0], 
[4.0, 3.0, 2.0]]], [[[5.0, 4.0, 4.0], [3.0, 8.0, 6.0], [2.0, 4.0, 1.0]], [[7.0, 7.0, 6.0], [6.0, 4.0, 3.0], [8.0, 7.0, 1.0]], 
[[4.0, 7.0, 2.0], [3.0, 3.0, 1.0], [5.0, 4.0, 5.0]]], [[[8.0, 7.0, 2.0], [5.0, 6.0, 1.0], [5.0, 7.0, 3.0]], [[1.0, 7.0, 4.0], 
[8.0, 7.0, 5.0], [6.0, 6.0, 8.0]], [[5.0, 6.0, 3.0], [2.0, 1.0, 4.0], [6.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRLN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 3.0], [1.0, 4.0, 7.0], [8.0, 8.0, 3.0]], [[5.0, 8.0, 3.0], [3.0, 1.0, 5.0], [5.0, 2.0, 
8.0]], [[1.0, 5.0, 6.0], [6.0, 3.0, 6.0], [4.0, 3.0, 7.0]]], [[[6.0, 8.0, 5.0], [4.0, 5.0, 1.0], [3.0, 7.0, 1.0]], [[7.0, 7.0, 
4.0], [3.0, 1.0, 7.0], [7.0, 5.0, 1.0]], [[4.0, 6.0, 3.0], [2.0, 2.0, 1.0], [5.0, 2.0, 1.0]]], [[[8.0, 4.0, 5.0], [7.0, 4.0, 
6.0], [6.0, 6.0, 5.0]], [[7.0, 7.0, 4.0], [1.0, 7.0, 6.0], [7.0, 4.0, 2.0]], [[8.0, 5.0, 4.0], [4.0, 4.0, 1.0], [5.0, 8.0, 
2.0]]], [[[1.0, 3.0, 4.0], [6.0, 8.0, 4.0], [6.0, 8.0, 4.0]], [[7.0, 3.0, 1.0], [4.0, 7.0, 7.0], [1.0, 2.0, 7.0]], [[2.0, 4.0, 
8.0], [4.0, 5.0, 3.0], [4.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 7.0], [6.0, 7.0, 2.0], [8.0, 8.0, 5.0]], [[3.0, 5.0, 1.0], [2.0, 8.0, 
4.0], [5.0, 2.0, 2.0]], [[4.0, 2.0, 5.0], [8.0, 2.0, 3.0], [1.0, 1.0, 2.0]]], [[[7.0, 4.0, 7.0], [5.0, 5.0, 8.0], [8.0, 2.0, 
2.0]], [[4.0, 7.0, 8.0], [8.0, 2.0, 5.0], [6.0, 5.0, 7.0]], [[2.0, 8.0, 2.0], [3.0, 7.0, 2.0], [5.0, 8.0, 7.0]]], [[[5.0, 3.0, 
4.0], [8.0, 5.0, 8.0], [7.0, 2.0, 7.0]], [[7.0, 4.0, 4.0], [7.0, 2.0, 4.0], [4.0, 6.0, 6.0]], [[3.0, 2.0, 4.0], [6.0, 7.0, 
7.0], [4.0, 7.0, 4.0]]], [[[1.0, 3.0, 3.0], [5.0, 2.0, 4.0], [6.0, 7.0, 8.0]], [[1.0, 7.0, 8.0], [2.0, 3.0, 7.0], [2.0, 6.0, 
1.0]], [[2.0, 7.0, 8.0], [2.0, 8.0, 4.0], [6.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[[5.0, 8.0, 3.0]]], [[[4.0, 3.0, 3.0]]], [[[5.0, 3.0, 8.0]]], [[[3.0, 3.0, 
7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 2.0]]], [[[3.0, 8.0, 3.0]]], [[[2.0, 7.0, 2.0]]], [[[6.0, 2.0, 6.0]]]]))
      arg[:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 3.0], [1.0, 4.0, 7.0], [8.0, 8.0, 3.0]], [[5.0, 8.0, 3.0], [3.0, 1.0, 5.0], [5.0, 
8.0, 3.0]], [[1.0, 5.0, 6.0], [6.0, 3.0, 6.0], [4.0, 3.0, 7.0]]], [[[6.0, 8.0, 5.0], [4.0, 5.0, 1.0], [3.0, 7.0, 1.0]], [[7.0, 
7.0, 4.0], [3.0, 1.0, 7.0], [4.0, 3.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 2.0, 1.0], [5.0, 2.0, 1.0]]], [[[8.0, 4.0, 5.0], [7.0, 
4.0, 6.0], [6.0, 6.0, 5.0]], [[7.0, 7.0, 4.0], [1.0, 7.0, 6.0], [5.0, 3.0, 8.0]], [[8.0, 5.0, 4.0], [4.0, 4.0, 1.0], [5.0, 8.0, 
2.0]]], [[[1.0, 3.0, 4.0], [6.0, 8.0, 4.0], [6.0, 8.0, 4.0]], [[7.0, 3.0, 1.0], [4.0, 7.0, 7.0], [3.0, 3.0, 7.0]], [[2.0, 4.0, 
8.0], [4.0, 5.0, 3.0], [4.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 7.0], [6.0, 7.0, 2.0], [8.0, 8.0, 5.0]], 
[[3.0, 5.0, 1.0], [2.0, 8.0, 4.0], [3.0, 1.0, 2.0]], [[4.0, 2.0, 5.0], [8.0, 2.0, 3.0], [1.0, 1.0, 2.0]]], [[[7.0, 4.0, 7.0], 
[5.0, 5.0, 8.0], [8.0, 2.0, 2.0]], [[4.0, 7.0, 8.0], [8.0, 2.0, 5.0], [3.0, 8.0, 3.0]], [[2.0, 8.0, 2.0], [3.0, 7.0, 2.0], 
[5.0, 8.0, 7.0]]], [[[5.0, 3.0, 4.0], [8.0, 5.0, 8.0], [7.0, 2.0, 7.0]], [[7.0, 4.0, 4.0], [7.0, 2.0, 4.0], [2.0, 7.0, 2.0]], 
[[3.0, 2.0, 4.0], [6.0, 7.0, 7.0], [4.0, 7.0, 4.0]]], [[[1.0, 3.0, 3.0], [5.0, 2.0, 4.0], [6.0, 7.0, 8.0]], [[1.0, 7.0, 8.0], 
[2.0, 3.0, 7.0], [6.0, 2.0, 6.0]], [[2.0, 7.0, 8.0], [2.0, 8.0, 4.0], [6.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRLN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 6.0], [2.0, 1.0, 6.0], [3.0, 8.0, 8.0]], [[4.0, 5.0, 5.0], [8.0, 2.0, 6.0], [8.0, 2.0, 
7.0]], [[5.0, 6.0, 2.0], [6.0, 2.0, 2.0], [7.0, 1.0, 3.0]]], [[[7.0, 6.0, 4.0], [8.0, 5.0, 7.0], [4.0, 4.0, 1.0]], [[6.0, 2.0, 
5.0], [7.0, 2.0, 8.0], [2.0, 8.0, 3.0]], [[4.0, 7.0, 1.0], [5.0, 5.0, 6.0], [7.0, 6.0, 2.0]]], [[[4.0, 1.0, 8.0], [1.0, 1.0, 
1.0], [4.0, 7.0, 1.0]], [[4.0, 4.0, 6.0], [7.0, 1.0, 6.0], [5.0, 3.0, 7.0]], [[6.0, 6.0, 5.0], [3.0, 6.0, 1.0], [3.0, 4.0, 
4.0]]], [[[1.0, 7.0, 7.0], [8.0, 5.0, 7.0], [6.0, 2.0, 6.0]], [[3.0, 3.0, 5.0], [5.0, 5.0, 8.0], [7.0, 3.0, 7.0]], [[4.0, 4.0, 
8.0], [1.0, 6.0, 5.0], [4.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 6.0], [6.0, 4.0, 2.0], [6.0, 8.0, 7.0]], [[1.0, 2.0, 1.0], [8.0, 2.0, 
1.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 6.0], [4.0, 4.0, 6.0], [7.0, 7.0, 8.0]]], [[[3.0, 3.0, 4.0], [2.0, 2.0, 1.0], [7.0, 5.0, 
1.0]], [[8.0, 5.0, 8.0], [6.0, 4.0, 6.0], [4.0, 3.0, 4.0]], [[7.0, 1.0, 8.0], [8.0, 4.0, 3.0], [2.0, 2.0, 3.0]]], [[[7.0, 3.0, 
8.0], [6.0, 1.0, 3.0], [4.0, 6.0, 5.0]], [[6.0, 2.0, 3.0], [4.0, 5.0, 4.0], [4.0, 4.0, 8.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 
2.0], [1.0, 2.0, 6.0]]], [[[1.0, 6.0, 7.0], [5.0, 5.0, 8.0], [6.0, 2.0, 1.0]], [[3.0, 4.0, 5.0], [2.0, 5.0, 3.0], [7.0, 7.0, 
2.0]], [[1.0, 7.0, 1.0], [4.0, 2.0, 8.0], [8.0, 4.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 6.0, 7.0]]], [[[2.0, 3.0, 1.0]]], [[[3.0, 8.0, 6.0]]], [[[4.0, 1.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 1.0, 3.0]]], [[[2.0, 5.0, 4.0]]], [[[3.0, 2.0, 4.0]]], [[[6.0, 6.0, 6.0]]]])
      arg[:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 6.0], [2.0, 1.0, 6.0], [3.0, 8.0, 8.0]], [[4.0, 5.0, 5.0], [8.0, 2.0, 6.0], [2.0, 
6.0, 7.0]], [[5.0, 6.0, 2.0], [6.0, 2.0, 2.0], [7.0, 1.0, 3.0]]], [[[7.0, 6.0, 4.0], [8.0, 5.0, 7.0], [4.0, 4.0, 1.0]], [[6.0, 
2.0, 5.0], [7.0, 2.0, 8.0], [2.0, 3.0, 1.0]], [[4.0, 7.0, 1.0], [5.0, 5.0, 6.0], [7.0, 6.0, 2.0]]], [[[4.0, 1.0, 8.0], [1.0, 
1.0, 1.0], [4.0, 7.0, 1.0]], [[4.0, 4.0, 6.0], [7.0, 1.0, 6.0], [3.0, 8.0, 6.0]], [[6.0, 6.0, 5.0], [3.0, 6.0, 1.0], [3.0, 4.0, 
4.0]]], [[[1.0, 7.0, 7.0], [8.0, 5.0, 7.0], [6.0, 2.0, 6.0]], [[3.0, 3.0, 5.0], [5.0, 5.0, 8.0], [4.0, 1.0, 7.0]], [[4.0, 4.0, 
8.0], [1.0, 6.0, 5.0], [4.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 6.0], [6.0, 4.0, 2.0], [6.0, 8.0, 7.0]], 
[[1.0, 2.0, 1.0], [8.0, 2.0, 1.0], [6.0, 1.0, 3.0]], [[6.0, 4.0, 6.0], [4.0, 4.0, 6.0], [7.0, 7.0, 8.0]]], [[[3.0, 3.0, 4.0], 
[2.0, 2.0, 1.0], [7.0, 5.0, 1.0]], [[8.0, 5.0, 8.0], [6.0, 4.0, 6.0], [2.0, 5.0, 4.0]], [[7.0, 1.0, 8.0], [8.0, 4.0, 3.0], 
[2.0, 2.0, 3.0]]], [[[7.0, 3.0, 8.0], [6.0, 1.0, 3.0], [4.0, 6.0, 5.0]], [[6.0, 2.0, 3.0], [4.0, 5.0, 4.0], [3.0, 2.0, 4.0]], 
[[6.0, 6.0, 3.0], [3.0, 8.0, 2.0], [1.0, 2.0, 6.0]]], [[[1.0, 6.0, 7.0], [5.0, 5.0, 8.0], [6.0, 2.0, 1.0]], [[3.0, 4.0, 5.0], 
[2.0, 5.0, 3.0], [6.0, 6.0, 6.0]], [[1.0, 7.0, 1.0], [4.0, 2.0, 8.0], [8.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LPLU(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 1.0], [1.0, 5.0, 2.0], [5.0, 4.0, 6.0]], [[4.0, 1.0, 2.0], [4.0, 1.0, 6.0], [4.0, 6.0, 
3.0]], [[2.0, 6.0, 3.0], [3.0, 8.0, 7.0], [3.0, 8.0, 6.0]]], [[[1.0, 1.0, 2.0], [3.0, 5.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 3.0, 
4.0], [7.0, 4.0, 5.0], [2.0, 7.0, 1.0]], [[3.0, 7.0, 1.0], [6.0, 5.0, 4.0], [3.0, 1.0, 7.0]]], [[[2.0, 1.0, 6.0], [2.0, 2.0, 
4.0], [4.0, 7.0, 1.0]], [[3.0, 4.0, 2.0], [7.0, 4.0, 3.0], [5.0, 2.0, 2.0]], [[4.0, 8.0, 6.0], [1.0, 7.0, 2.0], [3.0, 4.0, 
1.0]]], [[[2.0, 7.0, 8.0], [2.0, 5.0, 8.0], [1.0, 8.0, 5.0]], [[2.0, 4.0, 1.0], [6.0, 5.0, 1.0], [4.0, 7.0, 5.0]], [[6.0, 5.0, 
2.0], [7.0, 8.0, 8.0], [6.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 4.0], [2.0, 8.0, 3.0], [3.0, 4.0, 5.0]], [[1.0, 3.0, 7.0], [5.0, 4.0, 
4.0], [7.0, 1.0, 7.0]], [[2.0, 4.0, 7.0], [4.0, 3.0, 5.0], [1.0, 7.0, 3.0]]], [[[1.0, 5.0, 8.0], [7.0, 8.0, 6.0], [8.0, 8.0, 
7.0]], [[7.0, 6.0, 8.0], [6.0, 5.0, 4.0], [1.0, 5.0, 6.0]], [[8.0, 1.0, 6.0], [7.0, 4.0, 3.0], [6.0, 7.0, 6.0]]], [[[5.0, 8.0, 
2.0], [2.0, 3.0, 1.0], [6.0, 3.0, 4.0]], [[2.0, 8.0, 3.0], [5.0, 5.0, 5.0], [1.0, 5.0, 7.0]], [[3.0, 3.0, 8.0], [7.0, 2.0, 
3.0], [7.0, 7.0, 4.0]]], [[[8.0, 8.0, 8.0], [6.0, 3.0, 4.0], [4.0, 6.0, 2.0]], [[3.0, 7.0, 6.0], [7.0, 2.0, 8.0], [2.0, 8.0, 
5.0]], [[8.0, 4.0, 2.0], [8.0, 2.0, 1.0], [4.0, 6.0, 8.0]]]]))
      val=numpy.array([[[1.0, 1.0]], [[5.0, 5.0]]])
      arg[2:,1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 1.0], [1.0, 5.0, 2.0], [5.0, 4.0, 6.0]], [[4.0, 1.0, 2.0], [4.0, 1.0, 6.0], [4.0, 
6.0, 3.0]], [[2.0, 6.0, 3.0], [3.0, 8.0, 7.0], [3.0, 8.0, 6.0]]], [[[1.0, 1.0, 2.0], [3.0, 5.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 
3.0, 4.0], [7.0, 4.0, 5.0], [2.0, 7.0, 1.0]], [[3.0, 7.0, 1.0], [6.0, 5.0, 4.0], [3.0, 1.0, 7.0]]], [[[2.0, 1.0, 6.0], [2.0, 
2.0, 4.0], [4.0, 7.0, 1.0]], [[3.0, 4.0, 2.0], [7.0, 4.0, 3.0], [1.0, 1.0, 2.0]], [[4.0, 8.0, 6.0], [1.0, 7.0, 2.0], [3.0, 4.0, 
1.0]]], [[[2.0, 7.0, 8.0], [2.0, 5.0, 8.0], [1.0, 8.0, 5.0]], [[2.0, 4.0, 1.0], [6.0, 5.0, 1.0], [5.0, 5.0, 5.0]], [[6.0, 5.0, 
2.0], [7.0, 8.0, 8.0], [6.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [2.0, 8.0, 3.0], [3.0, 4.0, 5.0]], 
[[1.0, 3.0, 7.0], [5.0, 4.0, 4.0], [7.0, 1.0, 7.0]], [[2.0, 4.0, 7.0], [4.0, 3.0, 5.0], [1.0, 7.0, 3.0]]], [[[1.0, 5.0, 8.0], 
[7.0, 8.0, 6.0], [8.0, 8.0, 7.0]], [[7.0, 6.0, 8.0], [6.0, 5.0, 4.0], [1.0, 5.0, 6.0]], [[8.0, 1.0, 6.0], [7.0, 4.0, 3.0], 
[6.0, 7.0, 6.0]]], [[[5.0, 8.0, 2.0], [2.0, 3.0, 1.0], [6.0, 3.0, 4.0]], [[2.0, 8.0, 3.0], [5.0, 5.0, 5.0], [1.0, 1.0, 7.0]], 
[[3.0, 3.0, 8.0], [7.0, 2.0, 3.0], [7.0, 7.0, 4.0]]], [[[8.0, 8.0, 8.0], [6.0, 3.0, 4.0], [4.0, 6.0, 2.0]], [[3.0, 7.0, 6.0], 
[7.0, 2.0, 8.0], [5.0, 5.0, 5.0]], [[8.0, 4.0, 2.0], [8.0, 2.0, 1.0], [4.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LPLU(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 7.0], [8.0, 6.0, 2.0], [2.0, 5.0, 8.0]], [[4.0, 1.0, 4.0], [3.0, 1.0, 5.0], [2.0, 4.0, 
8.0]], [[5.0, 8.0, 7.0], [6.0, 2.0, 2.0], [3.0, 1.0, 5.0]]], [[[4.0, 1.0, 3.0], [3.0, 6.0, 7.0], [3.0, 1.0, 2.0]], [[4.0, 4.0, 
5.0], [8.0, 4.0, 2.0], [5.0, 5.0, 8.0]], [[4.0, 6.0, 2.0], [2.0, 6.0, 3.0], [1.0, 2.0, 3.0]]], [[[7.0, 7.0, 7.0], [3.0, 8.0, 
4.0], [2.0, 5.0, 4.0]], [[8.0, 7.0, 3.0], [7.0, 3.0, 3.0], [7.0, 6.0, 2.0]], [[7.0, 4.0, 8.0], [7.0, 4.0, 8.0], [1.0, 1.0, 
5.0]]], [[[5.0, 4.0, 8.0], [4.0, 5.0, 3.0], [5.0, 6.0, 8.0]], [[3.0, 4.0, 4.0], [3.0, 5.0, 1.0], [8.0, 2.0, 3.0]], [[8.0, 7.0, 
3.0], [7.0, 3.0, 3.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 6.0], [6.0, 8.0, 8.0], [5.0, 4.0, 4.0]], [[3.0, 4.0, 4.0], [2.0, 2.0, 
8.0], [1.0, 7.0, 3.0]], [[5.0, 5.0, 5.0], [1.0, 8.0, 1.0], [8.0, 7.0, 5.0]]], [[[3.0, 3.0, 6.0], [1.0, 1.0, 3.0], [8.0, 7.0, 
6.0]], [[4.0, 3.0, 8.0], [4.0, 1.0, 5.0], [6.0, 1.0, 7.0]], [[1.0, 1.0, 2.0], [5.0, 4.0, 2.0], [3.0, 1.0, 6.0]]], [[[4.0, 8.0, 
7.0], [1.0, 4.0, 4.0], [4.0, 8.0, 3.0]], [[1.0, 8.0, 5.0], [1.0, 5.0, 1.0], [1.0, 4.0, 4.0]], [[1.0, 5.0, 6.0], [1.0, 5.0, 
7.0], [7.0, 1.0, 4.0]]], [[[4.0, 6.0, 8.0], [2.0, 5.0, 3.0], [4.0, 3.0, 8.0]], [[7.0, 3.0, 7.0], [2.0, 8.0, 8.0], [7.0, 5.0, 
6.0]], [[8.0, 2.0, 2.0], [6.0, 6.0, 2.0], [6.0, 2.0, 6.0]]]]))
      val=Data(numpy.array([[[8.0, 8.0]], [[7.0, 5.0]]]),self.functionspace)
      arg[2:,1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 7.0], [8.0, 6.0, 2.0], [2.0, 5.0, 8.0]], [[4.0, 1.0, 4.0], [3.0, 1.0, 5.0], [2.0, 
4.0, 8.0]], [[5.0, 8.0, 7.0], [6.0, 2.0, 2.0], [3.0, 1.0, 5.0]]], [[[4.0, 1.0, 3.0], [3.0, 6.0, 7.0], [3.0, 1.0, 2.0]], [[4.0, 
4.0, 5.0], [8.0, 4.0, 2.0], [5.0, 5.0, 8.0]], [[4.0, 6.0, 2.0], [2.0, 6.0, 3.0], [1.0, 2.0, 3.0]]], [[[7.0, 7.0, 7.0], [3.0, 
8.0, 4.0], [2.0, 5.0, 4.0]], [[8.0, 7.0, 3.0], [7.0, 3.0, 3.0], [8.0, 8.0, 2.0]], [[7.0, 4.0, 8.0], [7.0, 4.0, 8.0], [1.0, 1.0, 
5.0]]], [[[5.0, 4.0, 8.0], [4.0, 5.0, 3.0], [5.0, 6.0, 8.0]], [[3.0, 4.0, 4.0], [3.0, 5.0, 1.0], [7.0, 5.0, 3.0]], [[8.0, 7.0, 
3.0], [7.0, 3.0, 3.0], [2.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 6.0], [6.0, 8.0, 8.0], [5.0, 4.0, 4.0]], 
[[3.0, 4.0, 4.0], [2.0, 2.0, 8.0], [1.0, 7.0, 3.0]], [[5.0, 5.0, 5.0], [1.0, 8.0, 1.0], [8.0, 7.0, 5.0]]], [[[3.0, 3.0, 6.0], 
[1.0, 1.0, 3.0], [8.0, 7.0, 6.0]], [[4.0, 3.0, 8.0], [4.0, 1.0, 5.0], [6.0, 1.0, 7.0]], [[1.0, 1.0, 2.0], [5.0, 4.0, 2.0], 
[3.0, 1.0, 6.0]]], [[[4.0, 8.0, 7.0], [1.0, 4.0, 4.0], [4.0, 8.0, 3.0]], [[1.0, 8.0, 5.0], [1.0, 5.0, 1.0], [8.0, 8.0, 4.0]], 
[[1.0, 5.0, 6.0], [1.0, 5.0, 7.0], [7.0, 1.0, 4.0]]], [[[4.0, 6.0, 8.0], [2.0, 5.0, 3.0], [4.0, 3.0, 8.0]], [[7.0, 3.0, 7.0], 
[2.0, 8.0, 8.0], [7.0, 5.0, 6.0]], [[8.0, 2.0, 2.0], [6.0, 6.0, 2.0], [6.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LPLU(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 7.0], [2.0, 4.0, 4.0], [5.0, 4.0, 7.0]], [[4.0, 2.0, 3.0], [7.0, 5.0, 2.0], [7.0, 4.0, 
3.0]], [[6.0, 1.0, 5.0], [8.0, 1.0, 8.0], [6.0, 6.0, 3.0]]], [[[2.0, 7.0, 6.0], [5.0, 2.0, 3.0], [6.0, 6.0, 3.0]], [[2.0, 8.0, 
7.0], [6.0, 5.0, 3.0], [4.0, 4.0, 2.0]], [[2.0, 1.0, 1.0], [8.0, 2.0, 1.0], [6.0, 1.0, 2.0]]], [[[5.0, 3.0, 3.0], [4.0, 4.0, 
6.0], [2.0, 4.0, 3.0]], [[1.0, 1.0, 8.0], [3.0, 5.0, 4.0], [1.0, 1.0, 4.0]], [[6.0, 6.0, 5.0], [5.0, 7.0, 8.0], [4.0, 7.0, 
8.0]]], [[[2.0, 2.0, 1.0], [5.0, 8.0, 7.0], [1.0, 2.0, 8.0]], [[8.0, 7.0, 6.0], [4.0, 2.0, 1.0], [5.0, 1.0, 1.0]], [[5.0, 7.0, 
3.0], [5.0, 1.0, 1.0], [7.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 5.0], [7.0, 3.0, 6.0], [7.0, 8.0, 1.0]], [[5.0, 4.0, 2.0], [3.0, 1.0, 
7.0], [3.0, 6.0, 1.0]], [[8.0, 7.0, 1.0], [4.0, 5.0, 2.0], [7.0, 8.0, 5.0]]], [[[2.0, 8.0, 5.0], [2.0, 7.0, 3.0], [3.0, 6.0, 
5.0]], [[6.0, 3.0, 5.0], [4.0, 2.0, 2.0], [4.0, 1.0, 5.0]], [[3.0, 5.0, 5.0], [6.0, 8.0, 7.0], [3.0, 6.0, 3.0]]], [[[7.0, 3.0, 
3.0], [5.0, 4.0, 2.0], [5.0, 4.0, 7.0]], [[2.0, 8.0, 5.0], [1.0, 5.0, 3.0], [3.0, 3.0, 3.0]], [[4.0, 1.0, 1.0], [1.0, 5.0, 
1.0], [8.0, 7.0, 5.0]]], [[[1.0, 2.0, 7.0], [5.0, 7.0, 1.0], [2.0, 2.0, 6.0]], [[5.0, 7.0, 1.0], [2.0, 1.0, 8.0], [5.0, 2.0, 
1.0]], [[2.0, 5.0, 6.0], [4.0, 6.0, 8.0], [8.0, 8.0, 8.0]]]]))
      val=Data(numpy.array([[[5.0, 2.0]], [[8.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 7.0]], [[1.0, 8.0]]]))
      arg[2:,1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 7.0], [2.0, 4.0, 4.0], [5.0, 4.0, 7.0]], [[4.0, 2.0, 3.0], [7.0, 5.0, 2.0], [7.0, 
4.0, 3.0]], [[6.0, 1.0, 5.0], [8.0, 1.0, 8.0], [6.0, 6.0, 3.0]]], [[[2.0, 7.0, 6.0], [5.0, 2.0, 3.0], [6.0, 6.0, 3.0]], [[2.0, 
8.0, 7.0], [6.0, 5.0, 3.0], [4.0, 4.0, 2.0]], [[2.0, 1.0, 1.0], [8.0, 2.0, 1.0], [6.0, 1.0, 2.0]]], [[[5.0, 3.0, 3.0], [4.0, 
4.0, 6.0], [2.0, 4.0, 3.0]], [[1.0, 1.0, 8.0], [3.0, 5.0, 4.0], [5.0, 2.0, 4.0]], [[6.0, 6.0, 5.0], [5.0, 7.0, 8.0], [4.0, 7.0, 
8.0]]], [[[2.0, 2.0, 1.0], [5.0, 8.0, 7.0], [1.0, 2.0, 8.0]], [[8.0, 7.0, 6.0], [4.0, 2.0, 1.0], [8.0, 8.0, 1.0]], [[5.0, 7.0, 
3.0], [5.0, 1.0, 1.0], [7.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 5.0], [7.0, 3.0, 6.0], [7.0, 8.0, 1.0]], 
[[5.0, 4.0, 2.0], [3.0, 1.0, 7.0], [3.0, 6.0, 1.0]], [[8.0, 7.0, 1.0], [4.0, 5.0, 2.0], [7.0, 8.0, 5.0]]], [[[2.0, 8.0, 5.0], 
[2.0, 7.0, 3.0], [3.0, 6.0, 5.0]], [[6.0, 3.0, 5.0], [4.0, 2.0, 2.0], [4.0, 1.0, 5.0]], [[3.0, 5.0, 5.0], [6.0, 8.0, 7.0], 
[3.0, 6.0, 3.0]]], [[[7.0, 3.0, 3.0], [5.0, 4.0, 2.0], [5.0, 4.0, 7.0]], [[2.0, 8.0, 5.0], [1.0, 5.0, 3.0], [4.0, 7.0, 3.0]], 
[[4.0, 1.0, 1.0], [1.0, 5.0, 1.0], [8.0, 7.0, 5.0]]], [[[1.0, 2.0, 7.0], [5.0, 7.0, 1.0], [2.0, 2.0, 6.0]], [[5.0, 7.0, 1.0], 
[2.0, 1.0, 8.0], [1.0, 8.0, 1.0]], [[2.0, 5.0, 6.0], [4.0, 6.0, 8.0], [8.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LPLU(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 3.0], [2.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[5.0, 8.0, 5.0], [8.0, 8.0, 1.0], [1.0, 1.0, 
3.0]], [[6.0, 7.0, 5.0], [6.0, 4.0, 2.0], [3.0, 4.0, 5.0]]], [[[5.0, 6.0, 5.0], [2.0, 4.0, 8.0], [4.0, 2.0, 7.0]], [[8.0, 7.0, 
7.0], [1.0, 5.0, 2.0], [1.0, 3.0, 2.0]], [[6.0, 4.0, 5.0], [2.0, 1.0, 6.0], [1.0, 7.0, 2.0]]], [[[3.0, 4.0, 8.0], [8.0, 8.0, 
7.0], [5.0, 3.0, 1.0]], [[5.0, 4.0, 6.0], [6.0, 8.0, 6.0], [3.0, 5.0, 8.0]], [[6.0, 5.0, 8.0], [5.0, 6.0, 1.0], [6.0, 5.0, 
8.0]]], [[[5.0, 2.0, 1.0], [5.0, 1.0, 8.0], [4.0, 7.0, 2.0]], [[7.0, 7.0, 4.0], [7.0, 2.0, 8.0], [4.0, 2.0, 2.0]], [[5.0, 8.0, 
1.0], [8.0, 8.0, 1.0], [7.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 2.0], [6.0, 7.0, 8.0], [1.0, 8.0, 8.0]], [[2.0, 5.0, 6.0], [7.0, 3.0, 
3.0], [2.0, 1.0, 3.0]], [[8.0, 5.0, 7.0], [6.0, 4.0, 3.0], [8.0, 6.0, 3.0]]], [[[6.0, 2.0, 1.0], [2.0, 1.0, 1.0], [8.0, 5.0, 
5.0]], [[4.0, 2.0, 8.0], [4.0, 2.0, 5.0], [2.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], [6.0, 8.0, 4.0], [4.0, 2.0, 2.0]]], [[[1.0, 3.0, 
3.0], [8.0, 2.0, 7.0], [7.0, 5.0, 7.0]], [[6.0, 6.0, 7.0], [1.0, 4.0, 5.0], [8.0, 5.0, 8.0]], [[7.0, 3.0, 3.0], [4.0, 5.0, 
2.0], [3.0, 4.0, 1.0]]], [[[7.0, 4.0, 3.0], [6.0, 1.0, 7.0], [1.0, 6.0, 5.0]], [[8.0, 4.0, 5.0], [5.0, 6.0, 7.0], [7.0, 1.0, 
8.0]], [[6.0, 6.0, 8.0], [3.0, 5.0, 2.0], [1.0, 3.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 8.0]], [[1.0, 4.0]]])+(1.-msk_val)*numpy.array([[[6.0, 1.0]], [[3.0, 8.0]]])
      arg[2:,1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 3.0], [2.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[5.0, 8.0, 5.0], [8.0, 8.0, 1.0], [1.0, 
1.0, 3.0]], [[6.0, 7.0, 5.0], [6.0, 4.0, 2.0], [3.0, 4.0, 5.0]]], [[[5.0, 6.0, 5.0], [2.0, 4.0, 8.0], [4.0, 2.0, 7.0]], [[8.0, 
7.0, 7.0], [1.0, 5.0, 2.0], [1.0, 3.0, 2.0]], [[6.0, 4.0, 5.0], [2.0, 1.0, 6.0], [1.0, 7.0, 2.0]]], [[[3.0, 4.0, 8.0], [8.0, 
8.0, 7.0], [5.0, 3.0, 1.0]], [[5.0, 4.0, 6.0], [6.0, 8.0, 6.0], [7.0, 8.0, 8.0]], [[6.0, 5.0, 8.0], [5.0, 6.0, 1.0], [6.0, 5.0, 
8.0]]], [[[5.0, 2.0, 1.0], [5.0, 1.0, 8.0], [4.0, 7.0, 2.0]], [[7.0, 7.0, 4.0], [7.0, 2.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 8.0, 
1.0], [8.0, 8.0, 1.0], [7.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 2.0], [6.0, 7.0, 8.0], [1.0, 8.0, 8.0]], 
[[2.0, 5.0, 6.0], [7.0, 3.0, 3.0], [2.0, 1.0, 3.0]], [[8.0, 5.0, 7.0], [6.0, 4.0, 3.0], [8.0, 6.0, 3.0]]], [[[6.0, 2.0, 1.0], 
[2.0, 1.0, 1.0], [8.0, 5.0, 5.0]], [[4.0, 2.0, 8.0], [4.0, 2.0, 5.0], [2.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], [6.0, 8.0, 4.0], 
[4.0, 2.0, 2.0]]], [[[1.0, 3.0, 3.0], [8.0, 2.0, 7.0], [7.0, 5.0, 7.0]], [[6.0, 6.0, 7.0], [1.0, 4.0, 5.0], [6.0, 1.0, 8.0]], 
[[7.0, 3.0, 3.0], [4.0, 5.0, 2.0], [3.0, 4.0, 1.0]]], [[[7.0, 4.0, 3.0], [6.0, 1.0, 7.0], [1.0, 6.0, 5.0]], [[8.0, 4.0, 5.0], 
[5.0, 6.0, 7.0], [3.0, 8.0, 8.0]], [[6.0, 6.0, 8.0], [3.0, 5.0, 2.0], [1.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLCP(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 5.0], [6.0, 3.0, 7.0], [2.0, 6.0, 3.0]], [[6.0, 6.0, 5.0], [6.0, 7.0, 5.0], [2.0, 8.0, 
1.0]], [[7.0, 2.0, 4.0], [4.0, 6.0, 7.0], [6.0, 4.0, 2.0]]], [[[1.0, 2.0, 3.0], [4.0, 6.0, 5.0], [3.0, 2.0, 1.0]], [[7.0, 3.0, 
7.0], [7.0, 1.0, 4.0], [8.0, 2.0, 4.0]], [[5.0, 1.0, 3.0], [4.0, 5.0, 3.0], [3.0, 8.0, 3.0]]], [[[8.0, 3.0, 4.0], [4.0, 3.0, 
8.0], [3.0, 3.0, 2.0]], [[8.0, 5.0, 1.0], [1.0, 2.0, 4.0], [3.0, 5.0, 2.0]], [[8.0, 4.0, 7.0], [7.0, 5.0, 7.0], [4.0, 7.0, 
3.0]]], [[[6.0, 2.0, 5.0], [2.0, 6.0, 5.0], [6.0, 8.0, 1.0]], [[3.0, 6.0, 5.0], [2.0, 6.0, 8.0], [3.0, 8.0, 5.0]], [[5.0, 3.0, 
1.0], [2.0, 8.0, 3.0], [7.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 2.0], [4.0, 4.0, 7.0], [2.0, 6.0, 3.0]], [[5.0, 6.0, 6.0], [6.0, 5.0, 
7.0], [2.0, 1.0, 8.0]], [[5.0, 1.0, 5.0], [1.0, 2.0, 6.0], [8.0, 7.0, 1.0]]], [[[5.0, 6.0, 3.0], [3.0, 5.0, 3.0], [6.0, 7.0, 
8.0]], [[5.0, 8.0, 1.0], [4.0, 3.0, 2.0], [1.0, 6.0, 7.0]], [[5.0, 6.0, 6.0], [6.0, 3.0, 2.0], [6.0, 8.0, 6.0]]], [[[2.0, 6.0, 
1.0], [2.0, 5.0, 4.0], [2.0, 5.0, 1.0]], [[7.0, 6.0, 3.0], [8.0, 8.0, 2.0], [6.0, 2.0, 2.0]], [[6.0, 7.0, 6.0], [8.0, 4.0, 
4.0], [8.0, 5.0, 7.0]]], [[[4.0, 6.0, 7.0], [8.0, 6.0, 2.0], [6.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [5.0, 1.0, 2.0], [2.0, 5.0, 
2.0]], [[8.0, 1.0, 4.0], [4.0, 2.0, 3.0], [8.0, 7.0, 7.0]]]]))
      val=numpy.array([[[6.0, 5.0, 5.0]], [[7.0, 3.0, 5.0]]])
      arg[2:,2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 5.0], [6.0, 3.0, 7.0], [2.0, 6.0, 3.0]], [[6.0, 6.0, 5.0], [6.0, 7.0, 5.0], [2.0, 
8.0, 1.0]], [[7.0, 2.0, 4.0], [4.0, 6.0, 7.0], [6.0, 4.0, 2.0]]], [[[1.0, 2.0, 3.0], [4.0, 6.0, 5.0], [3.0, 2.0, 1.0]], [[7.0, 
3.0, 7.0], [7.0, 1.0, 4.0], [8.0, 2.0, 4.0]], [[5.0, 1.0, 3.0], [4.0, 5.0, 3.0], [3.0, 8.0, 3.0]]], [[[8.0, 3.0, 4.0], [4.0, 
3.0, 8.0], [3.0, 3.0, 2.0]], [[8.0, 5.0, 1.0], [1.0, 2.0, 4.0], [3.0, 5.0, 2.0]], [[8.0, 6.0, 7.0], [7.0, 5.0, 7.0], [4.0, 5.0, 
3.0]]], [[[6.0, 2.0, 5.0], [2.0, 6.0, 5.0], [6.0, 8.0, 1.0]], [[3.0, 6.0, 5.0], [2.0, 6.0, 8.0], [3.0, 8.0, 5.0]], [[5.0, 7.0, 
1.0], [2.0, 3.0, 3.0], [7.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 2.0], [4.0, 4.0, 7.0], [2.0, 6.0, 3.0]], 
[[5.0, 6.0, 6.0], [6.0, 5.0, 7.0], [2.0, 1.0, 8.0]], [[5.0, 1.0, 5.0], [1.0, 2.0, 6.0], [8.0, 7.0, 1.0]]], [[[5.0, 6.0, 3.0], 
[3.0, 5.0, 3.0], [6.0, 7.0, 8.0]], [[5.0, 8.0, 1.0], [4.0, 3.0, 2.0], [1.0, 6.0, 7.0]], [[5.0, 6.0, 6.0], [6.0, 3.0, 2.0], 
[6.0, 8.0, 6.0]]], [[[2.0, 6.0, 1.0], [2.0, 5.0, 4.0], [2.0, 5.0, 1.0]], [[7.0, 6.0, 3.0], [8.0, 8.0, 2.0], [6.0, 2.0, 2.0]], 
[[6.0, 6.0, 6.0], [8.0, 5.0, 4.0], [8.0, 5.0, 7.0]]], [[[4.0, 6.0, 7.0], [8.0, 6.0, 2.0], [6.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], 
[5.0, 1.0, 2.0], [2.0, 5.0, 2.0]], [[8.0, 7.0, 4.0], [4.0, 3.0, 3.0], [8.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLCP(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 1.0], [3.0, 8.0, 2.0], [7.0, 6.0, 1.0]], [[4.0, 5.0, 7.0], [4.0, 2.0, 8.0], [2.0, 8.0, 
5.0]], [[8.0, 1.0, 7.0], [1.0, 5.0, 4.0], [6.0, 7.0, 6.0]]], [[[1.0, 5.0, 2.0], [7.0, 3.0, 4.0], [4.0, 6.0, 4.0]], [[5.0, 4.0, 
6.0], [5.0, 5.0, 4.0], [2.0, 6.0, 6.0]], [[6.0, 4.0, 2.0], [2.0, 6.0, 8.0], [8.0, 4.0, 6.0]]], [[[6.0, 5.0, 5.0], [7.0, 4.0, 
2.0], [6.0, 1.0, 6.0]], [[7.0, 5.0, 8.0], [8.0, 4.0, 5.0], [5.0, 7.0, 7.0]], [[2.0, 6.0, 8.0], [1.0, 5.0, 7.0], [3.0, 1.0, 
2.0]]], [[[1.0, 2.0, 6.0], [7.0, 6.0, 2.0], [8.0, 3.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 6.0, 2.0], [1.0, 8.0, 8.0]], [[5.0, 2.0, 
5.0], [8.0, 2.0, 6.0], [5.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 3.0], [1.0, 3.0, 4.0], [4.0, 8.0, 6.0]], [[6.0, 4.0, 1.0], [7.0, 6.0, 
4.0], [5.0, 5.0, 2.0]], [[4.0, 5.0, 8.0], [7.0, 1.0, 4.0], [3.0, 2.0, 2.0]]], [[[4.0, 3.0, 3.0], [1.0, 3.0, 3.0], [2.0, 8.0, 
4.0]], [[5.0, 2.0, 1.0], [1.0, 1.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 4.0, 3.0], [5.0, 7.0, 8.0], [6.0, 2.0, 8.0]]], [[[1.0, 3.0, 
6.0], [7.0, 7.0, 1.0], [2.0, 7.0, 6.0]], [[2.0, 4.0, 7.0], [5.0, 4.0, 5.0], [4.0, 3.0, 7.0]], [[4.0, 4.0, 2.0], [2.0, 7.0, 
6.0], [4.0, 2.0, 7.0]]], [[[6.0, 7.0, 1.0], [8.0, 7.0, 3.0], [6.0, 8.0, 5.0]], [[4.0, 5.0, 1.0], [7.0, 4.0, 5.0], [1.0, 6.0, 
4.0]], [[4.0, 3.0, 3.0], [5.0, 1.0, 4.0], [7.0, 2.0, 3.0]]]]))
      val=Data(numpy.array([[[5.0, 7.0, 4.0]], [[4.0, 7.0, 7.0]]]),self.functionspace)
      arg[2:,2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 1.0], [3.0, 8.0, 2.0], [7.0, 6.0, 1.0]], [[4.0, 5.0, 7.0], [4.0, 2.0, 8.0], [2.0, 
8.0, 5.0]], [[8.0, 1.0, 7.0], [1.0, 5.0, 4.0], [6.0, 7.0, 6.0]]], [[[1.0, 5.0, 2.0], [7.0, 3.0, 4.0], [4.0, 6.0, 4.0]], [[5.0, 
4.0, 6.0], [5.0, 5.0, 4.0], [2.0, 6.0, 6.0]], [[6.0, 4.0, 2.0], [2.0, 6.0, 8.0], [8.0, 4.0, 6.0]]], [[[6.0, 5.0, 5.0], [7.0, 
4.0, 2.0], [6.0, 1.0, 6.0]], [[7.0, 5.0, 8.0], [8.0, 4.0, 5.0], [5.0, 7.0, 7.0]], [[2.0, 5.0, 8.0], [1.0, 7.0, 7.0], [3.0, 4.0, 
2.0]]], [[[1.0, 2.0, 6.0], [7.0, 6.0, 2.0], [8.0, 3.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 6.0, 2.0], [1.0, 8.0, 8.0]], [[5.0, 4.0, 
5.0], [8.0, 7.0, 6.0], [5.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 3.0], [1.0, 3.0, 4.0], [4.0, 8.0, 6.0]], 
[[6.0, 4.0, 1.0], [7.0, 6.0, 4.0], [5.0, 5.0, 2.0]], [[4.0, 5.0, 8.0], [7.0, 1.0, 4.0], [3.0, 2.0, 2.0]]], [[[4.0, 3.0, 3.0], 
[1.0, 3.0, 3.0], [2.0, 8.0, 4.0]], [[5.0, 2.0, 1.0], [1.0, 1.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 4.0, 3.0], [5.0, 7.0, 8.0], 
[6.0, 2.0, 8.0]]], [[[1.0, 3.0, 6.0], [7.0, 7.0, 1.0], [2.0, 7.0, 6.0]], [[2.0, 4.0, 7.0], [5.0, 4.0, 5.0], [4.0, 3.0, 7.0]], 
[[4.0, 5.0, 2.0], [2.0, 7.0, 6.0], [4.0, 4.0, 7.0]]], [[[6.0, 7.0, 1.0], [8.0, 7.0, 3.0], [6.0, 8.0, 5.0]], [[4.0, 5.0, 1.0], 
[7.0, 4.0, 5.0], [1.0, 6.0, 4.0]], [[4.0, 4.0, 3.0], [5.0, 7.0, 4.0], [7.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLCP(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 5.0], [6.0, 8.0, 7.0], [1.0, 4.0, 3.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 6.0], [6.0, 6.0, 
8.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 2.0], [7.0, 3.0, 2.0]]], [[[7.0, 5.0, 1.0], [8.0, 8.0, 5.0], [4.0, 8.0, 4.0]], [[1.0, 7.0, 
6.0], [5.0, 1.0, 4.0], [4.0, 5.0, 6.0]], [[3.0, 1.0, 5.0], [8.0, 4.0, 2.0], [3.0, 1.0, 1.0]]], [[[8.0, 5.0, 7.0], [7.0, 1.0, 
2.0], [2.0, 6.0, 6.0]], [[7.0, 1.0, 4.0], [6.0, 8.0, 5.0], [3.0, 1.0, 1.0]], [[8.0, 1.0, 2.0], [1.0, 4.0, 8.0], [6.0, 5.0, 
8.0]]], [[[5.0, 6.0, 2.0], [6.0, 2.0, 6.0], [2.0, 5.0, 4.0]], [[2.0, 7.0, 7.0], [5.0, 3.0, 8.0], [6.0, 8.0, 5.0]], [[6.0, 6.0, 
6.0], [6.0, 2.0, 6.0], [3.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 1.0], [1.0, 5.0, 7.0], [6.0, 5.0, 1.0]], [[6.0, 5.0, 8.0], [7.0, 4.0, 
6.0], [4.0, 5.0, 1.0]], [[1.0, 4.0, 8.0], [6.0, 4.0, 5.0], [4.0, 7.0, 8.0]]], [[[8.0, 1.0, 3.0], [3.0, 4.0, 1.0], [8.0, 6.0, 
4.0]], [[7.0, 1.0, 2.0], [2.0, 1.0, 2.0], [7.0, 4.0, 1.0]], [[1.0, 2.0, 3.0], [8.0, 7.0, 7.0], [5.0, 8.0, 6.0]]], [[[7.0, 5.0, 
4.0], [5.0, 7.0, 3.0], [8.0, 8.0, 6.0]], [[5.0, 7.0, 1.0], [6.0, 8.0, 1.0], [8.0, 3.0, 1.0]], [[3.0, 5.0, 4.0], [3.0, 7.0, 
4.0], [7.0, 4.0, 3.0]]], [[[1.0, 3.0, 1.0], [7.0, 6.0, 4.0], [5.0, 1.0, 2.0]], [[8.0, 8.0, 3.0], [4.0, 6.0, 1.0], [8.0, 8.0, 
5.0]], [[1.0, 8.0, 3.0], [8.0, 4.0, 2.0], [6.0, 8.0, 4.0]]]]))
      val=Data(numpy.array([[[5.0, 8.0, 8.0]], [[7.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 3.0, 1.0]], [[4.0, 6.0, 1.0]]]))
      arg[2:,2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 5.0], [6.0, 8.0, 7.0], [1.0, 4.0, 3.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 6.0], [6.0, 
6.0, 8.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 2.0], [7.0, 3.0, 2.0]]], [[[7.0, 5.0, 1.0], [8.0, 8.0, 5.0], [4.0, 8.0, 4.0]], [[1.0, 
7.0, 6.0], [5.0, 1.0, 4.0], [4.0, 5.0, 6.0]], [[3.0, 1.0, 5.0], [8.0, 4.0, 2.0], [3.0, 1.0, 1.0]]], [[[8.0, 5.0, 7.0], [7.0, 
1.0, 2.0], [2.0, 6.0, 6.0]], [[7.0, 1.0, 4.0], [6.0, 8.0, 5.0], [3.0, 1.0, 1.0]], [[8.0, 5.0, 2.0], [1.0, 8.0, 8.0], [6.0, 8.0, 
8.0]]], [[[5.0, 6.0, 2.0], [6.0, 2.0, 6.0], [2.0, 5.0, 4.0]], [[2.0, 7.0, 7.0], [5.0, 3.0, 8.0], [6.0, 8.0, 5.0]], [[6.0, 7.0, 
6.0], [6.0, 3.0, 6.0], [3.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 1.0], [1.0, 5.0, 7.0], [6.0, 5.0, 1.0]], 
[[6.0, 5.0, 8.0], [7.0, 4.0, 6.0], [4.0, 5.0, 1.0]], [[1.0, 4.0, 8.0], [6.0, 4.0, 5.0], [4.0, 7.0, 8.0]]], [[[8.0, 1.0, 3.0], 
[3.0, 4.0, 1.0], [8.0, 6.0, 4.0]], [[7.0, 1.0, 2.0], [2.0, 1.0, 2.0], [7.0, 4.0, 1.0]], [[1.0, 2.0, 3.0], [8.0, 7.0, 7.0], 
[5.0, 8.0, 6.0]]], [[[7.0, 5.0, 4.0], [5.0, 7.0, 3.0], [8.0, 8.0, 6.0]], [[5.0, 7.0, 1.0], [6.0, 8.0, 1.0], [8.0, 3.0, 1.0]], 
[[3.0, 3.0, 4.0], [3.0, 3.0, 4.0], [7.0, 1.0, 3.0]]], [[[1.0, 3.0, 1.0], [7.0, 6.0, 4.0], [5.0, 1.0, 2.0]], [[8.0, 8.0, 3.0], 
[4.0, 6.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 4.0, 3.0], [8.0, 6.0, 2.0], [6.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLCP(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 6.0], [5.0, 4.0, 8.0], [1.0, 6.0, 8.0]], [[1.0, 6.0, 5.0], [3.0, 7.0, 4.0], [6.0, 2.0, 
6.0]], [[1.0, 6.0, 4.0], [8.0, 4.0, 7.0], [8.0, 3.0, 7.0]]], [[[2.0, 7.0, 6.0], [8.0, 2.0, 8.0], [1.0, 7.0, 3.0]], [[7.0, 4.0, 
2.0], [1.0, 6.0, 4.0], [3.0, 5.0, 6.0]], [[6.0, 4.0, 2.0], [1.0, 2.0, 8.0], [3.0, 7.0, 1.0]]], [[[3.0, 6.0, 8.0], [2.0, 3.0, 
6.0], [5.0, 2.0, 6.0]], [[2.0, 1.0, 4.0], [5.0, 1.0, 4.0], [7.0, 1.0, 3.0]], [[7.0, 4.0, 8.0], [8.0, 6.0, 7.0], [3.0, 7.0, 
2.0]]], [[[2.0, 4.0, 7.0], [7.0, 5.0, 1.0], [8.0, 7.0, 2.0]], [[7.0, 2.0, 6.0], [1.0, 6.0, 2.0], [3.0, 7.0, 1.0]], [[2.0, 5.0, 
2.0], [5.0, 2.0, 3.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 1.0], [6.0, 7.0, 1.0], [8.0, 6.0, 8.0]], [[3.0, 5.0, 1.0], [6.0, 6.0, 
8.0], [7.0, 8.0, 6.0]], [[4.0, 5.0, 4.0], [4.0, 7.0, 4.0], [3.0, 1.0, 7.0]]], [[[1.0, 6.0, 1.0], [8.0, 5.0, 8.0], [8.0, 4.0, 
3.0]], [[7.0, 5.0, 4.0], [7.0, 7.0, 2.0], [6.0, 2.0, 7.0]], [[6.0, 5.0, 5.0], [3.0, 8.0, 4.0], [7.0, 1.0, 6.0]]], [[[5.0, 3.0, 
5.0], [5.0, 5.0, 4.0], [3.0, 8.0, 8.0]], [[4.0, 8.0, 4.0], [3.0, 8.0, 3.0], [5.0, 3.0, 1.0]], [[4.0, 1.0, 1.0], [3.0, 8.0, 
3.0], [6.0, 6.0, 5.0]]], [[[6.0, 3.0, 7.0], [7.0, 3.0, 8.0], [2.0, 7.0, 7.0]], [[2.0, 1.0, 1.0], [8.0, 1.0, 2.0], [1.0, 4.0, 
8.0]], [[5.0, 4.0, 6.0], [3.0, 3.0, 4.0], [2.0, 3.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 4.0, 7.0]], [[6.0, 2.0, 8.0]]])+(1.-msk_val)*numpy.array([[[7.0, 3.0, 3.0]], [[4.0, 2.0, 
7.0]]])
      arg[2:,2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 6.0], [5.0, 4.0, 8.0], [1.0, 6.0, 8.0]], [[1.0, 6.0, 5.0], [3.0, 7.0, 4.0], [6.0, 
2.0, 6.0]], [[1.0, 6.0, 4.0], [8.0, 4.0, 7.0], [8.0, 3.0, 7.0]]], [[[2.0, 7.0, 6.0], [8.0, 2.0, 8.0], [1.0, 7.0, 3.0]], [[7.0, 
4.0, 2.0], [1.0, 6.0, 4.0], [3.0, 5.0, 6.0]], [[6.0, 4.0, 2.0], [1.0, 2.0, 8.0], [3.0, 7.0, 1.0]]], [[[3.0, 6.0, 8.0], [2.0, 
3.0, 6.0], [5.0, 2.0, 6.0]], [[2.0, 1.0, 4.0], [5.0, 1.0, 4.0], [7.0, 1.0, 3.0]], [[7.0, 7.0, 8.0], [8.0, 4.0, 7.0], [3.0, 7.0, 
2.0]]], [[[2.0, 4.0, 7.0], [7.0, 5.0, 1.0], [8.0, 7.0, 2.0]], [[7.0, 2.0, 6.0], [1.0, 6.0, 2.0], [3.0, 7.0, 1.0]], [[2.0, 6.0, 
2.0], [5.0, 2.0, 3.0], [1.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 1.0], [6.0, 7.0, 1.0], [8.0, 6.0, 8.0]], 
[[3.0, 5.0, 1.0], [6.0, 6.0, 8.0], [7.0, 8.0, 6.0]], [[4.0, 5.0, 4.0], [4.0, 7.0, 4.0], [3.0, 1.0, 7.0]]], [[[1.0, 6.0, 1.0], 
[8.0, 5.0, 8.0], [8.0, 4.0, 3.0]], [[7.0, 5.0, 4.0], [7.0, 7.0, 2.0], [6.0, 2.0, 7.0]], [[6.0, 5.0, 5.0], [3.0, 8.0, 4.0], 
[7.0, 1.0, 6.0]]], [[[5.0, 3.0, 5.0], [5.0, 5.0, 4.0], [3.0, 8.0, 8.0]], [[4.0, 8.0, 4.0], [3.0, 8.0, 3.0], [5.0, 3.0, 1.0]], 
[[4.0, 7.0, 1.0], [3.0, 3.0, 3.0], [6.0, 3.0, 5.0]]], [[[6.0, 3.0, 7.0], [7.0, 3.0, 8.0], [2.0, 7.0, 7.0]], [[2.0, 1.0, 1.0], 
[8.0, 1.0, 2.0], [1.0, 4.0, 8.0]], [[5.0, 4.0, 6.0], [3.0, 2.0, 4.0], [2.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CLUR(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 5.0], [7.0, 8.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 8.0, 1.0], [2.0, 7.0, 6.0], [3.0, 2.0, 
1.0]], [[4.0, 1.0, 8.0], [6.0, 5.0, 7.0], [4.0, 4.0, 4.0]]], [[[1.0, 7.0, 8.0], [4.0, 8.0, 5.0], [2.0, 8.0, 2.0]], [[1.0, 8.0, 
6.0], [1.0, 7.0, 2.0], [1.0, 4.0, 4.0]], [[5.0, 8.0, 8.0], [6.0, 3.0, 2.0], [5.0, 3.0, 6.0]]], [[[7.0, 4.0, 6.0], [3.0, 1.0, 
8.0], [1.0, 1.0, 2.0]], [[7.0, 8.0, 8.0], [1.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 6.0], [3.0, 6.0, 3.0], [5.0, 7.0, 
2.0]]], [[[1.0, 2.0, 1.0], [2.0, 2.0, 3.0], [1.0, 4.0, 8.0]], [[5.0, 7.0, 1.0], [8.0, 6.0, 6.0], [3.0, 1.0, 7.0]], [[6.0, 7.0, 
6.0], [8.0, 3.0, 3.0], [1.0, 5.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 2.0], [3.0, 4.0, 1.0], [1.0, 4.0, 6.0]], [[1.0, 1.0, 5.0], [3.0, 5.0, 
8.0], [1.0, 6.0, 2.0]], [[5.0, 1.0, 4.0], [2.0, 5.0, 3.0], [1.0, 6.0, 7.0]]], [[[4.0, 5.0, 8.0], [5.0, 8.0, 5.0], [7.0, 7.0, 
6.0]], [[8.0, 1.0, 1.0], [1.0, 7.0, 4.0], [7.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [4.0, 8.0, 4.0], [2.0, 6.0, 3.0]]], [[[8.0, 2.0, 
2.0], [7.0, 8.0, 4.0], [2.0, 6.0, 8.0]], [[3.0, 3.0, 3.0], [7.0, 7.0, 2.0], [6.0, 5.0, 6.0]], [[3.0, 1.0, 8.0], [5.0, 2.0, 
5.0], [4.0, 1.0, 2.0]]], [[[1.0, 2.0, 4.0], [5.0, 4.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 8.0, 6.0], [7.0, 1.0, 4.0], [8.0, 4.0, 
3.0]], [[8.0, 6.0, 4.0], [5.0, 6.0, 4.0], [1.0, 4.0, 2.0]]]]))
      val=numpy.array([[[[2.0], [2.0]]], [[[6.0], [5.0]]], [[[2.0], [6.0]]], [[[8.0], [1.0]]]])
      arg[:,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 5.0], [7.0, 8.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 8.0, 1.0], [2.0, 7.0, 6.0], [3.0, 
2.0, 1.0]], [[4.0, 2.0, 8.0], [6.0, 2.0, 7.0], [4.0, 4.0, 4.0]]], [[[1.0, 7.0, 8.0], [4.0, 8.0, 5.0], [2.0, 8.0, 2.0]], [[1.0, 
8.0, 6.0], [1.0, 7.0, 2.0], [1.0, 4.0, 4.0]], [[5.0, 6.0, 8.0], [6.0, 5.0, 2.0], [5.0, 3.0, 6.0]]], [[[7.0, 4.0, 6.0], [3.0, 
1.0, 8.0], [1.0, 1.0, 2.0]], [[7.0, 8.0, 8.0], [1.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[5.0, 2.0, 6.0], [3.0, 6.0, 3.0], [5.0, 7.0, 
2.0]]], [[[1.0, 2.0, 1.0], [2.0, 2.0, 3.0], [1.0, 4.0, 8.0]], [[5.0, 7.0, 1.0], [8.0, 6.0, 6.0], [3.0, 1.0, 7.0]], [[6.0, 8.0, 
6.0], [8.0, 1.0, 3.0], [1.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 2.0], [3.0, 4.0, 1.0], [1.0, 4.0, 6.0]], 
[[1.0, 1.0, 5.0], [3.0, 5.0, 8.0], [1.0, 6.0, 2.0]], [[5.0, 2.0, 4.0], [2.0, 2.0, 3.0], [1.0, 6.0, 7.0]]], [[[4.0, 5.0, 8.0], 
[5.0, 8.0, 5.0], [7.0, 7.0, 6.0]], [[8.0, 1.0, 1.0], [1.0, 7.0, 4.0], [7.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [4.0, 5.0, 4.0], 
[2.0, 6.0, 3.0]]], [[[8.0, 2.0, 2.0], [7.0, 8.0, 4.0], [2.0, 6.0, 8.0]], [[3.0, 3.0, 3.0], [7.0, 7.0, 2.0], [6.0, 5.0, 6.0]], 
[[3.0, 2.0, 8.0], [5.0, 6.0, 5.0], [4.0, 1.0, 2.0]]], [[[1.0, 2.0, 4.0], [5.0, 4.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 8.0, 6.0], 
[7.0, 1.0, 4.0], [8.0, 4.0, 3.0]], [[8.0, 8.0, 4.0], [5.0, 1.0, 4.0], [1.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CLUR(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 3.0], [8.0, 1.0, 4.0], [1.0, 7.0, 1.0]], [[5.0, 3.0, 2.0], [3.0, 6.0, 4.0], [5.0, 8.0, 
3.0]], [[5.0, 3.0, 5.0], [7.0, 3.0, 2.0], [6.0, 3.0, 5.0]]], [[[6.0, 7.0, 7.0], [8.0, 1.0, 3.0], [4.0, 6.0, 7.0]], [[1.0, 4.0, 
6.0], [3.0, 7.0, 6.0], [1.0, 2.0, 5.0]], [[6.0, 7.0, 2.0], [6.0, 8.0, 2.0], [6.0, 3.0, 1.0]]], [[[4.0, 5.0, 3.0], [6.0, 7.0, 
4.0], [6.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [5.0, 7.0, 1.0], [7.0, 1.0, 5.0]], [[5.0, 2.0, 6.0], [8.0, 1.0, 2.0], [7.0, 7.0, 
1.0]]], [[[7.0, 7.0, 7.0], [6.0, 3.0, 6.0], [6.0, 2.0, 7.0]], [[6.0, 2.0, 8.0], [2.0, 2.0, 8.0], [5.0, 7.0, 2.0]], [[3.0, 4.0, 
2.0], [8.0, 8.0, 1.0], [5.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 4.0], [5.0, 2.0, 3.0], [7.0, 5.0, 6.0]], [[1.0, 7.0, 8.0], [4.0, 2.0, 
1.0], [3.0, 6.0, 2.0]], [[2.0, 7.0, 7.0], [7.0, 3.0, 2.0], [5.0, 2.0, 8.0]]], [[[3.0, 6.0, 7.0], [1.0, 6.0, 5.0], [3.0, 3.0, 
2.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 7.0], [6.0, 7.0, 6.0]], [[4.0, 8.0, 6.0], [2.0, 7.0, 7.0], [6.0, 5.0, 5.0]]], [[[6.0, 4.0, 
2.0], [7.0, 1.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 5.0, 5.0], [7.0, 6.0, 3.0], [8.0, 2.0, 4.0]], [[8.0, 3.0, 1.0], [4.0, 6.0, 
7.0], [2.0, 5.0, 1.0]]], [[[7.0, 1.0, 3.0], [4.0, 3.0, 5.0], [5.0, 8.0, 5.0]], [[6.0, 7.0, 2.0], [3.0, 6.0, 7.0], [4.0, 5.0, 
3.0]], [[2.0, 1.0, 6.0], [8.0, 5.0, 5.0], [3.0, 8.0, 3.0]]]]))
      val=Data(numpy.array([[[[1.0], [2.0]]], [[[3.0], [1.0]]], [[[7.0], [2.0]]], [[[6.0], [7.0]]]]),self.functionspace)
      arg[:,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 3.0], [8.0, 1.0, 4.0], [1.0, 7.0, 1.0]], [[5.0, 3.0, 2.0], [3.0, 6.0, 4.0], [5.0, 
8.0, 3.0]], [[5.0, 1.0, 5.0], [7.0, 2.0, 2.0], [6.0, 3.0, 5.0]]], [[[6.0, 7.0, 7.0], [8.0, 1.0, 3.0], [4.0, 6.0, 7.0]], [[1.0, 
4.0, 6.0], [3.0, 7.0, 6.0], [1.0, 2.0, 5.0]], [[6.0, 3.0, 2.0], [6.0, 1.0, 2.0], [6.0, 3.0, 1.0]]], [[[4.0, 5.0, 3.0], [6.0, 
7.0, 4.0], [6.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [5.0, 7.0, 1.0], [7.0, 1.0, 5.0]], [[5.0, 7.0, 6.0], [8.0, 2.0, 2.0], [7.0, 7.0, 
1.0]]], [[[7.0, 7.0, 7.0], [6.0, 3.0, 6.0], [6.0, 2.0, 7.0]], [[6.0, 2.0, 8.0], [2.0, 2.0, 8.0], [5.0, 7.0, 2.0]], [[3.0, 6.0, 
2.0], [8.0, 7.0, 1.0], [5.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 4.0], [5.0, 2.0, 3.0], [7.0, 5.0, 6.0]], 
[[1.0, 7.0, 8.0], [4.0, 2.0, 1.0], [3.0, 6.0, 2.0]], [[2.0, 1.0, 7.0], [7.0, 2.0, 2.0], [5.0, 2.0, 8.0]]], [[[3.0, 6.0, 7.0], 
[1.0, 6.0, 5.0], [3.0, 3.0, 2.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 7.0], [6.0, 7.0, 6.0]], [[4.0, 3.0, 6.0], [2.0, 1.0, 7.0], 
[6.0, 5.0, 5.0]]], [[[6.0, 4.0, 2.0], [7.0, 1.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 5.0, 5.0], [7.0, 6.0, 3.0], [8.0, 2.0, 4.0]], 
[[8.0, 7.0, 1.0], [4.0, 2.0, 7.0], [2.0, 5.0, 1.0]]], [[[7.0, 1.0, 3.0], [4.0, 3.0, 5.0], [5.0, 8.0, 5.0]], [[6.0, 7.0, 2.0], 
[3.0, 6.0, 7.0], [4.0, 5.0, 3.0]], [[2.0, 6.0, 6.0], [8.0, 7.0, 5.0], [3.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CLUR(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 6.0], [3.0, 6.0, 7.0], [3.0, 1.0, 8.0]], [[6.0, 5.0, 2.0], [2.0, 2.0, 4.0], [5.0, 6.0, 
6.0]], [[3.0, 4.0, 7.0], [8.0, 5.0, 4.0], [8.0, 8.0, 5.0]]], [[[5.0, 8.0, 4.0], [7.0, 3.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 6.0, 
2.0], [1.0, 3.0, 2.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 7.0], [5.0, 1.0, 3.0], [7.0, 3.0, 2.0]]], [[[2.0, 5.0, 6.0], [7.0, 3.0, 
8.0], [2.0, 1.0, 7.0]], [[6.0, 3.0, 7.0], [1.0, 2.0, 5.0], [3.0, 8.0, 3.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 2.0], [4.0, 1.0, 
3.0]]], [[[8.0, 4.0, 4.0], [3.0, 4.0, 8.0], [5.0, 5.0, 6.0]], [[8.0, 3.0, 7.0], [4.0, 4.0, 1.0], [2.0, 7.0, 4.0]], [[2.0, 4.0, 
8.0], [2.0, 3.0, 7.0], [3.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 7.0], [6.0, 1.0, 1.0], [6.0, 3.0, 3.0]], [[4.0, 8.0, 5.0], [2.0, 2.0, 
8.0], [1.0, 3.0, 7.0]], [[8.0, 8.0, 6.0], [8.0, 7.0, 8.0], [8.0, 3.0, 4.0]]], [[[8.0, 3.0, 3.0], [5.0, 2.0, 7.0], [7.0, 1.0, 
1.0]], [[8.0, 5.0, 8.0], [4.0, 4.0, 5.0], [2.0, 8.0, 3.0]], [[4.0, 7.0, 8.0], [6.0, 5.0, 8.0], [1.0, 6.0, 4.0]]], [[[8.0, 6.0, 
7.0], [2.0, 1.0, 4.0], [6.0, 5.0, 8.0]], [[2.0, 4.0, 1.0], [6.0, 8.0, 2.0], [1.0, 2.0, 7.0]], [[8.0, 1.0, 1.0], [5.0, 8.0, 
5.0], [3.0, 4.0, 4.0]]], [[[6.0, 7.0, 5.0], [8.0, 6.0, 3.0], [7.0, 6.0, 2.0]], [[1.0, 1.0, 7.0], [4.0, 6.0, 7.0], [2.0, 8.0, 
4.0]], [[3.0, 7.0, 3.0], [3.0, 7.0, 6.0], [8.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[[[6.0], [4.0]]], [[[8.0], [3.0]]], [[[7.0], [8.0]]], [[[7.0], [7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0], [7.0]]], [[[2.0], [7.0]]], [[[4.0], [7.0]]], [[[6.0], [5.0]]]]))
      arg[:,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 6.0], [3.0, 6.0, 7.0], [3.0, 1.0, 8.0]], [[6.0, 5.0, 2.0], [2.0, 2.0, 4.0], [5.0, 
6.0, 6.0]], [[3.0, 6.0, 7.0], [8.0, 4.0, 4.0], [8.0, 8.0, 5.0]]], [[[5.0, 8.0, 4.0], [7.0, 3.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 
6.0, 2.0], [1.0, 3.0, 2.0], [5.0, 1.0, 1.0]], [[1.0, 8.0, 7.0], [5.0, 3.0, 3.0], [7.0, 3.0, 2.0]]], [[[2.0, 5.0, 6.0], [7.0, 
3.0, 8.0], [2.0, 1.0, 7.0]], [[6.0, 3.0, 7.0], [1.0, 2.0, 5.0], [3.0, 8.0, 3.0]], [[3.0, 7.0, 5.0], [7.0, 8.0, 2.0], [4.0, 1.0, 
3.0]]], [[[8.0, 4.0, 4.0], [3.0, 4.0, 8.0], [5.0, 5.0, 6.0]], [[8.0, 3.0, 7.0], [4.0, 4.0, 1.0], [2.0, 7.0, 4.0]], [[2.0, 7.0, 
8.0], [2.0, 7.0, 7.0], [3.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 7.0], [6.0, 1.0, 1.0], [6.0, 3.0, 3.0]], 
[[4.0, 8.0, 5.0], [2.0, 2.0, 8.0], [1.0, 3.0, 7.0]], [[8.0, 6.0, 6.0], [8.0, 7.0, 8.0], [8.0, 3.0, 4.0]]], [[[8.0, 3.0, 3.0], 
[5.0, 2.0, 7.0], [7.0, 1.0, 1.0]], [[8.0, 5.0, 8.0], [4.0, 4.0, 5.0], [2.0, 8.0, 3.0]], [[4.0, 2.0, 8.0], [6.0, 7.0, 8.0], 
[1.0, 6.0, 4.0]]], [[[8.0, 6.0, 7.0], [2.0, 1.0, 4.0], [6.0, 5.0, 8.0]], [[2.0, 4.0, 1.0], [6.0, 8.0, 2.0], [1.0, 2.0, 7.0]], 
[[8.0, 4.0, 1.0], [5.0, 7.0, 5.0], [3.0, 4.0, 4.0]]], [[[6.0, 7.0, 5.0], [8.0, 6.0, 3.0], [7.0, 6.0, 2.0]], [[1.0, 1.0, 7.0], 
[4.0, 6.0, 7.0], [2.0, 8.0, 4.0]], [[3.0, 6.0, 3.0], [3.0, 5.0, 6.0], [8.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CLUR(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [7.0, 6.0, 7.0], [6.0, 6.0, 5.0]], [[3.0, 8.0, 3.0], [4.0, 8.0, 1.0], [8.0, 1.0, 
1.0]], [[2.0, 5.0, 8.0], [2.0, 5.0, 7.0], [3.0, 1.0, 1.0]]], [[[7.0, 6.0, 2.0], [8.0, 5.0, 1.0], [7.0, 6.0, 1.0]], [[6.0, 6.0, 
4.0], [3.0, 4.0, 4.0], [4.0, 2.0, 6.0]], [[1.0, 1.0, 4.0], [7.0, 3.0, 7.0], [4.0, 3.0, 4.0]]], [[[6.0, 2.0, 8.0], [4.0, 7.0, 
4.0], [6.0, 6.0, 8.0]], [[2.0, 4.0, 4.0], [1.0, 3.0, 8.0], [7.0, 2.0, 2.0]], [[5.0, 2.0, 3.0], [4.0, 1.0, 7.0], [6.0, 1.0, 
1.0]]], [[[1.0, 3.0, 6.0], [5.0, 6.0, 7.0], [2.0, 2.0, 8.0]], [[8.0, 3.0, 6.0], [2.0, 8.0, 8.0], [6.0, 3.0, 4.0]], [[5.0, 1.0, 
1.0], [8.0, 8.0, 2.0], [3.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 3.0], [5.0, 4.0, 3.0], [4.0, 6.0, 6.0]], [[7.0, 5.0, 8.0], [8.0, 3.0, 
5.0], [6.0, 1.0, 5.0]], [[6.0, 2.0, 3.0], [1.0, 6.0, 3.0], [5.0, 8.0, 1.0]]], [[[6.0, 4.0, 6.0], [5.0, 3.0, 5.0], [3.0, 5.0, 
7.0]], [[6.0, 5.0, 3.0], [4.0, 3.0, 6.0], [2.0, 8.0, 1.0]], [[2.0, 3.0, 4.0], [2.0, 1.0, 8.0], [6.0, 5.0, 3.0]]], [[[4.0, 5.0, 
7.0], [5.0, 5.0, 6.0], [5.0, 5.0, 6.0]], [[6.0, 7.0, 4.0], [4.0, 7.0, 5.0], [1.0, 1.0, 3.0]], [[6.0, 6.0, 5.0], [2.0, 6.0, 
3.0], [6.0, 1.0, 4.0]]], [[[1.0, 2.0, 1.0], [8.0, 4.0, 8.0], [1.0, 3.0, 7.0]], [[7.0, 1.0, 5.0], [2.0, 4.0, 1.0], [5.0, 6.0, 
4.0]], [[3.0, 7.0, 3.0], [2.0, 6.0, 5.0], [6.0, 8.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0], [5.0]]], [[[7.0], [5.0]]], [[[3.0], [2.0]]], [[[5.0], 
[3.0]]]])+(1.-msk_val)*numpy.array([[[[4.0], [8.0]]], [[[8.0], [2.0]]], [[[8.0], [7.0]]], [[[5.0], [5.0]]]])
      arg[:,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 6.0], [7.0, 6.0, 7.0], [6.0, 6.0, 5.0]], [[3.0, 8.0, 3.0], [4.0, 8.0, 1.0], [8.0, 
1.0, 1.0]], [[2.0, 3.0, 8.0], [2.0, 5.0, 7.0], [3.0, 1.0, 1.0]]], [[[7.0, 6.0, 2.0], [8.0, 5.0, 1.0], [7.0, 6.0, 1.0]], [[6.0, 
6.0, 4.0], [3.0, 4.0, 4.0], [4.0, 2.0, 6.0]], [[1.0, 7.0, 4.0], [7.0, 5.0, 7.0], [4.0, 3.0, 4.0]]], [[[6.0, 2.0, 8.0], [4.0, 
7.0, 4.0], [6.0, 6.0, 8.0]], [[2.0, 4.0, 4.0], [1.0, 3.0, 8.0], [7.0, 2.0, 2.0]], [[5.0, 3.0, 3.0], [4.0, 2.0, 7.0], [6.0, 1.0, 
1.0]]], [[[1.0, 3.0, 6.0], [5.0, 6.0, 7.0], [2.0, 2.0, 8.0]], [[8.0, 3.0, 6.0], [2.0, 8.0, 8.0], [6.0, 3.0, 4.0]], [[5.0, 5.0, 
1.0], [8.0, 3.0, 2.0], [3.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 3.0], [5.0, 4.0, 3.0], [4.0, 6.0, 6.0]], 
[[7.0, 5.0, 8.0], [8.0, 3.0, 5.0], [6.0, 1.0, 5.0]], [[6.0, 4.0, 3.0], [1.0, 8.0, 3.0], [5.0, 8.0, 1.0]]], [[[6.0, 4.0, 6.0], 
[5.0, 3.0, 5.0], [3.0, 5.0, 7.0]], [[6.0, 5.0, 3.0], [4.0, 3.0, 6.0], [2.0, 8.0, 1.0]], [[2.0, 8.0, 4.0], [2.0, 2.0, 8.0], 
[6.0, 5.0, 3.0]]], [[[4.0, 5.0, 7.0], [5.0, 5.0, 6.0], [5.0, 5.0, 6.0]], [[6.0, 7.0, 4.0], [4.0, 7.0, 5.0], [1.0, 1.0, 3.0]], 
[[6.0, 8.0, 5.0], [2.0, 7.0, 3.0], [6.0, 1.0, 4.0]]], [[[1.0, 2.0, 1.0], [8.0, 4.0, 8.0], [1.0, 3.0, 7.0]], [[7.0, 1.0, 5.0], 
[2.0, 4.0, 1.0], [5.0, 6.0, 4.0]], [[3.0, 5.0, 3.0], [2.0, 5.0, 5.0], [6.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LURN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 4.0], [6.0, 1.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 4.0, 7.0], [1.0, 7.0, 8.0], [8.0, 7.0, 
7.0]], [[4.0, 8.0, 5.0], [8.0, 5.0, 5.0], [4.0, 7.0, 5.0]]], [[[4.0, 7.0, 7.0], [2.0, 2.0, 4.0], [5.0, 4.0, 5.0]], [[2.0, 1.0, 
1.0], [3.0, 5.0, 8.0], [7.0, 5.0, 2.0]], [[3.0, 3.0, 6.0], [3.0, 8.0, 6.0], [3.0, 7.0, 5.0]]], [[[3.0, 4.0, 6.0], [2.0, 1.0, 
5.0], [2.0, 4.0, 7.0]], [[3.0, 6.0, 6.0], [5.0, 5.0, 5.0], [8.0, 1.0, 3.0]], [[2.0, 2.0, 5.0], [6.0, 2.0, 8.0], [4.0, 1.0, 
5.0]]], [[[7.0, 1.0, 4.0], [5.0, 3.0, 7.0], [1.0, 5.0, 8.0]], [[2.0, 7.0, 2.0], [2.0, 2.0, 8.0], [3.0, 8.0, 1.0]], [[1.0, 6.0, 
7.0], [2.0, 3.0, 5.0], [2.0, 5.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 3.0], [7.0, 5.0, 7.0], [4.0, 2.0, 3.0]], [[7.0, 2.0, 1.0], [3.0, 6.0, 
6.0], [8.0, 8.0, 2.0]], [[7.0, 3.0, 4.0], [4.0, 5.0, 7.0], [4.0, 6.0, 5.0]]], [[[2.0, 3.0, 1.0], [2.0, 8.0, 3.0], [5.0, 5.0, 
8.0]], [[7.0, 8.0, 3.0], [4.0, 2.0, 5.0], [2.0, 7.0, 3.0]], [[7.0, 7.0, 3.0], [6.0, 1.0, 5.0], [7.0, 3.0, 3.0]]], [[[7.0, 6.0, 
4.0], [4.0, 2.0, 6.0], [1.0, 7.0, 3.0]], [[5.0, 3.0, 8.0], [3.0, 1.0, 5.0], [2.0, 5.0, 6.0]], [[5.0, 3.0, 7.0], [2.0, 2.0, 
1.0], [8.0, 6.0, 4.0]]], [[[2.0, 3.0, 8.0], [2.0, 6.0, 5.0], [5.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], [3.0, 6.0, 7.0], [2.0, 5.0, 
2.0]], [[5.0, 4.0, 5.0], [7.0, 5.0, 2.0], [5.0, 3.0, 4.0]]]]))
      val=numpy.array([[[[5.0, 8.0, 6.0]], [[7.0, 6.0, 1.0]]], [[[6.0, 7.0, 3.0]], [[5.0, 7.0, 3.0]]]])
      arg[2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 4.0], [6.0, 1.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 4.0, 7.0], [1.0, 7.0, 8.0], [8.0, 
7.0, 7.0]], [[4.0, 8.0, 5.0], [8.0, 5.0, 5.0], [4.0, 7.0, 5.0]]], [[[4.0, 7.0, 7.0], [2.0, 2.0, 4.0], [5.0, 4.0, 5.0]], [[2.0, 
1.0, 1.0], [3.0, 5.0, 8.0], [7.0, 5.0, 2.0]], [[3.0, 3.0, 6.0], [3.0, 8.0, 6.0], [3.0, 7.0, 5.0]]], [[[3.0, 4.0, 6.0], [5.0, 
8.0, 6.0], [2.0, 4.0, 7.0]], [[3.0, 6.0, 6.0], [7.0, 6.0, 1.0], [8.0, 1.0, 3.0]], [[2.0, 2.0, 5.0], [6.0, 2.0, 8.0], [4.0, 1.0, 
5.0]]], [[[7.0, 1.0, 4.0], [6.0, 7.0, 3.0], [1.0, 5.0, 8.0]], [[2.0, 7.0, 2.0], [5.0, 7.0, 3.0], [3.0, 8.0, 1.0]], [[1.0, 6.0, 
7.0], [2.0, 3.0, 5.0], [2.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 3.0], [7.0, 5.0, 7.0], [4.0, 2.0, 3.0]], 
[[7.0, 2.0, 1.0], [3.0, 6.0, 6.0], [8.0, 8.0, 2.0]], [[7.0, 3.0, 4.0], [4.0, 5.0, 7.0], [4.0, 6.0, 5.0]]], [[[2.0, 3.0, 1.0], 
[2.0, 8.0, 3.0], [5.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [4.0, 2.0, 5.0], [2.0, 7.0, 3.0]], [[7.0, 7.0, 3.0], [6.0, 1.0, 5.0], 
[7.0, 3.0, 3.0]]], [[[7.0, 6.0, 4.0], [5.0, 8.0, 6.0], [1.0, 7.0, 3.0]], [[5.0, 3.0, 8.0], [7.0, 6.0, 1.0], [2.0, 5.0, 6.0]], 
[[5.0, 3.0, 7.0], [2.0, 2.0, 1.0], [8.0, 6.0, 4.0]]], [[[2.0, 3.0, 8.0], [6.0, 7.0, 3.0], [5.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], 
[5.0, 7.0, 3.0], [2.0, 5.0, 2.0]], [[5.0, 4.0, 5.0], [7.0, 5.0, 2.0], [5.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LURN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 5.0], [3.0, 2.0, 2.0], [5.0, 3.0, 4.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 6.0], [1.0, 2.0, 
7.0]], [[4.0, 8.0, 2.0], [5.0, 1.0, 2.0], [6.0, 8.0, 8.0]]], [[[4.0, 3.0, 3.0], [7.0, 4.0, 8.0], [2.0, 4.0, 2.0]], [[5.0, 5.0, 
7.0], [4.0, 5.0, 3.0], [6.0, 6.0, 8.0]], [[7.0, 1.0, 7.0], [5.0, 7.0, 5.0], [4.0, 8.0, 4.0]]], [[[7.0, 6.0, 7.0], [6.0, 7.0, 
6.0], [1.0, 4.0, 8.0]], [[3.0, 8.0, 6.0], [7.0, 5.0, 5.0], [8.0, 1.0, 5.0]], [[6.0, 7.0, 2.0], [5.0, 2.0, 1.0], [7.0, 3.0, 
4.0]]], [[[5.0, 4.0, 6.0], [7.0, 3.0, 6.0], [2.0, 8.0, 7.0]], [[8.0, 2.0, 7.0], [7.0, 5.0, 5.0], [5.0, 4.0, 3.0]], [[3.0, 8.0, 
2.0], [4.0, 4.0, 7.0], [4.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 5.0], [5.0, 4.0, 3.0], [1.0, 3.0, 4.0]], [[3.0, 6.0, 8.0], [6.0, 8.0, 
7.0], [4.0, 2.0, 7.0]], [[7.0, 8.0, 2.0], [2.0, 4.0, 5.0], [6.0, 6.0, 4.0]]], [[[2.0, 7.0, 2.0], [2.0, 6.0, 4.0], [6.0, 7.0, 
8.0]], [[6.0, 2.0, 2.0], [8.0, 6.0, 7.0], [3.0, 7.0, 2.0]], [[5.0, 5.0, 5.0], [1.0, 3.0, 3.0], [4.0, 3.0, 1.0]]], [[[4.0, 6.0, 
4.0], [8.0, 8.0, 7.0], [1.0, 1.0, 8.0]], [[3.0, 1.0, 7.0], [4.0, 7.0, 3.0], [2.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [8.0, 5.0, 
4.0], [2.0, 1.0, 2.0]]], [[[3.0, 1.0, 7.0], [5.0, 1.0, 5.0], [8.0, 4.0, 8.0]], [[6.0, 4.0, 8.0], [6.0, 4.0, 3.0], [4.0, 8.0, 
3.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 2.0], [2.0, 8.0, 8.0]]]]))
      val=Data(numpy.array([[[[5.0, 6.0, 8.0]], [[7.0, 2.0, 2.0]]], [[[5.0, 8.0, 2.0]], [[3.0, 7.0, 2.0]]]]),self.functionspace)
      arg[2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 5.0], [3.0, 2.0, 2.0], [5.0, 3.0, 4.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 6.0], [1.0, 
2.0, 7.0]], [[4.0, 8.0, 2.0], [5.0, 1.0, 2.0], [6.0, 8.0, 8.0]]], [[[4.0, 3.0, 3.0], [7.0, 4.0, 8.0], [2.0, 4.0, 2.0]], [[5.0, 
5.0, 7.0], [4.0, 5.0, 3.0], [6.0, 6.0, 8.0]], [[7.0, 1.0, 7.0], [5.0, 7.0, 5.0], [4.0, 8.0, 4.0]]], [[[7.0, 6.0, 7.0], [5.0, 
6.0, 8.0], [1.0, 4.0, 8.0]], [[3.0, 8.0, 6.0], [7.0, 2.0, 2.0], [8.0, 1.0, 5.0]], [[6.0, 7.0, 2.0], [5.0, 2.0, 1.0], [7.0, 3.0, 
4.0]]], [[[5.0, 4.0, 6.0], [5.0, 8.0, 2.0], [2.0, 8.0, 7.0]], [[8.0, 2.0, 7.0], [3.0, 7.0, 2.0], [5.0, 4.0, 3.0]], [[3.0, 8.0, 
2.0], [4.0, 4.0, 7.0], [4.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 5.0], [5.0, 4.0, 3.0], [1.0, 3.0, 4.0]], 
[[3.0, 6.0, 8.0], [6.0, 8.0, 7.0], [4.0, 2.0, 7.0]], [[7.0, 8.0, 2.0], [2.0, 4.0, 5.0], [6.0, 6.0, 4.0]]], [[[2.0, 7.0, 2.0], 
[2.0, 6.0, 4.0], [6.0, 7.0, 8.0]], [[6.0, 2.0, 2.0], [8.0, 6.0, 7.0], [3.0, 7.0, 2.0]], [[5.0, 5.0, 5.0], [1.0, 3.0, 3.0], 
[4.0, 3.0, 1.0]]], [[[4.0, 6.0, 4.0], [5.0, 6.0, 8.0], [1.0, 1.0, 8.0]], [[3.0, 1.0, 7.0], [7.0, 2.0, 2.0], [2.0, 2.0, 3.0]], 
[[2.0, 1.0, 4.0], [8.0, 5.0, 4.0], [2.0, 1.0, 2.0]]], [[[3.0, 1.0, 7.0], [5.0, 8.0, 2.0], [8.0, 4.0, 8.0]], [[6.0, 4.0, 8.0], 
[3.0, 7.0, 2.0], [4.0, 8.0, 3.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 2.0], [2.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LURN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 8.0], [1.0, 2.0, 8.0], [2.0, 8.0, 4.0]], [[4.0, 2.0, 8.0], [4.0, 1.0, 5.0], [8.0, 6.0, 
5.0]], [[3.0, 7.0, 5.0], [6.0, 7.0, 4.0], [6.0, 2.0, 5.0]]], [[[8.0, 3.0, 6.0], [6.0, 8.0, 6.0], [5.0, 3.0, 5.0]], [[4.0, 6.0, 
5.0], [5.0, 1.0, 4.0], [5.0, 4.0, 7.0]], [[7.0, 6.0, 2.0], [8.0, 4.0, 1.0], [2.0, 1.0, 3.0]]], [[[3.0, 6.0, 1.0], [2.0, 4.0, 
1.0], [8.0, 3.0, 4.0]], [[4.0, 4.0, 6.0], [1.0, 4.0, 3.0], [4.0, 4.0, 1.0]], [[8.0, 4.0, 2.0], [7.0, 8.0, 7.0], [3.0, 7.0, 
6.0]]], [[[5.0, 7.0, 3.0], [7.0, 7.0, 2.0], [4.0, 7.0, 6.0]], [[5.0, 5.0, 2.0], [8.0, 6.0, 3.0], [8.0, 8.0, 4.0]], [[8.0, 1.0, 
7.0], [1.0, 4.0, 5.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 8.0], [6.0, 1.0, 2.0], [3.0, 5.0, 2.0]], [[3.0, 8.0, 3.0], [6.0, 1.0, 
6.0], [1.0, 7.0, 6.0]], [[3.0, 6.0, 2.0], [6.0, 7.0, 3.0], [1.0, 7.0, 7.0]]], [[[6.0, 4.0, 4.0], [5.0, 7.0, 1.0], [2.0, 2.0, 
7.0]], [[2.0, 1.0, 8.0], [8.0, 1.0, 8.0], [4.0, 8.0, 1.0]], [[3.0, 4.0, 7.0], [4.0, 6.0, 3.0], [6.0, 6.0, 4.0]]], [[[6.0, 7.0, 
4.0], [8.0, 2.0, 2.0], [6.0, 8.0, 4.0]], [[4.0, 6.0, 7.0], [7.0, 1.0, 2.0], [4.0, 2.0, 8.0]], [[8.0, 1.0, 5.0], [3.0, 7.0, 
7.0], [7.0, 6.0, 6.0]]], [[[3.0, 3.0, 8.0], [1.0, 6.0, 7.0], [8.0, 7.0, 6.0]], [[8.0, 4.0, 8.0], [3.0, 1.0, 5.0], [3.0, 2.0, 
8.0]], [[1.0, 2.0, 6.0], [6.0, 5.0, 3.0], [1.0, 3.0, 4.0]]]]))
      val=Data(numpy.array([[[[2.0, 5.0, 5.0]], [[4.0, 6.0, 7.0]]], [[[2.0, 3.0, 6.0]], [[4.0, 7.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 8.0]], [[7.0, 8.0, 8.0]]], [[[8.0, 8.0, 1.0]], [[4.0, 5.0, 1.0]]]]))
      arg[2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 8.0], [1.0, 2.0, 8.0], [2.0, 8.0, 4.0]], [[4.0, 2.0, 8.0], [4.0, 1.0, 5.0], [8.0, 
6.0, 5.0]], [[3.0, 7.0, 5.0], [6.0, 7.0, 4.0], [6.0, 2.0, 5.0]]], [[[8.0, 3.0, 6.0], [6.0, 8.0, 6.0], [5.0, 3.0, 5.0]], [[4.0, 
6.0, 5.0], [5.0, 1.0, 4.0], [5.0, 4.0, 7.0]], [[7.0, 6.0, 2.0], [8.0, 4.0, 1.0], [2.0, 1.0, 3.0]]], [[[3.0, 6.0, 1.0], [2.0, 
5.0, 5.0], [8.0, 3.0, 4.0]], [[4.0, 4.0, 6.0], [4.0, 6.0, 7.0], [4.0, 4.0, 1.0]], [[8.0, 4.0, 2.0], [7.0, 8.0, 7.0], [3.0, 7.0, 
6.0]]], [[[5.0, 7.0, 3.0], [2.0, 3.0, 6.0], [4.0, 7.0, 6.0]], [[5.0, 5.0, 2.0], [4.0, 7.0, 5.0], [8.0, 8.0, 4.0]], [[8.0, 1.0, 
7.0], [1.0, 4.0, 5.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 8.0], [6.0, 1.0, 2.0], [3.0, 5.0, 2.0]], 
[[3.0, 8.0, 3.0], [6.0, 1.0, 6.0], [1.0, 7.0, 6.0]], [[3.0, 6.0, 2.0], [6.0, 7.0, 3.0], [1.0, 7.0, 7.0]]], [[[6.0, 4.0, 4.0], 
[5.0, 7.0, 1.0], [2.0, 2.0, 7.0]], [[2.0, 1.0, 8.0], [8.0, 1.0, 8.0], [4.0, 8.0, 1.0]], [[3.0, 4.0, 7.0], [4.0, 6.0, 3.0], 
[6.0, 6.0, 4.0]]], [[[6.0, 7.0, 4.0], [5.0, 2.0, 8.0], [6.0, 8.0, 4.0]], [[4.0, 6.0, 7.0], [7.0, 8.0, 8.0], [4.0, 2.0, 8.0]], 
[[8.0, 1.0, 5.0], [3.0, 7.0, 7.0], [7.0, 6.0, 6.0]]], [[[3.0, 3.0, 8.0], [8.0, 8.0, 1.0], [8.0, 7.0, 6.0]], [[8.0, 4.0, 8.0], 
[4.0, 5.0, 1.0], [3.0, 2.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 5.0, 3.0], [1.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LURN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 4.0], [8.0, 8.0, 7.0], [1.0, 7.0, 8.0]], [[1.0, 8.0, 5.0], [8.0, 2.0, 5.0], [7.0, 8.0, 
5.0]], [[7.0, 7.0, 8.0], [2.0, 1.0, 8.0], [5.0, 3.0, 1.0]]], [[[8.0, 5.0, 4.0], [1.0, 3.0, 7.0], [1.0, 2.0, 3.0]], [[8.0, 6.0, 
4.0], [2.0, 1.0, 1.0], [3.0, 8.0, 6.0]], [[7.0, 7.0, 2.0], [4.0, 5.0, 6.0], [4.0, 5.0, 6.0]]], [[[1.0, 6.0, 2.0], [8.0, 1.0, 
3.0], [8.0, 8.0, 2.0]], [[1.0, 5.0, 1.0], [5.0, 5.0, 8.0], [8.0, 6.0, 5.0]], [[2.0, 4.0, 3.0], [5.0, 8.0, 1.0], [3.0, 6.0, 
8.0]]], [[[2.0, 2.0, 5.0], [3.0, 3.0, 1.0], [7.0, 2.0, 1.0]], [[1.0, 8.0, 8.0], [4.0, 1.0, 1.0], [1.0, 8.0, 5.0]], [[8.0, 8.0, 
3.0], [2.0, 2.0, 6.0], [2.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 3.0], [4.0, 6.0, 7.0], [3.0, 8.0, 3.0]], [[2.0, 8.0, 5.0], [7.0, 6.0, 
6.0], [6.0, 1.0, 5.0]], [[2.0, 1.0, 4.0], [6.0, 8.0, 2.0], [5.0, 7.0, 4.0]]], [[[2.0, 4.0, 3.0], [2.0, 5.0, 1.0], [6.0, 3.0, 
1.0]], [[5.0, 2.0, 1.0], [1.0, 3.0, 6.0], [4.0, 6.0, 3.0]], [[7.0, 1.0, 2.0], [6.0, 6.0, 5.0], [1.0, 3.0, 5.0]]], [[[7.0, 4.0, 
8.0], [4.0, 1.0, 7.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 1.0], [5.0, 2.0, 3.0], [2.0, 2.0, 5.0]], [[3.0, 4.0, 8.0], [6.0, 8.0, 
7.0], [7.0, 2.0, 3.0]]], [[[1.0, 4.0, 6.0], [6.0, 2.0, 8.0], [2.0, 4.0, 2.0]], [[3.0, 5.0, 8.0], [6.0, 8.0, 3.0], [6.0, 3.0, 
5.0]], [[6.0, 1.0, 6.0], [8.0, 3.0, 1.0], [3.0, 1.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 7.0, 3.0]], [[5.0, 6.0, 3.0]]], [[[5.0, 3.0, 7.0]], [[3.0, 8.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 3.0, 6.0]], [[6.0, 2.0, 6.0]]], [[[4.0, 5.0, 8.0]], [[7.0, 8.0, 6.0]]]])
      arg[2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 4.0], [8.0, 8.0, 7.0], [1.0, 7.0, 8.0]], [[1.0, 8.0, 5.0], [8.0, 2.0, 5.0], [7.0, 
8.0, 5.0]], [[7.0, 7.0, 8.0], [2.0, 1.0, 8.0], [5.0, 3.0, 1.0]]], [[[8.0, 5.0, 4.0], [1.0, 3.0, 7.0], [1.0, 2.0, 3.0]], [[8.0, 
6.0, 4.0], [2.0, 1.0, 1.0], [3.0, 8.0, 6.0]], [[7.0, 7.0, 2.0], [4.0, 5.0, 6.0], [4.0, 5.0, 6.0]]], [[[1.0, 6.0, 2.0], [2.0, 
7.0, 3.0], [8.0, 8.0, 2.0]], [[1.0, 5.0, 1.0], [5.0, 6.0, 3.0], [8.0, 6.0, 5.0]], [[2.0, 4.0, 3.0], [5.0, 8.0, 1.0], [3.0, 6.0, 
8.0]]], [[[2.0, 2.0, 5.0], [5.0, 3.0, 7.0], [7.0, 2.0, 1.0]], [[1.0, 8.0, 8.0], [3.0, 8.0, 5.0], [1.0, 8.0, 5.0]], [[8.0, 8.0, 
3.0], [2.0, 2.0, 6.0], [2.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 3.0], [4.0, 6.0, 7.0], [3.0, 8.0, 3.0]], 
[[2.0, 8.0, 5.0], [7.0, 6.0, 6.0], [6.0, 1.0, 5.0]], [[2.0, 1.0, 4.0], [6.0, 8.0, 2.0], [5.0, 7.0, 4.0]]], [[[2.0, 4.0, 3.0], 
[2.0, 5.0, 1.0], [6.0, 3.0, 1.0]], [[5.0, 2.0, 1.0], [1.0, 3.0, 6.0], [4.0, 6.0, 3.0]], [[7.0, 1.0, 2.0], [6.0, 6.0, 5.0], 
[1.0, 3.0, 5.0]]], [[[7.0, 4.0, 8.0], [5.0, 3.0, 6.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 1.0], [6.0, 2.0, 6.0], [2.0, 2.0, 5.0]], 
[[3.0, 4.0, 8.0], [6.0, 8.0, 7.0], [7.0, 2.0, 3.0]]], [[[1.0, 4.0, 6.0], [4.0, 5.0, 8.0], [2.0, 4.0, 2.0]], [[3.0, 5.0, 8.0], 
[7.0, 8.0, 6.0], [6.0, 3.0, 5.0]], [[6.0, 1.0, 6.0], [8.0, 3.0, 1.0], [3.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RCUN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 7.0], [3.0, 5.0, 2.0], [4.0, 6.0, 1.0]], [[4.0, 4.0, 6.0], [3.0, 8.0, 3.0], [4.0, 6.0, 
1.0]], [[8.0, 1.0, 7.0], [3.0, 5.0, 5.0], [6.0, 4.0, 5.0]]], [[[8.0, 1.0, 1.0], [7.0, 3.0, 1.0], [4.0, 1.0, 7.0]], [[3.0, 3.0, 
1.0], [1.0, 2.0, 7.0], [1.0, 4.0, 8.0]], [[1.0, 4.0, 2.0], [4.0, 6.0, 5.0], [4.0, 8.0, 1.0]]], [[[6.0, 6.0, 6.0], [1.0, 5.0, 
6.0], [5.0, 6.0, 6.0]], [[4.0, 3.0, 4.0], [2.0, 8.0, 5.0], [3.0, 4.0, 6.0]], [[5.0, 6.0, 5.0], [2.0, 4.0, 1.0], [2.0, 5.0, 
8.0]]], [[[1.0, 2.0, 3.0], [6.0, 5.0, 1.0], [1.0, 5.0, 4.0]], [[3.0, 3.0, 3.0], [5.0, 2.0, 1.0], [7.0, 7.0, 6.0]], [[8.0, 2.0, 
5.0], [1.0, 5.0, 1.0], [1.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 7.0], [5.0, 8.0, 7.0], [8.0, 7.0, 1.0]], [[2.0, 5.0, 7.0], [4.0, 7.0, 
7.0], [7.0, 4.0, 6.0]], [[3.0, 8.0, 4.0], [3.0, 2.0, 6.0], [5.0, 7.0, 2.0]]], [[[4.0, 5.0, 2.0], [1.0, 1.0, 5.0], [3.0, 6.0, 
2.0]], [[5.0, 8.0, 4.0], [7.0, 7.0, 6.0], [1.0, 6.0, 4.0]], [[5.0, 3.0, 7.0], [5.0, 7.0, 5.0], [8.0, 7.0, 2.0]]], [[[1.0, 3.0, 
7.0], [4.0, 4.0, 8.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 6.0], [6.0, 3.0, 5.0], [7.0, 6.0, 1.0]], [[8.0, 3.0, 4.0], [2.0, 7.0, 
4.0], [7.0, 3.0, 2.0]]], [[[4.0, 4.0, 1.0], [4.0, 6.0, 8.0], [4.0, 2.0, 5.0]], [[8.0, 8.0, 6.0], [4.0, 5.0, 5.0], [6.0, 5.0, 
6.0]], [[5.0, 2.0, 3.0], [8.0, 3.0, 3.0], [5.0, 1.0, 7.0]]]]))
      val=numpy.array([[[[7.0, 5.0, 3.0], [3.0, 1.0, 1.0]], [[6.0, 1.0, 3.0], [3.0, 7.0, 5.0]], [[1.0, 4.0, 3.0], [8.0, 4.0, 
1.0]]], [[[7.0, 6.0, 2.0], [8.0, 3.0, 5.0]], [[8.0, 1.0, 1.0], [2.0, 7.0, 1.0]], [[1.0, 4.0, 4.0], [7.0, 6.0, 3.0]]]])
      arg[1:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 7.0], [3.0, 5.0, 2.0], [4.0, 6.0, 1.0]], [[4.0, 4.0, 6.0], [3.0, 8.0, 3.0], [4.0, 
6.0, 1.0]], [[8.0, 1.0, 7.0], [3.0, 5.0, 5.0], [6.0, 4.0, 5.0]]], [[[7.0, 5.0, 3.0], [3.0, 1.0, 1.0], [4.0, 1.0, 7.0]], [[6.0, 
1.0, 3.0], [3.0, 7.0, 5.0], [1.0, 4.0, 8.0]], [[1.0, 4.0, 3.0], [8.0, 4.0, 1.0], [4.0, 8.0, 1.0]]], [[[7.0, 6.0, 2.0], [8.0, 
3.0, 5.0], [5.0, 6.0, 6.0]], [[8.0, 1.0, 1.0], [2.0, 7.0, 1.0], [3.0, 4.0, 6.0]], [[1.0, 4.0, 4.0], [7.0, 6.0, 3.0], [2.0, 5.0, 
8.0]]], [[[1.0, 2.0, 3.0], [6.0, 5.0, 1.0], [1.0, 5.0, 4.0]], [[3.0, 3.0, 3.0], [5.0, 2.0, 1.0], [7.0, 7.0, 6.0]], [[8.0, 2.0, 
5.0], [1.0, 5.0, 1.0], [1.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 7.0], [5.0, 8.0, 7.0], [8.0, 7.0, 1.0]], 
[[2.0, 5.0, 7.0], [4.0, 7.0, 7.0], [7.0, 4.0, 6.0]], [[3.0, 8.0, 4.0], [3.0, 2.0, 6.0], [5.0, 7.0, 2.0]]], [[[7.0, 5.0, 3.0], 
[3.0, 1.0, 1.0], [3.0, 6.0, 2.0]], [[6.0, 1.0, 3.0], [3.0, 7.0, 5.0], [1.0, 6.0, 4.0]], [[1.0, 4.0, 3.0], [8.0, 4.0, 1.0], 
[8.0, 7.0, 2.0]]], [[[7.0, 6.0, 2.0], [8.0, 3.0, 5.0], [5.0, 1.0, 1.0]], [[8.0, 1.0, 1.0], [2.0, 7.0, 1.0], [7.0, 6.0, 1.0]], 
[[1.0, 4.0, 4.0], [7.0, 6.0, 3.0], [7.0, 3.0, 2.0]]], [[[4.0, 4.0, 1.0], [4.0, 6.0, 8.0], [4.0, 2.0, 5.0]], [[8.0, 8.0, 6.0], 
[4.0, 5.0, 5.0], [6.0, 5.0, 6.0]], [[5.0, 2.0, 3.0], [8.0, 3.0, 3.0], [5.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RCUN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 6.0], [7.0, 4.0, 4.0], [5.0, 5.0, 5.0]], [[7.0, 4.0, 3.0], [5.0, 2.0, 4.0], [5.0, 1.0, 
8.0]], [[3.0, 8.0, 8.0], [5.0, 8.0, 8.0], [5.0, 7.0, 1.0]]], [[[7.0, 6.0, 8.0], [7.0, 8.0, 5.0], [7.0, 1.0, 3.0]], [[7.0, 7.0, 
5.0], [7.0, 1.0, 2.0], [1.0, 5.0, 5.0]], [[1.0, 2.0, 3.0], [3.0, 5.0, 1.0], [7.0, 6.0, 6.0]]], [[[3.0, 3.0, 1.0], [4.0, 1.0, 
3.0], [4.0, 1.0, 4.0]], [[2.0, 7.0, 6.0], [8.0, 1.0, 8.0], [5.0, 2.0, 8.0]], [[6.0, 7.0, 2.0], [1.0, 3.0, 3.0], [6.0, 2.0, 
7.0]]], [[[4.0, 2.0, 8.0], [7.0, 4.0, 2.0], [1.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [7.0, 7.0, 4.0], [5.0, 7.0, 1.0]], [[5.0, 5.0, 
3.0], [1.0, 3.0, 5.0], [2.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 3.0], [5.0, 3.0, 2.0], [8.0, 5.0, 4.0]], [[7.0, 3.0, 7.0], [5.0, 2.0, 
8.0], [8.0, 6.0, 2.0]], [[5.0, 6.0, 5.0], [4.0, 5.0, 5.0], [2.0, 1.0, 7.0]]], [[[2.0, 6.0, 5.0], [6.0, 3.0, 7.0], [8.0, 8.0, 
3.0]], [[8.0, 8.0, 3.0], [2.0, 4.0, 7.0], [4.0, 7.0, 7.0]], [[7.0, 5.0, 8.0], [5.0, 4.0, 5.0], [4.0, 7.0, 7.0]]], [[[5.0, 7.0, 
2.0], [6.0, 7.0, 6.0], [1.0, 8.0, 2.0]], [[5.0, 2.0, 2.0], [3.0, 3.0, 1.0], [1.0, 1.0, 3.0]], [[2.0, 7.0, 3.0], [4.0, 1.0, 
3.0], [8.0, 6.0, 2.0]]], [[[7.0, 7.0, 8.0], [1.0, 8.0, 8.0], [8.0, 6.0, 6.0]], [[2.0, 3.0, 3.0], [6.0, 6.0, 3.0], [8.0, 8.0, 
2.0]], [[5.0, 8.0, 2.0], [8.0, 1.0, 5.0], [4.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[[[4.0, 4.0, 5.0], [4.0, 2.0, 6.0]], [[1.0, 8.0, 6.0], [4.0, 8.0, 5.0]], [[5.0, 2.0, 2.0], [2.0, 
2.0, 7.0]]], [[[2.0, 6.0, 7.0], [6.0, 7.0, 8.0]], [[4.0, 5.0, 8.0], [6.0, 3.0, 8.0]], [[2.0, 3.0, 4.0], [3.0, 8.0, 
6.0]]]]),self.functionspace)
      arg[1:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 6.0], [7.0, 4.0, 4.0], [5.0, 5.0, 5.0]], [[7.0, 4.0, 3.0], [5.0, 2.0, 4.0], [5.0, 
1.0, 8.0]], [[3.0, 8.0, 8.0], [5.0, 8.0, 8.0], [5.0, 7.0, 1.0]]], [[[4.0, 4.0, 5.0], [4.0, 2.0, 6.0], [7.0, 1.0, 3.0]], [[1.0, 
8.0, 6.0], [4.0, 8.0, 5.0], [1.0, 5.0, 5.0]], [[5.0, 2.0, 2.0], [2.0, 2.0, 7.0], [7.0, 6.0, 6.0]]], [[[2.0, 6.0, 7.0], [6.0, 
7.0, 8.0], [4.0, 1.0, 4.0]], [[4.0, 5.0, 8.0], [6.0, 3.0, 8.0], [5.0, 2.0, 8.0]], [[2.0, 3.0, 4.0], [3.0, 8.0, 6.0], [6.0, 2.0, 
7.0]]], [[[4.0, 2.0, 8.0], [7.0, 4.0, 2.0], [1.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [7.0, 7.0, 4.0], [5.0, 7.0, 1.0]], [[5.0, 5.0, 
3.0], [1.0, 3.0, 5.0], [2.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 3.0], [5.0, 3.0, 2.0], [8.0, 5.0, 4.0]], 
[[7.0, 3.0, 7.0], [5.0, 2.0, 8.0], [8.0, 6.0, 2.0]], [[5.0, 6.0, 5.0], [4.0, 5.0, 5.0], [2.0, 1.0, 7.0]]], [[[4.0, 4.0, 5.0], 
[4.0, 2.0, 6.0], [8.0, 8.0, 3.0]], [[1.0, 8.0, 6.0], [4.0, 8.0, 5.0], [4.0, 7.0, 7.0]], [[5.0, 2.0, 2.0], [2.0, 2.0, 7.0], 
[4.0, 7.0, 7.0]]], [[[2.0, 6.0, 7.0], [6.0, 7.0, 8.0], [1.0, 8.0, 2.0]], [[4.0, 5.0, 8.0], [6.0, 3.0, 8.0], [1.0, 1.0, 3.0]], 
[[2.0, 3.0, 4.0], [3.0, 8.0, 6.0], [8.0, 6.0, 2.0]]], [[[7.0, 7.0, 8.0], [1.0, 8.0, 8.0], [8.0, 6.0, 6.0]], [[2.0, 3.0, 3.0], 
[6.0, 6.0, 3.0], [8.0, 8.0, 2.0]], [[5.0, 8.0, 2.0], [8.0, 1.0, 5.0], [4.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RCUN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 7.0], [7.0, 2.0, 6.0], [1.0, 3.0, 1.0]], [[2.0, 8.0, 7.0], [1.0, 8.0, 1.0], [4.0, 1.0, 
5.0]], [[8.0, 6.0, 4.0], [2.0, 5.0, 8.0], [6.0, 1.0, 5.0]]], [[[8.0, 4.0, 5.0], [8.0, 1.0, 4.0], [8.0, 7.0, 4.0]], [[1.0, 7.0, 
5.0], [1.0, 6.0, 6.0], [2.0, 5.0, 1.0]], [[2.0, 8.0, 8.0], [2.0, 8.0, 6.0], [3.0, 5.0, 2.0]]], [[[6.0, 4.0, 1.0], [7.0, 8.0, 
1.0], [4.0, 4.0, 6.0]], [[3.0, 5.0, 7.0], [3.0, 4.0, 2.0], [8.0, 2.0, 8.0]], [[4.0, 1.0, 3.0], [5.0, 1.0, 2.0], [2.0, 3.0, 
4.0]]], [[[7.0, 5.0, 4.0], [7.0, 3.0, 7.0], [4.0, 5.0, 8.0]], [[1.0, 2.0, 7.0], [4.0, 5.0, 2.0], [8.0, 8.0, 6.0]], [[8.0, 6.0, 
3.0], [5.0, 7.0, 3.0], [6.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 1.0], [6.0, 4.0, 1.0], [4.0, 4.0, 2.0]], [[3.0, 6.0, 4.0], [2.0, 1.0, 
3.0], [6.0, 5.0, 8.0]], [[2.0, 8.0, 8.0], [4.0, 4.0, 7.0], [3.0, 1.0, 7.0]]], [[[3.0, 4.0, 7.0], [2.0, 2.0, 4.0], [7.0, 2.0, 
4.0]], [[2.0, 3.0, 8.0], [3.0, 7.0, 3.0], [5.0, 8.0, 7.0]], [[5.0, 3.0, 4.0], [7.0, 7.0, 5.0], [7.0, 7.0, 3.0]]], [[[5.0, 2.0, 
4.0], [4.0, 4.0, 3.0], [4.0, 7.0, 1.0]], [[3.0, 7.0, 6.0], [8.0, 5.0, 6.0], [8.0, 5.0, 1.0]], [[4.0, 2.0, 5.0], [2.0, 8.0, 
1.0], [6.0, 3.0, 7.0]]], [[[2.0, 2.0, 4.0], [2.0, 1.0, 5.0], [1.0, 5.0, 7.0]], [[1.0, 2.0, 6.0], [8.0, 8.0, 1.0], [4.0, 5.0, 
5.0]], [[2.0, 1.0, 5.0], [1.0, 5.0, 7.0], [2.0, 4.0, 7.0]]]]))
      val=Data(numpy.array([[[[3.0, 1.0, 2.0], [1.0, 1.0, 5.0]], [[7.0, 2.0, 1.0], [6.0, 8.0, 5.0]], [[4.0, 8.0, 8.0], [2.0, 
2.0, 5.0]]], [[[1.0, 5.0, 7.0], [1.0, 5.0, 1.0]], [[4.0, 8.0, 8.0], [7.0, 1.0, 1.0]], [[7.0, 5.0, 1.0], [1.0, 2.0, 
6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 7.0], [3.0, 2.0, 3.0]], [[8.0, 2.0, 8.0], [1.0, 2.0, 8.0]], [[4.0, 7.0, 
5.0], [5.0, 4.0, 3.0]]], [[[5.0, 6.0, 7.0], [2.0, 3.0, 4.0]], [[4.0, 3.0, 5.0], [3.0, 8.0, 2.0]], [[5.0, 2.0, 5.0], [1.0, 2.0, 
6.0]]]]))
      arg[1:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 7.0], [7.0, 2.0, 6.0], [1.0, 3.0, 1.0]], [[2.0, 8.0, 7.0], [1.0, 8.0, 1.0], [4.0, 
1.0, 5.0]], [[8.0, 6.0, 4.0], [2.0, 5.0, 8.0], [6.0, 1.0, 5.0]]], [[[3.0, 1.0, 2.0], [1.0, 1.0, 5.0], [8.0, 7.0, 4.0]], [[7.0, 
2.0, 1.0], [6.0, 8.0, 5.0], [2.0, 5.0, 1.0]], [[4.0, 8.0, 8.0], [2.0, 2.0, 5.0], [3.0, 5.0, 2.0]]], [[[1.0, 5.0, 7.0], [1.0, 
5.0, 1.0], [4.0, 4.0, 6.0]], [[4.0, 8.0, 8.0], [7.0, 1.0, 1.0], [8.0, 2.0, 8.0]], [[7.0, 5.0, 1.0], [1.0, 2.0, 6.0], [2.0, 3.0, 
4.0]]], [[[7.0, 5.0, 4.0], [7.0, 3.0, 7.0], [4.0, 5.0, 8.0]], [[1.0, 2.0, 7.0], [4.0, 5.0, 2.0], [8.0, 8.0, 6.0]], [[8.0, 6.0, 
3.0], [5.0, 7.0, 3.0], [6.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 1.0], [6.0, 4.0, 1.0], [4.0, 4.0, 2.0]], 
[[3.0, 6.0, 4.0], [2.0, 1.0, 3.0], [6.0, 5.0, 8.0]], [[2.0, 8.0, 8.0], [4.0, 4.0, 7.0], [3.0, 1.0, 7.0]]], [[[2.0, 1.0, 7.0], 
[3.0, 2.0, 3.0], [7.0, 2.0, 4.0]], [[8.0, 2.0, 8.0], [1.0, 2.0, 8.0], [5.0, 8.0, 7.0]], [[4.0, 7.0, 5.0], [5.0, 4.0, 3.0], 
[7.0, 7.0, 3.0]]], [[[5.0, 6.0, 7.0], [2.0, 3.0, 4.0], [4.0, 7.0, 1.0]], [[4.0, 3.0, 5.0], [3.0, 8.0, 2.0], [8.0, 5.0, 1.0]], 
[[5.0, 2.0, 5.0], [1.0, 2.0, 6.0], [6.0, 3.0, 7.0]]], [[[2.0, 2.0, 4.0], [2.0, 1.0, 5.0], [1.0, 5.0, 7.0]], [[1.0, 2.0, 6.0], 
[8.0, 8.0, 1.0], [4.0, 5.0, 5.0]], [[2.0, 1.0, 5.0], [1.0, 5.0, 7.0], [2.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RCUN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 2.0], [1.0, 4.0, 8.0], [5.0, 4.0, 6.0]], [[7.0, 3.0, 8.0], [8.0, 8.0, 7.0], [8.0, 7.0, 
1.0]], [[6.0, 3.0, 5.0], [8.0, 5.0, 7.0], [1.0, 8.0, 2.0]]], [[[5.0, 4.0, 4.0], [7.0, 3.0, 4.0], [3.0, 6.0, 8.0]], [[2.0, 3.0, 
5.0], [1.0, 2.0, 1.0], [3.0, 2.0, 6.0]], [[7.0, 1.0, 5.0], [8.0, 8.0, 2.0], [3.0, 3.0, 1.0]]], [[[4.0, 1.0, 1.0], [8.0, 7.0, 
8.0], [8.0, 7.0, 7.0]], [[8.0, 7.0, 4.0], [8.0, 2.0, 5.0], [7.0, 4.0, 4.0]], [[6.0, 8.0, 3.0], [3.0, 6.0, 6.0], [3.0, 3.0, 
2.0]]], [[[4.0, 2.0, 4.0], [7.0, 3.0, 7.0], [7.0, 7.0, 2.0]], [[4.0, 6.0, 8.0], [8.0, 8.0, 7.0], [2.0, 6.0, 2.0]], [[7.0, 2.0, 
1.0], [4.0, 7.0, 1.0], [7.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 8.0], [5.0, 7.0, 7.0], [7.0, 3.0, 7.0]], [[2.0, 3.0, 2.0], [2.0, 7.0, 
8.0], [8.0, 7.0, 6.0]], [[1.0, 2.0, 3.0], [3.0, 6.0, 8.0], [5.0, 8.0, 6.0]]], [[[7.0, 4.0, 7.0], [8.0, 8.0, 6.0], [4.0, 7.0, 
3.0]], [[8.0, 2.0, 4.0], [7.0, 1.0, 8.0], [8.0, 4.0, 7.0]], [[2.0, 7.0, 1.0], [1.0, 6.0, 1.0], [2.0, 6.0, 4.0]]], [[[5.0, 2.0, 
4.0], [4.0, 2.0, 6.0], [3.0, 1.0, 7.0]], [[8.0, 4.0, 3.0], [2.0, 1.0, 3.0], [2.0, 1.0, 5.0]], [[8.0, 8.0, 6.0], [5.0, 8.0, 
5.0], [4.0, 1.0, 8.0]]], [[[8.0, 4.0, 3.0], [6.0, 7.0, 5.0], [2.0, 4.0, 5.0]], [[3.0, 5.0, 7.0], [8.0, 3.0, 6.0], [1.0, 6.0, 
2.0]], [[3.0, 7.0, 4.0], [7.0, 6.0, 8.0], [4.0, 6.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0, 2.0, 3.0], [2.0, 5.0, 5.0]], [[4.0, 2.0, 2.0], [5.0, 2.0, 3.0]], [[5.0, 5.0, 8.0], [3.0, 
8.0, 1.0]]], [[[5.0, 4.0, 8.0], [8.0, 8.0, 5.0]], [[2.0, 6.0, 4.0], [8.0, 5.0, 3.0]], [[5.0, 5.0, 4.0], [7.0, 1.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 3.0, 7.0], [6.0, 4.0, 1.0]], [[1.0, 7.0, 8.0], [3.0, 1.0, 8.0]], [[2.0, 2.0, 7.0], 
[2.0, 2.0, 2.0]]], [[[8.0, 4.0, 1.0], [2.0, 2.0, 3.0]], [[5.0, 2.0, 7.0], [3.0, 2.0, 2.0]], [[8.0, 3.0, 3.0], [7.0, 7.0, 
1.0]]]])
      arg[1:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 2.0], [1.0, 4.0, 8.0], [5.0, 4.0, 6.0]], [[7.0, 3.0, 8.0], [8.0, 8.0, 7.0], [8.0, 
7.0, 1.0]], [[6.0, 3.0, 5.0], [8.0, 5.0, 7.0], [1.0, 8.0, 2.0]]], [[[6.0, 2.0, 3.0], [2.0, 5.0, 5.0], [3.0, 6.0, 8.0]], [[4.0, 
2.0, 2.0], [5.0, 2.0, 3.0], [3.0, 2.0, 6.0]], [[5.0, 5.0, 8.0], [3.0, 8.0, 1.0], [3.0, 3.0, 1.0]]], [[[5.0, 4.0, 8.0], [8.0, 
8.0, 5.0], [8.0, 7.0, 7.0]], [[2.0, 6.0, 4.0], [8.0, 5.0, 3.0], [7.0, 4.0, 4.0]], [[5.0, 5.0, 4.0], [7.0, 1.0, 2.0], [3.0, 3.0, 
2.0]]], [[[4.0, 2.0, 4.0], [7.0, 3.0, 7.0], [7.0, 7.0, 2.0]], [[4.0, 6.0, 8.0], [8.0, 8.0, 7.0], [2.0, 6.0, 2.0]], [[7.0, 2.0, 
1.0], [4.0, 7.0, 1.0], [7.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 8.0], [5.0, 7.0, 7.0], [7.0, 3.0, 7.0]], 
[[2.0, 3.0, 2.0], [2.0, 7.0, 8.0], [8.0, 7.0, 6.0]], [[1.0, 2.0, 3.0], [3.0, 6.0, 8.0], [5.0, 8.0, 6.0]]], [[[5.0, 3.0, 7.0], 
[6.0, 4.0, 1.0], [4.0, 7.0, 3.0]], [[1.0, 7.0, 8.0], [3.0, 1.0, 8.0], [8.0, 4.0, 7.0]], [[2.0, 2.0, 7.0], [2.0, 2.0, 2.0], 
[2.0, 6.0, 4.0]]], [[[8.0, 4.0, 1.0], [2.0, 2.0, 3.0], [3.0, 1.0, 7.0]], [[5.0, 2.0, 7.0], [3.0, 2.0, 2.0], [2.0, 1.0, 5.0]], 
[[8.0, 3.0, 3.0], [7.0, 7.0, 1.0], [4.0, 1.0, 8.0]]], [[[8.0, 4.0, 3.0], [6.0, 7.0, 5.0], [2.0, 4.0, 5.0]], [[3.0, 5.0, 7.0], 
[8.0, 3.0, 6.0], [1.0, 6.0, 2.0]], [[3.0, 7.0, 4.0], [7.0, 6.0, 8.0], [4.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CUCN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 4.0], [8.0, 3.0, 7.0], [5.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [6.0, 7.0, 8.0], [8.0, 2.0, 
2.0]], [[5.0, 5.0, 2.0], [3.0, 2.0, 8.0], [1.0, 2.0, 1.0]]], [[[4.0, 4.0, 5.0], [6.0, 6.0, 7.0], [7.0, 6.0, 5.0]], [[3.0, 6.0, 
3.0], [7.0, 2.0, 8.0], [8.0, 8.0, 6.0]], [[4.0, 3.0, 8.0], [1.0, 1.0, 6.0], [6.0, 4.0, 1.0]]], [[[4.0, 1.0, 1.0], [3.0, 7.0, 
4.0], [5.0, 3.0, 3.0]], [[5.0, 1.0, 7.0], [8.0, 1.0, 3.0], [6.0, 7.0, 5.0]], [[2.0, 8.0, 2.0], [7.0, 4.0, 1.0], [2.0, 2.0, 
2.0]]], [[[5.0, 6.0, 1.0], [5.0, 5.0, 2.0], [4.0, 2.0, 1.0]], [[2.0, 5.0, 8.0], [5.0, 1.0, 2.0], [4.0, 4.0, 4.0]], [[7.0, 5.0, 
4.0], [3.0, 2.0, 5.0], [6.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 5.0], [2.0, 1.0, 2.0], [8.0, 8.0, 8.0]], [[3.0, 8.0, 2.0], [8.0, 8.0, 
4.0], [4.0, 7.0, 1.0]], [[5.0, 3.0, 1.0], [8.0, 1.0, 3.0], [4.0, 3.0, 8.0]]], [[[5.0, 6.0, 2.0], [8.0, 7.0, 8.0], [1.0, 7.0, 
2.0]], [[3.0, 8.0, 1.0], [6.0, 7.0, 5.0], [2.0, 2.0, 5.0]], [[2.0, 8.0, 5.0], [5.0, 4.0, 2.0], [1.0, 2.0, 5.0]]], [[[1.0, 1.0, 
4.0], [8.0, 8.0, 7.0], [7.0, 7.0, 1.0]], [[4.0, 6.0, 3.0], [7.0, 8.0, 5.0], [8.0, 6.0, 6.0]], [[5.0, 7.0, 7.0], [6.0, 7.0, 
8.0], [6.0, 5.0, 2.0]]], [[[5.0, 6.0, 7.0], [5.0, 5.0, 3.0], [8.0, 2.0, 4.0]], [[8.0, 1.0, 6.0], [6.0, 1.0, 8.0], [6.0, 7.0, 
6.0]], [[3.0, 5.0, 6.0], [5.0, 3.0, 8.0], [8.0, 5.0, 2.0]]]]))
      val=numpy.array([[[[6.0, 2.0, 7.0], [8.0, 7.0, 5.0], [4.0, 1.0, 8.0]], [[1.0, 1.0, 4.0], [8.0, 6.0, 7.0], [6.0, 7.0, 
3.0]]], [[[6.0, 1.0, 8.0], [6.0, 6.0, 8.0], [7.0, 3.0, 7.0]], [[3.0, 3.0, 1.0], [8.0, 8.0, 6.0], [3.0, 2.0, 8.0]]], [[[8.0, 
2.0, 1.0], [2.0, 1.0, 3.0], [2.0, 4.0, 6.0]], [[7.0, 7.0, 7.0], [6.0, 3.0, 5.0], [4.0, 1.0, 8.0]]], [[[2.0, 5.0, 2.0], [2.0, 
8.0, 7.0], [6.0, 2.0, 1.0]], [[2.0, 5.0, 8.0], [3.0, 2.0, 7.0], [4.0, 6.0, 8.0]]]])
      arg[:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 7.0], [8.0, 7.0, 5.0], [4.0, 1.0, 8.0]], [[1.0, 1.0, 4.0], [8.0, 6.0, 7.0], [6.0, 
7.0, 3.0]], [[5.0, 5.0, 2.0], [3.0, 2.0, 8.0], [1.0, 2.0, 1.0]]], [[[6.0, 1.0, 8.0], [6.0, 6.0, 8.0], [7.0, 3.0, 7.0]], [[3.0, 
3.0, 1.0], [8.0, 8.0, 6.0], [3.0, 2.0, 8.0]], [[4.0, 3.0, 8.0], [1.0, 1.0, 6.0], [6.0, 4.0, 1.0]]], [[[8.0, 2.0, 1.0], [2.0, 
1.0, 3.0], [2.0, 4.0, 6.0]], [[7.0, 7.0, 7.0], [6.0, 3.0, 5.0], [4.0, 1.0, 8.0]], [[2.0, 8.0, 2.0], [7.0, 4.0, 1.0], [2.0, 2.0, 
2.0]]], [[[2.0, 5.0, 2.0], [2.0, 8.0, 7.0], [6.0, 2.0, 1.0]], [[2.0, 5.0, 8.0], [3.0, 2.0, 7.0], [4.0, 6.0, 8.0]], [[7.0, 5.0, 
4.0], [3.0, 2.0, 5.0], [6.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 7.0], [8.0, 7.0, 5.0], [4.0, 1.0, 8.0]], 
[[1.0, 1.0, 4.0], [8.0, 6.0, 7.0], [6.0, 7.0, 3.0]], [[5.0, 3.0, 1.0], [8.0, 1.0, 3.0], [4.0, 3.0, 8.0]]], [[[6.0, 1.0, 8.0], 
[6.0, 6.0, 8.0], [7.0, 3.0, 7.0]], [[3.0, 3.0, 1.0], [8.0, 8.0, 6.0], [3.0, 2.0, 8.0]], [[2.0, 8.0, 5.0], [5.0, 4.0, 2.0], 
[1.0, 2.0, 5.0]]], [[[8.0, 2.0, 1.0], [2.0, 1.0, 3.0], [2.0, 4.0, 6.0]], [[7.0, 7.0, 7.0], [6.0, 3.0, 5.0], [4.0, 1.0, 8.0]], 
[[5.0, 7.0, 7.0], [6.0, 7.0, 8.0], [6.0, 5.0, 2.0]]], [[[2.0, 5.0, 2.0], [2.0, 8.0, 7.0], [6.0, 2.0, 1.0]], [[2.0, 5.0, 8.0], 
[3.0, 2.0, 7.0], [4.0, 6.0, 8.0]], [[3.0, 5.0, 6.0], [5.0, 3.0, 8.0], [8.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CUCN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 3.0], [3.0, 7.0, 7.0], [8.0, 2.0, 4.0]], [[1.0, 3.0, 8.0], [4.0, 1.0, 6.0], [2.0, 2.0, 
3.0]], [[1.0, 1.0, 8.0], [1.0, 6.0, 8.0], [3.0, 2.0, 8.0]]], [[[7.0, 3.0, 7.0], [2.0, 6.0, 6.0], [1.0, 1.0, 7.0]], [[7.0, 1.0, 
3.0], [7.0, 7.0, 2.0], [3.0, 6.0, 5.0]], [[1.0, 5.0, 5.0], [8.0, 5.0, 5.0], [3.0, 4.0, 1.0]]], [[[8.0, 7.0, 5.0], [5.0, 3.0, 
3.0], [6.0, 1.0, 7.0]], [[6.0, 1.0, 3.0], [5.0, 6.0, 3.0], [2.0, 4.0, 3.0]], [[1.0, 4.0, 5.0], [6.0, 6.0, 2.0], [3.0, 1.0, 
1.0]]], [[[4.0, 1.0, 3.0], [7.0, 5.0, 7.0], [2.0, 1.0, 8.0]], [[5.0, 5.0, 6.0], [6.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[1.0, 3.0, 
6.0], [4.0, 6.0, 8.0], [4.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 8.0], [4.0, 1.0, 1.0], [2.0, 6.0, 6.0]], [[4.0, 1.0, 1.0], [5.0, 2.0, 
5.0], [7.0, 2.0, 8.0]], [[3.0, 4.0, 6.0], [7.0, 4.0, 3.0], [7.0, 3.0, 8.0]]], [[[5.0, 8.0, 5.0], [7.0, 6.0, 2.0], [1.0, 1.0, 
3.0]], [[8.0, 2.0, 8.0], [8.0, 6.0, 7.0], [7.0, 4.0, 7.0]], [[4.0, 4.0, 5.0], [4.0, 6.0, 1.0], [6.0, 8.0, 2.0]]], [[[2.0, 3.0, 
6.0], [1.0, 2.0, 2.0], [1.0, 2.0, 2.0]], [[6.0, 6.0, 6.0], [3.0, 7.0, 8.0], [4.0, 2.0, 2.0]], [[3.0, 2.0, 8.0], [8.0, 3.0, 
6.0], [7.0, 7.0, 4.0]]], [[[6.0, 1.0, 6.0], [1.0, 2.0, 4.0], [7.0, 6.0, 5.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 3.0], [3.0, 8.0, 
8.0]], [[1.0, 5.0, 3.0], [8.0, 3.0, 5.0], [4.0, 4.0, 7.0]]]]))
      val=Data(numpy.array([[[[4.0, 1.0, 3.0], [1.0, 4.0, 4.0], [6.0, 8.0, 4.0]], [[1.0, 6.0, 2.0], [2.0, 8.0, 5.0], [4.0, 4.0, 
2.0]]], [[[4.0, 8.0, 3.0], [1.0, 8.0, 8.0], [7.0, 2.0, 3.0]], [[6.0, 7.0, 8.0], [4.0, 7.0, 4.0], [4.0, 5.0, 3.0]]], [[[5.0, 
4.0, 4.0], [8.0, 2.0, 4.0], [5.0, 2.0, 5.0]], [[4.0, 6.0, 1.0], [2.0, 8.0, 5.0], [3.0, 6.0, 6.0]]], [[[7.0, 2.0, 7.0], [5.0, 
3.0, 5.0], [4.0, 8.0, 2.0]], [[2.0, 8.0, 8.0], [5.0, 3.0, 2.0], [4.0, 6.0, 5.0]]]]),self.functionspace)
      arg[:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 3.0], [1.0, 4.0, 4.0], [6.0, 8.0, 4.0]], [[1.0, 6.0, 2.0], [2.0, 8.0, 5.0], [4.0, 
4.0, 2.0]], [[1.0, 1.0, 8.0], [1.0, 6.0, 8.0], [3.0, 2.0, 8.0]]], [[[4.0, 8.0, 3.0], [1.0, 8.0, 8.0], [7.0, 2.0, 3.0]], [[6.0, 
7.0, 8.0], [4.0, 7.0, 4.0], [4.0, 5.0, 3.0]], [[1.0, 5.0, 5.0], [8.0, 5.0, 5.0], [3.0, 4.0, 1.0]]], [[[5.0, 4.0, 4.0], [8.0, 
2.0, 4.0], [5.0, 2.0, 5.0]], [[4.0, 6.0, 1.0], [2.0, 8.0, 5.0], [3.0, 6.0, 6.0]], [[1.0, 4.0, 5.0], [6.0, 6.0, 2.0], [3.0, 1.0, 
1.0]]], [[[7.0, 2.0, 7.0], [5.0, 3.0, 5.0], [4.0, 8.0, 2.0]], [[2.0, 8.0, 8.0], [5.0, 3.0, 2.0], [4.0, 6.0, 5.0]], [[1.0, 3.0, 
6.0], [4.0, 6.0, 8.0], [4.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 3.0], [1.0, 4.0, 4.0], [6.0, 8.0, 4.0]], 
[[1.0, 6.0, 2.0], [2.0, 8.0, 5.0], [4.0, 4.0, 2.0]], [[3.0, 4.0, 6.0], [7.0, 4.0, 3.0], [7.0, 3.0, 8.0]]], [[[4.0, 8.0, 3.0], 
[1.0, 8.0, 8.0], [7.0, 2.0, 3.0]], [[6.0, 7.0, 8.0], [4.0, 7.0, 4.0], [4.0, 5.0, 3.0]], [[4.0, 4.0, 5.0], [4.0, 6.0, 1.0], 
[6.0, 8.0, 2.0]]], [[[5.0, 4.0, 4.0], [8.0, 2.0, 4.0], [5.0, 2.0, 5.0]], [[4.0, 6.0, 1.0], [2.0, 8.0, 5.0], [3.0, 6.0, 6.0]], 
[[3.0, 2.0, 8.0], [8.0, 3.0, 6.0], [7.0, 7.0, 4.0]]], [[[7.0, 2.0, 7.0], [5.0, 3.0, 5.0], [4.0, 8.0, 2.0]], [[2.0, 8.0, 8.0], 
[5.0, 3.0, 2.0], [4.0, 6.0, 5.0]], [[1.0, 5.0, 3.0], [8.0, 3.0, 5.0], [4.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CUCN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 6.0], [4.0, 6.0, 1.0], [2.0, 2.0, 5.0]], [[3.0, 4.0, 4.0], [1.0, 5.0, 7.0], [4.0, 1.0, 
8.0]], [[7.0, 3.0, 6.0], [8.0, 5.0, 4.0], [6.0, 5.0, 3.0]]], [[[1.0, 6.0, 3.0], [1.0, 6.0, 8.0], [2.0, 8.0, 3.0]], [[2.0, 8.0, 
5.0], [7.0, 1.0, 4.0], [7.0, 1.0, 6.0]], [[8.0, 1.0, 5.0], [3.0, 7.0, 6.0], [7.0, 7.0, 3.0]]], [[[8.0, 7.0, 5.0], [6.0, 7.0, 
7.0], [5.0, 5.0, 6.0]], [[5.0, 3.0, 8.0], [2.0, 5.0, 7.0], [1.0, 5.0, 8.0]], [[8.0, 1.0, 6.0], [1.0, 8.0, 7.0], [7.0, 8.0, 
6.0]]], [[[4.0, 4.0, 5.0], [8.0, 2.0, 8.0], [4.0, 1.0, 3.0]], [[6.0, 3.0, 8.0], [5.0, 3.0, 8.0], [8.0, 8.0, 6.0]], [[4.0, 5.0, 
8.0], [8.0, 4.0, 3.0], [1.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 1.0], [5.0, 5.0, 3.0], [7.0, 5.0, 5.0]], [[5.0, 8.0, 3.0], [8.0, 1.0, 
6.0], [3.0, 3.0, 1.0]], [[8.0, 6.0, 2.0], [5.0, 2.0, 5.0], [7.0, 7.0, 5.0]]], [[[8.0, 2.0, 6.0], [1.0, 8.0, 1.0], [6.0, 5.0, 
2.0]], [[5.0, 4.0, 2.0], [8.0, 6.0, 6.0], [6.0, 7.0, 6.0]], [[6.0, 4.0, 2.0], [4.0, 7.0, 2.0], [7.0, 7.0, 8.0]]], [[[5.0, 5.0, 
2.0], [3.0, 6.0, 1.0], [1.0, 2.0, 7.0]], [[4.0, 1.0, 7.0], [8.0, 8.0, 1.0], [8.0, 8.0, 1.0]], [[3.0, 5.0, 7.0], [3.0, 3.0, 
6.0], [3.0, 7.0, 4.0]]], [[[5.0, 7.0, 6.0], [1.0, 5.0, 8.0], [8.0, 1.0, 1.0]], [[1.0, 7.0, 7.0], [8.0, 3.0, 5.0], [3.0, 6.0, 
3.0]], [[6.0, 4.0, 1.0], [1.0, 7.0, 7.0], [4.0, 6.0, 2.0]]]]))
      val=Data(numpy.array([[[[6.0, 6.0, 2.0], [3.0, 3.0, 4.0], [3.0, 7.0, 4.0]], [[8.0, 4.0, 3.0], [5.0, 5.0, 3.0], [2.0, 7.0, 
2.0]]], [[[1.0, 3.0, 1.0], [6.0, 6.0, 8.0], [3.0, 7.0, 1.0]], [[4.0, 8.0, 3.0], [3.0, 8.0, 3.0], [3.0, 1.0, 2.0]]], [[[2.0, 
2.0, 7.0], [8.0, 5.0, 6.0], [7.0, 1.0, 8.0]], [[6.0, 2.0, 2.0], [4.0, 2.0, 2.0], [4.0, 6.0, 4.0]]], [[[1.0, 2.0, 4.0], [6.0, 
4.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 4.0, 4.0], [8.0, 5.0, 7.0], [8.0, 6.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 4.0], [2.0, 1.0, 8.0], [2.0, 8.0, 5.0]], [[5.0, 3.0, 1.0], [4.0, 6.0, 
2.0], [5.0, 7.0, 4.0]]], [[[4.0, 8.0, 8.0], [3.0, 3.0, 8.0], [8.0, 2.0, 1.0]], [[5.0, 5.0, 4.0], [1.0, 6.0, 7.0], [7.0, 8.0, 
2.0]]], [[[7.0, 2.0, 7.0], [4.0, 1.0, 7.0], [8.0, 5.0, 7.0]], [[2.0, 1.0, 1.0], [7.0, 5.0, 8.0], [4.0, 3.0, 6.0]]], [[[6.0, 
7.0, 1.0], [4.0, 8.0, 7.0], [1.0, 4.0, 4.0]], [[6.0, 3.0, 1.0], [8.0, 2.0, 1.0], [3.0, 1.0, 4.0]]]]))
      arg[:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 2.0], [3.0, 3.0, 4.0], [3.0, 7.0, 4.0]], [[8.0, 4.0, 3.0], [5.0, 5.0, 3.0], [2.0, 
7.0, 2.0]], [[7.0, 3.0, 6.0], [8.0, 5.0, 4.0], [6.0, 5.0, 3.0]]], [[[1.0, 3.0, 1.0], [6.0, 6.0, 8.0], [3.0, 7.0, 1.0]], [[4.0, 
8.0, 3.0], [3.0, 8.0, 3.0], [3.0, 1.0, 2.0]], [[8.0, 1.0, 5.0], [3.0, 7.0, 6.0], [7.0, 7.0, 3.0]]], [[[2.0, 2.0, 7.0], [8.0, 
5.0, 6.0], [7.0, 1.0, 8.0]], [[6.0, 2.0, 2.0], [4.0, 2.0, 2.0], [4.0, 6.0, 4.0]], [[8.0, 1.0, 6.0], [1.0, 8.0, 7.0], [7.0, 8.0, 
6.0]]], [[[1.0, 2.0, 4.0], [6.0, 4.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 4.0, 4.0], [8.0, 5.0, 7.0], [8.0, 6.0, 1.0]], [[4.0, 5.0, 
8.0], [8.0, 4.0, 3.0], [1.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 4.0], [2.0, 1.0, 8.0], [2.0, 8.0, 5.0]], 
[[5.0, 3.0, 1.0], [4.0, 6.0, 2.0], [5.0, 7.0, 4.0]], [[8.0, 6.0, 2.0], [5.0, 2.0, 5.0], [7.0, 7.0, 5.0]]], [[[4.0, 8.0, 8.0], 
[3.0, 3.0, 8.0], [8.0, 2.0, 1.0]], [[5.0, 5.0, 4.0], [1.0, 6.0, 7.0], [7.0, 8.0, 2.0]], [[6.0, 4.0, 2.0], [4.0, 7.0, 2.0], 
[7.0, 7.0, 8.0]]], [[[7.0, 2.0, 7.0], [4.0, 1.0, 7.0], [8.0, 5.0, 7.0]], [[2.0, 1.0, 1.0], [7.0, 5.0, 8.0], [4.0, 3.0, 6.0]], 
[[3.0, 5.0, 7.0], [3.0, 3.0, 6.0], [3.0, 7.0, 4.0]]], [[[6.0, 7.0, 1.0], [4.0, 8.0, 7.0], [1.0, 4.0, 4.0]], [[6.0, 3.0, 1.0], 
[8.0, 2.0, 1.0], [3.0, 1.0, 4.0]], [[6.0, 4.0, 1.0], [1.0, 7.0, 7.0], [4.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CUCN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 6.0], [7.0, 1.0, 1.0], [1.0, 1.0, 2.0]], [[6.0, 4.0, 3.0], [3.0, 4.0, 1.0], [8.0, 4.0, 
7.0]], [[7.0, 1.0, 6.0], [3.0, 6.0, 1.0], [4.0, 6.0, 2.0]]], [[[3.0, 1.0, 7.0], [3.0, 7.0, 7.0], [1.0, 1.0, 2.0]], [[1.0, 1.0, 
1.0], [4.0, 8.0, 3.0], [2.0, 1.0, 1.0]], [[4.0, 6.0, 1.0], [5.0, 1.0, 6.0], [5.0, 2.0, 7.0]]], [[[7.0, 2.0, 4.0], [4.0, 5.0, 
7.0], [8.0, 7.0, 4.0]], [[5.0, 1.0, 3.0], [3.0, 8.0, 7.0], [8.0, 2.0, 8.0]], [[5.0, 5.0, 7.0], [7.0, 7.0, 1.0], [2.0, 3.0, 
6.0]]], [[[2.0, 6.0, 6.0], [8.0, 7.0, 3.0], [1.0, 7.0, 5.0]], [[6.0, 4.0, 1.0], [8.0, 5.0, 3.0], [7.0, 2.0, 3.0]], [[2.0, 1.0, 
8.0], [8.0, 6.0, 4.0], [3.0, 5.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 6.0], [5.0, 8.0, 4.0], [3.0, 2.0, 1.0]], [[1.0, 3.0, 6.0], [6.0, 7.0, 
8.0], [6.0, 5.0, 6.0]], [[3.0, 4.0, 8.0], [6.0, 4.0, 4.0], [6.0, 6.0, 7.0]]], [[[2.0, 4.0, 3.0], [1.0, 8.0, 7.0], [5.0, 5.0, 
3.0]], [[6.0, 5.0, 5.0], [1.0, 7.0, 1.0], [5.0, 5.0, 5.0]], [[4.0, 1.0, 5.0], [5.0, 2.0, 1.0], [4.0, 2.0, 3.0]]], [[[5.0, 8.0, 
2.0], [3.0, 5.0, 8.0], [6.0, 3.0, 3.0]], [[8.0, 1.0, 3.0], [8.0, 3.0, 2.0], [8.0, 6.0, 5.0]], [[3.0, 5.0, 1.0], [4.0, 2.0, 
2.0], [3.0, 6.0, 2.0]]], [[[7.0, 3.0, 6.0], [7.0, 3.0, 1.0], [6.0, 7.0, 6.0]], [[2.0, 8.0, 1.0], [3.0, 4.0, 4.0], [1.0, 3.0, 
1.0]], [[7.0, 3.0, 3.0], [5.0, 1.0, 3.0], [1.0, 3.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 6.0, 3.0], [5.0, 5.0, 2.0], [3.0, 3.0, 4.0]], [[5.0, 6.0, 5.0], [8.0, 7.0, 8.0], [4.0, 
2.0, 2.0]]], [[[1.0, 1.0, 1.0], [2.0, 7.0, 4.0], [6.0, 6.0, 8.0]], [[6.0, 5.0, 8.0], [7.0, 7.0, 3.0], [3.0, 7.0, 1.0]]], 
[[[5.0, 1.0, 4.0], [8.0, 2.0, 6.0], [7.0, 2.0, 1.0]], [[5.0, 4.0, 2.0], [7.0, 2.0, 2.0], [2.0, 3.0, 8.0]]], [[[2.0, 8.0, 4.0], 
[7.0, 5.0, 8.0], [5.0, 6.0, 8.0]], [[5.0, 4.0, 6.0], [2.0, 6.0, 5.0], [3.0, 5.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 
3.0, 4.0], [6.0, 2.0, 1.0], [1.0, 7.0, 3.0]], [[6.0, 8.0, 8.0], [4.0, 6.0, 5.0], [4.0, 1.0, 8.0]]], [[[5.0, 6.0, 3.0], [2.0, 
3.0, 4.0], [4.0, 7.0, 4.0]], [[4.0, 4.0, 5.0], [7.0, 4.0, 4.0], [8.0, 7.0, 7.0]]], [[[6.0, 7.0, 1.0], [3.0, 7.0, 4.0], [1.0, 
1.0, 5.0]], [[7.0, 2.0, 4.0], [2.0, 3.0, 1.0], [3.0, 7.0, 1.0]]], [[[7.0, 4.0, 5.0], [8.0, 3.0, 1.0], [8.0, 6.0, 8.0]], [[5.0, 
1.0, 3.0], [5.0, 1.0, 5.0], [6.0, 7.0, 3.0]]]])
      arg[:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 3.0], [5.0, 5.0, 2.0], [3.0, 3.0, 4.0]], [[5.0, 6.0, 5.0], [8.0, 7.0, 8.0], [4.0, 
2.0, 2.0]], [[7.0, 1.0, 6.0], [3.0, 6.0, 1.0], [4.0, 6.0, 2.0]]], [[[1.0, 1.0, 1.0], [2.0, 7.0, 4.0], [6.0, 6.0, 8.0]], [[6.0, 
5.0, 8.0], [7.0, 7.0, 3.0], [3.0, 7.0, 1.0]], [[4.0, 6.0, 1.0], [5.0, 1.0, 6.0], [5.0, 2.0, 7.0]]], [[[5.0, 1.0, 4.0], [8.0, 
2.0, 6.0], [7.0, 2.0, 1.0]], [[5.0, 4.0, 2.0], [7.0, 2.0, 2.0], [2.0, 3.0, 8.0]], [[5.0, 5.0, 7.0], [7.0, 7.0, 1.0], [2.0, 3.0, 
6.0]]], [[[2.0, 8.0, 4.0], [7.0, 5.0, 8.0], [5.0, 6.0, 8.0]], [[5.0, 4.0, 6.0], [2.0, 6.0, 5.0], [3.0, 5.0, 8.0]], [[2.0, 1.0, 
8.0], [8.0, 6.0, 4.0], [3.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 4.0], [6.0, 2.0, 1.0], [1.0, 7.0, 3.0]], 
[[6.0, 8.0, 8.0], [4.0, 6.0, 5.0], [4.0, 1.0, 8.0]], [[3.0, 4.0, 8.0], [6.0, 4.0, 4.0], [6.0, 6.0, 7.0]]], [[[5.0, 6.0, 3.0], 
[2.0, 3.0, 4.0], [4.0, 7.0, 4.0]], [[4.0, 4.0, 5.0], [7.0, 4.0, 4.0], [8.0, 7.0, 7.0]], [[4.0, 1.0, 5.0], [5.0, 2.0, 1.0], 
[4.0, 2.0, 3.0]]], [[[6.0, 7.0, 1.0], [3.0, 7.0, 4.0], [1.0, 1.0, 5.0]], [[7.0, 2.0, 4.0], [2.0, 3.0, 1.0], [3.0, 7.0, 1.0]], 
[[3.0, 5.0, 1.0], [4.0, 2.0, 2.0], [3.0, 6.0, 2.0]]], [[[7.0, 4.0, 5.0], [8.0, 3.0, 1.0], [8.0, 6.0, 8.0]], [[5.0, 1.0, 3.0], 
[5.0, 1.0, 5.0], [6.0, 7.0, 3.0]], [[7.0, 3.0, 3.0], [5.0, 1.0, 3.0], [1.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPRU(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 4.0], [5.0, 4.0, 7.0], [4.0, 1.0, 2.0]], [[2.0, 1.0, 8.0], [5.0, 2.0, 5.0], [6.0, 7.0, 
4.0]], [[2.0, 3.0, 6.0], [4.0, 2.0, 4.0], [7.0, 3.0, 5.0]]], [[[4.0, 3.0, 1.0], [6.0, 7.0, 6.0], [8.0, 1.0, 3.0]], [[4.0, 6.0, 
7.0], [8.0, 8.0, 4.0], [7.0, 5.0, 5.0]], [[7.0, 8.0, 8.0], [1.0, 1.0, 8.0], [1.0, 5.0, 2.0]]], [[[3.0, 1.0, 7.0], [5.0, 2.0, 
2.0], [6.0, 7.0, 3.0]], [[6.0, 1.0, 8.0], [2.0, 2.0, 8.0], [4.0, 2.0, 3.0]], [[7.0, 1.0, 2.0], [4.0, 2.0, 2.0], [3.0, 4.0, 
3.0]]], [[[7.0, 3.0, 5.0], [7.0, 6.0, 5.0], [4.0, 7.0, 3.0]], [[4.0, 4.0, 3.0], [4.0, 5.0, 6.0], [8.0, 2.0, 6.0]], [[4.0, 3.0, 
6.0], [5.0, 6.0, 3.0], [4.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 2.0], [2.0, 3.0, 8.0], [5.0, 4.0, 1.0]], [[8.0, 8.0, 7.0], [2.0, 1.0, 
4.0], [5.0, 6.0, 7.0]], [[1.0, 1.0, 7.0], [6.0, 8.0, 1.0], [6.0, 2.0, 3.0]]], [[[2.0, 8.0, 2.0], [8.0, 7.0, 1.0], [5.0, 1.0, 
3.0]], [[1.0, 1.0, 1.0], [7.0, 6.0, 1.0], [8.0, 1.0, 8.0]], [[5.0, 5.0, 2.0], [4.0, 3.0, 4.0], [5.0, 8.0, 8.0]]], [[[1.0, 2.0, 
8.0], [1.0, 3.0, 5.0], [6.0, 8.0, 2.0]], [[7.0, 2.0, 1.0], [6.0, 8.0, 1.0], [5.0, 8.0, 6.0]], [[1.0, 8.0, 7.0], [2.0, 6.0, 
6.0], [6.0, 8.0, 4.0]]], [[[8.0, 8.0, 4.0], [3.0, 4.0, 4.0], [2.0, 3.0, 7.0]], [[2.0, 2.0, 5.0], [2.0, 4.0, 2.0], [8.0, 6.0, 
1.0]], [[8.0, 5.0, 8.0], [1.0, 5.0, 8.0], [2.0, 8.0, 7.0]]]]))
      val=numpy.array([[5.0, 1.0]])
      arg[2,1,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 4.0], [5.0, 4.0, 7.0], [4.0, 1.0, 2.0]], [[2.0, 1.0, 8.0], [5.0, 2.0, 5.0], [6.0, 
7.0, 4.0]], [[2.0, 3.0, 6.0], [4.0, 2.0, 4.0], [7.0, 3.0, 5.0]]], [[[4.0, 3.0, 1.0], [6.0, 7.0, 6.0], [8.0, 1.0, 3.0]], [[4.0, 
6.0, 7.0], [8.0, 8.0, 4.0], [7.0, 5.0, 5.0]], [[7.0, 8.0, 8.0], [1.0, 1.0, 8.0], [1.0, 5.0, 2.0]]], [[[3.0, 1.0, 7.0], [5.0, 
2.0, 2.0], [6.0, 7.0, 3.0]], [[6.0, 1.0, 8.0], [5.0, 1.0, 8.0], [4.0, 2.0, 3.0]], [[7.0, 1.0, 2.0], [4.0, 2.0, 2.0], [3.0, 4.0, 
3.0]]], [[[7.0, 3.0, 5.0], [7.0, 6.0, 5.0], [4.0, 7.0, 3.0]], [[4.0, 4.0, 3.0], [4.0, 5.0, 6.0], [8.0, 2.0, 6.0]], [[4.0, 3.0, 
6.0], [5.0, 6.0, 3.0], [4.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 2.0], [2.0, 3.0, 8.0], [5.0, 4.0, 1.0]], 
[[8.0, 8.0, 7.0], [2.0, 1.0, 4.0], [5.0, 6.0, 7.0]], [[1.0, 1.0, 7.0], [6.0, 8.0, 1.0], [6.0, 2.0, 3.0]]], [[[2.0, 8.0, 2.0], 
[8.0, 7.0, 1.0], [5.0, 1.0, 3.0]], [[1.0, 1.0, 1.0], [7.0, 6.0, 1.0], [8.0, 1.0, 8.0]], [[5.0, 5.0, 2.0], [4.0, 3.0, 4.0], 
[5.0, 8.0, 8.0]]], [[[1.0, 2.0, 8.0], [1.0, 3.0, 5.0], [6.0, 8.0, 2.0]], [[7.0, 2.0, 1.0], [5.0, 1.0, 1.0], [5.0, 8.0, 6.0]], 
[[1.0, 8.0, 7.0], [2.0, 6.0, 6.0], [6.0, 8.0, 4.0]]], [[[8.0, 8.0, 4.0], [3.0, 4.0, 4.0], [2.0, 3.0, 7.0]], [[2.0, 2.0, 5.0], 
[2.0, 4.0, 2.0], [8.0, 6.0, 1.0]], [[8.0, 5.0, 8.0], [1.0, 5.0, 8.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPRU(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 8.0], [6.0, 8.0, 5.0], [2.0, 4.0, 7.0]], [[4.0, 1.0, 1.0], [7.0, 6.0, 8.0], [7.0, 6.0, 
6.0]], [[1.0, 3.0, 3.0], [5.0, 7.0, 2.0], [3.0, 6.0, 7.0]]], [[[7.0, 8.0, 4.0], [5.0, 7.0, 5.0], [8.0, 6.0, 3.0]], [[6.0, 7.0, 
3.0], [3.0, 6.0, 3.0], [3.0, 8.0, 6.0]], [[8.0, 5.0, 1.0], [5.0, 2.0, 5.0], [5.0, 4.0, 7.0]]], [[[7.0, 8.0, 4.0], [6.0, 7.0, 
5.0], [6.0, 1.0, 8.0]], [[5.0, 2.0, 2.0], [7.0, 8.0, 8.0], [8.0, 4.0, 4.0]], [[4.0, 7.0, 1.0], [6.0, 1.0, 8.0], [4.0, 5.0, 
2.0]]], [[[5.0, 7.0, 1.0], [7.0, 5.0, 8.0], [6.0, 2.0, 8.0]], [[4.0, 5.0, 5.0], [8.0, 6.0, 4.0], [6.0, 7.0, 3.0]], [[1.0, 7.0, 
6.0], [1.0, 7.0, 3.0], [5.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 3.0], [7.0, 5.0, 1.0], [8.0, 8.0, 4.0]], [[7.0, 5.0, 2.0], [4.0, 3.0, 
1.0], [8.0, 6.0, 6.0]], [[1.0, 6.0, 5.0], [1.0, 4.0, 4.0], [3.0, 7.0, 4.0]]], [[[6.0, 1.0, 1.0], [7.0, 3.0, 1.0], [7.0, 3.0, 
1.0]], [[3.0, 5.0, 3.0], [1.0, 6.0, 1.0], [6.0, 6.0, 3.0]], [[4.0, 4.0, 5.0], [7.0, 2.0, 5.0], [3.0, 2.0, 1.0]]], [[[4.0, 8.0, 
4.0], [8.0, 1.0, 4.0], [2.0, 8.0, 2.0]], [[5.0, 7.0, 8.0], [8.0, 4.0, 1.0], [1.0, 5.0, 5.0]], [[5.0, 3.0, 7.0], [1.0, 5.0, 
2.0], [1.0, 1.0, 3.0]]], [[[7.0, 5.0, 3.0], [2.0, 2.0, 3.0], [3.0, 6.0, 8.0]], [[3.0, 6.0, 7.0], [7.0, 7.0, 3.0], [4.0, 5.0, 
1.0]], [[7.0, 5.0, 2.0], [3.0, 5.0, 3.0], [8.0, 6.0, 4.0]]]]))
      val=Data(numpy.array([[7.0, 6.0]]),self.functionspace)
      arg[2,1,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [6.0, 8.0, 5.0], [2.0, 4.0, 7.0]], [[4.0, 1.0, 1.0], [7.0, 6.0, 8.0], [7.0, 
6.0, 6.0]], [[1.0, 3.0, 3.0], [5.0, 7.0, 2.0], [3.0, 6.0, 7.0]]], [[[7.0, 8.0, 4.0], [5.0, 7.0, 5.0], [8.0, 6.0, 3.0]], [[6.0, 
7.0, 3.0], [3.0, 6.0, 3.0], [3.0, 8.0, 6.0]], [[8.0, 5.0, 1.0], [5.0, 2.0, 5.0], [5.0, 4.0, 7.0]]], [[[7.0, 8.0, 4.0], [6.0, 
7.0, 5.0], [6.0, 1.0, 8.0]], [[5.0, 2.0, 2.0], [7.0, 6.0, 8.0], [8.0, 4.0, 4.0]], [[4.0, 7.0, 1.0], [6.0, 1.0, 8.0], [4.0, 5.0, 
2.0]]], [[[5.0, 7.0, 1.0], [7.0, 5.0, 8.0], [6.0, 2.0, 8.0]], [[4.0, 5.0, 5.0], [8.0, 6.0, 4.0], [6.0, 7.0, 3.0]], [[1.0, 7.0, 
6.0], [1.0, 7.0, 3.0], [5.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 3.0], [7.0, 5.0, 1.0], [8.0, 8.0, 4.0]], 
[[7.0, 5.0, 2.0], [4.0, 3.0, 1.0], [8.0, 6.0, 6.0]], [[1.0, 6.0, 5.0], [1.0, 4.0, 4.0], [3.0, 7.0, 4.0]]], [[[6.0, 1.0, 1.0], 
[7.0, 3.0, 1.0], [7.0, 3.0, 1.0]], [[3.0, 5.0, 3.0], [1.0, 6.0, 1.0], [6.0, 6.0, 3.0]], [[4.0, 4.0, 5.0], [7.0, 2.0, 5.0], 
[3.0, 2.0, 1.0]]], [[[4.0, 8.0, 4.0], [8.0, 1.0, 4.0], [2.0, 8.0, 2.0]], [[5.0, 7.0, 8.0], [7.0, 6.0, 1.0], [1.0, 5.0, 5.0]], 
[[5.0, 3.0, 7.0], [1.0, 5.0, 2.0], [1.0, 1.0, 3.0]]], [[[7.0, 5.0, 3.0], [2.0, 2.0, 3.0], [3.0, 6.0, 8.0]], [[3.0, 6.0, 7.0], 
[7.0, 7.0, 3.0], [4.0, 5.0, 1.0]], [[7.0, 5.0, 2.0], [3.0, 5.0, 3.0], [8.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPRU(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 1.0], [5.0, 8.0, 8.0], [6.0, 8.0, 6.0]], [[4.0, 1.0, 6.0], [8.0, 7.0, 3.0], [1.0, 6.0, 
3.0]], [[3.0, 2.0, 6.0], [4.0, 6.0, 7.0], [7.0, 4.0, 5.0]]], [[[6.0, 1.0, 8.0], [5.0, 1.0, 8.0], [7.0, 8.0, 8.0]], [[4.0, 3.0, 
7.0], [4.0, 1.0, 8.0], [8.0, 3.0, 5.0]], [[8.0, 4.0, 2.0], [8.0, 6.0, 3.0], [2.0, 7.0, 3.0]]], [[[1.0, 3.0, 8.0], [8.0, 4.0, 
7.0], [4.0, 3.0, 7.0]], [[8.0, 7.0, 5.0], [6.0, 5.0, 2.0], [3.0, 3.0, 1.0]], [[2.0, 1.0, 8.0], [3.0, 1.0, 7.0], [8.0, 4.0, 
5.0]]], [[[1.0, 4.0, 7.0], [8.0, 8.0, 5.0], [5.0, 3.0, 4.0]], [[2.0, 5.0, 5.0], [8.0, 4.0, 4.0], [2.0, 2.0, 8.0]], [[6.0, 5.0, 
3.0], [5.0, 6.0, 6.0], [1.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 1.0], [8.0, 3.0, 7.0], [3.0, 2.0, 6.0]], [[3.0, 1.0, 8.0], [2.0, 4.0, 
7.0], [5.0, 8.0, 8.0]], [[7.0, 8.0, 1.0], [2.0, 2.0, 5.0], [5.0, 3.0, 7.0]]], [[[7.0, 5.0, 6.0], [5.0, 4.0, 5.0], [6.0, 5.0, 
2.0]], [[7.0, 5.0, 8.0], [1.0, 1.0, 7.0], [6.0, 4.0, 7.0]], [[6.0, 2.0, 7.0], [6.0, 5.0, 6.0], [8.0, 3.0, 1.0]]], [[[3.0, 6.0, 
3.0], [4.0, 4.0, 7.0], [5.0, 8.0, 3.0]], [[2.0, 5.0, 2.0], [3.0, 1.0, 7.0], [1.0, 6.0, 7.0]], [[2.0, 3.0, 1.0], [1.0, 2.0, 
8.0], [7.0, 6.0, 4.0]]], [[[5.0, 7.0, 5.0], [5.0, 3.0, 6.0], [7.0, 3.0, 8.0]], [[1.0, 2.0, 1.0], [2.0, 3.0, 2.0], [5.0, 3.0, 
7.0]], [[5.0, 3.0, 2.0], [8.0, 3.0, 8.0], [3.0, 2.0, 3.0]]]]))
      val=Data(numpy.array([[6.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 8.0]]))
      arg[2,1,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 1.0], [5.0, 8.0, 8.0], [6.0, 8.0, 6.0]], [[4.0, 1.0, 6.0], [8.0, 7.0, 3.0], [1.0, 
6.0, 3.0]], [[3.0, 2.0, 6.0], [4.0, 6.0, 7.0], [7.0, 4.0, 5.0]]], [[[6.0, 1.0, 8.0], [5.0, 1.0, 8.0], [7.0, 8.0, 8.0]], [[4.0, 
3.0, 7.0], [4.0, 1.0, 8.0], [8.0, 3.0, 5.0]], [[8.0, 4.0, 2.0], [8.0, 6.0, 3.0], [2.0, 7.0, 3.0]]], [[[1.0, 3.0, 8.0], [8.0, 
4.0, 7.0], [4.0, 3.0, 7.0]], [[8.0, 7.0, 5.0], [6.0, 2.0, 2.0], [3.0, 3.0, 1.0]], [[2.0, 1.0, 8.0], [3.0, 1.0, 7.0], [8.0, 4.0, 
5.0]]], [[[1.0, 4.0, 7.0], [8.0, 8.0, 5.0], [5.0, 3.0, 4.0]], [[2.0, 5.0, 5.0], [8.0, 4.0, 4.0], [2.0, 2.0, 8.0]], [[6.0, 5.0, 
3.0], [5.0, 6.0, 6.0], [1.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 1.0], [8.0, 3.0, 7.0], [3.0, 2.0, 6.0]], 
[[3.0, 1.0, 8.0], [2.0, 4.0, 7.0], [5.0, 8.0, 8.0]], [[7.0, 8.0, 1.0], [2.0, 2.0, 5.0], [5.0, 3.0, 7.0]]], [[[7.0, 5.0, 6.0], 
[5.0, 4.0, 5.0], [6.0, 5.0, 2.0]], [[7.0, 5.0, 8.0], [1.0, 1.0, 7.0], [6.0, 4.0, 7.0]], [[6.0, 2.0, 7.0], [6.0, 5.0, 6.0], 
[8.0, 3.0, 1.0]]], [[[3.0, 6.0, 3.0], [4.0, 4.0, 7.0], [5.0, 8.0, 3.0]], [[2.0, 5.0, 2.0], [7.0, 8.0, 7.0], [1.0, 6.0, 7.0]], 
[[2.0, 3.0, 1.0], [1.0, 2.0, 8.0], [7.0, 6.0, 4.0]]], [[[5.0, 7.0, 5.0], [5.0, 3.0, 6.0], [7.0, 3.0, 8.0]], [[1.0, 2.0, 1.0], 
[2.0, 3.0, 2.0], [5.0, 3.0, 7.0]], [[5.0, 3.0, 2.0], [8.0, 3.0, 8.0], [3.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPRU(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 8.0], [2.0, 7.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 6.0, 8.0], [3.0, 1.0, 2.0], [4.0, 7.0, 
7.0]], [[4.0, 7.0, 4.0], [1.0, 7.0, 2.0], [4.0, 3.0, 5.0]]], [[[1.0, 8.0, 2.0], [8.0, 4.0, 2.0], [5.0, 1.0, 4.0]], [[1.0, 3.0, 
3.0], [6.0, 7.0, 4.0], [6.0, 2.0, 7.0]], [[4.0, 5.0, 1.0], [3.0, 7.0, 5.0], [8.0, 5.0, 4.0]]], [[[3.0, 6.0, 8.0], [7.0, 4.0, 
4.0], [4.0, 5.0, 3.0]], [[5.0, 6.0, 3.0], [8.0, 3.0, 5.0], [5.0, 7.0, 7.0]], [[7.0, 3.0, 1.0], [8.0, 7.0, 2.0], [1.0, 2.0, 
1.0]]], [[[4.0, 5.0, 3.0], [5.0, 2.0, 4.0], [5.0, 6.0, 6.0]], [[5.0, 5.0, 8.0], [7.0, 2.0, 1.0], [8.0, 4.0, 2.0]], [[2.0, 4.0, 
8.0], [2.0, 5.0, 1.0], [7.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 6.0], [8.0, 3.0, 5.0], [3.0, 4.0, 8.0]], [[6.0, 7.0, 1.0], [1.0, 4.0, 
2.0], [5.0, 4.0, 6.0]], [[3.0, 7.0, 3.0], [4.0, 1.0, 5.0], [4.0, 1.0, 4.0]]], [[[5.0, 5.0, 7.0], [5.0, 8.0, 5.0], [5.0, 2.0, 
2.0]], [[1.0, 7.0, 2.0], [4.0, 7.0, 6.0], [7.0, 5.0, 8.0]], [[8.0, 8.0, 8.0], [5.0, 3.0, 6.0], [5.0, 8.0, 5.0]]], [[[5.0, 5.0, 
4.0], [6.0, 5.0, 2.0], [4.0, 3.0, 2.0]], [[3.0, 7.0, 4.0], [3.0, 5.0, 5.0], [4.0, 6.0, 5.0]], [[7.0, 4.0, 8.0], [1.0, 6.0, 
5.0], [8.0, 1.0, 5.0]]], [[[8.0, 2.0, 4.0], [7.0, 5.0, 7.0], [1.0, 2.0, 3.0]], [[6.0, 6.0, 7.0], [8.0, 2.0, 6.0], [4.0, 7.0, 
8.0]], [[4.0, 3.0, 5.0], [1.0, 4.0, 7.0], [5.0, 7.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[8.0, 8.0]])+(1.-msk_val)*numpy.array([[1.0, 3.0]])
      arg[2,1,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 8.0], [2.0, 7.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 6.0, 8.0], [3.0, 1.0, 2.0], [4.0, 
7.0, 7.0]], [[4.0, 7.0, 4.0], [1.0, 7.0, 2.0], [4.0, 3.0, 5.0]]], [[[1.0, 8.0, 2.0], [8.0, 4.0, 2.0], [5.0, 1.0, 4.0]], [[1.0, 
3.0, 3.0], [6.0, 7.0, 4.0], [6.0, 2.0, 7.0]], [[4.0, 5.0, 1.0], [3.0, 7.0, 5.0], [8.0, 5.0, 4.0]]], [[[3.0, 6.0, 8.0], [7.0, 
4.0, 4.0], [4.0, 5.0, 3.0]], [[5.0, 6.0, 3.0], [8.0, 8.0, 5.0], [5.0, 7.0, 7.0]], [[7.0, 3.0, 1.0], [8.0, 7.0, 2.0], [1.0, 2.0, 
1.0]]], [[[4.0, 5.0, 3.0], [5.0, 2.0, 4.0], [5.0, 6.0, 6.0]], [[5.0, 5.0, 8.0], [7.0, 2.0, 1.0], [8.0, 4.0, 2.0]], [[2.0, 4.0, 
8.0], [2.0, 5.0, 1.0], [7.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 6.0], [8.0, 3.0, 5.0], [3.0, 4.0, 8.0]], 
[[6.0, 7.0, 1.0], [1.0, 4.0, 2.0], [5.0, 4.0, 6.0]], [[3.0, 7.0, 3.0], [4.0, 1.0, 5.0], [4.0, 1.0, 4.0]]], [[[5.0, 5.0, 7.0], 
[5.0, 8.0, 5.0], [5.0, 2.0, 2.0]], [[1.0, 7.0, 2.0], [4.0, 7.0, 6.0], [7.0, 5.0, 8.0]], [[8.0, 8.0, 8.0], [5.0, 3.0, 6.0], 
[5.0, 8.0, 5.0]]], [[[5.0, 5.0, 4.0], [6.0, 5.0, 2.0], [4.0, 3.0, 2.0]], [[3.0, 7.0, 4.0], [1.0, 3.0, 5.0], [4.0, 6.0, 5.0]], 
[[7.0, 4.0, 8.0], [1.0, 6.0, 5.0], [8.0, 1.0, 5.0]]], [[[8.0, 2.0, 4.0], [7.0, 5.0, 7.0], [1.0, 2.0, 3.0]], [[6.0, 6.0, 7.0], 
[8.0, 2.0, 6.0], [4.0, 7.0, 8.0]], [[4.0, 3.0, 5.0], [1.0, 4.0, 7.0], [5.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCCP(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 2.0], [7.0, 3.0, 7.0], [7.0, 4.0, 4.0]], [[3.0, 5.0, 5.0], [8.0, 8.0, 8.0], [4.0, 1.0, 
8.0]], [[2.0, 7.0, 3.0], [6.0, 7.0, 8.0], [2.0, 7.0, 5.0]]], [[[4.0, 6.0, 6.0], [8.0, 7.0, 5.0], [2.0, 4.0, 2.0]], [[2.0, 7.0, 
6.0], [5.0, 4.0, 5.0], [6.0, 1.0, 7.0]], [[4.0, 7.0, 8.0], [6.0, 6.0, 8.0], [2.0, 2.0, 8.0]]], [[[6.0, 6.0, 7.0], [1.0, 5.0, 
2.0], [8.0, 2.0, 5.0]], [[8.0, 4.0, 8.0], [7.0, 6.0, 6.0], [7.0, 3.0, 3.0]], [[3.0, 3.0, 7.0], [2.0, 4.0, 7.0], [8.0, 4.0, 
4.0]]], [[[3.0, 8.0, 3.0], [5.0, 7.0, 2.0], [7.0, 6.0, 5.0]], [[2.0, 4.0, 8.0], [1.0, 2.0, 2.0], [5.0, 2.0, 8.0]], [[6.0, 4.0, 
5.0], [6.0, 2.0, 5.0], [8.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 5.0], [8.0, 2.0, 1.0], [2.0, 1.0, 8.0]], [[6.0, 6.0, 7.0], [3.0, 6.0, 
5.0], [8.0, 5.0, 1.0]], [[1.0, 8.0, 4.0], [5.0, 2.0, 5.0], [3.0, 4.0, 7.0]]], [[[1.0, 3.0, 4.0], [2.0, 2.0, 4.0], [3.0, 2.0, 
6.0]], [[3.0, 2.0, 5.0], [3.0, 4.0, 2.0], [7.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [4.0, 7.0, 6.0], [1.0, 6.0, 5.0]]], [[[4.0, 4.0, 
2.0], [2.0, 2.0, 3.0], [5.0, 2.0, 1.0]], [[3.0, 7.0, 1.0], [6.0, 2.0, 7.0], [7.0, 7.0, 7.0]], [[1.0, 5.0, 5.0], [1.0, 5.0, 
5.0], [4.0, 6.0, 7.0]]], [[[5.0, 7.0, 3.0], [8.0, 2.0, 7.0], [3.0, 8.0, 7.0]], [[8.0, 8.0, 7.0], [5.0, 5.0, 5.0], [6.0, 6.0, 
5.0]], [[2.0, 4.0, 2.0], [5.0, 3.0, 2.0], [5.0, 3.0, 3.0]]]]))
      val=numpy.array([[[7.0, 6.0, 5.0], [4.0, 5.0, 6.0], [4.0, 6.0, 1.0]], [[7.0, 7.0, 1.0], [1.0, 6.0, 3.0], [3.0, 8.0, 
3.0]], [[5.0, 8.0, 5.0], [6.0, 7.0, 8.0], [1.0, 3.0, 2.0]], [[2.0, 4.0, 1.0], [2.0, 3.0, 6.0], [8.0, 3.0, 8.0]]])
      arg[:,:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 2.0], [7.0, 6.0, 7.0], [7.0, 5.0, 4.0]], [[3.0, 4.0, 5.0], [8.0, 5.0, 8.0], [4.0, 
6.0, 8.0]], [[2.0, 4.0, 3.0], [6.0, 6.0, 8.0], [2.0, 1.0, 5.0]]], [[[4.0, 7.0, 6.0], [8.0, 7.0, 5.0], [2.0, 1.0, 2.0]], [[2.0, 
1.0, 6.0], [5.0, 6.0, 5.0], [6.0, 3.0, 7.0]], [[4.0, 3.0, 8.0], [6.0, 8.0, 8.0], [2.0, 3.0, 8.0]]], [[[6.0, 5.0, 7.0], [1.0, 
8.0, 2.0], [8.0, 5.0, 5.0]], [[8.0, 6.0, 8.0], [7.0, 7.0, 6.0], [7.0, 8.0, 3.0]], [[3.0, 1.0, 7.0], [2.0, 3.0, 7.0], [8.0, 2.0, 
4.0]]], [[[3.0, 2.0, 3.0], [5.0, 4.0, 2.0], [7.0, 1.0, 5.0]], [[2.0, 2.0, 8.0], [1.0, 3.0, 2.0], [5.0, 6.0, 8.0]], [[6.0, 8.0, 
5.0], [6.0, 3.0, 5.0], [8.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 5.0], [8.0, 6.0, 1.0], [2.0, 5.0, 8.0]], 
[[6.0, 4.0, 7.0], [3.0, 5.0, 5.0], [8.0, 6.0, 1.0]], [[1.0, 4.0, 4.0], [5.0, 6.0, 5.0], [3.0, 1.0, 7.0]]], [[[1.0, 7.0, 4.0], 
[2.0, 7.0, 4.0], [3.0, 1.0, 6.0]], [[3.0, 1.0, 5.0], [3.0, 6.0, 2.0], [7.0, 3.0, 1.0]], [[7.0, 3.0, 6.0], [4.0, 8.0, 6.0], 
[1.0, 3.0, 5.0]]], [[[4.0, 5.0, 2.0], [2.0, 8.0, 3.0], [5.0, 5.0, 1.0]], [[3.0, 6.0, 1.0], [6.0, 7.0, 7.0], [7.0, 8.0, 7.0]], 
[[1.0, 1.0, 5.0], [1.0, 3.0, 5.0], [4.0, 2.0, 7.0]]], [[[5.0, 2.0, 3.0], [8.0, 4.0, 7.0], [3.0, 1.0, 7.0]], [[8.0, 2.0, 7.0], 
[5.0, 3.0, 5.0], [6.0, 6.0, 5.0]], [[2.0, 8.0, 2.0], [5.0, 3.0, 2.0], [5.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCCP(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 1.0], [7.0, 5.0, 7.0], [7.0, 5.0, 5.0]], [[1.0, 7.0, 3.0], [8.0, 2.0, 2.0], [7.0, 8.0, 
5.0]], [[2.0, 5.0, 8.0], [7.0, 7.0, 4.0], [1.0, 5.0, 3.0]]], [[[5.0, 7.0, 7.0], [7.0, 7.0, 3.0], [1.0, 8.0, 5.0]], [[5.0, 6.0, 
3.0], [4.0, 3.0, 8.0], [5.0, 6.0, 3.0]], [[6.0, 6.0, 3.0], [5.0, 8.0, 3.0], [1.0, 7.0, 7.0]]], [[[4.0, 2.0, 8.0], [4.0, 7.0, 
6.0], [1.0, 8.0, 3.0]], [[2.0, 1.0, 3.0], [2.0, 8.0, 5.0], [6.0, 4.0, 2.0]], [[7.0, 5.0, 1.0], [6.0, 5.0, 4.0], [1.0, 8.0, 
5.0]]], [[[4.0, 3.0, 8.0], [1.0, 2.0, 1.0], [6.0, 8.0, 2.0]], [[7.0, 2.0, 1.0], [8.0, 3.0, 7.0], [1.0, 1.0, 5.0]], [[6.0, 5.0, 
3.0], [4.0, 3.0, 3.0], [4.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 2.0], [5.0, 1.0, 1.0], [4.0, 2.0, 8.0]], [[2.0, 8.0, 1.0], [8.0, 5.0, 
4.0], [6.0, 5.0, 7.0]], [[3.0, 4.0, 8.0], [2.0, 7.0, 7.0], [4.0, 4.0, 7.0]]], [[[7.0, 1.0, 6.0], [5.0, 7.0, 3.0], [1.0, 6.0, 
1.0]], [[4.0, 8.0, 3.0], [7.0, 4.0, 7.0], [6.0, 7.0, 3.0]], [[2.0, 8.0, 2.0], [8.0, 6.0, 1.0], [4.0, 2.0, 1.0]]], [[[5.0, 1.0, 
1.0], [4.0, 2.0, 2.0], [8.0, 6.0, 1.0]], [[6.0, 2.0, 7.0], [6.0, 4.0, 2.0], [5.0, 4.0, 1.0]], [[8.0, 3.0, 6.0], [8.0, 6.0, 
5.0], [4.0, 5.0, 8.0]]], [[[4.0, 8.0, 6.0], [7.0, 6.0, 2.0], [6.0, 2.0, 7.0]], [[5.0, 1.0, 3.0], [2.0, 1.0, 4.0], [4.0, 3.0, 
2.0]], [[1.0, 8.0, 8.0], [1.0, 4.0, 5.0], [2.0, 6.0, 5.0]]]]))
      val=Data(numpy.array([[[8.0, 6.0, 4.0], [6.0, 1.0, 1.0], [3.0, 1.0, 6.0]], [[2.0, 5.0, 7.0], [7.0, 2.0, 2.0], [2.0, 8.0, 
8.0]], [[2.0, 3.0, 2.0], [5.0, 6.0, 6.0], [5.0, 5.0, 4.0]], [[5.0, 6.0, 2.0], [6.0, 2.0, 1.0], [2.0, 7.0, 
7.0]]]),self.functionspace)
      arg[:,:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 1.0], [7.0, 6.0, 7.0], [7.0, 4.0, 5.0]], [[1.0, 6.0, 3.0], [8.0, 1.0, 2.0], [7.0, 
1.0, 5.0]], [[2.0, 3.0, 8.0], [7.0, 1.0, 4.0], [1.0, 6.0, 3.0]]], [[[5.0, 2.0, 7.0], [7.0, 5.0, 3.0], [1.0, 7.0, 5.0]], [[5.0, 
7.0, 3.0], [4.0, 2.0, 8.0], [5.0, 2.0, 3.0]], [[6.0, 2.0, 3.0], [5.0, 8.0, 3.0], [1.0, 8.0, 7.0]]], [[[4.0, 2.0, 8.0], [4.0, 
3.0, 6.0], [1.0, 2.0, 3.0]], [[2.0, 5.0, 3.0], [2.0, 6.0, 5.0], [6.0, 6.0, 2.0]], [[7.0, 5.0, 1.0], [6.0, 5.0, 4.0], [1.0, 4.0, 
5.0]]], [[[4.0, 5.0, 8.0], [1.0, 6.0, 1.0], [6.0, 2.0, 2.0]], [[7.0, 6.0, 1.0], [8.0, 2.0, 7.0], [1.0, 1.0, 5.0]], [[6.0, 2.0, 
3.0], [4.0, 7.0, 3.0], [4.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 2.0], [5.0, 6.0, 1.0], [4.0, 4.0, 8.0]], 
[[2.0, 6.0, 1.0], [8.0, 1.0, 4.0], [6.0, 1.0, 7.0]], [[3.0, 3.0, 8.0], [2.0, 1.0, 7.0], [4.0, 6.0, 7.0]]], [[[7.0, 2.0, 6.0], 
[5.0, 5.0, 3.0], [1.0, 7.0, 1.0]], [[4.0, 7.0, 3.0], [7.0, 2.0, 7.0], [6.0, 2.0, 3.0]], [[2.0, 2.0, 2.0], [8.0, 8.0, 1.0], 
[4.0, 8.0, 1.0]]], [[[5.0, 2.0, 1.0], [4.0, 3.0, 2.0], [8.0, 2.0, 1.0]], [[6.0, 5.0, 7.0], [6.0, 6.0, 2.0], [5.0, 6.0, 1.0]], 
[[8.0, 5.0, 6.0], [8.0, 5.0, 5.0], [4.0, 4.0, 8.0]]], [[[4.0, 5.0, 6.0], [7.0, 6.0, 2.0], [6.0, 2.0, 7.0]], [[5.0, 6.0, 3.0], 
[2.0, 2.0, 4.0], [4.0, 1.0, 2.0]], [[1.0, 2.0, 8.0], [1.0, 7.0, 5.0], [2.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCCP(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 8.0], [3.0, 1.0, 1.0], [3.0, 2.0, 5.0]], [[3.0, 7.0, 4.0], [8.0, 3.0, 5.0], [6.0, 6.0, 
4.0]], [[8.0, 6.0, 7.0], [7.0, 2.0, 2.0], [2.0, 5.0, 6.0]]], [[[5.0, 8.0, 5.0], [6.0, 8.0, 8.0], [3.0, 4.0, 7.0]], [[8.0, 2.0, 
8.0], [6.0, 7.0, 6.0], [2.0, 1.0, 6.0]], [[4.0, 1.0, 3.0], [1.0, 4.0, 1.0], [4.0, 4.0, 6.0]]], [[[4.0, 3.0, 4.0], [6.0, 2.0, 
1.0], [1.0, 6.0, 3.0]], [[5.0, 5.0, 2.0], [6.0, 7.0, 8.0], [1.0, 8.0, 3.0]], [[2.0, 1.0, 6.0], [8.0, 8.0, 1.0], [3.0, 5.0, 
7.0]]], [[[7.0, 5.0, 5.0], [3.0, 6.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 6.0, 7.0], [5.0, 4.0, 2.0]], [[5.0, 5.0, 
7.0], [4.0, 4.0, 3.0], [2.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 2.0], [7.0, 4.0, 6.0], [6.0, 5.0, 5.0]], [[2.0, 2.0, 3.0], [8.0, 7.0, 
2.0], [2.0, 2.0, 3.0]], [[8.0, 6.0, 2.0], [4.0, 5.0, 2.0], [4.0, 6.0, 4.0]]], [[[1.0, 7.0, 5.0], [7.0, 7.0, 2.0], [3.0, 5.0, 
5.0]], [[1.0, 6.0, 7.0], [5.0, 4.0, 5.0], [4.0, 7.0, 1.0]], [[8.0, 4.0, 1.0], [1.0, 6.0, 8.0], [5.0, 3.0, 5.0]]], [[[7.0, 4.0, 
8.0], [1.0, 6.0, 2.0], [3.0, 5.0, 3.0]], [[3.0, 8.0, 7.0], [1.0, 4.0, 7.0], [6.0, 3.0, 7.0]], [[6.0, 8.0, 3.0], [4.0, 7.0, 
7.0], [6.0, 6.0, 8.0]]], [[[2.0, 1.0, 5.0], [6.0, 7.0, 6.0], [4.0, 6.0, 1.0]], [[2.0, 8.0, 8.0], [7.0, 4.0, 2.0], [4.0, 5.0, 
1.0]], [[3.0, 6.0, 7.0], [7.0, 1.0, 5.0], [6.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[3.0, 2.0, 7.0], [3.0, 2.0, 8.0], [3.0, 3.0, 5.0]], [[4.0, 4.0, 2.0], [1.0, 1.0, 4.0], [1.0, 2.0, 
7.0]], [[7.0, 7.0, 2.0], [2.0, 8.0, 2.0], [2.0, 5.0, 5.0]], [[3.0, 1.0, 4.0], [1.0, 1.0, 7.0], [8.0, 3.0, 
2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 4.0, 7.0], [8.0, 6.0, 3.0], [2.0, 5.0, 7.0]], [[7.0, 2.0, 6.0], [7.0, 8.0, 4.0], 
[2.0, 8.0, 2.0]], [[7.0, 3.0, 3.0], [3.0, 2.0, 4.0], [8.0, 4.0, 1.0]], [[5.0, 4.0, 1.0], [8.0, 3.0, 7.0], [6.0, 7.0, 3.0]]]))
      arg[:,:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 8.0], [3.0, 2.0, 1.0], [3.0, 7.0, 5.0]], [[3.0, 3.0, 4.0], [8.0, 2.0, 5.0], [6.0, 
8.0, 4.0]], [[8.0, 3.0, 7.0], [7.0, 3.0, 2.0], [2.0, 5.0, 6.0]]], [[[5.0, 4.0, 5.0], [6.0, 4.0, 8.0], [3.0, 2.0, 7.0]], [[8.0, 
1.0, 8.0], [6.0, 1.0, 6.0], [2.0, 4.0, 6.0]], [[4.0, 1.0, 3.0], [1.0, 2.0, 1.0], [4.0, 7.0, 6.0]]], [[[4.0, 7.0, 4.0], [6.0, 
7.0, 1.0], [1.0, 2.0, 3.0]], [[5.0, 2.0, 2.0], [6.0, 8.0, 8.0], [1.0, 2.0, 3.0]], [[2.0, 2.0, 6.0], [8.0, 5.0, 1.0], [3.0, 5.0, 
7.0]]], [[[7.0, 3.0, 5.0], [3.0, 1.0, 3.0], [2.0, 4.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 1.0, 7.0], [5.0, 7.0, 2.0]], [[5.0, 8.0, 
7.0], [4.0, 3.0, 3.0], [2.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 2.0], [7.0, 4.0, 6.0], [6.0, 7.0, 5.0]], 
[[2.0, 8.0, 3.0], [8.0, 6.0, 2.0], [2.0, 3.0, 3.0]], [[8.0, 2.0, 2.0], [4.0, 5.0, 2.0], [4.0, 7.0, 4.0]]], [[[1.0, 7.0, 5.0], 
[7.0, 2.0, 2.0], [3.0, 6.0, 5.0]], [[1.0, 7.0, 7.0], [5.0, 8.0, 5.0], [4.0, 4.0, 1.0]], [[8.0, 2.0, 1.0], [1.0, 8.0, 8.0], 
[5.0, 2.0, 5.0]]], [[[7.0, 7.0, 8.0], [1.0, 3.0, 2.0], [3.0, 3.0, 3.0]], [[3.0, 3.0, 7.0], [1.0, 2.0, 7.0], [6.0, 4.0, 7.0]], 
[[6.0, 8.0, 3.0], [4.0, 4.0, 7.0], [6.0, 1.0, 8.0]]], [[[2.0, 5.0, 5.0], [6.0, 4.0, 6.0], [4.0, 1.0, 1.0]], [[2.0, 8.0, 8.0], 
[7.0, 3.0, 2.0], [4.0, 7.0, 1.0]], [[3.0, 6.0, 7.0], [7.0, 7.0, 5.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCCP(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 1.0], [2.0, 7.0, 5.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 7.0], [1.0, 8.0, 7.0], [6.0, 7.0, 
2.0]], [[5.0, 2.0, 7.0], [8.0, 3.0, 6.0], [3.0, 2.0, 7.0]]], [[[2.0, 2.0, 4.0], [2.0, 7.0, 8.0], [8.0, 1.0, 8.0]], [[8.0, 4.0, 
5.0], [1.0, 1.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 7.0, 1.0], [7.0, 7.0, 2.0], [4.0, 5.0, 5.0]]], [[[8.0, 6.0, 1.0], [2.0, 5.0, 
6.0], [2.0, 6.0, 4.0]], [[2.0, 1.0, 5.0], [3.0, 1.0, 1.0], [4.0, 4.0, 1.0]], [[2.0, 5.0, 4.0], [6.0, 3.0, 2.0], [3.0, 1.0, 
4.0]]], [[[4.0, 2.0, 1.0], [1.0, 3.0, 7.0], [6.0, 2.0, 1.0]], [[4.0, 4.0, 7.0], [6.0, 1.0, 7.0], [3.0, 5.0, 2.0]], [[1.0, 6.0, 
1.0], [6.0, 3.0, 5.0], [5.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 7.0], [2.0, 7.0, 1.0], [4.0, 2.0, 8.0]], [[7.0, 5.0, 8.0], [2.0, 8.0, 
7.0], [8.0, 3.0, 6.0]], [[6.0, 2.0, 3.0], [7.0, 3.0, 1.0], [3.0, 1.0, 6.0]]], [[[4.0, 7.0, 4.0], [6.0, 1.0, 2.0], [5.0, 2.0, 
6.0]], [[2.0, 7.0, 5.0], [2.0, 7.0, 1.0], [3.0, 1.0, 5.0]], [[1.0, 4.0, 5.0], [6.0, 1.0, 7.0], [7.0, 4.0, 8.0]]], [[[1.0, 3.0, 
3.0], [3.0, 4.0, 5.0], [3.0, 5.0, 5.0]], [[2.0, 1.0, 4.0], [1.0, 7.0, 2.0], [8.0, 7.0, 1.0]], [[2.0, 4.0, 8.0], [2.0, 4.0, 
2.0], [3.0, 4.0, 7.0]]], [[[3.0, 6.0, 2.0], [4.0, 8.0, 7.0], [3.0, 8.0, 6.0]], [[4.0, 1.0, 5.0], [1.0, 7.0, 5.0], [7.0, 4.0, 
4.0]], [[2.0, 5.0, 8.0], [5.0, 2.0, 8.0], [7.0, 1.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 5.0, 3.0], [3.0, 3.0, 1.0], [5.0, 4.0, 8.0]], [[2.0, 6.0, 5.0], [6.0, 5.0, 2.0], [7.0, 
7.0, 3.0]], [[7.0, 2.0, 1.0], [6.0, 6.0, 8.0], [5.0, 3.0, 6.0]], [[2.0, 1.0, 1.0], [2.0, 5.0, 7.0], [2.0, 1.0, 
2.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 3.0], [3.0, 7.0, 4.0], [7.0, 2.0, 2.0]], [[7.0, 7.0, 3.0], [1.0, 2.0, 6.0], [3.0, 
4.0, 8.0]], [[4.0, 1.0, 8.0], [2.0, 8.0, 8.0], [2.0, 4.0, 3.0]], [[1.0, 5.0, 1.0], [4.0, 5.0, 3.0], [4.0, 5.0, 6.0]]])
      arg[:,:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 1.0], [2.0, 5.0, 5.0], [4.0, 3.0, 2.0]], [[6.0, 3.0, 7.0], [1.0, 3.0, 7.0], [6.0, 
1.0, 2.0]], [[5.0, 5.0, 7.0], [8.0, 4.0, 6.0], [3.0, 8.0, 7.0]]], [[[2.0, 2.0, 4.0], [2.0, 6.0, 8.0], [8.0, 5.0, 8.0]], [[8.0, 
6.0, 5.0], [1.0, 5.0, 1.0], [5.0, 2.0, 3.0]], [[5.0, 7.0, 1.0], [7.0, 7.0, 2.0], [4.0, 3.0, 5.0]]], [[[8.0, 7.0, 1.0], [2.0, 
2.0, 6.0], [2.0, 1.0, 4.0]], [[2.0, 6.0, 5.0], [3.0, 6.0, 1.0], [4.0, 8.0, 1.0]], [[2.0, 5.0, 4.0], [6.0, 3.0, 2.0], [3.0, 6.0, 
4.0]]], [[[4.0, 2.0, 1.0], [1.0, 1.0, 7.0], [6.0, 1.0, 1.0]], [[4.0, 2.0, 7.0], [6.0, 5.0, 7.0], [3.0, 7.0, 2.0]], [[1.0, 2.0, 
1.0], [6.0, 1.0, 5.0], [5.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 7.0], [2.0, 6.0, 1.0], [4.0, 3.0, 8.0]], 
[[7.0, 3.0, 8.0], [2.0, 7.0, 7.0], [8.0, 4.0, 6.0]], [[6.0, 7.0, 3.0], [7.0, 2.0, 1.0], [3.0, 2.0, 6.0]]], [[[4.0, 7.0, 4.0], 
[6.0, 7.0, 2.0], [5.0, 3.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 2.0, 1.0], [3.0, 6.0, 5.0]], [[1.0, 3.0, 5.0], [6.0, 4.0, 7.0], 
[7.0, 8.0, 8.0]]], [[[1.0, 4.0, 3.0], [3.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[2.0, 2.0, 4.0], [1.0, 8.0, 2.0], [8.0, 8.0, 1.0]], 
[[2.0, 2.0, 8.0], [2.0, 4.0, 2.0], [3.0, 3.0, 7.0]]], [[[3.0, 1.0, 2.0], [4.0, 5.0, 7.0], [3.0, 1.0, 6.0]], [[4.0, 4.0, 5.0], 
[1.0, 5.0, 5.0], [7.0, 3.0, 4.0]], [[2.0, 4.0, 8.0], [5.0, 5.0, 8.0], [7.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LRUR(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 8.0], [7.0, 3.0, 7.0], [7.0, 1.0, 5.0]], [[8.0, 6.0, 4.0], [4.0, 4.0, 7.0], [1.0, 1.0, 
3.0]], [[4.0, 5.0, 8.0], [8.0, 3.0, 2.0], [7.0, 4.0, 7.0]]], [[[6.0, 1.0, 8.0], [4.0, 7.0, 7.0], [7.0, 6.0, 4.0]], [[6.0, 2.0, 
4.0], [2.0, 6.0, 2.0], [3.0, 3.0, 2.0]], [[4.0, 3.0, 5.0], [2.0, 6.0, 5.0], [3.0, 7.0, 1.0]]], [[[6.0, 4.0, 1.0], [3.0, 2.0, 
2.0], [2.0, 8.0, 2.0]], [[6.0, 4.0, 8.0], [1.0, 1.0, 6.0], [5.0, 4.0, 3.0]], [[2.0, 1.0, 5.0], [6.0, 6.0, 4.0], [3.0, 3.0, 
1.0]]], [[[8.0, 3.0, 1.0], [3.0, 6.0, 3.0], [8.0, 5.0, 1.0]], [[4.0, 7.0, 1.0], [8.0, 3.0, 6.0], [2.0, 1.0, 8.0]], [[2.0, 6.0, 
7.0], [7.0, 2.0, 8.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 2.0], [3.0, 3.0, 4.0], [1.0, 1.0, 1.0]], [[7.0, 3.0, 3.0], [8.0, 2.0, 
5.0], [4.0, 1.0, 1.0]], [[8.0, 4.0, 4.0], [2.0, 8.0, 2.0], [8.0, 2.0, 1.0]]], [[[3.0, 4.0, 4.0], [7.0, 1.0, 4.0], [3.0, 6.0, 
5.0]], [[3.0, 7.0, 6.0], [4.0, 6.0, 4.0], [6.0, 5.0, 1.0]], [[5.0, 6.0, 7.0], [8.0, 6.0, 7.0], [5.0, 2.0, 7.0]]], [[[5.0, 7.0, 
2.0], [1.0, 7.0, 7.0], [6.0, 1.0, 1.0]], [[6.0, 6.0, 2.0], [6.0, 3.0, 8.0], [5.0, 2.0, 3.0]], [[6.0, 8.0, 1.0], [7.0, 5.0, 
1.0], [2.0, 3.0, 8.0]]], [[[5.0, 5.0, 2.0], [7.0, 5.0, 7.0], [7.0, 4.0, 1.0]], [[5.0, 6.0, 5.0], [5.0, 6.0, 8.0], [2.0, 1.0, 
6.0]], [[8.0, 8.0, 7.0], [8.0, 7.0, 6.0], [7.0, 7.0, 1.0]]]]))
      val=numpy.array([[[[2.0], [8.0]]], [[[4.0], [2.0]]]])
      arg[2:,1:2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 8.0], [7.0, 3.0, 7.0], [7.0, 1.0, 5.0]], [[8.0, 6.0, 4.0], [4.0, 4.0, 7.0], [1.0, 
1.0, 3.0]], [[4.0, 5.0, 8.0], [8.0, 3.0, 2.0], [7.0, 4.0, 7.0]]], [[[6.0, 1.0, 8.0], [4.0, 7.0, 7.0], [7.0, 6.0, 4.0]], [[6.0, 
2.0, 4.0], [2.0, 6.0, 2.0], [3.0, 3.0, 2.0]], [[4.0, 3.0, 5.0], [2.0, 6.0, 5.0], [3.0, 7.0, 1.0]]], [[[6.0, 4.0, 1.0], [3.0, 
2.0, 2.0], [2.0, 8.0, 2.0]], [[6.0, 2.0, 8.0], [1.0, 8.0, 6.0], [5.0, 4.0, 3.0]], [[2.0, 1.0, 5.0], [6.0, 6.0, 4.0], [3.0, 3.0, 
1.0]]], [[[8.0, 3.0, 1.0], [3.0, 6.0, 3.0], [8.0, 5.0, 1.0]], [[4.0, 4.0, 1.0], [8.0, 2.0, 6.0], [2.0, 1.0, 8.0]], [[2.0, 6.0, 
7.0], [7.0, 2.0, 8.0], [2.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 2.0], [3.0, 3.0, 4.0], [1.0, 1.0, 1.0]], 
[[7.0, 3.0, 3.0], [8.0, 2.0, 5.0], [4.0, 1.0, 1.0]], [[8.0, 4.0, 4.0], [2.0, 8.0, 2.0], [8.0, 2.0, 1.0]]], [[[3.0, 4.0, 4.0], 
[7.0, 1.0, 4.0], [3.0, 6.0, 5.0]], [[3.0, 7.0, 6.0], [4.0, 6.0, 4.0], [6.0, 5.0, 1.0]], [[5.0, 6.0, 7.0], [8.0, 6.0, 7.0], 
[5.0, 2.0, 7.0]]], [[[5.0, 7.0, 2.0], [1.0, 7.0, 7.0], [6.0, 1.0, 1.0]], [[6.0, 2.0, 2.0], [6.0, 8.0, 8.0], [5.0, 2.0, 3.0]], 
[[6.0, 8.0, 1.0], [7.0, 5.0, 1.0], [2.0, 3.0, 8.0]]], [[[5.0, 5.0, 2.0], [7.0, 5.0, 7.0], [7.0, 4.0, 1.0]], [[5.0, 4.0, 5.0], 
[5.0, 2.0, 8.0], [2.0, 1.0, 6.0]], [[8.0, 8.0, 7.0], [8.0, 7.0, 6.0], [7.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LRUR(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 8.0], [4.0, 7.0, 3.0], [4.0, 6.0, 4.0]], [[7.0, 2.0, 7.0], [4.0, 6.0, 1.0], [5.0, 6.0, 
5.0]], [[1.0, 3.0, 4.0], [6.0, 7.0, 7.0], [7.0, 4.0, 4.0]]], [[[8.0, 4.0, 5.0], [7.0, 1.0, 4.0], [8.0, 7.0, 1.0]], [[1.0, 5.0, 
8.0], [8.0, 7.0, 7.0], [1.0, 6.0, 5.0]], [[6.0, 2.0, 8.0], [3.0, 8.0, 4.0], [2.0, 4.0, 4.0]]], [[[2.0, 4.0, 8.0], [5.0, 2.0, 
6.0], [4.0, 6.0, 6.0]], [[8.0, 8.0, 8.0], [2.0, 1.0, 4.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 7.0], [2.0, 1.0, 2.0], [3.0, 5.0, 
3.0]]], [[[2.0, 5.0, 8.0], [7.0, 6.0, 2.0], [6.0, 6.0, 7.0]], [[2.0, 4.0, 2.0], [6.0, 2.0, 8.0], [6.0, 6.0, 5.0]], [[2.0, 5.0, 
5.0], [4.0, 7.0, 1.0], [4.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 5.0], [8.0, 2.0, 6.0], [2.0, 3.0, 5.0]], [[1.0, 1.0, 6.0], [2.0, 8.0, 
4.0], [5.0, 1.0, 7.0]], [[7.0, 8.0, 7.0], [5.0, 8.0, 2.0], [5.0, 7.0, 8.0]]], [[[5.0, 5.0, 3.0], [3.0, 8.0, 6.0], [8.0, 8.0, 
8.0]], [[2.0, 1.0, 4.0], [1.0, 6.0, 7.0], [8.0, 2.0, 2.0]], [[2.0, 2.0, 3.0], [2.0, 7.0, 5.0], [2.0, 5.0, 7.0]]], [[[7.0, 2.0, 
2.0], [4.0, 2.0, 7.0], [6.0, 7.0, 7.0]], [[5.0, 7.0, 1.0], [6.0, 5.0, 1.0], [7.0, 3.0, 2.0]], [[2.0, 7.0, 4.0], [2.0, 3.0, 
5.0], [1.0, 5.0, 8.0]]], [[[4.0, 3.0, 3.0], [4.0, 7.0, 4.0], [2.0, 5.0, 3.0]], [[1.0, 1.0, 5.0], [2.0, 5.0, 1.0], [1.0, 8.0, 
6.0]], [[8.0, 8.0, 8.0], [6.0, 4.0, 7.0], [8.0, 5.0, 1.0]]]]))
      val=Data(numpy.array([[[[7.0], [4.0]]], [[[3.0], [3.0]]]]),self.functionspace)
      arg[2:,1:2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 8.0], [4.0, 7.0, 3.0], [4.0, 6.0, 4.0]], [[7.0, 2.0, 7.0], [4.0, 6.0, 1.0], [5.0, 
6.0, 5.0]], [[1.0, 3.0, 4.0], [6.0, 7.0, 7.0], [7.0, 4.0, 4.0]]], [[[8.0, 4.0, 5.0], [7.0, 1.0, 4.0], [8.0, 7.0, 1.0]], [[1.0, 
5.0, 8.0], [8.0, 7.0, 7.0], [1.0, 6.0, 5.0]], [[6.0, 2.0, 8.0], [3.0, 8.0, 4.0], [2.0, 4.0, 4.0]]], [[[2.0, 4.0, 8.0], [5.0, 
2.0, 6.0], [4.0, 6.0, 6.0]], [[8.0, 7.0, 8.0], [2.0, 4.0, 4.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 7.0], [2.0, 1.0, 2.0], [3.0, 5.0, 
3.0]]], [[[2.0, 5.0, 8.0], [7.0, 6.0, 2.0], [6.0, 6.0, 7.0]], [[2.0, 3.0, 2.0], [6.0, 3.0, 8.0], [6.0, 6.0, 5.0]], [[2.0, 5.0, 
5.0], [4.0, 7.0, 1.0], [4.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 5.0], [8.0, 2.0, 6.0], [2.0, 3.0, 5.0]], 
[[1.0, 1.0, 6.0], [2.0, 8.0, 4.0], [5.0, 1.0, 7.0]], [[7.0, 8.0, 7.0], [5.0, 8.0, 2.0], [5.0, 7.0, 8.0]]], [[[5.0, 5.0, 3.0], 
[3.0, 8.0, 6.0], [8.0, 8.0, 8.0]], [[2.0, 1.0, 4.0], [1.0, 6.0, 7.0], [8.0, 2.0, 2.0]], [[2.0, 2.0, 3.0], [2.0, 7.0, 5.0], 
[2.0, 5.0, 7.0]]], [[[7.0, 2.0, 2.0], [4.0, 2.0, 7.0], [6.0, 7.0, 7.0]], [[5.0, 7.0, 1.0], [6.0, 4.0, 1.0], [7.0, 3.0, 2.0]], 
[[2.0, 7.0, 4.0], [2.0, 3.0, 5.0], [1.0, 5.0, 8.0]]], [[[4.0, 3.0, 3.0], [4.0, 7.0, 4.0], [2.0, 5.0, 3.0]], [[1.0, 3.0, 5.0], 
[2.0, 3.0, 1.0], [1.0, 8.0, 6.0]], [[8.0, 8.0, 8.0], [6.0, 4.0, 7.0], [8.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LRUR(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 2.0], [1.0, 8.0, 7.0], [2.0, 7.0, 7.0]], [[5.0, 2.0, 2.0], [4.0, 6.0, 7.0], [4.0, 7.0, 
7.0]], [[1.0, 1.0, 5.0], [2.0, 4.0, 5.0], [5.0, 6.0, 1.0]]], [[[6.0, 8.0, 1.0], [7.0, 1.0, 1.0], [5.0, 4.0, 8.0]], [[1.0, 8.0, 
8.0], [1.0, 6.0, 4.0], [6.0, 2.0, 1.0]], [[1.0, 1.0, 8.0], [6.0, 3.0, 6.0], [7.0, 4.0, 6.0]]], [[[1.0, 4.0, 2.0], [4.0, 7.0, 
5.0], [1.0, 3.0, 5.0]], [[6.0, 6.0, 6.0], [8.0, 3.0, 2.0], [7.0, 4.0, 5.0]], [[8.0, 6.0, 5.0], [3.0, 3.0, 8.0], [1.0, 3.0, 
8.0]]], [[[1.0, 8.0, 5.0], [3.0, 8.0, 8.0], [8.0, 5.0, 6.0]], [[3.0, 3.0, 1.0], [6.0, 8.0, 3.0], [5.0, 5.0, 7.0]], [[6.0, 8.0, 
3.0], [4.0, 8.0, 8.0], [1.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 8.0], [1.0, 1.0, 5.0], [1.0, 3.0, 8.0]], [[6.0, 7.0, 5.0], [7.0, 1.0, 
5.0], [8.0, 2.0, 6.0]], [[1.0, 7.0, 7.0], [5.0, 1.0, 5.0], [7.0, 8.0, 8.0]]], [[[7.0, 6.0, 3.0], [7.0, 2.0, 8.0], [3.0, 7.0, 
6.0]], [[3.0, 8.0, 7.0], [4.0, 4.0, 2.0], [4.0, 7.0, 2.0]], [[7.0, 4.0, 5.0], [3.0, 4.0, 2.0], [8.0, 1.0, 3.0]]], [[[7.0, 7.0, 
6.0], [4.0, 5.0, 6.0], [7.0, 2.0, 3.0]], [[1.0, 6.0, 2.0], [8.0, 7.0, 7.0], [7.0, 3.0, 2.0]], [[3.0, 7.0, 4.0], [3.0, 1.0, 
8.0], [6.0, 5.0, 6.0]]], [[[3.0, 8.0, 8.0], [1.0, 6.0, 3.0], [8.0, 3.0, 7.0]], [[6.0, 1.0, 8.0], [7.0, 8.0, 3.0], [8.0, 7.0, 
2.0]], [[6.0, 6.0, 1.0], [3.0, 5.0, 8.0], [2.0, 1.0, 4.0]]]]))
      val=Data(numpy.array([[[[1.0], [6.0]]], [[[3.0], [2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0], [2.0]]], [[[7.0], [1.0]]]]))
      arg[2:,1:2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 2.0], [1.0, 8.0, 7.0], [2.0, 7.0, 7.0]], [[5.0, 2.0, 2.0], [4.0, 6.0, 7.0], [4.0, 
7.0, 7.0]], [[1.0, 1.0, 5.0], [2.0, 4.0, 5.0], [5.0, 6.0, 1.0]]], [[[6.0, 8.0, 1.0], [7.0, 1.0, 1.0], [5.0, 4.0, 8.0]], [[1.0, 
8.0, 8.0], [1.0, 6.0, 4.0], [6.0, 2.0, 1.0]], [[1.0, 1.0, 8.0], [6.0, 3.0, 6.0], [7.0, 4.0, 6.0]]], [[[1.0, 4.0, 2.0], [4.0, 
7.0, 5.0], [1.0, 3.0, 5.0]], [[6.0, 1.0, 6.0], [8.0, 6.0, 2.0], [7.0, 4.0, 5.0]], [[8.0, 6.0, 5.0], [3.0, 3.0, 8.0], [1.0, 3.0, 
8.0]]], [[[1.0, 8.0, 5.0], [3.0, 8.0, 8.0], [8.0, 5.0, 6.0]], [[3.0, 3.0, 1.0], [6.0, 2.0, 3.0], [5.0, 5.0, 7.0]], [[6.0, 8.0, 
3.0], [4.0, 8.0, 8.0], [1.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 8.0], [1.0, 1.0, 5.0], [1.0, 3.0, 8.0]], 
[[6.0, 7.0, 5.0], [7.0, 1.0, 5.0], [8.0, 2.0, 6.0]], [[1.0, 7.0, 7.0], [5.0, 1.0, 5.0], [7.0, 8.0, 8.0]]], [[[7.0, 6.0, 3.0], 
[7.0, 2.0, 8.0], [3.0, 7.0, 6.0]], [[3.0, 8.0, 7.0], [4.0, 4.0, 2.0], [4.0, 7.0, 2.0]], [[7.0, 4.0, 5.0], [3.0, 4.0, 2.0], 
[8.0, 1.0, 3.0]]], [[[7.0, 7.0, 6.0], [4.0, 5.0, 6.0], [7.0, 2.0, 3.0]], [[1.0, 5.0, 2.0], [8.0, 2.0, 7.0], [7.0, 3.0, 2.0]], 
[[3.0, 7.0, 4.0], [3.0, 1.0, 8.0], [6.0, 5.0, 6.0]]], [[[3.0, 8.0, 8.0], [1.0, 6.0, 3.0], [8.0, 3.0, 7.0]], [[6.0, 7.0, 8.0], 
[7.0, 1.0, 3.0], [8.0, 7.0, 2.0]], [[6.0, 6.0, 1.0], [3.0, 5.0, 8.0], [2.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LRUR(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 8.0], [8.0, 4.0, 8.0], [7.0, 4.0, 5.0]], [[3.0, 6.0, 3.0], [8.0, 7.0, 3.0], [7.0, 3.0, 
7.0]], [[3.0, 5.0, 7.0], [3.0, 5.0, 6.0], [2.0, 8.0, 1.0]]], [[[3.0, 7.0, 1.0], [5.0, 2.0, 5.0], [5.0, 2.0, 8.0]], [[5.0, 4.0, 
5.0], [5.0, 6.0, 5.0], [6.0, 4.0, 2.0]], [[7.0, 4.0, 3.0], [8.0, 5.0, 2.0], [2.0, 3.0, 2.0]]], [[[7.0, 5.0, 1.0], [8.0, 4.0, 
2.0], [5.0, 6.0, 4.0]], [[2.0, 3.0, 8.0], [2.0, 3.0, 4.0], [3.0, 7.0, 8.0]], [[2.0, 3.0, 5.0], [3.0, 2.0, 1.0], [5.0, 3.0, 
2.0]]], [[[7.0, 6.0, 6.0], [3.0, 4.0, 6.0], [6.0, 7.0, 4.0]], [[4.0, 8.0, 5.0], [8.0, 7.0, 1.0], [4.0, 7.0, 1.0]], [[3.0, 2.0, 
6.0], [6.0, 3.0, 6.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 4.0], [3.0, 3.0, 7.0], [5.0, 3.0, 1.0]], [[2.0, 1.0, 1.0], [8.0, 6.0, 
6.0], [7.0, 7.0, 7.0]], [[3.0, 8.0, 7.0], [8.0, 1.0, 4.0], [5.0, 2.0, 7.0]]], [[[1.0, 2.0, 6.0], [2.0, 7.0, 1.0], [7.0, 8.0, 
6.0]], [[5.0, 5.0, 5.0], [1.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[4.0, 7.0, 4.0], [2.0, 2.0, 3.0], [7.0, 3.0, 6.0]]], [[[1.0, 2.0, 
6.0], [2.0, 8.0, 3.0], [7.0, 5.0, 4.0]], [[1.0, 5.0, 1.0], [3.0, 2.0, 6.0], [6.0, 5.0, 3.0]], [[8.0, 2.0, 7.0], [7.0, 6.0, 
1.0], [6.0, 3.0, 3.0]]], [[[2.0, 4.0, 5.0], [1.0, 7.0, 6.0], [1.0, 8.0, 3.0]], [[1.0, 1.0, 7.0], [7.0, 5.0, 5.0], [7.0, 5.0, 
7.0]], [[6.0, 1.0, 5.0], [5.0, 6.0, 4.0], [4.0, 7.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0], [6.0]]], [[[8.0], [5.0]]]])+(1.-msk_val)*numpy.array([[[[3.0], [4.0]]], [[[5.0], 
[8.0]]]])
      arg[2:,1:2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 8.0], [8.0, 4.0, 8.0], [7.0, 4.0, 5.0]], [[3.0, 6.0, 3.0], [8.0, 7.0, 3.0], [7.0, 
3.0, 7.0]], [[3.0, 5.0, 7.0], [3.0, 5.0, 6.0], [2.0, 8.0, 1.0]]], [[[3.0, 7.0, 1.0], [5.0, 2.0, 5.0], [5.0, 2.0, 8.0]], [[5.0, 
4.0, 5.0], [5.0, 6.0, 5.0], [6.0, 4.0, 2.0]], [[7.0, 4.0, 3.0], [8.0, 5.0, 2.0], [2.0, 3.0, 2.0]]], [[[7.0, 5.0, 1.0], [8.0, 
4.0, 2.0], [5.0, 6.0, 4.0]], [[2.0, 1.0, 8.0], [2.0, 6.0, 4.0], [3.0, 7.0, 8.0]], [[2.0, 3.0, 5.0], [3.0, 2.0, 1.0], [5.0, 3.0, 
2.0]]], [[[7.0, 6.0, 6.0], [3.0, 4.0, 6.0], [6.0, 7.0, 4.0]], [[4.0, 8.0, 5.0], [8.0, 5.0, 1.0], [4.0, 7.0, 1.0]], [[3.0, 2.0, 
6.0], [6.0, 3.0, 6.0], [6.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 4.0], [3.0, 3.0, 7.0], [5.0, 3.0, 1.0]], 
[[2.0, 1.0, 1.0], [8.0, 6.0, 6.0], [7.0, 7.0, 7.0]], [[3.0, 8.0, 7.0], [8.0, 1.0, 4.0], [5.0, 2.0, 7.0]]], [[[1.0, 2.0, 6.0], 
[2.0, 7.0, 1.0], [7.0, 8.0, 6.0]], [[5.0, 5.0, 5.0], [1.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[4.0, 7.0, 4.0], [2.0, 2.0, 3.0], 
[7.0, 3.0, 6.0]]], [[[1.0, 2.0, 6.0], [2.0, 8.0, 3.0], [7.0, 5.0, 4.0]], [[1.0, 3.0, 1.0], [3.0, 4.0, 6.0], [6.0, 5.0, 3.0]], 
[[8.0, 2.0, 7.0], [7.0, 6.0, 1.0], [6.0, 3.0, 3.0]]], [[[2.0, 4.0, 5.0], [1.0, 7.0, 6.0], [1.0, 8.0, 3.0]], [[1.0, 5.0, 7.0], 
[7.0, 8.0, 5.0], [7.0, 5.0, 7.0]], [[6.0, 1.0, 5.0], [5.0, 6.0, 4.0], [4.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPRP(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 3.0], [2.0, 5.0, 1.0], [7.0, 4.0, 1.0]], [[6.0, 3.0, 8.0], [1.0, 5.0, 3.0], [4.0, 6.0, 
1.0]], [[4.0, 5.0, 5.0], [1.0, 8.0, 8.0], [6.0, 2.0, 6.0]]], [[[6.0, 8.0, 4.0], [3.0, 6.0, 7.0], [7.0, 5.0, 1.0]], [[3.0, 5.0, 
5.0], [6.0, 2.0, 4.0], [3.0, 7.0, 4.0]], [[6.0, 3.0, 8.0], [1.0, 6.0, 1.0], [1.0, 4.0, 4.0]]], [[[3.0, 4.0, 2.0], [4.0, 8.0, 
8.0], [2.0, 1.0, 3.0]], [[3.0, 8.0, 2.0], [6.0, 7.0, 5.0], [7.0, 7.0, 6.0]], [[6.0, 6.0, 5.0], [6.0, 2.0, 8.0], [7.0, 4.0, 
1.0]]], [[[2.0, 5.0, 6.0], [1.0, 8.0, 3.0], [2.0, 6.0, 6.0]], [[6.0, 8.0, 5.0], [4.0, 5.0, 8.0], [8.0, 6.0, 7.0]], [[6.0, 3.0, 
8.0], [2.0, 3.0, 5.0], [7.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 8.0], [1.0, 3.0, 3.0], [6.0, 6.0, 3.0]], [[5.0, 1.0, 3.0], [7.0, 7.0, 
6.0], [3.0, 1.0, 3.0]], [[2.0, 8.0, 2.0], [7.0, 3.0, 4.0], [6.0, 6.0, 7.0]]], [[[6.0, 3.0, 2.0], [6.0, 1.0, 4.0], [5.0, 3.0, 
4.0]], [[4.0, 5.0, 3.0], [2.0, 4.0, 3.0], [3.0, 2.0, 3.0]], [[2.0, 8.0, 6.0], [3.0, 8.0, 6.0], [8.0, 3.0, 2.0]]], [[[1.0, 5.0, 
5.0], [6.0, 8.0, 8.0], [2.0, 4.0, 4.0]], [[6.0, 4.0, 6.0], [1.0, 2.0, 8.0], [5.0, 3.0, 6.0]], [[3.0, 8.0, 4.0], [5.0, 8.0, 
6.0], [4.0, 2.0, 6.0]]], [[[5.0, 5.0, 6.0], [4.0, 1.0, 3.0], [3.0, 2.0, 3.0]], [[1.0, 2.0, 3.0], [6.0, 2.0, 2.0], [2.0, 4.0, 
6.0]], [[2.0, 1.0, 5.0], [2.0, 2.0, 6.0], [6.0, 7.0, 8.0]]]]))
      val=numpy.array([6.0])
      arg[2,1,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 3.0], [2.0, 5.0, 1.0], [7.0, 4.0, 1.0]], [[6.0, 3.0, 8.0], [1.0, 5.0, 3.0], [4.0, 
6.0, 1.0]], [[4.0, 5.0, 5.0], [1.0, 8.0, 8.0], [6.0, 2.0, 6.0]]], [[[6.0, 8.0, 4.0], [3.0, 6.0, 7.0], [7.0, 5.0, 1.0]], [[3.0, 
5.0, 5.0], [6.0, 2.0, 4.0], [3.0, 7.0, 4.0]], [[6.0, 3.0, 8.0], [1.0, 6.0, 1.0], [1.0, 4.0, 4.0]]], [[[3.0, 4.0, 2.0], [4.0, 
8.0, 8.0], [2.0, 1.0, 3.0]], [[3.0, 8.0, 2.0], [6.0, 6.0, 5.0], [7.0, 7.0, 6.0]], [[6.0, 6.0, 5.0], [6.0, 2.0, 8.0], [7.0, 4.0, 
1.0]]], [[[2.0, 5.0, 6.0], [1.0, 8.0, 3.0], [2.0, 6.0, 6.0]], [[6.0, 8.0, 5.0], [4.0, 5.0, 8.0], [8.0, 6.0, 7.0]], [[6.0, 3.0, 
8.0], [2.0, 3.0, 5.0], [7.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 8.0], [1.0, 3.0, 3.0], [6.0, 6.0, 3.0]], 
[[5.0, 1.0, 3.0], [7.0, 7.0, 6.0], [3.0, 1.0, 3.0]], [[2.0, 8.0, 2.0], [7.0, 3.0, 4.0], [6.0, 6.0, 7.0]]], [[[6.0, 3.0, 2.0], 
[6.0, 1.0, 4.0], [5.0, 3.0, 4.0]], [[4.0, 5.0, 3.0], [2.0, 4.0, 3.0], [3.0, 2.0, 3.0]], [[2.0, 8.0, 6.0], [3.0, 8.0, 6.0], 
[8.0, 3.0, 2.0]]], [[[1.0, 5.0, 5.0], [6.0, 8.0, 8.0], [2.0, 4.0, 4.0]], [[6.0, 4.0, 6.0], [1.0, 6.0, 8.0], [5.0, 3.0, 6.0]], 
[[3.0, 8.0, 4.0], [5.0, 8.0, 6.0], [4.0, 2.0, 6.0]]], [[[5.0, 5.0, 6.0], [4.0, 1.0, 3.0], [3.0, 2.0, 3.0]], [[1.0, 2.0, 3.0], 
[6.0, 2.0, 2.0], [2.0, 4.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 2.0, 6.0], [6.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPRP(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 8.0], [1.0, 8.0, 5.0], [3.0, 8.0, 2.0]], [[2.0, 4.0, 3.0], [4.0, 3.0, 5.0], [3.0, 8.0, 
2.0]], [[8.0, 7.0, 2.0], [6.0, 3.0, 6.0], [8.0, 7.0, 5.0]]], [[[6.0, 7.0, 4.0], [4.0, 7.0, 6.0], [7.0, 8.0, 3.0]], [[7.0, 2.0, 
6.0], [4.0, 1.0, 3.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 4.0], [7.0, 1.0, 3.0], [4.0, 1.0, 1.0]]], [[[7.0, 5.0, 1.0], [6.0, 4.0, 
4.0], [7.0, 5.0, 5.0]], [[2.0, 3.0, 4.0], [3.0, 3.0, 8.0], [7.0, 5.0, 2.0]], [[5.0, 6.0, 2.0], [5.0, 1.0, 3.0], [4.0, 5.0, 
3.0]]], [[[8.0, 5.0, 2.0], [1.0, 8.0, 1.0], [8.0, 1.0, 8.0]], [[3.0, 2.0, 2.0], [1.0, 7.0, 5.0], [3.0, 4.0, 1.0]], [[7.0, 1.0, 
7.0], [4.0, 4.0, 5.0], [7.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 4.0], [8.0, 6.0, 4.0], [6.0, 5.0, 6.0]], [[3.0, 6.0, 1.0], [1.0, 3.0, 
6.0], [3.0, 1.0, 1.0]], [[2.0, 2.0, 6.0], [7.0, 8.0, 2.0], [1.0, 3.0, 7.0]]], [[[2.0, 7.0, 6.0], [4.0, 3.0, 8.0], [3.0, 7.0, 
7.0]], [[7.0, 3.0, 6.0], [8.0, 2.0, 6.0], [8.0, 1.0, 7.0]], [[1.0, 4.0, 1.0], [1.0, 5.0, 3.0], [3.0, 7.0, 5.0]]], [[[2.0, 8.0, 
7.0], [5.0, 7.0, 6.0], [5.0, 3.0, 6.0]], [[6.0, 1.0, 4.0], [1.0, 6.0, 7.0], [5.0, 6.0, 3.0]], [[6.0, 7.0, 2.0], [5.0, 4.0, 
6.0], [4.0, 8.0, 1.0]]], [[[2.0, 3.0, 2.0], [3.0, 6.0, 4.0], [8.0, 6.0, 2.0]], [[4.0, 8.0, 6.0], [4.0, 1.0, 1.0], [4.0, 1.0, 
1.0]], [[6.0, 5.0, 8.0], [7.0, 6.0, 1.0], [3.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([4.0]),self.functionspace)
      arg[2,1,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 8.0], [1.0, 8.0, 5.0], [3.0, 8.0, 2.0]], [[2.0, 4.0, 3.0], [4.0, 3.0, 5.0], [3.0, 
8.0, 2.0]], [[8.0, 7.0, 2.0], [6.0, 3.0, 6.0], [8.0, 7.0, 5.0]]], [[[6.0, 7.0, 4.0], [4.0, 7.0, 6.0], [7.0, 8.0, 3.0]], [[7.0, 
2.0, 6.0], [4.0, 1.0, 3.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 4.0], [7.0, 1.0, 3.0], [4.0, 1.0, 1.0]]], [[[7.0, 5.0, 1.0], [6.0, 
4.0, 4.0], [7.0, 5.0, 5.0]], [[2.0, 3.0, 4.0], [3.0, 4.0, 8.0], [7.0, 5.0, 2.0]], [[5.0, 6.0, 2.0], [5.0, 1.0, 3.0], [4.0, 5.0, 
3.0]]], [[[8.0, 5.0, 2.0], [1.0, 8.0, 1.0], [8.0, 1.0, 8.0]], [[3.0, 2.0, 2.0], [1.0, 7.0, 5.0], [3.0, 4.0, 1.0]], [[7.0, 1.0, 
7.0], [4.0, 4.0, 5.0], [7.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 4.0], [8.0, 6.0, 4.0], [6.0, 5.0, 6.0]], 
[[3.0, 6.0, 1.0], [1.0, 3.0, 6.0], [3.0, 1.0, 1.0]], [[2.0, 2.0, 6.0], [7.0, 8.0, 2.0], [1.0, 3.0, 7.0]]], [[[2.0, 7.0, 6.0], 
[4.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[7.0, 3.0, 6.0], [8.0, 2.0, 6.0], [8.0, 1.0, 7.0]], [[1.0, 4.0, 1.0], [1.0, 5.0, 3.0], 
[3.0, 7.0, 5.0]]], [[[2.0, 8.0, 7.0], [5.0, 7.0, 6.0], [5.0, 3.0, 6.0]], [[6.0, 1.0, 4.0], [1.0, 4.0, 7.0], [5.0, 6.0, 3.0]], 
[[6.0, 7.0, 2.0], [5.0, 4.0, 6.0], [4.0, 8.0, 1.0]]], [[[2.0, 3.0, 2.0], [3.0, 6.0, 4.0], [8.0, 6.0, 2.0]], [[4.0, 8.0, 6.0], 
[4.0, 1.0, 1.0], [4.0, 1.0, 1.0]], [[6.0, 5.0, 8.0], [7.0, 6.0, 1.0], [3.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPRP(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 5.0], [4.0, 7.0, 3.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [8.0, 7.0, 7.0], [4.0, 2.0, 
8.0]], [[4.0, 1.0, 8.0], [7.0, 1.0, 1.0], [4.0, 6.0, 5.0]]], [[[3.0, 3.0, 6.0], [3.0, 8.0, 1.0], [7.0, 4.0, 3.0]], [[8.0, 6.0, 
2.0], [6.0, 7.0, 5.0], [4.0, 4.0, 4.0]], [[2.0, 7.0, 6.0], [4.0, 3.0, 5.0], [4.0, 7.0, 7.0]]], [[[7.0, 1.0, 4.0], [8.0, 2.0, 
7.0], [7.0, 1.0, 4.0]], [[3.0, 1.0, 4.0], [7.0, 6.0, 4.0], [8.0, 1.0, 3.0]], [[3.0, 5.0, 6.0], [1.0, 7.0, 3.0], [8.0, 3.0, 
3.0]]], [[[2.0, 1.0, 6.0], [8.0, 7.0, 4.0], [3.0, 7.0, 6.0]], [[7.0, 8.0, 2.0], [2.0, 7.0, 1.0], [7.0, 1.0, 3.0]], [[4.0, 3.0, 
3.0], [8.0, 8.0, 3.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 2.0], [1.0, 6.0, 6.0], [5.0, 5.0, 4.0]], [[2.0, 4.0, 5.0], [8.0, 2.0, 
6.0], [3.0, 6.0, 2.0]], [[2.0, 4.0, 5.0], [8.0, 7.0, 4.0], [4.0, 8.0, 6.0]]], [[[5.0, 3.0, 2.0], [3.0, 5.0, 3.0], [2.0, 4.0, 
5.0]], [[5.0, 4.0, 1.0], [7.0, 7.0, 3.0], [7.0, 3.0, 3.0]], [[7.0, 4.0, 7.0], [4.0, 7.0, 8.0], [2.0, 5.0, 5.0]]], [[[6.0, 3.0, 
5.0], [4.0, 7.0, 7.0], [3.0, 6.0, 1.0]], [[4.0, 3.0, 6.0], [5.0, 6.0, 1.0], [5.0, 5.0, 6.0]], [[1.0, 3.0, 2.0], [2.0, 6.0, 
7.0], [6.0, 4.0, 4.0]]], [[[8.0, 8.0, 1.0], [6.0, 7.0, 8.0], [2.0, 2.0, 8.0]], [[4.0, 7.0, 3.0], [8.0, 8.0, 6.0], [5.0, 2.0, 
7.0]], [[7.0, 1.0, 1.0], [1.0, 5.0, 5.0], [4.0, 3.0, 1.0]]]]))
      val=Data(numpy.array([5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0]))
      arg[2,1,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 5.0], [4.0, 7.0, 3.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [8.0, 7.0, 7.0], [4.0, 
2.0, 8.0]], [[4.0, 1.0, 8.0], [7.0, 1.0, 1.0], [4.0, 6.0, 5.0]]], [[[3.0, 3.0, 6.0], [3.0, 8.0, 1.0], [7.0, 4.0, 3.0]], [[8.0, 
6.0, 2.0], [6.0, 7.0, 5.0], [4.0, 4.0, 4.0]], [[2.0, 7.0, 6.0], [4.0, 3.0, 5.0], [4.0, 7.0, 7.0]]], [[[7.0, 1.0, 4.0], [8.0, 
2.0, 7.0], [7.0, 1.0, 4.0]], [[3.0, 1.0, 4.0], [7.0, 5.0, 4.0], [8.0, 1.0, 3.0]], [[3.0, 5.0, 6.0], [1.0, 7.0, 3.0], [8.0, 3.0, 
3.0]]], [[[2.0, 1.0, 6.0], [8.0, 7.0, 4.0], [3.0, 7.0, 6.0]], [[7.0, 8.0, 2.0], [2.0, 7.0, 1.0], [7.0, 1.0, 3.0]], [[4.0, 3.0, 
3.0], [8.0, 8.0, 3.0], [5.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 2.0], [1.0, 6.0, 6.0], [5.0, 5.0, 4.0]], 
[[2.0, 4.0, 5.0], [8.0, 2.0, 6.0], [3.0, 6.0, 2.0]], [[2.0, 4.0, 5.0], [8.0, 7.0, 4.0], [4.0, 8.0, 6.0]]], [[[5.0, 3.0, 2.0], 
[3.0, 5.0, 3.0], [2.0, 4.0, 5.0]], [[5.0, 4.0, 1.0], [7.0, 7.0, 3.0], [7.0, 3.0, 3.0]], [[7.0, 4.0, 7.0], [4.0, 7.0, 8.0], 
[2.0, 5.0, 5.0]]], [[[6.0, 3.0, 5.0], [4.0, 7.0, 7.0], [3.0, 6.0, 1.0]], [[4.0, 3.0, 6.0], [5.0, 3.0, 1.0], [5.0, 5.0, 6.0]], 
[[1.0, 3.0, 2.0], [2.0, 6.0, 7.0], [6.0, 4.0, 4.0]]], [[[8.0, 8.0, 1.0], [6.0, 7.0, 8.0], [2.0, 2.0, 8.0]], [[4.0, 7.0, 3.0], 
[8.0, 8.0, 6.0], [5.0, 2.0, 7.0]], [[7.0, 1.0, 1.0], [1.0, 5.0, 5.0], [4.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPRP(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 8.0], [4.0, 4.0, 1.0], [2.0, 4.0, 3.0]], [[4.0, 8.0, 5.0], [3.0, 5.0, 8.0], [4.0, 1.0, 
3.0]], [[3.0, 3.0, 1.0], [8.0, 7.0, 3.0], [5.0, 5.0, 7.0]]], [[[1.0, 7.0, 1.0], [2.0, 5.0, 4.0], [5.0, 6.0, 7.0]], [[5.0, 6.0, 
6.0], [2.0, 2.0, 1.0], [4.0, 4.0, 7.0]], [[8.0, 8.0, 6.0], [7.0, 6.0, 8.0], [6.0, 7.0, 2.0]]], [[[4.0, 8.0, 1.0], [1.0, 5.0, 
5.0], [1.0, 7.0, 4.0]], [[6.0, 5.0, 5.0], [2.0, 7.0, 2.0], [4.0, 5.0, 4.0]], [[2.0, 2.0, 8.0], [5.0, 5.0, 8.0], [4.0, 1.0, 
8.0]]], [[[6.0, 4.0, 2.0], [4.0, 7.0, 1.0], [8.0, 4.0, 3.0]], [[3.0, 7.0, 3.0], [7.0, 5.0, 2.0], [3.0, 7.0, 3.0]], [[6.0, 7.0, 
5.0], [7.0, 4.0, 2.0], [5.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 2.0], [4.0, 1.0, 6.0], [4.0, 4.0, 8.0]], [[8.0, 5.0, 6.0], [2.0, 8.0, 
2.0], [6.0, 6.0, 8.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 7.0], [8.0, 6.0, 6.0]]], [[[6.0, 4.0, 8.0], [2.0, 4.0, 6.0], [4.0, 7.0, 
4.0]], [[3.0, 8.0, 4.0], [6.0, 6.0, 8.0], [1.0, 4.0, 7.0]], [[2.0, 2.0, 2.0], [8.0, 8.0, 4.0], [6.0, 2.0, 1.0]]], [[[8.0, 5.0, 
1.0], [8.0, 6.0, 5.0], [1.0, 1.0, 5.0]], [[2.0, 6.0, 6.0], [7.0, 3.0, 1.0], [6.0, 8.0, 5.0]], [[4.0, 3.0, 6.0], [1.0, 7.0, 
7.0], [4.0, 6.0, 2.0]]], [[[8.0, 4.0, 7.0], [8.0, 5.0, 3.0], [4.0, 3.0, 5.0]], [[3.0, 4.0, 2.0], [8.0, 1.0, 2.0], [8.0, 3.0, 
5.0]], [[4.0, 7.0, 7.0], [8.0, 7.0, 4.0], [2.0, 4.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([2.0])+(1.-msk_val)*numpy.array([3.0])
      arg[2,1,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 8.0], [4.0, 4.0, 1.0], [2.0, 4.0, 3.0]], [[4.0, 8.0, 5.0], [3.0, 5.0, 8.0], [4.0, 
1.0, 3.0]], [[3.0, 3.0, 1.0], [8.0, 7.0, 3.0], [5.0, 5.0, 7.0]]], [[[1.0, 7.0, 1.0], [2.0, 5.0, 4.0], [5.0, 6.0, 7.0]], [[5.0, 
6.0, 6.0], [2.0, 2.0, 1.0], [4.0, 4.0, 7.0]], [[8.0, 8.0, 6.0], [7.0, 6.0, 8.0], [6.0, 7.0, 2.0]]], [[[4.0, 8.0, 1.0], [1.0, 
5.0, 5.0], [1.0, 7.0, 4.0]], [[6.0, 5.0, 5.0], [2.0, 2.0, 2.0], [4.0, 5.0, 4.0]], [[2.0, 2.0, 8.0], [5.0, 5.0, 8.0], [4.0, 1.0, 
8.0]]], [[[6.0, 4.0, 2.0], [4.0, 7.0, 1.0], [8.0, 4.0, 3.0]], [[3.0, 7.0, 3.0], [7.0, 5.0, 2.0], [3.0, 7.0, 3.0]], [[6.0, 7.0, 
5.0], [7.0, 4.0, 2.0], [5.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 2.0], [4.0, 1.0, 6.0], [4.0, 4.0, 8.0]], 
[[8.0, 5.0, 6.0], [2.0, 8.0, 2.0], [6.0, 6.0, 8.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 7.0], [8.0, 6.0, 6.0]]], [[[6.0, 4.0, 8.0], 
[2.0, 4.0, 6.0], [4.0, 7.0, 4.0]], [[3.0, 8.0, 4.0], [6.0, 6.0, 8.0], [1.0, 4.0, 7.0]], [[2.0, 2.0, 2.0], [8.0, 8.0, 4.0], 
[6.0, 2.0, 1.0]]], [[[8.0, 5.0, 1.0], [8.0, 6.0, 5.0], [1.0, 1.0, 5.0]], [[2.0, 6.0, 6.0], [7.0, 3.0, 1.0], [6.0, 8.0, 5.0]], 
[[4.0, 3.0, 6.0], [1.0, 7.0, 7.0], [4.0, 6.0, 2.0]]], [[[8.0, 4.0, 7.0], [8.0, 5.0, 3.0], [4.0, 3.0, 5.0]], [[3.0, 4.0, 2.0], 
[8.0, 1.0, 2.0], [8.0, 3.0, 5.0]], [[4.0, 7.0, 7.0], [8.0, 7.0, 4.0], [2.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRLP(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 1.0], [6.0, 8.0, 2.0], [3.0, 6.0, 8.0]], [[4.0, 5.0, 8.0], [7.0, 6.0, 7.0], [4.0, 8.0, 
3.0]], [[1.0, 7.0, 1.0], [3.0, 8.0, 5.0], [5.0, 3.0, 3.0]]], [[[7.0, 2.0, 6.0], [7.0, 4.0, 6.0], [6.0, 4.0, 1.0]], [[2.0, 1.0, 
8.0], [7.0, 3.0, 4.0], [5.0, 3.0, 8.0]], [[1.0, 8.0, 4.0], [7.0, 6.0, 7.0], [1.0, 4.0, 5.0]]], [[[6.0, 7.0, 2.0], [2.0, 4.0, 
7.0], [3.0, 1.0, 7.0]], [[6.0, 4.0, 1.0], [8.0, 6.0, 3.0], [1.0, 3.0, 7.0]], [[2.0, 3.0, 1.0], [5.0, 7.0, 1.0], [3.0, 1.0, 
5.0]]], [[[2.0, 5.0, 4.0], [4.0, 6.0, 6.0], [7.0, 3.0, 5.0]], [[6.0, 4.0, 1.0], [1.0, 1.0, 2.0], [7.0, 8.0, 4.0]], [[7.0, 2.0, 
2.0], [3.0, 1.0, 8.0], [2.0, 8.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 5.0], [5.0, 8.0, 2.0], [8.0, 8.0, 5.0]], [[5.0, 6.0, 3.0], [3.0, 4.0, 
6.0], [4.0, 7.0, 4.0]], [[8.0, 7.0, 2.0], [7.0, 2.0, 5.0], [4.0, 6.0, 7.0]]], [[[7.0, 8.0, 6.0], [7.0, 1.0, 5.0], [1.0, 2.0, 
6.0]], [[1.0, 4.0, 5.0], [7.0, 1.0, 6.0], [8.0, 8.0, 4.0]], [[1.0, 7.0, 7.0], [4.0, 5.0, 1.0], [3.0, 8.0, 4.0]]], [[[8.0, 6.0, 
3.0], [3.0, 3.0, 2.0], [8.0, 8.0, 1.0]], [[6.0, 5.0, 8.0], [8.0, 8.0, 4.0], [2.0, 2.0, 6.0]], [[5.0, 1.0, 4.0], [7.0, 1.0, 
3.0], [3.0, 3.0, 1.0]]], [[[5.0, 2.0, 5.0], [2.0, 2.0, 3.0], [8.0, 4.0, 2.0]], [[7.0, 4.0, 2.0], [2.0, 7.0, 8.0], [4.0, 2.0, 
7.0]], [[5.0, 3.0, 2.0], [4.0, 6.0, 4.0], [1.0, 1.0, 4.0]]]]))
      val=numpy.array([[[4.0]], [[5.0]], [[1.0]], [[6.0]]])
      arg[:,1:2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 1.0], [6.0, 8.0, 2.0], [3.0, 6.0, 8.0]], [[4.0, 5.0, 8.0], [7.0, 6.0, 7.0], [4.0, 
4.0, 3.0]], [[1.0, 7.0, 1.0], [3.0, 8.0, 5.0], [5.0, 3.0, 3.0]]], [[[7.0, 2.0, 6.0], [7.0, 4.0, 6.0], [6.0, 4.0, 1.0]], [[2.0, 
1.0, 8.0], [7.0, 3.0, 4.0], [5.0, 5.0, 8.0]], [[1.0, 8.0, 4.0], [7.0, 6.0, 7.0], [1.0, 4.0, 5.0]]], [[[6.0, 7.0, 2.0], [2.0, 
4.0, 7.0], [3.0, 1.0, 7.0]], [[6.0, 4.0, 1.0], [8.0, 6.0, 3.0], [1.0, 1.0, 7.0]], [[2.0, 3.0, 1.0], [5.0, 7.0, 1.0], [3.0, 1.0, 
5.0]]], [[[2.0, 5.0, 4.0], [4.0, 6.0, 6.0], [7.0, 3.0, 5.0]], [[6.0, 4.0, 1.0], [1.0, 1.0, 2.0], [7.0, 6.0, 4.0]], [[7.0, 2.0, 
2.0], [3.0, 1.0, 8.0], [2.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 5.0], [5.0, 8.0, 2.0], [8.0, 8.0, 5.0]], 
[[5.0, 6.0, 3.0], [3.0, 4.0, 6.0], [4.0, 4.0, 4.0]], [[8.0, 7.0, 2.0], [7.0, 2.0, 5.0], [4.0, 6.0, 7.0]]], [[[7.0, 8.0, 6.0], 
[7.0, 1.0, 5.0], [1.0, 2.0, 6.0]], [[1.0, 4.0, 5.0], [7.0, 1.0, 6.0], [8.0, 5.0, 4.0]], [[1.0, 7.0, 7.0], [4.0, 5.0, 1.0], 
[3.0, 8.0, 4.0]]], [[[8.0, 6.0, 3.0], [3.0, 3.0, 2.0], [8.0, 8.0, 1.0]], [[6.0, 5.0, 8.0], [8.0, 8.0, 4.0], [2.0, 1.0, 6.0]], 
[[5.0, 1.0, 4.0], [7.0, 1.0, 3.0], [3.0, 3.0, 1.0]]], [[[5.0, 2.0, 5.0], [2.0, 2.0, 3.0], [8.0, 4.0, 2.0]], [[7.0, 4.0, 2.0], 
[2.0, 7.0, 8.0], [4.0, 6.0, 7.0]], [[5.0, 3.0, 2.0], [4.0, 6.0, 4.0], [1.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRLP(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 6.0], [3.0, 8.0, 5.0], [1.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], [4.0, 7.0, 8.0], [6.0, 6.0, 
7.0]], [[5.0, 1.0, 3.0], [2.0, 8.0, 5.0], [8.0, 3.0, 6.0]]], [[[1.0, 1.0, 7.0], [7.0, 6.0, 3.0], [6.0, 1.0, 4.0]], [[4.0, 6.0, 
6.0], [2.0, 3.0, 4.0], [7.0, 6.0, 1.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 5.0], [3.0, 2.0, 1.0]]], [[[6.0, 8.0, 6.0], [5.0, 2.0, 
8.0], [7.0, 1.0, 1.0]], [[7.0, 2.0, 3.0], [1.0, 2.0, 2.0], [7.0, 5.0, 2.0]], [[3.0, 6.0, 2.0], [4.0, 8.0, 6.0], [3.0, 1.0, 
8.0]]], [[[3.0, 8.0, 3.0], [3.0, 5.0, 2.0], [6.0, 1.0, 8.0]], [[5.0, 2.0, 3.0], [3.0, 1.0, 7.0], [2.0, 2.0, 6.0]], [[4.0, 6.0, 
7.0], [8.0, 2.0, 2.0], [8.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 2.0], [2.0, 5.0, 6.0], [3.0, 1.0, 2.0]], [[4.0, 4.0, 6.0], [4.0, 6.0, 
7.0], [7.0, 5.0, 4.0]], [[7.0, 4.0, 4.0], [4.0, 7.0, 3.0], [5.0, 7.0, 6.0]]], [[[7.0, 1.0, 7.0], [4.0, 4.0, 3.0], [3.0, 1.0, 
1.0]], [[1.0, 8.0, 7.0], [3.0, 3.0, 2.0], [8.0, 6.0, 3.0]], [[4.0, 3.0, 2.0], [7.0, 1.0, 6.0], [8.0, 1.0, 7.0]]], [[[3.0, 6.0, 
7.0], [7.0, 1.0, 3.0], [4.0, 6.0, 6.0]], [[4.0, 7.0, 5.0], [4.0, 7.0, 7.0], [4.0, 8.0, 6.0]], [[3.0, 7.0, 6.0], [8.0, 7.0, 
2.0], [8.0, 4.0, 5.0]]], [[[8.0, 3.0, 1.0], [8.0, 8.0, 8.0], [6.0, 1.0, 4.0]], [[2.0, 7.0, 4.0], [4.0, 4.0, 5.0], [6.0, 3.0, 
1.0]], [[4.0, 3.0, 5.0], [2.0, 6.0, 5.0], [2.0, 8.0, 1.0]]]]))
      val=Data(numpy.array([[[1.0]], [[5.0]], [[7.0]], [[4.0]]]),self.functionspace)
      arg[:,1:2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 6.0], [3.0, 8.0, 5.0], [1.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], [4.0, 7.0, 8.0], [6.0, 
1.0, 7.0]], [[5.0, 1.0, 3.0], [2.0, 8.0, 5.0], [8.0, 3.0, 6.0]]], [[[1.0, 1.0, 7.0], [7.0, 6.0, 3.0], [6.0, 1.0, 4.0]], [[4.0, 
6.0, 6.0], [2.0, 3.0, 4.0], [7.0, 5.0, 1.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 5.0], [3.0, 2.0, 1.0]]], [[[6.0, 8.0, 6.0], [5.0, 
2.0, 8.0], [7.0, 1.0, 1.0]], [[7.0, 2.0, 3.0], [1.0, 2.0, 2.0], [7.0, 7.0, 2.0]], [[3.0, 6.0, 2.0], [4.0, 8.0, 6.0], [3.0, 1.0, 
8.0]]], [[[3.0, 8.0, 3.0], [3.0, 5.0, 2.0], [6.0, 1.0, 8.0]], [[5.0, 2.0, 3.0], [3.0, 1.0, 7.0], [2.0, 4.0, 6.0]], [[4.0, 6.0, 
7.0], [8.0, 2.0, 2.0], [8.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 2.0], [2.0, 5.0, 6.0], [3.0, 1.0, 2.0]], 
[[4.0, 4.0, 6.0], [4.0, 6.0, 7.0], [7.0, 1.0, 4.0]], [[7.0, 4.0, 4.0], [4.0, 7.0, 3.0], [5.0, 7.0, 6.0]]], [[[7.0, 1.0, 7.0], 
[4.0, 4.0, 3.0], [3.0, 1.0, 1.0]], [[1.0, 8.0, 7.0], [3.0, 3.0, 2.0], [8.0, 5.0, 3.0]], [[4.0, 3.0, 2.0], [7.0, 1.0, 6.0], 
[8.0, 1.0, 7.0]]], [[[3.0, 6.0, 7.0], [7.0, 1.0, 3.0], [4.0, 6.0, 6.0]], [[4.0, 7.0, 5.0], [4.0, 7.0, 7.0], [4.0, 7.0, 6.0]], 
[[3.0, 7.0, 6.0], [8.0, 7.0, 2.0], [8.0, 4.0, 5.0]]], [[[8.0, 3.0, 1.0], [8.0, 8.0, 8.0], [6.0, 1.0, 4.0]], [[2.0, 7.0, 4.0], 
[4.0, 4.0, 5.0], [6.0, 4.0, 1.0]], [[4.0, 3.0, 5.0], [2.0, 6.0, 5.0], [2.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRLP(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 6.0], [3.0, 5.0, 6.0], [3.0, 3.0, 8.0]], [[6.0, 2.0, 8.0], [8.0, 7.0, 2.0], [7.0, 7.0, 
6.0]], [[5.0, 7.0, 5.0], [8.0, 3.0, 1.0], [1.0, 6.0, 6.0]]], [[[8.0, 2.0, 6.0], [1.0, 7.0, 7.0], [1.0, 4.0, 4.0]], [[5.0, 6.0, 
1.0], [7.0, 1.0, 6.0], [8.0, 7.0, 7.0]], [[8.0, 1.0, 5.0], [3.0, 1.0, 5.0], [5.0, 5.0, 5.0]]], [[[8.0, 1.0, 8.0], [3.0, 5.0, 
4.0], [2.0, 1.0, 6.0]], [[3.0, 8.0, 1.0], [8.0, 7.0, 8.0], [8.0, 5.0, 1.0]], [[1.0, 2.0, 3.0], [7.0, 2.0, 1.0], [7.0, 5.0, 
8.0]]], [[[1.0, 6.0, 5.0], [1.0, 7.0, 7.0], [7.0, 8.0, 7.0]], [[4.0, 6.0, 7.0], [2.0, 6.0, 4.0], [2.0, 3.0, 8.0]], [[4.0, 3.0, 
5.0], [6.0, 5.0, 1.0], [2.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 3.0], [3.0, 6.0, 2.0], [8.0, 7.0, 3.0]], [[7.0, 8.0, 4.0], [2.0, 1.0, 
4.0], [5.0, 8.0, 2.0]], [[7.0, 8.0, 7.0], [2.0, 4.0, 1.0], [1.0, 2.0, 6.0]]], [[[6.0, 1.0, 7.0], [8.0, 3.0, 4.0], [4.0, 8.0, 
1.0]], [[7.0, 2.0, 2.0], [8.0, 2.0, 7.0], [2.0, 5.0, 3.0]], [[4.0, 6.0, 3.0], [4.0, 3.0, 4.0], [1.0, 1.0, 5.0]]], [[[6.0, 2.0, 
4.0], [6.0, 6.0, 1.0], [2.0, 5.0, 6.0]], [[8.0, 3.0, 2.0], [2.0, 8.0, 4.0], [7.0, 1.0, 4.0]], [[1.0, 2.0, 6.0], [4.0, 5.0, 
2.0], [4.0, 7.0, 4.0]]], [[[7.0, 2.0, 4.0], [4.0, 4.0, 1.0], [2.0, 6.0, 8.0]], [[3.0, 2.0, 2.0], [2.0, 6.0, 6.0], [2.0, 7.0, 
1.0]], [[6.0, 3.0, 3.0], [2.0, 8.0, 1.0], [5.0, 1.0, 8.0]]]]))
      val=Data(numpy.array([[[1.0]], [[3.0]], [[1.0]], [[4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0]], [[8.0]], [[3.0]], [[3.0]]]))
      arg[:,1:2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 6.0], [3.0, 5.0, 6.0], [3.0, 3.0, 8.0]], [[6.0, 2.0, 8.0], [8.0, 7.0, 2.0], [7.0, 
1.0, 6.0]], [[5.0, 7.0, 5.0], [8.0, 3.0, 1.0], [1.0, 6.0, 6.0]]], [[[8.0, 2.0, 6.0], [1.0, 7.0, 7.0], [1.0, 4.0, 4.0]], [[5.0, 
6.0, 1.0], [7.0, 1.0, 6.0], [8.0, 3.0, 7.0]], [[8.0, 1.0, 5.0], [3.0, 1.0, 5.0], [5.0, 5.0, 5.0]]], [[[8.0, 1.0, 8.0], [3.0, 
5.0, 4.0], [2.0, 1.0, 6.0]], [[3.0, 8.0, 1.0], [8.0, 7.0, 8.0], [8.0, 1.0, 1.0]], [[1.0, 2.0, 3.0], [7.0, 2.0, 1.0], [7.0, 5.0, 
8.0]]], [[[1.0, 6.0, 5.0], [1.0, 7.0, 7.0], [7.0, 8.0, 7.0]], [[4.0, 6.0, 7.0], [2.0, 6.0, 4.0], [2.0, 4.0, 8.0]], [[4.0, 3.0, 
5.0], [6.0, 5.0, 1.0], [2.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 3.0], [3.0, 6.0, 2.0], [8.0, 7.0, 3.0]], 
[[7.0, 8.0, 4.0], [2.0, 1.0, 4.0], [5.0, 4.0, 2.0]], [[7.0, 8.0, 7.0], [2.0, 4.0, 1.0], [1.0, 2.0, 6.0]]], [[[6.0, 1.0, 7.0], 
[8.0, 3.0, 4.0], [4.0, 8.0, 1.0]], [[7.0, 2.0, 2.0], [8.0, 2.0, 7.0], [2.0, 8.0, 3.0]], [[4.0, 6.0, 3.0], [4.0, 3.0, 4.0], 
[1.0, 1.0, 5.0]]], [[[6.0, 2.0, 4.0], [6.0, 6.0, 1.0], [2.0, 5.0, 6.0]], [[8.0, 3.0, 2.0], [2.0, 8.0, 4.0], [7.0, 3.0, 4.0]], 
[[1.0, 2.0, 6.0], [4.0, 5.0, 2.0], [4.0, 7.0, 4.0]]], [[[7.0, 2.0, 4.0], [4.0, 4.0, 1.0], [2.0, 6.0, 8.0]], [[3.0, 2.0, 2.0], 
[2.0, 6.0, 6.0], [2.0, 3.0, 1.0]], [[6.0, 3.0, 3.0], [2.0, 8.0, 1.0], [5.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRLP(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 6.0], [2.0, 2.0, 4.0], [5.0, 8.0, 2.0]], [[1.0, 4.0, 4.0], [1.0, 4.0, 4.0], [6.0, 8.0, 
1.0]], [[2.0, 8.0, 4.0], [8.0, 3.0, 1.0], [1.0, 7.0, 4.0]]], [[[2.0, 1.0, 1.0], [7.0, 4.0, 8.0], [4.0, 3.0, 5.0]], [[2.0, 8.0, 
5.0], [3.0, 8.0, 5.0], [7.0, 2.0, 4.0]], [[5.0, 1.0, 1.0], [4.0, 3.0, 5.0], [5.0, 2.0, 8.0]]], [[[4.0, 5.0, 7.0], [7.0, 8.0, 
8.0], [3.0, 1.0, 8.0]], [[4.0, 1.0, 2.0], [7.0, 6.0, 2.0], [8.0, 4.0, 3.0]], [[2.0, 3.0, 8.0], [8.0, 5.0, 8.0], [3.0, 1.0, 
4.0]]], [[[2.0, 7.0, 4.0], [5.0, 2.0, 7.0], [8.0, 4.0, 4.0]], [[1.0, 5.0, 7.0], [8.0, 4.0, 8.0], [3.0, 5.0, 6.0]], [[4.0, 5.0, 
8.0], [5.0, 4.0, 7.0], [7.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 8.0], [3.0, 1.0, 1.0], [6.0, 8.0, 4.0]], [[8.0, 5.0, 3.0], [7.0, 1.0, 
3.0], [8.0, 6.0, 3.0]], [[1.0, 3.0, 1.0], [3.0, 2.0, 5.0], [4.0, 7.0, 4.0]]], [[[8.0, 6.0, 7.0], [4.0, 1.0, 6.0], [7.0, 7.0, 
1.0]], [[4.0, 6.0, 3.0], [7.0, 2.0, 5.0], [4.0, 3.0, 4.0]], [[3.0, 4.0, 4.0], [4.0, 3.0, 3.0], [2.0, 3.0, 3.0]]], [[[2.0, 3.0, 
8.0], [3.0, 8.0, 5.0], [8.0, 1.0, 8.0]], [[4.0, 8.0, 1.0], [1.0, 1.0, 1.0], [4.0, 6.0, 6.0]], [[6.0, 4.0, 6.0], [5.0, 1.0, 
1.0], [7.0, 1.0, 8.0]]], [[[3.0, 4.0, 2.0], [1.0, 7.0, 8.0], [2.0, 3.0, 1.0]], [[1.0, 4.0, 6.0], [3.0, 1.0, 3.0], [4.0, 3.0, 
2.0]], [[7.0, 4.0, 2.0], [7.0, 1.0, 7.0], [4.0, 8.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0]], [[8.0]], [[2.0]], [[4.0]]])+(1.-msk_val)*numpy.array([[[3.0]], [[4.0]], [[4.0]], 
[[4.0]]])
      arg[:,1:2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 6.0], [2.0, 2.0, 4.0], [5.0, 8.0, 2.0]], [[1.0, 4.0, 4.0], [1.0, 4.0, 4.0], [6.0, 
1.0, 1.0]], [[2.0, 8.0, 4.0], [8.0, 3.0, 1.0], [1.0, 7.0, 4.0]]], [[[2.0, 1.0, 1.0], [7.0, 4.0, 8.0], [4.0, 3.0, 5.0]], [[2.0, 
8.0, 5.0], [3.0, 8.0, 5.0], [7.0, 8.0, 4.0]], [[5.0, 1.0, 1.0], [4.0, 3.0, 5.0], [5.0, 2.0, 8.0]]], [[[4.0, 5.0, 7.0], [7.0, 
8.0, 8.0], [3.0, 1.0, 8.0]], [[4.0, 1.0, 2.0], [7.0, 6.0, 2.0], [8.0, 2.0, 3.0]], [[2.0, 3.0, 8.0], [8.0, 5.0, 8.0], [3.0, 1.0, 
4.0]]], [[[2.0, 7.0, 4.0], [5.0, 2.0, 7.0], [8.0, 4.0, 4.0]], [[1.0, 5.0, 7.0], [8.0, 4.0, 8.0], [3.0, 4.0, 6.0]], [[4.0, 5.0, 
8.0], [5.0, 4.0, 7.0], [7.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 8.0], [3.0, 1.0, 1.0], [6.0, 8.0, 4.0]], 
[[8.0, 5.0, 3.0], [7.0, 1.0, 3.0], [8.0, 3.0, 3.0]], [[1.0, 3.0, 1.0], [3.0, 2.0, 5.0], [4.0, 7.0, 4.0]]], [[[8.0, 6.0, 7.0], 
[4.0, 1.0, 6.0], [7.0, 7.0, 1.0]], [[4.0, 6.0, 3.0], [7.0, 2.0, 5.0], [4.0, 4.0, 4.0]], [[3.0, 4.0, 4.0], [4.0, 3.0, 3.0], 
[2.0, 3.0, 3.0]]], [[[2.0, 3.0, 8.0], [3.0, 8.0, 5.0], [8.0, 1.0, 8.0]], [[4.0, 8.0, 1.0], [1.0, 1.0, 1.0], [4.0, 4.0, 6.0]], 
[[6.0, 4.0, 6.0], [5.0, 1.0, 1.0], [7.0, 1.0, 8.0]]], [[[3.0, 4.0, 2.0], [1.0, 7.0, 8.0], [2.0, 3.0, 1.0]], [[1.0, 4.0, 6.0], 
[3.0, 1.0, 3.0], [4.0, 4.0, 2.0]], [[7.0, 4.0, 2.0], [7.0, 1.0, 7.0], [4.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RCLN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 4.0], [2.0, 1.0, 5.0], [1.0, 5.0, 4.0]], [[4.0, 2.0, 6.0], [8.0, 7.0, 4.0], [4.0, 3.0, 
5.0]], [[7.0, 1.0, 3.0], [1.0, 4.0, 3.0], [5.0, 3.0, 3.0]]], [[[2.0, 8.0, 1.0], [5.0, 5.0, 6.0], [1.0, 5.0, 5.0]], [[4.0, 4.0, 
1.0], [8.0, 2.0, 5.0], [2.0, 1.0, 4.0]], [[3.0, 8.0, 6.0], [5.0, 7.0, 3.0], [1.0, 5.0, 2.0]]], [[[5.0, 6.0, 7.0], [5.0, 6.0, 
1.0], [4.0, 8.0, 1.0]], [[8.0, 6.0, 4.0], [8.0, 3.0, 6.0], [8.0, 7.0, 4.0]], [[6.0, 8.0, 7.0], [1.0, 3.0, 8.0], [7.0, 7.0, 
1.0]]], [[[1.0, 2.0, 3.0], [1.0, 5.0, 7.0], [8.0, 6.0, 6.0]], [[7.0, 7.0, 2.0], [2.0, 5.0, 1.0], [6.0, 7.0, 8.0]], [[2.0, 2.0, 
7.0], [4.0, 4.0, 7.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 5.0], [7.0, 6.0, 3.0], [8.0, 5.0, 5.0]], [[4.0, 8.0, 2.0], [7.0, 8.0, 
4.0], [6.0, 4.0, 2.0]], [[1.0, 2.0, 5.0], [7.0, 5.0, 1.0], [8.0, 4.0, 8.0]]], [[[3.0, 1.0, 7.0], [1.0, 7.0, 4.0], [5.0, 5.0, 
5.0]], [[4.0, 2.0, 2.0], [5.0, 8.0, 2.0], [5.0, 8.0, 1.0]], [[3.0, 4.0, 7.0], [6.0, 6.0, 3.0], [7.0, 5.0, 7.0]]], [[[6.0, 3.0, 
1.0], [4.0, 1.0, 6.0], [2.0, 7.0, 7.0]], [[4.0, 7.0, 4.0], [6.0, 7.0, 1.0], [2.0, 3.0, 1.0]], [[5.0, 1.0, 4.0], [5.0, 7.0, 
2.0], [8.0, 7.0, 7.0]]], [[[1.0, 2.0, 2.0], [4.0, 3.0, 4.0], [4.0, 3.0, 7.0]], [[6.0, 7.0, 6.0], [6.0, 1.0, 6.0], [6.0, 8.0, 
5.0]], [[6.0, 7.0, 3.0], [4.0, 6.0, 8.0], [6.0, 5.0, 2.0]]]]))
      val=numpy.array([[[[5.0, 4.0, 1.0]], [[4.0, 6.0, 2.0]], [[7.0, 2.0, 2.0]]], [[[8.0, 7.0, 8.0]], [[7.0, 6.0, 5.0]], [[3.0, 
3.0, 8.0]]]])
      arg[1:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 4.0], [2.0, 1.0, 5.0], [1.0, 5.0, 4.0]], [[4.0, 2.0, 6.0], [8.0, 7.0, 4.0], [4.0, 
3.0, 5.0]], [[7.0, 1.0, 3.0], [1.0, 4.0, 3.0], [5.0, 3.0, 3.0]]], [[[2.0, 8.0, 1.0], [5.0, 5.0, 6.0], [5.0, 4.0, 1.0]], [[4.0, 
4.0, 1.0], [8.0, 2.0, 5.0], [4.0, 6.0, 2.0]], [[3.0, 8.0, 6.0], [5.0, 7.0, 3.0], [7.0, 2.0, 2.0]]], [[[5.0, 6.0, 7.0], [5.0, 
6.0, 1.0], [8.0, 7.0, 8.0]], [[8.0, 6.0, 4.0], [8.0, 3.0, 6.0], [7.0, 6.0, 5.0]], [[6.0, 8.0, 7.0], [1.0, 3.0, 8.0], [3.0, 3.0, 
8.0]]], [[[1.0, 2.0, 3.0], [1.0, 5.0, 7.0], [8.0, 6.0, 6.0]], [[7.0, 7.0, 2.0], [2.0, 5.0, 1.0], [6.0, 7.0, 8.0]], [[2.0, 2.0, 
7.0], [4.0, 4.0, 7.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 5.0], [7.0, 6.0, 3.0], [8.0, 5.0, 5.0]], 
[[4.0, 8.0, 2.0], [7.0, 8.0, 4.0], [6.0, 4.0, 2.0]], [[1.0, 2.0, 5.0], [7.0, 5.0, 1.0], [8.0, 4.0, 8.0]]], [[[3.0, 1.0, 7.0], 
[1.0, 7.0, 4.0], [5.0, 4.0, 1.0]], [[4.0, 2.0, 2.0], [5.0, 8.0, 2.0], [4.0, 6.0, 2.0]], [[3.0, 4.0, 7.0], [6.0, 6.0, 3.0], 
[7.0, 2.0, 2.0]]], [[[6.0, 3.0, 1.0], [4.0, 1.0, 6.0], [8.0, 7.0, 8.0]], [[4.0, 7.0, 4.0], [6.0, 7.0, 1.0], [7.0, 6.0, 5.0]], 
[[5.0, 1.0, 4.0], [5.0, 7.0, 2.0], [3.0, 3.0, 8.0]]], [[[1.0, 2.0, 2.0], [4.0, 3.0, 4.0], [4.0, 3.0, 7.0]], [[6.0, 7.0, 6.0], 
[6.0, 1.0, 6.0], [6.0, 8.0, 5.0]], [[6.0, 7.0, 3.0], [4.0, 6.0, 8.0], [6.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RCLN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 3.0], [7.0, 1.0, 8.0], [4.0, 3.0, 6.0]], [[4.0, 4.0, 1.0], [5.0, 4.0, 5.0], [4.0, 2.0, 
2.0]], [[2.0, 1.0, 6.0], [2.0, 8.0, 1.0], [8.0, 5.0, 4.0]]], [[[4.0, 5.0, 8.0], [7.0, 7.0, 4.0], [2.0, 7.0, 1.0]], [[6.0, 8.0, 
8.0], [8.0, 5.0, 4.0], [5.0, 8.0, 8.0]], [[2.0, 4.0, 4.0], [1.0, 8.0, 8.0], [6.0, 8.0, 7.0]]], [[[7.0, 5.0, 3.0], [3.0, 5.0, 
8.0], [6.0, 3.0, 8.0]], [[3.0, 3.0, 8.0], [8.0, 4.0, 3.0], [5.0, 2.0, 4.0]], [[1.0, 1.0, 3.0], [5.0, 1.0, 5.0], [2.0, 7.0, 
8.0]]], [[[6.0, 2.0, 7.0], [7.0, 7.0, 2.0], [5.0, 4.0, 3.0]], [[7.0, 8.0, 3.0], [2.0, 7.0, 1.0], [6.0, 6.0, 2.0]], [[1.0, 4.0, 
5.0], [4.0, 6.0, 7.0], [3.0, 8.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 2.0], [5.0, 6.0, 8.0], [5.0, 4.0, 5.0]], [[6.0, 1.0, 6.0], [4.0, 6.0, 
7.0], [6.0, 8.0, 8.0]], [[7.0, 7.0, 6.0], [4.0, 7.0, 2.0], [6.0, 5.0, 6.0]]], [[[5.0, 7.0, 8.0], [3.0, 6.0, 2.0], [4.0, 2.0, 
1.0]], [[8.0, 3.0, 1.0], [3.0, 1.0, 3.0], [5.0, 8.0, 5.0]], [[1.0, 3.0, 3.0], [2.0, 4.0, 5.0], [1.0, 3.0, 5.0]]], [[[2.0, 4.0, 
6.0], [7.0, 2.0, 5.0], [2.0, 1.0, 6.0]], [[3.0, 3.0, 6.0], [7.0, 7.0, 4.0], [8.0, 2.0, 3.0]], [[1.0, 8.0, 8.0], [4.0, 5.0, 
1.0], [7.0, 8.0, 6.0]]], [[[1.0, 5.0, 2.0], [7.0, 8.0, 2.0], [2.0, 1.0, 5.0]], [[2.0, 6.0, 2.0], [4.0, 8.0, 8.0], [5.0, 3.0, 
5.0]], [[5.0, 2.0, 6.0], [4.0, 3.0, 6.0], [4.0, 2.0, 5.0]]]]))
      val=Data(numpy.array([[[[4.0, 2.0, 7.0]], [[3.0, 6.0, 3.0]], [[7.0, 6.0, 6.0]]], [[[2.0, 8.0, 3.0]], [[4.0, 1.0, 6.0]], 
[[4.0, 5.0, 3.0]]]]),self.functionspace)
      arg[1:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 3.0], [7.0, 1.0, 8.0], [4.0, 3.0, 6.0]], [[4.0, 4.0, 1.0], [5.0, 4.0, 5.0], [4.0, 
2.0, 2.0]], [[2.0, 1.0, 6.0], [2.0, 8.0, 1.0], [8.0, 5.0, 4.0]]], [[[4.0, 5.0, 8.0], [7.0, 7.0, 4.0], [4.0, 2.0, 7.0]], [[6.0, 
8.0, 8.0], [8.0, 5.0, 4.0], [3.0, 6.0, 3.0]], [[2.0, 4.0, 4.0], [1.0, 8.0, 8.0], [7.0, 6.0, 6.0]]], [[[7.0, 5.0, 3.0], [3.0, 
5.0, 8.0], [2.0, 8.0, 3.0]], [[3.0, 3.0, 8.0], [8.0, 4.0, 3.0], [4.0, 1.0, 6.0]], [[1.0, 1.0, 3.0], [5.0, 1.0, 5.0], [4.0, 5.0, 
3.0]]], [[[6.0, 2.0, 7.0], [7.0, 7.0, 2.0], [5.0, 4.0, 3.0]], [[7.0, 8.0, 3.0], [2.0, 7.0, 1.0], [6.0, 6.0, 2.0]], [[1.0, 4.0, 
5.0], [4.0, 6.0, 7.0], [3.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 2.0], [5.0, 6.0, 8.0], [5.0, 4.0, 5.0]], 
[[6.0, 1.0, 6.0], [4.0, 6.0, 7.0], [6.0, 8.0, 8.0]], [[7.0, 7.0, 6.0], [4.0, 7.0, 2.0], [6.0, 5.0, 6.0]]], [[[5.0, 7.0, 8.0], 
[3.0, 6.0, 2.0], [4.0, 2.0, 7.0]], [[8.0, 3.0, 1.0], [3.0, 1.0, 3.0], [3.0, 6.0, 3.0]], [[1.0, 3.0, 3.0], [2.0, 4.0, 5.0], 
[7.0, 6.0, 6.0]]], [[[2.0, 4.0, 6.0], [7.0, 2.0, 5.0], [2.0, 8.0, 3.0]], [[3.0, 3.0, 6.0], [7.0, 7.0, 4.0], [4.0, 1.0, 6.0]], 
[[1.0, 8.0, 8.0], [4.0, 5.0, 1.0], [4.0, 5.0, 3.0]]], [[[1.0, 5.0, 2.0], [7.0, 8.0, 2.0], [2.0, 1.0, 5.0]], [[2.0, 6.0, 2.0], 
[4.0, 8.0, 8.0], [5.0, 3.0, 5.0]], [[5.0, 2.0, 6.0], [4.0, 3.0, 6.0], [4.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RCLN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 3.0], [4.0, 1.0, 3.0], [4.0, 8.0, 8.0]], [[2.0, 5.0, 1.0], [7.0, 5.0, 8.0], [7.0, 4.0, 
7.0]], [[1.0, 5.0, 2.0], [8.0, 3.0, 1.0], [4.0, 7.0, 5.0]]], [[[5.0, 8.0, 5.0], [8.0, 3.0, 8.0], [1.0, 3.0, 8.0]], [[6.0, 1.0, 
2.0], [5.0, 8.0, 3.0], [4.0, 1.0, 3.0]], [[2.0, 2.0, 8.0], [8.0, 8.0, 2.0], [8.0, 1.0, 2.0]]], [[[7.0, 5.0, 8.0], [3.0, 8.0, 
5.0], [8.0, 6.0, 1.0]], [[5.0, 4.0, 1.0], [3.0, 1.0, 5.0], [5.0, 1.0, 1.0]], [[5.0, 3.0, 1.0], [7.0, 4.0, 7.0], [2.0, 1.0, 
8.0]]], [[[2.0, 2.0, 6.0], [8.0, 2.0, 1.0], [2.0, 6.0, 6.0]], [[6.0, 4.0, 1.0], [6.0, 1.0, 5.0], [5.0, 6.0, 3.0]], [[2.0, 8.0, 
8.0], [6.0, 6.0, 4.0], [8.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 5.0], [4.0, 6.0, 1.0], [3.0, 5.0, 2.0]], [[3.0, 5.0, 1.0], [2.0, 7.0, 
3.0], [6.0, 5.0, 7.0]], [[2.0, 8.0, 3.0], [4.0, 8.0, 4.0], [7.0, 6.0, 2.0]]], [[[2.0, 6.0, 2.0], [3.0, 3.0, 2.0], [2.0, 7.0, 
5.0]], [[4.0, 6.0, 8.0], [3.0, 3.0, 3.0], [3.0, 7.0, 5.0]], [[7.0, 2.0, 7.0], [7.0, 4.0, 2.0], [5.0, 6.0, 1.0]]], [[[3.0, 7.0, 
7.0], [2.0, 8.0, 8.0], [7.0, 3.0, 5.0]], [[8.0, 2.0, 8.0], [5.0, 7.0, 4.0], [8.0, 1.0, 2.0]], [[1.0, 8.0, 1.0], [6.0, 7.0, 
5.0], [5.0, 6.0, 6.0]]], [[[7.0, 3.0, 3.0], [1.0, 8.0, 2.0], [3.0, 7.0, 1.0]], [[4.0, 1.0, 4.0], [4.0, 8.0, 6.0], [3.0, 4.0, 
2.0]], [[5.0, 5.0, 6.0], [1.0, 2.0, 5.0], [6.0, 4.0, 7.0]]]]))
      val=Data(numpy.array([[[[8.0, 8.0, 6.0]], [[6.0, 1.0, 3.0]], [[1.0, 5.0, 7.0]]], [[[5.0, 8.0, 7.0]], [[3.0, 7.0, 5.0]], 
[[3.0, 8.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 7.0]], [[8.0, 4.0, 8.0]], [[6.0, 5.0, 1.0]]], [[[8.0, 5.0, 6.0]], [[3.0, 
3.0, 3.0]], [[8.0, 6.0, 3.0]]]]))
      arg[1:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 3.0], [4.0, 1.0, 3.0], [4.0, 8.0, 8.0]], [[2.0, 5.0, 1.0], [7.0, 5.0, 8.0], [7.0, 
4.0, 7.0]], [[1.0, 5.0, 2.0], [8.0, 3.0, 1.0], [4.0, 7.0, 5.0]]], [[[5.0, 8.0, 5.0], [8.0, 3.0, 8.0], [8.0, 8.0, 6.0]], [[6.0, 
1.0, 2.0], [5.0, 8.0, 3.0], [6.0, 1.0, 3.0]], [[2.0, 2.0, 8.0], [8.0, 8.0, 2.0], [1.0, 5.0, 7.0]]], [[[7.0, 5.0, 8.0], [3.0, 
8.0, 5.0], [5.0, 8.0, 7.0]], [[5.0, 4.0, 1.0], [3.0, 1.0, 5.0], [3.0, 7.0, 5.0]], [[5.0, 3.0, 1.0], [7.0, 4.0, 7.0], [3.0, 8.0, 
7.0]]], [[[2.0, 2.0, 6.0], [8.0, 2.0, 1.0], [2.0, 6.0, 6.0]], [[6.0, 4.0, 1.0], [6.0, 1.0, 5.0], [5.0, 6.0, 3.0]], [[2.0, 8.0, 
8.0], [6.0, 6.0, 4.0], [8.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 5.0], [4.0, 6.0, 1.0], [3.0, 5.0, 2.0]], 
[[3.0, 5.0, 1.0], [2.0, 7.0, 3.0], [6.0, 5.0, 7.0]], [[2.0, 8.0, 3.0], [4.0, 8.0, 4.0], [7.0, 6.0, 2.0]]], [[[2.0, 6.0, 2.0], 
[3.0, 3.0, 2.0], [5.0, 5.0, 7.0]], [[4.0, 6.0, 8.0], [3.0, 3.0, 3.0], [8.0, 4.0, 8.0]], [[7.0, 2.0, 7.0], [7.0, 4.0, 2.0], 
[6.0, 5.0, 1.0]]], [[[3.0, 7.0, 7.0], [2.0, 8.0, 8.0], [8.0, 5.0, 6.0]], [[8.0, 2.0, 8.0], [5.0, 7.0, 4.0], [3.0, 3.0, 3.0]], 
[[1.0, 8.0, 1.0], [6.0, 7.0, 5.0], [8.0, 6.0, 3.0]]], [[[7.0, 3.0, 3.0], [1.0, 8.0, 2.0], [3.0, 7.0, 1.0]], [[4.0, 1.0, 4.0], 
[4.0, 8.0, 6.0], [3.0, 4.0, 2.0]], [[5.0, 5.0, 6.0], [1.0, 2.0, 5.0], [6.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RCLN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 4.0], [6.0, 7.0, 1.0], [1.0, 1.0, 4.0]], [[3.0, 8.0, 3.0], [3.0, 1.0, 2.0], [1.0, 5.0, 
4.0]], [[6.0, 5.0, 1.0], [2.0, 7.0, 4.0], [7.0, 2.0, 2.0]]], [[[8.0, 3.0, 5.0], [5.0, 5.0, 8.0], [4.0, 1.0, 6.0]], [[7.0, 4.0, 
7.0], [6.0, 4.0, 6.0], [7.0, 3.0, 5.0]], [[2.0, 6.0, 6.0], [3.0, 3.0, 3.0], [5.0, 7.0, 4.0]]], [[[4.0, 5.0, 6.0], [8.0, 1.0, 
3.0], [5.0, 5.0, 7.0]], [[6.0, 5.0, 3.0], [2.0, 4.0, 7.0], [8.0, 8.0, 6.0]], [[5.0, 3.0, 5.0], [1.0, 5.0, 6.0], [3.0, 4.0, 
3.0]]], [[[8.0, 5.0, 6.0], [7.0, 1.0, 5.0], [5.0, 1.0, 1.0]], [[6.0, 7.0, 6.0], [6.0, 1.0, 6.0], [6.0, 7.0, 8.0]], [[5.0, 4.0, 
3.0], [7.0, 1.0, 3.0], [8.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 7.0], [4.0, 2.0, 5.0], [6.0, 4.0, 8.0]], [[4.0, 2.0, 1.0], [1.0, 5.0, 
5.0], [8.0, 4.0, 8.0]], [[8.0, 5.0, 4.0], [8.0, 2.0, 3.0], [7.0, 6.0, 8.0]]], [[[2.0, 8.0, 1.0], [3.0, 1.0, 8.0], [8.0, 4.0, 
1.0]], [[5.0, 8.0, 1.0], [8.0, 7.0, 1.0], [2.0, 6.0, 4.0]], [[3.0, 5.0, 8.0], [7.0, 8.0, 8.0], [5.0, 4.0, 6.0]]], [[[4.0, 7.0, 
4.0], [6.0, 8.0, 4.0], [6.0, 3.0, 5.0]], [[8.0, 5.0, 4.0], [2.0, 3.0, 3.0], [5.0, 4.0, 5.0]], [[4.0, 1.0, 8.0], [6.0, 5.0, 
7.0], [5.0, 4.0, 1.0]]], [[[5.0, 4.0, 1.0], [4.0, 8.0, 6.0], [8.0, 3.0, 7.0]], [[6.0, 2.0, 8.0], [5.0, 7.0, 8.0], [5.0, 1.0, 
8.0]], [[6.0, 7.0, 2.0], [5.0, 7.0, 1.0], [1.0, 3.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 6.0, 2.0]], [[2.0, 5.0, 1.0]], [[1.0, 4.0, 8.0]]], [[[2.0, 6.0, 8.0]], [[1.0, 7.0, 
6.0]], [[6.0, 6.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 6.0, 6.0]], [[6.0, 7.0, 4.0]], [[8.0, 1.0, 1.0]]], [[[3.0, 6.0, 
2.0]], [[5.0, 4.0, 6.0]], [[4.0, 7.0, 3.0]]]])
      arg[1:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 4.0], [6.0, 7.0, 1.0], [1.0, 1.0, 4.0]], [[3.0, 8.0, 3.0], [3.0, 1.0, 2.0], [1.0, 
5.0, 4.0]], [[6.0, 5.0, 1.0], [2.0, 7.0, 4.0], [7.0, 2.0, 2.0]]], [[[8.0, 3.0, 5.0], [5.0, 5.0, 8.0], [3.0, 6.0, 2.0]], [[7.0, 
4.0, 7.0], [6.0, 4.0, 6.0], [2.0, 5.0, 1.0]], [[2.0, 6.0, 6.0], [3.0, 3.0, 3.0], [1.0, 4.0, 8.0]]], [[[4.0, 5.0, 6.0], [8.0, 
1.0, 3.0], [2.0, 6.0, 8.0]], [[6.0, 5.0, 3.0], [2.0, 4.0, 7.0], [1.0, 7.0, 6.0]], [[5.0, 3.0, 5.0], [1.0, 5.0, 6.0], [6.0, 6.0, 
5.0]]], [[[8.0, 5.0, 6.0], [7.0, 1.0, 5.0], [5.0, 1.0, 1.0]], [[6.0, 7.0, 6.0], [6.0, 1.0, 6.0], [6.0, 7.0, 8.0]], [[5.0, 4.0, 
3.0], [7.0, 1.0, 3.0], [8.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 7.0], [4.0, 2.0, 5.0], [6.0, 4.0, 8.0]], 
[[4.0, 2.0, 1.0], [1.0, 5.0, 5.0], [8.0, 4.0, 8.0]], [[8.0, 5.0, 4.0], [8.0, 2.0, 3.0], [7.0, 6.0, 8.0]]], [[[2.0, 8.0, 1.0], 
[3.0, 1.0, 8.0], [8.0, 6.0, 6.0]], [[5.0, 8.0, 1.0], [8.0, 7.0, 1.0], [6.0, 7.0, 4.0]], [[3.0, 5.0, 8.0], [7.0, 8.0, 8.0], 
[8.0, 1.0, 1.0]]], [[[4.0, 7.0, 4.0], [6.0, 8.0, 4.0], [3.0, 6.0, 2.0]], [[8.0, 5.0, 4.0], [2.0, 3.0, 3.0], [5.0, 4.0, 6.0]], 
[[4.0, 1.0, 8.0], [6.0, 5.0, 7.0], [4.0, 7.0, 3.0]]], [[[5.0, 4.0, 1.0], [4.0, 8.0, 6.0], [8.0, 3.0, 7.0]], [[6.0, 2.0, 8.0], 
[5.0, 7.0, 8.0], [5.0, 1.0, 8.0]], [[6.0, 7.0, 2.0], [5.0, 7.0, 1.0], [1.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UUPP(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 8.0], [5.0, 6.0, 7.0], [1.0, 3.0, 4.0]], [[8.0, 8.0, 1.0], [2.0, 8.0, 1.0], [4.0, 7.0, 
6.0]], [[5.0, 4.0, 2.0], [7.0, 2.0, 8.0], [5.0, 3.0, 3.0]]], [[[8.0, 5.0, 1.0], [8.0, 4.0, 1.0], [5.0, 4.0, 7.0]], [[2.0, 3.0, 
2.0], [1.0, 2.0, 7.0], [8.0, 4.0, 1.0]], [[7.0, 5.0, 5.0], [5.0, 1.0, 5.0], [1.0, 4.0, 3.0]]], [[[2.0, 2.0, 3.0], [7.0, 6.0, 
6.0], [8.0, 1.0, 8.0]], [[1.0, 8.0, 2.0], [2.0, 3.0, 8.0], [1.0, 6.0, 3.0]], [[3.0, 6.0, 6.0], [4.0, 3.0, 5.0], [5.0, 2.0, 
8.0]]], [[[7.0, 6.0, 8.0], [5.0, 7.0, 8.0], [7.0, 5.0, 4.0]], [[7.0, 3.0, 5.0], [7.0, 3.0, 7.0], [7.0, 1.0, 1.0]], [[2.0, 3.0, 
7.0], [6.0, 6.0, 7.0], [2.0, 3.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 6.0], [3.0, 3.0, 7.0], [3.0, 1.0, 4.0]], [[6.0, 5.0, 6.0], [5.0, 4.0, 
2.0], [2.0, 2.0, 2.0]], [[4.0, 1.0, 8.0], [5.0, 5.0, 3.0], [5.0, 8.0, 2.0]]], [[[8.0, 3.0, 2.0], [7.0, 4.0, 3.0], [5.0, 4.0, 
7.0]], [[3.0, 4.0, 8.0], [7.0, 3.0, 5.0], [4.0, 5.0, 4.0]], [[8.0, 1.0, 6.0], [6.0, 5.0, 2.0], [1.0, 7.0, 8.0]]], [[[1.0, 6.0, 
5.0], [7.0, 8.0, 8.0], [6.0, 8.0, 2.0]], [[1.0, 3.0, 8.0], [4.0, 8.0, 3.0], [7.0, 6.0, 5.0]], [[4.0, 4.0, 6.0], [6.0, 1.0, 
1.0], [3.0, 4.0, 6.0]]], [[[5.0, 6.0, 5.0], [8.0, 8.0, 7.0], [3.0, 4.0, 5.0]], [[2.0, 6.0, 2.0], [3.0, 1.0, 1.0], [1.0, 3.0, 
4.0]], [[4.0, 6.0, 4.0], [2.0, 6.0, 2.0], [8.0, 4.0, 1.0]]]]))
      val=numpy.array([[4.0, 3.0], [3.0, 8.0], [7.0, 6.0]])
      arg[:3,:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 8.0], [5.0, 4.0, 7.0], [1.0, 3.0, 4.0]], [[8.0, 8.0, 1.0], [2.0, 3.0, 1.0], [4.0, 
7.0, 6.0]], [[5.0, 4.0, 2.0], [7.0, 2.0, 8.0], [5.0, 3.0, 3.0]]], [[[8.0, 5.0, 1.0], [8.0, 3.0, 1.0], [5.0, 4.0, 7.0]], [[2.0, 
3.0, 2.0], [1.0, 8.0, 7.0], [8.0, 4.0, 1.0]], [[7.0, 5.0, 5.0], [5.0, 1.0, 5.0], [1.0, 4.0, 3.0]]], [[[2.0, 2.0, 3.0], [7.0, 
7.0, 6.0], [8.0, 1.0, 8.0]], [[1.0, 8.0, 2.0], [2.0, 6.0, 8.0], [1.0, 6.0, 3.0]], [[3.0, 6.0, 6.0], [4.0, 3.0, 5.0], [5.0, 2.0, 
8.0]]], [[[7.0, 6.0, 8.0], [5.0, 7.0, 8.0], [7.0, 5.0, 4.0]], [[7.0, 3.0, 5.0], [7.0, 3.0, 7.0], [7.0, 1.0, 1.0]], [[2.0, 3.0, 
7.0], [6.0, 6.0, 7.0], [2.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 6.0], [3.0, 4.0, 7.0], [3.0, 1.0, 4.0]], 
[[6.0, 5.0, 6.0], [5.0, 3.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 1.0, 8.0], [5.0, 5.0, 3.0], [5.0, 8.0, 2.0]]], [[[8.0, 3.0, 2.0], 
[7.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[3.0, 4.0, 8.0], [7.0, 8.0, 5.0], [4.0, 5.0, 4.0]], [[8.0, 1.0, 6.0], [6.0, 5.0, 2.0], 
[1.0, 7.0, 8.0]]], [[[1.0, 6.0, 5.0], [7.0, 7.0, 8.0], [6.0, 8.0, 2.0]], [[1.0, 3.0, 8.0], [4.0, 6.0, 3.0], [7.0, 6.0, 5.0]], 
[[4.0, 4.0, 6.0], [6.0, 1.0, 1.0], [3.0, 4.0, 6.0]]], [[[5.0, 6.0, 5.0], [8.0, 8.0, 7.0], [3.0, 4.0, 5.0]], [[2.0, 6.0, 2.0], 
[3.0, 1.0, 1.0], [1.0, 3.0, 4.0]], [[4.0, 6.0, 4.0], [2.0, 6.0, 2.0], [8.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UUPP(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 5.0], [1.0, 7.0, 5.0], [1.0, 5.0, 8.0]], [[6.0, 1.0, 7.0], [2.0, 4.0, 5.0], [3.0, 3.0, 
7.0]], [[6.0, 8.0, 6.0], [6.0, 4.0, 6.0], [5.0, 1.0, 1.0]]], [[[4.0, 8.0, 2.0], [3.0, 5.0, 6.0], [8.0, 5.0, 3.0]], [[1.0, 1.0, 
6.0], [2.0, 7.0, 3.0], [7.0, 2.0, 6.0]], [[7.0, 8.0, 7.0], [2.0, 7.0, 1.0], [2.0, 8.0, 7.0]]], [[[6.0, 1.0, 6.0], [4.0, 7.0, 
1.0], [4.0, 7.0, 5.0]], [[6.0, 5.0, 3.0], [1.0, 3.0, 7.0], [2.0, 3.0, 8.0]], [[2.0, 1.0, 6.0], [4.0, 2.0, 3.0], [7.0, 7.0, 
4.0]]], [[[6.0, 3.0, 3.0], [5.0, 4.0, 2.0], [8.0, 8.0, 5.0]], [[1.0, 5.0, 1.0], [7.0, 1.0, 1.0], [3.0, 5.0, 6.0]], [[3.0, 5.0, 
1.0], [7.0, 1.0, 8.0], [3.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 6.0], [1.0, 4.0, 6.0], [4.0, 4.0, 2.0]], [[5.0, 1.0, 1.0], [6.0, 8.0, 
3.0], [7.0, 4.0, 2.0]], [[8.0, 4.0, 1.0], [4.0, 3.0, 1.0], [1.0, 7.0, 2.0]]], [[[2.0, 6.0, 1.0], [3.0, 7.0, 1.0], [2.0, 5.0, 
6.0]], [[4.0, 4.0, 1.0], [2.0, 7.0, 5.0], [8.0, 5.0, 5.0]], [[4.0, 5.0, 1.0], [8.0, 5.0, 4.0], [6.0, 6.0, 7.0]]], [[[5.0, 8.0, 
1.0], [1.0, 4.0, 8.0], [2.0, 5.0, 3.0]], [[7.0, 4.0, 3.0], [5.0, 7.0, 4.0], [3.0, 8.0, 7.0]], [[5.0, 2.0, 5.0], [4.0, 6.0, 
8.0], [4.0, 7.0, 2.0]]], [[[4.0, 7.0, 2.0], [7.0, 3.0, 5.0], [7.0, 7.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 2.0, 7.0], [8.0, 1.0, 
4.0]], [[7.0, 1.0, 8.0], [7.0, 4.0, 4.0], [3.0, 8.0, 4.0]]]]))
      val=Data(numpy.array([[5.0, 4.0], [4.0, 5.0], [4.0, 5.0]]),self.functionspace)
      arg[:3,:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 5.0], [1.0, 5.0, 5.0], [1.0, 5.0, 8.0]], [[6.0, 1.0, 7.0], [2.0, 4.0, 5.0], [3.0, 
3.0, 7.0]], [[6.0, 8.0, 6.0], [6.0, 4.0, 6.0], [5.0, 1.0, 1.0]]], [[[4.0, 8.0, 2.0], [3.0, 4.0, 6.0], [8.0, 5.0, 3.0]], [[1.0, 
1.0, 6.0], [2.0, 5.0, 3.0], [7.0, 2.0, 6.0]], [[7.0, 8.0, 7.0], [2.0, 7.0, 1.0], [2.0, 8.0, 7.0]]], [[[6.0, 1.0, 6.0], [4.0, 
4.0, 1.0], [4.0, 7.0, 5.0]], [[6.0, 5.0, 3.0], [1.0, 5.0, 7.0], [2.0, 3.0, 8.0]], [[2.0, 1.0, 6.0], [4.0, 2.0, 3.0], [7.0, 7.0, 
4.0]]], [[[6.0, 3.0, 3.0], [5.0, 4.0, 2.0], [8.0, 8.0, 5.0]], [[1.0, 5.0, 1.0], [7.0, 1.0, 1.0], [3.0, 5.0, 6.0]], [[3.0, 5.0, 
1.0], [7.0, 1.0, 8.0], [3.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 6.0], [1.0, 5.0, 6.0], [4.0, 4.0, 2.0]], 
[[5.0, 1.0, 1.0], [6.0, 4.0, 3.0], [7.0, 4.0, 2.0]], [[8.0, 4.0, 1.0], [4.0, 3.0, 1.0], [1.0, 7.0, 2.0]]], [[[2.0, 6.0, 1.0], 
[3.0, 4.0, 1.0], [2.0, 5.0, 6.0]], [[4.0, 4.0, 1.0], [2.0, 5.0, 5.0], [8.0, 5.0, 5.0]], [[4.0, 5.0, 1.0], [8.0, 5.0, 4.0], 
[6.0, 6.0, 7.0]]], [[[5.0, 8.0, 1.0], [1.0, 4.0, 8.0], [2.0, 5.0, 3.0]], [[7.0, 4.0, 3.0], [5.0, 5.0, 4.0], [3.0, 8.0, 7.0]], 
[[5.0, 2.0, 5.0], [4.0, 6.0, 8.0], [4.0, 7.0, 2.0]]], [[[4.0, 7.0, 2.0], [7.0, 3.0, 5.0], [7.0, 7.0, 3.0]], [[7.0, 7.0, 1.0], 
[5.0, 2.0, 7.0], [8.0, 1.0, 4.0]], [[7.0, 1.0, 8.0], [7.0, 4.0, 4.0], [3.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UUPP(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 5.0], [7.0, 6.0, 3.0], [5.0, 8.0, 4.0]], [[3.0, 1.0, 7.0], [5.0, 3.0, 1.0], [5.0, 8.0, 
7.0]], [[1.0, 7.0, 8.0], [6.0, 2.0, 2.0], [1.0, 2.0, 5.0]]], [[[8.0, 8.0, 7.0], [7.0, 7.0, 5.0], [4.0, 1.0, 4.0]], [[6.0, 5.0, 
6.0], [5.0, 4.0, 7.0], [1.0, 1.0, 2.0]], [[6.0, 3.0, 1.0], [4.0, 7.0, 2.0], [2.0, 7.0, 7.0]]], [[[5.0, 1.0, 7.0], [1.0, 8.0, 
2.0], [3.0, 8.0, 5.0]], [[3.0, 3.0, 6.0], [5.0, 3.0, 8.0], [7.0, 4.0, 4.0]], [[7.0, 6.0, 1.0], [7.0, 8.0, 5.0], [4.0, 7.0, 
3.0]]], [[[8.0, 7.0, 2.0], [8.0, 7.0, 1.0], [8.0, 2.0, 5.0]], [[3.0, 5.0, 6.0], [5.0, 5.0, 1.0], [7.0, 8.0, 7.0]], [[6.0, 2.0, 
3.0], [2.0, 6.0, 8.0], [2.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 3.0], [6.0, 5.0, 3.0], [2.0, 3.0, 2.0]], [[5.0, 4.0, 7.0], [4.0, 2.0, 
6.0], [6.0, 6.0, 1.0]], [[6.0, 3.0, 6.0], [2.0, 5.0, 3.0], [5.0, 7.0, 4.0]]], [[[4.0, 2.0, 4.0], [3.0, 5.0, 3.0], [4.0, 8.0, 
3.0]], [[6.0, 1.0, 2.0], [1.0, 7.0, 7.0], [3.0, 1.0, 4.0]], [[6.0, 8.0, 7.0], [7.0, 3.0, 2.0], [6.0, 8.0, 7.0]]], [[[4.0, 7.0, 
2.0], [1.0, 7.0, 6.0], [3.0, 6.0, 4.0]], [[6.0, 4.0, 6.0], [4.0, 4.0, 4.0], [7.0, 5.0, 3.0]], [[2.0, 6.0, 3.0], [5.0, 8.0, 
1.0], [6.0, 1.0, 3.0]]], [[[5.0, 4.0, 1.0], [7.0, 3.0, 7.0], [7.0, 2.0, 6.0]], [[7.0, 1.0, 8.0], [8.0, 2.0, 3.0], [4.0, 1.0, 
7.0]], [[8.0, 4.0, 6.0], [6.0, 6.0, 6.0], [7.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[4.0, 4.0], [6.0, 3.0], [4.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 2.0], [2.0, 2.0], [8.0, 8.0]]))
      arg[:3,:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 5.0], [7.0, 4.0, 3.0], [5.0, 8.0, 4.0]], [[3.0, 1.0, 7.0], [5.0, 4.0, 1.0], [5.0, 
8.0, 7.0]], [[1.0, 7.0, 8.0], [6.0, 2.0, 2.0], [1.0, 2.0, 5.0]]], [[[8.0, 8.0, 7.0], [7.0, 6.0, 5.0], [4.0, 1.0, 4.0]], [[6.0, 
5.0, 6.0], [5.0, 3.0, 7.0], [1.0, 1.0, 2.0]], [[6.0, 3.0, 1.0], [4.0, 7.0, 2.0], [2.0, 7.0, 7.0]]], [[[5.0, 1.0, 7.0], [1.0, 
4.0, 2.0], [3.0, 8.0, 5.0]], [[3.0, 3.0, 6.0], [5.0, 8.0, 8.0], [7.0, 4.0, 4.0]], [[7.0, 6.0, 1.0], [7.0, 8.0, 5.0], [4.0, 7.0, 
3.0]]], [[[8.0, 7.0, 2.0], [8.0, 7.0, 1.0], [8.0, 2.0, 5.0]], [[3.0, 5.0, 6.0], [5.0, 5.0, 1.0], [7.0, 8.0, 7.0]], [[6.0, 2.0, 
3.0], [2.0, 6.0, 8.0], [2.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 3.0], [6.0, 6.0, 3.0], [2.0, 3.0, 2.0]], 
[[5.0, 4.0, 7.0], [4.0, 2.0, 6.0], [6.0, 6.0, 1.0]], [[6.0, 3.0, 6.0], [2.0, 5.0, 3.0], [5.0, 7.0, 4.0]]], [[[4.0, 2.0, 4.0], 
[3.0, 2.0, 3.0], [4.0, 8.0, 3.0]], [[6.0, 1.0, 2.0], [1.0, 2.0, 7.0], [3.0, 1.0, 4.0]], [[6.0, 8.0, 7.0], [7.0, 3.0, 2.0], 
[6.0, 8.0, 7.0]]], [[[4.0, 7.0, 2.0], [1.0, 8.0, 6.0], [3.0, 6.0, 4.0]], [[6.0, 4.0, 6.0], [4.0, 8.0, 4.0], [7.0, 5.0, 3.0]], 
[[2.0, 6.0, 3.0], [5.0, 8.0, 1.0], [6.0, 1.0, 3.0]]], [[[5.0, 4.0, 1.0], [7.0, 3.0, 7.0], [7.0, 2.0, 6.0]], [[7.0, 1.0, 8.0], 
[8.0, 2.0, 3.0], [4.0, 1.0, 7.0]], [[8.0, 4.0, 6.0], [6.0, 6.0, 6.0], [7.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UUPP(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 8.0], [5.0, 6.0, 3.0], [5.0, 3.0, 4.0]], [[2.0, 6.0, 2.0], [7.0, 7.0, 4.0], [7.0, 2.0, 
6.0]], [[3.0, 2.0, 8.0], [7.0, 2.0, 5.0], [2.0, 2.0, 5.0]]], [[[8.0, 7.0, 5.0], [5.0, 3.0, 6.0], [8.0, 7.0, 2.0]], [[2.0, 8.0, 
2.0], [6.0, 7.0, 6.0], [4.0, 2.0, 4.0]], [[2.0, 4.0, 7.0], [1.0, 8.0, 5.0], [4.0, 8.0, 6.0]]], [[[4.0, 3.0, 1.0], [8.0, 6.0, 
7.0], [2.0, 6.0, 2.0]], [[6.0, 8.0, 8.0], [1.0, 6.0, 7.0], [8.0, 4.0, 6.0]], [[2.0, 7.0, 8.0], [3.0, 5.0, 3.0], [1.0, 1.0, 
6.0]]], [[[7.0, 2.0, 3.0], [5.0, 8.0, 1.0], [8.0, 3.0, 3.0]], [[1.0, 6.0, 2.0], [4.0, 6.0, 1.0], [2.0, 8.0, 7.0]], [[4.0, 3.0, 
2.0], [7.0, 4.0, 8.0], [7.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 7.0], [3.0, 2.0, 3.0], [5.0, 4.0, 8.0]], [[3.0, 7.0, 6.0], [5.0, 3.0, 
6.0], [5.0, 4.0, 4.0]], [[4.0, 6.0, 4.0], [5.0, 4.0, 6.0], [5.0, 6.0, 5.0]]], [[[8.0, 7.0, 8.0], [7.0, 2.0, 4.0], [4.0, 8.0, 
5.0]], [[3.0, 3.0, 7.0], [3.0, 4.0, 6.0], [5.0, 7.0, 2.0]], [[4.0, 3.0, 6.0], [8.0, 8.0, 7.0], [7.0, 5.0, 6.0]]], [[[1.0, 4.0, 
8.0], [8.0, 5.0, 6.0], [7.0, 1.0, 7.0]], [[8.0, 8.0, 8.0], [5.0, 3.0, 8.0], [1.0, 4.0, 7.0]], [[4.0, 7.0, 7.0], [3.0, 3.0, 
1.0], [6.0, 8.0, 8.0]]], [[[2.0, 6.0, 4.0], [2.0, 2.0, 5.0], [8.0, 3.0, 8.0]], [[2.0, 1.0, 2.0], [6.0, 8.0, 4.0], [5.0, 5.0, 
2.0]], [[5.0, 4.0, 4.0], [4.0, 7.0, 1.0], [7.0, 1.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[5.0, 6.0], [3.0, 5.0], [5.0, 7.0]])+(1.-msk_val)*numpy.array([[2.0, 4.0], [7.0, 2.0], [6.0, 
2.0]])
      arg[:3,:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 8.0], [5.0, 5.0, 3.0], [5.0, 3.0, 4.0]], [[2.0, 6.0, 2.0], [7.0, 6.0, 4.0], [7.0, 
2.0, 6.0]], [[3.0, 2.0, 8.0], [7.0, 2.0, 5.0], [2.0, 2.0, 5.0]]], [[[8.0, 7.0, 5.0], [5.0, 3.0, 6.0], [8.0, 7.0, 2.0]], [[2.0, 
8.0, 2.0], [6.0, 5.0, 6.0], [4.0, 2.0, 4.0]], [[2.0, 4.0, 7.0], [1.0, 8.0, 5.0], [4.0, 8.0, 6.0]]], [[[4.0, 3.0, 1.0], [8.0, 
5.0, 7.0], [2.0, 6.0, 2.0]], [[6.0, 8.0, 8.0], [1.0, 7.0, 7.0], [8.0, 4.0, 6.0]], [[2.0, 7.0, 8.0], [3.0, 5.0, 3.0], [1.0, 1.0, 
6.0]]], [[[7.0, 2.0, 3.0], [5.0, 8.0, 1.0], [8.0, 3.0, 3.0]], [[1.0, 6.0, 2.0], [4.0, 6.0, 1.0], [2.0, 8.0, 7.0]], [[4.0, 3.0, 
2.0], [7.0, 4.0, 8.0], [7.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 7.0], [3.0, 2.0, 3.0], [5.0, 4.0, 8.0]], 
[[3.0, 7.0, 6.0], [5.0, 4.0, 6.0], [5.0, 4.0, 4.0]], [[4.0, 6.0, 4.0], [5.0, 4.0, 6.0], [5.0, 6.0, 5.0]]], [[[8.0, 7.0, 8.0], 
[7.0, 7.0, 4.0], [4.0, 8.0, 5.0]], [[3.0, 3.0, 7.0], [3.0, 2.0, 6.0], [5.0, 7.0, 2.0]], [[4.0, 3.0, 6.0], [8.0, 8.0, 7.0], 
[7.0, 5.0, 6.0]]], [[[1.0, 4.0, 8.0], [8.0, 6.0, 6.0], [7.0, 1.0, 7.0]], [[8.0, 8.0, 8.0], [5.0, 2.0, 8.0], [1.0, 4.0, 7.0]], 
[[4.0, 7.0, 7.0], [3.0, 3.0, 1.0], [6.0, 8.0, 8.0]]], [[[2.0, 6.0, 4.0], [2.0, 2.0, 5.0], [8.0, 3.0, 8.0]], [[2.0, 1.0, 2.0], 
[6.0, 8.0, 4.0], [5.0, 5.0, 2.0]], [[5.0, 4.0, 4.0], [4.0, 7.0, 1.0], [7.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PRCC(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 7.0], [3.0, 8.0, 7.0], [7.0, 2.0, 1.0]], [[5.0, 2.0, 8.0], [2.0, 8.0, 4.0], [7.0, 2.0, 
5.0]], [[6.0, 4.0, 8.0], [7.0, 7.0, 8.0], [4.0, 7.0, 3.0]]], [[[1.0, 5.0, 1.0], [1.0, 8.0, 7.0], [3.0, 7.0, 7.0]], [[8.0, 2.0, 
4.0], [8.0, 2.0, 7.0], [2.0, 3.0, 8.0]], [[8.0, 2.0, 1.0], [6.0, 2.0, 3.0], [3.0, 2.0, 1.0]]], [[[5.0, 5.0, 1.0], [3.0, 1.0, 
2.0], [2.0, 5.0, 6.0]], [[3.0, 6.0, 5.0], [4.0, 5.0, 4.0], [5.0, 6.0, 7.0]], [[1.0, 4.0, 8.0], [1.0, 3.0, 8.0], [1.0, 3.0, 
7.0]]], [[[2.0, 4.0, 6.0], [1.0, 8.0, 5.0], [8.0, 1.0, 8.0]], [[4.0, 2.0, 3.0], [6.0, 5.0, 2.0], [4.0, 4.0, 1.0]], [[5.0, 1.0, 
7.0], [1.0, 6.0, 8.0], [1.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 7.0], [8.0, 7.0, 2.0], [4.0, 7.0, 1.0]], [[5.0, 3.0, 3.0], [3.0, 4.0, 
8.0], [7.0, 5.0, 3.0]], [[8.0, 3.0, 7.0], [2.0, 8.0, 6.0], [1.0, 5.0, 2.0]]], [[[6.0, 2.0, 3.0], [8.0, 7.0, 6.0], [1.0, 1.0, 
2.0]], [[7.0, 4.0, 6.0], [5.0, 6.0, 3.0], [1.0, 3.0, 5.0]], [[5.0, 7.0, 2.0], [5.0, 6.0, 4.0], [2.0, 1.0, 6.0]]], [[[8.0, 3.0, 
6.0], [1.0, 6.0, 1.0], [5.0, 5.0, 6.0]], [[4.0, 8.0, 2.0], [2.0, 1.0, 4.0], [8.0, 4.0, 2.0]], [[4.0, 8.0, 3.0], [1.0, 2.0, 
5.0], [5.0, 6.0, 3.0]]], [[[1.0, 7.0, 8.0], [8.0, 1.0, 5.0], [6.0, 4.0, 1.0]], [[5.0, 1.0, 5.0], [5.0, 3.0, 1.0], [1.0, 3.0, 
7.0]], [[2.0, 1.0, 6.0], [8.0, 3.0, 8.0], [3.0, 3.0, 4.0]]]]))
      val=numpy.array([[[8.0, 6.0, 6.0], [6.0, 1.0, 6.0], [5.0, 5.0, 3.0]]])
      arg[2,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 7.0], [3.0, 8.0, 7.0], [7.0, 2.0, 1.0]], [[5.0, 2.0, 8.0], [2.0, 8.0, 4.0], [7.0, 
2.0, 5.0]], [[6.0, 4.0, 8.0], [7.0, 7.0, 8.0], [4.0, 7.0, 3.0]]], [[[1.0, 5.0, 1.0], [1.0, 8.0, 7.0], [3.0, 7.0, 7.0]], [[8.0, 
2.0, 4.0], [8.0, 2.0, 7.0], [2.0, 3.0, 8.0]], [[8.0, 2.0, 1.0], [6.0, 2.0, 3.0], [3.0, 2.0, 1.0]]], [[[5.0, 5.0, 1.0], [3.0, 
1.0, 2.0], [2.0, 5.0, 6.0]], [[8.0, 6.0, 6.0], [6.0, 1.0, 6.0], [5.0, 5.0, 3.0]], [[1.0, 4.0, 8.0], [1.0, 3.0, 8.0], [1.0, 3.0, 
7.0]]], [[[2.0, 4.0, 6.0], [1.0, 8.0, 5.0], [8.0, 1.0, 8.0]], [[4.0, 2.0, 3.0], [6.0, 5.0, 2.0], [4.0, 4.0, 1.0]], [[5.0, 1.0, 
7.0], [1.0, 6.0, 8.0], [1.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 7.0], [8.0, 7.0, 2.0], [4.0, 7.0, 1.0]], 
[[5.0, 3.0, 3.0], [3.0, 4.0, 8.0], [7.0, 5.0, 3.0]], [[8.0, 3.0, 7.0], [2.0, 8.0, 6.0], [1.0, 5.0, 2.0]]], [[[6.0, 2.0, 3.0], 
[8.0, 7.0, 6.0], [1.0, 1.0, 2.0]], [[7.0, 4.0, 6.0], [5.0, 6.0, 3.0], [1.0, 3.0, 5.0]], [[5.0, 7.0, 2.0], [5.0, 6.0, 4.0], 
[2.0, 1.0, 6.0]]], [[[8.0, 3.0, 6.0], [1.0, 6.0, 1.0], [5.0, 5.0, 6.0]], [[8.0, 6.0, 6.0], [6.0, 1.0, 6.0], [5.0, 5.0, 3.0]], 
[[4.0, 8.0, 3.0], [1.0, 2.0, 5.0], [5.0, 6.0, 3.0]]], [[[1.0, 7.0, 8.0], [8.0, 1.0, 5.0], [6.0, 4.0, 1.0]], [[5.0, 1.0, 5.0], 
[5.0, 3.0, 1.0], [1.0, 3.0, 7.0]], [[2.0, 1.0, 6.0], [8.0, 3.0, 8.0], [3.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PRCC(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 5.0], [7.0, 6.0, 6.0], [8.0, 7.0, 6.0]], [[3.0, 3.0, 4.0], [2.0, 6.0, 5.0], [4.0, 2.0, 
6.0]], [[8.0, 4.0, 7.0], [8.0, 3.0, 3.0], [3.0, 2.0, 8.0]]], [[[3.0, 1.0, 7.0], [2.0, 7.0, 6.0], [3.0, 5.0, 2.0]], [[7.0, 8.0, 
3.0], [8.0, 8.0, 3.0], [6.0, 6.0, 4.0]], [[6.0, 7.0, 7.0], [4.0, 5.0, 1.0], [7.0, 6.0, 5.0]]], [[[1.0, 3.0, 7.0], [2.0, 3.0, 
3.0], [8.0, 2.0, 5.0]], [[7.0, 5.0, 2.0], [2.0, 1.0, 5.0], [7.0, 5.0, 7.0]], [[7.0, 4.0, 4.0], [2.0, 1.0, 5.0], [6.0, 2.0, 
2.0]]], [[[7.0, 4.0, 8.0], [7.0, 6.0, 6.0], [2.0, 4.0, 3.0]], [[4.0, 3.0, 1.0], [6.0, 7.0, 2.0], [4.0, 6.0, 1.0]], [[1.0, 3.0, 
1.0], [1.0, 4.0, 4.0], [2.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 3.0], [5.0, 6.0, 6.0], [8.0, 4.0, 6.0]], [[8.0, 7.0, 6.0], [6.0, 2.0, 
8.0], [1.0, 4.0, 5.0]], [[1.0, 3.0, 7.0], [3.0, 8.0, 2.0], [5.0, 7.0, 7.0]]], [[[5.0, 4.0, 2.0], [5.0, 8.0, 5.0], [2.0, 5.0, 
4.0]], [[7.0, 4.0, 7.0], [7.0, 5.0, 1.0], [4.0, 8.0, 4.0]], [[2.0, 7.0, 6.0], [3.0, 3.0, 7.0], [6.0, 7.0, 3.0]]], [[[3.0, 8.0, 
4.0], [8.0, 2.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 5.0, 3.0], [1.0, 5.0, 2.0], [1.0, 1.0, 1.0]], [[6.0, 1.0, 2.0], [6.0, 4.0, 
6.0], [5.0, 4.0, 5.0]]], [[[5.0, 5.0, 2.0], [6.0, 6.0, 3.0], [4.0, 5.0, 8.0]], [[8.0, 4.0, 1.0], [1.0, 7.0, 7.0], [3.0, 6.0, 
8.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 4.0], [1.0, 8.0, 8.0]]]]))
      val=Data(numpy.array([[[8.0, 8.0, 6.0], [6.0, 6.0, 8.0], [8.0, 1.0, 7.0]]]),self.functionspace)
      arg[2,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 5.0], [7.0, 6.0, 6.0], [8.0, 7.0, 6.0]], [[3.0, 3.0, 4.0], [2.0, 6.0, 5.0], [4.0, 
2.0, 6.0]], [[8.0, 4.0, 7.0], [8.0, 3.0, 3.0], [3.0, 2.0, 8.0]]], [[[3.0, 1.0, 7.0], [2.0, 7.0, 6.0], [3.0, 5.0, 2.0]], [[7.0, 
8.0, 3.0], [8.0, 8.0, 3.0], [6.0, 6.0, 4.0]], [[6.0, 7.0, 7.0], [4.0, 5.0, 1.0], [7.0, 6.0, 5.0]]], [[[1.0, 3.0, 7.0], [2.0, 
3.0, 3.0], [8.0, 2.0, 5.0]], [[8.0, 8.0, 6.0], [6.0, 6.0, 8.0], [8.0, 1.0, 7.0]], [[7.0, 4.0, 4.0], [2.0, 1.0, 5.0], [6.0, 2.0, 
2.0]]], [[[7.0, 4.0, 8.0], [7.0, 6.0, 6.0], [2.0, 4.0, 3.0]], [[4.0, 3.0, 1.0], [6.0, 7.0, 2.0], [4.0, 6.0, 1.0]], [[1.0, 3.0, 
1.0], [1.0, 4.0, 4.0], [2.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 3.0], [5.0, 6.0, 6.0], [8.0, 4.0, 6.0]], 
[[8.0, 7.0, 6.0], [6.0, 2.0, 8.0], [1.0, 4.0, 5.0]], [[1.0, 3.0, 7.0], [3.0, 8.0, 2.0], [5.0, 7.0, 7.0]]], [[[5.0, 4.0, 2.0], 
[5.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[7.0, 4.0, 7.0], [7.0, 5.0, 1.0], [4.0, 8.0, 4.0]], [[2.0, 7.0, 6.0], [3.0, 3.0, 7.0], 
[6.0, 7.0, 3.0]]], [[[3.0, 8.0, 4.0], [8.0, 2.0, 4.0], [1.0, 2.0, 1.0]], [[8.0, 8.0, 6.0], [6.0, 6.0, 8.0], [8.0, 1.0, 7.0]], 
[[6.0, 1.0, 2.0], [6.0, 4.0, 6.0], [5.0, 4.0, 5.0]]], [[[5.0, 5.0, 2.0], [6.0, 6.0, 3.0], [4.0, 5.0, 8.0]], [[8.0, 4.0, 1.0], 
[1.0, 7.0, 7.0], [3.0, 6.0, 8.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 4.0], [1.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PRCC(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 1.0], [3.0, 6.0, 4.0], [2.0, 6.0, 2.0]], [[1.0, 2.0, 5.0], [4.0, 2.0, 1.0], [4.0, 3.0, 
1.0]], [[8.0, 1.0, 8.0], [2.0, 8.0, 2.0], [5.0, 8.0, 3.0]]], [[[1.0, 1.0, 2.0], [3.0, 6.0, 8.0], [7.0, 7.0, 6.0]], [[5.0, 4.0, 
6.0], [1.0, 4.0, 5.0], [8.0, 6.0, 1.0]], [[4.0, 1.0, 3.0], [1.0, 1.0, 2.0], [8.0, 1.0, 8.0]]], [[[8.0, 3.0, 7.0], [3.0, 1.0, 
8.0], [2.0, 1.0, 1.0]], [[2.0, 1.0, 5.0], [3.0, 5.0, 5.0], [4.0, 3.0, 4.0]], [[2.0, 6.0, 1.0], [8.0, 4.0, 7.0], [5.0, 5.0, 
5.0]]], [[[2.0, 7.0, 2.0], [2.0, 7.0, 3.0], [4.0, 4.0, 3.0]], [[8.0, 2.0, 7.0], [1.0, 2.0, 6.0], [7.0, 7.0, 5.0]], [[8.0, 2.0, 
5.0], [1.0, 7.0, 5.0], [7.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 2.0], [8.0, 1.0, 6.0], [4.0, 7.0, 5.0]], [[3.0, 2.0, 1.0], [8.0, 7.0, 
7.0], [7.0, 3.0, 8.0]], [[7.0, 8.0, 3.0], [7.0, 2.0, 2.0], [1.0, 4.0, 2.0]]], [[[2.0, 8.0, 6.0], [6.0, 1.0, 7.0], [6.0, 6.0, 
5.0]], [[7.0, 7.0, 8.0], [2.0, 3.0, 6.0], [2.0, 4.0, 6.0]], [[5.0, 5.0, 8.0], [1.0, 1.0, 2.0], [6.0, 6.0, 2.0]]], [[[8.0, 8.0, 
4.0], [2.0, 7.0, 2.0], [8.0, 3.0, 8.0]], [[5.0, 1.0, 3.0], [1.0, 4.0, 2.0], [6.0, 1.0, 5.0]], [[8.0, 2.0, 7.0], [1.0, 6.0, 
6.0], [3.0, 6.0, 4.0]]], [[[2.0, 2.0, 8.0], [5.0, 5.0, 1.0], [6.0, 4.0, 1.0]], [[5.0, 8.0, 3.0], [5.0, 1.0, 7.0], [6.0, 4.0, 
8.0]], [[1.0, 2.0, 2.0], [8.0, 5.0, 4.0], [3.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[[5.0, 4.0, 7.0], [6.0, 6.0, 8.0], [4.0, 5.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 4.0, 8.0], [1.0, 3.0, 1.0], [3.0, 3.0, 6.0]]]))
      arg[2,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 1.0], [3.0, 6.0, 4.0], [2.0, 6.0, 2.0]], [[1.0, 2.0, 5.0], [4.0, 2.0, 1.0], [4.0, 
3.0, 1.0]], [[8.0, 1.0, 8.0], [2.0, 8.0, 2.0], [5.0, 8.0, 3.0]]], [[[1.0, 1.0, 2.0], [3.0, 6.0, 8.0], [7.0, 7.0, 6.0]], [[5.0, 
4.0, 6.0], [1.0, 4.0, 5.0], [8.0, 6.0, 1.0]], [[4.0, 1.0, 3.0], [1.0, 1.0, 2.0], [8.0, 1.0, 8.0]]], [[[8.0, 3.0, 7.0], [3.0, 
1.0, 8.0], [2.0, 1.0, 1.0]], [[5.0, 4.0, 7.0], [6.0, 6.0, 8.0], [4.0, 5.0, 3.0]], [[2.0, 6.0, 1.0], [8.0, 4.0, 7.0], [5.0, 5.0, 
5.0]]], [[[2.0, 7.0, 2.0], [2.0, 7.0, 3.0], [4.0, 4.0, 3.0]], [[8.0, 2.0, 7.0], [1.0, 2.0, 6.0], [7.0, 7.0, 5.0]], [[8.0, 2.0, 
5.0], [1.0, 7.0, 5.0], [7.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 2.0], [8.0, 1.0, 6.0], [4.0, 7.0, 5.0]], 
[[3.0, 2.0, 1.0], [8.0, 7.0, 7.0], [7.0, 3.0, 8.0]], [[7.0, 8.0, 3.0], [7.0, 2.0, 2.0], [1.0, 4.0, 2.0]]], [[[2.0, 8.0, 6.0], 
[6.0, 1.0, 7.0], [6.0, 6.0, 5.0]], [[7.0, 7.0, 8.0], [2.0, 3.0, 6.0], [2.0, 4.0, 6.0]], [[5.0, 5.0, 8.0], [1.0, 1.0, 2.0], 
[6.0, 6.0, 2.0]]], [[[8.0, 8.0, 4.0], [2.0, 7.0, 2.0], [8.0, 3.0, 8.0]], [[7.0, 4.0, 8.0], [1.0, 3.0, 1.0], [3.0, 3.0, 6.0]], 
[[8.0, 2.0, 7.0], [1.0, 6.0, 6.0], [3.0, 6.0, 4.0]]], [[[2.0, 2.0, 8.0], [5.0, 5.0, 1.0], [6.0, 4.0, 1.0]], [[5.0, 8.0, 3.0], 
[5.0, 1.0, 7.0], [6.0, 4.0, 8.0]], [[1.0, 2.0, 2.0], [8.0, 5.0, 4.0], [3.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PRCC(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 3.0], [1.0, 2.0, 2.0], [3.0, 2.0, 6.0]], [[5.0, 2.0, 3.0], [7.0, 2.0, 2.0], [7.0, 6.0, 
5.0]], [[2.0, 6.0, 2.0], [6.0, 3.0, 3.0], [8.0, 1.0, 2.0]]], [[[3.0, 3.0, 5.0], [7.0, 3.0, 1.0], [5.0, 7.0, 2.0]], [[2.0, 6.0, 
8.0], [2.0, 5.0, 2.0], [8.0, 3.0, 7.0]], [[5.0, 7.0, 2.0], [8.0, 4.0, 4.0], [3.0, 2.0, 2.0]]], [[[5.0, 4.0, 1.0], [3.0, 3.0, 
5.0], [8.0, 8.0, 5.0]], [[1.0, 2.0, 8.0], [2.0, 1.0, 7.0], [5.0, 2.0, 5.0]], [[1.0, 8.0, 3.0], [5.0, 3.0, 1.0], [7.0, 3.0, 
1.0]]], [[[5.0, 2.0, 2.0], [2.0, 7.0, 3.0], [5.0, 7.0, 6.0]], [[2.0, 6.0, 5.0], [3.0, 6.0, 2.0], [7.0, 5.0, 8.0]], [[3.0, 2.0, 
2.0], [7.0, 5.0, 3.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 8.0], [1.0, 8.0, 1.0], [7.0, 7.0, 4.0]], [[2.0, 4.0, 5.0], [4.0, 1.0, 
3.0], [5.0, 4.0, 4.0]], [[8.0, 1.0, 3.0], [8.0, 5.0, 7.0], [4.0, 1.0, 7.0]]], [[[5.0, 3.0, 7.0], [1.0, 3.0, 5.0], [8.0, 3.0, 
7.0]], [[2.0, 1.0, 3.0], [1.0, 3.0, 3.0], [6.0, 6.0, 2.0]], [[3.0, 8.0, 5.0], [1.0, 4.0, 1.0], [4.0, 1.0, 4.0]]], [[[4.0, 7.0, 
8.0], [3.0, 7.0, 5.0], [6.0, 4.0, 5.0]], [[7.0, 5.0, 8.0], [2.0, 6.0, 6.0], [6.0, 4.0, 7.0]], [[8.0, 8.0, 6.0], [4.0, 1.0, 
8.0], [3.0, 6.0, 5.0]]], [[[4.0, 4.0, 5.0], [5.0, 6.0, 5.0], [4.0, 6.0, 6.0]], [[4.0, 6.0, 3.0], [6.0, 3.0, 6.0], [5.0, 3.0, 
2.0]], [[2.0, 7.0, 8.0], [6.0, 1.0, 2.0], [3.0, 7.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 3.0, 2.0], [3.0, 2.0, 6.0], [4.0, 1.0, 3.0]]])+(1.-msk_val)*numpy.array([[[5.0, 7.0, 
7.0], [3.0, 3.0, 6.0], [2.0, 3.0, 4.0]]])
      arg[2,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 3.0], [1.0, 2.0, 2.0], [3.0, 2.0, 6.0]], [[5.0, 2.0, 3.0], [7.0, 2.0, 2.0], [7.0, 
6.0, 5.0]], [[2.0, 6.0, 2.0], [6.0, 3.0, 3.0], [8.0, 1.0, 2.0]]], [[[3.0, 3.0, 5.0], [7.0, 3.0, 1.0], [5.0, 7.0, 2.0]], [[2.0, 
6.0, 8.0], [2.0, 5.0, 2.0], [8.0, 3.0, 7.0]], [[5.0, 7.0, 2.0], [8.0, 4.0, 4.0], [3.0, 2.0, 2.0]]], [[[5.0, 4.0, 1.0], [3.0, 
3.0, 5.0], [8.0, 8.0, 5.0]], [[4.0, 3.0, 2.0], [3.0, 2.0, 6.0], [4.0, 1.0, 3.0]], [[1.0, 8.0, 3.0], [5.0, 3.0, 1.0], [7.0, 3.0, 
1.0]]], [[[5.0, 2.0, 2.0], [2.0, 7.0, 3.0], [5.0, 7.0, 6.0]], [[2.0, 6.0, 5.0], [3.0, 6.0, 2.0], [7.0, 5.0, 8.0]], [[3.0, 2.0, 
2.0], [7.0, 5.0, 3.0], [1.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 8.0], [1.0, 8.0, 1.0], [7.0, 7.0, 4.0]], 
[[2.0, 4.0, 5.0], [4.0, 1.0, 3.0], [5.0, 4.0, 4.0]], [[8.0, 1.0, 3.0], [8.0, 5.0, 7.0], [4.0, 1.0, 7.0]]], [[[5.0, 3.0, 7.0], 
[1.0, 3.0, 5.0], [8.0, 3.0, 7.0]], [[2.0, 1.0, 3.0], [1.0, 3.0, 3.0], [6.0, 6.0, 2.0]], [[3.0, 8.0, 5.0], [1.0, 4.0, 1.0], 
[4.0, 1.0, 4.0]]], [[[4.0, 7.0, 8.0], [3.0, 7.0, 5.0], [6.0, 4.0, 5.0]], [[5.0, 7.0, 7.0], [3.0, 3.0, 6.0], [2.0, 3.0, 4.0]], 
[[8.0, 8.0, 6.0], [4.0, 1.0, 8.0], [3.0, 6.0, 5.0]]], [[[4.0, 4.0, 5.0], [5.0, 6.0, 5.0], [4.0, 6.0, 6.0]], [[4.0, 6.0, 3.0], 
[6.0, 3.0, 6.0], [5.0, 3.0, 2.0]], [[2.0, 7.0, 8.0], [6.0, 1.0, 2.0], [3.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRCC(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 7.0], [4.0, 2.0, 3.0], [2.0, 1.0, 7.0]], [[3.0, 4.0, 2.0], [5.0, 8.0, 6.0], [1.0, 2.0, 
3.0]], [[3.0, 2.0, 3.0], [2.0, 5.0, 8.0], [4.0, 1.0, 7.0]]], [[[3.0, 3.0, 8.0], [8.0, 3.0, 8.0], [5.0, 7.0, 6.0]], [[5.0, 1.0, 
4.0], [7.0, 8.0, 2.0], [8.0, 7.0, 4.0]], [[4.0, 3.0, 5.0], [2.0, 4.0, 7.0], [5.0, 3.0, 4.0]]], [[[6.0, 7.0, 1.0], [3.0, 1.0, 
7.0], [1.0, 4.0, 2.0]], [[3.0, 7.0, 4.0], [7.0, 8.0, 6.0], [6.0, 3.0, 4.0]], [[1.0, 7.0, 6.0], [8.0, 7.0, 6.0], [8.0, 3.0, 
3.0]]], [[[7.0, 2.0, 8.0], [5.0, 6.0, 6.0], [6.0, 8.0, 7.0]], [[1.0, 8.0, 4.0], [4.0, 4.0, 7.0], [1.0, 3.0, 6.0]], [[8.0, 8.0, 
8.0], [7.0, 7.0, 3.0], [4.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 8.0], [6.0, 4.0, 1.0], [1.0, 7.0, 4.0]], [[7.0, 7.0, 6.0], [4.0, 5.0, 
6.0], [8.0, 5.0, 4.0]], [[7.0, 3.0, 8.0], [4.0, 2.0, 3.0], [7.0, 5.0, 1.0]]], [[[7.0, 5.0, 3.0], [8.0, 8.0, 3.0], [1.0, 3.0, 
2.0]], [[3.0, 5.0, 5.0], [7.0, 1.0, 2.0], [8.0, 1.0, 1.0]], [[4.0, 2.0, 7.0], [6.0, 2.0, 5.0], [5.0, 4.0, 3.0]]], [[[3.0, 2.0, 
1.0], [2.0, 6.0, 8.0], [5.0, 3.0, 8.0]], [[6.0, 4.0, 4.0], [4.0, 2.0, 8.0], [2.0, 2.0, 5.0]], [[4.0, 8.0, 6.0], [1.0, 3.0, 
8.0], [5.0, 5.0, 2.0]]], [[[6.0, 3.0, 7.0], [7.0, 6.0, 7.0], [8.0, 3.0, 5.0]], [[2.0, 7.0, 5.0], [8.0, 2.0, 2.0], [4.0, 5.0, 
6.0]], [[2.0, 5.0, 4.0], [2.0, 7.0, 7.0], [4.0, 8.0, 4.0]]]]))
      val=numpy.array([[[[7.0, 6.0, 6.0], [7.0, 1.0, 6.0], [2.0, 3.0, 7.0]]], [[[6.0, 7.0, 8.0], [2.0, 5.0, 5.0], [4.0, 8.0, 
8.0]]]])
      arg[1:3,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 7.0], [4.0, 2.0, 3.0], [2.0, 1.0, 7.0]], [[3.0, 4.0, 2.0], [5.0, 8.0, 6.0], [1.0, 
2.0, 3.0]], [[3.0, 2.0, 3.0], [2.0, 5.0, 8.0], [4.0, 1.0, 7.0]]], [[[3.0, 3.0, 8.0], [8.0, 3.0, 8.0], [5.0, 7.0, 6.0]], [[7.0, 
6.0, 6.0], [7.0, 1.0, 6.0], [2.0, 3.0, 7.0]], [[4.0, 3.0, 5.0], [2.0, 4.0, 7.0], [5.0, 3.0, 4.0]]], [[[6.0, 7.0, 1.0], [3.0, 
1.0, 7.0], [1.0, 4.0, 2.0]], [[6.0, 7.0, 8.0], [2.0, 5.0, 5.0], [4.0, 8.0, 8.0]], [[1.0, 7.0, 6.0], [8.0, 7.0, 6.0], [8.0, 3.0, 
3.0]]], [[[7.0, 2.0, 8.0], [5.0, 6.0, 6.0], [6.0, 8.0, 7.0]], [[1.0, 8.0, 4.0], [4.0, 4.0, 7.0], [1.0, 3.0, 6.0]], [[8.0, 8.0, 
8.0], [7.0, 7.0, 3.0], [4.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 8.0], [6.0, 4.0, 1.0], [1.0, 7.0, 4.0]], 
[[7.0, 7.0, 6.0], [4.0, 5.0, 6.0], [8.0, 5.0, 4.0]], [[7.0, 3.0, 8.0], [4.0, 2.0, 3.0], [7.0, 5.0, 1.0]]], [[[7.0, 5.0, 3.0], 
[8.0, 8.0, 3.0], [1.0, 3.0, 2.0]], [[7.0, 6.0, 6.0], [7.0, 1.0, 6.0], [2.0, 3.0, 7.0]], [[4.0, 2.0, 7.0], [6.0, 2.0, 5.0], 
[5.0, 4.0, 3.0]]], [[[3.0, 2.0, 1.0], [2.0, 6.0, 8.0], [5.0, 3.0, 8.0]], [[6.0, 7.0, 8.0], [2.0, 5.0, 5.0], [4.0, 8.0, 8.0]], 
[[4.0, 8.0, 6.0], [1.0, 3.0, 8.0], [5.0, 5.0, 2.0]]], [[[6.0, 3.0, 7.0], [7.0, 6.0, 7.0], [8.0, 3.0, 5.0]], [[2.0, 7.0, 5.0], 
[8.0, 2.0, 2.0], [4.0, 5.0, 6.0]], [[2.0, 5.0, 4.0], [2.0, 7.0, 7.0], [4.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRCC(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 7.0], [5.0, 2.0, 6.0], [3.0, 8.0, 7.0]], [[5.0, 7.0, 4.0], [6.0, 5.0, 7.0], [1.0, 1.0, 
2.0]], [[7.0, 7.0, 2.0], [2.0, 3.0, 3.0], [2.0, 2.0, 6.0]]], [[[4.0, 5.0, 8.0], [7.0, 1.0, 7.0], [5.0, 8.0, 2.0]], [[7.0, 5.0, 
2.0], [3.0, 2.0, 6.0], [6.0, 3.0, 2.0]], [[5.0, 2.0, 2.0], [7.0, 7.0, 7.0], [5.0, 7.0, 5.0]]], [[[6.0, 3.0, 1.0], [5.0, 3.0, 
7.0], [5.0, 8.0, 3.0]], [[6.0, 4.0, 1.0], [6.0, 4.0, 2.0], [6.0, 8.0, 6.0]], [[1.0, 8.0, 8.0], [6.0, 6.0, 6.0], [2.0, 1.0, 
8.0]]], [[[6.0, 5.0, 3.0], [1.0, 3.0, 4.0], [4.0, 6.0, 2.0]], [[4.0, 6.0, 6.0], [6.0, 8.0, 1.0], [1.0, 1.0, 7.0]], [[8.0, 1.0, 
4.0], [2.0, 6.0, 8.0], [3.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 1.0], [2.0, 5.0, 8.0], [8.0, 3.0, 4.0]], [[1.0, 3.0, 3.0], [8.0, 8.0, 
6.0], [3.0, 8.0, 8.0]], [[2.0, 1.0, 5.0], [4.0, 1.0, 3.0], [6.0, 8.0, 2.0]]], [[[3.0, 4.0, 1.0], [8.0, 1.0, 4.0], [6.0, 8.0, 
2.0]], [[5.0, 4.0, 4.0], [6.0, 8.0, 4.0], [8.0, 7.0, 5.0]], [[8.0, 5.0, 1.0], [3.0, 8.0, 2.0], [5.0, 6.0, 1.0]]], [[[2.0, 7.0, 
5.0], [2.0, 2.0, 2.0], [1.0, 1.0, 8.0]], [[2.0, 5.0, 5.0], [3.0, 8.0, 7.0], [8.0, 5.0, 5.0]], [[8.0, 2.0, 7.0], [6.0, 1.0, 
6.0], [6.0, 1.0, 2.0]]], [[[6.0, 4.0, 3.0], [2.0, 3.0, 6.0], [2.0, 7.0, 4.0]], [[2.0, 7.0, 8.0], [6.0, 4.0, 3.0], [3.0, 2.0, 
1.0]], [[5.0, 2.0, 5.0], [5.0, 1.0, 7.0], [6.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[[7.0, 3.0, 6.0], [5.0, 1.0, 3.0], [7.0, 7.0, 7.0]]], [[[8.0, 2.0, 4.0], [1.0, 8.0, 5.0], [3.0, 
5.0, 7.0]]]]),self.functionspace)
      arg[1:3,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 7.0], [5.0, 2.0, 6.0], [3.0, 8.0, 7.0]], [[5.0, 7.0, 4.0], [6.0, 5.0, 7.0], [1.0, 
1.0, 2.0]], [[7.0, 7.0, 2.0], [2.0, 3.0, 3.0], [2.0, 2.0, 6.0]]], [[[4.0, 5.0, 8.0], [7.0, 1.0, 7.0], [5.0, 8.0, 2.0]], [[7.0, 
3.0, 6.0], [5.0, 1.0, 3.0], [7.0, 7.0, 7.0]], [[5.0, 2.0, 2.0], [7.0, 7.0, 7.0], [5.0, 7.0, 5.0]]], [[[6.0, 3.0, 1.0], [5.0, 
3.0, 7.0], [5.0, 8.0, 3.0]], [[8.0, 2.0, 4.0], [1.0, 8.0, 5.0], [3.0, 5.0, 7.0]], [[1.0, 8.0, 8.0], [6.0, 6.0, 6.0], [2.0, 1.0, 
8.0]]], [[[6.0, 5.0, 3.0], [1.0, 3.0, 4.0], [4.0, 6.0, 2.0]], [[4.0, 6.0, 6.0], [6.0, 8.0, 1.0], [1.0, 1.0, 7.0]], [[8.0, 1.0, 
4.0], [2.0, 6.0, 8.0], [3.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 1.0], [2.0, 5.0, 8.0], [8.0, 3.0, 4.0]], 
[[1.0, 3.0, 3.0], [8.0, 8.0, 6.0], [3.0, 8.0, 8.0]], [[2.0, 1.0, 5.0], [4.0, 1.0, 3.0], [6.0, 8.0, 2.0]]], [[[3.0, 4.0, 1.0], 
[8.0, 1.0, 4.0], [6.0, 8.0, 2.0]], [[7.0, 3.0, 6.0], [5.0, 1.0, 3.0], [7.0, 7.0, 7.0]], [[8.0, 5.0, 1.0], [3.0, 8.0, 2.0], 
[5.0, 6.0, 1.0]]], [[[2.0, 7.0, 5.0], [2.0, 2.0, 2.0], [1.0, 1.0, 8.0]], [[8.0, 2.0, 4.0], [1.0, 8.0, 5.0], [3.0, 5.0, 7.0]], 
[[8.0, 2.0, 7.0], [6.0, 1.0, 6.0], [6.0, 1.0, 2.0]]], [[[6.0, 4.0, 3.0], [2.0, 3.0, 6.0], [2.0, 7.0, 4.0]], [[2.0, 7.0, 8.0], 
[6.0, 4.0, 3.0], [3.0, 2.0, 1.0]], [[5.0, 2.0, 5.0], [5.0, 1.0, 7.0], [6.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRCC(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 4.0], [3.0, 8.0, 8.0], [8.0, 8.0, 1.0]], [[3.0, 1.0, 6.0], [1.0, 4.0, 2.0], [5.0, 8.0, 
2.0]], [[8.0, 6.0, 3.0], [5.0, 5.0, 8.0], [3.0, 3.0, 8.0]]], [[[5.0, 8.0, 6.0], [1.0, 8.0, 6.0], [3.0, 4.0, 8.0]], [[3.0, 4.0, 
6.0], [6.0, 4.0, 4.0], [6.0, 2.0, 6.0]], [[4.0, 4.0, 2.0], [8.0, 5.0, 5.0], [2.0, 2.0, 5.0]]], [[[2.0, 6.0, 2.0], [8.0, 6.0, 
6.0], [8.0, 1.0, 7.0]], [[2.0, 8.0, 7.0], [4.0, 5.0, 3.0], [4.0, 2.0, 6.0]], [[1.0, 6.0, 4.0], [3.0, 7.0, 5.0], [8.0, 3.0, 
5.0]]], [[[7.0, 2.0, 1.0], [6.0, 2.0, 8.0], [1.0, 1.0, 6.0]], [[4.0, 1.0, 1.0], [7.0, 3.0, 4.0], [3.0, 7.0, 6.0]], [[6.0, 5.0, 
1.0], [6.0, 7.0, 8.0], [4.0, 3.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 4.0], [1.0, 6.0, 7.0], [5.0, 4.0, 2.0]], [[1.0, 7.0, 8.0], [1.0, 1.0, 
7.0], [3.0, 4.0, 1.0]], [[2.0, 8.0, 5.0], [3.0, 3.0, 8.0], [8.0, 2.0, 6.0]]], [[[4.0, 6.0, 7.0], [1.0, 3.0, 2.0], [6.0, 3.0, 
5.0]], [[8.0, 5.0, 3.0], [7.0, 3.0, 5.0], [4.0, 3.0, 6.0]], [[5.0, 1.0, 5.0], [5.0, 5.0, 7.0], [2.0, 2.0, 8.0]]], [[[4.0, 4.0, 
5.0], [2.0, 1.0, 5.0], [6.0, 4.0, 6.0]], [[8.0, 1.0, 2.0], [7.0, 6.0, 2.0], [8.0, 8.0, 8.0]], [[5.0, 7.0, 3.0], [7.0, 7.0, 
7.0], [2.0, 8.0, 2.0]]], [[[3.0, 8.0, 5.0], [5.0, 6.0, 4.0], [7.0, 1.0, 8.0]], [[6.0, 2.0, 7.0], [2.0, 5.0, 5.0], [1.0, 1.0, 
2.0]], [[5.0, 2.0, 4.0], [2.0, 1.0, 7.0], [2.0, 5.0, 4.0]]]]))
      val=Data(numpy.array([[[[7.0, 7.0, 8.0], [7.0, 4.0, 3.0], [2.0, 1.0, 4.0]]], [[[4.0, 8.0, 3.0], [7.0, 5.0, 7.0], [8.0, 
7.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 5.0], [6.0, 4.0, 8.0], [2.0, 4.0, 5.0]]], [[[7.0, 3.0, 3.0], [8.0, 3.0, 
3.0], [7.0, 5.0, 7.0]]]]))
      arg[1:3,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 4.0], [3.0, 8.0, 8.0], [8.0, 8.0, 1.0]], [[3.0, 1.0, 6.0], [1.0, 4.0, 2.0], [5.0, 
8.0, 2.0]], [[8.0, 6.0, 3.0], [5.0, 5.0, 8.0], [3.0, 3.0, 8.0]]], [[[5.0, 8.0, 6.0], [1.0, 8.0, 6.0], [3.0, 4.0, 8.0]], [[7.0, 
7.0, 8.0], [7.0, 4.0, 3.0], [2.0, 1.0, 4.0]], [[4.0, 4.0, 2.0], [8.0, 5.0, 5.0], [2.0, 2.0, 5.0]]], [[[2.0, 6.0, 2.0], [8.0, 
6.0, 6.0], [8.0, 1.0, 7.0]], [[4.0, 8.0, 3.0], [7.0, 5.0, 7.0], [8.0, 7.0, 8.0]], [[1.0, 6.0, 4.0], [3.0, 7.0, 5.0], [8.0, 3.0, 
5.0]]], [[[7.0, 2.0, 1.0], [6.0, 2.0, 8.0], [1.0, 1.0, 6.0]], [[4.0, 1.0, 1.0], [7.0, 3.0, 4.0], [3.0, 7.0, 6.0]], [[6.0, 5.0, 
1.0], [6.0, 7.0, 8.0], [4.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 4.0], [1.0, 6.0, 7.0], [5.0, 4.0, 2.0]], 
[[1.0, 7.0, 8.0], [1.0, 1.0, 7.0], [3.0, 4.0, 1.0]], [[2.0, 8.0, 5.0], [3.0, 3.0, 8.0], [8.0, 2.0, 6.0]]], [[[4.0, 6.0, 7.0], 
[1.0, 3.0, 2.0], [6.0, 3.0, 5.0]], [[8.0, 8.0, 5.0], [6.0, 4.0, 8.0], [2.0, 4.0, 5.0]], [[5.0, 1.0, 5.0], [5.0, 5.0, 7.0], 
[2.0, 2.0, 8.0]]], [[[4.0, 4.0, 5.0], [2.0, 1.0, 5.0], [6.0, 4.0, 6.0]], [[7.0, 3.0, 3.0], [8.0, 3.0, 3.0], [7.0, 5.0, 7.0]], 
[[5.0, 7.0, 3.0], [7.0, 7.0, 7.0], [2.0, 8.0, 2.0]]], [[[3.0, 8.0, 5.0], [5.0, 6.0, 4.0], [7.0, 1.0, 8.0]], [[6.0, 2.0, 7.0], 
[2.0, 5.0, 5.0], [1.0, 1.0, 2.0]], [[5.0, 2.0, 4.0], [2.0, 1.0, 7.0], [2.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRCC(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 1.0], [7.0, 5.0, 7.0], [6.0, 4.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 5.0, 6.0], [2.0, 2.0, 
4.0]], [[4.0, 3.0, 3.0], [2.0, 5.0, 7.0], [2.0, 8.0, 6.0]]], [[[3.0, 1.0, 6.0], [1.0, 3.0, 3.0], [3.0, 1.0, 7.0]], [[5.0, 1.0, 
2.0], [1.0, 2.0, 3.0], [8.0, 3.0, 7.0]], [[6.0, 3.0, 8.0], [7.0, 1.0, 4.0], [5.0, 3.0, 4.0]]], [[[2.0, 3.0, 6.0], [1.0, 6.0, 
2.0], [8.0, 2.0, 8.0]], [[7.0, 1.0, 5.0], [8.0, 7.0, 3.0], [4.0, 3.0, 8.0]], [[2.0, 4.0, 7.0], [8.0, 1.0, 8.0], [5.0, 3.0, 
5.0]]], [[[4.0, 5.0, 1.0], [8.0, 3.0, 6.0], [6.0, 1.0, 1.0]], [[5.0, 6.0, 2.0], [2.0, 7.0, 5.0], [1.0, 3.0, 4.0]], [[6.0, 2.0, 
3.0], [3.0, 5.0, 3.0], [2.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 1.0], [7.0, 7.0, 7.0], [6.0, 5.0, 8.0]], [[8.0, 7.0, 2.0], [8.0, 2.0, 
7.0], [8.0, 1.0, 8.0]], [[8.0, 8.0, 8.0], [3.0, 6.0, 1.0], [3.0, 6.0, 8.0]]], [[[5.0, 6.0, 6.0], [1.0, 3.0, 8.0], [8.0, 8.0, 
3.0]], [[3.0, 6.0, 5.0], [8.0, 4.0, 8.0], [4.0, 4.0, 8.0]], [[8.0, 2.0, 3.0], [8.0, 4.0, 7.0], [6.0, 4.0, 8.0]]], [[[8.0, 7.0, 
7.0], [8.0, 5.0, 3.0], [8.0, 7.0, 3.0]], [[5.0, 8.0, 6.0], [2.0, 8.0, 6.0], [7.0, 7.0, 6.0]], [[8.0, 1.0, 3.0], [7.0, 6.0, 
7.0], [2.0, 4.0, 7.0]]], [[[4.0, 5.0, 8.0], [2.0, 4.0, 5.0], [1.0, 8.0, 7.0]], [[1.0, 7.0, 8.0], [8.0, 2.0, 4.0], [7.0, 1.0, 
7.0]], [[7.0, 5.0, 6.0], [1.0, 3.0, 8.0], [7.0, 3.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0, 3.0, 1.0], [6.0, 5.0, 4.0], [1.0, 3.0, 5.0]]], [[[7.0, 4.0, 1.0], [7.0, 7.0, 4.0], [4.0, 
7.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 8.0, 4.0], [8.0, 8.0, 2.0], [8.0, 3.0, 2.0]]], [[[1.0, 2.0, 6.0], [3.0, 6.0, 
1.0], [8.0, 2.0, 6.0]]]])
      arg[1:3,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 1.0], [7.0, 5.0, 7.0], [6.0, 4.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 5.0, 6.0], [2.0, 
2.0, 4.0]], [[4.0, 3.0, 3.0], [2.0, 5.0, 7.0], [2.0, 8.0, 6.0]]], [[[3.0, 1.0, 6.0], [1.0, 3.0, 3.0], [3.0, 1.0, 7.0]], [[6.0, 
3.0, 1.0], [6.0, 5.0, 4.0], [1.0, 3.0, 5.0]], [[6.0, 3.0, 8.0], [7.0, 1.0, 4.0], [5.0, 3.0, 4.0]]], [[[2.0, 3.0, 6.0], [1.0, 
6.0, 2.0], [8.0, 2.0, 8.0]], [[7.0, 4.0, 1.0], [7.0, 7.0, 4.0], [4.0, 7.0, 4.0]], [[2.0, 4.0, 7.0], [8.0, 1.0, 8.0], [5.0, 3.0, 
5.0]]], [[[4.0, 5.0, 1.0], [8.0, 3.0, 6.0], [6.0, 1.0, 1.0]], [[5.0, 6.0, 2.0], [2.0, 7.0, 5.0], [1.0, 3.0, 4.0]], [[6.0, 2.0, 
3.0], [3.0, 5.0, 3.0], [2.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 1.0], [7.0, 7.0, 7.0], [6.0, 5.0, 8.0]], 
[[8.0, 7.0, 2.0], [8.0, 2.0, 7.0], [8.0, 1.0, 8.0]], [[8.0, 8.0, 8.0], [3.0, 6.0, 1.0], [3.0, 6.0, 8.0]]], [[[5.0, 6.0, 6.0], 
[1.0, 3.0, 8.0], [8.0, 8.0, 3.0]], [[4.0, 8.0, 4.0], [8.0, 8.0, 2.0], [8.0, 3.0, 2.0]], [[8.0, 2.0, 3.0], [8.0, 4.0, 7.0], 
[6.0, 4.0, 8.0]]], [[[8.0, 7.0, 7.0], [8.0, 5.0, 3.0], [8.0, 7.0, 3.0]], [[1.0, 2.0, 6.0], [3.0, 6.0, 1.0], [8.0, 2.0, 6.0]], 
[[8.0, 1.0, 3.0], [7.0, 6.0, 7.0], [2.0, 4.0, 7.0]]], [[[4.0, 5.0, 8.0], [2.0, 4.0, 5.0], [1.0, 8.0, 7.0]], [[1.0, 7.0, 8.0], 
[8.0, 2.0, 4.0], [7.0, 1.0, 7.0]], [[7.0, 5.0, 6.0], [1.0, 3.0, 8.0], [7.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPRL(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 6.0], [4.0, 8.0, 7.0], [1.0, 5.0, 8.0]], [[2.0, 6.0, 7.0], [7.0, 3.0, 7.0], [6.0, 6.0, 
8.0]], [[2.0, 6.0, 1.0], [7.0, 5.0, 3.0], [2.0, 1.0, 6.0]]], [[[2.0, 1.0, 7.0], [1.0, 6.0, 5.0], [6.0, 4.0, 7.0]], [[6.0, 8.0, 
3.0], [5.0, 6.0, 4.0], [4.0, 8.0, 3.0]], [[4.0, 4.0, 2.0], [2.0, 8.0, 5.0], [6.0, 1.0, 6.0]]], [[[8.0, 6.0, 8.0], [7.0, 6.0, 
5.0], [4.0, 2.0, 8.0]], [[8.0, 2.0, 7.0], [4.0, 1.0, 8.0], [3.0, 5.0, 2.0]], [[5.0, 5.0, 3.0], [6.0, 1.0, 6.0], [7.0, 1.0, 
7.0]]], [[[2.0, 3.0, 6.0], [1.0, 1.0, 1.0], [1.0, 5.0, 2.0]], [[6.0, 3.0, 6.0], [3.0, 1.0, 7.0], [1.0, 2.0, 3.0]], [[2.0, 5.0, 
3.0], [8.0, 3.0, 8.0], [2.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 4.0], [5.0, 6.0, 4.0], [7.0, 5.0, 4.0]], [[8.0, 1.0, 3.0], [5.0, 8.0, 
7.0], [7.0, 5.0, 1.0]], [[7.0, 3.0, 8.0], [8.0, 8.0, 3.0], [5.0, 4.0, 2.0]]], [[[7.0, 2.0, 1.0], [8.0, 3.0, 8.0], [7.0, 5.0, 
5.0]], [[8.0, 2.0, 7.0], [5.0, 5.0, 5.0], [2.0, 4.0, 3.0]], [[8.0, 6.0, 8.0], [3.0, 3.0, 6.0], [8.0, 2.0, 6.0]]], [[[4.0, 4.0, 
3.0], [1.0, 1.0, 2.0], [7.0, 7.0, 7.0]], [[2.0, 1.0, 4.0], [7.0, 1.0, 5.0], [7.0, 6.0, 5.0]], [[3.0, 8.0, 3.0], [1.0, 6.0, 
8.0], [8.0, 4.0, 7.0]]], [[[1.0, 6.0, 1.0], [4.0, 3.0, 2.0], [3.0, 6.0, 1.0]], [[7.0, 2.0, 7.0], [8.0, 7.0, 4.0], [7.0, 8.0, 
1.0]], [[7.0, 1.0, 5.0], [6.0, 7.0, 4.0], [3.0, 7.0, 4.0]]]]))
      val=numpy.array([[[1.0]], [[1.0]]])
      arg[1:3,1,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 8.0, 7.0], [1.0, 5.0, 8.0]], [[2.0, 6.0, 7.0], [7.0, 3.0, 7.0], [6.0, 
6.0, 8.0]], [[2.0, 6.0, 1.0], [7.0, 5.0, 3.0], [2.0, 1.0, 6.0]]], [[[2.0, 1.0, 7.0], [1.0, 6.0, 5.0], [6.0, 4.0, 7.0]], [[6.0, 
8.0, 3.0], [5.0, 6.0, 1.0], [4.0, 8.0, 3.0]], [[4.0, 4.0, 2.0], [2.0, 8.0, 5.0], [6.0, 1.0, 6.0]]], [[[8.0, 6.0, 8.0], [7.0, 
6.0, 5.0], [4.0, 2.0, 8.0]], [[8.0, 2.0, 7.0], [4.0, 1.0, 1.0], [3.0, 5.0, 2.0]], [[5.0, 5.0, 3.0], [6.0, 1.0, 6.0], [7.0, 1.0, 
7.0]]], [[[2.0, 3.0, 6.0], [1.0, 1.0, 1.0], [1.0, 5.0, 2.0]], [[6.0, 3.0, 6.0], [3.0, 1.0, 7.0], [1.0, 2.0, 3.0]], [[2.0, 5.0, 
3.0], [8.0, 3.0, 8.0], [2.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 4.0], [5.0, 6.0, 4.0], [7.0, 5.0, 4.0]], 
[[8.0, 1.0, 3.0], [5.0, 8.0, 7.0], [7.0, 5.0, 1.0]], [[7.0, 3.0, 8.0], [8.0, 8.0, 3.0], [5.0, 4.0, 2.0]]], [[[7.0, 2.0, 1.0], 
[8.0, 3.0, 8.0], [7.0, 5.0, 5.0]], [[8.0, 2.0, 7.0], [5.0, 5.0, 1.0], [2.0, 4.0, 3.0]], [[8.0, 6.0, 8.0], [3.0, 3.0, 6.0], 
[8.0, 2.0, 6.0]]], [[[4.0, 4.0, 3.0], [1.0, 1.0, 2.0], [7.0, 7.0, 7.0]], [[2.0, 1.0, 4.0], [7.0, 1.0, 1.0], [7.0, 6.0, 5.0]], 
[[3.0, 8.0, 3.0], [1.0, 6.0, 8.0], [8.0, 4.0, 7.0]]], [[[1.0, 6.0, 1.0], [4.0, 3.0, 2.0], [3.0, 6.0, 1.0]], [[7.0, 2.0, 7.0], 
[8.0, 7.0, 4.0], [7.0, 8.0, 1.0]], [[7.0, 1.0, 5.0], [6.0, 7.0, 4.0], [3.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPRL(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 6.0], [1.0, 8.0, 6.0], [1.0, 3.0, 1.0]], [[6.0, 5.0, 8.0], [7.0, 4.0, 4.0], [1.0, 8.0, 
6.0]], [[5.0, 6.0, 3.0], [2.0, 6.0, 6.0], [7.0, 2.0, 1.0]]], [[[5.0, 5.0, 4.0], [4.0, 8.0, 5.0], [3.0, 3.0, 4.0]], [[4.0, 8.0, 
1.0], [4.0, 2.0, 4.0], [6.0, 4.0, 7.0]], [[8.0, 2.0, 5.0], [1.0, 3.0, 8.0], [6.0, 5.0, 7.0]]], [[[6.0, 7.0, 2.0], [3.0, 2.0, 
5.0], [1.0, 5.0, 7.0]], [[4.0, 1.0, 1.0], [6.0, 2.0, 2.0], [4.0, 3.0, 1.0]], [[5.0, 5.0, 4.0], [5.0, 5.0, 7.0], [2.0, 8.0, 
3.0]]], [[[1.0, 5.0, 1.0], [5.0, 3.0, 6.0], [6.0, 5.0, 3.0]], [[1.0, 2.0, 4.0], [3.0, 7.0, 5.0], [1.0, 5.0, 1.0]], [[4.0, 4.0, 
4.0], [4.0, 5.0, 2.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 8.0], [7.0, 8.0, 1.0], [1.0, 2.0, 5.0]], [[1.0, 3.0, 5.0], [6.0, 7.0, 
5.0], [4.0, 5.0, 6.0]], [[8.0, 8.0, 1.0], [4.0, 4.0, 4.0], [5.0, 4.0, 2.0]]], [[[4.0, 2.0, 8.0], [5.0, 2.0, 6.0], [2.0, 7.0, 
8.0]], [[7.0, 8.0, 5.0], [7.0, 8.0, 2.0], [6.0, 8.0, 8.0]], [[5.0, 6.0, 5.0], [1.0, 8.0, 6.0], [7.0, 2.0, 6.0]]], [[[4.0, 5.0, 
5.0], [7.0, 3.0, 5.0], [2.0, 5.0, 2.0]], [[5.0, 7.0, 5.0], [1.0, 8.0, 7.0], [4.0, 4.0, 3.0]], [[6.0, 4.0, 8.0], [4.0, 1.0, 
4.0], [3.0, 5.0, 5.0]]], [[[2.0, 8.0, 3.0], [4.0, 7.0, 3.0], [2.0, 1.0, 4.0]], [[4.0, 6.0, 4.0], [5.0, 3.0, 7.0], [4.0, 5.0, 
1.0]], [[3.0, 8.0, 1.0], [2.0, 1.0, 6.0], [3.0, 5.0, 8.0]]]]))
      val=Data(numpy.array([[[1.0]], [[8.0]]]),self.functionspace)
      arg[1:3,1,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 6.0], [1.0, 8.0, 6.0], [1.0, 3.0, 1.0]], [[6.0, 5.0, 8.0], [7.0, 4.0, 4.0], [1.0, 
8.0, 6.0]], [[5.0, 6.0, 3.0], [2.0, 6.0, 6.0], [7.0, 2.0, 1.0]]], [[[5.0, 5.0, 4.0], [4.0, 8.0, 5.0], [3.0, 3.0, 4.0]], [[4.0, 
8.0, 1.0], [4.0, 2.0, 1.0], [6.0, 4.0, 7.0]], [[8.0, 2.0, 5.0], [1.0, 3.0, 8.0], [6.0, 5.0, 7.0]]], [[[6.0, 7.0, 2.0], [3.0, 
2.0, 5.0], [1.0, 5.0, 7.0]], [[4.0, 1.0, 1.0], [6.0, 2.0, 8.0], [4.0, 3.0, 1.0]], [[5.0, 5.0, 4.0], [5.0, 5.0, 7.0], [2.0, 8.0, 
3.0]]], [[[1.0, 5.0, 1.0], [5.0, 3.0, 6.0], [6.0, 5.0, 3.0]], [[1.0, 2.0, 4.0], [3.0, 7.0, 5.0], [1.0, 5.0, 1.0]], [[4.0, 4.0, 
4.0], [4.0, 5.0, 2.0], [5.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 8.0], [7.0, 8.0, 1.0], [1.0, 2.0, 5.0]], 
[[1.0, 3.0, 5.0], [6.0, 7.0, 5.0], [4.0, 5.0, 6.0]], [[8.0, 8.0, 1.0], [4.0, 4.0, 4.0], [5.0, 4.0, 2.0]]], [[[4.0, 2.0, 8.0], 
[5.0, 2.0, 6.0], [2.0, 7.0, 8.0]], [[7.0, 8.0, 5.0], [7.0, 8.0, 1.0], [6.0, 8.0, 8.0]], [[5.0, 6.0, 5.0], [1.0, 8.0, 6.0], 
[7.0, 2.0, 6.0]]], [[[4.0, 5.0, 5.0], [7.0, 3.0, 5.0], [2.0, 5.0, 2.0]], [[5.0, 7.0, 5.0], [1.0, 8.0, 8.0], [4.0, 4.0, 3.0]], 
[[6.0, 4.0, 8.0], [4.0, 1.0, 4.0], [3.0, 5.0, 5.0]]], [[[2.0, 8.0, 3.0], [4.0, 7.0, 3.0], [2.0, 1.0, 4.0]], [[4.0, 6.0, 4.0], 
[5.0, 3.0, 7.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 1.0], [2.0, 1.0, 6.0], [3.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPRL(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 6.0], [4.0, 6.0, 5.0], [7.0, 8.0, 2.0]], [[4.0, 4.0, 7.0], [4.0, 1.0, 4.0], [1.0, 8.0, 
3.0]], [[8.0, 6.0, 3.0], [8.0, 4.0, 7.0], [4.0, 8.0, 8.0]]], [[[4.0, 7.0, 8.0], [1.0, 7.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, 7.0, 
1.0], [6.0, 2.0, 1.0], [5.0, 3.0, 1.0]], [[6.0, 6.0, 5.0], [8.0, 1.0, 8.0], [4.0, 2.0, 2.0]]], [[[6.0, 7.0, 2.0], [3.0, 8.0, 
3.0], [5.0, 1.0, 8.0]], [[5.0, 8.0, 6.0], [2.0, 2.0, 6.0], [1.0, 1.0, 8.0]], [[2.0, 7.0, 4.0], [1.0, 4.0, 3.0], [3.0, 3.0, 
7.0]]], [[[1.0, 6.0, 4.0], [8.0, 4.0, 4.0], [8.0, 7.0, 8.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 3.0], [5.0, 3.0, 7.0]], [[7.0, 8.0, 
2.0], [1.0, 7.0, 3.0], [1.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 7.0], [1.0, 7.0, 2.0], [7.0, 1.0, 3.0]], [[4.0, 4.0, 8.0], [4.0, 4.0, 
6.0], [6.0, 6.0, 3.0]], [[4.0, 4.0, 4.0], [6.0, 2.0, 4.0], [8.0, 8.0, 6.0]]], [[[1.0, 3.0, 4.0], [5.0, 2.0, 2.0], [8.0, 4.0, 
3.0]], [[8.0, 3.0, 7.0], [8.0, 8.0, 5.0], [1.0, 5.0, 2.0]], [[5.0, 5.0, 4.0], [1.0, 8.0, 8.0], [5.0, 8.0, 5.0]]], [[[8.0, 2.0, 
5.0], [6.0, 1.0, 1.0], [7.0, 5.0, 8.0]], [[6.0, 5.0, 3.0], [6.0, 8.0, 5.0], [2.0, 2.0, 4.0]], [[6.0, 3.0, 8.0], [1.0, 4.0, 
2.0], [2.0, 2.0, 5.0]]], [[[5.0, 3.0, 1.0], [4.0, 1.0, 5.0], [8.0, 5.0, 8.0]], [[1.0, 3.0, 6.0], [5.0, 7.0, 8.0], [1.0, 4.0, 
8.0]], [[1.0, 1.0, 6.0], [7.0, 7.0, 8.0], [8.0, 8.0, 1.0]]]]))
      val=Data(numpy.array([[[3.0]], [[2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0]], [[4.0]]]))
      arg[1:3,1,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 6.0], [4.0, 6.0, 5.0], [7.0, 8.0, 2.0]], [[4.0, 4.0, 7.0], [4.0, 1.0, 4.0], [1.0, 
8.0, 3.0]], [[8.0, 6.0, 3.0], [8.0, 4.0, 7.0], [4.0, 8.0, 8.0]]], [[[4.0, 7.0, 8.0], [1.0, 7.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, 
7.0, 1.0], [6.0, 2.0, 3.0], [5.0, 3.0, 1.0]], [[6.0, 6.0, 5.0], [8.0, 1.0, 8.0], [4.0, 2.0, 2.0]]], [[[6.0, 7.0, 2.0], [3.0, 
8.0, 3.0], [5.0, 1.0, 8.0]], [[5.0, 8.0, 6.0], [2.0, 2.0, 2.0], [1.0, 1.0, 8.0]], [[2.0, 7.0, 4.0], [1.0, 4.0, 3.0], [3.0, 3.0, 
7.0]]], [[[1.0, 6.0, 4.0], [8.0, 4.0, 4.0], [8.0, 7.0, 8.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 3.0], [5.0, 3.0, 7.0]], [[7.0, 8.0, 
2.0], [1.0, 7.0, 3.0], [1.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 7.0], [1.0, 7.0, 2.0], [7.0, 1.0, 3.0]], 
[[4.0, 4.0, 8.0], [4.0, 4.0, 6.0], [6.0, 6.0, 3.0]], [[4.0, 4.0, 4.0], [6.0, 2.0, 4.0], [8.0, 8.0, 6.0]]], [[[1.0, 3.0, 4.0], 
[5.0, 2.0, 2.0], [8.0, 4.0, 3.0]], [[8.0, 3.0, 7.0], [8.0, 8.0, 7.0], [1.0, 5.0, 2.0]], [[5.0, 5.0, 4.0], [1.0, 8.0, 8.0], 
[5.0, 8.0, 5.0]]], [[[8.0, 2.0, 5.0], [6.0, 1.0, 1.0], [7.0, 5.0, 8.0]], [[6.0, 5.0, 3.0], [6.0, 8.0, 4.0], [2.0, 2.0, 4.0]], 
[[6.0, 3.0, 8.0], [1.0, 4.0, 2.0], [2.0, 2.0, 5.0]]], [[[5.0, 3.0, 1.0], [4.0, 1.0, 5.0], [8.0, 5.0, 8.0]], [[1.0, 3.0, 6.0], 
[5.0, 7.0, 8.0], [1.0, 4.0, 8.0]], [[1.0, 1.0, 6.0], [7.0, 7.0, 8.0], [8.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPRL(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 1.0], [6.0, 8.0, 3.0], [4.0, 3.0, 5.0]], [[8.0, 6.0, 5.0], [6.0, 6.0, 2.0], [6.0, 4.0, 
2.0]], [[8.0, 7.0, 2.0], [1.0, 2.0, 6.0], [6.0, 4.0, 5.0]]], [[[7.0, 3.0, 1.0], [2.0, 7.0, 3.0], [4.0, 4.0, 7.0]], [[3.0, 3.0, 
2.0], [6.0, 6.0, 3.0], [3.0, 3.0, 6.0]], [[2.0, 4.0, 5.0], [8.0, 4.0, 3.0], [2.0, 7.0, 3.0]]], [[[8.0, 4.0, 3.0], [3.0, 1.0, 
5.0], [4.0, 6.0, 4.0]], [[8.0, 6.0, 5.0], [4.0, 5.0, 4.0], [7.0, 3.0, 8.0]], [[2.0, 2.0, 7.0], [1.0, 2.0, 2.0], [5.0, 4.0, 
5.0]]], [[[5.0, 4.0, 3.0], [1.0, 5.0, 8.0], [6.0, 1.0, 2.0]], [[4.0, 5.0, 4.0], [3.0, 5.0, 3.0], [1.0, 8.0, 5.0]], [[3.0, 3.0, 
4.0], [7.0, 5.0, 8.0], [4.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 6.0], [1.0, 8.0, 6.0], [6.0, 3.0, 2.0]], [[5.0, 7.0, 1.0], [6.0, 3.0, 
1.0], [3.0, 3.0, 8.0]], [[1.0, 6.0, 3.0], [1.0, 6.0, 3.0], [4.0, 5.0, 5.0]]], [[[8.0, 4.0, 5.0], [8.0, 1.0, 2.0], [8.0, 6.0, 
7.0]], [[3.0, 5.0, 3.0], [2.0, 3.0, 2.0], [4.0, 8.0, 7.0]], [[1.0, 6.0, 1.0], [6.0, 8.0, 6.0], [3.0, 5.0, 2.0]]], [[[4.0, 7.0, 
7.0], [1.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[1.0, 3.0, 6.0], [8.0, 4.0, 1.0], [8.0, 5.0, 2.0]], [[7.0, 5.0, 6.0], [5.0, 5.0, 
1.0], [7.0, 3.0, 3.0]]], [[[8.0, 3.0, 1.0], [6.0, 7.0, 2.0], [8.0, 1.0, 5.0]], [[7.0, 7.0, 7.0], [7.0, 5.0, 7.0], [7.0, 3.0, 
4.0]], [[8.0, 3.0, 5.0], [3.0, 6.0, 6.0], [1.0, 1.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0]], [[1.0]]])+(1.-msk_val)*numpy.array([[[7.0]], [[3.0]]])
      arg[1:3,1,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 1.0], [6.0, 8.0, 3.0], [4.0, 3.0, 5.0]], [[8.0, 6.0, 5.0], [6.0, 6.0, 2.0], [6.0, 
4.0, 2.0]], [[8.0, 7.0, 2.0], [1.0, 2.0, 6.0], [6.0, 4.0, 5.0]]], [[[7.0, 3.0, 1.0], [2.0, 7.0, 3.0], [4.0, 4.0, 7.0]], [[3.0, 
3.0, 2.0], [6.0, 6.0, 8.0], [3.0, 3.0, 6.0]], [[2.0, 4.0, 5.0], [8.0, 4.0, 3.0], [2.0, 7.0, 3.0]]], [[[8.0, 4.0, 3.0], [3.0, 
1.0, 5.0], [4.0, 6.0, 4.0]], [[8.0, 6.0, 5.0], [4.0, 5.0, 1.0], [7.0, 3.0, 8.0]], [[2.0, 2.0, 7.0], [1.0, 2.0, 2.0], [5.0, 4.0, 
5.0]]], [[[5.0, 4.0, 3.0], [1.0, 5.0, 8.0], [6.0, 1.0, 2.0]], [[4.0, 5.0, 4.0], [3.0, 5.0, 3.0], [1.0, 8.0, 5.0]], [[3.0, 3.0, 
4.0], [7.0, 5.0, 8.0], [4.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 6.0], [1.0, 8.0, 6.0], [6.0, 3.0, 2.0]], 
[[5.0, 7.0, 1.0], [6.0, 3.0, 1.0], [3.0, 3.0, 8.0]], [[1.0, 6.0, 3.0], [1.0, 6.0, 3.0], [4.0, 5.0, 5.0]]], [[[8.0, 4.0, 5.0], 
[8.0, 1.0, 2.0], [8.0, 6.0, 7.0]], [[3.0, 5.0, 3.0], [2.0, 3.0, 7.0], [4.0, 8.0, 7.0]], [[1.0, 6.0, 1.0], [6.0, 8.0, 6.0], 
[3.0, 5.0, 2.0]]], [[[4.0, 7.0, 7.0], [1.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[1.0, 3.0, 6.0], [8.0, 4.0, 3.0], [8.0, 5.0, 2.0]], 
[[7.0, 5.0, 6.0], [5.0, 5.0, 1.0], [7.0, 3.0, 3.0]]], [[[8.0, 3.0, 1.0], [6.0, 7.0, 2.0], [8.0, 1.0, 5.0]], [[7.0, 7.0, 7.0], 
[7.0, 5.0, 7.0], [7.0, 3.0, 4.0]], [[8.0, 3.0, 5.0], [3.0, 6.0, 6.0], [1.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RCRN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 5.0], [5.0, 4.0, 7.0], [7.0, 8.0, 4.0]], [[7.0, 8.0, 1.0], [4.0, 3.0, 4.0], [3.0, 8.0, 
6.0]], [[5.0, 2.0, 5.0], [3.0, 2.0, 2.0], [3.0, 1.0, 3.0]]], [[[1.0, 7.0, 4.0], [5.0, 2.0, 7.0], [6.0, 2.0, 4.0]], [[4.0, 1.0, 
4.0], [8.0, 4.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 5.0, 5.0], [5.0, 1.0, 1.0], [5.0, 5.0, 3.0]]], [[[1.0, 6.0, 2.0], [7.0, 4.0, 
3.0], [4.0, 2.0, 4.0]], [[7.0, 8.0, 7.0], [3.0, 1.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 1.0, 6.0], [4.0, 5.0, 6.0], [1.0, 3.0, 
7.0]]], [[[3.0, 5.0, 6.0], [7.0, 7.0, 3.0], [8.0, 8.0, 4.0]], [[8.0, 5.0, 1.0], [3.0, 3.0, 6.0], [2.0, 3.0, 2.0]], [[4.0, 2.0, 
6.0], [2.0, 5.0, 8.0], [7.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 5.0], [8.0, 2.0, 7.0], [7.0, 4.0, 7.0]], [[6.0, 8.0, 6.0], [7.0, 8.0, 
7.0], [6.0, 4.0, 8.0]], [[4.0, 4.0, 4.0], [5.0, 5.0, 5.0], [8.0, 2.0, 1.0]]], [[[3.0, 1.0, 2.0], [1.0, 4.0, 5.0], [3.0, 3.0, 
6.0]], [[8.0, 8.0, 5.0], [1.0, 3.0, 4.0], [2.0, 6.0, 2.0]], [[6.0, 3.0, 3.0], [4.0, 5.0, 3.0], [8.0, 4.0, 8.0]]], [[[3.0, 5.0, 
2.0], [2.0, 4.0, 5.0], [1.0, 4.0, 6.0]], [[8.0, 7.0, 7.0], [6.0, 6.0, 4.0], [8.0, 5.0, 6.0]], [[3.0, 8.0, 6.0], [4.0, 3.0, 
7.0], [1.0, 3.0, 4.0]]], [[[8.0, 7.0, 1.0], [3.0, 4.0, 7.0], [1.0, 7.0, 2.0]], [[6.0, 1.0, 7.0], [1.0, 4.0, 8.0], [7.0, 4.0, 
5.0]], [[1.0, 6.0, 4.0], [4.0, 2.0, 5.0], [1.0, 7.0, 3.0]]]]))
      val=numpy.array([[[[4.0, 3.0, 6.0]], [[8.0, 2.0, 6.0]], [[5.0, 6.0, 7.0]]], [[[2.0, 4.0, 1.0]], [[5.0, 6.0, 4.0]], [[2.0, 
5.0, 3.0]]]])
      arg[1:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 5.0], [5.0, 4.0, 7.0], [7.0, 8.0, 4.0]], [[7.0, 8.0, 1.0], [4.0, 3.0, 4.0], [3.0, 
8.0, 6.0]], [[5.0, 2.0, 5.0], [3.0, 2.0, 2.0], [3.0, 1.0, 3.0]]], [[[1.0, 7.0, 4.0], [4.0, 3.0, 6.0], [6.0, 2.0, 4.0]], [[4.0, 
1.0, 4.0], [8.0, 2.0, 6.0], [7.0, 1.0, 4.0]], [[5.0, 5.0, 5.0], [5.0, 6.0, 7.0], [5.0, 5.0, 3.0]]], [[[1.0, 6.0, 2.0], [2.0, 
4.0, 1.0], [4.0, 2.0, 4.0]], [[7.0, 8.0, 7.0], [5.0, 6.0, 4.0], [5.0, 3.0, 7.0]], [[5.0, 1.0, 6.0], [2.0, 5.0, 3.0], [1.0, 3.0, 
7.0]]], [[[3.0, 5.0, 6.0], [7.0, 7.0, 3.0], [8.0, 8.0, 4.0]], [[8.0, 5.0, 1.0], [3.0, 3.0, 6.0], [2.0, 3.0, 2.0]], [[4.0, 2.0, 
6.0], [2.0, 5.0, 8.0], [7.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 5.0], [8.0, 2.0, 7.0], [7.0, 4.0, 7.0]], 
[[6.0, 8.0, 6.0], [7.0, 8.0, 7.0], [6.0, 4.0, 8.0]], [[4.0, 4.0, 4.0], [5.0, 5.0, 5.0], [8.0, 2.0, 1.0]]], [[[3.0, 1.0, 2.0], 
[4.0, 3.0, 6.0], [3.0, 3.0, 6.0]], [[8.0, 8.0, 5.0], [8.0, 2.0, 6.0], [2.0, 6.0, 2.0]], [[6.0, 3.0, 3.0], [5.0, 6.0, 7.0], 
[8.0, 4.0, 8.0]]], [[[3.0, 5.0, 2.0], [2.0, 4.0, 1.0], [1.0, 4.0, 6.0]], [[8.0, 7.0, 7.0], [5.0, 6.0, 4.0], [8.0, 5.0, 6.0]], 
[[3.0, 8.0, 6.0], [2.0, 5.0, 3.0], [1.0, 3.0, 4.0]]], [[[8.0, 7.0, 1.0], [3.0, 4.0, 7.0], [1.0, 7.0, 2.0]], [[6.0, 1.0, 7.0], 
[1.0, 4.0, 8.0], [7.0, 4.0, 5.0]], [[1.0, 6.0, 4.0], [4.0, 2.0, 5.0], [1.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RCRN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 4.0], [7.0, 6.0, 6.0], [7.0, 7.0, 6.0]], [[7.0, 3.0, 8.0], [6.0, 1.0, 5.0], [5.0, 8.0, 
5.0]], [[3.0, 4.0, 6.0], [2.0, 2.0, 8.0], [4.0, 1.0, 2.0]]], [[[2.0, 6.0, 4.0], [5.0, 6.0, 4.0], [7.0, 6.0, 7.0]], [[7.0, 3.0, 
8.0], [2.0, 4.0, 6.0], [7.0, 2.0, 2.0]], [[1.0, 3.0, 5.0], [2.0, 3.0, 7.0], [4.0, 2.0, 2.0]]], [[[8.0, 7.0, 1.0], [5.0, 1.0, 
5.0], [2.0, 5.0, 8.0]], [[3.0, 1.0, 6.0], [7.0, 7.0, 2.0], [7.0, 7.0, 1.0]], [[6.0, 5.0, 2.0], [7.0, 4.0, 2.0], [1.0, 4.0, 
8.0]]], [[[6.0, 4.0, 5.0], [1.0, 6.0, 7.0], [4.0, 7.0, 7.0]], [[3.0, 8.0, 5.0], [3.0, 1.0, 2.0], [7.0, 8.0, 8.0]], [[1.0, 8.0, 
4.0], [1.0, 4.0, 8.0], [6.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 1.0], [4.0, 3.0, 4.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 7.0], [3.0, 6.0, 
5.0], [3.0, 3.0, 7.0]], [[7.0, 6.0, 2.0], [7.0, 6.0, 5.0], [3.0, 1.0, 3.0]]], [[[2.0, 4.0, 5.0], [8.0, 1.0, 5.0], [4.0, 8.0, 
7.0]], [[1.0, 3.0, 5.0], [7.0, 4.0, 1.0], [3.0, 4.0, 2.0]], [[8.0, 5.0, 2.0], [1.0, 8.0, 5.0], [3.0, 1.0, 4.0]]], [[[4.0, 1.0, 
7.0], [6.0, 3.0, 3.0], [2.0, 3.0, 8.0]], [[4.0, 2.0, 3.0], [8.0, 5.0, 2.0], [8.0, 2.0, 7.0]], [[4.0, 1.0, 8.0], [2.0, 2.0, 
2.0], [1.0, 5.0, 5.0]]], [[[7.0, 8.0, 3.0], [1.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[2.0, 5.0, 4.0], [1.0, 1.0, 4.0], [2.0, 8.0, 
8.0]], [[4.0, 2.0, 7.0], [1.0, 7.0, 5.0], [5.0, 6.0, 4.0]]]]))
      val=Data(numpy.array([[[[5.0, 8.0, 7.0]], [[5.0, 8.0, 7.0]], [[4.0, 5.0, 1.0]]], [[[2.0, 5.0, 3.0]], [[8.0, 7.0, 5.0]], 
[[1.0, 2.0, 3.0]]]]),self.functionspace)
      arg[1:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 4.0], [7.0, 6.0, 6.0], [7.0, 7.0, 6.0]], [[7.0, 3.0, 8.0], [6.0, 1.0, 5.0], [5.0, 
8.0, 5.0]], [[3.0, 4.0, 6.0], [2.0, 2.0, 8.0], [4.0, 1.0, 2.0]]], [[[2.0, 6.0, 4.0], [5.0, 8.0, 7.0], [7.0, 6.0, 7.0]], [[7.0, 
3.0, 8.0], [5.0, 8.0, 7.0], [7.0, 2.0, 2.0]], [[1.0, 3.0, 5.0], [4.0, 5.0, 1.0], [4.0, 2.0, 2.0]]], [[[8.0, 7.0, 1.0], [2.0, 
5.0, 3.0], [2.0, 5.0, 8.0]], [[3.0, 1.0, 6.0], [8.0, 7.0, 5.0], [7.0, 7.0, 1.0]], [[6.0, 5.0, 2.0], [1.0, 2.0, 3.0], [1.0, 4.0, 
8.0]]], [[[6.0, 4.0, 5.0], [1.0, 6.0, 7.0], [4.0, 7.0, 7.0]], [[3.0, 8.0, 5.0], [3.0, 1.0, 2.0], [7.0, 8.0, 8.0]], [[1.0, 8.0, 
4.0], [1.0, 4.0, 8.0], [6.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 1.0], [4.0, 3.0, 4.0], [2.0, 6.0, 4.0]], 
[[5.0, 5.0, 7.0], [3.0, 6.0, 5.0], [3.0, 3.0, 7.0]], [[7.0, 6.0, 2.0], [7.0, 6.0, 5.0], [3.0, 1.0, 3.0]]], [[[2.0, 4.0, 5.0], 
[5.0, 8.0, 7.0], [4.0, 8.0, 7.0]], [[1.0, 3.0, 5.0], [5.0, 8.0, 7.0], [3.0, 4.0, 2.0]], [[8.0, 5.0, 2.0], [4.0, 5.0, 1.0], 
[3.0, 1.0, 4.0]]], [[[4.0, 1.0, 7.0], [2.0, 5.0, 3.0], [2.0, 3.0, 8.0]], [[4.0, 2.0, 3.0], [8.0, 7.0, 5.0], [8.0, 2.0, 7.0]], 
[[4.0, 1.0, 8.0], [1.0, 2.0, 3.0], [1.0, 5.0, 5.0]]], [[[7.0, 8.0, 3.0], [1.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[2.0, 5.0, 4.0], 
[1.0, 1.0, 4.0], [2.0, 8.0, 8.0]], [[4.0, 2.0, 7.0], [1.0, 7.0, 5.0], [5.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RCRN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 5.0], [4.0, 1.0, 4.0], [3.0, 1.0, 6.0]], [[1.0, 3.0, 8.0], [3.0, 4.0, 1.0], [8.0, 1.0, 
5.0]], [[3.0, 3.0, 1.0], [2.0, 8.0, 4.0], [1.0, 3.0, 7.0]]], [[[5.0, 6.0, 4.0], [6.0, 4.0, 8.0], [4.0, 8.0, 6.0]], [[6.0, 3.0, 
6.0], [7.0, 1.0, 1.0], [7.0, 4.0, 6.0]], [[4.0, 4.0, 8.0], [2.0, 7.0, 8.0], [3.0, 8.0, 4.0]]], [[[3.0, 8.0, 1.0], [1.0, 3.0, 
4.0], [8.0, 1.0, 8.0]], [[8.0, 5.0, 7.0], [1.0, 1.0, 3.0], [5.0, 3.0, 8.0]], [[5.0, 8.0, 2.0], [3.0, 5.0, 1.0], [4.0, 6.0, 
4.0]]], [[[1.0, 1.0, 3.0], [5.0, 1.0, 4.0], [8.0, 7.0, 1.0]], [[3.0, 6.0, 5.0], [5.0, 2.0, 8.0], [6.0, 3.0, 1.0]], [[8.0, 6.0, 
5.0], [3.0, 5.0, 3.0], [7.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 8.0], [2.0, 1.0, 1.0], [7.0, 3.0, 2.0]], [[3.0, 3.0, 5.0], [3.0, 7.0, 
3.0], [7.0, 3.0, 7.0]], [[8.0, 1.0, 3.0], [2.0, 2.0, 4.0], [1.0, 6.0, 5.0]]], [[[6.0, 5.0, 5.0], [5.0, 4.0, 1.0], [1.0, 6.0, 
1.0]], [[4.0, 4.0, 1.0], [7.0, 7.0, 2.0], [3.0, 1.0, 2.0]], [[6.0, 7.0, 7.0], [5.0, 5.0, 8.0], [3.0, 7.0, 6.0]]], [[[1.0, 1.0, 
8.0], [8.0, 5.0, 8.0], [7.0, 3.0, 5.0]], [[6.0, 2.0, 2.0], [7.0, 1.0, 5.0], [4.0, 4.0, 6.0]], [[6.0, 6.0, 1.0], [1.0, 8.0, 
1.0], [8.0, 6.0, 2.0]]], [[[6.0, 1.0, 2.0], [1.0, 3.0, 8.0], [7.0, 2.0, 1.0]], [[1.0, 5.0, 1.0], [1.0, 5.0, 4.0], [7.0, 1.0, 
5.0]], [[4.0, 7.0, 3.0], [1.0, 3.0, 1.0], [7.0, 4.0, 7.0]]]]))
      val=Data(numpy.array([[[[7.0, 2.0, 1.0]], [[5.0, 6.0, 3.0]], [[4.0, 3.0, 6.0]]], [[[2.0, 4.0, 3.0]], [[4.0, 1.0, 2.0]], 
[[1.0, 3.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 3.0]], [[8.0, 1.0, 6.0]], [[3.0, 3.0, 8.0]]], [[[4.0, 2.0, 6.0]], [[6.0, 
6.0, 5.0]], [[6.0, 4.0, 6.0]]]]))
      arg[1:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 5.0], [4.0, 1.0, 4.0], [3.0, 1.0, 6.0]], [[1.0, 3.0, 8.0], [3.0, 4.0, 1.0], [8.0, 
1.0, 5.0]], [[3.0, 3.0, 1.0], [2.0, 8.0, 4.0], [1.0, 3.0, 7.0]]], [[[5.0, 6.0, 4.0], [7.0, 2.0, 1.0], [4.0, 8.0, 6.0]], [[6.0, 
3.0, 6.0], [5.0, 6.0, 3.0], [7.0, 4.0, 6.0]], [[4.0, 4.0, 8.0], [4.0, 3.0, 6.0], [3.0, 8.0, 4.0]]], [[[3.0, 8.0, 1.0], [2.0, 
4.0, 3.0], [8.0, 1.0, 8.0]], [[8.0, 5.0, 7.0], [4.0, 1.0, 2.0], [5.0, 3.0, 8.0]], [[5.0, 8.0, 2.0], [1.0, 3.0, 3.0], [4.0, 6.0, 
4.0]]], [[[1.0, 1.0, 3.0], [5.0, 1.0, 4.0], [8.0, 7.0, 1.0]], [[3.0, 6.0, 5.0], [5.0, 2.0, 8.0], [6.0, 3.0, 1.0]], [[8.0, 6.0, 
5.0], [3.0, 5.0, 3.0], [7.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 8.0], [2.0, 1.0, 1.0], [7.0, 3.0, 2.0]], 
[[3.0, 3.0, 5.0], [3.0, 7.0, 3.0], [7.0, 3.0, 7.0]], [[8.0, 1.0, 3.0], [2.0, 2.0, 4.0], [1.0, 6.0, 5.0]]], [[[6.0, 5.0, 5.0], 
[2.0, 4.0, 3.0], [1.0, 6.0, 1.0]], [[4.0, 4.0, 1.0], [8.0, 1.0, 6.0], [3.0, 1.0, 2.0]], [[6.0, 7.0, 7.0], [3.0, 3.0, 8.0], 
[3.0, 7.0, 6.0]]], [[[1.0, 1.0, 8.0], [4.0, 2.0, 6.0], [7.0, 3.0, 5.0]], [[6.0, 2.0, 2.0], [6.0, 6.0, 5.0], [4.0, 4.0, 6.0]], 
[[6.0, 6.0, 1.0], [6.0, 4.0, 6.0], [8.0, 6.0, 2.0]]], [[[6.0, 1.0, 2.0], [1.0, 3.0, 8.0], [7.0, 2.0, 1.0]], [[1.0, 5.0, 1.0], 
[1.0, 5.0, 4.0], [7.0, 1.0, 5.0]], [[4.0, 7.0, 3.0], [1.0, 3.0, 1.0], [7.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RCRN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 4.0], [2.0, 1.0, 6.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 5.0], [6.0, 4.0, 4.0], [8.0, 1.0, 
2.0]], [[8.0, 8.0, 3.0], [1.0, 8.0, 2.0], [2.0, 7.0, 7.0]]], [[[3.0, 2.0, 1.0], [5.0, 8.0, 6.0], [4.0, 8.0, 4.0]], [[2.0, 5.0, 
6.0], [8.0, 8.0, 5.0], [8.0, 8.0, 4.0]], [[8.0, 8.0, 7.0], [7.0, 5.0, 3.0], [4.0, 3.0, 6.0]]], [[[5.0, 8.0, 8.0], [6.0, 6.0, 
2.0], [3.0, 4.0, 6.0]], [[7.0, 8.0, 1.0], [1.0, 5.0, 4.0], [1.0, 2.0, 3.0]], [[3.0, 4.0, 1.0], [7.0, 7.0, 5.0], [4.0, 7.0, 
3.0]]], [[[1.0, 8.0, 8.0], [8.0, 3.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 5.0, 8.0], [1.0, 1.0, 4.0], [5.0, 4.0, 1.0]], [[2.0, 4.0, 
7.0], [1.0, 8.0, 2.0], [3.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 6.0], [5.0, 4.0, 6.0], [4.0, 1.0, 2.0]], [[4.0, 8.0, 5.0], [2.0, 4.0, 
4.0], [8.0, 3.0, 2.0]], [[2.0, 5.0, 6.0], [4.0, 2.0, 4.0], [7.0, 1.0, 1.0]]], [[[4.0, 1.0, 1.0], [5.0, 6.0, 2.0], [1.0, 1.0, 
8.0]], [[6.0, 3.0, 2.0], [5.0, 8.0, 1.0], [1.0, 6.0, 7.0]], [[5.0, 6.0, 7.0], [5.0, 2.0, 3.0], [2.0, 3.0, 7.0]]], [[[8.0, 3.0, 
5.0], [4.0, 7.0, 8.0], [7.0, 3.0, 5.0]], [[6.0, 5.0, 7.0], [1.0, 4.0, 2.0], [8.0, 2.0, 6.0]], [[5.0, 2.0, 4.0], [5.0, 3.0, 
2.0], [2.0, 6.0, 8.0]]], [[[2.0, 8.0, 2.0], [4.0, 3.0, 5.0], [6.0, 2.0, 6.0]], [[6.0, 5.0, 2.0], [2.0, 4.0, 8.0], [7.0, 7.0, 
8.0]], [[1.0, 5.0, 4.0], [3.0, 8.0, 3.0], [4.0, 8.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 6.0, 4.0]], [[8.0, 7.0, 6.0]], [[6.0, 4.0, 5.0]]], [[[4.0, 8.0, 4.0]], [[2.0, 1.0, 
7.0]], [[2.0, 8.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 5.0, 3.0]], [[4.0, 7.0, 2.0]], [[5.0, 7.0, 4.0]]], [[[4.0, 2.0, 
8.0]], [[8.0, 1.0, 3.0]], [[2.0, 3.0, 2.0]]]])
      arg[1:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 4.0], [2.0, 1.0, 6.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 5.0], [6.0, 4.0, 4.0], [8.0, 
1.0, 2.0]], [[8.0, 8.0, 3.0], [1.0, 8.0, 2.0], [2.0, 7.0, 7.0]]], [[[3.0, 2.0, 1.0], [5.0, 6.0, 4.0], [4.0, 8.0, 4.0]], [[2.0, 
5.0, 6.0], [8.0, 7.0, 6.0], [8.0, 8.0, 4.0]], [[8.0, 8.0, 7.0], [6.0, 4.0, 5.0], [4.0, 3.0, 6.0]]], [[[5.0, 8.0, 8.0], [4.0, 
8.0, 4.0], [3.0, 4.0, 6.0]], [[7.0, 8.0, 1.0], [2.0, 1.0, 7.0], [1.0, 2.0, 3.0]], [[3.0, 4.0, 1.0], [2.0, 8.0, 1.0], [4.0, 7.0, 
3.0]]], [[[1.0, 8.0, 8.0], [8.0, 3.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 5.0, 8.0], [1.0, 1.0, 4.0], [5.0, 4.0, 1.0]], [[2.0, 4.0, 
7.0], [1.0, 8.0, 2.0], [3.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 6.0], [5.0, 4.0, 6.0], [4.0, 1.0, 2.0]], 
[[4.0, 8.0, 5.0], [2.0, 4.0, 4.0], [8.0, 3.0, 2.0]], [[2.0, 5.0, 6.0], [4.0, 2.0, 4.0], [7.0, 1.0, 1.0]]], [[[4.0, 1.0, 1.0], 
[4.0, 5.0, 3.0], [1.0, 1.0, 8.0]], [[6.0, 3.0, 2.0], [4.0, 7.0, 2.0], [1.0, 6.0, 7.0]], [[5.0, 6.0, 7.0], [5.0, 7.0, 4.0], 
[2.0, 3.0, 7.0]]], [[[8.0, 3.0, 5.0], [4.0, 2.0, 8.0], [7.0, 3.0, 5.0]], [[6.0, 5.0, 7.0], [8.0, 1.0, 3.0], [8.0, 2.0, 6.0]], 
[[5.0, 2.0, 4.0], [2.0, 3.0, 2.0], [2.0, 6.0, 8.0]]], [[[2.0, 8.0, 2.0], [4.0, 3.0, 5.0], [6.0, 2.0, 6.0]], [[6.0, 5.0, 2.0], 
[2.0, 4.0, 8.0], [7.0, 7.0, 8.0]], [[1.0, 5.0, 4.0], [3.0, 8.0, 3.0], [4.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLCR(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 5.0], [8.0, 1.0, 2.0], [2.0, 6.0, 7.0]], [[1.0, 1.0, 3.0], [5.0, 3.0, 5.0], [7.0, 8.0, 
1.0]], [[8.0, 5.0, 6.0], [5.0, 6.0, 5.0], [8.0, 8.0, 3.0]]], [[[6.0, 2.0, 8.0], [5.0, 7.0, 7.0], [1.0, 4.0, 2.0]], [[5.0, 6.0, 
4.0], [5.0, 1.0, 8.0], [3.0, 5.0, 5.0]], [[2.0, 2.0, 8.0], [7.0, 2.0, 3.0], [4.0, 3.0, 6.0]]], [[[7.0, 2.0, 6.0], [7.0, 7.0, 
2.0], [8.0, 6.0, 2.0]], [[7.0, 4.0, 5.0], [8.0, 3.0, 8.0], [1.0, 7.0, 3.0]], [[8.0, 6.0, 3.0], [8.0, 6.0, 6.0], [4.0, 8.0, 
2.0]]], [[[1.0, 7.0, 5.0], [6.0, 3.0, 5.0], [5.0, 5.0, 5.0]], [[4.0, 7.0, 4.0], [5.0, 3.0, 6.0], [7.0, 6.0, 2.0]], [[6.0, 5.0, 
6.0], [7.0, 8.0, 6.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 2.0], [1.0, 4.0, 2.0], [5.0, 8.0, 8.0]], [[8.0, 2.0, 4.0], [1.0, 7.0, 
1.0], [1.0, 4.0, 4.0]], [[8.0, 6.0, 3.0], [2.0, 7.0, 6.0], [5.0, 3.0, 2.0]]], [[[4.0, 7.0, 7.0], [6.0, 7.0, 7.0], [2.0, 3.0, 
5.0]], [[4.0, 2.0, 6.0], [1.0, 5.0, 2.0], [6.0, 3.0, 3.0]], [[3.0, 6.0, 7.0], [5.0, 1.0, 3.0], [5.0, 3.0, 5.0]]], [[[2.0, 7.0, 
7.0], [3.0, 1.0, 1.0], [2.0, 1.0, 7.0]], [[4.0, 4.0, 7.0], [8.0, 2.0, 7.0], [1.0, 8.0, 7.0]], [[4.0, 5.0, 5.0], [2.0, 7.0, 
5.0], [8.0, 6.0, 3.0]]], [[[1.0, 7.0, 2.0], [2.0, 4.0, 6.0], [2.0, 1.0, 8.0]], [[5.0, 7.0, 7.0], [6.0, 6.0, 6.0], [6.0, 3.0, 
1.0]], [[1.0, 4.0, 2.0], [8.0, 2.0, 3.0], [8.0, 1.0, 3.0]]]]))
      val=numpy.array([[[[2.0], [2.0], [1.0]]], [[[5.0], [5.0], [7.0]]]])
      arg[2:,2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 5.0], [8.0, 1.0, 2.0], [2.0, 6.0, 7.0]], [[1.0, 1.0, 3.0], [5.0, 3.0, 5.0], [7.0, 
8.0, 1.0]], [[8.0, 5.0, 6.0], [5.0, 6.0, 5.0], [8.0, 8.0, 3.0]]], [[[6.0, 2.0, 8.0], [5.0, 7.0, 7.0], [1.0, 4.0, 2.0]], [[5.0, 
6.0, 4.0], [5.0, 1.0, 8.0], [3.0, 5.0, 5.0]], [[2.0, 2.0, 8.0], [7.0, 2.0, 3.0], [4.0, 3.0, 6.0]]], [[[7.0, 2.0, 6.0], [7.0, 
7.0, 2.0], [8.0, 6.0, 2.0]], [[7.0, 4.0, 5.0], [8.0, 3.0, 8.0], [1.0, 7.0, 3.0]], [[8.0, 2.0, 3.0], [8.0, 2.0, 6.0], [4.0, 1.0, 
2.0]]], [[[1.0, 7.0, 5.0], [6.0, 3.0, 5.0], [5.0, 5.0, 5.0]], [[4.0, 7.0, 4.0], [5.0, 3.0, 6.0], [7.0, 6.0, 2.0]], [[6.0, 5.0, 
6.0], [7.0, 5.0, 6.0], [1.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 2.0], [1.0, 4.0, 2.0], [5.0, 8.0, 8.0]], 
[[8.0, 2.0, 4.0], [1.0, 7.0, 1.0], [1.0, 4.0, 4.0]], [[8.0, 6.0, 3.0], [2.0, 7.0, 6.0], [5.0, 3.0, 2.0]]], [[[4.0, 7.0, 7.0], 
[6.0, 7.0, 7.0], [2.0, 3.0, 5.0]], [[4.0, 2.0, 6.0], [1.0, 5.0, 2.0], [6.0, 3.0, 3.0]], [[3.0, 6.0, 7.0], [5.0, 1.0, 3.0], 
[5.0, 3.0, 5.0]]], [[[2.0, 7.0, 7.0], [3.0, 1.0, 1.0], [2.0, 1.0, 7.0]], [[4.0, 4.0, 7.0], [8.0, 2.0, 7.0], [1.0, 8.0, 7.0]], 
[[4.0, 2.0, 5.0], [2.0, 2.0, 5.0], [8.0, 1.0, 3.0]]], [[[1.0, 7.0, 2.0], [2.0, 4.0, 6.0], [2.0, 1.0, 8.0]], [[5.0, 7.0, 7.0], 
[6.0, 6.0, 6.0], [6.0, 3.0, 1.0]], [[1.0, 5.0, 2.0], [8.0, 5.0, 3.0], [8.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLCR(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 8.0], [7.0, 6.0, 6.0], [6.0, 2.0, 7.0]], [[6.0, 3.0, 5.0], [7.0, 6.0, 1.0], [5.0, 6.0, 
4.0]], [[3.0, 3.0, 4.0], [4.0, 4.0, 6.0], [4.0, 3.0, 7.0]]], [[[7.0, 7.0, 5.0], [5.0, 4.0, 4.0], [7.0, 2.0, 1.0]], [[7.0, 3.0, 
1.0], [5.0, 6.0, 1.0], [7.0, 4.0, 5.0]], [[1.0, 2.0, 1.0], [2.0, 5.0, 5.0], [7.0, 3.0, 8.0]]], [[[8.0, 1.0, 3.0], [3.0, 1.0, 
1.0], [8.0, 6.0, 7.0]], [[7.0, 1.0, 3.0], [6.0, 8.0, 8.0], [4.0, 4.0, 7.0]], [[1.0, 8.0, 3.0], [3.0, 5.0, 6.0], [6.0, 6.0, 
8.0]]], [[[8.0, 1.0, 1.0], [5.0, 6.0, 2.0], [5.0, 5.0, 2.0]], [[1.0, 2.0, 3.0], [5.0, 2.0, 5.0], [5.0, 5.0, 7.0]], [[5.0, 4.0, 
3.0], [4.0, 3.0, 4.0], [4.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 1.0], [7.0, 6.0, 6.0], [1.0, 5.0, 6.0]], [[5.0, 1.0, 5.0], [3.0, 4.0, 
7.0], [6.0, 4.0, 6.0]], [[8.0, 6.0, 2.0], [8.0, 4.0, 7.0], [8.0, 8.0, 6.0]]], [[[2.0, 3.0, 1.0], [1.0, 2.0, 6.0], [7.0, 4.0, 
8.0]], [[6.0, 8.0, 1.0], [2.0, 7.0, 3.0], [2.0, 8.0, 8.0]], [[7.0, 6.0, 7.0], [4.0, 1.0, 7.0], [7.0, 2.0, 1.0]]], [[[5.0, 3.0, 
4.0], [8.0, 8.0, 3.0], [7.0, 1.0, 1.0]], [[6.0, 5.0, 5.0], [6.0, 8.0, 8.0], [4.0, 7.0, 6.0]], [[3.0, 8.0, 8.0], [4.0, 2.0, 
7.0], [4.0, 5.0, 6.0]]], [[[8.0, 2.0, 1.0], [5.0, 6.0, 7.0], [5.0, 7.0, 4.0]], [[3.0, 6.0, 2.0], [3.0, 8.0, 4.0], [8.0, 2.0, 
4.0]], [[4.0, 7.0, 3.0], [4.0, 5.0, 7.0], [5.0, 3.0, 2.0]]]]))
      val=Data(numpy.array([[[[8.0], [1.0], [5.0]]], [[[2.0], [2.0], [2.0]]]]),self.functionspace)
      arg[2:,2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 8.0], [7.0, 6.0, 6.0], [6.0, 2.0, 7.0]], [[6.0, 3.0, 5.0], [7.0, 6.0, 1.0], [5.0, 
6.0, 4.0]], [[3.0, 3.0, 4.0], [4.0, 4.0, 6.0], [4.0, 3.0, 7.0]]], [[[7.0, 7.0, 5.0], [5.0, 4.0, 4.0], [7.0, 2.0, 1.0]], [[7.0, 
3.0, 1.0], [5.0, 6.0, 1.0], [7.0, 4.0, 5.0]], [[1.0, 2.0, 1.0], [2.0, 5.0, 5.0], [7.0, 3.0, 8.0]]], [[[8.0, 1.0, 3.0], [3.0, 
1.0, 1.0], [8.0, 6.0, 7.0]], [[7.0, 1.0, 3.0], [6.0, 8.0, 8.0], [4.0, 4.0, 7.0]], [[1.0, 8.0, 3.0], [3.0, 1.0, 6.0], [6.0, 5.0, 
8.0]]], [[[8.0, 1.0, 1.0], [5.0, 6.0, 2.0], [5.0, 5.0, 2.0]], [[1.0, 2.0, 3.0], [5.0, 2.0, 5.0], [5.0, 5.0, 7.0]], [[5.0, 2.0, 
3.0], [4.0, 2.0, 4.0], [4.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 1.0], [7.0, 6.0, 6.0], [1.0, 5.0, 6.0]], 
[[5.0, 1.0, 5.0], [3.0, 4.0, 7.0], [6.0, 4.0, 6.0]], [[8.0, 6.0, 2.0], [8.0, 4.0, 7.0], [8.0, 8.0, 6.0]]], [[[2.0, 3.0, 1.0], 
[1.0, 2.0, 6.0], [7.0, 4.0, 8.0]], [[6.0, 8.0, 1.0], [2.0, 7.0, 3.0], [2.0, 8.0, 8.0]], [[7.0, 6.0, 7.0], [4.0, 1.0, 7.0], 
[7.0, 2.0, 1.0]]], [[[5.0, 3.0, 4.0], [8.0, 8.0, 3.0], [7.0, 1.0, 1.0]], [[6.0, 5.0, 5.0], [6.0, 8.0, 8.0], [4.0, 7.0, 6.0]], 
[[3.0, 8.0, 8.0], [4.0, 1.0, 7.0], [4.0, 5.0, 6.0]]], [[[8.0, 2.0, 1.0], [5.0, 6.0, 7.0], [5.0, 7.0, 4.0]], [[3.0, 6.0, 2.0], 
[3.0, 8.0, 4.0], [8.0, 2.0, 4.0]], [[4.0, 2.0, 3.0], [4.0, 2.0, 7.0], [5.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLCR(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 2.0], [7.0, 1.0, 8.0], [8.0, 7.0, 3.0]], [[8.0, 6.0, 2.0], [7.0, 5.0, 1.0], [3.0, 3.0, 
2.0]], [[3.0, 7.0, 2.0], [8.0, 5.0, 6.0], [4.0, 2.0, 2.0]]], [[[2.0, 2.0, 2.0], [3.0, 4.0, 7.0], [1.0, 2.0, 6.0]], [[2.0, 6.0, 
7.0], [6.0, 4.0, 8.0], [5.0, 8.0, 5.0]], [[5.0, 7.0, 8.0], [2.0, 8.0, 6.0], [6.0, 5.0, 5.0]]], [[[8.0, 1.0, 2.0], [3.0, 1.0, 
7.0], [1.0, 3.0, 8.0]], [[4.0, 2.0, 6.0], [6.0, 1.0, 7.0], [7.0, 5.0, 7.0]], [[6.0, 4.0, 7.0], [1.0, 3.0, 1.0], [5.0, 2.0, 
8.0]]], [[[5.0, 8.0, 8.0], [1.0, 7.0, 6.0], [3.0, 7.0, 6.0]], [[8.0, 5.0, 1.0], [3.0, 7.0, 4.0], [4.0, 2.0, 3.0]], [[4.0, 8.0, 
5.0], [7.0, 1.0, 1.0], [7.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 1.0], [5.0, 6.0, 6.0], [2.0, 5.0, 4.0]], [[8.0, 7.0, 6.0], [4.0, 5.0, 
5.0], [8.0, 4.0, 4.0]], [[4.0, 5.0, 7.0], [8.0, 5.0, 8.0], [1.0, 3.0, 4.0]]], [[[4.0, 7.0, 4.0], [1.0, 3.0, 6.0], [1.0, 4.0, 
2.0]], [[5.0, 7.0, 2.0], [4.0, 4.0, 4.0], [7.0, 3.0, 3.0]], [[6.0, 3.0, 2.0], [1.0, 7.0, 8.0], [6.0, 7.0, 4.0]]], [[[6.0, 5.0, 
2.0], [6.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[5.0, 5.0, 2.0], [1.0, 2.0, 1.0], [6.0, 2.0, 4.0]], [[7.0, 7.0, 6.0], [1.0, 4.0, 
3.0], [4.0, 5.0, 2.0]]], [[[7.0, 5.0, 5.0], [1.0, 3.0, 4.0], [4.0, 6.0, 3.0]], [[3.0, 5.0, 4.0], [3.0, 1.0, 3.0], [2.0, 1.0, 
2.0]], [[5.0, 4.0, 6.0], [2.0, 8.0, 8.0], [4.0, 1.0, 8.0]]]]))
      val=Data(numpy.array([[[[4.0], [7.0], [4.0]]], [[[7.0], [6.0], [6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0], [2.0], [3.0]]], [[[6.0], [6.0], [4.0]]]]))
      arg[2:,2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 2.0], [7.0, 1.0, 8.0], [8.0, 7.0, 3.0]], [[8.0, 6.0, 2.0], [7.0, 5.0, 1.0], [3.0, 
3.0, 2.0]], [[3.0, 7.0, 2.0], [8.0, 5.0, 6.0], [4.0, 2.0, 2.0]]], [[[2.0, 2.0, 2.0], [3.0, 4.0, 7.0], [1.0, 2.0, 6.0]], [[2.0, 
6.0, 7.0], [6.0, 4.0, 8.0], [5.0, 8.0, 5.0]], [[5.0, 7.0, 8.0], [2.0, 8.0, 6.0], [6.0, 5.0, 5.0]]], [[[8.0, 1.0, 2.0], [3.0, 
1.0, 7.0], [1.0, 3.0, 8.0]], [[4.0, 2.0, 6.0], [6.0, 1.0, 7.0], [7.0, 5.0, 7.0]], [[6.0, 4.0, 7.0], [1.0, 7.0, 1.0], [5.0, 4.0, 
8.0]]], [[[5.0, 8.0, 8.0], [1.0, 7.0, 6.0], [3.0, 7.0, 6.0]], [[8.0, 5.0, 1.0], [3.0, 7.0, 4.0], [4.0, 2.0, 3.0]], [[4.0, 7.0, 
5.0], [7.0, 6.0, 1.0], [7.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 1.0], [5.0, 6.0, 6.0], [2.0, 5.0, 4.0]], 
[[8.0, 7.0, 6.0], [4.0, 5.0, 5.0], [8.0, 4.0, 4.0]], [[4.0, 5.0, 7.0], [8.0, 5.0, 8.0], [1.0, 3.0, 4.0]]], [[[4.0, 7.0, 4.0], 
[1.0, 3.0, 6.0], [1.0, 4.0, 2.0]], [[5.0, 7.0, 2.0], [4.0, 4.0, 4.0], [7.0, 3.0, 3.0]], [[6.0, 3.0, 2.0], [1.0, 7.0, 8.0], 
[6.0, 7.0, 4.0]]], [[[6.0, 5.0, 2.0], [6.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[5.0, 5.0, 2.0], [1.0, 2.0, 1.0], [6.0, 2.0, 4.0]], 
[[7.0, 5.0, 6.0], [1.0, 2.0, 3.0], [4.0, 3.0, 2.0]]], [[[7.0, 5.0, 5.0], [1.0, 3.0, 4.0], [4.0, 6.0, 3.0]], [[3.0, 5.0, 4.0], 
[3.0, 1.0, 3.0], [2.0, 1.0, 2.0]], [[5.0, 6.0, 6.0], [2.0, 6.0, 8.0], [4.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLCR(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 4.0], [4.0, 3.0, 5.0], [1.0, 6.0, 6.0]], [[3.0, 3.0, 4.0], [3.0, 7.0, 2.0], [6.0, 7.0, 
7.0]], [[5.0, 6.0, 8.0], [3.0, 4.0, 7.0], [1.0, 1.0, 4.0]]], [[[8.0, 7.0, 2.0], [6.0, 1.0, 3.0], [8.0, 8.0, 6.0]], [[6.0, 3.0, 
5.0], [4.0, 1.0, 1.0], [1.0, 3.0, 2.0]], [[6.0, 3.0, 3.0], [6.0, 7.0, 7.0], [1.0, 1.0, 4.0]]], [[[2.0, 2.0, 3.0], [3.0, 3.0, 
3.0], [6.0, 4.0, 3.0]], [[6.0, 4.0, 1.0], [1.0, 3.0, 4.0], [6.0, 5.0, 8.0]], [[3.0, 8.0, 7.0], [4.0, 2.0, 3.0], [3.0, 4.0, 
3.0]]], [[[2.0, 1.0, 1.0], [2.0, 2.0, 5.0], [4.0, 4.0, 8.0]], [[8.0, 1.0, 6.0], [4.0, 5.0, 6.0], [5.0, 2.0, 1.0]], [[7.0, 1.0, 
6.0], [7.0, 7.0, 7.0], [7.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 4.0], [1.0, 4.0, 5.0], [4.0, 4.0, 2.0]], [[1.0, 1.0, 6.0], [6.0, 7.0, 
2.0], [5.0, 3.0, 5.0]], [[2.0, 6.0, 2.0], [1.0, 4.0, 5.0], [3.0, 5.0, 2.0]]], [[[7.0, 6.0, 1.0], [8.0, 6.0, 4.0], [5.0, 4.0, 
2.0]], [[2.0, 5.0, 3.0], [8.0, 7.0, 6.0], [5.0, 5.0, 4.0]], [[6.0, 1.0, 3.0], [7.0, 1.0, 8.0], [6.0, 8.0, 5.0]]], [[[5.0, 2.0, 
8.0], [5.0, 5.0, 4.0], [7.0, 3.0, 5.0]], [[4.0, 4.0, 4.0], [8.0, 4.0, 5.0], [7.0, 5.0, 7.0]], [[8.0, 3.0, 2.0], [1.0, 6.0, 
1.0], [2.0, 1.0, 7.0]]], [[[2.0, 4.0, 7.0], [5.0, 4.0, 6.0], [2.0, 5.0, 2.0]], [[4.0, 3.0, 8.0], [4.0, 6.0, 4.0], [5.0, 8.0, 
2.0]], [[7.0, 1.0, 2.0], [6.0, 8.0, 2.0], [5.0, 1.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0], [5.0], [6.0]]], [[[6.0], [5.0], [5.0]]]])+(1.-msk_val)*numpy.array([[[[8.0], [1.0], 
[8.0]]], [[[2.0], [1.0], [3.0]]]])
      arg[2:,2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 4.0], [4.0, 3.0, 5.0], [1.0, 6.0, 6.0]], [[3.0, 3.0, 4.0], [3.0, 7.0, 2.0], [6.0, 
7.0, 7.0]], [[5.0, 6.0, 8.0], [3.0, 4.0, 7.0], [1.0, 1.0, 4.0]]], [[[8.0, 7.0, 2.0], [6.0, 1.0, 3.0], [8.0, 8.0, 6.0]], [[6.0, 
3.0, 5.0], [4.0, 1.0, 1.0], [1.0, 3.0, 2.0]], [[6.0, 3.0, 3.0], [6.0, 7.0, 7.0], [1.0, 1.0, 4.0]]], [[[2.0, 2.0, 3.0], [3.0, 
3.0, 3.0], [6.0, 4.0, 3.0]], [[6.0, 4.0, 1.0], [1.0, 3.0, 4.0], [6.0, 5.0, 8.0]], [[3.0, 2.0, 7.0], [4.0, 5.0, 3.0], [3.0, 6.0, 
3.0]]], [[[2.0, 1.0, 1.0], [2.0, 2.0, 5.0], [4.0, 4.0, 8.0]], [[8.0, 1.0, 6.0], [4.0, 5.0, 6.0], [5.0, 2.0, 1.0]], [[7.0, 6.0, 
6.0], [7.0, 5.0, 7.0], [7.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 4.0], [1.0, 4.0, 5.0], [4.0, 4.0, 2.0]], 
[[1.0, 1.0, 6.0], [6.0, 7.0, 2.0], [5.0, 3.0, 5.0]], [[2.0, 6.0, 2.0], [1.0, 4.0, 5.0], [3.0, 5.0, 2.0]]], [[[7.0, 6.0, 1.0], 
[8.0, 6.0, 4.0], [5.0, 4.0, 2.0]], [[2.0, 5.0, 3.0], [8.0, 7.0, 6.0], [5.0, 5.0, 4.0]], [[6.0, 1.0, 3.0], [7.0, 1.0, 8.0], 
[6.0, 8.0, 5.0]]], [[[5.0, 2.0, 8.0], [5.0, 5.0, 4.0], [7.0, 3.0, 5.0]], [[4.0, 4.0, 4.0], [8.0, 4.0, 5.0], [7.0, 5.0, 7.0]], 
[[8.0, 8.0, 2.0], [1.0, 1.0, 1.0], [2.0, 8.0, 7.0]]], [[[2.0, 4.0, 7.0], [5.0, 4.0, 6.0], [2.0, 5.0, 2.0]], [[4.0, 3.0, 8.0], 
[4.0, 6.0, 4.0], [5.0, 8.0, 2.0]], [[7.0, 2.0, 2.0], [6.0, 1.0, 2.0], [5.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRCU(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 5.0], [6.0, 1.0, 3.0], [8.0, 2.0, 4.0]], [[8.0, 6.0, 2.0], [4.0, 4.0, 5.0], [1.0, 6.0, 
2.0]], [[1.0, 7.0, 7.0], [4.0, 7.0, 1.0], [8.0, 4.0, 7.0]]], [[[7.0, 4.0, 8.0], [2.0, 2.0, 4.0], [4.0, 4.0, 5.0]], [[2.0, 1.0, 
2.0], [8.0, 1.0, 4.0], [4.0, 8.0, 3.0]], [[8.0, 1.0, 3.0], [8.0, 4.0, 5.0], [5.0, 1.0, 2.0]]], [[[3.0, 7.0, 5.0], [3.0, 1.0, 
3.0], [1.0, 7.0, 5.0]], [[3.0, 6.0, 6.0], [4.0, 7.0, 8.0], [1.0, 1.0, 7.0]], [[8.0, 7.0, 1.0], [1.0, 5.0, 3.0], [1.0, 5.0, 
7.0]]], [[[4.0, 2.0, 5.0], [3.0, 2.0, 1.0], [8.0, 6.0, 7.0]], [[3.0, 7.0, 7.0], [1.0, 8.0, 1.0], [2.0, 2.0, 1.0]], [[1.0, 6.0, 
8.0], [8.0, 8.0, 2.0], [4.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 4.0], [4.0, 5.0, 6.0], [6.0, 1.0, 6.0]], [[1.0, 7.0, 5.0], [7.0, 3.0, 
8.0], [1.0, 2.0, 4.0]], [[3.0, 7.0, 6.0], [8.0, 3.0, 2.0], [8.0, 5.0, 4.0]]], [[[7.0, 3.0, 7.0], [5.0, 7.0, 8.0], [8.0, 6.0, 
3.0]], [[4.0, 2.0, 2.0], [7.0, 8.0, 8.0], [5.0, 5.0, 1.0]], [[1.0, 5.0, 7.0], [7.0, 3.0, 3.0], [2.0, 6.0, 2.0]]], [[[2.0, 6.0, 
1.0], [5.0, 1.0, 3.0], [8.0, 8.0, 3.0]], [[1.0, 5.0, 3.0], [2.0, 3.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 6.0, 3.0], [2.0, 7.0, 
1.0], [4.0, 5.0, 4.0]]], [[[3.0, 4.0, 5.0], [2.0, 6.0, 2.0], [8.0, 2.0, 8.0]], [[1.0, 5.0, 8.0], [1.0, 6.0, 4.0], [8.0, 3.0, 
4.0]], [[1.0, 6.0, 4.0], [2.0, 7.0, 4.0], [8.0, 8.0, 6.0]]]]))
      val=numpy.array([[[[2.0, 4.0], [8.0, 2.0], [5.0, 8.0]]], [[[8.0, 1.0], [3.0, 2.0], [4.0, 1.0]]]])
      arg[1:3,1:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 5.0], [6.0, 1.0, 3.0], [8.0, 2.0, 4.0]], [[8.0, 6.0, 2.0], [4.0, 4.0, 5.0], [1.0, 
6.0, 2.0]], [[1.0, 7.0, 7.0], [4.0, 7.0, 1.0], [8.0, 4.0, 7.0]]], [[[7.0, 4.0, 8.0], [2.0, 2.0, 4.0], [4.0, 4.0, 5.0]], [[2.0, 
4.0, 2.0], [8.0, 2.0, 4.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 3.0], [8.0, 4.0, 5.0], [5.0, 1.0, 2.0]]], [[[3.0, 7.0, 5.0], [3.0, 
1.0, 3.0], [1.0, 7.0, 5.0]], [[8.0, 1.0, 6.0], [3.0, 2.0, 8.0], [4.0, 1.0, 7.0]], [[8.0, 7.0, 1.0], [1.0, 5.0, 3.0], [1.0, 5.0, 
7.0]]], [[[4.0, 2.0, 5.0], [3.0, 2.0, 1.0], [8.0, 6.0, 7.0]], [[3.0, 7.0, 7.0], [1.0, 8.0, 1.0], [2.0, 2.0, 1.0]], [[1.0, 6.0, 
8.0], [8.0, 8.0, 2.0], [4.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [4.0, 5.0, 6.0], [6.0, 1.0, 6.0]], 
[[1.0, 7.0, 5.0], [7.0, 3.0, 8.0], [1.0, 2.0, 4.0]], [[3.0, 7.0, 6.0], [8.0, 3.0, 2.0], [8.0, 5.0, 4.0]]], [[[7.0, 3.0, 7.0], 
[5.0, 7.0, 8.0], [8.0, 6.0, 3.0]], [[2.0, 4.0, 2.0], [8.0, 2.0, 8.0], [5.0, 8.0, 1.0]], [[1.0, 5.0, 7.0], [7.0, 3.0, 3.0], 
[2.0, 6.0, 2.0]]], [[[2.0, 6.0, 1.0], [5.0, 1.0, 3.0], [8.0, 8.0, 3.0]], [[8.0, 1.0, 3.0], [3.0, 2.0, 2.0], [4.0, 1.0, 2.0]], 
[[7.0, 6.0, 3.0], [2.0, 7.0, 1.0], [4.0, 5.0, 4.0]]], [[[3.0, 4.0, 5.0], [2.0, 6.0, 2.0], [8.0, 2.0, 8.0]], [[1.0, 5.0, 8.0], 
[1.0, 6.0, 4.0], [8.0, 3.0, 4.0]], [[1.0, 6.0, 4.0], [2.0, 7.0, 4.0], [8.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRCU(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 3.0], [3.0, 3.0, 4.0], [2.0, 1.0, 4.0]], [[2.0, 3.0, 5.0], [3.0, 6.0, 5.0], [7.0, 3.0, 
3.0]], [[7.0, 2.0, 8.0], [5.0, 8.0, 7.0], [1.0, 4.0, 1.0]]], [[[2.0, 1.0, 8.0], [5.0, 3.0, 5.0], [3.0, 6.0, 1.0]], [[7.0, 3.0, 
6.0], [5.0, 3.0, 2.0], [3.0, 6.0, 3.0]], [[6.0, 2.0, 7.0], [7.0, 1.0, 4.0], [4.0, 8.0, 2.0]]], [[[2.0, 1.0, 5.0], [2.0, 5.0, 
6.0], [6.0, 8.0, 6.0]], [[8.0, 6.0, 7.0], [4.0, 1.0, 4.0], [7.0, 6.0, 2.0]], [[1.0, 4.0, 3.0], [5.0, 2.0, 6.0], [5.0, 7.0, 
1.0]]], [[[8.0, 5.0, 7.0], [8.0, 6.0, 1.0], [8.0, 7.0, 2.0]], [[3.0, 8.0, 2.0], [1.0, 5.0, 7.0], [6.0, 4.0, 2.0]], [[4.0, 6.0, 
2.0], [2.0, 2.0, 2.0], [8.0, 1.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 4.0], [5.0, 5.0, 4.0], [3.0, 2.0, 8.0]], [[3.0, 6.0, 7.0], [4.0, 6.0, 
7.0], [1.0, 7.0, 4.0]], [[2.0, 1.0, 4.0], [8.0, 7.0, 7.0], [2.0, 4.0, 3.0]]], [[[1.0, 4.0, 6.0], [8.0, 5.0, 4.0], [5.0, 7.0, 
8.0]], [[7.0, 7.0, 2.0], [4.0, 8.0, 4.0], [7.0, 5.0, 4.0]], [[3.0, 6.0, 1.0], [7.0, 2.0, 1.0], [2.0, 6.0, 5.0]]], [[[7.0, 5.0, 
2.0], [7.0, 8.0, 4.0], [4.0, 8.0, 4.0]], [[6.0, 2.0, 5.0], [8.0, 7.0, 3.0], [1.0, 4.0, 5.0]], [[4.0, 7.0, 5.0], [5.0, 8.0, 
5.0], [1.0, 2.0, 2.0]]], [[[3.0, 3.0, 1.0], [8.0, 8.0, 7.0], [8.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], [8.0, 3.0, 7.0], [3.0, 4.0, 
3.0]], [[5.0, 1.0, 1.0], [7.0, 5.0, 8.0], [3.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[[[2.0, 6.0], [2.0, 2.0], [2.0, 1.0]]], [[[7.0, 8.0], [8.0, 4.0], [5.0, 7.0]]]]),self.functionspace)
      arg[1:3,1:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 3.0], [3.0, 3.0, 4.0], [2.0, 1.0, 4.0]], [[2.0, 3.0, 5.0], [3.0, 6.0, 5.0], [7.0, 
3.0, 3.0]], [[7.0, 2.0, 8.0], [5.0, 8.0, 7.0], [1.0, 4.0, 1.0]]], [[[2.0, 1.0, 8.0], [5.0, 3.0, 5.0], [3.0, 6.0, 1.0]], [[2.0, 
6.0, 6.0], [2.0, 2.0, 2.0], [2.0, 1.0, 3.0]], [[6.0, 2.0, 7.0], [7.0, 1.0, 4.0], [4.0, 8.0, 2.0]]], [[[2.0, 1.0, 5.0], [2.0, 
5.0, 6.0], [6.0, 8.0, 6.0]], [[7.0, 8.0, 7.0], [8.0, 4.0, 4.0], [5.0, 7.0, 2.0]], [[1.0, 4.0, 3.0], [5.0, 2.0, 6.0], [5.0, 7.0, 
1.0]]], [[[8.0, 5.0, 7.0], [8.0, 6.0, 1.0], [8.0, 7.0, 2.0]], [[3.0, 8.0, 2.0], [1.0, 5.0, 7.0], [6.0, 4.0, 2.0]], [[4.0, 6.0, 
2.0], [2.0, 2.0, 2.0], [8.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 4.0], [5.0, 5.0, 4.0], [3.0, 2.0, 8.0]], 
[[3.0, 6.0, 7.0], [4.0, 6.0, 7.0], [1.0, 7.0, 4.0]], [[2.0, 1.0, 4.0], [8.0, 7.0, 7.0], [2.0, 4.0, 3.0]]], [[[1.0, 4.0, 6.0], 
[8.0, 5.0, 4.0], [5.0, 7.0, 8.0]], [[2.0, 6.0, 2.0], [2.0, 2.0, 4.0], [2.0, 1.0, 4.0]], [[3.0, 6.0, 1.0], [7.0, 2.0, 1.0], 
[2.0, 6.0, 5.0]]], [[[7.0, 5.0, 2.0], [7.0, 8.0, 4.0], [4.0, 8.0, 4.0]], [[7.0, 8.0, 5.0], [8.0, 4.0, 3.0], [5.0, 7.0, 5.0]], 
[[4.0, 7.0, 5.0], [5.0, 8.0, 5.0], [1.0, 2.0, 2.0]]], [[[3.0, 3.0, 1.0], [8.0, 8.0, 7.0], [8.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], 
[8.0, 3.0, 7.0], [3.0, 4.0, 3.0]], [[5.0, 1.0, 1.0], [7.0, 5.0, 8.0], [3.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRCU(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 6.0], [6.0, 5.0, 8.0], [8.0, 4.0, 1.0]], [[4.0, 7.0, 3.0], [5.0, 7.0, 1.0], [7.0, 5.0, 
1.0]], [[3.0, 4.0, 4.0], [2.0, 6.0, 2.0], [2.0, 4.0, 2.0]]], [[[5.0, 3.0, 3.0], [7.0, 3.0, 2.0], [4.0, 3.0, 1.0]], [[5.0, 3.0, 
3.0], [4.0, 6.0, 7.0], [4.0, 4.0, 5.0]], [[7.0, 2.0, 1.0], [1.0, 6.0, 7.0], [1.0, 1.0, 8.0]]], [[[5.0, 7.0, 8.0], [6.0, 7.0, 
7.0], [1.0, 1.0, 2.0]], [[1.0, 4.0, 3.0], [7.0, 7.0, 4.0], [8.0, 8.0, 3.0]], [[6.0, 8.0, 5.0], [7.0, 8.0, 5.0], [6.0, 7.0, 
8.0]]], [[[5.0, 5.0, 5.0], [7.0, 6.0, 6.0], [4.0, 8.0, 1.0]], [[7.0, 3.0, 8.0], [1.0, 5.0, 2.0], [2.0, 5.0, 8.0]], [[7.0, 1.0, 
3.0], [6.0, 5.0, 5.0], [8.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 3.0], [3.0, 7.0, 8.0], [6.0, 5.0, 2.0]], [[7.0, 7.0, 6.0], [5.0, 6.0, 
4.0], [7.0, 2.0, 5.0]], [[1.0, 8.0, 5.0], [2.0, 6.0, 4.0], [5.0, 2.0, 3.0]]], [[[2.0, 1.0, 6.0], [5.0, 7.0, 1.0], [1.0, 1.0, 
1.0]], [[5.0, 5.0, 2.0], [6.0, 6.0, 3.0], [4.0, 4.0, 8.0]], [[5.0, 4.0, 4.0], [4.0, 1.0, 3.0], [3.0, 4.0, 3.0]]], [[[3.0, 1.0, 
5.0], [7.0, 4.0, 6.0], [3.0, 8.0, 3.0]], [[2.0, 4.0, 8.0], [4.0, 3.0, 8.0], [3.0, 2.0, 5.0]], [[8.0, 4.0, 7.0], [3.0, 6.0, 
5.0], [3.0, 5.0, 2.0]]], [[[4.0, 6.0, 4.0], [2.0, 8.0, 2.0], [1.0, 4.0, 1.0]], [[1.0, 1.0, 8.0], [7.0, 2.0, 5.0], [8.0, 1.0, 
4.0]], [[8.0, 5.0, 2.0], [8.0, 8.0, 3.0], [1.0, 5.0, 6.0]]]]))
      val=Data(numpy.array([[[[4.0, 4.0], [2.0, 1.0], [7.0, 8.0]]], [[[3.0, 1.0], [8.0, 4.0], [4.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 1.0], [7.0, 6.0], [1.0, 1.0]]], [[[3.0, 1.0], [4.0, 7.0], [1.0, 8.0]]]]))
      arg[1:3,1:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 6.0], [6.0, 5.0, 8.0], [8.0, 4.0, 1.0]], [[4.0, 7.0, 3.0], [5.0, 7.0, 1.0], [7.0, 
5.0, 1.0]], [[3.0, 4.0, 4.0], [2.0, 6.0, 2.0], [2.0, 4.0, 2.0]]], [[[5.0, 3.0, 3.0], [7.0, 3.0, 2.0], [4.0, 3.0, 1.0]], [[4.0, 
4.0, 3.0], [2.0, 1.0, 7.0], [7.0, 8.0, 5.0]], [[7.0, 2.0, 1.0], [1.0, 6.0, 7.0], [1.0, 1.0, 8.0]]], [[[5.0, 7.0, 8.0], [6.0, 
7.0, 7.0], [1.0, 1.0, 2.0]], [[3.0, 1.0, 3.0], [8.0, 4.0, 4.0], [4.0, 4.0, 3.0]], [[6.0, 8.0, 5.0], [7.0, 8.0, 5.0], [6.0, 7.0, 
8.0]]], [[[5.0, 5.0, 5.0], [7.0, 6.0, 6.0], [4.0, 8.0, 1.0]], [[7.0, 3.0, 8.0], [1.0, 5.0, 2.0], [2.0, 5.0, 8.0]], [[7.0, 1.0, 
3.0], [6.0, 5.0, 5.0], [8.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 3.0], [3.0, 7.0, 8.0], [6.0, 5.0, 2.0]], 
[[7.0, 7.0, 6.0], [5.0, 6.0, 4.0], [7.0, 2.0, 5.0]], [[1.0, 8.0, 5.0], [2.0, 6.0, 4.0], [5.0, 2.0, 3.0]]], [[[2.0, 1.0, 6.0], 
[5.0, 7.0, 1.0], [1.0, 1.0, 1.0]], [[2.0, 1.0, 2.0], [7.0, 6.0, 3.0], [1.0, 1.0, 8.0]], [[5.0, 4.0, 4.0], [4.0, 1.0, 3.0], 
[3.0, 4.0, 3.0]]], [[[3.0, 1.0, 5.0], [7.0, 4.0, 6.0], [3.0, 8.0, 3.0]], [[3.0, 1.0, 8.0], [4.0, 7.0, 8.0], [1.0, 8.0, 5.0]], 
[[8.0, 4.0, 7.0], [3.0, 6.0, 5.0], [3.0, 5.0, 2.0]]], [[[4.0, 6.0, 4.0], [2.0, 8.0, 2.0], [1.0, 4.0, 1.0]], [[1.0, 1.0, 8.0], 
[7.0, 2.0, 5.0], [8.0, 1.0, 4.0]], [[8.0, 5.0, 2.0], [8.0, 8.0, 3.0], [1.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRCU(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 2.0], [1.0, 6.0, 8.0], [6.0, 8.0, 3.0]], [[8.0, 3.0, 4.0], [8.0, 7.0, 3.0], [2.0, 7.0, 
7.0]], [[7.0, 3.0, 2.0], [2.0, 2.0, 1.0], [6.0, 7.0, 1.0]]], [[[5.0, 5.0, 5.0], [2.0, 7.0, 8.0], [5.0, 5.0, 1.0]], [[6.0, 1.0, 
6.0], [4.0, 8.0, 1.0], [3.0, 7.0, 4.0]], [[6.0, 4.0, 8.0], [4.0, 5.0, 1.0], [7.0, 2.0, 5.0]]], [[[3.0, 3.0, 7.0], [7.0, 7.0, 
5.0], [1.0, 2.0, 5.0]], [[2.0, 3.0, 2.0], [1.0, 4.0, 4.0], [4.0, 4.0, 6.0]], [[7.0, 3.0, 6.0], [6.0, 1.0, 3.0], [7.0, 2.0, 
5.0]]], [[[7.0, 8.0, 4.0], [4.0, 5.0, 6.0], [5.0, 8.0, 1.0]], [[8.0, 7.0, 1.0], [1.0, 4.0, 8.0], [4.0, 4.0, 1.0]], [[6.0, 5.0, 
3.0], [5.0, 6.0, 6.0], [6.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 4.0], [4.0, 6.0, 4.0], [5.0, 4.0, 3.0]], [[8.0, 8.0, 4.0], [6.0, 6.0, 
7.0], [6.0, 2.0, 2.0]], [[3.0, 8.0, 4.0], [3.0, 2.0, 3.0], [8.0, 6.0, 5.0]]], [[[6.0, 8.0, 1.0], [2.0, 2.0, 1.0], [4.0, 4.0, 
3.0]], [[1.0, 6.0, 5.0], [4.0, 1.0, 7.0], [2.0, 8.0, 3.0]], [[6.0, 3.0, 6.0], [5.0, 3.0, 8.0], [4.0, 4.0, 6.0]]], [[[4.0, 4.0, 
2.0], [7.0, 8.0, 8.0], [1.0, 3.0, 3.0]], [[7.0, 3.0, 8.0], [2.0, 5.0, 6.0], [3.0, 7.0, 6.0]], [[3.0, 7.0, 2.0], [4.0, 8.0, 
3.0], [3.0, 2.0, 8.0]]], [[[4.0, 6.0, 2.0], [8.0, 7.0, 3.0], [7.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [8.0, 5.0, 7.0], [1.0, 6.0, 
2.0]], [[7.0, 1.0, 3.0], [5.0, 4.0, 5.0], [3.0, 1.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0, 3.0], [3.0, 6.0], [6.0, 2.0]]], [[[4.0, 7.0], [1.0, 8.0], [2.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 4.0], [5.0, 4.0], [6.0, 6.0]]], [[[2.0, 1.0], [5.0, 1.0], [2.0, 1.0]]]])
      arg[1:3,1:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 2.0], [1.0, 6.0, 8.0], [6.0, 8.0, 3.0]], [[8.0, 3.0, 4.0], [8.0, 7.0, 3.0], [2.0, 
7.0, 7.0]], [[7.0, 3.0, 2.0], [2.0, 2.0, 1.0], [6.0, 7.0, 1.0]]], [[[5.0, 5.0, 5.0], [2.0, 7.0, 8.0], [5.0, 5.0, 1.0]], [[7.0, 
3.0, 6.0], [3.0, 6.0, 1.0], [6.0, 2.0, 4.0]], [[6.0, 4.0, 8.0], [4.0, 5.0, 1.0], [7.0, 2.0, 5.0]]], [[[3.0, 3.0, 7.0], [7.0, 
7.0, 5.0], [1.0, 2.0, 5.0]], [[4.0, 7.0, 2.0], [1.0, 8.0, 4.0], [2.0, 6.0, 6.0]], [[7.0, 3.0, 6.0], [6.0, 1.0, 3.0], [7.0, 2.0, 
5.0]]], [[[7.0, 8.0, 4.0], [4.0, 5.0, 6.0], [5.0, 8.0, 1.0]], [[8.0, 7.0, 1.0], [1.0, 4.0, 8.0], [4.0, 4.0, 1.0]], [[6.0, 5.0, 
3.0], [5.0, 6.0, 6.0], [6.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 4.0], [4.0, 6.0, 4.0], [5.0, 4.0, 3.0]], 
[[8.0, 8.0, 4.0], [6.0, 6.0, 7.0], [6.0, 2.0, 2.0]], [[3.0, 8.0, 4.0], [3.0, 2.0, 3.0], [8.0, 6.0, 5.0]]], [[[6.0, 8.0, 1.0], 
[2.0, 2.0, 1.0], [4.0, 4.0, 3.0]], [[1.0, 4.0, 5.0], [5.0, 4.0, 7.0], [6.0, 6.0, 3.0]], [[6.0, 3.0, 6.0], [5.0, 3.0, 8.0], 
[4.0, 4.0, 6.0]]], [[[4.0, 4.0, 2.0], [7.0, 8.0, 8.0], [1.0, 3.0, 3.0]], [[2.0, 1.0, 8.0], [5.0, 1.0, 6.0], [2.0, 1.0, 6.0]], 
[[3.0, 7.0, 2.0], [4.0, 8.0, 3.0], [3.0, 2.0, 8.0]]], [[[4.0, 6.0, 2.0], [8.0, 7.0, 3.0], [7.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], 
[8.0, 5.0, 7.0], [1.0, 6.0, 2.0]], [[7.0, 1.0, 3.0], [5.0, 4.0, 5.0], [3.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RLCR(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 1.0], [1.0, 8.0, 7.0], [6.0, 5.0, 6.0]], [[5.0, 6.0, 6.0], [5.0, 7.0, 1.0], [2.0, 1.0, 
4.0]], [[8.0, 7.0, 8.0], [1.0, 7.0, 7.0], [3.0, 1.0, 3.0]]], [[[3.0, 4.0, 7.0], [5.0, 3.0, 7.0], [5.0, 8.0, 5.0]], [[5.0, 7.0, 
8.0], [8.0, 2.0, 1.0], [2.0, 7.0, 3.0]], [[4.0, 3.0, 3.0], [7.0, 3.0, 2.0], [3.0, 5.0, 8.0]]], [[[1.0, 5.0, 4.0], [7.0, 1.0, 
2.0], [1.0, 8.0, 6.0]], [[2.0, 8.0, 8.0], [3.0, 1.0, 3.0], [2.0, 1.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 3.0, 8.0], [5.0, 1.0, 
8.0]]], [[[1.0, 2.0, 3.0], [7.0, 2.0, 8.0], [2.0, 7.0, 8.0]], [[2.0, 5.0, 2.0], [4.0, 6.0, 4.0], [4.0, 2.0, 6.0]], [[3.0, 8.0, 
6.0], [2.0, 2.0, 2.0], [7.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 3.0], [5.0, 2.0, 1.0], [5.0, 7.0, 4.0]], [[6.0, 6.0, 8.0], [3.0, 7.0, 
7.0], [4.0, 1.0, 8.0]], [[7.0, 2.0, 4.0], [6.0, 8.0, 1.0], [4.0, 3.0, 8.0]]], [[[3.0, 7.0, 1.0], [7.0, 3.0, 5.0], [7.0, 2.0, 
6.0]], [[6.0, 2.0, 1.0], [6.0, 3.0, 1.0], [7.0, 4.0, 4.0]], [[4.0, 6.0, 3.0], [6.0, 6.0, 3.0], [7.0, 1.0, 5.0]]], [[[8.0, 2.0, 
6.0], [6.0, 7.0, 8.0], [7.0, 5.0, 5.0]], [[7.0, 4.0, 1.0], [8.0, 2.0, 3.0], [8.0, 5.0, 4.0]], [[7.0, 7.0, 1.0], [3.0, 6.0, 
8.0], [3.0, 1.0, 8.0]]], [[[2.0, 2.0, 7.0], [5.0, 1.0, 7.0], [2.0, 5.0, 4.0]], [[2.0, 7.0, 3.0], [6.0, 5.0, 2.0], [4.0, 4.0, 
1.0]], [[7.0, 2.0, 1.0], [5.0, 2.0, 3.0], [7.0, 5.0, 7.0]]]]))
      val=numpy.array([[[[3.0], [5.0], [3.0]]], [[[3.0], [3.0], [8.0]]]])
      arg[1:3,2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 1.0], [1.0, 8.0, 7.0], [6.0, 5.0, 6.0]], [[5.0, 6.0, 6.0], [5.0, 7.0, 1.0], [2.0, 
1.0, 4.0]], [[8.0, 7.0, 8.0], [1.0, 7.0, 7.0], [3.0, 1.0, 3.0]]], [[[3.0, 4.0, 7.0], [5.0, 3.0, 7.0], [5.0, 8.0, 5.0]], [[5.0, 
7.0, 8.0], [8.0, 2.0, 1.0], [2.0, 7.0, 3.0]], [[4.0, 3.0, 3.0], [7.0, 5.0, 2.0], [3.0, 3.0, 8.0]]], [[[1.0, 5.0, 4.0], [7.0, 
1.0, 2.0], [1.0, 8.0, 6.0]], [[2.0, 8.0, 8.0], [3.0, 1.0, 3.0], [2.0, 1.0, 4.0]], [[7.0, 3.0, 5.0], [3.0, 3.0, 8.0], [5.0, 8.0, 
8.0]]], [[[1.0, 2.0, 3.0], [7.0, 2.0, 8.0], [2.0, 7.0, 8.0]], [[2.0, 5.0, 2.0], [4.0, 6.0, 4.0], [4.0, 2.0, 6.0]], [[3.0, 8.0, 
6.0], [2.0, 2.0, 2.0], [7.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 3.0], [5.0, 2.0, 1.0], [5.0, 7.0, 4.0]], 
[[6.0, 6.0, 8.0], [3.0, 7.0, 7.0], [4.0, 1.0, 8.0]], [[7.0, 2.0, 4.0], [6.0, 8.0, 1.0], [4.0, 3.0, 8.0]]], [[[3.0, 7.0, 1.0], 
[7.0, 3.0, 5.0], [7.0, 2.0, 6.0]], [[6.0, 2.0, 1.0], [6.0, 3.0, 1.0], [7.0, 4.0, 4.0]], [[4.0, 3.0, 3.0], [6.0, 5.0, 3.0], 
[7.0, 3.0, 5.0]]], [[[8.0, 2.0, 6.0], [6.0, 7.0, 8.0], [7.0, 5.0, 5.0]], [[7.0, 4.0, 1.0], [8.0, 2.0, 3.0], [8.0, 5.0, 4.0]], 
[[7.0, 3.0, 1.0], [3.0, 3.0, 8.0], [3.0, 8.0, 8.0]]], [[[2.0, 2.0, 7.0], [5.0, 1.0, 7.0], [2.0, 5.0, 4.0]], [[2.0, 7.0, 3.0], 
[6.0, 5.0, 2.0], [4.0, 4.0, 1.0]], [[7.0, 2.0, 1.0], [5.0, 2.0, 3.0], [7.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RLCR(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 3.0], [4.0, 2.0, 5.0], [7.0, 4.0, 3.0]], [[3.0, 1.0, 5.0], [5.0, 1.0, 2.0], [5.0, 5.0, 
3.0]], [[4.0, 6.0, 8.0], [1.0, 7.0, 7.0], [3.0, 6.0, 1.0]]], [[[4.0, 2.0, 4.0], [6.0, 7.0, 4.0], [1.0, 8.0, 2.0]], [[3.0, 7.0, 
5.0], [5.0, 2.0, 4.0], [5.0, 3.0, 4.0]], [[7.0, 2.0, 1.0], [7.0, 7.0, 1.0], [6.0, 2.0, 5.0]]], [[[2.0, 3.0, 6.0], [6.0, 7.0, 
4.0], [7.0, 8.0, 1.0]], [[5.0, 7.0, 2.0], [4.0, 2.0, 1.0], [7.0, 7.0, 8.0]], [[1.0, 1.0, 1.0], [7.0, 5.0, 5.0], [2.0, 8.0, 
2.0]]], [[[5.0, 7.0, 5.0], [7.0, 3.0, 3.0], [3.0, 3.0, 8.0]], [[2.0, 3.0, 4.0], [4.0, 2.0, 5.0], [2.0, 5.0, 6.0]], [[7.0, 3.0, 
6.0], [7.0, 1.0, 7.0], [2.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 6.0], [3.0, 3.0, 6.0], [6.0, 4.0, 7.0]], [[1.0, 3.0, 6.0], [6.0, 5.0, 
3.0], [2.0, 2.0, 3.0]], [[5.0, 3.0, 7.0], [1.0, 6.0, 2.0], [5.0, 3.0, 4.0]]], [[[3.0, 4.0, 3.0], [8.0, 4.0, 4.0], [2.0, 4.0, 
6.0]], [[7.0, 4.0, 6.0], [8.0, 2.0, 6.0], [8.0, 2.0, 5.0]], [[1.0, 5.0, 5.0], [4.0, 4.0, 8.0], [8.0, 3.0, 4.0]]], [[[7.0, 2.0, 
6.0], [7.0, 3.0, 5.0], [6.0, 1.0, 1.0]], [[8.0, 5.0, 2.0], [2.0, 4.0, 3.0], [3.0, 8.0, 6.0]], [[3.0, 8.0, 8.0], [4.0, 7.0, 
3.0], [8.0, 1.0, 5.0]]], [[[6.0, 5.0, 2.0], [1.0, 8.0, 3.0], [2.0, 7.0, 5.0]], [[3.0, 7.0, 8.0], [6.0, 7.0, 6.0], [4.0, 1.0, 
1.0]], [[8.0, 8.0, 4.0], [5.0, 2.0, 3.0], [3.0, 3.0, 2.0]]]]))
      val=Data(numpy.array([[[[7.0], [4.0], [4.0]]], [[[3.0], [2.0], [1.0]]]]),self.functionspace)
      arg[1:3,2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 3.0], [4.0, 2.0, 5.0], [7.0, 4.0, 3.0]], [[3.0, 1.0, 5.0], [5.0, 1.0, 2.0], [5.0, 
5.0, 3.0]], [[4.0, 6.0, 8.0], [1.0, 7.0, 7.0], [3.0, 6.0, 1.0]]], [[[4.0, 2.0, 4.0], [6.0, 7.0, 4.0], [1.0, 8.0, 2.0]], [[3.0, 
7.0, 5.0], [5.0, 2.0, 4.0], [5.0, 3.0, 4.0]], [[7.0, 7.0, 1.0], [7.0, 4.0, 1.0], [6.0, 4.0, 5.0]]], [[[2.0, 3.0, 6.0], [6.0, 
7.0, 4.0], [7.0, 8.0, 1.0]], [[5.0, 7.0, 2.0], [4.0, 2.0, 1.0], [7.0, 7.0, 8.0]], [[1.0, 3.0, 1.0], [7.0, 2.0, 5.0], [2.0, 1.0, 
2.0]]], [[[5.0, 7.0, 5.0], [7.0, 3.0, 3.0], [3.0, 3.0, 8.0]], [[2.0, 3.0, 4.0], [4.0, 2.0, 5.0], [2.0, 5.0, 6.0]], [[7.0, 3.0, 
6.0], [7.0, 1.0, 7.0], [2.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 6.0], [3.0, 3.0, 6.0], [6.0, 4.0, 7.0]], 
[[1.0, 3.0, 6.0], [6.0, 5.0, 3.0], [2.0, 2.0, 3.0]], [[5.0, 3.0, 7.0], [1.0, 6.0, 2.0], [5.0, 3.0, 4.0]]], [[[3.0, 4.0, 3.0], 
[8.0, 4.0, 4.0], [2.0, 4.0, 6.0]], [[7.0, 4.0, 6.0], [8.0, 2.0, 6.0], [8.0, 2.0, 5.0]], [[1.0, 7.0, 5.0], [4.0, 4.0, 8.0], 
[8.0, 4.0, 4.0]]], [[[7.0, 2.0, 6.0], [7.0, 3.0, 5.0], [6.0, 1.0, 1.0]], [[8.0, 5.0, 2.0], [2.0, 4.0, 3.0], [3.0, 8.0, 6.0]], 
[[3.0, 3.0, 8.0], [4.0, 2.0, 3.0], [8.0, 1.0, 5.0]]], [[[6.0, 5.0, 2.0], [1.0, 8.0, 3.0], [2.0, 7.0, 5.0]], [[3.0, 7.0, 8.0], 
[6.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[8.0, 8.0, 4.0], [5.0, 2.0, 3.0], [3.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RLCR(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 1.0], [1.0, 6.0, 5.0], [3.0, 3.0, 6.0]], [[8.0, 4.0, 7.0], [6.0, 4.0, 1.0], [5.0, 6.0, 
3.0]], [[6.0, 5.0, 3.0], [7.0, 5.0, 6.0], [2.0, 8.0, 2.0]]], [[[2.0, 1.0, 3.0], [3.0, 7.0, 5.0], [5.0, 4.0, 2.0]], [[8.0, 3.0, 
4.0], [8.0, 7.0, 8.0], [4.0, 2.0, 6.0]], [[8.0, 2.0, 8.0], [3.0, 3.0, 4.0], [6.0, 6.0, 5.0]]], [[[1.0, 8.0, 6.0], [6.0, 8.0, 
6.0], [8.0, 5.0, 3.0]], [[8.0, 6.0, 7.0], [8.0, 1.0, 7.0], [5.0, 6.0, 4.0]], [[8.0, 4.0, 6.0], [8.0, 1.0, 3.0], [3.0, 4.0, 
3.0]]], [[[1.0, 6.0, 3.0], [2.0, 6.0, 4.0], [6.0, 3.0, 5.0]], [[4.0, 4.0, 4.0], [7.0, 4.0, 4.0], [7.0, 8.0, 1.0]], [[1.0, 3.0, 
7.0], [5.0, 5.0, 1.0], [4.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 5.0], [3.0, 6.0, 7.0], [6.0, 7.0, 2.0]], [[7.0, 1.0, 5.0], [2.0, 5.0, 
7.0], [1.0, 3.0, 3.0]], [[6.0, 3.0, 3.0], [4.0, 4.0, 1.0], [3.0, 5.0, 8.0]]], [[[6.0, 8.0, 7.0], [3.0, 5.0, 3.0], [6.0, 8.0, 
3.0]], [[5.0, 4.0, 8.0], [3.0, 5.0, 6.0], [2.0, 6.0, 7.0]], [[4.0, 3.0, 8.0], [1.0, 6.0, 3.0], [1.0, 7.0, 7.0]]], [[[3.0, 5.0, 
3.0], [7.0, 2.0, 4.0], [4.0, 5.0, 8.0]], [[8.0, 8.0, 7.0], [4.0, 2.0, 5.0], [7.0, 4.0, 8.0]], [[2.0, 4.0, 4.0], [2.0, 8.0, 
7.0], [3.0, 6.0, 6.0]]], [[[7.0, 7.0, 7.0], [1.0, 4.0, 1.0], [2.0, 7.0, 1.0]], [[1.0, 5.0, 7.0], [7.0, 4.0, 5.0], [4.0, 5.0, 
8.0]], [[4.0, 5.0, 6.0], [1.0, 2.0, 5.0], [4.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[[[7.0], [1.0], [7.0]]], [[[4.0], [6.0], [8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0], [6.0], [1.0]]], [[[6.0], [3.0], [8.0]]]]))
      arg[1:3,2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 1.0], [1.0, 6.0, 5.0], [3.0, 3.0, 6.0]], [[8.0, 4.0, 7.0], [6.0, 4.0, 1.0], [5.0, 
6.0, 3.0]], [[6.0, 5.0, 3.0], [7.0, 5.0, 6.0], [2.0, 8.0, 2.0]]], [[[2.0, 1.0, 3.0], [3.0, 7.0, 5.0], [5.0, 4.0, 2.0]], [[8.0, 
3.0, 4.0], [8.0, 7.0, 8.0], [4.0, 2.0, 6.0]], [[8.0, 7.0, 8.0], [3.0, 1.0, 4.0], [6.0, 7.0, 5.0]]], [[[1.0, 8.0, 6.0], [6.0, 
8.0, 6.0], [8.0, 5.0, 3.0]], [[8.0, 6.0, 7.0], [8.0, 1.0, 7.0], [5.0, 6.0, 4.0]], [[8.0, 4.0, 6.0], [8.0, 6.0, 3.0], [3.0, 8.0, 
3.0]]], [[[1.0, 6.0, 3.0], [2.0, 6.0, 4.0], [6.0, 3.0, 5.0]], [[4.0, 4.0, 4.0], [7.0, 4.0, 4.0], [7.0, 8.0, 1.0]], [[1.0, 3.0, 
7.0], [5.0, 5.0, 1.0], [4.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 5.0], [3.0, 6.0, 7.0], [6.0, 7.0, 2.0]], 
[[7.0, 1.0, 5.0], [2.0, 5.0, 7.0], [1.0, 3.0, 3.0]], [[6.0, 3.0, 3.0], [4.0, 4.0, 1.0], [3.0, 5.0, 8.0]]], [[[6.0, 8.0, 7.0], 
[3.0, 5.0, 3.0], [6.0, 8.0, 3.0]], [[5.0, 4.0, 8.0], [3.0, 5.0, 6.0], [2.0, 6.0, 7.0]], [[4.0, 6.0, 8.0], [1.0, 6.0, 3.0], 
[1.0, 1.0, 7.0]]], [[[3.0, 5.0, 3.0], [7.0, 2.0, 4.0], [4.0, 5.0, 8.0]], [[8.0, 8.0, 7.0], [4.0, 2.0, 5.0], [7.0, 4.0, 8.0]], 
[[2.0, 6.0, 4.0], [2.0, 3.0, 7.0], [3.0, 8.0, 6.0]]], [[[7.0, 7.0, 7.0], [1.0, 4.0, 1.0], [2.0, 7.0, 1.0]], [[1.0, 5.0, 7.0], 
[7.0, 4.0, 5.0], [4.0, 5.0, 8.0]], [[4.0, 5.0, 6.0], [1.0, 2.0, 5.0], [4.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RLCR(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 5.0], [8.0, 6.0, 4.0], [3.0, 6.0, 4.0]], [[6.0, 6.0, 5.0], [3.0, 5.0, 1.0], [5.0, 2.0, 
3.0]], [[7.0, 4.0, 5.0], [5.0, 7.0, 1.0], [1.0, 4.0, 6.0]]], [[[2.0, 5.0, 1.0], [7.0, 4.0, 4.0], [8.0, 1.0, 2.0]], [[3.0, 4.0, 
8.0], [3.0, 7.0, 6.0], [6.0, 7.0, 8.0]], [[3.0, 5.0, 5.0], [7.0, 8.0, 5.0], [8.0, 3.0, 7.0]]], [[[2.0, 1.0, 2.0], [8.0, 2.0, 
8.0], [4.0, 6.0, 1.0]], [[7.0, 4.0, 6.0], [4.0, 4.0, 3.0], [1.0, 7.0, 2.0]], [[3.0, 8.0, 4.0], [6.0, 7.0, 8.0], [8.0, 4.0, 
1.0]]], [[[5.0, 8.0, 4.0], [3.0, 5.0, 2.0], [4.0, 6.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 7.0, 3.0], [6.0, 3.0, 2.0]], [[1.0, 2.0, 
3.0], [8.0, 1.0, 8.0], [3.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 2.0], [1.0, 3.0, 4.0], [8.0, 2.0, 6.0]], [[5.0, 4.0, 1.0], [4.0, 6.0, 
6.0], [4.0, 5.0, 6.0]], [[1.0, 7.0, 4.0], [5.0, 5.0, 1.0], [6.0, 3.0, 6.0]]], [[[1.0, 5.0, 5.0], [7.0, 5.0, 2.0], [8.0, 3.0, 
1.0]], [[4.0, 7.0, 6.0], [8.0, 7.0, 3.0], [6.0, 3.0, 8.0]], [[4.0, 1.0, 8.0], [7.0, 4.0, 7.0], [5.0, 4.0, 5.0]]], [[[7.0, 8.0, 
4.0], [6.0, 6.0, 6.0], [3.0, 7.0, 8.0]], [[1.0, 6.0, 7.0], [2.0, 5.0, 2.0], [8.0, 4.0, 1.0]], [[4.0, 1.0, 3.0], [3.0, 5.0, 
5.0], [1.0, 6.0, 1.0]]], [[[7.0, 7.0, 8.0], [3.0, 6.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 8.0, 3.0], [3.0, 5.0, 2.0], [8.0, 3.0, 
1.0]], [[1.0, 1.0, 3.0], [1.0, 4.0, 3.0], [3.0, 5.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0], [1.0], [1.0]]], [[[2.0], [7.0], [4.0]]]])+(1.-msk_val)*numpy.array([[[[5.0], [4.0], 
[4.0]]], [[[2.0], [3.0], [8.0]]]])
      arg[1:3,2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 5.0], [8.0, 6.0, 4.0], [3.0, 6.0, 4.0]], [[6.0, 6.0, 5.0], [3.0, 5.0, 1.0], [5.0, 
2.0, 3.0]], [[7.0, 4.0, 5.0], [5.0, 7.0, 1.0], [1.0, 4.0, 6.0]]], [[[2.0, 5.0, 1.0], [7.0, 4.0, 4.0], [8.0, 1.0, 2.0]], [[3.0, 
4.0, 8.0], [3.0, 7.0, 6.0], [6.0, 7.0, 8.0]], [[3.0, 4.0, 5.0], [7.0, 1.0, 5.0], [8.0, 1.0, 7.0]]], [[[2.0, 1.0, 2.0], [8.0, 
2.0, 8.0], [4.0, 6.0, 1.0]], [[7.0, 4.0, 6.0], [4.0, 4.0, 3.0], [1.0, 7.0, 2.0]], [[3.0, 2.0, 4.0], [6.0, 7.0, 8.0], [8.0, 4.0, 
1.0]]], [[[5.0, 8.0, 4.0], [3.0, 5.0, 2.0], [4.0, 6.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 7.0, 3.0], [6.0, 3.0, 2.0]], [[1.0, 2.0, 
3.0], [8.0, 1.0, 8.0], [3.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 2.0], [1.0, 3.0, 4.0], [8.0, 2.0, 6.0]], 
[[5.0, 4.0, 1.0], [4.0, 6.0, 6.0], [4.0, 5.0, 6.0]], [[1.0, 7.0, 4.0], [5.0, 5.0, 1.0], [6.0, 3.0, 6.0]]], [[[1.0, 5.0, 5.0], 
[7.0, 5.0, 2.0], [8.0, 3.0, 1.0]], [[4.0, 7.0, 6.0], [8.0, 7.0, 3.0], [6.0, 3.0, 8.0]], [[4.0, 5.0, 8.0], [7.0, 4.0, 7.0], 
[5.0, 4.0, 5.0]]], [[[7.0, 8.0, 4.0], [6.0, 6.0, 6.0], [3.0, 7.0, 8.0]], [[1.0, 6.0, 7.0], [2.0, 5.0, 2.0], [8.0, 4.0, 1.0]], 
[[4.0, 2.0, 3.0], [3.0, 3.0, 5.0], [1.0, 8.0, 1.0]]], [[[7.0, 7.0, 8.0], [3.0, 6.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 8.0, 3.0], 
[3.0, 5.0, 2.0], [8.0, 3.0, 1.0]], [[1.0, 1.0, 3.0], [1.0, 4.0, 3.0], [3.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRPN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 2.0], [1.0, 2.0, 7.0], [2.0, 3.0, 4.0]], [[5.0, 7.0, 2.0], [8.0, 8.0, 2.0], [6.0, 4.0, 
1.0]], [[3.0, 1.0, 1.0], [5.0, 8.0, 6.0], [1.0, 2.0, 2.0]]], [[[7.0, 7.0, 3.0], [8.0, 4.0, 6.0], [6.0, 5.0, 5.0]], [[7.0, 6.0, 
4.0], [4.0, 3.0, 6.0], [3.0, 4.0, 6.0]], [[3.0, 1.0, 2.0], [5.0, 3.0, 5.0], [5.0, 2.0, 3.0]]], [[[2.0, 6.0, 6.0], [3.0, 7.0, 
8.0], [2.0, 5.0, 4.0]], [[8.0, 1.0, 3.0], [3.0, 8.0, 2.0], [7.0, 2.0, 3.0]], [[8.0, 3.0, 6.0], [3.0, 5.0, 3.0], [5.0, 1.0, 
5.0]]], [[[8.0, 1.0, 1.0], [1.0, 8.0, 8.0], [2.0, 6.0, 6.0]], [[5.0, 4.0, 8.0], [5.0, 2.0, 7.0], [1.0, 3.0, 3.0]], [[8.0, 1.0, 
7.0], [2.0, 1.0, 1.0], [1.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 3.0], [7.0, 5.0, 8.0], [7.0, 2.0, 2.0]], [[3.0, 2.0, 3.0], [7.0, 8.0, 
5.0], [6.0, 4.0, 4.0]], [[3.0, 8.0, 8.0], [3.0, 7.0, 7.0], [5.0, 1.0, 3.0]]], [[[7.0, 3.0, 1.0], [3.0, 8.0, 3.0], [5.0, 2.0, 
5.0]], [[7.0, 6.0, 2.0], [4.0, 1.0, 6.0], [3.0, 5.0, 5.0]], [[8.0, 5.0, 3.0], [2.0, 4.0, 7.0], [4.0, 6.0, 4.0]]], [[[1.0, 7.0, 
7.0], [4.0, 5.0, 2.0], [3.0, 3.0, 2.0]], [[3.0, 3.0, 2.0], [3.0, 5.0, 7.0], [3.0, 8.0, 8.0]], [[2.0, 6.0, 4.0], [1.0, 1.0, 
4.0], [4.0, 2.0, 4.0]]], [[[8.0, 5.0, 1.0], [2.0, 8.0, 1.0], [2.0, 1.0, 6.0]], [[8.0, 6.0, 4.0], [7.0, 7.0, 8.0], [4.0, 1.0, 
8.0]], [[2.0, 2.0, 6.0], [4.0, 6.0, 1.0], [8.0, 8.0, 7.0]]]]))
      val=numpy.array([[[8.0, 7.0, 7.0]], [[1.0, 7.0, 4.0]], [[8.0, 2.0, 2.0]], [[4.0, 3.0, 5.0]]])
      arg[:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 2.0], [1.0, 2.0, 7.0], [2.0, 3.0, 4.0]], [[5.0, 7.0, 2.0], [8.0, 7.0, 7.0], [6.0, 
4.0, 1.0]], [[3.0, 1.0, 1.0], [5.0, 8.0, 6.0], [1.0, 2.0, 2.0]]], [[[7.0, 7.0, 3.0], [8.0, 4.0, 6.0], [6.0, 5.0, 5.0]], [[7.0, 
6.0, 4.0], [1.0, 7.0, 4.0], [3.0, 4.0, 6.0]], [[3.0, 1.0, 2.0], [5.0, 3.0, 5.0], [5.0, 2.0, 3.0]]], [[[2.0, 6.0, 6.0], [3.0, 
7.0, 8.0], [2.0, 5.0, 4.0]], [[8.0, 1.0, 3.0], [8.0, 2.0, 2.0], [7.0, 2.0, 3.0]], [[8.0, 3.0, 6.0], [3.0, 5.0, 3.0], [5.0, 1.0, 
5.0]]], [[[8.0, 1.0, 1.0], [1.0, 8.0, 8.0], [2.0, 6.0, 6.0]], [[5.0, 4.0, 8.0], [4.0, 3.0, 5.0], [1.0, 3.0, 3.0]], [[8.0, 1.0, 
7.0], [2.0, 1.0, 1.0], [1.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 3.0], [7.0, 5.0, 8.0], [7.0, 2.0, 2.0]], 
[[3.0, 2.0, 3.0], [8.0, 7.0, 7.0], [6.0, 4.0, 4.0]], [[3.0, 8.0, 8.0], [3.0, 7.0, 7.0], [5.0, 1.0, 3.0]]], [[[7.0, 3.0, 1.0], 
[3.0, 8.0, 3.0], [5.0, 2.0, 5.0]], [[7.0, 6.0, 2.0], [1.0, 7.0, 4.0], [3.0, 5.0, 5.0]], [[8.0, 5.0, 3.0], [2.0, 4.0, 7.0], 
[4.0, 6.0, 4.0]]], [[[1.0, 7.0, 7.0], [4.0, 5.0, 2.0], [3.0, 3.0, 2.0]], [[3.0, 3.0, 2.0], [8.0, 2.0, 2.0], [3.0, 8.0, 8.0]], 
[[2.0, 6.0, 4.0], [1.0, 1.0, 4.0], [4.0, 2.0, 4.0]]], [[[8.0, 5.0, 1.0], [2.0, 8.0, 1.0], [2.0, 1.0, 6.0]], [[8.0, 6.0, 4.0], 
[4.0, 3.0, 5.0], [4.0, 1.0, 8.0]], [[2.0, 2.0, 6.0], [4.0, 6.0, 1.0], [8.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRPN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 2.0], [6.0, 3.0, 6.0], [8.0, 4.0, 5.0]], [[8.0, 6.0, 2.0], [5.0, 6.0, 8.0], [6.0, 4.0, 
1.0]], [[1.0, 1.0, 4.0], [1.0, 5.0, 5.0], [8.0, 3.0, 5.0]]], [[[4.0, 4.0, 5.0], [4.0, 2.0, 2.0], [7.0, 4.0, 8.0]], [[1.0, 7.0, 
1.0], [2.0, 1.0, 4.0], [4.0, 5.0, 8.0]], [[6.0, 6.0, 2.0], [8.0, 7.0, 8.0], [1.0, 8.0, 5.0]]], [[[2.0, 6.0, 5.0], [2.0, 6.0, 
5.0], [3.0, 5.0, 1.0]], [[2.0, 1.0, 5.0], [1.0, 3.0, 4.0], [3.0, 8.0, 2.0]], [[4.0, 8.0, 4.0], [5.0, 7.0, 7.0], [6.0, 2.0, 
1.0]]], [[[7.0, 5.0, 5.0], [5.0, 2.0, 5.0], [7.0, 6.0, 4.0]], [[8.0, 7.0, 2.0], [5.0, 8.0, 3.0], [8.0, 3.0, 6.0]], [[3.0, 7.0, 
6.0], [5.0, 8.0, 4.0], [5.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 1.0], [2.0, 3.0, 2.0], [1.0, 7.0, 4.0]], [[6.0, 6.0, 4.0], [7.0, 1.0, 
6.0], [5.0, 6.0, 5.0]], [[7.0, 7.0, 8.0], [3.0, 3.0, 5.0], [2.0, 1.0, 1.0]]], [[[8.0, 7.0, 5.0], [6.0, 4.0, 1.0], [6.0, 6.0, 
5.0]], [[4.0, 7.0, 7.0], [2.0, 7.0, 3.0], [5.0, 1.0, 1.0]], [[7.0, 1.0, 4.0], [5.0, 7.0, 2.0], [1.0, 3.0, 7.0]]], [[[7.0, 7.0, 
2.0], [1.0, 6.0, 8.0], [3.0, 4.0, 3.0]], [[8.0, 7.0, 1.0], [7.0, 4.0, 2.0], [2.0, 5.0, 5.0]], [[7.0, 4.0, 1.0], [4.0, 1.0, 
6.0], [4.0, 6.0, 1.0]]], [[[1.0, 6.0, 2.0], [8.0, 8.0, 4.0], [8.0, 2.0, 7.0]], [[2.0, 8.0, 8.0], [6.0, 6.0, 2.0], [5.0, 8.0, 
1.0]], [[3.0, 2.0, 4.0], [3.0, 8.0, 3.0], [7.0, 2.0, 7.0]]]]))
      val=Data(numpy.array([[[6.0, 6.0, 1.0]], [[4.0, 8.0, 4.0]], [[6.0, 7.0, 1.0]], [[2.0, 7.0, 7.0]]]),self.functionspace)
      arg[:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [6.0, 3.0, 6.0], [8.0, 4.0, 5.0]], [[8.0, 6.0, 2.0], [6.0, 6.0, 1.0], [6.0, 
4.0, 1.0]], [[1.0, 1.0, 4.0], [1.0, 5.0, 5.0], [8.0, 3.0, 5.0]]], [[[4.0, 4.0, 5.0], [4.0, 2.0, 2.0], [7.0, 4.0, 8.0]], [[1.0, 
7.0, 1.0], [4.0, 8.0, 4.0], [4.0, 5.0, 8.0]], [[6.0, 6.0, 2.0], [8.0, 7.0, 8.0], [1.0, 8.0, 5.0]]], [[[2.0, 6.0, 5.0], [2.0, 
6.0, 5.0], [3.0, 5.0, 1.0]], [[2.0, 1.0, 5.0], [6.0, 7.0, 1.0], [3.0, 8.0, 2.0]], [[4.0, 8.0, 4.0], [5.0, 7.0, 7.0], [6.0, 2.0, 
1.0]]], [[[7.0, 5.0, 5.0], [5.0, 2.0, 5.0], [7.0, 6.0, 4.0]], [[8.0, 7.0, 2.0], [2.0, 7.0, 7.0], [8.0, 3.0, 6.0]], [[3.0, 7.0, 
6.0], [5.0, 8.0, 4.0], [5.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 1.0], [2.0, 3.0, 2.0], [1.0, 7.0, 4.0]], 
[[6.0, 6.0, 4.0], [6.0, 6.0, 1.0], [5.0, 6.0, 5.0]], [[7.0, 7.0, 8.0], [3.0, 3.0, 5.0], [2.0, 1.0, 1.0]]], [[[8.0, 7.0, 5.0], 
[6.0, 4.0, 1.0], [6.0, 6.0, 5.0]], [[4.0, 7.0, 7.0], [4.0, 8.0, 4.0], [5.0, 1.0, 1.0]], [[7.0, 1.0, 4.0], [5.0, 7.0, 2.0], 
[1.0, 3.0, 7.0]]], [[[7.0, 7.0, 2.0], [1.0, 6.0, 8.0], [3.0, 4.0, 3.0]], [[8.0, 7.0, 1.0], [6.0, 7.0, 1.0], [2.0, 5.0, 5.0]], 
[[7.0, 4.0, 1.0], [4.0, 1.0, 6.0], [4.0, 6.0, 1.0]]], [[[1.0, 6.0, 2.0], [8.0, 8.0, 4.0], [8.0, 2.0, 7.0]], [[2.0, 8.0, 8.0], 
[2.0, 7.0, 7.0], [5.0, 8.0, 1.0]], [[3.0, 2.0, 4.0], [3.0, 8.0, 3.0], [7.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRPN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [4.0, 7.0, 5.0], [2.0, 1.0, 4.0]], [[3.0, 2.0, 1.0], [1.0, 4.0, 3.0], [6.0, 2.0, 
1.0]], [[1.0, 6.0, 8.0], [7.0, 6.0, 4.0], [4.0, 5.0, 3.0]]], [[[5.0, 7.0, 1.0], [3.0, 8.0, 1.0], [4.0, 3.0, 6.0]], [[8.0, 4.0, 
4.0], [5.0, 2.0, 5.0], [8.0, 2.0, 7.0]], [[1.0, 8.0, 5.0], [2.0, 4.0, 8.0], [7.0, 4.0, 6.0]]], [[[3.0, 7.0, 1.0], [5.0, 5.0, 
6.0], [7.0, 1.0, 7.0]], [[6.0, 5.0, 1.0], [7.0, 7.0, 7.0], [1.0, 2.0, 7.0]], [[2.0, 5.0, 5.0], [3.0, 4.0, 3.0], [7.0, 8.0, 
5.0]]], [[[1.0, 3.0, 1.0], [4.0, 4.0, 3.0], [5.0, 4.0, 6.0]], [[7.0, 7.0, 7.0], [6.0, 1.0, 3.0], [1.0, 2.0, 4.0]], [[8.0, 7.0, 
1.0], [7.0, 3.0, 5.0], [6.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 8.0], [5.0, 2.0, 6.0], [7.0, 5.0, 3.0]], [[5.0, 2.0, 7.0], [1.0, 6.0, 
7.0], [8.0, 8.0, 8.0]], [[4.0, 4.0, 7.0], [4.0, 3.0, 2.0], [5.0, 2.0, 5.0]]], [[[7.0, 2.0, 3.0], [6.0, 8.0, 7.0], [5.0, 4.0, 
5.0]], [[4.0, 8.0, 8.0], [6.0, 6.0, 4.0], [5.0, 2.0, 2.0]], [[6.0, 8.0, 5.0], [6.0, 6.0, 7.0], [8.0, 8.0, 6.0]]], [[[3.0, 4.0, 
7.0], [5.0, 1.0, 2.0], [1.0, 1.0, 4.0]], [[2.0, 2.0, 3.0], [8.0, 4.0, 4.0], [3.0, 7.0, 6.0]], [[3.0, 2.0, 5.0], [5.0, 8.0, 
8.0], [7.0, 6.0, 4.0]]], [[[8.0, 5.0, 4.0], [7.0, 6.0, 3.0], [1.0, 4.0, 1.0]], [[6.0, 3.0, 8.0], [6.0, 7.0, 1.0], [1.0, 7.0, 
6.0]], [[1.0, 3.0, 7.0], [6.0, 8.0, 6.0], [4.0, 8.0, 4.0]]]]))
      val=Data(numpy.array([[[2.0, 8.0, 5.0]], [[5.0, 3.0, 4.0]], [[3.0, 4.0, 5.0]], [[4.0, 2.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 4.0, 3.0]], [[7.0, 6.0, 4.0]], [[8.0, 3.0, 3.0]], [[7.0, 8.0, 8.0]]]))
      arg[:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 6.0], [4.0, 7.0, 5.0], [2.0, 1.0, 4.0]], [[3.0, 2.0, 1.0], [2.0, 8.0, 5.0], [6.0, 
2.0, 1.0]], [[1.0, 6.0, 8.0], [7.0, 6.0, 4.0], [4.0, 5.0, 3.0]]], [[[5.0, 7.0, 1.0], [3.0, 8.0, 1.0], [4.0, 3.0, 6.0]], [[8.0, 
4.0, 4.0], [5.0, 3.0, 4.0], [8.0, 2.0, 7.0]], [[1.0, 8.0, 5.0], [2.0, 4.0, 8.0], [7.0, 4.0, 6.0]]], [[[3.0, 7.0, 1.0], [5.0, 
5.0, 6.0], [7.0, 1.0, 7.0]], [[6.0, 5.0, 1.0], [3.0, 4.0, 5.0], [1.0, 2.0, 7.0]], [[2.0, 5.0, 5.0], [3.0, 4.0, 3.0], [7.0, 8.0, 
5.0]]], [[[1.0, 3.0, 1.0], [4.0, 4.0, 3.0], [5.0, 4.0, 6.0]], [[7.0, 7.0, 7.0], [4.0, 2.0, 7.0], [1.0, 2.0, 4.0]], [[8.0, 7.0, 
1.0], [7.0, 3.0, 5.0], [6.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 8.0], [5.0, 2.0, 6.0], [7.0, 5.0, 3.0]], 
[[5.0, 2.0, 7.0], [2.0, 4.0, 3.0], [8.0, 8.0, 8.0]], [[4.0, 4.0, 7.0], [4.0, 3.0, 2.0], [5.0, 2.0, 5.0]]], [[[7.0, 2.0, 3.0], 
[6.0, 8.0, 7.0], [5.0, 4.0, 5.0]], [[4.0, 8.0, 8.0], [7.0, 6.0, 4.0], [5.0, 2.0, 2.0]], [[6.0, 8.0, 5.0], [6.0, 6.0, 7.0], 
[8.0, 8.0, 6.0]]], [[[3.0, 4.0, 7.0], [5.0, 1.0, 2.0], [1.0, 1.0, 4.0]], [[2.0, 2.0, 3.0], [8.0, 3.0, 3.0], [3.0, 7.0, 6.0]], 
[[3.0, 2.0, 5.0], [5.0, 8.0, 8.0], [7.0, 6.0, 4.0]]], [[[8.0, 5.0, 4.0], [7.0, 6.0, 3.0], [1.0, 4.0, 1.0]], [[6.0, 3.0, 8.0], 
[7.0, 8.0, 8.0], [1.0, 7.0, 6.0]], [[1.0, 3.0, 7.0], [6.0, 8.0, 6.0], [4.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRPN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 7.0], [3.0, 1.0, 1.0], [1.0, 4.0, 4.0]], [[7.0, 4.0, 2.0], [8.0, 5.0, 2.0], [5.0, 6.0, 
8.0]], [[3.0, 6.0, 3.0], [6.0, 6.0, 3.0], [3.0, 2.0, 4.0]]], [[[2.0, 6.0, 7.0], [4.0, 5.0, 5.0], [2.0, 8.0, 3.0]], [[5.0, 5.0, 
1.0], [7.0, 6.0, 6.0], [6.0, 2.0, 3.0]], [[7.0, 1.0, 8.0], [6.0, 7.0, 3.0], [3.0, 3.0, 3.0]]], [[[7.0, 7.0, 2.0], [6.0, 3.0, 
1.0], [3.0, 7.0, 6.0]], [[3.0, 3.0, 3.0], [4.0, 1.0, 5.0], [1.0, 2.0, 8.0]], [[7.0, 7.0, 1.0], [5.0, 4.0, 1.0], [8.0, 8.0, 
3.0]]], [[[6.0, 3.0, 6.0], [5.0, 8.0, 8.0], [1.0, 3.0, 4.0]], [[8.0, 1.0, 6.0], [4.0, 3.0, 8.0], [8.0, 5.0, 4.0]], [[1.0, 1.0, 
6.0], [3.0, 8.0, 8.0], [2.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 8.0], [8.0, 5.0, 2.0], [3.0, 6.0, 6.0]], [[3.0, 3.0, 8.0], [8.0, 3.0, 
4.0], [7.0, 7.0, 4.0]], [[2.0, 6.0, 5.0], [4.0, 1.0, 6.0], [6.0, 3.0, 3.0]]], [[[8.0, 6.0, 2.0], [7.0, 5.0, 4.0], [3.0, 4.0, 
1.0]], [[4.0, 5.0, 4.0], [1.0, 4.0, 1.0], [7.0, 8.0, 2.0]], [[3.0, 4.0, 3.0], [2.0, 5.0, 6.0], [2.0, 8.0, 1.0]]], [[[5.0, 1.0, 
4.0], [7.0, 4.0, 3.0], [7.0, 6.0, 3.0]], [[4.0, 2.0, 1.0], [8.0, 7.0, 2.0], [7.0, 1.0, 8.0]], [[4.0, 7.0, 6.0], [3.0, 3.0, 
2.0], [5.0, 7.0, 4.0]]], [[[7.0, 5.0, 6.0], [2.0, 2.0, 3.0], [3.0, 3.0, 3.0]], [[3.0, 6.0, 2.0], [8.0, 3.0, 8.0], [2.0, 2.0, 
6.0]], [[7.0, 2.0, 5.0], [2.0, 6.0, 8.0], [2.0, 8.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[6.0, 6.0, 6.0]], [[8.0, 7.0, 7.0]], [[6.0, 8.0, 2.0]], [[6.0, 8.0, 
3.0]]])+(1.-msk_val)*numpy.array([[[8.0, 3.0, 3.0]], [[6.0, 7.0, 4.0]], [[4.0, 7.0, 4.0]], [[8.0, 8.0, 3.0]]])
      arg[:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 7.0], [3.0, 1.0, 1.0], [1.0, 4.0, 4.0]], [[7.0, 4.0, 2.0], [6.0, 6.0, 6.0], [5.0, 
6.0, 8.0]], [[3.0, 6.0, 3.0], [6.0, 6.0, 3.0], [3.0, 2.0, 4.0]]], [[[2.0, 6.0, 7.0], [4.0, 5.0, 5.0], [2.0, 8.0, 3.0]], [[5.0, 
5.0, 1.0], [8.0, 7.0, 7.0], [6.0, 2.0, 3.0]], [[7.0, 1.0, 8.0], [6.0, 7.0, 3.0], [3.0, 3.0, 3.0]]], [[[7.0, 7.0, 2.0], [6.0, 
3.0, 1.0], [3.0, 7.0, 6.0]], [[3.0, 3.0, 3.0], [6.0, 8.0, 2.0], [1.0, 2.0, 8.0]], [[7.0, 7.0, 1.0], [5.0, 4.0, 1.0], [8.0, 8.0, 
3.0]]], [[[6.0, 3.0, 6.0], [5.0, 8.0, 8.0], [1.0, 3.0, 4.0]], [[8.0, 1.0, 6.0], [6.0, 8.0, 3.0], [8.0, 5.0, 4.0]], [[1.0, 1.0, 
6.0], [3.0, 8.0, 8.0], [2.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 8.0], [8.0, 5.0, 2.0], [3.0, 6.0, 6.0]], 
[[3.0, 3.0, 8.0], [8.0, 3.0, 3.0], [7.0, 7.0, 4.0]], [[2.0, 6.0, 5.0], [4.0, 1.0, 6.0], [6.0, 3.0, 3.0]]], [[[8.0, 6.0, 2.0], 
[7.0, 5.0, 4.0], [3.0, 4.0, 1.0]], [[4.0, 5.0, 4.0], [6.0, 7.0, 4.0], [7.0, 8.0, 2.0]], [[3.0, 4.0, 3.0], [2.0, 5.0, 6.0], 
[2.0, 8.0, 1.0]]], [[[5.0, 1.0, 4.0], [7.0, 4.0, 3.0], [7.0, 6.0, 3.0]], [[4.0, 2.0, 1.0], [4.0, 7.0, 4.0], [7.0, 1.0, 8.0]], 
[[4.0, 7.0, 6.0], [3.0, 3.0, 2.0], [5.0, 7.0, 4.0]]], [[[7.0, 5.0, 6.0], [2.0, 2.0, 3.0], [3.0, 3.0, 3.0]], [[3.0, 6.0, 2.0], 
[8.0, 8.0, 3.0], [2.0, 2.0, 6.0]], [[7.0, 2.0, 5.0], [2.0, 6.0, 8.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UUPC(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 3.0], [2.0, 8.0, 7.0], [4.0, 5.0, 5.0]], [[7.0, 2.0, 1.0], [3.0, 8.0, 8.0], [7.0, 6.0, 
2.0]], [[3.0, 4.0, 6.0], [3.0, 5.0, 6.0], [1.0, 3.0, 8.0]]], [[[7.0, 6.0, 5.0], [6.0, 1.0, 6.0], [5.0, 3.0, 3.0]], [[6.0, 2.0, 
7.0], [3.0, 4.0, 1.0], [1.0, 6.0, 2.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 7.0], [7.0, 5.0, 3.0]]], [[[8.0, 7.0, 5.0], [5.0, 7.0, 
1.0], [4.0, 5.0, 3.0]], [[2.0, 6.0, 7.0], [1.0, 5.0, 3.0], [3.0, 1.0, 7.0]], [[3.0, 6.0, 1.0], [8.0, 7.0, 7.0], [1.0, 5.0, 
5.0]]], [[[3.0, 6.0, 5.0], [4.0, 4.0, 3.0], [8.0, 4.0, 2.0]], [[5.0, 2.0, 7.0], [2.0, 1.0, 1.0], [2.0, 3.0, 5.0]], [[7.0, 7.0, 
4.0], [4.0, 4.0, 4.0], [2.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 8.0], [8.0, 8.0, 2.0], [7.0, 8.0, 5.0]], [[5.0, 3.0, 1.0], [4.0, 6.0, 
6.0], [8.0, 5.0, 7.0]], [[7.0, 4.0, 5.0], [8.0, 3.0, 1.0], [1.0, 7.0, 6.0]]], [[[3.0, 7.0, 7.0], [3.0, 5.0, 3.0], [8.0, 3.0, 
7.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 7.0], [4.0, 6.0, 2.0]], [[8.0, 4.0, 6.0], [8.0, 4.0, 6.0], [1.0, 3.0, 2.0]]], [[[3.0, 6.0, 
2.0], [2.0, 2.0, 6.0], [6.0, 6.0, 5.0]], [[5.0, 8.0, 7.0], [8.0, 1.0, 2.0], [6.0, 3.0, 2.0]], [[8.0, 7.0, 3.0], [6.0, 6.0, 
1.0], [2.0, 5.0, 5.0]]], [[[7.0, 1.0, 7.0], [4.0, 5.0, 2.0], [8.0, 3.0, 5.0]], [[2.0, 4.0, 6.0], [6.0, 4.0, 3.0], [7.0, 8.0, 
8.0]], [[5.0, 8.0, 8.0], [8.0, 4.0, 3.0], [5.0, 6.0, 3.0]]]]))
      val=numpy.array([[[6.0, 5.0, 4.0], [1.0, 3.0, 8.0]], [[8.0, 3.0, 2.0], [2.0, 8.0, 8.0]], [[1.0, 5.0, 2.0], [2.0, 1.0, 
3.0]]])
      arg[:3,:2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 3.0], [6.0, 5.0, 4.0], [4.0, 5.0, 5.0]], [[7.0, 2.0, 1.0], [1.0, 3.0, 8.0], [7.0, 
6.0, 2.0]], [[3.0, 4.0, 6.0], [3.0, 5.0, 6.0], [1.0, 3.0, 8.0]]], [[[7.0, 6.0, 5.0], [8.0, 3.0, 2.0], [5.0, 3.0, 3.0]], [[6.0, 
2.0, 7.0], [2.0, 8.0, 8.0], [1.0, 6.0, 2.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 7.0], [7.0, 5.0, 3.0]]], [[[8.0, 7.0, 5.0], [1.0, 
5.0, 2.0], [4.0, 5.0, 3.0]], [[2.0, 6.0, 7.0], [2.0, 1.0, 3.0], [3.0, 1.0, 7.0]], [[3.0, 6.0, 1.0], [8.0, 7.0, 7.0], [1.0, 5.0, 
5.0]]], [[[3.0, 6.0, 5.0], [4.0, 4.0, 3.0], [8.0, 4.0, 2.0]], [[5.0, 2.0, 7.0], [2.0, 1.0, 1.0], [2.0, 3.0, 5.0]], [[7.0, 7.0, 
4.0], [4.0, 4.0, 4.0], [2.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 8.0], [6.0, 5.0, 4.0], [7.0, 8.0, 5.0]], 
[[5.0, 3.0, 1.0], [1.0, 3.0, 8.0], [8.0, 5.0, 7.0]], [[7.0, 4.0, 5.0], [8.0, 3.0, 1.0], [1.0, 7.0, 6.0]]], [[[3.0, 7.0, 7.0], 
[8.0, 3.0, 2.0], [8.0, 3.0, 7.0]], [[3.0, 8.0, 8.0], [2.0, 8.0, 8.0], [4.0, 6.0, 2.0]], [[8.0, 4.0, 6.0], [8.0, 4.0, 6.0], 
[1.0, 3.0, 2.0]]], [[[3.0, 6.0, 2.0], [1.0, 5.0, 2.0], [6.0, 6.0, 5.0]], [[5.0, 8.0, 7.0], [2.0, 1.0, 3.0], [6.0, 3.0, 2.0]], 
[[8.0, 7.0, 3.0], [6.0, 6.0, 1.0], [2.0, 5.0, 5.0]]], [[[7.0, 1.0, 7.0], [4.0, 5.0, 2.0], [8.0, 3.0, 5.0]], [[2.0, 4.0, 6.0], 
[6.0, 4.0, 3.0], [7.0, 8.0, 8.0]], [[5.0, 8.0, 8.0], [8.0, 4.0, 3.0], [5.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UUPC(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 4.0], [4.0, 5.0, 4.0], [4.0, 7.0, 2.0]], [[5.0, 4.0, 2.0], [8.0, 5.0, 7.0], [4.0, 6.0, 
7.0]], [[5.0, 2.0, 7.0], [6.0, 6.0, 2.0], [2.0, 3.0, 7.0]]], [[[3.0, 4.0, 2.0], [2.0, 2.0, 2.0], [4.0, 6.0, 5.0]], [[4.0, 2.0, 
7.0], [3.0, 4.0, 6.0], [6.0, 8.0, 6.0]], [[6.0, 8.0, 3.0], [8.0, 2.0, 4.0], [2.0, 8.0, 6.0]]], [[[6.0, 4.0, 3.0], [6.0, 2.0, 
4.0], [4.0, 5.0, 3.0]], [[3.0, 8.0, 4.0], [5.0, 2.0, 8.0], [6.0, 6.0, 2.0]], [[1.0, 1.0, 1.0], [5.0, 4.0, 4.0], [3.0, 8.0, 
1.0]]], [[[3.0, 8.0, 8.0], [4.0, 6.0, 3.0], [7.0, 5.0, 5.0]], [[3.0, 7.0, 5.0], [2.0, 6.0, 6.0], [2.0, 3.0, 6.0]], [[6.0, 2.0, 
5.0], [8.0, 6.0, 2.0], [7.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 6.0], [8.0, 5.0, 6.0], [5.0, 8.0, 2.0]], [[3.0, 4.0, 7.0], [5.0, 3.0, 
3.0], [6.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [4.0, 2.0, 5.0], [3.0, 1.0, 3.0]]], [[[8.0, 3.0, 2.0], [8.0, 7.0, 6.0], [4.0, 5.0, 
5.0]], [[7.0, 7.0, 5.0], [8.0, 5.0, 5.0], [4.0, 1.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 6.0, 5.0], [2.0, 5.0, 7.0]]], [[[8.0, 8.0, 
8.0], [6.0, 6.0, 5.0], [1.0, 4.0, 1.0]], [[2.0, 4.0, 5.0], [1.0, 1.0, 7.0], [7.0, 2.0, 7.0]], [[2.0, 8.0, 2.0], [3.0, 7.0, 
8.0], [1.0, 5.0, 7.0]]], [[[2.0, 5.0, 8.0], [6.0, 2.0, 7.0], [8.0, 6.0, 1.0]], [[3.0, 5.0, 7.0], [1.0, 7.0, 1.0], [1.0, 6.0, 
1.0]], [[3.0, 3.0, 5.0], [1.0, 1.0, 2.0], [2.0, 1.0, 5.0]]]]))
      val=Data(numpy.array([[[8.0, 5.0, 5.0], [3.0, 3.0, 1.0]], [[8.0, 2.0, 7.0], [3.0, 5.0, 2.0]], [[5.0, 7.0, 6.0], [5.0, 
1.0, 1.0]]]),self.functionspace)
      arg[:3,:2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 4.0], [8.0, 5.0, 5.0], [4.0, 7.0, 2.0]], [[5.0, 4.0, 2.0], [3.0, 3.0, 1.0], [4.0, 
6.0, 7.0]], [[5.0, 2.0, 7.0], [6.0, 6.0, 2.0], [2.0, 3.0, 7.0]]], [[[3.0, 4.0, 2.0], [8.0, 2.0, 7.0], [4.0, 6.0, 5.0]], [[4.0, 
2.0, 7.0], [3.0, 5.0, 2.0], [6.0, 8.0, 6.0]], [[6.0, 8.0, 3.0], [8.0, 2.0, 4.0], [2.0, 8.0, 6.0]]], [[[6.0, 4.0, 3.0], [5.0, 
7.0, 6.0], [4.0, 5.0, 3.0]], [[3.0, 8.0, 4.0], [5.0, 1.0, 1.0], [6.0, 6.0, 2.0]], [[1.0, 1.0, 1.0], [5.0, 4.0, 4.0], [3.0, 8.0, 
1.0]]], [[[3.0, 8.0, 8.0], [4.0, 6.0, 3.0], [7.0, 5.0, 5.0]], [[3.0, 7.0, 5.0], [2.0, 6.0, 6.0], [2.0, 3.0, 6.0]], [[6.0, 2.0, 
5.0], [8.0, 6.0, 2.0], [7.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 6.0], [8.0, 5.0, 5.0], [5.0, 8.0, 2.0]], 
[[3.0, 4.0, 7.0], [3.0, 3.0, 1.0], [6.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [4.0, 2.0, 5.0], [3.0, 1.0, 3.0]]], [[[8.0, 3.0, 2.0], 
[8.0, 2.0, 7.0], [4.0, 5.0, 5.0]], [[7.0, 7.0, 5.0], [3.0, 5.0, 2.0], [4.0, 1.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 6.0, 5.0], 
[2.0, 5.0, 7.0]]], [[[8.0, 8.0, 8.0], [5.0, 7.0, 6.0], [1.0, 4.0, 1.0]], [[2.0, 4.0, 5.0], [5.0, 1.0, 1.0], [7.0, 2.0, 7.0]], 
[[2.0, 8.0, 2.0], [3.0, 7.0, 8.0], [1.0, 5.0, 7.0]]], [[[2.0, 5.0, 8.0], [6.0, 2.0, 7.0], [8.0, 6.0, 1.0]], [[3.0, 5.0, 7.0], 
[1.0, 7.0, 1.0], [1.0, 6.0, 1.0]], [[3.0, 3.0, 5.0], [1.0, 1.0, 2.0], [2.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UUPC(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 1.0], [4.0, 1.0, 1.0], [3.0, 4.0, 4.0]], [[1.0, 8.0, 5.0], [7.0, 8.0, 3.0], [3.0, 2.0, 
4.0]], [[7.0, 1.0, 4.0], [1.0, 4.0, 6.0], [2.0, 2.0, 7.0]]], [[[4.0, 4.0, 2.0], [8.0, 4.0, 3.0], [8.0, 1.0, 2.0]], [[3.0, 4.0, 
7.0], [7.0, 8.0, 2.0], [5.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [5.0, 8.0, 7.0], [6.0, 6.0, 4.0]]], [[[1.0, 1.0, 6.0], [3.0, 4.0, 
4.0], [2.0, 6.0, 3.0]], [[1.0, 4.0, 5.0], [3.0, 3.0, 1.0], [1.0, 2.0, 2.0]], [[7.0, 6.0, 2.0], [1.0, 8.0, 4.0], [2.0, 6.0, 
3.0]]], [[[1.0, 3.0, 6.0], [6.0, 8.0, 5.0], [7.0, 5.0, 8.0]], [[8.0, 8.0, 5.0], [2.0, 4.0, 8.0], [8.0, 1.0, 8.0]], [[4.0, 1.0, 
8.0], [4.0, 7.0, 4.0], [6.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 3.0], [1.0, 6.0, 2.0], [2.0, 1.0, 6.0]], [[2.0, 4.0, 4.0], [8.0, 8.0, 
8.0], [6.0, 1.0, 1.0]], [[8.0, 7.0, 7.0], [3.0, 1.0, 4.0], [6.0, 6.0, 8.0]]], [[[8.0, 1.0, 2.0], [2.0, 3.0, 1.0], [8.0, 8.0, 
8.0]], [[3.0, 4.0, 6.0], [2.0, 5.0, 1.0], [6.0, 1.0, 2.0]], [[7.0, 6.0, 5.0], [3.0, 7.0, 3.0], [4.0, 3.0, 6.0]]], [[[5.0, 6.0, 
2.0], [5.0, 4.0, 5.0], [4.0, 7.0, 1.0]], [[5.0, 4.0, 4.0], [3.0, 8.0, 6.0], [4.0, 5.0, 3.0]], [[4.0, 2.0, 7.0], [5.0, 3.0, 
8.0], [3.0, 5.0, 1.0]]], [[[3.0, 7.0, 1.0], [2.0, 6.0, 5.0], [3.0, 6.0, 4.0]], [[1.0, 5.0, 2.0], [2.0, 3.0, 4.0], [4.0, 8.0, 
8.0]], [[3.0, 7.0, 5.0], [6.0, 3.0, 3.0], [3.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[[3.0, 6.0, 4.0], [8.0, 1.0, 8.0]], [[3.0, 8.0, 2.0], [4.0, 7.0, 1.0]], [[1.0, 4.0, 5.0], [4.0, 
2.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 7.0, 1.0], [6.0, 5.0, 8.0]], [[2.0, 3.0, 1.0], [3.0, 2.0, 3.0]], [[7.0, 3.0, 
1.0], [3.0, 4.0, 6.0]]]))
      arg[:3,:2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 1.0], [3.0, 6.0, 4.0], [3.0, 4.0, 4.0]], [[1.0, 8.0, 5.0], [8.0, 1.0, 8.0], [3.0, 
2.0, 4.0]], [[7.0, 1.0, 4.0], [1.0, 4.0, 6.0], [2.0, 2.0, 7.0]]], [[[4.0, 4.0, 2.0], [3.0, 8.0, 2.0], [8.0, 1.0, 2.0]], [[3.0, 
4.0, 7.0], [4.0, 7.0, 1.0], [5.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [5.0, 8.0, 7.0], [6.0, 6.0, 4.0]]], [[[1.0, 1.0, 6.0], [1.0, 
4.0, 5.0], [2.0, 6.0, 3.0]], [[1.0, 4.0, 5.0], [4.0, 2.0, 3.0], [1.0, 2.0, 2.0]], [[7.0, 6.0, 2.0], [1.0, 8.0, 4.0], [2.0, 6.0, 
3.0]]], [[[1.0, 3.0, 6.0], [6.0, 8.0, 5.0], [7.0, 5.0, 8.0]], [[8.0, 8.0, 5.0], [2.0, 4.0, 8.0], [8.0, 1.0, 8.0]], [[4.0, 1.0, 
8.0], [4.0, 7.0, 4.0], [6.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 3.0], [6.0, 7.0, 1.0], [2.0, 1.0, 6.0]], 
[[2.0, 4.0, 4.0], [6.0, 5.0, 8.0], [6.0, 1.0, 1.0]], [[8.0, 7.0, 7.0], [3.0, 1.0, 4.0], [6.0, 6.0, 8.0]]], [[[8.0, 1.0, 2.0], 
[2.0, 3.0, 1.0], [8.0, 8.0, 8.0]], [[3.0, 4.0, 6.0], [3.0, 2.0, 3.0], [6.0, 1.0, 2.0]], [[7.0, 6.0, 5.0], [3.0, 7.0, 3.0], 
[4.0, 3.0, 6.0]]], [[[5.0, 6.0, 2.0], [7.0, 3.0, 1.0], [4.0, 7.0, 1.0]], [[5.0, 4.0, 4.0], [3.0, 4.0, 6.0], [4.0, 5.0, 3.0]], 
[[4.0, 2.0, 7.0], [5.0, 3.0, 8.0], [3.0, 5.0, 1.0]]], [[[3.0, 7.0, 1.0], [2.0, 6.0, 5.0], [3.0, 6.0, 4.0]], [[1.0, 5.0, 2.0], 
[2.0, 3.0, 4.0], [4.0, 8.0, 8.0]], [[3.0, 7.0, 5.0], [6.0, 3.0, 3.0], [3.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UUPC(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 6.0], [5.0, 7.0, 8.0], [5.0, 5.0, 4.0]], [[3.0, 1.0, 3.0], [7.0, 8.0, 6.0], [1.0, 3.0, 
6.0]], [[6.0, 3.0, 1.0], [1.0, 1.0, 8.0], [1.0, 2.0, 5.0]]], [[[3.0, 7.0, 3.0], [8.0, 3.0, 1.0], [3.0, 8.0, 5.0]], [[1.0, 8.0, 
1.0], [1.0, 6.0, 4.0], [1.0, 6.0, 1.0]], [[3.0, 5.0, 4.0], [8.0, 1.0, 3.0], [1.0, 1.0, 2.0]]], [[[3.0, 2.0, 2.0], [6.0, 2.0, 
6.0], [3.0, 2.0, 4.0]], [[8.0, 7.0, 3.0], [5.0, 4.0, 4.0], [2.0, 3.0, 7.0]], [[3.0, 2.0, 3.0], [7.0, 5.0, 2.0], [6.0, 2.0, 
4.0]]], [[[2.0, 1.0, 6.0], [4.0, 3.0, 2.0], [6.0, 8.0, 7.0]], [[2.0, 6.0, 5.0], [7.0, 5.0, 8.0], [3.0, 2.0, 7.0]], [[5.0, 8.0, 
7.0], [1.0, 6.0, 2.0], [8.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 5.0], [4.0, 2.0, 1.0], [3.0, 2.0, 4.0]], [[3.0, 6.0, 4.0], [6.0, 5.0, 
7.0], [1.0, 3.0, 3.0]], [[2.0, 7.0, 8.0], [2.0, 4.0, 7.0], [3.0, 5.0, 2.0]]], [[[5.0, 5.0, 5.0], [1.0, 6.0, 1.0], [1.0, 4.0, 
1.0]], [[3.0, 8.0, 5.0], [8.0, 1.0, 3.0], [3.0, 7.0, 8.0]], [[8.0, 8.0, 5.0], [7.0, 5.0, 3.0], [2.0, 6.0, 8.0]]], [[[3.0, 2.0, 
8.0], [7.0, 5.0, 2.0], [8.0, 6.0, 1.0]], [[1.0, 3.0, 4.0], [7.0, 5.0, 2.0], [4.0, 3.0, 1.0]], [[4.0, 3.0, 6.0], [6.0, 7.0, 
8.0], [7.0, 6.0, 4.0]]], [[[6.0, 3.0, 6.0], [8.0, 1.0, 1.0], [3.0, 4.0, 4.0]], [[1.0, 5.0, 1.0], [7.0, 2.0, 2.0], [8.0, 5.0, 
1.0]], [[5.0, 4.0, 7.0], [2.0, 1.0, 8.0], [1.0, 4.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[2.0, 3.0, 3.0], [3.0, 3.0, 3.0]], [[7.0, 8.0, 7.0], [5.0, 2.0, 7.0]], [[3.0, 6.0, 5.0], [8.0, 
1.0, 7.0]]])+(1.-msk_val)*numpy.array([[[8.0, 5.0, 3.0], [2.0, 2.0, 7.0]], [[8.0, 2.0, 1.0], [1.0, 7.0, 1.0]], [[5.0, 5.0, 
8.0], [7.0, 3.0, 4.0]]])
      arg[:3,:2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 6.0], [2.0, 3.0, 3.0], [5.0, 5.0, 4.0]], [[3.0, 1.0, 3.0], [3.0, 3.0, 3.0], [1.0, 
3.0, 6.0]], [[6.0, 3.0, 1.0], [1.0, 1.0, 8.0], [1.0, 2.0, 5.0]]], [[[3.0, 7.0, 3.0], [7.0, 8.0, 7.0], [3.0, 8.0, 5.0]], [[1.0, 
8.0, 1.0], [5.0, 2.0, 7.0], [1.0, 6.0, 1.0]], [[3.0, 5.0, 4.0], [8.0, 1.0, 3.0], [1.0, 1.0, 2.0]]], [[[3.0, 2.0, 2.0], [3.0, 
6.0, 5.0], [3.0, 2.0, 4.0]], [[8.0, 7.0, 3.0], [8.0, 1.0, 7.0], [2.0, 3.0, 7.0]], [[3.0, 2.0, 3.0], [7.0, 5.0, 2.0], [6.0, 2.0, 
4.0]]], [[[2.0, 1.0, 6.0], [4.0, 3.0, 2.0], [6.0, 8.0, 7.0]], [[2.0, 6.0, 5.0], [7.0, 5.0, 8.0], [3.0, 2.0, 7.0]], [[5.0, 8.0, 
7.0], [1.0, 6.0, 2.0], [8.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 5.0], [8.0, 5.0, 3.0], [3.0, 2.0, 4.0]], 
[[3.0, 6.0, 4.0], [2.0, 2.0, 7.0], [1.0, 3.0, 3.0]], [[2.0, 7.0, 8.0], [2.0, 4.0, 7.0], [3.0, 5.0, 2.0]]], [[[5.0, 5.0, 5.0], 
[8.0, 2.0, 1.0], [1.0, 4.0, 1.0]], [[3.0, 8.0, 5.0], [1.0, 7.0, 1.0], [3.0, 7.0, 8.0]], [[8.0, 8.0, 5.0], [7.0, 5.0, 3.0], 
[2.0, 6.0, 8.0]]], [[[3.0, 2.0, 8.0], [5.0, 5.0, 8.0], [8.0, 6.0, 1.0]], [[1.0, 3.0, 4.0], [7.0, 3.0, 4.0], [4.0, 3.0, 1.0]], 
[[4.0, 3.0, 6.0], [6.0, 7.0, 8.0], [7.0, 6.0, 4.0]]], [[[6.0, 3.0, 6.0], [8.0, 1.0, 1.0], [3.0, 4.0, 4.0]], [[1.0, 5.0, 1.0], 
[7.0, 2.0, 2.0], [8.0, 5.0, 1.0]], [[5.0, 4.0, 7.0], [2.0, 1.0, 8.0], [1.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_URRR(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 5.0], [1.0, 8.0, 3.0], [1.0, 8.0, 2.0]], [[2.0, 5.0, 5.0], [3.0, 1.0, 8.0], [5.0, 7.0, 
3.0]], [[7.0, 7.0, 2.0], [3.0, 2.0, 6.0], [3.0, 4.0, 5.0]]], [[[1.0, 1.0, 3.0], [5.0, 7.0, 8.0], [2.0, 8.0, 7.0]], [[6.0, 8.0, 
7.0], [6.0, 2.0, 5.0], [8.0, 1.0, 3.0]], [[8.0, 2.0, 1.0], [1.0, 1.0, 6.0], [2.0, 2.0, 3.0]]], [[[5.0, 2.0, 2.0], [7.0, 3.0, 
8.0], [1.0, 2.0, 4.0]], [[8.0, 2.0, 1.0], [2.0, 4.0, 2.0], [1.0, 3.0, 1.0]], [[1.0, 1.0, 5.0], [3.0, 1.0, 4.0], [3.0, 4.0, 
5.0]]], [[[2.0, 7.0, 8.0], [8.0, 6.0, 2.0], [4.0, 7.0, 7.0]], [[5.0, 4.0, 1.0], [3.0, 6.0, 8.0], [4.0, 1.0, 2.0]], [[5.0, 8.0, 
7.0], [6.0, 2.0, 6.0], [3.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 5.0], [5.0, 6.0, 6.0], [5.0, 8.0, 1.0]], [[5.0, 8.0, 3.0], [2.0, 7.0, 
7.0], [1.0, 5.0, 7.0]], [[2.0, 7.0, 6.0], [2.0, 2.0, 8.0], [2.0, 6.0, 2.0]]], [[[6.0, 5.0, 8.0], [2.0, 6.0, 1.0], [4.0, 7.0, 
2.0]], [[2.0, 7.0, 3.0], [3.0, 7.0, 5.0], [7.0, 4.0, 2.0]], [[8.0, 6.0, 8.0], [8.0, 4.0, 8.0], [7.0, 8.0, 7.0]]], [[[4.0, 8.0, 
3.0], [5.0, 8.0, 7.0], [7.0, 1.0, 1.0]], [[6.0, 4.0, 5.0], [3.0, 2.0, 1.0], [4.0, 3.0, 3.0]], [[3.0, 2.0, 7.0], [3.0, 8.0, 
6.0], [2.0, 4.0, 2.0]]], [[[8.0, 8.0, 3.0], [5.0, 5.0, 7.0], [8.0, 2.0, 1.0]], [[1.0, 3.0, 5.0], [5.0, 5.0, 2.0], [4.0, 1.0, 
1.0]], [[3.0, 8.0, 2.0], [7.0, 1.0, 3.0], [6.0, 7.0, 1.0]]]]))
      val=numpy.array([[[[2.0]]], [[[4.0]]], [[[4.0]]]])
      arg[:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 5.0], [1.0, 8.0, 3.0], [1.0, 8.0, 2.0]], [[2.0, 5.0, 5.0], [3.0, 2.0, 8.0], [5.0, 
7.0, 3.0]], [[7.0, 7.0, 2.0], [3.0, 2.0, 6.0], [3.0, 4.0, 5.0]]], [[[1.0, 1.0, 3.0], [5.0, 7.0, 8.0], [2.0, 8.0, 7.0]], [[6.0, 
8.0, 7.0], [6.0, 4.0, 5.0], [8.0, 1.0, 3.0]], [[8.0, 2.0, 1.0], [1.0, 1.0, 6.0], [2.0, 2.0, 3.0]]], [[[5.0, 2.0, 2.0], [7.0, 
3.0, 8.0], [1.0, 2.0, 4.0]], [[8.0, 2.0, 1.0], [2.0, 4.0, 2.0], [1.0, 3.0, 1.0]], [[1.0, 1.0, 5.0], [3.0, 1.0, 4.0], [3.0, 4.0, 
5.0]]], [[[2.0, 7.0, 8.0], [8.0, 6.0, 2.0], [4.0, 7.0, 7.0]], [[5.0, 4.0, 1.0], [3.0, 6.0, 8.0], [4.0, 1.0, 2.0]], [[5.0, 8.0, 
7.0], [6.0, 2.0, 6.0], [3.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 5.0], [5.0, 6.0, 6.0], [5.0, 8.0, 1.0]], 
[[5.0, 8.0, 3.0], [2.0, 2.0, 7.0], [1.0, 5.0, 7.0]], [[2.0, 7.0, 6.0], [2.0, 2.0, 8.0], [2.0, 6.0, 2.0]]], [[[6.0, 5.0, 8.0], 
[2.0, 6.0, 1.0], [4.0, 7.0, 2.0]], [[2.0, 7.0, 3.0], [3.0, 4.0, 5.0], [7.0, 4.0, 2.0]], [[8.0, 6.0, 8.0], [8.0, 4.0, 8.0], 
[7.0, 8.0, 7.0]]], [[[4.0, 8.0, 3.0], [5.0, 8.0, 7.0], [7.0, 1.0, 1.0]], [[6.0, 4.0, 5.0], [3.0, 4.0, 1.0], [4.0, 3.0, 3.0]], 
[[3.0, 2.0, 7.0], [3.0, 8.0, 6.0], [2.0, 4.0, 2.0]]], [[[8.0, 8.0, 3.0], [5.0, 5.0, 7.0], [8.0, 2.0, 1.0]], [[1.0, 3.0, 5.0], 
[5.0, 5.0, 2.0], [4.0, 1.0, 1.0]], [[3.0, 8.0, 2.0], [7.0, 1.0, 3.0], [6.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_URRR(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 4.0], [5.0, 4.0, 8.0], [3.0, 5.0, 7.0]], [[1.0, 5.0, 4.0], [1.0, 8.0, 1.0], [1.0, 1.0, 
6.0]], [[8.0, 4.0, 8.0], [6.0, 3.0, 1.0], [8.0, 3.0, 2.0]]], [[[7.0, 7.0, 7.0], [4.0, 2.0, 4.0], [2.0, 6.0, 8.0]], [[8.0, 1.0, 
2.0], [4.0, 5.0, 2.0], [4.0, 8.0, 1.0]], [[1.0, 4.0, 3.0], [6.0, 5.0, 6.0], [3.0, 8.0, 8.0]]], [[[6.0, 1.0, 8.0], [7.0, 7.0, 
2.0], [5.0, 6.0, 1.0]], [[2.0, 7.0, 8.0], [6.0, 1.0, 2.0], [1.0, 2.0, 1.0]], [[3.0, 4.0, 4.0], [1.0, 7.0, 1.0], [8.0, 7.0, 
2.0]]], [[[3.0, 5.0, 6.0], [2.0, 5.0, 4.0], [4.0, 3.0, 1.0]], [[1.0, 8.0, 1.0], [4.0, 7.0, 5.0], [8.0, 7.0, 7.0]], [[1.0, 2.0, 
1.0], [2.0, 8.0, 8.0], [2.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 6.0], [6.0, 6.0, 4.0], [6.0, 8.0, 7.0]], [[5.0, 1.0, 3.0], [6.0, 2.0, 
5.0], [8.0, 8.0, 1.0]], [[2.0, 1.0, 4.0], [3.0, 1.0, 3.0], [3.0, 2.0, 3.0]]], [[[6.0, 1.0, 4.0], [2.0, 3.0, 5.0], [8.0, 7.0, 
5.0]], [[4.0, 8.0, 2.0], [5.0, 2.0, 4.0], [7.0, 3.0, 3.0]], [[4.0, 7.0, 5.0], [3.0, 3.0, 8.0], [8.0, 4.0, 6.0]]], [[[2.0, 5.0, 
1.0], [1.0, 8.0, 5.0], [4.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], [3.0, 1.0, 7.0], [7.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [1.0, 5.0, 
3.0], [8.0, 3.0, 1.0]]], [[[1.0, 5.0, 7.0], [2.0, 5.0, 7.0], [6.0, 3.0, 7.0]], [[3.0, 3.0, 2.0], [4.0, 7.0, 8.0], [7.0, 7.0, 
6.0]], [[8.0, 6.0, 1.0], [4.0, 4.0, 7.0], [1.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[[2.0]]], [[[5.0]]], [[[4.0]]]]),self.functionspace)
      arg[:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 4.0], [5.0, 4.0, 8.0], [3.0, 5.0, 7.0]], [[1.0, 5.0, 4.0], [1.0, 2.0, 1.0], [1.0, 
1.0, 6.0]], [[8.0, 4.0, 8.0], [6.0, 3.0, 1.0], [8.0, 3.0, 2.0]]], [[[7.0, 7.0, 7.0], [4.0, 2.0, 4.0], [2.0, 6.0, 8.0]], [[8.0, 
1.0, 2.0], [4.0, 5.0, 2.0], [4.0, 8.0, 1.0]], [[1.0, 4.0, 3.0], [6.0, 5.0, 6.0], [3.0, 8.0, 8.0]]], [[[6.0, 1.0, 8.0], [7.0, 
7.0, 2.0], [5.0, 6.0, 1.0]], [[2.0, 7.0, 8.0], [6.0, 4.0, 2.0], [1.0, 2.0, 1.0]], [[3.0, 4.0, 4.0], [1.0, 7.0, 1.0], [8.0, 7.0, 
2.0]]], [[[3.0, 5.0, 6.0], [2.0, 5.0, 4.0], [4.0, 3.0, 1.0]], [[1.0, 8.0, 1.0], [4.0, 7.0, 5.0], [8.0, 7.0, 7.0]], [[1.0, 2.0, 
1.0], [2.0, 8.0, 8.0], [2.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 6.0], [6.0, 6.0, 4.0], [6.0, 8.0, 7.0]], 
[[5.0, 1.0, 3.0], [6.0, 2.0, 5.0], [8.0, 8.0, 1.0]], [[2.0, 1.0, 4.0], [3.0, 1.0, 3.0], [3.0, 2.0, 3.0]]], [[[6.0, 1.0, 4.0], 
[2.0, 3.0, 5.0], [8.0, 7.0, 5.0]], [[4.0, 8.0, 2.0], [5.0, 5.0, 4.0], [7.0, 3.0, 3.0]], [[4.0, 7.0, 5.0], [3.0, 3.0, 8.0], 
[8.0, 4.0, 6.0]]], [[[2.0, 5.0, 1.0], [1.0, 8.0, 5.0], [4.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], [3.0, 4.0, 7.0], [7.0, 3.0, 7.0]], 
[[6.0, 6.0, 1.0], [1.0, 5.0, 3.0], [8.0, 3.0, 1.0]]], [[[1.0, 5.0, 7.0], [2.0, 5.0, 7.0], [6.0, 3.0, 7.0]], [[3.0, 3.0, 2.0], 
[4.0, 7.0, 8.0], [7.0, 7.0, 6.0]], [[8.0, 6.0, 1.0], [4.0, 4.0, 7.0], [1.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_URRR(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 6.0], [1.0, 3.0, 6.0], [2.0, 2.0, 1.0]], [[5.0, 3.0, 5.0], [3.0, 3.0, 8.0], [5.0, 5.0, 
4.0]], [[8.0, 1.0, 3.0], [7.0, 1.0, 4.0], [3.0, 7.0, 1.0]]], [[[4.0, 5.0, 5.0], [5.0, 5.0, 5.0], [3.0, 4.0, 7.0]], [[8.0, 7.0, 
8.0], [6.0, 4.0, 3.0], [8.0, 5.0, 2.0]], [[1.0, 4.0, 6.0], [2.0, 4.0, 3.0], [1.0, 6.0, 8.0]]], [[[4.0, 7.0, 2.0], [8.0, 8.0, 
3.0], [1.0, 4.0, 6.0]], [[1.0, 5.0, 7.0], [7.0, 7.0, 7.0], [8.0, 4.0, 6.0]], [[8.0, 6.0, 5.0], [6.0, 8.0, 8.0], [2.0, 2.0, 
3.0]]], [[[8.0, 2.0, 5.0], [7.0, 3.0, 5.0], [4.0, 3.0, 2.0]], [[7.0, 3.0, 7.0], [2.0, 3.0, 6.0], [8.0, 3.0, 1.0]], [[3.0, 4.0, 
2.0], [6.0, 1.0, 4.0], [3.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 2.0], [2.0, 8.0, 7.0], [6.0, 6.0, 4.0]], [[3.0, 7.0, 1.0], [1.0, 3.0, 
2.0], [5.0, 6.0, 7.0]], [[2.0, 5.0, 5.0], [1.0, 3.0, 5.0], [3.0, 7.0, 1.0]]], [[[2.0, 3.0, 8.0], [4.0, 4.0, 5.0], [8.0, 7.0, 
3.0]], [[6.0, 4.0, 5.0], [2.0, 1.0, 2.0], [7.0, 5.0, 7.0]], [[6.0, 3.0, 1.0], [1.0, 5.0, 3.0], [6.0, 1.0, 4.0]]], [[[7.0, 2.0, 
3.0], [2.0, 4.0, 5.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 4.0], [3.0, 4.0, 1.0], [2.0, 6.0, 8.0]], [[7.0, 1.0, 8.0], [7.0, 2.0, 
1.0], [3.0, 3.0, 1.0]]], [[[3.0, 6.0, 8.0], [7.0, 2.0, 7.0], [6.0, 1.0, 5.0]], [[4.0, 3.0, 7.0], [4.0, 7.0, 8.0], [5.0, 2.0, 
6.0]], [[7.0, 7.0, 3.0], [4.0, 2.0, 1.0], [6.0, 6.0, 3.0]]]]))
      val=Data(numpy.array([[[[6.0]]], [[[6.0]]], [[[2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0]]], [[[7.0]]], [[[8.0]]]]))
      arg[:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 6.0], [1.0, 3.0, 6.0], [2.0, 2.0, 1.0]], [[5.0, 3.0, 5.0], [3.0, 6.0, 8.0], [5.0, 
5.0, 4.0]], [[8.0, 1.0, 3.0], [7.0, 1.0, 4.0], [3.0, 7.0, 1.0]]], [[[4.0, 5.0, 5.0], [5.0, 5.0, 5.0], [3.0, 4.0, 7.0]], [[8.0, 
7.0, 8.0], [6.0, 6.0, 3.0], [8.0, 5.0, 2.0]], [[1.0, 4.0, 6.0], [2.0, 4.0, 3.0], [1.0, 6.0, 8.0]]], [[[4.0, 7.0, 2.0], [8.0, 
8.0, 3.0], [1.0, 4.0, 6.0]], [[1.0, 5.0, 7.0], [7.0, 2.0, 7.0], [8.0, 4.0, 6.0]], [[8.0, 6.0, 5.0], [6.0, 8.0, 8.0], [2.0, 2.0, 
3.0]]], [[[8.0, 2.0, 5.0], [7.0, 3.0, 5.0], [4.0, 3.0, 2.0]], [[7.0, 3.0, 7.0], [2.0, 3.0, 6.0], [8.0, 3.0, 1.0]], [[3.0, 4.0, 
2.0], [6.0, 1.0, 4.0], [3.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 2.0], [2.0, 8.0, 7.0], [6.0, 6.0, 4.0]], 
[[3.0, 7.0, 1.0], [1.0, 7.0, 2.0], [5.0, 6.0, 7.0]], [[2.0, 5.0, 5.0], [1.0, 3.0, 5.0], [3.0, 7.0, 1.0]]], [[[2.0, 3.0, 8.0], 
[4.0, 4.0, 5.0], [8.0, 7.0, 3.0]], [[6.0, 4.0, 5.0], [2.0, 7.0, 2.0], [7.0, 5.0, 7.0]], [[6.0, 3.0, 1.0], [1.0, 5.0, 3.0], 
[6.0, 1.0, 4.0]]], [[[7.0, 2.0, 3.0], [2.0, 4.0, 5.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 4.0], [3.0, 8.0, 1.0], [2.0, 6.0, 8.0]], 
[[7.0, 1.0, 8.0], [7.0, 2.0, 1.0], [3.0, 3.0, 1.0]]], [[[3.0, 6.0, 8.0], [7.0, 2.0, 7.0], [6.0, 1.0, 5.0]], [[4.0, 3.0, 7.0], 
[4.0, 7.0, 8.0], [5.0, 2.0, 6.0]], [[7.0, 7.0, 3.0], [4.0, 2.0, 1.0], [6.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_URRR(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 4.0], [2.0, 8.0, 8.0], [1.0, 1.0, 1.0]], [[6.0, 4.0, 8.0], [4.0, 5.0, 4.0], [2.0, 8.0, 
7.0]], [[7.0, 8.0, 1.0], [7.0, 6.0, 8.0], [4.0, 4.0, 2.0]]], [[[8.0, 5.0, 8.0], [5.0, 4.0, 4.0], [6.0, 2.0, 8.0]], [[5.0, 7.0, 
8.0], [6.0, 5.0, 8.0], [6.0, 4.0, 4.0]], [[2.0, 8.0, 4.0], [7.0, 5.0, 6.0], [7.0, 8.0, 4.0]]], [[[1.0, 2.0, 4.0], [7.0, 5.0, 
5.0], [2.0, 3.0, 8.0]], [[1.0, 8.0, 4.0], [5.0, 1.0, 8.0], [3.0, 4.0, 5.0]], [[3.0, 1.0, 1.0], [1.0, 8.0, 5.0], [2.0, 6.0, 
2.0]]], [[[8.0, 4.0, 2.0], [4.0, 5.0, 5.0], [2.0, 4.0, 8.0]], [[7.0, 7.0, 8.0], [5.0, 2.0, 6.0], [4.0, 7.0, 7.0]], [[4.0, 7.0, 
7.0], [5.0, 6.0, 4.0], [4.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 3.0], [5.0, 2.0, 6.0], [3.0, 1.0, 1.0]], [[3.0, 1.0, 8.0], [8.0, 5.0, 
7.0], [4.0, 2.0, 7.0]], [[3.0, 2.0, 5.0], [6.0, 3.0, 1.0], [5.0, 3.0, 7.0]]], [[[4.0, 3.0, 1.0], [2.0, 8.0, 4.0], [5.0, 8.0, 
4.0]], [[1.0, 7.0, 7.0], [5.0, 6.0, 3.0], [6.0, 4.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 6.0, 6.0], [6.0, 1.0, 8.0]]], [[[7.0, 8.0, 
7.0], [3.0, 6.0, 4.0], [8.0, 1.0, 5.0]], [[3.0, 2.0, 7.0], [1.0, 3.0, 1.0], [5.0, 3.0, 3.0]], [[5.0, 2.0, 8.0], [7.0, 6.0, 
2.0], [1.0, 5.0, 8.0]]], [[[5.0, 2.0, 2.0], [8.0, 2.0, 7.0], [8.0, 5.0, 1.0]], [[1.0, 2.0, 1.0], [6.0, 8.0, 1.0], [5.0, 5.0, 
1.0]], [[3.0, 3.0, 8.0], [1.0, 4.0, 5.0], [2.0, 6.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0]]], [[[1.0]]], [[[2.0]]]])+(1.-msk_val)*numpy.array([[[[3.0]]], [[[3.0]]], [[[7.0]]]])
      arg[:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 4.0], [2.0, 8.0, 8.0], [1.0, 1.0, 1.0]], [[6.0, 4.0, 8.0], [4.0, 7.0, 4.0], [2.0, 
8.0, 7.0]], [[7.0, 8.0, 1.0], [7.0, 6.0, 8.0], [4.0, 4.0, 2.0]]], [[[8.0, 5.0, 8.0], [5.0, 4.0, 4.0], [6.0, 2.0, 8.0]], [[5.0, 
7.0, 8.0], [6.0, 1.0, 8.0], [6.0, 4.0, 4.0]], [[2.0, 8.0, 4.0], [7.0, 5.0, 6.0], [7.0, 8.0, 4.0]]], [[[1.0, 2.0, 4.0], [7.0, 
5.0, 5.0], [2.0, 3.0, 8.0]], [[1.0, 8.0, 4.0], [5.0, 2.0, 8.0], [3.0, 4.0, 5.0]], [[3.0, 1.0, 1.0], [1.0, 8.0, 5.0], [2.0, 6.0, 
2.0]]], [[[8.0, 4.0, 2.0], [4.0, 5.0, 5.0], [2.0, 4.0, 8.0]], [[7.0, 7.0, 8.0], [5.0, 2.0, 6.0], [4.0, 7.0, 7.0]], [[4.0, 7.0, 
7.0], [5.0, 6.0, 4.0], [4.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 3.0], [5.0, 2.0, 6.0], [3.0, 1.0, 1.0]], 
[[3.0, 1.0, 8.0], [8.0, 3.0, 7.0], [4.0, 2.0, 7.0]], [[3.0, 2.0, 5.0], [6.0, 3.0, 1.0], [5.0, 3.0, 7.0]]], [[[4.0, 3.0, 1.0], 
[2.0, 8.0, 4.0], [5.0, 8.0, 4.0]], [[1.0, 7.0, 7.0], [5.0, 3.0, 3.0], [6.0, 4.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 6.0, 6.0], 
[6.0, 1.0, 8.0]]], [[[7.0, 8.0, 7.0], [3.0, 6.0, 4.0], [8.0, 1.0, 5.0]], [[3.0, 2.0, 7.0], [1.0, 7.0, 1.0], [5.0, 3.0, 3.0]], 
[[5.0, 2.0, 8.0], [7.0, 6.0, 2.0], [1.0, 5.0, 8.0]]], [[[5.0, 2.0, 2.0], [8.0, 2.0, 7.0], [8.0, 5.0, 1.0]], [[1.0, 2.0, 1.0], 
[6.0, 8.0, 1.0], [5.0, 5.0, 1.0]], [[3.0, 3.0, 8.0], [1.0, 4.0, 5.0], [2.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLCN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 7.0], [5.0, 3.0, 1.0], [6.0, 8.0, 1.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 7.0], [5.0, 4.0, 
7.0]], [[7.0, 8.0, 6.0], [8.0, 5.0, 3.0], [4.0, 3.0, 1.0]]], [[[4.0, 6.0, 1.0], [5.0, 3.0, 5.0], [5.0, 2.0, 7.0]], [[2.0, 1.0, 
6.0], [7.0, 3.0, 5.0], [2.0, 5.0, 1.0]], [[6.0, 1.0, 6.0], [7.0, 2.0, 5.0], [1.0, 1.0, 4.0]]], [[[3.0, 7.0, 4.0], [5.0, 8.0, 
7.0], [5.0, 1.0, 2.0]], [[4.0, 8.0, 4.0], [4.0, 8.0, 4.0], [1.0, 4.0, 7.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 1.0], [2.0, 5.0, 
7.0]]], [[[6.0, 6.0, 2.0], [7.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[2.0, 4.0, 8.0], [4.0, 3.0, 7.0], [5.0, 7.0, 7.0]], [[7.0, 1.0, 
8.0], [8.0, 6.0, 4.0], [3.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 7.0], [3.0, 2.0, 4.0], [2.0, 3.0, 8.0]], [[4.0, 8.0, 4.0], [7.0, 3.0, 
7.0], [2.0, 1.0, 6.0]], [[8.0, 4.0, 3.0], [3.0, 2.0, 3.0], [8.0, 3.0, 1.0]]], [[[2.0, 8.0, 8.0], [7.0, 5.0, 6.0], [7.0, 3.0, 
4.0]], [[8.0, 7.0, 4.0], [6.0, 3.0, 6.0], [8.0, 4.0, 2.0]], [[6.0, 3.0, 5.0], [5.0, 6.0, 8.0], [2.0, 7.0, 7.0]]], [[[5.0, 4.0, 
6.0], [3.0, 2.0, 1.0], [4.0, 8.0, 8.0]], [[7.0, 8.0, 5.0], [6.0, 4.0, 3.0], [7.0, 5.0, 3.0]], [[1.0, 1.0, 6.0], [6.0, 4.0, 
7.0], [7.0, 5.0, 1.0]]], [[[4.0, 1.0, 7.0], [1.0, 1.0, 5.0], [1.0, 6.0, 4.0]], [[8.0, 2.0, 3.0], [7.0, 7.0, 2.0], [6.0, 3.0, 
5.0]], [[4.0, 3.0, 3.0], [8.0, 7.0, 4.0], [3.0, 3.0, 6.0]]]]))
      val=numpy.array([[[6.0, 6.0, 7.0], [3.0, 7.0, 2.0], [4.0, 3.0, 1.0]]])
      arg[2,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 7.0], [5.0, 3.0, 1.0], [6.0, 8.0, 1.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 7.0], [5.0, 
4.0, 7.0]], [[7.0, 8.0, 6.0], [8.0, 5.0, 3.0], [4.0, 3.0, 1.0]]], [[[4.0, 6.0, 1.0], [5.0, 3.0, 5.0], [5.0, 2.0, 7.0]], [[2.0, 
1.0, 6.0], [7.0, 3.0, 5.0], [2.0, 5.0, 1.0]], [[6.0, 1.0, 6.0], [7.0, 2.0, 5.0], [1.0, 1.0, 4.0]]], [[[3.0, 7.0, 4.0], [5.0, 
8.0, 7.0], [5.0, 1.0, 2.0]], [[4.0, 8.0, 4.0], [4.0, 8.0, 4.0], [1.0, 4.0, 7.0]], [[6.0, 6.0, 7.0], [3.0, 7.0, 2.0], [4.0, 3.0, 
1.0]]], [[[6.0, 6.0, 2.0], [7.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[2.0, 4.0, 8.0], [4.0, 3.0, 7.0], [5.0, 7.0, 7.0]], [[7.0, 1.0, 
8.0], [8.0, 6.0, 4.0], [3.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 7.0], [3.0, 2.0, 4.0], [2.0, 3.0, 8.0]], 
[[4.0, 8.0, 4.0], [7.0, 3.0, 7.0], [2.0, 1.0, 6.0]], [[8.0, 4.0, 3.0], [3.0, 2.0, 3.0], [8.0, 3.0, 1.0]]], [[[2.0, 8.0, 8.0], 
[7.0, 5.0, 6.0], [7.0, 3.0, 4.0]], [[8.0, 7.0, 4.0], [6.0, 3.0, 6.0], [8.0, 4.0, 2.0]], [[6.0, 3.0, 5.0], [5.0, 6.0, 8.0], 
[2.0, 7.0, 7.0]]], [[[5.0, 4.0, 6.0], [3.0, 2.0, 1.0], [4.0, 8.0, 8.0]], [[7.0, 8.0, 5.0], [6.0, 4.0, 3.0], [7.0, 5.0, 3.0]], 
[[6.0, 6.0, 7.0], [3.0, 7.0, 2.0], [4.0, 3.0, 1.0]]], [[[4.0, 1.0, 7.0], [1.0, 1.0, 5.0], [1.0, 6.0, 4.0]], [[8.0, 2.0, 3.0], 
[7.0, 7.0, 2.0], [6.0, 3.0, 5.0]], [[4.0, 3.0, 3.0], [8.0, 7.0, 4.0], [3.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLCN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 3.0], [4.0, 6.0, 1.0], [7.0, 4.0, 4.0]], [[1.0, 1.0, 5.0], [4.0, 7.0, 2.0], [5.0, 7.0, 
7.0]], [[8.0, 8.0, 1.0], [4.0, 2.0, 8.0], [8.0, 5.0, 5.0]]], [[[8.0, 6.0, 7.0], [4.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[3.0, 1.0, 
7.0], [2.0, 3.0, 8.0], [8.0, 5.0, 6.0]], [[4.0, 3.0, 4.0], [3.0, 8.0, 1.0], [5.0, 4.0, 3.0]]], [[[5.0, 1.0, 4.0], [2.0, 7.0, 
2.0], [1.0, 4.0, 4.0]], [[6.0, 1.0, 8.0], [3.0, 2.0, 4.0], [8.0, 5.0, 6.0]], [[4.0, 8.0, 7.0], [4.0, 3.0, 6.0], [8.0, 8.0, 
1.0]]], [[[5.0, 4.0, 4.0], [6.0, 8.0, 3.0], [2.0, 4.0, 6.0]], [[2.0, 2.0, 5.0], [3.0, 4.0, 6.0], [8.0, 5.0, 6.0]], [[6.0, 4.0, 
8.0], [7.0, 8.0, 6.0], [2.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 6.0], [4.0, 3.0, 5.0], [5.0, 5.0, 3.0]], [[6.0, 3.0, 6.0], [7.0, 2.0, 
7.0], [2.0, 1.0, 6.0]], [[8.0, 7.0, 6.0], [7.0, 5.0, 1.0], [6.0, 5.0, 4.0]]], [[[2.0, 2.0, 7.0], [5.0, 2.0, 4.0], [2.0, 2.0, 
4.0]], [[6.0, 4.0, 4.0], [5.0, 6.0, 4.0], [4.0, 6.0, 2.0]], [[2.0, 7.0, 8.0], [5.0, 2.0, 1.0], [7.0, 4.0, 5.0]]], [[[2.0, 5.0, 
1.0], [6.0, 5.0, 7.0], [6.0, 6.0, 8.0]], [[6.0, 8.0, 8.0], [4.0, 7.0, 2.0], [8.0, 2.0, 7.0]], [[5.0, 4.0, 4.0], [8.0, 4.0, 
5.0], [6.0, 5.0, 1.0]]], [[[4.0, 6.0, 3.0], [3.0, 4.0, 1.0], [8.0, 1.0, 7.0]], [[3.0, 1.0, 3.0], [7.0, 4.0, 5.0], [2.0, 8.0, 
6.0]], [[6.0, 8.0, 1.0], [2.0, 2.0, 8.0], [7.0, 6.0, 8.0]]]]))
      val=Data(numpy.array([[[6.0, 7.0, 2.0], [6.0, 5.0, 2.0], [7.0, 7.0, 7.0]]]),self.functionspace)
      arg[2,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 3.0], [4.0, 6.0, 1.0], [7.0, 4.0, 4.0]], [[1.0, 1.0, 5.0], [4.0, 7.0, 2.0], [5.0, 
7.0, 7.0]], [[8.0, 8.0, 1.0], [4.0, 2.0, 8.0], [8.0, 5.0, 5.0]]], [[[8.0, 6.0, 7.0], [4.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[3.0, 
1.0, 7.0], [2.0, 3.0, 8.0], [8.0, 5.0, 6.0]], [[4.0, 3.0, 4.0], [3.0, 8.0, 1.0], [5.0, 4.0, 3.0]]], [[[5.0, 1.0, 4.0], [2.0, 
7.0, 2.0], [1.0, 4.0, 4.0]], [[6.0, 1.0, 8.0], [3.0, 2.0, 4.0], [8.0, 5.0, 6.0]], [[6.0, 7.0, 2.0], [6.0, 5.0, 2.0], [7.0, 7.0, 
7.0]]], [[[5.0, 4.0, 4.0], [6.0, 8.0, 3.0], [2.0, 4.0, 6.0]], [[2.0, 2.0, 5.0], [3.0, 4.0, 6.0], [8.0, 5.0, 6.0]], [[6.0, 4.0, 
8.0], [7.0, 8.0, 6.0], [2.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 6.0], [4.0, 3.0, 5.0], [5.0, 5.0, 3.0]], 
[[6.0, 3.0, 6.0], [7.0, 2.0, 7.0], [2.0, 1.0, 6.0]], [[8.0, 7.0, 6.0], [7.0, 5.0, 1.0], [6.0, 5.0, 4.0]]], [[[2.0, 2.0, 7.0], 
[5.0, 2.0, 4.0], [2.0, 2.0, 4.0]], [[6.0, 4.0, 4.0], [5.0, 6.0, 4.0], [4.0, 6.0, 2.0]], [[2.0, 7.0, 8.0], [5.0, 2.0, 1.0], 
[7.0, 4.0, 5.0]]], [[[2.0, 5.0, 1.0], [6.0, 5.0, 7.0], [6.0, 6.0, 8.0]], [[6.0, 8.0, 8.0], [4.0, 7.0, 2.0], [8.0, 2.0, 7.0]], 
[[6.0, 7.0, 2.0], [6.0, 5.0, 2.0], [7.0, 7.0, 7.0]]], [[[4.0, 6.0, 3.0], [3.0, 4.0, 1.0], [8.0, 1.0, 7.0]], [[3.0, 1.0, 3.0], 
[7.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[6.0, 8.0, 1.0], [2.0, 2.0, 8.0], [7.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 2.0], [1.0, 7.0, 1.0], [1.0, 8.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 8.0, 7.0], [3.0, 8.0, 
3.0]], [[4.0, 5.0, 5.0], [3.0, 1.0, 7.0], [6.0, 3.0, 2.0]]], [[[7.0, 6.0, 3.0], [8.0, 1.0, 7.0], [1.0, 8.0, 8.0]], [[5.0, 1.0, 
6.0], [4.0, 6.0, 7.0], [2.0, 7.0, 5.0]], [[4.0, 4.0, 7.0], [8.0, 4.0, 6.0], [1.0, 5.0, 1.0]]], [[[3.0, 1.0, 4.0], [7.0, 1.0, 
4.0], [5.0, 2.0, 8.0]], [[2.0, 3.0, 1.0], [4.0, 3.0, 6.0], [1.0, 6.0, 7.0]], [[7.0, 8.0, 2.0], [5.0, 3.0, 4.0], [6.0, 2.0, 
3.0]]], [[[8.0, 3.0, 6.0], [5.0, 5.0, 4.0], [1.0, 5.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 5.0, 4.0], [8.0, 6.0, 5.0]], [[4.0, 1.0, 
5.0], [3.0, 4.0, 2.0], [8.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 5.0], [5.0, 2.0, 2.0], [3.0, 8.0, 2.0]], [[5.0, 2.0, 6.0], [3.0, 5.0, 
7.0], [2.0, 8.0, 4.0]], [[4.0, 2.0, 2.0], [1.0, 6.0, 4.0], [4.0, 8.0, 5.0]]], [[[1.0, 3.0, 1.0], [1.0, 7.0, 4.0], [7.0, 6.0, 
5.0]], [[8.0, 5.0, 3.0], [5.0, 4.0, 2.0], [1.0, 3.0, 2.0]], [[8.0, 3.0, 5.0], [4.0, 8.0, 8.0], [3.0, 3.0, 2.0]]], [[[6.0, 5.0, 
2.0], [5.0, 7.0, 6.0], [2.0, 4.0, 6.0]], [[8.0, 8.0, 3.0], [3.0, 4.0, 2.0], [7.0, 1.0, 1.0]], [[7.0, 3.0, 8.0], [2.0, 2.0, 
5.0], [6.0, 3.0, 6.0]]], [[[6.0, 8.0, 5.0], [6.0, 3.0, 5.0], [7.0, 3.0, 1.0]], [[5.0, 5.0, 1.0], [2.0, 3.0, 5.0], [6.0, 3.0, 
4.0]], [[2.0, 2.0, 5.0], [3.0, 1.0, 3.0], [6.0, 8.0, 6.0]]]]))
      val=Data(numpy.array([[[3.0, 7.0, 1.0], [5.0, 2.0, 3.0], [5.0, 4.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 2.0, 8.0], [6.0, 5.0, 3.0], [8.0, 6.0, 5.0]]]))
      arg[2,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 2.0], [1.0, 7.0, 1.0], [1.0, 8.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 8.0, 7.0], [3.0, 
8.0, 3.0]], [[4.0, 5.0, 5.0], [3.0, 1.0, 7.0], [6.0, 3.0, 2.0]]], [[[7.0, 6.0, 3.0], [8.0, 1.0, 7.0], [1.0, 8.0, 8.0]], [[5.0, 
1.0, 6.0], [4.0, 6.0, 7.0], [2.0, 7.0, 5.0]], [[4.0, 4.0, 7.0], [8.0, 4.0, 6.0], [1.0, 5.0, 1.0]]], [[[3.0, 1.0, 4.0], [7.0, 
1.0, 4.0], [5.0, 2.0, 8.0]], [[2.0, 3.0, 1.0], [4.0, 3.0, 6.0], [1.0, 6.0, 7.0]], [[3.0, 7.0, 1.0], [5.0, 2.0, 3.0], [5.0, 4.0, 
8.0]]], [[[8.0, 3.0, 6.0], [5.0, 5.0, 4.0], [1.0, 5.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 5.0, 4.0], [8.0, 6.0, 5.0]], [[4.0, 1.0, 
5.0], [3.0, 4.0, 2.0], [8.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 5.0], [5.0, 2.0, 2.0], [3.0, 8.0, 2.0]], 
[[5.0, 2.0, 6.0], [3.0, 5.0, 7.0], [2.0, 8.0, 4.0]], [[4.0, 2.0, 2.0], [1.0, 6.0, 4.0], [4.0, 8.0, 5.0]]], [[[1.0, 3.0, 1.0], 
[1.0, 7.0, 4.0], [7.0, 6.0, 5.0]], [[8.0, 5.0, 3.0], [5.0, 4.0, 2.0], [1.0, 3.0, 2.0]], [[8.0, 3.0, 5.0], [4.0, 8.0, 8.0], 
[3.0, 3.0, 2.0]]], [[[6.0, 5.0, 2.0], [5.0, 7.0, 6.0], [2.0, 4.0, 6.0]], [[8.0, 8.0, 3.0], [3.0, 4.0, 2.0], [7.0, 1.0, 1.0]], 
[[3.0, 2.0, 8.0], [6.0, 5.0, 3.0], [8.0, 6.0, 5.0]]], [[[6.0, 8.0, 5.0], [6.0, 3.0, 5.0], [7.0, 3.0, 1.0]], [[5.0, 5.0, 1.0], 
[2.0, 3.0, 5.0], [6.0, 3.0, 4.0]], [[2.0, 2.0, 5.0], [3.0, 1.0, 3.0], [6.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLCN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 7.0], [1.0, 3.0, 7.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [4.0, 2.0, 1.0], [3.0, 5.0, 
6.0]], [[7.0, 4.0, 2.0], [4.0, 7.0, 6.0], [3.0, 2.0, 2.0]]], [[[8.0, 6.0, 7.0], [1.0, 7.0, 2.0], [8.0, 3.0, 2.0]], [[7.0, 8.0, 
8.0], [6.0, 7.0, 4.0], [2.0, 4.0, 5.0]], [[4.0, 5.0, 2.0], [3.0, 3.0, 5.0], [6.0, 8.0, 5.0]]], [[[8.0, 5.0, 2.0], [8.0, 6.0, 
1.0], [8.0, 5.0, 8.0]], [[2.0, 6.0, 2.0], [6.0, 4.0, 7.0], [7.0, 7.0, 2.0]], [[3.0, 3.0, 6.0], [8.0, 8.0, 8.0], [8.0, 1.0, 
6.0]]], [[[1.0, 3.0, 2.0], [2.0, 1.0, 3.0], [4.0, 7.0, 4.0]], [[7.0, 1.0, 1.0], [8.0, 6.0, 8.0], [8.0, 3.0, 3.0]], [[1.0, 5.0, 
2.0], [1.0, 6.0, 1.0], [5.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 1.0], [1.0, 4.0, 2.0], [3.0, 7.0, 8.0]], [[2.0, 2.0, 7.0], [3.0, 3.0, 
1.0], [4.0, 7.0, 8.0]], [[5.0, 2.0, 5.0], [7.0, 1.0, 8.0], [5.0, 5.0, 5.0]]], [[[5.0, 4.0, 4.0], [6.0, 6.0, 4.0], [5.0, 3.0, 
1.0]], [[7.0, 3.0, 3.0], [3.0, 3.0, 8.0], [2.0, 3.0, 5.0]], [[8.0, 8.0, 5.0], [8.0, 3.0, 7.0], [5.0, 3.0, 7.0]]], [[[1.0, 7.0, 
1.0], [6.0, 6.0, 4.0], [8.0, 6.0, 2.0]], [[4.0, 6.0, 6.0], [6.0, 4.0, 6.0], [4.0, 4.0, 7.0]], [[8.0, 5.0, 8.0], [7.0, 4.0, 
1.0], [5.0, 4.0, 1.0]]], [[[6.0, 3.0, 6.0], [2.0, 7.0, 8.0], [3.0, 7.0, 6.0]], [[7.0, 6.0, 2.0], [8.0, 6.0, 4.0], [3.0, 6.0, 
1.0]], [[4.0, 3.0, 6.0], [1.0, 2.0, 6.0], [2.0, 7.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 5.0, 4.0], [7.0, 4.0, 4.0], [7.0, 3.0, 5.0]]])+(1.-msk_val)*numpy.array([[[5.0, 4.0, 
1.0], [7.0, 8.0, 4.0], [6.0, 2.0, 2.0]]])
      arg[2,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 7.0], [1.0, 3.0, 7.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [4.0, 2.0, 1.0], [3.0, 
5.0, 6.0]], [[7.0, 4.0, 2.0], [4.0, 7.0, 6.0], [3.0, 2.0, 2.0]]], [[[8.0, 6.0, 7.0], [1.0, 7.0, 2.0], [8.0, 3.0, 2.0]], [[7.0, 
8.0, 8.0], [6.0, 7.0, 4.0], [2.0, 4.0, 5.0]], [[4.0, 5.0, 2.0], [3.0, 3.0, 5.0], [6.0, 8.0, 5.0]]], [[[8.0, 5.0, 2.0], [8.0, 
6.0, 1.0], [8.0, 5.0, 8.0]], [[2.0, 6.0, 2.0], [6.0, 4.0, 7.0], [7.0, 7.0, 2.0]], [[3.0, 5.0, 4.0], [7.0, 4.0, 4.0], [7.0, 3.0, 
5.0]]], [[[1.0, 3.0, 2.0], [2.0, 1.0, 3.0], [4.0, 7.0, 4.0]], [[7.0, 1.0, 1.0], [8.0, 6.0, 8.0], [8.0, 3.0, 3.0]], [[1.0, 5.0, 
2.0], [1.0, 6.0, 1.0], [5.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 1.0], [1.0, 4.0, 2.0], [3.0, 7.0, 8.0]], 
[[2.0, 2.0, 7.0], [3.0, 3.0, 1.0], [4.0, 7.0, 8.0]], [[5.0, 2.0, 5.0], [7.0, 1.0, 8.0], [5.0, 5.0, 5.0]]], [[[5.0, 4.0, 4.0], 
[6.0, 6.0, 4.0], [5.0, 3.0, 1.0]], [[7.0, 3.0, 3.0], [3.0, 3.0, 8.0], [2.0, 3.0, 5.0]], [[8.0, 8.0, 5.0], [8.0, 3.0, 7.0], 
[5.0, 3.0, 7.0]]], [[[1.0, 7.0, 1.0], [6.0, 6.0, 4.0], [8.0, 6.0, 2.0]], [[4.0, 6.0, 6.0], [6.0, 4.0, 6.0], [4.0, 4.0, 7.0]], 
[[5.0, 4.0, 1.0], [7.0, 8.0, 4.0], [6.0, 2.0, 2.0]]], [[[6.0, 3.0, 6.0], [2.0, 7.0, 8.0], [3.0, 7.0, 6.0]], [[7.0, 6.0, 2.0], 
[8.0, 6.0, 4.0], [3.0, 6.0, 1.0]], [[4.0, 3.0, 6.0], [1.0, 2.0, 6.0], [2.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLPU(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 7.0], [4.0, 5.0, 1.0], [4.0, 2.0, 4.0]], [[3.0, 6.0, 2.0], [1.0, 3.0, 6.0], [5.0, 3.0, 
5.0]], [[8.0, 8.0, 3.0], [4.0, 5.0, 6.0], [8.0, 4.0, 7.0]]], [[[5.0, 1.0, 5.0], [6.0, 3.0, 3.0], [2.0, 6.0, 7.0]], [[8.0, 5.0, 
7.0], [3.0, 3.0, 8.0], [4.0, 1.0, 8.0]], [[4.0, 1.0, 6.0], [1.0, 4.0, 4.0], [3.0, 6.0, 8.0]]], [[[5.0, 7.0, 2.0], [3.0, 7.0, 
3.0], [3.0, 8.0, 7.0]], [[5.0, 3.0, 7.0], [6.0, 7.0, 5.0], [7.0, 1.0, 6.0]], [[4.0, 5.0, 5.0], [7.0, 6.0, 1.0], [1.0, 5.0, 
7.0]]], [[[1.0, 8.0, 4.0], [2.0, 1.0, 8.0], [4.0, 6.0, 6.0]], [[7.0, 1.0, 8.0], [8.0, 8.0, 7.0], [5.0, 2.0, 8.0]], [[1.0, 3.0, 
8.0], [4.0, 3.0, 7.0], [2.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 3.0], [2.0, 2.0, 6.0], [7.0, 8.0, 6.0]], [[5.0, 2.0, 6.0], [5.0, 8.0, 
2.0], [1.0, 4.0, 7.0]], [[1.0, 6.0, 4.0], [5.0, 1.0, 6.0], [6.0, 5.0, 8.0]]], [[[5.0, 8.0, 8.0], [2.0, 2.0, 8.0], [6.0, 2.0, 
2.0]], [[2.0, 8.0, 7.0], [6.0, 3.0, 1.0], [1.0, 6.0, 7.0]], [[3.0, 7.0, 7.0], [2.0, 3.0, 3.0], [8.0, 4.0, 4.0]]], [[[5.0, 6.0, 
3.0], [5.0, 7.0, 3.0], [1.0, 2.0, 8.0]], [[3.0, 6.0, 7.0], [6.0, 1.0, 3.0], [7.0, 8.0, 1.0]], [[2.0, 4.0, 1.0], [3.0, 6.0, 
6.0], [2.0, 6.0, 1.0]]], [[[4.0, 8.0, 6.0], [1.0, 1.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 3.0, 1.0], [3.0, 3.0, 4.0], [1.0, 2.0, 
1.0]], [[2.0, 6.0, 4.0], [8.0, 7.0, 5.0], [1.0, 4.0, 7.0]]]]))
      val=numpy.array([[1.0, 7.0]])
      arg[2,2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 7.0], [4.0, 5.0, 1.0], [4.0, 2.0, 4.0]], [[3.0, 6.0, 2.0], [1.0, 3.0, 6.0], [5.0, 
3.0, 5.0]], [[8.0, 8.0, 3.0], [4.0, 5.0, 6.0], [8.0, 4.0, 7.0]]], [[[5.0, 1.0, 5.0], [6.0, 3.0, 3.0], [2.0, 6.0, 7.0]], [[8.0, 
5.0, 7.0], [3.0, 3.0, 8.0], [4.0, 1.0, 8.0]], [[4.0, 1.0, 6.0], [1.0, 4.0, 4.0], [3.0, 6.0, 8.0]]], [[[5.0, 7.0, 2.0], [3.0, 
7.0, 3.0], [3.0, 8.0, 7.0]], [[5.0, 3.0, 7.0], [6.0, 7.0, 5.0], [7.0, 1.0, 6.0]], [[4.0, 5.0, 5.0], [1.0, 7.0, 1.0], [1.0, 5.0, 
7.0]]], [[[1.0, 8.0, 4.0], [2.0, 1.0, 8.0], [4.0, 6.0, 6.0]], [[7.0, 1.0, 8.0], [8.0, 8.0, 7.0], [5.0, 2.0, 8.0]], [[1.0, 3.0, 
8.0], [4.0, 3.0, 7.0], [2.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 3.0], [2.0, 2.0, 6.0], [7.0, 8.0, 6.0]], 
[[5.0, 2.0, 6.0], [5.0, 8.0, 2.0], [1.0, 4.0, 7.0]], [[1.0, 6.0, 4.0], [5.0, 1.0, 6.0], [6.0, 5.0, 8.0]]], [[[5.0, 8.0, 8.0], 
[2.0, 2.0, 8.0], [6.0, 2.0, 2.0]], [[2.0, 8.0, 7.0], [6.0, 3.0, 1.0], [1.0, 6.0, 7.0]], [[3.0, 7.0, 7.0], [2.0, 3.0, 3.0], 
[8.0, 4.0, 4.0]]], [[[5.0, 6.0, 3.0], [5.0, 7.0, 3.0], [1.0, 2.0, 8.0]], [[3.0, 6.0, 7.0], [6.0, 1.0, 3.0], [7.0, 8.0, 1.0]], 
[[2.0, 4.0, 1.0], [1.0, 7.0, 6.0], [2.0, 6.0, 1.0]]], [[[4.0, 8.0, 6.0], [1.0, 1.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 3.0, 1.0], 
[3.0, 3.0, 4.0], [1.0, 2.0, 1.0]], [[2.0, 6.0, 4.0], [8.0, 7.0, 5.0], [1.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLPU(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 3.0], [6.0, 8.0, 5.0], [5.0, 8.0, 8.0]], [[1.0, 7.0, 8.0], [7.0, 1.0, 3.0], [7.0, 8.0, 
5.0]], [[1.0, 7.0, 7.0], [3.0, 4.0, 2.0], [8.0, 5.0, 6.0]]], [[[8.0, 3.0, 1.0], [8.0, 5.0, 7.0], [8.0, 6.0, 8.0]], [[3.0, 8.0, 
6.0], [1.0, 4.0, 2.0], [3.0, 4.0, 4.0]], [[3.0, 6.0, 4.0], [7.0, 1.0, 1.0], [4.0, 5.0, 2.0]]], [[[6.0, 7.0, 8.0], [7.0, 4.0, 
8.0], [6.0, 8.0, 6.0]], [[7.0, 2.0, 8.0], [8.0, 8.0, 6.0], [3.0, 6.0, 8.0]], [[4.0, 5.0, 4.0], [5.0, 8.0, 8.0], [6.0, 4.0, 
3.0]]], [[[7.0, 8.0, 5.0], [4.0, 6.0, 5.0], [5.0, 2.0, 3.0]], [[2.0, 4.0, 3.0], [3.0, 3.0, 8.0], [6.0, 5.0, 3.0]], [[5.0, 4.0, 
7.0], [2.0, 3.0, 1.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 5.0], [2.0, 8.0, 5.0], [4.0, 2.0, 2.0]], [[7.0, 6.0, 2.0], [4.0, 8.0, 
5.0], [8.0, 3.0, 4.0]], [[4.0, 1.0, 7.0], [2.0, 2.0, 7.0], [2.0, 1.0, 2.0]]], [[[3.0, 2.0, 6.0], [5.0, 7.0, 6.0], [1.0, 4.0, 
7.0]], [[3.0, 2.0, 1.0], [1.0, 8.0, 4.0], [5.0, 6.0, 8.0]], [[8.0, 6.0, 1.0], [1.0, 7.0, 8.0], [1.0, 7.0, 6.0]]], [[[2.0, 7.0, 
4.0], [8.0, 5.0, 7.0], [7.0, 5.0, 8.0]], [[4.0, 6.0, 5.0], [1.0, 5.0, 1.0], [4.0, 2.0, 6.0]], [[6.0, 6.0, 5.0], [7.0, 7.0, 
7.0], [1.0, 6.0, 4.0]]], [[[6.0, 7.0, 3.0], [3.0, 4.0, 6.0], [2.0, 3.0, 3.0]], [[4.0, 7.0, 5.0], [6.0, 1.0, 7.0], [3.0, 2.0, 
5.0]], [[6.0, 4.0, 7.0], [6.0, 2.0, 6.0], [5.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([[3.0, 6.0]]),self.functionspace)
      arg[2,2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 3.0], [6.0, 8.0, 5.0], [5.0, 8.0, 8.0]], [[1.0, 7.0, 8.0], [7.0, 1.0, 3.0], [7.0, 
8.0, 5.0]], [[1.0, 7.0, 7.0], [3.0, 4.0, 2.0], [8.0, 5.0, 6.0]]], [[[8.0, 3.0, 1.0], [8.0, 5.0, 7.0], [8.0, 6.0, 8.0]], [[3.0, 
8.0, 6.0], [1.0, 4.0, 2.0], [3.0, 4.0, 4.0]], [[3.0, 6.0, 4.0], [7.0, 1.0, 1.0], [4.0, 5.0, 2.0]]], [[[6.0, 7.0, 8.0], [7.0, 
4.0, 8.0], [6.0, 8.0, 6.0]], [[7.0, 2.0, 8.0], [8.0, 8.0, 6.0], [3.0, 6.0, 8.0]], [[4.0, 5.0, 4.0], [3.0, 6.0, 8.0], [6.0, 4.0, 
3.0]]], [[[7.0, 8.0, 5.0], [4.0, 6.0, 5.0], [5.0, 2.0, 3.0]], [[2.0, 4.0, 3.0], [3.0, 3.0, 8.0], [6.0, 5.0, 3.0]], [[5.0, 4.0, 
7.0], [2.0, 3.0, 1.0], [1.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 5.0], [2.0, 8.0, 5.0], [4.0, 2.0, 2.0]], 
[[7.0, 6.0, 2.0], [4.0, 8.0, 5.0], [8.0, 3.0, 4.0]], [[4.0, 1.0, 7.0], [2.0, 2.0, 7.0], [2.0, 1.0, 2.0]]], [[[3.0, 2.0, 6.0], 
[5.0, 7.0, 6.0], [1.0, 4.0, 7.0]], [[3.0, 2.0, 1.0], [1.0, 8.0, 4.0], [5.0, 6.0, 8.0]], [[8.0, 6.0, 1.0], [1.0, 7.0, 8.0], 
[1.0, 7.0, 6.0]]], [[[2.0, 7.0, 4.0], [8.0, 5.0, 7.0], [7.0, 5.0, 8.0]], [[4.0, 6.0, 5.0], [1.0, 5.0, 1.0], [4.0, 2.0, 6.0]], 
[[6.0, 6.0, 5.0], [3.0, 6.0, 7.0], [1.0, 6.0, 4.0]]], [[[6.0, 7.0, 3.0], [3.0, 4.0, 6.0], [2.0, 3.0, 3.0]], [[4.0, 7.0, 5.0], 
[6.0, 1.0, 7.0], [3.0, 2.0, 5.0]], [[6.0, 4.0, 7.0], [6.0, 2.0, 6.0], [5.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLPU(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 3.0], [6.0, 6.0, 4.0], [5.0, 7.0, 2.0]], [[5.0, 5.0, 1.0], [6.0, 8.0, 8.0], [6.0, 3.0, 
1.0]], [[2.0, 5.0, 8.0], [7.0, 3.0, 7.0], [4.0, 1.0, 5.0]]], [[[3.0, 7.0, 1.0], [1.0, 4.0, 1.0], [2.0, 1.0, 3.0]], [[4.0, 8.0, 
6.0], [2.0, 5.0, 3.0], [5.0, 7.0, 5.0]], [[3.0, 7.0, 6.0], [3.0, 3.0, 1.0], [4.0, 5.0, 1.0]]], [[[2.0, 1.0, 3.0], [6.0, 8.0, 
5.0], [8.0, 4.0, 7.0]], [[5.0, 4.0, 3.0], [2.0, 2.0, 1.0], [1.0, 8.0, 5.0]], [[5.0, 8.0, 4.0], [5.0, 2.0, 1.0], [1.0, 5.0, 
7.0]]], [[[4.0, 1.0, 2.0], [3.0, 5.0, 4.0], [6.0, 2.0, 4.0]], [[7.0, 1.0, 5.0], [7.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[8.0, 5.0, 
4.0], [3.0, 6.0, 1.0], [5.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 3.0], [1.0, 7.0, 4.0], [5.0, 1.0, 8.0]], [[7.0, 1.0, 8.0], [2.0, 5.0, 
1.0], [7.0, 1.0, 4.0]], [[6.0, 3.0, 7.0], [8.0, 7.0, 1.0], [8.0, 5.0, 2.0]]], [[[2.0, 3.0, 2.0], [2.0, 4.0, 2.0], [8.0, 5.0, 
1.0]], [[6.0, 4.0, 5.0], [2.0, 1.0, 7.0], [7.0, 7.0, 6.0]], [[8.0, 8.0, 2.0], [5.0, 2.0, 3.0], [7.0, 1.0, 1.0]]], [[[3.0, 4.0, 
4.0], [1.0, 4.0, 8.0], [2.0, 4.0, 1.0]], [[5.0, 6.0, 4.0], [4.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[8.0, 1.0, 5.0], [1.0, 2.0, 
8.0], [4.0, 1.0, 7.0]]], [[[7.0, 7.0, 7.0], [2.0, 5.0, 4.0], [3.0, 6.0, 2.0]], [[1.0, 6.0, 4.0], [1.0, 6.0, 1.0], [7.0, 5.0, 
7.0]], [[4.0, 3.0, 7.0], [4.0, 3.0, 7.0], [2.0, 5.0, 8.0]]]]))
      val=Data(numpy.array([[5.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 8.0]]))
      arg[2,2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 3.0], [6.0, 6.0, 4.0], [5.0, 7.0, 2.0]], [[5.0, 5.0, 1.0], [6.0, 8.0, 8.0], [6.0, 
3.0, 1.0]], [[2.0, 5.0, 8.0], [7.0, 3.0, 7.0], [4.0, 1.0, 5.0]]], [[[3.0, 7.0, 1.0], [1.0, 4.0, 1.0], [2.0, 1.0, 3.0]], [[4.0, 
8.0, 6.0], [2.0, 5.0, 3.0], [5.0, 7.0, 5.0]], [[3.0, 7.0, 6.0], [3.0, 3.0, 1.0], [4.0, 5.0, 1.0]]], [[[2.0, 1.0, 3.0], [6.0, 
8.0, 5.0], [8.0, 4.0, 7.0]], [[5.0, 4.0, 3.0], [2.0, 2.0, 1.0], [1.0, 8.0, 5.0]], [[5.0, 8.0, 4.0], [5.0, 8.0, 1.0], [1.0, 5.0, 
7.0]]], [[[4.0, 1.0, 2.0], [3.0, 5.0, 4.0], [6.0, 2.0, 4.0]], [[7.0, 1.0, 5.0], [7.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[8.0, 5.0, 
4.0], [3.0, 6.0, 1.0], [5.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 3.0], [1.0, 7.0, 4.0], [5.0, 1.0, 8.0]], 
[[7.0, 1.0, 8.0], [2.0, 5.0, 1.0], [7.0, 1.0, 4.0]], [[6.0, 3.0, 7.0], [8.0, 7.0, 1.0], [8.0, 5.0, 2.0]]], [[[2.0, 3.0, 2.0], 
[2.0, 4.0, 2.0], [8.0, 5.0, 1.0]], [[6.0, 4.0, 5.0], [2.0, 1.0, 7.0], [7.0, 7.0, 6.0]], [[8.0, 8.0, 2.0], [5.0, 2.0, 3.0], 
[7.0, 1.0, 1.0]]], [[[3.0, 4.0, 4.0], [1.0, 4.0, 8.0], [2.0, 4.0, 1.0]], [[5.0, 6.0, 4.0], [4.0, 4.0, 5.0], [2.0, 8.0, 6.0]], 
[[8.0, 1.0, 5.0], [8.0, 8.0, 8.0], [4.0, 1.0, 7.0]]], [[[7.0, 7.0, 7.0], [2.0, 5.0, 4.0], [3.0, 6.0, 2.0]], [[1.0, 6.0, 4.0], 
[1.0, 6.0, 1.0], [7.0, 5.0, 7.0]], [[4.0, 3.0, 7.0], [4.0, 3.0, 7.0], [2.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLPU(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 4.0], [8.0, 3.0, 4.0], [3.0, 2.0, 5.0]], [[4.0, 4.0, 7.0], [2.0, 4.0, 8.0], [7.0, 1.0, 
8.0]], [[4.0, 3.0, 1.0], [6.0, 1.0, 1.0], [6.0, 7.0, 5.0]]], [[[4.0, 4.0, 3.0], [6.0, 8.0, 3.0], [7.0, 4.0, 5.0]], [[8.0, 7.0, 
4.0], [5.0, 7.0, 2.0], [6.0, 7.0, 3.0]], [[3.0, 5.0, 7.0], [1.0, 1.0, 2.0], [1.0, 1.0, 3.0]]], [[[4.0, 4.0, 7.0], [6.0, 5.0, 
8.0], [1.0, 5.0, 3.0]], [[5.0, 1.0, 2.0], [1.0, 4.0, 4.0], [4.0, 2.0, 4.0]], [[6.0, 7.0, 1.0], [7.0, 2.0, 5.0], [7.0, 2.0, 
7.0]]], [[[1.0, 5.0, 3.0], [2.0, 1.0, 3.0], [2.0, 2.0, 5.0]], [[3.0, 8.0, 2.0], [6.0, 5.0, 4.0], [6.0, 5.0, 7.0]], [[2.0, 8.0, 
4.0], [5.0, 1.0, 8.0], [5.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 1.0], [8.0, 2.0, 6.0], [3.0, 1.0, 6.0]], [[1.0, 3.0, 1.0], [7.0, 3.0, 
7.0], [6.0, 4.0, 5.0]], [[3.0, 2.0, 5.0], [7.0, 3.0, 3.0], [4.0, 8.0, 7.0]]], [[[8.0, 2.0, 5.0], [3.0, 1.0, 2.0], [4.0, 6.0, 
3.0]], [[8.0, 6.0, 3.0], [6.0, 7.0, 2.0], [4.0, 7.0, 2.0]], [[4.0, 1.0, 1.0], [5.0, 4.0, 7.0], [8.0, 7.0, 6.0]]], [[[6.0, 1.0, 
3.0], [6.0, 7.0, 2.0], [1.0, 1.0, 5.0]], [[5.0, 8.0, 2.0], [4.0, 6.0, 2.0], [5.0, 7.0, 5.0]], [[3.0, 7.0, 6.0], [7.0, 2.0, 
2.0], [4.0, 5.0, 1.0]]], [[[5.0, 7.0, 5.0], [5.0, 4.0, 7.0], [8.0, 6.0, 1.0]], [[1.0, 8.0, 1.0], [5.0, 3.0, 5.0], [5.0, 2.0, 
1.0]], [[3.0, 3.0, 6.0], [4.0, 6.0, 7.0], [1.0, 5.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 7.0]])+(1.-msk_val)*numpy.array([[1.0, 2.0]])
      arg[2,2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 4.0], [8.0, 3.0, 4.0], [3.0, 2.0, 5.0]], [[4.0, 4.0, 7.0], [2.0, 4.0, 8.0], [7.0, 
1.0, 8.0]], [[4.0, 3.0, 1.0], [6.0, 1.0, 1.0], [6.0, 7.0, 5.0]]], [[[4.0, 4.0, 3.0], [6.0, 8.0, 3.0], [7.0, 4.0, 5.0]], [[8.0, 
7.0, 4.0], [5.0, 7.0, 2.0], [6.0, 7.0, 3.0]], [[3.0, 5.0, 7.0], [1.0, 1.0, 2.0], [1.0, 1.0, 3.0]]], [[[4.0, 4.0, 7.0], [6.0, 
5.0, 8.0], [1.0, 5.0, 3.0]], [[5.0, 1.0, 2.0], [1.0, 4.0, 4.0], [4.0, 2.0, 4.0]], [[6.0, 7.0, 1.0], [6.0, 7.0, 5.0], [7.0, 2.0, 
7.0]]], [[[1.0, 5.0, 3.0], [2.0, 1.0, 3.0], [2.0, 2.0, 5.0]], [[3.0, 8.0, 2.0], [6.0, 5.0, 4.0], [6.0, 5.0, 7.0]], [[2.0, 8.0, 
4.0], [5.0, 1.0, 8.0], [5.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 1.0], [8.0, 2.0, 6.0], [3.0, 1.0, 6.0]], 
[[1.0, 3.0, 1.0], [7.0, 3.0, 7.0], [6.0, 4.0, 5.0]], [[3.0, 2.0, 5.0], [7.0, 3.0, 3.0], [4.0, 8.0, 7.0]]], [[[8.0, 2.0, 5.0], 
[3.0, 1.0, 2.0], [4.0, 6.0, 3.0]], [[8.0, 6.0, 3.0], [6.0, 7.0, 2.0], [4.0, 7.0, 2.0]], [[4.0, 1.0, 1.0], [5.0, 4.0, 7.0], 
[8.0, 7.0, 6.0]]], [[[6.0, 1.0, 3.0], [6.0, 7.0, 2.0], [1.0, 1.0, 5.0]], [[5.0, 8.0, 2.0], [4.0, 6.0, 2.0], [5.0, 7.0, 5.0]], 
[[3.0, 7.0, 6.0], [1.0, 2.0, 2.0], [4.0, 5.0, 1.0]]], [[[5.0, 7.0, 5.0], [5.0, 4.0, 7.0], [8.0, 6.0, 1.0]], [[1.0, 8.0, 1.0], 
[5.0, 3.0, 5.0], [5.0, 2.0, 1.0]], [[3.0, 3.0, 6.0], [4.0, 6.0, 7.0], [1.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRPP(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 1.0], [1.0, 1.0, 1.0], [8.0, 8.0, 2.0]], [[5.0, 8.0, 1.0], [3.0, 7.0, 7.0], [7.0, 6.0, 
1.0]], [[2.0, 1.0, 2.0], [1.0, 2.0, 2.0], [4.0, 1.0, 1.0]]], [[[6.0, 4.0, 3.0], [8.0, 8.0, 5.0], [3.0, 7.0, 7.0]], [[5.0, 7.0, 
7.0], [3.0, 8.0, 3.0], [8.0, 6.0, 6.0]], [[8.0, 4.0, 4.0], [1.0, 7.0, 5.0], [3.0, 2.0, 5.0]]], [[[4.0, 4.0, 7.0], [2.0, 3.0, 
6.0], [3.0, 6.0, 8.0]], [[2.0, 8.0, 4.0], [5.0, 8.0, 8.0], [7.0, 6.0, 4.0]], [[6.0, 6.0, 1.0], [3.0, 7.0, 4.0], [4.0, 5.0, 
4.0]]], [[[4.0, 7.0, 2.0], [3.0, 2.0, 7.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 2.0], [1.0, 3.0, 7.0], [7.0, 1.0, 1.0]], [[8.0, 4.0, 
2.0], [3.0, 7.0, 8.0], [7.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 7.0], [3.0, 3.0, 7.0], [5.0, 8.0, 4.0]], [[5.0, 6.0, 8.0], [1.0, 6.0, 
4.0], [7.0, 4.0, 2.0]], [[3.0, 4.0, 2.0], [7.0, 6.0, 1.0], [1.0, 2.0, 1.0]]], [[[5.0, 2.0, 5.0], [1.0, 4.0, 5.0], [2.0, 1.0, 
7.0]], [[3.0, 4.0, 2.0], [4.0, 6.0, 7.0], [8.0, 1.0, 6.0]], [[4.0, 6.0, 8.0], [6.0, 3.0, 5.0], [1.0, 1.0, 7.0]]], [[[4.0, 4.0, 
5.0], [7.0, 7.0, 6.0], [5.0, 2.0, 6.0]], [[7.0, 4.0, 5.0], [1.0, 6.0, 1.0], [5.0, 7.0, 7.0]], [[7.0, 1.0, 7.0], [5.0, 6.0, 
2.0], [1.0, 5.0, 6.0]]], [[[8.0, 4.0, 5.0], [4.0, 8.0, 7.0], [2.0, 3.0, 6.0]], [[6.0, 7.0, 8.0], [1.0, 7.0, 2.0], [5.0, 2.0, 
1.0]], [[5.0, 2.0, 4.0], [3.0, 6.0, 5.0], [5.0, 6.0, 1.0]]]]))
      val=numpy.array([[2.0], [1.0], [6.0], [1.0]])
      arg[:,1:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 1.0], [1.0, 1.0, 1.0], [8.0, 8.0, 2.0]], [[5.0, 8.0, 1.0], [3.0, 2.0, 7.0], [7.0, 
6.0, 1.0]], [[2.0, 1.0, 2.0], [1.0, 2.0, 2.0], [4.0, 1.0, 1.0]]], [[[6.0, 4.0, 3.0], [8.0, 8.0, 5.0], [3.0, 7.0, 7.0]], [[5.0, 
7.0, 7.0], [3.0, 1.0, 3.0], [8.0, 6.0, 6.0]], [[8.0, 4.0, 4.0], [1.0, 7.0, 5.0], [3.0, 2.0, 5.0]]], [[[4.0, 4.0, 7.0], [2.0, 
3.0, 6.0], [3.0, 6.0, 8.0]], [[2.0, 8.0, 4.0], [5.0, 6.0, 8.0], [7.0, 6.0, 4.0]], [[6.0, 6.0, 1.0], [3.0, 7.0, 4.0], [4.0, 5.0, 
4.0]]], [[[4.0, 7.0, 2.0], [3.0, 2.0, 7.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 2.0], [1.0, 1.0, 7.0], [7.0, 1.0, 1.0]], [[8.0, 4.0, 
2.0], [3.0, 7.0, 8.0], [7.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 7.0], [3.0, 3.0, 7.0], [5.0, 8.0, 4.0]], 
[[5.0, 6.0, 8.0], [1.0, 2.0, 4.0], [7.0, 4.0, 2.0]], [[3.0, 4.0, 2.0], [7.0, 6.0, 1.0], [1.0, 2.0, 1.0]]], [[[5.0, 2.0, 5.0], 
[1.0, 4.0, 5.0], [2.0, 1.0, 7.0]], [[3.0, 4.0, 2.0], [4.0, 1.0, 7.0], [8.0, 1.0, 6.0]], [[4.0, 6.0, 8.0], [6.0, 3.0, 5.0], 
[1.0, 1.0, 7.0]]], [[[4.0, 4.0, 5.0], [7.0, 7.0, 6.0], [5.0, 2.0, 6.0]], [[7.0, 4.0, 5.0], [1.0, 6.0, 1.0], [5.0, 7.0, 7.0]], 
[[7.0, 1.0, 7.0], [5.0, 6.0, 2.0], [1.0, 5.0, 6.0]]], [[[8.0, 4.0, 5.0], [4.0, 8.0, 7.0], [2.0, 3.0, 6.0]], [[6.0, 7.0, 8.0], 
[1.0, 1.0, 2.0], [5.0, 2.0, 1.0]], [[5.0, 2.0, 4.0], [3.0, 6.0, 5.0], [5.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRPP(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 1.0], [1.0, 2.0, 7.0], [6.0, 7.0, 6.0]], [[7.0, 1.0, 4.0], [2.0, 8.0, 2.0], [4.0, 8.0, 
3.0]], [[6.0, 8.0, 2.0], [1.0, 4.0, 1.0], [5.0, 1.0, 4.0]]], [[[5.0, 6.0, 3.0], [2.0, 8.0, 5.0], [3.0, 5.0, 3.0]], [[7.0, 2.0, 
4.0], [6.0, 2.0, 8.0], [1.0, 6.0, 8.0]], [[8.0, 8.0, 4.0], [3.0, 2.0, 4.0], [1.0, 8.0, 3.0]]], [[[6.0, 2.0, 1.0], [6.0, 7.0, 
8.0], [7.0, 7.0, 4.0]], [[2.0, 2.0, 7.0], [4.0, 8.0, 3.0], [6.0, 8.0, 3.0]], [[5.0, 2.0, 4.0], [5.0, 5.0, 6.0], [3.0, 4.0, 
7.0]]], [[[2.0, 7.0, 8.0], [2.0, 6.0, 1.0], [6.0, 8.0, 8.0]], [[8.0, 2.0, 1.0], [1.0, 5.0, 7.0], [5.0, 1.0, 3.0]], [[6.0, 8.0, 
8.0], [5.0, 6.0, 5.0], [3.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 6.0], [1.0, 7.0, 2.0], [1.0, 3.0, 8.0]], [[5.0, 1.0, 5.0], [2.0, 7.0, 
8.0], [1.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [2.0, 3.0, 4.0], [4.0, 5.0, 8.0]]], [[[8.0, 7.0, 2.0], [6.0, 6.0, 6.0], [7.0, 7.0, 
1.0]], [[6.0, 6.0, 2.0], [7.0, 4.0, 5.0], [8.0, 7.0, 1.0]], [[3.0, 1.0, 2.0], [4.0, 3.0, 1.0], [1.0, 7.0, 2.0]]], [[[4.0, 7.0, 
6.0], [5.0, 2.0, 2.0], [6.0, 5.0, 3.0]], [[1.0, 4.0, 2.0], [3.0, 8.0, 1.0], [1.0, 3.0, 8.0]], [[8.0, 4.0, 6.0], [4.0, 3.0, 
6.0], [5.0, 8.0, 2.0]]], [[[8.0, 6.0, 7.0], [2.0, 7.0, 4.0], [2.0, 6.0, 5.0]], [[5.0, 6.0, 8.0], [2.0, 4.0, 3.0], [4.0, 2.0, 
1.0]], [[6.0, 3.0, 6.0], [1.0, 8.0, 3.0], [8.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[7.0], [5.0], [1.0], [5.0]]),self.functionspace)
      arg[:,1:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 1.0], [1.0, 2.0, 7.0], [6.0, 7.0, 6.0]], [[7.0, 1.0, 4.0], [2.0, 7.0, 2.0], [4.0, 
8.0, 3.0]], [[6.0, 8.0, 2.0], [1.0, 4.0, 1.0], [5.0, 1.0, 4.0]]], [[[5.0, 6.0, 3.0], [2.0, 8.0, 5.0], [3.0, 5.0, 3.0]], [[7.0, 
2.0, 4.0], [6.0, 5.0, 8.0], [1.0, 6.0, 8.0]], [[8.0, 8.0, 4.0], [3.0, 2.0, 4.0], [1.0, 8.0, 3.0]]], [[[6.0, 2.0, 1.0], [6.0, 
7.0, 8.0], [7.0, 7.0, 4.0]], [[2.0, 2.0, 7.0], [4.0, 1.0, 3.0], [6.0, 8.0, 3.0]], [[5.0, 2.0, 4.0], [5.0, 5.0, 6.0], [3.0, 4.0, 
7.0]]], [[[2.0, 7.0, 8.0], [2.0, 6.0, 1.0], [6.0, 8.0, 8.0]], [[8.0, 2.0, 1.0], [1.0, 5.0, 7.0], [5.0, 1.0, 3.0]], [[6.0, 8.0, 
8.0], [5.0, 6.0, 5.0], [3.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 6.0], [1.0, 7.0, 2.0], [1.0, 3.0, 8.0]], 
[[5.0, 1.0, 5.0], [2.0, 7.0, 8.0], [1.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [2.0, 3.0, 4.0], [4.0, 5.0, 8.0]]], [[[8.0, 7.0, 2.0], 
[6.0, 6.0, 6.0], [7.0, 7.0, 1.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 5.0], [8.0, 7.0, 1.0]], [[3.0, 1.0, 2.0], [4.0, 3.0, 1.0], 
[1.0, 7.0, 2.0]]], [[[4.0, 7.0, 6.0], [5.0, 2.0, 2.0], [6.0, 5.0, 3.0]], [[1.0, 4.0, 2.0], [3.0, 1.0, 1.0], [1.0, 3.0, 8.0]], 
[[8.0, 4.0, 6.0], [4.0, 3.0, 6.0], [5.0, 8.0, 2.0]]], [[[8.0, 6.0, 7.0], [2.0, 7.0, 4.0], [2.0, 6.0, 5.0]], [[5.0, 6.0, 8.0], 
[2.0, 5.0, 3.0], [4.0, 2.0, 1.0]], [[6.0, 3.0, 6.0], [1.0, 8.0, 3.0], [8.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRPP(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 6.0], [2.0, 7.0, 3.0], [3.0, 6.0, 5.0]], [[8.0, 4.0, 1.0], [3.0, 3.0, 4.0], [3.0, 7.0, 
6.0]], [[8.0, 3.0, 2.0], [4.0, 4.0, 4.0], [2.0, 7.0, 5.0]]], [[[7.0, 3.0, 5.0], [3.0, 1.0, 8.0], [7.0, 8.0, 3.0]], [[4.0, 2.0, 
8.0], [5.0, 1.0, 3.0], [5.0, 5.0, 7.0]], [[1.0, 4.0, 8.0], [8.0, 4.0, 7.0], [3.0, 4.0, 4.0]]], [[[7.0, 4.0, 4.0], [6.0, 3.0, 
3.0], [4.0, 6.0, 8.0]], [[1.0, 6.0, 1.0], [8.0, 3.0, 2.0], [6.0, 6.0, 8.0]], [[5.0, 4.0, 5.0], [2.0, 3.0, 4.0], [3.0, 5.0, 
5.0]]], [[[3.0, 3.0, 2.0], [7.0, 5.0, 2.0], [5.0, 3.0, 6.0]], [[3.0, 5.0, 4.0], [5.0, 6.0, 8.0], [5.0, 1.0, 1.0]], [[6.0, 4.0, 
1.0], [3.0, 5.0, 6.0], [8.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 8.0], [4.0, 3.0, 8.0], [5.0, 8.0, 6.0]], [[4.0, 2.0, 8.0], [2.0, 3.0, 
5.0], [1.0, 7.0, 7.0]], [[4.0, 6.0, 6.0], [3.0, 1.0, 5.0], [3.0, 3.0, 2.0]]], [[[4.0, 4.0, 4.0], [2.0, 5.0, 1.0], [3.0, 3.0, 
4.0]], [[7.0, 6.0, 2.0], [8.0, 3.0, 4.0], [4.0, 5.0, 6.0]], [[2.0, 1.0, 3.0], [4.0, 8.0, 8.0], [7.0, 3.0, 2.0]]], [[[5.0, 1.0, 
3.0], [2.0, 7.0, 6.0], [4.0, 5.0, 6.0]], [[5.0, 5.0, 2.0], [6.0, 8.0, 5.0], [4.0, 8.0, 3.0]], [[8.0, 4.0, 2.0], [8.0, 1.0, 
2.0], [6.0, 1.0, 7.0]]], [[[1.0, 2.0, 1.0], [2.0, 7.0, 2.0], [2.0, 3.0, 7.0]], [[6.0, 5.0, 3.0], [7.0, 1.0, 3.0], [5.0, 3.0, 
1.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 1.0], [7.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[7.0], [4.0], [4.0], [7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0], [4.0], [8.0], [6.0]]))
      arg[:,1:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 6.0], [2.0, 7.0, 3.0], [3.0, 6.0, 5.0]], [[8.0, 4.0, 1.0], [3.0, 7.0, 4.0], [3.0, 
7.0, 6.0]], [[8.0, 3.0, 2.0], [4.0, 4.0, 4.0], [2.0, 7.0, 5.0]]], [[[7.0, 3.0, 5.0], [3.0, 1.0, 8.0], [7.0, 8.0, 3.0]], [[4.0, 
2.0, 8.0], [5.0, 4.0, 3.0], [5.0, 5.0, 7.0]], [[1.0, 4.0, 8.0], [8.0, 4.0, 7.0], [3.0, 4.0, 4.0]]], [[[7.0, 4.0, 4.0], [6.0, 
3.0, 3.0], [4.0, 6.0, 8.0]], [[1.0, 6.0, 1.0], [8.0, 4.0, 2.0], [6.0, 6.0, 8.0]], [[5.0, 4.0, 5.0], [2.0, 3.0, 4.0], [3.0, 5.0, 
5.0]]], [[[3.0, 3.0, 2.0], [7.0, 5.0, 2.0], [5.0, 3.0, 6.0]], [[3.0, 5.0, 4.0], [5.0, 7.0, 8.0], [5.0, 1.0, 1.0]], [[6.0, 4.0, 
1.0], [3.0, 5.0, 6.0], [8.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 8.0], [4.0, 3.0, 8.0], [5.0, 8.0, 6.0]], 
[[4.0, 2.0, 8.0], [2.0, 3.0, 5.0], [1.0, 7.0, 7.0]], [[4.0, 6.0, 6.0], [3.0, 1.0, 5.0], [3.0, 3.0, 2.0]]], [[[4.0, 4.0, 4.0], 
[2.0, 5.0, 1.0], [3.0, 3.0, 4.0]], [[7.0, 6.0, 2.0], [8.0, 4.0, 4.0], [4.0, 5.0, 6.0]], [[2.0, 1.0, 3.0], [4.0, 8.0, 8.0], 
[7.0, 3.0, 2.0]]], [[[5.0, 1.0, 3.0], [2.0, 7.0, 6.0], [4.0, 5.0, 6.0]], [[5.0, 5.0, 2.0], [6.0, 8.0, 5.0], [4.0, 8.0, 3.0]], 
[[8.0, 4.0, 2.0], [8.0, 1.0, 2.0], [6.0, 1.0, 7.0]]], [[[1.0, 2.0, 1.0], [2.0, 7.0, 2.0], [2.0, 3.0, 7.0]], [[6.0, 5.0, 3.0], 
[7.0, 6.0, 3.0], [5.0, 3.0, 1.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 1.0], [7.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRPP(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 6.0], [5.0, 3.0, 3.0], [1.0, 6.0, 2.0]], [[1.0, 6.0, 2.0], [7.0, 4.0, 1.0], [1.0, 1.0, 
3.0]], [[3.0, 4.0, 4.0], [3.0, 5.0, 5.0], [5.0, 1.0, 1.0]]], [[[8.0, 7.0, 6.0], [5.0, 3.0, 1.0], [4.0, 1.0, 3.0]], [[8.0, 8.0, 
1.0], [1.0, 7.0, 6.0], [2.0, 6.0, 6.0]], [[5.0, 3.0, 1.0], [1.0, 6.0, 8.0], [3.0, 5.0, 5.0]]], [[[7.0, 4.0, 7.0], [2.0, 1.0, 
6.0], [1.0, 3.0, 8.0]], [[6.0, 1.0, 1.0], [2.0, 7.0, 3.0], [4.0, 3.0, 3.0]], [[4.0, 7.0, 7.0], [1.0, 6.0, 1.0], [7.0, 7.0, 
1.0]]], [[[6.0, 7.0, 1.0], [8.0, 5.0, 7.0], [6.0, 2.0, 2.0]], [[8.0, 3.0, 5.0], [8.0, 8.0, 6.0], [3.0, 3.0, 5.0]], [[6.0, 4.0, 
3.0], [7.0, 4.0, 5.0], [4.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 8.0], [4.0, 3.0, 7.0], [7.0, 7.0, 1.0]], [[2.0, 2.0, 7.0], [3.0, 2.0, 
5.0], [7.0, 2.0, 5.0]], [[1.0, 2.0, 4.0], [7.0, 2.0, 7.0], [5.0, 2.0, 7.0]]], [[[7.0, 5.0, 1.0], [8.0, 6.0, 4.0], [7.0, 5.0, 
6.0]], [[2.0, 4.0, 5.0], [4.0, 1.0, 3.0], [8.0, 7.0, 7.0]], [[1.0, 1.0, 7.0], [8.0, 7.0, 3.0], [7.0, 8.0, 6.0]]], [[[4.0, 2.0, 
8.0], [8.0, 7.0, 6.0], [8.0, 1.0, 6.0]], [[2.0, 8.0, 2.0], [3.0, 1.0, 2.0], [6.0, 8.0, 4.0]], [[7.0, 8.0, 4.0], [7.0, 7.0, 
8.0], [5.0, 4.0, 4.0]]], [[[3.0, 3.0, 1.0], [1.0, 7.0, 4.0], [1.0, 8.0, 8.0]], [[5.0, 5.0, 8.0], [1.0, 4.0, 7.0], [1.0, 4.0, 
5.0]], [[6.0, 5.0, 8.0], [4.0, 3.0, 5.0], [2.0, 6.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[2.0], [3.0], [5.0], [3.0]])+(1.-msk_val)*numpy.array([[6.0], [6.0], [5.0], [1.0]])
      arg[:,1:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 6.0], [5.0, 3.0, 3.0], [1.0, 6.0, 2.0]], [[1.0, 6.0, 2.0], [7.0, 2.0, 1.0], [1.0, 
1.0, 3.0]], [[3.0, 4.0, 4.0], [3.0, 5.0, 5.0], [5.0, 1.0, 1.0]]], [[[8.0, 7.0, 6.0], [5.0, 3.0, 1.0], [4.0, 1.0, 3.0]], [[8.0, 
8.0, 1.0], [1.0, 3.0, 6.0], [2.0, 6.0, 6.0]], [[5.0, 3.0, 1.0], [1.0, 6.0, 8.0], [3.0, 5.0, 5.0]]], [[[7.0, 4.0, 7.0], [2.0, 
1.0, 6.0], [1.0, 3.0, 8.0]], [[6.0, 1.0, 1.0], [2.0, 5.0, 3.0], [4.0, 3.0, 3.0]], [[4.0, 7.0, 7.0], [1.0, 6.0, 1.0], [7.0, 7.0, 
1.0]]], [[[6.0, 7.0, 1.0], [8.0, 5.0, 7.0], [6.0, 2.0, 2.0]], [[8.0, 3.0, 5.0], [8.0, 3.0, 6.0], [3.0, 3.0, 5.0]], [[6.0, 4.0, 
3.0], [7.0, 4.0, 5.0], [4.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 8.0], [4.0, 3.0, 7.0], [7.0, 7.0, 1.0]], 
[[2.0, 2.0, 7.0], [3.0, 6.0, 5.0], [7.0, 2.0, 5.0]], [[1.0, 2.0, 4.0], [7.0, 2.0, 7.0], [5.0, 2.0, 7.0]]], [[[7.0, 5.0, 1.0], 
[8.0, 6.0, 4.0], [7.0, 5.0, 6.0]], [[2.0, 4.0, 5.0], [4.0, 6.0, 3.0], [8.0, 7.0, 7.0]], [[1.0, 1.0, 7.0], [8.0, 7.0, 3.0], 
[7.0, 8.0, 6.0]]], [[[4.0, 2.0, 8.0], [8.0, 7.0, 6.0], [8.0, 1.0, 6.0]], [[2.0, 8.0, 2.0], [3.0, 5.0, 2.0], [6.0, 8.0, 4.0]], 
[[7.0, 8.0, 4.0], [7.0, 7.0, 8.0], [5.0, 4.0, 4.0]]], [[[3.0, 3.0, 1.0], [1.0, 7.0, 4.0], [1.0, 8.0, 8.0]], [[5.0, 5.0, 8.0], 
[1.0, 1.0, 7.0], [1.0, 4.0, 5.0]], [[6.0, 5.0, 8.0], [4.0, 3.0, 5.0], [2.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PCCN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 7.0], [7.0, 7.0, 2.0], [4.0, 4.0, 3.0]], [[2.0, 1.0, 6.0], [4.0, 6.0, 2.0], [6.0, 7.0, 
4.0]], [[1.0, 1.0, 7.0], [7.0, 5.0, 7.0], [3.0, 8.0, 5.0]]], [[[7.0, 1.0, 6.0], [7.0, 7.0, 4.0], [7.0, 8.0, 6.0]], [[5.0, 5.0, 
8.0], [7.0, 3.0, 6.0], [3.0, 6.0, 3.0]], [[4.0, 7.0, 5.0], [6.0, 1.0, 5.0], [5.0, 5.0, 7.0]]], [[[3.0, 2.0, 2.0], [5.0, 5.0, 
2.0], [8.0, 1.0, 5.0]], [[4.0, 4.0, 4.0], [2.0, 1.0, 3.0], [1.0, 1.0, 1.0]], [[5.0, 6.0, 1.0], [5.0, 6.0, 3.0], [3.0, 6.0, 
5.0]]], [[[1.0, 3.0, 7.0], [3.0, 8.0, 5.0], [5.0, 7.0, 7.0]], [[6.0, 2.0, 4.0], [8.0, 6.0, 3.0], [5.0, 7.0, 2.0]], [[4.0, 5.0, 
3.0], [5.0, 3.0, 3.0], [7.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 6.0], [6.0, 1.0, 8.0], [5.0, 8.0, 6.0]], [[4.0, 3.0, 2.0], [3.0, 8.0, 
6.0], [3.0, 7.0, 1.0]], [[6.0, 5.0, 3.0], [2.0, 4.0, 7.0], [2.0, 2.0, 6.0]]], [[[5.0, 6.0, 1.0], [6.0, 1.0, 2.0], [6.0, 1.0, 
1.0]], [[5.0, 4.0, 7.0], [1.0, 2.0, 5.0], [8.0, 8.0, 6.0]], [[5.0, 3.0, 2.0], [5.0, 8.0, 3.0], [2.0, 4.0, 4.0]]], [[[5.0, 4.0, 
1.0], [1.0, 1.0, 6.0], [5.0, 3.0, 6.0]], [[5.0, 4.0, 4.0], [4.0, 1.0, 7.0], [6.0, 6.0, 1.0]], [[6.0, 6.0, 4.0], [6.0, 6.0, 
5.0], [2.0, 6.0, 8.0]]], [[[2.0, 1.0, 8.0], [7.0, 1.0, 7.0], [3.0, 7.0, 8.0]], [[7.0, 8.0, 7.0], [2.0, 3.0, 7.0], [3.0, 6.0, 
7.0]], [[3.0, 5.0, 6.0], [5.0, 8.0, 4.0], [5.0, 8.0, 6.0]]]]))
      val=numpy.array([[[3.0, 2.0, 7.0], [2.0, 4.0, 6.0], [7.0, 1.0, 6.0]], [[5.0, 6.0, 2.0], [6.0, 6.0, 8.0], [4.0, 8.0, 
8.0]], [[4.0, 1.0, 2.0], [2.0, 6.0, 3.0], [3.0, 7.0, 4.0]]])
      arg[2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 7.0], [7.0, 7.0, 2.0], [4.0, 4.0, 3.0]], [[2.0, 1.0, 6.0], [4.0, 6.0, 2.0], [6.0, 
7.0, 4.0]], [[1.0, 1.0, 7.0], [7.0, 5.0, 7.0], [3.0, 8.0, 5.0]]], [[[7.0, 1.0, 6.0], [7.0, 7.0, 4.0], [7.0, 8.0, 6.0]], [[5.0, 
5.0, 8.0], [7.0, 3.0, 6.0], [3.0, 6.0, 3.0]], [[4.0, 7.0, 5.0], [6.0, 1.0, 5.0], [5.0, 5.0, 7.0]]], [[[3.0, 2.0, 7.0], [2.0, 
4.0, 6.0], [7.0, 1.0, 6.0]], [[5.0, 6.0, 2.0], [6.0, 6.0, 8.0], [4.0, 8.0, 8.0]], [[4.0, 1.0, 2.0], [2.0, 6.0, 3.0], [3.0, 7.0, 
4.0]]], [[[1.0, 3.0, 7.0], [3.0, 8.0, 5.0], [5.0, 7.0, 7.0]], [[6.0, 2.0, 4.0], [8.0, 6.0, 3.0], [5.0, 7.0, 2.0]], [[4.0, 5.0, 
3.0], [5.0, 3.0, 3.0], [7.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 6.0], [6.0, 1.0, 8.0], [5.0, 8.0, 6.0]], 
[[4.0, 3.0, 2.0], [3.0, 8.0, 6.0], [3.0, 7.0, 1.0]], [[6.0, 5.0, 3.0], [2.0, 4.0, 7.0], [2.0, 2.0, 6.0]]], [[[5.0, 6.0, 1.0], 
[6.0, 1.0, 2.0], [6.0, 1.0, 1.0]], [[5.0, 4.0, 7.0], [1.0, 2.0, 5.0], [8.0, 8.0, 6.0]], [[5.0, 3.0, 2.0], [5.0, 8.0, 3.0], 
[2.0, 4.0, 4.0]]], [[[3.0, 2.0, 7.0], [2.0, 4.0, 6.0], [7.0, 1.0, 6.0]], [[5.0, 6.0, 2.0], [6.0, 6.0, 8.0], [4.0, 8.0, 8.0]], 
[[4.0, 1.0, 2.0], [2.0, 6.0, 3.0], [3.0, 7.0, 4.0]]], [[[2.0, 1.0, 8.0], [7.0, 1.0, 7.0], [3.0, 7.0, 8.0]], [[7.0, 8.0, 7.0], 
[2.0, 3.0, 7.0], [3.0, 6.0, 7.0]], [[3.0, 5.0, 6.0], [5.0, 8.0, 4.0], [5.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PCCN(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 7.0], [8.0, 4.0, 3.0], [4.0, 5.0, 1.0]], [[1.0, 4.0, 3.0], [1.0, 7.0, 3.0], [3.0, 4.0, 
2.0]], [[3.0, 3.0, 4.0], [5.0, 5.0, 4.0], [3.0, 8.0, 6.0]]], [[[6.0, 8.0, 1.0], [7.0, 1.0, 5.0], [1.0, 6.0, 2.0]], [[7.0, 1.0, 
1.0], [5.0, 8.0, 2.0], [2.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [6.0, 2.0, 2.0], [4.0, 1.0, 8.0]]], [[[4.0, 8.0, 1.0], [3.0, 1.0, 
1.0], [2.0, 1.0, 5.0]], [[5.0, 6.0, 6.0], [8.0, 1.0, 6.0], [2.0, 8.0, 2.0]], [[2.0, 7.0, 6.0], [3.0, 4.0, 7.0], [2.0, 6.0, 
5.0]]], [[[2.0, 6.0, 6.0], [8.0, 5.0, 4.0], [8.0, 8.0, 4.0]], [[3.0, 8.0, 5.0], [8.0, 7.0, 8.0], [6.0, 4.0, 5.0]], [[3.0, 8.0, 
4.0], [4.0, 1.0, 4.0], [1.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 7.0], [7.0, 4.0, 4.0], [7.0, 3.0, 3.0]], [[4.0, 6.0, 2.0], [8.0, 7.0, 
8.0], [5.0, 4.0, 2.0]], [[1.0, 1.0, 8.0], [4.0, 5.0, 2.0], [6.0, 2.0, 6.0]]], [[[8.0, 5.0, 1.0], [8.0, 7.0, 7.0], [7.0, 7.0, 
3.0]], [[7.0, 2.0, 3.0], [2.0, 1.0, 8.0], [5.0, 2.0, 1.0]], [[8.0, 7.0, 8.0], [5.0, 5.0, 4.0], [7.0, 8.0, 5.0]]], [[[2.0, 1.0, 
5.0], [2.0, 1.0, 6.0], [1.0, 7.0, 1.0]], [[6.0, 4.0, 6.0], [6.0, 7.0, 5.0], [5.0, 2.0, 1.0]], [[1.0, 8.0, 7.0], [8.0, 2.0, 
7.0], [4.0, 8.0, 5.0]]], [[[6.0, 3.0, 1.0], [7.0, 4.0, 4.0], [5.0, 8.0, 8.0]], [[2.0, 8.0, 6.0], [7.0, 1.0, 5.0], [5.0, 5.0, 
6.0]], [[6.0, 1.0, 4.0], [8.0, 8.0, 2.0], [8.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[[7.0, 8.0, 4.0], [7.0, 2.0, 6.0], [7.0, 2.0, 4.0]], [[1.0, 1.0, 5.0], [4.0, 3.0, 7.0], [5.0, 7.0, 
4.0]], [[4.0, 5.0, 7.0], [6.0, 2.0, 6.0], [4.0, 3.0, 7.0]]]),self.functionspace)
      arg[2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 7.0], [8.0, 4.0, 3.0], [4.0, 5.0, 1.0]], [[1.0, 4.0, 3.0], [1.0, 7.0, 3.0], [3.0, 
4.0, 2.0]], [[3.0, 3.0, 4.0], [5.0, 5.0, 4.0], [3.0, 8.0, 6.0]]], [[[6.0, 8.0, 1.0], [7.0, 1.0, 5.0], [1.0, 6.0, 2.0]], [[7.0, 
1.0, 1.0], [5.0, 8.0, 2.0], [2.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [6.0, 2.0, 2.0], [4.0, 1.0, 8.0]]], [[[7.0, 8.0, 4.0], [7.0, 
2.0, 6.0], [7.0, 2.0, 4.0]], [[1.0, 1.0, 5.0], [4.0, 3.0, 7.0], [5.0, 7.0, 4.0]], [[4.0, 5.0, 7.0], [6.0, 2.0, 6.0], [4.0, 3.0, 
7.0]]], [[[2.0, 6.0, 6.0], [8.0, 5.0, 4.0], [8.0, 8.0, 4.0]], [[3.0, 8.0, 5.0], [8.0, 7.0, 8.0], [6.0, 4.0, 5.0]], [[3.0, 8.0, 
4.0], [4.0, 1.0, 4.0], [1.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 7.0], [7.0, 4.0, 4.0], [7.0, 3.0, 3.0]], 
[[4.0, 6.0, 2.0], [8.0, 7.0, 8.0], [5.0, 4.0, 2.0]], [[1.0, 1.0, 8.0], [4.0, 5.0, 2.0], [6.0, 2.0, 6.0]]], [[[8.0, 5.0, 1.0], 
[8.0, 7.0, 7.0], [7.0, 7.0, 3.0]], [[7.0, 2.0, 3.0], [2.0, 1.0, 8.0], [5.0, 2.0, 1.0]], [[8.0, 7.0, 8.0], [5.0, 5.0, 4.0], 
[7.0, 8.0, 5.0]]], [[[7.0, 8.0, 4.0], [7.0, 2.0, 6.0], [7.0, 2.0, 4.0]], [[1.0, 1.0, 5.0], [4.0, 3.0, 7.0], [5.0, 7.0, 4.0]], 
[[4.0, 5.0, 7.0], [6.0, 2.0, 6.0], [4.0, 3.0, 7.0]]], [[[6.0, 3.0, 1.0], [7.0, 4.0, 4.0], [5.0, 8.0, 8.0]], [[2.0, 8.0, 6.0], 
[7.0, 1.0, 5.0], [5.0, 5.0, 6.0]], [[6.0, 1.0, 4.0], [8.0, 8.0, 2.0], [8.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PCCN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 1.0], [5.0, 6.0, 4.0], [2.0, 1.0, 4.0]], [[4.0, 1.0, 6.0], [8.0, 1.0, 2.0], [1.0, 5.0, 
3.0]], [[5.0, 5.0, 2.0], [4.0, 3.0, 7.0], [3.0, 6.0, 8.0]]], [[[8.0, 3.0, 2.0], [4.0, 3.0, 2.0], [5.0, 3.0, 5.0]], [[8.0, 6.0, 
8.0], [7.0, 4.0, 5.0], [7.0, 5.0, 1.0]], [[1.0, 1.0, 6.0], [6.0, 4.0, 8.0], [1.0, 4.0, 1.0]]], [[[8.0, 5.0, 4.0], [6.0, 5.0, 
8.0], [3.0, 3.0, 4.0]], [[7.0, 2.0, 1.0], [3.0, 7.0, 5.0], [2.0, 3.0, 1.0]], [[3.0, 5.0, 3.0], [6.0, 7.0, 1.0], [1.0, 3.0, 
1.0]]], [[[2.0, 6.0, 5.0], [3.0, 3.0, 2.0], [5.0, 4.0, 3.0]], [[3.0, 6.0, 5.0], [6.0, 1.0, 1.0], [3.0, 1.0, 6.0]], [[8.0, 7.0, 
5.0], [6.0, 8.0, 4.0], [6.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 4.0], [8.0, 1.0, 8.0], [3.0, 7.0, 5.0]], [[1.0, 4.0, 6.0], [6.0, 4.0, 
6.0], [6.0, 4.0, 6.0]], [[8.0, 8.0, 1.0], [1.0, 5.0, 5.0], [5.0, 3.0, 4.0]]], [[[3.0, 6.0, 4.0], [8.0, 1.0, 8.0], [4.0, 4.0, 
7.0]], [[1.0, 1.0, 4.0], [7.0, 2.0, 2.0], [4.0, 7.0, 7.0]], [[2.0, 7.0, 6.0], [2.0, 5.0, 5.0], [2.0, 6.0, 1.0]]], [[[1.0, 3.0, 
1.0], [2.0, 8.0, 3.0], [1.0, 8.0, 2.0]], [[3.0, 6.0, 2.0], [6.0, 4.0, 1.0], [7.0, 2.0, 3.0]], [[2.0, 1.0, 2.0], [4.0, 1.0, 
2.0], [8.0, 6.0, 6.0]]], [[[3.0, 7.0, 1.0], [7.0, 3.0, 6.0], [3.0, 2.0, 6.0]], [[4.0, 5.0, 3.0], [5.0, 3.0, 8.0], [1.0, 8.0, 
8.0]], [[3.0, 6.0, 5.0], [8.0, 4.0, 7.0], [4.0, 2.0, 7.0]]]]))
      val=Data(numpy.array([[[3.0, 7.0, 1.0], [8.0, 4.0, 4.0], [5.0, 3.0, 2.0]], [[2.0, 2.0, 3.0], [7.0, 6.0, 1.0], [5.0, 4.0, 
6.0]], [[3.0, 5.0, 3.0], [3.0, 1.0, 2.0], [7.0, 4.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 4.0, 7.0], [6.0, 6.0, 2.0], [4.0, 6.0, 1.0]], [[1.0, 6.0, 4.0], [6.0, 6.0, 4.0], 
[2.0, 3.0, 1.0]], [[4.0, 7.0, 4.0], [2.0, 7.0, 8.0], [2.0, 6.0, 4.0]]]))
      arg[2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [5.0, 6.0, 4.0], [2.0, 1.0, 4.0]], [[4.0, 1.0, 6.0], [8.0, 1.0, 2.0], [1.0, 
5.0, 3.0]], [[5.0, 5.0, 2.0], [4.0, 3.0, 7.0], [3.0, 6.0, 8.0]]], [[[8.0, 3.0, 2.0], [4.0, 3.0, 2.0], [5.0, 3.0, 5.0]], [[8.0, 
6.0, 8.0], [7.0, 4.0, 5.0], [7.0, 5.0, 1.0]], [[1.0, 1.0, 6.0], [6.0, 4.0, 8.0], [1.0, 4.0, 1.0]]], [[[3.0, 7.0, 1.0], [8.0, 
4.0, 4.0], [5.0, 3.0, 2.0]], [[2.0, 2.0, 3.0], [7.0, 6.0, 1.0], [5.0, 4.0, 6.0]], [[3.0, 5.0, 3.0], [3.0, 1.0, 2.0], [7.0, 4.0, 
2.0]]], [[[2.0, 6.0, 5.0], [3.0, 3.0, 2.0], [5.0, 4.0, 3.0]], [[3.0, 6.0, 5.0], [6.0, 1.0, 1.0], [3.0, 1.0, 6.0]], [[8.0, 7.0, 
5.0], [6.0, 8.0, 4.0], [6.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 4.0], [8.0, 1.0, 8.0], [3.0, 7.0, 5.0]], 
[[1.0, 4.0, 6.0], [6.0, 4.0, 6.0], [6.0, 4.0, 6.0]], [[8.0, 8.0, 1.0], [1.0, 5.0, 5.0], [5.0, 3.0, 4.0]]], [[[3.0, 6.0, 4.0], 
[8.0, 1.0, 8.0], [4.0, 4.0, 7.0]], [[1.0, 1.0, 4.0], [7.0, 2.0, 2.0], [4.0, 7.0, 7.0]], [[2.0, 7.0, 6.0], [2.0, 5.0, 5.0], 
[2.0, 6.0, 1.0]]], [[[2.0, 4.0, 7.0], [6.0, 6.0, 2.0], [4.0, 6.0, 1.0]], [[1.0, 6.0, 4.0], [6.0, 6.0, 4.0], [2.0, 3.0, 1.0]], 
[[4.0, 7.0, 4.0], [2.0, 7.0, 8.0], [2.0, 6.0, 4.0]]], [[[3.0, 7.0, 1.0], [7.0, 3.0, 6.0], [3.0, 2.0, 6.0]], [[4.0, 5.0, 3.0], 
[5.0, 3.0, 8.0], [1.0, 8.0, 8.0]], [[3.0, 6.0, 5.0], [8.0, 4.0, 7.0], [4.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PCCN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 7.0], [3.0, 7.0, 4.0], [2.0, 5.0, 3.0]], [[4.0, 2.0, 4.0], [8.0, 8.0, 6.0], [1.0, 3.0, 
4.0]], [[7.0, 2.0, 2.0], [3.0, 3.0, 1.0], [6.0, 7.0, 8.0]]], [[[5.0, 1.0, 5.0], [8.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[2.0, 7.0, 
1.0], [1.0, 1.0, 1.0], [7.0, 7.0, 8.0]], [[7.0, 1.0, 2.0], [4.0, 4.0, 3.0], [7.0, 4.0, 1.0]]], [[[1.0, 2.0, 2.0], [4.0, 8.0, 
6.0], [7.0, 8.0, 7.0]], [[7.0, 7.0, 8.0], [1.0, 1.0, 3.0], [8.0, 1.0, 7.0]], [[1.0, 5.0, 1.0], [2.0, 4.0, 8.0], [8.0, 1.0, 
6.0]]], [[[2.0, 7.0, 7.0], [1.0, 5.0, 4.0], [7.0, 8.0, 4.0]], [[7.0, 1.0, 3.0], [1.0, 3.0, 3.0], [4.0, 1.0, 2.0]], [[5.0, 8.0, 
7.0], [4.0, 8.0, 3.0], [8.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 6.0], [2.0, 5.0, 2.0], [5.0, 6.0, 1.0]], [[4.0, 8.0, 4.0], [3.0, 6.0, 
7.0], [2.0, 4.0, 8.0]], [[8.0, 3.0, 5.0], [4.0, 3.0, 4.0], [5.0, 2.0, 2.0]]], [[[4.0, 1.0, 3.0], [5.0, 4.0, 3.0], [6.0, 6.0, 
6.0]], [[1.0, 2.0, 5.0], [1.0, 6.0, 3.0], [5.0, 6.0, 6.0]], [[7.0, 8.0, 5.0], [5.0, 2.0, 4.0], [7.0, 3.0, 2.0]]], [[[2.0, 3.0, 
1.0], [8.0, 3.0, 3.0], [8.0, 2.0, 3.0]], [[2.0, 2.0, 1.0], [3.0, 5.0, 2.0], [7.0, 2.0, 5.0]], [[4.0, 5.0, 3.0], [3.0, 4.0, 
7.0], [7.0, 7.0, 1.0]]], [[[7.0, 8.0, 6.0], [1.0, 5.0, 8.0], [8.0, 2.0, 5.0]], [[2.0, 5.0, 5.0], [4.0, 7.0, 8.0], [2.0, 4.0, 
5.0]], [[4.0, 5.0, 4.0], [6.0, 3.0, 4.0], [8.0, 3.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[2.0, 6.0, 5.0], [5.0, 7.0, 6.0], [6.0, 8.0, 7.0]], [[8.0, 1.0, 2.0], [1.0, 2.0, 6.0], [4.0, 
8.0, 1.0]], [[8.0, 4.0, 5.0], [8.0, 8.0, 7.0], [2.0, 5.0, 6.0]]])+(1.-msk_val)*numpy.array([[[3.0, 4.0, 6.0], [1.0, 5.0, 5.0], 
[3.0, 4.0, 8.0]], [[4.0, 7.0, 5.0], [1.0, 8.0, 1.0], [2.0, 1.0, 1.0]], [[1.0, 7.0, 4.0], [2.0, 6.0, 5.0], [5.0, 2.0, 7.0]]])
      arg[2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 7.0], [3.0, 7.0, 4.0], [2.0, 5.0, 3.0]], [[4.0, 2.0, 4.0], [8.0, 8.0, 6.0], [1.0, 
3.0, 4.0]], [[7.0, 2.0, 2.0], [3.0, 3.0, 1.0], [6.0, 7.0, 8.0]]], [[[5.0, 1.0, 5.0], [8.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[2.0, 
7.0, 1.0], [1.0, 1.0, 1.0], [7.0, 7.0, 8.0]], [[7.0, 1.0, 2.0], [4.0, 4.0, 3.0], [7.0, 4.0, 1.0]]], [[[2.0, 6.0, 5.0], [5.0, 
7.0, 6.0], [6.0, 8.0, 7.0]], [[8.0, 1.0, 2.0], [1.0, 2.0, 6.0], [4.0, 8.0, 1.0]], [[8.0, 4.0, 5.0], [8.0, 8.0, 7.0], [2.0, 5.0, 
6.0]]], [[[2.0, 7.0, 7.0], [1.0, 5.0, 4.0], [7.0, 8.0, 4.0]], [[7.0, 1.0, 3.0], [1.0, 3.0, 3.0], [4.0, 1.0, 2.0]], [[5.0, 8.0, 
7.0], [4.0, 8.0, 3.0], [8.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 6.0], [2.0, 5.0, 2.0], [5.0, 6.0, 1.0]], 
[[4.0, 8.0, 4.0], [3.0, 6.0, 7.0], [2.0, 4.0, 8.0]], [[8.0, 3.0, 5.0], [4.0, 3.0, 4.0], [5.0, 2.0, 2.0]]], [[[4.0, 1.0, 3.0], 
[5.0, 4.0, 3.0], [6.0, 6.0, 6.0]], [[1.0, 2.0, 5.0], [1.0, 6.0, 3.0], [5.0, 6.0, 6.0]], [[7.0, 8.0, 5.0], [5.0, 2.0, 4.0], 
[7.0, 3.0, 2.0]]], [[[3.0, 4.0, 6.0], [1.0, 5.0, 5.0], [3.0, 4.0, 8.0]], [[4.0, 7.0, 5.0], [1.0, 8.0, 1.0], [2.0, 1.0, 1.0]], 
[[1.0, 7.0, 4.0], [2.0, 6.0, 5.0], [5.0, 2.0, 7.0]]], [[[7.0, 8.0, 6.0], [1.0, 5.0, 8.0], [8.0, 2.0, 5.0]], [[2.0, 5.0, 5.0], 
[4.0, 7.0, 8.0], [2.0, 4.0, 5.0]], [[4.0, 5.0, 4.0], [6.0, 3.0, 4.0], [8.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RURN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 7.0], [4.0, 5.0, 6.0], [1.0, 3.0, 4.0]], [[7.0, 3.0, 1.0], [2.0, 3.0, 1.0], [6.0, 2.0, 
6.0]], [[5.0, 8.0, 8.0], [7.0, 6.0, 2.0], [3.0, 6.0, 1.0]]], [[[3.0, 7.0, 3.0], [2.0, 1.0, 5.0], [8.0, 6.0, 2.0]], [[4.0, 5.0, 
6.0], [8.0, 8.0, 4.0], [5.0, 3.0, 7.0]], [[3.0, 8.0, 1.0], [4.0, 8.0, 3.0], [3.0, 2.0, 2.0]]], [[[2.0, 2.0, 7.0], [5.0, 2.0, 
8.0], [2.0, 8.0, 4.0]], [[8.0, 6.0, 2.0], [7.0, 6.0, 2.0], [7.0, 8.0, 1.0]], [[3.0, 6.0, 1.0], [1.0, 3.0, 2.0], [5.0, 1.0, 
4.0]]], [[[5.0, 6.0, 1.0], [8.0, 5.0, 8.0], [3.0, 5.0, 2.0]], [[6.0, 2.0, 8.0], [6.0, 3.0, 3.0], [3.0, 3.0, 1.0]], [[2.0, 7.0, 
4.0], [8.0, 6.0, 3.0], [2.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 3.0], [6.0, 7.0, 6.0], [7.0, 4.0, 7.0]], [[6.0, 4.0, 2.0], [7.0, 8.0, 
2.0], [8.0, 2.0, 5.0]], [[1.0, 2.0, 3.0], [7.0, 6.0, 2.0], [3.0, 5.0, 3.0]]], [[[3.0, 1.0, 3.0], [4.0, 2.0, 7.0], [6.0, 6.0, 
5.0]], [[3.0, 3.0, 5.0], [1.0, 2.0, 3.0], [7.0, 1.0, 2.0]], [[5.0, 1.0, 8.0], [2.0, 4.0, 4.0], [2.0, 3.0, 8.0]]], [[[1.0, 5.0, 
3.0], [3.0, 7.0, 3.0], [4.0, 7.0, 4.0]], [[8.0, 6.0, 7.0], [4.0, 6.0, 4.0], [3.0, 5.0, 1.0]], [[1.0, 5.0, 8.0], [5.0, 3.0, 
4.0], [1.0, 6.0, 5.0]]], [[[8.0, 4.0, 7.0], [5.0, 3.0, 3.0], [6.0, 2.0, 2.0]], [[7.0, 4.0, 2.0], [6.0, 4.0, 6.0], [3.0, 7.0, 
5.0]], [[3.0, 1.0, 7.0], [1.0, 6.0, 3.0], [5.0, 3.0, 5.0]]]]))
      val=numpy.array([[[[2.0, 3.0, 2.0]], [[4.0, 8.0, 7.0]]], [[[8.0, 8.0, 1.0]], [[5.0, 6.0, 1.0]]]])
      arg[1:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 7.0], [4.0, 5.0, 6.0], [1.0, 3.0, 4.0]], [[7.0, 3.0, 1.0], [2.0, 3.0, 1.0], [6.0, 
2.0, 6.0]], [[5.0, 8.0, 8.0], [7.0, 6.0, 2.0], [3.0, 6.0, 1.0]]], [[[3.0, 7.0, 3.0], [2.0, 3.0, 2.0], [8.0, 6.0, 2.0]], [[4.0, 
5.0, 6.0], [4.0, 8.0, 7.0], [5.0, 3.0, 7.0]], [[3.0, 8.0, 1.0], [4.0, 8.0, 3.0], [3.0, 2.0, 2.0]]], [[[2.0, 2.0, 7.0], [8.0, 
8.0, 1.0], [2.0, 8.0, 4.0]], [[8.0, 6.0, 2.0], [5.0, 6.0, 1.0], [7.0, 8.0, 1.0]], [[3.0, 6.0, 1.0], [1.0, 3.0, 2.0], [5.0, 1.0, 
4.0]]], [[[5.0, 6.0, 1.0], [8.0, 5.0, 8.0], [3.0, 5.0, 2.0]], [[6.0, 2.0, 8.0], [6.0, 3.0, 3.0], [3.0, 3.0, 1.0]], [[2.0, 7.0, 
4.0], [8.0, 6.0, 3.0], [2.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 3.0], [6.0, 7.0, 6.0], [7.0, 4.0, 7.0]], 
[[6.0, 4.0, 2.0], [7.0, 8.0, 2.0], [8.0, 2.0, 5.0]], [[1.0, 2.0, 3.0], [7.0, 6.0, 2.0], [3.0, 5.0, 3.0]]], [[[3.0, 1.0, 3.0], 
[2.0, 3.0, 2.0], [6.0, 6.0, 5.0]], [[3.0, 3.0, 5.0], [4.0, 8.0, 7.0], [7.0, 1.0, 2.0]], [[5.0, 1.0, 8.0], [2.0, 4.0, 4.0], 
[2.0, 3.0, 8.0]]], [[[1.0, 5.0, 3.0], [8.0, 8.0, 1.0], [4.0, 7.0, 4.0]], [[8.0, 6.0, 7.0], [5.0, 6.0, 1.0], [3.0, 5.0, 1.0]], 
[[1.0, 5.0, 8.0], [5.0, 3.0, 4.0], [1.0, 6.0, 5.0]]], [[[8.0, 4.0, 7.0], [5.0, 3.0, 3.0], [6.0, 2.0, 2.0]], [[7.0, 4.0, 2.0], 
[6.0, 4.0, 6.0], [3.0, 7.0, 5.0]], [[3.0, 1.0, 7.0], [1.0, 6.0, 3.0], [5.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RURN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 8.0], [6.0, 8.0, 1.0], [2.0, 3.0, 8.0]], [[8.0, 1.0, 8.0], [6.0, 6.0, 2.0], [2.0, 1.0, 
2.0]], [[1.0, 7.0, 1.0], [1.0, 1.0, 3.0], [1.0, 8.0, 1.0]]], [[[7.0, 6.0, 1.0], [2.0, 1.0, 2.0], [7.0, 2.0, 7.0]], [[2.0, 7.0, 
4.0], [3.0, 2.0, 2.0], [5.0, 6.0, 4.0]], [[8.0, 4.0, 3.0], [1.0, 1.0, 7.0], [1.0, 8.0, 2.0]]], [[[7.0, 4.0, 3.0], [2.0, 6.0, 
8.0], [7.0, 8.0, 6.0]], [[1.0, 6.0, 5.0], [1.0, 2.0, 6.0], [1.0, 8.0, 5.0]], [[7.0, 3.0, 7.0], [6.0, 3.0, 7.0], [7.0, 6.0, 
4.0]]], [[[5.0, 7.0, 2.0], [7.0, 2.0, 3.0], [7.0, 6.0, 6.0]], [[3.0, 3.0, 7.0], [3.0, 6.0, 2.0], [1.0, 6.0, 2.0]], [[1.0, 4.0, 
6.0], [5.0, 1.0, 5.0], [7.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 3.0], [1.0, 3.0, 4.0], [4.0, 2.0, 5.0]], [[3.0, 6.0, 8.0], [7.0, 2.0, 
5.0], [7.0, 6.0, 4.0]], [[8.0, 1.0, 8.0], [1.0, 2.0, 2.0], [6.0, 1.0, 7.0]]], [[[3.0, 8.0, 1.0], [7.0, 3.0, 7.0], [5.0, 2.0, 
7.0]], [[6.0, 5.0, 4.0], [3.0, 2.0, 8.0], [5.0, 2.0, 1.0]], [[3.0, 1.0, 7.0], [5.0, 6.0, 8.0], [1.0, 4.0, 8.0]]], [[[5.0, 3.0, 
3.0], [1.0, 5.0, 2.0], [8.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [6.0, 4.0, 8.0], [6.0, 3.0, 1.0]], [[7.0, 8.0, 1.0], [6.0, 6.0, 
8.0], [8.0, 8.0, 6.0]]], [[[7.0, 7.0, 6.0], [2.0, 7.0, 5.0], [5.0, 7.0, 2.0]], [[2.0, 8.0, 5.0], [7.0, 2.0, 7.0], [8.0, 2.0, 
1.0]], [[5.0, 7.0, 8.0], [4.0, 7.0, 7.0], [6.0, 4.0, 5.0]]]]))
      val=Data(numpy.array([[[[6.0, 1.0, 8.0]], [[5.0, 5.0, 8.0]]], [[[6.0, 2.0, 7.0]], [[7.0, 5.0, 4.0]]]]),self.functionspace)
      arg[1:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 8.0], [6.0, 8.0, 1.0], [2.0, 3.0, 8.0]], [[8.0, 1.0, 8.0], [6.0, 6.0, 2.0], [2.0, 
1.0, 2.0]], [[1.0, 7.0, 1.0], [1.0, 1.0, 3.0], [1.0, 8.0, 1.0]]], [[[7.0, 6.0, 1.0], [6.0, 1.0, 8.0], [7.0, 2.0, 7.0]], [[2.0, 
7.0, 4.0], [5.0, 5.0, 8.0], [5.0, 6.0, 4.0]], [[8.0, 4.0, 3.0], [1.0, 1.0, 7.0], [1.0, 8.0, 2.0]]], [[[7.0, 4.0, 3.0], [6.0, 
2.0, 7.0], [7.0, 8.0, 6.0]], [[1.0, 6.0, 5.0], [7.0, 5.0, 4.0], [1.0, 8.0, 5.0]], [[7.0, 3.0, 7.0], [6.0, 3.0, 7.0], [7.0, 6.0, 
4.0]]], [[[5.0, 7.0, 2.0], [7.0, 2.0, 3.0], [7.0, 6.0, 6.0]], [[3.0, 3.0, 7.0], [3.0, 6.0, 2.0], [1.0, 6.0, 2.0]], [[1.0, 4.0, 
6.0], [5.0, 1.0, 5.0], [7.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 3.0], [1.0, 3.0, 4.0], [4.0, 2.0, 5.0]], 
[[3.0, 6.0, 8.0], [7.0, 2.0, 5.0], [7.0, 6.0, 4.0]], [[8.0, 1.0, 8.0], [1.0, 2.0, 2.0], [6.0, 1.0, 7.0]]], [[[3.0, 8.0, 1.0], 
[6.0, 1.0, 8.0], [5.0, 2.0, 7.0]], [[6.0, 5.0, 4.0], [5.0, 5.0, 8.0], [5.0, 2.0, 1.0]], [[3.0, 1.0, 7.0], [5.0, 6.0, 8.0], 
[1.0, 4.0, 8.0]]], [[[5.0, 3.0, 3.0], [6.0, 2.0, 7.0], [8.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [7.0, 5.0, 4.0], [6.0, 3.0, 1.0]], 
[[7.0, 8.0, 1.0], [6.0, 6.0, 8.0], [8.0, 8.0, 6.0]]], [[[7.0, 7.0, 6.0], [2.0, 7.0, 5.0], [5.0, 7.0, 2.0]], [[2.0, 8.0, 5.0], 
[7.0, 2.0, 7.0], [8.0, 2.0, 1.0]], [[5.0, 7.0, 8.0], [4.0, 7.0, 7.0], [6.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RURN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 2.0], [5.0, 8.0, 5.0], [1.0, 2.0, 8.0]], [[3.0, 2.0, 2.0], [5.0, 4.0, 1.0], [4.0, 8.0, 
8.0]], [[6.0, 3.0, 8.0], [3.0, 6.0, 1.0], [5.0, 6.0, 3.0]]], [[[6.0, 1.0, 3.0], [7.0, 8.0, 4.0], [5.0, 4.0, 2.0]], [[5.0, 1.0, 
3.0], [2.0, 5.0, 8.0], [8.0, 6.0, 2.0]], [[3.0, 4.0, 8.0], [3.0, 4.0, 7.0], [7.0, 4.0, 4.0]]], [[[4.0, 6.0, 3.0], [6.0, 8.0, 
3.0], [8.0, 2.0, 7.0]], [[6.0, 3.0, 5.0], [1.0, 2.0, 7.0], [4.0, 3.0, 8.0]], [[2.0, 5.0, 3.0], [3.0, 2.0, 1.0], [2.0, 1.0, 
4.0]]], [[[8.0, 7.0, 2.0], [8.0, 6.0, 7.0], [8.0, 8.0, 6.0]], [[1.0, 4.0, 7.0], [8.0, 7.0, 6.0], [4.0, 2.0, 8.0]], [[6.0, 2.0, 
7.0], [3.0, 8.0, 5.0], [4.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 3.0], [4.0, 7.0, 8.0], [1.0, 6.0, 7.0]], [[8.0, 3.0, 3.0], [5.0, 7.0, 
8.0], [7.0, 4.0, 6.0]], [[2.0, 7.0, 4.0], [6.0, 8.0, 4.0], [7.0, 5.0, 8.0]]], [[[6.0, 5.0, 4.0], [3.0, 6.0, 2.0], [2.0, 6.0, 
8.0]], [[3.0, 8.0, 4.0], [6.0, 6.0, 7.0], [6.0, 8.0, 2.0]], [[5.0, 4.0, 6.0], [2.0, 8.0, 2.0], [5.0, 5.0, 5.0]]], [[[5.0, 2.0, 
8.0], [5.0, 1.0, 1.0], [3.0, 3.0, 5.0]], [[7.0, 3.0, 8.0], [6.0, 8.0, 6.0], [3.0, 3.0, 5.0]], [[8.0, 8.0, 2.0], [4.0, 7.0, 
7.0], [7.0, 5.0, 3.0]]], [[[4.0, 5.0, 2.0], [6.0, 3.0, 6.0], [7.0, 2.0, 5.0]], [[4.0, 7.0, 3.0], [4.0, 4.0, 4.0], [8.0, 4.0, 
6.0]], [[7.0, 3.0, 2.0], [5.0, 7.0, 5.0], [4.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[[8.0, 5.0, 3.0]], [[8.0, 5.0, 8.0]]], [[[1.0, 2.0, 6.0]], [[1.0, 1.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 8.0]], [[7.0, 5.0, 1.0]]], [[[4.0, 4.0, 1.0]], [[7.0, 4.0, 4.0]]]]))
      arg[1:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 2.0], [5.0, 8.0, 5.0], [1.0, 2.0, 8.0]], [[3.0, 2.0, 2.0], [5.0, 4.0, 1.0], [4.0, 
8.0, 8.0]], [[6.0, 3.0, 8.0], [3.0, 6.0, 1.0], [5.0, 6.0, 3.0]]], [[[6.0, 1.0, 3.0], [8.0, 5.0, 3.0], [5.0, 4.0, 2.0]], [[5.0, 
1.0, 3.0], [8.0, 5.0, 8.0], [8.0, 6.0, 2.0]], [[3.0, 4.0, 8.0], [3.0, 4.0, 7.0], [7.0, 4.0, 4.0]]], [[[4.0, 6.0, 3.0], [1.0, 
2.0, 6.0], [8.0, 2.0, 7.0]], [[6.0, 3.0, 5.0], [1.0, 1.0, 2.0], [4.0, 3.0, 8.0]], [[2.0, 5.0, 3.0], [3.0, 2.0, 1.0], [2.0, 1.0, 
4.0]]], [[[8.0, 7.0, 2.0], [8.0, 6.0, 7.0], [8.0, 8.0, 6.0]], [[1.0, 4.0, 7.0], [8.0, 7.0, 6.0], [4.0, 2.0, 8.0]], [[6.0, 2.0, 
7.0], [3.0, 8.0, 5.0], [4.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 3.0], [4.0, 7.0, 8.0], [1.0, 6.0, 7.0]], 
[[8.0, 3.0, 3.0], [5.0, 7.0, 8.0], [7.0, 4.0, 6.0]], [[2.0, 7.0, 4.0], [6.0, 8.0, 4.0], [7.0, 5.0, 8.0]]], [[[6.0, 5.0, 4.0], 
[8.0, 1.0, 8.0], [2.0, 6.0, 8.0]], [[3.0, 8.0, 4.0], [7.0, 5.0, 1.0], [6.0, 8.0, 2.0]], [[5.0, 4.0, 6.0], [2.0, 8.0, 2.0], 
[5.0, 5.0, 5.0]]], [[[5.0, 2.0, 8.0], [4.0, 4.0, 1.0], [3.0, 3.0, 5.0]], [[7.0, 3.0, 8.0], [7.0, 4.0, 4.0], [3.0, 3.0, 5.0]], 
[[8.0, 8.0, 2.0], [4.0, 7.0, 7.0], [7.0, 5.0, 3.0]]], [[[4.0, 5.0, 2.0], [6.0, 3.0, 6.0], [7.0, 2.0, 5.0]], [[4.0, 7.0, 3.0], 
[4.0, 4.0, 4.0], [8.0, 4.0, 6.0]], [[7.0, 3.0, 2.0], [5.0, 7.0, 5.0], [4.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RURN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [2.0, 4.0, 1.0], [1.0, 7.0, 5.0]], [[6.0, 7.0, 2.0], [1.0, 1.0, 1.0], [7.0, 3.0, 
6.0]], [[5.0, 6.0, 3.0], [5.0, 2.0, 3.0], [3.0, 3.0, 8.0]]], [[[2.0, 6.0, 2.0], [6.0, 6.0, 7.0], [1.0, 8.0, 7.0]], [[5.0, 1.0, 
4.0], [5.0, 1.0, 8.0], [5.0, 2.0, 1.0]], [[6.0, 8.0, 3.0], [4.0, 4.0, 8.0], [5.0, 4.0, 7.0]]], [[[8.0, 4.0, 2.0], [2.0, 1.0, 
4.0], [2.0, 3.0, 3.0]], [[8.0, 8.0, 6.0], [4.0, 5.0, 8.0], [1.0, 2.0, 2.0]], [[1.0, 8.0, 6.0], [8.0, 3.0, 2.0], [8.0, 7.0, 
7.0]]], [[[3.0, 6.0, 1.0], [4.0, 5.0, 1.0], [5.0, 1.0, 4.0]], [[5.0, 2.0, 2.0], [1.0, 7.0, 8.0], [2.0, 1.0, 5.0]], [[8.0, 6.0, 
1.0], [5.0, 7.0, 8.0], [7.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 3.0], [2.0, 4.0, 1.0], [7.0, 5.0, 4.0]], [[4.0, 8.0, 6.0], [8.0, 4.0, 
5.0], [5.0, 7.0, 7.0]], [[4.0, 7.0, 5.0], [5.0, 1.0, 2.0], [1.0, 7.0, 6.0]]], [[[4.0, 8.0, 3.0], [8.0, 2.0, 7.0], [8.0, 5.0, 
2.0]], [[5.0, 3.0, 2.0], [5.0, 5.0, 2.0], [3.0, 7.0, 3.0]], [[6.0, 5.0, 2.0], [1.0, 8.0, 4.0], [8.0, 8.0, 3.0]]], [[[7.0, 7.0, 
8.0], [3.0, 1.0, 6.0], [1.0, 4.0, 8.0]], [[8.0, 4.0, 7.0], [6.0, 3.0, 2.0], [5.0, 6.0, 5.0]], [[2.0, 8.0, 8.0], [1.0, 6.0, 
2.0], [6.0, 6.0, 4.0]]], [[[5.0, 7.0, 2.0], [8.0, 4.0, 4.0], [6.0, 4.0, 4.0]], [[1.0, 7.0, 7.0], [6.0, 6.0, 8.0], [6.0, 3.0, 
1.0]], [[2.0, 5.0, 2.0], [3.0, 8.0, 6.0], [7.0, 3.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 1.0, 6.0]], [[6.0, 8.0, 3.0]]], [[[6.0, 4.0, 7.0]], [[2.0, 1.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 1.0, 2.0]], [[3.0, 7.0, 8.0]]], [[[3.0, 7.0, 2.0]], [[1.0, 5.0, 4.0]]]])
      arg[1:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 7.0], [2.0, 4.0, 1.0], [1.0, 7.0, 5.0]], [[6.0, 7.0, 2.0], [1.0, 1.0, 1.0], [7.0, 
3.0, 6.0]], [[5.0, 6.0, 3.0], [5.0, 2.0, 3.0], [3.0, 3.0, 8.0]]], [[[2.0, 6.0, 2.0], [2.0, 1.0, 6.0], [1.0, 8.0, 7.0]], [[5.0, 
1.0, 4.0], [6.0, 8.0, 3.0], [5.0, 2.0, 1.0]], [[6.0, 8.0, 3.0], [4.0, 4.0, 8.0], [5.0, 4.0, 7.0]]], [[[8.0, 4.0, 2.0], [6.0, 
4.0, 7.0], [2.0, 3.0, 3.0]], [[8.0, 8.0, 6.0], [2.0, 1.0, 8.0], [1.0, 2.0, 2.0]], [[1.0, 8.0, 6.0], [8.0, 3.0, 2.0], [8.0, 7.0, 
7.0]]], [[[3.0, 6.0, 1.0], [4.0, 5.0, 1.0], [5.0, 1.0, 4.0]], [[5.0, 2.0, 2.0], [1.0, 7.0, 8.0], [2.0, 1.0, 5.0]], [[8.0, 6.0, 
1.0], [5.0, 7.0, 8.0], [7.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 3.0], [2.0, 4.0, 1.0], [7.0, 5.0, 4.0]], 
[[4.0, 8.0, 6.0], [8.0, 4.0, 5.0], [5.0, 7.0, 7.0]], [[4.0, 7.0, 5.0], [5.0, 1.0, 2.0], [1.0, 7.0, 6.0]]], [[[4.0, 8.0, 3.0], 
[2.0, 1.0, 2.0], [8.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [3.0, 7.0, 8.0], [3.0, 7.0, 3.0]], [[6.0, 5.0, 2.0], [1.0, 8.0, 4.0], 
[8.0, 8.0, 3.0]]], [[[7.0, 7.0, 8.0], [3.0, 7.0, 2.0], [1.0, 4.0, 8.0]], [[8.0, 4.0, 7.0], [1.0, 5.0, 4.0], [5.0, 6.0, 5.0]], 
[[2.0, 8.0, 8.0], [1.0, 6.0, 2.0], [6.0, 6.0, 4.0]]], [[[5.0, 7.0, 2.0], [8.0, 4.0, 4.0], [6.0, 4.0, 4.0]], [[1.0, 7.0, 7.0], 
[6.0, 6.0, 8.0], [6.0, 3.0, 1.0]], [[2.0, 5.0, 2.0], [3.0, 8.0, 6.0], [7.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_URUL(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 8.0], [7.0, 2.0, 8.0], [4.0, 7.0, 5.0]], [[3.0, 8.0, 7.0], [4.0, 1.0, 8.0], [2.0, 7.0, 
8.0]], [[2.0, 7.0, 6.0], [4.0, 5.0, 4.0], [5.0, 4.0, 1.0]]], [[[8.0, 6.0, 2.0], [5.0, 3.0, 1.0], [7.0, 7.0, 6.0]], [[8.0, 1.0, 
5.0], [3.0, 4.0, 3.0], [3.0, 6.0, 5.0]], [[4.0, 3.0, 6.0], [2.0, 6.0, 8.0], [1.0, 8.0, 7.0]]], [[[5.0, 4.0, 4.0], [6.0, 6.0, 
6.0], [4.0, 3.0, 2.0]], [[4.0, 3.0, 2.0], [4.0, 3.0, 7.0], [6.0, 6.0, 2.0]], [[1.0, 8.0, 2.0], [4.0, 3.0, 1.0], [4.0, 8.0, 
5.0]]], [[[5.0, 5.0, 6.0], [3.0, 5.0, 4.0], [1.0, 7.0, 4.0]], [[6.0, 7.0, 4.0], [5.0, 2.0, 5.0], [7.0, 1.0, 3.0]], [[5.0, 1.0, 
6.0], [4.0, 7.0, 1.0], [2.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 7.0], [1.0, 5.0, 1.0], [5.0, 4.0, 7.0]], [[7.0, 7.0, 3.0], [5.0, 3.0, 
7.0], [6.0, 5.0, 7.0]], [[4.0, 8.0, 2.0], [3.0, 1.0, 3.0], [3.0, 5.0, 5.0]]], [[[7.0, 8.0, 3.0], [7.0, 6.0, 4.0], [4.0, 8.0, 
8.0]], [[3.0, 2.0, 4.0], [1.0, 2.0, 6.0], [8.0, 6.0, 8.0]], [[6.0, 6.0, 3.0], [7.0, 4.0, 6.0], [8.0, 7.0, 3.0]]], [[[5.0, 3.0, 
1.0], [1.0, 4.0, 2.0], [7.0, 2.0, 1.0]], [[1.0, 6.0, 4.0], [8.0, 6.0, 7.0], [4.0, 4.0, 6.0]], [[3.0, 8.0, 6.0], [3.0, 8.0, 
1.0], [6.0, 1.0, 6.0]]], [[[3.0, 4.0, 6.0], [5.0, 7.0, 2.0], [4.0, 4.0, 6.0]], [[7.0, 4.0, 1.0], [6.0, 6.0, 8.0], [5.0, 7.0, 
7.0]], [[1.0, 4.0, 1.0], [4.0, 8.0, 2.0], [7.0, 3.0, 3.0]]]]))
      val=numpy.array([[[[8.0], [2.0]]], [[[2.0], [1.0]]], [[[8.0], [3.0]]]])
      arg[:3,1:2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 8.0], [7.0, 2.0, 8.0], [4.0, 7.0, 5.0]], [[3.0, 8.0, 8.0], [4.0, 1.0, 2.0], [2.0, 
7.0, 8.0]], [[2.0, 7.0, 6.0], [4.0, 5.0, 4.0], [5.0, 4.0, 1.0]]], [[[8.0, 6.0, 2.0], [5.0, 3.0, 1.0], [7.0, 7.0, 6.0]], [[8.0, 
1.0, 2.0], [3.0, 4.0, 1.0], [3.0, 6.0, 5.0]], [[4.0, 3.0, 6.0], [2.0, 6.0, 8.0], [1.0, 8.0, 7.0]]], [[[5.0, 4.0, 4.0], [6.0, 
6.0, 6.0], [4.0, 3.0, 2.0]], [[4.0, 3.0, 8.0], [4.0, 3.0, 3.0], [6.0, 6.0, 2.0]], [[1.0, 8.0, 2.0], [4.0, 3.0, 1.0], [4.0, 8.0, 
5.0]]], [[[5.0, 5.0, 6.0], [3.0, 5.0, 4.0], [1.0, 7.0, 4.0]], [[6.0, 7.0, 4.0], [5.0, 2.0, 5.0], [7.0, 1.0, 3.0]], [[5.0, 1.0, 
6.0], [4.0, 7.0, 1.0], [2.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 7.0], [1.0, 5.0, 1.0], [5.0, 4.0, 7.0]], 
[[7.0, 7.0, 8.0], [5.0, 3.0, 2.0], [6.0, 5.0, 7.0]], [[4.0, 8.0, 2.0], [3.0, 1.0, 3.0], [3.0, 5.0, 5.0]]], [[[7.0, 8.0, 3.0], 
[7.0, 6.0, 4.0], [4.0, 8.0, 8.0]], [[3.0, 2.0, 2.0], [1.0, 2.0, 1.0], [8.0, 6.0, 8.0]], [[6.0, 6.0, 3.0], [7.0, 4.0, 6.0], 
[8.0, 7.0, 3.0]]], [[[5.0, 3.0, 1.0], [1.0, 4.0, 2.0], [7.0, 2.0, 1.0]], [[1.0, 6.0, 8.0], [8.0, 6.0, 3.0], [4.0, 4.0, 6.0]], 
[[3.0, 8.0, 6.0], [3.0, 8.0, 1.0], [6.0, 1.0, 6.0]]], [[[3.0, 4.0, 6.0], [5.0, 7.0, 2.0], [4.0, 4.0, 6.0]], [[7.0, 4.0, 1.0], 
[6.0, 6.0, 8.0], [5.0, 7.0, 7.0]], [[1.0, 4.0, 1.0], [4.0, 8.0, 2.0], [7.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_URUL(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 3.0], [1.0, 4.0, 3.0], [6.0, 2.0, 5.0]], [[5.0, 4.0, 3.0], [1.0, 8.0, 2.0], [4.0, 5.0, 
2.0]], [[7.0, 6.0, 8.0], [1.0, 8.0, 4.0], [7.0, 6.0, 6.0]]], [[[7.0, 7.0, 1.0], [7.0, 5.0, 1.0], [1.0, 8.0, 2.0]], [[8.0, 4.0, 
5.0], [8.0, 5.0, 7.0], [7.0, 8.0, 7.0]], [[3.0, 6.0, 7.0], [8.0, 4.0, 8.0], [3.0, 2.0, 2.0]]], [[[6.0, 2.0, 1.0], [7.0, 2.0, 
3.0], [8.0, 2.0, 2.0]], [[8.0, 1.0, 3.0], [2.0, 8.0, 6.0], [6.0, 1.0, 3.0]], [[7.0, 5.0, 1.0], [5.0, 4.0, 5.0], [1.0, 6.0, 
3.0]]], [[[1.0, 5.0, 4.0], [2.0, 8.0, 1.0], [1.0, 3.0, 8.0]], [[7.0, 6.0, 3.0], [3.0, 3.0, 6.0], [4.0, 6.0, 3.0]], [[4.0, 1.0, 
1.0], [8.0, 2.0, 5.0], [6.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 3.0], [6.0, 4.0, 5.0], [4.0, 1.0, 8.0]], [[1.0, 8.0, 4.0], [5.0, 3.0, 
6.0], [5.0, 2.0, 1.0]], [[6.0, 1.0, 3.0], [5.0, 5.0, 5.0], [5.0, 3.0, 8.0]]], [[[7.0, 7.0, 7.0], [7.0, 3.0, 7.0], [4.0, 6.0, 
4.0]], [[4.0, 1.0, 1.0], [8.0, 7.0, 6.0], [1.0, 4.0, 6.0]], [[3.0, 6.0, 3.0], [6.0, 4.0, 5.0], [7.0, 8.0, 1.0]]], [[[4.0, 1.0, 
4.0], [4.0, 4.0, 8.0], [5.0, 1.0, 4.0]], [[8.0, 5.0, 2.0], [2.0, 4.0, 7.0], [7.0, 5.0, 4.0]], [[5.0, 3.0, 3.0], [2.0, 4.0, 
8.0], [6.0, 4.0, 7.0]]], [[[6.0, 4.0, 1.0], [3.0, 4.0, 7.0], [2.0, 1.0, 2.0]], [[2.0, 2.0, 1.0], [5.0, 8.0, 4.0], [1.0, 4.0, 
5.0]], [[8.0, 2.0, 5.0], [1.0, 4.0, 7.0], [6.0, 1.0, 7.0]]]]))
      val=Data(numpy.array([[[[5.0], [1.0]]], [[[5.0], [1.0]]], [[[7.0], [7.0]]]]),self.functionspace)
      arg[:3,1:2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [1.0, 4.0, 3.0], [6.0, 2.0, 5.0]], [[5.0, 4.0, 5.0], [1.0, 8.0, 1.0], [4.0, 
5.0, 2.0]], [[7.0, 6.0, 8.0], [1.0, 8.0, 4.0], [7.0, 6.0, 6.0]]], [[[7.0, 7.0, 1.0], [7.0, 5.0, 1.0], [1.0, 8.0, 2.0]], [[8.0, 
4.0, 5.0], [8.0, 5.0, 1.0], [7.0, 8.0, 7.0]], [[3.0, 6.0, 7.0], [8.0, 4.0, 8.0], [3.0, 2.0, 2.0]]], [[[6.0, 2.0, 1.0], [7.0, 
2.0, 3.0], [8.0, 2.0, 2.0]], [[8.0, 1.0, 7.0], [2.0, 8.0, 7.0], [6.0, 1.0, 3.0]], [[7.0, 5.0, 1.0], [5.0, 4.0, 5.0], [1.0, 6.0, 
3.0]]], [[[1.0, 5.0, 4.0], [2.0, 8.0, 1.0], [1.0, 3.0, 8.0]], [[7.0, 6.0, 3.0], [3.0, 3.0, 6.0], [4.0, 6.0, 3.0]], [[4.0, 1.0, 
1.0], [8.0, 2.0, 5.0], [6.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 3.0], [6.0, 4.0, 5.0], [4.0, 1.0, 8.0]], 
[[1.0, 8.0, 5.0], [5.0, 3.0, 1.0], [5.0, 2.0, 1.0]], [[6.0, 1.0, 3.0], [5.0, 5.0, 5.0], [5.0, 3.0, 8.0]]], [[[7.0, 7.0, 7.0], 
[7.0, 3.0, 7.0], [4.0, 6.0, 4.0]], [[4.0, 1.0, 5.0], [8.0, 7.0, 1.0], [1.0, 4.0, 6.0]], [[3.0, 6.0, 3.0], [6.0, 4.0, 5.0], 
[7.0, 8.0, 1.0]]], [[[4.0, 1.0, 4.0], [4.0, 4.0, 8.0], [5.0, 1.0, 4.0]], [[8.0, 5.0, 7.0], [2.0, 4.0, 7.0], [7.0, 5.0, 4.0]], 
[[5.0, 3.0, 3.0], [2.0, 4.0, 8.0], [6.0, 4.0, 7.0]]], [[[6.0, 4.0, 1.0], [3.0, 4.0, 7.0], [2.0, 1.0, 2.0]], [[2.0, 2.0, 1.0], 
[5.0, 8.0, 4.0], [1.0, 4.0, 5.0]], [[8.0, 2.0, 5.0], [1.0, 4.0, 7.0], [6.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_URUL(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 7.0], [2.0, 5.0, 8.0], [8.0, 4.0, 2.0]], [[4.0, 4.0, 8.0], [5.0, 3.0, 5.0], [3.0, 2.0, 
7.0]], [[8.0, 6.0, 1.0], [6.0, 5.0, 8.0], [4.0, 5.0, 7.0]]], [[[3.0, 2.0, 5.0], [8.0, 4.0, 5.0], [3.0, 5.0, 5.0]], [[2.0, 6.0, 
8.0], [3.0, 4.0, 5.0], [3.0, 6.0, 8.0]], [[7.0, 7.0, 8.0], [5.0, 3.0, 2.0], [8.0, 6.0, 2.0]]], [[[3.0, 5.0, 8.0], [8.0, 4.0, 
4.0], [5.0, 7.0, 4.0]], [[6.0, 7.0, 4.0], [5.0, 7.0, 4.0], [3.0, 8.0, 5.0]], [[2.0, 8.0, 3.0], [6.0, 3.0, 4.0], [4.0, 5.0, 
1.0]]], [[[4.0, 4.0, 6.0], [5.0, 5.0, 4.0], [7.0, 3.0, 3.0]], [[1.0, 2.0, 3.0], [1.0, 7.0, 7.0], [5.0, 6.0, 6.0]], [[7.0, 6.0, 
5.0], [5.0, 8.0, 1.0], [1.0, 4.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 4.0], [1.0, 4.0, 3.0], [6.0, 6.0, 5.0]], [[6.0, 4.0, 2.0], [2.0, 1.0, 
7.0], [6.0, 1.0, 3.0]], [[2.0, 2.0, 7.0], [5.0, 2.0, 5.0], [6.0, 6.0, 5.0]]], [[[2.0, 4.0, 7.0], [4.0, 4.0, 1.0], [2.0, 8.0, 
8.0]], [[6.0, 5.0, 7.0], [2.0, 1.0, 6.0], [8.0, 5.0, 3.0]], [[4.0, 5.0, 6.0], [3.0, 6.0, 6.0], [4.0, 8.0, 5.0]]], [[[2.0, 5.0, 
8.0], [2.0, 2.0, 5.0], [3.0, 2.0, 7.0]], [[1.0, 7.0, 8.0], [1.0, 8.0, 1.0], [7.0, 4.0, 7.0]], [[7.0, 8.0, 7.0], [3.0, 4.0, 
6.0], [3.0, 7.0, 2.0]]], [[[2.0, 1.0, 7.0], [4.0, 1.0, 2.0], [2.0, 5.0, 8.0]], [[3.0, 4.0, 8.0], [6.0, 1.0, 2.0], [8.0, 4.0, 
1.0]], [[6.0, 2.0, 5.0], [5.0, 1.0, 7.0], [6.0, 6.0, 4.0]]]]))
      val=Data(numpy.array([[[[5.0], [4.0]]], [[[5.0], [3.0]]], [[[6.0], [2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0], [5.0]]], [[[3.0], [3.0]]], [[[8.0], [2.0]]]]))
      arg[:3,1:2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 7.0], [2.0, 5.0, 8.0], [8.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [5.0, 3.0, 4.0], [3.0, 
2.0, 7.0]], [[8.0, 6.0, 1.0], [6.0, 5.0, 8.0], [4.0, 5.0, 7.0]]], [[[3.0, 2.0, 5.0], [8.0, 4.0, 5.0], [3.0, 5.0, 5.0]], [[2.0, 
6.0, 5.0], [3.0, 4.0, 3.0], [3.0, 6.0, 8.0]], [[7.0, 7.0, 8.0], [5.0, 3.0, 2.0], [8.0, 6.0, 2.0]]], [[[3.0, 5.0, 8.0], [8.0, 
4.0, 4.0], [5.0, 7.0, 4.0]], [[6.0, 7.0, 6.0], [5.0, 7.0, 2.0], [3.0, 8.0, 5.0]], [[2.0, 8.0, 3.0], [6.0, 3.0, 4.0], [4.0, 5.0, 
1.0]]], [[[4.0, 4.0, 6.0], [5.0, 5.0, 4.0], [7.0, 3.0, 3.0]], [[1.0, 2.0, 3.0], [1.0, 7.0, 7.0], [5.0, 6.0, 6.0]], [[7.0, 6.0, 
5.0], [5.0, 8.0, 1.0], [1.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 4.0], [1.0, 4.0, 3.0], [6.0, 6.0, 5.0]], 
[[6.0, 4.0, 8.0], [2.0, 1.0, 5.0], [6.0, 1.0, 3.0]], [[2.0, 2.0, 7.0], [5.0, 2.0, 5.0], [6.0, 6.0, 5.0]]], [[[2.0, 4.0, 7.0], 
[4.0, 4.0, 1.0], [2.0, 8.0, 8.0]], [[6.0, 5.0, 3.0], [2.0, 1.0, 3.0], [8.0, 5.0, 3.0]], [[4.0, 5.0, 6.0], [3.0, 6.0, 6.0], 
[4.0, 8.0, 5.0]]], [[[2.0, 5.0, 8.0], [2.0, 2.0, 5.0], [3.0, 2.0, 7.0]], [[1.0, 7.0, 8.0], [1.0, 8.0, 2.0], [7.0, 4.0, 7.0]], 
[[7.0, 8.0, 7.0], [3.0, 4.0, 6.0], [3.0, 7.0, 2.0]]], [[[2.0, 1.0, 7.0], [4.0, 1.0, 2.0], [2.0, 5.0, 8.0]], [[3.0, 4.0, 8.0], 
[6.0, 1.0, 2.0], [8.0, 4.0, 1.0]], [[6.0, 2.0, 5.0], [5.0, 1.0, 7.0], [6.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_URUL(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 6.0], [1.0, 5.0, 3.0], [6.0, 6.0, 1.0]], [[4.0, 7.0, 4.0], [7.0, 7.0, 3.0], [8.0, 1.0, 
1.0]], [[4.0, 8.0, 4.0], [3.0, 2.0, 8.0], [7.0, 3.0, 8.0]]], [[[8.0, 3.0, 3.0], [1.0, 8.0, 4.0], [3.0, 4.0, 8.0]], [[8.0, 2.0, 
6.0], [6.0, 1.0, 3.0], [8.0, 3.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 6.0, 4.0], [4.0, 4.0, 4.0]]], [[[2.0, 7.0, 1.0], [4.0, 5.0, 
8.0], [8.0, 2.0, 1.0]], [[3.0, 4.0, 6.0], [5.0, 1.0, 7.0], [2.0, 3.0, 2.0]], [[5.0, 3.0, 5.0], [7.0, 4.0, 6.0], [7.0, 2.0, 
5.0]]], [[[7.0, 5.0, 6.0], [3.0, 3.0, 4.0], [8.0, 8.0, 6.0]], [[3.0, 5.0, 5.0], [6.0, 8.0, 6.0], [2.0, 5.0, 1.0]], [[4.0, 5.0, 
6.0], [3.0, 1.0, 6.0], [2.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 6.0], [4.0, 3.0, 4.0], [1.0, 4.0, 5.0]], [[4.0, 5.0, 1.0], [2.0, 2.0, 
5.0], [8.0, 5.0, 6.0]], [[5.0, 7.0, 1.0], [4.0, 6.0, 1.0], [2.0, 6.0, 6.0]]], [[[2.0, 2.0, 5.0], [3.0, 5.0, 4.0], [7.0, 2.0, 
2.0]], [[5.0, 1.0, 1.0], [5.0, 4.0, 2.0], [2.0, 3.0, 3.0]], [[4.0, 5.0, 2.0], [2.0, 1.0, 6.0], [8.0, 1.0, 4.0]]], [[[5.0, 3.0, 
3.0], [4.0, 6.0, 3.0], [8.0, 8.0, 3.0]], [[1.0, 4.0, 7.0], [3.0, 7.0, 5.0], [3.0, 7.0, 4.0]], [[3.0, 6.0, 5.0], [5.0, 5.0, 
6.0], [5.0, 7.0, 2.0]]], [[[3.0, 8.0, 8.0], [1.0, 4.0, 5.0], [7.0, 7.0, 6.0]], [[4.0, 2.0, 5.0], [7.0, 6.0, 7.0], [5.0, 2.0, 
5.0]], [[3.0, 3.0, 1.0], [4.0, 1.0, 3.0], [1.0, 8.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0], [2.0]]], [[[8.0], [1.0]]], [[[8.0], [7.0]]]])+(1.-msk_val)*numpy.array([[[[4.0], 
[8.0]]], [[[2.0], [1.0]]], [[[6.0], [5.0]]]])
      arg[:3,1:2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 6.0], [1.0, 5.0, 3.0], [6.0, 6.0, 1.0]], [[4.0, 7.0, 3.0], [7.0, 7.0, 2.0], [8.0, 
1.0, 1.0]], [[4.0, 8.0, 4.0], [3.0, 2.0, 8.0], [7.0, 3.0, 8.0]]], [[[8.0, 3.0, 3.0], [1.0, 8.0, 4.0], [3.0, 4.0, 8.0]], [[8.0, 
2.0, 8.0], [6.0, 1.0, 1.0], [8.0, 3.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 6.0, 4.0], [4.0, 4.0, 4.0]]], [[[2.0, 7.0, 1.0], [4.0, 
5.0, 8.0], [8.0, 2.0, 1.0]], [[3.0, 4.0, 8.0], [5.0, 1.0, 7.0], [2.0, 3.0, 2.0]], [[5.0, 3.0, 5.0], [7.0, 4.0, 6.0], [7.0, 2.0, 
5.0]]], [[[7.0, 5.0, 6.0], [3.0, 3.0, 4.0], [8.0, 8.0, 6.0]], [[3.0, 5.0, 5.0], [6.0, 8.0, 6.0], [2.0, 5.0, 1.0]], [[4.0, 5.0, 
6.0], [3.0, 1.0, 6.0], [2.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 6.0], [4.0, 3.0, 4.0], [1.0, 4.0, 5.0]], 
[[4.0, 5.0, 4.0], [2.0, 2.0, 8.0], [8.0, 5.0, 6.0]], [[5.0, 7.0, 1.0], [4.0, 6.0, 1.0], [2.0, 6.0, 6.0]]], [[[2.0, 2.0, 5.0], 
[3.0, 5.0, 4.0], [7.0, 2.0, 2.0]], [[5.0, 1.0, 2.0], [5.0, 4.0, 1.0], [2.0, 3.0, 3.0]], [[4.0, 5.0, 2.0], [2.0, 1.0, 6.0], 
[8.0, 1.0, 4.0]]], [[[5.0, 3.0, 3.0], [4.0, 6.0, 3.0], [8.0, 8.0, 3.0]], [[1.0, 4.0, 6.0], [3.0, 7.0, 5.0], [3.0, 7.0, 4.0]], 
[[3.0, 6.0, 5.0], [5.0, 5.0, 6.0], [5.0, 7.0, 2.0]]], [[[3.0, 8.0, 8.0], [1.0, 4.0, 5.0], [7.0, 7.0, 6.0]], [[4.0, 2.0, 5.0], 
[7.0, 6.0, 7.0], [5.0, 2.0, 5.0]], [[3.0, 3.0, 1.0], [4.0, 1.0, 3.0], [1.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LPLN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 5.0], [3.0, 3.0, 3.0], [4.0, 6.0, 4.0]], [[1.0, 8.0, 2.0], [6.0, 2.0, 2.0], [4.0, 2.0, 
2.0]], [[7.0, 6.0, 8.0], [5.0, 6.0, 4.0], [5.0, 3.0, 4.0]]], [[[8.0, 2.0, 2.0], [5.0, 5.0, 5.0], [5.0, 5.0, 3.0]], [[1.0, 4.0, 
2.0], [4.0, 1.0, 8.0], [4.0, 4.0, 7.0]], [[5.0, 6.0, 4.0], [4.0, 1.0, 8.0], [6.0, 2.0, 6.0]]], [[[1.0, 7.0, 6.0], [1.0, 3.0, 
6.0], [7.0, 5.0, 4.0]], [[2.0, 7.0, 3.0], [5.0, 4.0, 1.0], [2.0, 6.0, 6.0]], [[7.0, 7.0, 7.0], [1.0, 3.0, 8.0], [7.0, 8.0, 
1.0]]], [[[5.0, 2.0, 5.0], [4.0, 1.0, 3.0], [3.0, 6.0, 2.0]], [[7.0, 5.0, 6.0], [6.0, 2.0, 4.0], [4.0, 2.0, 7.0]], [[7.0, 1.0, 
8.0], [6.0, 4.0, 7.0], [1.0, 5.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 6.0], [5.0, 2.0, 3.0], [6.0, 7.0, 4.0]], [[8.0, 3.0, 7.0], [1.0, 8.0, 
2.0], [3.0, 4.0, 5.0]], [[3.0, 2.0, 8.0], [8.0, 2.0, 2.0], [8.0, 8.0, 3.0]]], [[[6.0, 1.0, 2.0], [2.0, 4.0, 5.0], [8.0, 7.0, 
2.0]], [[8.0, 3.0, 5.0], [3.0, 2.0, 4.0], [4.0, 7.0, 2.0]], [[3.0, 4.0, 4.0], [8.0, 3.0, 1.0], [8.0, 2.0, 2.0]]], [[[6.0, 1.0, 
4.0], [5.0, 3.0, 4.0], [7.0, 4.0, 5.0]], [[4.0, 4.0, 5.0], [7.0, 1.0, 2.0], [4.0, 4.0, 1.0]], [[5.0, 2.0, 2.0], [2.0, 1.0, 
4.0], [6.0, 8.0, 5.0]]], [[[4.0, 4.0, 4.0], [4.0, 6.0, 3.0], [8.0, 8.0, 7.0]], [[6.0, 3.0, 6.0], [5.0, 5.0, 5.0], [7.0, 4.0, 
1.0]], [[6.0, 1.0, 8.0], [4.0, 1.0, 8.0], [7.0, 7.0, 7.0]]]]))
      val=numpy.array([[[1.0, 2.0, 8.0]], [[4.0, 3.0, 6.0]]])
      arg[2:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 5.0], [3.0, 3.0, 3.0], [4.0, 6.0, 4.0]], [[1.0, 8.0, 2.0], [6.0, 2.0, 2.0], [4.0, 
2.0, 2.0]], [[7.0, 6.0, 8.0], [5.0, 6.0, 4.0], [5.0, 3.0, 4.0]]], [[[8.0, 2.0, 2.0], [5.0, 5.0, 5.0], [5.0, 5.0, 3.0]], [[1.0, 
4.0, 2.0], [4.0, 1.0, 8.0], [4.0, 4.0, 7.0]], [[5.0, 6.0, 4.0], [4.0, 1.0, 8.0], [6.0, 2.0, 6.0]]], [[[1.0, 7.0, 6.0], [1.0, 
3.0, 6.0], [7.0, 5.0, 4.0]], [[2.0, 7.0, 3.0], [5.0, 4.0, 1.0], [1.0, 2.0, 8.0]], [[7.0, 7.0, 7.0], [1.0, 3.0, 8.0], [7.0, 8.0, 
1.0]]], [[[5.0, 2.0, 5.0], [4.0, 1.0, 3.0], [3.0, 6.0, 2.0]], [[7.0, 5.0, 6.0], [6.0, 2.0, 4.0], [4.0, 3.0, 6.0]], [[7.0, 1.0, 
8.0], [6.0, 4.0, 7.0], [1.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 6.0], [5.0, 2.0, 3.0], [6.0, 7.0, 4.0]], 
[[8.0, 3.0, 7.0], [1.0, 8.0, 2.0], [3.0, 4.0, 5.0]], [[3.0, 2.0, 8.0], [8.0, 2.0, 2.0], [8.0, 8.0, 3.0]]], [[[6.0, 1.0, 2.0], 
[2.0, 4.0, 5.0], [8.0, 7.0, 2.0]], [[8.0, 3.0, 5.0], [3.0, 2.0, 4.0], [4.0, 7.0, 2.0]], [[3.0, 4.0, 4.0], [8.0, 3.0, 1.0], 
[8.0, 2.0, 2.0]]], [[[6.0, 1.0, 4.0], [5.0, 3.0, 4.0], [7.0, 4.0, 5.0]], [[4.0, 4.0, 5.0], [7.0, 1.0, 2.0], [1.0, 2.0, 8.0]], 
[[5.0, 2.0, 2.0], [2.0, 1.0, 4.0], [6.0, 8.0, 5.0]]], [[[4.0, 4.0, 4.0], [4.0, 6.0, 3.0], [8.0, 8.0, 7.0]], [[6.0, 3.0, 6.0], 
[5.0, 5.0, 5.0], [4.0, 3.0, 6.0]], [[6.0, 1.0, 8.0], [4.0, 1.0, 8.0], [7.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LPLN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 1.0], [8.0, 8.0, 7.0], [8.0, 5.0, 7.0]], [[7.0, 7.0, 4.0], [7.0, 8.0, 5.0], [6.0, 5.0, 
3.0]], [[4.0, 2.0, 3.0], [2.0, 4.0, 2.0], [5.0, 4.0, 4.0]]], [[[2.0, 8.0, 2.0], [8.0, 5.0, 7.0], [5.0, 6.0, 6.0]], [[8.0, 5.0, 
7.0], [7.0, 4.0, 4.0], [5.0, 2.0, 2.0]], [[3.0, 2.0, 5.0], [2.0, 4.0, 4.0], [5.0, 7.0, 1.0]]], [[[3.0, 4.0, 4.0], [1.0, 6.0, 
1.0], [4.0, 3.0, 1.0]], [[5.0, 8.0, 1.0], [7.0, 3.0, 1.0], [7.0, 1.0, 4.0]], [[1.0, 5.0, 3.0], [4.0, 6.0, 8.0], [1.0, 1.0, 
8.0]]], [[[6.0, 5.0, 3.0], [7.0, 7.0, 1.0], [8.0, 3.0, 8.0]], [[6.0, 8.0, 6.0], [3.0, 8.0, 3.0], [4.0, 6.0, 2.0]], [[8.0, 5.0, 
1.0], [8.0, 2.0, 4.0], [3.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 3.0], [1.0, 6.0, 5.0], [4.0, 7.0, 7.0]], [[4.0, 2.0, 6.0], [7.0, 4.0, 
2.0], [5.0, 2.0, 7.0]], [[3.0, 2.0, 5.0], [8.0, 2.0, 4.0], [6.0, 2.0, 7.0]]], [[[5.0, 2.0, 6.0], [5.0, 7.0, 8.0], [8.0, 6.0, 
3.0]], [[3.0, 5.0, 8.0], [2.0, 6.0, 4.0], [5.0, 2.0, 3.0]], [[6.0, 7.0, 7.0], [1.0, 4.0, 1.0], [6.0, 3.0, 5.0]]], [[[6.0, 4.0, 
8.0], [1.0, 3.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 2.0, 2.0], [3.0, 5.0, 3.0], [8.0, 6.0, 4.0]], [[5.0, 5.0, 7.0], [1.0, 7.0, 
6.0], [5.0, 1.0, 1.0]]], [[[3.0, 6.0, 1.0], [7.0, 8.0, 1.0], [1.0, 4.0, 7.0]], [[5.0, 5.0, 2.0], [8.0, 8.0, 7.0], [3.0, 7.0, 
7.0]], [[3.0, 4.0, 5.0], [8.0, 8.0, 7.0], [1.0, 6.0, 8.0]]]]))
      val=Data(numpy.array([[[5.0, 1.0, 8.0]], [[6.0, 1.0, 2.0]]]),self.functionspace)
      arg[2:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 1.0], [8.0, 8.0, 7.0], [8.0, 5.0, 7.0]], [[7.0, 7.0, 4.0], [7.0, 8.0, 5.0], [6.0, 
5.0, 3.0]], [[4.0, 2.0, 3.0], [2.0, 4.0, 2.0], [5.0, 4.0, 4.0]]], [[[2.0, 8.0, 2.0], [8.0, 5.0, 7.0], [5.0, 6.0, 6.0]], [[8.0, 
5.0, 7.0], [7.0, 4.0, 4.0], [5.0, 2.0, 2.0]], [[3.0, 2.0, 5.0], [2.0, 4.0, 4.0], [5.0, 7.0, 1.0]]], [[[3.0, 4.0, 4.0], [1.0, 
6.0, 1.0], [4.0, 3.0, 1.0]], [[5.0, 8.0, 1.0], [7.0, 3.0, 1.0], [5.0, 1.0, 8.0]], [[1.0, 5.0, 3.0], [4.0, 6.0, 8.0], [1.0, 1.0, 
8.0]]], [[[6.0, 5.0, 3.0], [7.0, 7.0, 1.0], [8.0, 3.0, 8.0]], [[6.0, 8.0, 6.0], [3.0, 8.0, 3.0], [6.0, 1.0, 2.0]], [[8.0, 5.0, 
1.0], [8.0, 2.0, 4.0], [3.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 3.0], [1.0, 6.0, 5.0], [4.0, 7.0, 7.0]], 
[[4.0, 2.0, 6.0], [7.0, 4.0, 2.0], [5.0, 2.0, 7.0]], [[3.0, 2.0, 5.0], [8.0, 2.0, 4.0], [6.0, 2.0, 7.0]]], [[[5.0, 2.0, 6.0], 
[5.0, 7.0, 8.0], [8.0, 6.0, 3.0]], [[3.0, 5.0, 8.0], [2.0, 6.0, 4.0], [5.0, 2.0, 3.0]], [[6.0, 7.0, 7.0], [1.0, 4.0, 1.0], 
[6.0, 3.0, 5.0]]], [[[6.0, 4.0, 8.0], [1.0, 3.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 2.0, 2.0], [3.0, 5.0, 3.0], [5.0, 1.0, 8.0]], 
[[5.0, 5.0, 7.0], [1.0, 7.0, 6.0], [5.0, 1.0, 1.0]]], [[[3.0, 6.0, 1.0], [7.0, 8.0, 1.0], [1.0, 4.0, 7.0]], [[5.0, 5.0, 2.0], 
[8.0, 8.0, 7.0], [6.0, 1.0, 2.0]], [[3.0, 4.0, 5.0], [8.0, 8.0, 7.0], [1.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LPLN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 5.0], [6.0, 4.0, 7.0], [5.0, 8.0, 5.0]], [[5.0, 4.0, 6.0], [7.0, 3.0, 2.0], [8.0, 1.0, 
3.0]], [[5.0, 7.0, 2.0], [6.0, 1.0, 8.0], [1.0, 3.0, 2.0]]], [[[5.0, 6.0, 1.0], [1.0, 4.0, 4.0], [6.0, 3.0, 1.0]], [[6.0, 4.0, 
3.0], [7.0, 6.0, 4.0], [5.0, 8.0, 4.0]], [[4.0, 5.0, 1.0], [3.0, 8.0, 7.0], [7.0, 8.0, 8.0]]], [[[3.0, 1.0, 1.0], [4.0, 1.0, 
8.0], [7.0, 2.0, 2.0]], [[2.0, 3.0, 7.0], [6.0, 6.0, 4.0], [3.0, 4.0, 8.0]], [[5.0, 7.0, 3.0], [7.0, 3.0, 1.0], [2.0, 3.0, 
7.0]]], [[[2.0, 2.0, 8.0], [7.0, 5.0, 4.0], [6.0, 2.0, 5.0]], [[5.0, 4.0, 4.0], [5.0, 8.0, 8.0], [5.0, 6.0, 2.0]], [[2.0, 8.0, 
5.0], [2.0, 4.0, 4.0], [8.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 4.0], [2.0, 8.0, 7.0], [1.0, 8.0, 8.0]], [[3.0, 4.0, 4.0], [3.0, 3.0, 
6.0], [3.0, 2.0, 7.0]], [[6.0, 1.0, 1.0], [6.0, 2.0, 3.0], [1.0, 4.0, 4.0]]], [[[7.0, 5.0, 2.0], [5.0, 4.0, 6.0], [3.0, 6.0, 
4.0]], [[2.0, 5.0, 6.0], [3.0, 2.0, 1.0], [3.0, 6.0, 4.0]], [[8.0, 4.0, 4.0], [1.0, 4.0, 2.0], [3.0, 7.0, 5.0]]], [[[2.0, 4.0, 
7.0], [7.0, 7.0, 1.0], [3.0, 2.0, 3.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 5.0], [1.0, 4.0, 6.0]], [[3.0, 6.0, 8.0], [3.0, 2.0, 
2.0], [5.0, 1.0, 8.0]]], [[[7.0, 1.0, 4.0], [8.0, 6.0, 7.0], [3.0, 1.0, 6.0]], [[8.0, 3.0, 5.0], [2.0, 5.0, 5.0], [7.0, 5.0, 
7.0]], [[3.0, 1.0, 2.0], [4.0, 4.0, 8.0], [3.0, 4.0, 3.0]]]]))
      val=Data(numpy.array([[[5.0, 6.0, 2.0]], [[4.0, 7.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 3.0, 3.0]], [[3.0, 8.0, 2.0]]]))
      arg[2:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 5.0], [6.0, 4.0, 7.0], [5.0, 8.0, 5.0]], [[5.0, 4.0, 6.0], [7.0, 3.0, 2.0], [8.0, 
1.0, 3.0]], [[5.0, 7.0, 2.0], [6.0, 1.0, 8.0], [1.0, 3.0, 2.0]]], [[[5.0, 6.0, 1.0], [1.0, 4.0, 4.0], [6.0, 3.0, 1.0]], [[6.0, 
4.0, 3.0], [7.0, 6.0, 4.0], [5.0, 8.0, 4.0]], [[4.0, 5.0, 1.0], [3.0, 8.0, 7.0], [7.0, 8.0, 8.0]]], [[[3.0, 1.0, 1.0], [4.0, 
1.0, 8.0], [7.0, 2.0, 2.0]], [[2.0, 3.0, 7.0], [6.0, 6.0, 4.0], [5.0, 6.0, 2.0]], [[5.0, 7.0, 3.0], [7.0, 3.0, 1.0], [2.0, 3.0, 
7.0]]], [[[2.0, 2.0, 8.0], [7.0, 5.0, 4.0], [6.0, 2.0, 5.0]], [[5.0, 4.0, 4.0], [5.0, 8.0, 8.0], [4.0, 7.0, 5.0]], [[2.0, 8.0, 
5.0], [2.0, 4.0, 4.0], [8.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 4.0], [2.0, 8.0, 7.0], [1.0, 8.0, 8.0]], 
[[3.0, 4.0, 4.0], [3.0, 3.0, 6.0], [3.0, 2.0, 7.0]], [[6.0, 1.0, 1.0], [6.0, 2.0, 3.0], [1.0, 4.0, 4.0]]], [[[7.0, 5.0, 2.0], 
[5.0, 4.0, 6.0], [3.0, 6.0, 4.0]], [[2.0, 5.0, 6.0], [3.0, 2.0, 1.0], [3.0, 6.0, 4.0]], [[8.0, 4.0, 4.0], [1.0, 4.0, 2.0], 
[3.0, 7.0, 5.0]]], [[[2.0, 4.0, 7.0], [7.0, 7.0, 1.0], [3.0, 2.0, 3.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 5.0], [5.0, 3.0, 3.0]], 
[[3.0, 6.0, 8.0], [3.0, 2.0, 2.0], [5.0, 1.0, 8.0]]], [[[7.0, 1.0, 4.0], [8.0, 6.0, 7.0], [3.0, 1.0, 6.0]], [[8.0, 3.0, 5.0], 
[2.0, 5.0, 5.0], [3.0, 8.0, 2.0]], [[3.0, 1.0, 2.0], [4.0, 4.0, 8.0], [3.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LPLN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 8.0], [2.0, 6.0, 5.0], [8.0, 6.0, 6.0]], [[4.0, 4.0, 5.0], [6.0, 1.0, 8.0], [5.0, 5.0, 
5.0]], [[5.0, 1.0, 4.0], [1.0, 8.0, 5.0], [1.0, 7.0, 3.0]]], [[[7.0, 1.0, 2.0], [5.0, 2.0, 8.0], [3.0, 6.0, 8.0]], [[7.0, 2.0, 
4.0], [3.0, 6.0, 5.0], [5.0, 3.0, 4.0]], [[2.0, 1.0, 3.0], [5.0, 8.0, 2.0], [3.0, 8.0, 1.0]]], [[[6.0, 8.0, 7.0], [3.0, 8.0, 
6.0], [6.0, 3.0, 5.0]], [[4.0, 7.0, 2.0], [3.0, 2.0, 7.0], [4.0, 2.0, 4.0]], [[5.0, 4.0, 1.0], [8.0, 4.0, 8.0], [1.0, 3.0, 
6.0]]], [[[1.0, 5.0, 6.0], [8.0, 4.0, 1.0], [7.0, 4.0, 7.0]], [[1.0, 7.0, 8.0], [1.0, 7.0, 2.0], [3.0, 6.0, 4.0]], [[3.0, 3.0, 
5.0], [4.0, 1.0, 7.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 8.0], [5.0, 5.0, 4.0], [6.0, 1.0, 3.0]], [[2.0, 7.0, 5.0], [8.0, 2.0, 
4.0], [4.0, 4.0, 6.0]], [[7.0, 6.0, 1.0], [4.0, 2.0, 3.0], [1.0, 7.0, 8.0]]], [[[1.0, 5.0, 7.0], [1.0, 6.0, 3.0], [4.0, 1.0, 
6.0]], [[4.0, 7.0, 2.0], [4.0, 2.0, 4.0], [6.0, 8.0, 4.0]], [[4.0, 3.0, 8.0], [5.0, 7.0, 1.0], [3.0, 8.0, 2.0]]], [[[1.0, 5.0, 
7.0], [7.0, 7.0, 7.0], [2.0, 8.0, 8.0]], [[6.0, 7.0, 7.0], [5.0, 6.0, 7.0], [3.0, 6.0, 4.0]], [[3.0, 5.0, 3.0], [2.0, 2.0, 
2.0], [5.0, 8.0, 6.0]]], [[[8.0, 6.0, 4.0], [4.0, 3.0, 3.0], [3.0, 1.0, 2.0]], [[2.0, 1.0, 7.0], [7.0, 3.0, 1.0], [4.0, 6.0, 
8.0]], [[8.0, 6.0, 1.0], [6.0, 2.0, 6.0], [2.0, 4.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 1.0, 1.0]], [[4.0, 8.0, 5.0]]])+(1.-msk_val)*numpy.array([[[5.0, 7.0, 8.0]], [[8.0, 8.0, 
6.0]]])
      arg[2:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [2.0, 6.0, 5.0], [8.0, 6.0, 6.0]], [[4.0, 4.0, 5.0], [6.0, 1.0, 8.0], [5.0, 
5.0, 5.0]], [[5.0, 1.0, 4.0], [1.0, 8.0, 5.0], [1.0, 7.0, 3.0]]], [[[7.0, 1.0, 2.0], [5.0, 2.0, 8.0], [3.0, 6.0, 8.0]], [[7.0, 
2.0, 4.0], [3.0, 6.0, 5.0], [5.0, 3.0, 4.0]], [[2.0, 1.0, 3.0], [5.0, 8.0, 2.0], [3.0, 8.0, 1.0]]], [[[6.0, 8.0, 7.0], [3.0, 
8.0, 6.0], [6.0, 3.0, 5.0]], [[4.0, 7.0, 2.0], [3.0, 2.0, 7.0], [7.0, 1.0, 1.0]], [[5.0, 4.0, 1.0], [8.0, 4.0, 8.0], [1.0, 3.0, 
6.0]]], [[[1.0, 5.0, 6.0], [8.0, 4.0, 1.0], [7.0, 4.0, 7.0]], [[1.0, 7.0, 8.0], [1.0, 7.0, 2.0], [4.0, 8.0, 5.0]], [[3.0, 3.0, 
5.0], [4.0, 1.0, 7.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 8.0], [5.0, 5.0, 4.0], [6.0, 1.0, 3.0]], 
[[2.0, 7.0, 5.0], [8.0, 2.0, 4.0], [4.0, 4.0, 6.0]], [[7.0, 6.0, 1.0], [4.0, 2.0, 3.0], [1.0, 7.0, 8.0]]], [[[1.0, 5.0, 7.0], 
[1.0, 6.0, 3.0], [4.0, 1.0, 6.0]], [[4.0, 7.0, 2.0], [4.0, 2.0, 4.0], [6.0, 8.0, 4.0]], [[4.0, 3.0, 8.0], [5.0, 7.0, 1.0], 
[3.0, 8.0, 2.0]]], [[[1.0, 5.0, 7.0], [7.0, 7.0, 7.0], [2.0, 8.0, 8.0]], [[6.0, 7.0, 7.0], [5.0, 6.0, 7.0], [5.0, 7.0, 8.0]], 
[[3.0, 5.0, 3.0], [2.0, 2.0, 2.0], [5.0, 8.0, 6.0]]], [[[8.0, 6.0, 4.0], [4.0, 3.0, 3.0], [3.0, 1.0, 2.0]], [[2.0, 1.0, 7.0], 
[7.0, 3.0, 1.0], [8.0, 8.0, 6.0]], [[8.0, 6.0, 1.0], [6.0, 2.0, 6.0], [2.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPPP(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 3.0], [8.0, 2.0, 3.0], [1.0, 4.0, 7.0]], [[5.0, 5.0, 1.0], [4.0, 4.0, 5.0], [8.0, 4.0, 
8.0]], [[2.0, 6.0, 4.0], [4.0, 6.0, 6.0], [5.0, 1.0, 3.0]]], [[[7.0, 3.0, 8.0], [4.0, 5.0, 1.0], [7.0, 1.0, 5.0]], [[1.0, 6.0, 
1.0], [8.0, 2.0, 8.0], [1.0, 2.0, 3.0]], [[6.0, 8.0, 7.0], [6.0, 7.0, 2.0], [4.0, 3.0, 6.0]]], [[[7.0, 6.0, 1.0], [3.0, 2.0, 
1.0], [7.0, 1.0, 7.0]], [[7.0, 7.0, 6.0], [6.0, 4.0, 3.0], [7.0, 7.0, 1.0]], [[7.0, 5.0, 2.0], [5.0, 2.0, 3.0], [1.0, 5.0, 
4.0]]], [[[2.0, 6.0, 8.0], [5.0, 1.0, 2.0], [7.0, 5.0, 7.0]], [[2.0, 2.0, 6.0], [1.0, 5.0, 7.0], [5.0, 2.0, 2.0]], [[1.0, 6.0, 
8.0], [7.0, 8.0, 2.0], [8.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 5.0], [7.0, 1.0, 7.0], [6.0, 7.0, 7.0]], [[4.0, 3.0, 1.0], [4.0, 6.0, 
1.0], [4.0, 6.0, 6.0]], [[7.0, 6.0, 3.0], [6.0, 5.0, 8.0], [8.0, 5.0, 6.0]]], [[[3.0, 4.0, 7.0], [1.0, 5.0, 1.0], [7.0, 3.0, 
6.0]], [[6.0, 3.0, 1.0], [1.0, 8.0, 5.0], [6.0, 7.0, 7.0]], [[8.0, 5.0, 6.0], [6.0, 4.0, 6.0], [5.0, 6.0, 3.0]]], [[[4.0, 8.0, 
4.0], [5.0, 7.0, 1.0], [8.0, 1.0, 3.0]], [[6.0, 1.0, 8.0], [7.0, 3.0, 3.0], [5.0, 7.0, 3.0]], [[4.0, 8.0, 8.0], [6.0, 5.0, 
7.0], [8.0, 4.0, 6.0]]], [[[3.0, 8.0, 8.0], [2.0, 4.0, 2.0], [1.0, 4.0, 5.0]], [[5.0, 1.0, 6.0], [6.0, 4.0, 7.0], [6.0, 7.0, 
3.0]], [[5.0, 1.0, 5.0], [7.0, 2.0, 2.0], [8.0, 2.0, 2.0]]]]))
      val=numpy.array([1.0, 2.0, 3.0, 4.0])
      arg[:,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 3.0], [8.0, 2.0, 3.0], [1.0, 4.0, 7.0]], [[5.0, 5.0, 1.0], [4.0, 1.0, 5.0], [8.0, 
4.0, 8.0]], [[2.0, 6.0, 4.0], [4.0, 6.0, 6.0], [5.0, 1.0, 3.0]]], [[[7.0, 3.0, 8.0], [4.0, 5.0, 1.0], [7.0, 1.0, 5.0]], [[1.0, 
6.0, 1.0], [8.0, 2.0, 8.0], [1.0, 2.0, 3.0]], [[6.0, 8.0, 7.0], [6.0, 7.0, 2.0], [4.0, 3.0, 6.0]]], [[[7.0, 6.0, 1.0], [3.0, 
2.0, 1.0], [7.0, 1.0, 7.0]], [[7.0, 7.0, 6.0], [6.0, 3.0, 3.0], [7.0, 7.0, 1.0]], [[7.0, 5.0, 2.0], [5.0, 2.0, 3.0], [1.0, 5.0, 
4.0]]], [[[2.0, 6.0, 8.0], [5.0, 1.0, 2.0], [7.0, 5.0, 7.0]], [[2.0, 2.0, 6.0], [1.0, 4.0, 7.0], [5.0, 2.0, 2.0]], [[1.0, 6.0, 
8.0], [7.0, 8.0, 2.0], [8.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 5.0], [7.0, 1.0, 7.0], [6.0, 7.0, 7.0]], 
[[4.0, 3.0, 1.0], [4.0, 1.0, 1.0], [4.0, 6.0, 6.0]], [[7.0, 6.0, 3.0], [6.0, 5.0, 8.0], [8.0, 5.0, 6.0]]], [[[3.0, 4.0, 7.0], 
[1.0, 5.0, 1.0], [7.0, 3.0, 6.0]], [[6.0, 3.0, 1.0], [1.0, 2.0, 5.0], [6.0, 7.0, 7.0]], [[8.0, 5.0, 6.0], [6.0, 4.0, 6.0], 
[5.0, 6.0, 3.0]]], [[[4.0, 8.0, 4.0], [5.0, 7.0, 1.0], [8.0, 1.0, 3.0]], [[6.0, 1.0, 8.0], [7.0, 3.0, 3.0], [5.0, 7.0, 3.0]], 
[[4.0, 8.0, 8.0], [6.0, 5.0, 7.0], [8.0, 4.0, 6.0]]], [[[3.0, 8.0, 8.0], [2.0, 4.0, 2.0], [1.0, 4.0, 5.0]], [[5.0, 1.0, 6.0], 
[6.0, 4.0, 7.0], [6.0, 7.0, 3.0]], [[5.0, 1.0, 5.0], [7.0, 2.0, 2.0], [8.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPPP(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 2.0], [7.0, 6.0, 6.0], [8.0, 4.0, 7.0]], [[7.0, 2.0, 7.0], [8.0, 6.0, 1.0], [3.0, 2.0, 
4.0]], [[3.0, 1.0, 6.0], [5.0, 3.0, 1.0], [6.0, 5.0, 7.0]]], [[[4.0, 1.0, 5.0], [8.0, 7.0, 5.0], [8.0, 7.0, 4.0]], [[6.0, 1.0, 
5.0], [4.0, 1.0, 3.0], [3.0, 2.0, 7.0]], [[3.0, 2.0, 8.0], [1.0, 1.0, 6.0], [7.0, 1.0, 8.0]]], [[[6.0, 7.0, 7.0], [1.0, 2.0, 
4.0], [4.0, 2.0, 4.0]], [[4.0, 6.0, 8.0], [3.0, 8.0, 6.0], [3.0, 2.0, 3.0]], [[5.0, 4.0, 4.0], [7.0, 7.0, 7.0], [7.0, 8.0, 
1.0]]], [[[5.0, 2.0, 7.0], [8.0, 4.0, 6.0], [5.0, 4.0, 2.0]], [[2.0, 6.0, 7.0], [2.0, 5.0, 2.0], [1.0, 3.0, 7.0]], [[5.0, 1.0, 
4.0], [5.0, 6.0, 5.0], [3.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 3.0], [5.0, 7.0, 8.0], [6.0, 3.0, 5.0]], [[6.0, 5.0, 4.0], [4.0, 7.0, 
7.0], [5.0, 4.0, 2.0]], [[1.0, 2.0, 3.0], [2.0, 2.0, 7.0], [4.0, 5.0, 6.0]]], [[[2.0, 3.0, 2.0], [5.0, 3.0, 5.0], [2.0, 1.0, 
7.0]], [[5.0, 2.0, 7.0], [3.0, 2.0, 8.0], [6.0, 6.0, 1.0]], [[5.0, 2.0, 3.0], [6.0, 2.0, 2.0], [3.0, 6.0, 1.0]]], [[[3.0, 5.0, 
3.0], [4.0, 4.0, 3.0], [6.0, 1.0, 5.0]], [[8.0, 3.0, 2.0], [5.0, 2.0, 2.0], [3.0, 4.0, 6.0]], [[2.0, 8.0, 5.0], [6.0, 4.0, 
2.0], [1.0, 1.0, 8.0]]], [[[3.0, 4.0, 7.0], [2.0, 5.0, 2.0], [4.0, 2.0, 3.0]], [[1.0, 4.0, 5.0], [6.0, 2.0, 7.0], [7.0, 5.0, 
8.0]], [[5.0, 3.0, 3.0], [7.0, 1.0, 6.0], [3.0, 4.0, 4.0]]]]))
      val=Data(numpy.array([8.0, 5.0, 3.0, 2.0]),self.functionspace)
      arg[:,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 2.0], [7.0, 6.0, 6.0], [8.0, 4.0, 7.0]], [[7.0, 2.0, 7.0], [8.0, 8.0, 1.0], [3.0, 
2.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 3.0, 1.0], [6.0, 5.0, 7.0]]], [[[4.0, 1.0, 5.0], [8.0, 7.0, 5.0], [8.0, 7.0, 4.0]], [[6.0, 
1.0, 5.0], [4.0, 5.0, 3.0], [3.0, 2.0, 7.0]], [[3.0, 2.0, 8.0], [1.0, 1.0, 6.0], [7.0, 1.0, 8.0]]], [[[6.0, 7.0, 7.0], [1.0, 
2.0, 4.0], [4.0, 2.0, 4.0]], [[4.0, 6.0, 8.0], [3.0, 3.0, 6.0], [3.0, 2.0, 3.0]], [[5.0, 4.0, 4.0], [7.0, 7.0, 7.0], [7.0, 8.0, 
1.0]]], [[[5.0, 2.0, 7.0], [8.0, 4.0, 6.0], [5.0, 4.0, 2.0]], [[2.0, 6.0, 7.0], [2.0, 2.0, 2.0], [1.0, 3.0, 7.0]], [[5.0, 1.0, 
4.0], [5.0, 6.0, 5.0], [3.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 3.0], [5.0, 7.0, 8.0], [6.0, 3.0, 5.0]], 
[[6.0, 5.0, 4.0], [4.0, 8.0, 7.0], [5.0, 4.0, 2.0]], [[1.0, 2.0, 3.0], [2.0, 2.0, 7.0], [4.0, 5.0, 6.0]]], [[[2.0, 3.0, 2.0], 
[5.0, 3.0, 5.0], [2.0, 1.0, 7.0]], [[5.0, 2.0, 7.0], [3.0, 5.0, 8.0], [6.0, 6.0, 1.0]], [[5.0, 2.0, 3.0], [6.0, 2.0, 2.0], 
[3.0, 6.0, 1.0]]], [[[3.0, 5.0, 3.0], [4.0, 4.0, 3.0], [6.0, 1.0, 5.0]], [[8.0, 3.0, 2.0], [5.0, 3.0, 2.0], [3.0, 4.0, 6.0]], 
[[2.0, 8.0, 5.0], [6.0, 4.0, 2.0], [1.0, 1.0, 8.0]]], [[[3.0, 4.0, 7.0], [2.0, 5.0, 2.0], [4.0, 2.0, 3.0]], [[1.0, 4.0, 5.0], 
[6.0, 2.0, 7.0], [7.0, 5.0, 8.0]], [[5.0, 3.0, 3.0], [7.0, 1.0, 6.0], [3.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPPP(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 8.0], [8.0, 5.0, 3.0], [6.0, 6.0, 8.0]], [[4.0, 1.0, 8.0], [3.0, 5.0, 5.0], [6.0, 2.0, 
1.0]], [[2.0, 1.0, 4.0], [3.0, 3.0, 6.0], [5.0, 7.0, 5.0]]], [[[5.0, 1.0, 6.0], [7.0, 5.0, 3.0], [7.0, 4.0, 2.0]], [[4.0, 6.0, 
7.0], [8.0, 5.0, 2.0], [3.0, 8.0, 6.0]], [[5.0, 7.0, 7.0], [2.0, 4.0, 3.0], [2.0, 3.0, 6.0]]], [[[8.0, 5.0, 5.0], [7.0, 3.0, 
7.0], [4.0, 3.0, 7.0]], [[1.0, 6.0, 6.0], [3.0, 2.0, 2.0], [6.0, 3.0, 4.0]], [[8.0, 5.0, 8.0], [2.0, 6.0, 4.0], [7.0, 7.0, 
3.0]]], [[[8.0, 5.0, 4.0], [5.0, 6.0, 4.0], [2.0, 6.0, 6.0]], [[5.0, 4.0, 7.0], [4.0, 6.0, 3.0], [2.0, 5.0, 2.0]], [[5.0, 4.0, 
8.0], [4.0, 6.0, 3.0], [3.0, 7.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 6.0], [4.0, 1.0, 3.0], [1.0, 5.0, 2.0]], [[6.0, 4.0, 5.0], [4.0, 3.0, 
8.0], [8.0, 3.0, 3.0]], [[3.0, 5.0, 4.0], [3.0, 6.0, 7.0], [1.0, 1.0, 8.0]]], [[[1.0, 8.0, 1.0], [3.0, 7.0, 3.0], [1.0, 6.0, 
1.0]], [[2.0, 5.0, 8.0], [3.0, 8.0, 3.0], [2.0, 3.0, 5.0]], [[7.0, 7.0, 7.0], [7.0, 1.0, 7.0], [7.0, 5.0, 2.0]]], [[[5.0, 5.0, 
6.0], [2.0, 4.0, 5.0], [3.0, 7.0, 2.0]], [[5.0, 7.0, 2.0], [6.0, 7.0, 1.0], [3.0, 4.0, 1.0]], [[7.0, 5.0, 7.0], [6.0, 8.0, 
2.0], [2.0, 4.0, 3.0]]], [[[2.0, 3.0, 8.0], [8.0, 1.0, 3.0], [3.0, 7.0, 2.0]], [[4.0, 8.0, 7.0], [4.0, 6.0, 4.0], [4.0, 1.0, 
7.0]], [[8.0, 6.0, 8.0], [3.0, 3.0, 3.0], [6.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([6.0, 6.0, 2.0, 5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([2.0, 4.0, 3.0, 8.0]))
      arg[:,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 8.0], [8.0, 5.0, 3.0], [6.0, 6.0, 8.0]], [[4.0, 1.0, 8.0], [3.0, 6.0, 5.0], [6.0, 
2.0, 1.0]], [[2.0, 1.0, 4.0], [3.0, 3.0, 6.0], [5.0, 7.0, 5.0]]], [[[5.0, 1.0, 6.0], [7.0, 5.0, 3.0], [7.0, 4.0, 2.0]], [[4.0, 
6.0, 7.0], [8.0, 6.0, 2.0], [3.0, 8.0, 6.0]], [[5.0, 7.0, 7.0], [2.0, 4.0, 3.0], [2.0, 3.0, 6.0]]], [[[8.0, 5.0, 5.0], [7.0, 
3.0, 7.0], [4.0, 3.0, 7.0]], [[1.0, 6.0, 6.0], [3.0, 2.0, 2.0], [6.0, 3.0, 4.0]], [[8.0, 5.0, 8.0], [2.0, 6.0, 4.0], [7.0, 7.0, 
3.0]]], [[[8.0, 5.0, 4.0], [5.0, 6.0, 4.0], [2.0, 6.0, 6.0]], [[5.0, 4.0, 7.0], [4.0, 5.0, 3.0], [2.0, 5.0, 2.0]], [[5.0, 4.0, 
8.0], [4.0, 6.0, 3.0], [3.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 6.0], [4.0, 1.0, 3.0], [1.0, 5.0, 2.0]], 
[[6.0, 4.0, 5.0], [4.0, 2.0, 8.0], [8.0, 3.0, 3.0]], [[3.0, 5.0, 4.0], [3.0, 6.0, 7.0], [1.0, 1.0, 8.0]]], [[[1.0, 8.0, 1.0], 
[3.0, 7.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 5.0, 8.0], [3.0, 4.0, 3.0], [2.0, 3.0, 5.0]], [[7.0, 7.0, 7.0], [7.0, 1.0, 7.0], 
[7.0, 5.0, 2.0]]], [[[5.0, 5.0, 6.0], [2.0, 4.0, 5.0], [3.0, 7.0, 2.0]], [[5.0, 7.0, 2.0], [6.0, 3.0, 1.0], [3.0, 4.0, 1.0]], 
[[7.0, 5.0, 7.0], [6.0, 8.0, 2.0], [2.0, 4.0, 3.0]]], [[[2.0, 3.0, 8.0], [8.0, 1.0, 3.0], [3.0, 7.0, 2.0]], [[4.0, 8.0, 7.0], 
[4.0, 8.0, 4.0], [4.0, 1.0, 7.0]], [[8.0, 6.0, 8.0], [3.0, 3.0, 3.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPPP(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 3.0], [7.0, 2.0, 3.0], [4.0, 3.0, 2.0]], [[7.0, 5.0, 1.0], [2.0, 3.0, 2.0], [3.0, 8.0, 
1.0]], [[3.0, 4.0, 5.0], [2.0, 4.0, 1.0], [7.0, 1.0, 7.0]]], [[[8.0, 2.0, 2.0], [6.0, 6.0, 4.0], [8.0, 2.0, 6.0]], [[4.0, 2.0, 
5.0], [8.0, 3.0, 5.0], [1.0, 8.0, 2.0]], [[8.0, 1.0, 8.0], [4.0, 5.0, 4.0], [5.0, 6.0, 6.0]]], [[[5.0, 7.0, 5.0], [4.0, 5.0, 
8.0], [8.0, 7.0, 7.0]], [[1.0, 8.0, 8.0], [7.0, 2.0, 6.0], [3.0, 2.0, 7.0]], [[8.0, 6.0, 2.0], [4.0, 1.0, 6.0], [6.0, 7.0, 
3.0]]], [[[8.0, 4.0, 7.0], [1.0, 4.0, 6.0], [6.0, 3.0, 7.0]], [[3.0, 8.0, 1.0], [6.0, 4.0, 2.0], [8.0, 1.0, 2.0]], [[5.0, 6.0, 
1.0], [7.0, 1.0, 1.0], [4.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 7.0], [2.0, 5.0, 1.0], [3.0, 5.0, 3.0]], [[4.0, 1.0, 4.0], [3.0, 1.0, 
8.0], [7.0, 1.0, 1.0]], [[3.0, 8.0, 6.0], [4.0, 1.0, 6.0], [8.0, 8.0, 5.0]]], [[[6.0, 7.0, 1.0], [7.0, 4.0, 6.0], [6.0, 1.0, 
5.0]], [[2.0, 5.0, 4.0], [4.0, 2.0, 2.0], [6.0, 3.0, 2.0]], [[4.0, 6.0, 8.0], [4.0, 5.0, 1.0], [3.0, 5.0, 7.0]]], [[[8.0, 7.0, 
7.0], [2.0, 1.0, 1.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 5.0], [3.0, 1.0, 2.0], [7.0, 5.0, 1.0]], [[2.0, 6.0, 2.0], [4.0, 1.0, 
1.0], [6.0, 4.0, 4.0]]], [[[1.0, 2.0, 7.0], [1.0, 6.0, 3.0], [5.0, 3.0, 5.0]], [[6.0, 6.0, 6.0], [4.0, 3.0, 6.0], [4.0, 7.0, 
1.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 3.0], [6.0, 7.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([6.0, 3.0, 8.0, 1.0])+(1.-msk_val)*numpy.array([3.0, 4.0, 5.0, 5.0])
      arg[:,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 3.0], [7.0, 2.0, 3.0], [4.0, 3.0, 2.0]], [[7.0, 5.0, 1.0], [2.0, 6.0, 2.0], [3.0, 
8.0, 1.0]], [[3.0, 4.0, 5.0], [2.0, 4.0, 1.0], [7.0, 1.0, 7.0]]], [[[8.0, 2.0, 2.0], [6.0, 6.0, 4.0], [8.0, 2.0, 6.0]], [[4.0, 
2.0, 5.0], [8.0, 3.0, 5.0], [1.0, 8.0, 2.0]], [[8.0, 1.0, 8.0], [4.0, 5.0, 4.0], [5.0, 6.0, 6.0]]], [[[5.0, 7.0, 5.0], [4.0, 
5.0, 8.0], [8.0, 7.0, 7.0]], [[1.0, 8.0, 8.0], [7.0, 8.0, 6.0], [3.0, 2.0, 7.0]], [[8.0, 6.0, 2.0], [4.0, 1.0, 6.0], [6.0, 7.0, 
3.0]]], [[[8.0, 4.0, 7.0], [1.0, 4.0, 6.0], [6.0, 3.0, 7.0]], [[3.0, 8.0, 1.0], [6.0, 1.0, 2.0], [8.0, 1.0, 2.0]], [[5.0, 6.0, 
1.0], [7.0, 1.0, 1.0], [4.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 7.0], [2.0, 5.0, 1.0], [3.0, 5.0, 3.0]], 
[[4.0, 1.0, 4.0], [3.0, 3.0, 8.0], [7.0, 1.0, 1.0]], [[3.0, 8.0, 6.0], [4.0, 1.0, 6.0], [8.0, 8.0, 5.0]]], [[[6.0, 7.0, 1.0], 
[7.0, 4.0, 6.0], [6.0, 1.0, 5.0]], [[2.0, 5.0, 4.0], [4.0, 4.0, 2.0], [6.0, 3.0, 2.0]], [[4.0, 6.0, 8.0], [4.0, 5.0, 1.0], 
[3.0, 5.0, 7.0]]], [[[8.0, 7.0, 7.0], [2.0, 1.0, 1.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 5.0], [3.0, 5.0, 2.0], [7.0, 5.0, 1.0]], 
[[2.0, 6.0, 2.0], [4.0, 1.0, 1.0], [6.0, 4.0, 4.0]]], [[[1.0, 2.0, 7.0], [1.0, 6.0, 3.0], [5.0, 3.0, 5.0]], [[6.0, 6.0, 6.0], 
[4.0, 5.0, 6.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 3.0], [6.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PUUN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 2.0], [7.0, 5.0, 3.0], [7.0, 2.0, 7.0]], [[7.0, 1.0, 7.0], [2.0, 1.0, 5.0], [6.0, 4.0, 
3.0]], [[4.0, 8.0, 6.0], [8.0, 2.0, 5.0], [1.0, 4.0, 6.0]]], [[[8.0, 4.0, 3.0], [5.0, 4.0, 6.0], [2.0, 1.0, 5.0]], [[1.0, 2.0, 
3.0], [1.0, 4.0, 1.0], [3.0, 6.0, 2.0]], [[6.0, 5.0, 5.0], [4.0, 2.0, 2.0], [8.0, 8.0, 1.0]]], [[[6.0, 8.0, 8.0], [5.0, 8.0, 
3.0], [4.0, 4.0, 7.0]], [[4.0, 2.0, 5.0], [3.0, 1.0, 3.0], [1.0, 2.0, 4.0]], [[6.0, 4.0, 6.0], [5.0, 5.0, 8.0], [3.0, 2.0, 
6.0]]], [[[1.0, 8.0, 2.0], [7.0, 4.0, 6.0], [1.0, 3.0, 3.0]], [[7.0, 1.0, 2.0], [6.0, 7.0, 2.0], [7.0, 6.0, 7.0]], [[6.0, 6.0, 
1.0], [1.0, 4.0, 8.0], [6.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 5.0], [2.0, 6.0, 2.0], [8.0, 8.0, 5.0]], [[1.0, 8.0, 5.0], [4.0, 1.0, 
6.0], [2.0, 4.0, 5.0]], [[2.0, 7.0, 2.0], [2.0, 7.0, 6.0], [3.0, 8.0, 5.0]]], [[[4.0, 4.0, 2.0], [4.0, 8.0, 6.0], [8.0, 2.0, 
4.0]], [[1.0, 4.0, 6.0], [4.0, 6.0, 6.0], [7.0, 6.0, 6.0]], [[7.0, 1.0, 3.0], [4.0, 6.0, 1.0], [2.0, 7.0, 6.0]]], [[[3.0, 6.0, 
3.0], [4.0, 6.0, 2.0], [4.0, 3.0, 3.0]], [[4.0, 2.0, 4.0], [1.0, 1.0, 7.0], [4.0, 4.0, 2.0]], [[2.0, 8.0, 3.0], [3.0, 6.0, 
1.0], [5.0, 7.0, 7.0]]], [[[8.0, 2.0, 4.0], [7.0, 2.0, 4.0], [3.0, 2.0, 8.0]], [[6.0, 4.0, 4.0], [6.0, 7.0, 5.0], [6.0, 8.0, 
4.0]], [[1.0, 1.0, 1.0], [7.0, 8.0, 6.0], [2.0, 2.0, 6.0]]]]))
      val=numpy.array([[[5.0, 1.0, 3.0], [4.0, 3.0, 7.0]], [[4.0, 1.0, 7.0], [8.0, 7.0, 5.0]]])
      arg[2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 2.0], [7.0, 5.0, 3.0], [7.0, 2.0, 7.0]], [[7.0, 1.0, 7.0], [2.0, 1.0, 5.0], [6.0, 
4.0, 3.0]], [[4.0, 8.0, 6.0], [8.0, 2.0, 5.0], [1.0, 4.0, 6.0]]], [[[8.0, 4.0, 3.0], [5.0, 4.0, 6.0], [2.0, 1.0, 5.0]], [[1.0, 
2.0, 3.0], [1.0, 4.0, 1.0], [3.0, 6.0, 2.0]], [[6.0, 5.0, 5.0], [4.0, 2.0, 2.0], [8.0, 8.0, 1.0]]], [[[5.0, 1.0, 3.0], [4.0, 
3.0, 7.0], [4.0, 4.0, 7.0]], [[4.0, 1.0, 7.0], [8.0, 7.0, 5.0], [1.0, 2.0, 4.0]], [[6.0, 4.0, 6.0], [5.0, 5.0, 8.0], [3.0, 2.0, 
6.0]]], [[[1.0, 8.0, 2.0], [7.0, 4.0, 6.0], [1.0, 3.0, 3.0]], [[7.0, 1.0, 2.0], [6.0, 7.0, 2.0], [7.0, 6.0, 7.0]], [[6.0, 6.0, 
1.0], [1.0, 4.0, 8.0], [6.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 5.0], [2.0, 6.0, 2.0], [8.0, 8.0, 5.0]], 
[[1.0, 8.0, 5.0], [4.0, 1.0, 6.0], [2.0, 4.0, 5.0]], [[2.0, 7.0, 2.0], [2.0, 7.0, 6.0], [3.0, 8.0, 5.0]]], [[[4.0, 4.0, 2.0], 
[4.0, 8.0, 6.0], [8.0, 2.0, 4.0]], [[1.0, 4.0, 6.0], [4.0, 6.0, 6.0], [7.0, 6.0, 6.0]], [[7.0, 1.0, 3.0], [4.0, 6.0, 1.0], 
[2.0, 7.0, 6.0]]], [[[5.0, 1.0, 3.0], [4.0, 3.0, 7.0], [4.0, 3.0, 3.0]], [[4.0, 1.0, 7.0], [8.0, 7.0, 5.0], [4.0, 4.0, 2.0]], 
[[2.0, 8.0, 3.0], [3.0, 6.0, 1.0], [5.0, 7.0, 7.0]]], [[[8.0, 2.0, 4.0], [7.0, 2.0, 4.0], [3.0, 2.0, 8.0]], [[6.0, 4.0, 4.0], 
[6.0, 7.0, 5.0], [6.0, 8.0, 4.0]], [[1.0, 1.0, 1.0], [7.0, 8.0, 6.0], [2.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PUUN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 3.0], [5.0, 2.0, 5.0], [5.0, 5.0, 1.0]], [[8.0, 5.0, 5.0], [6.0, 1.0, 5.0], [3.0, 3.0, 
2.0]], [[1.0, 8.0, 8.0], [6.0, 1.0, 7.0], [4.0, 1.0, 4.0]]], [[[4.0, 2.0, 7.0], [5.0, 5.0, 6.0], [4.0, 3.0, 6.0]], [[3.0, 4.0, 
7.0], [3.0, 6.0, 7.0], [4.0, 7.0, 5.0]], [[2.0, 1.0, 8.0], [1.0, 6.0, 5.0], [3.0, 6.0, 5.0]]], [[[3.0, 4.0, 7.0], [3.0, 3.0, 
5.0], [4.0, 7.0, 1.0]], [[1.0, 6.0, 5.0], [6.0, 7.0, 2.0], [3.0, 7.0, 5.0]], [[7.0, 4.0, 4.0], [4.0, 6.0, 4.0], [1.0, 3.0, 
4.0]]], [[[6.0, 8.0, 4.0], [7.0, 6.0, 1.0], [5.0, 6.0, 3.0]], [[1.0, 6.0, 3.0], [1.0, 5.0, 6.0], [3.0, 4.0, 4.0]], [[6.0, 2.0, 
3.0], [4.0, 7.0, 2.0], [7.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 8.0], [1.0, 1.0, 6.0], [4.0, 8.0, 4.0]], [[7.0, 6.0, 1.0], [4.0, 1.0, 
3.0], [1.0, 6.0, 4.0]], [[3.0, 2.0, 7.0], [5.0, 8.0, 2.0], [5.0, 2.0, 1.0]]], [[[3.0, 5.0, 7.0], [7.0, 6.0, 2.0], [1.0, 5.0, 
3.0]], [[7.0, 8.0, 2.0], [6.0, 5.0, 1.0], [7.0, 2.0, 6.0]], [[8.0, 5.0, 3.0], [4.0, 6.0, 8.0], [6.0, 7.0, 2.0]]], [[[2.0, 4.0, 
7.0], [8.0, 7.0, 7.0], [6.0, 1.0, 4.0]], [[3.0, 6.0, 7.0], [3.0, 1.0, 3.0], [8.0, 7.0, 8.0]], [[3.0, 1.0, 4.0], [5.0, 2.0, 
5.0], [1.0, 3.0, 6.0]]], [[[6.0, 7.0, 1.0], [4.0, 2.0, 4.0], [5.0, 7.0, 3.0]], [[8.0, 6.0, 4.0], [7.0, 4.0, 8.0], [1.0, 3.0, 
4.0]], [[8.0, 7.0, 8.0], [8.0, 2.0, 8.0], [3.0, 5.0, 4.0]]]]))
      val=Data(numpy.array([[[5.0, 7.0, 6.0], [6.0, 6.0, 7.0]], [[1.0, 4.0, 8.0], [8.0, 8.0, 4.0]]]),self.functionspace)
      arg[2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 3.0], [5.0, 2.0, 5.0], [5.0, 5.0, 1.0]], [[8.0, 5.0, 5.0], [6.0, 1.0, 5.0], [3.0, 
3.0, 2.0]], [[1.0, 8.0, 8.0], [6.0, 1.0, 7.0], [4.0, 1.0, 4.0]]], [[[4.0, 2.0, 7.0], [5.0, 5.0, 6.0], [4.0, 3.0, 6.0]], [[3.0, 
4.0, 7.0], [3.0, 6.0, 7.0], [4.0, 7.0, 5.0]], [[2.0, 1.0, 8.0], [1.0, 6.0, 5.0], [3.0, 6.0, 5.0]]], [[[5.0, 7.0, 6.0], [6.0, 
6.0, 7.0], [4.0, 7.0, 1.0]], [[1.0, 4.0, 8.0], [8.0, 8.0, 4.0], [3.0, 7.0, 5.0]], [[7.0, 4.0, 4.0], [4.0, 6.0, 4.0], [1.0, 3.0, 
4.0]]], [[[6.0, 8.0, 4.0], [7.0, 6.0, 1.0], [5.0, 6.0, 3.0]], [[1.0, 6.0, 3.0], [1.0, 5.0, 6.0], [3.0, 4.0, 4.0]], [[6.0, 2.0, 
3.0], [4.0, 7.0, 2.0], [7.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 8.0], [1.0, 1.0, 6.0], [4.0, 8.0, 4.0]], 
[[7.0, 6.0, 1.0], [4.0, 1.0, 3.0], [1.0, 6.0, 4.0]], [[3.0, 2.0, 7.0], [5.0, 8.0, 2.0], [5.0, 2.0, 1.0]]], [[[3.0, 5.0, 7.0], 
[7.0, 6.0, 2.0], [1.0, 5.0, 3.0]], [[7.0, 8.0, 2.0], [6.0, 5.0, 1.0], [7.0, 2.0, 6.0]], [[8.0, 5.0, 3.0], [4.0, 6.0, 8.0], 
[6.0, 7.0, 2.0]]], [[[5.0, 7.0, 6.0], [6.0, 6.0, 7.0], [6.0, 1.0, 4.0]], [[1.0, 4.0, 8.0], [8.0, 8.0, 4.0], [8.0, 7.0, 8.0]], 
[[3.0, 1.0, 4.0], [5.0, 2.0, 5.0], [1.0, 3.0, 6.0]]], [[[6.0, 7.0, 1.0], [4.0, 2.0, 4.0], [5.0, 7.0, 3.0]], [[8.0, 6.0, 4.0], 
[7.0, 4.0, 8.0], [1.0, 3.0, 4.0]], [[8.0, 7.0, 8.0], [8.0, 2.0, 8.0], [3.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PUUN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 1.0], [4.0, 8.0, 7.0], [7.0, 1.0, 1.0]], [[1.0, 7.0, 5.0], [8.0, 6.0, 4.0], [5.0, 3.0, 
3.0]], [[3.0, 2.0, 1.0], [2.0, 6.0, 4.0], [5.0, 8.0, 8.0]]], [[[4.0, 6.0, 8.0], [1.0, 7.0, 8.0], [8.0, 6.0, 3.0]], [[4.0, 7.0, 
1.0], [6.0, 3.0, 6.0], [6.0, 7.0, 3.0]], [[6.0, 6.0, 8.0], [7.0, 4.0, 8.0], [8.0, 2.0, 1.0]]], [[[5.0, 8.0, 4.0], [5.0, 5.0, 
5.0], [6.0, 2.0, 4.0]], [[1.0, 3.0, 1.0], [1.0, 8.0, 6.0], [4.0, 6.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 4.0, 1.0], [5.0, 6.0, 
1.0]]], [[[8.0, 2.0, 8.0], [3.0, 3.0, 5.0], [8.0, 7.0, 5.0]], [[4.0, 6.0, 1.0], [4.0, 6.0, 8.0], [4.0, 3.0, 5.0]], [[6.0, 6.0, 
7.0], [2.0, 7.0, 6.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 1.0], [7.0, 1.0, 7.0], [3.0, 2.0, 6.0]], [[2.0, 3.0, 4.0], [6.0, 5.0, 
2.0], [7.0, 4.0, 7.0]], [[4.0, 8.0, 5.0], [4.0, 5.0, 1.0], [8.0, 2.0, 6.0]]], [[[6.0, 3.0, 4.0], [6.0, 8.0, 6.0], [3.0, 3.0, 
2.0]], [[5.0, 2.0, 8.0], [8.0, 8.0, 4.0], [8.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], [4.0, 4.0, 7.0], [6.0, 4.0, 2.0]]], [[[5.0, 6.0, 
6.0], [3.0, 4.0, 8.0], [3.0, 5.0, 3.0]], [[5.0, 5.0, 5.0], [5.0, 6.0, 4.0], [6.0, 5.0, 1.0]], [[2.0, 7.0, 1.0], [2.0, 5.0, 
6.0], [1.0, 2.0, 5.0]]], [[[6.0, 4.0, 5.0], [8.0, 4.0, 1.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 5.0], [1.0, 4.0, 7.0], [8.0, 8.0, 
6.0]], [[8.0, 8.0, 6.0], [5.0, 2.0, 1.0], [6.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[5.0, 5.0, 3.0], [6.0, 1.0, 2.0]], [[5.0, 1.0, 7.0], [1.0, 6.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 7.0, 7.0], [7.0, 3.0, 2.0]], [[6.0, 7.0, 3.0], [2.0, 7.0, 3.0]]]))
      arg[2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 1.0], [4.0, 8.0, 7.0], [7.0, 1.0, 1.0]], [[1.0, 7.0, 5.0], [8.0, 6.0, 4.0], [5.0, 
3.0, 3.0]], [[3.0, 2.0, 1.0], [2.0, 6.0, 4.0], [5.0, 8.0, 8.0]]], [[[4.0, 6.0, 8.0], [1.0, 7.0, 8.0], [8.0, 6.0, 3.0]], [[4.0, 
7.0, 1.0], [6.0, 3.0, 6.0], [6.0, 7.0, 3.0]], [[6.0, 6.0, 8.0], [7.0, 4.0, 8.0], [8.0, 2.0, 1.0]]], [[[5.0, 5.0, 3.0], [6.0, 
1.0, 2.0], [6.0, 2.0, 4.0]], [[5.0, 1.0, 7.0], [1.0, 6.0, 8.0], [4.0, 6.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 4.0, 1.0], [5.0, 6.0, 
1.0]]], [[[8.0, 2.0, 8.0], [3.0, 3.0, 5.0], [8.0, 7.0, 5.0]], [[4.0, 6.0, 1.0], [4.0, 6.0, 8.0], [4.0, 3.0, 5.0]], [[6.0, 6.0, 
7.0], [2.0, 7.0, 6.0], [1.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 1.0], [7.0, 1.0, 7.0], [3.0, 2.0, 6.0]], 
[[2.0, 3.0, 4.0], [6.0, 5.0, 2.0], [7.0, 4.0, 7.0]], [[4.0, 8.0, 5.0], [4.0, 5.0, 1.0], [8.0, 2.0, 6.0]]], [[[6.0, 3.0, 4.0], 
[6.0, 8.0, 6.0], [3.0, 3.0, 2.0]], [[5.0, 2.0, 8.0], [8.0, 8.0, 4.0], [8.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], [4.0, 4.0, 7.0], 
[6.0, 4.0, 2.0]]], [[[6.0, 7.0, 7.0], [7.0, 3.0, 2.0], [3.0, 5.0, 3.0]], [[6.0, 7.0, 3.0], [2.0, 7.0, 3.0], [6.0, 5.0, 1.0]], 
[[2.0, 7.0, 1.0], [2.0, 5.0, 6.0], [1.0, 2.0, 5.0]]], [[[6.0, 4.0, 5.0], [8.0, 4.0, 1.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 5.0], 
[1.0, 4.0, 7.0], [8.0, 8.0, 6.0]], [[8.0, 8.0, 6.0], [5.0, 2.0, 1.0], [6.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PUUN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 6.0], [3.0, 8.0, 2.0], [2.0, 1.0, 4.0]], [[4.0, 1.0, 7.0], [2.0, 5.0, 3.0], [5.0, 1.0, 
3.0]], [[8.0, 5.0, 7.0], [7.0, 6.0, 5.0], [3.0, 8.0, 1.0]]], [[[6.0, 1.0, 6.0], [3.0, 3.0, 5.0], [2.0, 5.0, 7.0]], [[2.0, 3.0, 
8.0], [1.0, 7.0, 4.0], [4.0, 7.0, 7.0]], [[3.0, 6.0, 2.0], [6.0, 5.0, 2.0], [1.0, 6.0, 8.0]]], [[[2.0, 8.0, 2.0], [3.0, 7.0, 
1.0], [5.0, 1.0, 2.0]], [[6.0, 5.0, 6.0], [3.0, 8.0, 6.0], [6.0, 1.0, 7.0]], [[2.0, 4.0, 6.0], [5.0, 7.0, 4.0], [1.0, 1.0, 
8.0]]], [[[1.0, 6.0, 5.0], [1.0, 8.0, 5.0], [7.0, 5.0, 5.0]], [[8.0, 4.0, 8.0], [7.0, 4.0, 4.0], [3.0, 2.0, 5.0]], [[4.0, 3.0, 
5.0], [2.0, 4.0, 6.0], [5.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 8.0], [2.0, 5.0, 7.0], [8.0, 7.0, 8.0]], [[5.0, 1.0, 4.0], [6.0, 4.0, 
3.0], [8.0, 5.0, 6.0]], [[3.0, 7.0, 8.0], [8.0, 6.0, 5.0], [6.0, 2.0, 8.0]]], [[[8.0, 3.0, 8.0], [2.0, 1.0, 7.0], [4.0, 6.0, 
1.0]], [[1.0, 6.0, 6.0], [7.0, 8.0, 4.0], [4.0, 1.0, 1.0]], [[4.0, 7.0, 8.0], [8.0, 6.0, 7.0], [8.0, 4.0, 4.0]]], [[[3.0, 5.0, 
5.0], [5.0, 3.0, 3.0], [7.0, 2.0, 8.0]], [[8.0, 4.0, 7.0], [2.0, 1.0, 6.0], [8.0, 2.0, 5.0]], [[2.0, 6.0, 8.0], [5.0, 5.0, 
1.0], [4.0, 6.0, 3.0]]], [[[5.0, 2.0, 2.0], [3.0, 8.0, 7.0], [4.0, 1.0, 1.0]], [[7.0, 5.0, 6.0], [5.0, 3.0, 4.0], [1.0, 5.0, 
3.0]], [[2.0, 7.0, 5.0], [6.0, 6.0, 5.0], [8.0, 2.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 8.0, 2.0], [8.0, 5.0, 8.0]], [[4.0, 3.0, 1.0], [8.0, 7.0, 
7.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 7.0], [4.0, 4.0, 5.0]], [[3.0, 3.0, 5.0], [6.0, 3.0, 1.0]]])
      arg[2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 6.0], [3.0, 8.0, 2.0], [2.0, 1.0, 4.0]], [[4.0, 1.0, 7.0], [2.0, 5.0, 3.0], [5.0, 
1.0, 3.0]], [[8.0, 5.0, 7.0], [7.0, 6.0, 5.0], [3.0, 8.0, 1.0]]], [[[6.0, 1.0, 6.0], [3.0, 3.0, 5.0], [2.0, 5.0, 7.0]], [[2.0, 
3.0, 8.0], [1.0, 7.0, 4.0], [4.0, 7.0, 7.0]], [[3.0, 6.0, 2.0], [6.0, 5.0, 2.0], [1.0, 6.0, 8.0]]], [[[3.0, 8.0, 2.0], [8.0, 
5.0, 8.0], [5.0, 1.0, 2.0]], [[4.0, 3.0, 1.0], [8.0, 7.0, 7.0], [6.0, 1.0, 7.0]], [[2.0, 4.0, 6.0], [5.0, 7.0, 4.0], [1.0, 1.0, 
8.0]]], [[[1.0, 6.0, 5.0], [1.0, 8.0, 5.0], [7.0, 5.0, 5.0]], [[8.0, 4.0, 8.0], [7.0, 4.0, 4.0], [3.0, 2.0, 5.0]], [[4.0, 3.0, 
5.0], [2.0, 4.0, 6.0], [5.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 8.0], [2.0, 5.0, 7.0], [8.0, 7.0, 8.0]], 
[[5.0, 1.0, 4.0], [6.0, 4.0, 3.0], [8.0, 5.0, 6.0]], [[3.0, 7.0, 8.0], [8.0, 6.0, 5.0], [6.0, 2.0, 8.0]]], [[[8.0, 3.0, 8.0], 
[2.0, 1.0, 7.0], [4.0, 6.0, 1.0]], [[1.0, 6.0, 6.0], [7.0, 8.0, 4.0], [4.0, 1.0, 1.0]], [[4.0, 7.0, 8.0], [8.0, 6.0, 7.0], 
[8.0, 4.0, 4.0]]], [[[4.0, 6.0, 7.0], [4.0, 4.0, 5.0], [7.0, 2.0, 8.0]], [[3.0, 3.0, 5.0], [6.0, 3.0, 1.0], [8.0, 2.0, 5.0]], 
[[2.0, 6.0, 8.0], [5.0, 5.0, 1.0], [4.0, 6.0, 3.0]]], [[[5.0, 2.0, 2.0], [3.0, 8.0, 7.0], [4.0, 1.0, 1.0]], [[7.0, 5.0, 6.0], 
[5.0, 3.0, 4.0], [1.0, 5.0, 3.0]], [[2.0, 7.0, 5.0], [6.0, 6.0, 5.0], [8.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RUUC(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 6.0], [4.0, 4.0, 8.0], [8.0, 6.0, 3.0]], [[1.0, 8.0, 2.0], [1.0, 6.0, 7.0], [2.0, 4.0, 
6.0]], [[2.0, 2.0, 4.0], [7.0, 2.0, 2.0], [3.0, 5.0, 6.0]]], [[[5.0, 7.0, 5.0], [4.0, 5.0, 4.0], [6.0, 3.0, 1.0]], [[8.0, 3.0, 
8.0], [2.0, 3.0, 4.0], [8.0, 1.0, 3.0]], [[2.0, 2.0, 5.0], [6.0, 8.0, 8.0], [7.0, 2.0, 2.0]]], [[[2.0, 2.0, 7.0], [2.0, 1.0, 
3.0], [2.0, 4.0, 5.0]], [[3.0, 8.0, 8.0], [2.0, 1.0, 2.0], [8.0, 8.0, 4.0]], [[2.0, 4.0, 7.0], [4.0, 7.0, 7.0], [4.0, 4.0, 
3.0]]], [[[6.0, 2.0, 5.0], [4.0, 1.0, 7.0], [8.0, 3.0, 1.0]], [[4.0, 4.0, 4.0], [8.0, 8.0, 6.0], [7.0, 4.0, 3.0]], [[1.0, 6.0, 
5.0], [3.0, 3.0, 5.0], [6.0, 3.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 8.0], [5.0, 6.0, 2.0], [7.0, 8.0, 1.0]], [[1.0, 4.0, 1.0], [7.0, 4.0, 
6.0], [6.0, 1.0, 4.0]], [[8.0, 5.0, 3.0], [6.0, 3.0, 1.0], [7.0, 1.0, 1.0]]], [[[2.0, 7.0, 3.0], [8.0, 3.0, 1.0], [2.0, 4.0, 
8.0]], [[1.0, 8.0, 3.0], [1.0, 2.0, 8.0], [6.0, 1.0, 6.0]], [[1.0, 4.0, 2.0], [8.0, 3.0, 6.0], [4.0, 7.0, 3.0]]], [[[1.0, 7.0, 
2.0], [8.0, 3.0, 7.0], [3.0, 1.0, 3.0]], [[1.0, 5.0, 1.0], [6.0, 1.0, 7.0], [1.0, 6.0, 7.0]], [[1.0, 7.0, 6.0], [6.0, 1.0, 
2.0], [4.0, 3.0, 1.0]]], [[[4.0, 1.0, 8.0], [4.0, 4.0, 1.0], [8.0, 6.0, 4.0]], [[7.0, 3.0, 5.0], [3.0, 1.0, 3.0], [4.0, 1.0, 
5.0]], [[2.0, 6.0, 3.0], [8.0, 7.0, 5.0], [8.0, 5.0, 5.0]]]]))
      val=numpy.array([[[[1.0, 4.0, 2.0], [4.0, 2.0, 6.0]], [[1.0, 8.0, 5.0], [5.0, 4.0, 2.0]]], [[[1.0, 1.0, 4.0], [1.0, 2.0, 
4.0]], [[3.0, 6.0, 1.0], [5.0, 2.0, 3.0]]]])
      arg[1:3,:2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 6.0], [4.0, 4.0, 8.0], [8.0, 6.0, 3.0]], [[1.0, 8.0, 2.0], [1.0, 6.0, 7.0], [2.0, 
4.0, 6.0]], [[2.0, 2.0, 4.0], [7.0, 2.0, 2.0], [3.0, 5.0, 6.0]]], [[[1.0, 4.0, 2.0], [4.0, 2.0, 6.0], [6.0, 3.0, 1.0]], [[1.0, 
8.0, 5.0], [5.0, 4.0, 2.0], [8.0, 1.0, 3.0]], [[2.0, 2.0, 5.0], [6.0, 8.0, 8.0], [7.0, 2.0, 2.0]]], [[[1.0, 1.0, 4.0], [1.0, 
2.0, 4.0], [2.0, 4.0, 5.0]], [[3.0, 6.0, 1.0], [5.0, 2.0, 3.0], [8.0, 8.0, 4.0]], [[2.0, 4.0, 7.0], [4.0, 7.0, 7.0], [4.0, 4.0, 
3.0]]], [[[6.0, 2.0, 5.0], [4.0, 1.0, 7.0], [8.0, 3.0, 1.0]], [[4.0, 4.0, 4.0], [8.0, 8.0, 6.0], [7.0, 4.0, 3.0]], [[1.0, 6.0, 
5.0], [3.0, 3.0, 5.0], [6.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 8.0], [5.0, 6.0, 2.0], [7.0, 8.0, 1.0]], 
[[1.0, 4.0, 1.0], [7.0, 4.0, 6.0], [6.0, 1.0, 4.0]], [[8.0, 5.0, 3.0], [6.0, 3.0, 1.0], [7.0, 1.0, 1.0]]], [[[1.0, 4.0, 2.0], 
[4.0, 2.0, 6.0], [2.0, 4.0, 8.0]], [[1.0, 8.0, 5.0], [5.0, 4.0, 2.0], [6.0, 1.0, 6.0]], [[1.0, 4.0, 2.0], [8.0, 3.0, 6.0], 
[4.0, 7.0, 3.0]]], [[[1.0, 1.0, 4.0], [1.0, 2.0, 4.0], [3.0, 1.0, 3.0]], [[3.0, 6.0, 1.0], [5.0, 2.0, 3.0], [1.0, 6.0, 7.0]], 
[[1.0, 7.0, 6.0], [6.0, 1.0, 2.0], [4.0, 3.0, 1.0]]], [[[4.0, 1.0, 8.0], [4.0, 4.0, 1.0], [8.0, 6.0, 4.0]], [[7.0, 3.0, 5.0], 
[3.0, 1.0, 3.0], [4.0, 1.0, 5.0]], [[2.0, 6.0, 3.0], [8.0, 7.0, 5.0], [8.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RUUC(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 2.0], [8.0, 4.0, 1.0], [3.0, 1.0, 1.0]], [[7.0, 1.0, 1.0], [2.0, 7.0, 4.0], [6.0, 3.0, 
3.0]], [[4.0, 5.0, 8.0], [3.0, 2.0, 7.0], [4.0, 4.0, 6.0]]], [[[2.0, 2.0, 5.0], [7.0, 5.0, 7.0], [3.0, 8.0, 7.0]], [[8.0, 8.0, 
2.0], [1.0, 6.0, 1.0], [8.0, 4.0, 8.0]], [[2.0, 6.0, 2.0], [3.0, 3.0, 3.0], [3.0, 1.0, 4.0]]], [[[4.0, 1.0, 8.0], [7.0, 7.0, 
8.0], [7.0, 1.0, 5.0]], [[3.0, 7.0, 8.0], [7.0, 7.0, 8.0], [8.0, 6.0, 1.0]], [[1.0, 2.0, 5.0], [1.0, 1.0, 3.0], [3.0, 8.0, 
1.0]]], [[[8.0, 7.0, 3.0], [6.0, 3.0, 6.0], [4.0, 7.0, 7.0]], [[8.0, 7.0, 3.0], [6.0, 5.0, 1.0], [1.0, 7.0, 7.0]], [[3.0, 4.0, 
7.0], [4.0, 5.0, 8.0], [6.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 7.0], [7.0, 8.0, 6.0], [5.0, 2.0, 3.0]], [[7.0, 1.0, 2.0], [5.0, 4.0, 
8.0], [3.0, 5.0, 5.0]], [[1.0, 5.0, 5.0], [2.0, 1.0, 2.0], [8.0, 3.0, 7.0]]], [[[3.0, 5.0, 5.0], [1.0, 3.0, 1.0], [7.0, 4.0, 
7.0]], [[8.0, 3.0, 8.0], [4.0, 2.0, 4.0], [7.0, 1.0, 6.0]], [[4.0, 7.0, 6.0], [1.0, 6.0, 4.0], [1.0, 4.0, 3.0]]], [[[2.0, 3.0, 
2.0], [4.0, 4.0, 8.0], [4.0, 8.0, 2.0]], [[3.0, 3.0, 5.0], [1.0, 1.0, 7.0], [7.0, 5.0, 3.0]], [[7.0, 4.0, 1.0], [4.0, 1.0, 
1.0], [4.0, 5.0, 3.0]]], [[[5.0, 1.0, 7.0], [4.0, 2.0, 6.0], [1.0, 4.0, 4.0]], [[2.0, 7.0, 7.0], [5.0, 7.0, 2.0], [4.0, 5.0, 
2.0]], [[4.0, 1.0, 8.0], [4.0, 8.0, 7.0], [6.0, 6.0, 4.0]]]]))
      val=Data(numpy.array([[[[3.0, 3.0, 4.0], [6.0, 1.0, 5.0]], [[7.0, 4.0, 5.0], [4.0, 7.0, 3.0]]], [[[4.0, 4.0, 6.0], [5.0, 
3.0, 6.0]], [[5.0, 8.0, 3.0], [6.0, 6.0, 5.0]]]]),self.functionspace)
      arg[1:3,:2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 2.0], [8.0, 4.0, 1.0], [3.0, 1.0, 1.0]], [[7.0, 1.0, 1.0], [2.0, 7.0, 4.0], [6.0, 
3.0, 3.0]], [[4.0, 5.0, 8.0], [3.0, 2.0, 7.0], [4.0, 4.0, 6.0]]], [[[3.0, 3.0, 4.0], [6.0, 1.0, 5.0], [3.0, 8.0, 7.0]], [[7.0, 
4.0, 5.0], [4.0, 7.0, 3.0], [8.0, 4.0, 8.0]], [[2.0, 6.0, 2.0], [3.0, 3.0, 3.0], [3.0, 1.0, 4.0]]], [[[4.0, 4.0, 6.0], [5.0, 
3.0, 6.0], [7.0, 1.0, 5.0]], [[5.0, 8.0, 3.0], [6.0, 6.0, 5.0], [8.0, 6.0, 1.0]], [[1.0, 2.0, 5.0], [1.0, 1.0, 3.0], [3.0, 8.0, 
1.0]]], [[[8.0, 7.0, 3.0], [6.0, 3.0, 6.0], [4.0, 7.0, 7.0]], [[8.0, 7.0, 3.0], [6.0, 5.0, 1.0], [1.0, 7.0, 7.0]], [[3.0, 4.0, 
7.0], [4.0, 5.0, 8.0], [6.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 7.0], [7.0, 8.0, 6.0], [5.0, 2.0, 3.0]], 
[[7.0, 1.0, 2.0], [5.0, 4.0, 8.0], [3.0, 5.0, 5.0]], [[1.0, 5.0, 5.0], [2.0, 1.0, 2.0], [8.0, 3.0, 7.0]]], [[[3.0, 3.0, 4.0], 
[6.0, 1.0, 5.0], [7.0, 4.0, 7.0]], [[7.0, 4.0, 5.0], [4.0, 7.0, 3.0], [7.0, 1.0, 6.0]], [[4.0, 7.0, 6.0], [1.0, 6.0, 4.0], 
[1.0, 4.0, 3.0]]], [[[4.0, 4.0, 6.0], [5.0, 3.0, 6.0], [4.0, 8.0, 2.0]], [[5.0, 8.0, 3.0], [6.0, 6.0, 5.0], [7.0, 5.0, 3.0]], 
[[7.0, 4.0, 1.0], [4.0, 1.0, 1.0], [4.0, 5.0, 3.0]]], [[[5.0, 1.0, 7.0], [4.0, 2.0, 6.0], [1.0, 4.0, 4.0]], [[2.0, 7.0, 7.0], 
[5.0, 7.0, 2.0], [4.0, 5.0, 2.0]], [[4.0, 1.0, 8.0], [4.0, 8.0, 7.0], [6.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RUUC(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 3.0], [4.0, 3.0, 8.0], [1.0, 3.0, 2.0]], [[8.0, 7.0, 5.0], [3.0, 3.0, 6.0], [1.0, 3.0, 
1.0]], [[4.0, 7.0, 4.0], [4.0, 2.0, 7.0], [4.0, 5.0, 1.0]]], [[[5.0, 3.0, 2.0], [8.0, 3.0, 2.0], [1.0, 7.0, 6.0]], [[6.0, 3.0, 
5.0], [5.0, 3.0, 2.0], [3.0, 3.0, 8.0]], [[1.0, 7.0, 4.0], [8.0, 8.0, 5.0], [6.0, 8.0, 5.0]]], [[[8.0, 5.0, 6.0], [6.0, 5.0, 
3.0], [6.0, 1.0, 8.0]], [[8.0, 3.0, 7.0], [3.0, 2.0, 2.0], [4.0, 2.0, 2.0]], [[7.0, 8.0, 1.0], [6.0, 6.0, 5.0], [8.0, 2.0, 
2.0]]], [[[4.0, 8.0, 6.0], [3.0, 4.0, 6.0], [3.0, 5.0, 3.0]], [[5.0, 1.0, 8.0], [3.0, 7.0, 6.0], [5.0, 3.0, 7.0]], [[2.0, 8.0, 
3.0], [5.0, 3.0, 8.0], [2.0, 5.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 7.0], [3.0, 6.0, 4.0], [7.0, 8.0, 2.0]], [[5.0, 4.0, 4.0], [5.0, 5.0, 
6.0], [2.0, 3.0, 6.0]], [[5.0, 7.0, 8.0], [7.0, 5.0, 1.0], [1.0, 1.0, 7.0]]], [[[8.0, 3.0, 2.0], [6.0, 6.0, 3.0], [5.0, 2.0, 
4.0]], [[1.0, 6.0, 6.0], [4.0, 4.0, 7.0], [7.0, 4.0, 2.0]], [[6.0, 6.0, 2.0], [3.0, 2.0, 3.0], [7.0, 7.0, 7.0]]], [[[8.0, 3.0, 
8.0], [4.0, 1.0, 8.0], [8.0, 3.0, 2.0]], [[3.0, 5.0, 8.0], [7.0, 4.0, 1.0], [8.0, 4.0, 3.0]], [[8.0, 7.0, 6.0], [5.0, 7.0, 
5.0], [2.0, 1.0, 6.0]]], [[[4.0, 3.0, 1.0], [5.0, 2.0, 7.0], [6.0, 3.0, 1.0]], [[3.0, 6.0, 7.0], [5.0, 3.0, 2.0], [5.0, 1.0, 
3.0]], [[2.0, 5.0, 8.0], [5.0, 5.0, 6.0], [7.0, 5.0, 7.0]]]]))
      val=Data(numpy.array([[[[4.0, 5.0, 3.0], [5.0, 8.0, 2.0]], [[7.0, 7.0, 6.0], [7.0, 3.0, 5.0]]], [[[6.0, 6.0, 4.0], [5.0, 
4.0, 7.0]], [[6.0, 7.0, 3.0], [6.0, 6.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 5.0], [8.0, 7.0, 4.0]], [[2.0, 1.0, 2.0], [5.0, 7.0, 6.0]]], [[[7.0, 8.0, 
1.0], [8.0, 6.0, 2.0]], [[3.0, 7.0, 2.0], [6.0, 3.0, 1.0]]]]))
      arg[1:3,:2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 3.0], [4.0, 3.0, 8.0], [1.0, 3.0, 2.0]], [[8.0, 7.0, 5.0], [3.0, 3.0, 6.0], [1.0, 
3.0, 1.0]], [[4.0, 7.0, 4.0], [4.0, 2.0, 7.0], [4.0, 5.0, 1.0]]], [[[4.0, 5.0, 3.0], [5.0, 8.0, 2.0], [1.0, 7.0, 6.0]], [[7.0, 
7.0, 6.0], [7.0, 3.0, 5.0], [3.0, 3.0, 8.0]], [[1.0, 7.0, 4.0], [8.0, 8.0, 5.0], [6.0, 8.0, 5.0]]], [[[6.0, 6.0, 4.0], [5.0, 
4.0, 7.0], [6.0, 1.0, 8.0]], [[6.0, 7.0, 3.0], [6.0, 6.0, 1.0], [4.0, 2.0, 2.0]], [[7.0, 8.0, 1.0], [6.0, 6.0, 5.0], [8.0, 2.0, 
2.0]]], [[[4.0, 8.0, 6.0], [3.0, 4.0, 6.0], [3.0, 5.0, 3.0]], [[5.0, 1.0, 8.0], [3.0, 7.0, 6.0], [5.0, 3.0, 7.0]], [[2.0, 8.0, 
3.0], [5.0, 3.0, 8.0], [2.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 7.0], [3.0, 6.0, 4.0], [7.0, 8.0, 2.0]], 
[[5.0, 4.0, 4.0], [5.0, 5.0, 6.0], [2.0, 3.0, 6.0]], [[5.0, 7.0, 8.0], [7.0, 5.0, 1.0], [1.0, 1.0, 7.0]]], [[[5.0, 3.0, 5.0], 
[8.0, 7.0, 4.0], [5.0, 2.0, 4.0]], [[2.0, 1.0, 2.0], [5.0, 7.0, 6.0], [7.0, 4.0, 2.0]], [[6.0, 6.0, 2.0], [3.0, 2.0, 3.0], 
[7.0, 7.0, 7.0]]], [[[7.0, 8.0, 1.0], [8.0, 6.0, 2.0], [8.0, 3.0, 2.0]], [[3.0, 7.0, 2.0], [6.0, 3.0, 1.0], [8.0, 4.0, 3.0]], 
[[8.0, 7.0, 6.0], [5.0, 7.0, 5.0], [2.0, 1.0, 6.0]]], [[[4.0, 3.0, 1.0], [5.0, 2.0, 7.0], [6.0, 3.0, 1.0]], [[3.0, 6.0, 7.0], 
[5.0, 3.0, 2.0], [5.0, 1.0, 3.0]], [[2.0, 5.0, 8.0], [5.0, 5.0, 6.0], [7.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RUUC(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 3.0], [7.0, 5.0, 6.0], [7.0, 6.0, 1.0]], [[7.0, 5.0, 8.0], [3.0, 1.0, 4.0], [7.0, 3.0, 
4.0]], [[8.0, 7.0, 5.0], [6.0, 2.0, 5.0], [3.0, 8.0, 7.0]]], [[[4.0, 4.0, 8.0], [2.0, 2.0, 5.0], [2.0, 3.0, 8.0]], [[2.0, 7.0, 
1.0], [1.0, 5.0, 6.0], [7.0, 1.0, 7.0]], [[3.0, 1.0, 6.0], [7.0, 8.0, 2.0], [2.0, 3.0, 7.0]]], [[[4.0, 5.0, 8.0], [8.0, 1.0, 
3.0], [1.0, 3.0, 4.0]], [[3.0, 5.0, 4.0], [8.0, 6.0, 4.0], [4.0, 2.0, 6.0]], [[8.0, 2.0, 3.0], [4.0, 2.0, 8.0], [8.0, 2.0, 
6.0]]], [[[1.0, 7.0, 6.0], [6.0, 5.0, 5.0], [7.0, 2.0, 7.0]], [[8.0, 8.0, 1.0], [5.0, 1.0, 5.0], [4.0, 8.0, 6.0]], [[6.0, 2.0, 
2.0], [1.0, 4.0, 3.0], [2.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 5.0], [7.0, 1.0, 5.0], [5.0, 2.0, 3.0]], [[5.0, 5.0, 1.0], [7.0, 3.0, 
4.0], [7.0, 7.0, 8.0]], [[6.0, 1.0, 4.0], [3.0, 2.0, 7.0], [8.0, 3.0, 6.0]]], [[[4.0, 8.0, 5.0], [3.0, 5.0, 3.0], [3.0, 7.0, 
4.0]], [[3.0, 8.0, 2.0], [5.0, 6.0, 7.0], [1.0, 4.0, 2.0]], [[1.0, 6.0, 3.0], [7.0, 4.0, 2.0], [8.0, 2.0, 2.0]]], [[[8.0, 6.0, 
7.0], [6.0, 1.0, 2.0], [7.0, 5.0, 2.0]], [[8.0, 7.0, 2.0], [7.0, 4.0, 1.0], [1.0, 4.0, 5.0]], [[2.0, 4.0, 7.0], [2.0, 6.0, 
8.0], [1.0, 6.0, 2.0]]], [[[6.0, 4.0, 2.0], [7.0, 1.0, 3.0], [5.0, 8.0, 2.0]], [[6.0, 6.0, 1.0], [1.0, 7.0, 6.0], [3.0, 7.0, 
8.0]], [[3.0, 3.0, 8.0], [6.0, 8.0, 5.0], [2.0, 6.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 4.0, 2.0], [8.0, 4.0, 1.0]], [[5.0, 8.0, 2.0], [1.0, 7.0, 2.0]]], [[[5.0, 1.0, 8.0], 
[5.0, 4.0, 1.0]], [[1.0, 2.0, 5.0], [2.0, 2.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 3.0, 8.0], [2.0, 2.0, 3.0]], [[6.0, 
7.0, 8.0], [1.0, 8.0, 2.0]]], [[[5.0, 4.0, 6.0], [2.0, 8.0, 1.0]], [[2.0, 8.0, 2.0], [5.0, 3.0, 7.0]]]])
      arg[1:3,:2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 3.0], [7.0, 5.0, 6.0], [7.0, 6.0, 1.0]], [[7.0, 5.0, 8.0], [3.0, 1.0, 4.0], [7.0, 
3.0, 4.0]], [[8.0, 7.0, 5.0], [6.0, 2.0, 5.0], [3.0, 8.0, 7.0]]], [[[8.0, 4.0, 2.0], [8.0, 4.0, 1.0], [2.0, 3.0, 8.0]], [[5.0, 
8.0, 2.0], [1.0, 7.0, 2.0], [7.0, 1.0, 7.0]], [[3.0, 1.0, 6.0], [7.0, 8.0, 2.0], [2.0, 3.0, 7.0]]], [[[5.0, 1.0, 8.0], [5.0, 
4.0, 1.0], [1.0, 3.0, 4.0]], [[1.0, 2.0, 5.0], [2.0, 2.0, 4.0], [4.0, 2.0, 6.0]], [[8.0, 2.0, 3.0], [4.0, 2.0, 8.0], [8.0, 2.0, 
6.0]]], [[[1.0, 7.0, 6.0], [6.0, 5.0, 5.0], [7.0, 2.0, 7.0]], [[8.0, 8.0, 1.0], [5.0, 1.0, 5.0], [4.0, 8.0, 6.0]], [[6.0, 2.0, 
2.0], [1.0, 4.0, 3.0], [2.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 5.0], [7.0, 1.0, 5.0], [5.0, 2.0, 3.0]], 
[[5.0, 5.0, 1.0], [7.0, 3.0, 4.0], [7.0, 7.0, 8.0]], [[6.0, 1.0, 4.0], [3.0, 2.0, 7.0], [8.0, 3.0, 6.0]]], [[[6.0, 3.0, 8.0], 
[2.0, 2.0, 3.0], [3.0, 7.0, 4.0]], [[6.0, 7.0, 8.0], [1.0, 8.0, 2.0], [1.0, 4.0, 2.0]], [[1.0, 6.0, 3.0], [7.0, 4.0, 2.0], 
[8.0, 2.0, 2.0]]], [[[5.0, 4.0, 6.0], [2.0, 8.0, 1.0], [7.0, 5.0, 2.0]], [[2.0, 8.0, 2.0], [5.0, 3.0, 7.0], [1.0, 4.0, 5.0]], 
[[2.0, 4.0, 7.0], [2.0, 6.0, 8.0], [1.0, 6.0, 2.0]]], [[[6.0, 4.0, 2.0], [7.0, 1.0, 3.0], [5.0, 8.0, 2.0]], [[6.0, 6.0, 1.0], 
[1.0, 7.0, 6.0], [3.0, 7.0, 8.0]], [[3.0, 3.0, 8.0], [6.0, 8.0, 5.0], [2.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PRRR(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 4.0], [5.0, 4.0, 2.0], [6.0, 4.0, 1.0]], [[7.0, 3.0, 5.0], [3.0, 7.0, 4.0], [7.0, 3.0, 
6.0]], [[7.0, 5.0, 6.0], [1.0, 4.0, 4.0], [6.0, 8.0, 8.0]]], [[[4.0, 7.0, 2.0], [2.0, 4.0, 3.0], [4.0, 4.0, 6.0]], [[8.0, 6.0, 
8.0], [3.0, 2.0, 5.0], [8.0, 8.0, 1.0]], [[1.0, 4.0, 8.0], [7.0, 3.0, 2.0], [4.0, 4.0, 4.0]]], [[[2.0, 6.0, 8.0], [5.0, 2.0, 
2.0], [5.0, 3.0, 5.0]], [[7.0, 3.0, 2.0], [3.0, 1.0, 6.0], [6.0, 8.0, 5.0]], [[6.0, 5.0, 7.0], [6.0, 1.0, 3.0], [7.0, 6.0, 
7.0]]], [[[2.0, 6.0, 3.0], [2.0, 3.0, 5.0], [3.0, 6.0, 4.0]], [[5.0, 5.0, 4.0], [4.0, 3.0, 7.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 
5.0], [5.0, 6.0, 4.0], [8.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 1.0], [3.0, 6.0, 5.0], [1.0, 1.0, 7.0]], [[1.0, 3.0, 8.0], [4.0, 6.0, 
4.0], [2.0, 8.0, 4.0]], [[6.0, 2.0, 6.0], [3.0, 6.0, 2.0], [8.0, 6.0, 6.0]]], [[[4.0, 3.0, 6.0], [6.0, 6.0, 3.0], [8.0, 7.0, 
6.0]], [[2.0, 5.0, 6.0], [5.0, 2.0, 4.0], [2.0, 8.0, 7.0]], [[8.0, 5.0, 1.0], [2.0, 6.0, 7.0], [1.0, 4.0, 2.0]]], [[[8.0, 1.0, 
4.0], [8.0, 1.0, 7.0], [8.0, 6.0, 6.0]], [[2.0, 6.0, 8.0], [2.0, 7.0, 8.0], [1.0, 3.0, 4.0]], [[5.0, 4.0, 6.0], [8.0, 4.0, 
1.0], [3.0, 4.0, 2.0]]], [[[3.0, 7.0, 5.0], [8.0, 2.0, 7.0], [2.0, 7.0, 6.0]], [[5.0, 8.0, 3.0], [2.0, 4.0, 1.0], [4.0, 7.0, 
4.0]], [[3.0, 8.0, 1.0], [4.0, 7.0, 6.0], [2.0, 8.0, 2.0]]]]))
      val=numpy.array([[[5.0]]])
      arg[2,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 4.0], [5.0, 4.0, 2.0], [6.0, 4.0, 1.0]], [[7.0, 3.0, 5.0], [3.0, 7.0, 4.0], [7.0, 
3.0, 6.0]], [[7.0, 5.0, 6.0], [1.0, 4.0, 4.0], [6.0, 8.0, 8.0]]], [[[4.0, 7.0, 2.0], [2.0, 4.0, 3.0], [4.0, 4.0, 6.0]], [[8.0, 
6.0, 8.0], [3.0, 2.0, 5.0], [8.0, 8.0, 1.0]], [[1.0, 4.0, 8.0], [7.0, 3.0, 2.0], [4.0, 4.0, 4.0]]], [[[2.0, 6.0, 8.0], [5.0, 
2.0, 2.0], [5.0, 3.0, 5.0]], [[7.0, 3.0, 2.0], [3.0, 5.0, 6.0], [6.0, 8.0, 5.0]], [[6.0, 5.0, 7.0], [6.0, 1.0, 3.0], [7.0, 6.0, 
7.0]]], [[[2.0, 6.0, 3.0], [2.0, 3.0, 5.0], [3.0, 6.0, 4.0]], [[5.0, 5.0, 4.0], [4.0, 3.0, 7.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 
5.0], [5.0, 6.0, 4.0], [8.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 1.0], [3.0, 6.0, 5.0], [1.0, 1.0, 7.0]], 
[[1.0, 3.0, 8.0], [4.0, 6.0, 4.0], [2.0, 8.0, 4.0]], [[6.0, 2.0, 6.0], [3.0, 6.0, 2.0], [8.0, 6.0, 6.0]]], [[[4.0, 3.0, 6.0], 
[6.0, 6.0, 3.0], [8.0, 7.0, 6.0]], [[2.0, 5.0, 6.0], [5.0, 2.0, 4.0], [2.0, 8.0, 7.0]], [[8.0, 5.0, 1.0], [2.0, 6.0, 7.0], 
[1.0, 4.0, 2.0]]], [[[8.0, 1.0, 4.0], [8.0, 1.0, 7.0], [8.0, 6.0, 6.0]], [[2.0, 6.0, 8.0], [2.0, 5.0, 8.0], [1.0, 3.0, 4.0]], 
[[5.0, 4.0, 6.0], [8.0, 4.0, 1.0], [3.0, 4.0, 2.0]]], [[[3.0, 7.0, 5.0], [8.0, 2.0, 7.0], [2.0, 7.0, 6.0]], [[5.0, 8.0, 3.0], 
[2.0, 4.0, 1.0], [4.0, 7.0, 4.0]], [[3.0, 8.0, 1.0], [4.0, 7.0, 6.0], [2.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PRRR(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [8.0, 5.0, 5.0], [7.0, 8.0, 3.0]], [[1.0, 1.0, 3.0], [5.0, 8.0, 5.0], [1.0, 3.0, 
5.0]], [[2.0, 4.0, 6.0], [4.0, 7.0, 8.0], [1.0, 3.0, 3.0]]], [[[6.0, 7.0, 5.0], [5.0, 7.0, 5.0], [7.0, 3.0, 7.0]], [[7.0, 5.0, 
1.0], [2.0, 8.0, 6.0], [8.0, 5.0, 8.0]], [[4.0, 7.0, 6.0], [5.0, 4.0, 5.0], [7.0, 5.0, 8.0]]], [[[6.0, 2.0, 7.0], [3.0, 7.0, 
7.0], [5.0, 7.0, 3.0]], [[7.0, 1.0, 3.0], [3.0, 3.0, 2.0], [2.0, 4.0, 3.0]], [[7.0, 2.0, 1.0], [5.0, 2.0, 5.0], [2.0, 6.0, 
4.0]]], [[[5.0, 8.0, 3.0], [6.0, 4.0, 1.0], [5.0, 3.0, 7.0]], [[6.0, 8.0, 4.0], [8.0, 4.0, 2.0], [2.0, 7.0, 8.0]], [[1.0, 5.0, 
6.0], [5.0, 6.0, 2.0], [8.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 3.0], [2.0, 2.0, 5.0], [5.0, 7.0, 1.0]], [[4.0, 2.0, 5.0], [6.0, 5.0, 
6.0], [2.0, 6.0, 6.0]], [[7.0, 2.0, 3.0], [6.0, 3.0, 4.0], [5.0, 6.0, 1.0]]], [[[3.0, 3.0, 7.0], [5.0, 6.0, 8.0], [8.0, 1.0, 
7.0]], [[8.0, 3.0, 2.0], [4.0, 1.0, 2.0], [4.0, 2.0, 2.0]], [[3.0, 2.0, 8.0], [1.0, 5.0, 8.0], [5.0, 4.0, 1.0]]], [[[8.0, 6.0, 
3.0], [6.0, 3.0, 2.0], [2.0, 1.0, 4.0]], [[7.0, 4.0, 3.0], [6.0, 4.0, 6.0], [3.0, 3.0, 1.0]], [[3.0, 6.0, 3.0], [2.0, 5.0, 
7.0], [4.0, 8.0, 3.0]]], [[[2.0, 6.0, 2.0], [8.0, 1.0, 4.0], [2.0, 1.0, 5.0]], [[2.0, 1.0, 5.0], [6.0, 2.0, 4.0], [2.0, 1.0, 
7.0]], [[1.0, 8.0, 7.0], [2.0, 3.0, 5.0], [3.0, 4.0, 5.0]]]]))
      val=Data(numpy.array([[[7.0]]]),self.functionspace)
      arg[2,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [8.0, 5.0, 5.0], [7.0, 8.0, 3.0]], [[1.0, 1.0, 3.0], [5.0, 8.0, 5.0], [1.0, 
3.0, 5.0]], [[2.0, 4.0, 6.0], [4.0, 7.0, 8.0], [1.0, 3.0, 3.0]]], [[[6.0, 7.0, 5.0], [5.0, 7.0, 5.0], [7.0, 3.0, 7.0]], [[7.0, 
5.0, 1.0], [2.0, 8.0, 6.0], [8.0, 5.0, 8.0]], [[4.0, 7.0, 6.0], [5.0, 4.0, 5.0], [7.0, 5.0, 8.0]]], [[[6.0, 2.0, 7.0], [3.0, 
7.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 1.0, 3.0], [3.0, 7.0, 2.0], [2.0, 4.0, 3.0]], [[7.0, 2.0, 1.0], [5.0, 2.0, 5.0], [2.0, 6.0, 
4.0]]], [[[5.0, 8.0, 3.0], [6.0, 4.0, 1.0], [5.0, 3.0, 7.0]], [[6.0, 8.0, 4.0], [8.0, 4.0, 2.0], [2.0, 7.0, 8.0]], [[1.0, 5.0, 
6.0], [5.0, 6.0, 2.0], [8.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 3.0], [2.0, 2.0, 5.0], [5.0, 7.0, 1.0]], 
[[4.0, 2.0, 5.0], [6.0, 5.0, 6.0], [2.0, 6.0, 6.0]], [[7.0, 2.0, 3.0], [6.0, 3.0, 4.0], [5.0, 6.0, 1.0]]], [[[3.0, 3.0, 7.0], 
[5.0, 6.0, 8.0], [8.0, 1.0, 7.0]], [[8.0, 3.0, 2.0], [4.0, 1.0, 2.0], [4.0, 2.0, 2.0]], [[3.0, 2.0, 8.0], [1.0, 5.0, 8.0], 
[5.0, 4.0, 1.0]]], [[[8.0, 6.0, 3.0], [6.0, 3.0, 2.0], [2.0, 1.0, 4.0]], [[7.0, 4.0, 3.0], [6.0, 7.0, 6.0], [3.0, 3.0, 1.0]], 
[[3.0, 6.0, 3.0], [2.0, 5.0, 7.0], [4.0, 8.0, 3.0]]], [[[2.0, 6.0, 2.0], [8.0, 1.0, 4.0], [2.0, 1.0, 5.0]], [[2.0, 1.0, 5.0], 
[6.0, 2.0, 4.0], [2.0, 1.0, 7.0]], [[1.0, 8.0, 7.0], [2.0, 3.0, 5.0], [3.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PRRR(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 8.0], [6.0, 1.0, 5.0], [5.0, 1.0, 1.0]], [[5.0, 4.0, 4.0], [7.0, 2.0, 1.0], [7.0, 8.0, 
8.0]], [[5.0, 7.0, 1.0], [4.0, 7.0, 4.0], [7.0, 7.0, 6.0]]], [[[7.0, 8.0, 5.0], [8.0, 3.0, 7.0], [8.0, 1.0, 1.0]], [[5.0, 6.0, 
7.0], [8.0, 5.0, 5.0], [8.0, 7.0, 2.0]], [[7.0, 2.0, 8.0], [3.0, 5.0, 8.0], [6.0, 5.0, 8.0]]], [[[4.0, 1.0, 6.0], [6.0, 4.0, 
4.0], [3.0, 3.0, 7.0]], [[7.0, 5.0, 4.0], [8.0, 5.0, 4.0], [7.0, 5.0, 4.0]], [[6.0, 8.0, 8.0], [5.0, 8.0, 4.0], [8.0, 2.0, 
7.0]]], [[[1.0, 3.0, 2.0], [5.0, 5.0, 2.0], [3.0, 5.0, 3.0]], [[2.0, 8.0, 6.0], [8.0, 7.0, 3.0], [4.0, 2.0, 2.0]], [[2.0, 8.0, 
2.0], [4.0, 6.0, 3.0], [6.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 3.0], [5.0, 6.0, 6.0], [5.0, 1.0, 5.0]], [[5.0, 3.0, 6.0], [3.0, 7.0, 
8.0], [6.0, 2.0, 3.0]], [[2.0, 2.0, 8.0], [1.0, 3.0, 5.0], [1.0, 5.0, 6.0]]], [[[7.0, 6.0, 1.0], [1.0, 4.0, 2.0], [3.0, 3.0, 
6.0]], [[4.0, 3.0, 8.0], [4.0, 2.0, 3.0], [6.0, 5.0, 3.0]], [[2.0, 7.0, 6.0], [5.0, 5.0, 1.0], [1.0, 5.0, 1.0]]], [[[5.0, 4.0, 
4.0], [6.0, 2.0, 5.0], [6.0, 3.0, 4.0]], [[6.0, 7.0, 8.0], [3.0, 5.0, 6.0], [1.0, 2.0, 2.0]], [[7.0, 1.0, 5.0], [1.0, 3.0, 
1.0], [4.0, 8.0, 5.0]]], [[[1.0, 5.0, 4.0], [1.0, 7.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 4.0, 3.0], [7.0, 1.0, 6.0], [1.0, 4.0, 
3.0]], [[5.0, 2.0, 2.0], [2.0, 7.0, 3.0], [4.0, 5.0, 6.0]]]]))
      val=Data(numpy.array([[[5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0]]]))
      arg[2,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 8.0], [6.0, 1.0, 5.0], [5.0, 1.0, 1.0]], [[5.0, 4.0, 4.0], [7.0, 2.0, 1.0], [7.0, 
8.0, 8.0]], [[5.0, 7.0, 1.0], [4.0, 7.0, 4.0], [7.0, 7.0, 6.0]]], [[[7.0, 8.0, 5.0], [8.0, 3.0, 7.0], [8.0, 1.0, 1.0]], [[5.0, 
6.0, 7.0], [8.0, 5.0, 5.0], [8.0, 7.0, 2.0]], [[7.0, 2.0, 8.0], [3.0, 5.0, 8.0], [6.0, 5.0, 8.0]]], [[[4.0, 1.0, 6.0], [6.0, 
4.0, 4.0], [3.0, 3.0, 7.0]], [[7.0, 5.0, 4.0], [8.0, 5.0, 4.0], [7.0, 5.0, 4.0]], [[6.0, 8.0, 8.0], [5.0, 8.0, 4.0], [8.0, 2.0, 
7.0]]], [[[1.0, 3.0, 2.0], [5.0, 5.0, 2.0], [3.0, 5.0, 3.0]], [[2.0, 8.0, 6.0], [8.0, 7.0, 3.0], [4.0, 2.0, 2.0]], [[2.0, 8.0, 
2.0], [4.0, 6.0, 3.0], [6.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 3.0], [5.0, 6.0, 6.0], [5.0, 1.0, 5.0]], 
[[5.0, 3.0, 6.0], [3.0, 7.0, 8.0], [6.0, 2.0, 3.0]], [[2.0, 2.0, 8.0], [1.0, 3.0, 5.0], [1.0, 5.0, 6.0]]], [[[7.0, 6.0, 1.0], 
[1.0, 4.0, 2.0], [3.0, 3.0, 6.0]], [[4.0, 3.0, 8.0], [4.0, 2.0, 3.0], [6.0, 5.0, 3.0]], [[2.0, 7.0, 6.0], [5.0, 5.0, 1.0], 
[1.0, 5.0, 1.0]]], [[[5.0, 4.0, 4.0], [6.0, 2.0, 5.0], [6.0, 3.0, 4.0]], [[6.0, 7.0, 8.0], [3.0, 1.0, 6.0], [1.0, 2.0, 2.0]], 
[[7.0, 1.0, 5.0], [1.0, 3.0, 1.0], [4.0, 8.0, 5.0]]], [[[1.0, 5.0, 4.0], [1.0, 7.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 4.0, 3.0], 
[7.0, 1.0, 6.0], [1.0, 4.0, 3.0]], [[5.0, 2.0, 2.0], [2.0, 7.0, 3.0], [4.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PRRR(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 4.0], [4.0, 2.0, 7.0], [3.0, 6.0, 5.0]], [[4.0, 4.0, 8.0], [4.0, 7.0, 4.0], [6.0, 4.0, 
3.0]], [[8.0, 1.0, 2.0], [1.0, 4.0, 1.0], [1.0, 7.0, 8.0]]], [[[7.0, 2.0, 8.0], [2.0, 3.0, 4.0], [8.0, 4.0, 3.0]], [[4.0, 2.0, 
2.0], [8.0, 5.0, 7.0], [1.0, 8.0, 2.0]], [[3.0, 3.0, 6.0], [7.0, 4.0, 8.0], [6.0, 7.0, 6.0]]], [[[3.0, 4.0, 8.0], [2.0, 1.0, 
3.0], [8.0, 7.0, 5.0]], [[8.0, 6.0, 6.0], [6.0, 1.0, 3.0], [2.0, 4.0, 5.0]], [[2.0, 7.0, 1.0], [5.0, 8.0, 7.0], [4.0, 5.0, 
5.0]]], [[[4.0, 8.0, 7.0], [5.0, 8.0, 3.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 2.0], [1.0, 1.0, 8.0], [6.0, 7.0, 6.0]], [[6.0, 1.0, 
3.0], [6.0, 1.0, 6.0], [3.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 6.0], [2.0, 8.0, 4.0], [3.0, 7.0, 4.0]], [[5.0, 2.0, 2.0], [1.0, 4.0, 
1.0], [3.0, 6.0, 8.0]], [[2.0, 6.0, 5.0], [1.0, 3.0, 6.0], [8.0, 4.0, 2.0]]], [[[4.0, 1.0, 2.0], [3.0, 6.0, 7.0], [2.0, 6.0, 
2.0]], [[5.0, 5.0, 6.0], [8.0, 4.0, 4.0], [7.0, 2.0, 8.0]], [[8.0, 8.0, 8.0], [2.0, 5.0, 1.0], [4.0, 4.0, 7.0]]], [[[7.0, 2.0, 
8.0], [6.0, 8.0, 5.0], [2.0, 7.0, 1.0]], [[6.0, 2.0, 5.0], [4.0, 6.0, 4.0], [8.0, 1.0, 2.0]], [[3.0, 2.0, 4.0], [7.0, 5.0, 
3.0], [3.0, 4.0, 3.0]]], [[[8.0, 4.0, 6.0], [8.0, 3.0, 4.0], [4.0, 2.0, 5.0]], [[8.0, 8.0, 4.0], [1.0, 7.0, 6.0], [3.0, 2.0, 
6.0]], [[8.0, 4.0, 7.0], [3.0, 1.0, 3.0], [7.0, 1.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0]]])+(1.-msk_val)*numpy.array([[[2.0]]])
      arg[2,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 4.0], [4.0, 2.0, 7.0], [3.0, 6.0, 5.0]], [[4.0, 4.0, 8.0], [4.0, 7.0, 4.0], [6.0, 
4.0, 3.0]], [[8.0, 1.0, 2.0], [1.0, 4.0, 1.0], [1.0, 7.0, 8.0]]], [[[7.0, 2.0, 8.0], [2.0, 3.0, 4.0], [8.0, 4.0, 3.0]], [[4.0, 
2.0, 2.0], [8.0, 5.0, 7.0], [1.0, 8.0, 2.0]], [[3.0, 3.0, 6.0], [7.0, 4.0, 8.0], [6.0, 7.0, 6.0]]], [[[3.0, 4.0, 8.0], [2.0, 
1.0, 3.0], [8.0, 7.0, 5.0]], [[8.0, 6.0, 6.0], [6.0, 5.0, 3.0], [2.0, 4.0, 5.0]], [[2.0, 7.0, 1.0], [5.0, 8.0, 7.0], [4.0, 5.0, 
5.0]]], [[[4.0, 8.0, 7.0], [5.0, 8.0, 3.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 2.0], [1.0, 1.0, 8.0], [6.0, 7.0, 6.0]], [[6.0, 1.0, 
3.0], [6.0, 1.0, 6.0], [3.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [2.0, 8.0, 4.0], [3.0, 7.0, 4.0]], 
[[5.0, 2.0, 2.0], [1.0, 4.0, 1.0], [3.0, 6.0, 8.0]], [[2.0, 6.0, 5.0], [1.0, 3.0, 6.0], [8.0, 4.0, 2.0]]], [[[4.0, 1.0, 2.0], 
[3.0, 6.0, 7.0], [2.0, 6.0, 2.0]], [[5.0, 5.0, 6.0], [8.0, 4.0, 4.0], [7.0, 2.0, 8.0]], [[8.0, 8.0, 8.0], [2.0, 5.0, 1.0], 
[4.0, 4.0, 7.0]]], [[[7.0, 2.0, 8.0], [6.0, 8.0, 5.0], [2.0, 7.0, 1.0]], [[6.0, 2.0, 5.0], [4.0, 2.0, 4.0], [8.0, 1.0, 2.0]], 
[[3.0, 2.0, 4.0], [7.0, 5.0, 3.0], [3.0, 4.0, 3.0]]], [[[8.0, 4.0, 6.0], [8.0, 3.0, 4.0], [4.0, 2.0, 5.0]], [[8.0, 8.0, 4.0], 
[1.0, 7.0, 6.0], [3.0, 2.0, 6.0]], [[8.0, 4.0, 7.0], [3.0, 1.0, 3.0], [7.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRUN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 8.0], [4.0, 2.0, 5.0], [3.0, 4.0, 2.0]], [[6.0, 2.0, 3.0], [5.0, 1.0, 4.0], [2.0, 7.0, 
5.0]], [[5.0, 8.0, 2.0], [6.0, 4.0, 6.0], [5.0, 5.0, 8.0]]], [[[1.0, 7.0, 5.0], [3.0, 5.0, 1.0], [2.0, 7.0, 1.0]], [[5.0, 5.0, 
5.0], [1.0, 3.0, 7.0], [7.0, 8.0, 6.0]], [[5.0, 2.0, 2.0], [1.0, 8.0, 7.0], [2.0, 2.0, 5.0]]], [[[4.0, 6.0, 4.0], [5.0, 6.0, 
3.0], [2.0, 2.0, 8.0]], [[8.0, 4.0, 3.0], [4.0, 2.0, 2.0], [4.0, 3.0, 5.0]], [[2.0, 6.0, 7.0], [1.0, 4.0, 6.0], [7.0, 5.0, 
5.0]]], [[[2.0, 5.0, 4.0], [7.0, 6.0, 4.0], [7.0, 7.0, 4.0]], [[5.0, 4.0, 4.0], [8.0, 7.0, 1.0], [7.0, 2.0, 6.0]], [[6.0, 2.0, 
6.0], [5.0, 2.0, 1.0], [2.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 4.0], [8.0, 5.0, 6.0], [3.0, 1.0, 7.0]], [[8.0, 2.0, 7.0], [4.0, 1.0, 
7.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 3.0], [2.0, 4.0, 4.0], [1.0, 1.0, 5.0]]], [[[8.0, 2.0, 8.0], [2.0, 7.0, 6.0], [7.0, 6.0, 
4.0]], [[1.0, 1.0, 4.0], [7.0, 6.0, 3.0], [4.0, 2.0, 6.0]], [[3.0, 6.0, 7.0], [8.0, 4.0, 5.0], [6.0, 3.0, 1.0]]], [[[1.0, 7.0, 
7.0], [2.0, 4.0, 5.0], [2.0, 3.0, 3.0]], [[7.0, 2.0, 6.0], [6.0, 5.0, 4.0], [6.0, 2.0, 1.0]], [[3.0, 1.0, 8.0], [4.0, 5.0, 
2.0], [1.0, 3.0, 3.0]]], [[[8.0, 8.0, 6.0], [6.0, 3.0, 1.0], [3.0, 2.0, 3.0]], [[1.0, 7.0, 3.0], [3.0, 1.0, 1.0], [8.0, 7.0, 
4.0]], [[4.0, 5.0, 7.0], [7.0, 4.0, 2.0], [5.0, 6.0, 4.0]]]]))
      val=numpy.array([[[[8.0, 3.0, 6.0], [6.0, 4.0, 3.0]]], [[[2.0, 2.0, 1.0], [3.0, 3.0, 4.0]]], [[[8.0, 6.0, 7.0], [3.0, 
3.0, 8.0]]], [[[7.0, 3.0, 5.0], [1.0, 1.0, 2.0]]]])
      arg[:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 8.0], [4.0, 2.0, 5.0], [3.0, 4.0, 2.0]], [[8.0, 3.0, 6.0], [6.0, 4.0, 3.0], [2.0, 
7.0, 5.0]], [[5.0, 8.0, 2.0], [6.0, 4.0, 6.0], [5.0, 5.0, 8.0]]], [[[1.0, 7.0, 5.0], [3.0, 5.0, 1.0], [2.0, 7.0, 1.0]], [[2.0, 
2.0, 1.0], [3.0, 3.0, 4.0], [7.0, 8.0, 6.0]], [[5.0, 2.0, 2.0], [1.0, 8.0, 7.0], [2.0, 2.0, 5.0]]], [[[4.0, 6.0, 4.0], [5.0, 
6.0, 3.0], [2.0, 2.0, 8.0]], [[8.0, 6.0, 7.0], [3.0, 3.0, 8.0], [4.0, 3.0, 5.0]], [[2.0, 6.0, 7.0], [1.0, 4.0, 6.0], [7.0, 5.0, 
5.0]]], [[[2.0, 5.0, 4.0], [7.0, 6.0, 4.0], [7.0, 7.0, 4.0]], [[7.0, 3.0, 5.0], [1.0, 1.0, 2.0], [7.0, 2.0, 6.0]], [[6.0, 2.0, 
6.0], [5.0, 2.0, 1.0], [2.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 4.0], [8.0, 5.0, 6.0], [3.0, 1.0, 7.0]], 
[[8.0, 3.0, 6.0], [6.0, 4.0, 3.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 3.0], [2.0, 4.0, 4.0], [1.0, 1.0, 5.0]]], [[[8.0, 2.0, 8.0], 
[2.0, 7.0, 6.0], [7.0, 6.0, 4.0]], [[2.0, 2.0, 1.0], [3.0, 3.0, 4.0], [4.0, 2.0, 6.0]], [[3.0, 6.0, 7.0], [8.0, 4.0, 5.0], 
[6.0, 3.0, 1.0]]], [[[1.0, 7.0, 7.0], [2.0, 4.0, 5.0], [2.0, 3.0, 3.0]], [[8.0, 6.0, 7.0], [3.0, 3.0, 8.0], [6.0, 2.0, 1.0]], 
[[3.0, 1.0, 8.0], [4.0, 5.0, 2.0], [1.0, 3.0, 3.0]]], [[[8.0, 8.0, 6.0], [6.0, 3.0, 1.0], [3.0, 2.0, 3.0]], [[7.0, 3.0, 5.0], 
[1.0, 1.0, 2.0], [8.0, 7.0, 4.0]], [[4.0, 5.0, 7.0], [7.0, 4.0, 2.0], [5.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRUN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 8.0], [6.0, 4.0, 6.0], [3.0, 5.0, 6.0]], [[6.0, 6.0, 5.0], [7.0, 8.0, 6.0], [7.0, 1.0, 
3.0]], [[3.0, 7.0, 2.0], [4.0, 2.0, 4.0], [1.0, 2.0, 3.0]]], [[[7.0, 1.0, 5.0], [2.0, 8.0, 6.0], [6.0, 6.0, 8.0]], [[2.0, 6.0, 
5.0], [5.0, 1.0, 8.0], [8.0, 3.0, 5.0]], [[6.0, 2.0, 5.0], [4.0, 1.0, 1.0], [8.0, 5.0, 5.0]]], [[[6.0, 1.0, 5.0], [8.0, 1.0, 
4.0], [1.0, 3.0, 8.0]], [[4.0, 7.0, 6.0], [8.0, 7.0, 7.0], [1.0, 2.0, 1.0]], [[3.0, 1.0, 1.0], [4.0, 4.0, 3.0], [5.0, 1.0, 
6.0]]], [[[6.0, 2.0, 5.0], [8.0, 2.0, 7.0], [5.0, 3.0, 1.0]], [[3.0, 1.0, 8.0], [8.0, 3.0, 1.0], [1.0, 7.0, 5.0]], [[4.0, 7.0, 
2.0], [6.0, 1.0, 4.0], [6.0, 5.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 3.0], [2.0, 4.0, 3.0], [5.0, 2.0, 8.0]], [[4.0, 8.0, 4.0], [4.0, 8.0, 
7.0], [8.0, 1.0, 4.0]], [[4.0, 6.0, 8.0], [8.0, 1.0, 1.0], [6.0, 7.0, 2.0]]], [[[2.0, 8.0, 8.0], [6.0, 4.0, 3.0], [4.0, 5.0, 
5.0]], [[8.0, 1.0, 8.0], [2.0, 5.0, 7.0], [6.0, 2.0, 7.0]], [[8.0, 2.0, 1.0], [3.0, 7.0, 1.0], [3.0, 1.0, 7.0]]], [[[2.0, 1.0, 
7.0], [2.0, 7.0, 4.0], [5.0, 5.0, 3.0]], [[6.0, 4.0, 2.0], [6.0, 4.0, 6.0], [3.0, 3.0, 5.0]], [[1.0, 5.0, 2.0], [6.0, 4.0, 
6.0], [4.0, 7.0, 3.0]]], [[[8.0, 4.0, 3.0], [7.0, 7.0, 7.0], [3.0, 8.0, 8.0]], [[6.0, 8.0, 1.0], [3.0, 4.0, 2.0], [4.0, 8.0, 
2.0]], [[7.0, 2.0, 5.0], [6.0, 4.0, 8.0], [3.0, 1.0, 3.0]]]]))
      val=Data(numpy.array([[[[3.0, 5.0, 6.0], [5.0, 4.0, 5.0]]], [[[5.0, 8.0, 8.0], [6.0, 6.0, 4.0]]], [[[4.0, 2.0, 6.0], 
[6.0, 1.0, 6.0]]], [[[8.0, 8.0, 3.0], [6.0, 7.0, 8.0]]]]),self.functionspace)
      arg[:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 8.0], [6.0, 4.0, 6.0], [3.0, 5.0, 6.0]], [[3.0, 5.0, 6.0], [5.0, 4.0, 5.0], [7.0, 
1.0, 3.0]], [[3.0, 7.0, 2.0], [4.0, 2.0, 4.0], [1.0, 2.0, 3.0]]], [[[7.0, 1.0, 5.0], [2.0, 8.0, 6.0], [6.0, 6.0, 8.0]], [[5.0, 
8.0, 8.0], [6.0, 6.0, 4.0], [8.0, 3.0, 5.0]], [[6.0, 2.0, 5.0], [4.0, 1.0, 1.0], [8.0, 5.0, 5.0]]], [[[6.0, 1.0, 5.0], [8.0, 
1.0, 4.0], [1.0, 3.0, 8.0]], [[4.0, 2.0, 6.0], [6.0, 1.0, 6.0], [1.0, 2.0, 1.0]], [[3.0, 1.0, 1.0], [4.0, 4.0, 3.0], [5.0, 1.0, 
6.0]]], [[[6.0, 2.0, 5.0], [8.0, 2.0, 7.0], [5.0, 3.0, 1.0]], [[8.0, 8.0, 3.0], [6.0, 7.0, 8.0], [1.0, 7.0, 5.0]], [[4.0, 7.0, 
2.0], [6.0, 1.0, 4.0], [6.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 3.0], [2.0, 4.0, 3.0], [5.0, 2.0, 8.0]], 
[[3.0, 5.0, 6.0], [5.0, 4.0, 5.0], [8.0, 1.0, 4.0]], [[4.0, 6.0, 8.0], [8.0, 1.0, 1.0], [6.0, 7.0, 2.0]]], [[[2.0, 8.0, 8.0], 
[6.0, 4.0, 3.0], [4.0, 5.0, 5.0]], [[5.0, 8.0, 8.0], [6.0, 6.0, 4.0], [6.0, 2.0, 7.0]], [[8.0, 2.0, 1.0], [3.0, 7.0, 1.0], 
[3.0, 1.0, 7.0]]], [[[2.0, 1.0, 7.0], [2.0, 7.0, 4.0], [5.0, 5.0, 3.0]], [[4.0, 2.0, 6.0], [6.0, 1.0, 6.0], [3.0, 3.0, 5.0]], 
[[1.0, 5.0, 2.0], [6.0, 4.0, 6.0], [4.0, 7.0, 3.0]]], [[[8.0, 4.0, 3.0], [7.0, 7.0, 7.0], [3.0, 8.0, 8.0]], [[8.0, 8.0, 3.0], 
[6.0, 7.0, 8.0], [4.0, 8.0, 2.0]], [[7.0, 2.0, 5.0], [6.0, 4.0, 8.0], [3.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRUN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 4.0], [7.0, 4.0, 6.0], [6.0, 5.0, 8.0]], [[7.0, 5.0, 3.0], [2.0, 3.0, 8.0], [3.0, 5.0, 
8.0]], [[3.0, 7.0, 4.0], [4.0, 3.0, 6.0], [6.0, 5.0, 4.0]]], [[[4.0, 5.0, 4.0], [6.0, 4.0, 1.0], [5.0, 8.0, 3.0]], [[2.0, 2.0, 
6.0], [7.0, 5.0, 8.0], [8.0, 8.0, 4.0]], [[3.0, 4.0, 6.0], [2.0, 4.0, 4.0], [5.0, 1.0, 2.0]]], [[[5.0, 5.0, 1.0], [7.0, 6.0, 
1.0], [8.0, 3.0, 4.0]], [[1.0, 2.0, 1.0], [3.0, 6.0, 3.0], [5.0, 6.0, 3.0]], [[8.0, 8.0, 3.0], [7.0, 4.0, 6.0], [5.0, 7.0, 
7.0]]], [[[6.0, 4.0, 2.0], [4.0, 2.0, 1.0], [8.0, 2.0, 3.0]], [[5.0, 7.0, 8.0], [3.0, 7.0, 4.0], [2.0, 2.0, 2.0]], [[2.0, 4.0, 
8.0], [5.0, 7.0, 5.0], [1.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 7.0], [8.0, 2.0, 6.0], [8.0, 6.0, 3.0]], [[2.0, 2.0, 2.0], [4.0, 5.0, 
6.0], [7.0, 2.0, 5.0]], [[3.0, 3.0, 7.0], [1.0, 3.0, 8.0], [1.0, 5.0, 7.0]]], [[[7.0, 8.0, 3.0], [2.0, 1.0, 7.0], [5.0, 5.0, 
1.0]], [[1.0, 6.0, 2.0], [3.0, 5.0, 2.0], [7.0, 8.0, 5.0]], [[6.0, 5.0, 5.0], [5.0, 6.0, 7.0], [8.0, 8.0, 8.0]]], [[[8.0, 8.0, 
6.0], [2.0, 5.0, 5.0], [2.0, 5.0, 4.0]], [[4.0, 4.0, 1.0], [1.0, 2.0, 2.0], [5.0, 7.0, 1.0]], [[2.0, 5.0, 3.0], [2.0, 5.0, 
8.0], [7.0, 8.0, 7.0]]], [[[7.0, 1.0, 4.0], [1.0, 7.0, 2.0], [6.0, 1.0, 4.0]], [[1.0, 3.0, 5.0], [5.0, 6.0, 3.0], [4.0, 8.0, 
3.0]], [[3.0, 8.0, 7.0], [1.0, 5.0, 4.0], [2.0, 2.0, 5.0]]]]))
      val=Data(numpy.array([[[[7.0, 4.0, 7.0], [6.0, 8.0, 1.0]]], [[[3.0, 7.0, 6.0], [3.0, 3.0, 1.0]]], [[[8.0, 8.0, 8.0], 
[5.0, 3.0, 4.0]]], [[[4.0, 5.0, 8.0], [2.0, 1.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 3.0], [7.0, 1.0, 6.0]]], [[[3.0, 8.0, 8.0], [7.0, 4.0, 4.0]]], [[[6.0, 
4.0, 1.0], [5.0, 7.0, 3.0]]], [[[8.0, 1.0, 1.0], [4.0, 2.0, 5.0]]]]))
      arg[:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 4.0], [7.0, 4.0, 6.0], [6.0, 5.0, 8.0]], [[7.0, 4.0, 7.0], [6.0, 8.0, 1.0], [3.0, 
5.0, 8.0]], [[3.0, 7.0, 4.0], [4.0, 3.0, 6.0], [6.0, 5.0, 4.0]]], [[[4.0, 5.0, 4.0], [6.0, 4.0, 1.0], [5.0, 8.0, 3.0]], [[3.0, 
7.0, 6.0], [3.0, 3.0, 1.0], [8.0, 8.0, 4.0]], [[3.0, 4.0, 6.0], [2.0, 4.0, 4.0], [5.0, 1.0, 2.0]]], [[[5.0, 5.0, 1.0], [7.0, 
6.0, 1.0], [8.0, 3.0, 4.0]], [[8.0, 8.0, 8.0], [5.0, 3.0, 4.0], [5.0, 6.0, 3.0]], [[8.0, 8.0, 3.0], [7.0, 4.0, 6.0], [5.0, 7.0, 
7.0]]], [[[6.0, 4.0, 2.0], [4.0, 2.0, 1.0], [8.0, 2.0, 3.0]], [[4.0, 5.0, 8.0], [2.0, 1.0, 4.0], [2.0, 2.0, 2.0]], [[2.0, 4.0, 
8.0], [5.0, 7.0, 5.0], [1.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 7.0], [8.0, 2.0, 6.0], [8.0, 6.0, 3.0]], 
[[8.0, 3.0, 3.0], [7.0, 1.0, 6.0], [7.0, 2.0, 5.0]], [[3.0, 3.0, 7.0], [1.0, 3.0, 8.0], [1.0, 5.0, 7.0]]], [[[7.0, 8.0, 3.0], 
[2.0, 1.0, 7.0], [5.0, 5.0, 1.0]], [[3.0, 8.0, 8.0], [7.0, 4.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 5.0, 5.0], [5.0, 6.0, 7.0], 
[8.0, 8.0, 8.0]]], [[[8.0, 8.0, 6.0], [2.0, 5.0, 5.0], [2.0, 5.0, 4.0]], [[6.0, 4.0, 1.0], [5.0, 7.0, 3.0], [5.0, 7.0, 1.0]], 
[[2.0, 5.0, 3.0], [2.0, 5.0, 8.0], [7.0, 8.0, 7.0]]], [[[7.0, 1.0, 4.0], [1.0, 7.0, 2.0], [6.0, 1.0, 4.0]], [[8.0, 1.0, 1.0], 
[4.0, 2.0, 5.0], [4.0, 8.0, 3.0]], [[3.0, 8.0, 7.0], [1.0, 5.0, 4.0], [2.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRUN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 5.0], [1.0, 6.0, 5.0], [3.0, 8.0, 4.0]], [[1.0, 3.0, 5.0], [7.0, 6.0, 1.0], [5.0, 1.0, 
8.0]], [[8.0, 2.0, 1.0], [8.0, 1.0, 2.0], [3.0, 4.0, 5.0]]], [[[2.0, 3.0, 6.0], [6.0, 6.0, 4.0], [2.0, 5.0, 3.0]], [[7.0, 6.0, 
6.0], [5.0, 6.0, 1.0], [6.0, 8.0, 5.0]], [[4.0, 7.0, 6.0], [6.0, 2.0, 7.0], [7.0, 7.0, 5.0]]], [[[7.0, 5.0, 4.0], [7.0, 5.0, 
8.0], [3.0, 5.0, 6.0]], [[6.0, 4.0, 1.0], [4.0, 4.0, 1.0], [1.0, 7.0, 3.0]], [[6.0, 4.0, 3.0], [7.0, 4.0, 4.0], [7.0, 3.0, 
3.0]]], [[[3.0, 3.0, 3.0], [8.0, 1.0, 6.0], [4.0, 1.0, 2.0]], [[1.0, 8.0, 7.0], [4.0, 2.0, 7.0], [4.0, 5.0, 2.0]], [[3.0, 7.0, 
2.0], [4.0, 6.0, 8.0], [1.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 5.0], [4.0, 6.0, 4.0], [8.0, 6.0, 4.0]], [[7.0, 1.0, 7.0], [6.0, 8.0, 
2.0], [1.0, 6.0, 7.0]], [[1.0, 1.0, 7.0], [8.0, 6.0, 4.0], [4.0, 2.0, 4.0]]], [[[1.0, 4.0, 6.0], [1.0, 6.0, 2.0], [7.0, 8.0, 
3.0]], [[2.0, 3.0, 2.0], [4.0, 8.0, 2.0], [2.0, 7.0, 1.0]], [[2.0, 2.0, 3.0], [1.0, 2.0, 1.0], [1.0, 2.0, 5.0]]], [[[1.0, 4.0, 
7.0], [1.0, 4.0, 3.0], [4.0, 5.0, 4.0]], [[1.0, 5.0, 5.0], [7.0, 5.0, 4.0], [3.0, 5.0, 2.0]], [[4.0, 6.0, 8.0], [7.0, 5.0, 
6.0], [6.0, 1.0, 7.0]]], [[[7.0, 8.0, 7.0], [3.0, 5.0, 8.0], [1.0, 3.0, 6.0]], [[1.0, 5.0, 1.0], [4.0, 2.0, 7.0], [4.0, 3.0, 
2.0]], [[8.0, 5.0, 7.0], [2.0, 8.0, 3.0], [2.0, 5.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 5.0, 8.0], [8.0, 1.0, 5.0]]], [[[7.0, 1.0, 3.0], [8.0, 5.0, 1.0]]], [[[4.0, 1.0, 5.0], 
[3.0, 7.0, 3.0]]], [[[6.0, 1.0, 3.0], [7.0, 4.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 2.0, 3.0], [5.0, 2.0, 2.0]]], 
[[[5.0, 6.0, 5.0], [4.0, 2.0, 6.0]]], [[[3.0, 6.0, 7.0], [7.0, 6.0, 7.0]]], [[[3.0, 1.0, 8.0], [7.0, 3.0, 6.0]]]])
      arg[:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 5.0], [1.0, 6.0, 5.0], [3.0, 8.0, 4.0]], [[2.0, 5.0, 8.0], [8.0, 1.0, 5.0], [5.0, 
1.0, 8.0]], [[8.0, 2.0, 1.0], [8.0, 1.0, 2.0], [3.0, 4.0, 5.0]]], [[[2.0, 3.0, 6.0], [6.0, 6.0, 4.0], [2.0, 5.0, 3.0]], [[7.0, 
1.0, 3.0], [8.0, 5.0, 1.0], [6.0, 8.0, 5.0]], [[4.0, 7.0, 6.0], [6.0, 2.0, 7.0], [7.0, 7.0, 5.0]]], [[[7.0, 5.0, 4.0], [7.0, 
5.0, 8.0], [3.0, 5.0, 6.0]], [[4.0, 1.0, 5.0], [3.0, 7.0, 3.0], [1.0, 7.0, 3.0]], [[6.0, 4.0, 3.0], [7.0, 4.0, 4.0], [7.0, 3.0, 
3.0]]], [[[3.0, 3.0, 3.0], [8.0, 1.0, 6.0], [4.0, 1.0, 2.0]], [[6.0, 1.0, 3.0], [7.0, 4.0, 4.0], [4.0, 5.0, 2.0]], [[3.0, 7.0, 
2.0], [4.0, 6.0, 8.0], [1.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 5.0], [4.0, 6.0, 4.0], [8.0, 6.0, 4.0]], 
[[8.0, 2.0, 3.0], [5.0, 2.0, 2.0], [1.0, 6.0, 7.0]], [[1.0, 1.0, 7.0], [8.0, 6.0, 4.0], [4.0, 2.0, 4.0]]], [[[1.0, 4.0, 6.0], 
[1.0, 6.0, 2.0], [7.0, 8.0, 3.0]], [[5.0, 6.0, 5.0], [4.0, 2.0, 6.0], [2.0, 7.0, 1.0]], [[2.0, 2.0, 3.0], [1.0, 2.0, 1.0], 
[1.0, 2.0, 5.0]]], [[[1.0, 4.0, 7.0], [1.0, 4.0, 3.0], [4.0, 5.0, 4.0]], [[3.0, 6.0, 7.0], [7.0, 6.0, 7.0], [3.0, 5.0, 2.0]], 
[[4.0, 6.0, 8.0], [7.0, 5.0, 6.0], [6.0, 1.0, 7.0]]], [[[7.0, 8.0, 7.0], [3.0, 5.0, 8.0], [1.0, 3.0, 6.0]], [[3.0, 1.0, 8.0], 
[7.0, 3.0, 6.0], [4.0, 3.0, 2.0]], [[8.0, 5.0, 7.0], [2.0, 8.0, 3.0], [2.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRCL(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 8.0], [5.0, 3.0, 6.0], [4.0, 7.0, 7.0]], [[8.0, 4.0, 2.0], [1.0, 1.0, 3.0], [2.0, 2.0, 
7.0]], [[3.0, 4.0, 5.0], [3.0, 8.0, 2.0], [3.0, 1.0, 3.0]]], [[[4.0, 8.0, 8.0], [4.0, 4.0, 1.0], [6.0, 1.0, 1.0]], [[4.0, 1.0, 
8.0], [2.0, 3.0, 1.0], [2.0, 5.0, 2.0]], [[8.0, 4.0, 8.0], [2.0, 2.0, 4.0], [2.0, 1.0, 5.0]]], [[[6.0, 7.0, 5.0], [4.0, 5.0, 
4.0], [2.0, 3.0, 6.0]], [[2.0, 5.0, 1.0], [7.0, 2.0, 2.0], [4.0, 1.0, 3.0]], [[8.0, 1.0, 7.0], [5.0, 3.0, 4.0], [8.0, 1.0, 
3.0]]], [[[3.0, 7.0, 6.0], [8.0, 1.0, 3.0], [3.0, 4.0, 2.0]], [[6.0, 2.0, 3.0], [3.0, 3.0, 8.0], [3.0, 3.0, 6.0]], [[5.0, 7.0, 
1.0], [6.0, 3.0, 4.0], [4.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 2.0], [2.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[7.0, 8.0, 7.0], [4.0, 6.0, 
3.0], [5.0, 3.0, 3.0]], [[6.0, 6.0, 3.0], [5.0, 3.0, 5.0], [5.0, 4.0, 4.0]]], [[[8.0, 2.0, 6.0], [1.0, 8.0, 2.0], [4.0, 8.0, 
8.0]], [[8.0, 3.0, 5.0], [6.0, 5.0, 4.0], [8.0, 2.0, 4.0]], [[4.0, 5.0, 2.0], [1.0, 8.0, 4.0], [1.0, 7.0, 8.0]]], [[[1.0, 5.0, 
3.0], [2.0, 6.0, 7.0], [3.0, 6.0, 7.0]], [[4.0, 3.0, 3.0], [1.0, 4.0, 8.0], [4.0, 3.0, 5.0]], [[5.0, 3.0, 6.0], [5.0, 4.0, 
2.0], [3.0, 6.0, 7.0]]], [[[1.0, 4.0, 8.0], [2.0, 7.0, 8.0], [4.0, 6.0, 3.0]], [[5.0, 7.0, 8.0], [8.0, 8.0, 4.0], [2.0, 3.0, 
7.0]], [[8.0, 7.0, 1.0], [4.0, 1.0, 7.0], [8.0, 7.0, 6.0]]]]))
      val=numpy.array([[[[6.0], [2.0], [1.0]]], [[[4.0], [6.0], [5.0]]], [[[8.0], [5.0], [6.0]]], [[[1.0], [8.0], [8.0]]]])
      arg[:,1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 8.0], [5.0, 3.0, 6.0], [4.0, 7.0, 7.0]], [[8.0, 4.0, 6.0], [1.0, 1.0, 2.0], [2.0, 
2.0, 1.0]], [[3.0, 4.0, 5.0], [3.0, 8.0, 2.0], [3.0, 1.0, 3.0]]], [[[4.0, 8.0, 8.0], [4.0, 4.0, 1.0], [6.0, 1.0, 1.0]], [[4.0, 
1.0, 4.0], [2.0, 3.0, 6.0], [2.0, 5.0, 5.0]], [[8.0, 4.0, 8.0], [2.0, 2.0, 4.0], [2.0, 1.0, 5.0]]], [[[6.0, 7.0, 5.0], [4.0, 
5.0, 4.0], [2.0, 3.0, 6.0]], [[2.0, 5.0, 8.0], [7.0, 2.0, 5.0], [4.0, 1.0, 6.0]], [[8.0, 1.0, 7.0], [5.0, 3.0, 4.0], [8.0, 1.0, 
3.0]]], [[[3.0, 7.0, 6.0], [8.0, 1.0, 3.0], [3.0, 4.0, 2.0]], [[6.0, 2.0, 1.0], [3.0, 3.0, 8.0], [3.0, 3.0, 8.0]], [[5.0, 7.0, 
1.0], [6.0, 3.0, 4.0], [4.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 2.0], [2.0, 4.0, 5.0], [2.0, 8.0, 6.0]], 
[[7.0, 8.0, 6.0], [4.0, 6.0, 2.0], [5.0, 3.0, 1.0]], [[6.0, 6.0, 3.0], [5.0, 3.0, 5.0], [5.0, 4.0, 4.0]]], [[[8.0, 2.0, 6.0], 
[1.0, 8.0, 2.0], [4.0, 8.0, 8.0]], [[8.0, 3.0, 4.0], [6.0, 5.0, 6.0], [8.0, 2.0, 5.0]], [[4.0, 5.0, 2.0], [1.0, 8.0, 4.0], 
[1.0, 7.0, 8.0]]], [[[1.0, 5.0, 3.0], [2.0, 6.0, 7.0], [3.0, 6.0, 7.0]], [[4.0, 3.0, 8.0], [1.0, 4.0, 5.0], [4.0, 3.0, 6.0]], 
[[5.0, 3.0, 6.0], [5.0, 4.0, 2.0], [3.0, 6.0, 7.0]]], [[[1.0, 4.0, 8.0], [2.0, 7.0, 8.0], [4.0, 6.0, 3.0]], [[5.0, 7.0, 1.0], 
[8.0, 8.0, 8.0], [2.0, 3.0, 8.0]], [[8.0, 7.0, 1.0], [4.0, 1.0, 7.0], [8.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRCL(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 8.0], [2.0, 8.0, 5.0], [2.0, 3.0, 8.0]], [[3.0, 6.0, 5.0], [5.0, 1.0, 5.0], [8.0, 3.0, 
2.0]], [[5.0, 8.0, 1.0], [3.0, 1.0, 4.0], [1.0, 5.0, 8.0]]], [[[8.0, 7.0, 3.0], [2.0, 6.0, 7.0], [7.0, 8.0, 2.0]], [[7.0, 1.0, 
2.0], [8.0, 5.0, 1.0], [4.0, 5.0, 2.0]], [[4.0, 7.0, 3.0], [7.0, 3.0, 7.0], [8.0, 3.0, 6.0]]], [[[6.0, 2.0, 4.0], [8.0, 2.0, 
7.0], [2.0, 3.0, 7.0]], [[8.0, 3.0, 4.0], [3.0, 2.0, 8.0], [1.0, 1.0, 7.0]], [[6.0, 2.0, 7.0], [3.0, 5.0, 7.0], [8.0, 5.0, 
6.0]]], [[[8.0, 3.0, 6.0], [2.0, 8.0, 7.0], [4.0, 6.0, 5.0]], [[8.0, 3.0, 6.0], [6.0, 2.0, 8.0], [1.0, 4.0, 3.0]], [[2.0, 3.0, 
6.0], [2.0, 8.0, 8.0], [3.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 1.0], [1.0, 6.0, 1.0], [4.0, 4.0, 1.0]], [[1.0, 2.0, 8.0], [7.0, 4.0, 
6.0], [8.0, 8.0, 4.0]], [[5.0, 1.0, 8.0], [8.0, 2.0, 4.0], [5.0, 6.0, 6.0]]], [[[8.0, 4.0, 5.0], [7.0, 5.0, 3.0], [7.0, 7.0, 
5.0]], [[4.0, 3.0, 6.0], [6.0, 5.0, 5.0], [3.0, 3.0, 6.0]], [[1.0, 6.0, 1.0], [6.0, 7.0, 3.0], [6.0, 6.0, 2.0]]], [[[8.0, 2.0, 
7.0], [5.0, 5.0, 4.0], [2.0, 7.0, 4.0]], [[7.0, 3.0, 3.0], [6.0, 1.0, 3.0], [8.0, 7.0, 5.0]], [[8.0, 5.0, 6.0], [8.0, 5.0, 
5.0], [2.0, 2.0, 1.0]]], [[[3.0, 6.0, 6.0], [5.0, 8.0, 1.0], [7.0, 1.0, 6.0]], [[1.0, 4.0, 3.0], [1.0, 6.0, 2.0], [6.0, 1.0, 
6.0]], [[7.0, 3.0, 3.0], [5.0, 3.0, 4.0], [4.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[[1.0], [2.0], [6.0]]], [[[1.0], [5.0], [6.0]]], [[[4.0], [8.0], [4.0]]], [[[4.0], [6.0], 
[4.0]]]]),self.functionspace)
      arg[:,1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 8.0], [2.0, 8.0, 5.0], [2.0, 3.0, 8.0]], [[3.0, 6.0, 1.0], [5.0, 1.0, 2.0], [8.0, 
3.0, 6.0]], [[5.0, 8.0, 1.0], [3.0, 1.0, 4.0], [1.0, 5.0, 8.0]]], [[[8.0, 7.0, 3.0], [2.0, 6.0, 7.0], [7.0, 8.0, 2.0]], [[7.0, 
1.0, 1.0], [8.0, 5.0, 5.0], [4.0, 5.0, 6.0]], [[4.0, 7.0, 3.0], [7.0, 3.0, 7.0], [8.0, 3.0, 6.0]]], [[[6.0, 2.0, 4.0], [8.0, 
2.0, 7.0], [2.0, 3.0, 7.0]], [[8.0, 3.0, 4.0], [3.0, 2.0, 8.0], [1.0, 1.0, 4.0]], [[6.0, 2.0, 7.0], [3.0, 5.0, 7.0], [8.0, 5.0, 
6.0]]], [[[8.0, 3.0, 6.0], [2.0, 8.0, 7.0], [4.0, 6.0, 5.0]], [[8.0, 3.0, 4.0], [6.0, 2.0, 6.0], [1.0, 4.0, 4.0]], [[2.0, 3.0, 
6.0], [2.0, 8.0, 8.0], [3.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 1.0], [1.0, 6.0, 1.0], [4.0, 4.0, 1.0]], 
[[1.0, 2.0, 1.0], [7.0, 4.0, 2.0], [8.0, 8.0, 6.0]], [[5.0, 1.0, 8.0], [8.0, 2.0, 4.0], [5.0, 6.0, 6.0]]], [[[8.0, 4.0, 5.0], 
[7.0, 5.0, 3.0], [7.0, 7.0, 5.0]], [[4.0, 3.0, 1.0], [6.0, 5.0, 5.0], [3.0, 3.0, 6.0]], [[1.0, 6.0, 1.0], [6.0, 7.0, 3.0], 
[6.0, 6.0, 2.0]]], [[[8.0, 2.0, 7.0], [5.0, 5.0, 4.0], [2.0, 7.0, 4.0]], [[7.0, 3.0, 4.0], [6.0, 1.0, 8.0], [8.0, 7.0, 4.0]], 
[[8.0, 5.0, 6.0], [8.0, 5.0, 5.0], [2.0, 2.0, 1.0]]], [[[3.0, 6.0, 6.0], [5.0, 8.0, 1.0], [7.0, 1.0, 6.0]], [[1.0, 4.0, 4.0], 
[1.0, 6.0, 6.0], [6.0, 1.0, 4.0]], [[7.0, 3.0, 3.0], [5.0, 3.0, 4.0], [4.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRCL(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 2.0], [6.0, 4.0, 5.0], [3.0, 7.0, 4.0]], [[6.0, 4.0, 7.0], [8.0, 6.0, 3.0], [8.0, 1.0, 
7.0]], [[6.0, 3.0, 5.0], [2.0, 8.0, 6.0], [7.0, 3.0, 6.0]]], [[[4.0, 3.0, 4.0], [4.0, 8.0, 4.0], [2.0, 3.0, 3.0]], [[4.0, 4.0, 
7.0], [5.0, 3.0, 4.0], [8.0, 4.0, 5.0]], [[2.0, 5.0, 8.0], [8.0, 3.0, 6.0], [2.0, 2.0, 7.0]]], [[[8.0, 3.0, 8.0], [1.0, 2.0, 
7.0], [1.0, 7.0, 6.0]], [[7.0, 3.0, 6.0], [3.0, 8.0, 8.0], [8.0, 7.0, 3.0]], [[2.0, 7.0, 2.0], [4.0, 4.0, 6.0], [2.0, 6.0, 
3.0]]], [[[6.0, 3.0, 6.0], [4.0, 2.0, 1.0], [5.0, 7.0, 4.0]], [[5.0, 4.0, 3.0], [7.0, 3.0, 4.0], [3.0, 2.0, 6.0]], [[1.0, 2.0, 
3.0], [7.0, 1.0, 6.0], [4.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 1.0], [5.0, 6.0, 5.0], [6.0, 1.0, 4.0]], [[5.0, 5.0, 8.0], [8.0, 4.0, 
4.0], [5.0, 6.0, 1.0]], [[6.0, 4.0, 4.0], [3.0, 7.0, 2.0], [1.0, 6.0, 8.0]]], [[[7.0, 3.0, 6.0], [6.0, 2.0, 1.0], [5.0, 7.0, 
8.0]], [[4.0, 3.0, 2.0], [6.0, 1.0, 3.0], [8.0, 7.0, 7.0]], [[6.0, 6.0, 3.0], [8.0, 3.0, 4.0], [7.0, 3.0, 6.0]]], [[[8.0, 1.0, 
2.0], [4.0, 7.0, 3.0], [4.0, 8.0, 4.0]], [[2.0, 4.0, 2.0], [2.0, 3.0, 8.0], [4.0, 3.0, 2.0]], [[3.0, 4.0, 1.0], [2.0, 3.0, 
2.0], [7.0, 2.0, 6.0]]], [[[5.0, 5.0, 4.0], [7.0, 3.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 5.0, 3.0], [4.0, 6.0, 3.0], [7.0, 1.0, 
5.0]], [[6.0, 2.0, 1.0], [7.0, 3.0, 6.0], [2.0, 7.0, 1.0]]]]))
      val=Data(numpy.array([[[[7.0], [6.0], [1.0]]], [[[2.0], [6.0], [8.0]]], [[[4.0], [5.0], [5.0]]], [[[4.0], [7.0], 
[2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0], [2.0], [5.0]]], [[[6.0], [5.0], [6.0]]], [[[3.0], [2.0], [5.0]]], [[[1.0], 
[5.0], [4.0]]]]))
      arg[:,1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 2.0], [6.0, 4.0, 5.0], [3.0, 7.0, 4.0]], [[6.0, 4.0, 7.0], [8.0, 6.0, 6.0], [8.0, 
1.0, 1.0]], [[6.0, 3.0, 5.0], [2.0, 8.0, 6.0], [7.0, 3.0, 6.0]]], [[[4.0, 3.0, 4.0], [4.0, 8.0, 4.0], [2.0, 3.0, 3.0]], [[4.0, 
4.0, 2.0], [5.0, 3.0, 6.0], [8.0, 4.0, 8.0]], [[2.0, 5.0, 8.0], [8.0, 3.0, 6.0], [2.0, 2.0, 7.0]]], [[[8.0, 3.0, 8.0], [1.0, 
2.0, 7.0], [1.0, 7.0, 6.0]], [[7.0, 3.0, 4.0], [3.0, 8.0, 5.0], [8.0, 7.0, 5.0]], [[2.0, 7.0, 2.0], [4.0, 4.0, 6.0], [2.0, 6.0, 
3.0]]], [[[6.0, 3.0, 6.0], [4.0, 2.0, 1.0], [5.0, 7.0, 4.0]], [[5.0, 4.0, 4.0], [7.0, 3.0, 7.0], [3.0, 2.0, 2.0]], [[1.0, 2.0, 
3.0], [7.0, 1.0, 6.0], [4.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 1.0], [5.0, 6.0, 5.0], [6.0, 1.0, 4.0]], 
[[5.0, 5.0, 7.0], [8.0, 4.0, 2.0], [5.0, 6.0, 5.0]], [[6.0, 4.0, 4.0], [3.0, 7.0, 2.0], [1.0, 6.0, 8.0]]], [[[7.0, 3.0, 6.0], 
[6.0, 2.0, 1.0], [5.0, 7.0, 8.0]], [[4.0, 3.0, 6.0], [6.0, 1.0, 5.0], [8.0, 7.0, 6.0]], [[6.0, 6.0, 3.0], [8.0, 3.0, 4.0], 
[7.0, 3.0, 6.0]]], [[[8.0, 1.0, 2.0], [4.0, 7.0, 3.0], [4.0, 8.0, 4.0]], [[2.0, 4.0, 3.0], [2.0, 3.0, 2.0], [4.0, 3.0, 5.0]], 
[[3.0, 4.0, 1.0], [2.0, 3.0, 2.0], [7.0, 2.0, 6.0]]], [[[5.0, 5.0, 4.0], [7.0, 3.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 5.0, 1.0], 
[4.0, 6.0, 5.0], [7.0, 1.0, 4.0]], [[6.0, 2.0, 1.0], [7.0, 3.0, 6.0], [2.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRCL(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 1.0], [8.0, 4.0, 8.0], [6.0, 8.0, 2.0]], [[7.0, 7.0, 7.0], [8.0, 4.0, 5.0], [5.0, 2.0, 
1.0]], [[4.0, 8.0, 3.0], [1.0, 7.0, 5.0], [2.0, 8.0, 2.0]]], [[[8.0, 2.0, 2.0], [2.0, 7.0, 6.0], [2.0, 6.0, 6.0]], [[5.0, 2.0, 
8.0], [1.0, 5.0, 4.0], [6.0, 2.0, 1.0]], [[3.0, 7.0, 3.0], [2.0, 1.0, 1.0], [3.0, 2.0, 8.0]]], [[[4.0, 6.0, 2.0], [6.0, 6.0, 
3.0], [2.0, 3.0, 7.0]], [[5.0, 6.0, 1.0], [3.0, 8.0, 7.0], [2.0, 4.0, 4.0]], [[8.0, 6.0, 6.0], [3.0, 4.0, 8.0], [6.0, 1.0, 
3.0]]], [[[1.0, 6.0, 8.0], [4.0, 3.0, 6.0], [6.0, 5.0, 6.0]], [[7.0, 2.0, 1.0], [7.0, 8.0, 1.0], [5.0, 1.0, 8.0]], [[3.0, 1.0, 
2.0], [5.0, 4.0, 7.0], [8.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 7.0], [5.0, 2.0, 5.0], [7.0, 4.0, 1.0]], [[5.0, 2.0, 7.0], [6.0, 2.0, 
3.0], [6.0, 6.0, 2.0]], [[8.0, 2.0, 2.0], [8.0, 1.0, 5.0], [2.0, 3.0, 4.0]]], [[[3.0, 4.0, 2.0], [8.0, 7.0, 2.0], [5.0, 6.0, 
8.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 4.0], [6.0, 3.0, 1.0]], [[6.0, 2.0, 6.0], [4.0, 1.0, 2.0], [6.0, 6.0, 1.0]]], [[[8.0, 2.0, 
2.0], [6.0, 7.0, 6.0], [3.0, 7.0, 3.0]], [[7.0, 6.0, 4.0], [5.0, 1.0, 4.0], [5.0, 4.0, 5.0]], [[7.0, 5.0, 5.0], [6.0, 3.0, 
1.0], [3.0, 6.0, 1.0]]], [[[8.0, 2.0, 4.0], [1.0, 8.0, 1.0], [6.0, 1.0, 2.0]], [[3.0, 8.0, 1.0], [4.0, 2.0, 5.0], [2.0, 1.0, 
7.0]], [[1.0, 3.0, 7.0], [1.0, 5.0, 5.0], [8.0, 7.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0], [4.0], [7.0]]], [[[2.0], [8.0], [3.0]]], [[[3.0], [3.0], [8.0]]], [[[8.0], [7.0], 
[2.0]]]])+(1.-msk_val)*numpy.array([[[[7.0], [7.0], [3.0]]], [[[2.0], [5.0], [3.0]]], [[[4.0], [3.0], [6.0]]], [[[3.0], [5.0], 
[4.0]]]])
      arg[:,1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 1.0], [8.0, 4.0, 8.0], [6.0, 8.0, 2.0]], [[7.0, 7.0, 7.0], [8.0, 4.0, 4.0], [5.0, 
2.0, 7.0]], [[4.0, 8.0, 3.0], [1.0, 7.0, 5.0], [2.0, 8.0, 2.0]]], [[[8.0, 2.0, 2.0], [2.0, 7.0, 6.0], [2.0, 6.0, 6.0]], [[5.0, 
2.0, 2.0], [1.0, 5.0, 8.0], [6.0, 2.0, 3.0]], [[3.0, 7.0, 3.0], [2.0, 1.0, 1.0], [3.0, 2.0, 8.0]]], [[[4.0, 6.0, 2.0], [6.0, 
6.0, 3.0], [2.0, 3.0, 7.0]], [[5.0, 6.0, 3.0], [3.0, 8.0, 3.0], [2.0, 4.0, 8.0]], [[8.0, 6.0, 6.0], [3.0, 4.0, 8.0], [6.0, 1.0, 
3.0]]], [[[1.0, 6.0, 8.0], [4.0, 3.0, 6.0], [6.0, 5.0, 6.0]], [[7.0, 2.0, 8.0], [7.0, 8.0, 7.0], [5.0, 1.0, 2.0]], [[3.0, 1.0, 
2.0], [5.0, 4.0, 7.0], [8.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 7.0], [5.0, 2.0, 5.0], [7.0, 4.0, 1.0]], 
[[5.0, 2.0, 7.0], [6.0, 2.0, 7.0], [6.0, 6.0, 3.0]], [[8.0, 2.0, 2.0], [8.0, 1.0, 5.0], [2.0, 3.0, 4.0]]], [[[3.0, 4.0, 2.0], 
[8.0, 7.0, 2.0], [5.0, 6.0, 8.0]], [[7.0, 5.0, 2.0], [1.0, 7.0, 5.0], [6.0, 3.0, 3.0]], [[6.0, 2.0, 6.0], [4.0, 1.0, 2.0], 
[6.0, 6.0, 1.0]]], [[[8.0, 2.0, 2.0], [6.0, 7.0, 6.0], [3.0, 7.0, 3.0]], [[7.0, 6.0, 4.0], [5.0, 1.0, 3.0], [5.0, 4.0, 6.0]], 
[[7.0, 5.0, 5.0], [6.0, 3.0, 1.0], [3.0, 6.0, 1.0]]], [[[8.0, 2.0, 4.0], [1.0, 8.0, 1.0], [6.0, 1.0, 2.0]], [[3.0, 8.0, 3.0], 
[4.0, 2.0, 5.0], [2.0, 1.0, 4.0]], [[1.0, 3.0, 7.0], [1.0, 5.0, 5.0], [8.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPPL(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 1.0], [6.0, 4.0, 6.0], [7.0, 6.0, 6.0]], [[6.0, 8.0, 4.0], [8.0, 7.0, 1.0], [7.0, 7.0, 
8.0]], [[7.0, 4.0, 1.0], [7.0, 8.0, 4.0], [2.0, 1.0, 2.0]]], [[[4.0, 3.0, 7.0], [1.0, 7.0, 3.0], [1.0, 8.0, 5.0]], [[8.0, 8.0, 
1.0], [2.0, 6.0, 4.0], [7.0, 1.0, 7.0]], [[5.0, 1.0, 7.0], [5.0, 7.0, 4.0], [8.0, 1.0, 6.0]]], [[[1.0, 6.0, 5.0], [7.0, 6.0, 
6.0], [5.0, 4.0, 5.0]], [[4.0, 4.0, 5.0], [1.0, 6.0, 7.0], [6.0, 8.0, 4.0]], [[7.0, 7.0, 1.0], [8.0, 1.0, 5.0], [4.0, 3.0, 
4.0]]], [[[7.0, 2.0, 3.0], [6.0, 8.0, 5.0], [7.0, 5.0, 5.0]], [[4.0, 4.0, 5.0], [5.0, 7.0, 8.0], [4.0, 5.0, 3.0]], [[1.0, 5.0, 
3.0], [2.0, 7.0, 7.0], [2.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 4.0], [6.0, 8.0, 8.0], [2.0, 1.0, 5.0]], [[2.0, 2.0, 3.0], [5.0, 8.0, 
8.0], [7.0, 2.0, 7.0]], [[8.0, 3.0, 4.0], [8.0, 1.0, 1.0], [5.0, 1.0, 2.0]]], [[[5.0, 2.0, 1.0], [7.0, 7.0, 7.0], [2.0, 2.0, 
7.0]], [[6.0, 4.0, 2.0], [6.0, 2.0, 6.0], [1.0, 6.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 6.0, 7.0], [2.0, 7.0, 3.0]]], [[[6.0, 3.0, 
8.0], [8.0, 4.0, 7.0], [3.0, 4.0, 4.0]], [[1.0, 8.0, 2.0], [4.0, 7.0, 1.0], [2.0, 1.0, 1.0]], [[7.0, 4.0, 6.0], [2.0, 4.0, 
5.0], [1.0, 6.0, 1.0]]], [[[6.0, 7.0, 6.0], [6.0, 6.0, 2.0], [2.0, 4.0, 8.0]], [[1.0, 6.0, 8.0], [1.0, 1.0, 4.0], [2.0, 1.0, 
4.0]], [[4.0, 6.0, 3.0], [5.0, 3.0, 3.0], [7.0, 5.0, 3.0]]]]))
      val=numpy.array([[2.0], [1.0], [4.0], [1.0]])
      arg[:,1,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 1.0], [6.0, 4.0, 6.0], [7.0, 6.0, 6.0]], [[6.0, 8.0, 4.0], [8.0, 7.0, 2.0], [7.0, 
7.0, 8.0]], [[7.0, 4.0, 1.0], [7.0, 8.0, 4.0], [2.0, 1.0, 2.0]]], [[[4.0, 3.0, 7.0], [1.0, 7.0, 3.0], [1.0, 8.0, 5.0]], [[8.0, 
8.0, 1.0], [2.0, 6.0, 1.0], [7.0, 1.0, 7.0]], [[5.0, 1.0, 7.0], [5.0, 7.0, 4.0], [8.0, 1.0, 6.0]]], [[[1.0, 6.0, 5.0], [7.0, 
6.0, 6.0], [5.0, 4.0, 5.0]], [[4.0, 4.0, 5.0], [1.0, 6.0, 4.0], [6.0, 8.0, 4.0]], [[7.0, 7.0, 1.0], [8.0, 1.0, 5.0], [4.0, 3.0, 
4.0]]], [[[7.0, 2.0, 3.0], [6.0, 8.0, 5.0], [7.0, 5.0, 5.0]], [[4.0, 4.0, 5.0], [5.0, 7.0, 1.0], [4.0, 5.0, 3.0]], [[1.0, 5.0, 
3.0], [2.0, 7.0, 7.0], [2.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 4.0], [6.0, 8.0, 8.0], [2.0, 1.0, 5.0]], 
[[2.0, 2.0, 3.0], [5.0, 8.0, 2.0], [7.0, 2.0, 7.0]], [[8.0, 3.0, 4.0], [8.0, 1.0, 1.0], [5.0, 1.0, 2.0]]], [[[5.0, 2.0, 1.0], 
[7.0, 7.0, 7.0], [2.0, 2.0, 7.0]], [[6.0, 4.0, 2.0], [6.0, 2.0, 1.0], [1.0, 6.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 6.0, 7.0], 
[2.0, 7.0, 3.0]]], [[[6.0, 3.0, 8.0], [8.0, 4.0, 7.0], [3.0, 4.0, 4.0]], [[1.0, 8.0, 2.0], [4.0, 7.0, 4.0], [2.0, 1.0, 1.0]], 
[[7.0, 4.0, 6.0], [2.0, 4.0, 5.0], [1.0, 6.0, 1.0]]], [[[6.0, 7.0, 6.0], [6.0, 6.0, 2.0], [2.0, 4.0, 8.0]], [[1.0, 6.0, 8.0], 
[1.0, 1.0, 1.0], [2.0, 1.0, 4.0]], [[4.0, 6.0, 3.0], [5.0, 3.0, 3.0], [7.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPPL(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 1.0], [8.0, 1.0, 1.0], [1.0, 3.0, 5.0]], [[1.0, 4.0, 4.0], [7.0, 6.0, 7.0], [2.0, 2.0, 
3.0]], [[8.0, 8.0, 2.0], [5.0, 6.0, 4.0], [5.0, 4.0, 7.0]]], [[[7.0, 8.0, 6.0], [5.0, 4.0, 7.0], [2.0, 3.0, 3.0]], [[4.0, 2.0, 
2.0], [2.0, 1.0, 7.0], [3.0, 5.0, 2.0]], [[4.0, 6.0, 4.0], [3.0, 1.0, 5.0], [5.0, 4.0, 2.0]]], [[[2.0, 7.0, 3.0], [1.0, 7.0, 
6.0], [2.0, 1.0, 1.0]], [[8.0, 7.0, 2.0], [1.0, 1.0, 2.0], [6.0, 2.0, 1.0]], [[7.0, 1.0, 8.0], [8.0, 3.0, 5.0], [2.0, 3.0, 
5.0]]], [[[6.0, 1.0, 7.0], [8.0, 8.0, 6.0], [3.0, 1.0, 2.0]], [[1.0, 6.0, 6.0], [5.0, 5.0, 6.0], [8.0, 1.0, 2.0]], [[4.0, 1.0, 
8.0], [5.0, 4.0, 8.0], [8.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 2.0], [3.0, 1.0, 5.0], [6.0, 7.0, 4.0]], [[3.0, 4.0, 1.0], [3.0, 1.0, 
3.0], [8.0, 4.0, 6.0]], [[6.0, 5.0, 1.0], [7.0, 1.0, 8.0], [1.0, 2.0, 5.0]]], [[[6.0, 3.0, 5.0], [6.0, 4.0, 7.0], [1.0, 1.0, 
1.0]], [[8.0, 4.0, 2.0], [3.0, 6.0, 1.0], [6.0, 8.0, 6.0]], [[2.0, 3.0, 7.0], [4.0, 2.0, 2.0], [6.0, 2.0, 4.0]]], [[[2.0, 3.0, 
5.0], [2.0, 1.0, 3.0], [4.0, 2.0, 7.0]], [[5.0, 7.0, 5.0], [8.0, 1.0, 2.0], [8.0, 2.0, 3.0]], [[7.0, 2.0, 5.0], [5.0, 4.0, 
7.0], [8.0, 5.0, 1.0]]], [[[2.0, 1.0, 4.0], [3.0, 6.0, 3.0], [1.0, 6.0, 8.0]], [[1.0, 5.0, 5.0], [6.0, 5.0, 4.0], [1.0, 8.0, 
4.0]], [[6.0, 8.0, 7.0], [3.0, 8.0, 7.0], [8.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[6.0], [8.0], [8.0], [3.0]]),self.functionspace)
      arg[:,1,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 1.0], [8.0, 1.0, 1.0], [1.0, 3.0, 5.0]], [[1.0, 4.0, 4.0], [7.0, 6.0, 6.0], [2.0, 
2.0, 3.0]], [[8.0, 8.0, 2.0], [5.0, 6.0, 4.0], [5.0, 4.0, 7.0]]], [[[7.0, 8.0, 6.0], [5.0, 4.0, 7.0], [2.0, 3.0, 3.0]], [[4.0, 
2.0, 2.0], [2.0, 1.0, 8.0], [3.0, 5.0, 2.0]], [[4.0, 6.0, 4.0], [3.0, 1.0, 5.0], [5.0, 4.0, 2.0]]], [[[2.0, 7.0, 3.0], [1.0, 
7.0, 6.0], [2.0, 1.0, 1.0]], [[8.0, 7.0, 2.0], [1.0, 1.0, 8.0], [6.0, 2.0, 1.0]], [[7.0, 1.0, 8.0], [8.0, 3.0, 5.0], [2.0, 3.0, 
5.0]]], [[[6.0, 1.0, 7.0], [8.0, 8.0, 6.0], [3.0, 1.0, 2.0]], [[1.0, 6.0, 6.0], [5.0, 5.0, 3.0], [8.0, 1.0, 2.0]], [[4.0, 1.0, 
8.0], [5.0, 4.0, 8.0], [8.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 2.0], [3.0, 1.0, 5.0], [6.0, 7.0, 4.0]], 
[[3.0, 4.0, 1.0], [3.0, 1.0, 6.0], [8.0, 4.0, 6.0]], [[6.0, 5.0, 1.0], [7.0, 1.0, 8.0], [1.0, 2.0, 5.0]]], [[[6.0, 3.0, 5.0], 
[6.0, 4.0, 7.0], [1.0, 1.0, 1.0]], [[8.0, 4.0, 2.0], [3.0, 6.0, 8.0], [6.0, 8.0, 6.0]], [[2.0, 3.0, 7.0], [4.0, 2.0, 2.0], 
[6.0, 2.0, 4.0]]], [[[2.0, 3.0, 5.0], [2.0, 1.0, 3.0], [4.0, 2.0, 7.0]], [[5.0, 7.0, 5.0], [8.0, 1.0, 8.0], [8.0, 2.0, 3.0]], 
[[7.0, 2.0, 5.0], [5.0, 4.0, 7.0], [8.0, 5.0, 1.0]]], [[[2.0, 1.0, 4.0], [3.0, 6.0, 3.0], [1.0, 6.0, 8.0]], [[1.0, 5.0, 5.0], 
[6.0, 5.0, 3.0], [1.0, 8.0, 4.0]], [[6.0, 8.0, 7.0], [3.0, 8.0, 7.0], [8.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPPL(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 4.0], [7.0, 6.0, 1.0], [4.0, 2.0, 8.0]], [[7.0, 8.0, 1.0], [3.0, 1.0, 1.0], [2.0, 2.0, 
7.0]], [[7.0, 7.0, 2.0], [6.0, 8.0, 2.0], [7.0, 6.0, 2.0]]], [[[8.0, 5.0, 8.0], [5.0, 7.0, 4.0], [7.0, 8.0, 3.0]], [[2.0, 6.0, 
4.0], [6.0, 5.0, 4.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 7.0], [7.0, 6.0, 8.0], [2.0, 2.0, 4.0]]], [[[1.0, 2.0, 3.0], [1.0, 5.0, 
5.0], [6.0, 3.0, 2.0]], [[2.0, 4.0, 4.0], [5.0, 6.0, 8.0], [3.0, 1.0, 7.0]], [[4.0, 1.0, 6.0], [8.0, 6.0, 3.0], [5.0, 6.0, 
1.0]]], [[[1.0, 2.0, 3.0], [7.0, 3.0, 6.0], [2.0, 5.0, 7.0]], [[7.0, 7.0, 8.0], [8.0, 5.0, 8.0], [6.0, 2.0, 6.0]], [[8.0, 8.0, 
5.0], [1.0, 2.0, 6.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 3.0], [8.0, 6.0, 5.0], [5.0, 6.0, 5.0]], [[2.0, 7.0, 1.0], [4.0, 6.0, 
4.0], [7.0, 4.0, 3.0]], [[3.0, 6.0, 2.0], [3.0, 6.0, 5.0], [1.0, 5.0, 4.0]]], [[[8.0, 2.0, 6.0], [7.0, 3.0, 8.0], [2.0, 8.0, 
4.0]], [[7.0, 5.0, 1.0], [3.0, 6.0, 4.0], [4.0, 7.0, 8.0]], [[1.0, 1.0, 4.0], [7.0, 6.0, 2.0], [3.0, 6.0, 6.0]]], [[[4.0, 7.0, 
4.0], [5.0, 3.0, 6.0], [1.0, 1.0, 1.0]], [[6.0, 4.0, 2.0], [6.0, 2.0, 2.0], [3.0, 8.0, 2.0]], [[4.0, 6.0, 8.0], [6.0, 4.0, 
4.0], [7.0, 6.0, 7.0]]], [[[1.0, 6.0, 2.0], [1.0, 3.0, 4.0], [2.0, 6.0, 4.0]], [[6.0, 8.0, 4.0], [8.0, 2.0, 2.0], [2.0, 4.0, 
3.0]], [[6.0, 4.0, 1.0], [3.0, 6.0, 5.0], [2.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[5.0], [5.0], [8.0], [5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0], [1.0], [8.0], [4.0]]))
      arg[:,1,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 4.0], [7.0, 6.0, 1.0], [4.0, 2.0, 8.0]], [[7.0, 8.0, 1.0], [3.0, 1.0, 5.0], [2.0, 
2.0, 7.0]], [[7.0, 7.0, 2.0], [6.0, 8.0, 2.0], [7.0, 6.0, 2.0]]], [[[8.0, 5.0, 8.0], [5.0, 7.0, 4.0], [7.0, 8.0, 3.0]], [[2.0, 
6.0, 4.0], [6.0, 5.0, 5.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 7.0], [7.0, 6.0, 8.0], [2.0, 2.0, 4.0]]], [[[1.0, 2.0, 3.0], [1.0, 
5.0, 5.0], [6.0, 3.0, 2.0]], [[2.0, 4.0, 4.0], [5.0, 6.0, 8.0], [3.0, 1.0, 7.0]], [[4.0, 1.0, 6.0], [8.0, 6.0, 3.0], [5.0, 6.0, 
1.0]]], [[[1.0, 2.0, 3.0], [7.0, 3.0, 6.0], [2.0, 5.0, 7.0]], [[7.0, 7.0, 8.0], [8.0, 5.0, 5.0], [6.0, 2.0, 6.0]], [[8.0, 8.0, 
5.0], [1.0, 2.0, 6.0], [7.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 3.0], [8.0, 6.0, 5.0], [5.0, 6.0, 5.0]], 
[[2.0, 7.0, 1.0], [4.0, 6.0, 8.0], [7.0, 4.0, 3.0]], [[3.0, 6.0, 2.0], [3.0, 6.0, 5.0], [1.0, 5.0, 4.0]]], [[[8.0, 2.0, 6.0], 
[7.0, 3.0, 8.0], [2.0, 8.0, 4.0]], [[7.0, 5.0, 1.0], [3.0, 6.0, 1.0], [4.0, 7.0, 8.0]], [[1.0, 1.0, 4.0], [7.0, 6.0, 2.0], 
[3.0, 6.0, 6.0]]], [[[4.0, 7.0, 4.0], [5.0, 3.0, 6.0], [1.0, 1.0, 1.0]], [[6.0, 4.0, 2.0], [6.0, 2.0, 8.0], [3.0, 8.0, 2.0]], 
[[4.0, 6.0, 8.0], [6.0, 4.0, 4.0], [7.0, 6.0, 7.0]]], [[[1.0, 6.0, 2.0], [1.0, 3.0, 4.0], [2.0, 6.0, 4.0]], [[6.0, 8.0, 4.0], 
[8.0, 2.0, 4.0], [2.0, 4.0, 3.0]], [[6.0, 4.0, 1.0], [3.0, 6.0, 5.0], [2.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPPL(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 4.0], [1.0, 6.0, 3.0], [3.0, 7.0, 4.0]], [[8.0, 2.0, 4.0], [5.0, 1.0, 8.0], [6.0, 1.0, 
6.0]], [[6.0, 2.0, 5.0], [8.0, 5.0, 5.0], [8.0, 8.0, 1.0]]], [[[7.0, 7.0, 7.0], [5.0, 1.0, 7.0], [3.0, 1.0, 2.0]], [[7.0, 1.0, 
1.0], [7.0, 5.0, 4.0], [2.0, 1.0, 3.0]], [[8.0, 1.0, 5.0], [4.0, 2.0, 8.0], [3.0, 1.0, 5.0]]], [[[3.0, 4.0, 7.0], [6.0, 1.0, 
8.0], [2.0, 8.0, 4.0]], [[4.0, 8.0, 3.0], [5.0, 4.0, 3.0], [2.0, 3.0, 2.0]], [[3.0, 5.0, 5.0], [1.0, 3.0, 2.0], [7.0, 5.0, 
7.0]]], [[[5.0, 8.0, 6.0], [5.0, 2.0, 4.0], [8.0, 3.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 6.0, 1.0], [3.0, 2.0, 2.0]], [[4.0, 5.0, 
4.0], [8.0, 1.0, 8.0], [4.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 7.0], [8.0, 4.0, 8.0], [8.0, 3.0, 1.0]], [[3.0, 7.0, 8.0], [5.0, 5.0, 
2.0], [2.0, 3.0, 6.0]], [[5.0, 3.0, 2.0], [7.0, 7.0, 8.0], [6.0, 4.0, 1.0]]], [[[2.0, 8.0, 1.0], [8.0, 4.0, 4.0], [3.0, 1.0, 
4.0]], [[6.0, 1.0, 5.0], [6.0, 1.0, 1.0], [2.0, 7.0, 5.0]], [[6.0, 2.0, 7.0], [1.0, 3.0, 7.0], [8.0, 8.0, 7.0]]], [[[8.0, 1.0, 
2.0], [7.0, 2.0, 4.0], [3.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], [7.0, 3.0, 1.0], [6.0, 3.0, 6.0]], [[4.0, 5.0, 3.0], [5.0, 5.0, 
4.0], [6.0, 2.0, 5.0]]], [[[1.0, 4.0, 4.0], [1.0, 7.0, 1.0], [8.0, 3.0, 8.0]], [[1.0, 8.0, 2.0], [7.0, 1.0, 2.0], [8.0, 4.0, 
1.0]], [[4.0, 2.0, 4.0], [1.0, 6.0, 8.0], [4.0, 5.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[3.0], [8.0], [8.0], [7.0]])+(1.-msk_val)*numpy.array([[4.0], [3.0], [5.0], [7.0]])
      arg[:,1,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 4.0], [1.0, 6.0, 3.0], [3.0, 7.0, 4.0]], [[8.0, 2.0, 4.0], [5.0, 1.0, 3.0], [6.0, 
1.0, 6.0]], [[6.0, 2.0, 5.0], [8.0, 5.0, 5.0], [8.0, 8.0, 1.0]]], [[[7.0, 7.0, 7.0], [5.0, 1.0, 7.0], [3.0, 1.0, 2.0]], [[7.0, 
1.0, 1.0], [7.0, 5.0, 8.0], [2.0, 1.0, 3.0]], [[8.0, 1.0, 5.0], [4.0, 2.0, 8.0], [3.0, 1.0, 5.0]]], [[[3.0, 4.0, 7.0], [6.0, 
1.0, 8.0], [2.0, 8.0, 4.0]], [[4.0, 8.0, 3.0], [5.0, 4.0, 8.0], [2.0, 3.0, 2.0]], [[3.0, 5.0, 5.0], [1.0, 3.0, 2.0], [7.0, 5.0, 
7.0]]], [[[5.0, 8.0, 6.0], [5.0, 2.0, 4.0], [8.0, 3.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 6.0, 7.0], [3.0, 2.0, 2.0]], [[4.0, 5.0, 
4.0], [8.0, 1.0, 8.0], [4.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 7.0], [8.0, 4.0, 8.0], [8.0, 3.0, 1.0]], 
[[3.0, 7.0, 8.0], [5.0, 5.0, 4.0], [2.0, 3.0, 6.0]], [[5.0, 3.0, 2.0], [7.0, 7.0, 8.0], [6.0, 4.0, 1.0]]], [[[2.0, 8.0, 1.0], 
[8.0, 4.0, 4.0], [3.0, 1.0, 4.0]], [[6.0, 1.0, 5.0], [6.0, 1.0, 3.0], [2.0, 7.0, 5.0]], [[6.0, 2.0, 7.0], [1.0, 3.0, 7.0], 
[8.0, 8.0, 7.0]]], [[[8.0, 1.0, 2.0], [7.0, 2.0, 4.0], [3.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], [7.0, 3.0, 5.0], [6.0, 3.0, 6.0]], 
[[4.0, 5.0, 3.0], [5.0, 5.0, 4.0], [6.0, 2.0, 5.0]]], [[[1.0, 4.0, 4.0], [1.0, 7.0, 1.0], [8.0, 3.0, 8.0]], [[1.0, 8.0, 2.0], 
[7.0, 1.0, 7.0], [8.0, 4.0, 1.0]], [[4.0, 2.0, 4.0], [1.0, 6.0, 8.0], [4.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RLPN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 8.0], [4.0, 5.0, 4.0], [4.0, 6.0, 5.0]], [[5.0, 4.0, 1.0], [8.0, 3.0, 8.0], [3.0, 1.0, 
1.0]], [[1.0, 3.0, 6.0], [6.0, 7.0, 3.0], [4.0, 7.0, 7.0]]], [[[1.0, 7.0, 4.0], [2.0, 5.0, 3.0], [4.0, 1.0, 2.0]], [[2.0, 2.0, 
3.0], [4.0, 1.0, 1.0], [3.0, 3.0, 1.0]], [[8.0, 8.0, 7.0], [3.0, 4.0, 5.0], [2.0, 7.0, 2.0]]], [[[5.0, 6.0, 4.0], [6.0, 5.0, 
2.0], [1.0, 6.0, 1.0]], [[8.0, 8.0, 1.0], [7.0, 1.0, 4.0], [2.0, 6.0, 6.0]], [[8.0, 8.0, 1.0], [1.0, 5.0, 6.0], [8.0, 7.0, 
4.0]]], [[[8.0, 8.0, 5.0], [6.0, 3.0, 3.0], [7.0, 1.0, 2.0]], [[6.0, 3.0, 4.0], [5.0, 8.0, 8.0], [1.0, 2.0, 4.0]], [[2.0, 7.0, 
8.0], [6.0, 4.0, 6.0], [6.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 7.0], [7.0, 6.0, 5.0], [2.0, 6.0, 8.0]], [[3.0, 3.0, 6.0], [8.0, 4.0, 
8.0], [1.0, 2.0, 6.0]], [[6.0, 8.0, 1.0], [8.0, 2.0, 5.0], [7.0, 1.0, 3.0]]], [[[1.0, 5.0, 4.0], [7.0, 1.0, 3.0], [2.0, 2.0, 
8.0]], [[7.0, 6.0, 6.0], [7.0, 2.0, 7.0], [1.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [7.0, 1.0, 5.0], [4.0, 5.0, 2.0]]], [[[4.0, 8.0, 
3.0], [8.0, 8.0, 2.0], [2.0, 6.0, 4.0]], [[2.0, 3.0, 7.0], [4.0, 8.0, 5.0], [3.0, 4.0, 6.0]], [[1.0, 1.0, 3.0], [5.0, 1.0, 
4.0], [4.0, 1.0, 1.0]]], [[[4.0, 5.0, 3.0], [7.0, 2.0, 3.0], [1.0, 4.0, 2.0]], [[8.0, 8.0, 2.0], [6.0, 3.0, 5.0], [6.0, 6.0, 
2.0]], [[4.0, 1.0, 7.0], [3.0, 5.0, 6.0], [2.0, 5.0, 1.0]]]]))
      val=numpy.array([[[4.0, 2.0, 2.0]], [[7.0, 7.0, 3.0]]])
      arg[1:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 8.0], [4.0, 5.0, 4.0], [4.0, 6.0, 5.0]], [[5.0, 4.0, 1.0], [8.0, 3.0, 8.0], [3.0, 
1.0, 1.0]], [[1.0, 3.0, 6.0], [6.0, 7.0, 3.0], [4.0, 7.0, 7.0]]], [[[1.0, 7.0, 4.0], [2.0, 5.0, 3.0], [4.0, 1.0, 2.0]], [[2.0, 
2.0, 3.0], [4.0, 1.0, 1.0], [3.0, 3.0, 1.0]], [[8.0, 8.0, 7.0], [4.0, 2.0, 2.0], [2.0, 7.0, 2.0]]], [[[5.0, 6.0, 4.0], [6.0, 
5.0, 2.0], [1.0, 6.0, 1.0]], [[8.0, 8.0, 1.0], [7.0, 1.0, 4.0], [2.0, 6.0, 6.0]], [[8.0, 8.0, 1.0], [7.0, 7.0, 3.0], [8.0, 7.0, 
4.0]]], [[[8.0, 8.0, 5.0], [6.0, 3.0, 3.0], [7.0, 1.0, 2.0]], [[6.0, 3.0, 4.0], [5.0, 8.0, 8.0], [1.0, 2.0, 4.0]], [[2.0, 7.0, 
8.0], [6.0, 4.0, 6.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 7.0], [7.0, 6.0, 5.0], [2.0, 6.0, 8.0]], 
[[3.0, 3.0, 6.0], [8.0, 4.0, 8.0], [1.0, 2.0, 6.0]], [[6.0, 8.0, 1.0], [8.0, 2.0, 5.0], [7.0, 1.0, 3.0]]], [[[1.0, 5.0, 4.0], 
[7.0, 1.0, 3.0], [2.0, 2.0, 8.0]], [[7.0, 6.0, 6.0], [7.0, 2.0, 7.0], [1.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [4.0, 2.0, 2.0], 
[4.0, 5.0, 2.0]]], [[[4.0, 8.0, 3.0], [8.0, 8.0, 2.0], [2.0, 6.0, 4.0]], [[2.0, 3.0, 7.0], [4.0, 8.0, 5.0], [3.0, 4.0, 6.0]], 
[[1.0, 1.0, 3.0], [7.0, 7.0, 3.0], [4.0, 1.0, 1.0]]], [[[4.0, 5.0, 3.0], [7.0, 2.0, 3.0], [1.0, 4.0, 2.0]], [[8.0, 8.0, 2.0], 
[6.0, 3.0, 5.0], [6.0, 6.0, 2.0]], [[4.0, 1.0, 7.0], [3.0, 5.0, 6.0], [2.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RLPN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 8.0], [7.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[1.0, 3.0, 3.0], [5.0, 8.0, 7.0], [8.0, 2.0, 
4.0]], [[2.0, 1.0, 7.0], [4.0, 2.0, 2.0], [1.0, 6.0, 7.0]]], [[[4.0, 4.0, 8.0], [8.0, 4.0, 8.0], [3.0, 4.0, 4.0]], [[4.0, 1.0, 
2.0], [5.0, 1.0, 7.0], [4.0, 2.0, 6.0]], [[4.0, 2.0, 6.0], [8.0, 1.0, 1.0], [8.0, 1.0, 7.0]]], [[[4.0, 1.0, 5.0], [5.0, 4.0, 
6.0], [8.0, 6.0, 5.0]], [[6.0, 6.0, 2.0], [2.0, 2.0, 2.0], [1.0, 3.0, 2.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 1.0], [2.0, 7.0, 
8.0]]], [[[7.0, 4.0, 3.0], [5.0, 8.0, 2.0], [3.0, 3.0, 1.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 2.0], [4.0, 4.0, 6.0]], [[1.0, 8.0, 
5.0], [5.0, 7.0, 2.0], [2.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 7.0], [1.0, 6.0, 2.0], [3.0, 5.0, 5.0]], [[3.0, 1.0, 3.0], [7.0, 3.0, 
6.0], [7.0, 2.0, 7.0]], [[3.0, 8.0, 7.0], [5.0, 8.0, 8.0], [2.0, 8.0, 3.0]]], [[[7.0, 5.0, 3.0], [3.0, 3.0, 5.0], [7.0, 3.0, 
4.0]], [[8.0, 4.0, 4.0], [7.0, 2.0, 7.0], [8.0, 8.0, 5.0]], [[7.0, 6.0, 1.0], [3.0, 3.0, 7.0], [2.0, 1.0, 2.0]]], [[[6.0, 4.0, 
3.0], [6.0, 3.0, 7.0], [5.0, 1.0, 7.0]], [[6.0, 2.0, 6.0], [2.0, 1.0, 2.0], [8.0, 7.0, 4.0]], [[4.0, 7.0, 7.0], [7.0, 1.0, 
8.0], [4.0, 6.0, 4.0]]], [[[1.0, 6.0, 2.0], [5.0, 6.0, 8.0], [2.0, 8.0, 3.0]], [[3.0, 1.0, 6.0], [6.0, 5.0, 8.0], [7.0, 3.0, 
6.0]], [[5.0, 1.0, 1.0], [2.0, 7.0, 1.0], [1.0, 5.0, 7.0]]]]))
      val=Data(numpy.array([[[7.0, 3.0, 5.0]], [[7.0, 4.0, 2.0]]]),self.functionspace)
      arg[1:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 8.0], [7.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[1.0, 3.0, 3.0], [5.0, 8.0, 7.0], [8.0, 
2.0, 4.0]], [[2.0, 1.0, 7.0], [4.0, 2.0, 2.0], [1.0, 6.0, 7.0]]], [[[4.0, 4.0, 8.0], [8.0, 4.0, 8.0], [3.0, 4.0, 4.0]], [[4.0, 
1.0, 2.0], [5.0, 1.0, 7.0], [4.0, 2.0, 6.0]], [[4.0, 2.0, 6.0], [7.0, 3.0, 5.0], [8.0, 1.0, 7.0]]], [[[4.0, 1.0, 5.0], [5.0, 
4.0, 6.0], [8.0, 6.0, 5.0]], [[6.0, 6.0, 2.0], [2.0, 2.0, 2.0], [1.0, 3.0, 2.0]], [[3.0, 4.0, 3.0], [7.0, 4.0, 2.0], [2.0, 7.0, 
8.0]]], [[[7.0, 4.0, 3.0], [5.0, 8.0, 2.0], [3.0, 3.0, 1.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 2.0], [4.0, 4.0, 6.0]], [[1.0, 8.0, 
5.0], [5.0, 7.0, 2.0], [2.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 7.0], [1.0, 6.0, 2.0], [3.0, 5.0, 5.0]], 
[[3.0, 1.0, 3.0], [7.0, 3.0, 6.0], [7.0, 2.0, 7.0]], [[3.0, 8.0, 7.0], [5.0, 8.0, 8.0], [2.0, 8.0, 3.0]]], [[[7.0, 5.0, 3.0], 
[3.0, 3.0, 5.0], [7.0, 3.0, 4.0]], [[8.0, 4.0, 4.0], [7.0, 2.0, 7.0], [8.0, 8.0, 5.0]], [[7.0, 6.0, 1.0], [7.0, 3.0, 5.0], 
[2.0, 1.0, 2.0]]], [[[6.0, 4.0, 3.0], [6.0, 3.0, 7.0], [5.0, 1.0, 7.0]], [[6.0, 2.0, 6.0], [2.0, 1.0, 2.0], [8.0, 7.0, 4.0]], 
[[4.0, 7.0, 7.0], [7.0, 4.0, 2.0], [4.0, 6.0, 4.0]]], [[[1.0, 6.0, 2.0], [5.0, 6.0, 8.0], [2.0, 8.0, 3.0]], [[3.0, 1.0, 6.0], 
[6.0, 5.0, 8.0], [7.0, 3.0, 6.0]], [[5.0, 1.0, 1.0], [2.0, 7.0, 1.0], [1.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RLPN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 7.0], [2.0, 3.0, 1.0], [7.0, 8.0, 5.0]], [[8.0, 4.0, 4.0], [5.0, 5.0, 3.0], [6.0, 4.0, 
4.0]], [[4.0, 3.0, 5.0], [5.0, 1.0, 4.0], [8.0, 7.0, 3.0]]], [[[8.0, 8.0, 3.0], [3.0, 7.0, 4.0], [2.0, 1.0, 3.0]], [[5.0, 8.0, 
7.0], [4.0, 3.0, 1.0], [6.0, 4.0, 8.0]], [[3.0, 1.0, 8.0], [3.0, 4.0, 2.0], [5.0, 2.0, 2.0]]], [[[7.0, 2.0, 5.0], [7.0, 3.0, 
3.0], [1.0, 6.0, 7.0]], [[8.0, 3.0, 6.0], [7.0, 1.0, 4.0], [1.0, 2.0, 5.0]], [[4.0, 1.0, 1.0], [6.0, 8.0, 1.0], [3.0, 8.0, 
6.0]]], [[[7.0, 6.0, 7.0], [7.0, 2.0, 3.0], [4.0, 1.0, 4.0]], [[2.0, 6.0, 4.0], [8.0, 6.0, 2.0], [1.0, 3.0, 2.0]], [[1.0, 4.0, 
8.0], [8.0, 6.0, 5.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 2.0], [8.0, 4.0, 1.0], [4.0, 8.0, 7.0]], [[1.0, 4.0, 6.0], [7.0, 6.0, 
4.0], [3.0, 5.0, 5.0]], [[3.0, 2.0, 6.0], [2.0, 8.0, 1.0], [7.0, 1.0, 2.0]]], [[[1.0, 4.0, 6.0], [6.0, 7.0, 1.0], [2.0, 5.0, 
2.0]], [[4.0, 8.0, 4.0], [4.0, 5.0, 5.0], [7.0, 3.0, 7.0]], [[7.0, 6.0, 7.0], [3.0, 5.0, 6.0], [1.0, 2.0, 5.0]]], [[[3.0, 5.0, 
1.0], [6.0, 1.0, 8.0], [4.0, 2.0, 8.0]], [[6.0, 2.0, 1.0], [8.0, 2.0, 8.0], [2.0, 2.0, 2.0]], [[8.0, 5.0, 7.0], [7.0, 2.0, 
8.0], [8.0, 1.0, 1.0]]], [[[5.0, 7.0, 4.0], [5.0, 5.0, 4.0], [6.0, 6.0, 4.0]], [[6.0, 1.0, 4.0], [8.0, 5.0, 4.0], [7.0, 4.0, 
4.0]], [[2.0, 7.0, 5.0], [7.0, 2.0, 5.0], [4.0, 5.0, 5.0]]]]))
      val=Data(numpy.array([[[7.0, 8.0, 2.0]], [[4.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0, 1.0]], [[4.0, 2.0, 8.0]]]))
      arg[1:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 7.0], [2.0, 3.0, 1.0], [7.0, 8.0, 5.0]], [[8.0, 4.0, 4.0], [5.0, 5.0, 3.0], [6.0, 
4.0, 4.0]], [[4.0, 3.0, 5.0], [5.0, 1.0, 4.0], [8.0, 7.0, 3.0]]], [[[8.0, 8.0, 3.0], [3.0, 7.0, 4.0], [2.0, 1.0, 3.0]], [[5.0, 
8.0, 7.0], [4.0, 3.0, 1.0], [6.0, 4.0, 8.0]], [[3.0, 1.0, 8.0], [7.0, 8.0, 2.0], [5.0, 2.0, 2.0]]], [[[7.0, 2.0, 5.0], [7.0, 
3.0, 3.0], [1.0, 6.0, 7.0]], [[8.0, 3.0, 6.0], [7.0, 1.0, 4.0], [1.0, 2.0, 5.0]], [[4.0, 1.0, 1.0], [4.0, 3.0, 1.0], [3.0, 8.0, 
6.0]]], [[[7.0, 6.0, 7.0], [7.0, 2.0, 3.0], [4.0, 1.0, 4.0]], [[2.0, 6.0, 4.0], [8.0, 6.0, 2.0], [1.0, 3.0, 2.0]], [[1.0, 4.0, 
8.0], [8.0, 6.0, 5.0], [1.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 2.0], [8.0, 4.0, 1.0], [4.0, 8.0, 7.0]], 
[[1.0, 4.0, 6.0], [7.0, 6.0, 4.0], [3.0, 5.0, 5.0]], [[3.0, 2.0, 6.0], [2.0, 8.0, 1.0], [7.0, 1.0, 2.0]]], [[[1.0, 4.0, 6.0], 
[6.0, 7.0, 1.0], [2.0, 5.0, 2.0]], [[4.0, 8.0, 4.0], [4.0, 5.0, 5.0], [7.0, 3.0, 7.0]], [[7.0, 6.0, 7.0], [3.0, 5.0, 1.0], 
[1.0, 2.0, 5.0]]], [[[3.0, 5.0, 1.0], [6.0, 1.0, 8.0], [4.0, 2.0, 8.0]], [[6.0, 2.0, 1.0], [8.0, 2.0, 8.0], [2.0, 2.0, 2.0]], 
[[8.0, 5.0, 7.0], [4.0, 2.0, 8.0], [8.0, 1.0, 1.0]]], [[[5.0, 7.0, 4.0], [5.0, 5.0, 4.0], [6.0, 6.0, 4.0]], [[6.0, 1.0, 4.0], 
[8.0, 5.0, 4.0], [7.0, 4.0, 4.0]], [[2.0, 7.0, 5.0], [7.0, 2.0, 5.0], [4.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RLPN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 8.0], [2.0, 1.0, 2.0], [3.0, 7.0, 6.0]], [[7.0, 4.0, 5.0], [6.0, 5.0, 4.0], [3.0, 5.0, 
8.0]], [[2.0, 1.0, 7.0], [5.0, 2.0, 8.0], [7.0, 3.0, 6.0]]], [[[2.0, 3.0, 6.0], [3.0, 3.0, 1.0], [4.0, 2.0, 1.0]], [[5.0, 8.0, 
7.0], [8.0, 4.0, 4.0], [6.0, 1.0, 6.0]], [[1.0, 7.0, 8.0], [3.0, 1.0, 4.0], [5.0, 7.0, 2.0]]], [[[2.0, 2.0, 6.0], [1.0, 1.0, 
2.0], [6.0, 7.0, 3.0]], [[4.0, 2.0, 5.0], [1.0, 8.0, 4.0], [4.0, 3.0, 8.0]], [[7.0, 6.0, 7.0], [5.0, 7.0, 1.0], [7.0, 8.0, 
6.0]]], [[[1.0, 1.0, 5.0], [7.0, 5.0, 1.0], [5.0, 4.0, 3.0]], [[2.0, 5.0, 2.0], [5.0, 6.0, 7.0], [7.0, 8.0, 7.0]], [[7.0, 5.0, 
4.0], [5.0, 4.0, 6.0], [7.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 8.0], [7.0, 6.0, 4.0], [5.0, 7.0, 6.0]], [[1.0, 3.0, 3.0], [4.0, 7.0, 
3.0], [3.0, 2.0, 4.0]], [[3.0, 3.0, 1.0], [5.0, 8.0, 7.0], [2.0, 5.0, 1.0]]], [[[3.0, 2.0, 2.0], [2.0, 1.0, 2.0], [3.0, 7.0, 
3.0]], [[7.0, 1.0, 3.0], [5.0, 3.0, 5.0], [8.0, 3.0, 5.0]], [[2.0, 6.0, 3.0], [7.0, 8.0, 5.0], [1.0, 7.0, 8.0]]], [[[6.0, 6.0, 
5.0], [4.0, 2.0, 5.0], [4.0, 5.0, 7.0]], [[4.0, 8.0, 1.0], [6.0, 2.0, 4.0], [8.0, 2.0, 8.0]], [[5.0, 4.0, 2.0], [3.0, 5.0, 
7.0], [6.0, 5.0, 2.0]]], [[[3.0, 1.0, 3.0], [5.0, 5.0, 3.0], [8.0, 3.0, 3.0]], [[2.0, 6.0, 2.0], [6.0, 5.0, 6.0], [3.0, 4.0, 
5.0]], [[1.0, 1.0, 2.0], [7.0, 2.0, 6.0], [5.0, 3.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0, 5.0, 1.0]], [[6.0, 1.0, 4.0]]])+(1.-msk_val)*numpy.array([[[3.0, 3.0, 4.0]], [[4.0, 6.0, 
5.0]]])
      arg[1:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 8.0], [2.0, 1.0, 2.0], [3.0, 7.0, 6.0]], [[7.0, 4.0, 5.0], [6.0, 5.0, 4.0], [3.0, 
5.0, 8.0]], [[2.0, 1.0, 7.0], [5.0, 2.0, 8.0], [7.0, 3.0, 6.0]]], [[[2.0, 3.0, 6.0], [3.0, 3.0, 1.0], [4.0, 2.0, 1.0]], [[5.0, 
8.0, 7.0], [8.0, 4.0, 4.0], [6.0, 1.0, 6.0]], [[1.0, 7.0, 8.0], [1.0, 5.0, 1.0], [5.0, 7.0, 2.0]]], [[[2.0, 2.0, 6.0], [1.0, 
1.0, 2.0], [6.0, 7.0, 3.0]], [[4.0, 2.0, 5.0], [1.0, 8.0, 4.0], [4.0, 3.0, 8.0]], [[7.0, 6.0, 7.0], [6.0, 1.0, 4.0], [7.0, 8.0, 
6.0]]], [[[1.0, 1.0, 5.0], [7.0, 5.0, 1.0], [5.0, 4.0, 3.0]], [[2.0, 5.0, 2.0], [5.0, 6.0, 7.0], [7.0, 8.0, 7.0]], [[7.0, 5.0, 
4.0], [5.0, 4.0, 6.0], [7.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 8.0], [7.0, 6.0, 4.0], [5.0, 7.0, 6.0]], 
[[1.0, 3.0, 3.0], [4.0, 7.0, 3.0], [3.0, 2.0, 4.0]], [[3.0, 3.0, 1.0], [5.0, 8.0, 7.0], [2.0, 5.0, 1.0]]], [[[3.0, 2.0, 2.0], 
[2.0, 1.0, 2.0], [3.0, 7.0, 3.0]], [[7.0, 1.0, 3.0], [5.0, 3.0, 5.0], [8.0, 3.0, 5.0]], [[2.0, 6.0, 3.0], [3.0, 3.0, 4.0], 
[1.0, 7.0, 8.0]]], [[[6.0, 6.0, 5.0], [4.0, 2.0, 5.0], [4.0, 5.0, 7.0]], [[4.0, 8.0, 1.0], [6.0, 2.0, 4.0], [8.0, 2.0, 8.0]], 
[[5.0, 4.0, 2.0], [4.0, 6.0, 5.0], [6.0, 5.0, 2.0]]], [[[3.0, 1.0, 3.0], [5.0, 5.0, 3.0], [8.0, 3.0, 3.0]], [[2.0, 6.0, 2.0], 
[6.0, 5.0, 6.0], [3.0, 4.0, 5.0]], [[1.0, 1.0, 2.0], [7.0, 2.0, 6.0], [5.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPCP(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 2.0], [2.0, 2.0, 3.0], [6.0, 5.0, 6.0]], [[4.0, 4.0, 5.0], [7.0, 7.0, 2.0], [6.0, 4.0, 
1.0]], [[7.0, 4.0, 5.0], [6.0, 1.0, 2.0], [5.0, 1.0, 3.0]]], [[[4.0, 3.0, 5.0], [3.0, 1.0, 6.0], [1.0, 2.0, 4.0]], [[5.0, 8.0, 
6.0], [1.0, 3.0, 8.0], [6.0, 3.0, 8.0]], [[2.0, 5.0, 2.0], [2.0, 1.0, 5.0], [3.0, 7.0, 8.0]]], [[[6.0, 2.0, 7.0], [3.0, 6.0, 
2.0], [4.0, 6.0, 2.0]], [[8.0, 3.0, 5.0], [6.0, 3.0, 1.0], [3.0, 7.0, 5.0]], [[4.0, 6.0, 7.0], [2.0, 7.0, 5.0], [6.0, 5.0, 
7.0]]], [[[5.0, 5.0, 3.0], [8.0, 2.0, 8.0], [6.0, 6.0, 3.0]], [[6.0, 6.0, 1.0], [5.0, 4.0, 5.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 
2.0], [4.0, 3.0, 8.0], [7.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 6.0], [8.0, 8.0, 6.0], [5.0, 4.0, 1.0]], [[6.0, 5.0, 3.0], [1.0, 4.0, 
6.0], [3.0, 2.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 2.0, 8.0], [8.0, 1.0, 1.0]]], [[[2.0, 5.0, 6.0], [8.0, 7.0, 7.0], [8.0, 4.0, 
4.0]], [[4.0, 4.0, 2.0], [2.0, 4.0, 4.0], [8.0, 3.0, 4.0]], [[5.0, 4.0, 3.0], [7.0, 7.0, 2.0], [4.0, 8.0, 6.0]]], [[[4.0, 3.0, 
2.0], [6.0, 8.0, 6.0], [2.0, 2.0, 6.0]], [[2.0, 7.0, 1.0], [5.0, 5.0, 1.0], [3.0, 6.0, 7.0]], [[1.0, 7.0, 2.0], [8.0, 5.0, 
6.0], [4.0, 5.0, 5.0]]], [[[1.0, 7.0, 1.0], [6.0, 8.0, 2.0], [3.0, 4.0, 4.0]], [[7.0, 5.0, 4.0], [5.0, 7.0, 5.0], [5.0, 1.0, 
6.0]], [[2.0, 4.0, 2.0], [3.0, 6.0, 2.0], [1.0, 7.0, 7.0]]]]))
      val=numpy.array([[5.0, 8.0, 8.0], [6.0, 4.0, 6.0], [4.0, 2.0, 8.0], [2.0, 4.0, 8.0]])
      arg[:,1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 2.0], [2.0, 2.0, 3.0], [6.0, 5.0, 6.0]], [[4.0, 5.0, 5.0], [7.0, 8.0, 2.0], [6.0, 
8.0, 1.0]], [[7.0, 4.0, 5.0], [6.0, 1.0, 2.0], [5.0, 1.0, 3.0]]], [[[4.0, 3.0, 5.0], [3.0, 1.0, 6.0], [1.0, 2.0, 4.0]], [[5.0, 
6.0, 6.0], [1.0, 4.0, 8.0], [6.0, 6.0, 8.0]], [[2.0, 5.0, 2.0], [2.0, 1.0, 5.0], [3.0, 7.0, 8.0]]], [[[6.0, 2.0, 7.0], [3.0, 
6.0, 2.0], [4.0, 6.0, 2.0]], [[8.0, 4.0, 5.0], [6.0, 2.0, 1.0], [3.0, 8.0, 5.0]], [[4.0, 6.0, 7.0], [2.0, 7.0, 5.0], [6.0, 5.0, 
7.0]]], [[[5.0, 5.0, 3.0], [8.0, 2.0, 8.0], [6.0, 6.0, 3.0]], [[6.0, 2.0, 1.0], [5.0, 4.0, 5.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 
2.0], [4.0, 3.0, 8.0], [7.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 6.0], [8.0, 8.0, 6.0], [5.0, 4.0, 1.0]], 
[[6.0, 5.0, 3.0], [1.0, 8.0, 6.0], [3.0, 8.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 2.0, 8.0], [8.0, 1.0, 1.0]]], [[[2.0, 5.0, 6.0], 
[8.0, 7.0, 7.0], [8.0, 4.0, 4.0]], [[4.0, 6.0, 2.0], [2.0, 4.0, 4.0], [8.0, 6.0, 4.0]], [[5.0, 4.0, 3.0], [7.0, 7.0, 2.0], 
[4.0, 8.0, 6.0]]], [[[4.0, 3.0, 2.0], [6.0, 8.0, 6.0], [2.0, 2.0, 6.0]], [[2.0, 4.0, 1.0], [5.0, 2.0, 1.0], [3.0, 8.0, 7.0]], 
[[1.0, 7.0, 2.0], [8.0, 5.0, 6.0], [4.0, 5.0, 5.0]]], [[[1.0, 7.0, 1.0], [6.0, 8.0, 2.0], [3.0, 4.0, 4.0]], [[7.0, 2.0, 4.0], 
[5.0, 4.0, 5.0], [5.0, 8.0, 6.0]], [[2.0, 4.0, 2.0], [3.0, 6.0, 2.0], [1.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPCP(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 6.0], [7.0, 3.0, 2.0], [3.0, 1.0, 5.0]], [[5.0, 7.0, 2.0], [8.0, 4.0, 3.0], [5.0, 3.0, 
5.0]], [[1.0, 8.0, 6.0], [2.0, 7.0, 6.0], [3.0, 8.0, 3.0]]], [[[8.0, 1.0, 3.0], [8.0, 2.0, 2.0], [2.0, 1.0, 5.0]], [[8.0, 5.0, 
2.0], [3.0, 2.0, 8.0], [1.0, 3.0, 2.0]], [[5.0, 7.0, 5.0], [5.0, 5.0, 7.0], [8.0, 8.0, 7.0]]], [[[4.0, 1.0, 1.0], [3.0, 2.0, 
4.0], [5.0, 8.0, 3.0]], [[8.0, 4.0, 3.0], [3.0, 5.0, 1.0], [2.0, 4.0, 1.0]], [[5.0, 3.0, 8.0], [8.0, 3.0, 3.0], [8.0, 5.0, 
7.0]]], [[[8.0, 1.0, 1.0], [8.0, 5.0, 6.0], [5.0, 3.0, 6.0]], [[1.0, 4.0, 4.0], [8.0, 2.0, 6.0], [8.0, 2.0, 1.0]], [[3.0, 5.0, 
4.0], [2.0, 6.0, 3.0], [8.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 2.0], [8.0, 1.0, 2.0], [8.0, 6.0, 6.0]], [[3.0, 7.0, 5.0], [5.0, 6.0, 
2.0], [7.0, 1.0, 4.0]], [[7.0, 1.0, 6.0], [4.0, 7.0, 8.0], [4.0, 3.0, 8.0]]], [[[5.0, 1.0, 8.0], [2.0, 6.0, 2.0], [2.0, 4.0, 
1.0]], [[3.0, 6.0, 6.0], [2.0, 7.0, 1.0], [6.0, 6.0, 2.0]], [[2.0, 3.0, 1.0], [4.0, 8.0, 4.0], [3.0, 5.0, 3.0]]], [[[7.0, 1.0, 
5.0], [7.0, 3.0, 1.0], [7.0, 1.0, 6.0]], [[2.0, 3.0, 2.0], [5.0, 7.0, 7.0], [5.0, 7.0, 2.0]], [[8.0, 8.0, 3.0], [7.0, 2.0, 
4.0], [8.0, 7.0, 7.0]]], [[[8.0, 6.0, 4.0], [4.0, 3.0, 5.0], [3.0, 2.0, 8.0]], [[4.0, 8.0, 3.0], [6.0, 1.0, 4.0], [6.0, 1.0, 
3.0]], [[3.0, 1.0, 6.0], [7.0, 5.0, 7.0], [1.0, 7.0, 8.0]]]]))
      val=Data(numpy.array([[7.0, 4.0, 1.0], [2.0, 7.0, 4.0], [8.0, 1.0, 1.0], [8.0, 1.0, 4.0]]),self.functionspace)
      arg[:,1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 6.0], [7.0, 3.0, 2.0], [3.0, 1.0, 5.0]], [[5.0, 7.0, 2.0], [8.0, 4.0, 3.0], [5.0, 
1.0, 5.0]], [[1.0, 8.0, 6.0], [2.0, 7.0, 6.0], [3.0, 8.0, 3.0]]], [[[8.0, 1.0, 3.0], [8.0, 2.0, 2.0], [2.0, 1.0, 5.0]], [[8.0, 
2.0, 2.0], [3.0, 7.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 7.0, 5.0], [5.0, 5.0, 7.0], [8.0, 8.0, 7.0]]], [[[4.0, 1.0, 1.0], [3.0, 
2.0, 4.0], [5.0, 8.0, 3.0]], [[8.0, 8.0, 3.0], [3.0, 1.0, 1.0], [2.0, 1.0, 1.0]], [[5.0, 3.0, 8.0], [8.0, 3.0, 3.0], [8.0, 5.0, 
7.0]]], [[[8.0, 1.0, 1.0], [8.0, 5.0, 6.0], [5.0, 3.0, 6.0]], [[1.0, 8.0, 4.0], [8.0, 1.0, 6.0], [8.0, 4.0, 1.0]], [[3.0, 5.0, 
4.0], [2.0, 6.0, 3.0], [8.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 2.0], [8.0, 1.0, 2.0], [8.0, 6.0, 6.0]], 
[[3.0, 7.0, 5.0], [5.0, 4.0, 2.0], [7.0, 1.0, 4.0]], [[7.0, 1.0, 6.0], [4.0, 7.0, 8.0], [4.0, 3.0, 8.0]]], [[[5.0, 1.0, 8.0], 
[2.0, 6.0, 2.0], [2.0, 4.0, 1.0]], [[3.0, 2.0, 6.0], [2.0, 7.0, 1.0], [6.0, 4.0, 2.0]], [[2.0, 3.0, 1.0], [4.0, 8.0, 4.0], 
[3.0, 5.0, 3.0]]], [[[7.0, 1.0, 5.0], [7.0, 3.0, 1.0], [7.0, 1.0, 6.0]], [[2.0, 8.0, 2.0], [5.0, 1.0, 7.0], [5.0, 1.0, 2.0]], 
[[8.0, 8.0, 3.0], [7.0, 2.0, 4.0], [8.0, 7.0, 7.0]]], [[[8.0, 6.0, 4.0], [4.0, 3.0, 5.0], [3.0, 2.0, 8.0]], [[4.0, 8.0, 3.0], 
[6.0, 1.0, 4.0], [6.0, 4.0, 3.0]], [[3.0, 1.0, 6.0], [7.0, 5.0, 7.0], [1.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPCP(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 3.0], [2.0, 8.0, 7.0], [8.0, 5.0, 7.0]], [[3.0, 8.0, 5.0], [5.0, 5.0, 2.0], [3.0, 3.0, 
2.0]], [[2.0, 4.0, 6.0], [7.0, 1.0, 1.0], [7.0, 3.0, 6.0]]], [[[3.0, 2.0, 8.0], [7.0, 1.0, 6.0], [4.0, 2.0, 5.0]], [[3.0, 6.0, 
6.0], [1.0, 6.0, 8.0], [3.0, 6.0, 7.0]], [[1.0, 4.0, 3.0], [3.0, 4.0, 1.0], [5.0, 7.0, 1.0]]], [[[6.0, 4.0, 5.0], [6.0, 6.0, 
2.0], [3.0, 5.0, 4.0]], [[6.0, 2.0, 4.0], [4.0, 7.0, 1.0], [7.0, 8.0, 3.0]], [[8.0, 1.0, 5.0], [7.0, 2.0, 3.0], [5.0, 6.0, 
4.0]]], [[[6.0, 8.0, 1.0], [8.0, 7.0, 3.0], [2.0, 8.0, 4.0]], [[6.0, 6.0, 4.0], [4.0, 1.0, 3.0], [8.0, 8.0, 3.0]], [[4.0, 3.0, 
5.0], [5.0, 8.0, 8.0], [4.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 8.0], [1.0, 1.0, 8.0], [8.0, 3.0, 5.0]], [[2.0, 8.0, 1.0], [8.0, 3.0, 
2.0], [8.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [4.0, 1.0, 4.0], [5.0, 1.0, 2.0]]], [[[3.0, 3.0, 5.0], [1.0, 3.0, 5.0], [3.0, 5.0, 
5.0]], [[7.0, 8.0, 1.0], [6.0, 5.0, 8.0], [8.0, 4.0, 4.0]], [[3.0, 5.0, 4.0], [6.0, 7.0, 2.0], [6.0, 6.0, 7.0]]], [[[1.0, 4.0, 
4.0], [6.0, 1.0, 8.0], [2.0, 4.0, 8.0]], [[6.0, 2.0, 6.0], [1.0, 6.0, 5.0], [7.0, 4.0, 2.0]], [[8.0, 2.0, 3.0], [4.0, 5.0, 
1.0], [2.0, 5.0, 6.0]]], [[[5.0, 1.0, 1.0], [7.0, 3.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 2.0, 1.0], [4.0, 5.0, 6.0], [6.0, 2.0, 
5.0]], [[3.0, 5.0, 8.0], [7.0, 8.0, 7.0], [7.0, 3.0, 7.0]]]]))
      val=Data(numpy.array([[7.0, 3.0, 6.0], [2.0, 5.0, 5.0], [3.0, 6.0, 3.0], [6.0, 8.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 4.0, 6.0], [2.0, 2.0, 2.0], [1.0, 6.0, 5.0], [2.0, 8.0, 1.0]]))
      arg[:,1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 3.0], [2.0, 8.0, 7.0], [8.0, 5.0, 7.0]], [[3.0, 7.0, 5.0], [5.0, 3.0, 2.0], [3.0, 
6.0, 2.0]], [[2.0, 4.0, 6.0], [7.0, 1.0, 1.0], [7.0, 3.0, 6.0]]], [[[3.0, 2.0, 8.0], [7.0, 1.0, 6.0], [4.0, 2.0, 5.0]], [[3.0, 
2.0, 6.0], [1.0, 5.0, 8.0], [3.0, 5.0, 7.0]], [[1.0, 4.0, 3.0], [3.0, 4.0, 1.0], [5.0, 7.0, 1.0]]], [[[6.0, 4.0, 5.0], [6.0, 
6.0, 2.0], [3.0, 5.0, 4.0]], [[6.0, 3.0, 4.0], [4.0, 6.0, 1.0], [7.0, 3.0, 3.0]], [[8.0, 1.0, 5.0], [7.0, 2.0, 3.0], [5.0, 6.0, 
4.0]]], [[[6.0, 8.0, 1.0], [8.0, 7.0, 3.0], [2.0, 8.0, 4.0]], [[6.0, 6.0, 4.0], [4.0, 8.0, 3.0], [8.0, 4.0, 3.0]], [[4.0, 3.0, 
5.0], [5.0, 8.0, 8.0], [4.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 8.0], [1.0, 1.0, 8.0], [8.0, 3.0, 5.0]], 
[[2.0, 5.0, 1.0], [8.0, 4.0, 2.0], [8.0, 6.0, 5.0]], [[1.0, 8.0, 5.0], [4.0, 1.0, 4.0], [5.0, 1.0, 2.0]]], [[[3.0, 3.0, 5.0], 
[1.0, 3.0, 5.0], [3.0, 5.0, 5.0]], [[7.0, 2.0, 1.0], [6.0, 2.0, 8.0], [8.0, 2.0, 4.0]], [[3.0, 5.0, 4.0], [6.0, 7.0, 2.0], 
[6.0, 6.0, 7.0]]], [[[1.0, 4.0, 4.0], [6.0, 1.0, 8.0], [2.0, 4.0, 8.0]], [[6.0, 1.0, 6.0], [1.0, 6.0, 5.0], [7.0, 5.0, 2.0]], 
[[8.0, 2.0, 3.0], [4.0, 5.0, 1.0], [2.0, 5.0, 6.0]]], [[[5.0, 1.0, 1.0], [7.0, 3.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 2.0, 1.0], 
[4.0, 8.0, 6.0], [6.0, 1.0, 5.0]], [[3.0, 5.0, 8.0], [7.0, 8.0, 7.0], [7.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPCP(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 1.0], [4.0, 3.0, 2.0], [4.0, 5.0, 6.0]], [[4.0, 5.0, 2.0], [6.0, 2.0, 6.0], [3.0, 5.0, 
2.0]], [[6.0, 3.0, 5.0], [4.0, 4.0, 4.0], [4.0, 4.0, 6.0]]], [[[1.0, 6.0, 7.0], [3.0, 5.0, 2.0], [7.0, 3.0, 6.0]], [[7.0, 7.0, 
3.0], [7.0, 3.0, 7.0], [8.0, 5.0, 2.0]], [[6.0, 2.0, 6.0], [1.0, 4.0, 3.0], [8.0, 7.0, 4.0]]], [[[1.0, 8.0, 7.0], [6.0, 7.0, 
4.0], [3.0, 5.0, 3.0]], [[1.0, 1.0, 8.0], [6.0, 2.0, 4.0], [5.0, 2.0, 7.0]], [[4.0, 1.0, 2.0], [1.0, 1.0, 2.0], [2.0, 5.0, 
8.0]]], [[[8.0, 5.0, 1.0], [6.0, 4.0, 6.0], [3.0, 4.0, 5.0]], [[5.0, 7.0, 6.0], [6.0, 3.0, 8.0], [7.0, 6.0, 6.0]], [[8.0, 3.0, 
7.0], [2.0, 8.0, 6.0], [5.0, 3.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 4.0], [4.0, 3.0, 8.0], [5.0, 4.0, 3.0]], [[4.0, 4.0, 2.0], [7.0, 6.0, 
5.0], [6.0, 5.0, 6.0]], [[5.0, 6.0, 8.0], [8.0, 3.0, 2.0], [2.0, 2.0, 8.0]]], [[[4.0, 3.0, 1.0], [3.0, 3.0, 7.0], [4.0, 3.0, 
6.0]], [[3.0, 4.0, 1.0], [2.0, 5.0, 8.0], [3.0, 4.0, 8.0]], [[6.0, 7.0, 5.0], [8.0, 4.0, 6.0], [3.0, 7.0, 4.0]]], [[[6.0, 6.0, 
6.0], [4.0, 3.0, 2.0], [8.0, 7.0, 3.0]], [[2.0, 2.0, 3.0], [3.0, 7.0, 3.0], [2.0, 7.0, 4.0]], [[6.0, 1.0, 6.0], [1.0, 2.0, 
8.0], [7.0, 6.0, 3.0]]], [[[4.0, 7.0, 2.0], [5.0, 6.0, 7.0], [6.0, 3.0, 6.0]], [[2.0, 1.0, 6.0], [3.0, 5.0, 2.0], [8.0, 1.0, 
6.0]], [[5.0, 8.0, 8.0], [1.0, 2.0, 1.0], [8.0, 7.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[4.0, 2.0, 7.0], [7.0, 6.0, 5.0], [6.0, 3.0, 3.0], [1.0, 8.0, 
7.0]])+(1.-msk_val)*numpy.array([[3.0, 1.0, 2.0], [2.0, 3.0, 8.0], [8.0, 5.0, 1.0], [2.0, 4.0, 4.0]])
      arg[:,1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 1.0], [4.0, 3.0, 2.0], [4.0, 5.0, 6.0]], [[4.0, 4.0, 2.0], [6.0, 2.0, 6.0], [3.0, 
7.0, 2.0]], [[6.0, 3.0, 5.0], [4.0, 4.0, 4.0], [4.0, 4.0, 6.0]]], [[[1.0, 6.0, 7.0], [3.0, 5.0, 2.0], [7.0, 3.0, 6.0]], [[7.0, 
7.0, 3.0], [7.0, 6.0, 7.0], [8.0, 5.0, 2.0]], [[6.0, 2.0, 6.0], [1.0, 4.0, 3.0], [8.0, 7.0, 4.0]]], [[[1.0, 8.0, 7.0], [6.0, 
7.0, 4.0], [3.0, 5.0, 3.0]], [[1.0, 6.0, 8.0], [6.0, 3.0, 4.0], [5.0, 3.0, 7.0]], [[4.0, 1.0, 2.0], [1.0, 1.0, 2.0], [2.0, 5.0, 
8.0]]], [[[8.0, 5.0, 1.0], [6.0, 4.0, 6.0], [3.0, 4.0, 5.0]], [[5.0, 1.0, 6.0], [6.0, 8.0, 8.0], [7.0, 7.0, 6.0]], [[8.0, 3.0, 
7.0], [2.0, 8.0, 6.0], [5.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 4.0], [4.0, 3.0, 8.0], [5.0, 4.0, 3.0]], 
[[4.0, 3.0, 2.0], [7.0, 1.0, 5.0], [6.0, 2.0, 6.0]], [[5.0, 6.0, 8.0], [8.0, 3.0, 2.0], [2.0, 2.0, 8.0]]], [[[4.0, 3.0, 1.0], 
[3.0, 3.0, 7.0], [4.0, 3.0, 6.0]], [[3.0, 2.0, 1.0], [2.0, 3.0, 8.0], [3.0, 8.0, 8.0]], [[6.0, 7.0, 5.0], [8.0, 4.0, 6.0], 
[3.0, 7.0, 4.0]]], [[[6.0, 6.0, 6.0], [4.0, 3.0, 2.0], [8.0, 7.0, 3.0]], [[2.0, 8.0, 3.0], [3.0, 5.0, 3.0], [2.0, 1.0, 4.0]], 
[[6.0, 1.0, 6.0], [1.0, 2.0, 8.0], [7.0, 6.0, 3.0]]], [[[4.0, 7.0, 2.0], [5.0, 6.0, 7.0], [6.0, 3.0, 6.0]], [[2.0, 2.0, 6.0], 
[3.0, 4.0, 2.0], [8.0, 4.0, 6.0]], [[5.0, 8.0, 8.0], [1.0, 2.0, 1.0], [8.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LCRN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 2.0], [5.0, 7.0, 3.0], [6.0, 1.0, 5.0]], [[4.0, 4.0, 4.0], [3.0, 1.0, 5.0], [1.0, 2.0, 
2.0]], [[8.0, 2.0, 5.0], [4.0, 4.0, 3.0], [7.0, 3.0, 6.0]]], [[[2.0, 8.0, 2.0], [2.0, 1.0, 1.0], [8.0, 6.0, 3.0]], [[4.0, 4.0, 
8.0], [2.0, 1.0, 4.0], [5.0, 1.0, 8.0]], [[3.0, 3.0, 1.0], [8.0, 4.0, 1.0], [1.0, 2.0, 7.0]]], [[[3.0, 6.0, 7.0], [3.0, 7.0, 
5.0], [1.0, 8.0, 4.0]], [[6.0, 8.0, 3.0], [7.0, 8.0, 4.0], [6.0, 4.0, 3.0]], [[4.0, 2.0, 4.0], [4.0, 6.0, 6.0], [7.0, 4.0, 
6.0]]], [[[6.0, 6.0, 3.0], [4.0, 4.0, 8.0], [2.0, 6.0, 8.0]], [[4.0, 8.0, 8.0], [3.0, 4.0, 3.0], [8.0, 5.0, 3.0]], [[5.0, 7.0, 
4.0], [8.0, 1.0, 5.0], [1.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 7.0], [7.0, 2.0, 1.0], [6.0, 7.0, 2.0]], [[2.0, 7.0, 1.0], [4.0, 6.0, 
1.0], [1.0, 4.0, 5.0]], [[8.0, 4.0, 3.0], [2.0, 4.0, 5.0], [2.0, 3.0, 3.0]]], [[[8.0, 6.0, 3.0], [2.0, 6.0, 3.0], [4.0, 2.0, 
5.0]], [[3.0, 1.0, 1.0], [8.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[4.0, 5.0, 4.0], [2.0, 6.0, 2.0], [2.0, 6.0, 1.0]]], [[[3.0, 2.0, 
4.0], [6.0, 2.0, 1.0], [3.0, 6.0, 5.0]], [[3.0, 6.0, 7.0], [2.0, 2.0, 4.0], [5.0, 7.0, 2.0]], [[8.0, 1.0, 3.0], [1.0, 8.0, 
3.0], [6.0, 4.0, 6.0]]], [[[6.0, 5.0, 5.0], [5.0, 1.0, 8.0], [4.0, 3.0, 6.0]], [[7.0, 3.0, 6.0], [3.0, 2.0, 5.0], [7.0, 2.0, 
3.0]], [[3.0, 4.0, 5.0], [1.0, 7.0, 2.0], [4.0, 8.0, 3.0]]]]))
      val=numpy.array([[[[3.0, 8.0, 6.0]], [[6.0, 8.0, 3.0]], [[8.0, 6.0, 2.0]]], [[[7.0, 6.0, 6.0]], [[8.0, 3.0, 8.0]], [[8.0, 
6.0, 5.0]]]])
      arg[2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 2.0], [5.0, 7.0, 3.0], [6.0, 1.0, 5.0]], [[4.0, 4.0, 4.0], [3.0, 1.0, 5.0], [1.0, 
2.0, 2.0]], [[8.0, 2.0, 5.0], [4.0, 4.0, 3.0], [7.0, 3.0, 6.0]]], [[[2.0, 8.0, 2.0], [2.0, 1.0, 1.0], [8.0, 6.0, 3.0]], [[4.0, 
4.0, 8.0], [2.0, 1.0, 4.0], [5.0, 1.0, 8.0]], [[3.0, 3.0, 1.0], [8.0, 4.0, 1.0], [1.0, 2.0, 7.0]]], [[[3.0, 6.0, 7.0], [3.0, 
8.0, 6.0], [1.0, 8.0, 4.0]], [[6.0, 8.0, 3.0], [6.0, 8.0, 3.0], [6.0, 4.0, 3.0]], [[4.0, 2.0, 4.0], [8.0, 6.0, 2.0], [7.0, 4.0, 
6.0]]], [[[6.0, 6.0, 3.0], [7.0, 6.0, 6.0], [2.0, 6.0, 8.0]], [[4.0, 8.0, 8.0], [8.0, 3.0, 8.0], [8.0, 5.0, 3.0]], [[5.0, 7.0, 
4.0], [8.0, 6.0, 5.0], [1.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 7.0], [7.0, 2.0, 1.0], [6.0, 7.0, 2.0]], 
[[2.0, 7.0, 1.0], [4.0, 6.0, 1.0], [1.0, 4.0, 5.0]], [[8.0, 4.0, 3.0], [2.0, 4.0, 5.0], [2.0, 3.0, 3.0]]], [[[8.0, 6.0, 3.0], 
[2.0, 6.0, 3.0], [4.0, 2.0, 5.0]], [[3.0, 1.0, 1.0], [8.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[4.0, 5.0, 4.0], [2.0, 6.0, 2.0], 
[2.0, 6.0, 1.0]]], [[[3.0, 2.0, 4.0], [3.0, 8.0, 6.0], [3.0, 6.0, 5.0]], [[3.0, 6.0, 7.0], [6.0, 8.0, 3.0], [5.0, 7.0, 2.0]], 
[[8.0, 1.0, 3.0], [8.0, 6.0, 2.0], [6.0, 4.0, 6.0]]], [[[6.0, 5.0, 5.0], [7.0, 6.0, 6.0], [4.0, 3.0, 6.0]], [[7.0, 3.0, 6.0], 
[8.0, 3.0, 8.0], [7.0, 2.0, 3.0]], [[3.0, 4.0, 5.0], [8.0, 6.0, 5.0], [4.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LCRN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 8.0], [4.0, 1.0, 5.0], [7.0, 3.0, 3.0]], [[1.0, 6.0, 1.0], [5.0, 4.0, 8.0], [2.0, 8.0, 
5.0]], [[7.0, 3.0, 6.0], [2.0, 7.0, 3.0], [8.0, 8.0, 8.0]]], [[[8.0, 4.0, 5.0], [2.0, 3.0, 2.0], [3.0, 5.0, 8.0]], [[1.0, 6.0, 
4.0], [8.0, 6.0, 5.0], [8.0, 7.0, 7.0]], [[6.0, 2.0, 2.0], [5.0, 1.0, 8.0], [2.0, 3.0, 2.0]]], [[[6.0, 2.0, 6.0], [2.0, 5.0, 
3.0], [3.0, 3.0, 8.0]], [[7.0, 7.0, 1.0], [8.0, 1.0, 7.0], [4.0, 5.0, 6.0]], [[5.0, 1.0, 3.0], [6.0, 1.0, 7.0], [2.0, 2.0, 
2.0]]], [[[5.0, 5.0, 8.0], [6.0, 3.0, 8.0], [2.0, 1.0, 1.0]], [[4.0, 5.0, 2.0], [8.0, 4.0, 3.0], [2.0, 6.0, 3.0]], [[7.0, 3.0, 
3.0], [7.0, 4.0, 7.0], [2.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 8.0], [4.0, 7.0, 3.0], [4.0, 2.0, 3.0]], [[1.0, 7.0, 7.0], [3.0, 1.0, 
4.0], [8.0, 7.0, 4.0]], [[6.0, 1.0, 5.0], [3.0, 7.0, 6.0], [4.0, 1.0, 6.0]]], [[[7.0, 1.0, 7.0], [1.0, 8.0, 8.0], [2.0, 7.0, 
3.0]], [[3.0, 6.0, 3.0], [6.0, 3.0, 3.0], [3.0, 7.0, 3.0]], [[5.0, 1.0, 8.0], [3.0, 8.0, 3.0], [1.0, 7.0, 6.0]]], [[[8.0, 4.0, 
3.0], [5.0, 2.0, 6.0], [1.0, 7.0, 7.0]], [[8.0, 3.0, 4.0], [6.0, 1.0, 3.0], [6.0, 3.0, 8.0]], [[7.0, 3.0, 6.0], [5.0, 4.0, 
7.0], [5.0, 6.0, 8.0]]], [[[5.0, 8.0, 2.0], [1.0, 1.0, 3.0], [3.0, 3.0, 8.0]], [[3.0, 1.0, 7.0], [4.0, 2.0, 5.0], [2.0, 7.0, 
5.0]], [[5.0, 5.0, 6.0], [3.0, 6.0, 4.0], [2.0, 7.0, 1.0]]]]))
      val=Data(numpy.array([[[[2.0, 2.0, 6.0]], [[2.0, 8.0, 4.0]], [[8.0, 4.0, 5.0]]], [[[5.0, 2.0, 4.0]], [[2.0, 3.0, 8.0]], 
[[8.0, 8.0, 3.0]]]]),self.functionspace)
      arg[2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 8.0], [4.0, 1.0, 5.0], [7.0, 3.0, 3.0]], [[1.0, 6.0, 1.0], [5.0, 4.0, 8.0], [2.0, 
8.0, 5.0]], [[7.0, 3.0, 6.0], [2.0, 7.0, 3.0], [8.0, 8.0, 8.0]]], [[[8.0, 4.0, 5.0], [2.0, 3.0, 2.0], [3.0, 5.0, 8.0]], [[1.0, 
6.0, 4.0], [8.0, 6.0, 5.0], [8.0, 7.0, 7.0]], [[6.0, 2.0, 2.0], [5.0, 1.0, 8.0], [2.0, 3.0, 2.0]]], [[[6.0, 2.0, 6.0], [2.0, 
2.0, 6.0], [3.0, 3.0, 8.0]], [[7.0, 7.0, 1.0], [2.0, 8.0, 4.0], [4.0, 5.0, 6.0]], [[5.0, 1.0, 3.0], [8.0, 4.0, 5.0], [2.0, 2.0, 
2.0]]], [[[5.0, 5.0, 8.0], [5.0, 2.0, 4.0], [2.0, 1.0, 1.0]], [[4.0, 5.0, 2.0], [2.0, 3.0, 8.0], [2.0, 6.0, 3.0]], [[7.0, 3.0, 
3.0], [8.0, 8.0, 3.0], [2.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 8.0], [4.0, 7.0, 3.0], [4.0, 2.0, 3.0]], 
[[1.0, 7.0, 7.0], [3.0, 1.0, 4.0], [8.0, 7.0, 4.0]], [[6.0, 1.0, 5.0], [3.0, 7.0, 6.0], [4.0, 1.0, 6.0]]], [[[7.0, 1.0, 7.0], 
[1.0, 8.0, 8.0], [2.0, 7.0, 3.0]], [[3.0, 6.0, 3.0], [6.0, 3.0, 3.0], [3.0, 7.0, 3.0]], [[5.0, 1.0, 8.0], [3.0, 8.0, 3.0], 
[1.0, 7.0, 6.0]]], [[[8.0, 4.0, 3.0], [2.0, 2.0, 6.0], [1.0, 7.0, 7.0]], [[8.0, 3.0, 4.0], [2.0, 8.0, 4.0], [6.0, 3.0, 8.0]], 
[[7.0, 3.0, 6.0], [8.0, 4.0, 5.0], [5.0, 6.0, 8.0]]], [[[5.0, 8.0, 2.0], [5.0, 2.0, 4.0], [3.0, 3.0, 8.0]], [[3.0, 1.0, 7.0], 
[2.0, 3.0, 8.0], [2.0, 7.0, 5.0]], [[5.0, 5.0, 6.0], [8.0, 8.0, 3.0], [2.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LCRN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 3.0], [8.0, 7.0, 3.0], [3.0, 3.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 7.0, 4.0], [3.0, 1.0, 
3.0]], [[5.0, 5.0, 5.0], [6.0, 6.0, 3.0], [8.0, 1.0, 6.0]]], [[[6.0, 8.0, 2.0], [7.0, 8.0, 1.0], [3.0, 5.0, 3.0]], [[8.0, 1.0, 
4.0], [4.0, 3.0, 5.0], [3.0, 4.0, 3.0]], [[5.0, 4.0, 4.0], [4.0, 7.0, 5.0], [1.0, 3.0, 6.0]]], [[[8.0, 6.0, 5.0], [8.0, 3.0, 
5.0], [7.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [7.0, 7.0, 3.0], [4.0, 5.0, 5.0]], [[4.0, 1.0, 5.0], [3.0, 5.0, 8.0], [8.0, 2.0, 
1.0]]], [[[8.0, 7.0, 3.0], [6.0, 3.0, 2.0], [5.0, 5.0, 3.0]], [[1.0, 6.0, 2.0], [2.0, 3.0, 3.0], [6.0, 1.0, 8.0]], [[4.0, 7.0, 
3.0], [6.0, 7.0, 1.0], [2.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 1.0], [6.0, 8.0, 3.0], [6.0, 4.0, 7.0]], [[1.0, 2.0, 2.0], [4.0, 4.0, 
5.0], [8.0, 1.0, 7.0]], [[2.0, 1.0, 2.0], [8.0, 2.0, 4.0], [7.0, 5.0, 3.0]]], [[[4.0, 6.0, 1.0], [6.0, 4.0, 7.0], [7.0, 2.0, 
4.0]], [[3.0, 8.0, 4.0], [2.0, 5.0, 7.0], [3.0, 5.0, 8.0]], [[2.0, 5.0, 2.0], [6.0, 5.0, 3.0], [3.0, 6.0, 6.0]]], [[[8.0, 3.0, 
1.0], [8.0, 6.0, 2.0], [3.0, 6.0, 6.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 3.0], [4.0, 8.0, 6.0]], [[8.0, 4.0, 3.0], [5.0, 7.0, 
6.0], [6.0, 2.0, 3.0]]], [[[7.0, 7.0, 6.0], [6.0, 1.0, 5.0], [7.0, 5.0, 4.0]], [[5.0, 8.0, 2.0], [1.0, 3.0, 7.0], [6.0, 6.0, 
3.0]], [[8.0, 4.0, 1.0], [6.0, 5.0, 1.0], [8.0, 6.0, 6.0]]]]))
      val=Data(numpy.array([[[[8.0, 1.0, 1.0]], [[2.0, 2.0, 5.0]], [[6.0, 2.0, 4.0]]], [[[4.0, 2.0, 8.0]], [[2.0, 8.0, 4.0]], 
[[3.0, 8.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 7.0]], [[2.0, 7.0, 5.0]], [[3.0, 7.0, 8.0]]], [[[5.0, 4.0, 3.0]], [[5.0, 
2.0, 1.0]], [[4.0, 5.0, 7.0]]]]))
      arg[2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [8.0, 7.0, 3.0], [3.0, 3.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 7.0, 4.0], [3.0, 
1.0, 3.0]], [[5.0, 5.0, 5.0], [6.0, 6.0, 3.0], [8.0, 1.0, 6.0]]], [[[6.0, 8.0, 2.0], [7.0, 8.0, 1.0], [3.0, 5.0, 3.0]], [[8.0, 
1.0, 4.0], [4.0, 3.0, 5.0], [3.0, 4.0, 3.0]], [[5.0, 4.0, 4.0], [4.0, 7.0, 5.0], [1.0, 3.0, 6.0]]], [[[8.0, 6.0, 5.0], [8.0, 
1.0, 1.0], [7.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [2.0, 2.0, 5.0], [4.0, 5.0, 5.0]], [[4.0, 1.0, 5.0], [6.0, 2.0, 4.0], [8.0, 2.0, 
1.0]]], [[[8.0, 7.0, 3.0], [4.0, 2.0, 8.0], [5.0, 5.0, 3.0]], [[1.0, 6.0, 2.0], [2.0, 8.0, 4.0], [6.0, 1.0, 8.0]], [[4.0, 7.0, 
3.0], [3.0, 8.0, 5.0], [2.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 1.0], [6.0, 8.0, 3.0], [6.0, 4.0, 7.0]], 
[[1.0, 2.0, 2.0], [4.0, 4.0, 5.0], [8.0, 1.0, 7.0]], [[2.0, 1.0, 2.0], [8.0, 2.0, 4.0], [7.0, 5.0, 3.0]]], [[[4.0, 6.0, 1.0], 
[6.0, 4.0, 7.0], [7.0, 2.0, 4.0]], [[3.0, 8.0, 4.0], [2.0, 5.0, 7.0], [3.0, 5.0, 8.0]], [[2.0, 5.0, 2.0], [6.0, 5.0, 3.0], 
[3.0, 6.0, 6.0]]], [[[8.0, 3.0, 1.0], [1.0, 3.0, 7.0], [3.0, 6.0, 6.0]], [[2.0, 6.0, 7.0], [2.0, 7.0, 5.0], [4.0, 8.0, 6.0]], 
[[8.0, 4.0, 3.0], [3.0, 7.0, 8.0], [6.0, 2.0, 3.0]]], [[[7.0, 7.0, 6.0], [5.0, 4.0, 3.0], [7.0, 5.0, 4.0]], [[5.0, 8.0, 2.0], 
[5.0, 2.0, 1.0], [6.0, 6.0, 3.0]], [[8.0, 4.0, 1.0], [4.0, 5.0, 7.0], [8.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LCRN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 8.0], [1.0, 8.0, 5.0], [8.0, 2.0, 2.0]], [[3.0, 3.0, 2.0], [3.0, 7.0, 8.0], [7.0, 2.0, 
7.0]], [[4.0, 2.0, 1.0], [4.0, 4.0, 1.0], [2.0, 1.0, 5.0]]], [[[8.0, 6.0, 6.0], [3.0, 3.0, 6.0], [7.0, 4.0, 1.0]], [[5.0, 1.0, 
1.0], [2.0, 2.0, 8.0], [5.0, 6.0, 6.0]], [[2.0, 5.0, 1.0], [4.0, 4.0, 7.0], [1.0, 2.0, 1.0]]], [[[1.0, 3.0, 3.0], [7.0, 6.0, 
8.0], [6.0, 8.0, 6.0]], [[8.0, 5.0, 3.0], [6.0, 6.0, 2.0], [8.0, 8.0, 8.0]], [[3.0, 3.0, 7.0], [4.0, 6.0, 1.0], [8.0, 4.0, 
4.0]]], [[[3.0, 8.0, 6.0], [8.0, 8.0, 8.0], [8.0, 2.0, 6.0]], [[4.0, 5.0, 7.0], [6.0, 5.0, 3.0], [4.0, 3.0, 2.0]], [[2.0, 6.0, 
1.0], [1.0, 1.0, 1.0], [7.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 8.0], [4.0, 4.0, 3.0], [6.0, 2.0, 5.0]], [[6.0, 7.0, 3.0], [4.0, 5.0, 
7.0], [5.0, 7.0, 3.0]], [[6.0, 1.0, 4.0], [2.0, 7.0, 8.0], [5.0, 2.0, 4.0]]], [[[6.0, 1.0, 8.0], [2.0, 4.0, 8.0], [3.0, 4.0, 
3.0]], [[4.0, 3.0, 5.0], [1.0, 3.0, 2.0], [7.0, 7.0, 2.0]], [[2.0, 8.0, 4.0], [8.0, 1.0, 3.0], [5.0, 6.0, 5.0]]], [[[3.0, 5.0, 
3.0], [8.0, 5.0, 3.0], [2.0, 6.0, 8.0]], [[3.0, 8.0, 8.0], [5.0, 4.0, 5.0], [5.0, 6.0, 4.0]], [[7.0, 6.0, 6.0], [4.0, 6.0, 
8.0], [6.0, 3.0, 8.0]]], [[[8.0, 8.0, 7.0], [8.0, 2.0, 1.0], [4.0, 2.0, 1.0]], [[4.0, 6.0, 1.0], [3.0, 2.0, 5.0], [4.0, 4.0, 
7.0]], [[7.0, 5.0, 6.0], [5.0, 2.0, 5.0], [8.0, 4.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0, 1.0, 4.0]], [[8.0, 7.0, 5.0]], [[2.0, 7.0, 2.0]]], [[[2.0, 5.0, 1.0]], [[6.0, 4.0, 
5.0]], [[8.0, 1.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 7.0, 1.0]], [[1.0, 5.0, 2.0]], [[6.0, 6.0, 8.0]]], [[[6.0, 6.0, 
3.0]], [[5.0, 7.0, 7.0]], [[5.0, 1.0, 3.0]]]])
      arg[2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 8.0], [1.0, 8.0, 5.0], [8.0, 2.0, 2.0]], [[3.0, 3.0, 2.0], [3.0, 7.0, 8.0], [7.0, 
2.0, 7.0]], [[4.0, 2.0, 1.0], [4.0, 4.0, 1.0], [2.0, 1.0, 5.0]]], [[[8.0, 6.0, 6.0], [3.0, 3.0, 6.0], [7.0, 4.0, 1.0]], [[5.0, 
1.0, 1.0], [2.0, 2.0, 8.0], [5.0, 6.0, 6.0]], [[2.0, 5.0, 1.0], [4.0, 4.0, 7.0], [1.0, 2.0, 1.0]]], [[[1.0, 3.0, 3.0], [4.0, 
1.0, 4.0], [6.0, 8.0, 6.0]], [[8.0, 5.0, 3.0], [8.0, 7.0, 5.0], [8.0, 8.0, 8.0]], [[3.0, 3.0, 7.0], [2.0, 7.0, 2.0], [8.0, 4.0, 
4.0]]], [[[3.0, 8.0, 6.0], [2.0, 5.0, 1.0], [8.0, 2.0, 6.0]], [[4.0, 5.0, 7.0], [6.0, 4.0, 5.0], [4.0, 3.0, 2.0]], [[2.0, 6.0, 
1.0], [8.0, 1.0, 3.0], [7.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 8.0], [4.0, 4.0, 3.0], [6.0, 2.0, 5.0]], 
[[6.0, 7.0, 3.0], [4.0, 5.0, 7.0], [5.0, 7.0, 3.0]], [[6.0, 1.0, 4.0], [2.0, 7.0, 8.0], [5.0, 2.0, 4.0]]], [[[6.0, 1.0, 8.0], 
[2.0, 4.0, 8.0], [3.0, 4.0, 3.0]], [[4.0, 3.0, 5.0], [1.0, 3.0, 2.0], [7.0, 7.0, 2.0]], [[2.0, 8.0, 4.0], [8.0, 1.0, 3.0], 
[5.0, 6.0, 5.0]]], [[[3.0, 5.0, 3.0], [4.0, 7.0, 1.0], [2.0, 6.0, 8.0]], [[3.0, 8.0, 8.0], [1.0, 5.0, 2.0], [5.0, 6.0, 4.0]], 
[[7.0, 6.0, 6.0], [6.0, 6.0, 8.0], [6.0, 3.0, 8.0]]], [[[8.0, 8.0, 7.0], [6.0, 6.0, 3.0], [4.0, 2.0, 1.0]], [[4.0, 6.0, 1.0], 
[5.0, 7.0, 7.0], [4.0, 4.0, 7.0]], [[7.0, 5.0, 6.0], [5.0, 1.0, 3.0], [8.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UCCC(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 4.0], [7.0, 6.0, 7.0], [7.0, 8.0, 6.0]], [[8.0, 6.0, 1.0], [8.0, 1.0, 3.0], [4.0, 3.0, 
2.0]], [[4.0, 1.0, 1.0], [3.0, 8.0, 6.0], [8.0, 2.0, 4.0]]], [[[5.0, 1.0, 5.0], [6.0, 7.0, 8.0], [1.0, 2.0, 1.0]], [[6.0, 3.0, 
4.0], [2.0, 1.0, 6.0], [5.0, 3.0, 8.0]], [[8.0, 7.0, 3.0], [8.0, 2.0, 8.0], [4.0, 3.0, 3.0]]], [[[6.0, 7.0, 3.0], [3.0, 1.0, 
6.0], [4.0, 4.0, 4.0]], [[6.0, 2.0, 8.0], [7.0, 4.0, 1.0], [1.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [5.0, 3.0, 8.0], [7.0, 8.0, 
4.0]]], [[[6.0, 1.0, 3.0], [5.0, 5.0, 8.0], [7.0, 8.0, 5.0]], [[5.0, 4.0, 8.0], [5.0, 4.0, 8.0], [5.0, 4.0, 4.0]], [[2.0, 7.0, 
3.0], [4.0, 7.0, 4.0], [2.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 3.0], [1.0, 2.0, 1.0], [7.0, 4.0, 6.0]], [[6.0, 8.0, 4.0], [4.0, 1.0, 
4.0], [1.0, 1.0, 3.0]], [[8.0, 6.0, 4.0], [7.0, 5.0, 5.0], [5.0, 7.0, 7.0]]], [[[4.0, 6.0, 7.0], [4.0, 3.0, 4.0], [4.0, 3.0, 
2.0]], [[5.0, 2.0, 8.0], [8.0, 7.0, 6.0], [3.0, 7.0, 5.0]], [[1.0, 5.0, 6.0], [2.0, 8.0, 3.0], [7.0, 4.0, 7.0]]], [[[2.0, 8.0, 
4.0], [1.0, 1.0, 5.0], [2.0, 6.0, 7.0]], [[4.0, 2.0, 2.0], [1.0, 4.0, 7.0], [2.0, 1.0, 7.0]], [[2.0, 7.0, 8.0], [4.0, 6.0, 
4.0], [3.0, 6.0, 5.0]]], [[[2.0, 8.0, 7.0], [6.0, 3.0, 6.0], [8.0, 7.0, 1.0]], [[5.0, 7.0, 2.0], [7.0, 5.0, 5.0], [4.0, 2.0, 
8.0]], [[7.0, 2.0, 7.0], [6.0, 8.0, 6.0], [2.0, 3.0, 3.0]]]]))
      val=numpy.array([[[[7.0, 1.0, 1.0], [5.0, 6.0, 1.0], [2.0, 5.0, 6.0]], [[3.0, 7.0, 4.0], [4.0, 8.0, 4.0], [7.0, 8.0, 
5.0]], [[6.0, 8.0, 4.0], [4.0, 5.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 8.0, 6.0], [1.0, 3.0, 2.0], [5.0, 2.0, 1.0]], [[2.0, 5.0, 
5.0], [8.0, 2.0, 7.0], [8.0, 8.0, 8.0]], [[2.0, 3.0, 5.0], [6.0, 4.0, 1.0], [6.0, 3.0, 4.0]]], [[[7.0, 7.0, 6.0], [1.0, 7.0, 
8.0], [6.0, 2.0, 4.0]], [[5.0, 6.0, 7.0], [7.0, 7.0, 1.0], [4.0, 6.0, 5.0]], [[6.0, 8.0, 8.0], [6.0, 8.0, 8.0], [2.0, 8.0, 
8.0]]]])
      arg[:3,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 1.0], [5.0, 6.0, 1.0], [2.0, 5.0, 6.0]], [[3.0, 7.0, 4.0], [4.0, 8.0, 4.0], [7.0, 
8.0, 5.0]], [[6.0, 8.0, 4.0], [4.0, 5.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 8.0, 6.0], [1.0, 3.0, 2.0], [5.0, 2.0, 1.0]], [[2.0, 
5.0, 5.0], [8.0, 2.0, 7.0], [8.0, 8.0, 8.0]], [[2.0, 3.0, 5.0], [6.0, 4.0, 1.0], [6.0, 3.0, 4.0]]], [[[7.0, 7.0, 6.0], [1.0, 
7.0, 8.0], [6.0, 2.0, 4.0]], [[5.0, 6.0, 7.0], [7.0, 7.0, 1.0], [4.0, 6.0, 5.0]], [[6.0, 8.0, 8.0], [6.0, 8.0, 8.0], [2.0, 8.0, 
8.0]]], [[[6.0, 1.0, 3.0], [5.0, 5.0, 8.0], [7.0, 8.0, 5.0]], [[5.0, 4.0, 8.0], [5.0, 4.0, 8.0], [5.0, 4.0, 4.0]], [[2.0, 7.0, 
3.0], [4.0, 7.0, 4.0], [2.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 1.0], [5.0, 6.0, 1.0], [2.0, 5.0, 6.0]], 
[[3.0, 7.0, 4.0], [4.0, 8.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 8.0, 4.0], [4.0, 5.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 8.0, 6.0], 
[1.0, 3.0, 2.0], [5.0, 2.0, 1.0]], [[2.0, 5.0, 5.0], [8.0, 2.0, 7.0], [8.0, 8.0, 8.0]], [[2.0, 3.0, 5.0], [6.0, 4.0, 1.0], 
[6.0, 3.0, 4.0]]], [[[7.0, 7.0, 6.0], [1.0, 7.0, 8.0], [6.0, 2.0, 4.0]], [[5.0, 6.0, 7.0], [7.0, 7.0, 1.0], [4.0, 6.0, 5.0]], 
[[6.0, 8.0, 8.0], [6.0, 8.0, 8.0], [2.0, 8.0, 8.0]]], [[[2.0, 8.0, 7.0], [6.0, 3.0, 6.0], [8.0, 7.0, 1.0]], [[5.0, 7.0, 2.0], 
[7.0, 5.0, 5.0], [4.0, 2.0, 8.0]], [[7.0, 2.0, 7.0], [6.0, 8.0, 6.0], [2.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UCCC(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 1.0], [5.0, 2.0, 8.0], [3.0, 4.0, 8.0]], [[8.0, 6.0, 7.0], [1.0, 5.0, 7.0], [3.0, 6.0, 
3.0]], [[4.0, 7.0, 4.0], [7.0, 1.0, 4.0], [5.0, 7.0, 4.0]]], [[[6.0, 7.0, 3.0], [2.0, 4.0, 3.0], [4.0, 1.0, 3.0]], [[8.0, 1.0, 
5.0], [1.0, 8.0, 5.0], [8.0, 6.0, 1.0]], [[1.0, 8.0, 5.0], [3.0, 7.0, 4.0], [8.0, 5.0, 7.0]]], [[[8.0, 3.0, 1.0], [4.0, 5.0, 
6.0], [2.0, 4.0, 2.0]], [[8.0, 4.0, 4.0], [6.0, 8.0, 7.0], [3.0, 8.0, 6.0]], [[4.0, 8.0, 4.0], [7.0, 7.0, 4.0], [4.0, 2.0, 
5.0]]], [[[2.0, 3.0, 3.0], [2.0, 3.0, 2.0], [7.0, 3.0, 6.0]], [[1.0, 2.0, 1.0], [6.0, 2.0, 8.0], [3.0, 8.0, 3.0]], [[5.0, 1.0, 
6.0], [6.0, 8.0, 8.0], [5.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 2.0], [7.0, 7.0, 6.0], [3.0, 4.0, 1.0]], [[8.0, 4.0, 2.0], [1.0, 5.0, 
5.0], [2.0, 4.0, 6.0]], [[5.0, 5.0, 1.0], [3.0, 7.0, 7.0], [7.0, 3.0, 4.0]]], [[[7.0, 2.0, 4.0], [6.0, 1.0, 6.0], [3.0, 3.0, 
5.0]], [[7.0, 8.0, 5.0], [7.0, 6.0, 8.0], [3.0, 4.0, 3.0]], [[6.0, 1.0, 2.0], [7.0, 1.0, 2.0], [1.0, 1.0, 5.0]]], [[[1.0, 3.0, 
6.0], [7.0, 5.0, 7.0], [8.0, 1.0, 4.0]], [[8.0, 3.0, 4.0], [1.0, 2.0, 3.0], [4.0, 2.0, 1.0]], [[1.0, 8.0, 8.0], [1.0, 5.0, 
3.0], [6.0, 8.0, 5.0]]], [[[3.0, 1.0, 6.0], [5.0, 6.0, 1.0], [2.0, 3.0, 8.0]], [[5.0, 7.0, 6.0], [6.0, 8.0, 6.0], [1.0, 1.0, 
8.0]], [[1.0, 1.0, 1.0], [4.0, 3.0, 6.0], [3.0, 7.0, 6.0]]]]))
      val=Data(numpy.array([[[[5.0, 5.0, 2.0], [3.0, 4.0, 7.0], [4.0, 6.0, 8.0]], [[4.0, 6.0, 8.0], [1.0, 7.0, 8.0], [8.0, 8.0, 
5.0]], [[6.0, 5.0, 1.0], [2.0, 2.0, 4.0], [8.0, 2.0, 6.0]]], [[[2.0, 8.0, 6.0], [6.0, 8.0, 5.0], [2.0, 3.0, 4.0]], [[7.0, 2.0, 
8.0], [2.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[1.0, 5.0, 7.0], [1.0, 2.0, 1.0], [1.0, 4.0, 7.0]]], [[[3.0, 4.0, 2.0], [8.0, 3.0, 
2.0], [3.0, 3.0, 1.0]], [[5.0, 8.0, 1.0], [4.0, 3.0, 2.0], [8.0, 2.0, 4.0]], [[6.0, 5.0, 2.0], [5.0, 5.0, 4.0], [6.0, 8.0, 
4.0]]]]),self.functionspace)
      arg[:3,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 2.0], [3.0, 4.0, 7.0], [4.0, 6.0, 8.0]], [[4.0, 6.0, 8.0], [1.0, 7.0, 8.0], [8.0, 
8.0, 5.0]], [[6.0, 5.0, 1.0], [2.0, 2.0, 4.0], [8.0, 2.0, 6.0]]], [[[2.0, 8.0, 6.0], [6.0, 8.0, 5.0], [2.0, 3.0, 4.0]], [[7.0, 
2.0, 8.0], [2.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[1.0, 5.0, 7.0], [1.0, 2.0, 1.0], [1.0, 4.0, 7.0]]], [[[3.0, 4.0, 2.0], [8.0, 
3.0, 2.0], [3.0, 3.0, 1.0]], [[5.0, 8.0, 1.0], [4.0, 3.0, 2.0], [8.0, 2.0, 4.0]], [[6.0, 5.0, 2.0], [5.0, 5.0, 4.0], [6.0, 8.0, 
4.0]]], [[[2.0, 3.0, 3.0], [2.0, 3.0, 2.0], [7.0, 3.0, 6.0]], [[1.0, 2.0, 1.0], [6.0, 2.0, 8.0], [3.0, 8.0, 3.0]], [[5.0, 1.0, 
6.0], [6.0, 8.0, 8.0], [5.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 2.0], [3.0, 4.0, 7.0], [4.0, 6.0, 8.0]], 
[[4.0, 6.0, 8.0], [1.0, 7.0, 8.0], [8.0, 8.0, 5.0]], [[6.0, 5.0, 1.0], [2.0, 2.0, 4.0], [8.0, 2.0, 6.0]]], [[[2.0, 8.0, 6.0], 
[6.0, 8.0, 5.0], [2.0, 3.0, 4.0]], [[7.0, 2.0, 8.0], [2.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[1.0, 5.0, 7.0], [1.0, 2.0, 1.0], 
[1.0, 4.0, 7.0]]], [[[3.0, 4.0, 2.0], [8.0, 3.0, 2.0], [3.0, 3.0, 1.0]], [[5.0, 8.0, 1.0], [4.0, 3.0, 2.0], [8.0, 2.0, 4.0]], 
[[6.0, 5.0, 2.0], [5.0, 5.0, 4.0], [6.0, 8.0, 4.0]]], [[[3.0, 1.0, 6.0], [5.0, 6.0, 1.0], [2.0, 3.0, 8.0]], [[5.0, 7.0, 6.0], 
[6.0, 8.0, 6.0], [1.0, 1.0, 8.0]], [[1.0, 1.0, 1.0], [4.0, 3.0, 6.0], [3.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UCCC(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 2.0], [8.0, 3.0, 7.0], [3.0, 6.0, 7.0]], [[3.0, 1.0, 1.0], [8.0, 2.0, 3.0], [6.0, 8.0, 
6.0]], [[7.0, 5.0, 7.0], [2.0, 4.0, 5.0], [5.0, 8.0, 6.0]]], [[[1.0, 6.0, 7.0], [2.0, 2.0, 8.0], [3.0, 4.0, 8.0]], [[7.0, 5.0, 
3.0], [6.0, 6.0, 7.0], [3.0, 4.0, 7.0]], [[6.0, 3.0, 4.0], [4.0, 5.0, 7.0], [8.0, 5.0, 8.0]]], [[[5.0, 4.0, 3.0], [4.0, 4.0, 
5.0], [8.0, 6.0, 6.0]], [[3.0, 6.0, 1.0], [1.0, 1.0, 6.0], [5.0, 4.0, 5.0]], [[7.0, 8.0, 7.0], [5.0, 1.0, 1.0], [6.0, 1.0, 
1.0]]], [[[7.0, 4.0, 1.0], [4.0, 2.0, 5.0], [4.0, 5.0, 7.0]], [[3.0, 1.0, 5.0], [6.0, 8.0, 5.0], [7.0, 2.0, 6.0]], [[5.0, 6.0, 
4.0], [7.0, 8.0, 4.0], [5.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 8.0], [7.0, 4.0, 4.0], [4.0, 2.0, 6.0]], [[2.0, 3.0, 1.0], [3.0, 2.0, 
1.0], [6.0, 2.0, 4.0]], [[6.0, 5.0, 7.0], [8.0, 8.0, 5.0], [2.0, 8.0, 4.0]]], [[[2.0, 7.0, 7.0], [5.0, 6.0, 8.0], [8.0, 3.0, 
4.0]], [[6.0, 6.0, 7.0], [6.0, 3.0, 7.0], [3.0, 5.0, 1.0]], [[4.0, 3.0, 7.0], [2.0, 6.0, 1.0], [8.0, 8.0, 1.0]]], [[[4.0, 7.0, 
6.0], [4.0, 2.0, 8.0], [6.0, 3.0, 3.0]], [[3.0, 5.0, 8.0], [2.0, 5.0, 4.0], [7.0, 2.0, 5.0]], [[4.0, 2.0, 1.0], [4.0, 2.0, 
1.0], [2.0, 1.0, 8.0]]], [[[1.0, 3.0, 3.0], [8.0, 7.0, 5.0], [2.0, 2.0, 3.0]], [[7.0, 6.0, 1.0], [3.0, 5.0, 3.0], [5.0, 3.0, 
7.0]], [[2.0, 8.0, 7.0], [8.0, 8.0, 6.0], [1.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[[2.0, 2.0, 1.0], [4.0, 5.0, 2.0], [6.0, 2.0, 7.0]], [[1.0, 8.0, 7.0], [3.0, 7.0, 8.0], [5.0, 5.0, 
1.0]], [[1.0, 4.0, 1.0], [6.0, 6.0, 1.0], [3.0, 7.0, 6.0]]], [[[5.0, 2.0, 5.0], [4.0, 1.0, 3.0], [6.0, 5.0, 5.0]], [[5.0, 5.0, 
3.0], [6.0, 4.0, 3.0], [2.0, 3.0, 8.0]], [[2.0, 5.0, 4.0], [8.0, 4.0, 2.0], [3.0, 4.0, 7.0]]], [[[7.0, 4.0, 1.0], [5.0, 7.0, 
7.0], [4.0, 5.0, 4.0]], [[8.0, 6.0, 5.0], [1.0, 2.0, 3.0], [4.0, 5.0, 1.0]], [[1.0, 7.0, 3.0], [4.0, 4.0, 8.0], [7.0, 5.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 4.0], [4.0, 6.0, 4.0], [4.0, 3.0, 6.0]], [[7.0, 4.0, 7.0], [2.0, 6.0, 
3.0], [2.0, 5.0, 5.0]], [[3.0, 4.0, 7.0], [1.0, 2.0, 5.0], [3.0, 4.0, 2.0]]], [[[7.0, 8.0, 6.0], [2.0, 7.0, 2.0], [8.0, 5.0, 
8.0]], [[3.0, 2.0, 1.0], [2.0, 6.0, 3.0], [1.0, 5.0, 7.0]], [[6.0, 3.0, 7.0], [5.0, 6.0, 3.0], [3.0, 3.0, 6.0]]], [[[7.0, 7.0, 
3.0], [6.0, 3.0, 1.0], [7.0, 5.0, 4.0]], [[4.0, 7.0, 5.0], [3.0, 2.0, 7.0], [3.0, 2.0, 6.0]], [[8.0, 3.0, 7.0], [2.0, 4.0, 
5.0], [6.0, 1.0, 6.0]]]]))
      arg[:3,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 1.0], [4.0, 5.0, 2.0], [6.0, 2.0, 7.0]], [[1.0, 8.0, 7.0], [3.0, 7.0, 8.0], [5.0, 
5.0, 1.0]], [[1.0, 4.0, 1.0], [6.0, 6.0, 1.0], [3.0, 7.0, 6.0]]], [[[5.0, 2.0, 5.0], [4.0, 1.0, 3.0], [6.0, 5.0, 5.0]], [[5.0, 
5.0, 3.0], [6.0, 4.0, 3.0], [2.0, 3.0, 8.0]], [[2.0, 5.0, 4.0], [8.0, 4.0, 2.0], [3.0, 4.0, 7.0]]], [[[7.0, 4.0, 1.0], [5.0, 
7.0, 7.0], [4.0, 5.0, 4.0]], [[8.0, 6.0, 5.0], [1.0, 2.0, 3.0], [4.0, 5.0, 1.0]], [[1.0, 7.0, 3.0], [4.0, 4.0, 8.0], [7.0, 5.0, 
2.0]]], [[[7.0, 4.0, 1.0], [4.0, 2.0, 5.0], [4.0, 5.0, 7.0]], [[3.0, 1.0, 5.0], [6.0, 8.0, 5.0], [7.0, 2.0, 6.0]], [[5.0, 6.0, 
4.0], [7.0, 8.0, 4.0], [5.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 4.0], [4.0, 6.0, 4.0], [4.0, 3.0, 6.0]], 
[[7.0, 4.0, 7.0], [2.0, 6.0, 3.0], [2.0, 5.0, 5.0]], [[3.0, 4.0, 7.0], [1.0, 2.0, 5.0], [3.0, 4.0, 2.0]]], [[[7.0, 8.0, 6.0], 
[2.0, 7.0, 2.0], [8.0, 5.0, 8.0]], [[3.0, 2.0, 1.0], [2.0, 6.0, 3.0], [1.0, 5.0, 7.0]], [[6.0, 3.0, 7.0], [5.0, 6.0, 3.0], 
[3.0, 3.0, 6.0]]], [[[7.0, 7.0, 3.0], [6.0, 3.0, 1.0], [7.0, 5.0, 4.0]], [[4.0, 7.0, 5.0], [3.0, 2.0, 7.0], [3.0, 2.0, 6.0]], 
[[8.0, 3.0, 7.0], [2.0, 4.0, 5.0], [6.0, 1.0, 6.0]]], [[[1.0, 3.0, 3.0], [8.0, 7.0, 5.0], [2.0, 2.0, 3.0]], [[7.0, 6.0, 1.0], 
[3.0, 5.0, 3.0], [5.0, 3.0, 7.0]], [[2.0, 8.0, 7.0], [8.0, 8.0, 6.0], [1.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UCCC(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 2.0], [7.0, 7.0, 6.0], [3.0, 7.0, 2.0]], [[5.0, 4.0, 6.0], [8.0, 6.0, 8.0], [1.0, 5.0, 
1.0]], [[7.0, 3.0, 1.0], [5.0, 7.0, 6.0], [5.0, 4.0, 5.0]]], [[[3.0, 7.0, 2.0], [3.0, 4.0, 7.0], [1.0, 3.0, 1.0]], [[5.0, 1.0, 
4.0], [8.0, 3.0, 6.0], [7.0, 4.0, 4.0]], [[8.0, 7.0, 5.0], [6.0, 2.0, 5.0], [5.0, 5.0, 4.0]]], [[[3.0, 7.0, 7.0], [7.0, 1.0, 
8.0], [5.0, 2.0, 5.0]], [[8.0, 8.0, 5.0], [4.0, 5.0, 7.0], [7.0, 1.0, 3.0]], [[5.0, 3.0, 1.0], [5.0, 3.0, 8.0], [8.0, 3.0, 
2.0]]], [[[6.0, 1.0, 1.0], [6.0, 6.0, 8.0], [8.0, 8.0, 3.0]], [[2.0, 8.0, 7.0], [8.0, 2.0, 3.0], [6.0, 4.0, 1.0]], [[7.0, 3.0, 
2.0], [2.0, 3.0, 6.0], [7.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 8.0], [4.0, 4.0, 4.0], [3.0, 3.0, 8.0]], [[2.0, 5.0, 4.0], [5.0, 7.0, 
8.0], [4.0, 2.0, 2.0]], [[6.0, 8.0, 8.0], [4.0, 4.0, 7.0], [7.0, 3.0, 3.0]]], [[[4.0, 6.0, 6.0], [7.0, 3.0, 1.0], [6.0, 4.0, 
5.0]], [[2.0, 4.0, 6.0], [3.0, 8.0, 8.0], [4.0, 2.0, 2.0]], [[3.0, 4.0, 4.0], [7.0, 3.0, 7.0], [2.0, 5.0, 4.0]]], [[[5.0, 3.0, 
4.0], [7.0, 3.0, 6.0], [2.0, 8.0, 6.0]], [[4.0, 7.0, 5.0], [8.0, 2.0, 8.0], [3.0, 4.0, 2.0]], [[3.0, 8.0, 7.0], [2.0, 7.0, 
2.0], [6.0, 6.0, 5.0]]], [[[4.0, 1.0, 1.0], [2.0, 6.0, 8.0], [4.0, 6.0, 8.0]], [[7.0, 4.0, 8.0], [6.0, 7.0, 6.0], [8.0, 3.0, 
6.0]], [[3.0, 1.0, 5.0], [3.0, 4.0, 3.0], [7.0, 1.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 7.0, 1.0], [8.0, 8.0, 8.0], [5.0, 6.0, 7.0]], [[1.0, 2.0, 5.0], [5.0, 4.0, 6.0], [2.0, 
6.0, 6.0]], [[6.0, 1.0, 1.0], [1.0, 7.0, 1.0], [3.0, 2.0, 4.0]]], [[[4.0, 3.0, 6.0], [3.0, 7.0, 3.0], [2.0, 8.0, 7.0]], [[7.0, 
8.0, 4.0], [1.0, 3.0, 2.0], [3.0, 1.0, 4.0]], [[7.0, 5.0, 2.0], [3.0, 6.0, 4.0], [3.0, 4.0, 5.0]]], [[[3.0, 7.0, 6.0], [3.0, 
3.0, 5.0], [5.0, 5.0, 4.0]], [[8.0, 3.0, 2.0], [8.0, 4.0, 8.0], [3.0, 6.0, 5.0]], [[5.0, 7.0, 8.0], [1.0, 5.0, 8.0], [2.0, 7.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 4.0, 6.0], [3.0, 7.0, 6.0], [8.0, 6.0, 3.0]], [[2.0, 4.0, 2.0], [1.0, 2.0, 1.0], 
[5.0, 6.0, 2.0]], [[3.0, 1.0, 8.0], [5.0, 2.0, 7.0], [7.0, 3.0, 2.0]]], [[[7.0, 7.0, 5.0], [8.0, 3.0, 6.0], [8.0, 2.0, 6.0]], 
[[1.0, 4.0, 7.0], [4.0, 7.0, 1.0], [2.0, 8.0, 3.0]], [[1.0, 1.0, 7.0], [7.0, 8.0, 7.0], [7.0, 8.0, 4.0]]], [[[5.0, 5.0, 2.0], 
[1.0, 7.0, 4.0], [2.0, 1.0, 4.0]], [[5.0, 5.0, 8.0], [8.0, 3.0, 2.0], [5.0, 7.0, 6.0]], [[4.0, 2.0, 6.0], [1.0, 1.0, 4.0], 
[8.0, 1.0, 2.0]]]])
      arg[:3,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 1.0], [8.0, 8.0, 8.0], [5.0, 6.0, 7.0]], [[1.0, 2.0, 5.0], [5.0, 4.0, 6.0], [2.0, 
6.0, 6.0]], [[6.0, 1.0, 1.0], [1.0, 7.0, 1.0], [3.0, 2.0, 4.0]]], [[[4.0, 3.0, 6.0], [3.0, 7.0, 3.0], [2.0, 8.0, 7.0]], [[7.0, 
8.0, 4.0], [1.0, 3.0, 2.0], [3.0, 1.0, 4.0]], [[7.0, 5.0, 2.0], [3.0, 6.0, 4.0], [3.0, 4.0, 5.0]]], [[[3.0, 7.0, 6.0], [3.0, 
3.0, 5.0], [5.0, 5.0, 4.0]], [[8.0, 3.0, 2.0], [8.0, 4.0, 8.0], [3.0, 6.0, 5.0]], [[5.0, 7.0, 8.0], [1.0, 5.0, 8.0], [2.0, 7.0, 
6.0]]], [[[6.0, 1.0, 1.0], [6.0, 6.0, 8.0], [8.0, 8.0, 3.0]], [[2.0, 8.0, 7.0], [8.0, 2.0, 3.0], [6.0, 4.0, 1.0]], [[7.0, 3.0, 
2.0], [2.0, 3.0, 6.0], [7.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 6.0], [3.0, 7.0, 6.0], [8.0, 6.0, 3.0]], 
[[2.0, 4.0, 2.0], [1.0, 2.0, 1.0], [5.0, 6.0, 2.0]], [[3.0, 1.0, 8.0], [5.0, 2.0, 7.0], [7.0, 3.0, 2.0]]], [[[7.0, 7.0, 5.0], 
[8.0, 3.0, 6.0], [8.0, 2.0, 6.0]], [[1.0, 4.0, 7.0], [4.0, 7.0, 1.0], [2.0, 8.0, 3.0]], [[1.0, 1.0, 7.0], [7.0, 8.0, 7.0], 
[7.0, 8.0, 4.0]]], [[[5.0, 5.0, 2.0], [1.0, 7.0, 4.0], [2.0, 1.0, 4.0]], [[5.0, 5.0, 8.0], [8.0, 3.0, 2.0], [5.0, 7.0, 6.0]], 
[[4.0, 2.0, 6.0], [1.0, 1.0, 4.0], [8.0, 1.0, 2.0]]], [[[4.0, 1.0, 1.0], [2.0, 6.0, 8.0], [4.0, 6.0, 8.0]], [[7.0, 4.0, 8.0], 
[6.0, 7.0, 6.0], [8.0, 3.0, 6.0]], [[3.0, 1.0, 5.0], [3.0, 4.0, 3.0], [7.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULPN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 7.0], [2.0, 4.0, 4.0], [7.0, 7.0, 3.0]], [[6.0, 6.0, 7.0], [7.0, 7.0, 4.0], [3.0, 3.0, 
1.0]], [[1.0, 7.0, 6.0], [5.0, 4.0, 2.0], [1.0, 7.0, 5.0]]], [[[7.0, 8.0, 1.0], [4.0, 3.0, 7.0], [1.0, 4.0, 4.0]], [[1.0, 2.0, 
7.0], [7.0, 4.0, 4.0], [1.0, 2.0, 7.0]], [[4.0, 5.0, 8.0], [5.0, 7.0, 8.0], [8.0, 7.0, 8.0]]], [[[3.0, 8.0, 3.0], [8.0, 2.0, 
1.0], [1.0, 7.0, 2.0]], [[7.0, 7.0, 2.0], [4.0, 2.0, 8.0], [8.0, 2.0, 4.0]], [[5.0, 2.0, 3.0], [5.0, 2.0, 7.0], [2.0, 4.0, 
1.0]]], [[[7.0, 2.0, 6.0], [6.0, 2.0, 5.0], [4.0, 1.0, 1.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 5.0], [1.0, 2.0, 4.0]], [[7.0, 4.0, 
7.0], [6.0, 7.0, 2.0], [3.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 5.0], [8.0, 3.0, 7.0], [7.0, 2.0, 5.0]], [[8.0, 7.0, 8.0], [6.0, 8.0, 
5.0], [4.0, 8.0, 8.0]], [[8.0, 7.0, 8.0], [6.0, 6.0, 4.0], [1.0, 6.0, 3.0]]], [[[6.0, 8.0, 8.0], [4.0, 6.0, 2.0], [6.0, 6.0, 
8.0]], [[7.0, 3.0, 3.0], [1.0, 8.0, 4.0], [8.0, 5.0, 6.0]], [[1.0, 1.0, 6.0], [7.0, 7.0, 7.0], [1.0, 3.0, 4.0]]], [[[8.0, 5.0, 
7.0], [2.0, 8.0, 5.0], [4.0, 6.0, 5.0]], [[4.0, 2.0, 6.0], [1.0, 8.0, 5.0], [7.0, 5.0, 3.0]], [[6.0, 3.0, 3.0], [2.0, 3.0, 
1.0], [7.0, 2.0, 5.0]]], [[[2.0, 2.0, 2.0], [8.0, 3.0, 3.0], [4.0, 8.0, 5.0]], [[2.0, 4.0, 7.0], [7.0, 7.0, 7.0], [2.0, 3.0, 
7.0]], [[4.0, 5.0, 2.0], [3.0, 8.0, 8.0], [5.0, 3.0, 1.0]]]]))
      val=numpy.array([[[8.0, 5.0, 4.0]], [[5.0, 2.0, 8.0]], [[3.0, 7.0, 1.0]]])
      arg[:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 7.0], [2.0, 4.0, 4.0], [7.0, 7.0, 3.0]], [[6.0, 6.0, 7.0], [7.0, 7.0, 4.0], [3.0, 
3.0, 1.0]], [[1.0, 7.0, 6.0], [8.0, 5.0, 4.0], [1.0, 7.0, 5.0]]], [[[7.0, 8.0, 1.0], [4.0, 3.0, 7.0], [1.0, 4.0, 4.0]], [[1.0, 
2.0, 7.0], [7.0, 4.0, 4.0], [1.0, 2.0, 7.0]], [[4.0, 5.0, 8.0], [5.0, 2.0, 8.0], [8.0, 7.0, 8.0]]], [[[3.0, 8.0, 3.0], [8.0, 
2.0, 1.0], [1.0, 7.0, 2.0]], [[7.0, 7.0, 2.0], [4.0, 2.0, 8.0], [8.0, 2.0, 4.0]], [[5.0, 2.0, 3.0], [3.0, 7.0, 1.0], [2.0, 4.0, 
1.0]]], [[[7.0, 2.0, 6.0], [6.0, 2.0, 5.0], [4.0, 1.0, 1.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 5.0], [1.0, 2.0, 4.0]], [[7.0, 4.0, 
7.0], [6.0, 7.0, 2.0], [3.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 5.0], [8.0, 3.0, 7.0], [7.0, 2.0, 5.0]], 
[[8.0, 7.0, 8.0], [6.0, 8.0, 5.0], [4.0, 8.0, 8.0]], [[8.0, 7.0, 8.0], [8.0, 5.0, 4.0], [1.0, 6.0, 3.0]]], [[[6.0, 8.0, 8.0], 
[4.0, 6.0, 2.0], [6.0, 6.0, 8.0]], [[7.0, 3.0, 3.0], [1.0, 8.0, 4.0], [8.0, 5.0, 6.0]], [[1.0, 1.0, 6.0], [5.0, 2.0, 8.0], 
[1.0, 3.0, 4.0]]], [[[8.0, 5.0, 7.0], [2.0, 8.0, 5.0], [4.0, 6.0, 5.0]], [[4.0, 2.0, 6.0], [1.0, 8.0, 5.0], [7.0, 5.0, 3.0]], 
[[6.0, 3.0, 3.0], [3.0, 7.0, 1.0], [7.0, 2.0, 5.0]]], [[[2.0, 2.0, 2.0], [8.0, 3.0, 3.0], [4.0, 8.0, 5.0]], [[2.0, 4.0, 7.0], 
[7.0, 7.0, 7.0], [2.0, 3.0, 7.0]], [[4.0, 5.0, 2.0], [3.0, 8.0, 8.0], [5.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULPN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 2.0], [1.0, 6.0, 4.0], [7.0, 3.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 7.0, 6.0], [5.0, 6.0, 
5.0]], [[4.0, 4.0, 1.0], [5.0, 4.0, 6.0], [7.0, 7.0, 4.0]]], [[[4.0, 6.0, 8.0], [3.0, 6.0, 5.0], [7.0, 8.0, 1.0]], [[4.0, 5.0, 
6.0], [1.0, 5.0, 4.0], [5.0, 5.0, 5.0]], [[1.0, 4.0, 7.0], [7.0, 2.0, 7.0], [3.0, 6.0, 7.0]]], [[[6.0, 2.0, 2.0], [6.0, 2.0, 
2.0], [7.0, 1.0, 6.0]], [[7.0, 5.0, 7.0], [6.0, 5.0, 5.0], [7.0, 4.0, 5.0]], [[1.0, 5.0, 4.0], [3.0, 4.0, 4.0], [5.0, 3.0, 
8.0]]], [[[5.0, 5.0, 2.0], [4.0, 8.0, 1.0], [7.0, 3.0, 3.0]], [[7.0, 2.0, 5.0], [6.0, 5.0, 7.0], [8.0, 3.0, 5.0]], [[3.0, 3.0, 
7.0], [8.0, 7.0, 1.0], [3.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 5.0], [4.0, 7.0, 2.0], [1.0, 5.0, 3.0]], [[3.0, 7.0, 7.0], [1.0, 3.0, 
5.0], [2.0, 8.0, 8.0]], [[2.0, 6.0, 7.0], [1.0, 5.0, 2.0], [4.0, 1.0, 4.0]]], [[[1.0, 5.0, 6.0], [1.0, 8.0, 6.0], [4.0, 2.0, 
8.0]], [[2.0, 8.0, 8.0], [3.0, 4.0, 8.0], [1.0, 3.0, 6.0]], [[8.0, 8.0, 8.0], [7.0, 8.0, 5.0], [6.0, 7.0, 4.0]]], [[[2.0, 5.0, 
5.0], [7.0, 5.0, 3.0], [2.0, 5.0, 3.0]], [[6.0, 2.0, 4.0], [7.0, 3.0, 1.0], [1.0, 7.0, 3.0]], [[6.0, 1.0, 1.0], [5.0, 7.0, 
5.0], [1.0, 1.0, 5.0]]], [[[6.0, 3.0, 6.0], [5.0, 1.0, 4.0], [7.0, 6.0, 5.0]], [[4.0, 4.0, 2.0], [1.0, 4.0, 3.0], [2.0, 4.0, 
1.0]], [[4.0, 4.0, 5.0], [7.0, 2.0, 5.0], [7.0, 4.0, 4.0]]]]))
      val=Data(numpy.array([[[7.0, 3.0, 5.0]], [[2.0, 3.0, 4.0]], [[7.0, 2.0, 1.0]]]),self.functionspace)
      arg[:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 2.0], [1.0, 6.0, 4.0], [7.0, 3.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 7.0, 6.0], [5.0, 
6.0, 5.0]], [[4.0, 4.0, 1.0], [7.0, 3.0, 5.0], [7.0, 7.0, 4.0]]], [[[4.0, 6.0, 8.0], [3.0, 6.0, 5.0], [7.0, 8.0, 1.0]], [[4.0, 
5.0, 6.0], [1.0, 5.0, 4.0], [5.0, 5.0, 5.0]], [[1.0, 4.0, 7.0], [2.0, 3.0, 4.0], [3.0, 6.0, 7.0]]], [[[6.0, 2.0, 2.0], [6.0, 
2.0, 2.0], [7.0, 1.0, 6.0]], [[7.0, 5.0, 7.0], [6.0, 5.0, 5.0], [7.0, 4.0, 5.0]], [[1.0, 5.0, 4.0], [7.0, 2.0, 1.0], [5.0, 3.0, 
8.0]]], [[[5.0, 5.0, 2.0], [4.0, 8.0, 1.0], [7.0, 3.0, 3.0]], [[7.0, 2.0, 5.0], [6.0, 5.0, 7.0], [8.0, 3.0, 5.0]], [[3.0, 3.0, 
7.0], [8.0, 7.0, 1.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 5.0], [4.0, 7.0, 2.0], [1.0, 5.0, 3.0]], 
[[3.0, 7.0, 7.0], [1.0, 3.0, 5.0], [2.0, 8.0, 8.0]], [[2.0, 6.0, 7.0], [7.0, 3.0, 5.0], [4.0, 1.0, 4.0]]], [[[1.0, 5.0, 6.0], 
[1.0, 8.0, 6.0], [4.0, 2.0, 8.0]], [[2.0, 8.0, 8.0], [3.0, 4.0, 8.0], [1.0, 3.0, 6.0]], [[8.0, 8.0, 8.0], [2.0, 3.0, 4.0], 
[6.0, 7.0, 4.0]]], [[[2.0, 5.0, 5.0], [7.0, 5.0, 3.0], [2.0, 5.0, 3.0]], [[6.0, 2.0, 4.0], [7.0, 3.0, 1.0], [1.0, 7.0, 3.0]], 
[[6.0, 1.0, 1.0], [7.0, 2.0, 1.0], [1.0, 1.0, 5.0]]], [[[6.0, 3.0, 6.0], [5.0, 1.0, 4.0], [7.0, 6.0, 5.0]], [[4.0, 4.0, 2.0], 
[1.0, 4.0, 3.0], [2.0, 4.0, 1.0]], [[4.0, 4.0, 5.0], [7.0, 2.0, 5.0], [7.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULPN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 3.0], [2.0, 3.0, 4.0], [8.0, 7.0, 1.0]], [[6.0, 4.0, 6.0], [8.0, 4.0, 8.0], [1.0, 3.0, 
1.0]], [[8.0, 1.0, 7.0], [3.0, 8.0, 5.0], [6.0, 6.0, 3.0]]], [[[6.0, 2.0, 2.0], [7.0, 3.0, 6.0], [8.0, 6.0, 5.0]], [[7.0, 5.0, 
8.0], [8.0, 2.0, 4.0], [7.0, 1.0, 1.0]], [[8.0, 4.0, 5.0], [1.0, 7.0, 4.0], [3.0, 7.0, 6.0]]], [[[4.0, 5.0, 5.0], [7.0, 8.0, 
5.0], [2.0, 6.0, 1.0]], [[4.0, 2.0, 4.0], [1.0, 6.0, 5.0], [4.0, 1.0, 7.0]], [[6.0, 4.0, 1.0], [6.0, 3.0, 3.0], [3.0, 2.0, 
7.0]]], [[[3.0, 4.0, 3.0], [7.0, 6.0, 3.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 3.0], [2.0, 6.0, 7.0], [6.0, 3.0, 5.0]], [[5.0, 7.0, 
1.0], [1.0, 3.0, 8.0], [4.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 8.0], [8.0, 7.0, 5.0], [2.0, 7.0, 7.0]], [[2.0, 2.0, 7.0], [6.0, 8.0, 
3.0], [5.0, 8.0, 5.0]], [[1.0, 6.0, 7.0], [3.0, 5.0, 8.0], [4.0, 8.0, 5.0]]], [[[2.0, 4.0, 5.0], [7.0, 7.0, 2.0], [8.0, 4.0, 
5.0]], [[1.0, 3.0, 1.0], [1.0, 4.0, 6.0], [2.0, 4.0, 7.0]], [[7.0, 1.0, 3.0], [1.0, 5.0, 5.0], [7.0, 3.0, 6.0]]], [[[1.0, 5.0, 
2.0], [6.0, 8.0, 1.0], [7.0, 5.0, 7.0]], [[3.0, 5.0, 6.0], [1.0, 8.0, 6.0], [5.0, 3.0, 8.0]], [[2.0, 2.0, 3.0], [5.0, 2.0, 
3.0], [4.0, 8.0, 5.0]]], [[[1.0, 6.0, 2.0], [4.0, 3.0, 5.0], [2.0, 2.0, 4.0]], [[3.0, 7.0, 1.0], [5.0, 4.0, 7.0], [5.0, 7.0, 
4.0]], [[5.0, 8.0, 3.0], [8.0, 1.0, 6.0], [5.0, 6.0, 8.0]]]]))
      val=Data(numpy.array([[[1.0, 8.0, 5.0]], [[7.0, 1.0, 1.0]], [[7.0, 4.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 5.0, 1.0]], [[1.0, 5.0, 3.0]], [[5.0, 4.0, 6.0]]]))
      arg[:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 3.0], [2.0, 3.0, 4.0], [8.0, 7.0, 1.0]], [[6.0, 4.0, 6.0], [8.0, 4.0, 8.0], [1.0, 
3.0, 1.0]], [[8.0, 1.0, 7.0], [1.0, 8.0, 5.0], [6.0, 6.0, 3.0]]], [[[6.0, 2.0, 2.0], [7.0, 3.0, 6.0], [8.0, 6.0, 5.0]], [[7.0, 
5.0, 8.0], [8.0, 2.0, 4.0], [7.0, 1.0, 1.0]], [[8.0, 4.0, 5.0], [7.0, 1.0, 1.0], [3.0, 7.0, 6.0]]], [[[4.0, 5.0, 5.0], [7.0, 
8.0, 5.0], [2.0, 6.0, 1.0]], [[4.0, 2.0, 4.0], [1.0, 6.0, 5.0], [4.0, 1.0, 7.0]], [[6.0, 4.0, 1.0], [7.0, 4.0, 7.0], [3.0, 2.0, 
7.0]]], [[[3.0, 4.0, 3.0], [7.0, 6.0, 3.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 3.0], [2.0, 6.0, 7.0], [6.0, 3.0, 5.0]], [[5.0, 7.0, 
1.0], [1.0, 3.0, 8.0], [4.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 8.0], [8.0, 7.0, 5.0], [2.0, 7.0, 7.0]], 
[[2.0, 2.0, 7.0], [6.0, 8.0, 3.0], [5.0, 8.0, 5.0]], [[1.0, 6.0, 7.0], [4.0, 5.0, 1.0], [4.0, 8.0, 5.0]]], [[[2.0, 4.0, 5.0], 
[7.0, 7.0, 2.0], [8.0, 4.0, 5.0]], [[1.0, 3.0, 1.0], [1.0, 4.0, 6.0], [2.0, 4.0, 7.0]], [[7.0, 1.0, 3.0], [1.0, 5.0, 3.0], 
[7.0, 3.0, 6.0]]], [[[1.0, 5.0, 2.0], [6.0, 8.0, 1.0], [7.0, 5.0, 7.0]], [[3.0, 5.0, 6.0], [1.0, 8.0, 6.0], [5.0, 3.0, 8.0]], 
[[2.0, 2.0, 3.0], [5.0, 4.0, 6.0], [4.0, 8.0, 5.0]]], [[[1.0, 6.0, 2.0], [4.0, 3.0, 5.0], [2.0, 2.0, 4.0]], [[3.0, 7.0, 1.0], 
[5.0, 4.0, 7.0], [5.0, 7.0, 4.0]], [[5.0, 8.0, 3.0], [8.0, 1.0, 6.0], [5.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULPN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 7.0], [2.0, 3.0, 4.0], [6.0, 8.0, 2.0]], [[4.0, 7.0, 3.0], [7.0, 8.0, 4.0], [4.0, 5.0, 
3.0]], [[1.0, 6.0, 2.0], [6.0, 2.0, 6.0], [5.0, 4.0, 3.0]]], [[[2.0, 2.0, 1.0], [8.0, 7.0, 7.0], [5.0, 8.0, 7.0]], [[1.0, 4.0, 
2.0], [5.0, 4.0, 4.0], [5.0, 7.0, 7.0]], [[2.0, 2.0, 4.0], [8.0, 4.0, 5.0], [4.0, 7.0, 3.0]]], [[[1.0, 2.0, 3.0], [4.0, 7.0, 
3.0], [1.0, 7.0, 4.0]], [[1.0, 1.0, 6.0], [3.0, 7.0, 5.0], [4.0, 2.0, 1.0]], [[1.0, 3.0, 6.0], [8.0, 5.0, 1.0], [5.0, 8.0, 
8.0]]], [[[8.0, 1.0, 8.0], [3.0, 7.0, 1.0], [1.0, 7.0, 2.0]], [[3.0, 5.0, 7.0], [5.0, 5.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 5.0, 
8.0], [6.0, 4.0, 2.0], [8.0, 2.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 6.0], [6.0, 5.0, 6.0], [3.0, 2.0, 4.0]], [[7.0, 3.0, 8.0], [1.0, 5.0, 
7.0], [8.0, 7.0, 7.0]], [[5.0, 8.0, 8.0], [1.0, 1.0, 4.0], [3.0, 8.0, 5.0]]], [[[6.0, 5.0, 5.0], [8.0, 8.0, 1.0], [2.0, 7.0, 
5.0]], [[8.0, 4.0, 2.0], [1.0, 5.0, 6.0], [6.0, 7.0, 4.0]], [[6.0, 5.0, 1.0], [1.0, 6.0, 6.0], [8.0, 1.0, 4.0]]], [[[4.0, 5.0, 
4.0], [1.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[8.0, 7.0, 1.0], [6.0, 2.0, 2.0], [6.0, 7.0, 1.0]], [[7.0, 7.0, 6.0], [4.0, 6.0, 
6.0], [1.0, 5.0, 3.0]]], [[[6.0, 6.0, 3.0], [5.0, 1.0, 2.0], [4.0, 4.0, 6.0]], [[7.0, 3.0, 2.0], [4.0, 2.0, 3.0], [1.0, 7.0, 
5.0]], [[4.0, 4.0, 2.0], [4.0, 2.0, 1.0], [6.0, 3.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0, 8.0, 2.0]], [[6.0, 2.0, 1.0]], [[5.0, 5.0, 3.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 
3.0]], [[3.0, 7.0, 7.0]], [[3.0, 5.0, 6.0]]])
      arg[:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 7.0], [2.0, 3.0, 4.0], [6.0, 8.0, 2.0]], [[4.0, 7.0, 3.0], [7.0, 8.0, 4.0], [4.0, 
5.0, 3.0]], [[1.0, 6.0, 2.0], [1.0, 8.0, 2.0], [5.0, 4.0, 3.0]]], [[[2.0, 2.0, 1.0], [8.0, 7.0, 7.0], [5.0, 8.0, 7.0]], [[1.0, 
4.0, 2.0], [5.0, 4.0, 4.0], [5.0, 7.0, 7.0]], [[2.0, 2.0, 4.0], [6.0, 2.0, 1.0], [4.0, 7.0, 3.0]]], [[[1.0, 2.0, 3.0], [4.0, 
7.0, 3.0], [1.0, 7.0, 4.0]], [[1.0, 1.0, 6.0], [3.0, 7.0, 5.0], [4.0, 2.0, 1.0]], [[1.0, 3.0, 6.0], [5.0, 5.0, 3.0], [5.0, 8.0, 
8.0]]], [[[8.0, 1.0, 8.0], [3.0, 7.0, 1.0], [1.0, 7.0, 2.0]], [[3.0, 5.0, 7.0], [5.0, 5.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 5.0, 
8.0], [6.0, 4.0, 2.0], [8.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 6.0], [6.0, 5.0, 6.0], [3.0, 2.0, 4.0]], 
[[7.0, 3.0, 8.0], [1.0, 5.0, 7.0], [8.0, 7.0, 7.0]], [[5.0, 8.0, 8.0], [4.0, 6.0, 3.0], [3.0, 8.0, 5.0]]], [[[6.0, 5.0, 5.0], 
[8.0, 8.0, 1.0], [2.0, 7.0, 5.0]], [[8.0, 4.0, 2.0], [1.0, 5.0, 6.0], [6.0, 7.0, 4.0]], [[6.0, 5.0, 1.0], [3.0, 7.0, 7.0], 
[8.0, 1.0, 4.0]]], [[[4.0, 5.0, 4.0], [1.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[8.0, 7.0, 1.0], [6.0, 2.0, 2.0], [6.0, 7.0, 1.0]], 
[[7.0, 7.0, 6.0], [3.0, 5.0, 6.0], [1.0, 5.0, 3.0]]], [[[6.0, 6.0, 3.0], [5.0, 1.0, 2.0], [4.0, 4.0, 6.0]], [[7.0, 3.0, 2.0], 
[4.0, 2.0, 3.0], [1.0, 7.0, 5.0]], [[4.0, 4.0, 2.0], [4.0, 2.0, 1.0], [6.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPRU(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 3.0], [4.0, 5.0, 4.0], [5.0, 3.0, 8.0]], [[3.0, 8.0, 5.0], [7.0, 4.0, 2.0], [7.0, 2.0, 
3.0]], [[7.0, 7.0, 6.0], [1.0, 1.0, 5.0], [3.0, 5.0, 5.0]]], [[[4.0, 1.0, 2.0], [5.0, 3.0, 5.0], [2.0, 6.0, 4.0]], [[7.0, 1.0, 
6.0], [4.0, 1.0, 7.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 7.0], [6.0, 5.0, 4.0], [2.0, 7.0, 1.0]]], [[[5.0, 6.0, 4.0], [6.0, 5.0, 
2.0], [4.0, 8.0, 2.0]], [[8.0, 4.0, 1.0], [8.0, 4.0, 5.0], [6.0, 8.0, 1.0]], [[1.0, 5.0, 1.0], [3.0, 2.0, 6.0], [2.0, 1.0, 
7.0]]], [[[5.0, 1.0, 5.0], [1.0, 7.0, 7.0], [4.0, 1.0, 3.0]], [[2.0, 1.0, 2.0], [8.0, 3.0, 5.0], [5.0, 8.0, 2.0]], [[4.0, 7.0, 
8.0], [6.0, 4.0, 7.0], [6.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 2.0], [1.0, 7.0, 7.0], [6.0, 1.0, 4.0]], [[8.0, 8.0, 8.0], [6.0, 2.0, 
8.0], [1.0, 1.0, 7.0]], [[4.0, 5.0, 1.0], [8.0, 8.0, 1.0], [3.0, 1.0, 7.0]]], [[[8.0, 1.0, 6.0], [5.0, 7.0, 3.0], [7.0, 3.0, 
8.0]], [[8.0, 6.0, 7.0], [3.0, 1.0, 7.0], [2.0, 8.0, 1.0]], [[4.0, 7.0, 6.0], [1.0, 1.0, 3.0], [1.0, 4.0, 5.0]]], [[[8.0, 1.0, 
6.0], [3.0, 7.0, 6.0], [7.0, 6.0, 2.0]], [[8.0, 4.0, 2.0], [2.0, 5.0, 2.0], [5.0, 8.0, 1.0]], [[2.0, 4.0, 4.0], [7.0, 6.0, 
6.0], [6.0, 4.0, 3.0]]], [[[3.0, 2.0, 6.0], [4.0, 6.0, 6.0], [4.0, 6.0, 2.0]], [[5.0, 4.0, 6.0], [8.0, 1.0, 8.0], [5.0, 3.0, 
3.0]], [[5.0, 7.0, 8.0], [6.0, 2.0, 2.0], [1.0, 1.0, 2.0]]]]))
      val=numpy.array([[[4.0, 3.0]], [[1.0, 5.0]]])
      arg[1:3,1,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 3.0], [4.0, 5.0, 4.0], [5.0, 3.0, 8.0]], [[3.0, 8.0, 5.0], [7.0, 4.0, 2.0], [7.0, 
2.0, 3.0]], [[7.0, 7.0, 6.0], [1.0, 1.0, 5.0], [3.0, 5.0, 5.0]]], [[[4.0, 1.0, 2.0], [5.0, 3.0, 5.0], [2.0, 6.0, 4.0]], [[7.0, 
1.0, 6.0], [4.0, 3.0, 7.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 7.0], [6.0, 5.0, 4.0], [2.0, 7.0, 1.0]]], [[[5.0, 6.0, 4.0], [6.0, 
5.0, 2.0], [4.0, 8.0, 2.0]], [[8.0, 4.0, 1.0], [1.0, 5.0, 5.0], [6.0, 8.0, 1.0]], [[1.0, 5.0, 1.0], [3.0, 2.0, 6.0], [2.0, 1.0, 
7.0]]], [[[5.0, 1.0, 5.0], [1.0, 7.0, 7.0], [4.0, 1.0, 3.0]], [[2.0, 1.0, 2.0], [8.0, 3.0, 5.0], [5.0, 8.0, 2.0]], [[4.0, 7.0, 
8.0], [6.0, 4.0, 7.0], [6.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 2.0], [1.0, 7.0, 7.0], [6.0, 1.0, 4.0]], 
[[8.0, 8.0, 8.0], [6.0, 2.0, 8.0], [1.0, 1.0, 7.0]], [[4.0, 5.0, 1.0], [8.0, 8.0, 1.0], [3.0, 1.0, 7.0]]], [[[8.0, 1.0, 6.0], 
[5.0, 7.0, 3.0], [7.0, 3.0, 8.0]], [[8.0, 6.0, 7.0], [4.0, 3.0, 7.0], [2.0, 8.0, 1.0]], [[4.0, 7.0, 6.0], [1.0, 1.0, 3.0], 
[1.0, 4.0, 5.0]]], [[[8.0, 1.0, 6.0], [3.0, 7.0, 6.0], [7.0, 6.0, 2.0]], [[8.0, 4.0, 2.0], [1.0, 5.0, 2.0], [5.0, 8.0, 1.0]], 
[[2.0, 4.0, 4.0], [7.0, 6.0, 6.0], [6.0, 4.0, 3.0]]], [[[3.0, 2.0, 6.0], [4.0, 6.0, 6.0], [4.0, 6.0, 2.0]], [[5.0, 4.0, 6.0], 
[8.0, 1.0, 8.0], [5.0, 3.0, 3.0]], [[5.0, 7.0, 8.0], [6.0, 2.0, 2.0], [1.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPRU(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 3.0], [3.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[6.0, 3.0, 3.0], [1.0, 6.0, 8.0], [1.0, 3.0, 
2.0]], [[7.0, 6.0, 4.0], [6.0, 8.0, 3.0], [2.0, 4.0, 5.0]]], [[[3.0, 4.0, 6.0], [7.0, 2.0, 1.0], [5.0, 5.0, 3.0]], [[6.0, 8.0, 
7.0], [3.0, 8.0, 4.0], [5.0, 6.0, 8.0]], [[8.0, 6.0, 6.0], [4.0, 8.0, 3.0], [7.0, 5.0, 1.0]]], [[[8.0, 3.0, 3.0], [4.0, 3.0, 
3.0], [2.0, 8.0, 7.0]], [[6.0, 8.0, 6.0], [2.0, 8.0, 4.0], [1.0, 3.0, 1.0]], [[2.0, 8.0, 2.0], [4.0, 4.0, 4.0], [4.0, 7.0, 
2.0]]], [[[5.0, 3.0, 2.0], [6.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[4.0, 6.0, 1.0], [6.0, 6.0, 1.0], [4.0, 2.0, 6.0]], [[3.0, 4.0, 
4.0], [5.0, 7.0, 2.0], [3.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 6.0], [4.0, 4.0, 6.0], [5.0, 7.0, 8.0]], [[5.0, 5.0, 8.0], [4.0, 5.0, 
8.0], [6.0, 6.0, 1.0]], [[6.0, 2.0, 7.0], [7.0, 5.0, 3.0], [3.0, 6.0, 1.0]]], [[[2.0, 5.0, 8.0], [4.0, 1.0, 3.0], [8.0, 8.0, 
7.0]], [[3.0, 4.0, 8.0], [2.0, 3.0, 1.0], [2.0, 4.0, 3.0]], [[7.0, 8.0, 3.0], [6.0, 7.0, 5.0], [2.0, 6.0, 5.0]]], [[[3.0, 7.0, 
8.0], [3.0, 8.0, 7.0], [5.0, 1.0, 7.0]], [[7.0, 3.0, 8.0], [1.0, 7.0, 1.0], [2.0, 6.0, 5.0]], [[8.0, 4.0, 1.0], [6.0, 3.0, 
3.0], [5.0, 2.0, 5.0]]], [[[4.0, 3.0, 8.0], [8.0, 1.0, 8.0], [2.0, 2.0, 8.0]], [[8.0, 2.0, 3.0], [8.0, 6.0, 5.0], [4.0, 1.0, 
2.0]], [[5.0, 4.0, 5.0], [6.0, 4.0, 4.0], [2.0, 1.0, 2.0]]]]))
      val=Data(numpy.array([[[2.0, 1.0]], [[1.0, 1.0]]]),self.functionspace)
      arg[1:3,1,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 3.0], [3.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[6.0, 3.0, 3.0], [1.0, 6.0, 8.0], [1.0, 
3.0, 2.0]], [[7.0, 6.0, 4.0], [6.0, 8.0, 3.0], [2.0, 4.0, 5.0]]], [[[3.0, 4.0, 6.0], [7.0, 2.0, 1.0], [5.0, 5.0, 3.0]], [[6.0, 
8.0, 7.0], [2.0, 1.0, 4.0], [5.0, 6.0, 8.0]], [[8.0, 6.0, 6.0], [4.0, 8.0, 3.0], [7.0, 5.0, 1.0]]], [[[8.0, 3.0, 3.0], [4.0, 
3.0, 3.0], [2.0, 8.0, 7.0]], [[6.0, 8.0, 6.0], [1.0, 1.0, 4.0], [1.0, 3.0, 1.0]], [[2.0, 8.0, 2.0], [4.0, 4.0, 4.0], [4.0, 7.0, 
2.0]]], [[[5.0, 3.0, 2.0], [6.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[4.0, 6.0, 1.0], [6.0, 6.0, 1.0], [4.0, 2.0, 6.0]], [[3.0, 4.0, 
4.0], [5.0, 7.0, 2.0], [3.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 6.0], [4.0, 4.0, 6.0], [5.0, 7.0, 8.0]], 
[[5.0, 5.0, 8.0], [4.0, 5.0, 8.0], [6.0, 6.0, 1.0]], [[6.0, 2.0, 7.0], [7.0, 5.0, 3.0], [3.0, 6.0, 1.0]]], [[[2.0, 5.0, 8.0], 
[4.0, 1.0, 3.0], [8.0, 8.0, 7.0]], [[3.0, 4.0, 8.0], [2.0, 1.0, 1.0], [2.0, 4.0, 3.0]], [[7.0, 8.0, 3.0], [6.0, 7.0, 5.0], 
[2.0, 6.0, 5.0]]], [[[3.0, 7.0, 8.0], [3.0, 8.0, 7.0], [5.0, 1.0, 7.0]], [[7.0, 3.0, 8.0], [1.0, 1.0, 1.0], [2.0, 6.0, 5.0]], 
[[8.0, 4.0, 1.0], [6.0, 3.0, 3.0], [5.0, 2.0, 5.0]]], [[[4.0, 3.0, 8.0], [8.0, 1.0, 8.0], [2.0, 2.0, 8.0]], [[8.0, 2.0, 3.0], 
[8.0, 6.0, 5.0], [4.0, 1.0, 2.0]], [[5.0, 4.0, 5.0], [6.0, 4.0, 4.0], [2.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPRU(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 2.0], [3.0, 4.0, 7.0], [4.0, 1.0, 1.0]], [[3.0, 6.0, 1.0], [6.0, 5.0, 3.0], [6.0, 6.0, 
5.0]], [[3.0, 6.0, 8.0], [1.0, 8.0, 2.0], [2.0, 1.0, 3.0]]], [[[4.0, 6.0, 2.0], [4.0, 8.0, 5.0], [4.0, 7.0, 5.0]], [[5.0, 3.0, 
7.0], [1.0, 3.0, 1.0], [4.0, 7.0, 8.0]], [[3.0, 6.0, 7.0], [7.0, 3.0, 4.0], [3.0, 1.0, 7.0]]], [[[8.0, 6.0, 5.0], [7.0, 4.0, 
2.0], [7.0, 2.0, 7.0]], [[1.0, 5.0, 4.0], [8.0, 4.0, 6.0], [3.0, 7.0, 6.0]], [[6.0, 7.0, 6.0], [5.0, 8.0, 4.0], [5.0, 5.0, 
5.0]]], [[[7.0, 3.0, 2.0], [6.0, 1.0, 6.0], [3.0, 4.0, 8.0]], [[1.0, 3.0, 1.0], [5.0, 1.0, 3.0], [3.0, 3.0, 8.0]], [[4.0, 1.0, 
1.0], [8.0, 3.0, 5.0], [8.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 6.0], [8.0, 3.0, 3.0], [1.0, 8.0, 7.0]], [[7.0, 1.0, 4.0], [4.0, 2.0, 
3.0], [2.0, 5.0, 6.0]], [[2.0, 2.0, 7.0], [3.0, 1.0, 2.0], [6.0, 6.0, 4.0]]], [[[4.0, 4.0, 1.0], [7.0, 5.0, 6.0], [4.0, 1.0, 
4.0]], [[6.0, 2.0, 3.0], [1.0, 5.0, 5.0], [7.0, 5.0, 2.0]], [[3.0, 1.0, 2.0], [1.0, 8.0, 3.0], [1.0, 7.0, 5.0]]], [[[6.0, 3.0, 
5.0], [1.0, 2.0, 3.0], [2.0, 7.0, 4.0]], [[3.0, 3.0, 7.0], [8.0, 7.0, 3.0], [6.0, 3.0, 5.0]], [[7.0, 4.0, 1.0], [8.0, 6.0, 
7.0], [5.0, 7.0, 8.0]]], [[[1.0, 8.0, 2.0], [7.0, 3.0, 5.0], [5.0, 3.0, 1.0]], [[3.0, 6.0, 7.0], [8.0, 5.0, 5.0], [1.0, 2.0, 
5.0]], [[7.0, 2.0, 1.0], [7.0, 1.0, 2.0], [3.0, 5.0, 6.0]]]]))
      val=Data(numpy.array([[[8.0, 3.0]], [[5.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0]], [[6.0, 3.0]]]))
      arg[1:3,1,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 2.0], [3.0, 4.0, 7.0], [4.0, 1.0, 1.0]], [[3.0, 6.0, 1.0], [6.0, 5.0, 3.0], [6.0, 
6.0, 5.0]], [[3.0, 6.0, 8.0], [1.0, 8.0, 2.0], [2.0, 1.0, 3.0]]], [[[4.0, 6.0, 2.0], [4.0, 8.0, 5.0], [4.0, 7.0, 5.0]], [[5.0, 
3.0, 7.0], [8.0, 3.0, 1.0], [4.0, 7.0, 8.0]], [[3.0, 6.0, 7.0], [7.0, 3.0, 4.0], [3.0, 1.0, 7.0]]], [[[8.0, 6.0, 5.0], [7.0, 
4.0, 2.0], [7.0, 2.0, 7.0]], [[1.0, 5.0, 4.0], [5.0, 3.0, 6.0], [3.0, 7.0, 6.0]], [[6.0, 7.0, 6.0], [5.0, 8.0, 4.0], [5.0, 5.0, 
5.0]]], [[[7.0, 3.0, 2.0], [6.0, 1.0, 6.0], [3.0, 4.0, 8.0]], [[1.0, 3.0, 1.0], [5.0, 1.0, 3.0], [3.0, 3.0, 8.0]], [[4.0, 1.0, 
1.0], [8.0, 3.0, 5.0], [8.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 6.0], [8.0, 3.0, 3.0], [1.0, 8.0, 7.0]], 
[[7.0, 1.0, 4.0], [4.0, 2.0, 3.0], [2.0, 5.0, 6.0]], [[2.0, 2.0, 7.0], [3.0, 1.0, 2.0], [6.0, 6.0, 4.0]]], [[[4.0, 4.0, 1.0], 
[7.0, 5.0, 6.0], [4.0, 1.0, 4.0]], [[6.0, 2.0, 3.0], [3.0, 5.0, 5.0], [7.0, 5.0, 2.0]], [[3.0, 1.0, 2.0], [1.0, 8.0, 3.0], 
[1.0, 7.0, 5.0]]], [[[6.0, 3.0, 5.0], [1.0, 2.0, 3.0], [2.0, 7.0, 4.0]], [[3.0, 3.0, 7.0], [6.0, 3.0, 3.0], [6.0, 3.0, 5.0]], 
[[7.0, 4.0, 1.0], [8.0, 6.0, 7.0], [5.0, 7.0, 8.0]]], [[[1.0, 8.0, 2.0], [7.0, 3.0, 5.0], [5.0, 3.0, 1.0]], [[3.0, 6.0, 7.0], 
[8.0, 5.0, 5.0], [1.0, 2.0, 5.0]], [[7.0, 2.0, 1.0], [7.0, 1.0, 2.0], [3.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPRU(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 4.0], [5.0, 4.0, 2.0], [8.0, 4.0, 4.0]], [[7.0, 1.0, 7.0], [1.0, 8.0, 1.0], [3.0, 8.0, 
6.0]], [[4.0, 1.0, 7.0], [4.0, 4.0, 8.0], [2.0, 2.0, 6.0]]], [[[7.0, 1.0, 7.0], [2.0, 8.0, 5.0], [4.0, 2.0, 5.0]], [[2.0, 5.0, 
4.0], [3.0, 4.0, 5.0], [1.0, 1.0, 2.0]], [[3.0, 3.0, 6.0], [5.0, 8.0, 6.0], [6.0, 6.0, 8.0]]], [[[1.0, 2.0, 5.0], [8.0, 1.0, 
8.0], [3.0, 3.0, 8.0]], [[4.0, 4.0, 2.0], [6.0, 6.0, 8.0], [7.0, 8.0, 6.0]], [[1.0, 2.0, 6.0], [4.0, 4.0, 3.0], [5.0, 1.0, 
2.0]]], [[[5.0, 2.0, 5.0], [1.0, 7.0, 6.0], [6.0, 3.0, 8.0]], [[2.0, 8.0, 1.0], [3.0, 6.0, 2.0], [2.0, 1.0, 1.0]], [[6.0, 7.0, 
3.0], [7.0, 8.0, 8.0], [1.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 7.0], [3.0, 6.0, 8.0], [8.0, 2.0, 6.0]], [[8.0, 5.0, 2.0], [7.0, 5.0, 
5.0], [5.0, 6.0, 6.0]], [[7.0, 5.0, 2.0], [6.0, 8.0, 8.0], [1.0, 2.0, 7.0]]], [[[7.0, 3.0, 6.0], [5.0, 8.0, 2.0], [4.0, 4.0, 
2.0]], [[8.0, 6.0, 8.0], [8.0, 7.0, 8.0], [6.0, 4.0, 1.0]], [[5.0, 5.0, 6.0], [6.0, 3.0, 7.0], [5.0, 2.0, 5.0]]], [[[2.0, 8.0, 
6.0], [6.0, 7.0, 8.0], [6.0, 5.0, 2.0]], [[1.0, 2.0, 4.0], [2.0, 3.0, 4.0], [7.0, 1.0, 6.0]], [[1.0, 7.0, 7.0], [7.0, 8.0, 
5.0], [3.0, 1.0, 2.0]]], [[[8.0, 4.0, 4.0], [3.0, 7.0, 7.0], [3.0, 8.0, 2.0]], [[4.0, 3.0, 3.0], [7.0, 3.0, 2.0], [3.0, 7.0, 
7.0]], [[7.0, 7.0, 3.0], [3.0, 4.0, 7.0], [7.0, 8.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 3.0]], [[5.0, 8.0]]])+(1.-msk_val)*numpy.array([[[5.0, 1.0]], [[5.0, 1.0]]])
      arg[1:3,1,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [5.0, 4.0, 2.0], [8.0, 4.0, 4.0]], [[7.0, 1.0, 7.0], [1.0, 8.0, 1.0], [3.0, 
8.0, 6.0]], [[4.0, 1.0, 7.0], [4.0, 4.0, 8.0], [2.0, 2.0, 6.0]]], [[[7.0, 1.0, 7.0], [2.0, 8.0, 5.0], [4.0, 2.0, 5.0]], [[2.0, 
5.0, 4.0], [4.0, 3.0, 5.0], [1.0, 1.0, 2.0]], [[3.0, 3.0, 6.0], [5.0, 8.0, 6.0], [6.0, 6.0, 8.0]]], [[[1.0, 2.0, 5.0], [8.0, 
1.0, 8.0], [3.0, 3.0, 8.0]], [[4.0, 4.0, 2.0], [5.0, 8.0, 8.0], [7.0, 8.0, 6.0]], [[1.0, 2.0, 6.0], [4.0, 4.0, 3.0], [5.0, 1.0, 
2.0]]], [[[5.0, 2.0, 5.0], [1.0, 7.0, 6.0], [6.0, 3.0, 8.0]], [[2.0, 8.0, 1.0], [3.0, 6.0, 2.0], [2.0, 1.0, 1.0]], [[6.0, 7.0, 
3.0], [7.0, 8.0, 8.0], [1.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 7.0], [3.0, 6.0, 8.0], [8.0, 2.0, 6.0]], 
[[8.0, 5.0, 2.0], [7.0, 5.0, 5.0], [5.0, 6.0, 6.0]], [[7.0, 5.0, 2.0], [6.0, 8.0, 8.0], [1.0, 2.0, 7.0]]], [[[7.0, 3.0, 6.0], 
[5.0, 8.0, 2.0], [4.0, 4.0, 2.0]], [[8.0, 6.0, 8.0], [5.0, 1.0, 8.0], [6.0, 4.0, 1.0]], [[5.0, 5.0, 6.0], [6.0, 3.0, 7.0], 
[5.0, 2.0, 5.0]]], [[[2.0, 8.0, 6.0], [6.0, 7.0, 8.0], [6.0, 5.0, 2.0]], [[1.0, 2.0, 4.0], [5.0, 1.0, 4.0], [7.0, 1.0, 6.0]], 
[[1.0, 7.0, 7.0], [7.0, 8.0, 5.0], [3.0, 1.0, 2.0]]], [[[8.0, 4.0, 4.0], [3.0, 7.0, 7.0], [3.0, 8.0, 2.0]], [[4.0, 3.0, 3.0], 
[7.0, 3.0, 2.0], [3.0, 7.0, 7.0]], [[7.0, 7.0, 3.0], [3.0, 4.0, 7.0], [7.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPPN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 6.0], [6.0, 2.0, 2.0], [6.0, 5.0, 3.0]], [[8.0, 5.0, 7.0], [1.0, 4.0, 4.0], [7.0, 1.0, 
7.0]], [[8.0, 3.0, 3.0], [1.0, 4.0, 2.0], [1.0, 5.0, 6.0]]], [[[5.0, 4.0, 8.0], [5.0, 7.0, 7.0], [1.0, 7.0, 6.0]], [[2.0, 5.0, 
2.0], [2.0, 1.0, 2.0], [1.0, 4.0, 3.0]], [[8.0, 1.0, 8.0], [5.0, 3.0, 7.0], [1.0, 4.0, 5.0]]], [[[1.0, 6.0, 7.0], [1.0, 6.0, 
5.0], [2.0, 8.0, 4.0]], [[7.0, 2.0, 2.0], [4.0, 7.0, 3.0], [5.0, 1.0, 4.0]], [[1.0, 8.0, 4.0], [7.0, 8.0, 2.0], [6.0, 4.0, 
3.0]]], [[[7.0, 7.0, 1.0], [8.0, 7.0, 6.0], [4.0, 7.0, 6.0]], [[4.0, 4.0, 8.0], [1.0, 8.0, 1.0], [5.0, 7.0, 6.0]], [[8.0, 7.0, 
7.0], [1.0, 8.0, 3.0], [2.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 8.0], [7.0, 5.0, 8.0], [7.0, 1.0, 5.0]], [[8.0, 3.0, 3.0], [6.0, 4.0, 
3.0], [4.0, 4.0, 2.0]], [[3.0, 2.0, 3.0], [8.0, 7.0, 1.0], [5.0, 8.0, 6.0]]], [[[7.0, 4.0, 4.0], [3.0, 4.0, 6.0], [3.0, 1.0, 
8.0]], [[6.0, 2.0, 2.0], [2.0, 1.0, 2.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 6.0], [3.0, 1.0, 8.0], [4.0, 2.0, 7.0]]], [[[1.0, 2.0, 
8.0], [3.0, 3.0, 1.0], [4.0, 8.0, 5.0]], [[7.0, 8.0, 5.0], [5.0, 5.0, 1.0], [7.0, 8.0, 7.0]], [[7.0, 1.0, 6.0], [2.0, 7.0, 
6.0], [8.0, 6.0, 1.0]]], [[[7.0, 7.0, 1.0], [4.0, 7.0, 1.0], [2.0, 3.0, 5.0]], [[4.0, 2.0, 2.0], [5.0, 6.0, 1.0], [5.0, 4.0, 
3.0]], [[7.0, 2.0, 7.0], [7.0, 2.0, 7.0], [4.0, 3.0, 6.0]]]]))
      val=numpy.array([[6.0, 7.0, 5.0], [2.0, 3.0, 1.0]])
      arg[1:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 6.0], [6.0, 2.0, 2.0], [6.0, 5.0, 3.0]], [[8.0, 5.0, 7.0], [1.0, 4.0, 4.0], [7.0, 
1.0, 7.0]], [[8.0, 3.0, 3.0], [1.0, 4.0, 2.0], [1.0, 5.0, 6.0]]], [[[5.0, 4.0, 8.0], [5.0, 7.0, 7.0], [1.0, 7.0, 6.0]], [[2.0, 
5.0, 2.0], [6.0, 7.0, 5.0], [1.0, 4.0, 3.0]], [[8.0, 1.0, 8.0], [5.0, 3.0, 7.0], [1.0, 4.0, 5.0]]], [[[1.0, 6.0, 7.0], [1.0, 
6.0, 5.0], [2.0, 8.0, 4.0]], [[7.0, 2.0, 2.0], [2.0, 3.0, 1.0], [5.0, 1.0, 4.0]], [[1.0, 8.0, 4.0], [7.0, 8.0, 2.0], [6.0, 4.0, 
3.0]]], [[[7.0, 7.0, 1.0], [8.0, 7.0, 6.0], [4.0, 7.0, 6.0]], [[4.0, 4.0, 8.0], [1.0, 8.0, 1.0], [5.0, 7.0, 6.0]], [[8.0, 7.0, 
7.0], [1.0, 8.0, 3.0], [2.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 8.0], [7.0, 5.0, 8.0], [7.0, 1.0, 5.0]], 
[[8.0, 3.0, 3.0], [6.0, 4.0, 3.0], [4.0, 4.0, 2.0]], [[3.0, 2.0, 3.0], [8.0, 7.0, 1.0], [5.0, 8.0, 6.0]]], [[[7.0, 4.0, 4.0], 
[3.0, 4.0, 6.0], [3.0, 1.0, 8.0]], [[6.0, 2.0, 2.0], [6.0, 7.0, 5.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 6.0], [3.0, 1.0, 8.0], 
[4.0, 2.0, 7.0]]], [[[1.0, 2.0, 8.0], [3.0, 3.0, 1.0], [4.0, 8.0, 5.0]], [[7.0, 8.0, 5.0], [2.0, 3.0, 1.0], [7.0, 8.0, 7.0]], 
[[7.0, 1.0, 6.0], [2.0, 7.0, 6.0], [8.0, 6.0, 1.0]]], [[[7.0, 7.0, 1.0], [4.0, 7.0, 1.0], [2.0, 3.0, 5.0]], [[4.0, 2.0, 2.0], 
[5.0, 6.0, 1.0], [5.0, 4.0, 3.0]], [[7.0, 2.0, 7.0], [7.0, 2.0, 7.0], [4.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPPN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 3.0], [3.0, 4.0, 5.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [5.0, 8.0, 1.0], [3.0, 2.0, 
6.0]], [[3.0, 4.0, 8.0], [7.0, 1.0, 1.0], [3.0, 6.0, 1.0]]], [[[7.0, 4.0, 4.0], [8.0, 2.0, 3.0], [5.0, 2.0, 8.0]], [[7.0, 7.0, 
8.0], [3.0, 5.0, 3.0], [8.0, 7.0, 8.0]], [[7.0, 6.0, 5.0], [3.0, 6.0, 5.0], [6.0, 5.0, 1.0]]], [[[1.0, 2.0, 3.0], [7.0, 5.0, 
3.0], [1.0, 7.0, 7.0]], [[5.0, 3.0, 1.0], [7.0, 4.0, 8.0], [4.0, 6.0, 4.0]], [[4.0, 5.0, 5.0], [2.0, 2.0, 2.0], [8.0, 4.0, 
8.0]]], [[[1.0, 7.0, 4.0], [2.0, 6.0, 2.0], [4.0, 6.0, 7.0]], [[2.0, 2.0, 2.0], [3.0, 3.0, 3.0], [2.0, 4.0, 3.0]], [[5.0, 2.0, 
1.0], [8.0, 3.0, 5.0], [4.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 4.0], [7.0, 4.0, 4.0], [8.0, 8.0, 5.0]], [[1.0, 7.0, 8.0], [1.0, 5.0, 
5.0], [5.0, 5.0, 5.0]], [[3.0, 4.0, 3.0], [1.0, 2.0, 3.0], [7.0, 3.0, 5.0]]], [[[3.0, 3.0, 8.0], [1.0, 1.0, 2.0], [6.0, 7.0, 
8.0]], [[1.0, 6.0, 2.0], [2.0, 7.0, 2.0], [7.0, 5.0, 3.0]], [[4.0, 1.0, 1.0], [4.0, 6.0, 2.0], [6.0, 3.0, 2.0]]], [[[3.0, 5.0, 
6.0], [3.0, 1.0, 4.0], [1.0, 3.0, 5.0]], [[8.0, 8.0, 2.0], [4.0, 3.0, 5.0], [5.0, 4.0, 6.0]], [[7.0, 1.0, 4.0], [5.0, 3.0, 
7.0], [5.0, 4.0, 2.0]]], [[[3.0, 2.0, 2.0], [1.0, 3.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 1.0, 7.0], [4.0, 6.0, 7.0], [5.0, 2.0, 
6.0]], [[2.0, 8.0, 7.0], [1.0, 6.0, 1.0], [3.0, 3.0, 1.0]]]]))
      val=Data(numpy.array([[6.0, 5.0, 3.0], [4.0, 1.0, 8.0]]),self.functionspace)
      arg[1:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 3.0], [3.0, 4.0, 5.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [5.0, 8.0, 1.0], [3.0, 
2.0, 6.0]], [[3.0, 4.0, 8.0], [7.0, 1.0, 1.0], [3.0, 6.0, 1.0]]], [[[7.0, 4.0, 4.0], [8.0, 2.0, 3.0], [5.0, 2.0, 8.0]], [[7.0, 
7.0, 8.0], [6.0, 5.0, 3.0], [8.0, 7.0, 8.0]], [[7.0, 6.0, 5.0], [3.0, 6.0, 5.0], [6.0, 5.0, 1.0]]], [[[1.0, 2.0, 3.0], [7.0, 
5.0, 3.0], [1.0, 7.0, 7.0]], [[5.0, 3.0, 1.0], [4.0, 1.0, 8.0], [4.0, 6.0, 4.0]], [[4.0, 5.0, 5.0], [2.0, 2.0, 2.0], [8.0, 4.0, 
8.0]]], [[[1.0, 7.0, 4.0], [2.0, 6.0, 2.0], [4.0, 6.0, 7.0]], [[2.0, 2.0, 2.0], [3.0, 3.0, 3.0], [2.0, 4.0, 3.0]], [[5.0, 2.0, 
1.0], [8.0, 3.0, 5.0], [4.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 4.0], [7.0, 4.0, 4.0], [8.0, 8.0, 5.0]], 
[[1.0, 7.0, 8.0], [1.0, 5.0, 5.0], [5.0, 5.0, 5.0]], [[3.0, 4.0, 3.0], [1.0, 2.0, 3.0], [7.0, 3.0, 5.0]]], [[[3.0, 3.0, 8.0], 
[1.0, 1.0, 2.0], [6.0, 7.0, 8.0]], [[1.0, 6.0, 2.0], [6.0, 5.0, 3.0], [7.0, 5.0, 3.0]], [[4.0, 1.0, 1.0], [4.0, 6.0, 2.0], 
[6.0, 3.0, 2.0]]], [[[3.0, 5.0, 6.0], [3.0, 1.0, 4.0], [1.0, 3.0, 5.0]], [[8.0, 8.0, 2.0], [4.0, 1.0, 8.0], [5.0, 4.0, 6.0]], 
[[7.0, 1.0, 4.0], [5.0, 3.0, 7.0], [5.0, 4.0, 2.0]]], [[[3.0, 2.0, 2.0], [1.0, 3.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 1.0, 7.0], 
[4.0, 6.0, 7.0], [5.0, 2.0, 6.0]], [[2.0, 8.0, 7.0], [1.0, 6.0, 1.0], [3.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPPN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 4.0], [2.0, 7.0, 6.0], [2.0, 2.0, 6.0]], [[1.0, 8.0, 2.0], [5.0, 2.0, 6.0], [3.0, 1.0, 
5.0]], [[5.0, 5.0, 4.0], [6.0, 6.0, 6.0], [3.0, 5.0, 1.0]]], [[[1.0, 7.0, 6.0], [4.0, 5.0, 1.0], [2.0, 3.0, 4.0]], [[2.0, 2.0, 
8.0], [4.0, 5.0, 1.0], [7.0, 5.0, 6.0]], [[6.0, 7.0, 4.0], [1.0, 1.0, 3.0], [3.0, 5.0, 5.0]]], [[[4.0, 2.0, 3.0], [3.0, 3.0, 
5.0], [3.0, 6.0, 3.0]], [[4.0, 3.0, 3.0], [4.0, 5.0, 3.0], [1.0, 1.0, 8.0]], [[6.0, 2.0, 1.0], [6.0, 1.0, 5.0], [1.0, 5.0, 
3.0]]], [[[5.0, 8.0, 5.0], [6.0, 6.0, 1.0], [2.0, 6.0, 5.0]], [[2.0, 6.0, 6.0], [7.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[1.0, 3.0, 
2.0], [1.0, 8.0, 7.0], [3.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 6.0], [4.0, 2.0, 8.0], [3.0, 4.0, 2.0]], [[2.0, 6.0, 7.0], [7.0, 1.0, 
4.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 4.0], [1.0, 5.0, 4.0], [4.0, 6.0, 7.0]]], [[[8.0, 4.0, 2.0], [5.0, 5.0, 2.0], [4.0, 6.0, 
6.0]], [[8.0, 1.0, 4.0], [3.0, 4.0, 4.0], [8.0, 8.0, 4.0]], [[7.0, 7.0, 6.0], [8.0, 2.0, 4.0], [4.0, 7.0, 2.0]]], [[[3.0, 6.0, 
1.0], [7.0, 6.0, 6.0], [7.0, 6.0, 8.0]], [[2.0, 7.0, 4.0], [4.0, 8.0, 5.0], [8.0, 5.0, 6.0]], [[4.0, 3.0, 4.0], [1.0, 6.0, 
6.0], [4.0, 4.0, 2.0]]], [[[8.0, 6.0, 7.0], [4.0, 5.0, 2.0], [8.0, 2.0, 2.0]], [[7.0, 4.0, 3.0], [8.0, 1.0, 1.0], [3.0, 7.0, 
6.0]], [[4.0, 6.0, 3.0], [5.0, 2.0, 7.0], [2.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([[6.0, 8.0, 1.0], [8.0, 2.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 6.0, 6.0], [5.0, 5.0, 2.0]]))
      arg[1:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 4.0], [2.0, 7.0, 6.0], [2.0, 2.0, 6.0]], [[1.0, 8.0, 2.0], [5.0, 2.0, 6.0], [3.0, 
1.0, 5.0]], [[5.0, 5.0, 4.0], [6.0, 6.0, 6.0], [3.0, 5.0, 1.0]]], [[[1.0, 7.0, 6.0], [4.0, 5.0, 1.0], [2.0, 3.0, 4.0]], [[2.0, 
2.0, 8.0], [6.0, 8.0, 1.0], [7.0, 5.0, 6.0]], [[6.0, 7.0, 4.0], [1.0, 1.0, 3.0], [3.0, 5.0, 5.0]]], [[[4.0, 2.0, 3.0], [3.0, 
3.0, 5.0], [3.0, 6.0, 3.0]], [[4.0, 3.0, 3.0], [8.0, 2.0, 8.0], [1.0, 1.0, 8.0]], [[6.0, 2.0, 1.0], [6.0, 1.0, 5.0], [1.0, 5.0, 
3.0]]], [[[5.0, 8.0, 5.0], [6.0, 6.0, 1.0], [2.0, 6.0, 5.0]], [[2.0, 6.0, 6.0], [7.0, 4.0, 4.0], [1.0, 4.0, 3.0]], [[1.0, 3.0, 
2.0], [1.0, 8.0, 7.0], [3.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 6.0], [4.0, 2.0, 8.0], [3.0, 4.0, 2.0]], 
[[2.0, 6.0, 7.0], [7.0, 1.0, 4.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 4.0], [1.0, 5.0, 4.0], [4.0, 6.0, 7.0]]], [[[8.0, 4.0, 2.0], 
[5.0, 5.0, 2.0], [4.0, 6.0, 6.0]], [[8.0, 1.0, 4.0], [6.0, 6.0, 6.0], [8.0, 8.0, 4.0]], [[7.0, 7.0, 6.0], [8.0, 2.0, 4.0], 
[4.0, 7.0, 2.0]]], [[[3.0, 6.0, 1.0], [7.0, 6.0, 6.0], [7.0, 6.0, 8.0]], [[2.0, 7.0, 4.0], [5.0, 5.0, 2.0], [8.0, 5.0, 6.0]], 
[[4.0, 3.0, 4.0], [1.0, 6.0, 6.0], [4.0, 4.0, 2.0]]], [[[8.0, 6.0, 7.0], [4.0, 5.0, 2.0], [8.0, 2.0, 2.0]], [[7.0, 4.0, 3.0], 
[8.0, 1.0, 1.0], [3.0, 7.0, 6.0]], [[4.0, 6.0, 3.0], [5.0, 2.0, 7.0], [2.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPPN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 5.0], [1.0, 7.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 4.0, 5.0], [6.0, 5.0, 2.0], [5.0, 8.0, 
6.0]], [[1.0, 8.0, 5.0], [8.0, 3.0, 4.0], [4.0, 1.0, 4.0]]], [[[1.0, 7.0, 1.0], [6.0, 8.0, 2.0], [2.0, 7.0, 5.0]], [[6.0, 4.0, 
1.0], [3.0, 5.0, 7.0], [1.0, 7.0, 8.0]], [[1.0, 3.0, 2.0], [6.0, 8.0, 2.0], [4.0, 4.0, 4.0]]], [[[1.0, 3.0, 5.0], [1.0, 6.0, 
6.0], [3.0, 1.0, 2.0]], [[7.0, 8.0, 3.0], [4.0, 4.0, 5.0], [2.0, 7.0, 2.0]], [[6.0, 6.0, 7.0], [5.0, 1.0, 5.0], [3.0, 3.0, 
7.0]]], [[[4.0, 8.0, 7.0], [3.0, 1.0, 3.0], [3.0, 7.0, 6.0]], [[5.0, 1.0, 1.0], [7.0, 1.0, 7.0], [7.0, 2.0, 2.0]], [[1.0, 2.0, 
5.0], [6.0, 2.0, 2.0], [1.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 7.0], [1.0, 6.0, 6.0], [7.0, 8.0, 7.0]], [[7.0, 8.0, 3.0], [3.0, 8.0, 
2.0], [5.0, 7.0, 5.0]], [[4.0, 7.0, 8.0], [8.0, 1.0, 3.0], [4.0, 7.0, 1.0]]], [[[4.0, 7.0, 6.0], [8.0, 1.0, 7.0], [7.0, 8.0, 
5.0]], [[5.0, 5.0, 5.0], [1.0, 7.0, 2.0], [4.0, 7.0, 2.0]], [[5.0, 5.0, 3.0], [2.0, 7.0, 7.0], [8.0, 2.0, 3.0]]], [[[4.0, 1.0, 
6.0], [3.0, 6.0, 7.0], [6.0, 2.0, 2.0]], [[5.0, 2.0, 1.0], [8.0, 7.0, 8.0], [3.0, 5.0, 6.0]], [[1.0, 6.0, 6.0], [2.0, 3.0, 
3.0], [5.0, 5.0, 5.0]]], [[[1.0, 8.0, 7.0], [8.0, 4.0, 6.0], [4.0, 4.0, 6.0]], [[6.0, 1.0, 2.0], [5.0, 8.0, 6.0], [6.0, 3.0, 
1.0]], [[1.0, 2.0, 2.0], [3.0, 8.0, 5.0], [2.0, 2.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[1.0, 7.0, 3.0], [8.0, 8.0, 6.0]])+(1.-msk_val)*numpy.array([[6.0, 2.0, 6.0], [6.0, 8.0, 5.0]])
      arg[1:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 5.0], [1.0, 7.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 4.0, 5.0], [6.0, 5.0, 2.0], [5.0, 
8.0, 6.0]], [[1.0, 8.0, 5.0], [8.0, 3.0, 4.0], [4.0, 1.0, 4.0]]], [[[1.0, 7.0, 1.0], [6.0, 8.0, 2.0], [2.0, 7.0, 5.0]], [[6.0, 
4.0, 1.0], [1.0, 7.0, 3.0], [1.0, 7.0, 8.0]], [[1.0, 3.0, 2.0], [6.0, 8.0, 2.0], [4.0, 4.0, 4.0]]], [[[1.0, 3.0, 5.0], [1.0, 
6.0, 6.0], [3.0, 1.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 8.0, 6.0], [2.0, 7.0, 2.0]], [[6.0, 6.0, 7.0], [5.0, 1.0, 5.0], [3.0, 3.0, 
7.0]]], [[[4.0, 8.0, 7.0], [3.0, 1.0, 3.0], [3.0, 7.0, 6.0]], [[5.0, 1.0, 1.0], [7.0, 1.0, 7.0], [7.0, 2.0, 2.0]], [[1.0, 2.0, 
5.0], [6.0, 2.0, 2.0], [1.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 7.0], [1.0, 6.0, 6.0], [7.0, 8.0, 7.0]], 
[[7.0, 8.0, 3.0], [3.0, 8.0, 2.0], [5.0, 7.0, 5.0]], [[4.0, 7.0, 8.0], [8.0, 1.0, 3.0], [4.0, 7.0, 1.0]]], [[[4.0, 7.0, 6.0], 
[8.0, 1.0, 7.0], [7.0, 8.0, 5.0]], [[5.0, 5.0, 5.0], [6.0, 2.0, 6.0], [4.0, 7.0, 2.0]], [[5.0, 5.0, 3.0], [2.0, 7.0, 7.0], 
[8.0, 2.0, 3.0]]], [[[4.0, 1.0, 6.0], [3.0, 6.0, 7.0], [6.0, 2.0, 2.0]], [[5.0, 2.0, 1.0], [6.0, 8.0, 5.0], [3.0, 5.0, 6.0]], 
[[1.0, 6.0, 6.0], [2.0, 3.0, 3.0], [5.0, 5.0, 5.0]]], [[[1.0, 8.0, 7.0], [8.0, 4.0, 6.0], [4.0, 4.0, 6.0]], [[6.0, 1.0, 2.0], 
[5.0, 8.0, 6.0], [6.0, 3.0, 1.0]], [[1.0, 2.0, 2.0], [3.0, 8.0, 5.0], [2.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LCUN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 1.0], [3.0, 6.0, 7.0], [3.0, 5.0, 1.0]], [[3.0, 7.0, 1.0], [2.0, 6.0, 2.0], [7.0, 5.0, 
6.0]], [[6.0, 4.0, 5.0], [7.0, 8.0, 8.0], [6.0, 2.0, 3.0]]], [[[4.0, 1.0, 5.0], [8.0, 5.0, 2.0], [1.0, 1.0, 4.0]], [[3.0, 1.0, 
8.0], [1.0, 8.0, 7.0], [7.0, 2.0, 3.0]], [[6.0, 5.0, 2.0], [7.0, 8.0, 3.0], [4.0, 6.0, 8.0]]], [[[7.0, 4.0, 2.0], [5.0, 1.0, 
1.0], [7.0, 5.0, 2.0]], [[3.0, 5.0, 4.0], [8.0, 2.0, 8.0], [3.0, 2.0, 5.0]], [[4.0, 2.0, 4.0], [2.0, 8.0, 1.0], [6.0, 1.0, 
3.0]]], [[[2.0, 3.0, 1.0], [6.0, 1.0, 8.0], [5.0, 4.0, 4.0]], [[2.0, 6.0, 4.0], [6.0, 7.0, 7.0], [5.0, 8.0, 4.0]], [[4.0, 5.0, 
3.0], [3.0, 5.0, 6.0], [3.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 6.0], [5.0, 6.0, 2.0], [1.0, 8.0, 7.0]], [[1.0, 6.0, 8.0], [8.0, 1.0, 
8.0], [2.0, 4.0, 2.0]], [[6.0, 5.0, 8.0], [3.0, 4.0, 8.0], [7.0, 8.0, 4.0]]], [[[7.0, 4.0, 7.0], [8.0, 3.0, 6.0], [7.0, 7.0, 
7.0]], [[8.0, 6.0, 5.0], [8.0, 3.0, 6.0], [1.0, 7.0, 6.0]], [[8.0, 6.0, 4.0], [5.0, 4.0, 8.0], [2.0, 8.0, 5.0]]], [[[7.0, 1.0, 
3.0], [4.0, 1.0, 4.0], [6.0, 7.0, 2.0]], [[5.0, 8.0, 2.0], [4.0, 3.0, 7.0], [1.0, 4.0, 6.0]], [[1.0, 5.0, 4.0], [8.0, 2.0, 
2.0], [6.0, 2.0, 2.0]]], [[[3.0, 3.0, 6.0], [3.0, 6.0, 1.0], [7.0, 7.0, 7.0]], [[1.0, 7.0, 7.0], [3.0, 2.0, 3.0], [2.0, 4.0, 
6.0]], [[2.0, 3.0, 1.0], [5.0, 8.0, 8.0], [7.0, 2.0, 2.0]]]]))
      val=numpy.array([[[[5.0, 6.0, 1.0], [8.0, 8.0, 2.0]], [[5.0, 6.0, 1.0], [4.0, 8.0, 8.0]], [[7.0, 2.0, 5.0], [6.0, 2.0, 
1.0]]], [[[3.0, 4.0, 2.0], [7.0, 1.0, 5.0]], [[7.0, 6.0, 8.0], [1.0, 5.0, 7.0]], [[1.0, 3.0, 2.0], [4.0, 6.0, 7.0]]]])
      arg[2:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 1.0], [3.0, 6.0, 7.0], [3.0, 5.0, 1.0]], [[3.0, 7.0, 1.0], [2.0, 6.0, 2.0], [7.0, 
5.0, 6.0]], [[6.0, 4.0, 5.0], [7.0, 8.0, 8.0], [6.0, 2.0, 3.0]]], [[[4.0, 1.0, 5.0], [8.0, 5.0, 2.0], [1.0, 1.0, 4.0]], [[3.0, 
1.0, 8.0], [1.0, 8.0, 7.0], [7.0, 2.0, 3.0]], [[6.0, 5.0, 2.0], [7.0, 8.0, 3.0], [4.0, 6.0, 8.0]]], [[[5.0, 6.0, 1.0], [8.0, 
8.0, 2.0], [7.0, 5.0, 2.0]], [[5.0, 6.0, 1.0], [4.0, 8.0, 8.0], [3.0, 2.0, 5.0]], [[7.0, 2.0, 5.0], [6.0, 2.0, 1.0], [6.0, 1.0, 
3.0]]], [[[3.0, 4.0, 2.0], [7.0, 1.0, 5.0], [5.0, 4.0, 4.0]], [[7.0, 6.0, 8.0], [1.0, 5.0, 7.0], [5.0, 8.0, 4.0]], [[1.0, 3.0, 
2.0], [4.0, 6.0, 7.0], [3.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 6.0], [5.0, 6.0, 2.0], [1.0, 8.0, 7.0]], 
[[1.0, 6.0, 8.0], [8.0, 1.0, 8.0], [2.0, 4.0, 2.0]], [[6.0, 5.0, 8.0], [3.0, 4.0, 8.0], [7.0, 8.0, 4.0]]], [[[7.0, 4.0, 7.0], 
[8.0, 3.0, 6.0], [7.0, 7.0, 7.0]], [[8.0, 6.0, 5.0], [8.0, 3.0, 6.0], [1.0, 7.0, 6.0]], [[8.0, 6.0, 4.0], [5.0, 4.0, 8.0], 
[2.0, 8.0, 5.0]]], [[[5.0, 6.0, 1.0], [8.0, 8.0, 2.0], [6.0, 7.0, 2.0]], [[5.0, 6.0, 1.0], [4.0, 8.0, 8.0], [1.0, 4.0, 6.0]], 
[[7.0, 2.0, 5.0], [6.0, 2.0, 1.0], [6.0, 2.0, 2.0]]], [[[3.0, 4.0, 2.0], [7.0, 1.0, 5.0], [7.0, 7.0, 7.0]], [[7.0, 6.0, 8.0], 
[1.0, 5.0, 7.0], [2.0, 4.0, 6.0]], [[1.0, 3.0, 2.0], [4.0, 6.0, 7.0], [7.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LCUN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 2.0], [3.0, 8.0, 6.0], [2.0, 1.0, 4.0]], [[4.0, 8.0, 1.0], [7.0, 4.0, 1.0], [8.0, 6.0, 
3.0]], [[4.0, 8.0, 3.0], [5.0, 6.0, 6.0], [5.0, 4.0, 2.0]]], [[[4.0, 2.0, 7.0], [8.0, 7.0, 7.0], [4.0, 2.0, 7.0]], [[1.0, 8.0, 
5.0], [4.0, 8.0, 4.0], [4.0, 3.0, 3.0]], [[8.0, 2.0, 8.0], [3.0, 6.0, 1.0], [2.0, 8.0, 5.0]]], [[[4.0, 5.0, 8.0], [7.0, 4.0, 
8.0], [4.0, 8.0, 7.0]], [[3.0, 3.0, 6.0], [7.0, 2.0, 2.0], [7.0, 7.0, 1.0]], [[2.0, 8.0, 3.0], [5.0, 7.0, 7.0], [1.0, 7.0, 
4.0]]], [[[3.0, 5.0, 6.0], [6.0, 2.0, 1.0], [1.0, 2.0, 1.0]], [[1.0, 3.0, 1.0], [4.0, 3.0, 1.0], [4.0, 1.0, 2.0]], [[8.0, 7.0, 
4.0], [4.0, 3.0, 7.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 7.0], [5.0, 8.0, 3.0], [4.0, 2.0, 3.0]], [[5.0, 3.0, 6.0], [3.0, 8.0, 
5.0], [5.0, 7.0, 1.0]], [[8.0, 4.0, 1.0], [7.0, 3.0, 2.0], [3.0, 6.0, 1.0]]], [[[7.0, 8.0, 1.0], [1.0, 5.0, 2.0], [5.0, 4.0, 
8.0]], [[2.0, 6.0, 4.0], [6.0, 2.0, 8.0], [8.0, 8.0, 2.0]], [[1.0, 2.0, 7.0], [6.0, 2.0, 2.0], [4.0, 7.0, 2.0]]], [[[1.0, 3.0, 
1.0], [2.0, 1.0, 6.0], [3.0, 3.0, 5.0]], [[1.0, 2.0, 6.0], [3.0, 6.0, 6.0], [4.0, 2.0, 7.0]], [[5.0, 3.0, 3.0], [4.0, 2.0, 
2.0], [5.0, 6.0, 8.0]]], [[[1.0, 4.0, 3.0], [4.0, 1.0, 7.0], [7.0, 8.0, 8.0]], [[2.0, 2.0, 7.0], [3.0, 8.0, 3.0], [6.0, 7.0, 
7.0]], [[2.0, 6.0, 7.0], [2.0, 4.0, 8.0], [2.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[[[7.0, 8.0, 2.0], [6.0, 7.0, 3.0]], [[6.0, 5.0, 7.0], [2.0, 4.0, 4.0]], [[2.0, 1.0, 2.0], [8.0, 
2.0, 7.0]]], [[[4.0, 4.0, 1.0], [8.0, 1.0, 5.0]], [[3.0, 2.0, 7.0], [2.0, 7.0, 2.0]], [[1.0, 6.0, 2.0], [8.0, 3.0, 
3.0]]]]),self.functionspace)
      arg[2:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 2.0], [3.0, 8.0, 6.0], [2.0, 1.0, 4.0]], [[4.0, 8.0, 1.0], [7.0, 4.0, 1.0], [8.0, 
6.0, 3.0]], [[4.0, 8.0, 3.0], [5.0, 6.0, 6.0], [5.0, 4.0, 2.0]]], [[[4.0, 2.0, 7.0], [8.0, 7.0, 7.0], [4.0, 2.0, 7.0]], [[1.0, 
8.0, 5.0], [4.0, 8.0, 4.0], [4.0, 3.0, 3.0]], [[8.0, 2.0, 8.0], [3.0, 6.0, 1.0], [2.0, 8.0, 5.0]]], [[[7.0, 8.0, 2.0], [6.0, 
7.0, 3.0], [4.0, 8.0, 7.0]], [[6.0, 5.0, 7.0], [2.0, 4.0, 4.0], [7.0, 7.0, 1.0]], [[2.0, 1.0, 2.0], [8.0, 2.0, 7.0], [1.0, 7.0, 
4.0]]], [[[4.0, 4.0, 1.0], [8.0, 1.0, 5.0], [1.0, 2.0, 1.0]], [[3.0, 2.0, 7.0], [2.0, 7.0, 2.0], [4.0, 1.0, 2.0]], [[1.0, 6.0, 
2.0], [8.0, 3.0, 3.0], [4.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 7.0], [5.0, 8.0, 3.0], [4.0, 2.0, 3.0]], 
[[5.0, 3.0, 6.0], [3.0, 8.0, 5.0], [5.0, 7.0, 1.0]], [[8.0, 4.0, 1.0], [7.0, 3.0, 2.0], [3.0, 6.0, 1.0]]], [[[7.0, 8.0, 1.0], 
[1.0, 5.0, 2.0], [5.0, 4.0, 8.0]], [[2.0, 6.0, 4.0], [6.0, 2.0, 8.0], [8.0, 8.0, 2.0]], [[1.0, 2.0, 7.0], [6.0, 2.0, 2.0], 
[4.0, 7.0, 2.0]]], [[[7.0, 8.0, 2.0], [6.0, 7.0, 3.0], [3.0, 3.0, 5.0]], [[6.0, 5.0, 7.0], [2.0, 4.0, 4.0], [4.0, 2.0, 7.0]], 
[[2.0, 1.0, 2.0], [8.0, 2.0, 7.0], [5.0, 6.0, 8.0]]], [[[4.0, 4.0, 1.0], [8.0, 1.0, 5.0], [7.0, 8.0, 8.0]], [[3.0, 2.0, 7.0], 
[2.0, 7.0, 2.0], [6.0, 7.0, 7.0]], [[1.0, 6.0, 2.0], [8.0, 3.0, 3.0], [2.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LCUN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 2.0], [4.0, 7.0, 8.0], [8.0, 4.0, 6.0]], [[3.0, 2.0, 1.0], [6.0, 4.0, 6.0], [1.0, 7.0, 
7.0]], [[7.0, 1.0, 2.0], [4.0, 1.0, 6.0], [1.0, 3.0, 7.0]]], [[[3.0, 5.0, 7.0], [6.0, 1.0, 7.0], [8.0, 8.0, 7.0]], [[8.0, 2.0, 
6.0], [3.0, 3.0, 7.0], [8.0, 5.0, 1.0]], [[2.0, 6.0, 8.0], [5.0, 2.0, 8.0], [3.0, 1.0, 7.0]]], [[[1.0, 5.0, 5.0], [7.0, 3.0, 
7.0], [4.0, 7.0, 3.0]], [[5.0, 8.0, 7.0], [8.0, 1.0, 3.0], [7.0, 6.0, 6.0]], [[5.0, 6.0, 6.0], [7.0, 1.0, 8.0], [1.0, 2.0, 
1.0]]], [[[3.0, 8.0, 2.0], [6.0, 5.0, 8.0], [6.0, 6.0, 4.0]], [[1.0, 3.0, 4.0], [4.0, 2.0, 4.0], [6.0, 2.0, 6.0]], [[8.0, 4.0, 
4.0], [5.0, 4.0, 3.0], [2.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 2.0], [8.0, 6.0, 7.0], [5.0, 7.0, 8.0]], [[4.0, 5.0, 4.0], [2.0, 3.0, 
2.0], [7.0, 7.0, 8.0]], [[6.0, 6.0, 3.0], [3.0, 1.0, 5.0], [1.0, 7.0, 5.0]]], [[[8.0, 1.0, 1.0], [5.0, 5.0, 6.0], [3.0, 6.0, 
2.0]], [[2.0, 5.0, 7.0], [1.0, 6.0, 3.0], [7.0, 6.0, 4.0]], [[8.0, 5.0, 3.0], [1.0, 2.0, 1.0], [3.0, 2.0, 1.0]]], [[[7.0, 1.0, 
8.0], [8.0, 1.0, 6.0], [3.0, 4.0, 1.0]], [[5.0, 6.0, 7.0], [4.0, 2.0, 2.0], [2.0, 6.0, 1.0]], [[4.0, 3.0, 7.0], [6.0, 4.0, 
5.0], [7.0, 4.0, 6.0]]], [[[6.0, 4.0, 2.0], [5.0, 5.0, 7.0], [5.0, 8.0, 4.0]], [[5.0, 4.0, 1.0], [7.0, 1.0, 8.0], [3.0, 4.0, 
3.0]], [[6.0, 7.0, 7.0], [1.0, 5.0, 3.0], [3.0, 6.0, 6.0]]]]))
      val=Data(numpy.array([[[[5.0, 2.0, 3.0], [5.0, 4.0, 2.0]], [[8.0, 4.0, 7.0], [5.0, 5.0, 8.0]], [[3.0, 4.0, 2.0], [3.0, 
6.0, 5.0]]], [[[4.0, 1.0, 6.0], [1.0, 1.0, 3.0]], [[8.0, 7.0, 1.0], [2.0, 6.0, 1.0]], [[2.0, 8.0, 5.0], [5.0, 3.0, 
5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 4.0], [7.0, 8.0, 2.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 2.0]], [[1.0, 4.0, 
7.0], [7.0, 2.0, 7.0]]], [[[2.0, 2.0, 2.0], [3.0, 4.0, 8.0]], [[3.0, 6.0, 8.0], [1.0, 6.0, 1.0]], [[7.0, 7.0, 1.0], [4.0, 7.0, 
3.0]]]]))
      arg[2:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 2.0], [4.0, 7.0, 8.0], [8.0, 4.0, 6.0]], [[3.0, 2.0, 1.0], [6.0, 4.0, 6.0], [1.0, 
7.0, 7.0]], [[7.0, 1.0, 2.0], [4.0, 1.0, 6.0], [1.0, 3.0, 7.0]]], [[[3.0, 5.0, 7.0], [6.0, 1.0, 7.0], [8.0, 8.0, 7.0]], [[8.0, 
2.0, 6.0], [3.0, 3.0, 7.0], [8.0, 5.0, 1.0]], [[2.0, 6.0, 8.0], [5.0, 2.0, 8.0], [3.0, 1.0, 7.0]]], [[[5.0, 2.0, 3.0], [5.0, 
4.0, 2.0], [4.0, 7.0, 3.0]], [[8.0, 4.0, 7.0], [5.0, 5.0, 8.0], [7.0, 6.0, 6.0]], [[3.0, 4.0, 2.0], [3.0, 6.0, 5.0], [1.0, 2.0, 
1.0]]], [[[4.0, 1.0, 6.0], [1.0, 1.0, 3.0], [6.0, 6.0, 4.0]], [[8.0, 7.0, 1.0], [2.0, 6.0, 1.0], [6.0, 2.0, 6.0]], [[2.0, 8.0, 
5.0], [5.0, 3.0, 5.0], [2.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 2.0], [8.0, 6.0, 7.0], [5.0, 7.0, 8.0]], 
[[4.0, 5.0, 4.0], [2.0, 3.0, 2.0], [7.0, 7.0, 8.0]], [[6.0, 6.0, 3.0], [3.0, 1.0, 5.0], [1.0, 7.0, 5.0]]], [[[8.0, 1.0, 1.0], 
[5.0, 5.0, 6.0], [3.0, 6.0, 2.0]], [[2.0, 5.0, 7.0], [1.0, 6.0, 3.0], [7.0, 6.0, 4.0]], [[8.0, 5.0, 3.0], [1.0, 2.0, 1.0], 
[3.0, 2.0, 1.0]]], [[[1.0, 7.0, 4.0], [7.0, 8.0, 2.0], [3.0, 4.0, 1.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 2.0], [2.0, 6.0, 1.0]], 
[[1.0, 4.0, 7.0], [7.0, 2.0, 7.0], [7.0, 4.0, 6.0]]], [[[2.0, 2.0, 2.0], [3.0, 4.0, 8.0], [5.0, 8.0, 4.0]], [[3.0, 6.0, 8.0], 
[1.0, 6.0, 1.0], [3.0, 4.0, 3.0]], [[7.0, 7.0, 1.0], [4.0, 7.0, 3.0], [3.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LCUN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 8.0], [3.0, 2.0, 2.0], [5.0, 1.0, 2.0]], [[6.0, 7.0, 1.0], [8.0, 1.0, 7.0], [1.0, 1.0, 
8.0]], [[3.0, 3.0, 6.0], [3.0, 6.0, 3.0], [8.0, 2.0, 6.0]]], [[[7.0, 7.0, 1.0], [7.0, 8.0, 4.0], [1.0, 2.0, 5.0]], [[5.0, 5.0, 
1.0], [7.0, 4.0, 7.0], [4.0, 2.0, 3.0]], [[4.0, 2.0, 3.0], [3.0, 8.0, 1.0], [8.0, 6.0, 8.0]]], [[[3.0, 8.0, 7.0], [2.0, 8.0, 
3.0], [7.0, 6.0, 4.0]], [[3.0, 1.0, 3.0], [7.0, 6.0, 8.0], [6.0, 8.0, 7.0]], [[2.0, 7.0, 1.0], [3.0, 3.0, 4.0], [3.0, 4.0, 
7.0]]], [[[6.0, 1.0, 6.0], [3.0, 7.0, 5.0], [5.0, 3.0, 7.0]], [[4.0, 1.0, 5.0], [1.0, 2.0, 2.0], [1.0, 1.0, 5.0]], [[7.0, 5.0, 
4.0], [4.0, 3.0, 6.0], [4.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 4.0], [3.0, 6.0, 5.0], [2.0, 3.0, 6.0]], [[5.0, 7.0, 2.0], [2.0, 5.0, 
5.0], [4.0, 3.0, 6.0]], [[5.0, 2.0, 8.0], [2.0, 5.0, 5.0], [3.0, 7.0, 1.0]]], [[[3.0, 5.0, 4.0], [1.0, 5.0, 3.0], [2.0, 4.0, 
6.0]], [[5.0, 2.0, 8.0], [3.0, 4.0, 8.0], [4.0, 2.0, 7.0]], [[2.0, 3.0, 3.0], [4.0, 1.0, 1.0], [3.0, 7.0, 5.0]]], [[[8.0, 5.0, 
4.0], [2.0, 2.0, 6.0], [7.0, 3.0, 5.0]], [[8.0, 8.0, 3.0], [6.0, 6.0, 1.0], [6.0, 1.0, 1.0]], [[3.0, 8.0, 3.0], [8.0, 2.0, 
4.0], [6.0, 8.0, 6.0]]], [[[7.0, 2.0, 7.0], [3.0, 4.0, 2.0], [4.0, 4.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 6.0, 6.0], [2.0, 6.0, 
3.0]], [[2.0, 1.0, 5.0], [3.0, 3.0, 7.0], [8.0, 1.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0, 6.0, 1.0], [8.0, 6.0, 5.0]], [[2.0, 3.0, 4.0], [5.0, 4.0, 7.0]], [[7.0, 8.0, 1.0], [7.0, 
1.0, 6.0]]], [[[2.0, 2.0, 3.0], [4.0, 5.0, 7.0]], [[5.0, 8.0, 2.0], [6.0, 2.0, 8.0]], [[8.0, 6.0, 4.0], [6.0, 6.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 3.0, 5.0], [5.0, 6.0, 4.0]], [[1.0, 7.0, 8.0], [6.0, 5.0, 6.0]], [[4.0, 6.0, 1.0], 
[7.0, 2.0, 6.0]]], [[[6.0, 8.0, 8.0], [3.0, 5.0, 5.0]], [[2.0, 8.0, 3.0], [2.0, 6.0, 6.0]], [[6.0, 6.0, 8.0], [1.0, 3.0, 
4.0]]]])
      arg[2:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 8.0], [3.0, 2.0, 2.0], [5.0, 1.0, 2.0]], [[6.0, 7.0, 1.0], [8.0, 1.0, 7.0], [1.0, 
1.0, 8.0]], [[3.0, 3.0, 6.0], [3.0, 6.0, 3.0], [8.0, 2.0, 6.0]]], [[[7.0, 7.0, 1.0], [7.0, 8.0, 4.0], [1.0, 2.0, 5.0]], [[5.0, 
5.0, 1.0], [7.0, 4.0, 7.0], [4.0, 2.0, 3.0]], [[4.0, 2.0, 3.0], [3.0, 8.0, 1.0], [8.0, 6.0, 8.0]]], [[[7.0, 6.0, 1.0], [8.0, 
6.0, 5.0], [7.0, 6.0, 4.0]], [[2.0, 3.0, 4.0], [5.0, 4.0, 7.0], [6.0, 8.0, 7.0]], [[7.0, 8.0, 1.0], [7.0, 1.0, 6.0], [3.0, 4.0, 
7.0]]], [[[2.0, 2.0, 3.0], [4.0, 5.0, 7.0], [5.0, 3.0, 7.0]], [[5.0, 8.0, 2.0], [6.0, 2.0, 8.0], [1.0, 1.0, 5.0]], [[8.0, 6.0, 
4.0], [6.0, 6.0, 7.0], [4.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 4.0], [3.0, 6.0, 5.0], [2.0, 3.0, 6.0]], 
[[5.0, 7.0, 2.0], [2.0, 5.0, 5.0], [4.0, 3.0, 6.0]], [[5.0, 2.0, 8.0], [2.0, 5.0, 5.0], [3.0, 7.0, 1.0]]], [[[3.0, 5.0, 4.0], 
[1.0, 5.0, 3.0], [2.0, 4.0, 6.0]], [[5.0, 2.0, 8.0], [3.0, 4.0, 8.0], [4.0, 2.0, 7.0]], [[2.0, 3.0, 3.0], [4.0, 1.0, 1.0], 
[3.0, 7.0, 5.0]]], [[[1.0, 3.0, 5.0], [5.0, 6.0, 4.0], [7.0, 3.0, 5.0]], [[1.0, 7.0, 8.0], [6.0, 5.0, 6.0], [6.0, 1.0, 1.0]], 
[[4.0, 6.0, 1.0], [7.0, 2.0, 6.0], [6.0, 8.0, 6.0]]], [[[6.0, 8.0, 8.0], [3.0, 5.0, 5.0], [4.0, 4.0, 6.0]], [[2.0, 8.0, 3.0], 
[2.0, 6.0, 6.0], [2.0, 6.0, 3.0]], [[6.0, 6.0, 8.0], [1.0, 3.0, 4.0], [8.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LCCU(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 4.0], [2.0, 7.0, 3.0], [4.0, 3.0, 3.0]], [[5.0, 2.0, 1.0], [1.0, 6.0, 8.0], [5.0, 1.0, 
5.0]], [[4.0, 7.0, 5.0], [7.0, 4.0, 7.0], [7.0, 4.0, 1.0]]], [[[6.0, 6.0, 3.0], [4.0, 7.0, 5.0], [7.0, 4.0, 6.0]], [[1.0, 5.0, 
5.0], [4.0, 5.0, 6.0], [5.0, 6.0, 2.0]], [[1.0, 6.0, 5.0], [2.0, 2.0, 1.0], [7.0, 5.0, 8.0]]], [[[3.0, 5.0, 4.0], [6.0, 4.0, 
2.0], [3.0, 4.0, 3.0]], [[1.0, 7.0, 2.0], [5.0, 2.0, 5.0], [2.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], [2.0, 6.0, 1.0], [1.0, 8.0, 
4.0]]], [[[1.0, 8.0, 8.0], [6.0, 7.0, 1.0], [3.0, 8.0, 8.0]], [[3.0, 3.0, 5.0], [3.0, 5.0, 6.0], [4.0, 5.0, 6.0]], [[4.0, 2.0, 
6.0], [6.0, 8.0, 7.0], [4.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 5.0], [5.0, 5.0, 2.0], [1.0, 3.0, 1.0]], [[2.0, 3.0, 5.0], [7.0, 7.0, 
2.0], [5.0, 4.0, 2.0]], [[1.0, 1.0, 2.0], [8.0, 2.0, 5.0], [2.0, 6.0, 2.0]]], [[[4.0, 2.0, 4.0], [4.0, 4.0, 8.0], [3.0, 2.0, 
6.0]], [[1.0, 8.0, 2.0], [7.0, 6.0, 4.0], [8.0, 4.0, 5.0]], [[8.0, 1.0, 4.0], [7.0, 8.0, 8.0], [5.0, 3.0, 2.0]]], [[[3.0, 6.0, 
5.0], [1.0, 5.0, 3.0], [3.0, 1.0, 8.0]], [[5.0, 1.0, 5.0], [4.0, 4.0, 6.0], [7.0, 3.0, 8.0]], [[2.0, 8.0, 8.0], [5.0, 4.0, 
4.0], [2.0, 4.0, 2.0]]], [[[4.0, 2.0, 7.0], [3.0, 5.0, 8.0], [5.0, 2.0, 7.0]], [[1.0, 3.0, 6.0], [5.0, 4.0, 8.0], [4.0, 4.0, 
7.0]], [[8.0, 5.0, 2.0], [2.0, 4.0, 7.0], [8.0, 4.0, 8.0]]]]))
      val=numpy.array([[[[1.0, 6.0], [5.0, 1.0], [6.0, 3.0]], [[8.0, 8.0], [1.0, 4.0], [1.0, 1.0]], [[8.0, 6.0], [5.0, 8.0], 
[1.0, 1.0]]], [[[2.0, 4.0], [4.0, 2.0], [1.0, 2.0]], [[3.0, 7.0], [7.0, 6.0], [3.0, 6.0]], [[1.0, 8.0], [7.0, 2.0], [8.0, 
6.0]]]])
      arg[2:,:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 4.0], [2.0, 7.0, 3.0], [4.0, 3.0, 3.0]], [[5.0, 2.0, 1.0], [1.0, 6.0, 8.0], [5.0, 
1.0, 5.0]], [[4.0, 7.0, 5.0], [7.0, 4.0, 7.0], [7.0, 4.0, 1.0]]], [[[6.0, 6.0, 3.0], [4.0, 7.0, 5.0], [7.0, 4.0, 6.0]], [[1.0, 
5.0, 5.0], [4.0, 5.0, 6.0], [5.0, 6.0, 2.0]], [[1.0, 6.0, 5.0], [2.0, 2.0, 1.0], [7.0, 5.0, 8.0]]], [[[1.0, 6.0, 4.0], [5.0, 
1.0, 2.0], [6.0, 3.0, 3.0]], [[8.0, 8.0, 2.0], [1.0, 4.0, 5.0], [1.0, 1.0, 5.0]], [[8.0, 6.0, 7.0], [5.0, 8.0, 1.0], [1.0, 1.0, 
4.0]]], [[[2.0, 4.0, 8.0], [4.0, 2.0, 1.0], [1.0, 2.0, 8.0]], [[3.0, 7.0, 5.0], [7.0, 6.0, 6.0], [3.0, 6.0, 6.0]], [[1.0, 8.0, 
6.0], [7.0, 2.0, 7.0], [8.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 5.0], [5.0, 5.0, 2.0], [1.0, 3.0, 1.0]], 
[[2.0, 3.0, 5.0], [7.0, 7.0, 2.0], [5.0, 4.0, 2.0]], [[1.0, 1.0, 2.0], [8.0, 2.0, 5.0], [2.0, 6.0, 2.0]]], [[[4.0, 2.0, 4.0], 
[4.0, 4.0, 8.0], [3.0, 2.0, 6.0]], [[1.0, 8.0, 2.0], [7.0, 6.0, 4.0], [8.0, 4.0, 5.0]], [[8.0, 1.0, 4.0], [7.0, 8.0, 8.0], 
[5.0, 3.0, 2.0]]], [[[1.0, 6.0, 5.0], [5.0, 1.0, 3.0], [6.0, 3.0, 8.0]], [[8.0, 8.0, 5.0], [1.0, 4.0, 6.0], [1.0, 1.0, 8.0]], 
[[8.0, 6.0, 8.0], [5.0, 8.0, 4.0], [1.0, 1.0, 2.0]]], [[[2.0, 4.0, 7.0], [4.0, 2.0, 8.0], [1.0, 2.0, 7.0]], [[3.0, 7.0, 6.0], 
[7.0, 6.0, 8.0], [3.0, 6.0, 7.0]], [[1.0, 8.0, 2.0], [7.0, 2.0, 7.0], [8.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LCCU(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 8.0], [5.0, 2.0, 1.0], [5.0, 4.0, 1.0]], [[3.0, 7.0, 1.0], [4.0, 8.0, 6.0], [4.0, 8.0, 
3.0]], [[4.0, 4.0, 5.0], [4.0, 5.0, 3.0], [8.0, 2.0, 3.0]]], [[[3.0, 1.0, 6.0], [2.0, 4.0, 5.0], [7.0, 5.0, 8.0]], [[1.0, 8.0, 
1.0], [8.0, 5.0, 4.0], [4.0, 6.0, 8.0]], [[1.0, 4.0, 3.0], [1.0, 4.0, 2.0], [6.0, 6.0, 8.0]]], [[[6.0, 4.0, 8.0], [8.0, 8.0, 
3.0], [8.0, 4.0, 3.0]], [[7.0, 2.0, 8.0], [8.0, 3.0, 3.0], [6.0, 1.0, 2.0]], [[8.0, 2.0, 4.0], [4.0, 4.0, 7.0], [8.0, 6.0, 
8.0]]], [[[5.0, 3.0, 4.0], [4.0, 6.0, 6.0], [4.0, 8.0, 8.0]], [[6.0, 6.0, 5.0], [2.0, 7.0, 4.0], [1.0, 5.0, 4.0]], [[6.0, 3.0, 
3.0], [3.0, 8.0, 8.0], [4.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 5.0], [8.0, 7.0, 2.0], [1.0, 4.0, 2.0]], [[2.0, 7.0, 8.0], [7.0, 3.0, 
4.0], [2.0, 4.0, 8.0]], [[5.0, 1.0, 2.0], [1.0, 6.0, 1.0], [4.0, 5.0, 4.0]]], [[[3.0, 4.0, 4.0], [7.0, 8.0, 6.0], [1.0, 3.0, 
2.0]], [[2.0, 6.0, 5.0], [4.0, 2.0, 8.0], [7.0, 3.0, 1.0]], [[8.0, 1.0, 2.0], [3.0, 6.0, 2.0], [3.0, 3.0, 5.0]]], [[[3.0, 4.0, 
8.0], [5.0, 2.0, 6.0], [6.0, 1.0, 5.0]], [[6.0, 3.0, 2.0], [8.0, 4.0, 4.0], [5.0, 2.0, 7.0]], [[4.0, 7.0, 3.0], [4.0, 3.0, 
2.0], [5.0, 1.0, 4.0]]], [[[6.0, 8.0, 6.0], [2.0, 1.0, 3.0], [7.0, 2.0, 3.0]], [[4.0, 2.0, 7.0], [8.0, 4.0, 4.0], [6.0, 3.0, 
6.0]], [[3.0, 6.0, 4.0], [3.0, 7.0, 3.0], [7.0, 7.0, 7.0]]]]))
      val=Data(numpy.array([[[[7.0, 6.0], [2.0, 4.0], [5.0, 6.0]], [[8.0, 5.0], [5.0, 2.0], [1.0, 2.0]], [[8.0, 7.0], [1.0, 
8.0], [4.0, 7.0]]], [[[2.0, 3.0], [1.0, 6.0], [3.0, 1.0]], [[7.0, 6.0], [8.0, 1.0], [5.0, 3.0]], [[3.0, 4.0], [3.0, 7.0], [7.0, 
2.0]]]]),self.functionspace)
      arg[2:,:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 8.0], [5.0, 2.0, 1.0], [5.0, 4.0, 1.0]], [[3.0, 7.0, 1.0], [4.0, 8.0, 6.0], [4.0, 
8.0, 3.0]], [[4.0, 4.0, 5.0], [4.0, 5.0, 3.0], [8.0, 2.0, 3.0]]], [[[3.0, 1.0, 6.0], [2.0, 4.0, 5.0], [7.0, 5.0, 8.0]], [[1.0, 
8.0, 1.0], [8.0, 5.0, 4.0], [4.0, 6.0, 8.0]], [[1.0, 4.0, 3.0], [1.0, 4.0, 2.0], [6.0, 6.0, 8.0]]], [[[7.0, 6.0, 8.0], [2.0, 
4.0, 3.0], [5.0, 6.0, 3.0]], [[8.0, 5.0, 8.0], [5.0, 2.0, 3.0], [1.0, 2.0, 2.0]], [[8.0, 7.0, 4.0], [1.0, 8.0, 7.0], [4.0, 7.0, 
8.0]]], [[[2.0, 3.0, 4.0], [1.0, 6.0, 6.0], [3.0, 1.0, 8.0]], [[7.0, 6.0, 5.0], [8.0, 1.0, 4.0], [5.0, 3.0, 4.0]], [[3.0, 4.0, 
3.0], [3.0, 7.0, 8.0], [7.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 5.0], [8.0, 7.0, 2.0], [1.0, 4.0, 2.0]], 
[[2.0, 7.0, 8.0], [7.0, 3.0, 4.0], [2.0, 4.0, 8.0]], [[5.0, 1.0, 2.0], [1.0, 6.0, 1.0], [4.0, 5.0, 4.0]]], [[[3.0, 4.0, 4.0], 
[7.0, 8.0, 6.0], [1.0, 3.0, 2.0]], [[2.0, 6.0, 5.0], [4.0, 2.0, 8.0], [7.0, 3.0, 1.0]], [[8.0, 1.0, 2.0], [3.0, 6.0, 2.0], 
[3.0, 3.0, 5.0]]], [[[7.0, 6.0, 8.0], [2.0, 4.0, 6.0], [5.0, 6.0, 5.0]], [[8.0, 5.0, 2.0], [5.0, 2.0, 4.0], [1.0, 2.0, 7.0]], 
[[8.0, 7.0, 3.0], [1.0, 8.0, 2.0], [4.0, 7.0, 4.0]]], [[[2.0, 3.0, 6.0], [1.0, 6.0, 3.0], [3.0, 1.0, 3.0]], [[7.0, 6.0, 7.0], 
[8.0, 1.0, 4.0], [5.0, 3.0, 6.0]], [[3.0, 4.0, 4.0], [3.0, 7.0, 3.0], [7.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LCCU(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 2.0], [3.0, 4.0, 3.0], [5.0, 4.0, 7.0]], [[2.0, 5.0, 7.0], [1.0, 5.0, 6.0], [6.0, 2.0, 
1.0]], [[5.0, 7.0, 6.0], [4.0, 2.0, 4.0], [7.0, 7.0, 8.0]]], [[[6.0, 4.0, 3.0], [8.0, 6.0, 2.0], [4.0, 4.0, 3.0]], [[4.0, 5.0, 
4.0], [6.0, 4.0, 5.0], [8.0, 7.0, 2.0]], [[3.0, 8.0, 7.0], [8.0, 2.0, 6.0], [4.0, 2.0, 6.0]]], [[[5.0, 2.0, 4.0], [1.0, 2.0, 
1.0], [7.0, 1.0, 7.0]], [[8.0, 1.0, 5.0], [5.0, 6.0, 2.0], [6.0, 8.0, 2.0]], [[7.0, 3.0, 8.0], [7.0, 1.0, 7.0], [6.0, 7.0, 
4.0]]], [[[4.0, 5.0, 6.0], [5.0, 8.0, 8.0], [5.0, 6.0, 8.0]], [[4.0, 4.0, 2.0], [2.0, 3.0, 5.0], [2.0, 8.0, 6.0]], [[7.0, 3.0, 
7.0], [3.0, 7.0, 6.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 1.0], [5.0, 8.0, 4.0], [5.0, 5.0, 8.0]], [[4.0, 3.0, 6.0], [1.0, 4.0, 
7.0], [3.0, 5.0, 6.0]], [[7.0, 4.0, 3.0], [2.0, 2.0, 6.0], [8.0, 6.0, 7.0]]], [[[3.0, 1.0, 4.0], [7.0, 3.0, 6.0], [3.0, 6.0, 
3.0]], [[6.0, 7.0, 3.0], [5.0, 3.0, 7.0], [2.0, 7.0, 1.0]], [[3.0, 2.0, 5.0], [8.0, 5.0, 3.0], [4.0, 1.0, 6.0]]], [[[8.0, 5.0, 
3.0], [1.0, 1.0, 6.0], [4.0, 3.0, 7.0]], [[6.0, 8.0, 1.0], [5.0, 8.0, 1.0], [1.0, 5.0, 2.0]], [[5.0, 5.0, 7.0], [7.0, 2.0, 
2.0], [6.0, 2.0, 4.0]]], [[[7.0, 1.0, 2.0], [8.0, 5.0, 6.0], [5.0, 3.0, 2.0]], [[8.0, 4.0, 7.0], [3.0, 3.0, 2.0], [4.0, 2.0, 
1.0]], [[8.0, 5.0, 5.0], [7.0, 2.0, 4.0], [4.0, 6.0, 4.0]]]]))
      val=Data(numpy.array([[[[7.0, 4.0], [7.0, 2.0], [8.0, 5.0]], [[2.0, 8.0], [8.0, 5.0], [4.0, 6.0]], [[1.0, 4.0], [5.0, 
5.0], [5.0, 2.0]]], [[[2.0, 3.0], [5.0, 3.0], [3.0, 5.0]], [[6.0, 1.0], [5.0, 6.0], [1.0, 4.0]], [[4.0, 4.0], [8.0, 4.0], [7.0, 
6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 3.0], [1.0, 8.0], [4.0, 4.0]], [[5.0, 1.0], [8.0, 2.0], [6.0, 6.0]], [[3.0, 
6.0], [5.0, 8.0], [3.0, 6.0]]], [[[3.0, 5.0], [2.0, 1.0], [1.0, 6.0]], [[2.0, 8.0], [5.0, 3.0], [7.0, 2.0]], [[8.0, 5.0], [3.0, 
4.0], [4.0, 7.0]]]]))
      arg[2:,:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 2.0], [3.0, 4.0, 3.0], [5.0, 4.0, 7.0]], [[2.0, 5.0, 7.0], [1.0, 5.0, 6.0], [6.0, 
2.0, 1.0]], [[5.0, 7.0, 6.0], [4.0, 2.0, 4.0], [7.0, 7.0, 8.0]]], [[[6.0, 4.0, 3.0], [8.0, 6.0, 2.0], [4.0, 4.0, 3.0]], [[4.0, 
5.0, 4.0], [6.0, 4.0, 5.0], [8.0, 7.0, 2.0]], [[3.0, 8.0, 7.0], [8.0, 2.0, 6.0], [4.0, 2.0, 6.0]]], [[[7.0, 4.0, 4.0], [7.0, 
2.0, 1.0], [8.0, 5.0, 7.0]], [[2.0, 8.0, 5.0], [8.0, 5.0, 2.0], [4.0, 6.0, 2.0]], [[1.0, 4.0, 8.0], [5.0, 5.0, 7.0], [5.0, 2.0, 
4.0]]], [[[2.0, 3.0, 6.0], [5.0, 3.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 1.0, 2.0], [5.0, 6.0, 5.0], [1.0, 4.0, 6.0]], [[4.0, 4.0, 
7.0], [8.0, 4.0, 6.0], [7.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 1.0], [5.0, 8.0, 4.0], [5.0, 5.0, 8.0]], 
[[4.0, 3.0, 6.0], [1.0, 4.0, 7.0], [3.0, 5.0, 6.0]], [[7.0, 4.0, 3.0], [2.0, 2.0, 6.0], [8.0, 6.0, 7.0]]], [[[3.0, 1.0, 4.0], 
[7.0, 3.0, 6.0], [3.0, 6.0, 3.0]], [[6.0, 7.0, 3.0], [5.0, 3.0, 7.0], [2.0, 7.0, 1.0]], [[3.0, 2.0, 5.0], [8.0, 5.0, 3.0], 
[4.0, 1.0, 6.0]]], [[[3.0, 3.0, 3.0], [1.0, 8.0, 6.0], [4.0, 4.0, 7.0]], [[5.0, 1.0, 1.0], [8.0, 2.0, 1.0], [6.0, 6.0, 2.0]], 
[[3.0, 6.0, 7.0], [5.0, 8.0, 2.0], [3.0, 6.0, 4.0]]], [[[3.0, 5.0, 2.0], [2.0, 1.0, 6.0], [1.0, 6.0, 2.0]], [[2.0, 8.0, 7.0], 
[5.0, 3.0, 2.0], [7.0, 2.0, 1.0]], [[8.0, 5.0, 5.0], [3.0, 4.0, 4.0], [4.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LCCU(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 1.0], [1.0, 7.0, 2.0], [4.0, 4.0, 7.0]], [[7.0, 4.0, 6.0], [3.0, 4.0, 4.0], [7.0, 2.0, 
8.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 8.0], [4.0, 5.0, 7.0]]], [[[8.0, 1.0, 7.0], [6.0, 8.0, 4.0], [5.0, 3.0, 3.0]], [[1.0, 2.0, 
4.0], [2.0, 7.0, 3.0], [7.0, 6.0, 8.0]], [[4.0, 7.0, 4.0], [2.0, 3.0, 2.0], [3.0, 2.0, 8.0]]], [[[8.0, 7.0, 7.0], [5.0, 6.0, 
3.0], [3.0, 7.0, 7.0]], [[6.0, 5.0, 7.0], [1.0, 1.0, 3.0], [3.0, 7.0, 3.0]], [[1.0, 1.0, 4.0], [1.0, 3.0, 7.0], [1.0, 7.0, 
7.0]]], [[[6.0, 7.0, 4.0], [8.0, 8.0, 6.0], [8.0, 4.0, 7.0]], [[3.0, 1.0, 5.0], [4.0, 7.0, 3.0], [7.0, 8.0, 8.0]], [[6.0, 2.0, 
4.0], [6.0, 1.0, 3.0], [7.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 7.0], [7.0, 7.0, 3.0], [5.0, 7.0, 5.0]], [[6.0, 4.0, 1.0], [2.0, 2.0, 
5.0], [4.0, 5.0, 1.0]], [[7.0, 8.0, 8.0], [7.0, 5.0, 5.0], [6.0, 7.0, 8.0]]], [[[3.0, 2.0, 5.0], [6.0, 5.0, 5.0], [6.0, 7.0, 
7.0]], [[8.0, 3.0, 3.0], [1.0, 8.0, 1.0], [6.0, 2.0, 8.0]], [[1.0, 6.0, 7.0], [4.0, 8.0, 1.0], [3.0, 1.0, 1.0]]], [[[1.0, 4.0, 
6.0], [6.0, 8.0, 6.0], [7.0, 2.0, 5.0]], [[2.0, 8.0, 1.0], [7.0, 5.0, 2.0], [2.0, 6.0, 7.0]], [[6.0, 1.0, 2.0], [4.0, 7.0, 
2.0], [5.0, 8.0, 6.0]]], [[[3.0, 5.0, 6.0], [3.0, 5.0, 5.0], [6.0, 3.0, 5.0]], [[2.0, 6.0, 6.0], [2.0, 5.0, 2.0], [6.0, 5.0, 
7.0]], [[7.0, 4.0, 7.0], [3.0, 3.0, 7.0], [4.0, 2.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 4.0], [6.0, 1.0], [6.0, 4.0]], [[4.0, 5.0], [3.0, 7.0], [8.0, 8.0]], [[1.0, 5.0], [4.0, 
8.0], [7.0, 3.0]]], [[[3.0, 2.0], [3.0, 4.0], [2.0, 1.0]], [[6.0, 8.0], [6.0, 5.0], [4.0, 2.0]], [[4.0, 5.0], [3.0, 4.0], [7.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 1.0], [7.0, 6.0], [7.0, 3.0]], [[4.0, 5.0], [5.0, 1.0], [2.0, 1.0]], [[8.0, 1.0], 
[2.0, 1.0], [5.0, 2.0]]], [[[5.0, 3.0], [2.0, 5.0], [3.0, 8.0]], [[1.0, 3.0], [3.0, 7.0], [6.0, 8.0]], [[8.0, 3.0], [8.0, 4.0], 
[6.0, 3.0]]]])
      arg[2:,:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 1.0], [1.0, 7.0, 2.0], [4.0, 4.0, 7.0]], [[7.0, 4.0, 6.0], [3.0, 4.0, 4.0], [7.0, 
2.0, 8.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 8.0], [4.0, 5.0, 7.0]]], [[[8.0, 1.0, 7.0], [6.0, 8.0, 4.0], [5.0, 3.0, 3.0]], [[1.0, 
2.0, 4.0], [2.0, 7.0, 3.0], [7.0, 6.0, 8.0]], [[4.0, 7.0, 4.0], [2.0, 3.0, 2.0], [3.0, 2.0, 8.0]]], [[[1.0, 4.0, 7.0], [6.0, 
1.0, 3.0], [6.0, 4.0, 7.0]], [[4.0, 5.0, 7.0], [3.0, 7.0, 3.0], [8.0, 8.0, 3.0]], [[1.0, 5.0, 4.0], [4.0, 8.0, 7.0], [7.0, 3.0, 
7.0]]], [[[3.0, 2.0, 4.0], [3.0, 4.0, 6.0], [2.0, 1.0, 7.0]], [[6.0, 8.0, 5.0], [6.0, 5.0, 3.0], [4.0, 2.0, 8.0]], [[4.0, 5.0, 
4.0], [3.0, 4.0, 3.0], [7.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 7.0], [7.0, 7.0, 3.0], [5.0, 7.0, 5.0]], 
[[6.0, 4.0, 1.0], [2.0, 2.0, 5.0], [4.0, 5.0, 1.0]], [[7.0, 8.0, 8.0], [7.0, 5.0, 5.0], [6.0, 7.0, 8.0]]], [[[3.0, 2.0, 5.0], 
[6.0, 5.0, 5.0], [6.0, 7.0, 7.0]], [[8.0, 3.0, 3.0], [1.0, 8.0, 1.0], [6.0, 2.0, 8.0]], [[1.0, 6.0, 7.0], [4.0, 8.0, 1.0], 
[3.0, 1.0, 1.0]]], [[[2.0, 1.0, 6.0], [7.0, 6.0, 6.0], [7.0, 3.0, 5.0]], [[4.0, 5.0, 1.0], [5.0, 1.0, 2.0], [2.0, 1.0, 7.0]], 
[[8.0, 1.0, 2.0], [2.0, 1.0, 2.0], [5.0, 2.0, 6.0]]], [[[5.0, 3.0, 6.0], [2.0, 5.0, 5.0], [3.0, 8.0, 5.0]], [[1.0, 3.0, 6.0], 
[3.0, 7.0, 2.0], [6.0, 8.0, 7.0]], [[8.0, 3.0, 7.0], [8.0, 4.0, 7.0], [6.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPLN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 5.0], [4.0, 5.0, 7.0], [2.0, 2.0, 4.0]], [[6.0, 3.0, 7.0], [5.0, 3.0, 7.0], [8.0, 8.0, 
5.0]], [[5.0, 5.0, 1.0], [5.0, 4.0, 8.0], [2.0, 3.0, 7.0]]], [[[2.0, 6.0, 3.0], [4.0, 3.0, 5.0], [2.0, 4.0, 3.0]], [[3.0, 8.0, 
5.0], [1.0, 8.0, 1.0], [3.0, 1.0, 5.0]], [[6.0, 1.0, 8.0], [8.0, 2.0, 7.0], [6.0, 2.0, 3.0]]], [[[5.0, 6.0, 3.0], [6.0, 8.0, 
7.0], [1.0, 8.0, 2.0]], [[5.0, 4.0, 4.0], [7.0, 8.0, 7.0], [8.0, 2.0, 6.0]], [[7.0, 7.0, 7.0], [3.0, 6.0, 2.0], [8.0, 6.0, 
5.0]]], [[[4.0, 4.0, 2.0], [2.0, 6.0, 7.0], [2.0, 6.0, 8.0]], [[6.0, 5.0, 6.0], [4.0, 7.0, 6.0], [6.0, 2.0, 6.0]], [[8.0, 6.0, 
7.0], [7.0, 4.0, 5.0], [3.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 8.0], [8.0, 5.0, 3.0], [7.0, 7.0, 2.0]], [[1.0, 8.0, 2.0], [1.0, 1.0, 
7.0], [8.0, 7.0, 2.0]], [[3.0, 6.0, 4.0], [8.0, 6.0, 6.0], [8.0, 5.0, 7.0]]], [[[4.0, 7.0, 1.0], [3.0, 5.0, 5.0], [1.0, 8.0, 
2.0]], [[8.0, 1.0, 6.0], [8.0, 4.0, 5.0], [2.0, 5.0, 6.0]], [[1.0, 1.0, 2.0], [4.0, 1.0, 4.0], [3.0, 1.0, 7.0]]], [[[2.0, 8.0, 
4.0], [6.0, 1.0, 7.0], [3.0, 7.0, 4.0]], [[1.0, 3.0, 7.0], [8.0, 1.0, 8.0], [1.0, 4.0, 1.0]], [[1.0, 4.0, 1.0], [5.0, 8.0, 
1.0], [2.0, 6.0, 8.0]]], [[[6.0, 1.0, 7.0], [8.0, 7.0, 4.0], [4.0, 7.0, 3.0]], [[7.0, 3.0, 2.0], [8.0, 8.0, 4.0], [3.0, 7.0, 
6.0]], [[7.0, 5.0, 7.0], [1.0, 8.0, 4.0], [4.0, 7.0, 6.0]]]]))
      val=numpy.array([[8.0, 4.0, 5.0]])
      arg[2,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 5.0], [4.0, 5.0, 7.0], [2.0, 2.0, 4.0]], [[6.0, 3.0, 7.0], [5.0, 3.0, 7.0], [8.0, 
8.0, 5.0]], [[5.0, 5.0, 1.0], [5.0, 4.0, 8.0], [2.0, 3.0, 7.0]]], [[[2.0, 6.0, 3.0], [4.0, 3.0, 5.0], [2.0, 4.0, 3.0]], [[3.0, 
8.0, 5.0], [1.0, 8.0, 1.0], [3.0, 1.0, 5.0]], [[6.0, 1.0, 8.0], [8.0, 2.0, 7.0], [6.0, 2.0, 3.0]]], [[[5.0, 6.0, 3.0], [6.0, 
8.0, 7.0], [1.0, 8.0, 2.0]], [[5.0, 4.0, 4.0], [7.0, 8.0, 7.0], [8.0, 4.0, 5.0]], [[7.0, 7.0, 7.0], [3.0, 6.0, 2.0], [8.0, 6.0, 
5.0]]], [[[4.0, 4.0, 2.0], [2.0, 6.0, 7.0], [2.0, 6.0, 8.0]], [[6.0, 5.0, 6.0], [4.0, 7.0, 6.0], [6.0, 2.0, 6.0]], [[8.0, 6.0, 
7.0], [7.0, 4.0, 5.0], [3.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 8.0], [8.0, 5.0, 3.0], [7.0, 7.0, 2.0]], 
[[1.0, 8.0, 2.0], [1.0, 1.0, 7.0], [8.0, 7.0, 2.0]], [[3.0, 6.0, 4.0], [8.0, 6.0, 6.0], [8.0, 5.0, 7.0]]], [[[4.0, 7.0, 1.0], 
[3.0, 5.0, 5.0], [1.0, 8.0, 2.0]], [[8.0, 1.0, 6.0], [8.0, 4.0, 5.0], [2.0, 5.0, 6.0]], [[1.0, 1.0, 2.0], [4.0, 1.0, 4.0], 
[3.0, 1.0, 7.0]]], [[[2.0, 8.0, 4.0], [6.0, 1.0, 7.0], [3.0, 7.0, 4.0]], [[1.0, 3.0, 7.0], [8.0, 1.0, 8.0], [8.0, 4.0, 5.0]], 
[[1.0, 4.0, 1.0], [5.0, 8.0, 1.0], [2.0, 6.0, 8.0]]], [[[6.0, 1.0, 7.0], [8.0, 7.0, 4.0], [4.0, 7.0, 3.0]], [[7.0, 3.0, 2.0], 
[8.0, 8.0, 4.0], [3.0, 7.0, 6.0]], [[7.0, 5.0, 7.0], [1.0, 8.0, 4.0], [4.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPLN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 1.0], [4.0, 7.0, 7.0], [5.0, 7.0, 1.0]], [[8.0, 7.0, 5.0], [6.0, 8.0, 7.0], [1.0, 2.0, 
2.0]], [[1.0, 6.0, 3.0], [7.0, 8.0, 2.0], [2.0, 2.0, 1.0]]], [[[2.0, 1.0, 2.0], [1.0, 4.0, 7.0], [1.0, 2.0, 5.0]], [[5.0, 5.0, 
8.0], [7.0, 7.0, 1.0], [8.0, 3.0, 8.0]], [[6.0, 2.0, 2.0], [7.0, 1.0, 4.0], [2.0, 5.0, 1.0]]], [[[2.0, 7.0, 8.0], [8.0, 3.0, 
1.0], [2.0, 3.0, 4.0]], [[2.0, 3.0, 2.0], [7.0, 8.0, 1.0], [6.0, 3.0, 4.0]], [[8.0, 1.0, 3.0], [1.0, 1.0, 2.0], [1.0, 7.0, 
5.0]]], [[[2.0, 4.0, 2.0], [1.0, 4.0, 6.0], [1.0, 2.0, 4.0]], [[2.0, 1.0, 4.0], [8.0, 3.0, 7.0], [3.0, 2.0, 7.0]], [[5.0, 7.0, 
1.0], [7.0, 8.0, 4.0], [4.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 2.0], [7.0, 3.0, 4.0], [6.0, 4.0, 2.0]], [[4.0, 1.0, 6.0], [1.0, 3.0, 
4.0], [8.0, 2.0, 8.0]], [[1.0, 5.0, 1.0], [3.0, 6.0, 1.0], [5.0, 8.0, 5.0]]], [[[5.0, 7.0, 7.0], [1.0, 5.0, 5.0], [2.0, 2.0, 
8.0]], [[8.0, 8.0, 1.0], [4.0, 8.0, 4.0], [3.0, 2.0, 6.0]], [[4.0, 5.0, 6.0], [7.0, 7.0, 4.0], [1.0, 5.0, 6.0]]], [[[5.0, 6.0, 
2.0], [2.0, 3.0, 1.0], [4.0, 3.0, 8.0]], [[8.0, 8.0, 2.0], [5.0, 6.0, 5.0], [6.0, 5.0, 4.0]], [[4.0, 7.0, 3.0], [5.0, 2.0, 
1.0], [7.0, 4.0, 7.0]]], [[[8.0, 2.0, 8.0], [1.0, 8.0, 1.0], [5.0, 4.0, 1.0]], [[6.0, 1.0, 6.0], [3.0, 2.0, 1.0], [6.0, 1.0, 
8.0]], [[1.0, 1.0, 8.0], [2.0, 7.0, 8.0], [4.0, 2.0, 3.0]]]]))
      val=Data(numpy.array([[8.0, 7.0, 6.0]]),self.functionspace)
      arg[2,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 1.0], [4.0, 7.0, 7.0], [5.0, 7.0, 1.0]], [[8.0, 7.0, 5.0], [6.0, 8.0, 7.0], [1.0, 
2.0, 2.0]], [[1.0, 6.0, 3.0], [7.0, 8.0, 2.0], [2.0, 2.0, 1.0]]], [[[2.0, 1.0, 2.0], [1.0, 4.0, 7.0], [1.0, 2.0, 5.0]], [[5.0, 
5.0, 8.0], [7.0, 7.0, 1.0], [8.0, 3.0, 8.0]], [[6.0, 2.0, 2.0], [7.0, 1.0, 4.0], [2.0, 5.0, 1.0]]], [[[2.0, 7.0, 8.0], [8.0, 
3.0, 1.0], [2.0, 3.0, 4.0]], [[2.0, 3.0, 2.0], [7.0, 8.0, 1.0], [8.0, 7.0, 6.0]], [[8.0, 1.0, 3.0], [1.0, 1.0, 2.0], [1.0, 7.0, 
5.0]]], [[[2.0, 4.0, 2.0], [1.0, 4.0, 6.0], [1.0, 2.0, 4.0]], [[2.0, 1.0, 4.0], [8.0, 3.0, 7.0], [3.0, 2.0, 7.0]], [[5.0, 7.0, 
1.0], [7.0, 8.0, 4.0], [4.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 2.0], [7.0, 3.0, 4.0], [6.0, 4.0, 2.0]], 
[[4.0, 1.0, 6.0], [1.0, 3.0, 4.0], [8.0, 2.0, 8.0]], [[1.0, 5.0, 1.0], [3.0, 6.0, 1.0], [5.0, 8.0, 5.0]]], [[[5.0, 7.0, 7.0], 
[1.0, 5.0, 5.0], [2.0, 2.0, 8.0]], [[8.0, 8.0, 1.0], [4.0, 8.0, 4.0], [3.0, 2.0, 6.0]], [[4.0, 5.0, 6.0], [7.0, 7.0, 4.0], 
[1.0, 5.0, 6.0]]], [[[5.0, 6.0, 2.0], [2.0, 3.0, 1.0], [4.0, 3.0, 8.0]], [[8.0, 8.0, 2.0], [5.0, 6.0, 5.0], [8.0, 7.0, 6.0]], 
[[4.0, 7.0, 3.0], [5.0, 2.0, 1.0], [7.0, 4.0, 7.0]]], [[[8.0, 2.0, 8.0], [1.0, 8.0, 1.0], [5.0, 4.0, 1.0]], [[6.0, 1.0, 6.0], 
[3.0, 2.0, 1.0], [6.0, 1.0, 8.0]], [[1.0, 1.0, 8.0], [2.0, 7.0, 8.0], [4.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPLN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 7.0], [3.0, 3.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 8.0, 3.0], [4.0, 1.0, 3.0], [2.0, 4.0, 
7.0]], [[2.0, 4.0, 1.0], [8.0, 6.0, 8.0], [7.0, 2.0, 6.0]]], [[[8.0, 3.0, 3.0], [8.0, 3.0, 4.0], [2.0, 6.0, 3.0]], [[2.0, 8.0, 
3.0], [3.0, 5.0, 2.0], [8.0, 4.0, 2.0]], [[7.0, 4.0, 7.0], [7.0, 8.0, 2.0], [2.0, 3.0, 5.0]]], [[[3.0, 6.0, 6.0], [6.0, 1.0, 
7.0], [5.0, 8.0, 7.0]], [[4.0, 6.0, 4.0], [8.0, 7.0, 4.0], [3.0, 4.0, 5.0]], [[1.0, 5.0, 5.0], [8.0, 5.0, 4.0], [3.0, 5.0, 
4.0]]], [[[8.0, 5.0, 4.0], [7.0, 6.0, 8.0], [5.0, 2.0, 1.0]], [[7.0, 7.0, 2.0], [1.0, 3.0, 3.0], [1.0, 7.0, 6.0]], [[7.0, 8.0, 
5.0], [1.0, 8.0, 5.0], [2.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 5.0], [7.0, 1.0, 5.0], [3.0, 5.0, 4.0]], [[8.0, 6.0, 7.0], [7.0, 3.0, 
2.0], [6.0, 8.0, 1.0]], [[7.0, 8.0, 6.0], [1.0, 1.0, 7.0], [3.0, 3.0, 7.0]]], [[[8.0, 2.0, 8.0], [4.0, 2.0, 8.0], [2.0, 6.0, 
8.0]], [[5.0, 3.0, 4.0], [5.0, 6.0, 4.0], [3.0, 2.0, 6.0]], [[3.0, 6.0, 6.0], [4.0, 2.0, 8.0], [4.0, 5.0, 5.0]]], [[[1.0, 2.0, 
2.0], [7.0, 8.0, 7.0], [6.0, 8.0, 1.0]], [[5.0, 5.0, 3.0], [4.0, 4.0, 7.0], [8.0, 4.0, 1.0]], [[4.0, 6.0, 4.0], [1.0, 2.0, 
3.0], [3.0, 1.0, 8.0]]], [[[6.0, 4.0, 7.0], [6.0, 7.0, 2.0], [2.0, 5.0, 6.0]], [[4.0, 2.0, 5.0], [7.0, 2.0, 6.0], [4.0, 4.0, 
4.0]], [[1.0, 2.0, 6.0], [2.0, 6.0, 7.0], [6.0, 5.0, 4.0]]]]))
      val=Data(numpy.array([[2.0, 3.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 3.0, 3.0]]))
      arg[2,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 7.0], [3.0, 3.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 8.0, 3.0], [4.0, 1.0, 3.0], [2.0, 
4.0, 7.0]], [[2.0, 4.0, 1.0], [8.0, 6.0, 8.0], [7.0, 2.0, 6.0]]], [[[8.0, 3.0, 3.0], [8.0, 3.0, 4.0], [2.0, 6.0, 3.0]], [[2.0, 
8.0, 3.0], [3.0, 5.0, 2.0], [8.0, 4.0, 2.0]], [[7.0, 4.0, 7.0], [7.0, 8.0, 2.0], [2.0, 3.0, 5.0]]], [[[3.0, 6.0, 6.0], [6.0, 
1.0, 7.0], [5.0, 8.0, 7.0]], [[4.0, 6.0, 4.0], [8.0, 7.0, 4.0], [2.0, 3.0, 1.0]], [[1.0, 5.0, 5.0], [8.0, 5.0, 4.0], [3.0, 5.0, 
4.0]]], [[[8.0, 5.0, 4.0], [7.0, 6.0, 8.0], [5.0, 2.0, 1.0]], [[7.0, 7.0, 2.0], [1.0, 3.0, 3.0], [1.0, 7.0, 6.0]], [[7.0, 8.0, 
5.0], [1.0, 8.0, 5.0], [2.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 5.0], [7.0, 1.0, 5.0], [3.0, 5.0, 4.0]], 
[[8.0, 6.0, 7.0], [7.0, 3.0, 2.0], [6.0, 8.0, 1.0]], [[7.0, 8.0, 6.0], [1.0, 1.0, 7.0], [3.0, 3.0, 7.0]]], [[[8.0, 2.0, 8.0], 
[4.0, 2.0, 8.0], [2.0, 6.0, 8.0]], [[5.0, 3.0, 4.0], [5.0, 6.0, 4.0], [3.0, 2.0, 6.0]], [[3.0, 6.0, 6.0], [4.0, 2.0, 8.0], 
[4.0, 5.0, 5.0]]], [[[1.0, 2.0, 2.0], [7.0, 8.0, 7.0], [6.0, 8.0, 1.0]], [[5.0, 5.0, 3.0], [4.0, 4.0, 7.0], [2.0, 3.0, 3.0]], 
[[4.0, 6.0, 4.0], [1.0, 2.0, 3.0], [3.0, 1.0, 8.0]]], [[[6.0, 4.0, 7.0], [6.0, 7.0, 2.0], [2.0, 5.0, 6.0]], [[4.0, 2.0, 5.0], 
[7.0, 2.0, 6.0], [4.0, 4.0, 4.0]], [[1.0, 2.0, 6.0], [2.0, 6.0, 7.0], [6.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPLN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [4.0, 6.0, 8.0], [4.0, 8.0, 2.0]], [[7.0, 3.0, 3.0], [5.0, 2.0, 4.0], [5.0, 4.0, 
5.0]], [[3.0, 3.0, 3.0], [6.0, 7.0, 7.0], [6.0, 4.0, 8.0]]], [[[4.0, 6.0, 5.0], [2.0, 2.0, 6.0], [3.0, 4.0, 2.0]], [[4.0, 5.0, 
4.0], [3.0, 8.0, 5.0], [4.0, 5.0, 3.0]], [[1.0, 5.0, 5.0], [4.0, 6.0, 8.0], [1.0, 4.0, 2.0]]], [[[7.0, 4.0, 4.0], [5.0, 4.0, 
3.0], [8.0, 1.0, 2.0]], [[6.0, 3.0, 4.0], [7.0, 7.0, 2.0], [4.0, 5.0, 1.0]], [[1.0, 2.0, 1.0], [5.0, 3.0, 3.0], [8.0, 3.0, 
1.0]]], [[[3.0, 3.0, 2.0], [8.0, 1.0, 6.0], [7.0, 7.0, 1.0]], [[1.0, 7.0, 8.0], [3.0, 3.0, 6.0], [1.0, 5.0, 5.0]], [[7.0, 5.0, 
3.0], [1.0, 1.0, 7.0], [2.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 4.0], [2.0, 2.0, 8.0], [2.0, 4.0, 4.0]], [[1.0, 1.0, 5.0], [4.0, 7.0, 
2.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 4.0], [3.0, 6.0, 7.0], [3.0, 5.0, 7.0]]], [[[1.0, 8.0, 7.0], [6.0, 4.0, 3.0], [3.0, 6.0, 
6.0]], [[8.0, 3.0, 6.0], [3.0, 1.0, 8.0], [5.0, 2.0, 8.0]], [[3.0, 2.0, 7.0], [8.0, 5.0, 8.0], [2.0, 5.0, 5.0]]], [[[1.0, 3.0, 
4.0], [5.0, 8.0, 2.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 6.0], [6.0, 7.0, 7.0], [1.0, 7.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 5.0, 
2.0], [5.0, 3.0, 2.0]]], [[[5.0, 4.0, 8.0], [1.0, 8.0, 5.0], [6.0, 3.0, 5.0]], [[5.0, 1.0, 1.0], [6.0, 7.0, 3.0], [5.0, 5.0, 
4.0]], [[7.0, 5.0, 3.0], [3.0, 1.0, 2.0], [8.0, 3.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[4.0, 5.0, 2.0]])+(1.-msk_val)*numpy.array([[3.0, 3.0, 6.0]])
      arg[2,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 7.0], [4.0, 6.0, 8.0], [4.0, 8.0, 2.0]], [[7.0, 3.0, 3.0], [5.0, 2.0, 4.0], [5.0, 
4.0, 5.0]], [[3.0, 3.0, 3.0], [6.0, 7.0, 7.0], [6.0, 4.0, 8.0]]], [[[4.0, 6.0, 5.0], [2.0, 2.0, 6.0], [3.0, 4.0, 2.0]], [[4.0, 
5.0, 4.0], [3.0, 8.0, 5.0], [4.0, 5.0, 3.0]], [[1.0, 5.0, 5.0], [4.0, 6.0, 8.0], [1.0, 4.0, 2.0]]], [[[7.0, 4.0, 4.0], [5.0, 
4.0, 3.0], [8.0, 1.0, 2.0]], [[6.0, 3.0, 4.0], [7.0, 7.0, 2.0], [4.0, 5.0, 2.0]], [[1.0, 2.0, 1.0], [5.0, 3.0, 3.0], [8.0, 3.0, 
1.0]]], [[[3.0, 3.0, 2.0], [8.0, 1.0, 6.0], [7.0, 7.0, 1.0]], [[1.0, 7.0, 8.0], [3.0, 3.0, 6.0], [1.0, 5.0, 5.0]], [[7.0, 5.0, 
3.0], [1.0, 1.0, 7.0], [2.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 4.0], [2.0, 2.0, 8.0], [2.0, 4.0, 4.0]], 
[[1.0, 1.0, 5.0], [4.0, 7.0, 2.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 4.0], [3.0, 6.0, 7.0], [3.0, 5.0, 7.0]]], [[[1.0, 8.0, 7.0], 
[6.0, 4.0, 3.0], [3.0, 6.0, 6.0]], [[8.0, 3.0, 6.0], [3.0, 1.0, 8.0], [5.0, 2.0, 8.0]], [[3.0, 2.0, 7.0], [8.0, 5.0, 8.0], 
[2.0, 5.0, 5.0]]], [[[1.0, 3.0, 4.0], [5.0, 8.0, 2.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 6.0], [6.0, 7.0, 7.0], [3.0, 3.0, 6.0]], 
[[7.0, 6.0, 6.0], [2.0, 5.0, 2.0], [5.0, 3.0, 2.0]]], [[[5.0, 4.0, 8.0], [1.0, 8.0, 5.0], [6.0, 3.0, 5.0]], [[5.0, 1.0, 1.0], 
[6.0, 7.0, 3.0], [5.0, 5.0, 4.0]], [[7.0, 5.0, 3.0], [3.0, 1.0, 2.0], [8.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLRN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 7.0], [5.0, 2.0, 6.0], [6.0, 2.0, 3.0]], [[7.0, 6.0, 3.0], [6.0, 8.0, 4.0], [7.0, 8.0, 
2.0]], [[2.0, 3.0, 7.0], [8.0, 7.0, 5.0], [3.0, 8.0, 4.0]]], [[[6.0, 7.0, 4.0], [7.0, 4.0, 4.0], [5.0, 1.0, 6.0]], [[7.0, 6.0, 
5.0], [3.0, 6.0, 1.0], [6.0, 2.0, 7.0]], [[6.0, 2.0, 1.0], [7.0, 1.0, 4.0], [4.0, 7.0, 7.0]]], [[[3.0, 2.0, 5.0], [1.0, 1.0, 
3.0], [2.0, 5.0, 6.0]], [[8.0, 1.0, 7.0], [3.0, 8.0, 3.0], [5.0, 6.0, 2.0]], [[3.0, 6.0, 4.0], [4.0, 5.0, 5.0], [2.0, 5.0, 
3.0]]], [[[5.0, 1.0, 1.0], [5.0, 2.0, 1.0], [1.0, 7.0, 7.0]], [[3.0, 8.0, 3.0], [2.0, 4.0, 1.0], [1.0, 1.0, 1.0]], [[8.0, 8.0, 
4.0], [2.0, 6.0, 4.0], [6.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 6.0], [5.0, 8.0, 5.0], [4.0, 2.0, 7.0]], [[3.0, 1.0, 6.0], [7.0, 7.0, 
8.0], [5.0, 6.0, 3.0]], [[8.0, 1.0, 4.0], [7.0, 5.0, 5.0], [7.0, 4.0, 7.0]]], [[[2.0, 1.0, 7.0], [5.0, 4.0, 5.0], [5.0, 2.0, 
6.0]], [[7.0, 5.0, 7.0], [6.0, 7.0, 4.0], [6.0, 6.0, 1.0]], [[8.0, 7.0, 2.0], [7.0, 4.0, 2.0], [8.0, 3.0, 6.0]]], [[[4.0, 5.0, 
5.0], [4.0, 4.0, 8.0], [7.0, 1.0, 2.0]], [[7.0, 3.0, 5.0], [4.0, 6.0, 4.0], [7.0, 3.0, 6.0]], [[8.0, 4.0, 7.0], [5.0, 6.0, 
8.0], [3.0, 4.0, 6.0]]], [[[2.0, 3.0, 1.0], [3.0, 4.0, 7.0], [3.0, 7.0, 3.0]], [[3.0, 5.0, 1.0], [6.0, 2.0, 3.0], [5.0, 8.0, 
1.0]], [[7.0, 1.0, 6.0], [4.0, 1.0, 6.0], [5.0, 6.0, 5.0]]]]))
      val=numpy.array([[[[4.0, 4.0, 6.0]]], [[[8.0, 5.0, 2.0]]]])
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 7.0], [5.0, 2.0, 6.0], [6.0, 2.0, 3.0]], [[7.0, 6.0, 3.0], [6.0, 8.0, 4.0], [7.0, 
8.0, 2.0]], [[2.0, 3.0, 7.0], [8.0, 7.0, 5.0], [3.0, 8.0, 4.0]]], [[[6.0, 7.0, 4.0], [7.0, 4.0, 4.0], [5.0, 1.0, 6.0]], [[7.0, 
6.0, 5.0], [3.0, 6.0, 1.0], [6.0, 2.0, 7.0]], [[6.0, 2.0, 1.0], [7.0, 1.0, 4.0], [4.0, 7.0, 7.0]]], [[[3.0, 2.0, 5.0], [1.0, 
1.0, 3.0], [2.0, 5.0, 6.0]], [[8.0, 1.0, 7.0], [3.0, 8.0, 3.0], [5.0, 6.0, 2.0]], [[3.0, 6.0, 4.0], [4.0, 4.0, 6.0], [2.0, 5.0, 
3.0]]], [[[5.0, 1.0, 1.0], [5.0, 2.0, 1.0], [1.0, 7.0, 7.0]], [[3.0, 8.0, 3.0], [2.0, 4.0, 1.0], [1.0, 1.0, 1.0]], [[8.0, 8.0, 
4.0], [8.0, 5.0, 2.0], [6.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 6.0], [5.0, 8.0, 5.0], [4.0, 2.0, 7.0]], 
[[3.0, 1.0, 6.0], [7.0, 7.0, 8.0], [5.0, 6.0, 3.0]], [[8.0, 1.0, 4.0], [7.0, 5.0, 5.0], [7.0, 4.0, 7.0]]], [[[2.0, 1.0, 7.0], 
[5.0, 4.0, 5.0], [5.0, 2.0, 6.0]], [[7.0, 5.0, 7.0], [6.0, 7.0, 4.0], [6.0, 6.0, 1.0]], [[8.0, 7.0, 2.0], [7.0, 4.0, 2.0], 
[8.0, 3.0, 6.0]]], [[[4.0, 5.0, 5.0], [4.0, 4.0, 8.0], [7.0, 1.0, 2.0]], [[7.0, 3.0, 5.0], [4.0, 6.0, 4.0], [7.0, 3.0, 6.0]], 
[[8.0, 4.0, 7.0], [4.0, 4.0, 6.0], [3.0, 4.0, 6.0]]], [[[2.0, 3.0, 1.0], [3.0, 4.0, 7.0], [3.0, 7.0, 3.0]], [[3.0, 5.0, 1.0], 
[6.0, 2.0, 3.0], [5.0, 8.0, 1.0]], [[7.0, 1.0, 6.0], [8.0, 5.0, 2.0], [5.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLRN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 2.0], [5.0, 3.0, 2.0], [2.0, 3.0, 2.0]], [[3.0, 4.0, 7.0], [4.0, 3.0, 5.0], [1.0, 4.0, 
7.0]], [[8.0, 6.0, 1.0], [8.0, 6.0, 8.0], [5.0, 8.0, 8.0]]], [[[6.0, 4.0, 2.0], [3.0, 5.0, 8.0], [5.0, 7.0, 6.0]], [[8.0, 1.0, 
2.0], [3.0, 4.0, 2.0], [4.0, 7.0, 5.0]], [[5.0, 2.0, 7.0], [5.0, 3.0, 4.0], [1.0, 8.0, 6.0]]], [[[8.0, 3.0, 3.0], [3.0, 1.0, 
4.0], [8.0, 5.0, 5.0]], [[1.0, 8.0, 1.0], [8.0, 1.0, 4.0], [2.0, 6.0, 7.0]], [[4.0, 3.0, 3.0], [1.0, 3.0, 5.0], [7.0, 6.0, 
1.0]]], [[[3.0, 7.0, 4.0], [4.0, 3.0, 7.0], [4.0, 5.0, 5.0]], [[5.0, 3.0, 5.0], [5.0, 1.0, 3.0], [8.0, 6.0, 1.0]], [[7.0, 1.0, 
1.0], [5.0, 2.0, 2.0], [2.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 3.0], [1.0, 3.0, 3.0], [3.0, 6.0, 1.0]], [[6.0, 3.0, 1.0], [7.0, 3.0, 
7.0], [2.0, 3.0, 5.0]], [[3.0, 7.0, 7.0], [7.0, 8.0, 1.0], [5.0, 3.0, 8.0]]], [[[5.0, 7.0, 3.0], [4.0, 8.0, 8.0], [5.0, 5.0, 
5.0]], [[6.0, 7.0, 8.0], [3.0, 5.0, 1.0], [8.0, 6.0, 8.0]], [[2.0, 6.0, 8.0], [6.0, 6.0, 8.0], [8.0, 5.0, 4.0]]], [[[4.0, 2.0, 
2.0], [3.0, 3.0, 6.0], [8.0, 2.0, 3.0]], [[1.0, 6.0, 6.0], [6.0, 1.0, 5.0], [7.0, 8.0, 7.0]], [[5.0, 3.0, 4.0], [3.0, 6.0, 
7.0], [1.0, 4.0, 5.0]]], [[[7.0, 2.0, 4.0], [4.0, 7.0, 4.0], [4.0, 2.0, 5.0]], [[2.0, 2.0, 4.0], [4.0, 2.0, 7.0], [3.0, 6.0, 
6.0]], [[6.0, 6.0, 3.0], [6.0, 6.0, 2.0], [6.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[[[7.0, 1.0, 2.0]]], [[[7.0, 7.0, 6.0]]]]),self.functionspace)
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 2.0], [5.0, 3.0, 2.0], [2.0, 3.0, 2.0]], [[3.0, 4.0, 7.0], [4.0, 3.0, 5.0], [1.0, 
4.0, 7.0]], [[8.0, 6.0, 1.0], [8.0, 6.0, 8.0], [5.0, 8.0, 8.0]]], [[[6.0, 4.0, 2.0], [3.0, 5.0, 8.0], [5.0, 7.0, 6.0]], [[8.0, 
1.0, 2.0], [3.0, 4.0, 2.0], [4.0, 7.0, 5.0]], [[5.0, 2.0, 7.0], [5.0, 3.0, 4.0], [1.0, 8.0, 6.0]]], [[[8.0, 3.0, 3.0], [3.0, 
1.0, 4.0], [8.0, 5.0, 5.0]], [[1.0, 8.0, 1.0], [8.0, 1.0, 4.0], [2.0, 6.0, 7.0]], [[4.0, 3.0, 3.0], [7.0, 1.0, 2.0], [7.0, 6.0, 
1.0]]], [[[3.0, 7.0, 4.0], [4.0, 3.0, 7.0], [4.0, 5.0, 5.0]], [[5.0, 3.0, 5.0], [5.0, 1.0, 3.0], [8.0, 6.0, 1.0]], [[7.0, 1.0, 
1.0], [7.0, 7.0, 6.0], [2.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 3.0], [1.0, 3.0, 3.0], [3.0, 6.0, 1.0]], 
[[6.0, 3.0, 1.0], [7.0, 3.0, 7.0], [2.0, 3.0, 5.0]], [[3.0, 7.0, 7.0], [7.0, 8.0, 1.0], [5.0, 3.0, 8.0]]], [[[5.0, 7.0, 3.0], 
[4.0, 8.0, 8.0], [5.0, 5.0, 5.0]], [[6.0, 7.0, 8.0], [3.0, 5.0, 1.0], [8.0, 6.0, 8.0]], [[2.0, 6.0, 8.0], [6.0, 6.0, 8.0], 
[8.0, 5.0, 4.0]]], [[[4.0, 2.0, 2.0], [3.0, 3.0, 6.0], [8.0, 2.0, 3.0]], [[1.0, 6.0, 6.0], [6.0, 1.0, 5.0], [7.0, 8.0, 7.0]], 
[[5.0, 3.0, 4.0], [7.0, 1.0, 2.0], [1.0, 4.0, 5.0]]], [[[7.0, 2.0, 4.0], [4.0, 7.0, 4.0], [4.0, 2.0, 5.0]], [[2.0, 2.0, 4.0], 
[4.0, 2.0, 7.0], [3.0, 6.0, 6.0]], [[6.0, 6.0, 3.0], [7.0, 7.0, 6.0], [6.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLRN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 4.0], [8.0, 7.0, 1.0], [4.0, 5.0, 5.0]], [[8.0, 8.0, 1.0], [5.0, 1.0, 7.0], [3.0, 7.0, 
3.0]], [[5.0, 4.0, 5.0], [3.0, 6.0, 1.0], [5.0, 5.0, 4.0]]], [[[6.0, 8.0, 4.0], [4.0, 5.0, 1.0], [1.0, 5.0, 3.0]], [[8.0, 8.0, 
4.0], [7.0, 4.0, 6.0], [7.0, 6.0, 6.0]], [[3.0, 5.0, 6.0], [1.0, 1.0, 1.0], [3.0, 8.0, 1.0]]], [[[4.0, 3.0, 6.0], [8.0, 5.0, 
3.0], [5.0, 2.0, 4.0]], [[4.0, 5.0, 6.0], [2.0, 3.0, 6.0], [4.0, 8.0, 4.0]], [[5.0, 5.0, 3.0], [2.0, 2.0, 3.0], [1.0, 1.0, 
4.0]]], [[[5.0, 8.0, 3.0], [8.0, 6.0, 1.0], [5.0, 4.0, 8.0]], [[6.0, 2.0, 1.0], [6.0, 6.0, 7.0], [2.0, 1.0, 3.0]], [[7.0, 3.0, 
3.0], [5.0, 7.0, 4.0], [6.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 7.0], [2.0, 2.0, 2.0], [5.0, 2.0, 3.0]], [[5.0, 4.0, 3.0], [7.0, 1.0, 
6.0], [1.0, 8.0, 8.0]], [[2.0, 3.0, 6.0], [2.0, 6.0, 1.0], [3.0, 2.0, 1.0]]], [[[5.0, 2.0, 7.0], [1.0, 3.0, 8.0], [7.0, 3.0, 
4.0]], [[3.0, 8.0, 2.0], [6.0, 4.0, 6.0], [1.0, 2.0, 7.0]], [[6.0, 7.0, 1.0], [2.0, 1.0, 6.0], [6.0, 4.0, 6.0]]], [[[8.0, 2.0, 
2.0], [4.0, 1.0, 8.0], [8.0, 7.0, 8.0]], [[3.0, 5.0, 1.0], [5.0, 3.0, 5.0], [3.0, 8.0, 6.0]], [[8.0, 7.0, 4.0], [4.0, 8.0, 
7.0], [5.0, 6.0, 8.0]]], [[[6.0, 4.0, 4.0], [6.0, 6.0, 5.0], [7.0, 4.0, 2.0]], [[8.0, 5.0, 4.0], [2.0, 4.0, 7.0], [2.0, 2.0, 
2.0]], [[4.0, 8.0, 2.0], [2.0, 2.0, 6.0], [2.0, 3.0, 6.0]]]]))
      val=Data(numpy.array([[[[2.0, 4.0, 7.0]]], [[[3.0, 2.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 1.0]]], [[[6.0, 1.0, 6.0]]]]))
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 4.0], [8.0, 7.0, 1.0], [4.0, 5.0, 5.0]], [[8.0, 8.0, 1.0], [5.0, 1.0, 7.0], [3.0, 
7.0, 3.0]], [[5.0, 4.0, 5.0], [3.0, 6.0, 1.0], [5.0, 5.0, 4.0]]], [[[6.0, 8.0, 4.0], [4.0, 5.0, 1.0], [1.0, 5.0, 3.0]], [[8.0, 
8.0, 4.0], [7.0, 4.0, 6.0], [7.0, 6.0, 6.0]], [[3.0, 5.0, 6.0], [1.0, 1.0, 1.0], [3.0, 8.0, 1.0]]], [[[4.0, 3.0, 6.0], [8.0, 
5.0, 3.0], [5.0, 2.0, 4.0]], [[4.0, 5.0, 6.0], [2.0, 3.0, 6.0], [4.0, 8.0, 4.0]], [[5.0, 5.0, 3.0], [2.0, 4.0, 7.0], [1.0, 1.0, 
4.0]]], [[[5.0, 8.0, 3.0], [8.0, 6.0, 1.0], [5.0, 4.0, 8.0]], [[6.0, 2.0, 1.0], [6.0, 6.0, 7.0], [2.0, 1.0, 3.0]], [[7.0, 3.0, 
3.0], [3.0, 2.0, 3.0], [6.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 7.0], [2.0, 2.0, 2.0], [5.0, 2.0, 3.0]], 
[[5.0, 4.0, 3.0], [7.0, 1.0, 6.0], [1.0, 8.0, 8.0]], [[2.0, 3.0, 6.0], [2.0, 6.0, 1.0], [3.0, 2.0, 1.0]]], [[[5.0, 2.0, 7.0], 
[1.0, 3.0, 8.0], [7.0, 3.0, 4.0]], [[3.0, 8.0, 2.0], [6.0, 4.0, 6.0], [1.0, 2.0, 7.0]], [[6.0, 7.0, 1.0], [2.0, 1.0, 6.0], 
[6.0, 4.0, 6.0]]], [[[8.0, 2.0, 2.0], [4.0, 1.0, 8.0], [8.0, 7.0, 8.0]], [[3.0, 5.0, 1.0], [5.0, 3.0, 5.0], [3.0, 8.0, 6.0]], 
[[8.0, 7.0, 4.0], [3.0, 1.0, 1.0], [5.0, 6.0, 8.0]]], [[[6.0, 4.0, 4.0], [6.0, 6.0, 5.0], [7.0, 4.0, 2.0]], [[8.0, 5.0, 4.0], 
[2.0, 4.0, 7.0], [2.0, 2.0, 2.0]], [[4.0, 8.0, 2.0], [6.0, 1.0, 6.0], [2.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLRN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 3.0], [5.0, 2.0, 3.0], [6.0, 5.0, 3.0]], [[1.0, 4.0, 3.0], [7.0, 6.0, 5.0], [6.0, 7.0, 
8.0]], [[7.0, 7.0, 2.0], [3.0, 3.0, 1.0], [3.0, 6.0, 5.0]]], [[[7.0, 5.0, 5.0], [5.0, 5.0, 4.0], [7.0, 5.0, 1.0]], [[7.0, 5.0, 
6.0], [1.0, 1.0, 8.0], [3.0, 4.0, 4.0]], [[4.0, 4.0, 7.0], [5.0, 8.0, 2.0], [2.0, 4.0, 7.0]]], [[[6.0, 4.0, 3.0], [7.0, 3.0, 
5.0], [2.0, 3.0, 1.0]], [[7.0, 7.0, 8.0], [4.0, 5.0, 4.0], [5.0, 2.0, 5.0]], [[7.0, 6.0, 5.0], [2.0, 4.0, 8.0], [3.0, 6.0, 
2.0]]], [[[1.0, 3.0, 8.0], [1.0, 4.0, 5.0], [5.0, 3.0, 8.0]], [[7.0, 2.0, 6.0], [1.0, 6.0, 6.0], [4.0, 2.0, 4.0]], [[2.0, 2.0, 
6.0], [4.0, 2.0, 8.0], [2.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 8.0], [1.0, 8.0, 4.0], [6.0, 7.0, 1.0]], [[3.0, 2.0, 8.0], [8.0, 5.0, 
8.0], [8.0, 3.0, 2.0]], [[7.0, 2.0, 6.0], [7.0, 5.0, 8.0], [1.0, 5.0, 2.0]]], [[[8.0, 3.0, 4.0], [3.0, 1.0, 7.0], [2.0, 5.0, 
1.0]], [[4.0, 5.0, 8.0], [4.0, 3.0, 3.0], [8.0, 7.0, 8.0]], [[3.0, 3.0, 7.0], [6.0, 6.0, 5.0], [4.0, 7.0, 8.0]]], [[[5.0, 8.0, 
5.0], [2.0, 4.0, 1.0], [7.0, 6.0, 6.0]], [[6.0, 8.0, 6.0], [5.0, 3.0, 8.0], [7.0, 3.0, 5.0]], [[3.0, 8.0, 5.0], [7.0, 6.0, 
5.0], [1.0, 3.0, 8.0]]], [[[7.0, 1.0, 8.0], [2.0, 1.0, 3.0], [6.0, 5.0, 5.0]], [[4.0, 6.0, 2.0], [6.0, 1.0, 1.0], [5.0, 3.0, 
2.0]], [[6.0, 2.0, 5.0], [1.0, 8.0, 4.0], [4.0, 8.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0, 8.0, 2.0]]], [[[3.0, 5.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 7.0, 8.0]]], 
[[[4.0, 4.0, 8.0]]]])
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 3.0], [5.0, 2.0, 3.0], [6.0, 5.0, 3.0]], [[1.0, 4.0, 3.0], [7.0, 6.0, 5.0], [6.0, 
7.0, 8.0]], [[7.0, 7.0, 2.0], [3.0, 3.0, 1.0], [3.0, 6.0, 5.0]]], [[[7.0, 5.0, 5.0], [5.0, 5.0, 4.0], [7.0, 5.0, 1.0]], [[7.0, 
5.0, 6.0], [1.0, 1.0, 8.0], [3.0, 4.0, 4.0]], [[4.0, 4.0, 7.0], [5.0, 8.0, 2.0], [2.0, 4.0, 7.0]]], [[[6.0, 4.0, 3.0], [7.0, 
3.0, 5.0], [2.0, 3.0, 1.0]], [[7.0, 7.0, 8.0], [4.0, 5.0, 4.0], [5.0, 2.0, 5.0]], [[7.0, 6.0, 5.0], [4.0, 8.0, 2.0], [3.0, 6.0, 
2.0]]], [[[1.0, 3.0, 8.0], [1.0, 4.0, 5.0], [5.0, 3.0, 8.0]], [[7.0, 2.0, 6.0], [1.0, 6.0, 6.0], [4.0, 2.0, 4.0]], [[2.0, 2.0, 
6.0], [3.0, 5.0, 2.0], [2.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 8.0], [1.0, 8.0, 4.0], [6.0, 7.0, 1.0]], 
[[3.0, 2.0, 8.0], [8.0, 5.0, 8.0], [8.0, 3.0, 2.0]], [[7.0, 2.0, 6.0], [7.0, 5.0, 8.0], [1.0, 5.0, 2.0]]], [[[8.0, 3.0, 4.0], 
[3.0, 1.0, 7.0], [2.0, 5.0, 1.0]], [[4.0, 5.0, 8.0], [4.0, 3.0, 3.0], [8.0, 7.0, 8.0]], [[3.0, 3.0, 7.0], [6.0, 6.0, 5.0], 
[4.0, 7.0, 8.0]]], [[[5.0, 8.0, 5.0], [2.0, 4.0, 1.0], [7.0, 6.0, 6.0]], [[6.0, 8.0, 6.0], [5.0, 3.0, 8.0], [7.0, 3.0, 5.0]], 
[[3.0, 8.0, 5.0], [8.0, 7.0, 8.0], [1.0, 3.0, 8.0]]], [[[7.0, 1.0, 8.0], [2.0, 1.0, 3.0], [6.0, 5.0, 5.0]], [[4.0, 6.0, 2.0], 
[6.0, 1.0, 1.0], [5.0, 3.0, 2.0]], [[6.0, 2.0, 5.0], [4.0, 4.0, 8.0], [4.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UCPN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 2.0], [8.0, 3.0, 4.0], [7.0, 5.0, 1.0]], [[2.0, 8.0, 2.0], [3.0, 6.0, 7.0], [4.0, 1.0, 
4.0]], [[6.0, 7.0, 5.0], [6.0, 4.0, 7.0], [1.0, 3.0, 4.0]]], [[[4.0, 6.0, 5.0], [8.0, 1.0, 2.0], [2.0, 1.0, 4.0]], [[4.0, 4.0, 
7.0], [7.0, 6.0, 4.0], [6.0, 2.0, 6.0]], [[5.0, 4.0, 6.0], [3.0, 5.0, 4.0], [6.0, 5.0, 6.0]]], [[[2.0, 3.0, 8.0], [3.0, 2.0, 
8.0], [3.0, 3.0, 1.0]], [[5.0, 6.0, 4.0], [5.0, 6.0, 6.0], [1.0, 8.0, 8.0]], [[1.0, 7.0, 7.0], [3.0, 1.0, 2.0], [1.0, 6.0, 
7.0]]], [[[5.0, 4.0, 3.0], [2.0, 2.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 8.0, 4.0], [1.0, 5.0, 4.0], [8.0, 5.0, 3.0]], [[7.0, 6.0, 
3.0], [2.0, 8.0, 7.0], [3.0, 7.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 2.0], [5.0, 7.0, 5.0], [2.0, 7.0, 2.0]], [[6.0, 5.0, 3.0], [3.0, 8.0, 
3.0], [1.0, 8.0, 5.0]], [[8.0, 1.0, 4.0], [7.0, 3.0, 6.0], [1.0, 2.0, 1.0]]], [[[3.0, 1.0, 2.0], [8.0, 6.0, 2.0], [3.0, 6.0, 
2.0]], [[5.0, 1.0, 2.0], [5.0, 7.0, 2.0], [6.0, 6.0, 7.0]], [[8.0, 6.0, 3.0], [3.0, 5.0, 5.0], [7.0, 3.0, 5.0]]], [[[1.0, 6.0, 
8.0], [8.0, 8.0, 3.0], [6.0, 7.0, 3.0]], [[3.0, 6.0, 6.0], [3.0, 5.0, 5.0], [2.0, 1.0, 5.0]], [[2.0, 2.0, 3.0], [1.0, 5.0, 
8.0], [1.0, 5.0, 4.0]]], [[[7.0, 5.0, 6.0], [8.0, 6.0, 2.0], [7.0, 6.0, 5.0]], [[1.0, 8.0, 7.0], [3.0, 7.0, 2.0], [4.0, 4.0, 
7.0]], [[1.0, 4.0, 8.0], [4.0, 7.0, 7.0], [2.0, 8.0, 5.0]]]]))
      val=numpy.array([[[4.0, 7.0, 8.0], [5.0, 3.0, 7.0], [6.0, 6.0, 2.0]], [[4.0, 2.0, 5.0], [8.0, 4.0, 7.0], [1.0, 1.0, 
2.0]], [[5.0, 2.0, 2.0], [5.0, 7.0, 2.0], [5.0, 2.0, 3.0]]])
      arg[:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 2.0], [4.0, 7.0, 8.0], [7.0, 5.0, 1.0]], [[2.0, 8.0, 2.0], [5.0, 3.0, 7.0], [4.0, 
1.0, 4.0]], [[6.0, 7.0, 5.0], [6.0, 6.0, 2.0], [1.0, 3.0, 4.0]]], [[[4.0, 6.0, 5.0], [4.0, 2.0, 5.0], [2.0, 1.0, 4.0]], [[4.0, 
4.0, 7.0], [8.0, 4.0, 7.0], [6.0, 2.0, 6.0]], [[5.0, 4.0, 6.0], [1.0, 1.0, 2.0], [6.0, 5.0, 6.0]]], [[[2.0, 3.0, 8.0], [5.0, 
2.0, 2.0], [3.0, 3.0, 1.0]], [[5.0, 6.0, 4.0], [5.0, 7.0, 2.0], [1.0, 8.0, 8.0]], [[1.0, 7.0, 7.0], [5.0, 2.0, 3.0], [1.0, 6.0, 
7.0]]], [[[5.0, 4.0, 3.0], [2.0, 2.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 8.0, 4.0], [1.0, 5.0, 4.0], [8.0, 5.0, 3.0]], [[7.0, 6.0, 
3.0], [2.0, 8.0, 7.0], [3.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 2.0], [4.0, 7.0, 8.0], [2.0, 7.0, 2.0]], 
[[6.0, 5.0, 3.0], [5.0, 3.0, 7.0], [1.0, 8.0, 5.0]], [[8.0, 1.0, 4.0], [6.0, 6.0, 2.0], [1.0, 2.0, 1.0]]], [[[3.0, 1.0, 2.0], 
[4.0, 2.0, 5.0], [3.0, 6.0, 2.0]], [[5.0, 1.0, 2.0], [8.0, 4.0, 7.0], [6.0, 6.0, 7.0]], [[8.0, 6.0, 3.0], [1.0, 1.0, 2.0], 
[7.0, 3.0, 5.0]]], [[[1.0, 6.0, 8.0], [5.0, 2.0, 2.0], [6.0, 7.0, 3.0]], [[3.0, 6.0, 6.0], [5.0, 7.0, 2.0], [2.0, 1.0, 5.0]], 
[[2.0, 2.0, 3.0], [5.0, 2.0, 3.0], [1.0, 5.0, 4.0]]], [[[7.0, 5.0, 6.0], [8.0, 6.0, 2.0], [7.0, 6.0, 5.0]], [[1.0, 8.0, 7.0], 
[3.0, 7.0, 2.0], [4.0, 4.0, 7.0]], [[1.0, 4.0, 8.0], [4.0, 7.0, 7.0], [2.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UCPN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 5.0], [8.0, 1.0, 5.0], [6.0, 5.0, 2.0]], [[8.0, 4.0, 7.0], [2.0, 2.0, 8.0], [5.0, 4.0, 
5.0]], [[6.0, 4.0, 5.0], [1.0, 5.0, 7.0], [4.0, 8.0, 8.0]]], [[[1.0, 6.0, 6.0], [5.0, 4.0, 6.0], [3.0, 7.0, 7.0]], [[4.0, 4.0, 
8.0], [7.0, 7.0, 3.0], [6.0, 4.0, 1.0]], [[2.0, 3.0, 2.0], [6.0, 5.0, 7.0], [3.0, 8.0, 3.0]]], [[[8.0, 7.0, 5.0], [3.0, 3.0, 
3.0], [7.0, 4.0, 5.0]], [[6.0, 1.0, 5.0], [8.0, 2.0, 4.0], [3.0, 6.0, 8.0]], [[6.0, 7.0, 6.0], [8.0, 4.0, 1.0], [4.0, 7.0, 
5.0]]], [[[8.0, 4.0, 6.0], [6.0, 6.0, 6.0], [5.0, 2.0, 7.0]], [[2.0, 6.0, 6.0], [5.0, 6.0, 6.0], [8.0, 6.0, 6.0]], [[1.0, 2.0, 
7.0], [5.0, 2.0, 1.0], [3.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 6.0], [4.0, 4.0, 8.0], [1.0, 4.0, 1.0]], [[6.0, 1.0, 5.0], [4.0, 2.0, 
8.0], [5.0, 5.0, 4.0]], [[3.0, 4.0, 3.0], [6.0, 5.0, 4.0], [6.0, 4.0, 3.0]]], [[[8.0, 5.0, 7.0], [2.0, 2.0, 5.0], [2.0, 5.0, 
5.0]], [[8.0, 8.0, 8.0], [2.0, 2.0, 2.0], [7.0, 8.0, 5.0]], [[3.0, 4.0, 3.0], [2.0, 2.0, 3.0], [6.0, 1.0, 7.0]]], [[[1.0, 2.0, 
1.0], [4.0, 8.0, 1.0], [5.0, 3.0, 4.0]], [[8.0, 6.0, 2.0], [3.0, 2.0, 1.0], [6.0, 7.0, 5.0]], [[4.0, 7.0, 3.0], [1.0, 3.0, 
6.0], [2.0, 3.0, 7.0]]], [[[3.0, 8.0, 8.0], [5.0, 7.0, 4.0], [2.0, 4.0, 3.0]], [[3.0, 1.0, 4.0], [3.0, 1.0, 2.0], [5.0, 6.0, 
4.0]], [[4.0, 1.0, 7.0], [6.0, 8.0, 8.0], [8.0, 6.0, 1.0]]]]))
      val=Data(numpy.array([[[8.0, 7.0, 8.0], [8.0, 1.0, 6.0], [4.0, 2.0, 7.0]], [[2.0, 7.0, 5.0], [8.0, 8.0, 2.0], [7.0, 8.0, 
8.0]], [[5.0, 8.0, 8.0], [4.0, 1.0, 7.0], [4.0, 7.0, 7.0]]]),self.functionspace)
      arg[:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 5.0], [8.0, 7.0, 8.0], [6.0, 5.0, 2.0]], [[8.0, 4.0, 7.0], [8.0, 1.0, 6.0], [5.0, 
4.0, 5.0]], [[6.0, 4.0, 5.0], [4.0, 2.0, 7.0], [4.0, 8.0, 8.0]]], [[[1.0, 6.0, 6.0], [2.0, 7.0, 5.0], [3.0, 7.0, 7.0]], [[4.0, 
4.0, 8.0], [8.0, 8.0, 2.0], [6.0, 4.0, 1.0]], [[2.0, 3.0, 2.0], [7.0, 8.0, 8.0], [3.0, 8.0, 3.0]]], [[[8.0, 7.0, 5.0], [5.0, 
8.0, 8.0], [7.0, 4.0, 5.0]], [[6.0, 1.0, 5.0], [4.0, 1.0, 7.0], [3.0, 6.0, 8.0]], [[6.0, 7.0, 6.0], [4.0, 7.0, 7.0], [4.0, 7.0, 
5.0]]], [[[8.0, 4.0, 6.0], [6.0, 6.0, 6.0], [5.0, 2.0, 7.0]], [[2.0, 6.0, 6.0], [5.0, 6.0, 6.0], [8.0, 6.0, 6.0]], [[1.0, 2.0, 
7.0], [5.0, 2.0, 1.0], [3.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 6.0], [8.0, 7.0, 8.0], [1.0, 4.0, 1.0]], 
[[6.0, 1.0, 5.0], [8.0, 1.0, 6.0], [5.0, 5.0, 4.0]], [[3.0, 4.0, 3.0], [4.0, 2.0, 7.0], [6.0, 4.0, 3.0]]], [[[8.0, 5.0, 7.0], 
[2.0, 7.0, 5.0], [2.0, 5.0, 5.0]], [[8.0, 8.0, 8.0], [8.0, 8.0, 2.0], [7.0, 8.0, 5.0]], [[3.0, 4.0, 3.0], [7.0, 8.0, 8.0], 
[6.0, 1.0, 7.0]]], [[[1.0, 2.0, 1.0], [5.0, 8.0, 8.0], [5.0, 3.0, 4.0]], [[8.0, 6.0, 2.0], [4.0, 1.0, 7.0], [6.0, 7.0, 5.0]], 
[[4.0, 7.0, 3.0], [4.0, 7.0, 7.0], [2.0, 3.0, 7.0]]], [[[3.0, 8.0, 8.0], [5.0, 7.0, 4.0], [2.0, 4.0, 3.0]], [[3.0, 1.0, 4.0], 
[3.0, 1.0, 2.0], [5.0, 6.0, 4.0]], [[4.0, 1.0, 7.0], [6.0, 8.0, 8.0], [8.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UCPN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 3.0], [8.0, 1.0, 2.0], [3.0, 1.0, 5.0]], [[4.0, 3.0, 5.0], [1.0, 3.0, 5.0], [7.0, 7.0, 
7.0]], [[5.0, 8.0, 7.0], [1.0, 8.0, 3.0], [5.0, 4.0, 1.0]]], [[[3.0, 5.0, 5.0], [5.0, 7.0, 8.0], [8.0, 7.0, 6.0]], [[6.0, 2.0, 
5.0], [5.0, 1.0, 5.0], [7.0, 8.0, 7.0]], [[4.0, 4.0, 1.0], [4.0, 8.0, 7.0], [5.0, 4.0, 8.0]]], [[[4.0, 2.0, 1.0], [2.0, 8.0, 
2.0], [5.0, 8.0, 6.0]], [[7.0, 4.0, 8.0], [6.0, 3.0, 5.0], [2.0, 4.0, 2.0]], [[7.0, 2.0, 6.0], [2.0, 3.0, 3.0], [7.0, 6.0, 
6.0]]], [[[2.0, 8.0, 5.0], [1.0, 8.0, 4.0], [7.0, 4.0, 8.0]], [[5.0, 5.0, 5.0], [7.0, 1.0, 6.0], [5.0, 8.0, 1.0]], [[4.0, 3.0, 
2.0], [3.0, 8.0, 6.0], [6.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 2.0], [3.0, 7.0, 3.0], [2.0, 7.0, 6.0]], [[1.0, 5.0, 2.0], [7.0, 6.0, 
6.0], [4.0, 4.0, 7.0]], [[5.0, 3.0, 5.0], [8.0, 3.0, 2.0], [7.0, 5.0, 4.0]]], [[[6.0, 5.0, 4.0], [2.0, 6.0, 8.0], [5.0, 8.0, 
3.0]], [[3.0, 8.0, 5.0], [5.0, 3.0, 8.0], [1.0, 4.0, 4.0]], [[1.0, 4.0, 1.0], [5.0, 2.0, 3.0], [6.0, 3.0, 6.0]]], [[[1.0, 8.0, 
5.0], [7.0, 8.0, 7.0], [1.0, 1.0, 1.0]], [[2.0, 1.0, 8.0], [3.0, 7.0, 8.0], [7.0, 3.0, 4.0]], [[7.0, 7.0, 5.0], [1.0, 2.0, 
3.0], [6.0, 8.0, 2.0]]], [[[5.0, 2.0, 5.0], [5.0, 5.0, 5.0], [1.0, 1.0, 7.0]], [[5.0, 2.0, 7.0], [3.0, 8.0, 5.0], [7.0, 6.0, 
6.0]], [[1.0, 7.0, 4.0], [6.0, 1.0, 2.0], [6.0, 6.0, 2.0]]]]))
      val=Data(numpy.array([[[8.0, 3.0, 3.0], [3.0, 4.0, 7.0], [1.0, 7.0, 7.0]], [[4.0, 7.0, 5.0], [7.0, 5.0, 6.0], [6.0, 5.0, 
6.0]], [[1.0, 1.0, 7.0], [7.0, 3.0, 6.0], [5.0, 4.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 4.0, 5.0], [3.0, 6.0, 5.0], [3.0, 4.0, 6.0]], [[3.0, 8.0, 3.0], [4.0, 3.0, 7.0], 
[2.0, 1.0, 6.0]], [[4.0, 8.0, 5.0], [5.0, 6.0, 3.0], [3.0, 1.0, 5.0]]]))
      arg[:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 3.0], [8.0, 3.0, 3.0], [3.0, 1.0, 5.0]], [[4.0, 3.0, 5.0], [3.0, 4.0, 7.0], [7.0, 
7.0, 7.0]], [[5.0, 8.0, 7.0], [1.0, 7.0, 7.0], [5.0, 4.0, 1.0]]], [[[3.0, 5.0, 5.0], [4.0, 7.0, 5.0], [8.0, 7.0, 6.0]], [[6.0, 
2.0, 5.0], [7.0, 5.0, 6.0], [7.0, 8.0, 7.0]], [[4.0, 4.0, 1.0], [6.0, 5.0, 6.0], [5.0, 4.0, 8.0]]], [[[4.0, 2.0, 1.0], [1.0, 
1.0, 7.0], [5.0, 8.0, 6.0]], [[7.0, 4.0, 8.0], [7.0, 3.0, 6.0], [2.0, 4.0, 2.0]], [[7.0, 2.0, 6.0], [5.0, 4.0, 1.0], [7.0, 6.0, 
6.0]]], [[[2.0, 8.0, 5.0], [1.0, 8.0, 4.0], [7.0, 4.0, 8.0]], [[5.0, 5.0, 5.0], [7.0, 1.0, 6.0], [5.0, 8.0, 1.0]], [[4.0, 3.0, 
2.0], [3.0, 8.0, 6.0], [6.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 2.0], [1.0, 4.0, 5.0], [2.0, 7.0, 6.0]], 
[[1.0, 5.0, 2.0], [3.0, 6.0, 5.0], [4.0, 4.0, 7.0]], [[5.0, 3.0, 5.0], [3.0, 4.0, 6.0], [7.0, 5.0, 4.0]]], [[[6.0, 5.0, 4.0], 
[3.0, 8.0, 3.0], [5.0, 8.0, 3.0]], [[3.0, 8.0, 5.0], [4.0, 3.0, 7.0], [1.0, 4.0, 4.0]], [[1.0, 4.0, 1.0], [2.0, 1.0, 6.0], 
[6.0, 3.0, 6.0]]], [[[1.0, 8.0, 5.0], [4.0, 8.0, 5.0], [1.0, 1.0, 1.0]], [[2.0, 1.0, 8.0], [5.0, 6.0, 3.0], [7.0, 3.0, 4.0]], 
[[7.0, 7.0, 5.0], [3.0, 1.0, 5.0], [6.0, 8.0, 2.0]]], [[[5.0, 2.0, 5.0], [5.0, 5.0, 5.0], [1.0, 1.0, 7.0]], [[5.0, 2.0, 7.0], 
[3.0, 8.0, 5.0], [7.0, 6.0, 6.0]], [[1.0, 7.0, 4.0], [6.0, 1.0, 2.0], [6.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UCPN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 5.0], [6.0, 3.0, 8.0], [3.0, 3.0, 8.0]], [[2.0, 3.0, 8.0], [6.0, 5.0, 3.0], [1.0, 1.0, 
7.0]], [[5.0, 5.0, 3.0], [8.0, 7.0, 1.0], [4.0, 7.0, 1.0]]], [[[3.0, 6.0, 8.0], [7.0, 5.0, 4.0], [4.0, 7.0, 6.0]], [[8.0, 7.0, 
2.0], [4.0, 5.0, 2.0], [8.0, 8.0, 6.0]], [[6.0, 7.0, 2.0], [6.0, 5.0, 6.0], [4.0, 4.0, 8.0]]], [[[5.0, 1.0, 5.0], [3.0, 3.0, 
4.0], [8.0, 5.0, 5.0]], [[8.0, 3.0, 6.0], [7.0, 5.0, 4.0], [5.0, 8.0, 5.0]], [[3.0, 4.0, 1.0], [8.0, 1.0, 1.0], [1.0, 8.0, 
7.0]]], [[[7.0, 5.0, 7.0], [6.0, 8.0, 1.0], [3.0, 5.0, 2.0]], [[8.0, 7.0, 2.0], [3.0, 6.0, 2.0], [2.0, 5.0, 8.0]], [[8.0, 7.0, 
6.0], [7.0, 5.0, 3.0], [2.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 8.0], [5.0, 2.0, 1.0], [7.0, 4.0, 6.0]], [[5.0, 4.0, 8.0], [5.0, 2.0, 
7.0], [3.0, 1.0, 1.0]], [[3.0, 6.0, 7.0], [3.0, 8.0, 7.0], [6.0, 8.0, 1.0]]], [[[2.0, 4.0, 1.0], [8.0, 1.0, 8.0], [6.0, 4.0, 
3.0]], [[6.0, 2.0, 5.0], [7.0, 5.0, 3.0], [3.0, 8.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 2.0, 1.0], [8.0, 4.0, 2.0]]], [[[1.0, 5.0, 
5.0], [6.0, 1.0, 6.0], [2.0, 7.0, 2.0]], [[3.0, 4.0, 7.0], [8.0, 6.0, 8.0], [3.0, 4.0, 6.0]], [[1.0, 2.0, 5.0], [6.0, 4.0, 
3.0], [6.0, 6.0, 2.0]]], [[[5.0, 5.0, 3.0], [1.0, 5.0, 8.0], [1.0, 7.0, 5.0]], [[1.0, 5.0, 4.0], [8.0, 1.0, 3.0], [1.0, 3.0, 
7.0]], [[1.0, 6.0, 8.0], [4.0, 3.0, 1.0], [7.0, 5.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 5.0, 7.0], [4.0, 7.0, 6.0], [4.0, 8.0, 3.0]], [[5.0, 3.0, 1.0], [5.0, 6.0, 2.0], [5.0, 
5.0, 5.0]], [[5.0, 4.0, 8.0], [8.0, 7.0, 8.0], [4.0, 1.0, 3.0]]])+(1.-msk_val)*numpy.array([[[3.0, 5.0, 1.0], [1.0, 1.0, 3.0], 
[1.0, 7.0, 4.0]], [[3.0, 1.0, 3.0], [1.0, 8.0, 5.0], [3.0, 5.0, 2.0]], [[1.0, 1.0, 5.0], [1.0, 3.0, 6.0], [6.0, 5.0, 3.0]]])
      arg[:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 5.0], [4.0, 5.0, 7.0], [3.0, 3.0, 8.0]], [[2.0, 3.0, 8.0], [4.0, 7.0, 6.0], [1.0, 
1.0, 7.0]], [[5.0, 5.0, 3.0], [4.0, 8.0, 3.0], [4.0, 7.0, 1.0]]], [[[3.0, 6.0, 8.0], [5.0, 3.0, 1.0], [4.0, 7.0, 6.0]], [[8.0, 
7.0, 2.0], [5.0, 6.0, 2.0], [8.0, 8.0, 6.0]], [[6.0, 7.0, 2.0], [5.0, 5.0, 5.0], [4.0, 4.0, 8.0]]], [[[5.0, 1.0, 5.0], [5.0, 
4.0, 8.0], [8.0, 5.0, 5.0]], [[8.0, 3.0, 6.0], [8.0, 7.0, 8.0], [5.0, 8.0, 5.0]], [[3.0, 4.0, 1.0], [4.0, 1.0, 3.0], [1.0, 8.0, 
7.0]]], [[[7.0, 5.0, 7.0], [6.0, 8.0, 1.0], [3.0, 5.0, 2.0]], [[8.0, 7.0, 2.0], [3.0, 6.0, 2.0], [2.0, 5.0, 8.0]], [[8.0, 7.0, 
6.0], [7.0, 5.0, 3.0], [2.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 8.0], [3.0, 5.0, 1.0], [7.0, 4.0, 6.0]], 
[[5.0, 4.0, 8.0], [1.0, 1.0, 3.0], [3.0, 1.0, 1.0]], [[3.0, 6.0, 7.0], [1.0, 7.0, 4.0], [6.0, 8.0, 1.0]]], [[[2.0, 4.0, 1.0], 
[3.0, 1.0, 3.0], [6.0, 4.0, 3.0]], [[6.0, 2.0, 5.0], [1.0, 8.0, 5.0], [3.0, 8.0, 3.0]], [[7.0, 7.0, 1.0], [3.0, 5.0, 2.0], 
[8.0, 4.0, 2.0]]], [[[1.0, 5.0, 5.0], [1.0, 1.0, 5.0], [2.0, 7.0, 2.0]], [[3.0, 4.0, 7.0], [1.0, 3.0, 6.0], [3.0, 4.0, 6.0]], 
[[1.0, 2.0, 5.0], [6.0, 5.0, 3.0], [6.0, 6.0, 2.0]]], [[[5.0, 5.0, 3.0], [1.0, 5.0, 8.0], [1.0, 7.0, 5.0]], [[1.0, 5.0, 4.0], 
[8.0, 1.0, 3.0], [1.0, 3.0, 7.0]], [[1.0, 6.0, 8.0], [4.0, 3.0, 1.0], [7.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RLLL(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 8.0], [8.0, 5.0, 4.0], [5.0, 8.0, 6.0]], [[4.0, 3.0, 7.0], [7.0, 2.0, 5.0], [7.0, 2.0, 
8.0]], [[3.0, 2.0, 1.0], [2.0, 7.0, 5.0], [2.0, 6.0, 4.0]]], [[[8.0, 3.0, 2.0], [4.0, 6.0, 5.0], [5.0, 2.0, 6.0]], [[1.0, 7.0, 
4.0], [4.0, 2.0, 8.0], [4.0, 5.0, 3.0]], [[1.0, 6.0, 1.0], [8.0, 3.0, 2.0], [7.0, 5.0, 6.0]]], [[[7.0, 4.0, 4.0], [4.0, 2.0, 
3.0], [2.0, 4.0, 8.0]], [[5.0, 8.0, 1.0], [2.0, 2.0, 4.0], [2.0, 6.0, 3.0]], [[8.0, 3.0, 2.0], [7.0, 5.0, 1.0], [6.0, 1.0, 
5.0]]], [[[8.0, 8.0, 6.0], [7.0, 3.0, 5.0], [6.0, 6.0, 3.0]], [[4.0, 1.0, 1.0], [3.0, 7.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 3.0, 
6.0], [6.0, 2.0, 4.0], [2.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 1.0], [8.0, 6.0, 5.0], [4.0, 2.0, 6.0]], [[2.0, 5.0, 1.0], [2.0, 4.0, 
2.0], [5.0, 1.0, 4.0]], [[5.0, 6.0, 6.0], [4.0, 6.0, 2.0], [8.0, 5.0, 6.0]]], [[[8.0, 4.0, 1.0], [8.0, 1.0, 7.0], [5.0, 1.0, 
4.0]], [[1.0, 2.0, 6.0], [8.0, 8.0, 5.0], [2.0, 4.0, 1.0]], [[8.0, 1.0, 5.0], [1.0, 3.0, 6.0], [2.0, 1.0, 1.0]]], [[[4.0, 4.0, 
7.0], [5.0, 7.0, 5.0], [7.0, 5.0, 6.0]], [[1.0, 8.0, 4.0], [7.0, 7.0, 3.0], [2.0, 1.0, 3.0]], [[2.0, 4.0, 4.0], [4.0, 8.0, 
7.0], [8.0, 1.0, 2.0]]], [[[8.0, 8.0, 7.0], [1.0, 5.0, 6.0], [3.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], [8.0, 6.0, 5.0], [3.0, 5.0, 
7.0]], [[8.0, 7.0, 2.0], [4.0, 2.0, 8.0], [5.0, 4.0, 4.0]]]]))
      val=numpy.array([[[[2.0]]], [[[8.0]]]])
      arg[1:3,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 8.0], [8.0, 5.0, 4.0], [5.0, 8.0, 6.0]], [[4.0, 3.0, 7.0], [7.0, 2.0, 5.0], [7.0, 
2.0, 8.0]], [[3.0, 2.0, 1.0], [2.0, 7.0, 5.0], [2.0, 6.0, 4.0]]], [[[8.0, 3.0, 2.0], [4.0, 6.0, 5.0], [5.0, 2.0, 6.0]], [[1.0, 
7.0, 4.0], [4.0, 2.0, 8.0], [4.0, 5.0, 3.0]], [[1.0, 6.0, 1.0], [8.0, 3.0, 2.0], [7.0, 5.0, 2.0]]], [[[7.0, 4.0, 4.0], [4.0, 
2.0, 3.0], [2.0, 4.0, 8.0]], [[5.0, 8.0, 1.0], [2.0, 2.0, 4.0], [2.0, 6.0, 3.0]], [[8.0, 3.0, 2.0], [7.0, 5.0, 1.0], [6.0, 1.0, 
8.0]]], [[[8.0, 8.0, 6.0], [7.0, 3.0, 5.0], [6.0, 6.0, 3.0]], [[4.0, 1.0, 1.0], [3.0, 7.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 3.0, 
6.0], [6.0, 2.0, 4.0], [2.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 1.0], [8.0, 6.0, 5.0], [4.0, 2.0, 6.0]], 
[[2.0, 5.0, 1.0], [2.0, 4.0, 2.0], [5.0, 1.0, 4.0]], [[5.0, 6.0, 6.0], [4.0, 6.0, 2.0], [8.0, 5.0, 6.0]]], [[[8.0, 4.0, 1.0], 
[8.0, 1.0, 7.0], [5.0, 1.0, 4.0]], [[1.0, 2.0, 6.0], [8.0, 8.0, 5.0], [2.0, 4.0, 1.0]], [[8.0, 1.0, 5.0], [1.0, 3.0, 6.0], 
[2.0, 1.0, 2.0]]], [[[4.0, 4.0, 7.0], [5.0, 7.0, 5.0], [7.0, 5.0, 6.0]], [[1.0, 8.0, 4.0], [7.0, 7.0, 3.0], [2.0, 1.0, 3.0]], 
[[2.0, 4.0, 4.0], [4.0, 8.0, 7.0], [8.0, 1.0, 8.0]]], [[[8.0, 8.0, 7.0], [1.0, 5.0, 6.0], [3.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], 
[8.0, 6.0, 5.0], [3.0, 5.0, 7.0]], [[8.0, 7.0, 2.0], [4.0, 2.0, 8.0], [5.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RLLL(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 6.0], [7.0, 5.0, 1.0], [3.0, 8.0, 5.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 6.0], [3.0, 6.0, 
6.0]], [[2.0, 4.0, 5.0], [2.0, 4.0, 6.0], [8.0, 7.0, 1.0]]], [[[5.0, 1.0, 1.0], [6.0, 5.0, 4.0], [7.0, 2.0, 6.0]], [[1.0, 6.0, 
2.0], [1.0, 5.0, 5.0], [6.0, 6.0, 7.0]], [[6.0, 4.0, 3.0], [8.0, 6.0, 7.0], [5.0, 2.0, 4.0]]], [[[2.0, 2.0, 3.0], [2.0, 4.0, 
6.0], [8.0, 6.0, 4.0]], [[6.0, 5.0, 8.0], [2.0, 6.0, 6.0], [6.0, 1.0, 4.0]], [[8.0, 2.0, 6.0], [7.0, 7.0, 2.0], [6.0, 8.0, 
1.0]]], [[[5.0, 2.0, 8.0], [5.0, 5.0, 7.0], [6.0, 6.0, 2.0]], [[8.0, 6.0, 7.0], [5.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[2.0, 4.0, 
5.0], [4.0, 2.0, 3.0], [4.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 1.0], [8.0, 7.0, 6.0], [5.0, 1.0, 2.0]], [[7.0, 1.0, 1.0], [7.0, 1.0, 
5.0], [2.0, 1.0, 4.0]], [[3.0, 6.0, 4.0], [6.0, 3.0, 5.0], [1.0, 7.0, 1.0]]], [[[1.0, 8.0, 5.0], [4.0, 2.0, 3.0], [3.0, 5.0, 
4.0]], [[6.0, 6.0, 5.0], [8.0, 4.0, 1.0], [2.0, 3.0, 6.0]], [[4.0, 5.0, 4.0], [2.0, 4.0, 3.0], [8.0, 3.0, 5.0]]], [[[3.0, 4.0, 
5.0], [1.0, 7.0, 7.0], [6.0, 5.0, 6.0]], [[8.0, 7.0, 2.0], [7.0, 4.0, 6.0], [5.0, 1.0, 7.0]], [[4.0, 8.0, 4.0], [6.0, 2.0, 
6.0], [1.0, 2.0, 6.0]]], [[[1.0, 8.0, 1.0], [6.0, 8.0, 7.0], [4.0, 6.0, 7.0]], [[1.0, 1.0, 6.0], [6.0, 7.0, 2.0], [2.0, 3.0, 
3.0]], [[2.0, 2.0, 1.0], [8.0, 6.0, 4.0], [5.0, 2.0, 5.0]]]]))
      val=Data(numpy.array([[[[1.0]]], [[[3.0]]]]),self.functionspace)
      arg[1:3,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 6.0], [7.0, 5.0, 1.0], [3.0, 8.0, 5.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 6.0], [3.0, 
6.0, 6.0]], [[2.0, 4.0, 5.0], [2.0, 4.0, 6.0], [8.0, 7.0, 1.0]]], [[[5.0, 1.0, 1.0], [6.0, 5.0, 4.0], [7.0, 2.0, 6.0]], [[1.0, 
6.0, 2.0], [1.0, 5.0, 5.0], [6.0, 6.0, 7.0]], [[6.0, 4.0, 3.0], [8.0, 6.0, 7.0], [5.0, 2.0, 1.0]]], [[[2.0, 2.0, 3.0], [2.0, 
4.0, 6.0], [8.0, 6.0, 4.0]], [[6.0, 5.0, 8.0], [2.0, 6.0, 6.0], [6.0, 1.0, 4.0]], [[8.0, 2.0, 6.0], [7.0, 7.0, 2.0], [6.0, 8.0, 
3.0]]], [[[5.0, 2.0, 8.0], [5.0, 5.0, 7.0], [6.0, 6.0, 2.0]], [[8.0, 6.0, 7.0], [5.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[2.0, 4.0, 
5.0], [4.0, 2.0, 3.0], [4.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 1.0], [8.0, 7.0, 6.0], [5.0, 1.0, 2.0]], 
[[7.0, 1.0, 1.0], [7.0, 1.0, 5.0], [2.0, 1.0, 4.0]], [[3.0, 6.0, 4.0], [6.0, 3.0, 5.0], [1.0, 7.0, 1.0]]], [[[1.0, 8.0, 5.0], 
[4.0, 2.0, 3.0], [3.0, 5.0, 4.0]], [[6.0, 6.0, 5.0], [8.0, 4.0, 1.0], [2.0, 3.0, 6.0]], [[4.0, 5.0, 4.0], [2.0, 4.0, 3.0], 
[8.0, 3.0, 1.0]]], [[[3.0, 4.0, 5.0], [1.0, 7.0, 7.0], [6.0, 5.0, 6.0]], [[8.0, 7.0, 2.0], [7.0, 4.0, 6.0], [5.0, 1.0, 7.0]], 
[[4.0, 8.0, 4.0], [6.0, 2.0, 6.0], [1.0, 2.0, 3.0]]], [[[1.0, 8.0, 1.0], [6.0, 8.0, 7.0], [4.0, 6.0, 7.0]], [[1.0, 1.0, 6.0], 
[6.0, 7.0, 2.0], [2.0, 3.0, 3.0]], [[2.0, 2.0, 1.0], [8.0, 6.0, 4.0], [5.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RLLL(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 1.0], [7.0, 1.0, 1.0], [6.0, 4.0, 4.0]], [[4.0, 5.0, 8.0], [1.0, 4.0, 8.0], [5.0, 3.0, 
7.0]], [[1.0, 2.0, 2.0], [5.0, 5.0, 8.0], [6.0, 8.0, 2.0]]], [[[2.0, 6.0, 2.0], [6.0, 5.0, 6.0], [7.0, 7.0, 4.0]], [[1.0, 7.0, 
4.0], [5.0, 5.0, 4.0], [8.0, 4.0, 1.0]], [[4.0, 3.0, 7.0], [7.0, 1.0, 7.0], [5.0, 3.0, 7.0]]], [[[5.0, 8.0, 1.0], [4.0, 6.0, 
5.0], [5.0, 4.0, 3.0]], [[4.0, 8.0, 6.0], [3.0, 2.0, 7.0], [1.0, 3.0, 8.0]], [[1.0, 7.0, 7.0], [3.0, 5.0, 1.0], [7.0, 7.0, 
6.0]]], [[[6.0, 6.0, 1.0], [5.0, 3.0, 3.0], [3.0, 1.0, 8.0]], [[4.0, 7.0, 3.0], [1.0, 8.0, 1.0], [3.0, 3.0, 8.0]], [[3.0, 1.0, 
7.0], [7.0, 1.0, 7.0], [5.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 2.0], [3.0, 6.0, 1.0], [4.0, 2.0, 2.0]], [[3.0, 6.0, 5.0], [4.0, 8.0, 
6.0], [8.0, 8.0, 5.0]], [[2.0, 2.0, 4.0], [6.0, 7.0, 2.0], [2.0, 8.0, 8.0]]], [[[2.0, 4.0, 6.0], [8.0, 4.0, 3.0], [3.0, 8.0, 
5.0]], [[4.0, 2.0, 7.0], [1.0, 1.0, 3.0], [4.0, 1.0, 6.0]], [[3.0, 2.0, 8.0], [6.0, 1.0, 8.0], [5.0, 2.0, 5.0]]], [[[4.0, 3.0, 
8.0], [6.0, 2.0, 7.0], [1.0, 7.0, 8.0]], [[6.0, 6.0, 3.0], [2.0, 7.0, 5.0], [2.0, 3.0, 4.0]], [[3.0, 3.0, 1.0], [5.0, 3.0, 
7.0], [6.0, 5.0, 8.0]]], [[[7.0, 4.0, 2.0], [1.0, 6.0, 2.0], [7.0, 4.0, 7.0]], [[5.0, 4.0, 5.0], [1.0, 1.0, 3.0], [8.0, 2.0, 
6.0]], [[6.0, 8.0, 4.0], [3.0, 7.0, 1.0], [3.0, 5.0, 3.0]]]]))
      val=Data(numpy.array([[[[1.0]]], [[[1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0]]], [[[8.0]]]]))
      arg[1:3,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 1.0], [7.0, 1.0, 1.0], [6.0, 4.0, 4.0]], [[4.0, 5.0, 8.0], [1.0, 4.0, 8.0], [5.0, 
3.0, 7.0]], [[1.0, 2.0, 2.0], [5.0, 5.0, 8.0], [6.0, 8.0, 2.0]]], [[[2.0, 6.0, 2.0], [6.0, 5.0, 6.0], [7.0, 7.0, 4.0]], [[1.0, 
7.0, 4.0], [5.0, 5.0, 4.0], [8.0, 4.0, 1.0]], [[4.0, 3.0, 7.0], [7.0, 1.0, 7.0], [5.0, 3.0, 1.0]]], [[[5.0, 8.0, 1.0], [4.0, 
6.0, 5.0], [5.0, 4.0, 3.0]], [[4.0, 8.0, 6.0], [3.0, 2.0, 7.0], [1.0, 3.0, 8.0]], [[1.0, 7.0, 7.0], [3.0, 5.0, 1.0], [7.0, 7.0, 
1.0]]], [[[6.0, 6.0, 1.0], [5.0, 3.0, 3.0], [3.0, 1.0, 8.0]], [[4.0, 7.0, 3.0], [1.0, 8.0, 1.0], [3.0, 3.0, 8.0]], [[3.0, 1.0, 
7.0], [7.0, 1.0, 7.0], [5.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 2.0], [3.0, 6.0, 1.0], [4.0, 2.0, 2.0]], 
[[3.0, 6.0, 5.0], [4.0, 8.0, 6.0], [8.0, 8.0, 5.0]], [[2.0, 2.0, 4.0], [6.0, 7.0, 2.0], [2.0, 8.0, 8.0]]], [[[2.0, 4.0, 6.0], 
[8.0, 4.0, 3.0], [3.0, 8.0, 5.0]], [[4.0, 2.0, 7.0], [1.0, 1.0, 3.0], [4.0, 1.0, 6.0]], [[3.0, 2.0, 8.0], [6.0, 1.0, 8.0], 
[5.0, 2.0, 4.0]]], [[[4.0, 3.0, 8.0], [6.0, 2.0, 7.0], [1.0, 7.0, 8.0]], [[6.0, 6.0, 3.0], [2.0, 7.0, 5.0], [2.0, 3.0, 4.0]], 
[[3.0, 3.0, 1.0], [5.0, 3.0, 7.0], [6.0, 5.0, 8.0]]], [[[7.0, 4.0, 2.0], [1.0, 6.0, 2.0], [7.0, 4.0, 7.0]], [[5.0, 4.0, 5.0], 
[1.0, 1.0, 3.0], [8.0, 2.0, 6.0]], [[6.0, 8.0, 4.0], [3.0, 7.0, 1.0], [3.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RLLL(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 5.0], [5.0, 2.0, 7.0], [1.0, 3.0, 7.0]], [[4.0, 3.0, 5.0], [4.0, 4.0, 7.0], [6.0, 8.0, 
5.0]], [[5.0, 2.0, 6.0], [7.0, 1.0, 3.0], [7.0, 5.0, 4.0]]], [[[5.0, 8.0, 3.0], [6.0, 3.0, 3.0], [8.0, 7.0, 2.0]], [[1.0, 1.0, 
8.0], [1.0, 1.0, 6.0], [8.0, 2.0, 6.0]], [[5.0, 4.0, 1.0], [7.0, 7.0, 8.0], [4.0, 1.0, 7.0]]], [[[5.0, 2.0, 5.0], [6.0, 2.0, 
1.0], [7.0, 5.0, 5.0]], [[6.0, 4.0, 1.0], [4.0, 3.0, 7.0], [8.0, 7.0, 1.0]], [[7.0, 8.0, 5.0], [4.0, 8.0, 7.0], [2.0, 6.0, 
3.0]]], [[[5.0, 6.0, 6.0], [8.0, 7.0, 1.0], [2.0, 5.0, 7.0]], [[4.0, 4.0, 2.0], [4.0, 5.0, 2.0], [2.0, 4.0, 4.0]], [[5.0, 6.0, 
7.0], [4.0, 6.0, 8.0], [8.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 3.0], [7.0, 7.0, 4.0], [4.0, 8.0, 5.0]], [[1.0, 2.0, 8.0], [8.0, 3.0, 
7.0], [5.0, 5.0, 8.0]], [[1.0, 7.0, 2.0], [1.0, 1.0, 6.0], [5.0, 3.0, 7.0]]], [[[5.0, 6.0, 2.0], [2.0, 5.0, 1.0], [3.0, 6.0, 
7.0]], [[3.0, 5.0, 7.0], [2.0, 2.0, 4.0], [5.0, 2.0, 2.0]], [[8.0, 5.0, 6.0], [7.0, 1.0, 6.0], [8.0, 3.0, 2.0]]], [[[8.0, 2.0, 
2.0], [5.0, 4.0, 3.0], [7.0, 4.0, 2.0]], [[3.0, 4.0, 6.0], [2.0, 6.0, 6.0], [7.0, 1.0, 2.0]], [[2.0, 8.0, 4.0], [1.0, 3.0, 
4.0], [3.0, 1.0, 4.0]]], [[[2.0, 7.0, 6.0], [4.0, 3.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 1.0, 3.0], [8.0, 2.0, 5.0], [8.0, 5.0, 
1.0]], [[1.0, 6.0, 7.0], [4.0, 1.0, 4.0], [2.0, 6.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0]]], [[[2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0]]], [[[4.0]]]])
      arg[1:3,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 5.0], [5.0, 2.0, 7.0], [1.0, 3.0, 7.0]], [[4.0, 3.0, 5.0], [4.0, 4.0, 7.0], [6.0, 
8.0, 5.0]], [[5.0, 2.0, 6.0], [7.0, 1.0, 3.0], [7.0, 5.0, 4.0]]], [[[5.0, 8.0, 3.0], [6.0, 3.0, 3.0], [8.0, 7.0, 2.0]], [[1.0, 
1.0, 8.0], [1.0, 1.0, 6.0], [8.0, 2.0, 6.0]], [[5.0, 4.0, 1.0], [7.0, 7.0, 8.0], [4.0, 1.0, 6.0]]], [[[5.0, 2.0, 5.0], [6.0, 
2.0, 1.0], [7.0, 5.0, 5.0]], [[6.0, 4.0, 1.0], [4.0, 3.0, 7.0], [8.0, 7.0, 1.0]], [[7.0, 8.0, 5.0], [4.0, 8.0, 7.0], [2.0, 6.0, 
2.0]]], [[[5.0, 6.0, 6.0], [8.0, 7.0, 1.0], [2.0, 5.0, 7.0]], [[4.0, 4.0, 2.0], [4.0, 5.0, 2.0], [2.0, 4.0, 4.0]], [[5.0, 6.0, 
7.0], [4.0, 6.0, 8.0], [8.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 3.0], [7.0, 7.0, 4.0], [4.0, 8.0, 5.0]], 
[[1.0, 2.0, 8.0], [8.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[1.0, 7.0, 2.0], [1.0, 1.0, 6.0], [5.0, 3.0, 7.0]]], [[[5.0, 6.0, 2.0], 
[2.0, 5.0, 1.0], [3.0, 6.0, 7.0]], [[3.0, 5.0, 7.0], [2.0, 2.0, 4.0], [5.0, 2.0, 2.0]], [[8.0, 5.0, 6.0], [7.0, 1.0, 6.0], 
[8.0, 3.0, 5.0]]], [[[8.0, 2.0, 2.0], [5.0, 4.0, 3.0], [7.0, 4.0, 2.0]], [[3.0, 4.0, 6.0], [2.0, 6.0, 6.0], [7.0, 1.0, 2.0]], 
[[2.0, 8.0, 4.0], [1.0, 3.0, 4.0], [3.0, 1.0, 4.0]]], [[[2.0, 7.0, 6.0], [4.0, 3.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 1.0, 3.0], 
[8.0, 2.0, 5.0], [8.0, 5.0, 1.0]], [[1.0, 6.0, 7.0], [4.0, 1.0, 4.0], [2.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRCN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 4.0], [4.0, 3.0, 1.0], [6.0, 4.0, 6.0]], [[4.0, 6.0, 1.0], [6.0, 3.0, 6.0], [7.0, 6.0, 
6.0]], [[5.0, 6.0, 4.0], [4.0, 1.0, 6.0], [3.0, 3.0, 6.0]]], [[[2.0, 1.0, 5.0], [5.0, 2.0, 4.0], [8.0, 4.0, 7.0]], [[4.0, 3.0, 
6.0], [7.0, 7.0, 1.0], [1.0, 5.0, 1.0]], [[7.0, 7.0, 2.0], [1.0, 8.0, 7.0], [2.0, 8.0, 3.0]]], [[[1.0, 5.0, 4.0], [7.0, 8.0, 
1.0], [6.0, 4.0, 2.0]], [[1.0, 3.0, 8.0], [2.0, 1.0, 3.0], [4.0, 1.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 8.0, 6.0], [2.0, 4.0, 
8.0]]], [[[5.0, 5.0, 2.0], [5.0, 6.0, 7.0], [8.0, 3.0, 4.0]], [[3.0, 6.0, 5.0], [4.0, 4.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 6.0, 
3.0], [7.0, 8.0, 4.0], [3.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 5.0], [6.0, 6.0, 1.0], [5.0, 3.0, 8.0]], [[5.0, 7.0, 8.0], [4.0, 7.0, 
7.0], [4.0, 7.0, 8.0]], [[8.0, 4.0, 6.0], [8.0, 7.0, 5.0], [1.0, 5.0, 4.0]]], [[[2.0, 4.0, 2.0], [4.0, 1.0, 8.0], [6.0, 3.0, 
4.0]], [[2.0, 7.0, 1.0], [2.0, 2.0, 1.0], [2.0, 2.0, 7.0]], [[5.0, 2.0, 4.0], [6.0, 3.0, 1.0], [3.0, 6.0, 4.0]]], [[[6.0, 4.0, 
1.0], [1.0, 7.0, 1.0], [7.0, 4.0, 3.0]], [[5.0, 5.0, 8.0], [7.0, 6.0, 2.0], [4.0, 4.0, 7.0]], [[8.0, 8.0, 8.0], [8.0, 2.0, 
2.0], [8.0, 5.0, 6.0]]], [[[5.0, 2.0, 3.0], [7.0, 6.0, 4.0], [3.0, 6.0, 1.0]], [[5.0, 6.0, 6.0], [7.0, 7.0, 1.0], [7.0, 7.0, 
2.0]], [[4.0, 3.0, 4.0], [4.0, 3.0, 5.0], [6.0, 4.0, 8.0]]]]))
      val=numpy.array([[[[2.0, 6.0, 8.0], [6.0, 1.0, 3.0], [8.0, 6.0, 7.0]]], [[[1.0, 8.0, 5.0], [1.0, 2.0, 5.0], [2.0, 1.0, 
3.0]]]])
      arg[1:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 4.0], [4.0, 3.0, 1.0], [6.0, 4.0, 6.0]], [[4.0, 6.0, 1.0], [6.0, 3.0, 6.0], [7.0, 
6.0, 6.0]], [[5.0, 6.0, 4.0], [4.0, 1.0, 6.0], [3.0, 3.0, 6.0]]], [[[2.0, 1.0, 5.0], [5.0, 2.0, 4.0], [8.0, 4.0, 7.0]], [[2.0, 
6.0, 8.0], [6.0, 1.0, 3.0], [8.0, 6.0, 7.0]], [[7.0, 7.0, 2.0], [1.0, 8.0, 7.0], [2.0, 8.0, 3.0]]], [[[1.0, 5.0, 4.0], [7.0, 
8.0, 1.0], [6.0, 4.0, 2.0]], [[1.0, 8.0, 5.0], [1.0, 2.0, 5.0], [2.0, 1.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 8.0, 6.0], [2.0, 4.0, 
8.0]]], [[[5.0, 5.0, 2.0], [5.0, 6.0, 7.0], [8.0, 3.0, 4.0]], [[3.0, 6.0, 5.0], [4.0, 4.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 6.0, 
3.0], [7.0, 8.0, 4.0], [3.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 5.0], [6.0, 6.0, 1.0], [5.0, 3.0, 8.0]], 
[[5.0, 7.0, 8.0], [4.0, 7.0, 7.0], [4.0, 7.0, 8.0]], [[8.0, 4.0, 6.0], [8.0, 7.0, 5.0], [1.0, 5.0, 4.0]]], [[[2.0, 4.0, 2.0], 
[4.0, 1.0, 8.0], [6.0, 3.0, 4.0]], [[2.0, 6.0, 8.0], [6.0, 1.0, 3.0], [8.0, 6.0, 7.0]], [[5.0, 2.0, 4.0], [6.0, 3.0, 1.0], 
[3.0, 6.0, 4.0]]], [[[6.0, 4.0, 1.0], [1.0, 7.0, 1.0], [7.0, 4.0, 3.0]], [[1.0, 8.0, 5.0], [1.0, 2.0, 5.0], [2.0, 1.0, 3.0]], 
[[8.0, 8.0, 8.0], [8.0, 2.0, 2.0], [8.0, 5.0, 6.0]]], [[[5.0, 2.0, 3.0], [7.0, 6.0, 4.0], [3.0, 6.0, 1.0]], [[5.0, 6.0, 6.0], 
[7.0, 7.0, 1.0], [7.0, 7.0, 2.0]], [[4.0, 3.0, 4.0], [4.0, 3.0, 5.0], [6.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRCN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 7.0], [8.0, 5.0, 2.0], [3.0, 5.0, 7.0]], [[3.0, 4.0, 8.0], [2.0, 7.0, 5.0], [7.0, 5.0, 
5.0]], [[5.0, 4.0, 4.0], [4.0, 6.0, 1.0], [4.0, 6.0, 1.0]]], [[[6.0, 3.0, 1.0], [4.0, 8.0, 1.0], [1.0, 3.0, 2.0]], [[5.0, 1.0, 
2.0], [7.0, 6.0, 7.0], [3.0, 7.0, 5.0]], [[2.0, 4.0, 6.0], [6.0, 5.0, 7.0], [3.0, 6.0, 2.0]]], [[[2.0, 8.0, 2.0], [4.0, 6.0, 
1.0], [3.0, 2.0, 6.0]], [[5.0, 8.0, 8.0], [1.0, 5.0, 1.0], [3.0, 2.0, 2.0]], [[4.0, 8.0, 5.0], [3.0, 1.0, 2.0], [3.0, 8.0, 
6.0]]], [[[6.0, 3.0, 8.0], [5.0, 2.0, 3.0], [2.0, 7.0, 3.0]], [[8.0, 5.0, 7.0], [2.0, 6.0, 5.0], [4.0, 2.0, 8.0]], [[8.0, 8.0, 
2.0], [1.0, 3.0, 5.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 5.0], [5.0, 2.0, 6.0], [3.0, 4.0, 2.0]], [[6.0, 8.0, 8.0], [4.0, 8.0, 
1.0], [8.0, 2.0, 5.0]], [[8.0, 8.0, 7.0], [5.0, 4.0, 3.0], [5.0, 8.0, 3.0]]], [[[8.0, 7.0, 7.0], [5.0, 6.0, 7.0], [7.0, 7.0, 
4.0]], [[5.0, 6.0, 4.0], [8.0, 8.0, 6.0], [4.0, 8.0, 3.0]], [[8.0, 7.0, 1.0], [5.0, 6.0, 8.0], [4.0, 5.0, 2.0]]], [[[4.0, 4.0, 
8.0], [6.0, 4.0, 6.0], [6.0, 2.0, 5.0]], [[7.0, 8.0, 2.0], [6.0, 2.0, 6.0], [3.0, 6.0, 2.0]], [[5.0, 3.0, 6.0], [2.0, 3.0, 
7.0], [6.0, 5.0, 2.0]]], [[[6.0, 8.0, 1.0], [3.0, 7.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 7.0, 4.0], [4.0, 5.0, 4.0], [5.0, 3.0, 
5.0]], [[4.0, 2.0, 5.0], [7.0, 3.0, 7.0], [7.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[[4.0, 1.0, 7.0], [2.0, 8.0, 7.0], [7.0, 1.0, 7.0]]], [[[6.0, 6.0, 8.0], [1.0, 4.0, 7.0], [7.0, 
5.0, 3.0]]]]),self.functionspace)
      arg[1:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 7.0], [8.0, 5.0, 2.0], [3.0, 5.0, 7.0]], [[3.0, 4.0, 8.0], [2.0, 7.0, 5.0], [7.0, 
5.0, 5.0]], [[5.0, 4.0, 4.0], [4.0, 6.0, 1.0], [4.0, 6.0, 1.0]]], [[[6.0, 3.0, 1.0], [4.0, 8.0, 1.0], [1.0, 3.0, 2.0]], [[4.0, 
1.0, 7.0], [2.0, 8.0, 7.0], [7.0, 1.0, 7.0]], [[2.0, 4.0, 6.0], [6.0, 5.0, 7.0], [3.0, 6.0, 2.0]]], [[[2.0, 8.0, 2.0], [4.0, 
6.0, 1.0], [3.0, 2.0, 6.0]], [[6.0, 6.0, 8.0], [1.0, 4.0, 7.0], [7.0, 5.0, 3.0]], [[4.0, 8.0, 5.0], [3.0, 1.0, 2.0], [3.0, 8.0, 
6.0]]], [[[6.0, 3.0, 8.0], [5.0, 2.0, 3.0], [2.0, 7.0, 3.0]], [[8.0, 5.0, 7.0], [2.0, 6.0, 5.0], [4.0, 2.0, 8.0]], [[8.0, 8.0, 
2.0], [1.0, 3.0, 5.0], [8.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 5.0], [5.0, 2.0, 6.0], [3.0, 4.0, 2.0]], 
[[6.0, 8.0, 8.0], [4.0, 8.0, 1.0], [8.0, 2.0, 5.0]], [[8.0, 8.0, 7.0], [5.0, 4.0, 3.0], [5.0, 8.0, 3.0]]], [[[8.0, 7.0, 7.0], 
[5.0, 6.0, 7.0], [7.0, 7.0, 4.0]], [[4.0, 1.0, 7.0], [2.0, 8.0, 7.0], [7.0, 1.0, 7.0]], [[8.0, 7.0, 1.0], [5.0, 6.0, 8.0], 
[4.0, 5.0, 2.0]]], [[[4.0, 4.0, 8.0], [6.0, 4.0, 6.0], [6.0, 2.0, 5.0]], [[6.0, 6.0, 8.0], [1.0, 4.0, 7.0], [7.0, 5.0, 3.0]], 
[[5.0, 3.0, 6.0], [2.0, 3.0, 7.0], [6.0, 5.0, 2.0]]], [[[6.0, 8.0, 1.0], [3.0, 7.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 7.0, 4.0], 
[4.0, 5.0, 4.0], [5.0, 3.0, 5.0]], [[4.0, 2.0, 5.0], [7.0, 3.0, 7.0], [7.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRCN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 1.0], [7.0, 5.0, 1.0], [4.0, 2.0, 6.0]], [[7.0, 8.0, 8.0], [5.0, 7.0, 5.0], [7.0, 5.0, 
7.0]], [[7.0, 5.0, 4.0], [1.0, 3.0, 2.0], [7.0, 1.0, 3.0]]], [[[6.0, 3.0, 1.0], [4.0, 5.0, 6.0], [6.0, 7.0, 4.0]], [[1.0, 1.0, 
6.0], [7.0, 1.0, 7.0], [3.0, 3.0, 6.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 2.0], [4.0, 5.0, 6.0]]], [[[7.0, 6.0, 3.0], [8.0, 7.0, 
1.0], [8.0, 3.0, 3.0]], [[3.0, 3.0, 4.0], [3.0, 3.0, 6.0], [8.0, 1.0, 8.0]], [[3.0, 3.0, 3.0], [3.0, 2.0, 3.0], [3.0, 3.0, 
3.0]]], [[[1.0, 5.0, 8.0], [2.0, 5.0, 3.0], [5.0, 6.0, 8.0]], [[1.0, 1.0, 2.0], [3.0, 8.0, 2.0], [6.0, 4.0, 8.0]], [[8.0, 8.0, 
4.0], [1.0, 4.0, 2.0], [2.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 6.0], [4.0, 6.0, 4.0], [4.0, 5.0, 4.0]], [[4.0, 5.0, 4.0], [8.0, 7.0, 
7.0], [2.0, 6.0, 8.0]], [[5.0, 7.0, 8.0], [8.0, 4.0, 7.0], [1.0, 2.0, 3.0]]], [[[6.0, 5.0, 2.0], [8.0, 7.0, 4.0], [8.0, 7.0, 
5.0]], [[4.0, 1.0, 1.0], [2.0, 5.0, 3.0], [2.0, 1.0, 1.0]], [[4.0, 6.0, 7.0], [8.0, 4.0, 8.0], [1.0, 2.0, 3.0]]], [[[2.0, 4.0, 
8.0], [1.0, 6.0, 2.0], [5.0, 2.0, 2.0]], [[8.0, 5.0, 8.0], [4.0, 8.0, 1.0], [2.0, 5.0, 5.0]], [[3.0, 4.0, 8.0], [4.0, 4.0, 
7.0], [2.0, 1.0, 4.0]]], [[[5.0, 4.0, 5.0], [4.0, 3.0, 1.0], [1.0, 4.0, 5.0]], [[2.0, 3.0, 8.0], [2.0, 4.0, 2.0], [7.0, 1.0, 
5.0]], [[8.0, 7.0, 5.0], [7.0, 7.0, 2.0], [1.0, 8.0, 7.0]]]]))
      val=Data(numpy.array([[[[3.0, 3.0, 7.0], [2.0, 6.0, 5.0], [2.0, 8.0, 4.0]]], [[[2.0, 7.0, 1.0], [7.0, 1.0, 3.0], [3.0, 
1.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 7.0], [5.0, 7.0, 7.0], [6.0, 1.0, 1.0]]], [[[6.0, 7.0, 4.0], [3.0, 8.0, 
1.0], [1.0, 5.0, 4.0]]]]))
      arg[1:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 1.0], [7.0, 5.0, 1.0], [4.0, 2.0, 6.0]], [[7.0, 8.0, 8.0], [5.0, 7.0, 5.0], [7.0, 
5.0, 7.0]], [[7.0, 5.0, 4.0], [1.0, 3.0, 2.0], [7.0, 1.0, 3.0]]], [[[6.0, 3.0, 1.0], [4.0, 5.0, 6.0], [6.0, 7.0, 4.0]], [[3.0, 
3.0, 7.0], [2.0, 6.0, 5.0], [2.0, 8.0, 4.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 2.0], [4.0, 5.0, 6.0]]], [[[7.0, 6.0, 3.0], [8.0, 
7.0, 1.0], [8.0, 3.0, 3.0]], [[2.0, 7.0, 1.0], [7.0, 1.0, 3.0], [3.0, 1.0, 6.0]], [[3.0, 3.0, 3.0], [3.0, 2.0, 3.0], [3.0, 3.0, 
3.0]]], [[[1.0, 5.0, 8.0], [2.0, 5.0, 3.0], [5.0, 6.0, 8.0]], [[1.0, 1.0, 2.0], [3.0, 8.0, 2.0], [6.0, 4.0, 8.0]], [[8.0, 8.0, 
4.0], [1.0, 4.0, 2.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 6.0], [4.0, 6.0, 4.0], [4.0, 5.0, 4.0]], 
[[4.0, 5.0, 4.0], [8.0, 7.0, 7.0], [2.0, 6.0, 8.0]], [[5.0, 7.0, 8.0], [8.0, 4.0, 7.0], [1.0, 2.0, 3.0]]], [[[6.0, 5.0, 2.0], 
[8.0, 7.0, 4.0], [8.0, 7.0, 5.0]], [[1.0, 2.0, 7.0], [5.0, 7.0, 7.0], [6.0, 1.0, 1.0]], [[4.0, 6.0, 7.0], [8.0, 4.0, 8.0], 
[1.0, 2.0, 3.0]]], [[[2.0, 4.0, 8.0], [1.0, 6.0, 2.0], [5.0, 2.0, 2.0]], [[6.0, 7.0, 4.0], [3.0, 8.0, 1.0], [1.0, 5.0, 4.0]], 
[[3.0, 4.0, 8.0], [4.0, 4.0, 7.0], [2.0, 1.0, 4.0]]], [[[5.0, 4.0, 5.0], [4.0, 3.0, 1.0], [1.0, 4.0, 5.0]], [[2.0, 3.0, 8.0], 
[2.0, 4.0, 2.0], [7.0, 1.0, 5.0]], [[8.0, 7.0, 5.0], [7.0, 7.0, 2.0], [1.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRCN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 3.0], [8.0, 6.0, 6.0], [7.0, 3.0, 2.0]], [[7.0, 7.0, 4.0], [8.0, 1.0, 6.0], [4.0, 8.0, 
3.0]], [[1.0, 2.0, 4.0], [4.0, 2.0, 4.0], [7.0, 7.0, 6.0]]], [[[4.0, 4.0, 6.0], [7.0, 7.0, 6.0], [1.0, 5.0, 3.0]], [[3.0, 2.0, 
7.0], [1.0, 3.0, 8.0], [8.0, 7.0, 1.0]], [[6.0, 2.0, 5.0], [3.0, 4.0, 5.0], [3.0, 1.0, 3.0]]], [[[1.0, 8.0, 2.0], [2.0, 4.0, 
1.0], [4.0, 3.0, 3.0]], [[8.0, 6.0, 3.0], [5.0, 5.0, 4.0], [4.0, 8.0, 1.0]], [[6.0, 1.0, 7.0], [5.0, 2.0, 3.0], [7.0, 5.0, 
4.0]]], [[[8.0, 8.0, 8.0], [7.0, 1.0, 8.0], [4.0, 3.0, 1.0]], [[8.0, 6.0, 7.0], [2.0, 4.0, 4.0], [8.0, 8.0, 2.0]], [[7.0, 4.0, 
4.0], [4.0, 6.0, 3.0], [5.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 2.0], [2.0, 1.0, 2.0], [5.0, 6.0, 3.0]], [[8.0, 3.0, 6.0], [5.0, 5.0, 
3.0], [3.0, 7.0, 5.0]], [[3.0, 6.0, 2.0], [8.0, 6.0, 8.0], [2.0, 6.0, 5.0]]], [[[3.0, 3.0, 6.0], [5.0, 4.0, 6.0], [2.0, 3.0, 
7.0]], [[2.0, 5.0, 3.0], [1.0, 7.0, 5.0], [7.0, 8.0, 1.0]], [[2.0, 1.0, 4.0], [4.0, 5.0, 1.0], [5.0, 6.0, 5.0]]], [[[3.0, 7.0, 
1.0], [4.0, 6.0, 5.0], [5.0, 5.0, 1.0]], [[3.0, 6.0, 2.0], [2.0, 7.0, 8.0], [3.0, 3.0, 1.0]], [[5.0, 7.0, 6.0], [8.0, 3.0, 
7.0], [4.0, 8.0, 6.0]]], [[[5.0, 7.0, 6.0], [6.0, 6.0, 5.0], [2.0, 1.0, 6.0]], [[3.0, 3.0, 4.0], [1.0, 4.0, 2.0], [3.0, 6.0, 
8.0]], [[4.0, 2.0, 5.0], [3.0, 5.0, 5.0], [6.0, 6.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 7.0, 2.0], [8.0, 8.0, 7.0], [1.0, 5.0, 6.0]]], [[[4.0, 5.0, 8.0], [5.0, 5.0, 8.0], [3.0, 
4.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 6.0, 3.0], [1.0, 7.0, 8.0], [4.0, 8.0, 1.0]]], [[[8.0, 2.0, 3.0], [2.0, 8.0, 
4.0], [2.0, 2.0, 4.0]]]])
      arg[1:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 3.0], [8.0, 6.0, 6.0], [7.0, 3.0, 2.0]], [[7.0, 7.0, 4.0], [8.0, 1.0, 6.0], [4.0, 
8.0, 3.0]], [[1.0, 2.0, 4.0], [4.0, 2.0, 4.0], [7.0, 7.0, 6.0]]], [[[4.0, 4.0, 6.0], [7.0, 7.0, 6.0], [1.0, 5.0, 3.0]], [[8.0, 
7.0, 2.0], [8.0, 8.0, 7.0], [1.0, 5.0, 6.0]], [[6.0, 2.0, 5.0], [3.0, 4.0, 5.0], [3.0, 1.0, 3.0]]], [[[1.0, 8.0, 2.0], [2.0, 
4.0, 1.0], [4.0, 3.0, 3.0]], [[4.0, 5.0, 8.0], [5.0, 5.0, 8.0], [3.0, 4.0, 8.0]], [[6.0, 1.0, 7.0], [5.0, 2.0, 3.0], [7.0, 5.0, 
4.0]]], [[[8.0, 8.0, 8.0], [7.0, 1.0, 8.0], [4.0, 3.0, 1.0]], [[8.0, 6.0, 7.0], [2.0, 4.0, 4.0], [8.0, 8.0, 2.0]], [[7.0, 4.0, 
4.0], [4.0, 6.0, 3.0], [5.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 2.0], [2.0, 1.0, 2.0], [5.0, 6.0, 3.0]], 
[[8.0, 3.0, 6.0], [5.0, 5.0, 3.0], [3.0, 7.0, 5.0]], [[3.0, 6.0, 2.0], [8.0, 6.0, 8.0], [2.0, 6.0, 5.0]]], [[[3.0, 3.0, 6.0], 
[5.0, 4.0, 6.0], [2.0, 3.0, 7.0]], [[7.0, 6.0, 3.0], [1.0, 7.0, 8.0], [4.0, 8.0, 1.0]], [[2.0, 1.0, 4.0], [4.0, 5.0, 1.0], 
[5.0, 6.0, 5.0]]], [[[3.0, 7.0, 1.0], [4.0, 6.0, 5.0], [5.0, 5.0, 1.0]], [[8.0, 2.0, 3.0], [2.0, 8.0, 4.0], [2.0, 2.0, 4.0]], 
[[5.0, 7.0, 6.0], [8.0, 3.0, 7.0], [4.0, 8.0, 6.0]]], [[[5.0, 7.0, 6.0], [6.0, 6.0, 5.0], [2.0, 1.0, 6.0]], [[3.0, 3.0, 4.0], 
[1.0, 4.0, 2.0], [3.0, 6.0, 8.0]], [[4.0, 2.0, 5.0], [3.0, 5.0, 5.0], [6.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UPLN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 1.0], [4.0, 5.0, 3.0], [8.0, 1.0, 2.0]], [[8.0, 6.0, 8.0], [3.0, 3.0, 7.0], [4.0, 6.0, 
3.0]], [[7.0, 1.0, 2.0], [8.0, 2.0, 8.0], [2.0, 7.0, 2.0]]], [[[6.0, 7.0, 5.0], [1.0, 8.0, 3.0], [5.0, 2.0, 7.0]], [[7.0, 2.0, 
5.0], [5.0, 4.0, 5.0], [7.0, 7.0, 3.0]], [[3.0, 3.0, 2.0], [1.0, 1.0, 8.0], [3.0, 4.0, 2.0]]], [[[7.0, 6.0, 3.0], [6.0, 5.0, 
5.0], [2.0, 2.0, 7.0]], [[8.0, 5.0, 1.0], [7.0, 3.0, 5.0], [3.0, 2.0, 7.0]], [[7.0, 1.0, 8.0], [7.0, 1.0, 7.0], [3.0, 1.0, 
1.0]]], [[[8.0, 1.0, 5.0], [5.0, 8.0, 4.0], [7.0, 6.0, 7.0]], [[4.0, 7.0, 5.0], [4.0, 7.0, 7.0], [6.0, 4.0, 7.0]], [[4.0, 4.0, 
6.0], [7.0, 7.0, 3.0], [4.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 5.0], [3.0, 8.0, 1.0], [3.0, 5.0, 8.0]], [[7.0, 8.0, 7.0], [8.0, 6.0, 
7.0], [1.0, 4.0, 5.0]], [[1.0, 3.0, 3.0], [6.0, 7.0, 2.0], [8.0, 3.0, 6.0]]], [[[6.0, 8.0, 4.0], [2.0, 2.0, 3.0], [5.0, 3.0, 
4.0]], [[5.0, 4.0, 7.0], [3.0, 6.0, 7.0], [8.0, 6.0, 6.0]], [[5.0, 4.0, 5.0], [3.0, 3.0, 6.0], [6.0, 3.0, 2.0]]], [[[5.0, 3.0, 
1.0], [1.0, 3.0, 8.0], [4.0, 8.0, 2.0]], [[7.0, 6.0, 2.0], [7.0, 6.0, 3.0], [1.0, 6.0, 5.0]], [[6.0, 7.0, 1.0], [8.0, 6.0, 
7.0], [1.0, 2.0, 7.0]]], [[[1.0, 4.0, 2.0], [3.0, 1.0, 3.0], [4.0, 2.0, 4.0]], [[7.0, 7.0, 4.0], [5.0, 2.0, 8.0], [8.0, 5.0, 
6.0]], [[4.0, 4.0, 8.0], [1.0, 7.0, 2.0], [8.0, 1.0, 6.0]]]]))
      val=numpy.array([[[5.0, 1.0, 7.0]], [[1.0, 6.0, 4.0]], [[4.0, 1.0, 6.0]]])
      arg[:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 1.0], [4.0, 5.0, 3.0], [8.0, 1.0, 2.0]], [[8.0, 6.0, 8.0], [3.0, 3.0, 7.0], [5.0, 
1.0, 7.0]], [[7.0, 1.0, 2.0], [8.0, 2.0, 8.0], [2.0, 7.0, 2.0]]], [[[6.0, 7.0, 5.0], [1.0, 8.0, 3.0], [5.0, 2.0, 7.0]], [[7.0, 
2.0, 5.0], [5.0, 4.0, 5.0], [1.0, 6.0, 4.0]], [[3.0, 3.0, 2.0], [1.0, 1.0, 8.0], [3.0, 4.0, 2.0]]], [[[7.0, 6.0, 3.0], [6.0, 
5.0, 5.0], [2.0, 2.0, 7.0]], [[8.0, 5.0, 1.0], [7.0, 3.0, 5.0], [4.0, 1.0, 6.0]], [[7.0, 1.0, 8.0], [7.0, 1.0, 7.0], [3.0, 1.0, 
1.0]]], [[[8.0, 1.0, 5.0], [5.0, 8.0, 4.0], [7.0, 6.0, 7.0]], [[4.0, 7.0, 5.0], [4.0, 7.0, 7.0], [6.0, 4.0, 7.0]], [[4.0, 4.0, 
6.0], [7.0, 7.0, 3.0], [4.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 5.0], [3.0, 8.0, 1.0], [3.0, 5.0, 8.0]], 
[[7.0, 8.0, 7.0], [8.0, 6.0, 7.0], [5.0, 1.0, 7.0]], [[1.0, 3.0, 3.0], [6.0, 7.0, 2.0], [8.0, 3.0, 6.0]]], [[[6.0, 8.0, 4.0], 
[2.0, 2.0, 3.0], [5.0, 3.0, 4.0]], [[5.0, 4.0, 7.0], [3.0, 6.0, 7.0], [1.0, 6.0, 4.0]], [[5.0, 4.0, 5.0], [3.0, 3.0, 6.0], 
[6.0, 3.0, 2.0]]], [[[5.0, 3.0, 1.0], [1.0, 3.0, 8.0], [4.0, 8.0, 2.0]], [[7.0, 6.0, 2.0], [7.0, 6.0, 3.0], [4.0, 1.0, 6.0]], 
[[6.0, 7.0, 1.0], [8.0, 6.0, 7.0], [1.0, 2.0, 7.0]]], [[[1.0, 4.0, 2.0], [3.0, 1.0, 3.0], [4.0, 2.0, 4.0]], [[7.0, 7.0, 4.0], 
[5.0, 2.0, 8.0], [8.0, 5.0, 6.0]], [[4.0, 4.0, 8.0], [1.0, 7.0, 2.0], [8.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UPLN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 6.0], [6.0, 8.0, 7.0], [4.0, 7.0, 4.0]], [[3.0, 6.0, 4.0], [1.0, 3.0, 3.0], [4.0, 2.0, 
8.0]], [[5.0, 7.0, 3.0], [7.0, 4.0, 5.0], [7.0, 7.0, 1.0]]], [[[4.0, 1.0, 5.0], [2.0, 2.0, 4.0], [2.0, 8.0, 4.0]], [[2.0, 5.0, 
2.0], [8.0, 2.0, 8.0], [1.0, 6.0, 1.0]], [[4.0, 3.0, 6.0], [4.0, 4.0, 6.0], [1.0, 5.0, 3.0]]], [[[7.0, 8.0, 5.0], [3.0, 6.0, 
4.0], [3.0, 4.0, 5.0]], [[2.0, 2.0, 7.0], [6.0, 5.0, 3.0], [6.0, 6.0, 5.0]], [[3.0, 7.0, 7.0], [6.0, 4.0, 1.0], [5.0, 6.0, 
5.0]]], [[[7.0, 2.0, 6.0], [2.0, 2.0, 2.0], [7.0, 6.0, 7.0]], [[3.0, 6.0, 6.0], [3.0, 3.0, 7.0], [8.0, 6.0, 6.0]], [[5.0, 6.0, 
8.0], [8.0, 2.0, 1.0], [3.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 6.0], [6.0, 8.0, 1.0], [5.0, 6.0, 2.0]], [[4.0, 1.0, 2.0], [2.0, 4.0, 
7.0], [8.0, 8.0, 6.0]], [[3.0, 4.0, 7.0], [7.0, 3.0, 1.0], [5.0, 1.0, 4.0]]], [[[8.0, 3.0, 1.0], [6.0, 4.0, 7.0], [8.0, 7.0, 
2.0]], [[2.0, 5.0, 6.0], [8.0, 6.0, 4.0], [8.0, 1.0, 6.0]], [[4.0, 3.0, 7.0], [7.0, 7.0, 4.0], [6.0, 1.0, 7.0]]], [[[1.0, 4.0, 
7.0], [3.0, 1.0, 5.0], [5.0, 1.0, 8.0]], [[2.0, 7.0, 4.0], [6.0, 4.0, 2.0], [6.0, 8.0, 6.0]], [[4.0, 1.0, 6.0], [6.0, 4.0, 
2.0], [2.0, 8.0, 5.0]]], [[[3.0, 2.0, 8.0], [5.0, 3.0, 4.0], [1.0, 1.0, 1.0]], [[4.0, 6.0, 6.0], [8.0, 8.0, 4.0], [6.0, 2.0, 
5.0]], [[3.0, 2.0, 2.0], [1.0, 1.0, 3.0], [8.0, 7.0, 4.0]]]]))
      val=Data(numpy.array([[[3.0, 7.0, 6.0]], [[6.0, 6.0, 6.0]], [[6.0, 5.0, 4.0]]]),self.functionspace)
      arg[:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 6.0], [6.0, 8.0, 7.0], [4.0, 7.0, 4.0]], [[3.0, 6.0, 4.0], [1.0, 3.0, 3.0], [3.0, 
7.0, 6.0]], [[5.0, 7.0, 3.0], [7.0, 4.0, 5.0], [7.0, 7.0, 1.0]]], [[[4.0, 1.0, 5.0], [2.0, 2.0, 4.0], [2.0, 8.0, 4.0]], [[2.0, 
5.0, 2.0], [8.0, 2.0, 8.0], [6.0, 6.0, 6.0]], [[4.0, 3.0, 6.0], [4.0, 4.0, 6.0], [1.0, 5.0, 3.0]]], [[[7.0, 8.0, 5.0], [3.0, 
6.0, 4.0], [3.0, 4.0, 5.0]], [[2.0, 2.0, 7.0], [6.0, 5.0, 3.0], [6.0, 5.0, 4.0]], [[3.0, 7.0, 7.0], [6.0, 4.0, 1.0], [5.0, 6.0, 
5.0]]], [[[7.0, 2.0, 6.0], [2.0, 2.0, 2.0], [7.0, 6.0, 7.0]], [[3.0, 6.0, 6.0], [3.0, 3.0, 7.0], [8.0, 6.0, 6.0]], [[5.0, 6.0, 
8.0], [8.0, 2.0, 1.0], [3.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 6.0], [6.0, 8.0, 1.0], [5.0, 6.0, 2.0]], 
[[4.0, 1.0, 2.0], [2.0, 4.0, 7.0], [3.0, 7.0, 6.0]], [[3.0, 4.0, 7.0], [7.0, 3.0, 1.0], [5.0, 1.0, 4.0]]], [[[8.0, 3.0, 1.0], 
[6.0, 4.0, 7.0], [8.0, 7.0, 2.0]], [[2.0, 5.0, 6.0], [8.0, 6.0, 4.0], [6.0, 6.0, 6.0]], [[4.0, 3.0, 7.0], [7.0, 7.0, 4.0], 
[6.0, 1.0, 7.0]]], [[[1.0, 4.0, 7.0], [3.0, 1.0, 5.0], [5.0, 1.0, 8.0]], [[2.0, 7.0, 4.0], [6.0, 4.0, 2.0], [6.0, 5.0, 4.0]], 
[[4.0, 1.0, 6.0], [6.0, 4.0, 2.0], [2.0, 8.0, 5.0]]], [[[3.0, 2.0, 8.0], [5.0, 3.0, 4.0], [1.0, 1.0, 1.0]], [[4.0, 6.0, 6.0], 
[8.0, 8.0, 4.0], [6.0, 2.0, 5.0]], [[3.0, 2.0, 2.0], [1.0, 1.0, 3.0], [8.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UPLN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 5.0], [5.0, 5.0, 8.0], [8.0, 2.0, 1.0]], [[8.0, 1.0, 1.0], [7.0, 2.0, 4.0], [6.0, 4.0, 
8.0]], [[7.0, 4.0, 5.0], [6.0, 6.0, 1.0], [5.0, 6.0, 5.0]]], [[[3.0, 6.0, 8.0], [4.0, 6.0, 5.0], [8.0, 4.0, 2.0]], [[6.0, 7.0, 
2.0], [4.0, 2.0, 3.0], [4.0, 6.0, 2.0]], [[6.0, 1.0, 8.0], [4.0, 6.0, 5.0], [6.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [7.0, 3.0, 
5.0], [6.0, 7.0, 3.0]], [[6.0, 5.0, 1.0], [7.0, 7.0, 4.0], [7.0, 1.0, 1.0]], [[6.0, 1.0, 8.0], [3.0, 2.0, 5.0], [6.0, 8.0, 
8.0]]], [[[6.0, 7.0, 6.0], [1.0, 8.0, 2.0], [8.0, 6.0, 3.0]], [[3.0, 3.0, 4.0], [8.0, 2.0, 2.0], [2.0, 1.0, 5.0]], [[8.0, 4.0, 
5.0], [4.0, 5.0, 8.0], [8.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 7.0], [1.0, 1.0, 5.0], [1.0, 6.0, 5.0]], [[2.0, 5.0, 8.0], [3.0, 2.0, 
4.0], [2.0, 7.0, 1.0]], [[3.0, 7.0, 5.0], [8.0, 3.0, 4.0], [4.0, 3.0, 1.0]]], [[[7.0, 5.0, 3.0], [8.0, 5.0, 3.0], [8.0, 3.0, 
2.0]], [[4.0, 8.0, 4.0], [8.0, 8.0, 8.0], [2.0, 2.0, 1.0]], [[2.0, 5.0, 2.0], [5.0, 8.0, 3.0], [2.0, 5.0, 5.0]]], [[[3.0, 1.0, 
5.0], [4.0, 1.0, 4.0], [1.0, 3.0, 5.0]], [[6.0, 6.0, 8.0], [3.0, 6.0, 3.0], [2.0, 4.0, 8.0]], [[4.0, 1.0, 7.0], [7.0, 2.0, 
6.0], [1.0, 8.0, 5.0]]], [[[2.0, 5.0, 7.0], [6.0, 3.0, 4.0], [1.0, 7.0, 5.0]], [[7.0, 1.0, 8.0], [1.0, 2.0, 5.0], [4.0, 7.0, 
5.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 5.0], [2.0, 1.0, 8.0]]]]))
      val=Data(numpy.array([[[8.0, 1.0, 5.0]], [[3.0, 8.0, 4.0]], [[3.0, 3.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 5.0, 7.0]], [[5.0, 5.0, 8.0]], [[5.0, 8.0, 1.0]]]))
      arg[:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 5.0], [5.0, 5.0, 8.0], [8.0, 2.0, 1.0]], [[8.0, 1.0, 1.0], [7.0, 2.0, 4.0], [8.0, 
1.0, 5.0]], [[7.0, 4.0, 5.0], [6.0, 6.0, 1.0], [5.0, 6.0, 5.0]]], [[[3.0, 6.0, 8.0], [4.0, 6.0, 5.0], [8.0, 4.0, 2.0]], [[6.0, 
7.0, 2.0], [4.0, 2.0, 3.0], [3.0, 8.0, 4.0]], [[6.0, 1.0, 8.0], [4.0, 6.0, 5.0], [6.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [7.0, 
3.0, 5.0], [6.0, 7.0, 3.0]], [[6.0, 5.0, 1.0], [7.0, 7.0, 4.0], [3.0, 3.0, 4.0]], [[6.0, 1.0, 8.0], [3.0, 2.0, 5.0], [6.0, 8.0, 
8.0]]], [[[6.0, 7.0, 6.0], [1.0, 8.0, 2.0], [8.0, 6.0, 3.0]], [[3.0, 3.0, 4.0], [8.0, 2.0, 2.0], [2.0, 1.0, 5.0]], [[8.0, 4.0, 
5.0], [4.0, 5.0, 8.0], [8.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 7.0], [1.0, 1.0, 5.0], [1.0, 6.0, 5.0]], 
[[2.0, 5.0, 8.0], [3.0, 2.0, 4.0], [8.0, 5.0, 7.0]], [[3.0, 7.0, 5.0], [8.0, 3.0, 4.0], [4.0, 3.0, 1.0]]], [[[7.0, 5.0, 3.0], 
[8.0, 5.0, 3.0], [8.0, 3.0, 2.0]], [[4.0, 8.0, 4.0], [8.0, 8.0, 8.0], [5.0, 5.0, 8.0]], [[2.0, 5.0, 2.0], [5.0, 8.0, 3.0], 
[2.0, 5.0, 5.0]]], [[[3.0, 1.0, 5.0], [4.0, 1.0, 4.0], [1.0, 3.0, 5.0]], [[6.0, 6.0, 8.0], [3.0, 6.0, 3.0], [5.0, 8.0, 1.0]], 
[[4.0, 1.0, 7.0], [7.0, 2.0, 6.0], [1.0, 8.0, 5.0]]], [[[2.0, 5.0, 7.0], [6.0, 3.0, 4.0], [1.0, 7.0, 5.0]], [[7.0, 1.0, 8.0], 
[1.0, 2.0, 5.0], [4.0, 7.0, 5.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 5.0], [2.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UPLN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 5.0], [2.0, 6.0, 7.0], [3.0, 4.0, 3.0]], [[6.0, 8.0, 6.0], [2.0, 2.0, 4.0], [2.0, 1.0, 
2.0]], [[2.0, 4.0, 5.0], [2.0, 7.0, 8.0], [7.0, 5.0, 8.0]]], [[[1.0, 7.0, 3.0], [7.0, 2.0, 2.0], [2.0, 3.0, 8.0]], [[6.0, 6.0, 
6.0], [3.0, 4.0, 8.0], [7.0, 8.0, 8.0]], [[2.0, 6.0, 8.0], [5.0, 7.0, 4.0], [7.0, 1.0, 8.0]]], [[[6.0, 8.0, 6.0], [2.0, 5.0, 
1.0], [6.0, 1.0, 2.0]], [[7.0, 3.0, 1.0], [3.0, 8.0, 8.0], [8.0, 2.0, 8.0]], [[4.0, 4.0, 8.0], [1.0, 4.0, 8.0], [8.0, 4.0, 
4.0]]], [[[8.0, 5.0, 6.0], [1.0, 1.0, 1.0], [1.0, 3.0, 1.0]], [[6.0, 5.0, 8.0], [6.0, 7.0, 3.0], [2.0, 7.0, 7.0]], [[7.0, 6.0, 
6.0], [2.0, 7.0, 1.0], [5.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 1.0], [6.0, 3.0, 1.0], [7.0, 5.0, 6.0]], [[8.0, 4.0, 2.0], [3.0, 1.0, 
6.0], [8.0, 2.0, 2.0]], [[3.0, 5.0, 6.0], [7.0, 5.0, 6.0], [5.0, 7.0, 6.0]]], [[[7.0, 6.0, 5.0], [2.0, 6.0, 7.0], [8.0, 7.0, 
3.0]], [[8.0, 7.0, 6.0], [5.0, 4.0, 4.0], [2.0, 3.0, 8.0]], [[3.0, 3.0, 1.0], [3.0, 4.0, 2.0], [7.0, 6.0, 7.0]]], [[[7.0, 6.0, 
2.0], [5.0, 5.0, 5.0], [4.0, 8.0, 7.0]], [[1.0, 3.0, 5.0], [3.0, 8.0, 6.0], [2.0, 2.0, 7.0]], [[7.0, 3.0, 5.0], [2.0, 8.0, 
1.0], [4.0, 5.0, 2.0]]], [[[6.0, 7.0, 5.0], [8.0, 5.0, 3.0], [8.0, 8.0, 4.0]], [[4.0, 5.0, 5.0], [5.0, 2.0, 7.0], [6.0, 4.0, 
6.0]], [[4.0, 1.0, 3.0], [8.0, 7.0, 6.0], [5.0, 6.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 4.0, 2.0]], [[5.0, 5.0, 7.0]], [[2.0, 6.0, 2.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 
3.0]], [[5.0, 3.0, 5.0]], [[5.0, 4.0, 7.0]]])
      arg[:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 5.0], [2.0, 6.0, 7.0], [3.0, 4.0, 3.0]], [[6.0, 8.0, 6.0], [2.0, 2.0, 4.0], [3.0, 
4.0, 2.0]], [[2.0, 4.0, 5.0], [2.0, 7.0, 8.0], [7.0, 5.0, 8.0]]], [[[1.0, 7.0, 3.0], [7.0, 2.0, 2.0], [2.0, 3.0, 8.0]], [[6.0, 
6.0, 6.0], [3.0, 4.0, 8.0], [5.0, 5.0, 7.0]], [[2.0, 6.0, 8.0], [5.0, 7.0, 4.0], [7.0, 1.0, 8.0]]], [[[6.0, 8.0, 6.0], [2.0, 
5.0, 1.0], [6.0, 1.0, 2.0]], [[7.0, 3.0, 1.0], [3.0, 8.0, 8.0], [2.0, 6.0, 2.0]], [[4.0, 4.0, 8.0], [1.0, 4.0, 8.0], [8.0, 4.0, 
4.0]]], [[[8.0, 5.0, 6.0], [1.0, 1.0, 1.0], [1.0, 3.0, 1.0]], [[6.0, 5.0, 8.0], [6.0, 7.0, 3.0], [2.0, 7.0, 7.0]], [[7.0, 6.0, 
6.0], [2.0, 7.0, 1.0], [5.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 1.0], [6.0, 3.0, 1.0], [7.0, 5.0, 6.0]], 
[[8.0, 4.0, 2.0], [3.0, 1.0, 6.0], [4.0, 6.0, 3.0]], [[3.0, 5.0, 6.0], [7.0, 5.0, 6.0], [5.0, 7.0, 6.0]]], [[[7.0, 6.0, 5.0], 
[2.0, 6.0, 7.0], [8.0, 7.0, 3.0]], [[8.0, 7.0, 6.0], [5.0, 4.0, 4.0], [5.0, 3.0, 5.0]], [[3.0, 3.0, 1.0], [3.0, 4.0, 2.0], 
[7.0, 6.0, 7.0]]], [[[7.0, 6.0, 2.0], [5.0, 5.0, 5.0], [4.0, 8.0, 7.0]], [[1.0, 3.0, 5.0], [3.0, 8.0, 6.0], [5.0, 4.0, 7.0]], 
[[7.0, 3.0, 5.0], [2.0, 8.0, 1.0], [4.0, 5.0, 2.0]]], [[[6.0, 7.0, 5.0], [8.0, 5.0, 3.0], [8.0, 8.0, 4.0]], [[4.0, 5.0, 5.0], 
[5.0, 2.0, 7.0], [6.0, 4.0, 6.0]], [[4.0, 1.0, 3.0], [8.0, 7.0, 6.0], [5.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PUUU(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 1.0], [2.0, 3.0, 7.0], [6.0, 6.0, 8.0]], [[2.0, 5.0, 6.0], [4.0, 1.0, 1.0], [7.0, 8.0, 
7.0]], [[1.0, 8.0, 2.0], [5.0, 3.0, 8.0], [6.0, 2.0, 7.0]]], [[[5.0, 8.0, 2.0], [4.0, 2.0, 5.0], [7.0, 6.0, 3.0]], [[7.0, 5.0, 
8.0], [3.0, 5.0, 7.0], [4.0, 2.0, 7.0]], [[2.0, 8.0, 5.0], [1.0, 7.0, 5.0], [4.0, 6.0, 4.0]]], [[[2.0, 2.0, 6.0], [6.0, 3.0, 
7.0], [4.0, 8.0, 4.0]], [[5.0, 4.0, 7.0], [8.0, 1.0, 4.0], [6.0, 1.0, 3.0]], [[6.0, 4.0, 6.0], [1.0, 6.0, 7.0], [1.0, 8.0, 
8.0]]], [[[2.0, 7.0, 3.0], [6.0, 2.0, 3.0], [3.0, 7.0, 8.0]], [[2.0, 4.0, 1.0], [7.0, 2.0, 1.0], [2.0, 6.0, 2.0]], [[7.0, 1.0, 
2.0], [8.0, 1.0, 2.0], [6.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 7.0], [6.0, 5.0, 8.0], [2.0, 8.0, 6.0]], [[5.0, 2.0, 6.0], [8.0, 1.0, 
6.0], [8.0, 8.0, 7.0]], [[1.0, 1.0, 6.0], [5.0, 1.0, 5.0], [6.0, 8.0, 5.0]]], [[[4.0, 4.0, 8.0], [7.0, 8.0, 8.0], [3.0, 8.0, 
5.0]], [[4.0, 1.0, 4.0], [2.0, 6.0, 7.0], [8.0, 1.0, 4.0]], [[7.0, 6.0, 1.0], [2.0, 7.0, 1.0], [7.0, 1.0, 6.0]]], [[[3.0, 1.0, 
1.0], [3.0, 6.0, 5.0], [5.0, 3.0, 3.0]], [[3.0, 5.0, 8.0], [3.0, 3.0, 8.0], [7.0, 8.0, 4.0]], [[6.0, 8.0, 5.0], [5.0, 5.0, 
6.0], [4.0, 1.0, 6.0]]], [[[2.0, 6.0, 8.0], [5.0, 1.0, 3.0], [2.0, 7.0, 4.0]], [[7.0, 2.0, 5.0], [5.0, 6.0, 2.0], [4.0, 1.0, 
5.0]], [[1.0, 5.0, 5.0], [2.0, 6.0, 1.0], [4.0, 8.0, 5.0]]]]))
      val=numpy.array([[[3.0, 1.0], [8.0, 1.0]], [[6.0, 5.0], [3.0, 4.0]]])
      arg[2,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 1.0], [2.0, 3.0, 7.0], [6.0, 6.0, 8.0]], [[2.0, 5.0, 6.0], [4.0, 1.0, 1.0], [7.0, 
8.0, 7.0]], [[1.0, 8.0, 2.0], [5.0, 3.0, 8.0], [6.0, 2.0, 7.0]]], [[[5.0, 8.0, 2.0], [4.0, 2.0, 5.0], [7.0, 6.0, 3.0]], [[7.0, 
5.0, 8.0], [3.0, 5.0, 7.0], [4.0, 2.0, 7.0]], [[2.0, 8.0, 5.0], [1.0, 7.0, 5.0], [4.0, 6.0, 4.0]]], [[[3.0, 1.0, 6.0], [8.0, 
1.0, 7.0], [4.0, 8.0, 4.0]], [[6.0, 5.0, 7.0], [3.0, 4.0, 4.0], [6.0, 1.0, 3.0]], [[6.0, 4.0, 6.0], [1.0, 6.0, 7.0], [1.0, 8.0, 
8.0]]], [[[2.0, 7.0, 3.0], [6.0, 2.0, 3.0], [3.0, 7.0, 8.0]], [[2.0, 4.0, 1.0], [7.0, 2.0, 1.0], [2.0, 6.0, 2.0]], [[7.0, 1.0, 
2.0], [8.0, 1.0, 2.0], [6.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 7.0], [6.0, 5.0, 8.0], [2.0, 8.0, 6.0]], 
[[5.0, 2.0, 6.0], [8.0, 1.0, 6.0], [8.0, 8.0, 7.0]], [[1.0, 1.0, 6.0], [5.0, 1.0, 5.0], [6.0, 8.0, 5.0]]], [[[4.0, 4.0, 8.0], 
[7.0, 8.0, 8.0], [3.0, 8.0, 5.0]], [[4.0, 1.0, 4.0], [2.0, 6.0, 7.0], [8.0, 1.0, 4.0]], [[7.0, 6.0, 1.0], [2.0, 7.0, 1.0], 
[7.0, 1.0, 6.0]]], [[[3.0, 1.0, 1.0], [8.0, 1.0, 5.0], [5.0, 3.0, 3.0]], [[6.0, 5.0, 8.0], [3.0, 4.0, 8.0], [7.0, 8.0, 4.0]], 
[[6.0, 8.0, 5.0], [5.0, 5.0, 6.0], [4.0, 1.0, 6.0]]], [[[2.0, 6.0, 8.0], [5.0, 1.0, 3.0], [2.0, 7.0, 4.0]], [[7.0, 2.0, 5.0], 
[5.0, 6.0, 2.0], [4.0, 1.0, 5.0]], [[1.0, 5.0, 5.0], [2.0, 6.0, 1.0], [4.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PUUU(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 5.0], [7.0, 1.0, 2.0], [4.0, 1.0, 7.0]], [[7.0, 1.0, 5.0], [2.0, 3.0, 1.0], [5.0, 7.0, 
8.0]], [[1.0, 1.0, 1.0], [2.0, 1.0, 1.0], [4.0, 1.0, 4.0]]], [[[4.0, 8.0, 3.0], [3.0, 7.0, 8.0], [7.0, 6.0, 4.0]], [[4.0, 5.0, 
4.0], [2.0, 2.0, 2.0], [2.0, 7.0, 8.0]], [[1.0, 7.0, 3.0], [6.0, 2.0, 2.0], [6.0, 6.0, 2.0]]], [[[2.0, 4.0, 5.0], [1.0, 1.0, 
4.0], [1.0, 1.0, 2.0]], [[2.0, 6.0, 7.0], [5.0, 8.0, 5.0], [8.0, 5.0, 1.0]], [[6.0, 6.0, 5.0], [7.0, 2.0, 5.0], [7.0, 4.0, 
3.0]]], [[[2.0, 8.0, 6.0], [3.0, 6.0, 7.0], [4.0, 2.0, 3.0]], [[8.0, 1.0, 4.0], [8.0, 7.0, 6.0], [8.0, 3.0, 1.0]], [[3.0, 3.0, 
1.0], [6.0, 8.0, 6.0], [3.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 6.0], [5.0, 8.0, 8.0], [6.0, 6.0, 5.0]], [[4.0, 4.0, 6.0], [5.0, 5.0, 
1.0], [5.0, 8.0, 6.0]], [[6.0, 6.0, 2.0], [2.0, 6.0, 3.0], [5.0, 8.0, 2.0]]], [[[6.0, 4.0, 7.0], [1.0, 8.0, 6.0], [4.0, 2.0, 
4.0]], [[6.0, 1.0, 3.0], [2.0, 3.0, 4.0], [5.0, 1.0, 7.0]], [[8.0, 3.0, 4.0], [2.0, 2.0, 8.0], [3.0, 8.0, 3.0]]], [[[8.0, 4.0, 
1.0], [1.0, 1.0, 5.0], [2.0, 7.0, 5.0]], [[7.0, 8.0, 2.0], [2.0, 2.0, 3.0], [7.0, 7.0, 1.0]], [[4.0, 6.0, 4.0], [1.0, 2.0, 
3.0], [5.0, 7.0, 2.0]]], [[[8.0, 5.0, 5.0], [8.0, 6.0, 7.0], [6.0, 4.0, 2.0]], [[3.0, 2.0, 7.0], [8.0, 5.0, 3.0], [1.0, 3.0, 
3.0]], [[2.0, 6.0, 3.0], [3.0, 6.0, 2.0], [6.0, 2.0, 5.0]]]]))
      val=Data(numpy.array([[[5.0, 8.0], [3.0, 2.0]], [[2.0, 7.0], [8.0, 3.0]]]),self.functionspace)
      arg[2,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 5.0], [7.0, 1.0, 2.0], [4.0, 1.0, 7.0]], [[7.0, 1.0, 5.0], [2.0, 3.0, 1.0], [5.0, 
7.0, 8.0]], [[1.0, 1.0, 1.0], [2.0, 1.0, 1.0], [4.0, 1.0, 4.0]]], [[[4.0, 8.0, 3.0], [3.0, 7.0, 8.0], [7.0, 6.0, 4.0]], [[4.0, 
5.0, 4.0], [2.0, 2.0, 2.0], [2.0, 7.0, 8.0]], [[1.0, 7.0, 3.0], [6.0, 2.0, 2.0], [6.0, 6.0, 2.0]]], [[[5.0, 8.0, 5.0], [3.0, 
2.0, 4.0], [1.0, 1.0, 2.0]], [[2.0, 7.0, 7.0], [8.0, 3.0, 5.0], [8.0, 5.0, 1.0]], [[6.0, 6.0, 5.0], [7.0, 2.0, 5.0], [7.0, 4.0, 
3.0]]], [[[2.0, 8.0, 6.0], [3.0, 6.0, 7.0], [4.0, 2.0, 3.0]], [[8.0, 1.0, 4.0], [8.0, 7.0, 6.0], [8.0, 3.0, 1.0]], [[3.0, 3.0, 
1.0], [6.0, 8.0, 6.0], [3.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 6.0], [5.0, 8.0, 8.0], [6.0, 6.0, 5.0]], 
[[4.0, 4.0, 6.0], [5.0, 5.0, 1.0], [5.0, 8.0, 6.0]], [[6.0, 6.0, 2.0], [2.0, 6.0, 3.0], [5.0, 8.0, 2.0]]], [[[6.0, 4.0, 7.0], 
[1.0, 8.0, 6.0], [4.0, 2.0, 4.0]], [[6.0, 1.0, 3.0], [2.0, 3.0, 4.0], [5.0, 1.0, 7.0]], [[8.0, 3.0, 4.0], [2.0, 2.0, 8.0], 
[3.0, 8.0, 3.0]]], [[[5.0, 8.0, 1.0], [3.0, 2.0, 5.0], [2.0, 7.0, 5.0]], [[2.0, 7.0, 2.0], [8.0, 3.0, 3.0], [7.0, 7.0, 1.0]], 
[[4.0, 6.0, 4.0], [1.0, 2.0, 3.0], [5.0, 7.0, 2.0]]], [[[8.0, 5.0, 5.0], [8.0, 6.0, 7.0], [6.0, 4.0, 2.0]], [[3.0, 2.0, 7.0], 
[8.0, 5.0, 3.0], [1.0, 3.0, 3.0]], [[2.0, 6.0, 3.0], [3.0, 6.0, 2.0], [6.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PUUU(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 8.0], [5.0, 8.0, 1.0], [8.0, 5.0, 4.0]], [[8.0, 8.0, 2.0], [7.0, 5.0, 8.0], [1.0, 3.0, 
5.0]], [[1.0, 7.0, 3.0], [5.0, 4.0, 6.0], [1.0, 1.0, 4.0]]], [[[2.0, 2.0, 8.0], [7.0, 8.0, 6.0], [8.0, 7.0, 2.0]], [[2.0, 6.0, 
6.0], [7.0, 1.0, 7.0], [3.0, 1.0, 4.0]], [[7.0, 4.0, 5.0], [1.0, 2.0, 4.0], [2.0, 6.0, 3.0]]], [[[5.0, 2.0, 5.0], [5.0, 6.0, 
7.0], [3.0, 2.0, 2.0]], [[6.0, 5.0, 1.0], [6.0, 2.0, 6.0], [5.0, 6.0, 2.0]], [[7.0, 2.0, 6.0], [8.0, 4.0, 4.0], [1.0, 5.0, 
8.0]]], [[[7.0, 5.0, 1.0], [2.0, 7.0, 6.0], [7.0, 7.0, 6.0]], [[3.0, 5.0, 2.0], [2.0, 7.0, 5.0], [6.0, 8.0, 6.0]], [[6.0, 2.0, 
4.0], [6.0, 6.0, 1.0], [3.0, 3.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 8.0], [4.0, 4.0, 3.0], [8.0, 6.0, 5.0]], [[7.0, 8.0, 1.0], [3.0, 3.0, 
1.0], [7.0, 2.0, 5.0]], [[7.0, 5.0, 8.0], [5.0, 4.0, 4.0], [8.0, 3.0, 1.0]]], [[[4.0, 8.0, 7.0], [3.0, 7.0, 3.0], [1.0, 2.0, 
4.0]], [[4.0, 8.0, 5.0], [6.0, 5.0, 2.0], [8.0, 7.0, 6.0]], [[5.0, 7.0, 2.0], [5.0, 7.0, 2.0], [5.0, 5.0, 6.0]]], [[[5.0, 7.0, 
1.0], [4.0, 4.0, 4.0], [5.0, 4.0, 2.0]], [[8.0, 1.0, 4.0], [2.0, 4.0, 3.0], [3.0, 2.0, 4.0]], [[3.0, 1.0, 4.0], [5.0, 2.0, 
5.0], [4.0, 6.0, 5.0]]], [[[6.0, 3.0, 5.0], [4.0, 3.0, 6.0], [2.0, 3.0, 5.0]], [[3.0, 2.0, 1.0], [7.0, 7.0, 2.0], [4.0, 3.0, 
8.0]], [[5.0, 8.0, 8.0], [3.0, 4.0, 3.0], [5.0, 5.0, 6.0]]]]))
      val=Data(numpy.array([[[6.0, 8.0], [5.0, 7.0]], [[4.0, 4.0], [1.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 8.0], [6.0, 3.0]], [[2.0, 2.0], [6.0, 3.0]]]))
      arg[2,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 8.0], [5.0, 8.0, 1.0], [8.0, 5.0, 4.0]], [[8.0, 8.0, 2.0], [7.0, 5.0, 8.0], [1.0, 
3.0, 5.0]], [[1.0, 7.0, 3.0], [5.0, 4.0, 6.0], [1.0, 1.0, 4.0]]], [[[2.0, 2.0, 8.0], [7.0, 8.0, 6.0], [8.0, 7.0, 2.0]], [[2.0, 
6.0, 6.0], [7.0, 1.0, 7.0], [3.0, 1.0, 4.0]], [[7.0, 4.0, 5.0], [1.0, 2.0, 4.0], [2.0, 6.0, 3.0]]], [[[6.0, 8.0, 5.0], [5.0, 
7.0, 7.0], [3.0, 2.0, 2.0]], [[4.0, 4.0, 1.0], [1.0, 1.0, 6.0], [5.0, 6.0, 2.0]], [[7.0, 2.0, 6.0], [8.0, 4.0, 4.0], [1.0, 5.0, 
8.0]]], [[[7.0, 5.0, 1.0], [2.0, 7.0, 6.0], [7.0, 7.0, 6.0]], [[3.0, 5.0, 2.0], [2.0, 7.0, 5.0], [6.0, 8.0, 6.0]], [[6.0, 2.0, 
4.0], [6.0, 6.0, 1.0], [3.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 8.0], [4.0, 4.0, 3.0], [8.0, 6.0, 5.0]], 
[[7.0, 8.0, 1.0], [3.0, 3.0, 1.0], [7.0, 2.0, 5.0]], [[7.0, 5.0, 8.0], [5.0, 4.0, 4.0], [8.0, 3.0, 1.0]]], [[[4.0, 8.0, 7.0], 
[3.0, 7.0, 3.0], [1.0, 2.0, 4.0]], [[4.0, 8.0, 5.0], [6.0, 5.0, 2.0], [8.0, 7.0, 6.0]], [[5.0, 7.0, 2.0], [5.0, 7.0, 2.0], 
[5.0, 5.0, 6.0]]], [[[1.0, 8.0, 1.0], [6.0, 3.0, 4.0], [5.0, 4.0, 2.0]], [[2.0, 2.0, 4.0], [6.0, 3.0, 3.0], [3.0, 2.0, 4.0]], 
[[3.0, 1.0, 4.0], [5.0, 2.0, 5.0], [4.0, 6.0, 5.0]]], [[[6.0, 3.0, 5.0], [4.0, 3.0, 6.0], [2.0, 3.0, 5.0]], [[3.0, 2.0, 1.0], 
[7.0, 7.0, 2.0], [4.0, 3.0, 8.0]], [[5.0, 8.0, 8.0], [3.0, 4.0, 3.0], [5.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PUUU(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 7.0], [5.0, 3.0, 8.0], [6.0, 7.0, 5.0]], [[8.0, 4.0, 7.0], [6.0, 5.0, 7.0], [2.0, 2.0, 
5.0]], [[7.0, 7.0, 3.0], [3.0, 7.0, 7.0], [7.0, 6.0, 5.0]]], [[[7.0, 6.0, 8.0], [3.0, 7.0, 4.0], [6.0, 6.0, 1.0]], [[6.0, 5.0, 
3.0], [6.0, 2.0, 3.0], [5.0, 4.0, 6.0]], [[3.0, 7.0, 5.0], [4.0, 2.0, 5.0], [2.0, 1.0, 7.0]]], [[[2.0, 6.0, 5.0], [2.0, 4.0, 
8.0], [5.0, 5.0, 6.0]], [[6.0, 4.0, 8.0], [6.0, 4.0, 1.0], [1.0, 1.0, 8.0]], [[2.0, 7.0, 3.0], [3.0, 4.0, 4.0], [5.0, 6.0, 
8.0]]], [[[2.0, 2.0, 6.0], [5.0, 3.0, 6.0], [6.0, 1.0, 2.0]], [[4.0, 4.0, 2.0], [5.0, 2.0, 5.0], [2.0, 3.0, 2.0]], [[7.0, 1.0, 
3.0], [8.0, 3.0, 8.0], [6.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 8.0], [7.0, 4.0, 8.0], [6.0, 2.0, 3.0]], [[6.0, 7.0, 1.0], [4.0, 6.0, 
5.0], [6.0, 4.0, 7.0]], [[2.0, 7.0, 5.0], [6.0, 6.0, 7.0], [7.0, 5.0, 2.0]]], [[[5.0, 1.0, 6.0], [8.0, 5.0, 4.0], [2.0, 4.0, 
8.0]], [[3.0, 6.0, 7.0], [5.0, 6.0, 7.0], [6.0, 3.0, 1.0]], [[5.0, 7.0, 8.0], [4.0, 6.0, 8.0], [5.0, 3.0, 6.0]]], [[[2.0, 3.0, 
4.0], [1.0, 4.0, 6.0], [7.0, 7.0, 5.0]], [[6.0, 3.0, 4.0], [5.0, 6.0, 5.0], [2.0, 1.0, 6.0]], [[4.0, 2.0, 5.0], [5.0, 3.0, 
8.0], [3.0, 7.0, 2.0]]], [[[2.0, 4.0, 1.0], [3.0, 2.0, 5.0], [3.0, 2.0, 4.0]], [[1.0, 5.0, 1.0], [8.0, 7.0, 6.0], [5.0, 8.0, 
6.0]], [[4.0, 5.0, 7.0], [5.0, 7.0, 1.0], [8.0, 2.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[2.0, 8.0], [2.0, 8.0]], [[6.0, 8.0], [3.0, 1.0]]])+(1.-msk_val)*numpy.array([[[4.0, 7.0], 
[2.0, 3.0]], [[3.0, 3.0], [1.0, 7.0]]])
      arg[2,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 7.0], [5.0, 3.0, 8.0], [6.0, 7.0, 5.0]], [[8.0, 4.0, 7.0], [6.0, 5.0, 7.0], [2.0, 
2.0, 5.0]], [[7.0, 7.0, 3.0], [3.0, 7.0, 7.0], [7.0, 6.0, 5.0]]], [[[7.0, 6.0, 8.0], [3.0, 7.0, 4.0], [6.0, 6.0, 1.0]], [[6.0, 
5.0, 3.0], [6.0, 2.0, 3.0], [5.0, 4.0, 6.0]], [[3.0, 7.0, 5.0], [4.0, 2.0, 5.0], [2.0, 1.0, 7.0]]], [[[2.0, 8.0, 5.0], [2.0, 
8.0, 8.0], [5.0, 5.0, 6.0]], [[6.0, 8.0, 8.0], [3.0, 1.0, 1.0], [1.0, 1.0, 8.0]], [[2.0, 7.0, 3.0], [3.0, 4.0, 4.0], [5.0, 6.0, 
8.0]]], [[[2.0, 2.0, 6.0], [5.0, 3.0, 6.0], [6.0, 1.0, 2.0]], [[4.0, 4.0, 2.0], [5.0, 2.0, 5.0], [2.0, 3.0, 2.0]], [[7.0, 1.0, 
3.0], [8.0, 3.0, 8.0], [6.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 8.0], [7.0, 4.0, 8.0], [6.0, 2.0, 3.0]], 
[[6.0, 7.0, 1.0], [4.0, 6.0, 5.0], [6.0, 4.0, 7.0]], [[2.0, 7.0, 5.0], [6.0, 6.0, 7.0], [7.0, 5.0, 2.0]]], [[[5.0, 1.0, 6.0], 
[8.0, 5.0, 4.0], [2.0, 4.0, 8.0]], [[3.0, 6.0, 7.0], [5.0, 6.0, 7.0], [6.0, 3.0, 1.0]], [[5.0, 7.0, 8.0], [4.0, 6.0, 8.0], 
[5.0, 3.0, 6.0]]], [[[4.0, 7.0, 4.0], [2.0, 3.0, 6.0], [7.0, 7.0, 5.0]], [[3.0, 3.0, 4.0], [1.0, 7.0, 5.0], [2.0, 1.0, 6.0]], 
[[4.0, 2.0, 5.0], [5.0, 3.0, 8.0], [3.0, 7.0, 2.0]]], [[[2.0, 4.0, 1.0], [3.0, 2.0, 5.0], [3.0, 2.0, 4.0]], [[1.0, 5.0, 1.0], 
[8.0, 7.0, 6.0], [5.0, 8.0, 6.0]], [[4.0, 5.0, 7.0], [5.0, 7.0, 1.0], [8.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PULN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 5.0], [3.0, 1.0, 6.0], [8.0, 8.0, 7.0]], [[7.0, 3.0, 8.0], [8.0, 5.0, 2.0], [3.0, 2.0, 
6.0]], [[7.0, 2.0, 1.0], [5.0, 8.0, 4.0], [2.0, 3.0, 7.0]]], [[[8.0, 6.0, 4.0], [1.0, 4.0, 7.0], [4.0, 6.0, 6.0]], [[3.0, 2.0, 
7.0], [7.0, 1.0, 2.0], [4.0, 3.0, 7.0]], [[8.0, 4.0, 2.0], [5.0, 2.0, 5.0], [4.0, 5.0, 3.0]]], [[[5.0, 5.0, 5.0], [3.0, 3.0, 
4.0], [7.0, 6.0, 2.0]], [[8.0, 3.0, 5.0], [8.0, 1.0, 5.0], [1.0, 8.0, 5.0]], [[7.0, 1.0, 8.0], [8.0, 4.0, 8.0], [5.0, 6.0, 
7.0]]], [[[8.0, 1.0, 4.0], [8.0, 4.0, 8.0], [7.0, 1.0, 4.0]], [[5.0, 8.0, 4.0], [2.0, 2.0, 6.0], [5.0, 8.0, 6.0]], [[4.0, 1.0, 
3.0], [5.0, 2.0, 8.0], [8.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 5.0], [4.0, 1.0, 8.0], [2.0, 1.0, 3.0]], [[5.0, 4.0, 1.0], [5.0, 5.0, 
6.0], [4.0, 7.0, 1.0]], [[7.0, 1.0, 4.0], [1.0, 1.0, 8.0], [2.0, 1.0, 6.0]]], [[[7.0, 1.0, 7.0], [2.0, 3.0, 5.0], [1.0, 5.0, 
5.0]], [[7.0, 8.0, 8.0], [7.0, 6.0, 7.0], [8.0, 7.0, 6.0]], [[4.0, 6.0, 8.0], [1.0, 2.0, 7.0], [4.0, 7.0, 7.0]]], [[[5.0, 4.0, 
2.0], [2.0, 7.0, 8.0], [6.0, 5.0, 5.0]], [[3.0, 8.0, 6.0], [8.0, 5.0, 4.0], [2.0, 3.0, 5.0]], [[6.0, 3.0, 2.0], [6.0, 1.0, 
5.0], [1.0, 8.0, 4.0]]], [[[2.0, 5.0, 3.0], [4.0, 7.0, 6.0], [6.0, 8.0, 3.0]], [[7.0, 2.0, 7.0], [4.0, 2.0, 2.0], [8.0, 5.0, 
3.0]], [[1.0, 8.0, 5.0], [6.0, 4.0, 2.0], [6.0, 8.0, 3.0]]]]))
      val=numpy.array([[[2.0, 4.0, 4.0]], [[5.0, 7.0, 2.0]]])
      arg[2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 5.0], [3.0, 1.0, 6.0], [8.0, 8.0, 7.0]], [[7.0, 3.0, 8.0], [8.0, 5.0, 2.0], [3.0, 
2.0, 6.0]], [[7.0, 2.0, 1.0], [5.0, 8.0, 4.0], [2.0, 3.0, 7.0]]], [[[8.0, 6.0, 4.0], [1.0, 4.0, 7.0], [4.0, 6.0, 6.0]], [[3.0, 
2.0, 7.0], [7.0, 1.0, 2.0], [4.0, 3.0, 7.0]], [[8.0, 4.0, 2.0], [5.0, 2.0, 5.0], [4.0, 5.0, 3.0]]], [[[5.0, 5.0, 5.0], [3.0, 
3.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 3.0, 5.0], [8.0, 1.0, 5.0], [5.0, 7.0, 2.0]], [[7.0, 1.0, 8.0], [8.0, 4.0, 8.0], [5.0, 6.0, 
7.0]]], [[[8.0, 1.0, 4.0], [8.0, 4.0, 8.0], [7.0, 1.0, 4.0]], [[5.0, 8.0, 4.0], [2.0, 2.0, 6.0], [5.0, 8.0, 6.0]], [[4.0, 1.0, 
3.0], [5.0, 2.0, 8.0], [8.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 5.0], [4.0, 1.0, 8.0], [2.0, 1.0, 3.0]], 
[[5.0, 4.0, 1.0], [5.0, 5.0, 6.0], [4.0, 7.0, 1.0]], [[7.0, 1.0, 4.0], [1.0, 1.0, 8.0], [2.0, 1.0, 6.0]]], [[[7.0, 1.0, 7.0], 
[2.0, 3.0, 5.0], [1.0, 5.0, 5.0]], [[7.0, 8.0, 8.0], [7.0, 6.0, 7.0], [8.0, 7.0, 6.0]], [[4.0, 6.0, 8.0], [1.0, 2.0, 7.0], 
[4.0, 7.0, 7.0]]], [[[5.0, 4.0, 2.0], [2.0, 7.0, 8.0], [2.0, 4.0, 4.0]], [[3.0, 8.0, 6.0], [8.0, 5.0, 4.0], [5.0, 7.0, 2.0]], 
[[6.0, 3.0, 2.0], [6.0, 1.0, 5.0], [1.0, 8.0, 4.0]]], [[[2.0, 5.0, 3.0], [4.0, 7.0, 6.0], [6.0, 8.0, 3.0]], [[7.0, 2.0, 7.0], 
[4.0, 2.0, 2.0], [8.0, 5.0, 3.0]], [[1.0, 8.0, 5.0], [6.0, 4.0, 2.0], [6.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PULN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 2.0], [6.0, 1.0, 6.0], [2.0, 1.0, 2.0]], [[6.0, 8.0, 6.0], [1.0, 6.0, 3.0], [6.0, 5.0, 
5.0]], [[3.0, 2.0, 4.0], [4.0, 5.0, 7.0], [5.0, 4.0, 5.0]]], [[[1.0, 8.0, 1.0], [3.0, 4.0, 1.0], [1.0, 8.0, 5.0]], [[4.0, 3.0, 
1.0], [2.0, 7.0, 7.0], [5.0, 4.0, 8.0]], [[2.0, 4.0, 6.0], [4.0, 4.0, 6.0], [2.0, 2.0, 7.0]]], [[[3.0, 4.0, 7.0], [1.0, 1.0, 
5.0], [2.0, 4.0, 6.0]], [[3.0, 6.0, 7.0], [7.0, 3.0, 2.0], [8.0, 1.0, 7.0]], [[4.0, 8.0, 6.0], [6.0, 5.0, 8.0], [7.0, 2.0, 
4.0]]], [[[5.0, 8.0, 5.0], [3.0, 4.0, 4.0], [2.0, 6.0, 8.0]], [[3.0, 4.0, 4.0], [8.0, 2.0, 6.0], [6.0, 7.0, 7.0]], [[2.0, 3.0, 
1.0], [7.0, 5.0, 1.0], [8.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 6.0], [6.0, 3.0, 7.0], [1.0, 1.0, 8.0]], [[3.0, 6.0, 2.0], [3.0, 4.0, 
6.0], [5.0, 6.0, 7.0]], [[1.0, 1.0, 2.0], [6.0, 4.0, 1.0], [1.0, 8.0, 6.0]]], [[[3.0, 8.0, 6.0], [1.0, 4.0, 8.0], [6.0, 5.0, 
7.0]], [[7.0, 1.0, 4.0], [7.0, 1.0, 1.0], [6.0, 6.0, 1.0]], [[5.0, 8.0, 3.0], [5.0, 7.0, 1.0], [5.0, 5.0, 7.0]]], [[[6.0, 3.0, 
4.0], [1.0, 6.0, 6.0], [2.0, 3.0, 1.0]], [[7.0, 2.0, 2.0], [4.0, 6.0, 4.0], [2.0, 1.0, 4.0]], [[6.0, 8.0, 5.0], [8.0, 4.0, 
5.0], [5.0, 7.0, 1.0]]], [[[3.0, 3.0, 6.0], [5.0, 8.0, 8.0], [7.0, 7.0, 2.0]], [[4.0, 3.0, 1.0], [3.0, 2.0, 3.0], [7.0, 5.0, 
3.0]], [[3.0, 4.0, 8.0], [1.0, 4.0, 2.0], [5.0, 5.0, 8.0]]]]))
      val=Data(numpy.array([[[4.0, 4.0, 6.0]], [[3.0, 7.0, 7.0]]]),self.functionspace)
      arg[2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 2.0], [6.0, 1.0, 6.0], [2.0, 1.0, 2.0]], [[6.0, 8.0, 6.0], [1.0, 6.0, 3.0], [6.0, 
5.0, 5.0]], [[3.0, 2.0, 4.0], [4.0, 5.0, 7.0], [5.0, 4.0, 5.0]]], [[[1.0, 8.0, 1.0], [3.0, 4.0, 1.0], [1.0, 8.0, 5.0]], [[4.0, 
3.0, 1.0], [2.0, 7.0, 7.0], [5.0, 4.0, 8.0]], [[2.0, 4.0, 6.0], [4.0, 4.0, 6.0], [2.0, 2.0, 7.0]]], [[[3.0, 4.0, 7.0], [1.0, 
1.0, 5.0], [4.0, 4.0, 6.0]], [[3.0, 6.0, 7.0], [7.0, 3.0, 2.0], [3.0, 7.0, 7.0]], [[4.0, 8.0, 6.0], [6.0, 5.0, 8.0], [7.0, 2.0, 
4.0]]], [[[5.0, 8.0, 5.0], [3.0, 4.0, 4.0], [2.0, 6.0, 8.0]], [[3.0, 4.0, 4.0], [8.0, 2.0, 6.0], [6.0, 7.0, 7.0]], [[2.0, 3.0, 
1.0], [7.0, 5.0, 1.0], [8.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 6.0], [6.0, 3.0, 7.0], [1.0, 1.0, 8.0]], 
[[3.0, 6.0, 2.0], [3.0, 4.0, 6.0], [5.0, 6.0, 7.0]], [[1.0, 1.0, 2.0], [6.0, 4.0, 1.0], [1.0, 8.0, 6.0]]], [[[3.0, 8.0, 6.0], 
[1.0, 4.0, 8.0], [6.0, 5.0, 7.0]], [[7.0, 1.0, 4.0], [7.0, 1.0, 1.0], [6.0, 6.0, 1.0]], [[5.0, 8.0, 3.0], [5.0, 7.0, 1.0], 
[5.0, 5.0, 7.0]]], [[[6.0, 3.0, 4.0], [1.0, 6.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 2.0, 2.0], [4.0, 6.0, 4.0], [3.0, 7.0, 7.0]], 
[[6.0, 8.0, 5.0], [8.0, 4.0, 5.0], [5.0, 7.0, 1.0]]], [[[3.0, 3.0, 6.0], [5.0, 8.0, 8.0], [7.0, 7.0, 2.0]], [[4.0, 3.0, 1.0], 
[3.0, 2.0, 3.0], [7.0, 5.0, 3.0]], [[3.0, 4.0, 8.0], [1.0, 4.0, 2.0], [5.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PULN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 2.0], [1.0, 1.0, 2.0], [1.0, 1.0, 4.0]], [[6.0, 3.0, 4.0], [1.0, 3.0, 2.0], [2.0, 8.0, 
3.0]], [[5.0, 5.0, 6.0], [2.0, 2.0, 8.0], [2.0, 7.0, 8.0]]], [[[5.0, 5.0, 2.0], [5.0, 3.0, 8.0], [4.0, 4.0, 6.0]], [[2.0, 6.0, 
1.0], [3.0, 7.0, 4.0], [6.0, 2.0, 3.0]], [[7.0, 6.0, 4.0], [5.0, 6.0, 2.0], [4.0, 7.0, 6.0]]], [[[1.0, 5.0, 5.0], [8.0, 2.0, 
4.0], [6.0, 3.0, 1.0]], [[4.0, 7.0, 6.0], [2.0, 3.0, 7.0], [7.0, 2.0, 3.0]], [[4.0, 6.0, 4.0], [8.0, 1.0, 4.0], [7.0, 2.0, 
3.0]]], [[[2.0, 6.0, 6.0], [2.0, 1.0, 2.0], [7.0, 6.0, 1.0]], [[6.0, 4.0, 4.0], [2.0, 2.0, 8.0], [8.0, 4.0, 1.0]], [[1.0, 7.0, 
6.0], [1.0, 6.0, 1.0], [8.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 2.0], [1.0, 7.0, 1.0], [5.0, 8.0, 7.0]], [[4.0, 1.0, 4.0], [6.0, 4.0, 
7.0], [2.0, 8.0, 6.0]], [[8.0, 4.0, 5.0], [4.0, 3.0, 2.0], [3.0, 2.0, 6.0]]], [[[8.0, 2.0, 6.0], [4.0, 5.0, 1.0], [2.0, 3.0, 
8.0]], [[8.0, 6.0, 4.0], [6.0, 5.0, 7.0], [1.0, 7.0, 7.0]], [[7.0, 2.0, 1.0], [8.0, 2.0, 8.0], [6.0, 1.0, 7.0]]], [[[2.0, 8.0, 
5.0], [6.0, 6.0, 3.0], [7.0, 3.0, 4.0]], [[1.0, 5.0, 1.0], [5.0, 2.0, 4.0], [2.0, 8.0, 6.0]], [[5.0, 5.0, 7.0], [2.0, 5.0, 
7.0], [5.0, 8.0, 1.0]]], [[[1.0, 6.0, 2.0], [3.0, 6.0, 1.0], [3.0, 2.0, 7.0]], [[8.0, 4.0, 3.0], [5.0, 7.0, 8.0], [1.0, 6.0, 
2.0]], [[4.0, 4.0, 7.0], [5.0, 3.0, 3.0], [5.0, 1.0, 5.0]]]]))
      val=Data(numpy.array([[[8.0, 6.0, 5.0]], [[2.0, 4.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0, 5.0]], [[5.0, 5.0, 7.0]]]))
      arg[2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 2.0], [1.0, 1.0, 2.0], [1.0, 1.0, 4.0]], [[6.0, 3.0, 4.0], [1.0, 3.0, 2.0], [2.0, 
8.0, 3.0]], [[5.0, 5.0, 6.0], [2.0, 2.0, 8.0], [2.0, 7.0, 8.0]]], [[[5.0, 5.0, 2.0], [5.0, 3.0, 8.0], [4.0, 4.0, 6.0]], [[2.0, 
6.0, 1.0], [3.0, 7.0, 4.0], [6.0, 2.0, 3.0]], [[7.0, 6.0, 4.0], [5.0, 6.0, 2.0], [4.0, 7.0, 6.0]]], [[[1.0, 5.0, 5.0], [8.0, 
2.0, 4.0], [8.0, 6.0, 5.0]], [[4.0, 7.0, 6.0], [2.0, 3.0, 7.0], [2.0, 4.0, 4.0]], [[4.0, 6.0, 4.0], [8.0, 1.0, 4.0], [7.0, 2.0, 
3.0]]], [[[2.0, 6.0, 6.0], [2.0, 1.0, 2.0], [7.0, 6.0, 1.0]], [[6.0, 4.0, 4.0], [2.0, 2.0, 8.0], [8.0, 4.0, 1.0]], [[1.0, 7.0, 
6.0], [1.0, 6.0, 1.0], [8.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 2.0], [1.0, 7.0, 1.0], [5.0, 8.0, 7.0]], 
[[4.0, 1.0, 4.0], [6.0, 4.0, 7.0], [2.0, 8.0, 6.0]], [[8.0, 4.0, 5.0], [4.0, 3.0, 2.0], [3.0, 2.0, 6.0]]], [[[8.0, 2.0, 6.0], 
[4.0, 5.0, 1.0], [2.0, 3.0, 8.0]], [[8.0, 6.0, 4.0], [6.0, 5.0, 7.0], [1.0, 7.0, 7.0]], [[7.0, 2.0, 1.0], [8.0, 2.0, 8.0], 
[6.0, 1.0, 7.0]]], [[[2.0, 8.0, 5.0], [6.0, 6.0, 3.0], [3.0, 5.0, 5.0]], [[1.0, 5.0, 1.0], [5.0, 2.0, 4.0], [5.0, 5.0, 7.0]], 
[[5.0, 5.0, 7.0], [2.0, 5.0, 7.0], [5.0, 8.0, 1.0]]], [[[1.0, 6.0, 2.0], [3.0, 6.0, 1.0], [3.0, 2.0, 7.0]], [[8.0, 4.0, 3.0], 
[5.0, 7.0, 8.0], [1.0, 6.0, 2.0]], [[4.0, 4.0, 7.0], [5.0, 3.0, 3.0], [5.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PULN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 4.0], [4.0, 5.0, 2.0], [5.0, 2.0, 1.0]], [[1.0, 5.0, 8.0], [2.0, 1.0, 1.0], [7.0, 7.0, 
5.0]], [[1.0, 1.0, 7.0], [1.0, 3.0, 4.0], [1.0, 1.0, 1.0]]], [[[2.0, 6.0, 5.0], [8.0, 3.0, 8.0], [5.0, 2.0, 7.0]], [[4.0, 5.0, 
2.0], [3.0, 5.0, 7.0], [7.0, 7.0, 5.0]], [[3.0, 5.0, 8.0], [2.0, 2.0, 4.0], [3.0, 2.0, 7.0]]], [[[6.0, 1.0, 5.0], [2.0, 8.0, 
2.0], [1.0, 4.0, 1.0]], [[8.0, 6.0, 7.0], [6.0, 1.0, 4.0], [3.0, 3.0, 1.0]], [[3.0, 4.0, 2.0], [2.0, 4.0, 6.0], [6.0, 2.0, 
3.0]]], [[[4.0, 1.0, 6.0], [5.0, 3.0, 6.0], [8.0, 8.0, 2.0]], [[1.0, 5.0, 6.0], [2.0, 7.0, 8.0], [7.0, 5.0, 5.0]], [[5.0, 3.0, 
7.0], [3.0, 7.0, 7.0], [7.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 6.0], [4.0, 5.0, 5.0], [3.0, 3.0, 5.0]], [[2.0, 4.0, 3.0], [8.0, 1.0, 
2.0], [6.0, 2.0, 4.0]], [[1.0, 4.0, 8.0], [3.0, 5.0, 5.0], [8.0, 5.0, 4.0]]], [[[5.0, 8.0, 5.0], [3.0, 3.0, 1.0], [3.0, 1.0, 
5.0]], [[7.0, 1.0, 8.0], [6.0, 6.0, 1.0], [2.0, 2.0, 5.0]], [[7.0, 2.0, 2.0], [6.0, 1.0, 6.0], [2.0, 7.0, 5.0]]], [[[6.0, 5.0, 
4.0], [4.0, 7.0, 6.0], [8.0, 2.0, 4.0]], [[6.0, 8.0, 8.0], [4.0, 4.0, 6.0], [8.0, 8.0, 4.0]], [[2.0, 8.0, 5.0], [3.0, 1.0, 
8.0], [7.0, 1.0, 7.0]]], [[[3.0, 5.0, 1.0], [3.0, 3.0, 8.0], [6.0, 3.0, 4.0]], [[4.0, 5.0, 8.0], [6.0, 2.0, 5.0], [8.0, 5.0, 
1.0]], [[8.0, 4.0, 5.0], [4.0, 5.0, 4.0], [1.0, 7.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 4.0, 3.0]], [[2.0, 8.0, 7.0]]])+(1.-msk_val)*numpy.array([[[5.0, 3.0, 3.0]], [[3.0, 2.0, 
7.0]]])
      arg[2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 4.0], [4.0, 5.0, 2.0], [5.0, 2.0, 1.0]], [[1.0, 5.0, 8.0], [2.0, 1.0, 1.0], [7.0, 
7.0, 5.0]], [[1.0, 1.0, 7.0], [1.0, 3.0, 4.0], [1.0, 1.0, 1.0]]], [[[2.0, 6.0, 5.0], [8.0, 3.0, 8.0], [5.0, 2.0, 7.0]], [[4.0, 
5.0, 2.0], [3.0, 5.0, 7.0], [7.0, 7.0, 5.0]], [[3.0, 5.0, 8.0], [2.0, 2.0, 4.0], [3.0, 2.0, 7.0]]], [[[6.0, 1.0, 5.0], [2.0, 
8.0, 2.0], [7.0, 4.0, 3.0]], [[8.0, 6.0, 7.0], [6.0, 1.0, 4.0], [2.0, 8.0, 7.0]], [[3.0, 4.0, 2.0], [2.0, 4.0, 6.0], [6.0, 2.0, 
3.0]]], [[[4.0, 1.0, 6.0], [5.0, 3.0, 6.0], [8.0, 8.0, 2.0]], [[1.0, 5.0, 6.0], [2.0, 7.0, 8.0], [7.0, 5.0, 5.0]], [[5.0, 3.0, 
7.0], [3.0, 7.0, 7.0], [7.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 6.0], [4.0, 5.0, 5.0], [3.0, 3.0, 5.0]], 
[[2.0, 4.0, 3.0], [8.0, 1.0, 2.0], [6.0, 2.0, 4.0]], [[1.0, 4.0, 8.0], [3.0, 5.0, 5.0], [8.0, 5.0, 4.0]]], [[[5.0, 8.0, 5.0], 
[3.0, 3.0, 1.0], [3.0, 1.0, 5.0]], [[7.0, 1.0, 8.0], [6.0, 6.0, 1.0], [2.0, 2.0, 5.0]], [[7.0, 2.0, 2.0], [6.0, 1.0, 6.0], 
[2.0, 7.0, 5.0]]], [[[6.0, 5.0, 4.0], [4.0, 7.0, 6.0], [5.0, 3.0, 3.0]], [[6.0, 8.0, 8.0], [4.0, 4.0, 6.0], [3.0, 2.0, 7.0]], 
[[2.0, 8.0, 5.0], [3.0, 1.0, 8.0], [7.0, 1.0, 7.0]]], [[[3.0, 5.0, 1.0], [3.0, 3.0, 8.0], [6.0, 3.0, 4.0]], [[4.0, 5.0, 8.0], 
[6.0, 2.0, 5.0], [8.0, 5.0, 1.0]], [[8.0, 4.0, 5.0], [4.0, 5.0, 4.0], [1.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UCUU(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 8.0], [1.0, 8.0, 3.0], [5.0, 2.0, 7.0]], [[4.0, 4.0, 7.0], [5.0, 1.0, 2.0], [1.0, 2.0, 
3.0]], [[1.0, 5.0, 7.0], [6.0, 6.0, 7.0], [1.0, 1.0, 7.0]]], [[[8.0, 3.0, 4.0], [8.0, 4.0, 2.0], [4.0, 1.0, 7.0]], [[7.0, 3.0, 
4.0], [1.0, 5.0, 5.0], [1.0, 8.0, 3.0]], [[7.0, 7.0, 1.0], [1.0, 1.0, 1.0], [1.0, 2.0, 1.0]]], [[[7.0, 4.0, 3.0], [3.0, 6.0, 
4.0], [5.0, 8.0, 3.0]], [[4.0, 7.0, 5.0], [7.0, 1.0, 6.0], [6.0, 1.0, 7.0]], [[3.0, 4.0, 7.0], [5.0, 6.0, 8.0], [2.0, 2.0, 
7.0]]], [[[6.0, 7.0, 8.0], [4.0, 8.0, 4.0], [3.0, 7.0, 5.0]], [[3.0, 4.0, 1.0], [3.0, 1.0, 4.0], [1.0, 1.0, 3.0]], [[8.0, 5.0, 
1.0], [6.0, 6.0, 5.0], [8.0, 8.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 8.0], [4.0, 8.0, 7.0], [3.0, 8.0, 3.0]], [[4.0, 1.0, 7.0], [1.0, 3.0, 
8.0], [4.0, 5.0, 4.0]], [[6.0, 5.0, 7.0], [6.0, 7.0, 1.0], [2.0, 7.0, 8.0]]], [[[3.0, 7.0, 2.0], [5.0, 3.0, 8.0], [3.0, 3.0, 
4.0]], [[7.0, 2.0, 5.0], [2.0, 5.0, 1.0], [3.0, 2.0, 2.0]], [[7.0, 2.0, 8.0], [1.0, 8.0, 1.0], [3.0, 6.0, 6.0]]], [[[8.0, 4.0, 
2.0], [6.0, 6.0, 3.0], [1.0, 6.0, 4.0]], [[8.0, 4.0, 5.0], [4.0, 5.0, 3.0], [1.0, 1.0, 7.0]], [[5.0, 4.0, 7.0], [4.0, 3.0, 
1.0], [5.0, 8.0, 8.0]]], [[[4.0, 4.0, 7.0], [2.0, 8.0, 1.0], [4.0, 5.0, 1.0]], [[4.0, 3.0, 5.0], [4.0, 6.0, 7.0], [2.0, 2.0, 
2.0]], [[3.0, 3.0, 7.0], [4.0, 5.0, 8.0], [8.0, 2.0, 1.0]]]]))
      val=numpy.array([[[[1.0, 3.0], [5.0, 8.0]], [[2.0, 7.0], [3.0, 6.0]], [[6.0, 7.0], [1.0, 5.0]]], [[[4.0, 3.0], [6.0, 
6.0]], [[1.0, 3.0], [3.0, 3.0]], [[4.0, 8.0], [8.0, 3.0]]], [[[7.0, 2.0], [6.0, 7.0]], [[2.0, 2.0], [3.0, 8.0]], [[4.0, 6.0], 
[2.0, 3.0]]]])
      arg[:3,:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 8.0], [5.0, 8.0, 3.0], [5.0, 2.0, 7.0]], [[2.0, 7.0, 7.0], [3.0, 6.0, 2.0], [1.0, 
2.0, 3.0]], [[6.0, 7.0, 7.0], [1.0, 5.0, 7.0], [1.0, 1.0, 7.0]]], [[[4.0, 3.0, 4.0], [6.0, 6.0, 2.0], [4.0, 1.0, 7.0]], [[1.0, 
3.0, 4.0], [3.0, 3.0, 5.0], [1.0, 8.0, 3.0]], [[4.0, 8.0, 1.0], [8.0, 3.0, 1.0], [1.0, 2.0, 1.0]]], [[[7.0, 2.0, 3.0], [6.0, 
7.0, 4.0], [5.0, 8.0, 3.0]], [[2.0, 2.0, 5.0], [3.0, 8.0, 6.0], [6.0, 1.0, 7.0]], [[4.0, 6.0, 7.0], [2.0, 3.0, 8.0], [2.0, 2.0, 
7.0]]], [[[6.0, 7.0, 8.0], [4.0, 8.0, 4.0], [3.0, 7.0, 5.0]], [[3.0, 4.0, 1.0], [3.0, 1.0, 4.0], [1.0, 1.0, 3.0]], [[8.0, 5.0, 
1.0], [6.0, 6.0, 5.0], [8.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 8.0], [5.0, 8.0, 7.0], [3.0, 8.0, 3.0]], 
[[2.0, 7.0, 7.0], [3.0, 6.0, 8.0], [4.0, 5.0, 4.0]], [[6.0, 7.0, 7.0], [1.0, 5.0, 1.0], [2.0, 7.0, 8.0]]], [[[4.0, 3.0, 2.0], 
[6.0, 6.0, 8.0], [3.0, 3.0, 4.0]], [[1.0, 3.0, 5.0], [3.0, 3.0, 1.0], [3.0, 2.0, 2.0]], [[4.0, 8.0, 8.0], [8.0, 3.0, 1.0], 
[3.0, 6.0, 6.0]]], [[[7.0, 2.0, 2.0], [6.0, 7.0, 3.0], [1.0, 6.0, 4.0]], [[2.0, 2.0, 5.0], [3.0, 8.0, 3.0], [1.0, 1.0, 7.0]], 
[[4.0, 6.0, 7.0], [2.0, 3.0, 1.0], [5.0, 8.0, 8.0]]], [[[4.0, 4.0, 7.0], [2.0, 8.0, 1.0], [4.0, 5.0, 1.0]], [[4.0, 3.0, 5.0], 
[4.0, 6.0, 7.0], [2.0, 2.0, 2.0]], [[3.0, 3.0, 7.0], [4.0, 5.0, 8.0], [8.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UCUU(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 2.0], [5.0, 8.0, 5.0], [7.0, 5.0, 1.0]], [[4.0, 1.0, 6.0], [7.0, 2.0, 4.0], [4.0, 2.0, 
3.0]], [[7.0, 7.0, 8.0], [4.0, 7.0, 5.0], [8.0, 6.0, 3.0]]], [[[4.0, 2.0, 3.0], [2.0, 3.0, 7.0], [7.0, 8.0, 2.0]], [[1.0, 7.0, 
6.0], [2.0, 5.0, 1.0], [3.0, 5.0, 2.0]], [[8.0, 1.0, 7.0], [4.0, 7.0, 1.0], [3.0, 7.0, 1.0]]], [[[8.0, 8.0, 5.0], [6.0, 5.0, 
6.0], [7.0, 2.0, 4.0]], [[6.0, 5.0, 6.0], [8.0, 8.0, 8.0], [5.0, 5.0, 7.0]], [[1.0, 2.0, 6.0], [4.0, 8.0, 4.0], [5.0, 1.0, 
1.0]]], [[[4.0, 4.0, 1.0], [6.0, 4.0, 8.0], [7.0, 5.0, 5.0]], [[6.0, 7.0, 6.0], [5.0, 8.0, 2.0], [6.0, 8.0, 8.0]], [[1.0, 7.0, 
5.0], [6.0, 1.0, 3.0], [3.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 2.0], [4.0, 1.0, 8.0], [8.0, 3.0, 8.0]], [[8.0, 6.0, 5.0], [3.0, 4.0, 
1.0], [2.0, 1.0, 2.0]], [[2.0, 8.0, 2.0], [3.0, 7.0, 2.0], [1.0, 8.0, 7.0]]], [[[1.0, 8.0, 5.0], [6.0, 6.0, 3.0], [5.0, 3.0, 
1.0]], [[2.0, 2.0, 4.0], [8.0, 1.0, 1.0], [2.0, 3.0, 3.0]], [[4.0, 1.0, 4.0], [2.0, 5.0, 6.0], [2.0, 2.0, 6.0]]], [[[8.0, 6.0, 
7.0], [4.0, 5.0, 6.0], [5.0, 6.0, 2.0]], [[2.0, 6.0, 5.0], [1.0, 4.0, 1.0], [6.0, 5.0, 8.0]], [[6.0, 3.0, 2.0], [5.0, 3.0, 
2.0], [4.0, 4.0, 8.0]]], [[[1.0, 4.0, 4.0], [1.0, 7.0, 8.0], [4.0, 5.0, 8.0]], [[6.0, 2.0, 8.0], [7.0, 2.0, 7.0], [8.0, 5.0, 
3.0]], [[1.0, 6.0, 3.0], [1.0, 6.0, 2.0], [6.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[[2.0, 2.0], [8.0, 5.0]], [[4.0, 4.0], [2.0, 3.0]], [[1.0, 8.0], [8.0, 8.0]]], [[[8.0, 7.0], [4.0, 
8.0]], [[5.0, 1.0], [1.0, 5.0]], [[6.0, 8.0], [4.0, 2.0]]], [[[5.0, 8.0], [2.0, 1.0]], [[3.0, 2.0], [7.0, 3.0]], [[3.0, 4.0], 
[1.0, 7.0]]]]),self.functionspace)
      arg[:3,:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 2.0], [8.0, 5.0, 5.0], [7.0, 5.0, 1.0]], [[4.0, 4.0, 6.0], [2.0, 3.0, 4.0], [4.0, 
2.0, 3.0]], [[1.0, 8.0, 8.0], [8.0, 8.0, 5.0], [8.0, 6.0, 3.0]]], [[[8.0, 7.0, 3.0], [4.0, 8.0, 7.0], [7.0, 8.0, 2.0]], [[5.0, 
1.0, 6.0], [1.0, 5.0, 1.0], [3.0, 5.0, 2.0]], [[6.0, 8.0, 7.0], [4.0, 2.0, 1.0], [3.0, 7.0, 1.0]]], [[[5.0, 8.0, 5.0], [2.0, 
1.0, 6.0], [7.0, 2.0, 4.0]], [[3.0, 2.0, 6.0], [7.0, 3.0, 8.0], [5.0, 5.0, 7.0]], [[3.0, 4.0, 6.0], [1.0, 7.0, 4.0], [5.0, 1.0, 
1.0]]], [[[4.0, 4.0, 1.0], [6.0, 4.0, 8.0], [7.0, 5.0, 5.0]], [[6.0, 7.0, 6.0], [5.0, 8.0, 2.0], [6.0, 8.0, 8.0]], [[1.0, 7.0, 
5.0], [6.0, 1.0, 3.0], [3.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 2.0], [8.0, 5.0, 8.0], [8.0, 3.0, 8.0]], 
[[4.0, 4.0, 5.0], [2.0, 3.0, 1.0], [2.0, 1.0, 2.0]], [[1.0, 8.0, 2.0], [8.0, 8.0, 2.0], [1.0, 8.0, 7.0]]], [[[8.0, 7.0, 5.0], 
[4.0, 8.0, 3.0], [5.0, 3.0, 1.0]], [[5.0, 1.0, 4.0], [1.0, 5.0, 1.0], [2.0, 3.0, 3.0]], [[6.0, 8.0, 4.0], [4.0, 2.0, 6.0], 
[2.0, 2.0, 6.0]]], [[[5.0, 8.0, 7.0], [2.0, 1.0, 6.0], [5.0, 6.0, 2.0]], [[3.0, 2.0, 5.0], [7.0, 3.0, 1.0], [6.0, 5.0, 8.0]], 
[[3.0, 4.0, 2.0], [1.0, 7.0, 2.0], [4.0, 4.0, 8.0]]], [[[1.0, 4.0, 4.0], [1.0, 7.0, 8.0], [4.0, 5.0, 8.0]], [[6.0, 2.0, 8.0], 
[7.0, 2.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 6.0, 3.0], [1.0, 6.0, 2.0], [6.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UCUU(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 3.0], [2.0, 6.0, 5.0], [3.0, 8.0, 7.0]], [[3.0, 2.0, 4.0], [3.0, 6.0, 1.0], [8.0, 6.0, 
4.0]], [[1.0, 4.0, 4.0], [4.0, 2.0, 5.0], [1.0, 5.0, 5.0]]], [[[7.0, 7.0, 7.0], [2.0, 3.0, 4.0], [6.0, 8.0, 3.0]], [[3.0, 7.0, 
2.0], [4.0, 5.0, 4.0], [7.0, 8.0, 7.0]], [[1.0, 8.0, 6.0], [7.0, 4.0, 2.0], [6.0, 8.0, 5.0]]], [[[7.0, 4.0, 3.0], [8.0, 1.0, 
6.0], [4.0, 1.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 4.0, 3.0], [5.0, 5.0, 3.0]], [[4.0, 2.0, 3.0], [5.0, 1.0, 2.0], [6.0, 1.0, 
4.0]]], [[[5.0, 4.0, 7.0], [2.0, 6.0, 8.0], [7.0, 7.0, 3.0]], [[4.0, 5.0, 3.0], [1.0, 1.0, 4.0], [5.0, 6.0, 8.0]], [[6.0, 6.0, 
4.0], [2.0, 7.0, 6.0], [5.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 7.0], [8.0, 6.0, 5.0], [7.0, 7.0, 4.0]], [[7.0, 2.0, 6.0], [3.0, 7.0, 
1.0], [2.0, 7.0, 6.0]], [[6.0, 4.0, 2.0], [4.0, 4.0, 7.0], [8.0, 4.0, 7.0]]], [[[4.0, 6.0, 8.0], [3.0, 5.0, 4.0], [1.0, 1.0, 
3.0]], [[8.0, 6.0, 4.0], [2.0, 1.0, 1.0], [7.0, 1.0, 6.0]], [[6.0, 1.0, 5.0], [5.0, 4.0, 3.0], [8.0, 7.0, 3.0]]], [[[6.0, 3.0, 
6.0], [3.0, 4.0, 1.0], [7.0, 4.0, 1.0]], [[8.0, 2.0, 1.0], [2.0, 3.0, 6.0], [1.0, 8.0, 7.0]], [[2.0, 3.0, 5.0], [6.0, 1.0, 
4.0], [2.0, 5.0, 5.0]]], [[[7.0, 8.0, 7.0], [7.0, 7.0, 3.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 4.0], [4.0, 3.0, 7.0], [6.0, 6.0, 
5.0]], [[4.0, 3.0, 1.0], [1.0, 5.0, 2.0], [2.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([[[[2.0, 6.0], [2.0, 5.0]], [[7.0, 5.0], [5.0, 7.0]], [[4.0, 3.0], [7.0, 3.0]]], [[[6.0, 7.0], [3.0, 
5.0]], [[5.0, 2.0], [6.0, 1.0]], [[8.0, 8.0], [5.0, 5.0]]], [[[8.0, 4.0], [7.0, 8.0]], [[1.0, 1.0], [2.0, 8.0]], [[6.0, 1.0], 
[3.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 3.0], [4.0, 2.0]], [[8.0, 2.0], [5.0, 2.0]], [[8.0, 1.0], [1.0, 3.0]]], [[[2.0, 
4.0], [5.0, 5.0]], [[7.0, 2.0], [2.0, 4.0]], [[8.0, 6.0], [7.0, 5.0]]], [[[1.0, 2.0], [1.0, 2.0]], [[5.0, 4.0], [6.0, 3.0]], 
[[4.0, 4.0], [6.0, 4.0]]]]))
      arg[:3,:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 3.0], [2.0, 5.0, 5.0], [3.0, 8.0, 7.0]], [[7.0, 5.0, 4.0], [5.0, 7.0, 1.0], [8.0, 
6.0, 4.0]], [[4.0, 3.0, 4.0], [7.0, 3.0, 5.0], [1.0, 5.0, 5.0]]], [[[6.0, 7.0, 7.0], [3.0, 5.0, 4.0], [6.0, 8.0, 3.0]], [[5.0, 
2.0, 2.0], [6.0, 1.0, 4.0], [7.0, 8.0, 7.0]], [[8.0, 8.0, 6.0], [5.0, 5.0, 2.0], [6.0, 8.0, 5.0]]], [[[8.0, 4.0, 3.0], [7.0, 
8.0, 6.0], [4.0, 1.0, 6.0]], [[1.0, 1.0, 4.0], [2.0, 8.0, 3.0], [5.0, 5.0, 3.0]], [[6.0, 1.0, 3.0], [3.0, 4.0, 2.0], [6.0, 1.0, 
4.0]]], [[[5.0, 4.0, 7.0], [2.0, 6.0, 8.0], [7.0, 7.0, 3.0]], [[4.0, 5.0, 3.0], [1.0, 1.0, 4.0], [5.0, 6.0, 8.0]], [[6.0, 6.0, 
4.0], [2.0, 7.0, 6.0], [5.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 7.0], [4.0, 2.0, 5.0], [7.0, 7.0, 4.0]], 
[[8.0, 2.0, 6.0], [5.0, 2.0, 1.0], [2.0, 7.0, 6.0]], [[8.0, 1.0, 2.0], [1.0, 3.0, 7.0], [8.0, 4.0, 7.0]]], [[[2.0, 4.0, 8.0], 
[5.0, 5.0, 4.0], [1.0, 1.0, 3.0]], [[7.0, 2.0, 4.0], [2.0, 4.0, 1.0], [7.0, 1.0, 6.0]], [[8.0, 6.0, 5.0], [7.0, 5.0, 3.0], 
[8.0, 7.0, 3.0]]], [[[1.0, 2.0, 6.0], [1.0, 2.0, 1.0], [7.0, 4.0, 1.0]], [[5.0, 4.0, 1.0], [6.0, 3.0, 6.0], [1.0, 8.0, 7.0]], 
[[4.0, 4.0, 5.0], [6.0, 4.0, 4.0], [2.0, 5.0, 5.0]]], [[[7.0, 8.0, 7.0], [7.0, 7.0, 3.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 4.0], 
[4.0, 3.0, 7.0], [6.0, 6.0, 5.0]], [[4.0, 3.0, 1.0], [1.0, 5.0, 2.0], [2.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UCUU(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 6.0], [3.0, 5.0, 2.0], [6.0, 5.0, 5.0]], [[2.0, 3.0, 2.0], [3.0, 4.0, 2.0], [6.0, 2.0, 
7.0]], [[8.0, 4.0, 8.0], [2.0, 5.0, 6.0], [7.0, 2.0, 1.0]]], [[[4.0, 2.0, 4.0], [8.0, 4.0, 2.0], [7.0, 6.0, 5.0]], [[6.0, 1.0, 
5.0], [2.0, 1.0, 2.0], [2.0, 6.0, 5.0]], [[2.0, 8.0, 7.0], [4.0, 3.0, 6.0], [6.0, 6.0, 3.0]]], [[[2.0, 3.0, 2.0], [6.0, 3.0, 
1.0], [7.0, 7.0, 7.0]], [[5.0, 7.0, 1.0], [4.0, 2.0, 8.0], [7.0, 2.0, 1.0]], [[1.0, 1.0, 1.0], [4.0, 1.0, 3.0], [3.0, 6.0, 
8.0]]], [[[2.0, 4.0, 2.0], [1.0, 1.0, 5.0], [5.0, 4.0, 2.0]], [[1.0, 4.0, 6.0], [5.0, 8.0, 4.0], [6.0, 3.0, 2.0]], [[3.0, 7.0, 
4.0], [6.0, 3.0, 2.0], [5.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 8.0], [8.0, 3.0, 1.0], [4.0, 8.0, 2.0]], [[8.0, 7.0, 7.0], [5.0, 3.0, 
5.0], [6.0, 5.0, 6.0]], [[7.0, 4.0, 1.0], [3.0, 5.0, 7.0], [1.0, 4.0, 2.0]]], [[[8.0, 7.0, 5.0], [8.0, 1.0, 4.0], [4.0, 4.0, 
3.0]], [[3.0, 1.0, 4.0], [3.0, 4.0, 8.0], [5.0, 5.0, 8.0]], [[3.0, 1.0, 2.0], [5.0, 7.0, 8.0], [8.0, 1.0, 2.0]]], [[[8.0, 5.0, 
2.0], [1.0, 2.0, 2.0], [6.0, 7.0, 2.0]], [[7.0, 4.0, 1.0], [1.0, 5.0, 7.0], [4.0, 1.0, 5.0]], [[3.0, 7.0, 6.0], [8.0, 6.0, 
1.0], [7.0, 5.0, 3.0]]], [[[4.0, 6.0, 4.0], [1.0, 5.0, 2.0], [8.0, 2.0, 8.0]], [[7.0, 1.0, 1.0], [5.0, 5.0, 2.0], [8.0, 6.0, 
5.0]], [[5.0, 5.0, 1.0], [1.0, 3.0, 8.0], [2.0, 7.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 7.0], [3.0, 5.0]], [[5.0, 1.0], [3.0, 7.0]], [[5.0, 1.0], [7.0, 2.0]]], [[[5.0, 4.0], 
[3.0, 5.0]], [[1.0, 4.0], [8.0, 4.0]], [[7.0, 5.0], [6.0, 3.0]]], [[[7.0, 3.0], [8.0, 4.0]], [[2.0, 3.0], [7.0, 3.0]], [[6.0, 
3.0], [5.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 8.0], [3.0, 5.0]], [[3.0, 6.0], [8.0, 5.0]], [[6.0, 5.0], [6.0, 5.0]]], 
[[[5.0, 3.0], [6.0, 4.0]], [[6.0, 8.0], [6.0, 8.0]], [[1.0, 5.0], [6.0, 3.0]]], [[[5.0, 2.0], [6.0, 4.0]], [[7.0, 4.0], [1.0, 
6.0]], [[4.0, 6.0], [6.0, 1.0]]]])
      arg[:3,:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 6.0], [3.0, 5.0, 2.0], [6.0, 5.0, 5.0]], [[5.0, 1.0, 2.0], [3.0, 7.0, 2.0], [6.0, 
2.0, 7.0]], [[5.0, 1.0, 8.0], [7.0, 2.0, 6.0], [7.0, 2.0, 1.0]]], [[[5.0, 4.0, 4.0], [3.0, 5.0, 2.0], [7.0, 6.0, 5.0]], [[1.0, 
4.0, 5.0], [8.0, 4.0, 2.0], [2.0, 6.0, 5.0]], [[7.0, 5.0, 7.0], [6.0, 3.0, 6.0], [6.0, 6.0, 3.0]]], [[[7.0, 3.0, 2.0], [8.0, 
4.0, 1.0], [7.0, 7.0, 7.0]], [[2.0, 3.0, 1.0], [7.0, 3.0, 8.0], [7.0, 2.0, 1.0]], [[6.0, 3.0, 1.0], [5.0, 3.0, 3.0], [3.0, 6.0, 
8.0]]], [[[2.0, 4.0, 2.0], [1.0, 1.0, 5.0], [5.0, 4.0, 2.0]], [[1.0, 4.0, 6.0], [5.0, 8.0, 4.0], [6.0, 3.0, 2.0]], [[3.0, 7.0, 
4.0], [6.0, 3.0, 2.0], [5.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 8.0], [3.0, 5.0, 1.0], [4.0, 8.0, 2.0]], 
[[3.0, 6.0, 7.0], [8.0, 5.0, 5.0], [6.0, 5.0, 6.0]], [[6.0, 5.0, 1.0], [6.0, 5.0, 7.0], [1.0, 4.0, 2.0]]], [[[5.0, 3.0, 5.0], 
[6.0, 4.0, 4.0], [4.0, 4.0, 3.0]], [[6.0, 8.0, 4.0], [6.0, 8.0, 8.0], [5.0, 5.0, 8.0]], [[1.0, 5.0, 2.0], [6.0, 3.0, 8.0], 
[8.0, 1.0, 2.0]]], [[[5.0, 2.0, 2.0], [6.0, 4.0, 2.0], [6.0, 7.0, 2.0]], [[7.0, 4.0, 1.0], [1.0, 6.0, 7.0], [4.0, 1.0, 5.0]], 
[[4.0, 6.0, 6.0], [6.0, 1.0, 1.0], [7.0, 5.0, 3.0]]], [[[4.0, 6.0, 4.0], [1.0, 5.0, 2.0], [8.0, 2.0, 8.0]], [[7.0, 1.0, 1.0], 
[5.0, 5.0, 2.0], [8.0, 6.0, 5.0]], [[5.0, 5.0, 1.0], [1.0, 3.0, 8.0], [2.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PUPN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 6.0], [4.0, 8.0, 7.0], [6.0, 5.0, 6.0]], [[8.0, 1.0, 1.0], [5.0, 3.0, 5.0], [2.0, 3.0, 
7.0]], [[8.0, 5.0, 4.0], [1.0, 8.0, 8.0], [8.0, 6.0, 5.0]]], [[[5.0, 6.0, 7.0], [5.0, 8.0, 7.0], [5.0, 5.0, 4.0]], [[2.0, 6.0, 
3.0], [5.0, 3.0, 1.0], [7.0, 2.0, 6.0]], [[2.0, 5.0, 8.0], [7.0, 2.0, 1.0], [2.0, 6.0, 8.0]]], [[[5.0, 6.0, 1.0], [4.0, 3.0, 
2.0], [8.0, 5.0, 2.0]], [[1.0, 2.0, 8.0], [6.0, 4.0, 5.0], [4.0, 6.0, 8.0]], [[4.0, 5.0, 5.0], [4.0, 2.0, 6.0], [2.0, 1.0, 
5.0]]], [[[5.0, 3.0, 6.0], [6.0, 3.0, 2.0], [8.0, 2.0, 4.0]], [[5.0, 2.0, 8.0], [2.0, 7.0, 7.0], [1.0, 1.0, 2.0]], [[2.0, 2.0, 
1.0], [7.0, 2.0, 5.0], [5.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 8.0], [7.0, 3.0, 7.0], [4.0, 6.0, 7.0]], [[2.0, 1.0, 6.0], [2.0, 4.0, 
1.0], [6.0, 8.0, 3.0]], [[3.0, 7.0, 6.0], [8.0, 7.0, 5.0], [5.0, 5.0, 8.0]]], [[[3.0, 1.0, 7.0], [8.0, 5.0, 6.0], [1.0, 3.0, 
5.0]], [[6.0, 3.0, 5.0], [6.0, 7.0, 8.0], [8.0, 6.0, 7.0]], [[6.0, 6.0, 6.0], [4.0, 6.0, 1.0], [3.0, 6.0, 4.0]]], [[[2.0, 4.0, 
7.0], [7.0, 1.0, 7.0], [5.0, 7.0, 6.0]], [[2.0, 6.0, 1.0], [8.0, 2.0, 5.0], [1.0, 1.0, 3.0]], [[6.0, 5.0, 7.0], [2.0, 8.0, 
3.0], [7.0, 3.0, 3.0]]], [[[3.0, 8.0, 8.0], [5.0, 3.0, 3.0], [7.0, 7.0, 2.0]], [[8.0, 1.0, 2.0], [4.0, 2.0, 3.0], [2.0, 5.0, 
4.0]], [[6.0, 7.0, 3.0], [8.0, 5.0, 1.0], [3.0, 8.0, 4.0]]]]))
      val=numpy.array([[1.0, 3.0, 7.0], [1.0, 5.0, 4.0]])
      arg[2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 6.0], [4.0, 8.0, 7.0], [6.0, 5.0, 6.0]], [[8.0, 1.0, 1.0], [5.0, 3.0, 5.0], [2.0, 
3.0, 7.0]], [[8.0, 5.0, 4.0], [1.0, 8.0, 8.0], [8.0, 6.0, 5.0]]], [[[5.0, 6.0, 7.0], [5.0, 8.0, 7.0], [5.0, 5.0, 4.0]], [[2.0, 
6.0, 3.0], [5.0, 3.0, 1.0], [7.0, 2.0, 6.0]], [[2.0, 5.0, 8.0], [7.0, 2.0, 1.0], [2.0, 6.0, 8.0]]], [[[5.0, 6.0, 1.0], [1.0, 
3.0, 7.0], [8.0, 5.0, 2.0]], [[1.0, 2.0, 8.0], [1.0, 5.0, 4.0], [4.0, 6.0, 8.0]], [[4.0, 5.0, 5.0], [4.0, 2.0, 6.0], [2.0, 1.0, 
5.0]]], [[[5.0, 3.0, 6.0], [6.0, 3.0, 2.0], [8.0, 2.0, 4.0]], [[5.0, 2.0, 8.0], [2.0, 7.0, 7.0], [1.0, 1.0, 2.0]], [[2.0, 2.0, 
1.0], [7.0, 2.0, 5.0], [5.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 8.0], [7.0, 3.0, 7.0], [4.0, 6.0, 7.0]], 
[[2.0, 1.0, 6.0], [2.0, 4.0, 1.0], [6.0, 8.0, 3.0]], [[3.0, 7.0, 6.0], [8.0, 7.0, 5.0], [5.0, 5.0, 8.0]]], [[[3.0, 1.0, 7.0], 
[8.0, 5.0, 6.0], [1.0, 3.0, 5.0]], [[6.0, 3.0, 5.0], [6.0, 7.0, 8.0], [8.0, 6.0, 7.0]], [[6.0, 6.0, 6.0], [4.0, 6.0, 1.0], 
[3.0, 6.0, 4.0]]], [[[2.0, 4.0, 7.0], [1.0, 3.0, 7.0], [5.0, 7.0, 6.0]], [[2.0, 6.0, 1.0], [1.0, 5.0, 4.0], [1.0, 1.0, 3.0]], 
[[6.0, 5.0, 7.0], [2.0, 8.0, 3.0], [7.0, 3.0, 3.0]]], [[[3.0, 8.0, 8.0], [5.0, 3.0, 3.0], [7.0, 7.0, 2.0]], [[8.0, 1.0, 2.0], 
[4.0, 2.0, 3.0], [2.0, 5.0, 4.0]], [[6.0, 7.0, 3.0], [8.0, 5.0, 1.0], [3.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PUPN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 2.0], [6.0, 6.0, 1.0], [6.0, 6.0, 7.0]], [[6.0, 5.0, 3.0], [1.0, 3.0, 4.0], [1.0, 3.0, 
2.0]], [[7.0, 3.0, 3.0], [6.0, 6.0, 7.0], [1.0, 1.0, 5.0]]], [[[1.0, 1.0, 3.0], [4.0, 8.0, 1.0], [5.0, 3.0, 2.0]], [[1.0, 7.0, 
2.0], [8.0, 4.0, 7.0], [2.0, 6.0, 2.0]], [[2.0, 2.0, 6.0], [2.0, 1.0, 6.0], [7.0, 2.0, 8.0]]], [[[8.0, 4.0, 2.0], [5.0, 1.0, 
8.0], [2.0, 1.0, 8.0]], [[8.0, 3.0, 3.0], [3.0, 6.0, 3.0], [8.0, 2.0, 3.0]], [[6.0, 6.0, 3.0], [5.0, 5.0, 2.0], [7.0, 7.0, 
3.0]]], [[[2.0, 3.0, 1.0], [3.0, 1.0, 8.0], [5.0, 8.0, 2.0]], [[1.0, 7.0, 1.0], [4.0, 8.0, 6.0], [4.0, 5.0, 8.0]], [[7.0, 6.0, 
7.0], [4.0, 7.0, 3.0], [3.0, 4.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 8.0], [6.0, 6.0, 1.0], [6.0, 5.0, 2.0]], [[1.0, 7.0, 3.0], [8.0, 2.0, 
3.0], [5.0, 4.0, 8.0]], [[7.0, 1.0, 2.0], [5.0, 1.0, 4.0], [1.0, 8.0, 2.0]]], [[[4.0, 6.0, 3.0], [8.0, 2.0, 6.0], [1.0, 8.0, 
7.0]], [[8.0, 5.0, 6.0], [8.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[5.0, 6.0, 6.0], [3.0, 7.0, 7.0], [3.0, 1.0, 3.0]]], [[[2.0, 2.0, 
4.0], [3.0, 2.0, 5.0], [8.0, 5.0, 8.0]], [[4.0, 1.0, 5.0], [4.0, 1.0, 4.0], [1.0, 3.0, 8.0]], [[7.0, 4.0, 2.0], [4.0, 5.0, 
8.0], [8.0, 8.0, 2.0]]], [[[5.0, 1.0, 5.0], [3.0, 7.0, 6.0], [6.0, 7.0, 6.0]], [[2.0, 3.0, 2.0], [4.0, 8.0, 7.0], [1.0, 5.0, 
2.0]], [[2.0, 3.0, 6.0], [2.0, 5.0, 3.0], [7.0, 1.0, 3.0]]]]))
      val=Data(numpy.array([[7.0, 1.0, 4.0], [3.0, 3.0, 2.0]]),self.functionspace)
      arg[2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [6.0, 6.0, 1.0], [6.0, 6.0, 7.0]], [[6.0, 5.0, 3.0], [1.0, 3.0, 4.0], [1.0, 
3.0, 2.0]], [[7.0, 3.0, 3.0], [6.0, 6.0, 7.0], [1.0, 1.0, 5.0]]], [[[1.0, 1.0, 3.0], [4.0, 8.0, 1.0], [5.0, 3.0, 2.0]], [[1.0, 
7.0, 2.0], [8.0, 4.0, 7.0], [2.0, 6.0, 2.0]], [[2.0, 2.0, 6.0], [2.0, 1.0, 6.0], [7.0, 2.0, 8.0]]], [[[8.0, 4.0, 2.0], [7.0, 
1.0, 4.0], [2.0, 1.0, 8.0]], [[8.0, 3.0, 3.0], [3.0, 3.0, 2.0], [8.0, 2.0, 3.0]], [[6.0, 6.0, 3.0], [5.0, 5.0, 2.0], [7.0, 7.0, 
3.0]]], [[[2.0, 3.0, 1.0], [3.0, 1.0, 8.0], [5.0, 8.0, 2.0]], [[1.0, 7.0, 1.0], [4.0, 8.0, 6.0], [4.0, 5.0, 8.0]], [[7.0, 6.0, 
7.0], [4.0, 7.0, 3.0], [3.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 8.0], [6.0, 6.0, 1.0], [6.0, 5.0, 2.0]], 
[[1.0, 7.0, 3.0], [8.0, 2.0, 3.0], [5.0, 4.0, 8.0]], [[7.0, 1.0, 2.0], [5.0, 1.0, 4.0], [1.0, 8.0, 2.0]]], [[[4.0, 6.0, 3.0], 
[8.0, 2.0, 6.0], [1.0, 8.0, 7.0]], [[8.0, 5.0, 6.0], [8.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[5.0, 6.0, 6.0], [3.0, 7.0, 7.0], 
[3.0, 1.0, 3.0]]], [[[2.0, 2.0, 4.0], [7.0, 1.0, 4.0], [8.0, 5.0, 8.0]], [[4.0, 1.0, 5.0], [3.0, 3.0, 2.0], [1.0, 3.0, 8.0]], 
[[7.0, 4.0, 2.0], [4.0, 5.0, 8.0], [8.0, 8.0, 2.0]]], [[[5.0, 1.0, 5.0], [3.0, 7.0, 6.0], [6.0, 7.0, 6.0]], [[2.0, 3.0, 2.0], 
[4.0, 8.0, 7.0], [1.0, 5.0, 2.0]], [[2.0, 3.0, 6.0], [2.0, 5.0, 3.0], [7.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PUPN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 5.0], [7.0, 1.0, 6.0], [1.0, 5.0, 6.0]], [[6.0, 4.0, 8.0], [7.0, 4.0, 5.0], [7.0, 2.0, 
4.0]], [[7.0, 5.0, 6.0], [4.0, 6.0, 6.0], [8.0, 2.0, 5.0]]], [[[3.0, 7.0, 6.0], [4.0, 2.0, 5.0], [1.0, 7.0, 7.0]], [[8.0, 6.0, 
5.0], [3.0, 8.0, 8.0], [8.0, 5.0, 5.0]], [[6.0, 1.0, 8.0], [5.0, 5.0, 5.0], [6.0, 4.0, 5.0]]], [[[8.0, 6.0, 7.0], [5.0, 1.0, 
2.0], [4.0, 5.0, 8.0]], [[2.0, 1.0, 3.0], [5.0, 8.0, 5.0], [5.0, 7.0, 8.0]], [[5.0, 2.0, 1.0], [7.0, 6.0, 3.0], [4.0, 8.0, 
5.0]]], [[[4.0, 5.0, 5.0], [7.0, 6.0, 8.0], [2.0, 8.0, 8.0]], [[1.0, 3.0, 4.0], [8.0, 1.0, 1.0], [8.0, 1.0, 6.0]], [[8.0, 8.0, 
8.0], [5.0, 6.0, 1.0], [5.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 2.0], [8.0, 3.0, 7.0], [4.0, 3.0, 6.0]], [[6.0, 8.0, 4.0], [5.0, 3.0, 
8.0], [1.0, 5.0, 4.0]], [[3.0, 7.0, 5.0], [5.0, 1.0, 1.0], [5.0, 2.0, 4.0]]], [[[2.0, 7.0, 2.0], [5.0, 5.0, 4.0], [1.0, 6.0, 
4.0]], [[3.0, 7.0, 8.0], [1.0, 4.0, 3.0], [3.0, 3.0, 7.0]], [[7.0, 5.0, 1.0], [6.0, 6.0, 4.0], [7.0, 3.0, 3.0]]], [[[3.0, 4.0, 
6.0], [2.0, 4.0, 6.0], [3.0, 8.0, 3.0]], [[7.0, 8.0, 3.0], [7.0, 6.0, 5.0], [4.0, 5.0, 2.0]], [[6.0, 2.0, 1.0], [7.0, 6.0, 
7.0], [3.0, 2.0, 2.0]]], [[[4.0, 8.0, 4.0], [3.0, 7.0, 2.0], [7.0, 4.0, 3.0]], [[2.0, 5.0, 1.0], [7.0, 7.0, 7.0], [2.0, 4.0, 
8.0]], [[8.0, 8.0, 4.0], [2.0, 8.0, 7.0], [3.0, 4.0, 6.0]]]]))
      val=Data(numpy.array([[6.0, 5.0, 2.0], [1.0, 2.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 2.0, 6.0], [6.0, 1.0, 5.0]]))
      arg[2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 5.0], [7.0, 1.0, 6.0], [1.0, 5.0, 6.0]], [[6.0, 4.0, 8.0], [7.0, 4.0, 5.0], [7.0, 
2.0, 4.0]], [[7.0, 5.0, 6.0], [4.0, 6.0, 6.0], [8.0, 2.0, 5.0]]], [[[3.0, 7.0, 6.0], [4.0, 2.0, 5.0], [1.0, 7.0, 7.0]], [[8.0, 
6.0, 5.0], [3.0, 8.0, 8.0], [8.0, 5.0, 5.0]], [[6.0, 1.0, 8.0], [5.0, 5.0, 5.0], [6.0, 4.0, 5.0]]], [[[8.0, 6.0, 7.0], [6.0, 
5.0, 2.0], [4.0, 5.0, 8.0]], [[2.0, 1.0, 3.0], [1.0, 2.0, 4.0], [5.0, 7.0, 8.0]], [[5.0, 2.0, 1.0], [7.0, 6.0, 3.0], [4.0, 8.0, 
5.0]]], [[[4.0, 5.0, 5.0], [7.0, 6.0, 8.0], [2.0, 8.0, 8.0]], [[1.0, 3.0, 4.0], [8.0, 1.0, 1.0], [8.0, 1.0, 6.0]], [[8.0, 8.0, 
8.0], [5.0, 6.0, 1.0], [5.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 2.0], [8.0, 3.0, 7.0], [4.0, 3.0, 6.0]], 
[[6.0, 8.0, 4.0], [5.0, 3.0, 8.0], [1.0, 5.0, 4.0]], [[3.0, 7.0, 5.0], [5.0, 1.0, 1.0], [5.0, 2.0, 4.0]]], [[[2.0, 7.0, 2.0], 
[5.0, 5.0, 4.0], [1.0, 6.0, 4.0]], [[3.0, 7.0, 8.0], [1.0, 4.0, 3.0], [3.0, 3.0, 7.0]], [[7.0, 5.0, 1.0], [6.0, 6.0, 4.0], 
[7.0, 3.0, 3.0]]], [[[3.0, 4.0, 6.0], [3.0, 2.0, 6.0], [3.0, 8.0, 3.0]], [[7.0, 8.0, 3.0], [6.0, 1.0, 5.0], [4.0, 5.0, 2.0]], 
[[6.0, 2.0, 1.0], [7.0, 6.0, 7.0], [3.0, 2.0, 2.0]]], [[[4.0, 8.0, 4.0], [3.0, 7.0, 2.0], [7.0, 4.0, 3.0]], [[2.0, 5.0, 1.0], 
[7.0, 7.0, 7.0], [2.0, 4.0, 8.0]], [[8.0, 8.0, 4.0], [2.0, 8.0, 7.0], [3.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PUPN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 8.0], [4.0, 1.0, 2.0], [8.0, 2.0, 1.0]], [[4.0, 8.0, 8.0], [4.0, 8.0, 1.0], [5.0, 4.0, 
4.0]], [[8.0, 1.0, 7.0], [5.0, 1.0, 8.0], [2.0, 2.0, 4.0]]], [[[7.0, 2.0, 6.0], [2.0, 8.0, 6.0], [4.0, 1.0, 2.0]], [[4.0, 8.0, 
3.0], [2.0, 2.0, 1.0], [3.0, 5.0, 8.0]], [[8.0, 7.0, 5.0], [5.0, 2.0, 4.0], [1.0, 5.0, 7.0]]], [[[8.0, 7.0, 8.0], [4.0, 3.0, 
5.0], [6.0, 7.0, 1.0]], [[8.0, 3.0, 3.0], [4.0, 3.0, 8.0], [5.0, 7.0, 5.0]], [[7.0, 5.0, 8.0], [5.0, 8.0, 3.0], [7.0, 2.0, 
5.0]]], [[[4.0, 5.0, 1.0], [1.0, 3.0, 4.0], [3.0, 6.0, 6.0]], [[1.0, 7.0, 8.0], [1.0, 6.0, 6.0], [7.0, 6.0, 4.0]], [[1.0, 7.0, 
8.0], [8.0, 8.0, 5.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 3.0], [6.0, 4.0, 3.0], [7.0, 2.0, 8.0]], [[1.0, 8.0, 6.0], [1.0, 1.0, 
6.0], [6.0, 2.0, 3.0]], [[6.0, 6.0, 3.0], [3.0, 5.0, 8.0], [3.0, 1.0, 7.0]]], [[[6.0, 7.0, 7.0], [4.0, 2.0, 4.0], [5.0, 8.0, 
3.0]], [[5.0, 2.0, 8.0], [3.0, 4.0, 7.0], [1.0, 5.0, 1.0]], [[4.0, 8.0, 6.0], [4.0, 4.0, 7.0], [2.0, 2.0, 1.0]]], [[[4.0, 7.0, 
6.0], [2.0, 3.0, 8.0], [3.0, 3.0, 3.0]], [[5.0, 6.0, 2.0], [2.0, 7.0, 5.0], [1.0, 7.0, 8.0]], [[3.0, 3.0, 7.0], [2.0, 3.0, 
6.0], [2.0, 8.0, 8.0]]], [[[4.0, 5.0, 2.0], [4.0, 1.0, 4.0], [5.0, 5.0, 4.0]], [[7.0, 1.0, 2.0], [3.0, 2.0, 2.0], [3.0, 8.0, 
4.0]], [[6.0, 2.0, 1.0], [3.0, 6.0, 6.0], [6.0, 6.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[5.0, 3.0, 4.0], [1.0, 1.0, 6.0]])+(1.-msk_val)*numpy.array([[1.0, 3.0, 6.0], [6.0, 6.0, 4.0]])
      arg[2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 8.0], [4.0, 1.0, 2.0], [8.0, 2.0, 1.0]], [[4.0, 8.0, 8.0], [4.0, 8.0, 1.0], [5.0, 
4.0, 4.0]], [[8.0, 1.0, 7.0], [5.0, 1.0, 8.0], [2.0, 2.0, 4.0]]], [[[7.0, 2.0, 6.0], [2.0, 8.0, 6.0], [4.0, 1.0, 2.0]], [[4.0, 
8.0, 3.0], [2.0, 2.0, 1.0], [3.0, 5.0, 8.0]], [[8.0, 7.0, 5.0], [5.0, 2.0, 4.0], [1.0, 5.0, 7.0]]], [[[8.0, 7.0, 8.0], [5.0, 
3.0, 4.0], [6.0, 7.0, 1.0]], [[8.0, 3.0, 3.0], [1.0, 1.0, 6.0], [5.0, 7.0, 5.0]], [[7.0, 5.0, 8.0], [5.0, 8.0, 3.0], [7.0, 2.0, 
5.0]]], [[[4.0, 5.0, 1.0], [1.0, 3.0, 4.0], [3.0, 6.0, 6.0]], [[1.0, 7.0, 8.0], [1.0, 6.0, 6.0], [7.0, 6.0, 4.0]], [[1.0, 7.0, 
8.0], [8.0, 8.0, 5.0], [6.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 3.0], [6.0, 4.0, 3.0], [7.0, 2.0, 8.0]], 
[[1.0, 8.0, 6.0], [1.0, 1.0, 6.0], [6.0, 2.0, 3.0]], [[6.0, 6.0, 3.0], [3.0, 5.0, 8.0], [3.0, 1.0, 7.0]]], [[[6.0, 7.0, 7.0], 
[4.0, 2.0, 4.0], [5.0, 8.0, 3.0]], [[5.0, 2.0, 8.0], [3.0, 4.0, 7.0], [1.0, 5.0, 1.0]], [[4.0, 8.0, 6.0], [4.0, 4.0, 7.0], 
[2.0, 2.0, 1.0]]], [[[4.0, 7.0, 6.0], [1.0, 3.0, 6.0], [3.0, 3.0, 3.0]], [[5.0, 6.0, 2.0], [6.0, 6.0, 4.0], [1.0, 7.0, 8.0]], 
[[3.0, 3.0, 7.0], [2.0, 3.0, 6.0], [2.0, 8.0, 8.0]]], [[[4.0, 5.0, 2.0], [4.0, 1.0, 4.0], [5.0, 5.0, 4.0]], [[7.0, 1.0, 2.0], 
[3.0, 2.0, 2.0], [3.0, 8.0, 4.0]], [[6.0, 2.0, 1.0], [3.0, 6.0, 6.0], [6.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLUL(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 6.0], [1.0, 1.0, 8.0], [4.0, 4.0, 5.0]], [[3.0, 4.0, 7.0], [2.0, 5.0, 8.0], [1.0, 2.0, 
4.0]], [[2.0, 6.0, 3.0], [4.0, 4.0, 5.0], [1.0, 5.0, 4.0]]], [[[1.0, 2.0, 1.0], [1.0, 8.0, 7.0], [8.0, 2.0, 4.0]], [[7.0, 2.0, 
7.0], [5.0, 8.0, 7.0], [6.0, 6.0, 2.0]], [[6.0, 1.0, 5.0], [6.0, 5.0, 2.0], [7.0, 3.0, 7.0]]], [[[6.0, 4.0, 4.0], [5.0, 3.0, 
6.0], [2.0, 7.0, 1.0]], [[8.0, 1.0, 5.0], [5.0, 6.0, 6.0], [5.0, 5.0, 5.0]], [[6.0, 3.0, 1.0], [6.0, 4.0, 6.0], [5.0, 1.0, 
1.0]]], [[[3.0, 4.0, 1.0], [5.0, 7.0, 8.0], [4.0, 2.0, 6.0]], [[2.0, 3.0, 7.0], [7.0, 4.0, 2.0], [6.0, 4.0, 1.0]], [[7.0, 4.0, 
4.0], [8.0, 8.0, 2.0], [6.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 8.0], [5.0, 4.0, 2.0], [3.0, 3.0, 7.0]], [[3.0, 7.0, 1.0], [3.0, 6.0, 
6.0], [2.0, 6.0, 3.0]], [[5.0, 2.0, 4.0], [4.0, 5.0, 3.0], [1.0, 2.0, 3.0]]], [[[6.0, 1.0, 4.0], [4.0, 3.0, 4.0], [1.0, 6.0, 
8.0]], [[5.0, 3.0, 3.0], [5.0, 4.0, 8.0], [2.0, 7.0, 6.0]], [[2.0, 6.0, 4.0], [8.0, 5.0, 3.0], [5.0, 8.0, 2.0]]], [[[8.0, 8.0, 
6.0], [5.0, 8.0, 8.0], [4.0, 6.0, 7.0]], [[4.0, 5.0, 7.0], [5.0, 2.0, 5.0], [1.0, 2.0, 4.0]], [[8.0, 1.0, 4.0], [7.0, 8.0, 
4.0], [8.0, 3.0, 3.0]]], [[[4.0, 3.0, 2.0], [8.0, 4.0, 7.0], [7.0, 5.0, 1.0]], [[2.0, 3.0, 6.0], [3.0, 1.0, 2.0], [4.0, 6.0, 
6.0]], [[2.0, 1.0, 7.0], [4.0, 3.0, 3.0], [2.0, 7.0, 6.0]]]]))
      val=numpy.array([[[[7.0], [7.0]]], [[[1.0], [3.0]]]])
      arg[2:,2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [1.0, 1.0, 8.0], [4.0, 4.0, 5.0]], [[3.0, 4.0, 7.0], [2.0, 5.0, 8.0], [1.0, 
2.0, 4.0]], [[2.0, 6.0, 3.0], [4.0, 4.0, 5.0], [1.0, 5.0, 4.0]]], [[[1.0, 2.0, 1.0], [1.0, 8.0, 7.0], [8.0, 2.0, 4.0]], [[7.0, 
2.0, 7.0], [5.0, 8.0, 7.0], [6.0, 6.0, 2.0]], [[6.0, 1.0, 5.0], [6.0, 5.0, 2.0], [7.0, 3.0, 7.0]]], [[[6.0, 4.0, 4.0], [5.0, 
3.0, 6.0], [2.0, 7.0, 1.0]], [[8.0, 1.0, 5.0], [5.0, 6.0, 6.0], [5.0, 5.0, 5.0]], [[6.0, 3.0, 7.0], [6.0, 4.0, 7.0], [5.0, 1.0, 
1.0]]], [[[3.0, 4.0, 1.0], [5.0, 7.0, 8.0], [4.0, 2.0, 6.0]], [[2.0, 3.0, 7.0], [7.0, 4.0, 2.0], [6.0, 4.0, 1.0]], [[7.0, 4.0, 
1.0], [8.0, 8.0, 3.0], [6.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 8.0], [5.0, 4.0, 2.0], [3.0, 3.0, 7.0]], 
[[3.0, 7.0, 1.0], [3.0, 6.0, 6.0], [2.0, 6.0, 3.0]], [[5.0, 2.0, 4.0], [4.0, 5.0, 3.0], [1.0, 2.0, 3.0]]], [[[6.0, 1.0, 4.0], 
[4.0, 3.0, 4.0], [1.0, 6.0, 8.0]], [[5.0, 3.0, 3.0], [5.0, 4.0, 8.0], [2.0, 7.0, 6.0]], [[2.0, 6.0, 4.0], [8.0, 5.0, 3.0], 
[5.0, 8.0, 2.0]]], [[[8.0, 8.0, 6.0], [5.0, 8.0, 8.0], [4.0, 6.0, 7.0]], [[4.0, 5.0, 7.0], [5.0, 2.0, 5.0], [1.0, 2.0, 4.0]], 
[[8.0, 1.0, 7.0], [7.0, 8.0, 7.0], [8.0, 3.0, 3.0]]], [[[4.0, 3.0, 2.0], [8.0, 4.0, 7.0], [7.0, 5.0, 1.0]], [[2.0, 3.0, 6.0], 
[3.0, 1.0, 2.0], [4.0, 6.0, 6.0]], [[2.0, 1.0, 1.0], [4.0, 3.0, 3.0], [2.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLUL(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 1.0], [4.0, 4.0, 6.0], [2.0, 8.0, 2.0]], [[8.0, 6.0, 6.0], [6.0, 8.0, 5.0], [5.0, 1.0, 
2.0]], [[7.0, 3.0, 8.0], [5.0, 6.0, 3.0], [7.0, 2.0, 7.0]]], [[[5.0, 6.0, 1.0], [7.0, 7.0, 1.0], [1.0, 8.0, 7.0]], [[1.0, 5.0, 
1.0], [8.0, 1.0, 2.0], [8.0, 5.0, 3.0]], [[3.0, 5.0, 4.0], [3.0, 1.0, 1.0], [1.0, 5.0, 3.0]]], [[[8.0, 5.0, 3.0], [8.0, 4.0, 
4.0], [1.0, 1.0, 2.0]], [[7.0, 5.0, 2.0], [1.0, 5.0, 6.0], [4.0, 6.0, 3.0]], [[5.0, 1.0, 4.0], [4.0, 4.0, 5.0], [5.0, 4.0, 
6.0]]], [[[7.0, 7.0, 3.0], [8.0, 4.0, 7.0], [7.0, 4.0, 1.0]], [[1.0, 1.0, 5.0], [7.0, 3.0, 3.0], [6.0, 3.0, 7.0]], [[5.0, 6.0, 
1.0], [4.0, 8.0, 1.0], [5.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 7.0], [5.0, 7.0, 6.0], [3.0, 6.0, 8.0]], [[8.0, 3.0, 2.0], [5.0, 5.0, 
7.0], [7.0, 1.0, 2.0]], [[1.0, 8.0, 5.0], [4.0, 8.0, 5.0], [1.0, 2.0, 2.0]]], [[[3.0, 3.0, 3.0], [6.0, 4.0, 8.0], [3.0, 5.0, 
3.0]], [[8.0, 7.0, 4.0], [4.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[5.0, 8.0, 8.0], [7.0, 3.0, 3.0], [7.0, 2.0, 3.0]]], [[[7.0, 3.0, 
3.0], [8.0, 7.0, 7.0], [8.0, 6.0, 1.0]], [[6.0, 6.0, 8.0], [1.0, 1.0, 8.0], [7.0, 4.0, 7.0]], [[6.0, 5.0, 2.0], [2.0, 2.0, 
5.0], [6.0, 4.0, 5.0]]], [[[6.0, 3.0, 2.0], [2.0, 1.0, 4.0], [1.0, 5.0, 2.0]], [[7.0, 8.0, 1.0], [1.0, 3.0, 8.0], [8.0, 1.0, 
4.0]], [[2.0, 1.0, 6.0], [4.0, 8.0, 7.0], [2.0, 5.0, 6.0]]]]))
      val=Data(numpy.array([[[[8.0], [3.0]]], [[[1.0], [6.0]]]]),self.functionspace)
      arg[2:,2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 1.0], [4.0, 4.0, 6.0], [2.0, 8.0, 2.0]], [[8.0, 6.0, 6.0], [6.0, 8.0, 5.0], [5.0, 
1.0, 2.0]], [[7.0, 3.0, 8.0], [5.0, 6.0, 3.0], [7.0, 2.0, 7.0]]], [[[5.0, 6.0, 1.0], [7.0, 7.0, 1.0], [1.0, 8.0, 7.0]], [[1.0, 
5.0, 1.0], [8.0, 1.0, 2.0], [8.0, 5.0, 3.0]], [[3.0, 5.0, 4.0], [3.0, 1.0, 1.0], [1.0, 5.0, 3.0]]], [[[8.0, 5.0, 3.0], [8.0, 
4.0, 4.0], [1.0, 1.0, 2.0]], [[7.0, 5.0, 2.0], [1.0, 5.0, 6.0], [4.0, 6.0, 3.0]], [[5.0, 1.0, 8.0], [4.0, 4.0, 3.0], [5.0, 4.0, 
6.0]]], [[[7.0, 7.0, 3.0], [8.0, 4.0, 7.0], [7.0, 4.0, 1.0]], [[1.0, 1.0, 5.0], [7.0, 3.0, 3.0], [6.0, 3.0, 7.0]], [[5.0, 6.0, 
1.0], [4.0, 8.0, 6.0], [5.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 7.0], [5.0, 7.0, 6.0], [3.0, 6.0, 8.0]], 
[[8.0, 3.0, 2.0], [5.0, 5.0, 7.0], [7.0, 1.0, 2.0]], [[1.0, 8.0, 5.0], [4.0, 8.0, 5.0], [1.0, 2.0, 2.0]]], [[[3.0, 3.0, 3.0], 
[6.0, 4.0, 8.0], [3.0, 5.0, 3.0]], [[8.0, 7.0, 4.0], [4.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[5.0, 8.0, 8.0], [7.0, 3.0, 3.0], 
[7.0, 2.0, 3.0]]], [[[7.0, 3.0, 3.0], [8.0, 7.0, 7.0], [8.0, 6.0, 1.0]], [[6.0, 6.0, 8.0], [1.0, 1.0, 8.0], [7.0, 4.0, 7.0]], 
[[6.0, 5.0, 8.0], [2.0, 2.0, 3.0], [6.0, 4.0, 5.0]]], [[[6.0, 3.0, 2.0], [2.0, 1.0, 4.0], [1.0, 5.0, 2.0]], [[7.0, 8.0, 1.0], 
[1.0, 3.0, 8.0], [8.0, 1.0, 4.0]], [[2.0, 1.0, 1.0], [4.0, 8.0, 6.0], [2.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLUL(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 3.0], [8.0, 3.0, 4.0], [5.0, 3.0, 3.0]], [[8.0, 5.0, 5.0], [7.0, 8.0, 3.0], [2.0, 1.0, 
1.0]], [[6.0, 3.0, 1.0], [2.0, 2.0, 2.0], [1.0, 4.0, 8.0]]], [[[5.0, 8.0, 5.0], [7.0, 2.0, 7.0], [4.0, 1.0, 2.0]], [[1.0, 4.0, 
6.0], [7.0, 5.0, 7.0], [6.0, 2.0, 1.0]], [[1.0, 1.0, 2.0], [7.0, 6.0, 3.0], [1.0, 1.0, 2.0]]], [[[4.0, 8.0, 7.0], [5.0, 5.0, 
5.0], [5.0, 3.0, 4.0]], [[7.0, 7.0, 6.0], [8.0, 1.0, 6.0], [6.0, 2.0, 5.0]], [[2.0, 5.0, 1.0], [4.0, 4.0, 5.0], [4.0, 2.0, 
8.0]]], [[[3.0, 3.0, 6.0], [6.0, 3.0, 1.0], [4.0, 6.0, 1.0]], [[3.0, 3.0, 4.0], [7.0, 7.0, 1.0], [1.0, 7.0, 1.0]], [[8.0, 7.0, 
6.0], [1.0, 3.0, 3.0], [2.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 8.0], [7.0, 1.0, 8.0], [7.0, 4.0, 4.0]], [[5.0, 1.0, 6.0], [2.0, 4.0, 
3.0], [2.0, 6.0, 6.0]], [[6.0, 2.0, 2.0], [4.0, 8.0, 3.0], [4.0, 7.0, 4.0]]], [[[1.0, 3.0, 7.0], [5.0, 7.0, 3.0], [1.0, 5.0, 
2.0]], [[8.0, 8.0, 4.0], [4.0, 6.0, 2.0], [2.0, 7.0, 5.0]], [[8.0, 3.0, 7.0], [3.0, 1.0, 5.0], [6.0, 6.0, 2.0]]], [[[2.0, 1.0, 
6.0], [4.0, 2.0, 3.0], [2.0, 3.0, 1.0]], [[8.0, 1.0, 3.0], [8.0, 8.0, 7.0], [4.0, 7.0, 8.0]], [[6.0, 6.0, 8.0], [4.0, 3.0, 
6.0], [7.0, 7.0, 6.0]]], [[[3.0, 7.0, 4.0], [2.0, 6.0, 5.0], [6.0, 7.0, 7.0]], [[4.0, 4.0, 7.0], [1.0, 2.0, 6.0], [7.0, 3.0, 
6.0]], [[3.0, 1.0, 4.0], [2.0, 7.0, 4.0], [7.0, 6.0, 6.0]]]]))
      val=Data(numpy.array([[[[3.0], [2.0]]], [[[4.0], [5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0], [5.0]]], [[[8.0], [8.0]]]]))
      arg[2:,2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 3.0], [8.0, 3.0, 4.0], [5.0, 3.0, 3.0]], [[8.0, 5.0, 5.0], [7.0, 8.0, 3.0], [2.0, 
1.0, 1.0]], [[6.0, 3.0, 1.0], [2.0, 2.0, 2.0], [1.0, 4.0, 8.0]]], [[[5.0, 8.0, 5.0], [7.0, 2.0, 7.0], [4.0, 1.0, 2.0]], [[1.0, 
4.0, 6.0], [7.0, 5.0, 7.0], [6.0, 2.0, 1.0]], [[1.0, 1.0, 2.0], [7.0, 6.0, 3.0], [1.0, 1.0, 2.0]]], [[[4.0, 8.0, 7.0], [5.0, 
5.0, 5.0], [5.0, 3.0, 4.0]], [[7.0, 7.0, 6.0], [8.0, 1.0, 6.0], [6.0, 2.0, 5.0]], [[2.0, 5.0, 3.0], [4.0, 4.0, 2.0], [4.0, 2.0, 
8.0]]], [[[3.0, 3.0, 6.0], [6.0, 3.0, 1.0], [4.0, 6.0, 1.0]], [[3.0, 3.0, 4.0], [7.0, 7.0, 1.0], [1.0, 7.0, 1.0]], [[8.0, 7.0, 
4.0], [1.0, 3.0, 5.0], [2.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 8.0], [7.0, 1.0, 8.0], [7.0, 4.0, 4.0]], 
[[5.0, 1.0, 6.0], [2.0, 4.0, 3.0], [2.0, 6.0, 6.0]], [[6.0, 2.0, 2.0], [4.0, 8.0, 3.0], [4.0, 7.0, 4.0]]], [[[1.0, 3.0, 7.0], 
[5.0, 7.0, 3.0], [1.0, 5.0, 2.0]], [[8.0, 8.0, 4.0], [4.0, 6.0, 2.0], [2.0, 7.0, 5.0]], [[8.0, 3.0, 7.0], [3.0, 1.0, 5.0], 
[6.0, 6.0, 2.0]]], [[[2.0, 1.0, 6.0], [4.0, 2.0, 3.0], [2.0, 3.0, 1.0]], [[8.0, 1.0, 3.0], [8.0, 8.0, 7.0], [4.0, 7.0, 8.0]], 
[[6.0, 6.0, 2.0], [4.0, 3.0, 5.0], [7.0, 7.0, 6.0]]], [[[3.0, 7.0, 4.0], [2.0, 6.0, 5.0], [6.0, 7.0, 7.0]], [[4.0, 4.0, 7.0], 
[1.0, 2.0, 6.0], [7.0, 3.0, 6.0]], [[3.0, 1.0, 8.0], [2.0, 7.0, 8.0], [7.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLUL(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 8.0], [6.0, 2.0, 5.0], [4.0, 4.0, 6.0]], [[4.0, 3.0, 8.0], [1.0, 3.0, 2.0], [1.0, 8.0, 
7.0]], [[4.0, 2.0, 8.0], [5.0, 6.0, 3.0], [4.0, 1.0, 2.0]]], [[[4.0, 4.0, 5.0], [3.0, 7.0, 3.0], [5.0, 4.0, 1.0]], [[7.0, 6.0, 
2.0], [6.0, 4.0, 1.0], [2.0, 5.0, 6.0]], [[2.0, 2.0, 6.0], [4.0, 8.0, 6.0], [4.0, 7.0, 8.0]]], [[[2.0, 7.0, 3.0], [1.0, 4.0, 
4.0], [1.0, 2.0, 2.0]], [[5.0, 2.0, 2.0], [3.0, 2.0, 1.0], [1.0, 8.0, 5.0]], [[1.0, 8.0, 6.0], [1.0, 5.0, 2.0], [1.0, 6.0, 
3.0]]], [[[5.0, 2.0, 3.0], [1.0, 3.0, 3.0], [3.0, 2.0, 2.0]], [[5.0, 8.0, 6.0], [5.0, 5.0, 8.0], [7.0, 5.0, 2.0]], [[7.0, 5.0, 
5.0], [1.0, 3.0, 5.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 1.0], [3.0, 4.0, 6.0], [2.0, 1.0, 8.0]], [[3.0, 2.0, 7.0], [1.0, 1.0, 
5.0], [1.0, 2.0, 5.0]], [[1.0, 5.0, 5.0], [2.0, 8.0, 2.0], [3.0, 4.0, 4.0]]], [[[3.0, 1.0, 4.0], [4.0, 3.0, 6.0], [6.0, 5.0, 
4.0]], [[5.0, 6.0, 5.0], [6.0, 6.0, 6.0], [2.0, 5.0, 3.0]], [[3.0, 8.0, 1.0], [8.0, 4.0, 3.0], [2.0, 5.0, 1.0]]], [[[4.0, 7.0, 
7.0], [6.0, 7.0, 7.0], [1.0, 6.0, 2.0]], [[2.0, 1.0, 8.0], [7.0, 6.0, 4.0], [8.0, 1.0, 4.0]], [[3.0, 7.0, 5.0], [6.0, 2.0, 
4.0], [2.0, 5.0, 3.0]]], [[[4.0, 8.0, 8.0], [5.0, 6.0, 1.0], [1.0, 8.0, 5.0]], [[4.0, 8.0, 6.0], [3.0, 8.0, 1.0], [7.0, 8.0, 
2.0]], [[1.0, 3.0, 4.0], [1.0, 2.0, 7.0], [2.0, 7.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0], [5.0]]], [[[7.0], [8.0]]]])+(1.-msk_val)*numpy.array([[[[7.0], [4.0]]], [[[6.0], 
[6.0]]]])
      arg[2:,2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 8.0], [6.0, 2.0, 5.0], [4.0, 4.0, 6.0]], [[4.0, 3.0, 8.0], [1.0, 3.0, 2.0], [1.0, 
8.0, 7.0]], [[4.0, 2.0, 8.0], [5.0, 6.0, 3.0], [4.0, 1.0, 2.0]]], [[[4.0, 4.0, 5.0], [3.0, 7.0, 3.0], [5.0, 4.0, 1.0]], [[7.0, 
6.0, 2.0], [6.0, 4.0, 1.0], [2.0, 5.0, 6.0]], [[2.0, 2.0, 6.0], [4.0, 8.0, 6.0], [4.0, 7.0, 8.0]]], [[[2.0, 7.0, 3.0], [1.0, 
4.0, 4.0], [1.0, 2.0, 2.0]], [[5.0, 2.0, 2.0], [3.0, 2.0, 1.0], [1.0, 8.0, 5.0]], [[1.0, 8.0, 5.0], [1.0, 5.0, 5.0], [1.0, 6.0, 
3.0]]], [[[5.0, 2.0, 3.0], [1.0, 3.0, 3.0], [3.0, 2.0, 2.0]], [[5.0, 8.0, 6.0], [5.0, 5.0, 8.0], [7.0, 5.0, 2.0]], [[7.0, 5.0, 
7.0], [1.0, 3.0, 8.0], [3.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 1.0], [3.0, 4.0, 6.0], [2.0, 1.0, 8.0]], 
[[3.0, 2.0, 7.0], [1.0, 1.0, 5.0], [1.0, 2.0, 5.0]], [[1.0, 5.0, 5.0], [2.0, 8.0, 2.0], [3.0, 4.0, 4.0]]], [[[3.0, 1.0, 4.0], 
[4.0, 3.0, 6.0], [6.0, 5.0, 4.0]], [[5.0, 6.0, 5.0], [6.0, 6.0, 6.0], [2.0, 5.0, 3.0]], [[3.0, 8.0, 1.0], [8.0, 4.0, 3.0], 
[2.0, 5.0, 1.0]]], [[[4.0, 7.0, 7.0], [6.0, 7.0, 7.0], [1.0, 6.0, 2.0]], [[2.0, 1.0, 8.0], [7.0, 6.0, 4.0], [8.0, 1.0, 4.0]], 
[[3.0, 7.0, 7.0], [6.0, 2.0, 4.0], [2.0, 5.0, 3.0]]], [[[4.0, 8.0, 8.0], [5.0, 6.0, 1.0], [1.0, 8.0, 5.0]], [[4.0, 8.0, 6.0], 
[3.0, 8.0, 1.0], [7.0, 8.0, 2.0]], [[1.0, 3.0, 6.0], [1.0, 2.0, 6.0], [2.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UUUU(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 8.0], [8.0, 7.0, 7.0], [1.0, 2.0, 6.0]], [[1.0, 1.0, 2.0], [7.0, 7.0, 8.0], [6.0, 6.0, 
2.0]], [[7.0, 3.0, 4.0], [8.0, 4.0, 5.0], [7.0, 7.0, 4.0]]], [[[5.0, 4.0, 7.0], [8.0, 7.0, 7.0], [2.0, 3.0, 5.0]], [[7.0, 6.0, 
1.0], [8.0, 8.0, 5.0], [6.0, 7.0, 3.0]], [[2.0, 8.0, 5.0], [6.0, 1.0, 5.0], [3.0, 7.0, 2.0]]], [[[6.0, 8.0, 8.0], [2.0, 5.0, 
6.0], [6.0, 5.0, 7.0]], [[4.0, 4.0, 2.0], [2.0, 1.0, 8.0], [4.0, 6.0, 7.0]], [[4.0, 8.0, 4.0], [7.0, 3.0, 4.0], [6.0, 4.0, 
7.0]]], [[[3.0, 5.0, 3.0], [6.0, 4.0, 4.0], [3.0, 7.0, 8.0]], [[1.0, 7.0, 1.0], [3.0, 6.0, 3.0], [8.0, 2.0, 3.0]], [[6.0, 2.0, 
8.0], [1.0, 1.0, 8.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 3.0], [3.0, 8.0, 3.0], [8.0, 8.0, 4.0]], [[7.0, 6.0, 4.0], [2.0, 2.0, 
4.0], [4.0, 1.0, 5.0]], [[2.0, 5.0, 2.0], [4.0, 3.0, 3.0], [5.0, 2.0, 7.0]]], [[[2.0, 2.0, 1.0], [6.0, 6.0, 2.0], [1.0, 8.0, 
6.0]], [[3.0, 4.0, 8.0], [7.0, 7.0, 6.0], [1.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [7.0, 6.0, 7.0], [7.0, 5.0, 7.0]]], [[[5.0, 7.0, 
6.0], [3.0, 2.0, 4.0], [3.0, 2.0, 2.0]], [[1.0, 3.0, 4.0], [5.0, 6.0, 8.0], [8.0, 5.0, 1.0]], [[2.0, 6.0, 4.0], [4.0, 6.0, 
4.0], [7.0, 5.0, 2.0]]], [[[1.0, 7.0, 7.0], [6.0, 4.0, 5.0], [1.0, 8.0, 3.0]], [[7.0, 4.0, 8.0], [4.0, 7.0, 2.0], [1.0, 5.0, 
3.0]], [[7.0, 7.0, 2.0], [4.0, 6.0, 1.0], [7.0, 3.0, 3.0]]]]))
      val=numpy.array([[[[5.0, 2.0], [3.0, 7.0]], [[3.0, 3.0], [4.0, 8.0]]], [[[5.0, 2.0], [4.0, 7.0]], [[5.0, 4.0], [4.0, 
3.0]]], [[[4.0, 2.0], [8.0, 6.0]], [[5.0, 3.0], [4.0, 2.0]]]])
      arg[:3,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 8.0], [3.0, 7.0, 7.0], [1.0, 2.0, 6.0]], [[3.0, 3.0, 2.0], [4.0, 8.0, 8.0], [6.0, 
6.0, 2.0]], [[7.0, 3.0, 4.0], [8.0, 4.0, 5.0], [7.0, 7.0, 4.0]]], [[[5.0, 2.0, 7.0], [4.0, 7.0, 7.0], [2.0, 3.0, 5.0]], [[5.0, 
4.0, 1.0], [4.0, 3.0, 5.0], [6.0, 7.0, 3.0]], [[2.0, 8.0, 5.0], [6.0, 1.0, 5.0], [3.0, 7.0, 2.0]]], [[[4.0, 2.0, 8.0], [8.0, 
6.0, 6.0], [6.0, 5.0, 7.0]], [[5.0, 3.0, 2.0], [4.0, 2.0, 8.0], [4.0, 6.0, 7.0]], [[4.0, 8.0, 4.0], [7.0, 3.0, 4.0], [6.0, 4.0, 
7.0]]], [[[3.0, 5.0, 3.0], [6.0, 4.0, 4.0], [3.0, 7.0, 8.0]], [[1.0, 7.0, 1.0], [3.0, 6.0, 3.0], [8.0, 2.0, 3.0]], [[6.0, 2.0, 
8.0], [1.0, 1.0, 8.0], [8.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 3.0], [3.0, 7.0, 3.0], [8.0, 8.0, 4.0]], 
[[3.0, 3.0, 4.0], [4.0, 8.0, 4.0], [4.0, 1.0, 5.0]], [[2.0, 5.0, 2.0], [4.0, 3.0, 3.0], [5.0, 2.0, 7.0]]], [[[5.0, 2.0, 1.0], 
[4.0, 7.0, 2.0], [1.0, 8.0, 6.0]], [[5.0, 4.0, 8.0], [4.0, 3.0, 6.0], [1.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [7.0, 6.0, 7.0], 
[7.0, 5.0, 7.0]]], [[[4.0, 2.0, 6.0], [8.0, 6.0, 4.0], [3.0, 2.0, 2.0]], [[5.0, 3.0, 4.0], [4.0, 2.0, 8.0], [8.0, 5.0, 1.0]], 
[[2.0, 6.0, 4.0], [4.0, 6.0, 4.0], [7.0, 5.0, 2.0]]], [[[1.0, 7.0, 7.0], [6.0, 4.0, 5.0], [1.0, 8.0, 3.0]], [[7.0, 4.0, 8.0], 
[4.0, 7.0, 2.0], [1.0, 5.0, 3.0]], [[7.0, 7.0, 2.0], [4.0, 6.0, 1.0], [7.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UUUU(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 2.0], [6.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[6.0, 3.0, 1.0], [2.0, 4.0, 2.0], [3.0, 8.0, 
4.0]], [[4.0, 1.0, 6.0], [3.0, 8.0, 1.0], [6.0, 6.0, 2.0]]], [[[7.0, 7.0, 7.0], [3.0, 8.0, 2.0], [1.0, 5.0, 7.0]], [[7.0, 6.0, 
4.0], [6.0, 6.0, 8.0], [8.0, 5.0, 1.0]], [[4.0, 3.0, 3.0], [5.0, 1.0, 4.0], [2.0, 1.0, 2.0]]], [[[5.0, 3.0, 1.0], [3.0, 7.0, 
7.0], [5.0, 8.0, 2.0]], [[4.0, 8.0, 1.0], [1.0, 6.0, 8.0], [5.0, 6.0, 7.0]], [[6.0, 8.0, 5.0], [1.0, 2.0, 5.0], [2.0, 6.0, 
4.0]]], [[[3.0, 7.0, 8.0], [4.0, 4.0, 2.0], [7.0, 2.0, 8.0]], [[8.0, 3.0, 7.0], [5.0, 4.0, 6.0], [1.0, 6.0, 3.0]], [[1.0, 4.0, 
6.0], [7.0, 6.0, 3.0], [5.0, 3.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 1.0], [7.0, 6.0, 3.0], [3.0, 6.0, 6.0]], [[7.0, 2.0, 7.0], [1.0, 6.0, 
3.0], [5.0, 8.0, 2.0]], [[5.0, 5.0, 1.0], [8.0, 4.0, 1.0], [5.0, 8.0, 4.0]]], [[[2.0, 7.0, 6.0], [5.0, 5.0, 1.0], [3.0, 3.0, 
8.0]], [[8.0, 5.0, 8.0], [8.0, 3.0, 3.0], [4.0, 4.0, 5.0]], [[1.0, 3.0, 2.0], [1.0, 1.0, 3.0], [3.0, 2.0, 2.0]]], [[[2.0, 6.0, 
8.0], [1.0, 2.0, 5.0], [1.0, 2.0, 7.0]], [[2.0, 6.0, 6.0], [2.0, 4.0, 2.0], [5.0, 8.0, 7.0]], [[8.0, 6.0, 2.0], [1.0, 4.0, 
1.0], [7.0, 1.0, 7.0]]], [[[5.0, 6.0, 2.0], [1.0, 5.0, 8.0], [1.0, 8.0, 2.0]], [[5.0, 8.0, 8.0], [3.0, 8.0, 3.0], [2.0, 3.0, 
2.0]], [[2.0, 2.0, 8.0], [7.0, 1.0, 7.0], [4.0, 3.0, 7.0]]]]))
      val=Data(numpy.array([[[[6.0, 7.0], [1.0, 7.0]], [[1.0, 6.0], [8.0, 4.0]]], [[[3.0, 5.0], [6.0, 7.0]], [[3.0, 8.0], [6.0, 
2.0]]], [[[1.0, 6.0], [4.0, 2.0]], [[3.0, 8.0], [6.0, 6.0]]]]),self.functionspace)
      arg[:3,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 2.0], [1.0, 7.0, 4.0], [1.0, 3.0, 4.0]], [[1.0, 6.0, 1.0], [8.0, 4.0, 2.0], [3.0, 
8.0, 4.0]], [[4.0, 1.0, 6.0], [3.0, 8.0, 1.0], [6.0, 6.0, 2.0]]], [[[3.0, 5.0, 7.0], [6.0, 7.0, 2.0], [1.0, 5.0, 7.0]], [[3.0, 
8.0, 4.0], [6.0, 2.0, 8.0], [8.0, 5.0, 1.0]], [[4.0, 3.0, 3.0], [5.0, 1.0, 4.0], [2.0, 1.0, 2.0]]], [[[1.0, 6.0, 1.0], [4.0, 
2.0, 7.0], [5.0, 8.0, 2.0]], [[3.0, 8.0, 1.0], [6.0, 6.0, 8.0], [5.0, 6.0, 7.0]], [[6.0, 8.0, 5.0], [1.0, 2.0, 5.0], [2.0, 6.0, 
4.0]]], [[[3.0, 7.0, 8.0], [4.0, 4.0, 2.0], [7.0, 2.0, 8.0]], [[8.0, 3.0, 7.0], [5.0, 4.0, 6.0], [1.0, 6.0, 3.0]], [[1.0, 4.0, 
6.0], [7.0, 6.0, 3.0], [5.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 1.0], [1.0, 7.0, 3.0], [3.0, 6.0, 6.0]], 
[[1.0, 6.0, 7.0], [8.0, 4.0, 3.0], [5.0, 8.0, 2.0]], [[5.0, 5.0, 1.0], [8.0, 4.0, 1.0], [5.0, 8.0, 4.0]]], [[[3.0, 5.0, 6.0], 
[6.0, 7.0, 1.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 8.0], [6.0, 2.0, 3.0], [4.0, 4.0, 5.0]], [[1.0, 3.0, 2.0], [1.0, 1.0, 3.0], 
[3.0, 2.0, 2.0]]], [[[1.0, 6.0, 8.0], [4.0, 2.0, 5.0], [1.0, 2.0, 7.0]], [[3.0, 8.0, 6.0], [6.0, 6.0, 2.0], [5.0, 8.0, 7.0]], 
[[8.0, 6.0, 2.0], [1.0, 4.0, 1.0], [7.0, 1.0, 7.0]]], [[[5.0, 6.0, 2.0], [1.0, 5.0, 8.0], [1.0, 8.0, 2.0]], [[5.0, 8.0, 8.0], 
[3.0, 8.0, 3.0], [2.0, 3.0, 2.0]], [[2.0, 2.0, 8.0], [7.0, 1.0, 7.0], [4.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UUUU(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 8.0], [8.0, 2.0, 7.0], [3.0, 2.0, 2.0]], [[3.0, 7.0, 3.0], [7.0, 6.0, 2.0], [2.0, 3.0, 
7.0]], [[1.0, 2.0, 1.0], [7.0, 8.0, 1.0], [7.0, 7.0, 7.0]]], [[[1.0, 8.0, 7.0], [5.0, 6.0, 8.0], [2.0, 7.0, 6.0]], [[4.0, 1.0, 
8.0], [8.0, 3.0, 4.0], [6.0, 1.0, 1.0]], [[5.0, 5.0, 7.0], [5.0, 3.0, 3.0], [5.0, 6.0, 4.0]]], [[[8.0, 1.0, 8.0], [2.0, 2.0, 
2.0], [1.0, 7.0, 8.0]], [[4.0, 4.0, 8.0], [2.0, 5.0, 1.0], [8.0, 7.0, 1.0]], [[3.0, 6.0, 6.0], [8.0, 1.0, 6.0], [3.0, 8.0, 
8.0]]], [[[2.0, 2.0, 8.0], [2.0, 2.0, 8.0], [6.0, 5.0, 7.0]], [[2.0, 7.0, 8.0], [5.0, 4.0, 8.0], [1.0, 3.0, 4.0]], [[4.0, 2.0, 
8.0], [8.0, 2.0, 6.0], [6.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 5.0], [2.0, 3.0, 8.0], [3.0, 8.0, 8.0]], [[4.0, 5.0, 5.0], [7.0, 5.0, 
7.0], [7.0, 5.0, 7.0]], [[4.0, 1.0, 1.0], [4.0, 1.0, 2.0], [1.0, 3.0, 6.0]]], [[[5.0, 2.0, 1.0], [8.0, 5.0, 7.0], [3.0, 8.0, 
5.0]], [[5.0, 1.0, 2.0], [3.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[4.0, 5.0, 8.0], [8.0, 3.0, 6.0], [4.0, 6.0, 1.0]]], [[[4.0, 3.0, 
8.0], [8.0, 2.0, 2.0], [7.0, 6.0, 1.0]], [[6.0, 2.0, 7.0], [2.0, 3.0, 1.0], [8.0, 6.0, 2.0]], [[5.0, 4.0, 6.0], [1.0, 3.0, 
6.0], [3.0, 2.0, 5.0]]], [[[7.0, 3.0, 7.0], [2.0, 7.0, 1.0], [2.0, 1.0, 3.0]], [[6.0, 8.0, 5.0], [6.0, 8.0, 7.0], [7.0, 8.0, 
5.0]], [[7.0, 5.0, 2.0], [6.0, 4.0, 8.0], [4.0, 4.0, 3.0]]]]))
      val=Data(numpy.array([[[[5.0, 3.0], [8.0, 8.0]], [[1.0, 4.0], [4.0, 4.0]]], [[[1.0, 1.0], [3.0, 2.0]], [[3.0, 4.0], [2.0, 
7.0]]], [[[5.0, 1.0], [6.0, 1.0]], [[5.0, 6.0], [7.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 3.0], [1.0, 1.0]], [[8.0, 8.0], [5.0, 1.0]]], [[[6.0, 7.0], [5.0, 2.0]], [[7.0, 
8.0], [6.0, 6.0]]], [[[2.0, 6.0], [1.0, 6.0]], [[7.0, 6.0], [8.0, 3.0]]]]))
      arg[:3,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 8.0], [8.0, 8.0, 7.0], [3.0, 2.0, 2.0]], [[1.0, 4.0, 3.0], [4.0, 4.0, 2.0], [2.0, 
3.0, 7.0]], [[1.0, 2.0, 1.0], [7.0, 8.0, 1.0], [7.0, 7.0, 7.0]]], [[[1.0, 1.0, 7.0], [3.0, 2.0, 8.0], [2.0, 7.0, 6.0]], [[3.0, 
4.0, 8.0], [2.0, 7.0, 4.0], [6.0, 1.0, 1.0]], [[5.0, 5.0, 7.0], [5.0, 3.0, 3.0], [5.0, 6.0, 4.0]]], [[[5.0, 1.0, 8.0], [6.0, 
1.0, 2.0], [1.0, 7.0, 8.0]], [[5.0, 6.0, 8.0], [7.0, 4.0, 1.0], [8.0, 7.0, 1.0]], [[3.0, 6.0, 6.0], [8.0, 1.0, 6.0], [3.0, 8.0, 
8.0]]], [[[2.0, 2.0, 8.0], [2.0, 2.0, 8.0], [6.0, 5.0, 7.0]], [[2.0, 7.0, 8.0], [5.0, 4.0, 8.0], [1.0, 3.0, 4.0]], [[4.0, 2.0, 
8.0], [8.0, 2.0, 6.0], [6.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 5.0], [1.0, 1.0, 8.0], [3.0, 8.0, 8.0]], 
[[8.0, 8.0, 5.0], [5.0, 1.0, 7.0], [7.0, 5.0, 7.0]], [[4.0, 1.0, 1.0], [4.0, 1.0, 2.0], [1.0, 3.0, 6.0]]], [[[6.0, 7.0, 1.0], 
[5.0, 2.0, 7.0], [3.0, 8.0, 5.0]], [[7.0, 8.0, 2.0], [6.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[4.0, 5.0, 8.0], [8.0, 3.0, 6.0], 
[4.0, 6.0, 1.0]]], [[[2.0, 6.0, 8.0], [1.0, 6.0, 2.0], [7.0, 6.0, 1.0]], [[7.0, 6.0, 7.0], [8.0, 3.0, 1.0], [8.0, 6.0, 2.0]], 
[[5.0, 4.0, 6.0], [1.0, 3.0, 6.0], [3.0, 2.0, 5.0]]], [[[7.0, 3.0, 7.0], [2.0, 7.0, 1.0], [2.0, 1.0, 3.0]], [[6.0, 8.0, 5.0], 
[6.0, 8.0, 7.0], [7.0, 8.0, 5.0]], [[7.0, 5.0, 2.0], [6.0, 4.0, 8.0], [4.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UUUU(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 7.0], [5.0, 5.0, 7.0], [5.0, 4.0, 2.0]], [[5.0, 6.0, 5.0], [7.0, 4.0, 2.0], [5.0, 8.0, 
2.0]], [[1.0, 4.0, 8.0], [8.0, 5.0, 7.0], [1.0, 1.0, 2.0]]], [[[7.0, 4.0, 4.0], [6.0, 8.0, 1.0], [2.0, 2.0, 7.0]], [[3.0, 1.0, 
1.0], [1.0, 2.0, 8.0], [8.0, 4.0, 4.0]], [[8.0, 1.0, 3.0], [8.0, 4.0, 5.0], [1.0, 2.0, 6.0]]], [[[8.0, 4.0, 3.0], [5.0, 7.0, 
4.0], [2.0, 2.0, 7.0]], [[8.0, 3.0, 4.0], [8.0, 4.0, 5.0], [7.0, 3.0, 2.0]], [[6.0, 8.0, 8.0], [2.0, 1.0, 5.0], [1.0, 2.0, 
2.0]]], [[[3.0, 4.0, 8.0], [5.0, 6.0, 3.0], [8.0, 7.0, 8.0]], [[5.0, 1.0, 7.0], [4.0, 8.0, 1.0], [7.0, 6.0, 5.0]], [[6.0, 3.0, 
1.0], [1.0, 5.0, 7.0], [7.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 8.0], [1.0, 2.0, 8.0], [5.0, 8.0, 5.0]], [[6.0, 7.0, 8.0], [8.0, 1.0, 
3.0], [6.0, 8.0, 1.0]], [[2.0, 7.0, 5.0], [1.0, 2.0, 4.0], [8.0, 1.0, 5.0]]], [[[2.0, 3.0, 5.0], [3.0, 2.0, 8.0], [7.0, 3.0, 
4.0]], [[7.0, 4.0, 2.0], [8.0, 4.0, 8.0], [8.0, 1.0, 6.0]], [[2.0, 6.0, 2.0], [2.0, 2.0, 2.0], [5.0, 6.0, 7.0]]], [[[1.0, 1.0, 
7.0], [1.0, 8.0, 3.0], [5.0, 7.0, 7.0]], [[4.0, 2.0, 7.0], [7.0, 2.0, 2.0], [5.0, 5.0, 5.0]], [[4.0, 8.0, 4.0], [5.0, 8.0, 
4.0], [6.0, 2.0, 2.0]]], [[[1.0, 2.0, 5.0], [8.0, 7.0, 7.0], [5.0, 6.0, 4.0]], [[7.0, 6.0, 2.0], [2.0, 2.0, 1.0], [8.0, 4.0, 
4.0]], [[7.0, 5.0, 6.0], [2.0, 6.0, 3.0], [4.0, 2.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 1.0], [7.0, 2.0]], [[4.0, 8.0], [2.0, 3.0]]], [[[4.0, 6.0], [7.0, 6.0]], [[8.0, 4.0], 
[7.0, 1.0]]], [[[6.0, 7.0], [6.0, 3.0]], [[4.0, 3.0], [6.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 5.0], [3.0, 5.0]], 
[[8.0, 6.0], [2.0, 4.0]]], [[[3.0, 5.0], [4.0, 2.0]], [[2.0, 4.0], [1.0, 6.0]]], [[[8.0, 6.0], [6.0, 7.0]], [[4.0, 1.0], [2.0, 
8.0]]]])
      arg[:3,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 7.0], [7.0, 2.0, 7.0], [5.0, 4.0, 2.0]], [[4.0, 8.0, 5.0], [2.0, 3.0, 2.0], [5.0, 
8.0, 2.0]], [[1.0, 4.0, 8.0], [8.0, 5.0, 7.0], [1.0, 1.0, 2.0]]], [[[4.0, 6.0, 4.0], [7.0, 6.0, 1.0], [2.0, 2.0, 7.0]], [[8.0, 
4.0, 1.0], [7.0, 1.0, 8.0], [8.0, 4.0, 4.0]], [[8.0, 1.0, 3.0], [8.0, 4.0, 5.0], [1.0, 2.0, 6.0]]], [[[6.0, 7.0, 3.0], [6.0, 
3.0, 4.0], [2.0, 2.0, 7.0]], [[4.0, 3.0, 4.0], [6.0, 7.0, 5.0], [7.0, 3.0, 2.0]], [[6.0, 8.0, 8.0], [2.0, 1.0, 5.0], [1.0, 2.0, 
2.0]]], [[[3.0, 4.0, 8.0], [5.0, 6.0, 3.0], [8.0, 7.0, 8.0]], [[5.0, 1.0, 7.0], [4.0, 8.0, 1.0], [7.0, 6.0, 5.0]], [[6.0, 3.0, 
1.0], [1.0, 5.0, 7.0], [7.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 8.0], [3.0, 5.0, 8.0], [5.0, 8.0, 5.0]], 
[[8.0, 6.0, 8.0], [2.0, 4.0, 3.0], [6.0, 8.0, 1.0]], [[2.0, 7.0, 5.0], [1.0, 2.0, 4.0], [8.0, 1.0, 5.0]]], [[[3.0, 5.0, 5.0], 
[4.0, 2.0, 8.0], [7.0, 3.0, 4.0]], [[2.0, 4.0, 2.0], [1.0, 6.0, 8.0], [8.0, 1.0, 6.0]], [[2.0, 6.0, 2.0], [2.0, 2.0, 2.0], 
[5.0, 6.0, 7.0]]], [[[8.0, 6.0, 7.0], [6.0, 7.0, 3.0], [5.0, 7.0, 7.0]], [[4.0, 1.0, 7.0], [2.0, 8.0, 2.0], [5.0, 5.0, 5.0]], 
[[4.0, 8.0, 4.0], [5.0, 8.0, 4.0], [6.0, 2.0, 2.0]]], [[[1.0, 2.0, 5.0], [8.0, 7.0, 7.0], [5.0, 6.0, 4.0]], [[7.0, 6.0, 2.0], 
[2.0, 2.0, 1.0], [8.0, 4.0, 4.0]], [[7.0, 5.0, 6.0], [2.0, 6.0, 3.0], [4.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRRL(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 3.0], [8.0, 2.0, 5.0], [4.0, 4.0, 1.0]], [[8.0, 4.0, 7.0], [7.0, 4.0, 6.0], [3.0, 1.0, 
5.0]], [[3.0, 1.0, 5.0], [8.0, 4.0, 2.0], [8.0, 2.0, 7.0]]], [[[8.0, 5.0, 4.0], [3.0, 1.0, 3.0], [3.0, 1.0, 5.0]], [[6.0, 1.0, 
7.0], [7.0, 2.0, 1.0], [8.0, 7.0, 5.0]], [[1.0, 4.0, 8.0], [7.0, 5.0, 5.0], [2.0, 2.0, 4.0]]], [[[4.0, 5.0, 4.0], [1.0, 3.0, 
7.0], [7.0, 1.0, 3.0]], [[8.0, 2.0, 3.0], [6.0, 6.0, 1.0], [8.0, 4.0, 6.0]], [[3.0, 6.0, 3.0], [6.0, 4.0, 4.0], [3.0, 6.0, 
2.0]]], [[[6.0, 7.0, 2.0], [1.0, 6.0, 5.0], [7.0, 2.0, 2.0]], [[1.0, 8.0, 4.0], [2.0, 8.0, 3.0], [3.0, 3.0, 7.0]], [[4.0, 4.0, 
6.0], [4.0, 4.0, 8.0], [5.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 3.0], [3.0, 6.0, 6.0], [8.0, 6.0, 1.0]], [[8.0, 2.0, 6.0], [5.0, 6.0, 
2.0], [3.0, 1.0, 3.0]], [[5.0, 6.0, 3.0], [8.0, 4.0, 7.0], [1.0, 3.0, 3.0]]], [[[5.0, 1.0, 4.0], [5.0, 2.0, 6.0], [8.0, 6.0, 
4.0]], [[7.0, 5.0, 8.0], [3.0, 1.0, 2.0], [1.0, 3.0, 3.0]], [[1.0, 3.0, 1.0], [6.0, 8.0, 8.0], [3.0, 4.0, 3.0]]], [[[7.0, 5.0, 
4.0], [8.0, 8.0, 2.0], [6.0, 6.0, 2.0]], [[7.0, 1.0, 8.0], [5.0, 5.0, 6.0], [2.0, 3.0, 6.0]], [[2.0, 3.0, 2.0], [7.0, 6.0, 
2.0], [8.0, 8.0, 1.0]]], [[[6.0, 7.0, 1.0], [5.0, 2.0, 1.0], [1.0, 7.0, 3.0]], [[8.0, 1.0, 7.0], [8.0, 6.0, 6.0], [2.0, 1.0, 
3.0]], [[7.0, 7.0, 3.0], [7.0, 7.0, 3.0], [2.0, 8.0, 4.0]]]]))
      val=numpy.array([[[[4.0]]], [[[2.0]]]])
      arg[1:3,1:2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 3.0], [8.0, 2.0, 5.0], [4.0, 4.0, 1.0]], [[8.0, 4.0, 7.0], [7.0, 4.0, 6.0], [3.0, 
1.0, 5.0]], [[3.0, 1.0, 5.0], [8.0, 4.0, 2.0], [8.0, 2.0, 7.0]]], [[[8.0, 5.0, 4.0], [3.0, 1.0, 3.0], [3.0, 1.0, 5.0]], [[6.0, 
1.0, 7.0], [7.0, 2.0, 4.0], [8.0, 7.0, 5.0]], [[1.0, 4.0, 8.0], [7.0, 5.0, 5.0], [2.0, 2.0, 4.0]]], [[[4.0, 5.0, 4.0], [1.0, 
3.0, 7.0], [7.0, 1.0, 3.0]], [[8.0, 2.0, 3.0], [6.0, 6.0, 2.0], [8.0, 4.0, 6.0]], [[3.0, 6.0, 3.0], [6.0, 4.0, 4.0], [3.0, 6.0, 
2.0]]], [[[6.0, 7.0, 2.0], [1.0, 6.0, 5.0], [7.0, 2.0, 2.0]], [[1.0, 8.0, 4.0], [2.0, 8.0, 3.0], [3.0, 3.0, 7.0]], [[4.0, 4.0, 
6.0], [4.0, 4.0, 8.0], [5.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 3.0], [3.0, 6.0, 6.0], [8.0, 6.0, 1.0]], 
[[8.0, 2.0, 6.0], [5.0, 6.0, 2.0], [3.0, 1.0, 3.0]], [[5.0, 6.0, 3.0], [8.0, 4.0, 7.0], [1.0, 3.0, 3.0]]], [[[5.0, 1.0, 4.0], 
[5.0, 2.0, 6.0], [8.0, 6.0, 4.0]], [[7.0, 5.0, 8.0], [3.0, 1.0, 4.0], [1.0, 3.0, 3.0]], [[1.0, 3.0, 1.0], [6.0, 8.0, 8.0], 
[3.0, 4.0, 3.0]]], [[[7.0, 5.0, 4.0], [8.0, 8.0, 2.0], [6.0, 6.0, 2.0]], [[7.0, 1.0, 8.0], [5.0, 5.0, 2.0], [2.0, 3.0, 6.0]], 
[[2.0, 3.0, 2.0], [7.0, 6.0, 2.0], [8.0, 8.0, 1.0]]], [[[6.0, 7.0, 1.0], [5.0, 2.0, 1.0], [1.0, 7.0, 3.0]], [[8.0, 1.0, 7.0], 
[8.0, 6.0, 6.0], [2.0, 1.0, 3.0]], [[7.0, 7.0, 3.0], [7.0, 7.0, 3.0], [2.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRRL(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 8.0], [4.0, 6.0, 5.0], [5.0, 1.0, 3.0]], [[1.0, 3.0, 2.0], [4.0, 2.0, 1.0], [2.0, 6.0, 
2.0]], [[5.0, 5.0, 6.0], [8.0, 3.0, 2.0], [1.0, 4.0, 4.0]]], [[[4.0, 6.0, 3.0], [1.0, 2.0, 1.0], [1.0, 5.0, 1.0]], [[6.0, 8.0, 
7.0], [3.0, 6.0, 7.0], [2.0, 6.0, 8.0]], [[3.0, 6.0, 8.0], [4.0, 2.0, 8.0], [4.0, 6.0, 4.0]]], [[[7.0, 1.0, 2.0], [6.0, 5.0, 
4.0], [2.0, 7.0, 7.0]], [[3.0, 4.0, 4.0], [5.0, 1.0, 1.0], [2.0, 8.0, 8.0]], [[2.0, 4.0, 5.0], [6.0, 7.0, 4.0], [7.0, 1.0, 
3.0]]], [[[7.0, 5.0, 5.0], [6.0, 5.0, 5.0], [2.0, 2.0, 8.0]], [[7.0, 4.0, 8.0], [6.0, 1.0, 6.0], [3.0, 8.0, 2.0]], [[3.0, 3.0, 
8.0], [4.0, 1.0, 8.0], [3.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 3.0], [2.0, 6.0, 4.0], [3.0, 6.0, 6.0]], [[7.0, 6.0, 8.0], [3.0, 4.0, 
4.0], [4.0, 1.0, 7.0]], [[8.0, 4.0, 1.0], [8.0, 8.0, 4.0], [5.0, 2.0, 2.0]]], [[[6.0, 7.0, 7.0], [1.0, 1.0, 2.0], [8.0, 4.0, 
6.0]], [[2.0, 7.0, 8.0], [3.0, 5.0, 6.0], [6.0, 6.0, 5.0]], [[8.0, 1.0, 1.0], [3.0, 6.0, 8.0], [8.0, 2.0, 3.0]]], [[[6.0, 5.0, 
1.0], [7.0, 7.0, 5.0], [6.0, 5.0, 1.0]], [[1.0, 1.0, 2.0], [1.0, 2.0, 8.0], [1.0, 7.0, 8.0]], [[5.0, 3.0, 7.0], [8.0, 7.0, 
2.0], [2.0, 7.0, 1.0]]], [[[3.0, 8.0, 3.0], [5.0, 5.0, 8.0], [5.0, 4.0, 7.0]], [[3.0, 4.0, 6.0], [7.0, 6.0, 5.0], [8.0, 2.0, 
2.0]], [[1.0, 7.0, 2.0], [2.0, 5.0, 6.0], [3.0, 2.0, 4.0]]]]))
      val=Data(numpy.array([[[[8.0]]], [[[8.0]]]]),self.functionspace)
      arg[1:3,1:2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 8.0], [4.0, 6.0, 5.0], [5.0, 1.0, 3.0]], [[1.0, 3.0, 2.0], [4.0, 2.0, 1.0], [2.0, 
6.0, 2.0]], [[5.0, 5.0, 6.0], [8.0, 3.0, 2.0], [1.0, 4.0, 4.0]]], [[[4.0, 6.0, 3.0], [1.0, 2.0, 1.0], [1.0, 5.0, 1.0]], [[6.0, 
8.0, 7.0], [3.0, 6.0, 8.0], [2.0, 6.0, 8.0]], [[3.0, 6.0, 8.0], [4.0, 2.0, 8.0], [4.0, 6.0, 4.0]]], [[[7.0, 1.0, 2.0], [6.0, 
5.0, 4.0], [2.0, 7.0, 7.0]], [[3.0, 4.0, 4.0], [5.0, 1.0, 8.0], [2.0, 8.0, 8.0]], [[2.0, 4.0, 5.0], [6.0, 7.0, 4.0], [7.0, 1.0, 
3.0]]], [[[7.0, 5.0, 5.0], [6.0, 5.0, 5.0], [2.0, 2.0, 8.0]], [[7.0, 4.0, 8.0], [6.0, 1.0, 6.0], [3.0, 8.0, 2.0]], [[3.0, 3.0, 
8.0], [4.0, 1.0, 8.0], [3.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 3.0], [2.0, 6.0, 4.0], [3.0, 6.0, 6.0]], 
[[7.0, 6.0, 8.0], [3.0, 4.0, 4.0], [4.0, 1.0, 7.0]], [[8.0, 4.0, 1.0], [8.0, 8.0, 4.0], [5.0, 2.0, 2.0]]], [[[6.0, 7.0, 7.0], 
[1.0, 1.0, 2.0], [8.0, 4.0, 6.0]], [[2.0, 7.0, 8.0], [3.0, 5.0, 8.0], [6.0, 6.0, 5.0]], [[8.0, 1.0, 1.0], [3.0, 6.0, 8.0], 
[8.0, 2.0, 3.0]]], [[[6.0, 5.0, 1.0], [7.0, 7.0, 5.0], [6.0, 5.0, 1.0]], [[1.0, 1.0, 2.0], [1.0, 2.0, 8.0], [1.0, 7.0, 8.0]], 
[[5.0, 3.0, 7.0], [8.0, 7.0, 2.0], [2.0, 7.0, 1.0]]], [[[3.0, 8.0, 3.0], [5.0, 5.0, 8.0], [5.0, 4.0, 7.0]], [[3.0, 4.0, 6.0], 
[7.0, 6.0, 5.0], [8.0, 2.0, 2.0]], [[1.0, 7.0, 2.0], [2.0, 5.0, 6.0], [3.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRRL(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 6.0], [7.0, 6.0, 3.0], [1.0, 3.0, 8.0]], [[8.0, 4.0, 7.0], [8.0, 7.0, 6.0], [7.0, 2.0, 
3.0]], [[2.0, 1.0, 8.0], [1.0, 8.0, 6.0], [4.0, 7.0, 6.0]]], [[[6.0, 5.0, 3.0], [1.0, 3.0, 3.0], [7.0, 1.0, 7.0]], [[5.0, 7.0, 
8.0], [8.0, 3.0, 8.0], [8.0, 3.0, 3.0]], [[8.0, 3.0, 8.0], [4.0, 3.0, 7.0], [4.0, 7.0, 8.0]]], [[[6.0, 7.0, 7.0], [4.0, 6.0, 
4.0], [7.0, 1.0, 5.0]], [[1.0, 4.0, 3.0], [8.0, 8.0, 8.0], [1.0, 2.0, 5.0]], [[2.0, 5.0, 6.0], [8.0, 3.0, 5.0], [2.0, 1.0, 
6.0]]], [[[2.0, 1.0, 6.0], [5.0, 2.0, 1.0], [6.0, 6.0, 3.0]], [[3.0, 3.0, 4.0], [5.0, 8.0, 1.0], [4.0, 2.0, 6.0]], [[3.0, 3.0, 
5.0], [6.0, 1.0, 6.0], [5.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 8.0], [6.0, 7.0, 8.0], [5.0, 3.0, 8.0]], [[3.0, 2.0, 7.0], [1.0, 8.0, 
6.0], [7.0, 8.0, 5.0]], [[6.0, 4.0, 1.0], [6.0, 3.0, 5.0], [3.0, 6.0, 7.0]]], [[[3.0, 2.0, 2.0], [4.0, 6.0, 1.0], [1.0, 3.0, 
4.0]], [[1.0, 1.0, 6.0], [5.0, 7.0, 4.0], [3.0, 3.0, 3.0]], [[5.0, 2.0, 1.0], [2.0, 3.0, 6.0], [4.0, 7.0, 5.0]]], [[[2.0, 7.0, 
2.0], [2.0, 4.0, 8.0], [8.0, 1.0, 6.0]], [[5.0, 1.0, 4.0], [2.0, 4.0, 8.0], [8.0, 7.0, 6.0]], [[7.0, 3.0, 8.0], [1.0, 3.0, 
2.0], [5.0, 4.0, 4.0]]], [[[7.0, 7.0, 4.0], [8.0, 5.0, 7.0], [5.0, 6.0, 5.0]], [[7.0, 4.0, 1.0], [8.0, 5.0, 8.0], [7.0, 2.0, 
6.0]], [[1.0, 6.0, 2.0], [2.0, 7.0, 3.0], [5.0, 1.0, 4.0]]]]))
      val=Data(numpy.array([[[[2.0]]], [[[4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0]]], [[[3.0]]]]))
      arg[1:3,1:2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 6.0], [7.0, 6.0, 3.0], [1.0, 3.0, 8.0]], [[8.0, 4.0, 7.0], [8.0, 7.0, 6.0], [7.0, 
2.0, 3.0]], [[2.0, 1.0, 8.0], [1.0, 8.0, 6.0], [4.0, 7.0, 6.0]]], [[[6.0, 5.0, 3.0], [1.0, 3.0, 3.0], [7.0, 1.0, 7.0]], [[5.0, 
7.0, 8.0], [8.0, 3.0, 2.0], [8.0, 3.0, 3.0]], [[8.0, 3.0, 8.0], [4.0, 3.0, 7.0], [4.0, 7.0, 8.0]]], [[[6.0, 7.0, 7.0], [4.0, 
6.0, 4.0], [7.0, 1.0, 5.0]], [[1.0, 4.0, 3.0], [8.0, 8.0, 4.0], [1.0, 2.0, 5.0]], [[2.0, 5.0, 6.0], [8.0, 3.0, 5.0], [2.0, 1.0, 
6.0]]], [[[2.0, 1.0, 6.0], [5.0, 2.0, 1.0], [6.0, 6.0, 3.0]], [[3.0, 3.0, 4.0], [5.0, 8.0, 1.0], [4.0, 2.0, 6.0]], [[3.0, 3.0, 
5.0], [6.0, 1.0, 6.0], [5.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 8.0], [6.0, 7.0, 8.0], [5.0, 3.0, 8.0]], 
[[3.0, 2.0, 7.0], [1.0, 8.0, 6.0], [7.0, 8.0, 5.0]], [[6.0, 4.0, 1.0], [6.0, 3.0, 5.0], [3.0, 6.0, 7.0]]], [[[3.0, 2.0, 2.0], 
[4.0, 6.0, 1.0], [1.0, 3.0, 4.0]], [[1.0, 1.0, 6.0], [5.0, 7.0, 6.0], [3.0, 3.0, 3.0]], [[5.0, 2.0, 1.0], [2.0, 3.0, 6.0], 
[4.0, 7.0, 5.0]]], [[[2.0, 7.0, 2.0], [2.0, 4.0, 8.0], [8.0, 1.0, 6.0]], [[5.0, 1.0, 4.0], [2.0, 4.0, 3.0], [8.0, 7.0, 6.0]], 
[[7.0, 3.0, 8.0], [1.0, 3.0, 2.0], [5.0, 4.0, 4.0]]], [[[7.0, 7.0, 4.0], [8.0, 5.0, 7.0], [5.0, 6.0, 5.0]], [[7.0, 4.0, 1.0], 
[8.0, 5.0, 8.0], [7.0, 2.0, 6.0]], [[1.0, 6.0, 2.0], [2.0, 7.0, 3.0], [5.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRRL(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 4.0], [4.0, 3.0, 8.0], [7.0, 5.0, 1.0]], [[8.0, 5.0, 2.0], [7.0, 7.0, 4.0], [4.0, 5.0, 
5.0]], [[6.0, 7.0, 3.0], [2.0, 7.0, 4.0], [3.0, 8.0, 2.0]]], [[[8.0, 6.0, 7.0], [7.0, 7.0, 7.0], [7.0, 7.0, 8.0]], [[1.0, 5.0, 
6.0], [3.0, 3.0, 8.0], [3.0, 7.0, 6.0]], [[8.0, 4.0, 6.0], [4.0, 1.0, 8.0], [6.0, 1.0, 5.0]]], [[[5.0, 4.0, 2.0], [5.0, 6.0, 
8.0], [4.0, 1.0, 8.0]], [[5.0, 3.0, 4.0], [2.0, 1.0, 7.0], [1.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [6.0, 3.0, 8.0], [7.0, 6.0, 
8.0]]], [[[5.0, 6.0, 3.0], [1.0, 7.0, 8.0], [1.0, 6.0, 2.0]], [[4.0, 2.0, 2.0], [8.0, 5.0, 8.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 
6.0], [8.0, 3.0, 5.0], [3.0, 2.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 7.0], [3.0, 5.0, 7.0], [6.0, 3.0, 2.0]], [[1.0, 2.0, 3.0], [8.0, 6.0, 
2.0], [7.0, 8.0, 7.0]], [[6.0, 4.0, 4.0], [5.0, 8.0, 8.0], [4.0, 4.0, 8.0]]], [[[7.0, 5.0, 8.0], [4.0, 8.0, 1.0], [5.0, 8.0, 
5.0]], [[2.0, 2.0, 3.0], [1.0, 4.0, 7.0], [1.0, 1.0, 7.0]], [[7.0, 4.0, 2.0], [1.0, 1.0, 5.0], [6.0, 1.0, 5.0]]], [[[5.0, 3.0, 
6.0], [4.0, 6.0, 5.0], [3.0, 3.0, 4.0]], [[1.0, 7.0, 3.0], [4.0, 7.0, 8.0], [5.0, 6.0, 8.0]], [[6.0, 5.0, 6.0], [4.0, 8.0, 
7.0], [5.0, 4.0, 3.0]]], [[[6.0, 2.0, 3.0], [5.0, 3.0, 6.0], [4.0, 6.0, 2.0]], [[2.0, 3.0, 6.0], [3.0, 5.0, 3.0], [2.0, 3.0, 
1.0]], [[1.0, 6.0, 1.0], [5.0, 1.0, 8.0], [4.0, 2.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0]]], [[[8.0]]]])+(1.-msk_val)*numpy.array([[[[1.0]]], [[[2.0]]]])
      arg[1:3,1:2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 4.0], [4.0, 3.0, 8.0], [7.0, 5.0, 1.0]], [[8.0, 5.0, 2.0], [7.0, 7.0, 4.0], [4.0, 
5.0, 5.0]], [[6.0, 7.0, 3.0], [2.0, 7.0, 4.0], [3.0, 8.0, 2.0]]], [[[8.0, 6.0, 7.0], [7.0, 7.0, 7.0], [7.0, 7.0, 8.0]], [[1.0, 
5.0, 6.0], [3.0, 3.0, 7.0], [3.0, 7.0, 6.0]], [[8.0, 4.0, 6.0], [4.0, 1.0, 8.0], [6.0, 1.0, 5.0]]], [[[5.0, 4.0, 2.0], [5.0, 
6.0, 8.0], [4.0, 1.0, 8.0]], [[5.0, 3.0, 4.0], [2.0, 1.0, 8.0], [1.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [6.0, 3.0, 8.0], [7.0, 6.0, 
8.0]]], [[[5.0, 6.0, 3.0], [1.0, 7.0, 8.0], [1.0, 6.0, 2.0]], [[4.0, 2.0, 2.0], [8.0, 5.0, 8.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 
6.0], [8.0, 3.0, 5.0], [3.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 7.0], [3.0, 5.0, 7.0], [6.0, 3.0, 2.0]], 
[[1.0, 2.0, 3.0], [8.0, 6.0, 2.0], [7.0, 8.0, 7.0]], [[6.0, 4.0, 4.0], [5.0, 8.0, 8.0], [4.0, 4.0, 8.0]]], [[[7.0, 5.0, 8.0], 
[4.0, 8.0, 1.0], [5.0, 8.0, 5.0]], [[2.0, 2.0, 3.0], [1.0, 4.0, 1.0], [1.0, 1.0, 7.0]], [[7.0, 4.0, 2.0], [1.0, 1.0, 5.0], 
[6.0, 1.0, 5.0]]], [[[5.0, 3.0, 6.0], [4.0, 6.0, 5.0], [3.0, 3.0, 4.0]], [[1.0, 7.0, 3.0], [4.0, 7.0, 2.0], [5.0, 6.0, 8.0]], 
[[6.0, 5.0, 6.0], [4.0, 8.0, 7.0], [5.0, 4.0, 3.0]]], [[[6.0, 2.0, 3.0], [5.0, 3.0, 6.0], [4.0, 6.0, 2.0]], [[2.0, 3.0, 6.0], 
[3.0, 5.0, 3.0], [2.0, 3.0, 1.0]], [[1.0, 6.0, 1.0], [5.0, 1.0, 8.0], [4.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPPP(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 3.0], [3.0, 8.0, 6.0], [3.0, 1.0, 4.0]], [[7.0, 4.0, 6.0], [4.0, 2.0, 7.0], [5.0, 1.0, 
5.0]], [[2.0, 8.0, 7.0], [1.0, 7.0, 2.0], [1.0, 1.0, 2.0]]], [[[5.0, 3.0, 3.0], [5.0, 8.0, 7.0], [5.0, 6.0, 8.0]], [[4.0, 2.0, 
4.0], [5.0, 2.0, 3.0], [7.0, 5.0, 8.0]], [[5.0, 1.0, 2.0], [5.0, 4.0, 1.0], [8.0, 8.0, 7.0]]], [[[5.0, 1.0, 3.0], [3.0, 7.0, 
5.0], [5.0, 3.0, 8.0]], [[8.0, 6.0, 7.0], [3.0, 6.0, 2.0], [3.0, 3.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 8.0, 2.0], [5.0, 8.0, 
3.0]]], [[[6.0, 1.0, 5.0], [7.0, 6.0, 2.0], [5.0, 1.0, 8.0]], [[2.0, 4.0, 4.0], [4.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[7.0, 7.0, 
3.0], [6.0, 3.0, 5.0], [8.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 5.0], [3.0, 3.0, 1.0], [7.0, 5.0, 8.0]], [[6.0, 5.0, 7.0], [8.0, 8.0, 
5.0], [8.0, 3.0, 2.0]], [[7.0, 6.0, 2.0], [3.0, 4.0, 6.0], [3.0, 8.0, 2.0]]], [[[6.0, 2.0, 3.0], [1.0, 2.0, 2.0], [5.0, 4.0, 
5.0]], [[6.0, 1.0, 4.0], [6.0, 5.0, 5.0], [1.0, 1.0, 5.0]], [[5.0, 7.0, 2.0], [5.0, 4.0, 3.0], [2.0, 5.0, 3.0]]], [[[4.0, 1.0, 
3.0], [4.0, 4.0, 7.0], [2.0, 5.0, 5.0]], [[8.0, 1.0, 5.0], [5.0, 2.0, 1.0], [6.0, 5.0, 3.0]], [[7.0, 4.0, 4.0], [6.0, 8.0, 
4.0], [8.0, 4.0, 7.0]]], [[[8.0, 3.0, 5.0], [1.0, 5.0, 2.0], [5.0, 5.0, 6.0]], [[8.0, 2.0, 3.0], [6.0, 4.0, 3.0], [2.0, 5.0, 
2.0]], [[7.0, 5.0, 5.0], [8.0, 8.0, 8.0], [5.0, 1.0, 7.0]]]]))
      val=numpy.array(1.0)
      arg[2,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 3.0], [3.0, 8.0, 6.0], [3.0, 1.0, 4.0]], [[7.0, 4.0, 6.0], [4.0, 2.0, 7.0], [5.0, 
1.0, 5.0]], [[2.0, 8.0, 7.0], [1.0, 7.0, 2.0], [1.0, 1.0, 2.0]]], [[[5.0, 3.0, 3.0], [5.0, 8.0, 7.0], [5.0, 6.0, 8.0]], [[4.0, 
2.0, 4.0], [5.0, 2.0, 3.0], [7.0, 5.0, 8.0]], [[5.0, 1.0, 2.0], [5.0, 4.0, 1.0], [8.0, 8.0, 7.0]]], [[[5.0, 1.0, 3.0], [3.0, 
7.0, 5.0], [5.0, 3.0, 8.0]], [[8.0, 6.0, 7.0], [3.0, 1.0, 2.0], [3.0, 3.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 8.0, 2.0], [5.0, 8.0, 
3.0]]], [[[6.0, 1.0, 5.0], [7.0, 6.0, 2.0], [5.0, 1.0, 8.0]], [[2.0, 4.0, 4.0], [4.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[7.0, 7.0, 
3.0], [6.0, 3.0, 5.0], [8.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 5.0], [3.0, 3.0, 1.0], [7.0, 5.0, 8.0]], 
[[6.0, 5.0, 7.0], [8.0, 8.0, 5.0], [8.0, 3.0, 2.0]], [[7.0, 6.0, 2.0], [3.0, 4.0, 6.0], [3.0, 8.0, 2.0]]], [[[6.0, 2.0, 3.0], 
[1.0, 2.0, 2.0], [5.0, 4.0, 5.0]], [[6.0, 1.0, 4.0], [6.0, 5.0, 5.0], [1.0, 1.0, 5.0]], [[5.0, 7.0, 2.0], [5.0, 4.0, 3.0], 
[2.0, 5.0, 3.0]]], [[[4.0, 1.0, 3.0], [4.0, 4.0, 7.0], [2.0, 5.0, 5.0]], [[8.0, 1.0, 5.0], [5.0, 1.0, 1.0], [6.0, 5.0, 3.0]], 
[[7.0, 4.0, 4.0], [6.0, 8.0, 4.0], [8.0, 4.0, 7.0]]], [[[8.0, 3.0, 5.0], [1.0, 5.0, 2.0], [5.0, 5.0, 6.0]], [[8.0, 2.0, 3.0], 
[6.0, 4.0, 3.0], [2.0, 5.0, 2.0]], [[7.0, 5.0, 5.0], [8.0, 8.0, 8.0], [5.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPPP(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 8.0], [4.0, 1.0, 1.0], [6.0, 4.0, 1.0]], [[4.0, 1.0, 1.0], [8.0, 5.0, 5.0], [8.0, 8.0, 
1.0]], [[1.0, 7.0, 4.0], [2.0, 6.0, 7.0], [2.0, 8.0, 2.0]]], [[[6.0, 5.0, 7.0], [3.0, 2.0, 1.0], [3.0, 2.0, 6.0]], [[7.0, 2.0, 
6.0], [1.0, 5.0, 4.0], [2.0, 1.0, 2.0]], [[1.0, 7.0, 4.0], [2.0, 7.0, 2.0], [8.0, 5.0, 3.0]]], [[[8.0, 1.0, 6.0], [2.0, 4.0, 
6.0], [3.0, 7.0, 3.0]], [[3.0, 1.0, 1.0], [3.0, 7.0, 2.0], [5.0, 2.0, 7.0]], [[7.0, 2.0, 4.0], [5.0, 4.0, 8.0], [5.0, 7.0, 
2.0]]], [[[5.0, 7.0, 8.0], [7.0, 2.0, 4.0], [8.0, 1.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 4.0, 3.0], [1.0, 4.0, 8.0]], [[8.0, 3.0, 
7.0], [4.0, 2.0, 5.0], [1.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 2.0], [4.0, 8.0, 1.0], [3.0, 7.0, 7.0]], [[2.0, 3.0, 4.0], [5.0, 4.0, 
5.0], [7.0, 8.0, 7.0]], [[3.0, 2.0, 1.0], [5.0, 4.0, 8.0], [6.0, 8.0, 2.0]]], [[[1.0, 1.0, 4.0], [7.0, 5.0, 6.0], [8.0, 1.0, 
4.0]], [[2.0, 2.0, 1.0], [2.0, 2.0, 3.0], [6.0, 3.0, 8.0]], [[1.0, 1.0, 4.0], [7.0, 3.0, 1.0], [7.0, 6.0, 3.0]]], [[[7.0, 8.0, 
6.0], [4.0, 7.0, 4.0], [2.0, 8.0, 7.0]], [[1.0, 4.0, 3.0], [8.0, 7.0, 1.0], [6.0, 6.0, 6.0]], [[5.0, 6.0, 7.0], [6.0, 8.0, 
1.0], [1.0, 8.0, 6.0]]], [[[7.0, 1.0, 2.0], [1.0, 2.0, 4.0], [5.0, 7.0, 3.0]], [[8.0, 5.0, 8.0], [2.0, 7.0, 7.0], [3.0, 8.0, 
6.0]], [[2.0, 3.0, 8.0], [8.0, 5.0, 1.0], [6.0, 8.0, 4.0]]]]))
      val=Data(1.0,self.functionspace)
      arg[2,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 8.0], [4.0, 1.0, 1.0], [6.0, 4.0, 1.0]], [[4.0, 1.0, 1.0], [8.0, 5.0, 5.0], [8.0, 
8.0, 1.0]], [[1.0, 7.0, 4.0], [2.0, 6.0, 7.0], [2.0, 8.0, 2.0]]], [[[6.0, 5.0, 7.0], [3.0, 2.0, 1.0], [3.0, 2.0, 6.0]], [[7.0, 
2.0, 6.0], [1.0, 5.0, 4.0], [2.0, 1.0, 2.0]], [[1.0, 7.0, 4.0], [2.0, 7.0, 2.0], [8.0, 5.0, 3.0]]], [[[8.0, 1.0, 6.0], [2.0, 
4.0, 6.0], [3.0, 7.0, 3.0]], [[3.0, 1.0, 1.0], [3.0, 1.0, 2.0], [5.0, 2.0, 7.0]], [[7.0, 2.0, 4.0], [5.0, 4.0, 8.0], [5.0, 7.0, 
2.0]]], [[[5.0, 7.0, 8.0], [7.0, 2.0, 4.0], [8.0, 1.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 4.0, 3.0], [1.0, 4.0, 8.0]], [[8.0, 3.0, 
7.0], [4.0, 2.0, 5.0], [1.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 2.0], [4.0, 8.0, 1.0], [3.0, 7.0, 7.0]], 
[[2.0, 3.0, 4.0], [5.0, 4.0, 5.0], [7.0, 8.0, 7.0]], [[3.0, 2.0, 1.0], [5.0, 4.0, 8.0], [6.0, 8.0, 2.0]]], [[[1.0, 1.0, 4.0], 
[7.0, 5.0, 6.0], [8.0, 1.0, 4.0]], [[2.0, 2.0, 1.0], [2.0, 2.0, 3.0], [6.0, 3.0, 8.0]], [[1.0, 1.0, 4.0], [7.0, 3.0, 1.0], 
[7.0, 6.0, 3.0]]], [[[7.0, 8.0, 6.0], [4.0, 7.0, 4.0], [2.0, 8.0, 7.0]], [[1.0, 4.0, 3.0], [8.0, 1.0, 1.0], [6.0, 6.0, 6.0]], 
[[5.0, 6.0, 7.0], [6.0, 8.0, 1.0], [1.0, 8.0, 6.0]]], [[[7.0, 1.0, 2.0], [1.0, 2.0, 4.0], [5.0, 7.0, 3.0]], [[8.0, 5.0, 8.0], 
[2.0, 7.0, 7.0], [3.0, 8.0, 6.0]], [[2.0, 3.0, 8.0], [8.0, 5.0, 1.0], [6.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPPP(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 3.0], [5.0, 4.0, 3.0], [8.0, 7.0, 5.0]], [[4.0, 6.0, 2.0], [6.0, 2.0, 6.0], [7.0, 3.0, 
3.0]], [[2.0, 3.0, 6.0], [4.0, 7.0, 7.0], [2.0, 3.0, 7.0]]], [[[4.0, 2.0, 6.0], [2.0, 6.0, 3.0], [6.0, 1.0, 8.0]], [[3.0, 4.0, 
8.0], [6.0, 8.0, 6.0], [5.0, 5.0, 1.0]], [[3.0, 8.0, 2.0], [7.0, 5.0, 1.0], [8.0, 6.0, 4.0]]], [[[1.0, 6.0, 4.0], [6.0, 5.0, 
2.0], [4.0, 3.0, 2.0]], [[1.0, 2.0, 8.0], [5.0, 2.0, 5.0], [8.0, 7.0, 1.0]], [[3.0, 5.0, 5.0], [3.0, 6.0, 2.0], [4.0, 4.0, 
8.0]]], [[[8.0, 2.0, 4.0], [5.0, 6.0, 2.0], [5.0, 8.0, 7.0]], [[6.0, 8.0, 8.0], [7.0, 8.0, 4.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 
5.0], [6.0, 6.0, 3.0], [7.0, 3.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 2.0], [1.0, 2.0, 2.0], [6.0, 1.0, 7.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 
5.0], [1.0, 1.0, 4.0]], [[4.0, 5.0, 6.0], [6.0, 3.0, 2.0], [7.0, 2.0, 7.0]]], [[[1.0, 7.0, 5.0], [7.0, 2.0, 7.0], [6.0, 4.0, 
1.0]], [[5.0, 4.0, 6.0], [5.0, 1.0, 1.0], [7.0, 3.0, 8.0]], [[5.0, 8.0, 3.0], [6.0, 1.0, 3.0], [7.0, 4.0, 4.0]]], [[[6.0, 5.0, 
3.0], [6.0, 7.0, 7.0], [2.0, 4.0, 8.0]], [[2.0, 7.0, 3.0], [7.0, 5.0, 2.0], [7.0, 1.0, 1.0]], [[5.0, 6.0, 1.0], [6.0, 7.0, 
6.0], [4.0, 2.0, 2.0]]], [[[2.0, 7.0, 3.0], [7.0, 2.0, 5.0], [3.0, 5.0, 8.0]], [[4.0, 1.0, 5.0], [7.0, 8.0, 6.0], [8.0, 2.0, 
7.0]], [[3.0, 7.0, 4.0], [3.0, 6.0, 8.0], [1.0, 4.0, 8.0]]]]))
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[2,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 3.0], [5.0, 4.0, 3.0], [8.0, 7.0, 5.0]], [[4.0, 6.0, 2.0], [6.0, 2.0, 6.0], [7.0, 
3.0, 3.0]], [[2.0, 3.0, 6.0], [4.0, 7.0, 7.0], [2.0, 3.0, 7.0]]], [[[4.0, 2.0, 6.0], [2.0, 6.0, 3.0], [6.0, 1.0, 8.0]], [[3.0, 
4.0, 8.0], [6.0, 8.0, 6.0], [5.0, 5.0, 1.0]], [[3.0, 8.0, 2.0], [7.0, 5.0, 1.0], [8.0, 6.0, 4.0]]], [[[1.0, 6.0, 4.0], [6.0, 
5.0, 2.0], [4.0, 3.0, 2.0]], [[1.0, 2.0, 8.0], [5.0, 1.0, 5.0], [8.0, 7.0, 1.0]], [[3.0, 5.0, 5.0], [3.0, 6.0, 2.0], [4.0, 4.0, 
8.0]]], [[[8.0, 2.0, 4.0], [5.0, 6.0, 2.0], [5.0, 8.0, 7.0]], [[6.0, 8.0, 8.0], [7.0, 8.0, 4.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 
5.0], [6.0, 6.0, 3.0], [7.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 2.0], [1.0, 2.0, 2.0], [6.0, 1.0, 7.0]], 
[[3.0, 8.0, 8.0], [4.0, 6.0, 5.0], [1.0, 1.0, 4.0]], [[4.0, 5.0, 6.0], [6.0, 3.0, 2.0], [7.0, 2.0, 7.0]]], [[[1.0, 7.0, 5.0], 
[7.0, 2.0, 7.0], [6.0, 4.0, 1.0]], [[5.0, 4.0, 6.0], [5.0, 1.0, 1.0], [7.0, 3.0, 8.0]], [[5.0, 8.0, 3.0], [6.0, 1.0, 3.0], 
[7.0, 4.0, 4.0]]], [[[6.0, 5.0, 3.0], [6.0, 7.0, 7.0], [2.0, 4.0, 8.0]], [[2.0, 7.0, 3.0], [7.0, 1.0, 2.0], [7.0, 1.0, 1.0]], 
[[5.0, 6.0, 1.0], [6.0, 7.0, 6.0], [4.0, 2.0, 2.0]]], [[[2.0, 7.0, 3.0], [7.0, 2.0, 5.0], [3.0, 5.0, 8.0]], [[4.0, 1.0, 5.0], 
[7.0, 8.0, 6.0], [8.0, 2.0, 7.0]], [[3.0, 7.0, 4.0], [3.0, 6.0, 8.0], [1.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPPP(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 6.0], [1.0, 4.0, 6.0], [3.0, 1.0, 6.0]], [[6.0, 3.0, 8.0], [5.0, 2.0, 5.0], [3.0, 1.0, 
8.0]], [[7.0, 5.0, 8.0], [7.0, 6.0, 6.0], [1.0, 1.0, 7.0]]], [[[2.0, 4.0, 5.0], [7.0, 3.0, 7.0], [8.0, 8.0, 5.0]], [[4.0, 4.0, 
8.0], [5.0, 1.0, 5.0], [8.0, 6.0, 7.0]], [[4.0, 3.0, 1.0], [5.0, 1.0, 7.0], [8.0, 5.0, 7.0]]], [[[2.0, 4.0, 4.0], [3.0, 3.0, 
6.0], [1.0, 8.0, 6.0]], [[7.0, 8.0, 1.0], [4.0, 6.0, 2.0], [2.0, 6.0, 8.0]], [[5.0, 1.0, 3.0], [3.0, 7.0, 5.0], [4.0, 3.0, 
1.0]]], [[[3.0, 2.0, 3.0], [6.0, 1.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 8.0, 4.0], [3.0, 7.0, 6.0], [5.0, 2.0, 6.0]], [[2.0, 5.0, 
8.0], [5.0, 2.0, 7.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 2.0], [5.0, 3.0, 4.0], [3.0, 7.0, 4.0]], [[8.0, 4.0, 7.0], [8.0, 1.0, 
3.0], [7.0, 2.0, 4.0]], [[8.0, 5.0, 3.0], [1.0, 3.0, 2.0], [8.0, 6.0, 2.0]]], [[[7.0, 6.0, 2.0], [2.0, 2.0, 5.0], [3.0, 1.0, 
4.0]], [[4.0, 3.0, 6.0], [1.0, 5.0, 8.0], [3.0, 8.0, 1.0]], [[2.0, 1.0, 6.0], [3.0, 1.0, 8.0], [8.0, 5.0, 3.0]]], [[[2.0, 6.0, 
8.0], [5.0, 3.0, 7.0], [4.0, 8.0, 7.0]], [[7.0, 4.0, 2.0], [3.0, 5.0, 7.0], [7.0, 8.0, 6.0]], [[5.0, 3.0, 7.0], [6.0, 6.0, 
5.0], [4.0, 8.0, 3.0]]], [[[1.0, 3.0, 6.0], [1.0, 1.0, 1.0], [3.0, 5.0, 8.0]], [[8.0, 7.0, 2.0], [7.0, 7.0, 6.0], [8.0, 2.0, 
1.0]], [[7.0, 6.0, 2.0], [6.0, 4.0, 6.0], [7.0, 3.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=(1.-msk_val)*(1.0)+msk_val*(1.0)
      arg[2,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 6.0], [1.0, 4.0, 6.0], [3.0, 1.0, 6.0]], [[6.0, 3.0, 8.0], [5.0, 2.0, 5.0], [3.0, 
1.0, 8.0]], [[7.0, 5.0, 8.0], [7.0, 6.0, 6.0], [1.0, 1.0, 7.0]]], [[[2.0, 4.0, 5.0], [7.0, 3.0, 7.0], [8.0, 8.0, 5.0]], [[4.0, 
4.0, 8.0], [5.0, 1.0, 5.0], [8.0, 6.0, 7.0]], [[4.0, 3.0, 1.0], [5.0, 1.0, 7.0], [8.0, 5.0, 7.0]]], [[[2.0, 4.0, 4.0], [3.0, 
3.0, 6.0], [1.0, 8.0, 6.0]], [[7.0, 8.0, 1.0], [4.0, 1.0, 2.0], [2.0, 6.0, 8.0]], [[5.0, 1.0, 3.0], [3.0, 7.0, 5.0], [4.0, 3.0, 
1.0]]], [[[3.0, 2.0, 3.0], [6.0, 1.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 8.0, 4.0], [3.0, 7.0, 6.0], [5.0, 2.0, 6.0]], [[2.0, 5.0, 
8.0], [5.0, 2.0, 7.0], [7.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 2.0], [5.0, 3.0, 4.0], [3.0, 7.0, 4.0]], 
[[8.0, 4.0, 7.0], [8.0, 1.0, 3.0], [7.0, 2.0, 4.0]], [[8.0, 5.0, 3.0], [1.0, 3.0, 2.0], [8.0, 6.0, 2.0]]], [[[7.0, 6.0, 2.0], 
[2.0, 2.0, 5.0], [3.0, 1.0, 4.0]], [[4.0, 3.0, 6.0], [1.0, 5.0, 8.0], [3.0, 8.0, 1.0]], [[2.0, 1.0, 6.0], [3.0, 1.0, 8.0], 
[8.0, 5.0, 3.0]]], [[[2.0, 6.0, 8.0], [5.0, 3.0, 7.0], [4.0, 8.0, 7.0]], [[7.0, 4.0, 2.0], [3.0, 1.0, 7.0], [7.0, 8.0, 6.0]], 
[[5.0, 3.0, 7.0], [6.0, 6.0, 5.0], [4.0, 8.0, 3.0]]], [[[1.0, 3.0, 6.0], [1.0, 1.0, 1.0], [3.0, 5.0, 8.0]], [[8.0, 7.0, 2.0], 
[7.0, 7.0, 6.0], [8.0, 2.0, 1.0]], [[7.0, 6.0, 2.0], [6.0, 4.0, 6.0], [7.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPLN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 3.0], [6.0, 8.0, 5.0], [7.0, 2.0, 1.0]], [[6.0, 1.0, 3.0], [6.0, 8.0, 6.0], [1.0, 4.0, 
1.0]], [[1.0, 3.0, 7.0], [6.0, 4.0, 8.0], [3.0, 8.0, 1.0]]], [[[2.0, 2.0, 5.0], [7.0, 6.0, 6.0], [5.0, 7.0, 4.0]], [[2.0, 2.0, 
5.0], [6.0, 4.0, 2.0], [3.0, 2.0, 4.0]], [[5.0, 2.0, 4.0], [4.0, 7.0, 4.0], [1.0, 8.0, 2.0]]], [[[4.0, 1.0, 3.0], [7.0, 1.0, 
2.0], [7.0, 1.0, 4.0]], [[6.0, 7.0, 7.0], [8.0, 1.0, 6.0], [4.0, 6.0, 8.0]], [[5.0, 7.0, 8.0], [5.0, 8.0, 5.0], [8.0, 2.0, 
8.0]]], [[[1.0, 2.0, 5.0], [7.0, 4.0, 5.0], [3.0, 8.0, 8.0]], [[6.0, 1.0, 2.0], [3.0, 6.0, 2.0], [5.0, 5.0, 1.0]], [[7.0, 2.0, 
3.0], [2.0, 6.0, 8.0], [6.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 8.0], [7.0, 7.0, 3.0], [4.0, 4.0, 1.0]], [[3.0, 7.0, 5.0], [6.0, 1.0, 
4.0], [6.0, 4.0, 6.0]], [[6.0, 2.0, 7.0], [2.0, 4.0, 2.0], [7.0, 4.0, 5.0]]], [[[2.0, 8.0, 6.0], [4.0, 1.0, 4.0], [7.0, 8.0, 
4.0]], [[3.0, 4.0, 1.0], [3.0, 2.0, 1.0], [1.0, 8.0, 3.0]], [[3.0, 4.0, 2.0], [8.0, 7.0, 4.0], [8.0, 5.0, 1.0]]], [[[6.0, 5.0, 
2.0], [2.0, 2.0, 8.0], [1.0, 2.0, 1.0]], [[8.0, 3.0, 4.0], [1.0, 7.0, 4.0], [5.0, 3.0, 2.0]], [[4.0, 5.0, 4.0], [4.0, 3.0, 
3.0], [8.0, 8.0, 4.0]]], [[[1.0, 2.0, 2.0], [8.0, 7.0, 7.0], [6.0, 4.0, 6.0]], [[3.0, 4.0, 5.0], [3.0, 5.0, 6.0], [5.0, 1.0, 
5.0]], [[7.0, 5.0, 1.0], [5.0, 4.0, 1.0], [1.0, 7.0, 2.0]]]]))
      val=numpy.array([[[5.0, 3.0, 6.0]], [[2.0, 7.0, 7.0]], [[3.0, 7.0, 5.0]], [[5.0, 8.0, 6.0]]])
      arg[:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [6.0, 8.0, 5.0], [7.0, 2.0, 1.0]], [[6.0, 1.0, 3.0], [6.0, 8.0, 6.0], [5.0, 
3.0, 6.0]], [[1.0, 3.0, 7.0], [6.0, 4.0, 8.0], [3.0, 8.0, 1.0]]], [[[2.0, 2.0, 5.0], [7.0, 6.0, 6.0], [5.0, 7.0, 4.0]], [[2.0, 
2.0, 5.0], [6.0, 4.0, 2.0], [2.0, 7.0, 7.0]], [[5.0, 2.0, 4.0], [4.0, 7.0, 4.0], [1.0, 8.0, 2.0]]], [[[4.0, 1.0, 3.0], [7.0, 
1.0, 2.0], [7.0, 1.0, 4.0]], [[6.0, 7.0, 7.0], [8.0, 1.0, 6.0], [3.0, 7.0, 5.0]], [[5.0, 7.0, 8.0], [5.0, 8.0, 5.0], [8.0, 2.0, 
8.0]]], [[[1.0, 2.0, 5.0], [7.0, 4.0, 5.0], [3.0, 8.0, 8.0]], [[6.0, 1.0, 2.0], [3.0, 6.0, 2.0], [5.0, 8.0, 6.0]], [[7.0, 2.0, 
3.0], [2.0, 6.0, 8.0], [6.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 8.0], [7.0, 7.0, 3.0], [4.0, 4.0, 1.0]], 
[[3.0, 7.0, 5.0], [6.0, 1.0, 4.0], [5.0, 3.0, 6.0]], [[6.0, 2.0, 7.0], [2.0, 4.0, 2.0], [7.0, 4.0, 5.0]]], [[[2.0, 8.0, 6.0], 
[4.0, 1.0, 4.0], [7.0, 8.0, 4.0]], [[3.0, 4.0, 1.0], [3.0, 2.0, 1.0], [2.0, 7.0, 7.0]], [[3.0, 4.0, 2.0], [8.0, 7.0, 4.0], 
[8.0, 5.0, 1.0]]], [[[6.0, 5.0, 2.0], [2.0, 2.0, 8.0], [1.0, 2.0, 1.0]], [[8.0, 3.0, 4.0], [1.0, 7.0, 4.0], [3.0, 7.0, 5.0]], 
[[4.0, 5.0, 4.0], [4.0, 3.0, 3.0], [8.0, 8.0, 4.0]]], [[[1.0, 2.0, 2.0], [8.0, 7.0, 7.0], [6.0, 4.0, 6.0]], [[3.0, 4.0, 5.0], 
[3.0, 5.0, 6.0], [5.0, 8.0, 6.0]], [[7.0, 5.0, 1.0], [5.0, 4.0, 1.0], [1.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPLN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 2.0], [4.0, 5.0, 5.0], [1.0, 7.0, 4.0]], [[3.0, 3.0, 5.0], [7.0, 2.0, 1.0], [6.0, 4.0, 
6.0]], [[4.0, 4.0, 8.0], [7.0, 5.0, 8.0], [2.0, 6.0, 7.0]]], [[[3.0, 2.0, 6.0], [4.0, 1.0, 8.0], [3.0, 3.0, 5.0]], [[6.0, 8.0, 
2.0], [2.0, 6.0, 8.0], [8.0, 2.0, 3.0]], [[1.0, 5.0, 6.0], [7.0, 2.0, 7.0], [1.0, 3.0, 1.0]]], [[[3.0, 7.0, 1.0], [4.0, 4.0, 
7.0], [2.0, 5.0, 6.0]], [[2.0, 4.0, 4.0], [6.0, 5.0, 6.0], [2.0, 7.0, 7.0]], [[3.0, 6.0, 6.0], [5.0, 6.0, 2.0], [2.0, 8.0, 
1.0]]], [[[3.0, 8.0, 3.0], [1.0, 5.0, 2.0], [8.0, 5.0, 3.0]], [[2.0, 7.0, 4.0], [8.0, 8.0, 8.0], [5.0, 8.0, 1.0]], [[7.0, 4.0, 
1.0], [5.0, 7.0, 7.0], [3.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 1.0], [1.0, 6.0, 6.0], [7.0, 8.0, 4.0]], [[8.0, 5.0, 4.0], [1.0, 6.0, 
8.0], [3.0, 1.0, 8.0]], [[4.0, 1.0, 8.0], [1.0, 8.0, 1.0], [3.0, 7.0, 1.0]]], [[[3.0, 7.0, 1.0], [2.0, 1.0, 4.0], [6.0, 5.0, 
8.0]], [[4.0, 2.0, 2.0], [8.0, 2.0, 5.0], [2.0, 5.0, 2.0]], [[7.0, 8.0, 4.0], [7.0, 8.0, 5.0], [3.0, 1.0, 5.0]]], [[[8.0, 6.0, 
2.0], [4.0, 1.0, 1.0], [4.0, 4.0, 3.0]], [[1.0, 7.0, 6.0], [6.0, 6.0, 4.0], [7.0, 6.0, 4.0]], [[8.0, 7.0, 1.0], [1.0, 7.0, 
4.0], [2.0, 5.0, 7.0]]], [[[3.0, 1.0, 2.0], [4.0, 5.0, 5.0], [3.0, 5.0, 8.0]], [[6.0, 6.0, 8.0], [4.0, 3.0, 6.0], [6.0, 8.0, 
6.0]], [[4.0, 4.0, 3.0], [3.0, 3.0, 8.0], [4.0, 1.0, 1.0]]]]))
      val=Data(numpy.array([[[2.0, 2.0, 5.0]], [[3.0, 5.0, 7.0]], [[4.0, 8.0, 3.0]], [[7.0, 6.0, 3.0]]]),self.functionspace)
      arg[:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 2.0], [4.0, 5.0, 5.0], [1.0, 7.0, 4.0]], [[3.0, 3.0, 5.0], [7.0, 2.0, 1.0], [2.0, 
2.0, 5.0]], [[4.0, 4.0, 8.0], [7.0, 5.0, 8.0], [2.0, 6.0, 7.0]]], [[[3.0, 2.0, 6.0], [4.0, 1.0, 8.0], [3.0, 3.0, 5.0]], [[6.0, 
8.0, 2.0], [2.0, 6.0, 8.0], [3.0, 5.0, 7.0]], [[1.0, 5.0, 6.0], [7.0, 2.0, 7.0], [1.0, 3.0, 1.0]]], [[[3.0, 7.0, 1.0], [4.0, 
4.0, 7.0], [2.0, 5.0, 6.0]], [[2.0, 4.0, 4.0], [6.0, 5.0, 6.0], [4.0, 8.0, 3.0]], [[3.0, 6.0, 6.0], [5.0, 6.0, 2.0], [2.0, 8.0, 
1.0]]], [[[3.0, 8.0, 3.0], [1.0, 5.0, 2.0], [8.0, 5.0, 3.0]], [[2.0, 7.0, 4.0], [8.0, 8.0, 8.0], [7.0, 6.0, 3.0]], [[7.0, 4.0, 
1.0], [5.0, 7.0, 7.0], [3.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 1.0], [1.0, 6.0, 6.0], [7.0, 8.0, 4.0]], 
[[8.0, 5.0, 4.0], [1.0, 6.0, 8.0], [2.0, 2.0, 5.0]], [[4.0, 1.0, 8.0], [1.0, 8.0, 1.0], [3.0, 7.0, 1.0]]], [[[3.0, 7.0, 1.0], 
[2.0, 1.0, 4.0], [6.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [8.0, 2.0, 5.0], [3.0, 5.0, 7.0]], [[7.0, 8.0, 4.0], [7.0, 8.0, 5.0], 
[3.0, 1.0, 5.0]]], [[[8.0, 6.0, 2.0], [4.0, 1.0, 1.0], [4.0, 4.0, 3.0]], [[1.0, 7.0, 6.0], [6.0, 6.0, 4.0], [4.0, 8.0, 3.0]], 
[[8.0, 7.0, 1.0], [1.0, 7.0, 4.0], [2.0, 5.0, 7.0]]], [[[3.0, 1.0, 2.0], [4.0, 5.0, 5.0], [3.0, 5.0, 8.0]], [[6.0, 6.0, 8.0], 
[4.0, 3.0, 6.0], [7.0, 6.0, 3.0]], [[4.0, 4.0, 3.0], [3.0, 3.0, 8.0], [4.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPLN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 7.0], [2.0, 6.0, 5.0], [3.0, 5.0, 5.0]], [[7.0, 4.0, 7.0], [8.0, 5.0, 5.0], [8.0, 7.0, 
6.0]], [[3.0, 3.0, 7.0], [7.0, 3.0, 6.0], [3.0, 4.0, 1.0]]], [[[3.0, 1.0, 8.0], [3.0, 2.0, 6.0], [2.0, 4.0, 7.0]], [[1.0, 1.0, 
5.0], [5.0, 1.0, 7.0], [7.0, 4.0, 6.0]], [[6.0, 1.0, 4.0], [3.0, 3.0, 5.0], [7.0, 1.0, 7.0]]], [[[1.0, 3.0, 5.0], [3.0, 2.0, 
2.0], [5.0, 3.0, 6.0]], [[3.0, 3.0, 5.0], [6.0, 3.0, 3.0], [4.0, 4.0, 5.0]], [[3.0, 5.0, 3.0], [3.0, 6.0, 8.0], [7.0, 2.0, 
6.0]]], [[[4.0, 4.0, 3.0], [7.0, 7.0, 2.0], [6.0, 6.0, 2.0]], [[7.0, 3.0, 2.0], [3.0, 3.0, 2.0], [6.0, 4.0, 4.0]], [[3.0, 3.0, 
8.0], [4.0, 1.0, 6.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 6.0], [5.0, 4.0, 8.0], [6.0, 5.0, 5.0]], [[5.0, 1.0, 8.0], [8.0, 2.0, 
3.0], [3.0, 8.0, 4.0]], [[4.0, 6.0, 4.0], [3.0, 4.0, 8.0], [1.0, 2.0, 5.0]]], [[[2.0, 8.0, 6.0], [6.0, 5.0, 4.0], [6.0, 7.0, 
5.0]], [[1.0, 6.0, 2.0], [6.0, 8.0, 5.0], [2.0, 3.0, 2.0]], [[2.0, 5.0, 5.0], [2.0, 7.0, 5.0], [6.0, 4.0, 2.0]]], [[[4.0, 1.0, 
2.0], [7.0, 7.0, 7.0], [3.0, 2.0, 6.0]], [[7.0, 6.0, 3.0], [4.0, 2.0, 3.0], [8.0, 2.0, 2.0]], [[2.0, 2.0, 4.0], [1.0, 1.0, 
5.0], [8.0, 5.0, 4.0]]], [[[6.0, 8.0, 7.0], [4.0, 4.0, 5.0], [5.0, 1.0, 5.0]], [[5.0, 6.0, 4.0], [6.0, 2.0, 2.0], [1.0, 7.0, 
2.0]], [[2.0, 6.0, 6.0], [2.0, 4.0, 1.0], [3.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[[7.0, 7.0, 6.0]], [[3.0, 1.0, 7.0]], [[8.0, 3.0, 4.0]], [[3.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 4.0, 1.0]], [[4.0, 3.0, 2.0]], [[1.0, 2.0, 3.0]], [[3.0, 4.0, 1.0]]]))
      arg[:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 7.0], [2.0, 6.0, 5.0], [3.0, 5.0, 5.0]], [[7.0, 4.0, 7.0], [8.0, 5.0, 5.0], [7.0, 
7.0, 6.0]], [[3.0, 3.0, 7.0], [7.0, 3.0, 6.0], [3.0, 4.0, 1.0]]], [[[3.0, 1.0, 8.0], [3.0, 2.0, 6.0], [2.0, 4.0, 7.0]], [[1.0, 
1.0, 5.0], [5.0, 1.0, 7.0], [3.0, 1.0, 7.0]], [[6.0, 1.0, 4.0], [3.0, 3.0, 5.0], [7.0, 1.0, 7.0]]], [[[1.0, 3.0, 5.0], [3.0, 
2.0, 2.0], [5.0, 3.0, 6.0]], [[3.0, 3.0, 5.0], [6.0, 3.0, 3.0], [8.0, 3.0, 4.0]], [[3.0, 5.0, 3.0], [3.0, 6.0, 8.0], [7.0, 2.0, 
6.0]]], [[[4.0, 4.0, 3.0], [7.0, 7.0, 2.0], [6.0, 6.0, 2.0]], [[7.0, 3.0, 2.0], [3.0, 3.0, 2.0], [3.0, 3.0, 1.0]], [[3.0, 3.0, 
8.0], [4.0, 1.0, 6.0], [6.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 6.0], [5.0, 4.0, 8.0], [6.0, 5.0, 5.0]], 
[[5.0, 1.0, 8.0], [8.0, 2.0, 3.0], [4.0, 4.0, 1.0]], [[4.0, 6.0, 4.0], [3.0, 4.0, 8.0], [1.0, 2.0, 5.0]]], [[[2.0, 8.0, 6.0], 
[6.0, 5.0, 4.0], [6.0, 7.0, 5.0]], [[1.0, 6.0, 2.0], [6.0, 8.0, 5.0], [4.0, 3.0, 2.0]], [[2.0, 5.0, 5.0], [2.0, 7.0, 5.0], 
[6.0, 4.0, 2.0]]], [[[4.0, 1.0, 2.0], [7.0, 7.0, 7.0], [3.0, 2.0, 6.0]], [[7.0, 6.0, 3.0], [4.0, 2.0, 3.0], [1.0, 2.0, 3.0]], 
[[2.0, 2.0, 4.0], [1.0, 1.0, 5.0], [8.0, 5.0, 4.0]]], [[[6.0, 8.0, 7.0], [4.0, 4.0, 5.0], [5.0, 1.0, 5.0]], [[5.0, 6.0, 4.0], 
[6.0, 2.0, 2.0], [3.0, 4.0, 1.0]], [[2.0, 6.0, 6.0], [2.0, 4.0, 1.0], [3.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPLN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 8.0], [7.0, 3.0, 4.0], [8.0, 5.0, 6.0]], [[4.0, 6.0, 1.0], [3.0, 7.0, 1.0], [3.0, 8.0, 
5.0]], [[2.0, 7.0, 5.0], [3.0, 2.0, 5.0], [8.0, 2.0, 8.0]]], [[[1.0, 5.0, 7.0], [1.0, 8.0, 3.0], [8.0, 7.0, 8.0]], [[8.0, 4.0, 
1.0], [6.0, 7.0, 3.0], [4.0, 4.0, 8.0]], [[2.0, 4.0, 6.0], [4.0, 3.0, 1.0], [8.0, 6.0, 3.0]]], [[[8.0, 6.0, 1.0], [3.0, 5.0, 
4.0], [1.0, 7.0, 5.0]], [[3.0, 1.0, 5.0], [7.0, 3.0, 7.0], [4.0, 5.0, 2.0]], [[4.0, 4.0, 7.0], [3.0, 1.0, 6.0], [2.0, 8.0, 
7.0]]], [[[2.0, 1.0, 5.0], [3.0, 5.0, 7.0], [2.0, 4.0, 3.0]], [[3.0, 5.0, 3.0], [7.0, 5.0, 6.0], [6.0, 1.0, 1.0]], [[6.0, 3.0, 
2.0], [4.0, 1.0, 7.0], [4.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 5.0], [7.0, 3.0, 8.0], [7.0, 7.0, 6.0]], [[6.0, 7.0, 7.0], [4.0, 7.0, 
1.0], [5.0, 8.0, 6.0]], [[7.0, 8.0, 7.0], [7.0, 5.0, 4.0], [5.0, 6.0, 8.0]]], [[[1.0, 5.0, 1.0], [8.0, 7.0, 8.0], [7.0, 2.0, 
3.0]], [[5.0, 4.0, 2.0], [3.0, 7.0, 4.0], [5.0, 3.0, 6.0]], [[8.0, 2.0, 6.0], [8.0, 6.0, 3.0], [1.0, 2.0, 4.0]]], [[[6.0, 4.0, 
3.0], [1.0, 6.0, 7.0], [3.0, 3.0, 4.0]], [[3.0, 5.0, 5.0], [5.0, 5.0, 8.0], [2.0, 1.0, 4.0]], [[7.0, 7.0, 1.0], [6.0, 4.0, 
7.0], [1.0, 3.0, 1.0]]], [[[3.0, 4.0, 2.0], [6.0, 6.0, 7.0], [4.0, 8.0, 1.0]], [[8.0, 1.0, 8.0], [3.0, 5.0, 8.0], [2.0, 6.0, 
5.0]], [[4.0, 1.0, 2.0], [3.0, 3.0, 4.0], [8.0, 3.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0, 2.0, 3.0]], [[7.0, 1.0, 5.0]], [[8.0, 7.0, 5.0]], [[4.0, 7.0, 
2.0]]])+(1.-msk_val)*numpy.array([[[6.0, 2.0, 7.0]], [[1.0, 2.0, 1.0]], [[7.0, 1.0, 7.0]], [[7.0, 7.0, 7.0]]])
      arg[:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 8.0], [7.0, 3.0, 4.0], [8.0, 5.0, 6.0]], [[4.0, 6.0, 1.0], [3.0, 7.0, 1.0], [1.0, 
2.0, 3.0]], [[2.0, 7.0, 5.0], [3.0, 2.0, 5.0], [8.0, 2.0, 8.0]]], [[[1.0, 5.0, 7.0], [1.0, 8.0, 3.0], [8.0, 7.0, 8.0]], [[8.0, 
4.0, 1.0], [6.0, 7.0, 3.0], [7.0, 1.0, 5.0]], [[2.0, 4.0, 6.0], [4.0, 3.0, 1.0], [8.0, 6.0, 3.0]]], [[[8.0, 6.0, 1.0], [3.0, 
5.0, 4.0], [1.0, 7.0, 5.0]], [[3.0, 1.0, 5.0], [7.0, 3.0, 7.0], [8.0, 7.0, 5.0]], [[4.0, 4.0, 7.0], [3.0, 1.0, 6.0], [2.0, 8.0, 
7.0]]], [[[2.0, 1.0, 5.0], [3.0, 5.0, 7.0], [2.0, 4.0, 3.0]], [[3.0, 5.0, 3.0], [7.0, 5.0, 6.0], [4.0, 7.0, 2.0]], [[6.0, 3.0, 
2.0], [4.0, 1.0, 7.0], [4.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 5.0], [7.0, 3.0, 8.0], [7.0, 7.0, 6.0]], 
[[6.0, 7.0, 7.0], [4.0, 7.0, 1.0], [6.0, 2.0, 7.0]], [[7.0, 8.0, 7.0], [7.0, 5.0, 4.0], [5.0, 6.0, 8.0]]], [[[1.0, 5.0, 1.0], 
[8.0, 7.0, 8.0], [7.0, 2.0, 3.0]], [[5.0, 4.0, 2.0], [3.0, 7.0, 4.0], [1.0, 2.0, 1.0]], [[8.0, 2.0, 6.0], [8.0, 6.0, 3.0], 
[1.0, 2.0, 4.0]]], [[[6.0, 4.0, 3.0], [1.0, 6.0, 7.0], [3.0, 3.0, 4.0]], [[3.0, 5.0, 5.0], [5.0, 5.0, 8.0], [7.0, 1.0, 7.0]], 
[[7.0, 7.0, 1.0], [6.0, 4.0, 7.0], [1.0, 3.0, 1.0]]], [[[3.0, 4.0, 2.0], [6.0, 6.0, 7.0], [4.0, 8.0, 1.0]], [[8.0, 1.0, 8.0], 
[3.0, 5.0, 8.0], [7.0, 7.0, 7.0]], [[4.0, 1.0, 2.0], [3.0, 3.0, 4.0], [8.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CULN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 8.0], [1.0, 2.0, 7.0], [1.0, 4.0, 1.0]], [[7.0, 3.0, 4.0], [5.0, 7.0, 8.0], [2.0, 5.0, 
5.0]], [[8.0, 5.0, 3.0], [6.0, 3.0, 5.0], [2.0, 5.0, 5.0]]], [[[2.0, 2.0, 4.0], [5.0, 2.0, 6.0], [1.0, 7.0, 4.0]], [[4.0, 1.0, 
4.0], [3.0, 1.0, 4.0], [7.0, 6.0, 2.0]], [[6.0, 1.0, 7.0], [5.0, 6.0, 3.0], [3.0, 4.0, 1.0]]], [[[8.0, 8.0, 1.0], [5.0, 1.0, 
1.0], [6.0, 6.0, 4.0]], [[7.0, 5.0, 8.0], [8.0, 6.0, 6.0], [6.0, 8.0, 5.0]], [[5.0, 3.0, 2.0], [8.0, 8.0, 6.0], [5.0, 1.0, 
7.0]]], [[[1.0, 2.0, 7.0], [5.0, 6.0, 7.0], [4.0, 5.0, 6.0]], [[6.0, 6.0, 1.0], [8.0, 4.0, 3.0], [8.0, 8.0, 6.0]], [[7.0, 3.0, 
7.0], [7.0, 3.0, 1.0], [4.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 7.0], [3.0, 8.0, 3.0], [6.0, 2.0, 2.0]], [[5.0, 4.0, 6.0], [5.0, 7.0, 
7.0], [7.0, 1.0, 2.0]], [[2.0, 6.0, 6.0], [2.0, 6.0, 8.0], [5.0, 2.0, 6.0]]], [[[8.0, 7.0, 8.0], [1.0, 4.0, 6.0], [7.0, 4.0, 
4.0]], [[7.0, 8.0, 2.0], [1.0, 7.0, 3.0], [7.0, 6.0, 1.0]], [[5.0, 8.0, 8.0], [1.0, 5.0, 8.0], [4.0, 2.0, 1.0]]], [[[5.0, 7.0, 
1.0], [3.0, 7.0, 8.0], [6.0, 5.0, 3.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 4.0], [7.0, 4.0, 7.0]], [[8.0, 4.0, 3.0], [5.0, 1.0, 
1.0], [6.0, 8.0, 4.0]]], [[[2.0, 1.0, 8.0], [8.0, 6.0, 1.0], [4.0, 5.0, 3.0]], [[5.0, 2.0, 7.0], [7.0, 1.0, 5.0], [5.0, 2.0, 
5.0]], [[7.0, 6.0, 5.0], [8.0, 7.0, 8.0], [6.0, 7.0, 1.0]]]]))
      val=numpy.array([[[[6.0, 6.0, 3.0]], [[1.0, 8.0, 3.0]]], [[[3.0, 1.0, 7.0]], [[4.0, 5.0, 4.0]]], [[[7.0, 6.0, 6.0]], 
[[6.0, 2.0, 4.0]]], [[[8.0, 1.0, 5.0]], [[7.0, 2.0, 7.0]]]])
      arg[:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 8.0], [1.0, 2.0, 7.0], [6.0, 6.0, 3.0]], [[7.0, 3.0, 4.0], [5.0, 7.0, 8.0], [1.0, 
8.0, 3.0]], [[8.0, 5.0, 3.0], [6.0, 3.0, 5.0], [2.0, 5.0, 5.0]]], [[[2.0, 2.0, 4.0], [5.0, 2.0, 6.0], [3.0, 1.0, 7.0]], [[4.0, 
1.0, 4.0], [3.0, 1.0, 4.0], [4.0, 5.0, 4.0]], [[6.0, 1.0, 7.0], [5.0, 6.0, 3.0], [3.0, 4.0, 1.0]]], [[[8.0, 8.0, 1.0], [5.0, 
1.0, 1.0], [7.0, 6.0, 6.0]], [[7.0, 5.0, 8.0], [8.0, 6.0, 6.0], [6.0, 2.0, 4.0]], [[5.0, 3.0, 2.0], [8.0, 8.0, 6.0], [5.0, 1.0, 
7.0]]], [[[1.0, 2.0, 7.0], [5.0, 6.0, 7.0], [8.0, 1.0, 5.0]], [[6.0, 6.0, 1.0], [8.0, 4.0, 3.0], [7.0, 2.0, 7.0]], [[7.0, 3.0, 
7.0], [7.0, 3.0, 1.0], [4.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 7.0], [3.0, 8.0, 3.0], [6.0, 6.0, 3.0]], 
[[5.0, 4.0, 6.0], [5.0, 7.0, 7.0], [1.0, 8.0, 3.0]], [[2.0, 6.0, 6.0], [2.0, 6.0, 8.0], [5.0, 2.0, 6.0]]], [[[8.0, 7.0, 8.0], 
[1.0, 4.0, 6.0], [3.0, 1.0, 7.0]], [[7.0, 8.0, 2.0], [1.0, 7.0, 3.0], [4.0, 5.0, 4.0]], [[5.0, 8.0, 8.0], [1.0, 5.0, 8.0], 
[4.0, 2.0, 1.0]]], [[[5.0, 7.0, 1.0], [3.0, 7.0, 8.0], [7.0, 6.0, 6.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 4.0], [6.0, 2.0, 4.0]], 
[[8.0, 4.0, 3.0], [5.0, 1.0, 1.0], [6.0, 8.0, 4.0]]], [[[2.0, 1.0, 8.0], [8.0, 6.0, 1.0], [8.0, 1.0, 5.0]], [[5.0, 2.0, 7.0], 
[7.0, 1.0, 5.0], [7.0, 2.0, 7.0]], [[7.0, 6.0, 5.0], [8.0, 7.0, 8.0], [6.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CULN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 5.0], [4.0, 8.0, 8.0], [5.0, 1.0, 1.0]], [[2.0, 1.0, 6.0], [1.0, 3.0, 4.0], [5.0, 1.0, 
6.0]], [[5.0, 1.0, 1.0], [8.0, 3.0, 1.0], [1.0, 6.0, 4.0]]], [[[3.0, 7.0, 1.0], [2.0, 4.0, 8.0], [4.0, 6.0, 6.0]], [[6.0, 5.0, 
2.0], [1.0, 4.0, 6.0], [4.0, 1.0, 3.0]], [[2.0, 1.0, 1.0], [5.0, 8.0, 4.0], [8.0, 5.0, 6.0]]], [[[7.0, 4.0, 7.0], [3.0, 1.0, 
6.0], [2.0, 7.0, 6.0]], [[8.0, 2.0, 1.0], [1.0, 1.0, 5.0], [2.0, 6.0, 4.0]], [[5.0, 3.0, 6.0], [6.0, 5.0, 8.0], [6.0, 7.0, 
6.0]]], [[[4.0, 2.0, 1.0], [4.0, 6.0, 8.0], [4.0, 4.0, 6.0]], [[6.0, 1.0, 7.0], [4.0, 4.0, 8.0], [4.0, 2.0, 4.0]], [[2.0, 2.0, 
8.0], [7.0, 1.0, 6.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 3.0], [7.0, 4.0, 3.0], [6.0, 7.0, 1.0]], [[4.0, 4.0, 4.0], [6.0, 2.0, 
1.0], [8.0, 7.0, 7.0]], [[8.0, 7.0, 7.0], [1.0, 4.0, 5.0], [5.0, 5.0, 4.0]]], [[[4.0, 1.0, 6.0], [8.0, 5.0, 7.0], [7.0, 8.0, 
3.0]], [[6.0, 7.0, 6.0], [8.0, 2.0, 4.0], [4.0, 1.0, 1.0]], [[2.0, 1.0, 1.0], [5.0, 4.0, 1.0], [3.0, 8.0, 2.0]]], [[[1.0, 1.0, 
5.0], [4.0, 2.0, 4.0], [7.0, 4.0, 2.0]], [[5.0, 5.0, 2.0], [7.0, 2.0, 7.0], [2.0, 7.0, 6.0]], [[4.0, 5.0, 2.0], [2.0, 5.0, 
7.0], [4.0, 7.0, 1.0]]], [[[3.0, 3.0, 7.0], [3.0, 3.0, 1.0], [7.0, 2.0, 3.0]], [[6.0, 8.0, 3.0], [1.0, 4.0, 4.0], [5.0, 8.0, 
1.0]], [[4.0, 6.0, 4.0], [2.0, 5.0, 4.0], [8.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[[2.0, 3.0, 2.0]], [[4.0, 3.0, 6.0]]], [[[2.0, 6.0, 7.0]], [[6.0, 6.0, 4.0]]], [[[1.0, 2.0, 5.0]], 
[[8.0, 4.0, 6.0]]], [[[8.0, 1.0, 2.0]], [[4.0, 2.0, 2.0]]]]),self.functionspace)
      arg[:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 5.0], [4.0, 8.0, 8.0], [2.0, 3.0, 2.0]], [[2.0, 1.0, 6.0], [1.0, 3.0, 4.0], [4.0, 
3.0, 6.0]], [[5.0, 1.0, 1.0], [8.0, 3.0, 1.0], [1.0, 6.0, 4.0]]], [[[3.0, 7.0, 1.0], [2.0, 4.0, 8.0], [2.0, 6.0, 7.0]], [[6.0, 
5.0, 2.0], [1.0, 4.0, 6.0], [6.0, 6.0, 4.0]], [[2.0, 1.0, 1.0], [5.0, 8.0, 4.0], [8.0, 5.0, 6.0]]], [[[7.0, 4.0, 7.0], [3.0, 
1.0, 6.0], [1.0, 2.0, 5.0]], [[8.0, 2.0, 1.0], [1.0, 1.0, 5.0], [8.0, 4.0, 6.0]], [[5.0, 3.0, 6.0], [6.0, 5.0, 8.0], [6.0, 7.0, 
6.0]]], [[[4.0, 2.0, 1.0], [4.0, 6.0, 8.0], [8.0, 1.0, 2.0]], [[6.0, 1.0, 7.0], [4.0, 4.0, 8.0], [4.0, 2.0, 2.0]], [[2.0, 2.0, 
8.0], [7.0, 1.0, 6.0], [4.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 3.0], [7.0, 4.0, 3.0], [2.0, 3.0, 2.0]], 
[[4.0, 4.0, 4.0], [6.0, 2.0, 1.0], [4.0, 3.0, 6.0]], [[8.0, 7.0, 7.0], [1.0, 4.0, 5.0], [5.0, 5.0, 4.0]]], [[[4.0, 1.0, 6.0], 
[8.0, 5.0, 7.0], [2.0, 6.0, 7.0]], [[6.0, 7.0, 6.0], [8.0, 2.0, 4.0], [6.0, 6.0, 4.0]], [[2.0, 1.0, 1.0], [5.0, 4.0, 1.0], 
[3.0, 8.0, 2.0]]], [[[1.0, 1.0, 5.0], [4.0, 2.0, 4.0], [1.0, 2.0, 5.0]], [[5.0, 5.0, 2.0], [7.0, 2.0, 7.0], [8.0, 4.0, 6.0]], 
[[4.0, 5.0, 2.0], [2.0, 5.0, 7.0], [4.0, 7.0, 1.0]]], [[[3.0, 3.0, 7.0], [3.0, 3.0, 1.0], [8.0, 1.0, 2.0]], [[6.0, 8.0, 3.0], 
[1.0, 4.0, 4.0], [4.0, 2.0, 2.0]], [[4.0, 6.0, 4.0], [2.0, 5.0, 4.0], [8.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CULN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 7.0], [8.0, 6.0, 4.0], [6.0, 4.0, 4.0]], [[4.0, 6.0, 3.0], [4.0, 6.0, 1.0], [3.0, 1.0, 
8.0]], [[1.0, 5.0, 7.0], [6.0, 8.0, 3.0], [5.0, 1.0, 2.0]]], [[[8.0, 7.0, 8.0], [4.0, 1.0, 6.0], [6.0, 5.0, 2.0]], [[3.0, 5.0, 
7.0], [2.0, 7.0, 5.0], [2.0, 2.0, 1.0]], [[4.0, 2.0, 2.0], [3.0, 2.0, 5.0], [1.0, 1.0, 1.0]]], [[[2.0, 5.0, 5.0], [7.0, 6.0, 
8.0], [8.0, 3.0, 1.0]], [[7.0, 1.0, 4.0], [5.0, 5.0, 8.0], [8.0, 4.0, 5.0]], [[3.0, 8.0, 4.0], [2.0, 8.0, 3.0], [8.0, 7.0, 
5.0]]], [[[6.0, 4.0, 6.0], [4.0, 1.0, 8.0], [1.0, 5.0, 8.0]], [[8.0, 4.0, 6.0], [7.0, 7.0, 3.0], [1.0, 5.0, 4.0]], [[7.0, 7.0, 
8.0], [3.0, 7.0, 7.0], [8.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 5.0], [2.0, 6.0, 7.0], [8.0, 1.0, 1.0]], [[1.0, 3.0, 6.0], [6.0, 4.0, 
1.0], [3.0, 5.0, 1.0]], [[2.0, 7.0, 2.0], [2.0, 8.0, 1.0], [4.0, 4.0, 8.0]]], [[[2.0, 4.0, 2.0], [6.0, 8.0, 8.0], [8.0, 2.0, 
2.0]], [[1.0, 3.0, 5.0], [6.0, 1.0, 4.0], [2.0, 7.0, 6.0]], [[8.0, 4.0, 1.0], [4.0, 6.0, 2.0], [5.0, 5.0, 7.0]]], [[[3.0, 8.0, 
1.0], [6.0, 4.0, 5.0], [1.0, 7.0, 3.0]], [[8.0, 8.0, 4.0], [2.0, 3.0, 3.0], [6.0, 4.0, 2.0]], [[6.0, 1.0, 1.0], [4.0, 2.0, 
7.0], [1.0, 6.0, 3.0]]], [[[5.0, 8.0, 1.0], [5.0, 4.0, 8.0], [6.0, 5.0, 8.0]], [[1.0, 5.0, 5.0], [2.0, 3.0, 4.0], [3.0, 3.0, 
3.0]], [[6.0, 3.0, 7.0], [8.0, 3.0, 5.0], [3.0, 5.0, 4.0]]]]))
      val=Data(numpy.array([[[[3.0, 5.0, 1.0]], [[8.0, 3.0, 5.0]]], [[[7.0, 2.0, 8.0]], [[2.0, 8.0, 7.0]]], [[[3.0, 3.0, 3.0]], 
[[3.0, 3.0, 1.0]]], [[[4.0, 5.0, 3.0]], [[5.0, 3.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 1.0]], [[8.0, 7.0, 1.0]]], [[[1.0, 5.0, 7.0]], [[6.0, 6.0, 1.0]]], [[[7.0, 
3.0, 2.0]], [[5.0, 7.0, 1.0]]], [[[3.0, 5.0, 3.0]], [[3.0, 8.0, 6.0]]]]))
      arg[:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 7.0], [8.0, 6.0, 4.0], [3.0, 5.0, 1.0]], [[4.0, 6.0, 3.0], [4.0, 6.0, 1.0], [8.0, 
3.0, 5.0]], [[1.0, 5.0, 7.0], [6.0, 8.0, 3.0], [5.0, 1.0, 2.0]]], [[[8.0, 7.0, 8.0], [4.0, 1.0, 6.0], [7.0, 2.0, 8.0]], [[3.0, 
5.0, 7.0], [2.0, 7.0, 5.0], [2.0, 8.0, 7.0]], [[4.0, 2.0, 2.0], [3.0, 2.0, 5.0], [1.0, 1.0, 1.0]]], [[[2.0, 5.0, 5.0], [7.0, 
6.0, 8.0], [3.0, 3.0, 3.0]], [[7.0, 1.0, 4.0], [5.0, 5.0, 8.0], [3.0, 3.0, 1.0]], [[3.0, 8.0, 4.0], [2.0, 8.0, 3.0], [8.0, 7.0, 
5.0]]], [[[6.0, 4.0, 6.0], [4.0, 1.0, 8.0], [4.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [7.0, 7.0, 3.0], [5.0, 3.0, 7.0]], [[7.0, 7.0, 
8.0], [3.0, 7.0, 7.0], [8.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 5.0], [2.0, 6.0, 7.0], [8.0, 4.0, 1.0]], 
[[1.0, 3.0, 6.0], [6.0, 4.0, 1.0], [8.0, 7.0, 1.0]], [[2.0, 7.0, 2.0], [2.0, 8.0, 1.0], [4.0, 4.0, 8.0]]], [[[2.0, 4.0, 2.0], 
[6.0, 8.0, 8.0], [1.0, 5.0, 7.0]], [[1.0, 3.0, 5.0], [6.0, 1.0, 4.0], [6.0, 6.0, 1.0]], [[8.0, 4.0, 1.0], [4.0, 6.0, 2.0], 
[5.0, 5.0, 7.0]]], [[[3.0, 8.0, 1.0], [6.0, 4.0, 5.0], [7.0, 3.0, 2.0]], [[8.0, 8.0, 4.0], [2.0, 3.0, 3.0], [5.0, 7.0, 1.0]], 
[[6.0, 1.0, 1.0], [4.0, 2.0, 7.0], [1.0, 6.0, 3.0]]], [[[5.0, 8.0, 1.0], [5.0, 4.0, 8.0], [3.0, 5.0, 3.0]], [[1.0, 5.0, 5.0], 
[2.0, 3.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 3.0, 7.0], [8.0, 3.0, 5.0], [3.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CULN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 3.0], [8.0, 5.0, 7.0], [7.0, 8.0, 7.0]], [[6.0, 3.0, 1.0], [7.0, 4.0, 1.0], [1.0, 6.0, 
8.0]], [[3.0, 2.0, 6.0], [7.0, 6.0, 2.0], [8.0, 3.0, 6.0]]], [[[2.0, 1.0, 7.0], [3.0, 3.0, 7.0], [7.0, 1.0, 2.0]], [[8.0, 7.0, 
1.0], [2.0, 1.0, 5.0], [7.0, 5.0, 3.0]], [[1.0, 3.0, 6.0], [7.0, 3.0, 1.0], [5.0, 3.0, 5.0]]], [[[2.0, 5.0, 1.0], [7.0, 7.0, 
1.0], [5.0, 1.0, 3.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 6.0], [2.0, 1.0, 8.0]], [[1.0, 4.0, 7.0], [6.0, 1.0, 2.0], [6.0, 5.0, 
8.0]]], [[[7.0, 2.0, 4.0], [8.0, 3.0, 2.0], [5.0, 2.0, 4.0]], [[7.0, 2.0, 5.0], [1.0, 2.0, 8.0], [6.0, 1.0, 7.0]], [[7.0, 1.0, 
8.0], [4.0, 8.0, 6.0], [3.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 5.0], [3.0, 5.0, 8.0], [7.0, 2.0, 2.0]], [[1.0, 1.0, 1.0], [7.0, 7.0, 
2.0], [2.0, 6.0, 3.0]], [[8.0, 1.0, 5.0], [2.0, 2.0, 5.0], [3.0, 6.0, 4.0]]], [[[5.0, 7.0, 2.0], [3.0, 8.0, 4.0], [6.0, 1.0, 
6.0]], [[3.0, 2.0, 7.0], [5.0, 5.0, 6.0], [7.0, 2.0, 8.0]], [[8.0, 8.0, 7.0], [6.0, 5.0, 2.0], [2.0, 1.0, 2.0]]], [[[4.0, 7.0, 
2.0], [4.0, 8.0, 6.0], [6.0, 5.0, 3.0]], [[6.0, 1.0, 1.0], [6.0, 6.0, 8.0], [4.0, 6.0, 8.0]], [[5.0, 4.0, 5.0], [3.0, 8.0, 
2.0], [4.0, 6.0, 6.0]]], [[[5.0, 8.0, 3.0], [4.0, 3.0, 5.0], [7.0, 5.0, 2.0]], [[7.0, 1.0, 6.0], [2.0, 2.0, 7.0], [5.0, 4.0, 
3.0]], [[1.0, 3.0, 3.0], [7.0, 8.0, 1.0], [1.0, 2.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 5.0, 1.0]], [[8.0, 6.0, 4.0]]], [[[6.0, 6.0, 7.0]], [[8.0, 5.0, 8.0]]], [[[5.0, 5.0, 
2.0]], [[3.0, 6.0, 1.0]]], [[[4.0, 6.0, 8.0]], [[2.0, 1.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 5.0, 8.0]], [[7.0, 7.0, 
7.0]]], [[[7.0, 2.0, 6.0]], [[8.0, 3.0, 6.0]]], [[[5.0, 1.0, 8.0]], [[4.0, 6.0, 2.0]]], [[[8.0, 1.0, 6.0]], [[7.0, 8.0, 4.0]]]])
      arg[:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 3.0], [8.0, 5.0, 7.0], [8.0, 5.0, 1.0]], [[6.0, 3.0, 1.0], [7.0, 4.0, 1.0], [8.0, 
6.0, 4.0]], [[3.0, 2.0, 6.0], [7.0, 6.0, 2.0], [8.0, 3.0, 6.0]]], [[[2.0, 1.0, 7.0], [3.0, 3.0, 7.0], [6.0, 6.0, 7.0]], [[8.0, 
7.0, 1.0], [2.0, 1.0, 5.0], [8.0, 5.0, 8.0]], [[1.0, 3.0, 6.0], [7.0, 3.0, 1.0], [5.0, 3.0, 5.0]]], [[[2.0, 5.0, 1.0], [7.0, 
7.0, 1.0], [5.0, 5.0, 2.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 6.0], [3.0, 6.0, 1.0]], [[1.0, 4.0, 7.0], [6.0, 1.0, 2.0], [6.0, 5.0, 
8.0]]], [[[7.0, 2.0, 4.0], [8.0, 3.0, 2.0], [4.0, 6.0, 8.0]], [[7.0, 2.0, 5.0], [1.0, 2.0, 8.0], [2.0, 1.0, 2.0]], [[7.0, 1.0, 
8.0], [4.0, 8.0, 6.0], [3.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 5.0], [3.0, 5.0, 8.0], [6.0, 5.0, 8.0]], 
[[1.0, 1.0, 1.0], [7.0, 7.0, 2.0], [7.0, 7.0, 7.0]], [[8.0, 1.0, 5.0], [2.0, 2.0, 5.0], [3.0, 6.0, 4.0]]], [[[5.0, 7.0, 2.0], 
[3.0, 8.0, 4.0], [7.0, 2.0, 6.0]], [[3.0, 2.0, 7.0], [5.0, 5.0, 6.0], [8.0, 3.0, 6.0]], [[8.0, 8.0, 7.0], [6.0, 5.0, 2.0], 
[2.0, 1.0, 2.0]]], [[[4.0, 7.0, 2.0], [4.0, 8.0, 6.0], [5.0, 1.0, 8.0]], [[6.0, 1.0, 1.0], [6.0, 6.0, 8.0], [4.0, 6.0, 2.0]], 
[[5.0, 4.0, 5.0], [3.0, 8.0, 2.0], [4.0, 6.0, 6.0]]], [[[5.0, 8.0, 3.0], [4.0, 3.0, 5.0], [8.0, 1.0, 6.0]], [[7.0, 1.0, 6.0], 
[2.0, 2.0, 7.0], [7.0, 8.0, 4.0]], [[1.0, 3.0, 3.0], [7.0, 8.0, 1.0], [1.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PRUN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 7.0], [7.0, 3.0, 5.0], [4.0, 5.0, 7.0]], [[6.0, 4.0, 2.0], [4.0, 2.0, 7.0], [3.0, 3.0, 
4.0]], [[7.0, 7.0, 7.0], [6.0, 6.0, 5.0], [2.0, 8.0, 5.0]]], [[[6.0, 2.0, 7.0], [6.0, 4.0, 1.0], [5.0, 2.0, 1.0]], [[5.0, 8.0, 
7.0], [1.0, 3.0, 8.0], [5.0, 8.0, 7.0]], [[1.0, 8.0, 5.0], [3.0, 1.0, 7.0], [2.0, 7.0, 3.0]]], [[[2.0, 6.0, 6.0], [3.0, 3.0, 
2.0], [5.0, 7.0, 7.0]], [[7.0, 6.0, 8.0], [8.0, 6.0, 5.0], [4.0, 8.0, 1.0]], [[2.0, 5.0, 8.0], [6.0, 4.0, 1.0], [7.0, 8.0, 
1.0]]], [[[4.0, 3.0, 8.0], [7.0, 6.0, 7.0], [7.0, 7.0, 6.0]], [[2.0, 1.0, 4.0], [2.0, 4.0, 3.0], [5.0, 4.0, 2.0]], [[6.0, 3.0, 
5.0], [4.0, 3.0, 3.0], [6.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 6.0], [3.0, 1.0, 2.0], [8.0, 5.0, 8.0]], [[8.0, 8.0, 5.0], [8.0, 1.0, 
8.0], [4.0, 8.0, 5.0]], [[4.0, 6.0, 8.0], [6.0, 2.0, 7.0], [4.0, 8.0, 4.0]]], [[[5.0, 7.0, 1.0], [4.0, 7.0, 3.0], [4.0, 4.0, 
3.0]], [[5.0, 5.0, 6.0], [4.0, 2.0, 1.0], [7.0, 1.0, 5.0]], [[2.0, 4.0, 3.0], [5.0, 1.0, 5.0], [3.0, 7.0, 8.0]]], [[[2.0, 8.0, 
6.0], [3.0, 4.0, 8.0], [5.0, 4.0, 4.0]], [[1.0, 7.0, 8.0], [8.0, 7.0, 4.0], [6.0, 8.0, 6.0]], [[1.0, 4.0, 2.0], [5.0, 7.0, 
4.0], [5.0, 2.0, 1.0]]], [[[4.0, 8.0, 5.0], [2.0, 6.0, 2.0], [5.0, 2.0, 1.0]], [[6.0, 3.0, 1.0], [5.0, 5.0, 1.0], [8.0, 4.0, 
7.0]], [[4.0, 2.0, 1.0], [5.0, 6.0, 4.0], [8.0, 2.0, 2.0]]]]))
      val=numpy.array([[[1.0, 3.0, 5.0], [4.0, 8.0, 5.0]]])
      arg[2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 7.0], [7.0, 3.0, 5.0], [4.0, 5.0, 7.0]], [[6.0, 4.0, 2.0], [4.0, 2.0, 7.0], [3.0, 
3.0, 4.0]], [[7.0, 7.0, 7.0], [6.0, 6.0, 5.0], [2.0, 8.0, 5.0]]], [[[6.0, 2.0, 7.0], [6.0, 4.0, 1.0], [5.0, 2.0, 1.0]], [[5.0, 
8.0, 7.0], [1.0, 3.0, 8.0], [5.0, 8.0, 7.0]], [[1.0, 8.0, 5.0], [3.0, 1.0, 7.0], [2.0, 7.0, 3.0]]], [[[2.0, 6.0, 6.0], [3.0, 
3.0, 2.0], [5.0, 7.0, 7.0]], [[1.0, 3.0, 5.0], [4.0, 8.0, 5.0], [4.0, 8.0, 1.0]], [[2.0, 5.0, 8.0], [6.0, 4.0, 1.0], [7.0, 8.0, 
1.0]]], [[[4.0, 3.0, 8.0], [7.0, 6.0, 7.0], [7.0, 7.0, 6.0]], [[2.0, 1.0, 4.0], [2.0, 4.0, 3.0], [5.0, 4.0, 2.0]], [[6.0, 3.0, 
5.0], [4.0, 3.0, 3.0], [6.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 6.0], [3.0, 1.0, 2.0], [8.0, 5.0, 8.0]], 
[[8.0, 8.0, 5.0], [8.0, 1.0, 8.0], [4.0, 8.0, 5.0]], [[4.0, 6.0, 8.0], [6.0, 2.0, 7.0], [4.0, 8.0, 4.0]]], [[[5.0, 7.0, 1.0], 
[4.0, 7.0, 3.0], [4.0, 4.0, 3.0]], [[5.0, 5.0, 6.0], [4.0, 2.0, 1.0], [7.0, 1.0, 5.0]], [[2.0, 4.0, 3.0], [5.0, 1.0, 5.0], 
[3.0, 7.0, 8.0]]], [[[2.0, 8.0, 6.0], [3.0, 4.0, 8.0], [5.0, 4.0, 4.0]], [[1.0, 3.0, 5.0], [4.0, 8.0, 5.0], [6.0, 8.0, 6.0]], 
[[1.0, 4.0, 2.0], [5.0, 7.0, 4.0], [5.0, 2.0, 1.0]]], [[[4.0, 8.0, 5.0], [2.0, 6.0, 2.0], [5.0, 2.0, 1.0]], [[6.0, 3.0, 1.0], 
[5.0, 5.0, 1.0], [8.0, 4.0, 7.0]], [[4.0, 2.0, 1.0], [5.0, 6.0, 4.0], [8.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PRUN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 1.0], [1.0, 1.0, 7.0], [3.0, 1.0, 3.0]], [[6.0, 8.0, 2.0], [6.0, 4.0, 3.0], [8.0, 8.0, 
3.0]], [[7.0, 4.0, 1.0], [6.0, 8.0, 5.0], [6.0, 4.0, 2.0]]], [[[1.0, 2.0, 3.0], [4.0, 5.0, 2.0], [2.0, 7.0, 5.0]], [[4.0, 8.0, 
6.0], [5.0, 1.0, 8.0], [5.0, 3.0, 8.0]], [[3.0, 8.0, 7.0], [8.0, 5.0, 7.0], [1.0, 1.0, 4.0]]], [[[5.0, 1.0, 7.0], [1.0, 4.0, 
2.0], [1.0, 7.0, 1.0]], [[3.0, 8.0, 6.0], [2.0, 6.0, 3.0], [8.0, 3.0, 5.0]], [[7.0, 4.0, 8.0], [4.0, 4.0, 3.0], [5.0, 3.0, 
2.0]]], [[[3.0, 7.0, 1.0], [6.0, 5.0, 3.0], [3.0, 5.0, 6.0]], [[8.0, 5.0, 1.0], [4.0, 7.0, 2.0], [8.0, 2.0, 3.0]], [[8.0, 4.0, 
3.0], [8.0, 1.0, 4.0], [5.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 7.0], [3.0, 2.0, 7.0], [1.0, 3.0, 4.0]], [[5.0, 1.0, 7.0], [2.0, 7.0, 
4.0], [1.0, 8.0, 4.0]], [[7.0, 8.0, 7.0], [1.0, 5.0, 6.0], [4.0, 1.0, 1.0]]], [[[7.0, 1.0, 6.0], [4.0, 7.0, 8.0], [6.0, 2.0, 
7.0]], [[3.0, 8.0, 5.0], [8.0, 2.0, 6.0], [4.0, 4.0, 8.0]], [[7.0, 7.0, 3.0], [7.0, 3.0, 2.0], [6.0, 4.0, 4.0]]], [[[4.0, 5.0, 
7.0], [2.0, 4.0, 6.0], [7.0, 6.0, 6.0]], [[1.0, 4.0, 7.0], [3.0, 8.0, 5.0], [7.0, 3.0, 8.0]], [[1.0, 7.0, 5.0], [4.0, 1.0, 
8.0], [7.0, 6.0, 5.0]]], [[[4.0, 1.0, 6.0], [6.0, 7.0, 1.0], [6.0, 6.0, 3.0]], [[8.0, 2.0, 2.0], [4.0, 8.0, 5.0], [4.0, 4.0, 
3.0]], [[3.0, 5.0, 1.0], [4.0, 4.0, 8.0], [5.0, 1.0, 5.0]]]]))
      val=Data(numpy.array([[[8.0, 4.0, 6.0], [6.0, 1.0, 7.0]]]),self.functionspace)
      arg[2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 1.0], [1.0, 1.0, 7.0], [3.0, 1.0, 3.0]], [[6.0, 8.0, 2.0], [6.0, 4.0, 3.0], [8.0, 
8.0, 3.0]], [[7.0, 4.0, 1.0], [6.0, 8.0, 5.0], [6.0, 4.0, 2.0]]], [[[1.0, 2.0, 3.0], [4.0, 5.0, 2.0], [2.0, 7.0, 5.0]], [[4.0, 
8.0, 6.0], [5.0, 1.0, 8.0], [5.0, 3.0, 8.0]], [[3.0, 8.0, 7.0], [8.0, 5.0, 7.0], [1.0, 1.0, 4.0]]], [[[5.0, 1.0, 7.0], [1.0, 
4.0, 2.0], [1.0, 7.0, 1.0]], [[8.0, 4.0, 6.0], [6.0, 1.0, 7.0], [8.0, 3.0, 5.0]], [[7.0, 4.0, 8.0], [4.0, 4.0, 3.0], [5.0, 3.0, 
2.0]]], [[[3.0, 7.0, 1.0], [6.0, 5.0, 3.0], [3.0, 5.0, 6.0]], [[8.0, 5.0, 1.0], [4.0, 7.0, 2.0], [8.0, 2.0, 3.0]], [[8.0, 4.0, 
3.0], [8.0, 1.0, 4.0], [5.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 7.0], [3.0, 2.0, 7.0], [1.0, 3.0, 4.0]], 
[[5.0, 1.0, 7.0], [2.0, 7.0, 4.0], [1.0, 8.0, 4.0]], [[7.0, 8.0, 7.0], [1.0, 5.0, 6.0], [4.0, 1.0, 1.0]]], [[[7.0, 1.0, 6.0], 
[4.0, 7.0, 8.0], [6.0, 2.0, 7.0]], [[3.0, 8.0, 5.0], [8.0, 2.0, 6.0], [4.0, 4.0, 8.0]], [[7.0, 7.0, 3.0], [7.0, 3.0, 2.0], 
[6.0, 4.0, 4.0]]], [[[4.0, 5.0, 7.0], [2.0, 4.0, 6.0], [7.0, 6.0, 6.0]], [[8.0, 4.0, 6.0], [6.0, 1.0, 7.0], [7.0, 3.0, 8.0]], 
[[1.0, 7.0, 5.0], [4.0, 1.0, 8.0], [7.0, 6.0, 5.0]]], [[[4.0, 1.0, 6.0], [6.0, 7.0, 1.0], [6.0, 6.0, 3.0]], [[8.0, 2.0, 2.0], 
[4.0, 8.0, 5.0], [4.0, 4.0, 3.0]], [[3.0, 5.0, 1.0], [4.0, 4.0, 8.0], [5.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PRUN(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 6.0], [1.0, 7.0, 3.0], [8.0, 2.0, 2.0]], [[7.0, 1.0, 1.0], [7.0, 8.0, 1.0], [7.0, 5.0, 
1.0]], [[8.0, 4.0, 5.0], [3.0, 3.0, 8.0], [1.0, 2.0, 6.0]]], [[[8.0, 6.0, 6.0], [6.0, 3.0, 6.0], [1.0, 6.0, 4.0]], [[4.0, 3.0, 
4.0], [6.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[6.0, 3.0, 6.0], [8.0, 6.0, 1.0], [2.0, 4.0, 5.0]]], [[[5.0, 8.0, 6.0], [1.0, 3.0, 
2.0], [3.0, 2.0, 7.0]], [[2.0, 4.0, 8.0], [8.0, 3.0, 6.0], [8.0, 4.0, 2.0]], [[2.0, 4.0, 1.0], [7.0, 5.0, 2.0], [1.0, 1.0, 
8.0]]], [[[1.0, 6.0, 7.0], [3.0, 8.0, 1.0], [3.0, 4.0, 2.0]], [[4.0, 5.0, 3.0], [6.0, 3.0, 1.0], [8.0, 6.0, 3.0]], [[5.0, 7.0, 
7.0], [6.0, 6.0, 5.0], [3.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 4.0], [8.0, 7.0, 8.0], [6.0, 7.0, 5.0]], [[3.0, 3.0, 1.0], [1.0, 4.0, 
1.0], [8.0, 2.0, 8.0]], [[8.0, 2.0, 7.0], [6.0, 8.0, 6.0], [5.0, 7.0, 1.0]]], [[[1.0, 6.0, 7.0], [1.0, 8.0, 1.0], [1.0, 5.0, 
5.0]], [[6.0, 1.0, 7.0], [1.0, 8.0, 4.0], [7.0, 3.0, 6.0]], [[1.0, 8.0, 2.0], [7.0, 1.0, 1.0], [2.0, 5.0, 5.0]]], [[[5.0, 5.0, 
4.0], [1.0, 6.0, 8.0], [5.0, 7.0, 3.0]], [[4.0, 7.0, 8.0], [5.0, 6.0, 4.0], [8.0, 3.0, 7.0]], [[8.0, 1.0, 7.0], [4.0, 1.0, 
7.0], [2.0, 1.0, 4.0]]], [[[7.0, 6.0, 1.0], [3.0, 8.0, 2.0], [2.0, 1.0, 7.0]], [[6.0, 5.0, 2.0], [6.0, 4.0, 8.0], [6.0, 7.0, 
1.0]], [[8.0, 8.0, 4.0], [7.0, 6.0, 3.0], [4.0, 5.0, 8.0]]]]))
      val=Data(numpy.array([[[7.0, 6.0, 4.0], [1.0, 6.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 3.0, 6.0], [5.0, 7.0, 3.0]]]))
      arg[2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 6.0], [1.0, 7.0, 3.0], [8.0, 2.0, 2.0]], [[7.0, 1.0, 1.0], [7.0, 8.0, 1.0], [7.0, 
5.0, 1.0]], [[8.0, 4.0, 5.0], [3.0, 3.0, 8.0], [1.0, 2.0, 6.0]]], [[[8.0, 6.0, 6.0], [6.0, 3.0, 6.0], [1.0, 6.0, 4.0]], [[4.0, 
3.0, 4.0], [6.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[6.0, 3.0, 6.0], [8.0, 6.0, 1.0], [2.0, 4.0, 5.0]]], [[[5.0, 8.0, 6.0], [1.0, 
3.0, 2.0], [3.0, 2.0, 7.0]], [[7.0, 6.0, 4.0], [1.0, 6.0, 5.0], [8.0, 4.0, 2.0]], [[2.0, 4.0, 1.0], [7.0, 5.0, 2.0], [1.0, 1.0, 
8.0]]], [[[1.0, 6.0, 7.0], [3.0, 8.0, 1.0], [3.0, 4.0, 2.0]], [[4.0, 5.0, 3.0], [6.0, 3.0, 1.0], [8.0, 6.0, 3.0]], [[5.0, 7.0, 
7.0], [6.0, 6.0, 5.0], [3.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 4.0], [8.0, 7.0, 8.0], [6.0, 7.0, 5.0]], 
[[3.0, 3.0, 1.0], [1.0, 4.0, 1.0], [8.0, 2.0, 8.0]], [[8.0, 2.0, 7.0], [6.0, 8.0, 6.0], [5.0, 7.0, 1.0]]], [[[1.0, 6.0, 7.0], 
[1.0, 8.0, 1.0], [1.0, 5.0, 5.0]], [[6.0, 1.0, 7.0], [1.0, 8.0, 4.0], [7.0, 3.0, 6.0]], [[1.0, 8.0, 2.0], [7.0, 1.0, 1.0], 
[2.0, 5.0, 5.0]]], [[[5.0, 5.0, 4.0], [1.0, 6.0, 8.0], [5.0, 7.0, 3.0]], [[4.0, 3.0, 6.0], [5.0, 7.0, 3.0], [8.0, 3.0, 7.0]], 
[[8.0, 1.0, 7.0], [4.0, 1.0, 7.0], [2.0, 1.0, 4.0]]], [[[7.0, 6.0, 1.0], [3.0, 8.0, 2.0], [2.0, 1.0, 7.0]], [[6.0, 5.0, 2.0], 
[6.0, 4.0, 8.0], [6.0, 7.0, 1.0]], [[8.0, 8.0, 4.0], [7.0, 6.0, 3.0], [4.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PRUN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 3.0], [3.0, 6.0, 6.0], [1.0, 2.0, 8.0]], [[1.0, 4.0, 5.0], [1.0, 4.0, 6.0], [8.0, 2.0, 
3.0]], [[7.0, 8.0, 2.0], [5.0, 1.0, 1.0], [5.0, 5.0, 7.0]]], [[[4.0, 7.0, 8.0], [1.0, 5.0, 5.0], [5.0, 8.0, 4.0]], [[6.0, 5.0, 
1.0], [6.0, 2.0, 4.0], [2.0, 8.0, 1.0]], [[1.0, 3.0, 2.0], [1.0, 7.0, 8.0], [4.0, 1.0, 4.0]]], [[[4.0, 6.0, 1.0], [3.0, 1.0, 
4.0], [3.0, 6.0, 5.0]], [[1.0, 3.0, 6.0], [2.0, 2.0, 4.0], [7.0, 5.0, 1.0]], [[5.0, 4.0, 5.0], [6.0, 3.0, 8.0], [6.0, 3.0, 
4.0]]], [[[4.0, 7.0, 8.0], [4.0, 6.0, 5.0], [8.0, 4.0, 8.0]], [[5.0, 1.0, 1.0], [2.0, 4.0, 8.0], [7.0, 7.0, 5.0]], [[5.0, 5.0, 
2.0], [5.0, 7.0, 2.0], [5.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 4.0], [5.0, 2.0, 5.0], [4.0, 6.0, 6.0]], [[3.0, 4.0, 4.0], [6.0, 8.0, 
5.0], [3.0, 1.0, 3.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 8.0], [6.0, 5.0, 2.0]]], [[[4.0, 3.0, 6.0], [1.0, 2.0, 3.0], [1.0, 1.0, 
1.0]], [[6.0, 8.0, 8.0], [2.0, 1.0, 5.0], [2.0, 6.0, 4.0]], [[1.0, 2.0, 6.0], [7.0, 8.0, 4.0], [4.0, 1.0, 5.0]]], [[[4.0, 6.0, 
2.0], [3.0, 4.0, 7.0], [5.0, 4.0, 4.0]], [[1.0, 8.0, 2.0], [4.0, 2.0, 1.0], [7.0, 4.0, 2.0]], [[1.0, 7.0, 1.0], [1.0, 4.0, 
3.0], [5.0, 2.0, 8.0]]], [[[1.0, 4.0, 3.0], [7.0, 5.0, 1.0], [8.0, 2.0, 8.0]], [[2.0, 6.0, 8.0], [3.0, 3.0, 8.0], [2.0, 2.0, 
8.0]], [[3.0, 8.0, 2.0], [3.0, 1.0, 4.0], [2.0, 7.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 2.0, 7.0], [1.0, 4.0, 7.0]]])+(1.-msk_val)*numpy.array([[[8.0, 4.0, 4.0], [8.0, 1.0, 
6.0]]])
      arg[2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 3.0], [3.0, 6.0, 6.0], [1.0, 2.0, 8.0]], [[1.0, 4.0, 5.0], [1.0, 4.0, 6.0], [8.0, 
2.0, 3.0]], [[7.0, 8.0, 2.0], [5.0, 1.0, 1.0], [5.0, 5.0, 7.0]]], [[[4.0, 7.0, 8.0], [1.0, 5.0, 5.0], [5.0, 8.0, 4.0]], [[6.0, 
5.0, 1.0], [6.0, 2.0, 4.0], [2.0, 8.0, 1.0]], [[1.0, 3.0, 2.0], [1.0, 7.0, 8.0], [4.0, 1.0, 4.0]]], [[[4.0, 6.0, 1.0], [3.0, 
1.0, 4.0], [3.0, 6.0, 5.0]], [[4.0, 2.0, 7.0], [1.0, 4.0, 7.0], [7.0, 5.0, 1.0]], [[5.0, 4.0, 5.0], [6.0, 3.0, 8.0], [6.0, 3.0, 
4.0]]], [[[4.0, 7.0, 8.0], [4.0, 6.0, 5.0], [8.0, 4.0, 8.0]], [[5.0, 1.0, 1.0], [2.0, 4.0, 8.0], [7.0, 7.0, 5.0]], [[5.0, 5.0, 
2.0], [5.0, 7.0, 2.0], [5.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 4.0], [5.0, 2.0, 5.0], [4.0, 6.0, 6.0]], 
[[3.0, 4.0, 4.0], [6.0, 8.0, 5.0], [3.0, 1.0, 3.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 8.0], [6.0, 5.0, 2.0]]], [[[4.0, 3.0, 6.0], 
[1.0, 2.0, 3.0], [1.0, 1.0, 1.0]], [[6.0, 8.0, 8.0], [2.0, 1.0, 5.0], [2.0, 6.0, 4.0]], [[1.0, 2.0, 6.0], [7.0, 8.0, 4.0], 
[4.0, 1.0, 5.0]]], [[[4.0, 6.0, 2.0], [3.0, 4.0, 7.0], [5.0, 4.0, 4.0]], [[8.0, 4.0, 4.0], [8.0, 1.0, 6.0], [7.0, 4.0, 2.0]], 
[[1.0, 7.0, 1.0], [1.0, 4.0, 3.0], [5.0, 2.0, 8.0]]], [[[1.0, 4.0, 3.0], [7.0, 5.0, 1.0], [8.0, 2.0, 8.0]], [[2.0, 6.0, 8.0], 
[3.0, 3.0, 8.0], [2.0, 2.0, 8.0]], [[3.0, 8.0, 2.0], [3.0, 1.0, 4.0], [2.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRLN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 1.0], [6.0, 7.0, 1.0], [5.0, 4.0, 8.0]], [[5.0, 3.0, 4.0], [7.0, 1.0, 6.0], [4.0, 5.0, 
5.0]], [[7.0, 3.0, 1.0], [4.0, 4.0, 1.0], [6.0, 7.0, 6.0]]], [[[6.0, 6.0, 7.0], [3.0, 6.0, 3.0], [7.0, 4.0, 1.0]], [[3.0, 2.0, 
5.0], [4.0, 4.0, 2.0], [5.0, 6.0, 7.0]], [[1.0, 2.0, 6.0], [1.0, 8.0, 4.0], [3.0, 1.0, 1.0]]], [[[5.0, 7.0, 1.0], [5.0, 7.0, 
8.0], [1.0, 5.0, 8.0]], [[3.0, 2.0, 2.0], [7.0, 7.0, 6.0], [8.0, 5.0, 8.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 1.0], [6.0, 1.0, 
2.0]]], [[[2.0, 2.0, 5.0], [2.0, 6.0, 8.0], [4.0, 8.0, 1.0]], [[4.0, 7.0, 3.0], [3.0, 2.0, 1.0], [6.0, 1.0, 2.0]], [[8.0, 8.0, 
8.0], [5.0, 6.0, 7.0], [7.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 5.0], [5.0, 6.0, 5.0], [3.0, 4.0, 3.0]], [[5.0, 8.0, 4.0], [6.0, 5.0, 
1.0], [5.0, 6.0, 2.0]], [[1.0, 5.0, 2.0], [8.0, 1.0, 8.0], [2.0, 4.0, 2.0]]], [[[6.0, 1.0, 1.0], [7.0, 4.0, 7.0], [8.0, 4.0, 
6.0]], [[5.0, 7.0, 4.0], [3.0, 2.0, 4.0], [6.0, 4.0, 7.0]], [[1.0, 4.0, 7.0], [2.0, 6.0, 6.0], [1.0, 1.0, 6.0]]], [[[5.0, 3.0, 
8.0], [8.0, 1.0, 2.0], [3.0, 3.0, 3.0]], [[7.0, 7.0, 5.0], [3.0, 2.0, 1.0], [4.0, 6.0, 1.0]], [[6.0, 6.0, 6.0], [2.0, 8.0, 
5.0], [2.0, 4.0, 8.0]]], [[[8.0, 5.0, 3.0], [2.0, 5.0, 8.0], [8.0, 6.0, 8.0]], [[4.0, 3.0, 7.0], [2.0, 5.0, 4.0], [8.0, 1.0, 
8.0]], [[1.0, 1.0, 6.0], [2.0, 4.0, 6.0], [2.0, 3.0, 2.0]]]]))
      val=numpy.array([[[[2.0, 1.0, 1.0]]], [[[3.0, 8.0, 7.0]]]])
      arg[1:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 1.0], [6.0, 7.0, 1.0], [5.0, 4.0, 8.0]], [[5.0, 3.0, 4.0], [7.0, 1.0, 6.0], [4.0, 
5.0, 5.0]], [[7.0, 3.0, 1.0], [4.0, 4.0, 1.0], [6.0, 7.0, 6.0]]], [[[6.0, 6.0, 7.0], [3.0, 6.0, 3.0], [7.0, 4.0, 1.0]], [[3.0, 
2.0, 5.0], [4.0, 4.0, 2.0], [2.0, 1.0, 1.0]], [[1.0, 2.0, 6.0], [1.0, 8.0, 4.0], [3.0, 1.0, 1.0]]], [[[5.0, 7.0, 1.0], [5.0, 
7.0, 8.0], [1.0, 5.0, 8.0]], [[3.0, 2.0, 2.0], [7.0, 7.0, 6.0], [3.0, 8.0, 7.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 1.0], [6.0, 1.0, 
2.0]]], [[[2.0, 2.0, 5.0], [2.0, 6.0, 8.0], [4.0, 8.0, 1.0]], [[4.0, 7.0, 3.0], [3.0, 2.0, 1.0], [6.0, 1.0, 2.0]], [[8.0, 8.0, 
8.0], [5.0, 6.0, 7.0], [7.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 5.0], [5.0, 6.0, 5.0], [3.0, 4.0, 3.0]], 
[[5.0, 8.0, 4.0], [6.0, 5.0, 1.0], [5.0, 6.0, 2.0]], [[1.0, 5.0, 2.0], [8.0, 1.0, 8.0], [2.0, 4.0, 2.0]]], [[[6.0, 1.0, 1.0], 
[7.0, 4.0, 7.0], [8.0, 4.0, 6.0]], [[5.0, 7.0, 4.0], [3.0, 2.0, 4.0], [2.0, 1.0, 1.0]], [[1.0, 4.0, 7.0], [2.0, 6.0, 6.0], 
[1.0, 1.0, 6.0]]], [[[5.0, 3.0, 8.0], [8.0, 1.0, 2.0], [3.0, 3.0, 3.0]], [[7.0, 7.0, 5.0], [3.0, 2.0, 1.0], [3.0, 8.0, 7.0]], 
[[6.0, 6.0, 6.0], [2.0, 8.0, 5.0], [2.0, 4.0, 8.0]]], [[[8.0, 5.0, 3.0], [2.0, 5.0, 8.0], [8.0, 6.0, 8.0]], [[4.0, 3.0, 7.0], 
[2.0, 5.0, 4.0], [8.0, 1.0, 8.0]], [[1.0, 1.0, 6.0], [2.0, 4.0, 6.0], [2.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRLN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 4.0], [1.0, 7.0, 7.0], [7.0, 6.0, 2.0]], [[7.0, 6.0, 8.0], [8.0, 6.0, 8.0], [5.0, 2.0, 
2.0]], [[3.0, 4.0, 3.0], [2.0, 8.0, 2.0], [4.0, 6.0, 1.0]]], [[[7.0, 6.0, 1.0], [6.0, 2.0, 6.0], [7.0, 4.0, 3.0]], [[1.0, 8.0, 
6.0], [5.0, 2.0, 2.0], [5.0, 2.0, 1.0]], [[4.0, 5.0, 7.0], [4.0, 8.0, 2.0], [6.0, 7.0, 4.0]]], [[[5.0, 1.0, 2.0], [2.0, 3.0, 
2.0], [2.0, 1.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 3.0, 2.0], [2.0, 4.0, 3.0]], [[4.0, 7.0, 3.0], [5.0, 2.0, 7.0], [8.0, 8.0, 
7.0]]], [[[5.0, 2.0, 5.0], [1.0, 4.0, 6.0], [4.0, 3.0, 8.0]], [[3.0, 6.0, 6.0], [5.0, 3.0, 1.0], [8.0, 7.0, 6.0]], [[5.0, 5.0, 
8.0], [7.0, 7.0, 8.0], [1.0, 3.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 1.0], [8.0, 5.0, 3.0], [5.0, 5.0, 1.0]], [[2.0, 7.0, 1.0], [4.0, 2.0, 
3.0], [6.0, 3.0, 5.0]], [[2.0, 1.0, 1.0], [3.0, 1.0, 8.0], [7.0, 2.0, 7.0]]], [[[2.0, 3.0, 6.0], [4.0, 6.0, 5.0], [5.0, 5.0, 
7.0]], [[7.0, 4.0, 4.0], [1.0, 7.0, 5.0], [8.0, 5.0, 4.0]], [[7.0, 8.0, 2.0], [8.0, 2.0, 6.0], [3.0, 1.0, 3.0]]], [[[3.0, 4.0, 
4.0], [2.0, 5.0, 4.0], [4.0, 5.0, 1.0]], [[1.0, 3.0, 8.0], [8.0, 2.0, 6.0], [4.0, 3.0, 3.0]], [[7.0, 1.0, 2.0], [6.0, 3.0, 
2.0], [5.0, 6.0, 2.0]]], [[[3.0, 5.0, 6.0], [3.0, 5.0, 3.0], [4.0, 3.0, 5.0]], [[6.0, 6.0, 8.0], [3.0, 4.0, 8.0], [7.0, 7.0, 
3.0]], [[5.0, 3.0, 3.0], [5.0, 2.0, 2.0], [7.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[[2.0, 5.0, 5.0]]], [[[1.0, 4.0, 7.0]]]]),self.functionspace)
      arg[1:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 4.0], [1.0, 7.0, 7.0], [7.0, 6.0, 2.0]], [[7.0, 6.0, 8.0], [8.0, 6.0, 8.0], [5.0, 
2.0, 2.0]], [[3.0, 4.0, 3.0], [2.0, 8.0, 2.0], [4.0, 6.0, 1.0]]], [[[7.0, 6.0, 1.0], [6.0, 2.0, 6.0], [7.0, 4.0, 3.0]], [[1.0, 
8.0, 6.0], [5.0, 2.0, 2.0], [2.0, 5.0, 5.0]], [[4.0, 5.0, 7.0], [4.0, 8.0, 2.0], [6.0, 7.0, 4.0]]], [[[5.0, 1.0, 2.0], [2.0, 
3.0, 2.0], [2.0, 1.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 3.0, 2.0], [1.0, 4.0, 7.0]], [[4.0, 7.0, 3.0], [5.0, 2.0, 7.0], [8.0, 8.0, 
7.0]]], [[[5.0, 2.0, 5.0], [1.0, 4.0, 6.0], [4.0, 3.0, 8.0]], [[3.0, 6.0, 6.0], [5.0, 3.0, 1.0], [8.0, 7.0, 6.0]], [[5.0, 5.0, 
8.0], [7.0, 7.0, 8.0], [1.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 1.0], [8.0, 5.0, 3.0], [5.0, 5.0, 1.0]], 
[[2.0, 7.0, 1.0], [4.0, 2.0, 3.0], [6.0, 3.0, 5.0]], [[2.0, 1.0, 1.0], [3.0, 1.0, 8.0], [7.0, 2.0, 7.0]]], [[[2.0, 3.0, 6.0], 
[4.0, 6.0, 5.0], [5.0, 5.0, 7.0]], [[7.0, 4.0, 4.0], [1.0, 7.0, 5.0], [2.0, 5.0, 5.0]], [[7.0, 8.0, 2.0], [8.0, 2.0, 6.0], 
[3.0, 1.0, 3.0]]], [[[3.0, 4.0, 4.0], [2.0, 5.0, 4.0], [4.0, 5.0, 1.0]], [[1.0, 3.0, 8.0], [8.0, 2.0, 6.0], [1.0, 4.0, 7.0]], 
[[7.0, 1.0, 2.0], [6.0, 3.0, 2.0], [5.0, 6.0, 2.0]]], [[[3.0, 5.0, 6.0], [3.0, 5.0, 3.0], [4.0, 3.0, 5.0]], [[6.0, 6.0, 8.0], 
[3.0, 4.0, 8.0], [7.0, 7.0, 3.0]], [[5.0, 3.0, 3.0], [5.0, 2.0, 2.0], [7.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRLN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 8.0], [3.0, 4.0, 4.0], [4.0, 3.0, 2.0]], [[2.0, 7.0, 6.0], [1.0, 4.0, 5.0], [3.0, 5.0, 
4.0]], [[4.0, 3.0, 6.0], [3.0, 2.0, 5.0], [1.0, 8.0, 8.0]]], [[[8.0, 6.0, 2.0], [6.0, 6.0, 6.0], [8.0, 7.0, 4.0]], [[2.0, 4.0, 
3.0], [3.0, 5.0, 1.0], [5.0, 3.0, 1.0]], [[2.0, 8.0, 1.0], [2.0, 4.0, 1.0], [7.0, 4.0, 2.0]]], [[[1.0, 1.0, 4.0], [2.0, 5.0, 
3.0], [3.0, 1.0, 4.0]], [[5.0, 4.0, 3.0], [1.0, 4.0, 1.0], [5.0, 1.0, 4.0]], [[7.0, 1.0, 2.0], [4.0, 2.0, 8.0], [3.0, 6.0, 
1.0]]], [[[5.0, 3.0, 1.0], [3.0, 8.0, 4.0], [8.0, 2.0, 3.0]], [[4.0, 4.0, 6.0], [7.0, 1.0, 7.0], [5.0, 2.0, 5.0]], [[3.0, 1.0, 
5.0], [2.0, 6.0, 2.0], [4.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 4.0], [6.0, 6.0, 8.0], [8.0, 3.0, 6.0]], [[8.0, 6.0, 2.0], [5.0, 2.0, 
2.0], [2.0, 6.0, 3.0]], [[4.0, 6.0, 1.0], [6.0, 6.0, 5.0], [3.0, 6.0, 1.0]]], [[[6.0, 7.0, 5.0], [8.0, 6.0, 3.0], [4.0, 6.0, 
8.0]], [[7.0, 3.0, 6.0], [6.0, 5.0, 6.0], [8.0, 5.0, 1.0]], [[7.0, 1.0, 4.0], [2.0, 4.0, 8.0], [7.0, 4.0, 5.0]]], [[[2.0, 8.0, 
7.0], [7.0, 4.0, 5.0], [7.0, 6.0, 1.0]], [[4.0, 3.0, 4.0], [5.0, 1.0, 1.0], [1.0, 2.0, 8.0]], [[4.0, 4.0, 5.0], [8.0, 3.0, 
6.0], [3.0, 8.0, 7.0]]], [[[3.0, 5.0, 6.0], [2.0, 5.0, 3.0], [7.0, 5.0, 1.0]], [[6.0, 4.0, 1.0], [3.0, 3.0, 6.0], [2.0, 7.0, 
5.0]], [[4.0, 5.0, 5.0], [3.0, 5.0, 3.0], [2.0, 5.0, 3.0]]]]))
      val=Data(numpy.array([[[[4.0, 1.0, 8.0]]], [[[8.0, 1.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 6.0]]], [[[3.0, 1.0, 6.0]]]]))
      arg[1:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 8.0], [3.0, 4.0, 4.0], [4.0, 3.0, 2.0]], [[2.0, 7.0, 6.0], [1.0, 4.0, 5.0], [3.0, 
5.0, 4.0]], [[4.0, 3.0, 6.0], [3.0, 2.0, 5.0], [1.0, 8.0, 8.0]]], [[[8.0, 6.0, 2.0], [6.0, 6.0, 6.0], [8.0, 7.0, 4.0]], [[2.0, 
4.0, 3.0], [3.0, 5.0, 1.0], [4.0, 1.0, 8.0]], [[2.0, 8.0, 1.0], [2.0, 4.0, 1.0], [7.0, 4.0, 2.0]]], [[[1.0, 1.0, 4.0], [2.0, 
5.0, 3.0], [3.0, 1.0, 4.0]], [[5.0, 4.0, 3.0], [1.0, 4.0, 1.0], [8.0, 1.0, 1.0]], [[7.0, 1.0, 2.0], [4.0, 2.0, 8.0], [3.0, 6.0, 
1.0]]], [[[5.0, 3.0, 1.0], [3.0, 8.0, 4.0], [8.0, 2.0, 3.0]], [[4.0, 4.0, 6.0], [7.0, 1.0, 7.0], [5.0, 2.0, 5.0]], [[3.0, 1.0, 
5.0], [2.0, 6.0, 2.0], [4.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 4.0], [6.0, 6.0, 8.0], [8.0, 3.0, 6.0]], 
[[8.0, 6.0, 2.0], [5.0, 2.0, 2.0], [2.0, 6.0, 3.0]], [[4.0, 6.0, 1.0], [6.0, 6.0, 5.0], [3.0, 6.0, 1.0]]], [[[6.0, 7.0, 5.0], 
[8.0, 6.0, 3.0], [4.0, 6.0, 8.0]], [[7.0, 3.0, 6.0], [6.0, 5.0, 6.0], [3.0, 3.0, 6.0]], [[7.0, 1.0, 4.0], [2.0, 4.0, 8.0], 
[7.0, 4.0, 5.0]]], [[[2.0, 8.0, 7.0], [7.0, 4.0, 5.0], [7.0, 6.0, 1.0]], [[4.0, 3.0, 4.0], [5.0, 1.0, 1.0], [3.0, 1.0, 6.0]], 
[[4.0, 4.0, 5.0], [8.0, 3.0, 6.0], [3.0, 8.0, 7.0]]], [[[3.0, 5.0, 6.0], [2.0, 5.0, 3.0], [7.0, 5.0, 1.0]], [[6.0, 4.0, 1.0], 
[3.0, 3.0, 6.0], [2.0, 7.0, 5.0]], [[4.0, 5.0, 5.0], [3.0, 5.0, 3.0], [2.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRLN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 5.0], [3.0, 7.0, 2.0], [3.0, 7.0, 8.0]], [[2.0, 5.0, 3.0], [3.0, 4.0, 6.0], [7.0, 2.0, 
4.0]], [[1.0, 6.0, 1.0], [7.0, 5.0, 4.0], [1.0, 6.0, 6.0]]], [[[5.0, 6.0, 2.0], [5.0, 4.0, 7.0], [3.0, 1.0, 2.0]], [[8.0, 5.0, 
7.0], [6.0, 6.0, 6.0], [5.0, 1.0, 8.0]], [[6.0, 8.0, 6.0], [2.0, 8.0, 2.0], [6.0, 2.0, 5.0]]], [[[8.0, 1.0, 1.0], [3.0, 8.0, 
5.0], [8.0, 7.0, 7.0]], [[7.0, 6.0, 8.0], [2.0, 2.0, 8.0], [2.0, 6.0, 6.0]], [[5.0, 8.0, 3.0], [2.0, 6.0, 1.0], [1.0, 5.0, 
6.0]]], [[[2.0, 1.0, 6.0], [2.0, 6.0, 5.0], [2.0, 5.0, 2.0]], [[1.0, 3.0, 5.0], [5.0, 7.0, 4.0], [5.0, 2.0, 7.0]], [[3.0, 8.0, 
5.0], [2.0, 4.0, 7.0], [7.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 7.0], [5.0, 5.0, 7.0], [4.0, 7.0, 4.0]], [[1.0, 3.0, 7.0], [4.0, 4.0, 
7.0], [4.0, 5.0, 4.0]], [[8.0, 3.0, 6.0], [2.0, 8.0, 1.0], [5.0, 7.0, 6.0]]], [[[8.0, 8.0, 8.0], [4.0, 8.0, 4.0], [2.0, 5.0, 
3.0]], [[6.0, 8.0, 7.0], [3.0, 4.0, 4.0], [8.0, 8.0, 8.0]], [[7.0, 7.0, 6.0], [4.0, 8.0, 8.0], [1.0, 7.0, 7.0]]], [[[2.0, 3.0, 
6.0], [5.0, 5.0, 3.0], [6.0, 8.0, 4.0]], [[1.0, 6.0, 4.0], [5.0, 3.0, 5.0], [2.0, 5.0, 4.0]], [[3.0, 1.0, 7.0], [6.0, 3.0, 
6.0], [1.0, 3.0, 5.0]]], [[[1.0, 7.0, 8.0], [1.0, 3.0, 5.0], [4.0, 2.0, 5.0]], [[6.0, 5.0, 8.0], [8.0, 4.0, 7.0], [4.0, 8.0, 
1.0]], [[8.0, 8.0, 2.0], [5.0, 5.0, 5.0], [6.0, 5.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 3.0, 4.0]]], [[[8.0, 6.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 5.0, 6.0]]], 
[[[4.0, 5.0, 7.0]]]])
      arg[1:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 5.0], [3.0, 7.0, 2.0], [3.0, 7.0, 8.0]], [[2.0, 5.0, 3.0], [3.0, 4.0, 6.0], [7.0, 
2.0, 4.0]], [[1.0, 6.0, 1.0], [7.0, 5.0, 4.0], [1.0, 6.0, 6.0]]], [[[5.0, 6.0, 2.0], [5.0, 4.0, 7.0], [3.0, 1.0, 2.0]], [[8.0, 
5.0, 7.0], [6.0, 6.0, 6.0], [5.0, 3.0, 4.0]], [[6.0, 8.0, 6.0], [2.0, 8.0, 2.0], [6.0, 2.0, 5.0]]], [[[8.0, 1.0, 1.0], [3.0, 
8.0, 5.0], [8.0, 7.0, 7.0]], [[7.0, 6.0, 8.0], [2.0, 2.0, 8.0], [8.0, 6.0, 2.0]], [[5.0, 8.0, 3.0], [2.0, 6.0, 1.0], [1.0, 5.0, 
6.0]]], [[[2.0, 1.0, 6.0], [2.0, 6.0, 5.0], [2.0, 5.0, 2.0]], [[1.0, 3.0, 5.0], [5.0, 7.0, 4.0], [5.0, 2.0, 7.0]], [[3.0, 8.0, 
5.0], [2.0, 4.0, 7.0], [7.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 7.0], [5.0, 5.0, 7.0], [4.0, 7.0, 4.0]], 
[[1.0, 3.0, 7.0], [4.0, 4.0, 7.0], [4.0, 5.0, 4.0]], [[8.0, 3.0, 6.0], [2.0, 8.0, 1.0], [5.0, 7.0, 6.0]]], [[[8.0, 8.0, 8.0], 
[4.0, 8.0, 4.0], [2.0, 5.0, 3.0]], [[6.0, 8.0, 7.0], [3.0, 4.0, 4.0], [2.0, 5.0, 6.0]], [[7.0, 7.0, 6.0], [4.0, 8.0, 8.0], 
[1.0, 7.0, 7.0]]], [[[2.0, 3.0, 6.0], [5.0, 5.0, 3.0], [6.0, 8.0, 4.0]], [[1.0, 6.0, 4.0], [5.0, 3.0, 5.0], [4.0, 5.0, 7.0]], 
[[3.0, 1.0, 7.0], [6.0, 3.0, 6.0], [1.0, 3.0, 5.0]]], [[[1.0, 7.0, 8.0], [1.0, 3.0, 5.0], [4.0, 2.0, 5.0]], [[6.0, 5.0, 8.0], 
[8.0, 4.0, 7.0], [4.0, 8.0, 1.0]], [[8.0, 8.0, 2.0], [5.0, 5.0, 5.0], [6.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UCUN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 4.0], [7.0, 7.0, 8.0], [2.0, 6.0, 1.0]], [[1.0, 2.0, 2.0], [5.0, 2.0, 7.0], [7.0, 8.0, 
6.0]], [[2.0, 7.0, 7.0], [7.0, 8.0, 2.0], [4.0, 5.0, 5.0]]], [[[1.0, 8.0, 1.0], [1.0, 6.0, 2.0], [6.0, 5.0, 2.0]], [[4.0, 8.0, 
8.0], [2.0, 8.0, 2.0], [1.0, 4.0, 6.0]], [[3.0, 3.0, 1.0], [3.0, 6.0, 3.0], [8.0, 4.0, 3.0]]], [[[5.0, 2.0, 2.0], [2.0, 7.0, 
4.0], [4.0, 4.0, 1.0]], [[2.0, 6.0, 7.0], [5.0, 6.0, 1.0], [5.0, 6.0, 4.0]], [[2.0, 1.0, 3.0], [6.0, 6.0, 5.0], [3.0, 8.0, 
1.0]]], [[[6.0, 2.0, 6.0], [8.0, 7.0, 2.0], [7.0, 6.0, 1.0]], [[5.0, 1.0, 8.0], [4.0, 3.0, 3.0], [4.0, 6.0, 3.0]], [[8.0, 1.0, 
4.0], [2.0, 1.0, 1.0], [1.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 2.0], [2.0, 2.0, 3.0], [1.0, 7.0, 5.0]], [[8.0, 2.0, 6.0], [5.0, 4.0, 
8.0], [8.0, 5.0, 6.0]], [[2.0, 1.0, 3.0], [1.0, 3.0, 1.0], [8.0, 2.0, 4.0]]], [[[6.0, 1.0, 8.0], [6.0, 5.0, 4.0], [1.0, 7.0, 
6.0]], [[2.0, 7.0, 6.0], [1.0, 5.0, 4.0], [4.0, 8.0, 6.0]], [[1.0, 4.0, 1.0], [4.0, 3.0, 1.0], [5.0, 2.0, 8.0]]], [[[6.0, 1.0, 
2.0], [5.0, 2.0, 1.0], [6.0, 6.0, 4.0]], [[6.0, 6.0, 8.0], [8.0, 4.0, 3.0], [6.0, 5.0, 7.0]], [[7.0, 4.0, 3.0], [2.0, 8.0, 
4.0], [7.0, 1.0, 2.0]]], [[[6.0, 4.0, 2.0], [6.0, 8.0, 5.0], [6.0, 6.0, 3.0]], [[7.0, 7.0, 8.0], [2.0, 1.0, 4.0], [4.0, 5.0, 
7.0]], [[7.0, 7.0, 6.0], [1.0, 2.0, 6.0], [6.0, 1.0, 8.0]]]]))
      val=numpy.array([[[[6.0, 5.0, 2.0], [5.0, 3.0, 5.0]], [[4.0, 5.0, 3.0], [1.0, 2.0, 2.0]], [[2.0, 5.0, 7.0], [1.0, 7.0, 
6.0]]], [[[4.0, 4.0, 4.0], [2.0, 2.0, 8.0]], [[8.0, 3.0, 2.0], [4.0, 3.0, 2.0]], [[1.0, 8.0, 2.0], [7.0, 5.0, 8.0]]], [[[8.0, 
7.0, 6.0], [6.0, 2.0, 7.0]], [[5.0, 8.0, 5.0], [3.0, 4.0, 1.0]], [[5.0, 5.0, 6.0], [6.0, 4.0, 6.0]]]])
      arg[:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 2.0], [5.0, 3.0, 5.0], [2.0, 6.0, 1.0]], [[4.0, 5.0, 3.0], [1.0, 2.0, 2.0], [7.0, 
8.0, 6.0]], [[2.0, 5.0, 7.0], [1.0, 7.0, 6.0], [4.0, 5.0, 5.0]]], [[[4.0, 4.0, 4.0], [2.0, 2.0, 8.0], [6.0, 5.0, 2.0]], [[8.0, 
3.0, 2.0], [4.0, 3.0, 2.0], [1.0, 4.0, 6.0]], [[1.0, 8.0, 2.0], [7.0, 5.0, 8.0], [8.0, 4.0, 3.0]]], [[[8.0, 7.0, 6.0], [6.0, 
2.0, 7.0], [4.0, 4.0, 1.0]], [[5.0, 8.0, 5.0], [3.0, 4.0, 1.0], [5.0, 6.0, 4.0]], [[5.0, 5.0, 6.0], [6.0, 4.0, 6.0], [3.0, 8.0, 
1.0]]], [[[6.0, 2.0, 6.0], [8.0, 7.0, 2.0], [7.0, 6.0, 1.0]], [[5.0, 1.0, 8.0], [4.0, 3.0, 3.0], [4.0, 6.0, 3.0]], [[8.0, 1.0, 
4.0], [2.0, 1.0, 1.0], [1.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 2.0], [5.0, 3.0, 5.0], [1.0, 7.0, 5.0]], 
[[4.0, 5.0, 3.0], [1.0, 2.0, 2.0], [8.0, 5.0, 6.0]], [[2.0, 5.0, 7.0], [1.0, 7.0, 6.0], [8.0, 2.0, 4.0]]], [[[4.0, 4.0, 4.0], 
[2.0, 2.0, 8.0], [1.0, 7.0, 6.0]], [[8.0, 3.0, 2.0], [4.0, 3.0, 2.0], [4.0, 8.0, 6.0]], [[1.0, 8.0, 2.0], [7.0, 5.0, 8.0], 
[5.0, 2.0, 8.0]]], [[[8.0, 7.0, 6.0], [6.0, 2.0, 7.0], [6.0, 6.0, 4.0]], [[5.0, 8.0, 5.0], [3.0, 4.0, 1.0], [6.0, 5.0, 7.0]], 
[[5.0, 5.0, 6.0], [6.0, 4.0, 6.0], [7.0, 1.0, 2.0]]], [[[6.0, 4.0, 2.0], [6.0, 8.0, 5.0], [6.0, 6.0, 3.0]], [[7.0, 7.0, 8.0], 
[2.0, 1.0, 4.0], [4.0, 5.0, 7.0]], [[7.0, 7.0, 6.0], [1.0, 2.0, 6.0], [6.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UCUN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 2.0], [6.0, 1.0, 2.0], [5.0, 1.0, 1.0]], [[2.0, 6.0, 1.0], [4.0, 1.0, 4.0], [7.0, 7.0, 
3.0]], [[4.0, 4.0, 6.0], [5.0, 4.0, 4.0], [8.0, 2.0, 3.0]]], [[[1.0, 7.0, 7.0], [8.0, 6.0, 8.0], [2.0, 5.0, 6.0]], [[6.0, 4.0, 
1.0], [2.0, 6.0, 7.0], [6.0, 4.0, 1.0]], [[1.0, 2.0, 6.0], [8.0, 4.0, 2.0], [1.0, 7.0, 3.0]]], [[[2.0, 1.0, 2.0], [5.0, 1.0, 
2.0], [4.0, 1.0, 8.0]], [[4.0, 6.0, 7.0], [1.0, 6.0, 2.0], [3.0, 4.0, 7.0]], [[6.0, 4.0, 4.0], [8.0, 7.0, 7.0], [7.0, 4.0, 
7.0]]], [[[7.0, 7.0, 3.0], [4.0, 8.0, 6.0], [1.0, 6.0, 1.0]], [[4.0, 2.0, 3.0], [1.0, 3.0, 1.0], [4.0, 3.0, 8.0]], [[5.0, 5.0, 
5.0], [1.0, 2.0, 6.0], [4.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 8.0], [1.0, 4.0, 5.0], [5.0, 6.0, 5.0]], [[8.0, 2.0, 8.0], [1.0, 2.0, 
8.0], [5.0, 6.0, 6.0]], [[5.0, 5.0, 2.0], [2.0, 4.0, 4.0], [1.0, 6.0, 2.0]]], [[[7.0, 6.0, 5.0], [7.0, 3.0, 1.0], [3.0, 3.0, 
4.0]], [[5.0, 1.0, 6.0], [7.0, 7.0, 5.0], [5.0, 4.0, 4.0]], [[3.0, 5.0, 3.0], [3.0, 3.0, 5.0], [7.0, 3.0, 1.0]]], [[[3.0, 6.0, 
8.0], [7.0, 3.0, 6.0], [5.0, 8.0, 3.0]], [[2.0, 5.0, 5.0], [6.0, 6.0, 2.0], [5.0, 3.0, 4.0]], [[7.0, 4.0, 2.0], [5.0, 3.0, 
2.0], [4.0, 8.0, 2.0]]], [[[1.0, 5.0, 8.0], [8.0, 1.0, 6.0], [1.0, 2.0, 5.0]], [[7.0, 1.0, 2.0], [3.0, 8.0, 4.0], [1.0, 7.0, 
8.0]], [[2.0, 3.0, 3.0], [1.0, 4.0, 6.0], [5.0, 5.0, 4.0]]]]))
      val=Data(numpy.array([[[[8.0, 5.0, 8.0], [4.0, 8.0, 5.0]], [[7.0, 2.0, 5.0], [2.0, 5.0, 5.0]], [[5.0, 4.0, 6.0], [6.0, 
4.0, 4.0]]], [[[1.0, 1.0, 2.0], [2.0, 5.0, 1.0]], [[3.0, 6.0, 1.0], [1.0, 7.0, 6.0]], [[7.0, 8.0, 2.0], [7.0, 8.0, 4.0]]], 
[[[5.0, 4.0, 5.0], [1.0, 5.0, 4.0]], [[6.0, 4.0, 3.0], [1.0, 6.0, 4.0]], [[8.0, 8.0, 1.0], [7.0, 3.0, 
6.0]]]]),self.functionspace)
      arg[:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 8.0], [4.0, 8.0, 5.0], [5.0, 1.0, 1.0]], [[7.0, 2.0, 5.0], [2.0, 5.0, 5.0], [7.0, 
7.0, 3.0]], [[5.0, 4.0, 6.0], [6.0, 4.0, 4.0], [8.0, 2.0, 3.0]]], [[[1.0, 1.0, 2.0], [2.0, 5.0, 1.0], [2.0, 5.0, 6.0]], [[3.0, 
6.0, 1.0], [1.0, 7.0, 6.0], [6.0, 4.0, 1.0]], [[7.0, 8.0, 2.0], [7.0, 8.0, 4.0], [1.0, 7.0, 3.0]]], [[[5.0, 4.0, 5.0], [1.0, 
5.0, 4.0], [4.0, 1.0, 8.0]], [[6.0, 4.0, 3.0], [1.0, 6.0, 4.0], [3.0, 4.0, 7.0]], [[8.0, 8.0, 1.0], [7.0, 3.0, 6.0], [7.0, 4.0, 
7.0]]], [[[7.0, 7.0, 3.0], [4.0, 8.0, 6.0], [1.0, 6.0, 1.0]], [[4.0, 2.0, 3.0], [1.0, 3.0, 1.0], [4.0, 3.0, 8.0]], [[5.0, 5.0, 
5.0], [1.0, 2.0, 6.0], [4.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 8.0], [4.0, 8.0, 5.0], [5.0, 6.0, 5.0]], 
[[7.0, 2.0, 5.0], [2.0, 5.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 4.0, 6.0], [6.0, 4.0, 4.0], [1.0, 6.0, 2.0]]], [[[1.0, 1.0, 2.0], 
[2.0, 5.0, 1.0], [3.0, 3.0, 4.0]], [[3.0, 6.0, 1.0], [1.0, 7.0, 6.0], [5.0, 4.0, 4.0]], [[7.0, 8.0, 2.0], [7.0, 8.0, 4.0], 
[7.0, 3.0, 1.0]]], [[[5.0, 4.0, 5.0], [1.0, 5.0, 4.0], [5.0, 8.0, 3.0]], [[6.0, 4.0, 3.0], [1.0, 6.0, 4.0], [5.0, 3.0, 4.0]], 
[[8.0, 8.0, 1.0], [7.0, 3.0, 6.0], [4.0, 8.0, 2.0]]], [[[1.0, 5.0, 8.0], [8.0, 1.0, 6.0], [1.0, 2.0, 5.0]], [[7.0, 1.0, 2.0], 
[3.0, 8.0, 4.0], [1.0, 7.0, 8.0]], [[2.0, 3.0, 3.0], [1.0, 4.0, 6.0], [5.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UCUN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 4.0], [4.0, 6.0, 8.0], [6.0, 7.0, 2.0]], [[2.0, 3.0, 7.0], [1.0, 2.0, 3.0], [1.0, 2.0, 
1.0]], [[3.0, 1.0, 2.0], [3.0, 6.0, 7.0], [3.0, 3.0, 7.0]]], [[[1.0, 8.0, 7.0], [3.0, 4.0, 6.0], [1.0, 7.0, 7.0]], [[6.0, 4.0, 
2.0], [8.0, 5.0, 8.0], [6.0, 5.0, 4.0]], [[8.0, 6.0, 6.0], [8.0, 5.0, 7.0], [1.0, 5.0, 6.0]]], [[[2.0, 1.0, 3.0], [2.0, 1.0, 
3.0], [8.0, 5.0, 4.0]], [[5.0, 2.0, 1.0], [5.0, 5.0, 6.0], [2.0, 3.0, 4.0]], [[1.0, 1.0, 7.0], [6.0, 5.0, 1.0], [5.0, 1.0, 
2.0]]], [[[8.0, 1.0, 7.0], [2.0, 3.0, 7.0], [2.0, 7.0, 1.0]], [[1.0, 6.0, 5.0], [2.0, 8.0, 1.0], [8.0, 2.0, 6.0]], [[2.0, 5.0, 
6.0], [6.0, 4.0, 1.0], [5.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 2.0], [1.0, 7.0, 2.0], [4.0, 7.0, 2.0]], [[5.0, 8.0, 3.0], [3.0, 6.0, 
6.0], [7.0, 6.0, 7.0]], [[6.0, 6.0, 6.0], [2.0, 7.0, 7.0], [6.0, 4.0, 1.0]]], [[[4.0, 8.0, 6.0], [2.0, 2.0, 5.0], [8.0, 6.0, 
3.0]], [[5.0, 3.0, 7.0], [4.0, 6.0, 8.0], [3.0, 4.0, 7.0]], [[2.0, 3.0, 2.0], [7.0, 8.0, 1.0], [7.0, 3.0, 4.0]]], [[[2.0, 7.0, 
2.0], [7.0, 7.0, 3.0], [6.0, 2.0, 7.0]], [[5.0, 5.0, 3.0], [7.0, 6.0, 7.0], [3.0, 2.0, 7.0]], [[7.0, 1.0, 2.0], [6.0, 4.0, 
2.0], [1.0, 2.0, 2.0]]], [[[2.0, 2.0, 1.0], [6.0, 3.0, 5.0], [4.0, 5.0, 2.0]], [[5.0, 7.0, 1.0], [7.0, 1.0, 8.0], [1.0, 6.0, 
3.0]], [[4.0, 8.0, 1.0], [4.0, 2.0, 1.0], [1.0, 1.0, 5.0]]]]))
      val=Data(numpy.array([[[[6.0, 3.0, 6.0], [3.0, 3.0, 8.0]], [[1.0, 2.0, 1.0], [6.0, 6.0, 4.0]], [[2.0, 3.0, 6.0], [6.0, 
3.0, 2.0]]], [[[5.0, 6.0, 2.0], [2.0, 6.0, 1.0]], [[8.0, 4.0, 5.0], [5.0, 6.0, 3.0]], [[6.0, 8.0, 6.0], [4.0, 4.0, 7.0]]], 
[[[3.0, 2.0, 8.0], [7.0, 8.0, 2.0]], [[1.0, 1.0, 6.0], [1.0, 8.0, 5.0]], [[4.0, 3.0, 6.0], [8.0, 2.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 4.0], [5.0, 4.0, 2.0]], [[5.0, 8.0, 8.0], [2.0, 8.0, 6.0]], [[3.0, 3.0, 
5.0], [4.0, 1.0, 8.0]]], [[[6.0, 2.0, 5.0], [3.0, 5.0, 3.0]], [[8.0, 3.0, 3.0], [5.0, 7.0, 6.0]], [[5.0, 5.0, 1.0], [5.0, 2.0, 
8.0]]], [[[6.0, 1.0, 2.0], [4.0, 7.0, 3.0]], [[5.0, 6.0, 5.0], [1.0, 1.0, 1.0]], [[6.0, 3.0, 8.0], [4.0, 7.0, 4.0]]]]))
      arg[:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 6.0], [3.0, 3.0, 8.0], [6.0, 7.0, 2.0]], [[1.0, 2.0, 1.0], [6.0, 6.0, 4.0], [1.0, 
2.0, 1.0]], [[2.0, 3.0, 6.0], [6.0, 3.0, 2.0], [3.0, 3.0, 7.0]]], [[[5.0, 6.0, 2.0], [2.0, 6.0, 1.0], [1.0, 7.0, 7.0]], [[8.0, 
4.0, 5.0], [5.0, 6.0, 3.0], [6.0, 5.0, 4.0]], [[6.0, 8.0, 6.0], [4.0, 4.0, 7.0], [1.0, 5.0, 6.0]]], [[[3.0, 2.0, 8.0], [7.0, 
8.0, 2.0], [8.0, 5.0, 4.0]], [[1.0, 1.0, 6.0], [1.0, 8.0, 5.0], [2.0, 3.0, 4.0]], [[4.0, 3.0, 6.0], [8.0, 2.0, 2.0], [5.0, 1.0, 
2.0]]], [[[8.0, 1.0, 7.0], [2.0, 3.0, 7.0], [2.0, 7.0, 1.0]], [[1.0, 6.0, 5.0], [2.0, 8.0, 1.0], [8.0, 2.0, 6.0]], [[2.0, 5.0, 
6.0], [6.0, 4.0, 1.0], [5.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 4.0], [5.0, 4.0, 2.0], [4.0, 7.0, 2.0]], 
[[5.0, 8.0, 8.0], [2.0, 8.0, 6.0], [7.0, 6.0, 7.0]], [[3.0, 3.0, 5.0], [4.0, 1.0, 8.0], [6.0, 4.0, 1.0]]], [[[6.0, 2.0, 5.0], 
[3.0, 5.0, 3.0], [8.0, 6.0, 3.0]], [[8.0, 3.0, 3.0], [5.0, 7.0, 6.0], [3.0, 4.0, 7.0]], [[5.0, 5.0, 1.0], [5.0, 2.0, 8.0], 
[7.0, 3.0, 4.0]]], [[[6.0, 1.0, 2.0], [4.0, 7.0, 3.0], [6.0, 2.0, 7.0]], [[5.0, 6.0, 5.0], [1.0, 1.0, 1.0], [3.0, 2.0, 7.0]], 
[[6.0, 3.0, 8.0], [4.0, 7.0, 4.0], [1.0, 2.0, 2.0]]], [[[2.0, 2.0, 1.0], [6.0, 3.0, 5.0], [4.0, 5.0, 2.0]], [[5.0, 7.0, 1.0], 
[7.0, 1.0, 8.0], [1.0, 6.0, 3.0]], [[4.0, 8.0, 1.0], [4.0, 2.0, 1.0], [1.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UCUN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 2.0], [2.0, 7.0, 6.0], [8.0, 5.0, 8.0]], [[5.0, 8.0, 7.0], [2.0, 4.0, 7.0], [2.0, 2.0, 
5.0]], [[1.0, 8.0, 7.0], [8.0, 5.0, 2.0], [7.0, 2.0, 8.0]]], [[[7.0, 6.0, 8.0], [8.0, 2.0, 5.0], [6.0, 6.0, 4.0]], [[5.0, 8.0, 
1.0], [1.0, 7.0, 7.0], [2.0, 1.0, 6.0]], [[3.0, 6.0, 2.0], [1.0, 4.0, 8.0], [3.0, 4.0, 1.0]]], [[[3.0, 1.0, 8.0], [4.0, 6.0, 
2.0], [7.0, 2.0, 2.0]], [[8.0, 2.0, 4.0], [1.0, 4.0, 4.0], [7.0, 3.0, 5.0]], [[7.0, 5.0, 7.0], [5.0, 6.0, 7.0], [2.0, 1.0, 
6.0]]], [[[5.0, 4.0, 2.0], [6.0, 2.0, 2.0], [5.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], [5.0, 8.0, 4.0], [1.0, 7.0, 7.0]], [[7.0, 6.0, 
1.0], [7.0, 7.0, 5.0], [1.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 7.0], [6.0, 2.0, 3.0], [4.0, 1.0, 6.0]], [[5.0, 6.0, 3.0], [7.0, 3.0, 
2.0], [1.0, 6.0, 5.0]], [[7.0, 2.0, 7.0], [4.0, 7.0, 7.0], [8.0, 4.0, 8.0]]], [[[3.0, 4.0, 3.0], [2.0, 4.0, 4.0], [8.0, 8.0, 
8.0]], [[4.0, 3.0, 2.0], [8.0, 1.0, 4.0], [8.0, 4.0, 7.0]], [[4.0, 3.0, 1.0], [2.0, 8.0, 1.0], [3.0, 7.0, 8.0]]], [[[5.0, 7.0, 
1.0], [4.0, 5.0, 3.0], [5.0, 4.0, 6.0]], [[2.0, 4.0, 1.0], [8.0, 4.0, 6.0], [5.0, 5.0, 1.0]], [[7.0, 1.0, 1.0], [3.0, 5.0, 
6.0], [5.0, 4.0, 1.0]]], [[[5.0, 3.0, 7.0], [1.0, 7.0, 6.0], [5.0, 1.0, 4.0]], [[1.0, 4.0, 5.0], [3.0, 8.0, 2.0], [3.0, 1.0, 
4.0]], [[3.0, 6.0, 5.0], [2.0, 7.0, 8.0], [8.0, 5.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 5.0, 4.0], [4.0, 1.0, 1.0]], [[6.0, 5.0, 7.0], [6.0, 7.0, 1.0]], [[1.0, 2.0, 2.0], [7.0, 
8.0, 5.0]]], [[[7.0, 3.0, 5.0], [3.0, 2.0, 3.0]], [[6.0, 8.0, 1.0], [2.0, 1.0, 1.0]], [[3.0, 2.0, 1.0], [2.0, 3.0, 1.0]]], 
[[[2.0, 4.0, 8.0], [1.0, 3.0, 7.0]], [[5.0, 5.0, 2.0], [6.0, 3.0, 5.0]], [[3.0, 1.0, 6.0], [4.0, 8.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 2.0, 5.0], [6.0, 8.0, 3.0]], [[1.0, 8.0, 4.0], [3.0, 2.0, 2.0]], [[7.0, 1.0, 6.0], 
[8.0, 1.0, 8.0]]], [[[7.0, 6.0, 8.0], [5.0, 3.0, 2.0]], [[2.0, 2.0, 7.0], [7.0, 1.0, 6.0]], [[4.0, 2.0, 6.0], [7.0, 4.0, 
4.0]]], [[[2.0, 5.0, 3.0], [8.0, 4.0, 2.0]], [[3.0, 1.0, 5.0], [3.0, 6.0, 4.0]], [[8.0, 6.0, 7.0], [5.0, 6.0, 5.0]]]])
      arg[:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [4.0, 1.0, 1.0], [8.0, 5.0, 8.0]], [[6.0, 5.0, 7.0], [6.0, 7.0, 1.0], [2.0, 
2.0, 5.0]], [[1.0, 2.0, 2.0], [7.0, 8.0, 5.0], [7.0, 2.0, 8.0]]], [[[7.0, 3.0, 5.0], [3.0, 2.0, 3.0], [6.0, 6.0, 4.0]], [[6.0, 
8.0, 1.0], [2.0, 1.0, 1.0], [2.0, 1.0, 6.0]], [[3.0, 2.0, 1.0], [2.0, 3.0, 1.0], [3.0, 4.0, 1.0]]], [[[2.0, 4.0, 8.0], [1.0, 
3.0, 7.0], [7.0, 2.0, 2.0]], [[5.0, 5.0, 2.0], [6.0, 3.0, 5.0], [7.0, 3.0, 5.0]], [[3.0, 1.0, 6.0], [4.0, 8.0, 1.0], [2.0, 1.0, 
6.0]]], [[[5.0, 4.0, 2.0], [6.0, 2.0, 2.0], [5.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], [5.0, 8.0, 4.0], [1.0, 7.0, 7.0]], [[7.0, 6.0, 
1.0], [7.0, 7.0, 5.0], [1.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 5.0], [6.0, 8.0, 3.0], [4.0, 1.0, 6.0]], 
[[1.0, 8.0, 4.0], [3.0, 2.0, 2.0], [1.0, 6.0, 5.0]], [[7.0, 1.0, 6.0], [8.0, 1.0, 8.0], [8.0, 4.0, 8.0]]], [[[7.0, 6.0, 8.0], 
[5.0, 3.0, 2.0], [8.0, 8.0, 8.0]], [[2.0, 2.0, 7.0], [7.0, 1.0, 6.0], [8.0, 4.0, 7.0]], [[4.0, 2.0, 6.0], [7.0, 4.0, 4.0], 
[3.0, 7.0, 8.0]]], [[[2.0, 5.0, 3.0], [8.0, 4.0, 2.0], [5.0, 4.0, 6.0]], [[3.0, 1.0, 5.0], [3.0, 6.0, 4.0], [5.0, 5.0, 1.0]], 
[[8.0, 6.0, 7.0], [5.0, 6.0, 5.0], [5.0, 4.0, 1.0]]], [[[5.0, 3.0, 7.0], [1.0, 7.0, 6.0], [5.0, 1.0, 4.0]], [[1.0, 4.0, 5.0], 
[3.0, 8.0, 2.0], [3.0, 1.0, 4.0]], [[3.0, 6.0, 5.0], [2.0, 7.0, 8.0], [8.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CUUN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 8.0], [4.0, 7.0, 5.0], [1.0, 3.0, 7.0]], [[4.0, 4.0, 4.0], [4.0, 5.0, 2.0], [1.0, 6.0, 
8.0]], [[4.0, 7.0, 3.0], [2.0, 3.0, 6.0], [8.0, 2.0, 1.0]]], [[[5.0, 3.0, 7.0], [1.0, 2.0, 7.0], [2.0, 3.0, 6.0]], [[5.0, 1.0, 
4.0], [8.0, 6.0, 1.0], [6.0, 4.0, 6.0]], [[6.0, 4.0, 4.0], [6.0, 2.0, 4.0], [5.0, 6.0, 3.0]]], [[[1.0, 2.0, 8.0], [1.0, 4.0, 
4.0], [2.0, 5.0, 8.0]], [[2.0, 4.0, 6.0], [8.0, 6.0, 6.0], [1.0, 5.0, 4.0]], [[5.0, 7.0, 1.0], [7.0, 7.0, 6.0], [1.0, 7.0, 
1.0]]], [[[3.0, 4.0, 3.0], [5.0, 6.0, 6.0], [3.0, 4.0, 2.0]], [[4.0, 2.0, 8.0], [4.0, 5.0, 3.0], [7.0, 5.0, 4.0]], [[7.0, 5.0, 
2.0], [5.0, 1.0, 5.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 2.0], [4.0, 4.0, 8.0], [7.0, 1.0, 5.0]], [[4.0, 5.0, 4.0], [8.0, 1.0, 
7.0], [5.0, 5.0, 8.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 7.0], [4.0, 2.0, 6.0]]], [[[4.0, 1.0, 5.0], [2.0, 1.0, 5.0], [3.0, 7.0, 
6.0]], [[4.0, 3.0, 6.0], [7.0, 7.0, 5.0], [1.0, 7.0, 6.0]], [[3.0, 4.0, 6.0], [6.0, 7.0, 7.0], [5.0, 8.0, 5.0]]], [[[6.0, 6.0, 
8.0], [4.0, 3.0, 3.0], [5.0, 8.0, 4.0]], [[3.0, 7.0, 2.0], [1.0, 7.0, 2.0], [2.0, 4.0, 2.0]], [[7.0, 6.0, 3.0], [2.0, 3.0, 
5.0], [8.0, 5.0, 7.0]]], [[[1.0, 1.0, 3.0], [2.0, 7.0, 4.0], [7.0, 4.0, 8.0]], [[8.0, 4.0, 5.0], [8.0, 6.0, 8.0], [3.0, 4.0, 
4.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 8.0], [4.0, 5.0, 6.0]]]]))
      val=numpy.array([[[[6.0, 6.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 1.0, 5.0], [5.0, 7.0, 3.0]]], [[[4.0, 3.0, 6.0], [2.0, 2.0, 
6.0]], [[7.0, 6.0, 8.0], [8.0, 5.0, 8.0]]], [[[7.0, 8.0, 7.0], [5.0, 7.0, 7.0]], [[7.0, 6.0, 7.0], [7.0, 6.0, 5.0]]], [[[4.0, 
5.0, 4.0], [4.0, 4.0, 3.0]], [[8.0, 4.0, 7.0], [2.0, 3.0, 5.0]]]])
      arg[:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 3.0], [6.0, 4.0, 5.0], [1.0, 3.0, 7.0]], [[8.0, 1.0, 5.0], [5.0, 7.0, 3.0], [1.0, 
6.0, 8.0]], [[4.0, 7.0, 3.0], [2.0, 3.0, 6.0], [8.0, 2.0, 1.0]]], [[[4.0, 3.0, 6.0], [2.0, 2.0, 6.0], [2.0, 3.0, 6.0]], [[7.0, 
6.0, 8.0], [8.0, 5.0, 8.0], [6.0, 4.0, 6.0]], [[6.0, 4.0, 4.0], [6.0, 2.0, 4.0], [5.0, 6.0, 3.0]]], [[[7.0, 8.0, 7.0], [5.0, 
7.0, 7.0], [2.0, 5.0, 8.0]], [[7.0, 6.0, 7.0], [7.0, 6.0, 5.0], [1.0, 5.0, 4.0]], [[5.0, 7.0, 1.0], [7.0, 7.0, 6.0], [1.0, 7.0, 
1.0]]], [[[4.0, 5.0, 4.0], [4.0, 4.0, 3.0], [3.0, 4.0, 2.0]], [[8.0, 4.0, 7.0], [2.0, 3.0, 5.0], [7.0, 5.0, 4.0]], [[7.0, 5.0, 
2.0], [5.0, 1.0, 5.0], [8.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 3.0], [6.0, 4.0, 5.0], [7.0, 1.0, 5.0]], 
[[8.0, 1.0, 5.0], [5.0, 7.0, 3.0], [5.0, 5.0, 8.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 7.0], [4.0, 2.0, 6.0]]], [[[4.0, 3.0, 6.0], 
[2.0, 2.0, 6.0], [3.0, 7.0, 6.0]], [[7.0, 6.0, 8.0], [8.0, 5.0, 8.0], [1.0, 7.0, 6.0]], [[3.0, 4.0, 6.0], [6.0, 7.0, 7.0], 
[5.0, 8.0, 5.0]]], [[[7.0, 8.0, 7.0], [5.0, 7.0, 7.0], [5.0, 8.0, 4.0]], [[7.0, 6.0, 7.0], [7.0, 6.0, 5.0], [2.0, 4.0, 2.0]], 
[[7.0, 6.0, 3.0], [2.0, 3.0, 5.0], [8.0, 5.0, 7.0]]], [[[4.0, 5.0, 4.0], [4.0, 4.0, 3.0], [7.0, 4.0, 8.0]], [[8.0, 4.0, 7.0], 
[2.0, 3.0, 5.0], [3.0, 4.0, 4.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 8.0], [4.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CUUN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 4.0], [4.0, 4.0, 4.0], [8.0, 6.0, 5.0]], [[5.0, 2.0, 2.0], [3.0, 1.0, 6.0], [6.0, 6.0, 
5.0]], [[6.0, 3.0, 5.0], [4.0, 4.0, 6.0], [2.0, 4.0, 8.0]]], [[[6.0, 6.0, 1.0], [7.0, 7.0, 8.0], [2.0, 8.0, 4.0]], [[2.0, 8.0, 
6.0], [7.0, 6.0, 1.0], [2.0, 8.0, 1.0]], [[1.0, 7.0, 7.0], [5.0, 7.0, 5.0], [6.0, 1.0, 7.0]]], [[[4.0, 7.0, 8.0], [2.0, 4.0, 
4.0], [3.0, 4.0, 8.0]], [[2.0, 2.0, 2.0], [4.0, 5.0, 7.0], [2.0, 1.0, 4.0]], [[4.0, 8.0, 4.0], [5.0, 6.0, 6.0], [3.0, 8.0, 
6.0]]], [[[5.0, 5.0, 1.0], [4.0, 6.0, 7.0], [4.0, 8.0, 2.0]], [[7.0, 2.0, 7.0], [4.0, 1.0, 5.0], [2.0, 5.0, 1.0]], [[3.0, 5.0, 
7.0], [8.0, 7.0, 1.0], [7.0, 3.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 8.0], [4.0, 2.0, 6.0], [5.0, 8.0, 5.0]], [[4.0, 1.0, 4.0], [2.0, 1.0, 
8.0], [3.0, 1.0, 5.0]], [[2.0, 1.0, 1.0], [1.0, 3.0, 5.0], [1.0, 6.0, 6.0]]], [[[5.0, 4.0, 5.0], [3.0, 3.0, 5.0], [2.0, 1.0, 
6.0]], [[3.0, 2.0, 6.0], [1.0, 7.0, 1.0], [1.0, 2.0, 2.0]], [[1.0, 3.0, 7.0], [8.0, 1.0, 2.0], [3.0, 7.0, 8.0]]], [[[5.0, 7.0, 
2.0], [6.0, 8.0, 1.0], [4.0, 4.0, 7.0]], [[1.0, 4.0, 8.0], [8.0, 2.0, 1.0], [6.0, 4.0, 7.0]], [[7.0, 7.0, 8.0], [5.0, 4.0, 
7.0], [4.0, 3.0, 7.0]]], [[[7.0, 2.0, 6.0], [3.0, 7.0, 3.0], [1.0, 2.0, 2.0]], [[5.0, 2.0, 2.0], [2.0, 4.0, 7.0], [3.0, 1.0, 
4.0]], [[4.0, 6.0, 4.0], [8.0, 5.0, 5.0], [8.0, 4.0, 5.0]]]]))
      val=Data(numpy.array([[[[8.0, 2.0, 4.0], [3.0, 4.0, 4.0]], [[5.0, 2.0, 1.0], [3.0, 2.0, 5.0]]], [[[5.0, 8.0, 6.0], [2.0, 
4.0, 5.0]], [[5.0, 7.0, 5.0], [6.0, 8.0, 8.0]]], [[[7.0, 2.0, 4.0], [1.0, 5.0, 6.0]], [[4.0, 2.0, 3.0], [2.0, 2.0, 2.0]]], 
[[[8.0, 2.0, 8.0], [4.0, 2.0, 1.0]], [[8.0, 8.0, 5.0], [4.0, 7.0, 5.0]]]]),self.functionspace)
      arg[:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 4.0], [3.0, 4.0, 4.0], [8.0, 6.0, 5.0]], [[5.0, 2.0, 1.0], [3.0, 2.0, 5.0], [6.0, 
6.0, 5.0]], [[6.0, 3.0, 5.0], [4.0, 4.0, 6.0], [2.0, 4.0, 8.0]]], [[[5.0, 8.0, 6.0], [2.0, 4.0, 5.0], [2.0, 8.0, 4.0]], [[5.0, 
7.0, 5.0], [6.0, 8.0, 8.0], [2.0, 8.0, 1.0]], [[1.0, 7.0, 7.0], [5.0, 7.0, 5.0], [6.0, 1.0, 7.0]]], [[[7.0, 2.0, 4.0], [1.0, 
5.0, 6.0], [3.0, 4.0, 8.0]], [[4.0, 2.0, 3.0], [2.0, 2.0, 2.0], [2.0, 1.0, 4.0]], [[4.0, 8.0, 4.0], [5.0, 6.0, 6.0], [3.0, 8.0, 
6.0]]], [[[8.0, 2.0, 8.0], [4.0, 2.0, 1.0], [4.0, 8.0, 2.0]], [[8.0, 8.0, 5.0], [4.0, 7.0, 5.0], [2.0, 5.0, 1.0]], [[3.0, 5.0, 
7.0], [8.0, 7.0, 1.0], [7.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 4.0], [3.0, 4.0, 4.0], [5.0, 8.0, 5.0]], 
[[5.0, 2.0, 1.0], [3.0, 2.0, 5.0], [3.0, 1.0, 5.0]], [[2.0, 1.0, 1.0], [1.0, 3.0, 5.0], [1.0, 6.0, 6.0]]], [[[5.0, 8.0, 6.0], 
[2.0, 4.0, 5.0], [2.0, 1.0, 6.0]], [[5.0, 7.0, 5.0], [6.0, 8.0, 8.0], [1.0, 2.0, 2.0]], [[1.0, 3.0, 7.0], [8.0, 1.0, 2.0], 
[3.0, 7.0, 8.0]]], [[[7.0, 2.0, 4.0], [1.0, 5.0, 6.0], [4.0, 4.0, 7.0]], [[4.0, 2.0, 3.0], [2.0, 2.0, 2.0], [6.0, 4.0, 7.0]], 
[[7.0, 7.0, 8.0], [5.0, 4.0, 7.0], [4.0, 3.0, 7.0]]], [[[8.0, 2.0, 8.0], [4.0, 2.0, 1.0], [1.0, 2.0, 2.0]], [[8.0, 8.0, 5.0], 
[4.0, 7.0, 5.0], [3.0, 1.0, 4.0]], [[4.0, 6.0, 4.0], [8.0, 5.0, 5.0], [8.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CUUN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 4.0], [7.0, 7.0, 2.0], [4.0, 3.0, 3.0]], [[8.0, 8.0, 1.0], [2.0, 8.0, 1.0], [3.0, 5.0, 
2.0]], [[3.0, 8.0, 6.0], [5.0, 3.0, 7.0], [7.0, 6.0, 6.0]]], [[[4.0, 7.0, 2.0], [2.0, 2.0, 4.0], [1.0, 3.0, 8.0]], [[8.0, 3.0, 
6.0], [5.0, 7.0, 3.0], [7.0, 4.0, 7.0]], [[1.0, 8.0, 8.0], [7.0, 7.0, 1.0], [5.0, 6.0, 4.0]]], [[[7.0, 5.0, 5.0], [2.0, 7.0, 
4.0], [2.0, 7.0, 6.0]], [[1.0, 7.0, 5.0], [3.0, 7.0, 4.0], [4.0, 4.0, 8.0]], [[3.0, 7.0, 7.0], [6.0, 7.0, 1.0], [3.0, 5.0, 
1.0]]], [[[5.0, 7.0, 4.0], [7.0, 2.0, 2.0], [5.0, 4.0, 4.0]], [[7.0, 4.0, 5.0], [4.0, 2.0, 8.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 
1.0], [8.0, 5.0, 4.0], [2.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 6.0], [8.0, 7.0, 7.0], [6.0, 7.0, 8.0]], [[1.0, 7.0, 5.0], [3.0, 2.0, 
5.0], [5.0, 2.0, 6.0]], [[7.0, 8.0, 5.0], [1.0, 6.0, 8.0], [6.0, 6.0, 8.0]]], [[[6.0, 7.0, 2.0], [7.0, 1.0, 3.0], [2.0, 8.0, 
3.0]], [[1.0, 4.0, 7.0], [3.0, 1.0, 1.0], [8.0, 6.0, 6.0]], [[2.0, 5.0, 6.0], [6.0, 8.0, 8.0], [3.0, 3.0, 1.0]]], [[[8.0, 2.0, 
2.0], [5.0, 4.0, 5.0], [4.0, 1.0, 8.0]], [[5.0, 6.0, 5.0], [3.0, 7.0, 1.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 4.0], [6.0, 2.0, 
7.0], [2.0, 6.0, 3.0]]], [[[3.0, 2.0, 1.0], [4.0, 2.0, 7.0], [2.0, 1.0, 3.0]], [[4.0, 7.0, 8.0], [6.0, 1.0, 4.0], [4.0, 4.0, 
8.0]], [[4.0, 5.0, 1.0], [7.0, 7.0, 5.0], [1.0, 4.0, 3.0]]]]))
      val=Data(numpy.array([[[[2.0, 3.0, 2.0], [2.0, 4.0, 6.0]], [[6.0, 2.0, 1.0], [7.0, 5.0, 7.0]]], [[[1.0, 8.0, 1.0], [6.0, 
5.0, 2.0]], [[7.0, 3.0, 6.0], [3.0, 7.0, 2.0]]], [[[1.0, 4.0, 8.0], [4.0, 4.0, 6.0]], [[3.0, 8.0, 1.0], [1.0, 7.0, 3.0]]], 
[[[6.0, 5.0, 4.0], [4.0, 6.0, 6.0]], [[8.0, 1.0, 4.0], [7.0, 4.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 3.0], [6.0, 2.0, 8.0]], [[5.0, 6.0, 7.0], [6.0, 1.0, 5.0]]], [[[6.0, 7.0, 
5.0], [2.0, 5.0, 1.0]], [[6.0, 8.0, 7.0], [4.0, 6.0, 3.0]]], [[[5.0, 7.0, 3.0], [1.0, 8.0, 5.0]], [[4.0, 1.0, 3.0], [5.0, 5.0, 
3.0]]], [[[8.0, 4.0, 4.0], [1.0, 4.0, 6.0]], [[4.0, 4.0, 1.0], [5.0, 8.0, 8.0]]]]))
      arg[:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 2.0], [2.0, 4.0, 6.0], [4.0, 3.0, 3.0]], [[6.0, 2.0, 1.0], [7.0, 5.0, 7.0], [3.0, 
5.0, 2.0]], [[3.0, 8.0, 6.0], [5.0, 3.0, 7.0], [7.0, 6.0, 6.0]]], [[[1.0, 8.0, 1.0], [6.0, 5.0, 2.0], [1.0, 3.0, 8.0]], [[7.0, 
3.0, 6.0], [3.0, 7.0, 2.0], [7.0, 4.0, 7.0]], [[1.0, 8.0, 8.0], [7.0, 7.0, 1.0], [5.0, 6.0, 4.0]]], [[[1.0, 4.0, 8.0], [4.0, 
4.0, 6.0], [2.0, 7.0, 6.0]], [[3.0, 8.0, 1.0], [1.0, 7.0, 3.0], [4.0, 4.0, 8.0]], [[3.0, 7.0, 7.0], [6.0, 7.0, 1.0], [3.0, 5.0, 
1.0]]], [[[6.0, 5.0, 4.0], [4.0, 6.0, 6.0], [5.0, 4.0, 4.0]], [[8.0, 1.0, 4.0], [7.0, 4.0, 2.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 
1.0], [8.0, 5.0, 4.0], [2.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 3.0], [6.0, 2.0, 8.0], [6.0, 7.0, 8.0]], 
[[5.0, 6.0, 7.0], [6.0, 1.0, 5.0], [5.0, 2.0, 6.0]], [[7.0, 8.0, 5.0], [1.0, 6.0, 8.0], [6.0, 6.0, 8.0]]], [[[6.0, 7.0, 5.0], 
[2.0, 5.0, 1.0], [2.0, 8.0, 3.0]], [[6.0, 8.0, 7.0], [4.0, 6.0, 3.0], [8.0, 6.0, 6.0]], [[2.0, 5.0, 6.0], [6.0, 8.0, 8.0], 
[3.0, 3.0, 1.0]]], [[[5.0, 7.0, 3.0], [1.0, 8.0, 5.0], [4.0, 1.0, 8.0]], [[4.0, 1.0, 3.0], [5.0, 5.0, 3.0], [1.0, 5.0, 7.0]], 
[[5.0, 7.0, 4.0], [6.0, 2.0, 7.0], [2.0, 6.0, 3.0]]], [[[8.0, 4.0, 4.0], [1.0, 4.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 4.0, 1.0], 
[5.0, 8.0, 8.0], [4.0, 4.0, 8.0]], [[4.0, 5.0, 1.0], [7.0, 7.0, 5.0], [1.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CUUN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 4.0], [1.0, 8.0, 8.0], [8.0, 8.0, 6.0]], [[3.0, 7.0, 5.0], [8.0, 2.0, 5.0], [8.0, 4.0, 
5.0]], [[6.0, 8.0, 3.0], [7.0, 1.0, 7.0], [1.0, 2.0, 4.0]]], [[[1.0, 6.0, 3.0], [3.0, 4.0, 6.0], [8.0, 3.0, 8.0]], [[6.0, 8.0, 
1.0], [3.0, 5.0, 2.0], [1.0, 1.0, 5.0]], [[1.0, 6.0, 1.0], [4.0, 2.0, 7.0], [6.0, 7.0, 1.0]]], [[[2.0, 4.0, 7.0], [4.0, 6.0, 
1.0], [1.0, 1.0, 2.0]], [[3.0, 8.0, 5.0], [6.0, 5.0, 8.0], [3.0, 8.0, 1.0]], [[8.0, 3.0, 2.0], [1.0, 6.0, 6.0], [2.0, 8.0, 
5.0]]], [[[6.0, 1.0, 1.0], [8.0, 3.0, 5.0], [3.0, 8.0, 5.0]], [[3.0, 6.0, 6.0], [7.0, 1.0, 7.0], [7.0, 7.0, 6.0]], [[5.0, 2.0, 
6.0], [6.0, 4.0, 6.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 4.0], [2.0, 5.0, 3.0], [5.0, 1.0, 7.0]], [[4.0, 4.0, 8.0], [5.0, 5.0, 
5.0], [8.0, 6.0, 3.0]], [[2.0, 5.0, 2.0], [5.0, 5.0, 7.0], [2.0, 7.0, 3.0]]], [[[7.0, 7.0, 5.0], [3.0, 8.0, 5.0], [3.0, 7.0, 
2.0]], [[7.0, 2.0, 8.0], [4.0, 3.0, 7.0], [7.0, 3.0, 6.0]], [[5.0, 3.0, 3.0], [7.0, 2.0, 8.0], [4.0, 1.0, 5.0]]], [[[5.0, 4.0, 
4.0], [3.0, 7.0, 4.0], [7.0, 4.0, 7.0]], [[8.0, 4.0, 6.0], [6.0, 4.0, 8.0], [6.0, 4.0, 2.0]], [[3.0, 2.0, 4.0], [7.0, 2.0, 
3.0], [1.0, 7.0, 5.0]]], [[[5.0, 8.0, 7.0], [2.0, 7.0, 5.0], [8.0, 7.0, 7.0]], [[1.0, 2.0, 6.0], [7.0, 1.0, 4.0], [8.0, 2.0, 
8.0]], [[6.0, 6.0, 8.0], [6.0, 2.0, 2.0], [8.0, 8.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 2.0, 1.0], [3.0, 3.0, 1.0]], [[6.0, 4.0, 7.0], [1.0, 1.0, 3.0]]], [[[8.0, 1.0, 8.0], 
[4.0, 3.0, 7.0]], [[5.0, 2.0, 3.0], [6.0, 8.0, 8.0]]], [[[5.0, 8.0, 4.0], [5.0, 1.0, 1.0]], [[4.0, 1.0, 5.0], [5.0, 5.0, 
4.0]]], [[[6.0, 3.0, 4.0], [6.0, 4.0, 2.0]], [[8.0, 5.0, 3.0], [3.0, 2.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 8.0, 8.0], 
[3.0, 1.0, 4.0]], [[3.0, 2.0, 7.0], [8.0, 7.0, 7.0]]], [[[7.0, 6.0, 8.0], [4.0, 8.0, 3.0]], [[7.0, 2.0, 4.0], [8.0, 8.0, 
1.0]]], [[[8.0, 5.0, 6.0], [4.0, 2.0, 3.0]], [[8.0, 1.0, 2.0], [8.0, 8.0, 8.0]]], [[[4.0, 6.0, 8.0], [8.0, 1.0, 4.0]], [[1.0, 
8.0, 5.0], [4.0, 4.0, 3.0]]]])
      arg[:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 1.0], [3.0, 3.0, 1.0], [8.0, 8.0, 6.0]], [[6.0, 4.0, 7.0], [1.0, 1.0, 3.0], [8.0, 
4.0, 5.0]], [[6.0, 8.0, 3.0], [7.0, 1.0, 7.0], [1.0, 2.0, 4.0]]], [[[8.0, 1.0, 8.0], [4.0, 3.0, 7.0], [8.0, 3.0, 8.0]], [[5.0, 
2.0, 3.0], [6.0, 8.0, 8.0], [1.0, 1.0, 5.0]], [[1.0, 6.0, 1.0], [4.0, 2.0, 7.0], [6.0, 7.0, 1.0]]], [[[5.0, 8.0, 4.0], [5.0, 
1.0, 1.0], [1.0, 1.0, 2.0]], [[4.0, 1.0, 5.0], [5.0, 5.0, 4.0], [3.0, 8.0, 1.0]], [[8.0, 3.0, 2.0], [1.0, 6.0, 6.0], [2.0, 8.0, 
5.0]]], [[[6.0, 3.0, 4.0], [6.0, 4.0, 2.0], [3.0, 8.0, 5.0]], [[8.0, 5.0, 3.0], [3.0, 2.0, 2.0], [7.0, 7.0, 6.0]], [[5.0, 2.0, 
6.0], [6.0, 4.0, 6.0], [1.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 8.0], [3.0, 1.0, 4.0], [5.0, 1.0, 7.0]], 
[[3.0, 2.0, 7.0], [8.0, 7.0, 7.0], [8.0, 6.0, 3.0]], [[2.0, 5.0, 2.0], [5.0, 5.0, 7.0], [2.0, 7.0, 3.0]]], [[[7.0, 6.0, 8.0], 
[4.0, 8.0, 3.0], [3.0, 7.0, 2.0]], [[7.0, 2.0, 4.0], [8.0, 8.0, 1.0], [7.0, 3.0, 6.0]], [[5.0, 3.0, 3.0], [7.0, 2.0, 8.0], 
[4.0, 1.0, 5.0]]], [[[8.0, 5.0, 6.0], [4.0, 2.0, 3.0], [7.0, 4.0, 7.0]], [[8.0, 1.0, 2.0], [8.0, 8.0, 8.0], [6.0, 4.0, 2.0]], 
[[3.0, 2.0, 4.0], [7.0, 2.0, 3.0], [1.0, 7.0, 5.0]]], [[[4.0, 6.0, 8.0], [8.0, 1.0, 4.0], [8.0, 7.0, 7.0]], [[1.0, 8.0, 5.0], 
[4.0, 4.0, 3.0], [8.0, 2.0, 8.0]], [[6.0, 6.0, 8.0], [6.0, 2.0, 2.0], [8.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLLP(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 2.0], [6.0, 3.0, 2.0], [8.0, 3.0, 6.0]], [[3.0, 3.0, 5.0], [8.0, 3.0, 2.0], [5.0, 5.0, 
5.0]], [[6.0, 3.0, 1.0], [3.0, 2.0, 5.0], [7.0, 3.0, 4.0]]], [[[1.0, 6.0, 6.0], [8.0, 8.0, 6.0], [4.0, 8.0, 6.0]], [[7.0, 8.0, 
3.0], [8.0, 1.0, 2.0], [3.0, 2.0, 7.0]], [[1.0, 5.0, 8.0], [7.0, 6.0, 7.0], [1.0, 1.0, 6.0]]], [[[7.0, 5.0, 3.0], [4.0, 1.0, 
3.0], [6.0, 7.0, 4.0]], [[8.0, 1.0, 4.0], [2.0, 6.0, 5.0], [4.0, 1.0, 6.0]], [[1.0, 5.0, 1.0], [6.0, 7.0, 5.0], [2.0, 2.0, 
3.0]]], [[[1.0, 4.0, 7.0], [4.0, 3.0, 4.0], [1.0, 3.0, 6.0]], [[7.0, 5.0, 8.0], [8.0, 8.0, 8.0], [2.0, 6.0, 3.0]], [[1.0, 3.0, 
8.0], [4.0, 5.0, 2.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 1.0], [7.0, 7.0, 4.0], [8.0, 4.0, 6.0]], [[2.0, 8.0, 8.0], [5.0, 3.0, 
7.0], [4.0, 2.0, 5.0]], [[6.0, 5.0, 7.0], [4.0, 4.0, 2.0], [8.0, 6.0, 6.0]]], [[[8.0, 2.0, 7.0], [4.0, 8.0, 2.0], [5.0, 5.0, 
1.0]], [[5.0, 3.0, 7.0], [3.0, 3.0, 3.0], [5.0, 6.0, 5.0]], [[5.0, 8.0, 4.0], [7.0, 4.0, 5.0], [8.0, 3.0, 4.0]]], [[[8.0, 7.0, 
8.0], [8.0, 5.0, 5.0], [7.0, 3.0, 4.0]], [[7.0, 7.0, 3.0], [5.0, 1.0, 3.0], [7.0, 5.0, 5.0]], [[6.0, 3.0, 2.0], [2.0, 7.0, 
2.0], [2.0, 3.0, 1.0]]], [[[8.0, 8.0, 7.0], [4.0, 5.0, 7.0], [4.0, 2.0, 7.0]], [[6.0, 7.0, 7.0], [3.0, 2.0, 1.0], [1.0, 8.0, 
8.0]], [[7.0, 5.0, 4.0], [3.0, 8.0, 1.0], [8.0, 1.0, 7.0]]]]))
      val=numpy.array([[[4.0]], [[2.0]]])
      arg[2:,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 2.0], [6.0, 3.0, 2.0], [8.0, 3.0, 6.0]], [[3.0, 3.0, 5.0], [8.0, 3.0, 2.0], [5.0, 
5.0, 5.0]], [[6.0, 3.0, 1.0], [3.0, 2.0, 5.0], [7.0, 3.0, 4.0]]], [[[1.0, 6.0, 6.0], [8.0, 8.0, 6.0], [4.0, 8.0, 6.0]], [[7.0, 
8.0, 3.0], [8.0, 1.0, 2.0], [3.0, 2.0, 7.0]], [[1.0, 5.0, 8.0], [7.0, 6.0, 7.0], [1.0, 1.0, 6.0]]], [[[7.0, 5.0, 3.0], [4.0, 
1.0, 3.0], [6.0, 7.0, 4.0]], [[8.0, 1.0, 4.0], [2.0, 6.0, 5.0], [4.0, 1.0, 6.0]], [[1.0, 5.0, 1.0], [6.0, 7.0, 5.0], [2.0, 4.0, 
3.0]]], [[[1.0, 4.0, 7.0], [4.0, 3.0, 4.0], [1.0, 3.0, 6.0]], [[7.0, 5.0, 8.0], [8.0, 8.0, 8.0], [2.0, 6.0, 3.0]], [[1.0, 3.0, 
8.0], [4.0, 5.0, 2.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 1.0], [7.0, 7.0, 4.0], [8.0, 4.0, 6.0]], 
[[2.0, 8.0, 8.0], [5.0, 3.0, 7.0], [4.0, 2.0, 5.0]], [[6.0, 5.0, 7.0], [4.0, 4.0, 2.0], [8.0, 6.0, 6.0]]], [[[8.0, 2.0, 7.0], 
[4.0, 8.0, 2.0], [5.0, 5.0, 1.0]], [[5.0, 3.0, 7.0], [3.0, 3.0, 3.0], [5.0, 6.0, 5.0]], [[5.0, 8.0, 4.0], [7.0, 4.0, 5.0], 
[8.0, 3.0, 4.0]]], [[[8.0, 7.0, 8.0], [8.0, 5.0, 5.0], [7.0, 3.0, 4.0]], [[7.0, 7.0, 3.0], [5.0, 1.0, 3.0], [7.0, 5.0, 5.0]], 
[[6.0, 3.0, 2.0], [2.0, 7.0, 2.0], [2.0, 4.0, 1.0]]], [[[8.0, 8.0, 7.0], [4.0, 5.0, 7.0], [4.0, 2.0, 7.0]], [[6.0, 7.0, 7.0], 
[3.0, 2.0, 1.0], [1.0, 8.0, 8.0]], [[7.0, 5.0, 4.0], [3.0, 8.0, 1.0], [8.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLLP(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 5.0], [5.0, 6.0, 4.0], [3.0, 3.0, 7.0]], [[6.0, 7.0, 8.0], [8.0, 7.0, 8.0], [2.0, 8.0, 
7.0]], [[7.0, 8.0, 2.0], [3.0, 7.0, 2.0], [6.0, 1.0, 7.0]]], [[[5.0, 1.0, 8.0], [8.0, 3.0, 1.0], [1.0, 3.0, 1.0]], [[1.0, 3.0, 
1.0], [8.0, 8.0, 5.0], [2.0, 3.0, 5.0]], [[7.0, 4.0, 1.0], [8.0, 5.0, 3.0], [1.0, 4.0, 5.0]]], [[[5.0, 6.0, 4.0], [8.0, 6.0, 
2.0], [5.0, 5.0, 1.0]], [[6.0, 4.0, 5.0], [7.0, 2.0, 2.0], [1.0, 8.0, 5.0]], [[1.0, 3.0, 8.0], [6.0, 1.0, 7.0], [5.0, 1.0, 
7.0]]], [[[1.0, 5.0, 4.0], [4.0, 7.0, 6.0], [7.0, 8.0, 5.0]], [[2.0, 3.0, 5.0], [2.0, 6.0, 8.0], [5.0, 8.0, 4.0]], [[4.0, 4.0, 
1.0], [8.0, 6.0, 3.0], [3.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 3.0], [6.0, 1.0, 1.0], [7.0, 6.0, 2.0]], [[7.0, 8.0, 6.0], [4.0, 6.0, 
5.0], [1.0, 8.0, 3.0]], [[1.0, 4.0, 5.0], [1.0, 7.0, 6.0], [6.0, 2.0, 7.0]]], [[[4.0, 3.0, 5.0], [1.0, 2.0, 3.0], [8.0, 4.0, 
8.0]], [[3.0, 2.0, 4.0], [6.0, 1.0, 3.0], [4.0, 5.0, 7.0]], [[6.0, 2.0, 8.0], [5.0, 7.0, 6.0], [6.0, 2.0, 7.0]]], [[[2.0, 4.0, 
4.0], [1.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[3.0, 6.0, 2.0], [3.0, 2.0, 8.0], [2.0, 2.0, 4.0]], [[8.0, 4.0, 2.0], [8.0, 4.0, 
3.0], [1.0, 7.0, 6.0]]], [[[3.0, 5.0, 1.0], [7.0, 7.0, 5.0], [7.0, 1.0, 5.0]], [[6.0, 4.0, 6.0], [6.0, 8.0, 7.0], [4.0, 4.0, 
5.0]], [[4.0, 6.0, 1.0], [5.0, 4.0, 8.0], [5.0, 3.0, 4.0]]]]))
      val=Data(numpy.array([[[8.0]], [[2.0]]]),self.functionspace)
      arg[2:,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 5.0], [5.0, 6.0, 4.0], [3.0, 3.0, 7.0]], [[6.0, 7.0, 8.0], [8.0, 7.0, 8.0], [2.0, 
8.0, 7.0]], [[7.0, 8.0, 2.0], [3.0, 7.0, 2.0], [6.0, 1.0, 7.0]]], [[[5.0, 1.0, 8.0], [8.0, 3.0, 1.0], [1.0, 3.0, 1.0]], [[1.0, 
3.0, 1.0], [8.0, 8.0, 5.0], [2.0, 3.0, 5.0]], [[7.0, 4.0, 1.0], [8.0, 5.0, 3.0], [1.0, 4.0, 5.0]]], [[[5.0, 6.0, 4.0], [8.0, 
6.0, 2.0], [5.0, 5.0, 1.0]], [[6.0, 4.0, 5.0], [7.0, 2.0, 2.0], [1.0, 8.0, 5.0]], [[1.0, 3.0, 8.0], [6.0, 1.0, 7.0], [5.0, 8.0, 
7.0]]], [[[1.0, 5.0, 4.0], [4.0, 7.0, 6.0], [7.0, 8.0, 5.0]], [[2.0, 3.0, 5.0], [2.0, 6.0, 8.0], [5.0, 8.0, 4.0]], [[4.0, 4.0, 
1.0], [8.0, 6.0, 3.0], [3.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 3.0], [6.0, 1.0, 1.0], [7.0, 6.0, 2.0]], 
[[7.0, 8.0, 6.0], [4.0, 6.0, 5.0], [1.0, 8.0, 3.0]], [[1.0, 4.0, 5.0], [1.0, 7.0, 6.0], [6.0, 2.0, 7.0]]], [[[4.0, 3.0, 5.0], 
[1.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[3.0, 2.0, 4.0], [6.0, 1.0, 3.0], [4.0, 5.0, 7.0]], [[6.0, 2.0, 8.0], [5.0, 7.0, 6.0], 
[6.0, 2.0, 7.0]]], [[[2.0, 4.0, 4.0], [1.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[3.0, 6.0, 2.0], [3.0, 2.0, 8.0], [2.0, 2.0, 4.0]], 
[[8.0, 4.0, 2.0], [8.0, 4.0, 3.0], [1.0, 8.0, 6.0]]], [[[3.0, 5.0, 1.0], [7.0, 7.0, 5.0], [7.0, 1.0, 5.0]], [[6.0, 4.0, 6.0], 
[6.0, 8.0, 7.0], [4.0, 4.0, 5.0]], [[4.0, 6.0, 1.0], [5.0, 4.0, 8.0], [5.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLLP(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 6.0], [4.0, 4.0, 8.0], [1.0, 7.0, 5.0]], [[7.0, 5.0, 4.0], [3.0, 4.0, 7.0], [8.0, 5.0, 
8.0]], [[3.0, 3.0, 2.0], [8.0, 6.0, 8.0], [8.0, 6.0, 6.0]]], [[[1.0, 5.0, 1.0], [2.0, 5.0, 3.0], [2.0, 2.0, 8.0]], [[2.0, 7.0, 
2.0], [8.0, 3.0, 2.0], [4.0, 6.0, 3.0]], [[4.0, 8.0, 1.0], [4.0, 3.0, 4.0], [1.0, 7.0, 1.0]]], [[[3.0, 5.0, 4.0], [3.0, 2.0, 
4.0], [5.0, 4.0, 6.0]], [[7.0, 1.0, 7.0], [6.0, 1.0, 2.0], [2.0, 4.0, 2.0]], [[5.0, 7.0, 7.0], [1.0, 6.0, 4.0], [7.0, 8.0, 
3.0]]], [[[2.0, 5.0, 1.0], [4.0, 7.0, 8.0], [4.0, 4.0, 8.0]], [[2.0, 2.0, 7.0], [3.0, 2.0, 2.0], [1.0, 7.0, 7.0]], [[2.0, 1.0, 
7.0], [3.0, 1.0, 8.0], [6.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 8.0], [7.0, 2.0, 8.0], [8.0, 7.0, 7.0]], [[1.0, 1.0, 3.0], [1.0, 1.0, 
3.0], [3.0, 2.0, 8.0]], [[8.0, 6.0, 3.0], [4.0, 6.0, 2.0], [7.0, 4.0, 7.0]]], [[[5.0, 4.0, 4.0], [7.0, 7.0, 7.0], [7.0, 1.0, 
1.0]], [[1.0, 2.0, 2.0], [8.0, 2.0, 7.0], [1.0, 7.0, 7.0]], [[2.0, 2.0, 7.0], [2.0, 5.0, 2.0], [4.0, 5.0, 6.0]]], [[[6.0, 2.0, 
6.0], [5.0, 3.0, 2.0], [5.0, 6.0, 4.0]], [[3.0, 6.0, 2.0], [6.0, 2.0, 2.0], [2.0, 1.0, 6.0]], [[7.0, 7.0, 3.0], [1.0, 2.0, 
7.0], [7.0, 6.0, 3.0]]], [[[1.0, 8.0, 2.0], [2.0, 3.0, 1.0], [2.0, 4.0, 3.0]], [[4.0, 3.0, 6.0], [7.0, 3.0, 6.0], [5.0, 4.0, 
3.0]], [[8.0, 3.0, 1.0], [7.0, 2.0, 3.0], [4.0, 1.0, 5.0]]]]))
      val=Data(numpy.array([[[7.0]], [[2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0]], [[2.0]]]))
      arg[2:,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 6.0], [4.0, 4.0, 8.0], [1.0, 7.0, 5.0]], [[7.0, 5.0, 4.0], [3.0, 4.0, 7.0], [8.0, 
5.0, 8.0]], [[3.0, 3.0, 2.0], [8.0, 6.0, 8.0], [8.0, 6.0, 6.0]]], [[[1.0, 5.0, 1.0], [2.0, 5.0, 3.0], [2.0, 2.0, 8.0]], [[2.0, 
7.0, 2.0], [8.0, 3.0, 2.0], [4.0, 6.0, 3.0]], [[4.0, 8.0, 1.0], [4.0, 3.0, 4.0], [1.0, 7.0, 1.0]]], [[[3.0, 5.0, 4.0], [3.0, 
2.0, 4.0], [5.0, 4.0, 6.0]], [[7.0, 1.0, 7.0], [6.0, 1.0, 2.0], [2.0, 4.0, 2.0]], [[5.0, 7.0, 7.0], [1.0, 6.0, 4.0], [7.0, 7.0, 
3.0]]], [[[2.0, 5.0, 1.0], [4.0, 7.0, 8.0], [4.0, 4.0, 8.0]], [[2.0, 2.0, 7.0], [3.0, 2.0, 2.0], [1.0, 7.0, 7.0]], [[2.0, 1.0, 
7.0], [3.0, 1.0, 8.0], [6.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 8.0], [7.0, 2.0, 8.0], [8.0, 7.0, 7.0]], 
[[1.0, 1.0, 3.0], [1.0, 1.0, 3.0], [3.0, 2.0, 8.0]], [[8.0, 6.0, 3.0], [4.0, 6.0, 2.0], [7.0, 4.0, 7.0]]], [[[5.0, 4.0, 4.0], 
[7.0, 7.0, 7.0], [7.0, 1.0, 1.0]], [[1.0, 2.0, 2.0], [8.0, 2.0, 7.0], [1.0, 7.0, 7.0]], [[2.0, 2.0, 7.0], [2.0, 5.0, 2.0], 
[4.0, 5.0, 6.0]]], [[[6.0, 2.0, 6.0], [5.0, 3.0, 2.0], [5.0, 6.0, 4.0]], [[3.0, 6.0, 2.0], [6.0, 2.0, 2.0], [2.0, 1.0, 6.0]], 
[[7.0, 7.0, 3.0], [1.0, 2.0, 7.0], [7.0, 1.0, 3.0]]], [[[1.0, 8.0, 2.0], [2.0, 3.0, 1.0], [2.0, 4.0, 3.0]], [[4.0, 3.0, 6.0], 
[7.0, 3.0, 6.0], [5.0, 4.0, 3.0]], [[8.0, 3.0, 1.0], [7.0, 2.0, 3.0], [4.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLLP(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 1.0], [5.0, 7.0, 5.0], [8.0, 6.0, 8.0]], [[6.0, 5.0, 3.0], [1.0, 6.0, 1.0], [4.0, 4.0, 
6.0]], [[5.0, 2.0, 6.0], [5.0, 8.0, 8.0], [7.0, 8.0, 2.0]]], [[[7.0, 4.0, 5.0], [7.0, 7.0, 2.0], [6.0, 7.0, 3.0]], [[6.0, 6.0, 
8.0], [5.0, 5.0, 6.0], [6.0, 5.0, 6.0]], [[6.0, 5.0, 5.0], [5.0, 8.0, 6.0], [5.0, 4.0, 5.0]]], [[[5.0, 8.0, 3.0], [5.0, 1.0, 
5.0], [3.0, 1.0, 5.0]], [[1.0, 7.0, 5.0], [8.0, 6.0, 6.0], [8.0, 7.0, 3.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 4.0], [3.0, 2.0, 
1.0]]], [[[4.0, 7.0, 2.0], [4.0, 6.0, 4.0], [7.0, 1.0, 5.0]], [[2.0, 1.0, 2.0], [1.0, 8.0, 7.0], [4.0, 7.0, 8.0]], [[2.0, 5.0, 
4.0], [1.0, 5.0, 6.0], [7.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 7.0], [1.0, 7.0, 2.0], [3.0, 6.0, 3.0]], [[6.0, 5.0, 7.0], [8.0, 5.0, 
2.0], [3.0, 3.0, 4.0]], [[6.0, 3.0, 5.0], [4.0, 8.0, 3.0], [1.0, 4.0, 2.0]]], [[[8.0, 7.0, 3.0], [3.0, 1.0, 2.0], [2.0, 4.0, 
6.0]], [[2.0, 2.0, 4.0], [3.0, 2.0, 6.0], [7.0, 6.0, 4.0]], [[3.0, 5.0, 3.0], [1.0, 2.0, 3.0], [2.0, 7.0, 8.0]]], [[[2.0, 6.0, 
6.0], [4.0, 5.0, 8.0], [8.0, 3.0, 3.0]], [[2.0, 6.0, 6.0], [2.0, 1.0, 5.0], [5.0, 6.0, 5.0]], [[4.0, 3.0, 3.0], [7.0, 2.0, 
7.0], [2.0, 5.0, 6.0]]], [[[1.0, 6.0, 6.0], [8.0, 4.0, 7.0], [2.0, 7.0, 5.0]], [[7.0, 8.0, 5.0], [3.0, 2.0, 2.0], [3.0, 2.0, 
6.0]], [[6.0, 6.0, 8.0], [8.0, 6.0, 8.0], [6.0, 4.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0]], [[5.0]]])+(1.-msk_val)*numpy.array([[[4.0]], [[1.0]]])
      arg[2:,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 1.0], [5.0, 7.0, 5.0], [8.0, 6.0, 8.0]], [[6.0, 5.0, 3.0], [1.0, 6.0, 1.0], [4.0, 
4.0, 6.0]], [[5.0, 2.0, 6.0], [5.0, 8.0, 8.0], [7.0, 8.0, 2.0]]], [[[7.0, 4.0, 5.0], [7.0, 7.0, 2.0], [6.0, 7.0, 3.0]], [[6.0, 
6.0, 8.0], [5.0, 5.0, 6.0], [6.0, 5.0, 6.0]], [[6.0, 5.0, 5.0], [5.0, 8.0, 6.0], [5.0, 4.0, 5.0]]], [[[5.0, 8.0, 3.0], [5.0, 
1.0, 5.0], [3.0, 1.0, 5.0]], [[1.0, 7.0, 5.0], [8.0, 6.0, 6.0], [8.0, 7.0, 3.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 4.0], [3.0, 1.0, 
1.0]]], [[[4.0, 7.0, 2.0], [4.0, 6.0, 4.0], [7.0, 1.0, 5.0]], [[2.0, 1.0, 2.0], [1.0, 8.0, 7.0], [4.0, 7.0, 8.0]], [[2.0, 5.0, 
4.0], [1.0, 5.0, 6.0], [7.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 7.0], [1.0, 7.0, 2.0], [3.0, 6.0, 3.0]], 
[[6.0, 5.0, 7.0], [8.0, 5.0, 2.0], [3.0, 3.0, 4.0]], [[6.0, 3.0, 5.0], [4.0, 8.0, 3.0], [1.0, 4.0, 2.0]]], [[[8.0, 7.0, 3.0], 
[3.0, 1.0, 2.0], [2.0, 4.0, 6.0]], [[2.0, 2.0, 4.0], [3.0, 2.0, 6.0], [7.0, 6.0, 4.0]], [[3.0, 5.0, 3.0], [1.0, 2.0, 3.0], 
[2.0, 7.0, 8.0]]], [[[2.0, 6.0, 6.0], [4.0, 5.0, 8.0], [8.0, 3.0, 3.0]], [[2.0, 6.0, 6.0], [2.0, 1.0, 5.0], [5.0, 6.0, 5.0]], 
[[4.0, 3.0, 3.0], [7.0, 2.0, 7.0], [2.0, 4.0, 6.0]]], [[[1.0, 6.0, 6.0], [8.0, 4.0, 7.0], [2.0, 7.0, 5.0]], [[7.0, 8.0, 5.0], 
[3.0, 2.0, 2.0], [3.0, 2.0, 6.0]], [[6.0, 6.0, 8.0], [8.0, 6.0, 8.0], [6.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LRRN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 2.0], [5.0, 4.0, 6.0], [2.0, 7.0, 6.0]], [[5.0, 2.0, 5.0], [3.0, 8.0, 2.0], [2.0, 3.0, 
3.0]], [[4.0, 2.0, 5.0], [5.0, 1.0, 4.0], [3.0, 6.0, 3.0]]], [[[3.0, 2.0, 3.0], [1.0, 4.0, 1.0], [3.0, 5.0, 6.0]], [[4.0, 4.0, 
1.0], [3.0, 4.0, 5.0], [6.0, 8.0, 4.0]], [[5.0, 7.0, 2.0], [2.0, 1.0, 1.0], [3.0, 7.0, 7.0]]], [[[3.0, 8.0, 5.0], [4.0, 7.0, 
6.0], [1.0, 1.0, 4.0]], [[7.0, 2.0, 4.0], [3.0, 1.0, 2.0], [7.0, 4.0, 4.0]], [[4.0, 7.0, 6.0], [4.0, 4.0, 5.0], [3.0, 4.0, 
6.0]]], [[[3.0, 5.0, 3.0], [6.0, 5.0, 1.0], [3.0, 2.0, 2.0]], [[2.0, 5.0, 2.0], [7.0, 7.0, 4.0], [6.0, 3.0, 2.0]], [[8.0, 5.0, 
4.0], [2.0, 1.0, 4.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 4.0], [7.0, 4.0, 3.0], [5.0, 4.0, 7.0]], [[7.0, 8.0, 8.0], [2.0, 2.0, 
2.0], [2.0, 2.0, 5.0]], [[5.0, 1.0, 7.0], [5.0, 4.0, 6.0], [1.0, 4.0, 5.0]]], [[[4.0, 4.0, 8.0], [2.0, 7.0, 5.0], [6.0, 2.0, 
7.0]], [[5.0, 3.0, 3.0], [5.0, 1.0, 5.0], [5.0, 5.0, 5.0]], [[4.0, 8.0, 6.0], [8.0, 8.0, 4.0], [2.0, 1.0, 6.0]]], [[[8.0, 6.0, 
3.0], [7.0, 8.0, 3.0], [7.0, 8.0, 4.0]], [[8.0, 2.0, 6.0], [7.0, 5.0, 8.0], [7.0, 7.0, 2.0]], [[4.0, 2.0, 7.0], [8.0, 6.0, 
1.0], [6.0, 7.0, 8.0]]], [[[8.0, 8.0, 1.0], [1.0, 6.0, 2.0], [4.0, 5.0, 7.0]], [[1.0, 3.0, 2.0], [3.0, 1.0, 3.0], [3.0, 6.0, 
1.0]], [[6.0, 6.0, 5.0], [3.0, 8.0, 1.0], [8.0, 2.0, 5.0]]]]))
      val=numpy.array([[[[1.0, 2.0, 1.0]]], [[[5.0, 3.0, 5.0]]]])
      arg[2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 2.0], [5.0, 4.0, 6.0], [2.0, 7.0, 6.0]], [[5.0, 2.0, 5.0], [3.0, 8.0, 2.0], [2.0, 
3.0, 3.0]], [[4.0, 2.0, 5.0], [5.0, 1.0, 4.0], [3.0, 6.0, 3.0]]], [[[3.0, 2.0, 3.0], [1.0, 4.0, 1.0], [3.0, 5.0, 6.0]], [[4.0, 
4.0, 1.0], [3.0, 4.0, 5.0], [6.0, 8.0, 4.0]], [[5.0, 7.0, 2.0], [2.0, 1.0, 1.0], [3.0, 7.0, 7.0]]], [[[3.0, 8.0, 5.0], [4.0, 
7.0, 6.0], [1.0, 1.0, 4.0]], [[7.0, 2.0, 4.0], [1.0, 2.0, 1.0], [7.0, 4.0, 4.0]], [[4.0, 7.0, 6.0], [4.0, 4.0, 5.0], [3.0, 4.0, 
6.0]]], [[[3.0, 5.0, 3.0], [6.0, 5.0, 1.0], [3.0, 2.0, 2.0]], [[2.0, 5.0, 2.0], [5.0, 3.0, 5.0], [6.0, 3.0, 2.0]], [[8.0, 5.0, 
4.0], [2.0, 1.0, 4.0], [6.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 4.0], [7.0, 4.0, 3.0], [5.0, 4.0, 7.0]], 
[[7.0, 8.0, 8.0], [2.0, 2.0, 2.0], [2.0, 2.0, 5.0]], [[5.0, 1.0, 7.0], [5.0, 4.0, 6.0], [1.0, 4.0, 5.0]]], [[[4.0, 4.0, 8.0], 
[2.0, 7.0, 5.0], [6.0, 2.0, 7.0]], [[5.0, 3.0, 3.0], [5.0, 1.0, 5.0], [5.0, 5.0, 5.0]], [[4.0, 8.0, 6.0], [8.0, 8.0, 4.0], 
[2.0, 1.0, 6.0]]], [[[8.0, 6.0, 3.0], [7.0, 8.0, 3.0], [7.0, 8.0, 4.0]], [[8.0, 2.0, 6.0], [1.0, 2.0, 1.0], [7.0, 7.0, 2.0]], 
[[4.0, 2.0, 7.0], [8.0, 6.0, 1.0], [6.0, 7.0, 8.0]]], [[[8.0, 8.0, 1.0], [1.0, 6.0, 2.0], [4.0, 5.0, 7.0]], [[1.0, 3.0, 2.0], 
[5.0, 3.0, 5.0], [3.0, 6.0, 1.0]], [[6.0, 6.0, 5.0], [3.0, 8.0, 1.0], [8.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LRRN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 4.0], [2.0, 7.0, 1.0], [4.0, 3.0, 5.0]], [[7.0, 6.0, 6.0], [8.0, 3.0, 3.0], [4.0, 6.0, 
6.0]], [[6.0, 4.0, 7.0], [7.0, 7.0, 6.0], [2.0, 4.0, 2.0]]], [[[6.0, 5.0, 1.0], [1.0, 2.0, 7.0], [5.0, 7.0, 8.0]], [[5.0, 7.0, 
2.0], [4.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 4.0, 8.0], [2.0, 7.0, 4.0]]], [[[4.0, 1.0, 7.0], [8.0, 6.0, 
4.0], [4.0, 7.0, 1.0]], [[4.0, 8.0, 7.0], [2.0, 4.0, 6.0], [8.0, 8.0, 5.0]], [[5.0, 2.0, 4.0], [6.0, 8.0, 7.0], [7.0, 2.0, 
3.0]]], [[[1.0, 6.0, 5.0], [3.0, 3.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 5.0, 6.0], [4.0, 6.0, 6.0], [8.0, 8.0, 2.0]], [[6.0, 7.0, 
4.0], [3.0, 8.0, 4.0], [4.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 4.0], [7.0, 6.0, 7.0], [7.0, 3.0, 6.0]], [[8.0, 3.0, 7.0], [3.0, 1.0, 
3.0], [8.0, 3.0, 4.0]], [[1.0, 4.0, 4.0], [8.0, 3.0, 3.0], [1.0, 1.0, 6.0]]], [[[2.0, 7.0, 4.0], [7.0, 5.0, 6.0], [7.0, 6.0, 
1.0]], [[7.0, 8.0, 6.0], [1.0, 8.0, 2.0], [3.0, 6.0, 2.0]], [[4.0, 8.0, 7.0], [7.0, 2.0, 4.0], [8.0, 1.0, 1.0]]], [[[5.0, 5.0, 
2.0], [1.0, 3.0, 3.0], [3.0, 7.0, 8.0]], [[1.0, 6.0, 5.0], [2.0, 5.0, 8.0], [5.0, 8.0, 2.0]], [[3.0, 3.0, 1.0], [1.0, 6.0, 
7.0], [5.0, 6.0, 3.0]]], [[[4.0, 4.0, 8.0], [1.0, 1.0, 2.0], [8.0, 5.0, 8.0]], [[5.0, 6.0, 2.0], [5.0, 8.0, 7.0], [2.0, 8.0, 
7.0]], [[2.0, 5.0, 1.0], [2.0, 3.0, 5.0], [1.0, 8.0, 7.0]]]]))
      val=Data(numpy.array([[[[7.0, 3.0, 7.0]]], [[[4.0, 8.0, 5.0]]]]),self.functionspace)
      arg[2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 4.0], [2.0, 7.0, 1.0], [4.0, 3.0, 5.0]], [[7.0, 6.0, 6.0], [8.0, 3.0, 3.0], [4.0, 
6.0, 6.0]], [[6.0, 4.0, 7.0], [7.0, 7.0, 6.0], [2.0, 4.0, 2.0]]], [[[6.0, 5.0, 1.0], [1.0, 2.0, 7.0], [5.0, 7.0, 8.0]], [[5.0, 
7.0, 2.0], [4.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 4.0, 8.0], [2.0, 7.0, 4.0]]], [[[4.0, 1.0, 7.0], [8.0, 
6.0, 4.0], [4.0, 7.0, 1.0]], [[4.0, 8.0, 7.0], [7.0, 3.0, 7.0], [8.0, 8.0, 5.0]], [[5.0, 2.0, 4.0], [6.0, 8.0, 7.0], [7.0, 2.0, 
3.0]]], [[[1.0, 6.0, 5.0], [3.0, 3.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 5.0, 6.0], [4.0, 8.0, 5.0], [8.0, 8.0, 2.0]], [[6.0, 7.0, 
4.0], [3.0, 8.0, 4.0], [4.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 4.0], [7.0, 6.0, 7.0], [7.0, 3.0, 6.0]], 
[[8.0, 3.0, 7.0], [3.0, 1.0, 3.0], [8.0, 3.0, 4.0]], [[1.0, 4.0, 4.0], [8.0, 3.0, 3.0], [1.0, 1.0, 6.0]]], [[[2.0, 7.0, 4.0], 
[7.0, 5.0, 6.0], [7.0, 6.0, 1.0]], [[7.0, 8.0, 6.0], [1.0, 8.0, 2.0], [3.0, 6.0, 2.0]], [[4.0, 8.0, 7.0], [7.0, 2.0, 4.0], 
[8.0, 1.0, 1.0]]], [[[5.0, 5.0, 2.0], [1.0, 3.0, 3.0], [3.0, 7.0, 8.0]], [[1.0, 6.0, 5.0], [7.0, 3.0, 7.0], [5.0, 8.0, 2.0]], 
[[3.0, 3.0, 1.0], [1.0, 6.0, 7.0], [5.0, 6.0, 3.0]]], [[[4.0, 4.0, 8.0], [1.0, 1.0, 2.0], [8.0, 5.0, 8.0]], [[5.0, 6.0, 2.0], 
[4.0, 8.0, 5.0], [2.0, 8.0, 7.0]], [[2.0, 5.0, 1.0], [2.0, 3.0, 5.0], [1.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LRRN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 5.0], [7.0, 5.0, 6.0], [1.0, 5.0, 7.0]], [[6.0, 6.0, 5.0], [6.0, 5.0, 4.0], [2.0, 7.0, 
6.0]], [[1.0, 1.0, 3.0], [1.0, 7.0, 4.0], [3.0, 5.0, 7.0]]], [[[8.0, 2.0, 2.0], [5.0, 3.0, 7.0], [2.0, 6.0, 8.0]], [[1.0, 3.0, 
2.0], [2.0, 6.0, 6.0], [6.0, 3.0, 8.0]], [[6.0, 8.0, 3.0], [6.0, 2.0, 5.0], [5.0, 2.0, 4.0]]], [[[7.0, 7.0, 7.0], [8.0, 5.0, 
8.0], [8.0, 7.0, 6.0]], [[3.0, 8.0, 6.0], [5.0, 7.0, 1.0], [3.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [5.0, 1.0, 6.0], [2.0, 2.0, 
7.0]]], [[[2.0, 1.0, 1.0], [3.0, 3.0, 2.0], [2.0, 1.0, 6.0]], [[7.0, 2.0, 6.0], [6.0, 5.0, 6.0], [6.0, 1.0, 6.0]], [[1.0, 2.0, 
6.0], [6.0, 3.0, 7.0], [8.0, 2.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 6.0], [5.0, 2.0, 3.0], [8.0, 5.0, 8.0]], [[6.0, 4.0, 5.0], [1.0, 3.0, 
6.0], [1.0, 5.0, 8.0]], [[3.0, 4.0, 4.0], [5.0, 7.0, 1.0], [2.0, 2.0, 8.0]]], [[[8.0, 5.0, 4.0], [8.0, 8.0, 5.0], [2.0, 3.0, 
8.0]], [[5.0, 5.0, 6.0], [6.0, 4.0, 6.0], [8.0, 5.0, 8.0]], [[2.0, 8.0, 8.0], [4.0, 5.0, 3.0], [5.0, 1.0, 7.0]]], [[[4.0, 1.0, 
7.0], [4.0, 3.0, 2.0], [8.0, 2.0, 1.0]], [[8.0, 1.0, 4.0], [8.0, 2.0, 8.0], [3.0, 8.0, 7.0]], [[5.0, 2.0, 3.0], [6.0, 3.0, 
3.0], [3.0, 7.0, 4.0]]], [[[3.0, 8.0, 6.0], [6.0, 8.0, 4.0], [7.0, 1.0, 2.0]], [[6.0, 4.0, 3.0], [7.0, 8.0, 6.0], [7.0, 6.0, 
2.0]], [[6.0, 2.0, 5.0], [4.0, 2.0, 2.0], [8.0, 3.0, 6.0]]]]))
      val=Data(numpy.array([[[[5.0, 1.0, 8.0]]], [[[5.0, 1.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 4.0]]], [[[3.0, 8.0, 7.0]]]]))
      arg[2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 5.0], [7.0, 5.0, 6.0], [1.0, 5.0, 7.0]], [[6.0, 6.0, 5.0], [6.0, 5.0, 4.0], [2.0, 
7.0, 6.0]], [[1.0, 1.0, 3.0], [1.0, 7.0, 4.0], [3.0, 5.0, 7.0]]], [[[8.0, 2.0, 2.0], [5.0, 3.0, 7.0], [2.0, 6.0, 8.0]], [[1.0, 
3.0, 2.0], [2.0, 6.0, 6.0], [6.0, 3.0, 8.0]], [[6.0, 8.0, 3.0], [6.0, 2.0, 5.0], [5.0, 2.0, 4.0]]], [[[7.0, 7.0, 7.0], [8.0, 
5.0, 8.0], [8.0, 7.0, 6.0]], [[3.0, 8.0, 6.0], [5.0, 1.0, 8.0], [3.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [5.0, 1.0, 6.0], [2.0, 2.0, 
7.0]]], [[[2.0, 1.0, 1.0], [3.0, 3.0, 2.0], [2.0, 1.0, 6.0]], [[7.0, 2.0, 6.0], [5.0, 1.0, 3.0], [6.0, 1.0, 6.0]], [[1.0, 2.0, 
6.0], [6.0, 3.0, 7.0], [8.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 6.0], [5.0, 2.0, 3.0], [8.0, 5.0, 8.0]], 
[[6.0, 4.0, 5.0], [1.0, 3.0, 6.0], [1.0, 5.0, 8.0]], [[3.0, 4.0, 4.0], [5.0, 7.0, 1.0], [2.0, 2.0, 8.0]]], [[[8.0, 5.0, 4.0], 
[8.0, 8.0, 5.0], [2.0, 3.0, 8.0]], [[5.0, 5.0, 6.0], [6.0, 4.0, 6.0], [8.0, 5.0, 8.0]], [[2.0, 8.0, 8.0], [4.0, 5.0, 3.0], 
[5.0, 1.0, 7.0]]], [[[4.0, 1.0, 7.0], [4.0, 3.0, 2.0], [8.0, 2.0, 1.0]], [[8.0, 1.0, 4.0], [4.0, 3.0, 4.0], [3.0, 8.0, 7.0]], 
[[5.0, 2.0, 3.0], [6.0, 3.0, 3.0], [3.0, 7.0, 4.0]]], [[[3.0, 8.0, 6.0], [6.0, 8.0, 4.0], [7.0, 1.0, 2.0]], [[6.0, 4.0, 3.0], 
[3.0, 8.0, 7.0], [7.0, 6.0, 2.0]], [[6.0, 2.0, 5.0], [4.0, 2.0, 2.0], [8.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LRRN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 6.0], [5.0, 8.0, 3.0], [6.0, 1.0, 5.0]], [[2.0, 6.0, 4.0], [3.0, 2.0, 5.0], [2.0, 5.0, 
5.0]], [[5.0, 6.0, 8.0], [5.0, 8.0, 2.0], [2.0, 1.0, 8.0]]], [[[5.0, 8.0, 7.0], [3.0, 1.0, 8.0], [3.0, 6.0, 4.0]], [[3.0, 3.0, 
6.0], [8.0, 2.0, 2.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 4.0], [4.0, 5.0, 2.0], [3.0, 3.0, 5.0]]], [[[5.0, 4.0, 5.0], [7.0, 7.0, 
3.0], [4.0, 3.0, 8.0]], [[6.0, 8.0, 8.0], [6.0, 5.0, 7.0], [7.0, 1.0, 2.0]], [[8.0, 3.0, 6.0], [6.0, 3.0, 3.0], [5.0, 3.0, 
6.0]]], [[[3.0, 3.0, 8.0], [3.0, 6.0, 7.0], [8.0, 4.0, 3.0]], [[2.0, 2.0, 8.0], [4.0, 6.0, 5.0], [3.0, 8.0, 6.0]], [[4.0, 4.0, 
1.0], [5.0, 4.0, 4.0], [7.0, 1.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 4.0], [1.0, 1.0, 2.0], [3.0, 3.0, 3.0]], [[5.0, 3.0, 4.0], [1.0, 5.0, 
8.0], [7.0, 7.0, 2.0]], [[2.0, 2.0, 7.0], [8.0, 7.0, 4.0], [8.0, 1.0, 5.0]]], [[[7.0, 5.0, 2.0], [5.0, 2.0, 3.0], [4.0, 6.0, 
5.0]], [[7.0, 6.0, 1.0], [6.0, 2.0, 3.0], [4.0, 8.0, 8.0]], [[2.0, 4.0, 1.0], [8.0, 1.0, 5.0], [2.0, 2.0, 3.0]]], [[[8.0, 5.0, 
5.0], [4.0, 1.0, 4.0], [8.0, 7.0, 7.0]], [[5.0, 2.0, 4.0], [1.0, 7.0, 1.0], [1.0, 3.0, 7.0]], [[3.0, 2.0, 3.0], [2.0, 1.0, 
6.0], [4.0, 2.0, 7.0]]], [[[6.0, 6.0, 5.0], [6.0, 7.0, 3.0], [8.0, 7.0, 1.0]], [[2.0, 6.0, 5.0], [5.0, 6.0, 1.0], [1.0, 4.0, 
2.0]], [[2.0, 6.0, 2.0], [2.0, 3.0, 3.0], [1.0, 8.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 8.0, 5.0]]], [[[1.0, 5.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 5.0, 6.0]]], 
[[[3.0, 6.0, 6.0]]]])
      arg[2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 6.0], [5.0, 8.0, 3.0], [6.0, 1.0, 5.0]], [[2.0, 6.0, 4.0], [3.0, 2.0, 5.0], [2.0, 
5.0, 5.0]], [[5.0, 6.0, 8.0], [5.0, 8.0, 2.0], [2.0, 1.0, 8.0]]], [[[5.0, 8.0, 7.0], [3.0, 1.0, 8.0], [3.0, 6.0, 4.0]], [[3.0, 
3.0, 6.0], [8.0, 2.0, 2.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 4.0], [4.0, 5.0, 2.0], [3.0, 3.0, 5.0]]], [[[5.0, 4.0, 5.0], [7.0, 
7.0, 3.0], [4.0, 3.0, 8.0]], [[6.0, 8.0, 8.0], [8.0, 8.0, 5.0], [7.0, 1.0, 2.0]], [[8.0, 3.0, 6.0], [6.0, 3.0, 3.0], [5.0, 3.0, 
6.0]]], [[[3.0, 3.0, 8.0], [3.0, 6.0, 7.0], [8.0, 4.0, 3.0]], [[2.0, 2.0, 8.0], [1.0, 5.0, 3.0], [3.0, 8.0, 6.0]], [[4.0, 4.0, 
1.0], [5.0, 4.0, 4.0], [7.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 4.0], [1.0, 1.0, 2.0], [3.0, 3.0, 3.0]], 
[[5.0, 3.0, 4.0], [1.0, 5.0, 8.0], [7.0, 7.0, 2.0]], [[2.0, 2.0, 7.0], [8.0, 7.0, 4.0], [8.0, 1.0, 5.0]]], [[[7.0, 5.0, 2.0], 
[5.0, 2.0, 3.0], [4.0, 6.0, 5.0]], [[7.0, 6.0, 1.0], [6.0, 2.0, 3.0], [4.0, 8.0, 8.0]], [[2.0, 4.0, 1.0], [8.0, 1.0, 5.0], 
[2.0, 2.0, 3.0]]], [[[8.0, 5.0, 5.0], [4.0, 1.0, 4.0], [8.0, 7.0, 7.0]], [[5.0, 2.0, 4.0], [5.0, 5.0, 6.0], [1.0, 3.0, 7.0]], 
[[3.0, 2.0, 3.0], [2.0, 1.0, 6.0], [4.0, 2.0, 7.0]]], [[[6.0, 6.0, 5.0], [6.0, 7.0, 3.0], [8.0, 7.0, 1.0]], [[2.0, 6.0, 5.0], 
[3.0, 6.0, 6.0], [1.0, 4.0, 2.0]], [[2.0, 6.0, 2.0], [2.0, 3.0, 3.0], [1.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UPUN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 2.0], [2.0, 4.0, 2.0], [5.0, 8.0, 7.0]], [[7.0, 8.0, 6.0], [2.0, 6.0, 3.0], [3.0, 3.0, 
6.0]], [[2.0, 4.0, 7.0], [7.0, 2.0, 1.0], [4.0, 2.0, 4.0]]], [[[4.0, 4.0, 1.0], [3.0, 8.0, 5.0], [5.0, 7.0, 3.0]], [[6.0, 6.0, 
4.0], [6.0, 6.0, 5.0], [1.0, 5.0, 1.0]], [[1.0, 5.0, 6.0], [7.0, 2.0, 5.0], [1.0, 6.0, 6.0]]], [[[3.0, 2.0, 5.0], [6.0, 7.0, 
6.0], [2.0, 5.0, 6.0]], [[7.0, 8.0, 8.0], [4.0, 6.0, 6.0], [6.0, 4.0, 5.0]], [[8.0, 5.0, 8.0], [8.0, 4.0, 7.0], [5.0, 2.0, 
6.0]]], [[[2.0, 5.0, 5.0], [3.0, 5.0, 4.0], [5.0, 5.0, 5.0]], [[8.0, 3.0, 7.0], [3.0, 5.0, 5.0], [4.0, 2.0, 7.0]], [[3.0, 4.0, 
3.0], [6.0, 3.0, 7.0], [7.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 2.0], [6.0, 1.0, 6.0], [2.0, 5.0, 4.0]], [[2.0, 6.0, 8.0], [6.0, 5.0, 
6.0], [2.0, 1.0, 5.0]], [[8.0, 8.0, 7.0], [8.0, 7.0, 3.0], [2.0, 6.0, 5.0]]], [[[3.0, 2.0, 2.0], [6.0, 1.0, 4.0], [8.0, 5.0, 
7.0]], [[4.0, 8.0, 8.0], [1.0, 3.0, 1.0], [3.0, 5.0, 4.0]], [[6.0, 6.0, 3.0], [4.0, 7.0, 6.0], [3.0, 1.0, 7.0]]], [[[5.0, 6.0, 
5.0], [4.0, 7.0, 2.0], [1.0, 5.0, 4.0]], [[1.0, 6.0, 4.0], [5.0, 5.0, 6.0], [3.0, 4.0, 2.0]], [[6.0, 7.0, 3.0], [6.0, 5.0, 
1.0], [1.0, 1.0, 3.0]]], [[[6.0, 8.0, 5.0], [8.0, 4.0, 2.0], [6.0, 4.0, 7.0]], [[4.0, 6.0, 5.0], [2.0, 7.0, 7.0], [8.0, 6.0, 
7.0]], [[6.0, 2.0, 3.0], [7.0, 2.0, 3.0], [5.0, 1.0, 6.0]]]]))
      val=numpy.array([[[1.0, 4.0, 1.0], [2.0, 3.0, 1.0]], [[5.0, 7.0, 6.0], [6.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [8.0, 7.0, 
3.0]]])
      arg[:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 2.0], [2.0, 4.0, 2.0], [5.0, 8.0, 7.0]], [[1.0, 4.0, 1.0], [2.0, 3.0, 1.0], [3.0, 
3.0, 6.0]], [[2.0, 4.0, 7.0], [7.0, 2.0, 1.0], [4.0, 2.0, 4.0]]], [[[4.0, 4.0, 1.0], [3.0, 8.0, 5.0], [5.0, 7.0, 3.0]], [[5.0, 
7.0, 6.0], [6.0, 5.0, 8.0], [1.0, 5.0, 1.0]], [[1.0, 5.0, 6.0], [7.0, 2.0, 5.0], [1.0, 6.0, 6.0]]], [[[3.0, 2.0, 5.0], [6.0, 
7.0, 6.0], [2.0, 5.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 7.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 5.0, 8.0], [8.0, 4.0, 7.0], [5.0, 2.0, 
6.0]]], [[[2.0, 5.0, 5.0], [3.0, 5.0, 4.0], [5.0, 5.0, 5.0]], [[8.0, 3.0, 7.0], [3.0, 5.0, 5.0], [4.0, 2.0, 7.0]], [[3.0, 4.0, 
3.0], [6.0, 3.0, 7.0], [7.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 2.0], [6.0, 1.0, 6.0], [2.0, 5.0, 4.0]], 
[[1.0, 4.0, 1.0], [2.0, 3.0, 1.0], [2.0, 1.0, 5.0]], [[8.0, 8.0, 7.0], [8.0, 7.0, 3.0], [2.0, 6.0, 5.0]]], [[[3.0, 2.0, 2.0], 
[6.0, 1.0, 4.0], [8.0, 5.0, 7.0]], [[5.0, 7.0, 6.0], [6.0, 5.0, 8.0], [3.0, 5.0, 4.0]], [[6.0, 6.0, 3.0], [4.0, 7.0, 6.0], 
[3.0, 1.0, 7.0]]], [[[5.0, 6.0, 5.0], [4.0, 7.0, 2.0], [1.0, 5.0, 4.0]], [[7.0, 8.0, 3.0], [8.0, 7.0, 3.0], [3.0, 4.0, 2.0]], 
[[6.0, 7.0, 3.0], [6.0, 5.0, 1.0], [1.0, 1.0, 3.0]]], [[[6.0, 8.0, 5.0], [8.0, 4.0, 2.0], [6.0, 4.0, 7.0]], [[4.0, 6.0, 5.0], 
[2.0, 7.0, 7.0], [8.0, 6.0, 7.0]], [[6.0, 2.0, 3.0], [7.0, 2.0, 3.0], [5.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UPUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [7.0, 6.0, 5.0], [3.0, 3.0, 3.0]], [[3.0, 5.0, 1.0], [8.0, 5.0, 1.0], [2.0, 1.0, 
2.0]], [[5.0, 2.0, 8.0], [7.0, 8.0, 2.0], [6.0, 7.0, 3.0]]], [[[2.0, 4.0, 7.0], [6.0, 5.0, 1.0], [8.0, 2.0, 8.0]], [[7.0, 6.0, 
6.0], [2.0, 8.0, 1.0], [3.0, 1.0, 1.0]], [[4.0, 2.0, 8.0], [2.0, 3.0, 8.0], [7.0, 1.0, 1.0]]], [[[2.0, 5.0, 6.0], [2.0, 6.0, 
5.0], [2.0, 6.0, 7.0]], [[2.0, 5.0, 6.0], [6.0, 5.0, 8.0], [8.0, 4.0, 7.0]], [[4.0, 7.0, 4.0], [8.0, 6.0, 4.0], [7.0, 2.0, 
6.0]]], [[[7.0, 5.0, 2.0], [2.0, 3.0, 7.0], [4.0, 7.0, 5.0]], [[4.0, 1.0, 6.0], [6.0, 5.0, 1.0], [1.0, 2.0, 6.0]], [[2.0, 1.0, 
5.0], [7.0, 7.0, 6.0], [3.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 3.0], [6.0, 5.0, 7.0], [1.0, 3.0, 1.0]], [[4.0, 1.0, 1.0], [4.0, 5.0, 
7.0], [4.0, 7.0, 5.0]], [[8.0, 2.0, 6.0], [4.0, 5.0, 6.0], [3.0, 4.0, 8.0]]], [[[2.0, 2.0, 8.0], [1.0, 3.0, 8.0], [7.0, 4.0, 
3.0]], [[1.0, 3.0, 3.0], [6.0, 7.0, 8.0], [4.0, 6.0, 8.0]], [[6.0, 7.0, 3.0], [4.0, 3.0, 5.0], [2.0, 2.0, 3.0]]], [[[2.0, 7.0, 
1.0], [7.0, 8.0, 3.0], [8.0, 7.0, 5.0]], [[6.0, 7.0, 8.0], [7.0, 3.0, 1.0], [5.0, 8.0, 5.0]], [[4.0, 6.0, 3.0], [1.0, 2.0, 
3.0], [3.0, 5.0, 8.0]]], [[[8.0, 2.0, 6.0], [8.0, 2.0, 7.0], [5.0, 7.0, 4.0]], [[2.0, 8.0, 1.0], [2.0, 5.0, 2.0], [8.0, 4.0, 
4.0]], [[2.0, 8.0, 8.0], [2.0, 6.0, 2.0], [6.0, 3.0, 6.0]]]]))
      val=Data(numpy.array([[[1.0, 2.0, 2.0], [6.0, 5.0, 6.0]], [[7.0, 1.0, 1.0], [4.0, 8.0, 8.0]], [[4.0, 4.0, 4.0], [8.0, 
7.0, 7.0]]]),self.functionspace)
      arg[:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [7.0, 6.0, 5.0], [3.0, 3.0, 3.0]], [[1.0, 2.0, 2.0], [6.0, 5.0, 6.0], [2.0, 
1.0, 2.0]], [[5.0, 2.0, 8.0], [7.0, 8.0, 2.0], [6.0, 7.0, 3.0]]], [[[2.0, 4.0, 7.0], [6.0, 5.0, 1.0], [8.0, 2.0, 8.0]], [[7.0, 
1.0, 1.0], [4.0, 8.0, 8.0], [3.0, 1.0, 1.0]], [[4.0, 2.0, 8.0], [2.0, 3.0, 8.0], [7.0, 1.0, 1.0]]], [[[2.0, 5.0, 6.0], [2.0, 
6.0, 5.0], [2.0, 6.0, 7.0]], [[4.0, 4.0, 4.0], [8.0, 7.0, 7.0], [8.0, 4.0, 7.0]], [[4.0, 7.0, 4.0], [8.0, 6.0, 4.0], [7.0, 2.0, 
6.0]]], [[[7.0, 5.0, 2.0], [2.0, 3.0, 7.0], [4.0, 7.0, 5.0]], [[4.0, 1.0, 6.0], [6.0, 5.0, 1.0], [1.0, 2.0, 6.0]], [[2.0, 1.0, 
5.0], [7.0, 7.0, 6.0], [3.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 3.0], [6.0, 5.0, 7.0], [1.0, 3.0, 1.0]], 
[[1.0, 2.0, 2.0], [6.0, 5.0, 6.0], [4.0, 7.0, 5.0]], [[8.0, 2.0, 6.0], [4.0, 5.0, 6.0], [3.0, 4.0, 8.0]]], [[[2.0, 2.0, 8.0], 
[1.0, 3.0, 8.0], [7.0, 4.0, 3.0]], [[7.0, 1.0, 1.0], [4.0, 8.0, 8.0], [4.0, 6.0, 8.0]], [[6.0, 7.0, 3.0], [4.0, 3.0, 5.0], 
[2.0, 2.0, 3.0]]], [[[2.0, 7.0, 1.0], [7.0, 8.0, 3.0], [8.0, 7.0, 5.0]], [[4.0, 4.0, 4.0], [8.0, 7.0, 7.0], [5.0, 8.0, 5.0]], 
[[4.0, 6.0, 3.0], [1.0, 2.0, 3.0], [3.0, 5.0, 8.0]]], [[[8.0, 2.0, 6.0], [8.0, 2.0, 7.0], [5.0, 7.0, 4.0]], [[2.0, 8.0, 1.0], 
[2.0, 5.0, 2.0], [8.0, 4.0, 4.0]], [[2.0, 8.0, 8.0], [2.0, 6.0, 2.0], [6.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UPUN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 8.0], [6.0, 7.0, 1.0], [2.0, 6.0, 4.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 4.0], [5.0, 2.0, 
6.0]], [[3.0, 6.0, 8.0], [5.0, 7.0, 8.0], [8.0, 4.0, 3.0]]], [[[7.0, 6.0, 2.0], [7.0, 2.0, 2.0], [3.0, 6.0, 7.0]], [[4.0, 4.0, 
6.0], [8.0, 2.0, 5.0], [8.0, 8.0, 5.0]], [[5.0, 7.0, 6.0], [2.0, 2.0, 2.0], [1.0, 7.0, 7.0]]], [[[2.0, 5.0, 5.0], [4.0, 2.0, 
2.0], [7.0, 6.0, 3.0]], [[8.0, 4.0, 7.0], [5.0, 7.0, 2.0], [6.0, 3.0, 4.0]], [[6.0, 4.0, 3.0], [4.0, 2.0, 6.0], [8.0, 2.0, 
1.0]]], [[[4.0, 8.0, 6.0], [8.0, 8.0, 4.0], [1.0, 8.0, 4.0]], [[5.0, 4.0, 2.0], [3.0, 1.0, 8.0], [5.0, 2.0, 1.0]], [[4.0, 5.0, 
1.0], [2.0, 4.0, 3.0], [6.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 6.0], [3.0, 3.0, 1.0], [2.0, 4.0, 4.0]], [[6.0, 1.0, 3.0], [7.0, 6.0, 
8.0], [3.0, 5.0, 1.0]], [[5.0, 6.0, 1.0], [3.0, 2.0, 8.0], [2.0, 1.0, 7.0]]], [[[3.0, 5.0, 5.0], [2.0, 5.0, 2.0], [5.0, 2.0, 
1.0]], [[8.0, 8.0, 3.0], [2.0, 4.0, 4.0], [2.0, 3.0, 7.0]], [[2.0, 8.0, 1.0], [2.0, 1.0, 8.0], [1.0, 3.0, 5.0]]], [[[1.0, 4.0, 
1.0], [6.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[7.0, 5.0, 4.0], [4.0, 2.0, 1.0], [4.0, 8.0, 6.0]], [[8.0, 2.0, 4.0], [6.0, 2.0, 
7.0], [3.0, 3.0, 3.0]]], [[[1.0, 8.0, 1.0], [3.0, 7.0, 2.0], [1.0, 8.0, 7.0]], [[1.0, 1.0, 1.0], [4.0, 4.0, 1.0], [5.0, 5.0, 
4.0]], [[8.0, 1.0, 4.0], [6.0, 2.0, 4.0], [4.0, 5.0, 5.0]]]]))
      val=Data(numpy.array([[[8.0, 8.0, 3.0], [2.0, 1.0, 4.0]], [[7.0, 7.0, 8.0], [7.0, 1.0, 2.0]], [[2.0, 2.0, 1.0], [6.0, 
1.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 6.0, 8.0], [5.0, 1.0, 7.0]], [[1.0, 8.0, 1.0], [6.0, 7.0, 8.0]], [[1.0, 1.0, 
3.0], [3.0, 6.0, 3.0]]]))
      arg[:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 8.0], [6.0, 7.0, 1.0], [2.0, 6.0, 4.0]], [[8.0, 8.0, 3.0], [2.0, 1.0, 4.0], [5.0, 
2.0, 6.0]], [[3.0, 6.0, 8.0], [5.0, 7.0, 8.0], [8.0, 4.0, 3.0]]], [[[7.0, 6.0, 2.0], [7.0, 2.0, 2.0], [3.0, 6.0, 7.0]], [[7.0, 
7.0, 8.0], [7.0, 1.0, 2.0], [8.0, 8.0, 5.0]], [[5.0, 7.0, 6.0], [2.0, 2.0, 2.0], [1.0, 7.0, 7.0]]], [[[2.0, 5.0, 5.0], [4.0, 
2.0, 2.0], [7.0, 6.0, 3.0]], [[2.0, 2.0, 1.0], [6.0, 1.0, 1.0], [6.0, 3.0, 4.0]], [[6.0, 4.0, 3.0], [4.0, 2.0, 6.0], [8.0, 2.0, 
1.0]]], [[[4.0, 8.0, 6.0], [8.0, 8.0, 4.0], [1.0, 8.0, 4.0]], [[5.0, 4.0, 2.0], [3.0, 1.0, 8.0], [5.0, 2.0, 1.0]], [[4.0, 5.0, 
1.0], [2.0, 4.0, 3.0], [6.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [3.0, 3.0, 1.0], [2.0, 4.0, 4.0]], 
[[1.0, 6.0, 8.0], [5.0, 1.0, 7.0], [3.0, 5.0, 1.0]], [[5.0, 6.0, 1.0], [3.0, 2.0, 8.0], [2.0, 1.0, 7.0]]], [[[3.0, 5.0, 5.0], 
[2.0, 5.0, 2.0], [5.0, 2.0, 1.0]], [[1.0, 8.0, 1.0], [6.0, 7.0, 8.0], [2.0, 3.0, 7.0]], [[2.0, 8.0, 1.0], [2.0, 1.0, 8.0], 
[1.0, 3.0, 5.0]]], [[[1.0, 4.0, 1.0], [6.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[1.0, 1.0, 3.0], [3.0, 6.0, 3.0], [4.0, 8.0, 6.0]], 
[[8.0, 2.0, 4.0], [6.0, 2.0, 7.0], [3.0, 3.0, 3.0]]], [[[1.0, 8.0, 1.0], [3.0, 7.0, 2.0], [1.0, 8.0, 7.0]], [[1.0, 1.0, 1.0], 
[4.0, 4.0, 1.0], [5.0, 5.0, 4.0]], [[8.0, 1.0, 4.0], [6.0, 2.0, 4.0], [4.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UPUN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 4.0], [7.0, 8.0, 7.0], [3.0, 6.0, 8.0]], [[1.0, 3.0, 2.0], [2.0, 2.0, 6.0], [5.0, 8.0, 
8.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 5.0], [2.0, 3.0, 3.0]]], [[[4.0, 7.0, 7.0], [5.0, 1.0, 4.0], [4.0, 5.0, 3.0]], [[4.0, 2.0, 
1.0], [4.0, 1.0, 6.0], [2.0, 8.0, 5.0]], [[6.0, 8.0, 3.0], [3.0, 2.0, 7.0], [8.0, 6.0, 5.0]]], [[[3.0, 8.0, 5.0], [6.0, 1.0, 
1.0], [2.0, 1.0, 4.0]], [[4.0, 6.0, 3.0], [1.0, 5.0, 6.0], [1.0, 2.0, 4.0]], [[1.0, 5.0, 5.0], [4.0, 7.0, 3.0], [1.0, 7.0, 
6.0]]], [[[8.0, 8.0, 8.0], [4.0, 3.0, 2.0], [8.0, 2.0, 1.0]], [[3.0, 3.0, 2.0], [8.0, 4.0, 8.0], [6.0, 6.0, 7.0]], [[1.0, 7.0, 
2.0], [5.0, 3.0, 4.0], [3.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 5.0], [6.0, 6.0, 2.0], [5.0, 2.0, 5.0]], [[8.0, 1.0, 5.0], [7.0, 6.0, 
8.0], [3.0, 7.0, 8.0]], [[3.0, 4.0, 6.0], [1.0, 7.0, 8.0], [6.0, 8.0, 6.0]]], [[[8.0, 1.0, 6.0], [3.0, 8.0, 8.0], [2.0, 3.0, 
5.0]], [[2.0, 8.0, 7.0], [4.0, 8.0, 7.0], [7.0, 6.0, 1.0]], [[3.0, 1.0, 7.0], [4.0, 6.0, 6.0], [3.0, 1.0, 7.0]]], [[[6.0, 3.0, 
5.0], [5.0, 1.0, 7.0], [7.0, 1.0, 3.0]], [[5.0, 6.0, 8.0], [7.0, 7.0, 8.0], [4.0, 4.0, 1.0]], [[4.0, 5.0, 2.0], [3.0, 1.0, 
7.0], [3.0, 2.0, 8.0]]], [[[6.0, 2.0, 4.0], [3.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[7.0, 7.0, 3.0], [7.0, 2.0, 7.0], [8.0, 7.0, 
3.0]], [[1.0, 4.0, 5.0], [1.0, 6.0, 3.0], [4.0, 5.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 4.0, 1.0], [1.0, 6.0, 5.0]], [[7.0, 2.0, 1.0], [6.0, 7.0, 1.0]], [[1.0, 7.0, 1.0], [8.0, 
6.0, 5.0]]])+(1.-msk_val)*numpy.array([[[5.0, 2.0, 2.0], [4.0, 5.0, 5.0]], [[8.0, 4.0, 6.0], [2.0, 2.0, 3.0]], [[7.0, 1.0, 
4.0], [8.0, 3.0, 4.0]]])
      arg[:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 4.0], [7.0, 8.0, 7.0], [3.0, 6.0, 8.0]], [[4.0, 4.0, 1.0], [1.0, 6.0, 5.0], [5.0, 
8.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 5.0], [2.0, 3.0, 3.0]]], [[[4.0, 7.0, 7.0], [5.0, 1.0, 4.0], [4.0, 5.0, 3.0]], [[7.0, 
2.0, 1.0], [6.0, 7.0, 1.0], [2.0, 8.0, 5.0]], [[6.0, 8.0, 3.0], [3.0, 2.0, 7.0], [8.0, 6.0, 5.0]]], [[[3.0, 8.0, 5.0], [6.0, 
1.0, 1.0], [2.0, 1.0, 4.0]], [[1.0, 7.0, 1.0], [8.0, 6.0, 5.0], [1.0, 2.0, 4.0]], [[1.0, 5.0, 5.0], [4.0, 7.0, 3.0], [1.0, 7.0, 
6.0]]], [[[8.0, 8.0, 8.0], [4.0, 3.0, 2.0], [8.0, 2.0, 1.0]], [[3.0, 3.0, 2.0], [8.0, 4.0, 8.0], [6.0, 6.0, 7.0]], [[1.0, 7.0, 
2.0], [5.0, 3.0, 4.0], [3.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 5.0], [6.0, 6.0, 2.0], [5.0, 2.0, 5.0]], 
[[5.0, 2.0, 2.0], [4.0, 5.0, 5.0], [3.0, 7.0, 8.0]], [[3.0, 4.0, 6.0], [1.0, 7.0, 8.0], [6.0, 8.0, 6.0]]], [[[8.0, 1.0, 6.0], 
[3.0, 8.0, 8.0], [2.0, 3.0, 5.0]], [[8.0, 4.0, 6.0], [2.0, 2.0, 3.0], [7.0, 6.0, 1.0]], [[3.0, 1.0, 7.0], [4.0, 6.0, 6.0], 
[3.0, 1.0, 7.0]]], [[[6.0, 3.0, 5.0], [5.0, 1.0, 7.0], [7.0, 1.0, 3.0]], [[7.0, 1.0, 4.0], [8.0, 3.0, 4.0], [4.0, 4.0, 1.0]], 
[[4.0, 5.0, 2.0], [3.0, 1.0, 7.0], [3.0, 2.0, 8.0]]], [[[6.0, 2.0, 4.0], [3.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[7.0, 7.0, 3.0], 
[7.0, 2.0, 7.0], [8.0, 7.0, 3.0]], [[1.0, 4.0, 5.0], [1.0, 6.0, 3.0], [4.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLUN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 7.0], [8.0, 6.0, 5.0], [3.0, 3.0, 5.0]], [[1.0, 3.0, 1.0], [7.0, 2.0, 7.0], [2.0, 8.0, 
5.0]], [[3.0, 7.0, 3.0], [6.0, 7.0, 4.0], [2.0, 4.0, 4.0]]], [[[3.0, 5.0, 4.0], [4.0, 6.0, 2.0], [2.0, 3.0, 7.0]], [[4.0, 4.0, 
6.0], [3.0, 4.0, 1.0], [3.0, 8.0, 1.0]], [[3.0, 3.0, 2.0], [7.0, 5.0, 7.0], [1.0, 8.0, 1.0]]], [[[8.0, 5.0, 8.0], [1.0, 7.0, 
4.0], [7.0, 5.0, 6.0]], [[3.0, 7.0, 7.0], [8.0, 1.0, 7.0], [2.0, 1.0, 1.0]], [[5.0, 7.0, 8.0], [3.0, 6.0, 6.0], [3.0, 4.0, 
2.0]]], [[[3.0, 5.0, 1.0], [1.0, 8.0, 8.0], [2.0, 2.0, 5.0]], [[2.0, 2.0, 2.0], [7.0, 7.0, 6.0], [2.0, 2.0, 1.0]], [[2.0, 5.0, 
1.0], [5.0, 8.0, 1.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 5.0], [4.0, 6.0, 1.0], [8.0, 7.0, 4.0]], [[1.0, 5.0, 2.0], [2.0, 3.0, 
3.0], [6.0, 2.0, 7.0]], [[2.0, 5.0, 4.0], [6.0, 1.0, 1.0], [1.0, 4.0, 5.0]]], [[[6.0, 5.0, 3.0], [8.0, 3.0, 2.0], [4.0, 1.0, 
7.0]], [[3.0, 4.0, 1.0], [1.0, 7.0, 5.0], [5.0, 7.0, 8.0]], [[5.0, 6.0, 4.0], [7.0, 4.0, 3.0], [3.0, 7.0, 2.0]]], [[[7.0, 2.0, 
5.0], [8.0, 7.0, 3.0], [5.0, 5.0, 6.0]], [[7.0, 4.0, 2.0], [6.0, 1.0, 8.0], [3.0, 5.0, 8.0]], [[8.0, 6.0, 3.0], [2.0, 6.0, 
6.0], [2.0, 3.0, 1.0]]], [[[8.0, 2.0, 6.0], [8.0, 4.0, 5.0], [4.0, 4.0, 1.0]], [[4.0, 6.0, 6.0], [7.0, 3.0, 3.0], [2.0, 5.0, 
8.0]], [[4.0, 8.0, 8.0], [1.0, 4.0, 4.0], [8.0, 5.0, 4.0]]]]))
      val=numpy.array([[[[3.0, 8.0, 6.0], [5.0, 2.0, 6.0]]], [[[8.0, 6.0, 4.0], [7.0, 5.0, 4.0]]]])
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 7.0], [8.0, 6.0, 5.0], [3.0, 3.0, 5.0]], [[1.0, 3.0, 1.0], [7.0, 2.0, 7.0], [2.0, 
8.0, 5.0]], [[3.0, 7.0, 3.0], [6.0, 7.0, 4.0], [2.0, 4.0, 4.0]]], [[[3.0, 5.0, 4.0], [4.0, 6.0, 2.0], [2.0, 3.0, 7.0]], [[4.0, 
4.0, 6.0], [3.0, 4.0, 1.0], [3.0, 8.0, 1.0]], [[3.0, 3.0, 2.0], [7.0, 5.0, 7.0], [1.0, 8.0, 1.0]]], [[[8.0, 5.0, 8.0], [1.0, 
7.0, 4.0], [7.0, 5.0, 6.0]], [[3.0, 7.0, 7.0], [8.0, 1.0, 7.0], [2.0, 1.0, 1.0]], [[3.0, 8.0, 6.0], [5.0, 2.0, 6.0], [3.0, 4.0, 
2.0]]], [[[3.0, 5.0, 1.0], [1.0, 8.0, 8.0], [2.0, 2.0, 5.0]], [[2.0, 2.0, 2.0], [7.0, 7.0, 6.0], [2.0, 2.0, 1.0]], [[8.0, 6.0, 
4.0], [7.0, 5.0, 4.0], [8.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 5.0], [4.0, 6.0, 1.0], [8.0, 7.0, 4.0]], 
[[1.0, 5.0, 2.0], [2.0, 3.0, 3.0], [6.0, 2.0, 7.0]], [[2.0, 5.0, 4.0], [6.0, 1.0, 1.0], [1.0, 4.0, 5.0]]], [[[6.0, 5.0, 3.0], 
[8.0, 3.0, 2.0], [4.0, 1.0, 7.0]], [[3.0, 4.0, 1.0], [1.0, 7.0, 5.0], [5.0, 7.0, 8.0]], [[5.0, 6.0, 4.0], [7.0, 4.0, 3.0], 
[3.0, 7.0, 2.0]]], [[[7.0, 2.0, 5.0], [8.0, 7.0, 3.0], [5.0, 5.0, 6.0]], [[7.0, 4.0, 2.0], [6.0, 1.0, 8.0], [3.0, 5.0, 8.0]], 
[[3.0, 8.0, 6.0], [5.0, 2.0, 6.0], [2.0, 3.0, 1.0]]], [[[8.0, 2.0, 6.0], [8.0, 4.0, 5.0], [4.0, 4.0, 1.0]], [[4.0, 6.0, 6.0], 
[7.0, 3.0, 3.0], [2.0, 5.0, 8.0]], [[8.0, 6.0, 4.0], [7.0, 5.0, 4.0], [8.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLUN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 2.0], [4.0, 3.0, 8.0], [8.0, 4.0, 7.0]], [[7.0, 3.0, 2.0], [7.0, 5.0, 4.0], [1.0, 7.0, 
1.0]], [[2.0, 4.0, 7.0], [2.0, 8.0, 5.0], [4.0, 4.0, 6.0]]], [[[4.0, 3.0, 5.0], [3.0, 7.0, 5.0], [7.0, 2.0, 8.0]], [[2.0, 2.0, 
3.0], [8.0, 3.0, 6.0], [4.0, 5.0, 2.0]], [[5.0, 3.0, 8.0], [5.0, 5.0, 7.0], [6.0, 8.0, 2.0]]], [[[7.0, 4.0, 1.0], [6.0, 2.0, 
7.0], [2.0, 2.0, 7.0]], [[3.0, 4.0, 2.0], [3.0, 3.0, 3.0], [6.0, 4.0, 4.0]], [[4.0, 8.0, 3.0], [4.0, 5.0, 3.0], [3.0, 1.0, 
8.0]]], [[[2.0, 4.0, 1.0], [4.0, 8.0, 7.0], [8.0, 5.0, 4.0]], [[3.0, 2.0, 7.0], [2.0, 8.0, 4.0], [1.0, 7.0, 2.0]], [[5.0, 8.0, 
8.0], [3.0, 1.0, 8.0], [1.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 7.0], [3.0, 6.0, 5.0], [3.0, 2.0, 1.0]], [[8.0, 4.0, 8.0], [7.0, 1.0, 
3.0], [4.0, 7.0, 2.0]], [[3.0, 8.0, 6.0], [1.0, 5.0, 8.0], [1.0, 3.0, 1.0]]], [[[2.0, 1.0, 7.0], [4.0, 2.0, 8.0], [7.0, 1.0, 
8.0]], [[3.0, 3.0, 5.0], [6.0, 5.0, 2.0], [1.0, 2.0, 7.0]], [[8.0, 8.0, 3.0], [3.0, 5.0, 6.0], [7.0, 3.0, 1.0]]], [[[2.0, 2.0, 
2.0], [8.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[7.0, 7.0, 8.0], [8.0, 1.0, 5.0], [8.0, 2.0, 1.0]], [[7.0, 6.0, 6.0], [5.0, 7.0, 
8.0], [8.0, 8.0, 7.0]]], [[[4.0, 6.0, 1.0], [5.0, 8.0, 2.0], [8.0, 5.0, 5.0]], [[1.0, 2.0, 8.0], [5.0, 4.0, 7.0], [5.0, 1.0, 
7.0]], [[4.0, 1.0, 8.0], [8.0, 7.0, 8.0], [7.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[[[6.0, 5.0, 7.0], [8.0, 1.0, 1.0]]], [[[2.0, 7.0, 8.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 2.0], [4.0, 3.0, 8.0], [8.0, 4.0, 7.0]], [[7.0, 3.0, 2.0], [7.0, 5.0, 4.0], [1.0, 
7.0, 1.0]], [[2.0, 4.0, 7.0], [2.0, 8.0, 5.0], [4.0, 4.0, 6.0]]], [[[4.0, 3.0, 5.0], [3.0, 7.0, 5.0], [7.0, 2.0, 8.0]], [[2.0, 
2.0, 3.0], [8.0, 3.0, 6.0], [4.0, 5.0, 2.0]], [[5.0, 3.0, 8.0], [5.0, 5.0, 7.0], [6.0, 8.0, 2.0]]], [[[7.0, 4.0, 1.0], [6.0, 
2.0, 7.0], [2.0, 2.0, 7.0]], [[3.0, 4.0, 2.0], [3.0, 3.0, 3.0], [6.0, 4.0, 4.0]], [[6.0, 5.0, 7.0], [8.0, 1.0, 1.0], [3.0, 1.0, 
8.0]]], [[[2.0, 4.0, 1.0], [4.0, 8.0, 7.0], [8.0, 5.0, 4.0]], [[3.0, 2.0, 7.0], [2.0, 8.0, 4.0], [1.0, 7.0, 2.0]], [[2.0, 7.0, 
8.0], [5.0, 4.0, 1.0], [1.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 7.0], [3.0, 6.0, 5.0], [3.0, 2.0, 1.0]], 
[[8.0, 4.0, 8.0], [7.0, 1.0, 3.0], [4.0, 7.0, 2.0]], [[3.0, 8.0, 6.0], [1.0, 5.0, 8.0], [1.0, 3.0, 1.0]]], [[[2.0, 1.0, 7.0], 
[4.0, 2.0, 8.0], [7.0, 1.0, 8.0]], [[3.0, 3.0, 5.0], [6.0, 5.0, 2.0], [1.0, 2.0, 7.0]], [[8.0, 8.0, 3.0], [3.0, 5.0, 6.0], 
[7.0, 3.0, 1.0]]], [[[2.0, 2.0, 2.0], [8.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[7.0, 7.0, 8.0], [8.0, 1.0, 5.0], [8.0, 2.0, 1.0]], 
[[6.0, 5.0, 7.0], [8.0, 1.0, 1.0], [8.0, 8.0, 7.0]]], [[[4.0, 6.0, 1.0], [5.0, 8.0, 2.0], [8.0, 5.0, 5.0]], [[1.0, 2.0, 8.0], 
[5.0, 4.0, 7.0], [5.0, 1.0, 7.0]], [[2.0, 7.0, 8.0], [5.0, 4.0, 1.0], [7.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLUN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 8.0], [7.0, 8.0, 7.0], [8.0, 7.0, 7.0]], [[2.0, 4.0, 6.0], [2.0, 1.0, 3.0], [4.0, 5.0, 
4.0]], [[5.0, 8.0, 5.0], [3.0, 7.0, 5.0], [6.0, 6.0, 8.0]]], [[[8.0, 1.0, 3.0], [5.0, 2.0, 3.0], [3.0, 3.0, 6.0]], [[6.0, 1.0, 
2.0], [2.0, 3.0, 2.0], [1.0, 3.0, 2.0]], [[4.0, 3.0, 7.0], [3.0, 3.0, 5.0], [2.0, 6.0, 7.0]]], [[[5.0, 2.0, 2.0], [5.0, 2.0, 
2.0], [8.0, 4.0, 4.0]], [[8.0, 8.0, 3.0], [2.0, 4.0, 2.0], [5.0, 2.0, 6.0]], [[5.0, 2.0, 2.0], [2.0, 5.0, 5.0], [2.0, 6.0, 
6.0]]], [[[7.0, 5.0, 3.0], [4.0, 7.0, 6.0], [6.0, 6.0, 2.0]], [[5.0, 7.0, 1.0], [5.0, 6.0, 8.0], [8.0, 7.0, 8.0]], [[8.0, 8.0, 
6.0], [5.0, 8.0, 5.0], [4.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 8.0], [1.0, 1.0, 3.0], [6.0, 5.0, 1.0]], [[1.0, 5.0, 4.0], [6.0, 1.0, 
1.0], [5.0, 3.0, 1.0]], [[4.0, 1.0, 8.0], [3.0, 1.0, 1.0], [8.0, 2.0, 4.0]]], [[[1.0, 1.0, 3.0], [7.0, 3.0, 7.0], [8.0, 8.0, 
6.0]], [[8.0, 1.0, 8.0], [7.0, 5.0, 1.0], [7.0, 8.0, 3.0]], [[2.0, 5.0, 6.0], [4.0, 1.0, 7.0], [6.0, 1.0, 1.0]]], [[[6.0, 4.0, 
6.0], [5.0, 6.0, 6.0], [7.0, 2.0, 2.0]], [[2.0, 4.0, 7.0], [3.0, 2.0, 7.0], [8.0, 4.0, 8.0]], [[8.0, 8.0, 4.0], [4.0, 3.0, 
3.0], [6.0, 4.0, 1.0]]], [[[5.0, 5.0, 7.0], [1.0, 5.0, 1.0], [4.0, 5.0, 8.0]], [[3.0, 2.0, 7.0], [3.0, 8.0, 5.0], [3.0, 4.0, 
7.0]], [[4.0, 6.0, 7.0], [6.0, 6.0, 3.0], [4.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0, 7.0, 8.0], [2.0, 5.0, 2.0]]], [[[3.0, 2.0, 4.0], [1.0, 3.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 2.0], [2.0, 1.0, 1.0]]], [[[2.0, 8.0, 5.0], [6.0, 2.0, 8.0]]]]))
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [7.0, 8.0, 7.0], [8.0, 7.0, 7.0]], [[2.0, 4.0, 6.0], [2.0, 1.0, 3.0], [4.0, 
5.0, 4.0]], [[5.0, 8.0, 5.0], [3.0, 7.0, 5.0], [6.0, 6.0, 8.0]]], [[[8.0, 1.0, 3.0], [5.0, 2.0, 3.0], [3.0, 3.0, 6.0]], [[6.0, 
1.0, 2.0], [2.0, 3.0, 2.0], [1.0, 3.0, 2.0]], [[4.0, 3.0, 7.0], [3.0, 3.0, 5.0], [2.0, 6.0, 7.0]]], [[[5.0, 2.0, 2.0], [5.0, 
2.0, 2.0], [8.0, 4.0, 4.0]], [[8.0, 8.0, 3.0], [2.0, 4.0, 2.0], [5.0, 2.0, 6.0]], [[5.0, 7.0, 8.0], [2.0, 5.0, 2.0], [2.0, 6.0, 
6.0]]], [[[7.0, 5.0, 3.0], [4.0, 7.0, 6.0], [6.0, 6.0, 2.0]], [[5.0, 7.0, 1.0], [5.0, 6.0, 8.0], [8.0, 7.0, 8.0]], [[3.0, 2.0, 
4.0], [1.0, 3.0, 4.0], [4.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 8.0], [1.0, 1.0, 3.0], [6.0, 5.0, 1.0]], 
[[1.0, 5.0, 4.0], [6.0, 1.0, 1.0], [5.0, 3.0, 1.0]], [[4.0, 1.0, 8.0], [3.0, 1.0, 1.0], [8.0, 2.0, 4.0]]], [[[1.0, 1.0, 3.0], 
[7.0, 3.0, 7.0], [8.0, 8.0, 6.0]], [[8.0, 1.0, 8.0], [7.0, 5.0, 1.0], [7.0, 8.0, 3.0]], [[2.0, 5.0, 6.0], [4.0, 1.0, 7.0], 
[6.0, 1.0, 1.0]]], [[[6.0, 4.0, 6.0], [5.0, 6.0, 6.0], [7.0, 2.0, 2.0]], [[2.0, 4.0, 7.0], [3.0, 2.0, 7.0], [8.0, 4.0, 8.0]], 
[[7.0, 7.0, 2.0], [2.0, 1.0, 1.0], [6.0, 4.0, 1.0]]], [[[5.0, 5.0, 7.0], [1.0, 5.0, 1.0], [4.0, 5.0, 8.0]], [[3.0, 2.0, 7.0], 
[3.0, 8.0, 5.0], [3.0, 4.0, 7.0]], [[2.0, 8.0, 5.0], [6.0, 2.0, 8.0], [4.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLUN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 6.0], [2.0, 7.0, 3.0], [7.0, 3.0, 3.0]], [[5.0, 3.0, 6.0], [4.0, 6.0, 5.0], [8.0, 7.0, 
4.0]], [[6.0, 1.0, 7.0], [8.0, 7.0, 1.0], [7.0, 6.0, 6.0]]], [[[3.0, 7.0, 4.0], [8.0, 6.0, 7.0], [1.0, 1.0, 3.0]], [[4.0, 4.0, 
5.0], [6.0, 2.0, 7.0], [3.0, 5.0, 1.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 7.0], [5.0, 5.0, 7.0]]], [[[3.0, 3.0, 2.0], [5.0, 4.0, 
1.0], [5.0, 1.0, 1.0]], [[6.0, 4.0, 8.0], [3.0, 2.0, 2.0], [3.0, 3.0, 7.0]], [[2.0, 3.0, 1.0], [7.0, 3.0, 3.0], [4.0, 7.0, 
6.0]]], [[[7.0, 6.0, 7.0], [3.0, 4.0, 5.0], [1.0, 7.0, 8.0]], [[7.0, 3.0, 7.0], [2.0, 7.0, 1.0], [7.0, 2.0, 1.0]], [[7.0, 5.0, 
4.0], [8.0, 7.0, 2.0], [7.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 1.0], [1.0, 4.0, 7.0], [2.0, 8.0, 1.0]], [[3.0, 4.0, 2.0], [6.0, 8.0, 
3.0], [7.0, 3.0, 1.0]], [[1.0, 6.0, 5.0], [8.0, 1.0, 8.0], [7.0, 7.0, 1.0]]], [[[8.0, 4.0, 5.0], [7.0, 1.0, 3.0], [2.0, 1.0, 
2.0]], [[8.0, 2.0, 5.0], [8.0, 1.0, 2.0], [8.0, 5.0, 4.0]], [[6.0, 5.0, 3.0], [5.0, 5.0, 2.0], [6.0, 6.0, 2.0]]], [[[7.0, 3.0, 
2.0], [6.0, 1.0, 5.0], [2.0, 4.0, 2.0]], [[8.0, 1.0, 2.0], [4.0, 5.0, 1.0], [6.0, 8.0, 5.0]], [[6.0, 5.0, 4.0], [4.0, 6.0, 
8.0], [5.0, 8.0, 1.0]]], [[[5.0, 7.0, 7.0], [4.0, 2.0, 6.0], [7.0, 6.0, 3.0]], [[1.0, 7.0, 8.0], [3.0, 5.0, 2.0], [1.0, 2.0, 
2.0]], [[5.0, 2.0, 1.0], [8.0, 4.0, 7.0], [1.0, 2.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 4.0, 5.0], [3.0, 2.0, 5.0]]], [[[7.0, 4.0, 2.0], [2.0, 6.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 1.0, 7.0], [3.0, 7.0, 2.0]]], [[[6.0, 7.0, 4.0], [8.0, 3.0, 8.0]]]])
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 6.0], [2.0, 7.0, 3.0], [7.0, 3.0, 3.0]], [[5.0, 3.0, 6.0], [4.0, 6.0, 5.0], [8.0, 
7.0, 4.0]], [[6.0, 1.0, 7.0], [8.0, 7.0, 1.0], [7.0, 6.0, 6.0]]], [[[3.0, 7.0, 4.0], [8.0, 6.0, 7.0], [1.0, 1.0, 3.0]], [[4.0, 
4.0, 5.0], [6.0, 2.0, 7.0], [3.0, 5.0, 1.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 7.0], [5.0, 5.0, 7.0]]], [[[3.0, 3.0, 2.0], [5.0, 
4.0, 1.0], [5.0, 1.0, 1.0]], [[6.0, 4.0, 8.0], [3.0, 2.0, 2.0], [3.0, 3.0, 7.0]], [[1.0, 4.0, 5.0], [3.0, 2.0, 5.0], [4.0, 7.0, 
6.0]]], [[[7.0, 6.0, 7.0], [3.0, 4.0, 5.0], [1.0, 7.0, 8.0]], [[7.0, 3.0, 7.0], [2.0, 7.0, 1.0], [7.0, 2.0, 1.0]], [[7.0, 4.0, 
2.0], [2.0, 6.0, 4.0], [7.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 1.0], [1.0, 4.0, 7.0], [2.0, 8.0, 1.0]], 
[[3.0, 4.0, 2.0], [6.0, 8.0, 3.0], [7.0, 3.0, 1.0]], [[1.0, 6.0, 5.0], [8.0, 1.0, 8.0], [7.0, 7.0, 1.0]]], [[[8.0, 4.0, 5.0], 
[7.0, 1.0, 3.0], [2.0, 1.0, 2.0]], [[8.0, 2.0, 5.0], [8.0, 1.0, 2.0], [8.0, 5.0, 4.0]], [[6.0, 5.0, 3.0], [5.0, 5.0, 2.0], 
[6.0, 6.0, 2.0]]], [[[7.0, 3.0, 2.0], [6.0, 1.0, 5.0], [2.0, 4.0, 2.0]], [[8.0, 1.0, 2.0], [4.0, 5.0, 1.0], [6.0, 8.0, 5.0]], 
[[3.0, 1.0, 7.0], [3.0, 7.0, 2.0], [5.0, 8.0, 1.0]]], [[[5.0, 7.0, 7.0], [4.0, 2.0, 6.0], [7.0, 6.0, 3.0]], [[1.0, 7.0, 8.0], 
[3.0, 5.0, 2.0], [1.0, 2.0, 2.0]], [[6.0, 7.0, 4.0], [8.0, 3.0, 8.0], [1.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UCRN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 1.0], [4.0, 7.0, 8.0], [8.0, 1.0, 7.0]], [[1.0, 3.0, 8.0], [2.0, 2.0, 7.0], [4.0, 2.0, 
1.0]], [[4.0, 4.0, 1.0], [5.0, 2.0, 7.0], [6.0, 1.0, 5.0]]], [[[8.0, 3.0, 4.0], [2.0, 4.0, 6.0], [2.0, 8.0, 4.0]], [[6.0, 7.0, 
5.0], [1.0, 7.0, 8.0], [1.0, 5.0, 1.0]], [[6.0, 8.0, 5.0], [8.0, 5.0, 8.0], [5.0, 8.0, 2.0]]], [[[8.0, 3.0, 3.0], [3.0, 5.0, 
3.0], [3.0, 4.0, 6.0]], [[6.0, 7.0, 7.0], [8.0, 8.0, 1.0], [2.0, 7.0, 8.0]], [[7.0, 3.0, 8.0], [2.0, 6.0, 7.0], [2.0, 6.0, 
3.0]]], [[[2.0, 7.0, 3.0], [3.0, 6.0, 2.0], [5.0, 1.0, 2.0]], [[7.0, 4.0, 5.0], [1.0, 5.0, 4.0], [8.0, 2.0, 6.0]], [[3.0, 2.0, 
8.0], [6.0, 4.0, 4.0], [4.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 4.0], [3.0, 7.0, 2.0], [6.0, 6.0, 6.0]], [[8.0, 1.0, 4.0], [4.0, 6.0, 
7.0], [1.0, 6.0, 6.0]], [[8.0, 5.0, 2.0], [8.0, 5.0, 7.0], [6.0, 3.0, 4.0]]], [[[1.0, 4.0, 4.0], [4.0, 2.0, 4.0], [1.0, 6.0, 
3.0]], [[2.0, 3.0, 3.0], [5.0, 5.0, 1.0], [8.0, 4.0, 8.0]], [[8.0, 8.0, 3.0], [6.0, 1.0, 1.0], [2.0, 6.0, 2.0]]], [[[3.0, 8.0, 
2.0], [8.0, 2.0, 7.0], [3.0, 8.0, 3.0]], [[1.0, 7.0, 1.0], [1.0, 2.0, 5.0], [5.0, 6.0, 6.0]], [[3.0, 2.0, 5.0], [4.0, 8.0, 
7.0], [2.0, 7.0, 4.0]]], [[[8.0, 7.0, 5.0], [8.0, 3.0, 2.0], [3.0, 4.0, 5.0]], [[8.0, 7.0, 5.0], [1.0, 2.0, 6.0], [3.0, 2.0, 
3.0]], [[5.0, 1.0, 1.0], [4.0, 3.0, 8.0], [2.0, 2.0, 8.0]]]]))
      val=numpy.array([[[[3.0, 8.0, 4.0]], [[1.0, 5.0, 2.0]], [[8.0, 1.0, 6.0]]], [[[6.0, 3.0, 6.0]], [[7.0, 8.0, 7.0]], [[5.0, 
5.0, 8.0]]], [[[7.0, 2.0, 3.0]], [[6.0, 6.0, 3.0]], [[7.0, 7.0, 4.0]]]])
      arg[:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 1.0], [3.0, 8.0, 4.0], [8.0, 1.0, 7.0]], [[1.0, 3.0, 8.0], [1.0, 5.0, 2.0], [4.0, 
2.0, 1.0]], [[4.0, 4.0, 1.0], [8.0, 1.0, 6.0], [6.0, 1.0, 5.0]]], [[[8.0, 3.0, 4.0], [6.0, 3.0, 6.0], [2.0, 8.0, 4.0]], [[6.0, 
7.0, 5.0], [7.0, 8.0, 7.0], [1.0, 5.0, 1.0]], [[6.0, 8.0, 5.0], [5.0, 5.0, 8.0], [5.0, 8.0, 2.0]]], [[[8.0, 3.0, 3.0], [7.0, 
2.0, 3.0], [3.0, 4.0, 6.0]], [[6.0, 7.0, 7.0], [6.0, 6.0, 3.0], [2.0, 7.0, 8.0]], [[7.0, 3.0, 8.0], [7.0, 7.0, 4.0], [2.0, 6.0, 
3.0]]], [[[2.0, 7.0, 3.0], [3.0, 6.0, 2.0], [5.0, 1.0, 2.0]], [[7.0, 4.0, 5.0], [1.0, 5.0, 4.0], [8.0, 2.0, 6.0]], [[3.0, 2.0, 
8.0], [6.0, 4.0, 4.0], [4.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 4.0], [3.0, 8.0, 4.0], [6.0, 6.0, 6.0]], 
[[8.0, 1.0, 4.0], [1.0, 5.0, 2.0], [1.0, 6.0, 6.0]], [[8.0, 5.0, 2.0], [8.0, 1.0, 6.0], [6.0, 3.0, 4.0]]], [[[1.0, 4.0, 4.0], 
[6.0, 3.0, 6.0], [1.0, 6.0, 3.0]], [[2.0, 3.0, 3.0], [7.0, 8.0, 7.0], [8.0, 4.0, 8.0]], [[8.0, 8.0, 3.0], [5.0, 5.0, 8.0], 
[2.0, 6.0, 2.0]]], [[[3.0, 8.0, 2.0], [7.0, 2.0, 3.0], [3.0, 8.0, 3.0]], [[1.0, 7.0, 1.0], [6.0, 6.0, 3.0], [5.0, 6.0, 6.0]], 
[[3.0, 2.0, 5.0], [7.0, 7.0, 4.0], [2.0, 7.0, 4.0]]], [[[8.0, 7.0, 5.0], [8.0, 3.0, 2.0], [3.0, 4.0, 5.0]], [[8.0, 7.0, 5.0], 
[1.0, 2.0, 6.0], [3.0, 2.0, 3.0]], [[5.0, 1.0, 1.0], [4.0, 3.0, 8.0], [2.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UCRN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 7.0], [6.0, 2.0, 8.0], [6.0, 7.0, 8.0]], [[4.0, 1.0, 5.0], [3.0, 8.0, 2.0], [8.0, 3.0, 
1.0]], [[6.0, 6.0, 2.0], [8.0, 7.0, 4.0], [4.0, 3.0, 4.0]]], [[[8.0, 8.0, 7.0], [7.0, 7.0, 6.0], [5.0, 1.0, 3.0]], [[1.0, 1.0, 
6.0], [5.0, 8.0, 5.0], [3.0, 2.0, 7.0]], [[4.0, 2.0, 7.0], [1.0, 3.0, 1.0], [7.0, 4.0, 1.0]]], [[[7.0, 7.0, 1.0], [2.0, 6.0, 
1.0], [3.0, 7.0, 8.0]], [[2.0, 4.0, 6.0], [3.0, 5.0, 8.0], [2.0, 6.0, 1.0]], [[8.0, 5.0, 1.0], [6.0, 1.0, 8.0], [2.0, 5.0, 
6.0]]], [[[4.0, 7.0, 5.0], [2.0, 3.0, 3.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 4.0], [4.0, 1.0, 1.0], [6.0, 5.0, 2.0]], [[7.0, 7.0, 
7.0], [5.0, 4.0, 5.0], [8.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 4.0], [7.0, 6.0, 2.0], [8.0, 3.0, 4.0]], [[1.0, 5.0, 2.0], [4.0, 1.0, 
4.0], [6.0, 4.0, 4.0]], [[1.0, 1.0, 2.0], [3.0, 4.0, 7.0], [3.0, 6.0, 6.0]]], [[[6.0, 1.0, 8.0], [8.0, 3.0, 2.0], [1.0, 6.0, 
1.0]], [[8.0, 8.0, 2.0], [1.0, 7.0, 5.0], [6.0, 1.0, 7.0]], [[7.0, 3.0, 3.0], [3.0, 4.0, 6.0], [6.0, 7.0, 1.0]]], [[[3.0, 1.0, 
8.0], [7.0, 6.0, 1.0], [4.0, 5.0, 1.0]], [[2.0, 2.0, 8.0], [7.0, 3.0, 5.0], [2.0, 4.0, 1.0]], [[3.0, 5.0, 2.0], [7.0, 5.0, 
7.0], [4.0, 7.0, 5.0]]], [[[6.0, 1.0, 2.0], [6.0, 7.0, 5.0], [2.0, 1.0, 6.0]], [[2.0, 4.0, 5.0], [4.0, 6.0, 5.0], [3.0, 5.0, 
1.0]], [[5.0, 8.0, 4.0], [7.0, 2.0, 7.0], [1.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[[1.0, 5.0, 3.0]], [[5.0, 7.0, 4.0]], [[6.0, 2.0, 1.0]]], [[[2.0, 3.0, 7.0]], [[1.0, 5.0, 5.0]], 
[[8.0, 2.0, 2.0]]], [[[2.0, 8.0, 8.0]], [[3.0, 4.0, 3.0]], [[6.0, 6.0, 7.0]]]]),self.functionspace)
      arg[:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 7.0], [1.0, 5.0, 3.0], [6.0, 7.0, 8.0]], [[4.0, 1.0, 5.0], [5.0, 7.0, 4.0], [8.0, 
3.0, 1.0]], [[6.0, 6.0, 2.0], [6.0, 2.0, 1.0], [4.0, 3.0, 4.0]]], [[[8.0, 8.0, 7.0], [2.0, 3.0, 7.0], [5.0, 1.0, 3.0]], [[1.0, 
1.0, 6.0], [1.0, 5.0, 5.0], [3.0, 2.0, 7.0]], [[4.0, 2.0, 7.0], [8.0, 2.0, 2.0], [7.0, 4.0, 1.0]]], [[[7.0, 7.0, 1.0], [2.0, 
8.0, 8.0], [3.0, 7.0, 8.0]], [[2.0, 4.0, 6.0], [3.0, 4.0, 3.0], [2.0, 6.0, 1.0]], [[8.0, 5.0, 1.0], [6.0, 6.0, 7.0], [2.0, 5.0, 
6.0]]], [[[4.0, 7.0, 5.0], [2.0, 3.0, 3.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 4.0], [4.0, 1.0, 1.0], [6.0, 5.0, 2.0]], [[7.0, 7.0, 
7.0], [5.0, 4.0, 5.0], [8.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 4.0], [1.0, 5.0, 3.0], [8.0, 3.0, 4.0]], 
[[1.0, 5.0, 2.0], [5.0, 7.0, 4.0], [6.0, 4.0, 4.0]], [[1.0, 1.0, 2.0], [6.0, 2.0, 1.0], [3.0, 6.0, 6.0]]], [[[6.0, 1.0, 8.0], 
[2.0, 3.0, 7.0], [1.0, 6.0, 1.0]], [[8.0, 8.0, 2.0], [1.0, 5.0, 5.0], [6.0, 1.0, 7.0]], [[7.0, 3.0, 3.0], [8.0, 2.0, 2.0], 
[6.0, 7.0, 1.0]]], [[[3.0, 1.0, 8.0], [2.0, 8.0, 8.0], [4.0, 5.0, 1.0]], [[2.0, 2.0, 8.0], [3.0, 4.0, 3.0], [2.0, 4.0, 1.0]], 
[[3.0, 5.0, 2.0], [6.0, 6.0, 7.0], [4.0, 7.0, 5.0]]], [[[6.0, 1.0, 2.0], [6.0, 7.0, 5.0], [2.0, 1.0, 6.0]], [[2.0, 4.0, 5.0], 
[4.0, 6.0, 5.0], [3.0, 5.0, 1.0]], [[5.0, 8.0, 4.0], [7.0, 2.0, 7.0], [1.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UCRN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 1.0], [1.0, 3.0, 7.0], [5.0, 6.0, 1.0]], [[8.0, 5.0, 4.0], [3.0, 6.0, 5.0], [1.0, 5.0, 
2.0]], [[2.0, 1.0, 2.0], [2.0, 7.0, 3.0], [8.0, 1.0, 1.0]]], [[[5.0, 4.0, 8.0], [8.0, 7.0, 8.0], [6.0, 3.0, 5.0]], [[8.0, 7.0, 
6.0], [1.0, 1.0, 2.0], [2.0, 8.0, 8.0]], [[3.0, 4.0, 8.0], [1.0, 5.0, 6.0], [5.0, 5.0, 8.0]]], [[[5.0, 3.0, 2.0], [7.0, 8.0, 
5.0], [3.0, 3.0, 4.0]], [[1.0, 3.0, 2.0], [2.0, 4.0, 6.0], [6.0, 8.0, 3.0]], [[7.0, 4.0, 7.0], [2.0, 7.0, 6.0], [5.0, 3.0, 
4.0]]], [[[3.0, 5.0, 4.0], [6.0, 1.0, 7.0], [2.0, 3.0, 6.0]], [[1.0, 8.0, 1.0], [7.0, 6.0, 6.0], [6.0, 7.0, 3.0]], [[2.0, 4.0, 
3.0], [7.0, 4.0, 4.0], [7.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 8.0], [6.0, 4.0, 8.0], [1.0, 5.0, 4.0]], [[5.0, 4.0, 3.0], [3.0, 3.0, 
6.0], [6.0, 3.0, 3.0]], [[3.0, 4.0, 4.0], [6.0, 6.0, 3.0], [1.0, 1.0, 6.0]]], [[[8.0, 4.0, 5.0], [2.0, 4.0, 4.0], [7.0, 6.0, 
6.0]], [[4.0, 8.0, 6.0], [5.0, 3.0, 4.0], [6.0, 5.0, 2.0]], [[5.0, 8.0, 8.0], [8.0, 5.0, 4.0], [1.0, 3.0, 4.0]]], [[[1.0, 7.0, 
7.0], [4.0, 2.0, 3.0], [4.0, 5.0, 4.0]], [[6.0, 5.0, 5.0], [7.0, 8.0, 2.0], [8.0, 7.0, 4.0]], [[8.0, 7.0, 8.0], [2.0, 6.0, 
2.0], [4.0, 3.0, 4.0]]], [[[5.0, 4.0, 3.0], [6.0, 8.0, 4.0], [8.0, 6.0, 8.0]], [[1.0, 5.0, 3.0], [6.0, 5.0, 5.0], [4.0, 2.0, 
3.0]], [[3.0, 7.0, 2.0], [5.0, 8.0, 5.0], [2.0, 2.0, 5.0]]]]))
      val=Data(numpy.array([[[[7.0, 5.0, 3.0]], [[6.0, 6.0, 7.0]], [[1.0, 1.0, 1.0]]], [[[1.0, 5.0, 1.0]], [[7.0, 8.0, 2.0]], 
[[7.0, 5.0, 3.0]]], [[[4.0, 1.0, 3.0]], [[4.0, 3.0, 2.0]], [[6.0, 7.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 7.0]], [[4.0, 6.0, 6.0]], [[4.0, 7.0, 7.0]]], [[[1.0, 5.0, 3.0]], [[1.0, 
5.0, 8.0]], [[4.0, 1.0, 4.0]]], [[[2.0, 4.0, 4.0]], [[3.0, 6.0, 8.0]], [[5.0, 2.0, 6.0]]]]))
      arg[:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 1.0], [7.0, 5.0, 3.0], [5.0, 6.0, 1.0]], [[8.0, 5.0, 4.0], [6.0, 6.0, 7.0], [1.0, 
5.0, 2.0]], [[2.0, 1.0, 2.0], [1.0, 1.0, 1.0], [8.0, 1.0, 1.0]]], [[[5.0, 4.0, 8.0], [1.0, 5.0, 1.0], [6.0, 3.0, 5.0]], [[8.0, 
7.0, 6.0], [7.0, 8.0, 2.0], [2.0, 8.0, 8.0]], [[3.0, 4.0, 8.0], [7.0, 5.0, 3.0], [5.0, 5.0, 8.0]]], [[[5.0, 3.0, 2.0], [4.0, 
1.0, 3.0], [3.0, 3.0, 4.0]], [[1.0, 3.0, 2.0], [4.0, 3.0, 2.0], [6.0, 8.0, 3.0]], [[7.0, 4.0, 7.0], [6.0, 7.0, 8.0], [5.0, 3.0, 
4.0]]], [[[3.0, 5.0, 4.0], [6.0, 1.0, 7.0], [2.0, 3.0, 6.0]], [[1.0, 8.0, 1.0], [7.0, 6.0, 6.0], [6.0, 7.0, 3.0]], [[2.0, 4.0, 
3.0], [7.0, 4.0, 4.0], [7.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 8.0], [6.0, 1.0, 7.0], [1.0, 5.0, 4.0]], 
[[5.0, 4.0, 3.0], [4.0, 6.0, 6.0], [6.0, 3.0, 3.0]], [[3.0, 4.0, 4.0], [4.0, 7.0, 7.0], [1.0, 1.0, 6.0]]], [[[8.0, 4.0, 5.0], 
[1.0, 5.0, 3.0], [7.0, 6.0, 6.0]], [[4.0, 8.0, 6.0], [1.0, 5.0, 8.0], [6.0, 5.0, 2.0]], [[5.0, 8.0, 8.0], [4.0, 1.0, 4.0], 
[1.0, 3.0, 4.0]]], [[[1.0, 7.0, 7.0], [2.0, 4.0, 4.0], [4.0, 5.0, 4.0]], [[6.0, 5.0, 5.0], [3.0, 6.0, 8.0], [8.0, 7.0, 4.0]], 
[[8.0, 7.0, 8.0], [5.0, 2.0, 6.0], [4.0, 3.0, 4.0]]], [[[5.0, 4.0, 3.0], [6.0, 8.0, 4.0], [8.0, 6.0, 8.0]], [[1.0, 5.0, 3.0], 
[6.0, 5.0, 5.0], [4.0, 2.0, 3.0]], [[3.0, 7.0, 2.0], [5.0, 8.0, 5.0], [2.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UCRN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 5.0], [7.0, 4.0, 4.0], [7.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [2.0, 6.0, 6.0], [2.0, 6.0, 
6.0]], [[6.0, 4.0, 8.0], [7.0, 1.0, 6.0], [2.0, 7.0, 6.0]]], [[[7.0, 1.0, 3.0], [3.0, 6.0, 8.0], [6.0, 8.0, 4.0]], [[1.0, 1.0, 
2.0], [6.0, 6.0, 1.0], [6.0, 6.0, 5.0]], [[4.0, 5.0, 3.0], [5.0, 1.0, 1.0], [5.0, 4.0, 2.0]]], [[[7.0, 3.0, 7.0], [3.0, 3.0, 
2.0], [2.0, 6.0, 6.0]], [[8.0, 7.0, 4.0], [3.0, 2.0, 1.0], [3.0, 5.0, 2.0]], [[5.0, 7.0, 3.0], [1.0, 4.0, 8.0], [4.0, 8.0, 
7.0]]], [[[7.0, 5.0, 4.0], [2.0, 5.0, 7.0], [6.0, 4.0, 3.0]], [[8.0, 2.0, 3.0], [8.0, 6.0, 7.0], [8.0, 7.0, 3.0]], [[4.0, 6.0, 
3.0], [3.0, 2.0, 5.0], [1.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 5.0], [8.0, 1.0, 6.0], [4.0, 5.0, 4.0]], [[7.0, 6.0, 2.0], [7.0, 2.0, 
6.0], [3.0, 5.0, 3.0]], [[4.0, 5.0, 1.0], [4.0, 1.0, 3.0], [7.0, 8.0, 7.0]]], [[[8.0, 1.0, 1.0], [4.0, 3.0, 4.0], [6.0, 7.0, 
8.0]], [[7.0, 2.0, 5.0], [1.0, 2.0, 1.0], [1.0, 3.0, 8.0]], [[6.0, 3.0, 2.0], [1.0, 3.0, 5.0], [4.0, 3.0, 2.0]]], [[[1.0, 3.0, 
5.0], [1.0, 7.0, 4.0], [2.0, 4.0, 6.0]], [[5.0, 3.0, 5.0], [5.0, 1.0, 2.0], [7.0, 4.0, 8.0]], [[4.0, 1.0, 8.0], [5.0, 2.0, 
1.0], [7.0, 1.0, 4.0]]], [[[1.0, 5.0, 7.0], [2.0, 5.0, 8.0], [8.0, 6.0, 2.0]], [[3.0, 1.0, 2.0], [5.0, 5.0, 1.0], [2.0, 5.0, 
5.0]], [[4.0, 7.0, 8.0], [5.0, 1.0, 5.0], [5.0, 7.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 3.0, 3.0]], [[3.0, 2.0, 1.0]], [[2.0, 8.0, 2.0]]], [[[2.0, 7.0, 5.0]], [[7.0, 3.0, 
7.0]], [[6.0, 5.0, 5.0]]], [[[1.0, 1.0, 8.0]], [[7.0, 1.0, 5.0]], [[5.0, 3.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 2.0, 
6.0]], [[3.0, 4.0, 3.0]], [[8.0, 4.0, 1.0]]], [[[4.0, 2.0, 7.0]], [[5.0, 2.0, 8.0]], [[5.0, 5.0, 5.0]]], [[[1.0, 7.0, 6.0]], 
[[5.0, 3.0, 2.0]], [[5.0, 1.0, 1.0]]]])
      arg[:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 5.0], [1.0, 3.0, 3.0], [7.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [3.0, 2.0, 1.0], [2.0, 
6.0, 6.0]], [[6.0, 4.0, 8.0], [2.0, 8.0, 2.0], [2.0, 7.0, 6.0]]], [[[7.0, 1.0, 3.0], [2.0, 7.0, 5.0], [6.0, 8.0, 4.0]], [[1.0, 
1.0, 2.0], [7.0, 3.0, 7.0], [6.0, 6.0, 5.0]], [[4.0, 5.0, 3.0], [6.0, 5.0, 5.0], [5.0, 4.0, 2.0]]], [[[7.0, 3.0, 7.0], [1.0, 
1.0, 8.0], [2.0, 6.0, 6.0]], [[8.0, 7.0, 4.0], [7.0, 1.0, 5.0], [3.0, 5.0, 2.0]], [[5.0, 7.0, 3.0], [5.0, 3.0, 8.0], [4.0, 8.0, 
7.0]]], [[[7.0, 5.0, 4.0], [2.0, 5.0, 7.0], [6.0, 4.0, 3.0]], [[8.0, 2.0, 3.0], [8.0, 6.0, 7.0], [8.0, 7.0, 3.0]], [[4.0, 6.0, 
3.0], [3.0, 2.0, 5.0], [1.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 5.0], [1.0, 2.0, 6.0], [4.0, 5.0, 4.0]], 
[[7.0, 6.0, 2.0], [3.0, 4.0, 3.0], [3.0, 5.0, 3.0]], [[4.0, 5.0, 1.0], [8.0, 4.0, 1.0], [7.0, 8.0, 7.0]]], [[[8.0, 1.0, 1.0], 
[4.0, 2.0, 7.0], [6.0, 7.0, 8.0]], [[7.0, 2.0, 5.0], [5.0, 2.0, 8.0], [1.0, 3.0, 8.0]], [[6.0, 3.0, 2.0], [5.0, 5.0, 5.0], 
[4.0, 3.0, 2.0]]], [[[1.0, 3.0, 5.0], [1.0, 7.0, 6.0], [2.0, 4.0, 6.0]], [[5.0, 3.0, 5.0], [5.0, 3.0, 2.0], [7.0, 4.0, 8.0]], 
[[4.0, 1.0, 8.0], [5.0, 1.0, 1.0], [7.0, 1.0, 4.0]]], [[[1.0, 5.0, 7.0], [2.0, 5.0, 8.0], [8.0, 6.0, 2.0]], [[3.0, 1.0, 2.0], 
[5.0, 5.0, 1.0], [2.0, 5.0, 5.0]], [[4.0, 7.0, 8.0], [5.0, 1.0, 5.0], [5.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRPN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 8.0], [2.0, 4.0, 1.0], [2.0, 3.0, 2.0]], [[5.0, 4.0, 8.0], [7.0, 2.0, 6.0], [5.0, 4.0, 
6.0]], [[8.0, 8.0, 3.0], [8.0, 4.0, 1.0], [5.0, 1.0, 1.0]]], [[[7.0, 3.0, 6.0], [7.0, 2.0, 6.0], [6.0, 8.0, 3.0]], [[3.0, 7.0, 
6.0], [5.0, 7.0, 1.0], [4.0, 3.0, 8.0]], [[5.0, 5.0, 1.0], [7.0, 8.0, 2.0], [4.0, 3.0, 3.0]]], [[[8.0, 7.0, 6.0], [1.0, 2.0, 
3.0], [4.0, 6.0, 3.0]], [[5.0, 6.0, 2.0], [6.0, 7.0, 1.0], [5.0, 2.0, 3.0]], [[2.0, 6.0, 3.0], [8.0, 1.0, 5.0], [2.0, 2.0, 
4.0]]], [[[7.0, 3.0, 5.0], [7.0, 3.0, 5.0], [2.0, 8.0, 3.0]], [[3.0, 4.0, 8.0], [7.0, 6.0, 1.0], [2.0, 4.0, 4.0]], [[3.0, 6.0, 
3.0], [2.0, 1.0, 4.0], [1.0, 7.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 7.0], [8.0, 3.0, 7.0], [7.0, 8.0, 5.0]], [[8.0, 6.0, 6.0], [4.0, 7.0, 
8.0], [6.0, 2.0, 8.0]], [[8.0, 6.0, 5.0], [8.0, 3.0, 2.0], [6.0, 8.0, 6.0]]], [[[7.0, 8.0, 4.0], [4.0, 5.0, 4.0], [8.0, 2.0, 
7.0]], [[2.0, 2.0, 4.0], [8.0, 3.0, 8.0], [3.0, 4.0, 4.0]], [[6.0, 2.0, 7.0], [2.0, 5.0, 3.0], [1.0, 3.0, 7.0]]], [[[8.0, 7.0, 
2.0], [6.0, 6.0, 8.0], [6.0, 6.0, 2.0]], [[2.0, 1.0, 6.0], [3.0, 7.0, 4.0], [5.0, 2.0, 8.0]], [[5.0, 3.0, 2.0], [7.0, 5.0, 
2.0], [6.0, 4.0, 1.0]]], [[[2.0, 3.0, 8.0], [4.0, 2.0, 7.0], [7.0, 3.0, 7.0]], [[1.0, 1.0, 1.0], [1.0, 5.0, 6.0], [8.0, 7.0, 
3.0]], [[4.0, 5.0, 4.0], [8.0, 8.0, 4.0], [1.0, 5.0, 4.0]]]]))
      val=numpy.array([[[2.0, 3.0, 6.0]], [[8.0, 8.0, 2.0]]])
      arg[1:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 8.0], [2.0, 4.0, 1.0], [2.0, 3.0, 2.0]], [[5.0, 4.0, 8.0], [7.0, 2.0, 6.0], [5.0, 
4.0, 6.0]], [[8.0, 8.0, 3.0], [8.0, 4.0, 1.0], [5.0, 1.0, 1.0]]], [[[7.0, 3.0, 6.0], [7.0, 2.0, 6.0], [6.0, 8.0, 3.0]], [[3.0, 
7.0, 6.0], [2.0, 3.0, 6.0], [4.0, 3.0, 8.0]], [[5.0, 5.0, 1.0], [7.0, 8.0, 2.0], [4.0, 3.0, 3.0]]], [[[8.0, 7.0, 6.0], [1.0, 
2.0, 3.0], [4.0, 6.0, 3.0]], [[5.0, 6.0, 2.0], [8.0, 8.0, 2.0], [5.0, 2.0, 3.0]], [[2.0, 6.0, 3.0], [8.0, 1.0, 5.0], [2.0, 2.0, 
4.0]]], [[[7.0, 3.0, 5.0], [7.0, 3.0, 5.0], [2.0, 8.0, 3.0]], [[3.0, 4.0, 8.0], [7.0, 6.0, 1.0], [2.0, 4.0, 4.0]], [[3.0, 6.0, 
3.0], [2.0, 1.0, 4.0], [1.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 7.0], [8.0, 3.0, 7.0], [7.0, 8.0, 5.0]], 
[[8.0, 6.0, 6.0], [4.0, 7.0, 8.0], [6.0, 2.0, 8.0]], [[8.0, 6.0, 5.0], [8.0, 3.0, 2.0], [6.0, 8.0, 6.0]]], [[[7.0, 8.0, 4.0], 
[4.0, 5.0, 4.0], [8.0, 2.0, 7.0]], [[2.0, 2.0, 4.0], [2.0, 3.0, 6.0], [3.0, 4.0, 4.0]], [[6.0, 2.0, 7.0], [2.0, 5.0, 3.0], 
[1.0, 3.0, 7.0]]], [[[8.0, 7.0, 2.0], [6.0, 6.0, 8.0], [6.0, 6.0, 2.0]], [[2.0, 1.0, 6.0], [8.0, 8.0, 2.0], [5.0, 2.0, 8.0]], 
[[5.0, 3.0, 2.0], [7.0, 5.0, 2.0], [6.0, 4.0, 1.0]]], [[[2.0, 3.0, 8.0], [4.0, 2.0, 7.0], [7.0, 3.0, 7.0]], [[1.0, 1.0, 1.0], 
[1.0, 5.0, 6.0], [8.0, 7.0, 3.0]], [[4.0, 5.0, 4.0], [8.0, 8.0, 4.0], [1.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRPN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 4.0], [5.0, 5.0, 6.0], [2.0, 4.0, 3.0]], [[7.0, 4.0, 7.0], [5.0, 1.0, 7.0], [1.0, 8.0, 
5.0]], [[8.0, 3.0, 3.0], [5.0, 4.0, 4.0], [3.0, 1.0, 7.0]]], [[[2.0, 6.0, 6.0], [5.0, 4.0, 6.0], [5.0, 6.0, 6.0]], [[3.0, 8.0, 
4.0], [8.0, 2.0, 7.0], [5.0, 7.0, 1.0]], [[2.0, 7.0, 2.0], [3.0, 2.0, 6.0], [5.0, 5.0, 5.0]]], [[[3.0, 8.0, 7.0], [4.0, 4.0, 
4.0], [1.0, 6.0, 6.0]], [[7.0, 3.0, 8.0], [8.0, 1.0, 3.0], [1.0, 1.0, 1.0]], [[3.0, 5.0, 5.0], [3.0, 5.0, 5.0], [5.0, 8.0, 
2.0]]], [[[6.0, 6.0, 1.0], [4.0, 5.0, 2.0], [4.0, 2.0, 3.0]], [[5.0, 1.0, 8.0], [4.0, 1.0, 5.0], [7.0, 5.0, 3.0]], [[6.0, 3.0, 
1.0], [6.0, 7.0, 3.0], [3.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 8.0], [2.0, 2.0, 7.0], [3.0, 6.0, 4.0]], [[1.0, 2.0, 6.0], [7.0, 4.0, 
4.0], [4.0, 7.0, 3.0]], [[8.0, 4.0, 4.0], [1.0, 3.0, 7.0], [3.0, 8.0, 8.0]]], [[[6.0, 1.0, 5.0], [8.0, 7.0, 3.0], [8.0, 8.0, 
6.0]], [[4.0, 7.0, 7.0], [7.0, 1.0, 3.0], [3.0, 5.0, 5.0]], [[1.0, 6.0, 4.0], [4.0, 1.0, 6.0], [5.0, 8.0, 3.0]]], [[[6.0, 1.0, 
6.0], [5.0, 4.0, 7.0], [7.0, 2.0, 4.0]], [[6.0, 8.0, 4.0], [8.0, 7.0, 5.0], [6.0, 1.0, 2.0]], [[5.0, 2.0, 5.0], [2.0, 3.0, 
6.0], [5.0, 4.0, 1.0]]], [[[2.0, 2.0, 8.0], [3.0, 1.0, 2.0], [6.0, 8.0, 7.0]], [[8.0, 6.0, 3.0], [4.0, 5.0, 8.0], [5.0, 4.0, 
2.0]], [[8.0, 6.0, 7.0], [2.0, 2.0, 6.0], [3.0, 8.0, 1.0]]]]))
      val=Data(numpy.array([[[2.0, 6.0, 7.0]], [[5.0, 8.0, 1.0]]]),self.functionspace)
      arg[1:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 4.0], [5.0, 5.0, 6.0], [2.0, 4.0, 3.0]], [[7.0, 4.0, 7.0], [5.0, 1.0, 7.0], [1.0, 
8.0, 5.0]], [[8.0, 3.0, 3.0], [5.0, 4.0, 4.0], [3.0, 1.0, 7.0]]], [[[2.0, 6.0, 6.0], [5.0, 4.0, 6.0], [5.0, 6.0, 6.0]], [[3.0, 
8.0, 4.0], [2.0, 6.0, 7.0], [5.0, 7.0, 1.0]], [[2.0, 7.0, 2.0], [3.0, 2.0, 6.0], [5.0, 5.0, 5.0]]], [[[3.0, 8.0, 7.0], [4.0, 
4.0, 4.0], [1.0, 6.0, 6.0]], [[7.0, 3.0, 8.0], [5.0, 8.0, 1.0], [1.0, 1.0, 1.0]], [[3.0, 5.0, 5.0], [3.0, 5.0, 5.0], [5.0, 8.0, 
2.0]]], [[[6.0, 6.0, 1.0], [4.0, 5.0, 2.0], [4.0, 2.0, 3.0]], [[5.0, 1.0, 8.0], [4.0, 1.0, 5.0], [7.0, 5.0, 3.0]], [[6.0, 3.0, 
1.0], [6.0, 7.0, 3.0], [3.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 8.0], [2.0, 2.0, 7.0], [3.0, 6.0, 4.0]], 
[[1.0, 2.0, 6.0], [7.0, 4.0, 4.0], [4.0, 7.0, 3.0]], [[8.0, 4.0, 4.0], [1.0, 3.0, 7.0], [3.0, 8.0, 8.0]]], [[[6.0, 1.0, 5.0], 
[8.0, 7.0, 3.0], [8.0, 8.0, 6.0]], [[4.0, 7.0, 7.0], [2.0, 6.0, 7.0], [3.0, 5.0, 5.0]], [[1.0, 6.0, 4.0], [4.0, 1.0, 6.0], 
[5.0, 8.0, 3.0]]], [[[6.0, 1.0, 6.0], [5.0, 4.0, 7.0], [7.0, 2.0, 4.0]], [[6.0, 8.0, 4.0], [5.0, 8.0, 1.0], [6.0, 1.0, 2.0]], 
[[5.0, 2.0, 5.0], [2.0, 3.0, 6.0], [5.0, 4.0, 1.0]]], [[[2.0, 2.0, 8.0], [3.0, 1.0, 2.0], [6.0, 8.0, 7.0]], [[8.0, 6.0, 3.0], 
[4.0, 5.0, 8.0], [5.0, 4.0, 2.0]], [[8.0, 6.0, 7.0], [2.0, 2.0, 6.0], [3.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRPN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 4.0], [3.0, 2.0, 4.0], [4.0, 7.0, 6.0]], [[3.0, 5.0, 4.0], [7.0, 4.0, 2.0], [7.0, 5.0, 
7.0]], [[5.0, 8.0, 6.0], [1.0, 4.0, 3.0], [3.0, 6.0, 5.0]]], [[[3.0, 4.0, 2.0], [7.0, 4.0, 5.0], [7.0, 2.0, 7.0]], [[2.0, 3.0, 
7.0], [7.0, 2.0, 8.0], [2.0, 7.0, 5.0]], [[3.0, 5.0, 2.0], [1.0, 1.0, 3.0], [1.0, 8.0, 5.0]]], [[[3.0, 4.0, 1.0], [2.0, 8.0, 
8.0], [5.0, 8.0, 8.0]], [[4.0, 2.0, 2.0], [5.0, 4.0, 1.0], [2.0, 1.0, 1.0]], [[4.0, 7.0, 1.0], [4.0, 3.0, 4.0], [4.0, 8.0, 
8.0]]], [[[8.0, 3.0, 3.0], [7.0, 5.0, 4.0], [2.0, 7.0, 3.0]], [[8.0, 3.0, 7.0], [2.0, 5.0, 8.0], [8.0, 1.0, 8.0]], [[1.0, 6.0, 
3.0], [1.0, 8.0, 8.0], [6.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 2.0], [7.0, 1.0, 5.0], [6.0, 2.0, 7.0]], [[6.0, 2.0, 6.0], [1.0, 4.0, 
6.0], [5.0, 2.0, 8.0]], [[7.0, 5.0, 8.0], [4.0, 5.0, 8.0], [5.0, 2.0, 7.0]]], [[[7.0, 8.0, 4.0], [3.0, 5.0, 6.0], [1.0, 3.0, 
3.0]], [[3.0, 2.0, 8.0], [3.0, 7.0, 6.0], [3.0, 1.0, 8.0]], [[4.0, 4.0, 8.0], [3.0, 5.0, 8.0], [6.0, 7.0, 2.0]]], [[[3.0, 8.0, 
5.0], [3.0, 7.0, 4.0], [4.0, 3.0, 6.0]], [[3.0, 1.0, 7.0], [6.0, 5.0, 7.0], [1.0, 8.0, 4.0]], [[7.0, 4.0, 8.0], [2.0, 2.0, 
6.0], [6.0, 1.0, 4.0]]], [[[2.0, 5.0, 7.0], [6.0, 3.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 7.0, 5.0], [8.0, 7.0, 3.0], [3.0, 1.0, 
2.0]], [[2.0, 7.0, 6.0], [4.0, 5.0, 5.0], [7.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[[8.0, 2.0, 1.0]], [[1.0, 6.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 1.0, 2.0]], [[1.0, 2.0, 7.0]]]))
      arg[1:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 4.0], [3.0, 2.0, 4.0], [4.0, 7.0, 6.0]], [[3.0, 5.0, 4.0], [7.0, 4.0, 2.0], [7.0, 
5.0, 7.0]], [[5.0, 8.0, 6.0], [1.0, 4.0, 3.0], [3.0, 6.0, 5.0]]], [[[3.0, 4.0, 2.0], [7.0, 4.0, 5.0], [7.0, 2.0, 7.0]], [[2.0, 
3.0, 7.0], [8.0, 2.0, 1.0], [2.0, 7.0, 5.0]], [[3.0, 5.0, 2.0], [1.0, 1.0, 3.0], [1.0, 8.0, 5.0]]], [[[3.0, 4.0, 1.0], [2.0, 
8.0, 8.0], [5.0, 8.0, 8.0]], [[4.0, 2.0, 2.0], [1.0, 6.0, 5.0], [2.0, 1.0, 1.0]], [[4.0, 7.0, 1.0], [4.0, 3.0, 4.0], [4.0, 8.0, 
8.0]]], [[[8.0, 3.0, 3.0], [7.0, 5.0, 4.0], [2.0, 7.0, 3.0]], [[8.0, 3.0, 7.0], [2.0, 5.0, 8.0], [8.0, 1.0, 8.0]], [[1.0, 6.0, 
3.0], [1.0, 8.0, 8.0], [6.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 2.0], [7.0, 1.0, 5.0], [6.0, 2.0, 7.0]], 
[[6.0, 2.0, 6.0], [1.0, 4.0, 6.0], [5.0, 2.0, 8.0]], [[7.0, 5.0, 8.0], [4.0, 5.0, 8.0], [5.0, 2.0, 7.0]]], [[[7.0, 8.0, 4.0], 
[3.0, 5.0, 6.0], [1.0, 3.0, 3.0]], [[3.0, 2.0, 8.0], [8.0, 1.0, 2.0], [3.0, 1.0, 8.0]], [[4.0, 4.0, 8.0], [3.0, 5.0, 8.0], 
[6.0, 7.0, 2.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 4.0], [4.0, 3.0, 6.0]], [[3.0, 1.0, 7.0], [1.0, 2.0, 7.0], [1.0, 8.0, 4.0]], 
[[7.0, 4.0, 8.0], [2.0, 2.0, 6.0], [6.0, 1.0, 4.0]]], [[[2.0, 5.0, 7.0], [6.0, 3.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 7.0, 5.0], 
[8.0, 7.0, 3.0], [3.0, 1.0, 2.0]], [[2.0, 7.0, 6.0], [4.0, 5.0, 5.0], [7.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRPN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 7.0], [3.0, 2.0, 2.0], [3.0, 1.0, 3.0]], [[7.0, 5.0, 7.0], [1.0, 3.0, 4.0], [1.0, 5.0, 
7.0]], [[4.0, 3.0, 7.0], [5.0, 8.0, 5.0], [1.0, 6.0, 8.0]]], [[[2.0, 8.0, 1.0], [7.0, 2.0, 1.0], [4.0, 3.0, 8.0]], [[3.0, 6.0, 
2.0], [8.0, 7.0, 1.0], [1.0, 8.0, 1.0]], [[3.0, 8.0, 6.0], [2.0, 2.0, 2.0], [4.0, 3.0, 2.0]]], [[[4.0, 6.0, 4.0], [3.0, 8.0, 
7.0], [7.0, 5.0, 4.0]], [[4.0, 4.0, 4.0], [5.0, 8.0, 6.0], [1.0, 6.0, 7.0]], [[6.0, 2.0, 8.0], [5.0, 5.0, 4.0], [1.0, 2.0, 
3.0]]], [[[2.0, 4.0, 8.0], [7.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[8.0, 8.0, 5.0], [5.0, 8.0, 1.0], [5.0, 1.0, 2.0]], [[7.0, 8.0, 
4.0], [1.0, 8.0, 4.0], [7.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 2.0], [1.0, 3.0, 1.0], [4.0, 2.0, 6.0]], [[8.0, 8.0, 8.0], [2.0, 1.0, 
8.0], [4.0, 4.0, 6.0]], [[4.0, 7.0, 6.0], [1.0, 4.0, 4.0], [8.0, 5.0, 4.0]]], [[[1.0, 6.0, 5.0], [2.0, 7.0, 3.0], [1.0, 6.0, 
7.0]], [[1.0, 8.0, 7.0], [2.0, 6.0, 5.0], [7.0, 3.0, 7.0]], [[2.0, 5.0, 2.0], [1.0, 6.0, 3.0], [4.0, 4.0, 3.0]]], [[[3.0, 6.0, 
4.0], [4.0, 3.0, 3.0], [5.0, 5.0, 4.0]], [[7.0, 6.0, 5.0], [5.0, 1.0, 2.0], [5.0, 8.0, 7.0]], [[4.0, 6.0, 7.0], [4.0, 7.0, 
7.0], [1.0, 7.0, 1.0]]], [[[2.0, 7.0, 8.0], [6.0, 8.0, 2.0], [8.0, 4.0, 8.0]], [[7.0, 4.0, 5.0], [1.0, 2.0, 6.0], [4.0, 2.0, 
1.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 8.0], [3.0, 1.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 2.0, 2.0]], [[8.0, 6.0, 7.0]]])+(1.-msk_val)*numpy.array([[[2.0, 4.0, 5.0]], [[3.0, 7.0, 
5.0]]])
      arg[1:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 7.0], [3.0, 2.0, 2.0], [3.0, 1.0, 3.0]], [[7.0, 5.0, 7.0], [1.0, 3.0, 4.0], [1.0, 
5.0, 7.0]], [[4.0, 3.0, 7.0], [5.0, 8.0, 5.0], [1.0, 6.0, 8.0]]], [[[2.0, 8.0, 1.0], [7.0, 2.0, 1.0], [4.0, 3.0, 8.0]], [[3.0, 
6.0, 2.0], [8.0, 2.0, 2.0], [1.0, 8.0, 1.0]], [[3.0, 8.0, 6.0], [2.0, 2.0, 2.0], [4.0, 3.0, 2.0]]], [[[4.0, 6.0, 4.0], [3.0, 
8.0, 7.0], [7.0, 5.0, 4.0]], [[4.0, 4.0, 4.0], [8.0, 6.0, 7.0], [1.0, 6.0, 7.0]], [[6.0, 2.0, 8.0], [5.0, 5.0, 4.0], [1.0, 2.0, 
3.0]]], [[[2.0, 4.0, 8.0], [7.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[8.0, 8.0, 5.0], [5.0, 8.0, 1.0], [5.0, 1.0, 2.0]], [[7.0, 8.0, 
4.0], [1.0, 8.0, 4.0], [7.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 2.0], [1.0, 3.0, 1.0], [4.0, 2.0, 6.0]], 
[[8.0, 8.0, 8.0], [2.0, 1.0, 8.0], [4.0, 4.0, 6.0]], [[4.0, 7.0, 6.0], [1.0, 4.0, 4.0], [8.0, 5.0, 4.0]]], [[[1.0, 6.0, 5.0], 
[2.0, 7.0, 3.0], [1.0, 6.0, 7.0]], [[1.0, 8.0, 7.0], [2.0, 4.0, 5.0], [7.0, 3.0, 7.0]], [[2.0, 5.0, 2.0], [1.0, 6.0, 3.0], 
[4.0, 4.0, 3.0]]], [[[3.0, 6.0, 4.0], [4.0, 3.0, 3.0], [5.0, 5.0, 4.0]], [[7.0, 6.0, 5.0], [3.0, 7.0, 5.0], [5.0, 8.0, 7.0]], 
[[4.0, 6.0, 7.0], [4.0, 7.0, 7.0], [1.0, 7.0, 1.0]]], [[[2.0, 7.0, 8.0], [6.0, 8.0, 2.0], [8.0, 4.0, 8.0]], [[7.0, 4.0, 5.0], 
[1.0, 2.0, 6.0], [4.0, 2.0, 1.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 8.0], [3.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 5.0], [3.0, 6.0, 1.0], [4.0, 7.0, 1.0]], [[8.0, 1.0, 4.0], [5.0, 2.0, 5.0], [4.0, 3.0, 
1.0]], [[1.0, 3.0, 3.0], [6.0, 7.0, 3.0], [3.0, 2.0, 2.0]]], [[[2.0, 3.0, 2.0], [5.0, 2.0, 7.0], [1.0, 1.0, 1.0]], [[2.0, 2.0, 
8.0], [2.0, 3.0, 5.0], [8.0, 2.0, 4.0]], [[1.0, 4.0, 6.0], [1.0, 6.0, 7.0], [3.0, 6.0, 2.0]]], [[[3.0, 6.0, 1.0], [3.0, 2.0, 
7.0], [2.0, 5.0, 2.0]], [[7.0, 4.0, 7.0], [3.0, 2.0, 5.0], [5.0, 7.0, 7.0]], [[8.0, 8.0, 5.0], [6.0, 4.0, 3.0], [5.0, 1.0, 
4.0]]], [[[3.0, 5.0, 5.0], [6.0, 5.0, 7.0], [2.0, 5.0, 3.0]], [[4.0, 8.0, 4.0], [2.0, 7.0, 6.0], [8.0, 1.0, 8.0]], [[8.0, 5.0, 
8.0], [4.0, 4.0, 1.0], [1.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 7.0], [5.0, 2.0, 1.0], [2.0, 7.0, 2.0]], [[5.0, 7.0, 7.0], [8.0, 8.0, 
3.0], [2.0, 1.0, 8.0]], [[8.0, 2.0, 4.0], [8.0, 2.0, 7.0], [7.0, 3.0, 4.0]]], [[[4.0, 8.0, 5.0], [7.0, 4.0, 6.0], [3.0, 3.0, 
1.0]], [[1.0, 1.0, 5.0], [5.0, 2.0, 2.0], [3.0, 2.0, 4.0]], [[7.0, 2.0, 6.0], [4.0, 5.0, 8.0], [3.0, 5.0, 7.0]]], [[[7.0, 8.0, 
1.0], [2.0, 5.0, 3.0], [4.0, 5.0, 3.0]], [[5.0, 3.0, 3.0], [8.0, 6.0, 2.0], [6.0, 2.0, 4.0]], [[1.0, 5.0, 2.0], [3.0, 3.0, 
4.0], [3.0, 4.0, 8.0]]], [[[6.0, 4.0, 1.0], [1.0, 1.0, 7.0], [8.0, 4.0, 3.0]], [[3.0, 1.0, 4.0], [8.0, 3.0, 5.0], [3.0, 1.0, 
7.0]], [[3.0, 3.0, 6.0], [8.0, 4.0, 8.0], [5.0, 3.0, 1.0]]]]))
      val=numpy.array([[[1.0, 6.0, 7.0], [3.0, 4.0, 1.0], [3.0, 8.0, 1.0]], [[1.0, 8.0, 7.0], [1.0, 4.0, 4.0], [8.0, 2.0, 
4.0]], [[1.0, 3.0, 4.0], [2.0, 4.0, 4.0], [3.0, 5.0, 4.0]], [[4.0, 7.0, 2.0], [5.0, 5.0, 8.0], [8.0, 7.0, 8.0]]])
      arg[:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 5.0], [3.0, 6.0, 1.0], [4.0, 7.0, 1.0]], [[1.0, 6.0, 7.0], [3.0, 4.0, 1.0], [3.0, 
8.0, 1.0]], [[1.0, 3.0, 3.0], [6.0, 7.0, 3.0], [3.0, 2.0, 2.0]]], [[[2.0, 3.0, 2.0], [5.0, 2.0, 7.0], [1.0, 1.0, 1.0]], [[1.0, 
8.0, 7.0], [1.0, 4.0, 4.0], [8.0, 2.0, 4.0]], [[1.0, 4.0, 6.0], [1.0, 6.0, 7.0], [3.0, 6.0, 2.0]]], [[[3.0, 6.0, 1.0], [3.0, 
2.0, 7.0], [2.0, 5.0, 2.0]], [[1.0, 3.0, 4.0], [2.0, 4.0, 4.0], [3.0, 5.0, 4.0]], [[8.0, 8.0, 5.0], [6.0, 4.0, 3.0], [5.0, 1.0, 
4.0]]], [[[3.0, 5.0, 5.0], [6.0, 5.0, 7.0], [2.0, 5.0, 3.0]], [[4.0, 7.0, 2.0], [5.0, 5.0, 8.0], [8.0, 7.0, 8.0]], [[8.0, 5.0, 
8.0], [4.0, 4.0, 1.0], [1.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 7.0], [5.0, 2.0, 1.0], [2.0, 7.0, 2.0]], 
[[1.0, 6.0, 7.0], [3.0, 4.0, 1.0], [3.0, 8.0, 1.0]], [[8.0, 2.0, 4.0], [8.0, 2.0, 7.0], [7.0, 3.0, 4.0]]], [[[4.0, 8.0, 5.0], 
[7.0, 4.0, 6.0], [3.0, 3.0, 1.0]], [[1.0, 8.0, 7.0], [1.0, 4.0, 4.0], [8.0, 2.0, 4.0]], [[7.0, 2.0, 6.0], [4.0, 5.0, 8.0], 
[3.0, 5.0, 7.0]]], [[[7.0, 8.0, 1.0], [2.0, 5.0, 3.0], [4.0, 5.0, 3.0]], [[1.0, 3.0, 4.0], [2.0, 4.0, 4.0], [3.0, 5.0, 4.0]], 
[[1.0, 5.0, 2.0], [3.0, 3.0, 4.0], [3.0, 4.0, 8.0]]], [[[6.0, 4.0, 1.0], [1.0, 1.0, 7.0], [8.0, 4.0, 3.0]], [[4.0, 7.0, 2.0], 
[5.0, 5.0, 8.0], [8.0, 7.0, 8.0]], [[3.0, 3.0, 6.0], [8.0, 4.0, 8.0], [5.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPCN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 7.0], [3.0, 1.0, 2.0], [7.0, 7.0, 7.0]], [[6.0, 3.0, 7.0], [6.0, 2.0, 1.0], [1.0, 8.0, 
8.0]], [[2.0, 4.0, 4.0], [2.0, 4.0, 5.0], [8.0, 3.0, 1.0]]], [[[7.0, 5.0, 3.0], [5.0, 3.0, 6.0], [3.0, 5.0, 6.0]], [[2.0, 5.0, 
8.0], [3.0, 4.0, 4.0], [1.0, 5.0, 1.0]], [[6.0, 2.0, 7.0], [1.0, 5.0, 4.0], [6.0, 3.0, 8.0]]], [[[4.0, 2.0, 2.0], [7.0, 1.0, 
7.0], [8.0, 3.0, 6.0]], [[1.0, 4.0, 3.0], [7.0, 7.0, 6.0], [5.0, 5.0, 1.0]], [[3.0, 5.0, 5.0], [1.0, 7.0, 3.0], [8.0, 6.0, 
5.0]]], [[[6.0, 7.0, 2.0], [8.0, 7.0, 7.0], [3.0, 8.0, 5.0]], [[7.0, 2.0, 1.0], [1.0, 4.0, 5.0], [8.0, 1.0, 3.0]], [[3.0, 5.0, 
3.0], [5.0, 4.0, 7.0], [3.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 1.0], [5.0, 1.0, 4.0], [8.0, 4.0, 7.0]], [[8.0, 2.0, 3.0], [6.0, 8.0, 
4.0], [5.0, 4.0, 4.0]], [[1.0, 7.0, 7.0], [8.0, 7.0, 3.0], [4.0, 8.0, 7.0]]], [[[7.0, 3.0, 7.0], [8.0, 8.0, 7.0], [7.0, 8.0, 
1.0]], [[6.0, 7.0, 7.0], [4.0, 7.0, 5.0], [5.0, 2.0, 5.0]], [[4.0, 5.0, 3.0], [2.0, 5.0, 8.0], [7.0, 7.0, 4.0]]], [[[7.0, 7.0, 
4.0], [1.0, 5.0, 8.0], [3.0, 8.0, 4.0]], [[3.0, 7.0, 4.0], [7.0, 2.0, 5.0], [7.0, 7.0, 8.0]], [[2.0, 2.0, 3.0], [1.0, 3.0, 
5.0], [1.0, 3.0, 3.0]]], [[[7.0, 7.0, 3.0], [8.0, 6.0, 5.0], [6.0, 1.0, 1.0]], [[7.0, 8.0, 6.0], [8.0, 2.0, 8.0], [3.0, 8.0, 
8.0]], [[8.0, 7.0, 2.0], [1.0, 7.0, 1.0], [2.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[4.0, 2.0, 8.0], [8.0, 4.0, 6.0], [4.0, 8.0, 4.0]], [[5.0, 6.0, 7.0], [8.0, 6.0, 6.0], [7.0, 1.0, 
7.0]], [[8.0, 6.0, 2.0], [4.0, 8.0, 1.0], [2.0, 4.0, 3.0]], [[3.0, 8.0, 3.0], [3.0, 8.0, 4.0], [8.0, 4.0, 
7.0]]]),self.functionspace)
      arg[:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 7.0], [3.0, 1.0, 2.0], [7.0, 7.0, 7.0]], [[4.0, 2.0, 8.0], [8.0, 4.0, 6.0], [4.0, 
8.0, 4.0]], [[2.0, 4.0, 4.0], [2.0, 4.0, 5.0], [8.0, 3.0, 1.0]]], [[[7.0, 5.0, 3.0], [5.0, 3.0, 6.0], [3.0, 5.0, 6.0]], [[5.0, 
6.0, 7.0], [8.0, 6.0, 6.0], [7.0, 1.0, 7.0]], [[6.0, 2.0, 7.0], [1.0, 5.0, 4.0], [6.0, 3.0, 8.0]]], [[[4.0, 2.0, 2.0], [7.0, 
1.0, 7.0], [8.0, 3.0, 6.0]], [[8.0, 6.0, 2.0], [4.0, 8.0, 1.0], [2.0, 4.0, 3.0]], [[3.0, 5.0, 5.0], [1.0, 7.0, 3.0], [8.0, 6.0, 
5.0]]], [[[6.0, 7.0, 2.0], [8.0, 7.0, 7.0], [3.0, 8.0, 5.0]], [[3.0, 8.0, 3.0], [3.0, 8.0, 4.0], [8.0, 4.0, 7.0]], [[3.0, 5.0, 
3.0], [5.0, 4.0, 7.0], [3.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 1.0], [5.0, 1.0, 4.0], [8.0, 4.0, 7.0]], 
[[4.0, 2.0, 8.0], [8.0, 4.0, 6.0], [4.0, 8.0, 4.0]], [[1.0, 7.0, 7.0], [8.0, 7.0, 3.0], [4.0, 8.0, 7.0]]], [[[7.0, 3.0, 7.0], 
[8.0, 8.0, 7.0], [7.0, 8.0, 1.0]], [[5.0, 6.0, 7.0], [8.0, 6.0, 6.0], [7.0, 1.0, 7.0]], [[4.0, 5.0, 3.0], [2.0, 5.0, 8.0], 
[7.0, 7.0, 4.0]]], [[[7.0, 7.0, 4.0], [1.0, 5.0, 8.0], [3.0, 8.0, 4.0]], [[8.0, 6.0, 2.0], [4.0, 8.0, 1.0], [2.0, 4.0, 3.0]], 
[[2.0, 2.0, 3.0], [1.0, 3.0, 5.0], [1.0, 3.0, 3.0]]], [[[7.0, 7.0, 3.0], [8.0, 6.0, 5.0], [6.0, 1.0, 1.0]], [[3.0, 8.0, 3.0], 
[3.0, 8.0, 4.0], [8.0, 4.0, 7.0]], [[8.0, 7.0, 2.0], [1.0, 7.0, 1.0], [2.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPCN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 6.0], [7.0, 7.0, 7.0], [8.0, 2.0, 7.0]], [[7.0, 6.0, 8.0], [5.0, 8.0, 2.0], [4.0, 4.0, 
1.0]], [[5.0, 5.0, 5.0], [5.0, 8.0, 7.0], [1.0, 1.0, 5.0]]], [[[4.0, 6.0, 5.0], [3.0, 6.0, 8.0], [2.0, 2.0, 4.0]], [[2.0, 3.0, 
4.0], [3.0, 7.0, 1.0], [3.0, 2.0, 1.0]], [[7.0, 6.0, 4.0], [5.0, 6.0, 6.0], [2.0, 4.0, 3.0]]], [[[1.0, 8.0, 5.0], [6.0, 8.0, 
1.0], [3.0, 7.0, 2.0]], [[5.0, 1.0, 2.0], [8.0, 4.0, 3.0], [6.0, 1.0, 2.0]], [[8.0, 4.0, 1.0], [1.0, 6.0, 5.0], [5.0, 4.0, 
4.0]]], [[[3.0, 3.0, 4.0], [7.0, 3.0, 4.0], [7.0, 5.0, 5.0]], [[7.0, 7.0, 7.0], [7.0, 2.0, 8.0], [7.0, 5.0, 3.0]], [[8.0, 3.0, 
4.0], [3.0, 6.0, 1.0], [5.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 6.0], [8.0, 8.0, 7.0], [5.0, 4.0, 6.0]], [[7.0, 7.0, 2.0], [4.0, 7.0, 
8.0], [2.0, 7.0, 7.0]], [[5.0, 1.0, 5.0], [5.0, 4.0, 7.0], [4.0, 2.0, 7.0]]], [[[5.0, 1.0, 6.0], [8.0, 4.0, 6.0], [5.0, 1.0, 
7.0]], [[8.0, 1.0, 7.0], [3.0, 1.0, 4.0], [3.0, 8.0, 4.0]], [[5.0, 8.0, 5.0], [1.0, 2.0, 7.0], [4.0, 7.0, 3.0]]], [[[8.0, 6.0, 
5.0], [7.0, 5.0, 5.0], [2.0, 2.0, 2.0]], [[7.0, 5.0, 5.0], [1.0, 1.0, 8.0], [6.0, 8.0, 4.0]], [[7.0, 1.0, 8.0], [6.0, 8.0, 
7.0], [4.0, 5.0, 2.0]]], [[[6.0, 4.0, 6.0], [8.0, 8.0, 3.0], [1.0, 2.0, 2.0]], [[7.0, 6.0, 5.0], [4.0, 6.0, 2.0], [1.0, 1.0, 
7.0]], [[1.0, 8.0, 1.0], [5.0, 4.0, 8.0], [2.0, 6.0, 6.0]]]]))
      val=Data(numpy.array([[[2.0, 3.0, 6.0], [4.0, 2.0, 5.0], [5.0, 8.0, 6.0]], [[8.0, 6.0, 1.0], [5.0, 8.0, 6.0], [8.0, 6.0, 
2.0]], [[2.0, 8.0, 5.0], [2.0, 3.0, 1.0], [5.0, 5.0, 8.0]], [[5.0, 6.0, 7.0], [4.0, 5.0, 8.0], [1.0, 3.0, 
5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 3.0, 4.0], [7.0, 6.0, 2.0], [7.0, 2.0, 5.0]], [[2.0, 8.0, 7.0], [2.0, 5.0, 5.0], 
[8.0, 8.0, 1.0]], [[5.0, 8.0, 5.0], [6.0, 7.0, 2.0], [7.0, 4.0, 1.0]], [[7.0, 7.0, 1.0], [6.0, 2.0, 8.0], [7.0, 1.0, 7.0]]]))
      arg[:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 6.0], [7.0, 7.0, 7.0], [8.0, 2.0, 7.0]], [[2.0, 3.0, 6.0], [4.0, 2.0, 5.0], [5.0, 
8.0, 6.0]], [[5.0, 5.0, 5.0], [5.0, 8.0, 7.0], [1.0, 1.0, 5.0]]], [[[4.0, 6.0, 5.0], [3.0, 6.0, 8.0], [2.0, 2.0, 4.0]], [[8.0, 
6.0, 1.0], [5.0, 8.0, 6.0], [8.0, 6.0, 2.0]], [[7.0, 6.0, 4.0], [5.0, 6.0, 6.0], [2.0, 4.0, 3.0]]], [[[1.0, 8.0, 5.0], [6.0, 
8.0, 1.0], [3.0, 7.0, 2.0]], [[2.0, 8.0, 5.0], [2.0, 3.0, 1.0], [5.0, 5.0, 8.0]], [[8.0, 4.0, 1.0], [1.0, 6.0, 5.0], [5.0, 4.0, 
4.0]]], [[[3.0, 3.0, 4.0], [7.0, 3.0, 4.0], [7.0, 5.0, 5.0]], [[5.0, 6.0, 7.0], [4.0, 5.0, 8.0], [1.0, 3.0, 5.0]], [[8.0, 3.0, 
4.0], [3.0, 6.0, 1.0], [5.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 6.0], [8.0, 8.0, 7.0], [5.0, 4.0, 6.0]], 
[[2.0, 3.0, 4.0], [7.0, 6.0, 2.0], [7.0, 2.0, 5.0]], [[5.0, 1.0, 5.0], [5.0, 4.0, 7.0], [4.0, 2.0, 7.0]]], [[[5.0, 1.0, 6.0], 
[8.0, 4.0, 6.0], [5.0, 1.0, 7.0]], [[2.0, 8.0, 7.0], [2.0, 5.0, 5.0], [8.0, 8.0, 1.0]], [[5.0, 8.0, 5.0], [1.0, 2.0, 7.0], 
[4.0, 7.0, 3.0]]], [[[8.0, 6.0, 5.0], [7.0, 5.0, 5.0], [2.0, 2.0, 2.0]], [[5.0, 8.0, 5.0], [6.0, 7.0, 2.0], [7.0, 4.0, 1.0]], 
[[7.0, 1.0, 8.0], [6.0, 8.0, 7.0], [4.0, 5.0, 2.0]]], [[[6.0, 4.0, 6.0], [8.0, 8.0, 3.0], [1.0, 2.0, 2.0]], [[7.0, 7.0, 1.0], 
[6.0, 2.0, 8.0], [7.0, 1.0, 7.0]], [[1.0, 8.0, 1.0], [5.0, 4.0, 8.0], [2.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPCN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 7.0], [8.0, 3.0, 4.0], [6.0, 5.0, 2.0]], [[7.0, 3.0, 7.0], [1.0, 1.0, 6.0], [4.0, 8.0, 
6.0]], [[3.0, 6.0, 1.0], [7.0, 5.0, 5.0], [8.0, 6.0, 4.0]]], [[[8.0, 4.0, 2.0], [7.0, 7.0, 2.0], [2.0, 3.0, 5.0]], [[2.0, 6.0, 
1.0], [3.0, 3.0, 3.0], [7.0, 8.0, 1.0]], [[6.0, 1.0, 5.0], [1.0, 1.0, 3.0], [5.0, 8.0, 5.0]]], [[[5.0, 7.0, 4.0], [2.0, 6.0, 
5.0], [6.0, 5.0, 8.0]], [[3.0, 4.0, 4.0], [7.0, 2.0, 3.0], [3.0, 8.0, 1.0]], [[5.0, 3.0, 7.0], [5.0, 7.0, 7.0], [6.0, 1.0, 
1.0]]], [[[1.0, 8.0, 1.0], [6.0, 4.0, 1.0], [2.0, 7.0, 3.0]], [[2.0, 7.0, 2.0], [2.0, 5.0, 1.0], [3.0, 7.0, 5.0]], [[8.0, 2.0, 
1.0], [7.0, 5.0, 6.0], [6.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 6.0], [5.0, 2.0, 7.0], [2.0, 6.0, 4.0]], [[4.0, 2.0, 4.0], [2.0, 1.0, 
4.0], [6.0, 5.0, 3.0]], [[4.0, 6.0, 7.0], [1.0, 1.0, 4.0], [5.0, 4.0, 1.0]]], [[[7.0, 6.0, 2.0], [6.0, 7.0, 3.0], [2.0, 7.0, 
8.0]], [[2.0, 2.0, 6.0], [1.0, 6.0, 7.0], [2.0, 3.0, 6.0]], [[1.0, 7.0, 1.0], [2.0, 3.0, 6.0], [1.0, 4.0, 4.0]]], [[[4.0, 5.0, 
8.0], [7.0, 1.0, 7.0], [6.0, 8.0, 5.0]], [[6.0, 6.0, 5.0], [8.0, 8.0, 8.0], [5.0, 7.0, 7.0]], [[4.0, 8.0, 4.0], [6.0, 1.0, 
1.0], [5.0, 7.0, 3.0]]], [[[7.0, 6.0, 7.0], [1.0, 8.0, 8.0], [8.0, 5.0, 4.0]], [[8.0, 3.0, 4.0], [8.0, 6.0, 5.0], [3.0, 2.0, 
1.0]], [[8.0, 6.0, 2.0], [1.0, 6.0, 8.0], [5.0, 2.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 3.0, 5.0], [3.0, 6.0, 5.0], [5.0, 6.0, 2.0]], [[4.0, 4.0, 4.0], [8.0, 8.0, 7.0], [7.0, 
7.0, 3.0]], [[3.0, 2.0, 4.0], [5.0, 2.0, 6.0], [6.0, 6.0, 1.0]], [[3.0, 1.0, 8.0], [8.0, 8.0, 5.0], [1.0, 5.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[2.0, 2.0, 4.0], [2.0, 5.0, 7.0], [7.0, 5.0, 6.0]], [[6.0, 1.0, 4.0], [5.0, 2.0, 1.0], [4.0, 
5.0, 4.0]], [[6.0, 6.0, 7.0], [2.0, 7.0, 6.0], [8.0, 1.0, 2.0]], [[1.0, 4.0, 7.0], [6.0, 8.0, 6.0], [1.0, 2.0, 1.0]]])
      arg[:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 7.0], [8.0, 3.0, 4.0], [6.0, 5.0, 2.0]], [[8.0, 3.0, 5.0], [3.0, 6.0, 5.0], [5.0, 
6.0, 2.0]], [[3.0, 6.0, 1.0], [7.0, 5.0, 5.0], [8.0, 6.0, 4.0]]], [[[8.0, 4.0, 2.0], [7.0, 7.0, 2.0], [2.0, 3.0, 5.0]], [[4.0, 
4.0, 4.0], [8.0, 8.0, 7.0], [7.0, 7.0, 3.0]], [[6.0, 1.0, 5.0], [1.0, 1.0, 3.0], [5.0, 8.0, 5.0]]], [[[5.0, 7.0, 4.0], [2.0, 
6.0, 5.0], [6.0, 5.0, 8.0]], [[3.0, 2.0, 4.0], [5.0, 2.0, 6.0], [6.0, 6.0, 1.0]], [[5.0, 3.0, 7.0], [5.0, 7.0, 7.0], [6.0, 1.0, 
1.0]]], [[[1.0, 8.0, 1.0], [6.0, 4.0, 1.0], [2.0, 7.0, 3.0]], [[3.0, 1.0, 8.0], [8.0, 8.0, 5.0], [1.0, 5.0, 4.0]], [[8.0, 2.0, 
1.0], [7.0, 5.0, 6.0], [6.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 6.0], [5.0, 2.0, 7.0], [2.0, 6.0, 4.0]], 
[[2.0, 2.0, 4.0], [2.0, 5.0, 7.0], [7.0, 5.0, 6.0]], [[4.0, 6.0, 7.0], [1.0, 1.0, 4.0], [5.0, 4.0, 1.0]]], [[[7.0, 6.0, 2.0], 
[6.0, 7.0, 3.0], [2.0, 7.0, 8.0]], [[6.0, 1.0, 4.0], [5.0, 2.0, 1.0], [4.0, 5.0, 4.0]], [[1.0, 7.0, 1.0], [2.0, 3.0, 6.0], 
[1.0, 4.0, 4.0]]], [[[4.0, 5.0, 8.0], [7.0, 1.0, 7.0], [6.0, 8.0, 5.0]], [[6.0, 6.0, 7.0], [2.0, 7.0, 6.0], [8.0, 1.0, 2.0]], 
[[4.0, 8.0, 4.0], [6.0, 1.0, 1.0], [5.0, 7.0, 3.0]]], [[[7.0, 6.0, 7.0], [1.0, 8.0, 8.0], [8.0, 5.0, 4.0]], [[1.0, 4.0, 7.0], 
[6.0, 8.0, 6.0], [1.0, 2.0, 1.0]], [[8.0, 6.0, 2.0], [1.0, 6.0, 8.0], [5.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULLN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 1.0], [7.0, 5.0, 2.0], [2.0, 6.0, 4.0]], [[5.0, 2.0, 3.0], [8.0, 6.0, 1.0], [2.0, 6.0, 
4.0]], [[1.0, 7.0, 4.0], [7.0, 8.0, 8.0], [7.0, 2.0, 7.0]]], [[[4.0, 3.0, 1.0], [5.0, 4.0, 6.0], [5.0, 3.0, 4.0]], [[4.0, 1.0, 
8.0], [2.0, 2.0, 4.0], [3.0, 2.0, 5.0]], [[5.0, 1.0, 6.0], [4.0, 5.0, 8.0], [4.0, 7.0, 3.0]]], [[[7.0, 1.0, 7.0], [4.0, 4.0, 
2.0], [8.0, 4.0, 4.0]], [[1.0, 3.0, 4.0], [6.0, 8.0, 3.0], [2.0, 6.0, 3.0]], [[6.0, 7.0, 4.0], [7.0, 3.0, 7.0], [3.0, 1.0, 
1.0]]], [[[3.0, 8.0, 2.0], [1.0, 8.0, 8.0], [4.0, 6.0, 8.0]], [[7.0, 4.0, 7.0], [4.0, 4.0, 5.0], [3.0, 7.0, 2.0]], [[4.0, 3.0, 
5.0], [1.0, 2.0, 4.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 8.0], [7.0, 6.0, 6.0], [3.0, 4.0, 3.0]], [[2.0, 1.0, 3.0], [1.0, 3.0, 
7.0], [2.0, 4.0, 2.0]], [[1.0, 1.0, 4.0], [7.0, 3.0, 4.0], [7.0, 4.0, 2.0]]], [[[2.0, 6.0, 6.0], [5.0, 2.0, 7.0], [5.0, 6.0, 
6.0]], [[8.0, 5.0, 3.0], [6.0, 5.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 3.0, 1.0], [2.0, 1.0, 7.0], [5.0, 3.0, 7.0]]], [[[3.0, 3.0, 
2.0], [3.0, 6.0, 6.0], [8.0, 7.0, 3.0]], [[2.0, 5.0, 1.0], [2.0, 2.0, 3.0], [6.0, 3.0, 5.0]], [[4.0, 8.0, 3.0], [5.0, 4.0, 
2.0], [6.0, 3.0, 1.0]]], [[[8.0, 4.0, 6.0], [4.0, 3.0, 7.0], [6.0, 4.0, 6.0]], [[8.0, 5.0, 7.0], [4.0, 5.0, 7.0], [3.0, 6.0, 
8.0]], [[5.0, 5.0, 6.0], [5.0, 4.0, 2.0], [7.0, 3.0, 5.0]]]]))
      val=numpy.array([[[[3.0, 6.0, 2.0]]], [[[4.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0]]]])
      arg[:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [7.0, 5.0, 2.0], [2.0, 6.0, 4.0]], [[5.0, 2.0, 3.0], [8.0, 6.0, 1.0], [2.0, 
6.0, 4.0]], [[1.0, 7.0, 4.0], [7.0, 8.0, 8.0], [3.0, 6.0, 2.0]]], [[[4.0, 3.0, 1.0], [5.0, 4.0, 6.0], [5.0, 3.0, 4.0]], [[4.0, 
1.0, 8.0], [2.0, 2.0, 4.0], [3.0, 2.0, 5.0]], [[5.0, 1.0, 6.0], [4.0, 5.0, 8.0], [4.0, 3.0, 4.0]]], [[[7.0, 1.0, 7.0], [4.0, 
4.0, 2.0], [8.0, 4.0, 4.0]], [[1.0, 3.0, 4.0], [6.0, 8.0, 3.0], [2.0, 6.0, 3.0]], [[6.0, 7.0, 4.0], [7.0, 3.0, 7.0], [5.0, 4.0, 
8.0]]], [[[3.0, 8.0, 2.0], [1.0, 8.0, 8.0], [4.0, 6.0, 8.0]], [[7.0, 4.0, 7.0], [4.0, 4.0, 5.0], [3.0, 7.0, 2.0]], [[4.0, 3.0, 
5.0], [1.0, 2.0, 4.0], [8.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 8.0], [7.0, 6.0, 6.0], [3.0, 4.0, 3.0]], 
[[2.0, 1.0, 3.0], [1.0, 3.0, 7.0], [2.0, 4.0, 2.0]], [[1.0, 1.0, 4.0], [7.0, 3.0, 4.0], [3.0, 6.0, 2.0]]], [[[2.0, 6.0, 6.0], 
[5.0, 2.0, 7.0], [5.0, 6.0, 6.0]], [[8.0, 5.0, 3.0], [6.0, 5.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 3.0, 1.0], [2.0, 1.0, 7.0], 
[4.0, 3.0, 4.0]]], [[[3.0, 3.0, 2.0], [3.0, 6.0, 6.0], [8.0, 7.0, 3.0]], [[2.0, 5.0, 1.0], [2.0, 2.0, 3.0], [6.0, 3.0, 5.0]], 
[[4.0, 8.0, 3.0], [5.0, 4.0, 2.0], [5.0, 4.0, 8.0]]], [[[8.0, 4.0, 6.0], [4.0, 3.0, 7.0], [6.0, 4.0, 6.0]], [[8.0, 5.0, 7.0], 
[4.0, 5.0, 7.0], [3.0, 6.0, 8.0]], [[5.0, 5.0, 6.0], [5.0, 4.0, 2.0], [7.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULLN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 5.0], [4.0, 4.0, 7.0], [4.0, 2.0, 4.0]], [[1.0, 6.0, 2.0], [8.0, 4.0, 5.0], [5.0, 1.0, 
2.0]], [[3.0, 2.0, 8.0], [1.0, 7.0, 6.0], [7.0, 7.0, 8.0]]], [[[6.0, 8.0, 8.0], [8.0, 2.0, 7.0], [2.0, 2.0, 4.0]], [[8.0, 2.0, 
6.0], [3.0, 6.0, 4.0], [7.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], [1.0, 5.0, 5.0], [5.0, 5.0, 1.0]]], [[[5.0, 7.0, 1.0], [6.0, 6.0, 
7.0], [2.0, 2.0, 1.0]], [[1.0, 5.0, 3.0], [4.0, 6.0, 3.0], [5.0, 8.0, 1.0]], [[7.0, 2.0, 8.0], [2.0, 4.0, 1.0], [5.0, 5.0, 
6.0]]], [[[3.0, 2.0, 6.0], [5.0, 5.0, 4.0], [3.0, 3.0, 8.0]], [[2.0, 6.0, 8.0], [2.0, 8.0, 3.0], [4.0, 3.0, 4.0]], [[3.0, 6.0, 
6.0], [2.0, 3.0, 3.0], [4.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 1.0], [2.0, 4.0, 7.0], [6.0, 8.0, 7.0]], [[3.0, 6.0, 8.0], [4.0, 6.0, 
4.0], [3.0, 8.0, 1.0]], [[7.0, 3.0, 4.0], [4.0, 1.0, 5.0], [4.0, 4.0, 3.0]]], [[[2.0, 8.0, 7.0], [7.0, 8.0, 5.0], [6.0, 4.0, 
1.0]], [[8.0, 7.0, 2.0], [4.0, 3.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 4.0, 8.0], [7.0, 8.0, 7.0], [1.0, 2.0, 5.0]]], [[[8.0, 3.0, 
3.0], [4.0, 6.0, 1.0], [5.0, 4.0, 2.0]], [[3.0, 1.0, 6.0], [8.0, 8.0, 7.0], [8.0, 4.0, 7.0]], [[6.0, 8.0, 7.0], [4.0, 1.0, 
6.0], [8.0, 7.0, 5.0]]], [[[8.0, 5.0, 2.0], [1.0, 5.0, 7.0], [3.0, 5.0, 5.0]], [[5.0, 3.0, 3.0], [2.0, 7.0, 3.0], [2.0, 5.0, 
4.0]], [[6.0, 8.0, 3.0], [6.0, 4.0, 6.0], [7.0, 5.0, 8.0]]]]))
      val=Data(numpy.array([[[[8.0, 6.0, 6.0]]], [[[8.0, 6.0, 2.0]]], [[[8.0, 5.0, 8.0]]]]),self.functionspace)
      arg[:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 5.0], [4.0, 4.0, 7.0], [4.0, 2.0, 4.0]], [[1.0, 6.0, 2.0], [8.0, 4.0, 5.0], [5.0, 
1.0, 2.0]], [[3.0, 2.0, 8.0], [1.0, 7.0, 6.0], [8.0, 6.0, 6.0]]], [[[6.0, 8.0, 8.0], [8.0, 2.0, 7.0], [2.0, 2.0, 4.0]], [[8.0, 
2.0, 6.0], [3.0, 6.0, 4.0], [7.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], [1.0, 5.0, 5.0], [8.0, 6.0, 2.0]]], [[[5.0, 7.0, 1.0], [6.0, 
6.0, 7.0], [2.0, 2.0, 1.0]], [[1.0, 5.0, 3.0], [4.0, 6.0, 3.0], [5.0, 8.0, 1.0]], [[7.0, 2.0, 8.0], [2.0, 4.0, 1.0], [8.0, 5.0, 
8.0]]], [[[3.0, 2.0, 6.0], [5.0, 5.0, 4.0], [3.0, 3.0, 8.0]], [[2.0, 6.0, 8.0], [2.0, 8.0, 3.0], [4.0, 3.0, 4.0]], [[3.0, 6.0, 
6.0], [2.0, 3.0, 3.0], [4.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 1.0], [2.0, 4.0, 7.0], [6.0, 8.0, 7.0]], 
[[3.0, 6.0, 8.0], [4.0, 6.0, 4.0], [3.0, 8.0, 1.0]], [[7.0, 3.0, 4.0], [4.0, 1.0, 5.0], [8.0, 6.0, 6.0]]], [[[2.0, 8.0, 7.0], 
[7.0, 8.0, 5.0], [6.0, 4.0, 1.0]], [[8.0, 7.0, 2.0], [4.0, 3.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 4.0, 8.0], [7.0, 8.0, 7.0], 
[8.0, 6.0, 2.0]]], [[[8.0, 3.0, 3.0], [4.0, 6.0, 1.0], [5.0, 4.0, 2.0]], [[3.0, 1.0, 6.0], [8.0, 8.0, 7.0], [8.0, 4.0, 7.0]], 
[[6.0, 8.0, 7.0], [4.0, 1.0, 6.0], [8.0, 5.0, 8.0]]], [[[8.0, 5.0, 2.0], [1.0, 5.0, 7.0], [3.0, 5.0, 5.0]], [[5.0, 3.0, 3.0], 
[2.0, 7.0, 3.0], [2.0, 5.0, 4.0]], [[6.0, 8.0, 3.0], [6.0, 4.0, 6.0], [7.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULLN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 5.0], [6.0, 1.0, 2.0], [1.0, 4.0, 7.0]], [[7.0, 3.0, 3.0], [7.0, 8.0, 6.0], [6.0, 4.0, 
3.0]], [[4.0, 3.0, 8.0], [3.0, 8.0, 1.0], [4.0, 5.0, 8.0]]], [[[3.0, 4.0, 4.0], [4.0, 3.0, 8.0], [7.0, 3.0, 2.0]], [[1.0, 8.0, 
4.0], [4.0, 5.0, 6.0], [4.0, 2.0, 3.0]], [[5.0, 1.0, 8.0], [6.0, 3.0, 6.0], [5.0, 8.0, 5.0]]], [[[4.0, 3.0, 4.0], [6.0, 8.0, 
2.0], [7.0, 3.0, 6.0]], [[7.0, 1.0, 8.0], [2.0, 8.0, 8.0], [7.0, 3.0, 7.0]], [[6.0, 1.0, 3.0], [4.0, 1.0, 7.0], [6.0, 7.0, 
4.0]]], [[[6.0, 4.0, 3.0], [8.0, 8.0, 6.0], [6.0, 4.0, 3.0]], [[3.0, 6.0, 5.0], [2.0, 2.0, 4.0], [8.0, 5.0, 1.0]], [[7.0, 2.0, 
2.0], [4.0, 3.0, 6.0], [1.0, 3.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 8.0], [8.0, 8.0, 3.0], [4.0, 4.0, 6.0]], [[5.0, 2.0, 5.0], [5.0, 1.0, 
2.0], [3.0, 4.0, 2.0]], [[3.0, 4.0, 1.0], [6.0, 5.0, 6.0], [5.0, 2.0, 6.0]]], [[[7.0, 7.0, 1.0], [4.0, 2.0, 6.0], [5.0, 4.0, 
6.0]], [[7.0, 2.0, 5.0], [2.0, 1.0, 2.0], [4.0, 2.0, 1.0]], [[7.0, 1.0, 1.0], [8.0, 3.0, 4.0], [5.0, 1.0, 1.0]]], [[[3.0, 8.0, 
7.0], [6.0, 6.0, 7.0], [6.0, 3.0, 4.0]], [[6.0, 8.0, 2.0], [4.0, 5.0, 6.0], [1.0, 2.0, 2.0]], [[4.0, 3.0, 7.0], [3.0, 8.0, 
3.0], [7.0, 6.0, 4.0]]], [[[4.0, 3.0, 6.0], [8.0, 1.0, 4.0], [2.0, 4.0, 3.0]], [[3.0, 5.0, 2.0], [1.0, 2.0, 1.0], [1.0, 4.0, 
2.0]], [[3.0, 8.0, 3.0], [3.0, 4.0, 6.0], [3.0, 2.0, 7.0]]]]))
      val=Data(numpy.array([[[[2.0, 7.0, 4.0]]], [[[6.0, 2.0, 8.0]]], [[[1.0, 2.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 4.0]]], [[[4.0, 6.0, 2.0]]], [[[7.0, 7.0, 2.0]]]]))
      arg[:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 5.0], [6.0, 1.0, 2.0], [1.0, 4.0, 7.0]], [[7.0, 3.0, 3.0], [7.0, 8.0, 6.0], [6.0, 
4.0, 3.0]], [[4.0, 3.0, 8.0], [3.0, 8.0, 1.0], [2.0, 7.0, 4.0]]], [[[3.0, 4.0, 4.0], [4.0, 3.0, 8.0], [7.0, 3.0, 2.0]], [[1.0, 
8.0, 4.0], [4.0, 5.0, 6.0], [4.0, 2.0, 3.0]], [[5.0, 1.0, 8.0], [6.0, 3.0, 6.0], [6.0, 2.0, 8.0]]], [[[4.0, 3.0, 4.0], [6.0, 
8.0, 2.0], [7.0, 3.0, 6.0]], [[7.0, 1.0, 8.0], [2.0, 8.0, 8.0], [7.0, 3.0, 7.0]], [[6.0, 1.0, 3.0], [4.0, 1.0, 7.0], [1.0, 2.0, 
2.0]]], [[[6.0, 4.0, 3.0], [8.0, 8.0, 6.0], [6.0, 4.0, 3.0]], [[3.0, 6.0, 5.0], [2.0, 2.0, 4.0], [8.0, 5.0, 1.0]], [[7.0, 2.0, 
2.0], [4.0, 3.0, 6.0], [1.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 8.0], [8.0, 8.0, 3.0], [4.0, 4.0, 6.0]], 
[[5.0, 2.0, 5.0], [5.0, 1.0, 2.0], [3.0, 4.0, 2.0]], [[3.0, 4.0, 1.0], [6.0, 5.0, 6.0], [6.0, 8.0, 4.0]]], [[[7.0, 7.0, 1.0], 
[4.0, 2.0, 6.0], [5.0, 4.0, 6.0]], [[7.0, 2.0, 5.0], [2.0, 1.0, 2.0], [4.0, 2.0, 1.0]], [[7.0, 1.0, 1.0], [8.0, 3.0, 4.0], 
[4.0, 6.0, 2.0]]], [[[3.0, 8.0, 7.0], [6.0, 6.0, 7.0], [6.0, 3.0, 4.0]], [[6.0, 8.0, 2.0], [4.0, 5.0, 6.0], [1.0, 2.0, 2.0]], 
[[4.0, 3.0, 7.0], [3.0, 8.0, 3.0], [7.0, 7.0, 2.0]]], [[[4.0, 3.0, 6.0], [8.0, 1.0, 4.0], [2.0, 4.0, 3.0]], [[3.0, 5.0, 2.0], 
[1.0, 2.0, 1.0], [1.0, 4.0, 2.0]], [[3.0, 8.0, 3.0], [3.0, 4.0, 6.0], [3.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULLN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 4.0], [5.0, 6.0, 3.0], [3.0, 2.0, 5.0]], [[1.0, 2.0, 7.0], [3.0, 2.0, 6.0], [7.0, 7.0, 
1.0]], [[7.0, 7.0, 5.0], [1.0, 6.0, 1.0], [2.0, 4.0, 8.0]]], [[[2.0, 3.0, 2.0], [3.0, 2.0, 1.0], [6.0, 7.0, 8.0]], [[5.0, 5.0, 
7.0], [7.0, 3.0, 4.0], [2.0, 4.0, 8.0]], [[5.0, 1.0, 4.0], [4.0, 6.0, 6.0], [5.0, 1.0, 2.0]]], [[[2.0, 3.0, 2.0], [1.0, 1.0, 
1.0], [7.0, 5.0, 7.0]], [[2.0, 2.0, 7.0], [8.0, 4.0, 2.0], [2.0, 3.0, 3.0]], [[4.0, 8.0, 8.0], [1.0, 3.0, 2.0], [4.0, 1.0, 
8.0]]], [[[5.0, 4.0, 3.0], [1.0, 3.0, 7.0], [7.0, 6.0, 5.0]], [[2.0, 8.0, 5.0], [1.0, 3.0, 5.0], [4.0, 8.0, 8.0]], [[7.0, 5.0, 
7.0], [3.0, 7.0, 2.0], [6.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 5.0], [7.0, 1.0, 5.0], [3.0, 2.0, 7.0]], [[5.0, 3.0, 5.0], [8.0, 2.0, 
1.0], [3.0, 4.0, 2.0]], [[6.0, 5.0, 4.0], [4.0, 3.0, 4.0], [4.0, 1.0, 1.0]]], [[[6.0, 4.0, 8.0], [8.0, 6.0, 4.0], [5.0, 5.0, 
2.0]], [[5.0, 5.0, 3.0], [2.0, 6.0, 5.0], [2.0, 5.0, 7.0]], [[4.0, 2.0, 4.0], [6.0, 1.0, 5.0], [1.0, 2.0, 2.0]]], [[[8.0, 5.0, 
3.0], [8.0, 6.0, 2.0], [1.0, 5.0, 7.0]], [[8.0, 1.0, 3.0], [1.0, 2.0, 2.0], [4.0, 6.0, 6.0]], [[8.0, 1.0, 1.0], [4.0, 5.0, 
4.0], [6.0, 4.0, 3.0]]], [[[2.0, 1.0, 2.0], [1.0, 7.0, 2.0], [4.0, 1.0, 1.0]], [[3.0, 2.0, 3.0], [6.0, 8.0, 6.0], [5.0, 4.0, 
6.0]], [[2.0, 6.0, 6.0], [6.0, 4.0, 5.0], [4.0, 4.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 1.0, 6.0]]], [[[3.0, 2.0, 5.0]]], [[[6.0, 2.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 7.0, 8.0]]], [[[7.0, 2.0, 2.0]]], [[[7.0, 3.0, 4.0]]]])
      arg[:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 4.0], [5.0, 6.0, 3.0], [3.0, 2.0, 5.0]], [[1.0, 2.0, 7.0], [3.0, 2.0, 6.0], [7.0, 
7.0, 1.0]], [[7.0, 7.0, 5.0], [1.0, 6.0, 1.0], [5.0, 1.0, 6.0]]], [[[2.0, 3.0, 2.0], [3.0, 2.0, 1.0], [6.0, 7.0, 8.0]], [[5.0, 
5.0, 7.0], [7.0, 3.0, 4.0], [2.0, 4.0, 8.0]], [[5.0, 1.0, 4.0], [4.0, 6.0, 6.0], [3.0, 2.0, 5.0]]], [[[2.0, 3.0, 2.0], [1.0, 
1.0, 1.0], [7.0, 5.0, 7.0]], [[2.0, 2.0, 7.0], [8.0, 4.0, 2.0], [2.0, 3.0, 3.0]], [[4.0, 8.0, 8.0], [1.0, 3.0, 2.0], [6.0, 2.0, 
2.0]]], [[[5.0, 4.0, 3.0], [1.0, 3.0, 7.0], [7.0, 6.0, 5.0]], [[2.0, 8.0, 5.0], [1.0, 3.0, 5.0], [4.0, 8.0, 8.0]], [[7.0, 5.0, 
7.0], [3.0, 7.0, 2.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 5.0], [7.0, 1.0, 5.0], [3.0, 2.0, 7.0]], 
[[5.0, 3.0, 5.0], [8.0, 2.0, 1.0], [3.0, 4.0, 2.0]], [[6.0, 5.0, 4.0], [4.0, 3.0, 4.0], [1.0, 7.0, 8.0]]], [[[6.0, 4.0, 8.0], 
[8.0, 6.0, 4.0], [5.0, 5.0, 2.0]], [[5.0, 5.0, 3.0], [2.0, 6.0, 5.0], [2.0, 5.0, 7.0]], [[4.0, 2.0, 4.0], [6.0, 1.0, 5.0], 
[7.0, 2.0, 2.0]]], [[[8.0, 5.0, 3.0], [8.0, 6.0, 2.0], [1.0, 5.0, 7.0]], [[8.0, 1.0, 3.0], [1.0, 2.0, 2.0], [4.0, 6.0, 6.0]], 
[[8.0, 1.0, 1.0], [4.0, 5.0, 4.0], [7.0, 3.0, 4.0]]], [[[2.0, 1.0, 2.0], [1.0, 7.0, 2.0], [4.0, 1.0, 1.0]], [[3.0, 2.0, 3.0], 
[6.0, 8.0, 6.0], [5.0, 4.0, 6.0]], [[2.0, 6.0, 6.0], [6.0, 4.0, 5.0], [4.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LCLN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 2.0], [7.0, 5.0, 1.0], [3.0, 1.0, 5.0]], [[1.0, 1.0, 8.0], [3.0, 4.0, 1.0], [5.0, 7.0, 
5.0]], [[2.0, 2.0, 6.0], [8.0, 4.0, 4.0], [3.0, 2.0, 1.0]]], [[[5.0, 5.0, 8.0], [5.0, 1.0, 5.0], [7.0, 7.0, 1.0]], [[8.0, 6.0, 
7.0], [8.0, 6.0, 8.0], [2.0, 3.0, 4.0]], [[2.0, 5.0, 3.0], [6.0, 8.0, 8.0], [7.0, 8.0, 1.0]]], [[[1.0, 6.0, 3.0], [2.0, 7.0, 
1.0], [2.0, 1.0, 6.0]], [[7.0, 6.0, 3.0], [5.0, 8.0, 3.0], [4.0, 1.0, 6.0]], [[6.0, 4.0, 1.0], [4.0, 1.0, 6.0], [3.0, 2.0, 
5.0]]], [[[6.0, 7.0, 7.0], [1.0, 3.0, 5.0], [7.0, 5.0, 6.0]], [[4.0, 4.0, 8.0], [5.0, 8.0, 3.0], [6.0, 1.0, 6.0]], [[3.0, 4.0, 
3.0], [7.0, 1.0, 2.0], [8.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 5.0], [4.0, 5.0, 6.0], [7.0, 8.0, 5.0]], [[5.0, 6.0, 5.0], [6.0, 7.0, 
7.0], [1.0, 2.0, 1.0]], [[2.0, 6.0, 2.0], [1.0, 4.0, 8.0], [3.0, 2.0, 8.0]]], [[[6.0, 6.0, 6.0], [2.0, 5.0, 7.0], [4.0, 5.0, 
1.0]], [[4.0, 7.0, 8.0], [1.0, 3.0, 4.0], [7.0, 3.0, 2.0]], [[6.0, 5.0, 3.0], [5.0, 6.0, 2.0], [1.0, 1.0, 5.0]]], [[[7.0, 7.0, 
6.0], [2.0, 7.0, 5.0], [7.0, 5.0, 8.0]], [[8.0, 8.0, 7.0], [7.0, 1.0, 3.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 5.0], [7.0, 2.0, 
4.0], [5.0, 5.0, 6.0]]], [[[4.0, 5.0, 8.0], [2.0, 5.0, 4.0], [7.0, 1.0, 2.0]], [[1.0, 5.0, 6.0], [7.0, 6.0, 5.0], [4.0, 2.0, 
3.0]], [[1.0, 2.0, 6.0], [1.0, 5.0, 6.0], [5.0, 1.0, 6.0]]]]))
      val=numpy.array([[[[2.0, 8.0, 4.0]], [[2.0, 2.0, 5.0]], [[1.0, 8.0, 8.0]]], [[[4.0, 6.0, 2.0]], [[5.0, 5.0, 2.0]], [[5.0, 
3.0, 4.0]]]])
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 2.0], [7.0, 5.0, 1.0], [3.0, 1.0, 5.0]], [[1.0, 1.0, 8.0], [3.0, 4.0, 1.0], [5.0, 
7.0, 5.0]], [[2.0, 2.0, 6.0], [8.0, 4.0, 4.0], [3.0, 2.0, 1.0]]], [[[5.0, 5.0, 8.0], [5.0, 1.0, 5.0], [7.0, 7.0, 1.0]], [[8.0, 
6.0, 7.0], [8.0, 6.0, 8.0], [2.0, 3.0, 4.0]], [[2.0, 5.0, 3.0], [6.0, 8.0, 8.0], [7.0, 8.0, 1.0]]], [[[1.0, 6.0, 3.0], [2.0, 
7.0, 1.0], [2.0, 8.0, 4.0]], [[7.0, 6.0, 3.0], [5.0, 8.0, 3.0], [2.0, 2.0, 5.0]], [[6.0, 4.0, 1.0], [4.0, 1.0, 6.0], [1.0, 8.0, 
8.0]]], [[[6.0, 7.0, 7.0], [1.0, 3.0, 5.0], [4.0, 6.0, 2.0]], [[4.0, 4.0, 8.0], [5.0, 8.0, 3.0], [5.0, 5.0, 2.0]], [[3.0, 4.0, 
3.0], [7.0, 1.0, 2.0], [5.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 5.0], [4.0, 5.0, 6.0], [7.0, 8.0, 5.0]], 
[[5.0, 6.0, 5.0], [6.0, 7.0, 7.0], [1.0, 2.0, 1.0]], [[2.0, 6.0, 2.0], [1.0, 4.0, 8.0], [3.0, 2.0, 8.0]]], [[[6.0, 6.0, 6.0], 
[2.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[4.0, 7.0, 8.0], [1.0, 3.0, 4.0], [7.0, 3.0, 2.0]], [[6.0, 5.0, 3.0], [5.0, 6.0, 2.0], 
[1.0, 1.0, 5.0]]], [[[7.0, 7.0, 6.0], [2.0, 7.0, 5.0], [2.0, 8.0, 4.0]], [[8.0, 8.0, 7.0], [7.0, 1.0, 3.0], [2.0, 2.0, 5.0]], 
[[6.0, 6.0, 5.0], [7.0, 2.0, 4.0], [1.0, 8.0, 8.0]]], [[[4.0, 5.0, 8.0], [2.0, 5.0, 4.0], [4.0, 6.0, 2.0]], [[1.0, 5.0, 6.0], 
[7.0, 6.0, 5.0], [5.0, 5.0, 2.0]], [[1.0, 2.0, 6.0], [1.0, 5.0, 6.0], [5.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LCLN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 7.0], [1.0, 7.0, 4.0], [5.0, 5.0, 4.0]], [[8.0, 1.0, 7.0], [4.0, 6.0, 1.0], [2.0, 8.0, 
3.0]], [[4.0, 2.0, 8.0], [3.0, 3.0, 4.0], [5.0, 2.0, 3.0]]], [[[4.0, 1.0, 7.0], [4.0, 2.0, 2.0], [6.0, 1.0, 8.0]], [[4.0, 5.0, 
8.0], [4.0, 6.0, 6.0], [3.0, 6.0, 6.0]], [[1.0, 5.0, 7.0], [4.0, 6.0, 2.0], [7.0, 2.0, 8.0]]], [[[2.0, 4.0, 2.0], [4.0, 3.0, 
6.0], [5.0, 7.0, 1.0]], [[1.0, 8.0, 7.0], [1.0, 5.0, 8.0], [5.0, 6.0, 8.0]], [[5.0, 5.0, 4.0], [8.0, 3.0, 7.0], [7.0, 6.0, 
8.0]]], [[[4.0, 3.0, 7.0], [1.0, 3.0, 7.0], [7.0, 8.0, 2.0]], [[8.0, 2.0, 6.0], [4.0, 3.0, 8.0], [7.0, 1.0, 4.0]], [[5.0, 6.0, 
2.0], [4.0, 4.0, 8.0], [4.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 1.0], [5.0, 8.0, 2.0], [7.0, 3.0, 3.0]], [[2.0, 2.0, 4.0], [7.0, 2.0, 
4.0], [5.0, 1.0, 8.0]], [[2.0, 8.0, 3.0], [3.0, 8.0, 1.0], [5.0, 8.0, 3.0]]], [[[8.0, 3.0, 4.0], [4.0, 6.0, 6.0], [5.0, 2.0, 
5.0]], [[1.0, 5.0, 4.0], [4.0, 2.0, 6.0], [1.0, 5.0, 2.0]], [[7.0, 4.0, 3.0], [2.0, 6.0, 4.0], [6.0, 2.0, 2.0]]], [[[3.0, 6.0, 
8.0], [8.0, 3.0, 3.0], [7.0, 5.0, 2.0]], [[7.0, 3.0, 5.0], [8.0, 6.0, 8.0], [3.0, 1.0, 3.0]], [[2.0, 6.0, 8.0], [1.0, 2.0, 
5.0], [3.0, 5.0, 3.0]]], [[[2.0, 5.0, 7.0], [4.0, 2.0, 6.0], [6.0, 1.0, 4.0]], [[5.0, 1.0, 3.0], [7.0, 2.0, 8.0], [2.0, 7.0, 
4.0]], [[8.0, 3.0, 3.0], [3.0, 2.0, 3.0], [8.0, 8.0, 3.0]]]]))
      val=Data(numpy.array([[[[7.0, 4.0, 1.0]], [[6.0, 1.0, 4.0]], [[6.0, 4.0, 7.0]]], [[[7.0, 4.0, 4.0]], [[5.0, 5.0, 4.0]], 
[[1.0, 8.0, 5.0]]]]),self.functionspace)
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 7.0], [1.0, 7.0, 4.0], [5.0, 5.0, 4.0]], [[8.0, 1.0, 7.0], [4.0, 6.0, 1.0], [2.0, 
8.0, 3.0]], [[4.0, 2.0, 8.0], [3.0, 3.0, 4.0], [5.0, 2.0, 3.0]]], [[[4.0, 1.0, 7.0], [4.0, 2.0, 2.0], [6.0, 1.0, 8.0]], [[4.0, 
5.0, 8.0], [4.0, 6.0, 6.0], [3.0, 6.0, 6.0]], [[1.0, 5.0, 7.0], [4.0, 6.0, 2.0], [7.0, 2.0, 8.0]]], [[[2.0, 4.0, 2.0], [4.0, 
3.0, 6.0], [7.0, 4.0, 1.0]], [[1.0, 8.0, 7.0], [1.0, 5.0, 8.0], [6.0, 1.0, 4.0]], [[5.0, 5.0, 4.0], [8.0, 3.0, 7.0], [6.0, 4.0, 
7.0]]], [[[4.0, 3.0, 7.0], [1.0, 3.0, 7.0], [7.0, 4.0, 4.0]], [[8.0, 2.0, 6.0], [4.0, 3.0, 8.0], [5.0, 5.0, 4.0]], [[5.0, 6.0, 
2.0], [4.0, 4.0, 8.0], [1.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 1.0], [5.0, 8.0, 2.0], [7.0, 3.0, 3.0]], 
[[2.0, 2.0, 4.0], [7.0, 2.0, 4.0], [5.0, 1.0, 8.0]], [[2.0, 8.0, 3.0], [3.0, 8.0, 1.0], [5.0, 8.0, 3.0]]], [[[8.0, 3.0, 4.0], 
[4.0, 6.0, 6.0], [5.0, 2.0, 5.0]], [[1.0, 5.0, 4.0], [4.0, 2.0, 6.0], [1.0, 5.0, 2.0]], [[7.0, 4.0, 3.0], [2.0, 6.0, 4.0], 
[6.0, 2.0, 2.0]]], [[[3.0, 6.0, 8.0], [8.0, 3.0, 3.0], [7.0, 4.0, 1.0]], [[7.0, 3.0, 5.0], [8.0, 6.0, 8.0], [6.0, 1.0, 4.0]], 
[[2.0, 6.0, 8.0], [1.0, 2.0, 5.0], [6.0, 4.0, 7.0]]], [[[2.0, 5.0, 7.0], [4.0, 2.0, 6.0], [7.0, 4.0, 4.0]], [[5.0, 1.0, 3.0], 
[7.0, 2.0, 8.0], [5.0, 5.0, 4.0]], [[8.0, 3.0, 3.0], [3.0, 2.0, 3.0], [1.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LCLN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 8.0], [5.0, 2.0, 2.0], [8.0, 7.0, 6.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 1.0], [3.0, 4.0, 
6.0]], [[8.0, 6.0, 7.0], [2.0, 5.0, 2.0], [1.0, 2.0, 8.0]]], [[[6.0, 2.0, 7.0], [2.0, 2.0, 6.0], [7.0, 1.0, 2.0]], [[6.0, 8.0, 
5.0], [8.0, 8.0, 7.0], [1.0, 5.0, 5.0]], [[8.0, 7.0, 5.0], [2.0, 4.0, 8.0], [1.0, 5.0, 7.0]]], [[[7.0, 6.0, 6.0], [7.0, 5.0, 
3.0], [7.0, 2.0, 8.0]], [[5.0, 5.0, 5.0], [7.0, 2.0, 1.0], [1.0, 7.0, 6.0]], [[7.0, 8.0, 8.0], [3.0, 6.0, 3.0], [4.0, 4.0, 
5.0]]], [[[2.0, 5.0, 1.0], [8.0, 4.0, 1.0], [7.0, 5.0, 2.0]], [[4.0, 4.0, 3.0], [3.0, 4.0, 2.0], [4.0, 4.0, 3.0]], [[3.0, 3.0, 
4.0], [7.0, 7.0, 8.0], [2.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 4.0], [5.0, 8.0, 3.0], [1.0, 6.0, 3.0]], [[1.0, 6.0, 7.0], [5.0, 2.0, 
7.0], [5.0, 4.0, 4.0]], [[3.0, 2.0, 2.0], [8.0, 2.0, 1.0], [3.0, 1.0, 3.0]]], [[[1.0, 8.0, 6.0], [7.0, 3.0, 4.0], [1.0, 7.0, 
2.0]], [[2.0, 4.0, 7.0], [2.0, 2.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [5.0, 5.0, 8.0], [6.0, 3.0, 5.0]]], [[[2.0, 4.0, 
1.0], [2.0, 6.0, 1.0], [1.0, 1.0, 7.0]], [[1.0, 7.0, 2.0], [7.0, 3.0, 5.0], [4.0, 3.0, 4.0]], [[8.0, 8.0, 1.0], [3.0, 7.0, 
3.0], [1.0, 7.0, 2.0]]], [[[8.0, 4.0, 7.0], [5.0, 3.0, 1.0], [1.0, 3.0, 1.0]], [[6.0, 2.0, 3.0], [6.0, 2.0, 4.0], [4.0, 8.0, 
7.0]], [[4.0, 4.0, 4.0], [1.0, 5.0, 3.0], [1.0, 6.0, 1.0]]]]))
      val=Data(numpy.array([[[[7.0, 1.0, 4.0]], [[3.0, 2.0, 8.0]], [[2.0, 1.0, 5.0]]], [[[6.0, 2.0, 6.0]], [[4.0, 3.0, 3.0]], 
[[7.0, 2.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 4.0]], [[5.0, 2.0, 4.0]], [[4.0, 2.0, 5.0]]], [[[3.0, 2.0, 3.0]], [[3.0, 
2.0, 4.0]], [[6.0, 2.0, 3.0]]]]))
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 8.0], [5.0, 2.0, 2.0], [8.0, 7.0, 6.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 1.0], [3.0, 
4.0, 6.0]], [[8.0, 6.0, 7.0], [2.0, 5.0, 2.0], [1.0, 2.0, 8.0]]], [[[6.0, 2.0, 7.0], [2.0, 2.0, 6.0], [7.0, 1.0, 2.0]], [[6.0, 
8.0, 5.0], [8.0, 8.0, 7.0], [1.0, 5.0, 5.0]], [[8.0, 7.0, 5.0], [2.0, 4.0, 8.0], [1.0, 5.0, 7.0]]], [[[7.0, 6.0, 6.0], [7.0, 
5.0, 3.0], [7.0, 1.0, 4.0]], [[5.0, 5.0, 5.0], [7.0, 2.0, 1.0], [3.0, 2.0, 8.0]], [[7.0, 8.0, 8.0], [3.0, 6.0, 3.0], [2.0, 1.0, 
5.0]]], [[[2.0, 5.0, 1.0], [8.0, 4.0, 1.0], [6.0, 2.0, 6.0]], [[4.0, 4.0, 3.0], [3.0, 4.0, 2.0], [4.0, 3.0, 3.0]], [[3.0, 3.0, 
4.0], [7.0, 7.0, 8.0], [7.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 4.0], [5.0, 8.0, 3.0], [1.0, 6.0, 3.0]], 
[[1.0, 6.0, 7.0], [5.0, 2.0, 7.0], [5.0, 4.0, 4.0]], [[3.0, 2.0, 2.0], [8.0, 2.0, 1.0], [3.0, 1.0, 3.0]]], [[[1.0, 8.0, 6.0], 
[7.0, 3.0, 4.0], [1.0, 7.0, 2.0]], [[2.0, 4.0, 7.0], [2.0, 2.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [5.0, 5.0, 8.0], 
[6.0, 3.0, 5.0]]], [[[2.0, 4.0, 1.0], [2.0, 6.0, 1.0], [5.0, 4.0, 4.0]], [[1.0, 7.0, 2.0], [7.0, 3.0, 5.0], [5.0, 2.0, 4.0]], 
[[8.0, 8.0, 1.0], [3.0, 7.0, 3.0], [4.0, 2.0, 5.0]]], [[[8.0, 4.0, 7.0], [5.0, 3.0, 1.0], [3.0, 2.0, 3.0]], [[6.0, 2.0, 3.0], 
[6.0, 2.0, 4.0], [3.0, 2.0, 4.0]], [[4.0, 4.0, 4.0], [1.0, 5.0, 3.0], [6.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LCLN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 2.0], [7.0, 6.0, 5.0], [7.0, 6.0, 6.0]], [[3.0, 8.0, 4.0], [8.0, 4.0, 1.0], [5.0, 2.0, 
1.0]], [[4.0, 8.0, 4.0], [1.0, 7.0, 4.0], [7.0, 6.0, 3.0]]], [[[6.0, 6.0, 6.0], [7.0, 1.0, 7.0], [6.0, 6.0, 3.0]], [[8.0, 6.0, 
3.0], [3.0, 8.0, 6.0], [6.0, 1.0, 1.0]], [[4.0, 7.0, 5.0], [7.0, 3.0, 3.0], [6.0, 6.0, 7.0]]], [[[5.0, 5.0, 3.0], [1.0, 5.0, 
8.0], [4.0, 7.0, 6.0]], [[4.0, 5.0, 2.0], [8.0, 8.0, 1.0], [4.0, 5.0, 7.0]], [[4.0, 6.0, 5.0], [7.0, 4.0, 2.0], [3.0, 8.0, 
4.0]]], [[[1.0, 2.0, 4.0], [1.0, 1.0, 4.0], [1.0, 2.0, 8.0]], [[5.0, 1.0, 6.0], [2.0, 3.0, 7.0], [8.0, 7.0, 8.0]], [[1.0, 1.0, 
1.0], [7.0, 3.0, 4.0], [7.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 2.0], [8.0, 2.0, 8.0], [5.0, 1.0, 7.0]], [[7.0, 2.0, 1.0], [2.0, 1.0, 
4.0], [4.0, 8.0, 6.0]], [[4.0, 1.0, 8.0], [3.0, 3.0, 1.0], [3.0, 7.0, 6.0]]], [[[2.0, 1.0, 5.0], [4.0, 4.0, 7.0], [4.0, 8.0, 
3.0]], [[6.0, 3.0, 1.0], [1.0, 2.0, 7.0], [3.0, 6.0, 3.0]], [[8.0, 6.0, 1.0], [5.0, 7.0, 4.0], [5.0, 4.0, 2.0]]], [[[6.0, 8.0, 
6.0], [7.0, 4.0, 2.0], [2.0, 4.0, 5.0]], [[7.0, 3.0, 6.0], [4.0, 5.0, 5.0], [3.0, 2.0, 4.0]], [[4.0, 4.0, 4.0], [8.0, 3.0, 
8.0], [2.0, 5.0, 4.0]]], [[[4.0, 5.0, 1.0], [2.0, 2.0, 6.0], [8.0, 7.0, 3.0]], [[2.0, 7.0, 2.0], [6.0, 8.0, 6.0], [4.0, 3.0, 
2.0]], [[4.0, 2.0, 7.0], [3.0, 7.0, 4.0], [3.0, 4.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0, 8.0, 1.0]], [[1.0, 1.0, 5.0]], [[1.0, 4.0, 8.0]]], [[[8.0, 3.0, 6.0]], [[2.0, 8.0, 
2.0]], [[5.0, 3.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 7.0, 3.0]], [[6.0, 3.0, 8.0]], [[5.0, 1.0, 4.0]]], [[[8.0, 8.0, 
3.0]], [[4.0, 1.0, 4.0]], [[4.0, 3.0, 8.0]]]])
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 2.0], [7.0, 6.0, 5.0], [7.0, 6.0, 6.0]], [[3.0, 8.0, 4.0], [8.0, 4.0, 1.0], [5.0, 
2.0, 1.0]], [[4.0, 8.0, 4.0], [1.0, 7.0, 4.0], [7.0, 6.0, 3.0]]], [[[6.0, 6.0, 6.0], [7.0, 1.0, 7.0], [6.0, 6.0, 3.0]], [[8.0, 
6.0, 3.0], [3.0, 8.0, 6.0], [6.0, 1.0, 1.0]], [[4.0, 7.0, 5.0], [7.0, 3.0, 3.0], [6.0, 6.0, 7.0]]], [[[5.0, 5.0, 3.0], [1.0, 
5.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 5.0, 2.0], [8.0, 8.0, 1.0], [1.0, 1.0, 5.0]], [[4.0, 6.0, 5.0], [7.0, 4.0, 2.0], [1.0, 4.0, 
8.0]]], [[[1.0, 2.0, 4.0], [1.0, 1.0, 4.0], [8.0, 3.0, 6.0]], [[5.0, 1.0, 6.0], [2.0, 3.0, 7.0], [2.0, 8.0, 2.0]], [[1.0, 1.0, 
1.0], [7.0, 3.0, 4.0], [5.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 2.0], [8.0, 2.0, 8.0], [5.0, 1.0, 7.0]], 
[[7.0, 2.0, 1.0], [2.0, 1.0, 4.0], [4.0, 8.0, 6.0]], [[4.0, 1.0, 8.0], [3.0, 3.0, 1.0], [3.0, 7.0, 6.0]]], [[[2.0, 1.0, 5.0], 
[4.0, 4.0, 7.0], [4.0, 8.0, 3.0]], [[6.0, 3.0, 1.0], [1.0, 2.0, 7.0], [3.0, 6.0, 3.0]], [[8.0, 6.0, 1.0], [5.0, 7.0, 4.0], 
[5.0, 4.0, 2.0]]], [[[6.0, 8.0, 6.0], [7.0, 4.0, 2.0], [4.0, 7.0, 3.0]], [[7.0, 3.0, 6.0], [4.0, 5.0, 5.0], [6.0, 3.0, 8.0]], 
[[4.0, 4.0, 4.0], [8.0, 3.0, 8.0], [5.0, 1.0, 4.0]]], [[[4.0, 5.0, 1.0], [2.0, 2.0, 6.0], [8.0, 8.0, 3.0]], [[2.0, 7.0, 2.0], 
[6.0, 8.0, 6.0], [4.0, 1.0, 4.0]], [[4.0, 2.0, 7.0], [3.0, 7.0, 4.0], [4.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LRLN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 5.0], [7.0, 1.0, 4.0], [4.0, 7.0, 6.0]], [[6.0, 4.0, 3.0], [1.0, 2.0, 5.0], [7.0, 4.0, 
6.0]], [[6.0, 2.0, 6.0], [3.0, 6.0, 4.0], [2.0, 3.0, 7.0]]], [[[2.0, 1.0, 4.0], [4.0, 8.0, 1.0], [3.0, 6.0, 7.0]], [[8.0, 8.0, 
8.0], [2.0, 4.0, 2.0], [3.0, 3.0, 6.0]], [[2.0, 2.0, 4.0], [7.0, 1.0, 2.0], [1.0, 4.0, 4.0]]], [[[3.0, 1.0, 8.0], [6.0, 6.0, 
1.0], [2.0, 8.0, 2.0]], [[2.0, 8.0, 4.0], [5.0, 3.0, 3.0], [1.0, 1.0, 2.0]], [[7.0, 2.0, 8.0], [1.0, 3.0, 1.0], [8.0, 8.0, 
8.0]]], [[[6.0, 3.0, 7.0], [5.0, 2.0, 1.0], [4.0, 1.0, 1.0]], [[4.0, 8.0, 6.0], [4.0, 7.0, 7.0], [2.0, 4.0, 7.0]], [[4.0, 2.0, 
2.0], [1.0, 6.0, 7.0], [6.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 1.0], [6.0, 4.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 7.0, 4.0], [2.0, 6.0, 
3.0], [3.0, 7.0, 2.0]], [[5.0, 4.0, 8.0], [3.0, 6.0, 5.0], [7.0, 7.0, 1.0]]], [[[8.0, 5.0, 8.0], [5.0, 1.0, 3.0], [2.0, 2.0, 
5.0]], [[6.0, 7.0, 8.0], [3.0, 5.0, 5.0], [5.0, 2.0, 6.0]], [[4.0, 2.0, 5.0], [1.0, 4.0, 8.0], [6.0, 1.0, 4.0]]], [[[8.0, 7.0, 
2.0], [5.0, 1.0, 2.0], [4.0, 1.0, 2.0]], [[4.0, 4.0, 3.0], [1.0, 4.0, 5.0], [1.0, 6.0, 7.0]], [[3.0, 1.0, 8.0], [2.0, 7.0, 
7.0], [3.0, 7.0, 3.0]]], [[[2.0, 1.0, 1.0], [4.0, 7.0, 8.0], [7.0, 6.0, 1.0]], [[1.0, 2.0, 4.0], [5.0, 3.0, 5.0], [4.0, 4.0, 
8.0]], [[7.0, 3.0, 6.0], [8.0, 8.0, 1.0], [5.0, 7.0, 8.0]]]]))
      val=numpy.array([[[[5.0, 5.0, 8.0]]], [[[2.0, 4.0, 4.0]]]])
      arg[2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 5.0], [7.0, 1.0, 4.0], [4.0, 7.0, 6.0]], [[6.0, 4.0, 3.0], [1.0, 2.0, 5.0], [7.0, 
4.0, 6.0]], [[6.0, 2.0, 6.0], [3.0, 6.0, 4.0], [2.0, 3.0, 7.0]]], [[[2.0, 1.0, 4.0], [4.0, 8.0, 1.0], [3.0, 6.0, 7.0]], [[8.0, 
8.0, 8.0], [2.0, 4.0, 2.0], [3.0, 3.0, 6.0]], [[2.0, 2.0, 4.0], [7.0, 1.0, 2.0], [1.0, 4.0, 4.0]]], [[[3.0, 1.0, 8.0], [6.0, 
6.0, 1.0], [2.0, 8.0, 2.0]], [[2.0, 8.0, 4.0], [5.0, 3.0, 3.0], [5.0, 5.0, 8.0]], [[7.0, 2.0, 8.0], [1.0, 3.0, 1.0], [8.0, 8.0, 
8.0]]], [[[6.0, 3.0, 7.0], [5.0, 2.0, 1.0], [4.0, 1.0, 1.0]], [[4.0, 8.0, 6.0], [4.0, 7.0, 7.0], [2.0, 4.0, 4.0]], [[4.0, 2.0, 
2.0], [1.0, 6.0, 7.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 1.0], [6.0, 4.0, 5.0], [5.0, 6.0, 6.0]], 
[[5.0, 7.0, 4.0], [2.0, 6.0, 3.0], [3.0, 7.0, 2.0]], [[5.0, 4.0, 8.0], [3.0, 6.0, 5.0], [7.0, 7.0, 1.0]]], [[[8.0, 5.0, 8.0], 
[5.0, 1.0, 3.0], [2.0, 2.0, 5.0]], [[6.0, 7.0, 8.0], [3.0, 5.0, 5.0], [5.0, 2.0, 6.0]], [[4.0, 2.0, 5.0], [1.0, 4.0, 8.0], 
[6.0, 1.0, 4.0]]], [[[8.0, 7.0, 2.0], [5.0, 1.0, 2.0], [4.0, 1.0, 2.0]], [[4.0, 4.0, 3.0], [1.0, 4.0, 5.0], [5.0, 5.0, 8.0]], 
[[3.0, 1.0, 8.0], [2.0, 7.0, 7.0], [3.0, 7.0, 3.0]]], [[[2.0, 1.0, 1.0], [4.0, 7.0, 8.0], [7.0, 6.0, 1.0]], [[1.0, 2.0, 4.0], 
[5.0, 3.0, 5.0], [2.0, 4.0, 4.0]], [[7.0, 3.0, 6.0], [8.0, 8.0, 1.0], [5.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LRLN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 6.0], [1.0, 8.0, 2.0], [5.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], [1.0, 5.0, 7.0], [6.0, 1.0, 
1.0]], [[1.0, 3.0, 8.0], [1.0, 6.0, 3.0], [3.0, 1.0, 8.0]]], [[[1.0, 1.0, 5.0], [8.0, 8.0, 5.0], [1.0, 6.0, 3.0]], [[4.0, 1.0, 
6.0], [6.0, 6.0, 3.0], [2.0, 3.0, 7.0]], [[2.0, 2.0, 1.0], [8.0, 5.0, 1.0], [8.0, 5.0, 2.0]]], [[[3.0, 8.0, 7.0], [2.0, 1.0, 
3.0], [5.0, 6.0, 7.0]], [[7.0, 7.0, 7.0], [5.0, 4.0, 2.0], [6.0, 3.0, 3.0]], [[2.0, 2.0, 4.0], [2.0, 1.0, 7.0], [1.0, 6.0, 
8.0]]], [[[8.0, 5.0, 2.0], [6.0, 3.0, 1.0], [8.0, 4.0, 7.0]], [[3.0, 6.0, 6.0], [1.0, 1.0, 8.0], [3.0, 3.0, 8.0]], [[2.0, 6.0, 
8.0], [5.0, 3.0, 7.0], [3.0, 4.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 3.0], [4.0, 3.0, 6.0], [5.0, 6.0, 5.0]], [[7.0, 3.0, 1.0], [4.0, 6.0, 
8.0], [3.0, 1.0, 1.0]], [[8.0, 7.0, 6.0], [8.0, 4.0, 2.0], [6.0, 2.0, 1.0]]], [[[4.0, 2.0, 1.0], [8.0, 8.0, 4.0], [6.0, 3.0, 
4.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 3.0], [1.0, 2.0, 6.0]], [[7.0, 4.0, 3.0], [1.0, 4.0, 4.0], [7.0, 2.0, 2.0]]], [[[8.0, 8.0, 
7.0], [7.0, 1.0, 1.0], [5.0, 8.0, 3.0]], [[5.0, 6.0, 4.0], [4.0, 8.0, 7.0], [3.0, 6.0, 3.0]], [[3.0, 7.0, 7.0], [2.0, 5.0, 
6.0], [8.0, 1.0, 6.0]]], [[[8.0, 1.0, 8.0], [6.0, 8.0, 5.0], [8.0, 4.0, 2.0]], [[4.0, 5.0, 4.0], [7.0, 3.0, 4.0], [8.0, 4.0, 
5.0]], [[7.0, 7.0, 4.0], [8.0, 4.0, 6.0], [7.0, 1.0, 4.0]]]]))
      val=Data(numpy.array([[[[4.0, 1.0, 2.0]]], [[[7.0, 4.0, 3.0]]]]),self.functionspace)
      arg[2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 6.0], [1.0, 8.0, 2.0], [5.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], [1.0, 5.0, 7.0], [6.0, 
1.0, 1.0]], [[1.0, 3.0, 8.0], [1.0, 6.0, 3.0], [3.0, 1.0, 8.0]]], [[[1.0, 1.0, 5.0], [8.0, 8.0, 5.0], [1.0, 6.0, 3.0]], [[4.0, 
1.0, 6.0], [6.0, 6.0, 3.0], [2.0, 3.0, 7.0]], [[2.0, 2.0, 1.0], [8.0, 5.0, 1.0], [8.0, 5.0, 2.0]]], [[[3.0, 8.0, 7.0], [2.0, 
1.0, 3.0], [5.0, 6.0, 7.0]], [[7.0, 7.0, 7.0], [5.0, 4.0, 2.0], [4.0, 1.0, 2.0]], [[2.0, 2.0, 4.0], [2.0, 1.0, 7.0], [1.0, 6.0, 
8.0]]], [[[8.0, 5.0, 2.0], [6.0, 3.0, 1.0], [8.0, 4.0, 7.0]], [[3.0, 6.0, 6.0], [1.0, 1.0, 8.0], [7.0, 4.0, 3.0]], [[2.0, 6.0, 
8.0], [5.0, 3.0, 7.0], [3.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 3.0], [4.0, 3.0, 6.0], [5.0, 6.0, 5.0]], 
[[7.0, 3.0, 1.0], [4.0, 6.0, 8.0], [3.0, 1.0, 1.0]], [[8.0, 7.0, 6.0], [8.0, 4.0, 2.0], [6.0, 2.0, 1.0]]], [[[4.0, 2.0, 1.0], 
[8.0, 8.0, 4.0], [6.0, 3.0, 4.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 3.0], [1.0, 2.0, 6.0]], [[7.0, 4.0, 3.0], [1.0, 4.0, 4.0], 
[7.0, 2.0, 2.0]]], [[[8.0, 8.0, 7.0], [7.0, 1.0, 1.0], [5.0, 8.0, 3.0]], [[5.0, 6.0, 4.0], [4.0, 8.0, 7.0], [4.0, 1.0, 2.0]], 
[[3.0, 7.0, 7.0], [2.0, 5.0, 6.0], [8.0, 1.0, 6.0]]], [[[8.0, 1.0, 8.0], [6.0, 8.0, 5.0], [8.0, 4.0, 2.0]], [[4.0, 5.0, 4.0], 
[7.0, 3.0, 4.0], [7.0, 4.0, 3.0]], [[7.0, 7.0, 4.0], [8.0, 4.0, 6.0], [7.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LRLN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 3.0], [7.0, 4.0, 7.0], [5.0, 5.0, 8.0]], [[3.0, 8.0, 6.0], [7.0, 5.0, 3.0], [1.0, 7.0, 
2.0]], [[7.0, 2.0, 8.0], [5.0, 2.0, 3.0], [5.0, 1.0, 1.0]]], [[[1.0, 7.0, 3.0], [4.0, 4.0, 1.0], [7.0, 1.0, 2.0]], [[1.0, 5.0, 
7.0], [4.0, 5.0, 7.0], [1.0, 2.0, 2.0]], [[2.0, 6.0, 7.0], [7.0, 2.0, 8.0], [3.0, 8.0, 7.0]]], [[[1.0, 8.0, 5.0], [4.0, 3.0, 
4.0], [4.0, 1.0, 7.0]], [[1.0, 4.0, 4.0], [8.0, 2.0, 3.0], [4.0, 3.0, 2.0]], [[5.0, 7.0, 3.0], [7.0, 3.0, 1.0], [8.0, 4.0, 
3.0]]], [[[8.0, 1.0, 1.0], [6.0, 3.0, 3.0], [6.0, 2.0, 7.0]], [[6.0, 4.0, 1.0], [6.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[8.0, 7.0, 
1.0], [1.0, 1.0, 8.0], [8.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 7.0], [2.0, 8.0, 2.0], [7.0, 7.0, 1.0]], [[3.0, 8.0, 4.0], [4.0, 8.0, 
7.0], [2.0, 4.0, 5.0]], [[5.0, 7.0, 5.0], [7.0, 5.0, 6.0], [5.0, 6.0, 3.0]]], [[[7.0, 3.0, 1.0], [4.0, 2.0, 3.0], [8.0, 7.0, 
6.0]], [[3.0, 6.0, 2.0], [2.0, 2.0, 5.0], [6.0, 5.0, 5.0]], [[1.0, 1.0, 3.0], [5.0, 8.0, 5.0], [4.0, 6.0, 1.0]]], [[[4.0, 5.0, 
1.0], [8.0, 1.0, 3.0], [2.0, 5.0, 7.0]], [[8.0, 4.0, 3.0], [3.0, 4.0, 4.0], [8.0, 4.0, 6.0]], [[5.0, 1.0, 2.0], [6.0, 5.0, 
2.0], [5.0, 2.0, 8.0]]], [[[6.0, 6.0, 7.0], [1.0, 8.0, 7.0], [2.0, 6.0, 8.0]], [[3.0, 2.0, 5.0], [8.0, 7.0, 2.0], [3.0, 7.0, 
7.0]], [[3.0, 5.0, 2.0], [5.0, 6.0, 3.0], [4.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[[8.0, 1.0, 7.0]]], [[[1.0, 7.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 3.0]]], [[[4.0, 3.0, 5.0]]]]))
      arg[2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 3.0], [7.0, 4.0, 7.0], [5.0, 5.0, 8.0]], [[3.0, 8.0, 6.0], [7.0, 5.0, 3.0], [1.0, 
7.0, 2.0]], [[7.0, 2.0, 8.0], [5.0, 2.0, 3.0], [5.0, 1.0, 1.0]]], [[[1.0, 7.0, 3.0], [4.0, 4.0, 1.0], [7.0, 1.0, 2.0]], [[1.0, 
5.0, 7.0], [4.0, 5.0, 7.0], [1.0, 2.0, 2.0]], [[2.0, 6.0, 7.0], [7.0, 2.0, 8.0], [3.0, 8.0, 7.0]]], [[[1.0, 8.0, 5.0], [4.0, 
3.0, 4.0], [4.0, 1.0, 7.0]], [[1.0, 4.0, 4.0], [8.0, 2.0, 3.0], [8.0, 1.0, 7.0]], [[5.0, 7.0, 3.0], [7.0, 3.0, 1.0], [8.0, 4.0, 
3.0]]], [[[8.0, 1.0, 1.0], [6.0, 3.0, 3.0], [6.0, 2.0, 7.0]], [[6.0, 4.0, 1.0], [6.0, 4.0, 4.0], [1.0, 7.0, 7.0]], [[8.0, 7.0, 
1.0], [1.0, 1.0, 8.0], [8.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 7.0], [2.0, 8.0, 2.0], [7.0, 7.0, 1.0]], 
[[3.0, 8.0, 4.0], [4.0, 8.0, 7.0], [2.0, 4.0, 5.0]], [[5.0, 7.0, 5.0], [7.0, 5.0, 6.0], [5.0, 6.0, 3.0]]], [[[7.0, 3.0, 1.0], 
[4.0, 2.0, 3.0], [8.0, 7.0, 6.0]], [[3.0, 6.0, 2.0], [2.0, 2.0, 5.0], [6.0, 5.0, 5.0]], [[1.0, 1.0, 3.0], [5.0, 8.0, 5.0], 
[4.0, 6.0, 1.0]]], [[[4.0, 5.0, 1.0], [8.0, 1.0, 3.0], [2.0, 5.0, 7.0]], [[8.0, 4.0, 3.0], [3.0, 4.0, 4.0], [3.0, 5.0, 3.0]], 
[[5.0, 1.0, 2.0], [6.0, 5.0, 2.0], [5.0, 2.0, 8.0]]], [[[6.0, 6.0, 7.0], [1.0, 8.0, 7.0], [2.0, 6.0, 8.0]], [[3.0, 2.0, 5.0], 
[8.0, 7.0, 2.0], [4.0, 3.0, 5.0]], [[3.0, 5.0, 2.0], [5.0, 6.0, 3.0], [4.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LRLN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 6.0], [5.0, 8.0, 1.0], [3.0, 1.0, 7.0]], [[2.0, 8.0, 3.0], [2.0, 3.0, 4.0], [1.0, 5.0, 
6.0]], [[2.0, 5.0, 3.0], [3.0, 7.0, 3.0], [7.0, 5.0, 7.0]]], [[[5.0, 6.0, 3.0], [1.0, 7.0, 1.0], [8.0, 8.0, 1.0]], [[2.0, 7.0, 
8.0], [4.0, 7.0, 3.0], [5.0, 8.0, 7.0]], [[8.0, 7.0, 1.0], [6.0, 4.0, 3.0], [5.0, 8.0, 4.0]]], [[[5.0, 3.0, 7.0], [5.0, 5.0, 
3.0], [1.0, 8.0, 2.0]], [[8.0, 4.0, 8.0], [2.0, 8.0, 3.0], [2.0, 5.0, 7.0]], [[8.0, 3.0, 4.0], [3.0, 2.0, 6.0], [5.0, 3.0, 
2.0]]], [[[2.0, 8.0, 1.0], [7.0, 3.0, 5.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 3.0], [1.0, 7.0, 3.0], [5.0, 7.0, 3.0]], [[4.0, 2.0, 
8.0], [2.0, 6.0, 6.0], [6.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 8.0], [8.0, 4.0, 2.0], [8.0, 4.0, 7.0]], [[7.0, 7.0, 7.0], [4.0, 7.0, 
1.0], [8.0, 2.0, 7.0]], [[3.0, 6.0, 5.0], [4.0, 8.0, 4.0], [5.0, 4.0, 1.0]]], [[[4.0, 2.0, 5.0], [6.0, 2.0, 2.0], [2.0, 4.0, 
1.0]], [[5.0, 2.0, 4.0], [3.0, 1.0, 6.0], [5.0, 5.0, 7.0]], [[8.0, 3.0, 1.0], [1.0, 1.0, 6.0], [2.0, 8.0, 7.0]]], [[[6.0, 3.0, 
3.0], [4.0, 6.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [4.0, 1.0, 6.0], [1.0, 4.0, 4.0]], [[8.0, 1.0, 4.0], [8.0, 4.0, 
2.0], [8.0, 2.0, 6.0]]], [[[6.0, 4.0, 5.0], [6.0, 8.0, 6.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 2.0], [5.0, 7.0, 8.0], [3.0, 6.0, 
2.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 7.0], [5.0, 1.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0, 4.0, 1.0]]], [[[7.0, 1.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 5.0]]], 
[[[1.0, 4.0, 6.0]]]])
      arg[2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 6.0], [5.0, 8.0, 1.0], [3.0, 1.0, 7.0]], [[2.0, 8.0, 3.0], [2.0, 3.0, 4.0], [1.0, 
5.0, 6.0]], [[2.0, 5.0, 3.0], [3.0, 7.0, 3.0], [7.0, 5.0, 7.0]]], [[[5.0, 6.0, 3.0], [1.0, 7.0, 1.0], [8.0, 8.0, 1.0]], [[2.0, 
7.0, 8.0], [4.0, 7.0, 3.0], [5.0, 8.0, 7.0]], [[8.0, 7.0, 1.0], [6.0, 4.0, 3.0], [5.0, 8.0, 4.0]]], [[[5.0, 3.0, 7.0], [5.0, 
5.0, 3.0], [1.0, 8.0, 2.0]], [[8.0, 4.0, 8.0], [2.0, 8.0, 3.0], [4.0, 4.0, 1.0]], [[8.0, 3.0, 4.0], [3.0, 2.0, 6.0], [5.0, 3.0, 
2.0]]], [[[2.0, 8.0, 1.0], [7.0, 3.0, 5.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 3.0], [1.0, 7.0, 3.0], [7.0, 1.0, 8.0]], [[4.0, 2.0, 
8.0], [2.0, 6.0, 6.0], [6.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 8.0], [8.0, 4.0, 2.0], [8.0, 4.0, 7.0]], 
[[7.0, 7.0, 7.0], [4.0, 7.0, 1.0], [8.0, 2.0, 7.0]], [[3.0, 6.0, 5.0], [4.0, 8.0, 4.0], [5.0, 4.0, 1.0]]], [[[4.0, 2.0, 5.0], 
[6.0, 2.0, 2.0], [2.0, 4.0, 1.0]], [[5.0, 2.0, 4.0], [3.0, 1.0, 6.0], [5.0, 5.0, 7.0]], [[8.0, 3.0, 1.0], [1.0, 1.0, 6.0], 
[2.0, 8.0, 7.0]]], [[[6.0, 3.0, 3.0], [4.0, 6.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [4.0, 1.0, 6.0], [5.0, 4.0, 5.0]], 
[[8.0, 1.0, 4.0], [8.0, 4.0, 2.0], [8.0, 2.0, 6.0]]], [[[6.0, 4.0, 5.0], [6.0, 8.0, 6.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 2.0], 
[5.0, 7.0, 8.0], [1.0, 4.0, 6.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 7.0], [5.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RULN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 3.0], [5.0, 5.0, 3.0], [5.0, 5.0, 8.0]], [[8.0, 6.0, 5.0], [2.0, 1.0, 1.0], [4.0, 7.0, 
8.0]], [[1.0, 4.0, 3.0], [8.0, 7.0, 6.0], [6.0, 1.0, 4.0]]], [[[1.0, 1.0, 7.0], [2.0, 5.0, 1.0], [1.0, 4.0, 4.0]], [[4.0, 1.0, 
7.0], [1.0, 1.0, 6.0], [1.0, 8.0, 2.0]], [[4.0, 6.0, 7.0], [6.0, 6.0, 7.0], [3.0, 4.0, 2.0]]], [[[5.0, 3.0, 5.0], [4.0, 6.0, 
2.0], [4.0, 3.0, 6.0]], [[3.0, 6.0, 4.0], [6.0, 6.0, 4.0], [8.0, 6.0, 3.0]], [[6.0, 2.0, 2.0], [5.0, 2.0, 7.0], [2.0, 2.0, 
2.0]]], [[[2.0, 7.0, 5.0], [5.0, 7.0, 4.0], [2.0, 3.0, 5.0]], [[5.0, 6.0, 1.0], [5.0, 5.0, 2.0], [8.0, 5.0, 7.0]], [[5.0, 6.0, 
5.0], [4.0, 7.0, 2.0], [5.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 8.0], [8.0, 1.0, 5.0], [1.0, 7.0, 4.0]], [[5.0, 1.0, 3.0], [3.0, 8.0, 
8.0], [7.0, 8.0, 4.0]], [[7.0, 1.0, 2.0], [2.0, 4.0, 1.0], [1.0, 6.0, 5.0]]], [[[7.0, 8.0, 8.0], [1.0, 6.0, 1.0], [4.0, 7.0, 
7.0]], [[2.0, 3.0, 4.0], [6.0, 5.0, 8.0], [2.0, 2.0, 7.0]], [[2.0, 8.0, 7.0], [1.0, 1.0, 3.0], [1.0, 2.0, 2.0]]], [[[4.0, 6.0, 
7.0], [7.0, 1.0, 4.0], [5.0, 3.0, 7.0]], [[5.0, 5.0, 2.0], [7.0, 6.0, 7.0], [3.0, 7.0, 4.0]], [[1.0, 7.0, 6.0], [3.0, 4.0, 
8.0], [1.0, 7.0, 1.0]]], [[[7.0, 7.0, 4.0], [2.0, 4.0, 8.0], [1.0, 7.0, 1.0]], [[8.0, 2.0, 6.0], [1.0, 1.0, 5.0], [1.0, 4.0, 
7.0]], [[4.0, 2.0, 1.0], [4.0, 2.0, 4.0], [3.0, 3.0, 3.0]]]]))
      val=numpy.array([[[[5.0, 2.0, 2.0]], [[8.0, 4.0, 8.0]]], [[[6.0, 3.0, 8.0]], [[7.0, 7.0, 4.0]]]])
      arg[1:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 3.0], [5.0, 5.0, 3.0], [5.0, 5.0, 8.0]], [[8.0, 6.0, 5.0], [2.0, 1.0, 1.0], [4.0, 
7.0, 8.0]], [[1.0, 4.0, 3.0], [8.0, 7.0, 6.0], [6.0, 1.0, 4.0]]], [[[1.0, 1.0, 7.0], [2.0, 5.0, 1.0], [5.0, 2.0, 2.0]], [[4.0, 
1.0, 7.0], [1.0, 1.0, 6.0], [8.0, 4.0, 8.0]], [[4.0, 6.0, 7.0], [6.0, 6.0, 7.0], [3.0, 4.0, 2.0]]], [[[5.0, 3.0, 5.0], [4.0, 
6.0, 2.0], [6.0, 3.0, 8.0]], [[3.0, 6.0, 4.0], [6.0, 6.0, 4.0], [7.0, 7.0, 4.0]], [[6.0, 2.0, 2.0], [5.0, 2.0, 7.0], [2.0, 2.0, 
2.0]]], [[[2.0, 7.0, 5.0], [5.0, 7.0, 4.0], [2.0, 3.0, 5.0]], [[5.0, 6.0, 1.0], [5.0, 5.0, 2.0], [8.0, 5.0, 7.0]], [[5.0, 6.0, 
5.0], [4.0, 7.0, 2.0], [5.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 8.0], [8.0, 1.0, 5.0], [1.0, 7.0, 4.0]], 
[[5.0, 1.0, 3.0], [3.0, 8.0, 8.0], [7.0, 8.0, 4.0]], [[7.0, 1.0, 2.0], [2.0, 4.0, 1.0], [1.0, 6.0, 5.0]]], [[[7.0, 8.0, 8.0], 
[1.0, 6.0, 1.0], [5.0, 2.0, 2.0]], [[2.0, 3.0, 4.0], [6.0, 5.0, 8.0], [8.0, 4.0, 8.0]], [[2.0, 8.0, 7.0], [1.0, 1.0, 3.0], 
[1.0, 2.0, 2.0]]], [[[4.0, 6.0, 7.0], [7.0, 1.0, 4.0], [6.0, 3.0, 8.0]], [[5.0, 5.0, 2.0], [7.0, 6.0, 7.0], [7.0, 7.0, 4.0]], 
[[1.0, 7.0, 6.0], [3.0, 4.0, 8.0], [1.0, 7.0, 1.0]]], [[[7.0, 7.0, 4.0], [2.0, 4.0, 8.0], [1.0, 7.0, 1.0]], [[8.0, 2.0, 6.0], 
[1.0, 1.0, 5.0], [1.0, 4.0, 7.0]], [[4.0, 2.0, 1.0], [4.0, 2.0, 4.0], [3.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RULN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 6.0], [8.0, 6.0, 4.0], [5.0, 6.0, 6.0]], [[1.0, 8.0, 8.0], [5.0, 8.0, 4.0], [2.0, 6.0, 
7.0]], [[5.0, 6.0, 7.0], [6.0, 3.0, 7.0], [7.0, 7.0, 6.0]]], [[[3.0, 6.0, 1.0], [7.0, 6.0, 8.0], [4.0, 3.0, 5.0]], [[3.0, 6.0, 
2.0], [4.0, 8.0, 6.0], [7.0, 2.0, 5.0]], [[5.0, 3.0, 1.0], [6.0, 7.0, 5.0], [3.0, 6.0, 4.0]]], [[[2.0, 4.0, 7.0], [6.0, 3.0, 
2.0], [1.0, 5.0, 8.0]], [[7.0, 8.0, 2.0], [7.0, 5.0, 3.0], [5.0, 6.0, 7.0]], [[4.0, 7.0, 2.0], [7.0, 1.0, 4.0], [7.0, 2.0, 
2.0]]], [[[1.0, 4.0, 5.0], [8.0, 1.0, 7.0], [3.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [3.0, 4.0, 7.0], [3.0, 2.0, 8.0]], [[4.0, 5.0, 
1.0], [6.0, 5.0, 2.0], [7.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 6.0], [4.0, 2.0, 5.0], [1.0, 8.0, 1.0]], [[1.0, 3.0, 3.0], [7.0, 1.0, 
6.0], [7.0, 8.0, 3.0]], [[5.0, 4.0, 3.0], [4.0, 2.0, 7.0], [5.0, 7.0, 1.0]]], [[[2.0, 8.0, 3.0], [2.0, 6.0, 5.0], [7.0, 2.0, 
7.0]], [[7.0, 2.0, 8.0], [6.0, 8.0, 8.0], [7.0, 4.0, 4.0]], [[8.0, 2.0, 7.0], [1.0, 8.0, 3.0], [5.0, 7.0, 4.0]]], [[[6.0, 7.0, 
1.0], [2.0, 7.0, 4.0], [8.0, 3.0, 2.0]], [[6.0, 6.0, 3.0], [4.0, 7.0, 6.0], [5.0, 7.0, 5.0]], [[4.0, 1.0, 1.0], [8.0, 4.0, 
6.0], [8.0, 6.0, 6.0]]], [[[5.0, 7.0, 3.0], [1.0, 7.0, 5.0], [1.0, 2.0, 7.0]], [[4.0, 8.0, 1.0], [3.0, 4.0, 7.0], [5.0, 5.0, 
6.0]], [[4.0, 1.0, 5.0], [3.0, 5.0, 1.0], [5.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[[3.0, 6.0, 7.0]], [[6.0, 7.0, 8.0]]], [[[3.0, 8.0, 4.0]], [[4.0, 7.0, 2.0]]]]),self.functionspace)
      arg[1:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 6.0], [8.0, 6.0, 4.0], [5.0, 6.0, 6.0]], [[1.0, 8.0, 8.0], [5.0, 8.0, 4.0], [2.0, 
6.0, 7.0]], [[5.0, 6.0, 7.0], [6.0, 3.0, 7.0], [7.0, 7.0, 6.0]]], [[[3.0, 6.0, 1.0], [7.0, 6.0, 8.0], [3.0, 6.0, 7.0]], [[3.0, 
6.0, 2.0], [4.0, 8.0, 6.0], [6.0, 7.0, 8.0]], [[5.0, 3.0, 1.0], [6.0, 7.0, 5.0], [3.0, 6.0, 4.0]]], [[[2.0, 4.0, 7.0], [6.0, 
3.0, 2.0], [3.0, 8.0, 4.0]], [[7.0, 8.0, 2.0], [7.0, 5.0, 3.0], [4.0, 7.0, 2.0]], [[4.0, 7.0, 2.0], [7.0, 1.0, 4.0], [7.0, 2.0, 
2.0]]], [[[1.0, 4.0, 5.0], [8.0, 1.0, 7.0], [3.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [3.0, 4.0, 7.0], [3.0, 2.0, 8.0]], [[4.0, 5.0, 
1.0], [6.0, 5.0, 2.0], [7.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 2.0, 5.0], [1.0, 8.0, 1.0]], 
[[1.0, 3.0, 3.0], [7.0, 1.0, 6.0], [7.0, 8.0, 3.0]], [[5.0, 4.0, 3.0], [4.0, 2.0, 7.0], [5.0, 7.0, 1.0]]], [[[2.0, 8.0, 3.0], 
[2.0, 6.0, 5.0], [3.0, 6.0, 7.0]], [[7.0, 2.0, 8.0], [6.0, 8.0, 8.0], [6.0, 7.0, 8.0]], [[8.0, 2.0, 7.0], [1.0, 8.0, 3.0], 
[5.0, 7.0, 4.0]]], [[[6.0, 7.0, 1.0], [2.0, 7.0, 4.0], [3.0, 8.0, 4.0]], [[6.0, 6.0, 3.0], [4.0, 7.0, 6.0], [4.0, 7.0, 2.0]], 
[[4.0, 1.0, 1.0], [8.0, 4.0, 6.0], [8.0, 6.0, 6.0]]], [[[5.0, 7.0, 3.0], [1.0, 7.0, 5.0], [1.0, 2.0, 7.0]], [[4.0, 8.0, 1.0], 
[3.0, 4.0, 7.0], [5.0, 5.0, 6.0]], [[4.0, 1.0, 5.0], [3.0, 5.0, 1.0], [5.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RULN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 5.0], [1.0, 7.0, 7.0], [5.0, 3.0, 6.0]], [[3.0, 8.0, 7.0], [6.0, 1.0, 1.0], [5.0, 7.0, 
7.0]], [[3.0, 7.0, 2.0], [1.0, 8.0, 3.0], [6.0, 5.0, 2.0]]], [[[4.0, 8.0, 1.0], [1.0, 6.0, 4.0], [7.0, 6.0, 1.0]], [[5.0, 4.0, 
7.0], [4.0, 2.0, 2.0], [4.0, 6.0, 5.0]], [[8.0, 4.0, 1.0], [5.0, 4.0, 2.0], [3.0, 3.0, 5.0]]], [[[2.0, 3.0, 5.0], [2.0, 4.0, 
5.0], [3.0, 1.0, 3.0]], [[5.0, 6.0, 2.0], [4.0, 6.0, 4.0], [1.0, 5.0, 1.0]], [[5.0, 2.0, 6.0], [2.0, 2.0, 7.0], [4.0, 1.0, 
3.0]]], [[[7.0, 7.0, 1.0], [4.0, 4.0, 8.0], [2.0, 5.0, 7.0]], [[6.0, 5.0, 7.0], [1.0, 7.0, 2.0], [4.0, 6.0, 1.0]], [[5.0, 7.0, 
6.0], [7.0, 3.0, 8.0], [8.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 7.0], [5.0, 8.0, 4.0], [6.0, 6.0, 5.0]], [[7.0, 6.0, 7.0], [6.0, 6.0, 
3.0], [3.0, 8.0, 7.0]], [[5.0, 1.0, 6.0], [3.0, 8.0, 2.0], [5.0, 4.0, 5.0]]], [[[6.0, 1.0, 3.0], [5.0, 5.0, 7.0], [5.0, 3.0, 
3.0]], [[5.0, 5.0, 8.0], [7.0, 7.0, 3.0], [5.0, 8.0, 3.0]], [[7.0, 1.0, 1.0], [1.0, 1.0, 6.0], [7.0, 1.0, 8.0]]], [[[3.0, 7.0, 
6.0], [2.0, 2.0, 1.0], [2.0, 7.0, 1.0]], [[5.0, 5.0, 2.0], [5.0, 4.0, 5.0], [5.0, 3.0, 4.0]], [[4.0, 5.0, 5.0], [6.0, 2.0, 
2.0], [3.0, 5.0, 7.0]]], [[[2.0, 1.0, 1.0], [6.0, 8.0, 5.0], [1.0, 8.0, 5.0]], [[8.0, 2.0, 1.0], [6.0, 4.0, 3.0], [4.0, 4.0, 
1.0]], [[2.0, 7.0, 1.0], [7.0, 8.0, 2.0], [2.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[[8.0, 7.0, 5.0]], [[7.0, 8.0, 2.0]]], [[[6.0, 4.0, 5.0]], [[4.0, 6.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 2.0]], [[6.0, 7.0, 5.0]]], [[[2.0, 4.0, 4.0]], [[3.0, 7.0, 8.0]]]]))
      arg[1:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 5.0], [1.0, 7.0, 7.0], [5.0, 3.0, 6.0]], [[3.0, 8.0, 7.0], [6.0, 1.0, 1.0], [5.0, 
7.0, 7.0]], [[3.0, 7.0, 2.0], [1.0, 8.0, 3.0], [6.0, 5.0, 2.0]]], [[[4.0, 8.0, 1.0], [1.0, 6.0, 4.0], [8.0, 7.0, 5.0]], [[5.0, 
4.0, 7.0], [4.0, 2.0, 2.0], [7.0, 8.0, 2.0]], [[8.0, 4.0, 1.0], [5.0, 4.0, 2.0], [3.0, 3.0, 5.0]]], [[[2.0, 3.0, 5.0], [2.0, 
4.0, 5.0], [6.0, 4.0, 5.0]], [[5.0, 6.0, 2.0], [4.0, 6.0, 4.0], [4.0, 6.0, 4.0]], [[5.0, 2.0, 6.0], [2.0, 2.0, 7.0], [4.0, 1.0, 
3.0]]], [[[7.0, 7.0, 1.0], [4.0, 4.0, 8.0], [2.0, 5.0, 7.0]], [[6.0, 5.0, 7.0], [1.0, 7.0, 2.0], [4.0, 6.0, 1.0]], [[5.0, 7.0, 
6.0], [7.0, 3.0, 8.0], [8.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 7.0], [5.0, 8.0, 4.0], [6.0, 6.0, 5.0]], 
[[7.0, 6.0, 7.0], [6.0, 6.0, 3.0], [3.0, 8.0, 7.0]], [[5.0, 1.0, 6.0], [3.0, 8.0, 2.0], [5.0, 4.0, 5.0]]], [[[6.0, 1.0, 3.0], 
[5.0, 5.0, 7.0], [3.0, 1.0, 2.0]], [[5.0, 5.0, 8.0], [7.0, 7.0, 3.0], [6.0, 7.0, 5.0]], [[7.0, 1.0, 1.0], [1.0, 1.0, 6.0], 
[7.0, 1.0, 8.0]]], [[[3.0, 7.0, 6.0], [2.0, 2.0, 1.0], [2.0, 4.0, 4.0]], [[5.0, 5.0, 2.0], [5.0, 4.0, 5.0], [3.0, 7.0, 8.0]], 
[[4.0, 5.0, 5.0], [6.0, 2.0, 2.0], [3.0, 5.0, 7.0]]], [[[2.0, 1.0, 1.0], [6.0, 8.0, 5.0], [1.0, 8.0, 5.0]], [[8.0, 2.0, 1.0], 
[6.0, 4.0, 3.0], [4.0, 4.0, 1.0]], [[2.0, 7.0, 1.0], [7.0, 8.0, 2.0], [2.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RULN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 2.0], [6.0, 8.0, 4.0], [2.0, 6.0, 6.0]], [[1.0, 6.0, 1.0], [7.0, 8.0, 6.0], [4.0, 1.0, 
7.0]], [[7.0, 7.0, 3.0], [5.0, 5.0, 4.0], [3.0, 2.0, 7.0]]], [[[5.0, 7.0, 8.0], [3.0, 3.0, 1.0], [5.0, 7.0, 6.0]], [[5.0, 2.0, 
5.0], [6.0, 8.0, 5.0], [1.0, 4.0, 3.0]], [[6.0, 8.0, 2.0], [3.0, 2.0, 5.0], [5.0, 2.0, 5.0]]], [[[7.0, 7.0, 2.0], [8.0, 1.0, 
3.0], [8.0, 3.0, 6.0]], [[8.0, 6.0, 3.0], [4.0, 6.0, 7.0], [5.0, 5.0, 5.0]], [[8.0, 2.0, 4.0], [6.0, 5.0, 3.0], [4.0, 7.0, 
8.0]]], [[[2.0, 7.0, 7.0], [2.0, 5.0, 4.0], [7.0, 7.0, 8.0]], [[5.0, 5.0, 6.0], [2.0, 4.0, 5.0], [1.0, 4.0, 5.0]], [[1.0, 4.0, 
8.0], [2.0, 5.0, 3.0], [2.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 6.0], [5.0, 8.0, 8.0], [6.0, 7.0, 7.0]], [[2.0, 5.0, 6.0], [3.0, 4.0, 
1.0], [1.0, 7.0, 8.0]], [[6.0, 4.0, 8.0], [7.0, 5.0, 7.0], [1.0, 2.0, 1.0]]], [[[5.0, 7.0, 8.0], [8.0, 1.0, 4.0], [7.0, 2.0, 
1.0]], [[4.0, 5.0, 7.0], [5.0, 5.0, 5.0], [7.0, 1.0, 2.0]], [[1.0, 6.0, 7.0], [7.0, 4.0, 2.0], [3.0, 8.0, 3.0]]], [[[6.0, 4.0, 
7.0], [6.0, 4.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 5.0, 1.0], [7.0, 3.0, 2.0], [3.0, 7.0, 6.0]], [[2.0, 6.0, 6.0], [6.0, 6.0, 
1.0], [3.0, 3.0, 7.0]]], [[[5.0, 4.0, 4.0], [8.0, 5.0, 1.0], [7.0, 8.0, 4.0]], [[6.0, 8.0, 6.0], [8.0, 6.0, 7.0], [4.0, 5.0, 
1.0]], [[6.0, 4.0, 6.0], [5.0, 2.0, 7.0], [7.0, 5.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 1.0, 4.0]], [[6.0, 4.0, 2.0]]], [[[6.0, 2.0, 1.0]], [[7.0, 6.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 2.0, 3.0]], [[8.0, 4.0, 2.0]]], [[[1.0, 6.0, 3.0]], [[6.0, 3.0, 3.0]]]])
      arg[1:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 2.0], [6.0, 8.0, 4.0], [2.0, 6.0, 6.0]], [[1.0, 6.0, 1.0], [7.0, 8.0, 6.0], [4.0, 
1.0, 7.0]], [[7.0, 7.0, 3.0], [5.0, 5.0, 4.0], [3.0, 2.0, 7.0]]], [[[5.0, 7.0, 8.0], [3.0, 3.0, 1.0], [2.0, 1.0, 4.0]], [[5.0, 
2.0, 5.0], [6.0, 8.0, 5.0], [6.0, 4.0, 2.0]], [[6.0, 8.0, 2.0], [3.0, 2.0, 5.0], [5.0, 2.0, 5.0]]], [[[7.0, 7.0, 2.0], [8.0, 
1.0, 3.0], [6.0, 2.0, 1.0]], [[8.0, 6.0, 3.0], [4.0, 6.0, 7.0], [7.0, 6.0, 1.0]], [[8.0, 2.0, 4.0], [6.0, 5.0, 3.0], [4.0, 7.0, 
8.0]]], [[[2.0, 7.0, 7.0], [2.0, 5.0, 4.0], [7.0, 7.0, 8.0]], [[5.0, 5.0, 6.0], [2.0, 4.0, 5.0], [1.0, 4.0, 5.0]], [[1.0, 4.0, 
8.0], [2.0, 5.0, 3.0], [2.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 6.0], [5.0, 8.0, 8.0], [6.0, 7.0, 7.0]], 
[[2.0, 5.0, 6.0], [3.0, 4.0, 1.0], [1.0, 7.0, 8.0]], [[6.0, 4.0, 8.0], [7.0, 5.0, 7.0], [1.0, 2.0, 1.0]]], [[[5.0, 7.0, 8.0], 
[8.0, 1.0, 4.0], [5.0, 2.0, 3.0]], [[4.0, 5.0, 7.0], [5.0, 5.0, 5.0], [8.0, 4.0, 2.0]], [[1.0, 6.0, 7.0], [7.0, 4.0, 2.0], 
[3.0, 8.0, 3.0]]], [[[6.0, 4.0, 7.0], [6.0, 4.0, 5.0], [1.0, 6.0, 3.0]], [[1.0, 5.0, 1.0], [7.0, 3.0, 2.0], [6.0, 3.0, 3.0]], 
[[2.0, 6.0, 6.0], [6.0, 6.0, 1.0], [3.0, 3.0, 7.0]]], [[[5.0, 4.0, 4.0], [8.0, 5.0, 1.0], [7.0, 8.0, 4.0]], [[6.0, 8.0, 6.0], 
[8.0, 6.0, 7.0], [4.0, 5.0, 1.0]], [[6.0, 4.0, 6.0], [5.0, 2.0, 7.0], [7.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LCCN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 1.0], [8.0, 7.0, 6.0], [6.0, 7.0, 3.0]], [[1.0, 4.0, 8.0], [6.0, 6.0, 8.0], [2.0, 1.0, 
2.0]], [[6.0, 5.0, 5.0], [3.0, 6.0, 3.0], [1.0, 7.0, 2.0]]], [[[2.0, 6.0, 1.0], [6.0, 2.0, 7.0], [6.0, 6.0, 4.0]], [[8.0, 7.0, 
5.0], [1.0, 3.0, 7.0], [7.0, 1.0, 5.0]], [[8.0, 8.0, 2.0], [7.0, 1.0, 3.0], [3.0, 2.0, 8.0]]], [[[7.0, 4.0, 5.0], [6.0, 5.0, 
6.0], [1.0, 2.0, 1.0]], [[4.0, 2.0, 8.0], [7.0, 8.0, 1.0], [3.0, 4.0, 6.0]], [[6.0, 7.0, 5.0], [2.0, 6.0, 8.0], [1.0, 2.0, 
3.0]]], [[[1.0, 1.0, 3.0], [2.0, 3.0, 6.0], [5.0, 1.0, 6.0]], [[6.0, 8.0, 1.0], [4.0, 7.0, 6.0], [3.0, 1.0, 2.0]], [[2.0, 3.0, 
7.0], [6.0, 2.0, 5.0], [4.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 5.0], [2.0, 1.0, 8.0], [3.0, 5.0, 1.0]], [[7.0, 3.0, 4.0], [2.0, 5.0, 
8.0], [8.0, 2.0, 2.0]], [[1.0, 6.0, 6.0], [8.0, 1.0, 2.0], [4.0, 8.0, 1.0]]], [[[2.0, 7.0, 1.0], [2.0, 6.0, 1.0], [8.0, 1.0, 
8.0]], [[7.0, 3.0, 1.0], [1.0, 6.0, 6.0], [2.0, 8.0, 6.0]], [[6.0, 5.0, 8.0], [2.0, 5.0, 8.0], [4.0, 5.0, 3.0]]], [[[4.0, 1.0, 
8.0], [8.0, 2.0, 5.0], [5.0, 2.0, 2.0]], [[4.0, 2.0, 1.0], [4.0, 1.0, 4.0], [8.0, 7.0, 6.0]], [[1.0, 4.0, 2.0], [7.0, 8.0, 
1.0], [3.0, 4.0, 4.0]]], [[[3.0, 7.0, 1.0], [4.0, 8.0, 5.0], [5.0, 3.0, 4.0]], [[2.0, 7.0, 4.0], [6.0, 6.0, 8.0], [5.0, 8.0, 
8.0]], [[8.0, 7.0, 8.0], [1.0, 2.0, 4.0], [6.0, 4.0, 3.0]]]]))
      val=numpy.array([[[[4.0, 2.0, 7.0], [1.0, 7.0, 2.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 2.0], [8.0, 4.0, 4.0], [8.0, 5.0, 
3.0]], [[3.0, 4.0, 7.0], [2.0, 4.0, 8.0], [1.0, 7.0, 7.0]]], [[[2.0, 8.0, 8.0], [3.0, 4.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 2.0, 
7.0], [8.0, 3.0, 4.0], [4.0, 7.0, 8.0]], [[5.0, 3.0, 3.0], [5.0, 5.0, 5.0], [1.0, 4.0, 8.0]]]])
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 1.0], [8.0, 7.0, 6.0], [6.0, 7.0, 3.0]], [[1.0, 4.0, 8.0], [6.0, 6.0, 8.0], [2.0, 
1.0, 2.0]], [[6.0, 5.0, 5.0], [3.0, 6.0, 3.0], [1.0, 7.0, 2.0]]], [[[2.0, 6.0, 1.0], [6.0, 2.0, 7.0], [6.0, 6.0, 4.0]], [[8.0, 
7.0, 5.0], [1.0, 3.0, 7.0], [7.0, 1.0, 5.0]], [[8.0, 8.0, 2.0], [7.0, 1.0, 3.0], [3.0, 2.0, 8.0]]], [[[4.0, 2.0, 7.0], [1.0, 
7.0, 2.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 2.0], [8.0, 4.0, 4.0], [8.0, 5.0, 3.0]], [[3.0, 4.0, 7.0], [2.0, 4.0, 8.0], [1.0, 7.0, 
7.0]]], [[[2.0, 8.0, 8.0], [3.0, 4.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 2.0, 7.0], [8.0, 3.0, 4.0], [4.0, 7.0, 8.0]], [[5.0, 3.0, 
3.0], [5.0, 5.0, 5.0], [1.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 5.0], [2.0, 1.0, 8.0], [3.0, 5.0, 1.0]], 
[[7.0, 3.0, 4.0], [2.0, 5.0, 8.0], [8.0, 2.0, 2.0]], [[1.0, 6.0, 6.0], [8.0, 1.0, 2.0], [4.0, 8.0, 1.0]]], [[[2.0, 7.0, 1.0], 
[2.0, 6.0, 1.0], [8.0, 1.0, 8.0]], [[7.0, 3.0, 1.0], [1.0, 6.0, 6.0], [2.0, 8.0, 6.0]], [[6.0, 5.0, 8.0], [2.0, 5.0, 8.0], 
[4.0, 5.0, 3.0]]], [[[4.0, 2.0, 7.0], [1.0, 7.0, 2.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 2.0], [8.0, 4.0, 4.0], [8.0, 5.0, 3.0]], 
[[3.0, 4.0, 7.0], [2.0, 4.0, 8.0], [1.0, 7.0, 7.0]]], [[[2.0, 8.0, 8.0], [3.0, 4.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 2.0, 7.0], 
[8.0, 3.0, 4.0], [4.0, 7.0, 8.0]], [[5.0, 3.0, 3.0], [5.0, 5.0, 5.0], [1.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LCCN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 4.0], [4.0, 2.0, 5.0], [2.0, 7.0, 3.0]], [[2.0, 8.0, 3.0], [2.0, 8.0, 3.0], [8.0, 8.0, 
5.0]], [[8.0, 5.0, 5.0], [6.0, 5.0, 2.0], [6.0, 4.0, 5.0]]], [[[2.0, 8.0, 2.0], [4.0, 6.0, 2.0], [4.0, 2.0, 8.0]], [[4.0, 2.0, 
2.0], [6.0, 1.0, 8.0], [6.0, 5.0, 7.0]], [[7.0, 4.0, 8.0], [1.0, 3.0, 4.0], [6.0, 6.0, 4.0]]], [[[2.0, 6.0, 1.0], [7.0, 8.0, 
5.0], [3.0, 5.0, 8.0]], [[6.0, 1.0, 8.0], [7.0, 7.0, 5.0], [8.0, 3.0, 7.0]], [[5.0, 6.0, 6.0], [5.0, 5.0, 2.0], [4.0, 2.0, 
8.0]]], [[[2.0, 8.0, 3.0], [7.0, 7.0, 5.0], [7.0, 3.0, 4.0]], [[3.0, 6.0, 8.0], [3.0, 6.0, 4.0], [6.0, 4.0, 6.0]], [[1.0, 6.0, 
8.0], [3.0, 3.0, 3.0], [2.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 4.0], [4.0, 1.0, 4.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 3.0], [4.0, 7.0, 
8.0], [1.0, 5.0, 5.0]], [[1.0, 4.0, 8.0], [1.0, 6.0, 4.0], [8.0, 1.0, 4.0]]], [[[5.0, 4.0, 1.0], [5.0, 2.0, 8.0], [5.0, 6.0, 
8.0]], [[5.0, 8.0, 5.0], [8.0, 7.0, 7.0], [7.0, 2.0, 7.0]], [[7.0, 6.0, 7.0], [4.0, 1.0, 8.0], [1.0, 3.0, 5.0]]], [[[5.0, 5.0, 
6.0], [6.0, 1.0, 4.0], [4.0, 6.0, 8.0]], [[1.0, 3.0, 8.0], [4.0, 8.0, 2.0], [4.0, 5.0, 3.0]], [[4.0, 8.0, 6.0], [4.0, 1.0, 
5.0], [3.0, 5.0, 8.0]]], [[[5.0, 8.0, 2.0], [5.0, 2.0, 6.0], [7.0, 4.0, 2.0]], [[2.0, 7.0, 7.0], [3.0, 8.0, 2.0], [2.0, 6.0, 
8.0]], [[3.0, 3.0, 7.0], [4.0, 1.0, 2.0], [4.0, 8.0, 3.0]]]]))
      val=Data(numpy.array([[[[7.0, 7.0, 6.0], [2.0, 1.0, 3.0], [1.0, 1.0, 1.0]], [[8.0, 4.0, 6.0], [7.0, 1.0, 3.0], [2.0, 3.0, 
6.0]], [[4.0, 5.0, 5.0], [8.0, 2.0, 1.0], [2.0, 7.0, 2.0]]], [[[8.0, 6.0, 8.0], [7.0, 1.0, 8.0], [7.0, 5.0, 8.0]], [[1.0, 3.0, 
8.0], [2.0, 7.0, 4.0], [6.0, 7.0, 2.0]], [[1.0, 7.0, 2.0], [6.0, 5.0, 2.0], [7.0, 4.0, 5.0]]]]),self.functionspace)
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 4.0], [4.0, 2.0, 5.0], [2.0, 7.0, 3.0]], [[2.0, 8.0, 3.0], [2.0, 8.0, 3.0], [8.0, 
8.0, 5.0]], [[8.0, 5.0, 5.0], [6.0, 5.0, 2.0], [6.0, 4.0, 5.0]]], [[[2.0, 8.0, 2.0], [4.0, 6.0, 2.0], [4.0, 2.0, 8.0]], [[4.0, 
2.0, 2.0], [6.0, 1.0, 8.0], [6.0, 5.0, 7.0]], [[7.0, 4.0, 8.0], [1.0, 3.0, 4.0], [6.0, 6.0, 4.0]]], [[[7.0, 7.0, 6.0], [2.0, 
1.0, 3.0], [1.0, 1.0, 1.0]], [[8.0, 4.0, 6.0], [7.0, 1.0, 3.0], [2.0, 3.0, 6.0]], [[4.0, 5.0, 5.0], [8.0, 2.0, 1.0], [2.0, 7.0, 
2.0]]], [[[8.0, 6.0, 8.0], [7.0, 1.0, 8.0], [7.0, 5.0, 8.0]], [[1.0, 3.0, 8.0], [2.0, 7.0, 4.0], [6.0, 7.0, 2.0]], [[1.0, 7.0, 
2.0], [6.0, 5.0, 2.0], [7.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 4.0], [4.0, 1.0, 4.0], [1.0, 1.0, 8.0]], 
[[5.0, 5.0, 3.0], [4.0, 7.0, 8.0], [1.0, 5.0, 5.0]], [[1.0, 4.0, 8.0], [1.0, 6.0, 4.0], [8.0, 1.0, 4.0]]], [[[5.0, 4.0, 1.0], 
[5.0, 2.0, 8.0], [5.0, 6.0, 8.0]], [[5.0, 8.0, 5.0], [8.0, 7.0, 7.0], [7.0, 2.0, 7.0]], [[7.0, 6.0, 7.0], [4.0, 1.0, 8.0], 
[1.0, 3.0, 5.0]]], [[[7.0, 7.0, 6.0], [2.0, 1.0, 3.0], [1.0, 1.0, 1.0]], [[8.0, 4.0, 6.0], [7.0, 1.0, 3.0], [2.0, 3.0, 6.0]], 
[[4.0, 5.0, 5.0], [8.0, 2.0, 1.0], [2.0, 7.0, 2.0]]], [[[8.0, 6.0, 8.0], [7.0, 1.0, 8.0], [7.0, 5.0, 8.0]], [[1.0, 3.0, 8.0], 
[2.0, 7.0, 4.0], [6.0, 7.0, 2.0]], [[1.0, 7.0, 2.0], [6.0, 5.0, 2.0], [7.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LCCN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 7.0], [1.0, 2.0, 2.0], [6.0, 1.0, 5.0]], [[6.0, 2.0, 6.0], [6.0, 2.0, 3.0], [4.0, 2.0, 
1.0]], [[1.0, 4.0, 7.0], [6.0, 5.0, 5.0], [1.0, 4.0, 7.0]]], [[[8.0, 6.0, 4.0], [2.0, 1.0, 5.0], [2.0, 8.0, 1.0]], [[2.0, 6.0, 
6.0], [7.0, 3.0, 7.0], [3.0, 6.0, 4.0]], [[1.0, 8.0, 4.0], [2.0, 5.0, 8.0], [3.0, 7.0, 7.0]]], [[[4.0, 6.0, 5.0], [7.0, 4.0, 
1.0], [8.0, 3.0, 4.0]], [[2.0, 5.0, 1.0], [4.0, 2.0, 2.0], [4.0, 1.0, 7.0]], [[7.0, 8.0, 8.0], [6.0, 8.0, 1.0], [2.0, 5.0, 
2.0]]], [[[4.0, 2.0, 3.0], [2.0, 6.0, 6.0], [5.0, 5.0, 8.0]], [[6.0, 7.0, 6.0], [6.0, 5.0, 7.0], [4.0, 8.0, 4.0]], [[6.0, 6.0, 
7.0], [8.0, 8.0, 7.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 2.0], [8.0, 1.0, 8.0], [7.0, 4.0, 1.0]], [[1.0, 7.0, 5.0], [1.0, 8.0, 
5.0], [1.0, 1.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 1.0, 7.0], [4.0, 2.0, 8.0]]], [[[2.0, 1.0, 8.0], [1.0, 1.0, 1.0], [1.0, 6.0, 
7.0]], [[3.0, 5.0, 8.0], [4.0, 6.0, 1.0], [5.0, 1.0, 1.0]], [[6.0, 8.0, 7.0], [8.0, 6.0, 7.0], [4.0, 8.0, 7.0]]], [[[6.0, 1.0, 
1.0], [1.0, 5.0, 6.0], [6.0, 1.0, 1.0]], [[6.0, 6.0, 8.0], [1.0, 2.0, 1.0], [5.0, 4.0, 6.0]], [[1.0, 7.0, 3.0], [3.0, 4.0, 
7.0], [6.0, 8.0, 1.0]]], [[[3.0, 1.0, 8.0], [3.0, 4.0, 8.0], [8.0, 8.0, 6.0]], [[5.0, 7.0, 3.0], [3.0, 1.0, 7.0], [4.0, 1.0, 
8.0]], [[4.0, 4.0, 2.0], [6.0, 6.0, 8.0], [5.0, 7.0, 6.0]]]]))
      val=Data(numpy.array([[[[2.0, 5.0, 5.0], [8.0, 3.0, 2.0], [3.0, 4.0, 8.0]], [[3.0, 8.0, 5.0], [1.0, 5.0, 1.0], [1.0, 1.0, 
8.0]], [[1.0, 6.0, 2.0], [6.0, 2.0, 7.0], [4.0, 5.0, 8.0]]], [[[4.0, 5.0, 2.0], [6.0, 2.0, 7.0], [3.0, 5.0, 7.0]], [[4.0, 6.0, 
5.0], [6.0, 2.0, 7.0], [3.0, 2.0, 4.0]], [[1.0, 5.0, 8.0], [3.0, 8.0, 2.0], [4.0, 7.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 6.0], [5.0, 8.0, 8.0], [7.0, 5.0, 5.0]], [[6.0, 8.0, 5.0], [6.0, 2.0, 
7.0], [1.0, 3.0, 4.0]], [[4.0, 1.0, 3.0], [1.0, 1.0, 2.0], [3.0, 1.0, 5.0]]], [[[2.0, 8.0, 8.0], [1.0, 5.0, 4.0], [5.0, 2.0, 
4.0]], [[4.0, 2.0, 5.0], [1.0, 8.0, 4.0], [2.0, 6.0, 5.0]], [[5.0, 1.0, 2.0], [8.0, 5.0, 7.0], [2.0, 3.0, 3.0]]]]))
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 7.0], [1.0, 2.0, 2.0], [6.0, 1.0, 5.0]], [[6.0, 2.0, 6.0], [6.0, 2.0, 3.0], [4.0, 
2.0, 1.0]], [[1.0, 4.0, 7.0], [6.0, 5.0, 5.0], [1.0, 4.0, 7.0]]], [[[8.0, 6.0, 4.0], [2.0, 1.0, 5.0], [2.0, 8.0, 1.0]], [[2.0, 
6.0, 6.0], [7.0, 3.0, 7.0], [3.0, 6.0, 4.0]], [[1.0, 8.0, 4.0], [2.0, 5.0, 8.0], [3.0, 7.0, 7.0]]], [[[2.0, 5.0, 5.0], [8.0, 
3.0, 2.0], [3.0, 4.0, 8.0]], [[3.0, 8.0, 5.0], [1.0, 5.0, 1.0], [1.0, 1.0, 8.0]], [[1.0, 6.0, 2.0], [6.0, 2.0, 7.0], [4.0, 5.0, 
8.0]]], [[[4.0, 5.0, 2.0], [6.0, 2.0, 7.0], [3.0, 5.0, 7.0]], [[4.0, 6.0, 5.0], [6.0, 2.0, 7.0], [3.0, 2.0, 4.0]], [[1.0, 5.0, 
8.0], [3.0, 8.0, 2.0], [4.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 2.0], [8.0, 1.0, 8.0], [7.0, 4.0, 1.0]], 
[[1.0, 7.0, 5.0], [1.0, 8.0, 5.0], [1.0, 1.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 1.0, 7.0], [4.0, 2.0, 8.0]]], [[[2.0, 1.0, 8.0], 
[1.0, 1.0, 1.0], [1.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [4.0, 6.0, 1.0], [5.0, 1.0, 1.0]], [[6.0, 8.0, 7.0], [8.0, 6.0, 7.0], 
[4.0, 8.0, 7.0]]], [[[2.0, 2.0, 6.0], [5.0, 8.0, 8.0], [7.0, 5.0, 5.0]], [[6.0, 8.0, 5.0], [6.0, 2.0, 7.0], [1.0, 3.0, 4.0]], 
[[4.0, 1.0, 3.0], [1.0, 1.0, 2.0], [3.0, 1.0, 5.0]]], [[[2.0, 8.0, 8.0], [1.0, 5.0, 4.0], [5.0, 2.0, 4.0]], [[4.0, 2.0, 5.0], 
[1.0, 8.0, 4.0], [2.0, 6.0, 5.0]], [[5.0, 1.0, 2.0], [8.0, 5.0, 7.0], [2.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LCCN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 8.0], [5.0, 5.0, 2.0], [3.0, 6.0, 5.0]], [[8.0, 2.0, 7.0], [2.0, 3.0, 3.0], [2.0, 4.0, 
6.0]], [[8.0, 8.0, 4.0], [5.0, 6.0, 5.0], [2.0, 2.0, 3.0]]], [[[7.0, 7.0, 4.0], [4.0, 6.0, 1.0], [1.0, 6.0, 5.0]], [[6.0, 3.0, 
2.0], [4.0, 6.0, 4.0], [7.0, 5.0, 3.0]], [[7.0, 8.0, 8.0], [6.0, 1.0, 1.0], [2.0, 8.0, 5.0]]], [[[3.0, 6.0, 7.0], [7.0, 4.0, 
5.0], [7.0, 4.0, 6.0]], [[2.0, 1.0, 2.0], [7.0, 1.0, 4.0], [3.0, 5.0, 6.0]], [[2.0, 3.0, 2.0], [8.0, 2.0, 8.0], [2.0, 1.0, 
7.0]]], [[[1.0, 2.0, 6.0], [3.0, 1.0, 3.0], [1.0, 5.0, 2.0]], [[2.0, 5.0, 3.0], [2.0, 4.0, 4.0], [6.0, 6.0, 2.0]], [[1.0, 3.0, 
6.0], [3.0, 5.0, 5.0], [4.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 3.0], [1.0, 2.0, 8.0], [7.0, 8.0, 6.0]], [[2.0, 7.0, 2.0], [3.0, 4.0, 
3.0], [4.0, 6.0, 7.0]], [[1.0, 5.0, 3.0], [7.0, 4.0, 1.0], [3.0, 1.0, 2.0]]], [[[2.0, 4.0, 4.0], [5.0, 5.0, 1.0], [1.0, 1.0, 
7.0]], [[8.0, 1.0, 7.0], [6.0, 8.0, 6.0], [1.0, 4.0, 2.0]], [[5.0, 4.0, 7.0], [5.0, 8.0, 6.0], [5.0, 3.0, 3.0]]], [[[5.0, 6.0, 
2.0], [6.0, 1.0, 1.0], [4.0, 8.0, 8.0]], [[3.0, 4.0, 7.0], [5.0, 5.0, 1.0], [3.0, 8.0, 4.0]], [[6.0, 7.0, 8.0], [6.0, 2.0, 
4.0], [5.0, 5.0, 8.0]]], [[[7.0, 5.0, 3.0], [6.0, 3.0, 1.0], [3.0, 5.0, 3.0]], [[6.0, 8.0, 8.0], [3.0, 2.0, 1.0], [2.0, 8.0, 
5.0]], [[4.0, 1.0, 6.0], [7.0, 5.0, 1.0], [4.0, 2.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 1.0, 4.0], [6.0, 8.0, 4.0], [1.0, 5.0, 1.0]], [[8.0, 1.0, 8.0], [3.0, 6.0, 6.0], [4.0, 
2.0, 4.0]], [[5.0, 5.0, 3.0], [3.0, 7.0, 6.0], [1.0, 3.0, 3.0]]], [[[1.0, 2.0, 7.0], [2.0, 2.0, 5.0], [5.0, 2.0, 3.0]], [[2.0, 
3.0, 2.0], [1.0, 1.0, 7.0], [8.0, 8.0, 5.0]], [[1.0, 5.0, 2.0], [6.0, 1.0, 2.0], [6.0, 8.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 5.0, 7.0], [7.0, 3.0, 1.0], [8.0, 2.0, 1.0]], [[4.0, 2.0, 4.0], [3.0, 6.0, 8.0], 
[3.0, 1.0, 6.0]], [[3.0, 1.0, 4.0], [4.0, 4.0, 5.0], [6.0, 5.0, 6.0]]], [[[2.0, 8.0, 6.0], [2.0, 2.0, 8.0], [2.0, 5.0, 8.0]], 
[[1.0, 8.0, 1.0], [3.0, 6.0, 8.0], [7.0, 5.0, 1.0]], [[7.0, 5.0, 1.0], [8.0, 6.0, 3.0], [7.0, 8.0, 4.0]]]])
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 8.0], [5.0, 5.0, 2.0], [3.0, 6.0, 5.0]], [[8.0, 2.0, 7.0], [2.0, 3.0, 3.0], [2.0, 
4.0, 6.0]], [[8.0, 8.0, 4.0], [5.0, 6.0, 5.0], [2.0, 2.0, 3.0]]], [[[7.0, 7.0, 4.0], [4.0, 6.0, 1.0], [1.0, 6.0, 5.0]], [[6.0, 
3.0, 2.0], [4.0, 6.0, 4.0], [7.0, 5.0, 3.0]], [[7.0, 8.0, 8.0], [6.0, 1.0, 1.0], [2.0, 8.0, 5.0]]], [[[3.0, 1.0, 4.0], [6.0, 
8.0, 4.0], [1.0, 5.0, 1.0]], [[8.0, 1.0, 8.0], [3.0, 6.0, 6.0], [4.0, 2.0, 4.0]], [[5.0, 5.0, 3.0], [3.0, 7.0, 6.0], [1.0, 3.0, 
3.0]]], [[[1.0, 2.0, 7.0], [2.0, 2.0, 5.0], [5.0, 2.0, 3.0]], [[2.0, 3.0, 2.0], [1.0, 1.0, 7.0], [8.0, 8.0, 5.0]], [[1.0, 5.0, 
2.0], [6.0, 1.0, 2.0], [6.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 3.0], [1.0, 2.0, 8.0], [7.0, 8.0, 6.0]], 
[[2.0, 7.0, 2.0], [3.0, 4.0, 3.0], [4.0, 6.0, 7.0]], [[1.0, 5.0, 3.0], [7.0, 4.0, 1.0], [3.0, 1.0, 2.0]]], [[[2.0, 4.0, 4.0], 
[5.0, 5.0, 1.0], [1.0, 1.0, 7.0]], [[8.0, 1.0, 7.0], [6.0, 8.0, 6.0], [1.0, 4.0, 2.0]], [[5.0, 4.0, 7.0], [5.0, 8.0, 6.0], 
[5.0, 3.0, 3.0]]], [[[1.0, 5.0, 7.0], [7.0, 3.0, 1.0], [8.0, 2.0, 1.0]], [[4.0, 2.0, 4.0], [3.0, 6.0, 8.0], [3.0, 1.0, 6.0]], 
[[3.0, 1.0, 4.0], [4.0, 4.0, 5.0], [6.0, 5.0, 6.0]]], [[[2.0, 8.0, 6.0], [2.0, 2.0, 8.0], [2.0, 5.0, 8.0]], [[1.0, 8.0, 1.0], 
[3.0, 6.0, 8.0], [7.0, 5.0, 1.0]], [[7.0, 5.0, 1.0], [8.0, 6.0, 3.0], [7.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CLUN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 3.0], [7.0, 7.0, 5.0], [6.0, 8.0, 1.0]], [[5.0, 4.0, 7.0], [4.0, 1.0, 2.0], [2.0, 4.0, 
1.0]], [[6.0, 7.0, 2.0], [5.0, 5.0, 3.0], [2.0, 5.0, 7.0]]], [[[2.0, 4.0, 4.0], [1.0, 2.0, 4.0], [4.0, 5.0, 8.0]], [[7.0, 3.0, 
6.0], [7.0, 3.0, 5.0], [6.0, 8.0, 1.0]], [[6.0, 8.0, 7.0], [4.0, 3.0, 6.0], [6.0, 2.0, 1.0]]], [[[3.0, 5.0, 1.0], [6.0, 4.0, 
8.0], [6.0, 4.0, 3.0]], [[2.0, 1.0, 1.0], [1.0, 7.0, 7.0], [3.0, 4.0, 4.0]], [[2.0, 3.0, 1.0], [1.0, 6.0, 4.0], [2.0, 8.0, 
1.0]]], [[[2.0, 6.0, 1.0], [3.0, 1.0, 6.0], [7.0, 1.0, 3.0]], [[4.0, 3.0, 1.0], [8.0, 3.0, 7.0], [2.0, 7.0, 5.0]], [[5.0, 2.0, 
2.0], [5.0, 4.0, 6.0], [3.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 1.0], [5.0, 5.0, 3.0], [4.0, 1.0, 5.0]], [[5.0, 4.0, 7.0], [3.0, 2.0, 
6.0], [3.0, 4.0, 6.0]], [[3.0, 7.0, 1.0], [7.0, 6.0, 3.0], [1.0, 2.0, 2.0]]], [[[7.0, 5.0, 8.0], [8.0, 4.0, 4.0], [3.0, 7.0, 
6.0]], [[3.0, 1.0, 2.0], [7.0, 3.0, 4.0], [2.0, 4.0, 2.0]], [[8.0, 7.0, 1.0], [3.0, 6.0, 5.0], [1.0, 3.0, 5.0]]], [[[3.0, 5.0, 
7.0], [1.0, 6.0, 5.0], [4.0, 2.0, 8.0]], [[2.0, 1.0, 2.0], [3.0, 6.0, 6.0], [1.0, 2.0, 6.0]], [[3.0, 2.0, 4.0], [3.0, 7.0, 
4.0], [6.0, 1.0, 3.0]]], [[[4.0, 2.0, 7.0], [8.0, 7.0, 7.0], [1.0, 3.0, 5.0]], [[1.0, 3.0, 8.0], [2.0, 7.0, 5.0], [5.0, 3.0, 
1.0]], [[6.0, 5.0, 4.0], [1.0, 1.0, 1.0], [7.0, 7.0, 5.0]]]]))
      val=numpy.array([[[[6.0, 2.0, 2.0], [1.0, 3.0, 8.0]]], [[[2.0, 6.0, 5.0], [8.0, 2.0, 2.0]]], [[[5.0, 6.0, 1.0], [1.0, 
7.0, 8.0]]], [[[7.0, 2.0, 3.0], [5.0, 2.0, 6.0]]]])
      arg[:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 3.0], [7.0, 7.0, 5.0], [6.0, 8.0, 1.0]], [[5.0, 4.0, 7.0], [4.0, 1.0, 2.0], [2.0, 
4.0, 1.0]], [[6.0, 2.0, 2.0], [1.0, 3.0, 8.0], [2.0, 5.0, 7.0]]], [[[2.0, 4.0, 4.0], [1.0, 2.0, 4.0], [4.0, 5.0, 8.0]], [[7.0, 
3.0, 6.0], [7.0, 3.0, 5.0], [6.0, 8.0, 1.0]], [[2.0, 6.0, 5.0], [8.0, 2.0, 2.0], [6.0, 2.0, 1.0]]], [[[3.0, 5.0, 1.0], [6.0, 
4.0, 8.0], [6.0, 4.0, 3.0]], [[2.0, 1.0, 1.0], [1.0, 7.0, 7.0], [3.0, 4.0, 4.0]], [[5.0, 6.0, 1.0], [1.0, 7.0, 8.0], [2.0, 8.0, 
1.0]]], [[[2.0, 6.0, 1.0], [3.0, 1.0, 6.0], [7.0, 1.0, 3.0]], [[4.0, 3.0, 1.0], [8.0, 3.0, 7.0], [2.0, 7.0, 5.0]], [[7.0, 2.0, 
3.0], [5.0, 2.0, 6.0], [3.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 1.0], [5.0, 5.0, 3.0], [4.0, 1.0, 5.0]], 
[[5.0, 4.0, 7.0], [3.0, 2.0, 6.0], [3.0, 4.0, 6.0]], [[6.0, 2.0, 2.0], [1.0, 3.0, 8.0], [1.0, 2.0, 2.0]]], [[[7.0, 5.0, 8.0], 
[8.0, 4.0, 4.0], [3.0, 7.0, 6.0]], [[3.0, 1.0, 2.0], [7.0, 3.0, 4.0], [2.0, 4.0, 2.0]], [[2.0, 6.0, 5.0], [8.0, 2.0, 2.0], 
[1.0, 3.0, 5.0]]], [[[3.0, 5.0, 7.0], [1.0, 6.0, 5.0], [4.0, 2.0, 8.0]], [[2.0, 1.0, 2.0], [3.0, 6.0, 6.0], [1.0, 2.0, 6.0]], 
[[5.0, 6.0, 1.0], [1.0, 7.0, 8.0], [6.0, 1.0, 3.0]]], [[[4.0, 2.0, 7.0], [8.0, 7.0, 7.0], [1.0, 3.0, 5.0]], [[1.0, 3.0, 8.0], 
[2.0, 7.0, 5.0], [5.0, 3.0, 1.0]], [[7.0, 2.0, 3.0], [5.0, 2.0, 6.0], [7.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CLUN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 3.0], [8.0, 8.0, 3.0], [2.0, 1.0, 2.0]], [[2.0, 2.0, 2.0], [6.0, 7.0, 2.0], [2.0, 1.0, 
7.0]], [[7.0, 4.0, 5.0], [2.0, 3.0, 7.0], [3.0, 1.0, 1.0]]], [[[8.0, 3.0, 4.0], [1.0, 6.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 7.0, 
1.0], [3.0, 4.0, 2.0], [3.0, 5.0, 6.0]], [[2.0, 7.0, 5.0], [5.0, 7.0, 8.0], [4.0, 5.0, 1.0]]], [[[8.0, 4.0, 8.0], [5.0, 2.0, 
4.0], [4.0, 3.0, 6.0]], [[5.0, 1.0, 1.0], [4.0, 3.0, 6.0], [5.0, 2.0, 7.0]], [[4.0, 4.0, 3.0], [8.0, 7.0, 8.0], [4.0, 1.0, 
6.0]]], [[[1.0, 8.0, 1.0], [1.0, 8.0, 7.0], [5.0, 6.0, 1.0]], [[2.0, 4.0, 6.0], [8.0, 7.0, 3.0], [3.0, 7.0, 1.0]], [[2.0, 8.0, 
6.0], [8.0, 3.0, 4.0], [8.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 5.0], [4.0, 2.0, 5.0], [5.0, 8.0, 4.0]], [[1.0, 4.0, 4.0], [2.0, 2.0, 
2.0], [5.0, 8.0, 5.0]], [[7.0, 6.0, 8.0], [1.0, 3.0, 3.0], [7.0, 5.0, 1.0]]], [[[8.0, 5.0, 4.0], [4.0, 5.0, 1.0], [5.0, 2.0, 
1.0]], [[6.0, 4.0, 2.0], [7.0, 1.0, 6.0], [3.0, 4.0, 7.0]], [[4.0, 7.0, 7.0], [2.0, 5.0, 5.0], [7.0, 6.0, 5.0]]], [[[1.0, 4.0, 
1.0], [3.0, 7.0, 4.0], [5.0, 1.0, 5.0]], [[1.0, 8.0, 3.0], [7.0, 5.0, 1.0], [2.0, 1.0, 1.0]], [[3.0, 3.0, 7.0], [1.0, 4.0, 
8.0], [1.0, 3.0, 5.0]]], [[[5.0, 5.0, 1.0], [6.0, 2.0, 1.0], [4.0, 8.0, 4.0]], [[8.0, 4.0, 6.0], [7.0, 8.0, 4.0], [8.0, 2.0, 
4.0]], [[3.0, 5.0, 8.0], [8.0, 6.0, 6.0], [7.0, 2.0, 7.0]]]]))
      val=Data(numpy.array([[[[1.0, 3.0, 4.0], [5.0, 7.0, 3.0]]], [[[6.0, 6.0, 8.0], [6.0, 4.0, 3.0]]], [[[5.0, 2.0, 4.0], 
[8.0, 3.0, 2.0]]], [[[4.0, 1.0, 1.0], [3.0, 5.0, 2.0]]]]),self.functionspace)
      arg[:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 3.0], [8.0, 8.0, 3.0], [2.0, 1.0, 2.0]], [[2.0, 2.0, 2.0], [6.0, 7.0, 2.0], [2.0, 
1.0, 7.0]], [[1.0, 3.0, 4.0], [5.0, 7.0, 3.0], [3.0, 1.0, 1.0]]], [[[8.0, 3.0, 4.0], [1.0, 6.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 
7.0, 1.0], [3.0, 4.0, 2.0], [3.0, 5.0, 6.0]], [[6.0, 6.0, 8.0], [6.0, 4.0, 3.0], [4.0, 5.0, 1.0]]], [[[8.0, 4.0, 8.0], [5.0, 
2.0, 4.0], [4.0, 3.0, 6.0]], [[5.0, 1.0, 1.0], [4.0, 3.0, 6.0], [5.0, 2.0, 7.0]], [[5.0, 2.0, 4.0], [8.0, 3.0, 2.0], [4.0, 1.0, 
6.0]]], [[[1.0, 8.0, 1.0], [1.0, 8.0, 7.0], [5.0, 6.0, 1.0]], [[2.0, 4.0, 6.0], [8.0, 7.0, 3.0], [3.0, 7.0, 1.0]], [[4.0, 1.0, 
1.0], [3.0, 5.0, 2.0], [8.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 5.0], [4.0, 2.0, 5.0], [5.0, 8.0, 4.0]], 
[[1.0, 4.0, 4.0], [2.0, 2.0, 2.0], [5.0, 8.0, 5.0]], [[1.0, 3.0, 4.0], [5.0, 7.0, 3.0], [7.0, 5.0, 1.0]]], [[[8.0, 5.0, 4.0], 
[4.0, 5.0, 1.0], [5.0, 2.0, 1.0]], [[6.0, 4.0, 2.0], [7.0, 1.0, 6.0], [3.0, 4.0, 7.0]], [[6.0, 6.0, 8.0], [6.0, 4.0, 3.0], 
[7.0, 6.0, 5.0]]], [[[1.0, 4.0, 1.0], [3.0, 7.0, 4.0], [5.0, 1.0, 5.0]], [[1.0, 8.0, 3.0], [7.0, 5.0, 1.0], [2.0, 1.0, 1.0]], 
[[5.0, 2.0, 4.0], [8.0, 3.0, 2.0], [1.0, 3.0, 5.0]]], [[[5.0, 5.0, 1.0], [6.0, 2.0, 1.0], [4.0, 8.0, 4.0]], [[8.0, 4.0, 6.0], 
[7.0, 8.0, 4.0], [8.0, 2.0, 4.0]], [[4.0, 1.0, 1.0], [3.0, 5.0, 2.0], [7.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CLUN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 1.0], [2.0, 7.0, 1.0], [1.0, 4.0, 8.0]], [[2.0, 4.0, 6.0], [1.0, 1.0, 5.0], [1.0, 2.0, 
8.0]], [[7.0, 7.0, 2.0], [8.0, 8.0, 4.0], [2.0, 6.0, 3.0]]], [[[7.0, 2.0, 1.0], [1.0, 4.0, 3.0], [1.0, 3.0, 7.0]], [[2.0, 4.0, 
4.0], [7.0, 6.0, 4.0], [7.0, 7.0, 8.0]], [[3.0, 3.0, 7.0], [8.0, 7.0, 5.0], [7.0, 7.0, 4.0]]], [[[3.0, 4.0, 4.0], [4.0, 5.0, 
2.0], [4.0, 1.0, 6.0]], [[6.0, 3.0, 6.0], [4.0, 7.0, 1.0], [3.0, 3.0, 4.0]], [[4.0, 2.0, 1.0], [4.0, 7.0, 8.0], [7.0, 5.0, 
1.0]]], [[[3.0, 4.0, 8.0], [8.0, 7.0, 8.0], [2.0, 8.0, 7.0]], [[3.0, 6.0, 3.0], [6.0, 3.0, 7.0], [4.0, 8.0, 3.0]], [[1.0, 4.0, 
4.0], [1.0, 3.0, 6.0], [8.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 3.0], [7.0, 5.0, 7.0], [7.0, 8.0, 8.0]], [[3.0, 3.0, 7.0], [6.0, 1.0, 
7.0], [8.0, 5.0, 7.0]], [[2.0, 2.0, 4.0], [5.0, 7.0, 1.0], [3.0, 2.0, 6.0]]], [[[3.0, 7.0, 4.0], [3.0, 8.0, 6.0], [7.0, 1.0, 
6.0]], [[1.0, 2.0, 2.0], [8.0, 7.0, 2.0], [1.0, 8.0, 8.0]], [[1.0, 8.0, 1.0], [5.0, 4.0, 1.0], [1.0, 8.0, 4.0]]], [[[1.0, 7.0, 
3.0], [3.0, 7.0, 4.0], [4.0, 5.0, 6.0]], [[4.0, 5.0, 1.0], [5.0, 4.0, 3.0], [3.0, 4.0, 7.0]], [[2.0, 7.0, 2.0], [5.0, 7.0, 
3.0], [2.0, 3.0, 1.0]]], [[[1.0, 6.0, 3.0], [7.0, 4.0, 6.0], [1.0, 8.0, 1.0]], [[3.0, 5.0, 1.0], [8.0, 2.0, 6.0], [4.0, 7.0, 
3.0]], [[3.0, 3.0, 5.0], [4.0, 4.0, 3.0], [4.0, 1.0, 1.0]]]]))
      val=Data(numpy.array([[[[7.0, 8.0, 6.0], [2.0, 6.0, 7.0]]], [[[8.0, 5.0, 3.0], [8.0, 1.0, 4.0]]], [[[3.0, 5.0, 8.0], 
[1.0, 5.0, 8.0]]], [[[2.0, 3.0, 5.0], [3.0, 5.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 7.0], [6.0, 4.0, 7.0]]], [[[7.0, 4.0, 3.0], [6.0, 6.0, 5.0]]], [[[6.0, 
5.0, 2.0], [2.0, 7.0, 4.0]]], [[[3.0, 3.0, 3.0], [6.0, 3.0, 3.0]]]]))
      arg[:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 1.0], [2.0, 7.0, 1.0], [1.0, 4.0, 8.0]], [[2.0, 4.0, 6.0], [1.0, 1.0, 5.0], [1.0, 
2.0, 8.0]], [[7.0, 8.0, 6.0], [2.0, 6.0, 7.0], [2.0, 6.0, 3.0]]], [[[7.0, 2.0, 1.0], [1.0, 4.0, 3.0], [1.0, 3.0, 7.0]], [[2.0, 
4.0, 4.0], [7.0, 6.0, 4.0], [7.0, 7.0, 8.0]], [[8.0, 5.0, 3.0], [8.0, 1.0, 4.0], [7.0, 7.0, 4.0]]], [[[3.0, 4.0, 4.0], [4.0, 
5.0, 2.0], [4.0, 1.0, 6.0]], [[6.0, 3.0, 6.0], [4.0, 7.0, 1.0], [3.0, 3.0, 4.0]], [[3.0, 5.0, 8.0], [1.0, 5.0, 8.0], [7.0, 5.0, 
1.0]]], [[[3.0, 4.0, 8.0], [8.0, 7.0, 8.0], [2.0, 8.0, 7.0]], [[3.0, 6.0, 3.0], [6.0, 3.0, 7.0], [4.0, 8.0, 3.0]], [[2.0, 3.0, 
5.0], [3.0, 5.0, 1.0], [8.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 3.0], [7.0, 5.0, 7.0], [7.0, 8.0, 8.0]], 
[[3.0, 3.0, 7.0], [6.0, 1.0, 7.0], [8.0, 5.0, 7.0]], [[8.0, 6.0, 7.0], [6.0, 4.0, 7.0], [3.0, 2.0, 6.0]]], [[[3.0, 7.0, 4.0], 
[3.0, 8.0, 6.0], [7.0, 1.0, 6.0]], [[1.0, 2.0, 2.0], [8.0, 7.0, 2.0], [1.0, 8.0, 8.0]], [[7.0, 4.0, 3.0], [6.0, 6.0, 5.0], 
[1.0, 8.0, 4.0]]], [[[1.0, 7.0, 3.0], [3.0, 7.0, 4.0], [4.0, 5.0, 6.0]], [[4.0, 5.0, 1.0], [5.0, 4.0, 3.0], [3.0, 4.0, 7.0]], 
[[6.0, 5.0, 2.0], [2.0, 7.0, 4.0], [2.0, 3.0, 1.0]]], [[[1.0, 6.0, 3.0], [7.0, 4.0, 6.0], [1.0, 8.0, 1.0]], [[3.0, 5.0, 1.0], 
[8.0, 2.0, 6.0], [4.0, 7.0, 3.0]], [[3.0, 3.0, 3.0], [6.0, 3.0, 3.0], [4.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CLUN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 8.0], [5.0, 3.0, 6.0], [5.0, 3.0, 1.0]], [[1.0, 5.0, 6.0], [1.0, 8.0, 7.0], [1.0, 8.0, 
7.0]], [[3.0, 4.0, 5.0], [2.0, 1.0, 8.0], [3.0, 2.0, 5.0]]], [[[6.0, 3.0, 4.0], [2.0, 1.0, 1.0], [4.0, 7.0, 2.0]], [[4.0, 1.0, 
1.0], [5.0, 8.0, 6.0], [8.0, 8.0, 3.0]], [[6.0, 3.0, 2.0], [6.0, 1.0, 6.0], [5.0, 2.0, 5.0]]], [[[6.0, 4.0, 2.0], [7.0, 5.0, 
7.0], [8.0, 1.0, 3.0]], [[7.0, 6.0, 2.0], [8.0, 3.0, 5.0], [1.0, 4.0, 3.0]], [[1.0, 7.0, 3.0], [5.0, 7.0, 5.0], [4.0, 6.0, 
2.0]]], [[[3.0, 7.0, 3.0], [8.0, 7.0, 5.0], [4.0, 6.0, 2.0]], [[8.0, 6.0, 4.0], [5.0, 5.0, 7.0], [8.0, 1.0, 2.0]], [[8.0, 5.0, 
4.0], [6.0, 6.0, 4.0], [4.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 2.0], [1.0, 2.0, 3.0], [8.0, 4.0, 7.0]], [[3.0, 7.0, 1.0], [8.0, 5.0, 
1.0], [7.0, 8.0, 6.0]], [[3.0, 2.0, 7.0], [7.0, 1.0, 1.0], [1.0, 6.0, 2.0]]], [[[2.0, 8.0, 8.0], [7.0, 2.0, 5.0], [4.0, 4.0, 
7.0]], [[4.0, 5.0, 3.0], [4.0, 3.0, 8.0], [4.0, 8.0, 8.0]], [[8.0, 2.0, 1.0], [4.0, 8.0, 8.0], [8.0, 7.0, 3.0]]], [[[4.0, 3.0, 
1.0], [3.0, 2.0, 2.0], [8.0, 8.0, 7.0]], [[8.0, 6.0, 3.0], [4.0, 4.0, 5.0], [3.0, 5.0, 7.0]], [[7.0, 4.0, 7.0], [8.0, 4.0, 
7.0], [8.0, 3.0, 6.0]]], [[[1.0, 1.0, 5.0], [6.0, 7.0, 1.0], [8.0, 8.0, 4.0]], [[2.0, 7.0, 5.0], [3.0, 5.0, 1.0], [2.0, 6.0, 
6.0]], [[8.0, 3.0, 5.0], [1.0, 1.0, 3.0], [5.0, 2.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 3.0, 5.0], [6.0, 2.0, 6.0]]], [[[6.0, 3.0, 3.0], [5.0, 1.0, 5.0]]], [[[7.0, 7.0, 1.0], 
[6.0, 7.0, 6.0]]], [[[8.0, 1.0, 2.0], [6.0, 3.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 2.0, 6.0], [3.0, 3.0, 1.0]]], 
[[[3.0, 8.0, 8.0], [1.0, 8.0, 1.0]]], [[[6.0, 6.0, 6.0], [3.0, 4.0, 7.0]]], [[[8.0, 1.0, 8.0], [4.0, 8.0, 4.0]]]])
      arg[:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 8.0], [5.0, 3.0, 6.0], [5.0, 3.0, 1.0]], [[1.0, 5.0, 6.0], [1.0, 8.0, 7.0], [1.0, 
8.0, 7.0]], [[1.0, 3.0, 5.0], [6.0, 2.0, 6.0], [3.0, 2.0, 5.0]]], [[[6.0, 3.0, 4.0], [2.0, 1.0, 1.0], [4.0, 7.0, 2.0]], [[4.0, 
1.0, 1.0], [5.0, 8.0, 6.0], [8.0, 8.0, 3.0]], [[6.0, 3.0, 3.0], [5.0, 1.0, 5.0], [5.0, 2.0, 5.0]]], [[[6.0, 4.0, 2.0], [7.0, 
5.0, 7.0], [8.0, 1.0, 3.0]], [[7.0, 6.0, 2.0], [8.0, 3.0, 5.0], [1.0, 4.0, 3.0]], [[7.0, 7.0, 1.0], [6.0, 7.0, 6.0], [4.0, 6.0, 
2.0]]], [[[3.0, 7.0, 3.0], [8.0, 7.0, 5.0], [4.0, 6.0, 2.0]], [[8.0, 6.0, 4.0], [5.0, 5.0, 7.0], [8.0, 1.0, 2.0]], [[8.0, 1.0, 
2.0], [6.0, 3.0, 6.0], [4.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 2.0], [1.0, 2.0, 3.0], [8.0, 4.0, 7.0]], 
[[3.0, 7.0, 1.0], [8.0, 5.0, 1.0], [7.0, 8.0, 6.0]], [[6.0, 2.0, 6.0], [3.0, 3.0, 1.0], [1.0, 6.0, 2.0]]], [[[2.0, 8.0, 8.0], 
[7.0, 2.0, 5.0], [4.0, 4.0, 7.0]], [[4.0, 5.0, 3.0], [4.0, 3.0, 8.0], [4.0, 8.0, 8.0]], [[3.0, 8.0, 8.0], [1.0, 8.0, 1.0], 
[8.0, 7.0, 3.0]]], [[[4.0, 3.0, 1.0], [3.0, 2.0, 2.0], [8.0, 8.0, 7.0]], [[8.0, 6.0, 3.0], [4.0, 4.0, 5.0], [3.0, 5.0, 7.0]], 
[[6.0, 6.0, 6.0], [3.0, 4.0, 7.0], [8.0, 3.0, 6.0]]], [[[1.0, 1.0, 5.0], [6.0, 7.0, 1.0], [8.0, 8.0, 4.0]], [[2.0, 7.0, 5.0], 
[3.0, 5.0, 1.0], [2.0, 6.0, 6.0]], [[8.0, 1.0, 8.0], [4.0, 8.0, 4.0], [5.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LPUN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 8.0], [7.0, 2.0, 8.0], [1.0, 6.0, 4.0]], [[3.0, 3.0, 2.0], [1.0, 7.0, 2.0], [5.0, 2.0, 
2.0]], [[5.0, 6.0, 7.0], [6.0, 4.0, 2.0], [5.0, 2.0, 7.0]]], [[[3.0, 2.0, 2.0], [7.0, 2.0, 5.0], [3.0, 7.0, 8.0]], [[8.0, 2.0, 
8.0], [8.0, 3.0, 5.0], [2.0, 2.0, 1.0]], [[1.0, 4.0, 7.0], [1.0, 4.0, 2.0], [2.0, 1.0, 2.0]]], [[[6.0, 1.0, 5.0], [2.0, 4.0, 
1.0], [7.0, 2.0, 4.0]], [[1.0, 4.0, 4.0], [5.0, 8.0, 7.0], [8.0, 4.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 4.0, 2.0], [8.0, 8.0, 
2.0]]], [[[2.0, 7.0, 5.0], [7.0, 1.0, 5.0], [6.0, 1.0, 2.0]], [[1.0, 8.0, 7.0], [3.0, 7.0, 8.0], [5.0, 1.0, 8.0]], [[6.0, 3.0, 
4.0], [5.0, 5.0, 1.0], [3.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 8.0], [5.0, 8.0, 7.0], [4.0, 2.0, 3.0]], [[2.0, 5.0, 4.0], [3.0, 1.0, 
1.0], [7.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 2.0, 4.0], [6.0, 2.0, 3.0]]], [[[6.0, 7.0, 5.0], [8.0, 3.0, 2.0], [7.0, 8.0, 
1.0]], [[5.0, 1.0, 2.0], [3.0, 3.0, 5.0], [2.0, 1.0, 4.0]], [[6.0, 3.0, 3.0], [4.0, 5.0, 3.0], [1.0, 4.0, 7.0]]], [[[4.0, 2.0, 
6.0], [1.0, 6.0, 2.0], [6.0, 6.0, 4.0]], [[6.0, 7.0, 4.0], [2.0, 1.0, 8.0], [7.0, 2.0, 8.0]], [[7.0, 5.0, 6.0], [1.0, 6.0, 
7.0], [1.0, 6.0, 5.0]]], [[[2.0, 8.0, 2.0], [8.0, 3.0, 2.0], [6.0, 7.0, 8.0]], [[4.0, 4.0, 1.0], [8.0, 3.0, 7.0], [5.0, 8.0, 
2.0]], [[4.0, 6.0, 6.0], [3.0, 8.0, 2.0], [6.0, 2.0, 1.0]]]]))
      val=numpy.array([[[2.0, 6.0, 4.0], [4.0, 1.0, 5.0]], [[7.0, 8.0, 2.0], [3.0, 2.0, 1.0]]])
      arg[2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 8.0], [7.0, 2.0, 8.0], [1.0, 6.0, 4.0]], [[3.0, 3.0, 2.0], [1.0, 7.0, 2.0], [5.0, 
2.0, 2.0]], [[5.0, 6.0, 7.0], [6.0, 4.0, 2.0], [5.0, 2.0, 7.0]]], [[[3.0, 2.0, 2.0], [7.0, 2.0, 5.0], [3.0, 7.0, 8.0]], [[8.0, 
2.0, 8.0], [8.0, 3.0, 5.0], [2.0, 2.0, 1.0]], [[1.0, 4.0, 7.0], [1.0, 4.0, 2.0], [2.0, 1.0, 2.0]]], [[[6.0, 1.0, 5.0], [2.0, 
4.0, 1.0], [7.0, 2.0, 4.0]], [[2.0, 6.0, 4.0], [4.0, 1.0, 5.0], [8.0, 4.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 4.0, 2.0], [8.0, 8.0, 
2.0]]], [[[2.0, 7.0, 5.0], [7.0, 1.0, 5.0], [6.0, 1.0, 2.0]], [[7.0, 8.0, 2.0], [3.0, 2.0, 1.0], [5.0, 1.0, 8.0]], [[6.0, 3.0, 
4.0], [5.0, 5.0, 1.0], [3.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 8.0], [5.0, 8.0, 7.0], [4.0, 2.0, 3.0]], 
[[2.0, 5.0, 4.0], [3.0, 1.0, 1.0], [7.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 2.0, 4.0], [6.0, 2.0, 3.0]]], [[[6.0, 7.0, 5.0], 
[8.0, 3.0, 2.0], [7.0, 8.0, 1.0]], [[5.0, 1.0, 2.0], [3.0, 3.0, 5.0], [2.0, 1.0, 4.0]], [[6.0, 3.0, 3.0], [4.0, 5.0, 3.0], 
[1.0, 4.0, 7.0]]], [[[4.0, 2.0, 6.0], [1.0, 6.0, 2.0], [6.0, 6.0, 4.0]], [[2.0, 6.0, 4.0], [4.0, 1.0, 5.0], [7.0, 2.0, 8.0]], 
[[7.0, 5.0, 6.0], [1.0, 6.0, 7.0], [1.0, 6.0, 5.0]]], [[[2.0, 8.0, 2.0], [8.0, 3.0, 2.0], [6.0, 7.0, 8.0]], [[7.0, 8.0, 2.0], 
[3.0, 2.0, 1.0], [5.0, 8.0, 2.0]], [[4.0, 6.0, 6.0], [3.0, 8.0, 2.0], [6.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LPUN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 8.0], [1.0, 2.0, 4.0], [5.0, 7.0, 1.0]], [[3.0, 8.0, 6.0], [2.0, 4.0, 2.0], [1.0, 8.0, 
5.0]], [[2.0, 3.0, 4.0], [4.0, 5.0, 3.0], [4.0, 8.0, 4.0]]], [[[4.0, 5.0, 6.0], [5.0, 2.0, 8.0], [5.0, 4.0, 6.0]], [[2.0, 7.0, 
2.0], [3.0, 4.0, 3.0], [7.0, 8.0, 8.0]], [[5.0, 2.0, 7.0], [3.0, 8.0, 3.0], [7.0, 5.0, 3.0]]], [[[1.0, 4.0, 6.0], [2.0, 1.0, 
3.0], [1.0, 3.0, 7.0]], [[2.0, 6.0, 3.0], [3.0, 6.0, 5.0], [6.0, 6.0, 2.0]], [[4.0, 1.0, 5.0], [5.0, 5.0, 2.0], [8.0, 7.0, 
4.0]]], [[[1.0, 2.0, 5.0], [1.0, 2.0, 3.0], [6.0, 1.0, 7.0]], [[3.0, 1.0, 4.0], [6.0, 4.0, 3.0], [2.0, 3.0, 6.0]], [[7.0, 1.0, 
3.0], [2.0, 1.0, 7.0], [1.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 4.0], [3.0, 8.0, 6.0], [5.0, 6.0, 1.0]], [[4.0, 2.0, 6.0], [4.0, 2.0, 
7.0], [8.0, 8.0, 7.0]], [[4.0, 2.0, 2.0], [2.0, 4.0, 2.0], [4.0, 2.0, 8.0]]], [[[3.0, 5.0, 5.0], [7.0, 3.0, 4.0], [5.0, 7.0, 
2.0]], [[2.0, 1.0, 8.0], [6.0, 6.0, 4.0], [7.0, 7.0, 7.0]], [[6.0, 6.0, 4.0], [6.0, 7.0, 3.0], [6.0, 3.0, 2.0]]], [[[8.0, 8.0, 
4.0], [2.0, 6.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 3.0, 3.0], [4.0, 5.0, 7.0], [8.0, 3.0, 7.0]], [[2.0, 3.0, 2.0], [3.0, 8.0, 
1.0], [7.0, 5.0, 5.0]]], [[[1.0, 2.0, 2.0], [2.0, 5.0, 2.0], [2.0, 5.0, 1.0]], [[4.0, 4.0, 3.0], [6.0, 3.0, 3.0], [4.0, 4.0, 
5.0]], [[3.0, 6.0, 7.0], [4.0, 7.0, 6.0], [8.0, 8.0, 4.0]]]]))
      val=Data(numpy.array([[[3.0, 7.0, 3.0], [5.0, 5.0, 4.0]], [[1.0, 1.0, 3.0], [1.0, 3.0, 7.0]]]),self.functionspace)
      arg[2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 8.0], [1.0, 2.0, 4.0], [5.0, 7.0, 1.0]], [[3.0, 8.0, 6.0], [2.0, 4.0, 2.0], [1.0, 
8.0, 5.0]], [[2.0, 3.0, 4.0], [4.0, 5.0, 3.0], [4.0, 8.0, 4.0]]], [[[4.0, 5.0, 6.0], [5.0, 2.0, 8.0], [5.0, 4.0, 6.0]], [[2.0, 
7.0, 2.0], [3.0, 4.0, 3.0], [7.0, 8.0, 8.0]], [[5.0, 2.0, 7.0], [3.0, 8.0, 3.0], [7.0, 5.0, 3.0]]], [[[1.0, 4.0, 6.0], [2.0, 
1.0, 3.0], [1.0, 3.0, 7.0]], [[3.0, 7.0, 3.0], [5.0, 5.0, 4.0], [6.0, 6.0, 2.0]], [[4.0, 1.0, 5.0], [5.0, 5.0, 2.0], [8.0, 7.0, 
4.0]]], [[[1.0, 2.0, 5.0], [1.0, 2.0, 3.0], [6.0, 1.0, 7.0]], [[1.0, 1.0, 3.0], [1.0, 3.0, 7.0], [2.0, 3.0, 6.0]], [[7.0, 1.0, 
3.0], [2.0, 1.0, 7.0], [1.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 4.0], [3.0, 8.0, 6.0], [5.0, 6.0, 1.0]], 
[[4.0, 2.0, 6.0], [4.0, 2.0, 7.0], [8.0, 8.0, 7.0]], [[4.0, 2.0, 2.0], [2.0, 4.0, 2.0], [4.0, 2.0, 8.0]]], [[[3.0, 5.0, 5.0], 
[7.0, 3.0, 4.0], [5.0, 7.0, 2.0]], [[2.0, 1.0, 8.0], [6.0, 6.0, 4.0], [7.0, 7.0, 7.0]], [[6.0, 6.0, 4.0], [6.0, 7.0, 3.0], 
[6.0, 3.0, 2.0]]], [[[8.0, 8.0, 4.0], [2.0, 6.0, 5.0], [2.0, 2.0, 4.0]], [[3.0, 7.0, 3.0], [5.0, 5.0, 4.0], [8.0, 3.0, 7.0]], 
[[2.0, 3.0, 2.0], [3.0, 8.0, 1.0], [7.0, 5.0, 5.0]]], [[[1.0, 2.0, 2.0], [2.0, 5.0, 2.0], [2.0, 5.0, 1.0]], [[1.0, 1.0, 3.0], 
[1.0, 3.0, 7.0], [4.0, 4.0, 5.0]], [[3.0, 6.0, 7.0], [4.0, 7.0, 6.0], [8.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LPUN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 4.0], [6.0, 1.0, 6.0], [2.0, 2.0, 5.0]], [[5.0, 5.0, 8.0], [1.0, 5.0, 5.0], [4.0, 7.0, 
3.0]], [[2.0, 6.0, 5.0], [7.0, 8.0, 4.0], [1.0, 6.0, 3.0]]], [[[4.0, 6.0, 1.0], [4.0, 2.0, 8.0], [3.0, 3.0, 4.0]], [[8.0, 5.0, 
1.0], [4.0, 3.0, 5.0], [6.0, 4.0, 4.0]], [[3.0, 2.0, 5.0], [1.0, 4.0, 1.0], [5.0, 1.0, 1.0]]], [[[6.0, 7.0, 2.0], [8.0, 8.0, 
8.0], [4.0, 6.0, 3.0]], [[6.0, 3.0, 4.0], [6.0, 2.0, 8.0], [6.0, 2.0, 1.0]], [[8.0, 3.0, 8.0], [6.0, 7.0, 4.0], [4.0, 1.0, 
2.0]]], [[[7.0, 6.0, 7.0], [2.0, 5.0, 6.0], [3.0, 6.0, 8.0]], [[5.0, 4.0, 8.0], [2.0, 2.0, 1.0], [7.0, 3.0, 1.0]], [[5.0, 1.0, 
5.0], [6.0, 2.0, 2.0], [2.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 2.0], [6.0, 8.0, 1.0], [2.0, 5.0, 7.0]], [[3.0, 5.0, 8.0], [8.0, 5.0, 
6.0], [5.0, 4.0, 7.0]], [[1.0, 2.0, 1.0], [7.0, 8.0, 3.0], [5.0, 5.0, 8.0]]], [[[4.0, 5.0, 8.0], [7.0, 5.0, 8.0], [3.0, 8.0, 
3.0]], [[5.0, 5.0, 8.0], [6.0, 2.0, 5.0], [3.0, 6.0, 4.0]], [[4.0, 1.0, 6.0], [7.0, 8.0, 1.0], [6.0, 2.0, 6.0]]], [[[7.0, 6.0, 
8.0], [2.0, 6.0, 4.0], [5.0, 6.0, 4.0]], [[3.0, 2.0, 6.0], [2.0, 6.0, 2.0], [2.0, 4.0, 7.0]], [[6.0, 2.0, 3.0], [1.0, 3.0, 
2.0], [4.0, 2.0, 1.0]]], [[[6.0, 8.0, 1.0], [3.0, 4.0, 3.0], [2.0, 5.0, 3.0]], [[6.0, 6.0, 4.0], [3.0, 7.0, 6.0], [7.0, 3.0, 
4.0]], [[3.0, 3.0, 6.0], [6.0, 4.0, 6.0], [2.0, 8.0, 8.0]]]]))
      val=Data(numpy.array([[[8.0, 6.0, 4.0], [4.0, 2.0, 8.0]], [[2.0, 1.0, 3.0], [5.0, 5.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 4.0, 2.0], [3.0, 8.0, 3.0]], [[4.0, 3.0, 5.0], [7.0, 1.0, 1.0]]]))
      arg[2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 4.0], [6.0, 1.0, 6.0], [2.0, 2.0, 5.0]], [[5.0, 5.0, 8.0], [1.0, 5.0, 5.0], [4.0, 
7.0, 3.0]], [[2.0, 6.0, 5.0], [7.0, 8.0, 4.0], [1.0, 6.0, 3.0]]], [[[4.0, 6.0, 1.0], [4.0, 2.0, 8.0], [3.0, 3.0, 4.0]], [[8.0, 
5.0, 1.0], [4.0, 3.0, 5.0], [6.0, 4.0, 4.0]], [[3.0, 2.0, 5.0], [1.0, 4.0, 1.0], [5.0, 1.0, 1.0]]], [[[6.0, 7.0, 2.0], [8.0, 
8.0, 8.0], [4.0, 6.0, 3.0]], [[8.0, 6.0, 4.0], [4.0, 2.0, 8.0], [6.0, 2.0, 1.0]], [[8.0, 3.0, 8.0], [6.0, 7.0, 4.0], [4.0, 1.0, 
2.0]]], [[[7.0, 6.0, 7.0], [2.0, 5.0, 6.0], [3.0, 6.0, 8.0]], [[2.0, 1.0, 3.0], [5.0, 5.0, 1.0], [7.0, 3.0, 1.0]], [[5.0, 1.0, 
5.0], [6.0, 2.0, 2.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 2.0], [6.0, 8.0, 1.0], [2.0, 5.0, 7.0]], 
[[3.0, 5.0, 8.0], [8.0, 5.0, 6.0], [5.0, 4.0, 7.0]], [[1.0, 2.0, 1.0], [7.0, 8.0, 3.0], [5.0, 5.0, 8.0]]], [[[4.0, 5.0, 8.0], 
[7.0, 5.0, 8.0], [3.0, 8.0, 3.0]], [[5.0, 5.0, 8.0], [6.0, 2.0, 5.0], [3.0, 6.0, 4.0]], [[4.0, 1.0, 6.0], [7.0, 8.0, 1.0], 
[6.0, 2.0, 6.0]]], [[[7.0, 6.0, 8.0], [2.0, 6.0, 4.0], [5.0, 6.0, 4.0]], [[4.0, 4.0, 2.0], [3.0, 8.0, 3.0], [2.0, 4.0, 7.0]], 
[[6.0, 2.0, 3.0], [1.0, 3.0, 2.0], [4.0, 2.0, 1.0]]], [[[6.0, 8.0, 1.0], [3.0, 4.0, 3.0], [2.0, 5.0, 3.0]], [[4.0, 3.0, 5.0], 
[7.0, 1.0, 1.0], [7.0, 3.0, 4.0]], [[3.0, 3.0, 6.0], [6.0, 4.0, 6.0], [2.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LPUN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 3.0], [3.0, 2.0, 4.0], [4.0, 8.0, 1.0]], [[5.0, 5.0, 2.0], [2.0, 8.0, 6.0], [7.0, 4.0, 
8.0]], [[8.0, 2.0, 2.0], [3.0, 7.0, 6.0], [4.0, 2.0, 2.0]]], [[[1.0, 3.0, 4.0], [8.0, 1.0, 6.0], [3.0, 4.0, 1.0]], [[6.0, 6.0, 
1.0], [1.0, 2.0, 7.0], [5.0, 6.0, 6.0]], [[8.0, 7.0, 6.0], [2.0, 8.0, 1.0], [4.0, 5.0, 1.0]]], [[[1.0, 5.0, 4.0], [3.0, 6.0, 
3.0], [8.0, 5.0, 4.0]], [[1.0, 6.0, 6.0], [2.0, 4.0, 3.0], [1.0, 4.0, 3.0]], [[7.0, 6.0, 7.0], [2.0, 7.0, 2.0], [6.0, 5.0, 
7.0]]], [[[8.0, 1.0, 5.0], [5.0, 1.0, 3.0], [6.0, 2.0, 8.0]], [[7.0, 2.0, 6.0], [4.0, 4.0, 6.0], [2.0, 2.0, 6.0]], [[8.0, 5.0, 
5.0], [4.0, 2.0, 6.0], [6.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 6.0], [8.0, 1.0, 6.0], [4.0, 4.0, 7.0]], [[4.0, 2.0, 3.0], [7.0, 2.0, 
3.0], [3.0, 8.0, 8.0]], [[1.0, 8.0, 4.0], [3.0, 6.0, 1.0], [1.0, 4.0, 4.0]]], [[[7.0, 7.0, 3.0], [8.0, 7.0, 4.0], [8.0, 3.0, 
5.0]], [[3.0, 7.0, 3.0], [3.0, 3.0, 2.0], [2.0, 5.0, 7.0]], [[6.0, 4.0, 6.0], [1.0, 1.0, 3.0], [5.0, 7.0, 2.0]]], [[[3.0, 7.0, 
7.0], [7.0, 1.0, 8.0], [2.0, 5.0, 8.0]], [[4.0, 4.0, 6.0], [5.0, 2.0, 8.0], [5.0, 2.0, 8.0]], [[7.0, 4.0, 1.0], [3.0, 8.0, 
8.0], [4.0, 8.0, 7.0]]], [[[2.0, 4.0, 5.0], [2.0, 7.0, 6.0], [1.0, 3.0, 7.0]], [[3.0, 3.0, 2.0], [7.0, 1.0, 1.0], [5.0, 4.0, 
6.0]], [[2.0, 3.0, 4.0], [1.0, 5.0, 7.0], [5.0, 5.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 2.0, 1.0], [8.0, 4.0, 6.0]], [[8.0, 6.0, 2.0], [1.0, 6.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[2.0, 8.0, 3.0], [5.0, 4.0, 3.0]], [[1.0, 7.0, 3.0], [6.0, 7.0, 4.0]]])
      arg[2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 3.0], [3.0, 2.0, 4.0], [4.0, 8.0, 1.0]], [[5.0, 5.0, 2.0], [2.0, 8.0, 6.0], [7.0, 
4.0, 8.0]], [[8.0, 2.0, 2.0], [3.0, 7.0, 6.0], [4.0, 2.0, 2.0]]], [[[1.0, 3.0, 4.0], [8.0, 1.0, 6.0], [3.0, 4.0, 1.0]], [[6.0, 
6.0, 1.0], [1.0, 2.0, 7.0], [5.0, 6.0, 6.0]], [[8.0, 7.0, 6.0], [2.0, 8.0, 1.0], [4.0, 5.0, 1.0]]], [[[1.0, 5.0, 4.0], [3.0, 
6.0, 3.0], [8.0, 5.0, 4.0]], [[8.0, 2.0, 1.0], [8.0, 4.0, 6.0], [1.0, 4.0, 3.0]], [[7.0, 6.0, 7.0], [2.0, 7.0, 2.0], [6.0, 5.0, 
7.0]]], [[[8.0, 1.0, 5.0], [5.0, 1.0, 3.0], [6.0, 2.0, 8.0]], [[8.0, 6.0, 2.0], [1.0, 6.0, 1.0], [2.0, 2.0, 6.0]], [[8.0, 5.0, 
5.0], [4.0, 2.0, 6.0], [6.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 6.0], [8.0, 1.0, 6.0], [4.0, 4.0, 7.0]], 
[[4.0, 2.0, 3.0], [7.0, 2.0, 3.0], [3.0, 8.0, 8.0]], [[1.0, 8.0, 4.0], [3.0, 6.0, 1.0], [1.0, 4.0, 4.0]]], [[[7.0, 7.0, 3.0], 
[8.0, 7.0, 4.0], [8.0, 3.0, 5.0]], [[3.0, 7.0, 3.0], [3.0, 3.0, 2.0], [2.0, 5.0, 7.0]], [[6.0, 4.0, 6.0], [1.0, 1.0, 3.0], 
[5.0, 7.0, 2.0]]], [[[3.0, 7.0, 7.0], [7.0, 1.0, 8.0], [2.0, 5.0, 8.0]], [[2.0, 8.0, 3.0], [5.0, 4.0, 3.0], [5.0, 2.0, 8.0]], 
[[7.0, 4.0, 1.0], [3.0, 8.0, 8.0], [4.0, 8.0, 7.0]]], [[[2.0, 4.0, 5.0], [2.0, 7.0, 6.0], [1.0, 3.0, 7.0]], [[1.0, 7.0, 3.0], 
[6.0, 7.0, 4.0], [5.0, 4.0, 6.0]], [[2.0, 3.0, 4.0], [1.0, 5.0, 7.0], [5.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LRPN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 5.0], [5.0, 4.0, 5.0], [3.0, 5.0, 4.0]], [[3.0, 6.0, 8.0], [6.0, 8.0, 1.0], [6.0, 3.0, 
3.0]], [[7.0, 8.0, 2.0], [1.0, 8.0, 4.0], [5.0, 7.0, 4.0]]], [[[3.0, 8.0, 2.0], [6.0, 1.0, 2.0], [5.0, 8.0, 1.0]], [[4.0, 1.0, 
1.0], [6.0, 4.0, 3.0], [2.0, 3.0, 4.0]], [[4.0, 8.0, 5.0], [4.0, 5.0, 2.0], [8.0, 3.0, 1.0]]], [[[4.0, 8.0, 5.0], [1.0, 4.0, 
1.0], [4.0, 1.0, 2.0]], [[2.0, 8.0, 7.0], [2.0, 2.0, 6.0], [6.0, 1.0, 8.0]], [[8.0, 8.0, 5.0], [5.0, 8.0, 7.0], [5.0, 8.0, 
5.0]]], [[[2.0, 8.0, 7.0], [5.0, 3.0, 3.0], [3.0, 1.0, 8.0]], [[3.0, 3.0, 1.0], [1.0, 6.0, 4.0], [6.0, 2.0, 7.0]], [[3.0, 8.0, 
3.0], [6.0, 8.0, 2.0], [2.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 8.0], [2.0, 7.0, 5.0], [7.0, 1.0, 6.0]], [[5.0, 8.0, 2.0], [6.0, 3.0, 
5.0], [1.0, 8.0, 2.0]], [[2.0, 2.0, 4.0], [1.0, 4.0, 8.0], [6.0, 4.0, 8.0]]], [[[7.0, 3.0, 2.0], [8.0, 2.0, 6.0], [6.0, 2.0, 
7.0]], [[7.0, 4.0, 7.0], [3.0, 2.0, 3.0], [2.0, 5.0, 4.0]], [[3.0, 6.0, 4.0], [5.0, 5.0, 1.0], [4.0, 8.0, 1.0]]], [[[3.0, 4.0, 
2.0], [3.0, 4.0, 2.0], [7.0, 2.0, 4.0]], [[7.0, 5.0, 4.0], [5.0, 2.0, 4.0], [5.0, 8.0, 2.0]], [[4.0, 2.0, 8.0], [7.0, 6.0, 
5.0], [3.0, 5.0, 1.0]]], [[[5.0, 2.0, 3.0], [2.0, 8.0, 7.0], [4.0, 2.0, 2.0]], [[2.0, 1.0, 4.0], [7.0, 1.0, 4.0], [1.0, 1.0, 
1.0]], [[5.0, 3.0, 5.0], [8.0, 6.0, 3.0], [6.0, 7.0, 4.0]]]]))
      val=numpy.array([[[2.0, 6.0, 6.0]], [[8.0, 7.0, 3.0]]])
      arg[2:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 5.0], [5.0, 4.0, 5.0], [3.0, 5.0, 4.0]], [[3.0, 6.0, 8.0], [6.0, 8.0, 1.0], [6.0, 
3.0, 3.0]], [[7.0, 8.0, 2.0], [1.0, 8.0, 4.0], [5.0, 7.0, 4.0]]], [[[3.0, 8.0, 2.0], [6.0, 1.0, 2.0], [5.0, 8.0, 1.0]], [[4.0, 
1.0, 1.0], [6.0, 4.0, 3.0], [2.0, 3.0, 4.0]], [[4.0, 8.0, 5.0], [4.0, 5.0, 2.0], [8.0, 3.0, 1.0]]], [[[4.0, 8.0, 5.0], [1.0, 
4.0, 1.0], [4.0, 1.0, 2.0]], [[2.0, 8.0, 7.0], [2.0, 6.0, 6.0], [6.0, 1.0, 8.0]], [[8.0, 8.0, 5.0], [5.0, 8.0, 7.0], [5.0, 8.0, 
5.0]]], [[[2.0, 8.0, 7.0], [5.0, 3.0, 3.0], [3.0, 1.0, 8.0]], [[3.0, 3.0, 1.0], [8.0, 7.0, 3.0], [6.0, 2.0, 7.0]], [[3.0, 8.0, 
3.0], [6.0, 8.0, 2.0], [2.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 8.0], [2.0, 7.0, 5.0], [7.0, 1.0, 6.0]], 
[[5.0, 8.0, 2.0], [6.0, 3.0, 5.0], [1.0, 8.0, 2.0]], [[2.0, 2.0, 4.0], [1.0, 4.0, 8.0], [6.0, 4.0, 8.0]]], [[[7.0, 3.0, 2.0], 
[8.0, 2.0, 6.0], [6.0, 2.0, 7.0]], [[7.0, 4.0, 7.0], [3.0, 2.0, 3.0], [2.0, 5.0, 4.0]], [[3.0, 6.0, 4.0], [5.0, 5.0, 1.0], 
[4.0, 8.0, 1.0]]], [[[3.0, 4.0, 2.0], [3.0, 4.0, 2.0], [7.0, 2.0, 4.0]], [[7.0, 5.0, 4.0], [2.0, 6.0, 6.0], [5.0, 8.0, 2.0]], 
[[4.0, 2.0, 8.0], [7.0, 6.0, 5.0], [3.0, 5.0, 1.0]]], [[[5.0, 2.0, 3.0], [2.0, 8.0, 7.0], [4.0, 2.0, 2.0]], [[2.0, 1.0, 4.0], 
[8.0, 7.0, 3.0], [1.0, 1.0, 1.0]], [[5.0, 3.0, 5.0], [8.0, 6.0, 3.0], [6.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LRPN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 6.0], [6.0, 7.0, 6.0], [8.0, 1.0, 3.0]], [[6.0, 8.0, 8.0], [3.0, 6.0, 2.0], [7.0, 5.0, 
2.0]], [[5.0, 3.0, 1.0], [7.0, 4.0, 8.0], [3.0, 3.0, 4.0]]], [[[8.0, 7.0, 7.0], [5.0, 6.0, 1.0], [3.0, 8.0, 3.0]], [[8.0, 7.0, 
6.0], [5.0, 6.0, 8.0], [8.0, 5.0, 4.0]], [[4.0, 7.0, 8.0], [1.0, 8.0, 3.0], [7.0, 7.0, 7.0]]], [[[1.0, 6.0, 7.0], [5.0, 3.0, 
4.0], [4.0, 6.0, 6.0]], [[3.0, 5.0, 1.0], [8.0, 2.0, 4.0], [4.0, 3.0, 4.0]], [[5.0, 7.0, 7.0], [8.0, 3.0, 6.0], [7.0, 2.0, 
1.0]]], [[[1.0, 3.0, 1.0], [6.0, 5.0, 8.0], [4.0, 3.0, 7.0]], [[3.0, 5.0, 3.0], [8.0, 2.0, 3.0], [6.0, 3.0, 6.0]], [[7.0, 8.0, 
1.0], [3.0, 1.0, 2.0], [3.0, 7.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 3.0], [2.0, 7.0, 2.0], [7.0, 1.0, 7.0]], [[5.0, 7.0, 5.0], [5.0, 6.0, 
3.0], [7.0, 8.0, 4.0]], [[4.0, 4.0, 1.0], [1.0, 1.0, 1.0], [6.0, 2.0, 7.0]]], [[[7.0, 6.0, 1.0], [7.0, 8.0, 2.0], [3.0, 4.0, 
2.0]], [[6.0, 3.0, 1.0], [7.0, 6.0, 8.0], [7.0, 7.0, 8.0]], [[1.0, 3.0, 2.0], [3.0, 2.0, 3.0], [7.0, 6.0, 3.0]]], [[[3.0, 1.0, 
4.0], [6.0, 4.0, 1.0], [7.0, 3.0, 8.0]], [[7.0, 5.0, 2.0], [5.0, 5.0, 3.0], [4.0, 3.0, 4.0]], [[7.0, 4.0, 7.0], [7.0, 4.0, 
5.0], [8.0, 6.0, 1.0]]], [[[7.0, 5.0, 7.0], [4.0, 1.0, 4.0], [4.0, 2.0, 7.0]], [[7.0, 2.0, 4.0], [6.0, 1.0, 5.0], [7.0, 6.0, 
3.0]], [[2.0, 2.0, 1.0], [5.0, 8.0, 2.0], [6.0, 1.0, 7.0]]]]))
      val=Data(numpy.array([[[3.0, 5.0, 1.0]], [[8.0, 8.0, 1.0]]]),self.functionspace)
      arg[2:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 6.0], [6.0, 7.0, 6.0], [8.0, 1.0, 3.0]], [[6.0, 8.0, 8.0], [3.0, 6.0, 2.0], [7.0, 
5.0, 2.0]], [[5.0, 3.0, 1.0], [7.0, 4.0, 8.0], [3.0, 3.0, 4.0]]], [[[8.0, 7.0, 7.0], [5.0, 6.0, 1.0], [3.0, 8.0, 3.0]], [[8.0, 
7.0, 6.0], [5.0, 6.0, 8.0], [8.0, 5.0, 4.0]], [[4.0, 7.0, 8.0], [1.0, 8.0, 3.0], [7.0, 7.0, 7.0]]], [[[1.0, 6.0, 7.0], [5.0, 
3.0, 4.0], [4.0, 6.0, 6.0]], [[3.0, 5.0, 1.0], [3.0, 5.0, 1.0], [4.0, 3.0, 4.0]], [[5.0, 7.0, 7.0], [8.0, 3.0, 6.0], [7.0, 2.0, 
1.0]]], [[[1.0, 3.0, 1.0], [6.0, 5.0, 8.0], [4.0, 3.0, 7.0]], [[3.0, 5.0, 3.0], [8.0, 8.0, 1.0], [6.0, 3.0, 6.0]], [[7.0, 8.0, 
1.0], [3.0, 1.0, 2.0], [3.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 3.0], [2.0, 7.0, 2.0], [7.0, 1.0, 7.0]], 
[[5.0, 7.0, 5.0], [5.0, 6.0, 3.0], [7.0, 8.0, 4.0]], [[4.0, 4.0, 1.0], [1.0, 1.0, 1.0], [6.0, 2.0, 7.0]]], [[[7.0, 6.0, 1.0], 
[7.0, 8.0, 2.0], [3.0, 4.0, 2.0]], [[6.0, 3.0, 1.0], [7.0, 6.0, 8.0], [7.0, 7.0, 8.0]], [[1.0, 3.0, 2.0], [3.0, 2.0, 3.0], 
[7.0, 6.0, 3.0]]], [[[3.0, 1.0, 4.0], [6.0, 4.0, 1.0], [7.0, 3.0, 8.0]], [[7.0, 5.0, 2.0], [3.0, 5.0, 1.0], [4.0, 3.0, 4.0]], 
[[7.0, 4.0, 7.0], [7.0, 4.0, 5.0], [8.0, 6.0, 1.0]]], [[[7.0, 5.0, 7.0], [4.0, 1.0, 4.0], [4.0, 2.0, 7.0]], [[7.0, 2.0, 4.0], 
[8.0, 8.0, 1.0], [7.0, 6.0, 3.0]], [[2.0, 2.0, 1.0], [5.0, 8.0, 2.0], [6.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LRPN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 6.0], [1.0, 2.0, 7.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 4.0], [1.0, 1.0, 4.0], [3.0, 3.0, 
8.0]], [[6.0, 4.0, 7.0], [5.0, 7.0, 8.0], [5.0, 5.0, 3.0]]], [[[5.0, 5.0, 6.0], [3.0, 3.0, 8.0], [4.0, 3.0, 5.0]], [[2.0, 4.0, 
7.0], [8.0, 1.0, 4.0], [4.0, 1.0, 5.0]], [[5.0, 7.0, 1.0], [4.0, 2.0, 4.0], [1.0, 7.0, 8.0]]], [[[1.0, 2.0, 8.0], [5.0, 2.0, 
2.0], [2.0, 3.0, 4.0]], [[8.0, 5.0, 6.0], [6.0, 6.0, 4.0], [3.0, 5.0, 7.0]], [[2.0, 7.0, 5.0], [2.0, 1.0, 2.0], [8.0, 3.0, 
8.0]]], [[[5.0, 3.0, 8.0], [3.0, 2.0, 4.0], [4.0, 8.0, 7.0]], [[8.0, 8.0, 6.0], [8.0, 5.0, 1.0], [6.0, 1.0, 7.0]], [[7.0, 4.0, 
3.0], [1.0, 5.0, 2.0], [4.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 2.0], [4.0, 2.0, 7.0], [5.0, 1.0, 5.0]], [[2.0, 5.0, 6.0], [3.0, 3.0, 
4.0], [8.0, 4.0, 5.0]], [[4.0, 7.0, 4.0], [5.0, 6.0, 6.0], [6.0, 2.0, 2.0]]], [[[6.0, 5.0, 8.0], [1.0, 3.0, 4.0], [5.0, 1.0, 
1.0]], [[6.0, 2.0, 6.0], [3.0, 1.0, 2.0], [3.0, 4.0, 3.0]], [[3.0, 8.0, 3.0], [6.0, 3.0, 7.0], [2.0, 8.0, 3.0]]], [[[7.0, 8.0, 
3.0], [2.0, 8.0, 4.0], [2.0, 5.0, 6.0]], [[5.0, 6.0, 2.0], [5.0, 2.0, 6.0], [7.0, 7.0, 8.0]], [[7.0, 6.0, 8.0], [2.0, 5.0, 
7.0], [4.0, 4.0, 6.0]]], [[[8.0, 6.0, 3.0], [3.0, 5.0, 3.0], [2.0, 5.0, 1.0]], [[5.0, 2.0, 5.0], [4.0, 1.0, 8.0], [7.0, 4.0, 
1.0]], [[4.0, 8.0, 2.0], [2.0, 7.0, 8.0], [8.0, 3.0, 2.0]]]]))
      val=Data(numpy.array([[[5.0, 5.0, 2.0]], [[3.0, 5.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 7.0, 1.0]], [[8.0, 7.0, 7.0]]]))
      arg[2:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 6.0], [1.0, 2.0, 7.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 4.0], [1.0, 1.0, 4.0], [3.0, 
3.0, 8.0]], [[6.0, 4.0, 7.0], [5.0, 7.0, 8.0], [5.0, 5.0, 3.0]]], [[[5.0, 5.0, 6.0], [3.0, 3.0, 8.0], [4.0, 3.0, 5.0]], [[2.0, 
4.0, 7.0], [8.0, 1.0, 4.0], [4.0, 1.0, 5.0]], [[5.0, 7.0, 1.0], [4.0, 2.0, 4.0], [1.0, 7.0, 8.0]]], [[[1.0, 2.0, 8.0], [5.0, 
2.0, 2.0], [2.0, 3.0, 4.0]], [[8.0, 5.0, 6.0], [5.0, 5.0, 2.0], [3.0, 5.0, 7.0]], [[2.0, 7.0, 5.0], [2.0, 1.0, 2.0], [8.0, 3.0, 
8.0]]], [[[5.0, 3.0, 8.0], [3.0, 2.0, 4.0], [4.0, 8.0, 7.0]], [[8.0, 8.0, 6.0], [3.0, 5.0, 2.0], [6.0, 1.0, 7.0]], [[7.0, 4.0, 
3.0], [1.0, 5.0, 2.0], [4.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 2.0], [4.0, 2.0, 7.0], [5.0, 1.0, 5.0]], 
[[2.0, 5.0, 6.0], [3.0, 3.0, 4.0], [8.0, 4.0, 5.0]], [[4.0, 7.0, 4.0], [5.0, 6.0, 6.0], [6.0, 2.0, 2.0]]], [[[6.0, 5.0, 8.0], 
[1.0, 3.0, 4.0], [5.0, 1.0, 1.0]], [[6.0, 2.0, 6.0], [3.0, 1.0, 2.0], [3.0, 4.0, 3.0]], [[3.0, 8.0, 3.0], [6.0, 3.0, 7.0], 
[2.0, 8.0, 3.0]]], [[[7.0, 8.0, 3.0], [2.0, 8.0, 4.0], [2.0, 5.0, 6.0]], [[5.0, 6.0, 2.0], [5.0, 7.0, 1.0], [7.0, 7.0, 8.0]], 
[[7.0, 6.0, 8.0], [2.0, 5.0, 7.0], [4.0, 4.0, 6.0]]], [[[8.0, 6.0, 3.0], [3.0, 5.0, 3.0], [2.0, 5.0, 1.0]], [[5.0, 2.0, 5.0], 
[8.0, 7.0, 7.0], [7.0, 4.0, 1.0]], [[4.0, 8.0, 2.0], [2.0, 7.0, 8.0], [8.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LRPN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 6.0], [6.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[5.0, 8.0, 7.0], [5.0, 7.0, 5.0], [8.0, 4.0, 
2.0]], [[5.0, 7.0, 3.0], [7.0, 7.0, 4.0], [1.0, 4.0, 6.0]]], [[[5.0, 2.0, 7.0], [5.0, 1.0, 5.0], [5.0, 1.0, 6.0]], [[3.0, 4.0, 
2.0], [5.0, 7.0, 1.0], [4.0, 8.0, 4.0]], [[3.0, 7.0, 5.0], [4.0, 6.0, 6.0], [7.0, 6.0, 6.0]]], [[[8.0, 4.0, 5.0], [5.0, 6.0, 
7.0], [7.0, 7.0, 8.0]], [[8.0, 6.0, 6.0], [3.0, 4.0, 6.0], [3.0, 5.0, 6.0]], [[7.0, 8.0, 4.0], [6.0, 4.0, 2.0], [4.0, 1.0, 
5.0]]], [[[2.0, 1.0, 2.0], [1.0, 5.0, 3.0], [4.0, 7.0, 6.0]], [[8.0, 6.0, 8.0], [8.0, 7.0, 2.0], [4.0, 5.0, 7.0]], [[5.0, 6.0, 
1.0], [2.0, 8.0, 6.0], [2.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 4.0], [6.0, 1.0, 2.0], [4.0, 3.0, 8.0]], [[1.0, 3.0, 6.0], [5.0, 8.0, 
1.0], [6.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], [7.0, 2.0, 3.0], [2.0, 8.0, 3.0]]], [[[2.0, 8.0, 8.0], [3.0, 3.0, 3.0], [6.0, 2.0, 
7.0]], [[8.0, 7.0, 6.0], [3.0, 4.0, 7.0], [4.0, 3.0, 8.0]], [[5.0, 3.0, 5.0], [3.0, 7.0, 3.0], [2.0, 4.0, 2.0]]], [[[5.0, 4.0, 
8.0], [2.0, 1.0, 3.0], [2.0, 1.0, 5.0]], [[3.0, 8.0, 3.0], [6.0, 6.0, 4.0], [5.0, 2.0, 1.0]], [[8.0, 4.0, 4.0], [2.0, 8.0, 
3.0], [2.0, 5.0, 3.0]]], [[[1.0, 2.0, 6.0], [4.0, 4.0, 7.0], [3.0, 4.0, 7.0]], [[5.0, 4.0, 5.0], [8.0, 8.0, 2.0], [8.0, 6.0, 
7.0]], [[8.0, 4.0, 7.0], [6.0, 3.0, 8.0], [6.0, 2.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 4.0, 5.0]], [[2.0, 8.0, 7.0]]])+(1.-msk_val)*numpy.array([[[4.0, 8.0, 6.0]], [[8.0, 1.0, 
8.0]]])
      arg[2:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 6.0], [6.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[5.0, 8.0, 7.0], [5.0, 7.0, 5.0], [8.0, 
4.0, 2.0]], [[5.0, 7.0, 3.0], [7.0, 7.0, 4.0], [1.0, 4.0, 6.0]]], [[[5.0, 2.0, 7.0], [5.0, 1.0, 5.0], [5.0, 1.0, 6.0]], [[3.0, 
4.0, 2.0], [5.0, 7.0, 1.0], [4.0, 8.0, 4.0]], [[3.0, 7.0, 5.0], [4.0, 6.0, 6.0], [7.0, 6.0, 6.0]]], [[[8.0, 4.0, 5.0], [5.0, 
6.0, 7.0], [7.0, 7.0, 8.0]], [[8.0, 6.0, 6.0], [5.0, 4.0, 5.0], [3.0, 5.0, 6.0]], [[7.0, 8.0, 4.0], [6.0, 4.0, 2.0], [4.0, 1.0, 
5.0]]], [[[2.0, 1.0, 2.0], [1.0, 5.0, 3.0], [4.0, 7.0, 6.0]], [[8.0, 6.0, 8.0], [2.0, 8.0, 7.0], [4.0, 5.0, 7.0]], [[5.0, 6.0, 
1.0], [2.0, 8.0, 6.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 4.0], [6.0, 1.0, 2.0], [4.0, 3.0, 8.0]], 
[[1.0, 3.0, 6.0], [5.0, 8.0, 1.0], [6.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], [7.0, 2.0, 3.0], [2.0, 8.0, 3.0]]], [[[2.0, 8.0, 8.0], 
[3.0, 3.0, 3.0], [6.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], [3.0, 4.0, 7.0], [4.0, 3.0, 8.0]], [[5.0, 3.0, 5.0], [3.0, 7.0, 3.0], 
[2.0, 4.0, 2.0]]], [[[5.0, 4.0, 8.0], [2.0, 1.0, 3.0], [2.0, 1.0, 5.0]], [[3.0, 8.0, 3.0], [4.0, 8.0, 6.0], [5.0, 2.0, 1.0]], 
[[8.0, 4.0, 4.0], [2.0, 8.0, 3.0], [2.0, 5.0, 3.0]]], [[[1.0, 2.0, 6.0], [4.0, 4.0, 7.0], [3.0, 4.0, 7.0]], [[5.0, 4.0, 5.0], 
[8.0, 1.0, 8.0], [8.0, 6.0, 7.0]], [[8.0, 4.0, 7.0], [6.0, 3.0, 8.0], [6.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPPU(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 2.0], [8.0, 8.0, 3.0], [2.0, 4.0, 4.0]], [[5.0, 3.0, 3.0], [4.0, 4.0, 5.0], [1.0, 8.0, 
5.0]], [[2.0, 1.0, 8.0], [1.0, 8.0, 1.0], [4.0, 1.0, 7.0]]], [[[2.0, 8.0, 7.0], [8.0, 8.0, 8.0], [5.0, 1.0, 7.0]], [[4.0, 3.0, 
8.0], [6.0, 1.0, 3.0], [6.0, 2.0, 8.0]], [[3.0, 6.0, 8.0], [1.0, 1.0, 4.0], [5.0, 5.0, 5.0]]], [[[6.0, 4.0, 2.0], [7.0, 7.0, 
8.0], [6.0, 3.0, 6.0]], [[1.0, 6.0, 8.0], [4.0, 4.0, 2.0], [2.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [6.0, 8.0, 4.0], [5.0, 1.0, 
3.0]]], [[[5.0, 8.0, 3.0], [3.0, 7.0, 5.0], [5.0, 4.0, 2.0]], [[1.0, 1.0, 3.0], [6.0, 2.0, 3.0], [6.0, 7.0, 3.0]], [[7.0, 2.0, 
5.0], [1.0, 5.0, 8.0], [4.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 5.0], [7.0, 2.0, 3.0], [7.0, 1.0, 5.0]], [[8.0, 8.0, 3.0], [6.0, 1.0, 
1.0], [5.0, 4.0, 1.0]], [[5.0, 4.0, 4.0], [6.0, 1.0, 2.0], [7.0, 7.0, 6.0]]], [[[7.0, 1.0, 5.0], [8.0, 4.0, 8.0], [5.0, 5.0, 
1.0]], [[5.0, 6.0, 1.0], [2.0, 5.0, 1.0], [6.0, 8.0, 1.0]], [[2.0, 3.0, 4.0], [3.0, 5.0, 1.0], [8.0, 1.0, 7.0]]], [[[5.0, 6.0, 
2.0], [5.0, 1.0, 8.0], [7.0, 3.0, 5.0]], [[3.0, 7.0, 4.0], [2.0, 8.0, 5.0], [3.0, 7.0, 2.0]], [[3.0, 5.0, 7.0], [4.0, 4.0, 
7.0], [8.0, 4.0, 1.0]]], [[[5.0, 6.0, 1.0], [4.0, 6.0, 4.0], [3.0, 8.0, 5.0]], [[3.0, 7.0, 4.0], [1.0, 5.0, 8.0], [7.0, 2.0, 
6.0]], [[3.0, 8.0, 7.0], [1.0, 8.0, 5.0], [5.0, 2.0, 1.0]]]]))
      val=numpy.array([6.0, 3.0])
      arg[2,1,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 2.0], [8.0, 8.0, 3.0], [2.0, 4.0, 4.0]], [[5.0, 3.0, 3.0], [4.0, 4.0, 5.0], [1.0, 
8.0, 5.0]], [[2.0, 1.0, 8.0], [1.0, 8.0, 1.0], [4.0, 1.0, 7.0]]], [[[2.0, 8.0, 7.0], [8.0, 8.0, 8.0], [5.0, 1.0, 7.0]], [[4.0, 
3.0, 8.0], [6.0, 1.0, 3.0], [6.0, 2.0, 8.0]], [[3.0, 6.0, 8.0], [1.0, 1.0, 4.0], [5.0, 5.0, 5.0]]], [[[6.0, 4.0, 2.0], [7.0, 
7.0, 8.0], [6.0, 3.0, 6.0]], [[1.0, 6.0, 8.0], [6.0, 3.0, 2.0], [2.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [6.0, 8.0, 4.0], [5.0, 1.0, 
3.0]]], [[[5.0, 8.0, 3.0], [3.0, 7.0, 5.0], [5.0, 4.0, 2.0]], [[1.0, 1.0, 3.0], [6.0, 2.0, 3.0], [6.0, 7.0, 3.0]], [[7.0, 2.0, 
5.0], [1.0, 5.0, 8.0], [4.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 5.0], [7.0, 2.0, 3.0], [7.0, 1.0, 5.0]], 
[[8.0, 8.0, 3.0], [6.0, 1.0, 1.0], [5.0, 4.0, 1.0]], [[5.0, 4.0, 4.0], [6.0, 1.0, 2.0], [7.0, 7.0, 6.0]]], [[[7.0, 1.0, 5.0], 
[8.0, 4.0, 8.0], [5.0, 5.0, 1.0]], [[5.0, 6.0, 1.0], [2.0, 5.0, 1.0], [6.0, 8.0, 1.0]], [[2.0, 3.0, 4.0], [3.0, 5.0, 1.0], 
[8.0, 1.0, 7.0]]], [[[5.0, 6.0, 2.0], [5.0, 1.0, 8.0], [7.0, 3.0, 5.0]], [[3.0, 7.0, 4.0], [6.0, 3.0, 5.0], [3.0, 7.0, 2.0]], 
[[3.0, 5.0, 7.0], [4.0, 4.0, 7.0], [8.0, 4.0, 1.0]]], [[[5.0, 6.0, 1.0], [4.0, 6.0, 4.0], [3.0, 8.0, 5.0]], [[3.0, 7.0, 4.0], 
[1.0, 5.0, 8.0], [7.0, 2.0, 6.0]], [[3.0, 8.0, 7.0], [1.0, 8.0, 5.0], [5.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPPU(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 5.0], [5.0, 8.0, 1.0], [7.0, 3.0, 4.0]], [[1.0, 4.0, 4.0], [1.0, 6.0, 6.0], [4.0, 5.0, 
2.0]], [[3.0, 3.0, 3.0], [2.0, 2.0, 5.0], [4.0, 5.0, 6.0]]], [[[4.0, 6.0, 7.0], [8.0, 4.0, 2.0], [2.0, 4.0, 8.0]], [[6.0, 4.0, 
7.0], [4.0, 6.0, 5.0], [3.0, 3.0, 6.0]], [[7.0, 6.0, 8.0], [2.0, 5.0, 5.0], [3.0, 1.0, 4.0]]], [[[8.0, 4.0, 2.0], [1.0, 6.0, 
7.0], [5.0, 1.0, 6.0]], [[2.0, 1.0, 4.0], [8.0, 5.0, 5.0], [3.0, 2.0, 6.0]], [[7.0, 2.0, 7.0], [8.0, 6.0, 5.0], [5.0, 7.0, 
8.0]]], [[[4.0, 5.0, 4.0], [4.0, 1.0, 1.0], [8.0, 8.0, 1.0]], [[7.0, 2.0, 6.0], [1.0, 3.0, 4.0], [3.0, 6.0, 3.0]], [[2.0, 8.0, 
5.0], [1.0, 5.0, 3.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 7.0], [8.0, 1.0, 3.0], [3.0, 5.0, 2.0]], [[1.0, 4.0, 6.0], [7.0, 3.0, 
2.0], [5.0, 8.0, 4.0]], [[7.0, 6.0, 3.0], [1.0, 7.0, 1.0], [6.0, 2.0, 2.0]]], [[[7.0, 7.0, 7.0], [5.0, 8.0, 5.0], [6.0, 7.0, 
4.0]], [[4.0, 5.0, 8.0], [1.0, 1.0, 2.0], [4.0, 3.0, 6.0]], [[3.0, 2.0, 6.0], [5.0, 8.0, 6.0], [4.0, 5.0, 6.0]]], [[[7.0, 3.0, 
8.0], [1.0, 5.0, 2.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 5.0], [7.0, 6.0, 5.0], [7.0, 1.0, 4.0]], [[6.0, 5.0, 5.0], [5.0, 4.0, 
7.0], [7.0, 4.0, 5.0]]], [[[8.0, 6.0, 5.0], [6.0, 8.0, 1.0], [3.0, 3.0, 2.0]], [[6.0, 7.0, 4.0], [4.0, 5.0, 4.0], [6.0, 5.0, 
1.0]], [[4.0, 7.0, 2.0], [4.0, 7.0, 8.0], [8.0, 3.0, 8.0]]]]))
      val=Data(numpy.array([6.0, 6.0]),self.functionspace)
      arg[2,1,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 5.0], [5.0, 8.0, 1.0], [7.0, 3.0, 4.0]], [[1.0, 4.0, 4.0], [1.0, 6.0, 6.0], [4.0, 
5.0, 2.0]], [[3.0, 3.0, 3.0], [2.0, 2.0, 5.0], [4.0, 5.0, 6.0]]], [[[4.0, 6.0, 7.0], [8.0, 4.0, 2.0], [2.0, 4.0, 8.0]], [[6.0, 
4.0, 7.0], [4.0, 6.0, 5.0], [3.0, 3.0, 6.0]], [[7.0, 6.0, 8.0], [2.0, 5.0, 5.0], [3.0, 1.0, 4.0]]], [[[8.0, 4.0, 2.0], [1.0, 
6.0, 7.0], [5.0, 1.0, 6.0]], [[2.0, 1.0, 4.0], [6.0, 6.0, 5.0], [3.0, 2.0, 6.0]], [[7.0, 2.0, 7.0], [8.0, 6.0, 5.0], [5.0, 7.0, 
8.0]]], [[[4.0, 5.0, 4.0], [4.0, 1.0, 1.0], [8.0, 8.0, 1.0]], [[7.0, 2.0, 6.0], [1.0, 3.0, 4.0], [3.0, 6.0, 3.0]], [[2.0, 8.0, 
5.0], [1.0, 5.0, 3.0], [4.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 7.0], [8.0, 1.0, 3.0], [3.0, 5.0, 2.0]], 
[[1.0, 4.0, 6.0], [7.0, 3.0, 2.0], [5.0, 8.0, 4.0]], [[7.0, 6.0, 3.0], [1.0, 7.0, 1.0], [6.0, 2.0, 2.0]]], [[[7.0, 7.0, 7.0], 
[5.0, 8.0, 5.0], [6.0, 7.0, 4.0]], [[4.0, 5.0, 8.0], [1.0, 1.0, 2.0], [4.0, 3.0, 6.0]], [[3.0, 2.0, 6.0], [5.0, 8.0, 6.0], 
[4.0, 5.0, 6.0]]], [[[7.0, 3.0, 8.0], [1.0, 5.0, 2.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 5.0], [6.0, 6.0, 5.0], [7.0, 1.0, 4.0]], 
[[6.0, 5.0, 5.0], [5.0, 4.0, 7.0], [7.0, 4.0, 5.0]]], [[[8.0, 6.0, 5.0], [6.0, 8.0, 1.0], [3.0, 3.0, 2.0]], [[6.0, 7.0, 4.0], 
[4.0, 5.0, 4.0], [6.0, 5.0, 1.0]], [[4.0, 7.0, 2.0], [4.0, 7.0, 8.0], [8.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPPU(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 4.0], [7.0, 4.0, 1.0], [6.0, 1.0, 5.0]], [[1.0, 8.0, 5.0], [1.0, 7.0, 3.0], [5.0, 5.0, 
7.0]], [[4.0, 2.0, 4.0], [6.0, 4.0, 5.0], [6.0, 3.0, 5.0]]], [[[7.0, 1.0, 1.0], [5.0, 8.0, 1.0], [7.0, 2.0, 3.0]], [[5.0, 3.0, 
4.0], [2.0, 2.0, 3.0], [1.0, 7.0, 4.0]], [[3.0, 2.0, 2.0], [1.0, 8.0, 5.0], [8.0, 1.0, 6.0]]], [[[5.0, 3.0, 5.0], [7.0, 8.0, 
6.0], [4.0, 7.0, 3.0]], [[7.0, 2.0, 2.0], [1.0, 2.0, 5.0], [6.0, 3.0, 7.0]], [[5.0, 5.0, 2.0], [8.0, 5.0, 4.0], [3.0, 3.0, 
4.0]]], [[[2.0, 6.0, 2.0], [2.0, 8.0, 2.0], [6.0, 7.0, 3.0]], [[2.0, 4.0, 7.0], [2.0, 2.0, 4.0], [5.0, 1.0, 5.0]], [[8.0, 2.0, 
3.0], [7.0, 8.0, 8.0], [8.0, 5.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 6.0], [3.0, 7.0, 7.0], [4.0, 5.0, 4.0]], [[2.0, 4.0, 6.0], [1.0, 7.0, 
5.0], [7.0, 2.0, 6.0]], [[8.0, 3.0, 6.0], [6.0, 1.0, 7.0], [7.0, 7.0, 8.0]]], [[[1.0, 3.0, 3.0], [8.0, 1.0, 5.0], [4.0, 3.0, 
7.0]], [[3.0, 2.0, 7.0], [4.0, 5.0, 4.0], [5.0, 5.0, 3.0]], [[6.0, 1.0, 4.0], [4.0, 6.0, 4.0], [1.0, 3.0, 8.0]]], [[[5.0, 7.0, 
4.0], [7.0, 2.0, 5.0], [4.0, 7.0, 2.0]], [[2.0, 8.0, 7.0], [6.0, 4.0, 7.0], [6.0, 4.0, 2.0]], [[2.0, 8.0, 1.0], [3.0, 7.0, 
6.0], [2.0, 2.0, 3.0]]], [[[6.0, 6.0, 4.0], [4.0, 1.0, 5.0], [2.0, 3.0, 2.0]], [[1.0, 2.0, 1.0], [5.0, 3.0, 3.0], [7.0, 6.0, 
2.0]], [[4.0, 1.0, 8.0], [2.0, 7.0, 6.0], [7.0, 7.0, 5.0]]]]))
      val=Data(numpy.array([5.0, 7.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([6.0, 1.0]))
      arg[2,1,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 4.0], [7.0, 4.0, 1.0], [6.0, 1.0, 5.0]], [[1.0, 8.0, 5.0], [1.0, 7.0, 3.0], [5.0, 
5.0, 7.0]], [[4.0, 2.0, 4.0], [6.0, 4.0, 5.0], [6.0, 3.0, 5.0]]], [[[7.0, 1.0, 1.0], [5.0, 8.0, 1.0], [7.0, 2.0, 3.0]], [[5.0, 
3.0, 4.0], [2.0, 2.0, 3.0], [1.0, 7.0, 4.0]], [[3.0, 2.0, 2.0], [1.0, 8.0, 5.0], [8.0, 1.0, 6.0]]], [[[5.0, 3.0, 5.0], [7.0, 
8.0, 6.0], [4.0, 7.0, 3.0]], [[7.0, 2.0, 2.0], [5.0, 7.0, 5.0], [6.0, 3.0, 7.0]], [[5.0, 5.0, 2.0], [8.0, 5.0, 4.0], [3.0, 3.0, 
4.0]]], [[[2.0, 6.0, 2.0], [2.0, 8.0, 2.0], [6.0, 7.0, 3.0]], [[2.0, 4.0, 7.0], [2.0, 2.0, 4.0], [5.0, 1.0, 5.0]], [[8.0, 2.0, 
3.0], [7.0, 8.0, 8.0], [8.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [3.0, 7.0, 7.0], [4.0, 5.0, 4.0]], 
[[2.0, 4.0, 6.0], [1.0, 7.0, 5.0], [7.0, 2.0, 6.0]], [[8.0, 3.0, 6.0], [6.0, 1.0, 7.0], [7.0, 7.0, 8.0]]], [[[1.0, 3.0, 3.0], 
[8.0, 1.0, 5.0], [4.0, 3.0, 7.0]], [[3.0, 2.0, 7.0], [4.0, 5.0, 4.0], [5.0, 5.0, 3.0]], [[6.0, 1.0, 4.0], [4.0, 6.0, 4.0], 
[1.0, 3.0, 8.0]]], [[[5.0, 7.0, 4.0], [7.0, 2.0, 5.0], [4.0, 7.0, 2.0]], [[2.0, 8.0, 7.0], [6.0, 1.0, 7.0], [6.0, 4.0, 2.0]], 
[[2.0, 8.0, 1.0], [3.0, 7.0, 6.0], [2.0, 2.0, 3.0]]], [[[6.0, 6.0, 4.0], [4.0, 1.0, 5.0], [2.0, 3.0, 2.0]], [[1.0, 2.0, 1.0], 
[5.0, 3.0, 3.0], [7.0, 6.0, 2.0]], [[4.0, 1.0, 8.0], [2.0, 7.0, 6.0], [7.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPPU(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 7.0], [8.0, 5.0, 6.0], [4.0, 4.0, 5.0]], [[5.0, 6.0, 4.0], [8.0, 1.0, 7.0], [8.0, 8.0, 
4.0]], [[2.0, 6.0, 3.0], [2.0, 2.0, 7.0], [5.0, 3.0, 1.0]]], [[[7.0, 2.0, 2.0], [3.0, 4.0, 6.0], [5.0, 4.0, 3.0]], [[7.0, 5.0, 
5.0], [7.0, 2.0, 3.0], [4.0, 6.0, 6.0]], [[7.0, 2.0, 4.0], [5.0, 6.0, 1.0], [7.0, 8.0, 5.0]]], [[[6.0, 5.0, 1.0], [6.0, 2.0, 
5.0], [8.0, 1.0, 1.0]], [[1.0, 8.0, 3.0], [8.0, 1.0, 4.0], [4.0, 8.0, 2.0]], [[5.0, 2.0, 6.0], [1.0, 6.0, 5.0], [7.0, 4.0, 
5.0]]], [[[6.0, 3.0, 2.0], [4.0, 6.0, 4.0], [5.0, 2.0, 2.0]], [[3.0, 1.0, 5.0], [4.0, 3.0, 3.0], [2.0, 4.0, 2.0]], [[6.0, 8.0, 
4.0], [3.0, 3.0, 2.0], [5.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 7.0], [5.0, 2.0, 7.0], [4.0, 4.0, 2.0]], [[6.0, 7.0, 3.0], [4.0, 5.0, 
3.0], [3.0, 5.0, 7.0]], [[4.0, 8.0, 4.0], [3.0, 6.0, 1.0], [7.0, 8.0, 3.0]]], [[[1.0, 7.0, 3.0], [3.0, 5.0, 1.0], [5.0, 7.0, 
6.0]], [[3.0, 3.0, 2.0], [6.0, 2.0, 8.0], [7.0, 8.0, 6.0]], [[5.0, 8.0, 1.0], [1.0, 5.0, 2.0], [4.0, 4.0, 5.0]]], [[[7.0, 8.0, 
8.0], [2.0, 4.0, 2.0], [4.0, 2.0, 6.0]], [[3.0, 5.0, 2.0], [4.0, 5.0, 6.0], [8.0, 1.0, 3.0]], [[6.0, 5.0, 2.0], [1.0, 6.0, 
3.0], [6.0, 5.0, 3.0]]], [[[1.0, 8.0, 5.0], [1.0, 2.0, 4.0], [4.0, 7.0, 1.0]], [[3.0, 2.0, 4.0], [4.0, 7.0, 3.0], [1.0, 8.0, 
5.0]], [[1.0, 3.0, 3.0], [8.0, 6.0, 4.0], [1.0, 7.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([7.0, 6.0])+(1.-msk_val)*numpy.array([3.0, 5.0])
      arg[2,1,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 7.0], [8.0, 5.0, 6.0], [4.0, 4.0, 5.0]], [[5.0, 6.0, 4.0], [8.0, 1.0, 7.0], [8.0, 
8.0, 4.0]], [[2.0, 6.0, 3.0], [2.0, 2.0, 7.0], [5.0, 3.0, 1.0]]], [[[7.0, 2.0, 2.0], [3.0, 4.0, 6.0], [5.0, 4.0, 3.0]], [[7.0, 
5.0, 5.0], [7.0, 2.0, 3.0], [4.0, 6.0, 6.0]], [[7.0, 2.0, 4.0], [5.0, 6.0, 1.0], [7.0, 8.0, 5.0]]], [[[6.0, 5.0, 1.0], [6.0, 
2.0, 5.0], [8.0, 1.0, 1.0]], [[1.0, 8.0, 3.0], [7.0, 6.0, 4.0], [4.0, 8.0, 2.0]], [[5.0, 2.0, 6.0], [1.0, 6.0, 5.0], [7.0, 4.0, 
5.0]]], [[[6.0, 3.0, 2.0], [4.0, 6.0, 4.0], [5.0, 2.0, 2.0]], [[3.0, 1.0, 5.0], [4.0, 3.0, 3.0], [2.0, 4.0, 2.0]], [[6.0, 8.0, 
4.0], [3.0, 3.0, 2.0], [5.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 7.0], [5.0, 2.0, 7.0], [4.0, 4.0, 2.0]], 
[[6.0, 7.0, 3.0], [4.0, 5.0, 3.0], [3.0, 5.0, 7.0]], [[4.0, 8.0, 4.0], [3.0, 6.0, 1.0], [7.0, 8.0, 3.0]]], [[[1.0, 7.0, 3.0], 
[3.0, 5.0, 1.0], [5.0, 7.0, 6.0]], [[3.0, 3.0, 2.0], [6.0, 2.0, 8.0], [7.0, 8.0, 6.0]], [[5.0, 8.0, 1.0], [1.0, 5.0, 2.0], 
[4.0, 4.0, 5.0]]], [[[7.0, 8.0, 8.0], [2.0, 4.0, 2.0], [4.0, 2.0, 6.0]], [[3.0, 5.0, 2.0], [3.0, 5.0, 6.0], [8.0, 1.0, 3.0]], 
[[6.0, 5.0, 2.0], [1.0, 6.0, 3.0], [6.0, 5.0, 3.0]]], [[[1.0, 8.0, 5.0], [1.0, 2.0, 4.0], [4.0, 7.0, 1.0]], [[3.0, 2.0, 4.0], 
[4.0, 7.0, 3.0], [1.0, 8.0, 5.0]], [[1.0, 3.0, 3.0], [8.0, 6.0, 4.0], [1.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULUU(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 7.0], [1.0, 7.0, 6.0], [6.0, 7.0, 7.0]], [[1.0, 3.0, 8.0], [8.0, 2.0, 7.0], [2.0, 2.0, 
2.0]], [[5.0, 6.0, 1.0], [3.0, 4.0, 3.0], [5.0, 7.0, 7.0]]], [[[2.0, 8.0, 7.0], [2.0, 7.0, 6.0], [8.0, 1.0, 8.0]], [[7.0, 6.0, 
6.0], [3.0, 2.0, 2.0], [2.0, 7.0, 8.0]], [[5.0, 1.0, 8.0], [7.0, 2.0, 2.0], [8.0, 4.0, 5.0]]], [[[8.0, 5.0, 6.0], [7.0, 4.0, 
6.0], [6.0, 7.0, 1.0]], [[3.0, 6.0, 7.0], [1.0, 8.0, 3.0], [8.0, 2.0, 8.0]], [[7.0, 4.0, 5.0], [8.0, 5.0, 5.0], [5.0, 6.0, 
1.0]]], [[[2.0, 8.0, 1.0], [1.0, 8.0, 4.0], [7.0, 7.0, 1.0]], [[4.0, 7.0, 4.0], [3.0, 2.0, 1.0], [4.0, 1.0, 4.0]], [[8.0, 7.0, 
8.0], [8.0, 6.0, 6.0], [1.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 4.0], [5.0, 6.0, 1.0], [4.0, 6.0, 7.0]], [[5.0, 8.0, 5.0], [1.0, 1.0, 
2.0], [7.0, 3.0, 8.0]], [[2.0, 4.0, 4.0], [5.0, 4.0, 6.0], [8.0, 3.0, 1.0]]], [[[7.0, 3.0, 7.0], [2.0, 2.0, 3.0], [6.0, 7.0, 
7.0]], [[1.0, 5.0, 3.0], [6.0, 1.0, 8.0], [1.0, 4.0, 3.0]], [[7.0, 1.0, 6.0], [4.0, 2.0, 2.0], [5.0, 1.0, 7.0]]], [[[8.0, 2.0, 
8.0], [6.0, 4.0, 5.0], [3.0, 2.0, 1.0]], [[3.0, 3.0, 7.0], [7.0, 6.0, 4.0], [2.0, 8.0, 5.0]], [[1.0, 7.0, 1.0], [2.0, 7.0, 
3.0], [3.0, 2.0, 1.0]]], [[[3.0, 2.0, 2.0], [2.0, 5.0, 2.0], [8.0, 6.0, 6.0]], [[1.0, 1.0, 4.0], [5.0, 5.0, 6.0], [6.0, 6.0, 
6.0]], [[1.0, 4.0, 3.0], [5.0, 8.0, 2.0], [8.0, 8.0, 4.0]]]]))
      val=numpy.array([[[[7.0, 4.0], [3.0, 2.0]]], [[[6.0, 1.0], [7.0, 1.0]]], [[[6.0, 4.0], [1.0, 4.0]]]])
      arg[:3,2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 7.0], [1.0, 7.0, 6.0], [6.0, 7.0, 7.0]], [[1.0, 3.0, 8.0], [8.0, 2.0, 7.0], [2.0, 
2.0, 2.0]], [[7.0, 4.0, 1.0], [3.0, 2.0, 3.0], [5.0, 7.0, 7.0]]], [[[2.0, 8.0, 7.0], [2.0, 7.0, 6.0], [8.0, 1.0, 8.0]], [[7.0, 
6.0, 6.0], [3.0, 2.0, 2.0], [2.0, 7.0, 8.0]], [[6.0, 1.0, 8.0], [7.0, 1.0, 2.0], [8.0, 4.0, 5.0]]], [[[8.0, 5.0, 6.0], [7.0, 
4.0, 6.0], [6.0, 7.0, 1.0]], [[3.0, 6.0, 7.0], [1.0, 8.0, 3.0], [8.0, 2.0, 8.0]], [[6.0, 4.0, 5.0], [1.0, 4.0, 5.0], [5.0, 6.0, 
1.0]]], [[[2.0, 8.0, 1.0], [1.0, 8.0, 4.0], [7.0, 7.0, 1.0]], [[4.0, 7.0, 4.0], [3.0, 2.0, 1.0], [4.0, 1.0, 4.0]], [[8.0, 7.0, 
8.0], [8.0, 6.0, 6.0], [1.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 4.0], [5.0, 6.0, 1.0], [4.0, 6.0, 7.0]], 
[[5.0, 8.0, 5.0], [1.0, 1.0, 2.0], [7.0, 3.0, 8.0]], [[7.0, 4.0, 4.0], [3.0, 2.0, 6.0], [8.0, 3.0, 1.0]]], [[[7.0, 3.0, 7.0], 
[2.0, 2.0, 3.0], [6.0, 7.0, 7.0]], [[1.0, 5.0, 3.0], [6.0, 1.0, 8.0], [1.0, 4.0, 3.0]], [[6.0, 1.0, 6.0], [7.0, 1.0, 2.0], 
[5.0, 1.0, 7.0]]], [[[8.0, 2.0, 8.0], [6.0, 4.0, 5.0], [3.0, 2.0, 1.0]], [[3.0, 3.0, 7.0], [7.0, 6.0, 4.0], [2.0, 8.0, 5.0]], 
[[6.0, 4.0, 1.0], [1.0, 4.0, 3.0], [3.0, 2.0, 1.0]]], [[[3.0, 2.0, 2.0], [2.0, 5.0, 2.0], [8.0, 6.0, 6.0]], [[1.0, 1.0, 4.0], 
[5.0, 5.0, 6.0], [6.0, 6.0, 6.0]], [[1.0, 4.0, 3.0], [5.0, 8.0, 2.0], [8.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULUU(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 5.0], [4.0, 2.0, 8.0], [3.0, 2.0, 6.0]], [[4.0, 2.0, 4.0], [3.0, 1.0, 5.0], [5.0, 3.0, 
2.0]], [[1.0, 8.0, 2.0], [8.0, 7.0, 5.0], [5.0, 4.0, 8.0]]], [[[5.0, 5.0, 2.0], [6.0, 3.0, 4.0], [1.0, 7.0, 2.0]], [[1.0, 5.0, 
3.0], [3.0, 6.0, 2.0], [8.0, 5.0, 6.0]], [[8.0, 8.0, 7.0], [8.0, 4.0, 7.0], [8.0, 7.0, 4.0]]], [[[4.0, 7.0, 8.0], [7.0, 2.0, 
1.0], [4.0, 1.0, 1.0]], [[3.0, 2.0, 8.0], [1.0, 1.0, 5.0], [6.0, 8.0, 2.0]], [[4.0, 2.0, 7.0], [4.0, 6.0, 2.0], [2.0, 1.0, 
2.0]]], [[[6.0, 8.0, 4.0], [2.0, 3.0, 2.0], [5.0, 5.0, 4.0]], [[1.0, 5.0, 5.0], [5.0, 2.0, 1.0], [2.0, 2.0, 2.0]], [[8.0, 7.0, 
6.0], [7.0, 7.0, 2.0], [7.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 3.0], [1.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[7.0, 6.0, 8.0], [3.0, 1.0, 
8.0], [2.0, 4.0, 4.0]], [[2.0, 5.0, 1.0], [8.0, 3.0, 4.0], [3.0, 8.0, 2.0]]], [[[5.0, 5.0, 8.0], [6.0, 7.0, 4.0], [4.0, 1.0, 
2.0]], [[8.0, 4.0, 3.0], [6.0, 7.0, 4.0], [1.0, 2.0, 1.0]], [[1.0, 8.0, 2.0], [8.0, 1.0, 2.0], [5.0, 5.0, 7.0]]], [[[5.0, 6.0, 
5.0], [1.0, 4.0, 2.0], [8.0, 6.0, 6.0]], [[6.0, 7.0, 8.0], [6.0, 7.0, 2.0], [7.0, 6.0, 5.0]], [[6.0, 1.0, 4.0], [6.0, 6.0, 
4.0], [7.0, 4.0, 5.0]]], [[[2.0, 6.0, 5.0], [5.0, 7.0, 8.0], [8.0, 1.0, 7.0]], [[1.0, 7.0, 4.0], [5.0, 3.0, 3.0], [2.0, 8.0, 
2.0]], [[3.0, 2.0, 6.0], [4.0, 5.0, 1.0], [1.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[[[2.0, 6.0], [7.0, 3.0]]], [[[3.0, 5.0], [6.0, 7.0]]], [[[6.0, 7.0], [5.0, 
5.0]]]]),self.functionspace)
      arg[:3,2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 5.0], [4.0, 2.0, 8.0], [3.0, 2.0, 6.0]], [[4.0, 2.0, 4.0], [3.0, 1.0, 5.0], [5.0, 
3.0, 2.0]], [[2.0, 6.0, 2.0], [7.0, 3.0, 5.0], [5.0, 4.0, 8.0]]], [[[5.0, 5.0, 2.0], [6.0, 3.0, 4.0], [1.0, 7.0, 2.0]], [[1.0, 
5.0, 3.0], [3.0, 6.0, 2.0], [8.0, 5.0, 6.0]], [[3.0, 5.0, 7.0], [6.0, 7.0, 7.0], [8.0, 7.0, 4.0]]], [[[4.0, 7.0, 8.0], [7.0, 
2.0, 1.0], [4.0, 1.0, 1.0]], [[3.0, 2.0, 8.0], [1.0, 1.0, 5.0], [6.0, 8.0, 2.0]], [[6.0, 7.0, 7.0], [5.0, 5.0, 2.0], [2.0, 1.0, 
2.0]]], [[[6.0, 8.0, 4.0], [2.0, 3.0, 2.0], [5.0, 5.0, 4.0]], [[1.0, 5.0, 5.0], [5.0, 2.0, 1.0], [2.0, 2.0, 2.0]], [[8.0, 7.0, 
6.0], [7.0, 7.0, 2.0], [7.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 3.0], [1.0, 5.0, 8.0], [4.0, 2.0, 8.0]], 
[[7.0, 6.0, 8.0], [3.0, 1.0, 8.0], [2.0, 4.0, 4.0]], [[2.0, 6.0, 1.0], [7.0, 3.0, 4.0], [3.0, 8.0, 2.0]]], [[[5.0, 5.0, 8.0], 
[6.0, 7.0, 4.0], [4.0, 1.0, 2.0]], [[8.0, 4.0, 3.0], [6.0, 7.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 5.0, 2.0], [6.0, 7.0, 2.0], 
[5.0, 5.0, 7.0]]], [[[5.0, 6.0, 5.0], [1.0, 4.0, 2.0], [8.0, 6.0, 6.0]], [[6.0, 7.0, 8.0], [6.0, 7.0, 2.0], [7.0, 6.0, 5.0]], 
[[6.0, 7.0, 4.0], [5.0, 5.0, 4.0], [7.0, 4.0, 5.0]]], [[[2.0, 6.0, 5.0], [5.0, 7.0, 8.0], [8.0, 1.0, 7.0]], [[1.0, 7.0, 4.0], 
[5.0, 3.0, 3.0], [2.0, 8.0, 2.0]], [[3.0, 2.0, 6.0], [4.0, 5.0, 1.0], [1.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULUU(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 3.0], [4.0, 4.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 8.0, 6.0], [1.0, 2.0, 5.0], [3.0, 7.0, 
7.0]], [[8.0, 1.0, 5.0], [3.0, 5.0, 2.0], [8.0, 5.0, 5.0]]], [[[1.0, 4.0, 1.0], [6.0, 3.0, 2.0], [4.0, 2.0, 7.0]], [[6.0, 4.0, 
2.0], [5.0, 3.0, 7.0], [3.0, 3.0, 2.0]], [[7.0, 1.0, 4.0], [5.0, 2.0, 6.0], [2.0, 4.0, 5.0]]], [[[6.0, 6.0, 4.0], [5.0, 1.0, 
1.0], [4.0, 4.0, 4.0]], [[4.0, 3.0, 8.0], [7.0, 5.0, 3.0], [4.0, 7.0, 5.0]], [[5.0, 1.0, 4.0], [1.0, 6.0, 8.0], [7.0, 5.0, 
5.0]]], [[[2.0, 1.0, 5.0], [3.0, 8.0, 2.0], [4.0, 2.0, 2.0]], [[5.0, 2.0, 2.0], [3.0, 4.0, 3.0], [6.0, 3.0, 1.0]], [[7.0, 6.0, 
4.0], [4.0, 5.0, 1.0], [1.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 7.0], [3.0, 1.0, 2.0], [2.0, 1.0, 8.0]], [[3.0, 7.0, 2.0], [7.0, 4.0, 
8.0], [2.0, 3.0, 2.0]], [[7.0, 7.0, 2.0], [1.0, 6.0, 6.0], [3.0, 2.0, 3.0]]], [[[8.0, 3.0, 3.0], [8.0, 6.0, 6.0], [7.0, 3.0, 
7.0]], [[7.0, 3.0, 6.0], [6.0, 7.0, 6.0], [4.0, 4.0, 3.0]], [[5.0, 3.0, 1.0], [1.0, 5.0, 2.0], [6.0, 1.0, 1.0]]], [[[7.0, 6.0, 
1.0], [6.0, 1.0, 4.0], [3.0, 1.0, 8.0]], [[7.0, 8.0, 4.0], [4.0, 1.0, 5.0], [4.0, 8.0, 6.0]], [[8.0, 1.0, 7.0], [7.0, 5.0, 
4.0], [8.0, 2.0, 2.0]]], [[[6.0, 1.0, 4.0], [4.0, 5.0, 7.0], [6.0, 4.0, 5.0]], [[4.0, 6.0, 4.0], [1.0, 8.0, 5.0], [7.0, 8.0, 
4.0]], [[6.0, 4.0, 4.0], [2.0, 7.0, 4.0], [6.0, 5.0, 7.0]]]]))
      val=Data(numpy.array([[[[8.0, 8.0], [3.0, 4.0]]], [[[3.0, 3.0], [7.0, 2.0]]], [[[8.0, 8.0], [2.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 1.0], [7.0, 6.0]]], [[[1.0, 1.0], [2.0, 8.0]]], [[[5.0, 8.0], [5.0, 5.0]]]]))
      arg[:3,2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 3.0], [4.0, 4.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 8.0, 6.0], [1.0, 2.0, 5.0], [3.0, 
7.0, 7.0]], [[8.0, 8.0, 5.0], [3.0, 4.0, 2.0], [8.0, 5.0, 5.0]]], [[[1.0, 4.0, 1.0], [6.0, 3.0, 2.0], [4.0, 2.0, 7.0]], [[6.0, 
4.0, 2.0], [5.0, 3.0, 7.0], [3.0, 3.0, 2.0]], [[3.0, 3.0, 4.0], [7.0, 2.0, 6.0], [2.0, 4.0, 5.0]]], [[[6.0, 6.0, 4.0], [5.0, 
1.0, 1.0], [4.0, 4.0, 4.0]], [[4.0, 3.0, 8.0], [7.0, 5.0, 3.0], [4.0, 7.0, 5.0]], [[8.0, 8.0, 4.0], [2.0, 2.0, 8.0], [7.0, 5.0, 
5.0]]], [[[2.0, 1.0, 5.0], [3.0, 8.0, 2.0], [4.0, 2.0, 2.0]], [[5.0, 2.0, 2.0], [3.0, 4.0, 3.0], [6.0, 3.0, 1.0]], [[7.0, 6.0, 
4.0], [4.0, 5.0, 1.0], [1.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 7.0], [3.0, 1.0, 2.0], [2.0, 1.0, 8.0]], 
[[3.0, 7.0, 2.0], [7.0, 4.0, 8.0], [2.0, 3.0, 2.0]], [[5.0, 1.0, 2.0], [7.0, 6.0, 6.0], [3.0, 2.0, 3.0]]], [[[8.0, 3.0, 3.0], 
[8.0, 6.0, 6.0], [7.0, 3.0, 7.0]], [[7.0, 3.0, 6.0], [6.0, 7.0, 6.0], [4.0, 4.0, 3.0]], [[1.0, 1.0, 1.0], [2.0, 8.0, 2.0], 
[6.0, 1.0, 1.0]]], [[[7.0, 6.0, 1.0], [6.0, 1.0, 4.0], [3.0, 1.0, 8.0]], [[7.0, 8.0, 4.0], [4.0, 1.0, 5.0], [4.0, 8.0, 6.0]], 
[[5.0, 8.0, 7.0], [5.0, 5.0, 4.0], [8.0, 2.0, 2.0]]], [[[6.0, 1.0, 4.0], [4.0, 5.0, 7.0], [6.0, 4.0, 5.0]], [[4.0, 6.0, 4.0], 
[1.0, 8.0, 5.0], [7.0, 8.0, 4.0]], [[6.0, 4.0, 4.0], [2.0, 7.0, 4.0], [6.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULUU(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 8.0], [8.0, 3.0, 2.0], [4.0, 3.0, 7.0]], [[8.0, 3.0, 4.0], [5.0, 5.0, 5.0], [3.0, 4.0, 
6.0]], [[5.0, 2.0, 1.0], [2.0, 2.0, 1.0], [6.0, 2.0, 2.0]]], [[[4.0, 1.0, 2.0], [3.0, 7.0, 5.0], [8.0, 1.0, 4.0]], [[8.0, 3.0, 
5.0], [2.0, 7.0, 3.0], [3.0, 2.0, 5.0]], [[1.0, 4.0, 8.0], [3.0, 7.0, 5.0], [2.0, 5.0, 3.0]]], [[[4.0, 2.0, 6.0], [8.0, 4.0, 
1.0], [8.0, 3.0, 1.0]], [[2.0, 2.0, 6.0], [5.0, 6.0, 3.0], [7.0, 3.0, 7.0]], [[4.0, 5.0, 6.0], [3.0, 1.0, 5.0], [2.0, 7.0, 
6.0]]], [[[8.0, 3.0, 2.0], [2.0, 1.0, 8.0], [6.0, 7.0, 3.0]], [[8.0, 5.0, 5.0], [2.0, 4.0, 3.0], [1.0, 7.0, 3.0]], [[1.0, 4.0, 
5.0], [6.0, 3.0, 7.0], [1.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 1.0], [5.0, 8.0, 5.0], [8.0, 5.0, 7.0]], [[2.0, 1.0, 5.0], [8.0, 6.0, 
8.0], [3.0, 5.0, 4.0]], [[7.0, 6.0, 6.0], [8.0, 5.0, 2.0], [4.0, 4.0, 7.0]]], [[[8.0, 5.0, 2.0], [7.0, 4.0, 4.0], [5.0, 3.0, 
2.0]], [[6.0, 2.0, 2.0], [4.0, 6.0, 6.0], [4.0, 1.0, 7.0]], [[2.0, 5.0, 1.0], [7.0, 8.0, 1.0], [2.0, 2.0, 1.0]]], [[[1.0, 3.0, 
4.0], [4.0, 4.0, 7.0], [5.0, 7.0, 1.0]], [[2.0, 6.0, 8.0], [7.0, 6.0, 2.0], [1.0, 5.0, 8.0]], [[5.0, 2.0, 1.0], [3.0, 6.0, 
5.0], [2.0, 3.0, 8.0]]], [[[1.0, 2.0, 7.0], [3.0, 4.0, 7.0], [6.0, 8.0, 4.0]], [[6.0, 1.0, 5.0], [8.0, 8.0, 5.0], [8.0, 4.0, 
7.0]], [[1.0, 8.0, 4.0], [3.0, 5.0, 6.0], [8.0, 1.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0, 4.0], [7.0, 4.0]]], [[[6.0, 2.0], [7.0, 1.0]]], [[[1.0, 8.0], [6.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 1.0], [5.0, 3.0]]], [[[4.0, 7.0], [8.0, 1.0]]], [[[1.0, 8.0], [1.0, 2.0]]]])
      arg[:3,2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 8.0], [8.0, 3.0, 2.0], [4.0, 3.0, 7.0]], [[8.0, 3.0, 4.0], [5.0, 5.0, 5.0], [3.0, 
4.0, 6.0]], [[7.0, 4.0, 1.0], [7.0, 4.0, 1.0], [6.0, 2.0, 2.0]]], [[[4.0, 1.0, 2.0], [3.0, 7.0, 5.0], [8.0, 1.0, 4.0]], [[8.0, 
3.0, 5.0], [2.0, 7.0, 3.0], [3.0, 2.0, 5.0]], [[6.0, 2.0, 8.0], [7.0, 1.0, 5.0], [2.0, 5.0, 3.0]]], [[[4.0, 2.0, 6.0], [8.0, 
4.0, 1.0], [8.0, 3.0, 1.0]], [[2.0, 2.0, 6.0], [5.0, 6.0, 3.0], [7.0, 3.0, 7.0]], [[1.0, 8.0, 6.0], [6.0, 1.0, 5.0], [2.0, 7.0, 
6.0]]], [[[8.0, 3.0, 2.0], [2.0, 1.0, 8.0], [6.0, 7.0, 3.0]], [[8.0, 5.0, 5.0], [2.0, 4.0, 3.0], [1.0, 7.0, 3.0]], [[1.0, 4.0, 
5.0], [6.0, 3.0, 7.0], [1.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 1.0], [5.0, 8.0, 5.0], [8.0, 5.0, 7.0]], 
[[2.0, 1.0, 5.0], [8.0, 6.0, 8.0], [3.0, 5.0, 4.0]], [[7.0, 1.0, 6.0], [5.0, 3.0, 2.0], [4.0, 4.0, 7.0]]], [[[8.0, 5.0, 2.0], 
[7.0, 4.0, 4.0], [5.0, 3.0, 2.0]], [[6.0, 2.0, 2.0], [4.0, 6.0, 6.0], [4.0, 1.0, 7.0]], [[4.0, 7.0, 1.0], [8.0, 1.0, 1.0], 
[2.0, 2.0, 1.0]]], [[[1.0, 3.0, 4.0], [4.0, 4.0, 7.0], [5.0, 7.0, 1.0]], [[2.0, 6.0, 8.0], [7.0, 6.0, 2.0], [1.0, 5.0, 8.0]], 
[[1.0, 8.0, 1.0], [1.0, 2.0, 5.0], [2.0, 3.0, 8.0]]], [[[1.0, 2.0, 7.0], [3.0, 4.0, 7.0], [6.0, 8.0, 4.0]], [[6.0, 1.0, 5.0], 
[8.0, 8.0, 5.0], [8.0, 4.0, 7.0]], [[1.0, 8.0, 4.0], [3.0, 5.0, 6.0], [8.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LUPN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 6.0], [8.0, 1.0, 2.0], [5.0, 2.0, 8.0]], [[2.0, 7.0, 7.0], [8.0, 5.0, 4.0], [5.0, 6.0, 
8.0]], [[6.0, 4.0, 2.0], [1.0, 6.0, 8.0], [1.0, 8.0, 7.0]]], [[[5.0, 8.0, 1.0], [4.0, 3.0, 4.0], [1.0, 1.0, 2.0]], [[2.0, 3.0, 
5.0], [1.0, 1.0, 8.0], [2.0, 6.0, 7.0]], [[7.0, 7.0, 8.0], [2.0, 5.0, 4.0], [8.0, 2.0, 8.0]]], [[[7.0, 8.0, 8.0], [5.0, 1.0, 
3.0], [5.0, 7.0, 8.0]], [[8.0, 7.0, 2.0], [2.0, 2.0, 8.0], [1.0, 5.0, 1.0]], [[2.0, 3.0, 8.0], [6.0, 1.0, 5.0], [6.0, 3.0, 
6.0]]], [[[4.0, 1.0, 4.0], [8.0, 3.0, 7.0], [5.0, 2.0, 1.0]], [[5.0, 6.0, 4.0], [8.0, 8.0, 3.0], [5.0, 8.0, 3.0]], [[1.0, 5.0, 
7.0], [4.0, 3.0, 2.0], [5.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 5.0], [8.0, 3.0, 5.0], [4.0, 7.0, 8.0]], [[5.0, 7.0, 5.0], [5.0, 6.0, 
6.0], [4.0, 6.0, 1.0]], [[7.0, 2.0, 2.0], [4.0, 5.0, 2.0], [3.0, 2.0, 4.0]]], [[[3.0, 3.0, 7.0], [2.0, 4.0, 4.0], [5.0, 6.0, 
5.0]], [[4.0, 4.0, 5.0], [2.0, 5.0, 8.0], [1.0, 4.0, 6.0]], [[2.0, 6.0, 2.0], [6.0, 6.0, 8.0], [2.0, 4.0, 5.0]]], [[[5.0, 1.0, 
6.0], [3.0, 1.0, 4.0], [7.0, 1.0, 5.0]], [[7.0, 4.0, 5.0], [2.0, 4.0, 1.0], [7.0, 3.0, 6.0]], [[8.0, 3.0, 3.0], [1.0, 2.0, 
4.0], [8.0, 8.0, 3.0]]], [[[1.0, 7.0, 1.0], [2.0, 7.0, 7.0], [6.0, 5.0, 5.0]], [[5.0, 3.0, 4.0], [1.0, 4.0, 7.0], [7.0, 1.0, 
6.0]], [[7.0, 1.0, 5.0], [4.0, 6.0, 6.0], [7.0, 3.0, 6.0]]]]))
      val=numpy.array([[[6.0, 1.0, 6.0], [6.0, 5.0, 6.0]], [[4.0, 8.0, 6.0], [4.0, 7.0, 6.0]]])
      arg[2:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 6.0], [8.0, 1.0, 2.0], [5.0, 2.0, 8.0]], [[2.0, 7.0, 7.0], [8.0, 5.0, 4.0], [5.0, 
6.0, 8.0]], [[6.0, 4.0, 2.0], [1.0, 6.0, 8.0], [1.0, 8.0, 7.0]]], [[[5.0, 8.0, 1.0], [4.0, 3.0, 4.0], [1.0, 1.0, 2.0]], [[2.0, 
3.0, 5.0], [1.0, 1.0, 8.0], [2.0, 6.0, 7.0]], [[7.0, 7.0, 8.0], [2.0, 5.0, 4.0], [8.0, 2.0, 8.0]]], [[[7.0, 8.0, 8.0], [6.0, 
1.0, 6.0], [5.0, 7.0, 8.0]], [[8.0, 7.0, 2.0], [6.0, 5.0, 6.0], [1.0, 5.0, 1.0]], [[2.0, 3.0, 8.0], [6.0, 1.0, 5.0], [6.0, 3.0, 
6.0]]], [[[4.0, 1.0, 4.0], [4.0, 8.0, 6.0], [5.0, 2.0, 1.0]], [[5.0, 6.0, 4.0], [4.0, 7.0, 6.0], [5.0, 8.0, 3.0]], [[1.0, 5.0, 
7.0], [4.0, 3.0, 2.0], [5.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 5.0], [8.0, 3.0, 5.0], [4.0, 7.0, 8.0]], 
[[5.0, 7.0, 5.0], [5.0, 6.0, 6.0], [4.0, 6.0, 1.0]], [[7.0, 2.0, 2.0], [4.0, 5.0, 2.0], [3.0, 2.0, 4.0]]], [[[3.0, 3.0, 7.0], 
[2.0, 4.0, 4.0], [5.0, 6.0, 5.0]], [[4.0, 4.0, 5.0], [2.0, 5.0, 8.0], [1.0, 4.0, 6.0]], [[2.0, 6.0, 2.0], [6.0, 6.0, 8.0], 
[2.0, 4.0, 5.0]]], [[[5.0, 1.0, 6.0], [6.0, 1.0, 6.0], [7.0, 1.0, 5.0]], [[7.0, 4.0, 5.0], [6.0, 5.0, 6.0], [7.0, 3.0, 6.0]], 
[[8.0, 3.0, 3.0], [1.0, 2.0, 4.0], [8.0, 8.0, 3.0]]], [[[1.0, 7.0, 1.0], [4.0, 8.0, 6.0], [6.0, 5.0, 5.0]], [[5.0, 3.0, 4.0], 
[4.0, 7.0, 6.0], [7.0, 1.0, 6.0]], [[7.0, 1.0, 5.0], [4.0, 6.0, 6.0], [7.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LUPN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 7.0], [6.0, 3.0, 6.0], [3.0, 2.0, 1.0]], [[5.0, 1.0, 1.0], [4.0, 7.0, 5.0], [8.0, 6.0, 
4.0]], [[8.0, 8.0, 8.0], [6.0, 7.0, 4.0], [7.0, 4.0, 1.0]]], [[[5.0, 8.0, 3.0], [6.0, 2.0, 2.0], [4.0, 5.0, 6.0]], [[2.0, 8.0, 
4.0], [5.0, 5.0, 2.0], [8.0, 3.0, 3.0]], [[3.0, 3.0, 7.0], [8.0, 1.0, 1.0], [5.0, 5.0, 1.0]]], [[[2.0, 6.0, 8.0], [8.0, 2.0, 
2.0], [4.0, 1.0, 8.0]], [[2.0, 6.0, 6.0], [8.0, 7.0, 7.0], [8.0, 4.0, 8.0]], [[1.0, 6.0, 1.0], [1.0, 1.0, 8.0], [1.0, 5.0, 
8.0]]], [[[7.0, 1.0, 8.0], [8.0, 1.0, 5.0], [3.0, 6.0, 4.0]], [[8.0, 2.0, 4.0], [8.0, 6.0, 6.0], [6.0, 2.0, 3.0]], [[3.0, 8.0, 
5.0], [3.0, 6.0, 6.0], [6.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 2.0], [3.0, 4.0, 5.0], [1.0, 7.0, 1.0]], [[6.0, 1.0, 1.0], [3.0, 3.0, 
1.0], [1.0, 6.0, 8.0]], [[4.0, 4.0, 7.0], [1.0, 2.0, 1.0], [8.0, 3.0, 2.0]]], [[[5.0, 4.0, 3.0], [2.0, 6.0, 1.0], [3.0, 3.0, 
8.0]], [[8.0, 7.0, 3.0], [1.0, 8.0, 4.0], [3.0, 4.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 5.0, 4.0], [2.0, 7.0, 4.0]]], [[[1.0, 7.0, 
6.0], [2.0, 2.0, 4.0], [1.0, 3.0, 3.0]], [[5.0, 1.0, 4.0], [4.0, 5.0, 7.0], [4.0, 4.0, 7.0]], [[7.0, 5.0, 8.0], [6.0, 5.0, 
1.0], [6.0, 2.0, 1.0]]], [[[2.0, 5.0, 7.0], [1.0, 2.0, 5.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 1.0], [1.0, 6.0, 2.0], [6.0, 6.0, 
5.0]], [[1.0, 8.0, 3.0], [4.0, 4.0, 3.0], [6.0, 7.0, 3.0]]]]))
      val=Data(numpy.array([[[2.0, 2.0, 7.0], [8.0, 2.0, 1.0]], [[6.0, 7.0, 2.0], [7.0, 2.0, 3.0]]]),self.functionspace)
      arg[2:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 7.0], [6.0, 3.0, 6.0], [3.0, 2.0, 1.0]], [[5.0, 1.0, 1.0], [4.0, 7.0, 5.0], [8.0, 
6.0, 4.0]], [[8.0, 8.0, 8.0], [6.0, 7.0, 4.0], [7.0, 4.0, 1.0]]], [[[5.0, 8.0, 3.0], [6.0, 2.0, 2.0], [4.0, 5.0, 6.0]], [[2.0, 
8.0, 4.0], [5.0, 5.0, 2.0], [8.0, 3.0, 3.0]], [[3.0, 3.0, 7.0], [8.0, 1.0, 1.0], [5.0, 5.0, 1.0]]], [[[2.0, 6.0, 8.0], [2.0, 
2.0, 7.0], [4.0, 1.0, 8.0]], [[2.0, 6.0, 6.0], [8.0, 2.0, 1.0], [8.0, 4.0, 8.0]], [[1.0, 6.0, 1.0], [1.0, 1.0, 8.0], [1.0, 5.0, 
8.0]]], [[[7.0, 1.0, 8.0], [6.0, 7.0, 2.0], [3.0, 6.0, 4.0]], [[8.0, 2.0, 4.0], [7.0, 2.0, 3.0], [6.0, 2.0, 3.0]], [[3.0, 8.0, 
5.0], [3.0, 6.0, 6.0], [6.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 2.0], [3.0, 4.0, 5.0], [1.0, 7.0, 1.0]], 
[[6.0, 1.0, 1.0], [3.0, 3.0, 1.0], [1.0, 6.0, 8.0]], [[4.0, 4.0, 7.0], [1.0, 2.0, 1.0], [8.0, 3.0, 2.0]]], [[[5.0, 4.0, 3.0], 
[2.0, 6.0, 1.0], [3.0, 3.0, 8.0]], [[8.0, 7.0, 3.0], [1.0, 8.0, 4.0], [3.0, 4.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 5.0, 4.0], 
[2.0, 7.0, 4.0]]], [[[1.0, 7.0, 6.0], [2.0, 2.0, 7.0], [1.0, 3.0, 3.0]], [[5.0, 1.0, 4.0], [8.0, 2.0, 1.0], [4.0, 4.0, 7.0]], 
[[7.0, 5.0, 8.0], [6.0, 5.0, 1.0], [6.0, 2.0, 1.0]]], [[[2.0, 5.0, 7.0], [6.0, 7.0, 2.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 1.0], 
[7.0, 2.0, 3.0], [6.0, 6.0, 5.0]], [[1.0, 8.0, 3.0], [4.0, 4.0, 3.0], [6.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LUPN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 4.0], [2.0, 3.0, 5.0], [1.0, 3.0, 8.0]], [[2.0, 7.0, 4.0], [7.0, 1.0, 7.0], [1.0, 6.0, 
7.0]], [[2.0, 6.0, 7.0], [6.0, 4.0, 7.0], [2.0, 3.0, 2.0]]], [[[5.0, 5.0, 3.0], [2.0, 2.0, 1.0], [5.0, 7.0, 4.0]], [[1.0, 3.0, 
4.0], [8.0, 8.0, 5.0], [2.0, 3.0, 4.0]], [[1.0, 6.0, 8.0], [8.0, 6.0, 4.0], [4.0, 6.0, 2.0]]], [[[3.0, 4.0, 1.0], [8.0, 4.0, 
1.0], [3.0, 5.0, 8.0]], [[3.0, 4.0, 4.0], [7.0, 2.0, 7.0], [3.0, 4.0, 4.0]], [[2.0, 4.0, 4.0], [3.0, 3.0, 4.0], [8.0, 6.0, 
4.0]]], [[[1.0, 2.0, 5.0], [1.0, 3.0, 4.0], [2.0, 6.0, 5.0]], [[4.0, 4.0, 1.0], [3.0, 1.0, 5.0], [8.0, 2.0, 1.0]], [[4.0, 7.0, 
3.0], [7.0, 6.0, 8.0], [7.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 4.0], [1.0, 1.0, 5.0], [2.0, 3.0, 3.0]], [[1.0, 7.0, 4.0], [6.0, 1.0, 
7.0], [6.0, 1.0, 3.0]], [[1.0, 6.0, 4.0], [1.0, 8.0, 8.0], [6.0, 2.0, 1.0]]], [[[3.0, 2.0, 5.0], [2.0, 4.0, 3.0], [1.0, 4.0, 
5.0]], [[3.0, 6.0, 8.0], [5.0, 2.0, 2.0], [2.0, 7.0, 5.0]], [[4.0, 5.0, 2.0], [3.0, 8.0, 7.0], [4.0, 6.0, 5.0]]], [[[4.0, 1.0, 
2.0], [5.0, 1.0, 6.0], [2.0, 8.0, 6.0]], [[2.0, 5.0, 5.0], [8.0, 1.0, 2.0], [6.0, 7.0, 1.0]], [[1.0, 8.0, 6.0], [4.0, 1.0, 
7.0], [1.0, 6.0, 8.0]]], [[[1.0, 3.0, 5.0], [2.0, 2.0, 1.0], [1.0, 3.0, 8.0]], [[4.0, 7.0, 1.0], [8.0, 3.0, 1.0], [5.0, 3.0, 
4.0]], [[2.0, 5.0, 3.0], [3.0, 1.0, 1.0], [7.0, 8.0, 8.0]]]]))
      val=Data(numpy.array([[[4.0, 2.0, 1.0], [2.0, 6.0, 4.0]], [[5.0, 3.0, 7.0], [4.0, 1.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 3.0, 6.0], [8.0, 4.0, 8.0]], [[8.0, 8.0, 5.0], [4.0, 4.0, 8.0]]]))
      arg[2:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 4.0], [2.0, 3.0, 5.0], [1.0, 3.0, 8.0]], [[2.0, 7.0, 4.0], [7.0, 1.0, 7.0], [1.0, 
6.0, 7.0]], [[2.0, 6.0, 7.0], [6.0, 4.0, 7.0], [2.0, 3.0, 2.0]]], [[[5.0, 5.0, 3.0], [2.0, 2.0, 1.0], [5.0, 7.0, 4.0]], [[1.0, 
3.0, 4.0], [8.0, 8.0, 5.0], [2.0, 3.0, 4.0]], [[1.0, 6.0, 8.0], [8.0, 6.0, 4.0], [4.0, 6.0, 2.0]]], [[[3.0, 4.0, 1.0], [4.0, 
2.0, 1.0], [3.0, 5.0, 8.0]], [[3.0, 4.0, 4.0], [2.0, 6.0, 4.0], [3.0, 4.0, 4.0]], [[2.0, 4.0, 4.0], [3.0, 3.0, 4.0], [8.0, 6.0, 
4.0]]], [[[1.0, 2.0, 5.0], [5.0, 3.0, 7.0], [2.0, 6.0, 5.0]], [[4.0, 4.0, 1.0], [4.0, 1.0, 4.0], [8.0, 2.0, 1.0]], [[4.0, 7.0, 
3.0], [7.0, 6.0, 8.0], [7.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [1.0, 1.0, 5.0], [2.0, 3.0, 3.0]], 
[[1.0, 7.0, 4.0], [6.0, 1.0, 7.0], [6.0, 1.0, 3.0]], [[1.0, 6.0, 4.0], [1.0, 8.0, 8.0], [6.0, 2.0, 1.0]]], [[[3.0, 2.0, 5.0], 
[2.0, 4.0, 3.0], [1.0, 4.0, 5.0]], [[3.0, 6.0, 8.0], [5.0, 2.0, 2.0], [2.0, 7.0, 5.0]], [[4.0, 5.0, 2.0], [3.0, 8.0, 7.0], 
[4.0, 6.0, 5.0]]], [[[4.0, 1.0, 2.0], [8.0, 3.0, 6.0], [2.0, 8.0, 6.0]], [[2.0, 5.0, 5.0], [8.0, 4.0, 8.0], [6.0, 7.0, 1.0]], 
[[1.0, 8.0, 6.0], [4.0, 1.0, 7.0], [1.0, 6.0, 8.0]]], [[[1.0, 3.0, 5.0], [8.0, 8.0, 5.0], [1.0, 3.0, 8.0]], [[4.0, 7.0, 1.0], 
[4.0, 4.0, 8.0], [5.0, 3.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 1.0, 1.0], [7.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LUPN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 3.0], [8.0, 5.0, 1.0], [3.0, 1.0, 2.0]], [[2.0, 4.0, 2.0], [6.0, 5.0, 5.0], [5.0, 4.0, 
5.0]], [[2.0, 7.0, 6.0], [4.0, 3.0, 1.0], [1.0, 8.0, 4.0]]], [[[1.0, 5.0, 5.0], [8.0, 2.0, 2.0], [5.0, 2.0, 6.0]], [[8.0, 5.0, 
3.0], [1.0, 7.0, 5.0], [5.0, 8.0, 4.0]], [[3.0, 7.0, 2.0], [7.0, 4.0, 1.0], [2.0, 1.0, 4.0]]], [[[2.0, 1.0, 8.0], [7.0, 5.0, 
8.0], [4.0, 1.0, 7.0]], [[7.0, 5.0, 4.0], [4.0, 7.0, 6.0], [1.0, 3.0, 2.0]], [[1.0, 1.0, 8.0], [8.0, 7.0, 6.0], [4.0, 8.0, 
4.0]]], [[[7.0, 6.0, 8.0], [8.0, 6.0, 2.0], [6.0, 7.0, 6.0]], [[8.0, 8.0, 6.0], [5.0, 6.0, 3.0], [5.0, 4.0, 4.0]], [[4.0, 4.0, 
7.0], [8.0, 4.0, 1.0], [1.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 8.0], [5.0, 1.0, 3.0], [6.0, 4.0, 7.0]], [[8.0, 3.0, 3.0], [6.0, 8.0, 
6.0], [3.0, 8.0, 6.0]], [[7.0, 3.0, 3.0], [2.0, 8.0, 7.0], [1.0, 7.0, 2.0]]], [[[6.0, 7.0, 4.0], [1.0, 7.0, 8.0], [5.0, 2.0, 
1.0]], [[6.0, 5.0, 2.0], [5.0, 6.0, 4.0], [7.0, 8.0, 1.0]], [[5.0, 6.0, 2.0], [2.0, 8.0, 5.0], [6.0, 3.0, 5.0]]], [[[1.0, 2.0, 
6.0], [8.0, 3.0, 7.0], [4.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [2.0, 6.0, 7.0], [8.0, 7.0, 8.0]], [[3.0, 3.0, 1.0], [1.0, 1.0, 
8.0], [2.0, 5.0, 4.0]]], [[[7.0, 1.0, 2.0], [6.0, 5.0, 4.0], [2.0, 8.0, 4.0]], [[2.0, 5.0, 2.0], [2.0, 1.0, 8.0], [1.0, 7.0, 
5.0]], [[7.0, 8.0, 3.0], [8.0, 2.0, 2.0], [1.0, 1.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 3.0, 1.0], [2.0, 7.0, 4.0]], [[7.0, 5.0, 1.0], [3.0, 3.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[3.0, 1.0, 6.0], [1.0, 7.0, 1.0]], [[6.0, 4.0, 4.0], [5.0, 5.0, 2.0]]])
      arg[2:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [8.0, 5.0, 1.0], [3.0, 1.0, 2.0]], [[2.0, 4.0, 2.0], [6.0, 5.0, 5.0], [5.0, 
4.0, 5.0]], [[2.0, 7.0, 6.0], [4.0, 3.0, 1.0], [1.0, 8.0, 4.0]]], [[[1.0, 5.0, 5.0], [8.0, 2.0, 2.0], [5.0, 2.0, 6.0]], [[8.0, 
5.0, 3.0], [1.0, 7.0, 5.0], [5.0, 8.0, 4.0]], [[3.0, 7.0, 2.0], [7.0, 4.0, 1.0], [2.0, 1.0, 4.0]]], [[[2.0, 1.0, 8.0], [8.0, 
3.0, 1.0], [4.0, 1.0, 7.0]], [[7.0, 5.0, 4.0], [2.0, 7.0, 4.0], [1.0, 3.0, 2.0]], [[1.0, 1.0, 8.0], [8.0, 7.0, 6.0], [4.0, 8.0, 
4.0]]], [[[7.0, 6.0, 8.0], [7.0, 5.0, 1.0], [6.0, 7.0, 6.0]], [[8.0, 8.0, 6.0], [3.0, 3.0, 4.0], [5.0, 4.0, 4.0]], [[4.0, 4.0, 
7.0], [8.0, 4.0, 1.0], [1.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 8.0], [5.0, 1.0, 3.0], [6.0, 4.0, 7.0]], 
[[8.0, 3.0, 3.0], [6.0, 8.0, 6.0], [3.0, 8.0, 6.0]], [[7.0, 3.0, 3.0], [2.0, 8.0, 7.0], [1.0, 7.0, 2.0]]], [[[6.0, 7.0, 4.0], 
[1.0, 7.0, 8.0], [5.0, 2.0, 1.0]], [[6.0, 5.0, 2.0], [5.0, 6.0, 4.0], [7.0, 8.0, 1.0]], [[5.0, 6.0, 2.0], [2.0, 8.0, 5.0], 
[6.0, 3.0, 5.0]]], [[[1.0, 2.0, 6.0], [3.0, 1.0, 6.0], [4.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [1.0, 7.0, 1.0], [8.0, 7.0, 8.0]], 
[[3.0, 3.0, 1.0], [1.0, 1.0, 8.0], [2.0, 5.0, 4.0]]], [[[7.0, 1.0, 2.0], [6.0, 4.0, 4.0], [2.0, 8.0, 4.0]], [[2.0, 5.0, 2.0], 
[5.0, 5.0, 2.0], [1.0, 7.0, 5.0]], [[7.0, 8.0, 3.0], [8.0, 2.0, 2.0], [1.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_URUU(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 6.0], [8.0, 3.0, 5.0], [4.0, 2.0, 6.0]], [[8.0, 5.0, 5.0], [7.0, 7.0, 8.0], [2.0, 1.0, 
1.0]], [[2.0, 7.0, 6.0], [5.0, 7.0, 6.0], [6.0, 5.0, 5.0]]], [[[1.0, 2.0, 6.0], [1.0, 3.0, 1.0], [8.0, 1.0, 1.0]], [[2.0, 8.0, 
7.0], [8.0, 6.0, 7.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 6.0], [1.0, 8.0, 4.0], [6.0, 4.0, 2.0]]], [[[7.0, 1.0, 6.0], [8.0, 7.0, 
1.0], [4.0, 5.0, 4.0]], [[4.0, 5.0, 8.0], [1.0, 1.0, 7.0], [2.0, 2.0, 2.0]], [[3.0, 4.0, 5.0], [4.0, 3.0, 2.0], [5.0, 2.0, 
2.0]]], [[[3.0, 5.0, 7.0], [5.0, 1.0, 3.0], [6.0, 1.0, 5.0]], [[5.0, 3.0, 4.0], [1.0, 8.0, 7.0], [4.0, 7.0, 1.0]], [[7.0, 8.0, 
8.0], [4.0, 2.0, 5.0], [4.0, 3.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 7.0], [6.0, 6.0, 4.0], [3.0, 8.0, 5.0]], [[4.0, 8.0, 7.0], [4.0, 4.0, 
5.0], [1.0, 4.0, 4.0]], [[7.0, 5.0, 7.0], [2.0, 2.0, 4.0], [7.0, 4.0, 1.0]]], [[[1.0, 6.0, 6.0], [4.0, 8.0, 7.0], [3.0, 4.0, 
3.0]], [[2.0, 5.0, 6.0], [2.0, 8.0, 7.0], [5.0, 8.0, 5.0]], [[8.0, 3.0, 3.0], [3.0, 1.0, 5.0], [1.0, 6.0, 6.0]]], [[[5.0, 4.0, 
7.0], [4.0, 3.0, 6.0], [8.0, 1.0, 6.0]], [[3.0, 3.0, 4.0], [3.0, 1.0, 5.0], [3.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], [3.0, 4.0, 
8.0], [6.0, 3.0, 5.0]]], [[[3.0, 4.0, 3.0], [5.0, 6.0, 2.0], [8.0, 7.0, 4.0]], [[5.0, 1.0, 2.0], [1.0, 7.0, 8.0], [2.0, 4.0, 
3.0]], [[1.0, 1.0, 8.0], [8.0, 4.0, 7.0], [1.0, 3.0, 4.0]]]]))
      val=numpy.array([[[[1.0, 5.0], [7.0, 7.0]]], [[[7.0, 3.0], [8.0, 6.0]]], [[[6.0, 2.0], [7.0, 2.0]]]])
      arg[:3,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 6.0], [8.0, 3.0, 5.0], [4.0, 2.0, 6.0]], [[1.0, 5.0, 5.0], [7.0, 7.0, 8.0], [2.0, 
1.0, 1.0]], [[2.0, 7.0, 6.0], [5.0, 7.0, 6.0], [6.0, 5.0, 5.0]]], [[[1.0, 2.0, 6.0], [1.0, 3.0, 1.0], [8.0, 1.0, 1.0]], [[7.0, 
3.0, 7.0], [8.0, 6.0, 7.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 6.0], [1.0, 8.0, 4.0], [6.0, 4.0, 2.0]]], [[[7.0, 1.0, 6.0], [8.0, 
7.0, 1.0], [4.0, 5.0, 4.0]], [[6.0, 2.0, 8.0], [7.0, 2.0, 7.0], [2.0, 2.0, 2.0]], [[3.0, 4.0, 5.0], [4.0, 3.0, 2.0], [5.0, 2.0, 
2.0]]], [[[3.0, 5.0, 7.0], [5.0, 1.0, 3.0], [6.0, 1.0, 5.0]], [[5.0, 3.0, 4.0], [1.0, 8.0, 7.0], [4.0, 7.0, 1.0]], [[7.0, 8.0, 
8.0], [4.0, 2.0, 5.0], [4.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 7.0], [6.0, 6.0, 4.0], [3.0, 8.0, 5.0]], 
[[1.0, 5.0, 7.0], [7.0, 7.0, 5.0], [1.0, 4.0, 4.0]], [[7.0, 5.0, 7.0], [2.0, 2.0, 4.0], [7.0, 4.0, 1.0]]], [[[1.0, 6.0, 6.0], 
[4.0, 8.0, 7.0], [3.0, 4.0, 3.0]], [[7.0, 3.0, 6.0], [8.0, 6.0, 7.0], [5.0, 8.0, 5.0]], [[8.0, 3.0, 3.0], [3.0, 1.0, 5.0], 
[1.0, 6.0, 6.0]]], [[[5.0, 4.0, 7.0], [4.0, 3.0, 6.0], [8.0, 1.0, 6.0]], [[6.0, 2.0, 4.0], [7.0, 2.0, 5.0], [3.0, 4.0, 5.0]], 
[[7.0, 4.0, 7.0], [3.0, 4.0, 8.0], [6.0, 3.0, 5.0]]], [[[3.0, 4.0, 3.0], [5.0, 6.0, 2.0], [8.0, 7.0, 4.0]], [[5.0, 1.0, 2.0], 
[1.0, 7.0, 8.0], [2.0, 4.0, 3.0]], [[1.0, 1.0, 8.0], [8.0, 4.0, 7.0], [1.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_URUU(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 4.0], [2.0, 8.0, 1.0], [8.0, 6.0, 5.0]], [[4.0, 7.0, 3.0], [3.0, 1.0, 5.0], [8.0, 7.0, 
4.0]], [[5.0, 7.0, 2.0], [2.0, 1.0, 5.0], [3.0, 4.0, 4.0]]], [[[7.0, 5.0, 1.0], [3.0, 7.0, 8.0], [4.0, 3.0, 3.0]], [[8.0, 1.0, 
6.0], [4.0, 3.0, 6.0], [4.0, 8.0, 1.0]], [[8.0, 6.0, 7.0], [6.0, 6.0, 2.0], [7.0, 4.0, 5.0]]], [[[2.0, 4.0, 6.0], [5.0, 6.0, 
3.0], [1.0, 2.0, 1.0]], [[2.0, 5.0, 6.0], [7.0, 4.0, 2.0], [3.0, 8.0, 5.0]], [[6.0, 4.0, 2.0], [7.0, 1.0, 1.0], [2.0, 2.0, 
6.0]]], [[[1.0, 4.0, 6.0], [7.0, 3.0, 4.0], [4.0, 7.0, 8.0]], [[4.0, 3.0, 6.0], [5.0, 6.0, 3.0], [2.0, 3.0, 1.0]], [[6.0, 5.0, 
3.0], [6.0, 6.0, 1.0], [2.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 1.0], [3.0, 3.0, 3.0], [6.0, 1.0, 4.0]], [[8.0, 7.0, 8.0], [7.0, 1.0, 
3.0], [6.0, 6.0, 5.0]], [[7.0, 8.0, 5.0], [6.0, 1.0, 2.0], [8.0, 7.0, 6.0]]], [[[6.0, 2.0, 3.0], [5.0, 4.0, 6.0], [7.0, 5.0, 
5.0]], [[8.0, 5.0, 8.0], [2.0, 4.0, 2.0], [5.0, 4.0, 6.0]], [[5.0, 1.0, 3.0], [5.0, 7.0, 8.0], [2.0, 2.0, 6.0]]], [[[8.0, 6.0, 
7.0], [6.0, 2.0, 7.0], [3.0, 7.0, 2.0]], [[6.0, 1.0, 4.0], [1.0, 8.0, 1.0], [5.0, 3.0, 2.0]], [[6.0, 8.0, 7.0], [4.0, 4.0, 
6.0], [3.0, 6.0, 4.0]]], [[[4.0, 3.0, 6.0], [5.0, 1.0, 4.0], [4.0, 1.0, 3.0]], [[8.0, 4.0, 6.0], [4.0, 4.0, 4.0], [3.0, 6.0, 
3.0]], [[2.0, 6.0, 7.0], [3.0, 6.0, 6.0], [2.0, 1.0, 7.0]]]]))
      val=Data(numpy.array([[[[5.0, 7.0], [3.0, 3.0]]], [[[5.0, 6.0], [4.0, 1.0]]], [[[1.0, 6.0], [4.0, 
4.0]]]]),self.functionspace)
      arg[:3,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 4.0], [2.0, 8.0, 1.0], [8.0, 6.0, 5.0]], [[5.0, 7.0, 3.0], [3.0, 3.0, 5.0], [8.0, 
7.0, 4.0]], [[5.0, 7.0, 2.0], [2.0, 1.0, 5.0], [3.0, 4.0, 4.0]]], [[[7.0, 5.0, 1.0], [3.0, 7.0, 8.0], [4.0, 3.0, 3.0]], [[5.0, 
6.0, 6.0], [4.0, 1.0, 6.0], [4.0, 8.0, 1.0]], [[8.0, 6.0, 7.0], [6.0, 6.0, 2.0], [7.0, 4.0, 5.0]]], [[[2.0, 4.0, 6.0], [5.0, 
6.0, 3.0], [1.0, 2.0, 1.0]], [[1.0, 6.0, 6.0], [4.0, 4.0, 2.0], [3.0, 8.0, 5.0]], [[6.0, 4.0, 2.0], [7.0, 1.0, 1.0], [2.0, 2.0, 
6.0]]], [[[1.0, 4.0, 6.0], [7.0, 3.0, 4.0], [4.0, 7.0, 8.0]], [[4.0, 3.0, 6.0], [5.0, 6.0, 3.0], [2.0, 3.0, 1.0]], [[6.0, 5.0, 
3.0], [6.0, 6.0, 1.0], [2.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 1.0], [3.0, 3.0, 3.0], [6.0, 1.0, 4.0]], 
[[5.0, 7.0, 8.0], [3.0, 3.0, 3.0], [6.0, 6.0, 5.0]], [[7.0, 8.0, 5.0], [6.0, 1.0, 2.0], [8.0, 7.0, 6.0]]], [[[6.0, 2.0, 3.0], 
[5.0, 4.0, 6.0], [7.0, 5.0, 5.0]], [[5.0, 6.0, 8.0], [4.0, 1.0, 2.0], [5.0, 4.0, 6.0]], [[5.0, 1.0, 3.0], [5.0, 7.0, 8.0], 
[2.0, 2.0, 6.0]]], [[[8.0, 6.0, 7.0], [6.0, 2.0, 7.0], [3.0, 7.0, 2.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 1.0], [5.0, 3.0, 2.0]], 
[[6.0, 8.0, 7.0], [4.0, 4.0, 6.0], [3.0, 6.0, 4.0]]], [[[4.0, 3.0, 6.0], [5.0, 1.0, 4.0], [4.0, 1.0, 3.0]], [[8.0, 4.0, 6.0], 
[4.0, 4.0, 4.0], [3.0, 6.0, 3.0]], [[2.0, 6.0, 7.0], [3.0, 6.0, 6.0], [2.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_URUU(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 5.0], [7.0, 8.0, 3.0], [5.0, 2.0, 2.0]], [[2.0, 1.0, 1.0], [5.0, 7.0, 8.0], [3.0, 5.0, 
6.0]], [[6.0, 8.0, 8.0], [8.0, 1.0, 7.0], [5.0, 3.0, 8.0]]], [[[7.0, 6.0, 3.0], [3.0, 7.0, 2.0], [1.0, 2.0, 8.0]], [[6.0, 6.0, 
8.0], [1.0, 5.0, 3.0], [8.0, 5.0, 4.0]], [[2.0, 5.0, 4.0], [2.0, 1.0, 6.0], [2.0, 7.0, 5.0]]], [[[7.0, 5.0, 3.0], [4.0, 4.0, 
3.0], [3.0, 5.0, 8.0]], [[7.0, 6.0, 2.0], [6.0, 3.0, 3.0], [4.0, 2.0, 7.0]], [[6.0, 5.0, 3.0], [1.0, 1.0, 5.0], [3.0, 2.0, 
6.0]]], [[[1.0, 5.0, 1.0], [2.0, 8.0, 7.0], [8.0, 7.0, 2.0]], [[5.0, 6.0, 7.0], [6.0, 8.0, 4.0], [3.0, 4.0, 4.0]], [[3.0, 1.0, 
2.0], [1.0, 1.0, 7.0], [7.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 1.0], [1.0, 3.0, 7.0], [3.0, 6.0, 3.0]], [[6.0, 8.0, 4.0], [6.0, 5.0, 
1.0], [7.0, 6.0, 2.0]], [[7.0, 1.0, 8.0], [4.0, 8.0, 3.0], [8.0, 5.0, 3.0]]], [[[6.0, 3.0, 3.0], [6.0, 2.0, 6.0], [6.0, 7.0, 
5.0]], [[6.0, 6.0, 6.0], [4.0, 4.0, 3.0], [7.0, 5.0, 1.0]], [[5.0, 7.0, 6.0], [3.0, 2.0, 2.0], [4.0, 8.0, 4.0]]], [[[4.0, 5.0, 
4.0], [5.0, 1.0, 1.0], [4.0, 2.0, 2.0]], [[2.0, 5.0, 6.0], [3.0, 1.0, 1.0], [7.0, 4.0, 3.0]], [[5.0, 2.0, 2.0], [3.0, 2.0, 
7.0], [1.0, 5.0, 1.0]]], [[[7.0, 6.0, 5.0], [8.0, 3.0, 1.0], [2.0, 4.0, 7.0]], [[8.0, 7.0, 8.0], [8.0, 5.0, 2.0], [7.0, 7.0, 
1.0]], [[3.0, 3.0, 8.0], [4.0, 5.0, 4.0], [8.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[[[7.0, 2.0], [8.0, 7.0]]], [[[1.0, 7.0], [1.0, 3.0]]], [[[3.0, 1.0], [1.0, 
7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 8.0], [6.0, 4.0]]], [[[4.0, 4.0], [1.0, 2.0]]], [[[2.0, 8.0], [7.0, 4.0]]]]))
      arg[:3,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [7.0, 8.0, 3.0], [5.0, 2.0, 2.0]], [[7.0, 2.0, 1.0], [8.0, 7.0, 8.0], [3.0, 
5.0, 6.0]], [[6.0, 8.0, 8.0], [8.0, 1.0, 7.0], [5.0, 3.0, 8.0]]], [[[7.0, 6.0, 3.0], [3.0, 7.0, 2.0], [1.0, 2.0, 8.0]], [[1.0, 
7.0, 8.0], [1.0, 3.0, 3.0], [8.0, 5.0, 4.0]], [[2.0, 5.0, 4.0], [2.0, 1.0, 6.0], [2.0, 7.0, 5.0]]], [[[7.0, 5.0, 3.0], [4.0, 
4.0, 3.0], [3.0, 5.0, 8.0]], [[3.0, 1.0, 2.0], [1.0, 7.0, 3.0], [4.0, 2.0, 7.0]], [[6.0, 5.0, 3.0], [1.0, 1.0, 5.0], [3.0, 2.0, 
6.0]]], [[[1.0, 5.0, 1.0], [2.0, 8.0, 7.0], [8.0, 7.0, 2.0]], [[5.0, 6.0, 7.0], [6.0, 8.0, 4.0], [3.0, 4.0, 4.0]], [[3.0, 1.0, 
2.0], [1.0, 1.0, 7.0], [7.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 1.0], [1.0, 3.0, 7.0], [3.0, 6.0, 3.0]], 
[[2.0, 8.0, 4.0], [6.0, 4.0, 1.0], [7.0, 6.0, 2.0]], [[7.0, 1.0, 8.0], [4.0, 8.0, 3.0], [8.0, 5.0, 3.0]]], [[[6.0, 3.0, 3.0], 
[6.0, 2.0, 6.0], [6.0, 7.0, 5.0]], [[4.0, 4.0, 6.0], [1.0, 2.0, 3.0], [7.0, 5.0, 1.0]], [[5.0, 7.0, 6.0], [3.0, 2.0, 2.0], 
[4.0, 8.0, 4.0]]], [[[4.0, 5.0, 4.0], [5.0, 1.0, 1.0], [4.0, 2.0, 2.0]], [[2.0, 8.0, 6.0], [7.0, 4.0, 1.0], [7.0, 4.0, 3.0]], 
[[5.0, 2.0, 2.0], [3.0, 2.0, 7.0], [1.0, 5.0, 1.0]]], [[[7.0, 6.0, 5.0], [8.0, 3.0, 1.0], [2.0, 4.0, 7.0]], [[8.0, 7.0, 8.0], 
[8.0, 5.0, 2.0], [7.0, 7.0, 1.0]], [[3.0, 3.0, 8.0], [4.0, 5.0, 4.0], [8.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_URUU(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 5.0], [1.0, 6.0, 3.0], [3.0, 1.0, 4.0]], [[4.0, 8.0, 8.0], [6.0, 4.0, 4.0], [8.0, 1.0, 
4.0]], [[1.0, 3.0, 8.0], [4.0, 1.0, 3.0], [3.0, 3.0, 3.0]]], [[[4.0, 1.0, 4.0], [1.0, 7.0, 5.0], [7.0, 6.0, 7.0]], [[4.0, 3.0, 
5.0], [2.0, 8.0, 7.0], [7.0, 8.0, 2.0]], [[4.0, 1.0, 8.0], [3.0, 8.0, 2.0], [1.0, 4.0, 7.0]]], [[[5.0, 2.0, 8.0], [6.0, 2.0, 
6.0], [8.0, 3.0, 3.0]], [[8.0, 2.0, 5.0], [3.0, 2.0, 6.0], [4.0, 4.0, 2.0]], [[1.0, 8.0, 1.0], [4.0, 7.0, 1.0], [1.0, 8.0, 
2.0]]], [[[8.0, 1.0, 4.0], [3.0, 5.0, 6.0], [1.0, 3.0, 7.0]], [[1.0, 1.0, 8.0], [5.0, 1.0, 3.0], [6.0, 4.0, 2.0]], [[1.0, 4.0, 
5.0], [2.0, 2.0, 4.0], [2.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 3.0], [3.0, 3.0, 2.0], [7.0, 2.0, 5.0]], [[5.0, 1.0, 8.0], [1.0, 3.0, 
4.0], [4.0, 1.0, 7.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 3.0], [3.0, 8.0, 1.0]]], [[[6.0, 4.0, 1.0], [3.0, 5.0, 2.0], [6.0, 5.0, 
1.0]], [[6.0, 4.0, 1.0], [7.0, 8.0, 8.0], [6.0, 6.0, 1.0]], [[4.0, 4.0, 1.0], [4.0, 7.0, 4.0], [8.0, 3.0, 1.0]]], [[[1.0, 5.0, 
6.0], [1.0, 6.0, 5.0], [4.0, 2.0, 6.0]], [[6.0, 5.0, 2.0], [7.0, 2.0, 6.0], [4.0, 4.0, 8.0]], [[8.0, 4.0, 2.0], [2.0, 1.0, 
1.0], [2.0, 7.0, 2.0]]], [[[8.0, 3.0, 3.0], [8.0, 5.0, 4.0], [5.0, 8.0, 5.0]], [[4.0, 5.0, 2.0], [4.0, 8.0, 7.0], [5.0, 2.0, 
8.0]], [[7.0, 2.0, 8.0], [8.0, 6.0, 2.0], [4.0, 2.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 5.0], [3.0, 6.0]]], [[[6.0, 8.0], [4.0, 6.0]]], [[[1.0, 4.0], [5.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 1.0], [4.0, 6.0]]], [[[6.0, 6.0], [8.0, 3.0]]], [[[8.0, 2.0], [2.0, 8.0]]]])
      arg[:3,1:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 5.0], [1.0, 6.0, 3.0], [3.0, 1.0, 4.0]], [[8.0, 5.0, 8.0], [3.0, 6.0, 4.0], [8.0, 
1.0, 4.0]], [[1.0, 3.0, 8.0], [4.0, 1.0, 3.0], [3.0, 3.0, 3.0]]], [[[4.0, 1.0, 4.0], [1.0, 7.0, 5.0], [7.0, 6.0, 7.0]], [[6.0, 
8.0, 5.0], [4.0, 6.0, 7.0], [7.0, 8.0, 2.0]], [[4.0, 1.0, 8.0], [3.0, 8.0, 2.0], [1.0, 4.0, 7.0]]], [[[5.0, 2.0, 8.0], [6.0, 
2.0, 6.0], [8.0, 3.0, 3.0]], [[1.0, 4.0, 5.0], [5.0, 3.0, 6.0], [4.0, 4.0, 2.0]], [[1.0, 8.0, 1.0], [4.0, 7.0, 1.0], [1.0, 8.0, 
2.0]]], [[[8.0, 1.0, 4.0], [3.0, 5.0, 6.0], [1.0, 3.0, 7.0]], [[1.0, 1.0, 8.0], [5.0, 1.0, 3.0], [6.0, 4.0, 2.0]], [[1.0, 4.0, 
5.0], [2.0, 2.0, 4.0], [2.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 3.0], [3.0, 3.0, 2.0], [7.0, 2.0, 5.0]], 
[[2.0, 1.0, 8.0], [4.0, 6.0, 4.0], [4.0, 1.0, 7.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 3.0], [3.0, 8.0, 1.0]]], [[[6.0, 4.0, 1.0], 
[3.0, 5.0, 2.0], [6.0, 5.0, 1.0]], [[6.0, 6.0, 1.0], [8.0, 3.0, 8.0], [6.0, 6.0, 1.0]], [[4.0, 4.0, 1.0], [4.0, 7.0, 4.0], 
[8.0, 3.0, 1.0]]], [[[1.0, 5.0, 6.0], [1.0, 6.0, 5.0], [4.0, 2.0, 6.0]], [[8.0, 2.0, 2.0], [2.0, 8.0, 6.0], [4.0, 4.0, 8.0]], 
[[8.0, 4.0, 2.0], [2.0, 1.0, 1.0], [2.0, 7.0, 2.0]]], [[[8.0, 3.0, 3.0], [8.0, 5.0, 4.0], [5.0, 8.0, 5.0]], [[4.0, 5.0, 2.0], 
[4.0, 8.0, 7.0], [5.0, 2.0, 8.0]], [[7.0, 2.0, 8.0], [8.0, 6.0, 2.0], [4.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PCRN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 5.0], [5.0, 5.0, 7.0], [5.0, 3.0, 1.0]], [[1.0, 3.0, 1.0], [7.0, 4.0, 6.0], [4.0, 8.0, 
1.0]], [[6.0, 7.0, 6.0], [4.0, 6.0, 3.0], [3.0, 8.0, 5.0]]], [[[3.0, 2.0, 4.0], [4.0, 6.0, 5.0], [7.0, 7.0, 6.0]], [[5.0, 6.0, 
5.0], [5.0, 1.0, 4.0], [8.0, 3.0, 1.0]], [[5.0, 4.0, 6.0], [7.0, 4.0, 4.0], [7.0, 7.0, 8.0]]], [[[8.0, 7.0, 3.0], [8.0, 6.0, 
4.0], [4.0, 5.0, 6.0]], [[1.0, 8.0, 2.0], [5.0, 8.0, 6.0], [2.0, 4.0, 2.0]], [[1.0, 4.0, 5.0], [7.0, 4.0, 3.0], [6.0, 3.0, 
8.0]]], [[[5.0, 7.0, 5.0], [7.0, 1.0, 2.0], [7.0, 1.0, 5.0]], [[7.0, 8.0, 5.0], [5.0, 3.0, 3.0], [7.0, 8.0, 8.0]], [[7.0, 3.0, 
2.0], [6.0, 8.0, 4.0], [3.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 3.0], [3.0, 2.0, 6.0], [1.0, 6.0, 5.0]], [[7.0, 1.0, 4.0], [6.0, 2.0, 
4.0], [7.0, 4.0, 7.0]], [[1.0, 1.0, 3.0], [3.0, 2.0, 7.0], [2.0, 6.0, 1.0]]], [[[3.0, 3.0, 4.0], [5.0, 6.0, 5.0], [7.0, 8.0, 
8.0]], [[5.0, 5.0, 4.0], [6.0, 1.0, 2.0], [1.0, 8.0, 2.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 6.0], [5.0, 7.0, 8.0]]], [[[3.0, 3.0, 
6.0], [7.0, 7.0, 3.0], [5.0, 3.0, 3.0]], [[2.0, 8.0, 8.0], [4.0, 2.0, 1.0], [2.0, 2.0, 8.0]], [[4.0, 2.0, 4.0], [5.0, 5.0, 
3.0], [8.0, 1.0, 7.0]]], [[[2.0, 5.0, 6.0], [6.0, 5.0, 2.0], [3.0, 7.0, 5.0]], [[7.0, 3.0, 8.0], [8.0, 1.0, 8.0], [5.0, 2.0, 
1.0]], [[3.0, 8.0, 6.0], [8.0, 7.0, 6.0], [3.0, 4.0, 7.0]]]]))
      val=numpy.array([[[5.0, 5.0, 6.0]], [[7.0, 2.0, 3.0]], [[6.0, 6.0, 6.0]]])
      arg[2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 5.0], [5.0, 5.0, 7.0], [5.0, 3.0, 1.0]], [[1.0, 3.0, 1.0], [7.0, 4.0, 6.0], [4.0, 
8.0, 1.0]], [[6.0, 7.0, 6.0], [4.0, 6.0, 3.0], [3.0, 8.0, 5.0]]], [[[3.0, 2.0, 4.0], [4.0, 6.0, 5.0], [7.0, 7.0, 6.0]], [[5.0, 
6.0, 5.0], [5.0, 1.0, 4.0], [8.0, 3.0, 1.0]], [[5.0, 4.0, 6.0], [7.0, 4.0, 4.0], [7.0, 7.0, 8.0]]], [[[8.0, 7.0, 3.0], [5.0, 
5.0, 6.0], [4.0, 5.0, 6.0]], [[1.0, 8.0, 2.0], [7.0, 2.0, 3.0], [2.0, 4.0, 2.0]], [[1.0, 4.0, 5.0], [6.0, 6.0, 6.0], [6.0, 3.0, 
8.0]]], [[[5.0, 7.0, 5.0], [7.0, 1.0, 2.0], [7.0, 1.0, 5.0]], [[7.0, 8.0, 5.0], [5.0, 3.0, 3.0], [7.0, 8.0, 8.0]], [[7.0, 3.0, 
2.0], [6.0, 8.0, 4.0], [3.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 3.0], [3.0, 2.0, 6.0], [1.0, 6.0, 5.0]], 
[[7.0, 1.0, 4.0], [6.0, 2.0, 4.0], [7.0, 4.0, 7.0]], [[1.0, 1.0, 3.0], [3.0, 2.0, 7.0], [2.0, 6.0, 1.0]]], [[[3.0, 3.0, 4.0], 
[5.0, 6.0, 5.0], [7.0, 8.0, 8.0]], [[5.0, 5.0, 4.0], [6.0, 1.0, 2.0], [1.0, 8.0, 2.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 6.0], 
[5.0, 7.0, 8.0]]], [[[3.0, 3.0, 6.0], [5.0, 5.0, 6.0], [5.0, 3.0, 3.0]], [[2.0, 8.0, 8.0], [7.0, 2.0, 3.0], [2.0, 2.0, 8.0]], 
[[4.0, 2.0, 4.0], [6.0, 6.0, 6.0], [8.0, 1.0, 7.0]]], [[[2.0, 5.0, 6.0], [6.0, 5.0, 2.0], [3.0, 7.0, 5.0]], [[7.0, 3.0, 8.0], 
[8.0, 1.0, 8.0], [5.0, 2.0, 1.0]], [[3.0, 8.0, 6.0], [8.0, 7.0, 6.0], [3.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PCRN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 7.0], [7.0, 1.0, 6.0], [7.0, 5.0, 1.0]], [[6.0, 1.0, 3.0], [4.0, 8.0, 7.0], [4.0, 1.0, 
8.0]], [[7.0, 8.0, 7.0], [2.0, 7.0, 8.0], [2.0, 7.0, 6.0]]], [[[4.0, 2.0, 4.0], [6.0, 8.0, 4.0], [2.0, 6.0, 2.0]], [[7.0, 1.0, 
4.0], [3.0, 8.0, 2.0], [2.0, 6.0, 2.0]], [[7.0, 8.0, 7.0], [6.0, 3.0, 2.0], [3.0, 3.0, 2.0]]], [[[2.0, 8.0, 1.0], [4.0, 7.0, 
6.0], [3.0, 4.0, 4.0]], [[1.0, 7.0, 3.0], [8.0, 6.0, 7.0], [1.0, 4.0, 3.0]], [[8.0, 1.0, 2.0], [8.0, 1.0, 5.0], [2.0, 4.0, 
7.0]]], [[[1.0, 6.0, 5.0], [4.0, 6.0, 4.0], [4.0, 2.0, 5.0]], [[7.0, 3.0, 8.0], [4.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[2.0, 3.0, 
6.0], [1.0, 3.0, 8.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 4.0], [3.0, 7.0, 3.0], [2.0, 4.0, 2.0]], [[2.0, 2.0, 8.0], [1.0, 6.0, 
4.0], [6.0, 6.0, 8.0]], [[2.0, 8.0, 7.0], [2.0, 6.0, 5.0], [2.0, 7.0, 1.0]]], [[[8.0, 4.0, 7.0], [7.0, 4.0, 8.0], [7.0, 8.0, 
2.0]], [[8.0, 5.0, 7.0], [6.0, 5.0, 3.0], [6.0, 4.0, 3.0]], [[8.0, 5.0, 1.0], [7.0, 2.0, 5.0], [3.0, 3.0, 3.0]]], [[[6.0, 2.0, 
4.0], [7.0, 1.0, 1.0], [7.0, 6.0, 5.0]], [[7.0, 7.0, 5.0], [2.0, 3.0, 5.0], [7.0, 7.0, 1.0]], [[5.0, 4.0, 8.0], [7.0, 3.0, 
2.0], [1.0, 7.0, 8.0]]], [[[3.0, 2.0, 2.0], [5.0, 7.0, 6.0], [7.0, 3.0, 8.0]], [[5.0, 3.0, 1.0], [1.0, 2.0, 7.0], [6.0, 4.0, 
3.0]], [[7.0, 1.0, 1.0], [6.0, 3.0, 6.0], [1.0, 1.0, 7.0]]]]))
      val=Data(numpy.array([[[7.0, 5.0, 7.0]], [[1.0, 1.0, 1.0]], [[2.0, 1.0, 4.0]]]),self.functionspace)
      arg[2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 7.0], [7.0, 1.0, 6.0], [7.0, 5.0, 1.0]], [[6.0, 1.0, 3.0], [4.0, 8.0, 7.0], [4.0, 
1.0, 8.0]], [[7.0, 8.0, 7.0], [2.0, 7.0, 8.0], [2.0, 7.0, 6.0]]], [[[4.0, 2.0, 4.0], [6.0, 8.0, 4.0], [2.0, 6.0, 2.0]], [[7.0, 
1.0, 4.0], [3.0, 8.0, 2.0], [2.0, 6.0, 2.0]], [[7.0, 8.0, 7.0], [6.0, 3.0, 2.0], [3.0, 3.0, 2.0]]], [[[2.0, 8.0, 1.0], [7.0, 
5.0, 7.0], [3.0, 4.0, 4.0]], [[1.0, 7.0, 3.0], [1.0, 1.0, 1.0], [1.0, 4.0, 3.0]], [[8.0, 1.0, 2.0], [2.0, 1.0, 4.0], [2.0, 4.0, 
7.0]]], [[[1.0, 6.0, 5.0], [4.0, 6.0, 4.0], [4.0, 2.0, 5.0]], [[7.0, 3.0, 8.0], [4.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[2.0, 3.0, 
6.0], [1.0, 3.0, 8.0], [3.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [3.0, 7.0, 3.0], [2.0, 4.0, 2.0]], 
[[2.0, 2.0, 8.0], [1.0, 6.0, 4.0], [6.0, 6.0, 8.0]], [[2.0, 8.0, 7.0], [2.0, 6.0, 5.0], [2.0, 7.0, 1.0]]], [[[8.0, 4.0, 7.0], 
[7.0, 4.0, 8.0], [7.0, 8.0, 2.0]], [[8.0, 5.0, 7.0], [6.0, 5.0, 3.0], [6.0, 4.0, 3.0]], [[8.0, 5.0, 1.0], [7.0, 2.0, 5.0], 
[3.0, 3.0, 3.0]]], [[[6.0, 2.0, 4.0], [7.0, 5.0, 7.0], [7.0, 6.0, 5.0]], [[7.0, 7.0, 5.0], [1.0, 1.0, 1.0], [7.0, 7.0, 1.0]], 
[[5.0, 4.0, 8.0], [2.0, 1.0, 4.0], [1.0, 7.0, 8.0]]], [[[3.0, 2.0, 2.0], [5.0, 7.0, 6.0], [7.0, 3.0, 8.0]], [[5.0, 3.0, 1.0], 
[1.0, 2.0, 7.0], [6.0, 4.0, 3.0]], [[7.0, 1.0, 1.0], [6.0, 3.0, 6.0], [1.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PCRN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 6.0], [1.0, 3.0, 4.0], [2.0, 3.0, 6.0]], [[6.0, 2.0, 5.0], [4.0, 5.0, 3.0], [6.0, 4.0, 
4.0]], [[5.0, 6.0, 3.0], [2.0, 8.0, 8.0], [8.0, 3.0, 7.0]]], [[[7.0, 7.0, 3.0], [2.0, 7.0, 5.0], [6.0, 5.0, 3.0]], [[8.0, 2.0, 
1.0], [5.0, 7.0, 6.0], [5.0, 1.0, 5.0]], [[5.0, 5.0, 8.0], [3.0, 2.0, 6.0], [2.0, 5.0, 1.0]]], [[[5.0, 7.0, 5.0], [4.0, 1.0, 
4.0], [6.0, 6.0, 5.0]], [[3.0, 2.0, 5.0], [5.0, 1.0, 5.0], [2.0, 6.0, 7.0]], [[6.0, 5.0, 2.0], [1.0, 8.0, 8.0], [7.0, 1.0, 
2.0]]], [[[1.0, 2.0, 4.0], [8.0, 5.0, 5.0], [1.0, 1.0, 6.0]], [[5.0, 2.0, 4.0], [8.0, 5.0, 2.0], [6.0, 4.0, 3.0]], [[6.0, 2.0, 
2.0], [6.0, 8.0, 2.0], [7.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 7.0], [5.0, 7.0, 4.0], [4.0, 7.0, 7.0]], [[8.0, 8.0, 2.0], [2.0, 6.0, 
1.0], [4.0, 2.0, 8.0]], [[2.0, 5.0, 8.0], [6.0, 7.0, 7.0], [3.0, 7.0, 1.0]]], [[[8.0, 1.0, 1.0], [4.0, 7.0, 4.0], [7.0, 1.0, 
7.0]], [[7.0, 3.0, 6.0], [8.0, 1.0, 5.0], [3.0, 4.0, 4.0]], [[4.0, 2.0, 1.0], [3.0, 2.0, 1.0], [7.0, 3.0, 8.0]]], [[[1.0, 3.0, 
6.0], [3.0, 3.0, 7.0], [5.0, 5.0, 6.0]], [[5.0, 2.0, 1.0], [4.0, 6.0, 7.0], [4.0, 5.0, 4.0]], [[7.0, 2.0, 8.0], [3.0, 6.0, 
5.0], [3.0, 3.0, 2.0]]], [[[7.0, 6.0, 6.0], [3.0, 1.0, 5.0], [2.0, 3.0, 7.0]], [[2.0, 8.0, 5.0], [8.0, 8.0, 7.0], [5.0, 8.0, 
2.0]], [[6.0, 3.0, 1.0], [2.0, 4.0, 1.0], [4.0, 5.0, 4.0]]]]))
      val=Data(numpy.array([[[6.0, 6.0, 3.0]], [[5.0, 7.0, 1.0]], [[1.0, 5.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 7.0, 3.0]], [[3.0, 4.0, 1.0]], [[8.0, 6.0, 8.0]]]))
      arg[2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 6.0], [1.0, 3.0, 4.0], [2.0, 3.0, 6.0]], [[6.0, 2.0, 5.0], [4.0, 5.0, 3.0], [6.0, 
4.0, 4.0]], [[5.0, 6.0, 3.0], [2.0, 8.0, 8.0], [8.0, 3.0, 7.0]]], [[[7.0, 7.0, 3.0], [2.0, 7.0, 5.0], [6.0, 5.0, 3.0]], [[8.0, 
2.0, 1.0], [5.0, 7.0, 6.0], [5.0, 1.0, 5.0]], [[5.0, 5.0, 8.0], [3.0, 2.0, 6.0], [2.0, 5.0, 1.0]]], [[[5.0, 7.0, 5.0], [6.0, 
6.0, 3.0], [6.0, 6.0, 5.0]], [[3.0, 2.0, 5.0], [5.0, 7.0, 1.0], [2.0, 6.0, 7.0]], [[6.0, 5.0, 2.0], [1.0, 5.0, 5.0], [7.0, 1.0, 
2.0]]], [[[1.0, 2.0, 4.0], [8.0, 5.0, 5.0], [1.0, 1.0, 6.0]], [[5.0, 2.0, 4.0], [8.0, 5.0, 2.0], [6.0, 4.0, 3.0]], [[6.0, 2.0, 
2.0], [6.0, 8.0, 2.0], [7.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 7.0], [5.0, 7.0, 4.0], [4.0, 7.0, 7.0]], 
[[8.0, 8.0, 2.0], [2.0, 6.0, 1.0], [4.0, 2.0, 8.0]], [[2.0, 5.0, 8.0], [6.0, 7.0, 7.0], [3.0, 7.0, 1.0]]], [[[8.0, 1.0, 1.0], 
[4.0, 7.0, 4.0], [7.0, 1.0, 7.0]], [[7.0, 3.0, 6.0], [8.0, 1.0, 5.0], [3.0, 4.0, 4.0]], [[4.0, 2.0, 1.0], [3.0, 2.0, 1.0], 
[7.0, 3.0, 8.0]]], [[[1.0, 3.0, 6.0], [3.0, 7.0, 3.0], [5.0, 5.0, 6.0]], [[5.0, 2.0, 1.0], [3.0, 4.0, 1.0], [4.0, 5.0, 4.0]], 
[[7.0, 2.0, 8.0], [8.0, 6.0, 8.0], [3.0, 3.0, 2.0]]], [[[7.0, 6.0, 6.0], [3.0, 1.0, 5.0], [2.0, 3.0, 7.0]], [[2.0, 8.0, 5.0], 
[8.0, 8.0, 7.0], [5.0, 8.0, 2.0]], [[6.0, 3.0, 1.0], [2.0, 4.0, 1.0], [4.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PCRN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 5.0], [7.0, 1.0, 6.0], [7.0, 5.0, 2.0]], [[8.0, 1.0, 4.0], [6.0, 7.0, 2.0], [6.0, 5.0, 
4.0]], [[5.0, 5.0, 5.0], [6.0, 1.0, 3.0], [8.0, 3.0, 8.0]]], [[[3.0, 8.0, 5.0], [6.0, 2.0, 8.0], [5.0, 7.0, 5.0]], [[7.0, 5.0, 
4.0], [1.0, 4.0, 6.0], [5.0, 2.0, 7.0]], [[2.0, 7.0, 5.0], [1.0, 2.0, 1.0], [3.0, 7.0, 5.0]]], [[[7.0, 6.0, 5.0], [5.0, 3.0, 
8.0], [7.0, 1.0, 6.0]], [[4.0, 5.0, 4.0], [8.0, 6.0, 6.0], [2.0, 5.0, 4.0]], [[7.0, 6.0, 4.0], [4.0, 6.0, 8.0], [5.0, 1.0, 
2.0]]], [[[6.0, 3.0, 6.0], [7.0, 5.0, 3.0], [7.0, 1.0, 3.0]], [[8.0, 5.0, 3.0], [4.0, 8.0, 1.0], [4.0, 4.0, 1.0]], [[3.0, 3.0, 
4.0], [6.0, 7.0, 7.0], [6.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 2.0], [7.0, 1.0, 2.0], [8.0, 8.0, 6.0]], [[3.0, 3.0, 3.0], [2.0, 7.0, 
2.0], [8.0, 1.0, 2.0]], [[7.0, 3.0, 6.0], [1.0, 2.0, 8.0], [6.0, 7.0, 7.0]]], [[[8.0, 8.0, 7.0], [2.0, 4.0, 8.0], [1.0, 3.0, 
8.0]], [[5.0, 2.0, 2.0], [1.0, 5.0, 8.0], [8.0, 8.0, 2.0]], [[2.0, 2.0, 8.0], [3.0, 7.0, 8.0], [6.0, 8.0, 1.0]]], [[[1.0, 3.0, 
5.0], [3.0, 7.0, 3.0], [4.0, 4.0, 4.0]], [[1.0, 8.0, 6.0], [4.0, 2.0, 4.0], [1.0, 5.0, 6.0]], [[6.0, 2.0, 3.0], [8.0, 5.0, 
8.0], [1.0, 8.0, 2.0]]], [[[3.0, 1.0, 2.0], [7.0, 4.0, 1.0], [4.0, 1.0, 1.0]], [[4.0, 4.0, 4.0], [4.0, 3.0, 5.0], [6.0, 7.0, 
5.0]], [[5.0, 2.0, 5.0], [2.0, 6.0, 1.0], [3.0, 1.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[2.0, 2.0, 8.0]], [[5.0, 5.0, 8.0]], [[2.0, 6.0, 5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 6.0, 
4.0]], [[4.0, 8.0, 1.0]], [[1.0, 6.0, 1.0]]])
      arg[2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 5.0], [7.0, 1.0, 6.0], [7.0, 5.0, 2.0]], [[8.0, 1.0, 4.0], [6.0, 7.0, 2.0], [6.0, 
5.0, 4.0]], [[5.0, 5.0, 5.0], [6.0, 1.0, 3.0], [8.0, 3.0, 8.0]]], [[[3.0, 8.0, 5.0], [6.0, 2.0, 8.0], [5.0, 7.0, 5.0]], [[7.0, 
5.0, 4.0], [1.0, 4.0, 6.0], [5.0, 2.0, 7.0]], [[2.0, 7.0, 5.0], [1.0, 2.0, 1.0], [3.0, 7.0, 5.0]]], [[[7.0, 6.0, 5.0], [2.0, 
2.0, 8.0], [7.0, 1.0, 6.0]], [[4.0, 5.0, 4.0], [5.0, 5.0, 8.0], [2.0, 5.0, 4.0]], [[7.0, 6.0, 4.0], [2.0, 6.0, 5.0], [5.0, 1.0, 
2.0]]], [[[6.0, 3.0, 6.0], [7.0, 5.0, 3.0], [7.0, 1.0, 3.0]], [[8.0, 5.0, 3.0], [4.0, 8.0, 1.0], [4.0, 4.0, 1.0]], [[3.0, 3.0, 
4.0], [6.0, 7.0, 7.0], [6.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 2.0], [7.0, 1.0, 2.0], [8.0, 8.0, 6.0]], 
[[3.0, 3.0, 3.0], [2.0, 7.0, 2.0], [8.0, 1.0, 2.0]], [[7.0, 3.0, 6.0], [1.0, 2.0, 8.0], [6.0, 7.0, 7.0]]], [[[8.0, 8.0, 7.0], 
[2.0, 4.0, 8.0], [1.0, 3.0, 8.0]], [[5.0, 2.0, 2.0], [1.0, 5.0, 8.0], [8.0, 8.0, 2.0]], [[2.0, 2.0, 8.0], [3.0, 7.0, 8.0], 
[6.0, 8.0, 1.0]]], [[[1.0, 3.0, 5.0], [8.0, 6.0, 4.0], [4.0, 4.0, 4.0]], [[1.0, 8.0, 6.0], [4.0, 8.0, 1.0], [1.0, 5.0, 6.0]], 
[[6.0, 2.0, 3.0], [1.0, 6.0, 1.0], [1.0, 8.0, 2.0]]], [[[3.0, 1.0, 2.0], [7.0, 4.0, 1.0], [4.0, 1.0, 1.0]], [[4.0, 4.0, 4.0], 
[4.0, 3.0, 5.0], [6.0, 7.0, 5.0]], [[5.0, 2.0, 5.0], [2.0, 6.0, 1.0], [3.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRRR(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 8.0], [1.0, 5.0, 2.0], [5.0, 8.0, 3.0]], [[8.0, 7.0, 4.0], [8.0, 6.0, 2.0], [3.0, 7.0, 
2.0]], [[2.0, 4.0, 7.0], [4.0, 3.0, 7.0], [1.0, 1.0, 6.0]]], [[[8.0, 4.0, 2.0], [8.0, 4.0, 6.0], [2.0, 7.0, 1.0]], [[2.0, 2.0, 
6.0], [5.0, 6.0, 5.0], [8.0, 7.0, 3.0]], [[6.0, 6.0, 4.0], [1.0, 6.0, 8.0], [1.0, 2.0, 3.0]]], [[[8.0, 3.0, 6.0], [3.0, 8.0, 
8.0], [5.0, 3.0, 4.0]], [[7.0, 2.0, 6.0], [2.0, 4.0, 8.0], [8.0, 7.0, 8.0]], [[5.0, 1.0, 5.0], [2.0, 5.0, 3.0], [8.0, 5.0, 
4.0]]], [[[4.0, 2.0, 4.0], [7.0, 2.0, 2.0], [1.0, 4.0, 6.0]], [[5.0, 5.0, 2.0], [4.0, 8.0, 2.0], [7.0, 3.0, 1.0]], [[3.0, 6.0, 
8.0], [7.0, 2.0, 5.0], [2.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 4.0], [1.0, 8.0, 2.0], [5.0, 5.0, 1.0]], [[4.0, 7.0, 1.0], [1.0, 6.0, 
5.0], [3.0, 8.0, 3.0]], [[6.0, 6.0, 6.0], [2.0, 3.0, 6.0], [4.0, 5.0, 3.0]]], [[[5.0, 1.0, 6.0], [4.0, 1.0, 1.0], [7.0, 1.0, 
4.0]], [[2.0, 8.0, 7.0], [7.0, 4.0, 2.0], [6.0, 5.0, 3.0]], [[3.0, 5.0, 3.0], [3.0, 6.0, 7.0], [1.0, 5.0, 8.0]]], [[[4.0, 6.0, 
3.0], [5.0, 8.0, 3.0], [2.0, 3.0, 4.0]], [[2.0, 6.0, 2.0], [3.0, 2.0, 7.0], [8.0, 5.0, 1.0]], [[8.0, 5.0, 3.0], [1.0, 2.0, 
6.0], [5.0, 5.0, 5.0]]], [[[6.0, 5.0, 2.0], [1.0, 4.0, 2.0], [1.0, 5.0, 3.0]], [[8.0, 7.0, 1.0], [6.0, 8.0, 2.0], [7.0, 2.0, 
5.0]], [[7.0, 6.0, 3.0], [2.0, 8.0, 8.0], [3.0, 2.0, 6.0]]]]))
      val=numpy.array([[[[6.0]]], [[[5.0]]], [[[2.0]]], [[[3.0]]]])
      arg[:,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 8.0], [1.0, 5.0, 2.0], [5.0, 8.0, 3.0]], [[8.0, 7.0, 4.0], [8.0, 6.0, 2.0], [3.0, 
7.0, 2.0]], [[2.0, 4.0, 7.0], [4.0, 3.0, 7.0], [1.0, 1.0, 6.0]]], [[[8.0, 4.0, 2.0], [8.0, 4.0, 6.0], [2.0, 7.0, 1.0]], [[2.0, 
2.0, 6.0], [5.0, 5.0, 5.0], [8.0, 7.0, 3.0]], [[6.0, 6.0, 4.0], [1.0, 6.0, 8.0], [1.0, 2.0, 3.0]]], [[[8.0, 3.0, 6.0], [3.0, 
8.0, 8.0], [5.0, 3.0, 4.0]], [[7.0, 2.0, 6.0], [2.0, 2.0, 8.0], [8.0, 7.0, 8.0]], [[5.0, 1.0, 5.0], [2.0, 5.0, 3.0], [8.0, 5.0, 
4.0]]], [[[4.0, 2.0, 4.0], [7.0, 2.0, 2.0], [1.0, 4.0, 6.0]], [[5.0, 5.0, 2.0], [4.0, 3.0, 2.0], [7.0, 3.0, 1.0]], [[3.0, 6.0, 
8.0], [7.0, 2.0, 5.0], [2.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 4.0], [1.0, 8.0, 2.0], [5.0, 5.0, 1.0]], 
[[4.0, 7.0, 1.0], [1.0, 6.0, 5.0], [3.0, 8.0, 3.0]], [[6.0, 6.0, 6.0], [2.0, 3.0, 6.0], [4.0, 5.0, 3.0]]], [[[5.0, 1.0, 6.0], 
[4.0, 1.0, 1.0], [7.0, 1.0, 4.0]], [[2.0, 8.0, 7.0], [7.0, 5.0, 2.0], [6.0, 5.0, 3.0]], [[3.0, 5.0, 3.0], [3.0, 6.0, 7.0], 
[1.0, 5.0, 8.0]]], [[[4.0, 6.0, 3.0], [5.0, 8.0, 3.0], [2.0, 3.0, 4.0]], [[2.0, 6.0, 2.0], [3.0, 2.0, 7.0], [8.0, 5.0, 1.0]], 
[[8.0, 5.0, 3.0], [1.0, 2.0, 6.0], [5.0, 5.0, 5.0]]], [[[6.0, 5.0, 2.0], [1.0, 4.0, 2.0], [1.0, 5.0, 3.0]], [[8.0, 7.0, 1.0], 
[6.0, 3.0, 2.0], [7.0, 2.0, 5.0]], [[7.0, 6.0, 3.0], [2.0, 8.0, 8.0], [3.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRRR(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 5.0], [3.0, 3.0, 5.0], [1.0, 1.0, 3.0]], [[7.0, 8.0, 8.0], [6.0, 2.0, 7.0], [6.0, 2.0, 
4.0]], [[5.0, 6.0, 5.0], [4.0, 7.0, 1.0], [6.0, 4.0, 8.0]]], [[[8.0, 4.0, 1.0], [2.0, 5.0, 7.0], [2.0, 8.0, 2.0]], [[5.0, 4.0, 
7.0], [8.0, 7.0, 8.0], [2.0, 4.0, 8.0]], [[7.0, 1.0, 8.0], [5.0, 8.0, 8.0], [2.0, 5.0, 2.0]]], [[[3.0, 2.0, 8.0], [5.0, 8.0, 
8.0], [6.0, 6.0, 7.0]], [[1.0, 3.0, 8.0], [8.0, 3.0, 5.0], [7.0, 4.0, 1.0]], [[7.0, 1.0, 2.0], [4.0, 5.0, 2.0], [3.0, 1.0, 
4.0]]], [[[5.0, 5.0, 1.0], [2.0, 3.0, 4.0], [7.0, 5.0, 7.0]], [[6.0, 4.0, 8.0], [7.0, 8.0, 1.0], [1.0, 1.0, 4.0]], [[1.0, 5.0, 
7.0], [2.0, 4.0, 2.0], [8.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 3.0], [8.0, 1.0, 7.0], [2.0, 4.0, 7.0]], [[7.0, 8.0, 3.0], [4.0, 3.0, 
3.0], [8.0, 7.0, 6.0]], [[8.0, 3.0, 3.0], [1.0, 4.0, 8.0], [4.0, 4.0, 1.0]]], [[[6.0, 6.0, 1.0], [4.0, 5.0, 2.0], [4.0, 3.0, 
6.0]], [[7.0, 5.0, 8.0], [2.0, 4.0, 1.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 4.0], [1.0, 1.0, 3.0], [2.0, 1.0, 1.0]]], [[[2.0, 5.0, 
7.0], [8.0, 4.0, 6.0], [5.0, 6.0, 3.0]], [[7.0, 1.0, 5.0], [6.0, 2.0, 5.0], [6.0, 2.0, 5.0]], [[5.0, 7.0, 3.0], [7.0, 5.0, 
2.0], [8.0, 3.0, 3.0]]], [[[1.0, 3.0, 7.0], [5.0, 3.0, 6.0], [1.0, 5.0, 4.0]], [[6.0, 7.0, 4.0], [5.0, 5.0, 4.0], [5.0, 4.0, 
1.0]], [[7.0, 1.0, 4.0], [7.0, 5.0, 8.0], [3.0, 3.0, 2.0]]]]))
      val=Data(numpy.array([[[[4.0]]], [[[7.0]]], [[[6.0]]], [[[1.0]]]]),self.functionspace)
      arg[:,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 5.0], [3.0, 3.0, 5.0], [1.0, 1.0, 3.0]], [[7.0, 8.0, 8.0], [6.0, 4.0, 7.0], [6.0, 
2.0, 4.0]], [[5.0, 6.0, 5.0], [4.0, 7.0, 1.0], [6.0, 4.0, 8.0]]], [[[8.0, 4.0, 1.0], [2.0, 5.0, 7.0], [2.0, 8.0, 2.0]], [[5.0, 
4.0, 7.0], [8.0, 7.0, 8.0], [2.0, 4.0, 8.0]], [[7.0, 1.0, 8.0], [5.0, 8.0, 8.0], [2.0, 5.0, 2.0]]], [[[3.0, 2.0, 8.0], [5.0, 
8.0, 8.0], [6.0, 6.0, 7.0]], [[1.0, 3.0, 8.0], [8.0, 6.0, 5.0], [7.0, 4.0, 1.0]], [[7.0, 1.0, 2.0], [4.0, 5.0, 2.0], [3.0, 1.0, 
4.0]]], [[[5.0, 5.0, 1.0], [2.0, 3.0, 4.0], [7.0, 5.0, 7.0]], [[6.0, 4.0, 8.0], [7.0, 1.0, 1.0], [1.0, 1.0, 4.0]], [[1.0, 5.0, 
7.0], [2.0, 4.0, 2.0], [8.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 3.0], [8.0, 1.0, 7.0], [2.0, 4.0, 7.0]], 
[[7.0, 8.0, 3.0], [4.0, 4.0, 3.0], [8.0, 7.0, 6.0]], [[8.0, 3.0, 3.0], [1.0, 4.0, 8.0], [4.0, 4.0, 1.0]]], [[[6.0, 6.0, 1.0], 
[4.0, 5.0, 2.0], [4.0, 3.0, 6.0]], [[7.0, 5.0, 8.0], [2.0, 7.0, 1.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 4.0], [1.0, 1.0, 3.0], 
[2.0, 1.0, 1.0]]], [[[2.0, 5.0, 7.0], [8.0, 4.0, 6.0], [5.0, 6.0, 3.0]], [[7.0, 1.0, 5.0], [6.0, 6.0, 5.0], [6.0, 2.0, 5.0]], 
[[5.0, 7.0, 3.0], [7.0, 5.0, 2.0], [8.0, 3.0, 3.0]]], [[[1.0, 3.0, 7.0], [5.0, 3.0, 6.0], [1.0, 5.0, 4.0]], [[6.0, 7.0, 4.0], 
[5.0, 1.0, 4.0], [5.0, 4.0, 1.0]], [[7.0, 1.0, 4.0], [7.0, 5.0, 8.0], [3.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRRR(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 1.0], [6.0, 1.0, 3.0], [6.0, 1.0, 1.0]], [[3.0, 6.0, 7.0], [5.0, 5.0, 7.0], [7.0, 8.0, 
4.0]], [[1.0, 4.0, 7.0], [2.0, 7.0, 1.0], [2.0, 4.0, 3.0]]], [[[6.0, 4.0, 8.0], [1.0, 8.0, 5.0], [6.0, 5.0, 7.0]], [[5.0, 2.0, 
5.0], [4.0, 1.0, 5.0], [2.0, 3.0, 4.0]], [[1.0, 8.0, 3.0], [5.0, 3.0, 6.0], [8.0, 1.0, 4.0]]], [[[6.0, 4.0, 6.0], [4.0, 8.0, 
1.0], [1.0, 6.0, 6.0]], [[3.0, 5.0, 6.0], [5.0, 4.0, 2.0], [4.0, 7.0, 3.0]], [[8.0, 5.0, 8.0], [3.0, 8.0, 7.0], [6.0, 7.0, 
8.0]]], [[[6.0, 7.0, 8.0], [1.0, 6.0, 7.0], [6.0, 4.0, 5.0]], [[6.0, 4.0, 6.0], [7.0, 4.0, 2.0], [3.0, 2.0, 3.0]], [[2.0, 7.0, 
5.0], [2.0, 1.0, 8.0], [5.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 2.0], [7.0, 1.0, 6.0], [7.0, 1.0, 7.0]], [[7.0, 2.0, 6.0], [1.0, 2.0, 
6.0], [5.0, 6.0, 3.0]], [[7.0, 5.0, 3.0], [4.0, 3.0, 5.0], [6.0, 2.0, 7.0]]], [[[1.0, 5.0, 2.0], [4.0, 5.0, 2.0], [3.0, 3.0, 
2.0]], [[4.0, 6.0, 3.0], [4.0, 1.0, 5.0], [7.0, 2.0, 8.0]], [[4.0, 2.0, 1.0], [3.0, 3.0, 5.0], [7.0, 8.0, 7.0]]], [[[1.0, 3.0, 
2.0], [8.0, 5.0, 3.0], [8.0, 1.0, 4.0]], [[1.0, 7.0, 1.0], [4.0, 5.0, 8.0], [4.0, 1.0, 2.0]], [[6.0, 8.0, 8.0], [4.0, 2.0, 
2.0], [4.0, 2.0, 8.0]]], [[[6.0, 6.0, 2.0], [1.0, 8.0, 7.0], [5.0, 3.0, 7.0]], [[2.0, 8.0, 1.0], [5.0, 5.0, 8.0], [7.0, 7.0, 
3.0]], [[2.0, 5.0, 3.0], [3.0, 3.0, 6.0], [2.0, 1.0, 8.0]]]]))
      val=Data(numpy.array([[[[5.0]]], [[[4.0]]], [[[4.0]]], [[[4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0]]], [[[3.0]]], [[[3.0]]], [[[6.0]]]]))
      arg[:,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 1.0], [6.0, 1.0, 3.0], [6.0, 1.0, 1.0]], [[3.0, 6.0, 7.0], [5.0, 5.0, 7.0], [7.0, 
8.0, 4.0]], [[1.0, 4.0, 7.0], [2.0, 7.0, 1.0], [2.0, 4.0, 3.0]]], [[[6.0, 4.0, 8.0], [1.0, 8.0, 5.0], [6.0, 5.0, 7.0]], [[5.0, 
2.0, 5.0], [4.0, 4.0, 5.0], [2.0, 3.0, 4.0]], [[1.0, 8.0, 3.0], [5.0, 3.0, 6.0], [8.0, 1.0, 4.0]]], [[[6.0, 4.0, 6.0], [4.0, 
8.0, 1.0], [1.0, 6.0, 6.0]], [[3.0, 5.0, 6.0], [5.0, 4.0, 2.0], [4.0, 7.0, 3.0]], [[8.0, 5.0, 8.0], [3.0, 8.0, 7.0], [6.0, 7.0, 
8.0]]], [[[6.0, 7.0, 8.0], [1.0, 6.0, 7.0], [6.0, 4.0, 5.0]], [[6.0, 4.0, 6.0], [7.0, 4.0, 2.0], [3.0, 2.0, 3.0]], [[2.0, 7.0, 
5.0], [2.0, 1.0, 8.0], [5.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 2.0], [7.0, 1.0, 6.0], [7.0, 1.0, 7.0]], 
[[7.0, 2.0, 6.0], [1.0, 3.0, 6.0], [5.0, 6.0, 3.0]], [[7.0, 5.0, 3.0], [4.0, 3.0, 5.0], [6.0, 2.0, 7.0]]], [[[1.0, 5.0, 2.0], 
[4.0, 5.0, 2.0], [3.0, 3.0, 2.0]], [[4.0, 6.0, 3.0], [4.0, 3.0, 5.0], [7.0, 2.0, 8.0]], [[4.0, 2.0, 1.0], [3.0, 3.0, 5.0], 
[7.0, 8.0, 7.0]]], [[[1.0, 3.0, 2.0], [8.0, 5.0, 3.0], [8.0, 1.0, 4.0]], [[1.0, 7.0, 1.0], [4.0, 3.0, 8.0], [4.0, 1.0, 2.0]], 
[[6.0, 8.0, 8.0], [4.0, 2.0, 2.0], [4.0, 2.0, 8.0]]], [[[6.0, 6.0, 2.0], [1.0, 8.0, 7.0], [5.0, 3.0, 7.0]], [[2.0, 8.0, 1.0], 
[5.0, 6.0, 8.0], [7.0, 7.0, 3.0]], [[2.0, 5.0, 3.0], [3.0, 3.0, 6.0], [2.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRRR(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 2.0], [3.0, 3.0, 3.0], [1.0, 4.0, 6.0]], [[3.0, 1.0, 3.0], [5.0, 3.0, 5.0], [1.0, 5.0, 
8.0]], [[3.0, 6.0, 6.0], [8.0, 7.0, 2.0], [3.0, 4.0, 3.0]]], [[[5.0, 3.0, 3.0], [2.0, 1.0, 3.0], [8.0, 4.0, 5.0]], [[6.0, 5.0, 
8.0], [1.0, 1.0, 6.0], [3.0, 7.0, 3.0]], [[4.0, 4.0, 5.0], [2.0, 5.0, 7.0], [2.0, 3.0, 6.0]]], [[[2.0, 2.0, 1.0], [3.0, 4.0, 
4.0], [7.0, 3.0, 3.0]], [[1.0, 6.0, 6.0], [3.0, 4.0, 7.0], [5.0, 5.0, 6.0]], [[5.0, 8.0, 4.0], [1.0, 6.0, 6.0], [1.0, 3.0, 
7.0]]], [[[1.0, 2.0, 8.0], [8.0, 2.0, 1.0], [6.0, 6.0, 7.0]], [[4.0, 3.0, 6.0], [5.0, 3.0, 8.0], [6.0, 5.0, 5.0]], [[6.0, 4.0, 
4.0], [6.0, 2.0, 4.0], [1.0, 5.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 8.0], [2.0, 5.0, 6.0], [4.0, 5.0, 5.0]], [[3.0, 3.0, 6.0], [1.0, 2.0, 
5.0], [2.0, 3.0, 1.0]], [[4.0, 6.0, 6.0], [6.0, 3.0, 1.0], [8.0, 7.0, 5.0]]], [[[6.0, 2.0, 6.0], [2.0, 8.0, 2.0], [7.0, 3.0, 
4.0]], [[8.0, 6.0, 8.0], [5.0, 1.0, 4.0], [1.0, 6.0, 6.0]], [[4.0, 1.0, 3.0], [8.0, 3.0, 4.0], [1.0, 1.0, 8.0]]], [[[2.0, 5.0, 
5.0], [7.0, 1.0, 7.0], [8.0, 3.0, 3.0]], [[8.0, 5.0, 5.0], [4.0, 5.0, 1.0], [7.0, 6.0, 2.0]], [[3.0, 4.0, 7.0], [4.0, 1.0, 
3.0], [2.0, 8.0, 5.0]]], [[[5.0, 7.0, 5.0], [6.0, 5.0, 2.0], [2.0, 8.0, 2.0]], [[4.0, 4.0, 6.0], [3.0, 3.0, 8.0], [8.0, 2.0, 
2.0]], [[4.0, 2.0, 2.0], [1.0, 8.0, 4.0], [7.0, 7.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0]]], [[[3.0]]], [[[2.0]]], [[[7.0]]]])+(1.-msk_val)*numpy.array([[[[5.0]]], [[[6.0]]], 
[[[7.0]]], [[[3.0]]]])
      arg[:,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 2.0], [3.0, 3.0, 3.0], [1.0, 4.0, 6.0]], [[3.0, 1.0, 3.0], [5.0, 8.0, 5.0], [1.0, 
5.0, 8.0]], [[3.0, 6.0, 6.0], [8.0, 7.0, 2.0], [3.0, 4.0, 3.0]]], [[[5.0, 3.0, 3.0], [2.0, 1.0, 3.0], [8.0, 4.0, 5.0]], [[6.0, 
5.0, 8.0], [1.0, 3.0, 6.0], [3.0, 7.0, 3.0]], [[4.0, 4.0, 5.0], [2.0, 5.0, 7.0], [2.0, 3.0, 6.0]]], [[[2.0, 2.0, 1.0], [3.0, 
4.0, 4.0], [7.0, 3.0, 3.0]], [[1.0, 6.0, 6.0], [3.0, 2.0, 7.0], [5.0, 5.0, 6.0]], [[5.0, 8.0, 4.0], [1.0, 6.0, 6.0], [1.0, 3.0, 
7.0]]], [[[1.0, 2.0, 8.0], [8.0, 2.0, 1.0], [6.0, 6.0, 7.0]], [[4.0, 3.0, 6.0], [5.0, 7.0, 8.0], [6.0, 5.0, 5.0]], [[6.0, 4.0, 
4.0], [6.0, 2.0, 4.0], [1.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 8.0], [2.0, 5.0, 6.0], [4.0, 5.0, 5.0]], 
[[3.0, 3.0, 6.0], [1.0, 5.0, 5.0], [2.0, 3.0, 1.0]], [[4.0, 6.0, 6.0], [6.0, 3.0, 1.0], [8.0, 7.0, 5.0]]], [[[6.0, 2.0, 6.0], 
[2.0, 8.0, 2.0], [7.0, 3.0, 4.0]], [[8.0, 6.0, 8.0], [5.0, 6.0, 4.0], [1.0, 6.0, 6.0]], [[4.0, 1.0, 3.0], [8.0, 3.0, 4.0], 
[1.0, 1.0, 8.0]]], [[[2.0, 5.0, 5.0], [7.0, 1.0, 7.0], [8.0, 3.0, 3.0]], [[8.0, 5.0, 5.0], [4.0, 7.0, 1.0], [7.0, 6.0, 2.0]], 
[[3.0, 4.0, 7.0], [4.0, 1.0, 3.0], [2.0, 8.0, 5.0]]], [[[5.0, 7.0, 5.0], [6.0, 5.0, 2.0], [2.0, 8.0, 2.0]], [[4.0, 4.0, 6.0], 
[3.0, 3.0, 8.0], [8.0, 2.0, 2.0]], [[4.0, 2.0, 2.0], [1.0, 8.0, 4.0], [7.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UPCN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 6.0], [2.0, 6.0, 6.0], [3.0, 5.0, 4.0]], [[2.0, 3.0, 2.0], [2.0, 8.0, 8.0], [4.0, 2.0, 
8.0]], [[6.0, 1.0, 8.0], [1.0, 8.0, 1.0], [3.0, 2.0, 5.0]]], [[[3.0, 2.0, 5.0], [1.0, 1.0, 3.0], [2.0, 1.0, 1.0]], [[4.0, 7.0, 
5.0], [2.0, 1.0, 5.0], [7.0, 6.0, 2.0]], [[6.0, 1.0, 2.0], [7.0, 5.0, 2.0], [5.0, 7.0, 8.0]]], [[[1.0, 3.0, 5.0], [5.0, 5.0, 
3.0], [3.0, 1.0, 1.0]], [[7.0, 3.0, 5.0], [1.0, 6.0, 2.0], [1.0, 3.0, 7.0]], [[8.0, 8.0, 8.0], [5.0, 5.0, 5.0], [3.0, 4.0, 
1.0]]], [[[6.0, 5.0, 3.0], [2.0, 5.0, 3.0], [3.0, 1.0, 5.0]], [[8.0, 4.0, 5.0], [6.0, 2.0, 7.0], [2.0, 7.0, 5.0]], [[8.0, 8.0, 
6.0], [3.0, 8.0, 5.0], [8.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 3.0], [5.0, 1.0, 4.0], [7.0, 3.0, 1.0]], [[5.0, 6.0, 4.0], [1.0, 7.0, 
6.0], [2.0, 5.0, 6.0]], [[7.0, 4.0, 8.0], [8.0, 8.0, 8.0], [1.0, 3.0, 7.0]]], [[[5.0, 2.0, 7.0], [2.0, 1.0, 3.0], [4.0, 5.0, 
3.0]], [[5.0, 8.0, 7.0], [2.0, 1.0, 5.0], [2.0, 6.0, 8.0]], [[4.0, 8.0, 8.0], [5.0, 3.0, 4.0], [6.0, 5.0, 8.0]]], [[[1.0, 2.0, 
6.0], [8.0, 2.0, 7.0], [3.0, 6.0, 3.0]], [[1.0, 6.0, 5.0], [6.0, 2.0, 8.0], [1.0, 5.0, 2.0]], [[7.0, 2.0, 3.0], [1.0, 6.0, 
7.0], [7.0, 2.0, 8.0]]], [[[5.0, 4.0, 4.0], [5.0, 1.0, 4.0], [3.0, 1.0, 3.0]], [[4.0, 4.0, 3.0], [1.0, 6.0, 7.0], [4.0, 4.0, 
1.0]], [[4.0, 8.0, 4.0], [2.0, 6.0, 7.0], [5.0, 4.0, 5.0]]]]))
      val=numpy.array([[[6.0, 7.0, 5.0], [4.0, 2.0, 2.0], [5.0, 7.0, 5.0]], [[6.0, 2.0, 1.0], [5.0, 8.0, 7.0], [8.0, 5.0, 
8.0]], [[6.0, 4.0, 1.0], [3.0, 7.0, 4.0], [2.0, 5.0, 2.0]]])
      arg[:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 6.0], [2.0, 6.0, 6.0], [3.0, 5.0, 4.0]], [[6.0, 7.0, 5.0], [4.0, 2.0, 2.0], [5.0, 
7.0, 5.0]], [[6.0, 1.0, 8.0], [1.0, 8.0, 1.0], [3.0, 2.0, 5.0]]], [[[3.0, 2.0, 5.0], [1.0, 1.0, 3.0], [2.0, 1.0, 1.0]], [[6.0, 
2.0, 1.0], [5.0, 8.0, 7.0], [8.0, 5.0, 8.0]], [[6.0, 1.0, 2.0], [7.0, 5.0, 2.0], [5.0, 7.0, 8.0]]], [[[1.0, 3.0, 5.0], [5.0, 
5.0, 3.0], [3.0, 1.0, 1.0]], [[6.0, 4.0, 1.0], [3.0, 7.0, 4.0], [2.0, 5.0, 2.0]], [[8.0, 8.0, 8.0], [5.0, 5.0, 5.0], [3.0, 4.0, 
1.0]]], [[[6.0, 5.0, 3.0], [2.0, 5.0, 3.0], [3.0, 1.0, 5.0]], [[8.0, 4.0, 5.0], [6.0, 2.0, 7.0], [2.0, 7.0, 5.0]], [[8.0, 8.0, 
6.0], [3.0, 8.0, 5.0], [8.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 3.0], [5.0, 1.0, 4.0], [7.0, 3.0, 1.0]], 
[[6.0, 7.0, 5.0], [4.0, 2.0, 2.0], [5.0, 7.0, 5.0]], [[7.0, 4.0, 8.0], [8.0, 8.0, 8.0], [1.0, 3.0, 7.0]]], [[[5.0, 2.0, 7.0], 
[2.0, 1.0, 3.0], [4.0, 5.0, 3.0]], [[6.0, 2.0, 1.0], [5.0, 8.0, 7.0], [8.0, 5.0, 8.0]], [[4.0, 8.0, 8.0], [5.0, 3.0, 4.0], 
[6.0, 5.0, 8.0]]], [[[1.0, 2.0, 6.0], [8.0, 2.0, 7.0], [3.0, 6.0, 3.0]], [[6.0, 4.0, 1.0], [3.0, 7.0, 4.0], [2.0, 5.0, 2.0]], 
[[7.0, 2.0, 3.0], [1.0, 6.0, 7.0], [7.0, 2.0, 8.0]]], [[[5.0, 4.0, 4.0], [5.0, 1.0, 4.0], [3.0, 1.0, 3.0]], [[4.0, 4.0, 3.0], 
[1.0, 6.0, 7.0], [4.0, 4.0, 1.0]], [[4.0, 8.0, 4.0], [2.0, 6.0, 7.0], [5.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UPCN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 5.0], [5.0, 1.0, 6.0], [8.0, 8.0, 5.0]], [[1.0, 6.0, 2.0], [8.0, 8.0, 6.0], [2.0, 2.0, 
3.0]], [[2.0, 4.0, 7.0], [3.0, 3.0, 6.0], [1.0, 1.0, 7.0]]], [[[6.0, 2.0, 6.0], [2.0, 3.0, 8.0], [2.0, 7.0, 3.0]], [[1.0, 7.0, 
3.0], [2.0, 4.0, 6.0], [5.0, 1.0, 3.0]], [[4.0, 4.0, 1.0], [5.0, 4.0, 4.0], [2.0, 5.0, 6.0]]], [[[7.0, 8.0, 6.0], [1.0, 4.0, 
8.0], [7.0, 1.0, 4.0]], [[4.0, 4.0, 4.0], [8.0, 6.0, 1.0], [5.0, 5.0, 4.0]], [[3.0, 2.0, 6.0], [5.0, 7.0, 2.0], [8.0, 6.0, 
7.0]]], [[[8.0, 2.0, 8.0], [5.0, 4.0, 2.0], [7.0, 2.0, 5.0]], [[6.0, 2.0, 3.0], [5.0, 6.0, 8.0], [2.0, 4.0, 8.0]], [[1.0, 4.0, 
2.0], [8.0, 1.0, 4.0], [6.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 3.0], [5.0, 8.0, 2.0], [8.0, 7.0, 7.0]], [[3.0, 6.0, 6.0], [8.0, 7.0, 
5.0], [5.0, 4.0, 3.0]], [[3.0, 5.0, 6.0], [4.0, 7.0, 6.0], [1.0, 6.0, 2.0]]], [[[8.0, 5.0, 3.0], [8.0, 4.0, 3.0], [6.0, 1.0, 
6.0]], [[3.0, 7.0, 2.0], [5.0, 7.0, 1.0], [4.0, 7.0, 4.0]], [[5.0, 7.0, 3.0], [3.0, 6.0, 5.0], [7.0, 3.0, 8.0]]], [[[3.0, 3.0, 
5.0], [3.0, 8.0, 5.0], [3.0, 8.0, 3.0]], [[6.0, 1.0, 7.0], [7.0, 2.0, 3.0], [8.0, 3.0, 8.0]], [[8.0, 1.0, 5.0], [4.0, 3.0, 
1.0], [4.0, 7.0, 7.0]]], [[[5.0, 3.0, 4.0], [4.0, 4.0, 3.0], [4.0, 5.0, 1.0]], [[5.0, 1.0, 4.0], [1.0, 3.0, 3.0], [4.0, 6.0, 
3.0]], [[5.0, 2.0, 8.0], [2.0, 1.0, 3.0], [2.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([[[7.0, 2.0, 1.0], [6.0, 8.0, 8.0], [7.0, 1.0, 3.0]], [[1.0, 7.0, 6.0], [7.0, 5.0, 7.0], [6.0, 1.0, 
6.0]], [[2.0, 1.0, 8.0], [1.0, 5.0, 6.0], [3.0, 4.0, 1.0]]]),self.functionspace)
      arg[:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 5.0], [5.0, 1.0, 6.0], [8.0, 8.0, 5.0]], [[7.0, 2.0, 1.0], [6.0, 8.0, 8.0], [7.0, 
1.0, 3.0]], [[2.0, 4.0, 7.0], [3.0, 3.0, 6.0], [1.0, 1.0, 7.0]]], [[[6.0, 2.0, 6.0], [2.0, 3.0, 8.0], [2.0, 7.0, 3.0]], [[1.0, 
7.0, 6.0], [7.0, 5.0, 7.0], [6.0, 1.0, 6.0]], [[4.0, 4.0, 1.0], [5.0, 4.0, 4.0], [2.0, 5.0, 6.0]]], [[[7.0, 8.0, 6.0], [1.0, 
4.0, 8.0], [7.0, 1.0, 4.0]], [[2.0, 1.0, 8.0], [1.0, 5.0, 6.0], [3.0, 4.0, 1.0]], [[3.0, 2.0, 6.0], [5.0, 7.0, 2.0], [8.0, 6.0, 
7.0]]], [[[8.0, 2.0, 8.0], [5.0, 4.0, 2.0], [7.0, 2.0, 5.0]], [[6.0, 2.0, 3.0], [5.0, 6.0, 8.0], [2.0, 4.0, 8.0]], [[1.0, 4.0, 
2.0], [8.0, 1.0, 4.0], [6.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 3.0], [5.0, 8.0, 2.0], [8.0, 7.0, 7.0]], 
[[7.0, 2.0, 1.0], [6.0, 8.0, 8.0], [7.0, 1.0, 3.0]], [[3.0, 5.0, 6.0], [4.0, 7.0, 6.0], [1.0, 6.0, 2.0]]], [[[8.0, 5.0, 3.0], 
[8.0, 4.0, 3.0], [6.0, 1.0, 6.0]], [[1.0, 7.0, 6.0], [7.0, 5.0, 7.0], [6.0, 1.0, 6.0]], [[5.0, 7.0, 3.0], [3.0, 6.0, 5.0], 
[7.0, 3.0, 8.0]]], [[[3.0, 3.0, 5.0], [3.0, 8.0, 5.0], [3.0, 8.0, 3.0]], [[2.0, 1.0, 8.0], [1.0, 5.0, 6.0], [3.0, 4.0, 1.0]], 
[[8.0, 1.0, 5.0], [4.0, 3.0, 1.0], [4.0, 7.0, 7.0]]], [[[5.0, 3.0, 4.0], [4.0, 4.0, 3.0], [4.0, 5.0, 1.0]], [[5.0, 1.0, 4.0], 
[1.0, 3.0, 3.0], [4.0, 6.0, 3.0]], [[5.0, 2.0, 8.0], [2.0, 1.0, 3.0], [2.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UPCN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 3.0], [6.0, 5.0, 6.0], [2.0, 3.0, 5.0]], [[8.0, 8.0, 7.0], [1.0, 1.0, 6.0], [2.0, 8.0, 
4.0]], [[2.0, 4.0, 8.0], [8.0, 4.0, 8.0], [2.0, 5.0, 8.0]]], [[[1.0, 4.0, 4.0], [1.0, 7.0, 8.0], [8.0, 6.0, 1.0]], [[1.0, 4.0, 
1.0], [4.0, 6.0, 3.0], [8.0, 4.0, 4.0]], [[7.0, 2.0, 4.0], [4.0, 6.0, 4.0], [7.0, 7.0, 4.0]]], [[[2.0, 7.0, 1.0], [1.0, 4.0, 
2.0], [4.0, 8.0, 4.0]], [[4.0, 3.0, 5.0], [6.0, 6.0, 4.0], [8.0, 1.0, 4.0]], [[2.0, 5.0, 7.0], [2.0, 1.0, 1.0], [2.0, 6.0, 
8.0]]], [[[6.0, 5.0, 2.0], [5.0, 4.0, 2.0], [5.0, 6.0, 5.0]], [[7.0, 1.0, 5.0], [7.0, 3.0, 1.0], [5.0, 4.0, 3.0]], [[2.0, 7.0, 
6.0], [1.0, 8.0, 5.0], [4.0, 1.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 1.0], [6.0, 5.0, 4.0], [1.0, 1.0, 7.0]], [[2.0, 5.0, 5.0], [5.0, 5.0, 
4.0], [2.0, 6.0, 6.0]], [[7.0, 1.0, 8.0], [3.0, 6.0, 6.0], [7.0, 7.0, 7.0]]], [[[5.0, 2.0, 5.0], [4.0, 5.0, 5.0], [1.0, 2.0, 
6.0]], [[8.0, 7.0, 5.0], [5.0, 2.0, 5.0], [2.0, 8.0, 8.0]], [[8.0, 1.0, 7.0], [6.0, 2.0, 2.0], [3.0, 4.0, 2.0]]], [[[7.0, 2.0, 
8.0], [3.0, 7.0, 8.0], [2.0, 1.0, 3.0]], [[5.0, 8.0, 2.0], [1.0, 3.0, 7.0], [8.0, 5.0, 5.0]], [[5.0, 2.0, 3.0], [4.0, 4.0, 
4.0], [8.0, 5.0, 5.0]]], [[[1.0, 6.0, 4.0], [6.0, 3.0, 4.0], [8.0, 4.0, 3.0]], [[5.0, 4.0, 4.0], [8.0, 6.0, 2.0], [7.0, 5.0, 
3.0]], [[1.0, 6.0, 5.0], [6.0, 5.0, 5.0], [3.0, 2.0, 4.0]]]]))
      val=Data(numpy.array([[[3.0, 8.0, 8.0], [2.0, 2.0, 8.0], [8.0, 1.0, 6.0]], [[4.0, 6.0, 3.0], [4.0, 4.0, 4.0], [8.0, 1.0, 
6.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 2.0], [4.0, 6.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 4.0, 4.0], [1.0, 1.0, 2.0], [1.0, 3.0, 3.0]], [[3.0, 7.0, 3.0], [7.0, 5.0, 7.0], 
[7.0, 8.0, 5.0]], [[5.0, 4.0, 7.0], [2.0, 5.0, 3.0], [3.0, 1.0, 7.0]]]))
      arg[:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 3.0], [6.0, 5.0, 6.0], [2.0, 3.0, 5.0]], [[3.0, 8.0, 8.0], [2.0, 2.0, 8.0], [8.0, 
1.0, 6.0]], [[2.0, 4.0, 8.0], [8.0, 4.0, 8.0], [2.0, 5.0, 8.0]]], [[[1.0, 4.0, 4.0], [1.0, 7.0, 8.0], [8.0, 6.0, 1.0]], [[4.0, 
6.0, 3.0], [4.0, 4.0, 4.0], [8.0, 1.0, 6.0]], [[7.0, 2.0, 4.0], [4.0, 6.0, 4.0], [7.0, 7.0, 4.0]]], [[[2.0, 7.0, 1.0], [1.0, 
4.0, 2.0], [4.0, 8.0, 4.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 2.0], [4.0, 6.0, 5.0]], [[2.0, 5.0, 7.0], [2.0, 1.0, 1.0], [2.0, 6.0, 
8.0]]], [[[6.0, 5.0, 2.0], [5.0, 4.0, 2.0], [5.0, 6.0, 5.0]], [[7.0, 1.0, 5.0], [7.0, 3.0, 1.0], [5.0, 4.0, 3.0]], [[2.0, 7.0, 
6.0], [1.0, 8.0, 5.0], [4.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 1.0], [6.0, 5.0, 4.0], [1.0, 1.0, 7.0]], 
[[4.0, 4.0, 4.0], [1.0, 1.0, 2.0], [1.0, 3.0, 3.0]], [[7.0, 1.0, 8.0], [3.0, 6.0, 6.0], [7.0, 7.0, 7.0]]], [[[5.0, 2.0, 5.0], 
[4.0, 5.0, 5.0], [1.0, 2.0, 6.0]], [[3.0, 7.0, 3.0], [7.0, 5.0, 7.0], [7.0, 8.0, 5.0]], [[8.0, 1.0, 7.0], [6.0, 2.0, 2.0], 
[3.0, 4.0, 2.0]]], [[[7.0, 2.0, 8.0], [3.0, 7.0, 8.0], [2.0, 1.0, 3.0]], [[5.0, 4.0, 7.0], [2.0, 5.0, 3.0], [3.0, 1.0, 7.0]], 
[[5.0, 2.0, 3.0], [4.0, 4.0, 4.0], [8.0, 5.0, 5.0]]], [[[1.0, 6.0, 4.0], [6.0, 3.0, 4.0], [8.0, 4.0, 3.0]], [[5.0, 4.0, 4.0], 
[8.0, 6.0, 2.0], [7.0, 5.0, 3.0]], [[1.0, 6.0, 5.0], [6.0, 5.0, 5.0], [3.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UPCN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 3.0], [5.0, 8.0, 6.0], [5.0, 2.0, 6.0]], [[3.0, 8.0, 8.0], [7.0, 2.0, 7.0], [4.0, 4.0, 
3.0]], [[3.0, 2.0, 4.0], [8.0, 6.0, 5.0], [3.0, 3.0, 6.0]]], [[[7.0, 8.0, 5.0], [5.0, 5.0, 5.0], [4.0, 7.0, 4.0]], [[7.0, 1.0, 
6.0], [4.0, 2.0, 5.0], [8.0, 2.0, 5.0]], [[7.0, 1.0, 1.0], [2.0, 3.0, 6.0], [2.0, 1.0, 7.0]]], [[[6.0, 8.0, 6.0], [2.0, 3.0, 
1.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 5.0], [3.0, 3.0, 3.0], [3.0, 5.0, 8.0]], [[8.0, 3.0, 5.0], [5.0, 4.0, 8.0], [6.0, 1.0, 
2.0]]], [[[4.0, 3.0, 4.0], [7.0, 5.0, 4.0], [4.0, 8.0, 3.0]], [[4.0, 5.0, 8.0], [7.0, 2.0, 4.0], [8.0, 4.0, 5.0]], [[4.0, 6.0, 
3.0], [6.0, 8.0, 8.0], [8.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 5.0], [3.0, 8.0, 8.0], [8.0, 7.0, 2.0]], [[4.0, 2.0, 1.0], [1.0, 6.0, 
1.0], [8.0, 8.0, 7.0]], [[4.0, 5.0, 3.0], [6.0, 8.0, 6.0], [7.0, 2.0, 3.0]]], [[[6.0, 6.0, 5.0], [4.0, 7.0, 1.0], [8.0, 6.0, 
7.0]], [[3.0, 3.0, 4.0], [2.0, 5.0, 6.0], [2.0, 6.0, 6.0]], [[4.0, 4.0, 4.0], [5.0, 6.0, 4.0], [5.0, 3.0, 4.0]]], [[[4.0, 4.0, 
4.0], [5.0, 4.0, 5.0], [3.0, 7.0, 3.0]], [[4.0, 6.0, 6.0], [3.0, 8.0, 8.0], [1.0, 3.0, 7.0]], [[2.0, 3.0, 8.0], [6.0, 1.0, 
2.0], [1.0, 5.0, 1.0]]], [[[6.0, 6.0, 6.0], [2.0, 6.0, 7.0], [6.0, 1.0, 6.0]], [[3.0, 1.0, 2.0], [5.0, 7.0, 1.0], [1.0, 1.0, 
7.0]], [[7.0, 5.0, 6.0], [8.0, 5.0, 4.0], [1.0, 4.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 6.0, 7.0], [7.0, 5.0, 2.0], [5.0, 5.0, 1.0]], [[7.0, 7.0, 4.0], [3.0, 4.0, 4.0], [5.0, 
1.0, 7.0]], [[1.0, 7.0, 6.0], [5.0, 8.0, 3.0], [3.0, 3.0, 6.0]]])+(1.-msk_val)*numpy.array([[[3.0, 8.0, 5.0], [2.0, 1.0, 3.0], 
[3.0, 4.0, 5.0]], [[7.0, 5.0, 4.0], [8.0, 1.0, 6.0], [1.0, 4.0, 6.0]], [[5.0, 1.0, 8.0], [4.0, 5.0, 7.0], [7.0, 4.0, 2.0]]])
      arg[:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 3.0], [5.0, 8.0, 6.0], [5.0, 2.0, 6.0]], [[5.0, 6.0, 7.0], [7.0, 5.0, 2.0], [5.0, 
5.0, 1.0]], [[3.0, 2.0, 4.0], [8.0, 6.0, 5.0], [3.0, 3.0, 6.0]]], [[[7.0, 8.0, 5.0], [5.0, 5.0, 5.0], [4.0, 7.0, 4.0]], [[7.0, 
7.0, 4.0], [3.0, 4.0, 4.0], [5.0, 1.0, 7.0]], [[7.0, 1.0, 1.0], [2.0, 3.0, 6.0], [2.0, 1.0, 7.0]]], [[[6.0, 8.0, 6.0], [2.0, 
3.0, 1.0], [8.0, 4.0, 1.0]], [[1.0, 7.0, 6.0], [5.0, 8.0, 3.0], [3.0, 3.0, 6.0]], [[8.0, 3.0, 5.0], [5.0, 4.0, 8.0], [6.0, 1.0, 
2.0]]], [[[4.0, 3.0, 4.0], [7.0, 5.0, 4.0], [4.0, 8.0, 3.0]], [[4.0, 5.0, 8.0], [7.0, 2.0, 4.0], [8.0, 4.0, 5.0]], [[4.0, 6.0, 
3.0], [6.0, 8.0, 8.0], [8.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 5.0], [3.0, 8.0, 8.0], [8.0, 7.0, 2.0]], 
[[3.0, 8.0, 5.0], [2.0, 1.0, 3.0], [3.0, 4.0, 5.0]], [[4.0, 5.0, 3.0], [6.0, 8.0, 6.0], [7.0, 2.0, 3.0]]], [[[6.0, 6.0, 5.0], 
[4.0, 7.0, 1.0], [8.0, 6.0, 7.0]], [[7.0, 5.0, 4.0], [8.0, 1.0, 6.0], [1.0, 4.0, 6.0]], [[4.0, 4.0, 4.0], [5.0, 6.0, 4.0], 
[5.0, 3.0, 4.0]]], [[[4.0, 4.0, 4.0], [5.0, 4.0, 5.0], [3.0, 7.0, 3.0]], [[5.0, 1.0, 8.0], [4.0, 5.0, 7.0], [7.0, 4.0, 2.0]], 
[[2.0, 3.0, 8.0], [6.0, 1.0, 2.0], [1.0, 5.0, 1.0]]], [[[6.0, 6.0, 6.0], [2.0, 6.0, 7.0], [6.0, 1.0, 6.0]], [[3.0, 1.0, 2.0], 
[5.0, 7.0, 1.0], [1.0, 1.0, 7.0]], [[7.0, 5.0, 6.0], [8.0, 5.0, 4.0], [1.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PRLN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 8.0], [3.0, 3.0, 8.0], [1.0, 7.0, 3.0]], [[8.0, 1.0, 7.0], [1.0, 8.0, 1.0], [6.0, 6.0, 
2.0]], [[7.0, 6.0, 4.0], [5.0, 7.0, 1.0], [1.0, 7.0, 4.0]]], [[[8.0, 3.0, 1.0], [1.0, 4.0, 7.0], [2.0, 7.0, 5.0]], [[5.0, 5.0, 
4.0], [3.0, 3.0, 3.0], [5.0, 2.0, 3.0]], [[7.0, 3.0, 8.0], [2.0, 7.0, 7.0], [1.0, 8.0, 6.0]]], [[[5.0, 4.0, 8.0], [3.0, 8.0, 
5.0], [8.0, 1.0, 8.0]], [[7.0, 5.0, 3.0], [6.0, 6.0, 6.0], [8.0, 1.0, 4.0]], [[6.0, 4.0, 7.0], [3.0, 1.0, 2.0], [4.0, 5.0, 
2.0]]], [[[7.0, 6.0, 3.0], [4.0, 7.0, 8.0], [8.0, 6.0, 5.0]], [[8.0, 2.0, 2.0], [3.0, 3.0, 2.0], [8.0, 1.0, 4.0]], [[4.0, 1.0, 
6.0], [6.0, 8.0, 7.0], [3.0, 1.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 8.0], [5.0, 6.0, 4.0], [4.0, 1.0, 7.0]], [[4.0, 1.0, 2.0], [6.0, 7.0, 
4.0], [4.0, 6.0, 3.0]], [[8.0, 8.0, 4.0], [2.0, 2.0, 7.0], [4.0, 3.0, 4.0]]], [[[6.0, 6.0, 6.0], [8.0, 6.0, 4.0], [4.0, 3.0, 
7.0]], [[3.0, 2.0, 7.0], [7.0, 2.0, 2.0], [1.0, 1.0, 4.0]], [[7.0, 3.0, 4.0], [8.0, 6.0, 2.0], [7.0, 8.0, 6.0]]], [[[7.0, 8.0, 
7.0], [1.0, 3.0, 8.0], [2.0, 2.0, 8.0]], [[3.0, 8.0, 1.0], [5.0, 1.0, 2.0], [4.0, 3.0, 7.0]], [[7.0, 8.0, 8.0], [6.0, 3.0, 
8.0], [5.0, 8.0, 3.0]]], [[[6.0, 8.0, 4.0], [7.0, 4.0, 3.0], [3.0, 1.0, 6.0]], [[8.0, 1.0, 6.0], [1.0, 7.0, 7.0], [3.0, 5.0, 
4.0]], [[2.0, 5.0, 4.0], [5.0, 1.0, 4.0], [1.0, 5.0, 4.0]]]]))
      val=numpy.array([[[6.0, 1.0, 2.0]]])
      arg[2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 8.0], [3.0, 3.0, 8.0], [1.0, 7.0, 3.0]], [[8.0, 1.0, 7.0], [1.0, 8.0, 1.0], [6.0, 
6.0, 2.0]], [[7.0, 6.0, 4.0], [5.0, 7.0, 1.0], [1.0, 7.0, 4.0]]], [[[8.0, 3.0, 1.0], [1.0, 4.0, 7.0], [2.0, 7.0, 5.0]], [[5.0, 
5.0, 4.0], [3.0, 3.0, 3.0], [5.0, 2.0, 3.0]], [[7.0, 3.0, 8.0], [2.0, 7.0, 7.0], [1.0, 8.0, 6.0]]], [[[5.0, 4.0, 8.0], [3.0, 
8.0, 5.0], [8.0, 1.0, 8.0]], [[7.0, 5.0, 3.0], [6.0, 6.0, 6.0], [6.0, 1.0, 2.0]], [[6.0, 4.0, 7.0], [3.0, 1.0, 2.0], [4.0, 5.0, 
2.0]]], [[[7.0, 6.0, 3.0], [4.0, 7.0, 8.0], [8.0, 6.0, 5.0]], [[8.0, 2.0, 2.0], [3.0, 3.0, 2.0], [8.0, 1.0, 4.0]], [[4.0, 1.0, 
6.0], [6.0, 8.0, 7.0], [3.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 8.0], [5.0, 6.0, 4.0], [4.0, 1.0, 7.0]], 
[[4.0, 1.0, 2.0], [6.0, 7.0, 4.0], [4.0, 6.0, 3.0]], [[8.0, 8.0, 4.0], [2.0, 2.0, 7.0], [4.0, 3.0, 4.0]]], [[[6.0, 6.0, 6.0], 
[8.0, 6.0, 4.0], [4.0, 3.0, 7.0]], [[3.0, 2.0, 7.0], [7.0, 2.0, 2.0], [1.0, 1.0, 4.0]], [[7.0, 3.0, 4.0], [8.0, 6.0, 2.0], 
[7.0, 8.0, 6.0]]], [[[7.0, 8.0, 7.0], [1.0, 3.0, 8.0], [2.0, 2.0, 8.0]], [[3.0, 8.0, 1.0], [5.0, 1.0, 2.0], [6.0, 1.0, 2.0]], 
[[7.0, 8.0, 8.0], [6.0, 3.0, 8.0], [5.0, 8.0, 3.0]]], [[[6.0, 8.0, 4.0], [7.0, 4.0, 3.0], [3.0, 1.0, 6.0]], [[8.0, 1.0, 6.0], 
[1.0, 7.0, 7.0], [3.0, 5.0, 4.0]], [[2.0, 5.0, 4.0], [5.0, 1.0, 4.0], [1.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PRLN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 4.0], [4.0, 4.0, 4.0], [4.0, 7.0, 8.0]], [[7.0, 6.0, 6.0], [7.0, 5.0, 4.0], [4.0, 2.0, 
2.0]], [[7.0, 8.0, 6.0], [7.0, 3.0, 3.0], [4.0, 4.0, 3.0]]], [[[5.0, 3.0, 1.0], [5.0, 4.0, 6.0], [8.0, 6.0, 3.0]], [[6.0, 2.0, 
6.0], [2.0, 1.0, 6.0], [2.0, 2.0, 3.0]], [[6.0, 3.0, 3.0], [2.0, 6.0, 3.0], [3.0, 2.0, 1.0]]], [[[5.0, 2.0, 2.0], [4.0, 2.0, 
5.0], [8.0, 2.0, 3.0]], [[6.0, 8.0, 6.0], [7.0, 8.0, 1.0], [3.0, 5.0, 1.0]], [[4.0, 7.0, 4.0], [2.0, 8.0, 1.0], [3.0, 3.0, 
6.0]]], [[[5.0, 7.0, 1.0], [1.0, 2.0, 7.0], [3.0, 1.0, 6.0]], [[1.0, 5.0, 5.0], [7.0, 6.0, 7.0], [1.0, 6.0, 6.0]], [[4.0, 7.0, 
6.0], [3.0, 7.0, 7.0], [2.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 4.0], [2.0, 1.0, 7.0], [4.0, 1.0, 8.0]], [[3.0, 2.0, 5.0], [2.0, 8.0, 
5.0], [5.0, 5.0, 7.0]], [[8.0, 4.0, 3.0], [3.0, 3.0, 8.0], [8.0, 8.0, 7.0]]], [[[8.0, 6.0, 4.0], [6.0, 1.0, 1.0], [5.0, 2.0, 
5.0]], [[8.0, 2.0, 4.0], [2.0, 1.0, 4.0], [8.0, 7.0, 7.0]], [[4.0, 5.0, 6.0], [5.0, 4.0, 6.0], [4.0, 3.0, 5.0]]], [[[6.0, 6.0, 
5.0], [3.0, 1.0, 7.0], [2.0, 5.0, 7.0]], [[7.0, 2.0, 6.0], [2.0, 5.0, 3.0], [3.0, 7.0, 6.0]], [[6.0, 4.0, 8.0], [2.0, 1.0, 
6.0], [1.0, 6.0, 1.0]]], [[[1.0, 2.0, 2.0], [1.0, 7.0, 5.0], [7.0, 5.0, 1.0]], [[5.0, 3.0, 1.0], [8.0, 2.0, 7.0], [8.0, 4.0, 
8.0]], [[2.0, 8.0, 8.0], [7.0, 6.0, 3.0], [8.0, 6.0, 3.0]]]]))
      val=Data(numpy.array([[[5.0, 6.0, 6.0]]]),self.functionspace)
      arg[2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 4.0], [4.0, 4.0, 4.0], [4.0, 7.0, 8.0]], [[7.0, 6.0, 6.0], [7.0, 5.0, 4.0], [4.0, 
2.0, 2.0]], [[7.0, 8.0, 6.0], [7.0, 3.0, 3.0], [4.0, 4.0, 3.0]]], [[[5.0, 3.0, 1.0], [5.0, 4.0, 6.0], [8.0, 6.0, 3.0]], [[6.0, 
2.0, 6.0], [2.0, 1.0, 6.0], [2.0, 2.0, 3.0]], [[6.0, 3.0, 3.0], [2.0, 6.0, 3.0], [3.0, 2.0, 1.0]]], [[[5.0, 2.0, 2.0], [4.0, 
2.0, 5.0], [8.0, 2.0, 3.0]], [[6.0, 8.0, 6.0], [7.0, 8.0, 1.0], [5.0, 6.0, 6.0]], [[4.0, 7.0, 4.0], [2.0, 8.0, 1.0], [3.0, 3.0, 
6.0]]], [[[5.0, 7.0, 1.0], [1.0, 2.0, 7.0], [3.0, 1.0, 6.0]], [[1.0, 5.0, 5.0], [7.0, 6.0, 7.0], [1.0, 6.0, 6.0]], [[4.0, 7.0, 
6.0], [3.0, 7.0, 7.0], [2.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 4.0], [2.0, 1.0, 7.0], [4.0, 1.0, 8.0]], 
[[3.0, 2.0, 5.0], [2.0, 8.0, 5.0], [5.0, 5.0, 7.0]], [[8.0, 4.0, 3.0], [3.0, 3.0, 8.0], [8.0, 8.0, 7.0]]], [[[8.0, 6.0, 4.0], 
[6.0, 1.0, 1.0], [5.0, 2.0, 5.0]], [[8.0, 2.0, 4.0], [2.0, 1.0, 4.0], [8.0, 7.0, 7.0]], [[4.0, 5.0, 6.0], [5.0, 4.0, 6.0], 
[4.0, 3.0, 5.0]]], [[[6.0, 6.0, 5.0], [3.0, 1.0, 7.0], [2.0, 5.0, 7.0]], [[7.0, 2.0, 6.0], [2.0, 5.0, 3.0], [5.0, 6.0, 6.0]], 
[[6.0, 4.0, 8.0], [2.0, 1.0, 6.0], [1.0, 6.0, 1.0]]], [[[1.0, 2.0, 2.0], [1.0, 7.0, 5.0], [7.0, 5.0, 1.0]], [[5.0, 3.0, 1.0], 
[8.0, 2.0, 7.0], [8.0, 4.0, 8.0]], [[2.0, 8.0, 8.0], [7.0, 6.0, 3.0], [8.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PRLN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 7.0], [4.0, 5.0, 7.0], [1.0, 2.0, 7.0]], [[7.0, 7.0, 8.0], [3.0, 6.0, 1.0], [6.0, 2.0, 
3.0]], [[2.0, 5.0, 7.0], [4.0, 6.0, 1.0], [2.0, 7.0, 7.0]]], [[[8.0, 8.0, 7.0], [1.0, 5.0, 5.0], [6.0, 4.0, 6.0]], [[1.0, 8.0, 
7.0], [7.0, 7.0, 3.0], [4.0, 2.0, 7.0]], [[6.0, 2.0, 4.0], [2.0, 5.0, 1.0], [4.0, 6.0, 8.0]]], [[[6.0, 5.0, 5.0], [4.0, 1.0, 
6.0], [8.0, 3.0, 3.0]], [[5.0, 3.0, 4.0], [5.0, 8.0, 2.0], [1.0, 8.0, 5.0]], [[2.0, 2.0, 5.0], [1.0, 4.0, 2.0], [6.0, 5.0, 
1.0]]], [[[1.0, 7.0, 3.0], [6.0, 4.0, 2.0], [2.0, 4.0, 6.0]], [[7.0, 8.0, 7.0], [3.0, 5.0, 2.0], [5.0, 1.0, 1.0]], [[5.0, 8.0, 
4.0], [2.0, 2.0, 8.0], [3.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 7.0], [8.0, 8.0, 1.0], [5.0, 5.0, 6.0]], [[7.0, 1.0, 2.0], [3.0, 6.0, 
1.0], [3.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [1.0, 4.0, 4.0], [6.0, 5.0, 2.0]]], [[[8.0, 2.0, 5.0], [8.0, 2.0, 1.0], [5.0, 6.0, 
8.0]], [[5.0, 6.0, 7.0], [6.0, 6.0, 1.0], [8.0, 2.0, 2.0]], [[4.0, 3.0, 1.0], [7.0, 7.0, 3.0], [6.0, 1.0, 3.0]]], [[[7.0, 1.0, 
7.0], [4.0, 6.0, 5.0], [4.0, 3.0, 2.0]], [[8.0, 7.0, 4.0], [5.0, 8.0, 5.0], [8.0, 1.0, 6.0]], [[5.0, 3.0, 7.0], [4.0, 2.0, 
1.0], [8.0, 6.0, 3.0]]], [[[8.0, 1.0, 6.0], [3.0, 2.0, 3.0], [5.0, 1.0, 3.0]], [[4.0, 3.0, 4.0], [8.0, 7.0, 1.0], [1.0, 6.0, 
1.0]], [[8.0, 1.0, 8.0], [3.0, 4.0, 8.0], [2.0, 5.0, 6.0]]]]))
      val=Data(numpy.array([[[7.0, 5.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 7.0, 2.0]]]))
      arg[2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 7.0], [4.0, 5.0, 7.0], [1.0, 2.0, 7.0]], [[7.0, 7.0, 8.0], [3.0, 6.0, 1.0], [6.0, 
2.0, 3.0]], [[2.0, 5.0, 7.0], [4.0, 6.0, 1.0], [2.0, 7.0, 7.0]]], [[[8.0, 8.0, 7.0], [1.0, 5.0, 5.0], [6.0, 4.0, 6.0]], [[1.0, 
8.0, 7.0], [7.0, 7.0, 3.0], [4.0, 2.0, 7.0]], [[6.0, 2.0, 4.0], [2.0, 5.0, 1.0], [4.0, 6.0, 8.0]]], [[[6.0, 5.0, 5.0], [4.0, 
1.0, 6.0], [8.0, 3.0, 3.0]], [[5.0, 3.0, 4.0], [5.0, 8.0, 2.0], [7.0, 5.0, 3.0]], [[2.0, 2.0, 5.0], [1.0, 4.0, 2.0], [6.0, 5.0, 
1.0]]], [[[1.0, 7.0, 3.0], [6.0, 4.0, 2.0], [2.0, 4.0, 6.0]], [[7.0, 8.0, 7.0], [3.0, 5.0, 2.0], [5.0, 1.0, 1.0]], [[5.0, 8.0, 
4.0], [2.0, 2.0, 8.0], [3.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 7.0], [8.0, 8.0, 1.0], [5.0, 5.0, 6.0]], 
[[7.0, 1.0, 2.0], [3.0, 6.0, 1.0], [3.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [1.0, 4.0, 4.0], [6.0, 5.0, 2.0]]], [[[8.0, 2.0, 5.0], 
[8.0, 2.0, 1.0], [5.0, 6.0, 8.0]], [[5.0, 6.0, 7.0], [6.0, 6.0, 1.0], [8.0, 2.0, 2.0]], [[4.0, 3.0, 1.0], [7.0, 7.0, 3.0], 
[6.0, 1.0, 3.0]]], [[[7.0, 1.0, 7.0], [4.0, 6.0, 5.0], [4.0, 3.0, 2.0]], [[8.0, 7.0, 4.0], [5.0, 8.0, 5.0], [4.0, 7.0, 2.0]], 
[[5.0, 3.0, 7.0], [4.0, 2.0, 1.0], [8.0, 6.0, 3.0]]], [[[8.0, 1.0, 6.0], [3.0, 2.0, 3.0], [5.0, 1.0, 3.0]], [[4.0, 3.0, 4.0], 
[8.0, 7.0, 1.0], [1.0, 6.0, 1.0]], [[8.0, 1.0, 8.0], [3.0, 4.0, 8.0], [2.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PRLN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 4.0], [3.0, 8.0, 5.0], [4.0, 5.0, 8.0]], [[8.0, 7.0, 3.0], [6.0, 3.0, 3.0], [7.0, 4.0, 
4.0]], [[3.0, 1.0, 6.0], [3.0, 4.0, 4.0], [3.0, 5.0, 4.0]]], [[[1.0, 8.0, 8.0], [2.0, 5.0, 2.0], [7.0, 1.0, 1.0]], [[1.0, 4.0, 
2.0], [2.0, 3.0, 4.0], [7.0, 6.0, 5.0]], [[8.0, 6.0, 2.0], [7.0, 3.0, 7.0], [6.0, 6.0, 8.0]]], [[[1.0, 2.0, 6.0], [3.0, 3.0, 
7.0], [2.0, 2.0, 7.0]], [[3.0, 4.0, 5.0], [1.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[5.0, 3.0, 5.0], [1.0, 7.0, 8.0], [3.0, 5.0, 
1.0]]], [[[2.0, 1.0, 8.0], [8.0, 6.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 4.0, 6.0], [8.0, 6.0, 8.0], [4.0, 4.0, 6.0]], [[6.0, 1.0, 
3.0], [2.0, 4.0, 6.0], [6.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 7.0], [6.0, 8.0, 3.0], [1.0, 4.0, 1.0]], [[6.0, 5.0, 6.0], [3.0, 6.0, 
8.0], [8.0, 6.0, 7.0]], [[2.0, 7.0, 7.0], [3.0, 5.0, 5.0], [3.0, 7.0, 7.0]]], [[[4.0, 5.0, 2.0], [8.0, 2.0, 7.0], [4.0, 2.0, 
7.0]], [[4.0, 4.0, 8.0], [1.0, 7.0, 8.0], [7.0, 4.0, 7.0]], [[8.0, 8.0, 5.0], [8.0, 5.0, 2.0], [4.0, 1.0, 1.0]]], [[[5.0, 1.0, 
3.0], [5.0, 6.0, 2.0], [7.0, 2.0, 3.0]], [[7.0, 1.0, 5.0], [2.0, 1.0, 7.0], [6.0, 1.0, 4.0]], [[5.0, 3.0, 7.0], [1.0, 5.0, 
3.0], [5.0, 5.0, 6.0]]], [[[2.0, 2.0, 2.0], [4.0, 8.0, 4.0], [8.0, 3.0, 5.0]], [[5.0, 7.0, 7.0], [1.0, 2.0, 4.0], [5.0, 6.0, 
7.0]], [[6.0, 6.0, 8.0], [3.0, 2.0, 1.0], [6.0, 4.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 2.0, 1.0]]])+(1.-msk_val)*numpy.array([[[5.0, 4.0, 8.0]]])
      arg[2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 4.0], [3.0, 8.0, 5.0], [4.0, 5.0, 8.0]], [[8.0, 7.0, 3.0], [6.0, 3.0, 3.0], [7.0, 
4.0, 4.0]], [[3.0, 1.0, 6.0], [3.0, 4.0, 4.0], [3.0, 5.0, 4.0]]], [[[1.0, 8.0, 8.0], [2.0, 5.0, 2.0], [7.0, 1.0, 1.0]], [[1.0, 
4.0, 2.0], [2.0, 3.0, 4.0], [7.0, 6.0, 5.0]], [[8.0, 6.0, 2.0], [7.0, 3.0, 7.0], [6.0, 6.0, 8.0]]], [[[1.0, 2.0, 6.0], [3.0, 
3.0, 7.0], [2.0, 2.0, 7.0]], [[3.0, 4.0, 5.0], [1.0, 4.0, 5.0], [4.0, 2.0, 1.0]], [[5.0, 3.0, 5.0], [1.0, 7.0, 8.0], [3.0, 5.0, 
1.0]]], [[[2.0, 1.0, 8.0], [8.0, 6.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 4.0, 6.0], [8.0, 6.0, 8.0], [4.0, 4.0, 6.0]], [[6.0, 1.0, 
3.0], [2.0, 4.0, 6.0], [6.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 7.0], [6.0, 8.0, 3.0], [1.0, 4.0, 1.0]], 
[[6.0, 5.0, 6.0], [3.0, 6.0, 8.0], [8.0, 6.0, 7.0]], [[2.0, 7.0, 7.0], [3.0, 5.0, 5.0], [3.0, 7.0, 7.0]]], [[[4.0, 5.0, 2.0], 
[8.0, 2.0, 7.0], [4.0, 2.0, 7.0]], [[4.0, 4.0, 8.0], [1.0, 7.0, 8.0], [7.0, 4.0, 7.0]], [[8.0, 8.0, 5.0], [8.0, 5.0, 2.0], 
[4.0, 1.0, 1.0]]], [[[5.0, 1.0, 3.0], [5.0, 6.0, 2.0], [7.0, 2.0, 3.0]], [[7.0, 1.0, 5.0], [2.0, 1.0, 7.0], [5.0, 4.0, 8.0]], 
[[5.0, 3.0, 7.0], [1.0, 5.0, 3.0], [5.0, 5.0, 6.0]]], [[[2.0, 2.0, 2.0], [4.0, 8.0, 4.0], [8.0, 3.0, 5.0]], [[5.0, 7.0, 7.0], 
[1.0, 2.0, 4.0], [5.0, 6.0, 7.0]], [[6.0, 6.0, 8.0], [3.0, 2.0, 1.0], [6.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRUN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 8.0], [3.0, 1.0, 2.0], [2.0, 1.0, 6.0]], [[2.0, 3.0, 7.0], [5.0, 8.0, 6.0], [1.0, 3.0, 
5.0]], [[7.0, 2.0, 8.0], [1.0, 1.0, 5.0], [3.0, 7.0, 5.0]]], [[[6.0, 2.0, 1.0], [4.0, 1.0, 7.0], [6.0, 2.0, 5.0]], [[1.0, 6.0, 
4.0], [2.0, 1.0, 6.0], [5.0, 7.0, 5.0]], [[3.0, 3.0, 4.0], [5.0, 2.0, 7.0], [3.0, 6.0, 8.0]]], [[[5.0, 1.0, 7.0], [1.0, 2.0, 
2.0], [4.0, 2.0, 2.0]], [[7.0, 4.0, 2.0], [5.0, 7.0, 1.0], [6.0, 8.0, 7.0]], [[7.0, 3.0, 1.0], [5.0, 2.0, 2.0], [4.0, 4.0, 
4.0]]], [[[4.0, 7.0, 5.0], [5.0, 3.0, 8.0], [1.0, 6.0, 4.0]], [[8.0, 3.0, 5.0], [3.0, 7.0, 2.0], [6.0, 8.0, 1.0]], [[2.0, 1.0, 
2.0], [8.0, 8.0, 7.0], [8.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 3.0], [2.0, 6.0, 1.0], [6.0, 6.0, 7.0]], [[8.0, 1.0, 6.0], [6.0, 1.0, 
3.0], [4.0, 4.0, 6.0]], [[2.0, 5.0, 3.0], [8.0, 8.0, 5.0], [7.0, 8.0, 6.0]]], [[[1.0, 8.0, 1.0], [5.0, 4.0, 2.0], [8.0, 4.0, 
6.0]], [[8.0, 6.0, 1.0], [2.0, 5.0, 2.0], [1.0, 6.0, 5.0]], [[6.0, 6.0, 5.0], [1.0, 6.0, 8.0], [4.0, 1.0, 3.0]]], [[[2.0, 7.0, 
1.0], [6.0, 2.0, 5.0], [6.0, 2.0, 4.0]], [[1.0, 8.0, 8.0], [3.0, 7.0, 7.0], [8.0, 1.0, 4.0]], [[1.0, 5.0, 4.0], [4.0, 5.0, 
5.0], [6.0, 6.0, 5.0]]], [[[5.0, 2.0, 7.0], [3.0, 6.0, 7.0], [6.0, 2.0, 7.0]], [[6.0, 7.0, 8.0], [6.0, 7.0, 7.0], [8.0, 6.0, 
4.0]], [[2.0, 4.0, 5.0], [6.0, 6.0, 1.0], [8.0, 5.0, 8.0]]]]))
      val=numpy.array([[[[6.0, 2.0, 5.0], [6.0, 1.0, 1.0]]], [[[7.0, 4.0, 1.0], [8.0, 1.0, 4.0]]]])
      arg[1:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 8.0], [3.0, 1.0, 2.0], [2.0, 1.0, 6.0]], [[2.0, 3.0, 7.0], [5.0, 8.0, 6.0], [1.0, 
3.0, 5.0]], [[7.0, 2.0, 8.0], [1.0, 1.0, 5.0], [3.0, 7.0, 5.0]]], [[[6.0, 2.0, 1.0], [4.0, 1.0, 7.0], [6.0, 2.0, 5.0]], [[6.0, 
2.0, 5.0], [6.0, 1.0, 1.0], [5.0, 7.0, 5.0]], [[3.0, 3.0, 4.0], [5.0, 2.0, 7.0], [3.0, 6.0, 8.0]]], [[[5.0, 1.0, 7.0], [1.0, 
2.0, 2.0], [4.0, 2.0, 2.0]], [[7.0, 4.0, 1.0], [8.0, 1.0, 4.0], [6.0, 8.0, 7.0]], [[7.0, 3.0, 1.0], [5.0, 2.0, 2.0], [4.0, 4.0, 
4.0]]], [[[4.0, 7.0, 5.0], [5.0, 3.0, 8.0], [1.0, 6.0, 4.0]], [[8.0, 3.0, 5.0], [3.0, 7.0, 2.0], [6.0, 8.0, 1.0]], [[2.0, 1.0, 
2.0], [8.0, 8.0, 7.0], [8.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 3.0], [2.0, 6.0, 1.0], [6.0, 6.0, 7.0]], 
[[8.0, 1.0, 6.0], [6.0, 1.0, 3.0], [4.0, 4.0, 6.0]], [[2.0, 5.0, 3.0], [8.0, 8.0, 5.0], [7.0, 8.0, 6.0]]], [[[1.0, 8.0, 1.0], 
[5.0, 4.0, 2.0], [8.0, 4.0, 6.0]], [[6.0, 2.0, 5.0], [6.0, 1.0, 1.0], [1.0, 6.0, 5.0]], [[6.0, 6.0, 5.0], [1.0, 6.0, 8.0], 
[4.0, 1.0, 3.0]]], [[[2.0, 7.0, 1.0], [6.0, 2.0, 5.0], [6.0, 2.0, 4.0]], [[7.0, 4.0, 1.0], [8.0, 1.0, 4.0], [8.0, 1.0, 4.0]], 
[[1.0, 5.0, 4.0], [4.0, 5.0, 5.0], [6.0, 6.0, 5.0]]], [[[5.0, 2.0, 7.0], [3.0, 6.0, 7.0], [6.0, 2.0, 7.0]], [[6.0, 7.0, 8.0], 
[6.0, 7.0, 7.0], [8.0, 6.0, 4.0]], [[2.0, 4.0, 5.0], [6.0, 6.0, 1.0], [8.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRUN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 1.0], [1.0, 7.0, 5.0], [5.0, 8.0, 1.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 7.0], [7.0, 7.0, 
3.0]], [[1.0, 8.0, 6.0], [7.0, 5.0, 3.0], [6.0, 1.0, 5.0]]], [[[4.0, 7.0, 5.0], [1.0, 2.0, 8.0], [4.0, 8.0, 6.0]], [[6.0, 5.0, 
4.0], [5.0, 2.0, 5.0], [1.0, 3.0, 5.0]], [[4.0, 8.0, 8.0], [1.0, 6.0, 2.0], [1.0, 8.0, 5.0]]], [[[8.0, 1.0, 2.0], [7.0, 3.0, 
8.0], [2.0, 2.0, 1.0]], [[3.0, 5.0, 3.0], [8.0, 5.0, 3.0], [7.0, 2.0, 1.0]], [[6.0, 1.0, 4.0], [6.0, 1.0, 3.0], [1.0, 2.0, 
7.0]]], [[[7.0, 8.0, 7.0], [8.0, 6.0, 1.0], [6.0, 3.0, 8.0]], [[6.0, 4.0, 8.0], [4.0, 3.0, 5.0], [2.0, 3.0, 4.0]], [[8.0, 2.0, 
5.0], [7.0, 8.0, 8.0], [8.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 5.0], [7.0, 1.0, 8.0], [2.0, 1.0, 6.0]], [[3.0, 4.0, 7.0], [3.0, 8.0, 
3.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 8.0], [1.0, 4.0, 1.0], [1.0, 8.0, 5.0]]], [[[8.0, 3.0, 2.0], [4.0, 2.0, 7.0], [8.0, 8.0, 
6.0]], [[2.0, 1.0, 7.0], [6.0, 6.0, 6.0], [5.0, 4.0, 5.0]], [[3.0, 8.0, 1.0], [6.0, 7.0, 8.0], [8.0, 6.0, 3.0]]], [[[3.0, 3.0, 
3.0], [7.0, 6.0, 2.0], [6.0, 3.0, 4.0]], [[2.0, 4.0, 7.0], [5.0, 7.0, 7.0], [7.0, 5.0, 3.0]], [[6.0, 8.0, 7.0], [2.0, 7.0, 
5.0], [3.0, 2.0, 3.0]]], [[[8.0, 2.0, 5.0], [2.0, 5.0, 4.0], [1.0, 7.0, 8.0]], [[8.0, 8.0, 1.0], [7.0, 5.0, 5.0], [6.0, 7.0, 
6.0]], [[5.0, 2.0, 7.0], [7.0, 8.0, 1.0], [4.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0, 4.0, 4.0], [3.0, 5.0, 1.0]]], [[[4.0, 5.0, 4.0], [7.0, 2.0, 1.0]]]]),self.functionspace)
      arg[1:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 1.0], [1.0, 7.0, 5.0], [5.0, 8.0, 1.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 7.0], [7.0, 
7.0, 3.0]], [[1.0, 8.0, 6.0], [7.0, 5.0, 3.0], [6.0, 1.0, 5.0]]], [[[4.0, 7.0, 5.0], [1.0, 2.0, 8.0], [4.0, 8.0, 6.0]], [[5.0, 
4.0, 4.0], [3.0, 5.0, 1.0], [1.0, 3.0, 5.0]], [[4.0, 8.0, 8.0], [1.0, 6.0, 2.0], [1.0, 8.0, 5.0]]], [[[8.0, 1.0, 2.0], [7.0, 
3.0, 8.0], [2.0, 2.0, 1.0]], [[4.0, 5.0, 4.0], [7.0, 2.0, 1.0], [7.0, 2.0, 1.0]], [[6.0, 1.0, 4.0], [6.0, 1.0, 3.0], [1.0, 2.0, 
7.0]]], [[[7.0, 8.0, 7.0], [8.0, 6.0, 1.0], [6.0, 3.0, 8.0]], [[6.0, 4.0, 8.0], [4.0, 3.0, 5.0], [2.0, 3.0, 4.0]], [[8.0, 2.0, 
5.0], [7.0, 8.0, 8.0], [8.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 5.0], [7.0, 1.0, 8.0], [2.0, 1.0, 6.0]], 
[[3.0, 4.0, 7.0], [3.0, 8.0, 3.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 8.0], [1.0, 4.0, 1.0], [1.0, 8.0, 5.0]]], [[[8.0, 3.0, 2.0], 
[4.0, 2.0, 7.0], [8.0, 8.0, 6.0]], [[5.0, 4.0, 4.0], [3.0, 5.0, 1.0], [5.0, 4.0, 5.0]], [[3.0, 8.0, 1.0], [6.0, 7.0, 8.0], 
[8.0, 6.0, 3.0]]], [[[3.0, 3.0, 3.0], [7.0, 6.0, 2.0], [6.0, 3.0, 4.0]], [[4.0, 5.0, 4.0], [7.0, 2.0, 1.0], [7.0, 5.0, 3.0]], 
[[6.0, 8.0, 7.0], [2.0, 7.0, 5.0], [3.0, 2.0, 3.0]]], [[[8.0, 2.0, 5.0], [2.0, 5.0, 4.0], [1.0, 7.0, 8.0]], [[8.0, 8.0, 1.0], 
[7.0, 5.0, 5.0], [6.0, 7.0, 6.0]], [[5.0, 2.0, 7.0], [7.0, 8.0, 1.0], [4.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [8.0, 7.0, 4.0], [3.0, 6.0, 2.0]], [[5.0, 5.0, 1.0], [6.0, 3.0, 4.0], [6.0, 2.0, 
7.0]], [[2.0, 4.0, 6.0], [3.0, 6.0, 1.0], [3.0, 2.0, 8.0]]], [[[1.0, 3.0, 7.0], [1.0, 2.0, 8.0], [4.0, 8.0, 6.0]], [[5.0, 5.0, 
7.0], [2.0, 5.0, 1.0], [5.0, 3.0, 5.0]], [[4.0, 8.0, 3.0], [2.0, 4.0, 7.0], [3.0, 8.0, 8.0]]], [[[7.0, 4.0, 1.0], [4.0, 4.0, 
2.0], [6.0, 3.0, 6.0]], [[6.0, 1.0, 7.0], [4.0, 5.0, 4.0], [8.0, 8.0, 8.0]], [[1.0, 2.0, 2.0], [2.0, 6.0, 5.0], [6.0, 8.0, 
1.0]]], [[[6.0, 8.0, 1.0], [8.0, 5.0, 2.0], [7.0, 1.0, 2.0]], [[4.0, 8.0, 3.0], [1.0, 5.0, 4.0], [1.0, 8.0, 2.0]], [[1.0, 7.0, 
2.0], [1.0, 4.0, 5.0], [2.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 5.0], [8.0, 1.0, 7.0], [5.0, 3.0, 4.0]], [[3.0, 5.0, 5.0], [1.0, 6.0, 
3.0], [1.0, 3.0, 7.0]], [[6.0, 8.0, 3.0], [2.0, 7.0, 5.0], [5.0, 4.0, 3.0]]], [[[7.0, 1.0, 3.0], [5.0, 3.0, 1.0], [2.0, 4.0, 
7.0]], [[4.0, 1.0, 5.0], [8.0, 7.0, 8.0], [4.0, 4.0, 2.0]], [[1.0, 1.0, 3.0], [2.0, 7.0, 7.0], [2.0, 4.0, 3.0]]], [[[3.0, 1.0, 
6.0], [2.0, 1.0, 3.0], [1.0, 2.0, 3.0]], [[6.0, 8.0, 3.0], [2.0, 1.0, 2.0], [3.0, 6.0, 5.0]], [[1.0, 6.0, 1.0], [6.0, 3.0, 
6.0], [6.0, 7.0, 7.0]]], [[[2.0, 4.0, 2.0], [8.0, 3.0, 4.0], [7.0, 8.0, 7.0]], [[1.0, 6.0, 8.0], [3.0, 1.0, 4.0], [3.0, 6.0, 
6.0]], [[3.0, 5.0, 4.0], [7.0, 6.0, 2.0], [5.0, 3.0, 3.0]]]]))
      val=Data(numpy.array([[[[3.0, 4.0, 5.0], [4.0, 2.0, 4.0]]], [[[5.0, 7.0, 8.0], [1.0, 5.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 5.0], [8.0, 2.0, 8.0]]], [[[6.0, 6.0, 2.0], [8.0, 3.0, 1.0]]]]))
      arg[1:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [8.0, 7.0, 4.0], [3.0, 6.0, 2.0]], [[5.0, 5.0, 1.0], [6.0, 3.0, 4.0], [6.0, 
2.0, 7.0]], [[2.0, 4.0, 6.0], [3.0, 6.0, 1.0], [3.0, 2.0, 8.0]]], [[[1.0, 3.0, 7.0], [1.0, 2.0, 8.0], [4.0, 8.0, 6.0]], [[3.0, 
4.0, 5.0], [4.0, 2.0, 4.0], [5.0, 3.0, 5.0]], [[4.0, 8.0, 3.0], [2.0, 4.0, 7.0], [3.0, 8.0, 8.0]]], [[[7.0, 4.0, 1.0], [4.0, 
4.0, 2.0], [6.0, 3.0, 6.0]], [[5.0, 7.0, 8.0], [1.0, 5.0, 1.0], [8.0, 8.0, 8.0]], [[1.0, 2.0, 2.0], [2.0, 6.0, 5.0], [6.0, 8.0, 
1.0]]], [[[6.0, 8.0, 1.0], [8.0, 5.0, 2.0], [7.0, 1.0, 2.0]], [[4.0, 8.0, 3.0], [1.0, 5.0, 4.0], [1.0, 8.0, 2.0]], [[1.0, 7.0, 
2.0], [1.0, 4.0, 5.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 5.0], [8.0, 1.0, 7.0], [5.0, 3.0, 4.0]], 
[[3.0, 5.0, 5.0], [1.0, 6.0, 3.0], [1.0, 3.0, 7.0]], [[6.0, 8.0, 3.0], [2.0, 7.0, 5.0], [5.0, 4.0, 3.0]]], [[[7.0, 1.0, 3.0], 
[5.0, 3.0, 1.0], [2.0, 4.0, 7.0]], [[6.0, 3.0, 5.0], [8.0, 2.0, 8.0], [4.0, 4.0, 2.0]], [[1.0, 1.0, 3.0], [2.0, 7.0, 7.0], 
[2.0, 4.0, 3.0]]], [[[3.0, 1.0, 6.0], [2.0, 1.0, 3.0], [1.0, 2.0, 3.0]], [[6.0, 6.0, 2.0], [8.0, 3.0, 1.0], [3.0, 6.0, 5.0]], 
[[1.0, 6.0, 1.0], [6.0, 3.0, 6.0], [6.0, 7.0, 7.0]]], [[[2.0, 4.0, 2.0], [8.0, 3.0, 4.0], [7.0, 8.0, 7.0]], [[1.0, 6.0, 8.0], 
[3.0, 1.0, 4.0], [3.0, 6.0, 6.0]], [[3.0, 5.0, 4.0], [7.0, 6.0, 2.0], [5.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [5.0, 3.0, 1.0], [4.0, 2.0, 3.0]], [[6.0, 6.0, 5.0], [1.0, 2.0, 2.0], [1.0, 2.0, 
3.0]], [[5.0, 3.0, 7.0], [3.0, 3.0, 4.0], [4.0, 2.0, 5.0]]], [[[8.0, 2.0, 6.0], [4.0, 4.0, 4.0], [8.0, 7.0, 4.0]], [[2.0, 7.0, 
4.0], [4.0, 6.0, 1.0], [8.0, 2.0, 1.0]], [[1.0, 2.0, 1.0], [3.0, 3.0, 7.0], [7.0, 1.0, 8.0]]], [[[6.0, 3.0, 1.0], [8.0, 2.0, 
8.0], [6.0, 4.0, 5.0]], [[2.0, 4.0, 7.0], [5.0, 4.0, 7.0], [6.0, 5.0, 7.0]], [[2.0, 6.0, 7.0], [5.0, 4.0, 7.0], [2.0, 2.0, 
3.0]]], [[[5.0, 1.0, 1.0], [4.0, 2.0, 5.0], [1.0, 8.0, 3.0]], [[6.0, 5.0, 2.0], [6.0, 1.0, 2.0], [7.0, 4.0, 4.0]], [[1.0, 4.0, 
6.0], [4.0, 8.0, 5.0], [4.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 4.0], [2.0, 2.0, 6.0], [6.0, 7.0, 3.0]], [[5.0, 4.0, 2.0], [8.0, 1.0, 
1.0], [5.0, 3.0, 8.0]], [[6.0, 3.0, 1.0], [8.0, 7.0, 6.0], [1.0, 4.0, 5.0]]], [[[8.0, 8.0, 3.0], [3.0, 1.0, 3.0], [7.0, 8.0, 
3.0]], [[8.0, 7.0, 1.0], [4.0, 4.0, 1.0], [7.0, 6.0, 5.0]], [[1.0, 1.0, 7.0], [4.0, 5.0, 4.0], [5.0, 8.0, 6.0]]], [[[8.0, 1.0, 
8.0], [4.0, 1.0, 6.0], [2.0, 4.0, 7.0]], [[8.0, 1.0, 8.0], [6.0, 8.0, 2.0], [4.0, 8.0, 1.0]], [[8.0, 6.0, 7.0], [1.0, 4.0, 
3.0], [8.0, 1.0, 3.0]]], [[[8.0, 5.0, 1.0], [2.0, 8.0, 4.0], [2.0, 4.0, 4.0]], [[2.0, 5.0, 8.0], [3.0, 3.0, 6.0], [6.0, 7.0, 
1.0]], [[5.0, 8.0, 8.0], [8.0, 4.0, 2.0], [4.0, 4.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 3.0, 8.0], [4.0, 5.0, 6.0]]], [[[4.0, 5.0, 2.0], [6.0, 4.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 7.0, 1.0], [3.0, 4.0, 8.0]]], [[[4.0, 5.0, 5.0], [2.0, 4.0, 2.0]]]])
      arg[1:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [5.0, 3.0, 1.0], [4.0, 2.0, 3.0]], [[6.0, 6.0, 5.0], [1.0, 2.0, 2.0], [1.0, 
2.0, 3.0]], [[5.0, 3.0, 7.0], [3.0, 3.0, 4.0], [4.0, 2.0, 5.0]]], [[[8.0, 2.0, 6.0], [4.0, 4.0, 4.0], [8.0, 7.0, 4.0]], [[5.0, 
3.0, 8.0], [4.0, 5.0, 6.0], [8.0, 2.0, 1.0]], [[1.0, 2.0, 1.0], [3.0, 3.0, 7.0], [7.0, 1.0, 8.0]]], [[[6.0, 3.0, 1.0], [8.0, 
2.0, 8.0], [6.0, 4.0, 5.0]], [[4.0, 5.0, 2.0], [6.0, 4.0, 5.0], [6.0, 5.0, 7.0]], [[2.0, 6.0, 7.0], [5.0, 4.0, 7.0], [2.0, 2.0, 
3.0]]], [[[5.0, 1.0, 1.0], [4.0, 2.0, 5.0], [1.0, 8.0, 3.0]], [[6.0, 5.0, 2.0], [6.0, 1.0, 2.0], [7.0, 4.0, 4.0]], [[1.0, 4.0, 
6.0], [4.0, 8.0, 5.0], [4.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 4.0], [2.0, 2.0, 6.0], [6.0, 7.0, 3.0]], 
[[5.0, 4.0, 2.0], [8.0, 1.0, 1.0], [5.0, 3.0, 8.0]], [[6.0, 3.0, 1.0], [8.0, 7.0, 6.0], [1.0, 4.0, 5.0]]], [[[8.0, 8.0, 3.0], 
[3.0, 1.0, 3.0], [7.0, 8.0, 3.0]], [[5.0, 7.0, 1.0], [3.0, 4.0, 8.0], [7.0, 6.0, 5.0]], [[1.0, 1.0, 7.0], [4.0, 5.0, 4.0], 
[5.0, 8.0, 6.0]]], [[[8.0, 1.0, 8.0], [4.0, 1.0, 6.0], [2.0, 4.0, 7.0]], [[4.0, 5.0, 5.0], [2.0, 4.0, 2.0], [4.0, 8.0, 1.0]], 
[[8.0, 6.0, 7.0], [1.0, 4.0, 3.0], [8.0, 1.0, 3.0]]], [[[8.0, 5.0, 1.0], [2.0, 8.0, 4.0], [2.0, 4.0, 4.0]], [[2.0, 5.0, 8.0], 
[3.0, 3.0, 6.0], [6.0, 7.0, 1.0]], [[5.0, 8.0, 8.0], [8.0, 4.0, 2.0], [4.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLLL(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 8.0], [2.0, 7.0, 2.0], [4.0, 6.0, 8.0]], [[1.0, 8.0, 7.0], [3.0, 5.0, 5.0], [2.0, 4.0, 
4.0]], [[7.0, 1.0, 5.0], [2.0, 4.0, 7.0], [4.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], [8.0, 3.0, 8.0], [5.0, 4.0, 4.0]], [[8.0, 5.0, 
2.0], [3.0, 5.0, 3.0], [8.0, 3.0, 7.0]], [[6.0, 6.0, 7.0], [3.0, 6.0, 7.0], [2.0, 7.0, 1.0]]], [[[6.0, 3.0, 1.0], [7.0, 5.0, 
4.0], [5.0, 1.0, 6.0]], [[3.0, 3.0, 7.0], [3.0, 2.0, 3.0], [2.0, 7.0, 1.0]], [[3.0, 5.0, 8.0], [3.0, 3.0, 3.0], [1.0, 1.0, 
8.0]]], [[[3.0, 2.0, 1.0], [2.0, 8.0, 2.0], [8.0, 8.0, 8.0]], [[6.0, 5.0, 2.0], [4.0, 1.0, 3.0], [6.0, 8.0, 6.0]], [[2.0, 6.0, 
1.0], [5.0, 8.0, 8.0], [8.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 2.0], [1.0, 5.0, 1.0], [4.0, 5.0, 5.0]], [[1.0, 1.0, 3.0], [1.0, 5.0, 
4.0], [2.0, 7.0, 2.0]], [[4.0, 3.0, 6.0], [6.0, 8.0, 8.0], [1.0, 3.0, 2.0]]], [[[8.0, 6.0, 1.0], [5.0, 4.0, 5.0], [8.0, 4.0, 
1.0]], [[5.0, 3.0, 3.0], [6.0, 8.0, 4.0], [3.0, 1.0, 7.0]], [[4.0, 1.0, 2.0], [5.0, 8.0, 6.0], [8.0, 2.0, 4.0]]], [[[5.0, 7.0, 
8.0], [1.0, 4.0, 3.0], [8.0, 4.0, 6.0]], [[3.0, 8.0, 6.0], [8.0, 7.0, 2.0], [7.0, 8.0, 3.0]], [[7.0, 7.0, 2.0], [6.0, 8.0, 
1.0], [3.0, 3.0, 6.0]]], [[[4.0, 7.0, 4.0], [4.0, 8.0, 7.0], [1.0, 6.0, 3.0]], [[2.0, 3.0, 6.0], [4.0, 4.0, 2.0], [7.0, 7.0, 
1.0]], [[4.0, 5.0, 6.0], [5.0, 5.0, 2.0], [8.0, 7.0, 3.0]]]]))
      val=numpy.array([[[[6.0]]], [[[1.0]]]])
      arg[2:,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 8.0], [2.0, 7.0, 2.0], [4.0, 6.0, 8.0]], [[1.0, 8.0, 7.0], [3.0, 5.0, 5.0], [2.0, 
4.0, 4.0]], [[7.0, 1.0, 5.0], [2.0, 4.0, 7.0], [4.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], [8.0, 3.0, 8.0], [5.0, 4.0, 4.0]], [[8.0, 
5.0, 2.0], [3.0, 5.0, 3.0], [8.0, 3.0, 7.0]], [[6.0, 6.0, 7.0], [3.0, 6.0, 7.0], [2.0, 7.0, 1.0]]], [[[6.0, 3.0, 1.0], [7.0, 
5.0, 4.0], [5.0, 1.0, 6.0]], [[3.0, 3.0, 7.0], [3.0, 2.0, 3.0], [2.0, 7.0, 1.0]], [[3.0, 5.0, 8.0], [3.0, 3.0, 3.0], [1.0, 1.0, 
6.0]]], [[[3.0, 2.0, 1.0], [2.0, 8.0, 2.0], [8.0, 8.0, 8.0]], [[6.0, 5.0, 2.0], [4.0, 1.0, 3.0], [6.0, 8.0, 6.0]], [[2.0, 6.0, 
1.0], [5.0, 8.0, 8.0], [8.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 2.0], [1.0, 5.0, 1.0], [4.0, 5.0, 5.0]], 
[[1.0, 1.0, 3.0], [1.0, 5.0, 4.0], [2.0, 7.0, 2.0]], [[4.0, 3.0, 6.0], [6.0, 8.0, 8.0], [1.0, 3.0, 2.0]]], [[[8.0, 6.0, 1.0], 
[5.0, 4.0, 5.0], [8.0, 4.0, 1.0]], [[5.0, 3.0, 3.0], [6.0, 8.0, 4.0], [3.0, 1.0, 7.0]], [[4.0, 1.0, 2.0], [5.0, 8.0, 6.0], 
[8.0, 2.0, 4.0]]], [[[5.0, 7.0, 8.0], [1.0, 4.0, 3.0], [8.0, 4.0, 6.0]], [[3.0, 8.0, 6.0], [8.0, 7.0, 2.0], [7.0, 8.0, 3.0]], 
[[7.0, 7.0, 2.0], [6.0, 8.0, 1.0], [3.0, 3.0, 6.0]]], [[[4.0, 7.0, 4.0], [4.0, 8.0, 7.0], [1.0, 6.0, 3.0]], [[2.0, 3.0, 6.0], 
[4.0, 4.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 5.0, 6.0], [5.0, 5.0, 2.0], [8.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLLL(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 6.0], [8.0, 3.0, 7.0], [2.0, 6.0, 1.0]], [[6.0, 1.0, 4.0], [1.0, 1.0, 1.0], [1.0, 4.0, 
6.0]], [[6.0, 1.0, 5.0], [7.0, 4.0, 2.0], [6.0, 1.0, 4.0]]], [[[6.0, 5.0, 5.0], [6.0, 4.0, 5.0], [3.0, 4.0, 3.0]], [[7.0, 8.0, 
2.0], [2.0, 3.0, 2.0], [6.0, 2.0, 2.0]], [[8.0, 8.0, 1.0], [8.0, 1.0, 4.0], [3.0, 4.0, 3.0]]], [[[1.0, 4.0, 3.0], [3.0, 8.0, 
8.0], [3.0, 4.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 2.0, 7.0], [7.0, 2.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 8.0, 8.0], [4.0, 4.0, 
6.0]]], [[[7.0, 7.0, 4.0], [7.0, 5.0, 5.0], [4.0, 6.0, 7.0]], [[7.0, 2.0, 6.0], [6.0, 7.0, 3.0], [2.0, 6.0, 8.0]], [[4.0, 5.0, 
4.0], [3.0, 7.0, 5.0], [6.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 5.0], [6.0, 2.0, 5.0], [7.0, 5.0, 8.0]], [[2.0, 7.0, 5.0], [2.0, 1.0, 
3.0], [5.0, 3.0, 3.0]], [[2.0, 4.0, 2.0], [1.0, 4.0, 5.0], [4.0, 1.0, 4.0]]], [[[4.0, 4.0, 1.0], [8.0, 8.0, 4.0], [1.0, 6.0, 
3.0]], [[1.0, 5.0, 5.0], [7.0, 2.0, 8.0], [2.0, 4.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 3.0, 8.0], [1.0, 8.0, 3.0]]], [[[5.0, 5.0, 
8.0], [8.0, 6.0, 6.0], [1.0, 4.0, 2.0]], [[3.0, 1.0, 7.0], [4.0, 8.0, 3.0], [6.0, 5.0, 3.0]], [[5.0, 2.0, 5.0], [4.0, 3.0, 
7.0], [4.0, 4.0, 7.0]]], [[[1.0, 7.0, 7.0], [8.0, 6.0, 1.0], [5.0, 4.0, 6.0]], [[3.0, 8.0, 7.0], [8.0, 7.0, 3.0], [8.0, 4.0, 
1.0]], [[1.0, 7.0, 4.0], [3.0, 6.0, 4.0], [8.0, 3.0, 4.0]]]]))
      val=Data(numpy.array([[[[1.0]]], [[[3.0]]]]),self.functionspace)
      arg[2:,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 6.0], [8.0, 3.0, 7.0], [2.0, 6.0, 1.0]], [[6.0, 1.0, 4.0], [1.0, 1.0, 1.0], [1.0, 
4.0, 6.0]], [[6.0, 1.0, 5.0], [7.0, 4.0, 2.0], [6.0, 1.0, 4.0]]], [[[6.0, 5.0, 5.0], [6.0, 4.0, 5.0], [3.0, 4.0, 3.0]], [[7.0, 
8.0, 2.0], [2.0, 3.0, 2.0], [6.0, 2.0, 2.0]], [[8.0, 8.0, 1.0], [8.0, 1.0, 4.0], [3.0, 4.0, 3.0]]], [[[1.0, 4.0, 3.0], [3.0, 
8.0, 8.0], [3.0, 4.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 2.0, 7.0], [7.0, 2.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 8.0, 8.0], [4.0, 4.0, 
1.0]]], [[[7.0, 7.0, 4.0], [7.0, 5.0, 5.0], [4.0, 6.0, 7.0]], [[7.0, 2.0, 6.0], [6.0, 7.0, 3.0], [2.0, 6.0, 8.0]], [[4.0, 5.0, 
4.0], [3.0, 7.0, 5.0], [6.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 5.0], [6.0, 2.0, 5.0], [7.0, 5.0, 8.0]], 
[[2.0, 7.0, 5.0], [2.0, 1.0, 3.0], [5.0, 3.0, 3.0]], [[2.0, 4.0, 2.0], [1.0, 4.0, 5.0], [4.0, 1.0, 4.0]]], [[[4.0, 4.0, 1.0], 
[8.0, 8.0, 4.0], [1.0, 6.0, 3.0]], [[1.0, 5.0, 5.0], [7.0, 2.0, 8.0], [2.0, 4.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 3.0, 8.0], 
[1.0, 8.0, 3.0]]], [[[5.0, 5.0, 8.0], [8.0, 6.0, 6.0], [1.0, 4.0, 2.0]], [[3.0, 1.0, 7.0], [4.0, 8.0, 3.0], [6.0, 5.0, 3.0]], 
[[5.0, 2.0, 5.0], [4.0, 3.0, 7.0], [4.0, 4.0, 1.0]]], [[[1.0, 7.0, 7.0], [8.0, 6.0, 1.0], [5.0, 4.0, 6.0]], [[3.0, 8.0, 7.0], 
[8.0, 7.0, 3.0], [8.0, 4.0, 1.0]], [[1.0, 7.0, 4.0], [3.0, 6.0, 4.0], [8.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLLL(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 2.0], [5.0, 7.0, 8.0], [4.0, 6.0, 6.0]], [[4.0, 7.0, 4.0], [2.0, 2.0, 5.0], [4.0, 7.0, 
6.0]], [[5.0, 1.0, 5.0], [4.0, 5.0, 3.0], [1.0, 4.0, 8.0]]], [[[4.0, 8.0, 2.0], [3.0, 8.0, 1.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 
6.0], [2.0, 1.0, 1.0], [3.0, 4.0, 5.0]], [[2.0, 3.0, 6.0], [2.0, 2.0, 3.0], [2.0, 5.0, 6.0]]], [[[5.0, 5.0, 8.0], [6.0, 1.0, 
6.0], [6.0, 3.0, 8.0]], [[2.0, 2.0, 1.0], [3.0, 8.0, 1.0], [6.0, 6.0, 7.0]], [[4.0, 5.0, 6.0], [8.0, 8.0, 6.0], [1.0, 4.0, 
3.0]]], [[[6.0, 7.0, 8.0], [2.0, 5.0, 7.0], [8.0, 5.0, 8.0]], [[3.0, 6.0, 3.0], [7.0, 2.0, 6.0], [7.0, 2.0, 3.0]], [[7.0, 5.0, 
7.0], [7.0, 1.0, 5.0], [3.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 6.0], [2.0, 3.0, 3.0], [2.0, 8.0, 4.0]], [[1.0, 8.0, 1.0], [4.0, 1.0, 
2.0], [5.0, 8.0, 4.0]], [[5.0, 8.0, 3.0], [8.0, 4.0, 8.0], [8.0, 6.0, 2.0]]], [[[4.0, 5.0, 7.0], [6.0, 3.0, 3.0], [4.0, 7.0, 
8.0]], [[7.0, 4.0, 4.0], [4.0, 6.0, 1.0], [4.0, 5.0, 3.0]], [[7.0, 3.0, 2.0], [8.0, 1.0, 7.0], [4.0, 1.0, 6.0]]], [[[7.0, 3.0, 
8.0], [7.0, 4.0, 5.0], [4.0, 3.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 1.0, 2.0], [5.0, 6.0, 1.0]], [[6.0, 6.0, 5.0], [6.0, 2.0, 
1.0], [4.0, 2.0, 1.0]]], [[[3.0, 2.0, 4.0], [4.0, 7.0, 1.0], [4.0, 2.0, 2.0]], [[8.0, 8.0, 3.0], [5.0, 8.0, 8.0], [3.0, 7.0, 
3.0]], [[7.0, 1.0, 5.0], [3.0, 2.0, 1.0], [4.0, 5.0, 4.0]]]]))
      val=Data(numpy.array([[[[7.0]]], [[[7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0]]], [[[7.0]]]]))
      arg[2:,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 2.0], [5.0, 7.0, 8.0], [4.0, 6.0, 6.0]], [[4.0, 7.0, 4.0], [2.0, 2.0, 5.0], [4.0, 
7.0, 6.0]], [[5.0, 1.0, 5.0], [4.0, 5.0, 3.0], [1.0, 4.0, 8.0]]], [[[4.0, 8.0, 2.0], [3.0, 8.0, 1.0], [5.0, 5.0, 8.0]], [[4.0, 
4.0, 6.0], [2.0, 1.0, 1.0], [3.0, 4.0, 5.0]], [[2.0, 3.0, 6.0], [2.0, 2.0, 3.0], [2.0, 5.0, 6.0]]], [[[5.0, 5.0, 8.0], [6.0, 
1.0, 6.0], [6.0, 3.0, 8.0]], [[2.0, 2.0, 1.0], [3.0, 8.0, 1.0], [6.0, 6.0, 7.0]], [[4.0, 5.0, 6.0], [8.0, 8.0, 6.0], [1.0, 4.0, 
7.0]]], [[[6.0, 7.0, 8.0], [2.0, 5.0, 7.0], [8.0, 5.0, 8.0]], [[3.0, 6.0, 3.0], [7.0, 2.0, 6.0], [7.0, 2.0, 3.0]], [[7.0, 5.0, 
7.0], [7.0, 1.0, 5.0], [3.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 6.0], [2.0, 3.0, 3.0], [2.0, 8.0, 4.0]], 
[[1.0, 8.0, 1.0], [4.0, 1.0, 2.0], [5.0, 8.0, 4.0]], [[5.0, 8.0, 3.0], [8.0, 4.0, 8.0], [8.0, 6.0, 2.0]]], [[[4.0, 5.0, 7.0], 
[6.0, 3.0, 3.0], [4.0, 7.0, 8.0]], [[7.0, 4.0, 4.0], [4.0, 6.0, 1.0], [4.0, 5.0, 3.0]], [[7.0, 3.0, 2.0], [8.0, 1.0, 7.0], 
[4.0, 1.0, 6.0]]], [[[7.0, 3.0, 8.0], [7.0, 4.0, 5.0], [4.0, 3.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 1.0, 2.0], [5.0, 6.0, 1.0]], 
[[6.0, 6.0, 5.0], [6.0, 2.0, 1.0], [4.0, 2.0, 5.0]]], [[[3.0, 2.0, 4.0], [4.0, 7.0, 1.0], [4.0, 2.0, 2.0]], [[8.0, 8.0, 3.0], 
[5.0, 8.0, 8.0], [3.0, 7.0, 3.0]], [[7.0, 1.0, 5.0], [3.0, 2.0, 1.0], [4.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLLL(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 7.0], [1.0, 8.0, 7.0], [3.0, 3.0, 5.0]], [[4.0, 2.0, 8.0], [5.0, 1.0, 3.0], [2.0, 3.0, 
3.0]], [[5.0, 2.0, 1.0], [6.0, 1.0, 5.0], [3.0, 7.0, 1.0]]], [[[7.0, 1.0, 3.0], [2.0, 7.0, 6.0], [3.0, 5.0, 7.0]], [[2.0, 2.0, 
7.0], [6.0, 4.0, 8.0], [6.0, 8.0, 3.0]], [[8.0, 7.0, 3.0], [7.0, 3.0, 1.0], [7.0, 3.0, 6.0]]], [[[3.0, 7.0, 6.0], [3.0, 8.0, 
1.0], [3.0, 4.0, 4.0]], [[3.0, 6.0, 1.0], [1.0, 7.0, 8.0], [5.0, 1.0, 2.0]], [[4.0, 1.0, 4.0], [8.0, 5.0, 7.0], [5.0, 5.0, 
1.0]]], [[[3.0, 8.0, 5.0], [4.0, 2.0, 8.0], [5.0, 7.0, 3.0]], [[5.0, 5.0, 6.0], [6.0, 3.0, 7.0], [2.0, 2.0, 5.0]], [[7.0, 5.0, 
1.0], [1.0, 3.0, 6.0], [8.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 7.0], [8.0, 7.0, 8.0], [2.0, 8.0, 2.0]], [[7.0, 3.0, 4.0], [6.0, 8.0, 
8.0], [4.0, 2.0, 3.0]], [[2.0, 3.0, 6.0], [8.0, 1.0, 2.0], [6.0, 1.0, 4.0]]], [[[7.0, 2.0, 8.0], [3.0, 1.0, 6.0], [5.0, 8.0, 
4.0]], [[7.0, 3.0, 3.0], [8.0, 4.0, 4.0], [1.0, 2.0, 4.0]], [[5.0, 1.0, 3.0], [1.0, 6.0, 6.0], [6.0, 8.0, 7.0]]], [[[2.0, 7.0, 
1.0], [2.0, 6.0, 3.0], [7.0, 1.0, 6.0]], [[3.0, 4.0, 4.0], [7.0, 4.0, 6.0], [1.0, 7.0, 4.0]], [[1.0, 5.0, 5.0], [8.0, 7.0, 
3.0], [4.0, 6.0, 5.0]]], [[[3.0, 7.0, 8.0], [2.0, 6.0, 2.0], [1.0, 3.0, 2.0]], [[3.0, 6.0, 4.0], [7.0, 5.0, 5.0], [6.0, 1.0, 
6.0]], [[7.0, 8.0, 6.0], [5.0, 7.0, 5.0], [1.0, 4.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0]]], [[[1.0]]]])+(1.-msk_val)*numpy.array([[[[2.0]]], [[[2.0]]]])
      arg[2:,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 7.0], [1.0, 8.0, 7.0], [3.0, 3.0, 5.0]], [[4.0, 2.0, 8.0], [5.0, 1.0, 3.0], [2.0, 
3.0, 3.0]], [[5.0, 2.0, 1.0], [6.0, 1.0, 5.0], [3.0, 7.0, 1.0]]], [[[7.0, 1.0, 3.0], [2.0, 7.0, 6.0], [3.0, 5.0, 7.0]], [[2.0, 
2.0, 7.0], [6.0, 4.0, 8.0], [6.0, 8.0, 3.0]], [[8.0, 7.0, 3.0], [7.0, 3.0, 1.0], [7.0, 3.0, 6.0]]], [[[3.0, 7.0, 6.0], [3.0, 
8.0, 1.0], [3.0, 4.0, 4.0]], [[3.0, 6.0, 1.0], [1.0, 7.0, 8.0], [5.0, 1.0, 2.0]], [[4.0, 1.0, 4.0], [8.0, 5.0, 7.0], [5.0, 5.0, 
4.0]]], [[[3.0, 8.0, 5.0], [4.0, 2.0, 8.0], [5.0, 7.0, 3.0]], [[5.0, 5.0, 6.0], [6.0, 3.0, 7.0], [2.0, 2.0, 5.0]], [[7.0, 5.0, 
1.0], [1.0, 3.0, 6.0], [8.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 7.0], [8.0, 7.0, 8.0], [2.0, 8.0, 2.0]], 
[[7.0, 3.0, 4.0], [6.0, 8.0, 8.0], [4.0, 2.0, 3.0]], [[2.0, 3.0, 6.0], [8.0, 1.0, 2.0], [6.0, 1.0, 4.0]]], [[[7.0, 2.0, 8.0], 
[3.0, 1.0, 6.0], [5.0, 8.0, 4.0]], [[7.0, 3.0, 3.0], [8.0, 4.0, 4.0], [1.0, 2.0, 4.0]], [[5.0, 1.0, 3.0], [1.0, 6.0, 6.0], 
[6.0, 8.0, 7.0]]], [[[2.0, 7.0, 1.0], [2.0, 6.0, 3.0], [7.0, 1.0, 6.0]], [[3.0, 4.0, 4.0], [7.0, 4.0, 6.0], [1.0, 7.0, 4.0]], 
[[1.0, 5.0, 5.0], [8.0, 7.0, 3.0], [4.0, 6.0, 2.0]]], [[[3.0, 7.0, 8.0], [2.0, 6.0, 2.0], [1.0, 3.0, 2.0]], [[3.0, 6.0, 4.0], 
[7.0, 5.0, 5.0], [6.0, 1.0, 6.0]], [[7.0, 8.0, 6.0], [5.0, 7.0, 5.0], [1.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PRPN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 5.0], [3.0, 5.0, 7.0], [5.0, 5.0, 8.0]], [[7.0, 8.0, 7.0], [3.0, 1.0, 3.0], [8.0, 8.0, 
5.0]], [[2.0, 4.0, 2.0], [2.0, 8.0, 6.0], [2.0, 5.0, 1.0]]], [[[7.0, 7.0, 1.0], [5.0, 4.0, 1.0], [3.0, 1.0, 6.0]], [[1.0, 2.0, 
4.0], [5.0, 5.0, 1.0], [5.0, 3.0, 2.0]], [[8.0, 8.0, 7.0], [8.0, 2.0, 3.0], [3.0, 2.0, 3.0]]], [[[6.0, 3.0, 1.0], [8.0, 3.0, 
1.0], [4.0, 3.0, 1.0]], [[5.0, 7.0, 3.0], [4.0, 3.0, 8.0], [7.0, 6.0, 7.0]], [[7.0, 7.0, 4.0], [5.0, 2.0, 7.0], [1.0, 3.0, 
7.0]]], [[[6.0, 4.0, 6.0], [6.0, 5.0, 8.0], [6.0, 3.0, 1.0]], [[5.0, 6.0, 5.0], [8.0, 7.0, 6.0], [5.0, 6.0, 4.0]], [[5.0, 6.0, 
5.0], [4.0, 3.0, 6.0], [7.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 6.0], [3.0, 4.0, 3.0], [8.0, 7.0, 5.0]], [[3.0, 3.0, 3.0], [8.0, 4.0, 
2.0], [8.0, 3.0, 2.0]], [[6.0, 1.0, 1.0], [8.0, 2.0, 6.0], [1.0, 5.0, 5.0]]], [[[7.0, 4.0, 4.0], [8.0, 4.0, 6.0], [8.0, 8.0, 
6.0]], [[4.0, 8.0, 3.0], [2.0, 8.0, 2.0], [1.0, 8.0, 5.0]], [[7.0, 3.0, 4.0], [3.0, 3.0, 4.0], [8.0, 1.0, 2.0]]], [[[5.0, 7.0, 
2.0], [6.0, 7.0, 5.0], [7.0, 8.0, 6.0]], [[2.0, 4.0, 2.0], [3.0, 2.0, 2.0], [1.0, 3.0, 4.0]], [[1.0, 7.0, 1.0], [1.0, 2.0, 
4.0], [3.0, 5.0, 1.0]]], [[[7.0, 8.0, 2.0], [3.0, 3.0, 2.0], [5.0, 5.0, 2.0]], [[1.0, 8.0, 7.0], [1.0, 8.0, 1.0], [7.0, 8.0, 
4.0]], [[6.0, 4.0, 2.0], [4.0, 6.0, 7.0], [7.0, 2.0, 8.0]]]]))
      val=numpy.array([[1.0, 6.0, 5.0]])
      arg[2,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 5.0], [3.0, 5.0, 7.0], [5.0, 5.0, 8.0]], [[7.0, 8.0, 7.0], [3.0, 1.0, 3.0], [8.0, 
8.0, 5.0]], [[2.0, 4.0, 2.0], [2.0, 8.0, 6.0], [2.0, 5.0, 1.0]]], [[[7.0, 7.0, 1.0], [5.0, 4.0, 1.0], [3.0, 1.0, 6.0]], [[1.0, 
2.0, 4.0], [5.0, 5.0, 1.0], [5.0, 3.0, 2.0]], [[8.0, 8.0, 7.0], [8.0, 2.0, 3.0], [3.0, 2.0, 3.0]]], [[[6.0, 3.0, 1.0], [8.0, 
3.0, 1.0], [4.0, 3.0, 1.0]], [[5.0, 7.0, 3.0], [1.0, 6.0, 5.0], [7.0, 6.0, 7.0]], [[7.0, 7.0, 4.0], [5.0, 2.0, 7.0], [1.0, 3.0, 
7.0]]], [[[6.0, 4.0, 6.0], [6.0, 5.0, 8.0], [6.0, 3.0, 1.0]], [[5.0, 6.0, 5.0], [8.0, 7.0, 6.0], [5.0, 6.0, 4.0]], [[5.0, 6.0, 
5.0], [4.0, 3.0, 6.0], [7.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 6.0], [3.0, 4.0, 3.0], [8.0, 7.0, 5.0]], 
[[3.0, 3.0, 3.0], [8.0, 4.0, 2.0], [8.0, 3.0, 2.0]], [[6.0, 1.0, 1.0], [8.0, 2.0, 6.0], [1.0, 5.0, 5.0]]], [[[7.0, 4.0, 4.0], 
[8.0, 4.0, 6.0], [8.0, 8.0, 6.0]], [[4.0, 8.0, 3.0], [2.0, 8.0, 2.0], [1.0, 8.0, 5.0]], [[7.0, 3.0, 4.0], [3.0, 3.0, 4.0], 
[8.0, 1.0, 2.0]]], [[[5.0, 7.0, 2.0], [6.0, 7.0, 5.0], [7.0, 8.0, 6.0]], [[2.0, 4.0, 2.0], [1.0, 6.0, 5.0], [1.0, 3.0, 4.0]], 
[[1.0, 7.0, 1.0], [1.0, 2.0, 4.0], [3.0, 5.0, 1.0]]], [[[7.0, 8.0, 2.0], [3.0, 3.0, 2.0], [5.0, 5.0, 2.0]], [[1.0, 8.0, 7.0], 
[1.0, 8.0, 1.0], [7.0, 8.0, 4.0]], [[6.0, 4.0, 2.0], [4.0, 6.0, 7.0], [7.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PRPN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 4.0], [7.0, 6.0, 5.0], [4.0, 8.0, 6.0]], [[6.0, 4.0, 3.0], [4.0, 8.0, 2.0], [4.0, 6.0, 
1.0]], [[2.0, 1.0, 8.0], [6.0, 2.0, 2.0], [8.0, 3.0, 5.0]]], [[[5.0, 6.0, 8.0], [6.0, 5.0, 5.0], [2.0, 3.0, 4.0]], [[1.0, 1.0, 
6.0], [2.0, 6.0, 6.0], [2.0, 8.0, 4.0]], [[8.0, 7.0, 7.0], [7.0, 4.0, 2.0], [2.0, 6.0, 4.0]]], [[[8.0, 6.0, 3.0], [4.0, 3.0, 
4.0], [5.0, 7.0, 4.0]], [[4.0, 7.0, 3.0], [5.0, 6.0, 7.0], [1.0, 5.0, 2.0]], [[5.0, 6.0, 7.0], [4.0, 2.0, 6.0], [1.0, 8.0, 
2.0]]], [[[7.0, 2.0, 2.0], [3.0, 7.0, 5.0], [4.0, 7.0, 6.0]], [[2.0, 4.0, 4.0], [5.0, 5.0, 7.0], [2.0, 3.0, 7.0]], [[5.0, 1.0, 
1.0], [4.0, 2.0, 5.0], [6.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 3.0], [4.0, 3.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 6.0, 2.0], [8.0, 7.0, 
3.0], [5.0, 6.0, 2.0]], [[4.0, 8.0, 6.0], [2.0, 8.0, 2.0], [6.0, 2.0, 7.0]]], [[[7.0, 5.0, 6.0], [6.0, 5.0, 5.0], [7.0, 5.0, 
5.0]], [[8.0, 4.0, 5.0], [8.0, 5.0, 6.0], [4.0, 4.0, 3.0]], [[1.0, 1.0, 3.0], [6.0, 4.0, 8.0], [8.0, 6.0, 7.0]]], [[[2.0, 5.0, 
6.0], [1.0, 3.0, 5.0], [1.0, 6.0, 4.0]], [[8.0, 2.0, 2.0], [1.0, 3.0, 6.0], [5.0, 3.0, 8.0]], [[5.0, 4.0, 8.0], [1.0, 7.0, 
8.0], [3.0, 5.0, 4.0]]], [[[2.0, 2.0, 7.0], [7.0, 4.0, 5.0], [4.0, 6.0, 8.0]], [[7.0, 1.0, 3.0], [7.0, 4.0, 3.0], [1.0, 5.0, 
5.0]], [[6.0, 6.0, 4.0], [1.0, 6.0, 6.0], [2.0, 7.0, 5.0]]]]))
      val=Data(numpy.array([[6.0, 2.0, 3.0]]),self.functionspace)
      arg[2,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 4.0], [7.0, 6.0, 5.0], [4.0, 8.0, 6.0]], [[6.0, 4.0, 3.0], [4.0, 8.0, 2.0], [4.0, 
6.0, 1.0]], [[2.0, 1.0, 8.0], [6.0, 2.0, 2.0], [8.0, 3.0, 5.0]]], [[[5.0, 6.0, 8.0], [6.0, 5.0, 5.0], [2.0, 3.0, 4.0]], [[1.0, 
1.0, 6.0], [2.0, 6.0, 6.0], [2.0, 8.0, 4.0]], [[8.0, 7.0, 7.0], [7.0, 4.0, 2.0], [2.0, 6.0, 4.0]]], [[[8.0, 6.0, 3.0], [4.0, 
3.0, 4.0], [5.0, 7.0, 4.0]], [[4.0, 7.0, 3.0], [6.0, 2.0, 3.0], [1.0, 5.0, 2.0]], [[5.0, 6.0, 7.0], [4.0, 2.0, 6.0], [1.0, 8.0, 
2.0]]], [[[7.0, 2.0, 2.0], [3.0, 7.0, 5.0], [4.0, 7.0, 6.0]], [[2.0, 4.0, 4.0], [5.0, 5.0, 7.0], [2.0, 3.0, 7.0]], [[5.0, 1.0, 
1.0], [4.0, 2.0, 5.0], [6.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 3.0], [4.0, 3.0, 3.0], [2.0, 4.0, 1.0]], 
[[3.0, 6.0, 2.0], [8.0, 7.0, 3.0], [5.0, 6.0, 2.0]], [[4.0, 8.0, 6.0], [2.0, 8.0, 2.0], [6.0, 2.0, 7.0]]], [[[7.0, 5.0, 6.0], 
[6.0, 5.0, 5.0], [7.0, 5.0, 5.0]], [[8.0, 4.0, 5.0], [8.0, 5.0, 6.0], [4.0, 4.0, 3.0]], [[1.0, 1.0, 3.0], [6.0, 4.0, 8.0], 
[8.0, 6.0, 7.0]]], [[[2.0, 5.0, 6.0], [1.0, 3.0, 5.0], [1.0, 6.0, 4.0]], [[8.0, 2.0, 2.0], [6.0, 2.0, 3.0], [5.0, 3.0, 8.0]], 
[[5.0, 4.0, 8.0], [1.0, 7.0, 8.0], [3.0, 5.0, 4.0]]], [[[2.0, 2.0, 7.0], [7.0, 4.0, 5.0], [4.0, 6.0, 8.0]], [[7.0, 1.0, 3.0], 
[7.0, 4.0, 3.0], [1.0, 5.0, 5.0]], [[6.0, 6.0, 4.0], [1.0, 6.0, 6.0], [2.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PRPN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 5.0], [6.0, 1.0, 7.0], [1.0, 4.0, 4.0]], [[1.0, 6.0, 8.0], [3.0, 7.0, 8.0], [8.0, 1.0, 
6.0]], [[1.0, 3.0, 1.0], [7.0, 1.0, 5.0], [8.0, 2.0, 5.0]]], [[[2.0, 3.0, 1.0], [4.0, 4.0, 8.0], [7.0, 6.0, 1.0]], [[4.0, 2.0, 
5.0], [1.0, 2.0, 6.0], [5.0, 5.0, 7.0]], [[5.0, 3.0, 2.0], [7.0, 7.0, 8.0], [8.0, 7.0, 3.0]]], [[[5.0, 3.0, 6.0], [2.0, 4.0, 
6.0], [6.0, 1.0, 4.0]], [[4.0, 8.0, 7.0], [5.0, 1.0, 6.0], [4.0, 8.0, 7.0]], [[8.0, 4.0, 7.0], [1.0, 3.0, 2.0], [8.0, 1.0, 
7.0]]], [[[6.0, 8.0, 8.0], [4.0, 5.0, 8.0], [8.0, 8.0, 5.0]], [[8.0, 3.0, 8.0], [4.0, 1.0, 7.0], [6.0, 5.0, 8.0]], [[4.0, 3.0, 
6.0], [7.0, 5.0, 4.0], [4.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 7.0], [2.0, 5.0, 3.0], [5.0, 8.0, 4.0]], [[4.0, 3.0, 3.0], [7.0, 8.0, 
4.0], [3.0, 2.0, 3.0]], [[2.0, 7.0, 2.0], [7.0, 7.0, 3.0], [3.0, 1.0, 7.0]]], [[[5.0, 8.0, 3.0], [7.0, 8.0, 2.0], [5.0, 7.0, 
5.0]], [[5.0, 8.0, 3.0], [6.0, 6.0, 5.0], [6.0, 1.0, 1.0]], [[3.0, 2.0, 8.0], [5.0, 7.0, 4.0], [4.0, 2.0, 8.0]]], [[[2.0, 6.0, 
5.0], [1.0, 8.0, 4.0], [5.0, 7.0, 1.0]], [[8.0, 4.0, 3.0], [4.0, 2.0, 5.0], [3.0, 2.0, 7.0]], [[8.0, 5.0, 4.0], [2.0, 4.0, 
3.0], [3.0, 7.0, 5.0]]], [[[6.0, 2.0, 4.0], [8.0, 2.0, 5.0], [7.0, 1.0, 2.0]], [[8.0, 2.0, 6.0], [4.0, 3.0, 3.0], [2.0, 6.0, 
1.0]], [[3.0, 3.0, 8.0], [5.0, 4.0, 5.0], [4.0, 6.0, 5.0]]]]))
      val=Data(numpy.array([[7.0, 8.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 3.0, 8.0]]))
      arg[2,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 5.0], [6.0, 1.0, 7.0], [1.0, 4.0, 4.0]], [[1.0, 6.0, 8.0], [3.0, 7.0, 8.0], [8.0, 
1.0, 6.0]], [[1.0, 3.0, 1.0], [7.0, 1.0, 5.0], [8.0, 2.0, 5.0]]], [[[2.0, 3.0, 1.0], [4.0, 4.0, 8.0], [7.0, 6.0, 1.0]], [[4.0, 
2.0, 5.0], [1.0, 2.0, 6.0], [5.0, 5.0, 7.0]], [[5.0, 3.0, 2.0], [7.0, 7.0, 8.0], [8.0, 7.0, 3.0]]], [[[5.0, 3.0, 6.0], [2.0, 
4.0, 6.0], [6.0, 1.0, 4.0]], [[4.0, 8.0, 7.0], [7.0, 8.0, 1.0], [4.0, 8.0, 7.0]], [[8.0, 4.0, 7.0], [1.0, 3.0, 2.0], [8.0, 1.0, 
7.0]]], [[[6.0, 8.0, 8.0], [4.0, 5.0, 8.0], [8.0, 8.0, 5.0]], [[8.0, 3.0, 8.0], [4.0, 1.0, 7.0], [6.0, 5.0, 8.0]], [[4.0, 3.0, 
6.0], [7.0, 5.0, 4.0], [4.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 7.0], [2.0, 5.0, 3.0], [5.0, 8.0, 4.0]], 
[[4.0, 3.0, 3.0], [7.0, 8.0, 4.0], [3.0, 2.0, 3.0]], [[2.0, 7.0, 2.0], [7.0, 7.0, 3.0], [3.0, 1.0, 7.0]]], [[[5.0, 8.0, 3.0], 
[7.0, 8.0, 2.0], [5.0, 7.0, 5.0]], [[5.0, 8.0, 3.0], [6.0, 6.0, 5.0], [6.0, 1.0, 1.0]], [[3.0, 2.0, 8.0], [5.0, 7.0, 4.0], 
[4.0, 2.0, 8.0]]], [[[2.0, 6.0, 5.0], [1.0, 8.0, 4.0], [5.0, 7.0, 1.0]], [[8.0, 4.0, 3.0], [6.0, 3.0, 8.0], [3.0, 2.0, 7.0]], 
[[8.0, 5.0, 4.0], [2.0, 4.0, 3.0], [3.0, 7.0, 5.0]]], [[[6.0, 2.0, 4.0], [8.0, 2.0, 5.0], [7.0, 1.0, 2.0]], [[8.0, 2.0, 6.0], 
[4.0, 3.0, 3.0], [2.0, 6.0, 1.0]], [[3.0, 3.0, 8.0], [5.0, 4.0, 5.0], [4.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PRPN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 3.0], [2.0, 6.0, 6.0], [2.0, 6.0, 1.0]], [[5.0, 4.0, 2.0], [1.0, 6.0, 7.0], [6.0, 3.0, 
7.0]], [[8.0, 6.0, 1.0], [8.0, 7.0, 6.0], [4.0, 7.0, 7.0]]], [[[1.0, 5.0, 6.0], [8.0, 4.0, 5.0], [8.0, 4.0, 4.0]], [[1.0, 1.0, 
6.0], [6.0, 4.0, 3.0], [2.0, 4.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 2.0, 7.0], [3.0, 5.0, 3.0]]], [[[5.0, 7.0, 1.0], [7.0, 2.0, 
8.0], [6.0, 1.0, 4.0]], [[7.0, 1.0, 4.0], [2.0, 2.0, 7.0], [1.0, 1.0, 2.0]], [[3.0, 6.0, 7.0], [2.0, 5.0, 3.0], [5.0, 4.0, 
4.0]]], [[[2.0, 4.0, 6.0], [3.0, 1.0, 5.0], [5.0, 6.0, 8.0]], [[2.0, 6.0, 4.0], [4.0, 3.0, 6.0], [2.0, 4.0, 4.0]], [[4.0, 1.0, 
5.0], [6.0, 2.0, 8.0], [1.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 7.0], [1.0, 5.0, 3.0], [7.0, 6.0, 1.0]], [[5.0, 6.0, 6.0], [7.0, 6.0, 
5.0], [4.0, 6.0, 6.0]], [[7.0, 1.0, 1.0], [1.0, 7.0, 6.0], [7.0, 8.0, 5.0]]], [[[6.0, 3.0, 7.0], [5.0, 6.0, 4.0], [2.0, 6.0, 
4.0]], [[5.0, 3.0, 6.0], [8.0, 1.0, 5.0], [3.0, 5.0, 3.0]], [[2.0, 5.0, 2.0], [4.0, 7.0, 8.0], [6.0, 2.0, 3.0]]], [[[8.0, 8.0, 
4.0], [5.0, 8.0, 5.0], [1.0, 8.0, 1.0]], [[6.0, 2.0, 4.0], [6.0, 4.0, 4.0], [5.0, 8.0, 6.0]], [[6.0, 4.0, 1.0], [2.0, 4.0, 
4.0], [6.0, 5.0, 8.0]]], [[[8.0, 6.0, 4.0], [8.0, 2.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 3.0, 4.0], [3.0, 1.0, 2.0], [6.0, 2.0, 
2.0]], [[8.0, 2.0, 8.0], [5.0, 4.0, 3.0], [4.0, 7.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[4.0, 8.0, 1.0]])+(1.-msk_val)*numpy.array([[8.0, 5.0, 4.0]])
      arg[2,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 3.0], [2.0, 6.0, 6.0], [2.0, 6.0, 1.0]], [[5.0, 4.0, 2.0], [1.0, 6.0, 7.0], [6.0, 
3.0, 7.0]], [[8.0, 6.0, 1.0], [8.0, 7.0, 6.0], [4.0, 7.0, 7.0]]], [[[1.0, 5.0, 6.0], [8.0, 4.0, 5.0], [8.0, 4.0, 4.0]], [[1.0, 
1.0, 6.0], [6.0, 4.0, 3.0], [2.0, 4.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 2.0, 7.0], [3.0, 5.0, 3.0]]], [[[5.0, 7.0, 1.0], [7.0, 
2.0, 8.0], [6.0, 1.0, 4.0]], [[7.0, 1.0, 4.0], [4.0, 8.0, 1.0], [1.0, 1.0, 2.0]], [[3.0, 6.0, 7.0], [2.0, 5.0, 3.0], [5.0, 4.0, 
4.0]]], [[[2.0, 4.0, 6.0], [3.0, 1.0, 5.0], [5.0, 6.0, 8.0]], [[2.0, 6.0, 4.0], [4.0, 3.0, 6.0], [2.0, 4.0, 4.0]], [[4.0, 1.0, 
5.0], [6.0, 2.0, 8.0], [1.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 7.0], [1.0, 5.0, 3.0], [7.0, 6.0, 1.0]], 
[[5.0, 6.0, 6.0], [7.0, 6.0, 5.0], [4.0, 6.0, 6.0]], [[7.0, 1.0, 1.0], [1.0, 7.0, 6.0], [7.0, 8.0, 5.0]]], [[[6.0, 3.0, 7.0], 
[5.0, 6.0, 4.0], [2.0, 6.0, 4.0]], [[5.0, 3.0, 6.0], [8.0, 1.0, 5.0], [3.0, 5.0, 3.0]], [[2.0, 5.0, 2.0], [4.0, 7.0, 8.0], 
[6.0, 2.0, 3.0]]], [[[8.0, 8.0, 4.0], [5.0, 8.0, 5.0], [1.0, 8.0, 1.0]], [[6.0, 2.0, 4.0], [8.0, 5.0, 4.0], [5.0, 8.0, 6.0]], 
[[6.0, 4.0, 1.0], [2.0, 4.0, 4.0], [6.0, 5.0, 8.0]]], [[[8.0, 6.0, 4.0], [8.0, 2.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 3.0, 4.0], 
[3.0, 1.0, 2.0], [6.0, 2.0, 2.0]], [[8.0, 2.0, 8.0], [5.0, 4.0, 3.0], [4.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPRN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 7.0], [6.0, 2.0, 7.0], [7.0, 2.0, 7.0]], [[6.0, 5.0, 3.0], [8.0, 6.0, 8.0], [6.0, 5.0, 
1.0]], [[6.0, 8.0, 4.0], [8.0, 4.0, 3.0], [2.0, 2.0, 8.0]]], [[[7.0, 6.0, 7.0], [2.0, 2.0, 2.0], [3.0, 8.0, 5.0]], [[5.0, 1.0, 
5.0], [5.0, 6.0, 4.0], [4.0, 8.0, 7.0]], [[3.0, 4.0, 1.0], [6.0, 6.0, 8.0], [2.0, 7.0, 4.0]]], [[[6.0, 7.0, 5.0], [7.0, 3.0, 
8.0], [5.0, 6.0, 1.0]], [[7.0, 2.0, 7.0], [1.0, 5.0, 1.0], [4.0, 6.0, 8.0]], [[8.0, 8.0, 7.0], [3.0, 7.0, 2.0], [4.0, 1.0, 
6.0]]], [[[8.0, 3.0, 5.0], [5.0, 2.0, 5.0], [2.0, 8.0, 1.0]], [[3.0, 8.0, 7.0], [1.0, 1.0, 7.0], [7.0, 1.0, 1.0]], [[5.0, 6.0, 
4.0], [7.0, 4.0, 5.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 5.0], [5.0, 7.0, 8.0], [6.0, 4.0, 6.0]], [[8.0, 8.0, 1.0], [3.0, 1.0, 
7.0], [6.0, 5.0, 1.0]], [[5.0, 7.0, 6.0], [2.0, 6.0, 1.0], [3.0, 2.0, 4.0]]], [[[7.0, 6.0, 4.0], [1.0, 8.0, 4.0], [4.0, 7.0, 
3.0]], [[3.0, 6.0, 7.0], [6.0, 3.0, 3.0], [8.0, 6.0, 8.0]], [[5.0, 5.0, 2.0], [3.0, 5.0, 4.0], [5.0, 3.0, 4.0]]], [[[3.0, 3.0, 
1.0], [3.0, 2.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 8.0, 8.0], [4.0, 4.0, 1.0], [3.0, 2.0, 2.0]], [[2.0, 4.0, 1.0], [6.0, 3.0, 
4.0], [8.0, 7.0, 3.0]]], [[[5.0, 2.0, 8.0], [8.0, 4.0, 6.0], [5.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], [2.0, 4.0, 8.0], [1.0, 5.0, 
1.0]], [[4.0, 4.0, 2.0], [7.0, 8.0, 8.0], [3.0, 1.0, 3.0]]]]))
      val=numpy.array([[[4.0, 1.0, 4.0]], [[2.0, 7.0, 7.0]]])
      arg[1:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 7.0], [6.0, 2.0, 7.0], [7.0, 2.0, 7.0]], [[6.0, 5.0, 3.0], [8.0, 6.0, 8.0], [6.0, 
5.0, 1.0]], [[6.0, 8.0, 4.0], [8.0, 4.0, 3.0], [2.0, 2.0, 8.0]]], [[[7.0, 6.0, 7.0], [2.0, 2.0, 2.0], [3.0, 8.0, 5.0]], [[5.0, 
1.0, 5.0], [4.0, 1.0, 4.0], [4.0, 8.0, 7.0]], [[3.0, 4.0, 1.0], [6.0, 6.0, 8.0], [2.0, 7.0, 4.0]]], [[[6.0, 7.0, 5.0], [7.0, 
3.0, 8.0], [5.0, 6.0, 1.0]], [[7.0, 2.0, 7.0], [2.0, 7.0, 7.0], [4.0, 6.0, 8.0]], [[8.0, 8.0, 7.0], [3.0, 7.0, 2.0], [4.0, 1.0, 
6.0]]], [[[8.0, 3.0, 5.0], [5.0, 2.0, 5.0], [2.0, 8.0, 1.0]], [[3.0, 8.0, 7.0], [1.0, 1.0, 7.0], [7.0, 1.0, 1.0]], [[5.0, 6.0, 
4.0], [7.0, 4.0, 5.0], [5.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 5.0], [5.0, 7.0, 8.0], [6.0, 4.0, 6.0]], 
[[8.0, 8.0, 1.0], [3.0, 1.0, 7.0], [6.0, 5.0, 1.0]], [[5.0, 7.0, 6.0], [2.0, 6.0, 1.0], [3.0, 2.0, 4.0]]], [[[7.0, 6.0, 4.0], 
[1.0, 8.0, 4.0], [4.0, 7.0, 3.0]], [[3.0, 6.0, 7.0], [4.0, 1.0, 4.0], [8.0, 6.0, 8.0]], [[5.0, 5.0, 2.0], [3.0, 5.0, 4.0], 
[5.0, 3.0, 4.0]]], [[[3.0, 3.0, 1.0], [3.0, 2.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 8.0, 8.0], [2.0, 7.0, 7.0], [3.0, 2.0, 2.0]], 
[[2.0, 4.0, 1.0], [6.0, 3.0, 4.0], [8.0, 7.0, 3.0]]], [[[5.0, 2.0, 8.0], [8.0, 4.0, 6.0], [5.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], 
[2.0, 4.0, 8.0], [1.0, 5.0, 1.0]], [[4.0, 4.0, 2.0], [7.0, 8.0, 8.0], [3.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPRN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 8.0], [2.0, 6.0, 1.0], [7.0, 4.0, 8.0]], [[8.0, 3.0, 4.0], [3.0, 4.0, 3.0], [8.0, 4.0, 
4.0]], [[2.0, 1.0, 2.0], [4.0, 6.0, 5.0], [6.0, 7.0, 8.0]]], [[[5.0, 3.0, 7.0], [5.0, 2.0, 6.0], [8.0, 6.0, 3.0]], [[2.0, 8.0, 
1.0], [2.0, 8.0, 5.0], [4.0, 1.0, 7.0]], [[6.0, 2.0, 5.0], [5.0, 3.0, 1.0], [6.0, 4.0, 3.0]]], [[[4.0, 4.0, 3.0], [2.0, 5.0, 
6.0], [1.0, 7.0, 8.0]], [[7.0, 3.0, 2.0], [5.0, 8.0, 5.0], [8.0, 2.0, 3.0]], [[1.0, 5.0, 7.0], [8.0, 7.0, 7.0], [2.0, 8.0, 
8.0]]], [[[5.0, 8.0, 3.0], [6.0, 8.0, 2.0], [8.0, 7.0, 4.0]], [[8.0, 5.0, 3.0], [5.0, 8.0, 8.0], [3.0, 2.0, 3.0]], [[5.0, 7.0, 
3.0], [7.0, 3.0, 6.0], [7.0, 2.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 6.0], [4.0, 7.0, 5.0], [1.0, 5.0, 4.0]], [[4.0, 1.0, 1.0], [1.0, 4.0, 
3.0], [3.0, 6.0, 6.0]], [[4.0, 6.0, 2.0], [4.0, 4.0, 5.0], [4.0, 4.0, 3.0]]], [[[5.0, 4.0, 2.0], [3.0, 5.0, 8.0], [3.0, 5.0, 
8.0]], [[8.0, 5.0, 2.0], [8.0, 3.0, 2.0], [4.0, 3.0, 7.0]], [[5.0, 5.0, 7.0], [6.0, 6.0, 5.0], [7.0, 7.0, 3.0]]], [[[7.0, 7.0, 
8.0], [4.0, 7.0, 1.0], [3.0, 6.0, 7.0]], [[2.0, 6.0, 6.0], [5.0, 3.0, 2.0], [4.0, 2.0, 3.0]], [[8.0, 2.0, 3.0], [6.0, 5.0, 
6.0], [1.0, 7.0, 8.0]]], [[[4.0, 5.0, 3.0], [2.0, 8.0, 5.0], [4.0, 7.0, 6.0]], [[4.0, 3.0, 8.0], [2.0, 7.0, 7.0], [1.0, 2.0, 
2.0]], [[4.0, 6.0, 3.0], [2.0, 6.0, 4.0], [5.0, 3.0, 3.0]]]]))
      val=Data(numpy.array([[[7.0, 8.0, 1.0]], [[8.0, 8.0, 8.0]]]),self.functionspace)
      arg[1:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 8.0], [2.0, 6.0, 1.0], [7.0, 4.0, 8.0]], [[8.0, 3.0, 4.0], [3.0, 4.0, 3.0], [8.0, 
4.0, 4.0]], [[2.0, 1.0, 2.0], [4.0, 6.0, 5.0], [6.0, 7.0, 8.0]]], [[[5.0, 3.0, 7.0], [5.0, 2.0, 6.0], [8.0, 6.0, 3.0]], [[2.0, 
8.0, 1.0], [7.0, 8.0, 1.0], [4.0, 1.0, 7.0]], [[6.0, 2.0, 5.0], [5.0, 3.0, 1.0], [6.0, 4.0, 3.0]]], [[[4.0, 4.0, 3.0], [2.0, 
5.0, 6.0], [1.0, 7.0, 8.0]], [[7.0, 3.0, 2.0], [8.0, 8.0, 8.0], [8.0, 2.0, 3.0]], [[1.0, 5.0, 7.0], [8.0, 7.0, 7.0], [2.0, 8.0, 
8.0]]], [[[5.0, 8.0, 3.0], [6.0, 8.0, 2.0], [8.0, 7.0, 4.0]], [[8.0, 5.0, 3.0], [5.0, 8.0, 8.0], [3.0, 2.0, 3.0]], [[5.0, 7.0, 
3.0], [7.0, 3.0, 6.0], [7.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 6.0], [4.0, 7.0, 5.0], [1.0, 5.0, 4.0]], 
[[4.0, 1.0, 1.0], [1.0, 4.0, 3.0], [3.0, 6.0, 6.0]], [[4.0, 6.0, 2.0], [4.0, 4.0, 5.0], [4.0, 4.0, 3.0]]], [[[5.0, 4.0, 2.0], 
[3.0, 5.0, 8.0], [3.0, 5.0, 8.0]], [[8.0, 5.0, 2.0], [7.0, 8.0, 1.0], [4.0, 3.0, 7.0]], [[5.0, 5.0, 7.0], [6.0, 6.0, 5.0], 
[7.0, 7.0, 3.0]]], [[[7.0, 7.0, 8.0], [4.0, 7.0, 1.0], [3.0, 6.0, 7.0]], [[2.0, 6.0, 6.0], [8.0, 8.0, 8.0], [4.0, 2.0, 3.0]], 
[[8.0, 2.0, 3.0], [6.0, 5.0, 6.0], [1.0, 7.0, 8.0]]], [[[4.0, 5.0, 3.0], [2.0, 8.0, 5.0], [4.0, 7.0, 6.0]], [[4.0, 3.0, 8.0], 
[2.0, 7.0, 7.0], [1.0, 2.0, 2.0]], [[4.0, 6.0, 3.0], [2.0, 6.0, 4.0], [5.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPRN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 5.0], [3.0, 7.0, 5.0], [4.0, 2.0, 6.0]], [[6.0, 5.0, 7.0], [8.0, 3.0, 8.0], [7.0, 6.0, 
8.0]], [[3.0, 1.0, 7.0], [3.0, 8.0, 5.0], [5.0, 7.0, 6.0]]], [[[8.0, 5.0, 1.0], [1.0, 3.0, 7.0], [6.0, 4.0, 4.0]], [[4.0, 2.0, 
2.0], [1.0, 2.0, 3.0], [7.0, 3.0, 6.0]], [[1.0, 5.0, 2.0], [2.0, 6.0, 7.0], [1.0, 8.0, 6.0]]], [[[2.0, 1.0, 8.0], [4.0, 1.0, 
8.0], [2.0, 2.0, 4.0]], [[7.0, 8.0, 8.0], [4.0, 8.0, 8.0], [6.0, 4.0, 6.0]], [[6.0, 7.0, 5.0], [6.0, 1.0, 8.0], [8.0, 8.0, 
4.0]]], [[[1.0, 1.0, 1.0], [1.0, 5.0, 2.0], [7.0, 2.0, 4.0]], [[2.0, 8.0, 2.0], [6.0, 3.0, 5.0], [5.0, 3.0, 7.0]], [[5.0, 8.0, 
3.0], [4.0, 8.0, 2.0], [1.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 6.0], [3.0, 4.0, 8.0], [1.0, 8.0, 2.0]], [[8.0, 2.0, 3.0], [4.0, 4.0, 
5.0], [1.0, 1.0, 8.0]], [[8.0, 1.0, 7.0], [8.0, 1.0, 2.0], [1.0, 1.0, 4.0]]], [[[5.0, 3.0, 2.0], [6.0, 8.0, 2.0], [6.0, 6.0, 
1.0]], [[2.0, 1.0, 4.0], [1.0, 1.0, 3.0], [6.0, 4.0, 7.0]], [[7.0, 8.0, 3.0], [1.0, 8.0, 8.0], [5.0, 4.0, 2.0]]], [[[4.0, 2.0, 
7.0], [2.0, 4.0, 4.0], [5.0, 5.0, 2.0]], [[4.0, 3.0, 1.0], [3.0, 5.0, 5.0], [2.0, 2.0, 4.0]], [[1.0, 3.0, 7.0], [4.0, 7.0, 
7.0], [2.0, 2.0, 3.0]]], [[[6.0, 5.0, 4.0], [7.0, 1.0, 3.0], [4.0, 4.0, 7.0]], [[6.0, 8.0, 8.0], [7.0, 4.0, 6.0], [6.0, 3.0, 
8.0]], [[5.0, 3.0, 8.0], [2.0, 5.0, 1.0], [8.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[5.0, 6.0, 7.0]], [[6.0, 1.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 5.0, 5.0]], [[5.0, 6.0, 4.0]]]))
      arg[1:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 5.0], [3.0, 7.0, 5.0], [4.0, 2.0, 6.0]], [[6.0, 5.0, 7.0], [8.0, 3.0, 8.0], [7.0, 
6.0, 8.0]], [[3.0, 1.0, 7.0], [3.0, 8.0, 5.0], [5.0, 7.0, 6.0]]], [[[8.0, 5.0, 1.0], [1.0, 3.0, 7.0], [6.0, 4.0, 4.0]], [[4.0, 
2.0, 2.0], [5.0, 6.0, 7.0], [7.0, 3.0, 6.0]], [[1.0, 5.0, 2.0], [2.0, 6.0, 7.0], [1.0, 8.0, 6.0]]], [[[2.0, 1.0, 8.0], [4.0, 
1.0, 8.0], [2.0, 2.0, 4.0]], [[7.0, 8.0, 8.0], [6.0, 1.0, 2.0], [6.0, 4.0, 6.0]], [[6.0, 7.0, 5.0], [6.0, 1.0, 8.0], [8.0, 8.0, 
4.0]]], [[[1.0, 1.0, 1.0], [1.0, 5.0, 2.0], [7.0, 2.0, 4.0]], [[2.0, 8.0, 2.0], [6.0, 3.0, 5.0], [5.0, 3.0, 7.0]], [[5.0, 8.0, 
3.0], [4.0, 8.0, 2.0], [1.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 6.0], [3.0, 4.0, 8.0], [1.0, 8.0, 2.0]], 
[[8.0, 2.0, 3.0], [4.0, 4.0, 5.0], [1.0, 1.0, 8.0]], [[8.0, 1.0, 7.0], [8.0, 1.0, 2.0], [1.0, 1.0, 4.0]]], [[[5.0, 3.0, 2.0], 
[6.0, 8.0, 2.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 4.0], [2.0, 5.0, 5.0], [6.0, 4.0, 7.0]], [[7.0, 8.0, 3.0], [1.0, 8.0, 8.0], 
[5.0, 4.0, 2.0]]], [[[4.0, 2.0, 7.0], [2.0, 4.0, 4.0], [5.0, 5.0, 2.0]], [[4.0, 3.0, 1.0], [5.0, 6.0, 4.0], [2.0, 2.0, 4.0]], 
[[1.0, 3.0, 7.0], [4.0, 7.0, 7.0], [2.0, 2.0, 3.0]]], [[[6.0, 5.0, 4.0], [7.0, 1.0, 3.0], [4.0, 4.0, 7.0]], [[6.0, 8.0, 8.0], 
[7.0, 4.0, 6.0], [6.0, 3.0, 8.0]], [[5.0, 3.0, 8.0], [2.0, 5.0, 1.0], [8.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPRN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 4.0], [1.0, 7.0, 6.0], [7.0, 3.0, 6.0]], [[4.0, 6.0, 5.0], [5.0, 6.0, 7.0], [3.0, 8.0, 
7.0]], [[2.0, 7.0, 8.0], [3.0, 1.0, 1.0], [5.0, 7.0, 3.0]]], [[[5.0, 1.0, 3.0], [4.0, 7.0, 3.0], [4.0, 1.0, 5.0]], [[5.0, 6.0, 
1.0], [6.0, 6.0, 5.0], [5.0, 8.0, 4.0]], [[3.0, 2.0, 6.0], [4.0, 4.0, 4.0], [3.0, 6.0, 5.0]]], [[[3.0, 3.0, 3.0], [2.0, 1.0, 
8.0], [7.0, 3.0, 3.0]], [[8.0, 3.0, 7.0], [2.0, 6.0, 7.0], [3.0, 4.0, 8.0]], [[3.0, 4.0, 5.0], [1.0, 2.0, 2.0], [2.0, 2.0, 
1.0]]], [[[2.0, 8.0, 3.0], [2.0, 6.0, 5.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 5.0], [5.0, 5.0, 1.0], [1.0, 2.0, 7.0]], [[2.0, 5.0, 
3.0], [4.0, 8.0, 6.0], [6.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 1.0], [1.0, 2.0, 4.0], [3.0, 2.0, 1.0]], [[1.0, 8.0, 2.0], [7.0, 5.0, 
3.0], [1.0, 6.0, 6.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 8.0], [5.0, 4.0, 5.0]]], [[[2.0, 7.0, 3.0], [4.0, 4.0, 1.0], [1.0, 8.0, 
5.0]], [[3.0, 4.0, 1.0], [2.0, 1.0, 3.0], [2.0, 2.0, 7.0]], [[3.0, 2.0, 8.0], [1.0, 1.0, 5.0], [2.0, 4.0, 1.0]]], [[[2.0, 2.0, 
3.0], [6.0, 8.0, 2.0], [7.0, 8.0, 1.0]], [[3.0, 6.0, 2.0], [8.0, 7.0, 6.0], [5.0, 4.0, 2.0]], [[2.0, 6.0, 8.0], [6.0, 3.0, 
8.0], [4.0, 6.0, 4.0]]], [[[5.0, 7.0, 5.0], [8.0, 7.0, 6.0], [3.0, 5.0, 3.0]], [[3.0, 8.0, 3.0], [7.0, 2.0, 1.0], [6.0, 2.0, 
4.0]], [[5.0, 7.0, 4.0], [2.0, 3.0, 7.0], [8.0, 7.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[2.0, 2.0, 3.0]], [[7.0, 4.0, 4.0]]])+(1.-msk_val)*numpy.array([[[4.0, 4.0, 1.0]], [[4.0, 5.0, 
4.0]]])
      arg[1:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 4.0], [1.0, 7.0, 6.0], [7.0, 3.0, 6.0]], [[4.0, 6.0, 5.0], [5.0, 6.0, 7.0], [3.0, 
8.0, 7.0]], [[2.0, 7.0, 8.0], [3.0, 1.0, 1.0], [5.0, 7.0, 3.0]]], [[[5.0, 1.0, 3.0], [4.0, 7.0, 3.0], [4.0, 1.0, 5.0]], [[5.0, 
6.0, 1.0], [2.0, 2.0, 3.0], [5.0, 8.0, 4.0]], [[3.0, 2.0, 6.0], [4.0, 4.0, 4.0], [3.0, 6.0, 5.0]]], [[[3.0, 3.0, 3.0], [2.0, 
1.0, 8.0], [7.0, 3.0, 3.0]], [[8.0, 3.0, 7.0], [7.0, 4.0, 4.0], [3.0, 4.0, 8.0]], [[3.0, 4.0, 5.0], [1.0, 2.0, 2.0], [2.0, 2.0, 
1.0]]], [[[2.0, 8.0, 3.0], [2.0, 6.0, 5.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 5.0], [5.0, 5.0, 1.0], [1.0, 2.0, 7.0]], [[2.0, 5.0, 
3.0], [4.0, 8.0, 6.0], [6.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 1.0], [1.0, 2.0, 4.0], [3.0, 2.0, 1.0]], 
[[1.0, 8.0, 2.0], [7.0, 5.0, 3.0], [1.0, 6.0, 6.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 8.0], [5.0, 4.0, 5.0]]], [[[2.0, 7.0, 3.0], 
[4.0, 4.0, 1.0], [1.0, 8.0, 5.0]], [[3.0, 4.0, 1.0], [4.0, 4.0, 1.0], [2.0, 2.0, 7.0]], [[3.0, 2.0, 8.0], [1.0, 1.0, 5.0], 
[2.0, 4.0, 1.0]]], [[[2.0, 2.0, 3.0], [6.0, 8.0, 2.0], [7.0, 8.0, 1.0]], [[3.0, 6.0, 2.0], [4.0, 5.0, 4.0], [5.0, 4.0, 2.0]], 
[[2.0, 6.0, 8.0], [6.0, 3.0, 8.0], [4.0, 6.0, 4.0]]], [[[5.0, 7.0, 5.0], [8.0, 7.0, 6.0], [3.0, 5.0, 3.0]], [[3.0, 8.0, 3.0], 
[7.0, 2.0, 1.0], [6.0, 2.0, 4.0]], [[5.0, 7.0, 4.0], [2.0, 3.0, 7.0], [8.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CURN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 6.0], [4.0, 5.0, 3.0], [3.0, 1.0, 4.0]], [[6.0, 7.0, 7.0], [6.0, 2.0, 3.0], [2.0, 7.0, 
4.0]], [[3.0, 3.0, 2.0], [6.0, 2.0, 8.0], [8.0, 2.0, 4.0]]], [[[7.0, 2.0, 5.0], [2.0, 4.0, 8.0], [4.0, 1.0, 6.0]], [[3.0, 8.0, 
7.0], [8.0, 3.0, 3.0], [8.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 8.0, 1.0], [4.0, 1.0, 5.0]]], [[[8.0, 2.0, 3.0], [3.0, 7.0, 
8.0], [3.0, 1.0, 5.0]], [[2.0, 5.0, 6.0], [1.0, 1.0, 6.0], [1.0, 7.0, 5.0]], [[1.0, 7.0, 3.0], [7.0, 7.0, 1.0], [4.0, 6.0, 
8.0]]], [[[5.0, 8.0, 6.0], [1.0, 8.0, 1.0], [2.0, 5.0, 1.0]], [[6.0, 3.0, 7.0], [3.0, 5.0, 6.0], [1.0, 5.0, 7.0]], [[4.0, 5.0, 
1.0], [4.0, 6.0, 2.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 4.0], [4.0, 6.0, 8.0], [7.0, 1.0, 2.0]], [[7.0, 5.0, 4.0], [5.0, 1.0, 
1.0], [5.0, 6.0, 3.0]], [[8.0, 6.0, 3.0], [4.0, 5.0, 8.0], [7.0, 3.0, 8.0]]], [[[3.0, 3.0, 7.0], [7.0, 5.0, 4.0], [5.0, 5.0, 
2.0]], [[5.0, 7.0, 3.0], [1.0, 1.0, 1.0], [6.0, 5.0, 4.0]], [[5.0, 2.0, 7.0], [3.0, 7.0, 5.0], [5.0, 7.0, 3.0]]], [[[4.0, 4.0, 
7.0], [5.0, 1.0, 1.0], [4.0, 4.0, 4.0]], [[3.0, 3.0, 6.0], [6.0, 1.0, 4.0], [8.0, 6.0, 5.0]], [[4.0, 4.0, 8.0], [7.0, 7.0, 
3.0], [7.0, 5.0, 6.0]]], [[[8.0, 5.0, 8.0], [4.0, 5.0, 4.0], [6.0, 4.0, 8.0]], [[7.0, 2.0, 4.0], [6.0, 1.0, 5.0], [5.0, 2.0, 
8.0]], [[8.0, 3.0, 7.0], [3.0, 4.0, 4.0], [3.0, 1.0, 3.0]]]]))
      val=numpy.array([[[[5.0, 1.0, 7.0]], [[6.0, 8.0, 5.0]]], [[[4.0, 2.0, 5.0]], [[8.0, 5.0, 3.0]]], [[[2.0, 8.0, 5.0]], 
[[6.0, 6.0, 8.0]]], [[[3.0, 7.0, 3.0]], [[5.0, 2.0, 5.0]]]])
      arg[:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 6.0], [5.0, 1.0, 7.0], [3.0, 1.0, 4.0]], [[6.0, 7.0, 7.0], [6.0, 8.0, 5.0], [2.0, 
7.0, 4.0]], [[3.0, 3.0, 2.0], [6.0, 2.0, 8.0], [8.0, 2.0, 4.0]]], [[[7.0, 2.0, 5.0], [4.0, 2.0, 5.0], [4.0, 1.0, 6.0]], [[3.0, 
8.0, 7.0], [8.0, 5.0, 3.0], [8.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 8.0, 1.0], [4.0, 1.0, 5.0]]], [[[8.0, 2.0, 3.0], [2.0, 
8.0, 5.0], [3.0, 1.0, 5.0]], [[2.0, 5.0, 6.0], [6.0, 6.0, 8.0], [1.0, 7.0, 5.0]], [[1.0, 7.0, 3.0], [7.0, 7.0, 1.0], [4.0, 6.0, 
8.0]]], [[[5.0, 8.0, 6.0], [3.0, 7.0, 3.0], [2.0, 5.0, 1.0]], [[6.0, 3.0, 7.0], [5.0, 2.0, 5.0], [1.0, 5.0, 7.0]], [[4.0, 5.0, 
1.0], [4.0, 6.0, 2.0], [6.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 4.0], [5.0, 1.0, 7.0], [7.0, 1.0, 2.0]], 
[[7.0, 5.0, 4.0], [6.0, 8.0, 5.0], [5.0, 6.0, 3.0]], [[8.0, 6.0, 3.0], [4.0, 5.0, 8.0], [7.0, 3.0, 8.0]]], [[[3.0, 3.0, 7.0], 
[4.0, 2.0, 5.0], [5.0, 5.0, 2.0]], [[5.0, 7.0, 3.0], [8.0, 5.0, 3.0], [6.0, 5.0, 4.0]], [[5.0, 2.0, 7.0], [3.0, 7.0, 5.0], 
[5.0, 7.0, 3.0]]], [[[4.0, 4.0, 7.0], [2.0, 8.0, 5.0], [4.0, 4.0, 4.0]], [[3.0, 3.0, 6.0], [6.0, 6.0, 8.0], [8.0, 6.0, 5.0]], 
[[4.0, 4.0, 8.0], [7.0, 7.0, 3.0], [7.0, 5.0, 6.0]]], [[[8.0, 5.0, 8.0], [3.0, 7.0, 3.0], [6.0, 4.0, 8.0]], [[7.0, 2.0, 4.0], 
[5.0, 2.0, 5.0], [5.0, 2.0, 8.0]], [[8.0, 3.0, 7.0], [3.0, 4.0, 4.0], [3.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CURN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 7.0], [8.0, 7.0, 1.0], [5.0, 6.0, 7.0]], [[1.0, 8.0, 7.0], [1.0, 5.0, 4.0], [3.0, 1.0, 
5.0]], [[5.0, 8.0, 1.0], [7.0, 3.0, 3.0], [4.0, 6.0, 4.0]]], [[[4.0, 8.0, 3.0], [1.0, 5.0, 2.0], [8.0, 7.0, 2.0]], [[6.0, 7.0, 
6.0], [5.0, 8.0, 7.0], [8.0, 1.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 3.0, 2.0], [7.0, 3.0, 6.0]]], [[[8.0, 2.0, 2.0], [5.0, 8.0, 
7.0], [8.0, 1.0, 2.0]], [[4.0, 6.0, 3.0], [1.0, 7.0, 2.0], [6.0, 8.0, 3.0]], [[8.0, 3.0, 5.0], [6.0, 8.0, 4.0], [6.0, 3.0, 
5.0]]], [[[4.0, 1.0, 7.0], [3.0, 6.0, 5.0], [1.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [8.0, 1.0, 3.0], [4.0, 2.0, 6.0]], [[2.0, 7.0, 
5.0], [2.0, 8.0, 4.0], [8.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 1.0], [3.0, 8.0, 1.0], [1.0, 8.0, 1.0]], [[8.0, 4.0, 5.0], [2.0, 5.0, 
1.0], [7.0, 4.0, 1.0]], [[5.0, 4.0, 1.0], [1.0, 3.0, 2.0], [2.0, 2.0, 8.0]]], [[[2.0, 7.0, 7.0], [4.0, 3.0, 8.0], [2.0, 3.0, 
3.0]], [[8.0, 8.0, 6.0], [1.0, 6.0, 7.0], [3.0, 7.0, 4.0]], [[7.0, 3.0, 4.0], [6.0, 1.0, 1.0], [7.0, 4.0, 2.0]]], [[[2.0, 3.0, 
7.0], [3.0, 1.0, 6.0], [3.0, 4.0, 5.0]], [[2.0, 1.0, 7.0], [5.0, 1.0, 2.0], [2.0, 2.0, 8.0]], [[4.0, 1.0, 8.0], [6.0, 1.0, 
3.0], [1.0, 6.0, 4.0]]], [[[1.0, 5.0, 5.0], [3.0, 1.0, 8.0], [4.0, 6.0, 8.0]], [[5.0, 7.0, 4.0], [4.0, 5.0, 2.0], [8.0, 6.0, 
6.0]], [[6.0, 2.0, 6.0], [7.0, 5.0, 4.0], [7.0, 5.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0, 2.0, 4.0]], [[8.0, 3.0, 8.0]]], [[[5.0, 1.0, 8.0]], [[7.0, 2.0, 5.0]]], [[[3.0, 5.0, 8.0]], 
[[7.0, 2.0, 5.0]]], [[[4.0, 8.0, 5.0]], [[1.0, 1.0, 2.0]]]]),self.functionspace)
      arg[:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 7.0], [5.0, 2.0, 4.0], [5.0, 6.0, 7.0]], [[1.0, 8.0, 7.0], [8.0, 3.0, 8.0], [3.0, 
1.0, 5.0]], [[5.0, 8.0, 1.0], [7.0, 3.0, 3.0], [4.0, 6.0, 4.0]]], [[[4.0, 8.0, 3.0], [5.0, 1.0, 8.0], [8.0, 7.0, 2.0]], [[6.0, 
7.0, 6.0], [7.0, 2.0, 5.0], [8.0, 1.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 3.0, 2.0], [7.0, 3.0, 6.0]]], [[[8.0, 2.0, 2.0], [3.0, 
5.0, 8.0], [8.0, 1.0, 2.0]], [[4.0, 6.0, 3.0], [7.0, 2.0, 5.0], [6.0, 8.0, 3.0]], [[8.0, 3.0, 5.0], [6.0, 8.0, 4.0], [6.0, 3.0, 
5.0]]], [[[4.0, 1.0, 7.0], [4.0, 8.0, 5.0], [1.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [1.0, 1.0, 2.0], [4.0, 2.0, 6.0]], [[2.0, 7.0, 
5.0], [2.0, 8.0, 4.0], [8.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 1.0], [5.0, 2.0, 4.0], [1.0, 8.0, 1.0]], 
[[8.0, 4.0, 5.0], [8.0, 3.0, 8.0], [7.0, 4.0, 1.0]], [[5.0, 4.0, 1.0], [1.0, 3.0, 2.0], [2.0, 2.0, 8.0]]], [[[2.0, 7.0, 7.0], 
[5.0, 1.0, 8.0], [2.0, 3.0, 3.0]], [[8.0, 8.0, 6.0], [7.0, 2.0, 5.0], [3.0, 7.0, 4.0]], [[7.0, 3.0, 4.0], [6.0, 1.0, 1.0], 
[7.0, 4.0, 2.0]]], [[[2.0, 3.0, 7.0], [3.0, 5.0, 8.0], [3.0, 4.0, 5.0]], [[2.0, 1.0, 7.0], [7.0, 2.0, 5.0], [2.0, 2.0, 8.0]], 
[[4.0, 1.0, 8.0], [6.0, 1.0, 3.0], [1.0, 6.0, 4.0]]], [[[1.0, 5.0, 5.0], [4.0, 8.0, 5.0], [4.0, 6.0, 8.0]], [[5.0, 7.0, 4.0], 
[1.0, 1.0, 2.0], [8.0, 6.0, 6.0]], [[6.0, 2.0, 6.0], [7.0, 5.0, 4.0], [7.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CURN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 3.0], [1.0, 6.0, 6.0], [7.0, 2.0, 6.0]], [[3.0, 4.0, 6.0], [4.0, 6.0, 2.0], [2.0, 6.0, 
5.0]], [[1.0, 6.0, 3.0], [1.0, 8.0, 5.0], [7.0, 7.0, 5.0]]], [[[1.0, 8.0, 8.0], [7.0, 4.0, 4.0], [4.0, 8.0, 5.0]], [[4.0, 1.0, 
3.0], [6.0, 2.0, 5.0], [8.0, 3.0, 4.0]], [[7.0, 8.0, 4.0], [4.0, 4.0, 4.0], [4.0, 5.0, 6.0]]], [[[1.0, 4.0, 5.0], [4.0, 6.0, 
2.0], [7.0, 7.0, 5.0]], [[1.0, 3.0, 6.0], [7.0, 3.0, 6.0], [2.0, 1.0, 6.0]], [[8.0, 7.0, 6.0], [7.0, 8.0, 6.0], [5.0, 6.0, 
7.0]]], [[[3.0, 1.0, 8.0], [3.0, 3.0, 5.0], [5.0, 4.0, 3.0]], [[1.0, 1.0, 2.0], [2.0, 1.0, 5.0], [1.0, 5.0, 4.0]], [[2.0, 1.0, 
5.0], [7.0, 5.0, 3.0], [4.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 3.0], [3.0, 1.0, 6.0], [7.0, 1.0, 3.0]], [[6.0, 5.0, 7.0], [3.0, 8.0, 
5.0], [1.0, 8.0, 3.0]], [[2.0, 6.0, 2.0], [1.0, 4.0, 8.0], [3.0, 5.0, 8.0]]], [[[3.0, 2.0, 7.0], [1.0, 8.0, 3.0], [6.0, 1.0, 
8.0]], [[7.0, 4.0, 6.0], [6.0, 8.0, 7.0], [3.0, 1.0, 4.0]], [[1.0, 2.0, 1.0], [7.0, 3.0, 1.0], [3.0, 3.0, 1.0]]], [[[7.0, 1.0, 
6.0], [6.0, 5.0, 7.0], [3.0, 7.0, 2.0]], [[4.0, 2.0, 8.0], [5.0, 2.0, 1.0], [4.0, 3.0, 2.0]], [[6.0, 6.0, 5.0], [7.0, 1.0, 
3.0], [1.0, 1.0, 2.0]]], [[[4.0, 5.0, 3.0], [3.0, 5.0, 4.0], [5.0, 4.0, 3.0]], [[2.0, 2.0, 7.0], [3.0, 3.0, 7.0], [5.0, 6.0, 
7.0]], [[4.0, 5.0, 3.0], [7.0, 8.0, 5.0], [5.0, 3.0, 6.0]]]]))
      val=Data(numpy.array([[[[1.0, 1.0, 6.0]], [[6.0, 1.0, 4.0]]], [[[5.0, 6.0, 8.0]], [[6.0, 1.0, 8.0]]], [[[3.0, 5.0, 5.0]], 
[[5.0, 6.0, 5.0]]], [[[1.0, 7.0, 2.0]], [[5.0, 7.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 2.0]], [[1.0, 2.0, 7.0]]], [[[5.0, 6.0, 8.0]], [[2.0, 5.0, 3.0]]], [[[2.0, 
1.0, 8.0]], [[7.0, 4.0, 3.0]]], [[[8.0, 4.0, 4.0]], [[7.0, 3.0, 3.0]]]]))
      arg[:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 3.0], [1.0, 1.0, 6.0], [7.0, 2.0, 6.0]], [[3.0, 4.0, 6.0], [6.0, 1.0, 4.0], [2.0, 
6.0, 5.0]], [[1.0, 6.0, 3.0], [1.0, 8.0, 5.0], [7.0, 7.0, 5.0]]], [[[1.0, 8.0, 8.0], [5.0, 6.0, 8.0], [4.0, 8.0, 5.0]], [[4.0, 
1.0, 3.0], [6.0, 1.0, 8.0], [8.0, 3.0, 4.0]], [[7.0, 8.0, 4.0], [4.0, 4.0, 4.0], [4.0, 5.0, 6.0]]], [[[1.0, 4.0, 5.0], [3.0, 
5.0, 5.0], [7.0, 7.0, 5.0]], [[1.0, 3.0, 6.0], [5.0, 6.0, 5.0], [2.0, 1.0, 6.0]], [[8.0, 7.0, 6.0], [7.0, 8.0, 6.0], [5.0, 6.0, 
7.0]]], [[[3.0, 1.0, 8.0], [1.0, 7.0, 2.0], [5.0, 4.0, 3.0]], [[1.0, 1.0, 2.0], [5.0, 7.0, 8.0], [1.0, 5.0, 4.0]], [[2.0, 1.0, 
5.0], [7.0, 5.0, 3.0], [4.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 3.0], [5.0, 6.0, 2.0], [7.0, 1.0, 3.0]], 
[[6.0, 5.0, 7.0], [1.0, 2.0, 7.0], [1.0, 8.0, 3.0]], [[2.0, 6.0, 2.0], [1.0, 4.0, 8.0], [3.0, 5.0, 8.0]]], [[[3.0, 2.0, 7.0], 
[5.0, 6.0, 8.0], [6.0, 1.0, 8.0]], [[7.0, 4.0, 6.0], [2.0, 5.0, 3.0], [3.0, 1.0, 4.0]], [[1.0, 2.0, 1.0], [7.0, 3.0, 1.0], 
[3.0, 3.0, 1.0]]], [[[7.0, 1.0, 6.0], [2.0, 1.0, 8.0], [3.0, 7.0, 2.0]], [[4.0, 2.0, 8.0], [7.0, 4.0, 3.0], [4.0, 3.0, 2.0]], 
[[6.0, 6.0, 5.0], [7.0, 1.0, 3.0], [1.0, 1.0, 2.0]]], [[[4.0, 5.0, 3.0], [8.0, 4.0, 4.0], [5.0, 4.0, 3.0]], [[2.0, 2.0, 7.0], 
[7.0, 3.0, 3.0], [5.0, 6.0, 7.0]], [[4.0, 5.0, 3.0], [7.0, 8.0, 5.0], [5.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CURN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 4.0], [1.0, 7.0, 5.0], [3.0, 6.0, 8.0]], [[4.0, 1.0, 8.0], [6.0, 1.0, 1.0], [6.0, 2.0, 
1.0]], [[5.0, 7.0, 7.0], [6.0, 3.0, 6.0], [6.0, 2.0, 8.0]]], [[[5.0, 4.0, 8.0], [5.0, 7.0, 3.0], [3.0, 6.0, 5.0]], [[1.0, 3.0, 
1.0], [1.0, 4.0, 2.0], [1.0, 2.0, 8.0]], [[8.0, 4.0, 2.0], [8.0, 6.0, 4.0], [1.0, 1.0, 2.0]]], [[[7.0, 7.0, 4.0], [6.0, 3.0, 
6.0], [8.0, 1.0, 8.0]], [[8.0, 7.0, 8.0], [6.0, 2.0, 2.0], [7.0, 1.0, 3.0]], [[1.0, 4.0, 2.0], [5.0, 6.0, 7.0], [7.0, 5.0, 
5.0]]], [[[1.0, 3.0, 4.0], [5.0, 7.0, 4.0], [5.0, 5.0, 4.0]], [[8.0, 6.0, 5.0], [3.0, 8.0, 7.0], [5.0, 1.0, 5.0]], [[2.0, 4.0, 
8.0], [7.0, 3.0, 2.0], [1.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 5.0], [1.0, 6.0, 5.0], [2.0, 4.0, 5.0]], [[6.0, 6.0, 6.0], [1.0, 8.0, 
2.0], [2.0, 1.0, 3.0]], [[4.0, 4.0, 5.0], [1.0, 2.0, 5.0], [8.0, 1.0, 2.0]]], [[[7.0, 8.0, 4.0], [6.0, 3.0, 1.0], [8.0, 6.0, 
5.0]], [[8.0, 2.0, 4.0], [3.0, 3.0, 6.0], [8.0, 6.0, 2.0]], [[6.0, 1.0, 7.0], [7.0, 1.0, 4.0], [4.0, 1.0, 2.0]]], [[[5.0, 4.0, 
8.0], [8.0, 7.0, 6.0], [7.0, 3.0, 7.0]], [[5.0, 8.0, 4.0], [3.0, 2.0, 2.0], [6.0, 5.0, 1.0]], [[3.0, 1.0, 7.0], [4.0, 3.0, 
3.0], [6.0, 8.0, 3.0]]], [[[8.0, 3.0, 6.0], [1.0, 6.0, 1.0], [2.0, 4.0, 1.0]], [[4.0, 8.0, 4.0], [3.0, 3.0, 6.0], [7.0, 8.0, 
3.0]], [[4.0, 7.0, 3.0], [6.0, 4.0, 6.0], [8.0, 3.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 3.0, 7.0]], [[8.0, 3.0, 3.0]]], [[[8.0, 5.0, 4.0]], [[2.0, 8.0, 3.0]]], [[[1.0, 7.0, 
7.0]], [[2.0, 7.0, 1.0]]], [[[5.0, 1.0, 8.0]], [[2.0, 6.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 7.0, 6.0]], [[2.0, 5.0, 
4.0]]], [[[8.0, 1.0, 3.0]], [[8.0, 4.0, 5.0]]], [[[1.0, 8.0, 1.0]], [[4.0, 7.0, 7.0]]], [[[4.0, 1.0, 8.0]], [[1.0, 5.0, 7.0]]]])
      arg[:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 4.0], [5.0, 3.0, 7.0], [3.0, 6.0, 8.0]], [[4.0, 1.0, 8.0], [8.0, 3.0, 3.0], [6.0, 
2.0, 1.0]], [[5.0, 7.0, 7.0], [6.0, 3.0, 6.0], [6.0, 2.0, 8.0]]], [[[5.0, 4.0, 8.0], [8.0, 5.0, 4.0], [3.0, 6.0, 5.0]], [[1.0, 
3.0, 1.0], [2.0, 8.0, 3.0], [1.0, 2.0, 8.0]], [[8.0, 4.0, 2.0], [8.0, 6.0, 4.0], [1.0, 1.0, 2.0]]], [[[7.0, 7.0, 4.0], [1.0, 
7.0, 7.0], [8.0, 1.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 7.0, 1.0], [7.0, 1.0, 3.0]], [[1.0, 4.0, 2.0], [5.0, 6.0, 7.0], [7.0, 5.0, 
5.0]]], [[[1.0, 3.0, 4.0], [5.0, 1.0, 8.0], [5.0, 5.0, 4.0]], [[8.0, 6.0, 5.0], [2.0, 6.0, 2.0], [5.0, 1.0, 5.0]], [[2.0, 4.0, 
8.0], [7.0, 3.0, 2.0], [1.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 5.0], [4.0, 7.0, 6.0], [2.0, 4.0, 5.0]], 
[[6.0, 6.0, 6.0], [2.0, 5.0, 4.0], [2.0, 1.0, 3.0]], [[4.0, 4.0, 5.0], [1.0, 2.0, 5.0], [8.0, 1.0, 2.0]]], [[[7.0, 8.0, 4.0], 
[8.0, 1.0, 3.0], [8.0, 6.0, 5.0]], [[8.0, 2.0, 4.0], [8.0, 4.0, 5.0], [8.0, 6.0, 2.0]], [[6.0, 1.0, 7.0], [7.0, 1.0, 4.0], 
[4.0, 1.0, 2.0]]], [[[5.0, 4.0, 8.0], [1.0, 8.0, 1.0], [7.0, 3.0, 7.0]], [[5.0, 8.0, 4.0], [4.0, 7.0, 7.0], [6.0, 5.0, 1.0]], 
[[3.0, 1.0, 7.0], [4.0, 3.0, 3.0], [6.0, 8.0, 3.0]]], [[[8.0, 3.0, 6.0], [4.0, 1.0, 8.0], [2.0, 4.0, 1.0]], [[4.0, 8.0, 4.0], 
[1.0, 5.0, 7.0], [7.0, 8.0, 3.0]], [[4.0, 7.0, 3.0], [6.0, 4.0, 6.0], [8.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CLPN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 3.0], [8.0, 5.0, 4.0], [8.0, 2.0, 8.0]], [[8.0, 7.0, 2.0], [2.0, 5.0, 6.0], [8.0, 1.0, 
7.0]], [[5.0, 4.0, 4.0], [3.0, 1.0, 8.0], [7.0, 8.0, 8.0]]], [[[6.0, 2.0, 5.0], [2.0, 2.0, 2.0], [3.0, 6.0, 8.0]], [[1.0, 7.0, 
4.0], [1.0, 2.0, 3.0], [4.0, 5.0, 3.0]], [[7.0, 7.0, 8.0], [7.0, 8.0, 5.0], [3.0, 1.0, 6.0]]], [[[4.0, 6.0, 6.0], [4.0, 6.0, 
4.0], [3.0, 7.0, 7.0]], [[4.0, 6.0, 7.0], [6.0, 2.0, 4.0], [4.0, 5.0, 3.0]], [[2.0, 2.0, 4.0], [8.0, 8.0, 7.0], [7.0, 8.0, 
8.0]]], [[[3.0, 1.0, 6.0], [2.0, 3.0, 2.0], [3.0, 5.0, 6.0]], [[3.0, 4.0, 2.0], [5.0, 5.0, 4.0], [7.0, 8.0, 1.0]], [[2.0, 6.0, 
8.0], [1.0, 3.0, 3.0], [1.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 8.0], [1.0, 8.0, 6.0], [7.0, 1.0, 7.0]], [[2.0, 2.0, 2.0], [7.0, 6.0, 
1.0], [2.0, 8.0, 7.0]], [[2.0, 4.0, 4.0], [5.0, 1.0, 3.0], [2.0, 8.0, 3.0]]], [[[7.0, 1.0, 5.0], [5.0, 3.0, 6.0], [1.0, 8.0, 
7.0]], [[4.0, 2.0, 3.0], [6.0, 6.0, 1.0], [7.0, 1.0, 6.0]], [[7.0, 3.0, 3.0], [4.0, 3.0, 5.0], [5.0, 1.0, 6.0]]], [[[1.0, 8.0, 
4.0], [5.0, 7.0, 3.0], [2.0, 2.0, 4.0]], [[7.0, 6.0, 2.0], [1.0, 6.0, 5.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 6.0], [2.0, 3.0, 
5.0], [7.0, 6.0, 3.0]]], [[[5.0, 3.0, 2.0], [2.0, 5.0, 8.0], [1.0, 1.0, 6.0]], [[1.0, 7.0, 3.0], [6.0, 3.0, 2.0], [2.0, 4.0, 
7.0]], [[4.0, 1.0, 1.0], [6.0, 6.0, 8.0], [1.0, 3.0, 5.0]]]]))
      val=numpy.array([[[1.0, 3.0, 4.0]], [[5.0, 7.0, 4.0]], [[2.0, 4.0, 3.0]], [[6.0, 5.0, 8.0]]])
      arg[:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 3.0], [8.0, 5.0, 4.0], [8.0, 2.0, 8.0]], [[8.0, 7.0, 2.0], [2.0, 5.0, 6.0], [8.0, 
1.0, 7.0]], [[5.0, 4.0, 4.0], [1.0, 3.0, 4.0], [7.0, 8.0, 8.0]]], [[[6.0, 2.0, 5.0], [2.0, 2.0, 2.0], [3.0, 6.0, 8.0]], [[1.0, 
7.0, 4.0], [1.0, 2.0, 3.0], [4.0, 5.0, 3.0]], [[7.0, 7.0, 8.0], [5.0, 7.0, 4.0], [3.0, 1.0, 6.0]]], [[[4.0, 6.0, 6.0], [4.0, 
6.0, 4.0], [3.0, 7.0, 7.0]], [[4.0, 6.0, 7.0], [6.0, 2.0, 4.0], [4.0, 5.0, 3.0]], [[2.0, 2.0, 4.0], [2.0, 4.0, 3.0], [7.0, 8.0, 
8.0]]], [[[3.0, 1.0, 6.0], [2.0, 3.0, 2.0], [3.0, 5.0, 6.0]], [[3.0, 4.0, 2.0], [5.0, 5.0, 4.0], [7.0, 8.0, 1.0]], [[2.0, 6.0, 
8.0], [6.0, 5.0, 8.0], [1.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 8.0], [1.0, 8.0, 6.0], [7.0, 1.0, 7.0]], 
[[2.0, 2.0, 2.0], [7.0, 6.0, 1.0], [2.0, 8.0, 7.0]], [[2.0, 4.0, 4.0], [1.0, 3.0, 4.0], [2.0, 8.0, 3.0]]], [[[7.0, 1.0, 5.0], 
[5.0, 3.0, 6.0], [1.0, 8.0, 7.0]], [[4.0, 2.0, 3.0], [6.0, 6.0, 1.0], [7.0, 1.0, 6.0]], [[7.0, 3.0, 3.0], [5.0, 7.0, 4.0], 
[5.0, 1.0, 6.0]]], [[[1.0, 8.0, 4.0], [5.0, 7.0, 3.0], [2.0, 2.0, 4.0]], [[7.0, 6.0, 2.0], [1.0, 6.0, 5.0], [7.0, 4.0, 3.0]], 
[[8.0, 7.0, 6.0], [2.0, 4.0, 3.0], [7.0, 6.0, 3.0]]], [[[5.0, 3.0, 2.0], [2.0, 5.0, 8.0], [1.0, 1.0, 6.0]], [[1.0, 7.0, 3.0], 
[6.0, 3.0, 2.0], [2.0, 4.0, 7.0]], [[4.0, 1.0, 1.0], [6.0, 5.0, 8.0], [1.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CLPN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 2.0], [3.0, 8.0, 3.0], [2.0, 5.0, 2.0]], [[4.0, 7.0, 8.0], [2.0, 2.0, 2.0], [2.0, 4.0, 
6.0]], [[1.0, 8.0, 8.0], [3.0, 3.0, 5.0], [1.0, 5.0, 5.0]]], [[[7.0, 3.0, 2.0], [3.0, 7.0, 3.0], [3.0, 2.0, 4.0]], [[1.0, 4.0, 
5.0], [7.0, 6.0, 6.0], [3.0, 7.0, 2.0]], [[3.0, 2.0, 1.0], [4.0, 2.0, 3.0], [6.0, 5.0, 5.0]]], [[[5.0, 5.0, 3.0], [8.0, 6.0, 
1.0], [7.0, 5.0, 7.0]], [[3.0, 1.0, 6.0], [4.0, 5.0, 5.0], [8.0, 7.0, 2.0]], [[7.0, 8.0, 5.0], [6.0, 3.0, 4.0], [4.0, 6.0, 
7.0]]], [[[8.0, 1.0, 7.0], [8.0, 6.0, 1.0], [3.0, 1.0, 3.0]], [[5.0, 2.0, 5.0], [7.0, 8.0, 4.0], [2.0, 3.0, 5.0]], [[7.0, 4.0, 
6.0], [6.0, 4.0, 7.0], [5.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 2.0], [8.0, 5.0, 2.0], [1.0, 5.0, 7.0]], [[2.0, 4.0, 2.0], [5.0, 6.0, 
2.0], [1.0, 2.0, 5.0]], [[2.0, 4.0, 4.0], [7.0, 5.0, 5.0], [6.0, 2.0, 3.0]]], [[[5.0, 5.0, 1.0], [4.0, 3.0, 5.0], [8.0, 2.0, 
8.0]], [[1.0, 1.0, 6.0], [4.0, 4.0, 5.0], [8.0, 1.0, 6.0]], [[4.0, 6.0, 3.0], [5.0, 7.0, 3.0], [3.0, 8.0, 8.0]]], [[[1.0, 1.0, 
6.0], [3.0, 3.0, 4.0], [3.0, 7.0, 3.0]], [[5.0, 7.0, 6.0], [1.0, 1.0, 7.0], [6.0, 7.0, 8.0]], [[7.0, 4.0, 6.0], [5.0, 2.0, 
2.0], [1.0, 5.0, 4.0]]], [[[8.0, 7.0, 8.0], [5.0, 8.0, 3.0], [7.0, 1.0, 8.0]], [[4.0, 8.0, 1.0], [2.0, 5.0, 7.0], [6.0, 2.0, 
4.0]], [[7.0, 3.0, 3.0], [4.0, 1.0, 7.0], [6.0, 3.0, 6.0]]]]))
      val=Data(numpy.array([[[1.0, 1.0, 8.0]], [[6.0, 2.0, 1.0]], [[2.0, 6.0, 7.0]], [[3.0, 1.0, 7.0]]]),self.functionspace)
      arg[:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 2.0], [3.0, 8.0, 3.0], [2.0, 5.0, 2.0]], [[4.0, 7.0, 8.0], [2.0, 2.0, 2.0], [2.0, 
4.0, 6.0]], [[1.0, 8.0, 8.0], [1.0, 1.0, 8.0], [1.0, 5.0, 5.0]]], [[[7.0, 3.0, 2.0], [3.0, 7.0, 3.0], [3.0, 2.0, 4.0]], [[1.0, 
4.0, 5.0], [7.0, 6.0, 6.0], [3.0, 7.0, 2.0]], [[3.0, 2.0, 1.0], [6.0, 2.0, 1.0], [6.0, 5.0, 5.0]]], [[[5.0, 5.0, 3.0], [8.0, 
6.0, 1.0], [7.0, 5.0, 7.0]], [[3.0, 1.0, 6.0], [4.0, 5.0, 5.0], [8.0, 7.0, 2.0]], [[7.0, 8.0, 5.0], [2.0, 6.0, 7.0], [4.0, 6.0, 
7.0]]], [[[8.0, 1.0, 7.0], [8.0, 6.0, 1.0], [3.0, 1.0, 3.0]], [[5.0, 2.0, 5.0], [7.0, 8.0, 4.0], [2.0, 3.0, 5.0]], [[7.0, 4.0, 
6.0], [3.0, 1.0, 7.0], [5.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 2.0], [8.0, 5.0, 2.0], [1.0, 5.0, 7.0]], 
[[2.0, 4.0, 2.0], [5.0, 6.0, 2.0], [1.0, 2.0, 5.0]], [[2.0, 4.0, 4.0], [1.0, 1.0, 8.0], [6.0, 2.0, 3.0]]], [[[5.0, 5.0, 1.0], 
[4.0, 3.0, 5.0], [8.0, 2.0, 8.0]], [[1.0, 1.0, 6.0], [4.0, 4.0, 5.0], [8.0, 1.0, 6.0]], [[4.0, 6.0, 3.0], [6.0, 2.0, 1.0], 
[3.0, 8.0, 8.0]]], [[[1.0, 1.0, 6.0], [3.0, 3.0, 4.0], [3.0, 7.0, 3.0]], [[5.0, 7.0, 6.0], [1.0, 1.0, 7.0], [6.0, 7.0, 8.0]], 
[[7.0, 4.0, 6.0], [2.0, 6.0, 7.0], [1.0, 5.0, 4.0]]], [[[8.0, 7.0, 8.0], [5.0, 8.0, 3.0], [7.0, 1.0, 8.0]], [[4.0, 8.0, 1.0], 
[2.0, 5.0, 7.0], [6.0, 2.0, 4.0]], [[7.0, 3.0, 3.0], [3.0, 1.0, 7.0], [6.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CLPN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 8.0], [1.0, 8.0, 6.0], [6.0, 2.0, 3.0]], [[3.0, 3.0, 8.0], [5.0, 6.0, 5.0], [4.0, 7.0, 
8.0]], [[8.0, 7.0, 5.0], [5.0, 8.0, 8.0], [1.0, 8.0, 3.0]]], [[[6.0, 4.0, 2.0], [4.0, 4.0, 3.0], [5.0, 1.0, 5.0]], [[3.0, 2.0, 
8.0], [7.0, 7.0, 7.0], [8.0, 8.0, 1.0]], [[6.0, 8.0, 5.0], [1.0, 5.0, 3.0], [4.0, 3.0, 5.0]]], [[[6.0, 8.0, 2.0], [6.0, 6.0, 
6.0], [2.0, 8.0, 1.0]], [[5.0, 8.0, 6.0], [1.0, 8.0, 7.0], [2.0, 8.0, 6.0]], [[2.0, 8.0, 7.0], [5.0, 1.0, 1.0], [2.0, 8.0, 
6.0]]], [[[6.0, 3.0, 7.0], [7.0, 8.0, 5.0], [1.0, 8.0, 2.0]], [[4.0, 4.0, 8.0], [5.0, 3.0, 4.0], [8.0, 4.0, 4.0]], [[1.0, 6.0, 
4.0], [8.0, 5.0, 6.0], [8.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 8.0], [8.0, 6.0, 8.0], [6.0, 1.0, 2.0]], [[4.0, 1.0, 7.0], [3.0, 2.0, 
3.0], [8.0, 1.0, 3.0]], [[4.0, 1.0, 4.0], [8.0, 7.0, 4.0], [5.0, 5.0, 4.0]]], [[[5.0, 5.0, 7.0], [3.0, 7.0, 8.0], [6.0, 5.0, 
6.0]], [[1.0, 2.0, 8.0], [3.0, 8.0, 2.0], [5.0, 3.0, 7.0]], [[2.0, 4.0, 3.0], [8.0, 2.0, 3.0], [4.0, 5.0, 1.0]]], [[[3.0, 3.0, 
3.0], [1.0, 3.0, 7.0], [5.0, 3.0, 8.0]], [[8.0, 7.0, 3.0], [1.0, 1.0, 1.0], [3.0, 8.0, 4.0]], [[4.0, 8.0, 1.0], [8.0, 7.0, 
8.0], [6.0, 4.0, 5.0]]], [[[2.0, 5.0, 6.0], [7.0, 3.0, 7.0], [5.0, 4.0, 2.0]], [[5.0, 7.0, 3.0], [5.0, 3.0, 6.0], [7.0, 6.0, 
3.0]], [[1.0, 8.0, 4.0], [4.0, 1.0, 6.0], [2.0, 3.0, 2.0]]]]))
      val=Data(numpy.array([[[7.0, 7.0, 5.0]], [[6.0, 7.0, 3.0]], [[3.0, 7.0, 6.0]], [[2.0, 7.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 8.0, 2.0]], [[4.0, 3.0, 2.0]], [[5.0, 8.0, 2.0]], [[3.0, 4.0, 3.0]]]))
      arg[:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [1.0, 8.0, 6.0], [6.0, 2.0, 3.0]], [[3.0, 3.0, 8.0], [5.0, 6.0, 5.0], [4.0, 
7.0, 8.0]], [[8.0, 7.0, 5.0], [7.0, 7.0, 5.0], [1.0, 8.0, 3.0]]], [[[6.0, 4.0, 2.0], [4.0, 4.0, 3.0], [5.0, 1.0, 5.0]], [[3.0, 
2.0, 8.0], [7.0, 7.0, 7.0], [8.0, 8.0, 1.0]], [[6.0, 8.0, 5.0], [6.0, 7.0, 3.0], [4.0, 3.0, 5.0]]], [[[6.0, 8.0, 2.0], [6.0, 
6.0, 6.0], [2.0, 8.0, 1.0]], [[5.0, 8.0, 6.0], [1.0, 8.0, 7.0], [2.0, 8.0, 6.0]], [[2.0, 8.0, 7.0], [3.0, 7.0, 6.0], [2.0, 8.0, 
6.0]]], [[[6.0, 3.0, 7.0], [7.0, 8.0, 5.0], [1.0, 8.0, 2.0]], [[4.0, 4.0, 8.0], [5.0, 3.0, 4.0], [8.0, 4.0, 4.0]], [[1.0, 6.0, 
4.0], [2.0, 7.0, 5.0], [8.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 8.0], [8.0, 6.0, 8.0], [6.0, 1.0, 2.0]], 
[[4.0, 1.0, 7.0], [3.0, 2.0, 3.0], [8.0, 1.0, 3.0]], [[4.0, 1.0, 4.0], [1.0, 8.0, 2.0], [5.0, 5.0, 4.0]]], [[[5.0, 5.0, 7.0], 
[3.0, 7.0, 8.0], [6.0, 5.0, 6.0]], [[1.0, 2.0, 8.0], [3.0, 8.0, 2.0], [5.0, 3.0, 7.0]], [[2.0, 4.0, 3.0], [4.0, 3.0, 2.0], 
[4.0, 5.0, 1.0]]], [[[3.0, 3.0, 3.0], [1.0, 3.0, 7.0], [5.0, 3.0, 8.0]], [[8.0, 7.0, 3.0], [1.0, 1.0, 1.0], [3.0, 8.0, 4.0]], 
[[4.0, 8.0, 1.0], [5.0, 8.0, 2.0], [6.0, 4.0, 5.0]]], [[[2.0, 5.0, 6.0], [7.0, 3.0, 7.0], [5.0, 4.0, 2.0]], [[5.0, 7.0, 3.0], 
[5.0, 3.0, 6.0], [7.0, 6.0, 3.0]], [[1.0, 8.0, 4.0], [3.0, 4.0, 3.0], [2.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CLPN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 8.0], [6.0, 1.0, 4.0], [1.0, 3.0, 1.0]], [[2.0, 2.0, 7.0], [4.0, 7.0, 3.0], [4.0, 3.0, 
5.0]], [[7.0, 1.0, 2.0], [6.0, 6.0, 2.0], [6.0, 1.0, 7.0]]], [[[8.0, 2.0, 5.0], [2.0, 1.0, 5.0], [5.0, 1.0, 1.0]], [[2.0, 8.0, 
5.0], [5.0, 6.0, 8.0], [8.0, 7.0, 3.0]], [[7.0, 5.0, 6.0], [3.0, 5.0, 3.0], [6.0, 1.0, 3.0]]], [[[6.0, 1.0, 6.0], [1.0, 5.0, 
6.0], [5.0, 2.0, 5.0]], [[2.0, 6.0, 6.0], [6.0, 4.0, 2.0], [5.0, 2.0, 5.0]], [[7.0, 1.0, 4.0], [3.0, 1.0, 6.0], [3.0, 2.0, 
4.0]]], [[[5.0, 8.0, 3.0], [4.0, 1.0, 4.0], [8.0, 2.0, 6.0]], [[5.0, 5.0, 8.0], [4.0, 6.0, 8.0], [3.0, 2.0, 6.0]], [[6.0, 8.0, 
7.0], [3.0, 1.0, 6.0], [6.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 5.0], [1.0, 6.0, 8.0], [2.0, 4.0, 8.0]], [[8.0, 6.0, 2.0], [6.0, 3.0, 
2.0], [1.0, 2.0, 1.0]], [[8.0, 6.0, 7.0], [6.0, 6.0, 7.0], [2.0, 7.0, 2.0]]], [[[1.0, 3.0, 7.0], [4.0, 1.0, 7.0], [5.0, 7.0, 
3.0]], [[6.0, 3.0, 4.0], [1.0, 1.0, 2.0], [5.0, 5.0, 2.0]], [[2.0, 8.0, 6.0], [6.0, 6.0, 7.0], [1.0, 3.0, 4.0]]], [[[5.0, 5.0, 
8.0], [6.0, 8.0, 8.0], [5.0, 5.0, 2.0]], [[2.0, 4.0, 3.0], [3.0, 4.0, 4.0], [4.0, 1.0, 7.0]], [[8.0, 4.0, 4.0], [7.0, 7.0, 
1.0], [3.0, 5.0, 7.0]]], [[[4.0, 8.0, 3.0], [6.0, 8.0, 1.0], [5.0, 5.0, 1.0]], [[4.0, 7.0, 6.0], [3.0, 8.0, 2.0], [6.0, 6.0, 
4.0]], [[2.0, 6.0, 6.0], [4.0, 6.0, 7.0], [1.0, 6.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 1.0, 3.0]], [[6.0, 8.0, 3.0]], [[6.0, 1.0, 4.0]], [[6.0, 6.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[4.0, 1.0, 1.0]], [[5.0, 5.0, 8.0]], [[8.0, 1.0, 7.0]], [[5.0, 4.0, 5.0]]])
      arg[:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 8.0], [6.0, 1.0, 4.0], [1.0, 3.0, 1.0]], [[2.0, 2.0, 7.0], [4.0, 7.0, 3.0], [4.0, 
3.0, 5.0]], [[7.0, 1.0, 2.0], [8.0, 1.0, 3.0], [6.0, 1.0, 7.0]]], [[[8.0, 2.0, 5.0], [2.0, 1.0, 5.0], [5.0, 1.0, 1.0]], [[2.0, 
8.0, 5.0], [5.0, 6.0, 8.0], [8.0, 7.0, 3.0]], [[7.0, 5.0, 6.0], [6.0, 8.0, 3.0], [6.0, 1.0, 3.0]]], [[[6.0, 1.0, 6.0], [1.0, 
5.0, 6.0], [5.0, 2.0, 5.0]], [[2.0, 6.0, 6.0], [6.0, 4.0, 2.0], [5.0, 2.0, 5.0]], [[7.0, 1.0, 4.0], [6.0, 1.0, 4.0], [3.0, 2.0, 
4.0]]], [[[5.0, 8.0, 3.0], [4.0, 1.0, 4.0], [8.0, 2.0, 6.0]], [[5.0, 5.0, 8.0], [4.0, 6.0, 8.0], [3.0, 2.0, 6.0]], [[6.0, 8.0, 
7.0], [6.0, 6.0, 4.0], [6.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 5.0], [1.0, 6.0, 8.0], [2.0, 4.0, 8.0]], 
[[8.0, 6.0, 2.0], [6.0, 3.0, 2.0], [1.0, 2.0, 1.0]], [[8.0, 6.0, 7.0], [4.0, 1.0, 1.0], [2.0, 7.0, 2.0]]], [[[1.0, 3.0, 7.0], 
[4.0, 1.0, 7.0], [5.0, 7.0, 3.0]], [[6.0, 3.0, 4.0], [1.0, 1.0, 2.0], [5.0, 5.0, 2.0]], [[2.0, 8.0, 6.0], [5.0, 5.0, 8.0], 
[1.0, 3.0, 4.0]]], [[[5.0, 5.0, 8.0], [6.0, 8.0, 8.0], [5.0, 5.0, 2.0]], [[2.0, 4.0, 3.0], [3.0, 4.0, 4.0], [4.0, 1.0, 7.0]], 
[[8.0, 4.0, 4.0], [8.0, 1.0, 7.0], [3.0, 5.0, 7.0]]], [[[4.0, 8.0, 3.0], [6.0, 8.0, 1.0], [5.0, 5.0, 1.0]], [[4.0, 7.0, 6.0], 
[3.0, 8.0, 2.0], [6.0, 6.0, 4.0]], [[2.0, 6.0, 6.0], [5.0, 4.0, 5.0], [1.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UUPN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 2.0], [1.0, 8.0, 6.0], [5.0, 1.0, 6.0]], [[4.0, 6.0, 7.0], [6.0, 2.0, 7.0], [8.0, 2.0, 
7.0]], [[7.0, 6.0, 7.0], [7.0, 5.0, 7.0], [8.0, 1.0, 2.0]]], [[[2.0, 1.0, 4.0], [2.0, 8.0, 8.0], [6.0, 6.0, 7.0]], [[8.0, 5.0, 
2.0], [1.0, 7.0, 5.0], [1.0, 3.0, 8.0]], [[7.0, 5.0, 1.0], [1.0, 7.0, 7.0], [6.0, 4.0, 8.0]]], [[[5.0, 7.0, 6.0], [1.0, 8.0, 
5.0], [4.0, 8.0, 2.0]], [[7.0, 3.0, 7.0], [5.0, 3.0, 1.0], [5.0, 5.0, 1.0]], [[3.0, 6.0, 2.0], [6.0, 7.0, 8.0], [7.0, 7.0, 
8.0]]], [[[4.0, 4.0, 8.0], [8.0, 3.0, 8.0], [5.0, 8.0, 4.0]], [[2.0, 8.0, 2.0], [6.0, 6.0, 4.0], [1.0, 1.0, 1.0]], [[5.0, 7.0, 
8.0], [7.0, 4.0, 8.0], [4.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 6.0], [5.0, 1.0, 5.0], [2.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [6.0, 6.0, 
3.0], [5.0, 5.0, 8.0]], [[6.0, 8.0, 7.0], [2.0, 3.0, 7.0], [3.0, 6.0, 5.0]]], [[[7.0, 1.0, 1.0], [3.0, 6.0, 8.0], [6.0, 8.0, 
7.0]], [[4.0, 7.0, 7.0], [3.0, 5.0, 6.0], [2.0, 7.0, 8.0]], [[7.0, 3.0, 2.0], [3.0, 8.0, 5.0], [1.0, 2.0, 4.0]]], [[[1.0, 8.0, 
4.0], [5.0, 4.0, 2.0], [1.0, 6.0, 5.0]], [[1.0, 7.0, 8.0], [8.0, 4.0, 4.0], [7.0, 1.0, 6.0]], [[8.0, 7.0, 6.0], [6.0, 8.0, 
5.0], [3.0, 5.0, 8.0]]], [[[5.0, 5.0, 5.0], [2.0, 1.0, 8.0], [8.0, 6.0, 4.0]], [[5.0, 6.0, 8.0], [6.0, 3.0, 5.0], [1.0, 1.0, 
7.0]], [[8.0, 7.0, 5.0], [1.0, 6.0, 3.0], [1.0, 7.0, 5.0]]]]))
      val=numpy.array([[[7.0, 4.0, 5.0], [1.0, 7.0, 7.0]], [[8.0, 1.0, 3.0], [7.0, 3.0, 5.0]], [[8.0, 3.0, 6.0], [6.0, 6.0, 
4.0]]])
      arg[:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 2.0], [7.0, 4.0, 5.0], [5.0, 1.0, 6.0]], [[4.0, 6.0, 7.0], [1.0, 7.0, 7.0], [8.0, 
2.0, 7.0]], [[7.0, 6.0, 7.0], [7.0, 5.0, 7.0], [8.0, 1.0, 2.0]]], [[[2.0, 1.0, 4.0], [8.0, 1.0, 3.0], [6.0, 6.0, 7.0]], [[8.0, 
5.0, 2.0], [7.0, 3.0, 5.0], [1.0, 3.0, 8.0]], [[7.0, 5.0, 1.0], [1.0, 7.0, 7.0], [6.0, 4.0, 8.0]]], [[[5.0, 7.0, 6.0], [8.0, 
3.0, 6.0], [4.0, 8.0, 2.0]], [[7.0, 3.0, 7.0], [6.0, 6.0, 4.0], [5.0, 5.0, 1.0]], [[3.0, 6.0, 2.0], [6.0, 7.0, 8.0], [7.0, 7.0, 
8.0]]], [[[4.0, 4.0, 8.0], [8.0, 3.0, 8.0], [5.0, 8.0, 4.0]], [[2.0, 8.0, 2.0], [6.0, 6.0, 4.0], [1.0, 1.0, 1.0]], [[5.0, 7.0, 
8.0], [7.0, 4.0, 8.0], [4.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 6.0], [7.0, 4.0, 5.0], [2.0, 4.0, 5.0]], 
[[7.0, 4.0, 3.0], [1.0, 7.0, 7.0], [5.0, 5.0, 8.0]], [[6.0, 8.0, 7.0], [2.0, 3.0, 7.0], [3.0, 6.0, 5.0]]], [[[7.0, 1.0, 1.0], 
[8.0, 1.0, 3.0], [6.0, 8.0, 7.0]], [[4.0, 7.0, 7.0], [7.0, 3.0, 5.0], [2.0, 7.0, 8.0]], [[7.0, 3.0, 2.0], [3.0, 8.0, 5.0], 
[1.0, 2.0, 4.0]]], [[[1.0, 8.0, 4.0], [8.0, 3.0, 6.0], [1.0, 6.0, 5.0]], [[1.0, 7.0, 8.0], [6.0, 6.0, 4.0], [7.0, 1.0, 6.0]], 
[[8.0, 7.0, 6.0], [6.0, 8.0, 5.0], [3.0, 5.0, 8.0]]], [[[5.0, 5.0, 5.0], [2.0, 1.0, 8.0], [8.0, 6.0, 4.0]], [[5.0, 6.0, 8.0], 
[6.0, 3.0, 5.0], [1.0, 1.0, 7.0]], [[8.0, 7.0, 5.0], [1.0, 6.0, 3.0], [1.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UUPN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 5.0], [4.0, 6.0, 6.0], [3.0, 1.0, 4.0]], [[7.0, 3.0, 1.0], [4.0, 7.0, 4.0], [5.0, 8.0, 
1.0]], [[3.0, 2.0, 6.0], [1.0, 5.0, 7.0], [4.0, 8.0, 8.0]]], [[[8.0, 1.0, 2.0], [1.0, 3.0, 6.0], [1.0, 2.0, 1.0]], [[4.0, 4.0, 
1.0], [4.0, 8.0, 8.0], [6.0, 7.0, 1.0]], [[5.0, 4.0, 6.0], [6.0, 5.0, 7.0], [3.0, 1.0, 8.0]]], [[[6.0, 5.0, 8.0], [1.0, 4.0, 
8.0], [7.0, 4.0, 4.0]], [[1.0, 7.0, 2.0], [8.0, 8.0, 8.0], [3.0, 6.0, 2.0]], [[5.0, 1.0, 6.0], [3.0, 3.0, 5.0], [6.0, 4.0, 
3.0]]], [[[3.0, 3.0, 1.0], [3.0, 8.0, 6.0], [2.0, 7.0, 4.0]], [[1.0, 4.0, 6.0], [4.0, 1.0, 1.0], [2.0, 1.0, 7.0]], [[1.0, 5.0, 
5.0], [8.0, 1.0, 7.0], [5.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 6.0], [7.0, 5.0, 2.0], [3.0, 6.0, 6.0]], [[5.0, 2.0, 1.0], [1.0, 1.0, 
1.0], [3.0, 2.0, 1.0]], [[6.0, 8.0, 7.0], [6.0, 5.0, 8.0], [7.0, 2.0, 7.0]]], [[[3.0, 7.0, 5.0], [8.0, 6.0, 8.0], [4.0, 6.0, 
6.0]], [[2.0, 8.0, 1.0], [1.0, 1.0, 5.0], [3.0, 7.0, 7.0]], [[8.0, 4.0, 5.0], [6.0, 5.0, 1.0], [5.0, 8.0, 2.0]]], [[[1.0, 2.0, 
2.0], [4.0, 7.0, 5.0], [5.0, 3.0, 8.0]], [[8.0, 6.0, 5.0], [8.0, 6.0, 5.0], [3.0, 8.0, 2.0]], [[6.0, 8.0, 5.0], [5.0, 8.0, 
3.0], [1.0, 7.0, 1.0]]], [[[3.0, 6.0, 8.0], [3.0, 6.0, 4.0], [6.0, 5.0, 4.0]], [[4.0, 2.0, 6.0], [5.0, 3.0, 8.0], [3.0, 6.0, 
7.0]], [[6.0, 5.0, 5.0], [3.0, 4.0, 6.0], [5.0, 1.0, 4.0]]]]))
      val=Data(numpy.array([[[2.0, 8.0, 5.0], [4.0, 5.0, 2.0]], [[4.0, 2.0, 6.0], [6.0, 2.0, 2.0]], [[8.0, 8.0, 7.0], [4.0, 
3.0, 2.0]]]),self.functionspace)
      arg[:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 5.0], [2.0, 8.0, 5.0], [3.0, 1.0, 4.0]], [[7.0, 3.0, 1.0], [4.0, 5.0, 2.0], [5.0, 
8.0, 1.0]], [[3.0, 2.0, 6.0], [1.0, 5.0, 7.0], [4.0, 8.0, 8.0]]], [[[8.0, 1.0, 2.0], [4.0, 2.0, 6.0], [1.0, 2.0, 1.0]], [[4.0, 
4.0, 1.0], [6.0, 2.0, 2.0], [6.0, 7.0, 1.0]], [[5.0, 4.0, 6.0], [6.0, 5.0, 7.0], [3.0, 1.0, 8.0]]], [[[6.0, 5.0, 8.0], [8.0, 
8.0, 7.0], [7.0, 4.0, 4.0]], [[1.0, 7.0, 2.0], [4.0, 3.0, 2.0], [3.0, 6.0, 2.0]], [[5.0, 1.0, 6.0], [3.0, 3.0, 5.0], [6.0, 4.0, 
3.0]]], [[[3.0, 3.0, 1.0], [3.0, 8.0, 6.0], [2.0, 7.0, 4.0]], [[1.0, 4.0, 6.0], [4.0, 1.0, 1.0], [2.0, 1.0, 7.0]], [[1.0, 5.0, 
5.0], [8.0, 1.0, 7.0], [5.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 6.0], [2.0, 8.0, 5.0], [3.0, 6.0, 6.0]], 
[[5.0, 2.0, 1.0], [4.0, 5.0, 2.0], [3.0, 2.0, 1.0]], [[6.0, 8.0, 7.0], [6.0, 5.0, 8.0], [7.0, 2.0, 7.0]]], [[[3.0, 7.0, 5.0], 
[4.0, 2.0, 6.0], [4.0, 6.0, 6.0]], [[2.0, 8.0, 1.0], [6.0, 2.0, 2.0], [3.0, 7.0, 7.0]], [[8.0, 4.0, 5.0], [6.0, 5.0, 1.0], 
[5.0, 8.0, 2.0]]], [[[1.0, 2.0, 2.0], [8.0, 8.0, 7.0], [5.0, 3.0, 8.0]], [[8.0, 6.0, 5.0], [4.0, 3.0, 2.0], [3.0, 8.0, 2.0]], 
[[6.0, 8.0, 5.0], [5.0, 8.0, 3.0], [1.0, 7.0, 1.0]]], [[[3.0, 6.0, 8.0], [3.0, 6.0, 4.0], [6.0, 5.0, 4.0]], [[4.0, 2.0, 6.0], 
[5.0, 3.0, 8.0], [3.0, 6.0, 7.0]], [[6.0, 5.0, 5.0], [3.0, 4.0, 6.0], [5.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UUPN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 7.0], [4.0, 4.0, 3.0], [8.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [5.0, 1.0, 5.0], [6.0, 5.0, 
8.0]], [[2.0, 3.0, 3.0], [1.0, 7.0, 2.0], [6.0, 8.0, 4.0]]], [[[7.0, 6.0, 5.0], [6.0, 4.0, 2.0], [8.0, 3.0, 5.0]], [[8.0, 7.0, 
3.0], [3.0, 5.0, 2.0], [3.0, 3.0, 6.0]], [[4.0, 8.0, 8.0], [7.0, 2.0, 2.0], [1.0, 7.0, 6.0]]], [[[1.0, 4.0, 7.0], [8.0, 6.0, 
5.0], [4.0, 3.0, 2.0]], [[7.0, 6.0, 4.0], [7.0, 3.0, 8.0], [2.0, 7.0, 4.0]], [[4.0, 5.0, 2.0], [5.0, 1.0, 7.0], [4.0, 6.0, 
8.0]]], [[[5.0, 1.0, 5.0], [7.0, 5.0, 5.0], [2.0, 4.0, 5.0]], [[3.0, 1.0, 5.0], [6.0, 2.0, 1.0], [5.0, 5.0, 4.0]], [[1.0, 4.0, 
3.0], [6.0, 7.0, 3.0], [4.0, 1.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 3.0], [7.0, 2.0, 8.0], [5.0, 8.0, 6.0]], [[8.0, 3.0, 4.0], [3.0, 1.0, 
3.0], [6.0, 8.0, 6.0]], [[6.0, 2.0, 6.0], [1.0, 1.0, 2.0], [8.0, 2.0, 2.0]]], [[[3.0, 4.0, 8.0], [5.0, 1.0, 4.0], [1.0, 4.0, 
4.0]], [[2.0, 7.0, 5.0], [2.0, 6.0, 5.0], [6.0, 7.0, 7.0]], [[6.0, 8.0, 8.0], [1.0, 2.0, 4.0], [6.0, 2.0, 2.0]]], [[[8.0, 6.0, 
6.0], [1.0, 7.0, 2.0], [4.0, 2.0, 1.0]], [[7.0, 3.0, 4.0], [5.0, 4.0, 6.0], [3.0, 4.0, 8.0]], [[2.0, 5.0, 1.0], [8.0, 3.0, 
4.0], [6.0, 3.0, 7.0]]], [[[6.0, 7.0, 5.0], [6.0, 5.0, 3.0], [4.0, 3.0, 2.0]], [[4.0, 5.0, 6.0], [8.0, 3.0, 4.0], [3.0, 4.0, 
3.0]], [[5.0, 5.0, 4.0], [1.0, 7.0, 5.0], [3.0, 1.0, 6.0]]]]))
      val=Data(numpy.array([[[6.0, 1.0, 3.0], [3.0, 5.0, 5.0]], [[8.0, 2.0, 4.0], [6.0, 8.0, 8.0]], [[5.0, 4.0, 3.0], [7.0, 
7.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 2.0, 4.0], [4.0, 6.0, 8.0]], [[1.0, 7.0, 8.0], [7.0, 5.0, 5.0]], [[1.0, 6.0, 
1.0], [8.0, 8.0, 6.0]]]))
      arg[:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 7.0], [6.0, 1.0, 3.0], [8.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [3.0, 5.0, 5.0], [6.0, 
5.0, 8.0]], [[2.0, 3.0, 3.0], [1.0, 7.0, 2.0], [6.0, 8.0, 4.0]]], [[[7.0, 6.0, 5.0], [8.0, 2.0, 4.0], [8.0, 3.0, 5.0]], [[8.0, 
7.0, 3.0], [6.0, 8.0, 8.0], [3.0, 3.0, 6.0]], [[4.0, 8.0, 8.0], [7.0, 2.0, 2.0], [1.0, 7.0, 6.0]]], [[[1.0, 4.0, 7.0], [5.0, 
4.0, 3.0], [4.0, 3.0, 2.0]], [[7.0, 6.0, 4.0], [7.0, 7.0, 3.0], [2.0, 7.0, 4.0]], [[4.0, 5.0, 2.0], [5.0, 1.0, 7.0], [4.0, 6.0, 
8.0]]], [[[5.0, 1.0, 5.0], [7.0, 5.0, 5.0], [2.0, 4.0, 5.0]], [[3.0, 1.0, 5.0], [6.0, 2.0, 1.0], [5.0, 5.0, 4.0]], [[1.0, 4.0, 
3.0], [6.0, 7.0, 3.0], [4.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 3.0], [8.0, 2.0, 4.0], [5.0, 8.0, 6.0]], 
[[8.0, 3.0, 4.0], [4.0, 6.0, 8.0], [6.0, 8.0, 6.0]], [[6.0, 2.0, 6.0], [1.0, 1.0, 2.0], [8.0, 2.0, 2.0]]], [[[3.0, 4.0, 8.0], 
[1.0, 7.0, 8.0], [1.0, 4.0, 4.0]], [[2.0, 7.0, 5.0], [7.0, 5.0, 5.0], [6.0, 7.0, 7.0]], [[6.0, 8.0, 8.0], [1.0, 2.0, 4.0], 
[6.0, 2.0, 2.0]]], [[[8.0, 6.0, 6.0], [1.0, 6.0, 1.0], [4.0, 2.0, 1.0]], [[7.0, 3.0, 4.0], [8.0, 8.0, 6.0], [3.0, 4.0, 8.0]], 
[[2.0, 5.0, 1.0], [8.0, 3.0, 4.0], [6.0, 3.0, 7.0]]], [[[6.0, 7.0, 5.0], [6.0, 5.0, 3.0], [4.0, 3.0, 2.0]], [[4.0, 5.0, 6.0], 
[8.0, 3.0, 4.0], [3.0, 4.0, 3.0]], [[5.0, 5.0, 4.0], [1.0, 7.0, 5.0], [3.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UUPN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 5.0], [1.0, 4.0, 5.0], [7.0, 7.0, 3.0]], [[5.0, 4.0, 8.0], [5.0, 3.0, 1.0], [8.0, 7.0, 
4.0]], [[8.0, 5.0, 3.0], [5.0, 4.0, 4.0], [2.0, 8.0, 5.0]]], [[[8.0, 5.0, 2.0], [7.0, 8.0, 1.0], [7.0, 2.0, 2.0]], [[5.0, 6.0, 
8.0], [3.0, 7.0, 6.0], [5.0, 4.0, 6.0]], [[2.0, 4.0, 2.0], [5.0, 5.0, 1.0], [4.0, 2.0, 5.0]]], [[[1.0, 1.0, 5.0], [4.0, 5.0, 
3.0], [2.0, 7.0, 8.0]], [[8.0, 2.0, 2.0], [1.0, 7.0, 8.0], [8.0, 1.0, 2.0]], [[6.0, 4.0, 3.0], [1.0, 6.0, 3.0], [7.0, 3.0, 
6.0]]], [[[2.0, 6.0, 4.0], [3.0, 7.0, 4.0], [1.0, 7.0, 7.0]], [[2.0, 7.0, 7.0], [1.0, 5.0, 8.0], [8.0, 3.0, 8.0]], [[1.0, 6.0, 
8.0], [5.0, 7.0, 8.0], [6.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 6.0], [8.0, 7.0, 5.0], [7.0, 2.0, 3.0]], [[4.0, 4.0, 1.0], [4.0, 2.0, 
6.0], [3.0, 3.0, 7.0]], [[8.0, 2.0, 8.0], [1.0, 2.0, 8.0], [3.0, 4.0, 1.0]]], [[[5.0, 4.0, 5.0], [6.0, 8.0, 7.0], [1.0, 2.0, 
3.0]], [[5.0, 1.0, 6.0], [8.0, 5.0, 1.0], [5.0, 4.0, 4.0]], [[2.0, 5.0, 3.0], [4.0, 7.0, 2.0], [7.0, 8.0, 3.0]]], [[[3.0, 1.0, 
4.0], [2.0, 5.0, 7.0], [6.0, 4.0, 1.0]], [[4.0, 3.0, 5.0], [5.0, 4.0, 6.0], [5.0, 5.0, 7.0]], [[2.0, 7.0, 1.0], [1.0, 8.0, 
5.0], [7.0, 6.0, 5.0]]], [[[8.0, 2.0, 8.0], [7.0, 5.0, 1.0], [2.0, 8.0, 1.0]], [[8.0, 7.0, 8.0], [3.0, 6.0, 2.0], [2.0, 2.0, 
4.0]], [[6.0, 2.0, 5.0], [1.0, 1.0, 5.0], [8.0, 4.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[6.0, 6.0, 2.0], [6.0, 6.0, 2.0]], [[6.0, 7.0, 3.0], [1.0, 3.0, 8.0]], [[6.0, 8.0, 6.0], [4.0, 
6.0, 7.0]]])+(1.-msk_val)*numpy.array([[[3.0, 8.0, 1.0], [4.0, 8.0, 6.0]], [[7.0, 3.0, 7.0], [3.0, 2.0, 2.0]], [[4.0, 1.0, 
4.0], [6.0, 2.0, 2.0]]])
      arg[:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 5.0], [6.0, 6.0, 2.0], [7.0, 7.0, 3.0]], [[5.0, 4.0, 8.0], [6.0, 6.0, 2.0], [8.0, 
7.0, 4.0]], [[8.0, 5.0, 3.0], [5.0, 4.0, 4.0], [2.0, 8.0, 5.0]]], [[[8.0, 5.0, 2.0], [6.0, 7.0, 3.0], [7.0, 2.0, 2.0]], [[5.0, 
6.0, 8.0], [1.0, 3.0, 8.0], [5.0, 4.0, 6.0]], [[2.0, 4.0, 2.0], [5.0, 5.0, 1.0], [4.0, 2.0, 5.0]]], [[[1.0, 1.0, 5.0], [6.0, 
8.0, 6.0], [2.0, 7.0, 8.0]], [[8.0, 2.0, 2.0], [4.0, 6.0, 7.0], [8.0, 1.0, 2.0]], [[6.0, 4.0, 3.0], [1.0, 6.0, 3.0], [7.0, 3.0, 
6.0]]], [[[2.0, 6.0, 4.0], [3.0, 7.0, 4.0], [1.0, 7.0, 7.0]], [[2.0, 7.0, 7.0], [1.0, 5.0, 8.0], [8.0, 3.0, 8.0]], [[1.0, 6.0, 
8.0], [5.0, 7.0, 8.0], [6.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 6.0], [3.0, 8.0, 1.0], [7.0, 2.0, 3.0]], 
[[4.0, 4.0, 1.0], [4.0, 8.0, 6.0], [3.0, 3.0, 7.0]], [[8.0, 2.0, 8.0], [1.0, 2.0, 8.0], [3.0, 4.0, 1.0]]], [[[5.0, 4.0, 5.0], 
[7.0, 3.0, 7.0], [1.0, 2.0, 3.0]], [[5.0, 1.0, 6.0], [3.0, 2.0, 2.0], [5.0, 4.0, 4.0]], [[2.0, 5.0, 3.0], [4.0, 7.0, 2.0], 
[7.0, 8.0, 3.0]]], [[[3.0, 1.0, 4.0], [4.0, 1.0, 4.0], [6.0, 4.0, 1.0]], [[4.0, 3.0, 5.0], [6.0, 2.0, 2.0], [5.0, 5.0, 7.0]], 
[[2.0, 7.0, 1.0], [1.0, 8.0, 5.0], [7.0, 6.0, 5.0]]], [[[8.0, 2.0, 8.0], [7.0, 5.0, 1.0], [2.0, 8.0, 1.0]], [[8.0, 7.0, 8.0], 
[3.0, 6.0, 2.0], [2.0, 2.0, 4.0]], [[6.0, 2.0, 5.0], [1.0, 1.0, 5.0], [8.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLPP(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 8.0], [8.0, 6.0, 5.0], [4.0, 4.0, 2.0]], [[3.0, 6.0, 4.0], [4.0, 8.0, 3.0], [5.0, 8.0, 
4.0]], [[8.0, 2.0, 8.0], [3.0, 5.0, 6.0], [2.0, 4.0, 6.0]]], [[[4.0, 6.0, 2.0], [3.0, 5.0, 5.0], [7.0, 6.0, 4.0]], [[6.0, 1.0, 
5.0], [7.0, 1.0, 1.0], [5.0, 5.0, 7.0]], [[2.0, 4.0, 8.0], [3.0, 7.0, 5.0], [6.0, 5.0, 8.0]]], [[[8.0, 4.0, 4.0], [5.0, 8.0, 
8.0], [4.0, 2.0, 4.0]], [[3.0, 5.0, 3.0], [5.0, 5.0, 6.0], [4.0, 1.0, 6.0]], [[8.0, 8.0, 3.0], [6.0, 7.0, 6.0], [7.0, 6.0, 
5.0]]], [[[2.0, 3.0, 2.0], [1.0, 2.0, 3.0], [3.0, 3.0, 6.0]], [[3.0, 7.0, 8.0], [4.0, 4.0, 5.0], [7.0, 4.0, 1.0]], [[4.0, 8.0, 
3.0], [1.0, 1.0, 3.0], [2.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 6.0], [4.0, 3.0, 4.0], [8.0, 1.0, 4.0]], [[3.0, 3.0, 7.0], [4.0, 2.0, 
2.0], [5.0, 8.0, 2.0]], [[3.0, 4.0, 3.0], [3.0, 3.0, 3.0], [1.0, 1.0, 7.0]]], [[[6.0, 4.0, 3.0], [3.0, 2.0, 4.0], [3.0, 8.0, 
7.0]], [[1.0, 4.0, 1.0], [1.0, 5.0, 5.0], [7.0, 4.0, 7.0]], [[3.0, 8.0, 6.0], [4.0, 2.0, 8.0], [4.0, 4.0, 2.0]]], [[[4.0, 3.0, 
2.0], [8.0, 1.0, 6.0], [6.0, 7.0, 3.0]], [[8.0, 2.0, 2.0], [2.0, 6.0, 6.0], [2.0, 5.0, 4.0]], [[8.0, 4.0, 7.0], [2.0, 7.0, 
7.0], [5.0, 6.0, 8.0]]], [[[7.0, 5.0, 5.0], [1.0, 8.0, 8.0], [6.0, 4.0, 7.0]], [[7.0, 6.0, 5.0], [4.0, 2.0, 6.0], [6.0, 8.0, 
1.0]], [[5.0, 3.0, 5.0], [5.0, 8.0, 6.0], [2.0, 3.0, 2.0]]]]))
      val=numpy.array([8.0])
      arg[2,2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 8.0], [8.0, 6.0, 5.0], [4.0, 4.0, 2.0]], [[3.0, 6.0, 4.0], [4.0, 8.0, 3.0], [5.0, 
8.0, 4.0]], [[8.0, 2.0, 8.0], [3.0, 5.0, 6.0], [2.0, 4.0, 6.0]]], [[[4.0, 6.0, 2.0], [3.0, 5.0, 5.0], [7.0, 6.0, 4.0]], [[6.0, 
1.0, 5.0], [7.0, 1.0, 1.0], [5.0, 5.0, 7.0]], [[2.0, 4.0, 8.0], [3.0, 7.0, 5.0], [6.0, 5.0, 8.0]]], [[[8.0, 4.0, 4.0], [5.0, 
8.0, 8.0], [4.0, 2.0, 4.0]], [[3.0, 5.0, 3.0], [5.0, 5.0, 6.0], [4.0, 1.0, 6.0]], [[8.0, 8.0, 3.0], [6.0, 8.0, 6.0], [7.0, 6.0, 
5.0]]], [[[2.0, 3.0, 2.0], [1.0, 2.0, 3.0], [3.0, 3.0, 6.0]], [[3.0, 7.0, 8.0], [4.0, 4.0, 5.0], [7.0, 4.0, 1.0]], [[4.0, 8.0, 
3.0], [1.0, 1.0, 3.0], [2.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 6.0], [4.0, 3.0, 4.0], [8.0, 1.0, 4.0]], 
[[3.0, 3.0, 7.0], [4.0, 2.0, 2.0], [5.0, 8.0, 2.0]], [[3.0, 4.0, 3.0], [3.0, 3.0, 3.0], [1.0, 1.0, 7.0]]], [[[6.0, 4.0, 3.0], 
[3.0, 2.0, 4.0], [3.0, 8.0, 7.0]], [[1.0, 4.0, 1.0], [1.0, 5.0, 5.0], [7.0, 4.0, 7.0]], [[3.0, 8.0, 6.0], [4.0, 2.0, 8.0], 
[4.0, 4.0, 2.0]]], [[[4.0, 3.0, 2.0], [8.0, 1.0, 6.0], [6.0, 7.0, 3.0]], [[8.0, 2.0, 2.0], [2.0, 6.0, 6.0], [2.0, 5.0, 4.0]], 
[[8.0, 4.0, 7.0], [2.0, 8.0, 7.0], [5.0, 6.0, 8.0]]], [[[7.0, 5.0, 5.0], [1.0, 8.0, 8.0], [6.0, 4.0, 7.0]], [[7.0, 6.0, 5.0], 
[4.0, 2.0, 6.0], [6.0, 8.0, 1.0]], [[5.0, 3.0, 5.0], [5.0, 8.0, 6.0], [2.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLPP(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 8.0], [7.0, 8.0, 7.0], [2.0, 6.0, 7.0]], [[7.0, 1.0, 5.0], [3.0, 6.0, 1.0], [7.0, 4.0, 
7.0]], [[7.0, 6.0, 8.0], [6.0, 3.0, 4.0], [7.0, 4.0, 4.0]]], [[[2.0, 7.0, 7.0], [2.0, 6.0, 4.0], [7.0, 2.0, 2.0]], [[3.0, 8.0, 
4.0], [1.0, 1.0, 1.0], [8.0, 4.0, 5.0]], [[6.0, 4.0, 7.0], [3.0, 8.0, 4.0], [6.0, 7.0, 5.0]]], [[[1.0, 5.0, 6.0], [8.0, 7.0, 
7.0], [4.0, 1.0, 1.0]], [[2.0, 7.0, 2.0], [3.0, 8.0, 6.0], [7.0, 7.0, 3.0]], [[8.0, 7.0, 2.0], [2.0, 8.0, 6.0], [8.0, 8.0, 
7.0]]], [[[1.0, 1.0, 5.0], [2.0, 7.0, 1.0], [1.0, 6.0, 4.0]], [[3.0, 8.0, 1.0], [4.0, 2.0, 7.0], [8.0, 1.0, 5.0]], [[6.0, 4.0, 
3.0], [2.0, 3.0, 5.0], [5.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 5.0], [1.0, 2.0, 7.0], [7.0, 4.0, 5.0]], [[4.0, 5.0, 5.0], [8.0, 7.0, 
6.0], [7.0, 4.0, 6.0]], [[3.0, 4.0, 8.0], [5.0, 1.0, 8.0], [4.0, 1.0, 1.0]]], [[[4.0, 1.0, 3.0], [8.0, 3.0, 1.0], [7.0, 7.0, 
3.0]], [[1.0, 8.0, 2.0], [3.0, 7.0, 8.0], [1.0, 8.0, 8.0]], [[1.0, 1.0, 4.0], [5.0, 4.0, 4.0], [6.0, 6.0, 3.0]]], [[[7.0, 6.0, 
7.0], [3.0, 6.0, 8.0], [5.0, 4.0, 6.0]], [[7.0, 8.0, 3.0], [3.0, 5.0, 1.0], [4.0, 2.0, 3.0]], [[1.0, 4.0, 6.0], [3.0, 3.0, 
7.0], [2.0, 5.0, 2.0]]], [[[2.0, 6.0, 2.0], [2.0, 6.0, 7.0], [4.0, 5.0, 5.0]], [[4.0, 6.0, 2.0], [3.0, 8.0, 8.0], [2.0, 3.0, 
7.0]], [[4.0, 2.0, 6.0], [1.0, 8.0, 1.0], [4.0, 5.0, 8.0]]]]))
      val=Data(numpy.array([4.0]),self.functionspace)
      arg[2,2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 8.0], [7.0, 8.0, 7.0], [2.0, 6.0, 7.0]], [[7.0, 1.0, 5.0], [3.0, 6.0, 1.0], [7.0, 
4.0, 7.0]], [[7.0, 6.0, 8.0], [6.0, 3.0, 4.0], [7.0, 4.0, 4.0]]], [[[2.0, 7.0, 7.0], [2.0, 6.0, 4.0], [7.0, 2.0, 2.0]], [[3.0, 
8.0, 4.0], [1.0, 1.0, 1.0], [8.0, 4.0, 5.0]], [[6.0, 4.0, 7.0], [3.0, 8.0, 4.0], [6.0, 7.0, 5.0]]], [[[1.0, 5.0, 6.0], [8.0, 
7.0, 7.0], [4.0, 1.0, 1.0]], [[2.0, 7.0, 2.0], [3.0, 8.0, 6.0], [7.0, 7.0, 3.0]], [[8.0, 7.0, 2.0], [2.0, 4.0, 6.0], [8.0, 8.0, 
7.0]]], [[[1.0, 1.0, 5.0], [2.0, 7.0, 1.0], [1.0, 6.0, 4.0]], [[3.0, 8.0, 1.0], [4.0, 2.0, 7.0], [8.0, 1.0, 5.0]], [[6.0, 4.0, 
3.0], [2.0, 3.0, 5.0], [5.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 5.0], [1.0, 2.0, 7.0], [7.0, 4.0, 5.0]], 
[[4.0, 5.0, 5.0], [8.0, 7.0, 6.0], [7.0, 4.0, 6.0]], [[3.0, 4.0, 8.0], [5.0, 1.0, 8.0], [4.0, 1.0, 1.0]]], [[[4.0, 1.0, 3.0], 
[8.0, 3.0, 1.0], [7.0, 7.0, 3.0]], [[1.0, 8.0, 2.0], [3.0, 7.0, 8.0], [1.0, 8.0, 8.0]], [[1.0, 1.0, 4.0], [5.0, 4.0, 4.0], 
[6.0, 6.0, 3.0]]], [[[7.0, 6.0, 7.0], [3.0, 6.0, 8.0], [5.0, 4.0, 6.0]], [[7.0, 8.0, 3.0], [3.0, 5.0, 1.0], [4.0, 2.0, 3.0]], 
[[1.0, 4.0, 6.0], [3.0, 4.0, 7.0], [2.0, 5.0, 2.0]]], [[[2.0, 6.0, 2.0], [2.0, 6.0, 7.0], [4.0, 5.0, 5.0]], [[4.0, 6.0, 2.0], 
[3.0, 8.0, 8.0], [2.0, 3.0, 7.0]], [[4.0, 2.0, 6.0], [1.0, 8.0, 1.0], [4.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLPP(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 2.0], [2.0, 5.0, 4.0], [2.0, 7.0, 6.0]], [[2.0, 6.0, 2.0], [3.0, 4.0, 1.0], [2.0, 2.0, 
5.0]], [[5.0, 1.0, 4.0], [8.0, 5.0, 7.0], [4.0, 3.0, 5.0]]], [[[7.0, 4.0, 7.0], [6.0, 6.0, 8.0], [2.0, 4.0, 5.0]], [[7.0, 7.0, 
1.0], [2.0, 3.0, 1.0], [8.0, 2.0, 2.0]], [[3.0, 6.0, 8.0], [4.0, 4.0, 5.0], [6.0, 8.0, 4.0]]], [[[1.0, 4.0, 6.0], [5.0, 8.0, 
6.0], [1.0, 2.0, 1.0]], [[5.0, 2.0, 1.0], [1.0, 7.0, 7.0], [5.0, 6.0, 6.0]], [[4.0, 7.0, 2.0], [4.0, 8.0, 1.0], [6.0, 4.0, 
8.0]]], [[[2.0, 6.0, 7.0], [3.0, 8.0, 7.0], [7.0, 6.0, 6.0]], [[7.0, 3.0, 6.0], [6.0, 3.0, 2.0], [3.0, 2.0, 4.0]], [[8.0, 8.0, 
4.0], [4.0, 7.0, 6.0], [5.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 2.0], [8.0, 1.0, 2.0], [3.0, 5.0, 7.0]], [[3.0, 4.0, 7.0], [5.0, 7.0, 
4.0], [5.0, 8.0, 5.0]], [[1.0, 5.0, 1.0], [5.0, 7.0, 8.0], [8.0, 7.0, 8.0]]], [[[7.0, 3.0, 6.0], [3.0, 8.0, 1.0], [3.0, 5.0, 
3.0]], [[4.0, 6.0, 1.0], [5.0, 4.0, 3.0], [4.0, 3.0, 6.0]], [[8.0, 8.0, 6.0], [7.0, 5.0, 7.0], [3.0, 8.0, 6.0]]], [[[3.0, 3.0, 
2.0], [3.0, 4.0, 5.0], [2.0, 4.0, 2.0]], [[8.0, 4.0, 1.0], [5.0, 1.0, 2.0], [1.0, 2.0, 5.0]], [[7.0, 2.0, 5.0], [1.0, 6.0, 
2.0], [6.0, 6.0, 3.0]]], [[[4.0, 1.0, 7.0], [6.0, 6.0, 5.0], [7.0, 6.0, 4.0]], [[8.0, 2.0, 5.0], [4.0, 1.0, 4.0], [2.0, 6.0, 
1.0]], [[1.0, 2.0, 3.0], [5.0, 3.0, 6.0], [1.0, 1.0, 6.0]]]]))
      val=Data(numpy.array([1.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([1.0]))
      arg[2,2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 2.0], [2.0, 5.0, 4.0], [2.0, 7.0, 6.0]], [[2.0, 6.0, 2.0], [3.0, 4.0, 1.0], [2.0, 
2.0, 5.0]], [[5.0, 1.0, 4.0], [8.0, 5.0, 7.0], [4.0, 3.0, 5.0]]], [[[7.0, 4.0, 7.0], [6.0, 6.0, 8.0], [2.0, 4.0, 5.0]], [[7.0, 
7.0, 1.0], [2.0, 3.0, 1.0], [8.0, 2.0, 2.0]], [[3.0, 6.0, 8.0], [4.0, 4.0, 5.0], [6.0, 8.0, 4.0]]], [[[1.0, 4.0, 6.0], [5.0, 
8.0, 6.0], [1.0, 2.0, 1.0]], [[5.0, 2.0, 1.0], [1.0, 7.0, 7.0], [5.0, 6.0, 6.0]], [[4.0, 7.0, 2.0], [4.0, 1.0, 1.0], [6.0, 4.0, 
8.0]]], [[[2.0, 6.0, 7.0], [3.0, 8.0, 7.0], [7.0, 6.0, 6.0]], [[7.0, 3.0, 6.0], [6.0, 3.0, 2.0], [3.0, 2.0, 4.0]], [[8.0, 8.0, 
4.0], [4.0, 7.0, 6.0], [5.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 2.0], [8.0, 1.0, 2.0], [3.0, 5.0, 7.0]], 
[[3.0, 4.0, 7.0], [5.0, 7.0, 4.0], [5.0, 8.0, 5.0]], [[1.0, 5.0, 1.0], [5.0, 7.0, 8.0], [8.0, 7.0, 8.0]]], [[[7.0, 3.0, 6.0], 
[3.0, 8.0, 1.0], [3.0, 5.0, 3.0]], [[4.0, 6.0, 1.0], [5.0, 4.0, 3.0], [4.0, 3.0, 6.0]], [[8.0, 8.0, 6.0], [7.0, 5.0, 7.0], 
[3.0, 8.0, 6.0]]], [[[3.0, 3.0, 2.0], [3.0, 4.0, 5.0], [2.0, 4.0, 2.0]], [[8.0, 4.0, 1.0], [5.0, 1.0, 2.0], [1.0, 2.0, 5.0]], 
[[7.0, 2.0, 5.0], [1.0, 1.0, 2.0], [6.0, 6.0, 3.0]]], [[[4.0, 1.0, 7.0], [6.0, 6.0, 5.0], [7.0, 6.0, 4.0]], [[8.0, 2.0, 5.0], 
[4.0, 1.0, 4.0], [2.0, 6.0, 1.0]], [[1.0, 2.0, 3.0], [5.0, 3.0, 6.0], [1.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLPP(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 7.0], [8.0, 1.0, 4.0], [8.0, 5.0, 1.0]], [[7.0, 2.0, 1.0], [5.0, 4.0, 8.0], [5.0, 7.0, 
6.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 2.0], [3.0, 4.0, 7.0]]], [[[5.0, 3.0, 8.0], [8.0, 1.0, 3.0], [7.0, 3.0, 2.0]], [[4.0, 7.0, 
6.0], [2.0, 7.0, 5.0], [2.0, 4.0, 7.0]], [[2.0, 5.0, 2.0], [2.0, 5.0, 2.0], [8.0, 5.0, 8.0]]], [[[5.0, 4.0, 2.0], [8.0, 7.0, 
1.0], [8.0, 4.0, 6.0]], [[1.0, 6.0, 3.0], [7.0, 4.0, 4.0], [3.0, 8.0, 1.0]], [[4.0, 4.0, 4.0], [3.0, 7.0, 7.0], [8.0, 4.0, 
1.0]]], [[[1.0, 5.0, 3.0], [8.0, 1.0, 4.0], [2.0, 4.0, 3.0]], [[5.0, 6.0, 2.0], [7.0, 5.0, 1.0], [1.0, 1.0, 3.0]], [[3.0, 2.0, 
4.0], [1.0, 6.0, 7.0], [7.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 2.0], [4.0, 6.0, 2.0], [6.0, 3.0, 3.0]], [[3.0, 5.0, 7.0], [2.0, 6.0, 
4.0], [3.0, 1.0, 2.0]], [[1.0, 4.0, 5.0], [1.0, 6.0, 7.0], [5.0, 7.0, 7.0]]], [[[8.0, 5.0, 3.0], [6.0, 6.0, 4.0], [5.0, 1.0, 
3.0]], [[4.0, 8.0, 2.0], [4.0, 8.0, 8.0], [4.0, 2.0, 8.0]], [[5.0, 1.0, 6.0], [4.0, 6.0, 8.0], [5.0, 4.0, 2.0]]], [[[7.0, 6.0, 
4.0], [1.0, 3.0, 4.0], [5.0, 2.0, 7.0]], [[8.0, 8.0, 3.0], [2.0, 7.0, 1.0], [1.0, 3.0, 4.0]], [[1.0, 3.0, 3.0], [3.0, 1.0, 
4.0], [2.0, 7.0, 1.0]]], [[[8.0, 2.0, 1.0], [5.0, 6.0, 7.0], [8.0, 6.0, 6.0]], [[5.0, 5.0, 8.0], [1.0, 6.0, 1.0], [6.0, 3.0, 
3.0]], [[6.0, 4.0, 7.0], [1.0, 7.0, 6.0], [6.0, 1.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([5.0])+(1.-msk_val)*numpy.array([3.0])
      arg[2,2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 7.0], [8.0, 1.0, 4.0], [8.0, 5.0, 1.0]], [[7.0, 2.0, 1.0], [5.0, 4.0, 8.0], [5.0, 
7.0, 6.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 2.0], [3.0, 4.0, 7.0]]], [[[5.0, 3.0, 8.0], [8.0, 1.0, 3.0], [7.0, 3.0, 2.0]], [[4.0, 
7.0, 6.0], [2.0, 7.0, 5.0], [2.0, 4.0, 7.0]], [[2.0, 5.0, 2.0], [2.0, 5.0, 2.0], [8.0, 5.0, 8.0]]], [[[5.0, 4.0, 2.0], [8.0, 
7.0, 1.0], [8.0, 4.0, 6.0]], [[1.0, 6.0, 3.0], [7.0, 4.0, 4.0], [3.0, 8.0, 1.0]], [[4.0, 4.0, 4.0], [3.0, 5.0, 7.0], [8.0, 4.0, 
1.0]]], [[[1.0, 5.0, 3.0], [8.0, 1.0, 4.0], [2.0, 4.0, 3.0]], [[5.0, 6.0, 2.0], [7.0, 5.0, 1.0], [1.0, 1.0, 3.0]], [[3.0, 2.0, 
4.0], [1.0, 6.0, 7.0], [7.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 2.0], [4.0, 6.0, 2.0], [6.0, 3.0, 3.0]], 
[[3.0, 5.0, 7.0], [2.0, 6.0, 4.0], [3.0, 1.0, 2.0]], [[1.0, 4.0, 5.0], [1.0, 6.0, 7.0], [5.0, 7.0, 7.0]]], [[[8.0, 5.0, 3.0], 
[6.0, 6.0, 4.0], [5.0, 1.0, 3.0]], [[4.0, 8.0, 2.0], [4.0, 8.0, 8.0], [4.0, 2.0, 8.0]], [[5.0, 1.0, 6.0], [4.0, 6.0, 8.0], 
[5.0, 4.0, 2.0]]], [[[7.0, 6.0, 4.0], [1.0, 3.0, 4.0], [5.0, 2.0, 7.0]], [[8.0, 8.0, 3.0], [2.0, 7.0, 1.0], [1.0, 3.0, 4.0]], 
[[1.0, 3.0, 3.0], [3.0, 3.0, 4.0], [2.0, 7.0, 1.0]]], [[[8.0, 2.0, 1.0], [5.0, 6.0, 7.0], [8.0, 6.0, 6.0]], [[5.0, 5.0, 8.0], 
[1.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[6.0, 4.0, 7.0], [1.0, 7.0, 6.0], [6.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CUPN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 2.0], [2.0, 7.0, 2.0], [8.0, 2.0, 6.0]], [[5.0, 4.0, 1.0], [3.0, 1.0, 5.0], [7.0, 2.0, 
7.0]], [[3.0, 4.0, 6.0], [8.0, 1.0, 2.0], [5.0, 1.0, 3.0]]], [[[6.0, 2.0, 1.0], [3.0, 4.0, 5.0], [8.0, 3.0, 3.0]], [[2.0, 5.0, 
4.0], [4.0, 5.0, 2.0], [2.0, 2.0, 7.0]], [[6.0, 5.0, 7.0], [7.0, 2.0, 8.0], [8.0, 5.0, 3.0]]], [[[4.0, 4.0, 5.0], [3.0, 4.0, 
3.0], [5.0, 6.0, 4.0]], [[7.0, 2.0, 1.0], [1.0, 3.0, 4.0], [7.0, 8.0, 8.0]], [[3.0, 6.0, 8.0], [4.0, 8.0, 3.0], [1.0, 4.0, 
3.0]]], [[[7.0, 4.0, 1.0], [3.0, 3.0, 8.0], [3.0, 3.0, 5.0]], [[4.0, 3.0, 3.0], [3.0, 1.0, 5.0], [5.0, 3.0, 1.0]], [[1.0, 1.0, 
7.0], [5.0, 4.0, 1.0], [5.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 6.0], [2.0, 6.0, 2.0], [5.0, 8.0, 8.0]], [[2.0, 1.0, 3.0], [5.0, 7.0, 
8.0], [5.0, 3.0, 1.0]], [[8.0, 2.0, 8.0], [5.0, 3.0, 2.0], [8.0, 1.0, 4.0]]], [[[2.0, 5.0, 8.0], [8.0, 1.0, 6.0], [3.0, 5.0, 
5.0]], [[7.0, 8.0, 3.0], [6.0, 5.0, 3.0], [6.0, 5.0, 6.0]], [[5.0, 4.0, 5.0], [1.0, 6.0, 3.0], [2.0, 4.0, 6.0]]], [[[5.0, 2.0, 
8.0], [3.0, 3.0, 7.0], [6.0, 3.0, 4.0]], [[3.0, 7.0, 1.0], [7.0, 7.0, 5.0], [2.0, 6.0, 7.0]], [[1.0, 2.0, 3.0], [2.0, 6.0, 
6.0], [1.0, 8.0, 2.0]]], [[[1.0, 6.0, 5.0], [4.0, 1.0, 2.0], [5.0, 4.0, 3.0]], [[3.0, 7.0, 4.0], [3.0, 8.0, 2.0], [8.0, 1.0, 
7.0]], [[2.0, 1.0, 6.0], [8.0, 7.0, 1.0], [3.0, 6.0, 1.0]]]]))
      val=numpy.array([[[6.0, 3.0, 6.0], [8.0, 6.0, 2.0]], [[6.0, 8.0, 1.0], [2.0, 1.0, 3.0]], [[4.0, 6.0, 6.0], [3.0, 3.0, 
2.0]], [[5.0, 8.0, 6.0], [3.0, 8.0, 6.0]]])
      arg[:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 2.0], [6.0, 3.0, 6.0], [8.0, 2.0, 6.0]], [[5.0, 4.0, 1.0], [8.0, 6.0, 2.0], [7.0, 
2.0, 7.0]], [[3.0, 4.0, 6.0], [8.0, 1.0, 2.0], [5.0, 1.0, 3.0]]], [[[6.0, 2.0, 1.0], [6.0, 8.0, 1.0], [8.0, 3.0, 3.0]], [[2.0, 
5.0, 4.0], [2.0, 1.0, 3.0], [2.0, 2.0, 7.0]], [[6.0, 5.0, 7.0], [7.0, 2.0, 8.0], [8.0, 5.0, 3.0]]], [[[4.0, 4.0, 5.0], [4.0, 
6.0, 6.0], [5.0, 6.0, 4.0]], [[7.0, 2.0, 1.0], [3.0, 3.0, 2.0], [7.0, 8.0, 8.0]], [[3.0, 6.0, 8.0], [4.0, 8.0, 3.0], [1.0, 4.0, 
3.0]]], [[[7.0, 4.0, 1.0], [5.0, 8.0, 6.0], [3.0, 3.0, 5.0]], [[4.0, 3.0, 3.0], [3.0, 8.0, 6.0], [5.0, 3.0, 1.0]], [[1.0, 1.0, 
7.0], [5.0, 4.0, 1.0], [5.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 6.0], [6.0, 3.0, 6.0], [5.0, 8.0, 8.0]], 
[[2.0, 1.0, 3.0], [8.0, 6.0, 2.0], [5.0, 3.0, 1.0]], [[8.0, 2.0, 8.0], [5.0, 3.0, 2.0], [8.0, 1.0, 4.0]]], [[[2.0, 5.0, 8.0], 
[6.0, 8.0, 1.0], [3.0, 5.0, 5.0]], [[7.0, 8.0, 3.0], [2.0, 1.0, 3.0], [6.0, 5.0, 6.0]], [[5.0, 4.0, 5.0], [1.0, 6.0, 3.0], 
[2.0, 4.0, 6.0]]], [[[5.0, 2.0, 8.0], [4.0, 6.0, 6.0], [6.0, 3.0, 4.0]], [[3.0, 7.0, 1.0], [3.0, 3.0, 2.0], [2.0, 6.0, 7.0]], 
[[1.0, 2.0, 3.0], [2.0, 6.0, 6.0], [1.0, 8.0, 2.0]]], [[[1.0, 6.0, 5.0], [5.0, 8.0, 6.0], [5.0, 4.0, 3.0]], [[3.0, 7.0, 4.0], 
[3.0, 8.0, 6.0], [8.0, 1.0, 7.0]], [[2.0, 1.0, 6.0], [8.0, 7.0, 1.0], [3.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CUPN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 3.0], [6.0, 7.0, 3.0], [1.0, 3.0, 2.0]], [[8.0, 5.0, 5.0], [4.0, 2.0, 8.0], [2.0, 4.0, 
7.0]], [[5.0, 4.0, 1.0], [7.0, 1.0, 3.0], [8.0, 3.0, 4.0]]], [[[3.0, 6.0, 3.0], [3.0, 1.0, 5.0], [8.0, 1.0, 8.0]], [[7.0, 6.0, 
3.0], [1.0, 1.0, 6.0], [7.0, 3.0, 5.0]], [[2.0, 8.0, 2.0], [1.0, 1.0, 8.0], [2.0, 1.0, 2.0]]], [[[5.0, 2.0, 7.0], [4.0, 7.0, 
6.0], [5.0, 1.0, 7.0]], [[8.0, 6.0, 4.0], [3.0, 1.0, 8.0], [5.0, 4.0, 6.0]], [[1.0, 2.0, 3.0], [6.0, 7.0, 5.0], [3.0, 7.0, 
3.0]]], [[[2.0, 4.0, 4.0], [4.0, 5.0, 8.0], [2.0, 1.0, 6.0]], [[2.0, 2.0, 8.0], [5.0, 5.0, 1.0], [1.0, 5.0, 7.0]], [[1.0, 3.0, 
8.0], [7.0, 4.0, 4.0], [5.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 6.0], [7.0, 1.0, 3.0], [1.0, 5.0, 1.0]], [[1.0, 4.0, 4.0], [4.0, 6.0, 
3.0], [3.0, 3.0, 2.0]], [[8.0, 1.0, 6.0], [6.0, 8.0, 8.0], [3.0, 3.0, 3.0]]], [[[2.0, 1.0, 8.0], [3.0, 7.0, 3.0], [7.0, 7.0, 
3.0]], [[5.0, 7.0, 3.0], [3.0, 7.0, 2.0], [8.0, 5.0, 4.0]], [[4.0, 5.0, 7.0], [4.0, 1.0, 3.0], [8.0, 8.0, 3.0]]], [[[2.0, 6.0, 
2.0], [7.0, 4.0, 8.0], [3.0, 1.0, 1.0]], [[7.0, 2.0, 3.0], [2.0, 1.0, 2.0], [7.0, 3.0, 2.0]], [[5.0, 1.0, 6.0], [1.0, 3.0, 
4.0], [5.0, 4.0, 3.0]]], [[[2.0, 7.0, 5.0], [6.0, 3.0, 3.0], [5.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 1.0, 6.0], [6.0, 1.0, 
4.0]], [[8.0, 1.0, 5.0], [6.0, 4.0, 7.0], [1.0, 4.0, 4.0]]]]))
      val=Data(numpy.array([[[6.0, 4.0, 1.0], [3.0, 4.0, 3.0]], [[1.0, 1.0, 4.0], [2.0, 4.0, 7.0]], [[2.0, 1.0, 7.0], [1.0, 
7.0, 3.0]], [[5.0, 7.0, 1.0], [1.0, 7.0, 5.0]]]),self.functionspace)
      arg[:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 3.0], [6.0, 4.0, 1.0], [1.0, 3.0, 2.0]], [[8.0, 5.0, 5.0], [3.0, 4.0, 3.0], [2.0, 
4.0, 7.0]], [[5.0, 4.0, 1.0], [7.0, 1.0, 3.0], [8.0, 3.0, 4.0]]], [[[3.0, 6.0, 3.0], [1.0, 1.0, 4.0], [8.0, 1.0, 8.0]], [[7.0, 
6.0, 3.0], [2.0, 4.0, 7.0], [7.0, 3.0, 5.0]], [[2.0, 8.0, 2.0], [1.0, 1.0, 8.0], [2.0, 1.0, 2.0]]], [[[5.0, 2.0, 7.0], [2.0, 
1.0, 7.0], [5.0, 1.0, 7.0]], [[8.0, 6.0, 4.0], [1.0, 7.0, 3.0], [5.0, 4.0, 6.0]], [[1.0, 2.0, 3.0], [6.0, 7.0, 5.0], [3.0, 7.0, 
3.0]]], [[[2.0, 4.0, 4.0], [5.0, 7.0, 1.0], [2.0, 1.0, 6.0]], [[2.0, 2.0, 8.0], [1.0, 7.0, 5.0], [1.0, 5.0, 7.0]], [[1.0, 3.0, 
8.0], [7.0, 4.0, 4.0], [5.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 6.0], [6.0, 4.0, 1.0], [1.0, 5.0, 1.0]], 
[[1.0, 4.0, 4.0], [3.0, 4.0, 3.0], [3.0, 3.0, 2.0]], [[8.0, 1.0, 6.0], [6.0, 8.0, 8.0], [3.0, 3.0, 3.0]]], [[[2.0, 1.0, 8.0], 
[1.0, 1.0, 4.0], [7.0, 7.0, 3.0]], [[5.0, 7.0, 3.0], [2.0, 4.0, 7.0], [8.0, 5.0, 4.0]], [[4.0, 5.0, 7.0], [4.0, 1.0, 3.0], 
[8.0, 8.0, 3.0]]], [[[2.0, 6.0, 2.0], [2.0, 1.0, 7.0], [3.0, 1.0, 1.0]], [[7.0, 2.0, 3.0], [1.0, 7.0, 3.0], [7.0, 3.0, 2.0]], 
[[5.0, 1.0, 6.0], [1.0, 3.0, 4.0], [5.0, 4.0, 3.0]]], [[[2.0, 7.0, 5.0], [5.0, 7.0, 1.0], [5.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], 
[1.0, 7.0, 5.0], [6.0, 1.0, 4.0]], [[8.0, 1.0, 5.0], [6.0, 4.0, 7.0], [1.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CUPN(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 4.0], [8.0, 7.0, 1.0], [2.0, 1.0, 6.0]], [[5.0, 5.0, 1.0], [2.0, 5.0, 3.0], [6.0, 1.0, 
4.0]], [[7.0, 8.0, 6.0], [3.0, 3.0, 5.0], [5.0, 8.0, 6.0]]], [[[4.0, 6.0, 5.0], [5.0, 3.0, 8.0], [4.0, 2.0, 2.0]], [[7.0, 3.0, 
4.0], [5.0, 5.0, 5.0], [1.0, 2.0, 5.0]], [[7.0, 6.0, 1.0], [6.0, 4.0, 1.0], [8.0, 1.0, 7.0]]], [[[4.0, 1.0, 5.0], [5.0, 5.0, 
5.0], [4.0, 5.0, 2.0]], [[4.0, 8.0, 3.0], [6.0, 7.0, 1.0], [4.0, 2.0, 3.0]], [[4.0, 4.0, 4.0], [6.0, 7.0, 6.0], [3.0, 7.0, 
4.0]]], [[[8.0, 7.0, 6.0], [4.0, 3.0, 2.0], [2.0, 1.0, 4.0]], [[3.0, 3.0, 3.0], [7.0, 3.0, 4.0], [1.0, 4.0, 1.0]], [[7.0, 2.0, 
8.0], [1.0, 7.0, 7.0], [4.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 4.0], [4.0, 2.0, 3.0], [2.0, 1.0, 7.0]], [[7.0, 5.0, 3.0], [1.0, 6.0, 
1.0], [6.0, 2.0, 1.0]], [[3.0, 5.0, 7.0], [1.0, 6.0, 6.0], [7.0, 5.0, 3.0]]], [[[5.0, 6.0, 4.0], [5.0, 2.0, 6.0], [3.0, 2.0, 
2.0]], [[8.0, 5.0, 6.0], [8.0, 6.0, 4.0], [2.0, 5.0, 3.0]], [[4.0, 2.0, 3.0], [2.0, 6.0, 6.0], [4.0, 3.0, 1.0]]], [[[4.0, 7.0, 
5.0], [4.0, 6.0, 5.0], [2.0, 2.0, 5.0]], [[2.0, 5.0, 2.0], [3.0, 4.0, 2.0], [8.0, 1.0, 7.0]], [[7.0, 1.0, 8.0], [6.0, 4.0, 
1.0], [2.0, 2.0, 1.0]]], [[[1.0, 8.0, 4.0], [3.0, 6.0, 2.0], [8.0, 7.0, 4.0]], [[2.0, 6.0, 5.0], [1.0, 3.0, 7.0], [2.0, 1.0, 
4.0]], [[7.0, 3.0, 4.0], [7.0, 6.0, 5.0], [7.0, 7.0, 7.0]]]]))
      val=Data(numpy.array([[[4.0, 8.0, 1.0], [1.0, 2.0, 4.0]], [[1.0, 8.0, 1.0], [3.0, 4.0, 4.0]], [[8.0, 2.0, 3.0], [4.0, 
6.0, 2.0]], [[2.0, 8.0, 4.0], [4.0, 6.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 1.0, 8.0], [3.0, 3.0, 1.0]], [[7.0, 7.0, 7.0], [5.0, 3.0, 5.0]], [[8.0, 5.0, 
3.0], [5.0, 2.0, 6.0]], [[5.0, 5.0, 5.0], [2.0, 1.0, 7.0]]]))
      arg[:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 4.0], [4.0, 8.0, 1.0], [2.0, 1.0, 6.0]], [[5.0, 5.0, 1.0], [1.0, 2.0, 4.0], [6.0, 
1.0, 4.0]], [[7.0, 8.0, 6.0], [3.0, 3.0, 5.0], [5.0, 8.0, 6.0]]], [[[4.0, 6.0, 5.0], [1.0, 8.0, 1.0], [4.0, 2.0, 2.0]], [[7.0, 
3.0, 4.0], [3.0, 4.0, 4.0], [1.0, 2.0, 5.0]], [[7.0, 6.0, 1.0], [6.0, 4.0, 1.0], [8.0, 1.0, 7.0]]], [[[4.0, 1.0, 5.0], [8.0, 
2.0, 3.0], [4.0, 5.0, 2.0]], [[4.0, 8.0, 3.0], [4.0, 6.0, 2.0], [4.0, 2.0, 3.0]], [[4.0, 4.0, 4.0], [6.0, 7.0, 6.0], [3.0, 7.0, 
4.0]]], [[[8.0, 7.0, 6.0], [2.0, 8.0, 4.0], [2.0, 1.0, 4.0]], [[3.0, 3.0, 3.0], [4.0, 6.0, 2.0], [1.0, 4.0, 1.0]], [[7.0, 2.0, 
8.0], [1.0, 7.0, 7.0], [4.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 4.0], [1.0, 1.0, 8.0], [2.0, 1.0, 7.0]], 
[[7.0, 5.0, 3.0], [3.0, 3.0, 1.0], [6.0, 2.0, 1.0]], [[3.0, 5.0, 7.0], [1.0, 6.0, 6.0], [7.0, 5.0, 3.0]]], [[[5.0, 6.0, 4.0], 
[7.0, 7.0, 7.0], [3.0, 2.0, 2.0]], [[8.0, 5.0, 6.0], [5.0, 3.0, 5.0], [2.0, 5.0, 3.0]], [[4.0, 2.0, 3.0], [2.0, 6.0, 6.0], 
[4.0, 3.0, 1.0]]], [[[4.0, 7.0, 5.0], [8.0, 5.0, 3.0], [2.0, 2.0, 5.0]], [[2.0, 5.0, 2.0], [5.0, 2.0, 6.0], [8.0, 1.0, 7.0]], 
[[7.0, 1.0, 8.0], [6.0, 4.0, 1.0], [2.0, 2.0, 1.0]]], [[[1.0, 8.0, 4.0], [5.0, 5.0, 5.0], [8.0, 7.0, 4.0]], [[2.0, 6.0, 5.0], 
[2.0, 1.0, 7.0], [2.0, 1.0, 4.0]], [[7.0, 3.0, 4.0], [7.0, 6.0, 5.0], [7.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CUPN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 3.0], [4.0, 3.0, 5.0], [5.0, 3.0, 1.0]], [[7.0, 3.0, 8.0], [4.0, 2.0, 2.0], [5.0, 5.0, 
6.0]], [[7.0, 1.0, 5.0], [6.0, 5.0, 5.0], [2.0, 1.0, 6.0]]], [[[1.0, 4.0, 8.0], [3.0, 4.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 1.0, 
1.0], [5.0, 3.0, 3.0], [8.0, 5.0, 4.0]], [[5.0, 3.0, 8.0], [2.0, 8.0, 8.0], [1.0, 1.0, 3.0]]], [[[8.0, 5.0, 3.0], [1.0, 8.0, 
1.0], [4.0, 4.0, 4.0]], [[8.0, 7.0, 2.0], [6.0, 3.0, 2.0], [7.0, 7.0, 6.0]], [[7.0, 4.0, 6.0], [4.0, 5.0, 8.0], [7.0, 6.0, 
2.0]]], [[[5.0, 4.0, 2.0], [6.0, 2.0, 2.0], [7.0, 7.0, 5.0]], [[1.0, 6.0, 5.0], [7.0, 3.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 1.0, 
2.0], [3.0, 4.0, 3.0], [7.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 1.0], [2.0, 7.0, 2.0], [6.0, 3.0, 8.0]], [[2.0, 1.0, 2.0], [8.0, 8.0, 
6.0], [5.0, 4.0, 1.0]], [[5.0, 7.0, 5.0], [5.0, 8.0, 8.0], [1.0, 4.0, 4.0]]], [[[1.0, 3.0, 4.0], [3.0, 5.0, 1.0], [2.0, 6.0, 
3.0]], [[6.0, 3.0, 8.0], [6.0, 8.0, 5.0], [6.0, 3.0, 8.0]], [[7.0, 5.0, 7.0], [1.0, 4.0, 6.0], [5.0, 7.0, 6.0]]], [[[2.0, 5.0, 
8.0], [6.0, 5.0, 2.0], [6.0, 5.0, 2.0]], [[3.0, 1.0, 5.0], [5.0, 5.0, 5.0], [4.0, 5.0, 5.0]], [[5.0, 4.0, 8.0], [2.0, 5.0, 
8.0], [5.0, 8.0, 6.0]]], [[[2.0, 3.0, 8.0], [8.0, 8.0, 3.0], [1.0, 5.0, 1.0]], [[4.0, 1.0, 4.0], [1.0, 2.0, 8.0], [5.0, 5.0, 
1.0]], [[5.0, 1.0, 6.0], [2.0, 3.0, 4.0], [3.0, 7.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 3.0, 6.0], [8.0, 2.0, 6.0]], [[4.0, 2.0, 4.0], [2.0, 4.0, 6.0]], [[6.0, 3.0, 1.0], [5.0, 
6.0, 8.0]], [[5.0, 2.0, 5.0], [8.0, 2.0, 5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 7.0, 6.0], [7.0, 5.0, 8.0]], [[8.0, 4.0, 
6.0], [8.0, 2.0, 6.0]], [[5.0, 3.0, 8.0], [1.0, 5.0, 3.0]], [[3.0, 6.0, 8.0], [7.0, 4.0, 6.0]]])
      arg[:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 3.0], [5.0, 3.0, 6.0], [5.0, 3.0, 1.0]], [[7.0, 3.0, 8.0], [8.0, 2.0, 6.0], [5.0, 
5.0, 6.0]], [[7.0, 1.0, 5.0], [6.0, 5.0, 5.0], [2.0, 1.0, 6.0]]], [[[1.0, 4.0, 8.0], [4.0, 2.0, 4.0], [3.0, 8.0, 8.0]], [[5.0, 
1.0, 1.0], [2.0, 4.0, 6.0], [8.0, 5.0, 4.0]], [[5.0, 3.0, 8.0], [2.0, 8.0, 8.0], [1.0, 1.0, 3.0]]], [[[8.0, 5.0, 3.0], [6.0, 
3.0, 1.0], [4.0, 4.0, 4.0]], [[8.0, 7.0, 2.0], [5.0, 6.0, 8.0], [7.0, 7.0, 6.0]], [[7.0, 4.0, 6.0], [4.0, 5.0, 8.0], [7.0, 6.0, 
2.0]]], [[[5.0, 4.0, 2.0], [5.0, 2.0, 5.0], [7.0, 7.0, 5.0]], [[1.0, 6.0, 5.0], [8.0, 2.0, 5.0], [2.0, 1.0, 7.0]], [[8.0, 1.0, 
2.0], [3.0, 4.0, 3.0], [7.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 1.0], [8.0, 7.0, 6.0], [6.0, 3.0, 8.0]], 
[[2.0, 1.0, 2.0], [7.0, 5.0, 8.0], [5.0, 4.0, 1.0]], [[5.0, 7.0, 5.0], [5.0, 8.0, 8.0], [1.0, 4.0, 4.0]]], [[[1.0, 3.0, 4.0], 
[8.0, 4.0, 6.0], [2.0, 6.0, 3.0]], [[6.0, 3.0, 8.0], [8.0, 2.0, 6.0], [6.0, 3.0, 8.0]], [[7.0, 5.0, 7.0], [1.0, 4.0, 6.0], 
[5.0, 7.0, 6.0]]], [[[2.0, 5.0, 8.0], [5.0, 3.0, 8.0], [6.0, 5.0, 2.0]], [[3.0, 1.0, 5.0], [1.0, 5.0, 3.0], [4.0, 5.0, 5.0]], 
[[5.0, 4.0, 8.0], [2.0, 5.0, 8.0], [5.0, 8.0, 6.0]]], [[[2.0, 3.0, 8.0], [3.0, 6.0, 8.0], [1.0, 5.0, 1.0]], [[4.0, 1.0, 4.0], 
[7.0, 4.0, 6.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 6.0], [2.0, 3.0, 4.0], [3.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPUN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 2.0], [3.0, 5.0, 7.0], [5.0, 8.0, 7.0]], [[1.0, 4.0, 8.0], [6.0, 7.0, 1.0], [6.0, 5.0, 
5.0]], [[3.0, 3.0, 3.0], [8.0, 1.0, 1.0], [1.0, 3.0, 2.0]]], [[[2.0, 2.0, 1.0], [5.0, 2.0, 3.0], [3.0, 3.0, 5.0]], [[6.0, 5.0, 
4.0], [4.0, 1.0, 4.0], [7.0, 1.0, 6.0]], [[2.0, 2.0, 7.0], [1.0, 8.0, 4.0], [3.0, 8.0, 5.0]]], [[[4.0, 8.0, 6.0], [3.0, 3.0, 
6.0], [7.0, 3.0, 3.0]], [[5.0, 6.0, 8.0], [7.0, 2.0, 7.0], [8.0, 2.0, 2.0]], [[7.0, 2.0, 1.0], [3.0, 1.0, 2.0], [6.0, 2.0, 
1.0]]], [[[7.0, 1.0, 4.0], [8.0, 7.0, 4.0], [6.0, 4.0, 2.0]], [[7.0, 3.0, 7.0], [1.0, 8.0, 5.0], [4.0, 2.0, 8.0]], [[4.0, 8.0, 
6.0], [6.0, 5.0, 4.0], [3.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 7.0], [2.0, 8.0, 5.0], [7.0, 1.0, 1.0]], [[3.0, 6.0, 2.0], [3.0, 8.0, 
7.0], [5.0, 2.0, 4.0]], [[8.0, 2.0, 7.0], [8.0, 6.0, 5.0], [2.0, 1.0, 2.0]]], [[[4.0, 2.0, 3.0], [8.0, 5.0, 2.0], [7.0, 6.0, 
6.0]], [[2.0, 5.0, 1.0], [6.0, 4.0, 5.0], [6.0, 3.0, 6.0]], [[7.0, 6.0, 1.0], [1.0, 4.0, 2.0], [2.0, 1.0, 7.0]]], [[[2.0, 3.0, 
7.0], [4.0, 3.0, 3.0], [8.0, 5.0, 7.0]], [[6.0, 5.0, 3.0], [4.0, 3.0, 6.0], [1.0, 4.0, 2.0]], [[1.0, 7.0, 4.0], [7.0, 6.0, 
8.0], [6.0, 8.0, 6.0]]], [[[5.0, 3.0, 3.0], [3.0, 4.0, 7.0], [5.0, 8.0, 2.0]], [[1.0, 2.0, 4.0], [1.0, 2.0, 5.0], [1.0, 4.0, 
8.0]], [[7.0, 4.0, 8.0], [1.0, 6.0, 3.0], [5.0, 5.0, 1.0]]]]))
      val=numpy.array([[[1.0, 8.0, 5.0], [6.0, 1.0, 2.0]], [[3.0, 7.0, 5.0], [3.0, 4.0, 7.0]], [[1.0, 3.0, 4.0], [8.0, 2.0, 
7.0]], [[8.0, 2.0, 3.0], [4.0, 4.0, 4.0]]])
      arg[:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 2.0], [3.0, 5.0, 7.0], [5.0, 8.0, 7.0]], [[1.0, 8.0, 5.0], [6.0, 1.0, 2.0], [6.0, 
5.0, 5.0]], [[3.0, 3.0, 3.0], [8.0, 1.0, 1.0], [1.0, 3.0, 2.0]]], [[[2.0, 2.0, 1.0], [5.0, 2.0, 3.0], [3.0, 3.0, 5.0]], [[3.0, 
7.0, 5.0], [3.0, 4.0, 7.0], [7.0, 1.0, 6.0]], [[2.0, 2.0, 7.0], [1.0, 8.0, 4.0], [3.0, 8.0, 5.0]]], [[[4.0, 8.0, 6.0], [3.0, 
3.0, 6.0], [7.0, 3.0, 3.0]], [[1.0, 3.0, 4.0], [8.0, 2.0, 7.0], [8.0, 2.0, 2.0]], [[7.0, 2.0, 1.0], [3.0, 1.0, 2.0], [6.0, 2.0, 
1.0]]], [[[7.0, 1.0, 4.0], [8.0, 7.0, 4.0], [6.0, 4.0, 2.0]], [[8.0, 2.0, 3.0], [4.0, 4.0, 4.0], [4.0, 2.0, 8.0]], [[4.0, 8.0, 
6.0], [6.0, 5.0, 4.0], [3.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 7.0], [2.0, 8.0, 5.0], [7.0, 1.0, 1.0]], 
[[1.0, 8.0, 5.0], [6.0, 1.0, 2.0], [5.0, 2.0, 4.0]], [[8.0, 2.0, 7.0], [8.0, 6.0, 5.0], [2.0, 1.0, 2.0]]], [[[4.0, 2.0, 3.0], 
[8.0, 5.0, 2.0], [7.0, 6.0, 6.0]], [[3.0, 7.0, 5.0], [3.0, 4.0, 7.0], [6.0, 3.0, 6.0]], [[7.0, 6.0, 1.0], [1.0, 4.0, 2.0], 
[2.0, 1.0, 7.0]]], [[[2.0, 3.0, 7.0], [4.0, 3.0, 3.0], [8.0, 5.0, 7.0]], [[1.0, 3.0, 4.0], [8.0, 2.0, 7.0], [1.0, 4.0, 2.0]], 
[[1.0, 7.0, 4.0], [7.0, 6.0, 8.0], [6.0, 8.0, 6.0]]], [[[5.0, 3.0, 3.0], [3.0, 4.0, 7.0], [5.0, 8.0, 2.0]], [[8.0, 2.0, 3.0], 
[4.0, 4.0, 4.0], [1.0, 4.0, 8.0]], [[7.0, 4.0, 8.0], [1.0, 6.0, 3.0], [5.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPUN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 1.0], [4.0, 1.0, 7.0], [5.0, 5.0, 1.0]], [[3.0, 1.0, 6.0], [6.0, 2.0, 6.0], [5.0, 4.0, 
4.0]], [[4.0, 4.0, 6.0], [7.0, 2.0, 7.0], [5.0, 8.0, 1.0]]], [[[5.0, 4.0, 5.0], [1.0, 2.0, 2.0], [4.0, 8.0, 2.0]], [[6.0, 4.0, 
2.0], [1.0, 5.0, 8.0], [1.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], [8.0, 5.0, 4.0], [6.0, 1.0, 3.0]]], [[[4.0, 2.0, 1.0], [1.0, 2.0, 
7.0], [7.0, 8.0, 6.0]], [[2.0, 8.0, 7.0], [7.0, 1.0, 4.0], [5.0, 2.0, 8.0]], [[3.0, 7.0, 4.0], [3.0, 6.0, 4.0], [2.0, 3.0, 
5.0]]], [[[6.0, 4.0, 2.0], [2.0, 7.0, 3.0], [4.0, 6.0, 1.0]], [[5.0, 1.0, 8.0], [8.0, 5.0, 8.0], [6.0, 4.0, 3.0]], [[5.0, 7.0, 
6.0], [3.0, 3.0, 8.0], [6.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 4.0], [2.0, 2.0, 6.0], [1.0, 5.0, 7.0]], [[2.0, 3.0, 1.0], [2.0, 7.0, 
8.0], [5.0, 3.0, 4.0]], [[2.0, 2.0, 5.0], [3.0, 6.0, 1.0], [8.0, 1.0, 2.0]]], [[[1.0, 3.0, 4.0], [1.0, 6.0, 8.0], [3.0, 5.0, 
7.0]], [[2.0, 3.0, 8.0], [6.0, 8.0, 8.0], [1.0, 5.0, 6.0]], [[7.0, 2.0, 2.0], [3.0, 5.0, 8.0], [7.0, 4.0, 2.0]]], [[[8.0, 2.0, 
5.0], [4.0, 3.0, 1.0], [3.0, 8.0, 1.0]], [[2.0, 8.0, 1.0], [3.0, 3.0, 4.0], [5.0, 8.0, 8.0]], [[5.0, 5.0, 7.0], [3.0, 6.0, 
2.0], [6.0, 8.0, 3.0]]], [[[1.0, 5.0, 1.0], [4.0, 4.0, 8.0], [7.0, 3.0, 6.0]], [[1.0, 4.0, 8.0], [2.0, 1.0, 5.0], [3.0, 2.0, 
7.0]], [[2.0, 2.0, 7.0], [2.0, 5.0, 1.0], [8.0, 7.0, 5.0]]]]))
      val=Data(numpy.array([[[5.0, 1.0, 6.0], [7.0, 3.0, 2.0]], [[4.0, 4.0, 5.0], [5.0, 6.0, 1.0]], [[6.0, 8.0, 8.0], [4.0, 
7.0, 1.0]], [[3.0, 8.0, 1.0], [2.0, 3.0, 5.0]]]),self.functionspace)
      arg[:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 1.0], [4.0, 1.0, 7.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 6.0], [7.0, 3.0, 2.0], [5.0, 
4.0, 4.0]], [[4.0, 4.0, 6.0], [7.0, 2.0, 7.0], [5.0, 8.0, 1.0]]], [[[5.0, 4.0, 5.0], [1.0, 2.0, 2.0], [4.0, 8.0, 2.0]], [[4.0, 
4.0, 5.0], [5.0, 6.0, 1.0], [1.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], [8.0, 5.0, 4.0], [6.0, 1.0, 3.0]]], [[[4.0, 2.0, 1.0], [1.0, 
2.0, 7.0], [7.0, 8.0, 6.0]], [[6.0, 8.0, 8.0], [4.0, 7.0, 1.0], [5.0, 2.0, 8.0]], [[3.0, 7.0, 4.0], [3.0, 6.0, 4.0], [2.0, 3.0, 
5.0]]], [[[6.0, 4.0, 2.0], [2.0, 7.0, 3.0], [4.0, 6.0, 1.0]], [[3.0, 8.0, 1.0], [2.0, 3.0, 5.0], [6.0, 4.0, 3.0]], [[5.0, 7.0, 
6.0], [3.0, 3.0, 8.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 4.0], [2.0, 2.0, 6.0], [1.0, 5.0, 7.0]], 
[[5.0, 1.0, 6.0], [7.0, 3.0, 2.0], [5.0, 3.0, 4.0]], [[2.0, 2.0, 5.0], [3.0, 6.0, 1.0], [8.0, 1.0, 2.0]]], [[[1.0, 3.0, 4.0], 
[1.0, 6.0, 8.0], [3.0, 5.0, 7.0]], [[4.0, 4.0, 5.0], [5.0, 6.0, 1.0], [1.0, 5.0, 6.0]], [[7.0, 2.0, 2.0], [3.0, 5.0, 8.0], 
[7.0, 4.0, 2.0]]], [[[8.0, 2.0, 5.0], [4.0, 3.0, 1.0], [3.0, 8.0, 1.0]], [[6.0, 8.0, 8.0], [4.0, 7.0, 1.0], [5.0, 8.0, 8.0]], 
[[5.0, 5.0, 7.0], [3.0, 6.0, 2.0], [6.0, 8.0, 3.0]]], [[[1.0, 5.0, 1.0], [4.0, 4.0, 8.0], [7.0, 3.0, 6.0]], [[3.0, 8.0, 1.0], 
[2.0, 3.0, 5.0], [3.0, 2.0, 7.0]], [[2.0, 2.0, 7.0], [2.0, 5.0, 1.0], [8.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPUN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 7.0], [5.0, 7.0, 4.0], [1.0, 1.0, 4.0]], [[5.0, 4.0, 3.0], [8.0, 2.0, 1.0], [4.0, 5.0, 
6.0]], [[7.0, 1.0, 3.0], [7.0, 3.0, 8.0], [6.0, 5.0, 8.0]]], [[[1.0, 6.0, 6.0], [6.0, 5.0, 1.0], [8.0, 5.0, 4.0]], [[1.0, 2.0, 
2.0], [6.0, 7.0, 4.0], [3.0, 7.0, 8.0]], [[4.0, 1.0, 5.0], [5.0, 8.0, 6.0], [7.0, 2.0, 6.0]]], [[[2.0, 2.0, 4.0], [2.0, 1.0, 
7.0], [6.0, 3.0, 6.0]], [[5.0, 1.0, 2.0], [3.0, 1.0, 5.0], [1.0, 8.0, 4.0]], [[8.0, 7.0, 6.0], [8.0, 2.0, 3.0], [2.0, 8.0, 
8.0]]], [[[8.0, 2.0, 2.0], [5.0, 3.0, 4.0], [1.0, 5.0, 3.0]], [[4.0, 4.0, 3.0], [5.0, 4.0, 3.0], [6.0, 7.0, 1.0]], [[1.0, 8.0, 
7.0], [6.0, 7.0, 8.0], [8.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 6.0], [2.0, 3.0, 6.0], [6.0, 8.0, 8.0]], [[7.0, 8.0, 4.0], [1.0, 3.0, 
5.0], [3.0, 6.0, 6.0]], [[4.0, 4.0, 7.0], [5.0, 3.0, 5.0], [3.0, 1.0, 5.0]]], [[[8.0, 6.0, 4.0], [5.0, 1.0, 6.0], [1.0, 2.0, 
3.0]], [[7.0, 4.0, 8.0], [8.0, 6.0, 2.0], [2.0, 1.0, 5.0]], [[8.0, 4.0, 8.0], [8.0, 7.0, 3.0], [3.0, 3.0, 4.0]]], [[[6.0, 3.0, 
2.0], [6.0, 2.0, 4.0], [6.0, 5.0, 7.0]], [[1.0, 7.0, 1.0], [8.0, 3.0, 4.0], [6.0, 3.0, 6.0]], [[1.0, 7.0, 1.0], [5.0, 3.0, 
8.0], [6.0, 4.0, 5.0]]], [[[2.0, 1.0, 1.0], [1.0, 7.0, 5.0], [3.0, 6.0, 3.0]], [[7.0, 7.0, 1.0], [1.0, 4.0, 6.0], [2.0, 8.0, 
1.0]], [[5.0, 7.0, 3.0], [4.0, 1.0, 3.0], [2.0, 3.0, 4.0]]]]))
      val=Data(numpy.array([[[7.0, 3.0, 8.0], [1.0, 4.0, 4.0]], [[7.0, 1.0, 1.0], [3.0, 3.0, 5.0]], [[8.0, 8.0, 3.0], [2.0, 
4.0, 7.0]], [[2.0, 3.0, 7.0], [5.0, 8.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 7.0, 8.0], [2.0, 8.0, 4.0]], [[4.0, 1.0, 4.0], [2.0, 7.0, 8.0]], [[3.0, 7.0, 
5.0], [1.0, 4.0, 6.0]], [[2.0, 6.0, 2.0], [1.0, 7.0, 5.0]]]))
      arg[:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 7.0], [5.0, 7.0, 4.0], [1.0, 1.0, 4.0]], [[7.0, 3.0, 8.0], [1.0, 4.0, 4.0], [4.0, 
5.0, 6.0]], [[7.0, 1.0, 3.0], [7.0, 3.0, 8.0], [6.0, 5.0, 8.0]]], [[[1.0, 6.0, 6.0], [6.0, 5.0, 1.0], [8.0, 5.0, 4.0]], [[7.0, 
1.0, 1.0], [3.0, 3.0, 5.0], [3.0, 7.0, 8.0]], [[4.0, 1.0, 5.0], [5.0, 8.0, 6.0], [7.0, 2.0, 6.0]]], [[[2.0, 2.0, 4.0], [2.0, 
1.0, 7.0], [6.0, 3.0, 6.0]], [[8.0, 8.0, 3.0], [2.0, 4.0, 7.0], [1.0, 8.0, 4.0]], [[8.0, 7.0, 6.0], [8.0, 2.0, 3.0], [2.0, 8.0, 
8.0]]], [[[8.0, 2.0, 2.0], [5.0, 3.0, 4.0], [1.0, 5.0, 3.0]], [[2.0, 3.0, 7.0], [5.0, 8.0, 4.0], [6.0, 7.0, 1.0]], [[1.0, 8.0, 
7.0], [6.0, 7.0, 8.0], [8.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 6.0], [2.0, 3.0, 6.0], [6.0, 8.0, 8.0]], 
[[8.0, 7.0, 8.0], [2.0, 8.0, 4.0], [3.0, 6.0, 6.0]], [[4.0, 4.0, 7.0], [5.0, 3.0, 5.0], [3.0, 1.0, 5.0]]], [[[8.0, 6.0, 4.0], 
[5.0, 1.0, 6.0], [1.0, 2.0, 3.0]], [[4.0, 1.0, 4.0], [2.0, 7.0, 8.0], [2.0, 1.0, 5.0]], [[8.0, 4.0, 8.0], [8.0, 7.0, 3.0], 
[3.0, 3.0, 4.0]]], [[[6.0, 3.0, 2.0], [6.0, 2.0, 4.0], [6.0, 5.0, 7.0]], [[3.0, 7.0, 5.0], [1.0, 4.0, 6.0], [6.0, 3.0, 6.0]], 
[[1.0, 7.0, 1.0], [5.0, 3.0, 8.0], [6.0, 4.0, 5.0]]], [[[2.0, 1.0, 1.0], [1.0, 7.0, 5.0], [3.0, 6.0, 3.0]], [[2.0, 6.0, 2.0], 
[1.0, 7.0, 5.0], [2.0, 8.0, 1.0]], [[5.0, 7.0, 3.0], [4.0, 1.0, 3.0], [2.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPUN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 2.0], [8.0, 3.0, 2.0], [5.0, 2.0, 7.0]], [[6.0, 2.0, 5.0], [1.0, 5.0, 5.0], [4.0, 1.0, 
1.0]], [[1.0, 8.0, 2.0], [7.0, 6.0, 1.0], [5.0, 6.0, 5.0]]], [[[6.0, 4.0, 6.0], [7.0, 8.0, 1.0], [3.0, 8.0, 3.0]], [[7.0, 1.0, 
8.0], [2.0, 3.0, 2.0], [8.0, 7.0, 4.0]], [[3.0, 5.0, 5.0], [6.0, 6.0, 7.0], [2.0, 7.0, 5.0]]], [[[8.0, 3.0, 7.0], [6.0, 4.0, 
1.0], [5.0, 1.0, 3.0]], [[3.0, 3.0, 4.0], [5.0, 1.0, 4.0], [4.0, 6.0, 8.0]], [[1.0, 2.0, 7.0], [2.0, 4.0, 6.0], [1.0, 7.0, 
2.0]]], [[[5.0, 8.0, 6.0], [7.0, 2.0, 1.0], [6.0, 2.0, 6.0]], [[2.0, 2.0, 1.0], [6.0, 4.0, 1.0], [5.0, 1.0, 2.0]], [[7.0, 3.0, 
6.0], [1.0, 8.0, 6.0], [3.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 5.0], [1.0, 2.0, 3.0], [4.0, 8.0, 7.0]], [[6.0, 2.0, 2.0], [2.0, 7.0, 
2.0], [5.0, 5.0, 8.0]], [[6.0, 4.0, 6.0], [6.0, 4.0, 6.0], [7.0, 6.0, 8.0]]], [[[5.0, 1.0, 8.0], [3.0, 8.0, 3.0], [4.0, 8.0, 
8.0]], [[7.0, 1.0, 7.0], [3.0, 4.0, 4.0], [5.0, 7.0, 7.0]], [[1.0, 4.0, 7.0], [5.0, 2.0, 2.0], [1.0, 8.0, 7.0]]], [[[4.0, 6.0, 
7.0], [2.0, 3.0, 1.0], [6.0, 7.0, 4.0]], [[8.0, 6.0, 3.0], [5.0, 1.0, 2.0], [2.0, 3.0, 8.0]], [[5.0, 4.0, 5.0], [2.0, 4.0, 
2.0], [1.0, 2.0, 8.0]]], [[[2.0, 2.0, 7.0], [6.0, 7.0, 8.0], [7.0, 2.0, 8.0]], [[7.0, 1.0, 4.0], [8.0, 2.0, 6.0], [7.0, 1.0, 
6.0]], [[6.0, 7.0, 5.0], [3.0, 4.0, 1.0], [7.0, 2.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 7.0, 7.0], [7.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], [7.0, 6.0, 1.0]], [[3.0, 4.0, 6.0], [1.0, 
7.0, 5.0]], [[4.0, 3.0, 4.0], [8.0, 2.0, 2.0]]])+(1.-msk_val)*numpy.array([[[7.0, 6.0, 3.0], [2.0, 4.0, 3.0]], [[3.0, 6.0, 
3.0], [1.0, 2.0, 1.0]], [[7.0, 1.0, 2.0], [4.0, 6.0, 5.0]], [[4.0, 7.0, 6.0], [1.0, 1.0, 8.0]]])
      arg[:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 2.0], [8.0, 3.0, 2.0], [5.0, 2.0, 7.0]], [[8.0, 7.0, 7.0], [7.0, 1.0, 5.0], [4.0, 
1.0, 1.0]], [[1.0, 8.0, 2.0], [7.0, 6.0, 1.0], [5.0, 6.0, 5.0]]], [[[6.0, 4.0, 6.0], [7.0, 8.0, 1.0], [3.0, 8.0, 3.0]], [[8.0, 
1.0, 1.0], [7.0, 6.0, 1.0], [8.0, 7.0, 4.0]], [[3.0, 5.0, 5.0], [6.0, 6.0, 7.0], [2.0, 7.0, 5.0]]], [[[8.0, 3.0, 7.0], [6.0, 
4.0, 1.0], [5.0, 1.0, 3.0]], [[3.0, 4.0, 6.0], [1.0, 7.0, 5.0], [4.0, 6.0, 8.0]], [[1.0, 2.0, 7.0], [2.0, 4.0, 6.0], [1.0, 7.0, 
2.0]]], [[[5.0, 8.0, 6.0], [7.0, 2.0, 1.0], [6.0, 2.0, 6.0]], [[4.0, 3.0, 4.0], [8.0, 2.0, 2.0], [5.0, 1.0, 2.0]], [[7.0, 3.0, 
6.0], [1.0, 8.0, 6.0], [3.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 5.0], [1.0, 2.0, 3.0], [4.0, 8.0, 7.0]], 
[[7.0, 6.0, 3.0], [2.0, 4.0, 3.0], [5.0, 5.0, 8.0]], [[6.0, 4.0, 6.0], [6.0, 4.0, 6.0], [7.0, 6.0, 8.0]]], [[[5.0, 1.0, 8.0], 
[3.0, 8.0, 3.0], [4.0, 8.0, 8.0]], [[3.0, 6.0, 3.0], [1.0, 2.0, 1.0], [5.0, 7.0, 7.0]], [[1.0, 4.0, 7.0], [5.0, 2.0, 2.0], 
[1.0, 8.0, 7.0]]], [[[4.0, 6.0, 7.0], [2.0, 3.0, 1.0], [6.0, 7.0, 4.0]], [[7.0, 1.0, 2.0], [4.0, 6.0, 5.0], [2.0, 3.0, 8.0]], 
[[5.0, 4.0, 5.0], [2.0, 4.0, 2.0], [1.0, 2.0, 8.0]]], [[[2.0, 2.0, 7.0], [6.0, 7.0, 8.0], [7.0, 2.0, 8.0]], [[4.0, 7.0, 6.0], 
[1.0, 1.0, 8.0], [7.0, 1.0, 6.0]], [[6.0, 7.0, 5.0], [3.0, 4.0, 1.0], [7.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPPN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 7.0], [4.0, 8.0, 4.0], [6.0, 7.0, 2.0]], [[8.0, 2.0, 4.0], [8.0, 8.0, 1.0], [8.0, 7.0, 
7.0]], [[8.0, 6.0, 5.0], [3.0, 2.0, 2.0], [7.0, 3.0, 5.0]]], [[[8.0, 3.0, 8.0], [5.0, 5.0, 6.0], [3.0, 8.0, 1.0]], [[8.0, 2.0, 
8.0], [6.0, 3.0, 8.0], [1.0, 4.0, 1.0]], [[2.0, 8.0, 6.0], [6.0, 4.0, 8.0], [1.0, 3.0, 4.0]]], [[[3.0, 1.0, 4.0], [2.0, 3.0, 
1.0], [6.0, 7.0, 1.0]], [[7.0, 5.0, 8.0], [5.0, 4.0, 5.0], [7.0, 4.0, 4.0]], [[4.0, 6.0, 5.0], [1.0, 5.0, 2.0], [2.0, 6.0, 
1.0]]], [[[5.0, 4.0, 1.0], [8.0, 4.0, 8.0], [1.0, 8.0, 5.0]], [[7.0, 8.0, 3.0], [7.0, 6.0, 2.0], [4.0, 4.0, 8.0]], [[2.0, 1.0, 
1.0], [6.0, 6.0, 4.0], [1.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 1.0], [7.0, 2.0, 3.0], [6.0, 7.0, 1.0]], [[4.0, 6.0, 6.0], [3.0, 3.0, 
6.0], [2.0, 3.0, 4.0]], [[5.0, 2.0, 6.0], [4.0, 8.0, 3.0], [3.0, 8.0, 5.0]]], [[[6.0, 2.0, 5.0], [4.0, 7.0, 1.0], [8.0, 4.0, 
5.0]], [[1.0, 1.0, 8.0], [2.0, 5.0, 7.0], [1.0, 4.0, 3.0]], [[6.0, 5.0, 1.0], [8.0, 6.0, 4.0], [3.0, 1.0, 6.0]]], [[[1.0, 6.0, 
7.0], [5.0, 3.0, 5.0], [1.0, 2.0, 4.0]], [[8.0, 5.0, 7.0], [4.0, 3.0, 2.0], [1.0, 7.0, 2.0]], [[1.0, 8.0, 5.0], [4.0, 2.0, 
2.0], [4.0, 6.0, 7.0]]], [[[2.0, 3.0, 4.0], [1.0, 4.0, 5.0], [8.0, 3.0, 8.0]], [[1.0, 4.0, 1.0], [2.0, 3.0, 3.0], [3.0, 7.0, 
5.0]], [[5.0, 6.0, 1.0], [7.0, 4.0, 2.0], [5.0, 7.0, 8.0]]]]))
      val=numpy.array([[4.0, 8.0, 2.0], [1.0, 3.0, 8.0], [7.0, 7.0, 1.0], [7.0, 2.0, 6.0]])
      arg[:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 7.0], [4.0, 8.0, 4.0], [6.0, 7.0, 2.0]], [[8.0, 2.0, 4.0], [4.0, 8.0, 2.0], [8.0, 
7.0, 7.0]], [[8.0, 6.0, 5.0], [3.0, 2.0, 2.0], [7.0, 3.0, 5.0]]], [[[8.0, 3.0, 8.0], [5.0, 5.0, 6.0], [3.0, 8.0, 1.0]], [[8.0, 
2.0, 8.0], [1.0, 3.0, 8.0], [1.0, 4.0, 1.0]], [[2.0, 8.0, 6.0], [6.0, 4.0, 8.0], [1.0, 3.0, 4.0]]], [[[3.0, 1.0, 4.0], [2.0, 
3.0, 1.0], [6.0, 7.0, 1.0]], [[7.0, 5.0, 8.0], [7.0, 7.0, 1.0], [7.0, 4.0, 4.0]], [[4.0, 6.0, 5.0], [1.0, 5.0, 2.0], [2.0, 6.0, 
1.0]]], [[[5.0, 4.0, 1.0], [8.0, 4.0, 8.0], [1.0, 8.0, 5.0]], [[7.0, 8.0, 3.0], [7.0, 2.0, 6.0], [4.0, 4.0, 8.0]], [[2.0, 1.0, 
1.0], [6.0, 6.0, 4.0], [1.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 1.0], [7.0, 2.0, 3.0], [6.0, 7.0, 1.0]], 
[[4.0, 6.0, 6.0], [4.0, 8.0, 2.0], [2.0, 3.0, 4.0]], [[5.0, 2.0, 6.0], [4.0, 8.0, 3.0], [3.0, 8.0, 5.0]]], [[[6.0, 2.0, 5.0], 
[4.0, 7.0, 1.0], [8.0, 4.0, 5.0]], [[1.0, 1.0, 8.0], [1.0, 3.0, 8.0], [1.0, 4.0, 3.0]], [[6.0, 5.0, 1.0], [8.0, 6.0, 4.0], 
[3.0, 1.0, 6.0]]], [[[1.0, 6.0, 7.0], [5.0, 3.0, 5.0], [1.0, 2.0, 4.0]], [[8.0, 5.0, 7.0], [7.0, 7.0, 1.0], [1.0, 7.0, 2.0]], 
[[1.0, 8.0, 5.0], [4.0, 2.0, 2.0], [4.0, 6.0, 7.0]]], [[[2.0, 3.0, 4.0], [1.0, 4.0, 5.0], [8.0, 3.0, 8.0]], [[1.0, 4.0, 1.0], 
[7.0, 2.0, 6.0], [3.0, 7.0, 5.0]], [[5.0, 6.0, 1.0], [7.0, 4.0, 2.0], [5.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPPN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 3.0], [5.0, 4.0, 1.0], [5.0, 2.0, 7.0]], [[3.0, 8.0, 5.0], [5.0, 2.0, 4.0], [8.0, 5.0, 
8.0]], [[1.0, 1.0, 5.0], [5.0, 5.0, 6.0], [6.0, 5.0, 6.0]]], [[[7.0, 3.0, 6.0], [8.0, 4.0, 3.0], [2.0, 1.0, 8.0]], [[4.0, 6.0, 
4.0], [7.0, 5.0, 3.0], [5.0, 7.0, 4.0]], [[1.0, 8.0, 8.0], [4.0, 6.0, 1.0], [1.0, 5.0, 5.0]]], [[[6.0, 7.0, 7.0], [2.0, 8.0, 
1.0], [4.0, 7.0, 2.0]], [[6.0, 6.0, 5.0], [8.0, 7.0, 5.0], [2.0, 8.0, 1.0]], [[6.0, 8.0, 1.0], [2.0, 1.0, 7.0], [7.0, 5.0, 
2.0]]], [[[1.0, 6.0, 6.0], [2.0, 8.0, 7.0], [1.0, 8.0, 2.0]], [[1.0, 4.0, 4.0], [2.0, 3.0, 6.0], [5.0, 5.0, 4.0]], [[6.0, 4.0, 
4.0], [1.0, 2.0, 1.0], [6.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 3.0], [2.0, 3.0, 7.0], [1.0, 1.0, 1.0]], [[3.0, 5.0, 7.0], [8.0, 8.0, 
4.0], [8.0, 1.0, 2.0]], [[1.0, 5.0, 1.0], [8.0, 5.0, 3.0], [4.0, 4.0, 8.0]]], [[[6.0, 5.0, 4.0], [2.0, 4.0, 3.0], [3.0, 8.0, 
1.0]], [[8.0, 5.0, 3.0], [2.0, 6.0, 8.0], [5.0, 4.0, 3.0]], [[7.0, 4.0, 5.0], [2.0, 6.0, 3.0], [8.0, 5.0, 7.0]]], [[[1.0, 1.0, 
6.0], [3.0, 5.0, 3.0], [2.0, 4.0, 5.0]], [[7.0, 1.0, 6.0], [7.0, 5.0, 1.0], [2.0, 6.0, 2.0]], [[8.0, 7.0, 6.0], [8.0, 6.0, 
6.0], [4.0, 2.0, 5.0]]], [[[5.0, 3.0, 7.0], [3.0, 2.0, 3.0], [8.0, 3.0, 7.0]], [[5.0, 1.0, 1.0], [3.0, 5.0, 1.0], [5.0, 7.0, 
1.0]], [[3.0, 1.0, 5.0], [5.0, 6.0, 6.0], [8.0, 7.0, 5.0]]]]))
      val=Data(numpy.array([[1.0, 7.0, 2.0], [6.0, 2.0, 5.0], [1.0, 1.0, 6.0], [6.0, 3.0, 7.0]]),self.functionspace)
      arg[:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 3.0], [5.0, 4.0, 1.0], [5.0, 2.0, 7.0]], [[3.0, 8.0, 5.0], [1.0, 7.0, 2.0], [8.0, 
5.0, 8.0]], [[1.0, 1.0, 5.0], [5.0, 5.0, 6.0], [6.0, 5.0, 6.0]]], [[[7.0, 3.0, 6.0], [8.0, 4.0, 3.0], [2.0, 1.0, 8.0]], [[4.0, 
6.0, 4.0], [6.0, 2.0, 5.0], [5.0, 7.0, 4.0]], [[1.0, 8.0, 8.0], [4.0, 6.0, 1.0], [1.0, 5.0, 5.0]]], [[[6.0, 7.0, 7.0], [2.0, 
8.0, 1.0], [4.0, 7.0, 2.0]], [[6.0, 6.0, 5.0], [1.0, 1.0, 6.0], [2.0, 8.0, 1.0]], [[6.0, 8.0, 1.0], [2.0, 1.0, 7.0], [7.0, 5.0, 
2.0]]], [[[1.0, 6.0, 6.0], [2.0, 8.0, 7.0], [1.0, 8.0, 2.0]], [[1.0, 4.0, 4.0], [6.0, 3.0, 7.0], [5.0, 5.0, 4.0]], [[6.0, 4.0, 
4.0], [1.0, 2.0, 1.0], [6.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 3.0], [2.0, 3.0, 7.0], [1.0, 1.0, 1.0]], 
[[3.0, 5.0, 7.0], [1.0, 7.0, 2.0], [8.0, 1.0, 2.0]], [[1.0, 5.0, 1.0], [8.0, 5.0, 3.0], [4.0, 4.0, 8.0]]], [[[6.0, 5.0, 4.0], 
[2.0, 4.0, 3.0], [3.0, 8.0, 1.0]], [[8.0, 5.0, 3.0], [6.0, 2.0, 5.0], [5.0, 4.0, 3.0]], [[7.0, 4.0, 5.0], [2.0, 6.0, 3.0], 
[8.0, 5.0, 7.0]]], [[[1.0, 1.0, 6.0], [3.0, 5.0, 3.0], [2.0, 4.0, 5.0]], [[7.0, 1.0, 6.0], [1.0, 1.0, 6.0], [2.0, 6.0, 2.0]], 
[[8.0, 7.0, 6.0], [8.0, 6.0, 6.0], [4.0, 2.0, 5.0]]], [[[5.0, 3.0, 7.0], [3.0, 2.0, 3.0], [8.0, 3.0, 7.0]], [[5.0, 1.0, 1.0], 
[6.0, 3.0, 7.0], [5.0, 7.0, 1.0]], [[3.0, 1.0, 5.0], [5.0, 6.0, 6.0], [8.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPPN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 5.0], [2.0, 3.0, 7.0], [5.0, 3.0, 1.0]], [[5.0, 8.0, 7.0], [5.0, 6.0, 8.0], [6.0, 1.0, 
4.0]], [[5.0, 5.0, 6.0], [6.0, 2.0, 8.0], [8.0, 8.0, 8.0]]], [[[4.0, 6.0, 3.0], [2.0, 3.0, 8.0], [6.0, 8.0, 4.0]], [[6.0, 1.0, 
6.0], [5.0, 3.0, 8.0], [6.0, 4.0, 8.0]], [[6.0, 8.0, 7.0], [5.0, 1.0, 5.0], [7.0, 5.0, 7.0]]], [[[7.0, 8.0, 1.0], [3.0, 4.0, 
1.0], [1.0, 5.0, 6.0]], [[4.0, 8.0, 3.0], [6.0, 6.0, 1.0], [8.0, 7.0, 8.0]], [[1.0, 1.0, 4.0], [4.0, 1.0, 5.0], [4.0, 3.0, 
8.0]]], [[[5.0, 7.0, 2.0], [2.0, 1.0, 3.0], [8.0, 8.0, 3.0]], [[6.0, 6.0, 4.0], [7.0, 5.0, 5.0], [7.0, 1.0, 4.0]], [[3.0, 5.0, 
8.0], [1.0, 1.0, 2.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 5.0], [6.0, 3.0, 2.0], [3.0, 1.0, 2.0]], [[7.0, 7.0, 7.0], [5.0, 1.0, 
5.0], [8.0, 4.0, 6.0]], [[2.0, 2.0, 4.0], [1.0, 1.0, 1.0], [5.0, 4.0, 4.0]]], [[[2.0, 6.0, 6.0], [5.0, 6.0, 2.0], [2.0, 4.0, 
6.0]], [[6.0, 5.0, 7.0], [2.0, 4.0, 1.0], [1.0, 6.0, 1.0]], [[6.0, 1.0, 2.0], [5.0, 4.0, 8.0], [8.0, 8.0, 1.0]]], [[[5.0, 5.0, 
5.0], [4.0, 7.0, 2.0], [8.0, 3.0, 8.0]], [[2.0, 4.0, 6.0], [3.0, 5.0, 7.0], [6.0, 1.0, 3.0]], [[2.0, 4.0, 1.0], [4.0, 2.0, 
4.0], [6.0, 7.0, 3.0]]], [[[2.0, 1.0, 8.0], [6.0, 3.0, 6.0], [4.0, 1.0, 6.0]], [[7.0, 5.0, 3.0], [6.0, 4.0, 2.0], [5.0, 5.0, 
8.0]], [[7.0, 1.0, 8.0], [2.0, 7.0, 2.0], [5.0, 3.0, 4.0]]]]))
      val=Data(numpy.array([[6.0, 1.0, 2.0], [5.0, 3.0, 7.0], [6.0, 5.0, 7.0], [5.0, 7.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 8.0, 6.0], [8.0, 7.0, 3.0], [7.0, 3.0, 8.0], [4.0, 3.0, 5.0]]))
      arg[:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 5.0], [2.0, 3.0, 7.0], [5.0, 3.0, 1.0]], [[5.0, 8.0, 7.0], [6.0, 1.0, 2.0], [6.0, 
1.0, 4.0]], [[5.0, 5.0, 6.0], [6.0, 2.0, 8.0], [8.0, 8.0, 8.0]]], [[[4.0, 6.0, 3.0], [2.0, 3.0, 8.0], [6.0, 8.0, 4.0]], [[6.0, 
1.0, 6.0], [5.0, 3.0, 7.0], [6.0, 4.0, 8.0]], [[6.0, 8.0, 7.0], [5.0, 1.0, 5.0], [7.0, 5.0, 7.0]]], [[[7.0, 8.0, 1.0], [3.0, 
4.0, 1.0], [1.0, 5.0, 6.0]], [[4.0, 8.0, 3.0], [6.0, 5.0, 7.0], [8.0, 7.0, 8.0]], [[1.0, 1.0, 4.0], [4.0, 1.0, 5.0], [4.0, 3.0, 
8.0]]], [[[5.0, 7.0, 2.0], [2.0, 1.0, 3.0], [8.0, 8.0, 3.0]], [[6.0, 6.0, 4.0], [5.0, 7.0, 5.0], [7.0, 1.0, 4.0]], [[3.0, 5.0, 
8.0], [1.0, 1.0, 2.0], [3.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 5.0], [6.0, 3.0, 2.0], [3.0, 1.0, 2.0]], 
[[7.0, 7.0, 7.0], [1.0, 8.0, 6.0], [8.0, 4.0, 6.0]], [[2.0, 2.0, 4.0], [1.0, 1.0, 1.0], [5.0, 4.0, 4.0]]], [[[2.0, 6.0, 6.0], 
[5.0, 6.0, 2.0], [2.0, 4.0, 6.0]], [[6.0, 5.0, 7.0], [8.0, 7.0, 3.0], [1.0, 6.0, 1.0]], [[6.0, 1.0, 2.0], [5.0, 4.0, 8.0], 
[8.0, 8.0, 1.0]]], [[[5.0, 5.0, 5.0], [4.0, 7.0, 2.0], [8.0, 3.0, 8.0]], [[2.0, 4.0, 6.0], [7.0, 3.0, 8.0], [6.0, 1.0, 3.0]], 
[[2.0, 4.0, 1.0], [4.0, 2.0, 4.0], [6.0, 7.0, 3.0]]], [[[2.0, 1.0, 8.0], [6.0, 3.0, 6.0], [4.0, 1.0, 6.0]], [[7.0, 5.0, 3.0], 
[4.0, 3.0, 5.0], [5.0, 5.0, 8.0]], [[7.0, 1.0, 8.0], [2.0, 7.0, 2.0], [5.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPPN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 1.0], [5.0, 3.0, 4.0], [4.0, 2.0, 2.0]], [[3.0, 6.0, 2.0], [2.0, 5.0, 6.0], [2.0, 1.0, 
2.0]], [[6.0, 4.0, 8.0], [7.0, 3.0, 1.0], [4.0, 5.0, 4.0]]], [[[6.0, 1.0, 1.0], [6.0, 3.0, 2.0], [1.0, 6.0, 2.0]], [[8.0, 5.0, 
5.0], [5.0, 7.0, 6.0], [7.0, 7.0, 6.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 7.0], [4.0, 4.0, 2.0]]], [[[1.0, 7.0, 2.0], [1.0, 8.0, 
6.0], [8.0, 8.0, 7.0]], [[2.0, 2.0, 7.0], [7.0, 5.0, 4.0], [1.0, 3.0, 1.0]], [[2.0, 5.0, 5.0], [1.0, 8.0, 3.0], [3.0, 4.0, 
3.0]]], [[[4.0, 7.0, 7.0], [6.0, 3.0, 7.0], [5.0, 5.0, 2.0]], [[1.0, 4.0, 6.0], [8.0, 2.0, 2.0], [4.0, 6.0, 4.0]], [[5.0, 4.0, 
5.0], [8.0, 5.0, 6.0], [5.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 2.0], [8.0, 2.0, 6.0], [2.0, 3.0, 7.0]], [[7.0, 4.0, 4.0], [7.0, 1.0, 
4.0], [8.0, 4.0, 8.0]], [[1.0, 4.0, 6.0], [4.0, 2.0, 7.0], [2.0, 3.0, 5.0]]], [[[1.0, 4.0, 4.0], [1.0, 7.0, 1.0], [5.0, 1.0, 
6.0]], [[1.0, 5.0, 7.0], [3.0, 3.0, 8.0], [1.0, 3.0, 8.0]], [[6.0, 1.0, 7.0], [6.0, 2.0, 8.0], [1.0, 6.0, 5.0]]], [[[5.0, 2.0, 
5.0], [6.0, 6.0, 7.0], [1.0, 5.0, 4.0]], [[5.0, 2.0, 1.0], [2.0, 1.0, 3.0], [5.0, 2.0, 7.0]], [[4.0, 6.0, 4.0], [1.0, 8.0, 
7.0], [4.0, 1.0, 2.0]]], [[[1.0, 6.0, 5.0], [5.0, 3.0, 7.0], [3.0, 7.0, 5.0]], [[7.0, 2.0, 6.0], [4.0, 1.0, 2.0], [4.0, 1.0, 
1.0]], [[1.0, 6.0, 5.0], [5.0, 2.0, 6.0], [1.0, 6.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[6.0, 8.0, 2.0], [6.0, 2.0, 6.0], [2.0, 7.0, 4.0], [8.0, 3.0, 
1.0]])+(1.-msk_val)*numpy.array([[8.0, 2.0, 5.0], [8.0, 1.0, 6.0], [1.0, 8.0, 1.0], [3.0, 3.0, 7.0]])
      arg[:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 1.0], [5.0, 3.0, 4.0], [4.0, 2.0, 2.0]], [[3.0, 6.0, 2.0], [6.0, 8.0, 2.0], [2.0, 
1.0, 2.0]], [[6.0, 4.0, 8.0], [7.0, 3.0, 1.0], [4.0, 5.0, 4.0]]], [[[6.0, 1.0, 1.0], [6.0, 3.0, 2.0], [1.0, 6.0, 2.0]], [[8.0, 
5.0, 5.0], [6.0, 2.0, 6.0], [7.0, 7.0, 6.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 7.0], [4.0, 4.0, 2.0]]], [[[1.0, 7.0, 2.0], [1.0, 
8.0, 6.0], [8.0, 8.0, 7.0]], [[2.0, 2.0, 7.0], [2.0, 7.0, 4.0], [1.0, 3.0, 1.0]], [[2.0, 5.0, 5.0], [1.0, 8.0, 3.0], [3.0, 4.0, 
3.0]]], [[[4.0, 7.0, 7.0], [6.0, 3.0, 7.0], [5.0, 5.0, 2.0]], [[1.0, 4.0, 6.0], [8.0, 3.0, 1.0], [4.0, 6.0, 4.0]], [[5.0, 4.0, 
5.0], [8.0, 5.0, 6.0], [5.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 2.0], [8.0, 2.0, 6.0], [2.0, 3.0, 7.0]], 
[[7.0, 4.0, 4.0], [8.0, 2.0, 5.0], [8.0, 4.0, 8.0]], [[1.0, 4.0, 6.0], [4.0, 2.0, 7.0], [2.0, 3.0, 5.0]]], [[[1.0, 4.0, 4.0], 
[1.0, 7.0, 1.0], [5.0, 1.0, 6.0]], [[1.0, 5.0, 7.0], [8.0, 1.0, 6.0], [1.0, 3.0, 8.0]], [[6.0, 1.0, 7.0], [6.0, 2.0, 8.0], 
[1.0, 6.0, 5.0]]], [[[5.0, 2.0, 5.0], [6.0, 6.0, 7.0], [1.0, 5.0, 4.0]], [[5.0, 2.0, 1.0], [1.0, 8.0, 1.0], [5.0, 2.0, 7.0]], 
[[4.0, 6.0, 4.0], [1.0, 8.0, 7.0], [4.0, 1.0, 2.0]]], [[[1.0, 6.0, 5.0], [5.0, 3.0, 7.0], [3.0, 7.0, 5.0]], [[7.0, 2.0, 6.0], 
[3.0, 3.0, 7.0], [4.0, 1.0, 1.0]], [[1.0, 6.0, 5.0], [5.0, 2.0, 6.0], [1.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LUCN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 1.0], [8.0, 5.0, 1.0], [7.0, 6.0, 2.0]], [[6.0, 2.0, 4.0], [3.0, 2.0, 7.0], [5.0, 8.0, 
8.0]], [[1.0, 5.0, 1.0], [5.0, 5.0, 8.0], [3.0, 6.0, 4.0]]], [[[7.0, 7.0, 3.0], [4.0, 1.0, 6.0], [8.0, 7.0, 6.0]], [[6.0, 6.0, 
7.0], [1.0, 1.0, 4.0], [5.0, 4.0, 4.0]], [[3.0, 1.0, 4.0], [1.0, 1.0, 8.0], [4.0, 4.0, 2.0]]], [[[2.0, 6.0, 1.0], [2.0, 2.0, 
1.0], [1.0, 5.0, 2.0]], [[3.0, 8.0, 6.0], [6.0, 5.0, 8.0], [2.0, 4.0, 6.0]], [[4.0, 5.0, 5.0], [8.0, 5.0, 1.0], [8.0, 5.0, 
4.0]]], [[[8.0, 2.0, 5.0], [1.0, 6.0, 5.0], [6.0, 1.0, 5.0]], [[6.0, 3.0, 3.0], [8.0, 2.0, 3.0], [3.0, 8.0, 7.0]], [[4.0, 8.0, 
7.0], [5.0, 5.0, 8.0], [3.0, 5.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 8.0], [3.0, 3.0, 2.0], [3.0, 2.0, 6.0]], [[6.0, 4.0, 4.0], [1.0, 1.0, 
1.0], [2.0, 8.0, 7.0]], [[3.0, 8.0, 2.0], [3.0, 8.0, 6.0], [6.0, 3.0, 7.0]]], [[[1.0, 3.0, 4.0], [4.0, 7.0, 2.0], [4.0, 7.0, 
5.0]], [[8.0, 1.0, 6.0], [3.0, 2.0, 1.0], [1.0, 2.0, 8.0]], [[8.0, 6.0, 6.0], [7.0, 2.0, 4.0], [4.0, 8.0, 2.0]]], [[[5.0, 4.0, 
7.0], [5.0, 5.0, 2.0], [2.0, 1.0, 3.0]], [[8.0, 3.0, 2.0], [5.0, 6.0, 1.0], [2.0, 6.0, 6.0]], [[7.0, 3.0, 6.0], [5.0, 3.0, 
3.0], [1.0, 2.0, 6.0]]], [[[4.0, 4.0, 2.0], [4.0, 2.0, 2.0], [1.0, 2.0, 1.0]], [[2.0, 4.0, 4.0], [6.0, 7.0, 4.0], [3.0, 5.0, 
6.0]], [[2.0, 3.0, 5.0], [8.0, 7.0, 8.0], [8.0, 4.0, 4.0]]]]))
      val=numpy.array([[[[3.0, 6.0, 3.0], [8.0, 8.0, 6.0], [8.0, 7.0, 4.0]], [[6.0, 1.0, 1.0], [7.0, 3.0, 3.0], [4.0, 7.0, 
1.0]]], [[[3.0, 3.0, 5.0], [1.0, 8.0, 3.0], [2.0, 4.0, 8.0]], [[2.0, 4.0, 1.0], [3.0, 3.0, 3.0], [8.0, 1.0, 1.0]]]])
      arg[2:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [8.0, 5.0, 1.0], [7.0, 6.0, 2.0]], [[6.0, 2.0, 4.0], [3.0, 2.0, 7.0], [5.0, 
8.0, 8.0]], [[1.0, 5.0, 1.0], [5.0, 5.0, 8.0], [3.0, 6.0, 4.0]]], [[[7.0, 7.0, 3.0], [4.0, 1.0, 6.0], [8.0, 7.0, 6.0]], [[6.0, 
6.0, 7.0], [1.0, 1.0, 4.0], [5.0, 4.0, 4.0]], [[3.0, 1.0, 4.0], [1.0, 1.0, 8.0], [4.0, 4.0, 2.0]]], [[[3.0, 6.0, 3.0], [8.0, 
8.0, 6.0], [8.0, 7.0, 4.0]], [[6.0, 1.0, 1.0], [7.0, 3.0, 3.0], [4.0, 7.0, 1.0]], [[4.0, 5.0, 5.0], [8.0, 5.0, 1.0], [8.0, 5.0, 
4.0]]], [[[3.0, 3.0, 5.0], [1.0, 8.0, 3.0], [2.0, 4.0, 8.0]], [[2.0, 4.0, 1.0], [3.0, 3.0, 3.0], [8.0, 1.0, 1.0]], [[4.0, 8.0, 
7.0], [5.0, 5.0, 8.0], [3.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 8.0], [3.0, 3.0, 2.0], [3.0, 2.0, 6.0]], 
[[6.0, 4.0, 4.0], [1.0, 1.0, 1.0], [2.0, 8.0, 7.0]], [[3.0, 8.0, 2.0], [3.0, 8.0, 6.0], [6.0, 3.0, 7.0]]], [[[1.0, 3.0, 4.0], 
[4.0, 7.0, 2.0], [4.0, 7.0, 5.0]], [[8.0, 1.0, 6.0], [3.0, 2.0, 1.0], [1.0, 2.0, 8.0]], [[8.0, 6.0, 6.0], [7.0, 2.0, 4.0], 
[4.0, 8.0, 2.0]]], [[[3.0, 6.0, 3.0], [8.0, 8.0, 6.0], [8.0, 7.0, 4.0]], [[6.0, 1.0, 1.0], [7.0, 3.0, 3.0], [4.0, 7.0, 1.0]], 
[[7.0, 3.0, 6.0], [5.0, 3.0, 3.0], [1.0, 2.0, 6.0]]], [[[3.0, 3.0, 5.0], [1.0, 8.0, 3.0], [2.0, 4.0, 8.0]], [[2.0, 4.0, 1.0], 
[3.0, 3.0, 3.0], [8.0, 1.0, 1.0]], [[2.0, 3.0, 5.0], [8.0, 7.0, 8.0], [8.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LUCN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 6.0], [2.0, 2.0, 1.0], [7.0, 1.0, 1.0]], [[5.0, 1.0, 7.0], [7.0, 1.0, 5.0], [4.0, 6.0, 
8.0]], [[3.0, 4.0, 1.0], [1.0, 6.0, 4.0], [4.0, 6.0, 2.0]]], [[[1.0, 4.0, 6.0], [6.0, 5.0, 5.0], [5.0, 7.0, 8.0]], [[3.0, 6.0, 
8.0], [2.0, 2.0, 5.0], [3.0, 6.0, 3.0]], [[7.0, 8.0, 1.0], [2.0, 5.0, 5.0], [6.0, 7.0, 4.0]]], [[[8.0, 1.0, 1.0], [2.0, 8.0, 
8.0], [4.0, 6.0, 1.0]], [[6.0, 7.0, 7.0], [5.0, 1.0, 5.0], [3.0, 7.0, 6.0]], [[7.0, 1.0, 8.0], [3.0, 2.0, 4.0], [6.0, 7.0, 
3.0]]], [[[3.0, 8.0, 2.0], [4.0, 7.0, 4.0], [8.0, 8.0, 6.0]], [[1.0, 7.0, 6.0], [3.0, 6.0, 1.0], [1.0, 8.0, 7.0]], [[6.0, 1.0, 
5.0], [1.0, 2.0, 2.0], [1.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 2.0], [3.0, 7.0, 5.0], [1.0, 1.0, 1.0]], [[4.0, 8.0, 3.0], [3.0, 4.0, 
3.0], [7.0, 1.0, 3.0]], [[3.0, 6.0, 6.0], [1.0, 3.0, 8.0], [3.0, 8.0, 5.0]]], [[[7.0, 1.0, 6.0], [3.0, 6.0, 4.0], [5.0, 6.0, 
2.0]], [[4.0, 3.0, 2.0], [7.0, 3.0, 7.0], [3.0, 5.0, 2.0]], [[5.0, 1.0, 1.0], [8.0, 7.0, 5.0], [3.0, 2.0, 4.0]]], [[[1.0, 5.0, 
3.0], [6.0, 7.0, 1.0], [7.0, 2.0, 3.0]], [[3.0, 2.0, 2.0], [4.0, 6.0, 4.0], [7.0, 7.0, 1.0]], [[3.0, 4.0, 3.0], [6.0, 3.0, 
8.0], [3.0, 6.0, 8.0]]], [[[4.0, 2.0, 6.0], [6.0, 4.0, 3.0], [3.0, 1.0, 1.0]], [[4.0, 6.0, 3.0], [5.0, 7.0, 7.0], [3.0, 7.0, 
6.0]], [[1.0, 5.0, 7.0], [1.0, 2.0, 6.0], [1.0, 7.0, 5.0]]]]))
      val=Data(numpy.array([[[[8.0, 6.0, 5.0], [3.0, 3.0, 3.0], [6.0, 1.0, 4.0]], [[5.0, 8.0, 3.0], [2.0, 1.0, 6.0], [5.0, 8.0, 
1.0]]], [[[1.0, 5.0, 7.0], [4.0, 3.0, 7.0], [1.0, 4.0, 8.0]], [[8.0, 7.0, 4.0], [5.0, 7.0, 8.0], [1.0, 7.0, 
2.0]]]]),self.functionspace)
      arg[2:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 6.0], [2.0, 2.0, 1.0], [7.0, 1.0, 1.0]], [[5.0, 1.0, 7.0], [7.0, 1.0, 5.0], [4.0, 
6.0, 8.0]], [[3.0, 4.0, 1.0], [1.0, 6.0, 4.0], [4.0, 6.0, 2.0]]], [[[1.0, 4.0, 6.0], [6.0, 5.0, 5.0], [5.0, 7.0, 8.0]], [[3.0, 
6.0, 8.0], [2.0, 2.0, 5.0], [3.0, 6.0, 3.0]], [[7.0, 8.0, 1.0], [2.0, 5.0, 5.0], [6.0, 7.0, 4.0]]], [[[8.0, 6.0, 5.0], [3.0, 
3.0, 3.0], [6.0, 1.0, 4.0]], [[5.0, 8.0, 3.0], [2.0, 1.0, 6.0], [5.0, 8.0, 1.0]], [[7.0, 1.0, 8.0], [3.0, 2.0, 4.0], [6.0, 7.0, 
3.0]]], [[[1.0, 5.0, 7.0], [4.0, 3.0, 7.0], [1.0, 4.0, 8.0]], [[8.0, 7.0, 4.0], [5.0, 7.0, 8.0], [1.0, 7.0, 2.0]], [[6.0, 1.0, 
5.0], [1.0, 2.0, 2.0], [1.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 2.0], [3.0, 7.0, 5.0], [1.0, 1.0, 1.0]], 
[[4.0, 8.0, 3.0], [3.0, 4.0, 3.0], [7.0, 1.0, 3.0]], [[3.0, 6.0, 6.0], [1.0, 3.0, 8.0], [3.0, 8.0, 5.0]]], [[[7.0, 1.0, 6.0], 
[3.0, 6.0, 4.0], [5.0, 6.0, 2.0]], [[4.0, 3.0, 2.0], [7.0, 3.0, 7.0], [3.0, 5.0, 2.0]], [[5.0, 1.0, 1.0], [8.0, 7.0, 5.0], 
[3.0, 2.0, 4.0]]], [[[8.0, 6.0, 5.0], [3.0, 3.0, 3.0], [6.0, 1.0, 4.0]], [[5.0, 8.0, 3.0], [2.0, 1.0, 6.0], [5.0, 8.0, 1.0]], 
[[3.0, 4.0, 3.0], [6.0, 3.0, 8.0], [3.0, 6.0, 8.0]]], [[[1.0, 5.0, 7.0], [4.0, 3.0, 7.0], [1.0, 4.0, 8.0]], [[8.0, 7.0, 4.0], 
[5.0, 7.0, 8.0], [1.0, 7.0, 2.0]], [[1.0, 5.0, 7.0], [1.0, 2.0, 6.0], [1.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LUCN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 6.0], [7.0, 8.0, 8.0], [4.0, 1.0, 6.0]], [[2.0, 8.0, 4.0], [5.0, 7.0, 3.0], [8.0, 5.0, 
7.0]], [[7.0, 2.0, 6.0], [7.0, 7.0, 4.0], [2.0, 7.0, 6.0]]], [[[3.0, 6.0, 6.0], [3.0, 2.0, 7.0], [8.0, 4.0, 5.0]], [[6.0, 8.0, 
4.0], [4.0, 2.0, 6.0], [7.0, 1.0, 3.0]], [[8.0, 1.0, 7.0], [1.0, 4.0, 5.0], [5.0, 7.0, 3.0]]], [[[3.0, 2.0, 5.0], [6.0, 3.0, 
8.0], [8.0, 5.0, 8.0]], [[8.0, 6.0, 4.0], [7.0, 3.0, 2.0], [8.0, 8.0, 8.0]], [[4.0, 6.0, 5.0], [6.0, 2.0, 2.0], [7.0, 8.0, 
7.0]]], [[[8.0, 3.0, 6.0], [2.0, 7.0, 1.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 5.0], [3.0, 2.0, 3.0], [3.0, 7.0, 8.0]], [[4.0, 1.0, 
8.0], [4.0, 2.0, 2.0], [7.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 2.0], [5.0, 6.0, 1.0], [7.0, 3.0, 8.0]], [[6.0, 1.0, 7.0], [8.0, 2.0, 
2.0], [5.0, 6.0, 1.0]], [[7.0, 3.0, 7.0], [1.0, 3.0, 7.0], [8.0, 5.0, 4.0]]], [[[4.0, 2.0, 2.0], [3.0, 1.0, 3.0], [2.0, 5.0, 
7.0]], [[7.0, 3.0, 4.0], [4.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[7.0, 5.0, 1.0], [2.0, 7.0, 6.0], [5.0, 2.0, 1.0]]], [[[4.0, 6.0, 
8.0], [5.0, 4.0, 3.0], [2.0, 1.0, 8.0]], [[4.0, 3.0, 5.0], [3.0, 4.0, 7.0], [4.0, 6.0, 2.0]], [[4.0, 4.0, 8.0], [5.0, 7.0, 
7.0], [4.0, 6.0, 4.0]]], [[[4.0, 6.0, 1.0], [4.0, 7.0, 5.0], [2.0, 8.0, 6.0]], [[8.0, 6.0, 2.0], [2.0, 5.0, 3.0], [2.0, 5.0, 
7.0]], [[6.0, 4.0, 6.0], [4.0, 3.0, 7.0], [5.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[[[8.0, 5.0, 2.0], [8.0, 6.0, 8.0], [3.0, 5.0, 5.0]], [[4.0, 2.0, 4.0], [5.0, 1.0, 6.0], [6.0, 3.0, 
5.0]]], [[[5.0, 6.0, 8.0], [8.0, 6.0, 8.0], [5.0, 4.0, 4.0]], [[2.0, 5.0, 6.0], [3.0, 4.0, 2.0], [7.0, 1.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 4.0], [8.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[5.0, 1.0, 8.0], [5.0, 2.0, 
3.0], [5.0, 1.0, 1.0]]], [[[1.0, 8.0, 5.0], [4.0, 6.0, 4.0], [2.0, 8.0, 6.0]], [[3.0, 8.0, 6.0], [6.0, 7.0, 6.0], [7.0, 6.0, 
5.0]]]]))
      arg[2:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 6.0], [7.0, 8.0, 8.0], [4.0, 1.0, 6.0]], [[2.0, 8.0, 4.0], [5.0, 7.0, 3.0], [8.0, 
5.0, 7.0]], [[7.0, 2.0, 6.0], [7.0, 7.0, 4.0], [2.0, 7.0, 6.0]]], [[[3.0, 6.0, 6.0], [3.0, 2.0, 7.0], [8.0, 4.0, 5.0]], [[6.0, 
8.0, 4.0], [4.0, 2.0, 6.0], [7.0, 1.0, 3.0]], [[8.0, 1.0, 7.0], [1.0, 4.0, 5.0], [5.0, 7.0, 3.0]]], [[[8.0, 5.0, 2.0], [8.0, 
6.0, 8.0], [3.0, 5.0, 5.0]], [[4.0, 2.0, 4.0], [5.0, 1.0, 6.0], [6.0, 3.0, 5.0]], [[4.0, 6.0, 5.0], [6.0, 2.0, 2.0], [7.0, 8.0, 
7.0]]], [[[5.0, 6.0, 8.0], [8.0, 6.0, 8.0], [5.0, 4.0, 4.0]], [[2.0, 5.0, 6.0], [3.0, 4.0, 2.0], [7.0, 1.0, 1.0]], [[4.0, 1.0, 
8.0], [4.0, 2.0, 2.0], [7.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 2.0], [5.0, 6.0, 1.0], [7.0, 3.0, 8.0]], 
[[6.0, 1.0, 7.0], [8.0, 2.0, 2.0], [5.0, 6.0, 1.0]], [[7.0, 3.0, 7.0], [1.0, 3.0, 7.0], [8.0, 5.0, 4.0]]], [[[4.0, 2.0, 2.0], 
[3.0, 1.0, 3.0], [2.0, 5.0, 7.0]], [[7.0, 3.0, 4.0], [4.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[7.0, 5.0, 1.0], [2.0, 7.0, 6.0], 
[5.0, 2.0, 1.0]]], [[[4.0, 1.0, 4.0], [8.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[5.0, 1.0, 8.0], [5.0, 2.0, 3.0], [5.0, 1.0, 1.0]], 
[[4.0, 4.0, 8.0], [5.0, 7.0, 7.0], [4.0, 6.0, 4.0]]], [[[1.0, 8.0, 5.0], [4.0, 6.0, 4.0], [2.0, 8.0, 6.0]], [[3.0, 8.0, 6.0], 
[6.0, 7.0, 6.0], [7.0, 6.0, 5.0]], [[6.0, 4.0, 6.0], [4.0, 3.0, 7.0], [5.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LUCN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 8.0], [7.0, 4.0, 7.0], [1.0, 7.0, 1.0]], [[5.0, 2.0, 8.0], [8.0, 8.0, 5.0], [5.0, 8.0, 
8.0]], [[5.0, 4.0, 2.0], [2.0, 4.0, 8.0], [7.0, 6.0, 8.0]]], [[[5.0, 7.0, 4.0], [6.0, 1.0, 1.0], [8.0, 7.0, 3.0]], [[1.0, 6.0, 
8.0], [6.0, 5.0, 5.0], [8.0, 1.0, 5.0]], [[4.0, 1.0, 6.0], [4.0, 4.0, 3.0], [5.0, 8.0, 3.0]]], [[[3.0, 5.0, 1.0], [4.0, 4.0, 
8.0], [8.0, 2.0, 6.0]], [[3.0, 8.0, 2.0], [8.0, 7.0, 5.0], [5.0, 7.0, 6.0]], [[8.0, 7.0, 7.0], [4.0, 2.0, 6.0], [1.0, 4.0, 
7.0]]], [[[7.0, 8.0, 7.0], [2.0, 2.0, 5.0], [8.0, 7.0, 2.0]], [[4.0, 5.0, 5.0], [2.0, 5.0, 1.0], [5.0, 8.0, 1.0]], [[4.0, 2.0, 
1.0], [4.0, 8.0, 3.0], [6.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 3.0], [8.0, 5.0, 4.0], [7.0, 7.0, 3.0]], [[3.0, 5.0, 3.0], [3.0, 7.0, 
5.0], [5.0, 1.0, 8.0]], [[1.0, 5.0, 1.0], [1.0, 5.0, 4.0], [5.0, 2.0, 2.0]]], [[[6.0, 6.0, 5.0], [1.0, 4.0, 6.0], [7.0, 8.0, 
5.0]], [[6.0, 5.0, 7.0], [7.0, 1.0, 2.0], [8.0, 5.0, 8.0]], [[5.0, 8.0, 7.0], [2.0, 1.0, 5.0], [6.0, 6.0, 1.0]]], [[[8.0, 4.0, 
6.0], [1.0, 1.0, 5.0], [5.0, 4.0, 4.0]], [[1.0, 2.0, 3.0], [4.0, 1.0, 4.0], [3.0, 3.0, 5.0]], [[2.0, 5.0, 7.0], [1.0, 4.0, 
1.0], [3.0, 5.0, 7.0]]], [[[1.0, 5.0, 4.0], [3.0, 2.0, 4.0], [2.0, 7.0, 6.0]], [[2.0, 5.0, 5.0], [5.0, 4.0, 3.0], [8.0, 3.0, 
1.0]], [[2.0, 1.0, 5.0], [6.0, 4.0, 2.0], [6.0, 8.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0, 3.0, 8.0], [6.0, 6.0, 4.0], [8.0, 1.0, 1.0]], [[1.0, 2.0, 3.0], [4.0, 6.0, 1.0], [7.0, 
8.0, 6.0]]], [[[4.0, 7.0, 6.0], [6.0, 3.0, 8.0], [4.0, 7.0, 4.0]], [[4.0, 3.0, 4.0], [1.0, 7.0, 2.0], [1.0, 1.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 7.0, 8.0], [3.0, 8.0, 6.0], [1.0, 1.0, 2.0]], [[3.0, 2.0, 1.0], [6.0, 7.0, 3.0], 
[8.0, 3.0, 7.0]]], [[[1.0, 5.0, 8.0], [6.0, 3.0, 2.0], [1.0, 1.0, 3.0]], [[1.0, 2.0, 7.0], [1.0, 6.0, 1.0], [2.0, 5.0, 3.0]]]])
      arg[2:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 8.0], [7.0, 4.0, 7.0], [1.0, 7.0, 1.0]], [[5.0, 2.0, 8.0], [8.0, 8.0, 5.0], [5.0, 
8.0, 8.0]], [[5.0, 4.0, 2.0], [2.0, 4.0, 8.0], [7.0, 6.0, 8.0]]], [[[5.0, 7.0, 4.0], [6.0, 1.0, 1.0], [8.0, 7.0, 3.0]], [[1.0, 
6.0, 8.0], [6.0, 5.0, 5.0], [8.0, 1.0, 5.0]], [[4.0, 1.0, 6.0], [4.0, 4.0, 3.0], [5.0, 8.0, 3.0]]], [[[4.0, 3.0, 8.0], [6.0, 
6.0, 4.0], [8.0, 1.0, 1.0]], [[1.0, 2.0, 3.0], [4.0, 6.0, 1.0], [7.0, 8.0, 6.0]], [[8.0, 7.0, 7.0], [4.0, 2.0, 6.0], [1.0, 4.0, 
7.0]]], [[[4.0, 7.0, 6.0], [6.0, 3.0, 8.0], [4.0, 7.0, 4.0]], [[4.0, 3.0, 4.0], [1.0, 7.0, 2.0], [1.0, 1.0, 6.0]], [[4.0, 2.0, 
1.0], [4.0, 8.0, 3.0], [6.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 3.0], [8.0, 5.0, 4.0], [7.0, 7.0, 3.0]], 
[[3.0, 5.0, 3.0], [3.0, 7.0, 5.0], [5.0, 1.0, 8.0]], [[1.0, 5.0, 1.0], [1.0, 5.0, 4.0], [5.0, 2.0, 2.0]]], [[[6.0, 6.0, 5.0], 
[1.0, 4.0, 6.0], [7.0, 8.0, 5.0]], [[6.0, 5.0, 7.0], [7.0, 1.0, 2.0], [8.0, 5.0, 8.0]], [[5.0, 8.0, 7.0], [2.0, 1.0, 5.0], 
[6.0, 6.0, 1.0]]], [[[2.0, 7.0, 8.0], [3.0, 8.0, 6.0], [1.0, 1.0, 2.0]], [[3.0, 2.0, 1.0], [6.0, 7.0, 3.0], [8.0, 3.0, 7.0]], 
[[2.0, 5.0, 7.0], [1.0, 4.0, 1.0], [3.0, 5.0, 7.0]]], [[[1.0, 5.0, 8.0], [6.0, 3.0, 2.0], [1.0, 1.0, 3.0]], [[1.0, 2.0, 7.0], 
[1.0, 6.0, 1.0], [2.0, 5.0, 3.0]], [[2.0, 1.0, 5.0], [6.0, 4.0, 2.0], [6.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RCCC(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 1.0], [1.0, 7.0, 6.0], [6.0, 5.0, 7.0]], [[3.0, 5.0, 4.0], [7.0, 2.0, 6.0], [3.0, 8.0, 
6.0]], [[2.0, 4.0, 2.0], [2.0, 5.0, 5.0], [3.0, 5.0, 3.0]]], [[[1.0, 5.0, 4.0], [4.0, 4.0, 2.0], [7.0, 6.0, 8.0]], [[1.0, 2.0, 
8.0], [3.0, 7.0, 2.0], [7.0, 6.0, 5.0]], [[8.0, 4.0, 5.0], [5.0, 6.0, 2.0], [5.0, 8.0, 5.0]]], [[[7.0, 5.0, 2.0], [3.0, 3.0, 
6.0], [7.0, 3.0, 6.0]], [[2.0, 5.0, 5.0], [7.0, 3.0, 1.0], [8.0, 1.0, 1.0]], [[6.0, 8.0, 3.0], [5.0, 7.0, 3.0], [8.0, 4.0, 
2.0]]], [[[1.0, 8.0, 5.0], [5.0, 1.0, 8.0], [3.0, 3.0, 8.0]], [[5.0, 3.0, 8.0], [1.0, 2.0, 3.0], [3.0, 1.0, 7.0]], [[5.0, 6.0, 
4.0], [3.0, 1.0, 1.0], [6.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 6.0], [5.0, 6.0, 4.0], [5.0, 4.0, 4.0]], [[7.0, 1.0, 8.0], [2.0, 5.0, 
8.0], [7.0, 1.0, 5.0]], [[4.0, 2.0, 2.0], [2.0, 8.0, 7.0], [7.0, 8.0, 6.0]]], [[[6.0, 4.0, 1.0], [6.0, 1.0, 3.0], [2.0, 2.0, 
2.0]], [[2.0, 8.0, 1.0], [5.0, 2.0, 1.0], [1.0, 4.0, 5.0]], [[3.0, 7.0, 6.0], [7.0, 8.0, 3.0], [7.0, 3.0, 4.0]]], [[[2.0, 1.0, 
6.0], [1.0, 2.0, 8.0], [5.0, 7.0, 3.0]], [[4.0, 1.0, 5.0], [2.0, 5.0, 3.0], [8.0, 7.0, 4.0]], [[1.0, 7.0, 3.0], [7.0, 6.0, 
8.0], [2.0, 2.0, 6.0]]], [[[5.0, 6.0, 5.0], [8.0, 4.0, 6.0], [5.0, 8.0, 3.0]], [[8.0, 3.0, 3.0], [6.0, 3.0, 1.0], [5.0, 3.0, 
7.0]], [[5.0, 4.0, 1.0], [8.0, 5.0, 8.0], [8.0, 7.0, 1.0]]]]))
      val=numpy.array([[[[7.0, 5.0, 5.0], [3.0, 1.0, 5.0], [2.0, 8.0, 8.0]], [[6.0, 3.0, 8.0], [7.0, 6.0, 5.0], [2.0, 8.0, 
8.0]], [[7.0, 6.0, 5.0], [1.0, 4.0, 8.0], [4.0, 7.0, 8.0]]], [[[6.0, 1.0, 1.0], [4.0, 3.0, 2.0], [3.0, 4.0, 7.0]], [[5.0, 6.0, 
6.0], [1.0, 7.0, 2.0], [5.0, 7.0, 6.0]], [[2.0, 6.0, 2.0], [4.0, 5.0, 5.0], [2.0, 3.0, 1.0]]]])
      arg[1:3,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 1.0], [1.0, 7.0, 6.0], [6.0, 5.0, 7.0]], [[3.0, 5.0, 4.0], [7.0, 2.0, 6.0], [3.0, 
8.0, 6.0]], [[2.0, 4.0, 2.0], [2.0, 5.0, 5.0], [3.0, 5.0, 3.0]]], [[[7.0, 5.0, 5.0], [3.0, 1.0, 5.0], [2.0, 8.0, 8.0]], [[6.0, 
3.0, 8.0], [7.0, 6.0, 5.0], [2.0, 8.0, 8.0]], [[7.0, 6.0, 5.0], [1.0, 4.0, 8.0], [4.0, 7.0, 8.0]]], [[[6.0, 1.0, 1.0], [4.0, 
3.0, 2.0], [3.0, 4.0, 7.0]], [[5.0, 6.0, 6.0], [1.0, 7.0, 2.0], [5.0, 7.0, 6.0]], [[2.0, 6.0, 2.0], [4.0, 5.0, 5.0], [2.0, 3.0, 
1.0]]], [[[1.0, 8.0, 5.0], [5.0, 1.0, 8.0], [3.0, 3.0, 8.0]], [[5.0, 3.0, 8.0], [1.0, 2.0, 3.0], [3.0, 1.0, 7.0]], [[5.0, 6.0, 
4.0], [3.0, 1.0, 1.0], [6.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 6.0], [5.0, 6.0, 4.0], [5.0, 4.0, 4.0]], 
[[7.0, 1.0, 8.0], [2.0, 5.0, 8.0], [7.0, 1.0, 5.0]], [[4.0, 2.0, 2.0], [2.0, 8.0, 7.0], [7.0, 8.0, 6.0]]], [[[7.0, 5.0, 5.0], 
[3.0, 1.0, 5.0], [2.0, 8.0, 8.0]], [[6.0, 3.0, 8.0], [7.0, 6.0, 5.0], [2.0, 8.0, 8.0]], [[7.0, 6.0, 5.0], [1.0, 4.0, 8.0], 
[4.0, 7.0, 8.0]]], [[[6.0, 1.0, 1.0], [4.0, 3.0, 2.0], [3.0, 4.0, 7.0]], [[5.0, 6.0, 6.0], [1.0, 7.0, 2.0], [5.0, 7.0, 6.0]], 
[[2.0, 6.0, 2.0], [4.0, 5.0, 5.0], [2.0, 3.0, 1.0]]], [[[5.0, 6.0, 5.0], [8.0, 4.0, 6.0], [5.0, 8.0, 3.0]], [[8.0, 3.0, 3.0], 
[6.0, 3.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 4.0, 1.0], [8.0, 5.0, 8.0], [8.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RCCC(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 7.0], [1.0, 1.0, 3.0], [3.0, 4.0, 1.0]], [[4.0, 8.0, 3.0], [8.0, 4.0, 8.0], [5.0, 6.0, 
2.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 6.0], [1.0, 7.0, 7.0]]], [[[2.0, 8.0, 3.0], [2.0, 2.0, 8.0], [1.0, 3.0, 1.0]], [[1.0, 8.0, 
3.0], [4.0, 6.0, 1.0], [8.0, 6.0, 1.0]], [[1.0, 2.0, 6.0], [2.0, 8.0, 8.0], [7.0, 4.0, 3.0]]], [[[2.0, 1.0, 4.0], [3.0, 5.0, 
8.0], [3.0, 2.0, 8.0]], [[8.0, 8.0, 7.0], [5.0, 6.0, 4.0], [6.0, 3.0, 4.0]], [[8.0, 2.0, 2.0], [6.0, 3.0, 1.0], [3.0, 1.0, 
8.0]]], [[[4.0, 5.0, 6.0], [6.0, 5.0, 5.0], [4.0, 6.0, 6.0]], [[5.0, 2.0, 3.0], [4.0, 3.0, 6.0], [4.0, 2.0, 2.0]], [[6.0, 5.0, 
7.0], [6.0, 6.0, 4.0], [1.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 3.0], [6.0, 4.0, 3.0], [4.0, 3.0, 3.0]], [[2.0, 1.0, 1.0], [8.0, 6.0, 
6.0], [2.0, 2.0, 4.0]], [[3.0, 1.0, 6.0], [4.0, 6.0, 3.0], [2.0, 3.0, 1.0]]], [[[1.0, 2.0, 3.0], [6.0, 7.0, 4.0], [4.0, 7.0, 
4.0]], [[1.0, 3.0, 7.0], [3.0, 1.0, 7.0], [2.0, 3.0, 1.0]], [[7.0, 2.0, 2.0], [1.0, 1.0, 8.0], [1.0, 2.0, 4.0]]], [[[1.0, 8.0, 
3.0], [1.0, 8.0, 1.0], [7.0, 6.0, 8.0]], [[2.0, 3.0, 7.0], [1.0, 6.0, 8.0], [2.0, 6.0, 4.0]], [[1.0, 1.0, 5.0], [5.0, 3.0, 
8.0], [6.0, 4.0, 4.0]]], [[[1.0, 6.0, 3.0], [4.0, 3.0, 2.0], [5.0, 1.0, 6.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 5.0], [6.0, 1.0, 
4.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 1.0], [8.0, 1.0, 2.0]]]]))
      val=Data(numpy.array([[[[8.0, 1.0, 1.0], [2.0, 8.0, 2.0], [3.0, 4.0, 1.0]], [[8.0, 4.0, 5.0], [2.0, 8.0, 1.0], [4.0, 5.0, 
4.0]], [[2.0, 2.0, 2.0], [2.0, 4.0, 5.0], [7.0, 6.0, 5.0]]], [[[7.0, 4.0, 1.0], [5.0, 3.0, 8.0], [3.0, 8.0, 6.0]], [[1.0, 5.0, 
4.0], [2.0, 2.0, 4.0], [8.0, 7.0, 8.0]], [[8.0, 5.0, 1.0], [2.0, 1.0, 8.0], [4.0, 7.0, 5.0]]]]),self.functionspace)
      arg[1:3,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 7.0], [1.0, 1.0, 3.0], [3.0, 4.0, 1.0]], [[4.0, 8.0, 3.0], [8.0, 4.0, 8.0], [5.0, 
6.0, 2.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 6.0], [1.0, 7.0, 7.0]]], [[[8.0, 1.0, 1.0], [2.0, 8.0, 2.0], [3.0, 4.0, 1.0]], [[8.0, 
4.0, 5.0], [2.0, 8.0, 1.0], [4.0, 5.0, 4.0]], [[2.0, 2.0, 2.0], [2.0, 4.0, 5.0], [7.0, 6.0, 5.0]]], [[[7.0, 4.0, 1.0], [5.0, 
3.0, 8.0], [3.0, 8.0, 6.0]], [[1.0, 5.0, 4.0], [2.0, 2.0, 4.0], [8.0, 7.0, 8.0]], [[8.0, 5.0, 1.0], [2.0, 1.0, 8.0], [4.0, 7.0, 
5.0]]], [[[4.0, 5.0, 6.0], [6.0, 5.0, 5.0], [4.0, 6.0, 6.0]], [[5.0, 2.0, 3.0], [4.0, 3.0, 6.0], [4.0, 2.0, 2.0]], [[6.0, 5.0, 
7.0], [6.0, 6.0, 4.0], [1.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 3.0], [6.0, 4.0, 3.0], [4.0, 3.0, 3.0]], 
[[2.0, 1.0, 1.0], [8.0, 6.0, 6.0], [2.0, 2.0, 4.0]], [[3.0, 1.0, 6.0], [4.0, 6.0, 3.0], [2.0, 3.0, 1.0]]], [[[8.0, 1.0, 1.0], 
[2.0, 8.0, 2.0], [3.0, 4.0, 1.0]], [[8.0, 4.0, 5.0], [2.0, 8.0, 1.0], [4.0, 5.0, 4.0]], [[2.0, 2.0, 2.0], [2.0, 4.0, 5.0], 
[7.0, 6.0, 5.0]]], [[[7.0, 4.0, 1.0], [5.0, 3.0, 8.0], [3.0, 8.0, 6.0]], [[1.0, 5.0, 4.0], [2.0, 2.0, 4.0], [8.0, 7.0, 8.0]], 
[[8.0, 5.0, 1.0], [2.0, 1.0, 8.0], [4.0, 7.0, 5.0]]], [[[1.0, 6.0, 3.0], [4.0, 3.0, 2.0], [5.0, 1.0, 6.0]], [[2.0, 1.0, 7.0], 
[2.0, 7.0, 5.0], [6.0, 1.0, 4.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 1.0], [8.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RCCC(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 3.0], [6.0, 3.0, 5.0], [4.0, 4.0, 2.0]], [[7.0, 4.0, 3.0], [3.0, 4.0, 1.0], [4.0, 6.0, 
4.0]], [[4.0, 2.0, 1.0], [7.0, 6.0, 8.0], [7.0, 3.0, 8.0]]], [[[4.0, 2.0, 3.0], [4.0, 6.0, 6.0], [3.0, 1.0, 6.0]], [[2.0, 3.0, 
5.0], [1.0, 4.0, 6.0], [8.0, 1.0, 8.0]], [[2.0, 6.0, 1.0], [6.0, 1.0, 3.0], [8.0, 8.0, 5.0]]], [[[4.0, 2.0, 8.0], [3.0, 5.0, 
2.0], [5.0, 1.0, 2.0]], [[6.0, 8.0, 2.0], [5.0, 5.0, 6.0], [5.0, 6.0, 7.0]], [[4.0, 1.0, 5.0], [5.0, 5.0, 6.0], [6.0, 7.0, 
2.0]]], [[[8.0, 3.0, 4.0], [4.0, 2.0, 8.0], [6.0, 1.0, 2.0]], [[8.0, 2.0, 3.0], [4.0, 5.0, 2.0], [6.0, 5.0, 1.0]], [[2.0, 1.0, 
1.0], [4.0, 1.0, 7.0], [4.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 6.0], [6.0, 5.0, 1.0], [4.0, 1.0, 3.0]], [[4.0, 1.0, 2.0], [7.0, 8.0, 
4.0], [8.0, 2.0, 6.0]], [[3.0, 2.0, 4.0], [1.0, 5.0, 6.0], [6.0, 8.0, 4.0]]], [[[2.0, 4.0, 1.0], [7.0, 4.0, 6.0], [5.0, 8.0, 
6.0]], [[3.0, 2.0, 3.0], [2.0, 4.0, 6.0], [4.0, 7.0, 4.0]], [[3.0, 1.0, 4.0], [1.0, 6.0, 2.0], [5.0, 3.0, 3.0]]], [[[7.0, 5.0, 
4.0], [2.0, 2.0, 4.0], [4.0, 5.0, 5.0]], [[3.0, 5.0, 2.0], [5.0, 2.0, 8.0], [5.0, 4.0, 4.0]], [[6.0, 6.0, 1.0], [6.0, 1.0, 
7.0], [3.0, 4.0, 4.0]]], [[[2.0, 1.0, 8.0], [6.0, 4.0, 3.0], [6.0, 3.0, 4.0]], [[7.0, 8.0, 1.0], [7.0, 8.0, 8.0], [3.0, 8.0, 
6.0]], [[6.0, 7.0, 4.0], [4.0, 3.0, 7.0], [6.0, 1.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0, 1.0, 6.0], [5.0, 7.0, 5.0], [7.0, 8.0, 7.0]], [[1.0, 5.0, 3.0], [4.0, 5.0, 5.0], [1.0, 3.0, 
3.0]], [[5.0, 4.0, 1.0], [5.0, 1.0, 5.0], [8.0, 6.0, 1.0]]], [[[3.0, 4.0, 1.0], [8.0, 8.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 7.0, 
4.0], [3.0, 8.0, 5.0], [2.0, 2.0, 3.0]], [[8.0, 7.0, 2.0], [6.0, 7.0, 5.0], [6.0, 1.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 1.0], [2.0, 2.0, 5.0], [6.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [1.0, 1.0, 
8.0], [7.0, 8.0, 1.0]], [[8.0, 5.0, 4.0], [5.0, 1.0, 8.0], [2.0, 8.0, 4.0]]], [[[6.0, 6.0, 2.0], [2.0, 6.0, 7.0], [5.0, 5.0, 
7.0]], [[4.0, 2.0, 6.0], [8.0, 3.0, 4.0], [5.0, 6.0, 1.0]], [[4.0, 5.0, 8.0], [4.0, 4.0, 3.0], [1.0, 7.0, 1.0]]]]))
      arg[1:3,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 3.0], [6.0, 3.0, 5.0], [4.0, 4.0, 2.0]], [[7.0, 4.0, 3.0], [3.0, 4.0, 1.0], [4.0, 
6.0, 4.0]], [[4.0, 2.0, 1.0], [7.0, 6.0, 8.0], [7.0, 3.0, 8.0]]], [[[5.0, 1.0, 6.0], [5.0, 7.0, 5.0], [7.0, 8.0, 7.0]], [[1.0, 
5.0, 3.0], [4.0, 5.0, 5.0], [1.0, 3.0, 3.0]], [[5.0, 4.0, 1.0], [5.0, 1.0, 5.0], [8.0, 6.0, 1.0]]], [[[3.0, 4.0, 1.0], [8.0, 
8.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 7.0, 4.0], [3.0, 8.0, 5.0], [2.0, 2.0, 3.0]], [[8.0, 7.0, 2.0], [6.0, 7.0, 5.0], [6.0, 1.0, 
3.0]]], [[[8.0, 3.0, 4.0], [4.0, 2.0, 8.0], [6.0, 1.0, 2.0]], [[8.0, 2.0, 3.0], [4.0, 5.0, 2.0], [6.0, 5.0, 1.0]], [[2.0, 1.0, 
1.0], [4.0, 1.0, 7.0], [4.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 6.0], [6.0, 5.0, 1.0], [4.0, 1.0, 3.0]], 
[[4.0, 1.0, 2.0], [7.0, 8.0, 4.0], [8.0, 2.0, 6.0]], [[3.0, 2.0, 4.0], [1.0, 5.0, 6.0], [6.0, 8.0, 4.0]]], [[[8.0, 2.0, 1.0], 
[2.0, 2.0, 5.0], [6.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [1.0, 1.0, 8.0], [7.0, 8.0, 1.0]], [[8.0, 5.0, 4.0], [5.0, 1.0, 8.0], 
[2.0, 8.0, 4.0]]], [[[6.0, 6.0, 2.0], [2.0, 6.0, 7.0], [5.0, 5.0, 7.0]], [[4.0, 2.0, 6.0], [8.0, 3.0, 4.0], [5.0, 6.0, 1.0]], 
[[4.0, 5.0, 8.0], [4.0, 4.0, 3.0], [1.0, 7.0, 1.0]]], [[[2.0, 1.0, 8.0], [6.0, 4.0, 3.0], [6.0, 3.0, 4.0]], [[7.0, 8.0, 1.0], 
[7.0, 8.0, 8.0], [3.0, 8.0, 6.0]], [[6.0, 7.0, 4.0], [4.0, 3.0, 7.0], [6.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RCCC(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 4.0], [3.0, 1.0, 2.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 4.0], [7.0, 1.0, 4.0], [1.0, 2.0, 
3.0]], [[1.0, 7.0, 2.0], [6.0, 4.0, 8.0], [1.0, 8.0, 7.0]]], [[[2.0, 3.0, 1.0], [3.0, 8.0, 1.0], [4.0, 3.0, 5.0]], [[8.0, 3.0, 
8.0], [8.0, 2.0, 1.0], [6.0, 7.0, 4.0]], [[4.0, 1.0, 7.0], [6.0, 1.0, 6.0], [4.0, 3.0, 5.0]]], [[[3.0, 6.0, 8.0], [6.0, 6.0, 
3.0], [2.0, 3.0, 6.0]], [[4.0, 1.0, 5.0], [6.0, 5.0, 3.0], [5.0, 3.0, 1.0]], [[4.0, 8.0, 6.0], [2.0, 4.0, 5.0], [3.0, 6.0, 
5.0]]], [[[1.0, 6.0, 6.0], [1.0, 5.0, 7.0], [1.0, 2.0, 3.0]], [[7.0, 5.0, 6.0], [8.0, 1.0, 1.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 
1.0], [4.0, 6.0, 7.0], [1.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 4.0], [4.0, 8.0, 5.0], [8.0, 8.0, 6.0]], [[3.0, 1.0, 4.0], [3.0, 5.0, 
5.0], [7.0, 7.0, 7.0]], [[5.0, 3.0, 7.0], [5.0, 7.0, 7.0], [7.0, 4.0, 4.0]]], [[[5.0, 6.0, 7.0], [7.0, 1.0, 7.0], [6.0, 1.0, 
3.0]], [[8.0, 6.0, 5.0], [1.0, 8.0, 5.0], [1.0, 3.0, 6.0]], [[4.0, 5.0, 4.0], [7.0, 2.0, 3.0], [3.0, 6.0, 2.0]]], [[[3.0, 8.0, 
7.0], [8.0, 1.0, 7.0], [4.0, 4.0, 7.0]], [[3.0, 1.0, 6.0], [3.0, 3.0, 5.0], [1.0, 6.0, 3.0]], [[4.0, 7.0, 7.0], [5.0, 3.0, 
5.0], [1.0, 2.0, 7.0]]], [[[2.0, 3.0, 6.0], [6.0, 8.0, 2.0], [7.0, 5.0, 7.0]], [[8.0, 3.0, 4.0], [5.0, 4.0, 8.0], [4.0, 6.0, 
1.0]], [[4.0, 1.0, 2.0], [3.0, 1.0, 3.0], [7.0, 4.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0, 2.0, 1.0], [1.0, 3.0, 2.0], [7.0, 1.0, 2.0]], [[8.0, 3.0, 1.0], [4.0, 7.0, 5.0], [3.0, 
1.0, 4.0]], [[6.0, 8.0, 2.0], [7.0, 7.0, 5.0], [4.0, 8.0, 1.0]]], [[[7.0, 3.0, 7.0], [1.0, 3.0, 1.0], [8.0, 2.0, 1.0]], [[5.0, 
1.0, 3.0], [5.0, 5.0, 5.0], [7.0, 3.0, 4.0]], [[4.0, 5.0, 6.0], [1.0, 7.0, 4.0], [3.0, 3.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 8.0, 2.0], [6.0, 7.0, 4.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 2.0], [1.0, 4.0, 1.0], 
[7.0, 2.0, 2.0]], [[8.0, 5.0, 4.0], [2.0, 2.0, 4.0], [4.0, 2.0, 3.0]]], [[[3.0, 3.0, 2.0], [8.0, 8.0, 8.0], [5.0, 1.0, 2.0]], 
[[8.0, 4.0, 8.0], [6.0, 6.0, 7.0], [2.0, 2.0, 8.0]], [[7.0, 2.0, 6.0], [2.0, 3.0, 6.0], [6.0, 3.0, 5.0]]]])
      arg[1:3,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 4.0], [3.0, 1.0, 2.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 4.0], [7.0, 1.0, 4.0], [1.0, 
2.0, 3.0]], [[1.0, 7.0, 2.0], [6.0, 4.0, 8.0], [1.0, 8.0, 7.0]]], [[[6.0, 2.0, 1.0], [1.0, 3.0, 2.0], [7.0, 1.0, 2.0]], [[8.0, 
3.0, 1.0], [4.0, 7.0, 5.0], [3.0, 1.0, 4.0]], [[6.0, 8.0, 2.0], [7.0, 7.0, 5.0], [4.0, 8.0, 1.0]]], [[[7.0, 3.0, 7.0], [1.0, 
3.0, 1.0], [8.0, 2.0, 1.0]], [[5.0, 1.0, 3.0], [5.0, 5.0, 5.0], [7.0, 3.0, 4.0]], [[4.0, 5.0, 6.0], [1.0, 7.0, 4.0], [3.0, 3.0, 
7.0]]], [[[1.0, 6.0, 6.0], [1.0, 5.0, 7.0], [1.0, 2.0, 3.0]], [[7.0, 5.0, 6.0], [8.0, 1.0, 1.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 
1.0], [4.0, 6.0, 7.0], [1.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 4.0], [4.0, 8.0, 5.0], [8.0, 8.0, 6.0]], 
[[3.0, 1.0, 4.0], [3.0, 5.0, 5.0], [7.0, 7.0, 7.0]], [[5.0, 3.0, 7.0], [5.0, 7.0, 7.0], [7.0, 4.0, 4.0]]], [[[7.0, 8.0, 2.0], 
[6.0, 7.0, 4.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 2.0], [1.0, 4.0, 1.0], [7.0, 2.0, 2.0]], [[8.0, 5.0, 4.0], [2.0, 2.0, 4.0], 
[4.0, 2.0, 3.0]]], [[[3.0, 3.0, 2.0], [8.0, 8.0, 8.0], [5.0, 1.0, 2.0]], [[8.0, 4.0, 8.0], [6.0, 6.0, 7.0], [2.0, 2.0, 8.0]], 
[[7.0, 2.0, 6.0], [2.0, 3.0, 6.0], [6.0, 3.0, 5.0]]], [[[2.0, 3.0, 6.0], [6.0, 8.0, 2.0], [7.0, 5.0, 7.0]], [[8.0, 3.0, 4.0], 
[5.0, 4.0, 8.0], [4.0, 6.0, 1.0]], [[4.0, 1.0, 2.0], [3.0, 1.0, 3.0], [7.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULCN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 4.0], [3.0, 1.0, 4.0], [7.0, 1.0, 2.0]], [[2.0, 1.0, 1.0], [8.0, 6.0, 3.0], [4.0, 8.0, 
6.0]], [[1.0, 8.0, 5.0], [4.0, 7.0, 7.0], [6.0, 6.0, 3.0]]], [[[1.0, 5.0, 8.0], [8.0, 6.0, 8.0], [7.0, 4.0, 3.0]], [[3.0, 7.0, 
7.0], [3.0, 4.0, 8.0], [5.0, 1.0, 7.0]], [[8.0, 8.0, 1.0], [2.0, 7.0, 6.0], [1.0, 4.0, 3.0]]], [[[8.0, 2.0, 3.0], [3.0, 3.0, 
5.0], [7.0, 4.0, 2.0]], [[5.0, 1.0, 2.0], [6.0, 1.0, 1.0], [4.0, 1.0, 4.0]], [[4.0, 2.0, 8.0], [1.0, 7.0, 7.0], [5.0, 8.0, 
8.0]]], [[[2.0, 7.0, 3.0], [2.0, 6.0, 6.0], [3.0, 1.0, 5.0]], [[6.0, 4.0, 8.0], [5.0, 1.0, 7.0], [6.0, 2.0, 1.0]], [[6.0, 8.0, 
8.0], [5.0, 5.0, 7.0], [1.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 5.0], [4.0, 1.0, 6.0], [4.0, 1.0, 8.0]], [[6.0, 4.0, 4.0], [4.0, 4.0, 
2.0], [7.0, 4.0, 3.0]], [[2.0, 3.0, 6.0], [7.0, 6.0, 2.0], [8.0, 8.0, 4.0]]], [[[4.0, 3.0, 1.0], [3.0, 2.0, 8.0], [6.0, 7.0, 
8.0]], [[3.0, 7.0, 3.0], [2.0, 4.0, 1.0], [2.0, 5.0, 3.0]], [[5.0, 3.0, 4.0], [8.0, 7.0, 7.0], [6.0, 7.0, 2.0]]], [[[8.0, 6.0, 
8.0], [5.0, 5.0, 5.0], [5.0, 1.0, 2.0]], [[8.0, 4.0, 7.0], [2.0, 3.0, 7.0], [3.0, 3.0, 4.0]], [[3.0, 7.0, 5.0], [8.0, 3.0, 
6.0], [5.0, 2.0, 3.0]]], [[[7.0, 2.0, 5.0], [3.0, 5.0, 5.0], [5.0, 6.0, 4.0]], [[3.0, 5.0, 8.0], [8.0, 6.0, 2.0], [7.0, 1.0, 
7.0]], [[1.0, 7.0, 3.0], [3.0, 3.0, 3.0], [3.0, 2.0, 6.0]]]]))
      val=numpy.array([[[[7.0, 2.0, 7.0], [5.0, 6.0, 2.0], [2.0, 2.0, 8.0]]], [[[4.0, 8.0, 6.0], [6.0, 2.0, 2.0], [2.0, 5.0, 
7.0]]], [[[7.0, 3.0, 3.0], [3.0, 7.0, 7.0], [4.0, 5.0, 3.0]]]])
      arg[:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [3.0, 1.0, 4.0], [7.0, 1.0, 2.0]], [[2.0, 1.0, 1.0], [8.0, 6.0, 3.0], [4.0, 
8.0, 6.0]], [[7.0, 2.0, 7.0], [5.0, 6.0, 2.0], [2.0, 2.0, 8.0]]], [[[1.0, 5.0, 8.0], [8.0, 6.0, 8.0], [7.0, 4.0, 3.0]], [[3.0, 
7.0, 7.0], [3.0, 4.0, 8.0], [5.0, 1.0, 7.0]], [[4.0, 8.0, 6.0], [6.0, 2.0, 2.0], [2.0, 5.0, 7.0]]], [[[8.0, 2.0, 3.0], [3.0, 
3.0, 5.0], [7.0, 4.0, 2.0]], [[5.0, 1.0, 2.0], [6.0, 1.0, 1.0], [4.0, 1.0, 4.0]], [[7.0, 3.0, 3.0], [3.0, 7.0, 7.0], [4.0, 5.0, 
3.0]]], [[[2.0, 7.0, 3.0], [2.0, 6.0, 6.0], [3.0, 1.0, 5.0]], [[6.0, 4.0, 8.0], [5.0, 1.0, 7.0], [6.0, 2.0, 1.0]], [[6.0, 8.0, 
8.0], [5.0, 5.0, 7.0], [1.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 5.0], [4.0, 1.0, 6.0], [4.0, 1.0, 8.0]], 
[[6.0, 4.0, 4.0], [4.0, 4.0, 2.0], [7.0, 4.0, 3.0]], [[7.0, 2.0, 7.0], [5.0, 6.0, 2.0], [2.0, 2.0, 8.0]]], [[[4.0, 3.0, 1.0], 
[3.0, 2.0, 8.0], [6.0, 7.0, 8.0]], [[3.0, 7.0, 3.0], [2.0, 4.0, 1.0], [2.0, 5.0, 3.0]], [[4.0, 8.0, 6.0], [6.0, 2.0, 2.0], 
[2.0, 5.0, 7.0]]], [[[8.0, 6.0, 8.0], [5.0, 5.0, 5.0], [5.0, 1.0, 2.0]], [[8.0, 4.0, 7.0], [2.0, 3.0, 7.0], [3.0, 3.0, 4.0]], 
[[7.0, 3.0, 3.0], [3.0, 7.0, 7.0], [4.0, 5.0, 3.0]]], [[[7.0, 2.0, 5.0], [3.0, 5.0, 5.0], [5.0, 6.0, 4.0]], [[3.0, 5.0, 8.0], 
[8.0, 6.0, 2.0], [7.0, 1.0, 7.0]], [[1.0, 7.0, 3.0], [3.0, 3.0, 3.0], [3.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULCN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 8.0], [3.0, 3.0, 7.0], [4.0, 1.0, 4.0]], [[1.0, 4.0, 1.0], [5.0, 1.0, 1.0], [1.0, 8.0, 
2.0]], [[2.0, 8.0, 7.0], [4.0, 5.0, 3.0], [8.0, 6.0, 8.0]]], [[[2.0, 2.0, 8.0], [4.0, 1.0, 1.0], [3.0, 4.0, 7.0]], [[8.0, 2.0, 
3.0], [1.0, 8.0, 5.0], [2.0, 6.0, 6.0]], [[6.0, 6.0, 1.0], [1.0, 4.0, 3.0], [4.0, 7.0, 1.0]]], [[[5.0, 6.0, 7.0], [5.0, 8.0, 
6.0], [4.0, 8.0, 5.0]], [[1.0, 7.0, 6.0], [1.0, 7.0, 6.0], [5.0, 1.0, 1.0]], [[2.0, 4.0, 3.0], [1.0, 6.0, 5.0], [4.0, 5.0, 
3.0]]], [[[3.0, 3.0, 5.0], [4.0, 3.0, 4.0], [2.0, 5.0, 1.0]], [[3.0, 5.0, 7.0], [1.0, 8.0, 8.0], [4.0, 6.0, 1.0]], [[4.0, 1.0, 
6.0], [5.0, 6.0, 3.0], [1.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 3.0], [1.0, 1.0, 2.0], [3.0, 4.0, 4.0]], [[6.0, 5.0, 5.0], [5.0, 1.0, 
5.0], [1.0, 6.0, 5.0]], [[7.0, 6.0, 6.0], [3.0, 6.0, 4.0], [7.0, 3.0, 8.0]]], [[[3.0, 5.0, 7.0], [3.0, 2.0, 7.0], [8.0, 4.0, 
7.0]], [[6.0, 8.0, 2.0], [2.0, 2.0, 5.0], [3.0, 2.0, 8.0]], [[3.0, 1.0, 3.0], [7.0, 7.0, 5.0], [4.0, 6.0, 5.0]]], [[[8.0, 1.0, 
2.0], [1.0, 2.0, 4.0], [8.0, 6.0, 2.0]], [[4.0, 6.0, 8.0], [6.0, 5.0, 2.0], [8.0, 3.0, 6.0]], [[7.0, 6.0, 7.0], [5.0, 3.0, 
5.0], [6.0, 7.0, 5.0]]], [[[3.0, 2.0, 4.0], [1.0, 5.0, 2.0], [6.0, 7.0, 8.0]], [[6.0, 3.0, 6.0], [3.0, 7.0, 8.0], [8.0, 6.0, 
5.0]], [[8.0, 8.0, 3.0], [2.0, 7.0, 5.0], [2.0, 2.0, 6.0]]]]))
      val=Data(numpy.array([[[[1.0, 3.0, 5.0], [1.0, 1.0, 7.0], [3.0, 5.0, 7.0]]], [[[5.0, 5.0, 5.0], [2.0, 8.0, 8.0], [6.0, 
2.0, 5.0]]], [[[7.0, 3.0, 1.0], [3.0, 8.0, 5.0], [2.0, 5.0, 1.0]]]]),self.functionspace)
      arg[:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 8.0], [3.0, 3.0, 7.0], [4.0, 1.0, 4.0]], [[1.0, 4.0, 1.0], [5.0, 1.0, 1.0], [1.0, 
8.0, 2.0]], [[1.0, 3.0, 5.0], [1.0, 1.0, 7.0], [3.0, 5.0, 7.0]]], [[[2.0, 2.0, 8.0], [4.0, 1.0, 1.0], [3.0, 4.0, 7.0]], [[8.0, 
2.0, 3.0], [1.0, 8.0, 5.0], [2.0, 6.0, 6.0]], [[5.0, 5.0, 5.0], [2.0, 8.0, 8.0], [6.0, 2.0, 5.0]]], [[[5.0, 6.0, 7.0], [5.0, 
8.0, 6.0], [4.0, 8.0, 5.0]], [[1.0, 7.0, 6.0], [1.0, 7.0, 6.0], [5.0, 1.0, 1.0]], [[7.0, 3.0, 1.0], [3.0, 8.0, 5.0], [2.0, 5.0, 
1.0]]], [[[3.0, 3.0, 5.0], [4.0, 3.0, 4.0], [2.0, 5.0, 1.0]], [[3.0, 5.0, 7.0], [1.0, 8.0, 8.0], [4.0, 6.0, 1.0]], [[4.0, 1.0, 
6.0], [5.0, 6.0, 3.0], [1.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 3.0], [1.0, 1.0, 2.0], [3.0, 4.0, 4.0]], 
[[6.0, 5.0, 5.0], [5.0, 1.0, 5.0], [1.0, 6.0, 5.0]], [[1.0, 3.0, 5.0], [1.0, 1.0, 7.0], [3.0, 5.0, 7.0]]], [[[3.0, 5.0, 7.0], 
[3.0, 2.0, 7.0], [8.0, 4.0, 7.0]], [[6.0, 8.0, 2.0], [2.0, 2.0, 5.0], [3.0, 2.0, 8.0]], [[5.0, 5.0, 5.0], [2.0, 8.0, 8.0], 
[6.0, 2.0, 5.0]]], [[[8.0, 1.0, 2.0], [1.0, 2.0, 4.0], [8.0, 6.0, 2.0]], [[4.0, 6.0, 8.0], [6.0, 5.0, 2.0], [8.0, 3.0, 6.0]], 
[[7.0, 3.0, 1.0], [3.0, 8.0, 5.0], [2.0, 5.0, 1.0]]], [[[3.0, 2.0, 4.0], [1.0, 5.0, 2.0], [6.0, 7.0, 8.0]], [[6.0, 3.0, 6.0], 
[3.0, 7.0, 8.0], [8.0, 6.0, 5.0]], [[8.0, 8.0, 3.0], [2.0, 7.0, 5.0], [2.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULCN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 5.0], [3.0, 3.0, 1.0], [2.0, 3.0, 2.0]], [[6.0, 2.0, 1.0], [7.0, 8.0, 6.0], [4.0, 6.0, 
7.0]], [[5.0, 8.0, 8.0], [6.0, 3.0, 1.0], [1.0, 8.0, 8.0]]], [[[6.0, 2.0, 5.0], [6.0, 1.0, 5.0], [2.0, 8.0, 8.0]], [[7.0, 3.0, 
5.0], [3.0, 5.0, 7.0], [6.0, 7.0, 1.0]], [[5.0, 5.0, 6.0], [4.0, 8.0, 1.0], [6.0, 8.0, 3.0]]], [[[6.0, 4.0, 8.0], [4.0, 5.0, 
7.0], [6.0, 1.0, 8.0]], [[2.0, 4.0, 7.0], [5.0, 2.0, 1.0], [3.0, 6.0, 6.0]], [[6.0, 7.0, 7.0], [1.0, 5.0, 6.0], [2.0, 5.0, 
7.0]]], [[[4.0, 3.0, 2.0], [4.0, 3.0, 4.0], [4.0, 1.0, 1.0]], [[7.0, 6.0, 6.0], [1.0, 7.0, 5.0], [5.0, 4.0, 3.0]], [[1.0, 7.0, 
3.0], [5.0, 4.0, 5.0], [7.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 6.0], [6.0, 5.0, 1.0], [6.0, 3.0, 4.0]], [[3.0, 4.0, 1.0], [2.0, 4.0, 
1.0], [6.0, 5.0, 6.0]], [[7.0, 1.0, 6.0], [7.0, 4.0, 7.0], [2.0, 6.0, 5.0]]], [[[2.0, 5.0, 7.0], [7.0, 8.0, 2.0], [6.0, 6.0, 
2.0]], [[7.0, 6.0, 3.0], [8.0, 5.0, 3.0], [1.0, 1.0, 8.0]], [[8.0, 8.0, 5.0], [6.0, 5.0, 5.0], [7.0, 2.0, 4.0]]], [[[3.0, 7.0, 
7.0], [4.0, 4.0, 6.0], [6.0, 4.0, 8.0]], [[8.0, 3.0, 6.0], [2.0, 3.0, 3.0], [3.0, 4.0, 1.0]], [[8.0, 6.0, 5.0], [1.0, 5.0, 
3.0], [4.0, 6.0, 6.0]]], [[[4.0, 2.0, 8.0], [7.0, 1.0, 3.0], [4.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], [7.0, 6.0, 6.0], [3.0, 7.0, 
5.0]], [[2.0, 2.0, 4.0], [7.0, 5.0, 8.0], [7.0, 2.0, 2.0]]]]))
      val=Data(numpy.array([[[[6.0, 4.0, 6.0], [5.0, 4.0, 7.0], [7.0, 1.0, 2.0]]], [[[5.0, 2.0, 1.0], [6.0, 1.0, 7.0], [3.0, 
5.0, 1.0]]], [[[7.0, 3.0, 3.0], [8.0, 7.0, 2.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 1.0], [7.0, 5.0, 4.0], [3.0, 8.0, 8.0]]], [[[6.0, 8.0, 3.0], [3.0, 3.0, 
7.0], [6.0, 1.0, 8.0]]], [[[2.0, 6.0, 2.0], [8.0, 6.0, 3.0], [6.0, 1.0, 4.0]]]]))
      arg[:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 5.0], [3.0, 3.0, 1.0], [2.0, 3.0, 2.0]], [[6.0, 2.0, 1.0], [7.0, 8.0, 6.0], [4.0, 
6.0, 7.0]], [[6.0, 4.0, 6.0], [5.0, 4.0, 7.0], [7.0, 1.0, 2.0]]], [[[6.0, 2.0, 5.0], [6.0, 1.0, 5.0], [2.0, 8.0, 8.0]], [[7.0, 
3.0, 5.0], [3.0, 5.0, 7.0], [6.0, 7.0, 1.0]], [[5.0, 2.0, 1.0], [6.0, 1.0, 7.0], [3.0, 5.0, 1.0]]], [[[6.0, 4.0, 8.0], [4.0, 
5.0, 7.0], [6.0, 1.0, 8.0]], [[2.0, 4.0, 7.0], [5.0, 2.0, 1.0], [3.0, 6.0, 6.0]], [[7.0, 3.0, 3.0], [8.0, 7.0, 2.0], [8.0, 5.0, 
8.0]]], [[[4.0, 3.0, 2.0], [4.0, 3.0, 4.0], [4.0, 1.0, 1.0]], [[7.0, 6.0, 6.0], [1.0, 7.0, 5.0], [5.0, 4.0, 3.0]], [[1.0, 7.0, 
3.0], [5.0, 4.0, 5.0], [7.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 6.0], [6.0, 5.0, 1.0], [6.0, 3.0, 4.0]], 
[[3.0, 4.0, 1.0], [2.0, 4.0, 1.0], [6.0, 5.0, 6.0]], [[7.0, 2.0, 1.0], [7.0, 5.0, 4.0], [3.0, 8.0, 8.0]]], [[[2.0, 5.0, 7.0], 
[7.0, 8.0, 2.0], [6.0, 6.0, 2.0]], [[7.0, 6.0, 3.0], [8.0, 5.0, 3.0], [1.0, 1.0, 8.0]], [[6.0, 8.0, 3.0], [3.0, 3.0, 7.0], 
[6.0, 1.0, 8.0]]], [[[3.0, 7.0, 7.0], [4.0, 4.0, 6.0], [6.0, 4.0, 8.0]], [[8.0, 3.0, 6.0], [2.0, 3.0, 3.0], [3.0, 4.0, 1.0]], 
[[2.0, 6.0, 2.0], [8.0, 6.0, 3.0], [6.0, 1.0, 4.0]]], [[[4.0, 2.0, 8.0], [7.0, 1.0, 3.0], [4.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], 
[7.0, 6.0, 6.0], [3.0, 7.0, 5.0]], [[2.0, 2.0, 4.0], [7.0, 5.0, 8.0], [7.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULCN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 7.0], [5.0, 3.0, 3.0], [2.0, 3.0, 1.0]], [[2.0, 2.0, 1.0], [3.0, 7.0, 1.0], [6.0, 3.0, 
4.0]], [[4.0, 8.0, 2.0], [7.0, 6.0, 2.0], [5.0, 5.0, 6.0]]], [[[4.0, 1.0, 7.0], [3.0, 7.0, 8.0], [2.0, 7.0, 2.0]], [[4.0, 8.0, 
8.0], [2.0, 6.0, 2.0], [4.0, 2.0, 5.0]], [[6.0, 1.0, 3.0], [7.0, 5.0, 4.0], [1.0, 2.0, 6.0]]], [[[2.0, 6.0, 7.0], [6.0, 7.0, 
8.0], [3.0, 4.0, 1.0]], [[7.0, 5.0, 2.0], [4.0, 6.0, 7.0], [2.0, 5.0, 4.0]], [[1.0, 4.0, 5.0], [1.0, 4.0, 3.0], [8.0, 6.0, 
1.0]]], [[[4.0, 2.0, 6.0], [3.0, 7.0, 2.0], [6.0, 8.0, 7.0]], [[7.0, 8.0, 2.0], [8.0, 7.0, 1.0], [2.0, 6.0, 1.0]], [[4.0, 7.0, 
1.0], [7.0, 4.0, 3.0], [5.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 4.0], [4.0, 1.0, 5.0], [3.0, 1.0, 3.0]], [[2.0, 3.0, 7.0], [2.0, 4.0, 
8.0], [3.0, 6.0, 3.0]], [[8.0, 5.0, 4.0], [4.0, 1.0, 3.0], [5.0, 6.0, 6.0]]], [[[4.0, 3.0, 4.0], [5.0, 4.0, 4.0], [2.0, 4.0, 
4.0]], [[3.0, 2.0, 8.0], [6.0, 5.0, 4.0], [3.0, 8.0, 2.0]], [[3.0, 7.0, 4.0], [8.0, 8.0, 8.0], [6.0, 8.0, 8.0]]], [[[3.0, 6.0, 
6.0], [5.0, 5.0, 1.0], [1.0, 7.0, 1.0]], [[7.0, 5.0, 2.0], [3.0, 7.0, 2.0], [2.0, 3.0, 3.0]], [[7.0, 7.0, 6.0], [7.0, 1.0, 
2.0], [5.0, 8.0, 2.0]]], [[[3.0, 3.0, 2.0], [8.0, 3.0, 6.0], [3.0, 7.0, 5.0]], [[6.0, 1.0, 1.0], [2.0, 6.0, 6.0], [3.0, 3.0, 
7.0]], [[7.0, 7.0, 1.0], [6.0, 3.0, 5.0], [6.0, 6.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 3.0, 6.0], [5.0, 5.0, 3.0], [5.0, 1.0, 6.0]]], [[[1.0, 7.0, 2.0], [8.0, 5.0, 7.0], [8.0, 
8.0, 4.0]]], [[[1.0, 6.0, 3.0], [6.0, 3.0, 2.0], [1.0, 3.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 5.0, 4.0], [8.0, 1.0, 
2.0], [4.0, 2.0, 7.0]]], [[[3.0, 4.0, 5.0], [5.0, 6.0, 3.0], [3.0, 8.0, 7.0]]], [[[2.0, 3.0, 4.0], [6.0, 5.0, 2.0], [6.0, 5.0, 
3.0]]]])
      arg[:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 7.0], [5.0, 3.0, 3.0], [2.0, 3.0, 1.0]], [[2.0, 2.0, 1.0], [3.0, 7.0, 1.0], [6.0, 
3.0, 4.0]], [[3.0, 3.0, 6.0], [5.0, 5.0, 3.0], [5.0, 1.0, 6.0]]], [[[4.0, 1.0, 7.0], [3.0, 7.0, 8.0], [2.0, 7.0, 2.0]], [[4.0, 
8.0, 8.0], [2.0, 6.0, 2.0], [4.0, 2.0, 5.0]], [[1.0, 7.0, 2.0], [8.0, 5.0, 7.0], [8.0, 8.0, 4.0]]], [[[2.0, 6.0, 7.0], [6.0, 
7.0, 8.0], [3.0, 4.0, 1.0]], [[7.0, 5.0, 2.0], [4.0, 6.0, 7.0], [2.0, 5.0, 4.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 2.0], [1.0, 3.0, 
2.0]]], [[[4.0, 2.0, 6.0], [3.0, 7.0, 2.0], [6.0, 8.0, 7.0]], [[7.0, 8.0, 2.0], [8.0, 7.0, 1.0], [2.0, 6.0, 1.0]], [[4.0, 7.0, 
1.0], [7.0, 4.0, 3.0], [5.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 4.0], [4.0, 1.0, 5.0], [3.0, 1.0, 3.0]], 
[[2.0, 3.0, 7.0], [2.0, 4.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 5.0, 4.0], [8.0, 1.0, 2.0], [4.0, 2.0, 7.0]]], [[[4.0, 3.0, 4.0], 
[5.0, 4.0, 4.0], [2.0, 4.0, 4.0]], [[3.0, 2.0, 8.0], [6.0, 5.0, 4.0], [3.0, 8.0, 2.0]], [[3.0, 4.0, 5.0], [5.0, 6.0, 3.0], 
[3.0, 8.0, 7.0]]], [[[3.0, 6.0, 6.0], [5.0, 5.0, 1.0], [1.0, 7.0, 1.0]], [[7.0, 5.0, 2.0], [3.0, 7.0, 2.0], [2.0, 3.0, 3.0]], 
[[2.0, 3.0, 4.0], [6.0, 5.0, 2.0], [6.0, 5.0, 3.0]]], [[[3.0, 3.0, 2.0], [8.0, 3.0, 6.0], [3.0, 7.0, 5.0]], [[6.0, 1.0, 1.0], 
[2.0, 6.0, 6.0], [3.0, 3.0, 7.0]], [[7.0, 7.0, 1.0], [6.0, 3.0, 5.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PRCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 6.0], [1.0, 6.0, 5.0], [1.0, 5.0, 6.0]], [[7.0, 8.0, 8.0], [3.0, 5.0, 2.0], [7.0, 7.0, 
6.0]], [[5.0, 6.0, 4.0], [7.0, 1.0, 1.0], [5.0, 1.0, 7.0]]], [[[7.0, 6.0, 4.0], [1.0, 3.0, 8.0], [1.0, 8.0, 6.0]], [[2.0, 5.0, 
2.0], [7.0, 6.0, 2.0], [8.0, 2.0, 5.0]], [[4.0, 3.0, 8.0], [7.0, 8.0, 4.0], [2.0, 8.0, 5.0]]], [[[6.0, 6.0, 1.0], [3.0, 3.0, 
1.0], [1.0, 3.0, 5.0]], [[4.0, 2.0, 5.0], [8.0, 8.0, 1.0], [2.0, 4.0, 8.0]], [[4.0, 4.0, 4.0], [5.0, 6.0, 8.0], [3.0, 4.0, 
5.0]]], [[[2.0, 8.0, 7.0], [7.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[4.0, 4.0, 1.0], [7.0, 7.0, 6.0], [3.0, 7.0, 3.0]], [[8.0, 8.0, 
4.0], [8.0, 8.0, 4.0], [7.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 8.0], [5.0, 6.0, 8.0], [4.0, 5.0, 1.0]], [[1.0, 4.0, 2.0], [6.0, 4.0, 
2.0], [6.0, 7.0, 1.0]], [[1.0, 2.0, 7.0], [6.0, 2.0, 8.0], [6.0, 2.0, 8.0]]], [[[4.0, 6.0, 1.0], [1.0, 3.0, 4.0], [2.0, 6.0, 
8.0]], [[2.0, 2.0, 2.0], [6.0, 4.0, 5.0], [6.0, 5.0, 6.0]], [[8.0, 6.0, 8.0], [2.0, 3.0, 2.0], [2.0, 6.0, 8.0]]], [[[3.0, 8.0, 
7.0], [4.0, 2.0, 1.0], [7.0, 4.0, 1.0]], [[6.0, 8.0, 2.0], [1.0, 3.0, 6.0], [7.0, 7.0, 2.0]], [[1.0, 4.0, 8.0], [4.0, 4.0, 
4.0], [7.0, 4.0, 4.0]]], [[[3.0, 4.0, 7.0], [6.0, 6.0, 7.0], [7.0, 2.0, 1.0]], [[1.0, 5.0, 8.0], [3.0, 1.0, 3.0], [1.0, 5.0, 
1.0]], [[1.0, 3.0, 8.0], [1.0, 6.0, 6.0], [4.0, 7.0, 7.0]]]]))
      val=numpy.array([[[2.0, 7.0, 3.0], [4.0, 2.0, 4.0], [7.0, 4.0, 8.0]]])
      arg[2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 6.0], [1.0, 6.0, 5.0], [1.0, 5.0, 6.0]], [[7.0, 8.0, 8.0], [3.0, 5.0, 2.0], [7.0, 
7.0, 6.0]], [[5.0, 6.0, 4.0], [7.0, 1.0, 1.0], [5.0, 1.0, 7.0]]], [[[7.0, 6.0, 4.0], [1.0, 3.0, 8.0], [1.0, 8.0, 6.0]], [[2.0, 
5.0, 2.0], [7.0, 6.0, 2.0], [8.0, 2.0, 5.0]], [[4.0, 3.0, 8.0], [7.0, 8.0, 4.0], [2.0, 8.0, 5.0]]], [[[6.0, 6.0, 1.0], [3.0, 
3.0, 1.0], [1.0, 3.0, 5.0]], [[2.0, 7.0, 3.0], [4.0, 2.0, 4.0], [7.0, 4.0, 8.0]], [[4.0, 4.0, 4.0], [5.0, 6.0, 8.0], [3.0, 4.0, 
5.0]]], [[[2.0, 8.0, 7.0], [7.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[4.0, 4.0, 1.0], [7.0, 7.0, 6.0], [3.0, 7.0, 3.0]], [[8.0, 8.0, 
4.0], [8.0, 8.0, 4.0], [7.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 8.0], [5.0, 6.0, 8.0], [4.0, 5.0, 1.0]], 
[[1.0, 4.0, 2.0], [6.0, 4.0, 2.0], [6.0, 7.0, 1.0]], [[1.0, 2.0, 7.0], [6.0, 2.0, 8.0], [6.0, 2.0, 8.0]]], [[[4.0, 6.0, 1.0], 
[1.0, 3.0, 4.0], [2.0, 6.0, 8.0]], [[2.0, 2.0, 2.0], [6.0, 4.0, 5.0], [6.0, 5.0, 6.0]], [[8.0, 6.0, 8.0], [2.0, 3.0, 2.0], 
[2.0, 6.0, 8.0]]], [[[3.0, 8.0, 7.0], [4.0, 2.0, 1.0], [7.0, 4.0, 1.0]], [[2.0, 7.0, 3.0], [4.0, 2.0, 4.0], [7.0, 4.0, 8.0]], 
[[1.0, 4.0, 8.0], [4.0, 4.0, 4.0], [7.0, 4.0, 4.0]]], [[[3.0, 4.0, 7.0], [6.0, 6.0, 7.0], [7.0, 2.0, 1.0]], [[1.0, 5.0, 8.0], 
[3.0, 1.0, 3.0], [1.0, 5.0, 1.0]], [[1.0, 3.0, 8.0], [1.0, 6.0, 6.0], [4.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PRCN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 8.0], [7.0, 4.0, 5.0], [1.0, 5.0, 2.0]], [[8.0, 8.0, 5.0], [1.0, 4.0, 5.0], [6.0, 4.0, 
4.0]], [[6.0, 3.0, 1.0], [3.0, 2.0, 6.0], [8.0, 4.0, 4.0]]], [[[6.0, 1.0, 4.0], [6.0, 8.0, 6.0], [6.0, 1.0, 6.0]], [[4.0, 8.0, 
7.0], [1.0, 7.0, 7.0], [2.0, 8.0, 2.0]], [[6.0, 7.0, 6.0], [3.0, 1.0, 5.0], [5.0, 3.0, 4.0]]], [[[2.0, 6.0, 7.0], [2.0, 2.0, 
3.0], [1.0, 4.0, 2.0]], [[6.0, 1.0, 4.0], [2.0, 6.0, 3.0], [5.0, 4.0, 2.0]], [[3.0, 1.0, 3.0], [7.0, 8.0, 4.0], [1.0, 3.0, 
5.0]]], [[[5.0, 8.0, 3.0], [3.0, 5.0, 7.0], [2.0, 2.0, 5.0]], [[2.0, 2.0, 6.0], [5.0, 7.0, 5.0], [3.0, 6.0, 3.0]], [[3.0, 2.0, 
4.0], [3.0, 4.0, 7.0], [1.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 7.0], [2.0, 8.0, 5.0], [6.0, 7.0, 5.0]], [[5.0, 2.0, 4.0], [3.0, 2.0, 
6.0], [1.0, 5.0, 7.0]], [[8.0, 4.0, 7.0], [4.0, 4.0, 4.0], [6.0, 6.0, 3.0]]], [[[1.0, 2.0, 2.0], [1.0, 4.0, 4.0], [3.0, 7.0, 
6.0]], [[1.0, 7.0, 8.0], [4.0, 8.0, 2.0], [1.0, 1.0, 5.0]], [[1.0, 4.0, 2.0], [1.0, 5.0, 8.0], [5.0, 5.0, 7.0]]], [[[4.0, 5.0, 
7.0], [5.0, 2.0, 5.0], [6.0, 3.0, 7.0]], [[2.0, 6.0, 7.0], [4.0, 5.0, 8.0], [3.0, 6.0, 8.0]], [[4.0, 5.0, 3.0], [4.0, 5.0, 
1.0], [1.0, 3.0, 8.0]]], [[[8.0, 2.0, 6.0], [8.0, 4.0, 8.0], [1.0, 2.0, 3.0]], [[6.0, 4.0, 2.0], [5.0, 8.0, 8.0], [8.0, 4.0, 
7.0]], [[7.0, 7.0, 5.0], [1.0, 7.0, 1.0], [6.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[[6.0, 8.0, 7.0], [4.0, 5.0, 3.0], [5.0, 5.0, 6.0]]]),self.functionspace)
      arg[2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 8.0], [7.0, 4.0, 5.0], [1.0, 5.0, 2.0]], [[8.0, 8.0, 5.0], [1.0, 4.0, 5.0], [6.0, 
4.0, 4.0]], [[6.0, 3.0, 1.0], [3.0, 2.0, 6.0], [8.0, 4.0, 4.0]]], [[[6.0, 1.0, 4.0], [6.0, 8.0, 6.0], [6.0, 1.0, 6.0]], [[4.0, 
8.0, 7.0], [1.0, 7.0, 7.0], [2.0, 8.0, 2.0]], [[6.0, 7.0, 6.0], [3.0, 1.0, 5.0], [5.0, 3.0, 4.0]]], [[[2.0, 6.0, 7.0], [2.0, 
2.0, 3.0], [1.0, 4.0, 2.0]], [[6.0, 8.0, 7.0], [4.0, 5.0, 3.0], [5.0, 5.0, 6.0]], [[3.0, 1.0, 3.0], [7.0, 8.0, 4.0], [1.0, 3.0, 
5.0]]], [[[5.0, 8.0, 3.0], [3.0, 5.0, 7.0], [2.0, 2.0, 5.0]], [[2.0, 2.0, 6.0], [5.0, 7.0, 5.0], [3.0, 6.0, 3.0]], [[3.0, 2.0, 
4.0], [3.0, 4.0, 7.0], [1.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 7.0], [2.0, 8.0, 5.0], [6.0, 7.0, 5.0]], 
[[5.0, 2.0, 4.0], [3.0, 2.0, 6.0], [1.0, 5.0, 7.0]], [[8.0, 4.0, 7.0], [4.0, 4.0, 4.0], [6.0, 6.0, 3.0]]], [[[1.0, 2.0, 2.0], 
[1.0, 4.0, 4.0], [3.0, 7.0, 6.0]], [[1.0, 7.0, 8.0], [4.0, 8.0, 2.0], [1.0, 1.0, 5.0]], [[1.0, 4.0, 2.0], [1.0, 5.0, 8.0], 
[5.0, 5.0, 7.0]]], [[[4.0, 5.0, 7.0], [5.0, 2.0, 5.0], [6.0, 3.0, 7.0]], [[6.0, 8.0, 7.0], [4.0, 5.0, 3.0], [5.0, 5.0, 6.0]], 
[[4.0, 5.0, 3.0], [4.0, 5.0, 1.0], [1.0, 3.0, 8.0]]], [[[8.0, 2.0, 6.0], [8.0, 4.0, 8.0], [1.0, 2.0, 3.0]], [[6.0, 4.0, 2.0], 
[5.0, 8.0, 8.0], [8.0, 4.0, 7.0]], [[7.0, 7.0, 5.0], [1.0, 7.0, 1.0], [6.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PRCN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 5.0], [7.0, 8.0, 5.0], [5.0, 8.0, 4.0]], [[6.0, 6.0, 4.0], [2.0, 7.0, 7.0], [3.0, 1.0, 
1.0]], [[1.0, 7.0, 6.0], [7.0, 5.0, 6.0], [5.0, 8.0, 8.0]]], [[[3.0, 8.0, 4.0], [5.0, 8.0, 7.0], [4.0, 5.0, 5.0]], [[6.0, 8.0, 
1.0], [4.0, 6.0, 7.0], [4.0, 4.0, 1.0]], [[2.0, 7.0, 5.0], [2.0, 8.0, 1.0], [5.0, 1.0, 8.0]]], [[[7.0, 5.0, 6.0], [4.0, 2.0, 
4.0], [7.0, 4.0, 5.0]], [[4.0, 5.0, 8.0], [1.0, 1.0, 2.0], [6.0, 7.0, 7.0]], [[7.0, 3.0, 5.0], [3.0, 1.0, 2.0], [5.0, 1.0, 
2.0]]], [[[6.0, 1.0, 2.0], [8.0, 3.0, 3.0], [4.0, 7.0, 5.0]], [[4.0, 4.0, 3.0], [8.0, 4.0, 3.0], [5.0, 3.0, 1.0]], [[7.0, 6.0, 
5.0], [1.0, 7.0, 6.0], [6.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 5.0], [1.0, 4.0, 3.0], [3.0, 2.0, 2.0]], [[4.0, 7.0, 1.0], [1.0, 3.0, 
2.0], [2.0, 4.0, 4.0]], [[2.0, 1.0, 3.0], [3.0, 4.0, 5.0], [6.0, 8.0, 6.0]]], [[[6.0, 4.0, 8.0], [1.0, 7.0, 7.0], [2.0, 1.0, 
8.0]], [[2.0, 5.0, 2.0], [1.0, 3.0, 3.0], [5.0, 6.0, 6.0]], [[5.0, 8.0, 4.0], [4.0, 7.0, 5.0], [6.0, 4.0, 5.0]]], [[[5.0, 7.0, 
6.0], [3.0, 8.0, 6.0], [2.0, 5.0, 7.0]], [[2.0, 2.0, 8.0], [2.0, 4.0, 3.0], [3.0, 3.0, 3.0]], [[1.0, 4.0, 7.0], [4.0, 4.0, 
5.0], [1.0, 8.0, 8.0]]], [[[6.0, 6.0, 5.0], [4.0, 2.0, 2.0], [3.0, 3.0, 7.0]], [[3.0, 8.0, 6.0], [4.0, 7.0, 4.0], [4.0, 6.0, 
5.0]], [[8.0, 5.0, 8.0], [5.0, 5.0, 6.0], [7.0, 7.0, 4.0]]]]))
      val=Data(numpy.array([[[4.0, 4.0, 3.0], [2.0, 4.0, 6.0], [3.0, 7.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 2.0, 7.0], [3.0, 4.0, 5.0], [2.0, 5.0, 8.0]]]))
      arg[2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 5.0], [7.0, 8.0, 5.0], [5.0, 8.0, 4.0]], [[6.0, 6.0, 4.0], [2.0, 7.0, 7.0], [3.0, 
1.0, 1.0]], [[1.0, 7.0, 6.0], [7.0, 5.0, 6.0], [5.0, 8.0, 8.0]]], [[[3.0, 8.0, 4.0], [5.0, 8.0, 7.0], [4.0, 5.0, 5.0]], [[6.0, 
8.0, 1.0], [4.0, 6.0, 7.0], [4.0, 4.0, 1.0]], [[2.0, 7.0, 5.0], [2.0, 8.0, 1.0], [5.0, 1.0, 8.0]]], [[[7.0, 5.0, 6.0], [4.0, 
2.0, 4.0], [7.0, 4.0, 5.0]], [[4.0, 4.0, 3.0], [2.0, 4.0, 6.0], [3.0, 7.0, 6.0]], [[7.0, 3.0, 5.0], [3.0, 1.0, 2.0], [5.0, 1.0, 
2.0]]], [[[6.0, 1.0, 2.0], [8.0, 3.0, 3.0], [4.0, 7.0, 5.0]], [[4.0, 4.0, 3.0], [8.0, 4.0, 3.0], [5.0, 3.0, 1.0]], [[7.0, 6.0, 
5.0], [1.0, 7.0, 6.0], [6.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 5.0], [1.0, 4.0, 3.0], [3.0, 2.0, 2.0]], 
[[4.0, 7.0, 1.0], [1.0, 3.0, 2.0], [2.0, 4.0, 4.0]], [[2.0, 1.0, 3.0], [3.0, 4.0, 5.0], [6.0, 8.0, 6.0]]], [[[6.0, 4.0, 8.0], 
[1.0, 7.0, 7.0], [2.0, 1.0, 8.0]], [[2.0, 5.0, 2.0], [1.0, 3.0, 3.0], [5.0, 6.0, 6.0]], [[5.0, 8.0, 4.0], [4.0, 7.0, 5.0], 
[6.0, 4.0, 5.0]]], [[[5.0, 7.0, 6.0], [3.0, 8.0, 6.0], [2.0, 5.0, 7.0]], [[2.0, 2.0, 7.0], [3.0, 4.0, 5.0], [2.0, 5.0, 8.0]], 
[[1.0, 4.0, 7.0], [4.0, 4.0, 5.0], [1.0, 8.0, 8.0]]], [[[6.0, 6.0, 5.0], [4.0, 2.0, 2.0], [3.0, 3.0, 7.0]], [[3.0, 8.0, 6.0], 
[4.0, 7.0, 4.0], [4.0, 6.0, 5.0]], [[8.0, 5.0, 8.0], [5.0, 5.0, 6.0], [7.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PRCN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 1.0], [8.0, 1.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 2.0, 5.0], [6.0, 1.0, 4.0], [8.0, 6.0, 
6.0]], [[5.0, 2.0, 6.0], [3.0, 5.0, 8.0], [4.0, 7.0, 6.0]]], [[[8.0, 8.0, 4.0], [5.0, 8.0, 8.0], [5.0, 7.0, 6.0]], [[1.0, 5.0, 
5.0], [2.0, 3.0, 7.0], [7.0, 7.0, 4.0]], [[4.0, 5.0, 1.0], [5.0, 1.0, 3.0], [2.0, 2.0, 3.0]]], [[[3.0, 1.0, 7.0], [5.0, 4.0, 
6.0], [8.0, 7.0, 6.0]], [[5.0, 6.0, 1.0], [8.0, 6.0, 1.0], [4.0, 7.0, 4.0]], [[5.0, 8.0, 4.0], [7.0, 8.0, 1.0], [3.0, 6.0, 
2.0]]], [[[1.0, 3.0, 2.0], [6.0, 7.0, 4.0], [8.0, 5.0, 6.0]], [[2.0, 2.0, 4.0], [7.0, 5.0, 7.0], [6.0, 8.0, 2.0]], [[2.0, 5.0, 
8.0], [3.0, 8.0, 7.0], [5.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 6.0], [3.0, 4.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 8.0, 1.0], [6.0, 6.0, 
4.0], [4.0, 6.0, 2.0]], [[3.0, 3.0, 5.0], [2.0, 5.0, 3.0], [4.0, 2.0, 8.0]]], [[[6.0, 1.0, 7.0], [7.0, 7.0, 5.0], [2.0, 7.0, 
8.0]], [[2.0, 8.0, 2.0], [2.0, 5.0, 4.0], [8.0, 7.0, 3.0]], [[6.0, 1.0, 7.0], [5.0, 5.0, 8.0], [1.0, 6.0, 6.0]]], [[[5.0, 5.0, 
5.0], [6.0, 5.0, 3.0], [1.0, 4.0, 3.0]], [[2.0, 8.0, 4.0], [4.0, 4.0, 2.0], [2.0, 4.0, 6.0]], [[8.0, 5.0, 4.0], [8.0, 8.0, 
2.0], [7.0, 3.0, 4.0]]], [[[7.0, 6.0, 7.0], [5.0, 4.0, 5.0], [5.0, 5.0, 6.0]], [[2.0, 5.0, 6.0], [7.0, 8.0, 3.0], [6.0, 5.0, 
1.0]], [[3.0, 8.0, 2.0], [3.0, 4.0, 7.0], [3.0, 8.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 3.0, 3.0], [6.0, 8.0, 6.0], [6.0, 3.0, 7.0]]])+(1.-msk_val)*numpy.array([[[8.0, 7.0, 
5.0], [7.0, 6.0, 1.0], [3.0, 5.0, 2.0]]])
      arg[2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 1.0], [8.0, 1.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 2.0, 5.0], [6.0, 1.0, 4.0], [8.0, 
6.0, 6.0]], [[5.0, 2.0, 6.0], [3.0, 5.0, 8.0], [4.0, 7.0, 6.0]]], [[[8.0, 8.0, 4.0], [5.0, 8.0, 8.0], [5.0, 7.0, 6.0]], [[1.0, 
5.0, 5.0], [2.0, 3.0, 7.0], [7.0, 7.0, 4.0]], [[4.0, 5.0, 1.0], [5.0, 1.0, 3.0], [2.0, 2.0, 3.0]]], [[[3.0, 1.0, 7.0], [5.0, 
4.0, 6.0], [8.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [6.0, 8.0, 6.0], [6.0, 3.0, 7.0]], [[5.0, 8.0, 4.0], [7.0, 8.0, 1.0], [3.0, 6.0, 
2.0]]], [[[1.0, 3.0, 2.0], [6.0, 7.0, 4.0], [8.0, 5.0, 6.0]], [[2.0, 2.0, 4.0], [7.0, 5.0, 7.0], [6.0, 8.0, 2.0]], [[2.0, 5.0, 
8.0], [3.0, 8.0, 7.0], [5.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 6.0], [3.0, 4.0, 6.0], [8.0, 5.0, 5.0]], 
[[3.0, 8.0, 1.0], [6.0, 6.0, 4.0], [4.0, 6.0, 2.0]], [[3.0, 3.0, 5.0], [2.0, 5.0, 3.0], [4.0, 2.0, 8.0]]], [[[6.0, 1.0, 7.0], 
[7.0, 7.0, 5.0], [2.0, 7.0, 8.0]], [[2.0, 8.0, 2.0], [2.0, 5.0, 4.0], [8.0, 7.0, 3.0]], [[6.0, 1.0, 7.0], [5.0, 5.0, 8.0], 
[1.0, 6.0, 6.0]]], [[[5.0, 5.0, 5.0], [6.0, 5.0, 3.0], [1.0, 4.0, 3.0]], [[8.0, 7.0, 5.0], [7.0, 6.0, 1.0], [3.0, 5.0, 2.0]], 
[[8.0, 5.0, 4.0], [8.0, 8.0, 2.0], [7.0, 3.0, 4.0]]], [[[7.0, 6.0, 7.0], [5.0, 4.0, 5.0], [5.0, 5.0, 6.0]], [[2.0, 5.0, 6.0], 
[7.0, 8.0, 3.0], [6.0, 5.0, 1.0]], [[3.0, 8.0, 2.0], [3.0, 4.0, 7.0], [3.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CLLN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 1.0], [3.0, 8.0, 4.0], [4.0, 6.0, 8.0]], [[7.0, 3.0, 4.0], [1.0, 5.0, 5.0], [8.0, 7.0, 
1.0]], [[2.0, 8.0, 3.0], [3.0, 3.0, 7.0], [3.0, 7.0, 5.0]]], [[[1.0, 7.0, 3.0], [3.0, 6.0, 2.0], [2.0, 8.0, 5.0]], [[7.0, 5.0, 
4.0], [2.0, 7.0, 1.0], [4.0, 3.0, 6.0]], [[6.0, 4.0, 4.0], [7.0, 5.0, 8.0], [2.0, 7.0, 3.0]]], [[[5.0, 3.0, 7.0], [5.0, 4.0, 
8.0], [4.0, 8.0, 7.0]], [[7.0, 7.0, 8.0], [7.0, 5.0, 7.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 7.0], [2.0, 3.0, 2.0], [7.0, 8.0, 
6.0]]], [[[7.0, 3.0, 1.0], [3.0, 2.0, 8.0], [6.0, 4.0, 3.0]], [[3.0, 5.0, 8.0], [7.0, 6.0, 6.0], [4.0, 1.0, 8.0]], [[7.0, 4.0, 
8.0], [6.0, 1.0, 3.0], [8.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 2.0], [5.0, 6.0, 1.0], [8.0, 7.0, 2.0]], [[3.0, 2.0, 1.0], [6.0, 6.0, 
6.0], [5.0, 3.0, 5.0]], [[4.0, 2.0, 7.0], [2.0, 7.0, 1.0], [3.0, 6.0, 7.0]]], [[[2.0, 1.0, 7.0], [6.0, 5.0, 5.0], [7.0, 3.0, 
6.0]], [[7.0, 7.0, 7.0], [1.0, 5.0, 6.0], [2.0, 1.0, 3.0]], [[7.0, 1.0, 3.0], [1.0, 3.0, 4.0], [4.0, 3.0, 2.0]]], [[[2.0, 5.0, 
8.0], [8.0, 8.0, 1.0], [4.0, 6.0, 2.0]], [[3.0, 4.0, 4.0], [8.0, 5.0, 1.0], [4.0, 3.0, 5.0]], [[7.0, 5.0, 5.0], [3.0, 3.0, 
4.0], [8.0, 6.0, 8.0]]], [[[3.0, 3.0, 3.0], [2.0, 5.0, 4.0], [6.0, 3.0, 7.0]], [[2.0, 3.0, 5.0], [8.0, 8.0, 7.0], [5.0, 4.0, 
4.0]], [[6.0, 7.0, 8.0], [7.0, 4.0, 7.0], [5.0, 5.0, 4.0]]]]))
      val=numpy.array([[[[6.0, 3.0, 6.0]]], [[[2.0, 6.0, 4.0]]], [[[4.0, 2.0, 6.0]]], [[[2.0, 3.0, 5.0]]]])
      arg[:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 1.0], [3.0, 8.0, 4.0], [4.0, 6.0, 8.0]], [[7.0, 3.0, 4.0], [1.0, 5.0, 5.0], [8.0, 
7.0, 1.0]], [[2.0, 8.0, 3.0], [3.0, 3.0, 7.0], [6.0, 3.0, 6.0]]], [[[1.0, 7.0, 3.0], [3.0, 6.0, 2.0], [2.0, 8.0, 5.0]], [[7.0, 
5.0, 4.0], [2.0, 7.0, 1.0], [4.0, 3.0, 6.0]], [[6.0, 4.0, 4.0], [7.0, 5.0, 8.0], [2.0, 6.0, 4.0]]], [[[5.0, 3.0, 7.0], [5.0, 
4.0, 8.0], [4.0, 8.0, 7.0]], [[7.0, 7.0, 8.0], [7.0, 5.0, 7.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 7.0], [2.0, 3.0, 2.0], [4.0, 2.0, 
6.0]]], [[[7.0, 3.0, 1.0], [3.0, 2.0, 8.0], [6.0, 4.0, 3.0]], [[3.0, 5.0, 8.0], [7.0, 6.0, 6.0], [4.0, 1.0, 8.0]], [[7.0, 4.0, 
8.0], [6.0, 1.0, 3.0], [2.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 2.0], [5.0, 6.0, 1.0], [8.0, 7.0, 2.0]], 
[[3.0, 2.0, 1.0], [6.0, 6.0, 6.0], [5.0, 3.0, 5.0]], [[4.0, 2.0, 7.0], [2.0, 7.0, 1.0], [6.0, 3.0, 6.0]]], [[[2.0, 1.0, 7.0], 
[6.0, 5.0, 5.0], [7.0, 3.0, 6.0]], [[7.0, 7.0, 7.0], [1.0, 5.0, 6.0], [2.0, 1.0, 3.0]], [[7.0, 1.0, 3.0], [1.0, 3.0, 4.0], 
[2.0, 6.0, 4.0]]], [[[2.0, 5.0, 8.0], [8.0, 8.0, 1.0], [4.0, 6.0, 2.0]], [[3.0, 4.0, 4.0], [8.0, 5.0, 1.0], [4.0, 3.0, 5.0]], 
[[7.0, 5.0, 5.0], [3.0, 3.0, 4.0], [4.0, 2.0, 6.0]]], [[[3.0, 3.0, 3.0], [2.0, 5.0, 4.0], [6.0, 3.0, 7.0]], [[2.0, 3.0, 5.0], 
[8.0, 8.0, 7.0], [5.0, 4.0, 4.0]], [[6.0, 7.0, 8.0], [7.0, 4.0, 7.0], [2.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CLLN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 5.0], [7.0, 6.0, 4.0], [3.0, 5.0, 7.0]], [[3.0, 4.0, 2.0], [6.0, 8.0, 5.0], [2.0, 4.0, 
8.0]], [[8.0, 3.0, 5.0], [1.0, 3.0, 8.0], [6.0, 1.0, 7.0]]], [[[2.0, 3.0, 2.0], [3.0, 2.0, 5.0], [7.0, 2.0, 4.0]], [[1.0, 1.0, 
3.0], [6.0, 6.0, 1.0], [2.0, 1.0, 5.0]], [[7.0, 5.0, 1.0], [6.0, 7.0, 3.0], [8.0, 4.0, 8.0]]], [[[2.0, 4.0, 8.0], [6.0, 3.0, 
1.0], [2.0, 4.0, 6.0]], [[3.0, 1.0, 7.0], [4.0, 1.0, 6.0], [8.0, 4.0, 5.0]], [[5.0, 6.0, 8.0], [1.0, 3.0, 1.0], [4.0, 2.0, 
6.0]]], [[[1.0, 1.0, 7.0], [7.0, 8.0, 8.0], [2.0, 2.0, 6.0]], [[5.0, 6.0, 8.0], [8.0, 7.0, 6.0], [7.0, 7.0, 4.0]], [[6.0, 4.0, 
6.0], [4.0, 5.0, 7.0], [5.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 1.0], [8.0, 5.0, 2.0], [4.0, 4.0, 7.0]], [[3.0, 3.0, 4.0], [3.0, 5.0, 
7.0], [3.0, 5.0, 2.0]], [[3.0, 4.0, 4.0], [1.0, 6.0, 8.0], [6.0, 5.0, 1.0]]], [[[6.0, 1.0, 7.0], [8.0, 1.0, 1.0], [6.0, 5.0, 
2.0]], [[6.0, 2.0, 3.0], [1.0, 3.0, 6.0], [3.0, 7.0, 6.0]], [[2.0, 2.0, 4.0], [5.0, 1.0, 3.0], [7.0, 1.0, 4.0]]], [[[2.0, 8.0, 
7.0], [5.0, 8.0, 1.0], [1.0, 2.0, 2.0]], [[8.0, 6.0, 1.0], [7.0, 6.0, 8.0], [1.0, 3.0, 6.0]], [[7.0, 3.0, 3.0], [5.0, 7.0, 
8.0], [4.0, 4.0, 4.0]]], [[[1.0, 8.0, 6.0], [5.0, 2.0, 8.0], [1.0, 7.0, 1.0]], [[2.0, 5.0, 5.0], [2.0, 8.0, 5.0], [6.0, 4.0, 
1.0]], [[3.0, 7.0, 5.0], [5.0, 7.0, 5.0], [2.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[[3.0, 6.0, 3.0]]], [[[8.0, 5.0, 8.0]]], [[[6.0, 5.0, 4.0]]], [[[5.0, 3.0, 
1.0]]]]),self.functionspace)
      arg[:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 5.0], [7.0, 6.0, 4.0], [3.0, 5.0, 7.0]], [[3.0, 4.0, 2.0], [6.0, 8.0, 5.0], [2.0, 
4.0, 8.0]], [[8.0, 3.0, 5.0], [1.0, 3.0, 8.0], [3.0, 6.0, 3.0]]], [[[2.0, 3.0, 2.0], [3.0, 2.0, 5.0], [7.0, 2.0, 4.0]], [[1.0, 
1.0, 3.0], [6.0, 6.0, 1.0], [2.0, 1.0, 5.0]], [[7.0, 5.0, 1.0], [6.0, 7.0, 3.0], [8.0, 5.0, 8.0]]], [[[2.0, 4.0, 8.0], [6.0, 
3.0, 1.0], [2.0, 4.0, 6.0]], [[3.0, 1.0, 7.0], [4.0, 1.0, 6.0], [8.0, 4.0, 5.0]], [[5.0, 6.0, 8.0], [1.0, 3.0, 1.0], [6.0, 5.0, 
4.0]]], [[[1.0, 1.0, 7.0], [7.0, 8.0, 8.0], [2.0, 2.0, 6.0]], [[5.0, 6.0, 8.0], [8.0, 7.0, 6.0], [7.0, 7.0, 4.0]], [[6.0, 4.0, 
6.0], [4.0, 5.0, 7.0], [5.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 1.0], [8.0, 5.0, 2.0], [4.0, 4.0, 7.0]], 
[[3.0, 3.0, 4.0], [3.0, 5.0, 7.0], [3.0, 5.0, 2.0]], [[3.0, 4.0, 4.0], [1.0, 6.0, 8.0], [3.0, 6.0, 3.0]]], [[[6.0, 1.0, 7.0], 
[8.0, 1.0, 1.0], [6.0, 5.0, 2.0]], [[6.0, 2.0, 3.0], [1.0, 3.0, 6.0], [3.0, 7.0, 6.0]], [[2.0, 2.0, 4.0], [5.0, 1.0, 3.0], 
[8.0, 5.0, 8.0]]], [[[2.0, 8.0, 7.0], [5.0, 8.0, 1.0], [1.0, 2.0, 2.0]], [[8.0, 6.0, 1.0], [7.0, 6.0, 8.0], [1.0, 3.0, 6.0]], 
[[7.0, 3.0, 3.0], [5.0, 7.0, 8.0], [6.0, 5.0, 4.0]]], [[[1.0, 8.0, 6.0], [5.0, 2.0, 8.0], [1.0, 7.0, 1.0]], [[2.0, 5.0, 5.0], 
[2.0, 8.0, 5.0], [6.0, 4.0, 1.0]], [[3.0, 7.0, 5.0], [5.0, 7.0, 5.0], [5.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CLLN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 7.0], [4.0, 4.0, 7.0], [1.0, 6.0, 4.0]], [[6.0, 1.0, 7.0], [3.0, 6.0, 3.0], [2.0, 5.0, 
5.0]], [[4.0, 1.0, 2.0], [3.0, 4.0, 2.0], [5.0, 2.0, 1.0]]], [[[3.0, 1.0, 1.0], [3.0, 6.0, 1.0], [1.0, 3.0, 7.0]], [[5.0, 7.0, 
5.0], [5.0, 2.0, 1.0], [5.0, 3.0, 5.0]], [[4.0, 8.0, 8.0], [6.0, 4.0, 2.0], [5.0, 3.0, 2.0]]], [[[7.0, 7.0, 6.0], [2.0, 7.0, 
6.0], [3.0, 7.0, 6.0]], [[3.0, 1.0, 8.0], [2.0, 2.0, 6.0], [1.0, 4.0, 8.0]], [[6.0, 7.0, 6.0], [3.0, 4.0, 4.0], [2.0, 7.0, 
2.0]]], [[[3.0, 3.0, 4.0], [1.0, 3.0, 5.0], [4.0, 3.0, 2.0]], [[5.0, 8.0, 5.0], [6.0, 7.0, 8.0], [1.0, 8.0, 6.0]], [[6.0, 1.0, 
4.0], [3.0, 8.0, 4.0], [3.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 2.0], [8.0, 8.0, 8.0], [2.0, 6.0, 3.0]], [[3.0, 2.0, 3.0], [8.0, 8.0, 
2.0], [7.0, 8.0, 6.0]], [[6.0, 3.0, 7.0], [6.0, 3.0, 3.0], [1.0, 7.0, 8.0]]], [[[7.0, 7.0, 7.0], [5.0, 5.0, 8.0], [7.0, 3.0, 
7.0]], [[1.0, 5.0, 1.0], [8.0, 8.0, 6.0], [7.0, 5.0, 3.0]], [[4.0, 4.0, 7.0], [2.0, 3.0, 2.0], [1.0, 6.0, 8.0]]], [[[2.0, 8.0, 
2.0], [3.0, 1.0, 3.0], [5.0, 2.0, 2.0]], [[4.0, 6.0, 1.0], [1.0, 4.0, 4.0], [6.0, 8.0, 2.0]], [[2.0, 2.0, 3.0], [8.0, 4.0, 
8.0], [3.0, 1.0, 7.0]]], [[[4.0, 7.0, 5.0], [7.0, 8.0, 7.0], [8.0, 2.0, 4.0]], [[5.0, 5.0, 8.0], [2.0, 2.0, 8.0], [8.0, 6.0, 
4.0]], [[5.0, 4.0, 5.0], [6.0, 5.0, 8.0], [3.0, 1.0, 4.0]]]]))
      val=Data(numpy.array([[[[7.0, 6.0, 3.0]]], [[[1.0, 7.0, 3.0]]], [[[3.0, 6.0, 5.0]]], [[[2.0, 2.0, 
3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 3.0]]], [[[4.0, 3.0, 4.0]]], [[[4.0, 5.0, 1.0]]], [[[5.0, 8.0, 8.0]]]]))
      arg[:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 7.0], [4.0, 4.0, 7.0], [1.0, 6.0, 4.0]], [[6.0, 1.0, 7.0], [3.0, 6.0, 3.0], [2.0, 
5.0, 5.0]], [[4.0, 1.0, 2.0], [3.0, 4.0, 2.0], [7.0, 6.0, 3.0]]], [[[3.0, 1.0, 1.0], [3.0, 6.0, 1.0], [1.0, 3.0, 7.0]], [[5.0, 
7.0, 5.0], [5.0, 2.0, 1.0], [5.0, 3.0, 5.0]], [[4.0, 8.0, 8.0], [6.0, 4.0, 2.0], [1.0, 7.0, 3.0]]], [[[7.0, 7.0, 6.0], [2.0, 
7.0, 6.0], [3.0, 7.0, 6.0]], [[3.0, 1.0, 8.0], [2.0, 2.0, 6.0], [1.0, 4.0, 8.0]], [[6.0, 7.0, 6.0], [3.0, 4.0, 4.0], [3.0, 6.0, 
5.0]]], [[[3.0, 3.0, 4.0], [1.0, 3.0, 5.0], [4.0, 3.0, 2.0]], [[5.0, 8.0, 5.0], [6.0, 7.0, 8.0], [1.0, 8.0, 6.0]], [[6.0, 1.0, 
4.0], [3.0, 8.0, 4.0], [2.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 2.0], [8.0, 8.0, 8.0], [2.0, 6.0, 3.0]], 
[[3.0, 2.0, 3.0], [8.0, 8.0, 2.0], [7.0, 8.0, 6.0]], [[6.0, 3.0, 7.0], [6.0, 3.0, 3.0], [5.0, 5.0, 3.0]]], [[[7.0, 7.0, 7.0], 
[5.0, 5.0, 8.0], [7.0, 3.0, 7.0]], [[1.0, 5.0, 1.0], [8.0, 8.0, 6.0], [7.0, 5.0, 3.0]], [[4.0, 4.0, 7.0], [2.0, 3.0, 2.0], 
[4.0, 3.0, 4.0]]], [[[2.0, 8.0, 2.0], [3.0, 1.0, 3.0], [5.0, 2.0, 2.0]], [[4.0, 6.0, 1.0], [1.0, 4.0, 4.0], [6.0, 8.0, 2.0]], 
[[2.0, 2.0, 3.0], [8.0, 4.0, 8.0], [4.0, 5.0, 1.0]]], [[[4.0, 7.0, 5.0], [7.0, 8.0, 7.0], [8.0, 2.0, 4.0]], [[5.0, 5.0, 8.0], 
[2.0, 2.0, 8.0], [8.0, 6.0, 4.0]], [[5.0, 4.0, 5.0], [6.0, 5.0, 8.0], [5.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CLLN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 6.0], [7.0, 4.0, 6.0], [4.0, 7.0, 1.0]], [[6.0, 7.0, 6.0], [7.0, 2.0, 4.0], [8.0, 1.0, 
5.0]], [[3.0, 6.0, 3.0], [3.0, 2.0, 4.0], [6.0, 3.0, 4.0]]], [[[5.0, 2.0, 4.0], [5.0, 7.0, 3.0], [7.0, 6.0, 2.0]], [[5.0, 6.0, 
8.0], [4.0, 4.0, 3.0], [5.0, 8.0, 8.0]], [[1.0, 1.0, 4.0], [4.0, 3.0, 1.0], [1.0, 5.0, 5.0]]], [[[3.0, 4.0, 2.0], [8.0, 6.0, 
6.0], [5.0, 4.0, 8.0]], [[3.0, 4.0, 3.0], [1.0, 4.0, 2.0], [3.0, 2.0, 3.0]], [[7.0, 2.0, 7.0], [3.0, 3.0, 8.0], [8.0, 2.0, 
6.0]]], [[[8.0, 2.0, 2.0], [1.0, 3.0, 4.0], [6.0, 5.0, 8.0]], [[2.0, 3.0, 5.0], [3.0, 5.0, 5.0], [7.0, 2.0, 5.0]], [[2.0, 3.0, 
7.0], [4.0, 5.0, 7.0], [2.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 1.0], [1.0, 6.0, 8.0], [2.0, 1.0, 8.0]], [[3.0, 7.0, 3.0], [1.0, 5.0, 
4.0], [2.0, 5.0, 6.0]], [[5.0, 5.0, 2.0], [8.0, 8.0, 3.0], [5.0, 1.0, 8.0]]], [[[7.0, 5.0, 6.0], [1.0, 1.0, 3.0], [6.0, 6.0, 
1.0]], [[5.0, 2.0, 7.0], [8.0, 7.0, 5.0], [5.0, 5.0, 3.0]], [[7.0, 5.0, 8.0], [7.0, 4.0, 8.0], [4.0, 1.0, 3.0]]], [[[8.0, 1.0, 
8.0], [2.0, 2.0, 3.0], [1.0, 6.0, 5.0]], [[7.0, 4.0, 1.0], [2.0, 5.0, 1.0], [6.0, 6.0, 1.0]], [[2.0, 3.0, 2.0], [4.0, 7.0, 
5.0], [5.0, 2.0, 7.0]]], [[[2.0, 6.0, 1.0], [2.0, 7.0, 1.0], [7.0, 5.0, 8.0]], [[6.0, 8.0, 3.0], [8.0, 1.0, 7.0], [1.0, 7.0, 
5.0]], [[2.0, 2.0, 7.0], [3.0, 6.0, 2.0], [3.0, 7.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 6.0, 1.0]]], [[[5.0, 8.0, 2.0]]], [[[4.0, 5.0, 6.0]]], [[[5.0, 5.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 6.0, 4.0]]], [[[8.0, 5.0, 4.0]]], [[[7.0, 5.0, 3.0]]], [[[3.0, 7.0, 8.0]]]])
      arg[:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 6.0], [7.0, 4.0, 6.0], [4.0, 7.0, 1.0]], [[6.0, 7.0, 6.0], [7.0, 2.0, 4.0], [8.0, 
1.0, 5.0]], [[3.0, 6.0, 3.0], [3.0, 2.0, 4.0], [2.0, 6.0, 1.0]]], [[[5.0, 2.0, 4.0], [5.0, 7.0, 3.0], [7.0, 6.0, 2.0]], [[5.0, 
6.0, 8.0], [4.0, 4.0, 3.0], [5.0, 8.0, 8.0]], [[1.0, 1.0, 4.0], [4.0, 3.0, 1.0], [5.0, 8.0, 2.0]]], [[[3.0, 4.0, 2.0], [8.0, 
6.0, 6.0], [5.0, 4.0, 8.0]], [[3.0, 4.0, 3.0], [1.0, 4.0, 2.0], [3.0, 2.0, 3.0]], [[7.0, 2.0, 7.0], [3.0, 3.0, 8.0], [4.0, 5.0, 
6.0]]], [[[8.0, 2.0, 2.0], [1.0, 3.0, 4.0], [6.0, 5.0, 8.0]], [[2.0, 3.0, 5.0], [3.0, 5.0, 5.0], [7.0, 2.0, 5.0]], [[2.0, 3.0, 
7.0], [4.0, 5.0, 7.0], [5.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 1.0], [1.0, 6.0, 8.0], [2.0, 1.0, 8.0]], 
[[3.0, 7.0, 3.0], [1.0, 5.0, 4.0], [2.0, 5.0, 6.0]], [[5.0, 5.0, 2.0], [8.0, 8.0, 3.0], [6.0, 6.0, 4.0]]], [[[7.0, 5.0, 6.0], 
[1.0, 1.0, 3.0], [6.0, 6.0, 1.0]], [[5.0, 2.0, 7.0], [8.0, 7.0, 5.0], [5.0, 5.0, 3.0]], [[7.0, 5.0, 8.0], [7.0, 4.0, 8.0], 
[8.0, 5.0, 4.0]]], [[[8.0, 1.0, 8.0], [2.0, 2.0, 3.0], [1.0, 6.0, 5.0]], [[7.0, 4.0, 1.0], [2.0, 5.0, 1.0], [6.0, 6.0, 1.0]], 
[[2.0, 3.0, 2.0], [4.0, 7.0, 5.0], [7.0, 5.0, 3.0]]], [[[2.0, 6.0, 1.0], [2.0, 7.0, 1.0], [7.0, 5.0, 8.0]], [[6.0, 8.0, 3.0], 
[8.0, 1.0, 7.0], [1.0, 7.0, 5.0]], [[2.0, 2.0, 7.0], [3.0, 6.0, 2.0], [3.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLPN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 3.0], [6.0, 6.0, 7.0], [7.0, 3.0, 3.0]], [[1.0, 4.0, 3.0], [5.0, 6.0, 5.0], [6.0, 6.0, 
4.0]], [[8.0, 2.0, 6.0], [5.0, 6.0, 7.0], [6.0, 3.0, 3.0]]], [[[3.0, 1.0, 5.0], [3.0, 2.0, 8.0], [4.0, 1.0, 5.0]], [[4.0, 7.0, 
5.0], [3.0, 3.0, 4.0], [4.0, 1.0, 7.0]], [[1.0, 5.0, 8.0], [7.0, 5.0, 1.0], [6.0, 7.0, 3.0]]], [[[3.0, 6.0, 8.0], [2.0, 3.0, 
3.0], [5.0, 5.0, 5.0]], [[2.0, 6.0, 1.0], [8.0, 3.0, 2.0], [4.0, 7.0, 5.0]], [[7.0, 3.0, 7.0], [5.0, 6.0, 5.0], [1.0, 1.0, 
7.0]]], [[[7.0, 3.0, 6.0], [5.0, 8.0, 2.0], [8.0, 1.0, 1.0]], [[2.0, 5.0, 6.0], [3.0, 2.0, 7.0], [2.0, 1.0, 7.0]], [[1.0, 3.0, 
1.0], [6.0, 1.0, 6.0], [4.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 1.0], [4.0, 1.0, 5.0], [1.0, 7.0, 8.0]], [[6.0, 1.0, 7.0], [5.0, 2.0, 
3.0], [3.0, 2.0, 6.0]], [[3.0, 3.0, 6.0], [4.0, 6.0, 3.0], [3.0, 4.0, 7.0]]], [[[2.0, 6.0, 2.0], [8.0, 4.0, 8.0], [5.0, 8.0, 
7.0]], [[7.0, 1.0, 6.0], [5.0, 4.0, 8.0], [4.0, 4.0, 5.0]], [[3.0, 3.0, 1.0], [1.0, 2.0, 3.0], [2.0, 7.0, 3.0]]], [[[6.0, 1.0, 
4.0], [2.0, 7.0, 7.0], [8.0, 3.0, 5.0]], [[2.0, 7.0, 5.0], [6.0, 6.0, 2.0], [4.0, 6.0, 4.0]], [[1.0, 5.0, 3.0], [7.0, 6.0, 
8.0], [5.0, 4.0, 4.0]]], [[[1.0, 6.0, 4.0], [4.0, 6.0, 7.0], [6.0, 2.0, 7.0]], [[2.0, 4.0, 3.0], [7.0, 8.0, 3.0], [2.0, 6.0, 
8.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 5.0], [7.0, 4.0, 7.0]]]]))
      val=numpy.array([[5.0, 6.0, 6.0]])
      arg[2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 3.0], [6.0, 6.0, 7.0], [7.0, 3.0, 3.0]], [[1.0, 4.0, 3.0], [5.0, 6.0, 5.0], [6.0, 
6.0, 4.0]], [[8.0, 2.0, 6.0], [5.0, 6.0, 7.0], [6.0, 3.0, 3.0]]], [[[3.0, 1.0, 5.0], [3.0, 2.0, 8.0], [4.0, 1.0, 5.0]], [[4.0, 
7.0, 5.0], [3.0, 3.0, 4.0], [4.0, 1.0, 7.0]], [[1.0, 5.0, 8.0], [7.0, 5.0, 1.0], [6.0, 7.0, 3.0]]], [[[3.0, 6.0, 8.0], [2.0, 
3.0, 3.0], [5.0, 5.0, 5.0]], [[2.0, 6.0, 1.0], [8.0, 3.0, 2.0], [4.0, 7.0, 5.0]], [[7.0, 3.0, 7.0], [5.0, 6.0, 6.0], [1.0, 1.0, 
7.0]]], [[[7.0, 3.0, 6.0], [5.0, 8.0, 2.0], [8.0, 1.0, 1.0]], [[2.0, 5.0, 6.0], [3.0, 2.0, 7.0], [2.0, 1.0, 7.0]], [[1.0, 3.0, 
1.0], [6.0, 1.0, 6.0], [4.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 1.0], [4.0, 1.0, 5.0], [1.0, 7.0, 8.0]], 
[[6.0, 1.0, 7.0], [5.0, 2.0, 3.0], [3.0, 2.0, 6.0]], [[3.0, 3.0, 6.0], [4.0, 6.0, 3.0], [3.0, 4.0, 7.0]]], [[[2.0, 6.0, 2.0], 
[8.0, 4.0, 8.0], [5.0, 8.0, 7.0]], [[7.0, 1.0, 6.0], [5.0, 4.0, 8.0], [4.0, 4.0, 5.0]], [[3.0, 3.0, 1.0], [1.0, 2.0, 3.0], 
[2.0, 7.0, 3.0]]], [[[6.0, 1.0, 4.0], [2.0, 7.0, 7.0], [8.0, 3.0, 5.0]], [[2.0, 7.0, 5.0], [6.0, 6.0, 2.0], [4.0, 6.0, 4.0]], 
[[1.0, 5.0, 3.0], [5.0, 6.0, 6.0], [5.0, 4.0, 4.0]]], [[[1.0, 6.0, 4.0], [4.0, 6.0, 7.0], [6.0, 2.0, 7.0]], [[2.0, 4.0, 3.0], 
[7.0, 8.0, 3.0], [2.0, 6.0, 8.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 5.0], [7.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLPN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 7.0], [8.0, 7.0, 1.0], [7.0, 7.0, 3.0]], [[5.0, 5.0, 7.0], [6.0, 8.0, 1.0], [8.0, 5.0, 
5.0]], [[5.0, 7.0, 6.0], [1.0, 5.0, 1.0], [5.0, 6.0, 1.0]]], [[[6.0, 4.0, 6.0], [5.0, 2.0, 1.0], [1.0, 8.0, 8.0]], [[7.0, 7.0, 
7.0], [8.0, 8.0, 3.0], [2.0, 4.0, 1.0]], [[1.0, 8.0, 1.0], [8.0, 7.0, 1.0], [8.0, 6.0, 2.0]]], [[[8.0, 3.0, 8.0], [6.0, 7.0, 
6.0], [3.0, 2.0, 4.0]], [[4.0, 6.0, 6.0], [5.0, 5.0, 1.0], [1.0, 5.0, 1.0]], [[5.0, 4.0, 2.0], [5.0, 6.0, 4.0], [2.0, 2.0, 
4.0]]], [[[4.0, 3.0, 8.0], [4.0, 7.0, 6.0], [4.0, 1.0, 2.0]], [[1.0, 6.0, 4.0], [6.0, 3.0, 4.0], [2.0, 5.0, 4.0]], [[5.0, 8.0, 
3.0], [4.0, 4.0, 1.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 8.0], [2.0, 7.0, 2.0], [6.0, 7.0, 5.0]], [[2.0, 6.0, 4.0], [4.0, 1.0, 
5.0], [5.0, 7.0, 4.0]], [[7.0, 6.0, 5.0], [5.0, 8.0, 1.0], [3.0, 3.0, 8.0]]], [[[7.0, 6.0, 5.0], [7.0, 8.0, 3.0], [3.0, 1.0, 
3.0]], [[3.0, 1.0, 4.0], [2.0, 5.0, 3.0], [5.0, 3.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 4.0, 4.0], [1.0, 6.0, 3.0]]], [[[8.0, 7.0, 
2.0], [2.0, 1.0, 1.0], [7.0, 5.0, 8.0]], [[8.0, 2.0, 6.0], [6.0, 5.0, 5.0], [4.0, 5.0, 8.0]], [[1.0, 6.0, 6.0], [3.0, 6.0, 
4.0], [5.0, 6.0, 4.0]]], [[[1.0, 6.0, 6.0], [3.0, 2.0, 2.0], [2.0, 2.0, 2.0]], [[5.0, 6.0, 5.0], [2.0, 4.0, 3.0], [1.0, 2.0, 
5.0]], [[2.0, 2.0, 6.0], [8.0, 7.0, 8.0], [4.0, 7.0, 1.0]]]]))
      val=Data(numpy.array([[1.0, 5.0, 6.0]]),self.functionspace)
      arg[2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 7.0], [8.0, 7.0, 1.0], [7.0, 7.0, 3.0]], [[5.0, 5.0, 7.0], [6.0, 8.0, 1.0], [8.0, 
5.0, 5.0]], [[5.0, 7.0, 6.0], [1.0, 5.0, 1.0], [5.0, 6.0, 1.0]]], [[[6.0, 4.0, 6.0], [5.0, 2.0, 1.0], [1.0, 8.0, 8.0]], [[7.0, 
7.0, 7.0], [8.0, 8.0, 3.0], [2.0, 4.0, 1.0]], [[1.0, 8.0, 1.0], [8.0, 7.0, 1.0], [8.0, 6.0, 2.0]]], [[[8.0, 3.0, 8.0], [6.0, 
7.0, 6.0], [3.0, 2.0, 4.0]], [[4.0, 6.0, 6.0], [5.0, 5.0, 1.0], [1.0, 5.0, 1.0]], [[5.0, 4.0, 2.0], [1.0, 5.0, 6.0], [2.0, 2.0, 
4.0]]], [[[4.0, 3.0, 8.0], [4.0, 7.0, 6.0], [4.0, 1.0, 2.0]], [[1.0, 6.0, 4.0], [6.0, 3.0, 4.0], [2.0, 5.0, 4.0]], [[5.0, 8.0, 
3.0], [4.0, 4.0, 1.0], [4.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 8.0], [2.0, 7.0, 2.0], [6.0, 7.0, 5.0]], 
[[2.0, 6.0, 4.0], [4.0, 1.0, 5.0], [5.0, 7.0, 4.0]], [[7.0, 6.0, 5.0], [5.0, 8.0, 1.0], [3.0, 3.0, 8.0]]], [[[7.0, 6.0, 5.0], 
[7.0, 8.0, 3.0], [3.0, 1.0, 3.0]], [[3.0, 1.0, 4.0], [2.0, 5.0, 3.0], [5.0, 3.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 4.0, 4.0], 
[1.0, 6.0, 3.0]]], [[[8.0, 7.0, 2.0], [2.0, 1.0, 1.0], [7.0, 5.0, 8.0]], [[8.0, 2.0, 6.0], [6.0, 5.0, 5.0], [4.0, 5.0, 8.0]], 
[[1.0, 6.0, 6.0], [1.0, 5.0, 6.0], [5.0, 6.0, 4.0]]], [[[1.0, 6.0, 6.0], [3.0, 2.0, 2.0], [2.0, 2.0, 2.0]], [[5.0, 6.0, 5.0], 
[2.0, 4.0, 3.0], [1.0, 2.0, 5.0]], [[2.0, 2.0, 6.0], [8.0, 7.0, 8.0], [4.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLPN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 6.0], [8.0, 7.0, 3.0], [3.0, 2.0, 2.0]], [[3.0, 6.0, 8.0], [2.0, 1.0, 3.0], [4.0, 1.0, 
8.0]], [[4.0, 6.0, 2.0], [4.0, 6.0, 8.0], [7.0, 8.0, 7.0]]], [[[6.0, 6.0, 4.0], [8.0, 3.0, 7.0], [6.0, 2.0, 2.0]], [[5.0, 5.0, 
1.0], [5.0, 1.0, 1.0], [7.0, 6.0, 6.0]], [[5.0, 5.0, 4.0], [6.0, 4.0, 4.0], [6.0, 8.0, 5.0]]], [[[5.0, 2.0, 3.0], [6.0, 8.0, 
8.0], [3.0, 1.0, 2.0]], [[4.0, 3.0, 8.0], [8.0, 3.0, 8.0], [7.0, 5.0, 1.0]], [[7.0, 3.0, 4.0], [2.0, 4.0, 2.0], [4.0, 7.0, 
1.0]]], [[[5.0, 5.0, 4.0], [3.0, 1.0, 1.0], [5.0, 6.0, 2.0]], [[3.0, 1.0, 1.0], [3.0, 6.0, 3.0], [5.0, 1.0, 5.0]], [[8.0, 6.0, 
4.0], [6.0, 6.0, 8.0], [2.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 3.0], [7.0, 4.0, 1.0], [1.0, 4.0, 3.0]], [[4.0, 8.0, 1.0], [6.0, 2.0, 
1.0], [4.0, 2.0, 6.0]], [[7.0, 1.0, 6.0], [6.0, 3.0, 8.0], [5.0, 5.0, 4.0]]], [[[2.0, 4.0, 2.0], [5.0, 4.0, 3.0], [6.0, 8.0, 
3.0]], [[2.0, 5.0, 8.0], [2.0, 3.0, 2.0], [7.0, 6.0, 8.0]], [[8.0, 2.0, 2.0], [7.0, 1.0, 8.0], [6.0, 1.0, 5.0]]], [[[8.0, 7.0, 
5.0], [2.0, 7.0, 6.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 2.0], [8.0, 6.0, 4.0], [2.0, 8.0, 8.0]], [[1.0, 7.0, 5.0], [4.0, 1.0, 
4.0], [8.0, 2.0, 8.0]]], [[[3.0, 4.0, 4.0], [5.0, 6.0, 1.0], [4.0, 6.0, 8.0]], [[2.0, 1.0, 6.0], [5.0, 4.0, 8.0], [8.0, 4.0, 
5.0]], [[8.0, 4.0, 8.0], [8.0, 3.0, 8.0], [8.0, 1.0, 5.0]]]]))
      val=Data(numpy.array([[4.0, 1.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 6.0, 6.0]]))
      arg[2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 6.0], [8.0, 7.0, 3.0], [3.0, 2.0, 2.0]], [[3.0, 6.0, 8.0], [2.0, 1.0, 3.0], [4.0, 
1.0, 8.0]], [[4.0, 6.0, 2.0], [4.0, 6.0, 8.0], [7.0, 8.0, 7.0]]], [[[6.0, 6.0, 4.0], [8.0, 3.0, 7.0], [6.0, 2.0, 2.0]], [[5.0, 
5.0, 1.0], [5.0, 1.0, 1.0], [7.0, 6.0, 6.0]], [[5.0, 5.0, 4.0], [6.0, 4.0, 4.0], [6.0, 8.0, 5.0]]], [[[5.0, 2.0, 3.0], [6.0, 
8.0, 8.0], [3.0, 1.0, 2.0]], [[4.0, 3.0, 8.0], [8.0, 3.0, 8.0], [7.0, 5.0, 1.0]], [[7.0, 3.0, 4.0], [4.0, 1.0, 3.0], [4.0, 7.0, 
1.0]]], [[[5.0, 5.0, 4.0], [3.0, 1.0, 1.0], [5.0, 6.0, 2.0]], [[3.0, 1.0, 1.0], [3.0, 6.0, 3.0], [5.0, 1.0, 5.0]], [[8.0, 6.0, 
4.0], [6.0, 6.0, 8.0], [2.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 3.0], [7.0, 4.0, 1.0], [1.0, 4.0, 3.0]], 
[[4.0, 8.0, 1.0], [6.0, 2.0, 1.0], [4.0, 2.0, 6.0]], [[7.0, 1.0, 6.0], [6.0, 3.0, 8.0], [5.0, 5.0, 4.0]]], [[[2.0, 4.0, 2.0], 
[5.0, 4.0, 3.0], [6.0, 8.0, 3.0]], [[2.0, 5.0, 8.0], [2.0, 3.0, 2.0], [7.0, 6.0, 8.0]], [[8.0, 2.0, 2.0], [7.0, 1.0, 8.0], 
[6.0, 1.0, 5.0]]], [[[8.0, 7.0, 5.0], [2.0, 7.0, 6.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 2.0], [8.0, 6.0, 4.0], [2.0, 8.0, 8.0]], 
[[1.0, 7.0, 5.0], [4.0, 6.0, 6.0], [8.0, 2.0, 8.0]]], [[[3.0, 4.0, 4.0], [5.0, 6.0, 1.0], [4.0, 6.0, 8.0]], [[2.0, 1.0, 6.0], 
[5.0, 4.0, 8.0], [8.0, 4.0, 5.0]], [[8.0, 4.0, 8.0], [8.0, 3.0, 8.0], [8.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLPN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 5.0], [1.0, 6.0, 8.0], [3.0, 5.0, 4.0]], [[7.0, 4.0, 6.0], [6.0, 3.0, 7.0], [5.0, 1.0, 
3.0]], [[1.0, 4.0, 4.0], [4.0, 5.0, 4.0], [1.0, 1.0, 6.0]]], [[[2.0, 2.0, 1.0], [2.0, 6.0, 1.0], [5.0, 2.0, 1.0]], [[4.0, 3.0, 
3.0], [1.0, 6.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 8.0, 4.0], [3.0, 7.0, 8.0], [2.0, 3.0, 3.0]]], [[[1.0, 1.0, 3.0], [5.0, 3.0, 
1.0], [3.0, 1.0, 1.0]], [[1.0, 6.0, 1.0], [2.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[8.0, 3.0, 4.0], [2.0, 2.0, 1.0], [1.0, 2.0, 
4.0]]], [[[2.0, 5.0, 4.0], [8.0, 2.0, 5.0], [4.0, 8.0, 5.0]], [[7.0, 5.0, 8.0], [4.0, 2.0, 2.0], [8.0, 7.0, 1.0]], [[6.0, 1.0, 
6.0], [6.0, 7.0, 4.0], [6.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 6.0], [3.0, 1.0, 3.0], [4.0, 4.0, 6.0]], [[2.0, 7.0, 7.0], [3.0, 3.0, 
4.0], [6.0, 6.0, 2.0]], [[4.0, 2.0, 4.0], [3.0, 2.0, 5.0], [3.0, 4.0, 7.0]]], [[[5.0, 5.0, 8.0], [5.0, 6.0, 6.0], [1.0, 1.0, 
8.0]], [[3.0, 1.0, 8.0], [6.0, 5.0, 4.0], [5.0, 6.0, 8.0]], [[6.0, 6.0, 1.0], [6.0, 4.0, 5.0], [7.0, 5.0, 1.0]]], [[[6.0, 5.0, 
1.0], [4.0, 6.0, 6.0], [7.0, 6.0, 5.0]], [[8.0, 4.0, 7.0], [4.0, 2.0, 3.0], [6.0, 5.0, 4.0]], [[1.0, 2.0, 3.0], [4.0, 1.0, 
8.0], [2.0, 8.0, 4.0]]], [[[8.0, 2.0, 2.0], [1.0, 1.0, 7.0], [4.0, 4.0, 2.0]], [[4.0, 6.0, 5.0], [1.0, 7.0, 4.0], [5.0, 1.0, 
5.0]], [[2.0, 5.0, 3.0], [3.0, 8.0, 3.0], [6.0, 5.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[1.0, 1.0, 5.0]])+(1.-msk_val)*numpy.array([[4.0, 2.0, 6.0]])
      arg[2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 5.0], [1.0, 6.0, 8.0], [3.0, 5.0, 4.0]], [[7.0, 4.0, 6.0], [6.0, 3.0, 7.0], [5.0, 
1.0, 3.0]], [[1.0, 4.0, 4.0], [4.0, 5.0, 4.0], [1.0, 1.0, 6.0]]], [[[2.0, 2.0, 1.0], [2.0, 6.0, 1.0], [5.0, 2.0, 1.0]], [[4.0, 
3.0, 3.0], [1.0, 6.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 8.0, 4.0], [3.0, 7.0, 8.0], [2.0, 3.0, 3.0]]], [[[1.0, 1.0, 3.0], [5.0, 
3.0, 1.0], [3.0, 1.0, 1.0]], [[1.0, 6.0, 1.0], [2.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[8.0, 3.0, 4.0], [1.0, 1.0, 5.0], [1.0, 2.0, 
4.0]]], [[[2.0, 5.0, 4.0], [8.0, 2.0, 5.0], [4.0, 8.0, 5.0]], [[7.0, 5.0, 8.0], [4.0, 2.0, 2.0], [8.0, 7.0, 1.0]], [[6.0, 1.0, 
6.0], [6.0, 7.0, 4.0], [6.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 6.0], [3.0, 1.0, 3.0], [4.0, 4.0, 6.0]], 
[[2.0, 7.0, 7.0], [3.0, 3.0, 4.0], [6.0, 6.0, 2.0]], [[4.0, 2.0, 4.0], [3.0, 2.0, 5.0], [3.0, 4.0, 7.0]]], [[[5.0, 5.0, 8.0], 
[5.0, 6.0, 6.0], [1.0, 1.0, 8.0]], [[3.0, 1.0, 8.0], [6.0, 5.0, 4.0], [5.0, 6.0, 8.0]], [[6.0, 6.0, 1.0], [6.0, 4.0, 5.0], 
[7.0, 5.0, 1.0]]], [[[6.0, 5.0, 1.0], [4.0, 6.0, 6.0], [7.0, 6.0, 5.0]], [[8.0, 4.0, 7.0], [4.0, 2.0, 3.0], [6.0, 5.0, 4.0]], 
[[1.0, 2.0, 3.0], [4.0, 2.0, 6.0], [2.0, 8.0, 4.0]]], [[[8.0, 2.0, 2.0], [1.0, 1.0, 7.0], [4.0, 4.0, 2.0]], [[4.0, 6.0, 5.0], 
[1.0, 7.0, 4.0], [5.0, 1.0, 5.0]], [[2.0, 5.0, 3.0], [3.0, 8.0, 3.0], [6.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UCCN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 3.0], [5.0, 8.0, 3.0], [2.0, 1.0, 2.0]], [[4.0, 8.0, 3.0], [3.0, 8.0, 7.0], [8.0, 7.0, 
5.0]], [[2.0, 6.0, 1.0], [5.0, 4.0, 4.0], [1.0, 3.0, 8.0]]], [[[2.0, 4.0, 4.0], [3.0, 8.0, 7.0], [8.0, 4.0, 2.0]], [[4.0, 3.0, 
2.0], [1.0, 7.0, 3.0], [2.0, 4.0, 7.0]], [[6.0, 2.0, 2.0], [2.0, 6.0, 2.0], [8.0, 6.0, 2.0]]], [[[2.0, 7.0, 1.0], [3.0, 3.0, 
7.0], [5.0, 1.0, 3.0]], [[8.0, 4.0, 8.0], [1.0, 1.0, 3.0], [6.0, 8.0, 1.0]], [[2.0, 6.0, 4.0], [8.0, 2.0, 4.0], [1.0, 4.0, 
1.0]]], [[[6.0, 3.0, 4.0], [7.0, 4.0, 2.0], [6.0, 1.0, 4.0]], [[7.0, 6.0, 7.0], [2.0, 2.0, 4.0], [1.0, 1.0, 6.0]], [[2.0, 2.0, 
1.0], [2.0, 1.0, 1.0], [1.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 6.0], [1.0, 5.0, 5.0], [3.0, 4.0, 1.0]], [[6.0, 5.0, 1.0], [1.0, 1.0, 
2.0], [8.0, 5.0, 2.0]], [[5.0, 1.0, 3.0], [4.0, 6.0, 1.0], [1.0, 2.0, 4.0]]], [[[6.0, 3.0, 6.0], [3.0, 4.0, 6.0], [1.0, 8.0, 
3.0]], [[6.0, 2.0, 3.0], [5.0, 2.0, 1.0], [6.0, 5.0, 7.0]], [[7.0, 7.0, 2.0], [8.0, 7.0, 3.0], [7.0, 1.0, 2.0]]], [[[6.0, 8.0, 
4.0], [7.0, 1.0, 5.0], [6.0, 1.0, 4.0]], [[7.0, 5.0, 2.0], [5.0, 5.0, 7.0], [6.0, 3.0, 2.0]], [[8.0, 4.0, 6.0], [7.0, 6.0, 
8.0], [7.0, 4.0, 3.0]]], [[[3.0, 6.0, 4.0], [7.0, 7.0, 5.0], [4.0, 4.0, 6.0]], [[6.0, 5.0, 3.0], [5.0, 1.0, 7.0], [3.0, 2.0, 
7.0]], [[4.0, 4.0, 8.0], [6.0, 6.0, 4.0], [7.0, 1.0, 2.0]]]]))
      val=numpy.array([[[[2.0, 7.0, 3.0], [2.0, 3.0, 8.0], [2.0, 5.0, 4.0]], [[1.0, 2.0, 4.0], [1.0, 2.0, 1.0], [1.0, 5.0, 
5.0]], [[3.0, 3.0, 1.0], [2.0, 5.0, 1.0], [5.0, 6.0, 7.0]]], [[[3.0, 2.0, 2.0], [7.0, 6.0, 3.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 
2.0], [4.0, 8.0, 1.0], [2.0, 3.0, 5.0]], [[5.0, 7.0, 7.0], [6.0, 4.0, 7.0], [7.0, 3.0, 6.0]]], [[[3.0, 5.0, 2.0], [4.0, 8.0, 
2.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 8.0], [3.0, 7.0, 7.0], [8.0, 1.0, 6.0]], [[5.0, 6.0, 5.0], [4.0, 2.0, 1.0], [2.0, 6.0, 
7.0]]]])
      arg[:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [2.0, 3.0, 8.0], [2.0, 5.0, 4.0]], [[1.0, 2.0, 4.0], [1.0, 2.0, 1.0], [1.0, 
5.0, 5.0]], [[3.0, 3.0, 1.0], [2.0, 5.0, 1.0], [5.0, 6.0, 7.0]]], [[[3.0, 2.0, 2.0], [7.0, 6.0, 3.0], [7.0, 4.0, 3.0]], [[8.0, 
7.0, 2.0], [4.0, 8.0, 1.0], [2.0, 3.0, 5.0]], [[5.0, 7.0, 7.0], [6.0, 4.0, 7.0], [7.0, 3.0, 6.0]]], [[[3.0, 5.0, 2.0], [4.0, 
8.0, 2.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 8.0], [3.0, 7.0, 7.0], [8.0, 1.0, 6.0]], [[5.0, 6.0, 5.0], [4.0, 2.0, 1.0], [2.0, 6.0, 
7.0]]], [[[6.0, 3.0, 4.0], [7.0, 4.0, 2.0], [6.0, 1.0, 4.0]], [[7.0, 6.0, 7.0], [2.0, 2.0, 4.0], [1.0, 1.0, 6.0]], [[2.0, 2.0, 
1.0], [2.0, 1.0, 1.0], [1.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 3.0], [2.0, 3.0, 8.0], [2.0, 5.0, 4.0]], 
[[1.0, 2.0, 4.0], [1.0, 2.0, 1.0], [1.0, 5.0, 5.0]], [[3.0, 3.0, 1.0], [2.0, 5.0, 1.0], [5.0, 6.0, 7.0]]], [[[3.0, 2.0, 2.0], 
[7.0, 6.0, 3.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 2.0], [4.0, 8.0, 1.0], [2.0, 3.0, 5.0]], [[5.0, 7.0, 7.0], [6.0, 4.0, 7.0], 
[7.0, 3.0, 6.0]]], [[[3.0, 5.0, 2.0], [4.0, 8.0, 2.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 8.0], [3.0, 7.0, 7.0], [8.0, 1.0, 6.0]], 
[[5.0, 6.0, 5.0], [4.0, 2.0, 1.0], [2.0, 6.0, 7.0]]], [[[3.0, 6.0, 4.0], [7.0, 7.0, 5.0], [4.0, 4.0, 6.0]], [[6.0, 5.0, 3.0], 
[5.0, 1.0, 7.0], [3.0, 2.0, 7.0]], [[4.0, 4.0, 8.0], [6.0, 6.0, 4.0], [7.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UCCN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 6.0], [8.0, 2.0, 1.0], [6.0, 3.0, 5.0]], [[1.0, 1.0, 2.0], [5.0, 4.0, 5.0], [8.0, 7.0, 
4.0]], [[8.0, 6.0, 2.0], [6.0, 1.0, 2.0], [4.0, 8.0, 2.0]]], [[[6.0, 4.0, 6.0], [4.0, 3.0, 3.0], [2.0, 4.0, 7.0]], [[5.0, 1.0, 
5.0], [6.0, 6.0, 8.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 5.0], [6.0, 8.0, 2.0], [2.0, 1.0, 5.0]]], [[[3.0, 6.0, 3.0], [3.0, 8.0, 
5.0], [4.0, 4.0, 4.0]], [[7.0, 2.0, 4.0], [8.0, 2.0, 7.0], [3.0, 4.0, 6.0]], [[6.0, 8.0, 5.0], [3.0, 5.0, 3.0], [8.0, 1.0, 
6.0]]], [[[8.0, 7.0, 2.0], [1.0, 6.0, 3.0], [1.0, 6.0, 1.0]], [[5.0, 5.0, 6.0], [2.0, 6.0, 6.0], [4.0, 2.0, 4.0]], [[8.0, 7.0, 
8.0], [1.0, 1.0, 6.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 2.0], [4.0, 4.0, 5.0], [7.0, 3.0, 3.0]], [[3.0, 6.0, 7.0], [8.0, 8.0, 
4.0], [5.0, 5.0, 7.0]], [[4.0, 1.0, 8.0], [3.0, 8.0, 5.0], [4.0, 4.0, 6.0]]], [[[6.0, 6.0, 7.0], [3.0, 4.0, 2.0], [1.0, 4.0, 
2.0]], [[2.0, 5.0, 8.0], [5.0, 1.0, 8.0], [2.0, 2.0, 6.0]], [[6.0, 3.0, 2.0], [8.0, 5.0, 3.0], [5.0, 2.0, 1.0]]], [[[6.0, 7.0, 
2.0], [6.0, 7.0, 8.0], [8.0, 3.0, 5.0]], [[8.0, 1.0, 3.0], [7.0, 1.0, 2.0], [3.0, 3.0, 8.0]], [[2.0, 5.0, 6.0], [1.0, 5.0, 
5.0], [7.0, 7.0, 6.0]]], [[[6.0, 1.0, 3.0], [4.0, 7.0, 2.0], [2.0, 7.0, 6.0]], [[8.0, 3.0, 1.0], [7.0, 7.0, 1.0], [3.0, 2.0, 
7.0]], [[2.0, 2.0, 4.0], [4.0, 8.0, 5.0], [2.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([[[[4.0, 4.0, 8.0], [2.0, 2.0, 1.0], [6.0, 8.0, 8.0]], [[6.0, 4.0, 2.0], [4.0, 2.0, 5.0], [6.0, 6.0, 
2.0]], [[4.0, 4.0, 1.0], [7.0, 5.0, 7.0], [4.0, 5.0, 3.0]]], [[[6.0, 3.0, 3.0], [8.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[8.0, 7.0, 
7.0], [5.0, 2.0, 6.0], [1.0, 8.0, 8.0]], [[3.0, 1.0, 3.0], [3.0, 1.0, 3.0], [6.0, 7.0, 4.0]]], [[[4.0, 5.0, 7.0], [5.0, 7.0, 
6.0], [7.0, 6.0, 6.0]], [[8.0, 8.0, 7.0], [8.0, 1.0, 5.0], [1.0, 4.0, 3.0]], [[1.0, 2.0, 7.0], [6.0, 1.0, 4.0], [5.0, 1.0, 
6.0]]]]),self.functionspace)
      arg[:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 8.0], [2.0, 2.0, 1.0], [6.0, 8.0, 8.0]], [[6.0, 4.0, 2.0], [4.0, 2.0, 5.0], [6.0, 
6.0, 2.0]], [[4.0, 4.0, 1.0], [7.0, 5.0, 7.0], [4.0, 5.0, 3.0]]], [[[6.0, 3.0, 3.0], [8.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[8.0, 
7.0, 7.0], [5.0, 2.0, 6.0], [1.0, 8.0, 8.0]], [[3.0, 1.0, 3.0], [3.0, 1.0, 3.0], [6.0, 7.0, 4.0]]], [[[4.0, 5.0, 7.0], [5.0, 
7.0, 6.0], [7.0, 6.0, 6.0]], [[8.0, 8.0, 7.0], [8.0, 1.0, 5.0], [1.0, 4.0, 3.0]], [[1.0, 2.0, 7.0], [6.0, 1.0, 4.0], [5.0, 1.0, 
6.0]]], [[[8.0, 7.0, 2.0], [1.0, 6.0, 3.0], [1.0, 6.0, 1.0]], [[5.0, 5.0, 6.0], [2.0, 6.0, 6.0], [4.0, 2.0, 4.0]], [[8.0, 7.0, 
8.0], [1.0, 1.0, 6.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 8.0], [2.0, 2.0, 1.0], [6.0, 8.0, 8.0]], 
[[6.0, 4.0, 2.0], [4.0, 2.0, 5.0], [6.0, 6.0, 2.0]], [[4.0, 4.0, 1.0], [7.0, 5.0, 7.0], [4.0, 5.0, 3.0]]], [[[6.0, 3.0, 3.0], 
[8.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[8.0, 7.0, 7.0], [5.0, 2.0, 6.0], [1.0, 8.0, 8.0]], [[3.0, 1.0, 3.0], [3.0, 1.0, 3.0], 
[6.0, 7.0, 4.0]]], [[[4.0, 5.0, 7.0], [5.0, 7.0, 6.0], [7.0, 6.0, 6.0]], [[8.0, 8.0, 7.0], [8.0, 1.0, 5.0], [1.0, 4.0, 3.0]], 
[[1.0, 2.0, 7.0], [6.0, 1.0, 4.0], [5.0, 1.0, 6.0]]], [[[6.0, 1.0, 3.0], [4.0, 7.0, 2.0], [2.0, 7.0, 6.0]], [[8.0, 3.0, 1.0], 
[7.0, 7.0, 1.0], [3.0, 2.0, 7.0]], [[2.0, 2.0, 4.0], [4.0, 8.0, 5.0], [2.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UCCN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 4.0], [8.0, 6.0, 5.0], [8.0, 8.0, 2.0]], [[1.0, 6.0, 8.0], [2.0, 3.0, 8.0], [1.0, 6.0, 
6.0]], [[8.0, 1.0, 1.0], [5.0, 8.0, 4.0], [6.0, 3.0, 2.0]]], [[[1.0, 7.0, 1.0], [4.0, 5.0, 3.0], [7.0, 6.0, 3.0]], [[8.0, 2.0, 
5.0], [7.0, 5.0, 4.0], [1.0, 4.0, 8.0]], [[8.0, 2.0, 5.0], [8.0, 4.0, 5.0], [7.0, 6.0, 3.0]]], [[[3.0, 1.0, 7.0], [5.0, 1.0, 
2.0], [5.0, 1.0, 2.0]], [[2.0, 2.0, 6.0], [4.0, 8.0, 4.0], [4.0, 6.0, 6.0]], [[6.0, 7.0, 8.0], [3.0, 6.0, 8.0], [2.0, 7.0, 
2.0]]], [[[6.0, 6.0, 1.0], [1.0, 4.0, 7.0], [6.0, 8.0, 3.0]], [[4.0, 7.0, 7.0], [5.0, 5.0, 4.0], [8.0, 1.0, 3.0]], [[5.0, 1.0, 
4.0], [8.0, 8.0, 2.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 7.0], [8.0, 4.0, 3.0], [2.0, 7.0, 1.0]], [[2.0, 7.0, 4.0], [2.0, 5.0, 
4.0], [1.0, 7.0, 2.0]], [[8.0, 3.0, 7.0], [1.0, 8.0, 5.0], [7.0, 8.0, 2.0]]], [[[1.0, 6.0, 5.0], [4.0, 5.0, 5.0], [6.0, 7.0, 
6.0]], [[8.0, 6.0, 3.0], [5.0, 2.0, 1.0], [5.0, 6.0, 2.0]], [[1.0, 8.0, 4.0], [8.0, 6.0, 5.0], [1.0, 1.0, 4.0]]], [[[2.0, 4.0, 
7.0], [4.0, 2.0, 1.0], [2.0, 4.0, 4.0]], [[2.0, 2.0, 8.0], [2.0, 4.0, 5.0], [6.0, 7.0, 7.0]], [[5.0, 7.0, 2.0], [5.0, 2.0, 
3.0], [7.0, 2.0, 7.0]]], [[[2.0, 6.0, 4.0], [3.0, 5.0, 1.0], [5.0, 3.0, 1.0]], [[4.0, 1.0, 4.0], [7.0, 8.0, 6.0], [7.0, 8.0, 
1.0]], [[3.0, 8.0, 1.0], [1.0, 7.0, 3.0], [5.0, 5.0, 6.0]]]]))
      val=Data(numpy.array([[[[1.0, 2.0, 2.0], [6.0, 3.0, 1.0], [1.0, 4.0, 6.0]], [[3.0, 2.0, 5.0], [6.0, 7.0, 2.0], [8.0, 2.0, 
8.0]], [[1.0, 6.0, 7.0], [1.0, 7.0, 5.0], [7.0, 8.0, 1.0]]], [[[8.0, 2.0, 1.0], [1.0, 1.0, 2.0], [8.0, 8.0, 4.0]], [[6.0, 8.0, 
6.0], [8.0, 3.0, 1.0], [4.0, 6.0, 2.0]], [[4.0, 1.0, 6.0], [4.0, 4.0, 4.0], [3.0, 8.0, 1.0]]], [[[6.0, 2.0, 8.0], [5.0, 1.0, 
2.0], [7.0, 5.0, 7.0]], [[3.0, 1.0, 4.0], [8.0, 3.0, 7.0], [1.0, 6.0, 7.0]], [[2.0, 7.0, 1.0], [2.0, 2.0, 8.0], [5.0, 7.0, 
5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 6.0], [6.0, 7.0, 6.0], [7.0, 8.0, 2.0]], [[7.0, 4.0, 8.0], [7.0, 2.0, 
1.0], [7.0, 3.0, 4.0]], [[2.0, 6.0, 8.0], [7.0, 7.0, 8.0], [6.0, 5.0, 6.0]]], [[[3.0, 1.0, 5.0], [1.0, 7.0, 2.0], [3.0, 2.0, 
5.0]], [[6.0, 1.0, 5.0], [1.0, 3.0, 1.0], [3.0, 5.0, 2.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 2.0], [1.0, 7.0, 5.0]]], [[[7.0, 5.0, 
6.0], [5.0, 3.0, 5.0], [6.0, 4.0, 3.0]], [[4.0, 3.0, 3.0], [3.0, 8.0, 5.0], [4.0, 1.0, 2.0]], [[2.0, 5.0, 5.0], [7.0, 3.0, 
8.0], [7.0, 3.0, 6.0]]]]))
      arg[:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 2.0], [6.0, 3.0, 1.0], [1.0, 4.0, 6.0]], [[3.0, 2.0, 5.0], [6.0, 7.0, 2.0], [8.0, 
2.0, 8.0]], [[1.0, 6.0, 7.0], [1.0, 7.0, 5.0], [7.0, 8.0, 1.0]]], [[[8.0, 2.0, 1.0], [1.0, 1.0, 2.0], [8.0, 8.0, 4.0]], [[6.0, 
8.0, 6.0], [8.0, 3.0, 1.0], [4.0, 6.0, 2.0]], [[4.0, 1.0, 6.0], [4.0, 4.0, 4.0], [3.0, 8.0, 1.0]]], [[[6.0, 2.0, 8.0], [5.0, 
1.0, 2.0], [7.0, 5.0, 7.0]], [[3.0, 1.0, 4.0], [8.0, 3.0, 7.0], [1.0, 6.0, 7.0]], [[2.0, 7.0, 1.0], [2.0, 2.0, 8.0], [5.0, 7.0, 
5.0]]], [[[6.0, 6.0, 1.0], [1.0, 4.0, 7.0], [6.0, 8.0, 3.0]], [[4.0, 7.0, 7.0], [5.0, 5.0, 4.0], [8.0, 1.0, 3.0]], [[5.0, 1.0, 
4.0], [8.0, 8.0, 2.0], [3.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 6.0], [6.0, 7.0, 6.0], [7.0, 8.0, 2.0]], 
[[7.0, 4.0, 8.0], [7.0, 2.0, 1.0], [7.0, 3.0, 4.0]], [[2.0, 6.0, 8.0], [7.0, 7.0, 8.0], [6.0, 5.0, 6.0]]], [[[3.0, 1.0, 5.0], 
[1.0, 7.0, 2.0], [3.0, 2.0, 5.0]], [[6.0, 1.0, 5.0], [1.0, 3.0, 1.0], [3.0, 5.0, 2.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 2.0], 
[1.0, 7.0, 5.0]]], [[[7.0, 5.0, 6.0], [5.0, 3.0, 5.0], [6.0, 4.0, 3.0]], [[4.0, 3.0, 3.0], [3.0, 8.0, 5.0], [4.0, 1.0, 2.0]], 
[[2.0, 5.0, 5.0], [7.0, 3.0, 8.0], [7.0, 3.0, 6.0]]], [[[2.0, 6.0, 4.0], [3.0, 5.0, 1.0], [5.0, 3.0, 1.0]], [[4.0, 1.0, 4.0], 
[7.0, 8.0, 6.0], [7.0, 8.0, 1.0]], [[3.0, 8.0, 1.0], [1.0, 7.0, 3.0], [5.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UCCN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 3.0], [7.0, 2.0, 3.0], [4.0, 5.0, 4.0]], [[1.0, 4.0, 6.0], [3.0, 8.0, 1.0], [3.0, 3.0, 
1.0]], [[8.0, 8.0, 3.0], [5.0, 4.0, 1.0], [6.0, 8.0, 7.0]]], [[[4.0, 2.0, 8.0], [1.0, 8.0, 7.0], [1.0, 6.0, 7.0]], [[6.0, 2.0, 
7.0], [3.0, 5.0, 6.0], [4.0, 5.0, 3.0]], [[4.0, 7.0, 4.0], [3.0, 6.0, 7.0], [8.0, 7.0, 5.0]]], [[[5.0, 2.0, 7.0], [5.0, 1.0, 
6.0], [8.0, 5.0, 6.0]], [[3.0, 1.0, 3.0], [3.0, 7.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 7.0, 8.0], [2.0, 7.0, 4.0], [4.0, 3.0, 
5.0]]], [[[2.0, 4.0, 6.0], [8.0, 3.0, 3.0], [3.0, 2.0, 3.0]], [[7.0, 8.0, 3.0], [6.0, 3.0, 1.0], [3.0, 7.0, 2.0]], [[2.0, 2.0, 
8.0], [7.0, 8.0, 3.0], [4.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 3.0], [6.0, 7.0, 3.0], [3.0, 8.0, 8.0]], [[3.0, 3.0, 1.0], [4.0, 3.0, 
1.0], [1.0, 1.0, 1.0]], [[7.0, 7.0, 1.0], [7.0, 7.0, 3.0], [2.0, 4.0, 3.0]]], [[[3.0, 8.0, 5.0], [6.0, 8.0, 8.0], [3.0, 7.0, 
3.0]], [[4.0, 3.0, 3.0], [3.0, 7.0, 7.0], [8.0, 4.0, 4.0]], [[6.0, 5.0, 4.0], [1.0, 8.0, 6.0], [2.0, 2.0, 7.0]]], [[[2.0, 4.0, 
7.0], [2.0, 4.0, 1.0], [5.0, 5.0, 3.0]], [[2.0, 1.0, 3.0], [7.0, 1.0, 3.0], [4.0, 7.0, 3.0]], [[7.0, 5.0, 1.0], [3.0, 2.0, 
3.0], [6.0, 8.0, 2.0]]], [[[5.0, 1.0, 4.0], [6.0, 8.0, 1.0], [2.0, 8.0, 1.0]], [[5.0, 5.0, 5.0], [2.0, 5.0, 7.0], [3.0, 3.0, 
8.0]], [[2.0, 2.0, 1.0], [3.0, 7.0, 8.0], [6.0, 5.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 5.0, 4.0], [8.0, 5.0, 7.0], [8.0, 2.0, 6.0]], [[4.0, 6.0, 1.0], [2.0, 6.0, 4.0], [2.0, 
1.0, 4.0]], [[5.0, 3.0, 7.0], [2.0, 7.0, 3.0], [6.0, 8.0, 2.0]]], [[[4.0, 5.0, 1.0], [8.0, 8.0, 2.0], [7.0, 8.0, 3.0]], [[2.0, 
5.0, 2.0], [3.0, 8.0, 7.0], [5.0, 7.0, 4.0]], [[6.0, 3.0, 7.0], [4.0, 1.0, 4.0], [7.0, 7.0, 7.0]]], [[[8.0, 4.0, 8.0], [6.0, 
3.0, 1.0], [7.0, 1.0, 2.0]], [[6.0, 4.0, 5.0], [2.0, 8.0, 7.0], [4.0, 7.0, 2.0]], [[6.0, 4.0, 4.0], [2.0, 8.0, 4.0], [1.0, 8.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 4.0, 3.0], [4.0, 4.0, 7.0], [4.0, 6.0, 3.0]], [[1.0, 7.0, 6.0], [2.0, 1.0, 6.0], 
[1.0, 2.0, 6.0]], [[3.0, 1.0, 2.0], [7.0, 3.0, 4.0], [5.0, 4.0, 3.0]]], [[[6.0, 7.0, 5.0], [2.0, 8.0, 5.0], [1.0, 6.0, 7.0]], 
[[3.0, 3.0, 2.0], [1.0, 1.0, 2.0], [4.0, 1.0, 4.0]], [[3.0, 6.0, 7.0], [2.0, 4.0, 4.0], [5.0, 2.0, 1.0]]], [[[4.0, 3.0, 4.0], 
[3.0, 5.0, 7.0], [8.0, 6.0, 1.0]], [[7.0, 7.0, 1.0], [1.0, 4.0, 2.0], [4.0, 1.0, 1.0]], [[1.0, 1.0, 7.0], [4.0, 3.0, 1.0], 
[2.0, 2.0, 7.0]]]])
      arg[:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 4.0], [8.0, 5.0, 7.0], [8.0, 2.0, 6.0]], [[4.0, 6.0, 1.0], [2.0, 6.0, 4.0], [2.0, 
1.0, 4.0]], [[5.0, 3.0, 7.0], [2.0, 7.0, 3.0], [6.0, 8.0, 2.0]]], [[[4.0, 5.0, 1.0], [8.0, 8.0, 2.0], [7.0, 8.0, 3.0]], [[2.0, 
5.0, 2.0], [3.0, 8.0, 7.0], [5.0, 7.0, 4.0]], [[6.0, 3.0, 7.0], [4.0, 1.0, 4.0], [7.0, 7.0, 7.0]]], [[[8.0, 4.0, 8.0], [6.0, 
3.0, 1.0], [7.0, 1.0, 2.0]], [[6.0, 4.0, 5.0], [2.0, 8.0, 7.0], [4.0, 7.0, 2.0]], [[6.0, 4.0, 4.0], [2.0, 8.0, 4.0], [1.0, 8.0, 
1.0]]], [[[2.0, 4.0, 6.0], [8.0, 3.0, 3.0], [3.0, 2.0, 3.0]], [[7.0, 8.0, 3.0], [6.0, 3.0, 1.0], [3.0, 7.0, 2.0]], [[2.0, 2.0, 
8.0], [7.0, 8.0, 3.0], [4.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 3.0], [4.0, 4.0, 7.0], [4.0, 6.0, 3.0]], 
[[1.0, 7.0, 6.0], [2.0, 1.0, 6.0], [1.0, 2.0, 6.0]], [[3.0, 1.0, 2.0], [7.0, 3.0, 4.0], [5.0, 4.0, 3.0]]], [[[6.0, 7.0, 5.0], 
[2.0, 8.0, 5.0], [1.0, 6.0, 7.0]], [[3.0, 3.0, 2.0], [1.0, 1.0, 2.0], [4.0, 1.0, 4.0]], [[3.0, 6.0, 7.0], [2.0, 4.0, 4.0], 
[5.0, 2.0, 1.0]]], [[[4.0, 3.0, 4.0], [3.0, 5.0, 7.0], [8.0, 6.0, 1.0]], [[7.0, 7.0, 1.0], [1.0, 4.0, 2.0], [4.0, 1.0, 1.0]], 
[[1.0, 1.0, 7.0], [4.0, 3.0, 1.0], [2.0, 2.0, 7.0]]], [[[5.0, 1.0, 4.0], [6.0, 8.0, 1.0], [2.0, 8.0, 1.0]], [[5.0, 5.0, 5.0], 
[2.0, 5.0, 7.0], [3.0, 3.0, 8.0]], [[2.0, 2.0, 1.0], [3.0, 7.0, 8.0], [6.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCLN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 3.0], [2.0, 8.0, 3.0], [5.0, 4.0, 6.0]], [[4.0, 8.0, 1.0], [5.0, 3.0, 4.0], [4.0, 6.0, 
3.0]], [[2.0, 8.0, 8.0], [5.0, 6.0, 3.0], [8.0, 1.0, 6.0]]], [[[2.0, 4.0, 5.0], [5.0, 5.0, 8.0], [4.0, 8.0, 6.0]], [[2.0, 3.0, 
1.0], [8.0, 6.0, 7.0], [4.0, 5.0, 5.0]], [[8.0, 4.0, 2.0], [7.0, 6.0, 3.0], [4.0, 2.0, 1.0]]], [[[4.0, 2.0, 6.0], [8.0, 5.0, 
5.0], [1.0, 1.0, 7.0]], [[6.0, 2.0, 2.0], [3.0, 8.0, 7.0], [6.0, 4.0, 6.0]], [[6.0, 5.0, 2.0], [1.0, 4.0, 7.0], [8.0, 3.0, 
5.0]]], [[[4.0, 4.0, 5.0], [5.0, 8.0, 2.0], [8.0, 3.0, 2.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 8.0], [1.0, 1.0, 8.0]], [[6.0, 4.0, 
7.0], [3.0, 1.0, 7.0], [7.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 1.0], [1.0, 5.0, 4.0], [6.0, 5.0, 7.0]], [[8.0, 4.0, 3.0], [5.0, 1.0, 
8.0], [1.0, 1.0, 4.0]], [[5.0, 6.0, 8.0], [5.0, 2.0, 2.0], [2.0, 5.0, 8.0]]], [[[7.0, 5.0, 5.0], [6.0, 2.0, 5.0], [6.0, 6.0, 
4.0]], [[6.0, 8.0, 1.0], [1.0, 2.0, 7.0], [6.0, 7.0, 1.0]], [[8.0, 6.0, 2.0], [7.0, 8.0, 1.0], [6.0, 5.0, 2.0]]], [[[4.0, 3.0, 
7.0], [2.0, 8.0, 7.0], [8.0, 8.0, 2.0]], [[2.0, 2.0, 1.0], [3.0, 3.0, 5.0], [5.0, 2.0, 1.0]], [[5.0, 7.0, 5.0], [2.0, 7.0, 
4.0], [2.0, 7.0, 6.0]]], [[[6.0, 6.0, 6.0], [5.0, 6.0, 5.0], [7.0, 6.0, 7.0]], [[3.0, 2.0, 8.0], [4.0, 5.0, 3.0], [1.0, 3.0, 
6.0]], [[6.0, 4.0, 1.0], [5.0, 5.0, 1.0], [6.0, 1.0, 8.0]]]]))
      val=numpy.array([[[[2.0, 7.0, 6.0]], [[7.0, 3.0, 5.0]], [[3.0, 3.0, 3.0]]], [[[2.0, 3.0, 6.0]], [[1.0, 4.0, 8.0]], [[4.0, 
8.0, 3.0]]], [[[3.0, 2.0, 5.0]], [[5.0, 3.0, 1.0]], [[2.0, 8.0, 7.0]]], [[[3.0, 6.0, 3.0]], [[3.0, 5.0, 1.0]], [[1.0, 2.0, 
5.0]]]])
      arg[:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 3.0], [2.0, 8.0, 3.0], [2.0, 7.0, 6.0]], [[4.0, 8.0, 1.0], [5.0, 3.0, 4.0], [7.0, 
3.0, 5.0]], [[2.0, 8.0, 8.0], [5.0, 6.0, 3.0], [3.0, 3.0, 3.0]]], [[[2.0, 4.0, 5.0], [5.0, 5.0, 8.0], [2.0, 3.0, 6.0]], [[2.0, 
3.0, 1.0], [8.0, 6.0, 7.0], [1.0, 4.0, 8.0]], [[8.0, 4.0, 2.0], [7.0, 6.0, 3.0], [4.0, 8.0, 3.0]]], [[[4.0, 2.0, 6.0], [8.0, 
5.0, 5.0], [3.0, 2.0, 5.0]], [[6.0, 2.0, 2.0], [3.0, 8.0, 7.0], [5.0, 3.0, 1.0]], [[6.0, 5.0, 2.0], [1.0, 4.0, 7.0], [2.0, 8.0, 
7.0]]], [[[4.0, 4.0, 5.0], [5.0, 8.0, 2.0], [3.0, 6.0, 3.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 8.0], [3.0, 5.0, 1.0]], [[6.0, 4.0, 
7.0], [3.0, 1.0, 7.0], [1.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 1.0], [1.0, 5.0, 4.0], [2.0, 7.0, 6.0]], 
[[8.0, 4.0, 3.0], [5.0, 1.0, 8.0], [7.0, 3.0, 5.0]], [[5.0, 6.0, 8.0], [5.0, 2.0, 2.0], [3.0, 3.0, 3.0]]], [[[7.0, 5.0, 5.0], 
[6.0, 2.0, 5.0], [2.0, 3.0, 6.0]], [[6.0, 8.0, 1.0], [1.0, 2.0, 7.0], [1.0, 4.0, 8.0]], [[8.0, 6.0, 2.0], [7.0, 8.0, 1.0], 
[4.0, 8.0, 3.0]]], [[[4.0, 3.0, 7.0], [2.0, 8.0, 7.0], [3.0, 2.0, 5.0]], [[2.0, 2.0, 1.0], [3.0, 3.0, 5.0], [5.0, 3.0, 1.0]], 
[[5.0, 7.0, 5.0], [2.0, 7.0, 4.0], [2.0, 8.0, 7.0]]], [[[6.0, 6.0, 6.0], [5.0, 6.0, 5.0], [3.0, 6.0, 3.0]], [[3.0, 2.0, 8.0], 
[4.0, 5.0, 3.0], [3.0, 5.0, 1.0]], [[6.0, 4.0, 1.0], [5.0, 5.0, 1.0], [1.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCLN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 3.0], [7.0, 8.0, 4.0], [2.0, 3.0, 4.0]], [[5.0, 1.0, 1.0], [2.0, 8.0, 2.0], [2.0, 5.0, 
6.0]], [[1.0, 2.0, 1.0], [4.0, 5.0, 6.0], [1.0, 4.0, 2.0]]], [[[5.0, 6.0, 1.0], [5.0, 8.0, 6.0], [6.0, 7.0, 1.0]], [[3.0, 7.0, 
8.0], [1.0, 5.0, 3.0], [1.0, 5.0, 1.0]], [[7.0, 4.0, 6.0], [4.0, 5.0, 2.0], [1.0, 8.0, 3.0]]], [[[6.0, 5.0, 6.0], [5.0, 4.0, 
2.0], [2.0, 1.0, 2.0]], [[6.0, 7.0, 8.0], [1.0, 6.0, 6.0], [5.0, 5.0, 1.0]], [[5.0, 5.0, 8.0], [7.0, 1.0, 8.0], [2.0, 2.0, 
5.0]]], [[[2.0, 1.0, 2.0], [2.0, 1.0, 7.0], [7.0, 8.0, 7.0]], [[6.0, 8.0, 6.0], [1.0, 2.0, 8.0], [4.0, 6.0, 1.0]], [[7.0, 6.0, 
7.0], [8.0, 7.0, 5.0], [6.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 3.0], [3.0, 8.0, 4.0], [1.0, 1.0, 8.0]], [[8.0, 7.0, 5.0], [8.0, 7.0, 
2.0], [5.0, 7.0, 6.0]], [[3.0, 7.0, 6.0], [5.0, 8.0, 3.0], [3.0, 6.0, 2.0]]], [[[3.0, 6.0, 1.0], [7.0, 7.0, 6.0], [6.0, 1.0, 
7.0]], [[2.0, 1.0, 1.0], [6.0, 5.0, 6.0], [8.0, 7.0, 6.0]], [[1.0, 7.0, 8.0], [2.0, 3.0, 2.0], [3.0, 2.0, 8.0]]], [[[5.0, 8.0, 
2.0], [2.0, 8.0, 6.0], [4.0, 8.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 3.0, 7.0], [7.0, 4.0, 1.0]], [[1.0, 2.0, 4.0], [8.0, 3.0, 
4.0], [1.0, 4.0, 7.0]]], [[[5.0, 8.0, 2.0], [2.0, 2.0, 6.0], [8.0, 6.0, 5.0]], [[4.0, 1.0, 5.0], [4.0, 2.0, 6.0], [4.0, 5.0, 
7.0]], [[8.0, 1.0, 6.0], [3.0, 3.0, 3.0], [4.0, 2.0, 6.0]]]]))
      val=Data(numpy.array([[[[5.0, 5.0, 1.0]], [[3.0, 4.0, 8.0]], [[2.0, 1.0, 1.0]]], [[[5.0, 3.0, 3.0]], [[2.0, 3.0, 8.0]], 
[[6.0, 3.0, 2.0]]], [[[7.0, 2.0, 6.0]], [[2.0, 8.0, 4.0]], [[8.0, 6.0, 6.0]]], [[[2.0, 8.0, 6.0]], [[8.0, 8.0, 2.0]], [[4.0, 
6.0, 3.0]]]]),self.functionspace)
      arg[:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 3.0], [7.0, 8.0, 4.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 1.0], [2.0, 8.0, 2.0], [3.0, 
4.0, 8.0]], [[1.0, 2.0, 1.0], [4.0, 5.0, 6.0], [2.0, 1.0, 1.0]]], [[[5.0, 6.0, 1.0], [5.0, 8.0, 6.0], [5.0, 3.0, 3.0]], [[3.0, 
7.0, 8.0], [1.0, 5.0, 3.0], [2.0, 3.0, 8.0]], [[7.0, 4.0, 6.0], [4.0, 5.0, 2.0], [6.0, 3.0, 2.0]]], [[[6.0, 5.0, 6.0], [5.0, 
4.0, 2.0], [7.0, 2.0, 6.0]], [[6.0, 7.0, 8.0], [1.0, 6.0, 6.0], [2.0, 8.0, 4.0]], [[5.0, 5.0, 8.0], [7.0, 1.0, 8.0], [8.0, 6.0, 
6.0]]], [[[2.0, 1.0, 2.0], [2.0, 1.0, 7.0], [2.0, 8.0, 6.0]], [[6.0, 8.0, 6.0], [1.0, 2.0, 8.0], [8.0, 8.0, 2.0]], [[7.0, 6.0, 
7.0], [8.0, 7.0, 5.0], [4.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 3.0], [3.0, 8.0, 4.0], [5.0, 5.0, 1.0]], 
[[8.0, 7.0, 5.0], [8.0, 7.0, 2.0], [3.0, 4.0, 8.0]], [[3.0, 7.0, 6.0], [5.0, 8.0, 3.0], [2.0, 1.0, 1.0]]], [[[3.0, 6.0, 1.0], 
[7.0, 7.0, 6.0], [5.0, 3.0, 3.0]], [[2.0, 1.0, 1.0], [6.0, 5.0, 6.0], [2.0, 3.0, 8.0]], [[1.0, 7.0, 8.0], [2.0, 3.0, 2.0], 
[6.0, 3.0, 2.0]]], [[[5.0, 8.0, 2.0], [2.0, 8.0, 6.0], [7.0, 2.0, 6.0]], [[6.0, 3.0, 7.0], [8.0, 3.0, 7.0], [2.0, 8.0, 4.0]], 
[[1.0, 2.0, 4.0], [8.0, 3.0, 4.0], [8.0, 6.0, 6.0]]], [[[5.0, 8.0, 2.0], [2.0, 2.0, 6.0], [2.0, 8.0, 6.0]], [[4.0, 1.0, 5.0], 
[4.0, 2.0, 6.0], [8.0, 8.0, 2.0]], [[8.0, 1.0, 6.0], [3.0, 3.0, 3.0], [4.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCLN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 4.0], [5.0, 4.0, 2.0], [2.0, 4.0, 7.0]], [[4.0, 6.0, 5.0], [3.0, 3.0, 8.0], [8.0, 5.0, 
1.0]], [[5.0, 3.0, 3.0], [2.0, 5.0, 5.0], [8.0, 7.0, 2.0]]], [[[3.0, 4.0, 4.0], [6.0, 7.0, 1.0], [3.0, 8.0, 3.0]], [[6.0, 6.0, 
7.0], [8.0, 7.0, 8.0], [6.0, 3.0, 5.0]], [[5.0, 8.0, 4.0], [4.0, 8.0, 1.0], [4.0, 8.0, 1.0]]], [[[4.0, 8.0, 8.0], [4.0, 4.0, 
2.0], [3.0, 8.0, 7.0]], [[5.0, 8.0, 5.0], [4.0, 5.0, 6.0], [3.0, 7.0, 5.0]], [[5.0, 6.0, 7.0], [3.0, 5.0, 6.0], [3.0, 4.0, 
3.0]]], [[[8.0, 7.0, 6.0], [7.0, 7.0, 5.0], [1.0, 7.0, 8.0]], [[4.0, 7.0, 3.0], [4.0, 3.0, 6.0], [8.0, 6.0, 4.0]], [[7.0, 7.0, 
5.0], [1.0, 7.0, 6.0], [3.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 8.0], [1.0, 7.0, 7.0], [7.0, 6.0, 3.0]], [[4.0, 5.0, 1.0], [2.0, 8.0, 
5.0], [2.0, 4.0, 8.0]], [[6.0, 3.0, 3.0], [7.0, 1.0, 2.0], [6.0, 6.0, 6.0]]], [[[7.0, 5.0, 2.0], [5.0, 4.0, 2.0], [1.0, 1.0, 
1.0]], [[8.0, 4.0, 5.0], [3.0, 4.0, 1.0], [5.0, 2.0, 4.0]], [[4.0, 6.0, 3.0], [5.0, 3.0, 3.0], [6.0, 8.0, 2.0]]], [[[2.0, 2.0, 
3.0], [1.0, 5.0, 8.0], [8.0, 2.0, 5.0]], [[7.0, 6.0, 6.0], [2.0, 5.0, 7.0], [8.0, 4.0, 1.0]], [[3.0, 2.0, 1.0], [8.0, 4.0, 
2.0], [5.0, 1.0, 6.0]]], [[[4.0, 8.0, 3.0], [8.0, 2.0, 8.0], [8.0, 6.0, 1.0]], [[1.0, 4.0, 1.0], [1.0, 5.0, 5.0], [7.0, 2.0, 
6.0]], [[6.0, 1.0, 1.0], [4.0, 1.0, 2.0], [2.0, 6.0, 3.0]]]]))
      val=Data(numpy.array([[[[5.0, 6.0, 5.0]], [[8.0, 4.0, 4.0]], [[3.0, 1.0, 8.0]]], [[[2.0, 2.0, 2.0]], [[8.0, 4.0, 4.0]], 
[[4.0, 1.0, 1.0]]], [[[6.0, 4.0, 5.0]], [[6.0, 4.0, 1.0]], [[2.0, 8.0, 8.0]]], [[[4.0, 4.0, 5.0]], [[5.0, 7.0, 2.0]], [[5.0, 
1.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 6.0]], [[5.0, 2.0, 8.0]], [[6.0, 8.0, 6.0]]], [[[3.0, 3.0, 6.0]], [[6.0, 
1.0, 6.0]], [[5.0, 8.0, 8.0]]], [[[4.0, 3.0, 3.0]], [[2.0, 2.0, 4.0]], [[3.0, 1.0, 1.0]]], [[[6.0, 8.0, 5.0]], [[6.0, 1.0, 
6.0]], [[2.0, 7.0, 4.0]]]]))
      arg[:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 4.0], [5.0, 4.0, 2.0], [5.0, 6.0, 5.0]], [[4.0, 6.0, 5.0], [3.0, 3.0, 8.0], [8.0, 
4.0, 4.0]], [[5.0, 3.0, 3.0], [2.0, 5.0, 5.0], [3.0, 1.0, 8.0]]], [[[3.0, 4.0, 4.0], [6.0, 7.0, 1.0], [2.0, 2.0, 2.0]], [[6.0, 
6.0, 7.0], [8.0, 7.0, 8.0], [8.0, 4.0, 4.0]], [[5.0, 8.0, 4.0], [4.0, 8.0, 1.0], [4.0, 1.0, 1.0]]], [[[4.0, 8.0, 8.0], [4.0, 
4.0, 2.0], [6.0, 4.0, 5.0]], [[5.0, 8.0, 5.0], [4.0, 5.0, 6.0], [6.0, 4.0, 1.0]], [[5.0, 6.0, 7.0], [3.0, 5.0, 6.0], [2.0, 8.0, 
8.0]]], [[[8.0, 7.0, 6.0], [7.0, 7.0, 5.0], [4.0, 4.0, 5.0]], [[4.0, 7.0, 3.0], [4.0, 3.0, 6.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 
5.0], [1.0, 7.0, 6.0], [5.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 8.0], [1.0, 7.0, 7.0], [6.0, 3.0, 6.0]], 
[[4.0, 5.0, 1.0], [2.0, 8.0, 5.0], [5.0, 2.0, 8.0]], [[6.0, 3.0, 3.0], [7.0, 1.0, 2.0], [6.0, 8.0, 6.0]]], [[[7.0, 5.0, 2.0], 
[5.0, 4.0, 2.0], [3.0, 3.0, 6.0]], [[8.0, 4.0, 5.0], [3.0, 4.0, 1.0], [6.0, 1.0, 6.0]], [[4.0, 6.0, 3.0], [5.0, 3.0, 3.0], 
[5.0, 8.0, 8.0]]], [[[2.0, 2.0, 3.0], [1.0, 5.0, 8.0], [4.0, 3.0, 3.0]], [[7.0, 6.0, 6.0], [2.0, 5.0, 7.0], [2.0, 2.0, 4.0]], 
[[3.0, 2.0, 1.0], [8.0, 4.0, 2.0], [3.0, 1.0, 1.0]]], [[[4.0, 8.0, 3.0], [8.0, 2.0, 8.0], [6.0, 8.0, 5.0]], [[1.0, 4.0, 1.0], 
[1.0, 5.0, 5.0], [6.0, 1.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 1.0, 2.0], [2.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCLN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 2.0], [8.0, 4.0, 8.0], [8.0, 6.0, 4.0]], [[7.0, 5.0, 1.0], [4.0, 6.0, 3.0], [4.0, 8.0, 
6.0]], [[1.0, 3.0, 6.0], [8.0, 3.0, 3.0], [3.0, 2.0, 7.0]]], [[[8.0, 5.0, 3.0], [3.0, 2.0, 4.0], [8.0, 8.0, 7.0]], [[4.0, 7.0, 
5.0], [6.0, 4.0, 3.0], [6.0, 2.0, 7.0]], [[3.0, 6.0, 8.0], [2.0, 1.0, 3.0], [6.0, 2.0, 1.0]]], [[[7.0, 7.0, 7.0], [7.0, 7.0, 
4.0], [8.0, 6.0, 7.0]], [[2.0, 5.0, 4.0], [6.0, 4.0, 8.0], [4.0, 1.0, 6.0]], [[5.0, 2.0, 3.0], [6.0, 3.0, 2.0], [1.0, 1.0, 
7.0]]], [[[8.0, 6.0, 3.0], [3.0, 4.0, 3.0], [2.0, 6.0, 7.0]], [[8.0, 5.0, 3.0], [3.0, 2.0, 3.0], [2.0, 1.0, 6.0]], [[6.0, 3.0, 
4.0], [4.0, 7.0, 7.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 2.0], [1.0, 4.0, 2.0], [8.0, 6.0, 6.0]], [[6.0, 4.0, 2.0], [1.0, 1.0, 
3.0], [6.0, 2.0, 8.0]], [[2.0, 5.0, 6.0], [6.0, 1.0, 4.0], [5.0, 7.0, 3.0]]], [[[5.0, 1.0, 4.0], [7.0, 1.0, 7.0], [2.0, 4.0, 
7.0]], [[1.0, 3.0, 3.0], [3.0, 4.0, 1.0], [7.0, 6.0, 4.0]], [[2.0, 5.0, 3.0], [5.0, 5.0, 2.0], [1.0, 6.0, 3.0]]], [[[7.0, 1.0, 
1.0], [1.0, 7.0, 6.0], [5.0, 3.0, 1.0]], [[5.0, 2.0, 6.0], [8.0, 8.0, 7.0], [2.0, 4.0, 4.0]], [[7.0, 4.0, 4.0], [4.0, 5.0, 
7.0], [4.0, 3.0, 3.0]]], [[[7.0, 2.0, 5.0], [5.0, 7.0, 8.0], [3.0, 2.0, 5.0]], [[4.0, 1.0, 1.0], [3.0, 6.0, 5.0], [4.0, 4.0, 
6.0]], [[7.0, 7.0, 6.0], [4.0, 1.0, 8.0], [2.0, 3.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0, 3.0, 7.0]], [[2.0, 2.0, 5.0]], [[4.0, 3.0, 8.0]]], [[[7.0, 6.0, 1.0]], [[8.0, 5.0, 
7.0]], [[1.0, 5.0, 3.0]]], [[[2.0, 1.0, 3.0]], [[8.0, 7.0, 7.0]], [[8.0, 3.0, 8.0]]], [[[7.0, 1.0, 3.0]], [[2.0, 3.0, 1.0]], 
[[3.0, 1.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 2.0, 7.0]], [[3.0, 3.0, 6.0]], [[8.0, 2.0, 2.0]]], [[[6.0, 1.0, 4.0]], 
[[4.0, 6.0, 4.0]], [[6.0, 5.0, 1.0]]], [[[8.0, 8.0, 2.0]], [[6.0, 1.0, 5.0]], [[1.0, 3.0, 3.0]]], [[[2.0, 3.0, 4.0]], [[2.0, 
8.0, 8.0]], [[5.0, 6.0, 1.0]]]])
      arg[:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 2.0], [8.0, 4.0, 8.0], [6.0, 3.0, 7.0]], [[7.0, 5.0, 1.0], [4.0, 6.0, 3.0], [2.0, 
2.0, 5.0]], [[1.0, 3.0, 6.0], [8.0, 3.0, 3.0], [4.0, 3.0, 8.0]]], [[[8.0, 5.0, 3.0], [3.0, 2.0, 4.0], [7.0, 6.0, 1.0]], [[4.0, 
7.0, 5.0], [6.0, 4.0, 3.0], [8.0, 5.0, 7.0]], [[3.0, 6.0, 8.0], [2.0, 1.0, 3.0], [1.0, 5.0, 3.0]]], [[[7.0, 7.0, 7.0], [7.0, 
7.0, 4.0], [2.0, 1.0, 3.0]], [[2.0, 5.0, 4.0], [6.0, 4.0, 8.0], [8.0, 7.0, 7.0]], [[5.0, 2.0, 3.0], [6.0, 3.0, 2.0], [8.0, 3.0, 
8.0]]], [[[8.0, 6.0, 3.0], [3.0, 4.0, 3.0], [7.0, 1.0, 3.0]], [[8.0, 5.0, 3.0], [3.0, 2.0, 3.0], [2.0, 3.0, 1.0]], [[6.0, 3.0, 
4.0], [4.0, 7.0, 7.0], [3.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 2.0], [1.0, 4.0, 2.0], [2.0, 2.0, 7.0]], 
[[6.0, 4.0, 2.0], [1.0, 1.0, 3.0], [3.0, 3.0, 6.0]], [[2.0, 5.0, 6.0], [6.0, 1.0, 4.0], [8.0, 2.0, 2.0]]], [[[5.0, 1.0, 4.0], 
[7.0, 1.0, 7.0], [6.0, 1.0, 4.0]], [[1.0, 3.0, 3.0], [3.0, 4.0, 1.0], [4.0, 6.0, 4.0]], [[2.0, 5.0, 3.0], [5.0, 5.0, 2.0], 
[6.0, 5.0, 1.0]]], [[[7.0, 1.0, 1.0], [1.0, 7.0, 6.0], [8.0, 8.0, 2.0]], [[5.0, 2.0, 6.0], [8.0, 8.0, 7.0], [6.0, 1.0, 5.0]], 
[[7.0, 4.0, 4.0], [4.0, 5.0, 7.0], [1.0, 3.0, 3.0]]], [[[7.0, 2.0, 5.0], [5.0, 7.0, 8.0], [2.0, 3.0, 4.0]], [[4.0, 1.0, 1.0], 
[3.0, 6.0, 5.0], [2.0, 8.0, 8.0]], [[7.0, 7.0, 6.0], [4.0, 1.0, 8.0], [5.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LPRN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 1.0], [1.0, 4.0, 1.0], [8.0, 2.0, 3.0]], [[6.0, 4.0, 8.0], [3.0, 6.0, 4.0], [6.0, 1.0, 
6.0]], [[8.0, 1.0, 5.0], [5.0, 3.0, 1.0], [2.0, 2.0, 7.0]]], [[[4.0, 6.0, 1.0], [3.0, 1.0, 7.0], [3.0, 7.0, 7.0]], [[4.0, 3.0, 
4.0], [4.0, 5.0, 8.0], [7.0, 6.0, 8.0]], [[5.0, 8.0, 2.0], [7.0, 1.0, 5.0], [1.0, 2.0, 7.0]]], [[[4.0, 2.0, 7.0], [7.0, 6.0, 
6.0], [2.0, 4.0, 4.0]], [[4.0, 4.0, 2.0], [6.0, 1.0, 6.0], [3.0, 4.0, 8.0]], [[1.0, 4.0, 8.0], [7.0, 1.0, 8.0], [4.0, 1.0, 
6.0]]], [[[2.0, 4.0, 8.0], [2.0, 5.0, 4.0], [7.0, 1.0, 8.0]], [[8.0, 3.0, 5.0], [4.0, 2.0, 7.0], [1.0, 7.0, 1.0]], [[5.0, 7.0, 
6.0], [7.0, 7.0, 6.0], [1.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 1.0], [2.0, 2.0, 1.0], [7.0, 6.0, 1.0]], [[1.0, 7.0, 4.0], [7.0, 1.0, 
8.0], [4.0, 5.0, 5.0]], [[8.0, 4.0, 7.0], [2.0, 6.0, 5.0], [8.0, 6.0, 2.0]]], [[[5.0, 3.0, 1.0], [1.0, 5.0, 3.0], [3.0, 4.0, 
7.0]], [[1.0, 4.0, 6.0], [4.0, 5.0, 2.0], [7.0, 5.0, 6.0]], [[5.0, 4.0, 2.0], [1.0, 8.0, 5.0], [6.0, 3.0, 2.0]]], [[[5.0, 3.0, 
2.0], [7.0, 4.0, 5.0], [8.0, 4.0, 3.0]], [[7.0, 3.0, 3.0], [1.0, 3.0, 4.0], [2.0, 2.0, 8.0]], [[7.0, 2.0, 8.0], [4.0, 5.0, 
1.0], [7.0, 4.0, 3.0]]], [[[1.0, 4.0, 6.0], [6.0, 2.0, 2.0], [3.0, 5.0, 1.0]], [[4.0, 6.0, 8.0], [3.0, 7.0, 1.0], [5.0, 2.0, 
5.0]], [[2.0, 2.0, 8.0], [3.0, 2.0, 8.0], [3.0, 5.0, 1.0]]]]))
      val=numpy.array([[[8.0, 2.0, 8.0]], [[5.0, 2.0, 6.0]]])
      arg[2:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 1.0], [1.0, 4.0, 1.0], [8.0, 2.0, 3.0]], [[6.0, 4.0, 8.0], [3.0, 6.0, 4.0], [6.0, 
1.0, 6.0]], [[8.0, 1.0, 5.0], [5.0, 3.0, 1.0], [2.0, 2.0, 7.0]]], [[[4.0, 6.0, 1.0], [3.0, 1.0, 7.0], [3.0, 7.0, 7.0]], [[4.0, 
3.0, 4.0], [4.0, 5.0, 8.0], [7.0, 6.0, 8.0]], [[5.0, 8.0, 2.0], [7.0, 1.0, 5.0], [1.0, 2.0, 7.0]]], [[[4.0, 2.0, 7.0], [7.0, 
6.0, 6.0], [2.0, 4.0, 4.0]], [[4.0, 4.0, 2.0], [8.0, 2.0, 8.0], [3.0, 4.0, 8.0]], [[1.0, 4.0, 8.0], [7.0, 1.0, 8.0], [4.0, 1.0, 
6.0]]], [[[2.0, 4.0, 8.0], [2.0, 5.0, 4.0], [7.0, 1.0, 8.0]], [[8.0, 3.0, 5.0], [5.0, 2.0, 6.0], [1.0, 7.0, 1.0]], [[5.0, 7.0, 
6.0], [7.0, 7.0, 6.0], [1.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 1.0], [2.0, 2.0, 1.0], [7.0, 6.0, 1.0]], 
[[1.0, 7.0, 4.0], [7.0, 1.0, 8.0], [4.0, 5.0, 5.0]], [[8.0, 4.0, 7.0], [2.0, 6.0, 5.0], [8.0, 6.0, 2.0]]], [[[5.0, 3.0, 1.0], 
[1.0, 5.0, 3.0], [3.0, 4.0, 7.0]], [[1.0, 4.0, 6.0], [4.0, 5.0, 2.0], [7.0, 5.0, 6.0]], [[5.0, 4.0, 2.0], [1.0, 8.0, 5.0], 
[6.0, 3.0, 2.0]]], [[[5.0, 3.0, 2.0], [7.0, 4.0, 5.0], [8.0, 4.0, 3.0]], [[7.0, 3.0, 3.0], [8.0, 2.0, 8.0], [2.0, 2.0, 8.0]], 
[[7.0, 2.0, 8.0], [4.0, 5.0, 1.0], [7.0, 4.0, 3.0]]], [[[1.0, 4.0, 6.0], [6.0, 2.0, 2.0], [3.0, 5.0, 1.0]], [[4.0, 6.0, 8.0], 
[5.0, 2.0, 6.0], [5.0, 2.0, 5.0]], [[2.0, 2.0, 8.0], [3.0, 2.0, 8.0], [3.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LPRN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 6.0], [3.0, 3.0, 3.0], [1.0, 1.0, 5.0]], [[8.0, 4.0, 2.0], [2.0, 1.0, 5.0], [4.0, 6.0, 
1.0]], [[8.0, 4.0, 8.0], [2.0, 3.0, 7.0], [6.0, 2.0, 4.0]]], [[[7.0, 4.0, 3.0], [4.0, 6.0, 7.0], [6.0, 7.0, 3.0]], [[3.0, 7.0, 
5.0], [2.0, 7.0, 1.0], [4.0, 1.0, 2.0]], [[4.0, 1.0, 2.0], [7.0, 2.0, 4.0], [4.0, 1.0, 2.0]]], [[[3.0, 5.0, 6.0], [5.0, 4.0, 
8.0], [7.0, 2.0, 5.0]], [[8.0, 4.0, 5.0], [5.0, 3.0, 4.0], [6.0, 3.0, 6.0]], [[4.0, 1.0, 2.0], [2.0, 8.0, 1.0], [8.0, 2.0, 
2.0]]], [[[6.0, 4.0, 7.0], [3.0, 5.0, 5.0], [2.0, 1.0, 8.0]], [[5.0, 2.0, 2.0], [7.0, 1.0, 5.0], [7.0, 6.0, 5.0]], [[2.0, 2.0, 
6.0], [4.0, 5.0, 7.0], [7.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 7.0], [8.0, 8.0, 5.0], [8.0, 3.0, 6.0]], [[6.0, 5.0, 6.0], [8.0, 8.0, 
3.0], [8.0, 1.0, 8.0]], [[2.0, 3.0, 1.0], [2.0, 1.0, 8.0], [7.0, 1.0, 3.0]]], [[[2.0, 1.0, 8.0], [3.0, 4.0, 5.0], [3.0, 4.0, 
2.0]], [[1.0, 3.0, 5.0], [2.0, 5.0, 7.0], [5.0, 7.0, 8.0]], [[2.0, 1.0, 4.0], [6.0, 2.0, 8.0], [7.0, 5.0, 8.0]]], [[[6.0, 4.0, 
3.0], [8.0, 8.0, 5.0], [4.0, 5.0, 1.0]], [[4.0, 1.0, 2.0], [7.0, 5.0, 8.0], [3.0, 2.0, 4.0]], [[1.0, 1.0, 6.0], [7.0, 2.0, 
1.0], [8.0, 4.0, 3.0]]], [[[2.0, 7.0, 4.0], [5.0, 7.0, 5.0], [2.0, 3.0, 7.0]], [[5.0, 8.0, 3.0], [6.0, 3.0, 6.0], [8.0, 5.0, 
7.0]], [[5.0, 1.0, 1.0], [1.0, 2.0, 5.0], [8.0, 4.0, 7.0]]]]))
      val=Data(numpy.array([[[5.0, 1.0, 4.0]], [[4.0, 1.0, 3.0]]]),self.functionspace)
      arg[2:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [3.0, 3.0, 3.0], [1.0, 1.0, 5.0]], [[8.0, 4.0, 2.0], [2.0, 1.0, 5.0], [4.0, 
6.0, 1.0]], [[8.0, 4.0, 8.0], [2.0, 3.0, 7.0], [6.0, 2.0, 4.0]]], [[[7.0, 4.0, 3.0], [4.0, 6.0, 7.0], [6.0, 7.0, 3.0]], [[3.0, 
7.0, 5.0], [2.0, 7.0, 1.0], [4.0, 1.0, 2.0]], [[4.0, 1.0, 2.0], [7.0, 2.0, 4.0], [4.0, 1.0, 2.0]]], [[[3.0, 5.0, 6.0], [5.0, 
4.0, 8.0], [7.0, 2.0, 5.0]], [[8.0, 4.0, 5.0], [5.0, 1.0, 4.0], [6.0, 3.0, 6.0]], [[4.0, 1.0, 2.0], [2.0, 8.0, 1.0], [8.0, 2.0, 
2.0]]], [[[6.0, 4.0, 7.0], [3.0, 5.0, 5.0], [2.0, 1.0, 8.0]], [[5.0, 2.0, 2.0], [4.0, 1.0, 3.0], [7.0, 6.0, 5.0]], [[2.0, 2.0, 
6.0], [4.0, 5.0, 7.0], [7.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 7.0], [8.0, 8.0, 5.0], [8.0, 3.0, 6.0]], 
[[6.0, 5.0, 6.0], [8.0, 8.0, 3.0], [8.0, 1.0, 8.0]], [[2.0, 3.0, 1.0], [2.0, 1.0, 8.0], [7.0, 1.0, 3.0]]], [[[2.0, 1.0, 8.0], 
[3.0, 4.0, 5.0], [3.0, 4.0, 2.0]], [[1.0, 3.0, 5.0], [2.0, 5.0, 7.0], [5.0, 7.0, 8.0]], [[2.0, 1.0, 4.0], [6.0, 2.0, 8.0], 
[7.0, 5.0, 8.0]]], [[[6.0, 4.0, 3.0], [8.0, 8.0, 5.0], [4.0, 5.0, 1.0]], [[4.0, 1.0, 2.0], [5.0, 1.0, 4.0], [3.0, 2.0, 4.0]], 
[[1.0, 1.0, 6.0], [7.0, 2.0, 1.0], [8.0, 4.0, 3.0]]], [[[2.0, 7.0, 4.0], [5.0, 7.0, 5.0], [2.0, 3.0, 7.0]], [[5.0, 8.0, 3.0], 
[4.0, 1.0, 3.0], [8.0, 5.0, 7.0]], [[5.0, 1.0, 1.0], [1.0, 2.0, 5.0], [8.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LPRN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 4.0], [1.0, 4.0, 1.0], [6.0, 2.0, 5.0]], [[6.0, 8.0, 2.0], [6.0, 2.0, 1.0], [3.0, 4.0, 
4.0]], [[7.0, 2.0, 1.0], [5.0, 5.0, 1.0], [5.0, 3.0, 3.0]]], [[[7.0, 3.0, 7.0], [5.0, 3.0, 8.0], [5.0, 8.0, 2.0]], [[2.0, 3.0, 
6.0], [3.0, 8.0, 3.0], [5.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], [7.0, 5.0, 5.0], [8.0, 3.0, 3.0]]], [[[3.0, 6.0, 4.0], [6.0, 1.0, 
7.0], [7.0, 5.0, 4.0]], [[5.0, 6.0, 1.0], [2.0, 5.0, 8.0], [4.0, 1.0, 7.0]], [[6.0, 4.0, 5.0], [5.0, 3.0, 3.0], [5.0, 4.0, 
5.0]]], [[[8.0, 7.0, 7.0], [8.0, 1.0, 8.0], [3.0, 4.0, 7.0]], [[4.0, 3.0, 3.0], [2.0, 3.0, 7.0], [6.0, 4.0, 1.0]], [[1.0, 2.0, 
7.0], [4.0, 4.0, 2.0], [8.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 6.0], [7.0, 8.0, 6.0], [3.0, 4.0, 5.0]], [[5.0, 6.0, 5.0], [3.0, 6.0, 
4.0], [5.0, 4.0, 4.0]], [[2.0, 3.0, 1.0], [1.0, 7.0, 2.0], [5.0, 5.0, 8.0]]], [[[4.0, 3.0, 7.0], [6.0, 1.0, 7.0], [8.0, 8.0, 
4.0]], [[4.0, 2.0, 1.0], [3.0, 4.0, 8.0], [2.0, 5.0, 6.0]], [[5.0, 7.0, 3.0], [3.0, 1.0, 3.0], [4.0, 7.0, 5.0]]], [[[7.0, 8.0, 
4.0], [1.0, 6.0, 5.0], [2.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [6.0, 2.0, 4.0], [1.0, 8.0, 7.0]], [[6.0, 8.0, 3.0], [1.0, 2.0, 
7.0], [1.0, 8.0, 4.0]]], [[[8.0, 8.0, 2.0], [8.0, 4.0, 6.0], [2.0, 1.0, 2.0]], [[7.0, 3.0, 3.0], [2.0, 5.0, 4.0], [7.0, 5.0, 
8.0]], [[3.0, 6.0, 1.0], [3.0, 8.0, 2.0], [7.0, 7.0, 7.0]]]]))
      val=Data(numpy.array([[[5.0, 5.0, 7.0]], [[3.0, 7.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 7.0, 7.0]], [[3.0, 7.0, 5.0]]]))
      arg[2:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 4.0], [1.0, 4.0, 1.0], [6.0, 2.0, 5.0]], [[6.0, 8.0, 2.0], [6.0, 2.0, 1.0], [3.0, 
4.0, 4.0]], [[7.0, 2.0, 1.0], [5.0, 5.0, 1.0], [5.0, 3.0, 3.0]]], [[[7.0, 3.0, 7.0], [5.0, 3.0, 8.0], [5.0, 8.0, 2.0]], [[2.0, 
3.0, 6.0], [3.0, 8.0, 3.0], [5.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], [7.0, 5.0, 5.0], [8.0, 3.0, 3.0]]], [[[3.0, 6.0, 4.0], [6.0, 
1.0, 7.0], [7.0, 5.0, 4.0]], [[5.0, 6.0, 1.0], [5.0, 5.0, 7.0], [4.0, 1.0, 7.0]], [[6.0, 4.0, 5.0], [5.0, 3.0, 3.0], [5.0, 4.0, 
5.0]]], [[[8.0, 7.0, 7.0], [8.0, 1.0, 8.0], [3.0, 4.0, 7.0]], [[4.0, 3.0, 3.0], [3.0, 7.0, 5.0], [6.0, 4.0, 1.0]], [[1.0, 2.0, 
7.0], [4.0, 4.0, 2.0], [8.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 6.0], [7.0, 8.0, 6.0], [3.0, 4.0, 5.0]], 
[[5.0, 6.0, 5.0], [3.0, 6.0, 4.0], [5.0, 4.0, 4.0]], [[2.0, 3.0, 1.0], [1.0, 7.0, 2.0], [5.0, 5.0, 8.0]]], [[[4.0, 3.0, 7.0], 
[6.0, 1.0, 7.0], [8.0, 8.0, 4.0]], [[4.0, 2.0, 1.0], [3.0, 4.0, 8.0], [2.0, 5.0, 6.0]], [[5.0, 7.0, 3.0], [3.0, 1.0, 3.0], 
[4.0, 7.0, 5.0]]], [[[7.0, 8.0, 4.0], [1.0, 6.0, 5.0], [2.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [8.0, 7.0, 7.0], [1.0, 8.0, 7.0]], 
[[6.0, 8.0, 3.0], [1.0, 2.0, 7.0], [1.0, 8.0, 4.0]]], [[[8.0, 8.0, 2.0], [8.0, 4.0, 6.0], [2.0, 1.0, 2.0]], [[7.0, 3.0, 3.0], 
[3.0, 7.0, 5.0], [7.0, 5.0, 8.0]], [[3.0, 6.0, 1.0], [3.0, 8.0, 2.0], [7.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LPRN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 7.0], [3.0, 8.0, 1.0], [3.0, 3.0, 1.0]], [[5.0, 2.0, 1.0], [2.0, 6.0, 6.0], [7.0, 7.0, 
5.0]], [[5.0, 1.0, 1.0], [3.0, 7.0, 3.0], [7.0, 2.0, 4.0]]], [[[8.0, 1.0, 2.0], [3.0, 6.0, 2.0], [5.0, 5.0, 4.0]], [[7.0, 3.0, 
3.0], [7.0, 2.0, 2.0], [7.0, 8.0, 3.0]], [[4.0, 1.0, 1.0], [4.0, 3.0, 5.0], [1.0, 5.0, 8.0]]], [[[6.0, 2.0, 2.0], [7.0, 1.0, 
4.0], [7.0, 1.0, 4.0]], [[5.0, 6.0, 1.0], [8.0, 6.0, 2.0], [8.0, 4.0, 7.0]], [[8.0, 6.0, 2.0], [2.0, 3.0, 3.0], [2.0, 5.0, 
7.0]]], [[[4.0, 3.0, 6.0], [7.0, 1.0, 1.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 8.0], [8.0, 5.0, 6.0], [5.0, 5.0, 5.0]], [[2.0, 1.0, 
5.0], [4.0, 4.0, 4.0], [3.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 4.0], [4.0, 2.0, 5.0], [8.0, 6.0, 4.0]], [[6.0, 5.0, 8.0], [5.0, 8.0, 
4.0], [4.0, 6.0, 6.0]], [[2.0, 2.0, 8.0], [3.0, 3.0, 3.0], [7.0, 3.0, 2.0]]], [[[6.0, 4.0, 6.0], [4.0, 6.0, 7.0], [1.0, 7.0, 
6.0]], [[4.0, 3.0, 6.0], [8.0, 5.0, 1.0], [4.0, 7.0, 5.0]], [[5.0, 6.0, 7.0], [8.0, 5.0, 8.0], [7.0, 1.0, 7.0]]], [[[4.0, 7.0, 
5.0], [6.0, 6.0, 5.0], [2.0, 5.0, 4.0]], [[3.0, 5.0, 5.0], [2.0, 3.0, 7.0], [7.0, 3.0, 4.0]], [[3.0, 2.0, 4.0], [8.0, 3.0, 
6.0], [6.0, 5.0, 5.0]]], [[[2.0, 6.0, 8.0], [2.0, 7.0, 7.0], [8.0, 6.0, 3.0]], [[8.0, 4.0, 5.0], [1.0, 6.0, 6.0], [7.0, 2.0, 
2.0]], [[6.0, 5.0, 5.0], [5.0, 3.0, 8.0], [8.0, 1.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 5.0, 3.0]], [[2.0, 8.0, 5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0, 7.0]], [[3.0, 2.0, 
6.0]]])
      arg[2:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 7.0], [3.0, 8.0, 1.0], [3.0, 3.0, 1.0]], [[5.0, 2.0, 1.0], [2.0, 6.0, 6.0], [7.0, 
7.0, 5.0]], [[5.0, 1.0, 1.0], [3.0, 7.0, 3.0], [7.0, 2.0, 4.0]]], [[[8.0, 1.0, 2.0], [3.0, 6.0, 2.0], [5.0, 5.0, 4.0]], [[7.0, 
3.0, 3.0], [7.0, 2.0, 2.0], [7.0, 8.0, 3.0]], [[4.0, 1.0, 1.0], [4.0, 3.0, 5.0], [1.0, 5.0, 8.0]]], [[[6.0, 2.0, 2.0], [7.0, 
1.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 6.0, 1.0], [8.0, 5.0, 3.0], [8.0, 4.0, 7.0]], [[8.0, 6.0, 2.0], [2.0, 3.0, 3.0], [2.0, 5.0, 
7.0]]], [[[4.0, 3.0, 6.0], [7.0, 1.0, 1.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 8.0], [2.0, 8.0, 5.0], [5.0, 5.0, 5.0]], [[2.0, 1.0, 
5.0], [4.0, 4.0, 4.0], [3.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 4.0], [4.0, 2.0, 5.0], [8.0, 6.0, 4.0]], 
[[6.0, 5.0, 8.0], [5.0, 8.0, 4.0], [4.0, 6.0, 6.0]], [[2.0, 2.0, 8.0], [3.0, 3.0, 3.0], [7.0, 3.0, 2.0]]], [[[6.0, 4.0, 6.0], 
[4.0, 6.0, 7.0], [1.0, 7.0, 6.0]], [[4.0, 3.0, 6.0], [8.0, 5.0, 1.0], [4.0, 7.0, 5.0]], [[5.0, 6.0, 7.0], [8.0, 5.0, 8.0], 
[7.0, 1.0, 7.0]]], [[[4.0, 7.0, 5.0], [6.0, 6.0, 5.0], [2.0, 5.0, 4.0]], [[3.0, 5.0, 5.0], [8.0, 1.0, 7.0], [7.0, 3.0, 4.0]], 
[[3.0, 2.0, 4.0], [8.0, 3.0, 6.0], [6.0, 5.0, 5.0]]], [[[2.0, 6.0, 8.0], [2.0, 7.0, 7.0], [8.0, 6.0, 3.0]], [[8.0, 4.0, 5.0], 
[3.0, 2.0, 6.0], [7.0, 2.0, 2.0]], [[6.0, 5.0, 5.0], [5.0, 3.0, 8.0], [8.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LPPN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 4.0], [1.0, 4.0, 3.0], [2.0, 7.0, 7.0]], [[1.0, 6.0, 5.0], [2.0, 4.0, 5.0], [7.0, 8.0, 
3.0]], [[1.0, 8.0, 4.0], [4.0, 4.0, 5.0], [8.0, 4.0, 2.0]]], [[[3.0, 6.0, 1.0], [8.0, 5.0, 5.0], [6.0, 2.0, 3.0]], [[2.0, 4.0, 
8.0], [4.0, 8.0, 3.0], [7.0, 4.0, 3.0]], [[8.0, 5.0, 6.0], [7.0, 8.0, 1.0], [2.0, 8.0, 6.0]]], [[[2.0, 5.0, 6.0], [3.0, 2.0, 
5.0], [8.0, 6.0, 3.0]], [[6.0, 7.0, 7.0], [7.0, 2.0, 2.0], [6.0, 4.0, 4.0]], [[6.0, 2.0, 8.0], [3.0, 7.0, 1.0], [6.0, 2.0, 
8.0]]], [[[1.0, 1.0, 6.0], [8.0, 4.0, 2.0], [4.0, 7.0, 5.0]], [[6.0, 8.0, 6.0], [7.0, 4.0, 5.0], [2.0, 8.0, 4.0]], [[8.0, 6.0, 
7.0], [1.0, 5.0, 4.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 5.0], [2.0, 1.0, 6.0], [6.0, 6.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 3.0, 
3.0], [6.0, 8.0, 1.0]], [[2.0, 7.0, 2.0], [5.0, 2.0, 7.0], [6.0, 3.0, 8.0]]], [[[4.0, 3.0, 3.0], [2.0, 3.0, 4.0], [8.0, 8.0, 
2.0]], [[3.0, 2.0, 2.0], [1.0, 4.0, 1.0], [2.0, 1.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 4.0, 6.0], [2.0, 8.0, 4.0]]], [[[8.0, 4.0, 
8.0], [7.0, 3.0, 6.0], [3.0, 8.0, 7.0]], [[1.0, 3.0, 7.0], [7.0, 8.0, 2.0], [1.0, 2.0, 7.0]], [[7.0, 2.0, 8.0], [4.0, 8.0, 
3.0], [1.0, 2.0, 8.0]]], [[[7.0, 1.0, 3.0], [1.0, 5.0, 6.0], [3.0, 3.0, 2.0]], [[5.0, 1.0, 1.0], [3.0, 3.0, 3.0], [7.0, 7.0, 
6.0]], [[1.0, 2.0, 3.0], [5.0, 1.0, 4.0], [4.0, 2.0, 4.0]]]]))
      val=numpy.array([[1.0, 2.0, 6.0], [6.0, 2.0, 1.0]])
      arg[2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 4.0], [1.0, 4.0, 3.0], [2.0, 7.0, 7.0]], [[1.0, 6.0, 5.0], [2.0, 4.0, 5.0], [7.0, 
8.0, 3.0]], [[1.0, 8.0, 4.0], [4.0, 4.0, 5.0], [8.0, 4.0, 2.0]]], [[[3.0, 6.0, 1.0], [8.0, 5.0, 5.0], [6.0, 2.0, 3.0]], [[2.0, 
4.0, 8.0], [4.0, 8.0, 3.0], [7.0, 4.0, 3.0]], [[8.0, 5.0, 6.0], [7.0, 8.0, 1.0], [2.0, 8.0, 6.0]]], [[[2.0, 5.0, 6.0], [3.0, 
2.0, 5.0], [8.0, 6.0, 3.0]], [[6.0, 7.0, 7.0], [1.0, 2.0, 6.0], [6.0, 4.0, 4.0]], [[6.0, 2.0, 8.0], [3.0, 7.0, 1.0], [6.0, 2.0, 
8.0]]], [[[1.0, 1.0, 6.0], [8.0, 4.0, 2.0], [4.0, 7.0, 5.0]], [[6.0, 8.0, 6.0], [6.0, 2.0, 1.0], [2.0, 8.0, 4.0]], [[8.0, 6.0, 
7.0], [1.0, 5.0, 4.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 5.0], [2.0, 1.0, 6.0], [6.0, 6.0, 8.0]], 
[[8.0, 7.0, 8.0], [2.0, 3.0, 3.0], [6.0, 8.0, 1.0]], [[2.0, 7.0, 2.0], [5.0, 2.0, 7.0], [6.0, 3.0, 8.0]]], [[[4.0, 3.0, 3.0], 
[2.0, 3.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 2.0, 2.0], [1.0, 4.0, 1.0], [2.0, 1.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 4.0, 6.0], 
[2.0, 8.0, 4.0]]], [[[8.0, 4.0, 8.0], [7.0, 3.0, 6.0], [3.0, 8.0, 7.0]], [[1.0, 3.0, 7.0], [1.0, 2.0, 6.0], [1.0, 2.0, 7.0]], 
[[7.0, 2.0, 8.0], [4.0, 8.0, 3.0], [1.0, 2.0, 8.0]]], [[[7.0, 1.0, 3.0], [1.0, 5.0, 6.0], [3.0, 3.0, 2.0]], [[5.0, 1.0, 1.0], 
[6.0, 2.0, 1.0], [7.0, 7.0, 6.0]], [[1.0, 2.0, 3.0], [5.0, 1.0, 4.0], [4.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LPPN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 8.0], [1.0, 2.0, 1.0], [5.0, 2.0, 6.0]], [[1.0, 3.0, 8.0], [7.0, 2.0, 7.0], [2.0, 4.0, 
7.0]], [[5.0, 8.0, 2.0], [1.0, 8.0, 4.0], [2.0, 7.0, 4.0]]], [[[4.0, 6.0, 6.0], [4.0, 1.0, 5.0], [7.0, 8.0, 5.0]], [[8.0, 5.0, 
6.0], [2.0, 6.0, 8.0], [6.0, 6.0, 6.0]], [[3.0, 7.0, 1.0], [2.0, 3.0, 7.0], [6.0, 1.0, 8.0]]], [[[2.0, 6.0, 4.0], [1.0, 5.0, 
2.0], [4.0, 7.0, 6.0]], [[6.0, 3.0, 3.0], [1.0, 7.0, 5.0], [6.0, 4.0, 2.0]], [[6.0, 5.0, 3.0], [8.0, 7.0, 2.0], [8.0, 8.0, 
8.0]]], [[[3.0, 8.0, 3.0], [7.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 1.0, 7.0], [1.0, 5.0, 3.0], [6.0, 4.0, 3.0]], [[6.0, 5.0, 
3.0], [7.0, 2.0, 8.0], [1.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 3.0], [7.0, 4.0, 4.0], [4.0, 2.0, 8.0]], [[1.0, 5.0, 1.0], [4.0, 4.0, 
5.0], [2.0, 1.0, 4.0]], [[4.0, 3.0, 2.0], [4.0, 2.0, 3.0], [4.0, 5.0, 4.0]]], [[[8.0, 8.0, 2.0], [6.0, 6.0, 6.0], [8.0, 4.0, 
8.0]], [[3.0, 5.0, 2.0], [1.0, 6.0, 2.0], [1.0, 8.0, 7.0]], [[7.0, 7.0, 5.0], [6.0, 1.0, 6.0], [7.0, 2.0, 7.0]]], [[[7.0, 3.0, 
5.0], [5.0, 1.0, 6.0], [5.0, 6.0, 5.0]], [[1.0, 5.0, 5.0], [1.0, 6.0, 7.0], [1.0, 1.0, 1.0]], [[1.0, 2.0, 5.0], [5.0, 7.0, 
3.0], [7.0, 6.0, 1.0]]], [[[7.0, 2.0, 2.0], [2.0, 8.0, 8.0], [4.0, 5.0, 5.0]], [[3.0, 1.0, 8.0], [8.0, 5.0, 8.0], [3.0, 5.0, 
7.0]], [[1.0, 6.0, 8.0], [8.0, 4.0, 6.0], [3.0, 1.0, 1.0]]]]))
      val=Data(numpy.array([[4.0, 6.0, 8.0], [8.0, 5.0, 3.0]]),self.functionspace)
      arg[2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 8.0], [1.0, 2.0, 1.0], [5.0, 2.0, 6.0]], [[1.0, 3.0, 8.0], [7.0, 2.0, 7.0], [2.0, 
4.0, 7.0]], [[5.0, 8.0, 2.0], [1.0, 8.0, 4.0], [2.0, 7.0, 4.0]]], [[[4.0, 6.0, 6.0], [4.0, 1.0, 5.0], [7.0, 8.0, 5.0]], [[8.0, 
5.0, 6.0], [2.0, 6.0, 8.0], [6.0, 6.0, 6.0]], [[3.0, 7.0, 1.0], [2.0, 3.0, 7.0], [6.0, 1.0, 8.0]]], [[[2.0, 6.0, 4.0], [1.0, 
5.0, 2.0], [4.0, 7.0, 6.0]], [[6.0, 3.0, 3.0], [4.0, 6.0, 8.0], [6.0, 4.0, 2.0]], [[6.0, 5.0, 3.0], [8.0, 7.0, 2.0], [8.0, 8.0, 
8.0]]], [[[3.0, 8.0, 3.0], [7.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 1.0, 7.0], [8.0, 5.0, 3.0], [6.0, 4.0, 3.0]], [[6.0, 5.0, 
3.0], [7.0, 2.0, 8.0], [1.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 3.0], [7.0, 4.0, 4.0], [4.0, 2.0, 8.0]], 
[[1.0, 5.0, 1.0], [4.0, 4.0, 5.0], [2.0, 1.0, 4.0]], [[4.0, 3.0, 2.0], [4.0, 2.0, 3.0], [4.0, 5.0, 4.0]]], [[[8.0, 8.0, 2.0], 
[6.0, 6.0, 6.0], [8.0, 4.0, 8.0]], [[3.0, 5.0, 2.0], [1.0, 6.0, 2.0], [1.0, 8.0, 7.0]], [[7.0, 7.0, 5.0], [6.0, 1.0, 6.0], 
[7.0, 2.0, 7.0]]], [[[7.0, 3.0, 5.0], [5.0, 1.0, 6.0], [5.0, 6.0, 5.0]], [[1.0, 5.0, 5.0], [4.0, 6.0, 8.0], [1.0, 1.0, 1.0]], 
[[1.0, 2.0, 5.0], [5.0, 7.0, 3.0], [7.0, 6.0, 1.0]]], [[[7.0, 2.0, 2.0], [2.0, 8.0, 8.0], [4.0, 5.0, 5.0]], [[3.0, 1.0, 8.0], 
[8.0, 5.0, 3.0], [3.0, 5.0, 7.0]], [[1.0, 6.0, 8.0], [8.0, 4.0, 6.0], [3.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LPPN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 1.0], [1.0, 7.0, 6.0], [2.0, 8.0, 2.0]], [[8.0, 6.0, 2.0], [3.0, 6.0, 7.0], [4.0, 7.0, 
1.0]], [[4.0, 8.0, 7.0], [7.0, 1.0, 5.0], [3.0, 7.0, 6.0]]], [[[6.0, 5.0, 7.0], [6.0, 2.0, 1.0], [4.0, 1.0, 1.0]], [[5.0, 7.0, 
4.0], [7.0, 7.0, 7.0], [5.0, 3.0, 8.0]], [[6.0, 5.0, 3.0], [8.0, 7.0, 4.0], [3.0, 3.0, 7.0]]], [[[2.0, 3.0, 4.0], [3.0, 2.0, 
2.0], [6.0, 3.0, 3.0]], [[7.0, 2.0, 8.0], [1.0, 5.0, 6.0], [3.0, 8.0, 8.0]], [[8.0, 5.0, 1.0], [4.0, 8.0, 2.0], [7.0, 7.0, 
8.0]]], [[[4.0, 2.0, 6.0], [6.0, 2.0, 5.0], [2.0, 5.0, 6.0]], [[1.0, 7.0, 6.0], [4.0, 8.0, 5.0], [6.0, 5.0, 8.0]], [[4.0, 3.0, 
7.0], [8.0, 7.0, 2.0], [2.0, 7.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 6.0], [5.0, 2.0, 5.0], [6.0, 1.0, 7.0]], [[4.0, 7.0, 3.0], [7.0, 5.0, 
1.0], [2.0, 6.0, 8.0]], [[6.0, 1.0, 4.0], [3.0, 3.0, 6.0], [3.0, 8.0, 6.0]]], [[[2.0, 4.0, 3.0], [2.0, 7.0, 6.0], [4.0, 1.0, 
1.0]], [[3.0, 4.0, 3.0], [3.0, 7.0, 8.0], [3.0, 4.0, 1.0]], [[1.0, 1.0, 2.0], [2.0, 7.0, 3.0], [3.0, 6.0, 8.0]]], [[[6.0, 5.0, 
1.0], [1.0, 3.0, 6.0], [7.0, 3.0, 3.0]], [[6.0, 6.0, 7.0], [5.0, 5.0, 1.0], [8.0, 4.0, 3.0]], [[6.0, 5.0, 8.0], [2.0, 5.0, 
1.0], [1.0, 6.0, 3.0]]], [[[5.0, 4.0, 1.0], [2.0, 3.0, 5.0], [1.0, 5.0, 1.0]], [[3.0, 2.0, 8.0], [6.0, 8.0, 5.0], [7.0, 4.0, 
4.0]], [[4.0, 1.0, 1.0], [6.0, 5.0, 3.0], [2.0, 4.0, 4.0]]]]))
      val=Data(numpy.array([[1.0, 3.0, 2.0], [5.0, 3.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 8.0, 3.0], [1.0, 6.0, 2.0]]))
      arg[2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 1.0], [1.0, 7.0, 6.0], [2.0, 8.0, 2.0]], [[8.0, 6.0, 2.0], [3.0, 6.0, 7.0], [4.0, 
7.0, 1.0]], [[4.0, 8.0, 7.0], [7.0, 1.0, 5.0], [3.0, 7.0, 6.0]]], [[[6.0, 5.0, 7.0], [6.0, 2.0, 1.0], [4.0, 1.0, 1.0]], [[5.0, 
7.0, 4.0], [7.0, 7.0, 7.0], [5.0, 3.0, 8.0]], [[6.0, 5.0, 3.0], [8.0, 7.0, 4.0], [3.0, 3.0, 7.0]]], [[[2.0, 3.0, 4.0], [3.0, 
2.0, 2.0], [6.0, 3.0, 3.0]], [[7.0, 2.0, 8.0], [1.0, 3.0, 2.0], [3.0, 8.0, 8.0]], [[8.0, 5.0, 1.0], [4.0, 8.0, 2.0], [7.0, 7.0, 
8.0]]], [[[4.0, 2.0, 6.0], [6.0, 2.0, 5.0], [2.0, 5.0, 6.0]], [[1.0, 7.0, 6.0], [5.0, 3.0, 6.0], [6.0, 5.0, 8.0]], [[4.0, 3.0, 
7.0], [8.0, 7.0, 2.0], [2.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 6.0], [5.0, 2.0, 5.0], [6.0, 1.0, 7.0]], 
[[4.0, 7.0, 3.0], [7.0, 5.0, 1.0], [2.0, 6.0, 8.0]], [[6.0, 1.0, 4.0], [3.0, 3.0, 6.0], [3.0, 8.0, 6.0]]], [[[2.0, 4.0, 3.0], 
[2.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[3.0, 4.0, 3.0], [3.0, 7.0, 8.0], [3.0, 4.0, 1.0]], [[1.0, 1.0, 2.0], [2.0, 7.0, 3.0], 
[3.0, 6.0, 8.0]]], [[[6.0, 5.0, 1.0], [1.0, 3.0, 6.0], [7.0, 3.0, 3.0]], [[6.0, 6.0, 7.0], [7.0, 8.0, 3.0], [8.0, 4.0, 3.0]], 
[[6.0, 5.0, 8.0], [2.0, 5.0, 1.0], [1.0, 6.0, 3.0]]], [[[5.0, 4.0, 1.0], [2.0, 3.0, 5.0], [1.0, 5.0, 1.0]], [[3.0, 2.0, 8.0], 
[1.0, 6.0, 2.0], [7.0, 4.0, 4.0]], [[4.0, 1.0, 1.0], [6.0, 5.0, 3.0], [2.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LPPN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 2.0], [2.0, 4.0, 8.0], [3.0, 3.0, 3.0]], [[6.0, 1.0, 2.0], [8.0, 3.0, 2.0], [5.0, 7.0, 
1.0]], [[5.0, 6.0, 3.0], [2.0, 2.0, 4.0], [4.0, 3.0, 2.0]]], [[[7.0, 8.0, 4.0], [5.0, 8.0, 6.0], [7.0, 2.0, 6.0]], [[5.0, 3.0, 
2.0], [8.0, 2.0, 7.0], [8.0, 1.0, 6.0]], [[4.0, 5.0, 2.0], [1.0, 8.0, 2.0], [1.0, 3.0, 7.0]]], [[[2.0, 3.0, 1.0], [1.0, 4.0, 
3.0], [6.0, 4.0, 7.0]], [[7.0, 4.0, 7.0], [5.0, 1.0, 5.0], [3.0, 2.0, 7.0]], [[1.0, 3.0, 5.0], [1.0, 6.0, 7.0], [7.0, 3.0, 
2.0]]], [[[4.0, 1.0, 2.0], [8.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[4.0, 5.0, 6.0], [8.0, 2.0, 1.0], [2.0, 5.0, 3.0]], [[6.0, 8.0, 
4.0], [6.0, 3.0, 6.0], [7.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 4.0], [6.0, 1.0, 4.0], [2.0, 6.0, 7.0]], [[1.0, 6.0, 4.0], [5.0, 1.0, 
8.0], [5.0, 8.0, 5.0]], [[4.0, 1.0, 7.0], [3.0, 7.0, 1.0], [6.0, 5.0, 4.0]]], [[[3.0, 6.0, 5.0], [1.0, 4.0, 8.0], [4.0, 5.0, 
2.0]], [[2.0, 4.0, 1.0], [7.0, 8.0, 2.0], [2.0, 8.0, 3.0]], [[5.0, 4.0, 3.0], [8.0, 3.0, 8.0], [3.0, 2.0, 7.0]]], [[[2.0, 4.0, 
2.0], [7.0, 4.0, 8.0], [4.0, 3.0, 4.0]], [[7.0, 4.0, 8.0], [1.0, 2.0, 4.0], [4.0, 7.0, 3.0]], [[2.0, 4.0, 8.0], [8.0, 7.0, 
6.0], [8.0, 6.0, 2.0]]], [[[2.0, 1.0, 2.0], [4.0, 3.0, 5.0], [6.0, 4.0, 6.0]], [[6.0, 7.0, 1.0], [3.0, 7.0, 1.0], [4.0, 1.0, 
8.0]], [[6.0, 5.0, 1.0], [7.0, 1.0, 2.0], [3.0, 8.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[3.0, 3.0, 8.0], [1.0, 3.0, 1.0]])+(1.-msk_val)*numpy.array([[3.0, 7.0, 8.0], [3.0, 2.0, 1.0]])
      arg[2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 2.0], [2.0, 4.0, 8.0], [3.0, 3.0, 3.0]], [[6.0, 1.0, 2.0], [8.0, 3.0, 2.0], [5.0, 
7.0, 1.0]], [[5.0, 6.0, 3.0], [2.0, 2.0, 4.0], [4.0, 3.0, 2.0]]], [[[7.0, 8.0, 4.0], [5.0, 8.0, 6.0], [7.0, 2.0, 6.0]], [[5.0, 
3.0, 2.0], [8.0, 2.0, 7.0], [8.0, 1.0, 6.0]], [[4.0, 5.0, 2.0], [1.0, 8.0, 2.0], [1.0, 3.0, 7.0]]], [[[2.0, 3.0, 1.0], [1.0, 
4.0, 3.0], [6.0, 4.0, 7.0]], [[7.0, 4.0, 7.0], [3.0, 3.0, 8.0], [3.0, 2.0, 7.0]], [[1.0, 3.0, 5.0], [1.0, 6.0, 7.0], [7.0, 3.0, 
2.0]]], [[[4.0, 1.0, 2.0], [8.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[4.0, 5.0, 6.0], [1.0, 3.0, 1.0], [2.0, 5.0, 3.0]], [[6.0, 8.0, 
4.0], [6.0, 3.0, 6.0], [7.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 4.0], [6.0, 1.0, 4.0], [2.0, 6.0, 7.0]], 
[[1.0, 6.0, 4.0], [5.0, 1.0, 8.0], [5.0, 8.0, 5.0]], [[4.0, 1.0, 7.0], [3.0, 7.0, 1.0], [6.0, 5.0, 4.0]]], [[[3.0, 6.0, 5.0], 
[1.0, 4.0, 8.0], [4.0, 5.0, 2.0]], [[2.0, 4.0, 1.0], [7.0, 8.0, 2.0], [2.0, 8.0, 3.0]], [[5.0, 4.0, 3.0], [8.0, 3.0, 8.0], 
[3.0, 2.0, 7.0]]], [[[2.0, 4.0, 2.0], [7.0, 4.0, 8.0], [4.0, 3.0, 4.0]], [[7.0, 4.0, 8.0], [3.0, 7.0, 8.0], [4.0, 7.0, 3.0]], 
[[2.0, 4.0, 8.0], [8.0, 7.0, 6.0], [8.0, 6.0, 2.0]]], [[[2.0, 1.0, 2.0], [4.0, 3.0, 5.0], [6.0, 4.0, 6.0]], [[6.0, 7.0, 1.0], 
[3.0, 2.0, 1.0], [4.0, 1.0, 8.0]], [[6.0, 5.0, 1.0], [7.0, 1.0, 2.0], [3.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_URLN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 6.0], [3.0, 3.0, 4.0], [4.0, 6.0, 4.0]], [[8.0, 8.0, 4.0], [5.0, 2.0, 5.0], [6.0, 6.0, 
5.0]], [[8.0, 1.0, 7.0], [4.0, 5.0, 1.0], [1.0, 6.0, 4.0]]], [[[3.0, 2.0, 1.0], [8.0, 8.0, 1.0], [5.0, 4.0, 7.0]], [[7.0, 5.0, 
3.0], [3.0, 5.0, 4.0], [2.0, 2.0, 2.0]], [[7.0, 5.0, 5.0], [3.0, 5.0, 3.0], [2.0, 8.0, 1.0]]], [[[8.0, 5.0, 4.0], [7.0, 3.0, 
8.0], [4.0, 5.0, 5.0]], [[5.0, 1.0, 5.0], [4.0, 2.0, 5.0], [8.0, 7.0, 7.0]], [[6.0, 4.0, 8.0], [3.0, 5.0, 7.0], [2.0, 6.0, 
1.0]]], [[[1.0, 5.0, 2.0], [3.0, 8.0, 8.0], [7.0, 5.0, 6.0]], [[8.0, 6.0, 6.0], [3.0, 6.0, 1.0], [8.0, 6.0, 6.0]], [[5.0, 2.0, 
3.0], [7.0, 5.0, 5.0], [1.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 4.0], [5.0, 7.0, 6.0], [7.0, 5.0, 6.0]], [[3.0, 6.0, 8.0], [8.0, 1.0, 
4.0], [7.0, 1.0, 5.0]], [[8.0, 7.0, 3.0], [4.0, 3.0, 5.0], [8.0, 7.0, 5.0]]], [[[1.0, 7.0, 4.0], [1.0, 1.0, 4.0], [7.0, 3.0, 
8.0]], [[1.0, 5.0, 3.0], [7.0, 4.0, 4.0], [6.0, 8.0, 7.0]], [[6.0, 7.0, 3.0], [6.0, 1.0, 2.0], [7.0, 5.0, 5.0]]], [[[3.0, 8.0, 
6.0], [6.0, 6.0, 2.0], [2.0, 8.0, 8.0]], [[2.0, 6.0, 5.0], [3.0, 8.0, 1.0], [2.0, 1.0, 6.0]], [[8.0, 1.0, 6.0], [3.0, 8.0, 
7.0], [8.0, 6.0, 6.0]]], [[[1.0, 5.0, 8.0], [1.0, 2.0, 6.0], [6.0, 4.0, 2.0]], [[5.0, 1.0, 4.0], [3.0, 1.0, 3.0], [1.0, 8.0, 
2.0]], [[2.0, 4.0, 2.0], [1.0, 1.0, 6.0], [1.0, 2.0, 1.0]]]]))
      val=numpy.array([[[[1.0, 2.0, 8.0]]], [[[6.0, 7.0, 6.0]]], [[[5.0, 7.0, 2.0]]]])
      arg[:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 6.0], [3.0, 3.0, 4.0], [4.0, 6.0, 4.0]], [[8.0, 8.0, 4.0], [5.0, 2.0, 5.0], [1.0, 
2.0, 8.0]], [[8.0, 1.0, 7.0], [4.0, 5.0, 1.0], [1.0, 6.0, 4.0]]], [[[3.0, 2.0, 1.0], [8.0, 8.0, 1.0], [5.0, 4.0, 7.0]], [[7.0, 
5.0, 3.0], [3.0, 5.0, 4.0], [6.0, 7.0, 6.0]], [[7.0, 5.0, 5.0], [3.0, 5.0, 3.0], [2.0, 8.0, 1.0]]], [[[8.0, 5.0, 4.0], [7.0, 
3.0, 8.0], [4.0, 5.0, 5.0]], [[5.0, 1.0, 5.0], [4.0, 2.0, 5.0], [5.0, 7.0, 2.0]], [[6.0, 4.0, 8.0], [3.0, 5.0, 7.0], [2.0, 6.0, 
1.0]]], [[[1.0, 5.0, 2.0], [3.0, 8.0, 8.0], [7.0, 5.0, 6.0]], [[8.0, 6.0, 6.0], [3.0, 6.0, 1.0], [8.0, 6.0, 6.0]], [[5.0, 2.0, 
3.0], [7.0, 5.0, 5.0], [1.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 4.0], [5.0, 7.0, 6.0], [7.0, 5.0, 6.0]], 
[[3.0, 6.0, 8.0], [8.0, 1.0, 4.0], [1.0, 2.0, 8.0]], [[8.0, 7.0, 3.0], [4.0, 3.0, 5.0], [8.0, 7.0, 5.0]]], [[[1.0, 7.0, 4.0], 
[1.0, 1.0, 4.0], [7.0, 3.0, 8.0]], [[1.0, 5.0, 3.0], [7.0, 4.0, 4.0], [6.0, 7.0, 6.0]], [[6.0, 7.0, 3.0], [6.0, 1.0, 2.0], 
[7.0, 5.0, 5.0]]], [[[3.0, 8.0, 6.0], [6.0, 6.0, 2.0], [2.0, 8.0, 8.0]], [[2.0, 6.0, 5.0], [3.0, 8.0, 1.0], [5.0, 7.0, 2.0]], 
[[8.0, 1.0, 6.0], [3.0, 8.0, 7.0], [8.0, 6.0, 6.0]]], [[[1.0, 5.0, 8.0], [1.0, 2.0, 6.0], [6.0, 4.0, 2.0]], [[5.0, 1.0, 4.0], 
[3.0, 1.0, 3.0], [1.0, 8.0, 2.0]], [[2.0, 4.0, 2.0], [1.0, 1.0, 6.0], [1.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_URLN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 7.0], [3.0, 8.0, 8.0], [1.0, 4.0, 6.0]], [[3.0, 8.0, 8.0], [2.0, 1.0, 8.0], [4.0, 3.0, 
7.0]], [[5.0, 2.0, 6.0], [3.0, 6.0, 4.0], [5.0, 7.0, 1.0]]], [[[6.0, 4.0, 5.0], [1.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 1.0, 
6.0], [5.0, 8.0, 7.0], [7.0, 5.0, 2.0]], [[1.0, 8.0, 6.0], [8.0, 2.0, 6.0], [1.0, 4.0, 5.0]]], [[[1.0, 4.0, 2.0], [6.0, 4.0, 
5.0], [1.0, 1.0, 4.0]], [[8.0, 8.0, 1.0], [1.0, 8.0, 8.0], [5.0, 4.0, 5.0]], [[1.0, 6.0, 7.0], [4.0, 1.0, 4.0], [2.0, 3.0, 
8.0]]], [[[8.0, 2.0, 8.0], [1.0, 7.0, 6.0], [2.0, 7.0, 6.0]], [[2.0, 6.0, 5.0], [3.0, 8.0, 5.0], [6.0, 8.0, 3.0]], [[7.0, 2.0, 
2.0], [7.0, 8.0, 6.0], [8.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 2.0], [2.0, 8.0, 7.0], [1.0, 8.0, 2.0]], [[4.0, 6.0, 5.0], [5.0, 5.0, 
5.0], [3.0, 6.0, 8.0]], [[1.0, 4.0, 5.0], [4.0, 8.0, 7.0], [4.0, 4.0, 6.0]]], [[[1.0, 1.0, 8.0], [7.0, 7.0, 3.0], [6.0, 7.0, 
2.0]], [[2.0, 2.0, 8.0], [5.0, 5.0, 3.0], [5.0, 4.0, 1.0]], [[7.0, 4.0, 5.0], [7.0, 2.0, 1.0], [7.0, 8.0, 1.0]]], [[[1.0, 8.0, 
3.0], [1.0, 5.0, 8.0], [5.0, 3.0, 1.0]], [[5.0, 8.0, 6.0], [1.0, 3.0, 3.0], [7.0, 8.0, 7.0]], [[2.0, 8.0, 6.0], [6.0, 3.0, 
3.0], [2.0, 1.0, 8.0]]], [[[8.0, 4.0, 5.0], [3.0, 2.0, 5.0], [5.0, 3.0, 5.0]], [[4.0, 7.0, 3.0], [5.0, 8.0, 8.0], [7.0, 4.0, 
8.0]], [[8.0, 3.0, 6.0], [8.0, 2.0, 1.0], [1.0, 4.0, 8.0]]]]))
      val=Data(numpy.array([[[[6.0, 3.0, 4.0]]], [[[2.0, 7.0, 6.0]]], [[[1.0, 5.0, 7.0]]]]),self.functionspace)
      arg[:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 7.0], [3.0, 8.0, 8.0], [1.0, 4.0, 6.0]], [[3.0, 8.0, 8.0], [2.0, 1.0, 8.0], [6.0, 
3.0, 4.0]], [[5.0, 2.0, 6.0], [3.0, 6.0, 4.0], [5.0, 7.0, 1.0]]], [[[6.0, 4.0, 5.0], [1.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 
1.0, 6.0], [5.0, 8.0, 7.0], [2.0, 7.0, 6.0]], [[1.0, 8.0, 6.0], [8.0, 2.0, 6.0], [1.0, 4.0, 5.0]]], [[[1.0, 4.0, 2.0], [6.0, 
4.0, 5.0], [1.0, 1.0, 4.0]], [[8.0, 8.0, 1.0], [1.0, 8.0, 8.0], [1.0, 5.0, 7.0]], [[1.0, 6.0, 7.0], [4.0, 1.0, 4.0], [2.0, 3.0, 
8.0]]], [[[8.0, 2.0, 8.0], [1.0, 7.0, 6.0], [2.0, 7.0, 6.0]], [[2.0, 6.0, 5.0], [3.0, 8.0, 5.0], [6.0, 8.0, 3.0]], [[7.0, 2.0, 
2.0], [7.0, 8.0, 6.0], [8.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 2.0], [2.0, 8.0, 7.0], [1.0, 8.0, 2.0]], 
[[4.0, 6.0, 5.0], [5.0, 5.0, 5.0], [6.0, 3.0, 4.0]], [[1.0, 4.0, 5.0], [4.0, 8.0, 7.0], [4.0, 4.0, 6.0]]], [[[1.0, 1.0, 8.0], 
[7.0, 7.0, 3.0], [6.0, 7.0, 2.0]], [[2.0, 2.0, 8.0], [5.0, 5.0, 3.0], [2.0, 7.0, 6.0]], [[7.0, 4.0, 5.0], [7.0, 2.0, 1.0], 
[7.0, 8.0, 1.0]]], [[[1.0, 8.0, 3.0], [1.0, 5.0, 8.0], [5.0, 3.0, 1.0]], [[5.0, 8.0, 6.0], [1.0, 3.0, 3.0], [1.0, 5.0, 7.0]], 
[[2.0, 8.0, 6.0], [6.0, 3.0, 3.0], [2.0, 1.0, 8.0]]], [[[8.0, 4.0, 5.0], [3.0, 2.0, 5.0], [5.0, 3.0, 5.0]], [[4.0, 7.0, 3.0], 
[5.0, 8.0, 8.0], [7.0, 4.0, 8.0]], [[8.0, 3.0, 6.0], [8.0, 2.0, 1.0], [1.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_URLN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 2.0], [3.0, 4.0, 5.0], [4.0, 6.0, 5.0]], [[7.0, 2.0, 7.0], [1.0, 6.0, 6.0], [3.0, 6.0, 
3.0]], [[4.0, 3.0, 2.0], [3.0, 7.0, 8.0], [2.0, 2.0, 8.0]]], [[[7.0, 2.0, 2.0], [6.0, 2.0, 7.0], [5.0, 2.0, 3.0]], [[8.0, 3.0, 
7.0], [3.0, 5.0, 4.0], [2.0, 2.0, 1.0]], [[4.0, 7.0, 7.0], [7.0, 2.0, 6.0], [1.0, 7.0, 8.0]]], [[[3.0, 7.0, 8.0], [1.0, 4.0, 
8.0], [1.0, 1.0, 8.0]], [[6.0, 7.0, 5.0], [1.0, 7.0, 1.0], [7.0, 4.0, 8.0]], [[1.0, 4.0, 7.0], [5.0, 7.0, 8.0], [5.0, 4.0, 
1.0]]], [[[7.0, 1.0, 3.0], [1.0, 4.0, 6.0], [6.0, 1.0, 6.0]], [[2.0, 5.0, 8.0], [3.0, 1.0, 8.0], [7.0, 1.0, 3.0]], [[1.0, 5.0, 
6.0], [1.0, 3.0, 6.0], [5.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 8.0], [6.0, 6.0, 4.0], [2.0, 5.0, 5.0]], [[8.0, 3.0, 3.0], [6.0, 1.0, 
7.0], [5.0, 3.0, 6.0]], [[1.0, 4.0, 4.0], [8.0, 7.0, 7.0], [1.0, 7.0, 7.0]]], [[[8.0, 3.0, 3.0], [3.0, 2.0, 3.0], [6.0, 4.0, 
4.0]], [[2.0, 7.0, 6.0], [1.0, 2.0, 3.0], [3.0, 8.0, 5.0]], [[6.0, 1.0, 6.0], [7.0, 5.0, 8.0], [1.0, 4.0, 3.0]]], [[[2.0, 8.0, 
5.0], [4.0, 3.0, 3.0], [6.0, 3.0, 6.0]], [[3.0, 7.0, 4.0], [3.0, 7.0, 5.0], [3.0, 4.0, 6.0]], [[3.0, 2.0, 5.0], [6.0, 3.0, 
5.0], [6.0, 4.0, 3.0]]], [[[1.0, 1.0, 3.0], [3.0, 2.0, 2.0], [5.0, 8.0, 8.0]], [[3.0, 4.0, 1.0], [7.0, 2.0, 1.0], [3.0, 5.0, 
4.0]], [[5.0, 8.0, 7.0], [4.0, 8.0, 8.0], [3.0, 7.0, 1.0]]]]))
      val=Data(numpy.array([[[[3.0, 1.0, 5.0]]], [[[5.0, 7.0, 1.0]]], [[[7.0, 8.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 5.0]]], [[[8.0, 5.0, 3.0]]], [[[4.0, 5.0, 3.0]]]]))
      arg[:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 2.0], [3.0, 4.0, 5.0], [4.0, 6.0, 5.0]], [[7.0, 2.0, 7.0], [1.0, 6.0, 6.0], [3.0, 
1.0, 5.0]], [[4.0, 3.0, 2.0], [3.0, 7.0, 8.0], [2.0, 2.0, 8.0]]], [[[7.0, 2.0, 2.0], [6.0, 2.0, 7.0], [5.0, 2.0, 3.0]], [[8.0, 
3.0, 7.0], [3.0, 5.0, 4.0], [5.0, 7.0, 1.0]], [[4.0, 7.0, 7.0], [7.0, 2.0, 6.0], [1.0, 7.0, 8.0]]], [[[3.0, 7.0, 8.0], [1.0, 
4.0, 8.0], [1.0, 1.0, 8.0]], [[6.0, 7.0, 5.0], [1.0, 7.0, 1.0], [7.0, 8.0, 4.0]], [[1.0, 4.0, 7.0], [5.0, 7.0, 8.0], [5.0, 4.0, 
1.0]]], [[[7.0, 1.0, 3.0], [1.0, 4.0, 6.0], [6.0, 1.0, 6.0]], [[2.0, 5.0, 8.0], [3.0, 1.0, 8.0], [7.0, 1.0, 3.0]], [[1.0, 5.0, 
6.0], [1.0, 3.0, 6.0], [5.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 8.0], [6.0, 6.0, 4.0], [2.0, 5.0, 5.0]], 
[[8.0, 3.0, 3.0], [6.0, 1.0, 7.0], [2.0, 2.0, 5.0]], [[1.0, 4.0, 4.0], [8.0, 7.0, 7.0], [1.0, 7.0, 7.0]]], [[[8.0, 3.0, 3.0], 
[3.0, 2.0, 3.0], [6.0, 4.0, 4.0]], [[2.0, 7.0, 6.0], [1.0, 2.0, 3.0], [8.0, 5.0, 3.0]], [[6.0, 1.0, 6.0], [7.0, 5.0, 8.0], 
[1.0, 4.0, 3.0]]], [[[2.0, 8.0, 5.0], [4.0, 3.0, 3.0], [6.0, 3.0, 6.0]], [[3.0, 7.0, 4.0], [3.0, 7.0, 5.0], [4.0, 5.0, 3.0]], 
[[3.0, 2.0, 5.0], [6.0, 3.0, 5.0], [6.0, 4.0, 3.0]]], [[[1.0, 1.0, 3.0], [3.0, 2.0, 2.0], [5.0, 8.0, 8.0]], [[3.0, 4.0, 1.0], 
[7.0, 2.0, 1.0], [3.0, 5.0, 4.0]], [[5.0, 8.0, 7.0], [4.0, 8.0, 8.0], [3.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_URLN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 8.0], [4.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[4.0, 2.0, 2.0], [5.0, 3.0, 2.0], [2.0, 6.0, 
8.0]], [[7.0, 2.0, 5.0], [7.0, 8.0, 1.0], [7.0, 1.0, 2.0]]], [[[2.0, 1.0, 5.0], [3.0, 3.0, 5.0], [7.0, 1.0, 2.0]], [[5.0, 6.0, 
8.0], [6.0, 6.0, 8.0], [2.0, 5.0, 7.0]], [[8.0, 6.0, 1.0], [7.0, 7.0, 5.0], [1.0, 3.0, 8.0]]], [[[6.0, 7.0, 1.0], [4.0, 1.0, 
4.0], [8.0, 4.0, 2.0]], [[6.0, 5.0, 5.0], [2.0, 2.0, 8.0], [8.0, 1.0, 7.0]], [[6.0, 1.0, 2.0], [2.0, 6.0, 5.0], [6.0, 3.0, 
7.0]]], [[[3.0, 7.0, 6.0], [1.0, 3.0, 5.0], [4.0, 7.0, 1.0]], [[7.0, 1.0, 7.0], [7.0, 6.0, 4.0], [7.0, 3.0, 5.0]], [[1.0, 6.0, 
7.0], [4.0, 3.0, 4.0], [2.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 6.0], [7.0, 3.0, 2.0], [8.0, 3.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 8.0, 
4.0], [2.0, 6.0, 4.0]], [[7.0, 8.0, 5.0], [7.0, 2.0, 3.0], [4.0, 5.0, 6.0]]], [[[2.0, 6.0, 6.0], [4.0, 1.0, 4.0], [6.0, 3.0, 
2.0]], [[2.0, 2.0, 6.0], [5.0, 2.0, 7.0], [1.0, 8.0, 8.0]], [[4.0, 3.0, 5.0], [2.0, 3.0, 1.0], [2.0, 6.0, 3.0]]], [[[1.0, 4.0, 
2.0], [7.0, 8.0, 6.0], [7.0, 4.0, 3.0]], [[8.0, 1.0, 2.0], [7.0, 4.0, 3.0], [4.0, 4.0, 1.0]], [[4.0, 1.0, 7.0], [3.0, 7.0, 
5.0], [4.0, 3.0, 5.0]]], [[[3.0, 4.0, 4.0], [2.0, 1.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 4.0, 8.0], [8.0, 8.0, 5.0], [1.0, 1.0, 
4.0]], [[4.0, 7.0, 2.0], [7.0, 6.0, 3.0], [5.0, 7.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0, 8.0, 6.0]]], [[[6.0, 7.0, 4.0]]], [[[4.0, 5.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 4.0, 2.0]]], [[[2.0, 5.0, 1.0]]], [[[4.0, 3.0, 5.0]]]])
      arg[:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 8.0], [4.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[4.0, 2.0, 2.0], [5.0, 3.0, 2.0], [4.0, 
8.0, 6.0]], [[7.0, 2.0, 5.0], [7.0, 8.0, 1.0], [7.0, 1.0, 2.0]]], [[[2.0, 1.0, 5.0], [3.0, 3.0, 5.0], [7.0, 1.0, 2.0]], [[5.0, 
6.0, 8.0], [6.0, 6.0, 8.0], [6.0, 7.0, 4.0]], [[8.0, 6.0, 1.0], [7.0, 7.0, 5.0], [1.0, 3.0, 8.0]]], [[[6.0, 7.0, 1.0], [4.0, 
1.0, 4.0], [8.0, 4.0, 2.0]], [[6.0, 5.0, 5.0], [2.0, 2.0, 8.0], [4.0, 5.0, 5.0]], [[6.0, 1.0, 2.0], [2.0, 6.0, 5.0], [6.0, 3.0, 
7.0]]], [[[3.0, 7.0, 6.0], [1.0, 3.0, 5.0], [4.0, 7.0, 1.0]], [[7.0, 1.0, 7.0], [7.0, 6.0, 4.0], [7.0, 3.0, 5.0]], [[1.0, 6.0, 
7.0], [4.0, 3.0, 4.0], [2.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 6.0], [7.0, 3.0, 2.0], [8.0, 3.0, 3.0]], 
[[1.0, 1.0, 7.0], [8.0, 8.0, 4.0], [3.0, 4.0, 2.0]], [[7.0, 8.0, 5.0], [7.0, 2.0, 3.0], [4.0, 5.0, 6.0]]], [[[2.0, 6.0, 6.0], 
[4.0, 1.0, 4.0], [6.0, 3.0, 2.0]], [[2.0, 2.0, 6.0], [5.0, 2.0, 7.0], [2.0, 5.0, 1.0]], [[4.0, 3.0, 5.0], [2.0, 3.0, 1.0], 
[2.0, 6.0, 3.0]]], [[[1.0, 4.0, 2.0], [7.0, 8.0, 6.0], [7.0, 4.0, 3.0]], [[8.0, 1.0, 2.0], [7.0, 4.0, 3.0], [4.0, 3.0, 5.0]], 
[[4.0, 1.0, 7.0], [3.0, 7.0, 5.0], [4.0, 3.0, 5.0]]], [[[3.0, 4.0, 4.0], [2.0, 1.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 4.0, 8.0], 
[8.0, 8.0, 5.0], [1.0, 1.0, 4.0]], [[4.0, 7.0, 2.0], [7.0, 6.0, 3.0], [5.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PUCN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 4.0], [2.0, 1.0, 3.0], [6.0, 5.0, 1.0]], [[8.0, 3.0, 3.0], [4.0, 4.0, 5.0], [3.0, 7.0, 
7.0]], [[3.0, 5.0, 7.0], [6.0, 4.0, 8.0], [1.0, 1.0, 6.0]]], [[[3.0, 3.0, 4.0], [8.0, 4.0, 8.0], [7.0, 6.0, 4.0]], [[5.0, 5.0, 
2.0], [1.0, 5.0, 2.0], [1.0, 1.0, 5.0]], [[2.0, 2.0, 2.0], [2.0, 7.0, 4.0], [3.0, 1.0, 8.0]]], [[[3.0, 2.0, 5.0], [2.0, 7.0, 
8.0], [7.0, 1.0, 4.0]], [[5.0, 2.0, 2.0], [3.0, 1.0, 2.0], [8.0, 5.0, 5.0]], [[2.0, 7.0, 5.0], [6.0, 5.0, 2.0], [5.0, 6.0, 
5.0]]], [[[6.0, 4.0, 2.0], [4.0, 4.0, 7.0], [2.0, 7.0, 5.0]], [[4.0, 6.0, 3.0], [8.0, 3.0, 8.0], [2.0, 5.0, 1.0]], [[6.0, 2.0, 
5.0], [3.0, 4.0, 1.0], [4.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 8.0], [3.0, 5.0, 2.0], [4.0, 7.0, 3.0]], [[1.0, 2.0, 8.0], [4.0, 6.0, 
8.0], [4.0, 3.0, 2.0]], [[1.0, 4.0, 1.0], [8.0, 1.0, 2.0], [1.0, 8.0, 3.0]]], [[[5.0, 3.0, 2.0], [5.0, 4.0, 7.0], [7.0, 8.0, 
8.0]], [[4.0, 7.0, 8.0], [5.0, 7.0, 5.0], [6.0, 2.0, 2.0]], [[8.0, 7.0, 8.0], [7.0, 2.0, 5.0], [8.0, 8.0, 1.0]]], [[[2.0, 3.0, 
2.0], [2.0, 6.0, 7.0], [2.0, 1.0, 6.0]], [[4.0, 1.0, 2.0], [7.0, 7.0, 1.0], [5.0, 8.0, 3.0]], [[6.0, 6.0, 5.0], [7.0, 3.0, 
2.0], [1.0, 1.0, 7.0]]], [[[6.0, 8.0, 2.0], [1.0, 4.0, 4.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 4.0], [6.0, 1.0, 5.0], [6.0, 6.0, 
7.0]], [[3.0, 8.0, 7.0], [3.0, 5.0, 2.0], [3.0, 8.0, 6.0]]]]))
      val=numpy.array([[[3.0, 1.0, 4.0], [2.0, 4.0, 6.0], [3.0, 5.0, 2.0]], [[6.0, 3.0, 3.0], [2.0, 6.0, 7.0], [7.0, 5.0, 
4.0]]])
      arg[2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 4.0], [2.0, 1.0, 3.0], [6.0, 5.0, 1.0]], [[8.0, 3.0, 3.0], [4.0, 4.0, 5.0], [3.0, 
7.0, 7.0]], [[3.0, 5.0, 7.0], [6.0, 4.0, 8.0], [1.0, 1.0, 6.0]]], [[[3.0, 3.0, 4.0], [8.0, 4.0, 8.0], [7.0, 6.0, 4.0]], [[5.0, 
5.0, 2.0], [1.0, 5.0, 2.0], [1.0, 1.0, 5.0]], [[2.0, 2.0, 2.0], [2.0, 7.0, 4.0], [3.0, 1.0, 8.0]]], [[[3.0, 1.0, 4.0], [2.0, 
4.0, 6.0], [3.0, 5.0, 2.0]], [[6.0, 3.0, 3.0], [2.0, 6.0, 7.0], [7.0, 5.0, 4.0]], [[2.0, 7.0, 5.0], [6.0, 5.0, 2.0], [5.0, 6.0, 
5.0]]], [[[6.0, 4.0, 2.0], [4.0, 4.0, 7.0], [2.0, 7.0, 5.0]], [[4.0, 6.0, 3.0], [8.0, 3.0, 8.0], [2.0, 5.0, 1.0]], [[6.0, 2.0, 
5.0], [3.0, 4.0, 1.0], [4.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 8.0], [3.0, 5.0, 2.0], [4.0, 7.0, 3.0]], 
[[1.0, 2.0, 8.0], [4.0, 6.0, 8.0], [4.0, 3.0, 2.0]], [[1.0, 4.0, 1.0], [8.0, 1.0, 2.0], [1.0, 8.0, 3.0]]], [[[5.0, 3.0, 2.0], 
[5.0, 4.0, 7.0], [7.0, 8.0, 8.0]], [[4.0, 7.0, 8.0], [5.0, 7.0, 5.0], [6.0, 2.0, 2.0]], [[8.0, 7.0, 8.0], [7.0, 2.0, 5.0], 
[8.0, 8.0, 1.0]]], [[[3.0, 1.0, 4.0], [2.0, 4.0, 6.0], [3.0, 5.0, 2.0]], [[6.0, 3.0, 3.0], [2.0, 6.0, 7.0], [7.0, 5.0, 4.0]], 
[[6.0, 6.0, 5.0], [7.0, 3.0, 2.0], [1.0, 1.0, 7.0]]], [[[6.0, 8.0, 2.0], [1.0, 4.0, 4.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 4.0], 
[6.0, 1.0, 5.0], [6.0, 6.0, 7.0]], [[3.0, 8.0, 7.0], [3.0, 5.0, 2.0], [3.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PUCN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 7.0], [1.0, 6.0, 3.0], [3.0, 6.0, 5.0]], [[7.0, 5.0, 1.0], [5.0, 7.0, 8.0], [6.0, 8.0, 
8.0]], [[7.0, 3.0, 7.0], [1.0, 7.0, 6.0], [3.0, 5.0, 7.0]]], [[[6.0, 8.0, 6.0], [2.0, 1.0, 4.0], [6.0, 2.0, 6.0]], [[1.0, 3.0, 
6.0], [6.0, 7.0, 8.0], [7.0, 2.0, 5.0]], [[3.0, 8.0, 4.0], [4.0, 7.0, 3.0], [8.0, 1.0, 4.0]]], [[[5.0, 2.0, 8.0], [3.0, 4.0, 
6.0], [5.0, 3.0, 4.0]], [[5.0, 4.0, 8.0], [3.0, 4.0, 1.0], [6.0, 4.0, 7.0]], [[6.0, 1.0, 2.0], [8.0, 1.0, 8.0], [3.0, 1.0, 
5.0]]], [[[4.0, 6.0, 7.0], [2.0, 8.0, 7.0], [2.0, 6.0, 3.0]], [[6.0, 5.0, 6.0], [7.0, 3.0, 7.0], [5.0, 7.0, 8.0]], [[2.0, 2.0, 
2.0], [1.0, 2.0, 2.0], [5.0, 7.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 8.0], [8.0, 8.0, 4.0], [7.0, 5.0, 5.0]], [[5.0, 6.0, 8.0], [4.0, 5.0, 
7.0], [6.0, 1.0, 4.0]], [[4.0, 3.0, 6.0], [5.0, 1.0, 8.0], [4.0, 8.0, 1.0]]], [[[5.0, 7.0, 7.0], [7.0, 7.0, 4.0], [8.0, 4.0, 
5.0]], [[6.0, 3.0, 1.0], [8.0, 8.0, 8.0], [7.0, 2.0, 1.0]], [[2.0, 8.0, 2.0], [3.0, 8.0, 8.0], [5.0, 3.0, 4.0]]], [[[4.0, 2.0, 
8.0], [7.0, 2.0, 3.0], [2.0, 8.0, 1.0]], [[6.0, 1.0, 5.0], [1.0, 8.0, 8.0], [5.0, 5.0, 6.0]], [[2.0, 7.0, 5.0], [1.0, 5.0, 
7.0], [4.0, 6.0, 1.0]]], [[[1.0, 7.0, 5.0], [7.0, 2.0, 4.0], [1.0, 1.0, 8.0]], [[5.0, 6.0, 5.0], [1.0, 5.0, 8.0], [1.0, 3.0, 
3.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 1.0], [7.0, 8.0, 3.0]]]]))
      val=Data(numpy.array([[[6.0, 7.0, 2.0], [8.0, 5.0, 7.0], [2.0, 3.0, 1.0]], [[8.0, 5.0, 4.0], [1.0, 2.0, 4.0], [3.0, 1.0, 
6.0]]]),self.functionspace)
      arg[2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 7.0], [1.0, 6.0, 3.0], [3.0, 6.0, 5.0]], [[7.0, 5.0, 1.0], [5.0, 7.0, 8.0], [6.0, 
8.0, 8.0]], [[7.0, 3.0, 7.0], [1.0, 7.0, 6.0], [3.0, 5.0, 7.0]]], [[[6.0, 8.0, 6.0], [2.0, 1.0, 4.0], [6.0, 2.0, 6.0]], [[1.0, 
3.0, 6.0], [6.0, 7.0, 8.0], [7.0, 2.0, 5.0]], [[3.0, 8.0, 4.0], [4.0, 7.0, 3.0], [8.0, 1.0, 4.0]]], [[[6.0, 7.0, 2.0], [8.0, 
5.0, 7.0], [2.0, 3.0, 1.0]], [[8.0, 5.0, 4.0], [1.0, 2.0, 4.0], [3.0, 1.0, 6.0]], [[6.0, 1.0, 2.0], [8.0, 1.0, 8.0], [3.0, 1.0, 
5.0]]], [[[4.0, 6.0, 7.0], [2.0, 8.0, 7.0], [2.0, 6.0, 3.0]], [[6.0, 5.0, 6.0], [7.0, 3.0, 7.0], [5.0, 7.0, 8.0]], [[2.0, 2.0, 
2.0], [1.0, 2.0, 2.0], [5.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 8.0, 4.0], [7.0, 5.0, 5.0]], 
[[5.0, 6.0, 8.0], [4.0, 5.0, 7.0], [6.0, 1.0, 4.0]], [[4.0, 3.0, 6.0], [5.0, 1.0, 8.0], [4.0, 8.0, 1.0]]], [[[5.0, 7.0, 7.0], 
[7.0, 7.0, 4.0], [8.0, 4.0, 5.0]], [[6.0, 3.0, 1.0], [8.0, 8.0, 8.0], [7.0, 2.0, 1.0]], [[2.0, 8.0, 2.0], [3.0, 8.0, 8.0], 
[5.0, 3.0, 4.0]]], [[[6.0, 7.0, 2.0], [8.0, 5.0, 7.0], [2.0, 3.0, 1.0]], [[8.0, 5.0, 4.0], [1.0, 2.0, 4.0], [3.0, 1.0, 6.0]], 
[[2.0, 7.0, 5.0], [1.0, 5.0, 7.0], [4.0, 6.0, 1.0]]], [[[1.0, 7.0, 5.0], [7.0, 2.0, 4.0], [1.0, 1.0, 8.0]], [[5.0, 6.0, 5.0], 
[1.0, 5.0, 8.0], [1.0, 3.0, 3.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 1.0], [7.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PUCN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 7.0], [6.0, 2.0, 7.0], [3.0, 8.0, 6.0]], [[5.0, 7.0, 4.0], [5.0, 1.0, 8.0], [1.0, 7.0, 
7.0]], [[2.0, 8.0, 2.0], [1.0, 5.0, 8.0], [7.0, 5.0, 7.0]]], [[[7.0, 5.0, 4.0], [6.0, 6.0, 6.0], [3.0, 7.0, 2.0]], [[6.0, 6.0, 
4.0], [7.0, 7.0, 3.0], [4.0, 1.0, 8.0]], [[3.0, 3.0, 7.0], [1.0, 2.0, 4.0], [6.0, 4.0, 8.0]]], [[[2.0, 3.0, 6.0], [4.0, 8.0, 
8.0], [6.0, 6.0, 1.0]], [[4.0, 8.0, 7.0], [6.0, 7.0, 1.0], [3.0, 7.0, 5.0]], [[1.0, 3.0, 7.0], [7.0, 8.0, 6.0], [8.0, 2.0, 
7.0]]], [[[1.0, 3.0, 1.0], [2.0, 1.0, 5.0], [1.0, 7.0, 1.0]], [[5.0, 5.0, 4.0], [2.0, 4.0, 8.0], [3.0, 8.0, 7.0]], [[7.0, 6.0, 
8.0], [2.0, 4.0, 2.0], [2.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 4.0], [2.0, 1.0, 7.0], [6.0, 1.0, 7.0]], [[3.0, 1.0, 3.0], [7.0, 8.0, 
3.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 2.0], [6.0, 5.0, 2.0], [3.0, 7.0, 7.0]]], [[[2.0, 2.0, 4.0], [3.0, 2.0, 3.0], [4.0, 6.0, 
1.0]], [[6.0, 3.0, 7.0], [6.0, 1.0, 4.0], [2.0, 1.0, 4.0]], [[2.0, 3.0, 3.0], [1.0, 4.0, 5.0], [2.0, 2.0, 7.0]]], [[[4.0, 2.0, 
3.0], [7.0, 6.0, 1.0], [7.0, 2.0, 3.0]], [[7.0, 5.0, 8.0], [7.0, 7.0, 2.0], [8.0, 1.0, 8.0]], [[6.0, 7.0, 2.0], [3.0, 6.0, 
4.0], [8.0, 7.0, 1.0]]], [[[5.0, 6.0, 6.0], [4.0, 8.0, 3.0], [5.0, 1.0, 3.0]], [[4.0, 2.0, 8.0], [3.0, 5.0, 2.0], [6.0, 8.0, 
2.0]], [[2.0, 6.0, 3.0], [2.0, 5.0, 2.0], [1.0, 3.0, 6.0]]]]))
      val=Data(numpy.array([[[7.0, 3.0, 3.0], [1.0, 2.0, 6.0], [1.0, 5.0, 3.0]], [[7.0, 4.0, 6.0], [3.0, 4.0, 1.0], [5.0, 4.0, 
1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 7.0, 7.0], [2.0, 3.0, 8.0], [7.0, 6.0, 6.0]], [[3.0, 6.0, 4.0], [1.0, 1.0, 1.0], 
[7.0, 1.0, 8.0]]]))
      arg[2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 7.0], [6.0, 2.0, 7.0], [3.0, 8.0, 6.0]], [[5.0, 7.0, 4.0], [5.0, 1.0, 8.0], [1.0, 
7.0, 7.0]], [[2.0, 8.0, 2.0], [1.0, 5.0, 8.0], [7.0, 5.0, 7.0]]], [[[7.0, 5.0, 4.0], [6.0, 6.0, 6.0], [3.0, 7.0, 2.0]], [[6.0, 
6.0, 4.0], [7.0, 7.0, 3.0], [4.0, 1.0, 8.0]], [[3.0, 3.0, 7.0], [1.0, 2.0, 4.0], [6.0, 4.0, 8.0]]], [[[7.0, 3.0, 3.0], [1.0, 
2.0, 6.0], [1.0, 5.0, 3.0]], [[7.0, 4.0, 6.0], [3.0, 4.0, 1.0], [5.0, 4.0, 1.0]], [[1.0, 3.0, 7.0], [7.0, 8.0, 6.0], [8.0, 2.0, 
7.0]]], [[[1.0, 3.0, 1.0], [2.0, 1.0, 5.0], [1.0, 7.0, 1.0]], [[5.0, 5.0, 4.0], [2.0, 4.0, 8.0], [3.0, 8.0, 7.0]], [[7.0, 6.0, 
8.0], [2.0, 4.0, 2.0], [2.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 4.0], [2.0, 1.0, 7.0], [6.0, 1.0, 7.0]], 
[[3.0, 1.0, 3.0], [7.0, 8.0, 3.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 2.0], [6.0, 5.0, 2.0], [3.0, 7.0, 7.0]]], [[[2.0, 2.0, 4.0], 
[3.0, 2.0, 3.0], [4.0, 6.0, 1.0]], [[6.0, 3.0, 7.0], [6.0, 1.0, 4.0], [2.0, 1.0, 4.0]], [[2.0, 3.0, 3.0], [1.0, 4.0, 5.0], 
[2.0, 2.0, 7.0]]], [[[1.0, 7.0, 7.0], [2.0, 3.0, 8.0], [7.0, 6.0, 6.0]], [[3.0, 6.0, 4.0], [1.0, 1.0, 1.0], [7.0, 1.0, 8.0]], 
[[6.0, 7.0, 2.0], [3.0, 6.0, 4.0], [8.0, 7.0, 1.0]]], [[[5.0, 6.0, 6.0], [4.0, 8.0, 3.0], [5.0, 1.0, 3.0]], [[4.0, 2.0, 8.0], 
[3.0, 5.0, 2.0], [6.0, 8.0, 2.0]], [[2.0, 6.0, 3.0], [2.0, 5.0, 2.0], [1.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PUCN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 8.0], [2.0, 5.0, 2.0], [8.0, 1.0, 5.0]], [[4.0, 4.0, 2.0], [8.0, 5.0, 1.0], [8.0, 8.0, 
8.0]], [[4.0, 2.0, 5.0], [4.0, 6.0, 2.0], [8.0, 2.0, 2.0]]], [[[3.0, 5.0, 2.0], [1.0, 8.0, 7.0], [4.0, 7.0, 3.0]], [[5.0, 2.0, 
6.0], [2.0, 1.0, 3.0], [4.0, 1.0, 7.0]], [[3.0, 6.0, 4.0], [3.0, 7.0, 6.0], [1.0, 7.0, 1.0]]], [[[7.0, 1.0, 3.0], [8.0, 7.0, 
3.0], [4.0, 4.0, 8.0]], [[7.0, 6.0, 1.0], [2.0, 7.0, 4.0], [1.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [5.0, 5.0, 4.0], [4.0, 2.0, 
8.0]]], [[[3.0, 6.0, 2.0], [2.0, 3.0, 5.0], [6.0, 5.0, 7.0]], [[3.0, 2.0, 7.0], [8.0, 1.0, 1.0], [2.0, 3.0, 2.0]], [[1.0, 6.0, 
3.0], [6.0, 2.0, 6.0], [5.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 8.0], [6.0, 5.0, 3.0], [8.0, 1.0, 8.0]], [[7.0, 3.0, 5.0], [3.0, 4.0, 
6.0], [1.0, 8.0, 2.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 1.0], [2.0, 1.0, 6.0]]], [[[7.0, 7.0, 1.0], [4.0, 5.0, 4.0], [4.0, 5.0, 
4.0]], [[3.0, 3.0, 5.0], [1.0, 2.0, 3.0], [1.0, 3.0, 7.0]], [[4.0, 8.0, 5.0], [6.0, 1.0, 2.0], [5.0, 3.0, 2.0]]], [[[2.0, 6.0, 
6.0], [8.0, 1.0, 4.0], [5.0, 8.0, 5.0]], [[3.0, 8.0, 1.0], [4.0, 6.0, 5.0], [1.0, 2.0, 3.0]], [[2.0, 2.0, 7.0], [3.0, 6.0, 
7.0], [6.0, 7.0, 5.0]]], [[[5.0, 4.0, 7.0], [1.0, 7.0, 6.0], [5.0, 3.0, 5.0]], [[5.0, 2.0, 1.0], [2.0, 1.0, 2.0], [2.0, 2.0, 
4.0]], [[2.0, 5.0, 5.0], [5.0, 7.0, 5.0], [3.0, 6.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 4.0, 6.0], [7.0, 2.0, 4.0], [2.0, 7.0, 8.0]], [[2.0, 3.0, 7.0], [2.0, 1.0, 8.0], [6.0, 
7.0, 4.0]]])+(1.-msk_val)*numpy.array([[[4.0, 7.0, 4.0], [8.0, 1.0, 3.0], [4.0, 6.0, 5.0]], [[3.0, 8.0, 1.0], [5.0, 6.0, 7.0], 
[3.0, 8.0, 4.0]]])
      arg[2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [2.0, 5.0, 2.0], [8.0, 1.0, 5.0]], [[4.0, 4.0, 2.0], [8.0, 5.0, 1.0], [8.0, 
8.0, 8.0]], [[4.0, 2.0, 5.0], [4.0, 6.0, 2.0], [8.0, 2.0, 2.0]]], [[[3.0, 5.0, 2.0], [1.0, 8.0, 7.0], [4.0, 7.0, 3.0]], [[5.0, 
2.0, 6.0], [2.0, 1.0, 3.0], [4.0, 1.0, 7.0]], [[3.0, 6.0, 4.0], [3.0, 7.0, 6.0], [1.0, 7.0, 1.0]]], [[[8.0, 4.0, 6.0], [7.0, 
2.0, 4.0], [2.0, 7.0, 8.0]], [[2.0, 3.0, 7.0], [2.0, 1.0, 8.0], [6.0, 7.0, 4.0]], [[7.0, 8.0, 7.0], [5.0, 5.0, 4.0], [4.0, 2.0, 
8.0]]], [[[3.0, 6.0, 2.0], [2.0, 3.0, 5.0], [6.0, 5.0, 7.0]], [[3.0, 2.0, 7.0], [8.0, 1.0, 1.0], [2.0, 3.0, 2.0]], [[1.0, 6.0, 
3.0], [6.0, 2.0, 6.0], [5.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 8.0], [6.0, 5.0, 3.0], [8.0, 1.0, 8.0]], 
[[7.0, 3.0, 5.0], [3.0, 4.0, 6.0], [1.0, 8.0, 2.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 1.0], [2.0, 1.0, 6.0]]], [[[7.0, 7.0, 1.0], 
[4.0, 5.0, 4.0], [4.0, 5.0, 4.0]], [[3.0, 3.0, 5.0], [1.0, 2.0, 3.0], [1.0, 3.0, 7.0]], [[4.0, 8.0, 5.0], [6.0, 1.0, 2.0], 
[5.0, 3.0, 2.0]]], [[[4.0, 7.0, 4.0], [8.0, 1.0, 3.0], [4.0, 6.0, 5.0]], [[3.0, 8.0, 1.0], [5.0, 6.0, 7.0], [3.0, 8.0, 4.0]], 
[[2.0, 2.0, 7.0], [3.0, 6.0, 7.0], [6.0, 7.0, 5.0]]], [[[5.0, 4.0, 7.0], [1.0, 7.0, 6.0], [5.0, 3.0, 5.0]], [[5.0, 2.0, 1.0], 
[2.0, 1.0, 2.0], [2.0, 2.0, 4.0]], [[2.0, 5.0, 5.0], [5.0, 7.0, 5.0], [3.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRRN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 5.0], [2.0, 5.0, 1.0], [8.0, 8.0, 6.0]], [[1.0, 1.0, 6.0], [3.0, 7.0, 2.0], [7.0, 6.0, 
6.0]], [[3.0, 2.0, 3.0], [5.0, 2.0, 3.0], [7.0, 1.0, 4.0]]], [[[2.0, 2.0, 6.0], [8.0, 7.0, 1.0], [6.0, 4.0, 2.0]], [[8.0, 6.0, 
7.0], [7.0, 7.0, 2.0], [4.0, 8.0, 6.0]], [[8.0, 5.0, 4.0], [2.0, 5.0, 5.0], [3.0, 2.0, 4.0]]], [[[7.0, 6.0, 3.0], [1.0, 7.0, 
7.0], [8.0, 6.0, 3.0]], [[2.0, 6.0, 2.0], [1.0, 3.0, 7.0], [7.0, 6.0, 8.0]], [[3.0, 4.0, 3.0], [5.0, 5.0, 8.0], [4.0, 3.0, 
2.0]]], [[[3.0, 1.0, 3.0], [4.0, 2.0, 8.0], [6.0, 8.0, 8.0]], [[6.0, 3.0, 8.0], [1.0, 4.0, 5.0], [8.0, 1.0, 5.0]], [[4.0, 2.0, 
1.0], [6.0, 8.0, 1.0], [2.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 4.0], [3.0, 2.0, 8.0], [6.0, 7.0, 8.0]], [[1.0, 6.0, 6.0], [6.0, 5.0, 
2.0], [8.0, 7.0, 1.0]], [[5.0, 5.0, 2.0], [3.0, 8.0, 8.0], [3.0, 5.0, 3.0]]], [[[4.0, 2.0, 7.0], [8.0, 8.0, 4.0], [8.0, 8.0, 
4.0]], [[3.0, 8.0, 7.0], [8.0, 7.0, 3.0], [8.0, 1.0, 3.0]], [[8.0, 5.0, 4.0], [5.0, 8.0, 2.0], [7.0, 8.0, 5.0]]], [[[2.0, 7.0, 
8.0], [7.0, 8.0, 5.0], [6.0, 7.0, 5.0]], [[3.0, 6.0, 1.0], [2.0, 6.0, 2.0], [3.0, 6.0, 5.0]], [[7.0, 8.0, 6.0], [5.0, 3.0, 
1.0], [6.0, 2.0, 3.0]]], [[[3.0, 2.0, 2.0], [7.0, 1.0, 1.0], [6.0, 3.0, 1.0]], [[7.0, 5.0, 2.0], [5.0, 5.0, 6.0], [2.0, 8.0, 
6.0]], [[3.0, 8.0, 7.0], [4.0, 1.0, 2.0], [5.0, 7.0, 5.0]]]]))
      val=numpy.array([[[[2.0, 3.0, 2.0]]], [[[1.0, 4.0, 8.0]]], [[[1.0, 5.0, 8.0]]], [[[5.0, 5.0, 2.0]]]])
      arg[:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 5.0], [2.0, 5.0, 1.0], [8.0, 8.0, 6.0]], [[1.0, 1.0, 6.0], [2.0, 3.0, 2.0], [7.0, 
6.0, 6.0]], [[3.0, 2.0, 3.0], [5.0, 2.0, 3.0], [7.0, 1.0, 4.0]]], [[[2.0, 2.0, 6.0], [8.0, 7.0, 1.0], [6.0, 4.0, 2.0]], [[8.0, 
6.0, 7.0], [1.0, 4.0, 8.0], [4.0, 8.0, 6.0]], [[8.0, 5.0, 4.0], [2.0, 5.0, 5.0], [3.0, 2.0, 4.0]]], [[[7.0, 6.0, 3.0], [1.0, 
7.0, 7.0], [8.0, 6.0, 3.0]], [[2.0, 6.0, 2.0], [1.0, 5.0, 8.0], [7.0, 6.0, 8.0]], [[3.0, 4.0, 3.0], [5.0, 5.0, 8.0], [4.0, 3.0, 
2.0]]], [[[3.0, 1.0, 3.0], [4.0, 2.0, 8.0], [6.0, 8.0, 8.0]], [[6.0, 3.0, 8.0], [5.0, 5.0, 2.0], [8.0, 1.0, 5.0]], [[4.0, 2.0, 
1.0], [6.0, 8.0, 1.0], [2.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 4.0], [3.0, 2.0, 8.0], [6.0, 7.0, 8.0]], 
[[1.0, 6.0, 6.0], [2.0, 3.0, 2.0], [8.0, 7.0, 1.0]], [[5.0, 5.0, 2.0], [3.0, 8.0, 8.0], [3.0, 5.0, 3.0]]], [[[4.0, 2.0, 7.0], 
[8.0, 8.0, 4.0], [8.0, 8.0, 4.0]], [[3.0, 8.0, 7.0], [1.0, 4.0, 8.0], [8.0, 1.0, 3.0]], [[8.0, 5.0, 4.0], [5.0, 8.0, 2.0], 
[7.0, 8.0, 5.0]]], [[[2.0, 7.0, 8.0], [7.0, 8.0, 5.0], [6.0, 7.0, 5.0]], [[3.0, 6.0, 1.0], [1.0, 5.0, 8.0], [3.0, 6.0, 5.0]], 
[[7.0, 8.0, 6.0], [5.0, 3.0, 1.0], [6.0, 2.0, 3.0]]], [[[3.0, 2.0, 2.0], [7.0, 1.0, 1.0], [6.0, 3.0, 1.0]], [[7.0, 5.0, 2.0], 
[5.0, 5.0, 2.0], [2.0, 8.0, 6.0]], [[3.0, 8.0, 7.0], [4.0, 1.0, 2.0], [5.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRRN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 7.0], [5.0, 6.0, 4.0], [4.0, 3.0, 2.0]], [[6.0, 1.0, 6.0], [3.0, 2.0, 4.0], [2.0, 8.0, 
1.0]], [[6.0, 5.0, 3.0], [4.0, 7.0, 4.0], [8.0, 8.0, 1.0]]], [[[3.0, 6.0, 8.0], [6.0, 7.0, 8.0], [2.0, 2.0, 1.0]], [[3.0, 4.0, 
5.0], [5.0, 8.0, 7.0], [8.0, 4.0, 5.0]], [[5.0, 4.0, 1.0], [5.0, 1.0, 7.0], [4.0, 6.0, 8.0]]], [[[1.0, 2.0, 6.0], [5.0, 8.0, 
1.0], [2.0, 1.0, 4.0]], [[3.0, 7.0, 2.0], [3.0, 7.0, 6.0], [2.0, 6.0, 1.0]], [[6.0, 7.0, 4.0], [6.0, 6.0, 6.0], [3.0, 5.0, 
2.0]]], [[[7.0, 2.0, 5.0], [7.0, 1.0, 1.0], [6.0, 3.0, 4.0]], [[7.0, 7.0, 8.0], [4.0, 2.0, 7.0], [8.0, 4.0, 3.0]], [[4.0, 3.0, 
7.0], [4.0, 7.0, 6.0], [1.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 4.0], [5.0, 2.0, 7.0], [1.0, 4.0, 4.0]], [[2.0, 1.0, 7.0], [5.0, 3.0, 
1.0], [5.0, 5.0, 1.0]], [[3.0, 1.0, 2.0], [6.0, 6.0, 2.0], [7.0, 8.0, 5.0]]], [[[7.0, 3.0, 4.0], [1.0, 1.0, 6.0], [5.0, 1.0, 
1.0]], [[2.0, 2.0, 7.0], [6.0, 6.0, 7.0], [5.0, 7.0, 7.0]], [[3.0, 7.0, 1.0], [3.0, 3.0, 3.0], [3.0, 4.0, 7.0]]], [[[8.0, 1.0, 
2.0], [7.0, 4.0, 6.0], [4.0, 4.0, 3.0]], [[2.0, 8.0, 1.0], [1.0, 7.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 7.0, 1.0], [5.0, 4.0, 
5.0], [4.0, 7.0, 3.0]]], [[[2.0, 1.0, 5.0], [1.0, 8.0, 3.0], [5.0, 8.0, 3.0]], [[1.0, 8.0, 8.0], [3.0, 2.0, 7.0], [7.0, 8.0, 
6.0]], [[8.0, 5.0, 6.0], [2.0, 5.0, 1.0], [7.0, 4.0, 6.0]]]]))
      val=Data(numpy.array([[[[6.0, 2.0, 1.0]]], [[[5.0, 6.0, 3.0]]], [[[2.0, 8.0, 5.0]]], [[[3.0, 1.0, 
1.0]]]]),self.functionspace)
      arg[:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 7.0], [5.0, 6.0, 4.0], [4.0, 3.0, 2.0]], [[6.0, 1.0, 6.0], [6.0, 2.0, 1.0], [2.0, 
8.0, 1.0]], [[6.0, 5.0, 3.0], [4.0, 7.0, 4.0], [8.0, 8.0, 1.0]]], [[[3.0, 6.0, 8.0], [6.0, 7.0, 8.0], [2.0, 2.0, 1.0]], [[3.0, 
4.0, 5.0], [5.0, 6.0, 3.0], [8.0, 4.0, 5.0]], [[5.0, 4.0, 1.0], [5.0, 1.0, 7.0], [4.0, 6.0, 8.0]]], [[[1.0, 2.0, 6.0], [5.0, 
8.0, 1.0], [2.0, 1.0, 4.0]], [[3.0, 7.0, 2.0], [2.0, 8.0, 5.0], [2.0, 6.0, 1.0]], [[6.0, 7.0, 4.0], [6.0, 6.0, 6.0], [3.0, 5.0, 
2.0]]], [[[7.0, 2.0, 5.0], [7.0, 1.0, 1.0], [6.0, 3.0, 4.0]], [[7.0, 7.0, 8.0], [3.0, 1.0, 1.0], [8.0, 4.0, 3.0]], [[4.0, 3.0, 
7.0], [4.0, 7.0, 6.0], [1.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 4.0], [5.0, 2.0, 7.0], [1.0, 4.0, 4.0]], 
[[2.0, 1.0, 7.0], [6.0, 2.0, 1.0], [5.0, 5.0, 1.0]], [[3.0, 1.0, 2.0], [6.0, 6.0, 2.0], [7.0, 8.0, 5.0]]], [[[7.0, 3.0, 4.0], 
[1.0, 1.0, 6.0], [5.0, 1.0, 1.0]], [[2.0, 2.0, 7.0], [5.0, 6.0, 3.0], [5.0, 7.0, 7.0]], [[3.0, 7.0, 1.0], [3.0, 3.0, 3.0], 
[3.0, 4.0, 7.0]]], [[[8.0, 1.0, 2.0], [7.0, 4.0, 6.0], [4.0, 4.0, 3.0]], [[2.0, 8.0, 1.0], [2.0, 8.0, 5.0], [4.0, 7.0, 1.0]], 
[[8.0, 7.0, 1.0], [5.0, 4.0, 5.0], [4.0, 7.0, 3.0]]], [[[2.0, 1.0, 5.0], [1.0, 8.0, 3.0], [5.0, 8.0, 3.0]], [[1.0, 8.0, 8.0], 
[3.0, 1.0, 1.0], [7.0, 8.0, 6.0]], [[8.0, 5.0, 6.0], [2.0, 5.0, 1.0], [7.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRRN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 8.0], [7.0, 5.0, 5.0], [8.0, 8.0, 2.0]], [[7.0, 5.0, 5.0], [3.0, 2.0, 5.0], [4.0, 5.0, 
1.0]], [[2.0, 8.0, 4.0], [4.0, 6.0, 7.0], [4.0, 2.0, 4.0]]], [[[3.0, 1.0, 8.0], [7.0, 3.0, 1.0], [1.0, 7.0, 1.0]], [[6.0, 5.0, 
8.0], [8.0, 6.0, 8.0], [3.0, 6.0, 1.0]], [[8.0, 5.0, 8.0], [2.0, 6.0, 2.0], [1.0, 1.0, 4.0]]], [[[7.0, 3.0, 1.0], [4.0, 5.0, 
1.0], [3.0, 8.0, 4.0]], [[3.0, 7.0, 6.0], [1.0, 5.0, 8.0], [8.0, 8.0, 8.0]], [[5.0, 3.0, 7.0], [1.0, 8.0, 5.0], [5.0, 7.0, 
3.0]]], [[[6.0, 2.0, 1.0], [6.0, 6.0, 5.0], [7.0, 7.0, 4.0]], [[3.0, 1.0, 7.0], [1.0, 6.0, 6.0], [5.0, 8.0, 2.0]], [[7.0, 4.0, 
4.0], [8.0, 3.0, 6.0], [7.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 5.0], [3.0, 4.0, 7.0], [5.0, 3.0, 5.0]], [[1.0, 5.0, 8.0], [8.0, 6.0, 
7.0], [3.0, 5.0, 7.0]], [[4.0, 1.0, 8.0], [1.0, 2.0, 2.0], [1.0, 5.0, 6.0]]], [[[3.0, 7.0, 5.0], [5.0, 8.0, 6.0], [3.0, 8.0, 
7.0]], [[5.0, 4.0, 2.0], [5.0, 2.0, 3.0], [3.0, 8.0, 2.0]], [[4.0, 2.0, 2.0], [5.0, 6.0, 6.0], [2.0, 3.0, 2.0]]], [[[2.0, 8.0, 
7.0], [2.0, 2.0, 1.0], [2.0, 2.0, 4.0]], [[4.0, 3.0, 4.0], [6.0, 5.0, 8.0], [6.0, 1.0, 1.0]], [[6.0, 8.0, 8.0], [8.0, 7.0, 
3.0], [6.0, 1.0, 5.0]]], [[[8.0, 7.0, 4.0], [3.0, 6.0, 8.0], [8.0, 6.0, 6.0]], [[6.0, 3.0, 2.0], [3.0, 3.0, 3.0], [1.0, 5.0, 
6.0]], [[2.0, 1.0, 8.0], [1.0, 5.0, 8.0], [1.0, 6.0, 8.0]]]]))
      val=Data(numpy.array([[[[2.0, 3.0, 8.0]]], [[[8.0, 1.0, 4.0]]], [[[2.0, 1.0, 7.0]]], [[[3.0, 2.0, 
3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 2.0]]], [[[8.0, 7.0, 4.0]]], [[[5.0, 2.0, 4.0]]], [[[1.0, 4.0, 1.0]]]]))
      arg[:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 8.0], [7.0, 5.0, 5.0], [8.0, 8.0, 2.0]], [[7.0, 5.0, 5.0], [2.0, 3.0, 8.0], [4.0, 
5.0, 1.0]], [[2.0, 8.0, 4.0], [4.0, 6.0, 7.0], [4.0, 2.0, 4.0]]], [[[3.0, 1.0, 8.0], [7.0, 3.0, 1.0], [1.0, 7.0, 1.0]], [[6.0, 
5.0, 8.0], [8.0, 1.0, 4.0], [3.0, 6.0, 1.0]], [[8.0, 5.0, 8.0], [2.0, 6.0, 2.0], [1.0, 1.0, 4.0]]], [[[7.0, 3.0, 1.0], [4.0, 
5.0, 1.0], [3.0, 8.0, 4.0]], [[3.0, 7.0, 6.0], [2.0, 1.0, 7.0], [8.0, 8.0, 8.0]], [[5.0, 3.0, 7.0], [1.0, 8.0, 5.0], [5.0, 7.0, 
3.0]]], [[[6.0, 2.0, 1.0], [6.0, 6.0, 5.0], [7.0, 7.0, 4.0]], [[3.0, 1.0, 7.0], [3.0, 2.0, 3.0], [5.0, 8.0, 2.0]], [[7.0, 4.0, 
4.0], [8.0, 3.0, 6.0], [7.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 5.0], [3.0, 4.0, 7.0], [5.0, 3.0, 5.0]], 
[[1.0, 5.0, 8.0], [5.0, 2.0, 2.0], [3.0, 5.0, 7.0]], [[4.0, 1.0, 8.0], [1.0, 2.0, 2.0], [1.0, 5.0, 6.0]]], [[[3.0, 7.0, 5.0], 
[5.0, 8.0, 6.0], [3.0, 8.0, 7.0]], [[5.0, 4.0, 2.0], [8.0, 7.0, 4.0], [3.0, 8.0, 2.0]], [[4.0, 2.0, 2.0], [5.0, 6.0, 6.0], 
[2.0, 3.0, 2.0]]], [[[2.0, 8.0, 7.0], [2.0, 2.0, 1.0], [2.0, 2.0, 4.0]], [[4.0, 3.0, 4.0], [5.0, 2.0, 4.0], [6.0, 1.0, 1.0]], 
[[6.0, 8.0, 8.0], [8.0, 7.0, 3.0], [6.0, 1.0, 5.0]]], [[[8.0, 7.0, 4.0], [3.0, 6.0, 8.0], [8.0, 6.0, 6.0]], [[6.0, 3.0, 2.0], 
[1.0, 4.0, 1.0], [1.0, 5.0, 6.0]], [[2.0, 1.0, 8.0], [1.0, 5.0, 8.0], [1.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRRN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 5.0], [5.0, 6.0, 6.0], [1.0, 2.0, 7.0]], [[7.0, 4.0, 7.0], [5.0, 7.0, 1.0], [4.0, 3.0, 
7.0]], [[8.0, 1.0, 1.0], [7.0, 8.0, 2.0], [7.0, 8.0, 8.0]]], [[[1.0, 1.0, 8.0], [5.0, 6.0, 5.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 
3.0], [5.0, 2.0, 6.0], [5.0, 2.0, 3.0]], [[3.0, 3.0, 2.0], [7.0, 7.0, 5.0], [8.0, 8.0, 2.0]]], [[[4.0, 7.0, 3.0], [7.0, 8.0, 
3.0], [4.0, 1.0, 5.0]], [[2.0, 8.0, 7.0], [8.0, 5.0, 3.0], [2.0, 8.0, 7.0]], [[4.0, 6.0, 2.0], [7.0, 7.0, 4.0], [1.0, 3.0, 
7.0]]], [[[7.0, 7.0, 1.0], [6.0, 1.0, 3.0], [5.0, 5.0, 2.0]], [[2.0, 2.0, 8.0], [5.0, 1.0, 5.0], [2.0, 7.0, 4.0]], [[8.0, 4.0, 
3.0], [2.0, 1.0, 6.0], [6.0, 8.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 3.0], [8.0, 8.0, 8.0], [8.0, 5.0, 2.0]], [[1.0, 5.0, 5.0], [4.0, 3.0, 
3.0], [5.0, 6.0, 1.0]], [[8.0, 3.0, 7.0], [7.0, 4.0, 4.0], [7.0, 1.0, 8.0]]], [[[5.0, 6.0, 8.0], [4.0, 1.0, 3.0], [7.0, 5.0, 
6.0]], [[7.0, 1.0, 6.0], [2.0, 6.0, 7.0], [5.0, 6.0, 1.0]], [[3.0, 5.0, 3.0], [3.0, 7.0, 1.0], [7.0, 8.0, 2.0]]], [[[4.0, 7.0, 
3.0], [5.0, 4.0, 3.0], [1.0, 1.0, 1.0]], [[4.0, 1.0, 5.0], [6.0, 8.0, 1.0], [1.0, 7.0, 4.0]], [[6.0, 8.0, 5.0], [3.0, 2.0, 
6.0], [6.0, 8.0, 5.0]]], [[[1.0, 3.0, 5.0], [3.0, 8.0, 2.0], [6.0, 4.0, 1.0]], [[1.0, 2.0, 4.0], [7.0, 5.0, 1.0], [4.0, 6.0, 
5.0]], [[7.0, 8.0, 2.0], [8.0, 3.0, 3.0], [2.0, 1.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 1.0, 8.0]]], [[[3.0, 7.0, 5.0]]], [[[1.0, 2.0, 8.0]]], [[[6.0, 7.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 3.0]]], [[[6.0, 7.0, 1.0]]], [[[7.0, 1.0, 3.0]]], [[[8.0, 5.0, 4.0]]]])
      arg[:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 5.0], [5.0, 6.0, 6.0], [1.0, 2.0, 7.0]], [[7.0, 4.0, 7.0], [5.0, 1.0, 8.0], [4.0, 
3.0, 7.0]], [[8.0, 1.0, 1.0], [7.0, 8.0, 2.0], [7.0, 8.0, 8.0]]], [[[1.0, 1.0, 8.0], [5.0, 6.0, 5.0], [3.0, 3.0, 8.0]], [[3.0, 
8.0, 3.0], [3.0, 7.0, 5.0], [5.0, 2.0, 3.0]], [[3.0, 3.0, 2.0], [7.0, 7.0, 5.0], [8.0, 8.0, 2.0]]], [[[4.0, 7.0, 3.0], [7.0, 
8.0, 3.0], [4.0, 1.0, 5.0]], [[2.0, 8.0, 7.0], [1.0, 2.0, 8.0], [2.0, 8.0, 7.0]], [[4.0, 6.0, 2.0], [7.0, 7.0, 4.0], [1.0, 3.0, 
7.0]]], [[[7.0, 7.0, 1.0], [6.0, 1.0, 3.0], [5.0, 5.0, 2.0]], [[2.0, 2.0, 8.0], [6.0, 7.0, 2.0], [2.0, 7.0, 4.0]], [[8.0, 4.0, 
3.0], [2.0, 1.0, 6.0], [6.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 3.0], [8.0, 8.0, 8.0], [8.0, 5.0, 2.0]], 
[[1.0, 5.0, 5.0], [5.0, 4.0, 3.0], [5.0, 6.0, 1.0]], [[8.0, 3.0, 7.0], [7.0, 4.0, 4.0], [7.0, 1.0, 8.0]]], [[[5.0, 6.0, 8.0], 
[4.0, 1.0, 3.0], [7.0, 5.0, 6.0]], [[7.0, 1.0, 6.0], [6.0, 7.0, 1.0], [5.0, 6.0, 1.0]], [[3.0, 5.0, 3.0], [3.0, 7.0, 1.0], 
[7.0, 8.0, 2.0]]], [[[4.0, 7.0, 3.0], [5.0, 4.0, 3.0], [1.0, 1.0, 1.0]], [[4.0, 1.0, 5.0], [7.0, 1.0, 3.0], [1.0, 7.0, 4.0]], 
[[6.0, 8.0, 5.0], [3.0, 2.0, 6.0], [6.0, 8.0, 5.0]]], [[[1.0, 3.0, 5.0], [3.0, 8.0, 2.0], [6.0, 4.0, 1.0]], [[1.0, 2.0, 4.0], 
[8.0, 5.0, 4.0], [4.0, 6.0, 5.0]], [[7.0, 8.0, 2.0], [8.0, 3.0, 3.0], [2.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CPRN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 3.0], [7.0, 1.0, 7.0], [7.0, 2.0, 6.0]], [[7.0, 7.0, 1.0], [6.0, 4.0, 6.0], [1.0, 8.0, 
1.0]], [[8.0, 6.0, 3.0], [6.0, 5.0, 3.0], [4.0, 2.0, 8.0]]], [[[2.0, 2.0, 5.0], [3.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[6.0, 2.0, 
4.0], [3.0, 5.0, 5.0], [1.0, 7.0, 8.0]], [[3.0, 6.0, 8.0], [2.0, 4.0, 4.0], [1.0, 2.0, 2.0]]], [[[5.0, 2.0, 5.0], [4.0, 4.0, 
3.0], [7.0, 2.0, 2.0]], [[3.0, 1.0, 6.0], [5.0, 3.0, 5.0], [2.0, 2.0, 2.0]], [[7.0, 3.0, 1.0], [8.0, 6.0, 6.0], [3.0, 8.0, 
8.0]]], [[[1.0, 3.0, 7.0], [1.0, 5.0, 2.0], [6.0, 8.0, 2.0]], [[7.0, 2.0, 1.0], [2.0, 4.0, 3.0], [2.0, 4.0, 1.0]], [[4.0, 3.0, 
1.0], [8.0, 8.0, 6.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 8.0], [1.0, 3.0, 7.0], [7.0, 6.0, 2.0]], [[3.0, 6.0, 5.0], [7.0, 4.0, 
7.0], [4.0, 8.0, 1.0]], [[5.0, 6.0, 8.0], [6.0, 1.0, 1.0], [5.0, 8.0, 7.0]]], [[[8.0, 2.0, 3.0], [1.0, 6.0, 7.0], [4.0, 3.0, 
4.0]], [[3.0, 2.0, 6.0], [4.0, 4.0, 6.0], [1.0, 7.0, 6.0]], [[7.0, 8.0, 6.0], [7.0, 8.0, 7.0], [7.0, 7.0, 6.0]]], [[[4.0, 3.0, 
3.0], [5.0, 3.0, 2.0], [3.0, 8.0, 4.0]], [[6.0, 1.0, 3.0], [2.0, 5.0, 4.0], [2.0, 8.0, 6.0]], [[6.0, 6.0, 1.0], [5.0, 8.0, 
4.0], [6.0, 2.0, 5.0]]], [[[3.0, 6.0, 1.0], [2.0, 4.0, 1.0], [4.0, 5.0, 1.0]], [[4.0, 4.0, 6.0], [4.0, 4.0, 4.0], [8.0, 7.0, 
3.0]], [[3.0, 2.0, 2.0], [8.0, 2.0, 4.0], [1.0, 3.0, 1.0]]]]))
      val=numpy.array([[[3.0, 2.0, 6.0]], [[8.0, 4.0, 7.0]], [[4.0, 4.0, 8.0]], [[8.0, 7.0, 3.0]]])
      arg[:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 3.0], [7.0, 1.0, 7.0], [7.0, 2.0, 6.0]], [[7.0, 7.0, 1.0], [3.0, 2.0, 6.0], [1.0, 
8.0, 1.0]], [[8.0, 6.0, 3.0], [6.0, 5.0, 3.0], [4.0, 2.0, 8.0]]], [[[2.0, 2.0, 5.0], [3.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[6.0, 
2.0, 4.0], [8.0, 4.0, 7.0], [1.0, 7.0, 8.0]], [[3.0, 6.0, 8.0], [2.0, 4.0, 4.0], [1.0, 2.0, 2.0]]], [[[5.0, 2.0, 5.0], [4.0, 
4.0, 3.0], [7.0, 2.0, 2.0]], [[3.0, 1.0, 6.0], [4.0, 4.0, 8.0], [2.0, 2.0, 2.0]], [[7.0, 3.0, 1.0], [8.0, 6.0, 6.0], [3.0, 8.0, 
8.0]]], [[[1.0, 3.0, 7.0], [1.0, 5.0, 2.0], [6.0, 8.0, 2.0]], [[7.0, 2.0, 1.0], [8.0, 7.0, 3.0], [2.0, 4.0, 1.0]], [[4.0, 3.0, 
1.0], [8.0, 8.0, 6.0], [8.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 8.0], [1.0, 3.0, 7.0], [7.0, 6.0, 2.0]], 
[[3.0, 6.0, 5.0], [3.0, 2.0, 6.0], [4.0, 8.0, 1.0]], [[5.0, 6.0, 8.0], [6.0, 1.0, 1.0], [5.0, 8.0, 7.0]]], [[[8.0, 2.0, 3.0], 
[1.0, 6.0, 7.0], [4.0, 3.0, 4.0]], [[3.0, 2.0, 6.0], [8.0, 4.0, 7.0], [1.0, 7.0, 6.0]], [[7.0, 8.0, 6.0], [7.0, 8.0, 7.0], 
[7.0, 7.0, 6.0]]], [[[4.0, 3.0, 3.0], [5.0, 3.0, 2.0], [3.0, 8.0, 4.0]], [[6.0, 1.0, 3.0], [4.0, 4.0, 8.0], [2.0, 8.0, 6.0]], 
[[6.0, 6.0, 1.0], [5.0, 8.0, 4.0], [6.0, 2.0, 5.0]]], [[[3.0, 6.0, 1.0], [2.0, 4.0, 1.0], [4.0, 5.0, 1.0]], [[4.0, 4.0, 6.0], 
[8.0, 7.0, 3.0], [8.0, 7.0, 3.0]], [[3.0, 2.0, 2.0], [8.0, 2.0, 4.0], [1.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CPRN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 7.0], [4.0, 3.0, 4.0], [7.0, 4.0, 8.0]], [[7.0, 2.0, 4.0], [6.0, 4.0, 7.0], [5.0, 3.0, 
3.0]], [[4.0, 3.0, 4.0], [4.0, 5.0, 6.0], [1.0, 4.0, 6.0]]], [[[3.0, 2.0, 8.0], [4.0, 8.0, 8.0], [5.0, 5.0, 6.0]], [[1.0, 8.0, 
5.0], [7.0, 2.0, 6.0], [4.0, 7.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 7.0, 5.0], [2.0, 4.0, 8.0]]], [[[5.0, 2.0, 4.0], [5.0, 4.0, 
6.0], [5.0, 2.0, 2.0]], [[8.0, 4.0, 3.0], [6.0, 7.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 5.0, 5.0], [5.0, 5.0, 4.0], [1.0, 6.0, 
1.0]]], [[[5.0, 7.0, 8.0], [6.0, 2.0, 3.0], [2.0, 6.0, 6.0]], [[1.0, 8.0, 5.0], [6.0, 6.0, 2.0], [6.0, 4.0, 3.0]], [[7.0, 7.0, 
1.0], [4.0, 3.0, 7.0], [1.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 3.0], [1.0, 2.0, 3.0], [2.0, 2.0, 6.0]], [[5.0, 8.0, 6.0], [5.0, 1.0, 
1.0], [8.0, 3.0, 8.0]], [[8.0, 6.0, 7.0], [6.0, 4.0, 4.0], [6.0, 5.0, 7.0]]], [[[4.0, 6.0, 2.0], [5.0, 2.0, 8.0], [1.0, 1.0, 
8.0]], [[1.0, 5.0, 6.0], [5.0, 3.0, 8.0], [7.0, 5.0, 7.0]], [[3.0, 6.0, 5.0], [5.0, 2.0, 8.0], [4.0, 5.0, 4.0]]], [[[2.0, 8.0, 
6.0], [4.0, 1.0, 7.0], [5.0, 6.0, 4.0]], [[8.0, 6.0, 8.0], [1.0, 1.0, 8.0], [6.0, 3.0, 1.0]], [[2.0, 3.0, 6.0], [5.0, 8.0, 
2.0], [8.0, 1.0, 4.0]]], [[[5.0, 5.0, 3.0], [1.0, 3.0, 5.0], [6.0, 4.0, 2.0]], [[3.0, 7.0, 2.0], [4.0, 8.0, 7.0], [2.0, 1.0, 
8.0]], [[7.0, 5.0, 3.0], [6.0, 5.0, 5.0], [5.0, 8.0, 6.0]]]]))
      val=Data(numpy.array([[[6.0, 6.0, 7.0]], [[4.0, 5.0, 3.0]], [[5.0, 3.0, 6.0]], [[1.0, 1.0, 1.0]]]),self.functionspace)
      arg[:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 7.0], [4.0, 3.0, 4.0], [7.0, 4.0, 8.0]], [[7.0, 2.0, 4.0], [6.0, 6.0, 7.0], [5.0, 
3.0, 3.0]], [[4.0, 3.0, 4.0], [4.0, 5.0, 6.0], [1.0, 4.0, 6.0]]], [[[3.0, 2.0, 8.0], [4.0, 8.0, 8.0], [5.0, 5.0, 6.0]], [[1.0, 
8.0, 5.0], [4.0, 5.0, 3.0], [4.0, 7.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 7.0, 5.0], [2.0, 4.0, 8.0]]], [[[5.0, 2.0, 4.0], [5.0, 
4.0, 6.0], [5.0, 2.0, 2.0]], [[8.0, 4.0, 3.0], [5.0, 3.0, 6.0], [5.0, 7.0, 3.0]], [[7.0, 5.0, 5.0], [5.0, 5.0, 4.0], [1.0, 6.0, 
1.0]]], [[[5.0, 7.0, 8.0], [6.0, 2.0, 3.0], [2.0, 6.0, 6.0]], [[1.0, 8.0, 5.0], [1.0, 1.0, 1.0], [6.0, 4.0, 3.0]], [[7.0, 7.0, 
1.0], [4.0, 3.0, 7.0], [1.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 3.0], [1.0, 2.0, 3.0], [2.0, 2.0, 6.0]], 
[[5.0, 8.0, 6.0], [6.0, 6.0, 7.0], [8.0, 3.0, 8.0]], [[8.0, 6.0, 7.0], [6.0, 4.0, 4.0], [6.0, 5.0, 7.0]]], [[[4.0, 6.0, 2.0], 
[5.0, 2.0, 8.0], [1.0, 1.0, 8.0]], [[1.0, 5.0, 6.0], [4.0, 5.0, 3.0], [7.0, 5.0, 7.0]], [[3.0, 6.0, 5.0], [5.0, 2.0, 8.0], 
[4.0, 5.0, 4.0]]], [[[2.0, 8.0, 6.0], [4.0, 1.0, 7.0], [5.0, 6.0, 4.0]], [[8.0, 6.0, 8.0], [5.0, 3.0, 6.0], [6.0, 3.0, 1.0]], 
[[2.0, 3.0, 6.0], [5.0, 8.0, 2.0], [8.0, 1.0, 4.0]]], [[[5.0, 5.0, 3.0], [1.0, 3.0, 5.0], [6.0, 4.0, 2.0]], [[3.0, 7.0, 2.0], 
[1.0, 1.0, 1.0], [2.0, 1.0, 8.0]], [[7.0, 5.0, 3.0], [6.0, 5.0, 5.0], [5.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CPRN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 8.0], [4.0, 8.0, 4.0], [2.0, 6.0, 7.0]], [[5.0, 8.0, 7.0], [5.0, 8.0, 5.0], [6.0, 6.0, 
8.0]], [[5.0, 7.0, 3.0], [8.0, 6.0, 1.0], [8.0, 1.0, 4.0]]], [[[5.0, 4.0, 8.0], [3.0, 2.0, 6.0], [2.0, 1.0, 1.0]], [[5.0, 2.0, 
4.0], [5.0, 6.0, 8.0], [6.0, 3.0, 3.0]], [[7.0, 7.0, 2.0], [1.0, 5.0, 4.0], [8.0, 7.0, 1.0]]], [[[7.0, 5.0, 2.0], [3.0, 1.0, 
1.0], [8.0, 1.0, 6.0]], [[6.0, 7.0, 5.0], [2.0, 5.0, 5.0], [7.0, 7.0, 5.0]], [[5.0, 6.0, 3.0], [3.0, 8.0, 3.0], [7.0, 3.0, 
6.0]]], [[[7.0, 5.0, 3.0], [2.0, 5.0, 4.0], [3.0, 4.0, 6.0]], [[2.0, 8.0, 4.0], [4.0, 5.0, 1.0], [7.0, 8.0, 6.0]], [[3.0, 3.0, 
3.0], [2.0, 2.0, 6.0], [6.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 2.0], [1.0, 3.0, 7.0], [7.0, 4.0, 2.0]], [[3.0, 4.0, 7.0], [6.0, 8.0, 
2.0], [2.0, 5.0, 2.0]], [[2.0, 7.0, 6.0], [4.0, 2.0, 5.0], [6.0, 8.0, 5.0]]], [[[6.0, 5.0, 6.0], [4.0, 3.0, 8.0], [1.0, 8.0, 
7.0]], [[5.0, 4.0, 6.0], [4.0, 1.0, 4.0], [5.0, 1.0, 6.0]], [[1.0, 2.0, 6.0], [1.0, 7.0, 4.0], [3.0, 3.0, 2.0]]], [[[7.0, 6.0, 
3.0], [2.0, 1.0, 4.0], [5.0, 2.0, 4.0]], [[4.0, 2.0, 1.0], [1.0, 7.0, 3.0], [3.0, 8.0, 5.0]], [[8.0, 1.0, 4.0], [7.0, 7.0, 
1.0], [1.0, 6.0, 7.0]]], [[[7.0, 7.0, 2.0], [6.0, 4.0, 5.0], [4.0, 5.0, 8.0]], [[1.0, 6.0, 5.0], [6.0, 4.0, 5.0], [3.0, 5.0, 
2.0]], [[4.0, 8.0, 1.0], [7.0, 8.0, 7.0], [3.0, 5.0, 7.0]]]]))
      val=Data(numpy.array([[[1.0, 1.0, 3.0]], [[1.0, 1.0, 5.0]], [[7.0, 8.0, 4.0]], [[2.0, 5.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 7.0, 6.0]], [[1.0, 1.0, 1.0]], [[6.0, 1.0, 2.0]], [[4.0, 5.0, 2.0]]]))
      arg[:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 8.0], [4.0, 8.0, 4.0], [2.0, 6.0, 7.0]], [[5.0, 8.0, 7.0], [1.0, 1.0, 3.0], [6.0, 
6.0, 8.0]], [[5.0, 7.0, 3.0], [8.0, 6.0, 1.0], [8.0, 1.0, 4.0]]], [[[5.0, 4.0, 8.0], [3.0, 2.0, 6.0], [2.0, 1.0, 1.0]], [[5.0, 
2.0, 4.0], [1.0, 1.0, 5.0], [6.0, 3.0, 3.0]], [[7.0, 7.0, 2.0], [1.0, 5.0, 4.0], [8.0, 7.0, 1.0]]], [[[7.0, 5.0, 2.0], [3.0, 
1.0, 1.0], [8.0, 1.0, 6.0]], [[6.0, 7.0, 5.0], [7.0, 8.0, 4.0], [7.0, 7.0, 5.0]], [[5.0, 6.0, 3.0], [3.0, 8.0, 3.0], [7.0, 3.0, 
6.0]]], [[[7.0, 5.0, 3.0], [2.0, 5.0, 4.0], [3.0, 4.0, 6.0]], [[2.0, 8.0, 4.0], [2.0, 5.0, 3.0], [7.0, 8.0, 6.0]], [[3.0, 3.0, 
3.0], [2.0, 2.0, 6.0], [6.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 2.0], [1.0, 3.0, 7.0], [7.0, 4.0, 2.0]], 
[[3.0, 4.0, 7.0], [3.0, 7.0, 6.0], [2.0, 5.0, 2.0]], [[2.0, 7.0, 6.0], [4.0, 2.0, 5.0], [6.0, 8.0, 5.0]]], [[[6.0, 5.0, 6.0], 
[4.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[5.0, 4.0, 6.0], [1.0, 1.0, 1.0], [5.0, 1.0, 6.0]], [[1.0, 2.0, 6.0], [1.0, 7.0, 4.0], 
[3.0, 3.0, 2.0]]], [[[7.0, 6.0, 3.0], [2.0, 1.0, 4.0], [5.0, 2.0, 4.0]], [[4.0, 2.0, 1.0], [6.0, 1.0, 2.0], [3.0, 8.0, 5.0]], 
[[8.0, 1.0, 4.0], [7.0, 7.0, 1.0], [1.0, 6.0, 7.0]]], [[[7.0, 7.0, 2.0], [6.0, 4.0, 5.0], [4.0, 5.0, 8.0]], [[1.0, 6.0, 5.0], 
[4.0, 5.0, 2.0], [3.0, 5.0, 2.0]], [[4.0, 8.0, 1.0], [7.0, 8.0, 7.0], [3.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CPRN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 6.0], [7.0, 6.0, 4.0], [8.0, 2.0, 2.0]], [[1.0, 2.0, 5.0], [8.0, 1.0, 1.0], [5.0, 6.0, 
4.0]], [[6.0, 2.0, 5.0], [5.0, 4.0, 4.0], [2.0, 6.0, 3.0]]], [[[7.0, 4.0, 3.0], [3.0, 3.0, 5.0], [8.0, 1.0, 5.0]], [[1.0, 8.0, 
3.0], [4.0, 5.0, 5.0], [8.0, 3.0, 2.0]], [[3.0, 6.0, 4.0], [3.0, 5.0, 1.0], [1.0, 2.0, 2.0]]], [[[6.0, 3.0, 2.0], [3.0, 1.0, 
7.0], [7.0, 2.0, 8.0]], [[1.0, 6.0, 1.0], [4.0, 3.0, 3.0], [6.0, 7.0, 6.0]], [[5.0, 7.0, 3.0], [8.0, 8.0, 5.0], [3.0, 4.0, 
1.0]]], [[[2.0, 6.0, 7.0], [7.0, 1.0, 4.0], [8.0, 4.0, 8.0]], [[2.0, 3.0, 1.0], [3.0, 1.0, 7.0], [4.0, 6.0, 1.0]], [[8.0, 3.0, 
8.0], [8.0, 2.0, 7.0], [4.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 6.0], [3.0, 2.0, 7.0], [2.0, 7.0, 4.0]], [[3.0, 5.0, 1.0], [4.0, 5.0, 
5.0], [8.0, 5.0, 4.0]], [[1.0, 2.0, 8.0], [6.0, 4.0, 8.0], [6.0, 1.0, 4.0]]], [[[1.0, 6.0, 3.0], [3.0, 8.0, 5.0], [4.0, 6.0, 
1.0]], [[2.0, 8.0, 1.0], [3.0, 6.0, 8.0], [4.0, 4.0, 6.0]], [[5.0, 2.0, 2.0], [2.0, 6.0, 8.0], [3.0, 2.0, 8.0]]], [[[4.0, 5.0, 
6.0], [4.0, 7.0, 4.0], [2.0, 6.0, 1.0]], [[8.0, 2.0, 8.0], [3.0, 4.0, 8.0], [2.0, 3.0, 4.0]], [[6.0, 6.0, 2.0], [3.0, 6.0, 
8.0], [2.0, 4.0, 6.0]]], [[[5.0, 3.0, 2.0], [2.0, 1.0, 4.0], [7.0, 7.0, 6.0]], [[2.0, 1.0, 3.0], [5.0, 8.0, 7.0], [4.0, 3.0, 
7.0]], [[5.0, 4.0, 8.0], [2.0, 6.0, 3.0], [3.0, 4.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[2.0, 7.0, 6.0]], [[7.0, 7.0, 1.0]], [[4.0, 8.0, 7.0]], [[2.0, 8.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[3.0, 3.0, 5.0]], [[4.0, 5.0, 1.0]], [[4.0, 5.0, 8.0]], [[4.0, 2.0, 3.0]]])
      arg[:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 6.0], [7.0, 6.0, 4.0], [8.0, 2.0, 2.0]], [[1.0, 2.0, 5.0], [2.0, 7.0, 6.0], [5.0, 
6.0, 4.0]], [[6.0, 2.0, 5.0], [5.0, 4.0, 4.0], [2.0, 6.0, 3.0]]], [[[7.0, 4.0, 3.0], [3.0, 3.0, 5.0], [8.0, 1.0, 5.0]], [[1.0, 
8.0, 3.0], [7.0, 7.0, 1.0], [8.0, 3.0, 2.0]], [[3.0, 6.0, 4.0], [3.0, 5.0, 1.0], [1.0, 2.0, 2.0]]], [[[6.0, 3.0, 2.0], [3.0, 
1.0, 7.0], [7.0, 2.0, 8.0]], [[1.0, 6.0, 1.0], [4.0, 8.0, 7.0], [6.0, 7.0, 6.0]], [[5.0, 7.0, 3.0], [8.0, 8.0, 5.0], [3.0, 4.0, 
1.0]]], [[[2.0, 6.0, 7.0], [7.0, 1.0, 4.0], [8.0, 4.0, 8.0]], [[2.0, 3.0, 1.0], [2.0, 8.0, 4.0], [4.0, 6.0, 1.0]], [[8.0, 3.0, 
8.0], [8.0, 2.0, 7.0], [4.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 6.0], [3.0, 2.0, 7.0], [2.0, 7.0, 4.0]], 
[[3.0, 5.0, 1.0], [3.0, 3.0, 5.0], [8.0, 5.0, 4.0]], [[1.0, 2.0, 8.0], [6.0, 4.0, 8.0], [6.0, 1.0, 4.0]]], [[[1.0, 6.0, 3.0], 
[3.0, 8.0, 5.0], [4.0, 6.0, 1.0]], [[2.0, 8.0, 1.0], [4.0, 5.0, 1.0], [4.0, 4.0, 6.0]], [[5.0, 2.0, 2.0], [2.0, 6.0, 8.0], 
[3.0, 2.0, 8.0]]], [[[4.0, 5.0, 6.0], [4.0, 7.0, 4.0], [2.0, 6.0, 1.0]], [[8.0, 2.0, 8.0], [4.0, 5.0, 8.0], [2.0, 3.0, 4.0]], 
[[6.0, 6.0, 2.0], [3.0, 6.0, 8.0], [2.0, 4.0, 6.0]]], [[[5.0, 3.0, 2.0], [2.0, 1.0, 4.0], [7.0, 7.0, 6.0]], [[2.0, 1.0, 3.0], 
[4.0, 2.0, 3.0], [4.0, 3.0, 7.0]], [[5.0, 4.0, 8.0], [2.0, 6.0, 3.0], [3.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_URUN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 5.0], [1.0, 5.0, 4.0], [8.0, 6.0, 2.0]], [[4.0, 8.0, 5.0], [3.0, 5.0, 3.0], [2.0, 2.0, 
6.0]], [[3.0, 2.0, 7.0], [3.0, 1.0, 5.0], [8.0, 6.0, 6.0]]], [[[1.0, 5.0, 5.0], [1.0, 4.0, 6.0], [5.0, 7.0, 3.0]], [[8.0, 3.0, 
6.0], [8.0, 4.0, 8.0], [6.0, 7.0, 4.0]], [[6.0, 8.0, 1.0], [4.0, 4.0, 7.0], [4.0, 1.0, 3.0]]], [[[5.0, 5.0, 8.0], [3.0, 8.0, 
1.0], [6.0, 6.0, 4.0]], [[5.0, 2.0, 6.0], [2.0, 4.0, 4.0], [5.0, 8.0, 5.0]], [[5.0, 5.0, 4.0], [2.0, 3.0, 8.0], [5.0, 5.0, 
5.0]]], [[[1.0, 6.0, 5.0], [2.0, 2.0, 1.0], [4.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [8.0, 1.0, 3.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 
5.0], [2.0, 2.0, 1.0], [8.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 1.0], [2.0, 3.0, 4.0], [1.0, 6.0, 7.0]], [[2.0, 8.0, 2.0], [3.0, 4.0, 
4.0], [4.0, 6.0, 8.0]], [[6.0, 1.0, 7.0], [6.0, 6.0, 5.0], [8.0, 8.0, 2.0]]], [[[7.0, 5.0, 5.0], [1.0, 3.0, 4.0], [2.0, 3.0, 
8.0]], [[2.0, 2.0, 3.0], [6.0, 6.0, 2.0], [2.0, 2.0, 3.0]], [[7.0, 2.0, 4.0], [8.0, 8.0, 4.0], [4.0, 1.0, 1.0]]], [[[5.0, 6.0, 
4.0], [2.0, 3.0, 4.0], [1.0, 2.0, 6.0]], [[2.0, 7.0, 2.0], [7.0, 5.0, 5.0], [5.0, 5.0, 8.0]], [[4.0, 1.0, 7.0], [3.0, 5.0, 
5.0], [5.0, 6.0, 7.0]]], [[[1.0, 8.0, 8.0], [5.0, 2.0, 5.0], [2.0, 2.0, 6.0]], [[1.0, 7.0, 3.0], [3.0, 3.0, 1.0], [2.0, 1.0, 
7.0]], [[5.0, 3.0, 8.0], [2.0, 4.0, 8.0], [2.0, 2.0, 7.0]]]]))
      val=numpy.array([[[[8.0, 4.0, 6.0], [8.0, 2.0, 5.0]]], [[[6.0, 7.0, 2.0], [1.0, 1.0, 7.0]]], [[[7.0, 6.0, 7.0], [7.0, 
1.0, 1.0]]]])
      arg[:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 5.0], [1.0, 5.0, 4.0], [8.0, 6.0, 2.0]], [[8.0, 4.0, 6.0], [8.0, 2.0, 5.0], [2.0, 
2.0, 6.0]], [[3.0, 2.0, 7.0], [3.0, 1.0, 5.0], [8.0, 6.0, 6.0]]], [[[1.0, 5.0, 5.0], [1.0, 4.0, 6.0], [5.0, 7.0, 3.0]], [[6.0, 
7.0, 2.0], [1.0, 1.0, 7.0], [6.0, 7.0, 4.0]], [[6.0, 8.0, 1.0], [4.0, 4.0, 7.0], [4.0, 1.0, 3.0]]], [[[5.0, 5.0, 8.0], [3.0, 
8.0, 1.0], [6.0, 6.0, 4.0]], [[7.0, 6.0, 7.0], [7.0, 1.0, 1.0], [5.0, 8.0, 5.0]], [[5.0, 5.0, 4.0], [2.0, 3.0, 8.0], [5.0, 5.0, 
5.0]]], [[[1.0, 6.0, 5.0], [2.0, 2.0, 1.0], [4.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [8.0, 1.0, 3.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 
5.0], [2.0, 2.0, 1.0], [8.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 1.0], [2.0, 3.0, 4.0], [1.0, 6.0, 7.0]], 
[[8.0, 4.0, 6.0], [8.0, 2.0, 5.0], [4.0, 6.0, 8.0]], [[6.0, 1.0, 7.0], [6.0, 6.0, 5.0], [8.0, 8.0, 2.0]]], [[[7.0, 5.0, 5.0], 
[1.0, 3.0, 4.0], [2.0, 3.0, 8.0]], [[6.0, 7.0, 2.0], [1.0, 1.0, 7.0], [2.0, 2.0, 3.0]], [[7.0, 2.0, 4.0], [8.0, 8.0, 4.0], 
[4.0, 1.0, 1.0]]], [[[5.0, 6.0, 4.0], [2.0, 3.0, 4.0], [1.0, 2.0, 6.0]], [[7.0, 6.0, 7.0], [7.0, 1.0, 1.0], [5.0, 5.0, 8.0]], 
[[4.0, 1.0, 7.0], [3.0, 5.0, 5.0], [5.0, 6.0, 7.0]]], [[[1.0, 8.0, 8.0], [5.0, 2.0, 5.0], [2.0, 2.0, 6.0]], [[1.0, 7.0, 3.0], 
[3.0, 3.0, 1.0], [2.0, 1.0, 7.0]], [[5.0, 3.0, 8.0], [2.0, 4.0, 8.0], [2.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_URUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 1.0], [2.0, 7.0, 7.0], [1.0, 4.0, 3.0]], [[5.0, 1.0, 8.0], [2.0, 4.0, 4.0], [5.0, 1.0, 
1.0]], [[2.0, 1.0, 7.0], [5.0, 2.0, 4.0], [1.0, 5.0, 4.0]]], [[[7.0, 2.0, 5.0], [8.0, 1.0, 7.0], [1.0, 5.0, 1.0]], [[8.0, 5.0, 
5.0], [4.0, 5.0, 8.0], [2.0, 1.0, 6.0]], [[2.0, 1.0, 2.0], [6.0, 5.0, 5.0], [8.0, 5.0, 7.0]]], [[[8.0, 7.0, 6.0], [8.0, 2.0, 
8.0], [2.0, 4.0, 3.0]], [[8.0, 5.0, 6.0], [2.0, 7.0, 1.0], [1.0, 8.0, 7.0]], [[1.0, 8.0, 1.0], [6.0, 1.0, 3.0], [7.0, 2.0, 
3.0]]], [[[7.0, 6.0, 2.0], [6.0, 4.0, 3.0], [2.0, 7.0, 8.0]], [[5.0, 2.0, 2.0], [7.0, 4.0, 5.0], [5.0, 2.0, 2.0]], [[4.0, 6.0, 
7.0], [5.0, 5.0, 6.0], [1.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 7.0], [6.0, 1.0, 5.0], [2.0, 2.0, 5.0]], [[4.0, 8.0, 5.0], [1.0, 6.0, 
8.0], [8.0, 5.0, 7.0]], [[7.0, 8.0, 6.0], [7.0, 3.0, 5.0], [1.0, 1.0, 3.0]]], [[[7.0, 4.0, 1.0], [3.0, 7.0, 1.0], [8.0, 4.0, 
5.0]], [[2.0, 7.0, 6.0], [5.0, 7.0, 3.0], [2.0, 4.0, 3.0]], [[4.0, 6.0, 5.0], [5.0, 3.0, 2.0], [8.0, 3.0, 5.0]]], [[[6.0, 6.0, 
7.0], [2.0, 4.0, 3.0], [4.0, 2.0, 8.0]], [[6.0, 7.0, 2.0], [2.0, 8.0, 6.0], [5.0, 5.0, 5.0]], [[1.0, 3.0, 1.0], [6.0, 4.0, 
5.0], [2.0, 2.0, 1.0]]], [[[3.0, 5.0, 6.0], [1.0, 2.0, 1.0], [6.0, 6.0, 4.0]], [[5.0, 5.0, 6.0], [2.0, 6.0, 8.0], [4.0, 8.0, 
5.0]], [[6.0, 3.0, 1.0], [3.0, 5.0, 6.0], [7.0, 6.0, 5.0]]]]))
      val=Data(numpy.array([[[[6.0, 4.0, 1.0], [3.0, 3.0, 8.0]]], [[[5.0, 2.0, 7.0], [2.0, 2.0, 7.0]]], [[[7.0, 7.0, 6.0], 
[2.0, 7.0, 2.0]]]]),self.functionspace)
      arg[:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 1.0], [2.0, 7.0, 7.0], [1.0, 4.0, 3.0]], [[6.0, 4.0, 1.0], [3.0, 3.0, 8.0], [5.0, 
1.0, 1.0]], [[2.0, 1.0, 7.0], [5.0, 2.0, 4.0], [1.0, 5.0, 4.0]]], [[[7.0, 2.0, 5.0], [8.0, 1.0, 7.0], [1.0, 5.0, 1.0]], [[5.0, 
2.0, 7.0], [2.0, 2.0, 7.0], [2.0, 1.0, 6.0]], [[2.0, 1.0, 2.0], [6.0, 5.0, 5.0], [8.0, 5.0, 7.0]]], [[[8.0, 7.0, 6.0], [8.0, 
2.0, 8.0], [2.0, 4.0, 3.0]], [[7.0, 7.0, 6.0], [2.0, 7.0, 2.0], [1.0, 8.0, 7.0]], [[1.0, 8.0, 1.0], [6.0, 1.0, 3.0], [7.0, 2.0, 
3.0]]], [[[7.0, 6.0, 2.0], [6.0, 4.0, 3.0], [2.0, 7.0, 8.0]], [[5.0, 2.0, 2.0], [7.0, 4.0, 5.0], [5.0, 2.0, 2.0]], [[4.0, 6.0, 
7.0], [5.0, 5.0, 6.0], [1.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 7.0], [6.0, 1.0, 5.0], [2.0, 2.0, 5.0]], 
[[6.0, 4.0, 1.0], [3.0, 3.0, 8.0], [8.0, 5.0, 7.0]], [[7.0, 8.0, 6.0], [7.0, 3.0, 5.0], [1.0, 1.0, 3.0]]], [[[7.0, 4.0, 1.0], 
[3.0, 7.0, 1.0], [8.0, 4.0, 5.0]], [[5.0, 2.0, 7.0], [2.0, 2.0, 7.0], [2.0, 4.0, 3.0]], [[4.0, 6.0, 5.0], [5.0, 3.0, 2.0], 
[8.0, 3.0, 5.0]]], [[[6.0, 6.0, 7.0], [2.0, 4.0, 3.0], [4.0, 2.0, 8.0]], [[7.0, 7.0, 6.0], [2.0, 7.0, 2.0], [5.0, 5.0, 5.0]], 
[[1.0, 3.0, 1.0], [6.0, 4.0, 5.0], [2.0, 2.0, 1.0]]], [[[3.0, 5.0, 6.0], [1.0, 2.0, 1.0], [6.0, 6.0, 4.0]], [[5.0, 5.0, 6.0], 
[2.0, 6.0, 8.0], [4.0, 8.0, 5.0]], [[6.0, 3.0, 1.0], [3.0, 5.0, 6.0], [7.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_URUN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 3.0], [1.0, 1.0, 4.0], [2.0, 4.0, 3.0]], [[1.0, 6.0, 7.0], [7.0, 8.0, 4.0], [8.0, 8.0, 
1.0]], [[5.0, 7.0, 7.0], [5.0, 7.0, 4.0], [7.0, 1.0, 6.0]]], [[[5.0, 1.0, 3.0], [6.0, 4.0, 1.0], [4.0, 4.0, 7.0]], [[7.0, 5.0, 
7.0], [4.0, 3.0, 5.0], [8.0, 3.0, 3.0]], [[7.0, 4.0, 6.0], [5.0, 5.0, 3.0], [5.0, 6.0, 5.0]]], [[[1.0, 1.0, 5.0], [7.0, 4.0, 
2.0], [3.0, 3.0, 7.0]], [[5.0, 3.0, 8.0], [1.0, 3.0, 7.0], [2.0, 5.0, 2.0]], [[3.0, 2.0, 4.0], [2.0, 7.0, 7.0], [1.0, 2.0, 
5.0]]], [[[2.0, 3.0, 5.0], [6.0, 8.0, 4.0], [6.0, 7.0, 1.0]], [[2.0, 6.0, 1.0], [1.0, 2.0, 7.0], [7.0, 5.0, 5.0]], [[1.0, 2.0, 
4.0], [7.0, 6.0, 5.0], [3.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 4.0], [8.0, 3.0, 3.0], [7.0, 8.0, 4.0]], [[1.0, 5.0, 8.0], [5.0, 1.0, 
3.0], [6.0, 4.0, 3.0]], [[3.0, 1.0, 6.0], [6.0, 2.0, 1.0], [2.0, 2.0, 3.0]]], [[[6.0, 8.0, 8.0], [6.0, 1.0, 8.0], [3.0, 2.0, 
4.0]], [[3.0, 6.0, 6.0], [1.0, 3.0, 6.0], [1.0, 5.0, 7.0]], [[5.0, 3.0, 7.0], [4.0, 1.0, 8.0], [7.0, 2.0, 4.0]]], [[[2.0, 4.0, 
5.0], [1.0, 1.0, 3.0], [4.0, 8.0, 1.0]], [[6.0, 8.0, 3.0], [5.0, 3.0, 7.0], [2.0, 7.0, 3.0]], [[4.0, 7.0, 8.0], [6.0, 4.0, 
3.0], [7.0, 4.0, 6.0]]], [[[5.0, 8.0, 5.0], [3.0, 2.0, 2.0], [6.0, 1.0, 8.0]], [[8.0, 4.0, 1.0], [2.0, 2.0, 3.0], [1.0, 4.0, 
7.0]], [[2.0, 6.0, 1.0], [3.0, 8.0, 4.0], [6.0, 1.0, 1.0]]]]))
      val=Data(numpy.array([[[[5.0, 8.0, 2.0], [5.0, 5.0, 4.0]]], [[[1.0, 6.0, 2.0], [8.0, 4.0, 7.0]]], [[[8.0, 3.0, 6.0], 
[4.0, 1.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 2.0], [3.0, 8.0, 8.0]]], [[[7.0, 6.0, 2.0], [5.0, 2.0, 8.0]]], [[[1.0, 
6.0, 6.0], [8.0, 6.0, 3.0]]]]))
      arg[:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 3.0], [1.0, 1.0, 4.0], [2.0, 4.0, 3.0]], [[5.0, 8.0, 2.0], [5.0, 5.0, 4.0], [8.0, 
8.0, 1.0]], [[5.0, 7.0, 7.0], [5.0, 7.0, 4.0], [7.0, 1.0, 6.0]]], [[[5.0, 1.0, 3.0], [6.0, 4.0, 1.0], [4.0, 4.0, 7.0]], [[1.0, 
6.0, 2.0], [8.0, 4.0, 7.0], [8.0, 3.0, 3.0]], [[7.0, 4.0, 6.0], [5.0, 5.0, 3.0], [5.0, 6.0, 5.0]]], [[[1.0, 1.0, 5.0], [7.0, 
4.0, 2.0], [3.0, 3.0, 7.0]], [[8.0, 3.0, 6.0], [4.0, 1.0, 4.0], [2.0, 5.0, 2.0]], [[3.0, 2.0, 4.0], [2.0, 7.0, 7.0], [1.0, 2.0, 
5.0]]], [[[2.0, 3.0, 5.0], [6.0, 8.0, 4.0], [6.0, 7.0, 1.0]], [[2.0, 6.0, 1.0], [1.0, 2.0, 7.0], [7.0, 5.0, 5.0]], [[1.0, 2.0, 
4.0], [7.0, 6.0, 5.0], [3.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [8.0, 3.0, 3.0], [7.0, 8.0, 4.0]], 
[[8.0, 5.0, 2.0], [3.0, 8.0, 8.0], [6.0, 4.0, 3.0]], [[3.0, 1.0, 6.0], [6.0, 2.0, 1.0], [2.0, 2.0, 3.0]]], [[[6.0, 8.0, 8.0], 
[6.0, 1.0, 8.0], [3.0, 2.0, 4.0]], [[7.0, 6.0, 2.0], [5.0, 2.0, 8.0], [1.0, 5.0, 7.0]], [[5.0, 3.0, 7.0], [4.0, 1.0, 8.0], 
[7.0, 2.0, 4.0]]], [[[2.0, 4.0, 5.0], [1.0, 1.0, 3.0], [4.0, 8.0, 1.0]], [[1.0, 6.0, 6.0], [8.0, 6.0, 3.0], [2.0, 7.0, 3.0]], 
[[4.0, 7.0, 8.0], [6.0, 4.0, 3.0], [7.0, 4.0, 6.0]]], [[[5.0, 8.0, 5.0], [3.0, 2.0, 2.0], [6.0, 1.0, 8.0]], [[8.0, 4.0, 1.0], 
[2.0, 2.0, 3.0], [1.0, 4.0, 7.0]], [[2.0, 6.0, 1.0], [3.0, 8.0, 4.0], [6.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_URUN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 1.0], [2.0, 3.0, 7.0], [7.0, 8.0, 6.0]], [[8.0, 7.0, 3.0], [1.0, 8.0, 4.0], [6.0, 3.0, 
6.0]], [[2.0, 4.0, 7.0], [8.0, 7.0, 6.0], [6.0, 1.0, 1.0]]], [[[6.0, 4.0, 2.0], [7.0, 6.0, 5.0], [1.0, 7.0, 4.0]], [[3.0, 6.0, 
1.0], [6.0, 2.0, 6.0], [8.0, 2.0, 7.0]], [[4.0, 4.0, 5.0], [3.0, 3.0, 3.0], [5.0, 5.0, 8.0]]], [[[5.0, 4.0, 8.0], [6.0, 6.0, 
3.0], [4.0, 8.0, 1.0]], [[2.0, 6.0, 6.0], [4.0, 3.0, 2.0], [3.0, 4.0, 1.0]], [[2.0, 7.0, 2.0], [8.0, 3.0, 6.0], [8.0, 4.0, 
4.0]]], [[[7.0, 2.0, 2.0], [5.0, 6.0, 6.0], [5.0, 3.0, 5.0]], [[8.0, 7.0, 2.0], [6.0, 7.0, 5.0], [7.0, 4.0, 6.0]], [[3.0, 5.0, 
1.0], [5.0, 6.0, 3.0], [3.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 7.0], [3.0, 3.0, 6.0], [1.0, 1.0, 1.0]], [[2.0, 5.0, 6.0], [3.0, 2.0, 
3.0], [2.0, 5.0, 3.0]], [[6.0, 5.0, 6.0], [6.0, 7.0, 1.0], [5.0, 6.0, 2.0]]], [[[8.0, 5.0, 3.0], [6.0, 6.0, 8.0], [8.0, 4.0, 
4.0]], [[3.0, 4.0, 4.0], [5.0, 2.0, 3.0], [7.0, 4.0, 6.0]], [[1.0, 3.0, 8.0], [2.0, 6.0, 3.0], [3.0, 1.0, 8.0]]], [[[5.0, 4.0, 
6.0], [7.0, 8.0, 8.0], [5.0, 8.0, 3.0]], [[7.0, 7.0, 6.0], [1.0, 6.0, 3.0], [2.0, 6.0, 2.0]], [[1.0, 2.0, 7.0], [2.0, 5.0, 
4.0], [3.0, 8.0, 6.0]]], [[[5.0, 7.0, 3.0], [3.0, 6.0, 5.0], [8.0, 7.0, 7.0]], [[5.0, 8.0, 7.0], [1.0, 2.0, 6.0], [7.0, 3.0, 
2.0]], [[7.0, 5.0, 6.0], [3.0, 6.0, 4.0], [7.0, 6.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 4.0, 1.0], [1.0, 8.0, 4.0]]], [[[5.0, 6.0, 7.0], [3.0, 3.0, 6.0]]], [[[7.0, 2.0, 5.0], 
[8.0, 5.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 5.0, 1.0], [3.0, 8.0, 6.0]]], [[[1.0, 5.0, 3.0], [7.0, 2.0, 6.0]]], 
[[[3.0, 3.0, 1.0], [5.0, 2.0, 6.0]]]])
      arg[:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 1.0], [2.0, 3.0, 7.0], [7.0, 8.0, 6.0]], [[1.0, 4.0, 1.0], [1.0, 8.0, 4.0], [6.0, 
3.0, 6.0]], [[2.0, 4.0, 7.0], [8.0, 7.0, 6.0], [6.0, 1.0, 1.0]]], [[[6.0, 4.0, 2.0], [7.0, 6.0, 5.0], [1.0, 7.0, 4.0]], [[5.0, 
6.0, 7.0], [3.0, 3.0, 6.0], [8.0, 2.0, 7.0]], [[4.0, 4.0, 5.0], [3.0, 3.0, 3.0], [5.0, 5.0, 8.0]]], [[[5.0, 4.0, 8.0], [6.0, 
6.0, 3.0], [4.0, 8.0, 1.0]], [[7.0, 2.0, 5.0], [8.0, 5.0, 2.0], [3.0, 4.0, 1.0]], [[2.0, 7.0, 2.0], [8.0, 3.0, 6.0], [8.0, 4.0, 
4.0]]], [[[7.0, 2.0, 2.0], [5.0, 6.0, 6.0], [5.0, 3.0, 5.0]], [[8.0, 7.0, 2.0], [6.0, 7.0, 5.0], [7.0, 4.0, 6.0]], [[3.0, 5.0, 
1.0], [5.0, 6.0, 3.0], [3.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 7.0], [3.0, 3.0, 6.0], [1.0, 1.0, 1.0]], 
[[6.0, 5.0, 1.0], [3.0, 8.0, 6.0], [2.0, 5.0, 3.0]], [[6.0, 5.0, 6.0], [6.0, 7.0, 1.0], [5.0, 6.0, 2.0]]], [[[8.0, 5.0, 3.0], 
[6.0, 6.0, 8.0], [8.0, 4.0, 4.0]], [[1.0, 5.0, 3.0], [7.0, 2.0, 6.0], [7.0, 4.0, 6.0]], [[1.0, 3.0, 8.0], [2.0, 6.0, 3.0], 
[3.0, 1.0, 8.0]]], [[[5.0, 4.0, 6.0], [7.0, 8.0, 8.0], [5.0, 8.0, 3.0]], [[3.0, 3.0, 1.0], [5.0, 2.0, 6.0], [2.0, 6.0, 2.0]], 
[[1.0, 2.0, 7.0], [2.0, 5.0, 4.0], [3.0, 8.0, 6.0]]], [[[5.0, 7.0, 3.0], [3.0, 6.0, 5.0], [8.0, 7.0, 7.0]], [[5.0, 8.0, 7.0], 
[1.0, 2.0, 6.0], [7.0, 3.0, 2.0]], [[7.0, 5.0, 6.0], [3.0, 6.0, 4.0], [7.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLUN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 3.0], [6.0, 4.0, 5.0], [6.0, 5.0, 3.0]], [[4.0, 5.0, 5.0], [2.0, 4.0, 6.0], [7.0, 2.0, 
2.0]], [[8.0, 7.0, 1.0], [1.0, 3.0, 8.0], [6.0, 3.0, 5.0]]], [[[3.0, 1.0, 7.0], [1.0, 8.0, 6.0], [8.0, 3.0, 5.0]], [[6.0, 6.0, 
4.0], [7.0, 5.0, 5.0], [4.0, 8.0, 2.0]], [[5.0, 2.0, 4.0], [8.0, 3.0, 8.0], [2.0, 3.0, 7.0]]], [[[7.0, 5.0, 3.0], [8.0, 2.0, 
7.0], [4.0, 6.0, 7.0]], [[5.0, 3.0, 5.0], [2.0, 3.0, 5.0], [6.0, 3.0, 7.0]], [[6.0, 6.0, 8.0], [2.0, 2.0, 4.0], [8.0, 3.0, 
4.0]]], [[[8.0, 7.0, 5.0], [5.0, 6.0, 8.0], [8.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [3.0, 6.0, 6.0], [7.0, 7.0, 4.0]], [[4.0, 7.0, 
2.0], [8.0, 4.0, 6.0], [4.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 7.0], [7.0, 4.0, 7.0], [1.0, 1.0, 1.0]], [[3.0, 1.0, 4.0], [8.0, 2.0, 
4.0], [8.0, 7.0, 6.0]], [[8.0, 1.0, 4.0], [2.0, 5.0, 3.0], [2.0, 4.0, 8.0]]], [[[2.0, 1.0, 7.0], [3.0, 2.0, 1.0], [1.0, 8.0, 
8.0]], [[8.0, 4.0, 3.0], [2.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[1.0, 2.0, 8.0], [1.0, 5.0, 8.0], [7.0, 1.0, 5.0]]], [[[5.0, 8.0, 
3.0], [6.0, 4.0, 1.0], [5.0, 8.0, 7.0]], [[6.0, 3.0, 2.0], [2.0, 4.0, 6.0], [3.0, 2.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 5.0, 
2.0], [4.0, 2.0, 4.0]]], [[[2.0, 1.0, 8.0], [4.0, 5.0, 3.0], [2.0, 6.0, 2.0]], [[1.0, 5.0, 7.0], [4.0, 5.0, 2.0], [3.0, 1.0, 
4.0]], [[4.0, 4.0, 7.0], [7.0, 7.0, 2.0], [6.0, 8.0, 5.0]]]]))
      val=numpy.array([[[6.0, 5.0, 1.0], [5.0, 7.0, 7.0]]])
      arg[2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 3.0], [6.0, 4.0, 5.0], [6.0, 5.0, 3.0]], [[4.0, 5.0, 5.0], [2.0, 4.0, 6.0], [7.0, 
2.0, 2.0]], [[8.0, 7.0, 1.0], [1.0, 3.0, 8.0], [6.0, 3.0, 5.0]]], [[[3.0, 1.0, 7.0], [1.0, 8.0, 6.0], [8.0, 3.0, 5.0]], [[6.0, 
6.0, 4.0], [7.0, 5.0, 5.0], [4.0, 8.0, 2.0]], [[5.0, 2.0, 4.0], [8.0, 3.0, 8.0], [2.0, 3.0, 7.0]]], [[[7.0, 5.0, 3.0], [8.0, 
2.0, 7.0], [4.0, 6.0, 7.0]], [[5.0, 3.0, 5.0], [2.0, 3.0, 5.0], [6.0, 3.0, 7.0]], [[6.0, 5.0, 1.0], [5.0, 7.0, 7.0], [8.0, 3.0, 
4.0]]], [[[8.0, 7.0, 5.0], [5.0, 6.0, 8.0], [8.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], [3.0, 6.0, 6.0], [7.0, 7.0, 4.0]], [[4.0, 7.0, 
2.0], [8.0, 4.0, 6.0], [4.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 7.0], [7.0, 4.0, 7.0], [1.0, 1.0, 1.0]], 
[[3.0, 1.0, 4.0], [8.0, 2.0, 4.0], [8.0, 7.0, 6.0]], [[8.0, 1.0, 4.0], [2.0, 5.0, 3.0], [2.0, 4.0, 8.0]]], [[[2.0, 1.0, 7.0], 
[3.0, 2.0, 1.0], [1.0, 8.0, 8.0]], [[8.0, 4.0, 3.0], [2.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[1.0, 2.0, 8.0], [1.0, 5.0, 8.0], 
[7.0, 1.0, 5.0]]], [[[5.0, 8.0, 3.0], [6.0, 4.0, 1.0], [5.0, 8.0, 7.0]], [[6.0, 3.0, 2.0], [2.0, 4.0, 6.0], [3.0, 2.0, 5.0]], 
[[6.0, 5.0, 1.0], [5.0, 7.0, 7.0], [4.0, 2.0, 4.0]]], [[[2.0, 1.0, 8.0], [4.0, 5.0, 3.0], [2.0, 6.0, 2.0]], [[1.0, 5.0, 7.0], 
[4.0, 5.0, 2.0], [3.0, 1.0, 4.0]], [[4.0, 4.0, 7.0], [7.0, 7.0, 2.0], [6.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLUN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 3.0], [3.0, 8.0, 2.0], [7.0, 8.0, 6.0]], [[8.0, 6.0, 5.0], [6.0, 3.0, 8.0], [6.0, 6.0, 
2.0]], [[5.0, 2.0, 3.0], [2.0, 7.0, 1.0], [1.0, 3.0, 1.0]]], [[[7.0, 7.0, 3.0], [8.0, 4.0, 5.0], [1.0, 6.0, 7.0]], [[5.0, 8.0, 
3.0], [5.0, 5.0, 5.0], [2.0, 5.0, 5.0]], [[1.0, 8.0, 5.0], [5.0, 2.0, 3.0], [6.0, 3.0, 2.0]]], [[[7.0, 8.0, 5.0], [1.0, 7.0, 
3.0], [2.0, 8.0, 3.0]], [[3.0, 5.0, 8.0], [5.0, 7.0, 1.0], [7.0, 6.0, 4.0]], [[2.0, 2.0, 6.0], [4.0, 2.0, 5.0], [8.0, 2.0, 
8.0]]], [[[5.0, 5.0, 8.0], [7.0, 8.0, 3.0], [4.0, 4.0, 5.0]], [[1.0, 1.0, 4.0], [5.0, 3.0, 5.0], [7.0, 5.0, 3.0]], [[7.0, 2.0, 
2.0], [6.0, 3.0, 4.0], [1.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 1.0], [1.0, 1.0, 3.0], [5.0, 3.0, 8.0]], [[1.0, 7.0, 8.0], [1.0, 3.0, 
7.0], [4.0, 7.0, 3.0]], [[2.0, 7.0, 2.0], [4.0, 3.0, 5.0], [4.0, 7.0, 7.0]]], [[[3.0, 3.0, 7.0], [2.0, 7.0, 3.0], [1.0, 7.0, 
5.0]], [[3.0, 5.0, 2.0], [1.0, 4.0, 4.0], [3.0, 2.0, 8.0]], [[1.0, 5.0, 4.0], [8.0, 4.0, 5.0], [7.0, 1.0, 4.0]]], [[[7.0, 8.0, 
5.0], [3.0, 6.0, 4.0], [7.0, 3.0, 1.0]], [[7.0, 4.0, 4.0], [1.0, 1.0, 8.0], [2.0, 6.0, 4.0]], [[4.0, 8.0, 1.0], [3.0, 3.0, 
7.0], [6.0, 1.0, 8.0]]], [[[5.0, 6.0, 5.0], [7.0, 4.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 1.0, 3.0], [5.0, 3.0, 6.0], [3.0, 7.0, 
8.0]], [[4.0, 7.0, 2.0], [6.0, 1.0, 8.0], [4.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[7.0, 8.0, 5.0], [5.0, 5.0, 5.0]]]),self.functionspace)
      arg[2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 3.0], [3.0, 8.0, 2.0], [7.0, 8.0, 6.0]], [[8.0, 6.0, 5.0], [6.0, 3.0, 8.0], [6.0, 
6.0, 2.0]], [[5.0, 2.0, 3.0], [2.0, 7.0, 1.0], [1.0, 3.0, 1.0]]], [[[7.0, 7.0, 3.0], [8.0, 4.0, 5.0], [1.0, 6.0, 7.0]], [[5.0, 
8.0, 3.0], [5.0, 5.0, 5.0], [2.0, 5.0, 5.0]], [[1.0, 8.0, 5.0], [5.0, 2.0, 3.0], [6.0, 3.0, 2.0]]], [[[7.0, 8.0, 5.0], [1.0, 
7.0, 3.0], [2.0, 8.0, 3.0]], [[3.0, 5.0, 8.0], [5.0, 7.0, 1.0], [7.0, 6.0, 4.0]], [[7.0, 8.0, 5.0], [5.0, 5.0, 5.0], [8.0, 2.0, 
8.0]]], [[[5.0, 5.0, 8.0], [7.0, 8.0, 3.0], [4.0, 4.0, 5.0]], [[1.0, 1.0, 4.0], [5.0, 3.0, 5.0], [7.0, 5.0, 3.0]], [[7.0, 2.0, 
2.0], [6.0, 3.0, 4.0], [1.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 1.0], [1.0, 1.0, 3.0], [5.0, 3.0, 8.0]], 
[[1.0, 7.0, 8.0], [1.0, 3.0, 7.0], [4.0, 7.0, 3.0]], [[2.0, 7.0, 2.0], [4.0, 3.0, 5.0], [4.0, 7.0, 7.0]]], [[[3.0, 3.0, 7.0], 
[2.0, 7.0, 3.0], [1.0, 7.0, 5.0]], [[3.0, 5.0, 2.0], [1.0, 4.0, 4.0], [3.0, 2.0, 8.0]], [[1.0, 5.0, 4.0], [8.0, 4.0, 5.0], 
[7.0, 1.0, 4.0]]], [[[7.0, 8.0, 5.0], [3.0, 6.0, 4.0], [7.0, 3.0, 1.0]], [[7.0, 4.0, 4.0], [1.0, 1.0, 8.0], [2.0, 6.0, 4.0]], 
[[7.0, 8.0, 5.0], [5.0, 5.0, 5.0], [6.0, 1.0, 8.0]]], [[[5.0, 6.0, 5.0], [7.0, 4.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 1.0, 3.0], 
[5.0, 3.0, 6.0], [3.0, 7.0, 8.0]], [[4.0, 7.0, 2.0], [6.0, 1.0, 8.0], [4.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLUN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 2.0], [4.0, 1.0, 3.0], [2.0, 8.0, 7.0]], [[1.0, 2.0, 5.0], [8.0, 7.0, 7.0], [2.0, 6.0, 
5.0]], [[6.0, 3.0, 5.0], [3.0, 4.0, 7.0], [3.0, 3.0, 7.0]]], [[[4.0, 7.0, 2.0], [7.0, 5.0, 4.0], [4.0, 6.0, 8.0]], [[3.0, 5.0, 
6.0], [5.0, 6.0, 7.0], [5.0, 5.0, 6.0]], [[8.0, 6.0, 8.0], [1.0, 7.0, 6.0], [5.0, 7.0, 4.0]]], [[[1.0, 8.0, 2.0], [3.0, 6.0, 
7.0], [6.0, 6.0, 8.0]], [[5.0, 6.0, 8.0], [7.0, 8.0, 1.0], [7.0, 3.0, 6.0]], [[1.0, 2.0, 7.0], [6.0, 3.0, 6.0], [1.0, 7.0, 
6.0]]], [[[1.0, 8.0, 6.0], [8.0, 3.0, 5.0], [2.0, 8.0, 5.0]], [[2.0, 3.0, 7.0], [4.0, 6.0, 1.0], [2.0, 7.0, 8.0]], [[2.0, 8.0, 
2.0], [8.0, 2.0, 1.0], [6.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 5.0], [8.0, 2.0, 8.0], [3.0, 2.0, 1.0]], [[1.0, 7.0, 1.0], [4.0, 1.0, 
5.0], [5.0, 1.0, 2.0]], [[5.0, 2.0, 5.0], [3.0, 4.0, 7.0], [8.0, 8.0, 1.0]]], [[[5.0, 1.0, 3.0], [5.0, 3.0, 6.0], [4.0, 3.0, 
8.0]], [[6.0, 7.0, 6.0], [8.0, 1.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 2.0, 5.0], [3.0, 6.0, 2.0], [4.0, 8.0, 3.0]]], [[[4.0, 2.0, 
3.0], [1.0, 2.0, 7.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 1.0], [8.0, 3.0, 4.0], [2.0, 6.0, 7.0]], [[4.0, 4.0, 1.0], [2.0, 2.0, 
3.0], [8.0, 5.0, 3.0]]], [[[4.0, 1.0, 8.0], [3.0, 1.0, 2.0], [1.0, 6.0, 2.0]], [[3.0, 1.0, 8.0], [7.0, 3.0, 4.0], [2.0, 6.0, 
1.0]], [[2.0, 6.0, 5.0], [7.0, 1.0, 5.0], [8.0, 3.0, 1.0]]]]))
      val=Data(numpy.array([[[6.0, 3.0, 5.0], [7.0, 5.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 1.0, 1.0], [6.0, 2.0, 1.0]]]))
      arg[2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 2.0], [4.0, 1.0, 3.0], [2.0, 8.0, 7.0]], [[1.0, 2.0, 5.0], [8.0, 7.0, 7.0], [2.0, 
6.0, 5.0]], [[6.0, 3.0, 5.0], [3.0, 4.0, 7.0], [3.0, 3.0, 7.0]]], [[[4.0, 7.0, 2.0], [7.0, 5.0, 4.0], [4.0, 6.0, 8.0]], [[3.0, 
5.0, 6.0], [5.0, 6.0, 7.0], [5.0, 5.0, 6.0]], [[8.0, 6.0, 8.0], [1.0, 7.0, 6.0], [5.0, 7.0, 4.0]]], [[[1.0, 8.0, 2.0], [3.0, 
6.0, 7.0], [6.0, 6.0, 8.0]], [[5.0, 6.0, 8.0], [7.0, 8.0, 1.0], [7.0, 3.0, 6.0]], [[6.0, 3.0, 5.0], [7.0, 5.0, 5.0], [1.0, 7.0, 
6.0]]], [[[1.0, 8.0, 6.0], [8.0, 3.0, 5.0], [2.0, 8.0, 5.0]], [[2.0, 3.0, 7.0], [4.0, 6.0, 1.0], [2.0, 7.0, 8.0]], [[2.0, 8.0, 
2.0], [8.0, 2.0, 1.0], [6.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 5.0], [8.0, 2.0, 8.0], [3.0, 2.0, 1.0]], 
[[1.0, 7.0, 1.0], [4.0, 1.0, 5.0], [5.0, 1.0, 2.0]], [[5.0, 2.0, 5.0], [3.0, 4.0, 7.0], [8.0, 8.0, 1.0]]], [[[5.0, 1.0, 3.0], 
[5.0, 3.0, 6.0], [4.0, 3.0, 8.0]], [[6.0, 7.0, 6.0], [8.0, 1.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 2.0, 5.0], [3.0, 6.0, 2.0], 
[4.0, 8.0, 3.0]]], [[[4.0, 2.0, 3.0], [1.0, 2.0, 7.0], [5.0, 1.0, 1.0]], [[1.0, 3.0, 1.0], [8.0, 3.0, 4.0], [2.0, 6.0, 7.0]], 
[[4.0, 1.0, 1.0], [6.0, 2.0, 1.0], [8.0, 5.0, 3.0]]], [[[4.0, 1.0, 8.0], [3.0, 1.0, 2.0], [1.0, 6.0, 2.0]], [[3.0, 1.0, 8.0], 
[7.0, 3.0, 4.0], [2.0, 6.0, 1.0]], [[2.0, 6.0, 5.0], [7.0, 1.0, 5.0], [8.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLUN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 7.0], [1.0, 5.0, 5.0], [2.0, 7.0, 3.0]], [[2.0, 2.0, 4.0], [6.0, 2.0, 3.0], [2.0, 3.0, 
1.0]], [[7.0, 6.0, 7.0], [2.0, 3.0, 5.0], [1.0, 7.0, 3.0]]], [[[1.0, 8.0, 8.0], [1.0, 3.0, 5.0], [1.0, 4.0, 2.0]], [[1.0, 5.0, 
7.0], [3.0, 3.0, 6.0], [6.0, 4.0, 4.0]], [[8.0, 8.0, 2.0], [3.0, 2.0, 4.0], [3.0, 8.0, 8.0]]], [[[8.0, 3.0, 2.0], [3.0, 7.0, 
2.0], [6.0, 7.0, 5.0]], [[4.0, 3.0, 4.0], [8.0, 2.0, 6.0], [8.0, 7.0, 8.0]], [[2.0, 8.0, 4.0], [5.0, 8.0, 3.0], [3.0, 1.0, 
1.0]]], [[[4.0, 2.0, 1.0], [2.0, 6.0, 2.0], [8.0, 6.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 5.0, 5.0], [7.0, 8.0, 3.0]], [[3.0, 3.0, 
4.0], [6.0, 1.0, 6.0], [3.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 1.0], [3.0, 1.0, 6.0], [4.0, 4.0, 7.0]], [[4.0, 1.0, 5.0], [7.0, 1.0, 
7.0], [2.0, 7.0, 5.0]], [[8.0, 7.0, 7.0], [1.0, 8.0, 6.0], [1.0, 8.0, 7.0]]], [[[1.0, 6.0, 8.0], [7.0, 1.0, 5.0], [7.0, 3.0, 
3.0]], [[1.0, 8.0, 6.0], [8.0, 5.0, 3.0], [4.0, 5.0, 4.0]], [[1.0, 5.0, 3.0], [4.0, 4.0, 5.0], [4.0, 5.0, 4.0]]], [[[2.0, 6.0, 
8.0], [6.0, 6.0, 4.0], [6.0, 7.0, 5.0]], [[7.0, 1.0, 4.0], [8.0, 4.0, 1.0], [2.0, 1.0, 7.0]], [[5.0, 3.0, 2.0], [4.0, 3.0, 
5.0], [8.0, 6.0, 5.0]]], [[[5.0, 6.0, 7.0], [7.0, 7.0, 6.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 4.0], [6.0, 8.0, 5.0], [8.0, 8.0, 
2.0]], [[1.0, 3.0, 6.0], [2.0, 5.0, 3.0], [3.0, 8.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[3.0, 1.0, 6.0], [3.0, 1.0, 5.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 5.0], [8.0, 6.0, 
8.0]]])
      arg[2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 7.0], [1.0, 5.0, 5.0], [2.0, 7.0, 3.0]], [[2.0, 2.0, 4.0], [6.0, 2.0, 3.0], [2.0, 
3.0, 1.0]], [[7.0, 6.0, 7.0], [2.0, 3.0, 5.0], [1.0, 7.0, 3.0]]], [[[1.0, 8.0, 8.0], [1.0, 3.0, 5.0], [1.0, 4.0, 2.0]], [[1.0, 
5.0, 7.0], [3.0, 3.0, 6.0], [6.0, 4.0, 4.0]], [[8.0, 8.0, 2.0], [3.0, 2.0, 4.0], [3.0, 8.0, 8.0]]], [[[8.0, 3.0, 2.0], [3.0, 
7.0, 2.0], [6.0, 7.0, 5.0]], [[4.0, 3.0, 4.0], [8.0, 2.0, 6.0], [8.0, 7.0, 8.0]], [[3.0, 1.0, 6.0], [3.0, 1.0, 5.0], [3.0, 1.0, 
1.0]]], [[[4.0, 2.0, 1.0], [2.0, 6.0, 2.0], [8.0, 6.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 5.0, 5.0], [7.0, 8.0, 3.0]], [[3.0, 3.0, 
4.0], [6.0, 1.0, 6.0], [3.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 1.0], [3.0, 1.0, 6.0], [4.0, 4.0, 7.0]], 
[[4.0, 1.0, 5.0], [7.0, 1.0, 7.0], [2.0, 7.0, 5.0]], [[8.0, 7.0, 7.0], [1.0, 8.0, 6.0], [1.0, 8.0, 7.0]]], [[[1.0, 6.0, 8.0], 
[7.0, 1.0, 5.0], [7.0, 3.0, 3.0]], [[1.0, 8.0, 6.0], [8.0, 5.0, 3.0], [4.0, 5.0, 4.0]], [[1.0, 5.0, 3.0], [4.0, 4.0, 5.0], 
[4.0, 5.0, 4.0]]], [[[2.0, 6.0, 8.0], [6.0, 6.0, 4.0], [6.0, 7.0, 5.0]], [[7.0, 1.0, 4.0], [8.0, 4.0, 1.0], [2.0, 1.0, 7.0]], 
[[4.0, 6.0, 5.0], [8.0, 6.0, 8.0], [8.0, 6.0, 5.0]]], [[[5.0, 6.0, 7.0], [7.0, 7.0, 6.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 4.0], 
[6.0, 8.0, 5.0], [8.0, 8.0, 2.0]], [[1.0, 3.0, 6.0], [2.0, 5.0, 3.0], [3.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPCN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 5.0], [3.0, 3.0, 7.0], [7.0, 1.0, 5.0]], [[6.0, 3.0, 2.0], [1.0, 4.0, 6.0], [5.0, 7.0, 
8.0]], [[5.0, 1.0, 8.0], [8.0, 3.0, 1.0], [2.0, 8.0, 3.0]]], [[[5.0, 7.0, 8.0], [4.0, 3.0, 5.0], [6.0, 5.0, 2.0]], [[8.0, 2.0, 
1.0], [6.0, 8.0, 1.0], [4.0, 8.0, 2.0]], [[7.0, 4.0, 5.0], [5.0, 8.0, 4.0], [1.0, 4.0, 2.0]]], [[[3.0, 2.0, 6.0], [5.0, 5.0, 
3.0], [6.0, 1.0, 5.0]], [[8.0, 1.0, 8.0], [5.0, 3.0, 7.0], [7.0, 1.0, 5.0]], [[6.0, 4.0, 6.0], [5.0, 4.0, 7.0], [5.0, 8.0, 
3.0]]], [[[4.0, 2.0, 4.0], [7.0, 5.0, 1.0], [2.0, 3.0, 3.0]], [[8.0, 4.0, 2.0], [4.0, 5.0, 8.0], [5.0, 8.0, 7.0]], [[6.0, 3.0, 
1.0], [8.0, 8.0, 7.0], [1.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 7.0], [5.0, 1.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 1.0, 3.0], [4.0, 5.0, 
6.0], [3.0, 7.0, 2.0]], [[1.0, 5.0, 1.0], [6.0, 8.0, 5.0], [1.0, 5.0, 2.0]]], [[[8.0, 2.0, 5.0], [8.0, 5.0, 4.0], [1.0, 5.0, 
3.0]], [[4.0, 2.0, 1.0], [6.0, 3.0, 7.0], [3.0, 2.0, 6.0]], [[8.0, 7.0, 4.0], [2.0, 3.0, 1.0], [4.0, 1.0, 8.0]]], [[[7.0, 6.0, 
6.0], [2.0, 6.0, 2.0], [6.0, 6.0, 4.0]], [[6.0, 6.0, 1.0], [8.0, 4.0, 4.0], [6.0, 6.0, 6.0]], [[8.0, 8.0, 6.0], [3.0, 1.0, 
1.0], [4.0, 6.0, 3.0]]], [[[7.0, 7.0, 5.0], [5.0, 7.0, 7.0], [6.0, 5.0, 5.0]], [[5.0, 6.0, 2.0], [8.0, 7.0, 3.0], [1.0, 3.0, 
2.0]], [[4.0, 5.0, 3.0], [1.0, 6.0, 3.0], [8.0, 6.0, 3.0]]]]))
      val=numpy.array([[4.0, 5.0, 2.0], [6.0, 6.0, 5.0], [7.0, 7.0, 7.0]])
      arg[2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 5.0], [3.0, 3.0, 7.0], [7.0, 1.0, 5.0]], [[6.0, 3.0, 2.0], [1.0, 4.0, 6.0], [5.0, 
7.0, 8.0]], [[5.0, 1.0, 8.0], [8.0, 3.0, 1.0], [2.0, 8.0, 3.0]]], [[[5.0, 7.0, 8.0], [4.0, 3.0, 5.0], [6.0, 5.0, 2.0]], [[8.0, 
2.0, 1.0], [6.0, 8.0, 1.0], [4.0, 8.0, 2.0]], [[7.0, 4.0, 5.0], [5.0, 8.0, 4.0], [1.0, 4.0, 2.0]]], [[[3.0, 2.0, 6.0], [5.0, 
5.0, 3.0], [6.0, 1.0, 5.0]], [[4.0, 5.0, 2.0], [6.0, 6.0, 5.0], [7.0, 7.0, 7.0]], [[6.0, 4.0, 6.0], [5.0, 4.0, 7.0], [5.0, 8.0, 
3.0]]], [[[4.0, 2.0, 4.0], [7.0, 5.0, 1.0], [2.0, 3.0, 3.0]], [[8.0, 4.0, 2.0], [4.0, 5.0, 8.0], [5.0, 8.0, 7.0]], [[6.0, 3.0, 
1.0], [8.0, 8.0, 7.0], [1.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 7.0], [5.0, 1.0, 6.0], [5.0, 3.0, 6.0]], 
[[8.0, 1.0, 3.0], [4.0, 5.0, 6.0], [3.0, 7.0, 2.0]], [[1.0, 5.0, 1.0], [6.0, 8.0, 5.0], [1.0, 5.0, 2.0]]], [[[8.0, 2.0, 5.0], 
[8.0, 5.0, 4.0], [1.0, 5.0, 3.0]], [[4.0, 2.0, 1.0], [6.0, 3.0, 7.0], [3.0, 2.0, 6.0]], [[8.0, 7.0, 4.0], [2.0, 3.0, 1.0], 
[4.0, 1.0, 8.0]]], [[[7.0, 6.0, 6.0], [2.0, 6.0, 2.0], [6.0, 6.0, 4.0]], [[4.0, 5.0, 2.0], [6.0, 6.0, 5.0], [7.0, 7.0, 7.0]], 
[[8.0, 8.0, 6.0], [3.0, 1.0, 1.0], [4.0, 6.0, 3.0]]], [[[7.0, 7.0, 5.0], [5.0, 7.0, 7.0], [6.0, 5.0, 5.0]], [[5.0, 6.0, 2.0], 
[8.0, 7.0, 3.0], [1.0, 3.0, 2.0]], [[4.0, 5.0, 3.0], [1.0, 6.0, 3.0], [8.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPCN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 3.0], [7.0, 4.0, 1.0], [5.0, 5.0, 5.0]], [[3.0, 8.0, 2.0], [6.0, 1.0, 7.0], [5.0, 5.0, 
6.0]], [[5.0, 4.0, 2.0], [2.0, 2.0, 3.0], [8.0, 4.0, 5.0]]], [[[2.0, 2.0, 6.0], [3.0, 8.0, 6.0], [5.0, 6.0, 5.0]], [[6.0, 7.0, 
3.0], [8.0, 2.0, 1.0], [3.0, 4.0, 7.0]], [[6.0, 2.0, 5.0], [2.0, 5.0, 1.0], [8.0, 6.0, 3.0]]], [[[6.0, 8.0, 2.0], [4.0, 5.0, 
3.0], [8.0, 6.0, 8.0]], [[6.0, 6.0, 5.0], [2.0, 3.0, 7.0], [6.0, 3.0, 4.0]], [[2.0, 4.0, 8.0], [1.0, 4.0, 3.0], [5.0, 8.0, 
4.0]]], [[[5.0, 8.0, 2.0], [5.0, 3.0, 1.0], [7.0, 6.0, 4.0]], [[2.0, 2.0, 7.0], [1.0, 1.0, 6.0], [2.0, 2.0, 6.0]], [[4.0, 4.0, 
5.0], [1.0, 3.0, 8.0], [4.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 2.0], [4.0, 3.0, 8.0], [6.0, 2.0, 5.0]], [[6.0, 5.0, 5.0], [4.0, 1.0, 
4.0], [4.0, 4.0, 2.0]], [[7.0, 4.0, 7.0], [3.0, 3.0, 6.0], [4.0, 5.0, 3.0]]], [[[1.0, 6.0, 4.0], [8.0, 8.0, 6.0], [6.0, 7.0, 
3.0]], [[2.0, 7.0, 5.0], [8.0, 6.0, 1.0], [3.0, 6.0, 5.0]], [[8.0, 4.0, 3.0], [5.0, 1.0, 6.0], [4.0, 7.0, 6.0]]], [[[8.0, 2.0, 
8.0], [6.0, 6.0, 5.0], [4.0, 8.0, 7.0]], [[6.0, 4.0, 4.0], [1.0, 3.0, 2.0], [1.0, 2.0, 4.0]], [[8.0, 7.0, 8.0], [7.0, 7.0, 
5.0], [8.0, 1.0, 5.0]]], [[[8.0, 4.0, 4.0], [8.0, 4.0, 3.0], [8.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [4.0, 4.0, 4.0], [4.0, 6.0, 
4.0]], [[1.0, 6.0, 3.0], [5.0, 2.0, 5.0], [4.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[4.0, 7.0, 8.0], [3.0, 3.0, 8.0], [6.0, 8.0, 8.0]]),self.functionspace)
      arg[2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 3.0], [7.0, 4.0, 1.0], [5.0, 5.0, 5.0]], [[3.0, 8.0, 2.0], [6.0, 1.0, 7.0], [5.0, 
5.0, 6.0]], [[5.0, 4.0, 2.0], [2.0, 2.0, 3.0], [8.0, 4.0, 5.0]]], [[[2.0, 2.0, 6.0], [3.0, 8.0, 6.0], [5.0, 6.0, 5.0]], [[6.0, 
7.0, 3.0], [8.0, 2.0, 1.0], [3.0, 4.0, 7.0]], [[6.0, 2.0, 5.0], [2.0, 5.0, 1.0], [8.0, 6.0, 3.0]]], [[[6.0, 8.0, 2.0], [4.0, 
5.0, 3.0], [8.0, 6.0, 8.0]], [[4.0, 7.0, 8.0], [3.0, 3.0, 8.0], [6.0, 8.0, 8.0]], [[2.0, 4.0, 8.0], [1.0, 4.0, 3.0], [5.0, 8.0, 
4.0]]], [[[5.0, 8.0, 2.0], [5.0, 3.0, 1.0], [7.0, 6.0, 4.0]], [[2.0, 2.0, 7.0], [1.0, 1.0, 6.0], [2.0, 2.0, 6.0]], [[4.0, 4.0, 
5.0], [1.0, 3.0, 8.0], [4.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 2.0], [4.0, 3.0, 8.0], [6.0, 2.0, 5.0]], 
[[6.0, 5.0, 5.0], [4.0, 1.0, 4.0], [4.0, 4.0, 2.0]], [[7.0, 4.0, 7.0], [3.0, 3.0, 6.0], [4.0, 5.0, 3.0]]], [[[1.0, 6.0, 4.0], 
[8.0, 8.0, 6.0], [6.0, 7.0, 3.0]], [[2.0, 7.0, 5.0], [8.0, 6.0, 1.0], [3.0, 6.0, 5.0]], [[8.0, 4.0, 3.0], [5.0, 1.0, 6.0], 
[4.0, 7.0, 6.0]]], [[[8.0, 2.0, 8.0], [6.0, 6.0, 5.0], [4.0, 8.0, 7.0]], [[4.0, 7.0, 8.0], [3.0, 3.0, 8.0], [6.0, 8.0, 8.0]], 
[[8.0, 7.0, 8.0], [7.0, 7.0, 5.0], [8.0, 1.0, 5.0]]], [[[8.0, 4.0, 4.0], [8.0, 4.0, 3.0], [8.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], 
[4.0, 4.0, 4.0], [4.0, 6.0, 4.0]], [[1.0, 6.0, 3.0], [5.0, 2.0, 5.0], [4.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPCN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 7.0], [1.0, 1.0, 5.0], [1.0, 5.0, 2.0]], [[7.0, 3.0, 2.0], [7.0, 5.0, 4.0], [5.0, 6.0, 
5.0]], [[1.0, 4.0, 2.0], [6.0, 7.0, 7.0], [3.0, 2.0, 2.0]]], [[[5.0, 3.0, 2.0], [4.0, 4.0, 4.0], [4.0, 5.0, 3.0]], [[6.0, 7.0, 
7.0], [6.0, 3.0, 4.0], [3.0, 6.0, 1.0]], [[3.0, 3.0, 5.0], [7.0, 7.0, 6.0], [1.0, 4.0, 3.0]]], [[[1.0, 5.0, 7.0], [2.0, 2.0, 
5.0], [6.0, 6.0, 1.0]], [[2.0, 3.0, 1.0], [2.0, 2.0, 6.0], [3.0, 6.0, 2.0]], [[6.0, 2.0, 4.0], [7.0, 7.0, 5.0], [5.0, 6.0, 
4.0]]], [[[6.0, 2.0, 8.0], [8.0, 1.0, 5.0], [2.0, 7.0, 7.0]], [[7.0, 1.0, 1.0], [7.0, 7.0, 2.0], [4.0, 5.0, 5.0]], [[5.0, 7.0, 
7.0], [7.0, 3.0, 1.0], [1.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 1.0], [3.0, 2.0, 5.0], [3.0, 5.0, 4.0]], [[1.0, 4.0, 6.0], [8.0, 4.0, 
2.0], [1.0, 2.0, 2.0]], [[3.0, 1.0, 1.0], [7.0, 8.0, 5.0], [5.0, 6.0, 3.0]]], [[[1.0, 3.0, 5.0], [5.0, 4.0, 8.0], [8.0, 1.0, 
4.0]], [[7.0, 6.0, 3.0], [3.0, 5.0, 2.0], [4.0, 8.0, 3.0]], [[7.0, 8.0, 3.0], [8.0, 3.0, 4.0], [2.0, 6.0, 5.0]]], [[[3.0, 1.0, 
5.0], [2.0, 4.0, 4.0], [2.0, 5.0, 6.0]], [[3.0, 1.0, 8.0], [1.0, 4.0, 5.0], [3.0, 7.0, 3.0]], [[5.0, 7.0, 2.0], [4.0, 6.0, 
5.0], [1.0, 8.0, 5.0]]], [[[1.0, 5.0, 1.0], [7.0, 8.0, 8.0], [7.0, 8.0, 8.0]], [[2.0, 8.0, 7.0], [8.0, 1.0, 8.0], [8.0, 6.0, 
8.0]], [[1.0, 6.0, 5.0], [2.0, 8.0, 5.0], [5.0, 2.0, 8.0]]]]))
      val=Data(numpy.array([[7.0, 7.0, 1.0], [8.0, 8.0, 8.0], [3.0, 3.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 8.0, 3.0], [7.0, 4.0, 3.0], [5.0, 2.0, 1.0]]))
      arg[2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 7.0], [1.0, 1.0, 5.0], [1.0, 5.0, 2.0]], [[7.0, 3.0, 2.0], [7.0, 5.0, 4.0], [5.0, 
6.0, 5.0]], [[1.0, 4.0, 2.0], [6.0, 7.0, 7.0], [3.0, 2.0, 2.0]]], [[[5.0, 3.0, 2.0], [4.0, 4.0, 4.0], [4.0, 5.0, 3.0]], [[6.0, 
7.0, 7.0], [6.0, 3.0, 4.0], [3.0, 6.0, 1.0]], [[3.0, 3.0, 5.0], [7.0, 7.0, 6.0], [1.0, 4.0, 3.0]]], [[[1.0, 5.0, 7.0], [2.0, 
2.0, 5.0], [6.0, 6.0, 1.0]], [[7.0, 7.0, 1.0], [8.0, 8.0, 8.0], [3.0, 3.0, 6.0]], [[6.0, 2.0, 4.0], [7.0, 7.0, 5.0], [5.0, 6.0, 
4.0]]], [[[6.0, 2.0, 8.0], [8.0, 1.0, 5.0], [2.0, 7.0, 7.0]], [[7.0, 1.0, 1.0], [7.0, 7.0, 2.0], [4.0, 5.0, 5.0]], [[5.0, 7.0, 
7.0], [7.0, 3.0, 1.0], [1.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 1.0], [3.0, 2.0, 5.0], [3.0, 5.0, 4.0]], 
[[1.0, 4.0, 6.0], [8.0, 4.0, 2.0], [1.0, 2.0, 2.0]], [[3.0, 1.0, 1.0], [7.0, 8.0, 5.0], [5.0, 6.0, 3.0]]], [[[1.0, 3.0, 5.0], 
[5.0, 4.0, 8.0], [8.0, 1.0, 4.0]], [[7.0, 6.0, 3.0], [3.0, 5.0, 2.0], [4.0, 8.0, 3.0]], [[7.0, 8.0, 3.0], [8.0, 3.0, 4.0], 
[2.0, 6.0, 5.0]]], [[[3.0, 1.0, 5.0], [2.0, 4.0, 4.0], [2.0, 5.0, 6.0]], [[3.0, 8.0, 3.0], [7.0, 4.0, 3.0], [5.0, 2.0, 1.0]], 
[[5.0, 7.0, 2.0], [4.0, 6.0, 5.0], [1.0, 8.0, 5.0]]], [[[1.0, 5.0, 1.0], [7.0, 8.0, 8.0], [7.0, 8.0, 8.0]], [[2.0, 8.0, 7.0], 
[8.0, 1.0, 8.0], [8.0, 6.0, 8.0]], [[1.0, 6.0, 5.0], [2.0, 8.0, 5.0], [5.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPCN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 3.0], [6.0, 7.0, 5.0], [4.0, 7.0, 5.0]], [[8.0, 2.0, 5.0], [1.0, 4.0, 4.0], [5.0, 3.0, 
3.0]], [[4.0, 2.0, 2.0], [8.0, 7.0, 8.0], [1.0, 3.0, 4.0]]], [[[8.0, 7.0, 4.0], [5.0, 7.0, 2.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 
8.0], [6.0, 8.0, 3.0], [6.0, 6.0, 1.0]], [[7.0, 3.0, 4.0], [1.0, 5.0, 3.0], [8.0, 3.0, 8.0]]], [[[6.0, 3.0, 7.0], [4.0, 6.0, 
5.0], [2.0, 1.0, 3.0]], [[6.0, 1.0, 8.0], [2.0, 1.0, 7.0], [7.0, 7.0, 2.0]], [[6.0, 4.0, 1.0], [6.0, 5.0, 5.0], [1.0, 3.0, 
2.0]]], [[[4.0, 6.0, 7.0], [1.0, 2.0, 4.0], [7.0, 6.0, 8.0]], [[1.0, 7.0, 1.0], [3.0, 4.0, 7.0], [2.0, 7.0, 3.0]], [[4.0, 4.0, 
4.0], [3.0, 3.0, 1.0], [4.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 3.0], [2.0, 7.0, 6.0], [7.0, 5.0, 6.0]], [[5.0, 3.0, 2.0], [7.0, 6.0, 
6.0], [1.0, 2.0, 6.0]], [[2.0, 1.0, 3.0], [1.0, 3.0, 1.0], [7.0, 2.0, 5.0]]], [[[1.0, 1.0, 7.0], [2.0, 1.0, 7.0], [7.0, 1.0, 
8.0]], [[2.0, 8.0, 7.0], [1.0, 4.0, 4.0], [3.0, 7.0, 5.0]], [[3.0, 4.0, 4.0], [3.0, 4.0, 6.0], [2.0, 2.0, 6.0]]], [[[4.0, 1.0, 
8.0], [3.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[8.0, 4.0, 4.0], [8.0, 8.0, 6.0], [6.0, 6.0, 6.0]], [[3.0, 5.0, 3.0], [7.0, 3.0, 
5.0], [5.0, 7.0, 3.0]]], [[[3.0, 5.0, 5.0], [1.0, 7.0, 6.0], [5.0, 1.0, 8.0]], [[8.0, 4.0, 6.0], [6.0, 2.0, 1.0], [5.0, 8.0, 
6.0]], [[4.0, 3.0, 2.0], [6.0, 7.0, 5.0], [7.0, 5.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[5.0, 2.0, 5.0], [6.0, 2.0, 6.0], [5.0, 5.0, 7.0]])+(1.-msk_val)*numpy.array([[8.0, 2.0, 2.0], 
[2.0, 6.0, 3.0], [8.0, 4.0, 7.0]])
      arg[2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 3.0], [6.0, 7.0, 5.0], [4.0, 7.0, 5.0]], [[8.0, 2.0, 5.0], [1.0, 4.0, 4.0], [5.0, 
3.0, 3.0]], [[4.0, 2.0, 2.0], [8.0, 7.0, 8.0], [1.0, 3.0, 4.0]]], [[[8.0, 7.0, 4.0], [5.0, 7.0, 2.0], [4.0, 8.0, 5.0]], [[4.0, 
5.0, 8.0], [6.0, 8.0, 3.0], [6.0, 6.0, 1.0]], [[7.0, 3.0, 4.0], [1.0, 5.0, 3.0], [8.0, 3.0, 8.0]]], [[[6.0, 3.0, 7.0], [4.0, 
6.0, 5.0], [2.0, 1.0, 3.0]], [[5.0, 2.0, 5.0], [6.0, 2.0, 6.0], [5.0, 5.0, 7.0]], [[6.0, 4.0, 1.0], [6.0, 5.0, 5.0], [1.0, 3.0, 
2.0]]], [[[4.0, 6.0, 7.0], [1.0, 2.0, 4.0], [7.0, 6.0, 8.0]], [[1.0, 7.0, 1.0], [3.0, 4.0, 7.0], [2.0, 7.0, 3.0]], [[4.0, 4.0, 
4.0], [3.0, 3.0, 1.0], [4.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 3.0], [2.0, 7.0, 6.0], [7.0, 5.0, 6.0]], 
[[5.0, 3.0, 2.0], [7.0, 6.0, 6.0], [1.0, 2.0, 6.0]], [[2.0, 1.0, 3.0], [1.0, 3.0, 1.0], [7.0, 2.0, 5.0]]], [[[1.0, 1.0, 7.0], 
[2.0, 1.0, 7.0], [7.0, 1.0, 8.0]], [[2.0, 8.0, 7.0], [1.0, 4.0, 4.0], [3.0, 7.0, 5.0]], [[3.0, 4.0, 4.0], [3.0, 4.0, 6.0], 
[2.0, 2.0, 6.0]]], [[[4.0, 1.0, 8.0], [3.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[8.0, 2.0, 2.0], [2.0, 6.0, 3.0], [8.0, 4.0, 7.0]], 
[[3.0, 5.0, 3.0], [7.0, 3.0, 5.0], [5.0, 7.0, 3.0]]], [[[3.0, 5.0, 5.0], [1.0, 7.0, 6.0], [5.0, 1.0, 8.0]], [[8.0, 4.0, 6.0], 
[6.0, 2.0, 1.0], [5.0, 8.0, 6.0]], [[4.0, 3.0, 2.0], [6.0, 7.0, 5.0], [7.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RUCN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 4.0], [1.0, 4.0, 3.0], [6.0, 3.0, 8.0]], [[5.0, 4.0, 1.0], [7.0, 4.0, 8.0], [5.0, 7.0, 
2.0]], [[1.0, 7.0, 3.0], [4.0, 5.0, 6.0], [3.0, 2.0, 7.0]]], [[[8.0, 7.0, 5.0], [4.0, 2.0, 7.0], [1.0, 3.0, 8.0]], [[5.0, 7.0, 
5.0], [6.0, 8.0, 1.0], [7.0, 2.0, 1.0]], [[8.0, 3.0, 4.0], [3.0, 1.0, 3.0], [1.0, 6.0, 6.0]]], [[[1.0, 2.0, 3.0], [1.0, 3.0, 
1.0], [5.0, 4.0, 4.0]], [[6.0, 3.0, 7.0], [6.0, 7.0, 6.0], [5.0, 6.0, 4.0]], [[3.0, 1.0, 8.0], [8.0, 1.0, 4.0], [4.0, 8.0, 
6.0]]], [[[6.0, 1.0, 1.0], [7.0, 6.0, 2.0], [5.0, 3.0, 3.0]], [[1.0, 2.0, 4.0], [2.0, 4.0, 1.0], [5.0, 2.0, 1.0]], [[2.0, 4.0, 
3.0], [8.0, 2.0, 3.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 1.0], [5.0, 2.0, 4.0], [3.0, 8.0, 4.0]], [[4.0, 3.0, 2.0], [8.0, 5.0, 
8.0], [1.0, 8.0, 1.0]], [[7.0, 3.0, 2.0], [4.0, 4.0, 2.0], [8.0, 6.0, 6.0]]], [[[1.0, 3.0, 6.0], [3.0, 7.0, 1.0], [6.0, 3.0, 
7.0]], [[4.0, 2.0, 2.0], [2.0, 2.0, 2.0], [1.0, 5.0, 3.0]], [[8.0, 2.0, 5.0], [4.0, 6.0, 5.0], [3.0, 3.0, 3.0]]], [[[1.0, 4.0, 
7.0], [8.0, 7.0, 4.0], [1.0, 8.0, 7.0]], [[1.0, 2.0, 8.0], [4.0, 6.0, 3.0], [3.0, 3.0, 2.0]], [[7.0, 7.0, 5.0], [4.0, 8.0, 
6.0], [1.0, 5.0, 4.0]]], [[[7.0, 1.0, 3.0], [7.0, 8.0, 8.0], [3.0, 1.0, 3.0]], [[4.0, 3.0, 8.0], [3.0, 6.0, 1.0], [4.0, 5.0, 
7.0]], [[3.0, 3.0, 1.0], [3.0, 3.0, 2.0], [7.0, 3.0, 3.0]]]]))
      val=numpy.array([[[[2.0, 7.0, 8.0], [4.0, 3.0, 2.0], [8.0, 7.0, 3.0]], [[4.0, 8.0, 5.0], [7.0, 5.0, 3.0], [5.0, 1.0, 
2.0]]], [[[3.0, 2.0, 8.0], [4.0, 5.0, 2.0], [2.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], [6.0, 7.0, 6.0], [5.0, 2.0, 3.0]]]])
      arg[1:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 4.0], [1.0, 4.0, 3.0], [6.0, 3.0, 8.0]], [[5.0, 4.0, 1.0], [7.0, 4.0, 8.0], [5.0, 
7.0, 2.0]], [[1.0, 7.0, 3.0], [4.0, 5.0, 6.0], [3.0, 2.0, 7.0]]], [[[2.0, 7.0, 8.0], [4.0, 3.0, 2.0], [8.0, 7.0, 3.0]], [[4.0, 
8.0, 5.0], [7.0, 5.0, 3.0], [5.0, 1.0, 2.0]], [[8.0, 3.0, 4.0], [3.0, 1.0, 3.0], [1.0, 6.0, 6.0]]], [[[3.0, 2.0, 8.0], [4.0, 
5.0, 2.0], [2.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], [6.0, 7.0, 6.0], [5.0, 2.0, 3.0]], [[3.0, 1.0, 8.0], [8.0, 1.0, 4.0], [4.0, 8.0, 
6.0]]], [[[6.0, 1.0, 1.0], [7.0, 6.0, 2.0], [5.0, 3.0, 3.0]], [[1.0, 2.0, 4.0], [2.0, 4.0, 1.0], [5.0, 2.0, 1.0]], [[2.0, 4.0, 
3.0], [8.0, 2.0, 3.0], [4.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 1.0], [5.0, 2.0, 4.0], [3.0, 8.0, 4.0]], 
[[4.0, 3.0, 2.0], [8.0, 5.0, 8.0], [1.0, 8.0, 1.0]], [[7.0, 3.0, 2.0], [4.0, 4.0, 2.0], [8.0, 6.0, 6.0]]], [[[2.0, 7.0, 8.0], 
[4.0, 3.0, 2.0], [8.0, 7.0, 3.0]], [[4.0, 8.0, 5.0], [7.0, 5.0, 3.0], [5.0, 1.0, 2.0]], [[8.0, 2.0, 5.0], [4.0, 6.0, 5.0], 
[3.0, 3.0, 3.0]]], [[[3.0, 2.0, 8.0], [4.0, 5.0, 2.0], [2.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], [6.0, 7.0, 6.0], [5.0, 2.0, 3.0]], 
[[7.0, 7.0, 5.0], [4.0, 8.0, 6.0], [1.0, 5.0, 4.0]]], [[[7.0, 1.0, 3.0], [7.0, 8.0, 8.0], [3.0, 1.0, 3.0]], [[4.0, 3.0, 8.0], 
[3.0, 6.0, 1.0], [4.0, 5.0, 7.0]], [[3.0, 3.0, 1.0], [3.0, 3.0, 2.0], [7.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RUCN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 6.0], [6.0, 7.0, 1.0], [7.0, 8.0, 2.0]], [[4.0, 4.0, 2.0], [4.0, 6.0, 5.0], [2.0, 4.0, 
3.0]], [[3.0, 6.0, 6.0], [2.0, 2.0, 2.0], [2.0, 2.0, 5.0]]], [[[4.0, 4.0, 5.0], [4.0, 2.0, 2.0], [6.0, 7.0, 5.0]], [[7.0, 5.0, 
6.0], [6.0, 7.0, 1.0], [2.0, 7.0, 4.0]], [[4.0, 4.0, 5.0], [8.0, 1.0, 6.0], [7.0, 8.0, 2.0]]], [[[7.0, 6.0, 6.0], [4.0, 8.0, 
6.0], [2.0, 2.0, 2.0]], [[7.0, 6.0, 6.0], [3.0, 2.0, 3.0], [4.0, 8.0, 2.0]], [[1.0, 6.0, 8.0], [6.0, 2.0, 1.0], [3.0, 3.0, 
8.0]]], [[[5.0, 3.0, 1.0], [2.0, 4.0, 3.0], [8.0, 4.0, 3.0]], [[1.0, 6.0, 5.0], [3.0, 3.0, 5.0], [2.0, 3.0, 6.0]], [[6.0, 1.0, 
6.0], [6.0, 5.0, 8.0], [3.0, 5.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 6.0], [3.0, 4.0, 4.0], [3.0, 3.0, 8.0]], [[8.0, 2.0, 7.0], [4.0, 4.0, 
5.0], [6.0, 6.0, 3.0]], [[6.0, 5.0, 6.0], [2.0, 1.0, 4.0], [7.0, 3.0, 4.0]]], [[[4.0, 6.0, 6.0], [7.0, 1.0, 1.0], [7.0, 3.0, 
5.0]], [[8.0, 3.0, 6.0], [2.0, 4.0, 6.0], [4.0, 8.0, 5.0]], [[4.0, 6.0, 4.0], [2.0, 2.0, 3.0], [8.0, 7.0, 1.0]]], [[[2.0, 3.0, 
3.0], [3.0, 1.0, 5.0], [2.0, 7.0, 1.0]], [[8.0, 4.0, 1.0], [6.0, 1.0, 7.0], [6.0, 8.0, 8.0]], [[4.0, 7.0, 6.0], [8.0, 3.0, 
6.0], [7.0, 1.0, 6.0]]], [[[8.0, 1.0, 1.0], [8.0, 5.0, 5.0], [3.0, 1.0, 8.0]], [[7.0, 2.0, 7.0], [2.0, 3.0, 2.0], [6.0, 1.0, 
6.0]], [[8.0, 8.0, 4.0], [2.0, 2.0, 5.0], [7.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([[[[7.0, 3.0, 5.0], [7.0, 1.0, 5.0], [6.0, 6.0, 6.0]], [[6.0, 4.0, 8.0], [7.0, 4.0, 8.0], [3.0, 2.0, 
7.0]]], [[[1.0, 3.0, 1.0], [2.0, 6.0, 2.0], [8.0, 5.0, 6.0]], [[4.0, 7.0, 7.0], [1.0, 4.0, 2.0], [8.0, 4.0, 
7.0]]]]),self.functionspace)
      arg[1:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 6.0], [6.0, 7.0, 1.0], [7.0, 8.0, 2.0]], [[4.0, 4.0, 2.0], [4.0, 6.0, 5.0], [2.0, 
4.0, 3.0]], [[3.0, 6.0, 6.0], [2.0, 2.0, 2.0], [2.0, 2.0, 5.0]]], [[[7.0, 3.0, 5.0], [7.0, 1.0, 5.0], [6.0, 6.0, 6.0]], [[6.0, 
4.0, 8.0], [7.0, 4.0, 8.0], [3.0, 2.0, 7.0]], [[4.0, 4.0, 5.0], [8.0, 1.0, 6.0], [7.0, 8.0, 2.0]]], [[[1.0, 3.0, 1.0], [2.0, 
6.0, 2.0], [8.0, 5.0, 6.0]], [[4.0, 7.0, 7.0], [1.0, 4.0, 2.0], [8.0, 4.0, 7.0]], [[1.0, 6.0, 8.0], [6.0, 2.0, 1.0], [3.0, 3.0, 
8.0]]], [[[5.0, 3.0, 1.0], [2.0, 4.0, 3.0], [8.0, 4.0, 3.0]], [[1.0, 6.0, 5.0], [3.0, 3.0, 5.0], [2.0, 3.0, 6.0]], [[6.0, 1.0, 
6.0], [6.0, 5.0, 8.0], [3.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 6.0], [3.0, 4.0, 4.0], [3.0, 3.0, 8.0]], 
[[8.0, 2.0, 7.0], [4.0, 4.0, 5.0], [6.0, 6.0, 3.0]], [[6.0, 5.0, 6.0], [2.0, 1.0, 4.0], [7.0, 3.0, 4.0]]], [[[7.0, 3.0, 5.0], 
[7.0, 1.0, 5.0], [6.0, 6.0, 6.0]], [[6.0, 4.0, 8.0], [7.0, 4.0, 8.0], [3.0, 2.0, 7.0]], [[4.0, 6.0, 4.0], [2.0, 2.0, 3.0], 
[8.0, 7.0, 1.0]]], [[[1.0, 3.0, 1.0], [2.0, 6.0, 2.0], [8.0, 5.0, 6.0]], [[4.0, 7.0, 7.0], [1.0, 4.0, 2.0], [8.0, 4.0, 7.0]], 
[[4.0, 7.0, 6.0], [8.0, 3.0, 6.0], [7.0, 1.0, 6.0]]], [[[8.0, 1.0, 1.0], [8.0, 5.0, 5.0], [3.0, 1.0, 8.0]], [[7.0, 2.0, 7.0], 
[2.0, 3.0, 2.0], [6.0, 1.0, 6.0]], [[8.0, 8.0, 4.0], [2.0, 2.0, 5.0], [7.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RUCN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 4.0], [1.0, 8.0, 8.0], [6.0, 2.0, 4.0]], [[1.0, 3.0, 4.0], [3.0, 8.0, 1.0], [8.0, 1.0, 
7.0]], [[3.0, 4.0, 6.0], [2.0, 2.0, 6.0], [1.0, 1.0, 2.0]]], [[[7.0, 1.0, 6.0], [3.0, 4.0, 3.0], [1.0, 2.0, 7.0]], [[6.0, 7.0, 
6.0], [5.0, 3.0, 8.0], [4.0, 4.0, 3.0]], [[8.0, 3.0, 7.0], [7.0, 8.0, 6.0], [7.0, 3.0, 5.0]]], [[[1.0, 2.0, 6.0], [5.0, 8.0, 
1.0], [6.0, 8.0, 7.0]], [[2.0, 3.0, 3.0], [4.0, 7.0, 1.0], [1.0, 2.0, 4.0]], [[1.0, 6.0, 2.0], [7.0, 6.0, 7.0], [8.0, 8.0, 
3.0]]], [[[8.0, 3.0, 7.0], [1.0, 1.0, 3.0], [3.0, 4.0, 4.0]], [[1.0, 1.0, 7.0], [5.0, 3.0, 3.0], [4.0, 1.0, 4.0]], [[4.0, 4.0, 
3.0], [6.0, 6.0, 1.0], [5.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 3.0], [4.0, 2.0, 4.0], [1.0, 5.0, 3.0]], [[5.0, 4.0, 2.0], [6.0, 2.0, 
3.0], [7.0, 8.0, 5.0]], [[2.0, 5.0, 8.0], [3.0, 7.0, 3.0], [1.0, 3.0, 7.0]]], [[[5.0, 5.0, 4.0], [3.0, 2.0, 1.0], [1.0, 7.0, 
2.0]], [[1.0, 4.0, 5.0], [3.0, 1.0, 5.0], [6.0, 6.0, 5.0]], [[5.0, 4.0, 4.0], [3.0, 5.0, 2.0], [1.0, 8.0, 5.0]]], [[[2.0, 2.0, 
3.0], [8.0, 6.0, 8.0], [8.0, 1.0, 5.0]], [[7.0, 7.0, 1.0], [5.0, 6.0, 7.0], [1.0, 3.0, 4.0]], [[6.0, 6.0, 3.0], [1.0, 2.0, 
5.0], [2.0, 1.0, 2.0]]], [[[6.0, 1.0, 7.0], [1.0, 3.0, 7.0], [6.0, 6.0, 2.0]], [[6.0, 5.0, 4.0], [2.0, 4.0, 7.0], [8.0, 7.0, 
1.0]], [[2.0, 5.0, 8.0], [7.0, 1.0, 5.0], [5.0, 6.0, 3.0]]]]))
      val=Data(numpy.array([[[[3.0, 1.0, 2.0], [7.0, 4.0, 7.0], [7.0, 2.0, 6.0]], [[4.0, 4.0, 4.0], [7.0, 5.0, 4.0], [8.0, 5.0, 
7.0]]], [[[6.0, 5.0, 2.0], [4.0, 2.0, 4.0], [4.0, 2.0, 6.0]], [[6.0, 8.0, 2.0], [7.0, 8.0, 6.0], [4.0, 8.0, 
7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 5.0], [2.0, 1.0, 6.0], [7.0, 6.0, 7.0]], [[5.0, 5.0, 7.0], [6.0, 2.0, 
4.0], [4.0, 7.0, 2.0]]], [[[2.0, 1.0, 4.0], [7.0, 4.0, 8.0], [2.0, 8.0, 6.0]], [[3.0, 6.0, 6.0], [3.0, 1.0, 3.0], [5.0, 7.0, 
6.0]]]]))
      arg[1:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 4.0], [1.0, 8.0, 8.0], [6.0, 2.0, 4.0]], [[1.0, 3.0, 4.0], [3.0, 8.0, 1.0], [8.0, 
1.0, 7.0]], [[3.0, 4.0, 6.0], [2.0, 2.0, 6.0], [1.0, 1.0, 2.0]]], [[[3.0, 1.0, 2.0], [7.0, 4.0, 7.0], [7.0, 2.0, 6.0]], [[4.0, 
4.0, 4.0], [7.0, 5.0, 4.0], [8.0, 5.0, 7.0]], [[8.0, 3.0, 7.0], [7.0, 8.0, 6.0], [7.0, 3.0, 5.0]]], [[[6.0, 5.0, 2.0], [4.0, 
2.0, 4.0], [4.0, 2.0, 6.0]], [[6.0, 8.0, 2.0], [7.0, 8.0, 6.0], [4.0, 8.0, 7.0]], [[1.0, 6.0, 2.0], [7.0, 6.0, 7.0], [8.0, 8.0, 
3.0]]], [[[8.0, 3.0, 7.0], [1.0, 1.0, 3.0], [3.0, 4.0, 4.0]], [[1.0, 1.0, 7.0], [5.0, 3.0, 3.0], [4.0, 1.0, 4.0]], [[4.0, 4.0, 
3.0], [6.0, 6.0, 1.0], [5.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 3.0], [4.0, 2.0, 4.0], [1.0, 5.0, 3.0]], 
[[5.0, 4.0, 2.0], [6.0, 2.0, 3.0], [7.0, 8.0, 5.0]], [[2.0, 5.0, 8.0], [3.0, 7.0, 3.0], [1.0, 3.0, 7.0]]], [[[3.0, 2.0, 5.0], 
[2.0, 1.0, 6.0], [7.0, 6.0, 7.0]], [[5.0, 5.0, 7.0], [6.0, 2.0, 4.0], [4.0, 7.0, 2.0]], [[5.0, 4.0, 4.0], [3.0, 5.0, 2.0], 
[1.0, 8.0, 5.0]]], [[[2.0, 1.0, 4.0], [7.0, 4.0, 8.0], [2.0, 8.0, 6.0]], [[3.0, 6.0, 6.0], [3.0, 1.0, 3.0], [5.0, 7.0, 6.0]], 
[[6.0, 6.0, 3.0], [1.0, 2.0, 5.0], [2.0, 1.0, 2.0]]], [[[6.0, 1.0, 7.0], [1.0, 3.0, 7.0], [6.0, 6.0, 2.0]], [[6.0, 5.0, 4.0], 
[2.0, 4.0, 7.0], [8.0, 7.0, 1.0]], [[2.0, 5.0, 8.0], [7.0, 1.0, 5.0], [5.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RUCN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 4.0], [6.0, 2.0, 3.0], [2.0, 3.0, 6.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 5.0], [4.0, 2.0, 
7.0]], [[8.0, 1.0, 8.0], [5.0, 1.0, 2.0], [1.0, 3.0, 8.0]]], [[[6.0, 2.0, 8.0], [5.0, 6.0, 5.0], [3.0, 1.0, 1.0]], [[6.0, 1.0, 
6.0], [1.0, 3.0, 1.0], [1.0, 7.0, 7.0]], [[8.0, 2.0, 2.0], [1.0, 5.0, 5.0], [1.0, 5.0, 8.0]]], [[[6.0, 7.0, 6.0], [2.0, 1.0, 
6.0], [8.0, 4.0, 2.0]], [[6.0, 2.0, 4.0], [8.0, 8.0, 2.0], [6.0, 7.0, 8.0]], [[7.0, 1.0, 3.0], [1.0, 8.0, 6.0], [7.0, 5.0, 
3.0]]], [[[1.0, 1.0, 7.0], [1.0, 3.0, 5.0], [2.0, 7.0, 6.0]], [[7.0, 2.0, 3.0], [2.0, 4.0, 6.0], [7.0, 6.0, 1.0]], [[8.0, 6.0, 
4.0], [2.0, 4.0, 4.0], [8.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 1.0], [3.0, 2.0, 4.0], [2.0, 6.0, 1.0]], [[1.0, 4.0, 5.0], [5.0, 6.0, 
2.0], [3.0, 2.0, 4.0]], [[7.0, 5.0, 1.0], [5.0, 7.0, 6.0], [2.0, 7.0, 5.0]]], [[[5.0, 8.0, 8.0], [2.0, 2.0, 5.0], [3.0, 1.0, 
7.0]], [[8.0, 1.0, 2.0], [4.0, 7.0, 3.0], [6.0, 1.0, 6.0]], [[6.0, 8.0, 6.0], [8.0, 3.0, 7.0], [2.0, 2.0, 6.0]]], [[[6.0, 7.0, 
7.0], [5.0, 8.0, 2.0], [5.0, 7.0, 4.0]], [[7.0, 8.0, 3.0], [6.0, 6.0, 8.0], [5.0, 3.0, 6.0]], [[5.0, 3.0, 3.0], [8.0, 4.0, 
2.0], [4.0, 1.0, 7.0]]], [[[2.0, 3.0, 4.0], [2.0, 1.0, 7.0], [2.0, 1.0, 1.0]], [[8.0, 2.0, 2.0], [3.0, 8.0, 4.0], [1.0, 4.0, 
6.0]], [[6.0, 6.0, 1.0], [4.0, 4.0, 5.0], [8.0, 1.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0, 2.0, 7.0], [5.0, 1.0, 4.0], [8.0, 6.0, 8.0]], [[5.0, 6.0, 2.0], [6.0, 8.0, 1.0], [7.0, 
8.0, 1.0]]], [[[5.0, 8.0, 1.0], [7.0, 3.0, 6.0], [2.0, 8.0, 6.0]], [[8.0, 2.0, 6.0], [4.0, 3.0, 4.0], [1.0, 4.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 8.0, 3.0], [8.0, 2.0, 2.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 1.0], [7.0, 2.0, 1.0], 
[7.0, 4.0, 5.0]]], [[[2.0, 5.0, 3.0], [7.0, 5.0, 4.0], [8.0, 4.0, 5.0]], [[1.0, 7.0, 8.0], [6.0, 2.0, 6.0], [1.0, 6.0, 3.0]]]])
      arg[1:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 4.0], [6.0, 2.0, 3.0], [2.0, 3.0, 6.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 5.0], [4.0, 
2.0, 7.0]], [[8.0, 1.0, 8.0], [5.0, 1.0, 2.0], [1.0, 3.0, 8.0]]], [[[6.0, 2.0, 7.0], [5.0, 1.0, 4.0], [8.0, 6.0, 8.0]], [[5.0, 
6.0, 2.0], [6.0, 8.0, 1.0], [7.0, 8.0, 1.0]], [[8.0, 2.0, 2.0], [1.0, 5.0, 5.0], [1.0, 5.0, 8.0]]], [[[5.0, 8.0, 1.0], [7.0, 
3.0, 6.0], [2.0, 8.0, 6.0]], [[8.0, 2.0, 6.0], [4.0, 3.0, 4.0], [1.0, 4.0, 6.0]], [[7.0, 1.0, 3.0], [1.0, 8.0, 6.0], [7.0, 5.0, 
3.0]]], [[[1.0, 1.0, 7.0], [1.0, 3.0, 5.0], [2.0, 7.0, 6.0]], [[7.0, 2.0, 3.0], [2.0, 4.0, 6.0], [7.0, 6.0, 1.0]], [[8.0, 6.0, 
4.0], [2.0, 4.0, 4.0], [8.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 1.0], [3.0, 2.0, 4.0], [2.0, 6.0, 1.0]], 
[[1.0, 4.0, 5.0], [5.0, 6.0, 2.0], [3.0, 2.0, 4.0]], [[7.0, 5.0, 1.0], [5.0, 7.0, 6.0], [2.0, 7.0, 5.0]]], [[[2.0, 8.0, 3.0], 
[8.0, 2.0, 2.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 1.0], [7.0, 2.0, 1.0], [7.0, 4.0, 5.0]], [[6.0, 8.0, 6.0], [8.0, 3.0, 7.0], 
[2.0, 2.0, 6.0]]], [[[2.0, 5.0, 3.0], [7.0, 5.0, 4.0], [8.0, 4.0, 5.0]], [[1.0, 7.0, 8.0], [6.0, 2.0, 6.0], [1.0, 6.0, 3.0]], 
[[5.0, 3.0, 3.0], [8.0, 4.0, 2.0], [4.0, 1.0, 7.0]]], [[[2.0, 3.0, 4.0], [2.0, 1.0, 7.0], [2.0, 1.0, 1.0]], [[8.0, 2.0, 2.0], 
[3.0, 8.0, 4.0], [1.0, 4.0, 6.0]], [[6.0, 6.0, 1.0], [4.0, 4.0, 5.0], [8.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_URRN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [8.0, 3.0, 8.0], [6.0, 2.0, 2.0]], [[4.0, 2.0, 5.0], [2.0, 7.0, 1.0], [5.0, 3.0, 
7.0]], [[1.0, 7.0, 8.0], [1.0, 6.0, 4.0], [8.0, 7.0, 1.0]]], [[[3.0, 3.0, 8.0], [8.0, 8.0, 3.0], [8.0, 3.0, 3.0]], [[7.0, 1.0, 
1.0], [3.0, 8.0, 3.0], [7.0, 1.0, 7.0]], [[6.0, 6.0, 7.0], [4.0, 6.0, 1.0], [5.0, 7.0, 1.0]]], [[[7.0, 6.0, 2.0], [8.0, 7.0, 
2.0], [5.0, 7.0, 1.0]], [[5.0, 1.0, 5.0], [3.0, 6.0, 8.0], [1.0, 5.0, 7.0]], [[5.0, 2.0, 4.0], [4.0, 6.0, 6.0], [6.0, 2.0, 
2.0]]], [[[5.0, 2.0, 1.0], [3.0, 7.0, 5.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 3.0], [8.0, 5.0, 7.0], [5.0, 7.0, 2.0]], [[1.0, 4.0, 
1.0], [5.0, 6.0, 3.0], [5.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 2.0], [3.0, 8.0, 2.0], [1.0, 5.0, 1.0]], [[1.0, 6.0, 2.0], [4.0, 1.0, 
7.0], [5.0, 5.0, 7.0]], [[3.0, 2.0, 4.0], [5.0, 6.0, 7.0], [6.0, 3.0, 5.0]]], [[[1.0, 6.0, 7.0], [7.0, 5.0, 1.0], [2.0, 1.0, 
2.0]], [[8.0, 3.0, 5.0], [6.0, 1.0, 1.0], [8.0, 5.0, 1.0]], [[7.0, 3.0, 2.0], [4.0, 6.0, 6.0], [5.0, 2.0, 2.0]]], [[[8.0, 4.0, 
8.0], [2.0, 4.0, 2.0], [4.0, 6.0, 6.0]], [[1.0, 3.0, 7.0], [6.0, 1.0, 3.0], [2.0, 4.0, 4.0]], [[6.0, 4.0, 2.0], [8.0, 6.0, 
2.0], [6.0, 3.0, 4.0]]], [[[5.0, 7.0, 1.0], [2.0, 4.0, 5.0], [3.0, 5.0, 2.0]], [[2.0, 1.0, 7.0], [8.0, 5.0, 7.0], [4.0, 2.0, 
2.0]], [[4.0, 1.0, 8.0], [7.0, 7.0, 7.0], [2.0, 4.0, 4.0]]]]))
      val=numpy.array([[[[5.0, 7.0, 8.0]]], [[[8.0, 7.0, 2.0]]], [[[7.0, 8.0, 8.0]]]])
      arg[:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [8.0, 3.0, 8.0], [6.0, 2.0, 2.0]], [[4.0, 2.0, 5.0], [5.0, 7.0, 8.0], [5.0, 
3.0, 7.0]], [[1.0, 7.0, 8.0], [1.0, 6.0, 4.0], [8.0, 7.0, 1.0]]], [[[3.0, 3.0, 8.0], [8.0, 8.0, 3.0], [8.0, 3.0, 3.0]], [[7.0, 
1.0, 1.0], [8.0, 7.0, 2.0], [7.0, 1.0, 7.0]], [[6.0, 6.0, 7.0], [4.0, 6.0, 1.0], [5.0, 7.0, 1.0]]], [[[7.0, 6.0, 2.0], [8.0, 
7.0, 2.0], [5.0, 7.0, 1.0]], [[5.0, 1.0, 5.0], [7.0, 8.0, 8.0], [1.0, 5.0, 7.0]], [[5.0, 2.0, 4.0], [4.0, 6.0, 6.0], [6.0, 2.0, 
2.0]]], [[[5.0, 2.0, 1.0], [3.0, 7.0, 5.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 3.0], [8.0, 5.0, 7.0], [5.0, 7.0, 2.0]], [[1.0, 4.0, 
1.0], [5.0, 6.0, 3.0], [5.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 2.0], [3.0, 8.0, 2.0], [1.0, 5.0, 1.0]], 
[[1.0, 6.0, 2.0], [5.0, 7.0, 8.0], [5.0, 5.0, 7.0]], [[3.0, 2.0, 4.0], [5.0, 6.0, 7.0], [6.0, 3.0, 5.0]]], [[[1.0, 6.0, 7.0], 
[7.0, 5.0, 1.0], [2.0, 1.0, 2.0]], [[8.0, 3.0, 5.0], [8.0, 7.0, 2.0], [8.0, 5.0, 1.0]], [[7.0, 3.0, 2.0], [4.0, 6.0, 6.0], 
[5.0, 2.0, 2.0]]], [[[8.0, 4.0, 8.0], [2.0, 4.0, 2.0], [4.0, 6.0, 6.0]], [[1.0, 3.0, 7.0], [7.0, 8.0, 8.0], [2.0, 4.0, 4.0]], 
[[6.0, 4.0, 2.0], [8.0, 6.0, 2.0], [6.0, 3.0, 4.0]]], [[[5.0, 7.0, 1.0], [2.0, 4.0, 5.0], [3.0, 5.0, 2.0]], [[2.0, 1.0, 7.0], 
[8.0, 5.0, 7.0], [4.0, 2.0, 2.0]], [[4.0, 1.0, 8.0], [7.0, 7.0, 7.0], [2.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_URRN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 7.0], [7.0, 6.0, 8.0], [6.0, 3.0, 5.0]], [[6.0, 8.0, 4.0], [7.0, 8.0, 6.0], [2.0, 5.0, 
6.0]], [[8.0, 7.0, 8.0], [5.0, 3.0, 6.0], [7.0, 3.0, 4.0]]], [[[1.0, 7.0, 5.0], [6.0, 1.0, 6.0], [4.0, 2.0, 7.0]], [[5.0, 4.0, 
6.0], [4.0, 2.0, 6.0], [4.0, 7.0, 2.0]], [[5.0, 4.0, 2.0], [4.0, 2.0, 4.0], [6.0, 1.0, 8.0]]], [[[2.0, 1.0, 5.0], [2.0, 8.0, 
7.0], [3.0, 7.0, 4.0]], [[7.0, 2.0, 8.0], [8.0, 4.0, 8.0], [2.0, 2.0, 4.0]], [[5.0, 6.0, 2.0], [6.0, 2.0, 4.0], [2.0, 6.0, 
8.0]]], [[[6.0, 2.0, 8.0], [2.0, 3.0, 7.0], [3.0, 4.0, 3.0]], [[2.0, 3.0, 6.0], [7.0, 3.0, 8.0], [2.0, 7.0, 2.0]], [[3.0, 7.0, 
5.0], [7.0, 4.0, 2.0], [2.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 6.0], [8.0, 2.0, 3.0], [4.0, 3.0, 3.0]], [[4.0, 7.0, 2.0], [6.0, 1.0, 
6.0], [4.0, 7.0, 1.0]], [[7.0, 4.0, 8.0], [8.0, 2.0, 5.0], [6.0, 4.0, 6.0]]], [[[1.0, 5.0, 4.0], [6.0, 5.0, 6.0], [4.0, 5.0, 
3.0]], [[7.0, 6.0, 5.0], [3.0, 6.0, 5.0], [2.0, 8.0, 1.0]], [[4.0, 8.0, 8.0], [1.0, 4.0, 4.0], [3.0, 1.0, 7.0]]], [[[6.0, 7.0, 
5.0], [7.0, 4.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 6.0, 1.0], [7.0, 8.0, 8.0], [3.0, 4.0, 8.0]], [[6.0, 3.0, 3.0], [7.0, 4.0, 
6.0], [2.0, 7.0, 6.0]]], [[[7.0, 6.0, 1.0], [8.0, 5.0, 6.0], [4.0, 2.0, 5.0]], [[8.0, 6.0, 4.0], [4.0, 3.0, 8.0], [7.0, 1.0, 
7.0]], [[1.0, 8.0, 7.0], [1.0, 3.0, 8.0], [4.0, 2.0, 6.0]]]]))
      val=Data(numpy.array([[[[7.0, 1.0, 3.0]]], [[[8.0, 3.0, 3.0]]], [[[6.0, 2.0, 1.0]]]]),self.functionspace)
      arg[:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 7.0], [7.0, 6.0, 8.0], [6.0, 3.0, 5.0]], [[6.0, 8.0, 4.0], [7.0, 1.0, 3.0], [2.0, 
5.0, 6.0]], [[8.0, 7.0, 8.0], [5.0, 3.0, 6.0], [7.0, 3.0, 4.0]]], [[[1.0, 7.0, 5.0], [6.0, 1.0, 6.0], [4.0, 2.0, 7.0]], [[5.0, 
4.0, 6.0], [8.0, 3.0, 3.0], [4.0, 7.0, 2.0]], [[5.0, 4.0, 2.0], [4.0, 2.0, 4.0], [6.0, 1.0, 8.0]]], [[[2.0, 1.0, 5.0], [2.0, 
8.0, 7.0], [3.0, 7.0, 4.0]], [[7.0, 2.0, 8.0], [6.0, 2.0, 1.0], [2.0, 2.0, 4.0]], [[5.0, 6.0, 2.0], [6.0, 2.0, 4.0], [2.0, 6.0, 
8.0]]], [[[6.0, 2.0, 8.0], [2.0, 3.0, 7.0], [3.0, 4.0, 3.0]], [[2.0, 3.0, 6.0], [7.0, 3.0, 8.0], [2.0, 7.0, 2.0]], [[3.0, 7.0, 
5.0], [7.0, 4.0, 2.0], [2.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 6.0], [8.0, 2.0, 3.0], [4.0, 3.0, 3.0]], 
[[4.0, 7.0, 2.0], [7.0, 1.0, 3.0], [4.0, 7.0, 1.0]], [[7.0, 4.0, 8.0], [8.0, 2.0, 5.0], [6.0, 4.0, 6.0]]], [[[1.0, 5.0, 4.0], 
[6.0, 5.0, 6.0], [4.0, 5.0, 3.0]], [[7.0, 6.0, 5.0], [8.0, 3.0, 3.0], [2.0, 8.0, 1.0]], [[4.0, 8.0, 8.0], [1.0, 4.0, 4.0], 
[3.0, 1.0, 7.0]]], [[[6.0, 7.0, 5.0], [7.0, 4.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 6.0, 1.0], [6.0, 2.0, 1.0], [3.0, 4.0, 8.0]], 
[[6.0, 3.0, 3.0], [7.0, 4.0, 6.0], [2.0, 7.0, 6.0]]], [[[7.0, 6.0, 1.0], [8.0, 5.0, 6.0], [4.0, 2.0, 5.0]], [[8.0, 6.0, 4.0], 
[4.0, 3.0, 8.0], [7.0, 1.0, 7.0]], [[1.0, 8.0, 7.0], [1.0, 3.0, 8.0], [4.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_URRN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 4.0], [1.0, 2.0, 3.0], [5.0, 6.0, 2.0]], [[8.0, 2.0, 6.0], [8.0, 5.0, 6.0], [4.0, 3.0, 
5.0]], [[6.0, 4.0, 1.0], [5.0, 4.0, 1.0], [4.0, 6.0, 6.0]]], [[[3.0, 7.0, 6.0], [1.0, 7.0, 2.0], [4.0, 8.0, 6.0]], [[1.0, 4.0, 
5.0], [6.0, 5.0, 4.0], [1.0, 5.0, 8.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 4.0], [3.0, 5.0, 3.0]]], [[[7.0, 1.0, 5.0], [2.0, 3.0, 
1.0], [8.0, 4.0, 8.0]], [[5.0, 7.0, 3.0], [7.0, 3.0, 2.0], [4.0, 8.0, 4.0]], [[5.0, 6.0, 7.0], [7.0, 1.0, 1.0], [8.0, 7.0, 
5.0]]], [[[8.0, 7.0, 7.0], [6.0, 3.0, 2.0], [1.0, 7.0, 2.0]], [[2.0, 5.0, 3.0], [7.0, 4.0, 7.0], [4.0, 1.0, 4.0]], [[3.0, 3.0, 
7.0], [6.0, 4.0, 5.0], [4.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 2.0], [1.0, 2.0, 4.0], [4.0, 2.0, 5.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 
6.0], [3.0, 4.0, 3.0]], [[4.0, 3.0, 1.0], [4.0, 8.0, 6.0], [8.0, 7.0, 6.0]]], [[[5.0, 8.0, 1.0], [7.0, 2.0, 8.0], [2.0, 1.0, 
1.0]], [[1.0, 8.0, 3.0], [6.0, 8.0, 7.0], [8.0, 4.0, 7.0]], [[1.0, 7.0, 7.0], [3.0, 8.0, 2.0], [8.0, 7.0, 6.0]]], [[[5.0, 5.0, 
2.0], [1.0, 1.0, 6.0], [8.0, 1.0, 7.0]], [[1.0, 2.0, 7.0], [5.0, 4.0, 8.0], [8.0, 8.0, 6.0]], [[7.0, 6.0, 1.0], [8.0, 3.0, 
2.0], [1.0, 5.0, 6.0]]], [[[1.0, 4.0, 2.0], [5.0, 4.0, 4.0], [8.0, 7.0, 2.0]], [[5.0, 5.0, 5.0], [2.0, 4.0, 6.0], [4.0, 5.0, 
1.0]], [[4.0, 6.0, 7.0], [6.0, 8.0, 6.0], [2.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[[5.0, 6.0, 2.0]]], [[[1.0, 6.0, 5.0]]], [[[1.0, 2.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 6.0]]], [[[5.0, 6.0, 8.0]]], [[[4.0, 7.0, 1.0]]]]))
      arg[:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 4.0], [1.0, 2.0, 3.0], [5.0, 6.0, 2.0]], [[8.0, 2.0, 6.0], [5.0, 6.0, 2.0], [4.0, 
3.0, 5.0]], [[6.0, 4.0, 1.0], [5.0, 4.0, 1.0], [4.0, 6.0, 6.0]]], [[[3.0, 7.0, 6.0], [1.0, 7.0, 2.0], [4.0, 8.0, 6.0]], [[1.0, 
4.0, 5.0], [1.0, 6.0, 5.0], [1.0, 5.0, 8.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 4.0], [3.0, 5.0, 3.0]]], [[[7.0, 1.0, 5.0], [2.0, 
3.0, 1.0], [8.0, 4.0, 8.0]], [[5.0, 7.0, 3.0], [1.0, 2.0, 5.0], [4.0, 8.0, 4.0]], [[5.0, 6.0, 7.0], [7.0, 1.0, 1.0], [8.0, 7.0, 
5.0]]], [[[8.0, 7.0, 7.0], [6.0, 3.0, 2.0], [1.0, 7.0, 2.0]], [[2.0, 5.0, 3.0], [7.0, 4.0, 7.0], [4.0, 1.0, 4.0]], [[3.0, 3.0, 
7.0], [6.0, 4.0, 5.0], [4.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 2.0], [1.0, 2.0, 4.0], [4.0, 2.0, 5.0]], 
[[6.0, 6.0, 6.0], [2.0, 3.0, 6.0], [3.0, 4.0, 3.0]], [[4.0, 3.0, 1.0], [4.0, 8.0, 6.0], [8.0, 7.0, 6.0]]], [[[5.0, 8.0, 1.0], 
[7.0, 2.0, 8.0], [2.0, 1.0, 1.0]], [[1.0, 8.0, 3.0], [5.0, 6.0, 8.0], [8.0, 4.0, 7.0]], [[1.0, 7.0, 7.0], [3.0, 8.0, 2.0], 
[8.0, 7.0, 6.0]]], [[[5.0, 5.0, 2.0], [1.0, 1.0, 6.0], [8.0, 1.0, 7.0]], [[1.0, 2.0, 7.0], [4.0, 7.0, 1.0], [8.0, 8.0, 6.0]], 
[[7.0, 6.0, 1.0], [8.0, 3.0, 2.0], [1.0, 5.0, 6.0]]], [[[1.0, 4.0, 2.0], [5.0, 4.0, 4.0], [8.0, 7.0, 2.0]], [[5.0, 5.0, 5.0], 
[2.0, 4.0, 6.0], [4.0, 5.0, 1.0]], [[4.0, 6.0, 7.0], [6.0, 8.0, 6.0], [2.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_URRN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 2.0], [5.0, 5.0, 2.0], [5.0, 8.0, 4.0]], [[4.0, 1.0, 7.0], [3.0, 3.0, 3.0], [3.0, 3.0, 
3.0]], [[2.0, 5.0, 3.0], [3.0, 5.0, 1.0], [2.0, 5.0, 7.0]]], [[[6.0, 8.0, 3.0], [5.0, 1.0, 1.0], [1.0, 2.0, 7.0]], [[7.0, 5.0, 
1.0], [6.0, 5.0, 3.0], [7.0, 6.0, 6.0]], [[1.0, 7.0, 7.0], [3.0, 2.0, 3.0], [2.0, 6.0, 2.0]]], [[[6.0, 4.0, 6.0], [4.0, 8.0, 
2.0], [2.0, 7.0, 1.0]], [[8.0, 5.0, 2.0], [2.0, 3.0, 3.0], [4.0, 1.0, 4.0]], [[3.0, 8.0, 2.0], [8.0, 4.0, 8.0], [8.0, 3.0, 
5.0]]], [[[8.0, 1.0, 2.0], [5.0, 1.0, 7.0], [5.0, 7.0, 1.0]], [[3.0, 5.0, 7.0], [2.0, 6.0, 6.0], [7.0, 8.0, 2.0]], [[8.0, 7.0, 
6.0], [5.0, 6.0, 3.0], [1.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 1.0], [6.0, 5.0, 2.0], [4.0, 5.0, 6.0]], [[2.0, 5.0, 2.0], [1.0, 7.0, 
8.0], [5.0, 8.0, 8.0]], [[2.0, 5.0, 8.0], [1.0, 4.0, 2.0], [4.0, 4.0, 1.0]]], [[[3.0, 1.0, 3.0], [7.0, 5.0, 8.0], [5.0, 6.0, 
3.0]], [[3.0, 2.0, 8.0], [4.0, 6.0, 1.0], [7.0, 3.0, 6.0]], [[6.0, 5.0, 8.0], [2.0, 5.0, 7.0], [2.0, 8.0, 6.0]]], [[[6.0, 4.0, 
5.0], [8.0, 8.0, 1.0], [8.0, 2.0, 5.0]], [[8.0, 7.0, 2.0], [2.0, 1.0, 2.0], [1.0, 4.0, 2.0]], [[8.0, 7.0, 5.0], [3.0, 1.0, 
8.0], [2.0, 5.0, 3.0]]], [[[2.0, 1.0, 4.0], [5.0, 6.0, 5.0], [6.0, 8.0, 7.0]], [[5.0, 3.0, 7.0], [5.0, 7.0, 5.0], [5.0, 7.0, 
3.0]], [[5.0, 5.0, 5.0], [8.0, 4.0, 7.0], [6.0, 3.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 6.0, 2.0]]], [[[5.0, 2.0, 3.0]]], [[[8.0, 1.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 6.0, 4.0]]], [[[5.0, 7.0, 2.0]]], [[[5.0, 5.0, 7.0]]]])
      arg[:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 2.0], [5.0, 5.0, 2.0], [5.0, 8.0, 4.0]], [[4.0, 1.0, 7.0], [1.0, 6.0, 2.0], [3.0, 
3.0, 3.0]], [[2.0, 5.0, 3.0], [3.0, 5.0, 1.0], [2.0, 5.0, 7.0]]], [[[6.0, 8.0, 3.0], [5.0, 1.0, 1.0], [1.0, 2.0, 7.0]], [[7.0, 
5.0, 1.0], [5.0, 2.0, 3.0], [7.0, 6.0, 6.0]], [[1.0, 7.0, 7.0], [3.0, 2.0, 3.0], [2.0, 6.0, 2.0]]], [[[6.0, 4.0, 6.0], [4.0, 
8.0, 2.0], [2.0, 7.0, 1.0]], [[8.0, 5.0, 2.0], [8.0, 1.0, 1.0], [4.0, 1.0, 4.0]], [[3.0, 8.0, 2.0], [8.0, 4.0, 8.0], [8.0, 3.0, 
5.0]]], [[[8.0, 1.0, 2.0], [5.0, 1.0, 7.0], [5.0, 7.0, 1.0]], [[3.0, 5.0, 7.0], [2.0, 6.0, 6.0], [7.0, 8.0, 2.0]], [[8.0, 7.0, 
6.0], [5.0, 6.0, 3.0], [1.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 1.0], [6.0, 5.0, 2.0], [4.0, 5.0, 6.0]], 
[[2.0, 5.0, 2.0], [4.0, 6.0, 4.0], [5.0, 8.0, 8.0]], [[2.0, 5.0, 8.0], [1.0, 4.0, 2.0], [4.0, 4.0, 1.0]]], [[[3.0, 1.0, 3.0], 
[7.0, 5.0, 8.0], [5.0, 6.0, 3.0]], [[3.0, 2.0, 8.0], [5.0, 7.0, 2.0], [7.0, 3.0, 6.0]], [[6.0, 5.0, 8.0], [2.0, 5.0, 7.0], 
[2.0, 8.0, 6.0]]], [[[6.0, 4.0, 5.0], [8.0, 8.0, 1.0], [8.0, 2.0, 5.0]], [[8.0, 7.0, 2.0], [5.0, 5.0, 7.0], [1.0, 4.0, 2.0]], 
[[8.0, 7.0, 5.0], [3.0, 1.0, 8.0], [2.0, 5.0, 3.0]]], [[[2.0, 1.0, 4.0], [5.0, 6.0, 5.0], [6.0, 8.0, 7.0]], [[5.0, 3.0, 7.0], 
[5.0, 7.0, 5.0], [5.0, 7.0, 3.0]], [[5.0, 5.0, 5.0], [8.0, 4.0, 7.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UCLN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 6.0], [3.0, 6.0, 5.0], [2.0, 8.0, 5.0]], [[2.0, 2.0, 4.0], [2.0, 3.0, 7.0], [5.0, 5.0, 
2.0]], [[8.0, 1.0, 2.0], [4.0, 7.0, 8.0], [5.0, 3.0, 8.0]]], [[[5.0, 3.0, 1.0], [2.0, 8.0, 1.0], [8.0, 3.0, 1.0]], [[3.0, 3.0, 
6.0], [1.0, 3.0, 1.0], [4.0, 6.0, 7.0]], [[2.0, 2.0, 6.0], [6.0, 5.0, 6.0], [3.0, 6.0, 8.0]]], [[[2.0, 2.0, 3.0], [1.0, 6.0, 
1.0], [2.0, 4.0, 8.0]], [[4.0, 8.0, 3.0], [1.0, 1.0, 3.0], [6.0, 3.0, 4.0]], [[8.0, 7.0, 6.0], [7.0, 2.0, 2.0], [7.0, 1.0, 
2.0]]], [[[2.0, 8.0, 7.0], [6.0, 7.0, 3.0], [3.0, 2.0, 3.0]], [[1.0, 5.0, 6.0], [6.0, 6.0, 3.0], [8.0, 5.0, 5.0]], [[6.0, 7.0, 
4.0], [5.0, 4.0, 7.0], [1.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 2.0], [6.0, 1.0, 4.0], [8.0, 2.0, 6.0]], [[1.0, 7.0, 1.0], [5.0, 8.0, 
5.0], [7.0, 7.0, 4.0]], [[2.0, 5.0, 2.0], [3.0, 1.0, 8.0], [8.0, 8.0, 4.0]]], [[[1.0, 3.0, 3.0], [3.0, 4.0, 5.0], [5.0, 8.0, 
2.0]], [[5.0, 3.0, 3.0], [5.0, 7.0, 2.0], [1.0, 6.0, 5.0]], [[1.0, 3.0, 8.0], [2.0, 8.0, 2.0], [3.0, 3.0, 2.0]]], [[[8.0, 6.0, 
2.0], [7.0, 7.0, 8.0], [8.0, 7.0, 5.0]], [[8.0, 5.0, 3.0], [5.0, 7.0, 6.0], [6.0, 2.0, 1.0]], [[6.0, 5.0, 7.0], [6.0, 2.0, 
6.0], [6.0, 1.0, 7.0]]], [[[4.0, 5.0, 3.0], [4.0, 3.0, 3.0], [8.0, 3.0, 8.0]], [[5.0, 1.0, 8.0], [7.0, 3.0, 6.0], [4.0, 7.0, 
3.0]], [[4.0, 3.0, 4.0], [3.0, 5.0, 6.0], [7.0, 6.0, 1.0]]]]))
      val=numpy.array([[[[1.0, 5.0, 5.0]], [[3.0, 7.0, 4.0]], [[3.0, 1.0, 4.0]]], [[[1.0, 7.0, 4.0]], [[1.0, 7.0, 7.0]], [[4.0, 
8.0, 2.0]]], [[[8.0, 6.0, 7.0]], [[1.0, 1.0, 1.0]], [[1.0, 3.0, 6.0]]]])
      arg[:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 6.0], [3.0, 6.0, 5.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 4.0], [2.0, 3.0, 7.0], [3.0, 
7.0, 4.0]], [[8.0, 1.0, 2.0], [4.0, 7.0, 8.0], [3.0, 1.0, 4.0]]], [[[5.0, 3.0, 1.0], [2.0, 8.0, 1.0], [1.0, 7.0, 4.0]], [[3.0, 
3.0, 6.0], [1.0, 3.0, 1.0], [1.0, 7.0, 7.0]], [[2.0, 2.0, 6.0], [6.0, 5.0, 6.0], [4.0, 8.0, 2.0]]], [[[2.0, 2.0, 3.0], [1.0, 
6.0, 1.0], [8.0, 6.0, 7.0]], [[4.0, 8.0, 3.0], [1.0, 1.0, 3.0], [1.0, 1.0, 1.0]], [[8.0, 7.0, 6.0], [7.0, 2.0, 2.0], [1.0, 3.0, 
6.0]]], [[[2.0, 8.0, 7.0], [6.0, 7.0, 3.0], [3.0, 2.0, 3.0]], [[1.0, 5.0, 6.0], [6.0, 6.0, 3.0], [8.0, 5.0, 5.0]], [[6.0, 7.0, 
4.0], [5.0, 4.0, 7.0], [1.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 2.0], [6.0, 1.0, 4.0], [1.0, 5.0, 5.0]], 
[[1.0, 7.0, 1.0], [5.0, 8.0, 5.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 2.0], [3.0, 1.0, 8.0], [3.0, 1.0, 4.0]]], [[[1.0, 3.0, 3.0], 
[3.0, 4.0, 5.0], [1.0, 7.0, 4.0]], [[5.0, 3.0, 3.0], [5.0, 7.0, 2.0], [1.0, 7.0, 7.0]], [[1.0, 3.0, 8.0], [2.0, 8.0, 2.0], 
[4.0, 8.0, 2.0]]], [[[8.0, 6.0, 2.0], [7.0, 7.0, 8.0], [8.0, 6.0, 7.0]], [[8.0, 5.0, 3.0], [5.0, 7.0, 6.0], [1.0, 1.0, 1.0]], 
[[6.0, 5.0, 7.0], [6.0, 2.0, 6.0], [1.0, 3.0, 6.0]]], [[[4.0, 5.0, 3.0], [4.0, 3.0, 3.0], [8.0, 3.0, 8.0]], [[5.0, 1.0, 8.0], 
[7.0, 3.0, 6.0], [4.0, 7.0, 3.0]], [[4.0, 3.0, 4.0], [3.0, 5.0, 6.0], [7.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UCLN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 4.0], [7.0, 3.0, 7.0], [7.0, 3.0, 1.0]], [[7.0, 6.0, 6.0], [7.0, 7.0, 4.0], [7.0, 1.0, 
1.0]], [[6.0, 6.0, 3.0], [5.0, 5.0, 1.0], [8.0, 8.0, 8.0]]], [[[3.0, 5.0, 6.0], [1.0, 2.0, 1.0], [3.0, 4.0, 8.0]], [[8.0, 2.0, 
4.0], [3.0, 8.0, 8.0], [8.0, 4.0, 2.0]], [[6.0, 4.0, 3.0], [2.0, 3.0, 2.0], [2.0, 3.0, 2.0]]], [[[1.0, 5.0, 7.0], [4.0, 3.0, 
3.0], [8.0, 2.0, 5.0]], [[7.0, 3.0, 3.0], [3.0, 7.0, 4.0], [3.0, 2.0, 7.0]], [[1.0, 8.0, 6.0], [7.0, 2.0, 1.0], [5.0, 7.0, 
6.0]]], [[[7.0, 6.0, 5.0], [5.0, 4.0, 7.0], [2.0, 5.0, 3.0]], [[5.0, 5.0, 6.0], [5.0, 3.0, 4.0], [6.0, 2.0, 6.0]], [[4.0, 4.0, 
8.0], [4.0, 6.0, 2.0], [1.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 1.0], [6.0, 4.0, 6.0], [3.0, 8.0, 5.0]], [[1.0, 1.0, 3.0], [5.0, 2.0, 
1.0], [5.0, 5.0, 3.0]], [[8.0, 8.0, 2.0], [4.0, 1.0, 6.0], [7.0, 7.0, 7.0]]], [[[1.0, 4.0, 7.0], [6.0, 3.0, 7.0], [4.0, 8.0, 
6.0]], [[3.0, 1.0, 6.0], [4.0, 5.0, 5.0], [6.0, 7.0, 7.0]], [[8.0, 4.0, 6.0], [2.0, 8.0, 7.0], [2.0, 7.0, 2.0]]], [[[7.0, 4.0, 
5.0], [4.0, 2.0, 6.0], [5.0, 3.0, 7.0]], [[2.0, 2.0, 4.0], [7.0, 3.0, 4.0], [5.0, 1.0, 7.0]], [[3.0, 2.0, 7.0], [3.0, 6.0, 
5.0], [8.0, 3.0, 6.0]]], [[[5.0, 3.0, 3.0], [5.0, 2.0, 4.0], [1.0, 1.0, 8.0]], [[3.0, 1.0, 6.0], [8.0, 7.0, 5.0], [1.0, 3.0, 
8.0]], [[5.0, 2.0, 2.0], [6.0, 5.0, 5.0], [7.0, 5.0, 3.0]]]]))
      val=Data(numpy.array([[[[1.0, 2.0, 7.0]], [[3.0, 3.0, 2.0]], [[2.0, 2.0, 8.0]]], [[[8.0, 6.0, 7.0]], [[6.0, 5.0, 4.0]], 
[[5.0, 5.0, 2.0]]], [[[7.0, 1.0, 5.0]], [[3.0, 5.0, 5.0]], [[5.0, 2.0, 8.0]]]]),self.functionspace)
      arg[:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 4.0], [7.0, 3.0, 7.0], [1.0, 2.0, 7.0]], [[7.0, 6.0, 6.0], [7.0, 7.0, 4.0], [3.0, 
3.0, 2.0]], [[6.0, 6.0, 3.0], [5.0, 5.0, 1.0], [2.0, 2.0, 8.0]]], [[[3.0, 5.0, 6.0], [1.0, 2.0, 1.0], [8.0, 6.0, 7.0]], [[8.0, 
2.0, 4.0], [3.0, 8.0, 8.0], [6.0, 5.0, 4.0]], [[6.0, 4.0, 3.0], [2.0, 3.0, 2.0], [5.0, 5.0, 2.0]]], [[[1.0, 5.0, 7.0], [4.0, 
3.0, 3.0], [7.0, 1.0, 5.0]], [[7.0, 3.0, 3.0], [3.0, 7.0, 4.0], [3.0, 5.0, 5.0]], [[1.0, 8.0, 6.0], [7.0, 2.0, 1.0], [5.0, 2.0, 
8.0]]], [[[7.0, 6.0, 5.0], [5.0, 4.0, 7.0], [2.0, 5.0, 3.0]], [[5.0, 5.0, 6.0], [5.0, 3.0, 4.0], [6.0, 2.0, 6.0]], [[4.0, 4.0, 
8.0], [4.0, 6.0, 2.0], [1.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 1.0], [6.0, 4.0, 6.0], [1.0, 2.0, 7.0]], 
[[1.0, 1.0, 3.0], [5.0, 2.0, 1.0], [3.0, 3.0, 2.0]], [[8.0, 8.0, 2.0], [4.0, 1.0, 6.0], [2.0, 2.0, 8.0]]], [[[1.0, 4.0, 7.0], 
[6.0, 3.0, 7.0], [8.0, 6.0, 7.0]], [[3.0, 1.0, 6.0], [4.0, 5.0, 5.0], [6.0, 5.0, 4.0]], [[8.0, 4.0, 6.0], [2.0, 8.0, 7.0], 
[5.0, 5.0, 2.0]]], [[[7.0, 4.0, 5.0], [4.0, 2.0, 6.0], [7.0, 1.0, 5.0]], [[2.0, 2.0, 4.0], [7.0, 3.0, 4.0], [3.0, 5.0, 5.0]], 
[[3.0, 2.0, 7.0], [3.0, 6.0, 5.0], [5.0, 2.0, 8.0]]], [[[5.0, 3.0, 3.0], [5.0, 2.0, 4.0], [1.0, 1.0, 8.0]], [[3.0, 1.0, 6.0], 
[8.0, 7.0, 5.0], [1.0, 3.0, 8.0]], [[5.0, 2.0, 2.0], [6.0, 5.0, 5.0], [7.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UCLN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 8.0], [5.0, 4.0, 7.0], [5.0, 5.0, 4.0]], [[4.0, 1.0, 7.0], [1.0, 1.0, 8.0], [1.0, 2.0, 
1.0]], [[6.0, 5.0, 5.0], [4.0, 5.0, 8.0], [5.0, 8.0, 5.0]]], [[[8.0, 6.0, 7.0], [6.0, 6.0, 6.0], [6.0, 2.0, 8.0]], [[5.0, 7.0, 
8.0], [6.0, 8.0, 3.0], [7.0, 5.0, 1.0]], [[3.0, 1.0, 2.0], [2.0, 7.0, 7.0], [2.0, 6.0, 5.0]]], [[[6.0, 3.0, 5.0], [3.0, 3.0, 
3.0], [8.0, 6.0, 1.0]], [[6.0, 8.0, 2.0], [7.0, 4.0, 7.0], [1.0, 8.0, 2.0]], [[7.0, 8.0, 7.0], [5.0, 3.0, 3.0], [1.0, 2.0, 
4.0]]], [[[3.0, 1.0, 2.0], [8.0, 1.0, 4.0], [2.0, 2.0, 7.0]], [[8.0, 2.0, 2.0], [4.0, 6.0, 1.0], [8.0, 7.0, 1.0]], [[3.0, 1.0, 
6.0], [6.0, 5.0, 4.0], [1.0, 1.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 6.0], [2.0, 4.0, 3.0], [3.0, 8.0, 2.0]], [[1.0, 6.0, 4.0], [8.0, 4.0, 
6.0], [6.0, 1.0, 8.0]], [[6.0, 5.0, 5.0], [3.0, 4.0, 5.0], [7.0, 5.0, 8.0]]], [[[5.0, 4.0, 8.0], [6.0, 2.0, 1.0], [7.0, 4.0, 
4.0]], [[4.0, 6.0, 1.0], [2.0, 2.0, 4.0], [2.0, 7.0, 1.0]], [[1.0, 8.0, 4.0], [6.0, 7.0, 6.0], [8.0, 2.0, 7.0]]], [[[7.0, 7.0, 
5.0], [5.0, 8.0, 7.0], [7.0, 5.0, 4.0]], [[4.0, 7.0, 8.0], [8.0, 7.0, 4.0], [3.0, 3.0, 3.0]], [[6.0, 4.0, 1.0], [6.0, 7.0, 
7.0], [4.0, 1.0, 7.0]]], [[[3.0, 7.0, 5.0], [6.0, 6.0, 5.0], [5.0, 3.0, 6.0]], [[2.0, 8.0, 8.0], [8.0, 7.0, 7.0], [3.0, 1.0, 
6.0]], [[3.0, 5.0, 6.0], [7.0, 8.0, 4.0], [5.0, 3.0, 1.0]]]]))
      val=Data(numpy.array([[[[4.0, 5.0, 6.0]], [[8.0, 6.0, 4.0]], [[2.0, 8.0, 7.0]]], [[[1.0, 8.0, 8.0]], [[8.0, 3.0, 8.0]], 
[[1.0, 8.0, 4.0]]], [[[8.0, 4.0, 7.0]], [[2.0, 6.0, 4.0]], [[5.0, 1.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 3.0]], [[1.0, 7.0, 3.0]], [[4.0, 2.0, 6.0]]], [[[7.0, 7.0, 5.0]], [[4.0, 
1.0, 6.0]], [[1.0, 2.0, 1.0]]], [[[2.0, 4.0, 5.0]], [[4.0, 2.0, 4.0]], [[5.0, 3.0, 8.0]]]]))
      arg[:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 8.0], [5.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[4.0, 1.0, 7.0], [1.0, 1.0, 8.0], [8.0, 
6.0, 4.0]], [[6.0, 5.0, 5.0], [4.0, 5.0, 8.0], [2.0, 8.0, 7.0]]], [[[8.0, 6.0, 7.0], [6.0, 6.0, 6.0], [1.0, 8.0, 8.0]], [[5.0, 
7.0, 8.0], [6.0, 8.0, 3.0], [8.0, 3.0, 8.0]], [[3.0, 1.0, 2.0], [2.0, 7.0, 7.0], [1.0, 8.0, 4.0]]], [[[6.0, 3.0, 5.0], [3.0, 
3.0, 3.0], [8.0, 4.0, 7.0]], [[6.0, 8.0, 2.0], [7.0, 4.0, 7.0], [2.0, 6.0, 4.0]], [[7.0, 8.0, 7.0], [5.0, 3.0, 3.0], [5.0, 1.0, 
6.0]]], [[[3.0, 1.0, 2.0], [8.0, 1.0, 4.0], [2.0, 2.0, 7.0]], [[8.0, 2.0, 2.0], [4.0, 6.0, 1.0], [8.0, 7.0, 1.0]], [[3.0, 1.0, 
6.0], [6.0, 5.0, 4.0], [1.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 6.0], [2.0, 4.0, 3.0], [6.0, 2.0, 3.0]], 
[[1.0, 6.0, 4.0], [8.0, 4.0, 6.0], [1.0, 7.0, 3.0]], [[6.0, 5.0, 5.0], [3.0, 4.0, 5.0], [4.0, 2.0, 6.0]]], [[[5.0, 4.0, 8.0], 
[6.0, 2.0, 1.0], [7.0, 7.0, 5.0]], [[4.0, 6.0, 1.0], [2.0, 2.0, 4.0], [4.0, 1.0, 6.0]], [[1.0, 8.0, 4.0], [6.0, 7.0, 6.0], 
[1.0, 2.0, 1.0]]], [[[7.0, 7.0, 5.0], [5.0, 8.0, 7.0], [2.0, 4.0, 5.0]], [[4.0, 7.0, 8.0], [8.0, 7.0, 4.0], [4.0, 2.0, 4.0]], 
[[6.0, 4.0, 1.0], [6.0, 7.0, 7.0], [5.0, 3.0, 8.0]]], [[[3.0, 7.0, 5.0], [6.0, 6.0, 5.0], [5.0, 3.0, 6.0]], [[2.0, 8.0, 8.0], 
[8.0, 7.0, 7.0], [3.0, 1.0, 6.0]], [[3.0, 5.0, 6.0], [7.0, 8.0, 4.0], [5.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UCLN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 4.0], [6.0, 2.0, 8.0], [7.0, 3.0, 7.0]], [[8.0, 6.0, 5.0], [6.0, 4.0, 8.0], [6.0, 4.0, 
8.0]], [[3.0, 3.0, 7.0], [3.0, 8.0, 4.0], [2.0, 7.0, 1.0]]], [[[4.0, 3.0, 2.0], [7.0, 2.0, 8.0], [8.0, 4.0, 7.0]], [[4.0, 7.0, 
5.0], [8.0, 6.0, 5.0], [2.0, 7.0, 4.0]], [[1.0, 4.0, 8.0], [4.0, 4.0, 2.0], [3.0, 8.0, 3.0]]], [[[8.0, 8.0, 5.0], [8.0, 8.0, 
2.0], [3.0, 5.0, 5.0]], [[7.0, 1.0, 4.0], [7.0, 2.0, 7.0], [2.0, 4.0, 8.0]], [[2.0, 7.0, 1.0], [3.0, 8.0, 5.0], [5.0, 8.0, 
8.0]]], [[[4.0, 4.0, 7.0], [4.0, 7.0, 4.0], [3.0, 7.0, 7.0]], [[1.0, 8.0, 8.0], [3.0, 3.0, 8.0], [5.0, 3.0, 3.0]], [[2.0, 3.0, 
8.0], [3.0, 4.0, 7.0], [8.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 1.0], [1.0, 6.0, 1.0], [5.0, 1.0, 1.0]], [[3.0, 7.0, 4.0], [2.0, 4.0, 
1.0], [8.0, 2.0, 6.0]], [[2.0, 6.0, 2.0], [6.0, 3.0, 4.0], [5.0, 7.0, 6.0]]], [[[2.0, 1.0, 3.0], [4.0, 1.0, 1.0], [5.0, 7.0, 
4.0]], [[6.0, 8.0, 6.0], [2.0, 8.0, 3.0], [2.0, 8.0, 6.0]], [[5.0, 4.0, 8.0], [4.0, 2.0, 7.0], [1.0, 4.0, 7.0]]], [[[4.0, 8.0, 
2.0], [3.0, 3.0, 7.0], [5.0, 1.0, 1.0]], [[8.0, 7.0, 5.0], [4.0, 1.0, 5.0], [1.0, 1.0, 7.0]], [[2.0, 8.0, 4.0], [8.0, 7.0, 
4.0], [4.0, 6.0, 7.0]]], [[[3.0, 5.0, 3.0], [6.0, 6.0, 3.0], [1.0, 6.0, 2.0]], [[4.0, 8.0, 3.0], [8.0, 3.0, 5.0], [2.0, 6.0, 
5.0]], [[3.0, 7.0, 8.0], [6.0, 8.0, 2.0], [6.0, 8.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0, 5.0, 2.0]], [[3.0, 3.0, 1.0]], [[7.0, 2.0, 4.0]]], [[[8.0, 5.0, 6.0]], [[1.0, 3.0, 
3.0]], [[3.0, 2.0, 4.0]]], [[[6.0, 5.0, 8.0]], [[5.0, 1.0, 7.0]], [[8.0, 6.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 2.0, 
8.0]], [[7.0, 4.0, 2.0]], [[1.0, 8.0, 6.0]]], [[[8.0, 4.0, 4.0]], [[2.0, 3.0, 8.0]], [[6.0, 1.0, 4.0]]], [[[5.0, 1.0, 6.0]], 
[[4.0, 7.0, 5.0]], [[6.0, 7.0, 5.0]]]])
      arg[:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 4.0], [6.0, 2.0, 8.0], [7.0, 5.0, 2.0]], [[8.0, 6.0, 5.0], [6.0, 4.0, 8.0], [3.0, 
3.0, 1.0]], [[3.0, 3.0, 7.0], [3.0, 8.0, 4.0], [7.0, 2.0, 4.0]]], [[[4.0, 3.0, 2.0], [7.0, 2.0, 8.0], [8.0, 5.0, 6.0]], [[4.0, 
7.0, 5.0], [8.0, 6.0, 5.0], [1.0, 3.0, 3.0]], [[1.0, 4.0, 8.0], [4.0, 4.0, 2.0], [3.0, 2.0, 4.0]]], [[[8.0, 8.0, 5.0], [8.0, 
8.0, 2.0], [6.0, 5.0, 8.0]], [[7.0, 1.0, 4.0], [7.0, 2.0, 7.0], [5.0, 1.0, 7.0]], [[2.0, 7.0, 1.0], [3.0, 8.0, 5.0], [8.0, 6.0, 
6.0]]], [[[4.0, 4.0, 7.0], [4.0, 7.0, 4.0], [3.0, 7.0, 7.0]], [[1.0, 8.0, 8.0], [3.0, 3.0, 8.0], [5.0, 3.0, 3.0]], [[2.0, 3.0, 
8.0], [3.0, 4.0, 7.0], [8.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 1.0], [1.0, 6.0, 1.0], [7.0, 2.0, 8.0]], 
[[3.0, 7.0, 4.0], [2.0, 4.0, 1.0], [7.0, 4.0, 2.0]], [[2.0, 6.0, 2.0], [6.0, 3.0, 4.0], [1.0, 8.0, 6.0]]], [[[2.0, 1.0, 3.0], 
[4.0, 1.0, 1.0], [8.0, 4.0, 4.0]], [[6.0, 8.0, 6.0], [2.0, 8.0, 3.0], [2.0, 3.0, 8.0]], [[5.0, 4.0, 8.0], [4.0, 2.0, 7.0], 
[6.0, 1.0, 4.0]]], [[[4.0, 8.0, 2.0], [3.0, 3.0, 7.0], [5.0, 1.0, 6.0]], [[8.0, 7.0, 5.0], [4.0, 1.0, 5.0], [4.0, 7.0, 5.0]], 
[[2.0, 8.0, 4.0], [8.0, 7.0, 4.0], [6.0, 7.0, 5.0]]], [[[3.0, 5.0, 3.0], [6.0, 6.0, 3.0], [1.0, 6.0, 2.0]], [[4.0, 8.0, 3.0], 
[8.0, 3.0, 5.0], [2.0, 6.0, 5.0]], [[3.0, 7.0, 8.0], [6.0, 8.0, 2.0], [6.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCRN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 2.0], [5.0, 3.0, 7.0], [1.0, 3.0, 2.0]], [[1.0, 7.0, 4.0], [2.0, 5.0, 5.0], [5.0, 8.0, 
2.0]], [[1.0, 6.0, 8.0], [6.0, 2.0, 7.0], [6.0, 2.0, 4.0]]], [[[3.0, 3.0, 1.0], [3.0, 7.0, 5.0], [5.0, 6.0, 3.0]], [[1.0, 3.0, 
8.0], [7.0, 7.0, 6.0], [2.0, 5.0, 1.0]], [[1.0, 5.0, 1.0], [3.0, 6.0, 3.0], [8.0, 7.0, 8.0]]], [[[6.0, 4.0, 7.0], [8.0, 2.0, 
4.0], [8.0, 3.0, 4.0]], [[3.0, 4.0, 2.0], [3.0, 5.0, 2.0], [8.0, 6.0, 5.0]], [[2.0, 5.0, 3.0], [4.0, 4.0, 1.0], [4.0, 6.0, 
6.0]]], [[[4.0, 1.0, 5.0], [7.0, 8.0, 3.0], [4.0, 4.0, 5.0]], [[4.0, 5.0, 8.0], [4.0, 7.0, 1.0], [8.0, 2.0, 3.0]], [[7.0, 1.0, 
7.0], [7.0, 3.0, 6.0], [7.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 4.0], [7.0, 3.0, 2.0], [5.0, 4.0, 1.0]], [[8.0, 3.0, 7.0], [3.0, 2.0, 
1.0], [8.0, 2.0, 4.0]], [[4.0, 5.0, 5.0], [8.0, 3.0, 7.0], [1.0, 4.0, 3.0]]], [[[1.0, 8.0, 7.0], [3.0, 7.0, 3.0], [7.0, 1.0, 
8.0]], [[2.0, 4.0, 7.0], [8.0, 2.0, 7.0], [3.0, 1.0, 7.0]], [[5.0, 3.0, 6.0], [2.0, 4.0, 3.0], [7.0, 7.0, 8.0]]], [[[5.0, 7.0, 
6.0], [8.0, 7.0, 6.0], [7.0, 2.0, 6.0]], [[5.0, 7.0, 3.0], [6.0, 3.0, 5.0], [2.0, 7.0, 6.0]], [[6.0, 2.0, 5.0], [2.0, 8.0, 
4.0], [8.0, 4.0, 2.0]]], [[[7.0, 3.0, 4.0], [2.0, 8.0, 5.0], [8.0, 7.0, 6.0]], [[4.0, 3.0, 8.0], [4.0, 6.0, 5.0], [3.0, 4.0, 
2.0]], [[3.0, 8.0, 4.0], [2.0, 5.0, 3.0], [3.0, 8.0, 4.0]]]]))
      val=numpy.array([[[[4.0, 3.0, 6.0]], [[3.0, 3.0, 1.0]], [[8.0, 8.0, 6.0]]], [[[6.0, 3.0, 6.0]], [[1.0, 8.0, 6.0]], [[2.0, 
7.0, 2.0]]], [[[8.0, 1.0, 6.0]], [[5.0, 6.0, 7.0]], [[4.0, 7.0, 8.0]]], [[[5.0, 5.0, 1.0]], [[6.0, 1.0, 4.0]], [[5.0, 6.0, 
5.0]]]])
      arg[:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 2.0], [4.0, 3.0, 6.0], [1.0, 3.0, 2.0]], [[1.0, 7.0, 4.0], [3.0, 3.0, 1.0], [5.0, 
8.0, 2.0]], [[1.0, 6.0, 8.0], [8.0, 8.0, 6.0], [6.0, 2.0, 4.0]]], [[[3.0, 3.0, 1.0], [6.0, 3.0, 6.0], [5.0, 6.0, 3.0]], [[1.0, 
3.0, 8.0], [1.0, 8.0, 6.0], [2.0, 5.0, 1.0]], [[1.0, 5.0, 1.0], [2.0, 7.0, 2.0], [8.0, 7.0, 8.0]]], [[[6.0, 4.0, 7.0], [8.0, 
1.0, 6.0], [8.0, 3.0, 4.0]], [[3.0, 4.0, 2.0], [5.0, 6.0, 7.0], [8.0, 6.0, 5.0]], [[2.0, 5.0, 3.0], [4.0, 7.0, 8.0], [4.0, 6.0, 
6.0]]], [[[4.0, 1.0, 5.0], [5.0, 5.0, 1.0], [4.0, 4.0, 5.0]], [[4.0, 5.0, 8.0], [6.0, 1.0, 4.0], [8.0, 2.0, 3.0]], [[7.0, 1.0, 
7.0], [5.0, 6.0, 5.0], [7.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 4.0], [4.0, 3.0, 6.0], [5.0, 4.0, 1.0]], 
[[8.0, 3.0, 7.0], [3.0, 3.0, 1.0], [8.0, 2.0, 4.0]], [[4.0, 5.0, 5.0], [8.0, 8.0, 6.0], [1.0, 4.0, 3.0]]], [[[1.0, 8.0, 7.0], 
[6.0, 3.0, 6.0], [7.0, 1.0, 8.0]], [[2.0, 4.0, 7.0], [1.0, 8.0, 6.0], [3.0, 1.0, 7.0]], [[5.0, 3.0, 6.0], [2.0, 7.0, 2.0], 
[7.0, 7.0, 8.0]]], [[[5.0, 7.0, 6.0], [8.0, 1.0, 6.0], [7.0, 2.0, 6.0]], [[5.0, 7.0, 3.0], [5.0, 6.0, 7.0], [2.0, 7.0, 6.0]], 
[[6.0, 2.0, 5.0], [4.0, 7.0, 8.0], [8.0, 4.0, 2.0]]], [[[7.0, 3.0, 4.0], [5.0, 5.0, 1.0], [8.0, 7.0, 6.0]], [[4.0, 3.0, 8.0], 
[6.0, 1.0, 4.0], [3.0, 4.0, 2.0]], [[3.0, 8.0, 4.0], [5.0, 6.0, 5.0], [3.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCRN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 2.0], [4.0, 5.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 5.0, 5.0], [5.0, 5.0, 8.0], [1.0, 7.0, 
2.0]], [[5.0, 4.0, 7.0], [7.0, 8.0, 7.0], [7.0, 6.0, 1.0]]], [[[2.0, 3.0, 7.0], [7.0, 7.0, 5.0], [8.0, 2.0, 6.0]], [[5.0, 5.0, 
7.0], [5.0, 8.0, 8.0], [2.0, 4.0, 6.0]], [[5.0, 7.0, 1.0], [7.0, 4.0, 4.0], [8.0, 5.0, 4.0]]], [[[4.0, 6.0, 2.0], [8.0, 7.0, 
7.0], [7.0, 7.0, 3.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 2.0], [2.0, 2.0, 3.0]], [[8.0, 5.0, 8.0], [7.0, 3.0, 4.0], [2.0, 5.0, 
2.0]]], [[[2.0, 8.0, 4.0], [1.0, 4.0, 2.0], [2.0, 4.0, 3.0]], [[3.0, 4.0, 3.0], [2.0, 8.0, 7.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 
3.0], [8.0, 4.0, 5.0], [8.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 5.0], [2.0, 1.0, 5.0], [3.0, 7.0, 8.0]], [[7.0, 1.0, 6.0], [1.0, 3.0, 
3.0], [5.0, 3.0, 8.0]], [[8.0, 8.0, 2.0], [5.0, 4.0, 7.0], [1.0, 7.0, 4.0]]], [[[7.0, 4.0, 1.0], [8.0, 4.0, 8.0], [5.0, 4.0, 
1.0]], [[1.0, 4.0, 7.0], [7.0, 1.0, 8.0], [1.0, 6.0, 7.0]], [[4.0, 7.0, 6.0], [4.0, 3.0, 6.0], [5.0, 8.0, 8.0]]], [[[6.0, 7.0, 
7.0], [6.0, 8.0, 7.0], [6.0, 4.0, 7.0]], [[7.0, 1.0, 4.0], [4.0, 7.0, 2.0], [5.0, 3.0, 6.0]], [[3.0, 3.0, 8.0], [6.0, 6.0, 
1.0], [2.0, 5.0, 3.0]]], [[[1.0, 3.0, 5.0], [7.0, 2.0, 4.0], [6.0, 3.0, 1.0]], [[6.0, 5.0, 8.0], [4.0, 7.0, 4.0], [1.0, 5.0, 
3.0]], [[2.0, 4.0, 6.0], [6.0, 6.0, 4.0], [2.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[[5.0, 2.0, 7.0]], [[3.0, 7.0, 1.0]], [[5.0, 8.0, 4.0]]], [[[4.0, 3.0, 2.0]], [[5.0, 7.0, 8.0]], 
[[4.0, 8.0, 6.0]]], [[[1.0, 3.0, 7.0]], [[8.0, 8.0, 3.0]], [[5.0, 8.0, 8.0]]], [[[4.0, 2.0, 3.0]], [[3.0, 4.0, 7.0]], [[5.0, 
2.0, 8.0]]]]),self.functionspace)
      arg[:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 2.0], [5.0, 2.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 5.0, 5.0], [3.0, 7.0, 1.0], [1.0, 
7.0, 2.0]], [[5.0, 4.0, 7.0], [5.0, 8.0, 4.0], [7.0, 6.0, 1.0]]], [[[2.0, 3.0, 7.0], [4.0, 3.0, 2.0], [8.0, 2.0, 6.0]], [[5.0, 
5.0, 7.0], [5.0, 7.0, 8.0], [2.0, 4.0, 6.0]], [[5.0, 7.0, 1.0], [4.0, 8.0, 6.0], [8.0, 5.0, 4.0]]], [[[4.0, 6.0, 2.0], [1.0, 
3.0, 7.0], [7.0, 7.0, 3.0]], [[5.0, 7.0, 6.0], [8.0, 8.0, 3.0], [2.0, 2.0, 3.0]], [[8.0, 5.0, 8.0], [5.0, 8.0, 8.0], [2.0, 5.0, 
2.0]]], [[[2.0, 8.0, 4.0], [4.0, 2.0, 3.0], [2.0, 4.0, 3.0]], [[3.0, 4.0, 3.0], [3.0, 4.0, 7.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 
3.0], [5.0, 2.0, 8.0], [8.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 5.0], [5.0, 2.0, 7.0], [3.0, 7.0, 8.0]], 
[[7.0, 1.0, 6.0], [3.0, 7.0, 1.0], [5.0, 3.0, 8.0]], [[8.0, 8.0, 2.0], [5.0, 8.0, 4.0], [1.0, 7.0, 4.0]]], [[[7.0, 4.0, 1.0], 
[4.0, 3.0, 2.0], [5.0, 4.0, 1.0]], [[1.0, 4.0, 7.0], [5.0, 7.0, 8.0], [1.0, 6.0, 7.0]], [[4.0, 7.0, 6.0], [4.0, 8.0, 6.0], 
[5.0, 8.0, 8.0]]], [[[6.0, 7.0, 7.0], [1.0, 3.0, 7.0], [6.0, 4.0, 7.0]], [[7.0, 1.0, 4.0], [8.0, 8.0, 3.0], [5.0, 3.0, 6.0]], 
[[3.0, 3.0, 8.0], [5.0, 8.0, 8.0], [2.0, 5.0, 3.0]]], [[[1.0, 3.0, 5.0], [4.0, 2.0, 3.0], [6.0, 3.0, 1.0]], [[6.0, 5.0, 8.0], 
[3.0, 4.0, 7.0], [1.0, 5.0, 3.0]], [[2.0, 4.0, 6.0], [5.0, 2.0, 8.0], [2.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCRN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 8.0], [2.0, 6.0, 5.0], [5.0, 6.0, 7.0]], [[3.0, 6.0, 8.0], [5.0, 6.0, 5.0], [3.0, 4.0, 
7.0]], [[7.0, 1.0, 7.0], [5.0, 6.0, 2.0], [6.0, 3.0, 8.0]]], [[[7.0, 6.0, 1.0], [5.0, 4.0, 4.0], [6.0, 2.0, 3.0]], [[4.0, 3.0, 
7.0], [7.0, 7.0, 8.0], [8.0, 5.0, 3.0]], [[7.0, 8.0, 1.0], [6.0, 7.0, 8.0], [8.0, 5.0, 8.0]]], [[[2.0, 6.0, 3.0], [3.0, 5.0, 
6.0], [3.0, 7.0, 2.0]], [[8.0, 4.0, 5.0], [1.0, 7.0, 4.0], [7.0, 1.0, 8.0]], [[3.0, 8.0, 2.0], [3.0, 7.0, 7.0], [2.0, 6.0, 
7.0]]], [[[6.0, 2.0, 2.0], [5.0, 6.0, 6.0], [8.0, 3.0, 1.0]], [[7.0, 5.0, 4.0], [5.0, 4.0, 5.0], [8.0, 2.0, 2.0]], [[7.0, 5.0, 
1.0], [4.0, 2.0, 4.0], [7.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 5.0], [6.0, 1.0, 5.0], [5.0, 5.0, 2.0]], [[5.0, 6.0, 2.0], [2.0, 1.0, 
8.0], [6.0, 4.0, 2.0]], [[1.0, 7.0, 6.0], [4.0, 4.0, 3.0], [6.0, 3.0, 6.0]]], [[[5.0, 8.0, 4.0], [2.0, 7.0, 6.0], [6.0, 7.0, 
4.0]], [[5.0, 5.0, 8.0], [2.0, 4.0, 7.0], [3.0, 8.0, 4.0]], [[1.0, 2.0, 1.0], [1.0, 4.0, 2.0], [4.0, 5.0, 1.0]]], [[[7.0, 1.0, 
3.0], [8.0, 6.0, 4.0], [5.0, 3.0, 3.0]], [[8.0, 2.0, 7.0], [2.0, 2.0, 3.0], [6.0, 7.0, 4.0]], [[4.0, 5.0, 2.0], [8.0, 4.0, 
1.0], [4.0, 8.0, 6.0]]], [[[4.0, 1.0, 5.0], [6.0, 3.0, 5.0], [3.0, 1.0, 1.0]], [[6.0, 1.0, 1.0], [8.0, 2.0, 4.0], [6.0, 4.0, 
4.0]], [[7.0, 5.0, 8.0], [8.0, 6.0, 2.0], [6.0, 8.0, 6.0]]]]))
      val=Data(numpy.array([[[[8.0, 6.0, 1.0]], [[2.0, 7.0, 7.0]], [[6.0, 7.0, 3.0]]], [[[2.0, 4.0, 3.0]], [[6.0, 7.0, 2.0]], 
[[8.0, 3.0, 7.0]]], [[[6.0, 6.0, 7.0]], [[8.0, 5.0, 7.0]], [[1.0, 8.0, 8.0]]], [[[7.0, 1.0, 3.0]], [[4.0, 4.0, 7.0]], [[2.0, 
4.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 8.0]], [[8.0, 5.0, 4.0]], [[5.0, 6.0, 5.0]]], [[[6.0, 3.0, 4.0]], [[2.0, 
1.0, 3.0]], [[7.0, 7.0, 4.0]]], [[[2.0, 1.0, 6.0]], [[6.0, 5.0, 8.0]], [[5.0, 4.0, 7.0]]], [[[8.0, 2.0, 5.0]], [[5.0, 1.0, 
3.0]], [[7.0, 6.0, 6.0]]]]))
      arg[:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 8.0], [8.0, 6.0, 1.0], [5.0, 6.0, 7.0]], [[3.0, 6.0, 8.0], [2.0, 7.0, 7.0], [3.0, 
4.0, 7.0]], [[7.0, 1.0, 7.0], [6.0, 7.0, 3.0], [6.0, 3.0, 8.0]]], [[[7.0, 6.0, 1.0], [2.0, 4.0, 3.0], [6.0, 2.0, 3.0]], [[4.0, 
3.0, 7.0], [6.0, 7.0, 2.0], [8.0, 5.0, 3.0]], [[7.0, 8.0, 1.0], [8.0, 3.0, 7.0], [8.0, 5.0, 8.0]]], [[[2.0, 6.0, 3.0], [6.0, 
6.0, 7.0], [3.0, 7.0, 2.0]], [[8.0, 4.0, 5.0], [8.0, 5.0, 7.0], [7.0, 1.0, 8.0]], [[3.0, 8.0, 2.0], [1.0, 8.0, 8.0], [2.0, 6.0, 
7.0]]], [[[6.0, 2.0, 2.0], [7.0, 1.0, 3.0], [8.0, 3.0, 1.0]], [[7.0, 5.0, 4.0], [4.0, 4.0, 7.0], [8.0, 2.0, 2.0]], [[7.0, 5.0, 
1.0], [2.0, 4.0, 3.0], [7.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 5.0], [6.0, 5.0, 8.0], [5.0, 5.0, 2.0]], 
[[5.0, 6.0, 2.0], [8.0, 5.0, 4.0], [6.0, 4.0, 2.0]], [[1.0, 7.0, 6.0], [5.0, 6.0, 5.0], [6.0, 3.0, 6.0]]], [[[5.0, 8.0, 4.0], 
[6.0, 3.0, 4.0], [6.0, 7.0, 4.0]], [[5.0, 5.0, 8.0], [2.0, 1.0, 3.0], [3.0, 8.0, 4.0]], [[1.0, 2.0, 1.0], [7.0, 7.0, 4.0], 
[4.0, 5.0, 1.0]]], [[[7.0, 1.0, 3.0], [2.0, 1.0, 6.0], [5.0, 3.0, 3.0]], [[8.0, 2.0, 7.0], [6.0, 5.0, 8.0], [6.0, 7.0, 4.0]], 
[[4.0, 5.0, 2.0], [5.0, 4.0, 7.0], [4.0, 8.0, 6.0]]], [[[4.0, 1.0, 5.0], [8.0, 2.0, 5.0], [3.0, 1.0, 1.0]], [[6.0, 1.0, 1.0], 
[5.0, 1.0, 3.0], [6.0, 4.0, 4.0]], [[7.0, 5.0, 8.0], [7.0, 6.0, 6.0], [6.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCRN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 4.0], [2.0, 5.0, 2.0], [7.0, 4.0, 1.0]], [[1.0, 3.0, 7.0], [3.0, 1.0, 6.0], [5.0, 1.0, 
4.0]], [[5.0, 8.0, 6.0], [2.0, 8.0, 1.0], [7.0, 1.0, 6.0]]], [[[1.0, 5.0, 3.0], [2.0, 5.0, 6.0], [5.0, 6.0, 7.0]], [[2.0, 7.0, 
2.0], [1.0, 8.0, 8.0], [7.0, 5.0, 2.0]], [[5.0, 5.0, 3.0], [3.0, 7.0, 6.0], [6.0, 8.0, 1.0]]], [[[2.0, 4.0, 8.0], [8.0, 7.0, 
4.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 3.0], [8.0, 3.0, 4.0], [7.0, 6.0, 1.0]], [[1.0, 4.0, 5.0], [4.0, 1.0, 5.0], [7.0, 6.0, 
1.0]]], [[[5.0, 5.0, 2.0], [2.0, 2.0, 1.0], [6.0, 2.0, 5.0]], [[2.0, 8.0, 8.0], [5.0, 3.0, 5.0], [1.0, 5.0, 4.0]], [[5.0, 8.0, 
2.0], [7.0, 7.0, 3.0], [6.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 1.0], [7.0, 4.0, 4.0], [4.0, 4.0, 7.0]], [[6.0, 5.0, 4.0], [5.0, 5.0, 
5.0], [7.0, 2.0, 4.0]], [[1.0, 7.0, 8.0], [5.0, 5.0, 6.0], [8.0, 2.0, 8.0]]], [[[4.0, 3.0, 8.0], [5.0, 7.0, 4.0], [5.0, 1.0, 
6.0]], [[8.0, 3.0, 4.0], [5.0, 2.0, 5.0], [7.0, 1.0, 7.0]], [[6.0, 6.0, 8.0], [5.0, 8.0, 1.0], [6.0, 7.0, 5.0]]], [[[7.0, 8.0, 
8.0], [5.0, 7.0, 1.0], [6.0, 7.0, 8.0]], [[1.0, 2.0, 6.0], [4.0, 6.0, 8.0], [7.0, 3.0, 5.0]], [[6.0, 8.0, 5.0], [6.0, 1.0, 
1.0], [5.0, 8.0, 6.0]]], [[[8.0, 2.0, 3.0], [7.0, 7.0, 1.0], [2.0, 4.0, 8.0]], [[1.0, 7.0, 8.0], [3.0, 3.0, 8.0], [1.0, 5.0, 
4.0]], [[5.0, 3.0, 4.0], [8.0, 2.0, 4.0], [6.0, 4.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 3.0, 5.0]], [[5.0, 2.0, 8.0]], [[4.0, 6.0, 6.0]]], [[[8.0, 3.0, 3.0]], [[5.0, 1.0, 
6.0]], [[3.0, 3.0, 7.0]]], [[[4.0, 3.0, 2.0]], [[2.0, 2.0, 5.0]], [[3.0, 6.0, 1.0]]], [[[5.0, 7.0, 5.0]], [[1.0, 7.0, 3.0]], 
[[6.0, 3.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 3.0, 5.0]], [[3.0, 2.0, 8.0]], [[8.0, 6.0, 8.0]]], [[[4.0, 7.0, 6.0]], 
[[4.0, 7.0, 4.0]], [[4.0, 4.0, 1.0]]], [[[8.0, 7.0, 2.0]], [[4.0, 5.0, 4.0]], [[7.0, 2.0, 6.0]]], [[[3.0, 7.0, 3.0]], [[5.0, 
6.0, 1.0]], [[1.0, 6.0, 1.0]]]])
      arg[:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 4.0], [1.0, 3.0, 5.0], [7.0, 4.0, 1.0]], [[1.0, 3.0, 7.0], [5.0, 2.0, 8.0], [5.0, 
1.0, 4.0]], [[5.0, 8.0, 6.0], [4.0, 6.0, 6.0], [7.0, 1.0, 6.0]]], [[[1.0, 5.0, 3.0], [8.0, 3.0, 3.0], [5.0, 6.0, 7.0]], [[2.0, 
7.0, 2.0], [5.0, 1.0, 6.0], [7.0, 5.0, 2.0]], [[5.0, 5.0, 3.0], [3.0, 3.0, 7.0], [6.0, 8.0, 1.0]]], [[[2.0, 4.0, 8.0], [4.0, 
3.0, 2.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 3.0], [2.0, 2.0, 5.0], [7.0, 6.0, 1.0]], [[1.0, 4.0, 5.0], [3.0, 6.0, 1.0], [7.0, 6.0, 
1.0]]], [[[5.0, 5.0, 2.0], [5.0, 7.0, 5.0], [6.0, 2.0, 5.0]], [[2.0, 8.0, 8.0], [1.0, 7.0, 3.0], [1.0, 5.0, 4.0]], [[5.0, 8.0, 
2.0], [6.0, 3.0, 3.0], [6.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 1.0], [5.0, 3.0, 5.0], [4.0, 4.0, 7.0]], 
[[6.0, 5.0, 4.0], [3.0, 2.0, 8.0], [7.0, 2.0, 4.0]], [[1.0, 7.0, 8.0], [8.0, 6.0, 8.0], [8.0, 2.0, 8.0]]], [[[4.0, 3.0, 8.0], 
[4.0, 7.0, 6.0], [5.0, 1.0, 6.0]], [[8.0, 3.0, 4.0], [4.0, 7.0, 4.0], [7.0, 1.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 4.0, 1.0], 
[6.0, 7.0, 5.0]]], [[[7.0, 8.0, 8.0], [8.0, 7.0, 2.0], [6.0, 7.0, 8.0]], [[1.0, 2.0, 6.0], [4.0, 5.0, 4.0], [7.0, 3.0, 5.0]], 
[[6.0, 8.0, 5.0], [7.0, 2.0, 6.0], [5.0, 8.0, 6.0]]], [[[8.0, 2.0, 3.0], [3.0, 7.0, 3.0], [2.0, 4.0, 8.0]], [[1.0, 7.0, 8.0], 
[5.0, 6.0, 1.0], [1.0, 5.0, 4.0]], [[5.0, 3.0, 4.0], [1.0, 6.0, 1.0], [6.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RCCN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 6.0], [5.0, 4.0, 8.0], [3.0, 1.0, 1.0]], [[3.0, 2.0, 8.0], [5.0, 8.0, 3.0], [1.0, 3.0, 
5.0]], [[8.0, 3.0, 4.0], [8.0, 1.0, 1.0], [5.0, 3.0, 6.0]]], [[[6.0, 8.0, 1.0], [3.0, 1.0, 4.0], [5.0, 2.0, 3.0]], [[7.0, 4.0, 
8.0], [7.0, 8.0, 3.0], [8.0, 8.0, 4.0]], [[8.0, 4.0, 1.0], [6.0, 5.0, 1.0], [4.0, 7.0, 4.0]]], [[[2.0, 1.0, 1.0], [5.0, 7.0, 
7.0], [6.0, 7.0, 1.0]], [[7.0, 2.0, 3.0], [6.0, 2.0, 3.0], [8.0, 7.0, 7.0]], [[1.0, 6.0, 8.0], [7.0, 5.0, 2.0], [4.0, 3.0, 
8.0]]], [[[4.0, 6.0, 1.0], [4.0, 4.0, 7.0], [6.0, 4.0, 7.0]], [[8.0, 8.0, 2.0], [3.0, 1.0, 4.0], [1.0, 1.0, 2.0]], [[3.0, 3.0, 
1.0], [2.0, 6.0, 5.0], [5.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 7.0], [4.0, 3.0, 8.0], [3.0, 2.0, 5.0]], [[7.0, 3.0, 7.0], [8.0, 8.0, 
2.0], [5.0, 3.0, 4.0]], [[3.0, 7.0, 8.0], [4.0, 2.0, 7.0], [1.0, 7.0, 8.0]]], [[[5.0, 7.0, 6.0], [8.0, 2.0, 8.0], [7.0, 4.0, 
3.0]], [[8.0, 2.0, 1.0], [2.0, 5.0, 6.0], [8.0, 5.0, 4.0]], [[3.0, 3.0, 4.0], [4.0, 3.0, 1.0], [6.0, 3.0, 6.0]]], [[[2.0, 7.0, 
6.0], [5.0, 6.0, 6.0], [1.0, 4.0, 2.0]], [[8.0, 8.0, 5.0], [6.0, 7.0, 5.0], [7.0, 8.0, 8.0]], [[3.0, 1.0, 4.0], [2.0, 8.0, 
2.0], [8.0, 5.0, 4.0]]], [[[3.0, 5.0, 8.0], [1.0, 5.0, 2.0], [4.0, 3.0, 4.0]], [[8.0, 6.0, 3.0], [3.0, 7.0, 6.0], [3.0, 5.0, 
2.0]], [[7.0, 2.0, 6.0], [4.0, 2.0, 7.0], [7.0, 3.0, 1.0]]]]))
      val=numpy.array([[[[1.0, 8.0, 7.0], [8.0, 1.0, 4.0], [6.0, 7.0, 7.0]], [[1.0, 8.0, 3.0], [6.0, 7.0, 5.0], [6.0, 6.0, 
5.0]], [[1.0, 5.0, 7.0], [8.0, 7.0, 5.0], [3.0, 7.0, 4.0]]], [[[6.0, 8.0, 3.0], [5.0, 1.0, 4.0], [5.0, 7.0, 6.0]], [[5.0, 7.0, 
4.0], [8.0, 6.0, 5.0], [5.0, 5.0, 6.0]], [[1.0, 4.0, 3.0], [8.0, 5.0, 2.0], [3.0, 1.0, 1.0]]]])
      arg[1:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 6.0], [5.0, 4.0, 8.0], [3.0, 1.0, 1.0]], [[3.0, 2.0, 8.0], [5.0, 8.0, 3.0], [1.0, 
3.0, 5.0]], [[8.0, 3.0, 4.0], [8.0, 1.0, 1.0], [5.0, 3.0, 6.0]]], [[[1.0, 8.0, 7.0], [8.0, 1.0, 4.0], [6.0, 7.0, 7.0]], [[1.0, 
8.0, 3.0], [6.0, 7.0, 5.0], [6.0, 6.0, 5.0]], [[1.0, 5.0, 7.0], [8.0, 7.0, 5.0], [3.0, 7.0, 4.0]]], [[[6.0, 8.0, 3.0], [5.0, 
1.0, 4.0], [5.0, 7.0, 6.0]], [[5.0, 7.0, 4.0], [8.0, 6.0, 5.0], [5.0, 5.0, 6.0]], [[1.0, 4.0, 3.0], [8.0, 5.0, 2.0], [3.0, 1.0, 
1.0]]], [[[4.0, 6.0, 1.0], [4.0, 4.0, 7.0], [6.0, 4.0, 7.0]], [[8.0, 8.0, 2.0], [3.0, 1.0, 4.0], [1.0, 1.0, 2.0]], [[3.0, 3.0, 
1.0], [2.0, 6.0, 5.0], [5.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 7.0], [4.0, 3.0, 8.0], [3.0, 2.0, 5.0]], 
[[7.0, 3.0, 7.0], [8.0, 8.0, 2.0], [5.0, 3.0, 4.0]], [[3.0, 7.0, 8.0], [4.0, 2.0, 7.0], [1.0, 7.0, 8.0]]], [[[1.0, 8.0, 7.0], 
[8.0, 1.0, 4.0], [6.0, 7.0, 7.0]], [[1.0, 8.0, 3.0], [6.0, 7.0, 5.0], [6.0, 6.0, 5.0]], [[1.0, 5.0, 7.0], [8.0, 7.0, 5.0], 
[3.0, 7.0, 4.0]]], [[[6.0, 8.0, 3.0], [5.0, 1.0, 4.0], [5.0, 7.0, 6.0]], [[5.0, 7.0, 4.0], [8.0, 6.0, 5.0], [5.0, 5.0, 6.0]], 
[[1.0, 4.0, 3.0], [8.0, 5.0, 2.0], [3.0, 1.0, 1.0]]], [[[3.0, 5.0, 8.0], [1.0, 5.0, 2.0], [4.0, 3.0, 4.0]], [[8.0, 6.0, 3.0], 
[3.0, 7.0, 6.0], [3.0, 5.0, 2.0]], [[7.0, 2.0, 6.0], [4.0, 2.0, 7.0], [7.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RCCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 2.0], [1.0, 1.0, 7.0], [4.0, 6.0, 4.0]], [[5.0, 4.0, 7.0], [6.0, 3.0, 4.0], [7.0, 7.0, 
8.0]], [[2.0, 4.0, 4.0], [5.0, 5.0, 5.0], [5.0, 7.0, 3.0]]], [[[3.0, 1.0, 8.0], [1.0, 5.0, 5.0], [3.0, 3.0, 3.0]], [[5.0, 8.0, 
6.0], [1.0, 2.0, 3.0], [1.0, 8.0, 8.0]], [[2.0, 2.0, 6.0], [3.0, 5.0, 1.0], [6.0, 5.0, 4.0]]], [[[6.0, 7.0, 4.0], [5.0, 3.0, 
7.0], [7.0, 4.0, 8.0]], [[2.0, 2.0, 4.0], [8.0, 5.0, 2.0], [8.0, 2.0, 2.0]], [[4.0, 1.0, 2.0], [8.0, 5.0, 8.0], [4.0, 6.0, 
8.0]]], [[[6.0, 1.0, 5.0], [6.0, 3.0, 8.0], [8.0, 8.0, 7.0]], [[4.0, 5.0, 2.0], [7.0, 3.0, 4.0], [3.0, 1.0, 2.0]], [[4.0, 8.0, 
7.0], [5.0, 5.0, 2.0], [5.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 5.0], [5.0, 4.0, 2.0], [1.0, 2.0, 6.0]], [[1.0, 7.0, 5.0], [6.0, 2.0, 
4.0], [8.0, 1.0, 5.0]], [[8.0, 1.0, 3.0], [8.0, 8.0, 6.0], [2.0, 2.0, 3.0]]], [[[7.0, 5.0, 8.0], [5.0, 6.0, 5.0], [6.0, 6.0, 
7.0]], [[2.0, 4.0, 7.0], [8.0, 1.0, 6.0], [5.0, 8.0, 2.0]], [[5.0, 1.0, 7.0], [3.0, 7.0, 7.0], [5.0, 7.0, 2.0]]], [[[4.0, 8.0, 
2.0], [8.0, 7.0, 2.0], [2.0, 5.0, 3.0]], [[3.0, 1.0, 3.0], [4.0, 1.0, 7.0], [3.0, 3.0, 1.0]], [[5.0, 6.0, 7.0], [8.0, 8.0, 
8.0], [4.0, 6.0, 8.0]]], [[[6.0, 5.0, 1.0], [7.0, 3.0, 3.0], [3.0, 7.0, 3.0]], [[8.0, 1.0, 3.0], [4.0, 8.0, 2.0], [3.0, 5.0, 
3.0]], [[4.0, 1.0, 8.0], [4.0, 7.0, 2.0], [8.0, 8.0, 4.0]]]]))
      val=Data(numpy.array([[[[2.0, 5.0, 7.0], [7.0, 7.0, 7.0], [2.0, 3.0, 5.0]], [[2.0, 8.0, 5.0], [3.0, 2.0, 3.0], [7.0, 3.0, 
5.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 4.0], [6.0, 7.0, 5.0]]], [[[3.0, 2.0, 3.0], [6.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[8.0, 5.0, 
7.0], [2.0, 7.0, 5.0], [1.0, 5.0, 1.0]], [[5.0, 6.0, 6.0], [6.0, 7.0, 8.0], [6.0, 2.0, 8.0]]]]),self.functionspace)
      arg[1:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 2.0], [1.0, 1.0, 7.0], [4.0, 6.0, 4.0]], [[5.0, 4.0, 7.0], [6.0, 3.0, 4.0], [7.0, 
7.0, 8.0]], [[2.0, 4.0, 4.0], [5.0, 5.0, 5.0], [5.0, 7.0, 3.0]]], [[[2.0, 5.0, 7.0], [7.0, 7.0, 7.0], [2.0, 3.0, 5.0]], [[2.0, 
8.0, 5.0], [3.0, 2.0, 3.0], [7.0, 3.0, 5.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 4.0], [6.0, 7.0, 5.0]]], [[[3.0, 2.0, 3.0], [6.0, 
4.0, 6.0], [1.0, 6.0, 8.0]], [[8.0, 5.0, 7.0], [2.0, 7.0, 5.0], [1.0, 5.0, 1.0]], [[5.0, 6.0, 6.0], [6.0, 7.0, 8.0], [6.0, 2.0, 
8.0]]], [[[6.0, 1.0, 5.0], [6.0, 3.0, 8.0], [8.0, 8.0, 7.0]], [[4.0, 5.0, 2.0], [7.0, 3.0, 4.0], [3.0, 1.0, 2.0]], [[4.0, 8.0, 
7.0], [5.0, 5.0, 2.0], [5.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 5.0], [5.0, 4.0, 2.0], [1.0, 2.0, 6.0]], 
[[1.0, 7.0, 5.0], [6.0, 2.0, 4.0], [8.0, 1.0, 5.0]], [[8.0, 1.0, 3.0], [8.0, 8.0, 6.0], [2.0, 2.0, 3.0]]], [[[2.0, 5.0, 7.0], 
[7.0, 7.0, 7.0], [2.0, 3.0, 5.0]], [[2.0, 8.0, 5.0], [3.0, 2.0, 3.0], [7.0, 3.0, 5.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 4.0], 
[6.0, 7.0, 5.0]]], [[[3.0, 2.0, 3.0], [6.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[8.0, 5.0, 7.0], [2.0, 7.0, 5.0], [1.0, 5.0, 1.0]], 
[[5.0, 6.0, 6.0], [6.0, 7.0, 8.0], [6.0, 2.0, 8.0]]], [[[6.0, 5.0, 1.0], [7.0, 3.0, 3.0], [3.0, 7.0, 3.0]], [[8.0, 1.0, 3.0], 
[4.0, 8.0, 2.0], [3.0, 5.0, 3.0]], [[4.0, 1.0, 8.0], [4.0, 7.0, 2.0], [8.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RCCN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 2.0], [8.0, 1.0, 8.0], [8.0, 5.0, 2.0]], [[4.0, 4.0, 4.0], [6.0, 6.0, 3.0], [6.0, 6.0, 
7.0]], [[4.0, 8.0, 5.0], [6.0, 5.0, 4.0], [7.0, 4.0, 6.0]]], [[[5.0, 5.0, 2.0], [6.0, 4.0, 6.0], [6.0, 1.0, 1.0]], [[8.0, 2.0, 
7.0], [4.0, 8.0, 8.0], [3.0, 8.0, 4.0]], [[5.0, 2.0, 7.0], [5.0, 5.0, 7.0], [3.0, 6.0, 8.0]]], [[[8.0, 6.0, 4.0], [4.0, 4.0, 
5.0], [3.0, 3.0, 3.0]], [[5.0, 5.0, 8.0], [1.0, 6.0, 7.0], [6.0, 8.0, 2.0]], [[4.0, 4.0, 1.0], [7.0, 6.0, 3.0], [7.0, 1.0, 
3.0]]], [[[7.0, 2.0, 8.0], [4.0, 4.0, 8.0], [6.0, 7.0, 7.0]], [[5.0, 3.0, 4.0], [3.0, 5.0, 4.0], [2.0, 3.0, 5.0]], [[2.0, 5.0, 
5.0], [8.0, 2.0, 7.0], [8.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 2.0], [3.0, 7.0, 1.0], [8.0, 7.0, 4.0]], [[4.0, 5.0, 1.0], [1.0, 7.0, 
7.0], [7.0, 5.0, 2.0]], [[6.0, 1.0, 4.0], [2.0, 2.0, 4.0], [4.0, 3.0, 1.0]]], [[[1.0, 3.0, 6.0], [7.0, 5.0, 3.0], [2.0, 4.0, 
7.0]], [[6.0, 4.0, 7.0], [5.0, 5.0, 1.0], [5.0, 2.0, 4.0]], [[7.0, 7.0, 7.0], [5.0, 8.0, 8.0], [3.0, 3.0, 2.0]]], [[[7.0, 5.0, 
2.0], [4.0, 4.0, 1.0], [6.0, 7.0, 4.0]], [[4.0, 8.0, 1.0], [6.0, 6.0, 6.0], [5.0, 6.0, 5.0]], [[3.0, 4.0, 1.0], [3.0, 3.0, 
4.0], [4.0, 7.0, 8.0]]], [[[2.0, 2.0, 4.0], [2.0, 6.0, 3.0], [7.0, 3.0, 7.0]], [[4.0, 2.0, 8.0], [2.0, 4.0, 2.0], [5.0, 4.0, 
4.0]], [[2.0, 8.0, 3.0], [5.0, 4.0, 8.0], [2.0, 2.0, 5.0]]]]))
      val=Data(numpy.array([[[[8.0, 8.0, 5.0], [1.0, 3.0, 5.0], [8.0, 1.0, 3.0]], [[3.0, 3.0, 4.0], [4.0, 5.0, 1.0], [1.0, 1.0, 
7.0]], [[3.0, 4.0, 5.0], [3.0, 6.0, 3.0], [7.0, 1.0, 4.0]]], [[[5.0, 6.0, 8.0], [2.0, 8.0, 1.0], [4.0, 1.0, 3.0]], [[8.0, 2.0, 
3.0], [8.0, 6.0, 6.0], [7.0, 5.0, 7.0]], [[7.0, 2.0, 5.0], [6.0, 2.0, 3.0], [1.0, 3.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 7.0], [1.0, 3.0, 5.0], [3.0, 4.0, 6.0]], [[3.0, 6.0, 1.0], [2.0, 1.0, 
4.0], [8.0, 4.0, 1.0]], [[6.0, 2.0, 7.0], [1.0, 3.0, 5.0], [5.0, 8.0, 1.0]]], [[[5.0, 8.0, 4.0], [8.0, 3.0, 2.0], [8.0, 5.0, 
2.0]], [[6.0, 3.0, 5.0], [8.0, 5.0, 1.0], [5.0, 1.0, 5.0]], [[4.0, 4.0, 5.0], [4.0, 2.0, 2.0], [6.0, 3.0, 4.0]]]]))
      arg[1:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 2.0], [8.0, 1.0, 8.0], [8.0, 5.0, 2.0]], [[4.0, 4.0, 4.0], [6.0, 6.0, 3.0], [6.0, 
6.0, 7.0]], [[4.0, 8.0, 5.0], [6.0, 5.0, 4.0], [7.0, 4.0, 6.0]]], [[[8.0, 8.0, 5.0], [1.0, 3.0, 5.0], [8.0, 1.0, 3.0]], [[3.0, 
3.0, 4.0], [4.0, 5.0, 1.0], [1.0, 1.0, 7.0]], [[3.0, 4.0, 5.0], [3.0, 6.0, 3.0], [7.0, 1.0, 4.0]]], [[[5.0, 6.0, 8.0], [2.0, 
8.0, 1.0], [4.0, 1.0, 3.0]], [[8.0, 2.0, 3.0], [8.0, 6.0, 6.0], [7.0, 5.0, 7.0]], [[7.0, 2.0, 5.0], [6.0, 2.0, 3.0], [1.0, 3.0, 
1.0]]], [[[7.0, 2.0, 8.0], [4.0, 4.0, 8.0], [6.0, 7.0, 7.0]], [[5.0, 3.0, 4.0], [3.0, 5.0, 4.0], [2.0, 3.0, 5.0]], [[2.0, 5.0, 
5.0], [8.0, 2.0, 7.0], [8.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 2.0], [3.0, 7.0, 1.0], [8.0, 7.0, 4.0]], 
[[4.0, 5.0, 1.0], [1.0, 7.0, 7.0], [7.0, 5.0, 2.0]], [[6.0, 1.0, 4.0], [2.0, 2.0, 4.0], [4.0, 3.0, 1.0]]], [[[5.0, 1.0, 7.0], 
[1.0, 3.0, 5.0], [3.0, 4.0, 6.0]], [[3.0, 6.0, 1.0], [2.0, 1.0, 4.0], [8.0, 4.0, 1.0]], [[6.0, 2.0, 7.0], [1.0, 3.0, 5.0], 
[5.0, 8.0, 1.0]]], [[[5.0, 8.0, 4.0], [8.0, 3.0, 2.0], [8.0, 5.0, 2.0]], [[6.0, 3.0, 5.0], [8.0, 5.0, 1.0], [5.0, 1.0, 5.0]], 
[[4.0, 4.0, 5.0], [4.0, 2.0, 2.0], [6.0, 3.0, 4.0]]], [[[2.0, 2.0, 4.0], [2.0, 6.0, 3.0], [7.0, 3.0, 7.0]], [[4.0, 2.0, 8.0], 
[2.0, 4.0, 2.0], [5.0, 4.0, 4.0]], [[2.0, 8.0, 3.0], [5.0, 4.0, 8.0], [2.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RCCN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 1.0], [2.0, 7.0, 2.0], [6.0, 7.0, 2.0]], [[1.0, 6.0, 6.0], [1.0, 3.0, 2.0], [2.0, 3.0, 
1.0]], [[6.0, 3.0, 5.0], [5.0, 4.0, 5.0], [6.0, 1.0, 1.0]]], [[[2.0, 5.0, 6.0], [2.0, 1.0, 2.0], [4.0, 5.0, 2.0]], [[2.0, 5.0, 
5.0], [5.0, 6.0, 6.0], [2.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [2.0, 4.0, 7.0], [4.0, 6.0, 2.0]]], [[[5.0, 5.0, 2.0], [4.0, 6.0, 
2.0], [4.0, 8.0, 3.0]], [[4.0, 1.0, 6.0], [2.0, 1.0, 1.0], [3.0, 3.0, 1.0]], [[8.0, 2.0, 8.0], [3.0, 4.0, 2.0], [6.0, 6.0, 
8.0]]], [[[8.0, 5.0, 4.0], [2.0, 8.0, 5.0], [4.0, 3.0, 8.0]], [[5.0, 5.0, 3.0], [8.0, 3.0, 7.0], [6.0, 3.0, 8.0]], [[4.0, 3.0, 
6.0], [2.0, 5.0, 3.0], [5.0, 8.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 7.0], [2.0, 1.0, 5.0], [4.0, 4.0, 8.0]], [[2.0, 1.0, 3.0], [3.0, 2.0, 
5.0], [2.0, 6.0, 7.0]], [[4.0, 4.0, 1.0], [6.0, 2.0, 3.0], [8.0, 3.0, 5.0]]], [[[5.0, 4.0, 6.0], [6.0, 6.0, 2.0], [6.0, 8.0, 
8.0]], [[8.0, 7.0, 4.0], [2.0, 5.0, 8.0], [6.0, 3.0, 5.0]], [[6.0, 8.0, 5.0], [4.0, 2.0, 2.0], [4.0, 7.0, 4.0]]], [[[3.0, 1.0, 
5.0], [5.0, 8.0, 6.0], [6.0, 4.0, 2.0]], [[5.0, 3.0, 1.0], [7.0, 2.0, 2.0], [2.0, 2.0, 3.0]], [[6.0, 8.0, 3.0], [6.0, 6.0, 
6.0], [7.0, 4.0, 6.0]]], [[[3.0, 8.0, 5.0], [1.0, 2.0, 2.0], [5.0, 5.0, 2.0]], [[8.0, 5.0, 1.0], [3.0, 2.0, 5.0], [7.0, 4.0, 
6.0]], [[1.0, 8.0, 1.0], [3.0, 3.0, 7.0], [6.0, 2.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 6.0, 4.0], [6.0, 2.0, 8.0], [6.0, 7.0, 2.0]], [[8.0, 7.0, 2.0], [4.0, 8.0, 4.0], [2.0, 
8.0, 5.0]], [[7.0, 6.0, 6.0], [4.0, 4.0, 4.0], [3.0, 8.0, 7.0]]], [[[7.0, 3.0, 6.0], [8.0, 7.0, 6.0], [8.0, 2.0, 8.0]], [[8.0, 
5.0, 1.0], [1.0, 4.0, 5.0], [7.0, 1.0, 6.0]], [[2.0, 1.0, 5.0], [3.0, 8.0, 1.0], [5.0, 3.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 5.0, 7.0], [2.0, 7.0, 3.0], [1.0, 5.0, 3.0]], [[5.0, 3.0, 5.0], [1.0, 3.0, 7.0], 
[8.0, 6.0, 8.0]], [[7.0, 5.0, 6.0], [7.0, 7.0, 1.0], [2.0, 2.0, 2.0]]], [[[8.0, 4.0, 4.0], [4.0, 1.0, 5.0], [1.0, 3.0, 1.0]], 
[[1.0, 4.0, 6.0], [2.0, 5.0, 5.0], [2.0, 7.0, 7.0]], [[8.0, 2.0, 2.0], [6.0, 5.0, 5.0], [8.0, 5.0, 3.0]]]])
      arg[1:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 1.0], [2.0, 7.0, 2.0], [6.0, 7.0, 2.0]], [[1.0, 6.0, 6.0], [1.0, 3.0, 2.0], [2.0, 
3.0, 1.0]], [[6.0, 3.0, 5.0], [5.0, 4.0, 5.0], [6.0, 1.0, 1.0]]], [[[2.0, 6.0, 4.0], [6.0, 2.0, 8.0], [6.0, 7.0, 2.0]], [[8.0, 
7.0, 2.0], [4.0, 8.0, 4.0], [2.0, 8.0, 5.0]], [[7.0, 6.0, 6.0], [4.0, 4.0, 4.0], [3.0, 8.0, 7.0]]], [[[7.0, 3.0, 6.0], [8.0, 
7.0, 6.0], [8.0, 2.0, 8.0]], [[8.0, 5.0, 1.0], [1.0, 4.0, 5.0], [7.0, 1.0, 6.0]], [[2.0, 1.0, 5.0], [3.0, 8.0, 1.0], [5.0, 3.0, 
2.0]]], [[[8.0, 5.0, 4.0], [2.0, 8.0, 5.0], [4.0, 3.0, 8.0]], [[5.0, 5.0, 3.0], [8.0, 3.0, 7.0], [6.0, 3.0, 8.0]], [[4.0, 3.0, 
6.0], [2.0, 5.0, 3.0], [5.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 7.0], [2.0, 1.0, 5.0], [4.0, 4.0, 8.0]], 
[[2.0, 1.0, 3.0], [3.0, 2.0, 5.0], [2.0, 6.0, 7.0]], [[4.0, 4.0, 1.0], [6.0, 2.0, 3.0], [8.0, 3.0, 5.0]]], [[[1.0, 5.0, 7.0], 
[2.0, 7.0, 3.0], [1.0, 5.0, 3.0]], [[5.0, 3.0, 5.0], [1.0, 3.0, 7.0], [8.0, 6.0, 8.0]], [[7.0, 5.0, 6.0], [7.0, 7.0, 1.0], 
[2.0, 2.0, 2.0]]], [[[8.0, 4.0, 4.0], [4.0, 1.0, 5.0], [1.0, 3.0, 1.0]], [[1.0, 4.0, 6.0], [2.0, 5.0, 5.0], [2.0, 7.0, 7.0]], 
[[8.0, 2.0, 2.0], [6.0, 5.0, 5.0], [8.0, 5.0, 3.0]]], [[[3.0, 8.0, 5.0], [1.0, 2.0, 2.0], [5.0, 5.0, 2.0]], [[8.0, 5.0, 1.0], 
[3.0, 2.0, 5.0], [7.0, 4.0, 6.0]], [[1.0, 8.0, 1.0], [3.0, 3.0, 7.0], [6.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPUN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 2.0], [2.0, 3.0, 1.0], [7.0, 6.0, 2.0]], [[8.0, 1.0, 1.0], [3.0, 8.0, 1.0], [6.0, 1.0, 
6.0]], [[1.0, 2.0, 7.0], [4.0, 8.0, 7.0], [1.0, 1.0, 2.0]]], [[[5.0, 5.0, 3.0], [5.0, 2.0, 4.0], [2.0, 7.0, 1.0]], [[3.0, 1.0, 
8.0], [3.0, 8.0, 7.0], [2.0, 8.0, 4.0]], [[2.0, 6.0, 6.0], [2.0, 1.0, 4.0], [6.0, 5.0, 5.0]]], [[[8.0, 8.0, 7.0], [6.0, 7.0, 
4.0], [6.0, 4.0, 7.0]], [[2.0, 5.0, 7.0], [6.0, 2.0, 7.0], [2.0, 5.0, 7.0]], [[6.0, 5.0, 5.0], [6.0, 1.0, 2.0], [8.0, 7.0, 
1.0]]], [[[4.0, 5.0, 3.0], [3.0, 8.0, 4.0], [6.0, 7.0, 8.0]], [[6.0, 1.0, 1.0], [3.0, 3.0, 2.0], [6.0, 4.0, 4.0]], [[1.0, 5.0, 
8.0], [7.0, 3.0, 3.0], [2.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 4.0], [7.0, 7.0, 5.0], [8.0, 5.0, 2.0]], [[5.0, 4.0, 8.0], [5.0, 8.0, 
2.0], [1.0, 6.0, 2.0]], [[5.0, 2.0, 7.0], [4.0, 1.0, 8.0], [7.0, 8.0, 6.0]]], [[[5.0, 3.0, 4.0], [7.0, 8.0, 8.0], [3.0, 8.0, 
6.0]], [[7.0, 4.0, 6.0], [5.0, 2.0, 4.0], [2.0, 2.0, 8.0]], [[5.0, 8.0, 1.0], [4.0, 2.0, 1.0], [5.0, 8.0, 7.0]]], [[[6.0, 6.0, 
4.0], [4.0, 6.0, 1.0], [3.0, 6.0, 5.0]], [[8.0, 3.0, 3.0], [4.0, 7.0, 5.0], [4.0, 4.0, 3.0]], [[4.0, 2.0, 5.0], [6.0, 1.0, 
2.0], [2.0, 4.0, 8.0]]], [[[4.0, 4.0, 6.0], [1.0, 5.0, 6.0], [1.0, 3.0, 4.0]], [[8.0, 1.0, 7.0], [4.0, 8.0, 5.0], [4.0, 6.0, 
6.0]], [[4.0, 4.0, 6.0], [2.0, 2.0, 5.0], [6.0, 6.0, 6.0]]]]))
      val=numpy.array([[6.0, 8.0, 7.0], [7.0, 6.0, 7.0]])
      arg[2,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 2.0], [2.0, 3.0, 1.0], [7.0, 6.0, 2.0]], [[8.0, 1.0, 1.0], [3.0, 8.0, 1.0], [6.0, 
1.0, 6.0]], [[1.0, 2.0, 7.0], [4.0, 8.0, 7.0], [1.0, 1.0, 2.0]]], [[[5.0, 5.0, 3.0], [5.0, 2.0, 4.0], [2.0, 7.0, 1.0]], [[3.0, 
1.0, 8.0], [3.0, 8.0, 7.0], [2.0, 8.0, 4.0]], [[2.0, 6.0, 6.0], [2.0, 1.0, 4.0], [6.0, 5.0, 5.0]]], [[[8.0, 8.0, 7.0], [6.0, 
7.0, 4.0], [6.0, 4.0, 7.0]], [[6.0, 8.0, 7.0], [7.0, 6.0, 7.0], [2.0, 5.0, 7.0]], [[6.0, 5.0, 5.0], [6.0, 1.0, 2.0], [8.0, 7.0, 
1.0]]], [[[4.0, 5.0, 3.0], [3.0, 8.0, 4.0], [6.0, 7.0, 8.0]], [[6.0, 1.0, 1.0], [3.0, 3.0, 2.0], [6.0, 4.0, 4.0]], [[1.0, 5.0, 
8.0], [7.0, 3.0, 3.0], [2.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 4.0], [7.0, 7.0, 5.0], [8.0, 5.0, 2.0]], 
[[5.0, 4.0, 8.0], [5.0, 8.0, 2.0], [1.0, 6.0, 2.0]], [[5.0, 2.0, 7.0], [4.0, 1.0, 8.0], [7.0, 8.0, 6.0]]], [[[5.0, 3.0, 4.0], 
[7.0, 8.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 4.0, 6.0], [5.0, 2.0, 4.0], [2.0, 2.0, 8.0]], [[5.0, 8.0, 1.0], [4.0, 2.0, 1.0], 
[5.0, 8.0, 7.0]]], [[[6.0, 6.0, 4.0], [4.0, 6.0, 1.0], [3.0, 6.0, 5.0]], [[6.0, 8.0, 7.0], [7.0, 6.0, 7.0], [4.0, 4.0, 3.0]], 
[[4.0, 2.0, 5.0], [6.0, 1.0, 2.0], [2.0, 4.0, 8.0]]], [[[4.0, 4.0, 6.0], [1.0, 5.0, 6.0], [1.0, 3.0, 4.0]], [[8.0, 1.0, 7.0], 
[4.0, 8.0, 5.0], [4.0, 6.0, 6.0]], [[4.0, 4.0, 6.0], [2.0, 2.0, 5.0], [6.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPUN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 5.0], [4.0, 7.0, 2.0], [5.0, 8.0, 1.0]], [[6.0, 1.0, 3.0], [1.0, 5.0, 1.0], [7.0, 5.0, 
2.0]], [[7.0, 4.0, 1.0], [2.0, 2.0, 2.0], [8.0, 4.0, 6.0]]], [[[7.0, 1.0, 7.0], [5.0, 5.0, 7.0], [1.0, 7.0, 7.0]], [[2.0, 7.0, 
8.0], [7.0, 3.0, 1.0], [5.0, 7.0, 2.0]], [[7.0, 3.0, 3.0], [1.0, 4.0, 5.0], [8.0, 1.0, 6.0]]], [[[7.0, 7.0, 3.0], [2.0, 1.0, 
2.0], [3.0, 5.0, 6.0]], [[2.0, 5.0, 5.0], [7.0, 8.0, 2.0], [6.0, 4.0, 8.0]], [[1.0, 5.0, 6.0], [2.0, 5.0, 1.0], [4.0, 8.0, 
5.0]]], [[[7.0, 3.0, 5.0], [3.0, 2.0, 3.0], [2.0, 1.0, 2.0]], [[4.0, 7.0, 2.0], [1.0, 6.0, 4.0], [6.0, 8.0, 6.0]], [[3.0, 8.0, 
2.0], [8.0, 7.0, 2.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 3.0], [6.0, 4.0, 1.0], [2.0, 7.0, 2.0]], [[8.0, 8.0, 1.0], [8.0, 2.0, 
1.0], [3.0, 8.0, 1.0]], [[1.0, 6.0, 5.0], [3.0, 4.0, 5.0], [3.0, 3.0, 1.0]]], [[[6.0, 2.0, 7.0], [4.0, 8.0, 4.0], [2.0, 4.0, 
2.0]], [[1.0, 5.0, 3.0], [6.0, 2.0, 6.0], [4.0, 3.0, 4.0]], [[4.0, 1.0, 1.0], [5.0, 1.0, 8.0], [2.0, 4.0, 2.0]]], [[[8.0, 4.0, 
2.0], [8.0, 5.0, 7.0], [3.0, 5.0, 1.0]], [[3.0, 3.0, 8.0], [7.0, 6.0, 2.0], [7.0, 1.0, 2.0]], [[6.0, 5.0, 1.0], [1.0, 8.0, 
4.0], [1.0, 8.0, 6.0]]], [[[2.0, 2.0, 4.0], [6.0, 6.0, 7.0], [2.0, 6.0, 6.0]], [[7.0, 7.0, 2.0], [1.0, 7.0, 8.0], [6.0, 6.0, 
5.0]], [[7.0, 2.0, 8.0], [2.0, 3.0, 3.0], [6.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[7.0, 3.0, 3.0], [2.0, 4.0, 1.0]]),self.functionspace)
      arg[2,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 5.0], [4.0, 7.0, 2.0], [5.0, 8.0, 1.0]], [[6.0, 1.0, 3.0], [1.0, 5.0, 1.0], [7.0, 
5.0, 2.0]], [[7.0, 4.0, 1.0], [2.0, 2.0, 2.0], [8.0, 4.0, 6.0]]], [[[7.0, 1.0, 7.0], [5.0, 5.0, 7.0], [1.0, 7.0, 7.0]], [[2.0, 
7.0, 8.0], [7.0, 3.0, 1.0], [5.0, 7.0, 2.0]], [[7.0, 3.0, 3.0], [1.0, 4.0, 5.0], [8.0, 1.0, 6.0]]], [[[7.0, 7.0, 3.0], [2.0, 
1.0, 2.0], [3.0, 5.0, 6.0]], [[7.0, 3.0, 3.0], [2.0, 4.0, 1.0], [6.0, 4.0, 8.0]], [[1.0, 5.0, 6.0], [2.0, 5.0, 1.0], [4.0, 8.0, 
5.0]]], [[[7.0, 3.0, 5.0], [3.0, 2.0, 3.0], [2.0, 1.0, 2.0]], [[4.0, 7.0, 2.0], [1.0, 6.0, 4.0], [6.0, 8.0, 6.0]], [[3.0, 8.0, 
2.0], [8.0, 7.0, 2.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 3.0], [6.0, 4.0, 1.0], [2.0, 7.0, 2.0]], 
[[8.0, 8.0, 1.0], [8.0, 2.0, 1.0], [3.0, 8.0, 1.0]], [[1.0, 6.0, 5.0], [3.0, 4.0, 5.0], [3.0, 3.0, 1.0]]], [[[6.0, 2.0, 7.0], 
[4.0, 8.0, 4.0], [2.0, 4.0, 2.0]], [[1.0, 5.0, 3.0], [6.0, 2.0, 6.0], [4.0, 3.0, 4.0]], [[4.0, 1.0, 1.0], [5.0, 1.0, 8.0], 
[2.0, 4.0, 2.0]]], [[[8.0, 4.0, 2.0], [8.0, 5.0, 7.0], [3.0, 5.0, 1.0]], [[7.0, 3.0, 3.0], [2.0, 4.0, 1.0], [7.0, 1.0, 2.0]], 
[[6.0, 5.0, 1.0], [1.0, 8.0, 4.0], [1.0, 8.0, 6.0]]], [[[2.0, 2.0, 4.0], [6.0, 6.0, 7.0], [2.0, 6.0, 6.0]], [[7.0, 7.0, 2.0], 
[1.0, 7.0, 8.0], [6.0, 6.0, 5.0]], [[7.0, 2.0, 8.0], [2.0, 3.0, 3.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPUN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 5.0], [3.0, 2.0, 7.0], [1.0, 6.0, 5.0]], [[3.0, 8.0, 2.0], [8.0, 7.0, 8.0], [3.0, 5.0, 
6.0]], [[5.0, 2.0, 1.0], [3.0, 5.0, 2.0], [3.0, 4.0, 6.0]]], [[[5.0, 1.0, 4.0], [6.0, 5.0, 8.0], [7.0, 3.0, 3.0]], [[3.0, 4.0, 
1.0], [8.0, 7.0, 1.0], [4.0, 5.0, 6.0]], [[2.0, 2.0, 6.0], [5.0, 2.0, 4.0], [1.0, 1.0, 4.0]]], [[[8.0, 6.0, 4.0], [8.0, 4.0, 
4.0], [3.0, 3.0, 6.0]], [[2.0, 7.0, 5.0], [3.0, 6.0, 1.0], [1.0, 5.0, 4.0]], [[7.0, 4.0, 3.0], [7.0, 3.0, 1.0], [5.0, 5.0, 
5.0]]], [[[7.0, 3.0, 4.0], [2.0, 3.0, 3.0], [3.0, 1.0, 3.0]], [[7.0, 5.0, 5.0], [3.0, 2.0, 4.0], [6.0, 8.0, 2.0]], [[1.0, 7.0, 
6.0], [8.0, 4.0, 6.0], [8.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 7.0], [3.0, 3.0, 8.0], [2.0, 7.0, 1.0]], [[8.0, 8.0, 6.0], [8.0, 2.0, 
1.0], [5.0, 6.0, 3.0]], [[5.0, 3.0, 5.0], [4.0, 7.0, 7.0], [1.0, 6.0, 6.0]]], [[[2.0, 5.0, 5.0], [3.0, 2.0, 5.0], [5.0, 6.0, 
6.0]], [[5.0, 2.0, 3.0], [6.0, 3.0, 8.0], [4.0, 5.0, 6.0]], [[2.0, 8.0, 7.0], [2.0, 6.0, 2.0], [7.0, 6.0, 5.0]]], [[[5.0, 8.0, 
3.0], [7.0, 7.0, 2.0], [5.0, 7.0, 6.0]], [[5.0, 2.0, 1.0], [1.0, 3.0, 8.0], [5.0, 2.0, 2.0]], [[3.0, 5.0, 3.0], [3.0, 6.0, 
2.0], [2.0, 7.0, 3.0]]], [[[1.0, 6.0, 4.0], [8.0, 7.0, 4.0], [8.0, 1.0, 6.0]], [[8.0, 4.0, 1.0], [3.0, 1.0, 2.0], [2.0, 2.0, 
7.0]], [[2.0, 8.0, 4.0], [8.0, 4.0, 7.0], [5.0, 6.0, 2.0]]]]))
      val=Data(numpy.array([[3.0, 8.0, 3.0], [6.0, 6.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 5.0, 8.0], [3.0, 8.0, 4.0]]))
      arg[2,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 5.0], [3.0, 2.0, 7.0], [1.0, 6.0, 5.0]], [[3.0, 8.0, 2.0], [8.0, 7.0, 8.0], [3.0, 
5.0, 6.0]], [[5.0, 2.0, 1.0], [3.0, 5.0, 2.0], [3.0, 4.0, 6.0]]], [[[5.0, 1.0, 4.0], [6.0, 5.0, 8.0], [7.0, 3.0, 3.0]], [[3.0, 
4.0, 1.0], [8.0, 7.0, 1.0], [4.0, 5.0, 6.0]], [[2.0, 2.0, 6.0], [5.0, 2.0, 4.0], [1.0, 1.0, 4.0]]], [[[8.0, 6.0, 4.0], [8.0, 
4.0, 4.0], [3.0, 3.0, 6.0]], [[3.0, 8.0, 3.0], [6.0, 6.0, 7.0], [1.0, 5.0, 4.0]], [[7.0, 4.0, 3.0], [7.0, 3.0, 1.0], [5.0, 5.0, 
5.0]]], [[[7.0, 3.0, 4.0], [2.0, 3.0, 3.0], [3.0, 1.0, 3.0]], [[7.0, 5.0, 5.0], [3.0, 2.0, 4.0], [6.0, 8.0, 2.0]], [[1.0, 7.0, 
6.0], [8.0, 4.0, 6.0], [8.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 7.0], [3.0, 3.0, 8.0], [2.0, 7.0, 1.0]], 
[[8.0, 8.0, 6.0], [8.0, 2.0, 1.0], [5.0, 6.0, 3.0]], [[5.0, 3.0, 5.0], [4.0, 7.0, 7.0], [1.0, 6.0, 6.0]]], [[[2.0, 5.0, 5.0], 
[3.0, 2.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 2.0, 3.0], [6.0, 3.0, 8.0], [4.0, 5.0, 6.0]], [[2.0, 8.0, 7.0], [2.0, 6.0, 2.0], 
[7.0, 6.0, 5.0]]], [[[5.0, 8.0, 3.0], [7.0, 7.0, 2.0], [5.0, 7.0, 6.0]], [[6.0, 5.0, 8.0], [3.0, 8.0, 4.0], [5.0, 2.0, 2.0]], 
[[3.0, 5.0, 3.0], [3.0, 6.0, 2.0], [2.0, 7.0, 3.0]]], [[[1.0, 6.0, 4.0], [8.0, 7.0, 4.0], [8.0, 1.0, 6.0]], [[8.0, 4.0, 1.0], 
[3.0, 1.0, 2.0], [2.0, 2.0, 7.0]], [[2.0, 8.0, 4.0], [8.0, 4.0, 7.0], [5.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPUN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 3.0], [7.0, 5.0, 1.0], [6.0, 5.0, 4.0]], [[5.0, 7.0, 5.0], [2.0, 2.0, 7.0], [3.0, 4.0, 
7.0]], [[4.0, 6.0, 8.0], [6.0, 4.0, 8.0], [8.0, 6.0, 7.0]]], [[[1.0, 4.0, 7.0], [8.0, 1.0, 6.0], [5.0, 2.0, 4.0]], [[3.0, 2.0, 
7.0], [7.0, 7.0, 6.0], [4.0, 4.0, 2.0]], [[6.0, 3.0, 7.0], [4.0, 8.0, 6.0], [7.0, 3.0, 2.0]]], [[[6.0, 4.0, 2.0], [8.0, 1.0, 
1.0], [3.0, 8.0, 5.0]], [[8.0, 8.0, 5.0], [5.0, 3.0, 1.0], [4.0, 8.0, 3.0]], [[5.0, 6.0, 7.0], [7.0, 7.0, 2.0], [1.0, 6.0, 
8.0]]], [[[5.0, 7.0, 2.0], [4.0, 3.0, 1.0], [1.0, 2.0, 8.0]], [[8.0, 4.0, 2.0], [4.0, 4.0, 7.0], [3.0, 2.0, 3.0]], [[2.0, 4.0, 
4.0], [2.0, 8.0, 4.0], [3.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 6.0], [4.0, 2.0, 6.0], [6.0, 2.0, 1.0]], [[4.0, 3.0, 7.0], [8.0, 3.0, 
7.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 8.0], [5.0, 6.0, 3.0], [4.0, 5.0, 5.0]]], [[[1.0, 1.0, 4.0], [3.0, 3.0, 5.0], [5.0, 2.0, 
1.0]], [[1.0, 1.0, 2.0], [1.0, 3.0, 7.0], [8.0, 3.0, 3.0]], [[8.0, 8.0, 1.0], [2.0, 4.0, 6.0], [6.0, 7.0, 1.0]]], [[[7.0, 3.0, 
3.0], [4.0, 8.0, 3.0], [3.0, 7.0, 8.0]], [[2.0, 8.0, 8.0], [3.0, 2.0, 1.0], [4.0, 8.0, 2.0]], [[6.0, 4.0, 5.0], [2.0, 7.0, 
1.0], [1.0, 1.0, 1.0]]], [[[7.0, 3.0, 4.0], [2.0, 5.0, 5.0], [5.0, 5.0, 3.0]], [[5.0, 3.0, 6.0], [3.0, 3.0, 4.0], [7.0, 8.0, 
5.0]], [[6.0, 8.0, 3.0], [7.0, 3.0, 2.0], [7.0, 8.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[2.0, 5.0, 3.0], [2.0, 7.0, 4.0]])+(1.-msk_val)*numpy.array([[1.0, 6.0, 8.0], [6.0, 8.0, 1.0]])
      arg[2,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [7.0, 5.0, 1.0], [6.0, 5.0, 4.0]], [[5.0, 7.0, 5.0], [2.0, 2.0, 7.0], [3.0, 
4.0, 7.0]], [[4.0, 6.0, 8.0], [6.0, 4.0, 8.0], [8.0, 6.0, 7.0]]], [[[1.0, 4.0, 7.0], [8.0, 1.0, 6.0], [5.0, 2.0, 4.0]], [[3.0, 
2.0, 7.0], [7.0, 7.0, 6.0], [4.0, 4.0, 2.0]], [[6.0, 3.0, 7.0], [4.0, 8.0, 6.0], [7.0, 3.0, 2.0]]], [[[6.0, 4.0, 2.0], [8.0, 
1.0, 1.0], [3.0, 8.0, 5.0]], [[2.0, 5.0, 3.0], [2.0, 7.0, 4.0], [4.0, 8.0, 3.0]], [[5.0, 6.0, 7.0], [7.0, 7.0, 2.0], [1.0, 6.0, 
8.0]]], [[[5.0, 7.0, 2.0], [4.0, 3.0, 1.0], [1.0, 2.0, 8.0]], [[8.0, 4.0, 2.0], [4.0, 4.0, 7.0], [3.0, 2.0, 3.0]], [[2.0, 4.0, 
4.0], [2.0, 8.0, 4.0], [3.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 6.0], [4.0, 2.0, 6.0], [6.0, 2.0, 1.0]], 
[[4.0, 3.0, 7.0], [8.0, 3.0, 7.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 8.0], [5.0, 6.0, 3.0], [4.0, 5.0, 5.0]]], [[[1.0, 1.0, 4.0], 
[3.0, 3.0, 5.0], [5.0, 2.0, 1.0]], [[1.0, 1.0, 2.0], [1.0, 3.0, 7.0], [8.0, 3.0, 3.0]], [[8.0, 8.0, 1.0], [2.0, 4.0, 6.0], 
[6.0, 7.0, 1.0]]], [[[7.0, 3.0, 3.0], [4.0, 8.0, 3.0], [3.0, 7.0, 8.0]], [[1.0, 6.0, 8.0], [6.0, 8.0, 1.0], [4.0, 8.0, 2.0]], 
[[6.0, 4.0, 5.0], [2.0, 7.0, 1.0], [1.0, 1.0, 1.0]]], [[[7.0, 3.0, 4.0], [2.0, 5.0, 5.0], [5.0, 5.0, 3.0]], [[5.0, 3.0, 6.0], 
[3.0, 3.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 8.0, 3.0], [7.0, 3.0, 2.0], [7.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CRCN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 3.0], [3.0, 4.0, 3.0], [6.0, 8.0, 5.0]], [[3.0, 8.0, 7.0], [6.0, 6.0, 2.0], [4.0, 4.0, 
7.0]], [[1.0, 1.0, 5.0], [5.0, 6.0, 7.0], [1.0, 7.0, 5.0]]], [[[8.0, 2.0, 8.0], [4.0, 3.0, 6.0], [6.0, 1.0, 2.0]], [[4.0, 1.0, 
6.0], [2.0, 1.0, 1.0], [3.0, 8.0, 2.0]], [[4.0, 1.0, 5.0], [6.0, 4.0, 4.0], [8.0, 2.0, 2.0]]], [[[1.0, 1.0, 2.0], [8.0, 8.0, 
8.0], [1.0, 2.0, 2.0]], [[1.0, 2.0, 4.0], [6.0, 5.0, 7.0], [8.0, 1.0, 6.0]], [[6.0, 8.0, 8.0], [5.0, 4.0, 5.0], [3.0, 6.0, 
5.0]]], [[[4.0, 3.0, 1.0], [5.0, 5.0, 7.0], [6.0, 3.0, 6.0]], [[8.0, 6.0, 3.0], [1.0, 5.0, 3.0], [8.0, 7.0, 8.0]], [[3.0, 1.0, 
7.0], [1.0, 2.0, 7.0], [4.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 6.0], [3.0, 7.0, 7.0], [3.0, 1.0, 6.0]], [[4.0, 6.0, 5.0], [4.0, 3.0, 
3.0], [7.0, 6.0, 6.0]], [[4.0, 2.0, 2.0], [5.0, 3.0, 5.0], [3.0, 3.0, 7.0]]], [[[2.0, 1.0, 7.0], [7.0, 5.0, 5.0], [6.0, 2.0, 
5.0]], [[3.0, 3.0, 2.0], [5.0, 2.0, 4.0], [8.0, 6.0, 4.0]], [[7.0, 5.0, 4.0], [8.0, 7.0, 5.0], [7.0, 5.0, 5.0]]], [[[3.0, 8.0, 
3.0], [6.0, 6.0, 5.0], [3.0, 8.0, 4.0]], [[2.0, 6.0, 8.0], [1.0, 7.0, 4.0], [6.0, 3.0, 2.0]], [[6.0, 2.0, 2.0], [6.0, 6.0, 
6.0], [4.0, 2.0, 5.0]]], [[[3.0, 5.0, 1.0], [5.0, 7.0, 1.0], [7.0, 1.0, 1.0]], [[4.0, 5.0, 2.0], [8.0, 1.0, 3.0], [4.0, 7.0, 
4.0]], [[2.0, 4.0, 5.0], [4.0, 3.0, 6.0], [2.0, 5.0, 1.0]]]]))
      val=numpy.array([[[[4.0, 5.0, 4.0], [3.0, 1.0, 7.0], [6.0, 7.0, 2.0]]], [[[1.0, 3.0, 4.0], [6.0, 4.0, 3.0], [3.0, 6.0, 
3.0]]], [[[3.0, 6.0, 6.0], [6.0, 4.0, 6.0], [7.0, 3.0, 4.0]]], [[[3.0, 1.0, 8.0], [3.0, 7.0, 1.0], [5.0, 7.0, 3.0]]]])
      arg[:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 3.0], [3.0, 4.0, 3.0], [6.0, 8.0, 5.0]], [[4.0, 5.0, 4.0], [3.0, 1.0, 7.0], [6.0, 
7.0, 2.0]], [[1.0, 1.0, 5.0], [5.0, 6.0, 7.0], [1.0, 7.0, 5.0]]], [[[8.0, 2.0, 8.0], [4.0, 3.0, 6.0], [6.0, 1.0, 2.0]], [[1.0, 
3.0, 4.0], [6.0, 4.0, 3.0], [3.0, 6.0, 3.0]], [[4.0, 1.0, 5.0], [6.0, 4.0, 4.0], [8.0, 2.0, 2.0]]], [[[1.0, 1.0, 2.0], [8.0, 
8.0, 8.0], [1.0, 2.0, 2.0]], [[3.0, 6.0, 6.0], [6.0, 4.0, 6.0], [7.0, 3.0, 4.0]], [[6.0, 8.0, 8.0], [5.0, 4.0, 5.0], [3.0, 6.0, 
5.0]]], [[[4.0, 3.0, 1.0], [5.0, 5.0, 7.0], [6.0, 3.0, 6.0]], [[3.0, 1.0, 8.0], [3.0, 7.0, 1.0], [5.0, 7.0, 3.0]], [[3.0, 1.0, 
7.0], [1.0, 2.0, 7.0], [4.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 6.0], [3.0, 7.0, 7.0], [3.0, 1.0, 6.0]], 
[[4.0, 5.0, 4.0], [3.0, 1.0, 7.0], [6.0, 7.0, 2.0]], [[4.0, 2.0, 2.0], [5.0, 3.0, 5.0], [3.0, 3.0, 7.0]]], [[[2.0, 1.0, 7.0], 
[7.0, 5.0, 5.0], [6.0, 2.0, 5.0]], [[1.0, 3.0, 4.0], [6.0, 4.0, 3.0], [3.0, 6.0, 3.0]], [[7.0, 5.0, 4.0], [8.0, 7.0, 5.0], 
[7.0, 5.0, 5.0]]], [[[3.0, 8.0, 3.0], [6.0, 6.0, 5.0], [3.0, 8.0, 4.0]], [[3.0, 6.0, 6.0], [6.0, 4.0, 6.0], [7.0, 3.0, 4.0]], 
[[6.0, 2.0, 2.0], [6.0, 6.0, 6.0], [4.0, 2.0, 5.0]]], [[[3.0, 5.0, 1.0], [5.0, 7.0, 1.0], [7.0, 1.0, 1.0]], [[3.0, 1.0, 8.0], 
[3.0, 7.0, 1.0], [5.0, 7.0, 3.0]], [[2.0, 4.0, 5.0], [4.0, 3.0, 6.0], [2.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CRCN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 4.0], [2.0, 3.0, 1.0], [8.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], [1.0, 2.0, 1.0], [5.0, 2.0, 
6.0]], [[7.0, 6.0, 2.0], [2.0, 5.0, 3.0], [3.0, 8.0, 1.0]]], [[[7.0, 1.0, 8.0], [5.0, 4.0, 4.0], [5.0, 7.0, 3.0]], [[8.0, 3.0, 
1.0], [8.0, 5.0, 8.0], [8.0, 6.0, 1.0]], [[1.0, 4.0, 2.0], [1.0, 3.0, 2.0], [4.0, 6.0, 1.0]]], [[[3.0, 1.0, 7.0], [2.0, 4.0, 
2.0], [4.0, 5.0, 2.0]], [[1.0, 5.0, 1.0], [1.0, 8.0, 2.0], [8.0, 6.0, 3.0]], [[5.0, 2.0, 4.0], [4.0, 3.0, 4.0], [8.0, 3.0, 
6.0]]], [[[8.0, 6.0, 2.0], [1.0, 8.0, 2.0], [5.0, 4.0, 6.0]], [[3.0, 8.0, 3.0], [4.0, 4.0, 8.0], [1.0, 2.0, 3.0]], [[6.0, 5.0, 
2.0], [7.0, 7.0, 4.0], [8.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 7.0], [5.0, 8.0, 5.0], [3.0, 7.0, 8.0]], [[5.0, 7.0, 6.0], [1.0, 5.0, 
4.0], [7.0, 8.0, 7.0]], [[3.0, 3.0, 6.0], [6.0, 1.0, 6.0], [2.0, 1.0, 3.0]]], [[[7.0, 8.0, 8.0], [4.0, 4.0, 2.0], [2.0, 5.0, 
4.0]], [[7.0, 5.0, 3.0], [5.0, 3.0, 7.0], [3.0, 8.0, 8.0]], [[2.0, 3.0, 5.0], [2.0, 6.0, 4.0], [1.0, 8.0, 6.0]]], [[[3.0, 2.0, 
4.0], [8.0, 2.0, 5.0], [6.0, 2.0, 4.0]], [[2.0, 1.0, 3.0], [7.0, 6.0, 4.0], [7.0, 7.0, 4.0]], [[1.0, 7.0, 6.0], [1.0, 5.0, 
1.0], [3.0, 4.0, 6.0]]], [[[8.0, 6.0, 3.0], [2.0, 7.0, 4.0], [5.0, 8.0, 2.0]], [[8.0, 7.0, 6.0], [6.0, 8.0, 4.0], [2.0, 1.0, 
7.0]], [[3.0, 3.0, 8.0], [8.0, 3.0, 5.0], [8.0, 7.0, 4.0]]]]))
      val=Data(numpy.array([[[[6.0, 8.0, 1.0], [6.0, 5.0, 5.0], [8.0, 1.0, 8.0]]], [[[3.0, 5.0, 3.0], [4.0, 1.0, 2.0], [2.0, 
1.0, 5.0]]], [[[4.0, 6.0, 1.0], [6.0, 4.0, 7.0], [5.0, 6.0, 5.0]]], [[[8.0, 7.0, 8.0], [7.0, 5.0, 8.0], [3.0, 8.0, 
8.0]]]]),self.functionspace)
      arg[:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 4.0], [2.0, 3.0, 1.0], [8.0, 2.0, 7.0]], [[6.0, 8.0, 1.0], [6.0, 5.0, 5.0], [8.0, 
1.0, 8.0]], [[7.0, 6.0, 2.0], [2.0, 5.0, 3.0], [3.0, 8.0, 1.0]]], [[[7.0, 1.0, 8.0], [5.0, 4.0, 4.0], [5.0, 7.0, 3.0]], [[3.0, 
5.0, 3.0], [4.0, 1.0, 2.0], [2.0, 1.0, 5.0]], [[1.0, 4.0, 2.0], [1.0, 3.0, 2.0], [4.0, 6.0, 1.0]]], [[[3.0, 1.0, 7.0], [2.0, 
4.0, 2.0], [4.0, 5.0, 2.0]], [[4.0, 6.0, 1.0], [6.0, 4.0, 7.0], [5.0, 6.0, 5.0]], [[5.0, 2.0, 4.0], [4.0, 3.0, 4.0], [8.0, 3.0, 
6.0]]], [[[8.0, 6.0, 2.0], [1.0, 8.0, 2.0], [5.0, 4.0, 6.0]], [[8.0, 7.0, 8.0], [7.0, 5.0, 8.0], [3.0, 8.0, 8.0]], [[6.0, 5.0, 
2.0], [7.0, 7.0, 4.0], [8.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 7.0], [5.0, 8.0, 5.0], [3.0, 7.0, 8.0]], 
[[6.0, 8.0, 1.0], [6.0, 5.0, 5.0], [8.0, 1.0, 8.0]], [[3.0, 3.0, 6.0], [6.0, 1.0, 6.0], [2.0, 1.0, 3.0]]], [[[7.0, 8.0, 8.0], 
[4.0, 4.0, 2.0], [2.0, 5.0, 4.0]], [[3.0, 5.0, 3.0], [4.0, 1.0, 2.0], [2.0, 1.0, 5.0]], [[2.0, 3.0, 5.0], [2.0, 6.0, 4.0], 
[1.0, 8.0, 6.0]]], [[[3.0, 2.0, 4.0], [8.0, 2.0, 5.0], [6.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], [6.0, 4.0, 7.0], [5.0, 6.0, 5.0]], 
[[1.0, 7.0, 6.0], [1.0, 5.0, 1.0], [3.0, 4.0, 6.0]]], [[[8.0, 6.0, 3.0], [2.0, 7.0, 4.0], [5.0, 8.0, 2.0]], [[8.0, 7.0, 8.0], 
[7.0, 5.0, 8.0], [3.0, 8.0, 8.0]], [[3.0, 3.0, 8.0], [8.0, 3.0, 5.0], [8.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CRCN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 1.0], [7.0, 7.0, 8.0], [2.0, 5.0, 7.0]], [[6.0, 7.0, 4.0], [6.0, 7.0, 5.0], [4.0, 3.0, 
2.0]], [[7.0, 3.0, 8.0], [6.0, 4.0, 4.0], [2.0, 2.0, 2.0]]], [[[8.0, 7.0, 2.0], [5.0, 4.0, 7.0], [1.0, 5.0, 5.0]], [[7.0, 4.0, 
4.0], [1.0, 6.0, 7.0], [3.0, 3.0, 3.0]], [[1.0, 4.0, 1.0], [7.0, 6.0, 7.0], [4.0, 7.0, 6.0]]], [[[7.0, 2.0, 7.0], [1.0, 2.0, 
6.0], [7.0, 5.0, 8.0]], [[6.0, 6.0, 5.0], [5.0, 1.0, 3.0], [4.0, 4.0, 5.0]], [[7.0, 5.0, 1.0], [6.0, 3.0, 6.0], [5.0, 1.0, 
5.0]]], [[[4.0, 3.0, 3.0], [6.0, 1.0, 3.0], [3.0, 6.0, 1.0]], [[1.0, 2.0, 1.0], [8.0, 3.0, 1.0], [4.0, 2.0, 4.0]], [[7.0, 2.0, 
5.0], [1.0, 7.0, 6.0], [2.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 5.0], [7.0, 2.0, 3.0], [6.0, 5.0, 8.0]], [[7.0, 4.0, 3.0], [7.0, 7.0, 
7.0], [1.0, 4.0, 7.0]], [[4.0, 1.0, 8.0], [1.0, 3.0, 8.0], [5.0, 7.0, 6.0]]], [[[6.0, 6.0, 2.0], [1.0, 2.0, 1.0], [5.0, 2.0, 
4.0]], [[1.0, 7.0, 8.0], [3.0, 6.0, 4.0], [5.0, 1.0, 8.0]], [[4.0, 3.0, 5.0], [5.0, 7.0, 6.0], [7.0, 7.0, 6.0]]], [[[3.0, 4.0, 
5.0], [2.0, 7.0, 4.0], [8.0, 7.0, 7.0]], [[1.0, 8.0, 5.0], [5.0, 5.0, 3.0], [3.0, 1.0, 5.0]], [[6.0, 4.0, 7.0], [4.0, 2.0, 
5.0], [8.0, 8.0, 6.0]]], [[[8.0, 7.0, 5.0], [7.0, 4.0, 5.0], [3.0, 7.0, 1.0]], [[3.0, 7.0, 2.0], [5.0, 5.0, 3.0], [5.0, 3.0, 
7.0]], [[8.0, 6.0, 3.0], [3.0, 6.0, 4.0], [3.0, 7.0, 7.0]]]]))
      val=Data(numpy.array([[[[6.0, 7.0, 6.0], [1.0, 3.0, 4.0], [8.0, 4.0, 2.0]]], [[[7.0, 5.0, 5.0], [2.0, 6.0, 7.0], [2.0, 
4.0, 8.0]]], [[[4.0, 2.0, 2.0], [4.0, 3.0, 8.0], [5.0, 1.0, 4.0]]], [[[4.0, 6.0, 3.0], [2.0, 2.0, 5.0], [3.0, 6.0, 
5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 6.0], [1.0, 6.0, 2.0], [6.0, 1.0, 8.0]]], [[[7.0, 4.0, 5.0], [2.0, 8.0, 
4.0], [6.0, 4.0, 4.0]]], [[[8.0, 4.0, 2.0], [2.0, 4.0, 3.0], [6.0, 6.0, 6.0]]], [[[8.0, 3.0, 5.0], [6.0, 7.0, 2.0], [2.0, 5.0, 
2.0]]]]))
      arg[:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 1.0], [7.0, 7.0, 8.0], [2.0, 5.0, 7.0]], [[6.0, 7.0, 6.0], [1.0, 3.0, 4.0], [8.0, 
4.0, 2.0]], [[7.0, 3.0, 8.0], [6.0, 4.0, 4.0], [2.0, 2.0, 2.0]]], [[[8.0, 7.0, 2.0], [5.0, 4.0, 7.0], [1.0, 5.0, 5.0]], [[7.0, 
5.0, 5.0], [2.0, 6.0, 7.0], [2.0, 4.0, 8.0]], [[1.0, 4.0, 1.0], [7.0, 6.0, 7.0], [4.0, 7.0, 6.0]]], [[[7.0, 2.0, 7.0], [1.0, 
2.0, 6.0], [7.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [4.0, 3.0, 8.0], [5.0, 1.0, 4.0]], [[7.0, 5.0, 1.0], [6.0, 3.0, 6.0], [5.0, 1.0, 
5.0]]], [[[4.0, 3.0, 3.0], [6.0, 1.0, 3.0], [3.0, 6.0, 1.0]], [[4.0, 6.0, 3.0], [2.0, 2.0, 5.0], [3.0, 6.0, 5.0]], [[7.0, 2.0, 
5.0], [1.0, 7.0, 6.0], [2.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 5.0], [7.0, 2.0, 3.0], [6.0, 5.0, 8.0]], 
[[3.0, 3.0, 6.0], [1.0, 6.0, 2.0], [6.0, 1.0, 8.0]], [[4.0, 1.0, 8.0], [1.0, 3.0, 8.0], [5.0, 7.0, 6.0]]], [[[6.0, 6.0, 2.0], 
[1.0, 2.0, 1.0], [5.0, 2.0, 4.0]], [[7.0, 4.0, 5.0], [2.0, 8.0, 4.0], [6.0, 4.0, 4.0]], [[4.0, 3.0, 5.0], [5.0, 7.0, 6.0], 
[7.0, 7.0, 6.0]]], [[[3.0, 4.0, 5.0], [2.0, 7.0, 4.0], [8.0, 7.0, 7.0]], [[8.0, 4.0, 2.0], [2.0, 4.0, 3.0], [6.0, 6.0, 6.0]], 
[[6.0, 4.0, 7.0], [4.0, 2.0, 5.0], [8.0, 8.0, 6.0]]], [[[8.0, 7.0, 5.0], [7.0, 4.0, 5.0], [3.0, 7.0, 1.0]], [[8.0, 3.0, 5.0], 
[6.0, 7.0, 2.0], [2.0, 5.0, 2.0]], [[8.0, 6.0, 3.0], [3.0, 6.0, 4.0], [3.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CRCN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 7.0], [5.0, 2.0, 2.0], [5.0, 3.0, 6.0]], [[5.0, 5.0, 8.0], [8.0, 7.0, 4.0], [6.0, 2.0, 
4.0]], [[6.0, 4.0, 5.0], [7.0, 8.0, 4.0], [3.0, 3.0, 8.0]]], [[[3.0, 7.0, 4.0], [8.0, 6.0, 3.0], [4.0, 1.0, 4.0]], [[2.0, 6.0, 
6.0], [6.0, 4.0, 7.0], [5.0, 5.0, 7.0]], [[8.0, 3.0, 5.0], [6.0, 1.0, 1.0], [3.0, 8.0, 4.0]]], [[[6.0, 8.0, 6.0], [6.0, 5.0, 
3.0], [6.0, 5.0, 6.0]], [[6.0, 3.0, 7.0], [2.0, 3.0, 5.0], [2.0, 6.0, 6.0]], [[3.0, 6.0, 6.0], [2.0, 8.0, 8.0], [8.0, 5.0, 
4.0]]], [[[2.0, 3.0, 6.0], [5.0, 1.0, 6.0], [4.0, 4.0, 6.0]], [[8.0, 8.0, 2.0], [7.0, 5.0, 5.0], [2.0, 1.0, 1.0]], [[1.0, 3.0, 
1.0], [6.0, 4.0, 5.0], [4.0, 8.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 2.0], [8.0, 1.0, 1.0], [8.0, 8.0, 7.0]], [[5.0, 7.0, 7.0], [5.0, 4.0, 
6.0], [1.0, 2.0, 3.0]], [[1.0, 1.0, 3.0], [2.0, 8.0, 7.0], [5.0, 4.0, 4.0]]], [[[8.0, 1.0, 4.0], [8.0, 4.0, 1.0], [8.0, 5.0, 
3.0]], [[1.0, 8.0, 6.0], [7.0, 3.0, 3.0], [5.0, 8.0, 1.0]], [[8.0, 2.0, 2.0], [8.0, 7.0, 1.0], [4.0, 7.0, 4.0]]], [[[4.0, 8.0, 
1.0], [8.0, 5.0, 5.0], [4.0, 4.0, 4.0]], [[7.0, 4.0, 4.0], [1.0, 6.0, 2.0], [8.0, 4.0, 2.0]], [[3.0, 7.0, 5.0], [3.0, 6.0, 
1.0], [1.0, 1.0, 8.0]]], [[[3.0, 8.0, 5.0], [1.0, 8.0, 7.0], [7.0, 6.0, 2.0]], [[2.0, 6.0, 5.0], [3.0, 3.0, 5.0], [6.0, 5.0, 
5.0]], [[1.0, 7.0, 4.0], [2.0, 4.0, 7.0], [3.0, 5.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 7.0, 4.0], [3.0, 6.0, 5.0], [3.0, 8.0, 6.0]]], [[[7.0, 1.0, 1.0], [5.0, 1.0, 8.0], [4.0, 
1.0, 4.0]]], [[[6.0, 4.0, 5.0], [6.0, 5.0, 6.0], [6.0, 5.0, 7.0]]], [[[2.0, 3.0, 5.0], [3.0, 1.0, 4.0], [6.0, 5.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 3.0, 4.0], [5.0, 3.0, 6.0], [6.0, 5.0, 4.0]]], [[[7.0, 6.0, 3.0], [4.0, 6.0, 6.0], 
[4.0, 4.0, 2.0]]], [[[8.0, 7.0, 2.0], [3.0, 7.0, 3.0], [3.0, 5.0, 8.0]]], [[[6.0, 6.0, 2.0], [6.0, 1.0, 5.0], [2.0, 6.0, 
2.0]]]])
      arg[:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 7.0], [5.0, 2.0, 2.0], [5.0, 3.0, 6.0]], [[2.0, 7.0, 4.0], [3.0, 6.0, 5.0], [3.0, 
8.0, 6.0]], [[6.0, 4.0, 5.0], [7.0, 8.0, 4.0], [3.0, 3.0, 8.0]]], [[[3.0, 7.0, 4.0], [8.0, 6.0, 3.0], [4.0, 1.0, 4.0]], [[7.0, 
1.0, 1.0], [5.0, 1.0, 8.0], [4.0, 1.0, 4.0]], [[8.0, 3.0, 5.0], [6.0, 1.0, 1.0], [3.0, 8.0, 4.0]]], [[[6.0, 8.0, 6.0], [6.0, 
5.0, 3.0], [6.0, 5.0, 6.0]], [[6.0, 4.0, 5.0], [6.0, 5.0, 6.0], [6.0, 5.0, 7.0]], [[3.0, 6.0, 6.0], [2.0, 8.0, 8.0], [8.0, 5.0, 
4.0]]], [[[2.0, 3.0, 6.0], [5.0, 1.0, 6.0], [4.0, 4.0, 6.0]], [[2.0, 3.0, 5.0], [3.0, 1.0, 4.0], [6.0, 5.0, 7.0]], [[1.0, 3.0, 
1.0], [6.0, 4.0, 5.0], [4.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 2.0], [8.0, 1.0, 1.0], [8.0, 8.0, 7.0]], 
[[3.0, 3.0, 4.0], [5.0, 3.0, 6.0], [6.0, 5.0, 4.0]], [[1.0, 1.0, 3.0], [2.0, 8.0, 7.0], [5.0, 4.0, 4.0]]], [[[8.0, 1.0, 4.0], 
[8.0, 4.0, 1.0], [8.0, 5.0, 3.0]], [[7.0, 6.0, 3.0], [4.0, 6.0, 6.0], [4.0, 4.0, 2.0]], [[8.0, 2.0, 2.0], [8.0, 7.0, 1.0], 
[4.0, 7.0, 4.0]]], [[[4.0, 8.0, 1.0], [8.0, 5.0, 5.0], [4.0, 4.0, 4.0]], [[8.0, 7.0, 2.0], [3.0, 7.0, 3.0], [3.0, 5.0, 8.0]], 
[[3.0, 7.0, 5.0], [3.0, 6.0, 1.0], [1.0, 1.0, 8.0]]], [[[3.0, 8.0, 5.0], [1.0, 8.0, 7.0], [7.0, 6.0, 2.0]], [[6.0, 6.0, 2.0], 
[6.0, 1.0, 5.0], [2.0, 6.0, 2.0]], [[1.0, 7.0, 4.0], [2.0, 4.0, 7.0], [3.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LULN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 4.0], [6.0, 4.0, 4.0], [1.0, 3.0, 3.0]], [[1.0, 7.0, 4.0], [5.0, 6.0, 7.0], [3.0, 2.0, 
7.0]], [[7.0, 2.0, 7.0], [7.0, 5.0, 4.0], [4.0, 5.0, 5.0]]], [[[2.0, 2.0, 8.0], [6.0, 4.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 7.0, 
3.0], [4.0, 4.0, 6.0], [6.0, 1.0, 5.0]], [[1.0, 6.0, 3.0], [7.0, 1.0, 7.0], [5.0, 5.0, 1.0]]], [[[7.0, 3.0, 3.0], [3.0, 1.0, 
1.0], [8.0, 3.0, 5.0]], [[4.0, 6.0, 6.0], [6.0, 1.0, 2.0], [1.0, 5.0, 6.0]], [[8.0, 4.0, 6.0], [3.0, 1.0, 4.0], [7.0, 4.0, 
5.0]]], [[[7.0, 2.0, 8.0], [2.0, 6.0, 2.0], [6.0, 8.0, 7.0]], [[3.0, 5.0, 5.0], [2.0, 5.0, 3.0], [2.0, 7.0, 3.0]], [[5.0, 5.0, 
3.0], [8.0, 2.0, 5.0], [5.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 6.0], [6.0, 1.0, 5.0], [1.0, 6.0, 1.0]], [[2.0, 2.0, 1.0], [6.0, 6.0, 
3.0], [6.0, 5.0, 1.0]], [[3.0, 8.0, 7.0], [8.0, 3.0, 8.0], [4.0, 5.0, 1.0]]], [[[4.0, 4.0, 3.0], [3.0, 8.0, 7.0], [3.0, 1.0, 
8.0]], [[6.0, 6.0, 4.0], [3.0, 4.0, 5.0], [8.0, 8.0, 3.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 2.0], [4.0, 5.0, 8.0]]], [[[2.0, 1.0, 
2.0], [5.0, 8.0, 7.0], [2.0, 8.0, 4.0]], [[4.0, 7.0, 1.0], [5.0, 8.0, 8.0], [6.0, 4.0, 2.0]], [[7.0, 2.0, 5.0], [3.0, 2.0, 
8.0], [4.0, 5.0, 1.0]]], [[[6.0, 5.0, 1.0], [5.0, 6.0, 4.0], [5.0, 6.0, 3.0]], [[8.0, 6.0, 6.0], [6.0, 1.0, 6.0], [5.0, 1.0, 
6.0]], [[8.0, 4.0, 6.0], [1.0, 8.0, 4.0], [8.0, 8.0, 8.0]]]]))
      val=numpy.array([[[[7.0, 1.0, 1.0]], [[5.0, 4.0, 8.0]]], [[[5.0, 2.0, 2.0]], [[8.0, 4.0, 5.0]]]])
      arg[2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 4.0], [6.0, 4.0, 4.0], [1.0, 3.0, 3.0]], [[1.0, 7.0, 4.0], [5.0, 6.0, 7.0], [3.0, 
2.0, 7.0]], [[7.0, 2.0, 7.0], [7.0, 5.0, 4.0], [4.0, 5.0, 5.0]]], [[[2.0, 2.0, 8.0], [6.0, 4.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 
7.0, 3.0], [4.0, 4.0, 6.0], [6.0, 1.0, 5.0]], [[1.0, 6.0, 3.0], [7.0, 1.0, 7.0], [5.0, 5.0, 1.0]]], [[[7.0, 3.0, 3.0], [3.0, 
1.0, 1.0], [7.0, 1.0, 1.0]], [[4.0, 6.0, 6.0], [6.0, 1.0, 2.0], [5.0, 4.0, 8.0]], [[8.0, 4.0, 6.0], [3.0, 1.0, 4.0], [7.0, 4.0, 
5.0]]], [[[7.0, 2.0, 8.0], [2.0, 6.0, 2.0], [5.0, 2.0, 2.0]], [[3.0, 5.0, 5.0], [2.0, 5.0, 3.0], [8.0, 4.0, 5.0]], [[5.0, 5.0, 
3.0], [8.0, 2.0, 5.0], [5.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 6.0], [6.0, 1.0, 5.0], [1.0, 6.0, 1.0]], 
[[2.0, 2.0, 1.0], [6.0, 6.0, 3.0], [6.0, 5.0, 1.0]], [[3.0, 8.0, 7.0], [8.0, 3.0, 8.0], [4.0, 5.0, 1.0]]], [[[4.0, 4.0, 3.0], 
[3.0, 8.0, 7.0], [3.0, 1.0, 8.0]], [[6.0, 6.0, 4.0], [3.0, 4.0, 5.0], [8.0, 8.0, 3.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 2.0], 
[4.0, 5.0, 8.0]]], [[[2.0, 1.0, 2.0], [5.0, 8.0, 7.0], [7.0, 1.0, 1.0]], [[4.0, 7.0, 1.0], [5.0, 8.0, 8.0], [5.0, 4.0, 8.0]], 
[[7.0, 2.0, 5.0], [3.0, 2.0, 8.0], [4.0, 5.0, 1.0]]], [[[6.0, 5.0, 1.0], [5.0, 6.0, 4.0], [5.0, 2.0, 2.0]], [[8.0, 6.0, 6.0], 
[6.0, 1.0, 6.0], [8.0, 4.0, 5.0]], [[8.0, 4.0, 6.0], [1.0, 8.0, 4.0], [8.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LULN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 8.0], [4.0, 4.0, 8.0], [6.0, 1.0, 8.0]], [[8.0, 6.0, 7.0], [1.0, 4.0, 2.0], [1.0, 6.0, 
6.0]], [[5.0, 3.0, 2.0], [4.0, 3.0, 7.0], [3.0, 6.0, 8.0]]], [[[7.0, 7.0, 5.0], [8.0, 5.0, 2.0], [2.0, 1.0, 1.0]], [[5.0, 5.0, 
7.0], [1.0, 2.0, 4.0], [7.0, 3.0, 3.0]], [[5.0, 6.0, 4.0], [6.0, 1.0, 3.0], [2.0, 6.0, 1.0]]], [[[8.0, 8.0, 7.0], [2.0, 3.0, 
5.0], [3.0, 7.0, 5.0]], [[8.0, 6.0, 8.0], [1.0, 7.0, 3.0], [5.0, 3.0, 3.0]], [[5.0, 8.0, 4.0], [1.0, 3.0, 2.0], [3.0, 3.0, 
8.0]]], [[[4.0, 3.0, 1.0], [1.0, 3.0, 3.0], [8.0, 7.0, 7.0]], [[5.0, 5.0, 7.0], [6.0, 1.0, 1.0], [8.0, 5.0, 3.0]], [[4.0, 5.0, 
6.0], [8.0, 6.0, 1.0], [5.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 2.0, 4.0], [4.0, 6.0, 5.0], [4.0, 1.0, 2.0]], [[1.0, 2.0, 6.0], [5.0, 4.0, 
8.0], [2.0, 1.0, 7.0]], [[6.0, 2.0, 3.0], [5.0, 3.0, 7.0], [5.0, 6.0, 4.0]]], [[[1.0, 2.0, 5.0], [8.0, 2.0, 4.0], [8.0, 4.0, 
6.0]], [[8.0, 6.0, 2.0], [5.0, 6.0, 6.0], [1.0, 8.0, 6.0]], [[5.0, 4.0, 1.0], [2.0, 4.0, 5.0], [8.0, 3.0, 2.0]]], [[[5.0, 5.0, 
2.0], [3.0, 7.0, 5.0], [3.0, 6.0, 8.0]], [[1.0, 8.0, 1.0], [1.0, 1.0, 7.0], [1.0, 7.0, 7.0]], [[6.0, 8.0, 5.0], [5.0, 7.0, 
1.0], [6.0, 4.0, 4.0]]], [[[1.0, 8.0, 1.0], [6.0, 5.0, 6.0], [5.0, 8.0, 1.0]], [[4.0, 5.0, 1.0], [4.0, 6.0, 4.0], [3.0, 7.0, 
2.0]], [[6.0, 2.0, 8.0], [8.0, 3.0, 7.0], [1.0, 7.0, 1.0]]]]))
      val=Data(numpy.array([[[[4.0, 2.0, 5.0]], [[2.0, 4.0, 1.0]]], [[[3.0, 4.0, 4.0]], [[6.0, 3.0, 8.0]]]]),self.functionspace)
      arg[2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [4.0, 4.0, 8.0], [6.0, 1.0, 8.0]], [[8.0, 6.0, 7.0], [1.0, 4.0, 2.0], [1.0, 
6.0, 6.0]], [[5.0, 3.0, 2.0], [4.0, 3.0, 7.0], [3.0, 6.0, 8.0]]], [[[7.0, 7.0, 5.0], [8.0, 5.0, 2.0], [2.0, 1.0, 1.0]], [[5.0, 
5.0, 7.0], [1.0, 2.0, 4.0], [7.0, 3.0, 3.0]], [[5.0, 6.0, 4.0], [6.0, 1.0, 3.0], [2.0, 6.0, 1.0]]], [[[8.0, 8.0, 7.0], [2.0, 
3.0, 5.0], [4.0, 2.0, 5.0]], [[8.0, 6.0, 8.0], [1.0, 7.0, 3.0], [2.0, 4.0, 1.0]], [[5.0, 8.0, 4.0], [1.0, 3.0, 2.0], [3.0, 3.0, 
8.0]]], [[[4.0, 3.0, 1.0], [1.0, 3.0, 3.0], [3.0, 4.0, 4.0]], [[5.0, 5.0, 7.0], [6.0, 1.0, 1.0], [6.0, 3.0, 8.0]], [[4.0, 5.0, 
6.0], [8.0, 6.0, 1.0], [5.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 4.0], [4.0, 6.0, 5.0], [4.0, 1.0, 2.0]], 
[[1.0, 2.0, 6.0], [5.0, 4.0, 8.0], [2.0, 1.0, 7.0]], [[6.0, 2.0, 3.0], [5.0, 3.0, 7.0], [5.0, 6.0, 4.0]]], [[[1.0, 2.0, 5.0], 
[8.0, 2.0, 4.0], [8.0, 4.0, 6.0]], [[8.0, 6.0, 2.0], [5.0, 6.0, 6.0], [1.0, 8.0, 6.0]], [[5.0, 4.0, 1.0], [2.0, 4.0, 5.0], 
[8.0, 3.0, 2.0]]], [[[5.0, 5.0, 2.0], [3.0, 7.0, 5.0], [4.0, 2.0, 5.0]], [[1.0, 8.0, 1.0], [1.0, 1.0, 7.0], [2.0, 4.0, 1.0]], 
[[6.0, 8.0, 5.0], [5.0, 7.0, 1.0], [6.0, 4.0, 4.0]]], [[[1.0, 8.0, 1.0], [6.0, 5.0, 6.0], [3.0, 4.0, 4.0]], [[4.0, 5.0, 1.0], 
[4.0, 6.0, 4.0], [6.0, 3.0, 8.0]], [[6.0, 2.0, 8.0], [8.0, 3.0, 7.0], [1.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LULN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 6.0], [6.0, 5.0, 4.0], [6.0, 2.0, 6.0]], [[6.0, 5.0, 8.0], [7.0, 7.0, 8.0], [8.0, 3.0, 
1.0]], [[8.0, 2.0, 6.0], [5.0, 5.0, 8.0], [6.0, 7.0, 1.0]]], [[[2.0, 1.0, 2.0], [5.0, 7.0, 4.0], [7.0, 2.0, 7.0]], [[3.0, 1.0, 
1.0], [1.0, 8.0, 5.0], [5.0, 1.0, 6.0]], [[4.0, 5.0, 1.0], [3.0, 1.0, 6.0], [1.0, 4.0, 1.0]]], [[[7.0, 7.0, 5.0], [4.0, 3.0, 
8.0], [3.0, 8.0, 2.0]], [[5.0, 2.0, 6.0], [6.0, 1.0, 4.0], [3.0, 4.0, 2.0]], [[7.0, 4.0, 5.0], [6.0, 3.0, 7.0], [4.0, 1.0, 
7.0]]], [[[3.0, 8.0, 1.0], [3.0, 7.0, 4.0], [6.0, 2.0, 6.0]], [[6.0, 6.0, 7.0], [2.0, 2.0, 2.0], [2.0, 4.0, 2.0]], [[3.0, 5.0, 
7.0], [7.0, 7.0, 3.0], [8.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 5.0], [1.0, 7.0, 2.0], [2.0, 7.0, 5.0]], [[5.0, 8.0, 7.0], [8.0, 4.0, 
7.0], [1.0, 1.0, 3.0]], [[1.0, 7.0, 1.0], [7.0, 1.0, 3.0], [5.0, 2.0, 4.0]]], [[[3.0, 1.0, 2.0], [1.0, 6.0, 6.0], [2.0, 6.0, 
2.0]], [[2.0, 7.0, 6.0], [1.0, 2.0, 2.0], [8.0, 5.0, 3.0]], [[3.0, 2.0, 2.0], [6.0, 7.0, 4.0], [5.0, 5.0, 8.0]]], [[[5.0, 1.0, 
6.0], [6.0, 2.0, 8.0], [5.0, 2.0, 1.0]], [[7.0, 6.0, 8.0], [6.0, 1.0, 2.0], [8.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [4.0, 5.0, 
8.0], [5.0, 1.0, 4.0]]], [[[3.0, 8.0, 1.0], [8.0, 2.0, 8.0], [8.0, 8.0, 2.0]], [[7.0, 7.0, 8.0], [4.0, 2.0, 4.0], [7.0, 6.0, 
4.0]], [[5.0, 8.0, 6.0], [2.0, 8.0, 7.0], [4.0, 7.0, 4.0]]]]))
      val=Data(numpy.array([[[[4.0, 5.0, 5.0]], [[8.0, 7.0, 2.0]]], [[[5.0, 8.0, 3.0]], [[4.0, 2.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 1.0]], [[1.0, 2.0, 5.0]]], [[[2.0, 5.0, 3.0]], [[2.0, 3.0, 2.0]]]]))
      arg[2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 6.0], [6.0, 5.0, 4.0], [6.0, 2.0, 6.0]], [[6.0, 5.0, 8.0], [7.0, 7.0, 8.0], [8.0, 
3.0, 1.0]], [[8.0, 2.0, 6.0], [5.0, 5.0, 8.0], [6.0, 7.0, 1.0]]], [[[2.0, 1.0, 2.0], [5.0, 7.0, 4.0], [7.0, 2.0, 7.0]], [[3.0, 
1.0, 1.0], [1.0, 8.0, 5.0], [5.0, 1.0, 6.0]], [[4.0, 5.0, 1.0], [3.0, 1.0, 6.0], [1.0, 4.0, 1.0]]], [[[7.0, 7.0, 5.0], [4.0, 
3.0, 8.0], [4.0, 5.0, 5.0]], [[5.0, 2.0, 6.0], [6.0, 1.0, 4.0], [8.0, 7.0, 2.0]], [[7.0, 4.0, 5.0], [6.0, 3.0, 7.0], [4.0, 1.0, 
7.0]]], [[[3.0, 8.0, 1.0], [3.0, 7.0, 4.0], [5.0, 8.0, 3.0]], [[6.0, 6.0, 7.0], [2.0, 2.0, 2.0], [4.0, 2.0, 6.0]], [[3.0, 5.0, 
7.0], [7.0, 7.0, 3.0], [8.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 5.0], [1.0, 7.0, 2.0], [2.0, 7.0, 5.0]], 
[[5.0, 8.0, 7.0], [8.0, 4.0, 7.0], [1.0, 1.0, 3.0]], [[1.0, 7.0, 1.0], [7.0, 1.0, 3.0], [5.0, 2.0, 4.0]]], [[[3.0, 1.0, 2.0], 
[1.0, 6.0, 6.0], [2.0, 6.0, 2.0]], [[2.0, 7.0, 6.0], [1.0, 2.0, 2.0], [8.0, 5.0, 3.0]], [[3.0, 2.0, 2.0], [6.0, 7.0, 4.0], 
[5.0, 5.0, 8.0]]], [[[5.0, 1.0, 6.0], [6.0, 2.0, 8.0], [1.0, 8.0, 1.0]], [[7.0, 6.0, 8.0], [6.0, 1.0, 2.0], [1.0, 2.0, 5.0]], 
[[5.0, 6.0, 5.0], [4.0, 5.0, 8.0], [5.0, 1.0, 4.0]]], [[[3.0, 8.0, 1.0], [8.0, 2.0, 8.0], [2.0, 5.0, 3.0]], [[7.0, 7.0, 8.0], 
[4.0, 2.0, 4.0], [2.0, 3.0, 2.0]], [[5.0, 8.0, 6.0], [2.0, 8.0, 7.0], [4.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LULN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 4.0], [3.0, 7.0, 3.0], [2.0, 3.0, 7.0]], [[6.0, 5.0, 5.0], [8.0, 8.0, 4.0], [5.0, 3.0, 
6.0]], [[2.0, 3.0, 7.0], [4.0, 8.0, 4.0], [8.0, 4.0, 5.0]]], [[[4.0, 6.0, 2.0], [4.0, 4.0, 7.0], [7.0, 7.0, 6.0]], [[6.0, 8.0, 
5.0], [5.0, 8.0, 5.0], [1.0, 7.0, 5.0]], [[2.0, 3.0, 8.0], [4.0, 8.0, 3.0], [5.0, 5.0, 7.0]]], [[[2.0, 4.0, 3.0], [6.0, 3.0, 
3.0], [4.0, 3.0, 1.0]], [[1.0, 3.0, 1.0], [8.0, 7.0, 6.0], [4.0, 3.0, 3.0]], [[2.0, 1.0, 5.0], [3.0, 8.0, 1.0], [1.0, 3.0, 
3.0]]], [[[6.0, 3.0, 3.0], [3.0, 8.0, 5.0], [2.0, 1.0, 8.0]], [[5.0, 3.0, 3.0], [2.0, 8.0, 5.0], [5.0, 3.0, 4.0]], [[7.0, 3.0, 
7.0], [6.0, 2.0, 3.0], [5.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 4.0], [3.0, 2.0, 7.0], [5.0, 7.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 1.0, 
8.0], [6.0, 5.0, 3.0]], [[8.0, 7.0, 3.0], [3.0, 6.0, 7.0], [6.0, 1.0, 4.0]]], [[[4.0, 5.0, 5.0], [2.0, 3.0, 6.0], [2.0, 3.0, 
2.0]], [[4.0, 5.0, 1.0], [7.0, 8.0, 6.0], [8.0, 6.0, 8.0]], [[3.0, 8.0, 6.0], [1.0, 3.0, 1.0], [5.0, 4.0, 1.0]]], [[[4.0, 6.0, 
5.0], [4.0, 3.0, 8.0], [5.0, 8.0, 1.0]], [[2.0, 6.0, 6.0], [4.0, 3.0, 4.0], [4.0, 6.0, 3.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 
2.0], [3.0, 2.0, 6.0]]], [[[6.0, 8.0, 1.0], [6.0, 4.0, 7.0], [7.0, 1.0, 8.0]], [[5.0, 6.0, 5.0], [8.0, 2.0, 4.0], [6.0, 3.0, 
3.0]], [[3.0, 4.0, 1.0], [7.0, 4.0, 6.0], [5.0, 4.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0, 8.0, 1.0]], [[1.0, 5.0, 1.0]]], [[[2.0, 2.0, 7.0]], [[8.0, 5.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 1.0, 4.0]], [[4.0, 1.0, 1.0]]], [[[6.0, 7.0, 7.0]], [[2.0, 7.0, 2.0]]]])
      arg[2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 4.0], [3.0, 7.0, 3.0], [2.0, 3.0, 7.0]], [[6.0, 5.0, 5.0], [8.0, 8.0, 4.0], [5.0, 
3.0, 6.0]], [[2.0, 3.0, 7.0], [4.0, 8.0, 4.0], [8.0, 4.0, 5.0]]], [[[4.0, 6.0, 2.0], [4.0, 4.0, 7.0], [7.0, 7.0, 6.0]], [[6.0, 
8.0, 5.0], [5.0, 8.0, 5.0], [1.0, 7.0, 5.0]], [[2.0, 3.0, 8.0], [4.0, 8.0, 3.0], [5.0, 5.0, 7.0]]], [[[2.0, 4.0, 3.0], [6.0, 
3.0, 3.0], [6.0, 8.0, 1.0]], [[1.0, 3.0, 1.0], [8.0, 7.0, 6.0], [1.0, 5.0, 1.0]], [[2.0, 1.0, 5.0], [3.0, 8.0, 1.0], [1.0, 3.0, 
3.0]]], [[[6.0, 3.0, 3.0], [3.0, 8.0, 5.0], [2.0, 2.0, 7.0]], [[5.0, 3.0, 3.0], [2.0, 8.0, 5.0], [8.0, 5.0, 6.0]], [[7.0, 3.0, 
7.0], [6.0, 2.0, 3.0], [5.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 4.0], [3.0, 2.0, 7.0], [5.0, 7.0, 3.0]], 
[[4.0, 6.0, 3.0], [2.0, 1.0, 8.0], [6.0, 5.0, 3.0]], [[8.0, 7.0, 3.0], [3.0, 6.0, 7.0], [6.0, 1.0, 4.0]]], [[[4.0, 5.0, 5.0], 
[2.0, 3.0, 6.0], [2.0, 3.0, 2.0]], [[4.0, 5.0, 1.0], [7.0, 8.0, 6.0], [8.0, 6.0, 8.0]], [[3.0, 8.0, 6.0], [1.0, 3.0, 1.0], 
[5.0, 4.0, 1.0]]], [[[4.0, 6.0, 5.0], [4.0, 3.0, 8.0], [1.0, 1.0, 4.0]], [[2.0, 6.0, 6.0], [4.0, 3.0, 4.0], [4.0, 1.0, 1.0]], 
[[3.0, 4.0, 3.0], [4.0, 4.0, 2.0], [3.0, 2.0, 6.0]]], [[[6.0, 8.0, 1.0], [6.0, 4.0, 7.0], [6.0, 7.0, 7.0]], [[5.0, 6.0, 5.0], 
[8.0, 2.0, 4.0], [2.0, 7.0, 2.0]], [[3.0, 4.0, 1.0], [7.0, 4.0, 6.0], [5.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UUUN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 5.0], [3.0, 4.0, 1.0], [1.0, 8.0, 4.0]], [[1.0, 7.0, 6.0], [1.0, 8.0, 6.0], [2.0, 1.0, 
3.0]], [[1.0, 3.0, 8.0], [4.0, 1.0, 6.0], [1.0, 7.0, 4.0]]], [[[3.0, 4.0, 5.0], [4.0, 5.0, 6.0], [1.0, 8.0, 6.0]], [[8.0, 3.0, 
3.0], [1.0, 6.0, 8.0], [4.0, 1.0, 5.0]], [[6.0, 3.0, 8.0], [5.0, 6.0, 5.0], [5.0, 4.0, 1.0]]], [[[5.0, 4.0, 4.0], [4.0, 8.0, 
1.0], [2.0, 1.0, 1.0]], [[3.0, 7.0, 5.0], [3.0, 4.0, 4.0], [8.0, 6.0, 2.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 8.0], [6.0, 5.0, 
4.0]]], [[[7.0, 8.0, 7.0], [8.0, 8.0, 1.0], [3.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], [2.0, 6.0, 4.0], [6.0, 4.0, 8.0]], [[3.0, 7.0, 
6.0], [7.0, 4.0, 2.0], [3.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 2.0], [3.0, 7.0, 4.0], [5.0, 3.0, 2.0]], [[8.0, 6.0, 1.0], [5.0, 7.0, 
4.0], [1.0, 8.0, 6.0]], [[5.0, 2.0, 2.0], [6.0, 1.0, 5.0], [6.0, 8.0, 2.0]]], [[[8.0, 8.0, 1.0], [6.0, 2.0, 7.0], [2.0, 7.0, 
3.0]], [[4.0, 7.0, 2.0], [6.0, 2.0, 2.0], [2.0, 2.0, 8.0]], [[4.0, 4.0, 5.0], [6.0, 3.0, 7.0], [4.0, 6.0, 1.0]]], [[[1.0, 7.0, 
8.0], [4.0, 8.0, 2.0], [4.0, 6.0, 5.0]], [[2.0, 5.0, 6.0], [2.0, 2.0, 3.0], [1.0, 3.0, 3.0]], [[4.0, 5.0, 3.0], [3.0, 8.0, 
6.0], [4.0, 2.0, 8.0]]], [[[1.0, 8.0, 1.0], [1.0, 2.0, 7.0], [2.0, 8.0, 1.0]], [[6.0, 3.0, 6.0], [3.0, 5.0, 7.0], [2.0, 5.0, 
2.0]], [[2.0, 7.0, 1.0], [6.0, 1.0, 1.0], [6.0, 4.0, 6.0]]]]))
      val=numpy.array([[[[1.0, 6.0, 3.0], [1.0, 6.0, 6.0]], [[1.0, 8.0, 5.0], [5.0, 1.0, 8.0]]], [[[8.0, 3.0, 5.0], [2.0, 5.0, 
6.0]], [[4.0, 3.0, 3.0], [4.0, 2.0, 2.0]]], [[[5.0, 5.0, 4.0], [7.0, 3.0, 6.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 6.0]]]])
      arg[:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 3.0], [1.0, 6.0, 6.0], [1.0, 8.0, 4.0]], [[1.0, 8.0, 5.0], [5.0, 1.0, 8.0], [2.0, 
1.0, 3.0]], [[1.0, 3.0, 8.0], [4.0, 1.0, 6.0], [1.0, 7.0, 4.0]]], [[[8.0, 3.0, 5.0], [2.0, 5.0, 6.0], [1.0, 8.0, 6.0]], [[4.0, 
3.0, 3.0], [4.0, 2.0, 2.0], [4.0, 1.0, 5.0]], [[6.0, 3.0, 8.0], [5.0, 6.0, 5.0], [5.0, 4.0, 1.0]]], [[[5.0, 5.0, 4.0], [7.0, 
3.0, 6.0], [2.0, 1.0, 1.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 6.0], [8.0, 6.0, 2.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 8.0], [6.0, 5.0, 
4.0]]], [[[7.0, 8.0, 7.0], [8.0, 8.0, 1.0], [3.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], [2.0, 6.0, 4.0], [6.0, 4.0, 8.0]], [[3.0, 7.0, 
6.0], [7.0, 4.0, 2.0], [3.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 3.0], [1.0, 6.0, 6.0], [5.0, 3.0, 2.0]], 
[[1.0, 8.0, 5.0], [5.0, 1.0, 8.0], [1.0, 8.0, 6.0]], [[5.0, 2.0, 2.0], [6.0, 1.0, 5.0], [6.0, 8.0, 2.0]]], [[[8.0, 3.0, 5.0], 
[2.0, 5.0, 6.0], [2.0, 7.0, 3.0]], [[4.0, 3.0, 3.0], [4.0, 2.0, 2.0], [2.0, 2.0, 8.0]], [[4.0, 4.0, 5.0], [6.0, 3.0, 7.0], 
[4.0, 6.0, 1.0]]], [[[5.0, 5.0, 4.0], [7.0, 3.0, 6.0], [4.0, 6.0, 5.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 6.0], [1.0, 3.0, 3.0]], 
[[4.0, 5.0, 3.0], [3.0, 8.0, 6.0], [4.0, 2.0, 8.0]]], [[[1.0, 8.0, 1.0], [1.0, 2.0, 7.0], [2.0, 8.0, 1.0]], [[6.0, 3.0, 6.0], 
[3.0, 5.0, 7.0], [2.0, 5.0, 2.0]], [[2.0, 7.0, 1.0], [6.0, 1.0, 1.0], [6.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UUUN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 6.0], [6.0, 6.0, 5.0], [1.0, 5.0, 3.0]], [[7.0, 4.0, 4.0], [7.0, 1.0, 2.0], [4.0, 2.0, 
1.0]], [[7.0, 7.0, 6.0], [7.0, 5.0, 4.0], [8.0, 8.0, 1.0]]], [[[2.0, 4.0, 6.0], [8.0, 4.0, 7.0], [6.0, 5.0, 8.0]], [[7.0, 3.0, 
3.0], [6.0, 5.0, 5.0], [1.0, 5.0, 2.0]], [[8.0, 8.0, 2.0], [4.0, 4.0, 8.0], [8.0, 6.0, 1.0]]], [[[6.0, 3.0, 6.0], [7.0, 1.0, 
2.0], [2.0, 2.0, 8.0]], [[2.0, 7.0, 3.0], [5.0, 3.0, 7.0], [1.0, 4.0, 7.0]], [[6.0, 8.0, 5.0], [3.0, 3.0, 5.0], [1.0, 5.0, 
2.0]]], [[[8.0, 3.0, 5.0], [4.0, 8.0, 1.0], [2.0, 7.0, 1.0]], [[5.0, 7.0, 4.0], [8.0, 1.0, 4.0], [5.0, 7.0, 5.0]], [[2.0, 2.0, 
7.0], [7.0, 8.0, 1.0], [2.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 6.0], [6.0, 4.0, 7.0], [4.0, 4.0, 7.0]], [[2.0, 3.0, 2.0], [5.0, 6.0, 
7.0], [8.0, 6.0, 8.0]], [[5.0, 4.0, 6.0], [1.0, 5.0, 6.0], [8.0, 8.0, 6.0]]], [[[3.0, 6.0, 4.0], [2.0, 1.0, 5.0], [4.0, 3.0, 
7.0]], [[6.0, 7.0, 7.0], [2.0, 3.0, 6.0], [3.0, 5.0, 3.0]], [[6.0, 4.0, 6.0], [7.0, 4.0, 5.0], [2.0, 5.0, 1.0]]], [[[4.0, 2.0, 
5.0], [4.0, 5.0, 4.0], [7.0, 1.0, 8.0]], [[2.0, 4.0, 1.0], [6.0, 1.0, 4.0], [1.0, 2.0, 3.0]], [[4.0, 8.0, 1.0], [7.0, 5.0, 
1.0], [4.0, 7.0, 2.0]]], [[[8.0, 6.0, 6.0], [4.0, 6.0, 5.0], [6.0, 7.0, 1.0]], [[3.0, 4.0, 1.0], [8.0, 5.0, 4.0], [3.0, 5.0, 
7.0]], [[8.0, 6.0, 4.0], [8.0, 4.0, 6.0], [2.0, 2.0, 4.0]]]]))
      val=Data(numpy.array([[[[1.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[1.0, 8.0, 4.0], [7.0, 1.0, 6.0]]], [[[1.0, 4.0, 6.0], [3.0, 
2.0, 4.0]], [[3.0, 4.0, 7.0], [1.0, 5.0, 7.0]]], [[[3.0, 3.0, 8.0], [8.0, 8.0, 8.0]], [[6.0, 2.0, 6.0], [1.0, 7.0, 
5.0]]]]),self.functionspace)
      arg[:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 2.0], [6.0, 4.0, 5.0], [1.0, 5.0, 3.0]], [[1.0, 8.0, 4.0], [7.0, 1.0, 6.0], [4.0, 
2.0, 1.0]], [[7.0, 7.0, 6.0], [7.0, 5.0, 4.0], [8.0, 8.0, 1.0]]], [[[1.0, 4.0, 6.0], [3.0, 2.0, 4.0], [6.0, 5.0, 8.0]], [[3.0, 
4.0, 7.0], [1.0, 5.0, 7.0], [1.0, 5.0, 2.0]], [[8.0, 8.0, 2.0], [4.0, 4.0, 8.0], [8.0, 6.0, 1.0]]], [[[3.0, 3.0, 8.0], [8.0, 
8.0, 8.0], [2.0, 2.0, 8.0]], [[6.0, 2.0, 6.0], [1.0, 7.0, 5.0], [1.0, 4.0, 7.0]], [[6.0, 8.0, 5.0], [3.0, 3.0, 5.0], [1.0, 5.0, 
2.0]]], [[[8.0, 3.0, 5.0], [4.0, 8.0, 1.0], [2.0, 7.0, 1.0]], [[5.0, 7.0, 4.0], [8.0, 1.0, 4.0], [5.0, 7.0, 5.0]], [[2.0, 2.0, 
7.0], [7.0, 8.0, 1.0], [2.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 2.0], [6.0, 4.0, 5.0], [4.0, 4.0, 7.0]], 
[[1.0, 8.0, 4.0], [7.0, 1.0, 6.0], [8.0, 6.0, 8.0]], [[5.0, 4.0, 6.0], [1.0, 5.0, 6.0], [8.0, 8.0, 6.0]]], [[[1.0, 4.0, 6.0], 
[3.0, 2.0, 4.0], [4.0, 3.0, 7.0]], [[3.0, 4.0, 7.0], [1.0, 5.0, 7.0], [3.0, 5.0, 3.0]], [[6.0, 4.0, 6.0], [7.0, 4.0, 5.0], 
[2.0, 5.0, 1.0]]], [[[3.0, 3.0, 8.0], [8.0, 8.0, 8.0], [7.0, 1.0, 8.0]], [[6.0, 2.0, 6.0], [1.0, 7.0, 5.0], [1.0, 2.0, 3.0]], 
[[4.0, 8.0, 1.0], [7.0, 5.0, 1.0], [4.0, 7.0, 2.0]]], [[[8.0, 6.0, 6.0], [4.0, 6.0, 5.0], [6.0, 7.0, 1.0]], [[3.0, 4.0, 1.0], 
[8.0, 5.0, 4.0], [3.0, 5.0, 7.0]], [[8.0, 6.0, 4.0], [8.0, 4.0, 6.0], [2.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UUUN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 2.0], [1.0, 2.0, 1.0], [2.0, 5.0, 6.0]], [[6.0, 6.0, 5.0], [5.0, 6.0, 8.0], [5.0, 2.0, 
7.0]], [[5.0, 1.0, 6.0], [7.0, 3.0, 1.0], [3.0, 3.0, 7.0]]], [[[5.0, 7.0, 3.0], [2.0, 6.0, 6.0], [1.0, 3.0, 3.0]], [[1.0, 3.0, 
1.0], [2.0, 1.0, 6.0], [7.0, 4.0, 4.0]], [[5.0, 3.0, 7.0], [8.0, 7.0, 4.0], [6.0, 1.0, 7.0]]], [[[4.0, 5.0, 1.0], [5.0, 2.0, 
7.0], [7.0, 2.0, 8.0]], [[7.0, 3.0, 6.0], [8.0, 8.0, 7.0], [2.0, 5.0, 6.0]], [[3.0, 8.0, 2.0], [3.0, 1.0, 3.0], [5.0, 6.0, 
6.0]]], [[[1.0, 1.0, 4.0], [8.0, 7.0, 1.0], [2.0, 4.0, 1.0]], [[4.0, 5.0, 7.0], [5.0, 4.0, 7.0], [7.0, 3.0, 4.0]], [[1.0, 2.0, 
5.0], [6.0, 5.0, 4.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 2.0], [2.0, 4.0, 4.0], [2.0, 5.0, 8.0]], [[6.0, 8.0, 6.0], [6.0, 6.0, 
7.0], [1.0, 4.0, 2.0]], [[1.0, 3.0, 6.0], [3.0, 7.0, 7.0], [4.0, 6.0, 8.0]]], [[[2.0, 1.0, 4.0], [5.0, 2.0, 1.0], [4.0, 3.0, 
4.0]], [[4.0, 7.0, 3.0], [4.0, 4.0, 7.0], [3.0, 8.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 8.0, 4.0], [1.0, 2.0, 5.0]]], [[[2.0, 1.0, 
7.0], [6.0, 8.0, 4.0], [2.0, 4.0, 4.0]], [[3.0, 6.0, 3.0], [6.0, 2.0, 8.0], [2.0, 8.0, 5.0]], [[1.0, 2.0, 6.0], [6.0, 3.0, 
2.0], [1.0, 2.0, 7.0]]], [[[1.0, 8.0, 7.0], [2.0, 2.0, 4.0], [6.0, 5.0, 3.0]], [[1.0, 1.0, 7.0], [6.0, 3.0, 1.0], [1.0, 1.0, 
4.0]], [[3.0, 6.0, 8.0], [3.0, 3.0, 6.0], [1.0, 6.0, 3.0]]]]))
      val=Data(numpy.array([[[[3.0, 7.0, 5.0], [8.0, 4.0, 2.0]], [[2.0, 2.0, 4.0], [8.0, 2.0, 2.0]]], [[[1.0, 3.0, 4.0], [3.0, 
2.0, 8.0]], [[8.0, 6.0, 5.0], [7.0, 5.0, 4.0]]], [[[2.0, 6.0, 4.0], [3.0, 6.0, 2.0]], [[3.0, 5.0, 8.0], [7.0, 6.0, 
4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 4.0], [8.0, 8.0, 5.0]], [[1.0, 2.0, 1.0], [3.0, 8.0, 8.0]]], [[[4.0, 7.0, 
1.0], [6.0, 2.0, 8.0]], [[6.0, 1.0, 4.0], [3.0, 8.0, 8.0]]], [[[6.0, 5.0, 5.0], [2.0, 1.0, 7.0]], [[2.0, 6.0, 2.0], [1.0, 3.0, 
6.0]]]]))
      arg[:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 5.0], [8.0, 4.0, 2.0], [2.0, 5.0, 6.0]], [[2.0, 2.0, 4.0], [8.0, 2.0, 2.0], [5.0, 
2.0, 7.0]], [[5.0, 1.0, 6.0], [7.0, 3.0, 1.0], [3.0, 3.0, 7.0]]], [[[1.0, 3.0, 4.0], [3.0, 2.0, 8.0], [1.0, 3.0, 3.0]], [[8.0, 
6.0, 5.0], [7.0, 5.0, 4.0], [7.0, 4.0, 4.0]], [[5.0, 3.0, 7.0], [8.0, 7.0, 4.0], [6.0, 1.0, 7.0]]], [[[2.0, 6.0, 4.0], [3.0, 
6.0, 2.0], [7.0, 2.0, 8.0]], [[3.0, 5.0, 8.0], [7.0, 6.0, 4.0], [2.0, 5.0, 6.0]], [[3.0, 8.0, 2.0], [3.0, 1.0, 3.0], [5.0, 6.0, 
6.0]]], [[[1.0, 1.0, 4.0], [8.0, 7.0, 1.0], [2.0, 4.0, 1.0]], [[4.0, 5.0, 7.0], [5.0, 4.0, 7.0], [7.0, 3.0, 4.0]], [[1.0, 2.0, 
5.0], [6.0, 5.0, 4.0], [7.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 4.0], [8.0, 8.0, 5.0], [2.0, 5.0, 8.0]], 
[[1.0, 2.0, 1.0], [3.0, 8.0, 8.0], [1.0, 4.0, 2.0]], [[1.0, 3.0, 6.0], [3.0, 7.0, 7.0], [4.0, 6.0, 8.0]]], [[[4.0, 7.0, 1.0], 
[6.0, 2.0, 8.0], [4.0, 3.0, 4.0]], [[6.0, 1.0, 4.0], [3.0, 8.0, 8.0], [3.0, 8.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 8.0, 4.0], 
[1.0, 2.0, 5.0]]], [[[6.0, 5.0, 5.0], [2.0, 1.0, 7.0], [2.0, 4.0, 4.0]], [[2.0, 6.0, 2.0], [1.0, 3.0, 6.0], [2.0, 8.0, 5.0]], 
[[1.0, 2.0, 6.0], [6.0, 3.0, 2.0], [1.0, 2.0, 7.0]]], [[[1.0, 8.0, 7.0], [2.0, 2.0, 4.0], [6.0, 5.0, 3.0]], [[1.0, 1.0, 7.0], 
[6.0, 3.0, 1.0], [1.0, 1.0, 4.0]], [[3.0, 6.0, 8.0], [3.0, 3.0, 6.0], [1.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UUUN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 6.0], [4.0, 8.0, 1.0], [2.0, 6.0, 7.0]], [[1.0, 5.0, 4.0], [8.0, 7.0, 7.0], [6.0, 4.0, 
2.0]], [[4.0, 1.0, 5.0], [2.0, 1.0, 4.0], [2.0, 2.0, 5.0]]], [[[8.0, 6.0, 4.0], [3.0, 7.0, 3.0], [2.0, 4.0, 4.0]], [[3.0, 6.0, 
7.0], [4.0, 1.0, 5.0], [8.0, 8.0, 3.0]], [[3.0, 7.0, 6.0], [7.0, 3.0, 7.0], [5.0, 8.0, 2.0]]], [[[5.0, 8.0, 2.0], [4.0, 6.0, 
6.0], [6.0, 4.0, 7.0]], [[2.0, 3.0, 8.0], [6.0, 1.0, 8.0], [7.0, 8.0, 6.0]], [[5.0, 2.0, 1.0], [5.0, 1.0, 5.0], [4.0, 7.0, 
8.0]]], [[[3.0, 1.0, 3.0], [3.0, 8.0, 5.0], [3.0, 1.0, 4.0]], [[1.0, 7.0, 6.0], [2.0, 4.0, 3.0], [1.0, 5.0, 4.0]], [[4.0, 7.0, 
7.0], [7.0, 3.0, 7.0], [1.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 8.0], [1.0, 1.0, 7.0], [6.0, 6.0, 3.0]], [[6.0, 8.0, 3.0], [6.0, 3.0, 
8.0], [3.0, 1.0, 7.0]], [[1.0, 6.0, 7.0], [7.0, 4.0, 6.0], [7.0, 5.0, 8.0]]], [[[6.0, 1.0, 3.0], [3.0, 3.0, 4.0], [8.0, 5.0, 
2.0]], [[5.0, 4.0, 1.0], [1.0, 5.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 4.0, 7.0], [3.0, 7.0, 8.0], [5.0, 7.0, 3.0]]], [[[7.0, 5.0, 
1.0], [1.0, 8.0, 3.0], [1.0, 7.0, 2.0]], [[6.0, 8.0, 2.0], [5.0, 6.0, 4.0], [4.0, 8.0, 3.0]], [[3.0, 8.0, 5.0], [4.0, 4.0, 
1.0], [1.0, 6.0, 1.0]]], [[[2.0, 7.0, 6.0], [6.0, 3.0, 2.0], [4.0, 4.0, 7.0]], [[5.0, 6.0, 8.0], [5.0, 4.0, 7.0], [6.0, 5.0, 
6.0]], [[4.0, 5.0, 7.0], [1.0, 4.0, 2.0], [5.0, 4.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 6.0, 8.0], [5.0, 5.0, 6.0]], [[2.0, 3.0, 3.0], [6.0, 2.0, 5.0]]], [[[7.0, 5.0, 2.0], 
[4.0, 3.0, 1.0]], [[7.0, 3.0, 4.0], [3.0, 4.0, 1.0]]], [[[8.0, 4.0, 3.0], [1.0, 7.0, 3.0]], [[8.0, 6.0, 3.0], [3.0, 6.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 2.0, 8.0], [1.0, 7.0, 7.0]], [[3.0, 6.0, 3.0], [7.0, 1.0, 5.0]]], [[[2.0, 3.0, 2.0], 
[2.0, 4.0, 2.0]], [[8.0, 6.0, 8.0], [3.0, 6.0, 5.0]]], [[[5.0, 1.0, 3.0], [2.0, 6.0, 1.0]], [[3.0, 2.0, 3.0], [4.0, 1.0, 
4.0]]]])
      arg[:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 8.0], [5.0, 5.0, 6.0], [2.0, 6.0, 7.0]], [[2.0, 3.0, 3.0], [6.0, 2.0, 5.0], [6.0, 
4.0, 2.0]], [[4.0, 1.0, 5.0], [2.0, 1.0, 4.0], [2.0, 2.0, 5.0]]], [[[7.0, 5.0, 2.0], [4.0, 3.0, 1.0], [2.0, 4.0, 4.0]], [[7.0, 
3.0, 4.0], [3.0, 4.0, 1.0], [8.0, 8.0, 3.0]], [[3.0, 7.0, 6.0], [7.0, 3.0, 7.0], [5.0, 8.0, 2.0]]], [[[8.0, 4.0, 3.0], [1.0, 
7.0, 3.0], [6.0, 4.0, 7.0]], [[8.0, 6.0, 3.0], [3.0, 6.0, 8.0], [7.0, 8.0, 6.0]], [[5.0, 2.0, 1.0], [5.0, 1.0, 5.0], [4.0, 7.0, 
8.0]]], [[[3.0, 1.0, 3.0], [3.0, 8.0, 5.0], [3.0, 1.0, 4.0]], [[1.0, 7.0, 6.0], [2.0, 4.0, 3.0], [1.0, 5.0, 4.0]], [[4.0, 7.0, 
7.0], [7.0, 3.0, 7.0], [1.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 8.0], [1.0, 7.0, 7.0], [6.0, 6.0, 3.0]], 
[[3.0, 6.0, 3.0], [7.0, 1.0, 5.0], [3.0, 1.0, 7.0]], [[1.0, 6.0, 7.0], [7.0, 4.0, 6.0], [7.0, 5.0, 8.0]]], [[[2.0, 3.0, 2.0], 
[2.0, 4.0, 2.0], [8.0, 5.0, 2.0]], [[8.0, 6.0, 8.0], [3.0, 6.0, 5.0], [8.0, 8.0, 5.0]], [[1.0, 4.0, 7.0], [3.0, 7.0, 8.0], 
[5.0, 7.0, 3.0]]], [[[5.0, 1.0, 3.0], [2.0, 6.0, 1.0], [1.0, 7.0, 2.0]], [[3.0, 2.0, 3.0], [4.0, 1.0, 4.0], [4.0, 8.0, 3.0]], 
[[3.0, 8.0, 5.0], [4.0, 4.0, 1.0], [1.0, 6.0, 1.0]]], [[[2.0, 7.0, 6.0], [6.0, 3.0, 2.0], [4.0, 4.0, 7.0]], [[5.0, 6.0, 8.0], 
[5.0, 4.0, 7.0], [6.0, 5.0, 6.0]], [[4.0, 5.0, 7.0], [1.0, 4.0, 2.0], [5.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULRN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 2.0], [2.0, 4.0, 6.0], [6.0, 1.0, 2.0]], [[5.0, 4.0, 5.0], [3.0, 7.0, 4.0], [3.0, 5.0, 
4.0]], [[1.0, 3.0, 5.0], [6.0, 6.0, 8.0], [2.0, 2.0, 3.0]]], [[[8.0, 5.0, 3.0], [2.0, 1.0, 5.0], [6.0, 7.0, 7.0]], [[3.0, 1.0, 
3.0], [1.0, 4.0, 6.0], [7.0, 8.0, 1.0]], [[7.0, 5.0, 8.0], [3.0, 2.0, 1.0], [7.0, 7.0, 3.0]]], [[[7.0, 5.0, 8.0], [4.0, 7.0, 
2.0], [2.0, 4.0, 7.0]], [[5.0, 8.0, 1.0], [7.0, 6.0, 6.0], [3.0, 4.0, 2.0]], [[4.0, 5.0, 5.0], [3.0, 3.0, 8.0], [7.0, 5.0, 
5.0]]], [[[8.0, 3.0, 8.0], [4.0, 5.0, 2.0], [4.0, 7.0, 6.0]], [[8.0, 8.0, 2.0], [8.0, 2.0, 7.0], [7.0, 2.0, 1.0]], [[6.0, 2.0, 
7.0], [4.0, 1.0, 6.0], [6.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 8.0], [7.0, 4.0, 1.0], [4.0, 4.0, 5.0]], [[4.0, 6.0, 1.0], [6.0, 6.0, 
8.0], [1.0, 8.0, 1.0]], [[3.0, 6.0, 1.0], [6.0, 2.0, 8.0], [2.0, 7.0, 7.0]]], [[[1.0, 6.0, 4.0], [7.0, 1.0, 1.0], [1.0, 8.0, 
6.0]], [[2.0, 6.0, 2.0], [4.0, 6.0, 4.0], [2.0, 3.0, 8.0]], [[1.0, 2.0, 8.0], [4.0, 8.0, 2.0], [8.0, 2.0, 5.0]]], [[[5.0, 8.0, 
7.0], [8.0, 7.0, 1.0], [7.0, 2.0, 8.0]], [[6.0, 1.0, 2.0], [5.0, 3.0, 8.0], [5.0, 4.0, 7.0]], [[8.0, 1.0, 8.0], [1.0, 1.0, 
5.0], [6.0, 4.0, 5.0]]], [[[6.0, 7.0, 6.0], [6.0, 7.0, 8.0], [1.0, 8.0, 4.0]], [[5.0, 5.0, 3.0], [5.0, 3.0, 4.0], [8.0, 4.0, 
6.0]], [[1.0, 1.0, 1.0], [1.0, 3.0, 7.0], [1.0, 4.0, 1.0]]]]))
      val=numpy.array([[[[3.0, 8.0, 2.0]]], [[[2.0, 5.0, 2.0]]], [[[2.0, 3.0, 3.0]]]])
      arg[:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 2.0], [2.0, 4.0, 6.0], [6.0, 1.0, 2.0]], [[5.0, 4.0, 5.0], [3.0, 7.0, 4.0], [3.0, 
5.0, 4.0]], [[1.0, 3.0, 5.0], [3.0, 8.0, 2.0], [2.0, 2.0, 3.0]]], [[[8.0, 5.0, 3.0], [2.0, 1.0, 5.0], [6.0, 7.0, 7.0]], [[3.0, 
1.0, 3.0], [1.0, 4.0, 6.0], [7.0, 8.0, 1.0]], [[7.0, 5.0, 8.0], [2.0, 5.0, 2.0], [7.0, 7.0, 3.0]]], [[[7.0, 5.0, 8.0], [4.0, 
7.0, 2.0], [2.0, 4.0, 7.0]], [[5.0, 8.0, 1.0], [7.0, 6.0, 6.0], [3.0, 4.0, 2.0]], [[4.0, 5.0, 5.0], [2.0, 3.0, 3.0], [7.0, 5.0, 
5.0]]], [[[8.0, 3.0, 8.0], [4.0, 5.0, 2.0], [4.0, 7.0, 6.0]], [[8.0, 8.0, 2.0], [8.0, 2.0, 7.0], [7.0, 2.0, 1.0]], [[6.0, 2.0, 
7.0], [4.0, 1.0, 6.0], [6.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 8.0], [7.0, 4.0, 1.0], [4.0, 4.0, 5.0]], 
[[4.0, 6.0, 1.0], [6.0, 6.0, 8.0], [1.0, 8.0, 1.0]], [[3.0, 6.0, 1.0], [3.0, 8.0, 2.0], [2.0, 7.0, 7.0]]], [[[1.0, 6.0, 4.0], 
[7.0, 1.0, 1.0], [1.0, 8.0, 6.0]], [[2.0, 6.0, 2.0], [4.0, 6.0, 4.0], [2.0, 3.0, 8.0]], [[1.0, 2.0, 8.0], [2.0, 5.0, 2.0], 
[8.0, 2.0, 5.0]]], [[[5.0, 8.0, 7.0], [8.0, 7.0, 1.0], [7.0, 2.0, 8.0]], [[6.0, 1.0, 2.0], [5.0, 3.0, 8.0], [5.0, 4.0, 7.0]], 
[[8.0, 1.0, 8.0], [2.0, 3.0, 3.0], [6.0, 4.0, 5.0]]], [[[6.0, 7.0, 6.0], [6.0, 7.0, 8.0], [1.0, 8.0, 4.0]], [[5.0, 5.0, 3.0], 
[5.0, 3.0, 4.0], [8.0, 4.0, 6.0]], [[1.0, 1.0, 1.0], [1.0, 3.0, 7.0], [1.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULRN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 8.0], [5.0, 8.0, 3.0], [5.0, 1.0, 5.0]], [[2.0, 4.0, 5.0], [5.0, 6.0, 6.0], [3.0, 6.0, 
6.0]], [[3.0, 8.0, 3.0], [8.0, 7.0, 6.0], [8.0, 6.0, 6.0]]], [[[3.0, 8.0, 2.0], [3.0, 8.0, 4.0], [7.0, 6.0, 4.0]], [[5.0, 4.0, 
3.0], [2.0, 3.0, 6.0], [2.0, 4.0, 2.0]], [[4.0, 8.0, 8.0], [3.0, 8.0, 3.0], [1.0, 4.0, 1.0]]], [[[3.0, 2.0, 2.0], [2.0, 4.0, 
2.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 8.0], [3.0, 3.0, 8.0], [8.0, 5.0, 7.0]], [[8.0, 4.0, 6.0], [4.0, 2.0, 5.0], [5.0, 1.0, 
1.0]]], [[[8.0, 1.0, 2.0], [8.0, 5.0, 7.0], [8.0, 2.0, 5.0]], [[8.0, 7.0, 2.0], [2.0, 7.0, 4.0], [5.0, 8.0, 8.0]], [[5.0, 8.0, 
2.0], [2.0, 2.0, 7.0], [8.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 2.0], [1.0, 8.0, 7.0], [2.0, 5.0, 5.0]], [[8.0, 5.0, 2.0], [3.0, 7.0, 
1.0], [6.0, 3.0, 1.0]], [[3.0, 2.0, 8.0], [2.0, 3.0, 6.0], [2.0, 1.0, 8.0]]], [[[6.0, 3.0, 1.0], [5.0, 3.0, 5.0], [8.0, 3.0, 
7.0]], [[7.0, 5.0, 4.0], [8.0, 8.0, 7.0], [3.0, 3.0, 1.0]], [[5.0, 2.0, 5.0], [8.0, 3.0, 8.0], [7.0, 6.0, 3.0]]], [[[7.0, 2.0, 
1.0], [2.0, 4.0, 2.0], [1.0, 4.0, 6.0]], [[4.0, 8.0, 7.0], [6.0, 6.0, 4.0], [5.0, 3.0, 2.0]], [[3.0, 4.0, 8.0], [5.0, 6.0, 
1.0], [2.0, 2.0, 8.0]]], [[[1.0, 1.0, 6.0], [6.0, 7.0, 7.0], [3.0, 6.0, 8.0]], [[7.0, 4.0, 2.0], [2.0, 3.0, 3.0], [2.0, 6.0, 
6.0]], [[6.0, 8.0, 5.0], [5.0, 6.0, 1.0], [5.0, 7.0, 4.0]]]]))
      val=Data(numpy.array([[[[3.0, 7.0, 1.0]]], [[[4.0, 4.0, 6.0]]], [[[7.0, 2.0, 1.0]]]]),self.functionspace)
      arg[:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 8.0], [5.0, 8.0, 3.0], [5.0, 1.0, 5.0]], [[2.0, 4.0, 5.0], [5.0, 6.0, 6.0], [3.0, 
6.0, 6.0]], [[3.0, 8.0, 3.0], [3.0, 7.0, 1.0], [8.0, 6.0, 6.0]]], [[[3.0, 8.0, 2.0], [3.0, 8.0, 4.0], [7.0, 6.0, 4.0]], [[5.0, 
4.0, 3.0], [2.0, 3.0, 6.0], [2.0, 4.0, 2.0]], [[4.0, 8.0, 8.0], [4.0, 4.0, 6.0], [1.0, 4.0, 1.0]]], [[[3.0, 2.0, 2.0], [2.0, 
4.0, 2.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 8.0], [3.0, 3.0, 8.0], [8.0, 5.0, 7.0]], [[8.0, 4.0, 6.0], [7.0, 2.0, 1.0], [5.0, 1.0, 
1.0]]], [[[8.0, 1.0, 2.0], [8.0, 5.0, 7.0], [8.0, 2.0, 5.0]], [[8.0, 7.0, 2.0], [2.0, 7.0, 4.0], [5.0, 8.0, 8.0]], [[5.0, 8.0, 
2.0], [2.0, 2.0, 7.0], [8.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 2.0], [1.0, 8.0, 7.0], [2.0, 5.0, 5.0]], 
[[8.0, 5.0, 2.0], [3.0, 7.0, 1.0], [6.0, 3.0, 1.0]], [[3.0, 2.0, 8.0], [3.0, 7.0, 1.0], [2.0, 1.0, 8.0]]], [[[6.0, 3.0, 1.0], 
[5.0, 3.0, 5.0], [8.0, 3.0, 7.0]], [[7.0, 5.0, 4.0], [8.0, 8.0, 7.0], [3.0, 3.0, 1.0]], [[5.0, 2.0, 5.0], [4.0, 4.0, 6.0], 
[7.0, 6.0, 3.0]]], [[[7.0, 2.0, 1.0], [2.0, 4.0, 2.0], [1.0, 4.0, 6.0]], [[4.0, 8.0, 7.0], [6.0, 6.0, 4.0], [5.0, 3.0, 2.0]], 
[[3.0, 4.0, 8.0], [7.0, 2.0, 1.0], [2.0, 2.0, 8.0]]], [[[1.0, 1.0, 6.0], [6.0, 7.0, 7.0], [3.0, 6.0, 8.0]], [[7.0, 4.0, 2.0], 
[2.0, 3.0, 3.0], [2.0, 6.0, 6.0]], [[6.0, 8.0, 5.0], [5.0, 6.0, 1.0], [5.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULRN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 6.0], [2.0, 1.0, 6.0], [3.0, 7.0, 7.0]], [[7.0, 2.0, 8.0], [3.0, 7.0, 6.0], [3.0, 6.0, 
6.0]], [[7.0, 7.0, 8.0], [1.0, 4.0, 1.0], [2.0, 5.0, 8.0]]], [[[6.0, 1.0, 4.0], [8.0, 6.0, 3.0], [1.0, 1.0, 7.0]], [[4.0, 7.0, 
6.0], [2.0, 4.0, 3.0], [6.0, 5.0, 2.0]], [[7.0, 4.0, 3.0], [4.0, 4.0, 7.0], [2.0, 8.0, 3.0]]], [[[4.0, 5.0, 5.0], [3.0, 1.0, 
8.0], [6.0, 1.0, 6.0]], [[4.0, 8.0, 2.0], [5.0, 6.0, 5.0], [5.0, 2.0, 3.0]], [[8.0, 5.0, 6.0], [6.0, 8.0, 3.0], [4.0, 4.0, 
8.0]]], [[[2.0, 2.0, 2.0], [2.0, 7.0, 7.0], [7.0, 8.0, 3.0]], [[4.0, 5.0, 7.0], [4.0, 3.0, 4.0], [2.0, 1.0, 2.0]], [[6.0, 4.0, 
5.0], [6.0, 4.0, 1.0], [8.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 5.0], [1.0, 5.0, 5.0], [2.0, 6.0, 3.0]], [[1.0, 8.0, 3.0], [8.0, 5.0, 
8.0], [3.0, 4.0, 6.0]], [[8.0, 6.0, 1.0], [7.0, 3.0, 3.0], [6.0, 3.0, 6.0]]], [[[2.0, 2.0, 7.0], [2.0, 8.0, 8.0], [8.0, 5.0, 
7.0]], [[3.0, 1.0, 1.0], [7.0, 1.0, 4.0], [4.0, 4.0, 1.0]], [[5.0, 8.0, 7.0], [4.0, 5.0, 2.0], [7.0, 1.0, 7.0]]], [[[3.0, 5.0, 
3.0], [7.0, 6.0, 1.0], [1.0, 5.0, 7.0]], [[8.0, 5.0, 4.0], [8.0, 2.0, 3.0], [1.0, 2.0, 5.0]], [[2.0, 5.0, 4.0], [5.0, 6.0, 
3.0], [1.0, 1.0, 8.0]]], [[[8.0, 7.0, 2.0], [4.0, 2.0, 8.0], [6.0, 8.0, 1.0]], [[3.0, 2.0, 1.0], [8.0, 8.0, 1.0], [2.0, 6.0, 
4.0]], [[4.0, 8.0, 4.0], [3.0, 4.0, 7.0], [2.0, 2.0, 2.0]]]]))
      val=Data(numpy.array([[[[4.0, 4.0, 4.0]]], [[[7.0, 2.0, 4.0]]], [[[7.0, 4.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 1.0]]], [[[2.0, 5.0, 3.0]]], [[[8.0, 5.0, 3.0]]]]))
      arg[:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 6.0], [2.0, 1.0, 6.0], [3.0, 7.0, 7.0]], [[7.0, 2.0, 8.0], [3.0, 7.0, 6.0], [3.0, 
6.0, 6.0]], [[7.0, 7.0, 8.0], [4.0, 4.0, 4.0], [2.0, 5.0, 8.0]]], [[[6.0, 1.0, 4.0], [8.0, 6.0, 3.0], [1.0, 1.0, 7.0]], [[4.0, 
7.0, 6.0], [2.0, 4.0, 3.0], [6.0, 5.0, 2.0]], [[7.0, 4.0, 3.0], [7.0, 2.0, 4.0], [2.0, 8.0, 3.0]]], [[[4.0, 5.0, 5.0], [3.0, 
1.0, 8.0], [6.0, 1.0, 6.0]], [[4.0, 8.0, 2.0], [5.0, 6.0, 5.0], [5.0, 2.0, 3.0]], [[8.0, 5.0, 6.0], [7.0, 4.0, 8.0], [4.0, 4.0, 
8.0]]], [[[2.0, 2.0, 2.0], [2.0, 7.0, 7.0], [7.0, 8.0, 3.0]], [[4.0, 5.0, 7.0], [4.0, 3.0, 4.0], [2.0, 1.0, 2.0]], [[6.0, 4.0, 
5.0], [6.0, 4.0, 1.0], [8.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 5.0], [1.0, 5.0, 5.0], [2.0, 6.0, 3.0]], 
[[1.0, 8.0, 3.0], [8.0, 5.0, 8.0], [3.0, 4.0, 6.0]], [[8.0, 6.0, 1.0], [8.0, 6.0, 1.0], [6.0, 3.0, 6.0]]], [[[2.0, 2.0, 7.0], 
[2.0, 8.0, 8.0], [8.0, 5.0, 7.0]], [[3.0, 1.0, 1.0], [7.0, 1.0, 4.0], [4.0, 4.0, 1.0]], [[5.0, 8.0, 7.0], [2.0, 5.0, 3.0], 
[7.0, 1.0, 7.0]]], [[[3.0, 5.0, 3.0], [7.0, 6.0, 1.0], [1.0, 5.0, 7.0]], [[8.0, 5.0, 4.0], [8.0, 2.0, 3.0], [1.0, 2.0, 5.0]], 
[[2.0, 5.0, 4.0], [8.0, 5.0, 3.0], [1.0, 1.0, 8.0]]], [[[8.0, 7.0, 2.0], [4.0, 2.0, 8.0], [6.0, 8.0, 1.0]], [[3.0, 2.0, 1.0], 
[8.0, 8.0, 1.0], [2.0, 6.0, 4.0]], [[4.0, 8.0, 4.0], [3.0, 4.0, 7.0], [2.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULRN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 5.0], [6.0, 3.0, 8.0], [4.0, 1.0, 1.0]], [[5.0, 4.0, 3.0], [2.0, 5.0, 6.0], [4.0, 6.0, 
7.0]], [[4.0, 1.0, 5.0], [5.0, 2.0, 5.0], [8.0, 5.0, 1.0]]], [[[6.0, 8.0, 1.0], [6.0, 6.0, 4.0], [7.0, 4.0, 4.0]], [[8.0, 1.0, 
7.0], [5.0, 5.0, 3.0], [7.0, 1.0, 4.0]], [[7.0, 4.0, 4.0], [8.0, 1.0, 6.0], [1.0, 6.0, 8.0]]], [[[2.0, 4.0, 6.0], [4.0, 7.0, 
4.0], [3.0, 8.0, 1.0]], [[3.0, 6.0, 8.0], [6.0, 1.0, 4.0], [3.0, 1.0, 2.0]], [[8.0, 2.0, 8.0], [4.0, 1.0, 4.0], [6.0, 6.0, 
1.0]]], [[[8.0, 4.0, 1.0], [2.0, 1.0, 5.0], [7.0, 8.0, 2.0]], [[4.0, 6.0, 8.0], [8.0, 6.0, 3.0], [7.0, 3.0, 4.0]], [[7.0, 7.0, 
6.0], [6.0, 3.0, 4.0], [8.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 1.0], [7.0, 7.0, 3.0], [2.0, 1.0, 4.0]], [[2.0, 4.0, 1.0], [2.0, 5.0, 
1.0], [3.0, 3.0, 8.0]], [[3.0, 4.0, 8.0], [6.0, 7.0, 7.0], [7.0, 2.0, 6.0]]], [[[8.0, 8.0, 7.0], [1.0, 3.0, 4.0], [6.0, 1.0, 
8.0]], [[6.0, 4.0, 7.0], [7.0, 4.0, 3.0], [2.0, 4.0, 3.0]], [[5.0, 2.0, 5.0], [8.0, 5.0, 6.0], [5.0, 1.0, 6.0]]], [[[3.0, 2.0, 
4.0], [8.0, 7.0, 7.0], [8.0, 4.0, 2.0]], [[3.0, 2.0, 5.0], [8.0, 8.0, 2.0], [1.0, 8.0, 5.0]], [[6.0, 4.0, 8.0], [1.0, 4.0, 
2.0], [6.0, 1.0, 1.0]]], [[[7.0, 3.0, 7.0], [5.0, 8.0, 4.0], [3.0, 6.0, 4.0]], [[8.0, 3.0, 3.0], [2.0, 7.0, 3.0], [8.0, 8.0, 
2.0]], [[4.0, 7.0, 6.0], [3.0, 4.0, 3.0], [4.0, 8.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 5.0, 8.0]]], [[[4.0, 5.0, 3.0]]], [[[2.0, 3.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 8.0, 4.0]]], [[[8.0, 8.0, 5.0]]], [[[1.0, 3.0, 4.0]]]])
      arg[:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 5.0], [6.0, 3.0, 8.0], [4.0, 1.0, 1.0]], [[5.0, 4.0, 3.0], [2.0, 5.0, 6.0], [4.0, 
6.0, 7.0]], [[4.0, 1.0, 5.0], [3.0, 5.0, 8.0], [8.0, 5.0, 1.0]]], [[[6.0, 8.0, 1.0], [6.0, 6.0, 4.0], [7.0, 4.0, 4.0]], [[8.0, 
1.0, 7.0], [5.0, 5.0, 3.0], [7.0, 1.0, 4.0]], [[7.0, 4.0, 4.0], [4.0, 5.0, 3.0], [1.0, 6.0, 8.0]]], [[[2.0, 4.0, 6.0], [4.0, 
7.0, 4.0], [3.0, 8.0, 1.0]], [[3.0, 6.0, 8.0], [6.0, 1.0, 4.0], [3.0, 1.0, 2.0]], [[8.0, 2.0, 8.0], [2.0, 3.0, 8.0], [6.0, 6.0, 
1.0]]], [[[8.0, 4.0, 1.0], [2.0, 1.0, 5.0], [7.0, 8.0, 2.0]], [[4.0, 6.0, 8.0], [8.0, 6.0, 3.0], [7.0, 3.0, 4.0]], [[7.0, 7.0, 
6.0], [6.0, 3.0, 4.0], [8.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 1.0], [7.0, 7.0, 3.0], [2.0, 1.0, 4.0]], 
[[2.0, 4.0, 1.0], [2.0, 5.0, 1.0], [3.0, 3.0, 8.0]], [[3.0, 4.0, 8.0], [1.0, 8.0, 4.0], [7.0, 2.0, 6.0]]], [[[8.0, 8.0, 7.0], 
[1.0, 3.0, 4.0], [6.0, 1.0, 8.0]], [[6.0, 4.0, 7.0], [7.0, 4.0, 3.0], [2.0, 4.0, 3.0]], [[5.0, 2.0, 5.0], [8.0, 8.0, 5.0], 
[5.0, 1.0, 6.0]]], [[[3.0, 2.0, 4.0], [8.0, 7.0, 7.0], [8.0, 4.0, 2.0]], [[3.0, 2.0, 5.0], [8.0, 8.0, 2.0], [1.0, 8.0, 5.0]], 
[[6.0, 4.0, 8.0], [1.0, 3.0, 4.0], [6.0, 1.0, 1.0]]], [[[7.0, 3.0, 7.0], [5.0, 8.0, 4.0], [3.0, 6.0, 4.0]], [[8.0, 3.0, 3.0], 
[2.0, 7.0, 3.0], [8.0, 8.0, 2.0]], [[4.0, 7.0, 6.0], [3.0, 4.0, 3.0], [4.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LUUN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 2.0], [3.0, 6.0, 8.0], [4.0, 7.0, 7.0]], [[7.0, 1.0, 4.0], [8.0, 2.0, 4.0], [7.0, 6.0, 
7.0]], [[8.0, 5.0, 6.0], [2.0, 7.0, 2.0], [6.0, 8.0, 4.0]]], [[[3.0, 8.0, 5.0], [3.0, 3.0, 4.0], [7.0, 2.0, 6.0]], [[6.0, 4.0, 
5.0], [4.0, 1.0, 5.0], [8.0, 7.0, 1.0]], [[6.0, 8.0, 8.0], [5.0, 3.0, 3.0], [3.0, 4.0, 5.0]]], [[[4.0, 4.0, 5.0], [6.0, 8.0, 
2.0], [2.0, 2.0, 5.0]], [[6.0, 5.0, 7.0], [2.0, 2.0, 6.0], [2.0, 7.0, 1.0]], [[7.0, 8.0, 2.0], [4.0, 5.0, 4.0], [1.0, 7.0, 
3.0]]], [[[1.0, 4.0, 1.0], [6.0, 8.0, 4.0], [7.0, 5.0, 8.0]], [[5.0, 2.0, 8.0], [3.0, 5.0, 2.0], [8.0, 4.0, 5.0]], [[3.0, 5.0, 
3.0], [1.0, 8.0, 8.0], [8.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 5.0], [1.0, 4.0, 4.0], [8.0, 6.0, 8.0]], [[2.0, 2.0, 6.0], [3.0, 7.0, 
8.0], [4.0, 2.0, 6.0]], [[1.0, 5.0, 4.0], [6.0, 8.0, 6.0], [6.0, 6.0, 1.0]]], [[[8.0, 4.0, 3.0], [5.0, 2.0, 4.0], [1.0, 5.0, 
6.0]], [[7.0, 2.0, 2.0], [5.0, 4.0, 1.0], [3.0, 6.0, 7.0]], [[1.0, 6.0, 2.0], [8.0, 3.0, 7.0], [7.0, 3.0, 3.0]]], [[[6.0, 1.0, 
3.0], [2.0, 8.0, 3.0], [1.0, 5.0, 5.0]], [[2.0, 7.0, 6.0], [8.0, 5.0, 8.0], [7.0, 6.0, 3.0]], [[5.0, 8.0, 1.0], [6.0, 5.0, 
6.0], [8.0, 6.0, 1.0]]], [[[2.0, 7.0, 6.0], [3.0, 1.0, 7.0], [6.0, 6.0, 3.0]], [[4.0, 5.0, 2.0], [5.0, 3.0, 7.0], [6.0, 3.0, 
3.0]], [[4.0, 4.0, 1.0], [8.0, 5.0, 1.0], [3.0, 3.0, 8.0]]]]))
      val=numpy.array([[[[8.0, 8.0, 3.0], [5.0, 3.0, 8.0]], [[8.0, 1.0, 8.0], [1.0, 7.0, 3.0]]], [[[7.0, 5.0, 8.0], [4.0, 2.0, 
2.0]], [[5.0, 2.0, 3.0], [1.0, 5.0, 4.0]]]])
      arg[2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 2.0], [3.0, 6.0, 8.0], [4.0, 7.0, 7.0]], [[7.0, 1.0, 4.0], [8.0, 2.0, 4.0], [7.0, 
6.0, 7.0]], [[8.0, 5.0, 6.0], [2.0, 7.0, 2.0], [6.0, 8.0, 4.0]]], [[[3.0, 8.0, 5.0], [3.0, 3.0, 4.0], [7.0, 2.0, 6.0]], [[6.0, 
4.0, 5.0], [4.0, 1.0, 5.0], [8.0, 7.0, 1.0]], [[6.0, 8.0, 8.0], [5.0, 3.0, 3.0], [3.0, 4.0, 5.0]]], [[[8.0, 8.0, 3.0], [5.0, 
3.0, 8.0], [2.0, 2.0, 5.0]], [[8.0, 1.0, 8.0], [1.0, 7.0, 3.0], [2.0, 7.0, 1.0]], [[7.0, 8.0, 2.0], [4.0, 5.0, 4.0], [1.0, 7.0, 
3.0]]], [[[7.0, 5.0, 8.0], [4.0, 2.0, 2.0], [7.0, 5.0, 8.0]], [[5.0, 2.0, 3.0], [1.0, 5.0, 4.0], [8.0, 4.0, 5.0]], [[3.0, 5.0, 
3.0], [1.0, 8.0, 8.0], [8.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 5.0], [1.0, 4.0, 4.0], [8.0, 6.0, 8.0]], 
[[2.0, 2.0, 6.0], [3.0, 7.0, 8.0], [4.0, 2.0, 6.0]], [[1.0, 5.0, 4.0], [6.0, 8.0, 6.0], [6.0, 6.0, 1.0]]], [[[8.0, 4.0, 3.0], 
[5.0, 2.0, 4.0], [1.0, 5.0, 6.0]], [[7.0, 2.0, 2.0], [5.0, 4.0, 1.0], [3.0, 6.0, 7.0]], [[1.0, 6.0, 2.0], [8.0, 3.0, 7.0], 
[7.0, 3.0, 3.0]]], [[[8.0, 8.0, 3.0], [5.0, 3.0, 8.0], [1.0, 5.0, 5.0]], [[8.0, 1.0, 8.0], [1.0, 7.0, 3.0], [7.0, 6.0, 3.0]], 
[[5.0, 8.0, 1.0], [6.0, 5.0, 6.0], [8.0, 6.0, 1.0]]], [[[7.0, 5.0, 8.0], [4.0, 2.0, 2.0], [6.0, 6.0, 3.0]], [[5.0, 2.0, 3.0], 
[1.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[4.0, 4.0, 1.0], [8.0, 5.0, 1.0], [3.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LUUN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 2.0], [4.0, 4.0, 7.0], [6.0, 7.0, 4.0]], [[2.0, 7.0, 5.0], [4.0, 2.0, 2.0], [2.0, 5.0, 
2.0]], [[3.0, 4.0, 1.0], [1.0, 6.0, 8.0], [7.0, 5.0, 3.0]]], [[[4.0, 3.0, 1.0], [4.0, 8.0, 2.0], [4.0, 3.0, 1.0]], [[5.0, 5.0, 
1.0], [1.0, 5.0, 2.0], [3.0, 5.0, 4.0]], [[3.0, 2.0, 1.0], [1.0, 3.0, 6.0], [2.0, 2.0, 1.0]]], [[[4.0, 3.0, 5.0], [3.0, 4.0, 
2.0], [2.0, 8.0, 8.0]], [[3.0, 5.0, 3.0], [1.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 4.0], [6.0, 1.0, 7.0], [6.0, 6.0, 
7.0]]], [[[1.0, 2.0, 3.0], [1.0, 5.0, 7.0], [8.0, 7.0, 3.0]], [[3.0, 5.0, 7.0], [7.0, 3.0, 2.0], [8.0, 2.0, 5.0]], [[5.0, 2.0, 
5.0], [8.0, 5.0, 4.0], [2.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 3.0], [4.0, 5.0, 4.0], [3.0, 2.0, 1.0]], [[8.0, 1.0, 2.0], [2.0, 3.0, 
8.0], [1.0, 8.0, 7.0]], [[7.0, 8.0, 2.0], [8.0, 5.0, 7.0], [2.0, 1.0, 6.0]]], [[[6.0, 1.0, 1.0], [8.0, 1.0, 6.0], [3.0, 3.0, 
3.0]], [[7.0, 4.0, 5.0], [4.0, 6.0, 3.0], [6.0, 2.0, 5.0]], [[8.0, 4.0, 7.0], [3.0, 2.0, 6.0], [8.0, 1.0, 6.0]]], [[[2.0, 6.0, 
7.0], [4.0, 6.0, 8.0], [5.0, 8.0, 5.0]], [[1.0, 4.0, 3.0], [6.0, 4.0, 1.0], [8.0, 7.0, 1.0]], [[6.0, 1.0, 5.0], [7.0, 3.0, 
2.0], [7.0, 5.0, 4.0]]], [[[4.0, 6.0, 4.0], [8.0, 7.0, 4.0], [2.0, 8.0, 7.0]], [[4.0, 1.0, 2.0], [2.0, 8.0, 6.0], [5.0, 3.0, 
2.0]], [[2.0, 4.0, 6.0], [1.0, 3.0, 6.0], [2.0, 8.0, 1.0]]]]))
      val=Data(numpy.array([[[[2.0, 2.0, 7.0], [3.0, 5.0, 5.0]], [[7.0, 2.0, 8.0], [7.0, 4.0, 3.0]]], [[[8.0, 5.0, 3.0], [5.0, 
4.0, 6.0]], [[7.0, 7.0, 5.0], [5.0, 8.0, 8.0]]]]),self.functionspace)
      arg[2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 2.0], [4.0, 4.0, 7.0], [6.0, 7.0, 4.0]], [[2.0, 7.0, 5.0], [4.0, 2.0, 2.0], [2.0, 
5.0, 2.0]], [[3.0, 4.0, 1.0], [1.0, 6.0, 8.0], [7.0, 5.0, 3.0]]], [[[4.0, 3.0, 1.0], [4.0, 8.0, 2.0], [4.0, 3.0, 1.0]], [[5.0, 
5.0, 1.0], [1.0, 5.0, 2.0], [3.0, 5.0, 4.0]], [[3.0, 2.0, 1.0], [1.0, 3.0, 6.0], [2.0, 2.0, 1.0]]], [[[2.0, 2.0, 7.0], [3.0, 
5.0, 5.0], [2.0, 8.0, 8.0]], [[7.0, 2.0, 8.0], [7.0, 4.0, 3.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 4.0], [6.0, 1.0, 7.0], [6.0, 6.0, 
7.0]]], [[[8.0, 5.0, 3.0], [5.0, 4.0, 6.0], [8.0, 7.0, 3.0]], [[7.0, 7.0, 5.0], [5.0, 8.0, 8.0], [8.0, 2.0, 5.0]], [[5.0, 2.0, 
5.0], [8.0, 5.0, 4.0], [2.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 3.0], [4.0, 5.0, 4.0], [3.0, 2.0, 1.0]], 
[[8.0, 1.0, 2.0], [2.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[7.0, 8.0, 2.0], [8.0, 5.0, 7.0], [2.0, 1.0, 6.0]]], [[[6.0, 1.0, 1.0], 
[8.0, 1.0, 6.0], [3.0, 3.0, 3.0]], [[7.0, 4.0, 5.0], [4.0, 6.0, 3.0], [6.0, 2.0, 5.0]], [[8.0, 4.0, 7.0], [3.0, 2.0, 6.0], 
[8.0, 1.0, 6.0]]], [[[2.0, 2.0, 7.0], [3.0, 5.0, 5.0], [5.0, 8.0, 5.0]], [[7.0, 2.0, 8.0], [7.0, 4.0, 3.0], [8.0, 7.0, 1.0]], 
[[6.0, 1.0, 5.0], [7.0, 3.0, 2.0], [7.0, 5.0, 4.0]]], [[[8.0, 5.0, 3.0], [5.0, 4.0, 6.0], [2.0, 8.0, 7.0]], [[7.0, 7.0, 5.0], 
[5.0, 8.0, 8.0], [5.0, 3.0, 2.0]], [[2.0, 4.0, 6.0], [1.0, 3.0, 6.0], [2.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LUUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 4.0], [4.0, 8.0, 2.0], [3.0, 8.0, 8.0]], [[3.0, 4.0, 6.0], [4.0, 3.0, 5.0], [5.0, 5.0, 
6.0]], [[1.0, 7.0, 4.0], [2.0, 7.0, 8.0], [3.0, 5.0, 3.0]]], [[[6.0, 1.0, 6.0], [5.0, 6.0, 5.0], [8.0, 7.0, 3.0]], [[1.0, 5.0, 
1.0], [3.0, 3.0, 5.0], [6.0, 4.0, 2.0]], [[1.0, 3.0, 4.0], [3.0, 3.0, 4.0], [5.0, 7.0, 6.0]]], [[[7.0, 8.0, 1.0], [4.0, 1.0, 
2.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 5.0], [8.0, 3.0, 4.0], [2.0, 4.0, 1.0]], [[3.0, 7.0, 8.0], [6.0, 1.0, 3.0], [5.0, 2.0, 
8.0]]], [[[2.0, 1.0, 4.0], [7.0, 1.0, 1.0], [4.0, 3.0, 5.0]], [[4.0, 5.0, 7.0], [8.0, 5.0, 4.0], [3.0, 5.0, 1.0]], [[8.0, 3.0, 
5.0], [2.0, 1.0, 1.0], [8.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 6.0], [2.0, 8.0, 3.0], [7.0, 2.0, 1.0]], [[3.0, 5.0, 5.0], [7.0, 8.0, 
7.0], [4.0, 4.0, 4.0]], [[6.0, 2.0, 8.0], [3.0, 6.0, 3.0], [6.0, 3.0, 3.0]]], [[[6.0, 5.0, 4.0], [5.0, 8.0, 4.0], [3.0, 7.0, 
3.0]], [[7.0, 2.0, 6.0], [5.0, 4.0, 4.0], [2.0, 3.0, 1.0]], [[4.0, 3.0, 4.0], [2.0, 5.0, 5.0], [7.0, 6.0, 7.0]]], [[[1.0, 3.0, 
3.0], [7.0, 6.0, 7.0], [5.0, 5.0, 4.0]], [[6.0, 1.0, 2.0], [8.0, 6.0, 1.0], [3.0, 8.0, 2.0]], [[8.0, 8.0, 1.0], [8.0, 7.0, 
8.0], [1.0, 7.0, 7.0]]], [[[1.0, 3.0, 6.0], [1.0, 4.0, 3.0], [2.0, 1.0, 6.0]], [[4.0, 7.0, 8.0], [5.0, 1.0, 1.0], [3.0, 2.0, 
5.0]], [[8.0, 2.0, 3.0], [8.0, 1.0, 1.0], [1.0, 3.0, 2.0]]]]))
      val=Data(numpy.array([[[[1.0, 6.0, 5.0], [4.0, 1.0, 3.0]], [[6.0, 1.0, 6.0], [5.0, 4.0, 3.0]]], [[[7.0, 4.0, 1.0], [2.0, 
3.0, 8.0]], [[1.0, 4.0, 3.0], [6.0, 7.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 1.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 1.0], [3.0, 4.0, 5.0]]], [[[7.0, 3.0, 
4.0], [6.0, 6.0, 3.0]], [[3.0, 1.0, 4.0], [6.0, 5.0, 4.0]]]]))
      arg[2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 4.0], [4.0, 8.0, 2.0], [3.0, 8.0, 8.0]], [[3.0, 4.0, 6.0], [4.0, 3.0, 5.0], [5.0, 
5.0, 6.0]], [[1.0, 7.0, 4.0], [2.0, 7.0, 8.0], [3.0, 5.0, 3.0]]], [[[6.0, 1.0, 6.0], [5.0, 6.0, 5.0], [8.0, 7.0, 3.0]], [[1.0, 
5.0, 1.0], [3.0, 3.0, 5.0], [6.0, 4.0, 2.0]], [[1.0, 3.0, 4.0], [3.0, 3.0, 4.0], [5.0, 7.0, 6.0]]], [[[1.0, 6.0, 5.0], [4.0, 
1.0, 3.0], [2.0, 2.0, 4.0]], [[6.0, 1.0, 6.0], [5.0, 4.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 7.0, 8.0], [6.0, 1.0, 3.0], [5.0, 2.0, 
8.0]]], [[[7.0, 4.0, 1.0], [2.0, 3.0, 8.0], [4.0, 3.0, 5.0]], [[1.0, 4.0, 3.0], [6.0, 7.0, 2.0], [3.0, 5.0, 1.0]], [[8.0, 3.0, 
5.0], [2.0, 1.0, 1.0], [8.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 6.0], [2.0, 8.0, 3.0], [7.0, 2.0, 1.0]], 
[[3.0, 5.0, 5.0], [7.0, 8.0, 7.0], [4.0, 4.0, 4.0]], [[6.0, 2.0, 8.0], [3.0, 6.0, 3.0], [6.0, 3.0, 3.0]]], [[[6.0, 5.0, 4.0], 
[5.0, 8.0, 4.0], [3.0, 7.0, 3.0]], [[7.0, 2.0, 6.0], [5.0, 4.0, 4.0], [2.0, 3.0, 1.0]], [[4.0, 3.0, 4.0], [2.0, 5.0, 5.0], 
[7.0, 6.0, 7.0]]], [[[3.0, 3.0, 1.0], [6.0, 2.0, 4.0], [5.0, 5.0, 4.0]], [[7.0, 2.0, 1.0], [3.0, 4.0, 5.0], [3.0, 8.0, 2.0]], 
[[8.0, 8.0, 1.0], [8.0, 7.0, 8.0], [1.0, 7.0, 7.0]]], [[[7.0, 3.0, 4.0], [6.0, 6.0, 3.0], [2.0, 1.0, 6.0]], [[3.0, 1.0, 4.0], 
[6.0, 5.0, 4.0], [3.0, 2.0, 5.0]], [[8.0, 2.0, 3.0], [8.0, 1.0, 1.0], [1.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LUUN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 1.0], [8.0, 3.0, 4.0], [7.0, 5.0, 3.0]], [[8.0, 8.0, 5.0], [6.0, 2.0, 4.0], [8.0, 3.0, 
5.0]], [[4.0, 2.0, 7.0], [5.0, 1.0, 8.0], [1.0, 8.0, 2.0]]], [[[2.0, 6.0, 2.0], [4.0, 2.0, 1.0], [6.0, 6.0, 1.0]], [[1.0, 3.0, 
1.0], [6.0, 7.0, 8.0], [6.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [1.0, 6.0, 5.0], [6.0, 2.0, 7.0]]], [[[5.0, 7.0, 4.0], [8.0, 7.0, 
5.0], [1.0, 4.0, 8.0]], [[8.0, 1.0, 8.0], [6.0, 3.0, 7.0], [8.0, 8.0, 4.0]], [[8.0, 7.0, 2.0], [2.0, 7.0, 4.0], [3.0, 1.0, 
5.0]]], [[[5.0, 3.0, 3.0], [3.0, 6.0, 4.0], [8.0, 8.0, 5.0]], [[1.0, 4.0, 5.0], [5.0, 5.0, 7.0], [5.0, 4.0, 5.0]], [[6.0, 4.0, 
6.0], [2.0, 2.0, 1.0], [4.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 6.0], [4.0, 8.0, 4.0], [7.0, 7.0, 5.0]], [[8.0, 4.0, 3.0], [5.0, 5.0, 
5.0], [8.0, 3.0, 7.0]], [[5.0, 2.0, 5.0], [1.0, 1.0, 2.0], [7.0, 7.0, 6.0]]], [[[6.0, 4.0, 5.0], [2.0, 5.0, 8.0], [7.0, 4.0, 
4.0]], [[3.0, 7.0, 1.0], [7.0, 7.0, 8.0], [1.0, 3.0, 1.0]], [[1.0, 7.0, 6.0], [7.0, 6.0, 4.0], [2.0, 6.0, 7.0]]], [[[5.0, 8.0, 
6.0], [1.0, 3.0, 1.0], [4.0, 1.0, 8.0]], [[6.0, 4.0, 8.0], [7.0, 3.0, 1.0], [8.0, 8.0, 3.0]], [[6.0, 8.0, 2.0], [3.0, 6.0, 
6.0], [8.0, 4.0, 8.0]]], [[[7.0, 7.0, 3.0], [7.0, 7.0, 5.0], [5.0, 3.0, 6.0]], [[4.0, 6.0, 1.0], [7.0, 5.0, 6.0], [4.0, 3.0, 
2.0]], [[7.0, 5.0, 8.0], [1.0, 6.0, 7.0], [3.0, 7.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0, 6.0, 4.0], [7.0, 2.0, 1.0]], [[6.0, 8.0, 8.0], [4.0, 5.0, 5.0]]], [[[2.0, 1.0, 8.0], 
[8.0, 5.0, 2.0]], [[1.0, 8.0, 2.0], [3.0, 4.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 1.0, 6.0], [6.0, 7.0, 1.0]], [[1.0, 
8.0, 1.0], [4.0, 5.0, 3.0]]], [[[7.0, 7.0, 8.0], [4.0, 3.0, 1.0]], [[3.0, 8.0, 4.0], [1.0, 7.0, 1.0]]]])
      arg[2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 1.0], [8.0, 3.0, 4.0], [7.0, 5.0, 3.0]], [[8.0, 8.0, 5.0], [6.0, 2.0, 4.0], [8.0, 
3.0, 5.0]], [[4.0, 2.0, 7.0], [5.0, 1.0, 8.0], [1.0, 8.0, 2.0]]], [[[2.0, 6.0, 2.0], [4.0, 2.0, 1.0], [6.0, 6.0, 1.0]], [[1.0, 
3.0, 1.0], [6.0, 7.0, 8.0], [6.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [1.0, 6.0, 5.0], [6.0, 2.0, 7.0]]], [[[7.0, 6.0, 4.0], [7.0, 
2.0, 1.0], [1.0, 4.0, 8.0]], [[6.0, 8.0, 8.0], [4.0, 5.0, 5.0], [8.0, 8.0, 4.0]], [[8.0, 7.0, 2.0], [2.0, 7.0, 4.0], [3.0, 1.0, 
5.0]]], [[[2.0, 1.0, 8.0], [8.0, 5.0, 2.0], [8.0, 8.0, 5.0]], [[1.0, 8.0, 2.0], [3.0, 4.0, 4.0], [5.0, 4.0, 5.0]], [[6.0, 4.0, 
6.0], [2.0, 2.0, 1.0], [4.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 6.0], [4.0, 8.0, 4.0], [7.0, 7.0, 5.0]], 
[[8.0, 4.0, 3.0], [5.0, 5.0, 5.0], [8.0, 3.0, 7.0]], [[5.0, 2.0, 5.0], [1.0, 1.0, 2.0], [7.0, 7.0, 6.0]]], [[[6.0, 4.0, 5.0], 
[2.0, 5.0, 8.0], [7.0, 4.0, 4.0]], [[3.0, 7.0, 1.0], [7.0, 7.0, 8.0], [1.0, 3.0, 1.0]], [[1.0, 7.0, 6.0], [7.0, 6.0, 4.0], 
[2.0, 6.0, 7.0]]], [[[3.0, 1.0, 6.0], [6.0, 7.0, 1.0], [4.0, 1.0, 8.0]], [[1.0, 8.0, 1.0], [4.0, 5.0, 3.0], [8.0, 8.0, 3.0]], 
[[6.0, 8.0, 2.0], [3.0, 6.0, 6.0], [8.0, 4.0, 8.0]]], [[[7.0, 7.0, 8.0], [4.0, 3.0, 1.0], [5.0, 3.0, 6.0]], [[3.0, 8.0, 4.0], 
[1.0, 7.0, 1.0], [4.0, 3.0, 2.0]], [[7.0, 5.0, 8.0], [1.0, 6.0, 7.0], [3.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UURN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 3.0], [4.0, 2.0, 3.0], [3.0, 4.0, 2.0]], [[5.0, 2.0, 6.0], [5.0, 3.0, 7.0], [4.0, 3.0, 
4.0]], [[8.0, 1.0, 7.0], [4.0, 6.0, 8.0], [7.0, 8.0, 4.0]]], [[[4.0, 1.0, 2.0], [3.0, 6.0, 5.0], [3.0, 3.0, 6.0]], [[8.0, 3.0, 
8.0], [7.0, 2.0, 5.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 3.0], [5.0, 1.0, 6.0], [3.0, 1.0, 2.0]]], [[[1.0, 3.0, 5.0], [5.0, 2.0, 
6.0], [4.0, 5.0, 8.0]], [[1.0, 8.0, 8.0], [1.0, 1.0, 3.0], [6.0, 8.0, 6.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 4.0], [8.0, 4.0, 
5.0]]], [[[4.0, 6.0, 1.0], [4.0, 1.0, 7.0], [7.0, 7.0, 5.0]], [[8.0, 3.0, 7.0], [8.0, 5.0, 3.0], [4.0, 6.0, 2.0]], [[5.0, 1.0, 
5.0], [1.0, 8.0, 6.0], [3.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 2.0], [1.0, 1.0, 3.0], [4.0, 7.0, 5.0]], [[4.0, 3.0, 7.0], [7.0, 3.0, 
1.0], [4.0, 3.0, 2.0]], [[6.0, 1.0, 7.0], [1.0, 3.0, 5.0], [1.0, 1.0, 6.0]]], [[[6.0, 8.0, 7.0], [1.0, 1.0, 6.0], [4.0, 1.0, 
6.0]], [[3.0, 4.0, 1.0], [3.0, 1.0, 5.0], [2.0, 5.0, 4.0]], [[1.0, 7.0, 7.0], [6.0, 8.0, 7.0], [6.0, 5.0, 6.0]]], [[[2.0, 6.0, 
7.0], [8.0, 7.0, 1.0], [7.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [8.0, 5.0, 2.0], [4.0, 7.0, 4.0]], [[7.0, 4.0, 4.0], [8.0, 1.0, 
8.0], [5.0, 3.0, 8.0]]], [[[3.0, 3.0, 1.0], [7.0, 5.0, 1.0], [4.0, 8.0, 3.0]], [[3.0, 4.0, 2.0], [3.0, 4.0, 4.0], [5.0, 3.0, 
8.0]], [[2.0, 1.0, 2.0], [1.0, 3.0, 5.0], [4.0, 5.0, 8.0]]]]))
      val=numpy.array([[[[3.0, 2.0, 2.0]], [[2.0, 3.0, 6.0]]], [[[7.0, 3.0, 5.0]], [[3.0, 5.0, 5.0]]], [[[5.0, 1.0, 7.0]], 
[[5.0, 2.0, 7.0]]]])
      arg[:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 3.0], [3.0, 2.0, 2.0], [3.0, 4.0, 2.0]], [[5.0, 2.0, 6.0], [2.0, 3.0, 6.0], [4.0, 
3.0, 4.0]], [[8.0, 1.0, 7.0], [4.0, 6.0, 8.0], [7.0, 8.0, 4.0]]], [[[4.0, 1.0, 2.0], [7.0, 3.0, 5.0], [3.0, 3.0, 6.0]], [[8.0, 
3.0, 8.0], [3.0, 5.0, 5.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 3.0], [5.0, 1.0, 6.0], [3.0, 1.0, 2.0]]], [[[1.0, 3.0, 5.0], [5.0, 
1.0, 7.0], [4.0, 5.0, 8.0]], [[1.0, 8.0, 8.0], [5.0, 2.0, 7.0], [6.0, 8.0, 6.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 4.0], [8.0, 4.0, 
5.0]]], [[[4.0, 6.0, 1.0], [4.0, 1.0, 7.0], [7.0, 7.0, 5.0]], [[8.0, 3.0, 7.0], [8.0, 5.0, 3.0], [4.0, 6.0, 2.0]], [[5.0, 1.0, 
5.0], [1.0, 8.0, 6.0], [3.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 2.0], [3.0, 2.0, 2.0], [4.0, 7.0, 5.0]], 
[[4.0, 3.0, 7.0], [2.0, 3.0, 6.0], [4.0, 3.0, 2.0]], [[6.0, 1.0, 7.0], [1.0, 3.0, 5.0], [1.0, 1.0, 6.0]]], [[[6.0, 8.0, 7.0], 
[7.0, 3.0, 5.0], [4.0, 1.0, 6.0]], [[3.0, 4.0, 1.0], [3.0, 5.0, 5.0], [2.0, 5.0, 4.0]], [[1.0, 7.0, 7.0], [6.0, 8.0, 7.0], 
[6.0, 5.0, 6.0]]], [[[2.0, 6.0, 7.0], [5.0, 1.0, 7.0], [7.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [5.0, 2.0, 7.0], [4.0, 7.0, 4.0]], 
[[7.0, 4.0, 4.0], [8.0, 1.0, 8.0], [5.0, 3.0, 8.0]]], [[[3.0, 3.0, 1.0], [7.0, 5.0, 1.0], [4.0, 8.0, 3.0]], [[3.0, 4.0, 2.0], 
[3.0, 4.0, 4.0], [5.0, 3.0, 8.0]], [[2.0, 1.0, 2.0], [1.0, 3.0, 5.0], [4.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UURN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 3.0], [5.0, 4.0, 7.0], [8.0, 3.0, 6.0]], [[4.0, 4.0, 5.0], [4.0, 5.0, 6.0], [7.0, 3.0, 
2.0]], [[8.0, 7.0, 1.0], [4.0, 5.0, 4.0], [2.0, 7.0, 4.0]]], [[[6.0, 3.0, 2.0], [5.0, 5.0, 7.0], [5.0, 6.0, 3.0]], [[7.0, 6.0, 
7.0], [5.0, 6.0, 2.0], [6.0, 2.0, 5.0]], [[5.0, 6.0, 7.0], [3.0, 2.0, 3.0], [4.0, 2.0, 1.0]]], [[[6.0, 6.0, 3.0], [8.0, 5.0, 
1.0], [4.0, 6.0, 2.0]], [[8.0, 8.0, 2.0], [7.0, 8.0, 3.0], [7.0, 3.0, 4.0]], [[1.0, 7.0, 1.0], [4.0, 2.0, 6.0], [1.0, 8.0, 
7.0]]], [[[3.0, 6.0, 2.0], [1.0, 2.0, 3.0], [1.0, 4.0, 6.0]], [[7.0, 4.0, 1.0], [7.0, 4.0, 6.0], [6.0, 8.0, 5.0]], [[2.0, 7.0, 
3.0], [7.0, 7.0, 6.0], [4.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 7.0], [3.0, 7.0, 3.0], [5.0, 2.0, 4.0]], [[3.0, 6.0, 1.0], [5.0, 8.0, 
5.0], [4.0, 6.0, 4.0]], [[4.0, 3.0, 7.0], [7.0, 8.0, 8.0], [8.0, 4.0, 7.0]]], [[[5.0, 6.0, 5.0], [4.0, 3.0, 1.0], [6.0, 1.0, 
5.0]], [[7.0, 8.0, 5.0], [5.0, 2.0, 3.0], [8.0, 2.0, 8.0]], [[1.0, 3.0, 8.0], [2.0, 5.0, 8.0], [7.0, 4.0, 5.0]]], [[[5.0, 5.0, 
2.0], [2.0, 3.0, 7.0], [6.0, 4.0, 3.0]], [[2.0, 6.0, 1.0], [7.0, 3.0, 1.0], [5.0, 2.0, 8.0]], [[7.0, 5.0, 1.0], [6.0, 3.0, 
8.0], [6.0, 8.0, 8.0]]], [[[4.0, 2.0, 4.0], [8.0, 1.0, 6.0], [1.0, 7.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 6.0, 5.0], [4.0, 4.0, 
1.0]], [[7.0, 7.0, 4.0], [3.0, 6.0, 1.0], [8.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0, 8.0, 8.0]], [[3.0, 8.0, 6.0]]], [[[1.0, 4.0, 5.0]], [[2.0, 3.0, 1.0]]], [[[1.0, 7.0, 4.0]], 
[[6.0, 1.0, 1.0]]]]),self.functionspace)
      arg[:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 3.0], [5.0, 8.0, 8.0], [8.0, 3.0, 6.0]], [[4.0, 4.0, 5.0], [3.0, 8.0, 6.0], [7.0, 
3.0, 2.0]], [[8.0, 7.0, 1.0], [4.0, 5.0, 4.0], [2.0, 7.0, 4.0]]], [[[6.0, 3.0, 2.0], [1.0, 4.0, 5.0], [5.0, 6.0, 3.0]], [[7.0, 
6.0, 7.0], [2.0, 3.0, 1.0], [6.0, 2.0, 5.0]], [[5.0, 6.0, 7.0], [3.0, 2.0, 3.0], [4.0, 2.0, 1.0]]], [[[6.0, 6.0, 3.0], [1.0, 
7.0, 4.0], [4.0, 6.0, 2.0]], [[8.0, 8.0, 2.0], [6.0, 1.0, 1.0], [7.0, 3.0, 4.0]], [[1.0, 7.0, 1.0], [4.0, 2.0, 6.0], [1.0, 8.0, 
7.0]]], [[[3.0, 6.0, 2.0], [1.0, 2.0, 3.0], [1.0, 4.0, 6.0]], [[7.0, 4.0, 1.0], [7.0, 4.0, 6.0], [6.0, 8.0, 5.0]], [[2.0, 7.0, 
3.0], [7.0, 7.0, 6.0], [4.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 7.0], [5.0, 8.0, 8.0], [5.0, 2.0, 4.0]], 
[[3.0, 6.0, 1.0], [3.0, 8.0, 6.0], [4.0, 6.0, 4.0]], [[4.0, 3.0, 7.0], [7.0, 8.0, 8.0], [8.0, 4.0, 7.0]]], [[[5.0, 6.0, 5.0], 
[1.0, 4.0, 5.0], [6.0, 1.0, 5.0]], [[7.0, 8.0, 5.0], [2.0, 3.0, 1.0], [8.0, 2.0, 8.0]], [[1.0, 3.0, 8.0], [2.0, 5.0, 8.0], 
[7.0, 4.0, 5.0]]], [[[5.0, 5.0, 2.0], [1.0, 7.0, 4.0], [6.0, 4.0, 3.0]], [[2.0, 6.0, 1.0], [6.0, 1.0, 1.0], [5.0, 2.0, 8.0]], 
[[7.0, 5.0, 1.0], [6.0, 3.0, 8.0], [6.0, 8.0, 8.0]]], [[[4.0, 2.0, 4.0], [8.0, 1.0, 6.0], [1.0, 7.0, 6.0]], [[8.0, 2.0, 7.0], 
[3.0, 6.0, 5.0], [4.0, 4.0, 1.0]], [[7.0, 7.0, 4.0], [3.0, 6.0, 1.0], [8.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UURN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 7.0], [7.0, 7.0, 6.0], [5.0, 7.0, 5.0]], [[2.0, 6.0, 2.0], [5.0, 3.0, 6.0], [7.0, 2.0, 
5.0]], [[5.0, 2.0, 1.0], [5.0, 1.0, 1.0], [2.0, 4.0, 1.0]]], [[[7.0, 5.0, 8.0], [7.0, 6.0, 2.0], [3.0, 1.0, 7.0]], [[5.0, 1.0, 
7.0], [4.0, 7.0, 6.0], [4.0, 2.0, 7.0]], [[5.0, 3.0, 6.0], [8.0, 6.0, 7.0], [3.0, 3.0, 5.0]]], [[[2.0, 8.0, 1.0], [3.0, 2.0, 
5.0], [1.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [7.0, 1.0, 1.0], [3.0, 1.0, 2.0]], [[6.0, 1.0, 7.0], [5.0, 5.0, 4.0], [8.0, 4.0, 
1.0]]], [[[8.0, 8.0, 2.0], [6.0, 7.0, 4.0], [6.0, 8.0, 4.0]], [[1.0, 6.0, 8.0], [1.0, 5.0, 3.0], [7.0, 7.0, 1.0]], [[3.0, 1.0, 
7.0], [8.0, 4.0, 1.0], [1.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 7.0], [5.0, 8.0, 2.0], [4.0, 2.0, 7.0]], [[5.0, 8.0, 5.0], [6.0, 7.0, 
4.0], [6.0, 3.0, 8.0]], [[4.0, 5.0, 8.0], [2.0, 6.0, 1.0], [6.0, 6.0, 4.0]]], [[[8.0, 1.0, 4.0], [5.0, 3.0, 4.0], [2.0, 3.0, 
8.0]], [[5.0, 8.0, 7.0], [2.0, 7.0, 6.0], [7.0, 5.0, 4.0]], [[4.0, 6.0, 4.0], [4.0, 2.0, 2.0], [3.0, 6.0, 2.0]]], [[[7.0, 1.0, 
1.0], [7.0, 1.0, 5.0], [6.0, 7.0, 3.0]], [[7.0, 3.0, 5.0], [1.0, 8.0, 3.0], [1.0, 3.0, 3.0]], [[1.0, 2.0, 7.0], [7.0, 3.0, 
6.0], [2.0, 4.0, 8.0]]], [[[8.0, 6.0, 5.0], [8.0, 2.0, 5.0], [3.0, 3.0, 6.0]], [[3.0, 7.0, 1.0], [1.0, 2.0, 5.0], [3.0, 5.0, 
5.0]], [[1.0, 7.0, 5.0], [5.0, 8.0, 4.0], [5.0, 4.0, 7.0]]]]))
      val=Data(numpy.array([[[[4.0, 4.0, 2.0]], [[4.0, 2.0, 3.0]]], [[[6.0, 3.0, 4.0]], [[4.0, 6.0, 1.0]]], [[[2.0, 7.0, 7.0]], 
[[2.0, 5.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 7.0]], [[5.0, 6.0, 3.0]]], [[[3.0, 2.0, 6.0]], [[4.0, 4.0, 7.0]]], [[[1.0, 
3.0, 1.0]], [[2.0, 5.0, 6.0]]]]))
      arg[:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 7.0], [4.0, 4.0, 2.0], [5.0, 7.0, 5.0]], [[2.0, 6.0, 2.0], [4.0, 2.0, 3.0], [7.0, 
2.0, 5.0]], [[5.0, 2.0, 1.0], [5.0, 1.0, 1.0], [2.0, 4.0, 1.0]]], [[[7.0, 5.0, 8.0], [6.0, 3.0, 4.0], [3.0, 1.0, 7.0]], [[5.0, 
1.0, 7.0], [4.0, 6.0, 1.0], [4.0, 2.0, 7.0]], [[5.0, 3.0, 6.0], [8.0, 6.0, 7.0], [3.0, 3.0, 5.0]]], [[[2.0, 8.0, 1.0], [2.0, 
7.0, 7.0], [1.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [2.0, 5.0, 8.0], [3.0, 1.0, 2.0]], [[6.0, 1.0, 7.0], [5.0, 5.0, 4.0], [8.0, 4.0, 
1.0]]], [[[8.0, 8.0, 2.0], [6.0, 7.0, 4.0], [6.0, 8.0, 4.0]], [[1.0, 6.0, 8.0], [1.0, 5.0, 3.0], [7.0, 7.0, 1.0]], [[3.0, 1.0, 
7.0], [8.0, 4.0, 1.0], [1.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 7.0], [3.0, 6.0, 7.0], [4.0, 2.0, 7.0]], 
[[5.0, 8.0, 5.0], [5.0, 6.0, 3.0], [6.0, 3.0, 8.0]], [[4.0, 5.0, 8.0], [2.0, 6.0, 1.0], [6.0, 6.0, 4.0]]], [[[8.0, 1.0, 4.0], 
[3.0, 2.0, 6.0], [2.0, 3.0, 8.0]], [[5.0, 8.0, 7.0], [4.0, 4.0, 7.0], [7.0, 5.0, 4.0]], [[4.0, 6.0, 4.0], [4.0, 2.0, 2.0], 
[3.0, 6.0, 2.0]]], [[[7.0, 1.0, 1.0], [1.0, 3.0, 1.0], [6.0, 7.0, 3.0]], [[7.0, 3.0, 5.0], [2.0, 5.0, 6.0], [1.0, 3.0, 3.0]], 
[[1.0, 2.0, 7.0], [7.0, 3.0, 6.0], [2.0, 4.0, 8.0]]], [[[8.0, 6.0, 5.0], [8.0, 2.0, 5.0], [3.0, 3.0, 6.0]], [[3.0, 7.0, 1.0], 
[1.0, 2.0, 5.0], [3.0, 5.0, 5.0]], [[1.0, 7.0, 5.0], [5.0, 8.0, 4.0], [5.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UURN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 3.0], [3.0, 3.0, 2.0], [8.0, 1.0, 6.0]], [[4.0, 2.0, 4.0], [1.0, 6.0, 3.0], [6.0, 8.0, 
3.0]], [[8.0, 5.0, 7.0], [7.0, 1.0, 2.0], [6.0, 3.0, 3.0]]], [[[1.0, 2.0, 2.0], [8.0, 6.0, 6.0], [6.0, 6.0, 5.0]], [[3.0, 2.0, 
6.0], [8.0, 1.0, 6.0], [4.0, 2.0, 2.0]], [[1.0, 2.0, 5.0], [2.0, 4.0, 1.0], [7.0, 2.0, 6.0]]], [[[8.0, 1.0, 8.0], [3.0, 5.0, 
4.0], [4.0, 8.0, 7.0]], [[2.0, 1.0, 4.0], [2.0, 6.0, 1.0], [4.0, 6.0, 3.0]], [[4.0, 1.0, 6.0], [2.0, 3.0, 1.0], [8.0, 6.0, 
1.0]]], [[[4.0, 1.0, 2.0], [7.0, 4.0, 5.0], [1.0, 5.0, 1.0]], [[3.0, 1.0, 6.0], [1.0, 1.0, 4.0], [4.0, 1.0, 4.0]], [[4.0, 8.0, 
6.0], [2.0, 3.0, 7.0], [7.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 4.0], [1.0, 2.0, 2.0], [1.0, 8.0, 3.0]], [[2.0, 7.0, 3.0], [5.0, 5.0, 
7.0], [6.0, 4.0, 6.0]], [[3.0, 7.0, 1.0], [7.0, 4.0, 3.0], [7.0, 5.0, 1.0]]], [[[1.0, 2.0, 6.0], [2.0, 8.0, 6.0], [3.0, 3.0, 
3.0]], [[1.0, 3.0, 3.0], [2.0, 3.0, 5.0], [7.0, 8.0, 8.0]], [[2.0, 5.0, 6.0], [3.0, 4.0, 2.0], [8.0, 1.0, 1.0]]], [[[1.0, 3.0, 
6.0], [2.0, 4.0, 5.0], [8.0, 8.0, 4.0]], [[4.0, 6.0, 7.0], [1.0, 4.0, 4.0], [4.0, 3.0, 1.0]], [[7.0, 3.0, 8.0], [6.0, 5.0, 
2.0], [1.0, 4.0, 8.0]]], [[[5.0, 2.0, 8.0], [5.0, 7.0, 6.0], [2.0, 8.0, 6.0]], [[8.0, 6.0, 2.0], [1.0, 3.0, 4.0], [1.0, 1.0, 
1.0]], [[6.0, 7.0, 2.0], [4.0, 3.0, 1.0], [4.0, 1.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[2.0, 6.0, 6.0]], [[3.0, 2.0, 6.0]]], [[[2.0, 1.0, 6.0]], [[7.0, 8.0, 4.0]]], [[[3.0, 6.0, 
4.0]], [[6.0, 1.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 1.0, 1.0]], [[2.0, 2.0, 1.0]]], [[[5.0, 1.0, 4.0]], [[8.0, 7.0, 
7.0]]], [[[5.0, 2.0, 6.0]], [[2.0, 5.0, 8.0]]]])
      arg[:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 3.0], [2.0, 6.0, 6.0], [8.0, 1.0, 6.0]], [[4.0, 2.0, 4.0], [3.0, 2.0, 6.0], [6.0, 
8.0, 3.0]], [[8.0, 5.0, 7.0], [7.0, 1.0, 2.0], [6.0, 3.0, 3.0]]], [[[1.0, 2.0, 2.0], [2.0, 1.0, 6.0], [6.0, 6.0, 5.0]], [[3.0, 
2.0, 6.0], [7.0, 8.0, 4.0], [4.0, 2.0, 2.0]], [[1.0, 2.0, 5.0], [2.0, 4.0, 1.0], [7.0, 2.0, 6.0]]], [[[8.0, 1.0, 8.0], [3.0, 
6.0, 4.0], [4.0, 8.0, 7.0]], [[2.0, 1.0, 4.0], [6.0, 1.0, 1.0], [4.0, 6.0, 3.0]], [[4.0, 1.0, 6.0], [2.0, 3.0, 1.0], [8.0, 6.0, 
1.0]]], [[[4.0, 1.0, 2.0], [7.0, 4.0, 5.0], [1.0, 5.0, 1.0]], [[3.0, 1.0, 6.0], [1.0, 1.0, 4.0], [4.0, 1.0, 4.0]], [[4.0, 8.0, 
6.0], [2.0, 3.0, 7.0], [7.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 4.0], [4.0, 1.0, 1.0], [1.0, 8.0, 3.0]], 
[[2.0, 7.0, 3.0], [2.0, 2.0, 1.0], [6.0, 4.0, 6.0]], [[3.0, 7.0, 1.0], [7.0, 4.0, 3.0], [7.0, 5.0, 1.0]]], [[[1.0, 2.0, 6.0], 
[5.0, 1.0, 4.0], [3.0, 3.0, 3.0]], [[1.0, 3.0, 3.0], [8.0, 7.0, 7.0], [7.0, 8.0, 8.0]], [[2.0, 5.0, 6.0], [3.0, 4.0, 2.0], 
[8.0, 1.0, 1.0]]], [[[1.0, 3.0, 6.0], [5.0, 2.0, 6.0], [8.0, 8.0, 4.0]], [[4.0, 6.0, 7.0], [2.0, 5.0, 8.0], [4.0, 3.0, 1.0]], 
[[7.0, 3.0, 8.0], [6.0, 5.0, 2.0], [1.0, 4.0, 8.0]]], [[[5.0, 2.0, 8.0], [5.0, 7.0, 6.0], [2.0, 8.0, 6.0]], [[8.0, 6.0, 2.0], 
[1.0, 3.0, 4.0], [1.0, 1.0, 1.0]], [[6.0, 7.0, 2.0], [4.0, 3.0, 1.0], [4.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPCN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 3.0], [5.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[5.0, 4.0, 1.0], [8.0, 5.0, 3.0], [6.0, 3.0, 
7.0]], [[1.0, 2.0, 1.0], [3.0, 2.0, 1.0], [4.0, 4.0, 6.0]]], [[[8.0, 5.0, 8.0], [8.0, 5.0, 2.0], [2.0, 6.0, 2.0]], [[8.0, 7.0, 
3.0], [8.0, 5.0, 6.0], [4.0, 2.0, 5.0]], [[2.0, 2.0, 1.0], [5.0, 4.0, 2.0], [8.0, 6.0, 3.0]]], [[[3.0, 5.0, 4.0], [3.0, 3.0, 
8.0], [6.0, 7.0, 3.0]], [[3.0, 3.0, 4.0], [4.0, 1.0, 8.0], [3.0, 4.0, 8.0]], [[5.0, 1.0, 2.0], [4.0, 5.0, 7.0], [5.0, 3.0, 
5.0]]], [[[4.0, 5.0, 6.0], [7.0, 1.0, 6.0], [2.0, 2.0, 7.0]], [[4.0, 7.0, 4.0], [5.0, 5.0, 5.0], [3.0, 1.0, 5.0]], [[4.0, 3.0, 
1.0], [3.0, 5.0, 4.0], [3.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 7.0], [2.0, 7.0, 4.0], [5.0, 5.0, 3.0]], [[1.0, 4.0, 1.0], [3.0, 5.0, 
2.0], [4.0, 8.0, 8.0]], [[7.0, 7.0, 2.0], [2.0, 6.0, 1.0], [8.0, 8.0, 7.0]]], [[[8.0, 7.0, 4.0], [7.0, 7.0, 6.0], [3.0, 2.0, 
4.0]], [[2.0, 8.0, 5.0], [5.0, 7.0, 6.0], [3.0, 4.0, 4.0]], [[8.0, 7.0, 7.0], [7.0, 4.0, 1.0], [4.0, 4.0, 8.0]]], [[[7.0, 4.0, 
5.0], [2.0, 4.0, 3.0], [2.0, 3.0, 8.0]], [[3.0, 1.0, 5.0], [1.0, 2.0, 3.0], [1.0, 3.0, 4.0]], [[8.0, 1.0, 3.0], [1.0, 4.0, 
5.0], [1.0, 2.0, 1.0]]], [[[6.0, 2.0, 4.0], [5.0, 8.0, 6.0], [8.0, 2.0, 6.0]], [[4.0, 6.0, 6.0], [1.0, 2.0, 2.0], [2.0, 2.0, 
6.0]], [[5.0, 8.0, 2.0], [5.0, 1.0, 6.0], [2.0, 5.0, 2.0]]]]))
      val=numpy.array([[[6.0, 5.0, 4.0], [8.0, 7.0, 7.0], [1.0, 2.0, 2.0]], [[6.0, 2.0, 5.0], [2.0, 7.0, 4.0], [3.0, 1.0, 
8.0]]])
      arg[1:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 3.0], [5.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[5.0, 4.0, 1.0], [8.0, 5.0, 3.0], [6.0, 
3.0, 7.0]], [[1.0, 2.0, 1.0], [3.0, 2.0, 1.0], [4.0, 4.0, 6.0]]], [[[8.0, 5.0, 8.0], [8.0, 5.0, 2.0], [2.0, 6.0, 2.0]], [[6.0, 
5.0, 4.0], [8.0, 7.0, 7.0], [1.0, 2.0, 2.0]], [[2.0, 2.0, 1.0], [5.0, 4.0, 2.0], [8.0, 6.0, 3.0]]], [[[3.0, 5.0, 4.0], [3.0, 
3.0, 8.0], [6.0, 7.0, 3.0]], [[6.0, 2.0, 5.0], [2.0, 7.0, 4.0], [3.0, 1.0, 8.0]], [[5.0, 1.0, 2.0], [4.0, 5.0, 7.0], [5.0, 3.0, 
5.0]]], [[[4.0, 5.0, 6.0], [7.0, 1.0, 6.0], [2.0, 2.0, 7.0]], [[4.0, 7.0, 4.0], [5.0, 5.0, 5.0], [3.0, 1.0, 5.0]], [[4.0, 3.0, 
1.0], [3.0, 5.0, 4.0], [3.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 7.0], [2.0, 7.0, 4.0], [5.0, 5.0, 3.0]], 
[[1.0, 4.0, 1.0], [3.0, 5.0, 2.0], [4.0, 8.0, 8.0]], [[7.0, 7.0, 2.0], [2.0, 6.0, 1.0], [8.0, 8.0, 7.0]]], [[[8.0, 7.0, 4.0], 
[7.0, 7.0, 6.0], [3.0, 2.0, 4.0]], [[6.0, 5.0, 4.0], [8.0, 7.0, 7.0], [1.0, 2.0, 2.0]], [[8.0, 7.0, 7.0], [7.0, 4.0, 1.0], 
[4.0, 4.0, 8.0]]], [[[7.0, 4.0, 5.0], [2.0, 4.0, 3.0], [2.0, 3.0, 8.0]], [[6.0, 2.0, 5.0], [2.0, 7.0, 4.0], [3.0, 1.0, 8.0]], 
[[8.0, 1.0, 3.0], [1.0, 4.0, 5.0], [1.0, 2.0, 1.0]]], [[[6.0, 2.0, 4.0], [5.0, 8.0, 6.0], [8.0, 2.0, 6.0]], [[4.0, 6.0, 6.0], 
[1.0, 2.0, 2.0], [2.0, 2.0, 6.0]], [[5.0, 8.0, 2.0], [5.0, 1.0, 6.0], [2.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPCN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 4.0], [7.0, 6.0, 2.0], [7.0, 7.0, 3.0]], [[1.0, 8.0, 4.0], [1.0, 1.0, 7.0], [7.0, 8.0, 
4.0]], [[7.0, 6.0, 3.0], [4.0, 2.0, 7.0], [6.0, 1.0, 4.0]]], [[[1.0, 4.0, 4.0], [6.0, 3.0, 2.0], [3.0, 3.0, 8.0]], [[7.0, 1.0, 
1.0], [5.0, 5.0, 8.0], [4.0, 4.0, 4.0]], [[6.0, 7.0, 3.0], [7.0, 1.0, 7.0], [1.0, 8.0, 8.0]]], [[[4.0, 8.0, 6.0], [2.0, 4.0, 
5.0], [2.0, 4.0, 3.0]], [[1.0, 5.0, 7.0], [7.0, 8.0, 1.0], [2.0, 1.0, 1.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 2.0], [1.0, 6.0, 
4.0]]], [[[2.0, 1.0, 2.0], [8.0, 7.0, 8.0], [5.0, 3.0, 8.0]], [[6.0, 3.0, 3.0], [7.0, 5.0, 7.0], [7.0, 1.0, 3.0]], [[8.0, 2.0, 
3.0], [6.0, 3.0, 5.0], [7.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 2.0], [7.0, 1.0, 3.0], [6.0, 1.0, 3.0]], [[2.0, 4.0, 2.0], [3.0, 6.0, 
5.0], [5.0, 2.0, 3.0]], [[4.0, 8.0, 7.0], [2.0, 4.0, 3.0], [5.0, 5.0, 3.0]]], [[[3.0, 7.0, 1.0], [6.0, 1.0, 5.0], [8.0, 8.0, 
3.0]], [[8.0, 1.0, 8.0], [8.0, 6.0, 8.0], [5.0, 3.0, 1.0]], [[4.0, 6.0, 3.0], [7.0, 6.0, 2.0], [6.0, 7.0, 5.0]]], [[[7.0, 3.0, 
2.0], [5.0, 7.0, 6.0], [1.0, 3.0, 7.0]], [[5.0, 7.0, 1.0], [2.0, 3.0, 2.0], [3.0, 6.0, 1.0]], [[2.0, 7.0, 7.0], [8.0, 1.0, 
6.0], [5.0, 7.0, 6.0]]], [[[7.0, 2.0, 7.0], [1.0, 6.0, 7.0], [1.0, 2.0, 7.0]], [[2.0, 5.0, 1.0], [6.0, 1.0, 1.0], [1.0, 3.0, 
2.0]], [[5.0, 7.0, 1.0], [2.0, 4.0, 3.0], [4.0, 3.0, 6.0]]]]))
      val=Data(numpy.array([[[2.0, 4.0, 5.0], [1.0, 3.0, 8.0], [6.0, 4.0, 1.0]], [[8.0, 6.0, 1.0], [3.0, 8.0, 4.0], [6.0, 7.0, 
8.0]]]),self.functionspace)
      arg[1:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [7.0, 6.0, 2.0], [7.0, 7.0, 3.0]], [[1.0, 8.0, 4.0], [1.0, 1.0, 7.0], [7.0, 
8.0, 4.0]], [[7.0, 6.0, 3.0], [4.0, 2.0, 7.0], [6.0, 1.0, 4.0]]], [[[1.0, 4.0, 4.0], [6.0, 3.0, 2.0], [3.0, 3.0, 8.0]], [[2.0, 
4.0, 5.0], [1.0, 3.0, 8.0], [6.0, 4.0, 1.0]], [[6.0, 7.0, 3.0], [7.0, 1.0, 7.0], [1.0, 8.0, 8.0]]], [[[4.0, 8.0, 6.0], [2.0, 
4.0, 5.0], [2.0, 4.0, 3.0]], [[8.0, 6.0, 1.0], [3.0, 8.0, 4.0], [6.0, 7.0, 8.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 2.0], [1.0, 6.0, 
4.0]]], [[[2.0, 1.0, 2.0], [8.0, 7.0, 8.0], [5.0, 3.0, 8.0]], [[6.0, 3.0, 3.0], [7.0, 5.0, 7.0], [7.0, 1.0, 3.0]], [[8.0, 2.0, 
3.0], [6.0, 3.0, 5.0], [7.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 2.0], [7.0, 1.0, 3.0], [6.0, 1.0, 3.0]], 
[[2.0, 4.0, 2.0], [3.0, 6.0, 5.0], [5.0, 2.0, 3.0]], [[4.0, 8.0, 7.0], [2.0, 4.0, 3.0], [5.0, 5.0, 3.0]]], [[[3.0, 7.0, 1.0], 
[6.0, 1.0, 5.0], [8.0, 8.0, 3.0]], [[2.0, 4.0, 5.0], [1.0, 3.0, 8.0], [6.0, 4.0, 1.0]], [[4.0, 6.0, 3.0], [7.0, 6.0, 2.0], 
[6.0, 7.0, 5.0]]], [[[7.0, 3.0, 2.0], [5.0, 7.0, 6.0], [1.0, 3.0, 7.0]], [[8.0, 6.0, 1.0], [3.0, 8.0, 4.0], [6.0, 7.0, 8.0]], 
[[2.0, 7.0, 7.0], [8.0, 1.0, 6.0], [5.0, 7.0, 6.0]]], [[[7.0, 2.0, 7.0], [1.0, 6.0, 7.0], [1.0, 2.0, 7.0]], [[2.0, 5.0, 1.0], 
[6.0, 1.0, 1.0], [1.0, 3.0, 2.0]], [[5.0, 7.0, 1.0], [2.0, 4.0, 3.0], [4.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPCN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 3.0], [5.0, 5.0, 4.0], [6.0, 3.0, 6.0]], [[2.0, 1.0, 6.0], [5.0, 1.0, 3.0], [8.0, 3.0, 
3.0]], [[7.0, 6.0, 2.0], [8.0, 2.0, 3.0], [4.0, 3.0, 4.0]]], [[[6.0, 2.0, 8.0], [1.0, 2.0, 5.0], [7.0, 7.0, 2.0]], [[7.0, 6.0, 
4.0], [7.0, 8.0, 8.0], [6.0, 2.0, 7.0]], [[7.0, 8.0, 6.0], [1.0, 1.0, 1.0], [7.0, 1.0, 7.0]]], [[[5.0, 3.0, 6.0], [4.0, 5.0, 
7.0], [3.0, 6.0, 5.0]], [[4.0, 1.0, 5.0], [4.0, 1.0, 5.0], [1.0, 7.0, 8.0]], [[1.0, 1.0, 6.0], [1.0, 7.0, 8.0], [7.0, 6.0, 
4.0]]], [[[5.0, 7.0, 6.0], [2.0, 2.0, 4.0], [4.0, 2.0, 5.0]], [[8.0, 5.0, 7.0], [4.0, 7.0, 6.0], [6.0, 7.0, 2.0]], [[7.0, 4.0, 
7.0], [6.0, 6.0, 5.0], [7.0, 4.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 2.0], [8.0, 1.0, 7.0], [7.0, 6.0, 5.0]], [[4.0, 5.0, 6.0], [3.0, 8.0, 
8.0], [3.0, 4.0, 1.0]], [[8.0, 2.0, 2.0], [1.0, 7.0, 5.0], [2.0, 1.0, 8.0]]], [[[8.0, 3.0, 3.0], [4.0, 1.0, 7.0], [3.0, 5.0, 
4.0]], [[5.0, 4.0, 1.0], [8.0, 6.0, 7.0], [2.0, 1.0, 1.0]], [[5.0, 7.0, 7.0], [7.0, 6.0, 6.0], [4.0, 1.0, 5.0]]], [[[8.0, 2.0, 
1.0], [3.0, 6.0, 7.0], [1.0, 8.0, 2.0]], [[8.0, 6.0, 8.0], [3.0, 1.0, 2.0], [7.0, 3.0, 3.0]], [[6.0, 1.0, 7.0], [4.0, 1.0, 
8.0], [6.0, 1.0, 5.0]]], [[[6.0, 1.0, 5.0], [7.0, 7.0, 1.0], [6.0, 5.0, 2.0]], [[3.0, 2.0, 4.0], [3.0, 8.0, 7.0], [4.0, 2.0, 
1.0]], [[6.0, 6.0, 3.0], [1.0, 8.0, 6.0], [7.0, 3.0, 1.0]]]]))
      val=Data(numpy.array([[[1.0, 7.0, 2.0], [2.0, 5.0, 5.0], [1.0, 8.0, 5.0]], [[1.0, 6.0, 7.0], [7.0, 1.0, 5.0], [5.0, 5.0, 
7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 3.0, 3.0], [4.0, 5.0, 5.0], [8.0, 6.0, 4.0]], [[5.0, 2.0, 6.0], [8.0, 7.0, 8.0], 
[7.0, 2.0, 1.0]]]))
      arg[1:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 3.0], [5.0, 5.0, 4.0], [6.0, 3.0, 6.0]], [[2.0, 1.0, 6.0], [5.0, 1.0, 3.0], [8.0, 
3.0, 3.0]], [[7.0, 6.0, 2.0], [8.0, 2.0, 3.0], [4.0, 3.0, 4.0]]], [[[6.0, 2.0, 8.0], [1.0, 2.0, 5.0], [7.0, 7.0, 2.0]], [[1.0, 
7.0, 2.0], [2.0, 5.0, 5.0], [1.0, 8.0, 5.0]], [[7.0, 8.0, 6.0], [1.0, 1.0, 1.0], [7.0, 1.0, 7.0]]], [[[5.0, 3.0, 6.0], [4.0, 
5.0, 7.0], [3.0, 6.0, 5.0]], [[1.0, 6.0, 7.0], [7.0, 1.0, 5.0], [5.0, 5.0, 7.0]], [[1.0, 1.0, 6.0], [1.0, 7.0, 8.0], [7.0, 6.0, 
4.0]]], [[[5.0, 7.0, 6.0], [2.0, 2.0, 4.0], [4.0, 2.0, 5.0]], [[8.0, 5.0, 7.0], [4.0, 7.0, 6.0], [6.0, 7.0, 2.0]], [[7.0, 4.0, 
7.0], [6.0, 6.0, 5.0], [7.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 2.0], [8.0, 1.0, 7.0], [7.0, 6.0, 5.0]], 
[[4.0, 5.0, 6.0], [3.0, 8.0, 8.0], [3.0, 4.0, 1.0]], [[8.0, 2.0, 2.0], [1.0, 7.0, 5.0], [2.0, 1.0, 8.0]]], [[[8.0, 3.0, 3.0], 
[4.0, 1.0, 7.0], [3.0, 5.0, 4.0]], [[2.0, 3.0, 3.0], [4.0, 5.0, 5.0], [8.0, 6.0, 4.0]], [[5.0, 7.0, 7.0], [7.0, 6.0, 6.0], 
[4.0, 1.0, 5.0]]], [[[8.0, 2.0, 1.0], [3.0, 6.0, 7.0], [1.0, 8.0, 2.0]], [[5.0, 2.0, 6.0], [8.0, 7.0, 8.0], [7.0, 2.0, 1.0]], 
[[6.0, 1.0, 7.0], [4.0, 1.0, 8.0], [6.0, 1.0, 5.0]]], [[[6.0, 1.0, 5.0], [7.0, 7.0, 1.0], [6.0, 5.0, 2.0]], [[3.0, 2.0, 4.0], 
[3.0, 8.0, 7.0], [4.0, 2.0, 1.0]], [[6.0, 6.0, 3.0], [1.0, 8.0, 6.0], [7.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPCN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 1.0], [5.0, 5.0, 7.0], [2.0, 6.0, 4.0]], [[5.0, 3.0, 2.0], [1.0, 3.0, 4.0], [3.0, 1.0, 
6.0]], [[1.0, 1.0, 1.0], [7.0, 1.0, 7.0], [1.0, 4.0, 6.0]]], [[[6.0, 4.0, 3.0], [8.0, 8.0, 5.0], [2.0, 4.0, 8.0]], [[6.0, 2.0, 
4.0], [5.0, 3.0, 4.0], [1.0, 6.0, 4.0]], [[3.0, 3.0, 1.0], [5.0, 2.0, 8.0], [4.0, 6.0, 8.0]]], [[[6.0, 4.0, 3.0], [3.0, 2.0, 
6.0], [6.0, 1.0, 3.0]], [[1.0, 6.0, 2.0], [6.0, 4.0, 5.0], [5.0, 4.0, 5.0]], [[2.0, 1.0, 7.0], [4.0, 8.0, 4.0], [8.0, 7.0, 
4.0]]], [[[1.0, 1.0, 3.0], [3.0, 1.0, 3.0], [4.0, 4.0, 5.0]], [[2.0, 8.0, 5.0], [8.0, 2.0, 5.0], [8.0, 7.0, 3.0]], [[8.0, 8.0, 
5.0], [5.0, 3.0, 6.0], [2.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 2.0], [1.0, 4.0, 5.0], [8.0, 5.0, 8.0]], [[1.0, 1.0, 4.0], [3.0, 5.0, 
6.0], [3.0, 4.0, 6.0]], [[6.0, 2.0, 6.0], [3.0, 3.0, 7.0], [1.0, 1.0, 3.0]]], [[[2.0, 2.0, 7.0], [6.0, 8.0, 8.0], [4.0, 3.0, 
1.0]], [[6.0, 8.0, 5.0], [4.0, 5.0, 3.0], [7.0, 4.0, 5.0]], [[7.0, 5.0, 5.0], [7.0, 5.0, 5.0], [1.0, 5.0, 3.0]]], [[[1.0, 4.0, 
2.0], [2.0, 6.0, 7.0], [8.0, 3.0, 7.0]], [[6.0, 8.0, 2.0], [5.0, 4.0, 5.0], [4.0, 2.0, 8.0]], [[6.0, 2.0, 6.0], [3.0, 7.0, 
1.0], [6.0, 4.0, 4.0]]], [[[6.0, 7.0, 7.0], [4.0, 8.0, 5.0], [3.0, 3.0, 7.0]], [[7.0, 1.0, 1.0], [3.0, 8.0, 7.0], [2.0, 7.0, 
1.0]], [[2.0, 2.0, 2.0], [7.0, 6.0, 4.0], [3.0, 5.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0, 5.0, 6.0], [1.0, 1.0, 5.0], [1.0, 4.0, 4.0]], [[8.0, 4.0, 2.0], [8.0, 4.0, 2.0], [4.0, 
6.0, 8.0]]])+(1.-msk_val)*numpy.array([[[1.0, 2.0, 6.0], [5.0, 1.0, 2.0], [4.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [6.0, 3.0, 6.0], 
[5.0, 4.0, 5.0]]])
      arg[1:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 1.0], [5.0, 5.0, 7.0], [2.0, 6.0, 4.0]], [[5.0, 3.0, 2.0], [1.0, 3.0, 4.0], [3.0, 
1.0, 6.0]], [[1.0, 1.0, 1.0], [7.0, 1.0, 7.0], [1.0, 4.0, 6.0]]], [[[6.0, 4.0, 3.0], [8.0, 8.0, 5.0], [2.0, 4.0, 8.0]], [[1.0, 
5.0, 6.0], [1.0, 1.0, 5.0], [1.0, 4.0, 4.0]], [[3.0, 3.0, 1.0], [5.0, 2.0, 8.0], [4.0, 6.0, 8.0]]], [[[6.0, 4.0, 3.0], [3.0, 
2.0, 6.0], [6.0, 1.0, 3.0]], [[8.0, 4.0, 2.0], [8.0, 4.0, 2.0], [4.0, 6.0, 8.0]], [[2.0, 1.0, 7.0], [4.0, 8.0, 4.0], [8.0, 7.0, 
4.0]]], [[[1.0, 1.0, 3.0], [3.0, 1.0, 3.0], [4.0, 4.0, 5.0]], [[2.0, 8.0, 5.0], [8.0, 2.0, 5.0], [8.0, 7.0, 3.0]], [[8.0, 8.0, 
5.0], [5.0, 3.0, 6.0], [2.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 2.0], [1.0, 4.0, 5.0], [8.0, 5.0, 8.0]], 
[[1.0, 1.0, 4.0], [3.0, 5.0, 6.0], [3.0, 4.0, 6.0]], [[6.0, 2.0, 6.0], [3.0, 3.0, 7.0], [1.0, 1.0, 3.0]]], [[[2.0, 2.0, 7.0], 
[6.0, 8.0, 8.0], [4.0, 3.0, 1.0]], [[1.0, 2.0, 6.0], [5.0, 1.0, 2.0], [4.0, 7.0, 1.0]], [[7.0, 5.0, 5.0], [7.0, 5.0, 5.0], 
[1.0, 5.0, 3.0]]], [[[1.0, 4.0, 2.0], [2.0, 6.0, 7.0], [8.0, 3.0, 7.0]], [[7.0, 2.0, 6.0], [6.0, 3.0, 6.0], [5.0, 4.0, 5.0]], 
[[6.0, 2.0, 6.0], [3.0, 7.0, 1.0], [6.0, 4.0, 4.0]]], [[[6.0, 7.0, 7.0], [4.0, 8.0, 5.0], [3.0, 3.0, 7.0]], [[7.0, 1.0, 1.0], 
[3.0, 8.0, 7.0], [2.0, 7.0, 1.0]], [[2.0, 2.0, 2.0], [7.0, 6.0, 4.0], [3.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 3.0], [5.0, 3.0, 4.0], [1.0, 4.0, 4.0]], [[8.0, 1.0, 2.0], [8.0, 4.0, 6.0], [7.0, 5.0, 
1.0]], [[5.0, 5.0, 1.0], [8.0, 4.0, 2.0], [2.0, 5.0, 5.0]]], [[[1.0, 2.0, 4.0], [2.0, 8.0, 8.0], [7.0, 5.0, 1.0]], [[4.0, 7.0, 
6.0], [4.0, 8.0, 5.0], [1.0, 5.0, 3.0]], [[4.0, 6.0, 3.0], [6.0, 8.0, 1.0], [4.0, 5.0, 8.0]]], [[[2.0, 8.0, 8.0], [4.0, 2.0, 
2.0], [7.0, 8.0, 6.0]], [[1.0, 5.0, 2.0], [6.0, 3.0, 1.0], [5.0, 5.0, 6.0]], [[3.0, 2.0, 2.0], [2.0, 1.0, 3.0], [6.0, 4.0, 
4.0]]], [[[5.0, 7.0, 4.0], [7.0, 6.0, 1.0], [1.0, 8.0, 6.0]], [[6.0, 8.0, 4.0], [4.0, 3.0, 1.0], [1.0, 6.0, 3.0]], [[6.0, 7.0, 
2.0], [1.0, 1.0, 1.0], [2.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 8.0], [3.0, 1.0, 5.0], [7.0, 5.0, 8.0]], [[5.0, 3.0, 3.0], [5.0, 3.0, 
6.0], [3.0, 4.0, 7.0]], [[5.0, 3.0, 1.0], [3.0, 4.0, 3.0], [6.0, 2.0, 2.0]]], [[[5.0, 8.0, 3.0], [4.0, 1.0, 5.0], [1.0, 8.0, 
1.0]], [[1.0, 7.0, 5.0], [8.0, 6.0, 4.0], [6.0, 3.0, 3.0]], [[5.0, 8.0, 8.0], [3.0, 5.0, 2.0], [3.0, 4.0, 5.0]]], [[[1.0, 3.0, 
2.0], [8.0, 3.0, 5.0], [2.0, 8.0, 7.0]], [[1.0, 5.0, 2.0], [3.0, 8.0, 2.0], [3.0, 1.0, 4.0]], [[8.0, 5.0, 5.0], [6.0, 2.0, 
5.0], [1.0, 4.0, 8.0]]], [[[7.0, 5.0, 3.0], [2.0, 4.0, 5.0], [6.0, 8.0, 5.0]], [[3.0, 7.0, 1.0], [4.0, 1.0, 3.0], [6.0, 6.0, 
5.0]], [[6.0, 2.0, 8.0], [8.0, 6.0, 6.0], [1.0, 8.0, 7.0]]]]))
      val=numpy.array([[[8.0, 1.0, 7.0], [5.0, 6.0, 1.0]], [[4.0, 4.0, 2.0], [8.0, 8.0, 7.0]]])
      arg[1:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 3.0], [5.0, 3.0, 4.0], [1.0, 4.0, 4.0]], [[8.0, 1.0, 2.0], [8.0, 4.0, 6.0], [7.0, 
5.0, 1.0]], [[5.0, 5.0, 1.0], [8.0, 4.0, 2.0], [2.0, 5.0, 5.0]]], [[[1.0, 2.0, 4.0], [2.0, 8.0, 8.0], [7.0, 5.0, 1.0]], [[8.0, 
1.0, 7.0], [5.0, 6.0, 1.0], [1.0, 5.0, 3.0]], [[4.0, 6.0, 3.0], [6.0, 8.0, 1.0], [4.0, 5.0, 8.0]]], [[[2.0, 8.0, 8.0], [4.0, 
2.0, 2.0], [7.0, 8.0, 6.0]], [[4.0, 4.0, 2.0], [8.0, 8.0, 7.0], [5.0, 5.0, 6.0]], [[3.0, 2.0, 2.0], [2.0, 1.0, 3.0], [6.0, 4.0, 
4.0]]], [[[5.0, 7.0, 4.0], [7.0, 6.0, 1.0], [1.0, 8.0, 6.0]], [[6.0, 8.0, 4.0], [4.0, 3.0, 1.0], [1.0, 6.0, 3.0]], [[6.0, 7.0, 
2.0], [1.0, 1.0, 1.0], [2.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 8.0], [3.0, 1.0, 5.0], [7.0, 5.0, 8.0]], 
[[5.0, 3.0, 3.0], [5.0, 3.0, 6.0], [3.0, 4.0, 7.0]], [[5.0, 3.0, 1.0], [3.0, 4.0, 3.0], [6.0, 2.0, 2.0]]], [[[5.0, 8.0, 3.0], 
[4.0, 1.0, 5.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 7.0], [5.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[5.0, 8.0, 8.0], [3.0, 5.0, 2.0], 
[3.0, 4.0, 5.0]]], [[[1.0, 3.0, 2.0], [8.0, 3.0, 5.0], [2.0, 8.0, 7.0]], [[4.0, 4.0, 2.0], [8.0, 8.0, 7.0], [3.0, 1.0, 4.0]], 
[[8.0, 5.0, 5.0], [6.0, 2.0, 5.0], [1.0, 4.0, 8.0]]], [[[7.0, 5.0, 3.0], [2.0, 4.0, 5.0], [6.0, 8.0, 5.0]], [[3.0, 7.0, 1.0], 
[4.0, 1.0, 3.0], [6.0, 6.0, 5.0]], [[6.0, 2.0, 8.0], [8.0, 6.0, 6.0], [1.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPUN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 3.0], [8.0, 1.0, 6.0], [3.0, 3.0, 5.0]], [[6.0, 4.0, 2.0], [7.0, 7.0, 7.0], [5.0, 2.0, 
4.0]], [[8.0, 6.0, 4.0], [3.0, 1.0, 4.0], [2.0, 4.0, 3.0]]], [[[7.0, 5.0, 2.0], [7.0, 5.0, 4.0], [4.0, 8.0, 2.0]], [[4.0, 8.0, 
5.0], [4.0, 5.0, 1.0], [6.0, 5.0, 8.0]], [[2.0, 4.0, 1.0], [8.0, 3.0, 5.0], [4.0, 8.0, 4.0]]], [[[3.0, 7.0, 2.0], [5.0, 3.0, 
3.0], [2.0, 8.0, 2.0]], [[3.0, 1.0, 4.0], [2.0, 8.0, 2.0], [6.0, 1.0, 6.0]], [[1.0, 3.0, 5.0], [8.0, 6.0, 8.0], [1.0, 3.0, 
4.0]]], [[[6.0, 6.0, 4.0], [7.0, 6.0, 1.0], [6.0, 1.0, 2.0]], [[8.0, 4.0, 1.0], [3.0, 6.0, 8.0], [1.0, 5.0, 6.0]], [[2.0, 2.0, 
3.0], [6.0, 4.0, 8.0], [3.0, 3.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 5.0], [1.0, 6.0, 5.0], [1.0, 7.0, 3.0]], [[3.0, 4.0, 6.0], [3.0, 4.0, 
5.0], [5.0, 6.0, 8.0]], [[4.0, 8.0, 3.0], [6.0, 4.0, 8.0], [5.0, 5.0, 5.0]]], [[[1.0, 4.0, 7.0], [7.0, 6.0, 5.0], [1.0, 6.0, 
4.0]], [[7.0, 7.0, 1.0], [3.0, 4.0, 6.0], [2.0, 3.0, 8.0]], [[6.0, 5.0, 4.0], [2.0, 2.0, 1.0], [7.0, 4.0, 1.0]]], [[[6.0, 1.0, 
7.0], [4.0, 8.0, 8.0], [2.0, 2.0, 5.0]], [[3.0, 1.0, 2.0], [4.0, 1.0, 2.0], [2.0, 5.0, 6.0]], [[5.0, 1.0, 2.0], [5.0, 8.0, 
4.0], [3.0, 7.0, 1.0]]], [[[7.0, 1.0, 6.0], [6.0, 5.0, 7.0], [6.0, 2.0, 8.0]], [[4.0, 2.0, 8.0], [2.0, 5.0, 5.0], [6.0, 3.0, 
5.0]], [[4.0, 2.0, 4.0], [2.0, 1.0, 1.0], [8.0, 4.0, 1.0]]]]))
      val=Data(numpy.array([[[4.0, 4.0, 6.0], [7.0, 1.0, 5.0]], [[7.0, 3.0, 5.0], [8.0, 7.0, 3.0]]]),self.functionspace)
      arg[1:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 3.0], [8.0, 1.0, 6.0], [3.0, 3.0, 5.0]], [[6.0, 4.0, 2.0], [7.0, 7.0, 7.0], [5.0, 
2.0, 4.0]], [[8.0, 6.0, 4.0], [3.0, 1.0, 4.0], [2.0, 4.0, 3.0]]], [[[7.0, 5.0, 2.0], [7.0, 5.0, 4.0], [4.0, 8.0, 2.0]], [[4.0, 
4.0, 6.0], [7.0, 1.0, 5.0], [6.0, 5.0, 8.0]], [[2.0, 4.0, 1.0], [8.0, 3.0, 5.0], [4.0, 8.0, 4.0]]], [[[3.0, 7.0, 2.0], [5.0, 
3.0, 3.0], [2.0, 8.0, 2.0]], [[7.0, 3.0, 5.0], [8.0, 7.0, 3.0], [6.0, 1.0, 6.0]], [[1.0, 3.0, 5.0], [8.0, 6.0, 8.0], [1.0, 3.0, 
4.0]]], [[[6.0, 6.0, 4.0], [7.0, 6.0, 1.0], [6.0, 1.0, 2.0]], [[8.0, 4.0, 1.0], [3.0, 6.0, 8.0], [1.0, 5.0, 6.0]], [[2.0, 2.0, 
3.0], [6.0, 4.0, 8.0], [3.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 5.0], [1.0, 6.0, 5.0], [1.0, 7.0, 3.0]], 
[[3.0, 4.0, 6.0], [3.0, 4.0, 5.0], [5.0, 6.0, 8.0]], [[4.0, 8.0, 3.0], [6.0, 4.0, 8.0], [5.0, 5.0, 5.0]]], [[[1.0, 4.0, 7.0], 
[7.0, 6.0, 5.0], [1.0, 6.0, 4.0]], [[4.0, 4.0, 6.0], [7.0, 1.0, 5.0], [2.0, 3.0, 8.0]], [[6.0, 5.0, 4.0], [2.0, 2.0, 1.0], 
[7.0, 4.0, 1.0]]], [[[6.0, 1.0, 7.0], [4.0, 8.0, 8.0], [2.0, 2.0, 5.0]], [[7.0, 3.0, 5.0], [8.0, 7.0, 3.0], [2.0, 5.0, 6.0]], 
[[5.0, 1.0, 2.0], [5.0, 8.0, 4.0], [3.0, 7.0, 1.0]]], [[[7.0, 1.0, 6.0], [6.0, 5.0, 7.0], [6.0, 2.0, 8.0]], [[4.0, 2.0, 8.0], 
[2.0, 5.0, 5.0], [6.0, 3.0, 5.0]], [[4.0, 2.0, 4.0], [2.0, 1.0, 1.0], [8.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPUN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 8.0], [8.0, 3.0, 8.0], [8.0, 4.0, 6.0]], [[3.0, 7.0, 6.0], [4.0, 2.0, 7.0], [4.0, 4.0, 
2.0]], [[5.0, 7.0, 7.0], [4.0, 6.0, 7.0], [6.0, 1.0, 3.0]]], [[[6.0, 2.0, 4.0], [7.0, 2.0, 5.0], [3.0, 3.0, 2.0]], [[2.0, 8.0, 
5.0], [8.0, 1.0, 3.0], [1.0, 4.0, 6.0]], [[5.0, 4.0, 2.0], [8.0, 6.0, 6.0], [7.0, 2.0, 3.0]]], [[[3.0, 5.0, 6.0], [6.0, 2.0, 
6.0], [5.0, 5.0, 8.0]], [[5.0, 7.0, 7.0], [5.0, 8.0, 2.0], [6.0, 2.0, 4.0]], [[7.0, 1.0, 4.0], [3.0, 2.0, 4.0], [5.0, 5.0, 
4.0]]], [[[2.0, 2.0, 1.0], [4.0, 5.0, 7.0], [8.0, 1.0, 6.0]], [[4.0, 4.0, 2.0], [7.0, 2.0, 3.0], [4.0, 1.0, 5.0]], [[4.0, 7.0, 
3.0], [4.0, 4.0, 6.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 8.0], [4.0, 5.0, 5.0], [1.0, 2.0, 6.0]], [[4.0, 7.0, 2.0], [5.0, 6.0, 
6.0], [3.0, 3.0, 3.0]], [[6.0, 5.0, 1.0], [1.0, 4.0, 3.0], [1.0, 4.0, 7.0]]], [[[7.0, 3.0, 2.0], [2.0, 8.0, 5.0], [3.0, 1.0, 
2.0]], [[6.0, 3.0, 3.0], [8.0, 7.0, 7.0], [7.0, 1.0, 6.0]], [[6.0, 2.0, 7.0], [6.0, 6.0, 5.0], [7.0, 7.0, 8.0]]], [[[2.0, 2.0, 
7.0], [7.0, 2.0, 4.0], [2.0, 7.0, 8.0]], [[5.0, 3.0, 7.0], [1.0, 4.0, 2.0], [5.0, 5.0, 1.0]], [[1.0, 4.0, 7.0], [1.0, 6.0, 
6.0], [3.0, 6.0, 3.0]]], [[[2.0, 3.0, 8.0], [5.0, 8.0, 8.0], [1.0, 1.0, 8.0]], [[6.0, 2.0, 7.0], [6.0, 1.0, 7.0], [8.0, 2.0, 
7.0]], [[2.0, 6.0, 7.0], [8.0, 8.0, 2.0], [3.0, 1.0, 5.0]]]]))
      val=Data(numpy.array([[[1.0, 1.0, 8.0], [6.0, 5.0, 8.0]], [[3.0, 1.0, 1.0], [3.0, 2.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 4.0, 3.0], [4.0, 8.0, 2.0]], [[3.0, 1.0, 3.0], [8.0, 2.0, 8.0]]]))
      arg[1:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 8.0], [8.0, 3.0, 8.0], [8.0, 4.0, 6.0]], [[3.0, 7.0, 6.0], [4.0, 2.0, 7.0], [4.0, 
4.0, 2.0]], [[5.0, 7.0, 7.0], [4.0, 6.0, 7.0], [6.0, 1.0, 3.0]]], [[[6.0, 2.0, 4.0], [7.0, 2.0, 5.0], [3.0, 3.0, 2.0]], [[1.0, 
1.0, 8.0], [6.0, 5.0, 8.0], [1.0, 4.0, 6.0]], [[5.0, 4.0, 2.0], [8.0, 6.0, 6.0], [7.0, 2.0, 3.0]]], [[[3.0, 5.0, 6.0], [6.0, 
2.0, 6.0], [5.0, 5.0, 8.0]], [[3.0, 1.0, 1.0], [3.0, 2.0, 8.0], [6.0, 2.0, 4.0]], [[7.0, 1.0, 4.0], [3.0, 2.0, 4.0], [5.0, 5.0, 
4.0]]], [[[2.0, 2.0, 1.0], [4.0, 5.0, 7.0], [8.0, 1.0, 6.0]], [[4.0, 4.0, 2.0], [7.0, 2.0, 3.0], [4.0, 1.0, 5.0]], [[4.0, 7.0, 
3.0], [4.0, 4.0, 6.0], [6.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 8.0], [4.0, 5.0, 5.0], [1.0, 2.0, 6.0]], 
[[4.0, 7.0, 2.0], [5.0, 6.0, 6.0], [3.0, 3.0, 3.0]], [[6.0, 5.0, 1.0], [1.0, 4.0, 3.0], [1.0, 4.0, 7.0]]], [[[7.0, 3.0, 2.0], 
[2.0, 8.0, 5.0], [3.0, 1.0, 2.0]], [[8.0, 4.0, 3.0], [4.0, 8.0, 2.0], [7.0, 1.0, 6.0]], [[6.0, 2.0, 7.0], [6.0, 6.0, 5.0], 
[7.0, 7.0, 8.0]]], [[[2.0, 2.0, 7.0], [7.0, 2.0, 4.0], [2.0, 7.0, 8.0]], [[3.0, 1.0, 3.0], [8.0, 2.0, 8.0], [5.0, 5.0, 1.0]], 
[[1.0, 4.0, 7.0], [1.0, 6.0, 6.0], [3.0, 6.0, 3.0]]], [[[2.0, 3.0, 8.0], [5.0, 8.0, 8.0], [1.0, 1.0, 8.0]], [[6.0, 2.0, 7.0], 
[6.0, 1.0, 7.0], [8.0, 2.0, 7.0]], [[2.0, 6.0, 7.0], [8.0, 8.0, 2.0], [3.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPUN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 7.0], [5.0, 6.0, 7.0], [4.0, 2.0, 5.0]], [[7.0, 2.0, 8.0], [6.0, 5.0, 7.0], [6.0, 3.0, 
6.0]], [[8.0, 7.0, 4.0], [6.0, 7.0, 2.0], [7.0, 8.0, 4.0]]], [[[8.0, 8.0, 4.0], [3.0, 4.0, 2.0], [2.0, 5.0, 4.0]], [[6.0, 6.0, 
6.0], [2.0, 1.0, 8.0], [8.0, 8.0, 4.0]], [[8.0, 4.0, 3.0], [2.0, 1.0, 2.0], [6.0, 1.0, 6.0]]], [[[6.0, 2.0, 6.0], [4.0, 8.0, 
3.0], [5.0, 5.0, 6.0]], [[6.0, 8.0, 8.0], [5.0, 5.0, 3.0], [2.0, 8.0, 6.0]], [[7.0, 7.0, 1.0], [8.0, 6.0, 1.0], [4.0, 6.0, 
6.0]]], [[[7.0, 3.0, 6.0], [4.0, 2.0, 5.0], [5.0, 6.0, 2.0]], [[3.0, 4.0, 8.0], [8.0, 1.0, 3.0], [8.0, 8.0, 4.0]], [[1.0, 6.0, 
8.0], [8.0, 2.0, 4.0], [1.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 3.0], [4.0, 7.0, 2.0], [7.0, 5.0, 4.0]], [[3.0, 2.0, 7.0], [4.0, 4.0, 
8.0], [1.0, 3.0, 3.0]], [[5.0, 6.0, 4.0], [1.0, 3.0, 6.0], [6.0, 6.0, 4.0]]], [[[8.0, 4.0, 4.0], [2.0, 2.0, 8.0], [5.0, 1.0, 
3.0]], [[5.0, 3.0, 5.0], [5.0, 6.0, 7.0], [2.0, 6.0, 1.0]], [[5.0, 8.0, 8.0], [6.0, 3.0, 2.0], [3.0, 2.0, 3.0]]], [[[7.0, 6.0, 
4.0], [7.0, 2.0, 8.0], [3.0, 7.0, 2.0]], [[1.0, 5.0, 6.0], [2.0, 3.0, 8.0], [3.0, 3.0, 2.0]], [[3.0, 4.0, 4.0], [6.0, 1.0, 
6.0], [3.0, 5.0, 1.0]]], [[[7.0, 3.0, 6.0], [3.0, 1.0, 4.0], [1.0, 1.0, 6.0]], [[6.0, 4.0, 1.0], [1.0, 4.0, 6.0], [5.0, 6.0, 
8.0]], [[7.0, 8.0, 5.0], [2.0, 1.0, 8.0], [4.0, 5.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 6.0, 3.0], [2.0, 6.0, 7.0]], [[7.0, 2.0, 3.0], [2.0, 7.0, 
2.0]]])+(1.-msk_val)*numpy.array([[[8.0, 7.0, 1.0], [8.0, 8.0, 3.0]], [[2.0, 1.0, 5.0], [6.0, 7.0, 3.0]]])
      arg[1:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 7.0], [5.0, 6.0, 7.0], [4.0, 2.0, 5.0]], [[7.0, 2.0, 8.0], [6.0, 5.0, 7.0], [6.0, 
3.0, 6.0]], [[8.0, 7.0, 4.0], [6.0, 7.0, 2.0], [7.0, 8.0, 4.0]]], [[[8.0, 8.0, 4.0], [3.0, 4.0, 2.0], [2.0, 5.0, 4.0]], [[5.0, 
6.0, 3.0], [2.0, 6.0, 7.0], [8.0, 8.0, 4.0]], [[8.0, 4.0, 3.0], [2.0, 1.0, 2.0], [6.0, 1.0, 6.0]]], [[[6.0, 2.0, 6.0], [4.0, 
8.0, 3.0], [5.0, 5.0, 6.0]], [[7.0, 2.0, 3.0], [2.0, 7.0, 2.0], [2.0, 8.0, 6.0]], [[7.0, 7.0, 1.0], [8.0, 6.0, 1.0], [4.0, 6.0, 
6.0]]], [[[7.0, 3.0, 6.0], [4.0, 2.0, 5.0], [5.0, 6.0, 2.0]], [[3.0, 4.0, 8.0], [8.0, 1.0, 3.0], [8.0, 8.0, 4.0]], [[1.0, 6.0, 
8.0], [8.0, 2.0, 4.0], [1.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 3.0], [4.0, 7.0, 2.0], [7.0, 5.0, 4.0]], 
[[3.0, 2.0, 7.0], [4.0, 4.0, 8.0], [1.0, 3.0, 3.0]], [[5.0, 6.0, 4.0], [1.0, 3.0, 6.0], [6.0, 6.0, 4.0]]], [[[8.0, 4.0, 4.0], 
[2.0, 2.0, 8.0], [5.0, 1.0, 3.0]], [[8.0, 7.0, 1.0], [8.0, 8.0, 3.0], [2.0, 6.0, 1.0]], [[5.0, 8.0, 8.0], [6.0, 3.0, 2.0], 
[3.0, 2.0, 3.0]]], [[[7.0, 6.0, 4.0], [7.0, 2.0, 8.0], [3.0, 7.0, 2.0]], [[2.0, 1.0, 5.0], [6.0, 7.0, 3.0], [3.0, 3.0, 2.0]], 
[[3.0, 4.0, 4.0], [6.0, 1.0, 6.0], [3.0, 5.0, 1.0]]], [[[7.0, 3.0, 6.0], [3.0, 1.0, 4.0], [1.0, 1.0, 6.0]], [[6.0, 4.0, 1.0], 
[1.0, 4.0, 6.0], [5.0, 6.0, 8.0]], [[7.0, 8.0, 5.0], [2.0, 1.0, 8.0], [4.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LPCN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 6.0], [7.0, 6.0, 4.0], [2.0, 3.0, 7.0]], [[4.0, 2.0, 8.0], [3.0, 3.0, 1.0], [1.0, 5.0, 
3.0]], [[7.0, 5.0, 3.0], [5.0, 7.0, 6.0], [3.0, 7.0, 1.0]]], [[[7.0, 5.0, 7.0], [7.0, 2.0, 3.0], [1.0, 8.0, 2.0]], [[8.0, 7.0, 
7.0], [2.0, 3.0, 1.0], [6.0, 8.0, 8.0]], [[1.0, 3.0, 3.0], [1.0, 6.0, 5.0], [8.0, 1.0, 4.0]]], [[[4.0, 4.0, 8.0], [8.0, 4.0, 
7.0], [8.0, 6.0, 1.0]], [[3.0, 4.0, 4.0], [5.0, 6.0, 1.0], [4.0, 2.0, 2.0]], [[4.0, 1.0, 3.0], [4.0, 6.0, 4.0], [7.0, 7.0, 
2.0]]], [[[5.0, 4.0, 4.0], [7.0, 4.0, 3.0], [5.0, 5.0, 7.0]], [[3.0, 6.0, 4.0], [7.0, 3.0, 8.0], [7.0, 6.0, 4.0]], [[5.0, 7.0, 
7.0], [2.0, 3.0, 1.0], [4.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 6.0], [7.0, 4.0, 6.0], [6.0, 8.0, 2.0]], [[8.0, 1.0, 6.0], [6.0, 5.0, 
8.0], [8.0, 4.0, 4.0]], [[3.0, 7.0, 3.0], [6.0, 5.0, 7.0], [4.0, 2.0, 8.0]]], [[[8.0, 7.0, 5.0], [2.0, 4.0, 2.0], [8.0, 1.0, 
3.0]], [[3.0, 1.0, 3.0], [8.0, 7.0, 1.0], [2.0, 5.0, 7.0]], [[3.0, 5.0, 3.0], [5.0, 5.0, 5.0], [1.0, 6.0, 5.0]]], [[[5.0, 3.0, 
8.0], [6.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[5.0, 4.0, 7.0], [1.0, 1.0, 4.0], [2.0, 2.0, 7.0]], [[2.0, 2.0, 2.0], [1.0, 6.0, 
4.0], [2.0, 4.0, 5.0]]], [[[3.0, 5.0, 8.0], [7.0, 2.0, 6.0], [3.0, 3.0, 3.0]], [[3.0, 3.0, 7.0], [5.0, 3.0, 3.0], [5.0, 2.0, 
8.0]], [[3.0, 3.0, 4.0], [1.0, 3.0, 8.0], [7.0, 7.0, 7.0]]]]))
      val=numpy.array([[[2.0, 3.0, 7.0], [1.0, 1.0, 6.0], [5.0, 8.0, 7.0]], [[3.0, 5.0, 8.0], [8.0, 4.0, 4.0], [1.0, 1.0, 
6.0]]])
      arg[2:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [7.0, 6.0, 4.0], [2.0, 3.0, 7.0]], [[4.0, 2.0, 8.0], [3.0, 3.0, 1.0], [1.0, 
5.0, 3.0]], [[7.0, 5.0, 3.0], [5.0, 7.0, 6.0], [3.0, 7.0, 1.0]]], [[[7.0, 5.0, 7.0], [7.0, 2.0, 3.0], [1.0, 8.0, 2.0]], [[8.0, 
7.0, 7.0], [2.0, 3.0, 1.0], [6.0, 8.0, 8.0]], [[1.0, 3.0, 3.0], [1.0, 6.0, 5.0], [8.0, 1.0, 4.0]]], [[[4.0, 4.0, 8.0], [8.0, 
4.0, 7.0], [8.0, 6.0, 1.0]], [[2.0, 3.0, 7.0], [1.0, 1.0, 6.0], [5.0, 8.0, 7.0]], [[4.0, 1.0, 3.0], [4.0, 6.0, 4.0], [7.0, 7.0, 
2.0]]], [[[5.0, 4.0, 4.0], [7.0, 4.0, 3.0], [5.0, 5.0, 7.0]], [[3.0, 5.0, 8.0], [8.0, 4.0, 4.0], [1.0, 1.0, 6.0]], [[5.0, 7.0, 
7.0], [2.0, 3.0, 1.0], [4.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 6.0], [7.0, 4.0, 6.0], [6.0, 8.0, 2.0]], 
[[8.0, 1.0, 6.0], [6.0, 5.0, 8.0], [8.0, 4.0, 4.0]], [[3.0, 7.0, 3.0], [6.0, 5.0, 7.0], [4.0, 2.0, 8.0]]], [[[8.0, 7.0, 5.0], 
[2.0, 4.0, 2.0], [8.0, 1.0, 3.0]], [[3.0, 1.0, 3.0], [8.0, 7.0, 1.0], [2.0, 5.0, 7.0]], [[3.0, 5.0, 3.0], [5.0, 5.0, 5.0], 
[1.0, 6.0, 5.0]]], [[[5.0, 3.0, 8.0], [6.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[2.0, 3.0, 7.0], [1.0, 1.0, 6.0], [5.0, 8.0, 7.0]], 
[[2.0, 2.0, 2.0], [1.0, 6.0, 4.0], [2.0, 4.0, 5.0]]], [[[3.0, 5.0, 8.0], [7.0, 2.0, 6.0], [3.0, 3.0, 3.0]], [[3.0, 5.0, 8.0], 
[8.0, 4.0, 4.0], [1.0, 1.0, 6.0]], [[3.0, 3.0, 4.0], [1.0, 3.0, 8.0], [7.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LPCN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 5.0], [7.0, 2.0, 6.0], [6.0, 5.0, 2.0]], [[8.0, 2.0, 8.0], [6.0, 1.0, 6.0], [6.0, 8.0, 
8.0]], [[3.0, 8.0, 7.0], [8.0, 6.0, 6.0], [4.0, 2.0, 1.0]]], [[[7.0, 3.0, 4.0], [8.0, 2.0, 5.0], [8.0, 1.0, 2.0]], [[1.0, 2.0, 
7.0], [7.0, 6.0, 6.0], [2.0, 6.0, 1.0]], [[2.0, 3.0, 1.0], [2.0, 7.0, 2.0], [8.0, 7.0, 6.0]]], [[[8.0, 3.0, 5.0], [4.0, 5.0, 
7.0], [2.0, 6.0, 2.0]], [[6.0, 1.0, 7.0], [6.0, 2.0, 6.0], [8.0, 2.0, 1.0]], [[5.0, 5.0, 3.0], [3.0, 4.0, 6.0], [3.0, 1.0, 
5.0]]], [[[1.0, 7.0, 1.0], [7.0, 4.0, 6.0], [1.0, 5.0, 3.0]], [[7.0, 7.0, 2.0], [7.0, 6.0, 8.0], [2.0, 8.0, 6.0]], [[5.0, 7.0, 
4.0], [7.0, 7.0, 2.0], [1.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 7.0], [5.0, 4.0, 1.0], [6.0, 1.0, 1.0]], [[6.0, 6.0, 6.0], [2.0, 4.0, 
2.0], [3.0, 3.0, 5.0]], [[5.0, 7.0, 1.0], [5.0, 3.0, 8.0], [8.0, 7.0, 8.0]]], [[[2.0, 3.0, 5.0], [4.0, 8.0, 2.0], [6.0, 4.0, 
7.0]], [[4.0, 2.0, 5.0], [4.0, 1.0, 7.0], [5.0, 8.0, 6.0]], [[3.0, 2.0, 6.0], [1.0, 7.0, 4.0], [6.0, 7.0, 1.0]]], [[[1.0, 2.0, 
3.0], [8.0, 7.0, 6.0], [1.0, 1.0, 5.0]], [[4.0, 5.0, 1.0], [4.0, 6.0, 8.0], [3.0, 6.0, 5.0]], [[2.0, 1.0, 8.0], [6.0, 7.0, 
3.0], [8.0, 7.0, 1.0]]], [[[1.0, 8.0, 4.0], [3.0, 4.0, 3.0], [4.0, 4.0, 6.0]], [[5.0, 1.0, 3.0], [8.0, 8.0, 4.0], [2.0, 6.0, 
5.0]], [[7.0, 4.0, 2.0], [6.0, 1.0, 2.0], [5.0, 5.0, 4.0]]]]))
      val=Data(numpy.array([[[4.0, 1.0, 4.0], [6.0, 5.0, 8.0], [3.0, 8.0, 1.0]], [[1.0, 8.0, 1.0], [5.0, 5.0, 4.0], [8.0, 3.0, 
3.0]]]),self.functionspace)
      arg[2:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 5.0], [7.0, 2.0, 6.0], [6.0, 5.0, 2.0]], [[8.0, 2.0, 8.0], [6.0, 1.0, 6.0], [6.0, 
8.0, 8.0]], [[3.0, 8.0, 7.0], [8.0, 6.0, 6.0], [4.0, 2.0, 1.0]]], [[[7.0, 3.0, 4.0], [8.0, 2.0, 5.0], [8.0, 1.0, 2.0]], [[1.0, 
2.0, 7.0], [7.0, 6.0, 6.0], [2.0, 6.0, 1.0]], [[2.0, 3.0, 1.0], [2.0, 7.0, 2.0], [8.0, 7.0, 6.0]]], [[[8.0, 3.0, 5.0], [4.0, 
5.0, 7.0], [2.0, 6.0, 2.0]], [[4.0, 1.0, 4.0], [6.0, 5.0, 8.0], [3.0, 8.0, 1.0]], [[5.0, 5.0, 3.0], [3.0, 4.0, 6.0], [3.0, 1.0, 
5.0]]], [[[1.0, 7.0, 1.0], [7.0, 4.0, 6.0], [1.0, 5.0, 3.0]], [[1.0, 8.0, 1.0], [5.0, 5.0, 4.0], [8.0, 3.0, 3.0]], [[5.0, 7.0, 
4.0], [7.0, 7.0, 2.0], [1.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 7.0], [5.0, 4.0, 1.0], [6.0, 1.0, 1.0]], 
[[6.0, 6.0, 6.0], [2.0, 4.0, 2.0], [3.0, 3.0, 5.0]], [[5.0, 7.0, 1.0], [5.0, 3.0, 8.0], [8.0, 7.0, 8.0]]], [[[2.0, 3.0, 5.0], 
[4.0, 8.0, 2.0], [6.0, 4.0, 7.0]], [[4.0, 2.0, 5.0], [4.0, 1.0, 7.0], [5.0, 8.0, 6.0]], [[3.0, 2.0, 6.0], [1.0, 7.0, 4.0], 
[6.0, 7.0, 1.0]]], [[[1.0, 2.0, 3.0], [8.0, 7.0, 6.0], [1.0, 1.0, 5.0]], [[4.0, 1.0, 4.0], [6.0, 5.0, 8.0], [3.0, 8.0, 1.0]], 
[[2.0, 1.0, 8.0], [6.0, 7.0, 3.0], [8.0, 7.0, 1.0]]], [[[1.0, 8.0, 4.0], [3.0, 4.0, 3.0], [4.0, 4.0, 6.0]], [[1.0, 8.0, 1.0], 
[5.0, 5.0, 4.0], [8.0, 3.0, 3.0]], [[7.0, 4.0, 2.0], [6.0, 1.0, 2.0], [5.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LPCN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 5.0], [5.0, 2.0, 2.0], [3.0, 3.0, 7.0]], [[6.0, 1.0, 1.0], [7.0, 6.0, 1.0], [5.0, 6.0, 
5.0]], [[5.0, 6.0, 2.0], [1.0, 4.0, 4.0], [7.0, 7.0, 7.0]]], [[[2.0, 7.0, 7.0], [8.0, 6.0, 5.0], [4.0, 2.0, 5.0]], [[6.0, 3.0, 
8.0], [8.0, 2.0, 1.0], [4.0, 2.0, 2.0]], [[2.0, 7.0, 2.0], [5.0, 3.0, 8.0], [5.0, 4.0, 3.0]]], [[[6.0, 4.0, 3.0], [1.0, 2.0, 
4.0], [6.0, 3.0, 4.0]], [[5.0, 2.0, 7.0], [6.0, 6.0, 5.0], [5.0, 5.0, 7.0]], [[2.0, 8.0, 4.0], [5.0, 4.0, 4.0], [2.0, 8.0, 
7.0]]], [[[7.0, 1.0, 3.0], [7.0, 6.0, 3.0], [5.0, 8.0, 8.0]], [[4.0, 1.0, 2.0], [3.0, 8.0, 7.0], [1.0, 2.0, 6.0]], [[1.0, 2.0, 
7.0], [1.0, 3.0, 2.0], [6.0, 3.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 7.0], [6.0, 7.0, 4.0], [6.0, 2.0, 2.0]], [[7.0, 6.0, 1.0], [3.0, 7.0, 
4.0], [7.0, 3.0, 1.0]], [[5.0, 1.0, 8.0], [7.0, 7.0, 2.0], [6.0, 1.0, 1.0]]], [[[6.0, 5.0, 6.0], [6.0, 1.0, 3.0], [5.0, 4.0, 
8.0]], [[1.0, 7.0, 7.0], [1.0, 8.0, 3.0], [8.0, 1.0, 6.0]], [[2.0, 5.0, 5.0], [6.0, 3.0, 2.0], [1.0, 4.0, 1.0]]], [[[8.0, 1.0, 
4.0], [3.0, 3.0, 5.0], [3.0, 6.0, 3.0]], [[6.0, 7.0, 2.0], [4.0, 1.0, 7.0], [5.0, 6.0, 5.0]], [[6.0, 2.0, 4.0], [2.0, 2.0, 
2.0], [7.0, 6.0, 8.0]]], [[[6.0, 1.0, 5.0], [3.0, 5.0, 1.0], [2.0, 8.0, 8.0]], [[6.0, 2.0, 6.0], [7.0, 5.0, 8.0], [1.0, 8.0, 
8.0]], [[4.0, 3.0, 4.0], [6.0, 7.0, 8.0], [4.0, 7.0, 8.0]]]]))
      val=Data(numpy.array([[[8.0, 5.0, 7.0], [8.0, 7.0, 1.0], [5.0, 3.0, 8.0]], [[7.0, 2.0, 2.0], [8.0, 1.0, 3.0], [1.0, 4.0, 
3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 5.0, 3.0], [7.0, 4.0, 3.0], [8.0, 6.0, 3.0]], [[2.0, 4.0, 4.0], [7.0, 4.0, 3.0], 
[2.0, 7.0, 1.0]]]))
      arg[2:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 5.0], [5.0, 2.0, 2.0], [3.0, 3.0, 7.0]], [[6.0, 1.0, 1.0], [7.0, 6.0, 1.0], [5.0, 
6.0, 5.0]], [[5.0, 6.0, 2.0], [1.0, 4.0, 4.0], [7.0, 7.0, 7.0]]], [[[2.0, 7.0, 7.0], [8.0, 6.0, 5.0], [4.0, 2.0, 5.0]], [[6.0, 
3.0, 8.0], [8.0, 2.0, 1.0], [4.0, 2.0, 2.0]], [[2.0, 7.0, 2.0], [5.0, 3.0, 8.0], [5.0, 4.0, 3.0]]], [[[6.0, 4.0, 3.0], [1.0, 
2.0, 4.0], [6.0, 3.0, 4.0]], [[8.0, 5.0, 7.0], [8.0, 7.0, 1.0], [5.0, 3.0, 8.0]], [[2.0, 8.0, 4.0], [5.0, 4.0, 4.0], [2.0, 8.0, 
7.0]]], [[[7.0, 1.0, 3.0], [7.0, 6.0, 3.0], [5.0, 8.0, 8.0]], [[7.0, 2.0, 2.0], [8.0, 1.0, 3.0], [1.0, 4.0, 3.0]], [[1.0, 2.0, 
7.0], [1.0, 3.0, 2.0], [6.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 7.0], [6.0, 7.0, 4.0], [6.0, 2.0, 2.0]], 
[[7.0, 6.0, 1.0], [3.0, 7.0, 4.0], [7.0, 3.0, 1.0]], [[5.0, 1.0, 8.0], [7.0, 7.0, 2.0], [6.0, 1.0, 1.0]]], [[[6.0, 5.0, 6.0], 
[6.0, 1.0, 3.0], [5.0, 4.0, 8.0]], [[1.0, 7.0, 7.0], [1.0, 8.0, 3.0], [8.0, 1.0, 6.0]], [[2.0, 5.0, 5.0], [6.0, 3.0, 2.0], 
[1.0, 4.0, 1.0]]], [[[8.0, 1.0, 4.0], [3.0, 3.0, 5.0], [3.0, 6.0, 3.0]], [[6.0, 5.0, 3.0], [7.0, 4.0, 3.0], [8.0, 6.0, 3.0]], 
[[6.0, 2.0, 4.0], [2.0, 2.0, 2.0], [7.0, 6.0, 8.0]]], [[[6.0, 1.0, 5.0], [3.0, 5.0, 1.0], [2.0, 8.0, 8.0]], [[2.0, 4.0, 4.0], 
[7.0, 4.0, 3.0], [2.0, 7.0, 1.0]], [[4.0, 3.0, 4.0], [6.0, 7.0, 8.0], [4.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LPCN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 5.0], [2.0, 2.0, 8.0], [1.0, 8.0, 3.0]], [[4.0, 8.0, 2.0], [1.0, 8.0, 1.0], [8.0, 5.0, 
1.0]], [[8.0, 5.0, 8.0], [2.0, 3.0, 5.0], [5.0, 8.0, 6.0]]], [[[5.0, 1.0, 5.0], [6.0, 4.0, 3.0], [7.0, 8.0, 3.0]], [[2.0, 8.0, 
1.0], [4.0, 5.0, 2.0], [5.0, 5.0, 4.0]], [[5.0, 1.0, 1.0], [7.0, 1.0, 8.0], [5.0, 4.0, 4.0]]], [[[4.0, 7.0, 1.0], [7.0, 1.0, 
1.0], [8.0, 6.0, 5.0]], [[7.0, 4.0, 5.0], [5.0, 7.0, 2.0], [5.0, 8.0, 6.0]], [[4.0, 3.0, 5.0], [8.0, 2.0, 8.0], [7.0, 7.0, 
4.0]]], [[[3.0, 3.0, 5.0], [2.0, 5.0, 8.0], [8.0, 2.0, 8.0]], [[2.0, 8.0, 2.0], [6.0, 8.0, 8.0], [3.0, 3.0, 4.0]], [[3.0, 2.0, 
2.0], [1.0, 7.0, 8.0], [3.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 7.0], [8.0, 7.0, 1.0], [4.0, 4.0, 1.0]], [[6.0, 1.0, 8.0], [2.0, 5.0, 
1.0], [5.0, 2.0, 1.0]], [[3.0, 4.0, 5.0], [6.0, 5.0, 1.0], [2.0, 1.0, 8.0]]], [[[1.0, 1.0, 3.0], [8.0, 4.0, 2.0], [5.0, 3.0, 
8.0]], [[7.0, 5.0, 4.0], [2.0, 8.0, 6.0], [6.0, 8.0, 4.0]], [[2.0, 6.0, 6.0], [2.0, 8.0, 8.0], [2.0, 6.0, 3.0]]], [[[6.0, 1.0, 
5.0], [6.0, 4.0, 8.0], [8.0, 4.0, 8.0]], [[5.0, 7.0, 8.0], [3.0, 1.0, 5.0], [3.0, 6.0, 8.0]], [[6.0, 1.0, 5.0], [6.0, 7.0, 
7.0], [7.0, 4.0, 5.0]]], [[[1.0, 8.0, 1.0], [5.0, 8.0, 8.0], [8.0, 1.0, 6.0]], [[4.0, 6.0, 5.0], [2.0, 1.0, 6.0], [1.0, 1.0, 
5.0]], [[6.0, 1.0, 1.0], [5.0, 6.0, 2.0], [2.0, 3.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 8.0, 5.0], [1.0, 7.0, 3.0], [1.0, 8.0, 2.0]], [[6.0, 6.0, 2.0], [7.0, 1.0, 1.0], [8.0, 
8.0, 7.0]]])+(1.-msk_val)*numpy.array([[[7.0, 2.0, 2.0], [2.0, 4.0, 4.0], [2.0, 2.0, 1.0]], [[3.0, 6.0, 8.0], [3.0, 4.0, 6.0], 
[8.0, 7.0, 7.0]]])
      arg[2:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 5.0], [2.0, 2.0, 8.0], [1.0, 8.0, 3.0]], [[4.0, 8.0, 2.0], [1.0, 8.0, 1.0], [8.0, 
5.0, 1.0]], [[8.0, 5.0, 8.0], [2.0, 3.0, 5.0], [5.0, 8.0, 6.0]]], [[[5.0, 1.0, 5.0], [6.0, 4.0, 3.0], [7.0, 8.0, 3.0]], [[2.0, 
8.0, 1.0], [4.0, 5.0, 2.0], [5.0, 5.0, 4.0]], [[5.0, 1.0, 1.0], [7.0, 1.0, 8.0], [5.0, 4.0, 4.0]]], [[[4.0, 7.0, 1.0], [7.0, 
1.0, 1.0], [8.0, 6.0, 5.0]], [[5.0, 8.0, 5.0], [1.0, 7.0, 3.0], [1.0, 8.0, 2.0]], [[4.0, 3.0, 5.0], [8.0, 2.0, 8.0], [7.0, 7.0, 
4.0]]], [[[3.0, 3.0, 5.0], [2.0, 5.0, 8.0], [8.0, 2.0, 8.0]], [[6.0, 6.0, 2.0], [7.0, 1.0, 1.0], [8.0, 8.0, 7.0]], [[3.0, 2.0, 
2.0], [1.0, 7.0, 8.0], [3.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 7.0], [8.0, 7.0, 1.0], [4.0, 4.0, 1.0]], 
[[6.0, 1.0, 8.0], [2.0, 5.0, 1.0], [5.0, 2.0, 1.0]], [[3.0, 4.0, 5.0], [6.0, 5.0, 1.0], [2.0, 1.0, 8.0]]], [[[1.0, 1.0, 3.0], 
[8.0, 4.0, 2.0], [5.0, 3.0, 8.0]], [[7.0, 5.0, 4.0], [2.0, 8.0, 6.0], [6.0, 8.0, 4.0]], [[2.0, 6.0, 6.0], [2.0, 8.0, 8.0], 
[2.0, 6.0, 3.0]]], [[[6.0, 1.0, 5.0], [6.0, 4.0, 8.0], [8.0, 4.0, 8.0]], [[7.0, 2.0, 2.0], [2.0, 4.0, 4.0], [2.0, 2.0, 1.0]], 
[[6.0, 1.0, 5.0], [6.0, 7.0, 7.0], [7.0, 4.0, 5.0]]], [[[1.0, 8.0, 1.0], [5.0, 8.0, 8.0], [8.0, 1.0, 6.0]], [[3.0, 6.0, 8.0], 
[3.0, 4.0, 6.0], [8.0, 7.0, 7.0]], [[6.0, 1.0, 1.0], [5.0, 6.0, 2.0], [2.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPPN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 2.0], [3.0, 1.0, 5.0], [2.0, 4.0, 4.0]], [[3.0, 2.0, 2.0], [5.0, 2.0, 2.0], [2.0, 7.0, 
3.0]], [[6.0, 3.0, 3.0], [3.0, 1.0, 4.0], [8.0, 8.0, 6.0]]], [[[3.0, 1.0, 6.0], [2.0, 7.0, 1.0], [6.0, 3.0, 3.0]], [[8.0, 4.0, 
8.0], [1.0, 1.0, 3.0], [5.0, 2.0, 2.0]], [[5.0, 3.0, 5.0], [7.0, 8.0, 7.0], [3.0, 2.0, 5.0]]], [[[2.0, 5.0, 2.0], [1.0, 4.0, 
7.0], [3.0, 7.0, 6.0]], [[1.0, 5.0, 8.0], [6.0, 6.0, 4.0], [3.0, 3.0, 1.0]], [[4.0, 5.0, 6.0], [1.0, 7.0, 2.0], [1.0, 5.0, 
5.0]]], [[[1.0, 7.0, 1.0], [1.0, 7.0, 3.0], [7.0, 5.0, 8.0]], [[3.0, 6.0, 7.0], [8.0, 3.0, 8.0], [2.0, 1.0, 4.0]], [[7.0, 7.0, 
6.0], [7.0, 6.0, 2.0], [1.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 5.0], [5.0, 5.0, 2.0], [1.0, 5.0, 8.0]], [[4.0, 8.0, 6.0], [8.0, 4.0, 
6.0], [5.0, 5.0, 2.0]], [[7.0, 3.0, 2.0], [8.0, 2.0, 2.0], [5.0, 4.0, 6.0]]], [[[7.0, 6.0, 6.0], [7.0, 4.0, 7.0], [5.0, 6.0, 
8.0]], [[6.0, 4.0, 4.0], [2.0, 7.0, 5.0], [3.0, 1.0, 4.0]], [[1.0, 7.0, 4.0], [7.0, 7.0, 8.0], [7.0, 2.0, 7.0]]], [[[7.0, 4.0, 
6.0], [6.0, 7.0, 1.0], [3.0, 6.0, 5.0]], [[2.0, 4.0, 7.0], [4.0, 2.0, 8.0], [3.0, 8.0, 7.0]], [[4.0, 3.0, 6.0], [1.0, 1.0, 
4.0], [3.0, 8.0, 2.0]]], [[[2.0, 1.0, 1.0], [8.0, 7.0, 3.0], [8.0, 5.0, 7.0]], [[1.0, 6.0, 5.0], [1.0, 6.0, 7.0], [8.0, 7.0, 
1.0]], [[5.0, 1.0, 5.0], [5.0, 4.0, 6.0], [1.0, 3.0, 6.0]]]]))
      val=numpy.array([6.0, 4.0, 3.0])
      arg[2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 2.0], [3.0, 1.0, 5.0], [2.0, 4.0, 4.0]], [[3.0, 2.0, 2.0], [5.0, 2.0, 2.0], [2.0, 
7.0, 3.0]], [[6.0, 3.0, 3.0], [3.0, 1.0, 4.0], [8.0, 8.0, 6.0]]], [[[3.0, 1.0, 6.0], [2.0, 7.0, 1.0], [6.0, 3.0, 3.0]], [[8.0, 
4.0, 8.0], [1.0, 1.0, 3.0], [5.0, 2.0, 2.0]], [[5.0, 3.0, 5.0], [7.0, 8.0, 7.0], [3.0, 2.0, 5.0]]], [[[2.0, 5.0, 2.0], [1.0, 
4.0, 7.0], [3.0, 7.0, 6.0]], [[1.0, 5.0, 8.0], [6.0, 4.0, 3.0], [3.0, 3.0, 1.0]], [[4.0, 5.0, 6.0], [1.0, 7.0, 2.0], [1.0, 5.0, 
5.0]]], [[[1.0, 7.0, 1.0], [1.0, 7.0, 3.0], [7.0, 5.0, 8.0]], [[3.0, 6.0, 7.0], [8.0, 3.0, 8.0], [2.0, 1.0, 4.0]], [[7.0, 7.0, 
6.0], [7.0, 6.0, 2.0], [1.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 5.0], [5.0, 5.0, 2.0], [1.0, 5.0, 8.0]], 
[[4.0, 8.0, 6.0], [8.0, 4.0, 6.0], [5.0, 5.0, 2.0]], [[7.0, 3.0, 2.0], [8.0, 2.0, 2.0], [5.0, 4.0, 6.0]]], [[[7.0, 6.0, 6.0], 
[7.0, 4.0, 7.0], [5.0, 6.0, 8.0]], [[6.0, 4.0, 4.0], [2.0, 7.0, 5.0], [3.0, 1.0, 4.0]], [[1.0, 7.0, 4.0], [7.0, 7.0, 8.0], 
[7.0, 2.0, 7.0]]], [[[7.0, 4.0, 6.0], [6.0, 7.0, 1.0], [3.0, 6.0, 5.0]], [[2.0, 4.0, 7.0], [6.0, 4.0, 3.0], [3.0, 8.0, 7.0]], 
[[4.0, 3.0, 6.0], [1.0, 1.0, 4.0], [3.0, 8.0, 2.0]]], [[[2.0, 1.0, 1.0], [8.0, 7.0, 3.0], [8.0, 5.0, 7.0]], [[1.0, 6.0, 5.0], 
[1.0, 6.0, 7.0], [8.0, 7.0, 1.0]], [[5.0, 1.0, 5.0], [5.0, 4.0, 6.0], [1.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPPN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 6.0], [3.0, 4.0, 4.0], [3.0, 5.0, 7.0]], [[3.0, 3.0, 8.0], [5.0, 8.0, 7.0], [1.0, 7.0, 
3.0]], [[7.0, 7.0, 1.0], [5.0, 5.0, 6.0], [2.0, 8.0, 7.0]]], [[[1.0, 3.0, 4.0], [6.0, 3.0, 5.0], [7.0, 4.0, 3.0]], [[7.0, 2.0, 
1.0], [4.0, 8.0, 1.0], [3.0, 8.0, 8.0]], [[7.0, 7.0, 4.0], [8.0, 6.0, 8.0], [4.0, 2.0, 4.0]]], [[[7.0, 6.0, 1.0], [7.0, 3.0, 
2.0], [7.0, 1.0, 4.0]], [[8.0, 2.0, 4.0], [8.0, 7.0, 1.0], [7.0, 5.0, 1.0]], [[8.0, 5.0, 8.0], [7.0, 7.0, 8.0], [3.0, 5.0, 
1.0]]], [[[3.0, 5.0, 3.0], [3.0, 2.0, 1.0], [2.0, 7.0, 2.0]], [[3.0, 2.0, 4.0], [3.0, 3.0, 7.0], [5.0, 3.0, 1.0]], [[6.0, 5.0, 
2.0], [3.0, 2.0, 1.0], [1.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 2.0], [4.0, 6.0, 5.0], [6.0, 2.0, 7.0]], [[6.0, 5.0, 8.0], [7.0, 1.0, 
5.0], [5.0, 6.0, 8.0]], [[4.0, 2.0, 6.0], [5.0, 5.0, 1.0], [1.0, 2.0, 1.0]]], [[[3.0, 8.0, 4.0], [2.0, 6.0, 6.0], [8.0, 3.0, 
8.0]], [[8.0, 1.0, 4.0], [1.0, 1.0, 8.0], [1.0, 4.0, 5.0]], [[1.0, 5.0, 5.0], [4.0, 2.0, 6.0], [4.0, 5.0, 7.0]]], [[[7.0, 7.0, 
7.0], [2.0, 1.0, 6.0], [8.0, 8.0, 5.0]], [[8.0, 1.0, 7.0], [1.0, 6.0, 8.0], [6.0, 8.0, 3.0]], [[1.0, 5.0, 3.0], [8.0, 8.0, 
3.0], [4.0, 3.0, 1.0]]], [[[1.0, 2.0, 8.0], [7.0, 6.0, 3.0], [8.0, 5.0, 4.0]], [[4.0, 6.0, 2.0], [4.0, 6.0, 7.0], [8.0, 3.0, 
4.0]], [[1.0, 8.0, 3.0], [1.0, 5.0, 7.0], [3.0, 6.0, 4.0]]]]))
      val=Data(numpy.array([1.0, 8.0, 4.0]),self.functionspace)
      arg[2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 6.0], [3.0, 4.0, 4.0], [3.0, 5.0, 7.0]], [[3.0, 3.0, 8.0], [5.0, 8.0, 7.0], [1.0, 
7.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 5.0, 6.0], [2.0, 8.0, 7.0]]], [[[1.0, 3.0, 4.0], [6.0, 3.0, 5.0], [7.0, 4.0, 3.0]], [[7.0, 
2.0, 1.0], [4.0, 8.0, 1.0], [3.0, 8.0, 8.0]], [[7.0, 7.0, 4.0], [8.0, 6.0, 8.0], [4.0, 2.0, 4.0]]], [[[7.0, 6.0, 1.0], [7.0, 
3.0, 2.0], [7.0, 1.0, 4.0]], [[8.0, 2.0, 4.0], [1.0, 8.0, 4.0], [7.0, 5.0, 1.0]], [[8.0, 5.0, 8.0], [7.0, 7.0, 8.0], [3.0, 5.0, 
1.0]]], [[[3.0, 5.0, 3.0], [3.0, 2.0, 1.0], [2.0, 7.0, 2.0]], [[3.0, 2.0, 4.0], [3.0, 3.0, 7.0], [5.0, 3.0, 1.0]], [[6.0, 5.0, 
2.0], [3.0, 2.0, 1.0], [1.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 2.0], [4.0, 6.0, 5.0], [6.0, 2.0, 7.0]], 
[[6.0, 5.0, 8.0], [7.0, 1.0, 5.0], [5.0, 6.0, 8.0]], [[4.0, 2.0, 6.0], [5.0, 5.0, 1.0], [1.0, 2.0, 1.0]]], [[[3.0, 8.0, 4.0], 
[2.0, 6.0, 6.0], [8.0, 3.0, 8.0]], [[8.0, 1.0, 4.0], [1.0, 1.0, 8.0], [1.0, 4.0, 5.0]], [[1.0, 5.0, 5.0], [4.0, 2.0, 6.0], 
[4.0, 5.0, 7.0]]], [[[7.0, 7.0, 7.0], [2.0, 1.0, 6.0], [8.0, 8.0, 5.0]], [[8.0, 1.0, 7.0], [1.0, 8.0, 4.0], [6.0, 8.0, 3.0]], 
[[1.0, 5.0, 3.0], [8.0, 8.0, 3.0], [4.0, 3.0, 1.0]]], [[[1.0, 2.0, 8.0], [7.0, 6.0, 3.0], [8.0, 5.0, 4.0]], [[4.0, 6.0, 2.0], 
[4.0, 6.0, 7.0], [8.0, 3.0, 4.0]], [[1.0, 8.0, 3.0], [1.0, 5.0, 7.0], [3.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPPN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 7.0], [5.0, 1.0, 4.0], [2.0, 3.0, 7.0]], [[7.0, 2.0, 7.0], [5.0, 5.0, 8.0], [2.0, 4.0, 
8.0]], [[7.0, 7.0, 8.0], [6.0, 6.0, 8.0], [1.0, 1.0, 3.0]]], [[[2.0, 7.0, 5.0], [4.0, 1.0, 6.0], [3.0, 5.0, 2.0]], [[7.0, 4.0, 
2.0], [4.0, 5.0, 6.0], [4.0, 7.0, 3.0]], [[3.0, 5.0, 3.0], [1.0, 1.0, 5.0], [4.0, 4.0, 3.0]]], [[[6.0, 7.0, 1.0], [2.0, 1.0, 
1.0], [3.0, 1.0, 2.0]], [[7.0, 3.0, 6.0], [7.0, 8.0, 1.0], [3.0, 8.0, 6.0]], [[8.0, 8.0, 5.0], [6.0, 2.0, 4.0], [3.0, 4.0, 
1.0]]], [[[5.0, 7.0, 4.0], [5.0, 1.0, 4.0], [4.0, 7.0, 4.0]], [[4.0, 8.0, 2.0], [1.0, 5.0, 8.0], [1.0, 5.0, 7.0]], [[7.0, 7.0, 
6.0], [7.0, 6.0, 3.0], [3.0, 7.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 7.0], [7.0, 8.0, 1.0], [5.0, 3.0, 4.0]], [[8.0, 1.0, 4.0], [1.0, 1.0, 
5.0], [2.0, 8.0, 4.0]], [[1.0, 8.0, 5.0], [1.0, 7.0, 8.0], [3.0, 5.0, 7.0]]], [[[5.0, 7.0, 7.0], [2.0, 3.0, 6.0], [2.0, 6.0, 
1.0]], [[1.0, 3.0, 4.0], [6.0, 6.0, 8.0], [8.0, 5.0, 4.0]], [[1.0, 7.0, 7.0], [1.0, 7.0, 1.0], [8.0, 1.0, 1.0]]], [[[3.0, 4.0, 
2.0], [6.0, 4.0, 1.0], [8.0, 7.0, 7.0]], [[6.0, 2.0, 7.0], [1.0, 8.0, 8.0], [8.0, 8.0, 3.0]], [[6.0, 5.0, 3.0], [5.0, 1.0, 
8.0], [6.0, 3.0, 7.0]]], [[[6.0, 5.0, 3.0], [4.0, 1.0, 1.0], [8.0, 3.0, 5.0]], [[6.0, 3.0, 7.0], [6.0, 3.0, 8.0], [6.0, 8.0, 
8.0]], [[4.0, 7.0, 3.0], [7.0, 3.0, 3.0], [2.0, 8.0, 6.0]]]]))
      val=Data(numpy.array([6.0, 5.0, 2.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([1.0, 5.0, 3.0]))
      arg[2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 7.0], [5.0, 1.0, 4.0], [2.0, 3.0, 7.0]], [[7.0, 2.0, 7.0], [5.0, 5.0, 8.0], [2.0, 
4.0, 8.0]], [[7.0, 7.0, 8.0], [6.0, 6.0, 8.0], [1.0, 1.0, 3.0]]], [[[2.0, 7.0, 5.0], [4.0, 1.0, 6.0], [3.0, 5.0, 2.0]], [[7.0, 
4.0, 2.0], [4.0, 5.0, 6.0], [4.0, 7.0, 3.0]], [[3.0, 5.0, 3.0], [1.0, 1.0, 5.0], [4.0, 4.0, 3.0]]], [[[6.0, 7.0, 1.0], [2.0, 
1.0, 1.0], [3.0, 1.0, 2.0]], [[7.0, 3.0, 6.0], [6.0, 5.0, 2.0], [3.0, 8.0, 6.0]], [[8.0, 8.0, 5.0], [6.0, 2.0, 4.0], [3.0, 4.0, 
1.0]]], [[[5.0, 7.0, 4.0], [5.0, 1.0, 4.0], [4.0, 7.0, 4.0]], [[4.0, 8.0, 2.0], [1.0, 5.0, 8.0], [1.0, 5.0, 7.0]], [[7.0, 7.0, 
6.0], [7.0, 6.0, 3.0], [3.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 7.0], [7.0, 8.0, 1.0], [5.0, 3.0, 4.0]], 
[[8.0, 1.0, 4.0], [1.0, 1.0, 5.0], [2.0, 8.0, 4.0]], [[1.0, 8.0, 5.0], [1.0, 7.0, 8.0], [3.0, 5.0, 7.0]]], [[[5.0, 7.0, 7.0], 
[2.0, 3.0, 6.0], [2.0, 6.0, 1.0]], [[1.0, 3.0, 4.0], [6.0, 6.0, 8.0], [8.0, 5.0, 4.0]], [[1.0, 7.0, 7.0], [1.0, 7.0, 1.0], 
[8.0, 1.0, 1.0]]], [[[3.0, 4.0, 2.0], [6.0, 4.0, 1.0], [8.0, 7.0, 7.0]], [[6.0, 2.0, 7.0], [1.0, 5.0, 3.0], [8.0, 8.0, 3.0]], 
[[6.0, 5.0, 3.0], [5.0, 1.0, 8.0], [6.0, 3.0, 7.0]]], [[[6.0, 5.0, 3.0], [4.0, 1.0, 1.0], [8.0, 3.0, 5.0]], [[6.0, 3.0, 7.0], 
[6.0, 3.0, 8.0], [6.0, 8.0, 8.0]], [[4.0, 7.0, 3.0], [7.0, 3.0, 3.0], [2.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPPN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 7.0], [1.0, 6.0, 8.0], [6.0, 7.0, 7.0]], [[4.0, 1.0, 5.0], [5.0, 3.0, 4.0], [1.0, 2.0, 
4.0]], [[5.0, 2.0, 2.0], [2.0, 5.0, 3.0], [7.0, 1.0, 1.0]]], [[[3.0, 4.0, 7.0], [1.0, 7.0, 1.0], [5.0, 8.0, 1.0]], [[6.0, 4.0, 
2.0], [7.0, 3.0, 1.0], [3.0, 7.0, 4.0]], [[4.0, 5.0, 8.0], [8.0, 1.0, 6.0], [2.0, 2.0, 1.0]]], [[[6.0, 3.0, 4.0], [6.0, 7.0, 
5.0], [5.0, 4.0, 6.0]], [[5.0, 2.0, 4.0], [7.0, 4.0, 1.0], [4.0, 7.0, 5.0]], [[1.0, 2.0, 2.0], [8.0, 6.0, 7.0], [2.0, 2.0, 
3.0]]], [[[5.0, 7.0, 3.0], [7.0, 2.0, 3.0], [4.0, 8.0, 3.0]], [[6.0, 1.0, 3.0], [3.0, 5.0, 6.0], [8.0, 6.0, 3.0]], [[4.0, 2.0, 
3.0], [3.0, 4.0, 5.0], [1.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 8.0], [1.0, 2.0, 4.0], [3.0, 3.0, 2.0]], [[8.0, 5.0, 6.0], [1.0, 6.0, 
8.0], [2.0, 3.0, 1.0]], [[4.0, 4.0, 4.0], [7.0, 8.0, 7.0], [5.0, 4.0, 2.0]]], [[[6.0, 4.0, 2.0], [4.0, 1.0, 3.0], [8.0, 2.0, 
1.0]], [[2.0, 6.0, 3.0], [2.0, 6.0, 6.0], [1.0, 3.0, 4.0]], [[6.0, 4.0, 6.0], [6.0, 4.0, 5.0], [6.0, 7.0, 6.0]]], [[[2.0, 3.0, 
5.0], [1.0, 4.0, 6.0], [3.0, 1.0, 5.0]], [[6.0, 3.0, 8.0], [2.0, 1.0, 4.0], [8.0, 6.0, 5.0]], [[3.0, 5.0, 7.0], [4.0, 4.0, 
2.0], [7.0, 5.0, 3.0]]], [[[3.0, 5.0, 8.0], [3.0, 4.0, 4.0], [6.0, 5.0, 3.0]], [[2.0, 3.0, 5.0], [6.0, 2.0, 4.0], [1.0, 1.0, 
3.0]], [[7.0, 8.0, 7.0], [8.0, 2.0, 3.0], [7.0, 2.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([2.0, 3.0, 3.0])+(1.-msk_val)*numpy.array([7.0, 3.0, 4.0])
      arg[2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 7.0], [1.0, 6.0, 8.0], [6.0, 7.0, 7.0]], [[4.0, 1.0, 5.0], [5.0, 3.0, 4.0], [1.0, 
2.0, 4.0]], [[5.0, 2.0, 2.0], [2.0, 5.0, 3.0], [7.0, 1.0, 1.0]]], [[[3.0, 4.0, 7.0], [1.0, 7.0, 1.0], [5.0, 8.0, 1.0]], [[6.0, 
4.0, 2.0], [7.0, 3.0, 1.0], [3.0, 7.0, 4.0]], [[4.0, 5.0, 8.0], [8.0, 1.0, 6.0], [2.0, 2.0, 1.0]]], [[[6.0, 3.0, 4.0], [6.0, 
7.0, 5.0], [5.0, 4.0, 6.0]], [[5.0, 2.0, 4.0], [2.0, 3.0, 3.0], [4.0, 7.0, 5.0]], [[1.0, 2.0, 2.0], [8.0, 6.0, 7.0], [2.0, 2.0, 
3.0]]], [[[5.0, 7.0, 3.0], [7.0, 2.0, 3.0], [4.0, 8.0, 3.0]], [[6.0, 1.0, 3.0], [3.0, 5.0, 6.0], [8.0, 6.0, 3.0]], [[4.0, 2.0, 
3.0], [3.0, 4.0, 5.0], [1.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 8.0], [1.0, 2.0, 4.0], [3.0, 3.0, 2.0]], 
[[8.0, 5.0, 6.0], [1.0, 6.0, 8.0], [2.0, 3.0, 1.0]], [[4.0, 4.0, 4.0], [7.0, 8.0, 7.0], [5.0, 4.0, 2.0]]], [[[6.0, 4.0, 2.0], 
[4.0, 1.0, 3.0], [8.0, 2.0, 1.0]], [[2.0, 6.0, 3.0], [2.0, 6.0, 6.0], [1.0, 3.0, 4.0]], [[6.0, 4.0, 6.0], [6.0, 4.0, 5.0], 
[6.0, 7.0, 6.0]]], [[[2.0, 3.0, 5.0], [1.0, 4.0, 6.0], [3.0, 1.0, 5.0]], [[6.0, 3.0, 8.0], [7.0, 3.0, 4.0], [8.0, 6.0, 5.0]], 
[[3.0, 5.0, 7.0], [4.0, 4.0, 2.0], [7.0, 5.0, 3.0]]], [[[3.0, 5.0, 8.0], [3.0, 4.0, 4.0], [6.0, 5.0, 3.0]], [[2.0, 3.0, 5.0], 
[6.0, 2.0, 4.0], [1.0, 1.0, 3.0]], [[7.0, 8.0, 7.0], [8.0, 2.0, 3.0], [7.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UUCN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 1.0], [3.0, 6.0, 6.0], [2.0, 1.0, 7.0]], [[7.0, 5.0, 8.0], [3.0, 6.0, 2.0], [1.0, 1.0, 
8.0]], [[2.0, 7.0, 7.0], [6.0, 1.0, 6.0], [8.0, 3.0, 3.0]]], [[[8.0, 2.0, 4.0], [8.0, 2.0, 6.0], [7.0, 8.0, 3.0]], [[2.0, 3.0, 
3.0], [5.0, 3.0, 6.0], [6.0, 8.0, 7.0]], [[4.0, 5.0, 4.0], [4.0, 7.0, 3.0], [4.0, 6.0, 8.0]]], [[[2.0, 3.0, 3.0], [7.0, 2.0, 
5.0], [5.0, 6.0, 6.0]], [[6.0, 7.0, 3.0], [8.0, 5.0, 7.0], [4.0, 8.0, 8.0]], [[4.0, 8.0, 5.0], [1.0, 6.0, 7.0], [2.0, 4.0, 
3.0]]], [[[5.0, 5.0, 7.0], [7.0, 3.0, 5.0], [6.0, 5.0, 2.0]], [[1.0, 3.0, 1.0], [2.0, 4.0, 4.0], [6.0, 2.0, 4.0]], [[6.0, 2.0, 
7.0], [6.0, 3.0, 3.0], [5.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 1.0], [7.0, 7.0, 3.0], [2.0, 1.0, 5.0]], [[3.0, 8.0, 3.0], [6.0, 3.0, 
3.0], [8.0, 3.0, 2.0]], [[5.0, 3.0, 6.0], [1.0, 8.0, 2.0], [3.0, 3.0, 3.0]]], [[[3.0, 2.0, 6.0], [6.0, 4.0, 4.0], [1.0, 2.0, 
3.0]], [[1.0, 4.0, 6.0], [1.0, 1.0, 4.0], [6.0, 3.0, 5.0]], [[1.0, 6.0, 5.0], [8.0, 7.0, 6.0], [6.0, 7.0, 4.0]]], [[[1.0, 1.0, 
1.0], [4.0, 2.0, 6.0], [2.0, 2.0, 5.0]], [[4.0, 7.0, 6.0], [4.0, 5.0, 7.0], [5.0, 7.0, 4.0]], [[2.0, 5.0, 4.0], [7.0, 8.0, 
6.0], [1.0, 7.0, 1.0]]], [[[1.0, 3.0, 5.0], [5.0, 8.0, 8.0], [2.0, 5.0, 2.0]], [[2.0, 2.0, 2.0], [4.0, 6.0, 4.0], [2.0, 7.0, 
4.0]], [[3.0, 5.0, 5.0], [3.0, 3.0, 6.0], [7.0, 2.0, 3.0]]]]))
      val=numpy.array([[[[2.0, 5.0, 3.0], [1.0, 5.0, 3.0], [6.0, 5.0, 7.0]], [[2.0, 5.0, 5.0], [1.0, 6.0, 4.0], [6.0, 7.0, 
3.0]]], [[[4.0, 4.0, 1.0], [6.0, 7.0, 4.0], [6.0, 1.0, 5.0]], [[5.0, 8.0, 5.0], [8.0, 8.0, 6.0], [1.0, 5.0, 7.0]]], [[[7.0, 
8.0, 6.0], [2.0, 1.0, 3.0], [7.0, 7.0, 4.0]], [[5.0, 5.0, 1.0], [5.0, 6.0, 5.0], [4.0, 6.0, 1.0]]]])
      arg[:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 3.0], [1.0, 5.0, 3.0], [6.0, 5.0, 7.0]], [[2.0, 5.0, 5.0], [1.0, 6.0, 4.0], [6.0, 
7.0, 3.0]], [[2.0, 7.0, 7.0], [6.0, 1.0, 6.0], [8.0, 3.0, 3.0]]], [[[4.0, 4.0, 1.0], [6.0, 7.0, 4.0], [6.0, 1.0, 5.0]], [[5.0, 
8.0, 5.0], [8.0, 8.0, 6.0], [1.0, 5.0, 7.0]], [[4.0, 5.0, 4.0], [4.0, 7.0, 3.0], [4.0, 6.0, 8.0]]], [[[7.0, 8.0, 6.0], [2.0, 
1.0, 3.0], [7.0, 7.0, 4.0]], [[5.0, 5.0, 1.0], [5.0, 6.0, 5.0], [4.0, 6.0, 1.0]], [[4.0, 8.0, 5.0], [1.0, 6.0, 7.0], [2.0, 4.0, 
3.0]]], [[[5.0, 5.0, 7.0], [7.0, 3.0, 5.0], [6.0, 5.0, 2.0]], [[1.0, 3.0, 1.0], [2.0, 4.0, 4.0], [6.0, 2.0, 4.0]], [[6.0, 2.0, 
7.0], [6.0, 3.0, 3.0], [5.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 3.0], [1.0, 5.0, 3.0], [6.0, 5.0, 7.0]], 
[[2.0, 5.0, 5.0], [1.0, 6.0, 4.0], [6.0, 7.0, 3.0]], [[5.0, 3.0, 6.0], [1.0, 8.0, 2.0], [3.0, 3.0, 3.0]]], [[[4.0, 4.0, 1.0], 
[6.0, 7.0, 4.0], [6.0, 1.0, 5.0]], [[5.0, 8.0, 5.0], [8.0, 8.0, 6.0], [1.0, 5.0, 7.0]], [[1.0, 6.0, 5.0], [8.0, 7.0, 6.0], 
[6.0, 7.0, 4.0]]], [[[7.0, 8.0, 6.0], [2.0, 1.0, 3.0], [7.0, 7.0, 4.0]], [[5.0, 5.0, 1.0], [5.0, 6.0, 5.0], [4.0, 6.0, 1.0]], 
[[2.0, 5.0, 4.0], [7.0, 8.0, 6.0], [1.0, 7.0, 1.0]]], [[[1.0, 3.0, 5.0], [5.0, 8.0, 8.0], [2.0, 5.0, 2.0]], [[2.0, 2.0, 2.0], 
[4.0, 6.0, 4.0], [2.0, 7.0, 4.0]], [[3.0, 5.0, 5.0], [3.0, 3.0, 6.0], [7.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UUCN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 8.0], [8.0, 2.0, 4.0], [2.0, 3.0, 7.0]], [[7.0, 4.0, 1.0], [1.0, 6.0, 1.0], [1.0, 3.0, 
5.0]], [[6.0, 6.0, 6.0], [5.0, 2.0, 2.0], [7.0, 7.0, 7.0]]], [[[3.0, 6.0, 4.0], [1.0, 3.0, 1.0], [4.0, 7.0, 8.0]], [[6.0, 5.0, 
5.0], [5.0, 1.0, 7.0], [5.0, 4.0, 6.0]], [[1.0, 2.0, 7.0], [6.0, 7.0, 4.0], [1.0, 5.0, 4.0]]], [[[6.0, 5.0, 2.0], [4.0, 2.0, 
1.0], [1.0, 3.0, 1.0]], [[8.0, 8.0, 7.0], [1.0, 5.0, 1.0], [1.0, 5.0, 1.0]], [[2.0, 1.0, 3.0], [5.0, 1.0, 1.0], [1.0, 5.0, 
6.0]]], [[[8.0, 4.0, 5.0], [6.0, 4.0, 8.0], [8.0, 1.0, 2.0]], [[4.0, 4.0, 1.0], [1.0, 1.0, 8.0], [8.0, 2.0, 6.0]], [[3.0, 8.0, 
5.0], [5.0, 8.0, 4.0], [1.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 1.0], [3.0, 6.0, 2.0], [1.0, 1.0, 1.0]], [[3.0, 4.0, 7.0], [4.0, 8.0, 
8.0], [6.0, 3.0, 6.0]], [[6.0, 6.0, 4.0], [4.0, 2.0, 8.0], [7.0, 4.0, 2.0]]], [[[4.0, 2.0, 4.0], [2.0, 2.0, 7.0], [8.0, 7.0, 
2.0]], [[6.0, 8.0, 5.0], [8.0, 2.0, 3.0], [4.0, 8.0, 3.0]], [[1.0, 8.0, 7.0], [7.0, 4.0, 8.0], [6.0, 3.0, 2.0]]], [[[3.0, 4.0, 
5.0], [6.0, 1.0, 4.0], [5.0, 7.0, 4.0]], [[8.0, 2.0, 4.0], [6.0, 8.0, 5.0], [6.0, 7.0, 1.0]], [[2.0, 3.0, 6.0], [6.0, 8.0, 
5.0], [6.0, 7.0, 4.0]]], [[[3.0, 2.0, 3.0], [7.0, 8.0, 6.0], [7.0, 5.0, 8.0]], [[6.0, 6.0, 3.0], [6.0, 4.0, 6.0], [3.0, 7.0, 
8.0]], [[2.0, 2.0, 2.0], [8.0, 8.0, 1.0], [2.0, 5.0, 5.0]]]]))
      val=Data(numpy.array([[[[1.0, 1.0, 2.0], [3.0, 3.0, 2.0], [6.0, 2.0, 4.0]], [[8.0, 1.0, 8.0], [2.0, 2.0, 3.0], [4.0, 2.0, 
7.0]]], [[[2.0, 3.0, 1.0], [3.0, 4.0, 7.0], [8.0, 6.0, 2.0]], [[3.0, 1.0, 3.0], [7.0, 2.0, 3.0], [5.0, 5.0, 4.0]]], [[[3.0, 
7.0, 5.0], [3.0, 4.0, 6.0], [3.0, 1.0, 5.0]], [[2.0, 6.0, 6.0], [2.0, 7.0, 5.0], [4.0, 7.0, 7.0]]]]),self.functionspace)
      arg[:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 2.0], [3.0, 3.0, 2.0], [6.0, 2.0, 4.0]], [[8.0, 1.0, 8.0], [2.0, 2.0, 3.0], [4.0, 
2.0, 7.0]], [[6.0, 6.0, 6.0], [5.0, 2.0, 2.0], [7.0, 7.0, 7.0]]], [[[2.0, 3.0, 1.0], [3.0, 4.0, 7.0], [8.0, 6.0, 2.0]], [[3.0, 
1.0, 3.0], [7.0, 2.0, 3.0], [5.0, 5.0, 4.0]], [[1.0, 2.0, 7.0], [6.0, 7.0, 4.0], [1.0, 5.0, 4.0]]], [[[3.0, 7.0, 5.0], [3.0, 
4.0, 6.0], [3.0, 1.0, 5.0]], [[2.0, 6.0, 6.0], [2.0, 7.0, 5.0], [4.0, 7.0, 7.0]], [[2.0, 1.0, 3.0], [5.0, 1.0, 1.0], [1.0, 5.0, 
6.0]]], [[[8.0, 4.0, 5.0], [6.0, 4.0, 8.0], [8.0, 1.0, 2.0]], [[4.0, 4.0, 1.0], [1.0, 1.0, 8.0], [8.0, 2.0, 6.0]], [[3.0, 8.0, 
5.0], [5.0, 8.0, 4.0], [1.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 2.0], [3.0, 3.0, 2.0], [6.0, 2.0, 4.0]], 
[[8.0, 1.0, 8.0], [2.0, 2.0, 3.0], [4.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [4.0, 2.0, 8.0], [7.0, 4.0, 2.0]]], [[[2.0, 3.0, 1.0], 
[3.0, 4.0, 7.0], [8.0, 6.0, 2.0]], [[3.0, 1.0, 3.0], [7.0, 2.0, 3.0], [5.0, 5.0, 4.0]], [[1.0, 8.0, 7.0], [7.0, 4.0, 8.0], 
[6.0, 3.0, 2.0]]], [[[3.0, 7.0, 5.0], [3.0, 4.0, 6.0], [3.0, 1.0, 5.0]], [[2.0, 6.0, 6.0], [2.0, 7.0, 5.0], [4.0, 7.0, 7.0]], 
[[2.0, 3.0, 6.0], [6.0, 8.0, 5.0], [6.0, 7.0, 4.0]]], [[[3.0, 2.0, 3.0], [7.0, 8.0, 6.0], [7.0, 5.0, 8.0]], [[6.0, 6.0, 3.0], 
[6.0, 4.0, 6.0], [3.0, 7.0, 8.0]], [[2.0, 2.0, 2.0], [8.0, 8.0, 1.0], [2.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UUCN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 6.0], [8.0, 7.0, 3.0], [7.0, 6.0, 3.0]], [[7.0, 3.0, 2.0], [1.0, 4.0, 3.0], [7.0, 8.0, 
6.0]], [[4.0, 8.0, 8.0], [2.0, 1.0, 6.0], [2.0, 5.0, 6.0]]], [[[1.0, 5.0, 6.0], [2.0, 6.0, 5.0], [7.0, 1.0, 2.0]], [[7.0, 4.0, 
4.0], [7.0, 6.0, 8.0], [3.0, 2.0, 4.0]], [[2.0, 8.0, 8.0], [2.0, 6.0, 3.0], [1.0, 1.0, 4.0]]], [[[5.0, 2.0, 5.0], [7.0, 1.0, 
4.0], [3.0, 7.0, 6.0]], [[6.0, 2.0, 4.0], [1.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[5.0, 1.0, 4.0], [5.0, 6.0, 8.0], [1.0, 5.0, 
7.0]]], [[[6.0, 1.0, 4.0], [8.0, 4.0, 4.0], [7.0, 6.0, 8.0]], [[4.0, 2.0, 8.0], [4.0, 8.0, 5.0], [7.0, 8.0, 3.0]], [[4.0, 8.0, 
3.0], [1.0, 6.0, 8.0], [3.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 8.0], [1.0, 6.0, 2.0], [4.0, 6.0, 1.0]], [[1.0, 2.0, 1.0], [6.0, 7.0, 
5.0], [2.0, 7.0, 4.0]], [[8.0, 1.0, 1.0], [7.0, 7.0, 2.0], [7.0, 6.0, 4.0]]], [[[5.0, 7.0, 6.0], [1.0, 2.0, 8.0], [8.0, 6.0, 
2.0]], [[8.0, 1.0, 6.0], [4.0, 2.0, 8.0], [3.0, 7.0, 4.0]], [[5.0, 5.0, 5.0], [8.0, 3.0, 2.0], [2.0, 3.0, 6.0]]], [[[2.0, 1.0, 
2.0], [2.0, 5.0, 8.0], [1.0, 5.0, 4.0]], [[5.0, 6.0, 4.0], [8.0, 3.0, 2.0], [2.0, 4.0, 7.0]], [[8.0, 5.0, 7.0], [3.0, 5.0, 
3.0], [4.0, 3.0, 2.0]]], [[[2.0, 4.0, 4.0], [4.0, 2.0, 6.0], [8.0, 7.0, 8.0]], [[4.0, 8.0, 4.0], [8.0, 7.0, 2.0], [8.0, 3.0, 
8.0]], [[8.0, 5.0, 6.0], [6.0, 3.0, 8.0], [1.0, 6.0, 3.0]]]]))
      val=Data(numpy.array([[[[7.0, 4.0, 4.0], [4.0, 7.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 7.0, 7.0], [3.0, 6.0, 7.0], [7.0, 1.0, 
6.0]]], [[[2.0, 4.0, 2.0], [4.0, 4.0, 3.0], [4.0, 5.0, 3.0]], [[8.0, 3.0, 5.0], [3.0, 6.0, 3.0], [2.0, 1.0, 4.0]]], [[[1.0, 
7.0, 1.0], [2.0, 1.0, 5.0], [4.0, 3.0, 3.0]], [[7.0, 3.0, 2.0], [4.0, 7.0, 3.0], [8.0, 2.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 8.0], [4.0, 4.0, 7.0], [4.0, 5.0, 3.0]], [[6.0, 5.0, 2.0], [5.0, 3.0, 
6.0], [6.0, 7.0, 5.0]]], [[[6.0, 4.0, 2.0], [3.0, 1.0, 3.0], [4.0, 2.0, 3.0]], [[5.0, 8.0, 2.0], [6.0, 2.0, 1.0], [7.0, 2.0, 
1.0]]], [[[7.0, 4.0, 6.0], [5.0, 1.0, 3.0], [2.0, 2.0, 4.0]], [[7.0, 2.0, 6.0], [2.0, 2.0, 4.0], [7.0, 6.0, 1.0]]]]))
      arg[:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 4.0], [4.0, 7.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 7.0, 7.0], [3.0, 6.0, 7.0], [7.0, 
1.0, 6.0]], [[4.0, 8.0, 8.0], [2.0, 1.0, 6.0], [2.0, 5.0, 6.0]]], [[[2.0, 4.0, 2.0], [4.0, 4.0, 3.0], [4.0, 5.0, 3.0]], [[8.0, 
3.0, 5.0], [3.0, 6.0, 3.0], [2.0, 1.0, 4.0]], [[2.0, 8.0, 8.0], [2.0, 6.0, 3.0], [1.0, 1.0, 4.0]]], [[[1.0, 7.0, 1.0], [2.0, 
1.0, 5.0], [4.0, 3.0, 3.0]], [[7.0, 3.0, 2.0], [4.0, 7.0, 3.0], [8.0, 2.0, 5.0]], [[5.0, 1.0, 4.0], [5.0, 6.0, 8.0], [1.0, 5.0, 
7.0]]], [[[6.0, 1.0, 4.0], [8.0, 4.0, 4.0], [7.0, 6.0, 8.0]], [[4.0, 2.0, 8.0], [4.0, 8.0, 5.0], [7.0, 8.0, 3.0]], [[4.0, 8.0, 
3.0], [1.0, 6.0, 8.0], [3.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 8.0], [4.0, 4.0, 7.0], [4.0, 5.0, 3.0]], 
[[6.0, 5.0, 2.0], [5.0, 3.0, 6.0], [6.0, 7.0, 5.0]], [[8.0, 1.0, 1.0], [7.0, 7.0, 2.0], [7.0, 6.0, 4.0]]], [[[6.0, 4.0, 2.0], 
[3.0, 1.0, 3.0], [4.0, 2.0, 3.0]], [[5.0, 8.0, 2.0], [6.0, 2.0, 1.0], [7.0, 2.0, 1.0]], [[5.0, 5.0, 5.0], [8.0, 3.0, 2.0], 
[2.0, 3.0, 6.0]]], [[[7.0, 4.0, 6.0], [5.0, 1.0, 3.0], [2.0, 2.0, 4.0]], [[7.0, 2.0, 6.0], [2.0, 2.0, 4.0], [7.0, 6.0, 1.0]], 
[[8.0, 5.0, 7.0], [3.0, 5.0, 3.0], [4.0, 3.0, 2.0]]], [[[2.0, 4.0, 4.0], [4.0, 2.0, 6.0], [8.0, 7.0, 8.0]], [[4.0, 8.0, 4.0], 
[8.0, 7.0, 2.0], [8.0, 3.0, 8.0]], [[8.0, 5.0, 6.0], [6.0, 3.0, 8.0], [1.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UUCN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 2.0], [3.0, 7.0, 3.0], [2.0, 2.0, 2.0]], [[2.0, 4.0, 6.0], [8.0, 2.0, 7.0], [3.0, 8.0, 
8.0]], [[4.0, 5.0, 1.0], [3.0, 3.0, 8.0], [4.0, 4.0, 3.0]]], [[[5.0, 8.0, 4.0], [1.0, 2.0, 8.0], [5.0, 8.0, 6.0]], [[5.0, 3.0, 
6.0], [2.0, 8.0, 1.0], [3.0, 3.0, 3.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 6.0], [7.0, 2.0, 7.0]]], [[[1.0, 7.0, 2.0], [7.0, 2.0, 
4.0], [1.0, 7.0, 4.0]], [[4.0, 7.0, 4.0], [2.0, 6.0, 6.0], [7.0, 3.0, 4.0]], [[1.0, 1.0, 7.0], [3.0, 3.0, 1.0], [8.0, 3.0, 
5.0]]], [[[8.0, 7.0, 7.0], [2.0, 3.0, 4.0], [3.0, 5.0, 6.0]], [[7.0, 1.0, 8.0], [6.0, 7.0, 2.0], [5.0, 8.0, 4.0]], [[3.0, 8.0, 
3.0], [2.0, 4.0, 7.0], [7.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 2.0], [3.0, 1.0, 1.0], [8.0, 7.0, 8.0]], [[7.0, 1.0, 2.0], [3.0, 5.0, 
7.0], [7.0, 4.0, 7.0]], [[5.0, 1.0, 3.0], [7.0, 8.0, 6.0], [8.0, 5.0, 4.0]]], [[[1.0, 6.0, 5.0], [2.0, 7.0, 4.0], [5.0, 4.0, 
2.0]], [[2.0, 8.0, 5.0], [8.0, 2.0, 3.0], [6.0, 8.0, 5.0]], [[3.0, 7.0, 4.0], [5.0, 6.0, 5.0], [8.0, 5.0, 4.0]]], [[[3.0, 8.0, 
1.0], [6.0, 2.0, 8.0], [4.0, 8.0, 8.0]], [[3.0, 8.0, 7.0], [7.0, 1.0, 8.0], [4.0, 2.0, 8.0]], [[7.0, 4.0, 6.0], [7.0, 8.0, 
1.0], [8.0, 1.0, 2.0]]], [[[2.0, 8.0, 1.0], [8.0, 2.0, 6.0], [2.0, 6.0, 4.0]], [[6.0, 6.0, 3.0], [3.0, 6.0, 7.0], [4.0, 5.0, 
5.0]], [[1.0, 5.0, 3.0], [2.0, 6.0, 8.0], [5.0, 6.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 2.0, 8.0], [7.0, 1.0, 3.0], [5.0, 6.0, 5.0]], [[6.0, 2.0, 4.0], [8.0, 6.0, 7.0], [2.0, 
3.0, 5.0]]], [[[8.0, 4.0, 6.0], [4.0, 4.0, 1.0], [2.0, 1.0, 5.0]], [[2.0, 3.0, 2.0], [5.0, 5.0, 3.0], [8.0, 5.0, 4.0]]], 
[[[3.0, 4.0, 5.0], [4.0, 5.0, 4.0], [8.0, 6.0, 5.0]], [[7.0, 1.0, 8.0], [5.0, 7.0, 4.0], [1.0, 2.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 1.0, 7.0], [6.0, 7.0, 7.0], [8.0, 6.0, 1.0]], [[1.0, 7.0, 7.0], [7.0, 6.0, 8.0], 
[8.0, 3.0, 3.0]]], [[[5.0, 5.0, 3.0], [8.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[7.0, 1.0, 3.0], [3.0, 6.0, 2.0], [7.0, 6.0, 3.0]]], 
[[[2.0, 1.0, 6.0], [2.0, 2.0, 3.0], [2.0, 2.0, 5.0]], [[2.0, 1.0, 7.0], [5.0, 8.0, 4.0], [6.0, 6.0, 2.0]]]])
      arg[:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 8.0], [7.0, 1.0, 3.0], [5.0, 6.0, 5.0]], [[6.0, 2.0, 4.0], [8.0, 6.0, 7.0], [2.0, 
3.0, 5.0]], [[4.0, 5.0, 1.0], [3.0, 3.0, 8.0], [4.0, 4.0, 3.0]]], [[[8.0, 4.0, 6.0], [4.0, 4.0, 1.0], [2.0, 1.0, 5.0]], [[2.0, 
3.0, 2.0], [5.0, 5.0, 3.0], [8.0, 5.0, 4.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 6.0], [7.0, 2.0, 7.0]]], [[[3.0, 4.0, 5.0], [4.0, 
5.0, 4.0], [8.0, 6.0, 5.0]], [[7.0, 1.0, 8.0], [5.0, 7.0, 4.0], [1.0, 2.0, 7.0]], [[1.0, 1.0, 7.0], [3.0, 3.0, 1.0], [8.0, 3.0, 
5.0]]], [[[8.0, 7.0, 7.0], [2.0, 3.0, 4.0], [3.0, 5.0, 6.0]], [[7.0, 1.0, 8.0], [6.0, 7.0, 2.0], [5.0, 8.0, 4.0]], [[3.0, 8.0, 
3.0], [2.0, 4.0, 7.0], [7.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 7.0], [6.0, 7.0, 7.0], [8.0, 6.0, 1.0]], 
[[1.0, 7.0, 7.0], [7.0, 6.0, 8.0], [8.0, 3.0, 3.0]], [[5.0, 1.0, 3.0], [7.0, 8.0, 6.0], [8.0, 5.0, 4.0]]], [[[5.0, 5.0, 3.0], 
[8.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[7.0, 1.0, 3.0], [3.0, 6.0, 2.0], [7.0, 6.0, 3.0]], [[3.0, 7.0, 4.0], [5.0, 6.0, 5.0], 
[8.0, 5.0, 4.0]]], [[[2.0, 1.0, 6.0], [2.0, 2.0, 3.0], [2.0, 2.0, 5.0]], [[2.0, 1.0, 7.0], [5.0, 8.0, 4.0], [6.0, 6.0, 2.0]], 
[[7.0, 4.0, 6.0], [7.0, 8.0, 1.0], [8.0, 1.0, 2.0]]], [[[2.0, 8.0, 1.0], [8.0, 2.0, 6.0], [2.0, 6.0, 4.0]], [[6.0, 6.0, 3.0], 
[3.0, 6.0, 7.0], [4.0, 5.0, 5.0]], [[1.0, 5.0, 3.0], [2.0, 6.0, 8.0], [5.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCPN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 8.0], [8.0, 6.0, 1.0], [5.0, 1.0, 5.0]], [[3.0, 4.0, 1.0], [6.0, 8.0, 3.0], [6.0, 4.0, 
5.0]], [[6.0, 4.0, 8.0], [3.0, 8.0, 3.0], [3.0, 6.0, 8.0]]], [[[1.0, 3.0, 4.0], [6.0, 4.0, 3.0], [8.0, 8.0, 2.0]], [[8.0, 7.0, 
1.0], [4.0, 4.0, 2.0], [3.0, 6.0, 1.0]], [[4.0, 7.0, 5.0], [8.0, 5.0, 1.0], [6.0, 6.0, 7.0]]], [[[5.0, 7.0, 8.0], [8.0, 1.0, 
4.0], [8.0, 3.0, 4.0]], [[6.0, 6.0, 6.0], [8.0, 3.0, 2.0], [6.0, 2.0, 4.0]], [[4.0, 8.0, 1.0], [3.0, 6.0, 6.0], [7.0, 2.0, 
1.0]]], [[[6.0, 4.0, 4.0], [5.0, 5.0, 2.0], [4.0, 6.0, 8.0]], [[3.0, 7.0, 2.0], [4.0, 7.0, 7.0], [4.0, 7.0, 7.0]], [[7.0, 8.0, 
6.0], [3.0, 6.0, 5.0], [6.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 1.0], [1.0, 7.0, 1.0], [4.0, 3.0, 2.0]], [[4.0, 1.0, 1.0], [8.0, 5.0, 
3.0], [5.0, 8.0, 8.0]], [[3.0, 4.0, 8.0], [4.0, 5.0, 2.0], [4.0, 8.0, 5.0]]], [[[2.0, 2.0, 3.0], [8.0, 2.0, 8.0], [1.0, 6.0, 
5.0]], [[4.0, 4.0, 3.0], [8.0, 4.0, 4.0], [3.0, 3.0, 5.0]], [[4.0, 6.0, 3.0], [6.0, 1.0, 1.0], [8.0, 1.0, 2.0]]], [[[6.0, 7.0, 
7.0], [1.0, 3.0, 6.0], [7.0, 5.0, 2.0]], [[1.0, 5.0, 2.0], [8.0, 1.0, 3.0], [4.0, 7.0, 5.0]], [[5.0, 7.0, 4.0], [2.0, 4.0, 
4.0], [7.0, 2.0, 8.0]]], [[[6.0, 3.0, 7.0], [2.0, 7.0, 5.0], [1.0, 6.0, 8.0]], [[6.0, 7.0, 6.0], [4.0, 4.0, 7.0], [2.0, 4.0, 
5.0]], [[1.0, 1.0, 1.0], [6.0, 3.0, 5.0], [4.0, 8.0, 5.0]]]]))
      val=numpy.array([[[4.0, 5.0, 3.0], [1.0, 6.0, 7.0], [8.0, 5.0, 6.0]], [[6.0, 1.0, 1.0], [8.0, 3.0, 5.0], [4.0, 7.0, 
3.0]], [[7.0, 8.0, 2.0], [8.0, 2.0, 5.0], [8.0, 4.0, 3.0]], [[2.0, 2.0, 1.0], [1.0, 2.0, 3.0], [7.0, 7.0, 8.0]]])
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 8.0], [4.0, 5.0, 3.0], [5.0, 1.0, 5.0]], [[3.0, 4.0, 1.0], [1.0, 6.0, 7.0], [6.0, 
4.0, 5.0]], [[6.0, 4.0, 8.0], [8.0, 5.0, 6.0], [3.0, 6.0, 8.0]]], [[[1.0, 3.0, 4.0], [6.0, 1.0, 1.0], [8.0, 8.0, 2.0]], [[8.0, 
7.0, 1.0], [8.0, 3.0, 5.0], [3.0, 6.0, 1.0]], [[4.0, 7.0, 5.0], [4.0, 7.0, 3.0], [6.0, 6.0, 7.0]]], [[[5.0, 7.0, 8.0], [7.0, 
8.0, 2.0], [8.0, 3.0, 4.0]], [[6.0, 6.0, 6.0], [8.0, 2.0, 5.0], [6.0, 2.0, 4.0]], [[4.0, 8.0, 1.0], [8.0, 4.0, 3.0], [7.0, 2.0, 
1.0]]], [[[6.0, 4.0, 4.0], [2.0, 2.0, 1.0], [4.0, 6.0, 8.0]], [[3.0, 7.0, 2.0], [1.0, 2.0, 3.0], [4.0, 7.0, 7.0]], [[7.0, 8.0, 
6.0], [7.0, 7.0, 8.0], [6.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 1.0], [4.0, 5.0, 3.0], [4.0, 3.0, 2.0]], 
[[4.0, 1.0, 1.0], [1.0, 6.0, 7.0], [5.0, 8.0, 8.0]], [[3.0, 4.0, 8.0], [8.0, 5.0, 6.0], [4.0, 8.0, 5.0]]], [[[2.0, 2.0, 3.0], 
[6.0, 1.0, 1.0], [1.0, 6.0, 5.0]], [[4.0, 4.0, 3.0], [8.0, 3.0, 5.0], [3.0, 3.0, 5.0]], [[4.0, 6.0, 3.0], [4.0, 7.0, 3.0], 
[8.0, 1.0, 2.0]]], [[[6.0, 7.0, 7.0], [7.0, 8.0, 2.0], [7.0, 5.0, 2.0]], [[1.0, 5.0, 2.0], [8.0, 2.0, 5.0], [4.0, 7.0, 5.0]], 
[[5.0, 7.0, 4.0], [8.0, 4.0, 3.0], [7.0, 2.0, 8.0]]], [[[6.0, 3.0, 7.0], [2.0, 2.0, 1.0], [1.0, 6.0, 8.0]], [[6.0, 7.0, 6.0], 
[1.0, 2.0, 3.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 1.0], [7.0, 7.0, 8.0], [4.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCPN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 2.0], [3.0, 6.0, 4.0], [6.0, 7.0, 2.0]], [[2.0, 2.0, 6.0], [2.0, 1.0, 7.0], [6.0, 2.0, 
2.0]], [[2.0, 7.0, 7.0], [4.0, 6.0, 1.0], [4.0, 7.0, 4.0]]], [[[5.0, 7.0, 8.0], [7.0, 7.0, 1.0], [8.0, 5.0, 4.0]], [[2.0, 5.0, 
2.0], [8.0, 4.0, 4.0], [8.0, 6.0, 7.0]], [[3.0, 2.0, 5.0], [2.0, 8.0, 3.0], [1.0, 6.0, 6.0]]], [[[2.0, 5.0, 3.0], [2.0, 1.0, 
8.0], [7.0, 5.0, 4.0]], [[4.0, 8.0, 4.0], [4.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 1.0], [2.0, 4.0, 5.0], [1.0, 7.0, 
1.0]]], [[[5.0, 6.0, 5.0], [6.0, 8.0, 6.0], [3.0, 6.0, 6.0]], [[3.0, 1.0, 2.0], [1.0, 7.0, 2.0], [3.0, 2.0, 3.0]], [[4.0, 4.0, 
1.0], [2.0, 1.0, 8.0], [1.0, 8.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 2.0], [4.0, 5.0, 5.0], [3.0, 6.0, 5.0]], [[3.0, 6.0, 6.0], [1.0, 7.0, 
4.0], [3.0, 3.0, 7.0]], [[7.0, 3.0, 6.0], [3.0, 7.0, 6.0], [5.0, 1.0, 3.0]]], [[[6.0, 4.0, 4.0], [8.0, 1.0, 5.0], [8.0, 3.0, 
6.0]], [[1.0, 7.0, 1.0], [2.0, 3.0, 4.0], [4.0, 4.0, 6.0]], [[2.0, 7.0, 4.0], [8.0, 4.0, 8.0], [8.0, 4.0, 6.0]]], [[[2.0, 6.0, 
3.0], [6.0, 2.0, 2.0], [8.0, 6.0, 1.0]], [[8.0, 8.0, 4.0], [3.0, 6.0, 4.0], [5.0, 7.0, 5.0]], [[1.0, 2.0, 4.0], [1.0, 5.0, 
8.0], [3.0, 5.0, 3.0]]], [[[5.0, 5.0, 5.0], [7.0, 1.0, 7.0], [1.0, 8.0, 5.0]], [[4.0, 7.0, 7.0], [3.0, 8.0, 1.0], [4.0, 5.0, 
3.0]], [[6.0, 2.0, 7.0], [7.0, 1.0, 5.0], [5.0, 2.0, 5.0]]]]))
      val=Data(numpy.array([[[3.0, 1.0, 6.0], [4.0, 1.0, 8.0], [4.0, 4.0, 7.0]], [[8.0, 6.0, 2.0], [7.0, 7.0, 7.0], [1.0, 7.0, 
2.0]], [[7.0, 1.0, 7.0], [1.0, 3.0, 3.0], [3.0, 4.0, 2.0]], [[2.0, 7.0, 5.0], [8.0, 6.0, 4.0], [6.0, 5.0, 
7.0]]]),self.functionspace)
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 2.0], [3.0, 1.0, 6.0], [6.0, 7.0, 2.0]], [[2.0, 2.0, 6.0], [4.0, 1.0, 8.0], [6.0, 
2.0, 2.0]], [[2.0, 7.0, 7.0], [4.0, 4.0, 7.0], [4.0, 7.0, 4.0]]], [[[5.0, 7.0, 8.0], [8.0, 6.0, 2.0], [8.0, 5.0, 4.0]], [[2.0, 
5.0, 2.0], [7.0, 7.0, 7.0], [8.0, 6.0, 7.0]], [[3.0, 2.0, 5.0], [1.0, 7.0, 2.0], [1.0, 6.0, 6.0]]], [[[2.0, 5.0, 3.0], [7.0, 
1.0, 7.0], [7.0, 5.0, 4.0]], [[4.0, 8.0, 4.0], [1.0, 3.0, 3.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 1.0], [3.0, 4.0, 2.0], [1.0, 7.0, 
1.0]]], [[[5.0, 6.0, 5.0], [2.0, 7.0, 5.0], [3.0, 6.0, 6.0]], [[3.0, 1.0, 2.0], [8.0, 6.0, 4.0], [3.0, 2.0, 3.0]], [[4.0, 4.0, 
1.0], [6.0, 5.0, 7.0], [1.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 2.0], [3.0, 1.0, 6.0], [3.0, 6.0, 5.0]], 
[[3.0, 6.0, 6.0], [4.0, 1.0, 8.0], [3.0, 3.0, 7.0]], [[7.0, 3.0, 6.0], [4.0, 4.0, 7.0], [5.0, 1.0, 3.0]]], [[[6.0, 4.0, 4.0], 
[8.0, 6.0, 2.0], [8.0, 3.0, 6.0]], [[1.0, 7.0, 1.0], [7.0, 7.0, 7.0], [4.0, 4.0, 6.0]], [[2.0, 7.0, 4.0], [1.0, 7.0, 2.0], 
[8.0, 4.0, 6.0]]], [[[2.0, 6.0, 3.0], [7.0, 1.0, 7.0], [8.0, 6.0, 1.0]], [[8.0, 8.0, 4.0], [1.0, 3.0, 3.0], [5.0, 7.0, 5.0]], 
[[1.0, 2.0, 4.0], [3.0, 4.0, 2.0], [3.0, 5.0, 3.0]]], [[[5.0, 5.0, 5.0], [2.0, 7.0, 5.0], [1.0, 8.0, 5.0]], [[4.0, 7.0, 7.0], 
[8.0, 6.0, 4.0], [4.0, 5.0, 3.0]], [[6.0, 2.0, 7.0], [6.0, 5.0, 7.0], [5.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCPN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 3.0], [6.0, 8.0, 7.0], [2.0, 6.0, 3.0]], [[3.0, 7.0, 4.0], [6.0, 7.0, 4.0], [7.0, 2.0, 
5.0]], [[4.0, 2.0, 7.0], [1.0, 7.0, 1.0], [5.0, 5.0, 6.0]]], [[[1.0, 2.0, 6.0], [7.0, 5.0, 6.0], [6.0, 7.0, 7.0]], [[5.0, 7.0, 
3.0], [3.0, 4.0, 5.0], [5.0, 8.0, 4.0]], [[6.0, 2.0, 4.0], [4.0, 5.0, 1.0], [5.0, 7.0, 8.0]]], [[[5.0, 5.0, 2.0], [6.0, 2.0, 
7.0], [2.0, 8.0, 1.0]], [[1.0, 8.0, 8.0], [6.0, 2.0, 3.0], [6.0, 2.0, 3.0]], [[6.0, 1.0, 3.0], [1.0, 7.0, 3.0], [8.0, 3.0, 
2.0]]], [[[7.0, 3.0, 2.0], [8.0, 5.0, 8.0], [7.0, 7.0, 6.0]], [[1.0, 4.0, 5.0], [4.0, 7.0, 1.0], [8.0, 6.0, 1.0]], [[8.0, 1.0, 
1.0], [8.0, 6.0, 1.0], [3.0, 5.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 4.0], [2.0, 1.0, 3.0], [6.0, 2.0, 8.0]], [[3.0, 2.0, 1.0], [3.0, 1.0, 
7.0], [1.0, 6.0, 3.0]], [[7.0, 8.0, 2.0], [5.0, 7.0, 4.0], [1.0, 6.0, 1.0]]], [[[7.0, 3.0, 4.0], [1.0, 2.0, 1.0], [4.0, 3.0, 
6.0]], [[1.0, 7.0, 8.0], [6.0, 6.0, 6.0], [7.0, 2.0, 6.0]], [[3.0, 4.0, 7.0], [4.0, 2.0, 7.0], [4.0, 2.0, 8.0]]], [[[4.0, 7.0, 
4.0], [2.0, 2.0, 5.0], [7.0, 1.0, 5.0]], [[5.0, 6.0, 3.0], [1.0, 6.0, 6.0], [3.0, 5.0, 1.0]], [[7.0, 4.0, 7.0], [4.0, 6.0, 
5.0], [2.0, 3.0, 2.0]]], [[[8.0, 5.0, 6.0], [6.0, 2.0, 1.0], [6.0, 1.0, 8.0]], [[8.0, 3.0, 3.0], [4.0, 1.0, 4.0], [3.0, 8.0, 
4.0]], [[1.0, 2.0, 2.0], [2.0, 2.0, 7.0], [7.0, 3.0, 1.0]]]]))
      val=Data(numpy.array([[[4.0, 6.0, 6.0], [4.0, 7.0, 6.0], [4.0, 5.0, 2.0]], [[3.0, 7.0, 3.0], [8.0, 6.0, 5.0], [2.0, 1.0, 
7.0]], [[1.0, 3.0, 7.0], [8.0, 5.0, 4.0], [4.0, 4.0, 2.0]], [[2.0, 3.0, 2.0], [7.0, 5.0, 1.0], [8.0, 3.0, 
8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 1.0, 7.0], [3.0, 7.0, 6.0], [5.0, 8.0, 1.0]], [[7.0, 6.0, 2.0], [6.0, 5.0, 1.0], 
[7.0, 5.0, 3.0]], [[3.0, 4.0, 4.0], [2.0, 5.0, 8.0], [2.0, 3.0, 3.0]], [[8.0, 3.0, 3.0], [1.0, 4.0, 2.0], [5.0, 5.0, 4.0]]]))
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 3.0], [4.0, 6.0, 6.0], [2.0, 6.0, 3.0]], [[3.0, 7.0, 4.0], [4.0, 7.0, 6.0], [7.0, 
2.0, 5.0]], [[4.0, 2.0, 7.0], [4.0, 5.0, 2.0], [5.0, 5.0, 6.0]]], [[[1.0, 2.0, 6.0], [3.0, 7.0, 3.0], [6.0, 7.0, 7.0]], [[5.0, 
7.0, 3.0], [8.0, 6.0, 5.0], [5.0, 8.0, 4.0]], [[6.0, 2.0, 4.0], [2.0, 1.0, 7.0], [5.0, 7.0, 8.0]]], [[[5.0, 5.0, 2.0], [1.0, 
3.0, 7.0], [2.0, 8.0, 1.0]], [[1.0, 8.0, 8.0], [8.0, 5.0, 4.0], [6.0, 2.0, 3.0]], [[6.0, 1.0, 3.0], [4.0, 4.0, 2.0], [8.0, 3.0, 
2.0]]], [[[7.0, 3.0, 2.0], [2.0, 3.0, 2.0], [7.0, 7.0, 6.0]], [[1.0, 4.0, 5.0], [7.0, 5.0, 1.0], [8.0, 6.0, 1.0]], [[8.0, 1.0, 
1.0], [8.0, 3.0, 8.0], [3.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 4.0], [8.0, 1.0, 7.0], [6.0, 2.0, 8.0]], 
[[3.0, 2.0, 1.0], [3.0, 7.0, 6.0], [1.0, 6.0, 3.0]], [[7.0, 8.0, 2.0], [5.0, 8.0, 1.0], [1.0, 6.0, 1.0]]], [[[7.0, 3.0, 4.0], 
[7.0, 6.0, 2.0], [4.0, 3.0, 6.0]], [[1.0, 7.0, 8.0], [6.0, 5.0, 1.0], [7.0, 2.0, 6.0]], [[3.0, 4.0, 7.0], [7.0, 5.0, 3.0], 
[4.0, 2.0, 8.0]]], [[[4.0, 7.0, 4.0], [3.0, 4.0, 4.0], [7.0, 1.0, 5.0]], [[5.0, 6.0, 3.0], [2.0, 5.0, 8.0], [3.0, 5.0, 1.0]], 
[[7.0, 4.0, 7.0], [2.0, 3.0, 3.0], [2.0, 3.0, 2.0]]], [[[8.0, 5.0, 6.0], [8.0, 3.0, 3.0], [6.0, 1.0, 8.0]], [[8.0, 3.0, 3.0], 
[1.0, 4.0, 2.0], [3.0, 8.0, 4.0]], [[1.0, 2.0, 2.0], [5.0, 5.0, 4.0], [7.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCPN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 6.0], [2.0, 3.0, 2.0], [1.0, 3.0, 6.0]], [[3.0, 7.0, 3.0], [5.0, 3.0, 3.0], [6.0, 1.0, 
1.0]], [[2.0, 4.0, 7.0], [8.0, 5.0, 3.0], [6.0, 4.0, 1.0]]], [[[3.0, 3.0, 7.0], [3.0, 8.0, 1.0], [4.0, 5.0, 8.0]], [[5.0, 6.0, 
7.0], [6.0, 5.0, 6.0], [8.0, 6.0, 1.0]], [[5.0, 2.0, 2.0], [6.0, 5.0, 2.0], [6.0, 7.0, 2.0]]], [[[3.0, 5.0, 3.0], [2.0, 8.0, 
7.0], [1.0, 5.0, 2.0]], [[1.0, 7.0, 1.0], [4.0, 1.0, 1.0], [6.0, 1.0, 7.0]], [[3.0, 3.0, 5.0], [7.0, 5.0, 4.0], [2.0, 1.0, 
8.0]]], [[[7.0, 1.0, 7.0], [6.0, 8.0, 1.0], [3.0, 5.0, 3.0]], [[2.0, 4.0, 7.0], [5.0, 8.0, 5.0], [7.0, 3.0, 6.0]], [[8.0, 5.0, 
7.0], [8.0, 5.0, 8.0], [4.0, 6.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 6.0], [5.0, 1.0, 6.0], [2.0, 5.0, 5.0]], [[3.0, 4.0, 6.0], [2.0, 4.0, 
2.0], [7.0, 1.0, 5.0]], [[8.0, 1.0, 5.0], [7.0, 8.0, 6.0], [8.0, 4.0, 6.0]]], [[[3.0, 2.0, 6.0], [2.0, 1.0, 8.0], [7.0, 7.0, 
5.0]], [[8.0, 5.0, 2.0], [5.0, 8.0, 3.0], [5.0, 2.0, 7.0]], [[1.0, 4.0, 6.0], [1.0, 4.0, 2.0], [6.0, 1.0, 5.0]]], [[[4.0, 8.0, 
3.0], [2.0, 4.0, 7.0], [7.0, 5.0, 6.0]], [[5.0, 3.0, 6.0], [3.0, 1.0, 7.0], [4.0, 7.0, 3.0]], [[5.0, 7.0, 4.0], [7.0, 4.0, 
4.0], [1.0, 1.0, 4.0]]], [[[2.0, 3.0, 7.0], [3.0, 8.0, 8.0], [1.0, 7.0, 4.0]], [[2.0, 4.0, 2.0], [4.0, 3.0, 2.0], [1.0, 8.0, 
3.0]], [[1.0, 4.0, 5.0], [5.0, 4.0, 1.0], [6.0, 8.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[4.0, 5.0, 6.0], [3.0, 1.0, 4.0], [5.0, 3.0, 7.0]], [[5.0, 3.0, 7.0], [2.0, 3.0, 3.0], [5.0, 
6.0, 3.0]], [[4.0, 8.0, 2.0], [2.0, 4.0, 5.0], [2.0, 2.0, 2.0]], [[7.0, 2.0, 4.0], [5.0, 5.0, 2.0], [5.0, 3.0, 
3.0]]])+(1.-msk_val)*numpy.array([[[3.0, 2.0, 2.0], [3.0, 8.0, 5.0], [5.0, 5.0, 4.0]], [[1.0, 6.0, 2.0], [7.0, 7.0, 6.0], [5.0, 
1.0, 7.0]], [[2.0, 1.0, 2.0], [4.0, 8.0, 8.0], [3.0, 1.0, 4.0]], [[5.0, 3.0, 1.0], [7.0, 6.0, 4.0], [7.0, 3.0, 5.0]]])
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 6.0], [4.0, 5.0, 6.0], [1.0, 3.0, 6.0]], [[3.0, 7.0, 3.0], [3.0, 1.0, 4.0], [6.0, 
1.0, 1.0]], [[2.0, 4.0, 7.0], [5.0, 3.0, 7.0], [6.0, 4.0, 1.0]]], [[[3.0, 3.0, 7.0], [5.0, 3.0, 7.0], [4.0, 5.0, 8.0]], [[5.0, 
6.0, 7.0], [2.0, 3.0, 3.0], [8.0, 6.0, 1.0]], [[5.0, 2.0, 2.0], [5.0, 6.0, 3.0], [6.0, 7.0, 2.0]]], [[[3.0, 5.0, 3.0], [4.0, 
8.0, 2.0], [1.0, 5.0, 2.0]], [[1.0, 7.0, 1.0], [2.0, 4.0, 5.0], [6.0, 1.0, 7.0]], [[3.0, 3.0, 5.0], [2.0, 2.0, 2.0], [2.0, 1.0, 
8.0]]], [[[7.0, 1.0, 7.0], [7.0, 2.0, 4.0], [3.0, 5.0, 3.0]], [[2.0, 4.0, 7.0], [5.0, 5.0, 2.0], [7.0, 3.0, 6.0]], [[8.0, 5.0, 
7.0], [5.0, 3.0, 3.0], [4.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 6.0], [3.0, 2.0, 2.0], [2.0, 5.0, 5.0]], 
[[3.0, 4.0, 6.0], [3.0, 8.0, 5.0], [7.0, 1.0, 5.0]], [[8.0, 1.0, 5.0], [5.0, 5.0, 4.0], [8.0, 4.0, 6.0]]], [[[3.0, 2.0, 6.0], 
[1.0, 6.0, 2.0], [7.0, 7.0, 5.0]], [[8.0, 5.0, 2.0], [7.0, 7.0, 6.0], [5.0, 2.0, 7.0]], [[1.0, 4.0, 6.0], [5.0, 1.0, 7.0], 
[6.0, 1.0, 5.0]]], [[[4.0, 8.0, 3.0], [2.0, 1.0, 2.0], [7.0, 5.0, 6.0]], [[5.0, 3.0, 6.0], [4.0, 8.0, 8.0], [4.0, 7.0, 3.0]], 
[[5.0, 7.0, 4.0], [3.0, 1.0, 4.0], [1.0, 1.0, 4.0]]], [[[2.0, 3.0, 7.0], [5.0, 3.0, 1.0], [1.0, 7.0, 4.0]], [[2.0, 4.0, 2.0], 
[7.0, 6.0, 4.0], [1.0, 8.0, 3.0]], [[1.0, 4.0, 5.0], [7.0, 3.0, 5.0], [6.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LRCN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 1.0], [2.0, 7.0, 2.0], [6.0, 8.0, 7.0]], [[6.0, 1.0, 3.0], [8.0, 7.0, 5.0], [4.0, 2.0, 
6.0]], [[8.0, 8.0, 7.0], [6.0, 7.0, 6.0], [7.0, 7.0, 5.0]]], [[[2.0, 3.0, 1.0], [3.0, 4.0, 8.0], [8.0, 6.0, 6.0]], [[1.0, 1.0, 
6.0], [3.0, 1.0, 2.0], [7.0, 5.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 3.0, 6.0], [6.0, 2.0, 5.0]]], [[[5.0, 5.0, 8.0], [1.0, 6.0, 
2.0], [5.0, 2.0, 6.0]], [[3.0, 4.0, 2.0], [2.0, 1.0, 1.0], [3.0, 1.0, 7.0]], [[2.0, 4.0, 8.0], [4.0, 8.0, 2.0], [6.0, 6.0, 
2.0]]], [[[4.0, 6.0, 7.0], [5.0, 4.0, 3.0], [4.0, 4.0, 1.0]], [[8.0, 4.0, 2.0], [4.0, 1.0, 4.0], [6.0, 4.0, 7.0]], [[1.0, 4.0, 
8.0], [5.0, 8.0, 1.0], [7.0, 7.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 4.0], [3.0, 3.0, 7.0], [8.0, 1.0, 8.0]], [[4.0, 3.0, 4.0], [1.0, 2.0, 
7.0], [8.0, 2.0, 1.0]], [[8.0, 8.0, 6.0], [1.0, 7.0, 1.0], [1.0, 1.0, 4.0]]], [[[8.0, 2.0, 1.0], [2.0, 1.0, 1.0], [1.0, 1.0, 
4.0]], [[8.0, 8.0, 7.0], [6.0, 1.0, 4.0], [7.0, 5.0, 8.0]], [[1.0, 8.0, 7.0], [4.0, 1.0, 1.0], [8.0, 1.0, 6.0]]], [[[5.0, 4.0, 
7.0], [4.0, 3.0, 6.0], [2.0, 6.0, 8.0]], [[3.0, 5.0, 3.0], [4.0, 1.0, 6.0], [5.0, 7.0, 1.0]], [[8.0, 2.0, 1.0], [5.0, 5.0, 
3.0], [1.0, 4.0, 3.0]]], [[[8.0, 4.0, 2.0], [8.0, 8.0, 8.0], [6.0, 5.0, 5.0]], [[3.0, 2.0, 7.0], [7.0, 3.0, 8.0], [3.0, 7.0, 
4.0]], [[7.0, 7.0, 3.0], [5.0, 2.0, 8.0], [8.0, 4.0, 6.0]]]]))
      val=numpy.array([[[[3.0, 3.0, 3.0], [2.0, 1.0, 8.0], [3.0, 6.0, 5.0]]], [[[6.0, 6.0, 1.0], [3.0, 1.0, 1.0], [3.0, 7.0, 
8.0]]]])
      arg[2:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 1.0], [2.0, 7.0, 2.0], [6.0, 8.0, 7.0]], [[6.0, 1.0, 3.0], [8.0, 7.0, 5.0], [4.0, 
2.0, 6.0]], [[8.0, 8.0, 7.0], [6.0, 7.0, 6.0], [7.0, 7.0, 5.0]]], [[[2.0, 3.0, 1.0], [3.0, 4.0, 8.0], [8.0, 6.0, 6.0]], [[1.0, 
1.0, 6.0], [3.0, 1.0, 2.0], [7.0, 5.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 3.0, 6.0], [6.0, 2.0, 5.0]]], [[[5.0, 5.0, 8.0], [1.0, 
6.0, 2.0], [5.0, 2.0, 6.0]], [[3.0, 3.0, 3.0], [2.0, 1.0, 8.0], [3.0, 6.0, 5.0]], [[2.0, 4.0, 8.0], [4.0, 8.0, 2.0], [6.0, 6.0, 
2.0]]], [[[4.0, 6.0, 7.0], [5.0, 4.0, 3.0], [4.0, 4.0, 1.0]], [[6.0, 6.0, 1.0], [3.0, 1.0, 1.0], [3.0, 7.0, 8.0]], [[1.0, 4.0, 
8.0], [5.0, 8.0, 1.0], [7.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 4.0], [3.0, 3.0, 7.0], [8.0, 1.0, 8.0]], 
[[4.0, 3.0, 4.0], [1.0, 2.0, 7.0], [8.0, 2.0, 1.0]], [[8.0, 8.0, 6.0], [1.0, 7.0, 1.0], [1.0, 1.0, 4.0]]], [[[8.0, 2.0, 1.0], 
[2.0, 1.0, 1.0], [1.0, 1.0, 4.0]], [[8.0, 8.0, 7.0], [6.0, 1.0, 4.0], [7.0, 5.0, 8.0]], [[1.0, 8.0, 7.0], [4.0, 1.0, 1.0], 
[8.0, 1.0, 6.0]]], [[[5.0, 4.0, 7.0], [4.0, 3.0, 6.0], [2.0, 6.0, 8.0]], [[3.0, 3.0, 3.0], [2.0, 1.0, 8.0], [3.0, 6.0, 5.0]], 
[[8.0, 2.0, 1.0], [5.0, 5.0, 3.0], [1.0, 4.0, 3.0]]], [[[8.0, 4.0, 2.0], [8.0, 8.0, 8.0], [6.0, 5.0, 5.0]], [[6.0, 6.0, 1.0], 
[3.0, 1.0, 1.0], [3.0, 7.0, 8.0]], [[7.0, 7.0, 3.0], [5.0, 2.0, 8.0], [8.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LRCN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 6.0], [8.0, 1.0, 1.0], [4.0, 8.0, 5.0]], [[4.0, 8.0, 8.0], [1.0, 3.0, 3.0], [7.0, 7.0, 
8.0]], [[8.0, 1.0, 6.0], [5.0, 8.0, 3.0], [5.0, 1.0, 6.0]]], [[[2.0, 4.0, 5.0], [4.0, 6.0, 1.0], [5.0, 1.0, 5.0]], [[4.0, 1.0, 
5.0], [5.0, 2.0, 4.0], [3.0, 4.0, 2.0]], [[8.0, 2.0, 2.0], [7.0, 1.0, 7.0], [7.0, 2.0, 3.0]]], [[[1.0, 6.0, 4.0], [1.0, 2.0, 
3.0], [8.0, 6.0, 1.0]], [[3.0, 6.0, 1.0], [1.0, 3.0, 2.0], [1.0, 5.0, 2.0]], [[7.0, 1.0, 7.0], [3.0, 5.0, 8.0], [3.0, 3.0, 
1.0]]], [[[1.0, 4.0, 8.0], [3.0, 1.0, 2.0], [8.0, 6.0, 7.0]], [[6.0, 1.0, 5.0], [7.0, 1.0, 5.0], [1.0, 6.0, 7.0]], [[2.0, 8.0, 
4.0], [8.0, 4.0, 8.0], [3.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 4.0], [5.0, 4.0, 8.0], [3.0, 3.0, 6.0]], [[4.0, 6.0, 3.0], [3.0, 8.0, 
6.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 4.0], [5.0, 7.0, 5.0], [5.0, 2.0, 5.0]]], [[[3.0, 6.0, 3.0], [2.0, 5.0, 3.0], [2.0, 5.0, 
7.0]], [[7.0, 8.0, 7.0], [8.0, 2.0, 8.0], [4.0, 2.0, 1.0]], [[1.0, 1.0, 5.0], [3.0, 7.0, 4.0], [3.0, 3.0, 6.0]]], [[[6.0, 8.0, 
6.0], [4.0, 4.0, 5.0], [8.0, 6.0, 8.0]], [[3.0, 4.0, 8.0], [4.0, 8.0, 1.0], [6.0, 2.0, 8.0]], [[8.0, 1.0, 2.0], [2.0, 6.0, 
7.0], [6.0, 3.0, 8.0]]], [[[8.0, 7.0, 7.0], [6.0, 2.0, 5.0], [6.0, 8.0, 7.0]], [[7.0, 4.0, 8.0], [3.0, 1.0, 1.0], [8.0, 7.0, 
3.0]], [[5.0, 8.0, 6.0], [3.0, 8.0, 4.0], [8.0, 7.0, 7.0]]]]))
      val=Data(numpy.array([[[[5.0, 4.0, 2.0], [1.0, 1.0, 2.0], [6.0, 3.0, 7.0]]], [[[7.0, 4.0, 5.0], [8.0, 6.0, 6.0], [2.0, 
3.0, 6.0]]]]),self.functionspace)
      arg[2:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 6.0], [8.0, 1.0, 1.0], [4.0, 8.0, 5.0]], [[4.0, 8.0, 8.0], [1.0, 3.0, 3.0], [7.0, 
7.0, 8.0]], [[8.0, 1.0, 6.0], [5.0, 8.0, 3.0], [5.0, 1.0, 6.0]]], [[[2.0, 4.0, 5.0], [4.0, 6.0, 1.0], [5.0, 1.0, 5.0]], [[4.0, 
1.0, 5.0], [5.0, 2.0, 4.0], [3.0, 4.0, 2.0]], [[8.0, 2.0, 2.0], [7.0, 1.0, 7.0], [7.0, 2.0, 3.0]]], [[[1.0, 6.0, 4.0], [1.0, 
2.0, 3.0], [8.0, 6.0, 1.0]], [[5.0, 4.0, 2.0], [1.0, 1.0, 2.0], [6.0, 3.0, 7.0]], [[7.0, 1.0, 7.0], [3.0, 5.0, 8.0], [3.0, 3.0, 
1.0]]], [[[1.0, 4.0, 8.0], [3.0, 1.0, 2.0], [8.0, 6.0, 7.0]], [[7.0, 4.0, 5.0], [8.0, 6.0, 6.0], [2.0, 3.0, 6.0]], [[2.0, 8.0, 
4.0], [8.0, 4.0, 8.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 4.0], [5.0, 4.0, 8.0], [3.0, 3.0, 6.0]], 
[[4.0, 6.0, 3.0], [3.0, 8.0, 6.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 4.0], [5.0, 7.0, 5.0], [5.0, 2.0, 5.0]]], [[[3.0, 6.0, 3.0], 
[2.0, 5.0, 3.0], [2.0, 5.0, 7.0]], [[7.0, 8.0, 7.0], [8.0, 2.0, 8.0], [4.0, 2.0, 1.0]], [[1.0, 1.0, 5.0], [3.0, 7.0, 4.0], 
[3.0, 3.0, 6.0]]], [[[6.0, 8.0, 6.0], [4.0, 4.0, 5.0], [8.0, 6.0, 8.0]], [[5.0, 4.0, 2.0], [1.0, 1.0, 2.0], [6.0, 3.0, 7.0]], 
[[8.0, 1.0, 2.0], [2.0, 6.0, 7.0], [6.0, 3.0, 8.0]]], [[[8.0, 7.0, 7.0], [6.0, 2.0, 5.0], [6.0, 8.0, 7.0]], [[7.0, 4.0, 5.0], 
[8.0, 6.0, 6.0], [2.0, 3.0, 6.0]], [[5.0, 8.0, 6.0], [3.0, 8.0, 4.0], [8.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LRCN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 6.0], [2.0, 6.0, 8.0], [8.0, 4.0, 8.0]], [[2.0, 3.0, 6.0], [7.0, 1.0, 8.0], [6.0, 6.0, 
6.0]], [[5.0, 4.0, 2.0], [7.0, 6.0, 4.0], [5.0, 6.0, 1.0]]], [[[5.0, 7.0, 2.0], [1.0, 5.0, 2.0], [8.0, 2.0, 6.0]], [[7.0, 8.0, 
2.0], [5.0, 5.0, 1.0], [5.0, 2.0, 3.0]], [[2.0, 3.0, 5.0], [3.0, 2.0, 4.0], [1.0, 2.0, 1.0]]], [[[1.0, 5.0, 2.0], [6.0, 6.0, 
8.0], [5.0, 1.0, 5.0]], [[7.0, 5.0, 8.0], [6.0, 4.0, 8.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 3.0], [2.0, 4.0, 7.0], [5.0, 6.0, 
1.0]]], [[[1.0, 3.0, 7.0], [6.0, 6.0, 3.0], [2.0, 1.0, 6.0]], [[6.0, 8.0, 2.0], [1.0, 4.0, 7.0], [2.0, 7.0, 7.0]], [[6.0, 5.0, 
6.0], [6.0, 3.0, 2.0], [6.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 3.0, 6.0], [3.0, 1.0, 7.0], [1.0, 3.0, 8.0]], [[5.0, 8.0, 2.0], [6.0, 8.0, 
6.0], [4.0, 4.0, 4.0]], [[1.0, 3.0, 7.0], [4.0, 5.0, 3.0], [1.0, 5.0, 1.0]]], [[[6.0, 4.0, 3.0], [6.0, 3.0, 4.0], [5.0, 5.0, 
5.0]], [[5.0, 4.0, 7.0], [7.0, 4.0, 5.0], [4.0, 7.0, 2.0]], [[6.0, 1.0, 6.0], [3.0, 3.0, 5.0], [5.0, 6.0, 1.0]]], [[[7.0, 7.0, 
1.0], [1.0, 6.0, 7.0], [3.0, 7.0, 2.0]], [[3.0, 6.0, 1.0], [5.0, 5.0, 1.0], [3.0, 3.0, 6.0]], [[5.0, 6.0, 6.0], [3.0, 4.0, 
3.0], [4.0, 1.0, 7.0]]], [[[2.0, 1.0, 4.0], [1.0, 6.0, 3.0], [8.0, 6.0, 4.0]], [[4.0, 1.0, 8.0], [8.0, 3.0, 3.0], [4.0, 5.0, 
5.0]], [[1.0, 7.0, 5.0], [7.0, 4.0, 8.0], [8.0, 5.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0, 6.0, 6.0], [6.0, 2.0, 2.0], [2.0, 5.0, 2.0]]], [[[1.0, 7.0, 8.0], [6.0, 4.0, 7.0], [1.0, 
1.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 3.0], [4.0, 5.0, 6.0], [8.0, 3.0, 6.0]]], [[[6.0, 7.0, 4.0], [8.0, 3.0, 
4.0], [3.0, 4.0, 7.0]]]]))
      arg[2:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 6.0], [2.0, 6.0, 8.0], [8.0, 4.0, 8.0]], [[2.0, 3.0, 6.0], [7.0, 1.0, 8.0], [6.0, 
6.0, 6.0]], [[5.0, 4.0, 2.0], [7.0, 6.0, 4.0], [5.0, 6.0, 1.0]]], [[[5.0, 7.0, 2.0], [1.0, 5.0, 2.0], [8.0, 2.0, 6.0]], [[7.0, 
8.0, 2.0], [5.0, 5.0, 1.0], [5.0, 2.0, 3.0]], [[2.0, 3.0, 5.0], [3.0, 2.0, 4.0], [1.0, 2.0, 1.0]]], [[[1.0, 5.0, 2.0], [6.0, 
6.0, 8.0], [5.0, 1.0, 5.0]], [[5.0, 6.0, 6.0], [6.0, 2.0, 2.0], [2.0, 5.0, 2.0]], [[8.0, 6.0, 3.0], [2.0, 4.0, 7.0], [5.0, 6.0, 
1.0]]], [[[1.0, 3.0, 7.0], [6.0, 6.0, 3.0], [2.0, 1.0, 6.0]], [[1.0, 7.0, 8.0], [6.0, 4.0, 7.0], [1.0, 1.0, 2.0]], [[6.0, 5.0, 
6.0], [6.0, 3.0, 2.0], [6.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 6.0], [3.0, 1.0, 7.0], [1.0, 3.0, 8.0]], 
[[5.0, 8.0, 2.0], [6.0, 8.0, 6.0], [4.0, 4.0, 4.0]], [[1.0, 3.0, 7.0], [4.0, 5.0, 3.0], [1.0, 5.0, 1.0]]], [[[6.0, 4.0, 3.0], 
[6.0, 3.0, 4.0], [5.0, 5.0, 5.0]], [[5.0, 4.0, 7.0], [7.0, 4.0, 5.0], [4.0, 7.0, 2.0]], [[6.0, 1.0, 6.0], [3.0, 3.0, 5.0], 
[5.0, 6.0, 1.0]]], [[[7.0, 7.0, 1.0], [1.0, 6.0, 7.0], [3.0, 7.0, 2.0]], [[8.0, 7.0, 3.0], [4.0, 5.0, 6.0], [8.0, 3.0, 6.0]], 
[[5.0, 6.0, 6.0], [3.0, 4.0, 3.0], [4.0, 1.0, 7.0]]], [[[2.0, 1.0, 4.0], [1.0, 6.0, 3.0], [8.0, 6.0, 4.0]], [[6.0, 7.0, 4.0], 
[8.0, 3.0, 4.0], [3.0, 4.0, 7.0]], [[1.0, 7.0, 5.0], [7.0, 4.0, 8.0], [8.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LRCN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 4.0], [3.0, 3.0, 8.0], [1.0, 2.0, 4.0]], [[3.0, 6.0, 8.0], [7.0, 5.0, 7.0], [5.0, 8.0, 
6.0]], [[1.0, 6.0, 4.0], [2.0, 8.0, 4.0], [3.0, 1.0, 6.0]]], [[[3.0, 3.0, 4.0], [1.0, 2.0, 3.0], [5.0, 2.0, 1.0]], [[2.0, 2.0, 
3.0], [2.0, 5.0, 2.0], [7.0, 6.0, 3.0]], [[5.0, 5.0, 8.0], [8.0, 7.0, 3.0], [5.0, 4.0, 1.0]]], [[[1.0, 5.0, 6.0], [3.0, 1.0, 
7.0], [2.0, 7.0, 1.0]], [[8.0, 5.0, 1.0], [1.0, 3.0, 7.0], [4.0, 2.0, 8.0]], [[2.0, 5.0, 5.0], [8.0, 4.0, 7.0], [5.0, 4.0, 
4.0]]], [[[8.0, 5.0, 7.0], [8.0, 6.0, 6.0], [7.0, 4.0, 8.0]], [[6.0, 1.0, 2.0], [7.0, 3.0, 7.0], [6.0, 3.0, 4.0]], [[1.0, 4.0, 
6.0], [6.0, 7.0, 6.0], [5.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 2.0], [7.0, 7.0, 1.0], [6.0, 2.0, 1.0]], [[7.0, 2.0, 7.0], [1.0, 4.0, 
2.0], [4.0, 7.0, 8.0]], [[8.0, 4.0, 7.0], [6.0, 3.0, 6.0], [8.0, 7.0, 3.0]]], [[[3.0, 7.0, 5.0], [8.0, 8.0, 7.0], [8.0, 7.0, 
3.0]], [[2.0, 4.0, 6.0], [1.0, 3.0, 1.0], [2.0, 2.0, 1.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 5.0], [7.0, 8.0, 6.0]]], [[[5.0, 1.0, 
7.0], [2.0, 1.0, 7.0], [7.0, 4.0, 5.0]], [[4.0, 4.0, 3.0], [1.0, 5.0, 3.0], [3.0, 6.0, 7.0]], [[2.0, 2.0, 6.0], [2.0, 3.0, 
7.0], [4.0, 8.0, 5.0]]], [[[7.0, 1.0, 3.0], [2.0, 3.0, 5.0], [3.0, 5.0, 4.0]], [[4.0, 6.0, 3.0], [4.0, 4.0, 2.0], [2.0, 2.0, 
7.0]], [[2.0, 5.0, 5.0], [1.0, 6.0, 2.0], [6.0, 6.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[6.0, 7.0, 8.0], [3.0, 7.0, 2.0], [5.0, 4.0, 2.0]]], [[[7.0, 8.0, 3.0], [8.0, 3.0, 3.0], [2.0, 
4.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 8.0, 6.0], [5.0, 5.0, 2.0], [7.0, 7.0, 8.0]]], [[[1.0, 6.0, 1.0], [7.0, 3.0, 
5.0], [4.0, 4.0, 8.0]]]])
      arg[2:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 4.0], [3.0, 3.0, 8.0], [1.0, 2.0, 4.0]], [[3.0, 6.0, 8.0], [7.0, 5.0, 7.0], [5.0, 
8.0, 6.0]], [[1.0, 6.0, 4.0], [2.0, 8.0, 4.0], [3.0, 1.0, 6.0]]], [[[3.0, 3.0, 4.0], [1.0, 2.0, 3.0], [5.0, 2.0, 1.0]], [[2.0, 
2.0, 3.0], [2.0, 5.0, 2.0], [7.0, 6.0, 3.0]], [[5.0, 5.0, 8.0], [8.0, 7.0, 3.0], [5.0, 4.0, 1.0]]], [[[1.0, 5.0, 6.0], [3.0, 
1.0, 7.0], [2.0, 7.0, 1.0]], [[6.0, 7.0, 8.0], [3.0, 7.0, 2.0], [5.0, 4.0, 2.0]], [[2.0, 5.0, 5.0], [8.0, 4.0, 7.0], [5.0, 4.0, 
4.0]]], [[[8.0, 5.0, 7.0], [8.0, 6.0, 6.0], [7.0, 4.0, 8.0]], [[7.0, 8.0, 3.0], [8.0, 3.0, 3.0], [2.0, 4.0, 7.0]], [[1.0, 4.0, 
6.0], [6.0, 7.0, 6.0], [5.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 2.0], [7.0, 7.0, 1.0], [6.0, 2.0, 1.0]], 
[[7.0, 2.0, 7.0], [1.0, 4.0, 2.0], [4.0, 7.0, 8.0]], [[8.0, 4.0, 7.0], [6.0, 3.0, 6.0], [8.0, 7.0, 3.0]]], [[[3.0, 7.0, 5.0], 
[8.0, 8.0, 7.0], [8.0, 7.0, 3.0]], [[2.0, 4.0, 6.0], [1.0, 3.0, 1.0], [2.0, 2.0, 1.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 5.0], 
[7.0, 8.0, 6.0]]], [[[5.0, 1.0, 7.0], [2.0, 1.0, 7.0], [7.0, 4.0, 5.0]], [[5.0, 8.0, 6.0], [5.0, 5.0, 2.0], [7.0, 7.0, 8.0]], 
[[2.0, 2.0, 6.0], [2.0, 3.0, 7.0], [4.0, 8.0, 5.0]]], [[[7.0, 1.0, 3.0], [2.0, 3.0, 5.0], [3.0, 5.0, 4.0]], [[1.0, 6.0, 1.0], 
[7.0, 3.0, 5.0], [4.0, 4.0, 8.0]], [[2.0, 5.0, 5.0], [1.0, 6.0, 2.0], [6.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UULN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 5.0], [6.0, 5.0, 5.0], [8.0, 3.0, 1.0]], [[7.0, 3.0, 2.0], [8.0, 6.0, 3.0], [1.0, 3.0, 
7.0]], [[8.0, 8.0, 6.0], [4.0, 6.0, 5.0], [2.0, 5.0, 1.0]]], [[[4.0, 1.0, 2.0], [2.0, 7.0, 8.0], [5.0, 3.0, 2.0]], [[8.0, 6.0, 
7.0], [8.0, 7.0, 6.0], [8.0, 1.0, 8.0]], [[7.0, 7.0, 8.0], [8.0, 4.0, 3.0], [3.0, 5.0, 4.0]]], [[[3.0, 5.0, 8.0], [6.0, 7.0, 
5.0], [4.0, 3.0, 1.0]], [[3.0, 4.0, 2.0], [1.0, 5.0, 6.0], [2.0, 4.0, 5.0]], [[3.0, 6.0, 8.0], [5.0, 4.0, 2.0], [3.0, 6.0, 
7.0]]], [[[2.0, 4.0, 1.0], [1.0, 1.0, 7.0], [3.0, 8.0, 4.0]], [[4.0, 8.0, 4.0], [7.0, 3.0, 8.0], [2.0, 7.0, 1.0]], [[2.0, 1.0, 
5.0], [1.0, 8.0, 4.0], [2.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 8.0], [2.0, 2.0, 3.0], [5.0, 4.0, 6.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 
4.0], [2.0, 8.0, 7.0]], [[8.0, 8.0, 4.0], [2.0, 5.0, 3.0], [3.0, 5.0, 4.0]]], [[[7.0, 6.0, 8.0], [1.0, 2.0, 7.0], [6.0, 5.0, 
5.0]], [[6.0, 2.0, 7.0], [2.0, 7.0, 1.0], [8.0, 2.0, 8.0]], [[8.0, 4.0, 8.0], [5.0, 4.0, 6.0], [8.0, 1.0, 6.0]]], [[[1.0, 8.0, 
2.0], [1.0, 3.0, 8.0], [4.0, 8.0, 1.0]], [[8.0, 7.0, 4.0], [4.0, 8.0, 3.0], [2.0, 6.0, 6.0]], [[8.0, 6.0, 4.0], [4.0, 7.0, 
4.0], [7.0, 6.0, 3.0]]], [[[8.0, 7.0, 1.0], [6.0, 8.0, 1.0], [8.0, 6.0, 4.0]], [[7.0, 2.0, 3.0], [7.0, 8.0, 7.0], [4.0, 5.0, 
8.0]], [[1.0, 7.0, 6.0], [6.0, 2.0, 6.0], [5.0, 5.0, 1.0]]]]))
      val=numpy.array([[[[3.0, 5.0, 3.0]], [[5.0, 6.0, 6.0]]], [[[5.0, 7.0, 1.0]], [[8.0, 3.0, 6.0]]], [[[5.0, 7.0, 2.0]], 
[[1.0, 8.0, 6.0]]]])
      arg[:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 5.0], [6.0, 5.0, 5.0], [3.0, 5.0, 3.0]], [[7.0, 3.0, 2.0], [8.0, 6.0, 3.0], [5.0, 
6.0, 6.0]], [[8.0, 8.0, 6.0], [4.0, 6.0, 5.0], [2.0, 5.0, 1.0]]], [[[4.0, 1.0, 2.0], [2.0, 7.0, 8.0], [5.0, 7.0, 1.0]], [[8.0, 
6.0, 7.0], [8.0, 7.0, 6.0], [8.0, 3.0, 6.0]], [[7.0, 7.0, 8.0], [8.0, 4.0, 3.0], [3.0, 5.0, 4.0]]], [[[3.0, 5.0, 8.0], [6.0, 
7.0, 5.0], [5.0, 7.0, 2.0]], [[3.0, 4.0, 2.0], [1.0, 5.0, 6.0], [1.0, 8.0, 6.0]], [[3.0, 6.0, 8.0], [5.0, 4.0, 2.0], [3.0, 6.0, 
7.0]]], [[[2.0, 4.0, 1.0], [1.0, 1.0, 7.0], [3.0, 8.0, 4.0]], [[4.0, 8.0, 4.0], [7.0, 3.0, 8.0], [2.0, 7.0, 1.0]], [[2.0, 1.0, 
5.0], [1.0, 8.0, 4.0], [2.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 8.0], [2.0, 2.0, 3.0], [3.0, 5.0, 3.0]], 
[[6.0, 6.0, 6.0], [2.0, 2.0, 4.0], [5.0, 6.0, 6.0]], [[8.0, 8.0, 4.0], [2.0, 5.0, 3.0], [3.0, 5.0, 4.0]]], [[[7.0, 6.0, 8.0], 
[1.0, 2.0, 7.0], [5.0, 7.0, 1.0]], [[6.0, 2.0, 7.0], [2.0, 7.0, 1.0], [8.0, 3.0, 6.0]], [[8.0, 4.0, 8.0], [5.0, 4.0, 6.0], 
[8.0, 1.0, 6.0]]], [[[1.0, 8.0, 2.0], [1.0, 3.0, 8.0], [5.0, 7.0, 2.0]], [[8.0, 7.0, 4.0], [4.0, 8.0, 3.0], [1.0, 8.0, 6.0]], 
[[8.0, 6.0, 4.0], [4.0, 7.0, 4.0], [7.0, 6.0, 3.0]]], [[[8.0, 7.0, 1.0], [6.0, 8.0, 1.0], [8.0, 6.0, 4.0]], [[7.0, 2.0, 3.0], 
[7.0, 8.0, 7.0], [4.0, 5.0, 8.0]], [[1.0, 7.0, 6.0], [6.0, 2.0, 6.0], [5.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UULN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 6.0], [6.0, 7.0, 8.0], [1.0, 7.0, 7.0]], [[8.0, 8.0, 5.0], [5.0, 2.0, 5.0], [2.0, 5.0, 
5.0]], [[2.0, 4.0, 5.0], [7.0, 5.0, 3.0], [3.0, 1.0, 7.0]]], [[[6.0, 3.0, 6.0], [5.0, 4.0, 5.0], [2.0, 8.0, 3.0]], [[7.0, 2.0, 
1.0], [4.0, 3.0, 1.0], [3.0, 5.0, 2.0]], [[2.0, 7.0, 7.0], [4.0, 6.0, 1.0], [7.0, 8.0, 2.0]]], [[[4.0, 6.0, 5.0], [6.0, 4.0, 
5.0], [6.0, 7.0, 1.0]], [[6.0, 2.0, 8.0], [3.0, 5.0, 8.0], [4.0, 4.0, 2.0]], [[5.0, 1.0, 2.0], [2.0, 8.0, 7.0], [6.0, 6.0, 
5.0]]], [[[1.0, 3.0, 2.0], [2.0, 3.0, 5.0], [1.0, 3.0, 4.0]], [[8.0, 4.0, 5.0], [2.0, 5.0, 6.0], [5.0, 6.0, 1.0]], [[8.0, 1.0, 
5.0], [4.0, 8.0, 8.0], [8.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 5.0], [8.0, 5.0, 2.0], [3.0, 8.0, 7.0]], [[2.0, 7.0, 6.0], [8.0, 1.0, 
4.0], [6.0, 5.0, 8.0]], [[7.0, 8.0, 7.0], [7.0, 1.0, 3.0], [4.0, 2.0, 1.0]]], [[[3.0, 4.0, 6.0], [2.0, 3.0, 4.0], [5.0, 6.0, 
3.0]], [[1.0, 7.0, 7.0], [2.0, 6.0, 3.0], [4.0, 8.0, 4.0]], [[1.0, 1.0, 4.0], [5.0, 6.0, 7.0], [3.0, 4.0, 5.0]]], [[[1.0, 4.0, 
5.0], [7.0, 3.0, 4.0], [3.0, 1.0, 8.0]], [[2.0, 6.0, 8.0], [8.0, 3.0, 6.0], [6.0, 2.0, 3.0]], [[3.0, 1.0, 4.0], [7.0, 1.0, 
6.0], [2.0, 1.0, 1.0]]], [[[3.0, 1.0, 3.0], [7.0, 4.0, 2.0], [7.0, 4.0, 4.0]], [[1.0, 6.0, 1.0], [1.0, 8.0, 1.0], [2.0, 6.0, 
7.0]], [[1.0, 4.0, 2.0], [5.0, 2.0, 4.0], [1.0, 4.0, 8.0]]]]))
      val=Data(numpy.array([[[[3.0, 4.0, 6.0]], [[1.0, 8.0, 6.0]]], [[[6.0, 3.0, 1.0]], [[8.0, 3.0, 6.0]]], [[[6.0, 3.0, 5.0]], 
[[6.0, 6.0, 6.0]]]]),self.functionspace)
      arg[:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 6.0], [6.0, 7.0, 8.0], [3.0, 4.0, 6.0]], [[8.0, 8.0, 5.0], [5.0, 2.0, 5.0], [1.0, 
8.0, 6.0]], [[2.0, 4.0, 5.0], [7.0, 5.0, 3.0], [3.0, 1.0, 7.0]]], [[[6.0, 3.0, 6.0], [5.0, 4.0, 5.0], [6.0, 3.0, 1.0]], [[7.0, 
2.0, 1.0], [4.0, 3.0, 1.0], [8.0, 3.0, 6.0]], [[2.0, 7.0, 7.0], [4.0, 6.0, 1.0], [7.0, 8.0, 2.0]]], [[[4.0, 6.0, 5.0], [6.0, 
4.0, 5.0], [6.0, 3.0, 5.0]], [[6.0, 2.0, 8.0], [3.0, 5.0, 8.0], [6.0, 6.0, 6.0]], [[5.0, 1.0, 2.0], [2.0, 8.0, 7.0], [6.0, 6.0, 
5.0]]], [[[1.0, 3.0, 2.0], [2.0, 3.0, 5.0], [1.0, 3.0, 4.0]], [[8.0, 4.0, 5.0], [2.0, 5.0, 6.0], [5.0, 6.0, 1.0]], [[8.0, 1.0, 
5.0], [4.0, 8.0, 8.0], [8.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 5.0], [8.0, 5.0, 2.0], [3.0, 4.0, 6.0]], 
[[2.0, 7.0, 6.0], [8.0, 1.0, 4.0], [1.0, 8.0, 6.0]], [[7.0, 8.0, 7.0], [7.0, 1.0, 3.0], [4.0, 2.0, 1.0]]], [[[3.0, 4.0, 6.0], 
[2.0, 3.0, 4.0], [6.0, 3.0, 1.0]], [[1.0, 7.0, 7.0], [2.0, 6.0, 3.0], [8.0, 3.0, 6.0]], [[1.0, 1.0, 4.0], [5.0, 6.0, 7.0], 
[3.0, 4.0, 5.0]]], [[[1.0, 4.0, 5.0], [7.0, 3.0, 4.0], [6.0, 3.0, 5.0]], [[2.0, 6.0, 8.0], [8.0, 3.0, 6.0], [6.0, 6.0, 6.0]], 
[[3.0, 1.0, 4.0], [7.0, 1.0, 6.0], [2.0, 1.0, 1.0]]], [[[3.0, 1.0, 3.0], [7.0, 4.0, 2.0], [7.0, 4.0, 4.0]], [[1.0, 6.0, 1.0], 
[1.0, 8.0, 1.0], [2.0, 6.0, 7.0]], [[1.0, 4.0, 2.0], [5.0, 2.0, 4.0], [1.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UULN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 4.0], [6.0, 5.0, 1.0], [2.0, 7.0, 2.0]], [[5.0, 3.0, 3.0], [3.0, 7.0, 8.0], [2.0, 5.0, 
8.0]], [[3.0, 2.0, 1.0], [7.0, 2.0, 7.0], [7.0, 8.0, 2.0]]], [[[4.0, 4.0, 6.0], [4.0, 3.0, 1.0], [3.0, 8.0, 3.0]], [[2.0, 1.0, 
3.0], [8.0, 8.0, 4.0], [2.0, 6.0, 1.0]], [[1.0, 8.0, 2.0], [3.0, 4.0, 2.0], [5.0, 8.0, 5.0]]], [[[2.0, 2.0, 8.0], [3.0, 7.0, 
8.0], [1.0, 3.0, 8.0]], [[2.0, 5.0, 4.0], [8.0, 8.0, 4.0], [7.0, 8.0, 5.0]], [[3.0, 3.0, 1.0], [6.0, 2.0, 1.0], [1.0, 8.0, 
7.0]]], [[[6.0, 1.0, 1.0], [4.0, 8.0, 2.0], [3.0, 7.0, 6.0]], [[3.0, 1.0, 8.0], [3.0, 3.0, 1.0], [8.0, 6.0, 6.0]], [[8.0, 6.0, 
8.0], [2.0, 1.0, 7.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 3.0], [8.0, 4.0, 5.0], [1.0, 1.0, 4.0]], [[1.0, 2.0, 3.0], [7.0, 3.0, 
8.0], [7.0, 8.0, 7.0]], [[6.0, 3.0, 1.0], [3.0, 8.0, 5.0], [8.0, 1.0, 2.0]]], [[[1.0, 1.0, 7.0], [5.0, 2.0, 2.0], [6.0, 4.0, 
7.0]], [[5.0, 1.0, 6.0], [7.0, 2.0, 6.0], [6.0, 7.0, 4.0]], [[4.0, 5.0, 4.0], [5.0, 1.0, 8.0], [5.0, 7.0, 5.0]]], [[[2.0, 2.0, 
6.0], [7.0, 7.0, 8.0], [8.0, 8.0, 2.0]], [[4.0, 1.0, 6.0], [2.0, 4.0, 3.0], [7.0, 4.0, 7.0]], [[7.0, 1.0, 5.0], [6.0, 3.0, 
6.0], [7.0, 7.0, 8.0]]], [[[1.0, 1.0, 8.0], [1.0, 2.0, 7.0], [5.0, 2.0, 3.0]], [[8.0, 2.0, 4.0], [7.0, 8.0, 3.0], [8.0, 5.0, 
2.0]], [[6.0, 3.0, 6.0], [2.0, 4.0, 7.0], [6.0, 7.0, 6.0]]]]))
      val=Data(numpy.array([[[[4.0, 2.0, 7.0]], [[5.0, 6.0, 3.0]]], [[[8.0, 7.0, 2.0]], [[7.0, 4.0, 5.0]]], [[[3.0, 3.0, 8.0]], 
[[8.0, 1.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 8.0]], [[2.0, 8.0, 5.0]]], [[[2.0, 6.0, 2.0]], [[6.0, 6.0, 2.0]]], [[[6.0, 
8.0, 1.0]], [[3.0, 6.0, 1.0]]]]))
      arg[:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 4.0], [6.0, 5.0, 1.0], [4.0, 2.0, 7.0]], [[5.0, 3.0, 3.0], [3.0, 7.0, 8.0], [5.0, 
6.0, 3.0]], [[3.0, 2.0, 1.0], [7.0, 2.0, 7.0], [7.0, 8.0, 2.0]]], [[[4.0, 4.0, 6.0], [4.0, 3.0, 1.0], [8.0, 7.0, 2.0]], [[2.0, 
1.0, 3.0], [8.0, 8.0, 4.0], [7.0, 4.0, 5.0]], [[1.0, 8.0, 2.0], [3.0, 4.0, 2.0], [5.0, 8.0, 5.0]]], [[[2.0, 2.0, 8.0], [3.0, 
7.0, 8.0], [3.0, 3.0, 8.0]], [[2.0, 5.0, 4.0], [8.0, 8.0, 4.0], [8.0, 1.0, 7.0]], [[3.0, 3.0, 1.0], [6.0, 2.0, 1.0], [1.0, 8.0, 
7.0]]], [[[6.0, 1.0, 1.0], [4.0, 8.0, 2.0], [3.0, 7.0, 6.0]], [[3.0, 1.0, 8.0], [3.0, 3.0, 1.0], [8.0, 6.0, 6.0]], [[8.0, 6.0, 
8.0], [2.0, 1.0, 7.0], [3.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 3.0], [8.0, 4.0, 5.0], [5.0, 5.0, 8.0]], 
[[1.0, 2.0, 3.0], [7.0, 3.0, 8.0], [2.0, 8.0, 5.0]], [[6.0, 3.0, 1.0], [3.0, 8.0, 5.0], [8.0, 1.0, 2.0]]], [[[1.0, 1.0, 7.0], 
[5.0, 2.0, 2.0], [2.0, 6.0, 2.0]], [[5.0, 1.0, 6.0], [7.0, 2.0, 6.0], [6.0, 6.0, 2.0]], [[4.0, 5.0, 4.0], [5.0, 1.0, 8.0], 
[5.0, 7.0, 5.0]]], [[[2.0, 2.0, 6.0], [7.0, 7.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 1.0, 6.0], [2.0, 4.0, 3.0], [3.0, 6.0, 1.0]], 
[[7.0, 1.0, 5.0], [6.0, 3.0, 6.0], [7.0, 7.0, 8.0]]], [[[1.0, 1.0, 8.0], [1.0, 2.0, 7.0], [5.0, 2.0, 3.0]], [[8.0, 2.0, 4.0], 
[7.0, 8.0, 3.0], [8.0, 5.0, 2.0]], [[6.0, 3.0, 6.0], [2.0, 4.0, 7.0], [6.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UULN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 7.0], [1.0, 3.0, 1.0], [2.0, 6.0, 8.0]], [[4.0, 2.0, 6.0], [2.0, 3.0, 4.0], [5.0, 6.0, 
6.0]], [[7.0, 3.0, 3.0], [4.0, 4.0, 8.0], [6.0, 1.0, 7.0]]], [[[4.0, 5.0, 8.0], [8.0, 2.0, 3.0], [8.0, 7.0, 8.0]], [[6.0, 4.0, 
5.0], [3.0, 8.0, 6.0], [2.0, 8.0, 7.0]], [[7.0, 7.0, 6.0], [7.0, 6.0, 7.0], [7.0, 6.0, 6.0]]], [[[1.0, 7.0, 2.0], [3.0, 4.0, 
4.0], [5.0, 5.0, 1.0]], [[3.0, 3.0, 5.0], [3.0, 4.0, 8.0], [7.0, 1.0, 2.0]], [[1.0, 1.0, 5.0], [6.0, 2.0, 1.0], [4.0, 8.0, 
4.0]]], [[[7.0, 1.0, 8.0], [6.0, 1.0, 6.0], [7.0, 4.0, 7.0]], [[1.0, 1.0, 8.0], [7.0, 2.0, 8.0], [7.0, 6.0, 8.0]], [[1.0, 2.0, 
4.0], [3.0, 4.0, 2.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 6.0], [5.0, 6.0, 4.0], [4.0, 6.0, 2.0]], [[5.0, 7.0, 1.0], [6.0, 5.0, 
2.0], [8.0, 5.0, 2.0]], [[7.0, 2.0, 8.0], [3.0, 2.0, 3.0], [8.0, 2.0, 4.0]]], [[[2.0, 2.0, 2.0], [3.0, 4.0, 7.0], [2.0, 4.0, 
7.0]], [[4.0, 8.0, 2.0], [1.0, 4.0, 8.0], [8.0, 5.0, 6.0]], [[5.0, 6.0, 6.0], [8.0, 6.0, 4.0], [5.0, 4.0, 7.0]]], [[[7.0, 8.0, 
6.0], [1.0, 7.0, 4.0], [5.0, 5.0, 7.0]], [[4.0, 3.0, 2.0], [7.0, 5.0, 1.0], [7.0, 4.0, 6.0]], [[6.0, 4.0, 4.0], [1.0, 6.0, 
8.0], [4.0, 2.0, 5.0]]], [[[3.0, 1.0, 8.0], [1.0, 3.0, 7.0], [4.0, 1.0, 4.0]], [[2.0, 5.0, 7.0], [4.0, 2.0, 6.0], [1.0, 4.0, 
1.0]], [[1.0, 2.0, 5.0], [7.0, 5.0, 1.0], [7.0, 1.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0, 4.0, 7.0]], [[7.0, 5.0, 6.0]]], [[[8.0, 4.0, 7.0]], [[6.0, 1.0, 1.0]]], [[[2.0, 7.0, 
1.0]], [[7.0, 8.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 8.0, 4.0]], [[5.0, 2.0, 3.0]]], [[[7.0, 7.0, 2.0]], [[5.0, 3.0, 
1.0]]], [[[4.0, 7.0, 8.0]], [[6.0, 2.0, 5.0]]]])
      arg[:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 7.0], [1.0, 3.0, 1.0], [4.0, 4.0, 7.0]], [[4.0, 2.0, 6.0], [2.0, 3.0, 4.0], [7.0, 
5.0, 6.0]], [[7.0, 3.0, 3.0], [4.0, 4.0, 8.0], [6.0, 1.0, 7.0]]], [[[4.0, 5.0, 8.0], [8.0, 2.0, 3.0], [8.0, 4.0, 7.0]], [[6.0, 
4.0, 5.0], [3.0, 8.0, 6.0], [6.0, 1.0, 1.0]], [[7.0, 7.0, 6.0], [7.0, 6.0, 7.0], [7.0, 6.0, 6.0]]], [[[1.0, 7.0, 2.0], [3.0, 
4.0, 4.0], [2.0, 7.0, 1.0]], [[3.0, 3.0, 5.0], [3.0, 4.0, 8.0], [7.0, 8.0, 6.0]], [[1.0, 1.0, 5.0], [6.0, 2.0, 1.0], [4.0, 8.0, 
4.0]]], [[[7.0, 1.0, 8.0], [6.0, 1.0, 6.0], [7.0, 4.0, 7.0]], [[1.0, 1.0, 8.0], [7.0, 2.0, 8.0], [7.0, 6.0, 8.0]], [[1.0, 2.0, 
4.0], [3.0, 4.0, 2.0], [7.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 6.0], [5.0, 6.0, 4.0], [6.0, 8.0, 4.0]], 
[[5.0, 7.0, 1.0], [6.0, 5.0, 2.0], [5.0, 2.0, 3.0]], [[7.0, 2.0, 8.0], [3.0, 2.0, 3.0], [8.0, 2.0, 4.0]]], [[[2.0, 2.0, 2.0], 
[3.0, 4.0, 7.0], [7.0, 7.0, 2.0]], [[4.0, 8.0, 2.0], [1.0, 4.0, 8.0], [5.0, 3.0, 1.0]], [[5.0, 6.0, 6.0], [8.0, 6.0, 4.0], 
[5.0, 4.0, 7.0]]], [[[7.0, 8.0, 6.0], [1.0, 7.0, 4.0], [4.0, 7.0, 8.0]], [[4.0, 3.0, 2.0], [7.0, 5.0, 1.0], [6.0, 2.0, 5.0]], 
[[6.0, 4.0, 4.0], [1.0, 6.0, 8.0], [4.0, 2.0, 5.0]]], [[[3.0, 1.0, 8.0], [1.0, 3.0, 7.0], [4.0, 1.0, 4.0]], [[2.0, 5.0, 7.0], 
[4.0, 2.0, 6.0], [1.0, 4.0, 1.0]], [[1.0, 2.0, 5.0], [7.0, 5.0, 1.0], [7.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PURN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 2.0], [8.0, 3.0, 7.0], [1.0, 7.0, 5.0]], [[5.0, 1.0, 6.0], [6.0, 2.0, 7.0], [1.0, 2.0, 
4.0]], [[7.0, 6.0, 8.0], [1.0, 8.0, 4.0], [3.0, 2.0, 5.0]]], [[[3.0, 2.0, 6.0], [6.0, 6.0, 3.0], [8.0, 1.0, 7.0]], [[6.0, 4.0, 
5.0], [3.0, 7.0, 5.0], [3.0, 5.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 8.0, 7.0], [8.0, 8.0, 2.0]]], [[[4.0, 7.0, 4.0], [5.0, 2.0, 
4.0], [2.0, 8.0, 2.0]], [[3.0, 1.0, 6.0], [2.0, 1.0, 1.0], [8.0, 6.0, 2.0]], [[8.0, 1.0, 5.0], [2.0, 1.0, 2.0], [2.0, 2.0, 
1.0]]], [[[4.0, 7.0, 2.0], [2.0, 5.0, 2.0], [4.0, 2.0, 1.0]], [[7.0, 2.0, 5.0], [4.0, 2.0, 5.0], [6.0, 4.0, 3.0]], [[5.0, 3.0, 
7.0], [4.0, 2.0, 6.0], [7.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 2.0], [2.0, 8.0, 2.0], [7.0, 2.0, 5.0]], [[7.0, 7.0, 4.0], [8.0, 6.0, 
4.0], [1.0, 6.0, 8.0]], [[7.0, 6.0, 5.0], [4.0, 3.0, 3.0], [7.0, 3.0, 1.0]]], [[[8.0, 3.0, 3.0], [7.0, 6.0, 8.0], [5.0, 1.0, 
3.0]], [[4.0, 7.0, 2.0], [7.0, 8.0, 2.0], [2.0, 3.0, 2.0]], [[4.0, 2.0, 5.0], [5.0, 7.0, 7.0], [8.0, 6.0, 2.0]]], [[[6.0, 4.0, 
5.0], [8.0, 2.0, 6.0], [4.0, 2.0, 3.0]], [[8.0, 2.0, 1.0], [8.0, 6.0, 3.0], [2.0, 6.0, 7.0]], [[2.0, 4.0, 6.0], [7.0, 6.0, 
8.0], [6.0, 7.0, 5.0]]], [[[5.0, 8.0, 6.0], [4.0, 8.0, 1.0], [3.0, 7.0, 4.0]], [[1.0, 5.0, 2.0], [7.0, 5.0, 8.0], [3.0, 2.0, 
6.0]], [[7.0, 4.0, 5.0], [6.0, 5.0, 6.0], [7.0, 1.0, 7.0]]]]))
      val=numpy.array([[[2.0, 7.0, 4.0]], [[8.0, 3.0, 2.0]]])
      arg[2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 2.0], [8.0, 3.0, 7.0], [1.0, 7.0, 5.0]], [[5.0, 1.0, 6.0], [6.0, 2.0, 7.0], [1.0, 
2.0, 4.0]], [[7.0, 6.0, 8.0], [1.0, 8.0, 4.0], [3.0, 2.0, 5.0]]], [[[3.0, 2.0, 6.0], [6.0, 6.0, 3.0], [8.0, 1.0, 7.0]], [[6.0, 
4.0, 5.0], [3.0, 7.0, 5.0], [3.0, 5.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 8.0, 7.0], [8.0, 8.0, 2.0]]], [[[4.0, 7.0, 4.0], [2.0, 
7.0, 4.0], [2.0, 8.0, 2.0]], [[3.0, 1.0, 6.0], [8.0, 3.0, 2.0], [8.0, 6.0, 2.0]], [[8.0, 1.0, 5.0], [2.0, 1.0, 2.0], [2.0, 2.0, 
1.0]]], [[[4.0, 7.0, 2.0], [2.0, 5.0, 2.0], [4.0, 2.0, 1.0]], [[7.0, 2.0, 5.0], [4.0, 2.0, 5.0], [6.0, 4.0, 3.0]], [[5.0, 3.0, 
7.0], [4.0, 2.0, 6.0], [7.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 2.0], [2.0, 8.0, 2.0], [7.0, 2.0, 5.0]], 
[[7.0, 7.0, 4.0], [8.0, 6.0, 4.0], [1.0, 6.0, 8.0]], [[7.0, 6.0, 5.0], [4.0, 3.0, 3.0], [7.0, 3.0, 1.0]]], [[[8.0, 3.0, 3.0], 
[7.0, 6.0, 8.0], [5.0, 1.0, 3.0]], [[4.0, 7.0, 2.0], [7.0, 8.0, 2.0], [2.0, 3.0, 2.0]], [[4.0, 2.0, 5.0], [5.0, 7.0, 7.0], 
[8.0, 6.0, 2.0]]], [[[6.0, 4.0, 5.0], [2.0, 7.0, 4.0], [4.0, 2.0, 3.0]], [[8.0, 2.0, 1.0], [8.0, 3.0, 2.0], [2.0, 6.0, 7.0]], 
[[2.0, 4.0, 6.0], [7.0, 6.0, 8.0], [6.0, 7.0, 5.0]]], [[[5.0, 8.0, 6.0], [4.0, 8.0, 1.0], [3.0, 7.0, 4.0]], [[1.0, 5.0, 2.0], 
[7.0, 5.0, 8.0], [3.0, 2.0, 6.0]], [[7.0, 4.0, 5.0], [6.0, 5.0, 6.0], [7.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PURN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 1.0], [8.0, 5.0, 3.0], [8.0, 6.0, 8.0]], [[4.0, 2.0, 3.0], [2.0, 3.0, 5.0], [5.0, 7.0, 
2.0]], [[3.0, 6.0, 7.0], [2.0, 4.0, 5.0], [8.0, 4.0, 6.0]]], [[[5.0, 7.0, 1.0], [8.0, 7.0, 4.0], [6.0, 8.0, 4.0]], [[2.0, 7.0, 
2.0], [2.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[5.0, 8.0, 2.0], [3.0, 2.0, 7.0], [4.0, 3.0, 1.0]]], [[[7.0, 4.0, 4.0], [1.0, 8.0, 
2.0], [1.0, 8.0, 2.0]], [[5.0, 8.0, 5.0], [8.0, 5.0, 4.0], [2.0, 8.0, 4.0]], [[4.0, 1.0, 1.0], [7.0, 8.0, 4.0], [7.0, 1.0, 
2.0]]], [[[6.0, 7.0, 6.0], [5.0, 5.0, 4.0], [3.0, 1.0, 1.0]], [[2.0, 5.0, 2.0], [7.0, 7.0, 5.0], [1.0, 8.0, 4.0]], [[6.0, 2.0, 
4.0], [6.0, 5.0, 1.0], [1.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 2.0], [6.0, 1.0, 7.0], [2.0, 6.0, 1.0]], [[2.0, 6.0, 6.0], [6.0, 3.0, 
8.0], [1.0, 4.0, 3.0]], [[7.0, 3.0, 5.0], [3.0, 4.0, 6.0], [5.0, 1.0, 5.0]]], [[[4.0, 7.0, 8.0], [3.0, 1.0, 6.0], [1.0, 5.0, 
8.0]], [[6.0, 2.0, 4.0], [2.0, 3.0, 6.0], [7.0, 7.0, 8.0]], [[8.0, 5.0, 5.0], [2.0, 8.0, 1.0], [2.0, 7.0, 6.0]]], [[[7.0, 1.0, 
4.0], [4.0, 7.0, 6.0], [6.0, 3.0, 6.0]], [[3.0, 3.0, 7.0], [7.0, 2.0, 5.0], [1.0, 7.0, 5.0]], [[8.0, 4.0, 1.0], [7.0, 5.0, 
2.0], [5.0, 2.0, 2.0]]], [[[2.0, 2.0, 6.0], [1.0, 1.0, 2.0], [6.0, 8.0, 2.0]], [[8.0, 7.0, 4.0], [4.0, 4.0, 5.0], [1.0, 1.0, 
8.0]], [[1.0, 3.0, 2.0], [2.0, 5.0, 5.0], [7.0, 4.0, 4.0]]]]))
      val=Data(numpy.array([[[1.0, 7.0, 3.0]], [[1.0, 3.0, 4.0]]]),self.functionspace)
      arg[2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 1.0], [8.0, 5.0, 3.0], [8.0, 6.0, 8.0]], [[4.0, 2.0, 3.0], [2.0, 3.0, 5.0], [5.0, 
7.0, 2.0]], [[3.0, 6.0, 7.0], [2.0, 4.0, 5.0], [8.0, 4.0, 6.0]]], [[[5.0, 7.0, 1.0], [8.0, 7.0, 4.0], [6.0, 8.0, 4.0]], [[2.0, 
7.0, 2.0], [2.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[5.0, 8.0, 2.0], [3.0, 2.0, 7.0], [4.0, 3.0, 1.0]]], [[[7.0, 4.0, 4.0], [1.0, 
7.0, 3.0], [1.0, 8.0, 2.0]], [[5.0, 8.0, 5.0], [1.0, 3.0, 4.0], [2.0, 8.0, 4.0]], [[4.0, 1.0, 1.0], [7.0, 8.0, 4.0], [7.0, 1.0, 
2.0]]], [[[6.0, 7.0, 6.0], [5.0, 5.0, 4.0], [3.0, 1.0, 1.0]], [[2.0, 5.0, 2.0], [7.0, 7.0, 5.0], [1.0, 8.0, 4.0]], [[6.0, 2.0, 
4.0], [6.0, 5.0, 1.0], [1.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 2.0], [6.0, 1.0, 7.0], [2.0, 6.0, 1.0]], 
[[2.0, 6.0, 6.0], [6.0, 3.0, 8.0], [1.0, 4.0, 3.0]], [[7.0, 3.0, 5.0], [3.0, 4.0, 6.0], [5.0, 1.0, 5.0]]], [[[4.0, 7.0, 8.0], 
[3.0, 1.0, 6.0], [1.0, 5.0, 8.0]], [[6.0, 2.0, 4.0], [2.0, 3.0, 6.0], [7.0, 7.0, 8.0]], [[8.0, 5.0, 5.0], [2.0, 8.0, 1.0], 
[2.0, 7.0, 6.0]]], [[[7.0, 1.0, 4.0], [1.0, 7.0, 3.0], [6.0, 3.0, 6.0]], [[3.0, 3.0, 7.0], [1.0, 3.0, 4.0], [1.0, 7.0, 5.0]], 
[[8.0, 4.0, 1.0], [7.0, 5.0, 2.0], [5.0, 2.0, 2.0]]], [[[2.0, 2.0, 6.0], [1.0, 1.0, 2.0], [6.0, 8.0, 2.0]], [[8.0, 7.0, 4.0], 
[4.0, 4.0, 5.0], [1.0, 1.0, 8.0]], [[1.0, 3.0, 2.0], [2.0, 5.0, 5.0], [7.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PURN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 8.0], [8.0, 8.0, 4.0], [7.0, 1.0, 7.0]], [[3.0, 3.0, 4.0], [8.0, 1.0, 5.0], [4.0, 7.0, 
6.0]], [[6.0, 4.0, 1.0], [7.0, 4.0, 7.0], [7.0, 6.0, 5.0]]], [[[2.0, 4.0, 1.0], [1.0, 1.0, 3.0], [7.0, 3.0, 1.0]], [[4.0, 3.0, 
7.0], [3.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[1.0, 8.0, 4.0], [3.0, 1.0, 8.0], [7.0, 2.0, 7.0]]], [[[1.0, 3.0, 5.0], [1.0, 4.0, 
4.0], [5.0, 4.0, 6.0]], [[4.0, 5.0, 4.0], [2.0, 8.0, 1.0], [3.0, 7.0, 5.0]], [[3.0, 4.0, 2.0], [3.0, 4.0, 1.0], [7.0, 3.0, 
2.0]]], [[[2.0, 2.0, 2.0], [2.0, 2.0, 2.0], [3.0, 6.0, 4.0]], [[8.0, 3.0, 8.0], [6.0, 5.0, 5.0], [3.0, 6.0, 1.0]], [[7.0, 8.0, 
4.0], [2.0, 2.0, 1.0], [5.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 6.0], [7.0, 4.0, 1.0], [7.0, 1.0, 4.0]], [[1.0, 2.0, 4.0], [1.0, 2.0, 
8.0], [7.0, 6.0, 5.0]], [[5.0, 7.0, 6.0], [7.0, 3.0, 2.0], [3.0, 3.0, 1.0]]], [[[3.0, 8.0, 1.0], [5.0, 2.0, 2.0], [3.0, 8.0, 
3.0]], [[6.0, 4.0, 7.0], [4.0, 3.0, 1.0], [5.0, 4.0, 8.0]], [[8.0, 6.0, 1.0], [7.0, 6.0, 2.0], [7.0, 2.0, 3.0]]], [[[7.0, 5.0, 
5.0], [7.0, 8.0, 8.0], [6.0, 4.0, 3.0]], [[8.0, 5.0, 5.0], [5.0, 4.0, 5.0], [4.0, 8.0, 1.0]], [[6.0, 3.0, 6.0], [1.0, 2.0, 
2.0], [8.0, 6.0, 6.0]]], [[[7.0, 7.0, 2.0], [4.0, 1.0, 4.0], [3.0, 1.0, 3.0]], [[1.0, 2.0, 7.0], [7.0, 7.0, 6.0], [4.0, 8.0, 
4.0]], [[5.0, 7.0, 6.0], [6.0, 6.0, 2.0], [7.0, 3.0, 7.0]]]]))
      val=Data(numpy.array([[[4.0, 4.0, 6.0]], [[2.0, 5.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 4.0, 7.0]], [[4.0, 4.0, 3.0]]]))
      arg[2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [8.0, 8.0, 4.0], [7.0, 1.0, 7.0]], [[3.0, 3.0, 4.0], [8.0, 1.0, 5.0], [4.0, 
7.0, 6.0]], [[6.0, 4.0, 1.0], [7.0, 4.0, 7.0], [7.0, 6.0, 5.0]]], [[[2.0, 4.0, 1.0], [1.0, 1.0, 3.0], [7.0, 3.0, 1.0]], [[4.0, 
3.0, 7.0], [3.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[1.0, 8.0, 4.0], [3.0, 1.0, 8.0], [7.0, 2.0, 7.0]]], [[[1.0, 3.0, 5.0], [4.0, 
4.0, 6.0], [5.0, 4.0, 6.0]], [[4.0, 5.0, 4.0], [2.0, 5.0, 5.0], [3.0, 7.0, 5.0]], [[3.0, 4.0, 2.0], [3.0, 4.0, 1.0], [7.0, 3.0, 
2.0]]], [[[2.0, 2.0, 2.0], [2.0, 2.0, 2.0], [3.0, 6.0, 4.0]], [[8.0, 3.0, 8.0], [6.0, 5.0, 5.0], [3.0, 6.0, 1.0]], [[7.0, 8.0, 
4.0], [2.0, 2.0, 1.0], [5.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 6.0], [7.0, 4.0, 1.0], [7.0, 1.0, 4.0]], 
[[1.0, 2.0, 4.0], [1.0, 2.0, 8.0], [7.0, 6.0, 5.0]], [[5.0, 7.0, 6.0], [7.0, 3.0, 2.0], [3.0, 3.0, 1.0]]], [[[3.0, 8.0, 1.0], 
[5.0, 2.0, 2.0], [3.0, 8.0, 3.0]], [[6.0, 4.0, 7.0], [4.0, 3.0, 1.0], [5.0, 4.0, 8.0]], [[8.0, 6.0, 1.0], [7.0, 6.0, 2.0], 
[7.0, 2.0, 3.0]]], [[[7.0, 5.0, 5.0], [1.0, 4.0, 7.0], [6.0, 4.0, 3.0]], [[8.0, 5.0, 5.0], [4.0, 4.0, 3.0], [4.0, 8.0, 1.0]], 
[[6.0, 3.0, 6.0], [1.0, 2.0, 2.0], [8.0, 6.0, 6.0]]], [[[7.0, 7.0, 2.0], [4.0, 1.0, 4.0], [3.0, 1.0, 3.0]], [[1.0, 2.0, 7.0], 
[7.0, 7.0, 6.0], [4.0, 8.0, 4.0]], [[5.0, 7.0, 6.0], [6.0, 6.0, 2.0], [7.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PURN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 6.0], [7.0, 4.0, 7.0], [4.0, 8.0, 8.0]], [[3.0, 8.0, 4.0], [6.0, 6.0, 3.0], [5.0, 8.0, 
8.0]], [[2.0, 4.0, 4.0], [5.0, 3.0, 5.0], [7.0, 4.0, 8.0]]], [[[7.0, 8.0, 5.0], [3.0, 8.0, 3.0], [8.0, 7.0, 5.0]], [[4.0, 3.0, 
4.0], [5.0, 7.0, 5.0], [3.0, 5.0, 4.0]], [[7.0, 8.0, 7.0], [4.0, 5.0, 2.0], [3.0, 8.0, 8.0]]], [[[4.0, 1.0, 7.0], [7.0, 7.0, 
5.0], [8.0, 3.0, 3.0]], [[3.0, 2.0, 7.0], [4.0, 5.0, 1.0], [3.0, 2.0, 4.0]], [[2.0, 4.0, 8.0], [6.0, 4.0, 3.0], [3.0, 1.0, 
2.0]]], [[[1.0, 5.0, 5.0], [8.0, 1.0, 2.0], [3.0, 6.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 5.0, 6.0], [1.0, 1.0, 3.0]], [[6.0, 8.0, 
3.0], [8.0, 2.0, 4.0], [6.0, 3.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 8.0], [5.0, 7.0, 5.0], [8.0, 8.0, 8.0]], [[2.0, 5.0, 1.0], [5.0, 3.0, 
3.0], [5.0, 4.0, 2.0]], [[6.0, 6.0, 2.0], [4.0, 4.0, 6.0], [6.0, 2.0, 5.0]]], [[[4.0, 1.0, 5.0], [1.0, 5.0, 4.0], [2.0, 2.0, 
7.0]], [[7.0, 6.0, 3.0], [8.0, 7.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 5.0, 1.0], [2.0, 8.0, 2.0], [8.0, 1.0, 1.0]]], [[[3.0, 6.0, 
2.0], [4.0, 4.0, 5.0], [8.0, 3.0, 1.0]], [[5.0, 3.0, 8.0], [1.0, 4.0, 1.0], [8.0, 5.0, 5.0]], [[5.0, 1.0, 2.0], [1.0, 3.0, 
2.0], [3.0, 5.0, 4.0]]], [[[1.0, 4.0, 2.0], [1.0, 1.0, 1.0], [1.0, 2.0, 8.0]], [[6.0, 2.0, 2.0], [5.0, 6.0, 1.0], [3.0, 7.0, 
2.0]], [[4.0, 5.0, 8.0], [1.0, 7.0, 2.0], [4.0, 3.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[6.0, 5.0, 5.0]], [[7.0, 6.0, 3.0]]])+(1.-msk_val)*numpy.array([[[1.0, 3.0, 8.0]], [[2.0, 6.0, 
3.0]]])
      arg[2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 6.0], [7.0, 4.0, 7.0], [4.0, 8.0, 8.0]], [[3.0, 8.0, 4.0], [6.0, 6.0, 3.0], [5.0, 
8.0, 8.0]], [[2.0, 4.0, 4.0], [5.0, 3.0, 5.0], [7.0, 4.0, 8.0]]], [[[7.0, 8.0, 5.0], [3.0, 8.0, 3.0], [8.0, 7.0, 5.0]], [[4.0, 
3.0, 4.0], [5.0, 7.0, 5.0], [3.0, 5.0, 4.0]], [[7.0, 8.0, 7.0], [4.0, 5.0, 2.0], [3.0, 8.0, 8.0]]], [[[4.0, 1.0, 7.0], [6.0, 
5.0, 5.0], [8.0, 3.0, 3.0]], [[3.0, 2.0, 7.0], [7.0, 6.0, 3.0], [3.0, 2.0, 4.0]], [[2.0, 4.0, 8.0], [6.0, 4.0, 3.0], [3.0, 1.0, 
2.0]]], [[[1.0, 5.0, 5.0], [8.0, 1.0, 2.0], [3.0, 6.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 5.0, 6.0], [1.0, 1.0, 3.0]], [[6.0, 8.0, 
3.0], [8.0, 2.0, 4.0], [6.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 8.0], [5.0, 7.0, 5.0], [8.0, 8.0, 8.0]], 
[[2.0, 5.0, 1.0], [5.0, 3.0, 3.0], [5.0, 4.0, 2.0]], [[6.0, 6.0, 2.0], [4.0, 4.0, 6.0], [6.0, 2.0, 5.0]]], [[[4.0, 1.0, 5.0], 
[1.0, 5.0, 4.0], [2.0, 2.0, 7.0]], [[7.0, 6.0, 3.0], [8.0, 7.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 5.0, 1.0], [2.0, 8.0, 2.0], 
[8.0, 1.0, 1.0]]], [[[3.0, 6.0, 2.0], [1.0, 3.0, 8.0], [8.0, 3.0, 1.0]], [[5.0, 3.0, 8.0], [2.0, 6.0, 3.0], [8.0, 5.0, 5.0]], 
[[5.0, 1.0, 2.0], [1.0, 3.0, 2.0], [3.0, 5.0, 4.0]]], [[[1.0, 4.0, 2.0], [1.0, 1.0, 1.0], [1.0, 2.0, 8.0]], [[6.0, 2.0, 2.0], 
[5.0, 6.0, 1.0], [3.0, 7.0, 2.0]], [[4.0, 5.0, 8.0], [1.0, 7.0, 2.0], [4.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLLN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 3.0], [2.0, 2.0, 4.0], [6.0, 5.0, 3.0]], [[3.0, 3.0, 5.0], [1.0, 5.0, 6.0], [3.0, 3.0, 
4.0]], [[4.0, 3.0, 3.0], [1.0, 5.0, 5.0], [6.0, 6.0, 5.0]]], [[[6.0, 2.0, 5.0], [2.0, 2.0, 5.0], [1.0, 4.0, 3.0]], [[1.0, 8.0, 
6.0], [5.0, 2.0, 7.0], [3.0, 5.0, 1.0]], [[3.0, 2.0, 3.0], [1.0, 1.0, 7.0], [5.0, 3.0, 8.0]]], [[[1.0, 7.0, 8.0], [5.0, 5.0, 
6.0], [6.0, 5.0, 7.0]], [[8.0, 2.0, 2.0], [4.0, 4.0, 1.0], [5.0, 2.0, 5.0]], [[1.0, 3.0, 6.0], [2.0, 5.0, 7.0], [3.0, 4.0, 
5.0]]], [[[4.0, 6.0, 4.0], [8.0, 3.0, 2.0], [5.0, 5.0, 2.0]], [[1.0, 4.0, 1.0], [4.0, 6.0, 4.0], [4.0, 7.0, 3.0]], [[5.0, 6.0, 
8.0], [5.0, 5.0, 1.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 8.0], [1.0, 6.0, 7.0], [1.0, 3.0, 5.0]], [[1.0, 8.0, 8.0], [4.0, 7.0, 
3.0], [1.0, 4.0, 5.0]], [[2.0, 2.0, 8.0], [5.0, 4.0, 6.0], [3.0, 3.0, 7.0]]], [[[6.0, 8.0, 3.0], [8.0, 2.0, 4.0], [6.0, 2.0, 
2.0]], [[8.0, 2.0, 5.0], [1.0, 5.0, 6.0], [4.0, 8.0, 6.0]], [[4.0, 3.0, 6.0], [2.0, 7.0, 7.0], [1.0, 5.0, 6.0]]], [[[7.0, 4.0, 
4.0], [8.0, 4.0, 1.0], [3.0, 7.0, 5.0]], [[2.0, 7.0, 8.0], [4.0, 6.0, 3.0], [2.0, 7.0, 5.0]], [[3.0, 8.0, 5.0], [4.0, 4.0, 
4.0], [6.0, 5.0, 6.0]]], [[[6.0, 5.0, 8.0], [3.0, 3.0, 1.0], [7.0, 2.0, 7.0]], [[6.0, 3.0, 8.0], [3.0, 5.0, 7.0], [7.0, 1.0, 
7.0]], [[2.0, 5.0, 4.0], [1.0, 6.0, 3.0], [5.0, 1.0, 5.0]]]]))
      val=numpy.array([[[[6.0, 8.0, 8.0]]], [[[5.0, 8.0, 5.0]]]])
      arg[2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 3.0], [2.0, 2.0, 4.0], [6.0, 5.0, 3.0]], [[3.0, 3.0, 5.0], [1.0, 5.0, 6.0], [3.0, 
3.0, 4.0]], [[4.0, 3.0, 3.0], [1.0, 5.0, 5.0], [6.0, 6.0, 5.0]]], [[[6.0, 2.0, 5.0], [2.0, 2.0, 5.0], [1.0, 4.0, 3.0]], [[1.0, 
8.0, 6.0], [5.0, 2.0, 7.0], [3.0, 5.0, 1.0]], [[3.0, 2.0, 3.0], [1.0, 1.0, 7.0], [5.0, 3.0, 8.0]]], [[[1.0, 7.0, 8.0], [5.0, 
5.0, 6.0], [6.0, 5.0, 7.0]], [[8.0, 2.0, 2.0], [4.0, 4.0, 1.0], [5.0, 2.0, 5.0]], [[1.0, 3.0, 6.0], [2.0, 5.0, 7.0], [6.0, 8.0, 
8.0]]], [[[4.0, 6.0, 4.0], [8.0, 3.0, 2.0], [5.0, 5.0, 2.0]], [[1.0, 4.0, 1.0], [4.0, 6.0, 4.0], [4.0, 7.0, 3.0]], [[5.0, 6.0, 
8.0], [5.0, 5.0, 1.0], [5.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 8.0], [1.0, 6.0, 7.0], [1.0, 3.0, 5.0]], 
[[1.0, 8.0, 8.0], [4.0, 7.0, 3.0], [1.0, 4.0, 5.0]], [[2.0, 2.0, 8.0], [5.0, 4.0, 6.0], [3.0, 3.0, 7.0]]], [[[6.0, 8.0, 3.0], 
[8.0, 2.0, 4.0], [6.0, 2.0, 2.0]], [[8.0, 2.0, 5.0], [1.0, 5.0, 6.0], [4.0, 8.0, 6.0]], [[4.0, 3.0, 6.0], [2.0, 7.0, 7.0], 
[1.0, 5.0, 6.0]]], [[[7.0, 4.0, 4.0], [8.0, 4.0, 1.0], [3.0, 7.0, 5.0]], [[2.0, 7.0, 8.0], [4.0, 6.0, 3.0], [2.0, 7.0, 5.0]], 
[[3.0, 8.0, 5.0], [4.0, 4.0, 4.0], [6.0, 8.0, 8.0]]], [[[6.0, 5.0, 8.0], [3.0, 3.0, 1.0], [7.0, 2.0, 7.0]], [[6.0, 3.0, 8.0], 
[3.0, 5.0, 7.0], [7.0, 1.0, 7.0]], [[2.0, 5.0, 4.0], [1.0, 6.0, 3.0], [5.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLLN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 1.0], [1.0, 8.0, 7.0], [8.0, 8.0, 2.0]], [[4.0, 8.0, 7.0], [5.0, 7.0, 5.0], [4.0, 2.0, 
7.0]], [[2.0, 4.0, 7.0], [8.0, 6.0, 6.0], [3.0, 5.0, 7.0]]], [[[5.0, 5.0, 6.0], [5.0, 6.0, 4.0], [6.0, 7.0, 7.0]], [[3.0, 2.0, 
1.0], [5.0, 3.0, 8.0], [3.0, 6.0, 4.0]], [[2.0, 1.0, 2.0], [7.0, 3.0, 1.0], [3.0, 3.0, 3.0]]], [[[5.0, 6.0, 3.0], [6.0, 6.0, 
2.0], [3.0, 7.0, 6.0]], [[2.0, 8.0, 5.0], [1.0, 1.0, 6.0], [1.0, 1.0, 2.0]], [[6.0, 6.0, 1.0], [1.0, 2.0, 2.0], [6.0, 2.0, 
7.0]]], [[[3.0, 6.0, 3.0], [3.0, 4.0, 1.0], [8.0, 4.0, 3.0]], [[6.0, 2.0, 7.0], [2.0, 6.0, 5.0], [5.0, 2.0, 1.0]], [[7.0, 5.0, 
7.0], [2.0, 1.0, 2.0], [4.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 8.0], [6.0, 2.0, 5.0], [4.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [3.0, 3.0, 
4.0], [6.0, 6.0, 6.0]], [[7.0, 1.0, 7.0], [3.0, 8.0, 5.0], [3.0, 6.0, 6.0]]], [[[5.0, 1.0, 4.0], [1.0, 1.0, 6.0], [8.0, 4.0, 
4.0]], [[2.0, 2.0, 3.0], [7.0, 2.0, 8.0], [8.0, 2.0, 1.0]], [[7.0, 5.0, 1.0], [8.0, 3.0, 3.0], [2.0, 4.0, 4.0]]], [[[1.0, 1.0, 
1.0], [7.0, 8.0, 4.0], [8.0, 3.0, 3.0]], [[2.0, 3.0, 3.0], [4.0, 6.0, 2.0], [6.0, 6.0, 7.0]], [[7.0, 7.0, 5.0], [7.0, 8.0, 
7.0], [2.0, 1.0, 7.0]]], [[[5.0, 8.0, 8.0], [2.0, 3.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 3.0, 8.0], [8.0, 3.0, 4.0], [7.0, 5.0, 
5.0]], [[4.0, 2.0, 7.0], [3.0, 2.0, 6.0], [2.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[[[7.0, 7.0, 3.0]]], [[[5.0, 2.0, 7.0]]]]),self.functionspace)
      arg[2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 1.0], [1.0, 8.0, 7.0], [8.0, 8.0, 2.0]], [[4.0, 8.0, 7.0], [5.0, 7.0, 5.0], [4.0, 
2.0, 7.0]], [[2.0, 4.0, 7.0], [8.0, 6.0, 6.0], [3.0, 5.0, 7.0]]], [[[5.0, 5.0, 6.0], [5.0, 6.0, 4.0], [6.0, 7.0, 7.0]], [[3.0, 
2.0, 1.0], [5.0, 3.0, 8.0], [3.0, 6.0, 4.0]], [[2.0, 1.0, 2.0], [7.0, 3.0, 1.0], [3.0, 3.0, 3.0]]], [[[5.0, 6.0, 3.0], [6.0, 
6.0, 2.0], [3.0, 7.0, 6.0]], [[2.0, 8.0, 5.0], [1.0, 1.0, 6.0], [1.0, 1.0, 2.0]], [[6.0, 6.0, 1.0], [1.0, 2.0, 2.0], [7.0, 7.0, 
3.0]]], [[[3.0, 6.0, 3.0], [3.0, 4.0, 1.0], [8.0, 4.0, 3.0]], [[6.0, 2.0, 7.0], [2.0, 6.0, 5.0], [5.0, 2.0, 1.0]], [[7.0, 5.0, 
7.0], [2.0, 1.0, 2.0], [5.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 8.0], [6.0, 2.0, 5.0], [4.0, 8.0, 7.0]], 
[[4.0, 8.0, 4.0], [3.0, 3.0, 4.0], [6.0, 6.0, 6.0]], [[7.0, 1.0, 7.0], [3.0, 8.0, 5.0], [3.0, 6.0, 6.0]]], [[[5.0, 1.0, 4.0], 
[1.0, 1.0, 6.0], [8.0, 4.0, 4.0]], [[2.0, 2.0, 3.0], [7.0, 2.0, 8.0], [8.0, 2.0, 1.0]], [[7.0, 5.0, 1.0], [8.0, 3.0, 3.0], 
[2.0, 4.0, 4.0]]], [[[1.0, 1.0, 1.0], [7.0, 8.0, 4.0], [8.0, 3.0, 3.0]], [[2.0, 3.0, 3.0], [4.0, 6.0, 2.0], [6.0, 6.0, 7.0]], 
[[7.0, 7.0, 5.0], [7.0, 8.0, 7.0], [7.0, 7.0, 3.0]]], [[[5.0, 8.0, 8.0], [2.0, 3.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 3.0, 8.0], 
[8.0, 3.0, 4.0], [7.0, 5.0, 5.0]], [[4.0, 2.0, 7.0], [3.0, 2.0, 6.0], [5.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLLN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 4.0], [6.0, 5.0, 5.0], [7.0, 1.0, 5.0]], [[4.0, 6.0, 4.0], [2.0, 3.0, 6.0], [1.0, 3.0, 
6.0]], [[1.0, 3.0, 6.0], [6.0, 7.0, 7.0], [8.0, 2.0, 1.0]]], [[[8.0, 7.0, 4.0], [8.0, 1.0, 2.0], [5.0, 4.0, 3.0]], [[4.0, 7.0, 
1.0], [2.0, 3.0, 3.0], [4.0, 7.0, 8.0]], [[1.0, 3.0, 8.0], [7.0, 7.0, 4.0], [6.0, 3.0, 5.0]]], [[[7.0, 3.0, 8.0], [4.0, 5.0, 
3.0], [6.0, 1.0, 7.0]], [[6.0, 5.0, 7.0], [7.0, 6.0, 2.0], [1.0, 4.0, 3.0]], [[8.0, 4.0, 3.0], [1.0, 6.0, 8.0], [3.0, 2.0, 
6.0]]], [[[3.0, 5.0, 4.0], [7.0, 3.0, 5.0], [6.0, 8.0, 8.0]], [[4.0, 2.0, 4.0], [8.0, 7.0, 8.0], [3.0, 7.0, 5.0]], [[2.0, 5.0, 
6.0], [6.0, 5.0, 8.0], [7.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 8.0], [5.0, 8.0, 2.0], [7.0, 2.0, 2.0]], [[8.0, 3.0, 1.0], [3.0, 8.0, 
1.0], [2.0, 8.0, 8.0]], [[5.0, 6.0, 5.0], [8.0, 8.0, 1.0], [1.0, 4.0, 7.0]]], [[[1.0, 7.0, 7.0], [1.0, 5.0, 3.0], [6.0, 7.0, 
3.0]], [[4.0, 2.0, 4.0], [4.0, 5.0, 4.0], [8.0, 7.0, 4.0]], [[4.0, 3.0, 6.0], [8.0, 7.0, 7.0], [1.0, 8.0, 4.0]]], [[[8.0, 7.0, 
4.0], [5.0, 3.0, 7.0], [4.0, 3.0, 8.0]], [[3.0, 1.0, 6.0], [8.0, 1.0, 7.0], [5.0, 8.0, 1.0]], [[5.0, 2.0, 6.0], [1.0, 8.0, 
6.0], [4.0, 7.0, 3.0]]], [[[2.0, 3.0, 7.0], [8.0, 8.0, 1.0], [1.0, 7.0, 4.0]], [[4.0, 6.0, 8.0], [5.0, 5.0, 1.0], [7.0, 6.0, 
7.0]], [[4.0, 8.0, 7.0], [4.0, 4.0, 6.0], [2.0, 2.0, 7.0]]]]))
      val=Data(numpy.array([[[[3.0, 6.0, 8.0]]], [[[1.0, 4.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 5.0]]], [[[6.0, 8.0, 4.0]]]]))
      arg[2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 4.0], [6.0, 5.0, 5.0], [7.0, 1.0, 5.0]], [[4.0, 6.0, 4.0], [2.0, 3.0, 6.0], [1.0, 
3.0, 6.0]], [[1.0, 3.0, 6.0], [6.0, 7.0, 7.0], [8.0, 2.0, 1.0]]], [[[8.0, 7.0, 4.0], [8.0, 1.0, 2.0], [5.0, 4.0, 3.0]], [[4.0, 
7.0, 1.0], [2.0, 3.0, 3.0], [4.0, 7.0, 8.0]], [[1.0, 3.0, 8.0], [7.0, 7.0, 4.0], [6.0, 3.0, 5.0]]], [[[7.0, 3.0, 8.0], [4.0, 
5.0, 3.0], [6.0, 1.0, 7.0]], [[6.0, 5.0, 7.0], [7.0, 6.0, 2.0], [1.0, 4.0, 3.0]], [[8.0, 4.0, 3.0], [1.0, 6.0, 8.0], [3.0, 6.0, 
8.0]]], [[[3.0, 5.0, 4.0], [7.0, 3.0, 5.0], [6.0, 8.0, 8.0]], [[4.0, 2.0, 4.0], [8.0, 7.0, 8.0], [3.0, 7.0, 5.0]], [[2.0, 5.0, 
6.0], [6.0, 5.0, 8.0], [1.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 8.0], [5.0, 8.0, 2.0], [7.0, 2.0, 2.0]], 
[[8.0, 3.0, 1.0], [3.0, 8.0, 1.0], [2.0, 8.0, 8.0]], [[5.0, 6.0, 5.0], [8.0, 8.0, 1.0], [1.0, 4.0, 7.0]]], [[[1.0, 7.0, 7.0], 
[1.0, 5.0, 3.0], [6.0, 7.0, 3.0]], [[4.0, 2.0, 4.0], [4.0, 5.0, 4.0], [8.0, 7.0, 4.0]], [[4.0, 3.0, 6.0], [8.0, 7.0, 7.0], 
[1.0, 8.0, 4.0]]], [[[8.0, 7.0, 4.0], [5.0, 3.0, 7.0], [4.0, 3.0, 8.0]], [[3.0, 1.0, 6.0], [8.0, 1.0, 7.0], [5.0, 8.0, 1.0]], 
[[5.0, 2.0, 6.0], [1.0, 8.0, 6.0], [4.0, 3.0, 5.0]]], [[[2.0, 3.0, 7.0], [8.0, 8.0, 1.0], [1.0, 7.0, 4.0]], [[4.0, 6.0, 8.0], 
[5.0, 5.0, 1.0], [7.0, 6.0, 7.0]], [[4.0, 8.0, 7.0], [4.0, 4.0, 6.0], [6.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLLN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 5.0], [2.0, 2.0, 2.0], [1.0, 5.0, 3.0]], [[5.0, 6.0, 2.0], [1.0, 1.0, 2.0], [2.0, 3.0, 
4.0]], [[8.0, 3.0, 6.0], [6.0, 8.0, 2.0], [1.0, 5.0, 7.0]]], [[[3.0, 3.0, 2.0], [7.0, 5.0, 4.0], [6.0, 3.0, 7.0]], [[2.0, 1.0, 
3.0], [2.0, 7.0, 3.0], [3.0, 5.0, 1.0]], [[6.0, 1.0, 3.0], [7.0, 5.0, 5.0], [6.0, 7.0, 5.0]]], [[[6.0, 7.0, 7.0], [8.0, 5.0, 
5.0], [6.0, 5.0, 8.0]], [[4.0, 7.0, 8.0], [4.0, 6.0, 4.0], [4.0, 3.0, 3.0]], [[8.0, 1.0, 2.0], [4.0, 3.0, 1.0], [7.0, 8.0, 
7.0]]], [[[3.0, 5.0, 6.0], [1.0, 4.0, 7.0], [3.0, 5.0, 3.0]], [[6.0, 4.0, 1.0], [7.0, 8.0, 5.0], [1.0, 1.0, 5.0]], [[4.0, 8.0, 
8.0], [8.0, 2.0, 2.0], [3.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 7.0], [2.0, 7.0, 1.0], [7.0, 3.0, 1.0]], [[1.0, 4.0, 8.0], [7.0, 3.0, 
5.0], [1.0, 4.0, 1.0]], [[5.0, 2.0, 1.0], [1.0, 1.0, 6.0], [7.0, 6.0, 8.0]]], [[[5.0, 7.0, 6.0], [4.0, 4.0, 1.0], [2.0, 1.0, 
4.0]], [[6.0, 7.0, 7.0], [1.0, 1.0, 3.0], [5.0, 5.0, 7.0]], [[2.0, 4.0, 3.0], [2.0, 6.0, 5.0], [5.0, 6.0, 3.0]]], [[[2.0, 2.0, 
7.0], [1.0, 5.0, 2.0], [8.0, 4.0, 5.0]], [[1.0, 7.0, 3.0], [2.0, 2.0, 1.0], [1.0, 2.0, 6.0]], [[2.0, 4.0, 2.0], [4.0, 3.0, 
3.0], [1.0, 3.0, 2.0]]], [[[1.0, 2.0, 1.0], [6.0, 8.0, 2.0], [8.0, 4.0, 8.0]], [[6.0, 3.0, 6.0], [4.0, 6.0, 6.0], [4.0, 6.0, 
3.0]], [[4.0, 2.0, 2.0], [6.0, 4.0, 5.0], [5.0, 1.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0, 2.0, 3.0]]], [[[5.0, 8.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 7.0, 8.0]]], 
[[[5.0, 6.0, 1.0]]]])
      arg[2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 5.0], [2.0, 2.0, 2.0], [1.0, 5.0, 3.0]], [[5.0, 6.0, 2.0], [1.0, 1.0, 2.0], [2.0, 
3.0, 4.0]], [[8.0, 3.0, 6.0], [6.0, 8.0, 2.0], [1.0, 5.0, 7.0]]], [[[3.0, 3.0, 2.0], [7.0, 5.0, 4.0], [6.0, 3.0, 7.0]], [[2.0, 
1.0, 3.0], [2.0, 7.0, 3.0], [3.0, 5.0, 1.0]], [[6.0, 1.0, 3.0], [7.0, 5.0, 5.0], [6.0, 7.0, 5.0]]], [[[6.0, 7.0, 7.0], [8.0, 
5.0, 5.0], [6.0, 5.0, 8.0]], [[4.0, 7.0, 8.0], [4.0, 6.0, 4.0], [4.0, 3.0, 3.0]], [[8.0, 1.0, 2.0], [4.0, 3.0, 1.0], [7.0, 2.0, 
3.0]]], [[[3.0, 5.0, 6.0], [1.0, 4.0, 7.0], [3.0, 5.0, 3.0]], [[6.0, 4.0, 1.0], [7.0, 8.0, 5.0], [1.0, 1.0, 5.0]], [[4.0, 8.0, 
8.0], [8.0, 2.0, 2.0], [5.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 7.0], [2.0, 7.0, 1.0], [7.0, 3.0, 1.0]], 
[[1.0, 4.0, 8.0], [7.0, 3.0, 5.0], [1.0, 4.0, 1.0]], [[5.0, 2.0, 1.0], [1.0, 1.0, 6.0], [7.0, 6.0, 8.0]]], [[[5.0, 7.0, 6.0], 
[4.0, 4.0, 1.0], [2.0, 1.0, 4.0]], [[6.0, 7.0, 7.0], [1.0, 1.0, 3.0], [5.0, 5.0, 7.0]], [[2.0, 4.0, 3.0], [2.0, 6.0, 5.0], 
[5.0, 6.0, 3.0]]], [[[2.0, 2.0, 7.0], [1.0, 5.0, 2.0], [8.0, 4.0, 5.0]], [[1.0, 7.0, 3.0], [2.0, 2.0, 1.0], [1.0, 2.0, 6.0]], 
[[2.0, 4.0, 2.0], [4.0, 3.0, 3.0], [2.0, 7.0, 8.0]]], [[[1.0, 2.0, 1.0], [6.0, 8.0, 2.0], [8.0, 4.0, 8.0]], [[6.0, 3.0, 6.0], 
[4.0, 6.0, 6.0], [4.0, 6.0, 3.0]], [[4.0, 2.0, 2.0], [6.0, 4.0, 5.0], [5.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RLRN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 6.0], [1.0, 6.0, 6.0], [7.0, 4.0, 6.0]], [[5.0, 4.0, 6.0], [6.0, 7.0, 8.0], [3.0, 3.0, 
8.0]], [[2.0, 2.0, 8.0], [3.0, 2.0, 7.0], [7.0, 7.0, 2.0]]], [[[3.0, 1.0, 4.0], [2.0, 2.0, 5.0], [1.0, 3.0, 7.0]], [[6.0, 8.0, 
8.0], [2.0, 6.0, 1.0], [3.0, 6.0, 7.0]], [[6.0, 5.0, 4.0], [2.0, 1.0, 3.0], [7.0, 3.0, 6.0]]], [[[1.0, 3.0, 3.0], [6.0, 5.0, 
5.0], [4.0, 2.0, 8.0]], [[4.0, 4.0, 3.0], [1.0, 2.0, 7.0], [8.0, 4.0, 3.0]], [[8.0, 4.0, 1.0], [1.0, 2.0, 3.0], [1.0, 4.0, 
4.0]]], [[[5.0, 8.0, 1.0], [4.0, 5.0, 6.0], [8.0, 2.0, 8.0]], [[4.0, 5.0, 3.0], [8.0, 1.0, 8.0], [3.0, 5.0, 7.0]], [[5.0, 1.0, 
5.0], [8.0, 5.0, 6.0], [5.0, 1.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 6.0], [3.0, 2.0, 7.0], [1.0, 8.0, 7.0]], [[4.0, 8.0, 5.0], [4.0, 4.0, 
2.0], [8.0, 3.0, 3.0]], [[3.0, 8.0, 1.0], [7.0, 4.0, 1.0], [7.0, 5.0, 8.0]]], [[[5.0, 7.0, 7.0], [3.0, 5.0, 4.0], [3.0, 1.0, 
1.0]], [[1.0, 1.0, 1.0], [3.0, 5.0, 3.0], [5.0, 3.0, 2.0]], [[3.0, 8.0, 2.0], [8.0, 4.0, 3.0], [8.0, 3.0, 1.0]]], [[[2.0, 3.0, 
2.0], [6.0, 1.0, 1.0], [4.0, 6.0, 4.0]], [[6.0, 2.0, 4.0], [2.0, 1.0, 8.0], [6.0, 2.0, 2.0]], [[7.0, 1.0, 4.0], [6.0, 7.0, 
6.0], [1.0, 2.0, 5.0]]], [[[4.0, 6.0, 4.0], [1.0, 1.0, 7.0], [6.0, 4.0, 4.0]], [[8.0, 2.0, 8.0], [6.0, 5.0, 6.0], [8.0, 1.0, 
4.0]], [[8.0, 1.0, 1.0], [8.0, 8.0, 6.0], [5.0, 4.0, 5.0]]]]))
      val=numpy.array([[[[2.0, 4.0, 4.0]]], [[[5.0, 1.0, 8.0]]]])
      arg[1:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 6.0], [1.0, 6.0, 6.0], [7.0, 4.0, 6.0]], [[5.0, 4.0, 6.0], [6.0, 7.0, 8.0], [3.0, 
3.0, 8.0]], [[2.0, 2.0, 8.0], [3.0, 2.0, 7.0], [7.0, 7.0, 2.0]]], [[[3.0, 1.0, 4.0], [2.0, 2.0, 5.0], [1.0, 3.0, 7.0]], [[6.0, 
8.0, 8.0], [2.0, 6.0, 1.0], [3.0, 6.0, 7.0]], [[6.0, 5.0, 4.0], [2.0, 4.0, 4.0], [7.0, 3.0, 6.0]]], [[[1.0, 3.0, 3.0], [6.0, 
5.0, 5.0], [4.0, 2.0, 8.0]], [[4.0, 4.0, 3.0], [1.0, 2.0, 7.0], [8.0, 4.0, 3.0]], [[8.0, 4.0, 1.0], [5.0, 1.0, 8.0], [1.0, 4.0, 
4.0]]], [[[5.0, 8.0, 1.0], [4.0, 5.0, 6.0], [8.0, 2.0, 8.0]], [[4.0, 5.0, 3.0], [8.0, 1.0, 8.0], [3.0, 5.0, 7.0]], [[5.0, 1.0, 
5.0], [8.0, 5.0, 6.0], [5.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 6.0], [3.0, 2.0, 7.0], [1.0, 8.0, 7.0]], 
[[4.0, 8.0, 5.0], [4.0, 4.0, 2.0], [8.0, 3.0, 3.0]], [[3.0, 8.0, 1.0], [7.0, 4.0, 1.0], [7.0, 5.0, 8.0]]], [[[5.0, 7.0, 7.0], 
[3.0, 5.0, 4.0], [3.0, 1.0, 1.0]], [[1.0, 1.0, 1.0], [3.0, 5.0, 3.0], [5.0, 3.0, 2.0]], [[3.0, 8.0, 2.0], [2.0, 4.0, 4.0], 
[8.0, 3.0, 1.0]]], [[[2.0, 3.0, 2.0], [6.0, 1.0, 1.0], [4.0, 6.0, 4.0]], [[6.0, 2.0, 4.0], [2.0, 1.0, 8.0], [6.0, 2.0, 2.0]], 
[[7.0, 1.0, 4.0], [5.0, 1.0, 8.0], [1.0, 2.0, 5.0]]], [[[4.0, 6.0, 4.0], [1.0, 1.0, 7.0], [6.0, 4.0, 4.0]], [[8.0, 2.0, 8.0], 
[6.0, 5.0, 6.0], [8.0, 1.0, 4.0]], [[8.0, 1.0, 1.0], [8.0, 8.0, 6.0], [5.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RLRN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 6.0], [8.0, 2.0, 6.0], [7.0, 2.0, 4.0]], [[5.0, 8.0, 1.0], [4.0, 2.0, 7.0], [3.0, 6.0, 
3.0]], [[4.0, 7.0, 6.0], [2.0, 6.0, 6.0], [7.0, 5.0, 5.0]]], [[[8.0, 7.0, 2.0], [6.0, 1.0, 4.0], [8.0, 5.0, 3.0]], [[1.0, 1.0, 
5.0], [1.0, 7.0, 4.0], [3.0, 1.0, 3.0]], [[2.0, 6.0, 1.0], [8.0, 6.0, 5.0], [4.0, 2.0, 8.0]]], [[[2.0, 8.0, 6.0], [7.0, 4.0, 
8.0], [3.0, 8.0, 3.0]], [[5.0, 7.0, 4.0], [6.0, 2.0, 6.0], [6.0, 1.0, 7.0]], [[3.0, 2.0, 3.0], [3.0, 1.0, 6.0], [4.0, 5.0, 
4.0]]], [[[2.0, 5.0, 8.0], [5.0, 7.0, 2.0], [6.0, 7.0, 8.0]], [[1.0, 1.0, 7.0], [8.0, 6.0, 5.0], [3.0, 7.0, 4.0]], [[3.0, 8.0, 
6.0], [2.0, 8.0, 3.0], [5.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 1.0], [7.0, 6.0, 2.0], [6.0, 2.0, 7.0]], [[5.0, 5.0, 6.0], [5.0, 7.0, 
4.0], [4.0, 4.0, 4.0]], [[8.0, 1.0, 2.0], [8.0, 8.0, 5.0], [1.0, 4.0, 8.0]]], [[[3.0, 1.0, 4.0], [2.0, 2.0, 3.0], [6.0, 6.0, 
5.0]], [[5.0, 4.0, 1.0], [2.0, 1.0, 1.0], [6.0, 8.0, 5.0]], [[7.0, 5.0, 6.0], [6.0, 3.0, 1.0], [2.0, 8.0, 6.0]]], [[[3.0, 7.0, 
2.0], [7.0, 6.0, 5.0], [8.0, 8.0, 3.0]], [[2.0, 2.0, 1.0], [8.0, 6.0, 8.0], [3.0, 4.0, 4.0]], [[5.0, 3.0, 2.0], [8.0, 6.0, 
1.0], [3.0, 4.0, 2.0]]], [[[3.0, 6.0, 2.0], [8.0, 2.0, 8.0], [7.0, 1.0, 2.0]], [[4.0, 6.0, 3.0], [1.0, 1.0, 7.0], [3.0, 8.0, 
3.0]], [[3.0, 8.0, 4.0], [7.0, 2.0, 3.0], [3.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[[[5.0, 1.0, 8.0]]], [[[8.0, 6.0, 5.0]]]]),self.functionspace)
      arg[1:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 6.0], [8.0, 2.0, 6.0], [7.0, 2.0, 4.0]], [[5.0, 8.0, 1.0], [4.0, 2.0, 7.0], [3.0, 
6.0, 3.0]], [[4.0, 7.0, 6.0], [2.0, 6.0, 6.0], [7.0, 5.0, 5.0]]], [[[8.0, 7.0, 2.0], [6.0, 1.0, 4.0], [8.0, 5.0, 3.0]], [[1.0, 
1.0, 5.0], [1.0, 7.0, 4.0], [3.0, 1.0, 3.0]], [[2.0, 6.0, 1.0], [5.0, 1.0, 8.0], [4.0, 2.0, 8.0]]], [[[2.0, 8.0, 6.0], [7.0, 
4.0, 8.0], [3.0, 8.0, 3.0]], [[5.0, 7.0, 4.0], [6.0, 2.0, 6.0], [6.0, 1.0, 7.0]], [[3.0, 2.0, 3.0], [8.0, 6.0, 5.0], [4.0, 5.0, 
4.0]]], [[[2.0, 5.0, 8.0], [5.0, 7.0, 2.0], [6.0, 7.0, 8.0]], [[1.0, 1.0, 7.0], [8.0, 6.0, 5.0], [3.0, 7.0, 4.0]], [[3.0, 8.0, 
6.0], [2.0, 8.0, 3.0], [5.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 1.0], [7.0, 6.0, 2.0], [6.0, 2.0, 7.0]], 
[[5.0, 5.0, 6.0], [5.0, 7.0, 4.0], [4.0, 4.0, 4.0]], [[8.0, 1.0, 2.0], [8.0, 8.0, 5.0], [1.0, 4.0, 8.0]]], [[[3.0, 1.0, 4.0], 
[2.0, 2.0, 3.0], [6.0, 6.0, 5.0]], [[5.0, 4.0, 1.0], [2.0, 1.0, 1.0], [6.0, 8.0, 5.0]], [[7.0, 5.0, 6.0], [5.0, 1.0, 8.0], 
[2.0, 8.0, 6.0]]], [[[3.0, 7.0, 2.0], [7.0, 6.0, 5.0], [8.0, 8.0, 3.0]], [[2.0, 2.0, 1.0], [8.0, 6.0, 8.0], [3.0, 4.0, 4.0]], 
[[5.0, 3.0, 2.0], [8.0, 6.0, 5.0], [3.0, 4.0, 2.0]]], [[[3.0, 6.0, 2.0], [8.0, 2.0, 8.0], [7.0, 1.0, 2.0]], [[4.0, 6.0, 3.0], 
[1.0, 1.0, 7.0], [3.0, 8.0, 3.0]], [[3.0, 8.0, 4.0], [7.0, 2.0, 3.0], [3.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RLRN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 6.0], [8.0, 8.0, 6.0], [3.0, 5.0, 3.0]], [[1.0, 2.0, 8.0], [2.0, 2.0, 7.0], [5.0, 5.0, 
3.0]], [[2.0, 1.0, 4.0], [3.0, 6.0, 2.0], [4.0, 6.0, 7.0]]], [[[2.0, 1.0, 6.0], [8.0, 3.0, 6.0], [5.0, 2.0, 6.0]], [[1.0, 1.0, 
6.0], [3.0, 8.0, 2.0], [4.0, 1.0, 5.0]], [[3.0, 3.0, 1.0], [3.0, 7.0, 3.0], [5.0, 7.0, 6.0]]], [[[6.0, 7.0, 1.0], [3.0, 2.0, 
2.0], [3.0, 5.0, 3.0]], [[8.0, 5.0, 5.0], [7.0, 5.0, 3.0], [7.0, 6.0, 7.0]], [[8.0, 2.0, 4.0], [6.0, 8.0, 1.0], [8.0, 2.0, 
8.0]]], [[[2.0, 5.0, 2.0], [6.0, 7.0, 5.0], [1.0, 5.0, 2.0]], [[2.0, 1.0, 1.0], [2.0, 3.0, 4.0], [7.0, 8.0, 1.0]], [[6.0, 2.0, 
4.0], [1.0, 8.0, 8.0], [7.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 7.0], [8.0, 4.0, 6.0], [7.0, 7.0, 8.0]], [[6.0, 1.0, 5.0], [2.0, 2.0, 
7.0], [3.0, 4.0, 6.0]], [[8.0, 7.0, 1.0], [3.0, 4.0, 2.0], [5.0, 6.0, 5.0]]], [[[6.0, 8.0, 4.0], [7.0, 4.0, 4.0], [6.0, 3.0, 
6.0]], [[4.0, 3.0, 1.0], [5.0, 6.0, 6.0], [8.0, 2.0, 7.0]], [[8.0, 5.0, 7.0], [4.0, 7.0, 1.0], [3.0, 2.0, 5.0]]], [[[1.0, 6.0, 
6.0], [2.0, 8.0, 2.0], [1.0, 4.0, 2.0]], [[2.0, 4.0, 2.0], [5.0, 3.0, 6.0], [5.0, 4.0, 7.0]], [[4.0, 7.0, 7.0], [4.0, 2.0, 
2.0], [4.0, 6.0, 1.0]]], [[[5.0, 7.0, 7.0], [5.0, 8.0, 1.0], [1.0, 3.0, 3.0]], [[5.0, 3.0, 1.0], [2.0, 4.0, 5.0], [2.0, 6.0, 
3.0]], [[8.0, 5.0, 2.0], [4.0, 8.0, 5.0], [8.0, 6.0, 5.0]]]]))
      val=Data(numpy.array([[[[1.0, 4.0, 6.0]]], [[[8.0, 8.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 8.0]]], [[[2.0, 6.0, 7.0]]]]))
      arg[1:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 6.0], [8.0, 8.0, 6.0], [3.0, 5.0, 3.0]], [[1.0, 2.0, 8.0], [2.0, 2.0, 7.0], [5.0, 
5.0, 3.0]], [[2.0, 1.0, 4.0], [3.0, 6.0, 2.0], [4.0, 6.0, 7.0]]], [[[2.0, 1.0, 6.0], [8.0, 3.0, 6.0], [5.0, 2.0, 6.0]], [[1.0, 
1.0, 6.0], [3.0, 8.0, 2.0], [4.0, 1.0, 5.0]], [[3.0, 3.0, 1.0], [1.0, 4.0, 6.0], [5.0, 7.0, 6.0]]], [[[6.0, 7.0, 1.0], [3.0, 
2.0, 2.0], [3.0, 5.0, 3.0]], [[8.0, 5.0, 5.0], [7.0, 5.0, 3.0], [7.0, 6.0, 7.0]], [[8.0, 2.0, 4.0], [8.0, 8.0, 1.0], [8.0, 2.0, 
8.0]]], [[[2.0, 5.0, 2.0], [6.0, 7.0, 5.0], [1.0, 5.0, 2.0]], [[2.0, 1.0, 1.0], [2.0, 3.0, 4.0], [7.0, 8.0, 1.0]], [[6.0, 2.0, 
4.0], [1.0, 8.0, 8.0], [7.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 7.0], [8.0, 4.0, 6.0], [7.0, 7.0, 8.0]], 
[[6.0, 1.0, 5.0], [2.0, 2.0, 7.0], [3.0, 4.0, 6.0]], [[8.0, 7.0, 1.0], [3.0, 4.0, 2.0], [5.0, 6.0, 5.0]]], [[[6.0, 8.0, 4.0], 
[7.0, 4.0, 4.0], [6.0, 3.0, 6.0]], [[4.0, 3.0, 1.0], [5.0, 6.0, 6.0], [8.0, 2.0, 7.0]], [[8.0, 5.0, 7.0], [4.0, 4.0, 8.0], 
[3.0, 2.0, 5.0]]], [[[1.0, 6.0, 6.0], [2.0, 8.0, 2.0], [1.0, 4.0, 2.0]], [[2.0, 4.0, 2.0], [5.0, 3.0, 6.0], [5.0, 4.0, 7.0]], 
[[4.0, 7.0, 7.0], [2.0, 6.0, 7.0], [4.0, 6.0, 1.0]]], [[[5.0, 7.0, 7.0], [5.0, 8.0, 1.0], [1.0, 3.0, 3.0]], [[5.0, 3.0, 1.0], 
[2.0, 4.0, 5.0], [2.0, 6.0, 3.0]], [[8.0, 5.0, 2.0], [4.0, 8.0, 5.0], [8.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RLRN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 7.0], [2.0, 1.0, 4.0], [1.0, 4.0, 7.0]], [[7.0, 5.0, 4.0], [1.0, 1.0, 4.0], [5.0, 5.0, 
3.0]], [[7.0, 5.0, 1.0], [6.0, 4.0, 3.0], [2.0, 4.0, 2.0]]], [[[1.0, 5.0, 4.0], [3.0, 1.0, 5.0], [8.0, 8.0, 1.0]], [[5.0, 7.0, 
4.0], [2.0, 5.0, 6.0], [7.0, 5.0, 3.0]], [[8.0, 1.0, 5.0], [6.0, 4.0, 3.0], [8.0, 2.0, 1.0]]], [[[1.0, 5.0, 2.0], [8.0, 1.0, 
6.0], [1.0, 1.0, 2.0]], [[6.0, 3.0, 3.0], [7.0, 4.0, 6.0], [3.0, 4.0, 1.0]], [[5.0, 6.0, 1.0], [2.0, 3.0, 4.0], [6.0, 2.0, 
2.0]]], [[[4.0, 7.0, 8.0], [1.0, 6.0, 1.0], [2.0, 1.0, 6.0]], [[6.0, 4.0, 7.0], [7.0, 1.0, 4.0], [6.0, 4.0, 7.0]], [[5.0, 5.0, 
1.0], [3.0, 4.0, 3.0], [1.0, 8.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 4.0], [8.0, 2.0, 5.0], [3.0, 6.0, 5.0]], [[6.0, 7.0, 7.0], [1.0, 8.0, 
5.0], [3.0, 8.0, 5.0]], [[6.0, 3.0, 2.0], [8.0, 7.0, 1.0], [2.0, 2.0, 7.0]]], [[[6.0, 2.0, 2.0], [2.0, 1.0, 3.0], [7.0, 7.0, 
6.0]], [[4.0, 3.0, 1.0], [5.0, 8.0, 2.0], [8.0, 4.0, 8.0]], [[7.0, 3.0, 3.0], [8.0, 2.0, 4.0], [4.0, 4.0, 5.0]]], [[[8.0, 3.0, 
1.0], [8.0, 6.0, 1.0], [2.0, 6.0, 6.0]], [[2.0, 5.0, 8.0], [3.0, 6.0, 5.0], [7.0, 4.0, 2.0]], [[3.0, 5.0, 6.0], [6.0, 1.0, 
5.0], [1.0, 6.0, 5.0]]], [[[8.0, 3.0, 6.0], [8.0, 2.0, 2.0], [5.0, 3.0, 5.0]], [[8.0, 6.0, 1.0], [3.0, 1.0, 8.0], [4.0, 8.0, 
1.0]], [[1.0, 4.0, 6.0], [8.0, 7.0, 4.0], [5.0, 8.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 4.0, 8.0]]], [[[7.0, 8.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 4.0, 6.0]]], 
[[[5.0, 5.0, 7.0]]]])
      arg[1:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 7.0], [2.0, 1.0, 4.0], [1.0, 4.0, 7.0]], [[7.0, 5.0, 4.0], [1.0, 1.0, 4.0], [5.0, 
5.0, 3.0]], [[7.0, 5.0, 1.0], [6.0, 4.0, 3.0], [2.0, 4.0, 2.0]]], [[[1.0, 5.0, 4.0], [3.0, 1.0, 5.0], [8.0, 8.0, 1.0]], [[5.0, 
7.0, 4.0], [2.0, 5.0, 6.0], [7.0, 5.0, 3.0]], [[8.0, 1.0, 5.0], [8.0, 4.0, 8.0], [8.0, 2.0, 1.0]]], [[[1.0, 5.0, 2.0], [8.0, 
1.0, 6.0], [1.0, 1.0, 2.0]], [[6.0, 3.0, 3.0], [7.0, 4.0, 6.0], [3.0, 4.0, 1.0]], [[5.0, 6.0, 1.0], [7.0, 8.0, 6.0], [6.0, 2.0, 
2.0]]], [[[4.0, 7.0, 8.0], [1.0, 6.0, 1.0], [2.0, 1.0, 6.0]], [[6.0, 4.0, 7.0], [7.0, 1.0, 4.0], [6.0, 4.0, 7.0]], [[5.0, 5.0, 
1.0], [3.0, 4.0, 3.0], [1.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 4.0], [8.0, 2.0, 5.0], [3.0, 6.0, 5.0]], 
[[6.0, 7.0, 7.0], [1.0, 8.0, 5.0], [3.0, 8.0, 5.0]], [[6.0, 3.0, 2.0], [8.0, 7.0, 1.0], [2.0, 2.0, 7.0]]], [[[6.0, 2.0, 2.0], 
[2.0, 1.0, 3.0], [7.0, 7.0, 6.0]], [[4.0, 3.0, 1.0], [5.0, 8.0, 2.0], [8.0, 4.0, 8.0]], [[7.0, 3.0, 3.0], [6.0, 4.0, 6.0], 
[4.0, 4.0, 5.0]]], [[[8.0, 3.0, 1.0], [8.0, 6.0, 1.0], [2.0, 6.0, 6.0]], [[2.0, 5.0, 8.0], [3.0, 6.0, 5.0], [7.0, 4.0, 2.0]], 
[[3.0, 5.0, 6.0], [5.0, 5.0, 7.0], [1.0, 6.0, 5.0]]], [[[8.0, 3.0, 6.0], [8.0, 2.0, 2.0], [5.0, 3.0, 5.0]], [[8.0, 6.0, 1.0], 
[3.0, 1.0, 8.0], [4.0, 8.0, 1.0]], [[1.0, 4.0, 6.0], [8.0, 7.0, 4.0], [5.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RRRN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 6.0], [1.0, 8.0, 5.0], [7.0, 4.0, 8.0]], [[1.0, 5.0, 5.0], [4.0, 7.0, 4.0], [7.0, 6.0, 
4.0]], [[3.0, 8.0, 3.0], [8.0, 6.0, 1.0], [1.0, 7.0, 3.0]]], [[[2.0, 1.0, 3.0], [7.0, 1.0, 1.0], [7.0, 4.0, 2.0]], [[5.0, 7.0, 
4.0], [5.0, 6.0, 3.0], [8.0, 2.0, 6.0]], [[6.0, 2.0, 5.0], [4.0, 4.0, 3.0], [7.0, 5.0, 2.0]]], [[[2.0, 4.0, 8.0], [4.0, 2.0, 
7.0], [3.0, 7.0, 5.0]], [[7.0, 7.0, 4.0], [6.0, 2.0, 1.0], [5.0, 2.0, 5.0]], [[5.0, 7.0, 1.0], [8.0, 5.0, 2.0], [7.0, 4.0, 
2.0]]], [[[3.0, 8.0, 1.0], [8.0, 7.0, 5.0], [8.0, 5.0, 6.0]], [[3.0, 6.0, 7.0], [6.0, 4.0, 4.0], [8.0, 1.0, 3.0]], [[5.0, 7.0, 
3.0], [6.0, 2.0, 3.0], [8.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 1.0, 2.0], [2.0, 7.0, 3.0], [5.0, 6.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 5.0, 
7.0], [4.0, 7.0, 2.0]], [[5.0, 8.0, 2.0], [5.0, 2.0, 3.0], [5.0, 5.0, 1.0]]], [[[7.0, 1.0, 8.0], [3.0, 1.0, 1.0], [6.0, 6.0, 
7.0]], [[7.0, 1.0, 7.0], [1.0, 2.0, 8.0], [1.0, 3.0, 2.0]], [[5.0, 1.0, 5.0], [7.0, 3.0, 5.0], [7.0, 5.0, 6.0]]], [[[8.0, 2.0, 
1.0], [4.0, 7.0, 3.0], [6.0, 1.0, 6.0]], [[2.0, 6.0, 4.0], [4.0, 6.0, 6.0], [4.0, 3.0, 8.0]], [[8.0, 3.0, 6.0], [3.0, 7.0, 
1.0], [4.0, 7.0, 7.0]]], [[[4.0, 7.0, 3.0], [5.0, 3.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 4.0, 8.0], [8.0, 8.0, 3.0], [1.0, 7.0, 
2.0]], [[2.0, 2.0, 1.0], [8.0, 7.0, 7.0], [3.0, 7.0, 2.0]]]]))
      val=numpy.array([[[[3.0, 6.0, 3.0]]], [[[4.0, 8.0, 7.0]]]])
      arg[1:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 6.0], [1.0, 8.0, 5.0], [7.0, 4.0, 8.0]], [[1.0, 5.0, 5.0], [4.0, 7.0, 4.0], [7.0, 
6.0, 4.0]], [[3.0, 8.0, 3.0], [8.0, 6.0, 1.0], [1.0, 7.0, 3.0]]], [[[2.0, 1.0, 3.0], [7.0, 1.0, 1.0], [7.0, 4.0, 2.0]], [[5.0, 
7.0, 4.0], [3.0, 6.0, 3.0], [8.0, 2.0, 6.0]], [[6.0, 2.0, 5.0], [4.0, 4.0, 3.0], [7.0, 5.0, 2.0]]], [[[2.0, 4.0, 8.0], [4.0, 
2.0, 7.0], [3.0, 7.0, 5.0]], [[7.0, 7.0, 4.0], [4.0, 8.0, 7.0], [5.0, 2.0, 5.0]], [[5.0, 7.0, 1.0], [8.0, 5.0, 2.0], [7.0, 4.0, 
2.0]]], [[[3.0, 8.0, 1.0], [8.0, 7.0, 5.0], [8.0, 5.0, 6.0]], [[3.0, 6.0, 7.0], [6.0, 4.0, 4.0], [8.0, 1.0, 3.0]], [[5.0, 7.0, 
3.0], [6.0, 2.0, 3.0], [8.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 2.0], [2.0, 7.0, 3.0], [5.0, 6.0, 8.0]], 
[[1.0, 2.0, 6.0], [6.0, 5.0, 7.0], [4.0, 7.0, 2.0]], [[5.0, 8.0, 2.0], [5.0, 2.0, 3.0], [5.0, 5.0, 1.0]]], [[[7.0, 1.0, 8.0], 
[3.0, 1.0, 1.0], [6.0, 6.0, 7.0]], [[7.0, 1.0, 7.0], [3.0, 6.0, 3.0], [1.0, 3.0, 2.0]], [[5.0, 1.0, 5.0], [7.0, 3.0, 5.0], 
[7.0, 5.0, 6.0]]], [[[8.0, 2.0, 1.0], [4.0, 7.0, 3.0], [6.0, 1.0, 6.0]], [[2.0, 6.0, 4.0], [4.0, 8.0, 7.0], [4.0, 3.0, 8.0]], 
[[8.0, 3.0, 6.0], [3.0, 7.0, 1.0], [4.0, 7.0, 7.0]]], [[[4.0, 7.0, 3.0], [5.0, 3.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 4.0, 8.0], 
[8.0, 8.0, 3.0], [1.0, 7.0, 2.0]], [[2.0, 2.0, 1.0], [8.0, 7.0, 7.0], [3.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RRRN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 5.0], [4.0, 7.0, 2.0], [6.0, 6.0, 3.0]], [[8.0, 7.0, 8.0], [1.0, 1.0, 4.0], [8.0, 4.0, 
5.0]], [[1.0, 2.0, 4.0], [6.0, 2.0, 4.0], [4.0, 7.0, 1.0]]], [[[2.0, 8.0, 3.0], [1.0, 6.0, 6.0], [1.0, 6.0, 5.0]], [[3.0, 6.0, 
8.0], [7.0, 7.0, 2.0], [7.0, 3.0, 5.0]], [[7.0, 1.0, 4.0], [2.0, 4.0, 8.0], [1.0, 7.0, 5.0]]], [[[5.0, 8.0, 6.0], [8.0, 3.0, 
3.0], [6.0, 6.0, 3.0]], [[3.0, 7.0, 2.0], [1.0, 5.0, 5.0], [7.0, 3.0, 1.0]], [[6.0, 3.0, 1.0], [6.0, 3.0, 6.0], [1.0, 1.0, 
8.0]]], [[[1.0, 6.0, 1.0], [2.0, 5.0, 5.0], [2.0, 6.0, 6.0]], [[8.0, 2.0, 5.0], [5.0, 5.0, 6.0], [7.0, 8.0, 6.0]], [[3.0, 3.0, 
6.0], [1.0, 6.0, 1.0], [4.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 1.0], [5.0, 6.0, 4.0], [6.0, 4.0, 7.0]], [[5.0, 6.0, 3.0], [8.0, 8.0, 
6.0], [3.0, 2.0, 6.0]], [[4.0, 4.0, 4.0], [5.0, 8.0, 5.0], [3.0, 8.0, 5.0]]], [[[7.0, 8.0, 1.0], [8.0, 2.0, 6.0], [5.0, 1.0, 
4.0]], [[4.0, 5.0, 1.0], [6.0, 4.0, 7.0], [6.0, 2.0, 4.0]], [[8.0, 2.0, 8.0], [3.0, 8.0, 1.0], [4.0, 2.0, 5.0]]], [[[6.0, 6.0, 
5.0], [3.0, 7.0, 4.0], [5.0, 5.0, 2.0]], [[5.0, 6.0, 1.0], [2.0, 2.0, 4.0], [3.0, 1.0, 5.0]], [[4.0, 8.0, 7.0], [8.0, 1.0, 
2.0], [5.0, 2.0, 5.0]]], [[[4.0, 3.0, 1.0], [5.0, 5.0, 6.0], [2.0, 3.0, 1.0]], [[7.0, 1.0, 1.0], [8.0, 7.0, 2.0], [6.0, 5.0, 
3.0]], [[7.0, 7.0, 3.0], [8.0, 4.0, 1.0], [8.0, 6.0, 5.0]]]]))
      val=Data(numpy.array([[[[3.0, 6.0, 1.0]]], [[[8.0, 4.0, 6.0]]]]),self.functionspace)
      arg[1:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 5.0], [4.0, 7.0, 2.0], [6.0, 6.0, 3.0]], [[8.0, 7.0, 8.0], [1.0, 1.0, 4.0], [8.0, 
4.0, 5.0]], [[1.0, 2.0, 4.0], [6.0, 2.0, 4.0], [4.0, 7.0, 1.0]]], [[[2.0, 8.0, 3.0], [1.0, 6.0, 6.0], [1.0, 6.0, 5.0]], [[3.0, 
6.0, 8.0], [3.0, 6.0, 1.0], [7.0, 3.0, 5.0]], [[7.0, 1.0, 4.0], [2.0, 4.0, 8.0], [1.0, 7.0, 5.0]]], [[[5.0, 8.0, 6.0], [8.0, 
3.0, 3.0], [6.0, 6.0, 3.0]], [[3.0, 7.0, 2.0], [8.0, 4.0, 6.0], [7.0, 3.0, 1.0]], [[6.0, 3.0, 1.0], [6.0, 3.0, 6.0], [1.0, 1.0, 
8.0]]], [[[1.0, 6.0, 1.0], [2.0, 5.0, 5.0], [2.0, 6.0, 6.0]], [[8.0, 2.0, 5.0], [5.0, 5.0, 6.0], [7.0, 8.0, 6.0]], [[3.0, 3.0, 
6.0], [1.0, 6.0, 1.0], [4.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 1.0], [5.0, 6.0, 4.0], [6.0, 4.0, 7.0]], 
[[5.0, 6.0, 3.0], [8.0, 8.0, 6.0], [3.0, 2.0, 6.0]], [[4.0, 4.0, 4.0], [5.0, 8.0, 5.0], [3.0, 8.0, 5.0]]], [[[7.0, 8.0, 1.0], 
[8.0, 2.0, 6.0], [5.0, 1.0, 4.0]], [[4.0, 5.0, 1.0], [3.0, 6.0, 1.0], [6.0, 2.0, 4.0]], [[8.0, 2.0, 8.0], [3.0, 8.0, 1.0], 
[4.0, 2.0, 5.0]]], [[[6.0, 6.0, 5.0], [3.0, 7.0, 4.0], [5.0, 5.0, 2.0]], [[5.0, 6.0, 1.0], [8.0, 4.0, 6.0], [3.0, 1.0, 5.0]], 
[[4.0, 8.0, 7.0], [8.0, 1.0, 2.0], [5.0, 2.0, 5.0]]], [[[4.0, 3.0, 1.0], [5.0, 5.0, 6.0], [2.0, 3.0, 1.0]], [[7.0, 1.0, 1.0], 
[8.0, 7.0, 2.0], [6.0, 5.0, 3.0]], [[7.0, 7.0, 3.0], [8.0, 4.0, 1.0], [8.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RRRN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 3.0], [1.0, 8.0, 5.0], [1.0, 3.0, 3.0]], [[4.0, 4.0, 6.0], [2.0, 4.0, 3.0], [3.0, 3.0, 
6.0]], [[4.0, 7.0, 2.0], [7.0, 8.0, 2.0], [8.0, 4.0, 5.0]]], [[[6.0, 3.0, 5.0], [5.0, 7.0, 1.0], [4.0, 1.0, 5.0]], [[7.0, 5.0, 
1.0], [2.0, 8.0, 4.0], [3.0, 5.0, 7.0]], [[2.0, 2.0, 1.0], [5.0, 3.0, 5.0], [8.0, 2.0, 1.0]]], [[[2.0, 8.0, 3.0], [3.0, 3.0, 
4.0], [2.0, 7.0, 8.0]], [[1.0, 6.0, 5.0], [8.0, 3.0, 4.0], [5.0, 8.0, 1.0]], [[1.0, 5.0, 5.0], [4.0, 3.0, 1.0], [8.0, 7.0, 
2.0]]], [[[8.0, 8.0, 6.0], [8.0, 5.0, 3.0], [7.0, 8.0, 7.0]], [[2.0, 3.0, 3.0], [2.0, 6.0, 6.0], [2.0, 5.0, 6.0]], [[8.0, 4.0, 
8.0], [3.0, 2.0, 6.0], [5.0, 6.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 2.0], [2.0, 8.0, 8.0], [3.0, 5.0, 7.0]], [[7.0, 2.0, 8.0], [7.0, 2.0, 
2.0], [8.0, 1.0, 4.0]], [[7.0, 1.0, 3.0], [3.0, 3.0, 6.0], [4.0, 8.0, 2.0]]], [[[8.0, 2.0, 8.0], [7.0, 3.0, 7.0], [6.0, 2.0, 
8.0]], [[8.0, 4.0, 2.0], [4.0, 2.0, 1.0], [7.0, 2.0, 1.0]], [[4.0, 8.0, 2.0], [4.0, 3.0, 8.0], [4.0, 8.0, 6.0]]], [[[7.0, 8.0, 
7.0], [8.0, 3.0, 2.0], [5.0, 1.0, 2.0]], [[4.0, 6.0, 6.0], [6.0, 3.0, 1.0], [1.0, 1.0, 8.0]], [[5.0, 8.0, 4.0], [6.0, 7.0, 
6.0], [6.0, 3.0, 5.0]]], [[[6.0, 2.0, 3.0], [2.0, 8.0, 4.0], [4.0, 7.0, 1.0]], [[5.0, 8.0, 3.0], [1.0, 2.0, 2.0], [1.0, 6.0, 
3.0]], [[4.0, 8.0, 5.0], [6.0, 3.0, 4.0], [5.0, 8.0, 7.0]]]]))
      val=Data(numpy.array([[[[6.0, 7.0, 8.0]]], [[[8.0, 8.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 2.0]]], [[[1.0, 4.0, 3.0]]]]))
      arg[1:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [1.0, 8.0, 5.0], [1.0, 3.0, 3.0]], [[4.0, 4.0, 6.0], [2.0, 4.0, 3.0], [3.0, 
3.0, 6.0]], [[4.0, 7.0, 2.0], [7.0, 8.0, 2.0], [8.0, 4.0, 5.0]]], [[[6.0, 3.0, 5.0], [5.0, 7.0, 1.0], [4.0, 1.0, 5.0]], [[7.0, 
5.0, 1.0], [6.0, 7.0, 8.0], [3.0, 5.0, 7.0]], [[2.0, 2.0, 1.0], [5.0, 3.0, 5.0], [8.0, 2.0, 1.0]]], [[[2.0, 8.0, 3.0], [3.0, 
3.0, 4.0], [2.0, 7.0, 8.0]], [[1.0, 6.0, 5.0], [8.0, 8.0, 3.0], [5.0, 8.0, 1.0]], [[1.0, 5.0, 5.0], [4.0, 3.0, 1.0], [8.0, 7.0, 
2.0]]], [[[8.0, 8.0, 6.0], [8.0, 5.0, 3.0], [7.0, 8.0, 7.0]], [[2.0, 3.0, 3.0], [2.0, 6.0, 6.0], [2.0, 5.0, 6.0]], [[8.0, 4.0, 
8.0], [3.0, 2.0, 6.0], [5.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 2.0], [2.0, 8.0, 8.0], [3.0, 5.0, 7.0]], 
[[7.0, 2.0, 8.0], [7.0, 2.0, 2.0], [8.0, 1.0, 4.0]], [[7.0, 1.0, 3.0], [3.0, 3.0, 6.0], [4.0, 8.0, 2.0]]], [[[8.0, 2.0, 8.0], 
[7.0, 3.0, 7.0], [6.0, 2.0, 8.0]], [[8.0, 4.0, 2.0], [6.0, 1.0, 2.0], [7.0, 2.0, 1.0]], [[4.0, 8.0, 2.0], [4.0, 3.0, 8.0], 
[4.0, 8.0, 6.0]]], [[[7.0, 8.0, 7.0], [8.0, 3.0, 2.0], [5.0, 1.0, 2.0]], [[4.0, 6.0, 6.0], [1.0, 4.0, 3.0], [1.0, 1.0, 8.0]], 
[[5.0, 8.0, 4.0], [6.0, 7.0, 6.0], [6.0, 3.0, 5.0]]], [[[6.0, 2.0, 3.0], [2.0, 8.0, 4.0], [4.0, 7.0, 1.0]], [[5.0, 8.0, 3.0], 
[1.0, 2.0, 2.0], [1.0, 6.0, 3.0]], [[4.0, 8.0, 5.0], [6.0, 3.0, 4.0], [5.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RRRN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 1.0], [8.0, 4.0, 2.0], [6.0, 7.0, 3.0]], [[2.0, 2.0, 6.0], [6.0, 5.0, 7.0], [6.0, 8.0, 
7.0]], [[3.0, 4.0, 1.0], [7.0, 5.0, 5.0], [7.0, 3.0, 3.0]]], [[[7.0, 5.0, 7.0], [7.0, 3.0, 8.0], [4.0, 4.0, 3.0]], [[2.0, 7.0, 
5.0], [8.0, 7.0, 4.0], [5.0, 3.0, 6.0]], [[7.0, 8.0, 7.0], [7.0, 7.0, 4.0], [7.0, 1.0, 3.0]]], [[[5.0, 7.0, 4.0], [2.0, 3.0, 
8.0], [8.0, 3.0, 5.0]], [[1.0, 6.0, 4.0], [5.0, 1.0, 6.0], [4.0, 4.0, 1.0]], [[7.0, 6.0, 3.0], [3.0, 6.0, 8.0], [2.0, 7.0, 
1.0]]], [[[3.0, 2.0, 7.0], [4.0, 6.0, 2.0], [7.0, 2.0, 6.0]], [[7.0, 8.0, 6.0], [3.0, 7.0, 4.0], [8.0, 6.0, 7.0]], [[3.0, 3.0, 
6.0], [1.0, 8.0, 1.0], [1.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 6.0], [4.0, 8.0, 7.0], [1.0, 6.0, 4.0]], [[7.0, 7.0, 6.0], [1.0, 6.0, 
1.0], [8.0, 5.0, 3.0]], [[8.0, 6.0, 8.0], [1.0, 3.0, 8.0], [5.0, 7.0, 1.0]]], [[[8.0, 3.0, 7.0], [4.0, 4.0, 4.0], [2.0, 2.0, 
5.0]], [[2.0, 7.0, 2.0], [4.0, 5.0, 3.0], [5.0, 3.0, 5.0]], [[2.0, 3.0, 1.0], [7.0, 8.0, 3.0], [8.0, 4.0, 7.0]]], [[[1.0, 1.0, 
3.0], [1.0, 7.0, 2.0], [5.0, 1.0, 4.0]], [[8.0, 7.0, 4.0], [7.0, 5.0, 5.0], [5.0, 1.0, 7.0]], [[4.0, 6.0, 2.0], [7.0, 1.0, 
3.0], [1.0, 2.0, 7.0]]], [[[6.0, 1.0, 2.0], [3.0, 2.0, 7.0], [6.0, 3.0, 2.0]], [[7.0, 1.0, 5.0], [3.0, 5.0, 7.0], [8.0, 8.0, 
2.0]], [[1.0, 2.0, 1.0], [1.0, 3.0, 8.0], [1.0, 7.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 3.0, 7.0]]], [[[2.0, 6.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 1.0, 5.0]]], 
[[[7.0, 7.0, 3.0]]]])
      arg[1:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 1.0], [8.0, 4.0, 2.0], [6.0, 7.0, 3.0]], [[2.0, 2.0, 6.0], [6.0, 5.0, 7.0], [6.0, 
8.0, 7.0]], [[3.0, 4.0, 1.0], [7.0, 5.0, 5.0], [7.0, 3.0, 3.0]]], [[[7.0, 5.0, 7.0], [7.0, 3.0, 8.0], [4.0, 4.0, 3.0]], [[2.0, 
7.0, 5.0], [5.0, 3.0, 7.0], [5.0, 3.0, 6.0]], [[7.0, 8.0, 7.0], [7.0, 7.0, 4.0], [7.0, 1.0, 3.0]]], [[[5.0, 7.0, 4.0], [2.0, 
3.0, 8.0], [8.0, 3.0, 5.0]], [[1.0, 6.0, 4.0], [2.0, 6.0, 2.0], [4.0, 4.0, 1.0]], [[7.0, 6.0, 3.0], [3.0, 6.0, 8.0], [2.0, 7.0, 
1.0]]], [[[3.0, 2.0, 7.0], [4.0, 6.0, 2.0], [7.0, 2.0, 6.0]], [[7.0, 8.0, 6.0], [3.0, 7.0, 4.0], [8.0, 6.0, 7.0]], [[3.0, 3.0, 
6.0], [1.0, 8.0, 1.0], [1.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 6.0], [4.0, 8.0, 7.0], [1.0, 6.0, 4.0]], 
[[7.0, 7.0, 6.0], [1.0, 6.0, 1.0], [8.0, 5.0, 3.0]], [[8.0, 6.0, 8.0], [1.0, 3.0, 8.0], [5.0, 7.0, 1.0]]], [[[8.0, 3.0, 7.0], 
[4.0, 4.0, 4.0], [2.0, 2.0, 5.0]], [[2.0, 7.0, 2.0], [3.0, 1.0, 5.0], [5.0, 3.0, 5.0]], [[2.0, 3.0, 1.0], [7.0, 8.0, 3.0], 
[8.0, 4.0, 7.0]]], [[[1.0, 1.0, 3.0], [1.0, 7.0, 2.0], [5.0, 1.0, 4.0]], [[8.0, 7.0, 4.0], [7.0, 7.0, 3.0], [5.0, 1.0, 7.0]], 
[[4.0, 6.0, 2.0], [7.0, 1.0, 3.0], [1.0, 2.0, 7.0]]], [[[6.0, 1.0, 2.0], [3.0, 2.0, 7.0], [6.0, 3.0, 2.0]], [[7.0, 1.0, 5.0], 
[3.0, 5.0, 7.0], [8.0, 8.0, 2.0]], [[1.0, 2.0, 1.0], [1.0, 3.0, 8.0], [1.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCCC(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 7.0], [3.0, 4.0, 1.0], [1.0, 7.0, 3.0]], [[6.0, 8.0, 2.0], [1.0, 6.0, 1.0], [7.0, 1.0, 
8.0]], [[4.0, 3.0, 1.0], [1.0, 7.0, 8.0], [3.0, 6.0, 3.0]]], [[[8.0, 7.0, 6.0], [6.0, 2.0, 4.0], [6.0, 6.0, 3.0]], [[5.0, 7.0, 
8.0], [4.0, 1.0, 3.0], [2.0, 6.0, 2.0]], [[3.0, 4.0, 2.0], [3.0, 1.0, 2.0], [2.0, 5.0, 7.0]]], [[[1.0, 1.0, 4.0], [3.0, 5.0, 
1.0], [5.0, 8.0, 2.0]], [[4.0, 1.0, 6.0], [2.0, 6.0, 6.0], [3.0, 1.0, 1.0]], [[8.0, 6.0, 3.0], [8.0, 1.0, 8.0], [1.0, 6.0, 
5.0]]], [[[4.0, 1.0, 1.0], [5.0, 4.0, 7.0], [2.0, 3.0, 1.0]], [[2.0, 5.0, 5.0], [3.0, 3.0, 1.0], [1.0, 5.0, 1.0]], [[1.0, 4.0, 
5.0], [4.0, 4.0, 7.0], [1.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 2.0], [1.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[1.0, 2.0, 1.0], [5.0, 2.0, 
4.0], [3.0, 2.0, 7.0]], [[7.0, 8.0, 4.0], [7.0, 3.0, 6.0], [6.0, 7.0, 2.0]]], [[[3.0, 4.0, 4.0], [4.0, 5.0, 7.0], [2.0, 1.0, 
3.0]], [[6.0, 6.0, 3.0], [5.0, 5.0, 7.0], [4.0, 1.0, 2.0]], [[6.0, 8.0, 5.0], [1.0, 3.0, 6.0], [8.0, 3.0, 3.0]]], [[[6.0, 1.0, 
6.0], [4.0, 6.0, 1.0], [6.0, 1.0, 2.0]], [[7.0, 4.0, 8.0], [5.0, 8.0, 8.0], [4.0, 3.0, 4.0]], [[7.0, 2.0, 2.0], [8.0, 3.0, 
2.0], [7.0, 4.0, 1.0]]], [[[5.0, 4.0, 4.0], [8.0, 7.0, 8.0], [7.0, 4.0, 5.0]], [[3.0, 2.0, 6.0], [2.0, 7.0, 3.0], [5.0, 3.0, 
5.0]], [[8.0, 5.0, 2.0], [6.0, 8.0, 6.0], [8.0, 3.0, 6.0]]]]))
      val=numpy.array([[[[2.0, 8.0, 3.0], [2.0, 3.0, 8.0], [1.0, 5.0, 8.0]], [[1.0, 1.0, 2.0], [1.0, 3.0, 4.0], [8.0, 5.0, 
4.0]], [[6.0, 5.0, 1.0], [5.0, 8.0, 3.0], [2.0, 1.0, 5.0]]], [[[7.0, 4.0, 7.0], [2.0, 6.0, 2.0], [6.0, 3.0, 1.0]], [[3.0, 2.0, 
2.0], [8.0, 3.0, 2.0], [1.0, 3.0, 1.0]], [[1.0, 2.0, 7.0], [8.0, 7.0, 2.0], [6.0, 8.0, 4.0]]], [[[6.0, 6.0, 8.0], [6.0, 5.0, 
3.0], [5.0, 5.0, 3.0]], [[3.0, 6.0, 3.0], [6.0, 1.0, 4.0], [2.0, 6.0, 2.0]], [[8.0, 8.0, 4.0], [1.0, 5.0, 4.0], [7.0, 2.0, 
5.0]]], [[[4.0, 2.0, 3.0], [2.0, 1.0, 6.0], [8.0, 4.0, 3.0]], [[8.0, 5.0, 2.0], [1.0, 1.0, 7.0], [4.0, 8.0, 6.0]], [[8.0, 3.0, 
3.0], [6.0, 3.0, 1.0], [1.0, 6.0, 4.0]]]])
      arg[:,:,:,:]=val
      ref=Data(numpy.array([[[[2.0, 8.0, 3.0], [2.0, 3.0, 8.0], [1.0, 5.0, 8.0]], [[1.0, 1.0, 2.0], [1.0, 3.0, 4.0], [8.0, 5.0, 
4.0]], [[6.0, 5.0, 1.0], [5.0, 8.0, 3.0], [2.0, 1.0, 5.0]]], [[[7.0, 4.0, 7.0], [2.0, 6.0, 2.0], [6.0, 3.0, 1.0]], [[3.0, 2.0, 
2.0], [8.0, 3.0, 2.0], [1.0, 3.0, 1.0]], [[1.0, 2.0, 7.0], [8.0, 7.0, 2.0], [6.0, 8.0, 4.0]]], [[[6.0, 6.0, 8.0], [6.0, 5.0, 
3.0], [5.0, 5.0, 3.0]], [[3.0, 6.0, 3.0], [6.0, 1.0, 4.0], [2.0, 6.0, 2.0]], [[8.0, 8.0, 4.0], [1.0, 5.0, 4.0], [7.0, 2.0, 
5.0]]], [[[4.0, 2.0, 3.0], [2.0, 1.0, 6.0], [8.0, 4.0, 3.0]], [[8.0, 5.0, 2.0], [1.0, 1.0, 7.0], [4.0, 8.0, 6.0]], [[8.0, 3.0, 
3.0], [6.0, 3.0, 1.0], [1.0, 6.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCCC(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 2.0], [7.0, 6.0, 1.0], [8.0, 5.0, 6.0]], [[3.0, 4.0, 1.0], [1.0, 4.0, 3.0], [5.0, 3.0, 
5.0]], [[8.0, 6.0, 6.0], [1.0, 3.0, 8.0], [7.0, 7.0, 1.0]]], [[[8.0, 7.0, 2.0], [2.0, 6.0, 1.0], [4.0, 6.0, 6.0]], [[3.0, 3.0, 
8.0], [8.0, 4.0, 4.0], [4.0, 8.0, 6.0]], [[8.0, 2.0, 1.0], [5.0, 8.0, 7.0], [6.0, 5.0, 2.0]]], [[[1.0, 5.0, 8.0], [6.0, 8.0, 
6.0], [1.0, 3.0, 4.0]], [[4.0, 4.0, 3.0], [1.0, 6.0, 4.0], [1.0, 3.0, 8.0]], [[5.0, 8.0, 3.0], [2.0, 8.0, 4.0], [4.0, 2.0, 
4.0]]], [[[3.0, 4.0, 8.0], [2.0, 8.0, 4.0], [7.0, 1.0, 7.0]], [[5.0, 3.0, 2.0], [1.0, 5.0, 3.0], [7.0, 6.0, 2.0]], [[7.0, 7.0, 
3.0], [8.0, 3.0, 2.0], [7.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 5.0], [4.0, 4.0, 1.0], [3.0, 2.0, 5.0]], [[8.0, 6.0, 7.0], [4.0, 2.0, 
8.0], [8.0, 6.0, 1.0]], [[5.0, 5.0, 8.0], [5.0, 2.0, 4.0], [8.0, 2.0, 6.0]]], [[[1.0, 4.0, 3.0], [3.0, 1.0, 7.0], [5.0, 8.0, 
6.0]], [[4.0, 1.0, 2.0], [3.0, 7.0, 7.0], [6.0, 8.0, 3.0]], [[5.0, 1.0, 6.0], [6.0, 3.0, 8.0], [2.0, 8.0, 4.0]]], [[[3.0, 4.0, 
8.0], [1.0, 6.0, 7.0], [6.0, 1.0, 5.0]], [[8.0, 6.0, 1.0], [4.0, 4.0, 2.0], [6.0, 5.0, 5.0]], [[6.0, 3.0, 5.0], [3.0, 3.0, 
5.0], [6.0, 7.0, 8.0]]], [[[7.0, 8.0, 5.0], [8.0, 4.0, 6.0], [2.0, 5.0, 2.0]], [[5.0, 8.0, 6.0], [1.0, 8.0, 8.0], [8.0, 4.0, 
1.0]], [[2.0, 2.0, 2.0], [4.0, 8.0, 5.0], [3.0, 2.0, 1.0]]]]))
      val=Data(numpy.array([[[[4.0, 8.0, 7.0], [6.0, 7.0, 7.0], [8.0, 5.0, 4.0]], [[3.0, 3.0, 4.0], [3.0, 4.0, 1.0], [3.0, 3.0, 
4.0]], [[2.0, 3.0, 4.0], [8.0, 4.0, 5.0], [3.0, 8.0, 3.0]]], [[[3.0, 7.0, 5.0], [1.0, 7.0, 8.0], [6.0, 5.0, 3.0]], [[5.0, 5.0, 
8.0], [6.0, 3.0, 2.0], [3.0, 5.0, 8.0]], [[2.0, 4.0, 7.0], [2.0, 6.0, 1.0], [6.0, 3.0, 8.0]]], [[[2.0, 6.0, 2.0], [2.0, 5.0, 
7.0], [8.0, 1.0, 8.0]], [[7.0, 8.0, 5.0], [1.0, 8.0, 8.0], [2.0, 2.0, 6.0]], [[3.0, 4.0, 3.0], [7.0, 4.0, 5.0], [4.0, 1.0, 
4.0]]], [[[4.0, 7.0, 8.0], [2.0, 4.0, 5.0], [4.0, 6.0, 8.0]], [[5.0, 3.0, 5.0], [6.0, 2.0, 2.0], [2.0, 8.0, 8.0]], [[8.0, 1.0, 
4.0], [3.0, 6.0, 3.0], [6.0, 5.0, 2.0]]]]),self.functionspace)
      arg[:,:,:,:]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 7.0], [6.0, 7.0, 7.0], [8.0, 5.0, 4.0]], [[3.0, 3.0, 4.0], [3.0, 4.0, 1.0], [3.0, 3.0, 
4.0]], [[2.0, 3.0, 4.0], [8.0, 4.0, 5.0], [3.0, 8.0, 3.0]]], [[[3.0, 7.0, 5.0], [1.0, 7.0, 8.0], [6.0, 5.0, 3.0]], [[5.0, 5.0, 
8.0], [6.0, 3.0, 2.0], [3.0, 5.0, 8.0]], [[2.0, 4.0, 7.0], [2.0, 6.0, 1.0], [6.0, 3.0, 8.0]]], [[[2.0, 6.0, 2.0], [2.0, 5.0, 
7.0], [8.0, 1.0, 8.0]], [[7.0, 8.0, 5.0], [1.0, 8.0, 8.0], [2.0, 2.0, 6.0]], [[3.0, 4.0, 3.0], [7.0, 4.0, 5.0], [4.0, 1.0, 
4.0]]], [[[4.0, 7.0, 8.0], [2.0, 4.0, 5.0], [4.0, 6.0, 8.0]], [[5.0, 3.0, 5.0], [6.0, 2.0, 2.0], [2.0, 8.0, 8.0]], [[8.0, 1.0, 
4.0], [3.0, 6.0, 3.0], [6.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCCC(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 3.0], [7.0, 5.0, 2.0], [6.0, 8.0, 5.0]], [[8.0, 7.0, 4.0], [8.0, 6.0, 1.0], [8.0, 5.0, 
3.0]], [[6.0, 1.0, 6.0], [7.0, 6.0, 6.0], [6.0, 7.0, 5.0]]], [[[5.0, 4.0, 5.0], [3.0, 5.0, 8.0], [1.0, 6.0, 6.0]], [[4.0, 6.0, 
3.0], [4.0, 2.0, 8.0], [5.0, 6.0, 3.0]], [[1.0, 2.0, 8.0], [2.0, 5.0, 2.0], [7.0, 4.0, 4.0]]], [[[6.0, 1.0, 4.0], [1.0, 4.0, 
8.0], [2.0, 6.0, 3.0]], [[4.0, 5.0, 4.0], [4.0, 8.0, 7.0], [8.0, 1.0, 1.0]], [[5.0, 3.0, 5.0], [5.0, 8.0, 4.0], [6.0, 4.0, 
5.0]]], [[[6.0, 3.0, 2.0], [3.0, 1.0, 4.0], [1.0, 7.0, 5.0]], [[8.0, 6.0, 1.0], [7.0, 6.0, 5.0], [6.0, 8.0, 1.0]], [[4.0, 4.0, 
3.0], [5.0, 2.0, 6.0], [4.0, 2.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 2.0], [6.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 8.0, 3.0], [5.0, 1.0, 
7.0], [4.0, 5.0, 4.0]], [[6.0, 3.0, 2.0], [7.0, 4.0, 6.0], [4.0, 8.0, 1.0]]], [[[4.0, 8.0, 7.0], [5.0, 2.0, 7.0], [2.0, 4.0, 
3.0]], [[6.0, 4.0, 4.0], [1.0, 2.0, 4.0], [8.0, 3.0, 3.0]], [[8.0, 4.0, 2.0], [6.0, 3.0, 3.0], [8.0, 3.0, 1.0]]], [[[2.0, 6.0, 
2.0], [3.0, 3.0, 1.0], [7.0, 2.0, 4.0]], [[1.0, 1.0, 3.0], [4.0, 5.0, 7.0], [2.0, 1.0, 5.0]], [[8.0, 8.0, 4.0], [7.0, 2.0, 
8.0], [2.0, 1.0, 1.0]]], [[[4.0, 7.0, 8.0], [6.0, 1.0, 2.0], [3.0, 5.0, 4.0]], [[6.0, 1.0, 7.0], [4.0, 1.0, 1.0], [1.0, 1.0, 
6.0]], [[5.0, 7.0, 6.0], [1.0, 4.0, 4.0], [3.0, 3.0, 4.0]]]]))
      val=Data(numpy.array([[[[4.0, 4.0, 2.0], [8.0, 3.0, 1.0], [8.0, 2.0, 8.0]], [[3.0, 2.0, 3.0], [3.0, 2.0, 6.0], [2.0, 6.0, 
1.0]], [[8.0, 6.0, 6.0], [5.0, 4.0, 7.0], [7.0, 7.0, 7.0]]], [[[8.0, 1.0, 6.0], [2.0, 5.0, 8.0], [5.0, 6.0, 7.0]], [[2.0, 7.0, 
3.0], [4.0, 8.0, 7.0], [4.0, 3.0, 4.0]], [[5.0, 3.0, 6.0], [8.0, 5.0, 4.0], [5.0, 7.0, 7.0]]], [[[1.0, 4.0, 1.0], [5.0, 1.0, 
8.0], [5.0, 5.0, 2.0]], [[6.0, 8.0, 2.0], [8.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[8.0, 7.0, 5.0], [6.0, 6.0, 8.0], [3.0, 1.0, 
1.0]]], [[[7.0, 8.0, 2.0], [3.0, 5.0, 4.0], [8.0, 3.0, 7.0]], [[4.0, 4.0, 6.0], [7.0, 5.0, 4.0], [4.0, 2.0, 1.0]], [[3.0, 3.0, 
1.0], [2.0, 4.0, 2.0], [2.0, 2.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 7.0, 6.0], [1.0, 6.0, 5.0], [8.0, 4.0, 8.0]], [[8.0, 8.0, 8.0], [3.0, 2.0, 
4.0], [5.0, 8.0, 5.0]], [[6.0, 5.0, 7.0], [7.0, 2.0, 4.0], [2.0, 1.0, 7.0]]], [[[6.0, 1.0, 1.0], [4.0, 5.0, 4.0], [8.0, 2.0, 
6.0]], [[4.0, 1.0, 8.0], [4.0, 1.0, 4.0], [6.0, 2.0, 1.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 7.0], [4.0, 3.0, 6.0]]], [[[5.0, 6.0, 
1.0], [5.0, 4.0, 8.0], [5.0, 1.0, 5.0]], [[2.0, 2.0, 1.0], [2.0, 8.0, 7.0], [4.0, 8.0, 5.0]], [[1.0, 4.0, 1.0], [3.0, 1.0, 
5.0], [3.0, 3.0, 6.0]]], [[[3.0, 1.0, 3.0], [6.0, 8.0, 6.0], [2.0, 2.0, 2.0]], [[1.0, 2.0, 1.0], [5.0, 8.0, 8.0], [4.0, 4.0, 
1.0]], [[2.0, 3.0, 3.0], [3.0, 8.0, 7.0], [7.0, 1.0, 6.0]]]]))
      arg[:,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 2.0], [8.0, 3.0, 1.0], [8.0, 2.0, 8.0]], [[3.0, 2.0, 3.0], [3.0, 2.0, 6.0], [2.0, 
6.0, 1.0]], [[8.0, 6.0, 6.0], [5.0, 4.0, 7.0], [7.0, 7.0, 7.0]]], [[[8.0, 1.0, 6.0], [2.0, 5.0, 8.0], [5.0, 6.0, 7.0]], [[2.0, 
7.0, 3.0], [4.0, 8.0, 7.0], [4.0, 3.0, 4.0]], [[5.0, 3.0, 6.0], [8.0, 5.0, 4.0], [5.0, 7.0, 7.0]]], [[[1.0, 4.0, 1.0], [5.0, 
1.0, 8.0], [5.0, 5.0, 2.0]], [[6.0, 8.0, 2.0], [8.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[8.0, 7.0, 5.0], [6.0, 6.0, 8.0], [3.0, 1.0, 
1.0]]], [[[7.0, 8.0, 2.0], [3.0, 5.0, 4.0], [8.0, 3.0, 7.0]], [[4.0, 4.0, 6.0], [7.0, 5.0, 4.0], [4.0, 2.0, 1.0]], [[3.0, 3.0, 
1.0], [2.0, 4.0, 2.0], [2.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 6.0], [1.0, 6.0, 5.0], [8.0, 4.0, 8.0]], 
[[8.0, 8.0, 8.0], [3.0, 2.0, 4.0], [5.0, 8.0, 5.0]], [[6.0, 5.0, 7.0], [7.0, 2.0, 4.0], [2.0, 1.0, 7.0]]], [[[6.0, 1.0, 1.0], 
[4.0, 5.0, 4.0], [8.0, 2.0, 6.0]], [[4.0, 1.0, 8.0], [4.0, 1.0, 4.0], [6.0, 2.0, 1.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 7.0], 
[4.0, 3.0, 6.0]]], [[[5.0, 6.0, 1.0], [5.0, 4.0, 8.0], [5.0, 1.0, 5.0]], [[2.0, 2.0, 1.0], [2.0, 8.0, 7.0], [4.0, 8.0, 5.0]], 
[[1.0, 4.0, 1.0], [3.0, 1.0, 5.0], [3.0, 3.0, 6.0]]], [[[3.0, 1.0, 3.0], [6.0, 8.0, 6.0], [2.0, 2.0, 2.0]], [[1.0, 2.0, 1.0], 
[5.0, 8.0, 8.0], [4.0, 4.0, 1.0]], [[2.0, 3.0, 3.0], [3.0, 8.0, 7.0], [7.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCCC(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 5.0], [4.0, 7.0, 6.0], [5.0, 7.0, 1.0]], [[3.0, 4.0, 7.0], [2.0, 8.0, 5.0], [7.0, 8.0, 
7.0]], [[7.0, 5.0, 6.0], [6.0, 2.0, 6.0], [2.0, 7.0, 7.0]]], [[[8.0, 2.0, 8.0], [3.0, 1.0, 5.0], [6.0, 8.0, 6.0]], [[4.0, 8.0, 
7.0], [8.0, 6.0, 4.0], [5.0, 3.0, 3.0]], [[3.0, 3.0, 3.0], [8.0, 4.0, 5.0], [7.0, 3.0, 6.0]]], [[[3.0, 7.0, 3.0], [5.0, 8.0, 
5.0], [4.0, 5.0, 2.0]], [[8.0, 8.0, 6.0], [6.0, 8.0, 2.0], [7.0, 6.0, 4.0]], [[5.0, 3.0, 6.0], [5.0, 6.0, 2.0], [4.0, 5.0, 
6.0]]], [[[7.0, 1.0, 8.0], [1.0, 3.0, 3.0], [7.0, 6.0, 3.0]], [[3.0, 8.0, 1.0], [2.0, 7.0, 7.0], [1.0, 6.0, 6.0]], [[5.0, 6.0, 
4.0], [2.0, 5.0, 8.0], [7.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 2.0], [3.0, 4.0, 8.0], [8.0, 4.0, 3.0]], [[3.0, 7.0, 6.0], [3.0, 7.0, 
6.0], [5.0, 6.0, 5.0]], [[6.0, 2.0, 7.0], [2.0, 3.0, 2.0], [6.0, 3.0, 3.0]]], [[[8.0, 7.0, 1.0], [6.0, 4.0, 2.0], [6.0, 5.0, 
4.0]], [[8.0, 5.0, 7.0], [5.0, 5.0, 7.0], [4.0, 6.0, 1.0]], [[1.0, 2.0, 3.0], [2.0, 3.0, 6.0], [4.0, 3.0, 7.0]]], [[[8.0, 5.0, 
4.0], [6.0, 7.0, 6.0], [6.0, 4.0, 4.0]], [[2.0, 2.0, 2.0], [7.0, 6.0, 2.0], [8.0, 7.0, 1.0]], [[4.0, 4.0, 6.0], [4.0, 4.0, 
1.0], [6.0, 4.0, 4.0]]], [[[6.0, 6.0, 2.0], [8.0, 5.0, 3.0], [5.0, 4.0, 2.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 8.0], [2.0, 3.0, 
2.0]], [[3.0, 5.0, 6.0], [7.0, 1.0, 3.0], [3.0, 5.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 1.0, 2.0], [5.0, 7.0, 2.0], [5.0, 4.0, 2.0]], [[1.0, 6.0, 4.0], [5.0, 5.0, 6.0], [8.0, 
8.0, 6.0]], [[5.0, 4.0, 6.0], [4.0, 4.0, 7.0], [5.0, 2.0, 2.0]]], [[[4.0, 2.0, 3.0], [6.0, 1.0, 2.0], [2.0, 6.0, 7.0]], [[4.0, 
1.0, 3.0], [7.0, 8.0, 4.0], [5.0, 4.0, 6.0]], [[7.0, 8.0, 7.0], [3.0, 1.0, 7.0], [7.0, 3.0, 2.0]]], [[[6.0, 1.0, 7.0], [2.0, 
3.0, 8.0], [8.0, 1.0, 6.0]], [[6.0, 2.0, 3.0], [2.0, 5.0, 6.0], [2.0, 8.0, 3.0]], [[4.0, 5.0, 8.0], [8.0, 2.0, 7.0], [8.0, 8.0, 
2.0]]], [[[5.0, 7.0, 3.0], [5.0, 6.0, 8.0], [3.0, 1.0, 6.0]], [[3.0, 7.0, 6.0], [3.0, 3.0, 1.0], [7.0, 7.0, 4.0]], [[2.0, 6.0, 
6.0], [5.0, 5.0, 3.0], [8.0, 4.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 1.0, 5.0], [6.0, 1.0, 6.0], [1.0, 4.0, 5.0]], 
[[5.0, 8.0, 2.0], [1.0, 5.0, 6.0], [6.0, 1.0, 1.0]], [[3.0, 4.0, 3.0], [3.0, 1.0, 8.0], [2.0, 4.0, 2.0]]], [[[2.0, 2.0, 5.0], 
[4.0, 6.0, 3.0], [7.0, 2.0, 4.0]], [[3.0, 1.0, 5.0], [3.0, 7.0, 2.0], [5.0, 3.0, 3.0]], [[3.0, 1.0, 3.0], [3.0, 4.0, 7.0], 
[7.0, 5.0, 3.0]]], [[[2.0, 6.0, 6.0], [5.0, 5.0, 1.0], [6.0, 6.0, 2.0]], [[3.0, 7.0, 8.0], [2.0, 5.0, 4.0], [6.0, 4.0, 7.0]], 
[[3.0, 3.0, 2.0], [3.0, 5.0, 7.0], [3.0, 5.0, 6.0]]], [[[3.0, 3.0, 8.0], [4.0, 2.0, 2.0], [1.0, 5.0, 5.0]], [[1.0, 6.0, 2.0], 
[7.0, 5.0, 7.0], [1.0, 4.0, 1.0]], [[3.0, 8.0, 6.0], [6.0, 1.0, 1.0], [1.0, 2.0, 5.0]]]])
      arg[:,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 2.0], [5.0, 7.0, 2.0], [5.0, 4.0, 2.0]], [[1.0, 6.0, 4.0], [5.0, 5.0, 6.0], [8.0, 
8.0, 6.0]], [[5.0, 4.0, 6.0], [4.0, 4.0, 7.0], [5.0, 2.0, 2.0]]], [[[4.0, 2.0, 3.0], [6.0, 1.0, 2.0], [2.0, 6.0, 7.0]], [[4.0, 
1.0, 3.0], [7.0, 8.0, 4.0], [5.0, 4.0, 6.0]], [[7.0, 8.0, 7.0], [3.0, 1.0, 7.0], [7.0, 3.0, 2.0]]], [[[6.0, 1.0, 7.0], [2.0, 
3.0, 8.0], [8.0, 1.0, 6.0]], [[6.0, 2.0, 3.0], [2.0, 5.0, 6.0], [2.0, 8.0, 3.0]], [[4.0, 5.0, 8.0], [8.0, 2.0, 7.0], [8.0, 8.0, 
2.0]]], [[[5.0, 7.0, 3.0], [5.0, 6.0, 8.0], [3.0, 1.0, 6.0]], [[3.0, 7.0, 6.0], [3.0, 3.0, 1.0], [7.0, 7.0, 4.0]], [[2.0, 6.0, 
6.0], [5.0, 5.0, 3.0], [8.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 5.0], [6.0, 1.0, 6.0], [1.0, 4.0, 5.0]], 
[[5.0, 8.0, 2.0], [1.0, 5.0, 6.0], [6.0, 1.0, 1.0]], [[3.0, 4.0, 3.0], [3.0, 1.0, 8.0], [2.0, 4.0, 2.0]]], [[[2.0, 2.0, 5.0], 
[4.0, 6.0, 3.0], [7.0, 2.0, 4.0]], [[3.0, 1.0, 5.0], [3.0, 7.0, 2.0], [5.0, 3.0, 3.0]], [[3.0, 1.0, 3.0], [3.0, 4.0, 7.0], 
[7.0, 5.0, 3.0]]], [[[2.0, 6.0, 6.0], [5.0, 5.0, 1.0], [6.0, 6.0, 2.0]], [[3.0, 7.0, 8.0], [2.0, 5.0, 4.0], [6.0, 4.0, 7.0]], 
[[3.0, 3.0, 2.0], [3.0, 5.0, 7.0], [3.0, 5.0, 6.0]]], [[[3.0, 3.0, 8.0], [4.0, 2.0, 2.0], [1.0, 5.0, 5.0]], [[1.0, 6.0, 2.0], 
[7.0, 5.0, 7.0], [1.0, 4.0, 1.0]], [[3.0, 8.0, 6.0], [6.0, 1.0, 1.0], [1.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLLN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [7.0, 7.0, 3.0], [3.0, 4.0, 1.0]], [[3.0, 4.0, 5.0], [4.0, 3.0, 2.0], [4.0, 8.0, 
4.0]], [[2.0, 7.0, 1.0], [8.0, 1.0, 3.0], [5.0, 7.0, 8.0]]], [[[8.0, 8.0, 4.0], [7.0, 1.0, 1.0], [5.0, 3.0, 6.0]], [[1.0, 1.0, 
4.0], [4.0, 1.0, 7.0], [3.0, 5.0, 1.0]], [[4.0, 5.0, 8.0], [6.0, 7.0, 4.0], [8.0, 4.0, 6.0]]], [[[6.0, 4.0, 4.0], [3.0, 2.0, 
3.0], [7.0, 4.0, 1.0]], [[5.0, 8.0, 5.0], [5.0, 1.0, 1.0], [5.0, 6.0, 1.0]], [[3.0, 4.0, 2.0], [1.0, 8.0, 3.0], [8.0, 5.0, 
5.0]]], [[[8.0, 1.0, 1.0], [4.0, 7.0, 5.0], [8.0, 5.0, 6.0]], [[7.0, 1.0, 3.0], [8.0, 5.0, 4.0], [2.0, 3.0, 1.0]], [[1.0, 6.0, 
6.0], [7.0, 7.0, 8.0], [7.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 7.0], [3.0, 2.0, 5.0], [1.0, 4.0, 3.0]], [[4.0, 6.0, 2.0], [8.0, 7.0, 
2.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 5.0, 3.0], [6.0, 4.0, 4.0]]], [[[6.0, 5.0, 7.0], [2.0, 4.0, 7.0], [1.0, 1.0, 
6.0]], [[6.0, 3.0, 5.0], [6.0, 7.0, 3.0], [6.0, 5.0, 4.0]], [[1.0, 8.0, 6.0], [7.0, 4.0, 6.0], [5.0, 6.0, 2.0]]], [[[7.0, 2.0, 
3.0], [7.0, 7.0, 2.0], [2.0, 8.0, 1.0]], [[6.0, 8.0, 5.0], [1.0, 6.0, 2.0], [2.0, 6.0, 2.0]], [[5.0, 3.0, 1.0], [5.0, 5.0, 
3.0], [2.0, 1.0, 1.0]]], [[[5.0, 4.0, 7.0], [6.0, 4.0, 3.0], [5.0, 6.0, 1.0]], [[3.0, 3.0, 7.0], [3.0, 5.0, 7.0], [3.0, 6.0, 
8.0]], [[1.0, 4.0, 3.0], [4.0, 1.0, 3.0], [3.0, 2.0, 3.0]]]]))
      val=numpy.array([[[5.0, 8.0, 1.0]]])
      arg[2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [7.0, 7.0, 3.0], [3.0, 4.0, 1.0]], [[3.0, 4.0, 5.0], [4.0, 3.0, 2.0], [4.0, 
8.0, 4.0]], [[2.0, 7.0, 1.0], [8.0, 1.0, 3.0], [5.0, 7.0, 8.0]]], [[[8.0, 8.0, 4.0], [7.0, 1.0, 1.0], [5.0, 3.0, 6.0]], [[1.0, 
1.0, 4.0], [4.0, 1.0, 7.0], [3.0, 5.0, 1.0]], [[4.0, 5.0, 8.0], [6.0, 7.0, 4.0], [8.0, 4.0, 6.0]]], [[[6.0, 4.0, 4.0], [3.0, 
2.0, 3.0], [7.0, 4.0, 1.0]], [[5.0, 8.0, 5.0], [5.0, 1.0, 1.0], [5.0, 6.0, 1.0]], [[3.0, 4.0, 2.0], [1.0, 8.0, 3.0], [5.0, 8.0, 
1.0]]], [[[8.0, 1.0, 1.0], [4.0, 7.0, 5.0], [8.0, 5.0, 6.0]], [[7.0, 1.0, 3.0], [8.0, 5.0, 4.0], [2.0, 3.0, 1.0]], [[1.0, 6.0, 
6.0], [7.0, 7.0, 8.0], [7.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 7.0], [3.0, 2.0, 5.0], [1.0, 4.0, 3.0]], 
[[4.0, 6.0, 2.0], [8.0, 7.0, 2.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 5.0, 3.0], [6.0, 4.0, 4.0]]], [[[6.0, 5.0, 7.0], 
[2.0, 4.0, 7.0], [1.0, 1.0, 6.0]], [[6.0, 3.0, 5.0], [6.0, 7.0, 3.0], [6.0, 5.0, 4.0]], [[1.0, 8.0, 6.0], [7.0, 4.0, 6.0], 
[5.0, 6.0, 2.0]]], [[[7.0, 2.0, 3.0], [7.0, 7.0, 2.0], [2.0, 8.0, 1.0]], [[6.0, 8.0, 5.0], [1.0, 6.0, 2.0], [2.0, 6.0, 2.0]], 
[[5.0, 3.0, 1.0], [5.0, 5.0, 3.0], [5.0, 8.0, 1.0]]], [[[5.0, 4.0, 7.0], [6.0, 4.0, 3.0], [5.0, 6.0, 1.0]], [[3.0, 3.0, 7.0], 
[3.0, 5.0, 7.0], [3.0, 6.0, 8.0]], [[1.0, 4.0, 3.0], [4.0, 1.0, 3.0], [3.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLLN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 3.0], [1.0, 5.0, 5.0], [7.0, 6.0, 1.0]], [[3.0, 1.0, 1.0], [7.0, 3.0, 1.0], [7.0, 8.0, 
5.0]], [[7.0, 3.0, 4.0], [8.0, 5.0, 4.0], [1.0, 4.0, 5.0]]], [[[4.0, 2.0, 8.0], [3.0, 7.0, 7.0], [2.0, 5.0, 6.0]], [[1.0, 7.0, 
6.0], [4.0, 3.0, 4.0], [5.0, 7.0, 6.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 5.0], [1.0, 5.0, 5.0]]], [[[5.0, 6.0, 4.0], [6.0, 5.0, 
4.0], [7.0, 1.0, 8.0]], [[3.0, 7.0, 1.0], [2.0, 2.0, 8.0], [6.0, 5.0, 1.0]], [[6.0, 8.0, 5.0], [5.0, 5.0, 1.0], [6.0, 6.0, 
6.0]]], [[[5.0, 8.0, 8.0], [1.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[3.0, 7.0, 5.0], [2.0, 3.0, 8.0], [1.0, 6.0, 1.0]], [[1.0, 5.0, 
3.0], [3.0, 6.0, 8.0], [1.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 7.0], [5.0, 1.0, 1.0], [3.0, 8.0, 5.0]], [[1.0, 4.0, 8.0], [8.0, 5.0, 
2.0], [8.0, 8.0, 2.0]], [[6.0, 6.0, 8.0], [3.0, 8.0, 4.0], [8.0, 3.0, 5.0]]], [[[2.0, 4.0, 3.0], [4.0, 7.0, 1.0], [6.0, 4.0, 
4.0]], [[6.0, 1.0, 8.0], [2.0, 4.0, 3.0], [4.0, 7.0, 3.0]], [[2.0, 4.0, 3.0], [6.0, 7.0, 5.0], [5.0, 6.0, 4.0]]], [[[3.0, 2.0, 
6.0], [6.0, 8.0, 3.0], [5.0, 8.0, 5.0]], [[8.0, 3.0, 3.0], [6.0, 2.0, 3.0], [5.0, 7.0, 1.0]], [[5.0, 6.0, 6.0], [5.0, 4.0, 
2.0], [6.0, 6.0, 8.0]]], [[[2.0, 6.0, 2.0], [5.0, 4.0, 6.0], [3.0, 3.0, 5.0]], [[3.0, 2.0, 8.0], [1.0, 2.0, 3.0], [7.0, 8.0, 
7.0]], [[8.0, 4.0, 5.0], [3.0, 3.0, 3.0], [7.0, 8.0, 2.0]]]]))
      val=Data(numpy.array([[[5.0, 1.0, 1.0]]]),self.functionspace)
      arg[2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 3.0], [1.0, 5.0, 5.0], [7.0, 6.0, 1.0]], [[3.0, 1.0, 1.0], [7.0, 3.0, 1.0], [7.0, 
8.0, 5.0]], [[7.0, 3.0, 4.0], [8.0, 5.0, 4.0], [1.0, 4.0, 5.0]]], [[[4.0, 2.0, 8.0], [3.0, 7.0, 7.0], [2.0, 5.0, 6.0]], [[1.0, 
7.0, 6.0], [4.0, 3.0, 4.0], [5.0, 7.0, 6.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 5.0], [1.0, 5.0, 5.0]]], [[[5.0, 6.0, 4.0], [6.0, 
5.0, 4.0], [7.0, 1.0, 8.0]], [[3.0, 7.0, 1.0], [2.0, 2.0, 8.0], [6.0, 5.0, 1.0]], [[6.0, 8.0, 5.0], [5.0, 5.0, 1.0], [5.0, 1.0, 
1.0]]], [[[5.0, 8.0, 8.0], [1.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[3.0, 7.0, 5.0], [2.0, 3.0, 8.0], [1.0, 6.0, 1.0]], [[1.0, 5.0, 
3.0], [3.0, 6.0, 8.0], [1.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 7.0], [5.0, 1.0, 1.0], [3.0, 8.0, 5.0]], 
[[1.0, 4.0, 8.0], [8.0, 5.0, 2.0], [8.0, 8.0, 2.0]], [[6.0, 6.0, 8.0], [3.0, 8.0, 4.0], [8.0, 3.0, 5.0]]], [[[2.0, 4.0, 3.0], 
[4.0, 7.0, 1.0], [6.0, 4.0, 4.0]], [[6.0, 1.0, 8.0], [2.0, 4.0, 3.0], [4.0, 7.0, 3.0]], [[2.0, 4.0, 3.0], [6.0, 7.0, 5.0], 
[5.0, 6.0, 4.0]]], [[[3.0, 2.0, 6.0], [6.0, 8.0, 3.0], [5.0, 8.0, 5.0]], [[8.0, 3.0, 3.0], [6.0, 2.0, 3.0], [5.0, 7.0, 1.0]], 
[[5.0, 6.0, 6.0], [5.0, 4.0, 2.0], [5.0, 1.0, 1.0]]], [[[2.0, 6.0, 2.0], [5.0, 4.0, 6.0], [3.0, 3.0, 5.0]], [[3.0, 2.0, 8.0], 
[1.0, 2.0, 3.0], [7.0, 8.0, 7.0]], [[8.0, 4.0, 5.0], [3.0, 3.0, 3.0], [7.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLLN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 6.0], [8.0, 3.0, 2.0], [8.0, 5.0, 2.0]], [[1.0, 3.0, 7.0], [1.0, 8.0, 6.0], [7.0, 2.0, 
6.0]], [[2.0, 1.0, 2.0], [2.0, 5.0, 1.0], [3.0, 1.0, 2.0]]], [[[8.0, 3.0, 1.0], [5.0, 6.0, 6.0], [8.0, 3.0, 7.0]], [[3.0, 1.0, 
4.0], [6.0, 8.0, 3.0], [3.0, 4.0, 7.0]], [[7.0, 2.0, 8.0], [8.0, 1.0, 4.0], [1.0, 4.0, 8.0]]], [[[6.0, 6.0, 3.0], [4.0, 3.0, 
5.0], [5.0, 5.0, 8.0]], [[2.0, 3.0, 2.0], [4.0, 2.0, 3.0], [6.0, 2.0, 1.0]], [[2.0, 7.0, 3.0], [6.0, 6.0, 7.0], [3.0, 5.0, 
3.0]]], [[[5.0, 2.0, 8.0], [7.0, 8.0, 6.0], [3.0, 4.0, 5.0]], [[3.0, 3.0, 3.0], [2.0, 6.0, 7.0], [4.0, 8.0, 7.0]], [[1.0, 1.0, 
4.0], [4.0, 1.0, 1.0], [4.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 5.0], [3.0, 5.0, 6.0], [4.0, 7.0, 8.0]], [[3.0, 7.0, 8.0], [8.0, 5.0, 
5.0], [3.0, 2.0, 7.0]], [[7.0, 2.0, 4.0], [4.0, 5.0, 7.0], [3.0, 6.0, 5.0]]], [[[2.0, 1.0, 5.0], [2.0, 8.0, 1.0], [8.0, 6.0, 
3.0]], [[4.0, 4.0, 4.0], [3.0, 4.0, 3.0], [2.0, 7.0, 7.0]], [[5.0, 2.0, 8.0], [6.0, 6.0, 2.0], [1.0, 5.0, 6.0]]], [[[1.0, 4.0, 
6.0], [3.0, 7.0, 5.0], [3.0, 5.0, 6.0]], [[6.0, 7.0, 2.0], [7.0, 6.0, 7.0], [2.0, 3.0, 5.0]], [[2.0, 5.0, 1.0], [2.0, 7.0, 
1.0], [1.0, 8.0, 8.0]]], [[[5.0, 6.0, 8.0], [1.0, 6.0, 1.0], [7.0, 5.0, 5.0]], [[1.0, 8.0, 8.0], [6.0, 5.0, 3.0], [4.0, 6.0, 
5.0]], [[6.0, 7.0, 7.0], [6.0, 3.0, 2.0], [3.0, 5.0, 2.0]]]]))
      val=Data(numpy.array([[[6.0, 6.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 8.0, 5.0]]]))
      arg[2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 6.0], [8.0, 3.0, 2.0], [8.0, 5.0, 2.0]], [[1.0, 3.0, 7.0], [1.0, 8.0, 6.0], [7.0, 
2.0, 6.0]], [[2.0, 1.0, 2.0], [2.0, 5.0, 1.0], [3.0, 1.0, 2.0]]], [[[8.0, 3.0, 1.0], [5.0, 6.0, 6.0], [8.0, 3.0, 7.0]], [[3.0, 
1.0, 4.0], [6.0, 8.0, 3.0], [3.0, 4.0, 7.0]], [[7.0, 2.0, 8.0], [8.0, 1.0, 4.0], [1.0, 4.0, 8.0]]], [[[6.0, 6.0, 3.0], [4.0, 
3.0, 5.0], [5.0, 5.0, 8.0]], [[2.0, 3.0, 2.0], [4.0, 2.0, 3.0], [6.0, 2.0, 1.0]], [[2.0, 7.0, 3.0], [6.0, 6.0, 7.0], [6.0, 6.0, 
4.0]]], [[[5.0, 2.0, 8.0], [7.0, 8.0, 6.0], [3.0, 4.0, 5.0]], [[3.0, 3.0, 3.0], [2.0, 6.0, 7.0], [4.0, 8.0, 7.0]], [[1.0, 1.0, 
4.0], [4.0, 1.0, 1.0], [4.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 5.0], [3.0, 5.0, 6.0], [4.0, 7.0, 8.0]], 
[[3.0, 7.0, 8.0], [8.0, 5.0, 5.0], [3.0, 2.0, 7.0]], [[7.0, 2.0, 4.0], [4.0, 5.0, 7.0], [3.0, 6.0, 5.0]]], [[[2.0, 1.0, 5.0], 
[2.0, 8.0, 1.0], [8.0, 6.0, 3.0]], [[4.0, 4.0, 4.0], [3.0, 4.0, 3.0], [2.0, 7.0, 7.0]], [[5.0, 2.0, 8.0], [6.0, 6.0, 2.0], 
[1.0, 5.0, 6.0]]], [[[1.0, 4.0, 6.0], [3.0, 7.0, 5.0], [3.0, 5.0, 6.0]], [[6.0, 7.0, 2.0], [7.0, 6.0, 7.0], [2.0, 3.0, 5.0]], 
[[2.0, 5.0, 1.0], [2.0, 7.0, 1.0], [3.0, 8.0, 5.0]]], [[[5.0, 6.0, 8.0], [1.0, 6.0, 1.0], [7.0, 5.0, 5.0]], [[1.0, 8.0, 8.0], 
[6.0, 5.0, 3.0], [4.0, 6.0, 5.0]], [[6.0, 7.0, 7.0], [6.0, 3.0, 2.0], [3.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLLN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 6.0], [7.0, 1.0, 4.0], [8.0, 7.0, 4.0]], [[8.0, 4.0, 3.0], [2.0, 4.0, 3.0], [2.0, 2.0, 
5.0]], [[4.0, 2.0, 4.0], [8.0, 1.0, 8.0], [4.0, 6.0, 8.0]]], [[[3.0, 7.0, 5.0], [7.0, 7.0, 7.0], [3.0, 1.0, 7.0]], [[8.0, 4.0, 
7.0], [7.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[3.0, 2.0, 8.0], [8.0, 7.0, 5.0], [1.0, 4.0, 4.0]]], [[[5.0, 5.0, 5.0], [5.0, 1.0, 
4.0], [1.0, 1.0, 8.0]], [[7.0, 5.0, 7.0], [7.0, 5.0, 3.0], [5.0, 7.0, 6.0]], [[4.0, 7.0, 8.0], [4.0, 1.0, 1.0], [8.0, 1.0, 
5.0]]], [[[8.0, 4.0, 2.0], [6.0, 7.0, 7.0], [1.0, 4.0, 6.0]], [[4.0, 8.0, 3.0], [4.0, 6.0, 6.0], [5.0, 3.0, 3.0]], [[5.0, 4.0, 
2.0], [1.0, 3.0, 4.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 7.0], [8.0, 4.0, 1.0], [8.0, 8.0, 4.0]], [[2.0, 2.0, 7.0], [6.0, 8.0, 
1.0], [7.0, 1.0, 8.0]], [[7.0, 6.0, 7.0], [5.0, 5.0, 5.0], [4.0, 1.0, 1.0]]], [[[3.0, 4.0, 5.0], [2.0, 4.0, 2.0], [4.0, 8.0, 
8.0]], [[7.0, 3.0, 4.0], [4.0, 3.0, 7.0], [4.0, 1.0, 8.0]], [[6.0, 2.0, 2.0], [6.0, 1.0, 4.0], [7.0, 6.0, 1.0]]], [[[5.0, 1.0, 
6.0], [4.0, 8.0, 4.0], [6.0, 8.0, 4.0]], [[1.0, 4.0, 7.0], [7.0, 1.0, 4.0], [1.0, 5.0, 2.0]], [[6.0, 5.0, 3.0], [3.0, 5.0, 
5.0], [5.0, 7.0, 7.0]]], [[[8.0, 2.0, 4.0], [4.0, 7.0, 3.0], [2.0, 2.0, 7.0]], [[8.0, 6.0, 2.0], [3.0, 5.0, 7.0], [7.0, 1.0, 
4.0]], [[6.0, 3.0, 5.0], [1.0, 2.0, 1.0], [4.0, 2.0, 5.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[2.0, 8.0, 3.0]]])+(1.-msk_val)*numpy.array([[[7.0, 3.0, 8.0]]])
      arg[2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 6.0], [7.0, 1.0, 4.0], [8.0, 7.0, 4.0]], [[8.0, 4.0, 3.0], [2.0, 4.0, 3.0], [2.0, 
2.0, 5.0]], [[4.0, 2.0, 4.0], [8.0, 1.0, 8.0], [4.0, 6.0, 8.0]]], [[[3.0, 7.0, 5.0], [7.0, 7.0, 7.0], [3.0, 1.0, 7.0]], [[8.0, 
4.0, 7.0], [7.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[3.0, 2.0, 8.0], [8.0, 7.0, 5.0], [1.0, 4.0, 4.0]]], [[[5.0, 5.0, 5.0], [5.0, 
1.0, 4.0], [1.0, 1.0, 8.0]], [[7.0, 5.0, 7.0], [7.0, 5.0, 3.0], [5.0, 7.0, 6.0]], [[4.0, 7.0, 8.0], [4.0, 1.0, 1.0], [2.0, 8.0, 
3.0]]], [[[8.0, 4.0, 2.0], [6.0, 7.0, 7.0], [1.0, 4.0, 6.0]], [[4.0, 8.0, 3.0], [4.0, 6.0, 6.0], [5.0, 3.0, 3.0]], [[5.0, 4.0, 
2.0], [1.0, 3.0, 4.0], [5.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 7.0], [8.0, 4.0, 1.0], [8.0, 8.0, 4.0]], 
[[2.0, 2.0, 7.0], [6.0, 8.0, 1.0], [7.0, 1.0, 8.0]], [[7.0, 6.0, 7.0], [5.0, 5.0, 5.0], [4.0, 1.0, 1.0]]], [[[3.0, 4.0, 5.0], 
[2.0, 4.0, 2.0], [4.0, 8.0, 8.0]], [[7.0, 3.0, 4.0], [4.0, 3.0, 7.0], [4.0, 1.0, 8.0]], [[6.0, 2.0, 2.0], [6.0, 1.0, 4.0], 
[7.0, 6.0, 1.0]]], [[[5.0, 1.0, 6.0], [4.0, 8.0, 4.0], [6.0, 8.0, 4.0]], [[1.0, 4.0, 7.0], [7.0, 1.0, 4.0], [1.0, 5.0, 2.0]], 
[[6.0, 5.0, 3.0], [3.0, 5.0, 5.0], [7.0, 3.0, 8.0]]], [[[8.0, 2.0, 4.0], [4.0, 7.0, 3.0], [2.0, 2.0, 7.0]], [[8.0, 6.0, 2.0], 
[3.0, 5.0, 7.0], [7.0, 1.0, 4.0]], [[6.0, 3.0, 5.0], [1.0, 2.0, 1.0], [4.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RCPN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 1.0], [6.0, 6.0, 8.0], [1.0, 4.0, 7.0]], [[2.0, 5.0, 2.0], [8.0, 5.0, 7.0], [7.0, 1.0, 
8.0]], [[5.0, 7.0, 7.0], [7.0, 6.0, 5.0], [3.0, 5.0, 7.0]]], [[[7.0, 7.0, 4.0], [4.0, 5.0, 3.0], [1.0, 7.0, 7.0]], [[3.0, 8.0, 
7.0], [7.0, 2.0, 5.0], [2.0, 6.0, 4.0]], [[8.0, 8.0, 2.0], [4.0, 3.0, 2.0], [3.0, 7.0, 7.0]]], [[[4.0, 4.0, 1.0], [1.0, 8.0, 
4.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 1.0], [8.0, 2.0, 1.0], [8.0, 5.0, 2.0]], [[7.0, 4.0, 6.0], [4.0, 2.0, 7.0], [6.0, 5.0, 
8.0]]], [[[3.0, 1.0, 8.0], [7.0, 4.0, 3.0], [6.0, 1.0, 8.0]], [[1.0, 3.0, 6.0], [4.0, 7.0, 5.0], [2.0, 2.0, 3.0]], [[2.0, 7.0, 
5.0], [2.0, 5.0, 4.0], [5.0, 7.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 4.0], [7.0, 7.0, 6.0], [7.0, 7.0, 1.0]], [[6.0, 1.0, 2.0], [7.0, 6.0, 
4.0], [6.0, 4.0, 2.0]], [[2.0, 1.0, 1.0], [1.0, 7.0, 3.0], [8.0, 3.0, 1.0]]], [[[5.0, 7.0, 1.0], [7.0, 7.0, 8.0], [4.0, 2.0, 
6.0]], [[5.0, 1.0, 6.0], [3.0, 8.0, 2.0], [3.0, 8.0, 8.0]], [[3.0, 7.0, 1.0], [2.0, 4.0, 4.0], [4.0, 2.0, 6.0]]], [[[6.0, 4.0, 
8.0], [8.0, 6.0, 8.0], [1.0, 4.0, 8.0]], [[3.0, 2.0, 1.0], [2.0, 1.0, 5.0], [5.0, 5.0, 8.0]], [[3.0, 7.0, 7.0], [1.0, 1.0, 
5.0], [7.0, 6.0, 3.0]]], [[[5.0, 7.0, 7.0], [4.0, 2.0, 8.0], [4.0, 8.0, 8.0]], [[1.0, 6.0, 6.0], [1.0, 4.0, 8.0], [5.0, 5.0, 
1.0]], [[2.0, 7.0, 1.0], [8.0, 3.0, 8.0], [5.0, 8.0, 1.0]]]]))
      val=numpy.array([[[8.0, 1.0, 2.0], [2.0, 8.0, 1.0], [7.0, 2.0, 6.0]], [[6.0, 6.0, 7.0], [8.0, 3.0, 7.0], [2.0, 7.0, 
4.0]]])
      arg[1:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 1.0], [6.0, 6.0, 8.0], [1.0, 4.0, 7.0]], [[2.0, 5.0, 2.0], [8.0, 5.0, 7.0], [7.0, 
1.0, 8.0]], [[5.0, 7.0, 7.0], [7.0, 6.0, 5.0], [3.0, 5.0, 7.0]]], [[[7.0, 7.0, 4.0], [8.0, 1.0, 2.0], [1.0, 7.0, 7.0]], [[3.0, 
8.0, 7.0], [2.0, 8.0, 1.0], [2.0, 6.0, 4.0]], [[8.0, 8.0, 2.0], [7.0, 2.0, 6.0], [3.0, 7.0, 7.0]]], [[[4.0, 4.0, 1.0], [6.0, 
6.0, 7.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 1.0], [8.0, 3.0, 7.0], [8.0, 5.0, 2.0]], [[7.0, 4.0, 6.0], [2.0, 7.0, 4.0], [6.0, 5.0, 
8.0]]], [[[3.0, 1.0, 8.0], [7.0, 4.0, 3.0], [6.0, 1.0, 8.0]], [[1.0, 3.0, 6.0], [4.0, 7.0, 5.0], [2.0, 2.0, 3.0]], [[2.0, 7.0, 
5.0], [2.0, 5.0, 4.0], [5.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 4.0], [7.0, 7.0, 6.0], [7.0, 7.0, 1.0]], 
[[6.0, 1.0, 2.0], [7.0, 6.0, 4.0], [6.0, 4.0, 2.0]], [[2.0, 1.0, 1.0], [1.0, 7.0, 3.0], [8.0, 3.0, 1.0]]], [[[5.0, 7.0, 1.0], 
[8.0, 1.0, 2.0], [4.0, 2.0, 6.0]], [[5.0, 1.0, 6.0], [2.0, 8.0, 1.0], [3.0, 8.0, 8.0]], [[3.0, 7.0, 1.0], [7.0, 2.0, 6.0], 
[4.0, 2.0, 6.0]]], [[[6.0, 4.0, 8.0], [6.0, 6.0, 7.0], [1.0, 4.0, 8.0]], [[3.0, 2.0, 1.0], [8.0, 3.0, 7.0], [5.0, 5.0, 8.0]], 
[[3.0, 7.0, 7.0], [2.0, 7.0, 4.0], [7.0, 6.0, 3.0]]], [[[5.0, 7.0, 7.0], [4.0, 2.0, 8.0], [4.0, 8.0, 8.0]], [[1.0, 6.0, 6.0], 
[1.0, 4.0, 8.0], [5.0, 5.0, 1.0]], [[2.0, 7.0, 1.0], [8.0, 3.0, 8.0], [5.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RCPN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 3.0], [7.0, 4.0, 2.0], [5.0, 7.0, 5.0]], [[8.0, 6.0, 5.0], [2.0, 1.0, 8.0], [4.0, 2.0, 
3.0]], [[2.0, 6.0, 6.0], [4.0, 1.0, 7.0], [6.0, 7.0, 3.0]]], [[[6.0, 1.0, 1.0], [4.0, 8.0, 2.0], [4.0, 1.0, 1.0]], [[2.0, 3.0, 
7.0], [2.0, 8.0, 5.0], [3.0, 4.0, 4.0]], [[7.0, 1.0, 8.0], [7.0, 4.0, 3.0], [5.0, 4.0, 4.0]]], [[[1.0, 5.0, 6.0], [4.0, 8.0, 
5.0], [5.0, 1.0, 4.0]], [[2.0, 2.0, 2.0], [8.0, 8.0, 7.0], [4.0, 6.0, 3.0]], [[7.0, 7.0, 1.0], [3.0, 8.0, 5.0], [4.0, 4.0, 
1.0]]], [[[2.0, 1.0, 1.0], [6.0, 4.0, 3.0], [6.0, 7.0, 7.0]], [[2.0, 3.0, 6.0], [7.0, 2.0, 3.0], [1.0, 5.0, 1.0]], [[2.0, 4.0, 
6.0], [7.0, 3.0, 6.0], [3.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 3.0], [3.0, 3.0, 3.0], [7.0, 3.0, 4.0]], [[3.0, 7.0, 3.0], [5.0, 8.0, 
5.0], [2.0, 4.0, 7.0]], [[6.0, 1.0, 2.0], [4.0, 4.0, 1.0], [4.0, 7.0, 5.0]]], [[[2.0, 3.0, 7.0], [2.0, 4.0, 4.0], [2.0, 8.0, 
8.0]], [[2.0, 6.0, 1.0], [3.0, 3.0, 6.0], [1.0, 2.0, 1.0]], [[7.0, 5.0, 5.0], [6.0, 6.0, 1.0], [5.0, 7.0, 2.0]]], [[[8.0, 4.0, 
5.0], [7.0, 3.0, 1.0], [4.0, 1.0, 7.0]], [[4.0, 6.0, 8.0], [1.0, 4.0, 6.0], [4.0, 7.0, 5.0]], [[8.0, 3.0, 7.0], [2.0, 1.0, 
6.0], [5.0, 3.0, 3.0]]], [[[3.0, 3.0, 3.0], [1.0, 7.0, 8.0], [5.0, 6.0, 2.0]], [[5.0, 7.0, 4.0], [8.0, 7.0, 7.0], [5.0, 1.0, 
1.0]], [[5.0, 7.0, 1.0], [6.0, 6.0, 5.0], [8.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[[3.0, 5.0, 1.0], [4.0, 3.0, 8.0], [8.0, 7.0, 1.0]], [[4.0, 3.0, 5.0], [4.0, 1.0, 3.0], [5.0, 4.0, 
3.0]]]),self.functionspace)
      arg[1:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 3.0], [7.0, 4.0, 2.0], [5.0, 7.0, 5.0]], [[8.0, 6.0, 5.0], [2.0, 1.0, 8.0], [4.0, 
2.0, 3.0]], [[2.0, 6.0, 6.0], [4.0, 1.0, 7.0], [6.0, 7.0, 3.0]]], [[[6.0, 1.0, 1.0], [3.0, 5.0, 1.0], [4.0, 1.0, 1.0]], [[2.0, 
3.0, 7.0], [4.0, 3.0, 8.0], [3.0, 4.0, 4.0]], [[7.0, 1.0, 8.0], [8.0, 7.0, 1.0], [5.0, 4.0, 4.0]]], [[[1.0, 5.0, 6.0], [4.0, 
3.0, 5.0], [5.0, 1.0, 4.0]], [[2.0, 2.0, 2.0], [4.0, 1.0, 3.0], [4.0, 6.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 4.0, 3.0], [4.0, 4.0, 
1.0]]], [[[2.0, 1.0, 1.0], [6.0, 4.0, 3.0], [6.0, 7.0, 7.0]], [[2.0, 3.0, 6.0], [7.0, 2.0, 3.0], [1.0, 5.0, 1.0]], [[2.0, 4.0, 
6.0], [7.0, 3.0, 6.0], [3.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 3.0], [3.0, 3.0, 3.0], [7.0, 3.0, 4.0]], 
[[3.0, 7.0, 3.0], [5.0, 8.0, 5.0], [2.0, 4.0, 7.0]], [[6.0, 1.0, 2.0], [4.0, 4.0, 1.0], [4.0, 7.0, 5.0]]], [[[2.0, 3.0, 7.0], 
[3.0, 5.0, 1.0], [2.0, 8.0, 8.0]], [[2.0, 6.0, 1.0], [4.0, 3.0, 8.0], [1.0, 2.0, 1.0]], [[7.0, 5.0, 5.0], [8.0, 7.0, 1.0], 
[5.0, 7.0, 2.0]]], [[[8.0, 4.0, 5.0], [4.0, 3.0, 5.0], [4.0, 1.0, 7.0]], [[4.0, 6.0, 8.0], [4.0, 1.0, 3.0], [4.0, 7.0, 5.0]], 
[[8.0, 3.0, 7.0], [5.0, 4.0, 3.0], [5.0, 3.0, 3.0]]], [[[3.0, 3.0, 3.0], [1.0, 7.0, 8.0], [5.0, 6.0, 2.0]], [[5.0, 7.0, 4.0], 
[8.0, 7.0, 7.0], [5.0, 1.0, 1.0]], [[5.0, 7.0, 1.0], [6.0, 6.0, 5.0], [8.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RCPN(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 7.0], [1.0, 7.0, 1.0], [8.0, 4.0, 1.0]], [[3.0, 2.0, 6.0], [7.0, 3.0, 1.0], [6.0, 3.0, 
6.0]], [[1.0, 3.0, 7.0], [8.0, 8.0, 8.0], [1.0, 6.0, 2.0]]], [[[1.0, 6.0, 1.0], [5.0, 1.0, 7.0], [7.0, 5.0, 2.0]], [[3.0, 4.0, 
4.0], [6.0, 4.0, 6.0], [7.0, 3.0, 4.0]], [[1.0, 5.0, 6.0], [7.0, 3.0, 8.0], [6.0, 3.0, 7.0]]], [[[7.0, 2.0, 6.0], [6.0, 5.0, 
8.0], [7.0, 4.0, 5.0]], [[7.0, 1.0, 7.0], [5.0, 1.0, 7.0], [8.0, 4.0, 4.0]], [[5.0, 4.0, 2.0], [8.0, 5.0, 5.0], [3.0, 1.0, 
5.0]]], [[[2.0, 8.0, 5.0], [2.0, 8.0, 1.0], [2.0, 6.0, 1.0]], [[5.0, 4.0, 6.0], [5.0, 3.0, 3.0], [2.0, 5.0, 1.0]], [[8.0, 5.0, 
7.0], [4.0, 7.0, 4.0], [8.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 8.0], [8.0, 6.0, 6.0], [7.0, 2.0, 1.0]], [[2.0, 7.0, 3.0], [1.0, 1.0, 
5.0], [6.0, 7.0, 5.0]], [[4.0, 2.0, 1.0], [3.0, 5.0, 1.0], [8.0, 1.0, 7.0]]], [[[6.0, 3.0, 7.0], [8.0, 7.0, 7.0], [4.0, 1.0, 
2.0]], [[3.0, 8.0, 8.0], [6.0, 5.0, 6.0], [5.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [6.0, 7.0, 5.0], [7.0, 4.0, 3.0]]], [[[6.0, 1.0, 
2.0], [2.0, 7.0, 1.0], [8.0, 2.0, 5.0]], [[6.0, 7.0, 3.0], [4.0, 3.0, 2.0], [1.0, 1.0, 8.0]], [[2.0, 6.0, 7.0], [4.0, 2.0, 
2.0], [4.0, 5.0, 7.0]]], [[[5.0, 8.0, 4.0], [5.0, 8.0, 2.0], [8.0, 1.0, 7.0]], [[3.0, 2.0, 1.0], [5.0, 4.0, 7.0], [3.0, 8.0, 
5.0]], [[3.0, 8.0, 3.0], [5.0, 7.0, 5.0], [8.0, 3.0, 4.0]]]]))
      val=Data(numpy.array([[[2.0, 6.0, 5.0], [2.0, 3.0, 2.0], [5.0, 8.0, 2.0]], [[2.0, 6.0, 3.0], [7.0, 4.0, 4.0], [3.0, 7.0, 
6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 2.0, 1.0], [6.0, 8.0, 4.0], [1.0, 5.0, 8.0]], [[2.0, 6.0, 1.0], [1.0, 1.0, 5.0], 
[8.0, 5.0, 4.0]]]))
      arg[1:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 7.0], [1.0, 7.0, 1.0], [8.0, 4.0, 1.0]], [[3.0, 2.0, 6.0], [7.0, 3.0, 1.0], [6.0, 
3.0, 6.0]], [[1.0, 3.0, 7.0], [8.0, 8.0, 8.0], [1.0, 6.0, 2.0]]], [[[1.0, 6.0, 1.0], [2.0, 6.0, 5.0], [7.0, 5.0, 2.0]], [[3.0, 
4.0, 4.0], [2.0, 3.0, 2.0], [7.0, 3.0, 4.0]], [[1.0, 5.0, 6.0], [5.0, 8.0, 2.0], [6.0, 3.0, 7.0]]], [[[7.0, 2.0, 6.0], [2.0, 
6.0, 3.0], [7.0, 4.0, 5.0]], [[7.0, 1.0, 7.0], [7.0, 4.0, 4.0], [8.0, 4.0, 4.0]], [[5.0, 4.0, 2.0], [3.0, 7.0, 6.0], [3.0, 1.0, 
5.0]]], [[[2.0, 8.0, 5.0], [2.0, 8.0, 1.0], [2.0, 6.0, 1.0]], [[5.0, 4.0, 6.0], [5.0, 3.0, 3.0], [2.0, 5.0, 1.0]], [[8.0, 5.0, 
7.0], [4.0, 7.0, 4.0], [8.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 8.0], [8.0, 6.0, 6.0], [7.0, 2.0, 1.0]], 
[[2.0, 7.0, 3.0], [1.0, 1.0, 5.0], [6.0, 7.0, 5.0]], [[4.0, 2.0, 1.0], [3.0, 5.0, 1.0], [8.0, 1.0, 7.0]]], [[[6.0, 3.0, 7.0], 
[1.0, 2.0, 1.0], [4.0, 1.0, 2.0]], [[3.0, 8.0, 8.0], [6.0, 8.0, 4.0], [5.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [1.0, 5.0, 8.0], 
[7.0, 4.0, 3.0]]], [[[6.0, 1.0, 2.0], [2.0, 6.0, 1.0], [8.0, 2.0, 5.0]], [[6.0, 7.0, 3.0], [1.0, 1.0, 5.0], [1.0, 1.0, 8.0]], 
[[2.0, 6.0, 7.0], [8.0, 5.0, 4.0], [4.0, 5.0, 7.0]]], [[[5.0, 8.0, 4.0], [5.0, 8.0, 2.0], [8.0, 1.0, 7.0]], [[3.0, 2.0, 1.0], 
[5.0, 4.0, 7.0], [3.0, 8.0, 5.0]], [[3.0, 8.0, 3.0], [5.0, 7.0, 5.0], [8.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RCPN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 3.0], [1.0, 7.0, 4.0], [1.0, 8.0, 1.0]], [[7.0, 8.0, 3.0], [5.0, 4.0, 4.0], [6.0, 2.0, 
5.0]], [[5.0, 5.0, 5.0], [1.0, 5.0, 4.0], [7.0, 7.0, 5.0]]], [[[6.0, 3.0, 3.0], [6.0, 1.0, 5.0], [6.0, 5.0, 3.0]], [[8.0, 5.0, 
1.0], [8.0, 8.0, 5.0], [4.0, 2.0, 6.0]], [[7.0, 4.0, 5.0], [4.0, 6.0, 3.0], [6.0, 6.0, 1.0]]], [[[2.0, 7.0, 3.0], [8.0, 2.0, 
2.0], [1.0, 1.0, 6.0]], [[1.0, 3.0, 1.0], [2.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[3.0, 8.0, 3.0], [8.0, 3.0, 1.0], [8.0, 5.0, 
4.0]]], [[[7.0, 5.0, 4.0], [7.0, 8.0, 4.0], [8.0, 4.0, 1.0]], [[3.0, 4.0, 4.0], [2.0, 3.0, 1.0], [8.0, 3.0, 6.0]], [[5.0, 5.0, 
4.0], [8.0, 5.0, 3.0], [3.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 2.0], [4.0, 3.0, 1.0], [7.0, 5.0, 7.0]], [[3.0, 7.0, 7.0], [3.0, 4.0, 
1.0], [1.0, 6.0, 8.0]], [[6.0, 7.0, 3.0], [3.0, 5.0, 2.0], [7.0, 7.0, 5.0]]], [[[6.0, 7.0, 2.0], [2.0, 1.0, 3.0], [8.0, 2.0, 
2.0]], [[6.0, 6.0, 4.0], [6.0, 7.0, 4.0], [6.0, 4.0, 2.0]], [[8.0, 2.0, 1.0], [8.0, 8.0, 7.0], [6.0, 4.0, 2.0]]], [[[7.0, 6.0, 
7.0], [3.0, 7.0, 6.0], [1.0, 3.0, 6.0]], [[6.0, 4.0, 8.0], [3.0, 7.0, 4.0], [6.0, 2.0, 1.0]], [[6.0, 2.0, 6.0], [6.0, 3.0, 
2.0], [6.0, 1.0, 5.0]]], [[[8.0, 5.0, 8.0], [3.0, 8.0, 2.0], [5.0, 6.0, 3.0]], [[5.0, 5.0, 2.0], [2.0, 8.0, 1.0], [4.0, 5.0, 
6.0]], [[6.0, 3.0, 7.0], [5.0, 4.0, 6.0], [3.0, 1.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 6.0, 8.0], [7.0, 1.0, 6.0], [8.0, 2.0, 1.0]], [[8.0, 8.0, 3.0], [7.0, 3.0, 6.0], [2.0, 
2.0, 7.0]]])+(1.-msk_val)*numpy.array([[[3.0, 6.0, 4.0], [3.0, 1.0, 5.0], [5.0, 7.0, 1.0]], [[7.0, 3.0, 4.0], [8.0, 3.0, 5.0], 
[3.0, 3.0, 1.0]]])
      arg[1:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 3.0], [1.0, 7.0, 4.0], [1.0, 8.0, 1.0]], [[7.0, 8.0, 3.0], [5.0, 4.0, 4.0], [6.0, 
2.0, 5.0]], [[5.0, 5.0, 5.0], [1.0, 5.0, 4.0], [7.0, 7.0, 5.0]]], [[[6.0, 3.0, 3.0], [5.0, 6.0, 8.0], [6.0, 5.0, 3.0]], [[8.0, 
5.0, 1.0], [7.0, 1.0, 6.0], [4.0, 2.0, 6.0]], [[7.0, 4.0, 5.0], [8.0, 2.0, 1.0], [6.0, 6.0, 1.0]]], [[[2.0, 7.0, 3.0], [8.0, 
8.0, 3.0], [1.0, 1.0, 6.0]], [[1.0, 3.0, 1.0], [7.0, 3.0, 6.0], [5.0, 4.0, 7.0]], [[3.0, 8.0, 3.0], [2.0, 2.0, 7.0], [8.0, 5.0, 
4.0]]], [[[7.0, 5.0, 4.0], [7.0, 8.0, 4.0], [8.0, 4.0, 1.0]], [[3.0, 4.0, 4.0], [2.0, 3.0, 1.0], [8.0, 3.0, 6.0]], [[5.0, 5.0, 
4.0], [8.0, 5.0, 3.0], [3.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 2.0], [4.0, 3.0, 1.0], [7.0, 5.0, 7.0]], 
[[3.0, 7.0, 7.0], [3.0, 4.0, 1.0], [1.0, 6.0, 8.0]], [[6.0, 7.0, 3.0], [3.0, 5.0, 2.0], [7.0, 7.0, 5.0]]], [[[6.0, 7.0, 2.0], 
[3.0, 6.0, 4.0], [8.0, 2.0, 2.0]], [[6.0, 6.0, 4.0], [3.0, 1.0, 5.0], [6.0, 4.0, 2.0]], [[8.0, 2.0, 1.0], [5.0, 7.0, 1.0], 
[6.0, 4.0, 2.0]]], [[[7.0, 6.0, 7.0], [7.0, 3.0, 4.0], [1.0, 3.0, 6.0]], [[6.0, 4.0, 8.0], [8.0, 3.0, 5.0], [6.0, 2.0, 1.0]], 
[[6.0, 2.0, 6.0], [3.0, 3.0, 1.0], [6.0, 1.0, 5.0]]], [[[8.0, 5.0, 8.0], [3.0, 8.0, 2.0], [5.0, 6.0, 3.0]], [[5.0, 5.0, 2.0], 
[2.0, 8.0, 1.0], [4.0, 5.0, 6.0]], [[6.0, 3.0, 7.0], [5.0, 4.0, 6.0], [3.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PCLN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 2.0], [8.0, 1.0, 2.0], [7.0, 7.0, 6.0]], [[4.0, 6.0, 8.0], [3.0, 8.0, 1.0], [7.0, 7.0, 
2.0]], [[6.0, 6.0, 2.0], [6.0, 3.0, 6.0], [4.0, 1.0, 5.0]]], [[[5.0, 2.0, 6.0], [6.0, 1.0, 5.0], [8.0, 7.0, 6.0]], [[6.0, 8.0, 
4.0], [8.0, 8.0, 6.0], [2.0, 1.0, 5.0]], [[4.0, 4.0, 8.0], [5.0, 2.0, 5.0], [2.0, 5.0, 6.0]]], [[[3.0, 5.0, 8.0], [6.0, 3.0, 
6.0], [3.0, 5.0, 4.0]], [[3.0, 1.0, 3.0], [6.0, 6.0, 2.0], [8.0, 5.0, 7.0]], [[5.0, 4.0, 3.0], [4.0, 2.0, 6.0], [4.0, 7.0, 
2.0]]], [[[6.0, 4.0, 4.0], [7.0, 3.0, 3.0], [8.0, 3.0, 6.0]], [[6.0, 4.0, 4.0], [4.0, 2.0, 4.0], [7.0, 1.0, 1.0]], [[4.0, 7.0, 
6.0], [3.0, 5.0, 4.0], [1.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 8.0], [2.0, 4.0, 6.0], [2.0, 1.0, 5.0]], [[5.0, 1.0, 3.0], [8.0, 2.0, 
3.0], [8.0, 6.0, 2.0]], [[2.0, 5.0, 7.0], [5.0, 4.0, 7.0], [8.0, 6.0, 8.0]]], [[[3.0, 5.0, 6.0], [2.0, 6.0, 6.0], [8.0, 6.0, 
7.0]], [[1.0, 3.0, 6.0], [8.0, 1.0, 3.0], [6.0, 6.0, 6.0]], [[2.0, 1.0, 4.0], [5.0, 3.0, 8.0], [3.0, 8.0, 1.0]]], [[[3.0, 4.0, 
2.0], [4.0, 5.0, 2.0], [6.0, 6.0, 8.0]], [[7.0, 7.0, 4.0], [6.0, 7.0, 4.0], [5.0, 6.0, 5.0]], [[3.0, 2.0, 2.0], [7.0, 2.0, 
7.0], [1.0, 1.0, 6.0]]], [[[3.0, 5.0, 7.0], [3.0, 8.0, 7.0], [3.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], [5.0, 5.0, 5.0], [3.0, 6.0, 
6.0]], [[1.0, 1.0, 3.0], [6.0, 4.0, 8.0], [8.0, 6.0, 6.0]]]]))
      val=numpy.array([[[2.0, 3.0, 7.0]], [[2.0, 3.0, 8.0]], [[5.0, 7.0, 6.0]]])
      arg[2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 2.0], [8.0, 1.0, 2.0], [7.0, 7.0, 6.0]], [[4.0, 6.0, 8.0], [3.0, 8.0, 1.0], [7.0, 
7.0, 2.0]], [[6.0, 6.0, 2.0], [6.0, 3.0, 6.0], [4.0, 1.0, 5.0]]], [[[5.0, 2.0, 6.0], [6.0, 1.0, 5.0], [8.0, 7.0, 6.0]], [[6.0, 
8.0, 4.0], [8.0, 8.0, 6.0], [2.0, 1.0, 5.0]], [[4.0, 4.0, 8.0], [5.0, 2.0, 5.0], [2.0, 5.0, 6.0]]], [[[3.0, 5.0, 8.0], [6.0, 
3.0, 6.0], [2.0, 3.0, 7.0]], [[3.0, 1.0, 3.0], [6.0, 6.0, 2.0], [2.0, 3.0, 8.0]], [[5.0, 4.0, 3.0], [4.0, 2.0, 6.0], [5.0, 7.0, 
6.0]]], [[[6.0, 4.0, 4.0], [7.0, 3.0, 3.0], [8.0, 3.0, 6.0]], [[6.0, 4.0, 4.0], [4.0, 2.0, 4.0], [7.0, 1.0, 1.0]], [[4.0, 7.0, 
6.0], [3.0, 5.0, 4.0], [1.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 8.0], [2.0, 4.0, 6.0], [2.0, 1.0, 5.0]], 
[[5.0, 1.0, 3.0], [8.0, 2.0, 3.0], [8.0, 6.0, 2.0]], [[2.0, 5.0, 7.0], [5.0, 4.0, 7.0], [8.0, 6.0, 8.0]]], [[[3.0, 5.0, 6.0], 
[2.0, 6.0, 6.0], [8.0, 6.0, 7.0]], [[1.0, 3.0, 6.0], [8.0, 1.0, 3.0], [6.0, 6.0, 6.0]], [[2.0, 1.0, 4.0], [5.0, 3.0, 8.0], 
[3.0, 8.0, 1.0]]], [[[3.0, 4.0, 2.0], [4.0, 5.0, 2.0], [2.0, 3.0, 7.0]], [[7.0, 7.0, 4.0], [6.0, 7.0, 4.0], [2.0, 3.0, 8.0]], 
[[3.0, 2.0, 2.0], [7.0, 2.0, 7.0], [5.0, 7.0, 6.0]]], [[[3.0, 5.0, 7.0], [3.0, 8.0, 7.0], [3.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], 
[5.0, 5.0, 5.0], [3.0, 6.0, 6.0]], [[1.0, 1.0, 3.0], [6.0, 4.0, 8.0], [8.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PCLN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 7.0], [7.0, 2.0, 5.0], [8.0, 4.0, 1.0]], [[1.0, 1.0, 8.0], [4.0, 6.0, 2.0], [6.0, 3.0, 
5.0]], [[6.0, 1.0, 1.0], [4.0, 6.0, 2.0], [8.0, 6.0, 8.0]]], [[[7.0, 4.0, 3.0], [7.0, 7.0, 8.0], [1.0, 4.0, 8.0]], [[8.0, 5.0, 
6.0], [1.0, 5.0, 6.0], [7.0, 1.0, 3.0]], [[1.0, 2.0, 1.0], [4.0, 1.0, 4.0], [7.0, 3.0, 4.0]]], [[[2.0, 3.0, 7.0], [7.0, 3.0, 
6.0], [2.0, 4.0, 1.0]], [[8.0, 5.0, 6.0], [3.0, 6.0, 5.0], [1.0, 4.0, 3.0]], [[4.0, 7.0, 3.0], [8.0, 3.0, 8.0], [6.0, 6.0, 
8.0]]], [[[8.0, 3.0, 4.0], [2.0, 5.0, 4.0], [1.0, 6.0, 5.0]], [[3.0, 3.0, 7.0], [7.0, 7.0, 4.0], [5.0, 1.0, 4.0]], [[2.0, 1.0, 
8.0], [4.0, 5.0, 7.0], [6.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 6.0], [3.0, 1.0, 4.0], [5.0, 1.0, 6.0]], [[3.0, 5.0, 7.0], [7.0, 5.0, 
7.0], [1.0, 2.0, 8.0]], [[1.0, 8.0, 5.0], [5.0, 4.0, 2.0], [8.0, 1.0, 2.0]]], [[[5.0, 4.0, 7.0], [8.0, 1.0, 2.0], [1.0, 6.0, 
8.0]], [[3.0, 5.0, 1.0], [1.0, 4.0, 6.0], [4.0, 5.0, 3.0]], [[6.0, 4.0, 7.0], [7.0, 5.0, 8.0], [4.0, 2.0, 7.0]]], [[[5.0, 7.0, 
8.0], [4.0, 8.0, 5.0], [3.0, 6.0, 7.0]], [[8.0, 1.0, 8.0], [3.0, 3.0, 4.0], [7.0, 3.0, 7.0]], [[1.0, 1.0, 3.0], [7.0, 4.0, 
5.0], [3.0, 1.0, 3.0]]], [[[3.0, 6.0, 2.0], [3.0, 6.0, 6.0], [3.0, 3.0, 1.0]], [[7.0, 6.0, 5.0], [7.0, 6.0, 4.0], [3.0, 1.0, 
3.0]], [[8.0, 8.0, 4.0], [5.0, 8.0, 4.0], [8.0, 8.0, 5.0]]]]))
      val=Data(numpy.array([[[8.0, 6.0, 8.0]], [[8.0, 6.0, 3.0]], [[2.0, 2.0, 7.0]]]),self.functionspace)
      arg[2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 7.0], [7.0, 2.0, 5.0], [8.0, 4.0, 1.0]], [[1.0, 1.0, 8.0], [4.0, 6.0, 2.0], [6.0, 
3.0, 5.0]], [[6.0, 1.0, 1.0], [4.0, 6.0, 2.0], [8.0, 6.0, 8.0]]], [[[7.0, 4.0, 3.0], [7.0, 7.0, 8.0], [1.0, 4.0, 8.0]], [[8.0, 
5.0, 6.0], [1.0, 5.0, 6.0], [7.0, 1.0, 3.0]], [[1.0, 2.0, 1.0], [4.0, 1.0, 4.0], [7.0, 3.0, 4.0]]], [[[2.0, 3.0, 7.0], [7.0, 
3.0, 6.0], [8.0, 6.0, 8.0]], [[8.0, 5.0, 6.0], [3.0, 6.0, 5.0], [8.0, 6.0, 3.0]], [[4.0, 7.0, 3.0], [8.0, 3.0, 8.0], [2.0, 2.0, 
7.0]]], [[[8.0, 3.0, 4.0], [2.0, 5.0, 4.0], [1.0, 6.0, 5.0]], [[3.0, 3.0, 7.0], [7.0, 7.0, 4.0], [5.0, 1.0, 4.0]], [[2.0, 1.0, 
8.0], [4.0, 5.0, 7.0], [6.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 6.0], [3.0, 1.0, 4.0], [5.0, 1.0, 6.0]], 
[[3.0, 5.0, 7.0], [7.0, 5.0, 7.0], [1.0, 2.0, 8.0]], [[1.0, 8.0, 5.0], [5.0, 4.0, 2.0], [8.0, 1.0, 2.0]]], [[[5.0, 4.0, 7.0], 
[8.0, 1.0, 2.0], [1.0, 6.0, 8.0]], [[3.0, 5.0, 1.0], [1.0, 4.0, 6.0], [4.0, 5.0, 3.0]], [[6.0, 4.0, 7.0], [7.0, 5.0, 8.0], 
[4.0, 2.0, 7.0]]], [[[5.0, 7.0, 8.0], [4.0, 8.0, 5.0], [8.0, 6.0, 8.0]], [[8.0, 1.0, 8.0], [3.0, 3.0, 4.0], [8.0, 6.0, 3.0]], 
[[1.0, 1.0, 3.0], [7.0, 4.0, 5.0], [2.0, 2.0, 7.0]]], [[[3.0, 6.0, 2.0], [3.0, 6.0, 6.0], [3.0, 3.0, 1.0]], [[7.0, 6.0, 5.0], 
[7.0, 6.0, 4.0], [3.0, 1.0, 3.0]], [[8.0, 8.0, 4.0], [5.0, 8.0, 4.0], [8.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PCLN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 6.0], [8.0, 1.0, 8.0], [2.0, 7.0, 6.0]], [[8.0, 1.0, 6.0], [1.0, 2.0, 5.0], [1.0, 7.0, 
6.0]], [[6.0, 4.0, 4.0], [5.0, 6.0, 4.0], [7.0, 5.0, 5.0]]], [[[3.0, 7.0, 1.0], [7.0, 1.0, 8.0], [7.0, 4.0, 3.0]], [[6.0, 6.0, 
2.0], [4.0, 7.0, 5.0], [6.0, 6.0, 3.0]], [[4.0, 7.0, 3.0], [1.0, 5.0, 8.0], [8.0, 5.0, 3.0]]], [[[5.0, 3.0, 4.0], [4.0, 2.0, 
7.0], [2.0, 8.0, 1.0]], [[5.0, 7.0, 2.0], [6.0, 8.0, 3.0], [6.0, 6.0, 7.0]], [[1.0, 1.0, 1.0], [5.0, 1.0, 4.0], [4.0, 6.0, 
3.0]]], [[[5.0, 4.0, 2.0], [7.0, 7.0, 4.0], [2.0, 7.0, 7.0]], [[8.0, 4.0, 4.0], [8.0, 7.0, 1.0], [1.0, 7.0, 2.0]], [[6.0, 8.0, 
4.0], [4.0, 8.0, 1.0], [7.0, 8.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 4.0], [1.0, 4.0, 3.0], [5.0, 5.0, 2.0]], [[4.0, 3.0, 7.0], [4.0, 3.0, 
5.0], [7.0, 4.0, 4.0]], [[3.0, 2.0, 3.0], [6.0, 5.0, 4.0], [3.0, 6.0, 5.0]]], [[[1.0, 6.0, 6.0], [3.0, 3.0, 1.0], [2.0, 3.0, 
4.0]], [[7.0, 5.0, 5.0], [1.0, 6.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 1.0, 5.0], [6.0, 3.0, 7.0], [6.0, 5.0, 1.0]]], [[[2.0, 6.0, 
2.0], [6.0, 4.0, 5.0], [3.0, 6.0, 7.0]], [[7.0, 4.0, 1.0], [5.0, 4.0, 1.0], [2.0, 6.0, 1.0]], [[2.0, 8.0, 2.0], [4.0, 4.0, 
8.0], [4.0, 8.0, 2.0]]], [[[6.0, 8.0, 2.0], [6.0, 8.0, 6.0], [8.0, 2.0, 6.0]], [[2.0, 1.0, 8.0], [4.0, 6.0, 3.0], [4.0, 7.0, 
1.0]], [[2.0, 3.0, 2.0], [4.0, 3.0, 4.0], [7.0, 5.0, 5.0]]]]))
      val=Data(numpy.array([[[4.0, 3.0, 6.0]], [[3.0, 8.0, 4.0]], [[3.0, 4.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 8.0, 2.0]], [[4.0, 6.0, 7.0]], [[5.0, 1.0, 6.0]]]))
      arg[2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 6.0], [8.0, 1.0, 8.0], [2.0, 7.0, 6.0]], [[8.0, 1.0, 6.0], [1.0, 2.0, 5.0], [1.0, 
7.0, 6.0]], [[6.0, 4.0, 4.0], [5.0, 6.0, 4.0], [7.0, 5.0, 5.0]]], [[[3.0, 7.0, 1.0], [7.0, 1.0, 8.0], [7.0, 4.0, 3.0]], [[6.0, 
6.0, 2.0], [4.0, 7.0, 5.0], [6.0, 6.0, 3.0]], [[4.0, 7.0, 3.0], [1.0, 5.0, 8.0], [8.0, 5.0, 3.0]]], [[[5.0, 3.0, 4.0], [4.0, 
2.0, 7.0], [4.0, 3.0, 6.0]], [[5.0, 7.0, 2.0], [6.0, 8.0, 3.0], [3.0, 8.0, 4.0]], [[1.0, 1.0, 1.0], [5.0, 1.0, 4.0], [3.0, 4.0, 
4.0]]], [[[5.0, 4.0, 2.0], [7.0, 7.0, 4.0], [2.0, 7.0, 7.0]], [[8.0, 4.0, 4.0], [8.0, 7.0, 1.0], [1.0, 7.0, 2.0]], [[6.0, 8.0, 
4.0], [4.0, 8.0, 1.0], [7.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 4.0], [1.0, 4.0, 3.0], [5.0, 5.0, 2.0]], 
[[4.0, 3.0, 7.0], [4.0, 3.0, 5.0], [7.0, 4.0, 4.0]], [[3.0, 2.0, 3.0], [6.0, 5.0, 4.0], [3.0, 6.0, 5.0]]], [[[1.0, 6.0, 6.0], 
[3.0, 3.0, 1.0], [2.0, 3.0, 4.0]], [[7.0, 5.0, 5.0], [1.0, 6.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 1.0, 5.0], [6.0, 3.0, 7.0], 
[6.0, 5.0, 1.0]]], [[[2.0, 6.0, 2.0], [6.0, 4.0, 5.0], [6.0, 8.0, 2.0]], [[7.0, 4.0, 1.0], [5.0, 4.0, 1.0], [4.0, 6.0, 7.0]], 
[[2.0, 8.0, 2.0], [4.0, 4.0, 8.0], [5.0, 1.0, 6.0]]], [[[6.0, 8.0, 2.0], [6.0, 8.0, 6.0], [8.0, 2.0, 6.0]], [[2.0, 1.0, 8.0], 
[4.0, 6.0, 3.0], [4.0, 7.0, 1.0]], [[2.0, 3.0, 2.0], [4.0, 3.0, 4.0], [7.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PCLN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 8.0], [3.0, 1.0, 7.0], [8.0, 1.0, 3.0]], [[4.0, 2.0, 8.0], [4.0, 7.0, 5.0], [5.0, 3.0, 
6.0]], [[6.0, 2.0, 7.0], [5.0, 8.0, 6.0], [3.0, 3.0, 4.0]]], [[[5.0, 7.0, 6.0], [7.0, 6.0, 7.0], [3.0, 4.0, 1.0]], [[8.0, 6.0, 
3.0], [4.0, 7.0, 6.0], [6.0, 1.0, 6.0]], [[2.0, 1.0, 5.0], [7.0, 4.0, 8.0], [5.0, 6.0, 8.0]]], [[[2.0, 7.0, 4.0], [6.0, 2.0, 
3.0], [2.0, 3.0, 3.0]], [[3.0, 1.0, 4.0], [8.0, 1.0, 8.0], [6.0, 6.0, 5.0]], [[7.0, 3.0, 2.0], [8.0, 8.0, 5.0], [8.0, 4.0, 
5.0]]], [[[2.0, 6.0, 7.0], [3.0, 1.0, 4.0], [7.0, 1.0, 8.0]], [[1.0, 4.0, 5.0], [5.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[8.0, 8.0, 
3.0], [5.0, 4.0, 8.0], [2.0, 5.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 4.0, 5.0], [4.0, 8.0, 2.0], [5.0, 6.0, 1.0]], [[2.0, 8.0, 6.0], [7.0, 7.0, 
8.0], [3.0, 1.0, 4.0]], [[5.0, 4.0, 4.0], [7.0, 8.0, 8.0], [3.0, 2.0, 5.0]]], [[[3.0, 5.0, 1.0], [1.0, 4.0, 3.0], [5.0, 5.0, 
5.0]], [[6.0, 7.0, 5.0], [2.0, 1.0, 8.0], [8.0, 1.0, 6.0]], [[8.0, 4.0, 2.0], [5.0, 1.0, 6.0], [8.0, 1.0, 6.0]]], [[[7.0, 4.0, 
1.0], [6.0, 6.0, 4.0], [6.0, 2.0, 2.0]], [[4.0, 5.0, 8.0], [5.0, 5.0, 5.0], [1.0, 3.0, 8.0]], [[7.0, 5.0, 8.0], [1.0, 1.0, 
6.0], [4.0, 5.0, 1.0]]], [[[1.0, 1.0, 6.0], [2.0, 3.0, 4.0], [7.0, 3.0, 5.0]], [[3.0, 7.0, 1.0], [1.0, 6.0, 7.0], [7.0, 8.0, 
6.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 6.0], [6.0, 3.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 8.0, 4.0]], [[2.0, 1.0, 8.0]], [[7.0, 2.0, 3.0]]])+(1.-msk_val)*numpy.array([[[2.0, 5.0, 
7.0]], [[8.0, 3.0, 6.0]], [[7.0, 7.0, 2.0]]])
      arg[2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 8.0], [3.0, 1.0, 7.0], [8.0, 1.0, 3.0]], [[4.0, 2.0, 8.0], [4.0, 7.0, 5.0], [5.0, 
3.0, 6.0]], [[6.0, 2.0, 7.0], [5.0, 8.0, 6.0], [3.0, 3.0, 4.0]]], [[[5.0, 7.0, 6.0], [7.0, 6.0, 7.0], [3.0, 4.0, 1.0]], [[8.0, 
6.0, 3.0], [4.0, 7.0, 6.0], [6.0, 1.0, 6.0]], [[2.0, 1.0, 5.0], [7.0, 4.0, 8.0], [5.0, 6.0, 8.0]]], [[[2.0, 7.0, 4.0], [6.0, 
2.0, 3.0], [7.0, 8.0, 4.0]], [[3.0, 1.0, 4.0], [8.0, 1.0, 8.0], [2.0, 1.0, 8.0]], [[7.0, 3.0, 2.0], [8.0, 8.0, 5.0], [7.0, 2.0, 
3.0]]], [[[2.0, 6.0, 7.0], [3.0, 1.0, 4.0], [7.0, 1.0, 8.0]], [[1.0, 4.0, 5.0], [5.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[8.0, 8.0, 
3.0], [5.0, 4.0, 8.0], [2.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 5.0], [4.0, 8.0, 2.0], [5.0, 6.0, 1.0]], 
[[2.0, 8.0, 6.0], [7.0, 7.0, 8.0], [3.0, 1.0, 4.0]], [[5.0, 4.0, 4.0], [7.0, 8.0, 8.0], [3.0, 2.0, 5.0]]], [[[3.0, 5.0, 1.0], 
[1.0, 4.0, 3.0], [5.0, 5.0, 5.0]], [[6.0, 7.0, 5.0], [2.0, 1.0, 8.0], [8.0, 1.0, 6.0]], [[8.0, 4.0, 2.0], [5.0, 1.0, 6.0], 
[8.0, 1.0, 6.0]]], [[[7.0, 4.0, 1.0], [6.0, 6.0, 4.0], [2.0, 5.0, 7.0]], [[4.0, 5.0, 8.0], [5.0, 5.0, 5.0], [8.0, 3.0, 6.0]], 
[[7.0, 5.0, 8.0], [1.0, 1.0, 6.0], [7.0, 7.0, 2.0]]], [[[1.0, 1.0, 6.0], [2.0, 3.0, 4.0], [7.0, 3.0, 5.0]], [[3.0, 7.0, 1.0], 
[1.0, 6.0, 7.0], [7.0, 8.0, 6.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 6.0], [6.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PLRN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 7.0], [4.0, 1.0, 7.0], [1.0, 4.0, 6.0]], [[8.0, 1.0, 3.0], [4.0, 4.0, 7.0], [3.0, 8.0, 
1.0]], [[6.0, 1.0, 8.0], [8.0, 6.0, 8.0], [8.0, 3.0, 5.0]]], [[[5.0, 3.0, 7.0], [2.0, 3.0, 1.0], [4.0, 3.0, 1.0]], [[5.0, 6.0, 
5.0], [7.0, 7.0, 3.0], [6.0, 8.0, 4.0]], [[8.0, 8.0, 5.0], [5.0, 5.0, 5.0], [3.0, 2.0, 4.0]]], [[[2.0, 5.0, 7.0], [3.0, 3.0, 
4.0], [3.0, 7.0, 4.0]], [[6.0, 1.0, 7.0], [3.0, 4.0, 8.0], [8.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [6.0, 3.0, 5.0], [4.0, 5.0, 
6.0]]], [[[5.0, 7.0, 4.0], [3.0, 2.0, 5.0], [1.0, 7.0, 8.0]], [[1.0, 1.0, 6.0], [1.0, 6.0, 5.0], [1.0, 1.0, 6.0]], [[1.0, 4.0, 
7.0], [7.0, 8.0, 8.0], [4.0, 4.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 1.0], [3.0, 2.0, 1.0], [2.0, 6.0, 7.0]], [[8.0, 7.0, 5.0], [4.0, 5.0, 
4.0], [7.0, 4.0, 3.0]], [[4.0, 1.0, 8.0], [7.0, 6.0, 8.0], [6.0, 8.0, 2.0]]], [[[4.0, 2.0, 2.0], [7.0, 1.0, 6.0], [7.0, 1.0, 
8.0]], [[4.0, 2.0, 6.0], [1.0, 6.0, 2.0], [4.0, 2.0, 6.0]], [[6.0, 5.0, 1.0], [5.0, 3.0, 4.0], [4.0, 4.0, 6.0]]], [[[7.0, 2.0, 
3.0], [6.0, 6.0, 4.0], [7.0, 2.0, 2.0]], [[8.0, 2.0, 4.0], [2.0, 7.0, 3.0], [3.0, 5.0, 4.0]], [[1.0, 5.0, 8.0], [3.0, 4.0, 
8.0], [2.0, 6.0, 3.0]]], [[[3.0, 1.0, 4.0], [2.0, 1.0, 4.0], [4.0, 2.0, 7.0]], [[6.0, 4.0, 8.0], [2.0, 8.0, 5.0], [7.0, 4.0, 
5.0]], [[1.0, 8.0, 7.0], [7.0, 5.0, 8.0], [7.0, 2.0, 3.0]]]]))
      val=numpy.array([[[6.0, 4.0, 4.0]]])
      arg[2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 7.0], [4.0, 1.0, 7.0], [1.0, 4.0, 6.0]], [[8.0, 1.0, 3.0], [4.0, 4.0, 7.0], [3.0, 
8.0, 1.0]], [[6.0, 1.0, 8.0], [8.0, 6.0, 8.0], [8.0, 3.0, 5.0]]], [[[5.0, 3.0, 7.0], [2.0, 3.0, 1.0], [4.0, 3.0, 1.0]], [[5.0, 
6.0, 5.0], [7.0, 7.0, 3.0], [6.0, 8.0, 4.0]], [[8.0, 8.0, 5.0], [5.0, 5.0, 5.0], [3.0, 2.0, 4.0]]], [[[2.0, 5.0, 7.0], [3.0, 
3.0, 4.0], [3.0, 7.0, 4.0]], [[6.0, 1.0, 7.0], [3.0, 4.0, 8.0], [8.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [6.0, 4.0, 4.0], [4.0, 5.0, 
6.0]]], [[[5.0, 7.0, 4.0], [3.0, 2.0, 5.0], [1.0, 7.0, 8.0]], [[1.0, 1.0, 6.0], [1.0, 6.0, 5.0], [1.0, 1.0, 6.0]], [[1.0, 4.0, 
7.0], [7.0, 8.0, 8.0], [4.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 1.0], [3.0, 2.0, 1.0], [2.0, 6.0, 7.0]], 
[[8.0, 7.0, 5.0], [4.0, 5.0, 4.0], [7.0, 4.0, 3.0]], [[4.0, 1.0, 8.0], [7.0, 6.0, 8.0], [6.0, 8.0, 2.0]]], [[[4.0, 2.0, 2.0], 
[7.0, 1.0, 6.0], [7.0, 1.0, 8.0]], [[4.0, 2.0, 6.0], [1.0, 6.0, 2.0], [4.0, 2.0, 6.0]], [[6.0, 5.0, 1.0], [5.0, 3.0, 4.0], 
[4.0, 4.0, 6.0]]], [[[7.0, 2.0, 3.0], [6.0, 6.0, 4.0], [7.0, 2.0, 2.0]], [[8.0, 2.0, 4.0], [2.0, 7.0, 3.0], [3.0, 5.0, 4.0]], 
[[1.0, 5.0, 8.0], [6.0, 4.0, 4.0], [2.0, 6.0, 3.0]]], [[[3.0, 1.0, 4.0], [2.0, 1.0, 4.0], [4.0, 2.0, 7.0]], [[6.0, 4.0, 8.0], 
[2.0, 8.0, 5.0], [7.0, 4.0, 5.0]], [[1.0, 8.0, 7.0], [7.0, 5.0, 8.0], [7.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PLRN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 6.0], [5.0, 4.0, 6.0], [6.0, 7.0, 7.0]], [[6.0, 2.0, 1.0], [2.0, 5.0, 2.0], [7.0, 6.0, 
1.0]], [[3.0, 5.0, 5.0], [2.0, 4.0, 1.0], [8.0, 1.0, 2.0]]], [[[1.0, 4.0, 7.0], [5.0, 3.0, 6.0], [6.0, 8.0, 1.0]], [[6.0, 6.0, 
2.0], [7.0, 2.0, 3.0], [4.0, 4.0, 4.0]], [[8.0, 6.0, 4.0], [8.0, 6.0, 8.0], [3.0, 6.0, 8.0]]], [[[1.0, 5.0, 3.0], [8.0, 3.0, 
3.0], [2.0, 5.0, 1.0]], [[5.0, 7.0, 2.0], [4.0, 5.0, 6.0], [5.0, 7.0, 4.0]], [[4.0, 5.0, 3.0], [4.0, 7.0, 8.0], [4.0, 2.0, 
8.0]]], [[[2.0, 5.0, 6.0], [4.0, 5.0, 2.0], [8.0, 7.0, 1.0]], [[2.0, 1.0, 4.0], [6.0, 7.0, 7.0], [3.0, 7.0, 3.0]], [[8.0, 6.0, 
3.0], [2.0, 1.0, 1.0], [8.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 4.0], [4.0, 6.0, 5.0], [8.0, 2.0, 8.0]], [[2.0, 5.0, 7.0], [2.0, 3.0, 
7.0], [7.0, 5.0, 1.0]], [[2.0, 4.0, 5.0], [7.0, 5.0, 6.0], [8.0, 1.0, 7.0]]], [[[8.0, 7.0, 4.0], [6.0, 2.0, 1.0], [6.0, 7.0, 
7.0]], [[5.0, 3.0, 2.0], [2.0, 1.0, 6.0], [2.0, 1.0, 4.0]], [[2.0, 8.0, 3.0], [5.0, 2.0, 3.0], [1.0, 3.0, 6.0]]], [[[2.0, 7.0, 
7.0], [3.0, 8.0, 6.0], [8.0, 7.0, 4.0]], [[1.0, 5.0, 8.0], [2.0, 1.0, 2.0], [7.0, 7.0, 5.0]], [[5.0, 5.0, 1.0], [2.0, 5.0, 
8.0], [1.0, 4.0, 4.0]]], [[[7.0, 5.0, 1.0], [7.0, 7.0, 3.0], [2.0, 3.0, 2.0]], [[7.0, 6.0, 3.0], [5.0, 7.0, 8.0], [1.0, 8.0, 
4.0]], [[2.0, 6.0, 3.0], [2.0, 3.0, 8.0], [3.0, 5.0, 1.0]]]]))
      val=Data(numpy.array([[[8.0, 7.0, 3.0]]]),self.functionspace)
      arg[2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 6.0], [5.0, 4.0, 6.0], [6.0, 7.0, 7.0]], [[6.0, 2.0, 1.0], [2.0, 5.0, 2.0], [7.0, 
6.0, 1.0]], [[3.0, 5.0, 5.0], [2.0, 4.0, 1.0], [8.0, 1.0, 2.0]]], [[[1.0, 4.0, 7.0], [5.0, 3.0, 6.0], [6.0, 8.0, 1.0]], [[6.0, 
6.0, 2.0], [7.0, 2.0, 3.0], [4.0, 4.0, 4.0]], [[8.0, 6.0, 4.0], [8.0, 6.0, 8.0], [3.0, 6.0, 8.0]]], [[[1.0, 5.0, 3.0], [8.0, 
3.0, 3.0], [2.0, 5.0, 1.0]], [[5.0, 7.0, 2.0], [4.0, 5.0, 6.0], [5.0, 7.0, 4.0]], [[4.0, 5.0, 3.0], [8.0, 7.0, 3.0], [4.0, 2.0, 
8.0]]], [[[2.0, 5.0, 6.0], [4.0, 5.0, 2.0], [8.0, 7.0, 1.0]], [[2.0, 1.0, 4.0], [6.0, 7.0, 7.0], [3.0, 7.0, 3.0]], [[8.0, 6.0, 
3.0], [2.0, 1.0, 1.0], [8.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 4.0], [4.0, 6.0, 5.0], [8.0, 2.0, 8.0]], 
[[2.0, 5.0, 7.0], [2.0, 3.0, 7.0], [7.0, 5.0, 1.0]], [[2.0, 4.0, 5.0], [7.0, 5.0, 6.0], [8.0, 1.0, 7.0]]], [[[8.0, 7.0, 4.0], 
[6.0, 2.0, 1.0], [6.0, 7.0, 7.0]], [[5.0, 3.0, 2.0], [2.0, 1.0, 6.0], [2.0, 1.0, 4.0]], [[2.0, 8.0, 3.0], [5.0, 2.0, 3.0], 
[1.0, 3.0, 6.0]]], [[[2.0, 7.0, 7.0], [3.0, 8.0, 6.0], [8.0, 7.0, 4.0]], [[1.0, 5.0, 8.0], [2.0, 1.0, 2.0], [7.0, 7.0, 5.0]], 
[[5.0, 5.0, 1.0], [8.0, 7.0, 3.0], [1.0, 4.0, 4.0]]], [[[7.0, 5.0, 1.0], [7.0, 7.0, 3.0], [2.0, 3.0, 2.0]], [[7.0, 6.0, 3.0], 
[5.0, 7.0, 8.0], [1.0, 8.0, 4.0]], [[2.0, 6.0, 3.0], [2.0, 3.0, 8.0], [3.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PLRN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 5.0], [8.0, 5.0, 6.0], [4.0, 8.0, 7.0]], [[1.0, 7.0, 6.0], [8.0, 2.0, 2.0], [8.0, 5.0, 
2.0]], [[2.0, 4.0, 7.0], [7.0, 4.0, 8.0], [7.0, 1.0, 4.0]]], [[[5.0, 2.0, 1.0], [2.0, 5.0, 2.0], [4.0, 1.0, 6.0]], [[5.0, 1.0, 
5.0], [2.0, 4.0, 6.0], [3.0, 3.0, 4.0]], [[7.0, 1.0, 3.0], [2.0, 2.0, 5.0], [2.0, 1.0, 6.0]]], [[[5.0, 7.0, 1.0], [8.0, 7.0, 
1.0], [3.0, 2.0, 5.0]], [[8.0, 6.0, 1.0], [1.0, 3.0, 7.0], [7.0, 6.0, 4.0]], [[8.0, 2.0, 2.0], [1.0, 6.0, 5.0], [2.0, 2.0, 
4.0]]], [[[7.0, 5.0, 3.0], [5.0, 8.0, 1.0], [4.0, 3.0, 8.0]], [[8.0, 6.0, 2.0], [7.0, 6.0, 8.0], [2.0, 3.0, 4.0]], [[8.0, 8.0, 
6.0], [4.0, 1.0, 6.0], [8.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 8.0], [3.0, 8.0, 8.0], [5.0, 3.0, 2.0]], [[8.0, 2.0, 8.0], [5.0, 5.0, 
4.0], [1.0, 2.0, 8.0]], [[7.0, 6.0, 3.0], [3.0, 7.0, 6.0], [8.0, 4.0, 5.0]]], [[[5.0, 3.0, 5.0], [3.0, 7.0, 4.0], [8.0, 6.0, 
1.0]], [[3.0, 3.0, 5.0], [8.0, 6.0, 8.0], [3.0, 3.0, 7.0]], [[5.0, 8.0, 4.0], [8.0, 6.0, 7.0], [3.0, 2.0, 3.0]]], [[[2.0, 5.0, 
8.0], [5.0, 3.0, 2.0], [8.0, 6.0, 7.0]], [[6.0, 5.0, 6.0], [1.0, 1.0, 8.0], [5.0, 7.0, 5.0]], [[5.0, 7.0, 4.0], [7.0, 2.0, 
8.0], [1.0, 3.0, 8.0]]], [[[5.0, 5.0, 8.0], [2.0, 6.0, 8.0], [3.0, 2.0, 2.0]], [[7.0, 1.0, 8.0], [3.0, 1.0, 6.0], [6.0, 3.0, 
4.0]], [[3.0, 6.0, 7.0], [5.0, 6.0, 8.0], [7.0, 5.0, 2.0]]]]))
      val=Data(numpy.array([[[2.0, 6.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 4.0, 4.0]]]))
      arg[2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 5.0], [8.0, 5.0, 6.0], [4.0, 8.0, 7.0]], [[1.0, 7.0, 6.0], [8.0, 2.0, 2.0], [8.0, 
5.0, 2.0]], [[2.0, 4.0, 7.0], [7.0, 4.0, 8.0], [7.0, 1.0, 4.0]]], [[[5.0, 2.0, 1.0], [2.0, 5.0, 2.0], [4.0, 1.0, 6.0]], [[5.0, 
1.0, 5.0], [2.0, 4.0, 6.0], [3.0, 3.0, 4.0]], [[7.0, 1.0, 3.0], [2.0, 2.0, 5.0], [2.0, 1.0, 6.0]]], [[[5.0, 7.0, 1.0], [8.0, 
7.0, 1.0], [3.0, 2.0, 5.0]], [[8.0, 6.0, 1.0], [1.0, 3.0, 7.0], [7.0, 6.0, 4.0]], [[8.0, 2.0, 2.0], [2.0, 6.0, 6.0], [2.0, 2.0, 
4.0]]], [[[7.0, 5.0, 3.0], [5.0, 8.0, 1.0], [4.0, 3.0, 8.0]], [[8.0, 6.0, 2.0], [7.0, 6.0, 8.0], [2.0, 3.0, 4.0]], [[8.0, 8.0, 
6.0], [4.0, 1.0, 6.0], [8.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 8.0], [3.0, 8.0, 8.0], [5.0, 3.0, 2.0]], 
[[8.0, 2.0, 8.0], [5.0, 5.0, 4.0], [1.0, 2.0, 8.0]], [[7.0, 6.0, 3.0], [3.0, 7.0, 6.0], [8.0, 4.0, 5.0]]], [[[5.0, 3.0, 5.0], 
[3.0, 7.0, 4.0], [8.0, 6.0, 1.0]], [[3.0, 3.0, 5.0], [8.0, 6.0, 8.0], [3.0, 3.0, 7.0]], [[5.0, 8.0, 4.0], [8.0, 6.0, 7.0], 
[3.0, 2.0, 3.0]]], [[[2.0, 5.0, 8.0], [5.0, 3.0, 2.0], [8.0, 6.0, 7.0]], [[6.0, 5.0, 6.0], [1.0, 1.0, 8.0], [5.0, 7.0, 5.0]], 
[[5.0, 7.0, 4.0], [7.0, 4.0, 4.0], [1.0, 3.0, 8.0]]], [[[5.0, 5.0, 8.0], [2.0, 6.0, 8.0], [3.0, 2.0, 2.0]], [[7.0, 1.0, 8.0], 
[3.0, 1.0, 6.0], [6.0, 3.0, 4.0]], [[3.0, 6.0, 7.0], [5.0, 6.0, 8.0], [7.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PLRN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 4.0], [6.0, 6.0, 8.0], [4.0, 5.0, 6.0]], [[8.0, 6.0, 3.0], [7.0, 6.0, 5.0], [2.0, 7.0, 
4.0]], [[2.0, 2.0, 1.0], [4.0, 8.0, 2.0], [1.0, 6.0, 7.0]]], [[[2.0, 8.0, 3.0], [8.0, 2.0, 7.0], [1.0, 1.0, 7.0]], [[8.0, 4.0, 
8.0], [4.0, 5.0, 4.0], [8.0, 2.0, 7.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 8.0], [8.0, 6.0, 3.0]]], [[[6.0, 2.0, 3.0], [6.0, 1.0, 
1.0], [2.0, 5.0, 6.0]], [[1.0, 6.0, 2.0], [2.0, 7.0, 4.0], [2.0, 2.0, 7.0]], [[2.0, 1.0, 3.0], [2.0, 6.0, 7.0], [8.0, 5.0, 
1.0]]], [[[1.0, 2.0, 3.0], [3.0, 6.0, 1.0], [3.0, 1.0, 5.0]], [[7.0, 7.0, 6.0], [6.0, 3.0, 2.0], [7.0, 1.0, 1.0]], [[8.0, 3.0, 
1.0], [2.0, 4.0, 8.0], [2.0, 8.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 4.0], [3.0, 1.0, 2.0], [3.0, 7.0, 1.0]], [[5.0, 1.0, 4.0], [2.0, 8.0, 
2.0], [1.0, 8.0, 5.0]], [[7.0, 1.0, 1.0], [8.0, 4.0, 8.0], [8.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], [2.0, 8.0, 3.0], [5.0, 2.0, 
3.0]], [[2.0, 1.0, 8.0], [3.0, 6.0, 2.0], [5.0, 4.0, 6.0]], [[5.0, 1.0, 7.0], [2.0, 3.0, 6.0], [7.0, 2.0, 7.0]]], [[[4.0, 3.0, 
1.0], [1.0, 2.0, 8.0], [4.0, 3.0, 2.0]], [[3.0, 5.0, 7.0], [1.0, 4.0, 4.0], [1.0, 5.0, 8.0]], [[6.0, 5.0, 8.0], [6.0, 6.0, 
1.0], [4.0, 2.0, 2.0]]], [[[5.0, 6.0, 7.0], [3.0, 1.0, 2.0], [4.0, 8.0, 5.0]], [[3.0, 7.0, 3.0], [7.0, 6.0, 5.0], [8.0, 2.0, 
5.0]], [[4.0, 2.0, 6.0], [2.0, 7.0, 4.0], [5.0, 6.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 7.0, 5.0]]])+(1.-msk_val)*numpy.array([[[3.0, 4.0, 3.0]]])
      arg[2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 4.0], [6.0, 6.0, 8.0], [4.0, 5.0, 6.0]], [[8.0, 6.0, 3.0], [7.0, 6.0, 5.0], [2.0, 
7.0, 4.0]], [[2.0, 2.0, 1.0], [4.0, 8.0, 2.0], [1.0, 6.0, 7.0]]], [[[2.0, 8.0, 3.0], [8.0, 2.0, 7.0], [1.0, 1.0, 7.0]], [[8.0, 
4.0, 8.0], [4.0, 5.0, 4.0], [8.0, 2.0, 7.0]], [[7.0, 7.0, 8.0], [1.0, 3.0, 8.0], [8.0, 6.0, 3.0]]], [[[6.0, 2.0, 3.0], [6.0, 
1.0, 1.0], [2.0, 5.0, 6.0]], [[1.0, 6.0, 2.0], [2.0, 7.0, 4.0], [2.0, 2.0, 7.0]], [[2.0, 1.0, 3.0], [7.0, 7.0, 5.0], [8.0, 5.0, 
1.0]]], [[[1.0, 2.0, 3.0], [3.0, 6.0, 1.0], [3.0, 1.0, 5.0]], [[7.0, 7.0, 6.0], [6.0, 3.0, 2.0], [7.0, 1.0, 1.0]], [[8.0, 3.0, 
1.0], [2.0, 4.0, 8.0], [2.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 4.0], [3.0, 1.0, 2.0], [3.0, 7.0, 1.0]], 
[[5.0, 1.0, 4.0], [2.0, 8.0, 2.0], [1.0, 8.0, 5.0]], [[7.0, 1.0, 1.0], [8.0, 4.0, 8.0], [8.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], 
[2.0, 8.0, 3.0], [5.0, 2.0, 3.0]], [[2.0, 1.0, 8.0], [3.0, 6.0, 2.0], [5.0, 4.0, 6.0]], [[5.0, 1.0, 7.0], [2.0, 3.0, 6.0], 
[7.0, 2.0, 7.0]]], [[[4.0, 3.0, 1.0], [1.0, 2.0, 8.0], [4.0, 3.0, 2.0]], [[3.0, 5.0, 7.0], [1.0, 4.0, 4.0], [1.0, 5.0, 8.0]], 
[[6.0, 5.0, 8.0], [3.0, 4.0, 3.0], [4.0, 2.0, 2.0]]], [[[5.0, 6.0, 7.0], [3.0, 1.0, 2.0], [4.0, 8.0, 5.0]], [[3.0, 7.0, 3.0], 
[7.0, 6.0, 5.0], [8.0, 2.0, 5.0]], [[4.0, 2.0, 6.0], [2.0, 7.0, 4.0], [5.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLPN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 3.0], [2.0, 4.0, 3.0], [7.0, 3.0, 4.0]], [[1.0, 4.0, 3.0], [1.0, 2.0, 4.0], [7.0, 6.0, 
6.0]], [[8.0, 4.0, 7.0], [1.0, 8.0, 6.0], [6.0, 6.0, 2.0]]], [[[5.0, 3.0, 8.0], [7.0, 4.0, 4.0], [3.0, 4.0, 8.0]], [[5.0, 3.0, 
5.0], [8.0, 2.0, 8.0], [5.0, 1.0, 6.0]], [[8.0, 5.0, 8.0], [1.0, 8.0, 1.0], [4.0, 6.0, 2.0]]], [[[6.0, 4.0, 6.0], [2.0, 5.0, 
6.0], [4.0, 8.0, 7.0]], [[4.0, 2.0, 1.0], [1.0, 6.0, 5.0], [2.0, 1.0, 2.0]], [[8.0, 2.0, 4.0], [7.0, 3.0, 8.0], [2.0, 6.0, 
5.0]]], [[[8.0, 2.0, 6.0], [8.0, 6.0, 3.0], [4.0, 3.0, 7.0]], [[3.0, 1.0, 3.0], [6.0, 5.0, 5.0], [3.0, 2.0, 1.0]], [[6.0, 5.0, 
5.0], [3.0, 1.0, 4.0], [1.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 4.0], [7.0, 3.0, 3.0], [4.0, 4.0, 3.0]], [[6.0, 1.0, 5.0], [2.0, 7.0, 
2.0], [7.0, 7.0, 6.0]], [[6.0, 7.0, 5.0], [6.0, 7.0, 2.0], [5.0, 4.0, 5.0]]], [[[2.0, 7.0, 8.0], [4.0, 7.0, 2.0], [3.0, 3.0, 
7.0]], [[1.0, 5.0, 7.0], [5.0, 6.0, 4.0], [8.0, 3.0, 4.0]], [[2.0, 5.0, 8.0], [2.0, 3.0, 4.0], [3.0, 8.0, 2.0]]], [[[4.0, 7.0, 
4.0], [6.0, 5.0, 5.0], [2.0, 5.0, 7.0]], [[5.0, 4.0, 3.0], [1.0, 3.0, 1.0], [7.0, 6.0, 3.0]], [[2.0, 7.0, 8.0], [1.0, 6.0, 
3.0], [6.0, 8.0, 4.0]]], [[[6.0, 4.0, 7.0], [1.0, 7.0, 5.0], [3.0, 4.0, 3.0]], [[7.0, 5.0, 1.0], [5.0, 1.0, 3.0], [2.0, 4.0, 
1.0]], [[2.0, 8.0, 2.0], [5.0, 4.0, 1.0], [2.0, 2.0, 1.0]]]]))
      val=numpy.array([[[7.0, 5.0, 7.0]], [[6.0, 5.0, 6.0]]])
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 3.0], [2.0, 4.0, 3.0], [7.0, 3.0, 4.0]], [[1.0, 4.0, 3.0], [1.0, 2.0, 4.0], [7.0, 
6.0, 6.0]], [[8.0, 4.0, 7.0], [1.0, 8.0, 6.0], [6.0, 6.0, 2.0]]], [[[5.0, 3.0, 8.0], [7.0, 4.0, 4.0], [3.0, 4.0, 8.0]], [[5.0, 
3.0, 5.0], [8.0, 2.0, 8.0], [5.0, 1.0, 6.0]], [[8.0, 5.0, 8.0], [1.0, 8.0, 1.0], [4.0, 6.0, 2.0]]], [[[6.0, 4.0, 6.0], [2.0, 
5.0, 6.0], [4.0, 8.0, 7.0]], [[4.0, 2.0, 1.0], [1.0, 6.0, 5.0], [2.0, 1.0, 2.0]], [[8.0, 2.0, 4.0], [7.0, 5.0, 7.0], [2.0, 6.0, 
5.0]]], [[[8.0, 2.0, 6.0], [8.0, 6.0, 3.0], [4.0, 3.0, 7.0]], [[3.0, 1.0, 3.0], [6.0, 5.0, 5.0], [3.0, 2.0, 1.0]], [[6.0, 5.0, 
5.0], [6.0, 5.0, 6.0], [1.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 4.0], [7.0, 3.0, 3.0], [4.0, 4.0, 3.0]], 
[[6.0, 1.0, 5.0], [2.0, 7.0, 2.0], [7.0, 7.0, 6.0]], [[6.0, 7.0, 5.0], [6.0, 7.0, 2.0], [5.0, 4.0, 5.0]]], [[[2.0, 7.0, 8.0], 
[4.0, 7.0, 2.0], [3.0, 3.0, 7.0]], [[1.0, 5.0, 7.0], [5.0, 6.0, 4.0], [8.0, 3.0, 4.0]], [[2.0, 5.0, 8.0], [2.0, 3.0, 4.0], 
[3.0, 8.0, 2.0]]], [[[4.0, 7.0, 4.0], [6.0, 5.0, 5.0], [2.0, 5.0, 7.0]], [[5.0, 4.0, 3.0], [1.0, 3.0, 1.0], [7.0, 6.0, 3.0]], 
[[2.0, 7.0, 8.0], [7.0, 5.0, 7.0], [6.0, 8.0, 4.0]]], [[[6.0, 4.0, 7.0], [1.0, 7.0, 5.0], [3.0, 4.0, 3.0]], [[7.0, 5.0, 1.0], 
[5.0, 1.0, 3.0], [2.0, 4.0, 1.0]], [[2.0, 8.0, 2.0], [6.0, 5.0, 6.0], [2.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLPN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 6.0], [5.0, 8.0, 7.0], [8.0, 2.0, 4.0]], [[6.0, 3.0, 2.0], [8.0, 6.0, 6.0], [4.0, 4.0, 
5.0]], [[7.0, 8.0, 3.0], [3.0, 8.0, 4.0], [7.0, 3.0, 6.0]]], [[[1.0, 1.0, 8.0], [8.0, 5.0, 4.0], [2.0, 4.0, 3.0]], [[6.0, 2.0, 
3.0], [6.0, 7.0, 5.0], [1.0, 2.0, 1.0]], [[1.0, 8.0, 7.0], [6.0, 6.0, 8.0], [2.0, 7.0, 6.0]]], [[[2.0, 7.0, 5.0], [6.0, 8.0, 
2.0], [6.0, 8.0, 2.0]], [[6.0, 5.0, 6.0], [4.0, 8.0, 8.0], [1.0, 4.0, 4.0]], [[4.0, 7.0, 7.0], [8.0, 8.0, 2.0], [4.0, 4.0, 
3.0]]], [[[7.0, 5.0, 8.0], [1.0, 4.0, 2.0], [8.0, 2.0, 8.0]], [[7.0, 3.0, 6.0], [3.0, 6.0, 8.0], [4.0, 5.0, 1.0]], [[8.0, 6.0, 
8.0], [7.0, 8.0, 6.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 6.0], [8.0, 3.0, 4.0], [7.0, 8.0, 6.0]], [[5.0, 8.0, 4.0], [8.0, 7.0, 
1.0], [3.0, 3.0, 1.0]], [[6.0, 5.0, 4.0], [8.0, 4.0, 1.0], [4.0, 1.0, 1.0]]], [[[3.0, 7.0, 4.0], [6.0, 4.0, 7.0], [4.0, 3.0, 
3.0]], [[2.0, 4.0, 4.0], [8.0, 7.0, 4.0], [2.0, 7.0, 4.0]], [[6.0, 7.0, 7.0], [7.0, 1.0, 5.0], [2.0, 8.0, 4.0]]], [[[8.0, 5.0, 
8.0], [7.0, 5.0, 2.0], [6.0, 4.0, 8.0]], [[4.0, 4.0, 3.0], [7.0, 1.0, 5.0], [6.0, 1.0, 1.0]], [[4.0, 1.0, 6.0], [3.0, 1.0, 
4.0], [5.0, 7.0, 6.0]]], [[[7.0, 2.0, 3.0], [7.0, 8.0, 7.0], [7.0, 5.0, 4.0]], [[7.0, 2.0, 3.0], [5.0, 1.0, 2.0], [2.0, 8.0, 
6.0]], [[1.0, 3.0, 1.0], [7.0, 6.0, 8.0], [2.0, 8.0, 7.0]]]]))
      val=Data(numpy.array([[[5.0, 3.0, 7.0]], [[1.0, 8.0, 4.0]]]),self.functionspace)
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 6.0], [5.0, 8.0, 7.0], [8.0, 2.0, 4.0]], [[6.0, 3.0, 2.0], [8.0, 6.0, 6.0], [4.0, 
4.0, 5.0]], [[7.0, 8.0, 3.0], [3.0, 8.0, 4.0], [7.0, 3.0, 6.0]]], [[[1.0, 1.0, 8.0], [8.0, 5.0, 4.0], [2.0, 4.0, 3.0]], [[6.0, 
2.0, 3.0], [6.0, 7.0, 5.0], [1.0, 2.0, 1.0]], [[1.0, 8.0, 7.0], [6.0, 6.0, 8.0], [2.0, 7.0, 6.0]]], [[[2.0, 7.0, 5.0], [6.0, 
8.0, 2.0], [6.0, 8.0, 2.0]], [[6.0, 5.0, 6.0], [4.0, 8.0, 8.0], [1.0, 4.0, 4.0]], [[4.0, 7.0, 7.0], [5.0, 3.0, 7.0], [4.0, 4.0, 
3.0]]], [[[7.0, 5.0, 8.0], [1.0, 4.0, 2.0], [8.0, 2.0, 8.0]], [[7.0, 3.0, 6.0], [3.0, 6.0, 8.0], [4.0, 5.0, 1.0]], [[8.0, 6.0, 
8.0], [1.0, 8.0, 4.0], [6.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 6.0], [8.0, 3.0, 4.0], [7.0, 8.0, 6.0]], 
[[5.0, 8.0, 4.0], [8.0, 7.0, 1.0], [3.0, 3.0, 1.0]], [[6.0, 5.0, 4.0], [8.0, 4.0, 1.0], [4.0, 1.0, 1.0]]], [[[3.0, 7.0, 4.0], 
[6.0, 4.0, 7.0], [4.0, 3.0, 3.0]], [[2.0, 4.0, 4.0], [8.0, 7.0, 4.0], [2.0, 7.0, 4.0]], [[6.0, 7.0, 7.0], [7.0, 1.0, 5.0], 
[2.0, 8.0, 4.0]]], [[[8.0, 5.0, 8.0], [7.0, 5.0, 2.0], [6.0, 4.0, 8.0]], [[4.0, 4.0, 3.0], [7.0, 1.0, 5.0], [6.0, 1.0, 1.0]], 
[[4.0, 1.0, 6.0], [5.0, 3.0, 7.0], [5.0, 7.0, 6.0]]], [[[7.0, 2.0, 3.0], [7.0, 8.0, 7.0], [7.0, 5.0, 4.0]], [[7.0, 2.0, 3.0], 
[5.0, 1.0, 2.0], [2.0, 8.0, 6.0]], [[1.0, 3.0, 1.0], [1.0, 8.0, 4.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLPN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 4.0], [7.0, 8.0, 8.0], [1.0, 5.0, 1.0]], [[8.0, 8.0, 5.0], [4.0, 4.0, 3.0], [5.0, 8.0, 
7.0]], [[4.0, 4.0, 2.0], [1.0, 8.0, 8.0], [1.0, 7.0, 2.0]]], [[[1.0, 8.0, 8.0], [8.0, 7.0, 7.0], [6.0, 8.0, 3.0]], [[6.0, 4.0, 
5.0], [8.0, 1.0, 1.0], [8.0, 1.0, 4.0]], [[1.0, 5.0, 3.0], [1.0, 2.0, 5.0], [4.0, 3.0, 5.0]]], [[[5.0, 2.0, 5.0], [8.0, 8.0, 
5.0], [6.0, 8.0, 2.0]], [[3.0, 4.0, 4.0], [1.0, 6.0, 8.0], [2.0, 4.0, 3.0]], [[5.0, 5.0, 8.0], [4.0, 4.0, 5.0], [1.0, 8.0, 
8.0]]], [[[4.0, 5.0, 8.0], [6.0, 2.0, 7.0], [5.0, 4.0, 1.0]], [[7.0, 3.0, 3.0], [8.0, 8.0, 8.0], [1.0, 1.0, 2.0]], [[2.0, 4.0, 
6.0], [1.0, 7.0, 8.0], [2.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 8.0], [2.0, 4.0, 8.0], [6.0, 8.0, 4.0]], [[6.0, 8.0, 8.0], [2.0, 5.0, 
6.0], [4.0, 6.0, 4.0]], [[6.0, 4.0, 8.0], [2.0, 4.0, 6.0], [3.0, 5.0, 7.0]]], [[[7.0, 1.0, 8.0], [3.0, 2.0, 7.0], [5.0, 3.0, 
1.0]], [[7.0, 1.0, 8.0], [4.0, 8.0, 1.0], [5.0, 2.0, 5.0]], [[2.0, 1.0, 4.0], [6.0, 5.0, 4.0], [8.0, 5.0, 2.0]]], [[[1.0, 1.0, 
5.0], [1.0, 6.0, 3.0], [8.0, 1.0, 8.0]], [[3.0, 3.0, 6.0], [3.0, 2.0, 8.0], [1.0, 8.0, 3.0]], [[8.0, 2.0, 6.0], [3.0, 5.0, 
5.0], [1.0, 3.0, 5.0]]], [[[7.0, 1.0, 8.0], [3.0, 4.0, 7.0], [7.0, 6.0, 6.0]], [[8.0, 3.0, 3.0], [7.0, 1.0, 7.0], [1.0, 7.0, 
2.0]], [[6.0, 5.0, 2.0], [1.0, 6.0, 2.0], [4.0, 3.0, 7.0]]]]))
      val=Data(numpy.array([[[5.0, 7.0, 3.0]], [[1.0, 4.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 5.0, 1.0]], [[5.0, 6.0, 4.0]]]))
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 4.0], [7.0, 8.0, 8.0], [1.0, 5.0, 1.0]], [[8.0, 8.0, 5.0], [4.0, 4.0, 3.0], [5.0, 
8.0, 7.0]], [[4.0, 4.0, 2.0], [1.0, 8.0, 8.0], [1.0, 7.0, 2.0]]], [[[1.0, 8.0, 8.0], [8.0, 7.0, 7.0], [6.0, 8.0, 3.0]], [[6.0, 
4.0, 5.0], [8.0, 1.0, 1.0], [8.0, 1.0, 4.0]], [[1.0, 5.0, 3.0], [1.0, 2.0, 5.0], [4.0, 3.0, 5.0]]], [[[5.0, 2.0, 5.0], [8.0, 
8.0, 5.0], [6.0, 8.0, 2.0]], [[3.0, 4.0, 4.0], [1.0, 6.0, 8.0], [2.0, 4.0, 3.0]], [[5.0, 5.0, 8.0], [5.0, 7.0, 3.0], [1.0, 8.0, 
8.0]]], [[[4.0, 5.0, 8.0], [6.0, 2.0, 7.0], [5.0, 4.0, 1.0]], [[7.0, 3.0, 3.0], [8.0, 8.0, 8.0], [1.0, 1.0, 2.0]], [[2.0, 4.0, 
6.0], [1.0, 4.0, 2.0], [2.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 8.0], [2.0, 4.0, 8.0], [6.0, 8.0, 4.0]], 
[[6.0, 8.0, 8.0], [2.0, 5.0, 6.0], [4.0, 6.0, 4.0]], [[6.0, 4.0, 8.0], [2.0, 4.0, 6.0], [3.0, 5.0, 7.0]]], [[[7.0, 1.0, 8.0], 
[3.0, 2.0, 7.0], [5.0, 3.0, 1.0]], [[7.0, 1.0, 8.0], [4.0, 8.0, 1.0], [5.0, 2.0, 5.0]], [[2.0, 1.0, 4.0], [6.0, 5.0, 4.0], 
[8.0, 5.0, 2.0]]], [[[1.0, 1.0, 5.0], [1.0, 6.0, 3.0], [8.0, 1.0, 8.0]], [[3.0, 3.0, 6.0], [3.0, 2.0, 8.0], [1.0, 8.0, 3.0]], 
[[8.0, 2.0, 6.0], [2.0, 5.0, 1.0], [1.0, 3.0, 5.0]]], [[[7.0, 1.0, 8.0], [3.0, 4.0, 7.0], [7.0, 6.0, 6.0]], [[8.0, 3.0, 3.0], 
[7.0, 1.0, 7.0], [1.0, 7.0, 2.0]], [[6.0, 5.0, 2.0], [5.0, 6.0, 4.0], [4.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLPN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 2.0], [6.0, 4.0, 8.0], [8.0, 7.0, 3.0]], [[6.0, 1.0, 2.0], [3.0, 1.0, 5.0], [7.0, 1.0, 
7.0]], [[5.0, 8.0, 2.0], [4.0, 7.0, 3.0], [2.0, 7.0, 2.0]]], [[[3.0, 4.0, 8.0], [7.0, 1.0, 3.0], [8.0, 8.0, 2.0]], [[7.0, 7.0, 
5.0], [5.0, 8.0, 8.0], [7.0, 8.0, 2.0]], [[7.0, 3.0, 1.0], [7.0, 3.0, 7.0], [2.0, 6.0, 5.0]]], [[[3.0, 2.0, 4.0], [3.0, 2.0, 
4.0], [4.0, 1.0, 8.0]], [[7.0, 1.0, 4.0], [7.0, 2.0, 8.0], [4.0, 3.0, 8.0]], [[6.0, 4.0, 3.0], [1.0, 7.0, 1.0], [4.0, 5.0, 
4.0]]], [[[7.0, 2.0, 7.0], [4.0, 6.0, 3.0], [2.0, 1.0, 3.0]], [[6.0, 8.0, 2.0], [6.0, 6.0, 1.0], [3.0, 4.0, 8.0]], [[7.0, 4.0, 
7.0], [5.0, 8.0, 8.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 8.0], [1.0, 7.0, 7.0], [5.0, 8.0, 8.0]], [[5.0, 5.0, 8.0], [4.0, 3.0, 
8.0], [2.0, 3.0, 1.0]], [[2.0, 4.0, 1.0], [3.0, 4.0, 1.0], [5.0, 3.0, 5.0]]], [[[8.0, 8.0, 4.0], [2.0, 1.0, 8.0], [1.0, 5.0, 
6.0]], [[2.0, 6.0, 8.0], [4.0, 7.0, 3.0], [4.0, 4.0, 6.0]], [[7.0, 2.0, 1.0], [1.0, 4.0, 4.0], [8.0, 7.0, 1.0]]], [[[3.0, 4.0, 
3.0], [6.0, 2.0, 6.0], [5.0, 6.0, 4.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 2.0], [2.0, 7.0, 5.0]], [[4.0, 4.0, 1.0], [7.0, 4.0, 
4.0], [8.0, 7.0, 1.0]]], [[[2.0, 7.0, 8.0], [5.0, 8.0, 1.0], [5.0, 6.0, 5.0]], [[5.0, 1.0, 4.0], [4.0, 3.0, 2.0], [6.0, 2.0, 
6.0]], [[4.0, 1.0, 6.0], [3.0, 5.0, 4.0], [2.0, 3.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 3.0, 5.0]], [[4.0, 8.0, 1.0]]])+(1.-msk_val)*numpy.array([[[5.0, 1.0, 3.0]], [[5.0, 7.0, 
7.0]]])
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 2.0], [6.0, 4.0, 8.0], [8.0, 7.0, 3.0]], [[6.0, 1.0, 2.0], [3.0, 1.0, 5.0], [7.0, 
1.0, 7.0]], [[5.0, 8.0, 2.0], [4.0, 7.0, 3.0], [2.0, 7.0, 2.0]]], [[[3.0, 4.0, 8.0], [7.0, 1.0, 3.0], [8.0, 8.0, 2.0]], [[7.0, 
7.0, 5.0], [5.0, 8.0, 8.0], [7.0, 8.0, 2.0]], [[7.0, 3.0, 1.0], [7.0, 3.0, 7.0], [2.0, 6.0, 5.0]]], [[[3.0, 2.0, 4.0], [3.0, 
2.0, 4.0], [4.0, 1.0, 8.0]], [[7.0, 1.0, 4.0], [7.0, 2.0, 8.0], [4.0, 3.0, 8.0]], [[6.0, 4.0, 3.0], [8.0, 3.0, 5.0], [4.0, 5.0, 
4.0]]], [[[7.0, 2.0, 7.0], [4.0, 6.0, 3.0], [2.0, 1.0, 3.0]], [[6.0, 8.0, 2.0], [6.0, 6.0, 1.0], [3.0, 4.0, 8.0]], [[7.0, 4.0, 
7.0], [4.0, 8.0, 1.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 8.0], [1.0, 7.0, 7.0], [5.0, 8.0, 8.0]], 
[[5.0, 5.0, 8.0], [4.0, 3.0, 8.0], [2.0, 3.0, 1.0]], [[2.0, 4.0, 1.0], [3.0, 4.0, 1.0], [5.0, 3.0, 5.0]]], [[[8.0, 8.0, 4.0], 
[2.0, 1.0, 8.0], [1.0, 5.0, 6.0]], [[2.0, 6.0, 8.0], [4.0, 7.0, 3.0], [4.0, 4.0, 6.0]], [[7.0, 2.0, 1.0], [1.0, 4.0, 4.0], 
[8.0, 7.0, 1.0]]], [[[3.0, 4.0, 3.0], [6.0, 2.0, 6.0], [5.0, 6.0, 4.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 2.0], [2.0, 7.0, 5.0]], 
[[4.0, 4.0, 1.0], [5.0, 1.0, 3.0], [8.0, 7.0, 1.0]]], [[[2.0, 7.0, 8.0], [5.0, 8.0, 1.0], [5.0, 6.0, 5.0]], [[5.0, 1.0, 4.0], 
[4.0, 3.0, 2.0], [6.0, 2.0, 6.0]], [[4.0, 1.0, 6.0], [5.0, 7.0, 7.0], [2.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UPRN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 5.0], [3.0, 1.0, 4.0], [3.0, 7.0, 7.0]], [[8.0, 4.0, 4.0], [6.0, 5.0, 8.0], [5.0, 2.0, 
3.0]], [[7.0, 1.0, 4.0], [4.0, 3.0, 1.0], [1.0, 6.0, 4.0]]], [[[2.0, 8.0, 1.0], [1.0, 7.0, 8.0], [4.0, 3.0, 2.0]], [[3.0, 7.0, 
4.0], [6.0, 7.0, 3.0], [3.0, 2.0, 7.0]], [[7.0, 4.0, 4.0], [5.0, 7.0, 4.0], [7.0, 5.0, 4.0]]], [[[2.0, 8.0, 7.0], [7.0, 5.0, 
3.0], [2.0, 4.0, 3.0]], [[6.0, 8.0, 8.0], [5.0, 8.0, 1.0], [4.0, 6.0, 2.0]], [[4.0, 8.0, 8.0], [7.0, 2.0, 3.0], [3.0, 2.0, 
5.0]]], [[[6.0, 6.0, 7.0], [8.0, 1.0, 5.0], [4.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], [4.0, 6.0, 6.0], [1.0, 3.0, 2.0]], [[6.0, 6.0, 
2.0], [3.0, 5.0, 6.0], [2.0, 7.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 8.0], [7.0, 8.0, 2.0], [5.0, 4.0, 4.0]], [[8.0, 1.0, 8.0], [1.0, 1.0, 
8.0], [3.0, 1.0, 7.0]], [[8.0, 4.0, 5.0], [6.0, 1.0, 2.0], [3.0, 7.0, 6.0]]], [[[7.0, 5.0, 7.0], [8.0, 3.0, 6.0], [4.0, 5.0, 
7.0]], [[2.0, 1.0, 2.0], [1.0, 6.0, 5.0], [5.0, 5.0, 5.0]], [[4.0, 3.0, 2.0], [2.0, 4.0, 8.0], [5.0, 2.0, 7.0]]], [[[1.0, 8.0, 
6.0], [4.0, 6.0, 6.0], [2.0, 2.0, 3.0]], [[1.0, 4.0, 1.0], [6.0, 4.0, 2.0], [7.0, 1.0, 2.0]], [[5.0, 3.0, 8.0], [2.0, 2.0, 
3.0], [2.0, 1.0, 4.0]]], [[[6.0, 3.0, 4.0], [8.0, 5.0, 2.0], [7.0, 4.0, 7.0]], [[4.0, 6.0, 5.0], [8.0, 3.0, 7.0], [3.0, 1.0, 
2.0]], [[5.0, 1.0, 3.0], [4.0, 4.0, 6.0], [3.0, 3.0, 5.0]]]]))
      val=numpy.array([[[8.0, 8.0, 2.0]], [[5.0, 4.0, 2.0]], [[6.0, 1.0, 6.0]]])
      arg[:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 5.0], [3.0, 1.0, 4.0], [3.0, 7.0, 7.0]], [[8.0, 4.0, 4.0], [8.0, 8.0, 2.0], [5.0, 
2.0, 3.0]], [[7.0, 1.0, 4.0], [4.0, 3.0, 1.0], [1.0, 6.0, 4.0]]], [[[2.0, 8.0, 1.0], [1.0, 7.0, 8.0], [4.0, 3.0, 2.0]], [[3.0, 
7.0, 4.0], [5.0, 4.0, 2.0], [3.0, 2.0, 7.0]], [[7.0, 4.0, 4.0], [5.0, 7.0, 4.0], [7.0, 5.0, 4.0]]], [[[2.0, 8.0, 7.0], [7.0, 
5.0, 3.0], [2.0, 4.0, 3.0]], [[6.0, 8.0, 8.0], [6.0, 1.0, 6.0], [4.0, 6.0, 2.0]], [[4.0, 8.0, 8.0], [7.0, 2.0, 3.0], [3.0, 2.0, 
5.0]]], [[[6.0, 6.0, 7.0], [8.0, 1.0, 5.0], [4.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], [4.0, 6.0, 6.0], [1.0, 3.0, 2.0]], [[6.0, 6.0, 
2.0], [3.0, 5.0, 6.0], [2.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 8.0], [7.0, 8.0, 2.0], [5.0, 4.0, 4.0]], 
[[8.0, 1.0, 8.0], [8.0, 8.0, 2.0], [3.0, 1.0, 7.0]], [[8.0, 4.0, 5.0], [6.0, 1.0, 2.0], [3.0, 7.0, 6.0]]], [[[7.0, 5.0, 7.0], 
[8.0, 3.0, 6.0], [4.0, 5.0, 7.0]], [[2.0, 1.0, 2.0], [5.0, 4.0, 2.0], [5.0, 5.0, 5.0]], [[4.0, 3.0, 2.0], [2.0, 4.0, 8.0], 
[5.0, 2.0, 7.0]]], [[[1.0, 8.0, 6.0], [4.0, 6.0, 6.0], [2.0, 2.0, 3.0]], [[1.0, 4.0, 1.0], [6.0, 1.0, 6.0], [7.0, 1.0, 2.0]], 
[[5.0, 3.0, 8.0], [2.0, 2.0, 3.0], [2.0, 1.0, 4.0]]], [[[6.0, 3.0, 4.0], [8.0, 5.0, 2.0], [7.0, 4.0, 7.0]], [[4.0, 6.0, 5.0], 
[8.0, 3.0, 7.0], [3.0, 1.0, 2.0]], [[5.0, 1.0, 3.0], [4.0, 4.0, 6.0], [3.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UPRN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 4.0], [2.0, 6.0, 5.0], [5.0, 4.0, 8.0]], [[6.0, 2.0, 1.0], [4.0, 8.0, 6.0], [3.0, 3.0, 
2.0]], [[4.0, 7.0, 6.0], [7.0, 8.0, 4.0], [8.0, 3.0, 5.0]]], [[[1.0, 7.0, 5.0], [4.0, 8.0, 3.0], [5.0, 2.0, 3.0]], [[1.0, 7.0, 
2.0], [5.0, 6.0, 2.0], [1.0, 5.0, 5.0]], [[1.0, 3.0, 1.0], [4.0, 8.0, 2.0], [7.0, 7.0, 1.0]]], [[[4.0, 1.0, 6.0], [5.0, 2.0, 
5.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 1.0], [7.0, 6.0, 3.0], [2.0, 6.0, 1.0]], [[5.0, 4.0, 4.0], [1.0, 7.0, 3.0], [2.0, 1.0, 
6.0]]], [[[3.0, 7.0, 4.0], [3.0, 4.0, 1.0], [2.0, 5.0, 1.0]], [[7.0, 6.0, 1.0], [4.0, 8.0, 2.0], [6.0, 7.0, 8.0]], [[3.0, 8.0, 
5.0], [5.0, 7.0, 6.0], [8.0, 2.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 4.0], [7.0, 1.0, 4.0], [5.0, 4.0, 7.0]], [[8.0, 1.0, 8.0], [4.0, 4.0, 
8.0], [2.0, 6.0, 3.0]], [[5.0, 5.0, 3.0], [6.0, 7.0, 5.0], [2.0, 1.0, 7.0]]], [[[6.0, 7.0, 8.0], [8.0, 3.0, 8.0], [7.0, 4.0, 
3.0]], [[8.0, 5.0, 6.0], [1.0, 7.0, 1.0], [3.0, 8.0, 3.0]], [[6.0, 6.0, 5.0], [6.0, 6.0, 3.0], [3.0, 4.0, 4.0]]], [[[5.0, 4.0, 
1.0], [6.0, 4.0, 1.0], [2.0, 7.0, 5.0]], [[2.0, 7.0, 1.0], [4.0, 6.0, 2.0], [6.0, 1.0, 6.0]], [[6.0, 6.0, 3.0], [6.0, 3.0, 
4.0], [5.0, 7.0, 2.0]]], [[[7.0, 7.0, 7.0], [2.0, 8.0, 4.0], [5.0, 4.0, 4.0]], [[6.0, 3.0, 7.0], [7.0, 4.0, 5.0], [7.0, 6.0, 
7.0]], [[7.0, 2.0, 7.0], [8.0, 6.0, 5.0], [1.0, 6.0, 6.0]]]]))
      val=Data(numpy.array([[[4.0, 2.0, 2.0]], [[8.0, 1.0, 2.0]], [[5.0, 3.0, 8.0]]]),self.functionspace)
      arg[:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 4.0], [2.0, 6.0, 5.0], [5.0, 4.0, 8.0]], [[6.0, 2.0, 1.0], [4.0, 2.0, 2.0], [3.0, 
3.0, 2.0]], [[4.0, 7.0, 6.0], [7.0, 8.0, 4.0], [8.0, 3.0, 5.0]]], [[[1.0, 7.0, 5.0], [4.0, 8.0, 3.0], [5.0, 2.0, 3.0]], [[1.0, 
7.0, 2.0], [8.0, 1.0, 2.0], [1.0, 5.0, 5.0]], [[1.0, 3.0, 1.0], [4.0, 8.0, 2.0], [7.0, 7.0, 1.0]]], [[[4.0, 1.0, 6.0], [5.0, 
2.0, 5.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 1.0], [5.0, 3.0, 8.0], [2.0, 6.0, 1.0]], [[5.0, 4.0, 4.0], [1.0, 7.0, 3.0], [2.0, 1.0, 
6.0]]], [[[3.0, 7.0, 4.0], [3.0, 4.0, 1.0], [2.0, 5.0, 1.0]], [[7.0, 6.0, 1.0], [4.0, 8.0, 2.0], [6.0, 7.0, 8.0]], [[3.0, 8.0, 
5.0], [5.0, 7.0, 6.0], [8.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 4.0], [7.0, 1.0, 4.0], [5.0, 4.0, 7.0]], 
[[8.0, 1.0, 8.0], [4.0, 2.0, 2.0], [2.0, 6.0, 3.0]], [[5.0, 5.0, 3.0], [6.0, 7.0, 5.0], [2.0, 1.0, 7.0]]], [[[6.0, 7.0, 8.0], 
[8.0, 3.0, 8.0], [7.0, 4.0, 3.0]], [[8.0, 5.0, 6.0], [8.0, 1.0, 2.0], [3.0, 8.0, 3.0]], [[6.0, 6.0, 5.0], [6.0, 6.0, 3.0], 
[3.0, 4.0, 4.0]]], [[[5.0, 4.0, 1.0], [6.0, 4.0, 1.0], [2.0, 7.0, 5.0]], [[2.0, 7.0, 1.0], [5.0, 3.0, 8.0], [6.0, 1.0, 6.0]], 
[[6.0, 6.0, 3.0], [6.0, 3.0, 4.0], [5.0, 7.0, 2.0]]], [[[7.0, 7.0, 7.0], [2.0, 8.0, 4.0], [5.0, 4.0, 4.0]], [[6.0, 3.0, 7.0], 
[7.0, 4.0, 5.0], [7.0, 6.0, 7.0]], [[7.0, 2.0, 7.0], [8.0, 6.0, 5.0], [1.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UPRN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 3.0], [6.0, 5.0, 6.0], [8.0, 6.0, 5.0]], [[6.0, 4.0, 5.0], [7.0, 8.0, 1.0], [1.0, 2.0, 
7.0]], [[5.0, 2.0, 8.0], [8.0, 7.0, 8.0], [6.0, 4.0, 3.0]]], [[[7.0, 4.0, 8.0], [5.0, 4.0, 5.0], [6.0, 3.0, 8.0]], [[1.0, 5.0, 
6.0], [3.0, 2.0, 3.0], [3.0, 5.0, 2.0]], [[3.0, 4.0, 5.0], [4.0, 8.0, 1.0], [7.0, 8.0, 4.0]]], [[[6.0, 1.0, 2.0], [7.0, 5.0, 
2.0], [8.0, 1.0, 5.0]], [[5.0, 4.0, 4.0], [1.0, 6.0, 2.0], [8.0, 2.0, 7.0]], [[8.0, 5.0, 4.0], [3.0, 2.0, 7.0], [6.0, 4.0, 
8.0]]], [[[1.0, 4.0, 2.0], [1.0, 5.0, 4.0], [1.0, 5.0, 7.0]], [[8.0, 5.0, 3.0], [6.0, 8.0, 8.0], [7.0, 7.0, 7.0]], [[8.0, 5.0, 
4.0], [4.0, 8.0, 6.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 4.0], [5.0, 4.0, 6.0], [1.0, 4.0, 2.0]], [[6.0, 5.0, 8.0], [3.0, 2.0, 
3.0], [8.0, 3.0, 1.0]], [[2.0, 2.0, 4.0], [5.0, 5.0, 3.0], [1.0, 8.0, 8.0]]], [[[7.0, 6.0, 3.0], [7.0, 5.0, 7.0], [6.0, 8.0, 
6.0]], [[7.0, 2.0, 4.0], [5.0, 6.0, 8.0], [5.0, 7.0, 6.0]], [[1.0, 2.0, 8.0], [3.0, 3.0, 8.0], [2.0, 8.0, 6.0]]], [[[1.0, 6.0, 
5.0], [2.0, 8.0, 4.0], [6.0, 1.0, 6.0]], [[2.0, 7.0, 3.0], [8.0, 5.0, 3.0], [1.0, 1.0, 1.0]], [[6.0, 1.0, 4.0], [2.0, 1.0, 
7.0], [2.0, 7.0, 7.0]]], [[[2.0, 1.0, 5.0], [4.0, 2.0, 1.0], [7.0, 5.0, 2.0]], [[6.0, 4.0, 3.0], [5.0, 8.0, 1.0], [6.0, 8.0, 
7.0]], [[1.0, 4.0, 3.0], [5.0, 5.0, 3.0], [2.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[[5.0, 6.0, 5.0]], [[1.0, 6.0, 7.0]], [[5.0, 7.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 3.0, 7.0]], [[6.0, 1.0, 2.0]], [[1.0, 5.0, 2.0]]]))
      arg[:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 3.0], [6.0, 5.0, 6.0], [8.0, 6.0, 5.0]], [[6.0, 4.0, 5.0], [5.0, 6.0, 5.0], [1.0, 
2.0, 7.0]], [[5.0, 2.0, 8.0], [8.0, 7.0, 8.0], [6.0, 4.0, 3.0]]], [[[7.0, 4.0, 8.0], [5.0, 4.0, 5.0], [6.0, 3.0, 8.0]], [[1.0, 
5.0, 6.0], [1.0, 6.0, 7.0], [3.0, 5.0, 2.0]], [[3.0, 4.0, 5.0], [4.0, 8.0, 1.0], [7.0, 8.0, 4.0]]], [[[6.0, 1.0, 2.0], [7.0, 
5.0, 2.0], [8.0, 1.0, 5.0]], [[5.0, 4.0, 4.0], [5.0, 7.0, 5.0], [8.0, 2.0, 7.0]], [[8.0, 5.0, 4.0], [3.0, 2.0, 7.0], [6.0, 4.0, 
8.0]]], [[[1.0, 4.0, 2.0], [1.0, 5.0, 4.0], [1.0, 5.0, 7.0]], [[8.0, 5.0, 3.0], [6.0, 8.0, 8.0], [7.0, 7.0, 7.0]], [[8.0, 5.0, 
4.0], [4.0, 8.0, 6.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 4.0], [5.0, 4.0, 6.0], [1.0, 4.0, 2.0]], 
[[6.0, 5.0, 8.0], [4.0, 3.0, 7.0], [8.0, 3.0, 1.0]], [[2.0, 2.0, 4.0], [5.0, 5.0, 3.0], [1.0, 8.0, 8.0]]], [[[7.0, 6.0, 3.0], 
[7.0, 5.0, 7.0], [6.0, 8.0, 6.0]], [[7.0, 2.0, 4.0], [6.0, 1.0, 2.0], [5.0, 7.0, 6.0]], [[1.0, 2.0, 8.0], [3.0, 3.0, 8.0], 
[2.0, 8.0, 6.0]]], [[[1.0, 6.0, 5.0], [2.0, 8.0, 4.0], [6.0, 1.0, 6.0]], [[2.0, 7.0, 3.0], [1.0, 5.0, 2.0], [1.0, 1.0, 1.0]], 
[[6.0, 1.0, 4.0], [2.0, 1.0, 7.0], [2.0, 7.0, 7.0]]], [[[2.0, 1.0, 5.0], [4.0, 2.0, 1.0], [7.0, 5.0, 2.0]], [[6.0, 4.0, 3.0], 
[5.0, 8.0, 1.0], [6.0, 8.0, 7.0]], [[1.0, 4.0, 3.0], [5.0, 5.0, 3.0], [2.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UPRN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 7.0], [5.0, 2.0, 3.0], [1.0, 8.0, 1.0]], [[4.0, 5.0, 7.0], [3.0, 7.0, 4.0], [5.0, 3.0, 
1.0]], [[2.0, 6.0, 8.0], [6.0, 3.0, 2.0], [6.0, 6.0, 5.0]]], [[[8.0, 4.0, 5.0], [3.0, 4.0, 2.0], [4.0, 6.0, 6.0]], [[2.0, 4.0, 
1.0], [2.0, 6.0, 5.0], [8.0, 3.0, 8.0]], [[5.0, 4.0, 2.0], [6.0, 5.0, 7.0], [2.0, 3.0, 5.0]]], [[[4.0, 1.0, 1.0], [7.0, 8.0, 
2.0], [8.0, 8.0, 6.0]], [[4.0, 5.0, 7.0], [3.0, 8.0, 7.0], [8.0, 3.0, 8.0]], [[4.0, 3.0, 1.0], [5.0, 7.0, 7.0], [4.0, 4.0, 
8.0]]], [[[4.0, 1.0, 6.0], [8.0, 2.0, 6.0], [5.0, 4.0, 2.0]], [[4.0, 4.0, 1.0], [3.0, 5.0, 7.0], [4.0, 1.0, 3.0]], [[1.0, 6.0, 
4.0], [5.0, 3.0, 3.0], [3.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 8.0], [7.0, 3.0, 7.0], [5.0, 1.0, 8.0]], [[6.0, 3.0, 3.0], [5.0, 3.0, 
2.0], [7.0, 4.0, 2.0]], [[6.0, 5.0, 3.0], [4.0, 3.0, 8.0], [4.0, 4.0, 5.0]]], [[[2.0, 4.0, 3.0], [1.0, 6.0, 3.0], [3.0, 4.0, 
1.0]], [[7.0, 6.0, 4.0], [7.0, 5.0, 4.0], [8.0, 6.0, 7.0]], [[2.0, 3.0, 3.0], [4.0, 2.0, 2.0], [4.0, 2.0, 1.0]]], [[[7.0, 4.0, 
4.0], [7.0, 5.0, 5.0], [7.0, 8.0, 5.0]], [[1.0, 1.0, 7.0], [1.0, 8.0, 4.0], [8.0, 5.0, 7.0]], [[6.0, 1.0, 1.0], [7.0, 5.0, 
8.0], [6.0, 5.0, 1.0]]], [[[3.0, 2.0, 1.0], [3.0, 3.0, 4.0], [1.0, 8.0, 3.0]], [[1.0, 3.0, 2.0], [1.0, 4.0, 8.0], [5.0, 5.0, 
1.0]], [[2.0, 2.0, 6.0], [7.0, 3.0, 4.0], [8.0, 1.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[7.0, 7.0, 4.0]], [[7.0, 2.0, 5.0]], [[2.0, 4.0, 1.0]]])+(1.-msk_val)*numpy.array([[[7.0, 8.0, 
8.0]], [[3.0, 4.0, 8.0]], [[3.0, 4.0, 7.0]]])
      arg[:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 7.0], [5.0, 2.0, 3.0], [1.0, 8.0, 1.0]], [[4.0, 5.0, 7.0], [7.0, 7.0, 4.0], [5.0, 
3.0, 1.0]], [[2.0, 6.0, 8.0], [6.0, 3.0, 2.0], [6.0, 6.0, 5.0]]], [[[8.0, 4.0, 5.0], [3.0, 4.0, 2.0], [4.0, 6.0, 6.0]], [[2.0, 
4.0, 1.0], [7.0, 2.0, 5.0], [8.0, 3.0, 8.0]], [[5.0, 4.0, 2.0], [6.0, 5.0, 7.0], [2.0, 3.0, 5.0]]], [[[4.0, 1.0, 1.0], [7.0, 
8.0, 2.0], [8.0, 8.0, 6.0]], [[4.0, 5.0, 7.0], [2.0, 4.0, 1.0], [8.0, 3.0, 8.0]], [[4.0, 3.0, 1.0], [5.0, 7.0, 7.0], [4.0, 4.0, 
8.0]]], [[[4.0, 1.0, 6.0], [8.0, 2.0, 6.0], [5.0, 4.0, 2.0]], [[4.0, 4.0, 1.0], [3.0, 5.0, 7.0], [4.0, 1.0, 3.0]], [[1.0, 6.0, 
4.0], [5.0, 3.0, 3.0], [3.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 8.0], [7.0, 3.0, 7.0], [5.0, 1.0, 8.0]], 
[[6.0, 3.0, 3.0], [7.0, 8.0, 8.0], [7.0, 4.0, 2.0]], [[6.0, 5.0, 3.0], [4.0, 3.0, 8.0], [4.0, 4.0, 5.0]]], [[[2.0, 4.0, 3.0], 
[1.0, 6.0, 3.0], [3.0, 4.0, 1.0]], [[7.0, 6.0, 4.0], [3.0, 4.0, 8.0], [8.0, 6.0, 7.0]], [[2.0, 3.0, 3.0], [4.0, 2.0, 2.0], 
[4.0, 2.0, 1.0]]], [[[7.0, 4.0, 4.0], [7.0, 5.0, 5.0], [7.0, 8.0, 5.0]], [[1.0, 1.0, 7.0], [3.0, 4.0, 7.0], [8.0, 5.0, 7.0]], 
[[6.0, 1.0, 1.0], [7.0, 5.0, 8.0], [6.0, 5.0, 1.0]]], [[[3.0, 2.0, 1.0], [3.0, 3.0, 4.0], [1.0, 8.0, 3.0]], [[1.0, 3.0, 2.0], 
[1.0, 4.0, 8.0], [5.0, 5.0, 1.0]], [[2.0, 2.0, 6.0], [7.0, 3.0, 4.0], [8.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_URPN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 5.0], [4.0, 3.0, 2.0], [1.0, 4.0, 4.0]], [[1.0, 5.0, 5.0], [2.0, 2.0, 7.0], [6.0, 7.0, 
6.0]], [[2.0, 5.0, 8.0], [4.0, 5.0, 2.0], [6.0, 8.0, 7.0]]], [[[8.0, 4.0, 1.0], [7.0, 8.0, 1.0], [3.0, 6.0, 5.0]], [[4.0, 1.0, 
5.0], [7.0, 6.0, 1.0], [8.0, 6.0, 2.0]], [[8.0, 7.0, 8.0], [3.0, 4.0, 4.0], [1.0, 6.0, 5.0]]], [[[7.0, 3.0, 5.0], [4.0, 7.0, 
6.0], [6.0, 6.0, 7.0]], [[1.0, 6.0, 7.0], [3.0, 4.0, 6.0], [3.0, 6.0, 4.0]], [[5.0, 4.0, 3.0], [4.0, 7.0, 1.0], [5.0, 2.0, 
4.0]]], [[[2.0, 1.0, 2.0], [2.0, 8.0, 8.0], [8.0, 3.0, 4.0]], [[6.0, 5.0, 5.0], [5.0, 7.0, 6.0], [7.0, 2.0, 4.0]], [[4.0, 6.0, 
2.0], [1.0, 1.0, 2.0], [6.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 4.0], [3.0, 8.0, 5.0], [8.0, 8.0, 7.0]], [[8.0, 8.0, 3.0], [2.0, 5.0, 
2.0], [4.0, 3.0, 7.0]], [[8.0, 2.0, 1.0], [8.0, 6.0, 6.0], [2.0, 6.0, 3.0]]], [[[4.0, 6.0, 3.0], [6.0, 6.0, 6.0], [5.0, 1.0, 
1.0]], [[1.0, 4.0, 3.0], [5.0, 6.0, 5.0], [1.0, 1.0, 3.0]], [[7.0, 3.0, 6.0], [5.0, 8.0, 3.0], [6.0, 2.0, 6.0]]], [[[5.0, 4.0, 
5.0], [5.0, 2.0, 1.0], [6.0, 4.0, 8.0]], [[7.0, 8.0, 1.0], [4.0, 5.0, 8.0], [1.0, 8.0, 1.0]], [[8.0, 3.0, 6.0], [5.0, 4.0, 
8.0], [1.0, 2.0, 4.0]]], [[[4.0, 4.0, 5.0], [2.0, 7.0, 3.0], [1.0, 2.0, 3.0]], [[8.0, 5.0, 2.0], [7.0, 5.0, 2.0], [5.0, 4.0, 
2.0]], [[1.0, 5.0, 5.0], [6.0, 5.0, 1.0], [1.0, 6.0, 6.0]]]]))
      val=numpy.array([[[6.0, 2.0, 5.0]], [[4.0, 4.0, 4.0]], [[8.0, 8.0, 5.0]]])
      arg[:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 5.0], [4.0, 3.0, 2.0], [1.0, 4.0, 4.0]], [[1.0, 5.0, 5.0], [6.0, 2.0, 5.0], [6.0, 
7.0, 6.0]], [[2.0, 5.0, 8.0], [4.0, 5.0, 2.0], [6.0, 8.0, 7.0]]], [[[8.0, 4.0, 1.0], [7.0, 8.0, 1.0], [3.0, 6.0, 5.0]], [[4.0, 
1.0, 5.0], [4.0, 4.0, 4.0], [8.0, 6.0, 2.0]], [[8.0, 7.0, 8.0], [3.0, 4.0, 4.0], [1.0, 6.0, 5.0]]], [[[7.0, 3.0, 5.0], [4.0, 
7.0, 6.0], [6.0, 6.0, 7.0]], [[1.0, 6.0, 7.0], [8.0, 8.0, 5.0], [3.0, 6.0, 4.0]], [[5.0, 4.0, 3.0], [4.0, 7.0, 1.0], [5.0, 2.0, 
4.0]]], [[[2.0, 1.0, 2.0], [2.0, 8.0, 8.0], [8.0, 3.0, 4.0]], [[6.0, 5.0, 5.0], [5.0, 7.0, 6.0], [7.0, 2.0, 4.0]], [[4.0, 6.0, 
2.0], [1.0, 1.0, 2.0], [6.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 4.0], [3.0, 8.0, 5.0], [8.0, 8.0, 7.0]], 
[[8.0, 8.0, 3.0], [6.0, 2.0, 5.0], [4.0, 3.0, 7.0]], [[8.0, 2.0, 1.0], [8.0, 6.0, 6.0], [2.0, 6.0, 3.0]]], [[[4.0, 6.0, 3.0], 
[6.0, 6.0, 6.0], [5.0, 1.0, 1.0]], [[1.0, 4.0, 3.0], [4.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[7.0, 3.0, 6.0], [5.0, 8.0, 3.0], 
[6.0, 2.0, 6.0]]], [[[5.0, 4.0, 5.0], [5.0, 2.0, 1.0], [6.0, 4.0, 8.0]], [[7.0, 8.0, 1.0], [8.0, 8.0, 5.0], [1.0, 8.0, 1.0]], 
[[8.0, 3.0, 6.0], [5.0, 4.0, 8.0], [1.0, 2.0, 4.0]]], [[[4.0, 4.0, 5.0], [2.0, 7.0, 3.0], [1.0, 2.0, 3.0]], [[8.0, 5.0, 2.0], 
[7.0, 5.0, 2.0], [5.0, 4.0, 2.0]], [[1.0, 5.0, 5.0], [6.0, 5.0, 1.0], [1.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_URPN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 6.0], [4.0, 7.0, 8.0], [1.0, 8.0, 6.0]], [[5.0, 5.0, 4.0], [2.0, 5.0, 1.0], [8.0, 1.0, 
5.0]], [[3.0, 8.0, 5.0], [1.0, 8.0, 4.0], [4.0, 8.0, 4.0]]], [[[7.0, 1.0, 4.0], [1.0, 6.0, 1.0], [4.0, 7.0, 3.0]], [[6.0, 5.0, 
5.0], [6.0, 8.0, 3.0], [6.0, 1.0, 5.0]], [[3.0, 4.0, 5.0], [5.0, 7.0, 7.0], [8.0, 1.0, 1.0]]], [[[8.0, 6.0, 2.0], [4.0, 5.0, 
6.0], [6.0, 4.0, 1.0]], [[3.0, 1.0, 2.0], [8.0, 1.0, 7.0], [2.0, 4.0, 4.0]], [[2.0, 5.0, 1.0], [2.0, 2.0, 6.0], [6.0, 6.0, 
6.0]]], [[[8.0, 8.0, 3.0], [6.0, 7.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 1.0, 4.0], [7.0, 6.0, 6.0], [5.0, 2.0, 8.0]], [[3.0, 7.0, 
7.0], [5.0, 8.0, 1.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 7.0], [1.0, 8.0, 7.0], [8.0, 3.0, 2.0]], [[6.0, 7.0, 1.0], [5.0, 3.0, 
2.0], [7.0, 5.0, 5.0]], [[6.0, 4.0, 6.0], [6.0, 1.0, 2.0], [8.0, 5.0, 1.0]]], [[[6.0, 7.0, 4.0], [6.0, 3.0, 4.0], [2.0, 8.0, 
2.0]], [[5.0, 2.0, 8.0], [6.0, 1.0, 1.0], [4.0, 1.0, 6.0]], [[2.0, 8.0, 6.0], [6.0, 6.0, 7.0], [5.0, 8.0, 4.0]]], [[[1.0, 1.0, 
2.0], [7.0, 2.0, 6.0], [2.0, 7.0, 3.0]], [[3.0, 6.0, 5.0], [8.0, 5.0, 8.0], [8.0, 4.0, 3.0]], [[6.0, 2.0, 5.0], [6.0, 7.0, 
5.0], [5.0, 1.0, 7.0]]], [[[7.0, 4.0, 7.0], [4.0, 3.0, 1.0], [5.0, 1.0, 4.0]], [[7.0, 7.0, 5.0], [1.0, 2.0, 1.0], [1.0, 7.0, 
8.0]], [[3.0, 5.0, 2.0], [5.0, 6.0, 7.0], [4.0, 4.0, 3.0]]]]))
      val=Data(numpy.array([[[8.0, 5.0, 4.0]], [[3.0, 5.0, 2.0]], [[6.0, 7.0, 1.0]]]),self.functionspace)
      arg[:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 6.0], [4.0, 7.0, 8.0], [1.0, 8.0, 6.0]], [[5.0, 5.0, 4.0], [8.0, 5.0, 4.0], [8.0, 
1.0, 5.0]], [[3.0, 8.0, 5.0], [1.0, 8.0, 4.0], [4.0, 8.0, 4.0]]], [[[7.0, 1.0, 4.0], [1.0, 6.0, 1.0], [4.0, 7.0, 3.0]], [[6.0, 
5.0, 5.0], [3.0, 5.0, 2.0], [6.0, 1.0, 5.0]], [[3.0, 4.0, 5.0], [5.0, 7.0, 7.0], [8.0, 1.0, 1.0]]], [[[8.0, 6.0, 2.0], [4.0, 
5.0, 6.0], [6.0, 4.0, 1.0]], [[3.0, 1.0, 2.0], [6.0, 7.0, 1.0], [2.0, 4.0, 4.0]], [[2.0, 5.0, 1.0], [2.0, 2.0, 6.0], [6.0, 6.0, 
6.0]]], [[[8.0, 8.0, 3.0], [6.0, 7.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 1.0, 4.0], [7.0, 6.0, 6.0], [5.0, 2.0, 8.0]], [[3.0, 7.0, 
7.0], [5.0, 8.0, 1.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 7.0], [1.0, 8.0, 7.0], [8.0, 3.0, 2.0]], 
[[6.0, 7.0, 1.0], [8.0, 5.0, 4.0], [7.0, 5.0, 5.0]], [[6.0, 4.0, 6.0], [6.0, 1.0, 2.0], [8.0, 5.0, 1.0]]], [[[6.0, 7.0, 4.0], 
[6.0, 3.0, 4.0], [2.0, 8.0, 2.0]], [[5.0, 2.0, 8.0], [3.0, 5.0, 2.0], [4.0, 1.0, 6.0]], [[2.0, 8.0, 6.0], [6.0, 6.0, 7.0], 
[5.0, 8.0, 4.0]]], [[[1.0, 1.0, 2.0], [7.0, 2.0, 6.0], [2.0, 7.0, 3.0]], [[3.0, 6.0, 5.0], [6.0, 7.0, 1.0], [8.0, 4.0, 3.0]], 
[[6.0, 2.0, 5.0], [6.0, 7.0, 5.0], [5.0, 1.0, 7.0]]], [[[7.0, 4.0, 7.0], [4.0, 3.0, 1.0], [5.0, 1.0, 4.0]], [[7.0, 7.0, 5.0], 
[1.0, 2.0, 1.0], [1.0, 7.0, 8.0]], [[3.0, 5.0, 2.0], [5.0, 6.0, 7.0], [4.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_URPN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 8.0], [8.0, 5.0, 5.0], [1.0, 3.0, 6.0]], [[2.0, 8.0, 6.0], [1.0, 5.0, 4.0], [7.0, 4.0, 
6.0]], [[8.0, 3.0, 1.0], [5.0, 5.0, 8.0], [5.0, 3.0, 4.0]]], [[[5.0, 1.0, 7.0], [1.0, 3.0, 4.0], [4.0, 6.0, 6.0]], [[3.0, 3.0, 
1.0], [5.0, 2.0, 8.0], [1.0, 5.0, 1.0]], [[4.0, 2.0, 1.0], [5.0, 4.0, 7.0], [8.0, 7.0, 4.0]]], [[[4.0, 6.0, 5.0], [6.0, 6.0, 
7.0], [5.0, 5.0, 6.0]], [[2.0, 7.0, 2.0], [5.0, 7.0, 1.0], [6.0, 4.0, 6.0]], [[8.0, 8.0, 5.0], [6.0, 4.0, 3.0], [2.0, 2.0, 
5.0]]], [[[6.0, 7.0, 8.0], [4.0, 4.0, 4.0], [1.0, 7.0, 4.0]], [[6.0, 7.0, 6.0], [8.0, 2.0, 2.0], [4.0, 8.0, 4.0]], [[4.0, 1.0, 
4.0], [8.0, 1.0, 1.0], [6.0, 4.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 7.0], [7.0, 6.0, 5.0], [3.0, 1.0, 1.0]], [[1.0, 7.0, 6.0], [7.0, 8.0, 
6.0], [3.0, 3.0, 1.0]], [[2.0, 5.0, 6.0], [3.0, 6.0, 3.0], [5.0, 2.0, 2.0]]], [[[3.0, 8.0, 5.0], [8.0, 1.0, 1.0], [8.0, 4.0, 
4.0]], [[4.0, 7.0, 7.0], [8.0, 2.0, 5.0], [1.0, 7.0, 5.0]], [[8.0, 2.0, 7.0], [4.0, 1.0, 6.0], [6.0, 7.0, 6.0]]], [[[2.0, 8.0, 
7.0], [2.0, 5.0, 6.0], [7.0, 3.0, 4.0]], [[2.0, 8.0, 8.0], [1.0, 5.0, 5.0], [8.0, 6.0, 7.0]], [[4.0, 8.0, 2.0], [8.0, 1.0, 
6.0], [4.0, 1.0, 2.0]]], [[[3.0, 6.0, 6.0], [5.0, 1.0, 8.0], [6.0, 1.0, 2.0]], [[5.0, 4.0, 6.0], [8.0, 7.0, 5.0], [3.0, 6.0, 
5.0]], [[6.0, 5.0, 7.0], [7.0, 5.0, 8.0], [5.0, 8.0, 6.0]]]]))
      val=Data(numpy.array([[[4.0, 4.0, 7.0]], [[7.0, 3.0, 4.0]], [[3.0, 8.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 2.0, 7.0]], [[8.0, 1.0, 3.0]], [[2.0, 7.0, 1.0]]]))
      arg[:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 8.0], [8.0, 5.0, 5.0], [1.0, 3.0, 6.0]], [[2.0, 8.0, 6.0], [4.0, 4.0, 7.0], [7.0, 
4.0, 6.0]], [[8.0, 3.0, 1.0], [5.0, 5.0, 8.0], [5.0, 3.0, 4.0]]], [[[5.0, 1.0, 7.0], [1.0, 3.0, 4.0], [4.0, 6.0, 6.0]], [[3.0, 
3.0, 1.0], [7.0, 3.0, 4.0], [1.0, 5.0, 1.0]], [[4.0, 2.0, 1.0], [5.0, 4.0, 7.0], [8.0, 7.0, 4.0]]], [[[4.0, 6.0, 5.0], [6.0, 
6.0, 7.0], [5.0, 5.0, 6.0]], [[2.0, 7.0, 2.0], [3.0, 8.0, 6.0], [6.0, 4.0, 6.0]], [[8.0, 8.0, 5.0], [6.0, 4.0, 3.0], [2.0, 2.0, 
5.0]]], [[[6.0, 7.0, 8.0], [4.0, 4.0, 4.0], [1.0, 7.0, 4.0]], [[6.0, 7.0, 6.0], [8.0, 2.0, 2.0], [4.0, 8.0, 4.0]], [[4.0, 1.0, 
4.0], [8.0, 1.0, 1.0], [6.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 7.0], [7.0, 6.0, 5.0], [3.0, 1.0, 1.0]], 
[[1.0, 7.0, 6.0], [1.0, 2.0, 7.0], [3.0, 3.0, 1.0]], [[2.0, 5.0, 6.0], [3.0, 6.0, 3.0], [5.0, 2.0, 2.0]]], [[[3.0, 8.0, 5.0], 
[8.0, 1.0, 1.0], [8.0, 4.0, 4.0]], [[4.0, 7.0, 7.0], [8.0, 1.0, 3.0], [1.0, 7.0, 5.0]], [[8.0, 2.0, 7.0], [4.0, 1.0, 6.0], 
[6.0, 7.0, 6.0]]], [[[2.0, 8.0, 7.0], [2.0, 5.0, 6.0], [7.0, 3.0, 4.0]], [[2.0, 8.0, 8.0], [2.0, 7.0, 1.0], [8.0, 6.0, 7.0]], 
[[4.0, 8.0, 2.0], [8.0, 1.0, 6.0], [4.0, 1.0, 2.0]]], [[[3.0, 6.0, 6.0], [5.0, 1.0, 8.0], [6.0, 1.0, 2.0]], [[5.0, 4.0, 6.0], 
[8.0, 7.0, 5.0], [3.0, 6.0, 5.0]], [[6.0, 5.0, 7.0], [7.0, 5.0, 8.0], [5.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_URPN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 8.0], [8.0, 3.0, 6.0], [1.0, 8.0, 3.0]], [[7.0, 1.0, 2.0], [3.0, 5.0, 2.0], [4.0, 7.0, 
7.0]], [[2.0, 1.0, 4.0], [3.0, 4.0, 8.0], [4.0, 3.0, 1.0]]], [[[1.0, 4.0, 7.0], [6.0, 4.0, 6.0], [2.0, 8.0, 3.0]], [[3.0, 8.0, 
8.0], [4.0, 7.0, 1.0], [2.0, 8.0, 3.0]], [[7.0, 2.0, 3.0], [4.0, 4.0, 5.0], [1.0, 2.0, 2.0]]], [[[3.0, 4.0, 6.0], [8.0, 5.0, 
8.0], [4.0, 3.0, 3.0]], [[2.0, 1.0, 3.0], [6.0, 4.0, 7.0], [5.0, 1.0, 6.0]], [[1.0, 1.0, 4.0], [4.0, 2.0, 8.0], [5.0, 7.0, 
4.0]]], [[[3.0, 1.0, 1.0], [6.0, 6.0, 7.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 1.0], [7.0, 5.0, 6.0], [8.0, 3.0, 6.0]], [[5.0, 3.0, 
1.0], [6.0, 5.0, 2.0], [5.0, 3.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 6.0, 8.0], [2.0, 6.0, 8.0], [3.0, 5.0, 3.0]], [[7.0, 1.0, 5.0], [6.0, 2.0, 
7.0], [4.0, 2.0, 7.0]], [[3.0, 2.0, 7.0], [2.0, 2.0, 1.0], [3.0, 5.0, 1.0]]], [[[8.0, 3.0, 6.0], [1.0, 7.0, 4.0], [7.0, 3.0, 
4.0]], [[3.0, 4.0, 2.0], [1.0, 2.0, 1.0], [1.0, 2.0, 7.0]], [[3.0, 1.0, 8.0], [3.0, 2.0, 3.0], [8.0, 6.0, 6.0]]], [[[3.0, 5.0, 
7.0], [4.0, 5.0, 4.0], [5.0, 8.0, 7.0]], [[1.0, 5.0, 2.0], [1.0, 8.0, 3.0], [7.0, 8.0, 2.0]], [[2.0, 6.0, 8.0], [4.0, 3.0, 
4.0], [4.0, 8.0, 3.0]]], [[[2.0, 1.0, 6.0], [3.0, 4.0, 8.0], [7.0, 6.0, 1.0]], [[5.0, 6.0, 6.0], [3.0, 8.0, 8.0], [7.0, 6.0, 
4.0]], [[3.0, 3.0, 4.0], [2.0, 5.0, 3.0], [6.0, 7.0, 7.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 8.0, 6.0]], [[1.0, 3.0, 6.0]], [[8.0, 2.0, 7.0]]])+(1.-msk_val)*numpy.array([[[1.0, 3.0, 
1.0]], [[4.0, 5.0, 2.0]], [[5.0, 2.0, 7.0]]])
      arg[:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 8.0], [8.0, 3.0, 6.0], [1.0, 8.0, 3.0]], [[7.0, 1.0, 2.0], [8.0, 8.0, 6.0], [4.0, 
7.0, 7.0]], [[2.0, 1.0, 4.0], [3.0, 4.0, 8.0], [4.0, 3.0, 1.0]]], [[[1.0, 4.0, 7.0], [6.0, 4.0, 6.0], [2.0, 8.0, 3.0]], [[3.0, 
8.0, 8.0], [1.0, 3.0, 6.0], [2.0, 8.0, 3.0]], [[7.0, 2.0, 3.0], [4.0, 4.0, 5.0], [1.0, 2.0, 2.0]]], [[[3.0, 4.0, 6.0], [8.0, 
5.0, 8.0], [4.0, 3.0, 3.0]], [[2.0, 1.0, 3.0], [8.0, 2.0, 7.0], [5.0, 1.0, 6.0]], [[1.0, 1.0, 4.0], [4.0, 2.0, 8.0], [5.0, 7.0, 
4.0]]], [[[3.0, 1.0, 1.0], [6.0, 6.0, 7.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 1.0], [7.0, 5.0, 6.0], [8.0, 3.0, 6.0]], [[5.0, 3.0, 
1.0], [6.0, 5.0, 2.0], [5.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 8.0], [2.0, 6.0, 8.0], [3.0, 5.0, 3.0]], 
[[7.0, 1.0, 5.0], [1.0, 3.0, 1.0], [4.0, 2.0, 7.0]], [[3.0, 2.0, 7.0], [2.0, 2.0, 1.0], [3.0, 5.0, 1.0]]], [[[8.0, 3.0, 6.0], 
[1.0, 7.0, 4.0], [7.0, 3.0, 4.0]], [[3.0, 4.0, 2.0], [4.0, 5.0, 2.0], [1.0, 2.0, 7.0]], [[3.0, 1.0, 8.0], [3.0, 2.0, 3.0], 
[8.0, 6.0, 6.0]]], [[[3.0, 5.0, 7.0], [4.0, 5.0, 4.0], [5.0, 8.0, 7.0]], [[1.0, 5.0, 2.0], [5.0, 2.0, 7.0], [7.0, 8.0, 2.0]], 
[[2.0, 6.0, 8.0], [4.0, 3.0, 4.0], [4.0, 8.0, 3.0]]], [[[2.0, 1.0, 6.0], [3.0, 4.0, 8.0], [7.0, 6.0, 1.0]], [[5.0, 6.0, 6.0], 
[3.0, 8.0, 8.0], [7.0, 6.0, 4.0]], [[3.0, 3.0, 4.0], [2.0, 5.0, 3.0], [6.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RLLN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 1.0], [1.0, 3.0, 6.0], [7.0, 8.0, 6.0]], [[2.0, 3.0, 6.0], [4.0, 2.0, 8.0], [8.0, 2.0, 
4.0]], [[1.0, 8.0, 4.0], [5.0, 3.0, 6.0], [8.0, 8.0, 3.0]]], [[[5.0, 3.0, 4.0], [6.0, 7.0, 3.0], [2.0, 3.0, 1.0]], [[5.0, 5.0, 
8.0], [2.0, 3.0, 6.0], [3.0, 3.0, 8.0]], [[7.0, 6.0, 4.0], [5.0, 3.0, 5.0], [6.0, 4.0, 2.0]]], [[[4.0, 3.0, 8.0], [2.0, 5.0, 
8.0], [8.0, 7.0, 4.0]], [[6.0, 5.0, 5.0], [2.0, 5.0, 2.0], [4.0, 8.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 7.0, 3.0], [6.0, 7.0, 
3.0]]], [[[3.0, 7.0, 8.0], [2.0, 1.0, 8.0], [1.0, 5.0, 2.0]], [[3.0, 3.0, 7.0], [6.0, 6.0, 1.0], [1.0, 2.0, 4.0]], [[8.0, 6.0, 
1.0], [2.0, 1.0, 8.0], [7.0, 2.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 8.0], [7.0, 1.0, 2.0], [1.0, 8.0, 5.0]], [[6.0, 2.0, 2.0], [4.0, 6.0, 
1.0], [7.0, 8.0, 8.0]], [[4.0, 2.0, 6.0], [1.0, 3.0, 4.0], [8.0, 1.0, 3.0]]], [[[1.0, 1.0, 6.0], [7.0, 1.0, 1.0], [3.0, 5.0, 
7.0]], [[2.0, 7.0, 4.0], [4.0, 8.0, 3.0], [7.0, 8.0, 7.0]], [[2.0, 2.0, 2.0], [3.0, 1.0, 6.0], [4.0, 1.0, 5.0]]], [[[2.0, 4.0, 
6.0], [3.0, 1.0, 8.0], [7.0, 5.0, 2.0]], [[8.0, 8.0, 4.0], [7.0, 3.0, 8.0], [8.0, 6.0, 6.0]], [[4.0, 4.0, 2.0], [3.0, 1.0, 
1.0], [4.0, 4.0, 3.0]]], [[[6.0, 1.0, 2.0], [3.0, 8.0, 7.0], [5.0, 3.0, 8.0]], [[5.0, 1.0, 8.0], [3.0, 6.0, 2.0], [4.0, 4.0, 
2.0]], [[3.0, 7.0, 6.0], [7.0, 3.0, 3.0], [3.0, 5.0, 3.0]]]]))
      val=numpy.array([[[[2.0, 8.0, 5.0]]], [[[2.0, 5.0, 3.0]]]])
      arg[1:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 1.0], [1.0, 3.0, 6.0], [7.0, 8.0, 6.0]], [[2.0, 3.0, 6.0], [4.0, 2.0, 8.0], [8.0, 
2.0, 4.0]], [[1.0, 8.0, 4.0], [5.0, 3.0, 6.0], [8.0, 8.0, 3.0]]], [[[5.0, 3.0, 4.0], [6.0, 7.0, 3.0], [2.0, 3.0, 1.0]], [[5.0, 
5.0, 8.0], [2.0, 3.0, 6.0], [3.0, 3.0, 8.0]], [[7.0, 6.0, 4.0], [5.0, 3.0, 5.0], [2.0, 8.0, 5.0]]], [[[4.0, 3.0, 8.0], [2.0, 
5.0, 8.0], [8.0, 7.0, 4.0]], [[6.0, 5.0, 5.0], [2.0, 5.0, 2.0], [4.0, 8.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 7.0, 3.0], [2.0, 5.0, 
3.0]]], [[[3.0, 7.0, 8.0], [2.0, 1.0, 8.0], [1.0, 5.0, 2.0]], [[3.0, 3.0, 7.0], [6.0, 6.0, 1.0], [1.0, 2.0, 4.0]], [[8.0, 6.0, 
1.0], [2.0, 1.0, 8.0], [7.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 8.0], [7.0, 1.0, 2.0], [1.0, 8.0, 5.0]], 
[[6.0, 2.0, 2.0], [4.0, 6.0, 1.0], [7.0, 8.0, 8.0]], [[4.0, 2.0, 6.0], [1.0, 3.0, 4.0], [8.0, 1.0, 3.0]]], [[[1.0, 1.0, 6.0], 
[7.0, 1.0, 1.0], [3.0, 5.0, 7.0]], [[2.0, 7.0, 4.0], [4.0, 8.0, 3.0], [7.0, 8.0, 7.0]], [[2.0, 2.0, 2.0], [3.0, 1.0, 6.0], 
[2.0, 8.0, 5.0]]], [[[2.0, 4.0, 6.0], [3.0, 1.0, 8.0], [7.0, 5.0, 2.0]], [[8.0, 8.0, 4.0], [7.0, 3.0, 8.0], [8.0, 6.0, 6.0]], 
[[4.0, 4.0, 2.0], [3.0, 1.0, 1.0], [2.0, 5.0, 3.0]]], [[[6.0, 1.0, 2.0], [3.0, 8.0, 7.0], [5.0, 3.0, 8.0]], [[5.0, 1.0, 8.0], 
[3.0, 6.0, 2.0], [4.0, 4.0, 2.0]], [[3.0, 7.0, 6.0], [7.0, 3.0, 3.0], [3.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RLLN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 4.0], [2.0, 8.0, 5.0], [4.0, 7.0, 3.0]], [[3.0, 8.0, 5.0], [4.0, 3.0, 6.0], [4.0, 7.0, 
2.0]], [[1.0, 8.0, 7.0], [4.0, 2.0, 1.0], [5.0, 6.0, 7.0]]], [[[5.0, 5.0, 6.0], [8.0, 8.0, 8.0], [1.0, 1.0, 6.0]], [[4.0, 6.0, 
4.0], [1.0, 2.0, 8.0], [4.0, 3.0, 4.0]], [[3.0, 2.0, 3.0], [8.0, 6.0, 2.0], [5.0, 7.0, 8.0]]], [[[7.0, 8.0, 2.0], [6.0, 2.0, 
3.0], [2.0, 1.0, 4.0]], [[1.0, 7.0, 1.0], [4.0, 5.0, 7.0], [2.0, 5.0, 4.0]], [[6.0, 2.0, 7.0], [7.0, 2.0, 4.0], [5.0, 5.0, 
7.0]]], [[[7.0, 8.0, 1.0], [1.0, 1.0, 4.0], [4.0, 2.0, 4.0]], [[4.0, 1.0, 3.0], [4.0, 1.0, 6.0], [7.0, 1.0, 2.0]], [[4.0, 3.0, 
2.0], [2.0, 2.0, 8.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 6.0], [5.0, 3.0, 1.0], [5.0, 1.0, 8.0]], [[7.0, 2.0, 6.0], [6.0, 4.0, 
2.0], [3.0, 8.0, 2.0]], [[5.0, 6.0, 2.0], [8.0, 2.0, 1.0], [7.0, 7.0, 6.0]]], [[[1.0, 4.0, 2.0], [2.0, 1.0, 5.0], [6.0, 2.0, 
3.0]], [[1.0, 1.0, 8.0], [7.0, 3.0, 2.0], [3.0, 8.0, 6.0]], [[8.0, 5.0, 3.0], [2.0, 2.0, 1.0], [4.0, 3.0, 4.0]]], [[[4.0, 7.0, 
2.0], [2.0, 7.0, 1.0], [2.0, 8.0, 1.0]], [[2.0, 8.0, 2.0], [7.0, 5.0, 7.0], [1.0, 8.0, 4.0]], [[2.0, 4.0, 3.0], [5.0, 3.0, 
5.0], [2.0, 4.0, 1.0]]], [[[2.0, 3.0, 8.0], [3.0, 8.0, 8.0], [1.0, 6.0, 2.0]], [[2.0, 2.0, 7.0], [3.0, 6.0, 6.0], [7.0, 3.0, 
7.0]], [[5.0, 2.0, 3.0], [3.0, 8.0, 1.0], [4.0, 8.0, 8.0]]]]))
      val=Data(numpy.array([[[[2.0, 2.0, 2.0]]], [[[3.0, 7.0, 8.0]]]]),self.functionspace)
      arg[1:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [2.0, 8.0, 5.0], [4.0, 7.0, 3.0]], [[3.0, 8.0, 5.0], [4.0, 3.0, 6.0], [4.0, 
7.0, 2.0]], [[1.0, 8.0, 7.0], [4.0, 2.0, 1.0], [5.0, 6.0, 7.0]]], [[[5.0, 5.0, 6.0], [8.0, 8.0, 8.0], [1.0, 1.0, 6.0]], [[4.0, 
6.0, 4.0], [1.0, 2.0, 8.0], [4.0, 3.0, 4.0]], [[3.0, 2.0, 3.0], [8.0, 6.0, 2.0], [2.0, 2.0, 2.0]]], [[[7.0, 8.0, 2.0], [6.0, 
2.0, 3.0], [2.0, 1.0, 4.0]], [[1.0, 7.0, 1.0], [4.0, 5.0, 7.0], [2.0, 5.0, 4.0]], [[6.0, 2.0, 7.0], [7.0, 2.0, 4.0], [3.0, 7.0, 
8.0]]], [[[7.0, 8.0, 1.0], [1.0, 1.0, 4.0], [4.0, 2.0, 4.0]], [[4.0, 1.0, 3.0], [4.0, 1.0, 6.0], [7.0, 1.0, 2.0]], [[4.0, 3.0, 
2.0], [2.0, 2.0, 8.0], [3.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 6.0], [5.0, 3.0, 1.0], [5.0, 1.0, 8.0]], 
[[7.0, 2.0, 6.0], [6.0, 4.0, 2.0], [3.0, 8.0, 2.0]], [[5.0, 6.0, 2.0], [8.0, 2.0, 1.0], [7.0, 7.0, 6.0]]], [[[1.0, 4.0, 2.0], 
[2.0, 1.0, 5.0], [6.0, 2.0, 3.0]], [[1.0, 1.0, 8.0], [7.0, 3.0, 2.0], [3.0, 8.0, 6.0]], [[8.0, 5.0, 3.0], [2.0, 2.0, 1.0], 
[2.0, 2.0, 2.0]]], [[[4.0, 7.0, 2.0], [2.0, 7.0, 1.0], [2.0, 8.0, 1.0]], [[2.0, 8.0, 2.0], [7.0, 5.0, 7.0], [1.0, 8.0, 4.0]], 
[[2.0, 4.0, 3.0], [5.0, 3.0, 5.0], [3.0, 7.0, 8.0]]], [[[2.0, 3.0, 8.0], [3.0, 8.0, 8.0], [1.0, 6.0, 2.0]], [[2.0, 2.0, 7.0], 
[3.0, 6.0, 6.0], [7.0, 3.0, 7.0]], [[5.0, 2.0, 3.0], [3.0, 8.0, 1.0], [4.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RLLN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 4.0], [4.0, 4.0, 7.0], [6.0, 1.0, 8.0]], [[2.0, 5.0, 4.0], [1.0, 7.0, 1.0], [1.0, 2.0, 
7.0]], [[5.0, 1.0, 7.0], [2.0, 6.0, 1.0], [3.0, 7.0, 8.0]]], [[[5.0, 6.0, 8.0], [7.0, 6.0, 1.0], [4.0, 5.0, 5.0]], [[3.0, 8.0, 
6.0], [3.0, 4.0, 2.0], [7.0, 8.0, 5.0]], [[8.0, 2.0, 7.0], [5.0, 5.0, 8.0], [4.0, 2.0, 2.0]]], [[[7.0, 3.0, 5.0], [5.0, 4.0, 
2.0], [5.0, 3.0, 2.0]], [[2.0, 2.0, 1.0], [5.0, 4.0, 1.0], [6.0, 2.0, 7.0]], [[7.0, 4.0, 8.0], [2.0, 7.0, 7.0], [8.0, 6.0, 
4.0]]], [[[2.0, 8.0, 4.0], [4.0, 1.0, 3.0], [5.0, 2.0, 3.0]], [[2.0, 3.0, 2.0], [1.0, 2.0, 4.0], [6.0, 3.0, 2.0]], [[4.0, 5.0, 
6.0], [7.0, 4.0, 4.0], [5.0, 5.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 1.0], [5.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[2.0, 8.0, 6.0], [7.0, 5.0, 
5.0], [8.0, 6.0, 7.0]], [[7.0, 7.0, 7.0], [8.0, 4.0, 3.0], [2.0, 8.0, 1.0]]], [[[8.0, 8.0, 7.0], [3.0, 8.0, 4.0], [3.0, 6.0, 
4.0]], [[4.0, 7.0, 4.0], [7.0, 4.0, 4.0], [6.0, 6.0, 1.0]], [[3.0, 7.0, 4.0], [2.0, 8.0, 5.0], [8.0, 5.0, 6.0]]], [[[5.0, 2.0, 
7.0], [5.0, 2.0, 2.0], [5.0, 5.0, 4.0]], [[1.0, 1.0, 3.0], [3.0, 1.0, 3.0], [3.0, 2.0, 6.0]], [[3.0, 2.0, 2.0], [1.0, 1.0, 
8.0], [3.0, 8.0, 8.0]]], [[[8.0, 3.0, 8.0], [5.0, 6.0, 8.0], [3.0, 2.0, 5.0]], [[4.0, 6.0, 8.0], [1.0, 4.0, 5.0], [6.0, 7.0, 
3.0]], [[7.0, 1.0, 1.0], [8.0, 6.0, 5.0], [2.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[[[1.0, 5.0, 6.0]]], [[[1.0, 5.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 3.0, 2.0]]], [[[4.0, 4.0, 6.0]]]]))
      arg[1:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 4.0], [4.0, 4.0, 7.0], [6.0, 1.0, 8.0]], [[2.0, 5.0, 4.0], [1.0, 7.0, 1.0], [1.0, 
2.0, 7.0]], [[5.0, 1.0, 7.0], [2.0, 6.0, 1.0], [3.0, 7.0, 8.0]]], [[[5.0, 6.0, 8.0], [7.0, 6.0, 1.0], [4.0, 5.0, 5.0]], [[3.0, 
8.0, 6.0], [3.0, 4.0, 2.0], [7.0, 8.0, 5.0]], [[8.0, 2.0, 7.0], [5.0, 5.0, 8.0], [1.0, 5.0, 6.0]]], [[[7.0, 3.0, 5.0], [5.0, 
4.0, 2.0], [5.0, 3.0, 2.0]], [[2.0, 2.0, 1.0], [5.0, 4.0, 1.0], [6.0, 2.0, 7.0]], [[7.0, 4.0, 8.0], [2.0, 7.0, 7.0], [1.0, 5.0, 
2.0]]], [[[2.0, 8.0, 4.0], [4.0, 1.0, 3.0], [5.0, 2.0, 3.0]], [[2.0, 3.0, 2.0], [1.0, 2.0, 4.0], [6.0, 3.0, 2.0]], [[4.0, 5.0, 
6.0], [7.0, 4.0, 4.0], [5.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 1.0], [5.0, 3.0, 7.0], [5.0, 5.0, 8.0]], 
[[2.0, 8.0, 6.0], [7.0, 5.0, 5.0], [8.0, 6.0, 7.0]], [[7.0, 7.0, 7.0], [8.0, 4.0, 3.0], [2.0, 8.0, 1.0]]], [[[8.0, 8.0, 7.0], 
[3.0, 8.0, 4.0], [3.0, 6.0, 4.0]], [[4.0, 7.0, 4.0], [7.0, 4.0, 4.0], [6.0, 6.0, 1.0]], [[3.0, 7.0, 4.0], [2.0, 8.0, 5.0], 
[4.0, 3.0, 2.0]]], [[[5.0, 2.0, 7.0], [5.0, 2.0, 2.0], [5.0, 5.0, 4.0]], [[1.0, 1.0, 3.0], [3.0, 1.0, 3.0], [3.0, 2.0, 6.0]], 
[[3.0, 2.0, 2.0], [1.0, 1.0, 8.0], [4.0, 4.0, 6.0]]], [[[8.0, 3.0, 8.0], [5.0, 6.0, 8.0], [3.0, 2.0, 5.0]], [[4.0, 6.0, 8.0], 
[1.0, 4.0, 5.0], [6.0, 7.0, 3.0]], [[7.0, 1.0, 1.0], [8.0, 6.0, 5.0], [2.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RLLN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 1.0], [8.0, 6.0, 4.0], [8.0, 1.0, 5.0]], [[1.0, 7.0, 6.0], [2.0, 5.0, 5.0], [6.0, 8.0, 
4.0]], [[8.0, 3.0, 4.0], [2.0, 2.0, 3.0], [2.0, 3.0, 1.0]]], [[[7.0, 7.0, 8.0], [8.0, 2.0, 7.0], [4.0, 7.0, 6.0]], [[3.0, 3.0, 
8.0], [3.0, 6.0, 1.0], [2.0, 6.0, 6.0]], [[1.0, 6.0, 1.0], [6.0, 7.0, 8.0], [7.0, 7.0, 7.0]]], [[[8.0, 8.0, 3.0], [5.0, 3.0, 
3.0], [4.0, 3.0, 1.0]], [[6.0, 7.0, 3.0], [4.0, 7.0, 6.0], [5.0, 1.0, 5.0]], [[4.0, 5.0, 1.0], [8.0, 7.0, 4.0], [6.0, 8.0, 
2.0]]], [[[5.0, 8.0, 7.0], [4.0, 8.0, 3.0], [1.0, 7.0, 1.0]], [[6.0, 1.0, 5.0], [5.0, 1.0, 8.0], [3.0, 7.0, 5.0]], [[8.0, 2.0, 
1.0], [2.0, 7.0, 8.0], [8.0, 8.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 5.0], [1.0, 8.0, 4.0], [1.0, 8.0, 5.0]], [[3.0, 8.0, 6.0], [4.0, 7.0, 
3.0], [6.0, 8.0, 4.0]], [[5.0, 4.0, 2.0], [5.0, 8.0, 6.0], [3.0, 5.0, 7.0]]], [[[3.0, 8.0, 4.0], [1.0, 1.0, 6.0], [6.0, 6.0, 
7.0]], [[7.0, 7.0, 5.0], [6.0, 1.0, 3.0], [2.0, 1.0, 4.0]], [[3.0, 6.0, 5.0], [2.0, 3.0, 1.0], [7.0, 7.0, 2.0]]], [[[6.0, 4.0, 
6.0], [7.0, 7.0, 7.0], [6.0, 2.0, 6.0]], [[5.0, 4.0, 7.0], [7.0, 6.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 5.0, 8.0], [5.0, 6.0, 
8.0], [1.0, 1.0, 7.0]]], [[[5.0, 5.0, 5.0], [3.0, 8.0, 3.0], [1.0, 7.0, 5.0]], [[7.0, 5.0, 5.0], [5.0, 3.0, 1.0], [8.0, 1.0, 
5.0]], [[2.0, 4.0, 5.0], [2.0, 6.0, 8.0], [2.0, 3.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 1.0, 4.0]]], [[[8.0, 6.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 7.0, 4.0]]], 
[[[4.0, 8.0, 2.0]]]])
      arg[1:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 1.0], [8.0, 6.0, 4.0], [8.0, 1.0, 5.0]], [[1.0, 7.0, 6.0], [2.0, 5.0, 5.0], [6.0, 
8.0, 4.0]], [[8.0, 3.0, 4.0], [2.0, 2.0, 3.0], [2.0, 3.0, 1.0]]], [[[7.0, 7.0, 8.0], [8.0, 2.0, 7.0], [4.0, 7.0, 6.0]], [[3.0, 
3.0, 8.0], [3.0, 6.0, 1.0], [2.0, 6.0, 6.0]], [[1.0, 6.0, 1.0], [6.0, 7.0, 8.0], [8.0, 1.0, 4.0]]], [[[8.0, 8.0, 3.0], [5.0, 
3.0, 3.0], [4.0, 3.0, 1.0]], [[6.0, 7.0, 3.0], [4.0, 7.0, 6.0], [5.0, 1.0, 5.0]], [[4.0, 5.0, 1.0], [8.0, 7.0, 4.0], [8.0, 6.0, 
2.0]]], [[[5.0, 8.0, 7.0], [4.0, 8.0, 3.0], [1.0, 7.0, 1.0]], [[6.0, 1.0, 5.0], [5.0, 1.0, 8.0], [3.0, 7.0, 5.0]], [[8.0, 2.0, 
1.0], [2.0, 7.0, 8.0], [8.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 5.0], [1.0, 8.0, 4.0], [1.0, 8.0, 5.0]], 
[[3.0, 8.0, 6.0], [4.0, 7.0, 3.0], [6.0, 8.0, 4.0]], [[5.0, 4.0, 2.0], [5.0, 8.0, 6.0], [3.0, 5.0, 7.0]]], [[[3.0, 8.0, 4.0], 
[1.0, 1.0, 6.0], [6.0, 6.0, 7.0]], [[7.0, 7.0, 5.0], [6.0, 1.0, 3.0], [2.0, 1.0, 4.0]], [[3.0, 6.0, 5.0], [2.0, 3.0, 1.0], 
[2.0, 7.0, 4.0]]], [[[6.0, 4.0, 6.0], [7.0, 7.0, 7.0], [6.0, 2.0, 6.0]], [[5.0, 4.0, 7.0], [7.0, 6.0, 1.0], [5.0, 7.0, 3.0]], 
[[5.0, 5.0, 8.0], [5.0, 6.0, 8.0], [4.0, 8.0, 2.0]]], [[[5.0, 5.0, 5.0], [3.0, 8.0, 3.0], [1.0, 7.0, 5.0]], [[7.0, 5.0, 5.0], 
[5.0, 3.0, 1.0], [8.0, 1.0, 5.0]], [[2.0, 4.0, 5.0], [2.0, 6.0, 8.0], [2.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PCPN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 8.0], [6.0, 3.0, 7.0], [1.0, 2.0, 7.0]], [[5.0, 4.0, 5.0], [6.0, 4.0, 8.0], [8.0, 4.0, 
8.0]], [[3.0, 8.0, 4.0], [8.0, 6.0, 4.0], [7.0, 3.0, 2.0]]], [[[5.0, 1.0, 1.0], [5.0, 5.0, 2.0], [5.0, 2.0, 5.0]], [[6.0, 6.0, 
1.0], [4.0, 5.0, 1.0], [8.0, 4.0, 8.0]], [[6.0, 1.0, 5.0], [3.0, 4.0, 1.0], [8.0, 6.0, 2.0]]], [[[3.0, 1.0, 4.0], [3.0, 2.0, 
3.0], [4.0, 1.0, 5.0]], [[4.0, 1.0, 3.0], [6.0, 5.0, 7.0], [5.0, 1.0, 5.0]], [[7.0, 1.0, 3.0], [5.0, 6.0, 2.0], [1.0, 4.0, 
3.0]]], [[[5.0, 2.0, 1.0], [2.0, 5.0, 6.0], [3.0, 8.0, 3.0]], [[4.0, 4.0, 4.0], [1.0, 2.0, 1.0], [6.0, 1.0, 6.0]], [[7.0, 4.0, 
6.0], [6.0, 3.0, 2.0], [7.0, 4.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 7.0], [6.0, 2.0, 4.0], [8.0, 1.0, 6.0]], [[2.0, 3.0, 5.0], [6.0, 7.0, 
4.0], [4.0, 8.0, 1.0]], [[5.0, 2.0, 7.0], [6.0, 5.0, 1.0], [1.0, 8.0, 4.0]]], [[[2.0, 3.0, 8.0], [5.0, 2.0, 3.0], [7.0, 8.0, 
4.0]], [[2.0, 3.0, 6.0], [8.0, 5.0, 6.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 4.0], [3.0, 4.0, 3.0], [8.0, 8.0, 7.0]]], [[[4.0, 7.0, 
4.0], [4.0, 7.0, 6.0], [5.0, 3.0, 6.0]], [[4.0, 8.0, 7.0], [3.0, 7.0, 8.0], [6.0, 4.0, 6.0]], [[6.0, 2.0, 8.0], [5.0, 7.0, 
3.0], [1.0, 4.0, 4.0]]], [[[8.0, 5.0, 4.0], [1.0, 6.0, 7.0], [7.0, 6.0, 6.0]], [[8.0, 2.0, 7.0], [8.0, 6.0, 2.0], [5.0, 7.0, 
2.0]], [[5.0, 6.0, 1.0], [1.0, 1.0, 7.0], [1.0, 1.0, 5.0]]]]))
      val=numpy.array([[3.0, 2.0, 5.0], [3.0, 1.0, 4.0], [2.0, 3.0, 5.0]])
      arg[2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 8.0], [6.0, 3.0, 7.0], [1.0, 2.0, 7.0]], [[5.0, 4.0, 5.0], [6.0, 4.0, 8.0], [8.0, 
4.0, 8.0]], [[3.0, 8.0, 4.0], [8.0, 6.0, 4.0], [7.0, 3.0, 2.0]]], [[[5.0, 1.0, 1.0], [5.0, 5.0, 2.0], [5.0, 2.0, 5.0]], [[6.0, 
6.0, 1.0], [4.0, 5.0, 1.0], [8.0, 4.0, 8.0]], [[6.0, 1.0, 5.0], [3.0, 4.0, 1.0], [8.0, 6.0, 2.0]]], [[[3.0, 1.0, 4.0], [3.0, 
2.0, 5.0], [4.0, 1.0, 5.0]], [[4.0, 1.0, 3.0], [3.0, 1.0, 4.0], [5.0, 1.0, 5.0]], [[7.0, 1.0, 3.0], [2.0, 3.0, 5.0], [1.0, 4.0, 
3.0]]], [[[5.0, 2.0, 1.0], [2.0, 5.0, 6.0], [3.0, 8.0, 3.0]], [[4.0, 4.0, 4.0], [1.0, 2.0, 1.0], [6.0, 1.0, 6.0]], [[7.0, 4.0, 
6.0], [6.0, 3.0, 2.0], [7.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 7.0], [6.0, 2.0, 4.0], [8.0, 1.0, 6.0]], 
[[2.0, 3.0, 5.0], [6.0, 7.0, 4.0], [4.0, 8.0, 1.0]], [[5.0, 2.0, 7.0], [6.0, 5.0, 1.0], [1.0, 8.0, 4.0]]], [[[2.0, 3.0, 8.0], 
[5.0, 2.0, 3.0], [7.0, 8.0, 4.0]], [[2.0, 3.0, 6.0], [8.0, 5.0, 6.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 4.0], [3.0, 4.0, 3.0], 
[8.0, 8.0, 7.0]]], [[[4.0, 7.0, 4.0], [3.0, 2.0, 5.0], [5.0, 3.0, 6.0]], [[4.0, 8.0, 7.0], [3.0, 1.0, 4.0], [6.0, 4.0, 6.0]], 
[[6.0, 2.0, 8.0], [2.0, 3.0, 5.0], [1.0, 4.0, 4.0]]], [[[8.0, 5.0, 4.0], [1.0, 6.0, 7.0], [7.0, 6.0, 6.0]], [[8.0, 2.0, 7.0], 
[8.0, 6.0, 2.0], [5.0, 7.0, 2.0]], [[5.0, 6.0, 1.0], [1.0, 1.0, 7.0], [1.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PCPN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 4.0], [7.0, 5.0, 7.0], [8.0, 7.0, 1.0]], [[3.0, 3.0, 3.0], [7.0, 3.0, 7.0], [6.0, 5.0, 
3.0]], [[4.0, 1.0, 3.0], [5.0, 3.0, 8.0], [5.0, 4.0, 1.0]]], [[[3.0, 8.0, 5.0], [1.0, 7.0, 4.0], [7.0, 1.0, 1.0]], [[3.0, 4.0, 
6.0], [3.0, 3.0, 5.0], [4.0, 6.0, 4.0]], [[6.0, 4.0, 4.0], [8.0, 8.0, 8.0], [5.0, 6.0, 3.0]]], [[[4.0, 2.0, 2.0], [4.0, 2.0, 
5.0], [3.0, 2.0, 5.0]], [[6.0, 7.0, 3.0], [4.0, 3.0, 6.0], [4.0, 4.0, 5.0]], [[2.0, 6.0, 1.0], [2.0, 7.0, 1.0], [4.0, 7.0, 
6.0]]], [[[7.0, 4.0, 5.0], [3.0, 2.0, 6.0], [1.0, 1.0, 1.0]], [[1.0, 1.0, 8.0], [4.0, 7.0, 5.0], [5.0, 1.0, 3.0]], [[1.0, 3.0, 
3.0], [1.0, 2.0, 8.0], [5.0, 5.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 8.0], [2.0, 3.0, 4.0], [4.0, 7.0, 5.0]], [[2.0, 3.0, 5.0], [1.0, 2.0, 
3.0], [8.0, 4.0, 7.0]], [[7.0, 7.0, 6.0], [5.0, 3.0, 1.0], [1.0, 7.0, 8.0]]], [[[2.0, 7.0, 4.0], [3.0, 1.0, 5.0], [3.0, 1.0, 
8.0]], [[6.0, 5.0, 5.0], [8.0, 7.0, 8.0], [5.0, 4.0, 2.0]], [[3.0, 2.0, 5.0], [6.0, 6.0, 7.0], [4.0, 7.0, 2.0]]], [[[4.0, 2.0, 
2.0], [5.0, 7.0, 8.0], [4.0, 3.0, 5.0]], [[2.0, 4.0, 3.0], [2.0, 4.0, 4.0], [6.0, 1.0, 7.0]], [[8.0, 2.0, 6.0], [6.0, 7.0, 
8.0], [4.0, 1.0, 8.0]]], [[[6.0, 2.0, 4.0], [7.0, 5.0, 2.0], [8.0, 7.0, 5.0]], [[6.0, 5.0, 2.0], [4.0, 6.0, 3.0], [8.0, 6.0, 
8.0]], [[1.0, 3.0, 7.0], [1.0, 6.0, 4.0], [5.0, 6.0, 7.0]]]]))
      val=Data(numpy.array([[2.0, 8.0, 1.0], [7.0, 8.0, 2.0], [5.0, 8.0, 7.0]]),self.functionspace)
      arg[2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 4.0], [7.0, 5.0, 7.0], [8.0, 7.0, 1.0]], [[3.0, 3.0, 3.0], [7.0, 3.0, 7.0], [6.0, 
5.0, 3.0]], [[4.0, 1.0, 3.0], [5.0, 3.0, 8.0], [5.0, 4.0, 1.0]]], [[[3.0, 8.0, 5.0], [1.0, 7.0, 4.0], [7.0, 1.0, 1.0]], [[3.0, 
4.0, 6.0], [3.0, 3.0, 5.0], [4.0, 6.0, 4.0]], [[6.0, 4.0, 4.0], [8.0, 8.0, 8.0], [5.0, 6.0, 3.0]]], [[[4.0, 2.0, 2.0], [2.0, 
8.0, 1.0], [3.0, 2.0, 5.0]], [[6.0, 7.0, 3.0], [7.0, 8.0, 2.0], [4.0, 4.0, 5.0]], [[2.0, 6.0, 1.0], [5.0, 8.0, 7.0], [4.0, 7.0, 
6.0]]], [[[7.0, 4.0, 5.0], [3.0, 2.0, 6.0], [1.0, 1.0, 1.0]], [[1.0, 1.0, 8.0], [4.0, 7.0, 5.0], [5.0, 1.0, 3.0]], [[1.0, 3.0, 
3.0], [1.0, 2.0, 8.0], [5.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 8.0], [2.0, 3.0, 4.0], [4.0, 7.0, 5.0]], 
[[2.0, 3.0, 5.0], [1.0, 2.0, 3.0], [8.0, 4.0, 7.0]], [[7.0, 7.0, 6.0], [5.0, 3.0, 1.0], [1.0, 7.0, 8.0]]], [[[2.0, 7.0, 4.0], 
[3.0, 1.0, 5.0], [3.0, 1.0, 8.0]], [[6.0, 5.0, 5.0], [8.0, 7.0, 8.0], [5.0, 4.0, 2.0]], [[3.0, 2.0, 5.0], [6.0, 6.0, 7.0], 
[4.0, 7.0, 2.0]]], [[[4.0, 2.0, 2.0], [2.0, 8.0, 1.0], [4.0, 3.0, 5.0]], [[2.0, 4.0, 3.0], [7.0, 8.0, 2.0], [6.0, 1.0, 7.0]], 
[[8.0, 2.0, 6.0], [5.0, 8.0, 7.0], [4.0, 1.0, 8.0]]], [[[6.0, 2.0, 4.0], [7.0, 5.0, 2.0], [8.0, 7.0, 5.0]], [[6.0, 5.0, 2.0], 
[4.0, 6.0, 3.0], [8.0, 6.0, 8.0]], [[1.0, 3.0, 7.0], [1.0, 6.0, 4.0], [5.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PCPN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 6.0], [2.0, 1.0, 4.0], [5.0, 6.0, 1.0]], [[7.0, 2.0, 4.0], [8.0, 7.0, 4.0], [8.0, 5.0, 
5.0]], [[8.0, 8.0, 8.0], [4.0, 7.0, 3.0], [8.0, 8.0, 4.0]]], [[[5.0, 6.0, 8.0], [6.0, 4.0, 3.0], [5.0, 5.0, 1.0]], [[4.0, 1.0, 
1.0], [5.0, 7.0, 8.0], [8.0, 8.0, 1.0]], [[4.0, 7.0, 2.0], [3.0, 7.0, 2.0], [2.0, 1.0, 6.0]]], [[[1.0, 5.0, 7.0], [8.0, 2.0, 
1.0], [2.0, 5.0, 4.0]], [[4.0, 3.0, 2.0], [5.0, 2.0, 6.0], [3.0, 8.0, 5.0]], [[8.0, 2.0, 6.0], [6.0, 2.0, 5.0], [8.0, 2.0, 
6.0]]], [[[7.0, 4.0, 6.0], [6.0, 2.0, 5.0], [2.0, 5.0, 8.0]], [[6.0, 3.0, 1.0], [7.0, 3.0, 2.0], [7.0, 5.0, 3.0]], [[6.0, 3.0, 
5.0], [2.0, 6.0, 1.0], [2.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 5.0], [5.0, 3.0, 3.0], [6.0, 1.0, 7.0]], [[6.0, 2.0, 5.0], [2.0, 5.0, 
8.0], [6.0, 5.0, 2.0]], [[2.0, 4.0, 7.0], [1.0, 2.0, 4.0], [8.0, 1.0, 4.0]]], [[[1.0, 5.0, 4.0], [1.0, 4.0, 8.0], [6.0, 1.0, 
2.0]], [[6.0, 2.0, 6.0], [4.0, 4.0, 3.0], [3.0, 8.0, 2.0]], [[7.0, 2.0, 6.0], [4.0, 5.0, 2.0], [6.0, 5.0, 8.0]]], [[[6.0, 6.0, 
5.0], [6.0, 1.0, 8.0], [8.0, 7.0, 3.0]], [[5.0, 6.0, 3.0], [2.0, 3.0, 2.0], [6.0, 4.0, 3.0]], [[3.0, 1.0, 2.0], [6.0, 2.0, 
4.0], [5.0, 5.0, 6.0]]], [[[7.0, 3.0, 6.0], [4.0, 2.0, 4.0], [5.0, 1.0, 5.0]], [[3.0, 4.0, 8.0], [1.0, 7.0, 8.0], [4.0, 8.0, 
4.0]], [[2.0, 3.0, 6.0], [6.0, 6.0, 2.0], [5.0, 5.0, 5.0]]]]))
      val=Data(numpy.array([[8.0, 6.0, 6.0], [5.0, 6.0, 1.0], [5.0, 2.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 1.0, 2.0], [2.0, 5.0, 5.0], [2.0, 4.0, 5.0]]))
      arg[2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [2.0, 1.0, 4.0], [5.0, 6.0, 1.0]], [[7.0, 2.0, 4.0], [8.0, 7.0, 4.0], [8.0, 
5.0, 5.0]], [[8.0, 8.0, 8.0], [4.0, 7.0, 3.0], [8.0, 8.0, 4.0]]], [[[5.0, 6.0, 8.0], [6.0, 4.0, 3.0], [5.0, 5.0, 1.0]], [[4.0, 
1.0, 1.0], [5.0, 7.0, 8.0], [8.0, 8.0, 1.0]], [[4.0, 7.0, 2.0], [3.0, 7.0, 2.0], [2.0, 1.0, 6.0]]], [[[1.0, 5.0, 7.0], [8.0, 
6.0, 6.0], [2.0, 5.0, 4.0]], [[4.0, 3.0, 2.0], [5.0, 6.0, 1.0], [3.0, 8.0, 5.0]], [[8.0, 2.0, 6.0], [5.0, 2.0, 7.0], [8.0, 2.0, 
6.0]]], [[[7.0, 4.0, 6.0], [6.0, 2.0, 5.0], [2.0, 5.0, 8.0]], [[6.0, 3.0, 1.0], [7.0, 3.0, 2.0], [7.0, 5.0, 3.0]], [[6.0, 3.0, 
5.0], [2.0, 6.0, 1.0], [2.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 5.0], [5.0, 3.0, 3.0], [6.0, 1.0, 7.0]], 
[[6.0, 2.0, 5.0], [2.0, 5.0, 8.0], [6.0, 5.0, 2.0]], [[2.0, 4.0, 7.0], [1.0, 2.0, 4.0], [8.0, 1.0, 4.0]]], [[[1.0, 5.0, 4.0], 
[1.0, 4.0, 8.0], [6.0, 1.0, 2.0]], [[6.0, 2.0, 6.0], [4.0, 4.0, 3.0], [3.0, 8.0, 2.0]], [[7.0, 2.0, 6.0], [4.0, 5.0, 2.0], 
[6.0, 5.0, 8.0]]], [[[6.0, 6.0, 5.0], [3.0, 1.0, 2.0], [8.0, 7.0, 3.0]], [[5.0, 6.0, 3.0], [2.0, 5.0, 5.0], [6.0, 4.0, 3.0]], 
[[3.0, 1.0, 2.0], [2.0, 4.0, 5.0], [5.0, 5.0, 6.0]]], [[[7.0, 3.0, 6.0], [4.0, 2.0, 4.0], [5.0, 1.0, 5.0]], [[3.0, 4.0, 8.0], 
[1.0, 7.0, 8.0], [4.0, 8.0, 4.0]], [[2.0, 3.0, 6.0], [6.0, 6.0, 2.0], [5.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PCPN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 5.0], [1.0, 6.0, 7.0], [8.0, 8.0, 8.0]], [[8.0, 8.0, 8.0], [5.0, 8.0, 1.0], [7.0, 8.0, 
2.0]], [[7.0, 6.0, 1.0], [6.0, 5.0, 1.0], [1.0, 2.0, 1.0]]], [[[7.0, 2.0, 7.0], [3.0, 7.0, 7.0], [6.0, 5.0, 8.0]], [[8.0, 1.0, 
6.0], [1.0, 1.0, 7.0], [1.0, 3.0, 5.0]], [[2.0, 7.0, 3.0], [8.0, 2.0, 5.0], [1.0, 8.0, 6.0]]], [[[7.0, 4.0, 5.0], [2.0, 3.0, 
8.0], [6.0, 2.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 6.0, 2.0], [2.0, 8.0, 8.0]], [[3.0, 5.0, 5.0], [6.0, 4.0, 4.0], [3.0, 4.0, 
1.0]]], [[[3.0, 2.0, 8.0], [7.0, 5.0, 6.0], [8.0, 8.0, 8.0]], [[3.0, 8.0, 3.0], [6.0, 7.0, 7.0], [1.0, 8.0, 7.0]], [[5.0, 7.0, 
1.0], [1.0, 8.0, 8.0], [5.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 5.0], [4.0, 7.0, 8.0], [4.0, 8.0, 5.0]], [[3.0, 4.0, 5.0], [7.0, 5.0, 
6.0], [2.0, 7.0, 3.0]], [[3.0, 1.0, 7.0], [2.0, 8.0, 3.0], [6.0, 4.0, 3.0]]], [[[4.0, 1.0, 1.0], [1.0, 2.0, 4.0], [4.0, 4.0, 
5.0]], [[2.0, 6.0, 2.0], [8.0, 6.0, 1.0], [2.0, 3.0, 1.0]], [[2.0, 3.0, 1.0], [6.0, 3.0, 7.0], [5.0, 8.0, 1.0]]], [[[4.0, 8.0, 
8.0], [5.0, 6.0, 8.0], [1.0, 6.0, 8.0]], [[1.0, 1.0, 1.0], [2.0, 1.0, 5.0], [2.0, 2.0, 4.0]], [[5.0, 4.0, 3.0], [5.0, 8.0, 
2.0], [4.0, 8.0, 8.0]]], [[[2.0, 8.0, 4.0], [5.0, 6.0, 5.0], [6.0, 2.0, 3.0]], [[5.0, 2.0, 2.0], [8.0, 5.0, 1.0], [7.0, 8.0, 
7.0]], [[5.0, 3.0, 8.0], [5.0, 6.0, 3.0], [1.0, 6.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[3.0, 5.0, 1.0], [4.0, 1.0, 6.0], [2.0, 8.0, 7.0]])+(1.-msk_val)*numpy.array([[7.0, 5.0, 7.0], 
[7.0, 6.0, 7.0], [6.0, 2.0, 8.0]])
      arg[2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 5.0], [1.0, 6.0, 7.0], [8.0, 8.0, 8.0]], [[8.0, 8.0, 8.0], [5.0, 8.0, 1.0], [7.0, 
8.0, 2.0]], [[7.0, 6.0, 1.0], [6.0, 5.0, 1.0], [1.0, 2.0, 1.0]]], [[[7.0, 2.0, 7.0], [3.0, 7.0, 7.0], [6.0, 5.0, 8.0]], [[8.0, 
1.0, 6.0], [1.0, 1.0, 7.0], [1.0, 3.0, 5.0]], [[2.0, 7.0, 3.0], [8.0, 2.0, 5.0], [1.0, 8.0, 6.0]]], [[[7.0, 4.0, 5.0], [3.0, 
5.0, 1.0], [6.0, 2.0, 1.0]], [[3.0, 5.0, 1.0], [4.0, 1.0, 6.0], [2.0, 8.0, 8.0]], [[3.0, 5.0, 5.0], [2.0, 8.0, 7.0], [3.0, 4.0, 
1.0]]], [[[3.0, 2.0, 8.0], [7.0, 5.0, 6.0], [8.0, 8.0, 8.0]], [[3.0, 8.0, 3.0], [6.0, 7.0, 7.0], [1.0, 8.0, 7.0]], [[5.0, 7.0, 
1.0], [1.0, 8.0, 8.0], [5.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 5.0], [4.0, 7.0, 8.0], [4.0, 8.0, 5.0]], 
[[3.0, 4.0, 5.0], [7.0, 5.0, 6.0], [2.0, 7.0, 3.0]], [[3.0, 1.0, 7.0], [2.0, 8.0, 3.0], [6.0, 4.0, 3.0]]], [[[4.0, 1.0, 1.0], 
[1.0, 2.0, 4.0], [4.0, 4.0, 5.0]], [[2.0, 6.0, 2.0], [8.0, 6.0, 1.0], [2.0, 3.0, 1.0]], [[2.0, 3.0, 1.0], [6.0, 3.0, 7.0], 
[5.0, 8.0, 1.0]]], [[[4.0, 8.0, 8.0], [7.0, 5.0, 7.0], [1.0, 6.0, 8.0]], [[1.0, 1.0, 1.0], [7.0, 6.0, 7.0], [2.0, 2.0, 4.0]], 
[[5.0, 4.0, 3.0], [6.0, 2.0, 8.0], [4.0, 8.0, 8.0]]], [[[2.0, 8.0, 4.0], [5.0, 6.0, 5.0], [6.0, 2.0, 3.0]], [[5.0, 2.0, 2.0], 
[8.0, 5.0, 1.0], [7.0, 8.0, 7.0]], [[5.0, 3.0, 8.0], [5.0, 6.0, 3.0], [1.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LRUN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 4.0], [2.0, 1.0, 8.0], [7.0, 6.0, 2.0]], [[5.0, 2.0, 5.0], [7.0, 7.0, 7.0], [2.0, 7.0, 
6.0]], [[7.0, 4.0, 7.0], [7.0, 2.0, 4.0], [7.0, 7.0, 4.0]]], [[[2.0, 6.0, 2.0], [7.0, 4.0, 6.0], [6.0, 3.0, 8.0]], [[3.0, 2.0, 
5.0], [6.0, 4.0, 1.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 1.0], [6.0, 8.0, 4.0], [6.0, 3.0, 1.0]]], [[[3.0, 3.0, 1.0], [3.0, 5.0, 
7.0], [4.0, 6.0, 8.0]], [[6.0, 2.0, 5.0], [6.0, 7.0, 7.0], [3.0, 1.0, 3.0]], [[5.0, 3.0, 2.0], [2.0, 3.0, 7.0], [5.0, 2.0, 
5.0]]], [[[3.0, 4.0, 4.0], [4.0, 2.0, 3.0], [2.0, 4.0, 6.0]], [[5.0, 3.0, 1.0], [1.0, 4.0, 1.0], [4.0, 3.0, 3.0]], [[8.0, 7.0, 
2.0], [5.0, 2.0, 6.0], [7.0, 6.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 6.0], [4.0, 7.0, 3.0], [6.0, 3.0, 5.0]], [[2.0, 1.0, 4.0], [5.0, 1.0, 
7.0], [8.0, 4.0, 7.0]], [[4.0, 1.0, 4.0], [8.0, 3.0, 1.0], [1.0, 3.0, 6.0]]], [[[7.0, 6.0, 7.0], [1.0, 6.0, 7.0], [7.0, 2.0, 
8.0]], [[1.0, 7.0, 3.0], [7.0, 3.0, 5.0], [6.0, 6.0, 3.0]], [[5.0, 5.0, 3.0], [1.0, 6.0, 7.0], [1.0, 3.0, 5.0]]], [[[6.0, 4.0, 
8.0], [7.0, 6.0, 2.0], [7.0, 3.0, 6.0]], [[1.0, 8.0, 3.0], [1.0, 1.0, 4.0], [1.0, 3.0, 4.0]], [[6.0, 5.0, 2.0], [8.0, 5.0, 
1.0], [7.0, 8.0, 6.0]]], [[[4.0, 2.0, 6.0], [1.0, 3.0, 6.0], [7.0, 1.0, 5.0]], [[3.0, 1.0, 7.0], [4.0, 6.0, 4.0], [3.0, 1.0, 
7.0]], [[1.0, 8.0, 3.0], [8.0, 5.0, 6.0], [5.0, 8.0, 1.0]]]]))
      val=numpy.array([[[[3.0, 5.0, 2.0], [4.0, 1.0, 7.0]]], [[[7.0, 1.0, 5.0], [2.0, 6.0, 4.0]]]])
      arg[2:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 4.0], [2.0, 1.0, 8.0], [7.0, 6.0, 2.0]], [[5.0, 2.0, 5.0], [7.0, 7.0, 7.0], [2.0, 
7.0, 6.0]], [[7.0, 4.0, 7.0], [7.0, 2.0, 4.0], [7.0, 7.0, 4.0]]], [[[2.0, 6.0, 2.0], [7.0, 4.0, 6.0], [6.0, 3.0, 8.0]], [[3.0, 
2.0, 5.0], [6.0, 4.0, 1.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 1.0], [6.0, 8.0, 4.0], [6.0, 3.0, 1.0]]], [[[3.0, 3.0, 1.0], [3.0, 
5.0, 7.0], [4.0, 6.0, 8.0]], [[3.0, 5.0, 2.0], [4.0, 1.0, 7.0], [3.0, 1.0, 3.0]], [[5.0, 3.0, 2.0], [2.0, 3.0, 7.0], [5.0, 2.0, 
5.0]]], [[[3.0, 4.0, 4.0], [4.0, 2.0, 3.0], [2.0, 4.0, 6.0]], [[7.0, 1.0, 5.0], [2.0, 6.0, 4.0], [4.0, 3.0, 3.0]], [[8.0, 7.0, 
2.0], [5.0, 2.0, 6.0], [7.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 6.0], [4.0, 7.0, 3.0], [6.0, 3.0, 5.0]], 
[[2.0, 1.0, 4.0], [5.0, 1.0, 7.0], [8.0, 4.0, 7.0]], [[4.0, 1.0, 4.0], [8.0, 3.0, 1.0], [1.0, 3.0, 6.0]]], [[[7.0, 6.0, 7.0], 
[1.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[1.0, 7.0, 3.0], [7.0, 3.0, 5.0], [6.0, 6.0, 3.0]], [[5.0, 5.0, 3.0], [1.0, 6.0, 7.0], 
[1.0, 3.0, 5.0]]], [[[6.0, 4.0, 8.0], [7.0, 6.0, 2.0], [7.0, 3.0, 6.0]], [[3.0, 5.0, 2.0], [4.0, 1.0, 7.0], [1.0, 3.0, 4.0]], 
[[6.0, 5.0, 2.0], [8.0, 5.0, 1.0], [7.0, 8.0, 6.0]]], [[[4.0, 2.0, 6.0], [1.0, 3.0, 6.0], [7.0, 1.0, 5.0]], [[7.0, 1.0, 5.0], 
[2.0, 6.0, 4.0], [3.0, 1.0, 7.0]], [[1.0, 8.0, 3.0], [8.0, 5.0, 6.0], [5.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LRUN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 7.0], [3.0, 6.0, 7.0], [1.0, 6.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 2.0, 5.0], [3.0, 6.0, 
8.0]], [[1.0, 3.0, 6.0], [5.0, 7.0, 6.0], [5.0, 2.0, 7.0]]], [[[3.0, 8.0, 2.0], [4.0, 7.0, 4.0], [8.0, 7.0, 5.0]], [[3.0, 2.0, 
2.0], [8.0, 6.0, 6.0], [1.0, 2.0, 4.0]], [[1.0, 7.0, 6.0], [2.0, 2.0, 3.0], [3.0, 3.0, 8.0]]], [[[2.0, 1.0, 1.0], [3.0, 6.0, 
4.0], [5.0, 2.0, 5.0]], [[6.0, 2.0, 3.0], [7.0, 6.0, 1.0], [3.0, 4.0, 8.0]], [[2.0, 2.0, 2.0], [5.0, 7.0, 7.0], [4.0, 2.0, 
1.0]]], [[[5.0, 6.0, 5.0], [5.0, 3.0, 8.0], [6.0, 6.0, 3.0]], [[4.0, 8.0, 1.0], [3.0, 7.0, 1.0], [3.0, 7.0, 1.0]], [[1.0, 8.0, 
3.0], [6.0, 6.0, 7.0], [8.0, 3.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 7.0], [2.0, 7.0, 6.0], [8.0, 7.0, 8.0]], [[5.0, 8.0, 1.0], [7.0, 6.0, 
7.0], [7.0, 8.0, 1.0]], [[5.0, 7.0, 5.0], [8.0, 7.0, 6.0], [1.0, 7.0, 6.0]]], [[[3.0, 4.0, 4.0], [1.0, 2.0, 5.0], [5.0, 1.0, 
7.0]], [[2.0, 7.0, 8.0], [5.0, 6.0, 7.0], [7.0, 1.0, 8.0]], [[3.0, 1.0, 8.0], [7.0, 3.0, 8.0], [6.0, 4.0, 7.0]]], [[[2.0, 3.0, 
2.0], [4.0, 7.0, 7.0], [4.0, 7.0, 6.0]], [[5.0, 6.0, 2.0], [2.0, 1.0, 5.0], [4.0, 1.0, 7.0]], [[7.0, 8.0, 4.0], [8.0, 8.0, 
3.0], [3.0, 8.0, 2.0]]], [[[5.0, 3.0, 6.0], [7.0, 6.0, 8.0], [3.0, 3.0, 3.0]], [[3.0, 1.0, 3.0], [5.0, 3.0, 3.0], [7.0, 3.0, 
4.0]], [[2.0, 5.0, 3.0], [6.0, 1.0, 3.0], [4.0, 3.0, 6.0]]]]))
      val=Data(numpy.array([[[[4.0, 7.0, 8.0], [4.0, 4.0, 3.0]]], [[[2.0, 4.0, 1.0], [5.0, 5.0, 7.0]]]]),self.functionspace)
      arg[2:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 7.0], [3.0, 6.0, 7.0], [1.0, 6.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 2.0, 5.0], [3.0, 
6.0, 8.0]], [[1.0, 3.0, 6.0], [5.0, 7.0, 6.0], [5.0, 2.0, 7.0]]], [[[3.0, 8.0, 2.0], [4.0, 7.0, 4.0], [8.0, 7.0, 5.0]], [[3.0, 
2.0, 2.0], [8.0, 6.0, 6.0], [1.0, 2.0, 4.0]], [[1.0, 7.0, 6.0], [2.0, 2.0, 3.0], [3.0, 3.0, 8.0]]], [[[2.0, 1.0, 1.0], [3.0, 
6.0, 4.0], [5.0, 2.0, 5.0]], [[4.0, 7.0, 8.0], [4.0, 4.0, 3.0], [3.0, 4.0, 8.0]], [[2.0, 2.0, 2.0], [5.0, 7.0, 7.0], [4.0, 2.0, 
1.0]]], [[[5.0, 6.0, 5.0], [5.0, 3.0, 8.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 1.0], [5.0, 5.0, 7.0], [3.0, 7.0, 1.0]], [[1.0, 8.0, 
3.0], [6.0, 6.0, 7.0], [8.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 7.0], [2.0, 7.0, 6.0], [8.0, 7.0, 8.0]], 
[[5.0, 8.0, 1.0], [7.0, 6.0, 7.0], [7.0, 8.0, 1.0]], [[5.0, 7.0, 5.0], [8.0, 7.0, 6.0], [1.0, 7.0, 6.0]]], [[[3.0, 4.0, 4.0], 
[1.0, 2.0, 5.0], [5.0, 1.0, 7.0]], [[2.0, 7.0, 8.0], [5.0, 6.0, 7.0], [7.0, 1.0, 8.0]], [[3.0, 1.0, 8.0], [7.0, 3.0, 8.0], 
[6.0, 4.0, 7.0]]], [[[2.0, 3.0, 2.0], [4.0, 7.0, 7.0], [4.0, 7.0, 6.0]], [[4.0, 7.0, 8.0], [4.0, 4.0, 3.0], [4.0, 1.0, 7.0]], 
[[7.0, 8.0, 4.0], [8.0, 8.0, 3.0], [3.0, 8.0, 2.0]]], [[[5.0, 3.0, 6.0], [7.0, 6.0, 8.0], [3.0, 3.0, 3.0]], [[2.0, 4.0, 1.0], 
[5.0, 5.0, 7.0], [7.0, 3.0, 4.0]], [[2.0, 5.0, 3.0], [6.0, 1.0, 3.0], [4.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LRUN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 6.0], [6.0, 5.0, 2.0], [7.0, 3.0, 3.0]], [[7.0, 7.0, 1.0], [6.0, 7.0, 4.0], [1.0, 3.0, 
1.0]], [[4.0, 8.0, 8.0], [8.0, 4.0, 7.0], [7.0, 5.0, 7.0]]], [[[1.0, 1.0, 4.0], [5.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[1.0, 3.0, 
7.0], [1.0, 4.0, 6.0], [2.0, 1.0, 1.0]], [[7.0, 1.0, 6.0], [1.0, 5.0, 2.0], [4.0, 1.0, 1.0]]], [[[6.0, 1.0, 1.0], [7.0, 1.0, 
6.0], [8.0, 5.0, 2.0]], [[4.0, 8.0, 8.0], [1.0, 5.0, 5.0], [1.0, 6.0, 5.0]], [[8.0, 5.0, 2.0], [1.0, 4.0, 1.0], [7.0, 7.0, 
6.0]]], [[[5.0, 5.0, 7.0], [3.0, 3.0, 3.0], [1.0, 6.0, 8.0]], [[7.0, 3.0, 5.0], [5.0, 2.0, 8.0], [8.0, 5.0, 3.0]], [[3.0, 1.0, 
7.0], [5.0, 4.0, 3.0], [2.0, 1.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 5.0], [1.0, 1.0, 6.0], [4.0, 1.0, 5.0]], [[1.0, 3.0, 6.0], [1.0, 3.0, 
1.0], [2.0, 7.0, 6.0]], [[2.0, 7.0, 2.0], [2.0, 7.0, 1.0], [2.0, 5.0, 6.0]]], [[[5.0, 3.0, 4.0], [2.0, 2.0, 1.0], [4.0, 5.0, 
2.0]], [[8.0, 7.0, 1.0], [6.0, 2.0, 8.0], [6.0, 2.0, 1.0]], [[1.0, 4.0, 4.0], [4.0, 3.0, 4.0], [5.0, 4.0, 7.0]]], [[[6.0, 4.0, 
1.0], [7.0, 3.0, 6.0], [8.0, 4.0, 2.0]], [[7.0, 7.0, 4.0], [1.0, 1.0, 3.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 7.0], [2.0, 5.0, 
2.0], [4.0, 5.0, 7.0]]], [[[3.0, 4.0, 5.0], [4.0, 3.0, 3.0], [6.0, 6.0, 2.0]], [[4.0, 8.0, 8.0], [6.0, 2.0, 5.0], [8.0, 5.0, 
8.0]], [[2.0, 3.0, 8.0], [6.0, 4.0, 8.0], [1.0, 4.0, 6.0]]]]))
      val=Data(numpy.array([[[[6.0, 6.0, 1.0], [1.0, 6.0, 6.0]]], [[[8.0, 1.0, 8.0], [6.0, 4.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 4.0], [6.0, 8.0, 7.0]]], [[[3.0, 3.0, 6.0], [1.0, 3.0, 4.0]]]]))
      arg[2:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 6.0], [6.0, 5.0, 2.0], [7.0, 3.0, 3.0]], [[7.0, 7.0, 1.0], [6.0, 7.0, 4.0], [1.0, 
3.0, 1.0]], [[4.0, 8.0, 8.0], [8.0, 4.0, 7.0], [7.0, 5.0, 7.0]]], [[[1.0, 1.0, 4.0], [5.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[1.0, 
3.0, 7.0], [1.0, 4.0, 6.0], [2.0, 1.0, 1.0]], [[7.0, 1.0, 6.0], [1.0, 5.0, 2.0], [4.0, 1.0, 1.0]]], [[[6.0, 1.0, 1.0], [7.0, 
1.0, 6.0], [8.0, 5.0, 2.0]], [[6.0, 6.0, 1.0], [1.0, 6.0, 6.0], [1.0, 6.0, 5.0]], [[8.0, 5.0, 2.0], [1.0, 4.0, 1.0], [7.0, 7.0, 
6.0]]], [[[5.0, 5.0, 7.0], [3.0, 3.0, 3.0], [1.0, 6.0, 8.0]], [[8.0, 1.0, 8.0], [6.0, 4.0, 5.0], [8.0, 5.0, 3.0]], [[3.0, 1.0, 
7.0], [5.0, 4.0, 3.0], [2.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 5.0], [1.0, 1.0, 6.0], [4.0, 1.0, 5.0]], 
[[1.0, 3.0, 6.0], [1.0, 3.0, 1.0], [2.0, 7.0, 6.0]], [[2.0, 7.0, 2.0], [2.0, 7.0, 1.0], [2.0, 5.0, 6.0]]], [[[5.0, 3.0, 4.0], 
[2.0, 2.0, 1.0], [4.0, 5.0, 2.0]], [[8.0, 7.0, 1.0], [6.0, 2.0, 8.0], [6.0, 2.0, 1.0]], [[1.0, 4.0, 4.0], [4.0, 3.0, 4.0], 
[5.0, 4.0, 7.0]]], [[[6.0, 4.0, 1.0], [7.0, 3.0, 6.0], [8.0, 4.0, 2.0]], [[3.0, 5.0, 4.0], [6.0, 8.0, 7.0], [3.0, 7.0, 4.0]], 
[[2.0, 5.0, 7.0], [2.0, 5.0, 2.0], [4.0, 5.0, 7.0]]], [[[3.0, 4.0, 5.0], [4.0, 3.0, 3.0], [6.0, 6.0, 2.0]], [[3.0, 3.0, 6.0], 
[1.0, 3.0, 4.0], [8.0, 5.0, 8.0]], [[2.0, 3.0, 8.0], [6.0, 4.0, 8.0], [1.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LRUN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 8.0], [5.0, 5.0, 1.0], [6.0, 5.0, 4.0]], [[5.0, 7.0, 5.0], [1.0, 5.0, 1.0], [4.0, 5.0, 
8.0]], [[3.0, 5.0, 5.0], [6.0, 2.0, 5.0], [6.0, 5.0, 4.0]]], [[[6.0, 1.0, 8.0], [8.0, 7.0, 4.0], [6.0, 6.0, 8.0]], [[3.0, 2.0, 
3.0], [5.0, 3.0, 2.0], [1.0, 4.0, 4.0]], [[8.0, 5.0, 4.0], [1.0, 3.0, 3.0], [5.0, 6.0, 5.0]]], [[[6.0, 3.0, 6.0], [8.0, 8.0, 
1.0], [5.0, 4.0, 7.0]], [[3.0, 2.0, 1.0], [7.0, 6.0, 8.0], [7.0, 6.0, 5.0]], [[2.0, 1.0, 4.0], [6.0, 7.0, 2.0], [7.0, 5.0, 
7.0]]], [[[6.0, 4.0, 6.0], [8.0, 4.0, 8.0], [6.0, 8.0, 4.0]], [[4.0, 2.0, 6.0], [3.0, 6.0, 4.0], [1.0, 5.0, 6.0]], [[7.0, 2.0, 
5.0], [4.0, 5.0, 2.0], [8.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 2.0], [8.0, 5.0, 4.0], [8.0, 6.0, 2.0]], [[3.0, 6.0, 7.0], [8.0, 1.0, 
5.0], [2.0, 2.0, 3.0]], [[1.0, 3.0, 8.0], [7.0, 6.0, 3.0], [6.0, 8.0, 8.0]]], [[[1.0, 3.0, 3.0], [5.0, 6.0, 5.0], [3.0, 5.0, 
4.0]], [[4.0, 4.0, 6.0], [1.0, 8.0, 3.0], [1.0, 5.0, 7.0]], [[5.0, 4.0, 3.0], [7.0, 3.0, 2.0], [2.0, 8.0, 4.0]]], [[[1.0, 6.0, 
8.0], [2.0, 7.0, 6.0], [7.0, 6.0, 7.0]], [[4.0, 3.0, 1.0], [3.0, 8.0, 4.0], [1.0, 1.0, 6.0]], [[7.0, 3.0, 2.0], [7.0, 4.0, 
5.0], [4.0, 1.0, 8.0]]], [[[3.0, 4.0, 6.0], [4.0, 2.0, 5.0], [3.0, 5.0, 7.0]], [[5.0, 3.0, 1.0], [7.0, 5.0, 4.0], [7.0, 7.0, 
8.0]], [[2.0, 5.0, 6.0], [4.0, 8.0, 7.0], [8.0, 3.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 5.0, 2.0], [1.0, 3.0, 3.0]]], [[[3.0, 4.0, 1.0], [2.0, 3.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 4.0, 8.0], [2.0, 4.0, 3.0]]], [[[1.0, 3.0, 8.0], [3.0, 6.0, 7.0]]]])
      arg[2:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 8.0], [5.0, 5.0, 1.0], [6.0, 5.0, 4.0]], [[5.0, 7.0, 5.0], [1.0, 5.0, 1.0], [4.0, 
5.0, 8.0]], [[3.0, 5.0, 5.0], [6.0, 2.0, 5.0], [6.0, 5.0, 4.0]]], [[[6.0, 1.0, 8.0], [8.0, 7.0, 4.0], [6.0, 6.0, 8.0]], [[3.0, 
2.0, 3.0], [5.0, 3.0, 2.0], [1.0, 4.0, 4.0]], [[8.0, 5.0, 4.0], [1.0, 3.0, 3.0], [5.0, 6.0, 5.0]]], [[[6.0, 3.0, 6.0], [8.0, 
8.0, 1.0], [5.0, 4.0, 7.0]], [[5.0, 5.0, 2.0], [1.0, 3.0, 3.0], [7.0, 6.0, 5.0]], [[2.0, 1.0, 4.0], [6.0, 7.0, 2.0], [7.0, 5.0, 
7.0]]], [[[6.0, 4.0, 6.0], [8.0, 4.0, 8.0], [6.0, 8.0, 4.0]], [[3.0, 4.0, 1.0], [2.0, 3.0, 2.0], [1.0, 5.0, 6.0]], [[7.0, 2.0, 
5.0], [4.0, 5.0, 2.0], [8.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 2.0], [8.0, 5.0, 4.0], [8.0, 6.0, 2.0]], 
[[3.0, 6.0, 7.0], [8.0, 1.0, 5.0], [2.0, 2.0, 3.0]], [[1.0, 3.0, 8.0], [7.0, 6.0, 3.0], [6.0, 8.0, 8.0]]], [[[1.0, 3.0, 3.0], 
[5.0, 6.0, 5.0], [3.0, 5.0, 4.0]], [[4.0, 4.0, 6.0], [1.0, 8.0, 3.0], [1.0, 5.0, 7.0]], [[5.0, 4.0, 3.0], [7.0, 3.0, 2.0], 
[2.0, 8.0, 4.0]]], [[[1.0, 6.0, 8.0], [2.0, 7.0, 6.0], [7.0, 6.0, 7.0]], [[4.0, 4.0, 8.0], [2.0, 4.0, 3.0], [1.0, 1.0, 6.0]], 
[[7.0, 3.0, 2.0], [7.0, 4.0, 5.0], [4.0, 1.0, 8.0]]], [[[3.0, 4.0, 6.0], [4.0, 2.0, 5.0], [3.0, 5.0, 7.0]], [[1.0, 3.0, 8.0], 
[3.0, 6.0, 7.0], [7.0, 7.0, 8.0]], [[2.0, 5.0, 6.0], [4.0, 8.0, 7.0], [8.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PPRN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 3.0], [8.0, 1.0, 1.0], [4.0, 6.0, 7.0]], [[8.0, 7.0, 1.0], [2.0, 2.0, 6.0], [2.0, 4.0, 
8.0]], [[7.0, 4.0, 2.0], [2.0, 4.0, 4.0], [7.0, 8.0, 1.0]]], [[[8.0, 6.0, 8.0], [7.0, 1.0, 8.0], [6.0, 2.0, 7.0]], [[5.0, 8.0, 
7.0], [2.0, 5.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 7.0, 1.0], [5.0, 7.0, 1.0], [4.0, 6.0, 3.0]]], [[[4.0, 6.0, 7.0], [7.0, 4.0, 
7.0], [5.0, 7.0, 8.0]], [[5.0, 5.0, 7.0], [3.0, 3.0, 7.0], [6.0, 8.0, 2.0]], [[8.0, 5.0, 5.0], [5.0, 4.0, 1.0], [3.0, 2.0, 
4.0]]], [[[7.0, 3.0, 2.0], [3.0, 4.0, 3.0], [4.0, 3.0, 6.0]], [[8.0, 7.0, 7.0], [8.0, 3.0, 5.0], [2.0, 7.0, 7.0]], [[7.0, 5.0, 
6.0], [2.0, 6.0, 6.0], [2.0, 2.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 7.0], [8.0, 8.0, 5.0], [4.0, 3.0, 2.0]], [[3.0, 7.0, 6.0], [3.0, 1.0, 
1.0], [3.0, 7.0, 4.0]], [[5.0, 7.0, 2.0], [5.0, 8.0, 3.0], [2.0, 2.0, 6.0]]], [[[8.0, 8.0, 6.0], [5.0, 1.0, 3.0], [1.0, 1.0, 
1.0]], [[6.0, 1.0, 3.0], [2.0, 4.0, 1.0], [2.0, 3.0, 4.0]], [[6.0, 1.0, 8.0], [8.0, 1.0, 2.0], [8.0, 7.0, 6.0]]], [[[4.0, 6.0, 
6.0], [1.0, 3.0, 6.0], [8.0, 4.0, 8.0]], [[5.0, 1.0, 1.0], [8.0, 5.0, 6.0], [3.0, 8.0, 3.0]], [[2.0, 3.0, 2.0], [2.0, 4.0, 
6.0], [7.0, 5.0, 1.0]]], [[[6.0, 1.0, 4.0], [5.0, 8.0, 7.0], [6.0, 7.0, 6.0]], [[8.0, 4.0, 2.0], [6.0, 8.0, 4.0], [8.0, 3.0, 
8.0]], [[6.0, 6.0, 7.0], [7.0, 5.0, 2.0], [7.0, 4.0, 6.0]]]]))
      val=numpy.array([[6.0, 8.0, 5.0]])
      arg[2,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 3.0], [8.0, 1.0, 1.0], [4.0, 6.0, 7.0]], [[8.0, 7.0, 1.0], [2.0, 2.0, 6.0], [2.0, 
4.0, 8.0]], [[7.0, 4.0, 2.0], [2.0, 4.0, 4.0], [7.0, 8.0, 1.0]]], [[[8.0, 6.0, 8.0], [7.0, 1.0, 8.0], [6.0, 2.0, 7.0]], [[5.0, 
8.0, 7.0], [2.0, 5.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 7.0, 1.0], [5.0, 7.0, 1.0], [4.0, 6.0, 3.0]]], [[[4.0, 6.0, 7.0], [7.0, 
4.0, 7.0], [5.0, 7.0, 8.0]], [[5.0, 5.0, 7.0], [6.0, 8.0, 5.0], [6.0, 8.0, 2.0]], [[8.0, 5.0, 5.0], [5.0, 4.0, 1.0], [3.0, 2.0, 
4.0]]], [[[7.0, 3.0, 2.0], [3.0, 4.0, 3.0], [4.0, 3.0, 6.0]], [[8.0, 7.0, 7.0], [8.0, 3.0, 5.0], [2.0, 7.0, 7.0]], [[7.0, 5.0, 
6.0], [2.0, 6.0, 6.0], [2.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 7.0], [8.0, 8.0, 5.0], [4.0, 3.0, 2.0]], 
[[3.0, 7.0, 6.0], [3.0, 1.0, 1.0], [3.0, 7.0, 4.0]], [[5.0, 7.0, 2.0], [5.0, 8.0, 3.0], [2.0, 2.0, 6.0]]], [[[8.0, 8.0, 6.0], 
[5.0, 1.0, 3.0], [1.0, 1.0, 1.0]], [[6.0, 1.0, 3.0], [2.0, 4.0, 1.0], [2.0, 3.0, 4.0]], [[6.0, 1.0, 8.0], [8.0, 1.0, 2.0], 
[8.0, 7.0, 6.0]]], [[[4.0, 6.0, 6.0], [1.0, 3.0, 6.0], [8.0, 4.0, 8.0]], [[5.0, 1.0, 1.0], [6.0, 8.0, 5.0], [3.0, 8.0, 3.0]], 
[[2.0, 3.0, 2.0], [2.0, 4.0, 6.0], [7.0, 5.0, 1.0]]], [[[6.0, 1.0, 4.0], [5.0, 8.0, 7.0], [6.0, 7.0, 6.0]], [[8.0, 4.0, 2.0], 
[6.0, 8.0, 4.0], [8.0, 3.0, 8.0]], [[6.0, 6.0, 7.0], [7.0, 5.0, 2.0], [7.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PPRN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 8.0], [2.0, 3.0, 2.0], [2.0, 2.0, 8.0]], [[1.0, 7.0, 5.0], [4.0, 3.0, 4.0], [3.0, 8.0, 
4.0]], [[4.0, 6.0, 3.0], [8.0, 4.0, 6.0], [2.0, 1.0, 7.0]]], [[[8.0, 1.0, 5.0], [4.0, 8.0, 1.0], [2.0, 3.0, 8.0]], [[3.0, 2.0, 
5.0], [2.0, 8.0, 8.0], [3.0, 5.0, 6.0]], [[8.0, 6.0, 5.0], [1.0, 7.0, 2.0], [6.0, 4.0, 8.0]]], [[[1.0, 8.0, 5.0], [8.0, 1.0, 
6.0], [4.0, 8.0, 5.0]], [[5.0, 4.0, 1.0], [8.0, 1.0, 6.0], [6.0, 4.0, 2.0]], [[8.0, 1.0, 1.0], [4.0, 2.0, 6.0], [7.0, 3.0, 
5.0]]], [[[6.0, 6.0, 2.0], [6.0, 7.0, 8.0], [8.0, 4.0, 7.0]], [[5.0, 2.0, 8.0], [4.0, 6.0, 3.0], [6.0, 4.0, 1.0]], [[6.0, 1.0, 
4.0], [3.0, 2.0, 1.0], [2.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 1.0, 1.0], [3.0, 3.0, 7.0], [2.0, 2.0, 1.0]], [[7.0, 3.0, 4.0], [7.0, 6.0, 
3.0], [1.0, 1.0, 7.0]], [[2.0, 1.0, 7.0], [5.0, 7.0, 7.0], [2.0, 6.0, 8.0]]], [[[6.0, 5.0, 2.0], [6.0, 5.0, 6.0], [6.0, 5.0, 
3.0]], [[6.0, 2.0, 1.0], [1.0, 3.0, 4.0], [2.0, 8.0, 7.0]], [[7.0, 8.0, 6.0], [4.0, 2.0, 3.0], [4.0, 3.0, 5.0]]], [[[3.0, 5.0, 
2.0], [4.0, 6.0, 5.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 4.0], [4.0, 5.0, 5.0], [2.0, 4.0, 2.0]], [[5.0, 5.0, 6.0], [5.0, 1.0, 
3.0], [3.0, 1.0, 3.0]]], [[[3.0, 4.0, 3.0], [8.0, 3.0, 7.0], [7.0, 7.0, 1.0]], [[2.0, 8.0, 3.0], [5.0, 7.0, 5.0], [5.0, 4.0, 
2.0]], [[4.0, 5.0, 1.0], [4.0, 3.0, 8.0], [3.0, 1.0, 8.0]]]]))
      val=Data(numpy.array([[7.0, 4.0, 7.0]]),self.functionspace)
      arg[2,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 8.0], [2.0, 3.0, 2.0], [2.0, 2.0, 8.0]], [[1.0, 7.0, 5.0], [4.0, 3.0, 4.0], [3.0, 
8.0, 4.0]], [[4.0, 6.0, 3.0], [8.0, 4.0, 6.0], [2.0, 1.0, 7.0]]], [[[8.0, 1.0, 5.0], [4.0, 8.0, 1.0], [2.0, 3.0, 8.0]], [[3.0, 
2.0, 5.0], [2.0, 8.0, 8.0], [3.0, 5.0, 6.0]], [[8.0, 6.0, 5.0], [1.0, 7.0, 2.0], [6.0, 4.0, 8.0]]], [[[1.0, 8.0, 5.0], [8.0, 
1.0, 6.0], [4.0, 8.0, 5.0]], [[5.0, 4.0, 1.0], [7.0, 4.0, 7.0], [6.0, 4.0, 2.0]], [[8.0, 1.0, 1.0], [4.0, 2.0, 6.0], [7.0, 3.0, 
5.0]]], [[[6.0, 6.0, 2.0], [6.0, 7.0, 8.0], [8.0, 4.0, 7.0]], [[5.0, 2.0, 8.0], [4.0, 6.0, 3.0], [6.0, 4.0, 1.0]], [[6.0, 1.0, 
4.0], [3.0, 2.0, 1.0], [2.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 1.0], [3.0, 3.0, 7.0], [2.0, 2.0, 1.0]], 
[[7.0, 3.0, 4.0], [7.0, 6.0, 3.0], [1.0, 1.0, 7.0]], [[2.0, 1.0, 7.0], [5.0, 7.0, 7.0], [2.0, 6.0, 8.0]]], [[[6.0, 5.0, 2.0], 
[6.0, 5.0, 6.0], [6.0, 5.0, 3.0]], [[6.0, 2.0, 1.0], [1.0, 3.0, 4.0], [2.0, 8.0, 7.0]], [[7.0, 8.0, 6.0], [4.0, 2.0, 3.0], 
[4.0, 3.0, 5.0]]], [[[3.0, 5.0, 2.0], [4.0, 6.0, 5.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 4.0], [7.0, 4.0, 7.0], [2.0, 4.0, 2.0]], 
[[5.0, 5.0, 6.0], [5.0, 1.0, 3.0], [3.0, 1.0, 3.0]]], [[[3.0, 4.0, 3.0], [8.0, 3.0, 7.0], [7.0, 7.0, 1.0]], [[2.0, 8.0, 3.0], 
[5.0, 7.0, 5.0], [5.0, 4.0, 2.0]], [[4.0, 5.0, 1.0], [4.0, 3.0, 8.0], [3.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PPRN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 1.0], [6.0, 4.0, 1.0], [6.0, 3.0, 1.0]], [[2.0, 4.0, 2.0], [5.0, 2.0, 5.0], [4.0, 8.0, 
1.0]], [[4.0, 2.0, 6.0], [3.0, 5.0, 2.0], [3.0, 2.0, 5.0]]], [[[1.0, 5.0, 2.0], [3.0, 8.0, 6.0], [2.0, 4.0, 7.0]], [[4.0, 7.0, 
5.0], [1.0, 2.0, 7.0], [2.0, 1.0, 3.0]], [[7.0, 7.0, 2.0], [1.0, 4.0, 3.0], [5.0, 3.0, 3.0]]], [[[5.0, 1.0, 6.0], [7.0, 2.0, 
8.0], [7.0, 6.0, 5.0]], [[1.0, 7.0, 2.0], [4.0, 8.0, 8.0], [2.0, 1.0, 4.0]], [[8.0, 3.0, 5.0], [8.0, 1.0, 7.0], [7.0, 6.0, 
2.0]]], [[[2.0, 3.0, 5.0], [4.0, 4.0, 2.0], [4.0, 1.0, 3.0]], [[8.0, 7.0, 4.0], [4.0, 7.0, 7.0], [5.0, 8.0, 2.0]], [[4.0, 5.0, 
1.0], [1.0, 4.0, 3.0], [3.0, 2.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 4.0], [7.0, 8.0, 2.0]], [[6.0, 4.0, 8.0], [6.0, 8.0, 
8.0], [3.0, 6.0, 2.0]], [[8.0, 1.0, 6.0], [1.0, 7.0, 4.0], [7.0, 1.0, 1.0]]], [[[5.0, 5.0, 2.0], [3.0, 6.0, 7.0], [7.0, 6.0, 
5.0]], [[7.0, 1.0, 5.0], [2.0, 3.0, 3.0], [3.0, 1.0, 8.0]], [[5.0, 5.0, 4.0], [3.0, 6.0, 7.0], [5.0, 2.0, 6.0]]], [[[7.0, 1.0, 
2.0], [1.0, 1.0, 8.0], [7.0, 8.0, 6.0]], [[5.0, 5.0, 8.0], [6.0, 2.0, 5.0], [2.0, 3.0, 3.0]], [[5.0, 1.0, 4.0], [8.0, 2.0, 
1.0], [2.0, 5.0, 3.0]]], [[[4.0, 4.0, 4.0], [3.0, 2.0, 4.0], [1.0, 2.0, 8.0]], [[5.0, 2.0, 7.0], [3.0, 3.0, 2.0], [7.0, 6.0, 
2.0]], [[2.0, 2.0, 6.0], [7.0, 8.0, 4.0], [1.0, 2.0, 6.0]]]]))
      val=Data(numpy.array([[8.0, 8.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 8.0, 1.0]]))
      arg[2,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [6.0, 4.0, 1.0], [6.0, 3.0, 1.0]], [[2.0, 4.0, 2.0], [5.0, 2.0, 5.0], [4.0, 
8.0, 1.0]], [[4.0, 2.0, 6.0], [3.0, 5.0, 2.0], [3.0, 2.0, 5.0]]], [[[1.0, 5.0, 2.0], [3.0, 8.0, 6.0], [2.0, 4.0, 7.0]], [[4.0, 
7.0, 5.0], [1.0, 2.0, 7.0], [2.0, 1.0, 3.0]], [[7.0, 7.0, 2.0], [1.0, 4.0, 3.0], [5.0, 3.0, 3.0]]], [[[5.0, 1.0, 6.0], [7.0, 
2.0, 8.0], [7.0, 6.0, 5.0]], [[1.0, 7.0, 2.0], [8.0, 8.0, 7.0], [2.0, 1.0, 4.0]], [[8.0, 3.0, 5.0], [8.0, 1.0, 7.0], [7.0, 6.0, 
2.0]]], [[[2.0, 3.0, 5.0], [4.0, 4.0, 2.0], [4.0, 1.0, 3.0]], [[8.0, 7.0, 4.0], [4.0, 7.0, 7.0], [5.0, 8.0, 2.0]], [[4.0, 5.0, 
1.0], [1.0, 4.0, 3.0], [3.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 4.0], [7.0, 8.0, 2.0]], 
[[6.0, 4.0, 8.0], [6.0, 8.0, 8.0], [3.0, 6.0, 2.0]], [[8.0, 1.0, 6.0], [1.0, 7.0, 4.0], [7.0, 1.0, 1.0]]], [[[5.0, 5.0, 2.0], 
[3.0, 6.0, 7.0], [7.0, 6.0, 5.0]], [[7.0, 1.0, 5.0], [2.0, 3.0, 3.0], [3.0, 1.0, 8.0]], [[5.0, 5.0, 4.0], [3.0, 6.0, 7.0], 
[5.0, 2.0, 6.0]]], [[[7.0, 1.0, 2.0], [1.0, 1.0, 8.0], [7.0, 8.0, 6.0]], [[5.0, 5.0, 8.0], [6.0, 8.0, 1.0], [2.0, 3.0, 3.0]], 
[[5.0, 1.0, 4.0], [8.0, 2.0, 1.0], [2.0, 5.0, 3.0]]], [[[4.0, 4.0, 4.0], [3.0, 2.0, 4.0], [1.0, 2.0, 8.0]], [[5.0, 2.0, 7.0], 
[3.0, 3.0, 2.0], [7.0, 6.0, 2.0]], [[2.0, 2.0, 6.0], [7.0, 8.0, 4.0], [1.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PPRN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 8.0], [2.0, 4.0, 2.0], [4.0, 1.0, 7.0]], [[1.0, 6.0, 7.0], [8.0, 8.0, 1.0], [4.0, 5.0, 
1.0]], [[2.0, 8.0, 8.0], [1.0, 7.0, 3.0], [1.0, 6.0, 5.0]]], [[[5.0, 2.0, 5.0], [6.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[1.0, 7.0, 
6.0], [2.0, 5.0, 5.0], [5.0, 5.0, 3.0]], [[5.0, 8.0, 1.0], [5.0, 5.0, 6.0], [3.0, 2.0, 5.0]]], [[[6.0, 1.0, 1.0], [3.0, 5.0, 
3.0], [5.0, 3.0, 1.0]], [[7.0, 1.0, 1.0], [4.0, 5.0, 2.0], [2.0, 5.0, 3.0]], [[6.0, 5.0, 1.0], [2.0, 4.0, 5.0], [2.0, 1.0, 
8.0]]], [[[1.0, 8.0, 5.0], [4.0, 3.0, 1.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 7.0], [5.0, 2.0, 5.0], [6.0, 4.0, 2.0]], [[7.0, 8.0, 
1.0], [7.0, 5.0, 2.0], [4.0, 3.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 3.0], [7.0, 3.0, 8.0], [1.0, 5.0, 7.0]], [[4.0, 3.0, 6.0], [1.0, 5.0, 
7.0], [5.0, 2.0, 2.0]], [[1.0, 8.0, 2.0], [3.0, 8.0, 2.0], [7.0, 6.0, 5.0]]], [[[5.0, 7.0, 4.0], [7.0, 4.0, 1.0], [1.0, 4.0, 
6.0]], [[7.0, 8.0, 3.0], [8.0, 3.0, 6.0], [4.0, 2.0, 7.0]], [[1.0, 8.0, 4.0], [1.0, 7.0, 3.0], [2.0, 5.0, 3.0]]], [[[3.0, 2.0, 
1.0], [7.0, 7.0, 4.0], [8.0, 4.0, 5.0]], [[4.0, 1.0, 3.0], [5.0, 6.0, 1.0], [4.0, 1.0, 8.0]], [[5.0, 4.0, 5.0], [1.0, 7.0, 
2.0], [2.0, 4.0, 2.0]]], [[[2.0, 1.0, 8.0], [7.0, 7.0, 6.0], [6.0, 8.0, 6.0]], [[3.0, 4.0, 5.0], [8.0, 4.0, 6.0], [8.0, 7.0, 
3.0]], [[4.0, 3.0, 3.0], [7.0, 5.0, 3.0], [4.0, 5.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[4.0, 6.0, 2.0]])+(1.-msk_val)*numpy.array([[1.0, 7.0, 1.0]])
      arg[2,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 8.0], [2.0, 4.0, 2.0], [4.0, 1.0, 7.0]], [[1.0, 6.0, 7.0], [8.0, 8.0, 1.0], [4.0, 
5.0, 1.0]], [[2.0, 8.0, 8.0], [1.0, 7.0, 3.0], [1.0, 6.0, 5.0]]], [[[5.0, 2.0, 5.0], [6.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[1.0, 
7.0, 6.0], [2.0, 5.0, 5.0], [5.0, 5.0, 3.0]], [[5.0, 8.0, 1.0], [5.0, 5.0, 6.0], [3.0, 2.0, 5.0]]], [[[6.0, 1.0, 1.0], [3.0, 
5.0, 3.0], [5.0, 3.0, 1.0]], [[7.0, 1.0, 1.0], [4.0, 6.0, 2.0], [2.0, 5.0, 3.0]], [[6.0, 5.0, 1.0], [2.0, 4.0, 5.0], [2.0, 1.0, 
8.0]]], [[[1.0, 8.0, 5.0], [4.0, 3.0, 1.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 7.0], [5.0, 2.0, 5.0], [6.0, 4.0, 2.0]], [[7.0, 8.0, 
1.0], [7.0, 5.0, 2.0], [4.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 3.0], [7.0, 3.0, 8.0], [1.0, 5.0, 7.0]], 
[[4.0, 3.0, 6.0], [1.0, 5.0, 7.0], [5.0, 2.0, 2.0]], [[1.0, 8.0, 2.0], [3.0, 8.0, 2.0], [7.0, 6.0, 5.0]]], [[[5.0, 7.0, 4.0], 
[7.0, 4.0, 1.0], [1.0, 4.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 3.0, 6.0], [4.0, 2.0, 7.0]], [[1.0, 8.0, 4.0], [1.0, 7.0, 3.0], 
[2.0, 5.0, 3.0]]], [[[3.0, 2.0, 1.0], [7.0, 7.0, 4.0], [8.0, 4.0, 5.0]], [[4.0, 1.0, 3.0], [1.0, 7.0, 1.0], [4.0, 1.0, 8.0]], 
[[5.0, 4.0, 5.0], [1.0, 7.0, 2.0], [2.0, 4.0, 2.0]]], [[[2.0, 1.0, 8.0], [7.0, 7.0, 6.0], [6.0, 8.0, 6.0]], [[3.0, 4.0, 5.0], 
[8.0, 4.0, 6.0], [8.0, 7.0, 3.0]], [[4.0, 3.0, 3.0], [7.0, 5.0, 3.0], [4.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_UPPN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 1.0], [5.0, 4.0, 3.0], [4.0, 8.0, 4.0]], [[1.0, 3.0, 1.0], [2.0, 5.0, 2.0], [1.0, 7.0, 
1.0]], [[7.0, 7.0, 2.0], [7.0, 4.0, 7.0], [2.0, 1.0, 4.0]]], [[[4.0, 4.0, 8.0], [5.0, 1.0, 2.0], [7.0, 1.0, 1.0]], [[3.0, 4.0, 
2.0], [7.0, 6.0, 8.0], [8.0, 3.0, 1.0]], [[8.0, 1.0, 2.0], [3.0, 8.0, 5.0], [4.0, 1.0, 5.0]]], [[[8.0, 3.0, 3.0], [5.0, 8.0, 
3.0], [6.0, 7.0, 2.0]], [[6.0, 4.0, 6.0], [7.0, 8.0, 8.0], [4.0, 8.0, 7.0]], [[1.0, 7.0, 7.0], [5.0, 2.0, 1.0], [2.0, 3.0, 
3.0]]], [[[7.0, 5.0, 6.0], [1.0, 8.0, 3.0], [2.0, 5.0, 7.0]], [[7.0, 6.0, 7.0], [8.0, 4.0, 1.0], [6.0, 8.0, 2.0]], [[1.0, 8.0, 
2.0], [8.0, 3.0, 5.0], [2.0, 6.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 2.0, 5.0], [1.0, 8.0, 8.0], [4.0, 5.0, 4.0]], [[3.0, 5.0, 7.0], [3.0, 1.0, 
4.0], [5.0, 1.0, 3.0]], [[4.0, 3.0, 4.0], [8.0, 8.0, 7.0], [2.0, 5.0, 8.0]]], [[[7.0, 5.0, 8.0], [2.0, 6.0, 3.0], [2.0, 4.0, 
5.0]], [[1.0, 1.0, 4.0], [7.0, 1.0, 4.0], [1.0, 7.0, 3.0]], [[6.0, 8.0, 4.0], [6.0, 3.0, 3.0], [8.0, 6.0, 5.0]]], [[[6.0, 3.0, 
6.0], [3.0, 4.0, 3.0], [3.0, 2.0, 5.0]], [[3.0, 3.0, 7.0], [5.0, 4.0, 6.0], [8.0, 6.0, 5.0]], [[1.0, 5.0, 2.0], [3.0, 1.0, 
6.0], [7.0, 1.0, 5.0]]], [[[1.0, 3.0, 5.0], [1.0, 1.0, 3.0], [4.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], [5.0, 6.0, 5.0], [7.0, 5.0, 
3.0]], [[2.0, 4.0, 6.0], [2.0, 3.0, 4.0], [1.0, 6.0, 8.0]]]]))
      val=numpy.array([[5.0, 4.0, 6.0], [3.0, 6.0, 2.0], [5.0, 5.0, 8.0]])
      arg[:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 1.0], [5.0, 4.0, 3.0], [4.0, 8.0, 4.0]], [[1.0, 3.0, 1.0], [5.0, 4.0, 6.0], [1.0, 
7.0, 1.0]], [[7.0, 7.0, 2.0], [7.0, 4.0, 7.0], [2.0, 1.0, 4.0]]], [[[4.0, 4.0, 8.0], [5.0, 1.0, 2.0], [7.0, 1.0, 1.0]], [[3.0, 
4.0, 2.0], [3.0, 6.0, 2.0], [8.0, 3.0, 1.0]], [[8.0, 1.0, 2.0], [3.0, 8.0, 5.0], [4.0, 1.0, 5.0]]], [[[8.0, 3.0, 3.0], [5.0, 
8.0, 3.0], [6.0, 7.0, 2.0]], [[6.0, 4.0, 6.0], [5.0, 5.0, 8.0], [4.0, 8.0, 7.0]], [[1.0, 7.0, 7.0], [5.0, 2.0, 1.0], [2.0, 3.0, 
3.0]]], [[[7.0, 5.0, 6.0], [1.0, 8.0, 3.0], [2.0, 5.0, 7.0]], [[7.0, 6.0, 7.0], [8.0, 4.0, 1.0], [6.0, 8.0, 2.0]], [[1.0, 8.0, 
2.0], [8.0, 3.0, 5.0], [2.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 5.0], [1.0, 8.0, 8.0], [4.0, 5.0, 4.0]], 
[[3.0, 5.0, 7.0], [5.0, 4.0, 6.0], [5.0, 1.0, 3.0]], [[4.0, 3.0, 4.0], [8.0, 8.0, 7.0], [2.0, 5.0, 8.0]]], [[[7.0, 5.0, 8.0], 
[2.0, 6.0, 3.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 4.0], [3.0, 6.0, 2.0], [1.0, 7.0, 3.0]], [[6.0, 8.0, 4.0], [6.0, 3.0, 3.0], 
[8.0, 6.0, 5.0]]], [[[6.0, 3.0, 6.0], [3.0, 4.0, 3.0], [3.0, 2.0, 5.0]], [[3.0, 3.0, 7.0], [5.0, 5.0, 8.0], [8.0, 6.0, 5.0]], 
[[1.0, 5.0, 2.0], [3.0, 1.0, 6.0], [7.0, 1.0, 5.0]]], [[[1.0, 3.0, 5.0], [1.0, 1.0, 3.0], [4.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], 
[5.0, 6.0, 5.0], [7.0, 5.0, 3.0]], [[2.0, 4.0, 6.0], [2.0, 3.0, 4.0], [1.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_UPPN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 3.0], [6.0, 7.0, 8.0], [8.0, 7.0, 3.0]], [[3.0, 5.0, 1.0], [6.0, 5.0, 2.0], [4.0, 3.0, 
5.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 7.0], [3.0, 7.0, 2.0]]], [[[4.0, 4.0, 5.0], [7.0, 4.0, 3.0], [5.0, 7.0, 7.0]], [[4.0, 3.0, 
1.0], [3.0, 2.0, 5.0], [6.0, 6.0, 7.0]], [[3.0, 5.0, 4.0], [4.0, 8.0, 5.0], [7.0, 6.0, 7.0]]], [[[8.0, 3.0, 1.0], [5.0, 3.0, 
2.0], [2.0, 7.0, 4.0]], [[8.0, 1.0, 7.0], [2.0, 1.0, 7.0], [7.0, 7.0, 1.0]], [[8.0, 7.0, 7.0], [4.0, 5.0, 6.0], [4.0, 2.0, 
8.0]]], [[[2.0, 5.0, 5.0], [5.0, 2.0, 5.0], [7.0, 3.0, 4.0]], [[5.0, 8.0, 7.0], [1.0, 2.0, 1.0], [3.0, 2.0, 7.0]], [[5.0, 4.0, 
4.0], [4.0, 4.0, 6.0], [7.0, 6.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 6.0], [5.0, 8.0, 8.0], [2.0, 1.0, 1.0]], [[4.0, 7.0, 7.0], [2.0, 8.0, 
6.0], [8.0, 1.0, 8.0]], [[1.0, 4.0, 2.0], [8.0, 3.0, 5.0], [7.0, 5.0, 5.0]]], [[[4.0, 4.0, 5.0], [5.0, 3.0, 5.0], [8.0, 1.0, 
8.0]], [[8.0, 2.0, 7.0], [2.0, 5.0, 1.0], [8.0, 7.0, 1.0]], [[3.0, 4.0, 8.0], [6.0, 2.0, 7.0], [6.0, 6.0, 6.0]]], [[[8.0, 3.0, 
6.0], [4.0, 4.0, 6.0], [6.0, 2.0, 3.0]], [[8.0, 1.0, 2.0], [5.0, 5.0, 7.0], [6.0, 5.0, 3.0]], [[8.0, 6.0, 1.0], [4.0, 4.0, 
1.0], [8.0, 7.0, 4.0]]], [[[6.0, 4.0, 8.0], [6.0, 7.0, 3.0], [6.0, 8.0, 3.0]], [[3.0, 1.0, 8.0], [6.0, 6.0, 4.0], [3.0, 8.0, 
2.0]], [[1.0, 1.0, 7.0], [7.0, 1.0, 1.0], [6.0, 3.0, 1.0]]]]))
      val=Data(numpy.array([[1.0, 8.0, 6.0], [1.0, 4.0, 2.0], [2.0, 5.0, 3.0]]),self.functionspace)
      arg[:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 3.0], [6.0, 7.0, 8.0], [8.0, 7.0, 3.0]], [[3.0, 5.0, 1.0], [1.0, 8.0, 6.0], [4.0, 
3.0, 5.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 7.0], [3.0, 7.0, 2.0]]], [[[4.0, 4.0, 5.0], [7.0, 4.0, 3.0], [5.0, 7.0, 7.0]], [[4.0, 
3.0, 1.0], [1.0, 4.0, 2.0], [6.0, 6.0, 7.0]], [[3.0, 5.0, 4.0], [4.0, 8.0, 5.0], [7.0, 6.0, 7.0]]], [[[8.0, 3.0, 1.0], [5.0, 
3.0, 2.0], [2.0, 7.0, 4.0]], [[8.0, 1.0, 7.0], [2.0, 5.0, 3.0], [7.0, 7.0, 1.0]], [[8.0, 7.0, 7.0], [4.0, 5.0, 6.0], [4.0, 2.0, 
8.0]]], [[[2.0, 5.0, 5.0], [5.0, 2.0, 5.0], [7.0, 3.0, 4.0]], [[5.0, 8.0, 7.0], [1.0, 2.0, 1.0], [3.0, 2.0, 7.0]], [[5.0, 4.0, 
4.0], [4.0, 4.0, 6.0], [7.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 6.0], [5.0, 8.0, 8.0], [2.0, 1.0, 1.0]], 
[[4.0, 7.0, 7.0], [1.0, 8.0, 6.0], [8.0, 1.0, 8.0]], [[1.0, 4.0, 2.0], [8.0, 3.0, 5.0], [7.0, 5.0, 5.0]]], [[[4.0, 4.0, 5.0], 
[5.0, 3.0, 5.0], [8.0, 1.0, 8.0]], [[8.0, 2.0, 7.0], [1.0, 4.0, 2.0], [8.0, 7.0, 1.0]], [[3.0, 4.0, 8.0], [6.0, 2.0, 7.0], 
[6.0, 6.0, 6.0]]], [[[8.0, 3.0, 6.0], [4.0, 4.0, 6.0], [6.0, 2.0, 3.0]], [[8.0, 1.0, 2.0], [2.0, 5.0, 3.0], [6.0, 5.0, 3.0]], 
[[8.0, 6.0, 1.0], [4.0, 4.0, 1.0], [8.0, 7.0, 4.0]]], [[[6.0, 4.0, 8.0], [6.0, 7.0, 3.0], [6.0, 8.0, 3.0]], [[3.0, 1.0, 8.0], 
[6.0, 6.0, 4.0], [3.0, 8.0, 2.0]], [[1.0, 1.0, 7.0], [7.0, 1.0, 1.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_UPPN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 8.0], [8.0, 6.0, 6.0], [3.0, 5.0, 7.0]], [[1.0, 1.0, 7.0], [5.0, 6.0, 3.0], [4.0, 6.0, 
3.0]], [[1.0, 6.0, 3.0], [1.0, 3.0, 5.0], [1.0, 8.0, 1.0]]], [[[4.0, 6.0, 1.0], [6.0, 1.0, 5.0], [5.0, 5.0, 1.0]], [[5.0, 4.0, 
6.0], [5.0, 3.0, 6.0], [4.0, 5.0, 6.0]], [[3.0, 1.0, 1.0], [1.0, 8.0, 8.0], [5.0, 7.0, 2.0]]], [[[6.0, 1.0, 5.0], [1.0, 8.0, 
6.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 2.0], [3.0, 3.0, 7.0], [4.0, 8.0, 6.0]], [[1.0, 1.0, 5.0], [8.0, 2.0, 3.0], [7.0, 1.0, 
7.0]]], [[[8.0, 1.0, 7.0], [3.0, 2.0, 8.0], [4.0, 7.0, 7.0]], [[4.0, 3.0, 4.0], [1.0, 3.0, 5.0], [2.0, 7.0, 1.0]], [[8.0, 2.0, 
1.0], [8.0, 8.0, 3.0], [8.0, 8.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 2.0], [5.0, 8.0, 4.0], [8.0, 1.0, 4.0]], [[2.0, 6.0, 3.0], [1.0, 6.0, 
8.0], [7.0, 3.0, 6.0]], [[3.0, 7.0, 7.0], [4.0, 2.0, 4.0], [1.0, 6.0, 2.0]]], [[[4.0, 2.0, 5.0], [6.0, 5.0, 5.0], [7.0, 3.0, 
1.0]], [[7.0, 8.0, 8.0], [7.0, 5.0, 4.0], [4.0, 4.0, 2.0]], [[2.0, 8.0, 6.0], [6.0, 7.0, 4.0], [7.0, 1.0, 8.0]]], [[[2.0, 6.0, 
5.0], [8.0, 5.0, 4.0], [3.0, 8.0, 1.0]], [[7.0, 2.0, 6.0], [8.0, 6.0, 8.0], [8.0, 3.0, 5.0]], [[5.0, 3.0, 6.0], [5.0, 7.0, 
8.0], [6.0, 4.0, 4.0]]], [[[4.0, 2.0, 7.0], [4.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 4.0, 7.0], [7.0, 4.0, 3.0], [2.0, 5.0, 
2.0]], [[2.0, 6.0, 1.0], [5.0, 3.0, 6.0], [2.0, 8.0, 4.0]]]]))
      val=Data(numpy.array([[7.0, 7.0, 1.0], [4.0, 6.0, 1.0], [2.0, 5.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 2.0, 7.0], [7.0, 8.0, 7.0], [7.0, 6.0, 2.0]]))
      arg[:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 8.0], [8.0, 6.0, 6.0], [3.0, 5.0, 7.0]], [[1.0, 1.0, 7.0], [7.0, 7.0, 1.0], [4.0, 
6.0, 3.0]], [[1.0, 6.0, 3.0], [1.0, 3.0, 5.0], [1.0, 8.0, 1.0]]], [[[4.0, 6.0, 1.0], [6.0, 1.0, 5.0], [5.0, 5.0, 1.0]], [[5.0, 
4.0, 6.0], [4.0, 6.0, 1.0], [4.0, 5.0, 6.0]], [[3.0, 1.0, 1.0], [1.0, 8.0, 8.0], [5.0, 7.0, 2.0]]], [[[6.0, 1.0, 5.0], [1.0, 
8.0, 6.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 2.0], [2.0, 5.0, 8.0], [4.0, 8.0, 6.0]], [[1.0, 1.0, 5.0], [8.0, 2.0, 3.0], [7.0, 1.0, 
7.0]]], [[[8.0, 1.0, 7.0], [3.0, 2.0, 8.0], [4.0, 7.0, 7.0]], [[4.0, 3.0, 4.0], [1.0, 3.0, 5.0], [2.0, 7.0, 1.0]], [[8.0, 2.0, 
1.0], [8.0, 8.0, 3.0], [8.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 2.0], [5.0, 8.0, 4.0], [8.0, 1.0, 4.0]], 
[[2.0, 6.0, 3.0], [8.0, 2.0, 7.0], [7.0, 3.0, 6.0]], [[3.0, 7.0, 7.0], [4.0, 2.0, 4.0], [1.0, 6.0, 2.0]]], [[[4.0, 2.0, 5.0], 
[6.0, 5.0, 5.0], [7.0, 3.0, 1.0]], [[7.0, 8.0, 8.0], [7.0, 8.0, 7.0], [4.0, 4.0, 2.0]], [[2.0, 8.0, 6.0], [6.0, 7.0, 4.0], 
[7.0, 1.0, 8.0]]], [[[2.0, 6.0, 5.0], [8.0, 5.0, 4.0], [3.0, 8.0, 1.0]], [[7.0, 2.0, 6.0], [7.0, 6.0, 2.0], [8.0, 3.0, 5.0]], 
[[5.0, 3.0, 6.0], [5.0, 7.0, 8.0], [6.0, 4.0, 4.0]]], [[[4.0, 2.0, 7.0], [4.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 4.0, 7.0], 
[7.0, 4.0, 3.0], [2.0, 5.0, 2.0]], [[2.0, 6.0, 1.0], [5.0, 3.0, 6.0], [2.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_UPPN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 1.0], [5.0, 1.0, 8.0], [1.0, 5.0, 8.0]], [[2.0, 5.0, 5.0], [2.0, 1.0, 5.0], [4.0, 7.0, 
2.0]], [[6.0, 1.0, 5.0], [1.0, 7.0, 1.0], [2.0, 8.0, 1.0]]], [[[7.0, 4.0, 4.0], [6.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[4.0, 8.0, 
5.0], [2.0, 8.0, 1.0], [1.0, 6.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 5.0, 8.0], [1.0, 5.0, 6.0]]], [[[3.0, 6.0, 7.0], [5.0, 5.0, 
2.0], [3.0, 7.0, 3.0]], [[4.0, 6.0, 3.0], [1.0, 4.0, 3.0], [3.0, 5.0, 4.0]], [[4.0, 1.0, 2.0], [8.0, 1.0, 4.0], [6.0, 5.0, 
6.0]]], [[[8.0, 8.0, 8.0], [5.0, 3.0, 7.0], [5.0, 5.0, 1.0]], [[7.0, 5.0, 4.0], [6.0, 6.0, 3.0], [2.0, 1.0, 8.0]], [[2.0, 7.0, 
4.0], [3.0, 3.0, 7.0], [5.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 5.0], [8.0, 1.0, 3.0], [3.0, 4.0, 8.0]], [[1.0, 3.0, 5.0], [3.0, 7.0, 
8.0], [8.0, 2.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 2.0, 1.0], [5.0, 3.0, 7.0]]], [[[6.0, 7.0, 4.0], [8.0, 1.0, 8.0], [1.0, 7.0, 
5.0]], [[3.0, 8.0, 6.0], [6.0, 8.0, 4.0], [6.0, 3.0, 7.0]], [[3.0, 4.0, 1.0], [1.0, 1.0, 3.0], [2.0, 5.0, 5.0]]], [[[1.0, 5.0, 
5.0], [2.0, 7.0, 2.0], [2.0, 5.0, 1.0]], [[3.0, 7.0, 7.0], [1.0, 2.0, 5.0], [6.0, 7.0, 7.0]], [[1.0, 4.0, 8.0], [1.0, 6.0, 
6.0], [8.0, 2.0, 6.0]]], [[[7.0, 8.0, 8.0], [1.0, 7.0, 4.0], [7.0, 2.0, 7.0]], [[5.0, 1.0, 4.0], [2.0, 6.0, 3.0], [8.0, 3.0, 
2.0]], [[8.0, 7.0, 6.0], [4.0, 3.0, 6.0], [4.0, 7.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[8.0, 1.0, 1.0], [5.0, 3.0, 2.0], [8.0, 7.0, 1.0]])+(1.-msk_val)*numpy.array([[7.0, 8.0, 2.0], 
[5.0, 3.0, 2.0], [5.0, 3.0, 8.0]])
      arg[:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 1.0], [5.0, 1.0, 8.0], [1.0, 5.0, 8.0]], [[2.0, 5.0, 5.0], [8.0, 1.0, 1.0], [4.0, 
7.0, 2.0]], [[6.0, 1.0, 5.0], [1.0, 7.0, 1.0], [2.0, 8.0, 1.0]]], [[[7.0, 4.0, 4.0], [6.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[4.0, 
8.0, 5.0], [5.0, 3.0, 2.0], [1.0, 6.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 5.0, 8.0], [1.0, 5.0, 6.0]]], [[[3.0, 6.0, 7.0], [5.0, 
5.0, 2.0], [3.0, 7.0, 3.0]], [[4.0, 6.0, 3.0], [8.0, 7.0, 1.0], [3.0, 5.0, 4.0]], [[4.0, 1.0, 2.0], [8.0, 1.0, 4.0], [6.0, 5.0, 
6.0]]], [[[8.0, 8.0, 8.0], [5.0, 3.0, 7.0], [5.0, 5.0, 1.0]], [[7.0, 5.0, 4.0], [6.0, 6.0, 3.0], [2.0, 1.0, 8.0]], [[2.0, 7.0, 
4.0], [3.0, 3.0, 7.0], [5.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 5.0], [8.0, 1.0, 3.0], [3.0, 4.0, 8.0]], 
[[1.0, 3.0, 5.0], [7.0, 8.0, 2.0], [8.0, 2.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 2.0, 1.0], [5.0, 3.0, 7.0]]], [[[6.0, 7.0, 4.0], 
[8.0, 1.0, 8.0], [1.0, 7.0, 5.0]], [[3.0, 8.0, 6.0], [5.0, 3.0, 2.0], [6.0, 3.0, 7.0]], [[3.0, 4.0, 1.0], [1.0, 1.0, 3.0], 
[2.0, 5.0, 5.0]]], [[[1.0, 5.0, 5.0], [2.0, 7.0, 2.0], [2.0, 5.0, 1.0]], [[3.0, 7.0, 7.0], [5.0, 3.0, 8.0], [6.0, 7.0, 7.0]], 
[[1.0, 4.0, 8.0], [1.0, 6.0, 6.0], [8.0, 2.0, 6.0]]], [[[7.0, 8.0, 8.0], [1.0, 7.0, 4.0], [7.0, 2.0, 7.0]], [[5.0, 1.0, 4.0], 
[2.0, 6.0, 3.0], [8.0, 3.0, 2.0]], [[8.0, 7.0, 6.0], [4.0, 3.0, 6.0], [4.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RUUN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 3.0], [7.0, 6.0, 3.0], [8.0, 3.0, 5.0]], [[1.0, 6.0, 6.0], [4.0, 8.0, 5.0], [4.0, 3.0, 
5.0]], [[1.0, 5.0, 4.0], [6.0, 8.0, 6.0], [8.0, 7.0, 1.0]]], [[[3.0, 3.0, 5.0], [7.0, 2.0, 5.0], [2.0, 1.0, 3.0]], [[3.0, 1.0, 
5.0], [4.0, 4.0, 1.0], [5.0, 6.0, 8.0]], [[3.0, 8.0, 3.0], [5.0, 1.0, 4.0], [4.0, 5.0, 8.0]]], [[[6.0, 4.0, 8.0], [7.0, 2.0, 
2.0], [6.0, 5.0, 5.0]], [[2.0, 8.0, 7.0], [2.0, 2.0, 5.0], [7.0, 7.0, 1.0]], [[7.0, 3.0, 6.0], [5.0, 8.0, 7.0], [8.0, 7.0, 
3.0]]], [[[4.0, 7.0, 2.0], [6.0, 4.0, 6.0], [7.0, 2.0, 5.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 5.0], [2.0, 1.0, 2.0]], [[3.0, 6.0, 
6.0], [8.0, 8.0, 2.0], [3.0, 5.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 7.0], [1.0, 2.0, 5.0], [2.0, 4.0, 7.0]], [[6.0, 2.0, 8.0], [2.0, 4.0, 
3.0], [5.0, 4.0, 2.0]], [[8.0, 5.0, 5.0], [8.0, 1.0, 8.0], [6.0, 7.0, 4.0]]], [[[7.0, 3.0, 2.0], [1.0, 3.0, 5.0], [2.0, 7.0, 
3.0]], [[3.0, 8.0, 3.0], [6.0, 5.0, 3.0], [5.0, 2.0, 1.0]], [[6.0, 4.0, 1.0], [6.0, 5.0, 3.0], [3.0, 7.0, 1.0]]], [[[5.0, 1.0, 
1.0], [7.0, 4.0, 2.0], [1.0, 8.0, 6.0]], [[6.0, 1.0, 5.0], [1.0, 8.0, 7.0], [8.0, 3.0, 4.0]], [[8.0, 7.0, 8.0], [6.0, 8.0, 
3.0], [4.0, 4.0, 6.0]]], [[[3.0, 2.0, 3.0], [3.0, 4.0, 7.0], [3.0, 5.0, 1.0]], [[8.0, 1.0, 1.0], [3.0, 1.0, 7.0], [2.0, 5.0, 
3.0]], [[6.0, 3.0, 8.0], [7.0, 4.0, 7.0], [6.0, 1.0, 8.0]]]]))
      val=numpy.array([[[[8.0, 7.0, 4.0], [4.0, 5.0, 4.0]], [[1.0, 3.0, 3.0], [3.0, 7.0, 8.0]]], [[[7.0, 7.0, 7.0], [4.0, 5.0, 
5.0]], [[8.0, 3.0, 6.0], [5.0, 1.0, 2.0]]]])
      arg[1:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 3.0], [7.0, 6.0, 3.0], [8.0, 3.0, 5.0]], [[1.0, 6.0, 6.0], [4.0, 8.0, 5.0], [4.0, 
3.0, 5.0]], [[1.0, 5.0, 4.0], [6.0, 8.0, 6.0], [8.0, 7.0, 1.0]]], [[[8.0, 7.0, 4.0], [4.0, 5.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 
3.0, 3.0], [3.0, 7.0, 8.0], [5.0, 6.0, 8.0]], [[3.0, 8.0, 3.0], [5.0, 1.0, 4.0], [4.0, 5.0, 8.0]]], [[[7.0, 7.0, 7.0], [4.0, 
5.0, 5.0], [6.0, 5.0, 5.0]], [[8.0, 3.0, 6.0], [5.0, 1.0, 2.0], [7.0, 7.0, 1.0]], [[7.0, 3.0, 6.0], [5.0, 8.0, 7.0], [8.0, 7.0, 
3.0]]], [[[4.0, 7.0, 2.0], [6.0, 4.0, 6.0], [7.0, 2.0, 5.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 5.0], [2.0, 1.0, 2.0]], [[3.0, 6.0, 
6.0], [8.0, 8.0, 2.0], [3.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 7.0], [1.0, 2.0, 5.0], [2.0, 4.0, 7.0]], 
[[6.0, 2.0, 8.0], [2.0, 4.0, 3.0], [5.0, 4.0, 2.0]], [[8.0, 5.0, 5.0], [8.0, 1.0, 8.0], [6.0, 7.0, 4.0]]], [[[8.0, 7.0, 4.0], 
[4.0, 5.0, 4.0], [2.0, 7.0, 3.0]], [[1.0, 3.0, 3.0], [3.0, 7.0, 8.0], [5.0, 2.0, 1.0]], [[6.0, 4.0, 1.0], [6.0, 5.0, 3.0], 
[3.0, 7.0, 1.0]]], [[[7.0, 7.0, 7.0], [4.0, 5.0, 5.0], [1.0, 8.0, 6.0]], [[8.0, 3.0, 6.0], [5.0, 1.0, 2.0], [8.0, 3.0, 4.0]], 
[[8.0, 7.0, 8.0], [6.0, 8.0, 3.0], [4.0, 4.0, 6.0]]], [[[3.0, 2.0, 3.0], [3.0, 4.0, 7.0], [3.0, 5.0, 1.0]], [[8.0, 1.0, 1.0], 
[3.0, 1.0, 7.0], [2.0, 5.0, 3.0]], [[6.0, 3.0, 8.0], [7.0, 4.0, 7.0], [6.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RUUN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 5.0], [2.0, 7.0, 6.0], [5.0, 7.0, 8.0]], [[5.0, 5.0, 5.0], [2.0, 8.0, 1.0], [7.0, 3.0, 
4.0]], [[5.0, 7.0, 8.0], [8.0, 7.0, 6.0], [8.0, 5.0, 8.0]]], [[[5.0, 1.0, 7.0], [3.0, 2.0, 4.0], [2.0, 4.0, 8.0]], [[5.0, 1.0, 
2.0], [2.0, 2.0, 2.0], [7.0, 2.0, 2.0]], [[2.0, 8.0, 6.0], [4.0, 4.0, 1.0], [3.0, 2.0, 8.0]]], [[[7.0, 2.0, 6.0], [6.0, 2.0, 
5.0], [6.0, 5.0, 7.0]], [[5.0, 7.0, 3.0], [4.0, 5.0, 4.0], [5.0, 7.0, 8.0]], [[4.0, 7.0, 3.0], [3.0, 3.0, 7.0], [8.0, 1.0, 
6.0]]], [[[1.0, 1.0, 8.0], [2.0, 2.0, 2.0], [8.0, 5.0, 1.0]], [[5.0, 3.0, 8.0], [8.0, 7.0, 7.0], [1.0, 3.0, 8.0]], [[6.0, 5.0, 
6.0], [4.0, 2.0, 8.0], [7.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 2.0], [8.0, 3.0, 5.0], [5.0, 5.0, 2.0]], [[5.0, 4.0, 2.0], [8.0, 5.0, 
2.0], [3.0, 2.0, 7.0]], [[2.0, 1.0, 6.0], [1.0, 3.0, 5.0], [5.0, 6.0, 7.0]]], [[[2.0, 1.0, 6.0], [3.0, 2.0, 8.0], [6.0, 4.0, 
7.0]], [[5.0, 6.0, 3.0], [2.0, 1.0, 1.0], [6.0, 3.0, 5.0]], [[4.0, 1.0, 8.0], [5.0, 4.0, 2.0], [6.0, 1.0, 2.0]]], [[[2.0, 5.0, 
3.0], [1.0, 2.0, 3.0], [1.0, 4.0, 3.0]], [[5.0, 3.0, 3.0], [2.0, 2.0, 2.0], [6.0, 1.0, 6.0]], [[8.0, 2.0, 8.0], [5.0, 2.0, 
3.0], [7.0, 8.0, 7.0]]], [[[5.0, 2.0, 2.0], [7.0, 3.0, 2.0], [6.0, 5.0, 1.0]], [[2.0, 5.0, 4.0], [2.0, 2.0, 7.0], [4.0, 6.0, 
5.0]], [[1.0, 2.0, 6.0], [7.0, 6.0, 1.0], [4.0, 4.0, 2.0]]]]))
      val=Data(numpy.array([[[[1.0, 6.0, 6.0], [4.0, 8.0, 6.0]], [[7.0, 8.0, 8.0], [6.0, 2.0, 3.0]]], [[[3.0, 5.0, 7.0], [8.0, 
8.0, 8.0]], [[6.0, 7.0, 5.0], [8.0, 3.0, 2.0]]]]),self.functionspace)
      arg[1:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 5.0], [2.0, 7.0, 6.0], [5.0, 7.0, 8.0]], [[5.0, 5.0, 5.0], [2.0, 8.0, 1.0], [7.0, 
3.0, 4.0]], [[5.0, 7.0, 8.0], [8.0, 7.0, 6.0], [8.0, 5.0, 8.0]]], [[[1.0, 6.0, 6.0], [4.0, 8.0, 6.0], [2.0, 4.0, 8.0]], [[7.0, 
8.0, 8.0], [6.0, 2.0, 3.0], [7.0, 2.0, 2.0]], [[2.0, 8.0, 6.0], [4.0, 4.0, 1.0], [3.0, 2.0, 8.0]]], [[[3.0, 5.0, 7.0], [8.0, 
8.0, 8.0], [6.0, 5.0, 7.0]], [[6.0, 7.0, 5.0], [8.0, 3.0, 2.0], [5.0, 7.0, 8.0]], [[4.0, 7.0, 3.0], [3.0, 3.0, 7.0], [8.0, 1.0, 
6.0]]], [[[1.0, 1.0, 8.0], [2.0, 2.0, 2.0], [8.0, 5.0, 1.0]], [[5.0, 3.0, 8.0], [8.0, 7.0, 7.0], [1.0, 3.0, 8.0]], [[6.0, 5.0, 
6.0], [4.0, 2.0, 8.0], [7.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 2.0], [8.0, 3.0, 5.0], [5.0, 5.0, 2.0]], 
[[5.0, 4.0, 2.0], [8.0, 5.0, 2.0], [3.0, 2.0, 7.0]], [[2.0, 1.0, 6.0], [1.0, 3.0, 5.0], [5.0, 6.0, 7.0]]], [[[1.0, 6.0, 6.0], 
[4.0, 8.0, 6.0], [6.0, 4.0, 7.0]], [[7.0, 8.0, 8.0], [6.0, 2.0, 3.0], [6.0, 3.0, 5.0]], [[4.0, 1.0, 8.0], [5.0, 4.0, 2.0], 
[6.0, 1.0, 2.0]]], [[[3.0, 5.0, 7.0], [8.0, 8.0, 8.0], [1.0, 4.0, 3.0]], [[6.0, 7.0, 5.0], [8.0, 3.0, 2.0], [6.0, 1.0, 6.0]], 
[[8.0, 2.0, 8.0], [5.0, 2.0, 3.0], [7.0, 8.0, 7.0]]], [[[5.0, 2.0, 2.0], [7.0, 3.0, 2.0], [6.0, 5.0, 1.0]], [[2.0, 5.0, 4.0], 
[2.0, 2.0, 7.0], [4.0, 6.0, 5.0]], [[1.0, 2.0, 6.0], [7.0, 6.0, 1.0], [4.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RUUN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 4.0], [4.0, 7.0, 6.0], [7.0, 4.0, 3.0]], [[6.0, 3.0, 2.0], [6.0, 5.0, 1.0], [3.0, 7.0, 
1.0]], [[2.0, 4.0, 4.0], [4.0, 6.0, 3.0], [2.0, 6.0, 7.0]]], [[[2.0, 2.0, 2.0], [7.0, 6.0, 2.0], [1.0, 6.0, 6.0]], [[2.0, 1.0, 
5.0], [5.0, 6.0, 7.0], [3.0, 5.0, 4.0]], [[1.0, 7.0, 1.0], [1.0, 5.0, 1.0], [4.0, 7.0, 4.0]]], [[[6.0, 6.0, 5.0], [2.0, 7.0, 
7.0], [3.0, 3.0, 8.0]], [[5.0, 5.0, 4.0], [8.0, 7.0, 8.0], [8.0, 1.0, 8.0]], [[4.0, 5.0, 1.0], [2.0, 5.0, 6.0], [8.0, 5.0, 
6.0]]], [[[8.0, 7.0, 4.0], [2.0, 1.0, 2.0], [7.0, 3.0, 8.0]], [[4.0, 5.0, 8.0], [5.0, 2.0, 8.0], [3.0, 1.0, 5.0]], [[4.0, 8.0, 
8.0], [4.0, 8.0, 8.0], [6.0, 2.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 2.0], [5.0, 7.0, 2.0], [5.0, 8.0, 1.0]], [[6.0, 4.0, 6.0], [6.0, 8.0, 
5.0], [2.0, 5.0, 5.0]], [[4.0, 4.0, 3.0], [3.0, 4.0, 1.0], [7.0, 6.0, 1.0]]], [[[8.0, 8.0, 6.0], [2.0, 3.0, 5.0], [2.0, 1.0, 
8.0]], [[8.0, 4.0, 8.0], [3.0, 2.0, 2.0], [1.0, 7.0, 3.0]], [[1.0, 8.0, 1.0], [8.0, 8.0, 5.0], [4.0, 4.0, 2.0]]], [[[6.0, 6.0, 
7.0], [2.0, 4.0, 2.0], [8.0, 2.0, 4.0]], [[1.0, 7.0, 8.0], [7.0, 5.0, 3.0], [8.0, 4.0, 6.0]], [[4.0, 8.0, 8.0], [3.0, 6.0, 
2.0], [2.0, 4.0, 2.0]]], [[[3.0, 6.0, 4.0], [4.0, 7.0, 2.0], [3.0, 7.0, 6.0]], [[2.0, 3.0, 8.0], [6.0, 6.0, 8.0], [5.0, 5.0, 
2.0]], [[8.0, 3.0, 8.0], [5.0, 6.0, 8.0], [5.0, 2.0, 3.0]]]]))
      val=Data(numpy.array([[[[6.0, 1.0, 7.0], [3.0, 1.0, 4.0]], [[5.0, 3.0, 1.0], [5.0, 4.0, 4.0]]], [[[2.0, 2.0, 6.0], [2.0, 
8.0, 3.0]], [[7.0, 7.0, 2.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 4.0], [4.0, 6.0, 2.0]], [[4.0, 8.0, 6.0], [4.0, 3.0, 8.0]]], [[[2.0, 3.0, 
8.0], [2.0, 4.0, 4.0]], [[8.0, 3.0, 4.0], [6.0, 7.0, 8.0]]]]))
      arg[1:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 4.0], [4.0, 7.0, 6.0], [7.0, 4.0, 3.0]], [[6.0, 3.0, 2.0], [6.0, 5.0, 1.0], [3.0, 
7.0, 1.0]], [[2.0, 4.0, 4.0], [4.0, 6.0, 3.0], [2.0, 6.0, 7.0]]], [[[6.0, 1.0, 7.0], [3.0, 1.0, 4.0], [1.0, 6.0, 6.0]], [[5.0, 
3.0, 1.0], [5.0, 4.0, 4.0], [3.0, 5.0, 4.0]], [[1.0, 7.0, 1.0], [1.0, 5.0, 1.0], [4.0, 7.0, 4.0]]], [[[2.0, 2.0, 6.0], [2.0, 
8.0, 3.0], [3.0, 3.0, 8.0]], [[7.0, 7.0, 2.0], [2.0, 7.0, 7.0], [8.0, 1.0, 8.0]], [[4.0, 5.0, 1.0], [2.0, 5.0, 6.0], [8.0, 5.0, 
6.0]]], [[[8.0, 7.0, 4.0], [2.0, 1.0, 2.0], [7.0, 3.0, 8.0]], [[4.0, 5.0, 8.0], [5.0, 2.0, 8.0], [3.0, 1.0, 5.0]], [[4.0, 8.0, 
8.0], [4.0, 8.0, 8.0], [6.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 2.0], [5.0, 7.0, 2.0], [5.0, 8.0, 1.0]], 
[[6.0, 4.0, 6.0], [6.0, 8.0, 5.0], [2.0, 5.0, 5.0]], [[4.0, 4.0, 3.0], [3.0, 4.0, 1.0], [7.0, 6.0, 1.0]]], [[[1.0, 3.0, 4.0], 
[4.0, 6.0, 2.0], [2.0, 1.0, 8.0]], [[4.0, 8.0, 6.0], [4.0, 3.0, 8.0], [1.0, 7.0, 3.0]], [[1.0, 8.0, 1.0], [8.0, 8.0, 5.0], 
[4.0, 4.0, 2.0]]], [[[2.0, 3.0, 8.0], [2.0, 4.0, 4.0], [8.0, 2.0, 4.0]], [[8.0, 3.0, 4.0], [6.0, 7.0, 8.0], [8.0, 4.0, 6.0]], 
[[4.0, 8.0, 8.0], [3.0, 6.0, 2.0], [2.0, 4.0, 2.0]]], [[[3.0, 6.0, 4.0], [4.0, 7.0, 2.0], [3.0, 7.0, 6.0]], [[2.0, 3.0, 8.0], 
[6.0, 6.0, 8.0], [5.0, 5.0, 2.0]], [[8.0, 3.0, 8.0], [5.0, 6.0, 8.0], [5.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RUUN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 7.0], [4.0, 8.0, 5.0], [2.0, 7.0, 1.0]], [[3.0, 8.0, 8.0], [1.0, 2.0, 6.0], [4.0, 3.0, 
4.0]], [[7.0, 4.0, 5.0], [6.0, 8.0, 2.0], [1.0, 6.0, 2.0]]], [[[6.0, 2.0, 2.0], [2.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[3.0, 4.0, 
6.0], [1.0, 4.0, 7.0], [4.0, 5.0, 4.0]], [[3.0, 4.0, 8.0], [4.0, 1.0, 1.0], [6.0, 6.0, 8.0]]], [[[4.0, 3.0, 3.0], [2.0, 5.0, 
1.0], [1.0, 6.0, 5.0]], [[4.0, 7.0, 8.0], [1.0, 8.0, 6.0], [4.0, 5.0, 4.0]], [[7.0, 6.0, 1.0], [6.0, 4.0, 4.0], [1.0, 1.0, 
5.0]]], [[[1.0, 6.0, 3.0], [6.0, 8.0, 1.0], [3.0, 1.0, 7.0]], [[1.0, 5.0, 4.0], [8.0, 1.0, 7.0], [3.0, 4.0, 5.0]], [[1.0, 1.0, 
5.0], [6.0, 8.0, 4.0], [4.0, 1.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 6.0], [7.0, 5.0, 6.0], [4.0, 3.0, 5.0]], [[5.0, 7.0, 4.0], [1.0, 4.0, 
5.0], [6.0, 5.0, 4.0]], [[7.0, 7.0, 2.0], [3.0, 8.0, 7.0], [7.0, 4.0, 8.0]]], [[[3.0, 1.0, 3.0], [3.0, 7.0, 8.0], [5.0, 2.0, 
7.0]], [[2.0, 6.0, 1.0], [6.0, 6.0, 2.0], [7.0, 1.0, 3.0]], [[7.0, 2.0, 6.0], [5.0, 2.0, 3.0], [6.0, 6.0, 6.0]]], [[[5.0, 1.0, 
6.0], [3.0, 3.0, 8.0], [2.0, 8.0, 1.0]], [[7.0, 7.0, 3.0], [4.0, 6.0, 2.0], [4.0, 1.0, 4.0]], [[4.0, 6.0, 7.0], [4.0, 8.0, 
7.0], [4.0, 8.0, 3.0]]], [[[4.0, 8.0, 5.0], [1.0, 6.0, 2.0], [3.0, 3.0, 5.0]], [[8.0, 3.0, 2.0], [4.0, 2.0, 8.0], [6.0, 5.0, 
4.0]], [[1.0, 5.0, 6.0], [5.0, 2.0, 6.0], [8.0, 3.0, 4.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[3.0, 2.0, 2.0], [7.0, 7.0, 7.0]], [[2.0, 5.0, 7.0], [3.0, 5.0, 7.0]]], [[[6.0, 3.0, 6.0], 
[6.0, 4.0, 2.0]], [[5.0, 3.0, 8.0], [3.0, 1.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 7.0, 3.0], [7.0, 2.0, 1.0]], [[2.0, 
3.0, 2.0], [7.0, 2.0, 5.0]]], [[[4.0, 3.0, 4.0], [7.0, 8.0, 1.0]], [[7.0, 3.0, 1.0], [1.0, 2.0, 6.0]]]])
      arg[1:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 7.0], [4.0, 8.0, 5.0], [2.0, 7.0, 1.0]], [[3.0, 8.0, 8.0], [1.0, 2.0, 6.0], [4.0, 
3.0, 4.0]], [[7.0, 4.0, 5.0], [6.0, 8.0, 2.0], [1.0, 6.0, 2.0]]], [[[3.0, 2.0, 2.0], [7.0, 7.0, 7.0], [8.0, 8.0, 1.0]], [[2.0, 
5.0, 7.0], [3.0, 5.0, 7.0], [4.0, 5.0, 4.0]], [[3.0, 4.0, 8.0], [4.0, 1.0, 1.0], [6.0, 6.0, 8.0]]], [[[6.0, 3.0, 6.0], [6.0, 
4.0, 2.0], [1.0, 6.0, 5.0]], [[5.0, 3.0, 8.0], [3.0, 1.0, 4.0], [4.0, 5.0, 4.0]], [[7.0, 6.0, 1.0], [6.0, 4.0, 4.0], [1.0, 1.0, 
5.0]]], [[[1.0, 6.0, 3.0], [6.0, 8.0, 1.0], [3.0, 1.0, 7.0]], [[1.0, 5.0, 4.0], [8.0, 1.0, 7.0], [3.0, 4.0, 5.0]], [[1.0, 1.0, 
5.0], [6.0, 8.0, 4.0], [4.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 6.0], [7.0, 5.0, 6.0], [4.0, 3.0, 5.0]], 
[[5.0, 7.0, 4.0], [1.0, 4.0, 5.0], [6.0, 5.0, 4.0]], [[7.0, 7.0, 2.0], [3.0, 8.0, 7.0], [7.0, 4.0, 8.0]]], [[[4.0, 7.0, 3.0], 
[7.0, 2.0, 1.0], [5.0, 2.0, 7.0]], [[2.0, 3.0, 2.0], [7.0, 2.0, 5.0], [7.0, 1.0, 3.0]], [[7.0, 2.0, 6.0], [5.0, 2.0, 3.0], 
[6.0, 6.0, 6.0]]], [[[4.0, 3.0, 4.0], [7.0, 8.0, 1.0], [2.0, 8.0, 1.0]], [[7.0, 3.0, 1.0], [1.0, 2.0, 6.0], [4.0, 1.0, 4.0]], 
[[4.0, 6.0, 7.0], [4.0, 8.0, 7.0], [4.0, 8.0, 3.0]]], [[[4.0, 8.0, 5.0], [1.0, 6.0, 2.0], [3.0, 3.0, 5.0]], [[8.0, 3.0, 2.0], 
[4.0, 2.0, 8.0], [6.0, 5.0, 4.0]], [[1.0, 5.0, 6.0], [5.0, 2.0, 6.0], [8.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LCPN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 4.0], [1.0, 8.0, 2.0], [8.0, 1.0, 6.0]], [[8.0, 6.0, 7.0], [7.0, 8.0, 5.0], [3.0, 8.0, 
7.0]], [[3.0, 3.0, 1.0], [7.0, 3.0, 1.0], [8.0, 1.0, 2.0]]], [[[1.0, 2.0, 8.0], [8.0, 2.0, 7.0], [4.0, 7.0, 6.0]], [[5.0, 4.0, 
1.0], [4.0, 4.0, 4.0], [3.0, 6.0, 1.0]], [[8.0, 7.0, 3.0], [6.0, 4.0, 3.0], [8.0, 8.0, 6.0]]], [[[4.0, 8.0, 4.0], [7.0, 2.0, 
4.0], [1.0, 6.0, 7.0]], [[1.0, 7.0, 4.0], [8.0, 5.0, 8.0], [8.0, 4.0, 4.0]], [[8.0, 2.0, 1.0], [2.0, 6.0, 1.0], [3.0, 2.0, 
3.0]]], [[[6.0, 1.0, 2.0], [2.0, 8.0, 4.0], [4.0, 6.0, 3.0]], [[8.0, 4.0, 4.0], [4.0, 8.0, 3.0], [7.0, 5.0, 3.0]], [[5.0, 8.0, 
1.0], [7.0, 5.0, 4.0], [8.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 7.0], [4.0, 6.0, 7.0], [2.0, 1.0, 7.0]], [[2.0, 4.0, 7.0], [3.0, 2.0, 
6.0], [8.0, 8.0, 1.0]], [[7.0, 4.0, 1.0], [3.0, 1.0, 8.0], [3.0, 1.0, 1.0]]], [[[6.0, 2.0, 6.0], [8.0, 3.0, 5.0], [1.0, 3.0, 
5.0]], [[2.0, 8.0, 8.0], [1.0, 6.0, 4.0], [7.0, 1.0, 4.0]], [[2.0, 8.0, 8.0], [3.0, 1.0, 3.0], [3.0, 4.0, 6.0]]], [[[4.0, 6.0, 
5.0], [7.0, 4.0, 4.0], [8.0, 6.0, 3.0]], [[4.0, 7.0, 6.0], [6.0, 7.0, 7.0], [6.0, 5.0, 7.0]], [[6.0, 2.0, 7.0], [8.0, 4.0, 
8.0], [2.0, 2.0, 1.0]]], [[[4.0, 4.0, 8.0], [1.0, 2.0, 6.0], [2.0, 2.0, 7.0]], [[5.0, 1.0, 7.0], [1.0, 6.0, 6.0], [5.0, 2.0, 
7.0]], [[5.0, 7.0, 3.0], [2.0, 5.0, 1.0], [8.0, 4.0, 1.0]]]]))
      val=numpy.array([[[1.0, 7.0, 6.0], [6.0, 4.0, 7.0], [7.0, 3.0, 6.0]], [[6.0, 8.0, 7.0], [1.0, 6.0, 2.0], [1.0, 4.0, 
3.0]]])
      arg[2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [1.0, 8.0, 2.0], [8.0, 1.0, 6.0]], [[8.0, 6.0, 7.0], [7.0, 8.0, 5.0], [3.0, 
8.0, 7.0]], [[3.0, 3.0, 1.0], [7.0, 3.0, 1.0], [8.0, 1.0, 2.0]]], [[[1.0, 2.0, 8.0], [8.0, 2.0, 7.0], [4.0, 7.0, 6.0]], [[5.0, 
4.0, 1.0], [4.0, 4.0, 4.0], [3.0, 6.0, 1.0]], [[8.0, 7.0, 3.0], [6.0, 4.0, 3.0], [8.0, 8.0, 6.0]]], [[[4.0, 8.0, 4.0], [1.0, 
7.0, 6.0], [1.0, 6.0, 7.0]], [[1.0, 7.0, 4.0], [6.0, 4.0, 7.0], [8.0, 4.0, 4.0]], [[8.0, 2.0, 1.0], [7.0, 3.0, 6.0], [3.0, 2.0, 
3.0]]], [[[6.0, 1.0, 2.0], [6.0, 8.0, 7.0], [4.0, 6.0, 3.0]], [[8.0, 4.0, 4.0], [1.0, 6.0, 2.0], [7.0, 5.0, 3.0]], [[5.0, 8.0, 
1.0], [1.0, 4.0, 3.0], [8.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 7.0], [4.0, 6.0, 7.0], [2.0, 1.0, 7.0]], 
[[2.0, 4.0, 7.0], [3.0, 2.0, 6.0], [8.0, 8.0, 1.0]], [[7.0, 4.0, 1.0], [3.0, 1.0, 8.0], [3.0, 1.0, 1.0]]], [[[6.0, 2.0, 6.0], 
[8.0, 3.0, 5.0], [1.0, 3.0, 5.0]], [[2.0, 8.0, 8.0], [1.0, 6.0, 4.0], [7.0, 1.0, 4.0]], [[2.0, 8.0, 8.0], [3.0, 1.0, 3.0], 
[3.0, 4.0, 6.0]]], [[[4.0, 6.0, 5.0], [1.0, 7.0, 6.0], [8.0, 6.0, 3.0]], [[4.0, 7.0, 6.0], [6.0, 4.0, 7.0], [6.0, 5.0, 7.0]], 
[[6.0, 2.0, 7.0], [7.0, 3.0, 6.0], [2.0, 2.0, 1.0]]], [[[4.0, 4.0, 8.0], [6.0, 8.0, 7.0], [2.0, 2.0, 7.0]], [[5.0, 1.0, 7.0], 
[1.0, 6.0, 2.0], [5.0, 2.0, 7.0]], [[5.0, 7.0, 3.0], [1.0, 4.0, 3.0], [8.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LCPN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 4.0], [3.0, 4.0, 3.0], [2.0, 8.0, 2.0]], [[5.0, 3.0, 2.0], [8.0, 1.0, 2.0], [3.0, 2.0, 
7.0]], [[2.0, 8.0, 5.0], [1.0, 5.0, 7.0], [8.0, 8.0, 7.0]]], [[[3.0, 4.0, 6.0], [2.0, 3.0, 8.0], [2.0, 6.0, 2.0]], [[2.0, 8.0, 
5.0], [4.0, 6.0, 4.0], [1.0, 3.0, 7.0]], [[3.0, 1.0, 2.0], [6.0, 6.0, 6.0], [6.0, 2.0, 1.0]]], [[[3.0, 8.0, 8.0], [3.0, 8.0, 
8.0], [1.0, 8.0, 4.0]], [[1.0, 2.0, 2.0], [8.0, 6.0, 8.0], [4.0, 2.0, 3.0]], [[4.0, 6.0, 6.0], [4.0, 2.0, 2.0], [1.0, 6.0, 
8.0]]], [[[5.0, 5.0, 5.0], [4.0, 7.0, 4.0], [7.0, 6.0, 5.0]], [[6.0, 1.0, 3.0], [8.0, 1.0, 7.0], [3.0, 7.0, 2.0]], [[5.0, 7.0, 
6.0], [3.0, 8.0, 4.0], [4.0, 1.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 3.0], [5.0, 1.0, 5.0], [3.0, 1.0, 1.0]], [[4.0, 1.0, 7.0], [4.0, 1.0, 
6.0], [3.0, 6.0, 3.0]], [[3.0, 3.0, 1.0], [8.0, 2.0, 4.0], [5.0, 5.0, 1.0]]], [[[6.0, 5.0, 8.0], [6.0, 8.0, 6.0], [6.0, 5.0, 
7.0]], [[4.0, 2.0, 2.0], [7.0, 7.0, 1.0], [3.0, 1.0, 2.0]], [[8.0, 5.0, 4.0], [3.0, 6.0, 4.0], [1.0, 8.0, 2.0]]], [[[2.0, 3.0, 
6.0], [6.0, 4.0, 1.0], [1.0, 1.0, 3.0]], [[4.0, 8.0, 4.0], [5.0, 6.0, 3.0], [4.0, 6.0, 7.0]], [[6.0, 5.0, 5.0], [1.0, 6.0, 
1.0], [7.0, 4.0, 4.0]]], [[[1.0, 8.0, 6.0], [4.0, 4.0, 8.0], [6.0, 6.0, 5.0]], [[8.0, 7.0, 1.0], [1.0, 7.0, 4.0], [7.0, 1.0, 
5.0]], [[8.0, 5.0, 3.0], [3.0, 8.0, 7.0], [6.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[7.0, 2.0, 8.0], [1.0, 7.0, 8.0], [1.0, 3.0, 7.0]], [[4.0, 5.0, 6.0], [3.0, 4.0, 4.0], [6.0, 3.0, 
2.0]]]),self.functionspace)
      arg[2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 4.0], [3.0, 4.0, 3.0], [2.0, 8.0, 2.0]], [[5.0, 3.0, 2.0], [8.0, 1.0, 2.0], [3.0, 
2.0, 7.0]], [[2.0, 8.0, 5.0], [1.0, 5.0, 7.0], [8.0, 8.0, 7.0]]], [[[3.0, 4.0, 6.0], [2.0, 3.0, 8.0], [2.0, 6.0, 2.0]], [[2.0, 
8.0, 5.0], [4.0, 6.0, 4.0], [1.0, 3.0, 7.0]], [[3.0, 1.0, 2.0], [6.0, 6.0, 6.0], [6.0, 2.0, 1.0]]], [[[3.0, 8.0, 8.0], [7.0, 
2.0, 8.0], [1.0, 8.0, 4.0]], [[1.0, 2.0, 2.0], [1.0, 7.0, 8.0], [4.0, 2.0, 3.0]], [[4.0, 6.0, 6.0], [1.0, 3.0, 7.0], [1.0, 6.0, 
8.0]]], [[[5.0, 5.0, 5.0], [4.0, 5.0, 6.0], [7.0, 6.0, 5.0]], [[6.0, 1.0, 3.0], [3.0, 4.0, 4.0], [3.0, 7.0, 2.0]], [[5.0, 7.0, 
6.0], [6.0, 3.0, 2.0], [4.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 3.0], [5.0, 1.0, 5.0], [3.0, 1.0, 1.0]], 
[[4.0, 1.0, 7.0], [4.0, 1.0, 6.0], [3.0, 6.0, 3.0]], [[3.0, 3.0, 1.0], [8.0, 2.0, 4.0], [5.0, 5.0, 1.0]]], [[[6.0, 5.0, 8.0], 
[6.0, 8.0, 6.0], [6.0, 5.0, 7.0]], [[4.0, 2.0, 2.0], [7.0, 7.0, 1.0], [3.0, 1.0, 2.0]], [[8.0, 5.0, 4.0], [3.0, 6.0, 4.0], 
[1.0, 8.0, 2.0]]], [[[2.0, 3.0, 6.0], [7.0, 2.0, 8.0], [1.0, 1.0, 3.0]], [[4.0, 8.0, 4.0], [1.0, 7.0, 8.0], [4.0, 6.0, 7.0]], 
[[6.0, 5.0, 5.0], [1.0, 3.0, 7.0], [7.0, 4.0, 4.0]]], [[[1.0, 8.0, 6.0], [4.0, 5.0, 6.0], [6.0, 6.0, 5.0]], [[8.0, 7.0, 1.0], 
[3.0, 4.0, 4.0], [7.0, 1.0, 5.0]], [[8.0, 5.0, 3.0], [6.0, 3.0, 2.0], [6.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LCPN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 2.0], [2.0, 6.0, 4.0], [6.0, 1.0, 6.0]], [[1.0, 3.0, 1.0], [5.0, 1.0, 5.0], [4.0, 6.0, 
4.0]], [[7.0, 4.0, 4.0], [7.0, 6.0, 4.0], [7.0, 5.0, 7.0]]], [[[5.0, 3.0, 3.0], [6.0, 5.0, 5.0], [1.0, 3.0, 5.0]], [[8.0, 2.0, 
4.0], [1.0, 3.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 3.0, 5.0], [2.0, 4.0, 3.0], [1.0, 6.0, 1.0]]], [[[5.0, 6.0, 8.0], [6.0, 3.0, 
2.0], [6.0, 5.0, 3.0]], [[5.0, 6.0, 6.0], [7.0, 5.0, 5.0], [8.0, 6.0, 8.0]], [[7.0, 1.0, 3.0], [8.0, 4.0, 5.0], [4.0, 1.0, 
3.0]]], [[[8.0, 4.0, 1.0], [7.0, 2.0, 6.0], [4.0, 7.0, 2.0]], [[3.0, 5.0, 1.0], [7.0, 4.0, 6.0], [2.0, 7.0, 5.0]], [[1.0, 2.0, 
8.0], [7.0, 5.0, 2.0], [8.0, 5.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 6.0], [1.0, 3.0, 4.0], [4.0, 3.0, 6.0]], [[5.0, 4.0, 7.0], [5.0, 3.0, 
6.0], [1.0, 6.0, 8.0]], [[4.0, 2.0, 4.0], [6.0, 1.0, 1.0], [5.0, 3.0, 1.0]]], [[[7.0, 6.0, 6.0], [8.0, 6.0, 8.0], [2.0, 1.0, 
5.0]], [[8.0, 1.0, 8.0], [3.0, 5.0, 8.0], [6.0, 2.0, 3.0]], [[7.0, 1.0, 7.0], [2.0, 5.0, 6.0], [8.0, 6.0, 6.0]]], [[[2.0, 7.0, 
7.0], [8.0, 5.0, 3.0], [6.0, 4.0, 2.0]], [[1.0, 2.0, 4.0], [2.0, 8.0, 1.0], [5.0, 6.0, 8.0]], [[7.0, 6.0, 3.0], [3.0, 3.0, 
8.0], [3.0, 3.0, 5.0]]], [[[8.0, 3.0, 4.0], [3.0, 2.0, 3.0], [8.0, 7.0, 7.0]], [[4.0, 3.0, 6.0], [2.0, 6.0, 7.0], [3.0, 3.0, 
6.0]], [[1.0, 3.0, 4.0], [7.0, 7.0, 3.0], [8.0, 8.0, 8.0]]]]))
      val=Data(numpy.array([[[6.0, 2.0, 2.0], [6.0, 2.0, 5.0], [8.0, 4.0, 3.0]], [[1.0, 5.0, 1.0], [8.0, 6.0, 4.0], [3.0, 4.0, 
6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 7.0, 2.0], [8.0, 6.0, 2.0], [2.0, 5.0, 3.0]], [[4.0, 3.0, 1.0], [8.0, 8.0, 1.0], 
[1.0, 5.0, 3.0]]]))
      arg[2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 2.0], [2.0, 6.0, 4.0], [6.0, 1.0, 6.0]], [[1.0, 3.0, 1.0], [5.0, 1.0, 5.0], [4.0, 
6.0, 4.0]], [[7.0, 4.0, 4.0], [7.0, 6.0, 4.0], [7.0, 5.0, 7.0]]], [[[5.0, 3.0, 3.0], [6.0, 5.0, 5.0], [1.0, 3.0, 5.0]], [[8.0, 
2.0, 4.0], [1.0, 3.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 3.0, 5.0], [2.0, 4.0, 3.0], [1.0, 6.0, 1.0]]], [[[5.0, 6.0, 8.0], [6.0, 
2.0, 2.0], [6.0, 5.0, 3.0]], [[5.0, 6.0, 6.0], [6.0, 2.0, 5.0], [8.0, 6.0, 8.0]], [[7.0, 1.0, 3.0], [8.0, 4.0, 3.0], [4.0, 1.0, 
3.0]]], [[[8.0, 4.0, 1.0], [1.0, 5.0, 1.0], [4.0, 7.0, 2.0]], [[3.0, 5.0, 1.0], [8.0, 6.0, 4.0], [2.0, 7.0, 5.0]], [[1.0, 2.0, 
8.0], [3.0, 4.0, 6.0], [8.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [1.0, 3.0, 4.0], [4.0, 3.0, 6.0]], 
[[5.0, 4.0, 7.0], [5.0, 3.0, 6.0], [1.0, 6.0, 8.0]], [[4.0, 2.0, 4.0], [6.0, 1.0, 1.0], [5.0, 3.0, 1.0]]], [[[7.0, 6.0, 6.0], 
[8.0, 6.0, 8.0], [2.0, 1.0, 5.0]], [[8.0, 1.0, 8.0], [3.0, 5.0, 8.0], [6.0, 2.0, 3.0]], [[7.0, 1.0, 7.0], [2.0, 5.0, 6.0], 
[8.0, 6.0, 6.0]]], [[[2.0, 7.0, 7.0], [5.0, 7.0, 2.0], [6.0, 4.0, 2.0]], [[1.0, 2.0, 4.0], [8.0, 6.0, 2.0], [5.0, 6.0, 8.0]], 
[[7.0, 6.0, 3.0], [2.0, 5.0, 3.0], [3.0, 3.0, 5.0]]], [[[8.0, 3.0, 4.0], [4.0, 3.0, 1.0], [8.0, 7.0, 7.0]], [[4.0, 3.0, 6.0], 
[8.0, 8.0, 1.0], [3.0, 3.0, 6.0]], [[1.0, 3.0, 4.0], [1.0, 5.0, 3.0], [8.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LCPN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 7.0], [1.0, 4.0, 6.0], [8.0, 7.0, 5.0]], [[1.0, 2.0, 2.0], [4.0, 7.0, 6.0], [8.0, 3.0, 
1.0]], [[4.0, 4.0, 6.0], [7.0, 2.0, 3.0], [4.0, 4.0, 3.0]]], [[[1.0, 5.0, 5.0], [1.0, 6.0, 6.0], [6.0, 7.0, 6.0]], [[5.0, 6.0, 
6.0], [3.0, 7.0, 4.0], [6.0, 1.0, 8.0]], [[4.0, 7.0, 7.0], [7.0, 6.0, 1.0], [4.0, 1.0, 7.0]]], [[[1.0, 1.0, 7.0], [1.0, 8.0, 
5.0], [6.0, 6.0, 3.0]], [[7.0, 2.0, 5.0], [2.0, 3.0, 6.0], [8.0, 5.0, 8.0]], [[6.0, 6.0, 7.0], [8.0, 8.0, 1.0], [6.0, 4.0, 
7.0]]], [[[8.0, 4.0, 1.0], [2.0, 3.0, 5.0], [5.0, 3.0, 3.0]], [[7.0, 4.0, 2.0], [4.0, 8.0, 8.0], [1.0, 3.0, 6.0]], [[1.0, 4.0, 
4.0], [4.0, 7.0, 8.0], [4.0, 6.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 3.0], [4.0, 2.0, 8.0], [1.0, 4.0, 1.0]], [[7.0, 3.0, 3.0], [7.0, 8.0, 
6.0], [8.0, 1.0, 6.0]], [[5.0, 2.0, 3.0], [5.0, 8.0, 2.0], [8.0, 3.0, 4.0]]], [[[7.0, 4.0, 7.0], [5.0, 2.0, 2.0], [8.0, 2.0, 
7.0]], [[7.0, 7.0, 8.0], [2.0, 1.0, 1.0], [4.0, 2.0, 8.0]], [[4.0, 6.0, 5.0], [2.0, 2.0, 7.0], [3.0, 8.0, 7.0]]], [[[4.0, 7.0, 
7.0], [2.0, 4.0, 1.0], [5.0, 3.0, 3.0]], [[3.0, 5.0, 5.0], [6.0, 4.0, 4.0], [2.0, 7.0, 2.0]], [[1.0, 1.0, 7.0], [6.0, 7.0, 
2.0], [1.0, 6.0, 2.0]]], [[[1.0, 4.0, 2.0], [6.0, 6.0, 3.0], [7.0, 6.0, 1.0]], [[2.0, 7.0, 8.0], [4.0, 6.0, 2.0], [7.0, 7.0, 
3.0]], [[3.0, 2.0, 2.0], [4.0, 4.0, 1.0], [5.0, 4.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[5.0, 6.0, 2.0], [5.0, 2.0, 1.0], [7.0, 1.0, 5.0]], [[8.0, 2.0, 1.0], [7.0, 7.0, 7.0], [8.0, 
7.0, 8.0]]])+(1.-msk_val)*numpy.array([[[2.0, 7.0, 6.0], [4.0, 6.0, 5.0], [4.0, 6.0, 5.0]], [[7.0, 7.0, 3.0], [7.0, 7.0, 8.0], 
[6.0, 4.0, 3.0]]])
      arg[2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 7.0], [1.0, 4.0, 6.0], [8.0, 7.0, 5.0]], [[1.0, 2.0, 2.0], [4.0, 7.0, 6.0], [8.0, 
3.0, 1.0]], [[4.0, 4.0, 6.0], [7.0, 2.0, 3.0], [4.0, 4.0, 3.0]]], [[[1.0, 5.0, 5.0], [1.0, 6.0, 6.0], [6.0, 7.0, 6.0]], [[5.0, 
6.0, 6.0], [3.0, 7.0, 4.0], [6.0, 1.0, 8.0]], [[4.0, 7.0, 7.0], [7.0, 6.0, 1.0], [4.0, 1.0, 7.0]]], [[[1.0, 1.0, 7.0], [5.0, 
6.0, 2.0], [6.0, 6.0, 3.0]], [[7.0, 2.0, 5.0], [5.0, 2.0, 1.0], [8.0, 5.0, 8.0]], [[6.0, 6.0, 7.0], [7.0, 1.0, 5.0], [6.0, 4.0, 
7.0]]], [[[8.0, 4.0, 1.0], [8.0, 2.0, 1.0], [5.0, 3.0, 3.0]], [[7.0, 4.0, 2.0], [7.0, 7.0, 7.0], [1.0, 3.0, 6.0]], [[1.0, 4.0, 
4.0], [8.0, 7.0, 8.0], [4.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 3.0], [4.0, 2.0, 8.0], [1.0, 4.0, 1.0]], 
[[7.0, 3.0, 3.0], [7.0, 8.0, 6.0], [8.0, 1.0, 6.0]], [[5.0, 2.0, 3.0], [5.0, 8.0, 2.0], [8.0, 3.0, 4.0]]], [[[7.0, 4.0, 7.0], 
[5.0, 2.0, 2.0], [8.0, 2.0, 7.0]], [[7.0, 7.0, 8.0], [2.0, 1.0, 1.0], [4.0, 2.0, 8.0]], [[4.0, 6.0, 5.0], [2.0, 2.0, 7.0], 
[3.0, 8.0, 7.0]]], [[[4.0, 7.0, 7.0], [2.0, 7.0, 6.0], [5.0, 3.0, 3.0]], [[3.0, 5.0, 5.0], [4.0, 6.0, 5.0], [2.0, 7.0, 2.0]], 
[[1.0, 1.0, 7.0], [4.0, 6.0, 5.0], [1.0, 6.0, 2.0]]], [[[1.0, 4.0, 2.0], [7.0, 7.0, 3.0], [7.0, 6.0, 1.0]], [[2.0, 7.0, 8.0], 
[7.0, 7.0, 8.0], [7.0, 7.0, 3.0]], [[3.0, 2.0, 2.0], [6.0, 4.0, 3.0], [5.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CLCN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 4.0], [2.0, 7.0, 1.0], [2.0, 4.0, 3.0]], [[2.0, 8.0, 1.0], [7.0, 1.0, 6.0], [7.0, 3.0, 
1.0]], [[4.0, 4.0, 8.0], [8.0, 2.0, 8.0], [8.0, 8.0, 6.0]]], [[[6.0, 3.0, 7.0], [8.0, 3.0, 1.0], [5.0, 4.0, 4.0]], [[2.0, 1.0, 
6.0], [6.0, 8.0, 4.0], [4.0, 1.0, 8.0]], [[7.0, 1.0, 1.0], [8.0, 8.0, 5.0], [8.0, 3.0, 5.0]]], [[[4.0, 6.0, 1.0], [1.0, 7.0, 
5.0], [8.0, 5.0, 3.0]], [[7.0, 5.0, 3.0], [8.0, 2.0, 4.0], [4.0, 4.0, 2.0]], [[7.0, 1.0, 7.0], [8.0, 2.0, 8.0], [3.0, 6.0, 
2.0]]], [[[1.0, 8.0, 3.0], [8.0, 2.0, 2.0], [3.0, 5.0, 6.0]], [[3.0, 8.0, 2.0], [7.0, 4.0, 2.0], [3.0, 8.0, 8.0]], [[6.0, 6.0, 
5.0], [6.0, 2.0, 2.0], [8.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 5.0], [1.0, 2.0, 6.0], [1.0, 2.0, 8.0]], [[4.0, 6.0, 1.0], [2.0, 8.0, 
1.0], [7.0, 7.0, 2.0]], [[8.0, 1.0, 6.0], [8.0, 5.0, 1.0], [7.0, 3.0, 3.0]]], [[[2.0, 6.0, 4.0], [6.0, 8.0, 1.0], [7.0, 3.0, 
4.0]], [[3.0, 6.0, 5.0], [4.0, 7.0, 5.0], [5.0, 7.0, 7.0]], [[7.0, 7.0, 4.0], [1.0, 7.0, 5.0], [6.0, 4.0, 5.0]]], [[[4.0, 5.0, 
3.0], [8.0, 7.0, 3.0], [7.0, 3.0, 3.0]], [[2.0, 4.0, 1.0], [7.0, 1.0, 5.0], [5.0, 4.0, 8.0]], [[2.0, 4.0, 7.0], [4.0, 2.0, 
3.0], [6.0, 2.0, 5.0]]], [[[8.0, 4.0, 5.0], [7.0, 7.0, 4.0], [4.0, 1.0, 4.0]], [[5.0, 4.0, 4.0], [7.0, 4.0, 3.0], [1.0, 8.0, 
2.0]], [[6.0, 8.0, 1.0], [3.0, 5.0, 2.0], [5.0, 7.0, 3.0]]]]))
      val=numpy.array([[[[5.0, 5.0, 7.0], [4.0, 3.0, 3.0], [7.0, 6.0, 4.0]]], [[[3.0, 5.0, 6.0], [6.0, 6.0, 5.0], [6.0, 1.0, 
5.0]]], [[[5.0, 7.0, 5.0], [7.0, 6.0, 6.0], [7.0, 6.0, 2.0]]], [[[2.0, 4.0, 1.0], [3.0, 1.0, 8.0], [1.0, 6.0, 7.0]]]])
      arg[:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 4.0], [2.0, 7.0, 1.0], [2.0, 4.0, 3.0]], [[2.0, 8.0, 1.0], [7.0, 1.0, 6.0], [7.0, 
3.0, 1.0]], [[5.0, 5.0, 7.0], [4.0, 3.0, 3.0], [7.0, 6.0, 4.0]]], [[[6.0, 3.0, 7.0], [8.0, 3.0, 1.0], [5.0, 4.0, 4.0]], [[2.0, 
1.0, 6.0], [6.0, 8.0, 4.0], [4.0, 1.0, 8.0]], [[3.0, 5.0, 6.0], [6.0, 6.0, 5.0], [6.0, 1.0, 5.0]]], [[[4.0, 6.0, 1.0], [1.0, 
7.0, 5.0], [8.0, 5.0, 3.0]], [[7.0, 5.0, 3.0], [8.0, 2.0, 4.0], [4.0, 4.0, 2.0]], [[5.0, 7.0, 5.0], [7.0, 6.0, 6.0], [7.0, 6.0, 
2.0]]], [[[1.0, 8.0, 3.0], [8.0, 2.0, 2.0], [3.0, 5.0, 6.0]], [[3.0, 8.0, 2.0], [7.0, 4.0, 2.0], [3.0, 8.0, 8.0]], [[2.0, 4.0, 
1.0], [3.0, 1.0, 8.0], [1.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 5.0], [1.0, 2.0, 6.0], [1.0, 2.0, 8.0]], 
[[4.0, 6.0, 1.0], [2.0, 8.0, 1.0], [7.0, 7.0, 2.0]], [[5.0, 5.0, 7.0], [4.0, 3.0, 3.0], [7.0, 6.0, 4.0]]], [[[2.0, 6.0, 4.0], 
[6.0, 8.0, 1.0], [7.0, 3.0, 4.0]], [[3.0, 6.0, 5.0], [4.0, 7.0, 5.0], [5.0, 7.0, 7.0]], [[3.0, 5.0, 6.0], [6.0, 6.0, 5.0], 
[6.0, 1.0, 5.0]]], [[[4.0, 5.0, 3.0], [8.0, 7.0, 3.0], [7.0, 3.0, 3.0]], [[2.0, 4.0, 1.0], [7.0, 1.0, 5.0], [5.0, 4.0, 8.0]], 
[[5.0, 7.0, 5.0], [7.0, 6.0, 6.0], [7.0, 6.0, 2.0]]], [[[8.0, 4.0, 5.0], [7.0, 7.0, 4.0], [4.0, 1.0, 4.0]], [[5.0, 4.0, 4.0], 
[7.0, 4.0, 3.0], [1.0, 8.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 1.0, 8.0], [1.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CLCN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 3.0], [8.0, 4.0, 6.0], [2.0, 7.0, 1.0]], [[8.0, 4.0, 2.0], [6.0, 4.0, 1.0], [1.0, 4.0, 
8.0]], [[3.0, 5.0, 1.0], [5.0, 2.0, 4.0], [1.0, 4.0, 4.0]]], [[[4.0, 7.0, 5.0], [1.0, 1.0, 8.0], [3.0, 2.0, 6.0]], [[4.0, 8.0, 
4.0], [3.0, 2.0, 6.0], [8.0, 2.0, 1.0]], [[6.0, 1.0, 5.0], [2.0, 7.0, 5.0], [2.0, 6.0, 6.0]]], [[[2.0, 1.0, 4.0], [2.0, 5.0, 
7.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 6.0], [6.0, 5.0, 3.0], [6.0, 3.0, 5.0]], [[4.0, 3.0, 8.0], [3.0, 8.0, 7.0], [2.0, 3.0, 
3.0]]], [[[2.0, 3.0, 7.0], [3.0, 7.0, 7.0], [2.0, 3.0, 3.0]], [[1.0, 8.0, 4.0], [2.0, 5.0, 5.0], [1.0, 7.0, 4.0]], [[4.0, 3.0, 
3.0], [8.0, 5.0, 3.0], [6.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 5.0], [5.0, 4.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 6.0, 5.0], [6.0, 5.0, 
1.0], [6.0, 7.0, 6.0]], [[8.0, 6.0, 8.0], [4.0, 8.0, 5.0], [3.0, 6.0, 3.0]]], [[[5.0, 8.0, 8.0], [6.0, 8.0, 5.0], [4.0, 6.0, 
5.0]], [[4.0, 8.0, 5.0], [6.0, 8.0, 1.0], [5.0, 1.0, 4.0]], [[7.0, 8.0, 5.0], [5.0, 3.0, 6.0], [8.0, 2.0, 1.0]]], [[[6.0, 8.0, 
5.0], [8.0, 5.0, 5.0], [8.0, 5.0, 7.0]], [[6.0, 6.0, 2.0], [2.0, 4.0, 7.0], [6.0, 4.0, 6.0]], [[1.0, 6.0, 6.0], [4.0, 5.0, 
2.0], [5.0, 6.0, 8.0]]], [[[4.0, 7.0, 3.0], [8.0, 5.0, 7.0], [3.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], [6.0, 2.0, 6.0], [8.0, 2.0, 
4.0]], [[8.0, 7.0, 7.0], [5.0, 1.0, 7.0], [7.0, 6.0, 5.0]]]]))
      val=Data(numpy.array([[[[2.0, 8.0, 6.0], [6.0, 5.0, 3.0], [1.0, 2.0, 1.0]]], [[[5.0, 3.0, 7.0], [4.0, 6.0, 6.0], [6.0, 
3.0, 2.0]]], [[[6.0, 7.0, 5.0], [4.0, 1.0, 4.0], [3.0, 6.0, 1.0]]], [[[6.0, 6.0, 7.0], [5.0, 3.0, 7.0], [2.0, 6.0, 
4.0]]]]),self.functionspace)
      arg[:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 3.0], [8.0, 4.0, 6.0], [2.0, 7.0, 1.0]], [[8.0, 4.0, 2.0], [6.0, 4.0, 1.0], [1.0, 
4.0, 8.0]], [[2.0, 8.0, 6.0], [6.0, 5.0, 3.0], [1.0, 2.0, 1.0]]], [[[4.0, 7.0, 5.0], [1.0, 1.0, 8.0], [3.0, 2.0, 6.0]], [[4.0, 
8.0, 4.0], [3.0, 2.0, 6.0], [8.0, 2.0, 1.0]], [[5.0, 3.0, 7.0], [4.0, 6.0, 6.0], [6.0, 3.0, 2.0]]], [[[2.0, 1.0, 4.0], [2.0, 
5.0, 7.0], [2.0, 3.0, 1.0]], [[6.0, 4.0, 6.0], [6.0, 5.0, 3.0], [6.0, 3.0, 5.0]], [[6.0, 7.0, 5.0], [4.0, 1.0, 4.0], [3.0, 6.0, 
1.0]]], [[[2.0, 3.0, 7.0], [3.0, 7.0, 7.0], [2.0, 3.0, 3.0]], [[1.0, 8.0, 4.0], [2.0, 5.0, 5.0], [1.0, 7.0, 4.0]], [[6.0, 6.0, 
7.0], [5.0, 3.0, 7.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 5.0], [5.0, 4.0, 5.0], [6.0, 5.0, 6.0]], 
[[2.0, 6.0, 5.0], [6.0, 5.0, 1.0], [6.0, 7.0, 6.0]], [[2.0, 8.0, 6.0], [6.0, 5.0, 3.0], [1.0, 2.0, 1.0]]], [[[5.0, 8.0, 8.0], 
[6.0, 8.0, 5.0], [4.0, 6.0, 5.0]], [[4.0, 8.0, 5.0], [6.0, 8.0, 1.0], [5.0, 1.0, 4.0]], [[5.0, 3.0, 7.0], [4.0, 6.0, 6.0], 
[6.0, 3.0, 2.0]]], [[[6.0, 8.0, 5.0], [8.0, 5.0, 5.0], [8.0, 5.0, 7.0]], [[6.0, 6.0, 2.0], [2.0, 4.0, 7.0], [6.0, 4.0, 6.0]], 
[[6.0, 7.0, 5.0], [4.0, 1.0, 4.0], [3.0, 6.0, 1.0]]], [[[4.0, 7.0, 3.0], [8.0, 5.0, 7.0], [3.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], 
[6.0, 2.0, 6.0], [8.0, 2.0, 4.0]], [[6.0, 6.0, 7.0], [5.0, 3.0, 7.0], [2.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CLCN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 1.0], [8.0, 7.0, 2.0], [5.0, 2.0, 2.0]], [[7.0, 1.0, 7.0], [5.0, 5.0, 7.0], [5.0, 2.0, 
7.0]], [[7.0, 4.0, 1.0], [7.0, 3.0, 7.0], [1.0, 6.0, 7.0]]], [[[2.0, 8.0, 6.0], [2.0, 8.0, 8.0], [8.0, 6.0, 7.0]], [[3.0, 8.0, 
1.0], [4.0, 3.0, 2.0], [3.0, 6.0, 8.0]], [[5.0, 7.0, 5.0], [6.0, 3.0, 2.0], [5.0, 7.0, 5.0]]], [[[1.0, 3.0, 2.0], [5.0, 7.0, 
2.0], [3.0, 8.0, 3.0]], [[8.0, 8.0, 6.0], [1.0, 8.0, 6.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 3.0], [8.0, 1.0, 3.0], [8.0, 8.0, 
4.0]]], [[[7.0, 1.0, 2.0], [6.0, 8.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 4.0, 6.0], [2.0, 7.0, 8.0], [8.0, 8.0, 1.0]], [[6.0, 2.0, 
1.0], [5.0, 4.0, 3.0], [5.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 3.0], [1.0, 5.0, 1.0], [5.0, 4.0, 6.0]], [[4.0, 5.0, 6.0], [2.0, 4.0, 
8.0], [2.0, 8.0, 5.0]], [[3.0, 4.0, 5.0], [7.0, 3.0, 6.0], [7.0, 5.0, 3.0]]], [[[4.0, 8.0, 1.0], [5.0, 7.0, 2.0], [2.0, 6.0, 
5.0]], [[6.0, 5.0, 5.0], [5.0, 1.0, 8.0], [8.0, 8.0, 2.0]], [[2.0, 2.0, 4.0], [1.0, 3.0, 5.0], [4.0, 3.0, 8.0]]], [[[5.0, 5.0, 
2.0], [7.0, 1.0, 6.0], [7.0, 3.0, 1.0]], [[3.0, 1.0, 8.0], [8.0, 6.0, 4.0], [7.0, 7.0, 2.0]], [[6.0, 8.0, 8.0], [8.0, 6.0, 
5.0], [7.0, 2.0, 2.0]]], [[[2.0, 8.0, 2.0], [5.0, 6.0, 8.0], [4.0, 3.0, 4.0]], [[2.0, 4.0, 2.0], [1.0, 6.0, 1.0], [6.0, 7.0, 
6.0]], [[2.0, 5.0, 5.0], [6.0, 2.0, 3.0], [3.0, 6.0, 4.0]]]]))
      val=Data(numpy.array([[[[4.0, 6.0, 2.0], [2.0, 5.0, 5.0], [7.0, 5.0, 2.0]]], [[[2.0, 3.0, 3.0], [5.0, 6.0, 3.0], [7.0, 
3.0, 7.0]]], [[[8.0, 7.0, 3.0], [1.0, 6.0, 1.0], [4.0, 7.0, 5.0]]], [[[1.0, 4.0, 2.0], [6.0, 2.0, 3.0], [3.0, 8.0, 
5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 2.0], [1.0, 4.0, 6.0], [8.0, 6.0, 1.0]]], [[[8.0, 3.0, 7.0], [8.0, 5.0, 
8.0], [7.0, 5.0, 2.0]]], [[[6.0, 4.0, 8.0], [2.0, 5.0, 1.0], [5.0, 1.0, 5.0]]], [[[3.0, 6.0, 7.0], [7.0, 5.0, 5.0], [1.0, 8.0, 
3.0]]]]))
      arg[:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 1.0], [8.0, 7.0, 2.0], [5.0, 2.0, 2.0]], [[7.0, 1.0, 7.0], [5.0, 5.0, 7.0], [5.0, 
2.0, 7.0]], [[4.0, 6.0, 2.0], [2.0, 5.0, 5.0], [7.0, 5.0, 2.0]]], [[[2.0, 8.0, 6.0], [2.0, 8.0, 8.0], [8.0, 6.0, 7.0]], [[3.0, 
8.0, 1.0], [4.0, 3.0, 2.0], [3.0, 6.0, 8.0]], [[2.0, 3.0, 3.0], [5.0, 6.0, 3.0], [7.0, 3.0, 7.0]]], [[[1.0, 3.0, 2.0], [5.0, 
7.0, 2.0], [3.0, 8.0, 3.0]], [[8.0, 8.0, 6.0], [1.0, 8.0, 6.0], [2.0, 4.0, 7.0]], [[8.0, 7.0, 3.0], [1.0, 6.0, 1.0], [4.0, 7.0, 
5.0]]], [[[7.0, 1.0, 2.0], [6.0, 8.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 4.0, 6.0], [2.0, 7.0, 8.0], [8.0, 8.0, 1.0]], [[1.0, 4.0, 
2.0], [6.0, 2.0, 3.0], [3.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 3.0], [1.0, 5.0, 1.0], [5.0, 4.0, 6.0]], 
[[4.0, 5.0, 6.0], [2.0, 4.0, 8.0], [2.0, 8.0, 5.0]], [[3.0, 1.0, 2.0], [1.0, 4.0, 6.0], [8.0, 6.0, 1.0]]], [[[4.0, 8.0, 1.0], 
[5.0, 7.0, 2.0], [2.0, 6.0, 5.0]], [[6.0, 5.0, 5.0], [5.0, 1.0, 8.0], [8.0, 8.0, 2.0]], [[8.0, 3.0, 7.0], [8.0, 5.0, 8.0], 
[7.0, 5.0, 2.0]]], [[[5.0, 5.0, 2.0], [7.0, 1.0, 6.0], [7.0, 3.0, 1.0]], [[3.0, 1.0, 8.0], [8.0, 6.0, 4.0], [7.0, 7.0, 2.0]], 
[[6.0, 4.0, 8.0], [2.0, 5.0, 1.0], [5.0, 1.0, 5.0]]], [[[2.0, 8.0, 2.0], [5.0, 6.0, 8.0], [4.0, 3.0, 4.0]], [[2.0, 4.0, 2.0], 
[1.0, 6.0, 1.0], [6.0, 7.0, 6.0]], [[3.0, 6.0, 7.0], [7.0, 5.0, 5.0], [1.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CLCN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 7.0], [2.0, 3.0, 2.0], [1.0, 8.0, 7.0]], [[2.0, 1.0, 3.0], [5.0, 3.0, 2.0], [3.0, 8.0, 
1.0]], [[7.0, 3.0, 8.0], [4.0, 2.0, 4.0], [7.0, 1.0, 3.0]]], [[[7.0, 2.0, 6.0], [3.0, 7.0, 3.0], [7.0, 7.0, 3.0]], [[2.0, 6.0, 
5.0], [8.0, 7.0, 5.0], [1.0, 4.0, 1.0]], [[7.0, 6.0, 3.0], [3.0, 5.0, 8.0], [6.0, 8.0, 6.0]]], [[[4.0, 2.0, 3.0], [8.0, 4.0, 
1.0], [8.0, 6.0, 4.0]], [[6.0, 3.0, 7.0], [1.0, 4.0, 4.0], [3.0, 2.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 4.0, 4.0], [4.0, 6.0, 
4.0]]], [[[5.0, 8.0, 1.0], [8.0, 1.0, 8.0], [3.0, 3.0, 5.0]], [[4.0, 8.0, 1.0], [6.0, 7.0, 5.0], [7.0, 6.0, 5.0]], [[6.0, 5.0, 
6.0], [2.0, 5.0, 8.0], [3.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 7.0], [1.0, 6.0, 7.0], [5.0, 6.0, 8.0]], [[5.0, 1.0, 7.0], [6.0, 2.0, 
8.0], [8.0, 1.0, 4.0]], [[1.0, 6.0, 6.0], [2.0, 5.0, 3.0], [8.0, 4.0, 8.0]]], [[[1.0, 8.0, 5.0], [2.0, 6.0, 5.0], [7.0, 8.0, 
8.0]], [[6.0, 5.0, 5.0], [4.0, 3.0, 6.0], [6.0, 7.0, 8.0]], [[4.0, 3.0, 3.0], [2.0, 7.0, 6.0], [3.0, 4.0, 5.0]]], [[[7.0, 5.0, 
1.0], [8.0, 3.0, 4.0], [6.0, 3.0, 5.0]], [[4.0, 4.0, 3.0], [6.0, 6.0, 5.0], [3.0, 8.0, 5.0]], [[4.0, 7.0, 6.0], [5.0, 6.0, 
7.0], [2.0, 2.0, 7.0]]], [[[8.0, 7.0, 5.0], [7.0, 6.0, 5.0], [2.0, 4.0, 1.0]], [[7.0, 4.0, 4.0], [2.0, 8.0, 4.0], [1.0, 2.0, 
2.0]], [[1.0, 2.0, 8.0], [7.0, 4.0, 2.0], [8.0, 2.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[7.0, 1.0, 5.0], [8.0, 1.0, 2.0], [6.0, 8.0, 5.0]]], [[[6.0, 1.0, 3.0], [1.0, 5.0, 7.0], [7.0, 
2.0, 4.0]]], [[[8.0, 3.0, 1.0], [2.0, 5.0, 5.0], [1.0, 8.0, 5.0]]], [[[3.0, 6.0, 5.0], [3.0, 4.0, 1.0], [1.0, 7.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 6.0, 6.0], [3.0, 6.0, 5.0], [3.0, 5.0, 3.0]]], [[[5.0, 7.0, 7.0], [8.0, 2.0, 5.0], 
[2.0, 3.0, 7.0]]], [[[8.0, 5.0, 1.0], [6.0, 2.0, 3.0], [5.0, 3.0, 8.0]]], [[[8.0, 3.0, 7.0], [8.0, 4.0, 1.0], [2.0, 4.0, 
4.0]]]])
      arg[:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 7.0], [2.0, 3.0, 2.0], [1.0, 8.0, 7.0]], [[2.0, 1.0, 3.0], [5.0, 3.0, 2.0], [3.0, 
8.0, 1.0]], [[7.0, 1.0, 5.0], [8.0, 1.0, 2.0], [6.0, 8.0, 5.0]]], [[[7.0, 2.0, 6.0], [3.0, 7.0, 3.0], [7.0, 7.0, 3.0]], [[2.0, 
6.0, 5.0], [8.0, 7.0, 5.0], [1.0, 4.0, 1.0]], [[6.0, 1.0, 3.0], [1.0, 5.0, 7.0], [7.0, 2.0, 4.0]]], [[[4.0, 2.0, 3.0], [8.0, 
4.0, 1.0], [8.0, 6.0, 4.0]], [[6.0, 3.0, 7.0], [1.0, 4.0, 4.0], [3.0, 2.0, 8.0]], [[8.0, 3.0, 1.0], [2.0, 5.0, 5.0], [1.0, 8.0, 
5.0]]], [[[5.0, 8.0, 1.0], [8.0, 1.0, 8.0], [3.0, 3.0, 5.0]], [[4.0, 8.0, 1.0], [6.0, 7.0, 5.0], [7.0, 6.0, 5.0]], [[3.0, 6.0, 
5.0], [3.0, 4.0, 1.0], [1.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 7.0], [1.0, 6.0, 7.0], [5.0, 6.0, 8.0]], 
[[5.0, 1.0, 7.0], [6.0, 2.0, 8.0], [8.0, 1.0, 4.0]], [[7.0, 6.0, 6.0], [3.0, 6.0, 5.0], [3.0, 5.0, 3.0]]], [[[1.0, 8.0, 5.0], 
[2.0, 6.0, 5.0], [7.0, 8.0, 8.0]], [[6.0, 5.0, 5.0], [4.0, 3.0, 6.0], [6.0, 7.0, 8.0]], [[5.0, 7.0, 7.0], [8.0, 2.0, 5.0], 
[2.0, 3.0, 7.0]]], [[[7.0, 5.0, 1.0], [8.0, 3.0, 4.0], [6.0, 3.0, 5.0]], [[4.0, 4.0, 3.0], [6.0, 6.0, 5.0], [3.0, 8.0, 5.0]], 
[[8.0, 5.0, 1.0], [6.0, 2.0, 3.0], [5.0, 3.0, 8.0]]], [[[8.0, 7.0, 5.0], [7.0, 6.0, 5.0], [2.0, 4.0, 1.0]], [[7.0, 4.0, 4.0], 
[2.0, 8.0, 4.0], [1.0, 2.0, 2.0]], [[8.0, 3.0, 7.0], [8.0, 4.0, 1.0], [2.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_LLCN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 2.0], [5.0, 8.0, 7.0], [5.0, 5.0, 2.0]], [[4.0, 1.0, 3.0], [4.0, 2.0, 2.0], [4.0, 6.0, 
6.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 2.0], [5.0, 8.0, 7.0]]], [[[6.0, 6.0, 8.0], [5.0, 8.0, 2.0], [2.0, 3.0, 1.0]], [[6.0, 6.0, 
8.0], [7.0, 8.0, 1.0], [2.0, 8.0, 7.0]], [[6.0, 1.0, 6.0], [3.0, 1.0, 2.0], [7.0, 6.0, 2.0]]], [[[6.0, 6.0, 8.0], [8.0, 4.0, 
8.0], [4.0, 8.0, 3.0]], [[8.0, 5.0, 3.0], [1.0, 5.0, 2.0], [8.0, 5.0, 5.0]], [[3.0, 3.0, 4.0], [6.0, 4.0, 1.0], [5.0, 6.0, 
1.0]]], [[[5.0, 8.0, 4.0], [3.0, 3.0, 2.0], [1.0, 6.0, 1.0]], [[7.0, 2.0, 2.0], [6.0, 2.0, 6.0], [5.0, 5.0, 4.0]], [[3.0, 8.0, 
6.0], [5.0, 8.0, 5.0], [7.0, 8.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 5.0], [8.0, 8.0, 4.0], [6.0, 5.0, 4.0]], [[1.0, 4.0, 7.0], [2.0, 2.0, 
3.0], [7.0, 6.0, 3.0]], [[5.0, 3.0, 2.0], [6.0, 8.0, 2.0], [3.0, 8.0, 2.0]]], [[[1.0, 1.0, 2.0], [3.0, 2.0, 2.0], [2.0, 3.0, 
5.0]], [[8.0, 7.0, 5.0], [5.0, 2.0, 7.0], [3.0, 3.0, 6.0]], [[6.0, 7.0, 1.0], [6.0, 2.0, 5.0], [4.0, 6.0, 7.0]]], [[[1.0, 1.0, 
2.0], [2.0, 4.0, 7.0], [7.0, 2.0, 2.0]], [[1.0, 4.0, 6.0], [4.0, 1.0, 6.0], [2.0, 2.0, 2.0]], [[2.0, 3.0, 4.0], [8.0, 5.0, 
6.0], [3.0, 7.0, 8.0]]], [[[4.0, 8.0, 4.0], [5.0, 6.0, 5.0], [4.0, 7.0, 8.0]], [[1.0, 4.0, 5.0], [5.0, 8.0, 7.0], [8.0, 1.0, 
7.0]], [[5.0, 5.0, 2.0], [5.0, 8.0, 5.0], [3.0, 1.0, 7.0]]]]))
      val=numpy.array([[[[7.0, 1.0, 8.0], [1.0, 2.0, 5.0], [5.0, 6.0, 1.0]]], [[[6.0, 6.0, 3.0], [8.0, 8.0, 5.0], [3.0, 3.0, 
8.0]]]])
      arg[2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 2.0], [5.0, 8.0, 7.0], [5.0, 5.0, 2.0]], [[4.0, 1.0, 3.0], [4.0, 2.0, 2.0], [4.0, 
6.0, 6.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 2.0], [5.0, 8.0, 7.0]]], [[[6.0, 6.0, 8.0], [5.0, 8.0, 2.0], [2.0, 3.0, 1.0]], [[6.0, 
6.0, 8.0], [7.0, 8.0, 1.0], [2.0, 8.0, 7.0]], [[6.0, 1.0, 6.0], [3.0, 1.0, 2.0], [7.0, 6.0, 2.0]]], [[[6.0, 6.0, 8.0], [8.0, 
4.0, 8.0], [4.0, 8.0, 3.0]], [[8.0, 5.0, 3.0], [1.0, 5.0, 2.0], [8.0, 5.0, 5.0]], [[7.0, 1.0, 8.0], [1.0, 2.0, 5.0], [5.0, 6.0, 
1.0]]], [[[5.0, 8.0, 4.0], [3.0, 3.0, 2.0], [1.0, 6.0, 1.0]], [[7.0, 2.0, 2.0], [6.0, 2.0, 6.0], [5.0, 5.0, 4.0]], [[6.0, 6.0, 
3.0], [8.0, 8.0, 5.0], [3.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 5.0], [8.0, 8.0, 4.0], [6.0, 5.0, 4.0]], 
[[1.0, 4.0, 7.0], [2.0, 2.0, 3.0], [7.0, 6.0, 3.0]], [[5.0, 3.0, 2.0], [6.0, 8.0, 2.0], [3.0, 8.0, 2.0]]], [[[1.0, 1.0, 2.0], 
[3.0, 2.0, 2.0], [2.0, 3.0, 5.0]], [[8.0, 7.0, 5.0], [5.0, 2.0, 7.0], [3.0, 3.0, 6.0]], [[6.0, 7.0, 1.0], [6.0, 2.0, 5.0], 
[4.0, 6.0, 7.0]]], [[[1.0, 1.0, 2.0], [2.0, 4.0, 7.0], [7.0, 2.0, 2.0]], [[1.0, 4.0, 6.0], [4.0, 1.0, 6.0], [2.0, 2.0, 2.0]], 
[[7.0, 1.0, 8.0], [1.0, 2.0, 5.0], [5.0, 6.0, 1.0]]], [[[4.0, 8.0, 4.0], [5.0, 6.0, 5.0], [4.0, 7.0, 8.0]], [[1.0, 4.0, 5.0], 
[5.0, 8.0, 7.0], [8.0, 1.0, 7.0]], [[6.0, 6.0, 3.0], [8.0, 8.0, 5.0], [3.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_LLCN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 2.0], [7.0, 7.0, 3.0], [4.0, 6.0, 1.0]], [[4.0, 7.0, 2.0], [7.0, 8.0, 1.0], [2.0, 5.0, 
4.0]], [[3.0, 5.0, 8.0], [1.0, 5.0, 2.0], [2.0, 2.0, 2.0]]], [[[3.0, 3.0, 2.0], [8.0, 1.0, 3.0], [5.0, 6.0, 6.0]], [[7.0, 3.0, 
7.0], [8.0, 5.0, 3.0], [5.0, 4.0, 1.0]], [[1.0, 5.0, 5.0], [1.0, 3.0, 4.0], [8.0, 1.0, 4.0]]], [[[2.0, 3.0, 4.0], [5.0, 7.0, 
2.0], [6.0, 7.0, 7.0]], [[4.0, 2.0, 5.0], [7.0, 5.0, 4.0], [4.0, 3.0, 4.0]], [[4.0, 4.0, 7.0], [7.0, 2.0, 8.0], [3.0, 4.0, 
5.0]]], [[[8.0, 1.0, 2.0], [3.0, 1.0, 2.0], [1.0, 6.0, 3.0]], [[3.0, 6.0, 7.0], [8.0, 3.0, 5.0], [4.0, 8.0, 5.0]], [[3.0, 2.0, 
1.0], [6.0, 2.0, 3.0], [4.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 3.0, 4.0], [3.0, 2.0, 2.0], [5.0, 8.0, 6.0]], [[6.0, 5.0, 6.0], [4.0, 1.0, 
6.0], [6.0, 3.0, 6.0]], [[3.0, 7.0, 4.0], [4.0, 7.0, 8.0], [6.0, 4.0, 4.0]]], [[[6.0, 2.0, 2.0], [7.0, 3.0, 1.0], [7.0, 1.0, 
2.0]], [[3.0, 6.0, 2.0], [3.0, 3.0, 8.0], [4.0, 3.0, 4.0]], [[7.0, 7.0, 6.0], [2.0, 8.0, 4.0], [8.0, 2.0, 8.0]]], [[[8.0, 8.0, 
6.0], [6.0, 2.0, 3.0], [8.0, 1.0, 1.0]], [[5.0, 7.0, 1.0], [6.0, 3.0, 2.0], [3.0, 8.0, 2.0]], [[3.0, 1.0, 1.0], [1.0, 2.0, 
4.0], [8.0, 7.0, 2.0]]], [[[8.0, 5.0, 3.0], [5.0, 7.0, 8.0], [2.0, 3.0, 2.0]], [[5.0, 3.0, 4.0], [8.0, 4.0, 2.0], [3.0, 3.0, 
8.0]], [[5.0, 2.0, 7.0], [6.0, 3.0, 7.0], [8.0, 6.0, 5.0]]]]))
      val=Data(numpy.array([[[[4.0, 7.0, 1.0], [4.0, 7.0, 2.0], [5.0, 1.0, 7.0]]], [[[7.0, 4.0, 4.0], [2.0, 6.0, 6.0], [3.0, 
3.0, 3.0]]]]),self.functionspace)
      arg[2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 2.0], [7.0, 7.0, 3.0], [4.0, 6.0, 1.0]], [[4.0, 7.0, 2.0], [7.0, 8.0, 1.0], [2.0, 
5.0, 4.0]], [[3.0, 5.0, 8.0], [1.0, 5.0, 2.0], [2.0, 2.0, 2.0]]], [[[3.0, 3.0, 2.0], [8.0, 1.0, 3.0], [5.0, 6.0, 6.0]], [[7.0, 
3.0, 7.0], [8.0, 5.0, 3.0], [5.0, 4.0, 1.0]], [[1.0, 5.0, 5.0], [1.0, 3.0, 4.0], [8.0, 1.0, 4.0]]], [[[2.0, 3.0, 4.0], [5.0, 
7.0, 2.0], [6.0, 7.0, 7.0]], [[4.0, 2.0, 5.0], [7.0, 5.0, 4.0], [4.0, 3.0, 4.0]], [[4.0, 7.0, 1.0], [4.0, 7.0, 2.0], [5.0, 1.0, 
7.0]]], [[[8.0, 1.0, 2.0], [3.0, 1.0, 2.0], [1.0, 6.0, 3.0]], [[3.0, 6.0, 7.0], [8.0, 3.0, 5.0], [4.0, 8.0, 5.0]], [[7.0, 4.0, 
4.0], [2.0, 6.0, 6.0], [3.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 4.0], [3.0, 2.0, 2.0], [5.0, 8.0, 6.0]], 
[[6.0, 5.0, 6.0], [4.0, 1.0, 6.0], [6.0, 3.0, 6.0]], [[3.0, 7.0, 4.0], [4.0, 7.0, 8.0], [6.0, 4.0, 4.0]]], [[[6.0, 2.0, 2.0], 
[7.0, 3.0, 1.0], [7.0, 1.0, 2.0]], [[3.0, 6.0, 2.0], [3.0, 3.0, 8.0], [4.0, 3.0, 4.0]], [[7.0, 7.0, 6.0], [2.0, 8.0, 4.0], 
[8.0, 2.0, 8.0]]], [[[8.0, 8.0, 6.0], [6.0, 2.0, 3.0], [8.0, 1.0, 1.0]], [[5.0, 7.0, 1.0], [6.0, 3.0, 2.0], [3.0, 8.0, 2.0]], 
[[4.0, 7.0, 1.0], [4.0, 7.0, 2.0], [5.0, 1.0, 7.0]]], [[[8.0, 5.0, 3.0], [5.0, 7.0, 8.0], [2.0, 3.0, 2.0]], [[5.0, 3.0, 4.0], 
[8.0, 4.0, 2.0], [3.0, 3.0, 8.0]], [[7.0, 4.0, 4.0], [2.0, 6.0, 6.0], [3.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_LLCN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 5.0], [5.0, 5.0, 3.0], [8.0, 8.0, 2.0]], [[3.0, 8.0, 7.0], [8.0, 6.0, 1.0], [3.0, 2.0, 
1.0]], [[2.0, 2.0, 6.0], [8.0, 3.0, 7.0], [6.0, 3.0, 8.0]]], [[[4.0, 8.0, 3.0], [5.0, 5.0, 5.0], [4.0, 3.0, 1.0]], [[6.0, 1.0, 
4.0], [5.0, 7.0, 2.0], [3.0, 8.0, 4.0]], [[4.0, 7.0, 2.0], [8.0, 3.0, 3.0], [4.0, 6.0, 1.0]]], [[[3.0, 8.0, 5.0], [6.0, 7.0, 
6.0], [1.0, 7.0, 4.0]], [[2.0, 6.0, 3.0], [1.0, 1.0, 4.0], [5.0, 3.0, 8.0]], [[8.0, 7.0, 2.0], [5.0, 1.0, 5.0], [5.0, 1.0, 
3.0]]], [[[5.0, 8.0, 5.0], [7.0, 7.0, 5.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 1.0], [2.0, 2.0, 2.0], [3.0, 7.0, 5.0]], [[8.0, 6.0, 
8.0], [3.0, 5.0, 6.0], [7.0, 1.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 5.0], [3.0, 4.0, 3.0], [7.0, 2.0, 7.0]], [[4.0, 4.0, 1.0], [6.0, 8.0, 
7.0], [5.0, 5.0, 8.0]], [[7.0, 3.0, 5.0], [8.0, 8.0, 8.0], [7.0, 5.0, 2.0]]], [[[8.0, 3.0, 1.0], [7.0, 5.0, 1.0], [2.0, 7.0, 
5.0]], [[4.0, 3.0, 6.0], [6.0, 8.0, 6.0], [3.0, 7.0, 1.0]], [[4.0, 5.0, 5.0], [2.0, 6.0, 1.0], [3.0, 4.0, 3.0]]], [[[2.0, 4.0, 
2.0], [6.0, 8.0, 3.0], [7.0, 6.0, 6.0]], [[5.0, 6.0, 7.0], [3.0, 1.0, 6.0], [6.0, 1.0, 1.0]], [[1.0, 5.0, 8.0], [8.0, 1.0, 
2.0], [4.0, 5.0, 5.0]]], [[[5.0, 2.0, 7.0], [5.0, 8.0, 6.0], [5.0, 1.0, 6.0]], [[1.0, 7.0, 5.0], [2.0, 1.0, 8.0], [7.0, 7.0, 
7.0]], [[5.0, 4.0, 3.0], [7.0, 6.0, 8.0], [6.0, 1.0, 1.0]]]]))
      val=Data(numpy.array([[[[6.0, 5.0, 3.0], [4.0, 8.0, 5.0], [1.0, 7.0, 4.0]]], [[[3.0, 3.0, 7.0], [6.0, 6.0, 4.0], [6.0, 
8.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 8.0], [3.0, 3.0, 5.0], [4.0, 4.0, 8.0]]], [[[5.0, 4.0, 8.0], [5.0, 6.0, 
2.0], [5.0, 2.0, 7.0]]]]))
      arg[2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 5.0], [5.0, 5.0, 3.0], [8.0, 8.0, 2.0]], [[3.0, 8.0, 7.0], [8.0, 6.0, 1.0], [3.0, 
2.0, 1.0]], [[2.0, 2.0, 6.0], [8.0, 3.0, 7.0], [6.0, 3.0, 8.0]]], [[[4.0, 8.0, 3.0], [5.0, 5.0, 5.0], [4.0, 3.0, 1.0]], [[6.0, 
1.0, 4.0], [5.0, 7.0, 2.0], [3.0, 8.0, 4.0]], [[4.0, 7.0, 2.0], [8.0, 3.0, 3.0], [4.0, 6.0, 1.0]]], [[[3.0, 8.0, 5.0], [6.0, 
7.0, 6.0], [1.0, 7.0, 4.0]], [[2.0, 6.0, 3.0], [1.0, 1.0, 4.0], [5.0, 3.0, 8.0]], [[6.0, 5.0, 3.0], [4.0, 8.0, 5.0], [1.0, 7.0, 
4.0]]], [[[5.0, 8.0, 5.0], [7.0, 7.0, 5.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 1.0], [2.0, 2.0, 2.0], [3.0, 7.0, 5.0]], [[3.0, 3.0, 
7.0], [6.0, 6.0, 4.0], [6.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 5.0], [3.0, 4.0, 3.0], [7.0, 2.0, 7.0]], 
[[4.0, 4.0, 1.0], [6.0, 8.0, 7.0], [5.0, 5.0, 8.0]], [[7.0, 3.0, 5.0], [8.0, 8.0, 8.0], [7.0, 5.0, 2.0]]], [[[8.0, 3.0, 1.0], 
[7.0, 5.0, 1.0], [2.0, 7.0, 5.0]], [[4.0, 3.0, 6.0], [6.0, 8.0, 6.0], [3.0, 7.0, 1.0]], [[4.0, 5.0, 5.0], [2.0, 6.0, 1.0], 
[3.0, 4.0, 3.0]]], [[[2.0, 4.0, 2.0], [6.0, 8.0, 3.0], [7.0, 6.0, 6.0]], [[5.0, 6.0, 7.0], [3.0, 1.0, 6.0], [6.0, 1.0, 1.0]], 
[[1.0, 6.0, 8.0], [3.0, 3.0, 5.0], [4.0, 4.0, 8.0]]], [[[5.0, 2.0, 7.0], [5.0, 8.0, 6.0], [5.0, 1.0, 6.0]], [[1.0, 7.0, 5.0], 
[2.0, 1.0, 8.0], [7.0, 7.0, 7.0]], [[5.0, 4.0, 8.0], [5.0, 6.0, 2.0], [5.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_LLCN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 8.0], [6.0, 8.0, 6.0], [8.0, 6.0, 3.0]], [[1.0, 1.0, 3.0], [5.0, 2.0, 4.0], [8.0, 4.0, 
7.0]], [[7.0, 8.0, 8.0], [6.0, 5.0, 6.0], [7.0, 7.0, 2.0]]], [[[6.0, 1.0, 8.0], [1.0, 3.0, 8.0], [7.0, 2.0, 7.0]], [[3.0, 5.0, 
4.0], [3.0, 6.0, 6.0], [4.0, 2.0, 7.0]], [[4.0, 6.0, 5.0], [7.0, 7.0, 8.0], [2.0, 4.0, 3.0]]], [[[4.0, 7.0, 8.0], [3.0, 6.0, 
6.0], [5.0, 3.0, 5.0]], [[8.0, 5.0, 2.0], [6.0, 1.0, 3.0], [8.0, 6.0, 2.0]], [[5.0, 4.0, 2.0], [3.0, 6.0, 8.0], [2.0, 4.0, 
2.0]]], [[[6.0, 7.0, 7.0], [3.0, 2.0, 4.0], [7.0, 5.0, 4.0]], [[2.0, 6.0, 1.0], [4.0, 4.0, 8.0], [7.0, 7.0, 7.0]], [[1.0, 5.0, 
1.0], [5.0, 1.0, 3.0], [3.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 5.0, 1.0], [1.0, 1.0, 5.0], [8.0, 2.0, 6.0]], [[4.0, 2.0, 8.0], [6.0, 4.0, 
4.0], [5.0, 1.0, 1.0]], [[1.0, 7.0, 5.0], [3.0, 2.0, 8.0], [3.0, 3.0, 8.0]]], [[[3.0, 3.0, 1.0], [3.0, 4.0, 1.0], [6.0, 6.0, 
5.0]], [[5.0, 6.0, 3.0], [8.0, 5.0, 8.0], [8.0, 3.0, 4.0]], [[4.0, 3.0, 2.0], [4.0, 4.0, 6.0], [5.0, 1.0, 1.0]]], [[[5.0, 5.0, 
5.0], [4.0, 6.0, 8.0], [7.0, 5.0, 6.0]], [[2.0, 5.0, 5.0], [8.0, 7.0, 8.0], [3.0, 2.0, 3.0]], [[3.0, 5.0, 6.0], [6.0, 1.0, 
4.0], [2.0, 1.0, 8.0]]], [[[6.0, 3.0, 7.0], [4.0, 2.0, 4.0], [3.0, 7.0, 4.0]], [[8.0, 2.0, 3.0], [4.0, 6.0, 4.0], [6.0, 8.0, 
7.0]], [[2.0, 8.0, 7.0], [5.0, 5.0, 5.0], [8.0, 5.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[8.0, 5.0, 5.0], [1.0, 6.0, 8.0], [5.0, 3.0, 4.0]]], [[[2.0, 2.0, 7.0], [1.0, 6.0, 4.0], [6.0, 
4.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 2.0, 4.0], [7.0, 4.0, 1.0], [4.0, 1.0, 2.0]]], [[[7.0, 6.0, 7.0], [7.0, 4.0, 
4.0], [7.0, 2.0, 1.0]]]])
      arg[2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 8.0], [6.0, 8.0, 6.0], [8.0, 6.0, 3.0]], [[1.0, 1.0, 3.0], [5.0, 2.0, 4.0], [8.0, 
4.0, 7.0]], [[7.0, 8.0, 8.0], [6.0, 5.0, 6.0], [7.0, 7.0, 2.0]]], [[[6.0, 1.0, 8.0], [1.0, 3.0, 8.0], [7.0, 2.0, 7.0]], [[3.0, 
5.0, 4.0], [3.0, 6.0, 6.0], [4.0, 2.0, 7.0]], [[4.0, 6.0, 5.0], [7.0, 7.0, 8.0], [2.0, 4.0, 3.0]]], [[[4.0, 7.0, 8.0], [3.0, 
6.0, 6.0], [5.0, 3.0, 5.0]], [[8.0, 5.0, 2.0], [6.0, 1.0, 3.0], [8.0, 6.0, 2.0]], [[8.0, 5.0, 5.0], [1.0, 6.0, 8.0], [5.0, 3.0, 
4.0]]], [[[6.0, 7.0, 7.0], [3.0, 2.0, 4.0], [7.0, 5.0, 4.0]], [[2.0, 6.0, 1.0], [4.0, 4.0, 8.0], [7.0, 7.0, 7.0]], [[2.0, 2.0, 
7.0], [1.0, 6.0, 4.0], [6.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 1.0], [1.0, 1.0, 5.0], [8.0, 2.0, 6.0]], 
[[4.0, 2.0, 8.0], [6.0, 4.0, 4.0], [5.0, 1.0, 1.0]], [[1.0, 7.0, 5.0], [3.0, 2.0, 8.0], [3.0, 3.0, 8.0]]], [[[3.0, 3.0, 1.0], 
[3.0, 4.0, 1.0], [6.0, 6.0, 5.0]], [[5.0, 6.0, 3.0], [8.0, 5.0, 8.0], [8.0, 3.0, 4.0]], [[4.0, 3.0, 2.0], [4.0, 4.0, 6.0], 
[5.0, 1.0, 1.0]]], [[[5.0, 5.0, 5.0], [4.0, 6.0, 8.0], [7.0, 5.0, 6.0]], [[2.0, 5.0, 5.0], [8.0, 7.0, 8.0], [3.0, 2.0, 3.0]], 
[[8.0, 2.0, 4.0], [7.0, 4.0, 1.0], [4.0, 1.0, 2.0]]], [[[6.0, 3.0, 7.0], [4.0, 2.0, 4.0], [3.0, 7.0, 4.0]], [[8.0, 2.0, 3.0], 
[4.0, 6.0, 4.0], [6.0, 8.0, 7.0]], [[7.0, 6.0, 7.0], [7.0, 4.0, 4.0], [7.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_RPLN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 7.0], [1.0, 8.0, 1.0], [4.0, 6.0, 3.0]], [[7.0, 5.0, 7.0], [2.0, 4.0, 3.0], [2.0, 6.0, 
5.0]], [[4.0, 1.0, 3.0], [3.0, 2.0, 1.0], [5.0, 3.0, 3.0]]], [[[2.0, 8.0, 8.0], [7.0, 6.0, 6.0], [5.0, 2.0, 8.0]], [[3.0, 4.0, 
8.0], [5.0, 2.0, 8.0], [6.0, 3.0, 3.0]], [[1.0, 4.0, 5.0], [2.0, 8.0, 6.0], [1.0, 7.0, 4.0]]], [[[4.0, 7.0, 6.0], [2.0, 1.0, 
1.0], [6.0, 3.0, 5.0]], [[5.0, 2.0, 1.0], [6.0, 6.0, 5.0], [8.0, 6.0, 4.0]], [[5.0, 4.0, 3.0], [8.0, 1.0, 3.0], [8.0, 8.0, 
4.0]]], [[[7.0, 2.0, 7.0], [4.0, 7.0, 2.0], [8.0, 5.0, 1.0]], [[7.0, 6.0, 3.0], [8.0, 6.0, 3.0], [5.0, 4.0, 4.0]], [[6.0, 1.0, 
4.0], [2.0, 8.0, 2.0], [2.0, 4.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 8.0], [2.0, 1.0, 2.0], [1.0, 4.0, 3.0]], [[7.0, 1.0, 2.0], [4.0, 5.0, 
1.0], [5.0, 1.0, 4.0]], [[8.0, 6.0, 2.0], [6.0, 7.0, 7.0], [3.0, 7.0, 1.0]]], [[[3.0, 6.0, 6.0], [4.0, 1.0, 1.0], [5.0, 2.0, 
3.0]], [[6.0, 4.0, 8.0], [5.0, 4.0, 2.0], [5.0, 6.0, 2.0]], [[3.0, 1.0, 2.0], [3.0, 8.0, 6.0], [3.0, 1.0, 4.0]]], [[[1.0, 8.0, 
6.0], [2.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[7.0, 4.0, 5.0], [3.0, 1.0, 3.0], [1.0, 4.0, 6.0]], [[8.0, 7.0, 7.0], [1.0, 6.0, 
2.0], [5.0, 7.0, 4.0]]], [[[2.0, 1.0, 4.0], [8.0, 5.0, 7.0], [2.0, 8.0, 4.0]], [[7.0, 7.0, 4.0], [1.0, 7.0, 3.0], [4.0, 7.0, 
8.0]], [[8.0, 3.0, 1.0], [1.0, 7.0, 8.0], [4.0, 5.0, 1.0]]]]))
      val=numpy.array([[[2.0, 8.0, 3.0]], [[7.0, 8.0, 2.0]]])
      arg[1:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 7.0], [1.0, 8.0, 1.0], [4.0, 6.0, 3.0]], [[7.0, 5.0, 7.0], [2.0, 4.0, 3.0], [2.0, 
6.0, 5.0]], [[4.0, 1.0, 3.0], [3.0, 2.0, 1.0], [5.0, 3.0, 3.0]]], [[[2.0, 8.0, 8.0], [7.0, 6.0, 6.0], [5.0, 2.0, 8.0]], [[3.0, 
4.0, 8.0], [5.0, 2.0, 8.0], [2.0, 8.0, 3.0]], [[1.0, 4.0, 5.0], [2.0, 8.0, 6.0], [1.0, 7.0, 4.0]]], [[[4.0, 7.0, 6.0], [2.0, 
1.0, 1.0], [6.0, 3.0, 5.0]], [[5.0, 2.0, 1.0], [6.0, 6.0, 5.0], [7.0, 8.0, 2.0]], [[5.0, 4.0, 3.0], [8.0, 1.0, 3.0], [8.0, 8.0, 
4.0]]], [[[7.0, 2.0, 7.0], [4.0, 7.0, 2.0], [8.0, 5.0, 1.0]], [[7.0, 6.0, 3.0], [8.0, 6.0, 3.0], [5.0, 4.0, 4.0]], [[6.0, 1.0, 
4.0], [2.0, 8.0, 2.0], [2.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 8.0], [2.0, 1.0, 2.0], [1.0, 4.0, 3.0]], 
[[7.0, 1.0, 2.0], [4.0, 5.0, 1.0], [5.0, 1.0, 4.0]], [[8.0, 6.0, 2.0], [6.0, 7.0, 7.0], [3.0, 7.0, 1.0]]], [[[3.0, 6.0, 6.0], 
[4.0, 1.0, 1.0], [5.0, 2.0, 3.0]], [[6.0, 4.0, 8.0], [5.0, 4.0, 2.0], [2.0, 8.0, 3.0]], [[3.0, 1.0, 2.0], [3.0, 8.0, 6.0], 
[3.0, 1.0, 4.0]]], [[[1.0, 8.0, 6.0], [2.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[7.0, 4.0, 5.0], [3.0, 1.0, 3.0], [7.0, 8.0, 2.0]], 
[[8.0, 7.0, 7.0], [1.0, 6.0, 2.0], [5.0, 7.0, 4.0]]], [[[2.0, 1.0, 4.0], [8.0, 5.0, 7.0], [2.0, 8.0, 4.0]], [[7.0, 7.0, 4.0], 
[1.0, 7.0, 3.0], [4.0, 7.0, 8.0]], [[8.0, 3.0, 1.0], [1.0, 7.0, 8.0], [4.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_RPLN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 1.0], [4.0, 3.0, 5.0], [2.0, 4.0, 1.0]], [[4.0, 1.0, 3.0], [5.0, 2.0, 7.0], [4.0, 3.0, 
4.0]], [[3.0, 6.0, 5.0], [2.0, 8.0, 1.0], [4.0, 5.0, 4.0]]], [[[5.0, 5.0, 1.0], [1.0, 5.0, 6.0], [1.0, 4.0, 8.0]], [[7.0, 4.0, 
1.0], [1.0, 3.0, 1.0], [8.0, 7.0, 3.0]], [[7.0, 8.0, 1.0], [4.0, 5.0, 4.0], [2.0, 8.0, 3.0]]], [[[4.0, 3.0, 8.0], [4.0, 5.0, 
4.0], [2.0, 6.0, 6.0]], [[7.0, 6.0, 7.0], [7.0, 5.0, 5.0], [8.0, 2.0, 3.0]], [[4.0, 6.0, 7.0], [2.0, 5.0, 4.0], [6.0, 1.0, 
1.0]]], [[[8.0, 3.0, 4.0], [3.0, 8.0, 5.0], [1.0, 4.0, 7.0]], [[6.0, 8.0, 3.0], [8.0, 2.0, 7.0], [8.0, 4.0, 4.0]], [[8.0, 4.0, 
7.0], [1.0, 6.0, 8.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 1.0, 5.0], [2.0, 7.0, 8.0], [6.0, 1.0, 8.0]], [[2.0, 8.0, 2.0], [3.0, 5.0, 
2.0], [6.0, 5.0, 1.0]], [[1.0, 5.0, 6.0], [1.0, 4.0, 1.0], [8.0, 7.0, 2.0]]], [[[3.0, 6.0, 6.0], [4.0, 1.0, 7.0], [4.0, 1.0, 
7.0]], [[3.0, 7.0, 8.0], [4.0, 7.0, 1.0], [5.0, 2.0, 2.0]], [[1.0, 4.0, 7.0], [3.0, 6.0, 3.0], [5.0, 7.0, 8.0]]], [[[1.0, 7.0, 
1.0], [8.0, 4.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 5.0, 1.0], [2.0, 7.0, 7.0], [1.0, 3.0, 2.0]], [[6.0, 5.0, 2.0], [1.0, 1.0, 
1.0], [6.0, 5.0, 2.0]]], [[[3.0, 3.0, 6.0], [1.0, 3.0, 7.0], [1.0, 4.0, 8.0]], [[8.0, 8.0, 5.0], [2.0, 5.0, 1.0], [3.0, 7.0, 
6.0]], [[5.0, 6.0, 1.0], [6.0, 5.0, 8.0], [4.0, 4.0, 4.0]]]]))
      val=Data(numpy.array([[[2.0, 1.0, 5.0]], [[7.0, 6.0, 1.0]]]),self.functionspace)
      arg[1:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 1.0], [4.0, 3.0, 5.0], [2.0, 4.0, 1.0]], [[4.0, 1.0, 3.0], [5.0, 2.0, 7.0], [4.0, 
3.0, 4.0]], [[3.0, 6.0, 5.0], [2.0, 8.0, 1.0], [4.0, 5.0, 4.0]]], [[[5.0, 5.0, 1.0], [1.0, 5.0, 6.0], [1.0, 4.0, 8.0]], [[7.0, 
4.0, 1.0], [1.0, 3.0, 1.0], [2.0, 1.0, 5.0]], [[7.0, 8.0, 1.0], [4.0, 5.0, 4.0], [2.0, 8.0, 3.0]]], [[[4.0, 3.0, 8.0], [4.0, 
5.0, 4.0], [2.0, 6.0, 6.0]], [[7.0, 6.0, 7.0], [7.0, 5.0, 5.0], [7.0, 6.0, 1.0]], [[4.0, 6.0, 7.0], [2.0, 5.0, 4.0], [6.0, 1.0, 
1.0]]], [[[8.0, 3.0, 4.0], [3.0, 8.0, 5.0], [1.0, 4.0, 7.0]], [[6.0, 8.0, 3.0], [8.0, 2.0, 7.0], [8.0, 4.0, 4.0]], [[8.0, 4.0, 
7.0], [1.0, 6.0, 8.0], [3.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 5.0], [2.0, 7.0, 8.0], [6.0, 1.0, 8.0]], 
[[2.0, 8.0, 2.0], [3.0, 5.0, 2.0], [6.0, 5.0, 1.0]], [[1.0, 5.0, 6.0], [1.0, 4.0, 1.0], [8.0, 7.0, 2.0]]], [[[3.0, 6.0, 6.0], 
[4.0, 1.0, 7.0], [4.0, 1.0, 7.0]], [[3.0, 7.0, 8.0], [4.0, 7.0, 1.0], [2.0, 1.0, 5.0]], [[1.0, 4.0, 7.0], [3.0, 6.0, 3.0], 
[5.0, 7.0, 8.0]]], [[[1.0, 7.0, 1.0], [8.0, 4.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 5.0, 1.0], [2.0, 7.0, 7.0], [7.0, 6.0, 1.0]], 
[[6.0, 5.0, 2.0], [1.0, 1.0, 1.0], [6.0, 5.0, 2.0]]], [[[3.0, 3.0, 6.0], [1.0, 3.0, 7.0], [1.0, 4.0, 8.0]], [[8.0, 8.0, 5.0], 
[2.0, 5.0, 1.0], [3.0, 7.0, 6.0]], [[5.0, 6.0, 1.0], [6.0, 5.0, 8.0], [4.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_RPLN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 7.0], [4.0, 4.0, 5.0], [4.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 4.0, 4.0], [4.0, 6.0, 
8.0]], [[6.0, 4.0, 4.0], [6.0, 5.0, 2.0], [8.0, 4.0, 7.0]]], [[[7.0, 3.0, 8.0], [2.0, 6.0, 8.0], [8.0, 8.0, 4.0]], [[3.0, 1.0, 
3.0], [5.0, 3.0, 7.0], [8.0, 6.0, 5.0]], [[8.0, 8.0, 2.0], [8.0, 4.0, 4.0], [4.0, 4.0, 5.0]]], [[[8.0, 4.0, 8.0], [3.0, 8.0, 
3.0], [5.0, 3.0, 7.0]], [[5.0, 8.0, 2.0], [3.0, 4.0, 8.0], [4.0, 3.0, 1.0]], [[5.0, 1.0, 8.0], [8.0, 8.0, 6.0], [1.0, 8.0, 
6.0]]], [[[7.0, 3.0, 1.0], [5.0, 1.0, 4.0], [8.0, 3.0, 3.0]], [[8.0, 6.0, 7.0], [8.0, 3.0, 8.0], [3.0, 5.0, 8.0]], [[2.0, 4.0, 
1.0], [6.0, 8.0, 5.0], [7.0, 3.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 3.0], [3.0, 3.0, 5.0], [3.0, 6.0, 7.0]], [[8.0, 6.0, 2.0], [8.0, 7.0, 
4.0], [6.0, 7.0, 2.0]], [[7.0, 2.0, 2.0], [3.0, 5.0, 4.0], [5.0, 6.0, 4.0]]], [[[5.0, 2.0, 4.0], [1.0, 4.0, 7.0], [2.0, 3.0, 
3.0]], [[3.0, 3.0, 5.0], [2.0, 4.0, 7.0], [3.0, 8.0, 6.0]], [[7.0, 6.0, 6.0], [4.0, 2.0, 3.0], [7.0, 2.0, 8.0]]], [[[6.0, 2.0, 
7.0], [3.0, 6.0, 8.0], [5.0, 7.0, 8.0]], [[5.0, 2.0, 2.0], [8.0, 8.0, 2.0], [6.0, 5.0, 6.0]], [[6.0, 5.0, 4.0], [7.0, 7.0, 
5.0], [5.0, 4.0, 1.0]]], [[[2.0, 4.0, 1.0], [3.0, 5.0, 5.0], [5.0, 6.0, 6.0]], [[1.0, 5.0, 6.0], [4.0, 7.0, 6.0], [8.0, 3.0, 
8.0]], [[6.0, 5.0, 7.0], [5.0, 4.0, 3.0], [1.0, 5.0, 8.0]]]]))
      val=Data(numpy.array([[[4.0, 4.0, 2.0]], [[2.0, 3.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 6.0, 4.0]], [[7.0, 1.0, 3.0]]]))
      arg[1:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 7.0], [4.0, 4.0, 5.0], [4.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 4.0, 4.0], [4.0, 
6.0, 8.0]], [[6.0, 4.0, 4.0], [6.0, 5.0, 2.0], [8.0, 4.0, 7.0]]], [[[7.0, 3.0, 8.0], [2.0, 6.0, 8.0], [8.0, 8.0, 4.0]], [[3.0, 
1.0, 3.0], [5.0, 3.0, 7.0], [4.0, 4.0, 2.0]], [[8.0, 8.0, 2.0], [8.0, 4.0, 4.0], [4.0, 4.0, 5.0]]], [[[8.0, 4.0, 8.0], [3.0, 
8.0, 3.0], [5.0, 3.0, 7.0]], [[5.0, 8.0, 2.0], [3.0, 4.0, 8.0], [2.0, 3.0, 6.0]], [[5.0, 1.0, 8.0], [8.0, 8.0, 6.0], [1.0, 8.0, 
6.0]]], [[[7.0, 3.0, 1.0], [5.0, 1.0, 4.0], [8.0, 3.0, 3.0]], [[8.0, 6.0, 7.0], [8.0, 3.0, 8.0], [3.0, 5.0, 8.0]], [[2.0, 4.0, 
1.0], [6.0, 8.0, 5.0], [7.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 3.0], [3.0, 3.0, 5.0], [3.0, 6.0, 7.0]], 
[[8.0, 6.0, 2.0], [8.0, 7.0, 4.0], [6.0, 7.0, 2.0]], [[7.0, 2.0, 2.0], [3.0, 5.0, 4.0], [5.0, 6.0, 4.0]]], [[[5.0, 2.0, 4.0], 
[1.0, 4.0, 7.0], [2.0, 3.0, 3.0]], [[3.0, 3.0, 5.0], [2.0, 4.0, 7.0], [5.0, 6.0, 4.0]], [[7.0, 6.0, 6.0], [4.0, 2.0, 3.0], 
[7.0, 2.0, 8.0]]], [[[6.0, 2.0, 7.0], [3.0, 6.0, 8.0], [5.0, 7.0, 8.0]], [[5.0, 2.0, 2.0], [8.0, 8.0, 2.0], [7.0, 1.0, 3.0]], 
[[6.0, 5.0, 4.0], [7.0, 7.0, 5.0], [5.0, 4.0, 1.0]]], [[[2.0, 4.0, 1.0], [3.0, 5.0, 5.0], [5.0, 6.0, 6.0]], [[1.0, 5.0, 6.0], 
[4.0, 7.0, 6.0], [8.0, 3.0, 8.0]], [[6.0, 5.0, 7.0], [5.0, 4.0, 3.0], [1.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_RPLN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 7.0], [5.0, 3.0, 7.0], [6.0, 8.0, 6.0]], [[4.0, 4.0, 1.0], [3.0, 1.0, 7.0], [4.0, 8.0, 
7.0]], [[5.0, 1.0, 3.0], [2.0, 7.0, 1.0], [8.0, 4.0, 6.0]]], [[[6.0, 7.0, 1.0], [6.0, 2.0, 3.0], [6.0, 4.0, 5.0]], [[2.0, 5.0, 
6.0], [3.0, 5.0, 2.0], [1.0, 1.0, 2.0]], [[8.0, 6.0, 6.0], [3.0, 7.0, 1.0], [8.0, 8.0, 5.0]]], [[[8.0, 7.0, 7.0], [2.0, 6.0, 
2.0], [3.0, 6.0, 1.0]], [[2.0, 2.0, 3.0], [3.0, 3.0, 1.0], [7.0, 1.0, 4.0]], [[2.0, 2.0, 3.0], [4.0, 4.0, 3.0], [1.0, 6.0, 
3.0]]], [[[6.0, 2.0, 5.0], [8.0, 1.0, 2.0], [5.0, 8.0, 6.0]], [[6.0, 8.0, 5.0], [7.0, 2.0, 1.0], [3.0, 7.0, 1.0]], [[2.0, 8.0, 
2.0], [2.0, 3.0, 4.0], [5.0, 4.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 5.0], [3.0, 4.0, 1.0], [1.0, 1.0, 3.0]], [[7.0, 6.0, 3.0], [6.0, 7.0, 
8.0], [5.0, 1.0, 2.0]], [[2.0, 7.0, 4.0], [1.0, 5.0, 8.0], [1.0, 3.0, 3.0]]], [[[3.0, 8.0, 8.0], [3.0, 3.0, 2.0], [6.0, 2.0, 
3.0]], [[3.0, 2.0, 5.0], [4.0, 1.0, 7.0], [8.0, 6.0, 6.0]], [[4.0, 3.0, 8.0], [2.0, 3.0, 4.0], [7.0, 8.0, 3.0]]], [[[2.0, 3.0, 
3.0], [7.0, 8.0, 6.0], [1.0, 3.0, 3.0]], [[2.0, 8.0, 4.0], [4.0, 7.0, 7.0], [2.0, 7.0, 6.0]], [[8.0, 5.0, 6.0], [8.0, 6.0, 
2.0], [6.0, 3.0, 8.0]]], [[[5.0, 2.0, 3.0], [4.0, 6.0, 5.0], [2.0, 3.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 4.0, 4.0], [4.0, 6.0, 
5.0]], [[7.0, 7.0, 8.0], [2.0, 5.0, 4.0], [1.0, 2.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[8.0, 2.0, 1.0]], [[5.0, 6.0, 6.0]]])+(1.-msk_val)*numpy.array([[[1.0, 6.0, 5.0]], [[2.0, 5.0, 
4.0]]])
      arg[1:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 7.0], [5.0, 3.0, 7.0], [6.0, 8.0, 6.0]], [[4.0, 4.0, 1.0], [3.0, 1.0, 7.0], [4.0, 
8.0, 7.0]], [[5.0, 1.0, 3.0], [2.0, 7.0, 1.0], [8.0, 4.0, 6.0]]], [[[6.0, 7.0, 1.0], [6.0, 2.0, 3.0], [6.0, 4.0, 5.0]], [[2.0, 
5.0, 6.0], [3.0, 5.0, 2.0], [8.0, 2.0, 1.0]], [[8.0, 6.0, 6.0], [3.0, 7.0, 1.0], [8.0, 8.0, 5.0]]], [[[8.0, 7.0, 7.0], [2.0, 
6.0, 2.0], [3.0, 6.0, 1.0]], [[2.0, 2.0, 3.0], [3.0, 3.0, 1.0], [5.0, 6.0, 6.0]], [[2.0, 2.0, 3.0], [4.0, 4.0, 3.0], [1.0, 6.0, 
3.0]]], [[[6.0, 2.0, 5.0], [8.0, 1.0, 2.0], [5.0, 8.0, 6.0]], [[6.0, 8.0, 5.0], [7.0, 2.0, 1.0], [3.0, 7.0, 1.0]], [[2.0, 8.0, 
2.0], [2.0, 3.0, 4.0], [5.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 5.0], [3.0, 4.0, 1.0], [1.0, 1.0, 3.0]], 
[[7.0, 6.0, 3.0], [6.0, 7.0, 8.0], [5.0, 1.0, 2.0]], [[2.0, 7.0, 4.0], [1.0, 5.0, 8.0], [1.0, 3.0, 3.0]]], [[[3.0, 8.0, 8.0], 
[3.0, 3.0, 2.0], [6.0, 2.0, 3.0]], [[3.0, 2.0, 5.0], [4.0, 1.0, 7.0], [1.0, 6.0, 5.0]], [[4.0, 3.0, 8.0], [2.0, 3.0, 4.0], 
[7.0, 8.0, 3.0]]], [[[2.0, 3.0, 3.0], [7.0, 8.0, 6.0], [1.0, 3.0, 3.0]], [[2.0, 8.0, 4.0], [4.0, 7.0, 7.0], [2.0, 5.0, 4.0]], 
[[8.0, 5.0, 6.0], [8.0, 6.0, 2.0], [6.0, 3.0, 8.0]]], [[[5.0, 2.0, 3.0], [4.0, 6.0, 5.0], [2.0, 3.0, 6.0]], [[7.0, 8.0, 3.0], 
[8.0, 4.0, 4.0], [4.0, 6.0, 5.0]], [[7.0, 7.0, 8.0], [2.0, 5.0, 4.0], [1.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_PCUN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 3.0], [8.0, 7.0, 4.0], [4.0, 5.0, 1.0]], [[3.0, 7.0, 7.0], [8.0, 8.0, 4.0], [3.0, 1.0, 
4.0]], [[4.0, 6.0, 2.0], [6.0, 3.0, 4.0], [4.0, 2.0, 4.0]]], [[[7.0, 1.0, 4.0], [5.0, 8.0, 6.0], [4.0, 6.0, 3.0]], [[8.0, 1.0, 
3.0], [4.0, 8.0, 2.0], [6.0, 5.0, 5.0]], [[6.0, 6.0, 7.0], [2.0, 4.0, 2.0], [4.0, 3.0, 5.0]]], [[[3.0, 1.0, 5.0], [7.0, 1.0, 
6.0], [8.0, 2.0, 6.0]], [[6.0, 4.0, 5.0], [3.0, 3.0, 5.0], [6.0, 5.0, 4.0]], [[6.0, 8.0, 5.0], [7.0, 2.0, 6.0], [3.0, 3.0, 
6.0]]], [[[2.0, 2.0, 7.0], [1.0, 2.0, 2.0], [6.0, 7.0, 4.0]], [[4.0, 5.0, 3.0], [7.0, 7.0, 3.0], [7.0, 3.0, 7.0]], [[8.0, 5.0, 
5.0], [2.0, 2.0, 8.0], [1.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 6.0], [4.0, 2.0, 4.0], [3.0, 1.0, 3.0]], [[2.0, 6.0, 8.0], [2.0, 8.0, 
5.0], [1.0, 1.0, 7.0]], [[4.0, 5.0, 2.0], [4.0, 7.0, 4.0], [2.0, 4.0, 1.0]]], [[[1.0, 6.0, 7.0], [8.0, 4.0, 1.0], [2.0, 3.0, 
4.0]], [[4.0, 8.0, 7.0], [7.0, 4.0, 6.0], [2.0, 7.0, 6.0]], [[2.0, 2.0, 4.0], [4.0, 1.0, 3.0], [2.0, 5.0, 1.0]]], [[[2.0, 4.0, 
5.0], [1.0, 2.0, 3.0], [7.0, 1.0, 2.0]], [[5.0, 1.0, 4.0], [4.0, 3.0, 1.0], [2.0, 2.0, 6.0]], [[3.0, 8.0, 5.0], [2.0, 7.0, 
6.0], [4.0, 2.0, 6.0]]], [[[8.0, 2.0, 1.0], [5.0, 7.0, 7.0], [1.0, 6.0, 4.0]], [[7.0, 2.0, 3.0], [8.0, 3.0, 8.0], [2.0, 1.0, 
6.0]], [[1.0, 8.0, 1.0], [3.0, 5.0, 7.0], [6.0, 1.0, 3.0]]]]))
      val=numpy.array([[[5.0, 4.0, 7.0], [7.0, 3.0, 7.0]], [[3.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[3.0, 2.0, 6.0], [7.0, 2.0, 
3.0]]])
      arg[2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 3.0], [8.0, 7.0, 4.0], [4.0, 5.0, 1.0]], [[3.0, 7.0, 7.0], [8.0, 8.0, 4.0], [3.0, 
1.0, 4.0]], [[4.0, 6.0, 2.0], [6.0, 3.0, 4.0], [4.0, 2.0, 4.0]]], [[[7.0, 1.0, 4.0], [5.0, 8.0, 6.0], [4.0, 6.0, 3.0]], [[8.0, 
1.0, 3.0], [4.0, 8.0, 2.0], [6.0, 5.0, 5.0]], [[6.0, 6.0, 7.0], [2.0, 4.0, 2.0], [4.0, 3.0, 5.0]]], [[[5.0, 4.0, 7.0], [7.0, 
3.0, 7.0], [8.0, 2.0, 6.0]], [[3.0, 7.0, 6.0], [4.0, 1.0, 1.0], [6.0, 5.0, 4.0]], [[3.0, 2.0, 6.0], [7.0, 2.0, 3.0], [3.0, 3.0, 
6.0]]], [[[2.0, 2.0, 7.0], [1.0, 2.0, 2.0], [6.0, 7.0, 4.0]], [[4.0, 5.0, 3.0], [7.0, 7.0, 3.0], [7.0, 3.0, 7.0]], [[8.0, 5.0, 
5.0], [2.0, 2.0, 8.0], [1.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 6.0], [4.0, 2.0, 4.0], [3.0, 1.0, 3.0]], 
[[2.0, 6.0, 8.0], [2.0, 8.0, 5.0], [1.0, 1.0, 7.0]], [[4.0, 5.0, 2.0], [4.0, 7.0, 4.0], [2.0, 4.0, 1.0]]], [[[1.0, 6.0, 7.0], 
[8.0, 4.0, 1.0], [2.0, 3.0, 4.0]], [[4.0, 8.0, 7.0], [7.0, 4.0, 6.0], [2.0, 7.0, 6.0]], [[2.0, 2.0, 4.0], [4.0, 1.0, 3.0], 
[2.0, 5.0, 1.0]]], [[[5.0, 4.0, 7.0], [7.0, 3.0, 7.0], [7.0, 1.0, 2.0]], [[3.0, 7.0, 6.0], [4.0, 1.0, 1.0], [2.0, 2.0, 6.0]], 
[[3.0, 2.0, 6.0], [7.0, 2.0, 3.0], [4.0, 2.0, 6.0]]], [[[8.0, 2.0, 1.0], [5.0, 7.0, 7.0], [1.0, 6.0, 4.0]], [[7.0, 2.0, 3.0], 
[8.0, 3.0, 8.0], [2.0, 1.0, 6.0]], [[1.0, 8.0, 1.0], [3.0, 5.0, 7.0], [6.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_PCUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 4.0], [4.0, 8.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 1.0, 7.0], [8.0, 7.0, 8.0], [7.0, 4.0, 
1.0]], [[6.0, 2.0, 5.0], [7.0, 8.0, 1.0], [4.0, 8.0, 6.0]]], [[[1.0, 3.0, 8.0], [6.0, 3.0, 5.0], [5.0, 7.0, 8.0]], [[4.0, 5.0, 
1.0], [5.0, 2.0, 2.0], [1.0, 4.0, 5.0]], [[4.0, 6.0, 6.0], [8.0, 7.0, 2.0], [7.0, 7.0, 2.0]]], [[[6.0, 2.0, 5.0], [7.0, 3.0, 
5.0], [7.0, 4.0, 1.0]], [[2.0, 3.0, 8.0], [5.0, 2.0, 1.0], [1.0, 2.0, 2.0]], [[6.0, 2.0, 2.0], [8.0, 8.0, 3.0], [4.0, 8.0, 
8.0]]], [[[3.0, 2.0, 1.0], [1.0, 7.0, 3.0], [4.0, 3.0, 7.0]], [[1.0, 1.0, 6.0], [3.0, 3.0, 3.0], [8.0, 7.0, 6.0]], [[1.0, 6.0, 
1.0], [5.0, 4.0, 7.0], [2.0, 8.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 8.0], [4.0, 6.0, 8.0], [1.0, 5.0, 8.0]], [[8.0, 4.0, 1.0], [3.0, 7.0, 
3.0], [4.0, 6.0, 1.0]], [[3.0, 6.0, 4.0], [6.0, 3.0, 2.0], [1.0, 6.0, 6.0]]], [[[4.0, 2.0, 4.0], [4.0, 2.0, 3.0], [7.0, 6.0, 
2.0]], [[5.0, 6.0, 4.0], [5.0, 2.0, 6.0], [3.0, 1.0, 3.0]], [[2.0, 8.0, 7.0], [4.0, 7.0, 1.0], [3.0, 6.0, 1.0]]], [[[6.0, 3.0, 
8.0], [4.0, 8.0, 2.0], [5.0, 8.0, 2.0]], [[4.0, 5.0, 5.0], [5.0, 7.0, 6.0], [4.0, 1.0, 7.0]], [[1.0, 7.0, 1.0], [4.0, 4.0, 
6.0], [6.0, 4.0, 4.0]]], [[[7.0, 8.0, 1.0], [8.0, 6.0, 2.0], [1.0, 4.0, 2.0]], [[8.0, 4.0, 5.0], [8.0, 3.0, 1.0], [3.0, 4.0, 
3.0]], [[5.0, 6.0, 8.0], [4.0, 1.0, 4.0], [8.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[4.0, 5.0, 4.0], [7.0, 1.0, 3.0]], [[6.0, 5.0, 1.0], [3.0, 6.0, 1.0]], [[2.0, 5.0, 6.0], [3.0, 
6.0, 8.0]]]),self.functionspace)
      arg[2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 4.0], [4.0, 8.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 1.0, 7.0], [8.0, 7.0, 8.0], [7.0, 
4.0, 1.0]], [[6.0, 2.0, 5.0], [7.0, 8.0, 1.0], [4.0, 8.0, 6.0]]], [[[1.0, 3.0, 8.0], [6.0, 3.0, 5.0], [5.0, 7.0, 8.0]], [[4.0, 
5.0, 1.0], [5.0, 2.0, 2.0], [1.0, 4.0, 5.0]], [[4.0, 6.0, 6.0], [8.0, 7.0, 2.0], [7.0, 7.0, 2.0]]], [[[4.0, 5.0, 4.0], [7.0, 
1.0, 3.0], [7.0, 4.0, 1.0]], [[6.0, 5.0, 1.0], [3.0, 6.0, 1.0], [1.0, 2.0, 2.0]], [[2.0, 5.0, 6.0], [3.0, 6.0, 8.0], [4.0, 8.0, 
8.0]]], [[[3.0, 2.0, 1.0], [1.0, 7.0, 3.0], [4.0, 3.0, 7.0]], [[1.0, 1.0, 6.0], [3.0, 3.0, 3.0], [8.0, 7.0, 6.0]], [[1.0, 6.0, 
1.0], [5.0, 4.0, 7.0], [2.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 8.0], [4.0, 6.0, 8.0], [1.0, 5.0, 8.0]], 
[[8.0, 4.0, 1.0], [3.0, 7.0, 3.0], [4.0, 6.0, 1.0]], [[3.0, 6.0, 4.0], [6.0, 3.0, 2.0], [1.0, 6.0, 6.0]]], [[[4.0, 2.0, 4.0], 
[4.0, 2.0, 3.0], [7.0, 6.0, 2.0]], [[5.0, 6.0, 4.0], [5.0, 2.0, 6.0], [3.0, 1.0, 3.0]], [[2.0, 8.0, 7.0], [4.0, 7.0, 1.0], 
[3.0, 6.0, 1.0]]], [[[4.0, 5.0, 4.0], [7.0, 1.0, 3.0], [5.0, 8.0, 2.0]], [[6.0, 5.0, 1.0], [3.0, 6.0, 1.0], [4.0, 1.0, 7.0]], 
[[2.0, 5.0, 6.0], [3.0, 6.0, 8.0], [6.0, 4.0, 4.0]]], [[[7.0, 8.0, 1.0], [8.0, 6.0, 2.0], [1.0, 4.0, 2.0]], [[8.0, 4.0, 5.0], 
[8.0, 3.0, 1.0], [3.0, 4.0, 3.0]], [[5.0, 6.0, 8.0], [4.0, 1.0, 4.0], [8.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_PCUN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 2.0], [3.0, 5.0, 2.0], [8.0, 2.0, 2.0]], [[7.0, 5.0, 8.0], [8.0, 7.0, 1.0], [7.0, 4.0, 
2.0]], [[1.0, 5.0, 4.0], [4.0, 4.0, 8.0], [7.0, 4.0, 5.0]]], [[[1.0, 1.0, 1.0], [8.0, 1.0, 5.0], [5.0, 2.0, 4.0]], [[8.0, 5.0, 
3.0], [1.0, 7.0, 2.0], [7.0, 8.0, 2.0]], [[2.0, 4.0, 8.0], [7.0, 3.0, 1.0], [3.0, 5.0, 3.0]]], [[[3.0, 3.0, 1.0], [5.0, 4.0, 
3.0], [1.0, 4.0, 7.0]], [[8.0, 4.0, 1.0], [1.0, 1.0, 6.0], [4.0, 4.0, 7.0]], [[5.0, 6.0, 2.0], [8.0, 5.0, 4.0], [4.0, 8.0, 
5.0]]], [[[1.0, 8.0, 7.0], [3.0, 6.0, 2.0], [3.0, 3.0, 4.0]], [[2.0, 7.0, 8.0], [5.0, 7.0, 1.0], [8.0, 1.0, 8.0]], [[6.0, 8.0, 
7.0], [3.0, 8.0, 4.0], [7.0, 7.0, 8.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 7.0], [5.0, 7.0, 6.0], [4.0, 4.0, 4.0]], [[1.0, 7.0, 2.0], [1.0, 8.0, 
4.0], [3.0, 8.0, 8.0]], [[3.0, 8.0, 1.0], [4.0, 8.0, 4.0], [5.0, 1.0, 1.0]]], [[[8.0, 1.0, 7.0], [6.0, 2.0, 8.0], [8.0, 6.0, 
7.0]], [[5.0, 3.0, 3.0], [6.0, 7.0, 3.0], [4.0, 1.0, 7.0]], [[6.0, 4.0, 5.0], [3.0, 1.0, 1.0], [7.0, 4.0, 5.0]]], [[[4.0, 1.0, 
2.0], [3.0, 8.0, 7.0], [7.0, 7.0, 6.0]], [[7.0, 7.0, 3.0], [5.0, 6.0, 7.0], [3.0, 6.0, 6.0]], [[2.0, 5.0, 1.0], [6.0, 3.0, 
2.0], [7.0, 2.0, 7.0]]], [[[2.0, 1.0, 2.0], [6.0, 5.0, 8.0], [3.0, 3.0, 4.0]], [[3.0, 8.0, 5.0], [2.0, 6.0, 8.0], [8.0, 3.0, 
3.0]], [[5.0, 4.0, 2.0], [7.0, 5.0, 8.0], [8.0, 3.0, 5.0]]]]))
      val=Data(numpy.array([[[8.0, 5.0, 8.0], [8.0, 8.0, 3.0]], [[1.0, 2.0, 4.0], [8.0, 1.0, 5.0]], [[2.0, 7.0, 7.0], [7.0, 
8.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 2.0, 1.0], [1.0, 1.0, 7.0]], [[8.0, 2.0, 5.0], [3.0, 7.0, 1.0]], [[8.0, 3.0, 
3.0], [3.0, 3.0, 4.0]]]))
      arg[2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 2.0], [3.0, 5.0, 2.0], [8.0, 2.0, 2.0]], [[7.0, 5.0, 8.0], [8.0, 7.0, 1.0], [7.0, 
4.0, 2.0]], [[1.0, 5.0, 4.0], [4.0, 4.0, 8.0], [7.0, 4.0, 5.0]]], [[[1.0, 1.0, 1.0], [8.0, 1.0, 5.0], [5.0, 2.0, 4.0]], [[8.0, 
5.0, 3.0], [1.0, 7.0, 2.0], [7.0, 8.0, 2.0]], [[2.0, 4.0, 8.0], [7.0, 3.0, 1.0], [3.0, 5.0, 3.0]]], [[[8.0, 5.0, 8.0], [8.0, 
8.0, 3.0], [1.0, 4.0, 7.0]], [[1.0, 2.0, 4.0], [8.0, 1.0, 5.0], [4.0, 4.0, 7.0]], [[2.0, 7.0, 7.0], [7.0, 8.0, 4.0], [4.0, 8.0, 
5.0]]], [[[1.0, 8.0, 7.0], [3.0, 6.0, 2.0], [3.0, 3.0, 4.0]], [[2.0, 7.0, 8.0], [5.0, 7.0, 1.0], [8.0, 1.0, 8.0]], [[6.0, 8.0, 
7.0], [3.0, 8.0, 4.0], [7.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 7.0], [5.0, 7.0, 6.0], [4.0, 4.0, 4.0]], 
[[1.0, 7.0, 2.0], [1.0, 8.0, 4.0], [3.0, 8.0, 8.0]], [[3.0, 8.0, 1.0], [4.0, 8.0, 4.0], [5.0, 1.0, 1.0]]], [[[8.0, 1.0, 7.0], 
[6.0, 2.0, 8.0], [8.0, 6.0, 7.0]], [[5.0, 3.0, 3.0], [6.0, 7.0, 3.0], [4.0, 1.0, 7.0]], [[6.0, 4.0, 5.0], [3.0, 1.0, 1.0], 
[7.0, 4.0, 5.0]]], [[[8.0, 2.0, 1.0], [1.0, 1.0, 7.0], [7.0, 7.0, 6.0]], [[8.0, 2.0, 5.0], [3.0, 7.0, 1.0], [3.0, 6.0, 6.0]], 
[[8.0, 3.0, 3.0], [3.0, 3.0, 4.0], [7.0, 2.0, 7.0]]], [[[2.0, 1.0, 2.0], [6.0, 5.0, 8.0], [3.0, 3.0, 4.0]], [[3.0, 8.0, 5.0], 
[2.0, 6.0, 8.0], [8.0, 3.0, 3.0]], [[5.0, 4.0, 2.0], [7.0, 5.0, 8.0], [8.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_PCUN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 4.0], [4.0, 2.0, 2.0], [6.0, 5.0, 1.0]], [[1.0, 2.0, 1.0], [6.0, 5.0, 8.0], [1.0, 7.0, 
4.0]], [[4.0, 8.0, 5.0], [1.0, 4.0, 2.0], [7.0, 3.0, 6.0]]], [[[8.0, 8.0, 3.0], [3.0, 6.0, 1.0], [8.0, 1.0, 8.0]], [[3.0, 3.0, 
6.0], [1.0, 4.0, 1.0], [2.0, 8.0, 6.0]], [[1.0, 1.0, 6.0], [7.0, 5.0, 3.0], [8.0, 5.0, 3.0]]], [[[5.0, 2.0, 1.0], [8.0, 2.0, 
4.0], [3.0, 2.0, 6.0]], [[2.0, 2.0, 6.0], [4.0, 1.0, 3.0], [6.0, 8.0, 7.0]], [[5.0, 1.0, 3.0], [7.0, 1.0, 1.0], [1.0, 8.0, 
3.0]]], [[[3.0, 3.0, 6.0], [3.0, 3.0, 5.0], [2.0, 5.0, 1.0]], [[1.0, 6.0, 5.0], [3.0, 1.0, 6.0], [1.0, 4.0, 5.0]], [[4.0, 4.0, 
7.0], [4.0, 5.0, 8.0], [4.0, 6.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 5.0], [1.0, 5.0, 5.0], [1.0, 7.0, 8.0]], [[6.0, 8.0, 6.0], [5.0, 7.0, 
2.0], [4.0, 2.0, 2.0]], [[6.0, 4.0, 7.0], [7.0, 2.0, 1.0], [1.0, 1.0, 1.0]]], [[[6.0, 2.0, 2.0], [6.0, 6.0, 4.0], [6.0, 6.0, 
3.0]], [[4.0, 7.0, 2.0], [6.0, 7.0, 7.0], [4.0, 6.0, 6.0]], [[8.0, 6.0, 6.0], [6.0, 4.0, 2.0], [4.0, 8.0, 5.0]]], [[[4.0, 8.0, 
2.0], [8.0, 4.0, 8.0], [5.0, 8.0, 1.0]], [[1.0, 2.0, 8.0], [6.0, 7.0, 8.0], [1.0, 4.0, 4.0]], [[6.0, 4.0, 6.0], [2.0, 4.0, 
3.0], [8.0, 8.0, 1.0]]], [[[5.0, 8.0, 2.0], [4.0, 2.0, 5.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 1.0], [7.0, 2.0, 6.0], [8.0, 5.0, 
3.0]], [[7.0, 7.0, 7.0], [2.0, 3.0, 6.0], [3.0, 5.0, 2.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[1.0, 1.0, 7.0], [1.0, 8.0, 4.0]], [[2.0, 8.0, 5.0], [7.0, 4.0, 2.0]], [[5.0, 1.0, 3.0], [4.0, 
3.0, 3.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0, 6.0], [4.0, 3.0, 7.0]], [[1.0, 1.0, 5.0], [7.0, 8.0, 7.0]], [[1.0, 4.0, 
7.0], [3.0, 2.0, 7.0]]])
      arg[2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 4.0], [4.0, 2.0, 2.0], [6.0, 5.0, 1.0]], [[1.0, 2.0, 1.0], [6.0, 5.0, 8.0], [1.0, 
7.0, 4.0]], [[4.0, 8.0, 5.0], [1.0, 4.0, 2.0], [7.0, 3.0, 6.0]]], [[[8.0, 8.0, 3.0], [3.0, 6.0, 1.0], [8.0, 1.0, 8.0]], [[3.0, 
3.0, 6.0], [1.0, 4.0, 1.0], [2.0, 8.0, 6.0]], [[1.0, 1.0, 6.0], [7.0, 5.0, 3.0], [8.0, 5.0, 3.0]]], [[[1.0, 1.0, 7.0], [1.0, 
8.0, 4.0], [3.0, 2.0, 6.0]], [[2.0, 8.0, 5.0], [7.0, 4.0, 2.0], [6.0, 8.0, 7.0]], [[5.0, 1.0, 3.0], [4.0, 3.0, 3.0], [1.0, 8.0, 
3.0]]], [[[3.0, 3.0, 6.0], [3.0, 3.0, 5.0], [2.0, 5.0, 1.0]], [[1.0, 6.0, 5.0], [3.0, 1.0, 6.0], [1.0, 4.0, 5.0]], [[4.0, 4.0, 
7.0], [4.0, 5.0, 8.0], [4.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 5.0], [1.0, 5.0, 5.0], [1.0, 7.0, 8.0]], 
[[6.0, 8.0, 6.0], [5.0, 7.0, 2.0], [4.0, 2.0, 2.0]], [[6.0, 4.0, 7.0], [7.0, 2.0, 1.0], [1.0, 1.0, 1.0]]], [[[6.0, 2.0, 2.0], 
[6.0, 6.0, 4.0], [6.0, 6.0, 3.0]], [[4.0, 7.0, 2.0], [6.0, 7.0, 7.0], [4.0, 6.0, 6.0]], [[8.0, 6.0, 6.0], [6.0, 4.0, 2.0], 
[4.0, 8.0, 5.0]]], [[[8.0, 1.0, 6.0], [4.0, 3.0, 7.0], [5.0, 8.0, 1.0]], [[1.0, 1.0, 5.0], [7.0, 8.0, 7.0], [1.0, 4.0, 4.0]], 
[[1.0, 4.0, 7.0], [3.0, 2.0, 7.0], [8.0, 8.0, 1.0]]], [[[5.0, 8.0, 2.0], [4.0, 2.0, 5.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 1.0], 
[7.0, 2.0, 6.0], [8.0, 5.0, 3.0]], [[7.0, 7.0, 7.0], [2.0, 3.0, 6.0], [3.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCUN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [7.0, 7.0, 7.0], [8.0, 5.0, 5.0]], [[5.0, 4.0, 3.0], [1.0, 7.0, 7.0], [2.0, 6.0, 
7.0]], [[7.0, 3.0, 1.0], [7.0, 6.0, 2.0], [1.0, 1.0, 8.0]]], [[[7.0, 4.0, 2.0], [6.0, 1.0, 2.0], [4.0, 7.0, 4.0]], [[1.0, 1.0, 
2.0], [2.0, 7.0, 3.0], [4.0, 7.0, 3.0]], [[5.0, 2.0, 1.0], [1.0, 8.0, 2.0], [1.0, 1.0, 3.0]]], [[[3.0, 3.0, 7.0], [4.0, 5.0, 
5.0], [5.0, 6.0, 5.0]], [[5.0, 3.0, 2.0], [2.0, 4.0, 8.0], [4.0, 5.0, 8.0]], [[6.0, 7.0, 5.0], [3.0, 2.0, 3.0], [5.0, 2.0, 
7.0]]], [[[1.0, 3.0, 7.0], [2.0, 3.0, 2.0], [3.0, 7.0, 1.0]], [[2.0, 3.0, 5.0], [2.0, 8.0, 7.0], [7.0, 1.0, 2.0]], [[1.0, 2.0, 
8.0], [1.0, 6.0, 2.0], [1.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 1.0], [6.0, 7.0, 5.0], [8.0, 6.0, 6.0]], [[7.0, 5.0, 6.0], [8.0, 1.0, 
7.0], [8.0, 4.0, 6.0]], [[1.0, 4.0, 6.0], [2.0, 4.0, 4.0], [1.0, 6.0, 3.0]]], [[[1.0, 5.0, 7.0], [5.0, 2.0, 1.0], [3.0, 4.0, 
6.0]], [[1.0, 4.0, 3.0], [5.0, 7.0, 5.0], [8.0, 8.0, 8.0]], [[2.0, 6.0, 8.0], [6.0, 8.0, 6.0], [4.0, 4.0, 8.0]]], [[[4.0, 3.0, 
1.0], [6.0, 3.0, 8.0], [6.0, 6.0, 7.0]], [[7.0, 2.0, 7.0], [7.0, 5.0, 2.0], [8.0, 6.0, 7.0]], [[3.0, 4.0, 4.0], [7.0, 7.0, 
2.0], [2.0, 3.0, 1.0]]], [[[8.0, 8.0, 6.0], [3.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[6.0, 7.0, 8.0], [8.0, 5.0, 7.0], [3.0, 6.0, 
7.0]], [[4.0, 6.0, 8.0], [4.0, 1.0, 1.0], [6.0, 7.0, 2.0]]]]))
      val=numpy.array([[[[1.0, 6.0, 5.0], [6.0, 5.0, 1.0]], [[5.0, 1.0, 3.0], [5.0, 4.0, 4.0]], [[8.0, 6.0, 6.0], [7.0, 5.0, 
6.0]]], [[[4.0, 4.0, 8.0], [6.0, 1.0, 8.0]], [[3.0, 3.0, 2.0], [2.0, 5.0, 7.0]], [[1.0, 7.0, 7.0], [2.0, 4.0, 6.0]]], [[[8.0, 
1.0, 3.0], [2.0, 5.0, 3.0]], [[5.0, 5.0, 5.0], [4.0, 1.0, 2.0]], [[4.0, 7.0, 4.0], [7.0, 1.0, 5.0]]], [[[2.0, 4.0, 5.0], [5.0, 
1.0, 5.0]], [[5.0, 2.0, 3.0], [2.0, 8.0, 4.0]], [[5.0, 1.0, 2.0], [3.0, 3.0, 1.0]]]])
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 5.0], [6.0, 5.0, 1.0], [8.0, 5.0, 5.0]], [[5.0, 1.0, 3.0], [5.0, 4.0, 4.0], [2.0, 
6.0, 7.0]], [[8.0, 6.0, 6.0], [7.0, 5.0, 6.0], [1.0, 1.0, 8.0]]], [[[4.0, 4.0, 8.0], [6.0, 1.0, 8.0], [4.0, 7.0, 4.0]], [[3.0, 
3.0, 2.0], [2.0, 5.0, 7.0], [4.0, 7.0, 3.0]], [[1.0, 7.0, 7.0], [2.0, 4.0, 6.0], [1.0, 1.0, 3.0]]], [[[8.0, 1.0, 3.0], [2.0, 
5.0, 3.0], [5.0, 6.0, 5.0]], [[5.0, 5.0, 5.0], [4.0, 1.0, 2.0], [4.0, 5.0, 8.0]], [[4.0, 7.0, 4.0], [7.0, 1.0, 5.0], [5.0, 2.0, 
7.0]]], [[[2.0, 4.0, 5.0], [5.0, 1.0, 5.0], [3.0, 7.0, 1.0]], [[5.0, 2.0, 3.0], [2.0, 8.0, 4.0], [7.0, 1.0, 2.0]], [[5.0, 1.0, 
2.0], [3.0, 3.0, 1.0], [1.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 5.0], [6.0, 5.0, 1.0], [8.0, 6.0, 6.0]], 
[[5.0, 1.0, 3.0], [5.0, 4.0, 4.0], [8.0, 4.0, 6.0]], [[8.0, 6.0, 6.0], [7.0, 5.0, 6.0], [1.0, 6.0, 3.0]]], [[[4.0, 4.0, 8.0], 
[6.0, 1.0, 8.0], [3.0, 4.0, 6.0]], [[3.0, 3.0, 2.0], [2.0, 5.0, 7.0], [8.0, 8.0, 8.0]], [[1.0, 7.0, 7.0], [2.0, 4.0, 6.0], 
[4.0, 4.0, 8.0]]], [[[8.0, 1.0, 3.0], [2.0, 5.0, 3.0], [6.0, 6.0, 7.0]], [[5.0, 5.0, 5.0], [4.0, 1.0, 2.0], [8.0, 6.0, 7.0]], 
[[4.0, 7.0, 4.0], [7.0, 1.0, 5.0], [2.0, 3.0, 1.0]]], [[[2.0, 4.0, 5.0], [5.0, 1.0, 5.0], [2.0, 7.0, 8.0]], [[5.0, 2.0, 3.0], 
[2.0, 8.0, 4.0], [3.0, 6.0, 7.0]], [[5.0, 1.0, 2.0], [3.0, 3.0, 1.0], [6.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCUN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 3.0], [1.0, 2.0, 3.0], [8.0, 3.0, 8.0]], [[7.0, 8.0, 5.0], [2.0, 1.0, 8.0], [5.0, 8.0, 
2.0]], [[2.0, 8.0, 7.0], [2.0, 2.0, 4.0], [8.0, 8.0, 2.0]]], [[[2.0, 4.0, 7.0], [6.0, 5.0, 1.0], [3.0, 1.0, 7.0]], [[1.0, 3.0, 
8.0], [6.0, 6.0, 4.0], [3.0, 8.0, 3.0]], [[2.0, 5.0, 2.0], [8.0, 5.0, 1.0], [8.0, 4.0, 4.0]]], [[[3.0, 2.0, 7.0], [6.0, 8.0, 
4.0], [6.0, 8.0, 5.0]], [[5.0, 5.0, 8.0], [5.0, 4.0, 2.0], [2.0, 3.0, 8.0]], [[7.0, 3.0, 5.0], [3.0, 1.0, 7.0], [3.0, 4.0, 
2.0]]], [[[6.0, 5.0, 5.0], [8.0, 8.0, 1.0], [5.0, 8.0, 4.0]], [[4.0, 2.0, 2.0], [7.0, 3.0, 5.0], [6.0, 2.0, 4.0]], [[2.0, 2.0, 
6.0], [3.0, 8.0, 7.0], [8.0, 7.0, 1.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 1.0], [2.0, 7.0, 6.0], [3.0, 2.0, 2.0]], [[7.0, 4.0, 8.0], [6.0, 6.0, 
4.0], [6.0, 2.0, 6.0]], [[6.0, 8.0, 3.0], [5.0, 6.0, 8.0], [3.0, 8.0, 5.0]]], [[[5.0, 7.0, 5.0], [5.0, 3.0, 6.0], [7.0, 6.0, 
5.0]], [[1.0, 8.0, 1.0], [8.0, 6.0, 1.0], [8.0, 1.0, 1.0]], [[5.0, 7.0, 1.0], [2.0, 8.0, 7.0], [3.0, 6.0, 5.0]]], [[[2.0, 6.0, 
3.0], [1.0, 7.0, 1.0], [7.0, 6.0, 6.0]], [[2.0, 5.0, 1.0], [1.0, 1.0, 2.0], [2.0, 4.0, 1.0]], [[7.0, 1.0, 1.0], [5.0, 5.0, 
2.0], [1.0, 4.0, 3.0]]], [[[6.0, 1.0, 2.0], [3.0, 4.0, 3.0], [5.0, 2.0, 8.0]], [[3.0, 4.0, 2.0], [5.0, 8.0, 4.0], [4.0, 3.0, 
2.0]], [[7.0, 3.0, 1.0], [6.0, 2.0, 5.0], [6.0, 1.0, 2.0]]]]))
      val=Data(numpy.array([[[[6.0, 7.0, 3.0], [1.0, 1.0, 3.0]], [[3.0, 7.0, 2.0], [7.0, 3.0, 6.0]], [[4.0, 3.0, 1.0], [8.0, 
5.0, 3.0]]], [[[5.0, 8.0, 5.0], [8.0, 1.0, 1.0]], [[2.0, 5.0, 2.0], [1.0, 4.0, 2.0]], [[2.0, 7.0, 8.0], [4.0, 3.0, 1.0]]], 
[[[4.0, 7.0, 5.0], [6.0, 3.0, 2.0]], [[4.0, 5.0, 6.0], [5.0, 7.0, 6.0]], [[1.0, 3.0, 1.0], [2.0, 1.0, 6.0]]], [[[4.0, 1.0, 
8.0], [6.0, 2.0, 6.0]], [[7.0, 8.0, 4.0], [4.0, 1.0, 1.0]], [[7.0, 5.0, 3.0], [4.0, 1.0, 2.0]]]]),self.functionspace)
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 3.0], [1.0, 1.0, 3.0], [8.0, 3.0, 8.0]], [[3.0, 7.0, 2.0], [7.0, 3.0, 6.0], [5.0, 
8.0, 2.0]], [[4.0, 3.0, 1.0], [8.0, 5.0, 3.0], [8.0, 8.0, 2.0]]], [[[5.0, 8.0, 5.0], [8.0, 1.0, 1.0], [3.0, 1.0, 7.0]], [[2.0, 
5.0, 2.0], [1.0, 4.0, 2.0], [3.0, 8.0, 3.0]], [[2.0, 7.0, 8.0], [4.0, 3.0, 1.0], [8.0, 4.0, 4.0]]], [[[4.0, 7.0, 5.0], [6.0, 
3.0, 2.0], [6.0, 8.0, 5.0]], [[4.0, 5.0, 6.0], [5.0, 7.0, 6.0], [2.0, 3.0, 8.0]], [[1.0, 3.0, 1.0], [2.0, 1.0, 6.0], [3.0, 4.0, 
2.0]]], [[[4.0, 1.0, 8.0], [6.0, 2.0, 6.0], [5.0, 8.0, 4.0]], [[7.0, 8.0, 4.0], [4.0, 1.0, 1.0], [6.0, 2.0, 4.0]], [[7.0, 5.0, 
3.0], [4.0, 1.0, 2.0], [8.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 3.0], [1.0, 1.0, 3.0], [3.0, 2.0, 2.0]], 
[[3.0, 7.0, 2.0], [7.0, 3.0, 6.0], [6.0, 2.0, 6.0]], [[4.0, 3.0, 1.0], [8.0, 5.0, 3.0], [3.0, 8.0, 5.0]]], [[[5.0, 8.0, 5.0], 
[8.0, 1.0, 1.0], [7.0, 6.0, 5.0]], [[2.0, 5.0, 2.0], [1.0, 4.0, 2.0], [8.0, 1.0, 1.0]], [[2.0, 7.0, 8.0], [4.0, 3.0, 1.0], 
[3.0, 6.0, 5.0]]], [[[4.0, 7.0, 5.0], [6.0, 3.0, 2.0], [7.0, 6.0, 6.0]], [[4.0, 5.0, 6.0], [5.0, 7.0, 6.0], [2.0, 4.0, 1.0]], 
[[1.0, 3.0, 1.0], [2.0, 1.0, 6.0], [1.0, 4.0, 3.0]]], [[[4.0, 1.0, 8.0], [6.0, 2.0, 6.0], [5.0, 2.0, 8.0]], [[7.0, 8.0, 4.0], 
[4.0, 1.0, 1.0], [4.0, 3.0, 2.0]], [[7.0, 5.0, 3.0], [4.0, 1.0, 2.0], [6.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCUN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 1.0], [5.0, 1.0, 3.0], [7.0, 2.0, 4.0]], [[1.0, 1.0, 3.0], [3.0, 3.0, 4.0], [3.0, 4.0, 
5.0]], [[4.0, 8.0, 3.0], [7.0, 4.0, 8.0], [8.0, 2.0, 8.0]]], [[[4.0, 6.0, 5.0], [8.0, 2.0, 2.0], [1.0, 2.0, 4.0]], [[7.0, 7.0, 
4.0], [8.0, 5.0, 2.0], [4.0, 1.0, 4.0]], [[8.0, 5.0, 7.0], [1.0, 1.0, 2.0], [6.0, 3.0, 6.0]]], [[[4.0, 3.0, 2.0], [5.0, 3.0, 
4.0], [4.0, 2.0, 7.0]], [[2.0, 8.0, 4.0], [1.0, 3.0, 4.0], [1.0, 1.0, 1.0]], [[5.0, 1.0, 1.0], [4.0, 3.0, 3.0], [2.0, 1.0, 
5.0]]], [[[5.0, 3.0, 8.0], [3.0, 6.0, 2.0], [3.0, 1.0, 3.0]], [[7.0, 4.0, 3.0], [7.0, 4.0, 3.0], [7.0, 6.0, 7.0]], [[7.0, 4.0, 
4.0], [8.0, 6.0, 7.0], [6.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 6.0], [4.0, 2.0, 4.0], [4.0, 5.0, 4.0]], [[8.0, 8.0, 3.0], [4.0, 3.0, 
8.0], [5.0, 8.0, 4.0]], [[1.0, 7.0, 5.0], [7.0, 6.0, 4.0], [6.0, 6.0, 2.0]]], [[[7.0, 4.0, 2.0], [4.0, 3.0, 2.0], [2.0, 3.0, 
5.0]], [[1.0, 5.0, 4.0], [4.0, 1.0, 7.0], [4.0, 1.0, 6.0]], [[8.0, 7.0, 8.0], [7.0, 5.0, 3.0], [6.0, 6.0, 5.0]]], [[[4.0, 8.0, 
1.0], [3.0, 2.0, 8.0], [2.0, 5.0, 1.0]], [[4.0, 1.0, 8.0], [4.0, 2.0, 4.0], [3.0, 1.0, 5.0]], [[2.0, 7.0, 8.0], [6.0, 4.0, 
6.0], [1.0, 7.0, 8.0]]], [[[5.0, 2.0, 2.0], [1.0, 1.0, 2.0], [1.0, 3.0, 3.0]], [[6.0, 4.0, 3.0], [4.0, 3.0, 4.0], [4.0, 8.0, 
2.0]], [[3.0, 2.0, 1.0], [6.0, 4.0, 8.0], [6.0, 4.0, 7.0]]]]))
      val=Data(numpy.array([[[[8.0, 1.0, 4.0], [7.0, 8.0, 5.0]], [[4.0, 8.0, 3.0], [7.0, 1.0, 2.0]], [[5.0, 8.0, 8.0], [7.0, 
5.0, 2.0]]], [[[5.0, 1.0, 5.0], [1.0, 2.0, 5.0]], [[7.0, 5.0, 3.0], [1.0, 6.0, 8.0]], [[4.0, 2.0, 6.0], [2.0, 2.0, 6.0]]], 
[[[7.0, 2.0, 5.0], [6.0, 2.0, 7.0]], [[8.0, 5.0, 3.0], [8.0, 1.0, 6.0]], [[8.0, 1.0, 6.0], [8.0, 4.0, 3.0]]], [[[2.0, 6.0, 
7.0], [6.0, 7.0, 4.0]], [[3.0, 7.0, 1.0], [3.0, 7.0, 1.0]], [[1.0, 1.0, 3.0], [6.0, 8.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 3.0, 8.0], [5.0, 7.0, 2.0]], [[7.0, 8.0, 2.0], [4.0, 6.0, 1.0]], [[2.0, 7.0, 
4.0], [1.0, 4.0, 5.0]]], [[[7.0, 2.0, 7.0], [2.0, 6.0, 1.0]], [[6.0, 1.0, 8.0], [2.0, 4.0, 7.0]], [[3.0, 1.0, 3.0], [8.0, 1.0, 
6.0]]], [[[1.0, 2.0, 1.0], [4.0, 5.0, 7.0]], [[3.0, 1.0, 5.0], [8.0, 3.0, 6.0]], [[5.0, 3.0, 5.0], [6.0, 2.0, 1.0]]], [[[5.0, 
7.0, 8.0], [4.0, 8.0, 4.0]], [[5.0, 2.0, 1.0], [5.0, 1.0, 2.0]], [[1.0, 7.0, 7.0], [8.0, 1.0, 4.0]]]]))
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 4.0], [7.0, 8.0, 5.0], [7.0, 2.0, 4.0]], [[4.0, 8.0, 3.0], [7.0, 1.0, 2.0], [3.0, 
4.0, 5.0]], [[5.0, 8.0, 8.0], [7.0, 5.0, 2.0], [8.0, 2.0, 8.0]]], [[[5.0, 1.0, 5.0], [1.0, 2.0, 5.0], [1.0, 2.0, 4.0]], [[7.0, 
5.0, 3.0], [1.0, 6.0, 8.0], [4.0, 1.0, 4.0]], [[4.0, 2.0, 6.0], [2.0, 2.0, 6.0], [6.0, 3.0, 6.0]]], [[[7.0, 2.0, 5.0], [6.0, 
2.0, 7.0], [4.0, 2.0, 7.0]], [[8.0, 5.0, 3.0], [8.0, 1.0, 6.0], [1.0, 1.0, 1.0]], [[8.0, 1.0, 6.0], [8.0, 4.0, 3.0], [2.0, 1.0, 
5.0]]], [[[2.0, 6.0, 7.0], [6.0, 7.0, 4.0], [3.0, 1.0, 3.0]], [[3.0, 7.0, 1.0], [3.0, 7.0, 1.0], [7.0, 6.0, 7.0]], [[1.0, 1.0, 
3.0], [6.0, 8.0, 4.0], [6.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 8.0], [5.0, 7.0, 2.0], [4.0, 5.0, 4.0]], 
[[7.0, 8.0, 2.0], [4.0, 6.0, 1.0], [5.0, 8.0, 4.0]], [[2.0, 7.0, 4.0], [1.0, 4.0, 5.0], [6.0, 6.0, 2.0]]], [[[7.0, 2.0, 7.0], 
[2.0, 6.0, 1.0], [2.0, 3.0, 5.0]], [[6.0, 1.0, 8.0], [2.0, 4.0, 7.0], [4.0, 1.0, 6.0]], [[3.0, 1.0, 3.0], [8.0, 1.0, 6.0], 
[6.0, 6.0, 5.0]]], [[[1.0, 2.0, 1.0], [4.0, 5.0, 7.0], [2.0, 5.0, 1.0]], [[3.0, 1.0, 5.0], [8.0, 3.0, 6.0], [3.0, 1.0, 5.0]], 
[[5.0, 3.0, 5.0], [6.0, 2.0, 1.0], [1.0, 7.0, 8.0]]], [[[5.0, 7.0, 8.0], [4.0, 8.0, 4.0], [1.0, 3.0, 3.0]], [[5.0, 2.0, 1.0], 
[5.0, 1.0, 2.0], [4.0, 8.0, 2.0]], [[1.0, 7.0, 7.0], [8.0, 1.0, 4.0], [6.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCUN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 5.0], [8.0, 3.0, 1.0], [5.0, 4.0, 8.0]], [[3.0, 6.0, 6.0], [8.0, 5.0, 2.0], [3.0, 1.0, 
4.0]], [[2.0, 7.0, 8.0], [4.0, 4.0, 7.0], [8.0, 8.0, 5.0]]], [[[2.0, 7.0, 3.0], [8.0, 3.0, 1.0], [8.0, 6.0, 4.0]], [[6.0, 1.0, 
6.0], [7.0, 8.0, 2.0], [3.0, 1.0, 7.0]], [[6.0, 8.0, 2.0], [7.0, 4.0, 7.0], [4.0, 1.0, 5.0]]], [[[4.0, 1.0, 2.0], [7.0, 5.0, 
8.0], [1.0, 3.0, 6.0]], [[3.0, 2.0, 5.0], [8.0, 6.0, 6.0], [3.0, 1.0, 7.0]], [[3.0, 6.0, 4.0], [6.0, 2.0, 8.0], [2.0, 8.0, 
7.0]]], [[[6.0, 6.0, 3.0], [6.0, 8.0, 5.0], [4.0, 7.0, 6.0]], [[4.0, 3.0, 6.0], [4.0, 8.0, 6.0], [3.0, 2.0, 3.0]], [[8.0, 3.0, 
7.0], [4.0, 3.0, 7.0], [8.0, 1.0, 6.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 5.0], [6.0, 5.0, 8.0], [4.0, 8.0, 5.0]], [[5.0, 6.0, 2.0], [5.0, 7.0, 
5.0], [1.0, 6.0, 3.0]], [[5.0, 3.0, 2.0], [7.0, 8.0, 8.0], [8.0, 5.0, 8.0]]], [[[1.0, 4.0, 8.0], [8.0, 3.0, 8.0], [7.0, 7.0, 
7.0]], [[6.0, 3.0, 8.0], [8.0, 6.0, 6.0], [8.0, 1.0, 3.0]], [[8.0, 2.0, 6.0], [8.0, 7.0, 2.0], [5.0, 2.0, 7.0]]], [[[5.0, 3.0, 
7.0], [4.0, 7.0, 6.0], [2.0, 8.0, 4.0]], [[6.0, 1.0, 3.0], [5.0, 5.0, 2.0], [6.0, 7.0, 3.0]], [[3.0, 5.0, 5.0], [1.0, 5.0, 
3.0], [5.0, 2.0, 3.0]]], [[[4.0, 3.0, 2.0], [7.0, 3.0, 8.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 5.0], [6.0, 7.0, 6.0], [5.0, 1.0, 
2.0]], [[8.0, 8.0, 5.0], [4.0, 1.0, 5.0], [2.0, 5.0, 1.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0, 3.0, 3.0], [1.0, 4.0, 8.0]], [[4.0, 1.0, 2.0], [1.0, 4.0, 4.0]], [[8.0, 6.0, 2.0], [5.0, 
8.0, 5.0]]], [[[1.0, 2.0, 3.0], [7.0, 7.0, 3.0]], [[2.0, 3.0, 1.0], [7.0, 8.0, 6.0]], [[7.0, 1.0, 7.0], [8.0, 1.0, 3.0]]], 
[[[6.0, 2.0, 6.0], [5.0, 3.0, 5.0]], [[1.0, 8.0, 7.0], [8.0, 4.0, 4.0]], [[4.0, 2.0, 4.0], [5.0, 1.0, 7.0]]], [[[5.0, 2.0, 
5.0], [1.0, 3.0, 5.0]], [[3.0, 1.0, 8.0], [7.0, 4.0, 8.0]], [[5.0, 7.0, 4.0], [7.0, 6.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 8.0, 6.0], [8.0, 3.0, 1.0]], [[4.0, 2.0, 6.0], [2.0, 5.0, 6.0]], [[7.0, 4.0, 8.0], 
[3.0, 8.0, 4.0]]], [[[7.0, 6.0, 2.0], [4.0, 1.0, 6.0]], [[3.0, 7.0, 7.0], [2.0, 4.0, 4.0]], [[4.0, 3.0, 1.0], [2.0, 5.0, 
7.0]]], [[[6.0, 8.0, 3.0], [6.0, 5.0, 1.0]], [[5.0, 8.0, 1.0], [1.0, 8.0, 4.0]], [[1.0, 3.0, 3.0], [3.0, 8.0, 2.0]]], [[[2.0, 
5.0, 2.0], [8.0, 2.0, 3.0]], [[8.0, 4.0, 5.0], [5.0, 8.0, 6.0]], [[4.0, 5.0, 7.0], [3.0, 8.0, 7.0]]]])
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 3.0], [1.0, 4.0, 8.0], [5.0, 4.0, 8.0]], [[4.0, 1.0, 2.0], [1.0, 4.0, 4.0], [3.0, 
1.0, 4.0]], [[8.0, 6.0, 2.0], [5.0, 8.0, 5.0], [8.0, 8.0, 5.0]]], [[[1.0, 2.0, 3.0], [7.0, 7.0, 3.0], [8.0, 6.0, 4.0]], [[2.0, 
3.0, 1.0], [7.0, 8.0, 6.0], [3.0, 1.0, 7.0]], [[7.0, 1.0, 7.0], [8.0, 1.0, 3.0], [4.0, 1.0, 5.0]]], [[[6.0, 2.0, 6.0], [5.0, 
3.0, 5.0], [1.0, 3.0, 6.0]], [[1.0, 8.0, 7.0], [8.0, 4.0, 4.0], [3.0, 1.0, 7.0]], [[4.0, 2.0, 4.0], [5.0, 1.0, 7.0], [2.0, 8.0, 
7.0]]], [[[5.0, 2.0, 5.0], [1.0, 3.0, 5.0], [4.0, 7.0, 6.0]], [[3.0, 1.0, 8.0], [7.0, 4.0, 8.0], [3.0, 2.0, 3.0]], [[5.0, 7.0, 
4.0], [7.0, 6.0, 7.0], [8.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 6.0], [8.0, 3.0, 1.0], [4.0, 8.0, 5.0]], 
[[4.0, 2.0, 6.0], [2.0, 5.0, 6.0], [1.0, 6.0, 3.0]], [[7.0, 4.0, 8.0], [3.0, 8.0, 4.0], [8.0, 5.0, 8.0]]], [[[7.0, 6.0, 2.0], 
[4.0, 1.0, 6.0], [7.0, 7.0, 7.0]], [[3.0, 7.0, 7.0], [2.0, 4.0, 4.0], [8.0, 1.0, 3.0]], [[4.0, 3.0, 1.0], [2.0, 5.0, 7.0], 
[5.0, 2.0, 7.0]]], [[[6.0, 8.0, 3.0], [6.0, 5.0, 1.0], [2.0, 8.0, 4.0]], [[5.0, 8.0, 1.0], [1.0, 8.0, 4.0], [6.0, 7.0, 3.0]], 
[[1.0, 3.0, 3.0], [3.0, 8.0, 2.0], [5.0, 2.0, 3.0]]], [[[2.0, 5.0, 2.0], [8.0, 2.0, 3.0], [5.0, 5.0, 2.0]], [[8.0, 4.0, 5.0], 
[5.0, 8.0, 6.0], [5.0, 1.0, 2.0]], [[4.0, 5.0, 7.0], [3.0, 8.0, 7.0], [2.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CCCN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 3.0], [6.0, 5.0, 4.0], [8.0, 2.0, 5.0]], [[8.0, 5.0, 5.0], [4.0, 2.0, 8.0], [5.0, 6.0, 
4.0]], [[3.0, 6.0, 6.0], [8.0, 1.0, 6.0], [3.0, 4.0, 8.0]]], [[[5.0, 4.0, 8.0], [1.0, 2.0, 1.0], [2.0, 1.0, 4.0]], [[1.0, 2.0, 
5.0], [6.0, 1.0, 5.0], [4.0, 7.0, 6.0]], [[4.0, 6.0, 2.0], [8.0, 3.0, 6.0], [6.0, 5.0, 6.0]]], [[[5.0, 4.0, 2.0], [5.0, 5.0, 
1.0], [1.0, 2.0, 6.0]], [[7.0, 7.0, 5.0], [6.0, 2.0, 2.0], [4.0, 1.0, 8.0]], [[3.0, 4.0, 1.0], [3.0, 5.0, 2.0], [2.0, 1.0, 
3.0]]], [[[2.0, 4.0, 6.0], [8.0, 7.0, 6.0], [8.0, 2.0, 8.0]], [[1.0, 8.0, 8.0], [5.0, 8.0, 2.0], [7.0, 1.0, 8.0]], [[4.0, 1.0, 
3.0], [1.0, 1.0, 2.0], [7.0, 4.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 5.0, 7.0], [5.0, 6.0, 6.0], [3.0, 6.0, 3.0]], [[5.0, 5.0, 3.0], [1.0, 3.0, 
8.0], [6.0, 2.0, 8.0]], [[4.0, 8.0, 1.0], [2.0, 7.0, 5.0], [2.0, 8.0, 8.0]]], [[[5.0, 8.0, 7.0], [8.0, 2.0, 6.0], [8.0, 5.0, 
6.0]], [[5.0, 7.0, 3.0], [1.0, 7.0, 4.0], [1.0, 3.0, 5.0]], [[5.0, 7.0, 2.0], [2.0, 2.0, 8.0], [7.0, 3.0, 3.0]]], [[[5.0, 7.0, 
6.0], [8.0, 1.0, 8.0], [5.0, 8.0, 3.0]], [[5.0, 2.0, 8.0], [6.0, 2.0, 5.0], [1.0, 8.0, 1.0]], [[4.0, 1.0, 1.0], [2.0, 4.0, 
1.0], [7.0, 3.0, 8.0]]], [[[6.0, 1.0, 2.0], [3.0, 5.0, 7.0], [5.0, 7.0, 2.0]], [[3.0, 5.0, 4.0], [5.0, 3.0, 5.0], [5.0, 7.0, 
8.0]], [[8.0, 2.0, 1.0], [4.0, 5.0, 2.0], [1.0, 3.0, 8.0]]]]))
      val=numpy.array([[[[3.0, 8.0, 3.0], [6.0, 2.0, 2.0], [4.0, 2.0, 8.0]], [[8.0, 5.0, 4.0], [5.0, 2.0, 5.0], [6.0, 2.0, 
5.0]], [[2.0, 4.0, 2.0], [7.0, 8.0, 3.0], [8.0, 5.0, 4.0]]], [[[7.0, 3.0, 3.0], [5.0, 7.0, 7.0], [3.0, 1.0, 8.0]], [[7.0, 3.0, 
3.0], [1.0, 1.0, 4.0], [5.0, 6.0, 8.0]], [[7.0, 5.0, 4.0], [7.0, 2.0, 8.0], [8.0, 3.0, 1.0]]], [[[1.0, 6.0, 1.0], [1.0, 8.0, 
4.0], [5.0, 4.0, 2.0]], [[6.0, 2.0, 1.0], [6.0, 3.0, 5.0], [7.0, 1.0, 3.0]], [[7.0, 4.0, 6.0], [1.0, 6.0, 8.0], [5.0, 4.0, 
3.0]]], [[[1.0, 6.0, 4.0], [3.0, 8.0, 5.0], [5.0, 2.0, 2.0]], [[4.0, 3.0, 4.0], [4.0, 6.0, 4.0], [2.0, 5.0, 5.0]], [[6.0, 6.0, 
3.0], [7.0, 5.0, 1.0], [5.0, 6.0, 3.0]]]])
      arg[:,:,:]=val
      ref=Data(numpy.array([[[[3.0, 8.0, 3.0], [6.0, 2.0, 2.0], [4.0, 2.0, 8.0]], [[8.0, 5.0, 4.0], [5.0, 2.0, 5.0], [6.0, 2.0, 
5.0]], [[2.0, 4.0, 2.0], [7.0, 8.0, 3.0], [8.0, 5.0, 4.0]]], [[[7.0, 3.0, 3.0], [5.0, 7.0, 7.0], [3.0, 1.0, 8.0]], [[7.0, 3.0, 
3.0], [1.0, 1.0, 4.0], [5.0, 6.0, 8.0]], [[7.0, 5.0, 4.0], [7.0, 2.0, 8.0], [8.0, 3.0, 1.0]]], [[[1.0, 6.0, 1.0], [1.0, 8.0, 
4.0], [5.0, 4.0, 2.0]], [[6.0, 2.0, 1.0], [6.0, 3.0, 5.0], [7.0, 1.0, 3.0]], [[7.0, 4.0, 6.0], [1.0, 6.0, 8.0], [5.0, 4.0, 
3.0]]], [[[1.0, 6.0, 4.0], [3.0, 8.0, 5.0], [5.0, 2.0, 2.0]], [[4.0, 3.0, 4.0], [4.0, 6.0, 4.0], [2.0, 5.0, 5.0]], [[6.0, 6.0, 
3.0], [7.0, 5.0, 1.0], [5.0, 6.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CCCN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 6.0], [1.0, 1.0, 1.0], [7.0, 3.0, 4.0]], [[1.0, 3.0, 1.0], [1.0, 6.0, 1.0], [7.0, 2.0, 
5.0]], [[5.0, 8.0, 2.0], [3.0, 6.0, 7.0], [7.0, 5.0, 3.0]]], [[[6.0, 2.0, 2.0], [8.0, 8.0, 7.0], [3.0, 7.0, 2.0]], [[8.0, 2.0, 
4.0], [6.0, 8.0, 2.0], [8.0, 6.0, 7.0]], [[2.0, 2.0, 4.0], [6.0, 7.0, 6.0], [2.0, 8.0, 3.0]]], [[[8.0, 6.0, 8.0], [4.0, 8.0, 
4.0], [5.0, 4.0, 1.0]], [[1.0, 4.0, 6.0], [8.0, 7.0, 2.0], [8.0, 3.0, 4.0]], [[8.0, 1.0, 4.0], [4.0, 3.0, 7.0], [7.0, 7.0, 
3.0]]], [[[3.0, 4.0, 1.0], [7.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[4.0, 3.0, 3.0], [8.0, 5.0, 6.0], [7.0, 4.0, 4.0]], [[3.0, 3.0, 
6.0], [5.0, 1.0, 8.0], [1.0, 6.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 4.0], [8.0, 1.0, 6.0], [7.0, 3.0, 7.0]], [[1.0, 4.0, 4.0], [8.0, 8.0, 
7.0], [8.0, 5.0, 2.0]], [[5.0, 3.0, 8.0], [5.0, 4.0, 5.0], [6.0, 8.0, 3.0]]], [[[7.0, 7.0, 2.0], [4.0, 7.0, 1.0], [5.0, 6.0, 
1.0]], [[6.0, 1.0, 3.0], [8.0, 1.0, 2.0], [6.0, 2.0, 1.0]], [[3.0, 8.0, 8.0], [8.0, 3.0, 6.0], [1.0, 6.0, 8.0]]], [[[7.0, 6.0, 
6.0], [5.0, 1.0, 7.0], [2.0, 2.0, 2.0]], [[3.0, 6.0, 4.0], [8.0, 6.0, 7.0], [4.0, 6.0, 3.0]], [[4.0, 1.0, 7.0], [5.0, 6.0, 
8.0], [7.0, 3.0, 8.0]]], [[[5.0, 8.0, 6.0], [5.0, 6.0, 2.0], [2.0, 1.0, 4.0]], [[2.0, 5.0, 5.0], [4.0, 7.0, 8.0], [3.0, 8.0, 
2.0]], [[3.0, 4.0, 3.0], [3.0, 4.0, 5.0], [8.0, 7.0, 7.0]]]]))
      val=Data(numpy.array([[[[2.0, 7.0, 4.0], [2.0, 2.0, 3.0], [2.0, 2.0, 6.0]], [[5.0, 6.0, 1.0], [5.0, 5.0, 7.0], [2.0, 4.0, 
2.0]], [[3.0, 1.0, 2.0], [8.0, 1.0, 2.0], [3.0, 3.0, 3.0]]], [[[5.0, 2.0, 7.0], [5.0, 2.0, 7.0], [1.0, 2.0, 4.0]], [[8.0, 3.0, 
5.0], [2.0, 4.0, 5.0], [2.0, 1.0, 8.0]], [[6.0, 4.0, 2.0], [2.0, 6.0, 2.0], [3.0, 6.0, 6.0]]], [[[7.0, 1.0, 5.0], [4.0, 4.0, 
8.0], [3.0, 1.0, 1.0]], [[1.0, 7.0, 7.0], [8.0, 7.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 1.0, 1.0], [2.0, 8.0, 4.0], [5.0, 3.0, 
6.0]]], [[[4.0, 1.0, 1.0], [8.0, 1.0, 7.0], [4.0, 8.0, 5.0]], [[5.0, 6.0, 4.0], [1.0, 4.0, 1.0], [4.0, 8.0, 6.0]], [[7.0, 7.0, 
8.0], [4.0, 3.0, 7.0], [2.0, 6.0, 6.0]]]]),self.functionspace)
      arg[:,:,:]=val
      ref=Data(numpy.array([[[[2.0, 7.0, 4.0], [2.0, 2.0, 3.0], [2.0, 2.0, 6.0]], [[5.0, 6.0, 1.0], [5.0, 5.0, 7.0], [2.0, 4.0, 
2.0]], [[3.0, 1.0, 2.0], [8.0, 1.0, 2.0], [3.0, 3.0, 3.0]]], [[[5.0, 2.0, 7.0], [5.0, 2.0, 7.0], [1.0, 2.0, 4.0]], [[8.0, 3.0, 
5.0], [2.0, 4.0, 5.0], [2.0, 1.0, 8.0]], [[6.0, 4.0, 2.0], [2.0, 6.0, 2.0], [3.0, 6.0, 6.0]]], [[[7.0, 1.0, 5.0], [4.0, 4.0, 
8.0], [3.0, 1.0, 1.0]], [[1.0, 7.0, 7.0], [8.0, 7.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 1.0, 1.0], [2.0, 8.0, 4.0], [5.0, 3.0, 
6.0]]], [[[4.0, 1.0, 1.0], [8.0, 1.0, 7.0], [4.0, 8.0, 5.0]], [[5.0, 6.0, 4.0], [1.0, 4.0, 1.0], [4.0, 8.0, 6.0]], [[7.0, 7.0, 
8.0], [4.0, 3.0, 7.0], [2.0, 6.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CCCN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 1.0], [5.0, 3.0, 4.0], [4.0, 5.0, 8.0]], [[7.0, 5.0, 4.0], [4.0, 2.0, 7.0], [7.0, 7.0, 
8.0]], [[6.0, 6.0, 2.0], [1.0, 1.0, 7.0], [6.0, 6.0, 7.0]]], [[[4.0, 5.0, 2.0], [7.0, 3.0, 3.0], [7.0, 1.0, 2.0]], [[4.0, 5.0, 
1.0], [8.0, 2.0, 1.0], [7.0, 7.0, 7.0]], [[2.0, 1.0, 2.0], [8.0, 7.0, 4.0], [2.0, 1.0, 6.0]]], [[[4.0, 3.0, 5.0], [3.0, 8.0, 
4.0], [8.0, 8.0, 1.0]], [[4.0, 3.0, 5.0], [8.0, 6.0, 8.0], [4.0, 3.0, 2.0]], [[3.0, 4.0, 6.0], [8.0, 8.0, 8.0], [5.0, 8.0, 
8.0]]], [[[4.0, 8.0, 3.0], [3.0, 6.0, 5.0], [7.0, 5.0, 4.0]], [[4.0, 5.0, 7.0], [5.0, 2.0, 2.0], [4.0, 4.0, 5.0]], [[2.0, 7.0, 
6.0], [5.0, 3.0, 7.0], [8.0, 3.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[1.0, 8.0, 4.0], [6.0, 8.0, 5.0], [8.0, 6.0, 1.0]], [[6.0, 6.0, 3.0], [3.0, 4.0, 
8.0], [5.0, 4.0, 4.0]], [[7.0, 4.0, 7.0], [8.0, 8.0, 7.0], [8.0, 2.0, 8.0]]], [[[6.0, 3.0, 3.0], [2.0, 1.0, 1.0], [8.0, 7.0, 
7.0]], [[7.0, 8.0, 7.0], [3.0, 1.0, 5.0], [4.0, 3.0, 8.0]], [[8.0, 8.0, 3.0], [7.0, 3.0, 7.0], [5.0, 3.0, 3.0]]], [[[2.0, 8.0, 
4.0], [3.0, 8.0, 2.0], [4.0, 5.0, 8.0]], [[7.0, 6.0, 7.0], [8.0, 8.0, 6.0], [3.0, 8.0, 7.0]], [[7.0, 3.0, 2.0], [6.0, 6.0, 
6.0], [8.0, 1.0, 7.0]]], [[[8.0, 4.0, 8.0], [5.0, 7.0, 1.0], [5.0, 1.0, 3.0]], [[2.0, 1.0, 8.0], [5.0, 7.0, 7.0], [4.0, 7.0, 
5.0]], [[5.0, 4.0, 2.0], [7.0, 7.0, 7.0], [5.0, 7.0, 2.0]]]]))
      val=Data(numpy.array([[[[2.0, 6.0, 3.0], [8.0, 8.0, 2.0], [7.0, 8.0, 4.0]], [[6.0, 3.0, 8.0], [7.0, 3.0, 1.0], [2.0, 8.0, 
6.0]], [[7.0, 4.0, 1.0], [5.0, 8.0, 8.0], [2.0, 3.0, 3.0]]], [[[3.0, 7.0, 1.0], [5.0, 4.0, 1.0], [5.0, 7.0, 5.0]], [[3.0, 2.0, 
1.0], [5.0, 4.0, 3.0], [4.0, 5.0, 4.0]], [[6.0, 7.0, 6.0], [4.0, 8.0, 7.0], [8.0, 6.0, 1.0]]], [[[8.0, 6.0, 1.0], [6.0, 7.0, 
3.0], [6.0, 1.0, 7.0]], [[4.0, 5.0, 2.0], [3.0, 8.0, 4.0], [7.0, 1.0, 7.0]], [[6.0, 1.0, 8.0], [3.0, 7.0, 2.0], [2.0, 7.0, 
7.0]]], [[[8.0, 5.0, 7.0], [1.0, 2.0, 7.0], [4.0, 4.0, 3.0]], [[1.0, 3.0, 4.0], [4.0, 6.0, 2.0], [6.0, 5.0, 3.0]], [[5.0, 1.0, 
7.0], [7.0, 1.0, 8.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 5.0], [6.0, 7.0, 6.0], [3.0, 6.0, 8.0]], [[5.0, 3.0, 7.0], [2.0, 7.0, 
6.0], [4.0, 1.0, 1.0]], [[5.0, 4.0, 5.0], [7.0, 4.0, 3.0], [5.0, 3.0, 6.0]]], [[[8.0, 2.0, 1.0], [5.0, 3.0, 8.0], [4.0, 8.0, 
3.0]], [[5.0, 3.0, 1.0], [4.0, 7.0, 5.0], [1.0, 1.0, 1.0]], [[8.0, 5.0, 3.0], [2.0, 1.0, 7.0], [1.0, 6.0, 5.0]]], [[[5.0, 4.0, 
5.0], [2.0, 3.0, 6.0], [3.0, 8.0, 2.0]], [[7.0, 2.0, 5.0], [8.0, 5.0, 6.0], [8.0, 5.0, 6.0]], [[1.0, 3.0, 4.0], [7.0, 8.0, 
1.0], [6.0, 3.0, 7.0]]], [[[3.0, 2.0, 1.0], [6.0, 2.0, 5.0], [5.0, 4.0, 1.0]], [[7.0, 4.0, 5.0], [3.0, 3.0, 6.0], [7.0, 1.0, 
6.0]], [[6.0, 2.0, 7.0], [5.0, 2.0, 6.0], [2.0, 4.0, 6.0]]]]))
      arg[:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 3.0], [8.0, 8.0, 2.0], [7.0, 8.0, 4.0]], [[6.0, 3.0, 8.0], [7.0, 3.0, 1.0], [2.0, 
8.0, 6.0]], [[7.0, 4.0, 1.0], [5.0, 8.0, 8.0], [2.0, 3.0, 3.0]]], [[[3.0, 7.0, 1.0], [5.0, 4.0, 1.0], [5.0, 7.0, 5.0]], [[3.0, 
2.0, 1.0], [5.0, 4.0, 3.0], [4.0, 5.0, 4.0]], [[6.0, 7.0, 6.0], [4.0, 8.0, 7.0], [8.0, 6.0, 1.0]]], [[[8.0, 6.0, 1.0], [6.0, 
7.0, 3.0], [6.0, 1.0, 7.0]], [[4.0, 5.0, 2.0], [3.0, 8.0, 4.0], [7.0, 1.0, 7.0]], [[6.0, 1.0, 8.0], [3.0, 7.0, 2.0], [2.0, 7.0, 
7.0]]], [[[8.0, 5.0, 7.0], [1.0, 2.0, 7.0], [4.0, 4.0, 3.0]], [[1.0, 3.0, 4.0], [4.0, 6.0, 2.0], [6.0, 5.0, 3.0]], [[5.0, 1.0, 
7.0], [7.0, 1.0, 8.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 5.0], [6.0, 7.0, 6.0], [3.0, 6.0, 8.0]], 
[[5.0, 3.0, 7.0], [2.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[5.0, 4.0, 5.0], [7.0, 4.0, 3.0], [5.0, 3.0, 6.0]]], [[[8.0, 2.0, 1.0], 
[5.0, 3.0, 8.0], [4.0, 8.0, 3.0]], [[5.0, 3.0, 1.0], [4.0, 7.0, 5.0], [1.0, 1.0, 1.0]], [[8.0, 5.0, 3.0], [2.0, 1.0, 7.0], 
[1.0, 6.0, 5.0]]], [[[5.0, 4.0, 5.0], [2.0, 3.0, 6.0], [3.0, 8.0, 2.0]], [[7.0, 2.0, 5.0], [8.0, 5.0, 6.0], [8.0, 5.0, 6.0]], 
[[1.0, 3.0, 4.0], [7.0, 8.0, 1.0], [6.0, 3.0, 7.0]]], [[[3.0, 2.0, 1.0], [6.0, 2.0, 5.0], [5.0, 4.0, 1.0]], [[7.0, 4.0, 5.0], 
[3.0, 3.0, 6.0], [7.0, 1.0, 6.0]], [[6.0, 2.0, 7.0], [5.0, 2.0, 6.0], [2.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CCCN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 3.0], [3.0, 4.0, 4.0], [4.0, 1.0, 5.0]], [[5.0, 3.0, 2.0], [6.0, 7.0, 5.0], [1.0, 7.0, 
8.0]], [[8.0, 8.0, 2.0], [4.0, 1.0, 3.0], [4.0, 1.0, 4.0]]], [[[7.0, 8.0, 6.0], [7.0, 4.0, 7.0], [1.0, 4.0, 2.0]], [[8.0, 6.0, 
7.0], [6.0, 1.0, 8.0], [8.0, 4.0, 1.0]], [[1.0, 2.0, 7.0], [8.0, 3.0, 1.0], [3.0, 3.0, 2.0]]], [[[5.0, 5.0, 5.0], [5.0, 2.0, 
8.0], [4.0, 7.0, 7.0]], [[7.0, 1.0, 8.0], [6.0, 6.0, 6.0], [4.0, 4.0, 2.0]], [[2.0, 8.0, 7.0], [5.0, 8.0, 2.0], [2.0, 4.0, 
8.0]]], [[[1.0, 6.0, 2.0], [4.0, 5.0, 6.0], [3.0, 7.0, 2.0]], [[8.0, 5.0, 1.0], [4.0, 3.0, 4.0], [2.0, 7.0, 3.0]], [[3.0, 1.0, 
4.0], [3.0, 2.0, 4.0], [6.0, 4.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 2.0], [7.0, 1.0, 2.0], [3.0, 7.0, 5.0]], [[5.0, 4.0, 3.0], [8.0, 4.0, 
8.0], [2.0, 3.0, 6.0]], [[1.0, 6.0, 4.0], [6.0, 2.0, 6.0], [7.0, 2.0, 4.0]]], [[[7.0, 3.0, 8.0], [5.0, 6.0, 7.0], [6.0, 4.0, 
4.0]], [[4.0, 4.0, 4.0], [8.0, 1.0, 7.0], [5.0, 1.0, 4.0]], [[3.0, 2.0, 8.0], [5.0, 1.0, 4.0], [3.0, 5.0, 8.0]]], [[[3.0, 5.0, 
6.0], [6.0, 1.0, 2.0], [3.0, 1.0, 2.0]], [[4.0, 3.0, 4.0], [6.0, 3.0, 7.0], [2.0, 2.0, 6.0]], [[3.0, 7.0, 3.0], [3.0, 1.0, 
2.0], [6.0, 8.0, 3.0]]], [[[1.0, 6.0, 3.0], [6.0, 6.0, 1.0], [6.0, 5.0, 1.0]], [[3.0, 2.0, 5.0], [5.0, 6.0, 6.0], [3.0, 6.0, 
3.0]], [[3.0, 5.0, 6.0], [4.0, 8.0, 1.0], [2.0, 3.0, 8.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[1.0, 2.0, 5.0], [1.0, 1.0, 3.0], [2.0, 3.0, 3.0]], [[6.0, 5.0, 5.0], [3.0, 7.0, 2.0], [4.0, 
4.0, 1.0]], [[4.0, 1.0, 2.0], [7.0, 7.0, 5.0], [2.0, 8.0, 2.0]]], [[[8.0, 2.0, 5.0], [2.0, 7.0, 3.0], [3.0, 4.0, 6.0]], [[2.0, 
7.0, 3.0], [3.0, 2.0, 6.0], [3.0, 5.0, 7.0]], [[6.0, 6.0, 5.0], [4.0, 3.0, 6.0], [7.0, 3.0, 4.0]]], [[[6.0, 8.0, 4.0], [1.0, 
5.0, 6.0], [4.0, 7.0, 4.0]], [[1.0, 8.0, 4.0], [8.0, 8.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, 2.0, 3.0], [3.0, 4.0, 2.0], [3.0, 4.0, 
7.0]]], [[[2.0, 6.0, 7.0], [6.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[4.0, 7.0, 2.0], [2.0, 4.0, 8.0], [7.0, 6.0, 1.0]], [[6.0, 1.0, 
2.0], [4.0, 7.0, 8.0], [1.0, 4.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 5.0, 1.0], [8.0, 5.0, 3.0], [2.0, 4.0, 2.0]], 
[[5.0, 6.0, 8.0], [1.0, 4.0, 4.0], [7.0, 7.0, 1.0]], [[1.0, 1.0, 2.0], [2.0, 3.0, 1.0], [7.0, 8.0, 5.0]]], [[[6.0, 5.0, 2.0], 
[7.0, 4.0, 8.0], [7.0, 2.0, 8.0]], [[8.0, 1.0, 7.0], [8.0, 2.0, 3.0], [2.0, 2.0, 3.0]], [[7.0, 6.0, 7.0], [7.0, 3.0, 3.0], 
[3.0, 8.0, 5.0]]], [[[1.0, 6.0, 6.0], [1.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[6.0, 3.0, 2.0], [5.0, 8.0, 1.0], [4.0, 5.0, 3.0]], 
[[1.0, 4.0, 7.0], [2.0, 1.0, 8.0], [1.0, 4.0, 8.0]]], [[[5.0, 8.0, 3.0], [2.0, 4.0, 5.0], [2.0, 5.0, 6.0]], [[6.0, 4.0, 5.0], 
[6.0, 6.0, 3.0], [2.0, 7.0, 7.0]], [[6.0, 1.0, 3.0], [1.0, 3.0, 6.0], [7.0, 7.0, 7.0]]]])
      arg[:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 5.0], [1.0, 1.0, 3.0], [2.0, 3.0, 3.0]], [[6.0, 5.0, 5.0], [3.0, 7.0, 2.0], [4.0, 
4.0, 1.0]], [[4.0, 1.0, 2.0], [7.0, 7.0, 5.0], [2.0, 8.0, 2.0]]], [[[8.0, 2.0, 5.0], [2.0, 7.0, 3.0], [3.0, 4.0, 6.0]], [[2.0, 
7.0, 3.0], [3.0, 2.0, 6.0], [3.0, 5.0, 7.0]], [[6.0, 6.0, 5.0], [4.0, 3.0, 6.0], [7.0, 3.0, 4.0]]], [[[6.0, 8.0, 4.0], [1.0, 
5.0, 6.0], [4.0, 7.0, 4.0]], [[1.0, 8.0, 4.0], [8.0, 8.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, 2.0, 3.0], [3.0, 4.0, 2.0], [3.0, 4.0, 
7.0]]], [[[2.0, 6.0, 7.0], [6.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[4.0, 7.0, 2.0], [2.0, 4.0, 8.0], [7.0, 6.0, 1.0]], [[6.0, 1.0, 
2.0], [4.0, 7.0, 8.0], [1.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 1.0], [8.0, 5.0, 3.0], [2.0, 4.0, 2.0]], 
[[5.0, 6.0, 8.0], [1.0, 4.0, 4.0], [7.0, 7.0, 1.0]], [[1.0, 1.0, 2.0], [2.0, 3.0, 1.0], [7.0, 8.0, 5.0]]], [[[6.0, 5.0, 2.0], 
[7.0, 4.0, 8.0], [7.0, 2.0, 8.0]], [[8.0, 1.0, 7.0], [8.0, 2.0, 3.0], [2.0, 2.0, 3.0]], [[7.0, 6.0, 7.0], [7.0, 3.0, 3.0], 
[3.0, 8.0, 5.0]]], [[[1.0, 6.0, 6.0], [1.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[6.0, 3.0, 2.0], [5.0, 8.0, 1.0], [4.0, 5.0, 3.0]], 
[[1.0, 4.0, 7.0], [2.0, 1.0, 8.0], [1.0, 4.0, 8.0]]], [[[5.0, 8.0, 3.0], [2.0, 4.0, 5.0], [2.0, 5.0, 6.0]], [[6.0, 4.0, 5.0], 
[6.0, 6.0, 3.0], [2.0, 7.0, 7.0]], [[6.0, 1.0, 3.0], [1.0, 3.0, 6.0], [7.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_ULUN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 6.0], [3.0, 5.0, 1.0], [6.0, 2.0, 1.0]], [[6.0, 7.0, 6.0], [4.0, 6.0, 2.0], [5.0, 3.0, 
7.0]], [[7.0, 3.0, 7.0], [6.0, 6.0, 8.0], [7.0, 5.0, 6.0]]], [[[4.0, 8.0, 5.0], [7.0, 4.0, 1.0], [3.0, 6.0, 6.0]], [[7.0, 3.0, 
6.0], [1.0, 1.0, 1.0], [3.0, 7.0, 3.0]], [[5.0, 5.0, 2.0], [6.0, 4.0, 1.0], [2.0, 3.0, 7.0]]], [[[8.0, 5.0, 3.0], [6.0, 6.0, 
1.0], [6.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [1.0, 8.0, 7.0], [8.0, 4.0, 1.0]], [[8.0, 6.0, 4.0], [8.0, 2.0, 4.0], [4.0, 2.0, 
5.0]]], [[[6.0, 1.0, 6.0], [6.0, 6.0, 2.0], [4.0, 7.0, 8.0]], [[8.0, 1.0, 7.0], [1.0, 6.0, 4.0], [1.0, 7.0, 6.0]], [[5.0, 5.0, 
5.0], [3.0, 6.0, 8.0], [8.0, 1.0, 2.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 3.0], [2.0, 1.0, 1.0], [3.0, 6.0, 6.0]], [[3.0, 6.0, 8.0], [8.0, 1.0, 
3.0], [4.0, 4.0, 2.0]], [[8.0, 1.0, 7.0], [2.0, 6.0, 7.0], [8.0, 6.0, 6.0]]], [[[4.0, 6.0, 2.0], [4.0, 1.0, 3.0], [5.0, 8.0, 
1.0]], [[7.0, 5.0, 4.0], [6.0, 4.0, 1.0], [4.0, 2.0, 6.0]], [[7.0, 6.0, 7.0], [3.0, 4.0, 3.0], [1.0, 2.0, 8.0]]], [[[1.0, 3.0, 
1.0], [3.0, 2.0, 8.0], [2.0, 3.0, 8.0]], [[7.0, 5.0, 5.0], [3.0, 8.0, 7.0], [1.0, 3.0, 3.0]], [[8.0, 8.0, 7.0], [2.0, 6.0, 
4.0], [3.0, 5.0, 8.0]]], [[[7.0, 3.0, 8.0], [1.0, 5.0, 3.0], [7.0, 3.0, 8.0]], [[2.0, 7.0, 2.0], [5.0, 6.0, 1.0], [5.0, 4.0, 
2.0]], [[4.0, 8.0, 3.0], [2.0, 7.0, 4.0], [8.0, 8.0, 2.0]]]]))
      val=numpy.array([[[[1.0, 4.0, 4.0], [8.0, 2.0, 4.0]]], [[[1.0, 5.0, 2.0], [5.0, 3.0, 3.0]]], [[[6.0, 3.0, 3.0], [5.0, 
6.0, 1.0]]]])
      arg[:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 6.0], [3.0, 5.0, 1.0], [6.0, 2.0, 1.0]], [[6.0, 7.0, 6.0], [4.0, 6.0, 2.0], [5.0, 
3.0, 7.0]], [[1.0, 4.0, 4.0], [8.0, 2.0, 4.0], [7.0, 5.0, 6.0]]], [[[4.0, 8.0, 5.0], [7.0, 4.0, 1.0], [3.0, 6.0, 6.0]], [[7.0, 
3.0, 6.0], [1.0, 1.0, 1.0], [3.0, 7.0, 3.0]], [[1.0, 5.0, 2.0], [5.0, 3.0, 3.0], [2.0, 3.0, 7.0]]], [[[8.0, 5.0, 3.0], [6.0, 
6.0, 1.0], [6.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [1.0, 8.0, 7.0], [8.0, 4.0, 1.0]], [[6.0, 3.0, 3.0], [5.0, 6.0, 1.0], [4.0, 2.0, 
5.0]]], [[[6.0, 1.0, 6.0], [6.0, 6.0, 2.0], [4.0, 7.0, 8.0]], [[8.0, 1.0, 7.0], [1.0, 6.0, 4.0], [1.0, 7.0, 6.0]], [[5.0, 5.0, 
5.0], [3.0, 6.0, 8.0], [8.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 3.0], [2.0, 1.0, 1.0], [3.0, 6.0, 6.0]], 
[[3.0, 6.0, 8.0], [8.0, 1.0, 3.0], [4.0, 4.0, 2.0]], [[1.0, 4.0, 4.0], [8.0, 2.0, 4.0], [8.0, 6.0, 6.0]]], [[[4.0, 6.0, 2.0], 
[4.0, 1.0, 3.0], [5.0, 8.0, 1.0]], [[7.0, 5.0, 4.0], [6.0, 4.0, 1.0], [4.0, 2.0, 6.0]], [[1.0, 5.0, 2.0], [5.0, 3.0, 3.0], 
[1.0, 2.0, 8.0]]], [[[1.0, 3.0, 1.0], [3.0, 2.0, 8.0], [2.0, 3.0, 8.0]], [[7.0, 5.0, 5.0], [3.0, 8.0, 7.0], [1.0, 3.0, 3.0]], 
[[6.0, 3.0, 3.0], [5.0, 6.0, 1.0], [3.0, 5.0, 8.0]]], [[[7.0, 3.0, 8.0], [1.0, 5.0, 3.0], [7.0, 3.0, 8.0]], [[2.0, 7.0, 2.0], 
[5.0, 6.0, 1.0], [5.0, 4.0, 2.0]], [[4.0, 8.0, 3.0], [2.0, 7.0, 4.0], [8.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_ULUN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 5.0], [6.0, 1.0, 8.0], [8.0, 1.0, 8.0]], [[6.0, 4.0, 2.0], [1.0, 5.0, 6.0], [7.0, 8.0, 
1.0]], [[4.0, 3.0, 1.0], [6.0, 2.0, 6.0], [6.0, 1.0, 3.0]]], [[[4.0, 6.0, 2.0], [1.0, 1.0, 6.0], [8.0, 8.0, 2.0]], [[3.0, 3.0, 
6.0], [7.0, 6.0, 7.0], [8.0, 7.0, 5.0]], [[8.0, 5.0, 1.0], [6.0, 7.0, 3.0], [6.0, 6.0, 4.0]]], [[[8.0, 6.0, 6.0], [4.0, 7.0, 
7.0], [4.0, 3.0, 3.0]], [[2.0, 7.0, 1.0], [1.0, 4.0, 7.0], [5.0, 2.0, 7.0]], [[1.0, 1.0, 5.0], [4.0, 8.0, 1.0], [4.0, 1.0, 
3.0]]], [[[3.0, 6.0, 1.0], [5.0, 7.0, 5.0], [3.0, 7.0, 6.0]], [[5.0, 8.0, 5.0], [3.0, 1.0, 1.0], [6.0, 3.0, 7.0]], [[7.0, 1.0, 
8.0], [5.0, 6.0, 4.0], [7.0, 2.0, 7.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 7.0], [6.0, 3.0, 6.0], [3.0, 5.0, 2.0]], [[1.0, 6.0, 6.0], [2.0, 5.0, 
8.0], [6.0, 2.0, 5.0]], [[8.0, 5.0, 3.0], [7.0, 4.0, 7.0], [7.0, 7.0, 6.0]]], [[[3.0, 2.0, 3.0], [8.0, 5.0, 2.0], [4.0, 2.0, 
8.0]], [[5.0, 6.0, 5.0], [8.0, 3.0, 1.0], [2.0, 8.0, 6.0]], [[8.0, 6.0, 8.0], [3.0, 1.0, 8.0], [7.0, 3.0, 1.0]]], [[[8.0, 5.0, 
3.0], [4.0, 4.0, 1.0], [4.0, 1.0, 5.0]], [[3.0, 2.0, 1.0], [3.0, 4.0, 4.0], [4.0, 8.0, 1.0]], [[4.0, 4.0, 2.0], [3.0, 2.0, 
3.0], [2.0, 8.0, 5.0]]], [[[1.0, 3.0, 5.0], [7.0, 6.0, 1.0], [4.0, 3.0, 3.0]], [[1.0, 4.0, 1.0], [7.0, 2.0, 5.0], [2.0, 1.0, 
1.0]], [[3.0, 3.0, 6.0], [3.0, 3.0, 6.0], [5.0, 4.0, 3.0]]]]))
      val=Data(numpy.array([[[[1.0, 7.0, 6.0], [4.0, 2.0, 3.0]]], [[[7.0, 4.0, 6.0], [4.0, 5.0, 7.0]]], [[[7.0, 1.0, 5.0], 
[3.0, 2.0, 5.0]]]]),self.functionspace)
      arg[:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 5.0], [6.0, 1.0, 8.0], [8.0, 1.0, 8.0]], [[6.0, 4.0, 2.0], [1.0, 5.0, 6.0], [7.0, 
8.0, 1.0]], [[1.0, 7.0, 6.0], [4.0, 2.0, 3.0], [6.0, 1.0, 3.0]]], [[[4.0, 6.0, 2.0], [1.0, 1.0, 6.0], [8.0, 8.0, 2.0]], [[3.0, 
3.0, 6.0], [7.0, 6.0, 7.0], [8.0, 7.0, 5.0]], [[7.0, 4.0, 6.0], [4.0, 5.0, 7.0], [6.0, 6.0, 4.0]]], [[[8.0, 6.0, 6.0], [4.0, 
7.0, 7.0], [4.0, 3.0, 3.0]], [[2.0, 7.0, 1.0], [1.0, 4.0, 7.0], [5.0, 2.0, 7.0]], [[7.0, 1.0, 5.0], [3.0, 2.0, 5.0], [4.0, 1.0, 
3.0]]], [[[3.0, 6.0, 1.0], [5.0, 7.0, 5.0], [3.0, 7.0, 6.0]], [[5.0, 8.0, 5.0], [3.0, 1.0, 1.0], [6.0, 3.0, 7.0]], [[7.0, 1.0, 
8.0], [5.0, 6.0, 4.0], [7.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 7.0], [6.0, 3.0, 6.0], [3.0, 5.0, 2.0]], 
[[1.0, 6.0, 6.0], [2.0, 5.0, 8.0], [6.0, 2.0, 5.0]], [[1.0, 7.0, 6.0], [4.0, 2.0, 3.0], [7.0, 7.0, 6.0]]], [[[3.0, 2.0, 3.0], 
[8.0, 5.0, 2.0], [4.0, 2.0, 8.0]], [[5.0, 6.0, 5.0], [8.0, 3.0, 1.0], [2.0, 8.0, 6.0]], [[7.0, 4.0, 6.0], [4.0, 5.0, 7.0], 
[7.0, 3.0, 1.0]]], [[[8.0, 5.0, 3.0], [4.0, 4.0, 1.0], [4.0, 1.0, 5.0]], [[3.0, 2.0, 1.0], [3.0, 4.0, 4.0], [4.0, 8.0, 1.0]], 
[[7.0, 1.0, 5.0], [3.0, 2.0, 5.0], [2.0, 8.0, 5.0]]], [[[1.0, 3.0, 5.0], [7.0, 6.0, 1.0], [4.0, 3.0, 3.0]], [[1.0, 4.0, 1.0], 
[7.0, 2.0, 5.0], [2.0, 1.0, 1.0]], [[3.0, 3.0, 6.0], [3.0, 3.0, 6.0], [5.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_ULUN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 7.0], [3.0, 1.0, 6.0], [3.0, 7.0, 4.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 7.0], [5.0, 6.0, 
6.0]], [[7.0, 1.0, 6.0], [4.0, 3.0, 3.0], [7.0, 7.0, 2.0]]], [[[3.0, 6.0, 8.0], [3.0, 5.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 7.0, 
1.0], [5.0, 1.0, 3.0], [1.0, 5.0, 3.0]], [[2.0, 6.0, 2.0], [2.0, 3.0, 4.0], [6.0, 8.0, 3.0]]], [[[5.0, 4.0, 5.0], [5.0, 1.0, 
6.0], [7.0, 8.0, 4.0]], [[4.0, 6.0, 6.0], [4.0, 8.0, 4.0], [1.0, 7.0, 8.0]], [[5.0, 2.0, 2.0], [4.0, 2.0, 1.0], [6.0, 6.0, 
4.0]]], [[[8.0, 8.0, 6.0], [5.0, 7.0, 6.0], [5.0, 4.0, 3.0]], [[2.0, 1.0, 7.0], [8.0, 1.0, 2.0], [8.0, 3.0, 1.0]], [[3.0, 6.0, 
1.0], [8.0, 8.0, 1.0], [8.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 5.0, 4.0], [2.0, 7.0, 4.0], [7.0, 8.0, 2.0]], [[7.0, 7.0, 6.0], [3.0, 2.0, 
7.0], [2.0, 4.0, 5.0]], [[2.0, 5.0, 8.0], [1.0, 1.0, 1.0], [1.0, 2.0, 4.0]]], [[[3.0, 6.0, 5.0], [8.0, 6.0, 7.0], [6.0, 8.0, 
2.0]], [[8.0, 6.0, 6.0], [2.0, 3.0, 3.0], [8.0, 3.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 7.0, 4.0], [5.0, 6.0, 5.0]]], [[[5.0, 8.0, 
6.0], [1.0, 6.0, 4.0], [3.0, 4.0, 6.0]], [[6.0, 4.0, 6.0], [2.0, 7.0, 4.0], [3.0, 2.0, 5.0]], [[7.0, 4.0, 6.0], [8.0, 3.0, 
3.0], [4.0, 5.0, 6.0]]], [[[8.0, 2.0, 8.0], [2.0, 3.0, 6.0], [3.0, 7.0, 6.0]], [[2.0, 2.0, 1.0], [2.0, 7.0, 8.0], [4.0, 2.0, 
4.0]], [[2.0, 4.0, 1.0], [4.0, 8.0, 3.0], [7.0, 2.0, 6.0]]]]))
      val=Data(numpy.array([[[[4.0, 3.0, 6.0], [4.0, 7.0, 2.0]]], [[[2.0, 6.0, 7.0], [1.0, 7.0, 5.0]]], [[[6.0, 1.0, 6.0], 
[4.0, 1.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 6.0], [5.0, 6.0, 5.0]]], [[[6.0, 8.0, 7.0], [4.0, 2.0, 4.0]]], [[[8.0, 
1.0, 3.0], [3.0, 8.0, 8.0]]]]))
      arg[:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 7.0], [3.0, 1.0, 6.0], [3.0, 7.0, 4.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 7.0], [5.0, 
6.0, 6.0]], [[4.0, 3.0, 6.0], [4.0, 7.0, 2.0], [7.0, 7.0, 2.0]]], [[[3.0, 6.0, 8.0], [3.0, 5.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 
7.0, 1.0], [5.0, 1.0, 3.0], [1.0, 5.0, 3.0]], [[2.0, 6.0, 7.0], [1.0, 7.0, 5.0], [6.0, 8.0, 3.0]]], [[[5.0, 4.0, 5.0], [5.0, 
1.0, 6.0], [7.0, 8.0, 4.0]], [[4.0, 6.0, 6.0], [4.0, 8.0, 4.0], [1.0, 7.0, 8.0]], [[6.0, 1.0, 6.0], [4.0, 1.0, 2.0], [6.0, 6.0, 
4.0]]], [[[8.0, 8.0, 6.0], [5.0, 7.0, 6.0], [5.0, 4.0, 3.0]], [[2.0, 1.0, 7.0], [8.0, 1.0, 2.0], [8.0, 3.0, 1.0]], [[3.0, 6.0, 
1.0], [8.0, 8.0, 1.0], [8.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 4.0], [2.0, 7.0, 4.0], [7.0, 8.0, 2.0]], 
[[7.0, 7.0, 6.0], [3.0, 2.0, 7.0], [2.0, 4.0, 5.0]], [[2.0, 1.0, 6.0], [5.0, 6.0, 5.0], [1.0, 2.0, 4.0]]], [[[3.0, 6.0, 5.0], 
[8.0, 6.0, 7.0], [6.0, 8.0, 2.0]], [[8.0, 6.0, 6.0], [2.0, 3.0, 3.0], [8.0, 3.0, 3.0]], [[6.0, 8.0, 7.0], [4.0, 2.0, 4.0], 
[5.0, 6.0, 5.0]]], [[[5.0, 8.0, 6.0], [1.0, 6.0, 4.0], [3.0, 4.0, 6.0]], [[6.0, 4.0, 6.0], [2.0, 7.0, 4.0], [3.0, 2.0, 5.0]], 
[[8.0, 1.0, 3.0], [3.0, 8.0, 8.0], [4.0, 5.0, 6.0]]], [[[8.0, 2.0, 8.0], [2.0, 3.0, 6.0], [3.0, 7.0, 6.0]], [[2.0, 2.0, 1.0], 
[2.0, 7.0, 8.0], [4.0, 2.0, 4.0]], [[2.0, 4.0, 1.0], [4.0, 8.0, 3.0], [7.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_ULUN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [8.0, 7.0, 3.0], [6.0, 2.0, 4.0]], [[5.0, 5.0, 4.0], [7.0, 3.0, 4.0], [7.0, 7.0, 
4.0]], [[6.0, 8.0, 8.0], [7.0, 5.0, 5.0], [8.0, 6.0, 2.0]]], [[[2.0, 6.0, 8.0], [3.0, 8.0, 1.0], [3.0, 4.0, 4.0]], [[6.0, 8.0, 
1.0], [7.0, 6.0, 8.0], [3.0, 1.0, 4.0]], [[2.0, 7.0, 1.0], [7.0, 8.0, 3.0], [2.0, 3.0, 4.0]]], [[[3.0, 1.0, 1.0], [5.0, 6.0, 
7.0], [4.0, 6.0, 5.0]], [[1.0, 5.0, 2.0], [1.0, 4.0, 2.0], [4.0, 3.0, 4.0]], [[5.0, 6.0, 8.0], [7.0, 5.0, 7.0], [5.0, 8.0, 
2.0]]], [[[8.0, 5.0, 8.0], [6.0, 2.0, 3.0], [1.0, 6.0, 2.0]], [[3.0, 7.0, 2.0], [8.0, 4.0, 8.0], [6.0, 4.0, 5.0]], [[5.0, 7.0, 
1.0], [3.0, 1.0, 4.0], [3.0, 2.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 2.0], [7.0, 4.0, 3.0], [6.0, 5.0, 1.0]], [[1.0, 5.0, 4.0], [6.0, 1.0, 
5.0], [7.0, 8.0, 2.0]], [[5.0, 8.0, 4.0], [8.0, 2.0, 8.0], [7.0, 5.0, 1.0]]], [[[2.0, 2.0, 5.0], [5.0, 2.0, 8.0], [1.0, 6.0, 
8.0]], [[2.0, 5.0, 5.0], [6.0, 6.0, 1.0], [5.0, 3.0, 2.0]], [[3.0, 8.0, 5.0], [7.0, 1.0, 3.0], [2.0, 4.0, 5.0]]], [[[1.0, 6.0, 
1.0], [6.0, 1.0, 8.0], [1.0, 3.0, 5.0]], [[3.0, 8.0, 3.0], [8.0, 2.0, 4.0], [5.0, 3.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 8.0, 
6.0], [3.0, 3.0, 1.0]]], [[[6.0, 6.0, 3.0], [2.0, 7.0, 7.0], [8.0, 2.0, 6.0]], [[3.0, 4.0, 3.0], [4.0, 3.0, 5.0], [5.0, 3.0, 
8.0]], [[3.0, 1.0, 4.0], [5.0, 3.0, 6.0], [7.0, 8.0, 6.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[4.0, 7.0, 5.0], [2.0, 5.0, 2.0]]], [[[6.0, 7.0, 7.0], [4.0, 2.0, 4.0]]], [[[2.0, 2.0, 7.0], 
[4.0, 4.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 8.0, 2.0], [6.0, 8.0, 5.0]]], [[[3.0, 2.0, 8.0], [6.0, 6.0, 5.0]]], 
[[[5.0, 3.0, 5.0], [2.0, 1.0, 4.0]]]])
      arg[:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 6.0], [8.0, 7.0, 3.0], [6.0, 2.0, 4.0]], [[5.0, 5.0, 4.0], [7.0, 3.0, 4.0], [7.0, 
7.0, 4.0]], [[4.0, 7.0, 5.0], [2.0, 5.0, 2.0], [8.0, 6.0, 2.0]]], [[[2.0, 6.0, 8.0], [3.0, 8.0, 1.0], [3.0, 4.0, 4.0]], [[6.0, 
8.0, 1.0], [7.0, 6.0, 8.0], [3.0, 1.0, 4.0]], [[6.0, 7.0, 7.0], [4.0, 2.0, 4.0], [2.0, 3.0, 4.0]]], [[[3.0, 1.0, 1.0], [5.0, 
6.0, 7.0], [4.0, 6.0, 5.0]], [[1.0, 5.0, 2.0], [1.0, 4.0, 2.0], [4.0, 3.0, 4.0]], [[2.0, 2.0, 7.0], [4.0, 4.0, 8.0], [5.0, 8.0, 
2.0]]], [[[8.0, 5.0, 8.0], [6.0, 2.0, 3.0], [1.0, 6.0, 2.0]], [[3.0, 7.0, 2.0], [8.0, 4.0, 8.0], [6.0, 4.0, 5.0]], [[5.0, 7.0, 
1.0], [3.0, 1.0, 4.0], [3.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 2.0], [7.0, 4.0, 3.0], [6.0, 5.0, 1.0]], 
[[1.0, 5.0, 4.0], [6.0, 1.0, 5.0], [7.0, 8.0, 2.0]], [[2.0, 8.0, 2.0], [6.0, 8.0, 5.0], [7.0, 5.0, 1.0]]], [[[2.0, 2.0, 5.0], 
[5.0, 2.0, 8.0], [1.0, 6.0, 8.0]], [[2.0, 5.0, 5.0], [6.0, 6.0, 1.0], [5.0, 3.0, 2.0]], [[3.0, 2.0, 8.0], [6.0, 6.0, 5.0], 
[2.0, 4.0, 5.0]]], [[[1.0, 6.0, 1.0], [6.0, 1.0, 8.0], [1.0, 3.0, 5.0]], [[3.0, 8.0, 3.0], [8.0, 2.0, 4.0], [5.0, 3.0, 7.0]], 
[[5.0, 3.0, 5.0], [2.0, 1.0, 4.0], [3.0, 3.0, 1.0]]], [[[6.0, 6.0, 3.0], [2.0, 7.0, 7.0], [8.0, 2.0, 6.0]], [[3.0, 4.0, 3.0], 
[4.0, 3.0, 5.0], [5.0, 3.0, 8.0]], [[3.0, 1.0, 4.0], [5.0, 3.0, 6.0], [7.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_array_CLRN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 2.0], [1.0, 4.0, 6.0], [4.0, 4.0, 2.0]], [[6.0, 3.0, 1.0], [5.0, 3.0, 3.0], [3.0, 6.0, 
2.0]], [[8.0, 5.0, 7.0], [7.0, 7.0, 2.0], [1.0, 8.0, 7.0]]], [[[4.0, 3.0, 6.0], [3.0, 6.0, 5.0], [5.0, 3.0, 7.0]], [[2.0, 1.0, 
7.0], [7.0, 2.0, 5.0], [2.0, 1.0, 6.0]], [[2.0, 6.0, 4.0], [2.0, 2.0, 4.0], [4.0, 2.0, 8.0]]], [[[6.0, 3.0, 2.0], [8.0, 7.0, 
2.0], [8.0, 2.0, 3.0]], [[4.0, 4.0, 7.0], [6.0, 7.0, 5.0], [2.0, 3.0, 3.0]], [[4.0, 6.0, 6.0], [7.0, 8.0, 5.0], [8.0, 5.0, 
6.0]]], [[[4.0, 8.0, 8.0], [7.0, 3.0, 2.0], [5.0, 3.0, 6.0]], [[8.0, 2.0, 3.0], [3.0, 7.0, 2.0], [5.0, 1.0, 4.0]], [[4.0, 2.0, 
5.0], [4.0, 3.0, 3.0], [5.0, 7.0, 4.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 8.0], [4.0, 6.0, 1.0], [4.0, 4.0, 2.0]], [[1.0, 2.0, 7.0], [3.0, 6.0, 
8.0], [4.0, 5.0, 3.0]], [[4.0, 4.0, 6.0], [2.0, 5.0, 5.0], [6.0, 5.0, 1.0]]], [[[6.0, 2.0, 8.0], [1.0, 3.0, 7.0], [8.0, 1.0, 
5.0]], [[1.0, 3.0, 7.0], [6.0, 2.0, 1.0], [3.0, 8.0, 8.0]], [[8.0, 4.0, 7.0], [3.0, 2.0, 1.0], [7.0, 3.0, 7.0]]], [[[5.0, 6.0, 
4.0], [4.0, 4.0, 1.0], [1.0, 3.0, 8.0]], [[3.0, 1.0, 3.0], [8.0, 7.0, 8.0], [6.0, 5.0, 8.0]], [[5.0, 5.0, 6.0], [3.0, 7.0, 
8.0], [1.0, 3.0, 3.0]]], [[[7.0, 2.0, 5.0], [8.0, 4.0, 5.0], [1.0, 2.0, 3.0]], [[8.0, 4.0, 5.0], [5.0, 5.0, 6.0], [4.0, 1.0, 
1.0]], [[5.0, 7.0, 2.0], [7.0, 4.0, 2.0], [7.0, 6.0, 4.0]]]]))
      val=numpy.array([[[[8.0, 5.0, 1.0]]], [[[2.0, 1.0, 7.0]]], [[[4.0, 4.0, 6.0]]], [[[3.0, 4.0, 6.0]]]])
      arg[:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 2.0], [1.0, 4.0, 6.0], [4.0, 4.0, 2.0]], [[6.0, 3.0, 1.0], [5.0, 3.0, 3.0], [3.0, 
6.0, 2.0]], [[8.0, 5.0, 7.0], [8.0, 5.0, 1.0], [1.0, 8.0, 7.0]]], [[[4.0, 3.0, 6.0], [3.0, 6.0, 5.0], [5.0, 3.0, 7.0]], [[2.0, 
1.0, 7.0], [7.0, 2.0, 5.0], [2.0, 1.0, 6.0]], [[2.0, 6.0, 4.0], [2.0, 1.0, 7.0], [4.0, 2.0, 8.0]]], [[[6.0, 3.0, 2.0], [8.0, 
7.0, 2.0], [8.0, 2.0, 3.0]], [[4.0, 4.0, 7.0], [6.0, 7.0, 5.0], [2.0, 3.0, 3.0]], [[4.0, 6.0, 6.0], [4.0, 4.0, 6.0], [8.0, 5.0, 
6.0]]], [[[4.0, 8.0, 8.0], [7.0, 3.0, 2.0], [5.0, 3.0, 6.0]], [[8.0, 2.0, 3.0], [3.0, 7.0, 2.0], [5.0, 1.0, 4.0]], [[4.0, 2.0, 
5.0], [3.0, 4.0, 6.0], [5.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 8.0], [4.0, 6.0, 1.0], [4.0, 4.0, 2.0]], 
[[1.0, 2.0, 7.0], [3.0, 6.0, 8.0], [4.0, 5.0, 3.0]], [[4.0, 4.0, 6.0], [8.0, 5.0, 1.0], [6.0, 5.0, 1.0]]], [[[6.0, 2.0, 8.0], 
[1.0, 3.0, 7.0], [8.0, 1.0, 5.0]], [[1.0, 3.0, 7.0], [6.0, 2.0, 1.0], [3.0, 8.0, 8.0]], [[8.0, 4.0, 7.0], [2.0, 1.0, 7.0], 
[7.0, 3.0, 7.0]]], [[[5.0, 6.0, 4.0], [4.0, 4.0, 1.0], [1.0, 3.0, 8.0]], [[3.0, 1.0, 3.0], [8.0, 7.0, 8.0], [6.0, 5.0, 8.0]], 
[[5.0, 5.0, 6.0], [4.0, 4.0, 6.0], [1.0, 3.0, 3.0]]], [[[7.0, 2.0, 5.0], [8.0, 4.0, 5.0], [1.0, 2.0, 3.0]], [[8.0, 4.0, 5.0], 
[5.0, 5.0, 6.0], [4.0, 1.0, 1.0]], [[5.0, 7.0, 2.0], [3.0, 4.0, 6.0], [7.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_constData_CLRN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 4.0], [2.0, 4.0, 8.0], [8.0, 2.0, 1.0]], [[3.0, 3.0, 7.0], [4.0, 2.0, 6.0], [2.0, 4.0, 
8.0]], [[6.0, 2.0, 8.0], [8.0, 8.0, 1.0], [6.0, 2.0, 6.0]]], [[[8.0, 2.0, 6.0], [1.0, 1.0, 1.0], [5.0, 5.0, 8.0]], [[7.0, 6.0, 
6.0], [2.0, 3.0, 1.0], [4.0, 1.0, 6.0]], [[3.0, 1.0, 4.0], [4.0, 8.0, 8.0], [1.0, 7.0, 1.0]]], [[[2.0, 8.0, 5.0], [2.0, 3.0, 
5.0], [7.0, 7.0, 1.0]], [[6.0, 5.0, 3.0], [6.0, 4.0, 8.0], [2.0, 8.0, 3.0]], [[1.0, 8.0, 3.0], [1.0, 1.0, 3.0], [7.0, 3.0, 
2.0]]], [[[6.0, 1.0, 6.0], [7.0, 4.0, 4.0], [4.0, 3.0, 8.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 2.0], [4.0, 2.0, 1.0]], [[2.0, 1.0, 
4.0], [8.0, 1.0, 4.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 4.0], [1.0, 4.0, 8.0], [6.0, 1.0, 1.0]], [[8.0, 1.0, 5.0], [8.0, 3.0, 
5.0], [4.0, 7.0, 2.0]], [[1.0, 1.0, 3.0], [5.0, 6.0, 2.0], [3.0, 6.0, 6.0]]], [[[6.0, 1.0, 4.0], [5.0, 5.0, 3.0], [7.0, 3.0, 
2.0]], [[4.0, 4.0, 4.0], [7.0, 1.0, 7.0], [2.0, 1.0, 1.0]], [[8.0, 7.0, 3.0], [1.0, 2.0, 2.0], [6.0, 6.0, 5.0]]], [[[2.0, 7.0, 
7.0], [7.0, 6.0, 3.0], [5.0, 4.0, 2.0]], [[3.0, 5.0, 8.0], [4.0, 7.0, 5.0], [7.0, 3.0, 4.0]], [[4.0, 4.0, 6.0], [7.0, 6.0, 
8.0], [1.0, 8.0, 3.0]]], [[[6.0, 2.0, 4.0], [5.0, 8.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 8.0, 5.0], [8.0, 1.0, 
7.0]], [[1.0, 8.0, 8.0], [2.0, 8.0, 4.0], [8.0, 3.0, 3.0]]]]))
      val=Data(numpy.array([[[[6.0, 8.0, 3.0]]], [[[6.0, 6.0, 5.0]]], [[[6.0, 1.0, 2.0]]], [[[8.0, 4.0, 
6.0]]]]),self.functionspace)
      arg[:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 4.0], [2.0, 4.0, 8.0], [8.0, 2.0, 1.0]], [[3.0, 3.0, 7.0], [4.0, 2.0, 6.0], [2.0, 
4.0, 8.0]], [[6.0, 2.0, 8.0], [6.0, 8.0, 3.0], [6.0, 2.0, 6.0]]], [[[8.0, 2.0, 6.0], [1.0, 1.0, 1.0], [5.0, 5.0, 8.0]], [[7.0, 
6.0, 6.0], [2.0, 3.0, 1.0], [4.0, 1.0, 6.0]], [[3.0, 1.0, 4.0], [6.0, 6.0, 5.0], [1.0, 7.0, 1.0]]], [[[2.0, 8.0, 5.0], [2.0, 
3.0, 5.0], [7.0, 7.0, 1.0]], [[6.0, 5.0, 3.0], [6.0, 4.0, 8.0], [2.0, 8.0, 3.0]], [[1.0, 8.0, 3.0], [6.0, 1.0, 2.0], [7.0, 3.0, 
2.0]]], [[[6.0, 1.0, 6.0], [7.0, 4.0, 4.0], [4.0, 3.0, 8.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 2.0], [4.0, 2.0, 1.0]], [[2.0, 1.0, 
4.0], [8.0, 4.0, 6.0], [3.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 4.0], [1.0, 4.0, 8.0], [6.0, 1.0, 1.0]], 
[[8.0, 1.0, 5.0], [8.0, 3.0, 5.0], [4.0, 7.0, 2.0]], [[1.0, 1.0, 3.0], [6.0, 8.0, 3.0], [3.0, 6.0, 6.0]]], [[[6.0, 1.0, 4.0], 
[5.0, 5.0, 3.0], [7.0, 3.0, 2.0]], [[4.0, 4.0, 4.0], [7.0, 1.0, 7.0], [2.0, 1.0, 1.0]], [[8.0, 7.0, 3.0], [6.0, 6.0, 5.0], 
[6.0, 6.0, 5.0]]], [[[2.0, 7.0, 7.0], [7.0, 6.0, 3.0], [5.0, 4.0, 2.0]], [[3.0, 5.0, 8.0], [4.0, 7.0, 5.0], [7.0, 3.0, 4.0]], 
[[4.0, 4.0, 6.0], [6.0, 1.0, 2.0], [1.0, 8.0, 3.0]]], [[[6.0, 2.0, 4.0], [5.0, 8.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 6.0, 6.0], 
[2.0, 8.0, 5.0], [8.0, 1.0, 7.0]], [[1.0, 8.0, 8.0], [8.0, 4.0, 6.0], [8.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_taggedData_CLRN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 8.0], [4.0, 5.0, 1.0], [4.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], [8.0, 2.0, 8.0], [7.0, 7.0, 
4.0]], [[2.0, 4.0, 5.0], [8.0, 6.0, 5.0], [3.0, 5.0, 5.0]]], [[[3.0, 6.0, 8.0], [1.0, 8.0, 3.0], [5.0, 1.0, 2.0]], [[3.0, 3.0, 
3.0], [6.0, 2.0, 7.0], [6.0, 5.0, 3.0]], [[5.0, 1.0, 1.0], [5.0, 3.0, 7.0], [1.0, 1.0, 3.0]]], [[[4.0, 1.0, 7.0], [6.0, 7.0, 
5.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 5.0], [3.0, 4.0, 8.0], [3.0, 8.0, 2.0]], [[8.0, 6.0, 5.0], [5.0, 8.0, 1.0], [3.0, 4.0, 
5.0]]], [[[7.0, 6.0, 5.0], [7.0, 7.0, 7.0], [5.0, 3.0, 5.0]], [[5.0, 5.0, 8.0], [8.0, 2.0, 6.0], [7.0, 8.0, 4.0]], [[3.0, 6.0, 
1.0], [1.0, 6.0, 4.0], [8.0, 2.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 2.0], [6.0, 8.0, 1.0], [6.0, 5.0, 6.0]], [[2.0, 8.0, 4.0], [4.0, 7.0, 
2.0], [6.0, 2.0, 2.0]], [[4.0, 7.0, 3.0], [8.0, 5.0, 3.0], [5.0, 8.0, 5.0]]], [[[7.0, 7.0, 1.0], [4.0, 3.0, 7.0], [7.0, 2.0, 
7.0]], [[8.0, 5.0, 1.0], [5.0, 1.0, 1.0], [5.0, 2.0, 2.0]], [[5.0, 3.0, 2.0], [7.0, 6.0, 5.0], [6.0, 3.0, 5.0]]], [[[8.0, 5.0, 
4.0], [5.0, 6.0, 2.0], [8.0, 8.0, 5.0]], [[1.0, 1.0, 6.0], [7.0, 5.0, 3.0], [6.0, 7.0, 4.0]], [[6.0, 5.0, 2.0], [2.0, 1.0, 
8.0], [8.0, 6.0, 3.0]]], [[[4.0, 5.0, 2.0], [3.0, 2.0, 3.0], [3.0, 6.0, 4.0]], [[8.0, 2.0, 3.0], [7.0, 4.0, 2.0], [7.0, 1.0, 
8.0]], [[8.0, 2.0, 1.0], [1.0, 2.0, 4.0], [8.0, 5.0, 1.0]]]]))
      val=Data(numpy.array([[[[7.0, 7.0, 5.0]]], [[[6.0, 2.0, 6.0]]], [[[7.0, 2.0, 1.0]]], [[[2.0, 3.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 8.0]]], [[[1.0, 1.0, 4.0]]], [[[7.0, 3.0, 4.0]]], [[[6.0, 5.0, 6.0]]]]))
      arg[:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 8.0], [4.0, 5.0, 1.0], [4.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], [8.0, 2.0, 8.0], [7.0, 
7.0, 4.0]], [[2.0, 4.0, 5.0], [7.0, 7.0, 5.0], [3.0, 5.0, 5.0]]], [[[3.0, 6.0, 8.0], [1.0, 8.0, 3.0], [5.0, 1.0, 2.0]], [[3.0, 
3.0, 3.0], [6.0, 2.0, 7.0], [6.0, 5.0, 3.0]], [[5.0, 1.0, 1.0], [6.0, 2.0, 6.0], [1.0, 1.0, 3.0]]], [[[4.0, 1.0, 7.0], [6.0, 
7.0, 5.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 5.0], [3.0, 4.0, 8.0], [3.0, 8.0, 2.0]], [[8.0, 6.0, 5.0], [7.0, 2.0, 1.0], [3.0, 4.0, 
5.0]]], [[[7.0, 6.0, 5.0], [7.0, 7.0, 7.0], [5.0, 3.0, 5.0]], [[5.0, 5.0, 8.0], [8.0, 2.0, 6.0], [7.0, 8.0, 4.0]], [[3.0, 6.0, 
1.0], [2.0, 3.0, 2.0], [8.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 2.0], [6.0, 8.0, 1.0], [6.0, 5.0, 6.0]], 
[[2.0, 8.0, 4.0], [4.0, 7.0, 2.0], [6.0, 2.0, 2.0]], [[4.0, 7.0, 3.0], [5.0, 5.0, 8.0], [5.0, 8.0, 5.0]]], [[[7.0, 7.0, 1.0], 
[4.0, 3.0, 7.0], [7.0, 2.0, 7.0]], [[8.0, 5.0, 1.0], [5.0, 1.0, 1.0], [5.0, 2.0, 2.0]], [[5.0, 3.0, 2.0], [1.0, 1.0, 4.0], 
[6.0, 3.0, 5.0]]], [[[8.0, 5.0, 4.0], [5.0, 6.0, 2.0], [8.0, 8.0, 5.0]], [[1.0, 1.0, 6.0], [7.0, 5.0, 3.0], [6.0, 7.0, 4.0]], 
[[6.0, 5.0, 2.0], [7.0, 3.0, 4.0], [8.0, 6.0, 3.0]]], [[[4.0, 5.0, 2.0], [3.0, 2.0, 3.0], [3.0, 6.0, 4.0]], [[8.0, 2.0, 3.0], 
[7.0, 4.0, 2.0], [7.0, 1.0, 8.0]], [[8.0, 2.0, 1.0], [6.0, 5.0, 6.0], [8.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_taggedData_rank4_expandedData_CLRN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 2.0], [7.0, 3.0, 5.0], [2.0, 6.0, 5.0]], [[4.0, 8.0, 8.0], [7.0, 4.0, 6.0], [3.0, 5.0, 
7.0]], [[2.0, 4.0, 6.0], [6.0, 1.0, 5.0], [5.0, 6.0, 4.0]]], [[[4.0, 1.0, 2.0], [5.0, 4.0, 3.0], [8.0, 5.0, 8.0]], [[5.0, 7.0, 
6.0], [3.0, 3.0, 3.0], [1.0, 4.0, 6.0]], [[8.0, 3.0, 8.0], [8.0, 5.0, 7.0], [2.0, 3.0, 5.0]]], [[[2.0, 5.0, 4.0], [4.0, 3.0, 
4.0], [4.0, 2.0, 5.0]], [[2.0, 4.0, 7.0], [2.0, 6.0, 7.0], [5.0, 7.0, 1.0]], [[2.0, 2.0, 4.0], [8.0, 3.0, 8.0], [5.0, 2.0, 
2.0]]], [[[5.0, 1.0, 4.0], [2.0, 7.0, 2.0], [6.0, 1.0, 8.0]], [[6.0, 4.0, 6.0], [4.0, 1.0, 3.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 
4.0], [3.0, 4.0, 5.0], [5.0, 5.0, 5.0]]]]),self.functionspace)
      arg.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 2.0], [3.0, 7.0, 8.0], [8.0, 2.0, 2.0]], [[6.0, 2.0, 2.0], [5.0, 7.0, 
5.0], [7.0, 2.0, 7.0]], [[3.0, 2.0, 4.0], [1.0, 3.0, 5.0], [1.0, 5.0, 7.0]]], [[[2.0, 6.0, 3.0], [7.0, 2.0, 1.0], [3.0, 5.0, 
6.0]], [[7.0, 8.0, 8.0], [6.0, 6.0, 6.0], [7.0, 1.0, 4.0]], [[6.0, 3.0, 7.0], [5.0, 7.0, 3.0], [8.0, 4.0, 8.0]]], [[[3.0, 2.0, 
5.0], [1.0, 6.0, 7.0], [4.0, 1.0, 7.0]], [[6.0, 6.0, 7.0], [2.0, 3.0, 6.0], [6.0, 2.0, 1.0]], [[4.0, 1.0, 5.0], [3.0, 4.0, 
2.0], [8.0, 2.0, 5.0]]], [[[2.0, 8.0, 8.0], [3.0, 4.0, 7.0], [1.0, 5.0, 3.0]], [[2.0, 4.0, 6.0], [4.0, 7.0, 4.0], [2.0, 7.0, 
6.0]], [[3.0, 3.0, 2.0], [3.0, 5.0, 8.0], [6.0, 2.0, 3.0]]]]))
      msk_val=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      val=msk_val*numpy.array([[[[5.0, 1.0, 6.0]]], [[[2.0, 2.0, 4.0]]], [[[1.0, 1.0, 2.0]]], [[[5.0, 4.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 6.0, 1.0]]], [[[1.0, 1.0, 8.0]]], [[[2.0, 3.0, 3.0]]], [[[6.0, 4.0, 8.0]]]])
      arg[:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [7.0, 3.0, 5.0], [2.0, 6.0, 5.0]], [[4.0, 8.0, 8.0], [7.0, 4.0, 6.0], [3.0, 
5.0, 7.0]], [[2.0, 4.0, 6.0], [5.0, 1.0, 6.0], [5.0, 6.0, 4.0]]], [[[4.0, 1.0, 2.0], [5.0, 4.0, 3.0], [8.0, 5.0, 8.0]], [[5.0, 
7.0, 6.0], [3.0, 3.0, 3.0], [1.0, 4.0, 6.0]], [[8.0, 3.0, 8.0], [2.0, 2.0, 4.0], [2.0, 3.0, 5.0]]], [[[2.0, 5.0, 4.0], [4.0, 
3.0, 4.0], [4.0, 2.0, 5.0]], [[2.0, 4.0, 7.0], [2.0, 6.0, 7.0], [5.0, 7.0, 1.0]], [[2.0, 2.0, 4.0], [1.0, 1.0, 2.0], [5.0, 2.0, 
2.0]]], [[[5.0, 1.0, 4.0], [2.0, 7.0, 2.0], [6.0, 1.0, 8.0]], [[6.0, 4.0, 6.0], [4.0, 1.0, 3.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 
4.0], [5.0, 4.0, 6.0], [5.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 2.0], [3.0, 7.0, 8.0], [8.0, 2.0, 2.0]], 
[[6.0, 2.0, 2.0], [5.0, 7.0, 5.0], [7.0, 2.0, 7.0]], [[3.0, 2.0, 4.0], [4.0, 6.0, 1.0], [1.0, 5.0, 7.0]]], [[[2.0, 6.0, 3.0], 
[7.0, 2.0, 1.0], [3.0, 5.0, 6.0]], [[7.0, 8.0, 8.0], [6.0, 6.0, 6.0], [7.0, 1.0, 4.0]], [[6.0, 3.0, 7.0], [1.0, 1.0, 8.0], 
[8.0, 4.0, 8.0]]], [[[3.0, 2.0, 5.0], [1.0, 6.0, 7.0], [4.0, 1.0, 7.0]], [[6.0, 6.0, 7.0], [2.0, 3.0, 6.0], [6.0, 2.0, 1.0]], 
[[4.0, 1.0, 5.0], [2.0, 3.0, 3.0], [8.0, 2.0, 5.0]]], [[[2.0, 8.0, 8.0], [3.0, 4.0, 7.0], [1.0, 5.0, 3.0]], [[2.0, 4.0, 6.0], 
[4.0, 7.0, 4.0], [2.0, 7.0, 6.0]], [[3.0, 3.0, 2.0], [6.0, 4.0, 8.0], [6.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_taggedData_R(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([5.0, 8.0, 7.0])+(1.-msk_arg)*numpy.array([2.0, 8.0, 2.0])
      val=Data(numpy.array([5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([5.0]))
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([5.0, 5.0, 7.0])+(1.-msk_ref)*numpy.array([2.0, 5.0, 2.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_taggedData_C(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([7.0, 3.0, 8.0])+(1.-msk_arg)*numpy.array([5.0, 2.0, 7.0])
      val=Data(numpy.array([5.0, 7.0, 7.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0, 5.0, 7.0]))
      arg[:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([5.0, 7.0, 7.0])+(1.-msk_ref)*numpy.array([8.0, 5.0, 7.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_taggedData_P(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([3.0, 1.0, 2.0])+(1.-msk_arg)*numpy.array([4.0, 8.0, 2.0])
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 1.0, 2.0])+(1.-msk_ref)*numpy.array([4.0, 1.0, 2.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_taggedData_U(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([3.0, 1.0, 3.0])+(1.-msk_arg)*numpy.array([6.0, 7.0, 8.0])
      val=Data(numpy.array([3.0, 2.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([5.0, 7.0]))
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 2.0, 3.0])+(1.-msk_ref)*numpy.array([5.0, 7.0, 8.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_taggedData_L(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([8.0, 4.0, 3.0])+(1.-msk_arg)*numpy.array([2.0, 8.0, 5.0])
      val=Data(numpy.array([8.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([5.0]))
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([8.0, 4.0, 8.0])+(1.-msk_ref)*numpy.array([2.0, 8.0, 5.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_PP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[5.0, 2.0, 5.0, 1.0], [6.0, 2.0, 6.0, 3.0], [4.0, 7.0, 2.0, 
2.0]])+(1.-msk_arg)*numpy.array([[4.0, 5.0, 4.0, 6.0], [8.0, 5.0, 3.0, 2.0], [6.0, 6.0, 8.0, 8.0]])
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 2.0, 5.0, 1.0], [6.0, 2.0, 1.0, 3.0], [4.0, 7.0, 2.0, 
2.0]])+(1.-msk_ref)*numpy.array([[4.0, 5.0, 4.0, 6.0], [8.0, 5.0, 1.0, 2.0], [6.0, 6.0, 8.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_RL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[8.0, 5.0, 4.0, 7.0], [3.0, 4.0, 7.0, 8.0], [7.0, 8.0, 3.0, 
3.0]])+(1.-msk_arg)*numpy.array([[5.0, 1.0, 6.0, 5.0], [5.0, 2.0, 7.0, 5.0], [7.0, 2.0, 5.0, 7.0]])
      val=Data(numpy.array([[2.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 2.0]]))
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 5.0, 4.0, 7.0], [3.0, 4.0, 2.0, 2.0], [7.0, 8.0, 3.0, 
3.0]])+(1.-msk_ref)*numpy.array([[5.0, 1.0, 6.0, 5.0], [5.0, 2.0, 1.0, 2.0], [7.0, 2.0, 5.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_CP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[5.0, 1.0, 1.0, 8.0], [5.0, 2.0, 8.0, 4.0], [2.0, 2.0, 7.0, 
4.0]])+(1.-msk_arg)*numpy.array([[1.0, 3.0, 8.0, 7.0], [4.0, 2.0, 6.0, 3.0], [3.0, 2.0, 8.0, 1.0]])
      val=Data(numpy.array([6.0, 5.0, 4.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([7.0, 3.0, 7.0]))
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 1.0, 6.0, 8.0], [5.0, 2.0, 5.0, 4.0], [2.0, 2.0, 4.0, 
4.0]])+(1.-msk_ref)*numpy.array([[1.0, 3.0, 7.0, 7.0], [4.0, 2.0, 3.0, 3.0], [3.0, 2.0, 7.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_PR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[5.0, 3.0, 7.0, 1.0], [7.0, 3.0, 6.0, 5.0], [4.0, 1.0, 2.0, 
6.0]])+(1.-msk_arg)*numpy.array([[7.0, 1.0, 7.0, 4.0], [1.0, 8.0, 5.0, 1.0], [8.0, 4.0, 3.0, 2.0]])
      val=Data(numpy.array([1.0, 6.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 4.0]))
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 3.0, 7.0, 1.0], [7.0, 1.0, 6.0, 5.0], [4.0, 1.0, 2.0, 
6.0]])+(1.-msk_ref)*numpy.array([[7.0, 1.0, 7.0, 4.0], [1.0, 3.0, 4.0, 1.0], [8.0, 4.0, 3.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_LP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[7.0, 2.0, 7.0, 8.0], [1.0, 4.0, 3.0, 5.0], [7.0, 6.0, 3.0, 
8.0]])+(1.-msk_arg)*numpy.array([[4.0, 6.0, 5.0, 7.0], [1.0, 4.0, 5.0, 3.0], [2.0, 4.0, 1.0, 7.0]])
      val=Data(numpy.array([6.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([1.0]))
      arg[2:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 2.0, 7.0, 8.0], [1.0, 4.0, 3.0, 5.0], [7.0, 6.0, 6.0, 
8.0]])+(1.-msk_ref)*numpy.array([[4.0, 6.0, 5.0, 7.0], [1.0, 4.0, 5.0, 3.0], [2.0, 4.0, 1.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_UU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[8.0, 7.0, 1.0, 4.0], [1.0, 2.0, 5.0, 4.0], [6.0, 2.0, 6.0, 
4.0]])+(1.-msk_arg)*numpy.array([[2.0, 3.0, 5.0, 8.0], [7.0, 2.0, 4.0, 3.0], [3.0, 8.0, 5.0, 3.0]])
      val=Data(numpy.array([[5.0, 7.0, 5.0], [4.0, 1.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 4.0, 7.0], [2.0, 7.0, 1.0]]))
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 7.0, 5.0, 4.0], [4.0, 1.0, 3.0, 4.0], [6.0, 2.0, 6.0, 
4.0]])+(1.-msk_ref)*numpy.array([[2.0, 4.0, 7.0, 8.0], [2.0, 7.0, 1.0, 3.0], [3.0, 8.0, 5.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_PU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[1.0, 2.0, 2.0, 1.0], [4.0, 4.0, 3.0, 8.0], [2.0, 5.0, 1.0, 
4.0]])+(1.-msk_arg)*numpy.array([[7.0, 7.0, 5.0, 4.0], [5.0, 5.0, 6.0, 7.0], [7.0, 6.0, 3.0, 5.0]])
      val=Data(numpy.array([1.0, 8.0, 3.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 3.0, 1.0]))
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 2.0, 2.0, 1.0], [1.0, 8.0, 3.0, 8.0], [2.0, 5.0, 1.0, 
4.0]])+(1.-msk_ref)*numpy.array([[7.0, 7.0, 5.0, 4.0], [3.0, 3.0, 1.0, 7.0], [7.0, 6.0, 3.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_RN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[4.0, 2.0, 2.0, 5.0], [5.0, 7.0, 5.0, 4.0], [4.0, 4.0, 3.0, 
6.0]])+(1.-msk_arg)*numpy.array([[4.0, 6.0, 7.0, 6.0], [7.0, 6.0, 3.0, 6.0], [2.0, 2.0, 3.0, 5.0]])
      val=Data(numpy.array([[3.0, 7.0, 8.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 2.0, 4.0, 1.0]]))
      arg[1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 2.0, 2.0, 5.0], [3.0, 7.0, 8.0, 6.0], [4.0, 4.0, 3.0, 
6.0]])+(1.-msk_ref)*numpy.array([[4.0, 6.0, 7.0, 6.0], [1.0, 2.0, 4.0, 1.0], [2.0, 2.0, 3.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_UL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[7.0, 1.0, 3.0, 8.0], [3.0, 1.0, 5.0, 1.0], [7.0, 2.0, 3.0, 
8.0]])+(1.-msk_arg)*numpy.array([[1.0, 7.0, 8.0, 7.0], [3.0, 8.0, 5.0, 8.0], [2.0, 5.0, 2.0, 6.0]])
      val=Data(numpy.array([[6.0, 1.0], [3.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 1.0], [3.0, 5.0]]))
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 1.0, 6.0, 1.0], [3.0, 1.0, 3.0, 4.0], [7.0, 2.0, 3.0, 
8.0]])+(1.-msk_ref)*numpy.array([[1.0, 7.0, 4.0, 1.0], [3.0, 8.0, 3.0, 5.0], [2.0, 5.0, 2.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_PN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[7.0, 6.0, 6.0, 4.0], [6.0, 1.0, 2.0, 7.0], [5.0, 2.0, 1.0, 
6.0]])+(1.-msk_arg)*numpy.array([[4.0, 5.0, 2.0, 5.0], [5.0, 6.0, 8.0, 4.0], [4.0, 5.0, 3.0, 3.0]])
      val=Data(numpy.array([5.0, 7.0, 6.0, 8.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([6.0, 2.0, 6.0, 1.0]))
      arg[1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 6.0, 6.0, 4.0], [5.0, 7.0, 6.0, 8.0], [5.0, 2.0, 1.0, 
6.0]])+(1.-msk_ref)*numpy.array([[4.0, 5.0, 2.0, 5.0], [6.0, 2.0, 6.0, 1.0], [4.0, 5.0, 3.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_CC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[4.0, 5.0, 8.0, 8.0], [1.0, 4.0, 6.0, 2.0], [6.0, 1.0, 4.0, 
7.0]])+(1.-msk_arg)*numpy.array([[7.0, 4.0, 7.0, 3.0], [3.0, 3.0, 4.0, 3.0], [6.0, 7.0, 5.0, 7.0]])
      val=Data(numpy.array([[3.0, 8.0, 4.0, 2.0], [2.0, 1.0, 5.0, 1.0], [8.0, 4.0, 3.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 8.0, 4.0, 6.0], [1.0, 8.0, 8.0, 7.0], [3.0, 1.0, 3.0, 6.0]]))
      arg[:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 8.0, 4.0, 2.0], [2.0, 1.0, 5.0, 1.0], [8.0, 4.0, 3.0, 
2.0]])+(1.-msk_ref)*numpy.array([[6.0, 8.0, 4.0, 6.0], [1.0, 8.0, 8.0, 7.0], [3.0, 1.0, 3.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_CL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[2.0, 7.0, 6.0, 2.0], [8.0, 4.0, 5.0, 8.0], [2.0, 5.0, 6.0, 
8.0]])+(1.-msk_arg)*numpy.array([[5.0, 7.0, 2.0, 4.0], [7.0, 4.0, 1.0, 6.0], [1.0, 2.0, 8.0, 3.0]])
      val=Data(numpy.array([[7.0, 2.0], [4.0, 1.0], [6.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 8.0], [5.0, 8.0], [7.0, 4.0]]))
      arg[:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 7.0, 7.0, 2.0], [8.0, 4.0, 4.0, 1.0], [2.0, 5.0, 6.0, 
2.0]])+(1.-msk_ref)*numpy.array([[5.0, 7.0, 7.0, 8.0], [7.0, 4.0, 5.0, 8.0], [1.0, 2.0, 7.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_CU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[6.0, 8.0, 4.0, 8.0], [2.0, 5.0, 5.0, 6.0], [1.0, 3.0, 6.0, 
4.0]])+(1.-msk_arg)*numpy.array([[6.0, 7.0, 4.0, 2.0], [3.0, 4.0, 1.0, 1.0], [7.0, 6.0, 7.0, 6.0]])
      val=Data(numpy.array([[2.0, 3.0, 8.0], [5.0, 2.0, 4.0], [5.0, 7.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 7.0, 8.0], [8.0, 7.0, 4.0], [3.0, 2.0, 7.0]]))
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, 3.0, 8.0, 8.0], [5.0, 2.0, 4.0, 6.0], [5.0, 7.0, 6.0, 
4.0]])+(1.-msk_ref)*numpy.array([[8.0, 7.0, 8.0, 2.0], [8.0, 7.0, 4.0, 1.0], [3.0, 2.0, 7.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_LN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[1.0, 4.0, 1.0, 7.0], [1.0, 2.0, 5.0, 3.0], [4.0, 7.0, 5.0, 
5.0]])+(1.-msk_arg)*numpy.array([[2.0, 8.0, 6.0, 2.0], [8.0, 3.0, 4.0, 5.0], [2.0, 6.0, 6.0, 1.0]])
      val=Data(numpy.array([[8.0, 6.0, 5.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 8.0, 7.0, 2.0]]))
      arg[2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 4.0, 1.0, 7.0], [1.0, 2.0, 5.0, 3.0], [8.0, 6.0, 5.0, 
7.0]])+(1.-msk_ref)*numpy.array([[2.0, 8.0, 6.0, 2.0], [8.0, 3.0, 4.0, 5.0], [7.0, 8.0, 7.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_RR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[3.0, 5.0, 3.0, 6.0], [5.0, 2.0, 6.0, 7.0], [5.0, 6.0, 1.0, 
2.0]])+(1.-msk_arg)*numpy.array([[7.0, 3.0, 4.0, 7.0], [5.0, 6.0, 8.0, 7.0], [4.0, 4.0, 6.0, 6.0]])
      val=Data(numpy.array([[1.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 3.0]]))
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 5.0, 3.0, 6.0], [5.0, 1.0, 8.0, 7.0], [5.0, 6.0, 1.0, 
2.0]])+(1.-msk_ref)*numpy.array([[7.0, 3.0, 4.0, 7.0], [5.0, 6.0, 3.0, 7.0], [4.0, 4.0, 6.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_LL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[3.0, 6.0, 6.0, 3.0], [2.0, 3.0, 1.0, 4.0], [5.0, 8.0, 1.0, 
8.0]])+(1.-msk_arg)*numpy.array([[5.0, 1.0, 8.0, 2.0], [1.0, 4.0, 6.0, 6.0], [3.0, 5.0, 6.0, 5.0]])
      val=Data(numpy.array([[2.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 2.0]]))
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 6.0, 6.0, 3.0], [2.0, 3.0, 1.0, 4.0], [5.0, 8.0, 2.0, 
8.0]])+(1.-msk_ref)*numpy.array([[5.0, 1.0, 8.0, 2.0], [1.0, 4.0, 6.0, 6.0], [3.0, 5.0, 6.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_UR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[7.0, 8.0, 6.0, 1.0], [5.0, 2.0, 1.0, 1.0], [8.0, 3.0, 3.0, 
2.0]])+(1.-msk_arg)*numpy.array([[8.0, 6.0, 6.0, 5.0], [3.0, 8.0, 2.0, 5.0], [5.0, 2.0, 8.0, 5.0]])
      val=Data(numpy.array([[5.0, 1.0], [1.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 4.0], [4.0, 3.0]]))
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 5.0, 1.0, 1.0], [5.0, 1.0, 7.0, 1.0], [8.0, 3.0, 3.0, 
2.0]])+(1.-msk_ref)*numpy.array([[8.0, 5.0, 4.0, 5.0], [3.0, 4.0, 3.0, 5.0], [5.0, 2.0, 8.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_CR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[7.0, 7.0, 8.0, 1.0], [6.0, 4.0, 4.0, 7.0], [7.0, 2.0, 2.0, 
5.0]])+(1.-msk_arg)*numpy.array([[7.0, 3.0, 2.0, 7.0], [3.0, 3.0, 2.0, 2.0], [7.0, 4.0, 1.0, 6.0]])
      val=Data(numpy.array([[8.0, 6.0], [3.0, 1.0], [4.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 3.0], [4.0, 4.0], [3.0, 5.0]]))
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 8.0, 6.0, 1.0], [6.0, 3.0, 1.0, 7.0], [7.0, 4.0, 3.0, 
5.0]])+(1.-msk_ref)*numpy.array([[7.0, 1.0, 3.0, 7.0], [3.0, 4.0, 4.0, 2.0], [7.0, 3.0, 5.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_CN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[1.0, 7.0, 1.0, 2.0], [1.0, 6.0, 5.0, 2.0], [8.0, 2.0, 2.0, 
7.0]])+(1.-msk_arg)*numpy.array([[2.0, 1.0, 4.0, 1.0], [2.0, 8.0, 2.0, 1.0], [1.0, 5.0, 5.0, 7.0]])
      val=Data(numpy.array([[1.0, 4.0, 4.0, 5.0], [2.0, 1.0, 2.0, 3.0], [7.0, 4.0, 7.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 6.0, 4.0, 1.0], [5.0, 7.0, 1.0, 2.0], [4.0, 1.0, 4.0, 4.0]]))
      arg[:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 4.0, 4.0, 5.0], [2.0, 1.0, 2.0, 3.0], [7.0, 4.0, 7.0, 
5.0]])+(1.-msk_ref)*numpy.array([[7.0, 6.0, 4.0, 1.0], [5.0, 7.0, 1.0, 2.0], [4.0, 1.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_taggedData_UN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[7.0, 1.0, 6.0, 8.0], [3.0, 8.0, 3.0, 2.0], [4.0, 3.0, 3.0, 
1.0]])+(1.-msk_arg)*numpy.array([[7.0, 8.0, 7.0, 8.0], [1.0, 6.0, 8.0, 8.0], [4.0, 5.0, 7.0, 5.0]])
      val=Data(numpy.array([[3.0, 3.0, 3.0, 2.0], [8.0, 6.0, 6.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 4.0, 1.0, 8.0], [5.0, 2.0, 4.0, 1.0]]))
      arg[:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 3.0, 3.0, 2.0], [8.0, 6.0, 6.0, 7.0], [4.0, 3.0, 3.0, 
1.0]])+(1.-msk_ref)*numpy.array([[3.0, 4.0, 1.0, 8.0], [5.0, 2.0, 4.0, 1.0], [4.0, 5.0, 7.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[4.0, 6.0, 3.0], [3.0, 8.0, 4.0], [4.0, 2.0, 5.0], [3.0, 3.0, 1.0]], [[5.0, 3.0, 3.0], [4.0, 
2.0, 8.0], [4.0, 3.0, 8.0], [3.0, 2.0, 5.0]], [[7.0, 1.0, 6.0], [6.0, 4.0, 2.0], [8.0, 5.0, 7.0], [8.0, 1.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 4.0, 8.0], [4.0, 7.0, 2.0], [5.0, 6.0, 6.0], [2.0, 7.0, 6.0]], [[8.0, 2.0, 2.0], [7.0, 
7.0, 3.0], [7.0, 6.0, 1.0], [5.0, 4.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 6.0, 5.0], [8.0, 1.0, 5.0], [1.0, 3.0, 3.0]]])
      val=Data(numpy.array([[[4.0, 8.0, 3.0], [6.0, 4.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 2.0, 6.0], [4.0, 4.0, 1.0]]]))
      arg[1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 3.0], [3.0, 8.0, 4.0], [4.0, 2.0, 5.0], [3.0, 3.0, 1.0]], [[5.0, 3.0, 3.0], [4.0, 
2.0, 8.0], [4.0, 8.0, 3.0], [6.0, 4.0, 2.0]], [[7.0, 1.0, 6.0], [6.0, 4.0, 2.0], [8.0, 5.0, 7.0], [8.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 8.0], [4.0, 7.0, 2.0], [5.0, 6.0, 6.0], [2.0, 7.0, 6.0]], [[8.0, 2.0, 2.0], [7.0, 
7.0, 3.0], [2.0, 2.0, 6.0], [4.0, 4.0, 1.0]], [[8.0, 2.0, 2.0], [1.0, 6.0, 5.0], [8.0, 1.0, 5.0], [1.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PCR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[1.0, 2.0, 2.0], [1.0, 3.0, 5.0], [1.0, 5.0, 8.0], [8.0, 7.0, 5.0]], [[6.0, 3.0, 3.0], [7.0, 
2.0, 7.0], [6.0, 1.0, 8.0], [5.0, 5.0, 8.0]], [[2.0, 3.0, 7.0], [4.0, 2.0, 2.0], [6.0, 1.0, 6.0], [6.0, 2.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 6.0, 4.0], [8.0, 5.0, 1.0], [4.0, 5.0, 1.0], [3.0, 1.0, 1.0]], [[3.0, 4.0, 6.0], [1.0, 
6.0, 1.0], [5.0, 3.0, 1.0], [7.0, 3.0, 7.0]], [[6.0, 6.0, 2.0], [6.0, 4.0, 3.0], [6.0, 2.0, 1.0], [6.0, 7.0, 8.0]]])
      val=Data(numpy.array([[3.0], [3.0], [2.0], [1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0], [1.0], [4.0], [1.0]]))
      arg[1,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 2.0], [1.0, 3.0, 5.0], [1.0, 5.0, 8.0], [8.0, 7.0, 5.0]], [[6.0, 3.0, 3.0], [7.0, 
3.0, 7.0], [6.0, 2.0, 8.0], [5.0, 1.0, 8.0]], [[2.0, 3.0, 7.0], [4.0, 2.0, 2.0], [6.0, 1.0, 6.0], [6.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 4.0], [8.0, 5.0, 1.0], [4.0, 5.0, 1.0], [3.0, 1.0, 1.0]], [[3.0, 7.0, 6.0], [1.0, 
1.0, 1.0], [5.0, 4.0, 1.0], [7.0, 1.0, 7.0]], [[6.0, 6.0, 2.0], [6.0, 4.0, 3.0], [6.0, 2.0, 1.0], [6.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[1.0, 1.0, 4.0], [1.0, 5.0, 7.0], [4.0, 7.0, 1.0], [2.0, 6.0, 6.0]], [[2.0, 7.0, 7.0], [7.0, 
5.0, 6.0], [8.0, 7.0, 3.0], [6.0, 3.0, 4.0]], [[1.0, 7.0, 6.0], [4.0, 4.0, 5.0], [2.0, 2.0, 8.0], [7.0, 7.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 8.0, 1.0], [5.0, 2.0, 3.0], [3.0, 4.0, 3.0], [7.0, 7.0, 6.0]], [[4.0, 2.0, 2.0], [2.0, 
3.0, 8.0], [5.0, 1.0, 6.0], [1.0, 7.0, 2.0]], [[8.0, 3.0, 6.0], [3.0, 8.0, 5.0], [3.0, 5.0, 7.0], [5.0, 2.0, 3.0]]])
      val=Data(numpy.array([[[5.0, 5.0, 8.0], [1.0, 1.0, 2.0], [6.0, 5.0, 7.0], [2.0, 2.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 3.0, 4.0], [3.0, 7.0, 2.0], [1.0, 4.0, 1.0], [5.0, 4.0, 1.0]]]))
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 4.0], [1.0, 5.0, 7.0], [4.0, 7.0, 1.0], [2.0, 6.0, 6.0]], [[2.0, 7.0, 7.0], [7.0, 
5.0, 6.0], [8.0, 7.0, 3.0], [6.0, 3.0, 4.0]], [[5.0, 5.0, 8.0], [1.0, 1.0, 2.0], [6.0, 5.0, 7.0], [2.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 1.0], [5.0, 2.0, 3.0], [3.0, 4.0, 3.0], [7.0, 7.0, 6.0]], [[4.0, 2.0, 2.0], [2.0, 
3.0, 8.0], [5.0, 1.0, 6.0], [1.0, 7.0, 2.0]], [[2.0, 3.0, 4.0], [3.0, 7.0, 2.0], [1.0, 4.0, 1.0], [5.0, 4.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_UPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 8.0, 2.0], [8.0, 8.0, 8.0], [4.0, 3.0, 3.0], [5.0, 6.0, 4.0]], [[4.0, 4.0, 3.0], [1.0, 
1.0, 8.0], [2.0, 5.0, 3.0], [2.0, 4.0, 4.0]], [[4.0, 7.0, 3.0], [4.0, 8.0, 7.0], [8.0, 3.0, 4.0], [5.0, 7.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 8.0, 5.0], [7.0, 8.0, 7.0], [5.0, 3.0, 8.0], [8.0, 5.0, 5.0]], [[3.0, 2.0, 6.0], [4.0, 
8.0, 4.0], [3.0, 2.0, 3.0], [5.0, 8.0, 1.0]], [[4.0, 3.0, 1.0], [3.0, 1.0, 5.0], [3.0, 7.0, 8.0], [4.0, 2.0, 3.0]]])
      val=Data(numpy.array([[4.0, 2.0, 7.0], [2.0, 6.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 2.0, 2.0], [6.0, 4.0, 2.0]]))
      arg[:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 2.0], [8.0, 8.0, 8.0], [4.0, 2.0, 7.0], [5.0, 6.0, 4.0]], [[4.0, 4.0, 3.0], [1.0, 
1.0, 8.0], [2.0, 6.0, 5.0], [2.0, 4.0, 4.0]], [[4.0, 7.0, 3.0], [4.0, 8.0, 7.0], [8.0, 3.0, 4.0], [5.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 5.0], [7.0, 8.0, 7.0], [4.0, 2.0, 2.0], [8.0, 5.0, 5.0]], [[3.0, 2.0, 6.0], [4.0, 
8.0, 4.0], [6.0, 4.0, 2.0], [5.0, 8.0, 1.0]], [[4.0, 3.0, 1.0], [3.0, 1.0, 5.0], [3.0, 7.0, 8.0], [4.0, 2.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_UPR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[6.0, 5.0, 2.0], [2.0, 4.0, 3.0], [1.0, 2.0, 5.0], [3.0, 6.0, 2.0]], [[8.0, 8.0, 8.0], [8.0, 
4.0, 8.0], [8.0, 5.0, 7.0], [4.0, 3.0, 2.0]], [[4.0, 1.0, 8.0], [1.0, 2.0, 4.0], [7.0, 5.0, 3.0], [7.0, 3.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 5.0, 3.0], [5.0, 5.0, 3.0], [8.0, 8.0, 7.0], [7.0, 4.0, 1.0]], [[8.0, 6.0, 1.0], [5.0, 
1.0, 3.0], [2.0, 1.0, 2.0], [2.0, 8.0, 3.0]], [[2.0, 4.0, 7.0], [7.0, 4.0, 7.0], [1.0, 5.0, 7.0], [5.0, 7.0, 5.0]]])
      val=Data(numpy.array([[3.0], [4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0], [2.0]]))
      arg[:2,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 2.0], [2.0, 4.0, 3.0], [1.0, 3.0, 5.0], [3.0, 6.0, 2.0]], [[8.0, 8.0, 8.0], [8.0, 
4.0, 8.0], [8.0, 4.0, 7.0], [4.0, 3.0, 2.0]], [[4.0, 1.0, 8.0], [1.0, 2.0, 4.0], [7.0, 5.0, 3.0], [7.0, 3.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 3.0], [5.0, 5.0, 3.0], [8.0, 4.0, 7.0], [7.0, 4.0, 1.0]], [[8.0, 6.0, 1.0], [5.0, 
1.0, 3.0], [2.0, 2.0, 2.0], [2.0, 8.0, 3.0]], [[2.0, 4.0, 7.0], [7.0, 4.0, 7.0], [1.0, 5.0, 7.0], [5.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RUC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 3.0, 7.0], [1.0, 4.0, 8.0], [5.0, 2.0, 6.0], [4.0, 4.0, 7.0]], [[7.0, 1.0, 3.0], [1.0, 
2.0, 8.0], [6.0, 5.0, 7.0], [3.0, 3.0, 7.0]], [[4.0, 4.0, 4.0], [2.0, 2.0, 4.0], [2.0, 4.0, 1.0], [7.0, 7.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 8.0, 1.0], [3.0, 7.0, 2.0], [5.0, 2.0, 2.0], [6.0, 2.0, 6.0]], [[3.0, 1.0, 3.0], [1.0, 
1.0, 7.0], [6.0, 7.0, 3.0], [7.0, 6.0, 8.0]], [[5.0, 6.0, 3.0], [7.0, 4.0, 3.0], [3.0, 5.0, 7.0], [6.0, 4.0, 1.0]]])
      val=Data(numpy.array([[[4.0, 2.0, 4.0], [7.0, 2.0, 5.0], [7.0, 8.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 2.0, 4.0], [4.0, 5.0, 8.0], [1.0, 3.0, 4.0]]]))
      arg[1:2,:3,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 3.0, 7.0], [1.0, 4.0, 8.0], [5.0, 2.0, 6.0], [4.0, 4.0, 7.0]], [[4.0, 2.0, 4.0], [7.0, 
2.0, 5.0], [7.0, 8.0, 7.0], [3.0, 3.0, 7.0]], [[4.0, 4.0, 4.0], [2.0, 2.0, 4.0], [2.0, 4.0, 1.0], [7.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 1.0], [3.0, 7.0, 2.0], [5.0, 2.0, 2.0], [6.0, 2.0, 6.0]], [[1.0, 2.0, 4.0], [4.0, 
5.0, 8.0], [1.0, 3.0, 4.0], [7.0, 6.0, 8.0]], [[5.0, 6.0, 3.0], [7.0, 4.0, 3.0], [3.0, 5.0, 7.0], [6.0, 4.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CUU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[1.0, 8.0, 1.0], [2.0, 4.0, 4.0], [6.0, 2.0, 2.0], [2.0, 1.0, 8.0]], [[8.0, 7.0, 6.0], [7.0, 
5.0, 4.0], [4.0, 5.0, 5.0], [5.0, 8.0, 8.0]], [[4.0, 1.0, 1.0], [5.0, 6.0, 8.0], [5.0, 1.0, 8.0], [6.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 6.0, 7.0], [5.0, 3.0, 6.0], [4.0, 7.0, 6.0], [7.0, 5.0, 2.0]], [[7.0, 7.0, 4.0], [3.0, 
2.0, 3.0], [6.0, 7.0, 7.0], [7.0, 8.0, 3.0]], [[5.0, 3.0, 4.0], [1.0, 6.0, 4.0], [6.0, 8.0, 2.0], [2.0, 1.0, 5.0]]])
      val=Data(numpy.array([[[2.0, 4.0], [6.0, 3.0], [6.0, 8.0]], [[7.0, 4.0], [1.0, 1.0], [6.0, 8.0]], [[4.0, 3.0], [3.0, 
7.0], [7.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0], [1.0, 1.0], [3.0, 5.0]], [[5.0, 4.0], [4.0, 6.0], [3.0, 4.0]], [[3.0, 
8.0], [3.0, 5.0], [7.0, 8.0]]]))
      arg[:,:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 1.0], [6.0, 3.0, 4.0], [6.0, 8.0, 2.0], [2.0, 1.0, 8.0]], [[7.0, 4.0, 6.0], [1.0, 
1.0, 4.0], [6.0, 8.0, 5.0], [5.0, 8.0, 8.0]], [[4.0, 3.0, 1.0], [3.0, 7.0, 8.0], [7.0, 2.0, 8.0], [6.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 7.0], [1.0, 1.0, 6.0], [3.0, 5.0, 6.0], [7.0, 5.0, 2.0]], [[5.0, 4.0, 4.0], [4.0, 
6.0, 3.0], [3.0, 4.0, 7.0], [7.0, 8.0, 3.0]], [[3.0, 8.0, 4.0], [3.0, 5.0, 4.0], [7.0, 8.0, 2.0], [2.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PRL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[1.0, 5.0, 5.0], [7.0, 5.0, 1.0], [1.0, 5.0, 4.0], [8.0, 8.0, 6.0]], [[1.0, 5.0, 6.0], [8.0, 
1.0, 5.0], [2.0, 2.0, 5.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 8.0, 7.0], [8.0, 8.0, 4.0], [6.0, 4.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 2.0, 1.0], [2.0, 7.0, 6.0], [7.0, 4.0, 5.0], [4.0, 4.0, 6.0]], [[8.0, 1.0, 8.0], [3.0, 
2.0, 7.0], [7.0, 1.0, 6.0], [4.0, 6.0, 6.0]], [[2.0, 6.0, 3.0], [4.0, 2.0, 5.0], [7.0, 4.0, 8.0], [4.0, 2.0, 3.0]]])
      val=Data(numpy.array([[5.0], [2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0], [4.0]]))
      arg[1,1:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 5.0], [7.0, 5.0, 1.0], [1.0, 5.0, 4.0], [8.0, 8.0, 6.0]], [[1.0, 5.0, 6.0], [8.0, 
1.0, 5.0], [2.0, 2.0, 2.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 8.0, 7.0], [8.0, 8.0, 4.0], [6.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 1.0], [2.0, 7.0, 6.0], [7.0, 4.0, 5.0], [4.0, 4.0, 6.0]], [[8.0, 1.0, 8.0], [3.0, 
2.0, 1.0], [7.0, 1.0, 4.0], [4.0, 6.0, 6.0]], [[2.0, 6.0, 3.0], [4.0, 2.0, 5.0], [7.0, 4.0, 8.0], [4.0, 2.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CLU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[8.0, 1.0, 1.0], [1.0, 7.0, 4.0], [3.0, 8.0, 6.0], [6.0, 1.0, 2.0]], [[5.0, 6.0, 8.0], [4.0, 
4.0, 3.0], [4.0, 8.0, 6.0], [6.0, 3.0, 6.0]], [[3.0, 5.0, 4.0], [8.0, 1.0, 7.0], [4.0, 6.0, 6.0], [2.0, 5.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 8.0, 4.0], [3.0, 8.0, 2.0], [7.0, 3.0, 4.0], [7.0, 2.0, 5.0]], [[2.0, 3.0, 2.0], [7.0, 
1.0, 1.0], [2.0, 2.0, 3.0], [7.0, 8.0, 2.0]], [[6.0, 7.0, 3.0], [5.0, 6.0, 2.0], [4.0, 3.0, 4.0], [1.0, 8.0, 1.0]]])
      val=Data(numpy.array([[[7.0, 4.0], [8.0, 7.0]], [[3.0, 4.0], [8.0, 1.0]], [[6.0, 8.0], [2.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 8.0], [7.0, 1.0]], [[6.0, 7.0], [7.0, 2.0]], [[3.0, 1.0], [2.0, 4.0]]]))
      arg[:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 1.0, 1.0], [1.0, 7.0, 4.0], [7.0, 4.0, 6.0], [8.0, 7.0, 2.0]], [[5.0, 6.0, 8.0], [4.0, 
4.0, 3.0], [3.0, 4.0, 6.0], [8.0, 1.0, 6.0]], [[3.0, 5.0, 4.0], [8.0, 1.0, 7.0], [6.0, 8.0, 6.0], [2.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 4.0], [3.0, 8.0, 2.0], [7.0, 8.0, 4.0], [7.0, 1.0, 5.0]], [[2.0, 3.0, 2.0], [7.0, 
1.0, 1.0], [6.0, 7.0, 3.0], [7.0, 2.0, 2.0]], [[6.0, 7.0, 3.0], [5.0, 6.0, 2.0], [3.0, 1.0, 4.0], [2.0, 4.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LRU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 8.0, 2.0], [8.0, 3.0, 8.0], [5.0, 3.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 8.0, 8.0], [6.0, 
7.0, 2.0], [1.0, 7.0, 5.0], [5.0, 2.0, 8.0]], [[8.0, 3.0, 6.0], [3.0, 1.0, 4.0], [3.0, 8.0, 5.0], [1.0, 4.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 7.0, 5.0], [6.0, 7.0, 5.0], [7.0, 2.0, 5.0], [8.0, 6.0, 7.0]], [[8.0, 6.0, 6.0], [6.0, 
6.0, 4.0], [3.0, 5.0, 2.0], [7.0, 7.0, 2.0]], [[8.0, 6.0, 8.0], [4.0, 4.0, 2.0], [2.0, 2.0, 8.0], [4.0, 7.0, 6.0]]])
      val=Data(numpy.array([[[6.0, 2.0], [4.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 7.0], [3.0, 6.0]]]))
      arg[2:,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 8.0, 2.0], [8.0, 3.0, 8.0], [5.0, 3.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 8.0, 8.0], [6.0, 
7.0, 2.0], [1.0, 7.0, 5.0], [5.0, 2.0, 8.0]], [[8.0, 3.0, 6.0], [6.0, 2.0, 4.0], [4.0, 5.0, 5.0], [1.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 7.0, 5.0], [6.0, 7.0, 5.0], [7.0, 2.0, 5.0], [8.0, 6.0, 7.0]], [[8.0, 6.0, 6.0], [6.0, 
6.0, 4.0], [3.0, 5.0, 2.0], [7.0, 7.0, 2.0]], [[8.0, 6.0, 8.0], [1.0, 7.0, 2.0], [3.0, 6.0, 8.0], [4.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RLC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 7.0, 4.0], [6.0, 6.0, 3.0], [1.0, 5.0, 2.0], [5.0, 2.0, 7.0]], [[6.0, 2.0, 5.0], [2.0, 
4.0, 8.0], [8.0, 6.0, 6.0], [1.0, 7.0, 8.0]], [[6.0, 4.0, 1.0], [5.0, 1.0, 8.0], [7.0, 6.0, 4.0], [6.0, 5.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 3.0, 1.0], [1.0, 2.0, 7.0], [4.0, 6.0, 2.0], [5.0, 3.0, 4.0]], [[3.0, 4.0, 4.0], [7.0, 
6.0, 8.0], [5.0, 7.0, 7.0], [8.0, 7.0, 7.0]], [[5.0, 4.0, 1.0], [7.0, 4.0, 3.0], [8.0, 7.0, 5.0], [7.0, 3.0, 8.0]]])
      val=Data(numpy.array([[[7.0, 3.0, 6.0], [6.0, 8.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 8.0, 4.0], [5.0, 3.0, 1.0]]]))
      arg[1:2,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 4.0], [6.0, 6.0, 3.0], [1.0, 5.0, 2.0], [5.0, 2.0, 7.0]], [[6.0, 2.0, 5.0], [2.0, 
4.0, 8.0], [7.0, 3.0, 6.0], [6.0, 8.0, 5.0]], [[6.0, 4.0, 1.0], [5.0, 1.0, 8.0], [7.0, 6.0, 4.0], [6.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 1.0], [1.0, 2.0, 7.0], [4.0, 6.0, 2.0], [5.0, 3.0, 4.0]], [[3.0, 4.0, 4.0], [7.0, 
6.0, 8.0], [1.0, 8.0, 4.0], [5.0, 3.0, 1.0]], [[5.0, 4.0, 1.0], [7.0, 4.0, 3.0], [8.0, 7.0, 5.0], [7.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 5.0, 3.0], [6.0, 2.0, 8.0], [5.0, 6.0, 7.0], [4.0, 7.0, 4.0]], [[1.0, 2.0, 7.0], [3.0, 
4.0, 3.0], [2.0, 5.0, 6.0], [8.0, 3.0, 2.0]], [[7.0, 3.0, 1.0], [2.0, 2.0, 1.0], [8.0, 8.0, 6.0], [2.0, 8.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 1.0, 1.0], [2.0, 8.0, 6.0], [1.0, 6.0, 2.0], [7.0, 2.0, 7.0]], [[1.0, 5.0, 7.0], [1.0, 
2.0, 2.0], [6.0, 3.0, 4.0], [8.0, 8.0, 1.0]], [[5.0, 4.0, 4.0], [3.0, 3.0, 6.0], [7.0, 8.0, 6.0], [7.0, 5.0, 2.0]]])
      val=Data(numpy.array([[7.0, 7.0, 4.0], [6.0, 2.0, 5.0], [4.0, 3.0, 5.0], [6.0, 5.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 3.0, 4.0], [1.0, 2.0, 7.0], [7.0, 6.0, 5.0], [7.0, 7.0, 4.0]]))
      arg[1,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 3.0], [6.0, 2.0, 8.0], [5.0, 6.0, 7.0], [4.0, 7.0, 4.0]], [[7.0, 7.0, 4.0], [6.0, 
2.0, 5.0], [4.0, 3.0, 5.0], [6.0, 5.0, 8.0]], [[7.0, 3.0, 1.0], [2.0, 2.0, 1.0], [8.0, 8.0, 6.0], [2.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 1.0], [2.0, 8.0, 6.0], [1.0, 6.0, 2.0], [7.0, 2.0, 7.0]], [[2.0, 3.0, 4.0], [1.0, 
2.0, 7.0], [7.0, 6.0, 5.0], [7.0, 7.0, 4.0]], [[5.0, 4.0, 4.0], [3.0, 3.0, 6.0], [7.0, 8.0, 6.0], [7.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CPU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 7.0, 5.0], [7.0, 6.0, 2.0], [7.0, 7.0, 1.0], [4.0, 7.0, 5.0]], [[8.0, 8.0, 6.0], [3.0, 
7.0, 7.0], [1.0, 6.0, 4.0], [4.0, 6.0, 2.0]], [[2.0, 5.0, 4.0], [2.0, 7.0, 8.0], [4.0, 8.0, 1.0], [3.0, 7.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 2.0, 4.0], [5.0, 4.0, 8.0], [3.0, 7.0, 1.0], [1.0, 2.0, 1.0]], [[4.0, 3.0, 5.0], [8.0, 
3.0, 5.0], [6.0, 6.0, 1.0], [6.0, 7.0, 6.0]], [[2.0, 6.0, 2.0], [4.0, 6.0, 5.0], [2.0, 7.0, 8.0], [1.0, 5.0, 1.0]]])
      val=Data(numpy.array([[3.0, 6.0], [1.0, 4.0], [3.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 4.0], [2.0, 8.0], [1.0, 4.0]]))
      arg[:,2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 5.0], [7.0, 6.0, 2.0], [3.0, 6.0, 1.0], [4.0, 7.0, 5.0]], [[8.0, 8.0, 6.0], [3.0, 
7.0, 7.0], [1.0, 4.0, 4.0], [4.0, 6.0, 2.0]], [[2.0, 5.0, 4.0], [2.0, 7.0, 8.0], [3.0, 1.0, 1.0], [3.0, 7.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 4.0], [5.0, 4.0, 8.0], [8.0, 4.0, 1.0], [1.0, 2.0, 1.0]], [[4.0, 3.0, 5.0], [8.0, 
3.0, 5.0], [2.0, 8.0, 1.0], [6.0, 7.0, 6.0]], [[2.0, 6.0, 2.0], [4.0, 6.0, 5.0], [1.0, 4.0, 8.0], [1.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RPP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 7.0, 4.0], [4.0, 6.0, 6.0], [1.0, 4.0, 2.0], [7.0, 4.0, 6.0]], [[2.0, 5.0, 4.0], [8.0, 
7.0, 7.0], [6.0, 3.0, 4.0], [6.0, 4.0, 3.0]], [[5.0, 1.0, 4.0], [5.0, 3.0, 5.0], [7.0, 4.0, 8.0], [8.0, 5.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 8.0, 6.0], [5.0, 3.0, 1.0], [7.0, 3.0, 4.0], [5.0, 1.0, 7.0]], [[2.0, 3.0, 1.0], [4.0, 
3.0, 4.0], [8.0, 7.0, 4.0], [8.0, 4.0, 3.0]], [[8.0, 8.0, 1.0], [5.0, 6.0, 6.0], [1.0, 6.0, 8.0], [3.0, 4.0, 8.0]]])
      val=Data(numpy.array([3.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([5.0]))
      arg[1:2,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 4.0], [4.0, 6.0, 6.0], [1.0, 4.0, 2.0], [7.0, 4.0, 6.0]], [[2.0, 5.0, 4.0], [8.0, 
7.0, 7.0], [6.0, 3.0, 4.0], [6.0, 4.0, 3.0]], [[5.0, 1.0, 4.0], [5.0, 3.0, 5.0], [7.0, 4.0, 8.0], [8.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 6.0], [5.0, 3.0, 1.0], [7.0, 3.0, 4.0], [5.0, 1.0, 7.0]], [[2.0, 3.0, 1.0], [4.0, 
3.0, 4.0], [8.0, 5.0, 4.0], [8.0, 4.0, 3.0]], [[8.0, 8.0, 1.0], [5.0, 6.0, 6.0], [1.0, 6.0, 8.0], [3.0, 4.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LUP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 6.0, 7.0], [1.0, 8.0, 6.0], [4.0, 6.0, 4.0], [5.0, 4.0, 7.0]], [[1.0, 7.0, 4.0], [5.0, 
2.0, 6.0], [3.0, 7.0, 4.0], [8.0, 5.0, 3.0]], [[5.0, 8.0, 2.0], [1.0, 8.0, 8.0], [1.0, 1.0, 4.0], [8.0, 6.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 1.0, 3.0], [3.0, 8.0, 1.0], [6.0, 4.0, 7.0], [6.0, 3.0, 8.0]], [[8.0, 3.0, 2.0], [4.0, 
3.0, 7.0], [1.0, 2.0, 5.0], [5.0, 4.0, 7.0]], [[2.0, 7.0, 3.0], [6.0, 4.0, 3.0], [8.0, 7.0, 1.0], [2.0, 1.0, 3.0]]])
      val=Data(numpy.array([[2.0, 1.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 6.0, 7.0]]))
      arg[2:,:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 6.0, 7.0], [1.0, 8.0, 6.0], [4.0, 6.0, 4.0], [5.0, 4.0, 7.0]], [[1.0, 7.0, 4.0], [5.0, 
2.0, 6.0], [3.0, 7.0, 4.0], [8.0, 5.0, 3.0]], [[5.0, 2.0, 2.0], [1.0, 1.0, 8.0], [1.0, 7.0, 4.0], [8.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 3.0], [3.0, 8.0, 1.0], [6.0, 4.0, 7.0], [6.0, 3.0, 8.0]], [[8.0, 3.0, 2.0], [4.0, 
3.0, 7.0], [1.0, 2.0, 5.0], [5.0, 4.0, 7.0]], [[2.0, 6.0, 3.0], [6.0, 6.0, 3.0], [8.0, 7.0, 1.0], [2.0, 1.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PLC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[4.0, 4.0, 7.0], [5.0, 7.0, 6.0], [2.0, 2.0, 1.0], [7.0, 1.0, 7.0]], [[5.0, 6.0, 5.0], [7.0, 
8.0, 6.0], [4.0, 7.0, 3.0], [1.0, 3.0, 1.0]], [[1.0, 1.0, 3.0], [1.0, 2.0, 5.0], [2.0, 4.0, 4.0], [1.0, 1.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 5.0, 5.0], [3.0, 3.0, 8.0], [2.0, 6.0, 2.0], [4.0, 6.0, 3.0]], [[3.0, 2.0, 1.0], [6.0, 
2.0, 7.0], [8.0, 5.0, 4.0], [3.0, 2.0, 2.0]], [[8.0, 8.0, 3.0], [6.0, 2.0, 6.0], [2.0, 8.0, 5.0], [4.0, 4.0, 4.0]]])
      val=Data(numpy.array([[2.0, 4.0, 7.0], [1.0, 2.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 4.0, 7.0], [7.0, 1.0, 7.0]]))
      arg[1,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 7.0], [5.0, 7.0, 6.0], [2.0, 2.0, 1.0], [7.0, 1.0, 7.0]], [[5.0, 6.0, 5.0], [7.0, 
8.0, 6.0], [2.0, 4.0, 7.0], [1.0, 2.0, 6.0]], [[1.0, 1.0, 3.0], [1.0, 2.0, 5.0], [2.0, 4.0, 4.0], [1.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 5.0], [3.0, 3.0, 8.0], [2.0, 6.0, 2.0], [4.0, 6.0, 3.0]], [[3.0, 2.0, 1.0], [6.0, 
2.0, 7.0], [4.0, 4.0, 7.0], [7.0, 1.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 2.0, 6.0], [2.0, 8.0, 5.0], [4.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LLU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 4.0, 7.0], [3.0, 2.0, 3.0], [3.0, 2.0, 1.0], [2.0, 2.0, 7.0]], [[8.0, 2.0, 5.0], [2.0, 
7.0, 7.0], [8.0, 5.0, 7.0], [1.0, 2.0, 2.0]], [[7.0, 4.0, 8.0], [7.0, 6.0, 4.0], [4.0, 2.0, 7.0], [3.0, 8.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 7.0, 2.0], [2.0, 8.0, 6.0], [5.0, 6.0, 7.0], [5.0, 3.0, 8.0]], [[3.0, 6.0, 4.0], [4.0, 
4.0, 8.0], [4.0, 4.0, 1.0], [5.0, 1.0, 1.0]], [[5.0, 3.0, 8.0], [8.0, 7.0, 3.0], [1.0, 5.0, 1.0], [3.0, 3.0, 8.0]]])
      val=Data(numpy.array([[[1.0, 4.0], [8.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 4.0], [7.0, 5.0]]]))
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 7.0], [3.0, 2.0, 3.0], [3.0, 2.0, 1.0], [2.0, 2.0, 7.0]], [[8.0, 2.0, 5.0], [2.0, 
7.0, 7.0], [8.0, 5.0, 7.0], [1.0, 2.0, 2.0]], [[7.0, 4.0, 8.0], [7.0, 6.0, 4.0], [1.0, 4.0, 7.0], [8.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 7.0, 2.0], [2.0, 8.0, 6.0], [5.0, 6.0, 7.0], [5.0, 3.0, 8.0]], [[3.0, 6.0, 4.0], [4.0, 
4.0, 8.0], [4.0, 4.0, 1.0], [5.0, 1.0, 1.0]], [[5.0, 3.0, 8.0], [8.0, 7.0, 3.0], [5.0, 4.0, 1.0], [7.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PPC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[7.0, 8.0, 5.0], [6.0, 6.0, 5.0], [6.0, 2.0, 3.0], [3.0, 2.0, 8.0]], [[8.0, 2.0, 1.0], [7.0, 
4.0, 6.0], [8.0, 2.0, 8.0], [3.0, 3.0, 7.0]], [[8.0, 1.0, 2.0], [5.0, 5.0, 5.0], [1.0, 5.0, 3.0], [2.0, 4.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 8.0, 6.0], [6.0, 4.0, 5.0], [2.0, 7.0, 6.0], [8.0, 4.0, 3.0]], [[8.0, 8.0, 8.0], [1.0, 
1.0, 8.0], [2.0, 1.0, 6.0], [5.0, 8.0, 5.0]], [[8.0, 2.0, 3.0], [8.0, 5.0, 3.0], [2.0, 4.0, 3.0], [6.0, 6.0, 4.0]]])
      val=Data(numpy.array([2.0, 1.0, 3.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0, 1.0, 4.0]))
      arg[1,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 5.0], [6.0, 6.0, 5.0], [6.0, 2.0, 3.0], [3.0, 2.0, 8.0]], [[8.0, 2.0, 1.0], [7.0, 
4.0, 6.0], [2.0, 1.0, 3.0], [3.0, 3.0, 7.0]], [[8.0, 1.0, 2.0], [5.0, 5.0, 5.0], [1.0, 5.0, 3.0], [2.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 6.0], [6.0, 4.0, 5.0], [2.0, 7.0, 6.0], [8.0, 4.0, 3.0]], [[8.0, 8.0, 8.0], [1.0, 
1.0, 8.0], [8.0, 1.0, 4.0], [5.0, 8.0, 5.0]], [[8.0, 2.0, 3.0], [8.0, 5.0, 3.0], [2.0, 4.0, 3.0], [6.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[7.0, 6.0, 4.0], [5.0, 1.0, 7.0], [5.0, 4.0, 8.0], [7.0, 1.0, 8.0]], [[6.0, 3.0, 1.0], [7.0, 
8.0, 2.0], [8.0, 7.0, 1.0], [7.0, 4.0, 3.0]], [[6.0, 1.0, 1.0], [6.0, 2.0, 7.0], [3.0, 6.0, 8.0], [5.0, 4.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 4.0, 3.0], [8.0, 2.0, 7.0], [4.0, 5.0, 7.0], [2.0, 7.0, 8.0]], [[8.0, 1.0, 5.0], [4.0, 
6.0, 2.0], [4.0, 2.0, 7.0], [4.0, 3.0, 6.0]], [[1.0, 2.0, 4.0], [2.0, 7.0, 3.0], [3.0, 3.0, 8.0], [5.0, 7.0, 1.0]]])
      val=Data(numpy.array([[[4.0], [6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0], [5.0]]]))
      arg[2:,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 4.0], [5.0, 1.0, 7.0], [5.0, 4.0, 8.0], [7.0, 1.0, 8.0]], [[6.0, 3.0, 1.0], [7.0, 
8.0, 2.0], [8.0, 7.0, 1.0], [7.0, 4.0, 3.0]], [[6.0, 1.0, 1.0], [6.0, 4.0, 7.0], [3.0, 6.0, 8.0], [5.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 4.0, 3.0], [8.0, 2.0, 7.0], [4.0, 5.0, 7.0], [2.0, 7.0, 8.0]], [[8.0, 1.0, 5.0], [4.0, 
6.0, 2.0], [4.0, 2.0, 7.0], [4.0, 3.0, 6.0]], [[1.0, 2.0, 4.0], [2.0, 7.0, 3.0], [3.0, 5.0, 8.0], [5.0, 7.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CCU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 8.0, 1.0], [4.0, 7.0, 2.0], [3.0, 7.0, 6.0], [4.0, 5.0, 2.0]], [[3.0, 1.0, 8.0], [4.0, 
8.0, 3.0], [1.0, 6.0, 8.0], [3.0, 2.0, 3.0]], [[5.0, 5.0, 8.0], [6.0, 7.0, 6.0], [5.0, 1.0, 1.0], [5.0, 6.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 6.0, 8.0], [1.0, 3.0, 7.0], [3.0, 8.0, 1.0], [2.0, 3.0, 2.0]], [[3.0, 1.0, 1.0], [8.0, 
5.0, 7.0], [4.0, 4.0, 2.0], [6.0, 7.0, 5.0]], [[3.0, 7.0, 3.0], [5.0, 7.0, 5.0], [3.0, 4.0, 2.0], [6.0, 6.0, 5.0]]])
      val=Data(numpy.array([[[6.0, 5.0], [7.0, 2.0], [2.0, 8.0], [7.0, 5.0]], [[4.0, 2.0], [4.0, 7.0], [8.0, 8.0], [3.0, 1.0]], 
[[1.0, 8.0], [8.0, 4.0], [5.0, 4.0], [8.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0], [1.0, 3.0], [6.0, 1.0], [3.0, 6.0]], [[5.0, 6.0], [1.0, 5.0], [1.0, 1.0], 
[5.0, 3.0]], [[2.0, 3.0], [3.0, 4.0], [3.0, 2.0], [6.0, 3.0]]]))
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 1.0], [7.0, 2.0, 2.0], [2.0, 8.0, 6.0], [7.0, 5.0, 2.0]], [[4.0, 2.0, 8.0], [4.0, 
7.0, 3.0], [8.0, 8.0, 8.0], [3.0, 1.0, 3.0]], [[1.0, 8.0, 8.0], [8.0, 4.0, 6.0], [5.0, 4.0, 1.0], [8.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 8.0], [1.0, 3.0, 7.0], [6.0, 1.0, 1.0], [3.0, 6.0, 2.0]], [[5.0, 6.0, 1.0], [1.0, 
5.0, 7.0], [1.0, 1.0, 2.0], [5.0, 3.0, 5.0]], [[2.0, 3.0, 3.0], [3.0, 4.0, 5.0], [3.0, 2.0, 2.0], [6.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 4.0, 4.0], [8.0, 2.0, 7.0], [8.0, 2.0, 7.0], [6.0, 5.0, 8.0]], [[8.0, 8.0, 2.0], [5.0, 
6.0, 7.0], [2.0, 3.0, 7.0], [4.0, 3.0, 5.0]], [[8.0, 1.0, 6.0], [6.0, 3.0, 2.0], [4.0, 8.0, 2.0], [8.0, 4.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 6.0, 1.0], [4.0, 4.0, 8.0], [6.0, 7.0, 8.0], [7.0, 4.0, 7.0]], [[3.0, 8.0, 7.0], [1.0, 
3.0, 6.0], [8.0, 1.0, 6.0], [4.0, 3.0, 5.0]], [[4.0, 3.0, 5.0], [8.0, 8.0, 5.0], [2.0, 3.0, 8.0], [1.0, 3.0, 2.0]]])
      val=Data(numpy.array([[[4.0, 1.0, 6.0], [2.0, 3.0, 1.0]], [[6.0, 8.0, 6.0], [4.0, 3.0, 1.0]], [[1.0, 6.0, 2.0], [1.0, 
7.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 7.0, 4.0], [1.0, 6.0, 4.0]], [[6.0, 2.0, 4.0], [3.0, 8.0, 1.0]], [[6.0, 6.0, 
2.0], [7.0, 7.0, 8.0]]]))
      arg[:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 4.0], [4.0, 1.0, 6.0], [2.0, 3.0, 1.0], [6.0, 5.0, 8.0]], [[8.0, 8.0, 2.0], [6.0, 
8.0, 6.0], [4.0, 3.0, 1.0], [4.0, 3.0, 5.0]], [[8.0, 1.0, 6.0], [1.0, 6.0, 2.0], [1.0, 7.0, 7.0], [8.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 1.0], [8.0, 7.0, 4.0], [1.0, 6.0, 4.0], [7.0, 4.0, 7.0]], [[3.0, 8.0, 7.0], [6.0, 
2.0, 4.0], [3.0, 8.0, 1.0], [4.0, 3.0, 5.0]], [[4.0, 3.0, 5.0], [6.0, 6.0, 2.0], [7.0, 7.0, 8.0], [1.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PPU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[8.0, 2.0, 8.0], [4.0, 4.0, 4.0], [2.0, 5.0, 6.0], [1.0, 5.0, 4.0]], [[2.0, 1.0, 1.0], [7.0, 
5.0, 8.0], [5.0, 4.0, 8.0], [7.0, 3.0, 7.0]], [[3.0, 3.0, 2.0], [8.0, 3.0, 7.0], [7.0, 3.0, 7.0], [6.0, 7.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 4.0, 5.0], [1.0, 7.0, 4.0], [1.0, 8.0, 7.0], [5.0, 1.0, 7.0]], [[6.0, 4.0, 3.0], [6.0, 
8.0, 1.0], [3.0, 8.0, 6.0], [5.0, 5.0, 3.0]], [[4.0, 4.0, 5.0], [7.0, 6.0, 8.0], [5.0, 1.0, 1.0], [5.0, 8.0, 8.0]]])
      val=Data(numpy.array([1.0, 5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 7.0]))
      arg[1,2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 8.0], [4.0, 4.0, 4.0], [2.0, 5.0, 6.0], [1.0, 5.0, 4.0]], [[2.0, 1.0, 1.0], [7.0, 
5.0, 8.0], [1.0, 5.0, 8.0], [7.0, 3.0, 7.0]], [[3.0, 3.0, 2.0], [8.0, 3.0, 7.0], [7.0, 3.0, 7.0], [6.0, 7.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 5.0], [1.0, 7.0, 4.0], [1.0, 8.0, 7.0], [5.0, 1.0, 7.0]], [[6.0, 4.0, 3.0], [6.0, 
8.0, 1.0], [3.0, 7.0, 6.0], [5.0, 5.0, 3.0]], [[4.0, 4.0, 5.0], [7.0, 6.0, 8.0], [5.0, 1.0, 1.0], [5.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CCR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 3.0, 7.0], [8.0, 2.0, 8.0], [3.0, 4.0, 4.0], [4.0, 4.0, 4.0]], [[2.0, 4.0, 5.0], [6.0, 
6.0, 4.0], [8.0, 6.0, 4.0], [1.0, 7.0, 2.0]], [[4.0, 1.0, 6.0], [8.0, 4.0, 3.0], [4.0, 7.0, 2.0], [5.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 7.0, 8.0], [1.0, 4.0, 1.0], [2.0, 1.0, 7.0], [2.0, 6.0, 4.0]], [[5.0, 8.0, 4.0], [3.0, 
5.0, 7.0], [6.0, 3.0, 1.0], [5.0, 1.0, 8.0]], [[7.0, 2.0, 8.0], [8.0, 5.0, 3.0], [8.0, 8.0, 8.0], [7.0, 7.0, 1.0]]])
      val=Data(numpy.array([[[1.0], [8.0], [8.0], [8.0]], [[5.0], [8.0], [7.0], [7.0]], [[5.0], [6.0], [8.0], 
[3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0], [8.0], [3.0], [3.0]], [[4.0], [4.0], [2.0], [7.0]], [[6.0], [6.0], [4.0], 
[8.0]]]))
      arg[:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 7.0], [8.0, 8.0, 8.0], [3.0, 8.0, 4.0], [4.0, 8.0, 4.0]], [[2.0, 5.0, 5.0], [6.0, 
8.0, 4.0], [8.0, 7.0, 4.0], [1.0, 7.0, 2.0]], [[4.0, 5.0, 6.0], [8.0, 6.0, 3.0], [4.0, 8.0, 2.0], [5.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 8.0, 8.0], [1.0, 8.0, 1.0], [2.0, 3.0, 7.0], [2.0, 3.0, 4.0]], [[5.0, 4.0, 4.0], [3.0, 
4.0, 7.0], [6.0, 2.0, 1.0], [5.0, 7.0, 8.0]], [[7.0, 6.0, 8.0], [8.0, 6.0, 3.0], [8.0, 4.0, 8.0], [7.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LLC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[8.0, 6.0, 2.0], [2.0, 1.0, 2.0], [4.0, 1.0, 5.0], [1.0, 8.0, 4.0]], [[7.0, 5.0, 5.0], [3.0, 
3.0, 1.0], [5.0, 7.0, 3.0], [4.0, 8.0, 4.0]], [[5.0, 2.0, 1.0], [4.0, 8.0, 6.0], [3.0, 1.0, 6.0], [3.0, 1.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 2.0, 3.0], [2.0, 1.0, 3.0], [1.0, 1.0, 3.0], [5.0, 5.0, 5.0]], [[5.0, 5.0, 4.0], [5.0, 
6.0, 2.0], [6.0, 5.0, 4.0], [1.0, 7.0, 6.0]], [[4.0, 3.0, 8.0], [7.0, 2.0, 1.0], [4.0, 1.0, 6.0], [4.0, 2.0, 1.0]]])
      val=Data(numpy.array([[[4.0, 2.0, 2.0], [7.0, 1.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 6.0, 3.0], [8.0, 7.0, 8.0]]]))
      arg[2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 2.0], [2.0, 1.0, 2.0], [4.0, 1.0, 5.0], [1.0, 8.0, 4.0]], [[7.0, 5.0, 5.0], [3.0, 
3.0, 1.0], [5.0, 7.0, 3.0], [4.0, 8.0, 4.0]], [[5.0, 2.0, 1.0], [4.0, 8.0, 6.0], [4.0, 2.0, 2.0], [7.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 3.0], [2.0, 1.0, 3.0], [1.0, 1.0, 3.0], [5.0, 5.0, 5.0]], [[5.0, 5.0, 4.0], [5.0, 
6.0, 2.0], [6.0, 5.0, 4.0], [1.0, 7.0, 6.0]], [[4.0, 3.0, 8.0], [7.0, 2.0, 1.0], [3.0, 6.0, 3.0], [8.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_URR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[6.0, 4.0, 5.0], [4.0, 8.0, 7.0], [3.0, 4.0, 8.0], [1.0, 4.0, 5.0]], [[5.0, 4.0, 4.0], [8.0, 
4.0, 1.0], [4.0, 3.0, 5.0], [8.0, 5.0, 1.0]], [[4.0, 8.0, 7.0], [2.0, 6.0, 5.0], [2.0, 4.0, 5.0], [5.0, 1.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 8.0, 5.0], [1.0, 7.0, 5.0], [1.0, 3.0, 6.0], [3.0, 6.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 
3.0, 2.0], [1.0, 5.0, 1.0], [5.0, 6.0, 3.0]], [[3.0, 6.0, 6.0], [3.0, 2.0, 5.0], [4.0, 5.0, 5.0], [2.0, 1.0, 5.0]]])
      val=Data(numpy.array([[[7.0], [4.0]], [[6.0], [1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0], [4.0]], [[1.0], [1.0]]]))
      arg[:2,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 5.0], [4.0, 7.0, 7.0], [3.0, 4.0, 8.0], [1.0, 4.0, 5.0]], [[5.0, 4.0, 4.0], [8.0, 
6.0, 1.0], [4.0, 1.0, 5.0], [8.0, 5.0, 1.0]], [[4.0, 8.0, 7.0], [2.0, 6.0, 5.0], [2.0, 4.0, 5.0], [5.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 8.0, 5.0], [1.0, 8.0, 5.0], [1.0, 4.0, 6.0], [3.0, 6.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 
1.0, 2.0], [1.0, 1.0, 1.0], [5.0, 6.0, 3.0]], [[3.0, 6.0, 6.0], [3.0, 2.0, 5.0], [4.0, 5.0, 5.0], [2.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_UUL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[1.0, 1.0, 5.0], [2.0, 8.0, 4.0], [3.0, 2.0, 8.0], [4.0, 5.0, 3.0]], [[1.0, 7.0, 6.0], [4.0, 
8.0, 1.0], [2.0, 8.0, 1.0], [6.0, 4.0, 2.0]], [[7.0, 6.0, 5.0], [4.0, 7.0, 1.0], [2.0, 1.0, 5.0], [6.0, 7.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 3.0, 7.0], [2.0, 3.0, 7.0], [6.0, 8.0, 2.0], [3.0, 1.0, 4.0]], [[8.0, 3.0, 7.0], [4.0, 
6.0, 2.0], [7.0, 2.0, 3.0], [8.0, 5.0, 2.0]], [[4.0, 5.0, 3.0], [5.0, 3.0, 6.0], [4.0, 3.0, 7.0], [4.0, 5.0, 3.0]]])
      val=Data(numpy.array([[[5.0], [8.0], [2.0]], [[7.0], [1.0], [8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0], [3.0], [6.0]], [[5.0], [6.0], [4.0]]]))
      arg[:2,:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 5.0], [2.0, 8.0, 8.0], [3.0, 2.0, 2.0], [4.0, 5.0, 3.0]], [[1.0, 7.0, 7.0], [4.0, 
8.0, 1.0], [2.0, 8.0, 8.0], [6.0, 4.0, 2.0]], [[7.0, 6.0, 5.0], [4.0, 7.0, 1.0], [2.0, 1.0, 5.0], [6.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 7.0], [2.0, 3.0, 3.0], [6.0, 8.0, 6.0], [3.0, 1.0, 4.0]], [[8.0, 3.0, 5.0], [4.0, 
6.0, 6.0], [7.0, 2.0, 4.0], [8.0, 5.0, 2.0]], [[4.0, 5.0, 3.0], [5.0, 3.0, 6.0], [4.0, 3.0, 7.0], [4.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[1.0, 3.0, 8.0], [1.0, 5.0, 2.0], [2.0, 4.0, 6.0], [4.0, 5.0, 7.0]], [[4.0, 8.0, 6.0], [2.0, 
3.0, 7.0], [7.0, 6.0, 6.0], [1.0, 1.0, 7.0]], [[8.0, 5.0, 4.0], [4.0, 4.0, 1.0], [5.0, 1.0, 1.0], [5.0, 8.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 1.0, 2.0], [1.0, 4.0, 4.0], [6.0, 5.0, 1.0], [1.0, 3.0, 2.0]], [[2.0, 1.0, 8.0], [3.0, 
2.0, 7.0], [1.0, 7.0, 3.0], [1.0, 8.0, 2.0]], [[4.0, 2.0, 1.0], [7.0, 2.0, 6.0], [7.0, 1.0, 5.0], [5.0, 5.0, 7.0]]])
      val=Data(numpy.array([[[3.0, 5.0, 7.0], [4.0, 4.0, 5.0], [1.0, 6.0, 1.0]], [[5.0, 5.0, 1.0], [3.0, 5.0, 5.0], [8.0, 8.0, 
4.0]], [[4.0, 5.0, 8.0], [5.0, 2.0, 8.0], [1.0, 7.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 8.0, 6.0], [8.0, 2.0, 3.0], [5.0, 8.0, 8.0]], [[5.0, 4.0, 5.0], [2.0, 7.0, 6.0], 
[8.0, 1.0, 8.0]], [[6.0, 2.0, 5.0], [6.0, 5.0, 6.0], [2.0, 3.0, 7.0]]]))
      arg[:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 7.0], [4.0, 4.0, 5.0], [1.0, 6.0, 1.0], [4.0, 5.0, 7.0]], [[5.0, 5.0, 1.0], [3.0, 
5.0, 5.0], [8.0, 8.0, 4.0], [1.0, 1.0, 7.0]], [[4.0, 5.0, 8.0], [5.0, 2.0, 8.0], [1.0, 7.0, 2.0], [5.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 6.0], [8.0, 2.0, 3.0], [5.0, 8.0, 8.0], [1.0, 3.0, 2.0]], [[5.0, 4.0, 5.0], [2.0, 
7.0, 6.0], [8.0, 1.0, 8.0], [1.0, 8.0, 2.0]], [[6.0, 2.0, 5.0], [6.0, 5.0, 6.0], [2.0, 3.0, 7.0], [5.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_ULR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 2.0, 2.0], [3.0, 7.0, 7.0], [7.0, 3.0, 7.0], [8.0, 5.0, 7.0]], [[8.0, 5.0, 6.0], [6.0, 
4.0, 4.0], [5.0, 3.0, 2.0], [4.0, 8.0, 7.0]], [[1.0, 1.0, 2.0], [1.0, 5.0, 5.0], [5.0, 3.0, 1.0], [2.0, 6.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 7.0, 3.0], [5.0, 6.0, 1.0], [5.0, 2.0, 7.0], [6.0, 6.0, 3.0]], [[3.0, 3.0, 7.0], [8.0, 
8.0, 7.0], [1.0, 7.0, 2.0], [6.0, 7.0, 1.0]], [[3.0, 3.0, 2.0], [5.0, 2.0, 2.0], [3.0, 2.0, 1.0], [6.0, 3.0, 8.0]]])
      val=Data(numpy.array([[[8.0], [5.0]], [[6.0], [7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0], [7.0]], [[8.0], [4.0]]]))
      arg[:2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 2.0], [3.0, 7.0, 7.0], [7.0, 8.0, 7.0], [8.0, 5.0, 7.0]], [[8.0, 5.0, 6.0], [6.0, 
4.0, 4.0], [5.0, 6.0, 2.0], [4.0, 7.0, 7.0]], [[1.0, 1.0, 2.0], [1.0, 5.0, 5.0], [5.0, 3.0, 1.0], [2.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 3.0], [5.0, 6.0, 1.0], [5.0, 5.0, 7.0], [6.0, 7.0, 3.0]], [[3.0, 3.0, 7.0], [8.0, 
8.0, 7.0], [1.0, 8.0, 2.0], [6.0, 4.0, 1.0]], [[3.0, 3.0, 2.0], [5.0, 2.0, 2.0], [3.0, 2.0, 1.0], [6.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[7.0, 8.0, 1.0], [8.0, 6.0, 2.0], [5.0, 3.0, 5.0], [3.0, 8.0, 6.0]], [[3.0, 6.0, 5.0], [2.0, 
6.0, 3.0], [8.0, 8.0, 4.0], [6.0, 2.0, 5.0]], [[5.0, 8.0, 5.0], [5.0, 7.0, 7.0], [2.0, 1.0, 3.0], [6.0, 8.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 7.0, 1.0], [8.0, 3.0, 7.0], [7.0, 6.0, 6.0], [4.0, 3.0, 7.0]], [[1.0, 6.0, 1.0], [8.0, 
5.0, 7.0], [8.0, 7.0, 5.0], [2.0, 2.0, 4.0]], [[5.0, 5.0, 4.0], [3.0, 3.0, 4.0], [5.0, 5.0, 7.0], [2.0, 5.0, 7.0]]])
      val=Data(numpy.array([[8.0, 6.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 2.0, 7.0]]))
      arg[1:2,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 1.0], [8.0, 6.0, 2.0], [5.0, 3.0, 5.0], [3.0, 8.0, 6.0]], [[3.0, 6.0, 5.0], [2.0, 
6.0, 3.0], [8.0, 6.0, 8.0], [6.0, 2.0, 5.0]], [[5.0, 8.0, 5.0], [5.0, 7.0, 7.0], [2.0, 1.0, 3.0], [6.0, 8.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 7.0, 1.0], [8.0, 3.0, 7.0], [7.0, 6.0, 6.0], [4.0, 3.0, 7.0]], [[1.0, 6.0, 1.0], [8.0, 
5.0, 7.0], [5.0, 2.0, 7.0], [2.0, 2.0, 4.0]], [[5.0, 5.0, 4.0], [3.0, 3.0, 4.0], [5.0, 5.0, 7.0], [2.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_UCP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 6.0, 2.0], [3.0, 5.0, 5.0], [5.0, 3.0, 8.0], [3.0, 5.0, 3.0]], [[4.0, 8.0, 3.0], [3.0, 
6.0, 5.0], [5.0, 6.0, 4.0], [8.0, 8.0, 4.0]], [[2.0, 6.0, 4.0], [4.0, 6.0, 2.0], [2.0, 3.0, 1.0], [6.0, 2.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 3.0, 2.0], [8.0, 4.0, 1.0], [1.0, 1.0, 6.0], [1.0, 8.0, 1.0]], [[2.0, 7.0, 4.0], [4.0, 
7.0, 1.0], [7.0, 4.0, 5.0], [3.0, 1.0, 5.0]], [[3.0, 3.0, 5.0], [2.0, 6.0, 4.0], [2.0, 1.0, 3.0], [1.0, 7.0, 7.0]]])
      val=Data(numpy.array([[5.0, 1.0, 5.0, 2.0], [1.0, 5.0, 2.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 8.0, 3.0, 5.0], [6.0, 3.0, 6.0, 2.0]]))
      arg[:2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 2.0], [3.0, 1.0, 5.0], [5.0, 5.0, 8.0], [3.0, 2.0, 3.0]], [[4.0, 1.0, 3.0], [3.0, 
5.0, 5.0], [5.0, 2.0, 4.0], [8.0, 7.0, 4.0]], [[2.0, 6.0, 4.0], [4.0, 6.0, 2.0], [2.0, 3.0, 1.0], [6.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 2.0], [8.0, 8.0, 1.0], [1.0, 3.0, 6.0], [1.0, 5.0, 1.0]], [[2.0, 6.0, 4.0], [4.0, 
3.0, 1.0], [7.0, 6.0, 5.0], [3.0, 2.0, 5.0]], [[3.0, 3.0, 5.0], [2.0, 6.0, 4.0], [2.0, 1.0, 3.0], [1.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PPP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 1.0, 6.0], [4.0, 7.0, 5.0], [4.0, 7.0, 8.0], [1.0, 2.0, 1.0]], [[7.0, 6.0, 4.0], [4.0, 
4.0, 5.0], [6.0, 5.0, 2.0], [3.0, 2.0, 4.0]], [[7.0, 5.0, 2.0], [5.0, 3.0, 7.0], [1.0, 8.0, 7.0], [2.0, 3.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 8.0, 5.0], [4.0, 8.0, 4.0], [2.0, 4.0, 4.0], [5.0, 8.0, 1.0]], [[6.0, 4.0, 1.0], [8.0, 
2.0, 4.0], [5.0, 2.0, 7.0], [1.0, 8.0, 3.0]], [[5.0, 6.0, 5.0], [1.0, 1.0, 4.0], [6.0, 4.0, 2.0], [3.0, 1.0, 1.0]]])
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[1,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 6.0], [4.0, 7.0, 5.0], [4.0, 7.0, 8.0], [1.0, 2.0, 1.0]], [[7.0, 6.0, 4.0], [4.0, 
4.0, 5.0], [6.0, 1.0, 2.0], [3.0, 2.0, 4.0]], [[7.0, 5.0, 2.0], [5.0, 3.0, 7.0], [1.0, 8.0, 7.0], [2.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 5.0], [4.0, 8.0, 4.0], [2.0, 4.0, 4.0], [5.0, 8.0, 1.0]], [[6.0, 4.0, 1.0], [8.0, 
2.0, 4.0], [5.0, 1.0, 7.0], [1.0, 8.0, 3.0]], [[5.0, 6.0, 5.0], [1.0, 1.0, 4.0], [6.0, 4.0, 2.0], [3.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RRP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[1.0, 6.0, 8.0], [1.0, 2.0, 4.0], [2.0, 1.0, 3.0], [8.0, 8.0, 6.0]], [[3.0, 3.0, 7.0], [7.0, 
1.0, 8.0], [2.0, 1.0, 4.0], [7.0, 1.0, 3.0]], [[8.0, 4.0, 4.0], [3.0, 3.0, 8.0], [3.0, 3.0, 6.0], [2.0, 2.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 5.0, 7.0], [4.0, 7.0, 4.0], [7.0, 6.0, 8.0], [1.0, 4.0, 2.0]], [[2.0, 7.0, 4.0], [8.0, 
8.0, 6.0], [6.0, 1.0, 2.0], [6.0, 8.0, 3.0]], [[6.0, 8.0, 5.0], [8.0, 2.0, 6.0], [3.0, 1.0, 4.0], [3.0, 2.0, 5.0]]])
      val=Data(numpy.array([[1.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 4.0]]))
      arg[1:2,1:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 8.0], [1.0, 2.0, 4.0], [2.0, 1.0, 3.0], [8.0, 8.0, 6.0]], [[3.0, 3.0, 7.0], [7.0, 
1.0, 8.0], [2.0, 1.0, 4.0], [7.0, 1.0, 3.0]], [[8.0, 4.0, 4.0], [3.0, 3.0, 8.0], [3.0, 3.0, 6.0], [2.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 7.0], [4.0, 7.0, 4.0], [7.0, 6.0, 8.0], [1.0, 4.0, 2.0]], [[2.0, 7.0, 4.0], [8.0, 
5.0, 6.0], [6.0, 4.0, 2.0], [6.0, 8.0, 3.0]], [[6.0, 8.0, 5.0], [8.0, 2.0, 6.0], [3.0, 1.0, 4.0], [3.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[4.0, 3.0, 2.0], [6.0, 6.0, 7.0], [8.0, 7.0, 2.0], [2.0, 8.0, 3.0]], [[3.0, 6.0, 7.0], [3.0, 
2.0, 1.0], [6.0, 3.0, 2.0], [6.0, 1.0, 1.0]], [[1.0, 7.0, 3.0], [7.0, 6.0, 2.0], [8.0, 4.0, 7.0], [1.0, 5.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 1.0, 6.0], [7.0, 7.0, 3.0], [3.0, 3.0, 8.0], [6.0, 3.0, 4.0]], [[6.0, 8.0, 6.0], [6.0, 
4.0, 6.0], [6.0, 4.0, 8.0], [3.0, 7.0, 6.0]], [[2.0, 2.0, 5.0], [8.0, 3.0, 8.0], [1.0, 5.0, 3.0], [3.0, 5.0, 4.0]]])
      val=Data(numpy.array([[[6.0, 8.0, 5.0], [3.0, 7.0, 2.0], [7.0, 8.0, 1.0], [1.0, 1.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 1.0, 3.0], [1.0, 2.0, 1.0], [8.0, 3.0, 3.0], [4.0, 3.0, 4.0]]]))
      arg[2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 2.0], [6.0, 6.0, 7.0], [8.0, 7.0, 2.0], [2.0, 8.0, 3.0]], [[3.0, 6.0, 7.0], [3.0, 
2.0, 1.0], [6.0, 3.0, 2.0], [6.0, 1.0, 1.0]], [[6.0, 8.0, 5.0], [3.0, 7.0, 2.0], [7.0, 8.0, 1.0], [1.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 6.0], [7.0, 7.0, 3.0], [3.0, 3.0, 8.0], [6.0, 3.0, 4.0]], [[6.0, 8.0, 6.0], [6.0, 
4.0, 6.0], [6.0, 4.0, 8.0], [3.0, 7.0, 6.0]], [[8.0, 1.0, 3.0], [1.0, 2.0, 1.0], [8.0, 3.0, 3.0], [4.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LLL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 3.0, 4.0], [2.0, 8.0, 1.0], [8.0, 3.0, 4.0], [6.0, 6.0, 8.0]], [[7.0, 8.0, 5.0], [8.0, 
2.0, 1.0], [1.0, 2.0, 8.0], [4.0, 4.0, 5.0]], [[4.0, 2.0, 4.0], [8.0, 6.0, 1.0], [1.0, 3.0, 4.0], [6.0, 5.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 6.0, 6.0], [4.0, 2.0, 7.0], [7.0, 1.0, 2.0], [1.0, 3.0, 4.0]], [[4.0, 7.0, 6.0], [8.0, 
6.0, 6.0], [2.0, 3.0, 2.0], [5.0, 1.0, 3.0]], [[4.0, 2.0, 8.0], [5.0, 6.0, 7.0], [6.0, 5.0, 5.0], [7.0, 8.0, 5.0]]])
      val=Data(numpy.array([[[8.0], [3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0], [3.0]]]))
      arg[2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 4.0], [2.0, 8.0, 1.0], [8.0, 3.0, 4.0], [6.0, 6.0, 8.0]], [[7.0, 8.0, 5.0], [8.0, 
2.0, 1.0], [1.0, 2.0, 8.0], [4.0, 4.0, 5.0]], [[4.0, 2.0, 4.0], [8.0, 6.0, 1.0], [1.0, 3.0, 8.0], [6.0, 5.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 6.0], [4.0, 2.0, 7.0], [7.0, 1.0, 2.0], [1.0, 3.0, 4.0]], [[4.0, 7.0, 6.0], [8.0, 
6.0, 6.0], [2.0, 3.0, 2.0], [5.0, 1.0, 3.0]], [[4.0, 2.0, 8.0], [5.0, 6.0, 7.0], [6.0, 5.0, 5.0], [7.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[6.0, 5.0, 3.0], [3.0, 2.0, 2.0], [1.0, 4.0, 6.0], [6.0, 7.0, 3.0]], [[8.0, 6.0, 1.0], [4.0, 
1.0, 3.0], [3.0, 5.0, 7.0], [3.0, 8.0, 3.0]], [[5.0, 8.0, 4.0], [2.0, 6.0, 7.0], [4.0, 3.0, 6.0], [3.0, 8.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 5.0, 6.0], [6.0, 1.0, 7.0], [7.0, 8.0, 7.0], [8.0, 6.0, 8.0]], [[8.0, 1.0, 6.0], [7.0, 
2.0, 1.0], [2.0, 8.0, 8.0], [3.0, 5.0, 6.0]], [[8.0, 2.0, 8.0], [2.0, 8.0, 5.0], [6.0, 5.0, 1.0], [8.0, 2.0, 5.0]]])
      val=Data(numpy.array([[[1.0, 2.0, 6.0], [1.0, 5.0, 8.0], [5.0, 6.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 5.0, 7.0], [2.0, 1.0, 1.0], [3.0, 4.0, 7.0]]]))
      arg[2:,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 3.0], [3.0, 2.0, 2.0], [1.0, 4.0, 6.0], [6.0, 7.0, 3.0]], [[8.0, 6.0, 1.0], [4.0, 
1.0, 3.0], [3.0, 5.0, 7.0], [3.0, 8.0, 3.0]], [[1.0, 2.0, 6.0], [1.0, 5.0, 8.0], [5.0, 6.0, 8.0], [3.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 6.0], [6.0, 1.0, 7.0], [7.0, 8.0, 7.0], [8.0, 6.0, 8.0]], [[8.0, 1.0, 6.0], [7.0, 
2.0, 1.0], [2.0, 8.0, 8.0], [3.0, 5.0, 6.0]], [[4.0, 5.0, 7.0], [2.0, 1.0, 1.0], [3.0, 4.0, 7.0], [8.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_URN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 3.0, 1.0], [4.0, 5.0, 5.0], [1.0, 5.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 8.0, 3.0], [7.0, 
5.0, 5.0], [7.0, 4.0, 2.0], [2.0, 5.0, 5.0]], [[7.0, 5.0, 6.0], [1.0, 2.0, 8.0], [5.0, 1.0, 3.0], [6.0, 3.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 4.0, 8.0], [8.0, 8.0, 4.0], [6.0, 1.0, 3.0], [3.0, 1.0, 2.0]], [[5.0, 1.0, 2.0], [2.0, 
8.0, 1.0], [4.0, 6.0, 2.0], [4.0, 2.0, 7.0]], [[5.0, 8.0, 8.0], [1.0, 4.0, 1.0], [1.0, 2.0, 2.0], [2.0, 8.0, 1.0]]])
      val=Data(numpy.array([[[4.0, 5.0, 7.0], [6.0, 6.0, 8.0]], [[2.0, 8.0, 7.0], [6.0, 5.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 4.0, 6.0], [6.0, 5.0, 6.0]], [[6.0, 6.0, 7.0], [3.0, 2.0, 3.0]]]))
      arg[:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 1.0], [4.0, 5.0, 7.0], [6.0, 6.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 8.0, 3.0], [2.0, 
8.0, 7.0], [6.0, 5.0, 2.0], [2.0, 5.0, 5.0]], [[7.0, 5.0, 6.0], [1.0, 2.0, 8.0], [5.0, 1.0, 3.0], [6.0, 3.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 8.0], [8.0, 4.0, 6.0], [6.0, 5.0, 6.0], [3.0, 1.0, 2.0]], [[5.0, 1.0, 2.0], [6.0, 
6.0, 7.0], [3.0, 2.0, 3.0], [4.0, 2.0, 7.0]], [[5.0, 8.0, 8.0], [1.0, 4.0, 1.0], [1.0, 2.0, 2.0], [2.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LRL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[4.0, 8.0, 5.0], [3.0, 5.0, 7.0], [4.0, 5.0, 1.0], [6.0, 8.0, 5.0]], [[5.0, 4.0, 2.0], [2.0, 
6.0, 4.0], [4.0, 1.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 1.0, 3.0], [6.0, 1.0, 6.0], [4.0, 8.0, 1.0], [6.0, 8.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 1.0, 1.0], [2.0, 4.0, 5.0], [1.0, 5.0, 6.0], [7.0, 1.0, 8.0]], [[2.0, 7.0, 1.0], [5.0, 
3.0, 7.0], [7.0, 2.0, 2.0], [3.0, 3.0, 5.0]], [[4.0, 8.0, 6.0], [1.0, 2.0, 1.0], [1.0, 3.0, 8.0], [8.0, 1.0, 6.0]]])
      val=Data(numpy.array([[[5.0], [7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0], [1.0]]]))
      arg[2:,1:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 5.0], [3.0, 5.0, 7.0], [4.0, 5.0, 1.0], [6.0, 8.0, 5.0]], [[5.0, 4.0, 2.0], [2.0, 
6.0, 4.0], [4.0, 1.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 1.0, 3.0], [6.0, 1.0, 5.0], [4.0, 8.0, 7.0], [6.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 1.0], [2.0, 4.0, 5.0], [1.0, 5.0, 6.0], [7.0, 1.0, 8.0]], [[2.0, 7.0, 1.0], [5.0, 
3.0, 7.0], [7.0, 2.0, 2.0], [3.0, 3.0, 5.0]], [[4.0, 8.0, 6.0], [1.0, 2.0, 5.0], [1.0, 3.0, 1.0], [8.0, 1.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CPL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[8.0, 4.0, 1.0], [6.0, 1.0, 5.0], [3.0, 5.0, 5.0], [6.0, 6.0, 5.0]], [[3.0, 6.0, 2.0], [3.0, 
6.0, 1.0], [4.0, 8.0, 5.0], [3.0, 6.0, 8.0]], [[1.0, 7.0, 3.0], [3.0, 4.0, 7.0], [8.0, 2.0, 7.0], [5.0, 8.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 2.0, 7.0], [5.0, 2.0, 2.0], [2.0, 3.0, 4.0], [8.0, 5.0, 5.0]], [[7.0, 8.0, 6.0], [6.0, 
8.0, 4.0], [6.0, 2.0, 4.0], [3.0, 8.0, 1.0]], [[2.0, 5.0, 3.0], [5.0, 3.0, 4.0], [6.0, 7.0, 5.0], [8.0, 1.0, 1.0]]])
      val=Data(numpy.array([[3.0], [8.0], [1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0], [7.0], [8.0]]))
      arg[:,2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 1.0], [6.0, 1.0, 5.0], [3.0, 5.0, 3.0], [6.0, 6.0, 5.0]], [[3.0, 6.0, 2.0], [3.0, 
6.0, 1.0], [4.0, 8.0, 8.0], [3.0, 6.0, 8.0]], [[1.0, 7.0, 3.0], [3.0, 4.0, 7.0], [8.0, 2.0, 1.0], [5.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 7.0], [5.0, 2.0, 2.0], [2.0, 3.0, 1.0], [8.0, 5.0, 5.0]], [[7.0, 8.0, 6.0], [6.0, 
8.0, 4.0], [6.0, 2.0, 7.0], [3.0, 8.0, 1.0]], [[2.0, 5.0, 3.0], [5.0, 3.0, 4.0], [6.0, 7.0, 8.0], [8.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 4.0, 5.0], [2.0, 3.0, 5.0], [6.0, 5.0, 5.0], [7.0, 1.0, 1.0]], [[2.0, 1.0, 4.0], [5.0, 
2.0, 6.0], [1.0, 1.0, 1.0], [7.0, 3.0, 5.0]], [[5.0, 6.0, 6.0], [5.0, 1.0, 4.0], [2.0, 6.0, 5.0], [6.0, 2.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 2.0, 1.0], [3.0, 7.0, 5.0], [7.0, 1.0, 6.0], [3.0, 7.0, 6.0]], [[1.0, 7.0, 4.0], [8.0, 
3.0, 2.0], [1.0, 8.0, 8.0], [8.0, 3.0, 8.0]], [[5.0, 3.0, 8.0], [6.0, 2.0, 7.0], [4.0, 7.0, 7.0], [8.0, 1.0, 1.0]]])
      val=Data(numpy.array([[6.0, 5.0, 7.0], [5.0, 3.0, 7.0], [7.0, 4.0, 7.0], [8.0, 8.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 1.0, 5.0], [6.0, 3.0, 2.0], [8.0, 5.0, 7.0], [3.0, 3.0, 2.0]]))
      arg[1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 5.0], [2.0, 3.0, 5.0], [6.0, 5.0, 5.0], [7.0, 1.0, 1.0]], [[6.0, 5.0, 7.0], [5.0, 
3.0, 7.0], [7.0, 4.0, 7.0], [8.0, 8.0, 3.0]], [[5.0, 6.0, 6.0], [5.0, 1.0, 4.0], [2.0, 6.0, 5.0], [6.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 1.0], [3.0, 7.0, 5.0], [7.0, 1.0, 6.0], [3.0, 7.0, 6.0]], [[1.0, 1.0, 5.0], [6.0, 
3.0, 2.0], [8.0, 5.0, 7.0], [3.0, 3.0, 2.0]], [[5.0, 3.0, 8.0], [6.0, 2.0, 7.0], [4.0, 7.0, 7.0], [8.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[7.0, 8.0, 2.0], [3.0, 3.0, 8.0], [6.0, 8.0, 8.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 8.0], [2.0, 
1.0, 2.0], [7.0, 1.0, 6.0], [3.0, 7.0, 1.0]], [[2.0, 3.0, 3.0], [8.0, 8.0, 3.0], [4.0, 5.0, 8.0], [6.0, 8.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 3.0, 3.0], [7.0, 6.0, 1.0], [6.0, 8.0, 3.0], [8.0, 1.0, 6.0]], [[2.0, 6.0, 1.0], [6.0, 
6.0, 5.0], [7.0, 4.0, 6.0], [8.0, 2.0, 3.0]], [[8.0, 1.0, 2.0], [8.0, 4.0, 7.0], [5.0, 7.0, 2.0], [8.0, 8.0, 6.0]]])
      val=Data(numpy.array([[[3.0, 3.0, 3.0], [2.0, 3.0, 7.0], [2.0, 2.0, 4.0], [7.0, 3.0, 7.0]], [[5.0, 6.0, 2.0], [8.0, 5.0, 
3.0], [6.0, 7.0, 8.0], [2.0, 5.0, 1.0]], [[8.0, 6.0, 8.0], [6.0, 2.0, 4.0], [1.0, 5.0, 1.0], [8.0, 6.0, 
5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0, 6.0], [6.0, 7.0, 8.0], [2.0, 7.0, 4.0], [5.0, 2.0, 4.0]], [[4.0, 3.0, 8.0], 
[1.0, 2.0, 3.0], [8.0, 6.0, 5.0], [8.0, 7.0, 4.0]], [[3.0, 1.0, 4.0], [4.0, 2.0, 7.0], [7.0, 3.0, 8.0], [2.0, 1.0, 8.0]]]))
      arg[:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 3.0], [2.0, 3.0, 7.0], [2.0, 2.0, 4.0], [7.0, 3.0, 7.0]], [[5.0, 6.0, 2.0], [8.0, 
5.0, 3.0], [6.0, 7.0, 8.0], [2.0, 5.0, 1.0]], [[8.0, 6.0, 8.0], [6.0, 2.0, 4.0], [1.0, 5.0, 1.0], [8.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 6.0], [6.0, 7.0, 8.0], [2.0, 7.0, 4.0], [5.0, 2.0, 4.0]], [[4.0, 3.0, 8.0], [1.0, 
2.0, 3.0], [8.0, 6.0, 5.0], [8.0, 7.0, 4.0]], [[3.0, 1.0, 4.0], [4.0, 2.0, 7.0], [7.0, 3.0, 8.0], [2.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_UCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[7.0, 7.0, 4.0], [4.0, 7.0, 4.0], [3.0, 5.0, 8.0], [3.0, 2.0, 3.0]], [[6.0, 6.0, 2.0], [8.0, 
2.0, 4.0], [2.0, 7.0, 5.0], [3.0, 8.0, 1.0]], [[1.0, 8.0, 8.0], [2.0, 2.0, 6.0], [6.0, 6.0, 2.0], [7.0, 2.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 6.0], [7.0, 5.0, 8.0], [8.0, 6.0, 2.0], [4.0, 6.0, 3.0]], [[4.0, 5.0, 3.0], [1.0, 
6.0, 8.0], [2.0, 2.0, 7.0], [7.0, 3.0, 4.0]], [[3.0, 1.0, 3.0], [1.0, 8.0, 1.0], [1.0, 8.0, 8.0], [8.0, 5.0, 5.0]]])
      val=Data(numpy.array([[[5.0, 5.0, 8.0], [5.0, 5.0, 6.0], [4.0, 1.0, 6.0], [7.0, 7.0, 7.0]], [[6.0, 7.0, 8.0], [5.0, 4.0, 
3.0], [2.0, 2.0, 3.0], [7.0, 4.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 4.0, 2.0], [3.0, 6.0, 7.0], [2.0, 1.0, 2.0], [3.0, 3.0, 6.0]], [[2.0, 7.0, 1.0], 
[7.0, 2.0, 1.0], [4.0, 2.0, 4.0], [7.0, 1.0, 2.0]]]))
      arg[:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 5.0, 8.0], [5.0, 5.0, 6.0], [4.0, 1.0, 6.0], [7.0, 7.0, 7.0]], [[6.0, 7.0, 8.0], [5.0, 
4.0, 3.0], [2.0, 2.0, 3.0], [7.0, 4.0, 6.0]], [[1.0, 8.0, 8.0], [2.0, 2.0, 6.0], [6.0, 6.0, 2.0], [7.0, 2.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 2.0], [3.0, 6.0, 7.0], [2.0, 1.0, 2.0], [3.0, 3.0, 6.0]], [[2.0, 7.0, 1.0], [7.0, 
2.0, 1.0], [4.0, 2.0, 4.0], [7.0, 1.0, 2.0]], [[3.0, 1.0, 3.0], [1.0, 8.0, 1.0], [1.0, 8.0, 8.0], [8.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_UUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[8.0, 5.0, 4.0], [8.0, 5.0, 6.0], [6.0, 3.0, 5.0], [4.0, 2.0, 1.0]], [[2.0, 8.0, 6.0], [8.0, 
3.0, 7.0], [7.0, 7.0, 2.0], [2.0, 3.0, 5.0]], [[4.0, 7.0, 5.0], [1.0, 8.0, 2.0], [8.0, 3.0, 8.0], [8.0, 1.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 5.0, 3.0], [5.0, 2.0, 2.0], [8.0, 7.0, 6.0], [6.0, 4.0, 7.0]], [[7.0, 3.0, 7.0], [8.0, 
4.0, 5.0], [2.0, 8.0, 7.0], [1.0, 7.0, 7.0]], [[8.0, 6.0, 6.0], [1.0, 2.0, 8.0], [3.0, 3.0, 1.0], [2.0, 5.0, 8.0]]])
      val=Data(numpy.array([[[8.0, 6.0, 6.0], [6.0, 1.0, 1.0], [6.0, 1.0, 1.0]], [[5.0, 5.0, 1.0], [2.0, 8.0, 8.0], [8.0, 3.0, 
7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 2.0, 7.0], [7.0, 6.0, 8.0], [3.0, 8.0, 4.0]], [[6.0, 1.0, 4.0], [7.0, 4.0, 3.0], 
[7.0, 3.0, 1.0]]]))
      arg[:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 6.0], [6.0, 1.0, 1.0], [6.0, 1.0, 1.0], [4.0, 2.0, 1.0]], [[5.0, 5.0, 1.0], [2.0, 
8.0, 8.0], [8.0, 3.0, 7.0], [2.0, 3.0, 5.0]], [[4.0, 7.0, 5.0], [1.0, 8.0, 2.0], [8.0, 3.0, 8.0], [8.0, 1.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 7.0], [7.0, 6.0, 8.0], [3.0, 8.0, 4.0], [6.0, 4.0, 7.0]], [[6.0, 1.0, 4.0], [7.0, 
4.0, 3.0], [7.0, 3.0, 1.0], [1.0, 7.0, 7.0]], [[8.0, 6.0, 6.0], [1.0, 2.0, 8.0], [3.0, 3.0, 1.0], [2.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 4.0, 8.0], [2.0, 5.0, 1.0], [6.0, 1.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 4.0, 4.0], [4.0, 
1.0, 3.0], [3.0, 2.0, 5.0], [2.0, 4.0, 2.0]], [[8.0, 3.0, 2.0], [2.0, 1.0, 5.0], [5.0, 7.0, 3.0], [4.0, 3.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 3.0, 6.0], [7.0, 6.0, 7.0], [2.0, 3.0, 6.0], [5.0, 4.0, 7.0]], [[7.0, 4.0, 5.0], [4.0, 
8.0, 8.0], [1.0, 4.0, 8.0], [4.0, 3.0, 2.0]], [[6.0, 6.0, 7.0], [4.0, 6.0, 1.0], [7.0, 3.0, 3.0], [8.0, 5.0, 7.0]]])
      val=Data(numpy.array([[[6.0, 2.0, 1.0], [3.0, 7.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 6.0, 4.0], [2.0, 8.0, 4.0]]]))
      arg[2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 8.0], [2.0, 5.0, 1.0], [6.0, 1.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 4.0, 4.0], [4.0, 
1.0, 3.0], [3.0, 2.0, 5.0], [2.0, 4.0, 2.0]], [[8.0, 3.0, 2.0], [2.0, 1.0, 5.0], [6.0, 2.0, 1.0], [3.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 6.0], [7.0, 6.0, 7.0], [2.0, 3.0, 6.0], [5.0, 4.0, 7.0]], [[7.0, 4.0, 5.0], [4.0, 
8.0, 8.0], [1.0, 4.0, 8.0], [4.0, 3.0, 2.0]], [[6.0, 6.0, 7.0], [4.0, 6.0, 1.0], [7.0, 6.0, 4.0], [2.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RPC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[6.0, 7.0, 6.0], [7.0, 3.0, 2.0], [1.0, 5.0, 4.0], [8.0, 6.0, 6.0]], [[6.0, 8.0, 7.0], [7.0, 
7.0, 3.0], [3.0, 6.0, 4.0], [7.0, 3.0, 5.0]], [[1.0, 3.0, 3.0], [1.0, 2.0, 6.0], [5.0, 5.0, 5.0], [7.0, 1.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 7.0, 7.0], [4.0, 4.0, 1.0], [4.0, 6.0, 1.0], [1.0, 5.0, 5.0]], [[7.0, 7.0, 6.0], [2.0, 
3.0, 6.0], [4.0, 1.0, 1.0], [2.0, 1.0, 7.0]], [[6.0, 8.0, 6.0], [3.0, 2.0, 4.0], [7.0, 7.0, 4.0], [2.0, 2.0, 6.0]]])
      val=Data(numpy.array([[1.0, 5.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 4.0, 7.0]]))
      arg[1:2,2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 6.0], [7.0, 3.0, 2.0], [1.0, 5.0, 4.0], [8.0, 6.0, 6.0]], [[6.0, 8.0, 7.0], [7.0, 
7.0, 3.0], [1.0, 5.0, 5.0], [7.0, 3.0, 5.0]], [[1.0, 3.0, 3.0], [1.0, 2.0, 6.0], [5.0, 5.0, 5.0], [7.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 7.0], [4.0, 4.0, 1.0], [4.0, 6.0, 1.0], [1.0, 5.0, 5.0]], [[7.0, 7.0, 6.0], [2.0, 
3.0, 6.0], [3.0, 4.0, 7.0], [2.0, 1.0, 7.0]], [[6.0, 8.0, 6.0], [3.0, 2.0, 4.0], [7.0, 7.0, 4.0], [2.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_URU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[6.0, 2.0, 4.0], [5.0, 8.0, 8.0], [3.0, 4.0, 8.0], [8.0, 8.0, 1.0]], [[6.0, 8.0, 1.0], [3.0, 
8.0, 4.0], [8.0, 2.0, 3.0], [2.0, 4.0, 4.0]], [[6.0, 8.0, 8.0], [6.0, 1.0, 5.0], [8.0, 6.0, 2.0], [3.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 8.0, 5.0], [3.0, 5.0, 6.0], [5.0, 1.0, 3.0], [5.0, 6.0, 2.0]], [[8.0, 4.0, 6.0], [4.0, 
2.0, 7.0], [4.0, 7.0, 8.0], [2.0, 6.0, 6.0]], [[6.0, 1.0, 7.0], [8.0, 6.0, 5.0], [6.0, 5.0, 3.0], [4.0, 3.0, 5.0]]])
      val=Data(numpy.array([[[3.0, 6.0], [5.0, 3.0]], [[8.0, 8.0], [7.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 6.0], [2.0, 8.0]], [[8.0, 2.0], [6.0, 8.0]]]))
      arg[:2,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 4.0], [3.0, 6.0, 8.0], [5.0, 3.0, 8.0], [8.0, 8.0, 1.0]], [[6.0, 8.0, 1.0], [8.0, 
8.0, 4.0], [7.0, 7.0, 3.0], [2.0, 4.0, 4.0]], [[6.0, 8.0, 8.0], [6.0, 1.0, 5.0], [8.0, 6.0, 2.0], [3.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 5.0], [1.0, 6.0, 6.0], [2.0, 8.0, 3.0], [5.0, 6.0, 2.0]], [[8.0, 4.0, 6.0], [8.0, 
2.0, 7.0], [6.0, 8.0, 8.0], [2.0, 6.0, 6.0]], [[6.0, 1.0, 7.0], [8.0, 6.0, 5.0], [6.0, 5.0, 3.0], [4.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[6.0, 7.0, 6.0], [4.0, 6.0, 8.0], [1.0, 1.0, 8.0], [2.0, 2.0, 5.0]], [[8.0, 5.0, 7.0], [7.0, 
1.0, 5.0], [8.0, 2.0, 7.0], [7.0, 8.0, 7.0]], [[1.0, 7.0, 5.0], [8.0, 6.0, 6.0], [7.0, 3.0, 8.0], [1.0, 6.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 8.0, 1.0], [8.0, 1.0, 8.0], [6.0, 3.0, 6.0], [6.0, 1.0, 4.0]], [[1.0, 7.0, 5.0], [1.0, 
4.0, 6.0], [2.0, 6.0, 1.0], [4.0, 8.0, 5.0]], [[1.0, 3.0, 7.0], [4.0, 1.0, 6.0], [4.0, 3.0, 3.0], [6.0, 5.0, 2.0]]])
      val=Data(numpy.array([[[8.0], [7.0]], [[4.0], [8.0]], [[2.0], [2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0], [5.0]], [[8.0], [7.0]], [[2.0], [7.0]]]))
      arg[:,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 6.0], [4.0, 8.0, 8.0], [1.0, 7.0, 8.0], [2.0, 2.0, 5.0]], [[8.0, 5.0, 7.0], [7.0, 
4.0, 5.0], [8.0, 8.0, 7.0], [7.0, 8.0, 7.0]], [[1.0, 7.0, 5.0], [8.0, 2.0, 6.0], [7.0, 2.0, 8.0], [1.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 1.0], [8.0, 6.0, 8.0], [6.0, 5.0, 6.0], [6.0, 1.0, 4.0]], [[1.0, 7.0, 5.0], [1.0, 
8.0, 6.0], [2.0, 7.0, 1.0], [4.0, 8.0, 5.0]], [[1.0, 3.0, 7.0], [4.0, 2.0, 6.0], [4.0, 7.0, 3.0], [6.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 7.0, 8.0], [5.0, 3.0, 1.0], [6.0, 6.0, 6.0], [5.0, 5.0, 3.0]], [[6.0, 8.0, 1.0], [8.0, 
5.0, 6.0], [5.0, 1.0, 5.0], [4.0, 2.0, 4.0]], [[2.0, 1.0, 5.0], [3.0, 8.0, 2.0], [8.0, 6.0, 8.0], [3.0, 1.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 5.0, 8.0], [8.0, 1.0, 1.0], [5.0, 8.0, 1.0], [6.0, 1.0, 6.0]], [[8.0, 3.0, 2.0], [3.0, 
2.0, 5.0], [6.0, 6.0, 4.0], [4.0, 1.0, 3.0]], [[2.0, 5.0, 1.0], [3.0, 3.0, 5.0], [7.0, 6.0, 1.0], [5.0, 2.0, 6.0]]])
      val=Data(numpy.array([[6.0, 7.0, 2.0], [3.0, 1.0, 3.0], [5.0, 2.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 2.0, 1.0], [6.0, 5.0, 3.0], [7.0, 7.0, 6.0]]))
      arg[1,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 8.0], [5.0, 3.0, 1.0], [6.0, 6.0, 6.0], [5.0, 5.0, 3.0]], [[6.0, 7.0, 2.0], [3.0, 
1.0, 3.0], [5.0, 2.0, 6.0], [4.0, 2.0, 4.0]], [[2.0, 1.0, 5.0], [3.0, 8.0, 2.0], [8.0, 6.0, 8.0], [3.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 8.0], [8.0, 1.0, 1.0], [5.0, 8.0, 1.0], [6.0, 1.0, 6.0]], [[8.0, 2.0, 1.0], [6.0, 
5.0, 3.0], [7.0, 7.0, 6.0], [4.0, 1.0, 3.0]], [[2.0, 5.0, 1.0], [3.0, 3.0, 5.0], [7.0, 6.0, 1.0], [5.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[6.0, 2.0, 3.0], [7.0, 2.0, 2.0], [8.0, 1.0, 8.0], [6.0, 6.0, 8.0]], [[7.0, 5.0, 8.0], [5.0, 
5.0, 3.0], [1.0, 5.0, 7.0], [4.0, 1.0, 7.0]], [[7.0, 2.0, 2.0], [6.0, 2.0, 1.0], [3.0, 8.0, 5.0], [8.0, 2.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 7.0, 3.0], [3.0, 1.0, 1.0], [2.0, 1.0, 2.0], [8.0, 6.0, 5.0]], [[7.0, 1.0, 4.0], [6.0, 
3.0, 5.0], [8.0, 4.0, 3.0], [5.0, 7.0, 6.0]], [[8.0, 3.0, 3.0], [6.0, 5.0, 3.0], [4.0, 3.0, 5.0], [4.0, 6.0, 5.0]]])
      val=Data(numpy.array([[1.0, 2.0, 3.0], [6.0, 2.0, 3.0], [2.0, 2.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 7.0, 3.0], [5.0, 5.0, 1.0], [4.0, 4.0, 2.0]]))
      arg[:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 3.0], [7.0, 2.0, 2.0], [1.0, 2.0, 3.0], [6.0, 6.0, 8.0]], [[7.0, 5.0, 8.0], [5.0, 
5.0, 3.0], [6.0, 2.0, 3.0], [4.0, 1.0, 7.0]], [[7.0, 2.0, 2.0], [6.0, 2.0, 1.0], [2.0, 2.0, 6.0], [8.0, 2.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 3.0], [3.0, 1.0, 1.0], [8.0, 7.0, 3.0], [8.0, 6.0, 5.0]], [[7.0, 1.0, 4.0], [6.0, 
3.0, 5.0], [5.0, 5.0, 1.0], [5.0, 7.0, 6.0]], [[8.0, 3.0, 3.0], [6.0, 5.0, 3.0], [4.0, 4.0, 2.0], [4.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_UUP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[8.0, 6.0, 1.0], [5.0, 1.0, 4.0], [3.0, 4.0, 5.0], [2.0, 6.0, 3.0]], [[7.0, 2.0, 6.0], [1.0, 
5.0, 4.0], [8.0, 4.0, 2.0], [3.0, 5.0, 7.0]], [[2.0, 6.0, 3.0], [4.0, 3.0, 5.0], [8.0, 1.0, 6.0], [5.0, 3.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 2.0], [6.0, 6.0, 2.0], [7.0, 5.0, 7.0], [1.0, 1.0, 6.0]], [[3.0, 4.0, 5.0], [6.0, 
7.0, 7.0], [5.0, 7.0, 8.0], [8.0, 8.0, 8.0]], [[5.0, 2.0, 4.0], [1.0, 3.0, 5.0], [2.0, 7.0, 1.0], [2.0, 3.0, 5.0]]])
      val=Data(numpy.array([[5.0, 6.0, 3.0], [1.0, 5.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 2.0, 3.0], [7.0, 4.0, 7.0]]))
      arg[:2,:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 1.0], [5.0, 6.0, 4.0], [3.0, 3.0, 5.0], [2.0, 6.0, 3.0]], [[7.0, 1.0, 6.0], [1.0, 
5.0, 4.0], [8.0, 4.0, 2.0], [3.0, 5.0, 7.0]], [[2.0, 6.0, 3.0], [4.0, 3.0, 5.0], [8.0, 1.0, 6.0], [5.0, 3.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 6.0, 2.0], [6.0, 2.0, 2.0], [7.0, 3.0, 7.0], [1.0, 1.0, 6.0]], [[3.0, 7.0, 5.0], [6.0, 
4.0, 7.0], [5.0, 7.0, 8.0], [8.0, 8.0, 8.0]], [[5.0, 2.0, 4.0], [1.0, 3.0, 5.0], [2.0, 7.0, 1.0], [2.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_UCL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[7.0, 3.0, 8.0], [2.0, 6.0, 6.0], [1.0, 8.0, 4.0], [4.0, 8.0, 7.0]], [[3.0, 4.0, 1.0], [4.0, 
6.0, 5.0], [3.0, 1.0, 4.0], [1.0, 1.0, 5.0]], [[6.0, 2.0, 5.0], [3.0, 7.0, 7.0], [2.0, 8.0, 3.0], [5.0, 7.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 4.0, 4.0], [7.0, 6.0, 8.0], [4.0, 3.0, 4.0], [5.0, 4.0, 8.0]], [[3.0, 1.0, 3.0], [1.0, 
6.0, 7.0], [1.0, 3.0, 8.0], [1.0, 3.0, 1.0]], [[4.0, 3.0, 1.0], [2.0, 2.0, 8.0], [7.0, 1.0, 2.0], [3.0, 3.0, 1.0]]])
      val=Data(numpy.array([[[2.0], [2.0], [3.0], [8.0]], [[7.0], [8.0], [2.0], [8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0], [2.0], [8.0], [7.0]], [[5.0], [6.0], [3.0], [4.0]]]))
      arg[:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 2.0], [2.0, 6.0, 2.0], [1.0, 8.0, 3.0], [4.0, 8.0, 8.0]], [[3.0, 4.0, 7.0], [4.0, 
6.0, 8.0], [3.0, 1.0, 2.0], [1.0, 1.0, 8.0]], [[6.0, 2.0, 5.0], [3.0, 7.0, 7.0], [2.0, 8.0, 3.0], [5.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 2.0], [7.0, 6.0, 2.0], [4.0, 3.0, 8.0], [5.0, 4.0, 7.0]], [[3.0, 1.0, 5.0], [1.0, 
6.0, 6.0], [1.0, 3.0, 3.0], [1.0, 3.0, 4.0]], [[4.0, 3.0, 1.0], [2.0, 2.0, 8.0], [7.0, 1.0, 2.0], [3.0, 3.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 6.0, 6.0], [8.0, 4.0, 6.0], [3.0, 3.0, 2.0], [1.0, 3.0, 1.0]], [[6.0, 1.0, 1.0], [7.0, 
4.0, 8.0], [7.0, 7.0, 8.0], [2.0, 2.0, 6.0]], [[2.0, 6.0, 1.0], [2.0, 8.0, 6.0], [2.0, 4.0, 6.0], [3.0, 8.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 7.0], [8.0, 4.0, 1.0], [6.0, 2.0, 3.0], [1.0, 8.0, 6.0]], [[1.0, 3.0, 8.0], [4.0, 
1.0, 4.0], [7.0, 5.0, 6.0], [8.0, 4.0, 8.0]], [[2.0, 4.0, 4.0], [1.0, 1.0, 3.0], [1.0, 7.0, 1.0], [4.0, 7.0, 6.0]]])
      val=Data(numpy.array([[[5.0, 7.0, 7.0], [6.0, 7.0, 2.0], [3.0, 8.0, 5.0], [5.0, 8.0, 3.0]], [[6.0, 4.0, 6.0], [5.0, 3.0, 
2.0], [5.0, 6.0, 7.0], [4.0, 3.0, 6.0]], [[8.0, 7.0, 8.0], [7.0, 5.0, 5.0], [2.0, 3.0, 3.0], [2.0, 4.0, 
8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 4.0, 8.0], [4.0, 1.0, 3.0], [7.0, 8.0, 5.0], [5.0, 7.0, 6.0]], [[4.0, 7.0, 3.0], 
[1.0, 1.0, 8.0], [2.0, 7.0, 7.0], [1.0, 4.0, 2.0]], [[3.0, 1.0, 4.0], [2.0, 7.0, 2.0], [2.0, 2.0, 5.0], [2.0, 2.0, 4.0]]]))
      arg[:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 7.0], [6.0, 7.0, 2.0], [3.0, 8.0, 5.0], [5.0, 8.0, 3.0]], [[6.0, 4.0, 6.0], [5.0, 
3.0, 2.0], [5.0, 6.0, 7.0], [4.0, 3.0, 6.0]], [[8.0, 7.0, 8.0], [7.0, 5.0, 5.0], [2.0, 3.0, 3.0], [2.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 8.0], [4.0, 1.0, 3.0], [7.0, 8.0, 5.0], [5.0, 7.0, 6.0]], [[4.0, 7.0, 3.0], [1.0, 
1.0, 8.0], [2.0, 7.0, 7.0], [1.0, 4.0, 2.0]], [[3.0, 1.0, 4.0], [2.0, 7.0, 2.0], [2.0, 2.0, 5.0], [2.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 8.0, 7.0], [8.0, 4.0, 4.0], [4.0, 4.0, 5.0], [8.0, 3.0, 5.0]], [[5.0, 4.0, 6.0], [7.0, 
2.0, 8.0], [4.0, 7.0, 1.0], [8.0, 3.0, 7.0]], [[4.0, 8.0, 1.0], [1.0, 7.0, 1.0], [5.0, 1.0, 5.0], [1.0, 3.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 6.0, 7.0], [3.0, 6.0, 5.0], [3.0, 6.0, 3.0], [2.0, 2.0, 4.0]], [[1.0, 3.0, 8.0], [1.0, 
2.0, 2.0], [6.0, 6.0, 5.0], [3.0, 6.0, 1.0]], [[8.0, 5.0, 5.0], [7.0, 3.0, 7.0], [6.0, 2.0, 6.0], [1.0, 1.0, 2.0]]])
      val=Data(numpy.array([[1.0, 6.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 3.0, 8.0]]))
      arg[2:,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 7.0], [8.0, 4.0, 4.0], [4.0, 4.0, 5.0], [8.0, 3.0, 5.0]], [[5.0, 4.0, 6.0], [7.0, 
2.0, 8.0], [4.0, 7.0, 1.0], [8.0, 3.0, 7.0]], [[4.0, 8.0, 1.0], [1.0, 7.0, 1.0], [1.0, 6.0, 4.0], [1.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 7.0], [3.0, 6.0, 5.0], [3.0, 6.0, 3.0], [2.0, 2.0, 4.0]], [[1.0, 3.0, 8.0], [1.0, 
2.0, 2.0], [6.0, 6.0, 5.0], [3.0, 6.0, 1.0]], [[8.0, 5.0, 5.0], [7.0, 3.0, 7.0], [1.0, 3.0, 8.0], [1.0, 1.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 4.0, 8.0], [6.0, 6.0, 6.0], [2.0, 3.0, 4.0], [6.0, 4.0, 3.0]], [[8.0, 5.0, 7.0], [3.0, 
7.0, 5.0], [6.0, 8.0, 8.0], [7.0, 7.0, 7.0]], [[2.0, 8.0, 3.0], [8.0, 4.0, 5.0], [1.0, 1.0, 4.0], [6.0, 2.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 6.0, 3.0], [4.0, 6.0, 5.0], [2.0, 1.0, 1.0], [1.0, 3.0, 2.0]], [[6.0, 1.0, 4.0], [5.0, 
8.0, 2.0], [8.0, 3.0, 1.0], [4.0, 5.0, 2.0]], [[1.0, 7.0, 2.0], [5.0, 3.0, 5.0], [7.0, 3.0, 3.0], [1.0, 2.0, 8.0]]])
      val=Data(numpy.array([[[1.0, 8.0, 5.0], [1.0, 1.0, 3.0], [1.0, 1.0, 1.0], [2.0, 6.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 2.0, 3.0], [1.0, 1.0, 3.0], [1.0, 4.0, 2.0], [4.0, 5.0, 1.0]]]))
      arg[1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 8.0], [6.0, 6.0, 6.0], [2.0, 3.0, 4.0], [6.0, 4.0, 3.0]], [[1.0, 8.0, 5.0], [1.0, 
1.0, 3.0], [1.0, 1.0, 1.0], [2.0, 6.0, 4.0]], [[2.0, 8.0, 3.0], [8.0, 4.0, 5.0], [1.0, 1.0, 4.0], [6.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 6.0, 3.0], [4.0, 6.0, 5.0], [2.0, 1.0, 1.0], [1.0, 3.0, 2.0]], [[5.0, 2.0, 3.0], [1.0, 
1.0, 3.0], [1.0, 4.0, 2.0], [4.0, 5.0, 1.0]], [[1.0, 7.0, 2.0], [5.0, 3.0, 5.0], [7.0, 3.0, 3.0], [1.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 7.0, 6.0], [7.0, 3.0, 8.0], [5.0, 7.0, 5.0], [1.0, 1.0, 4.0]], [[7.0, 8.0, 4.0], [6.0, 
4.0, 7.0], [8.0, 3.0, 8.0], [6.0, 8.0, 3.0]], [[1.0, 3.0, 7.0], [4.0, 6.0, 1.0], [6.0, 3.0, 5.0], [6.0, 3.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 1.0, 7.0], [5.0, 7.0, 3.0], [3.0, 8.0, 6.0], [5.0, 3.0, 5.0]], [[2.0, 7.0, 1.0], [1.0, 
8.0, 3.0], [5.0, 2.0, 4.0], [6.0, 3.0, 8.0]], [[6.0, 5.0, 6.0], [1.0, 2.0, 6.0], [3.0, 2.0, 8.0], [6.0, 3.0, 7.0]]])
      val=Data(numpy.array([5.0, 6.0, 1.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([8.0, 1.0, 1.0]))
      arg[1,2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 7.0, 6.0], [7.0, 3.0, 8.0], [5.0, 7.0, 5.0], [1.0, 1.0, 4.0]], [[7.0, 8.0, 4.0], [6.0, 
4.0, 7.0], [5.0, 6.0, 1.0], [6.0, 8.0, 3.0]], [[1.0, 3.0, 7.0], [4.0, 6.0, 1.0], [6.0, 3.0, 5.0], [6.0, 3.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 7.0], [5.0, 7.0, 3.0], [3.0, 8.0, 6.0], [5.0, 3.0, 5.0]], [[2.0, 7.0, 1.0], [1.0, 
8.0, 3.0], [8.0, 1.0, 1.0], [6.0, 3.0, 8.0]], [[6.0, 5.0, 6.0], [1.0, 2.0, 6.0], [3.0, 2.0, 8.0], [6.0, 3.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LPR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 6.0, 7.0], [1.0, 2.0, 3.0], [4.0, 7.0, 6.0], [5.0, 7.0, 2.0]], [[8.0, 1.0, 1.0], [4.0, 
7.0, 2.0], [5.0, 7.0, 7.0], [1.0, 6.0, 4.0]], [[6.0, 1.0, 8.0], [8.0, 2.0, 2.0], [2.0, 7.0, 5.0], [7.0, 5.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 8.0, 8.0], [4.0, 8.0, 6.0], [2.0, 4.0, 6.0], [3.0, 6.0, 8.0]], [[8.0, 2.0, 6.0], [5.0, 
7.0, 1.0], [7.0, 1.0, 4.0], [3.0, 4.0, 1.0]], [[4.0, 8.0, 7.0], [6.0, 5.0, 5.0], [2.0, 4.0, 1.0], [2.0, 3.0, 3.0]]])
      val=Data(numpy.array([[6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0]]))
      arg[2:,2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 7.0], [1.0, 2.0, 3.0], [4.0, 7.0, 6.0], [5.0, 7.0, 2.0]], [[8.0, 1.0, 1.0], [4.0, 
7.0, 2.0], [5.0, 7.0, 7.0], [1.0, 6.0, 4.0]], [[6.0, 1.0, 8.0], [8.0, 2.0, 2.0], [2.0, 6.0, 5.0], [7.0, 5.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 8.0], [4.0, 8.0, 6.0], [2.0, 4.0, 6.0], [3.0, 6.0, 8.0]], [[8.0, 2.0, 6.0], [5.0, 
7.0, 1.0], [7.0, 1.0, 4.0], [3.0, 4.0, 1.0]], [[4.0, 8.0, 7.0], [6.0, 5.0, 5.0], [2.0, 5.0, 1.0], [2.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LLP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[4.0, 6.0, 2.0], [3.0, 5.0, 5.0], [6.0, 5.0, 3.0], [2.0, 7.0, 6.0]], [[2.0, 3.0, 4.0], [3.0, 
4.0, 7.0], [3.0, 6.0, 4.0], [4.0, 2.0, 2.0]], [[6.0, 5.0, 7.0], [4.0, 5.0, 1.0], [7.0, 4.0, 3.0], [5.0, 2.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 1.0, 5.0], [6.0, 8.0, 1.0], [7.0, 6.0, 2.0], [6.0, 3.0, 5.0]], [[6.0, 1.0, 8.0], [8.0, 
1.0, 2.0], [6.0, 1.0, 5.0], [2.0, 4.0, 2.0]], [[6.0, 8.0, 7.0], [4.0, 5.0, 6.0], [1.0, 2.0, 2.0], [2.0, 2.0, 4.0]]])
      val=Data(numpy.array([[7.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 4.0]]))
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 2.0], [3.0, 5.0, 5.0], [6.0, 5.0, 3.0], [2.0, 7.0, 6.0]], [[2.0, 3.0, 4.0], [3.0, 
4.0, 7.0], [3.0, 6.0, 4.0], [4.0, 2.0, 2.0]], [[6.0, 5.0, 7.0], [4.0, 5.0, 1.0], [7.0, 7.0, 3.0], [5.0, 7.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 5.0], [6.0, 8.0, 1.0], [7.0, 6.0, 2.0], [6.0, 3.0, 5.0]], [[6.0, 1.0, 8.0], [8.0, 
1.0, 2.0], [6.0, 1.0, 5.0], [2.0, 4.0, 2.0]], [[6.0, 8.0, 7.0], [4.0, 5.0, 6.0], [1.0, 3.0, 2.0], [2.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LPP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 5.0, 7.0], [3.0, 1.0, 5.0], [1.0, 8.0, 2.0], [7.0, 4.0, 8.0]], [[8.0, 7.0, 1.0], [5.0, 
1.0, 4.0], [6.0, 4.0, 2.0], [6.0, 8.0, 2.0]], [[4.0, 7.0, 7.0], [5.0, 3.0, 5.0], [5.0, 8.0, 2.0], [6.0, 7.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 8.0, 4.0], [3.0, 5.0, 3.0], [2.0, 2.0, 4.0], [6.0, 5.0, 6.0]], [[7.0, 7.0, 1.0], [2.0, 
7.0, 1.0], [7.0, 4.0, 6.0], [5.0, 2.0, 5.0]], [[4.0, 7.0, 4.0], [7.0, 3.0, 1.0], [7.0, 8.0, 2.0], [7.0, 2.0, 1.0]]])
      val=Data(numpy.array([2.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0]))
      arg[2:,2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 5.0, 7.0], [3.0, 1.0, 5.0], [1.0, 8.0, 2.0], [7.0, 4.0, 8.0]], [[8.0, 7.0, 1.0], [5.0, 
1.0, 4.0], [6.0, 4.0, 2.0], [6.0, 8.0, 2.0]], [[4.0, 7.0, 7.0], [5.0, 3.0, 5.0], [5.0, 2.0, 2.0], [6.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 4.0], [3.0, 5.0, 3.0], [2.0, 2.0, 4.0], [6.0, 5.0, 6.0]], [[7.0, 7.0, 1.0], [2.0, 
7.0, 1.0], [7.0, 4.0, 6.0], [5.0, 2.0, 5.0]], [[4.0, 7.0, 4.0], [7.0, 3.0, 1.0], [7.0, 3.0, 2.0], [7.0, 2.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[6.0, 5.0, 1.0], [3.0, 5.0, 3.0], [6.0, 6.0, 8.0], [8.0, 2.0, 1.0]], [[3.0, 5.0, 5.0], [7.0, 
4.0, 5.0], [3.0, 2.0, 1.0], [6.0, 5.0, 4.0]], [[2.0, 3.0, 4.0], [7.0, 6.0, 1.0], [5.0, 3.0, 7.0], [8.0, 3.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 1.0], [2.0, 6.0, 7.0], [5.0, 1.0, 3.0], [8.0, 1.0, 4.0]], [[3.0, 5.0, 6.0], [2.0, 
4.0, 3.0], [6.0, 4.0, 2.0], [6.0, 4.0, 1.0]], [[5.0, 1.0, 5.0], [2.0, 3.0, 2.0], [1.0, 8.0, 4.0], [6.0, 5.0, 5.0]]])
      val=Data(numpy.array([[[3.0, 3.0, 1.0], [4.0, 4.0, 4.0], [5.0, 6.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 6.0, 6.0], [5.0, 4.0, 2.0], [2.0, 1.0, 1.0]]]))
      arg[1:2,:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 1.0], [3.0, 5.0, 3.0], [6.0, 6.0, 8.0], [8.0, 2.0, 1.0]], [[3.0, 3.0, 1.0], [4.0, 
4.0, 4.0], [5.0, 6.0, 3.0], [6.0, 5.0, 4.0]], [[2.0, 3.0, 4.0], [7.0, 6.0, 1.0], [5.0, 3.0, 7.0], [8.0, 3.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 1.0], [2.0, 6.0, 7.0], [5.0, 1.0, 3.0], [8.0, 1.0, 4.0]], [[1.0, 6.0, 6.0], [5.0, 
4.0, 2.0], [2.0, 1.0, 1.0], [6.0, 4.0, 1.0]], [[5.0, 1.0, 5.0], [2.0, 3.0, 2.0], [1.0, 8.0, 4.0], [6.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_ULN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 2.0, 3.0], [6.0, 7.0, 4.0], [2.0, 3.0, 3.0], [2.0, 4.0, 6.0]], [[3.0, 4.0, 3.0], [3.0, 
5.0, 6.0], [2.0, 6.0, 7.0], [6.0, 4.0, 5.0]], [[2.0, 7.0, 4.0], [1.0, 1.0, 5.0], [4.0, 5.0, 2.0], [2.0, 8.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 1.0, 4.0], [3.0, 8.0, 6.0], [7.0, 5.0, 2.0], [1.0, 4.0, 2.0]], [[8.0, 1.0, 5.0], [6.0, 
1.0, 2.0], [5.0, 8.0, 5.0], [1.0, 6.0, 4.0]], [[5.0, 2.0, 4.0], [6.0, 2.0, 2.0], [1.0, 6.0, 8.0], [5.0, 4.0, 6.0]]])
      val=Data(numpy.array([[[7.0, 8.0, 5.0], [7.0, 7.0, 3.0]], [[2.0, 2.0, 2.0], [1.0, 8.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 4.0, 1.0], [4.0, 5.0, 3.0]], [[7.0, 8.0, 2.0], [8.0, 3.0, 2.0]]]))
      arg[:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 3.0], [6.0, 7.0, 4.0], [7.0, 8.0, 5.0], [7.0, 7.0, 3.0]], [[3.0, 4.0, 3.0], [3.0, 
5.0, 6.0], [2.0, 2.0, 2.0], [1.0, 8.0, 4.0]], [[2.0, 7.0, 4.0], [1.0, 1.0, 5.0], [4.0, 5.0, 2.0], [2.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 4.0], [3.0, 8.0, 6.0], [7.0, 4.0, 1.0], [4.0, 5.0, 3.0]], [[8.0, 1.0, 5.0], [6.0, 
1.0, 2.0], [7.0, 8.0, 2.0], [8.0, 3.0, 2.0]], [[5.0, 2.0, 4.0], [6.0, 2.0, 2.0], [1.0, 6.0, 8.0], [5.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_LRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 3.0, 3.0], [3.0, 6.0, 5.0], [3.0, 2.0, 5.0], [8.0, 7.0, 5.0]], [[4.0, 8.0, 2.0], [6.0, 
7.0, 3.0], [7.0, 3.0, 2.0], [5.0, 5.0, 2.0]], [[5.0, 5.0, 2.0], [2.0, 7.0, 2.0], [3.0, 3.0, 5.0], [7.0, 2.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 5.0, 5.0], [4.0, 3.0, 7.0], [4.0, 7.0, 2.0], [1.0, 2.0, 5.0]], [[7.0, 7.0, 8.0], [1.0, 
4.0, 5.0], [6.0, 6.0, 7.0], [2.0, 7.0, 8.0]], [[1.0, 8.0, 7.0], [6.0, 7.0, 4.0], [8.0, 5.0, 7.0], [5.0, 5.0, 4.0]]])
      val=Data(numpy.array([[[3.0, 2.0, 6.0], [5.0, 6.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 2.0, 5.0], [7.0, 1.0, 5.0]]]))
      arg[2:,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 3.0], [3.0, 6.0, 5.0], [3.0, 2.0, 5.0], [8.0, 7.0, 5.0]], [[4.0, 8.0, 2.0], [6.0, 
7.0, 3.0], [7.0, 3.0, 2.0], [5.0, 5.0, 2.0]], [[5.0, 5.0, 2.0], [3.0, 2.0, 6.0], [5.0, 6.0, 5.0], [7.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 5.0], [4.0, 3.0, 7.0], [4.0, 7.0, 2.0], [1.0, 2.0, 5.0]], [[7.0, 7.0, 8.0], [1.0, 
4.0, 5.0], [6.0, 6.0, 7.0], [2.0, 7.0, 8.0]], [[1.0, 8.0, 7.0], [7.0, 2.0, 5.0], [7.0, 1.0, 5.0], [5.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 8.0, 8.0], [1.0, 3.0, 6.0], [5.0, 5.0, 5.0], [4.0, 8.0, 4.0]], [[4.0, 8.0, 8.0], [1.0, 
5.0, 7.0], [8.0, 8.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 8.0, 4.0], [5.0, 2.0, 8.0], [7.0, 7.0, 3.0], [6.0, 5.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 1.0, 3.0], [3.0, 1.0, 6.0], [1.0, 6.0, 1.0], [8.0, 4.0, 2.0]], [[3.0, 8.0, 5.0], [2.0, 
7.0, 3.0], [3.0, 4.0, 8.0], [2.0, 7.0, 8.0]], [[2.0, 2.0, 5.0], [7.0, 5.0, 7.0], [1.0, 7.0, 6.0], [7.0, 1.0, 4.0]]])
      val=Data(numpy.array([[[3.0, 6.0, 6.0], [1.0, 5.0, 1.0]], [[6.0, 6.0, 3.0], [4.0, 5.0, 4.0]], [[6.0, 7.0, 2.0], [7.0, 
7.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 4.0, 8.0], [2.0, 1.0, 2.0]], [[7.0, 4.0, 4.0], [5.0, 3.0, 4.0]], [[4.0, 3.0, 
6.0], [4.0, 1.0, 1.0]]]))
      arg[:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 8.0], [1.0, 3.0, 6.0], [3.0, 6.0, 6.0], [1.0, 5.0, 1.0]], [[4.0, 8.0, 8.0], [1.0, 
5.0, 7.0], [6.0, 6.0, 3.0], [4.0, 5.0, 4.0]], [[5.0, 8.0, 4.0], [5.0, 2.0, 8.0], [6.0, 7.0, 2.0], [7.0, 7.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 1.0, 3.0], [3.0, 1.0, 6.0], [8.0, 4.0, 8.0], [2.0, 1.0, 2.0]], [[3.0, 8.0, 5.0], [2.0, 
7.0, 3.0], [7.0, 4.0, 4.0], [5.0, 3.0, 4.0]], [[2.0, 2.0, 5.0], [7.0, 5.0, 7.0], [4.0, 3.0, 6.0], [4.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[2.0, 8.0, 1.0], [6.0, 3.0, 7.0], [6.0, 1.0, 1.0], [7.0, 2.0, 7.0]], [[6.0, 8.0, 1.0], [6.0, 
1.0, 8.0], [1.0, 6.0, 4.0], [8.0, 6.0, 5.0]], [[4.0, 3.0, 1.0], [3.0, 7.0, 2.0], [2.0, 7.0, 7.0], [2.0, 4.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 8.0, 8.0], [2.0, 6.0, 4.0], [5.0, 5.0, 7.0], [7.0, 7.0, 1.0]], [[2.0, 2.0, 3.0], [1.0, 
1.0, 5.0], [7.0, 5.0, 6.0], [2.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [3.0, 7.0, 3.0], [6.0, 1.0, 8.0], [3.0, 1.0, 5.0]]])
      val=Data(numpy.array([[3.0, 1.0, 3.0], [2.0, 5.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 5.0, 4.0], [1.0, 2.0, 6.0]]))
      arg[1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 8.0, 1.0], [6.0, 3.0, 7.0], [6.0, 1.0, 1.0], [7.0, 2.0, 7.0]], [[6.0, 8.0, 1.0], [6.0, 
1.0, 8.0], [3.0, 1.0, 3.0], [2.0, 5.0, 5.0]], [[4.0, 3.0, 1.0], [3.0, 7.0, 2.0], [2.0, 7.0, 7.0], [2.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 8.0], [2.0, 6.0, 4.0], [5.0, 5.0, 7.0], [7.0, 7.0, 1.0]], [[2.0, 2.0, 3.0], [1.0, 
1.0, 5.0], [4.0, 5.0, 4.0], [1.0, 2.0, 6.0]], [[7.0, 8.0, 6.0], [3.0, 7.0, 3.0], [6.0, 1.0, 8.0], [3.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[4.0, 3.0, 6.0], [4.0, 1.0, 7.0], [5.0, 7.0, 8.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 7.0], [4.0, 
3.0, 3.0], [3.0, 6.0, 1.0], [6.0, 6.0, 7.0]], [[6.0, 8.0, 2.0], [4.0, 5.0, 7.0], [3.0, 4.0, 8.0], [3.0, 1.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 6.0], [2.0, 2.0, 8.0], [3.0, 1.0, 2.0], [6.0, 2.0, 3.0]], [[6.0, 2.0, 6.0], [5.0, 
2.0, 6.0], [5.0, 8.0, 7.0], [6.0, 8.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 7.0, 7.0], [4.0, 6.0, 1.0], [8.0, 5.0, 3.0]]])
      val=Data(numpy.array([[[8.0, 7.0, 4.0], [1.0, 6.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 8.0, 1.0], [5.0, 3.0, 3.0]]]))
      arg[1:2,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 6.0], [4.0, 1.0, 7.0], [5.0, 7.0, 8.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 
7.0, 4.0], [1.0, 6.0, 5.0], [6.0, 6.0, 7.0]], [[6.0, 8.0, 2.0], [4.0, 5.0, 7.0], [3.0, 4.0, 8.0], [3.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 6.0], [2.0, 2.0, 8.0], [3.0, 1.0, 2.0], [6.0, 2.0, 3.0]], [[6.0, 2.0, 6.0], [3.0, 
8.0, 1.0], [5.0, 3.0, 3.0], [6.0, 8.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 7.0, 7.0], [4.0, 6.0, 1.0], [8.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 1.0, 3.0], [6.0, 6.0, 1.0], [2.0, 7.0, 4.0], [8.0, 8.0, 2.0]], [[5.0, 3.0, 8.0], [7.0, 
7.0, 8.0], [7.0, 4.0, 4.0], [8.0, 5.0, 7.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 3.0], [7.0, 3.0, 4.0], [3.0, 5.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 8.0, 7.0], [7.0, 8.0, 5.0], [7.0, 3.0, 2.0], [6.0, 3.0, 8.0]], [[8.0, 3.0, 8.0], [6.0, 
3.0, 6.0], [8.0, 5.0, 6.0], [8.0, 1.0, 5.0]], [[8.0, 6.0, 8.0], [7.0, 5.0, 4.0], [6.0, 6.0, 4.0], [1.0, 5.0, 5.0]]])
      val=Data(numpy.array([[3.0, 5.0, 5.0], [6.0, 2.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 8.0, 8.0], [2.0, 5.0, 6.0]]))
      arg[1,1:3]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 3.0], [6.0, 6.0, 1.0], [2.0, 7.0, 4.0], [8.0, 8.0, 2.0]], [[5.0, 3.0, 8.0], [3.0, 
5.0, 5.0], [6.0, 2.0, 5.0], [8.0, 5.0, 7.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 3.0], [7.0, 3.0, 4.0], [3.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 7.0], [7.0, 8.0, 5.0], [7.0, 3.0, 2.0], [6.0, 3.0, 8.0]], [[8.0, 3.0, 8.0], [7.0, 
8.0, 8.0], [2.0, 5.0, 6.0], [8.0, 1.0, 5.0]], [[8.0, 6.0, 8.0], [7.0, 5.0, 4.0], [6.0, 6.0, 4.0], [1.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CRL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[4.0, 4.0, 8.0], [4.0, 5.0, 7.0], [8.0, 4.0, 5.0], [3.0, 4.0, 1.0]], [[4.0, 1.0, 7.0], [1.0, 
3.0, 3.0], [3.0, 7.0, 3.0], [6.0, 8.0, 7.0]], [[4.0, 6.0, 5.0], [1.0, 4.0, 7.0], [1.0, 4.0, 4.0], [5.0, 6.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 3.0, 5.0], [3.0, 2.0, 5.0], [1.0, 8.0, 3.0], [6.0, 8.0, 2.0]], [[2.0, 8.0, 3.0], [2.0, 
1.0, 4.0], [2.0, 6.0, 4.0], [8.0, 3.0, 5.0]], [[8.0, 5.0, 2.0], [1.0, 7.0, 8.0], [5.0, 5.0, 5.0], [7.0, 6.0, 1.0]]])
      val=Data(numpy.array([[[2.0], [2.0]], [[5.0], [5.0]], [[8.0], [4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0], [5.0]], [[6.0], [1.0]], [[5.0], [6.0]]]))
      arg[:,1:3,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 8.0], [4.0, 5.0, 2.0], [8.0, 4.0, 2.0], [3.0, 4.0, 1.0]], [[4.0, 1.0, 7.0], [1.0, 
3.0, 5.0], [3.0, 7.0, 5.0], [6.0, 8.0, 7.0]], [[4.0, 6.0, 5.0], [1.0, 4.0, 8.0], [1.0, 4.0, 4.0], [5.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 5.0], [3.0, 2.0, 2.0], [1.0, 8.0, 5.0], [6.0, 8.0, 2.0]], [[2.0, 8.0, 3.0], [2.0, 
1.0, 6.0], [2.0, 6.0, 1.0], [8.0, 3.0, 5.0]], [[8.0, 5.0, 2.0], [1.0, 7.0, 5.0], [5.0, 5.0, 6.0], [7.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_PLU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[3.0, 1.0, 5.0], [1.0, 7.0, 4.0], [8.0, 7.0, 7.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 3.0], [8.0, 
3.0, 6.0], [8.0, 7.0, 5.0], [7.0, 8.0, 3.0]], [[2.0, 6.0, 1.0], [5.0, 5.0, 7.0], [4.0, 7.0, 4.0], [4.0, 2.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 8.0, 8.0], [3.0, 4.0, 2.0], [3.0, 6.0, 3.0], [8.0, 8.0, 7.0]], [[1.0, 5.0, 6.0], [1.0, 
4.0, 2.0], [4.0, 7.0, 3.0], [4.0, 1.0, 2.0]], [[6.0, 8.0, 6.0], [4.0, 1.0, 2.0], [7.0, 3.0, 7.0], [5.0, 1.0, 7.0]]])
      val=Data(numpy.array([[8.0, 1.0], [5.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 6.0], [3.0, 6.0]]))
      arg[1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 5.0], [1.0, 7.0, 4.0], [8.0, 7.0, 7.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 3.0], [8.0, 
3.0, 6.0], [8.0, 1.0, 5.0], [5.0, 8.0, 3.0]], [[2.0, 6.0, 1.0], [5.0, 5.0, 7.0], [4.0, 7.0, 4.0], [4.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 8.0], [3.0, 4.0, 2.0], [3.0, 6.0, 3.0], [8.0, 8.0, 7.0]], [[1.0, 5.0, 6.0], [1.0, 
4.0, 2.0], [8.0, 6.0, 3.0], [3.0, 6.0, 2.0]], [[6.0, 8.0, 6.0], [4.0, 1.0, 2.0], [7.0, 3.0, 7.0], [5.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RUP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[4.0, 2.0, 3.0], [2.0, 6.0, 6.0], [7.0, 5.0, 7.0], [6.0, 1.0, 4.0]], [[4.0, 7.0, 2.0], [7.0, 
3.0, 2.0], [5.0, 1.0, 2.0], [4.0, 6.0, 7.0]], [[1.0, 3.0, 1.0], [2.0, 5.0, 8.0], [4.0, 5.0, 2.0], [6.0, 3.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 7.0, 4.0], [2.0, 5.0, 6.0], [1.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[1.0, 6.0, 1.0], [5.0, 
7.0, 2.0], [8.0, 3.0, 3.0], [3.0, 7.0, 6.0]], [[8.0, 6.0, 6.0], [4.0, 2.0, 3.0], [6.0, 1.0, 5.0], [6.0, 6.0, 2.0]]])
      val=Data(numpy.array([[6.0, 2.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 3.0, 2.0]]))
      arg[1:2,:3,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 3.0], [2.0, 6.0, 6.0], [7.0, 5.0, 7.0], [6.0, 1.0, 4.0]], [[4.0, 6.0, 2.0], [7.0, 
2.0, 2.0], [5.0, 3.0, 2.0], [4.0, 6.0, 7.0]], [[1.0, 3.0, 1.0], [2.0, 5.0, 8.0], [4.0, 5.0, 2.0], [6.0, 3.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 7.0, 4.0], [2.0, 5.0, 6.0], [1.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[1.0, 6.0, 1.0], [5.0, 
3.0, 2.0], [8.0, 2.0, 3.0], [3.0, 7.0, 6.0]], [[8.0, 6.0, 6.0], [4.0, 2.0, 3.0], [6.0, 1.0, 5.0], [6.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_CRU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[1.0, 7.0, 6.0], [5.0, 6.0, 2.0], [2.0, 1.0, 2.0], [1.0, 3.0, 6.0]], [[6.0, 7.0, 2.0], [3.0, 
6.0, 2.0], [5.0, 4.0, 8.0], [4.0, 7.0, 7.0]], [[5.0, 7.0, 4.0], [6.0, 6.0, 5.0], [4.0, 5.0, 2.0], [7.0, 8.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 3.0, 6.0], [1.0, 3.0, 4.0], [7.0, 2.0, 3.0], [2.0, 5.0, 8.0]], [[4.0, 4.0, 2.0], [8.0, 
5.0, 1.0], [7.0, 3.0, 2.0], [4.0, 2.0, 1.0]], [[1.0, 2.0, 1.0], [1.0, 4.0, 7.0], [7.0, 2.0, 2.0], [6.0, 8.0, 2.0]]])
      val=Data(numpy.array([[[2.0, 6.0], [1.0, 6.0]], [[8.0, 1.0], [8.0, 6.0]], [[7.0, 3.0], [8.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 6.0], [3.0, 1.0]], [[3.0, 3.0], [2.0, 2.0]], [[8.0, 3.0], [7.0, 2.0]]]))
      arg[:,1:3,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 6.0], [2.0, 6.0, 2.0], [1.0, 6.0, 2.0], [1.0, 3.0, 6.0]], [[6.0, 7.0, 2.0], [8.0, 
1.0, 2.0], [8.0, 6.0, 8.0], [4.0, 7.0, 7.0]], [[5.0, 7.0, 4.0], [7.0, 3.0, 5.0], [8.0, 8.0, 2.0], [7.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 6.0], [1.0, 6.0, 4.0], [3.0, 1.0, 3.0], [2.0, 5.0, 8.0]], [[4.0, 4.0, 2.0], [3.0, 
3.0, 1.0], [2.0, 2.0, 2.0], [4.0, 2.0, 1.0]], [[1.0, 2.0, 1.0], [8.0, 3.0, 7.0], [7.0, 2.0, 2.0], [6.0, 8.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_taggedData_RRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[5.0, 8.0, 5.0], [3.0, 4.0, 4.0], [8.0, 3.0, 2.0], [3.0, 3.0, 8.0]], [[1.0, 2.0, 7.0], [8.0, 
4.0, 8.0], [5.0, 8.0, 6.0], [1.0, 5.0, 1.0]], [[3.0, 6.0, 2.0], [5.0, 1.0, 2.0], [5.0, 5.0, 8.0], [6.0, 6.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 3.0, 1.0], [4.0, 2.0, 8.0], [1.0, 2.0, 4.0], [4.0, 4.0, 4.0]], [[8.0, 3.0, 8.0], [6.0, 
8.0, 7.0], [4.0, 7.0, 1.0], [1.0, 1.0, 3.0]], [[5.0, 1.0, 3.0], [5.0, 6.0, 4.0], [7.0, 6.0, 7.0], [3.0, 4.0, 2.0]]])
      val=Data(numpy.array([[[4.0], [7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0], [4.0]]]))
      arg[1:2,1:3,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 5.0], [3.0, 4.0, 4.0], [8.0, 3.0, 2.0], [3.0, 3.0, 8.0]], [[1.0, 2.0, 7.0], [8.0, 
4.0, 8.0], [5.0, 7.0, 6.0], [1.0, 5.0, 1.0]], [[3.0, 6.0, 2.0], [5.0, 1.0, 2.0], [5.0, 5.0, 8.0], [6.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 1.0], [4.0, 2.0, 8.0], [1.0, 2.0, 4.0], [4.0, 4.0, 4.0]], [[8.0, 3.0, 8.0], [6.0, 
3.0, 7.0], [4.0, 4.0, 1.0], [1.0, 1.0, 3.0]], [[5.0, 1.0, 3.0], [5.0, 6.0, 4.0], [7.0, 6.0, 7.0], [3.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CRCR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 2.0], [5.0, 1.0, 2.0], [2.0, 8.0, 8.0]], [[2.0, 1.0, 3.0], [8.0, 5.0, 7.0], [6.0, 
2.0, 5.0]], [[5.0, 3.0, 7.0], [6.0, 5.0, 8.0], [5.0, 7.0, 2.0]]], [[[6.0, 2.0, 3.0], [2.0, 2.0, 7.0], [1.0, 4.0, 6.0]], [[7.0, 
8.0, 3.0], [6.0, 5.0, 7.0], [8.0, 8.0, 8.0]], [[4.0, 3.0, 4.0], [4.0, 6.0, 7.0], [3.0, 7.0, 4.0]]], [[[7.0, 6.0, 3.0], [4.0, 
8.0, 6.0], [4.0, 7.0, 1.0]], [[2.0, 6.0, 6.0], [1.0, 8.0, 1.0], [6.0, 4.0, 4.0]], [[2.0, 1.0, 6.0], [5.0, 1.0, 3.0], [5.0, 5.0, 
8.0]]], [[[5.0, 8.0, 7.0], [8.0, 3.0, 7.0], [1.0, 3.0, 6.0]], [[5.0, 2.0, 8.0], [3.0, 8.0, 8.0], [2.0, 5.0, 2.0]], [[4.0, 2.0, 
3.0], [1.0, 5.0, 4.0], [4.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 3.0], [3.0, 3.0, 3.0], [6.0, 8.0, 6.0]], 
[[4.0, 1.0, 8.0], [5.0, 5.0, 1.0], [4.0, 1.0, 7.0]], [[6.0, 5.0, 5.0], [4.0, 8.0, 3.0], [7.0, 3.0, 5.0]]], [[[2.0, 3.0, 6.0], 
[7.0, 1.0, 3.0], [5.0, 7.0, 2.0]], [[8.0, 6.0, 5.0], [8.0, 4.0, 5.0], [3.0, 6.0, 1.0]], [[6.0, 5.0, 1.0], [8.0, 1.0, 5.0], 
[8.0, 1.0, 1.0]]], [[[4.0, 1.0, 6.0], [8.0, 4.0, 3.0], [2.0, 6.0, 6.0]], [[2.0, 4.0, 5.0], [1.0, 2.0, 3.0], [2.0, 4.0, 2.0]], 
[[5.0, 7.0, 4.0], [2.0, 2.0, 7.0], [6.0, 5.0, 5.0]]], [[[3.0, 2.0, 3.0], [3.0, 7.0, 2.0], [3.0, 7.0, 1.0]], [[5.0, 3.0, 3.0], 
[5.0, 1.0, 7.0], [4.0, 4.0, 1.0]], [[5.0, 8.0, 3.0], [6.0, 6.0, 8.0], [5.0, 7.0, 6.0]]]])
      val=Data(numpy.array([[[[3.0], [4.0], [2.0]]], [[[4.0], [8.0], [3.0]]], [[[3.0], [7.0], [2.0]]], [[[3.0], [3.0], 
[6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0], [7.0], [3.0]]], [[[1.0], [2.0], [2.0]]], [[[4.0], [5.0], [5.0]]], [[[2.0], 
[4.0], [5.0]]]]))
      arg[:,1:2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 2.0], [5.0, 1.0, 2.0], [2.0, 8.0, 8.0]], [[2.0, 3.0, 3.0], [8.0, 4.0, 7.0], [6.0, 
2.0, 5.0]], [[5.0, 3.0, 7.0], [6.0, 5.0, 8.0], [5.0, 7.0, 2.0]]], [[[6.0, 2.0, 3.0], [2.0, 2.0, 7.0], [1.0, 4.0, 6.0]], [[7.0, 
4.0, 3.0], [6.0, 8.0, 7.0], [8.0, 3.0, 8.0]], [[4.0, 3.0, 4.0], [4.0, 6.0, 7.0], [3.0, 7.0, 4.0]]], [[[7.0, 6.0, 3.0], [4.0, 
8.0, 6.0], [4.0, 7.0, 1.0]], [[2.0, 3.0, 6.0], [1.0, 7.0, 1.0], [6.0, 2.0, 4.0]], [[2.0, 1.0, 6.0], [5.0, 1.0, 3.0], [5.0, 5.0, 
8.0]]], [[[5.0, 8.0, 7.0], [8.0, 3.0, 7.0], [1.0, 3.0, 6.0]], [[5.0, 3.0, 8.0], [3.0, 3.0, 8.0], [2.0, 6.0, 2.0]], [[4.0, 2.0, 
3.0], [1.0, 5.0, 4.0], [4.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 3.0], [3.0, 3.0, 3.0], [6.0, 8.0, 6.0]], 
[[4.0, 1.0, 8.0], [5.0, 7.0, 1.0], [4.0, 3.0, 7.0]], [[6.0, 5.0, 5.0], [4.0, 8.0, 3.0], [7.0, 3.0, 5.0]]], [[[2.0, 3.0, 6.0], 
[7.0, 1.0, 3.0], [5.0, 7.0, 2.0]], [[8.0, 1.0, 5.0], [8.0, 2.0, 5.0], [3.0, 2.0, 1.0]], [[6.0, 5.0, 1.0], [8.0, 1.0, 5.0], 
[8.0, 1.0, 1.0]]], [[[4.0, 1.0, 6.0], [8.0, 4.0, 3.0], [2.0, 6.0, 6.0]], [[2.0, 4.0, 5.0], [1.0, 5.0, 3.0], [2.0, 5.0, 2.0]], 
[[5.0, 7.0, 4.0], [2.0, 2.0, 7.0], [6.0, 5.0, 5.0]]], [[[3.0, 2.0, 3.0], [3.0, 7.0, 2.0], [3.0, 7.0, 1.0]], [[5.0, 2.0, 3.0], 
[5.0, 4.0, 7.0], [4.0, 5.0, 1.0]], [[5.0, 8.0, 3.0], [6.0, 6.0, 8.0], [5.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRLR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 4.0], [1.0, 2.0, 4.0], [6.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], [8.0, 1.0, 5.0], [5.0, 
5.0, 3.0]], [[1.0, 6.0, 7.0], [6.0, 4.0, 3.0], [7.0, 3.0, 5.0]]], [[[2.0, 2.0, 7.0], [2.0, 2.0, 2.0], [5.0, 2.0, 4.0]], [[5.0, 
8.0, 3.0], [5.0, 6.0, 3.0], [6.0, 3.0, 4.0]], [[8.0, 3.0, 4.0], [3.0, 5.0, 2.0], [3.0, 1.0, 5.0]]], [[[5.0, 4.0, 8.0], [2.0, 
6.0, 3.0], [7.0, 8.0, 2.0]], [[5.0, 6.0, 2.0], [3.0, 8.0, 2.0], [4.0, 2.0, 8.0]], [[8.0, 3.0, 4.0], [5.0, 4.0, 7.0], [2.0, 3.0, 
6.0]]], [[[6.0, 1.0, 1.0], [2.0, 4.0, 2.0], [4.0, 1.0, 5.0]], [[2.0, 2.0, 5.0], [6.0, 8.0, 3.0], [1.0, 1.0, 5.0]], [[5.0, 2.0, 
4.0], [6.0, 1.0, 7.0], [6.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 1.0], [4.0, 7.0, 6.0], [8.0, 5.0, 5.0]], 
[[6.0, 1.0, 6.0], [6.0, 3.0, 2.0], [5.0, 7.0, 2.0]], [[2.0, 7.0, 8.0], [4.0, 5.0, 5.0], [5.0, 7.0, 6.0]]], [[[4.0, 1.0, 6.0], 
[8.0, 1.0, 6.0], [4.0, 6.0, 5.0]], [[5.0, 6.0, 4.0], [3.0, 3.0, 3.0], [3.0, 7.0, 3.0]], [[1.0, 4.0, 6.0], [1.0, 4.0, 7.0], 
[1.0, 5.0, 8.0]]], [[[2.0, 1.0, 8.0], [5.0, 5.0, 1.0], [5.0, 7.0, 7.0]], [[1.0, 2.0, 1.0], [1.0, 3.0, 3.0], [5.0, 1.0, 3.0]], 
[[2.0, 7.0, 3.0], [3.0, 2.0, 5.0], [6.0, 1.0, 7.0]]], [[[4.0, 7.0, 3.0], [1.0, 8.0, 4.0], [3.0, 8.0, 3.0]], [[5.0, 4.0, 2.0], 
[6.0, 4.0, 7.0], [5.0, 2.0, 3.0]], [[8.0, 1.0, 4.0], [5.0, 6.0, 3.0], [3.0, 7.0, 1.0]]]])
      val=Data(numpy.array([[[[8.0]]], [[[6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0]]], [[[3.0]]]]))
      arg[2:,1:2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [1.0, 2.0, 4.0], [6.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], [8.0, 1.0, 5.0], [5.0, 
5.0, 3.0]], [[1.0, 6.0, 7.0], [6.0, 4.0, 3.0], [7.0, 3.0, 5.0]]], [[[2.0, 2.0, 7.0], [2.0, 2.0, 2.0], [5.0, 2.0, 4.0]], [[5.0, 
8.0, 3.0], [5.0, 6.0, 3.0], [6.0, 3.0, 4.0]], [[8.0, 3.0, 4.0], [3.0, 5.0, 2.0], [3.0, 1.0, 5.0]]], [[[5.0, 4.0, 8.0], [2.0, 
6.0, 3.0], [7.0, 8.0, 2.0]], [[5.0, 6.0, 2.0], [3.0, 8.0, 2.0], [4.0, 8.0, 8.0]], [[8.0, 3.0, 4.0], [5.0, 4.0, 7.0], [2.0, 3.0, 
6.0]]], [[[6.0, 1.0, 1.0], [2.0, 4.0, 2.0], [4.0, 1.0, 5.0]], [[2.0, 2.0, 5.0], [6.0, 8.0, 3.0], [1.0, 6.0, 5.0]], [[5.0, 2.0, 
4.0], [6.0, 1.0, 7.0], [6.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 1.0], [4.0, 7.0, 6.0], [8.0, 5.0, 5.0]], 
[[6.0, 1.0, 6.0], [6.0, 3.0, 2.0], [5.0, 7.0, 2.0]], [[2.0, 7.0, 8.0], [4.0, 5.0, 5.0], [5.0, 7.0, 6.0]]], [[[4.0, 1.0, 6.0], 
[8.0, 1.0, 6.0], [4.0, 6.0, 5.0]], [[5.0, 6.0, 4.0], [3.0, 3.0, 3.0], [3.0, 7.0, 3.0]], [[1.0, 4.0, 6.0], [1.0, 4.0, 7.0], 
[1.0, 5.0, 8.0]]], [[[2.0, 1.0, 8.0], [5.0, 5.0, 1.0], [5.0, 7.0, 7.0]], [[1.0, 2.0, 1.0], [1.0, 3.0, 3.0], [5.0, 5.0, 3.0]], 
[[2.0, 7.0, 3.0], [3.0, 2.0, 5.0], [6.0, 1.0, 7.0]]], [[[4.0, 7.0, 3.0], [1.0, 8.0, 4.0], [3.0, 8.0, 3.0]], [[5.0, 4.0, 2.0], 
[6.0, 4.0, 7.0], [5.0, 3.0, 3.0]], [[8.0, 1.0, 4.0], [5.0, 6.0, 3.0], [3.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCUU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 2.0], [3.0, 7.0, 8.0], [8.0, 2.0, 7.0]], [[1.0, 6.0, 3.0], [5.0, 2.0, 5.0], [6.0, 
8.0, 7.0]], [[5.0, 6.0, 1.0], [7.0, 5.0, 6.0], [5.0, 5.0, 4.0]]], [[[5.0, 3.0, 7.0], [1.0, 8.0, 2.0], [5.0, 5.0, 6.0]], [[8.0, 
5.0, 4.0], [6.0, 7.0, 6.0], [5.0, 5.0, 6.0]], [[6.0, 8.0, 5.0], [1.0, 6.0, 2.0], [7.0, 3.0, 5.0]]], [[[5.0, 2.0, 4.0], [5.0, 
6.0, 8.0], [3.0, 2.0, 7.0]], [[7.0, 3.0, 4.0], [7.0, 8.0, 7.0], [1.0, 1.0, 3.0]], [[6.0, 7.0, 3.0], [4.0, 5.0, 3.0], [1.0, 4.0, 
7.0]]], [[[7.0, 5.0, 1.0], [7.0, 8.0, 7.0], [3.0, 5.0, 7.0]], [[1.0, 5.0, 2.0], [3.0, 1.0, 3.0], [1.0, 3.0, 7.0]], [[1.0, 5.0, 
6.0], [8.0, 8.0, 5.0], [4.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 4.0], [4.0, 6.0, 4.0], [1.0, 4.0, 6.0]], 
[[4.0, 2.0, 8.0], [4.0, 3.0, 2.0], [8.0, 1.0, 1.0]], [[4.0, 7.0, 2.0], [6.0, 7.0, 4.0], [4.0, 1.0, 3.0]]], [[[6.0, 2.0, 6.0], 
[2.0, 5.0, 1.0], [7.0, 8.0, 3.0]], [[5.0, 1.0, 2.0], [4.0, 6.0, 4.0], [7.0, 4.0, 6.0]], [[5.0, 8.0, 4.0], [3.0, 3.0, 8.0], 
[2.0, 3.0, 6.0]]], [[[2.0, 3.0, 1.0], [3.0, 3.0, 2.0], [6.0, 7.0, 8.0]], [[2.0, 7.0, 8.0], [5.0, 5.0, 2.0], [6.0, 8.0, 6.0]], 
[[6.0, 4.0, 3.0], [4.0, 4.0, 3.0], [4.0, 4.0, 8.0]]], [[[3.0, 3.0, 8.0], [1.0, 1.0, 4.0], [5.0, 5.0, 8.0]], [[8.0, 6.0, 8.0], 
[5.0, 2.0, 3.0], [6.0, 4.0, 3.0]], [[1.0, 8.0, 5.0], [2.0, 2.0, 6.0], [4.0, 6.0, 4.0]]]])
      val=Data(numpy.array([[[[1.0, 7.0], [8.0, 6.0]], [[1.0, 1.0], [1.0, 1.0]], [[6.0, 1.0], [5.0, 3.0]]], [[[8.0, 2.0], [3.0, 
8.0]], [[7.0, 5.0], [1.0, 4.0]], [[6.0, 6.0], [6.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 6.0], [6.0, 1.0]], [[3.0, 8.0], [8.0, 1.0]], [[5.0, 3.0], [8.0, 4.0]]], [[[5.0, 
5.0], [2.0, 2.0]], [[7.0, 5.0], [2.0, 3.0]], [[6.0, 7.0], [8.0, 5.0]]]]))
      arg[2:,:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 2.0], [3.0, 7.0, 8.0], [8.0, 2.0, 7.0]], [[1.0, 6.0, 3.0], [5.0, 2.0, 5.0], [6.0, 
8.0, 7.0]], [[5.0, 6.0, 1.0], [7.0, 5.0, 6.0], [5.0, 5.0, 4.0]]], [[[5.0, 3.0, 7.0], [1.0, 8.0, 2.0], [5.0, 5.0, 6.0]], [[8.0, 
5.0, 4.0], [6.0, 7.0, 6.0], [5.0, 5.0, 6.0]], [[6.0, 8.0, 5.0], [1.0, 6.0, 2.0], [7.0, 3.0, 5.0]]], [[[1.0, 7.0, 4.0], [8.0, 
6.0, 8.0], [3.0, 2.0, 7.0]], [[1.0, 1.0, 4.0], [1.0, 1.0, 7.0], [1.0, 1.0, 3.0]], [[6.0, 1.0, 3.0], [5.0, 3.0, 3.0], [1.0, 4.0, 
7.0]]], [[[8.0, 2.0, 1.0], [3.0, 8.0, 7.0], [3.0, 5.0, 7.0]], [[7.0, 5.0, 2.0], [1.0, 4.0, 3.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 
6.0], [6.0, 3.0, 5.0], [4.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 4.0], [4.0, 6.0, 4.0], [1.0, 4.0, 6.0]], 
[[4.0, 2.0, 8.0], [4.0, 3.0, 2.0], [8.0, 1.0, 1.0]], [[4.0, 7.0, 2.0], [6.0, 7.0, 4.0], [4.0, 1.0, 3.0]]], [[[6.0, 2.0, 6.0], 
[2.0, 5.0, 1.0], [7.0, 8.0, 3.0]], [[5.0, 1.0, 2.0], [4.0, 6.0, 4.0], [7.0, 4.0, 6.0]], [[5.0, 8.0, 4.0], [3.0, 3.0, 8.0], 
[2.0, 3.0, 6.0]]], [[[3.0, 6.0, 1.0], [6.0, 1.0, 2.0], [6.0, 7.0, 8.0]], [[3.0, 8.0, 8.0], [8.0, 1.0, 2.0], [6.0, 8.0, 6.0]], 
[[5.0, 3.0, 3.0], [8.0, 4.0, 3.0], [4.0, 4.0, 8.0]]], [[[5.0, 5.0, 8.0], [2.0, 2.0, 4.0], [5.0, 5.0, 8.0]], [[7.0, 5.0, 8.0], 
[2.0, 3.0, 3.0], [6.0, 4.0, 3.0]], [[6.0, 7.0, 5.0], [8.0, 5.0, 6.0], [4.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PULR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 7.0], [5.0, 4.0, 8.0], [6.0, 6.0, 6.0]], [[7.0, 8.0, 8.0], [1.0, 7.0, 3.0], [1.0, 
8.0, 2.0]], [[5.0, 2.0, 5.0], [8.0, 4.0, 7.0], [2.0, 2.0, 6.0]]], [[[4.0, 4.0, 1.0], [5.0, 4.0, 7.0], [1.0, 1.0, 3.0]], [[2.0, 
2.0, 1.0], [4.0, 1.0, 3.0], [8.0, 8.0, 2.0]], [[4.0, 6.0, 4.0], [6.0, 7.0, 7.0], [4.0, 8.0, 4.0]]], [[[6.0, 8.0, 6.0], [2.0, 
6.0, 8.0], [2.0, 5.0, 1.0]], [[7.0, 4.0, 5.0], [1.0, 3.0, 8.0], [7.0, 6.0, 2.0]], [[8.0, 6.0, 7.0], [6.0, 4.0, 3.0], [2.0, 1.0, 
4.0]]], [[[6.0, 3.0, 3.0], [1.0, 3.0, 6.0], [6.0, 7.0, 4.0]], [[1.0, 2.0, 8.0], [8.0, 7.0, 3.0], [4.0, 7.0, 7.0]], [[7.0, 7.0, 
2.0], [4.0, 2.0, 6.0], [4.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 1.0], [2.0, 4.0, 4.0], [8.0, 6.0, 3.0]], 
[[5.0, 3.0, 7.0], [2.0, 7.0, 3.0], [3.0, 5.0, 8.0]], [[3.0, 2.0, 1.0], [7.0, 5.0, 4.0], [7.0, 2.0, 2.0]]], [[[1.0, 8.0, 4.0], 
[2.0, 2.0, 1.0], [5.0, 3.0, 7.0]], [[2.0, 2.0, 3.0], [8.0, 3.0, 3.0], [2.0, 8.0, 7.0]], [[5.0, 4.0, 4.0], [4.0, 6.0, 6.0], 
[1.0, 6.0, 4.0]]], [[[3.0, 4.0, 7.0], [4.0, 7.0, 1.0], [5.0, 4.0, 5.0]], [[4.0, 2.0, 3.0], [3.0, 4.0, 2.0], [4.0, 2.0, 7.0]], 
[[8.0, 3.0, 8.0], [5.0, 5.0, 4.0], [2.0, 7.0, 8.0]]], [[[4.0, 2.0, 7.0], [7.0, 1.0, 6.0], [2.0, 8.0, 4.0]], [[4.0, 2.0, 8.0], 
[4.0, 4.0, 7.0], [6.0, 5.0, 1.0]], [[6.0, 2.0, 8.0], [1.0, 4.0, 6.0], [7.0, 5.0, 7.0]]]])
      val=Data(numpy.array([[[6.0]], [[1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0]], [[2.0]]]))
      arg[2,:2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 7.0], [5.0, 4.0, 8.0], [6.0, 6.0, 6.0]], [[7.0, 8.0, 8.0], [1.0, 7.0, 3.0], [1.0, 
8.0, 2.0]], [[5.0, 2.0, 5.0], [8.0, 4.0, 7.0], [2.0, 2.0, 6.0]]], [[[4.0, 4.0, 1.0], [5.0, 4.0, 7.0], [1.0, 1.0, 3.0]], [[2.0, 
2.0, 1.0], [4.0, 1.0, 3.0], [8.0, 8.0, 2.0]], [[4.0, 6.0, 4.0], [6.0, 7.0, 7.0], [4.0, 8.0, 4.0]]], [[[6.0, 8.0, 6.0], [2.0, 
6.0, 8.0], [2.0, 6.0, 1.0]], [[7.0, 4.0, 5.0], [1.0, 3.0, 8.0], [7.0, 1.0, 2.0]], [[8.0, 6.0, 7.0], [6.0, 4.0, 3.0], [2.0, 1.0, 
4.0]]], [[[6.0, 3.0, 3.0], [1.0, 3.0, 6.0], [6.0, 7.0, 4.0]], [[1.0, 2.0, 8.0], [8.0, 7.0, 3.0], [4.0, 7.0, 7.0]], [[7.0, 7.0, 
2.0], [4.0, 2.0, 6.0], [4.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 1.0], [2.0, 4.0, 4.0], [8.0, 6.0, 3.0]], 
[[5.0, 3.0, 7.0], [2.0, 7.0, 3.0], [3.0, 5.0, 8.0]], [[3.0, 2.0, 1.0], [7.0, 5.0, 4.0], [7.0, 2.0, 2.0]]], [[[1.0, 8.0, 4.0], 
[2.0, 2.0, 1.0], [5.0, 3.0, 7.0]], [[2.0, 2.0, 3.0], [8.0, 3.0, 3.0], [2.0, 8.0, 7.0]], [[5.0, 4.0, 4.0], [4.0, 6.0, 6.0], 
[1.0, 6.0, 4.0]]], [[[3.0, 4.0, 7.0], [4.0, 7.0, 1.0], [5.0, 2.0, 5.0]], [[4.0, 2.0, 3.0], [3.0, 4.0, 2.0], [4.0, 2.0, 7.0]], 
[[8.0, 3.0, 8.0], [5.0, 5.0, 4.0], [2.0, 7.0, 8.0]]], [[[4.0, 2.0, 7.0], [7.0, 1.0, 6.0], [2.0, 8.0, 4.0]], [[4.0, 2.0, 8.0], 
[4.0, 4.0, 7.0], [6.0, 5.0, 1.0]], [[6.0, 2.0, 8.0], [1.0, 4.0, 6.0], [7.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULUR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 7.0], [8.0, 4.0, 5.0], [5.0, 2.0, 5.0]], [[3.0, 8.0, 7.0], [4.0, 4.0, 4.0], [6.0, 
8.0, 1.0]], [[7.0, 7.0, 2.0], [2.0, 2.0, 7.0], [5.0, 5.0, 2.0]]], [[[1.0, 8.0, 4.0], [5.0, 3.0, 6.0], [8.0, 2.0, 3.0]], [[6.0, 
1.0, 6.0], [3.0, 4.0, 8.0], [4.0, 5.0, 2.0]], [[7.0, 2.0, 3.0], [2.0, 6.0, 3.0], [4.0, 6.0, 1.0]]], [[[8.0, 1.0, 6.0], [4.0, 
5.0, 4.0], [4.0, 5.0, 4.0]], [[7.0, 5.0, 7.0], [1.0, 3.0, 1.0], [7.0, 3.0, 6.0]], [[2.0, 2.0, 3.0], [8.0, 8.0, 2.0], [5.0, 6.0, 
3.0]]], [[[7.0, 8.0, 5.0], [8.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[3.0, 4.0, 8.0], [1.0, 1.0, 1.0], [2.0, 7.0, 7.0]], [[1.0, 5.0, 
6.0], [1.0, 3.0, 4.0], [4.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 1.0], [8.0, 7.0, 3.0], [6.0, 5.0, 6.0]], 
[[5.0, 6.0, 1.0], [8.0, 5.0, 8.0], [2.0, 5.0, 2.0]], [[6.0, 3.0, 7.0], [1.0, 6.0, 1.0], [1.0, 7.0, 1.0]]], [[[7.0, 7.0, 1.0], 
[8.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[4.0, 5.0, 7.0], [7.0, 4.0, 6.0], [7.0, 2.0, 3.0]], [[7.0, 1.0, 4.0], [5.0, 8.0, 5.0], 
[5.0, 3.0, 6.0]]], [[[7.0, 2.0, 2.0], [7.0, 4.0, 1.0], [8.0, 3.0, 2.0]], [[1.0, 1.0, 3.0], [7.0, 6.0, 4.0], [2.0, 6.0, 1.0]], 
[[2.0, 4.0, 5.0], [1.0, 3.0, 6.0], [3.0, 7.0, 7.0]]], [[[7.0, 5.0, 8.0], [8.0, 5.0, 1.0], [1.0, 6.0, 1.0]], [[4.0, 1.0, 7.0], 
[7.0, 4.0, 2.0], [1.0, 7.0, 3.0]], [[7.0, 1.0, 6.0], [2.0, 1.0, 7.0], [7.0, 8.0, 3.0]]]])
      val=Data(numpy.array([[[[1.0], [4.0]]], [[[8.0], [8.0]]], [[[2.0], [7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0], [4.0]]], [[[8.0], [2.0]]], [[[5.0], [3.0]]]]))
      arg[:3,2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 7.0], [8.0, 4.0, 5.0], [5.0, 2.0, 5.0]], [[3.0, 8.0, 7.0], [4.0, 4.0, 4.0], [6.0, 
8.0, 1.0]], [[7.0, 1.0, 2.0], [2.0, 4.0, 7.0], [5.0, 5.0, 2.0]]], [[[1.0, 8.0, 4.0], [5.0, 3.0, 6.0], [8.0, 2.0, 3.0]], [[6.0, 
1.0, 6.0], [3.0, 4.0, 8.0], [4.0, 5.0, 2.0]], [[7.0, 8.0, 3.0], [2.0, 8.0, 3.0], [4.0, 6.0, 1.0]]], [[[8.0, 1.0, 6.0], [4.0, 
5.0, 4.0], [4.0, 5.0, 4.0]], [[7.0, 5.0, 7.0], [1.0, 3.0, 1.0], [7.0, 3.0, 6.0]], [[2.0, 2.0, 3.0], [8.0, 7.0, 2.0], [5.0, 6.0, 
3.0]]], [[[7.0, 8.0, 5.0], [8.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[3.0, 4.0, 8.0], [1.0, 1.0, 1.0], [2.0, 7.0, 7.0]], [[1.0, 5.0, 
6.0], [1.0, 3.0, 4.0], [4.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 1.0], [8.0, 7.0, 3.0], [6.0, 5.0, 6.0]], 
[[5.0, 6.0, 1.0], [8.0, 5.0, 8.0], [2.0, 5.0, 2.0]], [[6.0, 2.0, 7.0], [1.0, 4.0, 1.0], [1.0, 7.0, 1.0]]], [[[7.0, 7.0, 1.0], 
[8.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[4.0, 5.0, 7.0], [7.0, 4.0, 6.0], [7.0, 2.0, 3.0]], [[7.0, 8.0, 4.0], [5.0, 2.0, 5.0], 
[5.0, 3.0, 6.0]]], [[[7.0, 2.0, 2.0], [7.0, 4.0, 1.0], [8.0, 3.0, 2.0]], [[1.0, 1.0, 3.0], [7.0, 6.0, 4.0], [2.0, 6.0, 1.0]], 
[[2.0, 5.0, 5.0], [1.0, 3.0, 6.0], [3.0, 7.0, 7.0]]], [[[7.0, 5.0, 8.0], [8.0, 5.0, 1.0], [1.0, 6.0, 1.0]], [[4.0, 1.0, 7.0], 
[7.0, 4.0, 2.0], [1.0, 7.0, 3.0]], [[7.0, 1.0, 6.0], [2.0, 1.0, 7.0], [7.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CRUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 6.0, 7.0], [8.0, 8.0, 5.0], [2.0, 3.0, 5.0]], [[6.0, 3.0, 6.0], [6.0, 2.0, 3.0], [8.0, 
6.0, 5.0]], [[5.0, 2.0, 6.0], [4.0, 5.0, 4.0], [1.0, 4.0, 5.0]]], [[[8.0, 8.0, 1.0], [1.0, 8.0, 1.0], [2.0, 8.0, 6.0]], [[6.0, 
8.0, 1.0], [6.0, 4.0, 4.0], [2.0, 8.0, 3.0]], [[8.0, 1.0, 5.0], [7.0, 2.0, 7.0], [7.0, 3.0, 5.0]]], [[[2.0, 8.0, 7.0], [1.0, 
4.0, 7.0], [8.0, 4.0, 1.0]], [[6.0, 8.0, 1.0], [5.0, 3.0, 6.0], [6.0, 5.0, 1.0]], [[7.0, 6.0, 1.0], [3.0, 3.0, 3.0], [5.0, 1.0, 
6.0]]], [[[5.0, 6.0, 4.0], [2.0, 5.0, 8.0], [1.0, 4.0, 5.0]], [[6.0, 2.0, 1.0], [7.0, 2.0, 5.0], [2.0, 3.0, 2.0]], [[2.0, 8.0, 
8.0], [4.0, 5.0, 8.0], [5.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 3.0], [5.0, 3.0, 7.0], [2.0, 7.0, 5.0]], 
[[7.0, 2.0, 1.0], [7.0, 4.0, 8.0], [5.0, 2.0, 8.0]], [[2.0, 5.0, 1.0], [2.0, 8.0, 1.0], [8.0, 5.0, 3.0]]], [[[7.0, 4.0, 3.0], 
[7.0, 5.0, 2.0], [5.0, 2.0, 3.0]], [[6.0, 8.0, 8.0], [7.0, 5.0, 5.0], [1.0, 1.0, 7.0]], [[4.0, 7.0, 4.0], [5.0, 5.0, 7.0], 
[6.0, 4.0, 4.0]]], [[[1.0, 2.0, 6.0], [4.0, 4.0, 7.0], [7.0, 8.0, 2.0]], [[7.0, 3.0, 1.0], [5.0, 6.0, 2.0], [1.0, 8.0, 7.0]], 
[[5.0, 4.0, 4.0], [1.0, 4.0, 4.0], [1.0, 8.0, 6.0]]], [[[5.0, 3.0, 6.0], [3.0, 4.0, 5.0], [6.0, 3.0, 5.0]], [[8.0, 1.0, 2.0], 
[2.0, 2.0, 3.0], [6.0, 7.0, 8.0]], [[5.0, 4.0, 3.0], [2.0, 8.0, 1.0], [7.0, 4.0, 3.0]]]])
      val=Data(numpy.array([[[[2.0, 8.0, 7.0], [4.0, 5.0, 7.0]]], [[[5.0, 1.0, 4.0], [2.0, 5.0, 8.0]]], [[[6.0, 4.0, 4.0], 
[7.0, 2.0, 7.0]]], [[[2.0, 6.0, 6.0], [2.0, 1.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 2.0, 8.0], [2.0, 3.0, 1.0]]], [[[1.0, 5.0, 3.0], [3.0, 7.0, 2.0]]], [[[2.0, 
8.0, 1.0], [5.0, 4.0, 5.0]]], [[[8.0, 7.0, 2.0], [3.0, 6.0, 7.0]]]]))
      arg[:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 7.0], [8.0, 8.0, 5.0], [2.0, 3.0, 5.0]], [[2.0, 8.0, 7.0], [4.0, 5.0, 7.0], [8.0, 
6.0, 5.0]], [[5.0, 2.0, 6.0], [4.0, 5.0, 4.0], [1.0, 4.0, 5.0]]], [[[8.0, 8.0, 1.0], [1.0, 8.0, 1.0], [2.0, 8.0, 6.0]], [[5.0, 
1.0, 4.0], [2.0, 5.0, 8.0], [2.0, 8.0, 3.0]], [[8.0, 1.0, 5.0], [7.0, 2.0, 7.0], [7.0, 3.0, 5.0]]], [[[2.0, 8.0, 7.0], [1.0, 
4.0, 7.0], [8.0, 4.0, 1.0]], [[6.0, 4.0, 4.0], [7.0, 2.0, 7.0], [6.0, 5.0, 1.0]], [[7.0, 6.0, 1.0], [3.0, 3.0, 3.0], [5.0, 1.0, 
6.0]]], [[[5.0, 6.0, 4.0], [2.0, 5.0, 8.0], [1.0, 4.0, 5.0]], [[2.0, 6.0, 6.0], [2.0, 1.0, 8.0], [2.0, 3.0, 2.0]], [[2.0, 8.0, 
8.0], [4.0, 5.0, 8.0], [5.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 3.0], [5.0, 3.0, 7.0], [2.0, 7.0, 5.0]], 
[[1.0, 2.0, 8.0], [2.0, 3.0, 1.0], [5.0, 2.0, 8.0]], [[2.0, 5.0, 1.0], [2.0, 8.0, 1.0], [8.0, 5.0, 3.0]]], [[[7.0, 4.0, 3.0], 
[7.0, 5.0, 2.0], [5.0, 2.0, 3.0]], [[1.0, 5.0, 3.0], [3.0, 7.0, 2.0], [1.0, 1.0, 7.0]], [[4.0, 7.0, 4.0], [5.0, 5.0, 7.0], 
[6.0, 4.0, 4.0]]], [[[1.0, 2.0, 6.0], [4.0, 4.0, 7.0], [7.0, 8.0, 2.0]], [[2.0, 8.0, 1.0], [5.0, 4.0, 5.0], [1.0, 8.0, 7.0]], 
[[5.0, 4.0, 4.0], [1.0, 4.0, 4.0], [1.0, 8.0, 6.0]]], [[[5.0, 3.0, 6.0], [3.0, 4.0, 5.0], [6.0, 3.0, 5.0]], [[8.0, 7.0, 2.0], 
[3.0, 6.0, 7.0], [6.0, 7.0, 8.0]], [[5.0, 4.0, 3.0], [2.0, 8.0, 1.0], [7.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PUPU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 2.0], [4.0, 3.0, 8.0], [8.0, 7.0, 3.0]], [[5.0, 1.0, 3.0], [8.0, 4.0, 8.0], [5.0, 
6.0, 4.0]], [[3.0, 8.0, 6.0], [6.0, 7.0, 8.0], [1.0, 5.0, 2.0]]], [[[1.0, 3.0, 5.0], [1.0, 8.0, 8.0], [2.0, 7.0, 2.0]], [[1.0, 
2.0, 2.0], [4.0, 8.0, 2.0], [1.0, 7.0, 6.0]], [[2.0, 6.0, 8.0], [8.0, 4.0, 8.0], [2.0, 3.0, 7.0]]], [[[8.0, 8.0, 2.0], [4.0, 
6.0, 8.0], [4.0, 1.0, 6.0]], [[5.0, 2.0, 5.0], [3.0, 3.0, 6.0], [2.0, 2.0, 6.0]], [[1.0, 2.0, 8.0], [6.0, 3.0, 7.0], [2.0, 1.0, 
7.0]]], [[[3.0, 1.0, 1.0], [4.0, 7.0, 5.0], [2.0, 7.0, 6.0]], [[5.0, 1.0, 1.0], [7.0, 4.0, 5.0], [3.0, 8.0, 3.0]], [[5.0, 7.0, 
4.0], [1.0, 1.0, 5.0], [8.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 6.0], [6.0, 1.0, 2.0], [3.0, 8.0, 8.0]], 
[[5.0, 3.0, 6.0], [6.0, 1.0, 2.0], [8.0, 2.0, 5.0]], [[6.0, 1.0, 6.0], [6.0, 5.0, 1.0], [4.0, 2.0, 6.0]]], [[[5.0, 6.0, 3.0], 
[8.0, 4.0, 1.0], [2.0, 5.0, 7.0]], [[4.0, 1.0, 8.0], [6.0, 1.0, 1.0], [1.0, 7.0, 1.0]], [[8.0, 5.0, 3.0], [5.0, 8.0, 3.0], 
[4.0, 6.0, 4.0]]], [[[7.0, 3.0, 3.0], [8.0, 4.0, 6.0], [3.0, 3.0, 5.0]], [[2.0, 3.0, 4.0], [3.0, 2.0, 2.0], [6.0, 3.0, 2.0]], 
[[2.0, 4.0, 5.0], [5.0, 8.0, 1.0], [6.0, 4.0, 8.0]]], [[[2.0, 8.0, 5.0], [8.0, 5.0, 2.0], [7.0, 7.0, 5.0]], [[5.0, 4.0, 3.0], 
[3.0, 5.0, 7.0], [6.0, 7.0, 6.0]], [[5.0, 3.0, 8.0], [3.0, 3.0, 7.0], [3.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[5.0, 7.0], [6.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 2.0], [4.0, 5.0]]))
      arg[2,:2,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 2.0], [4.0, 3.0, 8.0], [8.0, 7.0, 3.0]], [[5.0, 1.0, 3.0], [8.0, 4.0, 8.0], [5.0, 
6.0, 4.0]], [[3.0, 8.0, 6.0], [6.0, 7.0, 8.0], [1.0, 5.0, 2.0]]], [[[1.0, 3.0, 5.0], [1.0, 8.0, 8.0], [2.0, 7.0, 2.0]], [[1.0, 
2.0, 2.0], [4.0, 8.0, 2.0], [1.0, 7.0, 6.0]], [[2.0, 6.0, 8.0], [8.0, 4.0, 8.0], [2.0, 3.0, 7.0]]], [[[8.0, 8.0, 2.0], [5.0, 
7.0, 8.0], [4.0, 1.0, 6.0]], [[5.0, 2.0, 5.0], [6.0, 4.0, 6.0], [2.0, 2.0, 6.0]], [[1.0, 2.0, 8.0], [6.0, 3.0, 7.0], [2.0, 1.0, 
7.0]]], [[[3.0, 1.0, 1.0], [4.0, 7.0, 5.0], [2.0, 7.0, 6.0]], [[5.0, 1.0, 1.0], [7.0, 4.0, 5.0], [3.0, 8.0, 3.0]], [[5.0, 7.0, 
4.0], [1.0, 1.0, 5.0], [8.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 6.0], [6.0, 1.0, 2.0], [3.0, 8.0, 8.0]], 
[[5.0, 3.0, 6.0], [6.0, 1.0, 2.0], [8.0, 2.0, 5.0]], [[6.0, 1.0, 6.0], [6.0, 5.0, 1.0], [4.0, 2.0, 6.0]]], [[[5.0, 6.0, 3.0], 
[8.0, 4.0, 1.0], [2.0, 5.0, 7.0]], [[4.0, 1.0, 8.0], [6.0, 1.0, 1.0], [1.0, 7.0, 1.0]], [[8.0, 5.0, 3.0], [5.0, 8.0, 3.0], 
[4.0, 6.0, 4.0]]], [[[7.0, 3.0, 3.0], [6.0, 2.0, 6.0], [3.0, 3.0, 5.0]], [[2.0, 3.0, 4.0], [4.0, 5.0, 2.0], [6.0, 3.0, 2.0]], 
[[2.0, 4.0, 5.0], [5.0, 8.0, 1.0], [6.0, 4.0, 8.0]]], [[[2.0, 8.0, 5.0], [8.0, 5.0, 2.0], [7.0, 7.0, 5.0]], [[5.0, 4.0, 3.0], 
[3.0, 5.0, 7.0], [6.0, 7.0, 6.0]], [[5.0, 3.0, 8.0], [3.0, 3.0, 7.0], [3.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRPL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 5.0], [7.0, 3.0, 2.0], [7.0, 7.0, 4.0]], [[4.0, 2.0, 3.0], [7.0, 3.0, 4.0], [3.0, 
7.0, 7.0]], [[3.0, 3.0, 3.0], [4.0, 8.0, 4.0], [5.0, 5.0, 6.0]]], [[[6.0, 4.0, 7.0], [3.0, 7.0, 3.0], [4.0, 1.0, 6.0]], [[8.0, 
7.0, 5.0], [1.0, 5.0, 3.0], [5.0, 6.0, 2.0]], [[8.0, 8.0, 6.0], [5.0, 4.0, 8.0], [4.0, 8.0, 2.0]]], [[[4.0, 5.0, 1.0], [1.0, 
7.0, 6.0], [6.0, 6.0, 7.0]], [[6.0, 2.0, 8.0], [4.0, 6.0, 5.0], [1.0, 1.0, 7.0]], [[2.0, 6.0, 5.0], [1.0, 5.0, 8.0], [4.0, 5.0, 
5.0]]], [[[2.0, 3.0, 4.0], [2.0, 2.0, 8.0], [6.0, 7.0, 6.0]], [[4.0, 3.0, 3.0], [8.0, 1.0, 7.0], [7.0, 2.0, 7.0]], [[2.0, 5.0, 
2.0], [6.0, 1.0, 1.0], [4.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 2.0], [5.0, 6.0, 2.0], [3.0, 4.0, 4.0]], 
[[8.0, 8.0, 7.0], [7.0, 8.0, 8.0], [1.0, 4.0, 3.0]], [[4.0, 4.0, 1.0], [2.0, 6.0, 2.0], [5.0, 5.0, 3.0]]], [[[3.0, 7.0, 8.0], 
[2.0, 7.0, 7.0], [8.0, 3.0, 1.0]], [[7.0, 7.0, 5.0], [2.0, 1.0, 3.0], [3.0, 2.0, 4.0]], [[1.0, 1.0, 6.0], [1.0, 6.0, 6.0], 
[8.0, 5.0, 5.0]]], [[[8.0, 5.0, 8.0], [1.0, 8.0, 3.0], [5.0, 3.0, 3.0]], [[1.0, 3.0, 6.0], [4.0, 4.0, 1.0], [6.0, 1.0, 5.0]], 
[[2.0, 4.0, 6.0], [8.0, 6.0, 1.0], [6.0, 5.0, 1.0]]], [[[2.0, 5.0, 1.0], [4.0, 2.0, 2.0], [5.0, 2.0, 6.0]], [[7.0, 7.0, 5.0], 
[3.0, 7.0, 3.0], [5.0, 4.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 1.0, 6.0], [6.0, 4.0, 1.0]]]])
      val=Data(numpy.array([[[8.0]], [[2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0]], [[1.0]]]))
      arg[1:3,1:2,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 5.0], [7.0, 3.0, 2.0], [7.0, 7.0, 4.0]], [[4.0, 2.0, 3.0], [7.0, 3.0, 4.0], [3.0, 
7.0, 7.0]], [[3.0, 3.0, 3.0], [4.0, 8.0, 4.0], [5.0, 5.0, 6.0]]], [[[6.0, 4.0, 7.0], [3.0, 7.0, 3.0], [4.0, 1.0, 6.0]], [[8.0, 
7.0, 5.0], [1.0, 5.0, 8.0], [5.0, 6.0, 2.0]], [[8.0, 8.0, 6.0], [5.0, 4.0, 8.0], [4.0, 8.0, 2.0]]], [[[4.0, 5.0, 1.0], [1.0, 
7.0, 6.0], [6.0, 6.0, 7.0]], [[6.0, 2.0, 8.0], [4.0, 6.0, 2.0], [1.0, 1.0, 7.0]], [[2.0, 6.0, 5.0], [1.0, 5.0, 8.0], [4.0, 5.0, 
5.0]]], [[[2.0, 3.0, 4.0], [2.0, 2.0, 8.0], [6.0, 7.0, 6.0]], [[4.0, 3.0, 3.0], [8.0, 1.0, 7.0], [7.0, 2.0, 7.0]], [[2.0, 5.0, 
2.0], [6.0, 1.0, 1.0], [4.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 2.0], [5.0, 6.0, 2.0], [3.0, 4.0, 4.0]], 
[[8.0, 8.0, 7.0], [7.0, 8.0, 8.0], [1.0, 4.0, 3.0]], [[4.0, 4.0, 1.0], [2.0, 6.0, 2.0], [5.0, 5.0, 3.0]]], [[[3.0, 7.0, 8.0], 
[2.0, 7.0, 7.0], [8.0, 3.0, 1.0]], [[7.0, 7.0, 5.0], [2.0, 1.0, 7.0], [3.0, 2.0, 4.0]], [[1.0, 1.0, 6.0], [1.0, 6.0, 6.0], 
[8.0, 5.0, 5.0]]], [[[8.0, 5.0, 8.0], [1.0, 8.0, 3.0], [5.0, 3.0, 3.0]], [[1.0, 3.0, 6.0], [4.0, 4.0, 1.0], [6.0, 1.0, 5.0]], 
[[2.0, 4.0, 6.0], [8.0, 6.0, 1.0], [6.0, 5.0, 1.0]]], [[[2.0, 5.0, 1.0], [4.0, 2.0, 2.0], [5.0, 2.0, 6.0]], [[7.0, 7.0, 5.0], 
[3.0, 7.0, 3.0], [5.0, 4.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 1.0, 6.0], [6.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CPRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 6.0], [3.0, 4.0, 2.0], [6.0, 3.0, 1.0]], [[6.0, 3.0, 1.0], [6.0, 5.0, 8.0], [4.0, 
8.0, 2.0]], [[2.0, 6.0, 7.0], [2.0, 1.0, 1.0], [1.0, 7.0, 2.0]]], [[[1.0, 7.0, 8.0], [5.0, 6.0, 4.0], [3.0, 3.0, 8.0]], [[6.0, 
8.0, 2.0], [1.0, 7.0, 8.0], [4.0, 8.0, 7.0]], [[6.0, 7.0, 7.0], [3.0, 5.0, 2.0], [2.0, 6.0, 1.0]]], [[[3.0, 6.0, 4.0], [8.0, 
5.0, 7.0], [7.0, 8.0, 8.0]], [[6.0, 8.0, 3.0], [1.0, 8.0, 7.0], [5.0, 6.0, 8.0]], [[3.0, 1.0, 7.0], [6.0, 3.0, 7.0], [2.0, 6.0, 
6.0]]], [[[3.0, 4.0, 4.0], [2.0, 3.0, 2.0], [2.0, 3.0, 7.0]], [[5.0, 7.0, 5.0], [4.0, 1.0, 5.0], [4.0, 5.0, 1.0]], [[3.0, 3.0, 
3.0], [6.0, 1.0, 1.0], [8.0, 5.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 2.0], [1.0, 3.0, 2.0], [1.0, 4.0, 3.0]], 
[[4.0, 6.0, 6.0], [5.0, 1.0, 7.0], [1.0, 4.0, 6.0]], [[3.0, 4.0, 8.0], [4.0, 5.0, 4.0], [6.0, 2.0, 3.0]]], [[[4.0, 6.0, 2.0], 
[4.0, 3.0, 8.0], [5.0, 2.0, 3.0]], [[5.0, 3.0, 6.0], [6.0, 5.0, 3.0], [4.0, 1.0, 3.0]], [[6.0, 4.0, 8.0], [1.0, 6.0, 8.0], 
[3.0, 3.0, 5.0]]], [[[2.0, 2.0, 8.0], [3.0, 7.0, 4.0], [7.0, 6.0, 5.0]], [[3.0, 7.0, 8.0], [1.0, 8.0, 2.0], [3.0, 1.0, 2.0]], 
[[5.0, 6.0, 3.0], [2.0, 6.0, 3.0], [4.0, 1.0, 4.0]]], [[[4.0, 3.0, 6.0], [8.0, 1.0, 3.0], [4.0, 7.0, 6.0]], [[7.0, 8.0, 2.0], 
[5.0, 8.0, 1.0], [7.0, 3.0, 1.0]], [[8.0, 3.0, 3.0], [8.0, 7.0, 2.0], [7.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[[8.0]], [[7.0]], [[2.0]], [[1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0]], [[5.0]], [[4.0]], [[1.0]]]))
      arg[:,1,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 6.0], [3.0, 4.0, 2.0], [6.0, 3.0, 1.0]], [[6.0, 3.0, 1.0], [6.0, 8.0, 8.0], [4.0, 
8.0, 2.0]], [[2.0, 6.0, 7.0], [2.0, 1.0, 1.0], [1.0, 7.0, 2.0]]], [[[1.0, 7.0, 8.0], [5.0, 6.0, 4.0], [3.0, 3.0, 8.0]], [[6.0, 
8.0, 2.0], [1.0, 7.0, 8.0], [4.0, 8.0, 7.0]], [[6.0, 7.0, 7.0], [3.0, 5.0, 2.0], [2.0, 6.0, 1.0]]], [[[3.0, 6.0, 4.0], [8.0, 
5.0, 7.0], [7.0, 8.0, 8.0]], [[6.0, 8.0, 3.0], [1.0, 2.0, 7.0], [5.0, 6.0, 8.0]], [[3.0, 1.0, 7.0], [6.0, 3.0, 7.0], [2.0, 6.0, 
6.0]]], [[[3.0, 4.0, 4.0], [2.0, 3.0, 2.0], [2.0, 3.0, 7.0]], [[5.0, 7.0, 5.0], [4.0, 1.0, 5.0], [4.0, 5.0, 1.0]], [[3.0, 3.0, 
3.0], [6.0, 1.0, 1.0], [8.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 2.0], [1.0, 3.0, 2.0], [1.0, 4.0, 3.0]], 
[[4.0, 6.0, 6.0], [5.0, 5.0, 7.0], [1.0, 4.0, 6.0]], [[3.0, 4.0, 8.0], [4.0, 5.0, 4.0], [6.0, 2.0, 3.0]]], [[[4.0, 6.0, 2.0], 
[4.0, 3.0, 8.0], [5.0, 2.0, 3.0]], [[5.0, 3.0, 6.0], [6.0, 5.0, 3.0], [4.0, 1.0, 3.0]], [[6.0, 4.0, 8.0], [1.0, 6.0, 8.0], 
[3.0, 3.0, 5.0]]], [[[2.0, 2.0, 8.0], [3.0, 7.0, 4.0], [7.0, 6.0, 5.0]], [[3.0, 7.0, 8.0], [1.0, 4.0, 2.0], [3.0, 1.0, 2.0]], 
[[5.0, 6.0, 3.0], [2.0, 6.0, 3.0], [4.0, 1.0, 4.0]]], [[[4.0, 3.0, 6.0], [8.0, 1.0, 3.0], [4.0, 7.0, 6.0]], [[7.0, 8.0, 2.0], 
[5.0, 1.0, 1.0], [7.0, 3.0, 1.0]], [[8.0, 3.0, 3.0], [8.0, 7.0, 2.0], [7.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULLC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 5.0], [6.0, 6.0, 3.0], [5.0, 3.0, 7.0]], [[6.0, 5.0, 5.0], [7.0, 2.0, 8.0], [7.0, 
2.0, 6.0]], [[1.0, 7.0, 1.0], [5.0, 5.0, 5.0], [2.0, 3.0, 8.0]]], [[[4.0, 6.0, 6.0], [5.0, 1.0, 5.0], [2.0, 7.0, 1.0]], [[6.0, 
6.0, 8.0], [4.0, 1.0, 2.0], [3.0, 6.0, 3.0]], [[6.0, 2.0, 7.0], [6.0, 3.0, 8.0], [8.0, 7.0, 1.0]]], [[[2.0, 1.0, 6.0], [4.0, 
1.0, 4.0], [4.0, 7.0, 7.0]], [[8.0, 8.0, 4.0], [1.0, 7.0, 6.0], [2.0, 8.0, 4.0]], [[7.0, 3.0, 5.0], [4.0, 6.0, 6.0], [4.0, 5.0, 
7.0]]], [[[4.0, 4.0, 7.0], [4.0, 3.0, 3.0], [6.0, 4.0, 3.0]], [[7.0, 5.0, 5.0], [8.0, 3.0, 3.0], [8.0, 8.0, 6.0]], [[4.0, 6.0, 
1.0], [1.0, 8.0, 6.0], [7.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 6.0], [1.0, 3.0, 2.0], [8.0, 8.0, 4.0]], 
[[2.0, 1.0, 5.0], [5.0, 6.0, 2.0], [7.0, 2.0, 4.0]], [[1.0, 3.0, 1.0], [4.0, 2.0, 4.0], [6.0, 4.0, 3.0]]], [[[7.0, 1.0, 4.0], 
[4.0, 1.0, 7.0], [4.0, 8.0, 5.0]], [[8.0, 3.0, 6.0], [2.0, 7.0, 8.0], [7.0, 6.0, 3.0]], [[8.0, 1.0, 4.0], [1.0, 1.0, 8.0], 
[3.0, 7.0, 2.0]]], [[[3.0, 7.0, 2.0], [6.0, 6.0, 1.0], [3.0, 8.0, 8.0]], [[4.0, 4.0, 8.0], [1.0, 7.0, 5.0], [4.0, 6.0, 2.0]], 
[[6.0, 8.0, 7.0], [7.0, 7.0, 5.0], [5.0, 2.0, 2.0]]], [[[6.0, 7.0, 8.0], [7.0, 2.0, 1.0], [7.0, 3.0, 1.0]], [[3.0, 4.0, 1.0], 
[2.0, 5.0, 6.0], [7.0, 2.0, 8.0]], [[3.0, 5.0, 3.0], [5.0, 1.0, 6.0], [5.0, 2.0, 4.0]]]])
      val=Data(numpy.array([[[[7.0, 6.0, 1.0]]], [[[4.0, 7.0, 1.0]]], [[[6.0, 8.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 8.0]]], [[[2.0, 4.0, 5.0]]], [[[1.0, 7.0, 4.0]]]]))
      arg[:3,2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 5.0], [6.0, 6.0, 3.0], [5.0, 3.0, 7.0]], [[6.0, 5.0, 5.0], [7.0, 2.0, 8.0], [7.0, 
2.0, 6.0]], [[1.0, 7.0, 1.0], [5.0, 5.0, 5.0], [7.0, 6.0, 1.0]]], [[[4.0, 6.0, 6.0], [5.0, 1.0, 5.0], [2.0, 7.0, 1.0]], [[6.0, 
6.0, 8.0], [4.0, 1.0, 2.0], [3.0, 6.0, 3.0]], [[6.0, 2.0, 7.0], [6.0, 3.0, 8.0], [4.0, 7.0, 1.0]]], [[[2.0, 1.0, 6.0], [4.0, 
1.0, 4.0], [4.0, 7.0, 7.0]], [[8.0, 8.0, 4.0], [1.0, 7.0, 6.0], [2.0, 8.0, 4.0]], [[7.0, 3.0, 5.0], [4.0, 6.0, 6.0], [6.0, 8.0, 
4.0]]], [[[4.0, 4.0, 7.0], [4.0, 3.0, 3.0], [6.0, 4.0, 3.0]], [[7.0, 5.0, 5.0], [8.0, 3.0, 3.0], [8.0, 8.0, 6.0]], [[4.0, 6.0, 
1.0], [1.0, 8.0, 6.0], [7.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 6.0], [1.0, 3.0, 2.0], [8.0, 8.0, 4.0]], 
[[2.0, 1.0, 5.0], [5.0, 6.0, 2.0], [7.0, 2.0, 4.0]], [[1.0, 3.0, 1.0], [4.0, 2.0, 4.0], [5.0, 5.0, 8.0]]], [[[7.0, 1.0, 4.0], 
[4.0, 1.0, 7.0], [4.0, 8.0, 5.0]], [[8.0, 3.0, 6.0], [2.0, 7.0, 8.0], [7.0, 6.0, 3.0]], [[8.0, 1.0, 4.0], [1.0, 1.0, 8.0], 
[2.0, 4.0, 5.0]]], [[[3.0, 7.0, 2.0], [6.0, 6.0, 1.0], [3.0, 8.0, 8.0]], [[4.0, 4.0, 8.0], [1.0, 7.0, 5.0], [4.0, 6.0, 2.0]], 
[[6.0, 8.0, 7.0], [7.0, 7.0, 5.0], [1.0, 7.0, 4.0]]], [[[6.0, 7.0, 8.0], [7.0, 2.0, 1.0], [7.0, 3.0, 1.0]], [[3.0, 4.0, 1.0], 
[2.0, 5.0, 6.0], [7.0, 2.0, 8.0]], [[3.0, 5.0, 3.0], [5.0, 1.0, 6.0], [5.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 1.0], [7.0, 3.0, 5.0], [6.0, 6.0, 5.0]], [[5.0, 2.0, 2.0], [6.0, 4.0, 8.0], [3.0, 
6.0, 1.0]], [[3.0, 7.0, 5.0], [2.0, 5.0, 7.0], [1.0, 3.0, 1.0]]], [[[5.0, 1.0, 7.0], [2.0, 5.0, 5.0], [3.0, 4.0, 4.0]], [[8.0, 
1.0, 5.0], [5.0, 7.0, 1.0], [1.0, 7.0, 8.0]], [[1.0, 7.0, 4.0], [3.0, 4.0, 3.0], [2.0, 3.0, 6.0]]], [[[6.0, 1.0, 5.0], [8.0, 
2.0, 7.0], [2.0, 1.0, 8.0]], [[4.0, 3.0, 7.0], [7.0, 8.0, 6.0], [6.0, 1.0, 7.0]], [[5.0, 1.0, 8.0], [2.0, 7.0, 5.0], [7.0, 8.0, 
2.0]]], [[[4.0, 7.0, 6.0], [7.0, 5.0, 3.0], [6.0, 6.0, 2.0]], [[1.0, 5.0, 2.0], [5.0, 6.0, 5.0], [5.0, 1.0, 7.0]], [[3.0, 5.0, 
5.0], [4.0, 2.0, 2.0], [8.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 1.0], [2.0, 2.0, 7.0], [7.0, 4.0, 3.0]], 
[[4.0, 4.0, 3.0], [6.0, 3.0, 1.0], [6.0, 7.0, 5.0]], [[3.0, 2.0, 1.0], [3.0, 5.0, 1.0], [5.0, 5.0, 2.0]]], [[[1.0, 2.0, 8.0], 
[2.0, 7.0, 4.0], [4.0, 2.0, 2.0]], [[7.0, 6.0, 6.0], [4.0, 5.0, 6.0], [4.0, 5.0, 3.0]], [[8.0, 7.0, 2.0], [1.0, 6.0, 5.0], 
[1.0, 2.0, 2.0]]], [[[7.0, 1.0, 7.0], [4.0, 2.0, 8.0], [7.0, 5.0, 3.0]], [[5.0, 1.0, 6.0], [4.0, 7.0, 5.0], [2.0, 2.0, 6.0]], 
[[4.0, 8.0, 3.0], [5.0, 2.0, 5.0], [2.0, 7.0, 7.0]]], [[[2.0, 4.0, 3.0], [7.0, 1.0, 4.0], [1.0, 8.0, 6.0]], [[8.0, 2.0, 2.0], 
[3.0, 5.0, 1.0], [6.0, 1.0, 1.0]], [[2.0, 1.0, 1.0], [5.0, 1.0, 4.0], [1.0, 1.0, 8.0]]]])
      val=Data(numpy.array([[[[8.0, 6.0, 5.0], [3.0, 6.0, 1.0], [1.0, 7.0, 6.0]]], [[[3.0, 1.0, 7.0], [4.0, 2.0, 1.0], [1.0, 
5.0, 8.0]]], [[[4.0, 6.0, 6.0], [7.0, 1.0, 1.0], [2.0, 8.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 2.0], [6.0, 3.0, 6.0], [7.0, 7.0, 4.0]]], [[[8.0, 6.0, 8.0], [2.0, 6.0, 
4.0], [5.0, 8.0, 4.0]]], [[[2.0, 6.0, 1.0], [3.0, 1.0, 1.0], [1.0, 5.0, 7.0]]]]))
      arg[:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 1.0], [7.0, 3.0, 5.0], [6.0, 6.0, 5.0]], [[8.0, 6.0, 5.0], [3.0, 6.0, 1.0], [1.0, 
7.0, 6.0]], [[3.0, 7.0, 5.0], [2.0, 5.0, 7.0], [1.0, 3.0, 1.0]]], [[[5.0, 1.0, 7.0], [2.0, 5.0, 5.0], [3.0, 4.0, 4.0]], [[3.0, 
1.0, 7.0], [4.0, 2.0, 1.0], [1.0, 5.0, 8.0]], [[1.0, 7.0, 4.0], [3.0, 4.0, 3.0], [2.0, 3.0, 6.0]]], [[[6.0, 1.0, 5.0], [8.0, 
2.0, 7.0], [2.0, 1.0, 8.0]], [[4.0, 6.0, 6.0], [7.0, 1.0, 1.0], [2.0, 8.0, 8.0]], [[5.0, 1.0, 8.0], [2.0, 7.0, 5.0], [7.0, 8.0, 
2.0]]], [[[4.0, 7.0, 6.0], [7.0, 5.0, 3.0], [6.0, 6.0, 2.0]], [[1.0, 5.0, 2.0], [5.0, 6.0, 5.0], [5.0, 1.0, 7.0]], [[3.0, 5.0, 
5.0], [4.0, 2.0, 2.0], [8.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 1.0], [2.0, 2.0, 7.0], [7.0, 4.0, 3.0]], 
[[7.0, 5.0, 2.0], [6.0, 3.0, 6.0], [7.0, 7.0, 4.0]], [[3.0, 2.0, 1.0], [3.0, 5.0, 1.0], [5.0, 5.0, 2.0]]], [[[1.0, 2.0, 8.0], 
[2.0, 7.0, 4.0], [4.0, 2.0, 2.0]], [[8.0, 6.0, 8.0], [2.0, 6.0, 4.0], [5.0, 8.0, 4.0]], [[8.0, 7.0, 2.0], [1.0, 6.0, 5.0], 
[1.0, 2.0, 2.0]]], [[[7.0, 1.0, 7.0], [4.0, 2.0, 8.0], [7.0, 5.0, 3.0]], [[2.0, 6.0, 1.0], [3.0, 1.0, 1.0], [1.0, 5.0, 7.0]], 
[[4.0, 8.0, 3.0], [5.0, 2.0, 5.0], [2.0, 7.0, 7.0]]], [[[2.0, 4.0, 3.0], [7.0, 1.0, 4.0], [1.0, 8.0, 6.0]], [[8.0, 2.0, 2.0], 
[3.0, 5.0, 1.0], [6.0, 1.0, 1.0]], [[2.0, 1.0, 1.0], [5.0, 1.0, 4.0], [1.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PUCU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 7.0], [3.0, 4.0, 3.0], [8.0, 4.0, 2.0]], [[3.0, 1.0, 7.0], [7.0, 6.0, 1.0], [5.0, 
3.0, 1.0]], [[2.0, 1.0, 2.0], [8.0, 8.0, 8.0], [1.0, 8.0, 5.0]]], [[[3.0, 8.0, 1.0], [1.0, 6.0, 2.0], [8.0, 6.0, 2.0]], [[1.0, 
6.0, 8.0], [5.0, 6.0, 8.0], [8.0, 2.0, 2.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 1.0], [4.0, 7.0, 1.0]]], [[[5.0, 2.0, 3.0], [6.0, 
8.0, 7.0], [3.0, 4.0, 4.0]], [[8.0, 8.0, 4.0], [1.0, 4.0, 2.0], [4.0, 5.0, 5.0]], [[4.0, 5.0, 4.0], [4.0, 2.0, 5.0], [2.0, 1.0, 
5.0]]], [[[2.0, 8.0, 3.0], [5.0, 5.0, 7.0], [7.0, 5.0, 4.0]], [[4.0, 3.0, 2.0], [3.0, 1.0, 2.0], [5.0, 8.0, 1.0]], [[5.0, 8.0, 
8.0], [3.0, 3.0, 1.0], [4.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 1.0], [6.0, 1.0, 7.0], [8.0, 4.0, 5.0]], 
[[6.0, 2.0, 6.0], [4.0, 7.0, 1.0], [4.0, 5.0, 6.0]], [[6.0, 3.0, 2.0], [2.0, 3.0, 4.0], [1.0, 8.0, 5.0]]], [[[7.0, 8.0, 7.0], 
[3.0, 7.0, 4.0], [7.0, 5.0, 3.0]], [[5.0, 5.0, 5.0], [5.0, 6.0, 3.0], [5.0, 6.0, 2.0]], [[6.0, 2.0, 1.0], [6.0, 7.0, 6.0], 
[1.0, 7.0, 3.0]]], [[[4.0, 3.0, 7.0], [4.0, 8.0, 3.0], [3.0, 8.0, 8.0]], [[6.0, 1.0, 1.0], [1.0, 8.0, 8.0], [3.0, 4.0, 6.0]], 
[[1.0, 4.0, 3.0], [6.0, 6.0, 4.0], [8.0, 1.0, 8.0]]], [[[8.0, 3.0, 6.0], [4.0, 1.0, 2.0], [5.0, 5.0, 3.0]], [[4.0, 4.0, 5.0], 
[8.0, 3.0, 6.0], [5.0, 7.0, 6.0]], [[8.0, 5.0, 5.0], [6.0, 4.0, 1.0], [5.0, 8.0, 3.0]]]])
      val=Data(numpy.array([[[5.0, 1.0], [8.0, 4.0], [3.0, 5.0]], [[4.0, 3.0], [5.0, 5.0], [7.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 4.0], [2.0, 8.0], [3.0, 2.0]], [[6.0, 3.0], [3.0, 2.0], [7.0, 8.0]]]))
      arg[2,:2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 7.0], [3.0, 4.0, 3.0], [8.0, 4.0, 2.0]], [[3.0, 1.0, 7.0], [7.0, 6.0, 1.0], [5.0, 
3.0, 1.0]], [[2.0, 1.0, 2.0], [8.0, 8.0, 8.0], [1.0, 8.0, 5.0]]], [[[3.0, 8.0, 1.0], [1.0, 6.0, 2.0], [8.0, 6.0, 2.0]], [[1.0, 
6.0, 8.0], [5.0, 6.0, 8.0], [8.0, 2.0, 2.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 1.0], [4.0, 7.0, 1.0]]], [[[5.0, 1.0, 3.0], [8.0, 
4.0, 7.0], [3.0, 5.0, 4.0]], [[4.0, 3.0, 4.0], [5.0, 5.0, 2.0], [7.0, 8.0, 5.0]], [[4.0, 5.0, 4.0], [4.0, 2.0, 5.0], [2.0, 1.0, 
5.0]]], [[[2.0, 8.0, 3.0], [5.0, 5.0, 7.0], [7.0, 5.0, 4.0]], [[4.0, 3.0, 2.0], [3.0, 1.0, 2.0], [5.0, 8.0, 1.0]], [[5.0, 8.0, 
8.0], [3.0, 3.0, 1.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 1.0], [6.0, 1.0, 7.0], [8.0, 4.0, 5.0]], 
[[6.0, 2.0, 6.0], [4.0, 7.0, 1.0], [4.0, 5.0, 6.0]], [[6.0, 3.0, 2.0], [2.0, 3.0, 4.0], [1.0, 8.0, 5.0]]], [[[7.0, 8.0, 7.0], 
[3.0, 7.0, 4.0], [7.0, 5.0, 3.0]], [[5.0, 5.0, 5.0], [5.0, 6.0, 3.0], [5.0, 6.0, 2.0]], [[6.0, 2.0, 1.0], [6.0, 7.0, 6.0], 
[1.0, 7.0, 3.0]]], [[[3.0, 4.0, 7.0], [2.0, 8.0, 3.0], [3.0, 2.0, 8.0]], [[6.0, 3.0, 1.0], [3.0, 2.0, 8.0], [7.0, 8.0, 6.0]], 
[[1.0, 4.0, 3.0], [6.0, 6.0, 4.0], [8.0, 1.0, 8.0]]], [[[8.0, 3.0, 6.0], [4.0, 1.0, 2.0], [5.0, 5.0, 3.0]], [[4.0, 4.0, 5.0], 
[8.0, 3.0, 6.0], [5.0, 7.0, 6.0]], [[8.0, 5.0, 5.0], [6.0, 4.0, 1.0], [5.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 3.0], [8.0, 7.0, 3.0], [1.0, 7.0, 5.0]], [[6.0, 1.0, 2.0], [3.0, 7.0, 1.0], [5.0, 
3.0, 8.0]], [[8.0, 3.0, 3.0], [4.0, 8.0, 5.0], [8.0, 4.0, 7.0]]], [[[3.0, 6.0, 5.0], [7.0, 4.0, 8.0], [3.0, 3.0, 7.0]], [[4.0, 
5.0, 1.0], [3.0, 5.0, 4.0], [2.0, 5.0, 7.0]], [[6.0, 8.0, 8.0], [1.0, 4.0, 5.0], [2.0, 2.0, 7.0]]], [[[7.0, 7.0, 2.0], [6.0, 
1.0, 3.0], [2.0, 8.0, 4.0]], [[5.0, 3.0, 8.0], [6.0, 2.0, 7.0], [1.0, 1.0, 3.0]], [[7.0, 4.0, 6.0], [3.0, 5.0, 5.0], [7.0, 1.0, 
4.0]]], [[[2.0, 7.0, 7.0], [2.0, 3.0, 3.0], [2.0, 4.0, 3.0]], [[4.0, 2.0, 7.0], [7.0, 3.0, 8.0], [1.0, 4.0, 4.0]], [[5.0, 7.0, 
4.0], [8.0, 4.0, 7.0], [7.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 1.0], [1.0, 2.0, 1.0], [6.0, 6.0, 1.0]], 
[[7.0, 3.0, 1.0], [1.0, 1.0, 4.0], [3.0, 7.0, 2.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 7.0], [5.0, 6.0, 7.0]]], [[[5.0, 8.0, 6.0], 
[3.0, 6.0, 1.0], [7.0, 5.0, 2.0]], [[3.0, 4.0, 4.0], [2.0, 1.0, 3.0], [4.0, 8.0, 4.0]], [[6.0, 6.0, 5.0], [7.0, 5.0, 8.0], 
[4.0, 2.0, 4.0]]], [[[1.0, 2.0, 2.0], [6.0, 2.0, 7.0], [5.0, 4.0, 8.0]], [[6.0, 8.0, 3.0], [5.0, 5.0, 1.0], [5.0, 7.0, 6.0]], 
[[2.0, 1.0, 7.0], [7.0, 2.0, 2.0], [6.0, 4.0, 4.0]]], [[[6.0, 7.0, 8.0], [7.0, 2.0, 2.0], [1.0, 4.0, 1.0]], [[2.0, 8.0, 7.0], 
[6.0, 7.0, 7.0], [6.0, 4.0, 7.0]], [[3.0, 2.0, 4.0], [6.0, 2.0, 4.0], [1.0, 7.0, 6.0]]]])
      val=Data(numpy.array([[[[8.0, 2.0, 7.0], [4.0, 7.0, 6.0], [1.0, 8.0, 5.0]], [[1.0, 7.0, 6.0], [1.0, 8.0, 1.0], [5.0, 2.0, 
1.0]], [[5.0, 1.0, 8.0], [6.0, 2.0, 8.0], [6.0, 4.0, 4.0]]], [[[1.0, 3.0, 4.0], [4.0, 1.0, 7.0], [7.0, 1.0, 1.0]], [[3.0, 3.0, 
3.0], [8.0, 3.0, 4.0], [4.0, 3.0, 7.0]], [[4.0, 1.0, 5.0], [3.0, 2.0, 5.0], [4.0, 7.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 6.0], [5.0, 4.0, 5.0], [2.0, 7.0, 8.0]], [[1.0, 7.0, 8.0], [4.0, 1.0, 
5.0], [3.0, 5.0, 3.0]], [[8.0, 1.0, 1.0], [4.0, 1.0, 5.0], [3.0, 1.0, 7.0]]], [[[7.0, 3.0, 3.0], [4.0, 7.0, 8.0], [2.0, 5.0, 
5.0]], [[3.0, 4.0, 1.0], [5.0, 7.0, 8.0], [2.0, 4.0, 6.0]], [[6.0, 5.0, 2.0], [6.0, 1.0, 8.0], [2.0, 8.0, 7.0]]]]))
      arg[2:,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 3.0], [8.0, 7.0, 3.0], [1.0, 7.0, 5.0]], [[6.0, 1.0, 2.0], [3.0, 7.0, 1.0], [5.0, 
3.0, 8.0]], [[8.0, 3.0, 3.0], [4.0, 8.0, 5.0], [8.0, 4.0, 7.0]]], [[[3.0, 6.0, 5.0], [7.0, 4.0, 8.0], [3.0, 3.0, 7.0]], [[4.0, 
5.0, 1.0], [3.0, 5.0, 4.0], [2.0, 5.0, 7.0]], [[6.0, 8.0, 8.0], [1.0, 4.0, 5.0], [2.0, 2.0, 7.0]]], [[[8.0, 2.0, 7.0], [4.0, 
7.0, 6.0], [1.0, 8.0, 5.0]], [[1.0, 7.0, 6.0], [1.0, 8.0, 1.0], [5.0, 2.0, 1.0]], [[5.0, 1.0, 8.0], [6.0, 2.0, 8.0], [6.0, 4.0, 
4.0]]], [[[1.0, 3.0, 4.0], [4.0, 1.0, 7.0], [7.0, 1.0, 1.0]], [[3.0, 3.0, 3.0], [8.0, 3.0, 4.0], [4.0, 3.0, 7.0]], [[4.0, 1.0, 
5.0], [3.0, 2.0, 5.0], [4.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 1.0], [1.0, 2.0, 1.0], [6.0, 6.0, 1.0]], 
[[7.0, 3.0, 1.0], [1.0, 1.0, 4.0], [3.0, 7.0, 2.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 7.0], [5.0, 6.0, 7.0]]], [[[5.0, 8.0, 6.0], 
[3.0, 6.0, 1.0], [7.0, 5.0, 2.0]], [[3.0, 4.0, 4.0], [2.0, 1.0, 3.0], [4.0, 8.0, 4.0]], [[6.0, 6.0, 5.0], [7.0, 5.0, 8.0], 
[4.0, 2.0, 4.0]]], [[[7.0, 8.0, 6.0], [5.0, 4.0, 5.0], [2.0, 7.0, 8.0]], [[1.0, 7.0, 8.0], [4.0, 1.0, 5.0], [3.0, 5.0, 3.0]], 
[[8.0, 1.0, 1.0], [4.0, 1.0, 5.0], [3.0, 1.0, 7.0]]], [[[7.0, 3.0, 3.0], [4.0, 7.0, 8.0], [2.0, 5.0, 5.0]], [[3.0, 4.0, 1.0], 
[5.0, 7.0, 8.0], [2.0, 4.0, 6.0]], [[6.0, 5.0, 2.0], [6.0, 1.0, 8.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CPUL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 6.0], [3.0, 4.0, 2.0], [4.0, 8.0, 4.0]], [[1.0, 3.0, 2.0], [8.0, 4.0, 8.0], [6.0, 
2.0, 3.0]], [[1.0, 6.0, 7.0], [2.0, 8.0, 8.0], [6.0, 4.0, 2.0]]], [[[2.0, 3.0, 2.0], [6.0, 3.0, 3.0], [3.0, 6.0, 5.0]], [[2.0, 
8.0, 4.0], [5.0, 3.0, 5.0], [3.0, 7.0, 5.0]], [[7.0, 2.0, 3.0], [4.0, 6.0, 8.0], [4.0, 7.0, 3.0]]], [[[1.0, 2.0, 3.0], [7.0, 
3.0, 2.0], [1.0, 5.0, 3.0]], [[4.0, 7.0, 7.0], [1.0, 4.0, 7.0], [2.0, 5.0, 8.0]], [[8.0, 7.0, 4.0], [7.0, 7.0, 8.0], [4.0, 8.0, 
2.0]]], [[[7.0, 3.0, 6.0], [4.0, 5.0, 6.0], [1.0, 8.0, 5.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 1.0], [6.0, 1.0, 7.0]], [[3.0, 6.0, 
2.0], [3.0, 1.0, 1.0], [5.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 5.0], [5.0, 8.0, 8.0], [2.0, 4.0, 6.0]], 
[[6.0, 6.0, 2.0], [1.0, 4.0, 1.0], [7.0, 4.0, 8.0]], [[2.0, 5.0, 3.0], [3.0, 4.0, 7.0], [4.0, 5.0, 7.0]]], [[[8.0, 1.0, 3.0], 
[6.0, 4.0, 6.0], [7.0, 5.0, 2.0]], [[7.0, 8.0, 4.0], [6.0, 3.0, 5.0], [6.0, 4.0, 8.0]], [[6.0, 6.0, 6.0], [8.0, 2.0, 6.0], 
[2.0, 2.0, 4.0]]], [[[3.0, 2.0, 4.0], [8.0, 5.0, 7.0], [7.0, 8.0, 8.0]], [[5.0, 4.0, 3.0], [7.0, 5.0, 4.0], [5.0, 8.0, 7.0]], 
[[8.0, 6.0, 7.0], [6.0, 5.0, 4.0], [5.0, 6.0, 4.0]]], [[[3.0, 1.0, 3.0], [3.0, 3.0, 5.0], [7.0, 1.0, 7.0]], [[2.0, 8.0, 2.0], 
[7.0, 2.0, 4.0], [3.0, 8.0, 8.0]], [[5.0, 6.0, 7.0], [8.0, 8.0, 3.0], [2.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[[1.0], [1.0]], [[1.0], [5.0]], [[8.0], [5.0]], [[8.0], [6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0], [1.0]], [[2.0], [5.0]], [[1.0], [8.0]], [[1.0], [8.0]]]))
      arg[:,1,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 6.0], [3.0, 4.0, 2.0], [4.0, 8.0, 4.0]], [[1.0, 3.0, 1.0], [8.0, 4.0, 1.0], [6.0, 
2.0, 3.0]], [[1.0, 6.0, 7.0], [2.0, 8.0, 8.0], [6.0, 4.0, 2.0]]], [[[2.0, 3.0, 2.0], [6.0, 3.0, 3.0], [3.0, 6.0, 5.0]], [[2.0, 
8.0, 1.0], [5.0, 3.0, 5.0], [3.0, 7.0, 5.0]], [[7.0, 2.0, 3.0], [4.0, 6.0, 8.0], [4.0, 7.0, 3.0]]], [[[1.0, 2.0, 3.0], [7.0, 
3.0, 2.0], [1.0, 5.0, 3.0]], [[4.0, 7.0, 8.0], [1.0, 4.0, 5.0], [2.0, 5.0, 8.0]], [[8.0, 7.0, 4.0], [7.0, 7.0, 8.0], [4.0, 8.0, 
2.0]]], [[[7.0, 3.0, 6.0], [4.0, 5.0, 6.0], [1.0, 8.0, 5.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 6.0], [6.0, 1.0, 7.0]], [[3.0, 6.0, 
2.0], [3.0, 1.0, 1.0], [5.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 5.0], [5.0, 8.0, 8.0], [2.0, 4.0, 6.0]], 
[[6.0, 6.0, 7.0], [1.0, 4.0, 1.0], [7.0, 4.0, 8.0]], [[2.0, 5.0, 3.0], [3.0, 4.0, 7.0], [4.0, 5.0, 7.0]]], [[[8.0, 1.0, 3.0], 
[6.0, 4.0, 6.0], [7.0, 5.0, 2.0]], [[7.0, 8.0, 2.0], [6.0, 3.0, 5.0], [6.0, 4.0, 8.0]], [[6.0, 6.0, 6.0], [8.0, 2.0, 6.0], 
[2.0, 2.0, 4.0]]], [[[3.0, 2.0, 4.0], [8.0, 5.0, 7.0], [7.0, 8.0, 8.0]], [[5.0, 4.0, 1.0], [7.0, 5.0, 8.0], [5.0, 8.0, 7.0]], 
[[8.0, 6.0, 7.0], [6.0, 5.0, 4.0], [5.0, 6.0, 4.0]]], [[[3.0, 1.0, 3.0], [3.0, 3.0, 5.0], [7.0, 1.0, 7.0]], [[2.0, 8.0, 1.0], 
[7.0, 2.0, 8.0], [3.0, 8.0, 8.0]], [[5.0, 6.0, 7.0], [8.0, 8.0, 3.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UURC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 6.0], [5.0, 2.0, 6.0], [5.0, 4.0, 8.0]], [[3.0, 6.0, 2.0], [5.0, 6.0, 2.0], [7.0, 
1.0, 2.0]], [[6.0, 5.0, 5.0], [1.0, 1.0, 5.0], [8.0, 6.0, 5.0]]], [[[8.0, 4.0, 7.0], [7.0, 7.0, 2.0], [5.0, 8.0, 8.0]], [[6.0, 
4.0, 2.0], [3.0, 4.0, 5.0], [5.0, 5.0, 2.0]], [[6.0, 1.0, 2.0], [4.0, 7.0, 5.0], [5.0, 5.0, 8.0]]], [[[1.0, 6.0, 3.0], [7.0, 
5.0, 3.0], [5.0, 1.0, 7.0]], [[1.0, 6.0, 1.0], [1.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[3.0, 6.0, 8.0], [5.0, 1.0, 8.0], [1.0, 6.0, 
8.0]]], [[[2.0, 3.0, 5.0], [1.0, 2.0, 1.0], [8.0, 7.0, 2.0]], [[1.0, 2.0, 8.0], [5.0, 3.0, 2.0], [8.0, 5.0, 7.0]], [[6.0, 1.0, 
2.0], [1.0, 4.0, 5.0], [2.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 3.0], [8.0, 2.0, 6.0], [5.0, 8.0, 1.0]], 
[[5.0, 2.0, 5.0], [3.0, 3.0, 5.0], [6.0, 2.0, 5.0]], [[7.0, 1.0, 1.0], [5.0, 2.0, 6.0], [2.0, 4.0, 2.0]]], [[[5.0, 4.0, 6.0], 
[6.0, 5.0, 6.0], [1.0, 2.0, 5.0]], [[2.0, 6.0, 3.0], [3.0, 1.0, 3.0], [8.0, 4.0, 8.0]], [[7.0, 8.0, 4.0], [4.0, 8.0, 6.0], 
[8.0, 3.0, 4.0]]], [[[3.0, 5.0, 3.0], [8.0, 5.0, 5.0], [8.0, 6.0, 1.0]], [[7.0, 3.0, 5.0], [1.0, 2.0, 7.0], [7.0, 5.0, 7.0]], 
[[4.0, 4.0, 5.0], [5.0, 6.0, 1.0], [5.0, 7.0, 6.0]]], [[[4.0, 8.0, 8.0], [6.0, 4.0, 3.0], [7.0, 5.0, 8.0]], [[2.0, 7.0, 7.0], 
[7.0, 7.0, 7.0], [5.0, 5.0, 1.0]], [[4.0, 2.0, 3.0], [3.0, 8.0, 2.0], [7.0, 2.0, 3.0]]]])
      val=Data(numpy.array([[[[4.0, 2.0, 3.0]], [[5.0, 4.0, 6.0]]], [[[6.0, 2.0, 3.0]], [[4.0, 2.0, 1.0]]], [[[3.0, 5.0, 1.0]], 
[[2.0, 2.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 7.0, 7.0]], [[8.0, 2.0, 4.0]]], [[[7.0, 2.0, 7.0]], [[2.0, 7.0, 3.0]]], [[[4.0, 
6.0, 2.0]], [[8.0, 2.0, 4.0]]]]))
      arg[:3,:2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 6.0], [4.0, 2.0, 3.0], [5.0, 4.0, 8.0]], [[3.0, 6.0, 2.0], [5.0, 4.0, 6.0], [7.0, 
1.0, 2.0]], [[6.0, 5.0, 5.0], [1.0, 1.0, 5.0], [8.0, 6.0, 5.0]]], [[[8.0, 4.0, 7.0], [6.0, 2.0, 3.0], [5.0, 8.0, 8.0]], [[6.0, 
4.0, 2.0], [4.0, 2.0, 1.0], [5.0, 5.0, 2.0]], [[6.0, 1.0, 2.0], [4.0, 7.0, 5.0], [5.0, 5.0, 8.0]]], [[[1.0, 6.0, 3.0], [3.0, 
5.0, 1.0], [5.0, 1.0, 7.0]], [[1.0, 6.0, 1.0], [2.0, 2.0, 7.0], [4.0, 5.0, 6.0]], [[3.0, 6.0, 8.0], [5.0, 1.0, 8.0], [1.0, 6.0, 
8.0]]], [[[2.0, 3.0, 5.0], [1.0, 2.0, 1.0], [8.0, 7.0, 2.0]], [[1.0, 2.0, 8.0], [5.0, 3.0, 2.0], [8.0, 5.0, 7.0]], [[6.0, 1.0, 
2.0], [1.0, 4.0, 5.0], [2.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 3.0], [6.0, 7.0, 7.0], [5.0, 8.0, 1.0]], 
[[5.0, 2.0, 5.0], [8.0, 2.0, 4.0], [6.0, 2.0, 5.0]], [[7.0, 1.0, 1.0], [5.0, 2.0, 6.0], [2.0, 4.0, 2.0]]], [[[5.0, 4.0, 6.0], 
[7.0, 2.0, 7.0], [1.0, 2.0, 5.0]], [[2.0, 6.0, 3.0], [2.0, 7.0, 3.0], [8.0, 4.0, 8.0]], [[7.0, 8.0, 4.0], [4.0, 8.0, 6.0], 
[8.0, 3.0, 4.0]]], [[[3.0, 5.0, 3.0], [4.0, 6.0, 2.0], [8.0, 6.0, 1.0]], [[7.0, 3.0, 5.0], [8.0, 2.0, 4.0], [7.0, 5.0, 7.0]], 
[[4.0, 4.0, 5.0], [5.0, 6.0, 1.0], [5.0, 7.0, 6.0]]], [[[4.0, 8.0, 8.0], [6.0, 4.0, 3.0], [7.0, 5.0, 8.0]], [[2.0, 7.0, 7.0], 
[7.0, 7.0, 7.0], [5.0, 5.0, 1.0]], [[4.0, 2.0, 3.0], [3.0, 8.0, 2.0], [7.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URPP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 2.0], [3.0, 7.0, 1.0], [6.0, 3.0, 6.0]], [[6.0, 6.0, 6.0], [3.0, 6.0, 4.0], [2.0, 
5.0, 7.0]], [[8.0, 2.0, 2.0], [8.0, 2.0, 8.0], [8.0, 4.0, 5.0]]], [[[6.0, 8.0, 6.0], [1.0, 1.0, 2.0], [8.0, 8.0, 6.0]], [[4.0, 
2.0, 7.0], [1.0, 8.0, 2.0], [3.0, 4.0, 4.0]], [[4.0, 6.0, 1.0], [1.0, 3.0, 4.0], [5.0, 1.0, 5.0]]], [[[3.0, 7.0, 2.0], [3.0, 
4.0, 5.0], [5.0, 2.0, 5.0]], [[3.0, 1.0, 6.0], [8.0, 1.0, 8.0], [6.0, 6.0, 6.0]], [[4.0, 2.0, 2.0], [8.0, 5.0, 2.0], [7.0, 4.0, 
8.0]]], [[[8.0, 1.0, 4.0], [2.0, 4.0, 6.0], [1.0, 6.0, 3.0]], [[7.0, 8.0, 6.0], [2.0, 6.0, 2.0], [6.0, 6.0, 1.0]], [[5.0, 8.0, 
2.0], [7.0, 1.0, 7.0], [8.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 2.0], [6.0, 4.0, 2.0], [8.0, 2.0, 5.0]], 
[[8.0, 1.0, 8.0], [8.0, 2.0, 4.0], [7.0, 7.0, 8.0]], [[8.0, 1.0, 5.0], [7.0, 8.0, 4.0], [4.0, 6.0, 7.0]]], [[[3.0, 6.0, 7.0], 
[6.0, 3.0, 5.0], [1.0, 3.0, 4.0]], [[3.0, 3.0, 8.0], [7.0, 8.0, 7.0], [4.0, 8.0, 3.0]], [[8.0, 2.0, 7.0], [4.0, 3.0, 7.0], 
[3.0, 7.0, 1.0]]], [[[2.0, 3.0, 8.0], [5.0, 3.0, 7.0], [5.0, 6.0, 5.0]], [[1.0, 7.0, 2.0], [8.0, 5.0, 8.0], [5.0, 7.0, 3.0]], 
[[1.0, 5.0, 3.0], [5.0, 1.0, 4.0], [2.0, 8.0, 4.0]]], [[[7.0, 2.0, 7.0], [2.0, 1.0, 6.0], [8.0, 3.0, 4.0]], [[8.0, 7.0, 2.0], 
[5.0, 1.0, 6.0], [3.0, 4.0, 6.0]], [[8.0, 1.0, 7.0], [7.0, 8.0, 6.0], [1.0, 2.0, 2.0]]]])
      val=Data(numpy.array([[7.0], [3.0], [2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0], [5.0], [8.0]]))
      arg[:3,1:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 2.0], [3.0, 7.0, 1.0], [6.0, 3.0, 6.0]], [[6.0, 6.0, 6.0], [3.0, 7.0, 4.0], [2.0, 
5.0, 7.0]], [[8.0, 2.0, 2.0], [8.0, 2.0, 8.0], [8.0, 4.0, 5.0]]], [[[6.0, 8.0, 6.0], [1.0, 1.0, 2.0], [8.0, 8.0, 6.0]], [[4.0, 
2.0, 7.0], [1.0, 3.0, 2.0], [3.0, 4.0, 4.0]], [[4.0, 6.0, 1.0], [1.0, 3.0, 4.0], [5.0, 1.0, 5.0]]], [[[3.0, 7.0, 2.0], [3.0, 
4.0, 5.0], [5.0, 2.0, 5.0]], [[3.0, 1.0, 6.0], [8.0, 2.0, 8.0], [6.0, 6.0, 6.0]], [[4.0, 2.0, 2.0], [8.0, 5.0, 2.0], [7.0, 4.0, 
8.0]]], [[[8.0, 1.0, 4.0], [2.0, 4.0, 6.0], [1.0, 6.0, 3.0]], [[7.0, 8.0, 6.0], [2.0, 6.0, 2.0], [6.0, 6.0, 1.0]], [[5.0, 8.0, 
2.0], [7.0, 1.0, 7.0], [8.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 2.0], [6.0, 4.0, 2.0], [8.0, 2.0, 5.0]], 
[[8.0, 1.0, 8.0], [8.0, 2.0, 4.0], [7.0, 7.0, 8.0]], [[8.0, 1.0, 5.0], [7.0, 8.0, 4.0], [4.0, 6.0, 7.0]]], [[[3.0, 6.0, 7.0], 
[6.0, 3.0, 5.0], [1.0, 3.0, 4.0]], [[3.0, 3.0, 8.0], [7.0, 5.0, 7.0], [4.0, 8.0, 3.0]], [[8.0, 2.0, 7.0], [4.0, 3.0, 7.0], 
[3.0, 7.0, 1.0]]], [[[2.0, 3.0, 8.0], [5.0, 3.0, 7.0], [5.0, 6.0, 5.0]], [[1.0, 7.0, 2.0], [8.0, 8.0, 8.0], [5.0, 7.0, 3.0]], 
[[1.0, 5.0, 3.0], [5.0, 1.0, 4.0], [2.0, 8.0, 4.0]]], [[[7.0, 2.0, 7.0], [2.0, 1.0, 6.0], [8.0, 3.0, 4.0]], [[8.0, 7.0, 2.0], 
[5.0, 1.0, 6.0], [3.0, 4.0, 6.0]], [[8.0, 1.0, 7.0], [7.0, 8.0, 6.0], [1.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UPRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 4.0], [5.0, 1.0, 2.0], [6.0, 1.0, 8.0]], [[7.0, 6.0, 3.0], [8.0, 8.0, 6.0], [1.0, 
6.0, 7.0]], [[4.0, 5.0, 5.0], [8.0, 6.0, 5.0], [6.0, 4.0, 1.0]]], [[[6.0, 7.0, 5.0], [5.0, 6.0, 2.0], [6.0, 7.0, 3.0]], [[1.0, 
7.0, 7.0], [4.0, 6.0, 5.0], [3.0, 6.0, 2.0]], [[2.0, 6.0, 5.0], [7.0, 4.0, 3.0], [1.0, 7.0, 5.0]]], [[[3.0, 3.0, 2.0], [5.0, 
2.0, 7.0], [5.0, 2.0, 6.0]], [[1.0, 7.0, 8.0], [5.0, 6.0, 5.0], [6.0, 1.0, 2.0]], [[3.0, 2.0, 2.0], [5.0, 8.0, 4.0], [2.0, 5.0, 
1.0]]], [[[6.0, 6.0, 2.0], [1.0, 6.0, 1.0], [4.0, 6.0, 8.0]], [[3.0, 4.0, 8.0], [5.0, 6.0, 4.0], [3.0, 1.0, 3.0]], [[1.0, 8.0, 
2.0], [6.0, 7.0, 5.0], [5.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 7.0], [6.0, 5.0, 5.0], [4.0, 4.0, 8.0]], 
[[2.0, 2.0, 8.0], [8.0, 6.0, 4.0], [2.0, 4.0, 6.0]], [[2.0, 3.0, 3.0], [6.0, 5.0, 7.0], [1.0, 4.0, 4.0]]], [[[6.0, 5.0, 1.0], 
[3.0, 2.0, 6.0], [4.0, 8.0, 7.0]], [[2.0, 7.0, 7.0], [6.0, 8.0, 6.0], [8.0, 5.0, 6.0]], [[5.0, 7.0, 4.0], [8.0, 6.0, 6.0], 
[5.0, 1.0, 8.0]]], [[[6.0, 8.0, 3.0], [4.0, 6.0, 3.0], [1.0, 8.0, 6.0]], [[4.0, 2.0, 7.0], [5.0, 5.0, 8.0], [3.0, 6.0, 4.0]], 
[[5.0, 5.0, 2.0], [1.0, 3.0, 7.0], [4.0, 5.0, 2.0]]], [[[2.0, 5.0, 8.0], [2.0, 4.0, 3.0], [5.0, 8.0, 5.0]], [[1.0, 6.0, 5.0], 
[6.0, 2.0, 3.0], [3.0, 1.0, 7.0]], [[3.0, 2.0, 6.0], [3.0, 6.0, 2.0], [2.0, 3.0, 4.0]]]])
      val=Data(numpy.array([[[4.0]], [[8.0]], [[8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0]], [[8.0]], [[2.0]]]))
      arg[:3,1,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 4.0], [5.0, 1.0, 2.0], [6.0, 1.0, 8.0]], [[7.0, 6.0, 3.0], [8.0, 4.0, 6.0], [1.0, 
6.0, 7.0]], [[4.0, 5.0, 5.0], [8.0, 6.0, 5.0], [6.0, 4.0, 1.0]]], [[[6.0, 7.0, 5.0], [5.0, 6.0, 2.0], [6.0, 7.0, 3.0]], [[1.0, 
7.0, 7.0], [4.0, 8.0, 5.0], [3.0, 6.0, 2.0]], [[2.0, 6.0, 5.0], [7.0, 4.0, 3.0], [1.0, 7.0, 5.0]]], [[[3.0, 3.0, 2.0], [5.0, 
2.0, 7.0], [5.0, 2.0, 6.0]], [[1.0, 7.0, 8.0], [5.0, 8.0, 5.0], [6.0, 1.0, 2.0]], [[3.0, 2.0, 2.0], [5.0, 8.0, 4.0], [2.0, 5.0, 
1.0]]], [[[6.0, 6.0, 2.0], [1.0, 6.0, 1.0], [4.0, 6.0, 8.0]], [[3.0, 4.0, 8.0], [5.0, 6.0, 4.0], [3.0, 1.0, 3.0]], [[1.0, 8.0, 
2.0], [6.0, 7.0, 5.0], [5.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 7.0], [6.0, 5.0, 5.0], [4.0, 4.0, 8.0]], 
[[2.0, 2.0, 8.0], [8.0, 5.0, 4.0], [2.0, 4.0, 6.0]], [[2.0, 3.0, 3.0], [6.0, 5.0, 7.0], [1.0, 4.0, 4.0]]], [[[6.0, 5.0, 1.0], 
[3.0, 2.0, 6.0], [4.0, 8.0, 7.0]], [[2.0, 7.0, 7.0], [6.0, 8.0, 6.0], [8.0, 5.0, 6.0]], [[5.0, 7.0, 4.0], [8.0, 6.0, 6.0], 
[5.0, 1.0, 8.0]]], [[[6.0, 8.0, 3.0], [4.0, 6.0, 3.0], [1.0, 8.0, 6.0]], [[4.0, 2.0, 7.0], [5.0, 2.0, 8.0], [3.0, 6.0, 4.0]], 
[[5.0, 5.0, 2.0], [1.0, 3.0, 7.0], [4.0, 5.0, 2.0]]], [[[2.0, 5.0, 8.0], [2.0, 4.0, 3.0], [5.0, 8.0, 5.0]], [[1.0, 6.0, 5.0], 
[6.0, 2.0, 3.0], [3.0, 1.0, 7.0]], [[3.0, 2.0, 6.0], [3.0, 6.0, 2.0], [2.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RULN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 3.0], [5.0, 5.0, 5.0], [1.0, 1.0, 6.0]], [[6.0, 1.0, 3.0], [8.0, 3.0, 5.0], [8.0, 
1.0, 3.0]], [[6.0, 3.0, 6.0], [5.0, 5.0, 5.0], [3.0, 2.0, 3.0]]], [[[1.0, 7.0, 7.0], [8.0, 6.0, 4.0], [2.0, 1.0, 5.0]], [[4.0, 
3.0, 1.0], [5.0, 5.0, 3.0], [5.0, 6.0, 5.0]], [[3.0, 6.0, 6.0], [8.0, 2.0, 7.0], [8.0, 6.0, 3.0]]], [[[1.0, 4.0, 3.0], [1.0, 
8.0, 4.0], [1.0, 1.0, 7.0]], [[6.0, 7.0, 5.0], [2.0, 6.0, 5.0], [2.0, 4.0, 6.0]], [[8.0, 5.0, 5.0], [6.0, 7.0, 3.0], [7.0, 4.0, 
7.0]]], [[[7.0, 8.0, 7.0], [7.0, 7.0, 7.0], [5.0, 2.0, 3.0]], [[1.0, 3.0, 7.0], [3.0, 4.0, 7.0], [7.0, 1.0, 4.0]], [[3.0, 7.0, 
1.0], [1.0, 5.0, 1.0], [1.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 2.0], [3.0, 7.0, 1.0], [8.0, 2.0, 4.0]], 
[[3.0, 1.0, 5.0], [2.0, 1.0, 7.0], [5.0, 7.0, 1.0]], [[4.0, 4.0, 8.0], [8.0, 3.0, 5.0], [3.0, 7.0, 6.0]]], [[[8.0, 6.0, 7.0], 
[8.0, 8.0, 3.0], [2.0, 5.0, 8.0]], [[7.0, 5.0, 3.0], [5.0, 4.0, 2.0], [1.0, 6.0, 8.0]], [[7.0, 1.0, 1.0], [1.0, 3.0, 7.0], 
[2.0, 7.0, 8.0]]], [[[4.0, 6.0, 1.0], [3.0, 5.0, 3.0], [1.0, 3.0, 1.0]], [[7.0, 4.0, 3.0], [5.0, 2.0, 7.0], [3.0, 8.0, 1.0]], 
[[4.0, 6.0, 3.0], [2.0, 6.0, 3.0], [1.0, 4.0, 7.0]]], [[[3.0, 8.0, 5.0], [3.0, 3.0, 7.0], [3.0, 6.0, 7.0]], [[2.0, 2.0, 5.0], 
[3.0, 6.0, 3.0], [6.0, 4.0, 4.0]], [[8.0, 2.0, 1.0], [3.0, 1.0, 1.0], [5.0, 4.0, 3.0]]]])
      val=Data(numpy.array([[[[3.0, 1.0, 7.0]], [[2.0, 5.0, 8.0]]], [[[1.0, 3.0, 5.0]], [[3.0, 2.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 5.0, 6.0]], [[8.0, 7.0, 6.0]]], [[[5.0, 7.0, 4.0]], [[3.0, 6.0, 8.0]]]]))
      arg[1:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 3.0], [5.0, 5.0, 5.0], [1.0, 1.0, 6.0]], [[6.0, 1.0, 3.0], [8.0, 3.0, 5.0], [8.0, 
1.0, 3.0]], [[6.0, 3.0, 6.0], [5.0, 5.0, 5.0], [3.0, 2.0, 3.0]]], [[[1.0, 7.0, 7.0], [8.0, 6.0, 4.0], [3.0, 1.0, 7.0]], [[4.0, 
3.0, 1.0], [5.0, 5.0, 3.0], [2.0, 5.0, 8.0]], [[3.0, 6.0, 6.0], [8.0, 2.0, 7.0], [8.0, 6.0, 3.0]]], [[[1.0, 4.0, 3.0], [1.0, 
8.0, 4.0], [1.0, 3.0, 5.0]], [[6.0, 7.0, 5.0], [2.0, 6.0, 5.0], [3.0, 2.0, 4.0]], [[8.0, 5.0, 5.0], [6.0, 7.0, 3.0], [7.0, 4.0, 
7.0]]], [[[7.0, 8.0, 7.0], [7.0, 7.0, 7.0], [5.0, 2.0, 3.0]], [[1.0, 3.0, 7.0], [3.0, 4.0, 7.0], [7.0, 1.0, 4.0]], [[3.0, 7.0, 
1.0], [1.0, 5.0, 1.0], [1.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 2.0], [3.0, 7.0, 1.0], [8.0, 2.0, 4.0]], 
[[3.0, 1.0, 5.0], [2.0, 1.0, 7.0], [5.0, 7.0, 1.0]], [[4.0, 4.0, 8.0], [8.0, 3.0, 5.0], [3.0, 7.0, 6.0]]], [[[8.0, 6.0, 7.0], 
[8.0, 8.0, 3.0], [4.0, 5.0, 6.0]], [[7.0, 5.0, 3.0], [5.0, 4.0, 2.0], [8.0, 7.0, 6.0]], [[7.0, 1.0, 1.0], [1.0, 3.0, 7.0], 
[2.0, 7.0, 8.0]]], [[[4.0, 6.0, 1.0], [3.0, 5.0, 3.0], [5.0, 7.0, 4.0]], [[7.0, 4.0, 3.0], [5.0, 2.0, 7.0], [3.0, 6.0, 8.0]], 
[[4.0, 6.0, 3.0], [2.0, 6.0, 3.0], [1.0, 4.0, 7.0]]], [[[3.0, 8.0, 5.0], [3.0, 3.0, 7.0], [3.0, 6.0, 7.0]], [[2.0, 2.0, 5.0], 
[3.0, 6.0, 3.0], [6.0, 4.0, 4.0]], [[8.0, 2.0, 1.0], [3.0, 1.0, 1.0], [5.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CURC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 2.0], [3.0, 8.0, 6.0], [8.0, 7.0, 7.0]], [[1.0, 8.0, 3.0], [5.0, 3.0, 8.0], [5.0, 
3.0, 6.0]], [[4.0, 4.0, 2.0], [1.0, 7.0, 8.0], [3.0, 3.0, 8.0]]], [[[3.0, 4.0, 4.0], [7.0, 4.0, 2.0], [5.0, 4.0, 1.0]], [[3.0, 
1.0, 6.0], [4.0, 2.0, 2.0], [5.0, 4.0, 2.0]], [[8.0, 5.0, 1.0], [3.0, 4.0, 7.0], [4.0, 4.0, 2.0]]], [[[2.0, 8.0, 5.0], [6.0, 
6.0, 4.0], [1.0, 2.0, 3.0]], [[3.0, 6.0, 1.0], [7.0, 8.0, 7.0], [2.0, 3.0, 7.0]], [[5.0, 3.0, 7.0], [1.0, 1.0, 3.0], [6.0, 3.0, 
2.0]]], [[[4.0, 5.0, 8.0], [3.0, 5.0, 8.0], [3.0, 2.0, 7.0]], [[6.0, 2.0, 3.0], [5.0, 4.0, 2.0], [8.0, 8.0, 7.0]], [[3.0, 6.0, 
2.0], [8.0, 4.0, 4.0], [8.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 3.0], [5.0, 8.0, 4.0], [3.0, 8.0, 7.0]], 
[[5.0, 8.0, 3.0], [7.0, 8.0, 5.0], [5.0, 8.0, 1.0]], [[4.0, 8.0, 8.0], [4.0, 6.0, 5.0], [1.0, 7.0, 2.0]]], [[[1.0, 6.0, 5.0], 
[8.0, 5.0, 2.0], [7.0, 6.0, 5.0]], [[7.0, 5.0, 6.0], [2.0, 8.0, 8.0], [2.0, 3.0, 5.0]], [[5.0, 5.0, 6.0], [5.0, 4.0, 4.0], 
[7.0, 5.0, 6.0]]], [[[8.0, 8.0, 6.0], [8.0, 5.0, 8.0], [1.0, 4.0, 5.0]], [[5.0, 3.0, 1.0], [4.0, 1.0, 8.0], [3.0, 4.0, 8.0]], 
[[1.0, 4.0, 4.0], [8.0, 5.0, 4.0], [5.0, 5.0, 6.0]]], [[[4.0, 3.0, 8.0], [3.0, 4.0, 4.0], [7.0, 3.0, 2.0]], [[7.0, 8.0, 6.0], 
[2.0, 1.0, 5.0], [5.0, 1.0, 7.0]], [[3.0, 1.0, 1.0], [4.0, 5.0, 5.0], [3.0, 5.0, 1.0]]]])
      val=Data(numpy.array([[[[3.0, 6.0, 7.0]], [[6.0, 8.0, 5.0]]], [[[6.0, 7.0, 3.0]], [[3.0, 6.0, 3.0]]], [[[3.0, 4.0, 5.0]], 
[[2.0, 4.0, 5.0]]], [[[3.0, 5.0, 3.0]], [[3.0, 1.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 1.0, 8.0]], [[1.0, 2.0, 6.0]]], [[[5.0, 5.0, 6.0]], [[7.0, 1.0, 7.0]]], [[[7.0, 
7.0, 6.0]], [[6.0, 7.0, 6.0]]], [[[7.0, 7.0, 3.0]], [[8.0, 6.0, 2.0]]]]))
      arg[:,:2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 2.0], [3.0, 6.0, 7.0], [8.0, 7.0, 7.0]], [[1.0, 8.0, 3.0], [6.0, 8.0, 5.0], [5.0, 
3.0, 6.0]], [[4.0, 4.0, 2.0], [1.0, 7.0, 8.0], [3.0, 3.0, 8.0]]], [[[3.0, 4.0, 4.0], [6.0, 7.0, 3.0], [5.0, 4.0, 1.0]], [[3.0, 
1.0, 6.0], [3.0, 6.0, 3.0], [5.0, 4.0, 2.0]], [[8.0, 5.0, 1.0], [3.0, 4.0, 7.0], [4.0, 4.0, 2.0]]], [[[2.0, 8.0, 5.0], [3.0, 
4.0, 5.0], [1.0, 2.0, 3.0]], [[3.0, 6.0, 1.0], [2.0, 4.0, 5.0], [2.0, 3.0, 7.0]], [[5.0, 3.0, 7.0], [1.0, 1.0, 3.0], [6.0, 3.0, 
2.0]]], [[[4.0, 5.0, 8.0], [3.0, 5.0, 3.0], [3.0, 2.0, 7.0]], [[6.0, 2.0, 3.0], [3.0, 1.0, 4.0], [8.0, 8.0, 7.0]], [[3.0, 6.0, 
2.0], [8.0, 4.0, 4.0], [8.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 3.0], [3.0, 1.0, 8.0], [3.0, 8.0, 7.0]], 
[[5.0, 8.0, 3.0], [1.0, 2.0, 6.0], [5.0, 8.0, 1.0]], [[4.0, 8.0, 8.0], [4.0, 6.0, 5.0], [1.0, 7.0, 2.0]]], [[[1.0, 6.0, 5.0], 
[5.0, 5.0, 6.0], [7.0, 6.0, 5.0]], [[7.0, 5.0, 6.0], [7.0, 1.0, 7.0], [2.0, 3.0, 5.0]], [[5.0, 5.0, 6.0], [5.0, 4.0, 4.0], 
[7.0, 5.0, 6.0]]], [[[8.0, 8.0, 6.0], [7.0, 7.0, 6.0], [1.0, 4.0, 5.0]], [[5.0, 3.0, 1.0], [6.0, 7.0, 6.0], [3.0, 4.0, 8.0]], 
[[1.0, 4.0, 4.0], [8.0, 5.0, 4.0], [5.0, 5.0, 6.0]]], [[[4.0, 3.0, 8.0], [7.0, 7.0, 3.0], [7.0, 3.0, 2.0]], [[7.0, 8.0, 6.0], 
[8.0, 6.0, 2.0], [5.0, 1.0, 7.0]], [[3.0, 1.0, 1.0], [4.0, 5.0, 5.0], [3.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLCP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 1.0], [8.0, 5.0, 3.0], [4.0, 8.0, 2.0]], [[1.0, 1.0, 5.0], [7.0, 6.0, 6.0], [3.0, 
4.0, 8.0]], [[3.0, 2.0, 1.0], [6.0, 3.0, 4.0], [5.0, 1.0, 2.0]]], [[[7.0, 4.0, 2.0], [5.0, 1.0, 7.0], [2.0, 6.0, 1.0]], [[1.0, 
7.0, 4.0], [2.0, 2.0, 5.0], [2.0, 3.0, 1.0]], [[5.0, 3.0, 3.0], [1.0, 3.0, 5.0], [3.0, 1.0, 7.0]]], [[[6.0, 5.0, 4.0], [3.0, 
5.0, 4.0], [3.0, 1.0, 8.0]], [[1.0, 4.0, 3.0], [8.0, 1.0, 7.0], [5.0, 3.0, 4.0]], [[5.0, 8.0, 3.0], [6.0, 4.0, 2.0], [6.0, 6.0, 
8.0]]], [[[3.0, 8.0, 6.0], [1.0, 6.0, 1.0], [6.0, 6.0, 7.0]], [[8.0, 1.0, 7.0], [2.0, 4.0, 3.0], [2.0, 5.0, 1.0]], [[5.0, 1.0, 
1.0], [8.0, 8.0, 7.0], [2.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 1.0], [4.0, 7.0, 7.0], [4.0, 3.0, 1.0]], 
[[1.0, 8.0, 5.0], [6.0, 5.0, 4.0], [8.0, 3.0, 4.0]], [[4.0, 3.0, 3.0], [3.0, 4.0, 2.0], [4.0, 7.0, 1.0]]], [[[5.0, 7.0, 7.0], 
[3.0, 6.0, 5.0], [3.0, 5.0, 6.0]], [[2.0, 7.0, 2.0], [6.0, 1.0, 4.0], [2.0, 7.0, 5.0]], [[7.0, 2.0, 2.0], [2.0, 1.0, 4.0], 
[2.0, 1.0, 4.0]]], [[[5.0, 3.0, 7.0], [8.0, 6.0, 4.0], [5.0, 8.0, 1.0]], [[3.0, 2.0, 8.0], [1.0, 5.0, 3.0], [2.0, 1.0, 1.0]], 
[[2.0, 4.0, 3.0], [6.0, 7.0, 3.0], [1.0, 5.0, 1.0]]], [[[3.0, 3.0, 1.0], [7.0, 6.0, 5.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 8.0], 
[3.0, 6.0, 4.0], [7.0, 7.0, 3.0]], [[5.0, 1.0, 7.0], [8.0, 3.0, 8.0], [8.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[4.0, 8.0, 8.0]], [[3.0, 3.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 5.0, 8.0]], [[4.0, 2.0, 4.0]]]))
      arg[2:,2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 1.0], [8.0, 5.0, 3.0], [4.0, 8.0, 2.0]], [[1.0, 1.0, 5.0], [7.0, 6.0, 6.0], [3.0, 
4.0, 8.0]], [[3.0, 2.0, 1.0], [6.0, 3.0, 4.0], [5.0, 1.0, 2.0]]], [[[7.0, 4.0, 2.0], [5.0, 1.0, 7.0], [2.0, 6.0, 1.0]], [[1.0, 
7.0, 4.0], [2.0, 2.0, 5.0], [2.0, 3.0, 1.0]], [[5.0, 3.0, 3.0], [1.0, 3.0, 5.0], [3.0, 1.0, 7.0]]], [[[6.0, 5.0, 4.0], [3.0, 
5.0, 4.0], [3.0, 1.0, 8.0]], [[1.0, 4.0, 3.0], [8.0, 1.0, 7.0], [5.0, 3.0, 4.0]], [[5.0, 4.0, 3.0], [6.0, 8.0, 2.0], [6.0, 8.0, 
8.0]]], [[[3.0, 8.0, 6.0], [1.0, 6.0, 1.0], [6.0, 6.0, 7.0]], [[8.0, 1.0, 7.0], [2.0, 4.0, 3.0], [2.0, 5.0, 1.0]], [[5.0, 3.0, 
1.0], [8.0, 3.0, 7.0], [2.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 1.0], [4.0, 7.0, 7.0], [4.0, 3.0, 1.0]], 
[[1.0, 8.0, 5.0], [6.0, 5.0, 4.0], [8.0, 3.0, 4.0]], [[4.0, 3.0, 3.0], [3.0, 4.0, 2.0], [4.0, 7.0, 1.0]]], [[[5.0, 7.0, 7.0], 
[3.0, 6.0, 5.0], [3.0, 5.0, 6.0]], [[2.0, 7.0, 2.0], [6.0, 1.0, 4.0], [2.0, 7.0, 5.0]], [[7.0, 2.0, 2.0], [2.0, 1.0, 4.0], 
[2.0, 1.0, 4.0]]], [[[5.0, 3.0, 7.0], [8.0, 6.0, 4.0], [5.0, 8.0, 1.0]], [[3.0, 2.0, 8.0], [1.0, 5.0, 3.0], [2.0, 1.0, 1.0]], 
[[2.0, 8.0, 3.0], [6.0, 5.0, 3.0], [1.0, 8.0, 1.0]]], [[[3.0, 3.0, 1.0], [7.0, 6.0, 5.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 8.0], 
[3.0, 6.0, 4.0], [7.0, 7.0, 3.0]], [[5.0, 4.0, 7.0], [8.0, 2.0, 8.0], [8.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CRLR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 6.0], [2.0, 5.0, 3.0], [3.0, 1.0, 2.0]], [[5.0, 3.0, 5.0], [5.0, 2.0, 2.0], [1.0, 
7.0, 6.0]], [[1.0, 7.0, 8.0], [7.0, 5.0, 2.0], [2.0, 8.0, 5.0]]], [[[2.0, 5.0, 6.0], [3.0, 4.0, 2.0], [5.0, 6.0, 3.0]], [[4.0, 
6.0, 7.0], [5.0, 5.0, 8.0], [6.0, 2.0, 6.0]], [[1.0, 3.0, 6.0], [7.0, 2.0, 3.0], [8.0, 3.0, 1.0]]], [[[1.0, 2.0, 1.0], [7.0, 
2.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 3.0, 5.0], [7.0, 1.0, 8.0], [4.0, 8.0, 7.0]], [[7.0, 4.0, 1.0], [1.0, 4.0, 1.0], [5.0, 1.0, 
2.0]]], [[[8.0, 7.0, 8.0], [3.0, 1.0, 2.0], [2.0, 8.0, 8.0]], [[4.0, 7.0, 1.0], [4.0, 8.0, 5.0], [7.0, 6.0, 6.0]], [[7.0, 8.0, 
4.0], [2.0, 4.0, 8.0], [7.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 1.0], [4.0, 3.0, 3.0], [2.0, 6.0, 6.0]], 
[[7.0, 6.0, 6.0], [2.0, 1.0, 1.0], [6.0, 8.0, 2.0]], [[1.0, 3.0, 1.0], [4.0, 2.0, 5.0], [7.0, 1.0, 8.0]]], [[[8.0, 7.0, 2.0], 
[1.0, 2.0, 8.0], [5.0, 1.0, 4.0]], [[5.0, 7.0, 5.0], [3.0, 2.0, 4.0], [8.0, 3.0, 7.0]], [[8.0, 3.0, 7.0], [2.0, 3.0, 5.0], 
[8.0, 6.0, 5.0]]], [[[1.0, 5.0, 3.0], [7.0, 5.0, 5.0], [4.0, 2.0, 6.0]], [[5.0, 3.0, 1.0], [4.0, 3.0, 1.0], [2.0, 8.0, 2.0]], 
[[5.0, 3.0, 6.0], [3.0, 8.0, 6.0], [5.0, 1.0, 3.0]]], [[[3.0, 2.0, 5.0], [2.0, 8.0, 5.0], [2.0, 3.0, 6.0]], [[6.0, 1.0, 3.0], 
[3.0, 7.0, 3.0], [6.0, 7.0, 8.0]], [[3.0, 6.0, 6.0], [2.0, 6.0, 6.0], [6.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[[[3.0]]], [[[8.0]]], [[[4.0]]], [[[3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0]]], [[[1.0]]], [[[4.0]]], [[[3.0]]]]))
      arg[:,1:2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 6.0], [2.0, 5.0, 3.0], [3.0, 1.0, 2.0]], [[5.0, 3.0, 5.0], [5.0, 2.0, 2.0], [1.0, 
3.0, 6.0]], [[1.0, 7.0, 8.0], [7.0, 5.0, 2.0], [2.0, 8.0, 5.0]]], [[[2.0, 5.0, 6.0], [3.0, 4.0, 2.0], [5.0, 6.0, 3.0]], [[4.0, 
6.0, 7.0], [5.0, 5.0, 8.0], [6.0, 8.0, 6.0]], [[1.0, 3.0, 6.0], [7.0, 2.0, 3.0], [8.0, 3.0, 1.0]]], [[[1.0, 2.0, 1.0], [7.0, 
2.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 3.0, 5.0], [7.0, 1.0, 8.0], [4.0, 4.0, 7.0]], [[7.0, 4.0, 1.0], [1.0, 4.0, 1.0], [5.0, 1.0, 
2.0]]], [[[8.0, 7.0, 8.0], [3.0, 1.0, 2.0], [2.0, 8.0, 8.0]], [[4.0, 7.0, 1.0], [4.0, 8.0, 5.0], [7.0, 3.0, 6.0]], [[7.0, 8.0, 
4.0], [2.0, 4.0, 8.0], [7.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 1.0], [4.0, 3.0, 3.0], [2.0, 6.0, 6.0]], 
[[7.0, 6.0, 6.0], [2.0, 1.0, 1.0], [6.0, 5.0, 2.0]], [[1.0, 3.0, 1.0], [4.0, 2.0, 5.0], [7.0, 1.0, 8.0]]], [[[8.0, 7.0, 2.0], 
[1.0, 2.0, 8.0], [5.0, 1.0, 4.0]], [[5.0, 7.0, 5.0], [3.0, 2.0, 4.0], [8.0, 1.0, 7.0]], [[8.0, 3.0, 7.0], [2.0, 3.0, 5.0], 
[8.0, 6.0, 5.0]]], [[[1.0, 5.0, 3.0], [7.0, 5.0, 5.0], [4.0, 2.0, 6.0]], [[5.0, 3.0, 1.0], [4.0, 3.0, 1.0], [2.0, 4.0, 2.0]], 
[[5.0, 3.0, 6.0], [3.0, 8.0, 6.0], [5.0, 1.0, 3.0]]], [[[3.0, 2.0, 5.0], [2.0, 8.0, 5.0], [2.0, 3.0, 6.0]], [[6.0, 1.0, 3.0], 
[3.0, 7.0, 3.0], [6.0, 3.0, 8.0]], [[3.0, 6.0, 6.0], [2.0, 6.0, 6.0], [6.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UURU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 5.0], [7.0, 5.0, 2.0], [3.0, 7.0, 4.0]], [[7.0, 3.0, 2.0], [5.0, 8.0, 1.0], [7.0, 
4.0, 3.0]], [[6.0, 7.0, 6.0], [2.0, 4.0, 2.0], [8.0, 5.0, 4.0]]], [[[5.0, 1.0, 5.0], [6.0, 2.0, 1.0], [3.0, 4.0, 5.0]], [[7.0, 
2.0, 3.0], [3.0, 6.0, 8.0], [5.0, 1.0, 6.0]], [[8.0, 4.0, 8.0], [1.0, 7.0, 6.0], [7.0, 3.0, 6.0]]], [[[2.0, 4.0, 7.0], [7.0, 
2.0, 1.0], [4.0, 7.0, 5.0]], [[6.0, 4.0, 7.0], [2.0, 8.0, 1.0], [5.0, 7.0, 6.0]], [[5.0, 4.0, 2.0], [5.0, 7.0, 5.0], [1.0, 7.0, 
2.0]]], [[[6.0, 5.0, 4.0], [7.0, 3.0, 6.0], [6.0, 5.0, 2.0]], [[7.0, 8.0, 1.0], [3.0, 2.0, 8.0], [4.0, 5.0, 7.0]], [[2.0, 2.0, 
8.0], [8.0, 6.0, 8.0], [3.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 2.0], [7.0, 8.0, 8.0], [1.0, 2.0, 2.0]], 
[[4.0, 2.0, 6.0], [5.0, 7.0, 4.0], [6.0, 1.0, 6.0]], [[7.0, 6.0, 2.0], [5.0, 6.0, 2.0], [6.0, 8.0, 5.0]]], [[[2.0, 7.0, 8.0], 
[3.0, 4.0, 6.0], [6.0, 3.0, 8.0]], [[2.0, 3.0, 5.0], [7.0, 2.0, 1.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 1.0], [2.0, 4.0, 6.0], 
[2.0, 8.0, 4.0]]], [[[4.0, 7.0, 8.0], [5.0, 2.0, 3.0], [6.0, 7.0, 6.0]], [[4.0, 4.0, 6.0], [7.0, 1.0, 8.0], [4.0, 6.0, 5.0]], 
[[7.0, 2.0, 4.0], [8.0, 7.0, 5.0], [1.0, 2.0, 5.0]]], [[[4.0, 5.0, 1.0], [5.0, 3.0, 3.0], [6.0, 6.0, 6.0]], [[4.0, 6.0, 4.0], 
[8.0, 6.0, 8.0], [3.0, 8.0, 6.0]], [[3.0, 4.0, 7.0], [4.0, 8.0, 6.0], [3.0, 7.0, 5.0]]]])
      val=Data(numpy.array([[[[3.0, 8.0]], [[1.0, 5.0]]], [[[8.0, 2.0]], [[5.0, 8.0]]], [[[8.0, 5.0]], [[2.0, 
4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 8.0]], [[8.0, 6.0]]], [[[8.0, 4.0]], [[7.0, 1.0]]], [[[5.0, 7.0]], [[3.0, 
3.0]]]]))
      arg[:3,:2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 5.0], [3.0, 8.0, 2.0], [3.0, 7.0, 4.0]], [[7.0, 3.0, 2.0], [1.0, 5.0, 1.0], [7.0, 
4.0, 3.0]], [[6.0, 7.0, 6.0], [2.0, 4.0, 2.0], [8.0, 5.0, 4.0]]], [[[5.0, 1.0, 5.0], [8.0, 2.0, 1.0], [3.0, 4.0, 5.0]], [[7.0, 
2.0, 3.0], [5.0, 8.0, 8.0], [5.0, 1.0, 6.0]], [[8.0, 4.0, 8.0], [1.0, 7.0, 6.0], [7.0, 3.0, 6.0]]], [[[2.0, 4.0, 7.0], [8.0, 
5.0, 1.0], [4.0, 7.0, 5.0]], [[6.0, 4.0, 7.0], [2.0, 4.0, 1.0], [5.0, 7.0, 6.0]], [[5.0, 4.0, 2.0], [5.0, 7.0, 5.0], [1.0, 7.0, 
2.0]]], [[[6.0, 5.0, 4.0], [7.0, 3.0, 6.0], [6.0, 5.0, 2.0]], [[7.0, 8.0, 1.0], [3.0, 2.0, 8.0], [4.0, 5.0, 7.0]], [[2.0, 2.0, 
8.0], [8.0, 6.0, 8.0], [3.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 2.0], [7.0, 8.0, 8.0], [1.0, 2.0, 2.0]], 
[[4.0, 2.0, 6.0], [8.0, 6.0, 4.0], [6.0, 1.0, 6.0]], [[7.0, 6.0, 2.0], [5.0, 6.0, 2.0], [6.0, 8.0, 5.0]]], [[[2.0, 7.0, 8.0], 
[8.0, 4.0, 6.0], [6.0, 3.0, 8.0]], [[2.0, 3.0, 5.0], [7.0, 1.0, 1.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 1.0], [2.0, 4.0, 6.0], 
[2.0, 8.0, 4.0]]], [[[4.0, 7.0, 8.0], [5.0, 7.0, 3.0], [6.0, 7.0, 6.0]], [[4.0, 4.0, 6.0], [3.0, 3.0, 8.0], [4.0, 6.0, 5.0]], 
[[7.0, 2.0, 4.0], [8.0, 7.0, 5.0], [1.0, 2.0, 5.0]]], [[[4.0, 5.0, 1.0], [5.0, 3.0, 3.0], [6.0, 6.0, 6.0]], [[4.0, 6.0, 4.0], 
[8.0, 6.0, 8.0], [3.0, 8.0, 6.0]], [[3.0, 4.0, 7.0], [4.0, 8.0, 6.0], [3.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UCRP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 5.0], [2.0, 1.0, 5.0], [7.0, 1.0, 2.0]], [[1.0, 6.0, 4.0], [8.0, 4.0, 8.0], [3.0, 
7.0, 6.0]], [[8.0, 3.0, 6.0], [2.0, 7.0, 7.0], [8.0, 4.0, 5.0]]], [[[3.0, 1.0, 8.0], [8.0, 4.0, 7.0], [5.0, 4.0, 3.0]], [[7.0, 
8.0, 4.0], [5.0, 5.0, 8.0], [1.0, 7.0, 6.0]], [[8.0, 7.0, 1.0], [3.0, 1.0, 7.0], [1.0, 1.0, 3.0]]], [[[2.0, 7.0, 2.0], [7.0, 
1.0, 8.0], [6.0, 1.0, 3.0]], [[4.0, 6.0, 5.0], [5.0, 7.0, 5.0], [2.0, 8.0, 5.0]], [[6.0, 8.0, 5.0], [8.0, 1.0, 3.0], [4.0, 6.0, 
3.0]]], [[[4.0, 8.0, 1.0], [3.0, 7.0, 1.0], [7.0, 8.0, 1.0]], [[8.0, 7.0, 4.0], [5.0, 8.0, 7.0], [2.0, 7.0, 5.0]], [[6.0, 1.0, 
5.0], [1.0, 2.0, 4.0], [6.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 7.0], [7.0, 2.0, 7.0], [5.0, 1.0, 2.0]], 
[[6.0, 2.0, 1.0], [7.0, 2.0, 4.0], [1.0, 1.0, 6.0]], [[2.0, 8.0, 4.0], [2.0, 2.0, 5.0], [5.0, 2.0, 1.0]]], [[[3.0, 5.0, 5.0], 
[3.0, 5.0, 4.0], [5.0, 6.0, 2.0]], [[8.0, 5.0, 8.0], [7.0, 3.0, 7.0], [2.0, 1.0, 4.0]], [[2.0, 2.0, 8.0], [1.0, 7.0, 5.0], 
[3.0, 5.0, 4.0]]], [[[3.0, 5.0, 6.0], [2.0, 2.0, 8.0], [5.0, 6.0, 1.0]], [[4.0, 5.0, 7.0], [6.0, 5.0, 2.0], [2.0, 7.0, 2.0]], 
[[3.0, 2.0, 7.0], [2.0, 1.0, 8.0], [7.0, 3.0, 6.0]]], [[[2.0, 4.0, 6.0], [5.0, 3.0, 5.0], [2.0, 1.0, 8.0]], [[1.0, 1.0, 5.0], 
[8.0, 5.0, 3.0], [1.0, 3.0, 6.0]], [[8.0, 3.0, 4.0], [5.0, 7.0, 3.0], [8.0, 2.0, 6.0]]]])
      val=Data(numpy.array([[[3.0], [7.0], [5.0]], [[4.0], [5.0], [8.0]], [[1.0], [6.0], [2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0], [2.0], [3.0]], [[5.0], [8.0], [7.0]], [[1.0], [2.0], [4.0]]]))
      arg[:3,:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 5.0], [2.0, 3.0, 5.0], [7.0, 1.0, 2.0]], [[1.0, 6.0, 4.0], [8.0, 7.0, 8.0], [3.0, 
7.0, 6.0]], [[8.0, 3.0, 6.0], [2.0, 5.0, 7.0], [8.0, 4.0, 5.0]]], [[[3.0, 1.0, 8.0], [8.0, 4.0, 7.0], [5.0, 4.0, 3.0]], [[7.0, 
8.0, 4.0], [5.0, 5.0, 8.0], [1.0, 7.0, 6.0]], [[8.0, 7.0, 1.0], [3.0, 8.0, 7.0], [1.0, 1.0, 3.0]]], [[[2.0, 7.0, 2.0], [7.0, 
1.0, 8.0], [6.0, 1.0, 3.0]], [[4.0, 6.0, 5.0], [5.0, 6.0, 5.0], [2.0, 8.0, 5.0]], [[6.0, 8.0, 5.0], [8.0, 2.0, 3.0], [4.0, 6.0, 
3.0]]], [[[4.0, 8.0, 1.0], [3.0, 7.0, 1.0], [7.0, 8.0, 1.0]], [[8.0, 7.0, 4.0], [5.0, 8.0, 7.0], [2.0, 7.0, 5.0]], [[6.0, 1.0, 
5.0], [1.0, 2.0, 4.0], [6.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 7.0], [7.0, 5.0, 7.0], [5.0, 1.0, 2.0]], 
[[6.0, 2.0, 1.0], [7.0, 2.0, 4.0], [1.0, 1.0, 6.0]], [[2.0, 8.0, 4.0], [2.0, 3.0, 5.0], [5.0, 2.0, 1.0]]], [[[3.0, 5.0, 5.0], 
[3.0, 5.0, 4.0], [5.0, 6.0, 2.0]], [[8.0, 5.0, 8.0], [7.0, 8.0, 7.0], [2.0, 1.0, 4.0]], [[2.0, 2.0, 8.0], [1.0, 7.0, 5.0], 
[3.0, 5.0, 4.0]]], [[[3.0, 5.0, 6.0], [2.0, 1.0, 8.0], [5.0, 6.0, 1.0]], [[4.0, 5.0, 7.0], [6.0, 2.0, 2.0], [2.0, 7.0, 2.0]], 
[[3.0, 2.0, 7.0], [2.0, 4.0, 8.0], [7.0, 3.0, 6.0]]], [[[2.0, 4.0, 6.0], [5.0, 3.0, 5.0], [2.0, 1.0, 8.0]], [[1.0, 1.0, 5.0], 
[8.0, 5.0, 3.0], [1.0, 3.0, 6.0]], [[8.0, 3.0, 4.0], [5.0, 7.0, 3.0], [8.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URPU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 6.0], [8.0, 2.0, 1.0], [7.0, 7.0, 1.0]], [[6.0, 5.0, 5.0], [8.0, 1.0, 7.0], [2.0, 
8.0, 8.0]], [[4.0, 5.0, 5.0], [4.0, 4.0, 3.0], [6.0, 7.0, 8.0]]], [[[8.0, 2.0, 3.0], [1.0, 4.0, 4.0], [6.0, 3.0, 2.0]], [[1.0, 
2.0, 2.0], [4.0, 7.0, 6.0], [1.0, 1.0, 6.0]], [[2.0, 7.0, 1.0], [8.0, 5.0, 2.0], [7.0, 4.0, 1.0]]], [[[8.0, 1.0, 1.0], [3.0, 
6.0, 4.0], [5.0, 4.0, 4.0]], [[5.0, 8.0, 1.0], [2.0, 6.0, 6.0], [2.0, 4.0, 6.0]], [[4.0, 5.0, 3.0], [2.0, 7.0, 8.0], [6.0, 3.0, 
7.0]]], [[[1.0, 6.0, 4.0], [7.0, 7.0, 5.0], [8.0, 2.0, 6.0]], [[7.0, 4.0, 3.0], [3.0, 2.0, 8.0], [4.0, 2.0, 3.0]], [[2.0, 7.0, 
8.0], [6.0, 6.0, 8.0], [6.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 2.0], [3.0, 1.0, 7.0], [7.0, 2.0, 2.0]], 
[[6.0, 5.0, 7.0], [6.0, 4.0, 1.0], [7.0, 8.0, 5.0]], [[2.0, 8.0, 6.0], [3.0, 6.0, 8.0], [7.0, 2.0, 3.0]]], [[[5.0, 3.0, 7.0], 
[2.0, 7.0, 7.0], [6.0, 7.0, 5.0]], [[1.0, 5.0, 2.0], [4.0, 8.0, 6.0], [8.0, 7.0, 2.0]], [[3.0, 2.0, 6.0], [6.0, 5.0, 4.0], 
[4.0, 2.0, 5.0]]], [[[6.0, 6.0, 5.0], [6.0, 5.0, 8.0], [1.0, 1.0, 7.0]], [[6.0, 7.0, 6.0], [7.0, 5.0, 1.0], [5.0, 5.0, 6.0]], 
[[3.0, 7.0, 3.0], [5.0, 2.0, 3.0], [8.0, 6.0, 3.0]]], [[[5.0, 3.0, 8.0], [5.0, 6.0, 8.0], [2.0, 7.0, 2.0]], [[8.0, 3.0, 3.0], 
[7.0, 8.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 3.0, 6.0], [8.0, 4.0, 6.0], [8.0, 1.0, 8.0]]]])
      val=Data(numpy.array([[[5.0, 4.0]], [[7.0, 3.0]], [[2.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 5.0]], [[6.0, 7.0]], [[7.0, 1.0]]]))
      arg[:3,1:2,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 6.0], [8.0, 2.0, 1.0], [7.0, 7.0, 1.0]], [[6.0, 5.0, 5.0], [5.0, 4.0, 7.0], [2.0, 
8.0, 8.0]], [[4.0, 5.0, 5.0], [4.0, 4.0, 3.0], [6.0, 7.0, 8.0]]], [[[8.0, 2.0, 3.0], [1.0, 4.0, 4.0], [6.0, 3.0, 2.0]], [[1.0, 
2.0, 2.0], [7.0, 3.0, 6.0], [1.0, 1.0, 6.0]], [[2.0, 7.0, 1.0], [8.0, 5.0, 2.0], [7.0, 4.0, 1.0]]], [[[8.0, 1.0, 1.0], [3.0, 
6.0, 4.0], [5.0, 4.0, 4.0]], [[5.0, 8.0, 1.0], [2.0, 7.0, 6.0], [2.0, 4.0, 6.0]], [[4.0, 5.0, 3.0], [2.0, 7.0, 8.0], [6.0, 3.0, 
7.0]]], [[[1.0, 6.0, 4.0], [7.0, 7.0, 5.0], [8.0, 2.0, 6.0]], [[7.0, 4.0, 3.0], [3.0, 2.0, 8.0], [4.0, 2.0, 3.0]], [[2.0, 7.0, 
8.0], [6.0, 6.0, 8.0], [6.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 2.0], [3.0, 1.0, 7.0], [7.0, 2.0, 2.0]], 
[[6.0, 5.0, 7.0], [6.0, 5.0, 1.0], [7.0, 8.0, 5.0]], [[2.0, 8.0, 6.0], [3.0, 6.0, 8.0], [7.0, 2.0, 3.0]]], [[[5.0, 3.0, 7.0], 
[2.0, 7.0, 7.0], [6.0, 7.0, 5.0]], [[1.0, 5.0, 2.0], [6.0, 7.0, 6.0], [8.0, 7.0, 2.0]], [[3.0, 2.0, 6.0], [6.0, 5.0, 4.0], 
[4.0, 2.0, 5.0]]], [[[6.0, 6.0, 5.0], [6.0, 5.0, 8.0], [1.0, 1.0, 7.0]], [[6.0, 7.0, 6.0], [7.0, 1.0, 1.0], [5.0, 5.0, 6.0]], 
[[3.0, 7.0, 3.0], [5.0, 2.0, 3.0], [8.0, 6.0, 3.0]]], [[[5.0, 3.0, 8.0], [5.0, 6.0, 8.0], [2.0, 7.0, 2.0]], [[8.0, 3.0, 3.0], 
[7.0, 8.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 3.0, 6.0], [8.0, 4.0, 6.0], [8.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UURR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 2.0], [1.0, 1.0, 6.0], [4.0, 2.0, 4.0]], [[4.0, 3.0, 7.0], [7.0, 4.0, 3.0], [4.0, 
2.0, 4.0]], [[8.0, 1.0, 4.0], [1.0, 2.0, 3.0], [8.0, 7.0, 2.0]]], [[[1.0, 7.0, 7.0], [6.0, 8.0, 2.0], [1.0, 2.0, 3.0]], [[8.0, 
3.0, 7.0], [1.0, 7.0, 8.0], [6.0, 4.0, 6.0]], [[7.0, 3.0, 8.0], [2.0, 6.0, 1.0], [5.0, 2.0, 7.0]]], [[[5.0, 3.0, 1.0], [8.0, 
5.0, 3.0], [4.0, 2.0, 3.0]], [[1.0, 3.0, 8.0], [3.0, 3.0, 6.0], [6.0, 2.0, 2.0]], [[6.0, 3.0, 1.0], [3.0, 8.0, 3.0], [6.0, 8.0, 
3.0]]], [[[2.0, 5.0, 7.0], [5.0, 8.0, 1.0], [4.0, 7.0, 3.0]], [[8.0, 3.0, 1.0], [5.0, 7.0, 5.0], [4.0, 1.0, 4.0]], [[5.0, 1.0, 
1.0], [6.0, 8.0, 7.0], [2.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 4.0], [7.0, 1.0, 6.0], [2.0, 6.0, 2.0]], 
[[7.0, 8.0, 5.0], [5.0, 8.0, 8.0], [6.0, 2.0, 4.0]], [[6.0, 2.0, 4.0], [2.0, 1.0, 6.0], [6.0, 3.0, 8.0]]], [[[6.0, 6.0, 2.0], 
[1.0, 3.0, 2.0], [8.0, 4.0, 1.0]], [[8.0, 2.0, 2.0], [4.0, 3.0, 8.0], [4.0, 8.0, 3.0]], [[4.0, 7.0, 3.0], [6.0, 4.0, 7.0], 
[2.0, 1.0, 4.0]]], [[[5.0, 3.0, 4.0], [6.0, 4.0, 5.0], [1.0, 1.0, 6.0]], [[5.0, 7.0, 6.0], [8.0, 7.0, 2.0], [8.0, 3.0, 5.0]], 
[[5.0, 1.0, 3.0], [8.0, 5.0, 3.0], [2.0, 1.0, 6.0]]], [[[4.0, 8.0, 3.0], [8.0, 7.0, 5.0], [4.0, 6.0, 5.0]], [[3.0, 1.0, 5.0], 
[2.0, 2.0, 6.0], [5.0, 6.0, 6.0]], [[3.0, 7.0, 8.0], [4.0, 3.0, 5.0], [8.0, 2.0, 6.0]]]])
      val=Data(numpy.array([[[[2.0]], [[1.0]]], [[[4.0]], [[2.0]]], [[[4.0]], [[7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0]], [[6.0]]], [[[5.0]], [[1.0]]], [[[4.0]], [[8.0]]]]))
      arg[:3,:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 2.0], [1.0, 2.0, 6.0], [4.0, 2.0, 4.0]], [[4.0, 3.0, 7.0], [7.0, 1.0, 3.0], [4.0, 
2.0, 4.0]], [[8.0, 1.0, 4.0], [1.0, 2.0, 3.0], [8.0, 7.0, 2.0]]], [[[1.0, 7.0, 7.0], [6.0, 4.0, 2.0], [1.0, 2.0, 3.0]], [[8.0, 
3.0, 7.0], [1.0, 2.0, 8.0], [6.0, 4.0, 6.0]], [[7.0, 3.0, 8.0], [2.0, 6.0, 1.0], [5.0, 2.0, 7.0]]], [[[5.0, 3.0, 1.0], [8.0, 
4.0, 3.0], [4.0, 2.0, 3.0]], [[1.0, 3.0, 8.0], [3.0, 7.0, 6.0], [6.0, 2.0, 2.0]], [[6.0, 3.0, 1.0], [3.0, 8.0, 3.0], [6.0, 8.0, 
3.0]]], [[[2.0, 5.0, 7.0], [5.0, 8.0, 1.0], [4.0, 7.0, 3.0]], [[8.0, 3.0, 1.0], [5.0, 7.0, 5.0], [4.0, 1.0, 4.0]], [[5.0, 1.0, 
1.0], [6.0, 8.0, 7.0], [2.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 4.0], [7.0, 7.0, 6.0], [2.0, 6.0, 2.0]], 
[[7.0, 8.0, 5.0], [5.0, 6.0, 8.0], [6.0, 2.0, 4.0]], [[6.0, 2.0, 4.0], [2.0, 1.0, 6.0], [6.0, 3.0, 8.0]]], [[[6.0, 6.0, 2.0], 
[1.0, 5.0, 2.0], [8.0, 4.0, 1.0]], [[8.0, 2.0, 2.0], [4.0, 1.0, 8.0], [4.0, 8.0, 3.0]], [[4.0, 7.0, 3.0], [6.0, 4.0, 7.0], 
[2.0, 1.0, 4.0]]], [[[5.0, 3.0, 4.0], [6.0, 4.0, 5.0], [1.0, 1.0, 6.0]], [[5.0, 7.0, 6.0], [8.0, 8.0, 2.0], [8.0, 3.0, 5.0]], 
[[5.0, 1.0, 3.0], [8.0, 5.0, 3.0], [2.0, 1.0, 6.0]]], [[[4.0, 8.0, 3.0], [8.0, 7.0, 5.0], [4.0, 6.0, 5.0]], [[3.0, 1.0, 5.0], 
[2.0, 2.0, 6.0], [5.0, 6.0, 6.0]], [[3.0, 7.0, 8.0], [4.0, 3.0, 5.0], [8.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PUPP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 3.0], [8.0, 7.0, 7.0], [5.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], [1.0, 6.0, 7.0], [8.0, 
7.0, 2.0]], [[5.0, 2.0, 1.0], [3.0, 1.0, 7.0], [1.0, 2.0, 6.0]]], [[[3.0, 3.0, 8.0], [2.0, 4.0, 3.0], [4.0, 3.0, 7.0]], [[8.0, 
7.0, 7.0], [3.0, 1.0, 3.0], [6.0, 1.0, 6.0]], [[4.0, 5.0, 6.0], [1.0, 1.0, 4.0], [3.0, 5.0, 1.0]]], [[[6.0, 7.0, 7.0], [2.0, 
6.0, 3.0], [2.0, 3.0, 8.0]], [[3.0, 7.0, 5.0], [6.0, 5.0, 3.0], [3.0, 8.0, 7.0]], [[2.0, 5.0, 6.0], [1.0, 8.0, 1.0], [8.0, 1.0, 
5.0]]], [[[3.0, 7.0, 8.0], [3.0, 2.0, 5.0], [5.0, 1.0, 1.0]], [[5.0, 7.0, 6.0], [8.0, 7.0, 4.0], [6.0, 5.0, 2.0]], [[6.0, 5.0, 
4.0], [1.0, 4.0, 8.0], [5.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 7.0, 2.0], [7.0, 4.0, 8.0], [3.0, 7.0, 7.0]], 
[[5.0, 7.0, 6.0], [5.0, 5.0, 7.0], [5.0, 8.0, 7.0]], [[2.0, 5.0, 7.0], [6.0, 7.0, 8.0], [6.0, 6.0, 7.0]]], [[[6.0, 7.0, 7.0], 
[3.0, 8.0, 7.0], [2.0, 7.0, 3.0]], [[6.0, 4.0, 1.0], [8.0, 5.0, 4.0], [1.0, 4.0, 4.0]], [[2.0, 7.0, 4.0], [8.0, 5.0, 3.0], 
[5.0, 3.0, 1.0]]], [[[8.0, 6.0, 7.0], [2.0, 6.0, 3.0], [6.0, 5.0, 1.0]], [[4.0, 2.0, 7.0], [4.0, 1.0, 7.0], [5.0, 1.0, 6.0]], 
[[7.0, 1.0, 3.0], [1.0, 1.0, 7.0], [4.0, 1.0, 7.0]]], [[[3.0, 3.0, 4.0], [3.0, 4.0, 3.0], [1.0, 7.0, 6.0]], [[5.0, 4.0, 6.0], 
[5.0, 5.0, 1.0], [5.0, 3.0, 1.0]], [[6.0, 4.0, 8.0], [7.0, 4.0, 5.0], [1.0, 2.0, 2.0]]]])
      val=Data(numpy.array([1.0, 5.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([5.0, 7.0]))
      arg[2,:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 3.0], [8.0, 7.0, 7.0], [5.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], [1.0, 6.0, 7.0], [8.0, 
7.0, 2.0]], [[5.0, 2.0, 1.0], [3.0, 1.0, 7.0], [1.0, 2.0, 6.0]]], [[[3.0, 3.0, 8.0], [2.0, 4.0, 3.0], [4.0, 3.0, 7.0]], [[8.0, 
7.0, 7.0], [3.0, 1.0, 3.0], [6.0, 1.0, 6.0]], [[4.0, 5.0, 6.0], [1.0, 1.0, 4.0], [3.0, 5.0, 1.0]]], [[[6.0, 7.0, 7.0], [2.0, 
1.0, 3.0], [2.0, 3.0, 8.0]], [[3.0, 7.0, 5.0], [6.0, 5.0, 3.0], [3.0, 8.0, 7.0]], [[2.0, 5.0, 6.0], [1.0, 8.0, 1.0], [8.0, 1.0, 
5.0]]], [[[3.0, 7.0, 8.0], [3.0, 2.0, 5.0], [5.0, 1.0, 1.0]], [[5.0, 7.0, 6.0], [8.0, 7.0, 4.0], [6.0, 5.0, 2.0]], [[6.0, 5.0, 
4.0], [1.0, 4.0, 8.0], [5.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 2.0], [7.0, 4.0, 8.0], [3.0, 7.0, 7.0]], 
[[5.0, 7.0, 6.0], [5.0, 5.0, 7.0], [5.0, 8.0, 7.0]], [[2.0, 5.0, 7.0], [6.0, 7.0, 8.0], [6.0, 6.0, 7.0]]], [[[6.0, 7.0, 7.0], 
[3.0, 8.0, 7.0], [2.0, 7.0, 3.0]], [[6.0, 4.0, 1.0], [8.0, 5.0, 4.0], [1.0, 4.0, 4.0]], [[2.0, 7.0, 4.0], [8.0, 5.0, 3.0], 
[5.0, 3.0, 1.0]]], [[[8.0, 6.0, 7.0], [2.0, 5.0, 3.0], [6.0, 5.0, 1.0]], [[4.0, 2.0, 7.0], [4.0, 7.0, 7.0], [5.0, 1.0, 6.0]], 
[[7.0, 1.0, 3.0], [1.0, 1.0, 7.0], [4.0, 1.0, 7.0]]], [[[3.0, 3.0, 4.0], [3.0, 4.0, 3.0], [1.0, 7.0, 6.0]], [[5.0, 4.0, 6.0], 
[5.0, 5.0, 1.0], [5.0, 3.0, 1.0]], [[6.0, 4.0, 8.0], [7.0, 4.0, 5.0], [1.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PLRP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 1.0], [8.0, 8.0, 5.0], [4.0, 3.0, 6.0]], [[3.0, 1.0, 8.0], [5.0, 1.0, 3.0], [1.0, 
6.0, 8.0]], [[1.0, 3.0, 2.0], [5.0, 8.0, 5.0], [3.0, 4.0, 6.0]]], [[[2.0, 5.0, 4.0], [8.0, 6.0, 7.0], [3.0, 2.0, 1.0]], [[5.0, 
2.0, 5.0], [3.0, 5.0, 6.0], [6.0, 5.0, 4.0]], [[6.0, 7.0, 6.0], [3.0, 7.0, 3.0], [5.0, 4.0, 1.0]]], [[[1.0, 3.0, 4.0], [3.0, 
6.0, 1.0], [4.0, 2.0, 6.0]], [[1.0, 4.0, 4.0], [8.0, 5.0, 1.0], [8.0, 1.0, 3.0]], [[3.0, 2.0, 8.0], [1.0, 5.0, 4.0], [3.0, 8.0, 
5.0]]], [[[7.0, 3.0, 1.0], [8.0, 4.0, 1.0], [3.0, 4.0, 2.0]], [[5.0, 2.0, 8.0], [7.0, 1.0, 2.0], [1.0, 2.0, 1.0]], [[1.0, 3.0, 
7.0], [5.0, 8.0, 6.0], [1.0, 6.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 3.0, 1.0], [3.0, 6.0, 1.0], [6.0, 7.0, 1.0]], 
[[8.0, 1.0, 1.0], [1.0, 5.0, 4.0], [8.0, 3.0, 4.0]], [[2.0, 4.0, 8.0], [5.0, 2.0, 3.0], [4.0, 1.0, 4.0]]], [[[4.0, 3.0, 7.0], 
[2.0, 3.0, 6.0], [2.0, 3.0, 5.0]], [[8.0, 1.0, 5.0], [1.0, 5.0, 2.0], [1.0, 2.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 5.0, 6.0], 
[7.0, 7.0, 2.0]]], [[[1.0, 7.0, 1.0], [6.0, 5.0, 4.0], [6.0, 2.0, 4.0]], [[6.0, 5.0, 5.0], [5.0, 7.0, 4.0], [8.0, 4.0, 4.0]], 
[[4.0, 6.0, 6.0], [4.0, 8.0, 3.0], [6.0, 4.0, 4.0]]], [[[4.0, 3.0, 5.0], [8.0, 5.0, 2.0], [7.0, 6.0, 5.0]], [[3.0, 7.0, 6.0], 
[8.0, 2.0, 3.0], [1.0, 4.0, 8.0]], [[8.0, 6.0, 4.0], [2.0, 6.0, 6.0], [8.0, 7.0, 3.0]]]])
      val=Data(numpy.array([[5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0]]))
      arg[2,2:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 1.0], [8.0, 8.0, 5.0], [4.0, 3.0, 6.0]], [[3.0, 1.0, 8.0], [5.0, 1.0, 3.0], [1.0, 
6.0, 8.0]], [[1.0, 3.0, 2.0], [5.0, 8.0, 5.0], [3.0, 4.0, 6.0]]], [[[2.0, 5.0, 4.0], [8.0, 6.0, 7.0], [3.0, 2.0, 1.0]], [[5.0, 
2.0, 5.0], [3.0, 5.0, 6.0], [6.0, 5.0, 4.0]], [[6.0, 7.0, 6.0], [3.0, 7.0, 3.0], [5.0, 4.0, 1.0]]], [[[1.0, 3.0, 4.0], [3.0, 
6.0, 1.0], [4.0, 2.0, 6.0]], [[1.0, 4.0, 4.0], [8.0, 5.0, 1.0], [8.0, 1.0, 3.0]], [[3.0, 2.0, 8.0], [1.0, 5.0, 4.0], [3.0, 8.0, 
5.0]]], [[[7.0, 3.0, 1.0], [8.0, 4.0, 1.0], [3.0, 4.0, 2.0]], [[5.0, 2.0, 8.0], [7.0, 1.0, 2.0], [1.0, 2.0, 1.0]], [[1.0, 3.0, 
7.0], [5.0, 8.0, 6.0], [1.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 1.0], [3.0, 6.0, 1.0], [6.0, 7.0, 1.0]], 
[[8.0, 1.0, 1.0], [1.0, 5.0, 4.0], [8.0, 3.0, 4.0]], [[2.0, 4.0, 8.0], [5.0, 2.0, 3.0], [4.0, 1.0, 4.0]]], [[[4.0, 3.0, 7.0], 
[2.0, 3.0, 6.0], [2.0, 3.0, 5.0]], [[8.0, 1.0, 5.0], [1.0, 5.0, 2.0], [1.0, 2.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 5.0, 6.0], 
[7.0, 7.0, 2.0]]], [[[1.0, 7.0, 1.0], [6.0, 5.0, 4.0], [6.0, 2.0, 4.0]], [[6.0, 5.0, 5.0], [5.0, 7.0, 4.0], [8.0, 4.0, 4.0]], 
[[4.0, 6.0, 6.0], [4.0, 7.0, 3.0], [6.0, 4.0, 4.0]]], [[[4.0, 3.0, 5.0], [8.0, 5.0, 2.0], [7.0, 6.0, 5.0]], [[3.0, 7.0, 6.0], 
[8.0, 2.0, 3.0], [1.0, 4.0, 8.0]], [[8.0, 6.0, 4.0], [2.0, 6.0, 6.0], [8.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCCP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 8.0], [1.0, 1.0, 5.0], [5.0, 4.0, 1.0]], [[1.0, 8.0, 5.0], [7.0, 5.0, 6.0], [2.0, 
8.0, 1.0]], [[6.0, 1.0, 4.0], [2.0, 6.0, 5.0], [6.0, 5.0, 6.0]]], [[[3.0, 4.0, 3.0], [7.0, 8.0, 4.0], [6.0, 2.0, 6.0]], [[8.0, 
4.0, 4.0], [3.0, 1.0, 6.0], [8.0, 7.0, 5.0]], [[3.0, 4.0, 1.0], [1.0, 8.0, 1.0], [6.0, 5.0, 7.0]]], [[[6.0, 7.0, 5.0], [6.0, 
4.0, 1.0], [7.0, 3.0, 4.0]], [[8.0, 7.0, 3.0], [2.0, 6.0, 3.0], [8.0, 3.0, 3.0]], [[6.0, 2.0, 5.0], [4.0, 4.0, 8.0], [2.0, 4.0, 
4.0]]], [[[2.0, 7.0, 2.0], [6.0, 6.0, 3.0], [7.0, 5.0, 4.0]], [[1.0, 8.0, 6.0], [6.0, 4.0, 3.0], [7.0, 1.0, 2.0]], [[4.0, 6.0, 
2.0], [6.0, 5.0, 6.0], [6.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 8.0], [5.0, 2.0, 3.0], [5.0, 7.0, 2.0]], 
[[1.0, 1.0, 2.0], [5.0, 3.0, 1.0], [2.0, 5.0, 4.0]], [[6.0, 5.0, 8.0], [3.0, 7.0, 2.0], [3.0, 8.0, 7.0]]], [[[5.0, 5.0, 4.0], 
[5.0, 6.0, 7.0], [1.0, 1.0, 7.0]], [[8.0, 8.0, 5.0], [5.0, 6.0, 7.0], [8.0, 1.0, 6.0]], [[2.0, 2.0, 8.0], [3.0, 3.0, 3.0], 
[8.0, 5.0, 3.0]]], [[[6.0, 2.0, 5.0], [7.0, 2.0, 7.0], [7.0, 4.0, 2.0]], [[2.0, 7.0, 4.0], [6.0, 8.0, 1.0], [8.0, 4.0, 6.0]], 
[[3.0, 4.0, 7.0], [4.0, 2.0, 8.0], [1.0, 7.0, 8.0]]], [[[3.0, 6.0, 8.0], [6.0, 3.0, 4.0], [2.0, 6.0, 8.0]], [[2.0, 1.0, 3.0], 
[8.0, 1.0, 1.0], [2.0, 3.0, 5.0]], [[8.0, 8.0, 1.0], [4.0, 1.0, 4.0], [5.0, 1.0, 2.0]]]])
      val=Data(numpy.array([[[1.0, 2.0, 4.0], [3.0, 2.0, 4.0], [6.0, 1.0, 6.0]], [[1.0, 1.0, 4.0], [1.0, 6.0, 2.0], [8.0, 1.0, 
8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 3.0, 2.0], [3.0, 6.0, 8.0], [4.0, 4.0, 4.0]], [[1.0, 1.0, 4.0], [5.0, 5.0, 2.0], 
[3.0, 3.0, 7.0]]]))
      arg[2:,:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 8.0], [1.0, 1.0, 5.0], [5.0, 4.0, 1.0]], [[1.0, 8.0, 5.0], [7.0, 5.0, 6.0], [2.0, 
8.0, 1.0]], [[6.0, 1.0, 4.0], [2.0, 6.0, 5.0], [6.0, 5.0, 6.0]]], [[[3.0, 4.0, 3.0], [7.0, 8.0, 4.0], [6.0, 2.0, 6.0]], [[8.0, 
4.0, 4.0], [3.0, 1.0, 6.0], [8.0, 7.0, 5.0]], [[3.0, 4.0, 1.0], [1.0, 8.0, 1.0], [6.0, 5.0, 7.0]]], [[[6.0, 1.0, 5.0], [6.0, 
2.0, 1.0], [7.0, 4.0, 4.0]], [[8.0, 3.0, 3.0], [2.0, 2.0, 3.0], [8.0, 4.0, 3.0]], [[6.0, 6.0, 5.0], [4.0, 1.0, 8.0], [2.0, 6.0, 
4.0]]], [[[2.0, 1.0, 2.0], [6.0, 1.0, 3.0], [7.0, 4.0, 4.0]], [[1.0, 1.0, 6.0], [6.0, 6.0, 3.0], [7.0, 2.0, 2.0]], [[4.0, 8.0, 
2.0], [6.0, 1.0, 6.0], [6.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 8.0], [5.0, 2.0, 3.0], [5.0, 7.0, 2.0]], 
[[1.0, 1.0, 2.0], [5.0, 3.0, 1.0], [2.0, 5.0, 4.0]], [[6.0, 5.0, 8.0], [3.0, 7.0, 2.0], [3.0, 8.0, 7.0]]], [[[5.0, 5.0, 4.0], 
[5.0, 6.0, 7.0], [1.0, 1.0, 7.0]], [[8.0, 8.0, 5.0], [5.0, 6.0, 7.0], [8.0, 1.0, 6.0]], [[2.0, 2.0, 8.0], [3.0, 3.0, 3.0], 
[8.0, 5.0, 3.0]]], [[[6.0, 2.0, 5.0], [7.0, 3.0, 7.0], [7.0, 2.0, 2.0]], [[2.0, 3.0, 4.0], [6.0, 6.0, 1.0], [8.0, 8.0, 6.0]], 
[[3.0, 4.0, 7.0], [4.0, 4.0, 8.0], [1.0, 4.0, 8.0]]], [[[3.0, 1.0, 8.0], [6.0, 1.0, 4.0], [2.0, 4.0, 8.0]], [[2.0, 5.0, 3.0], 
[8.0, 5.0, 1.0], [2.0, 2.0, 5.0]], [[8.0, 3.0, 1.0], [4.0, 3.0, 4.0], [5.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LURN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 8.0], [8.0, 6.0, 7.0], [7.0, 8.0, 2.0]], [[6.0, 3.0, 5.0], [8.0, 6.0, 3.0], [1.0, 
4.0, 8.0]], [[6.0, 5.0, 5.0], [4.0, 2.0, 7.0], [5.0, 7.0, 7.0]]], [[[3.0, 7.0, 3.0], [3.0, 2.0, 8.0], [7.0, 3.0, 5.0]], [[7.0, 
2.0, 2.0], [4.0, 2.0, 4.0], [3.0, 4.0, 5.0]], [[5.0, 3.0, 5.0], [8.0, 5.0, 4.0], [3.0, 7.0, 8.0]]], [[[8.0, 4.0, 8.0], [4.0, 
7.0, 1.0], [2.0, 7.0, 1.0]], [[1.0, 6.0, 3.0], [5.0, 8.0, 6.0], [8.0, 5.0, 5.0]], [[6.0, 5.0, 2.0], [3.0, 6.0, 4.0], [5.0, 6.0, 
3.0]]], [[[8.0, 6.0, 6.0], [7.0, 5.0, 2.0], [8.0, 3.0, 1.0]], [[3.0, 2.0, 4.0], [2.0, 5.0, 5.0], [7.0, 1.0, 3.0]], [[1.0, 4.0, 
6.0], [8.0, 5.0, 6.0], [7.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 7.0], [2.0, 3.0, 5.0], [6.0, 4.0, 6.0]], 
[[3.0, 4.0, 7.0], [4.0, 3.0, 3.0], [1.0, 1.0, 2.0]], [[6.0, 2.0, 8.0], [3.0, 2.0, 4.0], [4.0, 7.0, 6.0]]], [[[2.0, 4.0, 3.0], 
[4.0, 3.0, 6.0], [1.0, 3.0, 7.0]], [[6.0, 8.0, 2.0], [2.0, 6.0, 6.0], [8.0, 8.0, 1.0]], [[6.0, 3.0, 4.0], [7.0, 6.0, 6.0], 
[1.0, 2.0, 3.0]]], [[[7.0, 2.0, 4.0], [4.0, 5.0, 1.0], [1.0, 7.0, 4.0]], [[5.0, 3.0, 3.0], [4.0, 4.0, 6.0], [4.0, 3.0, 2.0]], 
[[4.0, 5.0, 7.0], [4.0, 7.0, 2.0], [7.0, 7.0, 1.0]]], [[[3.0, 3.0, 2.0], [7.0, 7.0, 6.0], [7.0, 5.0, 5.0]], [[2.0, 6.0, 7.0], 
[4.0, 3.0, 1.0], [4.0, 3.0, 6.0]], [[2.0, 6.0, 2.0], [7.0, 5.0, 1.0], [1.0, 3.0, 1.0]]]])
      val=Data(numpy.array([[[[6.0, 5.0, 4.0]], [[2.0, 8.0, 5.0]]], [[[3.0, 2.0, 6.0]], [[7.0, 1.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 6.0, 7.0]], [[2.0, 1.0, 6.0]]], [[[7.0, 6.0, 7.0]], [[8.0, 8.0, 4.0]]]]))
      arg[2:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 8.0], [8.0, 6.0, 7.0], [7.0, 8.0, 2.0]], [[6.0, 3.0, 5.0], [8.0, 6.0, 3.0], [1.0, 
4.0, 8.0]], [[6.0, 5.0, 5.0], [4.0, 2.0, 7.0], [5.0, 7.0, 7.0]]], [[[3.0, 7.0, 3.0], [3.0, 2.0, 8.0], [7.0, 3.0, 5.0]], [[7.0, 
2.0, 2.0], [4.0, 2.0, 4.0], [3.0, 4.0, 5.0]], [[5.0, 3.0, 5.0], [8.0, 5.0, 4.0], [3.0, 7.0, 8.0]]], [[[8.0, 4.0, 8.0], [6.0, 
5.0, 4.0], [2.0, 7.0, 1.0]], [[1.0, 6.0, 3.0], [2.0, 8.0, 5.0], [8.0, 5.0, 5.0]], [[6.0, 5.0, 2.0], [3.0, 6.0, 4.0], [5.0, 6.0, 
3.0]]], [[[8.0, 6.0, 6.0], [3.0, 2.0, 6.0], [8.0, 3.0, 1.0]], [[3.0, 2.0, 4.0], [7.0, 1.0, 6.0], [7.0, 1.0, 3.0]], [[1.0, 4.0, 
6.0], [8.0, 5.0, 6.0], [7.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 7.0], [2.0, 3.0, 5.0], [6.0, 4.0, 6.0]], 
[[3.0, 4.0, 7.0], [4.0, 3.0, 3.0], [1.0, 1.0, 2.0]], [[6.0, 2.0, 8.0], [3.0, 2.0, 4.0], [4.0, 7.0, 6.0]]], [[[2.0, 4.0, 3.0], 
[4.0, 3.0, 6.0], [1.0, 3.0, 7.0]], [[6.0, 8.0, 2.0], [2.0, 6.0, 6.0], [8.0, 8.0, 1.0]], [[6.0, 3.0, 4.0], [7.0, 6.0, 6.0], 
[1.0, 2.0, 3.0]]], [[[7.0, 2.0, 4.0], [1.0, 6.0, 7.0], [1.0, 7.0, 4.0]], [[5.0, 3.0, 3.0], [2.0, 1.0, 6.0], [4.0, 3.0, 2.0]], 
[[4.0, 5.0, 7.0], [4.0, 7.0, 2.0], [7.0, 7.0, 1.0]]], [[[3.0, 3.0, 2.0], [7.0, 6.0, 7.0], [7.0, 5.0, 5.0]], [[2.0, 6.0, 7.0], 
[8.0, 8.0, 4.0], [4.0, 3.0, 6.0]], [[2.0, 6.0, 2.0], [7.0, 5.0, 1.0], [1.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URRC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 6.0], [1.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[6.0, 5.0, 6.0], [3.0, 1.0, 6.0], [7.0, 
7.0, 3.0]], [[5.0, 7.0, 4.0], [1.0, 7.0, 4.0], [6.0, 4.0, 1.0]]], [[[1.0, 8.0, 3.0], [1.0, 1.0, 8.0], [2.0, 1.0, 8.0]], [[2.0, 
8.0, 1.0], [6.0, 6.0, 2.0], [4.0, 5.0, 7.0]], [[8.0, 2.0, 5.0], [5.0, 1.0, 5.0], [2.0, 4.0, 8.0]]], [[[8.0, 7.0, 7.0], [5.0, 
3.0, 1.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 2.0], [1.0, 5.0, 6.0], [7.0, 7.0, 4.0]], [[8.0, 3.0, 6.0], [3.0, 8.0, 8.0], [8.0, 4.0, 
4.0]]], [[[4.0, 4.0, 3.0], [7.0, 8.0, 7.0], [1.0, 8.0, 7.0]], [[3.0, 7.0, 2.0], [6.0, 4.0, 1.0], [6.0, 4.0, 8.0]], [[3.0, 7.0, 
2.0], [2.0, 5.0, 8.0], [7.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 2.0], [8.0, 3.0, 8.0], [2.0, 8.0, 8.0]], 
[[4.0, 4.0, 3.0], [8.0, 4.0, 7.0], [7.0, 5.0, 6.0]], [[6.0, 8.0, 7.0], [8.0, 1.0, 1.0], [2.0, 3.0, 7.0]]], [[[4.0, 8.0, 5.0], 
[2.0, 1.0, 3.0], [3.0, 6.0, 4.0]], [[7.0, 3.0, 7.0], [6.0, 5.0, 2.0], [1.0, 6.0, 5.0]], [[3.0, 5.0, 6.0], [7.0, 4.0, 5.0], 
[4.0, 5.0, 3.0]]], [[[5.0, 1.0, 2.0], [1.0, 1.0, 4.0], [6.0, 1.0, 6.0]], [[5.0, 6.0, 1.0], [8.0, 4.0, 5.0], [1.0, 8.0, 6.0]], 
[[7.0, 2.0, 5.0], [8.0, 8.0, 5.0], [3.0, 5.0, 8.0]]], [[[5.0, 1.0, 1.0], [1.0, 3.0, 4.0], [5.0, 1.0, 1.0]], [[5.0, 3.0, 4.0], 
[3.0, 4.0, 1.0], [6.0, 8.0, 2.0]], [[6.0, 8.0, 2.0], [1.0, 8.0, 1.0], [6.0, 5.0, 3.0]]]])
      val=Data(numpy.array([[[[4.0, 3.0, 5.0]]], [[[2.0, 7.0, 6.0]]], [[[1.0, 7.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 8.0, 2.0]]], [[[8.0, 6.0, 2.0]]], [[[6.0, 2.0, 2.0]]]]))
      arg[:3,1:2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 6.0], [1.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[6.0, 5.0, 6.0], [4.0, 3.0, 5.0], [7.0, 
7.0, 3.0]], [[5.0, 7.0, 4.0], [1.0, 7.0, 4.0], [6.0, 4.0, 1.0]]], [[[1.0, 8.0, 3.0], [1.0, 1.0, 8.0], [2.0, 1.0, 8.0]], [[2.0, 
8.0, 1.0], [2.0, 7.0, 6.0], [4.0, 5.0, 7.0]], [[8.0, 2.0, 5.0], [5.0, 1.0, 5.0], [2.0, 4.0, 8.0]]], [[[8.0, 7.0, 7.0], [5.0, 
3.0, 1.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 2.0], [1.0, 7.0, 6.0], [7.0, 7.0, 4.0]], [[8.0, 3.0, 6.0], [3.0, 8.0, 8.0], [8.0, 4.0, 
4.0]]], [[[4.0, 4.0, 3.0], [7.0, 8.0, 7.0], [1.0, 8.0, 7.0]], [[3.0, 7.0, 2.0], [6.0, 4.0, 1.0], [6.0, 4.0, 8.0]], [[3.0, 7.0, 
2.0], [2.0, 5.0, 8.0], [7.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 2.0], [8.0, 3.0, 8.0], [2.0, 8.0, 8.0]], 
[[4.0, 4.0, 3.0], [5.0, 8.0, 2.0], [7.0, 5.0, 6.0]], [[6.0, 8.0, 7.0], [8.0, 1.0, 1.0], [2.0, 3.0, 7.0]]], [[[4.0, 8.0, 5.0], 
[2.0, 1.0, 3.0], [3.0, 6.0, 4.0]], [[7.0, 3.0, 7.0], [8.0, 6.0, 2.0], [1.0, 6.0, 5.0]], [[3.0, 5.0, 6.0], [7.0, 4.0, 5.0], 
[4.0, 5.0, 3.0]]], [[[5.0, 1.0, 2.0], [1.0, 1.0, 4.0], [6.0, 1.0, 6.0]], [[5.0, 6.0, 1.0], [6.0, 2.0, 2.0], [1.0, 8.0, 6.0]], 
[[7.0, 2.0, 5.0], [8.0, 8.0, 5.0], [3.0, 5.0, 8.0]]], [[[5.0, 1.0, 1.0], [1.0, 3.0, 4.0], [5.0, 1.0, 1.0]], [[5.0, 3.0, 4.0], 
[3.0, 4.0, 1.0], [6.0, 8.0, 2.0]], [[6.0, 8.0, 2.0], [1.0, 8.0, 1.0], [6.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CPCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 3.0], [5.0, 5.0, 2.0], [4.0, 3.0, 7.0]], [[1.0, 8.0, 2.0], [8.0, 5.0, 4.0], [8.0, 
7.0, 2.0]], [[7.0, 8.0, 5.0], [2.0, 2.0, 1.0], [5.0, 4.0, 2.0]]], [[[3.0, 8.0, 6.0], [8.0, 8.0, 5.0], [4.0, 2.0, 6.0]], [[8.0, 
3.0, 8.0], [3.0, 7.0, 2.0], [2.0, 2.0, 7.0]], [[1.0, 2.0, 3.0], [4.0, 1.0, 4.0], [3.0, 3.0, 2.0]]], [[[5.0, 8.0, 4.0], [8.0, 
4.0, 1.0], [5.0, 7.0, 5.0]], [[8.0, 6.0, 8.0], [5.0, 4.0, 2.0], [3.0, 4.0, 1.0]], [[7.0, 4.0, 5.0], [2.0, 1.0, 5.0], [2.0, 1.0, 
1.0]]], [[[2.0, 7.0, 3.0], [3.0, 6.0, 1.0], [1.0, 8.0, 7.0]], [[3.0, 3.0, 3.0], [4.0, 4.0, 1.0], [1.0, 1.0, 6.0]], [[2.0, 1.0, 
1.0], [3.0, 3.0, 3.0], [5.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 3.0], [5.0, 5.0, 7.0], [7.0, 2.0, 4.0]], 
[[1.0, 8.0, 7.0], [8.0, 2.0, 2.0], [3.0, 8.0, 5.0]], [[4.0, 1.0, 1.0], [5.0, 2.0, 2.0], [7.0, 8.0, 6.0]]], [[[8.0, 7.0, 5.0], 
[3.0, 1.0, 1.0], [8.0, 5.0, 4.0]], [[4.0, 4.0, 5.0], [6.0, 2.0, 8.0], [8.0, 2.0, 5.0]], [[1.0, 7.0, 4.0], [2.0, 3.0, 8.0], 
[2.0, 1.0, 4.0]]], [[[6.0, 7.0, 6.0], [4.0, 7.0, 6.0], [2.0, 3.0, 7.0]], [[4.0, 4.0, 4.0], [2.0, 3.0, 2.0], [3.0, 6.0, 3.0]], 
[[1.0, 6.0, 7.0], [8.0, 3.0, 3.0], [7.0, 3.0, 7.0]]], [[[2.0, 3.0, 1.0], [1.0, 1.0, 4.0], [2.0, 2.0, 8.0]], [[1.0, 1.0, 4.0], 
[7.0, 4.0, 6.0], [2.0, 2.0, 2.0]], [[6.0, 1.0, 5.0], [2.0, 8.0, 3.0], [4.0, 7.0, 2.0]]]])
      val=Data(numpy.array([[[5.0, 8.0, 7.0], [7.0, 6.0, 5.0], [1.0, 6.0, 7.0]], [[1.0, 6.0, 2.0], [6.0, 4.0, 2.0], [4.0, 7.0, 
5.0]], [[4.0, 3.0, 1.0], [3.0, 1.0, 2.0], [7.0, 2.0, 2.0]], [[2.0, 2.0, 2.0], [2.0, 1.0, 3.0], [5.0, 2.0, 
5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 3.0, 8.0], [6.0, 4.0, 6.0], [7.0, 8.0, 1.0]], [[1.0, 3.0, 8.0], [2.0, 5.0, 5.0], 
[7.0, 3.0, 3.0]], [[2.0, 3.0, 3.0], [2.0, 2.0, 8.0], [8.0, 8.0, 4.0]], [[4.0, 2.0, 1.0], [1.0, 6.0, 4.0], [8.0, 6.0, 7.0]]]))
      arg[:,1,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 3.0], [5.0, 5.0, 2.0], [4.0, 3.0, 7.0]], [[5.0, 8.0, 7.0], [7.0, 6.0, 5.0], [1.0, 
6.0, 7.0]], [[7.0, 8.0, 5.0], [2.0, 2.0, 1.0], [5.0, 4.0, 2.0]]], [[[3.0, 8.0, 6.0], [8.0, 8.0, 5.0], [4.0, 2.0, 6.0]], [[1.0, 
6.0, 2.0], [6.0, 4.0, 2.0], [4.0, 7.0, 5.0]], [[1.0, 2.0, 3.0], [4.0, 1.0, 4.0], [3.0, 3.0, 2.0]]], [[[5.0, 8.0, 4.0], [8.0, 
4.0, 1.0], [5.0, 7.0, 5.0]], [[4.0, 3.0, 1.0], [3.0, 1.0, 2.0], [7.0, 2.0, 2.0]], [[7.0, 4.0, 5.0], [2.0, 1.0, 5.0], [2.0, 1.0, 
1.0]]], [[[2.0, 7.0, 3.0], [3.0, 6.0, 1.0], [1.0, 8.0, 7.0]], [[2.0, 2.0, 2.0], [2.0, 1.0, 3.0], [5.0, 2.0, 5.0]], [[2.0, 1.0, 
1.0], [3.0, 3.0, 3.0], [5.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 3.0], [5.0, 5.0, 7.0], [7.0, 2.0, 4.0]], 
[[4.0, 3.0, 8.0], [6.0, 4.0, 6.0], [7.0, 8.0, 1.0]], [[4.0, 1.0, 1.0], [5.0, 2.0, 2.0], [7.0, 8.0, 6.0]]], [[[8.0, 7.0, 5.0], 
[3.0, 1.0, 1.0], [8.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], [2.0, 5.0, 5.0], [7.0, 3.0, 3.0]], [[1.0, 7.0, 4.0], [2.0, 3.0, 8.0], 
[2.0, 1.0, 4.0]]], [[[6.0, 7.0, 6.0], [4.0, 7.0, 6.0], [2.0, 3.0, 7.0]], [[2.0, 3.0, 3.0], [2.0, 2.0, 8.0], [8.0, 8.0, 4.0]], 
[[1.0, 6.0, 7.0], [8.0, 3.0, 3.0], [7.0, 3.0, 7.0]]], [[[2.0, 3.0, 1.0], [1.0, 1.0, 4.0], [2.0, 2.0, 8.0]], [[4.0, 2.0, 1.0], 
[1.0, 6.0, 4.0], [8.0, 6.0, 7.0]], [[6.0, 1.0, 5.0], [2.0, 8.0, 3.0], [4.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRCL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 3.0], [7.0, 7.0, 2.0], [1.0, 4.0, 2.0]], [[5.0, 8.0, 7.0], [1.0, 4.0, 1.0], [7.0, 
5.0, 7.0]], [[4.0, 8.0, 2.0], [5.0, 4.0, 1.0], [2.0, 4.0, 5.0]]], [[[7.0, 3.0, 4.0], [2.0, 1.0, 3.0], [5.0, 7.0, 8.0]], [[7.0, 
6.0, 7.0], [4.0, 2.0, 5.0], [1.0, 7.0, 3.0]], [[8.0, 3.0, 3.0], [6.0, 3.0, 7.0], [5.0, 4.0, 3.0]]], [[[3.0, 8.0, 3.0], [5.0, 
8.0, 5.0], [8.0, 8.0, 8.0]], [[4.0, 8.0, 3.0], [5.0, 6.0, 4.0], [1.0, 6.0, 2.0]], [[2.0, 3.0, 7.0], [1.0, 4.0, 8.0], [1.0, 2.0, 
3.0]]], [[[5.0, 3.0, 8.0], [2.0, 2.0, 2.0], [7.0, 5.0, 8.0]], [[1.0, 3.0, 6.0], [5.0, 7.0, 3.0], [3.0, 4.0, 8.0]], [[8.0, 7.0, 
2.0], [4.0, 7.0, 8.0], [5.0, 7.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 7.0], [7.0, 5.0, 5.0], [1.0, 6.0, 5.0]], 
[[2.0, 6.0, 7.0], [3.0, 5.0, 5.0], [4.0, 3.0, 6.0]], [[4.0, 1.0, 6.0], [1.0, 6.0, 6.0], [3.0, 7.0, 6.0]]], [[[4.0, 5.0, 6.0], 
[1.0, 6.0, 8.0], [7.0, 4.0, 7.0]], [[2.0, 1.0, 6.0], [7.0, 8.0, 6.0], [2.0, 6.0, 2.0]], [[4.0, 7.0, 3.0], [4.0, 5.0, 2.0], 
[7.0, 4.0, 6.0]]], [[[3.0, 3.0, 7.0], [6.0, 3.0, 6.0], [8.0, 4.0, 6.0]], [[3.0, 1.0, 7.0], [6.0, 6.0, 2.0], [3.0, 6.0, 1.0]], 
[[7.0, 1.0, 6.0], [4.0, 7.0, 4.0], [4.0, 4.0, 4.0]]], [[[3.0, 8.0, 7.0], [5.0, 6.0, 4.0], [4.0, 8.0, 2.0]], [[2.0, 8.0, 1.0], 
[7.0, 3.0, 5.0], [6.0, 3.0, 7.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 8.0], [1.0, 2.0, 2.0]]]])
      val=Data(numpy.array([[[[4.0], [5.0], [8.0]]], [[[2.0], [4.0], [6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0], [5.0], [3.0]]], [[[2.0], [6.0], [8.0]]]]))
      arg[2:,1:2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 3.0], [7.0, 7.0, 2.0], [1.0, 4.0, 2.0]], [[5.0, 8.0, 7.0], [1.0, 4.0, 1.0], [7.0, 
5.0, 7.0]], [[4.0, 8.0, 2.0], [5.0, 4.0, 1.0], [2.0, 4.0, 5.0]]], [[[7.0, 3.0, 4.0], [2.0, 1.0, 3.0], [5.0, 7.0, 8.0]], [[7.0, 
6.0, 7.0], [4.0, 2.0, 5.0], [1.0, 7.0, 3.0]], [[8.0, 3.0, 3.0], [6.0, 3.0, 7.0], [5.0, 4.0, 3.0]]], [[[3.0, 8.0, 3.0], [5.0, 
8.0, 5.0], [8.0, 8.0, 8.0]], [[4.0, 8.0, 4.0], [5.0, 6.0, 5.0], [1.0, 6.0, 8.0]], [[2.0, 3.0, 7.0], [1.0, 4.0, 8.0], [1.0, 2.0, 
3.0]]], [[[5.0, 3.0, 8.0], [2.0, 2.0, 2.0], [7.0, 5.0, 8.0]], [[1.0, 3.0, 2.0], [5.0, 7.0, 4.0], [3.0, 4.0, 6.0]], [[8.0, 7.0, 
2.0], [4.0, 7.0, 8.0], [5.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 7.0], [7.0, 5.0, 5.0], [1.0, 6.0, 5.0]], 
[[2.0, 6.0, 7.0], [3.0, 5.0, 5.0], [4.0, 3.0, 6.0]], [[4.0, 1.0, 6.0], [1.0, 6.0, 6.0], [3.0, 7.0, 6.0]]], [[[4.0, 5.0, 6.0], 
[1.0, 6.0, 8.0], [7.0, 4.0, 7.0]], [[2.0, 1.0, 6.0], [7.0, 8.0, 6.0], [2.0, 6.0, 2.0]], [[4.0, 7.0, 3.0], [4.0, 5.0, 2.0], 
[7.0, 4.0, 6.0]]], [[[3.0, 3.0, 7.0], [6.0, 3.0, 6.0], [8.0, 4.0, 6.0]], [[3.0, 1.0, 3.0], [6.0, 6.0, 5.0], [3.0, 6.0, 3.0]], 
[[7.0, 1.0, 6.0], [4.0, 7.0, 4.0], [4.0, 4.0, 4.0]]], [[[3.0, 8.0, 7.0], [5.0, 6.0, 4.0], [4.0, 8.0, 2.0]], [[2.0, 8.0, 2.0], 
[7.0, 3.0, 6.0], [6.0, 3.0, 8.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 8.0], [1.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULLP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 3.0], [3.0, 7.0, 3.0], [5.0, 3.0, 4.0]], [[3.0, 3.0, 6.0], [2.0, 1.0, 8.0], [6.0, 
2.0, 8.0]], [[5.0, 8.0, 1.0], [8.0, 3.0, 6.0], [4.0, 7.0, 2.0]]], [[[8.0, 7.0, 5.0], [7.0, 4.0, 4.0], [6.0, 3.0, 3.0]], [[7.0, 
3.0, 7.0], [5.0, 8.0, 6.0], [3.0, 1.0, 3.0]], [[5.0, 5.0, 6.0], [2.0, 2.0, 7.0], [7.0, 7.0, 5.0]]], [[[7.0, 5.0, 2.0], [4.0, 
8.0, 8.0], [3.0, 7.0, 2.0]], [[5.0, 4.0, 5.0], [6.0, 6.0, 7.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 1.0], [3.0, 5.0, 5.0], [8.0, 7.0, 
2.0]]], [[[6.0, 4.0, 8.0], [7.0, 3.0, 6.0], [2.0, 1.0, 2.0]], [[3.0, 4.0, 1.0], [5.0, 5.0, 4.0], [2.0, 7.0, 3.0]], [[6.0, 2.0, 
6.0], [4.0, 7.0, 4.0], [7.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 4.0], [5.0, 4.0, 6.0], [4.0, 6.0, 6.0]], 
[[6.0, 1.0, 7.0], [4.0, 2.0, 1.0], [3.0, 1.0, 6.0]], [[8.0, 5.0, 5.0], [8.0, 3.0, 2.0], [5.0, 8.0, 3.0]]], [[[3.0, 6.0, 5.0], 
[3.0, 4.0, 1.0], [1.0, 3.0, 4.0]], [[3.0, 2.0, 1.0], [6.0, 1.0, 7.0], [5.0, 7.0, 4.0]], [[6.0, 5.0, 6.0], [8.0, 2.0, 2.0], 
[8.0, 8.0, 6.0]]], [[[5.0, 4.0, 3.0], [1.0, 3.0, 4.0], [1.0, 8.0, 4.0]], [[7.0, 5.0, 6.0], [3.0, 2.0, 8.0], [7.0, 5.0, 6.0]], 
[[8.0, 8.0, 4.0], [7.0, 3.0, 4.0], [2.0, 8.0, 6.0]]], [[[4.0, 1.0, 6.0], [4.0, 6.0, 6.0], [7.0, 3.0, 5.0]], [[1.0, 7.0, 8.0], 
[1.0, 1.0, 7.0], [3.0, 8.0, 6.0]], [[2.0, 7.0, 5.0], [8.0, 2.0, 5.0], [4.0, 5.0, 4.0]]]])
      val=Data(numpy.array([[[7.0]], [[6.0]], [[3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0]], [[6.0]], [[2.0]]]))
      arg[:3,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 3.0], [3.0, 7.0, 3.0], [5.0, 3.0, 4.0]], [[3.0, 3.0, 6.0], [2.0, 1.0, 8.0], [6.0, 
2.0, 8.0]], [[5.0, 8.0, 1.0], [8.0, 3.0, 6.0], [4.0, 7.0, 2.0]]], [[[8.0, 7.0, 5.0], [7.0, 4.0, 4.0], [6.0, 3.0, 3.0]], [[7.0, 
3.0, 7.0], [5.0, 8.0, 6.0], [3.0, 1.0, 3.0]], [[5.0, 5.0, 6.0], [2.0, 2.0, 7.0], [7.0, 6.0, 5.0]]], [[[7.0, 5.0, 2.0], [4.0, 
8.0, 8.0], [3.0, 7.0, 2.0]], [[5.0, 4.0, 5.0], [6.0, 6.0, 7.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 1.0], [3.0, 5.0, 5.0], [8.0, 3.0, 
2.0]]], [[[6.0, 4.0, 8.0], [7.0, 3.0, 6.0], [2.0, 1.0, 2.0]], [[3.0, 4.0, 1.0], [5.0, 5.0, 4.0], [2.0, 7.0, 3.0]], [[6.0, 2.0, 
6.0], [4.0, 7.0, 4.0], [7.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 4.0], [5.0, 4.0, 6.0], [4.0, 6.0, 6.0]], 
[[6.0, 1.0, 7.0], [4.0, 2.0, 1.0], [3.0, 1.0, 6.0]], [[8.0, 5.0, 5.0], [8.0, 3.0, 2.0], [5.0, 8.0, 3.0]]], [[[3.0, 6.0, 5.0], 
[3.0, 4.0, 1.0], [1.0, 3.0, 4.0]], [[3.0, 2.0, 1.0], [6.0, 1.0, 7.0], [5.0, 7.0, 4.0]], [[6.0, 5.0, 6.0], [8.0, 2.0, 2.0], 
[8.0, 6.0, 6.0]]], [[[5.0, 4.0, 3.0], [1.0, 3.0, 4.0], [1.0, 8.0, 4.0]], [[7.0, 5.0, 6.0], [3.0, 2.0, 8.0], [7.0, 5.0, 6.0]], 
[[8.0, 8.0, 4.0], [7.0, 3.0, 4.0], [2.0, 2.0, 6.0]]], [[[4.0, 1.0, 6.0], [4.0, 6.0, 6.0], [7.0, 3.0, 5.0]], [[1.0, 7.0, 8.0], 
[1.0, 1.0, 7.0], [3.0, 8.0, 6.0]], [[2.0, 7.0, 5.0], [8.0, 2.0, 5.0], [4.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 3.0], [3.0, 2.0, 6.0], [7.0, 6.0, 3.0]], [[4.0, 1.0, 5.0], [1.0, 2.0, 5.0], [7.0, 
8.0, 5.0]], [[7.0, 6.0, 3.0], [4.0, 6.0, 4.0], [7.0, 4.0, 3.0]]], [[[7.0, 7.0, 3.0], [1.0, 8.0, 7.0], [5.0, 4.0, 1.0]], [[4.0, 
4.0, 4.0], [8.0, 3.0, 7.0], [3.0, 4.0, 6.0]], [[3.0, 6.0, 6.0], [4.0, 2.0, 3.0], [2.0, 3.0, 3.0]]], [[[4.0, 1.0, 8.0], [5.0, 
5.0, 1.0], [2.0, 2.0, 7.0]], [[8.0, 5.0, 3.0], [5.0, 7.0, 4.0], [1.0, 1.0, 8.0]], [[3.0, 3.0, 7.0], [6.0, 2.0, 2.0], [2.0, 4.0, 
7.0]]], [[[4.0, 8.0, 2.0], [2.0, 2.0, 6.0], [1.0, 5.0, 7.0]], [[2.0, 7.0, 3.0], [2.0, 8.0, 2.0], [1.0, 7.0, 2.0]], [[2.0, 3.0, 
4.0], [8.0, 8.0, 6.0], [3.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 1.0], [6.0, 1.0, 4.0], [1.0, 1.0, 6.0]], 
[[2.0, 1.0, 6.0], [4.0, 7.0, 4.0], [8.0, 7.0, 7.0]], [[1.0, 4.0, 4.0], [7.0, 2.0, 4.0], [5.0, 4.0, 7.0]]], [[[3.0, 6.0, 1.0], 
[4.0, 6.0, 4.0], [8.0, 5.0, 8.0]], [[2.0, 1.0, 1.0], [1.0, 6.0, 8.0], [1.0, 7.0, 2.0]], [[1.0, 8.0, 3.0], [8.0, 1.0, 2.0], 
[8.0, 1.0, 1.0]]], [[[7.0, 4.0, 1.0], [7.0, 6.0, 8.0], [5.0, 2.0, 8.0]], [[2.0, 8.0, 6.0], [4.0, 2.0, 6.0], [7.0, 5.0, 1.0]], 
[[8.0, 6.0, 5.0], [6.0, 3.0, 4.0], [8.0, 8.0, 5.0]]], [[[6.0, 7.0, 8.0], [8.0, 6.0, 1.0], [2.0, 8.0, 8.0]], [[8.0, 4.0, 5.0], 
[6.0, 6.0, 3.0], [3.0, 1.0, 4.0]], [[3.0, 3.0, 7.0], [8.0, 5.0, 8.0], [6.0, 5.0, 7.0]]]])
      val=Data(numpy.array([[[[3.0, 6.0, 3.0], [5.0, 6.0, 1.0], [7.0, 3.0, 3.0]]], [[[3.0, 1.0, 4.0], [8.0, 7.0, 6.0], [8.0, 
6.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 5.0, 1.0], [4.0, 6.0, 2.0], [4.0, 8.0, 8.0]]], [[[4.0, 5.0, 2.0], [5.0, 5.0, 
4.0], [7.0, 5.0, 4.0]]]]))
      arg[2:,2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 3.0], [3.0, 2.0, 6.0], [7.0, 6.0, 3.0]], [[4.0, 1.0, 5.0], [1.0, 2.0, 5.0], [7.0, 
8.0, 5.0]], [[7.0, 6.0, 3.0], [4.0, 6.0, 4.0], [7.0, 4.0, 3.0]]], [[[7.0, 7.0, 3.0], [1.0, 8.0, 7.0], [5.0, 4.0, 1.0]], [[4.0, 
4.0, 4.0], [8.0, 3.0, 7.0], [3.0, 4.0, 6.0]], [[3.0, 6.0, 6.0], [4.0, 2.0, 3.0], [2.0, 3.0, 3.0]]], [[[4.0, 1.0, 8.0], [5.0, 
5.0, 1.0], [2.0, 2.0, 7.0]], [[8.0, 5.0, 3.0], [5.0, 7.0, 4.0], [1.0, 1.0, 8.0]], [[3.0, 6.0, 3.0], [5.0, 6.0, 1.0], [7.0, 3.0, 
3.0]]], [[[4.0, 8.0, 2.0], [2.0, 2.0, 6.0], [1.0, 5.0, 7.0]], [[2.0, 7.0, 3.0], [2.0, 8.0, 2.0], [1.0, 7.0, 2.0]], [[3.0, 1.0, 
4.0], [8.0, 7.0, 6.0], [8.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 1.0], [6.0, 1.0, 4.0], [1.0, 1.0, 6.0]], 
[[2.0, 1.0, 6.0], [4.0, 7.0, 4.0], [8.0, 7.0, 7.0]], [[1.0, 4.0, 4.0], [7.0, 2.0, 4.0], [5.0, 4.0, 7.0]]], [[[3.0, 6.0, 1.0], 
[4.0, 6.0, 4.0], [8.0, 5.0, 8.0]], [[2.0, 1.0, 1.0], [1.0, 6.0, 8.0], [1.0, 7.0, 2.0]], [[1.0, 8.0, 3.0], [8.0, 1.0, 2.0], 
[8.0, 1.0, 1.0]]], [[[7.0, 4.0, 1.0], [7.0, 6.0, 8.0], [5.0, 2.0, 8.0]], [[2.0, 8.0, 6.0], [4.0, 2.0, 6.0], [7.0, 5.0, 1.0]], 
[[2.0, 5.0, 1.0], [4.0, 6.0, 2.0], [4.0, 8.0, 8.0]]], [[[6.0, 7.0, 8.0], [8.0, 6.0, 1.0], [2.0, 8.0, 8.0]], [[8.0, 4.0, 5.0], 
[6.0, 6.0, 3.0], [3.0, 1.0, 4.0]], [[4.0, 5.0, 2.0], [5.0, 5.0, 4.0], [7.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCUC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 3.0], [1.0, 6.0, 3.0], [6.0, 7.0, 2.0]], [[1.0, 5.0, 8.0], [1.0, 1.0, 8.0], [7.0, 
7.0, 7.0]], [[6.0, 6.0, 4.0], [1.0, 6.0, 6.0], [7.0, 6.0, 5.0]]], [[[4.0, 4.0, 4.0], [8.0, 5.0, 1.0], [7.0, 2.0, 7.0]], [[8.0, 
2.0, 4.0], [5.0, 3.0, 4.0], [5.0, 4.0, 6.0]], [[3.0, 2.0, 5.0], [2.0, 4.0, 6.0], [1.0, 3.0, 1.0]]], [[[1.0, 7.0, 1.0], [5.0, 
6.0, 7.0], [6.0, 1.0, 5.0]], [[1.0, 5.0, 4.0], [1.0, 7.0, 7.0], [8.0, 8.0, 7.0]], [[5.0, 3.0, 4.0], [6.0, 2.0, 2.0], [2.0, 8.0, 
8.0]]], [[[4.0, 5.0, 1.0], [6.0, 7.0, 4.0], [7.0, 5.0, 8.0]], [[6.0, 8.0, 4.0], [4.0, 8.0, 3.0], [6.0, 8.0, 4.0]], [[5.0, 6.0, 
6.0], [5.0, 6.0, 2.0], [5.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 8.0], [8.0, 6.0, 3.0], [1.0, 6.0, 6.0]], 
[[1.0, 8.0, 6.0], [8.0, 3.0, 3.0], [7.0, 6.0, 1.0]], [[8.0, 2.0, 6.0], [1.0, 6.0, 6.0], [7.0, 3.0, 4.0]]], [[[2.0, 2.0, 3.0], 
[7.0, 5.0, 3.0], [2.0, 7.0, 4.0]], [[2.0, 1.0, 4.0], [7.0, 4.0, 6.0], [7.0, 1.0, 8.0]], [[3.0, 3.0, 4.0], [3.0, 6.0, 4.0], 
[3.0, 7.0, 6.0]]], [[[4.0, 7.0, 7.0], [6.0, 3.0, 4.0], [2.0, 8.0, 7.0]], [[5.0, 7.0, 2.0], [4.0, 1.0, 6.0], [5.0, 3.0, 8.0]], 
[[5.0, 1.0, 8.0], [1.0, 5.0, 7.0], [7.0, 8.0, 2.0]]], [[[2.0, 8.0, 5.0], [5.0, 3.0, 4.0], [2.0, 5.0, 3.0]], [[5.0, 5.0, 8.0], 
[5.0, 3.0, 2.0], [6.0, 4.0, 8.0]], [[4.0, 3.0, 2.0], [7.0, 6.0, 6.0], [5.0, 2.0, 5.0]]]])
      val=Data(numpy.array([[[[4.0, 6.0, 4.0], [3.0, 7.0, 1.0]], [[5.0, 6.0, 2.0], [2.0, 4.0, 1.0]], [[6.0, 3.0, 3.0], [7.0, 
3.0, 3.0]]], [[[2.0, 8.0, 2.0], [2.0, 8.0, 1.0]], [[4.0, 7.0, 5.0], [6.0, 2.0, 2.0]], [[8.0, 4.0, 8.0], [3.0, 5.0, 4.0]]], 
[[[7.0, 2.0, 6.0], [5.0, 1.0, 8.0]], [[8.0, 8.0, 3.0], [2.0, 1.0, 2.0]], [[3.0, 1.0, 4.0], [7.0, 2.0, 4.0]]], [[[7.0, 4.0, 
3.0], [6.0, 1.0, 5.0]], [[6.0, 2.0, 3.0], [3.0, 6.0, 8.0]], [[7.0, 1.0, 7.0], [3.0, 1.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 3.0, 5.0], [8.0, 1.0, 7.0]], [[4.0, 6.0, 8.0], [3.0, 6.0, 8.0]], [[6.0, 3.0, 
8.0], [5.0, 4.0, 8.0]]], [[[7.0, 4.0, 6.0], [3.0, 5.0, 1.0]], [[7.0, 3.0, 7.0], [4.0, 3.0, 3.0]], [[3.0, 5.0, 5.0], [7.0, 7.0, 
2.0]]], [[[8.0, 6.0, 7.0], [3.0, 4.0, 8.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 8.0]], [[2.0, 8.0, 2.0], [6.0, 8.0, 6.0]]], [[[3.0, 
6.0, 7.0], [6.0, 2.0, 2.0]], [[1.0, 1.0, 6.0], [6.0, 1.0, 3.0]], [[3.0, 5.0, 8.0], [8.0, 4.0, 7.0]]]]))
      arg[:,:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 4.0], [3.0, 7.0, 1.0], [6.0, 7.0, 2.0]], [[5.0, 6.0, 2.0], [2.0, 4.0, 1.0], [7.0, 
7.0, 7.0]], [[6.0, 3.0, 3.0], [7.0, 3.0, 3.0], [7.0, 6.0, 5.0]]], [[[2.0, 8.0, 2.0], [2.0, 8.0, 1.0], [7.0, 2.0, 7.0]], [[4.0, 
7.0, 5.0], [6.0, 2.0, 2.0], [5.0, 4.0, 6.0]], [[8.0, 4.0, 8.0], [3.0, 5.0, 4.0], [1.0, 3.0, 1.0]]], [[[7.0, 2.0, 6.0], [5.0, 
1.0, 8.0], [6.0, 1.0, 5.0]], [[8.0, 8.0, 3.0], [2.0, 1.0, 2.0], [8.0, 8.0, 7.0]], [[3.0, 1.0, 4.0], [7.0, 2.0, 4.0], [2.0, 8.0, 
8.0]]], [[[7.0, 4.0, 3.0], [6.0, 1.0, 5.0], [7.0, 5.0, 8.0]], [[6.0, 2.0, 3.0], [3.0, 6.0, 8.0], [6.0, 8.0, 4.0]], [[7.0, 1.0, 
7.0], [3.0, 1.0, 4.0], [5.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 5.0], [8.0, 1.0, 7.0], [1.0, 6.0, 6.0]], 
[[4.0, 6.0, 8.0], [3.0, 6.0, 8.0], [7.0, 6.0, 1.0]], [[6.0, 3.0, 8.0], [5.0, 4.0, 8.0], [7.0, 3.0, 4.0]]], [[[7.0, 4.0, 6.0], 
[3.0, 5.0, 1.0], [2.0, 7.0, 4.0]], [[7.0, 3.0, 7.0], [4.0, 3.0, 3.0], [7.0, 1.0, 8.0]], [[3.0, 5.0, 5.0], [7.0, 7.0, 2.0], 
[3.0, 7.0, 6.0]]], [[[8.0, 6.0, 7.0], [3.0, 4.0, 8.0], [2.0, 8.0, 7.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 8.0], [5.0, 3.0, 8.0]], 
[[2.0, 8.0, 2.0], [6.0, 8.0, 6.0], [7.0, 8.0, 2.0]]], [[[3.0, 6.0, 7.0], [6.0, 2.0, 2.0], [2.0, 5.0, 3.0]], [[1.0, 1.0, 6.0], 
[6.0, 1.0, 3.0], [6.0, 4.0, 8.0]], [[3.0, 5.0, 8.0], [8.0, 4.0, 7.0], [5.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PLPL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 3.0], [2.0, 4.0, 6.0], [7.0, 3.0, 1.0]], [[4.0, 2.0, 3.0], [3.0, 2.0, 4.0], [5.0, 
4.0, 4.0]], [[2.0, 2.0, 3.0], [3.0, 8.0, 4.0], [4.0, 1.0, 8.0]]], [[[5.0, 7.0, 7.0], [8.0, 3.0, 1.0], [7.0, 5.0, 2.0]], [[3.0, 
5.0, 8.0], [6.0, 8.0, 2.0], [8.0, 6.0, 5.0]], [[8.0, 4.0, 3.0], [4.0, 6.0, 2.0], [3.0, 1.0, 5.0]]], [[[4.0, 6.0, 8.0], [6.0, 
7.0, 2.0], [1.0, 7.0, 3.0]], [[5.0, 2.0, 7.0], [8.0, 2.0, 8.0], [6.0, 7.0, 7.0]], [[3.0, 3.0, 5.0], [5.0, 2.0, 4.0], [8.0, 4.0, 
7.0]]], [[[5.0, 6.0, 3.0], [5.0, 7.0, 8.0], [5.0, 1.0, 2.0]], [[5.0, 2.0, 4.0], [3.0, 1.0, 4.0], [7.0, 4.0, 8.0]], [[7.0, 8.0, 
4.0], [5.0, 1.0, 8.0], [6.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 5.0], [3.0, 4.0, 3.0], [2.0, 4.0, 3.0]], 
[[3.0, 2.0, 3.0], [5.0, 8.0, 1.0], [4.0, 1.0, 3.0]], [[1.0, 4.0, 2.0], [1.0, 3.0, 1.0], [2.0, 2.0, 4.0]]], [[[1.0, 8.0, 2.0], 
[2.0, 3.0, 3.0], [7.0, 1.0, 2.0]], [[1.0, 3.0, 3.0], [8.0, 8.0, 7.0], [8.0, 3.0, 7.0]], [[3.0, 6.0, 2.0], [8.0, 2.0, 8.0], 
[3.0, 8.0, 5.0]]], [[[5.0, 5.0, 4.0], [2.0, 6.0, 1.0], [1.0, 4.0, 4.0]], [[2.0, 7.0, 7.0], [2.0, 5.0, 5.0], [2.0, 2.0, 4.0]], 
[[7.0, 2.0, 2.0], [4.0, 8.0, 7.0], [5.0, 4.0, 1.0]]], [[[3.0, 2.0, 8.0], [1.0, 3.0, 5.0], [8.0, 5.0, 7.0]], [[1.0, 8.0, 6.0], 
[5.0, 2.0, 8.0], [4.0, 3.0, 5.0]], [[3.0, 1.0, 5.0], [4.0, 8.0, 2.0], [6.0, 1.0, 8.0]]]])
      val=Data(numpy.array([[2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0]]))
      arg[2,2:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 3.0], [2.0, 4.0, 6.0], [7.0, 3.0, 1.0]], [[4.0, 2.0, 3.0], [3.0, 2.0, 4.0], [5.0, 
4.0, 4.0]], [[2.0, 2.0, 3.0], [3.0, 8.0, 4.0], [4.0, 1.0, 8.0]]], [[[5.0, 7.0, 7.0], [8.0, 3.0, 1.0], [7.0, 5.0, 2.0]], [[3.0, 
5.0, 8.0], [6.0, 8.0, 2.0], [8.0, 6.0, 5.0]], [[8.0, 4.0, 3.0], [4.0, 6.0, 2.0], [3.0, 1.0, 5.0]]], [[[4.0, 6.0, 8.0], [6.0, 
7.0, 2.0], [1.0, 7.0, 3.0]], [[5.0, 2.0, 7.0], [8.0, 2.0, 8.0], [6.0, 7.0, 7.0]], [[3.0, 3.0, 5.0], [5.0, 2.0, 2.0], [8.0, 4.0, 
7.0]]], [[[5.0, 6.0, 3.0], [5.0, 7.0, 8.0], [5.0, 1.0, 2.0]], [[5.0, 2.0, 4.0], [3.0, 1.0, 4.0], [7.0, 4.0, 8.0]], [[7.0, 8.0, 
4.0], [5.0, 1.0, 8.0], [6.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 5.0], [3.0, 4.0, 3.0], [2.0, 4.0, 3.0]], 
[[3.0, 2.0, 3.0], [5.0, 8.0, 1.0], [4.0, 1.0, 3.0]], [[1.0, 4.0, 2.0], [1.0, 3.0, 1.0], [2.0, 2.0, 4.0]]], [[[1.0, 8.0, 2.0], 
[2.0, 3.0, 3.0], [7.0, 1.0, 2.0]], [[1.0, 3.0, 3.0], [8.0, 8.0, 7.0], [8.0, 3.0, 7.0]], [[3.0, 6.0, 2.0], [8.0, 2.0, 8.0], 
[3.0, 8.0, 5.0]]], [[[5.0, 5.0, 4.0], [2.0, 6.0, 1.0], [1.0, 4.0, 4.0]], [[2.0, 7.0, 7.0], [2.0, 5.0, 5.0], [2.0, 2.0, 4.0]], 
[[7.0, 2.0, 2.0], [4.0, 8.0, 1.0], [5.0, 4.0, 1.0]]], [[[3.0, 2.0, 8.0], [1.0, 3.0, 5.0], [8.0, 5.0, 7.0]], [[1.0, 8.0, 6.0], 
[5.0, 2.0, 8.0], [4.0, 3.0, 5.0]], [[3.0, 1.0, 5.0], [4.0, 8.0, 2.0], [6.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PCLR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 4.0], [6.0, 8.0, 6.0], [8.0, 6.0, 8.0]], [[3.0, 8.0, 4.0], [7.0, 8.0, 7.0], [8.0, 
1.0, 5.0]], [[5.0, 7.0, 4.0], [3.0, 4.0, 6.0], [1.0, 7.0, 6.0]]], [[[3.0, 1.0, 1.0], [7.0, 6.0, 2.0], [2.0, 2.0, 4.0]], [[5.0, 
6.0, 8.0], [8.0, 3.0, 3.0], [5.0, 3.0, 4.0]], [[1.0, 5.0, 2.0], [4.0, 4.0, 1.0], [1.0, 1.0, 7.0]]], [[[3.0, 8.0, 7.0], [3.0, 
8.0, 6.0], [5.0, 1.0, 8.0]], [[3.0, 4.0, 3.0], [6.0, 3.0, 8.0], [2.0, 5.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 1.0, 7.0], [2.0, 8.0, 
5.0]]], [[[2.0, 3.0, 3.0], [2.0, 6.0, 8.0], [1.0, 2.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 1.0, 2.0], [5.0, 2.0, 8.0]], [[1.0, 7.0, 
2.0], [6.0, 5.0, 6.0], [3.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 6.0], [2.0, 8.0, 8.0], [3.0, 4.0, 8.0]], 
[[4.0, 8.0, 5.0], [6.0, 4.0, 8.0], [2.0, 4.0, 2.0]], [[8.0, 5.0, 8.0], [7.0, 7.0, 6.0], [5.0, 4.0, 5.0]]], [[[8.0, 2.0, 4.0], 
[2.0, 6.0, 5.0], [3.0, 5.0, 5.0]], [[6.0, 8.0, 5.0], [8.0, 7.0, 2.0], [8.0, 3.0, 6.0]], [[1.0, 6.0, 4.0], [7.0, 7.0, 1.0], 
[6.0, 5.0, 2.0]]], [[[3.0, 3.0, 1.0], [6.0, 5.0, 3.0], [2.0, 4.0, 6.0]], [[2.0, 8.0, 4.0], [6.0, 8.0, 2.0], [5.0, 8.0, 8.0]], 
[[1.0, 3.0, 6.0], [5.0, 6.0, 3.0], [3.0, 1.0, 2.0]]], [[[1.0, 7.0, 7.0], [3.0, 2.0, 8.0], [7.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], 
[8.0, 5.0, 8.0], [2.0, 4.0, 7.0]], [[1.0, 3.0, 1.0], [5.0, 3.0, 5.0], [3.0, 7.0, 2.0]]]])
      val=Data(numpy.array([[[4.0]], [[1.0]], [[5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0]], [[7.0]], [[7.0]]]))
      arg[2,:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 4.0], [6.0, 8.0, 6.0], [8.0, 6.0, 8.0]], [[3.0, 8.0, 4.0], [7.0, 8.0, 7.0], [8.0, 
1.0, 5.0]], [[5.0, 7.0, 4.0], [3.0, 4.0, 6.0], [1.0, 7.0, 6.0]]], [[[3.0, 1.0, 1.0], [7.0, 6.0, 2.0], [2.0, 2.0, 4.0]], [[5.0, 
6.0, 8.0], [8.0, 3.0, 3.0], [5.0, 3.0, 4.0]], [[1.0, 5.0, 2.0], [4.0, 4.0, 1.0], [1.0, 1.0, 7.0]]], [[[3.0, 8.0, 7.0], [3.0, 
8.0, 6.0], [5.0, 4.0, 8.0]], [[3.0, 4.0, 3.0], [6.0, 3.0, 8.0], [2.0, 1.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 1.0, 7.0], [2.0, 5.0, 
5.0]]], [[[2.0, 3.0, 3.0], [2.0, 6.0, 8.0], [1.0, 2.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 1.0, 2.0], [5.0, 2.0, 8.0]], [[1.0, 7.0, 
2.0], [6.0, 5.0, 6.0], [3.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 6.0], [2.0, 8.0, 8.0], [3.0, 4.0, 8.0]], 
[[4.0, 8.0, 5.0], [6.0, 4.0, 8.0], [2.0, 4.0, 2.0]], [[8.0, 5.0, 8.0], [7.0, 7.0, 6.0], [5.0, 4.0, 5.0]]], [[[8.0, 2.0, 4.0], 
[2.0, 6.0, 5.0], [3.0, 5.0, 5.0]], [[6.0, 8.0, 5.0], [8.0, 7.0, 2.0], [8.0, 3.0, 6.0]], [[1.0, 6.0, 4.0], [7.0, 7.0, 1.0], 
[6.0, 5.0, 2.0]]], [[[3.0, 3.0, 1.0], [6.0, 5.0, 3.0], [2.0, 6.0, 6.0]], [[2.0, 8.0, 4.0], [6.0, 8.0, 2.0], [5.0, 7.0, 8.0]], 
[[1.0, 3.0, 6.0], [5.0, 6.0, 3.0], [3.0, 7.0, 2.0]]], [[[1.0, 7.0, 7.0], [3.0, 2.0, 8.0], [7.0, 2.0, 4.0]], [[4.0, 6.0, 1.0], 
[8.0, 5.0, 8.0], [2.0, 4.0, 7.0]], [[1.0, 3.0, 1.0], [5.0, 3.0, 5.0], [3.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RCRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 6.0], [1.0, 1.0, 7.0], [5.0, 1.0, 6.0]], [[7.0, 4.0, 5.0], [7.0, 2.0, 1.0], [5.0, 
7.0, 2.0]], [[2.0, 4.0, 3.0], [2.0, 3.0, 7.0], [8.0, 8.0, 7.0]]], [[[4.0, 3.0, 6.0], [3.0, 3.0, 3.0], [8.0, 2.0, 1.0]], [[5.0, 
4.0, 4.0], [4.0, 2.0, 8.0], [4.0, 3.0, 7.0]], [[2.0, 2.0, 8.0], [7.0, 4.0, 4.0], [3.0, 4.0, 4.0]]], [[[7.0, 8.0, 4.0], [3.0, 
1.0, 3.0], [6.0, 6.0, 7.0]], [[4.0, 4.0, 6.0], [6.0, 2.0, 4.0], [8.0, 6.0, 1.0]], [[1.0, 3.0, 1.0], [2.0, 7.0, 1.0], [7.0, 8.0, 
8.0]]], [[[8.0, 2.0, 3.0], [8.0, 2.0, 3.0], [1.0, 8.0, 1.0]], [[8.0, 3.0, 8.0], [8.0, 2.0, 7.0], [2.0, 5.0, 6.0]], [[8.0, 2.0, 
8.0], [2.0, 3.0, 8.0], [6.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 3.0], [3.0, 7.0, 5.0], [1.0, 6.0, 5.0]], 
[[4.0, 3.0, 6.0], [7.0, 1.0, 4.0], [1.0, 4.0, 7.0]], [[6.0, 5.0, 7.0], [2.0, 5.0, 3.0], [3.0, 7.0, 6.0]]], [[[3.0, 7.0, 7.0], 
[2.0, 2.0, 6.0], [3.0, 3.0, 5.0]], [[2.0, 3.0, 7.0], [7.0, 4.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 2.0, 1.0], [3.0, 7.0, 2.0], 
[5.0, 6.0, 6.0]]], [[[4.0, 7.0, 2.0], [5.0, 2.0, 4.0], [7.0, 5.0, 5.0]], [[6.0, 5.0, 3.0], [2.0, 8.0, 2.0], [4.0, 3.0, 7.0]], 
[[4.0, 8.0, 1.0], [2.0, 2.0, 3.0], [4.0, 2.0, 6.0]]], [[[2.0, 3.0, 4.0], [4.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[3.0, 1.0, 3.0], 
[7.0, 4.0, 2.0], [8.0, 4.0, 5.0]], [[1.0, 7.0, 1.0], [7.0, 1.0, 4.0], [6.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[[5.0]], [[8.0]], [[3.0]]], [[[8.0]], [[3.0]], [[2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0]], [[4.0]], [[5.0]]], [[[8.0]], [[8.0]], [[7.0]]]]))
      arg[1:3,:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 6.0], [1.0, 1.0, 7.0], [5.0, 1.0, 6.0]], [[7.0, 4.0, 5.0], [7.0, 2.0, 1.0], [5.0, 
7.0, 2.0]], [[2.0, 4.0, 3.0], [2.0, 3.0, 7.0], [8.0, 8.0, 7.0]]], [[[4.0, 3.0, 6.0], [3.0, 5.0, 3.0], [8.0, 2.0, 1.0]], [[5.0, 
4.0, 4.0], [4.0, 8.0, 8.0], [4.0, 3.0, 7.0]], [[2.0, 2.0, 8.0], [7.0, 3.0, 4.0], [3.0, 4.0, 4.0]]], [[[7.0, 8.0, 4.0], [3.0, 
8.0, 3.0], [6.0, 6.0, 7.0]], [[4.0, 4.0, 6.0], [6.0, 3.0, 4.0], [8.0, 6.0, 1.0]], [[1.0, 3.0, 1.0], [2.0, 2.0, 1.0], [7.0, 8.0, 
8.0]]], [[[8.0, 2.0, 3.0], [8.0, 2.0, 3.0], [1.0, 8.0, 1.0]], [[8.0, 3.0, 8.0], [8.0, 2.0, 7.0], [2.0, 5.0, 6.0]], [[8.0, 2.0, 
8.0], [2.0, 3.0, 8.0], [6.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 3.0], [3.0, 7.0, 5.0], [1.0, 6.0, 5.0]], 
[[4.0, 3.0, 6.0], [7.0, 1.0, 4.0], [1.0, 4.0, 7.0]], [[6.0, 5.0, 7.0], [2.0, 5.0, 3.0], [3.0, 7.0, 6.0]]], [[[3.0, 7.0, 7.0], 
[2.0, 5.0, 6.0], [3.0, 3.0, 5.0]], [[2.0, 3.0, 7.0], [7.0, 4.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 2.0, 1.0], [3.0, 5.0, 2.0], 
[5.0, 6.0, 6.0]]], [[[4.0, 7.0, 2.0], [5.0, 8.0, 4.0], [7.0, 5.0, 5.0]], [[6.0, 5.0, 3.0], [2.0, 8.0, 2.0], [4.0, 3.0, 7.0]], 
[[4.0, 8.0, 1.0], [2.0, 7.0, 3.0], [4.0, 2.0, 6.0]]], [[[2.0, 3.0, 4.0], [4.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[3.0, 1.0, 3.0], 
[7.0, 4.0, 2.0], [8.0, 4.0, 5.0]], [[1.0, 7.0, 1.0], [7.0, 1.0, 4.0], [6.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLRL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 7.0], [6.0, 5.0, 3.0], [3.0, 2.0, 6.0]], [[8.0, 8.0, 4.0], [4.0, 3.0, 4.0], [6.0, 
1.0, 4.0]], [[6.0, 3.0, 8.0], [1.0, 7.0, 4.0], [2.0, 1.0, 6.0]]], [[[3.0, 6.0, 3.0], [5.0, 8.0, 4.0], [3.0, 1.0, 6.0]], [[1.0, 
7.0, 1.0], [5.0, 7.0, 8.0], [5.0, 2.0, 6.0]], [[5.0, 3.0, 7.0], [4.0, 7.0, 3.0], [6.0, 1.0, 2.0]]], [[[5.0, 3.0, 8.0], [7.0, 
8.0, 3.0], [6.0, 7.0, 6.0]], [[4.0, 4.0, 4.0], [8.0, 6.0, 5.0], [6.0, 5.0, 4.0]], [[2.0, 8.0, 8.0], [4.0, 5.0, 3.0], [3.0, 5.0, 
2.0]]], [[[1.0, 1.0, 7.0], [5.0, 7.0, 5.0], [5.0, 8.0, 3.0]], [[6.0, 5.0, 5.0], [1.0, 1.0, 2.0], [4.0, 2.0, 4.0]], [[2.0, 5.0, 
2.0], [6.0, 7.0, 2.0], [5.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 7.0], [2.0, 4.0, 4.0], [3.0, 7.0, 1.0]], 
[[6.0, 5.0, 3.0], [2.0, 7.0, 4.0], [3.0, 4.0, 5.0]], [[6.0, 1.0, 8.0], [2.0, 8.0, 4.0], [3.0, 2.0, 1.0]]], [[[4.0, 7.0, 4.0], 
[4.0, 4.0, 4.0], [7.0, 4.0, 4.0]], [[3.0, 2.0, 3.0], [1.0, 2.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 7.0, 2.0], [1.0, 8.0, 7.0], 
[1.0, 2.0, 4.0]]], [[[2.0, 8.0, 7.0], [3.0, 6.0, 5.0], [1.0, 3.0, 8.0]], [[7.0, 3.0, 5.0], [1.0, 7.0, 6.0], [3.0, 3.0, 3.0]], 
[[6.0, 6.0, 3.0], [1.0, 1.0, 6.0], [6.0, 5.0, 7.0]]], [[[6.0, 2.0, 5.0], [5.0, 3.0, 1.0], [1.0, 1.0, 4.0]], [[6.0, 5.0, 2.0], 
[2.0, 6.0, 7.0], [7.0, 1.0, 7.0]], [[7.0, 8.0, 8.0], [6.0, 1.0, 7.0], [2.0, 1.0, 7.0]]]])
      val=Data(numpy.array([[[[5.0]]], [[[4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0]]], [[[7.0]]]]))
      arg[2:,2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 7.0], [6.0, 5.0, 3.0], [3.0, 2.0, 6.0]], [[8.0, 8.0, 4.0], [4.0, 3.0, 4.0], [6.0, 
1.0, 4.0]], [[6.0, 3.0, 8.0], [1.0, 7.0, 4.0], [2.0, 1.0, 6.0]]], [[[3.0, 6.0, 3.0], [5.0, 8.0, 4.0], [3.0, 1.0, 6.0]], [[1.0, 
7.0, 1.0], [5.0, 7.0, 8.0], [5.0, 2.0, 6.0]], [[5.0, 3.0, 7.0], [4.0, 7.0, 3.0], [6.0, 1.0, 2.0]]], [[[5.0, 3.0, 8.0], [7.0, 
8.0, 3.0], [6.0, 7.0, 6.0]], [[4.0, 4.0, 4.0], [8.0, 6.0, 5.0], [6.0, 5.0, 4.0]], [[2.0, 8.0, 8.0], [4.0, 5.0, 5.0], [3.0, 5.0, 
2.0]]], [[[1.0, 1.0, 7.0], [5.0, 7.0, 5.0], [5.0, 8.0, 3.0]], [[6.0, 5.0, 5.0], [1.0, 1.0, 2.0], [4.0, 2.0, 4.0]], [[2.0, 5.0, 
2.0], [6.0, 7.0, 4.0], [5.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 7.0], [2.0, 4.0, 4.0], [3.0, 7.0, 1.0]], 
[[6.0, 5.0, 3.0], [2.0, 7.0, 4.0], [3.0, 4.0, 5.0]], [[6.0, 1.0, 8.0], [2.0, 8.0, 4.0], [3.0, 2.0, 1.0]]], [[[4.0, 7.0, 4.0], 
[4.0, 4.0, 4.0], [7.0, 4.0, 4.0]], [[3.0, 2.0, 3.0], [1.0, 2.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 7.0, 2.0], [1.0, 8.0, 7.0], 
[1.0, 2.0, 4.0]]], [[[2.0, 8.0, 7.0], [3.0, 6.0, 5.0], [1.0, 3.0, 8.0]], [[7.0, 3.0, 5.0], [1.0, 7.0, 6.0], [3.0, 3.0, 3.0]], 
[[6.0, 6.0, 3.0], [1.0, 1.0, 4.0], [6.0, 5.0, 7.0]]], [[[6.0, 2.0, 5.0], [5.0, 3.0, 1.0], [1.0, 1.0, 4.0]], [[6.0, 5.0, 2.0], 
[2.0, 6.0, 7.0], [7.0, 1.0, 7.0]], [[7.0, 8.0, 8.0], [6.0, 1.0, 7.0], [2.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CRCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 7.0], [6.0, 1.0, 7.0], [2.0, 8.0, 4.0]], [[8.0, 1.0, 4.0], [3.0, 8.0, 1.0], [3.0, 
6.0, 6.0]], [[3.0, 4.0, 7.0], [7.0, 7.0, 4.0], [8.0, 2.0, 8.0]]], [[[2.0, 5.0, 7.0], [5.0, 1.0, 3.0], [1.0, 3.0, 6.0]], [[5.0, 
1.0, 2.0], [5.0, 7.0, 7.0], [7.0, 1.0, 7.0]], [[1.0, 2.0, 3.0], [5.0, 1.0, 5.0], [3.0, 8.0, 5.0]]], [[[4.0, 1.0, 2.0], [5.0, 
1.0, 6.0], [6.0, 7.0, 2.0]], [[2.0, 7.0, 3.0], [5.0, 3.0, 1.0], [4.0, 5.0, 5.0]], [[4.0, 7.0, 3.0], [8.0, 1.0, 8.0], [8.0, 6.0, 
8.0]]], [[[6.0, 3.0, 5.0], [5.0, 7.0, 8.0], [3.0, 8.0, 1.0]], [[8.0, 2.0, 2.0], [4.0, 7.0, 4.0], [8.0, 1.0, 1.0]], [[4.0, 7.0, 
1.0], [8.0, 1.0, 7.0], [1.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 4.0], [4.0, 3.0, 6.0], [4.0, 7.0, 4.0]], 
[[5.0, 8.0, 4.0], [3.0, 2.0, 1.0], [2.0, 3.0, 1.0]], [[5.0, 7.0, 8.0], [4.0, 3.0, 8.0], [1.0, 7.0, 5.0]]], [[[8.0, 5.0, 6.0], 
[7.0, 7.0, 8.0], [8.0, 5.0, 6.0]], [[7.0, 3.0, 1.0], [4.0, 3.0, 1.0], [8.0, 4.0, 1.0]], [[3.0, 5.0, 1.0], [4.0, 5.0, 5.0], 
[5.0, 6.0, 1.0]]], [[[2.0, 4.0, 3.0], [5.0, 6.0, 2.0], [5.0, 7.0, 3.0]], [[4.0, 2.0, 8.0], [3.0, 8.0, 4.0], [4.0, 8.0, 4.0]], 
[[5.0, 4.0, 5.0], [3.0, 6.0, 8.0], [3.0, 6.0, 3.0]]], [[[6.0, 2.0, 6.0], [1.0, 4.0, 8.0], [2.0, 8.0, 6.0]], [[5.0, 4.0, 6.0], 
[6.0, 4.0, 3.0], [8.0, 5.0, 1.0]], [[5.0, 2.0, 8.0], [5.0, 7.0, 1.0], [3.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[[5.0, 6.0, 7.0], [3.0, 5.0, 1.0], [3.0, 5.0, 3.0]]], [[[6.0, 6.0, 5.0], [3.0, 4.0, 6.0], [2.0, 
7.0, 4.0]]], [[[2.0, 1.0, 3.0], [4.0, 4.0, 6.0], [8.0, 7.0, 6.0]]], [[[6.0, 6.0, 7.0], [6.0, 4.0, 6.0], [3.0, 8.0, 
4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 2.0, 7.0], [1.0, 3.0, 1.0], [3.0, 1.0, 7.0]]], [[[5.0, 2.0, 5.0], [4.0, 8.0, 
8.0], [2.0, 1.0, 7.0]]], [[[3.0, 2.0, 1.0], [1.0, 5.0, 1.0], [2.0, 6.0, 2.0]]], [[[7.0, 4.0, 5.0], [1.0, 7.0, 3.0], [8.0, 3.0, 
7.0]]]]))
      arg[:,1:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 7.0], [6.0, 1.0, 7.0], [2.0, 8.0, 4.0]], [[5.0, 6.0, 7.0], [3.0, 5.0, 1.0], [3.0, 
5.0, 3.0]], [[3.0, 4.0, 7.0], [7.0, 7.0, 4.0], [8.0, 2.0, 8.0]]], [[[2.0, 5.0, 7.0], [5.0, 1.0, 3.0], [1.0, 3.0, 6.0]], [[6.0, 
6.0, 5.0], [3.0, 4.0, 6.0], [2.0, 7.0, 4.0]], [[1.0, 2.0, 3.0], [5.0, 1.0, 5.0], [3.0, 8.0, 5.0]]], [[[4.0, 1.0, 2.0], [5.0, 
1.0, 6.0], [6.0, 7.0, 2.0]], [[2.0, 1.0, 3.0], [4.0, 4.0, 6.0], [8.0, 7.0, 6.0]], [[4.0, 7.0, 3.0], [8.0, 1.0, 8.0], [8.0, 6.0, 
8.0]]], [[[6.0, 3.0, 5.0], [5.0, 7.0, 8.0], [3.0, 8.0, 1.0]], [[6.0, 6.0, 7.0], [6.0, 4.0, 6.0], [3.0, 8.0, 4.0]], [[4.0, 7.0, 
1.0], [8.0, 1.0, 7.0], [1.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 4.0], [4.0, 3.0, 6.0], [4.0, 7.0, 4.0]], 
[[7.0, 2.0, 7.0], [1.0, 3.0, 1.0], [3.0, 1.0, 7.0]], [[5.0, 7.0, 8.0], [4.0, 3.0, 8.0], [1.0, 7.0, 5.0]]], [[[8.0, 5.0, 6.0], 
[7.0, 7.0, 8.0], [8.0, 5.0, 6.0]], [[5.0, 2.0, 5.0], [4.0, 8.0, 8.0], [2.0, 1.0, 7.0]], [[3.0, 5.0, 1.0], [4.0, 5.0, 5.0], 
[5.0, 6.0, 1.0]]], [[[2.0, 4.0, 3.0], [5.0, 6.0, 2.0], [5.0, 7.0, 3.0]], [[3.0, 2.0, 1.0], [1.0, 5.0, 1.0], [2.0, 6.0, 2.0]], 
[[5.0, 4.0, 5.0], [3.0, 6.0, 8.0], [3.0, 6.0, 3.0]]], [[[6.0, 2.0, 6.0], [1.0, 4.0, 8.0], [2.0, 8.0, 6.0]], [[7.0, 4.0, 5.0], 
[1.0, 7.0, 3.0], [8.0, 3.0, 7.0]], [[5.0, 2.0, 8.0], [5.0, 7.0, 1.0], [3.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RUUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 6.0], [1.0, 4.0, 2.0], [3.0, 1.0, 7.0]], [[5.0, 6.0, 6.0], [3.0, 4.0, 6.0], [5.0, 
2.0, 6.0]], [[3.0, 5.0, 8.0], [7.0, 8.0, 5.0], [4.0, 7.0, 6.0]]], [[[7.0, 5.0, 6.0], [1.0, 8.0, 8.0], [3.0, 5.0, 5.0]], [[8.0, 
8.0, 4.0], [7.0, 2.0, 7.0], [3.0, 4.0, 2.0]], [[3.0, 1.0, 1.0], [4.0, 6.0, 5.0], [4.0, 1.0, 6.0]]], [[[3.0, 2.0, 7.0], [2.0, 
5.0, 7.0], [7.0, 3.0, 3.0]], [[1.0, 6.0, 2.0], [3.0, 8.0, 1.0], [3.0, 7.0, 3.0]], [[7.0, 4.0, 7.0], [7.0, 6.0, 2.0], [4.0, 4.0, 
8.0]]], [[[4.0, 4.0, 7.0], [6.0, 6.0, 5.0], [4.0, 2.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 4.0, 7.0], [5.0, 8.0, 1.0]], [[8.0, 4.0, 
4.0], [7.0, 6.0, 2.0], [7.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 3.0], [1.0, 5.0, 6.0], [3.0, 5.0, 3.0]], 
[[7.0, 4.0, 7.0], [1.0, 2.0, 8.0], [4.0, 2.0, 6.0]], [[4.0, 8.0, 2.0], [3.0, 8.0, 7.0], [3.0, 4.0, 1.0]]], [[[1.0, 2.0, 3.0], 
[1.0, 8.0, 8.0], [8.0, 8.0, 1.0]], [[3.0, 7.0, 7.0], [8.0, 4.0, 7.0], [5.0, 2.0, 6.0]], [[2.0, 8.0, 7.0], [4.0, 3.0, 6.0], 
[5.0, 2.0, 5.0]]], [[[5.0, 6.0, 5.0], [6.0, 3.0, 6.0], [7.0, 6.0, 5.0]], [[3.0, 3.0, 6.0], [4.0, 2.0, 7.0], [3.0, 6.0, 1.0]], 
[[8.0, 8.0, 4.0], [4.0, 5.0, 4.0], [8.0, 1.0, 3.0]]], [[[2.0, 1.0, 8.0], [1.0, 8.0, 2.0], [3.0, 7.0, 8.0]], [[7.0, 4.0, 2.0], 
[8.0, 2.0, 2.0], [2.0, 5.0, 5.0]], [[3.0, 6.0, 2.0], [6.0, 4.0, 3.0], [7.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[[7.0, 6.0, 5.0], [2.0, 3.0, 1.0]], [[4.0, 5.0, 7.0], [7.0, 4.0, 5.0]]], [[[7.0, 2.0, 8.0], [1.0, 
7.0, 4.0]], [[5.0, 8.0, 6.0], [6.0, 3.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 2.0], [7.0, 5.0, 3.0]], [[6.0, 1.0, 7.0], [8.0, 2.0, 2.0]]], [[[2.0, 5.0, 
8.0], [3.0, 4.0, 4.0]], [[7.0, 1.0, 6.0], [7.0, 8.0, 2.0]]]]))
      arg[1:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 6.0], [1.0, 4.0, 2.0], [3.0, 1.0, 7.0]], [[5.0, 6.0, 6.0], [3.0, 4.0, 6.0], [5.0, 
2.0, 6.0]], [[3.0, 5.0, 8.0], [7.0, 8.0, 5.0], [4.0, 7.0, 6.0]]], [[[7.0, 6.0, 5.0], [2.0, 3.0, 1.0], [3.0, 5.0, 5.0]], [[4.0, 
5.0, 7.0], [7.0, 4.0, 5.0], [3.0, 4.0, 2.0]], [[3.0, 1.0, 1.0], [4.0, 6.0, 5.0], [4.0, 1.0, 6.0]]], [[[7.0, 2.0, 8.0], [1.0, 
7.0, 4.0], [7.0, 3.0, 3.0]], [[5.0, 8.0, 6.0], [6.0, 3.0, 8.0], [3.0, 7.0, 3.0]], [[7.0, 4.0, 7.0], [7.0, 6.0, 2.0], [4.0, 4.0, 
8.0]]], [[[4.0, 4.0, 7.0], [6.0, 6.0, 5.0], [4.0, 2.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 4.0, 7.0], [5.0, 8.0, 1.0]], [[8.0, 4.0, 
4.0], [7.0, 6.0, 2.0], [7.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 3.0], [1.0, 5.0, 6.0], [3.0, 5.0, 3.0]], 
[[7.0, 4.0, 7.0], [1.0, 2.0, 8.0], [4.0, 2.0, 6.0]], [[4.0, 8.0, 2.0], [3.0, 8.0, 7.0], [3.0, 4.0, 1.0]]], [[[7.0, 4.0, 2.0], 
[7.0, 5.0, 3.0], [8.0, 8.0, 1.0]], [[6.0, 1.0, 7.0], [8.0, 2.0, 2.0], [5.0, 2.0, 6.0]], [[2.0, 8.0, 7.0], [4.0, 3.0, 6.0], 
[5.0, 2.0, 5.0]]], [[[2.0, 5.0, 8.0], [3.0, 4.0, 4.0], [7.0, 6.0, 5.0]], [[7.0, 1.0, 6.0], [7.0, 8.0, 2.0], [3.0, 6.0, 1.0]], 
[[8.0, 8.0, 4.0], [4.0, 5.0, 4.0], [8.0, 1.0, 3.0]]], [[[2.0, 1.0, 8.0], [1.0, 8.0, 2.0], [3.0, 7.0, 8.0]], [[7.0, 4.0, 2.0], 
[8.0, 2.0, 2.0], [2.0, 5.0, 5.0]], [[3.0, 6.0, 2.0], [6.0, 4.0, 3.0], [7.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URLC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 5.0], [3.0, 5.0, 7.0], [7.0, 2.0, 1.0]], [[8.0, 3.0, 8.0], [7.0, 5.0, 2.0], [6.0, 
4.0, 7.0]], [[2.0, 7.0, 6.0], [5.0, 2.0, 8.0], [2.0, 1.0, 6.0]]], [[[6.0, 4.0, 4.0], [6.0, 3.0, 2.0], [6.0, 7.0, 7.0]], [[7.0, 
1.0, 1.0], [2.0, 8.0, 7.0], [2.0, 3.0, 3.0]], [[7.0, 8.0, 6.0], [3.0, 7.0, 5.0], [5.0, 2.0, 7.0]]], [[[6.0, 4.0, 2.0], [4.0, 
1.0, 6.0], [2.0, 1.0, 4.0]], [[7.0, 7.0, 6.0], [1.0, 8.0, 5.0], [8.0, 7.0, 3.0]], [[4.0, 8.0, 2.0], [7.0, 6.0, 6.0], [6.0, 1.0, 
2.0]]], [[[1.0, 1.0, 7.0], [8.0, 6.0, 7.0], [6.0, 7.0, 3.0]], [[5.0, 7.0, 2.0], [8.0, 1.0, 8.0], [8.0, 5.0, 5.0]], [[4.0, 2.0, 
3.0], [8.0, 7.0, 5.0], [5.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 6.0], [5.0, 1.0, 5.0], [6.0, 5.0, 7.0]], 
[[8.0, 2.0, 5.0], [5.0, 7.0, 1.0], [8.0, 7.0, 6.0]], [[8.0, 1.0, 4.0], [8.0, 1.0, 7.0], [2.0, 3.0, 1.0]]], [[[4.0, 2.0, 1.0], 
[3.0, 2.0, 8.0], [2.0, 7.0, 4.0]], [[8.0, 7.0, 4.0], [8.0, 3.0, 5.0], [3.0, 6.0, 1.0]], [[7.0, 7.0, 5.0], [6.0, 2.0, 7.0], 
[4.0, 2.0, 8.0]]], [[[5.0, 1.0, 8.0], [5.0, 1.0, 1.0], [5.0, 6.0, 1.0]], [[7.0, 5.0, 5.0], [3.0, 7.0, 4.0], [6.0, 3.0, 5.0]], 
[[7.0, 5.0, 7.0], [4.0, 7.0, 6.0], [5.0, 5.0, 3.0]]], [[[7.0, 2.0, 7.0], [2.0, 8.0, 6.0], [5.0, 8.0, 2.0]], [[6.0, 1.0, 5.0], 
[6.0, 7.0, 8.0], [3.0, 5.0, 2.0]], [[3.0, 8.0, 5.0], [7.0, 4.0, 1.0], [3.0, 1.0, 5.0]]]])
      val=Data(numpy.array([[[[7.0, 8.0, 8.0]]], [[[4.0, 5.0, 7.0]]], [[[8.0, 1.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 2.0, 8.0]]], [[[3.0, 1.0, 4.0]]], [[[6.0, 1.0, 5.0]]]]))
      arg[:3,1:2,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 5.0], [3.0, 5.0, 7.0], [7.0, 2.0, 1.0]], [[8.0, 3.0, 8.0], [7.0, 5.0, 2.0], [7.0, 
8.0, 8.0]], [[2.0, 7.0, 6.0], [5.0, 2.0, 8.0], [2.0, 1.0, 6.0]]], [[[6.0, 4.0, 4.0], [6.0, 3.0, 2.0], [6.0, 7.0, 7.0]], [[7.0, 
1.0, 1.0], [2.0, 8.0, 7.0], [4.0, 5.0, 7.0]], [[7.0, 8.0, 6.0], [3.0, 7.0, 5.0], [5.0, 2.0, 7.0]]], [[[6.0, 4.0, 2.0], [4.0, 
1.0, 6.0], [2.0, 1.0, 4.0]], [[7.0, 7.0, 6.0], [1.0, 8.0, 5.0], [8.0, 1.0, 8.0]], [[4.0, 8.0, 2.0], [7.0, 6.0, 6.0], [6.0, 1.0, 
2.0]]], [[[1.0, 1.0, 7.0], [8.0, 6.0, 7.0], [6.0, 7.0, 3.0]], [[5.0, 7.0, 2.0], [8.0, 1.0, 8.0], [8.0, 5.0, 5.0]], [[4.0, 2.0, 
3.0], [8.0, 7.0, 5.0], [5.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 6.0], [5.0, 1.0, 5.0], [6.0, 5.0, 7.0]], 
[[8.0, 2.0, 5.0], [5.0, 7.0, 1.0], [3.0, 2.0, 8.0]], [[8.0, 1.0, 4.0], [8.0, 1.0, 7.0], [2.0, 3.0, 1.0]]], [[[4.0, 2.0, 1.0], 
[3.0, 2.0, 8.0], [2.0, 7.0, 4.0]], [[8.0, 7.0, 4.0], [8.0, 3.0, 5.0], [3.0, 1.0, 4.0]], [[7.0, 7.0, 5.0], [6.0, 2.0, 7.0], 
[4.0, 2.0, 8.0]]], [[[5.0, 1.0, 8.0], [5.0, 1.0, 1.0], [5.0, 6.0, 1.0]], [[7.0, 5.0, 5.0], [3.0, 7.0, 4.0], [6.0, 1.0, 5.0]], 
[[7.0, 5.0, 7.0], [4.0, 7.0, 6.0], [5.0, 5.0, 3.0]]], [[[7.0, 2.0, 7.0], [2.0, 8.0, 6.0], [5.0, 8.0, 2.0]], [[6.0, 1.0, 5.0], 
[6.0, 7.0, 8.0], [3.0, 5.0, 2.0]], [[3.0, 8.0, 5.0], [7.0, 4.0, 1.0], [3.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULLR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 1.0], [1.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[6.0, 3.0, 6.0], [3.0, 2.0, 3.0], [1.0, 
2.0, 7.0]], [[4.0, 3.0, 7.0], [1.0, 8.0, 1.0], [4.0, 2.0, 8.0]]], [[[3.0, 8.0, 4.0], [1.0, 4.0, 5.0], [3.0, 2.0, 2.0]], [[6.0, 
2.0, 5.0], [6.0, 3.0, 3.0], [4.0, 4.0, 1.0]], [[7.0, 2.0, 8.0], [4.0, 5.0, 4.0], [8.0, 7.0, 8.0]]], [[[4.0, 4.0, 4.0], [7.0, 
1.0, 7.0], [6.0, 1.0, 2.0]], [[5.0, 4.0, 1.0], [6.0, 6.0, 1.0], [5.0, 4.0, 2.0]], [[7.0, 4.0, 4.0], [6.0, 6.0, 5.0], [1.0, 1.0, 
7.0]]], [[[7.0, 3.0, 5.0], [1.0, 7.0, 1.0], [4.0, 1.0, 3.0]], [[4.0, 7.0, 7.0], [8.0, 3.0, 8.0], [2.0, 6.0, 5.0]], [[2.0, 6.0, 
6.0], [3.0, 7.0, 6.0], [2.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 4.0], [7.0, 1.0, 8.0], [8.0, 4.0, 8.0]], 
[[5.0, 2.0, 2.0], [8.0, 5.0, 2.0], [6.0, 5.0, 3.0]], [[8.0, 6.0, 1.0], [5.0, 5.0, 3.0], [4.0, 6.0, 8.0]]], [[[8.0, 8.0, 8.0], 
[1.0, 8.0, 5.0], [2.0, 7.0, 8.0]], [[7.0, 2.0, 1.0], [4.0, 7.0, 6.0], [8.0, 1.0, 4.0]], [[2.0, 8.0, 1.0], [1.0, 7.0, 4.0], 
[7.0, 4.0, 2.0]]], [[[1.0, 5.0, 1.0], [3.0, 4.0, 4.0], [1.0, 4.0, 5.0]], [[8.0, 8.0, 4.0], [5.0, 7.0, 3.0], [1.0, 7.0, 8.0]], 
[[1.0, 4.0, 8.0], [7.0, 4.0, 7.0], [5.0, 7.0, 8.0]]], [[[1.0, 7.0, 8.0], [7.0, 6.0, 1.0], [2.0, 5.0, 2.0]], [[4.0, 1.0, 8.0], 
[2.0, 8.0, 4.0], [1.0, 2.0, 2.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 4.0], [2.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[[[8.0]]], [[[8.0]]], [[[5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0]]], [[[7.0]]], [[[8.0]]]]))
      arg[:3,2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 1.0], [1.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[6.0, 3.0, 6.0], [3.0, 2.0, 3.0], [1.0, 
2.0, 7.0]], [[4.0, 3.0, 7.0], [1.0, 8.0, 1.0], [4.0, 8.0, 8.0]]], [[[3.0, 8.0, 4.0], [1.0, 4.0, 5.0], [3.0, 2.0, 2.0]], [[6.0, 
2.0, 5.0], [6.0, 3.0, 3.0], [4.0, 4.0, 1.0]], [[7.0, 2.0, 8.0], [4.0, 5.0, 4.0], [8.0, 8.0, 8.0]]], [[[4.0, 4.0, 4.0], [7.0, 
1.0, 7.0], [6.0, 1.0, 2.0]], [[5.0, 4.0, 1.0], [6.0, 6.0, 1.0], [5.0, 4.0, 2.0]], [[7.0, 4.0, 4.0], [6.0, 6.0, 5.0], [1.0, 5.0, 
7.0]]], [[[7.0, 3.0, 5.0], [1.0, 7.0, 1.0], [4.0, 1.0, 3.0]], [[4.0, 7.0, 7.0], [8.0, 3.0, 8.0], [2.0, 6.0, 5.0]], [[2.0, 6.0, 
6.0], [3.0, 7.0, 6.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 4.0], [7.0, 1.0, 8.0], [8.0, 4.0, 8.0]], 
[[5.0, 2.0, 2.0], [8.0, 5.0, 2.0], [6.0, 5.0, 3.0]], [[8.0, 6.0, 1.0], [5.0, 5.0, 3.0], [4.0, 3.0, 8.0]]], [[[8.0, 8.0, 8.0], 
[1.0, 8.0, 5.0], [2.0, 7.0, 8.0]], [[7.0, 2.0, 1.0], [4.0, 7.0, 6.0], [8.0, 1.0, 4.0]], [[2.0, 8.0, 1.0], [1.0, 7.0, 4.0], 
[7.0, 7.0, 2.0]]], [[[1.0, 5.0, 1.0], [3.0, 4.0, 4.0], [1.0, 4.0, 5.0]], [[8.0, 8.0, 4.0], [5.0, 7.0, 3.0], [1.0, 7.0, 8.0]], 
[[1.0, 4.0, 8.0], [7.0, 4.0, 7.0], [5.0, 8.0, 8.0]]], [[[1.0, 7.0, 8.0], [7.0, 6.0, 1.0], [2.0, 5.0, 2.0]], [[4.0, 1.0, 8.0], 
[2.0, 8.0, 4.0], [1.0, 2.0, 2.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 4.0], [2.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PUCP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 4.0], [7.0, 4.0, 6.0], [1.0, 3.0, 4.0]], [[1.0, 7.0, 3.0], [3.0, 4.0, 4.0], [6.0, 
4.0, 4.0]], [[4.0, 1.0, 5.0], [5.0, 8.0, 2.0], [1.0, 1.0, 1.0]]], [[[5.0, 6.0, 6.0], [8.0, 6.0, 6.0], [8.0, 3.0, 8.0]], [[3.0, 
7.0, 6.0], [7.0, 3.0, 6.0], [8.0, 2.0, 6.0]], [[8.0, 7.0, 6.0], [1.0, 5.0, 1.0], [6.0, 5.0, 5.0]]], [[[8.0, 3.0, 7.0], [8.0, 
7.0, 2.0], [7.0, 2.0, 7.0]], [[7.0, 3.0, 8.0], [4.0, 2.0, 2.0], [4.0, 3.0, 8.0]], [[5.0, 8.0, 4.0], [6.0, 1.0, 5.0], [2.0, 8.0, 
7.0]]], [[[7.0, 1.0, 4.0], [3.0, 3.0, 7.0], [5.0, 2.0, 2.0]], [[8.0, 6.0, 4.0], [1.0, 7.0, 3.0], [7.0, 6.0, 3.0]], [[8.0, 2.0, 
6.0], [6.0, 4.0, 6.0], [4.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 7.0], [1.0, 7.0, 5.0], [4.0, 7.0, 5.0]], 
[[4.0, 5.0, 1.0], [5.0, 7.0, 4.0], [6.0, 8.0, 4.0]], [[3.0, 5.0, 7.0], [1.0, 8.0, 1.0], [1.0, 8.0, 6.0]]], [[[5.0, 6.0, 6.0], 
[3.0, 6.0, 3.0], [8.0, 1.0, 6.0]], [[4.0, 8.0, 4.0], [2.0, 6.0, 4.0], [5.0, 5.0, 1.0]], [[5.0, 3.0, 5.0], [2.0, 4.0, 8.0], 
[6.0, 1.0, 8.0]]], [[[1.0, 6.0, 5.0], [7.0, 3.0, 1.0], [8.0, 8.0, 8.0]], [[2.0, 3.0, 3.0], [7.0, 2.0, 7.0], [4.0, 8.0, 2.0]], 
[[4.0, 3.0, 4.0], [6.0, 6.0, 6.0], [4.0, 8.0, 2.0]]], [[[4.0, 8.0, 7.0], [3.0, 3.0, 5.0], [1.0, 7.0, 6.0]], [[4.0, 6.0, 4.0], 
[4.0, 7.0, 5.0], [3.0, 6.0, 6.0]], [[8.0, 1.0, 6.0], [1.0, 8.0, 5.0], [1.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[1.0, 8.0, 7.0], [3.0, 6.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 2.0, 6.0], [7.0, 1.0, 7.0]]))
      arg[2,:2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 4.0], [7.0, 4.0, 6.0], [1.0, 3.0, 4.0]], [[1.0, 7.0, 3.0], [3.0, 4.0, 4.0], [6.0, 
4.0, 4.0]], [[4.0, 1.0, 5.0], [5.0, 8.0, 2.0], [1.0, 1.0, 1.0]]], [[[5.0, 6.0, 6.0], [8.0, 6.0, 6.0], [8.0, 3.0, 8.0]], [[3.0, 
7.0, 6.0], [7.0, 3.0, 6.0], [8.0, 2.0, 6.0]], [[8.0, 7.0, 6.0], [1.0, 5.0, 1.0], [6.0, 5.0, 5.0]]], [[[8.0, 1.0, 7.0], [8.0, 
8.0, 2.0], [7.0, 7.0, 7.0]], [[7.0, 3.0, 8.0], [4.0, 6.0, 2.0], [4.0, 2.0, 8.0]], [[5.0, 8.0, 4.0], [6.0, 1.0, 5.0], [2.0, 8.0, 
7.0]]], [[[7.0, 1.0, 4.0], [3.0, 3.0, 7.0], [5.0, 2.0, 2.0]], [[8.0, 6.0, 4.0], [1.0, 7.0, 3.0], [7.0, 6.0, 3.0]], [[8.0, 2.0, 
6.0], [6.0, 4.0, 6.0], [4.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 7.0], [1.0, 7.0, 5.0], [4.0, 7.0, 5.0]], 
[[4.0, 5.0, 1.0], [5.0, 7.0, 4.0], [6.0, 8.0, 4.0]], [[3.0, 5.0, 7.0], [1.0, 8.0, 1.0], [1.0, 8.0, 6.0]]], [[[5.0, 6.0, 6.0], 
[3.0, 6.0, 3.0], [8.0, 1.0, 6.0]], [[4.0, 8.0, 4.0], [2.0, 6.0, 4.0], [5.0, 5.0, 1.0]], [[5.0, 3.0, 5.0], [2.0, 4.0, 8.0], 
[6.0, 1.0, 8.0]]], [[[1.0, 3.0, 5.0], [7.0, 2.0, 1.0], [8.0, 6.0, 8.0]], [[2.0, 7.0, 3.0], [7.0, 1.0, 7.0], [4.0, 7.0, 2.0]], 
[[4.0, 3.0, 4.0], [6.0, 6.0, 6.0], [4.0, 8.0, 2.0]]], [[[4.0, 8.0, 7.0], [3.0, 3.0, 5.0], [1.0, 7.0, 6.0]], [[4.0, 6.0, 4.0], 
[4.0, 7.0, 5.0], [3.0, 6.0, 6.0]], [[8.0, 1.0, 6.0], [1.0, 8.0, 5.0], [1.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PURN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 7.0], [8.0, 5.0, 6.0], [7.0, 2.0, 8.0]], [[3.0, 2.0, 2.0], [8.0, 1.0, 4.0], [8.0, 
5.0, 1.0]], [[2.0, 3.0, 8.0], [5.0, 8.0, 7.0], [1.0, 3.0, 5.0]]], [[[7.0, 2.0, 3.0], [5.0, 7.0, 7.0], [1.0, 1.0, 8.0]], [[2.0, 
8.0, 8.0], [6.0, 8.0, 7.0], [4.0, 4.0, 5.0]], [[4.0, 6.0, 4.0], [4.0, 5.0, 2.0], [6.0, 8.0, 1.0]]], [[[2.0, 2.0, 2.0], [4.0, 
6.0, 5.0], [6.0, 7.0, 7.0]], [[8.0, 8.0, 8.0], [2.0, 1.0, 6.0], [5.0, 6.0, 6.0]], [[7.0, 6.0, 7.0], [5.0, 6.0, 2.0], [7.0, 5.0, 
8.0]]], [[[1.0, 5.0, 8.0], [6.0, 1.0, 5.0], [1.0, 5.0, 4.0]], [[7.0, 3.0, 2.0], [7.0, 6.0, 7.0], [7.0, 6.0, 3.0]], [[6.0, 4.0, 
3.0], [7.0, 8.0, 3.0], [3.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 4.0], [6.0, 7.0, 2.0], [1.0, 6.0, 4.0]], 
[[8.0, 7.0, 3.0], [7.0, 5.0, 2.0], [6.0, 4.0, 8.0]], [[7.0, 3.0, 6.0], [1.0, 1.0, 5.0], [7.0, 1.0, 1.0]]], [[[4.0, 4.0, 7.0], 
[3.0, 5.0, 6.0], [4.0, 4.0, 1.0]], [[2.0, 7.0, 2.0], [1.0, 1.0, 1.0], [4.0, 4.0, 7.0]], [[4.0, 4.0, 8.0], [2.0, 6.0, 2.0], 
[6.0, 6.0, 7.0]]], [[[3.0, 7.0, 5.0], [8.0, 2.0, 4.0], [3.0, 2.0, 1.0]], [[5.0, 2.0, 4.0], [1.0, 4.0, 4.0], [6.0, 4.0, 6.0]], 
[[2.0, 8.0, 2.0], [6.0, 7.0, 4.0], [7.0, 7.0, 4.0]]], [[[7.0, 5.0, 5.0], [2.0, 2.0, 2.0], [5.0, 7.0, 3.0]], [[4.0, 1.0, 4.0], 
[1.0, 2.0, 1.0], [6.0, 2.0, 5.0]], [[1.0, 6.0, 7.0], [2.0, 4.0, 5.0], [8.0, 6.0, 3.0]]]])
      val=Data(numpy.array([[[5.0, 4.0, 5.0]], [[5.0, 5.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 7.0, 6.0]], [[4.0, 8.0, 7.0]]]))
      arg[2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 7.0], [8.0, 5.0, 6.0], [7.0, 2.0, 8.0]], [[3.0, 2.0, 2.0], [8.0, 1.0, 4.0], [8.0, 
5.0, 1.0]], [[2.0, 3.0, 8.0], [5.0, 8.0, 7.0], [1.0, 3.0, 5.0]]], [[[7.0, 2.0, 3.0], [5.0, 7.0, 7.0], [1.0, 1.0, 8.0]], [[2.0, 
8.0, 8.0], [6.0, 8.0, 7.0], [4.0, 4.0, 5.0]], [[4.0, 6.0, 4.0], [4.0, 5.0, 2.0], [6.0, 8.0, 1.0]]], [[[2.0, 2.0, 2.0], [5.0, 
4.0, 5.0], [6.0, 7.0, 7.0]], [[8.0, 8.0, 8.0], [5.0, 5.0, 8.0], [5.0, 6.0, 6.0]], [[7.0, 6.0, 7.0], [5.0, 6.0, 2.0], [7.0, 5.0, 
8.0]]], [[[1.0, 5.0, 8.0], [6.0, 1.0, 5.0], [1.0, 5.0, 4.0]], [[7.0, 3.0, 2.0], [7.0, 6.0, 7.0], [7.0, 6.0, 3.0]], [[6.0, 4.0, 
3.0], [7.0, 8.0, 3.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 4.0], [6.0, 7.0, 2.0], [1.0, 6.0, 4.0]], 
[[8.0, 7.0, 3.0], [7.0, 5.0, 2.0], [6.0, 4.0, 8.0]], [[7.0, 3.0, 6.0], [1.0, 1.0, 5.0], [7.0, 1.0, 1.0]]], [[[4.0, 4.0, 7.0], 
[3.0, 5.0, 6.0], [4.0, 4.0, 1.0]], [[2.0, 7.0, 2.0], [1.0, 1.0, 1.0], [4.0, 4.0, 7.0]], [[4.0, 4.0, 8.0], [2.0, 6.0, 2.0], 
[6.0, 6.0, 7.0]]], [[[3.0, 7.0, 5.0], [7.0, 7.0, 6.0], [3.0, 2.0, 1.0]], [[5.0, 2.0, 4.0], [4.0, 8.0, 7.0], [6.0, 4.0, 6.0]], 
[[2.0, 8.0, 2.0], [6.0, 7.0, 4.0], [7.0, 7.0, 4.0]]], [[[7.0, 5.0, 5.0], [2.0, 2.0, 2.0], [5.0, 7.0, 3.0]], [[4.0, 1.0, 4.0], 
[1.0, 2.0, 1.0], [6.0, 2.0, 5.0]], [[1.0, 6.0, 7.0], [2.0, 4.0, 5.0], [8.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LPRL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 5.0], [7.0, 1.0, 7.0], [5.0, 4.0, 2.0]], [[1.0, 8.0, 1.0], [3.0, 4.0, 5.0], [4.0, 
3.0, 5.0]], [[4.0, 6.0, 2.0], [3.0, 3.0, 2.0], [2.0, 1.0, 6.0]]], [[[6.0, 4.0, 6.0], [6.0, 3.0, 3.0], [6.0, 7.0, 4.0]], [[3.0, 
5.0, 3.0], [8.0, 4.0, 2.0], [8.0, 7.0, 6.0]], [[4.0, 5.0, 3.0], [8.0, 2.0, 6.0], [8.0, 3.0, 1.0]]], [[[3.0, 5.0, 5.0], [2.0, 
4.0, 6.0], [7.0, 4.0, 1.0]], [[8.0, 5.0, 1.0], [8.0, 7.0, 7.0], [4.0, 1.0, 2.0]], [[6.0, 3.0, 7.0], [5.0, 6.0, 8.0], [7.0, 3.0, 
6.0]]], [[[4.0, 2.0, 4.0], [1.0, 1.0, 5.0], [8.0, 5.0, 7.0]], [[3.0, 5.0, 2.0], [4.0, 6.0, 2.0], [8.0, 6.0, 5.0]], [[1.0, 6.0, 
6.0], [2.0, 1.0, 1.0], [7.0, 7.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 5.0], [2.0, 6.0, 1.0], [8.0, 2.0, 2.0]], 
[[2.0, 3.0, 3.0], [1.0, 2.0, 2.0], [6.0, 1.0, 4.0]], [[5.0, 5.0, 1.0], [2.0, 7.0, 6.0], [6.0, 5.0, 4.0]]], [[[1.0, 6.0, 2.0], 
[7.0, 7.0, 4.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 2.0], [3.0, 3.0, 6.0], [6.0, 7.0, 8.0]], [[7.0, 3.0, 8.0], [7.0, 6.0, 1.0], 
[2.0, 8.0, 2.0]]], [[[3.0, 4.0, 6.0], [4.0, 7.0, 7.0], [6.0, 6.0, 3.0]], [[8.0, 8.0, 5.0], [5.0, 1.0, 3.0], [4.0, 2.0, 1.0]], 
[[8.0, 8.0, 8.0], [2.0, 7.0, 1.0], [4.0, 4.0, 8.0]]], [[[8.0, 1.0, 2.0], [4.0, 8.0, 7.0], [7.0, 6.0, 7.0]], [[6.0, 4.0, 6.0], 
[8.0, 3.0, 1.0], [5.0, 1.0, 6.0]], [[4.0, 5.0, 7.0], [2.0, 2.0, 5.0], [7.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[[6.0]], [[5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0]], [[4.0]]]))
      arg[2:,1,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 5.0], [7.0, 1.0, 7.0], [5.0, 4.0, 2.0]], [[1.0, 8.0, 1.0], [3.0, 4.0, 5.0], [4.0, 
3.0, 5.0]], [[4.0, 6.0, 2.0], [3.0, 3.0, 2.0], [2.0, 1.0, 6.0]]], [[[6.0, 4.0, 6.0], [6.0, 3.0, 3.0], [6.0, 7.0, 4.0]], [[3.0, 
5.0, 3.0], [8.0, 4.0, 2.0], [8.0, 7.0, 6.0]], [[4.0, 5.0, 3.0], [8.0, 2.0, 6.0], [8.0, 3.0, 1.0]]], [[[3.0, 5.0, 5.0], [2.0, 
4.0, 6.0], [7.0, 4.0, 1.0]], [[8.0, 5.0, 1.0], [8.0, 7.0, 6.0], [4.0, 1.0, 2.0]], [[6.0, 3.0, 7.0], [5.0, 6.0, 8.0], [7.0, 3.0, 
6.0]]], [[[4.0, 2.0, 4.0], [1.0, 1.0, 5.0], [8.0, 5.0, 7.0]], [[3.0, 5.0, 2.0], [4.0, 6.0, 5.0], [8.0, 6.0, 5.0]], [[1.0, 6.0, 
6.0], [2.0, 1.0, 1.0], [7.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 5.0], [2.0, 6.0, 1.0], [8.0, 2.0, 2.0]], 
[[2.0, 3.0, 3.0], [1.0, 2.0, 2.0], [6.0, 1.0, 4.0]], [[5.0, 5.0, 1.0], [2.0, 7.0, 6.0], [6.0, 5.0, 4.0]]], [[[1.0, 6.0, 2.0], 
[7.0, 7.0, 4.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 2.0], [3.0, 3.0, 6.0], [6.0, 7.0, 8.0]], [[7.0, 3.0, 8.0], [7.0, 6.0, 1.0], 
[2.0, 8.0, 2.0]]], [[[3.0, 4.0, 6.0], [4.0, 7.0, 7.0], [6.0, 6.0, 3.0]], [[8.0, 8.0, 5.0], [5.0, 1.0, 8.0], [4.0, 2.0, 1.0]], 
[[8.0, 8.0, 8.0], [2.0, 7.0, 1.0], [4.0, 4.0, 8.0]]], [[[8.0, 1.0, 2.0], [4.0, 8.0, 7.0], [7.0, 6.0, 7.0]], [[6.0, 4.0, 6.0], 
[8.0, 3.0, 4.0], [5.0, 1.0, 6.0]], [[4.0, 5.0, 7.0], [2.0, 2.0, 5.0], [7.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PCPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 2.0], [1.0, 8.0, 6.0], [7.0, 4.0, 4.0]], [[3.0, 2.0, 1.0], [5.0, 6.0, 8.0], [4.0, 
8.0, 3.0]], [[3.0, 5.0, 5.0], [6.0, 4.0, 7.0], [8.0, 6.0, 2.0]]], [[[2.0, 7.0, 6.0], [6.0, 7.0, 4.0], [3.0, 3.0, 3.0]], [[5.0, 
2.0, 8.0], [6.0, 2.0, 7.0], [8.0, 6.0, 2.0]], [[2.0, 4.0, 3.0], [5.0, 6.0, 1.0], [4.0, 4.0, 7.0]]], [[[2.0, 1.0, 1.0], [6.0, 
4.0, 2.0], [4.0, 7.0, 3.0]], [[5.0, 3.0, 1.0], [2.0, 5.0, 7.0], [4.0, 4.0, 1.0]], [[4.0, 5.0, 7.0], [4.0, 3.0, 2.0], [2.0, 2.0, 
4.0]]], [[[3.0, 7.0, 6.0], [2.0, 1.0, 3.0], [7.0, 5.0, 3.0]], [[1.0, 7.0, 8.0], [2.0, 4.0, 6.0], [3.0, 7.0, 1.0]], [[5.0, 5.0, 
5.0], [2.0, 1.0, 1.0], [8.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 7.0], [7.0, 7.0, 2.0], [2.0, 7.0, 1.0]], 
[[2.0, 2.0, 4.0], [3.0, 1.0, 1.0], [6.0, 1.0, 7.0]], [[7.0, 4.0, 5.0], [7.0, 5.0, 7.0], [6.0, 1.0, 3.0]]], [[[5.0, 2.0, 6.0], 
[1.0, 4.0, 1.0], [6.0, 5.0, 2.0]], [[1.0, 1.0, 8.0], [3.0, 8.0, 1.0], [4.0, 4.0, 6.0]], [[4.0, 7.0, 4.0], [2.0, 5.0, 5.0], 
[4.0, 4.0, 7.0]]], [[[8.0, 1.0, 5.0], [7.0, 5.0, 4.0], [3.0, 7.0, 6.0]], [[4.0, 1.0, 1.0], [2.0, 1.0, 3.0], [6.0, 7.0, 5.0]], 
[[4.0, 7.0, 3.0], [8.0, 5.0, 7.0], [4.0, 6.0, 3.0]]], [[[6.0, 1.0, 4.0], [7.0, 7.0, 6.0], [1.0, 6.0, 6.0]], [[5.0, 6.0, 7.0], 
[1.0, 6.0, 8.0], [1.0, 4.0, 8.0]], [[5.0, 6.0, 8.0], [4.0, 1.0, 1.0], [4.0, 4.0, 3.0]]]])
      val=Data(numpy.array([[1.0, 2.0, 1.0], [1.0, 1.0, 2.0], [4.0, 5.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 8.0, 6.0], [8.0, 7.0, 7.0], [2.0, 2.0, 6.0]]))
      arg[2,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 2.0], [1.0, 8.0, 6.0], [7.0, 4.0, 4.0]], [[3.0, 2.0, 1.0], [5.0, 6.0, 8.0], [4.0, 
8.0, 3.0]], [[3.0, 5.0, 5.0], [6.0, 4.0, 7.0], [8.0, 6.0, 2.0]]], [[[2.0, 7.0, 6.0], [6.0, 7.0, 4.0], [3.0, 3.0, 3.0]], [[5.0, 
2.0, 8.0], [6.0, 2.0, 7.0], [8.0, 6.0, 2.0]], [[2.0, 4.0, 3.0], [5.0, 6.0, 1.0], [4.0, 4.0, 7.0]]], [[[2.0, 1.0, 1.0], [1.0, 
2.0, 1.0], [4.0, 7.0, 3.0]], [[5.0, 3.0, 1.0], [1.0, 1.0, 2.0], [4.0, 4.0, 1.0]], [[4.0, 5.0, 7.0], [4.0, 5.0, 4.0], [2.0, 2.0, 
4.0]]], [[[3.0, 7.0, 6.0], [2.0, 1.0, 3.0], [7.0, 5.0, 3.0]], [[1.0, 7.0, 8.0], [2.0, 4.0, 6.0], [3.0, 7.0, 1.0]], [[5.0, 5.0, 
5.0], [2.0, 1.0, 1.0], [8.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 7.0], [7.0, 7.0, 2.0], [2.0, 7.0, 1.0]], 
[[2.0, 2.0, 4.0], [3.0, 1.0, 1.0], [6.0, 1.0, 7.0]], [[7.0, 4.0, 5.0], [7.0, 5.0, 7.0], [6.0, 1.0, 3.0]]], [[[5.0, 2.0, 6.0], 
[1.0, 4.0, 1.0], [6.0, 5.0, 2.0]], [[1.0, 1.0, 8.0], [3.0, 8.0, 1.0], [4.0, 4.0, 6.0]], [[4.0, 7.0, 4.0], [2.0, 5.0, 5.0], 
[4.0, 4.0, 7.0]]], [[[8.0, 1.0, 5.0], [6.0, 8.0, 6.0], [3.0, 7.0, 6.0]], [[4.0, 1.0, 1.0], [8.0, 7.0, 7.0], [6.0, 7.0, 5.0]], 
[[4.0, 7.0, 3.0], [2.0, 2.0, 6.0], [4.0, 6.0, 3.0]]], [[[6.0, 1.0, 4.0], [7.0, 7.0, 6.0], [1.0, 6.0, 6.0]], [[5.0, 6.0, 7.0], 
[1.0, 6.0, 8.0], [1.0, 4.0, 8.0]], [[5.0, 6.0, 8.0], [4.0, 1.0, 1.0], [4.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 8.0], [6.0, 8.0, 3.0], [4.0, 6.0, 4.0]], [[4.0, 1.0, 2.0], [4.0, 3.0, 8.0], [2.0, 
4.0, 3.0]], [[2.0, 4.0, 6.0], [2.0, 8.0, 4.0], [3.0, 3.0, 2.0]]], [[[5.0, 6.0, 1.0], [8.0, 4.0, 5.0], [5.0, 6.0, 3.0]], [[7.0, 
2.0, 3.0], [3.0, 5.0, 8.0], [2.0, 8.0, 5.0]], [[5.0, 1.0, 6.0], [3.0, 2.0, 6.0], [3.0, 7.0, 7.0]]], [[[8.0, 5.0, 1.0], [3.0, 
1.0, 5.0], [3.0, 8.0, 2.0]], [[3.0, 5.0, 3.0], [1.0, 2.0, 6.0], [4.0, 8.0, 4.0]], [[7.0, 5.0, 8.0], [6.0, 6.0, 5.0], [1.0, 7.0, 
3.0]]], [[[2.0, 6.0, 6.0], [3.0, 7.0, 7.0], [3.0, 2.0, 3.0]], [[1.0, 8.0, 3.0], [8.0, 2.0, 6.0], [3.0, 7.0, 6.0]], [[2.0, 8.0, 
1.0], [8.0, 8.0, 1.0], [4.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 2.0], [8.0, 5.0, 7.0], [1.0, 8.0, 5.0]], 
[[8.0, 2.0, 5.0], [8.0, 8.0, 3.0], [8.0, 3.0, 5.0]], [[2.0, 6.0, 4.0], [3.0, 5.0, 3.0], [5.0, 6.0, 4.0]]], [[[7.0, 4.0, 6.0], 
[7.0, 5.0, 1.0], [6.0, 7.0, 1.0]], [[8.0, 2.0, 6.0], [4.0, 7.0, 4.0], [6.0, 1.0, 8.0]], [[6.0, 2.0, 4.0], [3.0, 4.0, 3.0], 
[8.0, 1.0, 6.0]]], [[[5.0, 8.0, 5.0], [5.0, 7.0, 8.0], [7.0, 8.0, 4.0]], [[6.0, 8.0, 3.0], [1.0, 5.0, 2.0], [7.0, 2.0, 6.0]], 
[[3.0, 2.0, 6.0], [6.0, 3.0, 8.0], [5.0, 7.0, 8.0]]], [[[1.0, 7.0, 6.0], [2.0, 1.0, 7.0], [7.0, 7.0, 6.0]], [[3.0, 5.0, 4.0], 
[7.0, 1.0, 2.0], [6.0, 2.0, 5.0]], [[1.0, 8.0, 7.0], [3.0, 1.0, 8.0], [6.0, 7.0, 8.0]]]])
      val=Data(numpy.array([[[[2.0, 8.0, 2.0], [3.0, 1.0, 7.0], [2.0, 6.0, 5.0]], [[4.0, 7.0, 4.0], [7.0, 2.0, 4.0], [7.0, 3.0, 
7.0]], [[4.0, 6.0, 1.0], [5.0, 2.0, 3.0], [6.0, 2.0, 4.0]]], [[[7.0, 7.0, 1.0], [8.0, 6.0, 5.0], [8.0, 8.0, 2.0]], [[6.0, 4.0, 
2.0], [5.0, 8.0, 2.0], [4.0, 1.0, 3.0]], [[8.0, 5.0, 5.0], [7.0, 1.0, 2.0], [2.0, 3.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 4.0, 4.0], [6.0, 6.0, 7.0], [5.0, 5.0, 2.0]], [[3.0, 2.0, 7.0], [7.0, 3.0, 
3.0], [6.0, 2.0, 2.0]], [[7.0, 7.0, 6.0], [2.0, 3.0, 5.0], [3.0, 4.0, 8.0]]], [[[5.0, 5.0, 4.0], [2.0, 6.0, 6.0], [5.0, 4.0, 
2.0]], [[2.0, 8.0, 7.0], [3.0, 5.0, 8.0], [6.0, 8.0, 7.0]], [[6.0, 7.0, 7.0], [8.0, 3.0, 1.0], [3.0, 1.0, 5.0]]]]))
      arg[2:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 8.0], [6.0, 8.0, 3.0], [4.0, 6.0, 4.0]], [[4.0, 1.0, 2.0], [4.0, 3.0, 8.0], [2.0, 
4.0, 3.0]], [[2.0, 4.0, 6.0], [2.0, 8.0, 4.0], [3.0, 3.0, 2.0]]], [[[5.0, 6.0, 1.0], [8.0, 4.0, 5.0], [5.0, 6.0, 3.0]], [[7.0, 
2.0, 3.0], [3.0, 5.0, 8.0], [2.0, 8.0, 5.0]], [[5.0, 1.0, 6.0], [3.0, 2.0, 6.0], [3.0, 7.0, 7.0]]], [[[2.0, 8.0, 2.0], [3.0, 
1.0, 7.0], [2.0, 6.0, 5.0]], [[4.0, 7.0, 4.0], [7.0, 2.0, 4.0], [7.0, 3.0, 7.0]], [[4.0, 6.0, 1.0], [5.0, 2.0, 3.0], [6.0, 2.0, 
4.0]]], [[[7.0, 7.0, 1.0], [8.0, 6.0, 5.0], [8.0, 8.0, 2.0]], [[6.0, 4.0, 2.0], [5.0, 8.0, 2.0], [4.0, 1.0, 3.0]], [[8.0, 5.0, 
5.0], [7.0, 1.0, 2.0], [2.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 2.0], [8.0, 5.0, 7.0], [1.0, 8.0, 5.0]], 
[[8.0, 2.0, 5.0], [8.0, 8.0, 3.0], [8.0, 3.0, 5.0]], [[2.0, 6.0, 4.0], [3.0, 5.0, 3.0], [5.0, 6.0, 4.0]]], [[[7.0, 4.0, 6.0], 
[7.0, 5.0, 1.0], [6.0, 7.0, 1.0]], [[8.0, 2.0, 6.0], [4.0, 7.0, 4.0], [6.0, 1.0, 8.0]], [[6.0, 2.0, 4.0], [3.0, 4.0, 3.0], 
[8.0, 1.0, 6.0]]], [[[1.0, 4.0, 4.0], [6.0, 6.0, 7.0], [5.0, 5.0, 2.0]], [[3.0, 2.0, 7.0], [7.0, 3.0, 3.0], [6.0, 2.0, 2.0]], 
[[7.0, 7.0, 6.0], [2.0, 3.0, 5.0], [3.0, 4.0, 8.0]]], [[[5.0, 5.0, 4.0], [2.0, 6.0, 6.0], [5.0, 4.0, 2.0]], [[2.0, 8.0, 7.0], 
[3.0, 5.0, 8.0], [6.0, 8.0, 7.0]], [[6.0, 7.0, 7.0], [8.0, 3.0, 1.0], [3.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CUUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 1.0], [3.0, 7.0, 5.0], [2.0, 5.0, 6.0]], [[3.0, 1.0, 2.0], [4.0, 4.0, 8.0], [7.0, 
6.0, 2.0]], [[5.0, 8.0, 4.0], [7.0, 6.0, 3.0], [3.0, 7.0, 1.0]]], [[[6.0, 5.0, 6.0], [3.0, 7.0, 7.0], [6.0, 6.0, 1.0]], [[1.0, 
4.0, 2.0], [7.0, 2.0, 5.0], [4.0, 5.0, 1.0]], [[3.0, 6.0, 6.0], [1.0, 2.0, 2.0], [5.0, 7.0, 5.0]]], [[[7.0, 2.0, 1.0], [4.0, 
5.0, 4.0], [1.0, 1.0, 2.0]], [[6.0, 2.0, 7.0], [5.0, 8.0, 3.0], [3.0, 7.0, 4.0]], [[6.0, 3.0, 4.0], [5.0, 3.0, 3.0], [3.0, 2.0, 
2.0]]], [[[1.0, 2.0, 7.0], [2.0, 8.0, 1.0], [1.0, 3.0, 1.0]], [[5.0, 8.0, 5.0], [8.0, 3.0, 5.0], [2.0, 7.0, 7.0]], [[1.0, 6.0, 
2.0], [3.0, 3.0, 8.0], [5.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 5.0, 8.0], [2.0, 6.0, 1.0], [1.0, 5.0, 5.0]], 
[[8.0, 4.0, 7.0], [8.0, 8.0, 6.0], [8.0, 8.0, 6.0]], [[3.0, 4.0, 2.0], [8.0, 2.0, 7.0], [1.0, 3.0, 8.0]]], [[[1.0, 4.0, 6.0], 
[6.0, 2.0, 3.0], [5.0, 3.0, 3.0]], [[5.0, 2.0, 5.0], [8.0, 6.0, 2.0], [4.0, 6.0, 1.0]], [[4.0, 6.0, 5.0], [1.0, 4.0, 2.0], 
[7.0, 4.0, 7.0]]], [[[8.0, 3.0, 1.0], [4.0, 1.0, 1.0], [5.0, 2.0, 4.0]], [[8.0, 7.0, 1.0], [1.0, 2.0, 8.0], [7.0, 3.0, 3.0]], 
[[7.0, 4.0, 1.0], [2.0, 1.0, 1.0], [7.0, 8.0, 4.0]]], [[[3.0, 1.0, 4.0], [6.0, 8.0, 7.0], [1.0, 8.0, 3.0]], [[2.0, 1.0, 2.0], 
[7.0, 1.0, 8.0], [3.0, 8.0, 3.0]], [[7.0, 2.0, 2.0], [6.0, 3.0, 8.0], [2.0, 8.0, 5.0]]]])
      val=Data(numpy.array([[[[5.0, 3.0, 8.0], [6.0, 5.0, 8.0]], [[6.0, 3.0, 5.0], [2.0, 7.0, 5.0]]], [[[6.0, 4.0, 5.0], [8.0, 
2.0, 6.0]], [[6.0, 3.0, 8.0], [5.0, 5.0, 7.0]]], [[[3.0, 2.0, 4.0], [2.0, 8.0, 5.0]], [[4.0, 4.0, 6.0], [4.0, 4.0, 1.0]]], 
[[[3.0, 7.0, 6.0], [4.0, 3.0, 8.0]], [[5.0, 6.0, 6.0], [1.0, 8.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 7.0], [7.0, 7.0, 1.0]], [[6.0, 1.0, 5.0], [2.0, 3.0, 6.0]]], [[[2.0, 6.0, 
4.0], [5.0, 3.0, 4.0]], [[5.0, 2.0, 6.0], [2.0, 1.0, 8.0]]], [[[5.0, 6.0, 1.0], [4.0, 6.0, 8.0]], [[7.0, 4.0, 7.0], [2.0, 3.0, 
6.0]]], [[[1.0, 4.0, 7.0], [5.0, 3.0, 4.0]], [[2.0, 4.0, 4.0], [4.0, 3.0, 2.0]]]]))
      arg[:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 8.0], [6.0, 5.0, 8.0], [2.0, 5.0, 6.0]], [[6.0, 3.0, 5.0], [2.0, 7.0, 5.0], [7.0, 
6.0, 2.0]], [[5.0, 8.0, 4.0], [7.0, 6.0, 3.0], [3.0, 7.0, 1.0]]], [[[6.0, 4.0, 5.0], [8.0, 2.0, 6.0], [6.0, 6.0, 1.0]], [[6.0, 
3.0, 8.0], [5.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[3.0, 6.0, 6.0], [1.0, 2.0, 2.0], [5.0, 7.0, 5.0]]], [[[3.0, 2.0, 4.0], [2.0, 
8.0, 5.0], [1.0, 1.0, 2.0]], [[4.0, 4.0, 6.0], [4.0, 4.0, 1.0], [3.0, 7.0, 4.0]], [[6.0, 3.0, 4.0], [5.0, 3.0, 3.0], [3.0, 2.0, 
2.0]]], [[[3.0, 7.0, 6.0], [4.0, 3.0, 8.0], [1.0, 3.0, 1.0]], [[5.0, 6.0, 6.0], [1.0, 8.0, 7.0], [2.0, 7.0, 7.0]], [[1.0, 6.0, 
2.0], [3.0, 3.0, 8.0], [5.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 7.0], [7.0, 7.0, 1.0], [1.0, 5.0, 5.0]], 
[[6.0, 1.0, 5.0], [2.0, 3.0, 6.0], [8.0, 8.0, 6.0]], [[3.0, 4.0, 2.0], [8.0, 2.0, 7.0], [1.0, 3.0, 8.0]]], [[[2.0, 6.0, 4.0], 
[5.0, 3.0, 4.0], [5.0, 3.0, 3.0]], [[5.0, 2.0, 6.0], [2.0, 1.0, 8.0], [4.0, 6.0, 1.0]], [[4.0, 6.0, 5.0], [1.0, 4.0, 2.0], 
[7.0, 4.0, 7.0]]], [[[5.0, 6.0, 1.0], [4.0, 6.0, 8.0], [5.0, 2.0, 4.0]], [[7.0, 4.0, 7.0], [2.0, 3.0, 6.0], [7.0, 3.0, 3.0]], 
[[7.0, 4.0, 1.0], [2.0, 1.0, 1.0], [7.0, 8.0, 4.0]]], [[[1.0, 4.0, 7.0], [5.0, 3.0, 4.0], [1.0, 8.0, 3.0]], [[2.0, 4.0, 4.0], 
[4.0, 3.0, 2.0], [3.0, 8.0, 3.0]], [[7.0, 2.0, 2.0], [6.0, 3.0, 8.0], [2.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 6.0, 7.0], [1.0, 2.0, 1.0], [1.0, 7.0, 7.0]], [[1.0, 3.0, 3.0], [1.0, 6.0, 2.0], [8.0, 
5.0, 7.0]], [[5.0, 4.0, 1.0], [4.0, 1.0, 5.0], [7.0, 7.0, 5.0]]], [[[3.0, 7.0, 2.0], [7.0, 1.0, 5.0], [7.0, 4.0, 5.0]], [[3.0, 
4.0, 7.0], [2.0, 7.0, 1.0], [1.0, 2.0, 2.0]], [[5.0, 3.0, 4.0], [5.0, 6.0, 8.0], [6.0, 6.0, 5.0]]], [[[4.0, 6.0, 5.0], [2.0, 
5.0, 8.0], [1.0, 1.0, 2.0]], [[3.0, 7.0, 6.0], [1.0, 5.0, 2.0], [8.0, 1.0, 1.0]], [[5.0, 7.0, 6.0], [5.0, 1.0, 6.0], [6.0, 8.0, 
6.0]]], [[[5.0, 4.0, 7.0], [6.0, 6.0, 4.0], [3.0, 8.0, 3.0]], [[5.0, 5.0, 6.0], [5.0, 2.0, 7.0], [5.0, 8.0, 6.0]], [[8.0, 7.0, 
2.0], [7.0, 8.0, 1.0], [7.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 2.0], [8.0, 1.0, 8.0], [2.0, 7.0, 2.0]], 
[[7.0, 1.0, 1.0], [1.0, 5.0, 5.0], [3.0, 4.0, 6.0]], [[6.0, 3.0, 4.0], [5.0, 7.0, 3.0], [3.0, 3.0, 7.0]]], [[[5.0, 3.0, 1.0], 
[5.0, 2.0, 8.0], [3.0, 2.0, 8.0]], [[5.0, 6.0, 7.0], [4.0, 8.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 5.0, 7.0], [5.0, 4.0, 2.0], 
[7.0, 4.0, 2.0]]], [[[6.0, 6.0, 5.0], [4.0, 5.0, 7.0], [5.0, 1.0, 8.0]], [[4.0, 4.0, 1.0], [5.0, 3.0, 2.0], [2.0, 7.0, 4.0]], 
[[4.0, 2.0, 2.0], [7.0, 7.0, 2.0], [3.0, 8.0, 3.0]]], [[[3.0, 2.0, 4.0], [8.0, 5.0, 3.0], [7.0, 7.0, 4.0]], [[6.0, 4.0, 8.0], 
[4.0, 1.0, 6.0], [5.0, 5.0, 4.0]], [[8.0, 8.0, 2.0], [2.0, 3.0, 7.0], [6.0, 3.0, 8.0]]]])
      val=Data(numpy.array([[[[6.0, 6.0, 1.0], [6.0, 2.0, 1.0], [5.0, 3.0, 3.0]]], [[[5.0, 5.0, 5.0], [5.0, 5.0, 6.0], [3.0, 
2.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 6.0], [7.0, 1.0, 8.0], [6.0, 1.0, 2.0]]], [[[3.0, 8.0, 3.0], [1.0, 4.0, 
1.0], [7.0, 3.0, 1.0]]]]))
      arg[2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 7.0], [1.0, 2.0, 1.0], [1.0, 7.0, 7.0]], [[1.0, 3.0, 3.0], [1.0, 6.0, 2.0], [8.0, 
5.0, 7.0]], [[5.0, 4.0, 1.0], [4.0, 1.0, 5.0], [7.0, 7.0, 5.0]]], [[[3.0, 7.0, 2.0], [7.0, 1.0, 5.0], [7.0, 4.0, 5.0]], [[3.0, 
4.0, 7.0], [2.0, 7.0, 1.0], [1.0, 2.0, 2.0]], [[5.0, 3.0, 4.0], [5.0, 6.0, 8.0], [6.0, 6.0, 5.0]]], [[[4.0, 6.0, 5.0], [2.0, 
5.0, 8.0], [1.0, 1.0, 2.0]], [[3.0, 7.0, 6.0], [1.0, 5.0, 2.0], [8.0, 1.0, 1.0]], [[6.0, 6.0, 1.0], [6.0, 2.0, 1.0], [5.0, 3.0, 
3.0]]], [[[5.0, 4.0, 7.0], [6.0, 6.0, 4.0], [3.0, 8.0, 3.0]], [[5.0, 5.0, 6.0], [5.0, 2.0, 7.0], [5.0, 8.0, 6.0]], [[5.0, 5.0, 
5.0], [5.0, 5.0, 6.0], [3.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 2.0], [8.0, 1.0, 8.0], [2.0, 7.0, 2.0]], 
[[7.0, 1.0, 1.0], [1.0, 5.0, 5.0], [3.0, 4.0, 6.0]], [[6.0, 3.0, 4.0], [5.0, 7.0, 3.0], [3.0, 3.0, 7.0]]], [[[5.0, 3.0, 1.0], 
[5.0, 2.0, 8.0], [3.0, 2.0, 8.0]], [[5.0, 6.0, 7.0], [4.0, 8.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 5.0, 7.0], [5.0, 4.0, 2.0], 
[7.0, 4.0, 2.0]]], [[[6.0, 6.0, 5.0], [4.0, 5.0, 7.0], [5.0, 1.0, 8.0]], [[4.0, 4.0, 1.0], [5.0, 3.0, 2.0], [2.0, 7.0, 4.0]], 
[[1.0, 7.0, 6.0], [7.0, 1.0, 8.0], [6.0, 1.0, 2.0]]], [[[3.0, 2.0, 4.0], [8.0, 5.0, 3.0], [7.0, 7.0, 4.0]], [[6.0, 4.0, 8.0], 
[4.0, 1.0, 6.0], [5.0, 5.0, 4.0]], [[3.0, 8.0, 3.0], [1.0, 4.0, 1.0], [7.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPCR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 6.0], [6.0, 3.0, 3.0], [8.0, 3.0, 8.0]], [[4.0, 1.0, 3.0], [5.0, 8.0, 5.0], [1.0, 
3.0, 4.0]], [[1.0, 2.0, 8.0], [8.0, 1.0, 8.0], [3.0, 1.0, 6.0]]], [[[6.0, 5.0, 2.0], [4.0, 4.0, 6.0], [2.0, 1.0, 2.0]], [[3.0, 
1.0, 7.0], [1.0, 6.0, 2.0], [5.0, 5.0, 4.0]], [[1.0, 8.0, 1.0], [3.0, 6.0, 2.0], [2.0, 5.0, 1.0]]], [[[5.0, 8.0, 1.0], [2.0, 
6.0, 1.0], [8.0, 4.0, 8.0]], [[5.0, 5.0, 5.0], [6.0, 7.0, 4.0], [8.0, 2.0, 7.0]], [[6.0, 7.0, 2.0], [1.0, 8.0, 1.0], [3.0, 8.0, 
1.0]]], [[[5.0, 1.0, 1.0], [7.0, 4.0, 7.0], [6.0, 2.0, 8.0]], [[6.0, 1.0, 7.0], [3.0, 7.0, 8.0], [1.0, 3.0, 7.0]], [[4.0, 8.0, 
8.0], [3.0, 1.0, 4.0], [2.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 2.0], [8.0, 1.0, 1.0], [1.0, 1.0, 1.0]], 
[[2.0, 4.0, 5.0], [7.0, 6.0, 3.0], [7.0, 3.0, 4.0]], [[6.0, 4.0, 4.0], [3.0, 5.0, 4.0], [6.0, 8.0, 5.0]]], [[[7.0, 6.0, 6.0], 
[2.0, 2.0, 1.0], [4.0, 4.0, 2.0]], [[3.0, 7.0, 2.0], [2.0, 8.0, 5.0], [1.0, 5.0, 3.0]], [[2.0, 7.0, 8.0], [4.0, 1.0, 8.0], 
[4.0, 1.0, 3.0]]], [[[2.0, 7.0, 6.0], [3.0, 3.0, 3.0], [3.0, 3.0, 7.0]], [[1.0, 2.0, 2.0], [5.0, 2.0, 4.0], [8.0, 2.0, 5.0]], 
[[3.0, 6.0, 4.0], [3.0, 6.0, 8.0], [2.0, 2.0, 6.0]]], [[[2.0, 4.0, 4.0], [4.0, 8.0, 5.0], [7.0, 8.0, 7.0]], [[5.0, 1.0, 6.0], 
[5.0, 3.0, 2.0], [3.0, 5.0, 6.0]], [[7.0, 3.0, 7.0], [4.0, 5.0, 3.0], [6.0, 2.0, 6.0]]]])
      val=Data(numpy.array([[1.0], [7.0], [1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0], [5.0], [1.0]]))
      arg[2,1,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 6.0], [6.0, 3.0, 3.0], [8.0, 3.0, 8.0]], [[4.0, 1.0, 3.0], [5.0, 8.0, 5.0], [1.0, 
3.0, 4.0]], [[1.0, 2.0, 8.0], [8.0, 1.0, 8.0], [3.0, 1.0, 6.0]]], [[[6.0, 5.0, 2.0], [4.0, 4.0, 6.0], [2.0, 1.0, 2.0]], [[3.0, 
1.0, 7.0], [1.0, 6.0, 2.0], [5.0, 5.0, 4.0]], [[1.0, 8.0, 1.0], [3.0, 6.0, 2.0], [2.0, 5.0, 1.0]]], [[[5.0, 8.0, 1.0], [2.0, 
6.0, 1.0], [8.0, 4.0, 8.0]], [[5.0, 1.0, 5.0], [6.0, 7.0, 4.0], [8.0, 1.0, 7.0]], [[6.0, 7.0, 2.0], [1.0, 8.0, 1.0], [3.0, 8.0, 
1.0]]], [[[5.0, 1.0, 1.0], [7.0, 4.0, 7.0], [6.0, 2.0, 8.0]], [[6.0, 1.0, 7.0], [3.0, 7.0, 8.0], [1.0, 3.0, 7.0]], [[4.0, 8.0, 
8.0], [3.0, 1.0, 4.0], [2.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 2.0], [8.0, 1.0, 1.0], [1.0, 1.0, 1.0]], 
[[2.0, 4.0, 5.0], [7.0, 6.0, 3.0], [7.0, 3.0, 4.0]], [[6.0, 4.0, 4.0], [3.0, 5.0, 4.0], [6.0, 8.0, 5.0]]], [[[7.0, 6.0, 6.0], 
[2.0, 2.0, 1.0], [4.0, 4.0, 2.0]], [[3.0, 7.0, 2.0], [2.0, 8.0, 5.0], [1.0, 5.0, 3.0]], [[2.0, 7.0, 8.0], [4.0, 1.0, 8.0], 
[4.0, 1.0, 3.0]]], [[[2.0, 7.0, 6.0], [3.0, 3.0, 3.0], [3.0, 3.0, 7.0]], [[1.0, 3.0, 2.0], [5.0, 5.0, 4.0], [8.0, 1.0, 5.0]], 
[[3.0, 6.0, 4.0], [3.0, 6.0, 8.0], [2.0, 2.0, 6.0]]], [[[2.0, 4.0, 4.0], [4.0, 8.0, 5.0], [7.0, 8.0, 7.0]], [[5.0, 1.0, 6.0], 
[5.0, 3.0, 2.0], [3.0, 5.0, 6.0]], [[7.0, 3.0, 7.0], [4.0, 5.0, 3.0], [6.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 3.0], [8.0, 6.0, 7.0], [7.0, 8.0, 7.0]], [[2.0, 1.0, 2.0], [2.0, 6.0, 5.0], [5.0, 
4.0, 8.0]], [[8.0, 6.0, 6.0], [4.0, 8.0, 4.0], [8.0, 6.0, 2.0]]], [[[4.0, 1.0, 1.0], [6.0, 8.0, 6.0], [1.0, 7.0, 1.0]], [[8.0, 
7.0, 8.0], [7.0, 7.0, 3.0], [4.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [5.0, 6.0, 8.0], [2.0, 3.0, 4.0]]], [[[8.0, 7.0, 1.0], [2.0, 
5.0, 5.0], [4.0, 1.0, 4.0]], [[7.0, 5.0, 7.0], [6.0, 5.0, 2.0], [7.0, 5.0, 3.0]], [[4.0, 2.0, 1.0], [7.0, 8.0, 3.0], [5.0, 4.0, 
6.0]]], [[[4.0, 5.0, 8.0], [2.0, 1.0, 8.0], [6.0, 5.0, 2.0]], [[8.0, 6.0, 3.0], [7.0, 3.0, 5.0], [4.0, 5.0, 5.0]], [[7.0, 1.0, 
8.0], [7.0, 4.0, 3.0], [6.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 8.0], [8.0, 1.0, 5.0], [1.0, 3.0, 7.0]], 
[[2.0, 5.0, 3.0], [8.0, 5.0, 4.0], [6.0, 4.0, 7.0]], [[4.0, 5.0, 1.0], [3.0, 4.0, 3.0], [1.0, 2.0, 2.0]]], [[[3.0, 4.0, 7.0], 
[1.0, 8.0, 8.0], [6.0, 8.0, 8.0]], [[4.0, 1.0, 8.0], [2.0, 4.0, 2.0], [6.0, 3.0, 6.0]], [[2.0, 6.0, 7.0], [8.0, 8.0, 8.0], 
[2.0, 4.0, 8.0]]], [[[8.0, 3.0, 3.0], [1.0, 2.0, 5.0], [6.0, 3.0, 4.0]], [[6.0, 3.0, 6.0], [5.0, 2.0, 3.0], [7.0, 4.0, 7.0]], 
[[6.0, 8.0, 7.0], [3.0, 4.0, 3.0], [8.0, 5.0, 5.0]]], [[[2.0, 3.0, 6.0], [1.0, 5.0, 2.0], [7.0, 2.0, 3.0]], [[8.0, 7.0, 1.0], 
[5.0, 3.0, 4.0], [4.0, 2.0, 1.0]], [[6.0, 4.0, 5.0], [7.0, 3.0, 2.0], [3.0, 6.0, 1.0]]]])
      val=Data(numpy.array([[[[8.0, 2.0, 3.0], [1.0, 5.0, 8.0]], [[3.0, 5.0, 5.0], [7.0, 7.0, 2.0]], [[4.0, 7.0, 1.0], [3.0, 
7.0, 7.0]]], [[[2.0, 1.0, 5.0], [3.0, 5.0, 6.0]], [[8.0, 4.0, 1.0], [4.0, 5.0, 3.0]], [[5.0, 2.0, 7.0], [5.0, 2.0, 
8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 6.0, 5.0], [6.0, 2.0, 3.0]], [[8.0, 8.0, 2.0], [5.0, 8.0, 8.0]], [[5.0, 1.0, 
7.0], [6.0, 5.0, 4.0]]], [[[6.0, 3.0, 5.0], [6.0, 3.0, 4.0]], [[4.0, 6.0, 4.0], [3.0, 5.0, 4.0]], [[3.0, 1.0, 5.0], [2.0, 4.0, 
4.0]]]]))
      arg[2:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 3.0], [8.0, 6.0, 7.0], [7.0, 8.0, 7.0]], [[2.0, 1.0, 2.0], [2.0, 6.0, 5.0], [5.0, 
4.0, 8.0]], [[8.0, 6.0, 6.0], [4.0, 8.0, 4.0], [8.0, 6.0, 2.0]]], [[[4.0, 1.0, 1.0], [6.0, 8.0, 6.0], [1.0, 7.0, 1.0]], [[8.0, 
7.0, 8.0], [7.0, 7.0, 3.0], [4.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [5.0, 6.0, 8.0], [2.0, 3.0, 4.0]]], [[[8.0, 2.0, 3.0], [1.0, 
5.0, 8.0], [4.0, 1.0, 4.0]], [[3.0, 5.0, 5.0], [7.0, 7.0, 2.0], [7.0, 5.0, 3.0]], [[4.0, 7.0, 1.0], [3.0, 7.0, 7.0], [5.0, 4.0, 
6.0]]], [[[2.0, 1.0, 5.0], [3.0, 5.0, 6.0], [6.0, 5.0, 2.0]], [[8.0, 4.0, 1.0], [4.0, 5.0, 3.0], [4.0, 5.0, 5.0]], [[5.0, 2.0, 
7.0], [5.0, 2.0, 8.0], [6.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 8.0], [8.0, 1.0, 5.0], [1.0, 3.0, 7.0]], 
[[2.0, 5.0, 3.0], [8.0, 5.0, 4.0], [6.0, 4.0, 7.0]], [[4.0, 5.0, 1.0], [3.0, 4.0, 3.0], [1.0, 2.0, 2.0]]], [[[3.0, 4.0, 7.0], 
[1.0, 8.0, 8.0], [6.0, 8.0, 8.0]], [[4.0, 1.0, 8.0], [2.0, 4.0, 2.0], [6.0, 3.0, 6.0]], [[2.0, 6.0, 7.0], [8.0, 8.0, 8.0], 
[2.0, 4.0, 8.0]]], [[[4.0, 6.0, 5.0], [6.0, 2.0, 3.0], [6.0, 3.0, 4.0]], [[8.0, 8.0, 2.0], [5.0, 8.0, 8.0], [7.0, 4.0, 7.0]], 
[[5.0, 1.0, 7.0], [6.0, 5.0, 4.0], [8.0, 5.0, 5.0]]], [[[6.0, 3.0, 5.0], [6.0, 3.0, 4.0], [7.0, 2.0, 3.0]], [[4.0, 6.0, 4.0], 
[3.0, 5.0, 4.0], [4.0, 2.0, 1.0]], [[3.0, 1.0, 5.0], [2.0, 4.0, 4.0], [3.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 5.0], [7.0, 4.0, 2.0], [7.0, 4.0, 6.0]], [[1.0, 1.0, 1.0], [5.0, 7.0, 2.0], [7.0, 
2.0, 2.0]], [[2.0, 3.0, 7.0], [2.0, 4.0, 7.0], [1.0, 6.0, 3.0]]], [[[8.0, 6.0, 1.0], [8.0, 8.0, 4.0], [4.0, 3.0, 3.0]], [[6.0, 
3.0, 3.0], [5.0, 7.0, 3.0], [8.0, 8.0, 3.0]], [[5.0, 5.0, 1.0], [6.0, 8.0, 1.0], [8.0, 5.0, 3.0]]], [[[6.0, 1.0, 1.0], [8.0, 
5.0, 8.0], [7.0, 6.0, 4.0]], [[6.0, 7.0, 4.0], [6.0, 5.0, 8.0], [1.0, 7.0, 5.0]], [[1.0, 3.0, 5.0], [7.0, 3.0, 3.0], [6.0, 6.0, 
6.0]]], [[[8.0, 6.0, 6.0], [4.0, 6.0, 8.0], [8.0, 3.0, 4.0]], [[3.0, 3.0, 8.0], [2.0, 6.0, 3.0], [1.0, 1.0, 8.0]], [[6.0, 6.0, 
2.0], [3.0, 2.0, 7.0], [5.0, 7.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 7.0], [3.0, 4.0, 7.0], [2.0, 6.0, 4.0]], 
[[8.0, 7.0, 7.0], [8.0, 1.0, 5.0], [2.0, 7.0, 2.0]], [[5.0, 7.0, 3.0], [4.0, 1.0, 3.0], [2.0, 4.0, 4.0]]], [[[7.0, 2.0, 4.0], 
[3.0, 1.0, 8.0], [6.0, 2.0, 1.0]], [[5.0, 8.0, 3.0], [8.0, 3.0, 4.0], [3.0, 7.0, 7.0]], [[7.0, 7.0, 7.0], [4.0, 7.0, 2.0], 
[5.0, 8.0, 7.0]]], [[[1.0, 2.0, 5.0], [7.0, 8.0, 8.0], [5.0, 4.0, 6.0]], [[3.0, 2.0, 7.0], [8.0, 6.0, 5.0], [1.0, 6.0, 7.0]], 
[[4.0, 7.0, 3.0], [6.0, 7.0, 2.0], [8.0, 1.0, 7.0]]], [[[6.0, 7.0, 4.0], [8.0, 8.0, 8.0], [4.0, 4.0, 7.0]], [[1.0, 3.0, 1.0], 
[8.0, 8.0, 4.0], [7.0, 6.0, 7.0]], [[7.0, 4.0, 2.0], [3.0, 7.0, 6.0], [6.0, 4.0, 1.0]]]])
      val=Data(numpy.array([[[1.0, 1.0, 7.0], [4.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[3.0, 4.0, 1.0], [8.0, 8.0, 2.0], [1.0, 1.0, 
2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 8.0, 1.0], [3.0, 3.0, 6.0], [6.0, 4.0, 2.0]], [[2.0, 4.0, 5.0], [7.0, 6.0, 8.0], 
[8.0, 3.0, 5.0]]]))
      arg[2:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 5.0], [7.0, 4.0, 2.0], [7.0, 4.0, 6.0]], [[1.0, 1.0, 1.0], [5.0, 7.0, 2.0], [7.0, 
2.0, 2.0]], [[2.0, 3.0, 7.0], [2.0, 4.0, 7.0], [1.0, 6.0, 3.0]]], [[[8.0, 6.0, 1.0], [8.0, 8.0, 4.0], [4.0, 3.0, 3.0]], [[6.0, 
3.0, 3.0], [5.0, 7.0, 3.0], [8.0, 8.0, 3.0]], [[5.0, 5.0, 1.0], [6.0, 8.0, 1.0], [8.0, 5.0, 3.0]]], [[[6.0, 1.0, 1.0], [1.0, 
1.0, 7.0], [7.0, 6.0, 4.0]], [[6.0, 7.0, 4.0], [4.0, 6.0, 7.0], [1.0, 7.0, 5.0]], [[1.0, 3.0, 5.0], [6.0, 5.0, 5.0], [6.0, 6.0, 
6.0]]], [[[8.0, 6.0, 6.0], [3.0, 4.0, 1.0], [8.0, 3.0, 4.0]], [[3.0, 3.0, 8.0], [8.0, 8.0, 2.0], [1.0, 1.0, 8.0]], [[6.0, 6.0, 
2.0], [1.0, 1.0, 2.0], [5.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 7.0], [3.0, 4.0, 7.0], [2.0, 6.0, 4.0]], 
[[8.0, 7.0, 7.0], [8.0, 1.0, 5.0], [2.0, 7.0, 2.0]], [[5.0, 7.0, 3.0], [4.0, 1.0, 3.0], [2.0, 4.0, 4.0]]], [[[7.0, 2.0, 4.0], 
[3.0, 1.0, 8.0], [6.0, 2.0, 1.0]], [[5.0, 8.0, 3.0], [8.0, 3.0, 4.0], [3.0, 7.0, 7.0]], [[7.0, 7.0, 7.0], [4.0, 7.0, 2.0], 
[5.0, 8.0, 7.0]]], [[[1.0, 2.0, 5.0], [3.0, 8.0, 1.0], [5.0, 4.0, 6.0]], [[3.0, 2.0, 7.0], [3.0, 3.0, 6.0], [1.0, 6.0, 7.0]], 
[[4.0, 7.0, 3.0], [6.0, 4.0, 2.0], [8.0, 1.0, 7.0]]], [[[6.0, 7.0, 4.0], [2.0, 4.0, 5.0], [4.0, 4.0, 7.0]], [[1.0, 3.0, 1.0], 
[7.0, 6.0, 8.0], [7.0, 6.0, 7.0]], [[7.0, 4.0, 2.0], [8.0, 3.0, 5.0], [6.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RPCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 6.0], [7.0, 1.0, 8.0], [4.0, 5.0, 3.0]], [[6.0, 4.0, 3.0], [4.0, 4.0, 6.0], [7.0, 
6.0, 4.0]], [[5.0, 5.0, 6.0], [3.0, 6.0, 8.0], [8.0, 1.0, 5.0]]], [[[6.0, 1.0, 1.0], [3.0, 6.0, 4.0], [8.0, 7.0, 7.0]], [[8.0, 
8.0, 7.0], [3.0, 6.0, 2.0], [5.0, 3.0, 5.0]], [[4.0, 8.0, 7.0], [4.0, 4.0, 1.0], [2.0, 3.0, 8.0]]], [[[5.0, 7.0, 7.0], [5.0, 
4.0, 6.0], [1.0, 4.0, 3.0]], [[5.0, 5.0, 1.0], [3.0, 2.0, 8.0], [6.0, 3.0, 6.0]], [[8.0, 4.0, 4.0], [8.0, 6.0, 7.0], [1.0, 3.0, 
3.0]]], [[[7.0, 3.0, 8.0], [4.0, 6.0, 5.0], [7.0, 5.0, 6.0]], [[7.0, 2.0, 6.0], [7.0, 6.0, 1.0], [6.0, 4.0, 5.0]], [[6.0, 1.0, 
4.0], [6.0, 4.0, 2.0], [5.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 3.0], [2.0, 4.0, 7.0], [6.0, 6.0, 4.0]], 
[[8.0, 8.0, 8.0], [8.0, 7.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 1.0, 4.0], [5.0, 1.0, 6.0], [2.0, 3.0, 8.0]]], [[[6.0, 4.0, 6.0], 
[3.0, 2.0, 7.0], [1.0, 4.0, 1.0]], [[2.0, 1.0, 2.0], [3.0, 2.0, 5.0], [8.0, 7.0, 4.0]], [[6.0, 4.0, 2.0], [8.0, 7.0, 8.0], 
[2.0, 3.0, 2.0]]], [[[6.0, 3.0, 8.0], [6.0, 7.0, 2.0], [2.0, 8.0, 7.0]], [[1.0, 8.0, 1.0], [2.0, 6.0, 6.0], [7.0, 7.0, 5.0]], 
[[1.0, 6.0, 6.0], [7.0, 2.0, 5.0], [6.0, 3.0, 7.0]]], [[[6.0, 7.0, 7.0], [3.0, 1.0, 8.0], [1.0, 7.0, 2.0]], [[5.0, 2.0, 1.0], 
[4.0, 6.0, 6.0], [5.0, 8.0, 1.0]], [[1.0, 3.0, 2.0], [1.0, 4.0, 1.0], [4.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[[1.0, 3.0, 4.0], [8.0, 8.0, 5.0], [6.0, 3.0, 2.0]], [[4.0, 6.0, 6.0], [3.0, 5.0, 1.0], [7.0, 2.0, 
4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 2.0, 5.0], [8.0, 4.0, 5.0], [4.0, 3.0, 6.0]], [[6.0, 1.0, 8.0], [7.0, 8.0, 7.0], 
[6.0, 6.0, 3.0]]]))
      arg[1:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 6.0], [7.0, 1.0, 8.0], [4.0, 5.0, 3.0]], [[6.0, 4.0, 3.0], [4.0, 4.0, 6.0], [7.0, 
6.0, 4.0]], [[5.0, 5.0, 6.0], [3.0, 6.0, 8.0], [8.0, 1.0, 5.0]]], [[[6.0, 1.0, 1.0], [3.0, 6.0, 4.0], [8.0, 7.0, 7.0]], [[1.0, 
3.0, 4.0], [8.0, 8.0, 5.0], [6.0, 3.0, 2.0]], [[4.0, 8.0, 7.0], [4.0, 4.0, 1.0], [2.0, 3.0, 8.0]]], [[[5.0, 7.0, 7.0], [5.0, 
4.0, 6.0], [1.0, 4.0, 3.0]], [[4.0, 6.0, 6.0], [3.0, 5.0, 1.0], [7.0, 2.0, 4.0]], [[8.0, 4.0, 4.0], [8.0, 6.0, 7.0], [1.0, 3.0, 
3.0]]], [[[7.0, 3.0, 8.0], [4.0, 6.0, 5.0], [7.0, 5.0, 6.0]], [[7.0, 2.0, 6.0], [7.0, 6.0, 1.0], [6.0, 4.0, 5.0]], [[6.0, 1.0, 
4.0], [6.0, 4.0, 2.0], [5.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 3.0], [2.0, 4.0, 7.0], [6.0, 6.0, 4.0]], 
[[8.0, 8.0, 8.0], [8.0, 7.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 1.0, 4.0], [5.0, 1.0, 6.0], [2.0, 3.0, 8.0]]], [[[6.0, 4.0, 6.0], 
[3.0, 2.0, 7.0], [1.0, 4.0, 1.0]], [[5.0, 2.0, 5.0], [8.0, 4.0, 5.0], [4.0, 3.0, 6.0]], [[6.0, 4.0, 2.0], [8.0, 7.0, 8.0], 
[2.0, 3.0, 2.0]]], [[[6.0, 3.0, 8.0], [6.0, 7.0, 2.0], [2.0, 8.0, 7.0]], [[6.0, 1.0, 8.0], [7.0, 8.0, 7.0], [6.0, 6.0, 3.0]], 
[[1.0, 6.0, 6.0], [7.0, 2.0, 5.0], [6.0, 3.0, 7.0]]], [[[6.0, 7.0, 7.0], [3.0, 1.0, 8.0], [1.0, 7.0, 2.0]], [[5.0, 2.0, 1.0], 
[4.0, 6.0, 6.0], [5.0, 8.0, 1.0]], [[1.0, 3.0, 2.0], [1.0, 4.0, 1.0], [4.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCRP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 4.0], [2.0, 8.0, 4.0], [5.0, 8.0, 2.0]], [[4.0, 5.0, 4.0], [6.0, 3.0, 4.0], [5.0, 
2.0, 6.0]], [[7.0, 1.0, 2.0], [3.0, 5.0, 4.0], [7.0, 8.0, 7.0]]], [[[5.0, 2.0, 8.0], [1.0, 4.0, 6.0], [3.0, 7.0, 7.0]], [[6.0, 
4.0, 6.0], [4.0, 7.0, 8.0], [2.0, 6.0, 1.0]], [[6.0, 5.0, 7.0], [8.0, 3.0, 2.0], [7.0, 4.0, 2.0]]], [[[7.0, 1.0, 8.0], [8.0, 
2.0, 8.0], [7.0, 2.0, 2.0]], [[2.0, 1.0, 3.0], [1.0, 4.0, 7.0], [7.0, 7.0, 8.0]], [[4.0, 3.0, 8.0], [8.0, 6.0, 6.0], [8.0, 5.0, 
1.0]]], [[[5.0, 2.0, 5.0], [6.0, 7.0, 1.0], [8.0, 8.0, 6.0]], [[6.0, 2.0, 7.0], [1.0, 5.0, 3.0], [1.0, 4.0, 1.0]], [[7.0, 7.0, 
3.0], [4.0, 5.0, 1.0], [6.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 8.0, 6.0], [8.0, 3.0, 4.0], [7.0, 4.0, 8.0]], 
[[4.0, 4.0, 6.0], [2.0, 3.0, 3.0], [1.0, 8.0, 6.0]], [[3.0, 6.0, 5.0], [5.0, 7.0, 1.0], [1.0, 8.0, 3.0]]], [[[8.0, 8.0, 8.0], 
[7.0, 4.0, 2.0], [1.0, 3.0, 5.0]], [[4.0, 1.0, 5.0], [6.0, 7.0, 4.0], [2.0, 6.0, 2.0]], [[2.0, 5.0, 8.0], [3.0, 1.0, 1.0], 
[6.0, 2.0, 2.0]]], [[[1.0, 5.0, 2.0], [1.0, 8.0, 8.0], [3.0, 1.0, 5.0]], [[6.0, 3.0, 4.0], [3.0, 5.0, 3.0], [6.0, 7.0, 7.0]], 
[[8.0, 7.0, 7.0], [8.0, 1.0, 6.0], [5.0, 8.0, 5.0]]], [[[1.0, 2.0, 5.0], [5.0, 3.0, 1.0], [7.0, 7.0, 1.0]], [[7.0, 6.0, 1.0], 
[1.0, 6.0, 1.0], [3.0, 6.0, 6.0]], [[3.0, 7.0, 7.0], [5.0, 4.0, 5.0], [6.0, 3.0, 3.0]]]])
      val=Data(numpy.array([[[3.0], [5.0], [7.0]], [[6.0], [8.0], [7.0]], [[5.0], [6.0], [2.0]], [[4.0], [4.0], 
[1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0], [1.0], [4.0]], [[4.0], [2.0], [8.0]], [[6.0], [6.0], [1.0]], [[8.0], [3.0], 
[1.0]]]))
      arg[:,:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 4.0], [2.0, 3.0, 4.0], [5.0, 8.0, 2.0]], [[4.0, 5.0, 4.0], [6.0, 5.0, 4.0], [5.0, 
2.0, 6.0]], [[7.0, 1.0, 2.0], [3.0, 7.0, 4.0], [7.0, 8.0, 7.0]]], [[[5.0, 2.0, 8.0], [1.0, 6.0, 6.0], [3.0, 7.0, 7.0]], [[6.0, 
4.0, 6.0], [4.0, 8.0, 8.0], [2.0, 6.0, 1.0]], [[6.0, 5.0, 7.0], [8.0, 7.0, 2.0], [7.0, 4.0, 2.0]]], [[[7.0, 1.0, 8.0], [8.0, 
5.0, 8.0], [7.0, 2.0, 2.0]], [[2.0, 1.0, 3.0], [1.0, 6.0, 7.0], [7.0, 7.0, 8.0]], [[4.0, 3.0, 8.0], [8.0, 2.0, 6.0], [8.0, 5.0, 
1.0]]], [[[5.0, 2.0, 5.0], [6.0, 4.0, 1.0], [8.0, 8.0, 6.0]], [[6.0, 2.0, 7.0], [1.0, 4.0, 3.0], [1.0, 4.0, 1.0]], [[7.0, 7.0, 
3.0], [4.0, 1.0, 1.0], [6.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 6.0], [8.0, 7.0, 4.0], [7.0, 4.0, 8.0]], 
[[4.0, 4.0, 6.0], [2.0, 1.0, 3.0], [1.0, 8.0, 6.0]], [[3.0, 6.0, 5.0], [5.0, 4.0, 1.0], [1.0, 8.0, 3.0]]], [[[8.0, 8.0, 8.0], 
[7.0, 4.0, 2.0], [1.0, 3.0, 5.0]], [[4.0, 1.0, 5.0], [6.0, 2.0, 4.0], [2.0, 6.0, 2.0]], [[2.0, 5.0, 8.0], [3.0, 8.0, 1.0], 
[6.0, 2.0, 2.0]]], [[[1.0, 5.0, 2.0], [1.0, 6.0, 8.0], [3.0, 1.0, 5.0]], [[6.0, 3.0, 4.0], [3.0, 6.0, 3.0], [6.0, 7.0, 7.0]], 
[[8.0, 7.0, 7.0], [8.0, 1.0, 6.0], [5.0, 8.0, 5.0]]], [[[1.0, 2.0, 5.0], [5.0, 8.0, 1.0], [7.0, 7.0, 1.0]], [[7.0, 6.0, 1.0], 
[1.0, 3.0, 1.0], [3.0, 6.0, 6.0]], [[3.0, 7.0, 7.0], [5.0, 1.0, 5.0], [6.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PLRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 1.0], [2.0, 4.0, 1.0], [3.0, 4.0, 2.0]], [[1.0, 4.0, 2.0], [5.0, 1.0, 7.0], [7.0, 
1.0, 2.0]], [[4.0, 6.0, 8.0], [1.0, 3.0, 2.0], [3.0, 7.0, 5.0]]], [[[7.0, 6.0, 4.0], [2.0, 1.0, 6.0], [2.0, 4.0, 2.0]], [[1.0, 
8.0, 5.0], [7.0, 2.0, 1.0], [5.0, 1.0, 5.0]], [[4.0, 4.0, 5.0], [8.0, 5.0, 3.0], [8.0, 1.0, 8.0]]], [[[2.0, 5.0, 6.0], [8.0, 
4.0, 4.0], [6.0, 7.0, 4.0]], [[1.0, 8.0, 7.0], [7.0, 2.0, 8.0], [7.0, 2.0, 4.0]], [[3.0, 8.0, 2.0], [2.0, 6.0, 1.0], [7.0, 8.0, 
7.0]]], [[[2.0, 2.0, 1.0], [8.0, 5.0, 5.0], [4.0, 4.0, 5.0]], [[2.0, 3.0, 5.0], [7.0, 3.0, 7.0], [8.0, 1.0, 5.0]], [[8.0, 6.0, 
1.0], [4.0, 6.0, 6.0], [4.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 2.0], [1.0, 1.0, 5.0], [8.0, 6.0, 6.0]], 
[[5.0, 5.0, 1.0], [7.0, 4.0, 2.0], [7.0, 6.0, 3.0]], [[1.0, 5.0, 8.0], [2.0, 5.0, 1.0], [7.0, 7.0, 5.0]]], [[[2.0, 8.0, 8.0], 
[4.0, 2.0, 5.0], [1.0, 5.0, 5.0]], [[6.0, 2.0, 1.0], [6.0, 1.0, 1.0], [5.0, 1.0, 5.0]], [[5.0, 4.0, 3.0], [5.0, 4.0, 1.0], 
[1.0, 1.0, 8.0]]], [[[6.0, 6.0, 8.0], [5.0, 2.0, 8.0], [2.0, 6.0, 4.0]], [[6.0, 1.0, 3.0], [1.0, 5.0, 8.0], [4.0, 3.0, 4.0]], 
[[7.0, 2.0, 5.0], [5.0, 6.0, 7.0], [6.0, 8.0, 5.0]]], [[[8.0, 7.0, 8.0], [6.0, 3.0, 5.0], [4.0, 3.0, 1.0]], [[5.0, 2.0, 7.0], 
[8.0, 5.0, 6.0], [8.0, 3.0, 8.0]], [[7.0, 5.0, 4.0], [4.0, 5.0, 4.0], [4.0, 2.0, 6.0]]]])
      val=Data(numpy.array([[[6.0, 4.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 5.0, 2.0]]]))
      arg[2,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 1.0], [2.0, 4.0, 1.0], [3.0, 4.0, 2.0]], [[1.0, 4.0, 2.0], [5.0, 1.0, 7.0], [7.0, 
1.0, 2.0]], [[4.0, 6.0, 8.0], [1.0, 3.0, 2.0], [3.0, 7.0, 5.0]]], [[[7.0, 6.0, 4.0], [2.0, 1.0, 6.0], [2.0, 4.0, 2.0]], [[1.0, 
8.0, 5.0], [7.0, 2.0, 1.0], [5.0, 1.0, 5.0]], [[4.0, 4.0, 5.0], [8.0, 5.0, 3.0], [8.0, 1.0, 8.0]]], [[[2.0, 5.0, 6.0], [8.0, 
4.0, 4.0], [6.0, 7.0, 4.0]], [[1.0, 8.0, 7.0], [7.0, 2.0, 8.0], [7.0, 2.0, 4.0]], [[3.0, 8.0, 2.0], [6.0, 4.0, 8.0], [7.0, 8.0, 
7.0]]], [[[2.0, 2.0, 1.0], [8.0, 5.0, 5.0], [4.0, 4.0, 5.0]], [[2.0, 3.0, 5.0], [7.0, 3.0, 7.0], [8.0, 1.0, 5.0]], [[8.0, 6.0, 
1.0], [4.0, 6.0, 6.0], [4.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 2.0], [1.0, 1.0, 5.0], [8.0, 6.0, 6.0]], 
[[5.0, 5.0, 1.0], [7.0, 4.0, 2.0], [7.0, 6.0, 3.0]], [[1.0, 5.0, 8.0], [2.0, 5.0, 1.0], [7.0, 7.0, 5.0]]], [[[2.0, 8.0, 8.0], 
[4.0, 2.0, 5.0], [1.0, 5.0, 5.0]], [[6.0, 2.0, 1.0], [6.0, 1.0, 1.0], [5.0, 1.0, 5.0]], [[5.0, 4.0, 3.0], [5.0, 4.0, 1.0], 
[1.0, 1.0, 8.0]]], [[[6.0, 6.0, 8.0], [5.0, 2.0, 8.0], [2.0, 6.0, 4.0]], [[6.0, 1.0, 3.0], [1.0, 5.0, 8.0], [4.0, 3.0, 4.0]], 
[[7.0, 2.0, 5.0], [1.0, 5.0, 2.0], [6.0, 8.0, 5.0]]], [[[8.0, 7.0, 8.0], [6.0, 3.0, 5.0], [4.0, 3.0, 1.0]], [[5.0, 2.0, 7.0], 
[8.0, 5.0, 6.0], [8.0, 3.0, 8.0]], [[7.0, 5.0, 4.0], [4.0, 5.0, 4.0], [4.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 6.0], [3.0, 5.0, 8.0], [2.0, 4.0, 4.0]], [[5.0, 5.0, 7.0], [3.0, 8.0, 1.0], [5.0, 
7.0, 1.0]], [[4.0, 8.0, 1.0], [5.0, 4.0, 8.0], [7.0, 5.0, 7.0]]], [[[5.0, 3.0, 6.0], [8.0, 8.0, 1.0], [4.0, 8.0, 5.0]], [[4.0, 
5.0, 2.0], [5.0, 3.0, 4.0], [8.0, 7.0, 8.0]], [[8.0, 8.0, 4.0], [8.0, 4.0, 4.0], [6.0, 3.0, 2.0]]], [[[1.0, 5.0, 6.0], [1.0, 
2.0, 5.0], [8.0, 1.0, 8.0]], [[3.0, 8.0, 5.0], [7.0, 2.0, 6.0], [5.0, 7.0, 8.0]], [[5.0, 3.0, 8.0], [4.0, 4.0, 7.0], [7.0, 4.0, 
8.0]]], [[[7.0, 1.0, 6.0], [5.0, 4.0, 5.0], [6.0, 2.0, 6.0]], [[4.0, 3.0, 2.0], [1.0, 2.0, 2.0], [8.0, 4.0, 1.0]], [[4.0, 1.0, 
4.0], [3.0, 8.0, 2.0], [7.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 4.0], [2.0, 5.0, 7.0], [3.0, 1.0, 6.0]], 
[[5.0, 2.0, 1.0], [2.0, 5.0, 1.0], [7.0, 4.0, 2.0]], [[4.0, 6.0, 2.0], [1.0, 8.0, 3.0], [3.0, 2.0, 6.0]]], [[[5.0, 8.0, 8.0], 
[6.0, 6.0, 5.0], [5.0, 4.0, 7.0]], [[1.0, 4.0, 1.0], [4.0, 5.0, 8.0], [1.0, 2.0, 8.0]], [[1.0, 3.0, 3.0], [6.0, 5.0, 5.0], 
[1.0, 4.0, 5.0]]], [[[2.0, 8.0, 4.0], [5.0, 8.0, 3.0], [2.0, 7.0, 2.0]], [[1.0, 1.0, 5.0], [1.0, 8.0, 7.0], [7.0, 2.0, 7.0]], 
[[2.0, 3.0, 6.0], [6.0, 1.0, 4.0], [1.0, 4.0, 1.0]]], [[[3.0, 3.0, 3.0], [2.0, 7.0, 3.0], [6.0, 4.0, 3.0]], [[7.0, 5.0, 1.0], 
[2.0, 1.0, 8.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 5.0], [5.0, 6.0, 7.0], [3.0, 2.0, 4.0]]]])
      val=Data(numpy.array([8.0, 7.0, 4.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([3.0, 4.0, 6.0]))
      arg[2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 6.0], [3.0, 5.0, 8.0], [2.0, 4.0, 4.0]], [[5.0, 5.0, 7.0], [3.0, 8.0, 1.0], [5.0, 
7.0, 1.0]], [[4.0, 8.0, 1.0], [5.0, 4.0, 8.0], [7.0, 5.0, 7.0]]], [[[5.0, 3.0, 6.0], [8.0, 8.0, 1.0], [4.0, 8.0, 5.0]], [[4.0, 
5.0, 2.0], [5.0, 3.0, 4.0], [8.0, 7.0, 8.0]], [[8.0, 8.0, 4.0], [8.0, 4.0, 4.0], [6.0, 3.0, 2.0]]], [[[1.0, 5.0, 6.0], [1.0, 
2.0, 5.0], [8.0, 1.0, 8.0]], [[3.0, 8.0, 5.0], [8.0, 7.0, 4.0], [5.0, 7.0, 8.0]], [[5.0, 3.0, 8.0], [4.0, 4.0, 7.0], [7.0, 4.0, 
8.0]]], [[[7.0, 1.0, 6.0], [5.0, 4.0, 5.0], [6.0, 2.0, 6.0]], [[4.0, 3.0, 2.0], [1.0, 2.0, 2.0], [8.0, 4.0, 1.0]], [[4.0, 1.0, 
4.0], [3.0, 8.0, 2.0], [7.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 4.0], [2.0, 5.0, 7.0], [3.0, 1.0, 6.0]], 
[[5.0, 2.0, 1.0], [2.0, 5.0, 1.0], [7.0, 4.0, 2.0]], [[4.0, 6.0, 2.0], [1.0, 8.0, 3.0], [3.0, 2.0, 6.0]]], [[[5.0, 8.0, 8.0], 
[6.0, 6.0, 5.0], [5.0, 4.0, 7.0]], [[1.0, 4.0, 1.0], [4.0, 5.0, 8.0], [1.0, 2.0, 8.0]], [[1.0, 3.0, 3.0], [6.0, 5.0, 5.0], 
[1.0, 4.0, 5.0]]], [[[2.0, 8.0, 4.0], [5.0, 8.0, 3.0], [2.0, 7.0, 2.0]], [[1.0, 1.0, 5.0], [3.0, 4.0, 6.0], [7.0, 2.0, 7.0]], 
[[2.0, 3.0, 6.0], [6.0, 1.0, 4.0], [1.0, 4.0, 1.0]]], [[[3.0, 3.0, 3.0], [2.0, 7.0, 3.0], [6.0, 4.0, 3.0]], [[7.0, 5.0, 1.0], 
[2.0, 1.0, 8.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 5.0], [5.0, 6.0, 7.0], [3.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRRU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 7.0], [3.0, 3.0, 2.0], [5.0, 3.0, 5.0]], [[1.0, 6.0, 7.0], [3.0, 1.0, 8.0], [7.0, 
4.0, 4.0]], [[2.0, 5.0, 4.0], [1.0, 6.0, 1.0], [1.0, 7.0, 8.0]]], [[[2.0, 7.0, 3.0], [6.0, 5.0, 5.0], [2.0, 2.0, 7.0]], [[7.0, 
7.0, 1.0], [8.0, 1.0, 2.0], [1.0, 1.0, 1.0]], [[8.0, 5.0, 1.0], [2.0, 4.0, 3.0], [3.0, 1.0, 7.0]]], [[[4.0, 5.0, 5.0], [1.0, 
3.0, 5.0], [5.0, 2.0, 3.0]], [[1.0, 6.0, 1.0], [1.0, 7.0, 7.0], [4.0, 1.0, 7.0]], [[2.0, 2.0, 5.0], [7.0, 2.0, 5.0], [4.0, 7.0, 
8.0]]], [[[8.0, 8.0, 3.0], [2.0, 1.0, 3.0], [8.0, 6.0, 3.0]], [[2.0, 4.0, 4.0], [8.0, 8.0, 8.0], [8.0, 2.0, 4.0]], [[6.0, 1.0, 
7.0], [8.0, 3.0, 6.0], [5.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 2.0], [5.0, 1.0, 1.0], [8.0, 8.0, 3.0]], 
[[3.0, 8.0, 1.0], [5.0, 3.0, 8.0], [1.0, 3.0, 1.0]], [[4.0, 7.0, 5.0], [3.0, 3.0, 6.0], [5.0, 7.0, 5.0]]], [[[1.0, 2.0, 7.0], 
[5.0, 4.0, 2.0], [6.0, 2.0, 7.0]], [[7.0, 5.0, 2.0], [8.0, 7.0, 1.0], [6.0, 5.0, 8.0]], [[2.0, 4.0, 3.0], [2.0, 2.0, 4.0], 
[6.0, 4.0, 5.0]]], [[[8.0, 3.0, 8.0], [5.0, 2.0, 6.0], [8.0, 5.0, 4.0]], [[2.0, 7.0, 1.0], [6.0, 3.0, 7.0], [1.0, 3.0, 7.0]], 
[[2.0, 3.0, 6.0], [2.0, 6.0, 2.0], [1.0, 7.0, 6.0]]], [[[4.0, 8.0, 4.0], [3.0, 6.0, 6.0], [8.0, 2.0, 6.0]], [[6.0, 6.0, 3.0], 
[8.0, 3.0, 5.0], [3.0, 3.0, 7.0]], [[3.0, 8.0, 4.0], [2.0, 3.0, 3.0], [7.0, 3.0, 7.0]]]])
      val=Data(numpy.array([[[[8.0, 7.0]]], [[[1.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 6.0]]], [[[6.0, 1.0]]]]))
      arg[2:,1:2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 7.0], [3.0, 3.0, 2.0], [5.0, 3.0, 5.0]], [[1.0, 6.0, 7.0], [3.0, 1.0, 8.0], [7.0, 
4.0, 4.0]], [[2.0, 5.0, 4.0], [1.0, 6.0, 1.0], [1.0, 7.0, 8.0]]], [[[2.0, 7.0, 3.0], [6.0, 5.0, 5.0], [2.0, 2.0, 7.0]], [[7.0, 
7.0, 1.0], [8.0, 1.0, 2.0], [1.0, 1.0, 1.0]], [[8.0, 5.0, 1.0], [2.0, 4.0, 3.0], [3.0, 1.0, 7.0]]], [[[4.0, 5.0, 5.0], [1.0, 
3.0, 5.0], [5.0, 2.0, 3.0]], [[1.0, 6.0, 1.0], [8.0, 7.0, 7.0], [4.0, 1.0, 7.0]], [[2.0, 2.0, 5.0], [7.0, 2.0, 5.0], [4.0, 7.0, 
8.0]]], [[[8.0, 8.0, 3.0], [2.0, 1.0, 3.0], [8.0, 6.0, 3.0]], [[2.0, 4.0, 4.0], [1.0, 2.0, 8.0], [8.0, 2.0, 4.0]], [[6.0, 1.0, 
7.0], [8.0, 3.0, 6.0], [5.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 2.0], [5.0, 1.0, 1.0], [8.0, 8.0, 3.0]], 
[[3.0, 8.0, 1.0], [5.0, 3.0, 8.0], [1.0, 3.0, 1.0]], [[4.0, 7.0, 5.0], [3.0, 3.0, 6.0], [5.0, 7.0, 5.0]]], [[[1.0, 2.0, 7.0], 
[5.0, 4.0, 2.0], [6.0, 2.0, 7.0]], [[7.0, 5.0, 2.0], [8.0, 7.0, 1.0], [6.0, 5.0, 8.0]], [[2.0, 4.0, 3.0], [2.0, 2.0, 4.0], 
[6.0, 4.0, 5.0]]], [[[8.0, 3.0, 8.0], [5.0, 2.0, 6.0], [8.0, 5.0, 4.0]], [[2.0, 7.0, 1.0], [5.0, 6.0, 7.0], [1.0, 3.0, 7.0]], 
[[2.0, 3.0, 6.0], [2.0, 6.0, 2.0], [1.0, 7.0, 6.0]]], [[[4.0, 8.0, 4.0], [3.0, 6.0, 6.0], [8.0, 2.0, 6.0]], [[6.0, 6.0, 3.0], 
[6.0, 1.0, 5.0], [3.0, 3.0, 7.0]], [[3.0, 8.0, 4.0], [2.0, 3.0, 3.0], [7.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PCCP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 5.0], [1.0, 5.0, 7.0], [4.0, 5.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 1.0, 3.0], [7.0, 
2.0, 7.0]], [[5.0, 4.0, 1.0], [3.0, 1.0, 3.0], [4.0, 4.0, 6.0]]], [[[6.0, 7.0, 2.0], [5.0, 2.0, 7.0], [2.0, 5.0, 4.0]], [[5.0, 
2.0, 3.0], [2.0, 7.0, 8.0], [3.0, 5.0, 4.0]], [[5.0, 1.0, 8.0], [4.0, 4.0, 5.0], [1.0, 7.0, 4.0]]], [[[8.0, 2.0, 5.0], [7.0, 
2.0, 3.0], [5.0, 6.0, 2.0]], [[8.0, 7.0, 2.0], [1.0, 5.0, 2.0], [3.0, 1.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 8.0, 7.0], [7.0, 7.0, 
7.0]]], [[[7.0, 8.0, 1.0], [4.0, 1.0, 3.0], [4.0, 3.0, 6.0]], [[2.0, 5.0, 1.0], [2.0, 4.0, 1.0], [8.0, 6.0, 4.0]], [[2.0, 7.0, 
2.0], [7.0, 7.0, 7.0], [4.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 5.0], [8.0, 5.0, 5.0], [1.0, 5.0, 3.0]], 
[[1.0, 3.0, 6.0], [6.0, 4.0, 1.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 7.0], [2.0, 5.0, 4.0], [4.0, 3.0, 4.0]]], [[[4.0, 5.0, 5.0], 
[5.0, 2.0, 8.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 4.0], [2.0, 2.0, 7.0], [4.0, 2.0, 6.0]], [[5.0, 3.0, 1.0], [3.0, 4.0, 8.0], 
[6.0, 5.0, 3.0]]], [[[8.0, 1.0, 7.0], [1.0, 7.0, 1.0], [5.0, 2.0, 3.0]], [[2.0, 8.0, 1.0], [3.0, 3.0, 1.0], [7.0, 1.0, 2.0]], 
[[5.0, 7.0, 7.0], [5.0, 2.0, 3.0], [4.0, 8.0, 7.0]]], [[[1.0, 5.0, 2.0], [2.0, 1.0, 6.0], [8.0, 6.0, 3.0]], [[3.0, 8.0, 8.0], 
[4.0, 2.0, 2.0], [1.0, 8.0, 8.0]], [[2.0, 4.0, 4.0], [7.0, 8.0, 6.0], [2.0, 7.0, 7.0]]]])
      val=Data(numpy.array([[3.0, 3.0, 1.0], [6.0, 8.0, 6.0], [6.0, 8.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 4.0, 5.0], [7.0, 4.0, 8.0], [6.0, 3.0, 1.0]]))
      arg[2,:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 5.0], [1.0, 5.0, 7.0], [4.0, 5.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 1.0, 3.0], [7.0, 
2.0, 7.0]], [[5.0, 4.0, 1.0], [3.0, 1.0, 3.0], [4.0, 4.0, 6.0]]], [[[6.0, 7.0, 2.0], [5.0, 2.0, 7.0], [2.0, 5.0, 4.0]], [[5.0, 
2.0, 3.0], [2.0, 7.0, 8.0], [3.0, 5.0, 4.0]], [[5.0, 1.0, 8.0], [4.0, 4.0, 5.0], [1.0, 7.0, 4.0]]], [[[8.0, 3.0, 5.0], [7.0, 
3.0, 3.0], [5.0, 1.0, 2.0]], [[8.0, 6.0, 2.0], [1.0, 8.0, 2.0], [3.0, 6.0, 4.0]], [[3.0, 6.0, 6.0], [5.0, 8.0, 7.0], [7.0, 1.0, 
7.0]]], [[[7.0, 8.0, 1.0], [4.0, 1.0, 3.0], [4.0, 3.0, 6.0]], [[2.0, 5.0, 1.0], [2.0, 4.0, 1.0], [8.0, 6.0, 4.0]], [[2.0, 7.0, 
2.0], [7.0, 7.0, 7.0], [4.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 5.0], [8.0, 5.0, 5.0], [1.0, 5.0, 3.0]], 
[[1.0, 3.0, 6.0], [6.0, 4.0, 1.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 7.0], [2.0, 5.0, 4.0], [4.0, 3.0, 4.0]]], [[[4.0, 5.0, 5.0], 
[5.0, 2.0, 8.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 4.0], [2.0, 2.0, 7.0], [4.0, 2.0, 6.0]], [[5.0, 3.0, 1.0], [3.0, 4.0, 8.0], 
[6.0, 5.0, 3.0]]], [[[8.0, 2.0, 7.0], [1.0, 4.0, 1.0], [5.0, 5.0, 3.0]], [[2.0, 7.0, 1.0], [3.0, 4.0, 1.0], [7.0, 8.0, 2.0]], 
[[5.0, 6.0, 7.0], [5.0, 3.0, 3.0], [4.0, 1.0, 7.0]]], [[[1.0, 5.0, 2.0], [2.0, 1.0, 6.0], [8.0, 6.0, 3.0]], [[3.0, 8.0, 8.0], 
[4.0, 2.0, 2.0], [1.0, 8.0, 8.0]], [[2.0, 4.0, 4.0], [7.0, 8.0, 6.0], [2.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UPLU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 8.0], [4.0, 2.0, 4.0], [5.0, 2.0, 8.0]], [[5.0, 3.0, 8.0], [8.0, 2.0, 4.0], [8.0, 
1.0, 5.0]], [[4.0, 8.0, 7.0], [2.0, 5.0, 4.0], [5.0, 1.0, 5.0]]], [[[2.0, 1.0, 4.0], [2.0, 1.0, 4.0], [4.0, 7.0, 8.0]], [[8.0, 
7.0, 6.0], [6.0, 4.0, 7.0], [2.0, 4.0, 5.0]], [[6.0, 2.0, 3.0], [7.0, 2.0, 3.0], [5.0, 3.0, 1.0]]], [[[6.0, 2.0, 4.0], [2.0, 
3.0, 1.0], [7.0, 2.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 1.0, 6.0], [6.0, 4.0, 8.0]], [[6.0, 2.0, 4.0], [6.0, 2.0, 4.0], [5.0, 6.0, 
4.0]]], [[[2.0, 3.0, 5.0], [1.0, 5.0, 1.0], [2.0, 1.0, 2.0]], [[3.0, 1.0, 7.0], [4.0, 1.0, 4.0], [8.0, 4.0, 2.0]], [[4.0, 6.0, 
6.0], [2.0, 7.0, 3.0], [7.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 4.0], [8.0, 8.0, 2.0], [1.0, 2.0, 4.0]], 
[[3.0, 1.0, 2.0], [8.0, 5.0, 6.0], [1.0, 6.0, 4.0]], [[6.0, 2.0, 6.0], [2.0, 4.0, 5.0], [3.0, 8.0, 8.0]]], [[[8.0, 3.0, 4.0], 
[5.0, 3.0, 2.0], [1.0, 7.0, 2.0]], [[1.0, 5.0, 4.0], [2.0, 8.0, 1.0], [3.0, 6.0, 5.0]], [[7.0, 6.0, 1.0], [1.0, 3.0, 3.0], 
[2.0, 4.0, 2.0]]], [[[4.0, 3.0, 1.0], [7.0, 7.0, 1.0], [7.0, 3.0, 3.0]], [[7.0, 7.0, 4.0], [2.0, 7.0, 8.0], [4.0, 6.0, 4.0]], 
[[1.0, 1.0, 1.0], [1.0, 7.0, 6.0], [3.0, 8.0, 2.0]]], [[[7.0, 3.0, 5.0], [5.0, 5.0, 5.0], [7.0, 5.0, 3.0]], [[4.0, 6.0, 7.0], 
[7.0, 8.0, 6.0], [1.0, 4.0, 7.0]], [[3.0, 2.0, 5.0], [6.0, 6.0, 3.0], [3.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[2.0, 8.0]], [[2.0, 1.0]], [[5.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 4.0]], [[6.0, 1.0]], [[4.0, 8.0]]]))
      arg[:3,1,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 8.0], [4.0, 2.0, 4.0], [5.0, 2.0, 8.0]], [[5.0, 3.0, 8.0], [8.0, 2.0, 4.0], [2.0, 
8.0, 5.0]], [[4.0, 8.0, 7.0], [2.0, 5.0, 4.0], [5.0, 1.0, 5.0]]], [[[2.0, 1.0, 4.0], [2.0, 1.0, 4.0], [4.0, 7.0, 8.0]], [[8.0, 
7.0, 6.0], [6.0, 4.0, 7.0], [2.0, 1.0, 5.0]], [[6.0, 2.0, 3.0], [7.0, 2.0, 3.0], [5.0, 3.0, 1.0]]], [[[6.0, 2.0, 4.0], [2.0, 
3.0, 1.0], [7.0, 2.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 1.0, 6.0], [5.0, 2.0, 8.0]], [[6.0, 2.0, 4.0], [6.0, 2.0, 4.0], [5.0, 6.0, 
4.0]]], [[[2.0, 3.0, 5.0], [1.0, 5.0, 1.0], [2.0, 1.0, 2.0]], [[3.0, 1.0, 7.0], [4.0, 1.0, 4.0], [8.0, 4.0, 2.0]], [[4.0, 6.0, 
6.0], [2.0, 7.0, 3.0], [7.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 4.0], [8.0, 8.0, 2.0], [1.0, 2.0, 4.0]], 
[[3.0, 1.0, 2.0], [8.0, 5.0, 6.0], [2.0, 4.0, 4.0]], [[6.0, 2.0, 6.0], [2.0, 4.0, 5.0], [3.0, 8.0, 8.0]]], [[[8.0, 3.0, 4.0], 
[5.0, 3.0, 2.0], [1.0, 7.0, 2.0]], [[1.0, 5.0, 4.0], [2.0, 8.0, 1.0], [6.0, 1.0, 5.0]], [[7.0, 6.0, 1.0], [1.0, 3.0, 3.0], 
[2.0, 4.0, 2.0]]], [[[4.0, 3.0, 1.0], [7.0, 7.0, 1.0], [7.0, 3.0, 3.0]], [[7.0, 7.0, 4.0], [2.0, 7.0, 8.0], [4.0, 8.0, 4.0]], 
[[1.0, 1.0, 1.0], [1.0, 7.0, 6.0], [3.0, 8.0, 2.0]]], [[[7.0, 3.0, 5.0], [5.0, 5.0, 5.0], [7.0, 5.0, 3.0]], [[4.0, 6.0, 7.0], 
[7.0, 8.0, 6.0], [1.0, 4.0, 7.0]], [[3.0, 2.0, 5.0], [6.0, 6.0, 3.0], [3.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LPLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 7.0], [2.0, 1.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 6.0, 3.0], [3.0, 8.0, 1.0], [8.0, 
5.0, 1.0]], [[1.0, 5.0, 6.0], [4.0, 4.0, 8.0], [4.0, 4.0, 1.0]]], [[[4.0, 1.0, 6.0], [1.0, 7.0, 3.0], [5.0, 6.0, 3.0]], [[2.0, 
4.0, 4.0], [5.0, 4.0, 7.0], [7.0, 8.0, 4.0]], [[7.0, 3.0, 6.0], [6.0, 6.0, 2.0], [5.0, 4.0, 6.0]]], [[[4.0, 4.0, 8.0], [3.0, 
7.0, 7.0], [2.0, 3.0, 6.0]], [[4.0, 2.0, 3.0], [3.0, 7.0, 3.0], [3.0, 3.0, 2.0]], [[8.0, 8.0, 6.0], [6.0, 3.0, 7.0], [3.0, 8.0, 
7.0]]], [[[7.0, 5.0, 8.0], [1.0, 5.0, 1.0], [1.0, 4.0, 7.0]], [[4.0, 7.0, 1.0], [3.0, 1.0, 2.0], [5.0, 4.0, 6.0]], [[2.0, 3.0, 
2.0], [3.0, 1.0, 2.0], [4.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 5.0], [2.0, 8.0, 2.0], [7.0, 3.0, 2.0]], 
[[6.0, 8.0, 7.0], [2.0, 5.0, 8.0], [6.0, 8.0, 5.0]], [[1.0, 2.0, 1.0], [3.0, 5.0, 8.0], [2.0, 5.0, 6.0]]], [[[5.0, 6.0, 2.0], 
[2.0, 5.0, 7.0], [1.0, 7.0, 5.0]], [[7.0, 6.0, 5.0], [1.0, 2.0, 3.0], [2.0, 5.0, 5.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 3.0], 
[3.0, 7.0, 1.0]]], [[[8.0, 2.0, 5.0], [2.0, 2.0, 8.0], [2.0, 1.0, 8.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 6.0], [3.0, 7.0, 4.0]], 
[[3.0, 7.0, 1.0], [2.0, 7.0, 8.0], [8.0, 3.0, 1.0]]], [[[7.0, 7.0, 7.0], [7.0, 7.0, 4.0], [3.0, 8.0, 8.0]], [[4.0, 5.0, 8.0], 
[3.0, 8.0, 8.0], [1.0, 2.0, 5.0]], [[6.0, 7.0, 2.0], [2.0, 6.0, 6.0], [7.0, 6.0, 6.0]]]])
      val=Data(numpy.array([[[5.0, 3.0, 1.0]], [[7.0, 1.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 4.0, 7.0]], [[1.0, 6.0, 5.0]]]))
      arg[2:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 7.0], [2.0, 1.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 6.0, 3.0], [3.0, 8.0, 1.0], [8.0, 
5.0, 1.0]], [[1.0, 5.0, 6.0], [4.0, 4.0, 8.0], [4.0, 4.0, 1.0]]], [[[4.0, 1.0, 6.0], [1.0, 7.0, 3.0], [5.0, 6.0, 3.0]], [[2.0, 
4.0, 4.0], [5.0, 4.0, 7.0], [7.0, 8.0, 4.0]], [[7.0, 3.0, 6.0], [6.0, 6.0, 2.0], [5.0, 4.0, 6.0]]], [[[4.0, 4.0, 8.0], [3.0, 
7.0, 7.0], [2.0, 3.0, 6.0]], [[4.0, 2.0, 3.0], [3.0, 7.0, 3.0], [5.0, 3.0, 1.0]], [[8.0, 8.0, 6.0], [6.0, 3.0, 7.0], [3.0, 8.0, 
7.0]]], [[[7.0, 5.0, 8.0], [1.0, 5.0, 1.0], [1.0, 4.0, 7.0]], [[4.0, 7.0, 1.0], [3.0, 1.0, 2.0], [7.0, 1.0, 6.0]], [[2.0, 3.0, 
2.0], [3.0, 1.0, 2.0], [4.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 5.0], [2.0, 8.0, 2.0], [7.0, 3.0, 2.0]], 
[[6.0, 8.0, 7.0], [2.0, 5.0, 8.0], [6.0, 8.0, 5.0]], [[1.0, 2.0, 1.0], [3.0, 5.0, 8.0], [2.0, 5.0, 6.0]]], [[[5.0, 6.0, 2.0], 
[2.0, 5.0, 7.0], [1.0, 7.0, 5.0]], [[7.0, 6.0, 5.0], [1.0, 2.0, 3.0], [2.0, 5.0, 5.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 3.0], 
[3.0, 7.0, 1.0]]], [[[8.0, 2.0, 5.0], [2.0, 2.0, 8.0], [2.0, 1.0, 8.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 6.0], [7.0, 4.0, 7.0]], 
[[3.0, 7.0, 1.0], [2.0, 7.0, 8.0], [8.0, 3.0, 1.0]]], [[[7.0, 7.0, 7.0], [7.0, 7.0, 4.0], [3.0, 8.0, 8.0]], [[4.0, 5.0, 8.0], 
[3.0, 8.0, 8.0], [1.0, 6.0, 5.0]], [[6.0, 7.0, 2.0], [2.0, 6.0, 6.0], [7.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UULU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 7.0], [4.0, 8.0, 7.0], [6.0, 6.0, 3.0]], [[2.0, 5.0, 8.0], [7.0, 1.0, 2.0], [5.0, 
7.0, 6.0]], [[5.0, 1.0, 7.0], [6.0, 1.0, 2.0], [6.0, 5.0, 7.0]]], [[[2.0, 8.0, 2.0], [2.0, 1.0, 5.0], [4.0, 2.0, 1.0]], [[8.0, 
5.0, 1.0], [1.0, 4.0, 4.0], [8.0, 2.0, 1.0]], [[1.0, 2.0, 5.0], [7.0, 3.0, 4.0], [8.0, 8.0, 7.0]]], [[[1.0, 3.0, 5.0], [3.0, 
8.0, 1.0], [1.0, 8.0, 1.0]], [[7.0, 8.0, 6.0], [7.0, 3.0, 8.0], [3.0, 3.0, 6.0]], [[2.0, 8.0, 5.0], [7.0, 1.0, 8.0], [2.0, 7.0, 
2.0]]], [[[4.0, 8.0, 4.0], [2.0, 1.0, 1.0], [2.0, 7.0, 7.0]], [[8.0, 3.0, 8.0], [1.0, 3.0, 5.0], [2.0, 8.0, 8.0]], [[6.0, 4.0, 
2.0], [1.0, 1.0, 3.0], [3.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 8.0], [6.0, 7.0, 3.0], [4.0, 3.0, 3.0]], 
[[8.0, 3.0, 5.0], [5.0, 5.0, 1.0], [5.0, 1.0, 4.0]], [[8.0, 8.0, 8.0], [1.0, 2.0, 6.0], [7.0, 7.0, 8.0]]], [[[5.0, 4.0, 8.0], 
[6.0, 2.0, 8.0], [1.0, 7.0, 7.0]], [[5.0, 6.0, 7.0], [1.0, 8.0, 3.0], [2.0, 3.0, 1.0]], [[6.0, 3.0, 2.0], [1.0, 7.0, 5.0], 
[8.0, 2.0, 1.0]]], [[[2.0, 8.0, 2.0], [3.0, 1.0, 6.0], [8.0, 7.0, 5.0]], [[1.0, 7.0, 5.0], [1.0, 6.0, 2.0], [5.0, 2.0, 7.0]], 
[[3.0, 3.0, 8.0], [1.0, 1.0, 4.0], [8.0, 8.0, 4.0]]], [[[3.0, 7.0, 3.0], [1.0, 8.0, 6.0], [8.0, 4.0, 6.0]], [[8.0, 2.0, 1.0], 
[2.0, 8.0, 8.0], [4.0, 8.0, 1.0]], [[3.0, 6.0, 4.0], [3.0, 8.0, 3.0], [6.0, 6.0, 1.0]]]])
      val=Data(numpy.array([[[[7.0, 7.0]], [[1.0, 3.0]]], [[[3.0, 7.0]], [[7.0, 1.0]]], [[[7.0, 3.0]], [[7.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 3.0]], [[7.0, 3.0]]], [[[8.0, 6.0]], [[8.0, 8.0]]], [[[5.0, 5.0]], [[7.0, 
8.0]]]]))
      arg[:3,:2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 7.0], [4.0, 8.0, 7.0], [7.0, 7.0, 3.0]], [[2.0, 5.0, 8.0], [7.0, 1.0, 2.0], [1.0, 
3.0, 6.0]], [[5.0, 1.0, 7.0], [6.0, 1.0, 2.0], [6.0, 5.0, 7.0]]], [[[2.0, 8.0, 2.0], [2.0, 1.0, 5.0], [3.0, 7.0, 1.0]], [[8.0, 
5.0, 1.0], [1.0, 4.0, 4.0], [7.0, 1.0, 1.0]], [[1.0, 2.0, 5.0], [7.0, 3.0, 4.0], [8.0, 8.0, 7.0]]], [[[1.0, 3.0, 5.0], [3.0, 
8.0, 1.0], [7.0, 3.0, 1.0]], [[7.0, 8.0, 6.0], [7.0, 3.0, 8.0], [7.0, 1.0, 6.0]], [[2.0, 8.0, 5.0], [7.0, 1.0, 8.0], [2.0, 7.0, 
2.0]]], [[[4.0, 8.0, 4.0], [2.0, 1.0, 1.0], [2.0, 7.0, 7.0]], [[8.0, 3.0, 8.0], [1.0, 3.0, 5.0], [2.0, 8.0, 8.0]], [[6.0, 4.0, 
2.0], [1.0, 1.0, 3.0], [3.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 8.0], [6.0, 7.0, 3.0], [2.0, 3.0, 3.0]], 
[[8.0, 3.0, 5.0], [5.0, 5.0, 1.0], [7.0, 3.0, 4.0]], [[8.0, 8.0, 8.0], [1.0, 2.0, 6.0], [7.0, 7.0, 8.0]]], [[[5.0, 4.0, 8.0], 
[6.0, 2.0, 8.0], [8.0, 6.0, 7.0]], [[5.0, 6.0, 7.0], [1.0, 8.0, 3.0], [8.0, 8.0, 1.0]], [[6.0, 3.0, 2.0], [1.0, 7.0, 5.0], 
[8.0, 2.0, 1.0]]], [[[2.0, 8.0, 2.0], [3.0, 1.0, 6.0], [5.0, 5.0, 5.0]], [[1.0, 7.0, 5.0], [1.0, 6.0, 2.0], [7.0, 8.0, 7.0]], 
[[3.0, 3.0, 8.0], [1.0, 1.0, 4.0], [8.0, 8.0, 4.0]]], [[[3.0, 7.0, 3.0], [1.0, 8.0, 6.0], [8.0, 4.0, 6.0]], [[8.0, 2.0, 1.0], 
[2.0, 8.0, 8.0], [4.0, 8.0, 1.0]], [[3.0, 6.0, 4.0], [3.0, 8.0, 3.0], [6.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 3.0], [7.0, 5.0, 1.0], [4.0, 5.0, 4.0]], [[6.0, 5.0, 4.0], [6.0, 5.0, 2.0], [3.0, 
3.0, 6.0]], [[7.0, 1.0, 4.0], [6.0, 7.0, 6.0], [6.0, 1.0, 4.0]]], [[[6.0, 8.0, 7.0], [1.0, 4.0, 2.0], [6.0, 3.0, 2.0]], [[6.0, 
2.0, 2.0], [5.0, 8.0, 4.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 8.0], [7.0, 1.0, 2.0], [5.0, 8.0, 4.0]]], [[[7.0, 4.0, 6.0], [3.0, 
1.0, 5.0], [2.0, 4.0, 3.0]], [[3.0, 2.0, 8.0], [7.0, 8.0, 1.0], [3.0, 1.0, 5.0]], [[5.0, 1.0, 7.0], [6.0, 2.0, 8.0], [1.0, 3.0, 
7.0]]], [[[5.0, 5.0, 6.0], [7.0, 8.0, 6.0], [2.0, 8.0, 8.0]], [[2.0, 8.0, 5.0], [7.0, 2.0, 8.0], [5.0, 3.0, 5.0]], [[2.0, 1.0, 
3.0], [6.0, 6.0, 8.0], [2.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 4.0], [5.0, 3.0, 2.0], [8.0, 4.0, 4.0]], 
[[3.0, 8.0, 2.0], [6.0, 5.0, 6.0], [2.0, 6.0, 7.0]], [[8.0, 1.0, 8.0], [8.0, 3.0, 1.0], [1.0, 6.0, 6.0]]], [[[5.0, 1.0, 8.0], 
[1.0, 4.0, 6.0], [6.0, 6.0, 3.0]], [[8.0, 4.0, 8.0], [2.0, 7.0, 1.0], [7.0, 2.0, 7.0]], [[2.0, 2.0, 4.0], [7.0, 6.0, 1.0], 
[4.0, 5.0, 4.0]]], [[[2.0, 5.0, 7.0], [3.0, 6.0, 1.0], [8.0, 5.0, 8.0]], [[1.0, 8.0, 6.0], [4.0, 5.0, 3.0], [3.0, 7.0, 5.0]], 
[[5.0, 1.0, 1.0], [3.0, 3.0, 5.0], [8.0, 3.0, 7.0]]], [[[7.0, 1.0, 2.0], [3.0, 8.0, 5.0], [7.0, 2.0, 7.0]], [[8.0, 4.0, 8.0], 
[3.0, 1.0, 1.0], [3.0, 8.0, 3.0]], [[4.0, 1.0, 4.0], [6.0, 7.0, 5.0], [7.0, 1.0, 1.0]]]])
      val=Data(numpy.array([[[[2.0, 6.0, 1.0], [3.0, 5.0, 8.0]]], [[[2.0, 6.0, 1.0], [8.0, 1.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 2.0, 7.0], [4.0, 1.0, 1.0]]], [[[5.0, 2.0, 5.0], [8.0, 2.0, 6.0]]]]))
      arg[2:,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [7.0, 5.0, 1.0], [4.0, 5.0, 4.0]], [[6.0, 5.0, 4.0], [6.0, 5.0, 2.0], [3.0, 
3.0, 6.0]], [[7.0, 1.0, 4.0], [6.0, 7.0, 6.0], [6.0, 1.0, 4.0]]], [[[6.0, 8.0, 7.0], [1.0, 4.0, 2.0], [6.0, 3.0, 2.0]], [[6.0, 
2.0, 2.0], [5.0, 8.0, 4.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 8.0], [7.0, 1.0, 2.0], [5.0, 8.0, 4.0]]], [[[7.0, 4.0, 6.0], [3.0, 
1.0, 5.0], [2.0, 4.0, 3.0]], [[2.0, 6.0, 1.0], [3.0, 5.0, 8.0], [3.0, 1.0, 5.0]], [[5.0, 1.0, 7.0], [6.0, 2.0, 8.0], [1.0, 3.0, 
7.0]]], [[[5.0, 5.0, 6.0], [7.0, 8.0, 6.0], [2.0, 8.0, 8.0]], [[2.0, 6.0, 1.0], [8.0, 1.0, 3.0], [5.0, 3.0, 5.0]], [[2.0, 1.0, 
3.0], [6.0, 6.0, 8.0], [2.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 4.0], [5.0, 3.0, 2.0], [8.0, 4.0, 4.0]], 
[[3.0, 8.0, 2.0], [6.0, 5.0, 6.0], [2.0, 6.0, 7.0]], [[8.0, 1.0, 8.0], [8.0, 3.0, 1.0], [1.0, 6.0, 6.0]]], [[[5.0, 1.0, 8.0], 
[1.0, 4.0, 6.0], [6.0, 6.0, 3.0]], [[8.0, 4.0, 8.0], [2.0, 7.0, 1.0], [7.0, 2.0, 7.0]], [[2.0, 2.0, 4.0], [7.0, 6.0, 1.0], 
[4.0, 5.0, 4.0]]], [[[2.0, 5.0, 7.0], [3.0, 6.0, 1.0], [8.0, 5.0, 8.0]], [[4.0, 2.0, 7.0], [4.0, 1.0, 1.0], [3.0, 7.0, 5.0]], 
[[5.0, 1.0, 1.0], [3.0, 3.0, 5.0], [8.0, 3.0, 7.0]]], [[[7.0, 1.0, 2.0], [3.0, 8.0, 5.0], [7.0, 2.0, 7.0]], [[5.0, 2.0, 5.0], 
[8.0, 2.0, 6.0], [3.0, 8.0, 3.0]], [[4.0, 1.0, 4.0], [6.0, 7.0, 5.0], [7.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 7.0], [8.0, 3.0, 5.0], [2.0, 5.0, 7.0]], [[3.0, 3.0, 5.0], [7.0, 1.0, 8.0], [7.0, 
4.0, 8.0]], [[2.0, 5.0, 7.0], [4.0, 6.0, 7.0], [6.0, 8.0, 1.0]]], [[[5.0, 6.0, 5.0], [3.0, 7.0, 6.0], [6.0, 1.0, 7.0]], [[2.0, 
7.0, 2.0], [6.0, 2.0, 2.0], [3.0, 1.0, 4.0]], [[5.0, 8.0, 5.0], [5.0, 4.0, 8.0], [8.0, 4.0, 6.0]]], [[[4.0, 1.0, 6.0], [5.0, 
2.0, 2.0], [5.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [3.0, 6.0, 3.0], [6.0, 3.0, 4.0]], [[3.0, 7.0, 4.0], [7.0, 1.0, 5.0], [2.0, 4.0, 
8.0]]], [[[7.0, 5.0, 2.0], [1.0, 3.0, 6.0], [7.0, 8.0, 8.0]], [[4.0, 4.0, 1.0], [2.0, 6.0, 7.0], [6.0, 8.0, 1.0]], [[5.0, 3.0, 
1.0], [6.0, 4.0, 5.0], [6.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 7.0], [4.0, 7.0, 2.0], [5.0, 2.0, 1.0]], 
[[3.0, 5.0, 3.0], [1.0, 8.0, 5.0], [1.0, 4.0, 1.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 3.0], [7.0, 7.0, 2.0]]], [[[1.0, 6.0, 1.0], 
[8.0, 3.0, 2.0], [4.0, 8.0, 2.0]], [[7.0, 5.0, 6.0], [4.0, 1.0, 7.0], [5.0, 7.0, 4.0]], [[2.0, 3.0, 7.0], [5.0, 6.0, 1.0], 
[5.0, 2.0, 5.0]]], [[[1.0, 7.0, 3.0], [8.0, 8.0, 4.0], [3.0, 5.0, 2.0]], [[7.0, 4.0, 1.0], [7.0, 7.0, 1.0], [8.0, 8.0, 3.0]], 
[[2.0, 6.0, 4.0], [7.0, 3.0, 7.0], [5.0, 6.0, 2.0]]], [[[1.0, 3.0, 4.0], [5.0, 7.0, 3.0], [4.0, 8.0, 5.0]], [[6.0, 5.0, 6.0], 
[2.0, 7.0, 7.0], [1.0, 1.0, 7.0]], [[8.0, 4.0, 1.0], [1.0, 5.0, 1.0], [5.0, 4.0, 7.0]]]])
      val=Data(numpy.array([[[[5.0, 2.0, 6.0]]], [[[8.0, 6.0, 8.0]]], [[[2.0, 1.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 7.0]]], [[[7.0, 4.0, 5.0]]], [[[4.0, 4.0, 5.0]]]]))
      arg[:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 7.0], [8.0, 3.0, 5.0], [2.0, 5.0, 7.0]], [[3.0, 3.0, 5.0], [7.0, 1.0, 8.0], [5.0, 
2.0, 6.0]], [[2.0, 5.0, 7.0], [4.0, 6.0, 7.0], [6.0, 8.0, 1.0]]], [[[5.0, 6.0, 5.0], [3.0, 7.0, 6.0], [6.0, 1.0, 7.0]], [[2.0, 
7.0, 2.0], [6.0, 2.0, 2.0], [8.0, 6.0, 8.0]], [[5.0, 8.0, 5.0], [5.0, 4.0, 8.0], [8.0, 4.0, 6.0]]], [[[4.0, 1.0, 6.0], [5.0, 
2.0, 2.0], [5.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [3.0, 6.0, 3.0], [2.0, 1.0, 8.0]], [[3.0, 7.0, 4.0], [7.0, 1.0, 5.0], [2.0, 4.0, 
8.0]]], [[[7.0, 5.0, 2.0], [1.0, 3.0, 6.0], [7.0, 8.0, 8.0]], [[4.0, 4.0, 1.0], [2.0, 6.0, 7.0], [6.0, 8.0, 1.0]], [[5.0, 3.0, 
1.0], [6.0, 4.0, 5.0], [6.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 7.0], [4.0, 7.0, 2.0], [5.0, 2.0, 1.0]], 
[[3.0, 5.0, 3.0], [1.0, 8.0, 5.0], [3.0, 7.0, 7.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 3.0], [7.0, 7.0, 2.0]]], [[[1.0, 6.0, 1.0], 
[8.0, 3.0, 2.0], [4.0, 8.0, 2.0]], [[7.0, 5.0, 6.0], [4.0, 1.0, 7.0], [7.0, 4.0, 5.0]], [[2.0, 3.0, 7.0], [5.0, 6.0, 1.0], 
[5.0, 2.0, 5.0]]], [[[1.0, 7.0, 3.0], [8.0, 8.0, 4.0], [3.0, 5.0, 2.0]], [[7.0, 4.0, 1.0], [7.0, 7.0, 1.0], [4.0, 4.0, 5.0]], 
[[2.0, 6.0, 4.0], [7.0, 3.0, 7.0], [5.0, 6.0, 2.0]]], [[[1.0, 3.0, 4.0], [5.0, 7.0, 3.0], [4.0, 8.0, 5.0]], [[6.0, 5.0, 6.0], 
[2.0, 7.0, 7.0], [1.0, 1.0, 7.0]], [[8.0, 4.0, 1.0], [1.0, 5.0, 1.0], [5.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 8.0], [2.0, 4.0, 1.0], [8.0, 6.0, 8.0]], [[6.0, 8.0, 7.0], [4.0, 8.0, 1.0], [2.0, 
8.0, 4.0]], [[1.0, 6.0, 1.0], [5.0, 5.0, 7.0], [6.0, 3.0, 5.0]]], [[[6.0, 1.0, 3.0], [5.0, 8.0, 7.0], [1.0, 8.0, 4.0]], [[4.0, 
5.0, 6.0], [4.0, 6.0, 7.0], [1.0, 5.0, 4.0]], [[6.0, 1.0, 2.0], [3.0, 6.0, 2.0], [2.0, 4.0, 3.0]]], [[[2.0, 7.0, 2.0], [3.0, 
2.0, 3.0], [3.0, 6.0, 4.0]], [[7.0, 2.0, 2.0], [7.0, 6.0, 1.0], [6.0, 2.0, 4.0]], [[1.0, 8.0, 3.0], [1.0, 3.0, 7.0], [7.0, 2.0, 
1.0]]], [[[5.0, 3.0, 6.0], [5.0, 8.0, 7.0], [3.0, 4.0, 1.0]], [[3.0, 7.0, 4.0], [3.0, 3.0, 4.0], [4.0, 8.0, 6.0]], [[7.0, 3.0, 
7.0], [8.0, 5.0, 5.0], [5.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 8.0], [1.0, 5.0, 8.0], [1.0, 7.0, 8.0]], 
[[2.0, 4.0, 6.0], [4.0, 4.0, 4.0], [2.0, 4.0, 8.0]], [[3.0, 1.0, 3.0], [5.0, 8.0, 6.0], [3.0, 6.0, 8.0]]], [[[6.0, 3.0, 6.0], 
[4.0, 6.0, 5.0], [4.0, 2.0, 7.0]], [[6.0, 4.0, 3.0], [1.0, 5.0, 5.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 1.0], [7.0, 1.0, 3.0], 
[5.0, 2.0, 3.0]]], [[[8.0, 7.0, 7.0], [5.0, 5.0, 8.0], [3.0, 2.0, 6.0]], [[5.0, 4.0, 8.0], [4.0, 1.0, 8.0], [7.0, 3.0, 7.0]], 
[[5.0, 5.0, 5.0], [4.0, 3.0, 7.0], [8.0, 4.0, 7.0]]], [[[4.0, 6.0, 4.0], [3.0, 5.0, 8.0], [8.0, 2.0, 5.0]], [[7.0, 5.0, 2.0], 
[1.0, 6.0, 7.0], [5.0, 5.0, 5.0]], [[4.0, 4.0, 4.0], [4.0, 8.0, 2.0], [7.0, 2.0, 4.0]]]])
      val=Data(numpy.array([[[[4.0]]], [[[8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0]]], [[[2.0]]]]))
      arg[2:,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 8.0], [2.0, 4.0, 1.0], [8.0, 6.0, 8.0]], [[6.0, 8.0, 7.0], [4.0, 8.0, 1.0], [2.0, 
8.0, 4.0]], [[1.0, 6.0, 1.0], [5.0, 5.0, 7.0], [6.0, 3.0, 5.0]]], [[[6.0, 1.0, 3.0], [5.0, 8.0, 7.0], [1.0, 8.0, 4.0]], [[4.0, 
5.0, 6.0], [4.0, 6.0, 7.0], [1.0, 5.0, 4.0]], [[6.0, 1.0, 2.0], [3.0, 6.0, 2.0], [2.0, 4.0, 3.0]]], [[[2.0, 7.0, 2.0], [3.0, 
2.0, 3.0], [3.0, 6.0, 4.0]], [[7.0, 2.0, 2.0], [7.0, 4.0, 1.0], [6.0, 2.0, 4.0]], [[1.0, 8.0, 3.0], [1.0, 3.0, 7.0], [7.0, 2.0, 
1.0]]], [[[5.0, 3.0, 6.0], [5.0, 8.0, 7.0], [3.0, 4.0, 1.0]], [[3.0, 7.0, 4.0], [3.0, 8.0, 4.0], [4.0, 8.0, 6.0]], [[7.0, 3.0, 
7.0], [8.0, 5.0, 5.0], [5.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 8.0], [1.0, 5.0, 8.0], [1.0, 7.0, 8.0]], 
[[2.0, 4.0, 6.0], [4.0, 4.0, 4.0], [2.0, 4.0, 8.0]], [[3.0, 1.0, 3.0], [5.0, 8.0, 6.0], [3.0, 6.0, 8.0]]], [[[6.0, 3.0, 6.0], 
[4.0, 6.0, 5.0], [4.0, 2.0, 7.0]], [[6.0, 4.0, 3.0], [1.0, 5.0, 5.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 1.0], [7.0, 1.0, 3.0], 
[5.0, 2.0, 3.0]]], [[[8.0, 7.0, 7.0], [5.0, 5.0, 8.0], [3.0, 2.0, 6.0]], [[5.0, 4.0, 8.0], [4.0, 8.0, 8.0], [7.0, 3.0, 7.0]], 
[[5.0, 5.0, 5.0], [4.0, 3.0, 7.0], [8.0, 4.0, 7.0]]], [[[4.0, 6.0, 4.0], [3.0, 5.0, 8.0], [8.0, 2.0, 5.0]], [[7.0, 5.0, 2.0], 
[1.0, 2.0, 7.0], [5.0, 5.0, 5.0]], [[4.0, 4.0, 4.0], [4.0, 8.0, 2.0], [7.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 2.0], [3.0, 3.0, 3.0], [7.0, 5.0, 2.0]], [[8.0, 3.0, 1.0], [1.0, 6.0, 7.0], [3.0, 
7.0, 7.0]], [[4.0, 7.0, 1.0], [5.0, 5.0, 7.0], [7.0, 2.0, 8.0]]], [[[5.0, 7.0, 5.0], [4.0, 7.0, 8.0], [4.0, 6.0, 5.0]], [[5.0, 
8.0, 3.0], [1.0, 2.0, 3.0], [7.0, 3.0, 7.0]], [[4.0, 1.0, 3.0], [3.0, 4.0, 1.0], [7.0, 7.0, 6.0]]], [[[7.0, 5.0, 6.0], [2.0, 
7.0, 2.0], [1.0, 7.0, 1.0]], [[4.0, 2.0, 5.0], [1.0, 4.0, 1.0], [4.0, 2.0, 4.0]], [[4.0, 4.0, 1.0], [7.0, 5.0, 2.0], [2.0, 1.0, 
6.0]]], [[[2.0, 4.0, 1.0], [2.0, 7.0, 8.0], [6.0, 1.0, 8.0]], [[6.0, 2.0, 8.0], [1.0, 4.0, 6.0], [5.0, 8.0, 2.0]], [[2.0, 8.0, 
7.0], [2.0, 7.0, 6.0], [8.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 2.0], [3.0, 8.0, 6.0], [6.0, 2.0, 1.0]], 
[[6.0, 6.0, 3.0], [6.0, 4.0, 2.0], [4.0, 2.0, 3.0]], [[7.0, 7.0, 7.0], [8.0, 5.0, 3.0], [7.0, 4.0, 3.0]]], [[[7.0, 1.0, 2.0], 
[7.0, 7.0, 5.0], [1.0, 2.0, 4.0]], [[3.0, 1.0, 5.0], [1.0, 6.0, 7.0], [7.0, 7.0, 7.0]], [[5.0, 6.0, 8.0], [5.0, 2.0, 3.0], 
[5.0, 2.0, 1.0]]], [[[2.0, 5.0, 4.0], [5.0, 5.0, 8.0], [3.0, 5.0, 1.0]], [[5.0, 5.0, 7.0], [1.0, 2.0, 4.0], [4.0, 4.0, 7.0]], 
[[2.0, 7.0, 1.0], [7.0, 4.0, 3.0], [5.0, 6.0, 1.0]]], [[[7.0, 1.0, 5.0], [6.0, 6.0, 3.0], [2.0, 8.0, 6.0]], [[7.0, 2.0, 3.0], 
[8.0, 8.0, 3.0], [3.0, 4.0, 8.0]], [[5.0, 4.0, 1.0], [2.0, 3.0, 5.0], [5.0, 4.0, 1.0]]]])
      val=Data(numpy.array([[[[3.0, 3.0, 2.0], [7.0, 8.0, 5.0], [4.0, 1.0, 5.0]]], [[[6.0, 7.0, 6.0], [6.0, 5.0, 1.0], [3.0, 
8.0, 1.0]]], [[[7.0, 1.0, 2.0], [8.0, 2.0, 6.0], [6.0, 3.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 3.0], [7.0, 1.0, 1.0], [6.0, 5.0, 8.0]]], [[[2.0, 6.0, 3.0], [8.0, 1.0, 
2.0], [2.0, 1.0, 5.0]]], [[[4.0, 6.0, 8.0], [7.0, 6.0, 5.0], [4.0, 7.0, 2.0]]]]))
      arg[:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 2.0], [3.0, 3.0, 3.0], [7.0, 5.0, 2.0]], [[8.0, 3.0, 1.0], [1.0, 6.0, 7.0], [3.0, 
7.0, 7.0]], [[3.0, 3.0, 2.0], [7.0, 8.0, 5.0], [4.0, 1.0, 5.0]]], [[[5.0, 7.0, 5.0], [4.0, 7.0, 8.0], [4.0, 6.0, 5.0]], [[5.0, 
8.0, 3.0], [1.0, 2.0, 3.0], [7.0, 3.0, 7.0]], [[6.0, 7.0, 6.0], [6.0, 5.0, 1.0], [3.0, 8.0, 1.0]]], [[[7.0, 5.0, 6.0], [2.0, 
7.0, 2.0], [1.0, 7.0, 1.0]], [[4.0, 2.0, 5.0], [1.0, 4.0, 1.0], [4.0, 2.0, 4.0]], [[7.0, 1.0, 2.0], [8.0, 2.0, 6.0], [6.0, 3.0, 
5.0]]], [[[2.0, 4.0, 1.0], [2.0, 7.0, 8.0], [6.0, 1.0, 8.0]], [[6.0, 2.0, 8.0], [1.0, 4.0, 6.0], [5.0, 8.0, 2.0]], [[2.0, 8.0, 
7.0], [2.0, 7.0, 6.0], [8.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 2.0], [3.0, 8.0, 6.0], [6.0, 2.0, 1.0]], 
[[6.0, 6.0, 3.0], [6.0, 4.0, 2.0], [4.0, 2.0, 3.0]], [[2.0, 6.0, 3.0], [7.0, 1.0, 1.0], [6.0, 5.0, 8.0]]], [[[7.0, 1.0, 2.0], 
[7.0, 7.0, 5.0], [1.0, 2.0, 4.0]], [[3.0, 1.0, 5.0], [1.0, 6.0, 7.0], [7.0, 7.0, 7.0]], [[2.0, 6.0, 3.0], [8.0, 1.0, 2.0], 
[2.0, 1.0, 5.0]]], [[[2.0, 5.0, 4.0], [5.0, 5.0, 8.0], [3.0, 5.0, 1.0]], [[5.0, 5.0, 7.0], [1.0, 2.0, 4.0], [4.0, 4.0, 7.0]], 
[[4.0, 6.0, 8.0], [7.0, 6.0, 5.0], [4.0, 7.0, 2.0]]], [[[7.0, 1.0, 5.0], [6.0, 6.0, 3.0], [2.0, 8.0, 6.0]], [[7.0, 2.0, 3.0], 
[8.0, 8.0, 3.0], [3.0, 4.0, 8.0]], [[5.0, 4.0, 1.0], [2.0, 3.0, 5.0], [5.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RLPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 6.0], [8.0, 1.0, 2.0], [4.0, 2.0, 2.0]], [[3.0, 2.0, 8.0], [2.0, 3.0, 1.0], [3.0, 
3.0, 3.0]], [[7.0, 2.0, 4.0], [4.0, 5.0, 8.0], [2.0, 8.0, 7.0]]], [[[5.0, 3.0, 1.0], [4.0, 5.0, 3.0], [7.0, 7.0, 3.0]], [[3.0, 
8.0, 8.0], [1.0, 6.0, 7.0], [4.0, 3.0, 4.0]], [[1.0, 8.0, 7.0], [7.0, 6.0, 7.0], [7.0, 6.0, 2.0]]], [[[7.0, 3.0, 3.0], [2.0, 
5.0, 6.0], [2.0, 4.0, 5.0]], [[8.0, 7.0, 7.0], [6.0, 6.0, 8.0], [5.0, 5.0, 2.0]], [[7.0, 6.0, 1.0], [1.0, 3.0, 7.0], [8.0, 5.0, 
6.0]]], [[[2.0, 4.0, 4.0], [2.0, 7.0, 1.0], [1.0, 4.0, 1.0]], [[4.0, 6.0, 8.0], [7.0, 2.0, 3.0], [6.0, 3.0, 5.0]], [[2.0, 6.0, 
1.0], [5.0, 1.0, 5.0], [6.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 1.0], [6.0, 8.0, 5.0], [7.0, 2.0, 4.0]], 
[[6.0, 6.0, 1.0], [8.0, 1.0, 3.0], [7.0, 6.0, 1.0]], [[1.0, 4.0, 5.0], [4.0, 5.0, 2.0], [5.0, 2.0, 8.0]]], [[[1.0, 1.0, 7.0], 
[6.0, 4.0, 6.0], [2.0, 5.0, 8.0]], [[3.0, 5.0, 2.0], [7.0, 1.0, 6.0], [2.0, 2.0, 6.0]], [[3.0, 8.0, 2.0], [2.0, 1.0, 1.0], 
[6.0, 5.0, 1.0]]], [[[5.0, 6.0, 3.0], [1.0, 5.0, 1.0], [7.0, 4.0, 3.0]], [[6.0, 8.0, 7.0], [8.0, 3.0, 8.0], [3.0, 3.0, 2.0]], 
[[3.0, 6.0, 4.0], [1.0, 6.0, 7.0], [1.0, 8.0, 8.0]]], [[[3.0, 7.0, 5.0], [5.0, 6.0, 2.0], [6.0, 2.0, 2.0]], [[1.0, 4.0, 1.0], 
[3.0, 8.0, 5.0], [4.0, 3.0, 1.0]], [[3.0, 5.0, 7.0], [5.0, 1.0, 3.0], [5.0, 6.0, 8.0]]]])
      val=Data(numpy.array([[[2.0, 2.0, 5.0]], [[4.0, 3.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 3.0, 5.0]], [[6.0, 3.0, 8.0]]]))
      arg[1:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 6.0], [8.0, 1.0, 2.0], [4.0, 2.0, 2.0]], [[3.0, 2.0, 8.0], [2.0, 3.0, 1.0], [3.0, 
3.0, 3.0]], [[7.0, 2.0, 4.0], [4.0, 5.0, 8.0], [2.0, 8.0, 7.0]]], [[[5.0, 3.0, 1.0], [4.0, 5.0, 3.0], [7.0, 7.0, 3.0]], [[3.0, 
8.0, 8.0], [1.0, 6.0, 7.0], [4.0, 3.0, 4.0]], [[1.0, 8.0, 7.0], [2.0, 2.0, 5.0], [7.0, 6.0, 2.0]]], [[[7.0, 3.0, 3.0], [2.0, 
5.0, 6.0], [2.0, 4.0, 5.0]], [[8.0, 7.0, 7.0], [6.0, 6.0, 8.0], [5.0, 5.0, 2.0]], [[7.0, 6.0, 1.0], [4.0, 3.0, 4.0], [8.0, 5.0, 
6.0]]], [[[2.0, 4.0, 4.0], [2.0, 7.0, 1.0], [1.0, 4.0, 1.0]], [[4.0, 6.0, 8.0], [7.0, 2.0, 3.0], [6.0, 3.0, 5.0]], [[2.0, 6.0, 
1.0], [5.0, 1.0, 5.0], [6.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 1.0], [6.0, 8.0, 5.0], [7.0, 2.0, 4.0]], 
[[6.0, 6.0, 1.0], [8.0, 1.0, 3.0], [7.0, 6.0, 1.0]], [[1.0, 4.0, 5.0], [4.0, 5.0, 2.0], [5.0, 2.0, 8.0]]], [[[1.0, 1.0, 7.0], 
[6.0, 4.0, 6.0], [2.0, 5.0, 8.0]], [[3.0, 5.0, 2.0], [7.0, 1.0, 6.0], [2.0, 2.0, 6.0]], [[3.0, 8.0, 2.0], [2.0, 3.0, 5.0], 
[6.0, 5.0, 1.0]]], [[[5.0, 6.0, 3.0], [1.0, 5.0, 1.0], [7.0, 4.0, 3.0]], [[6.0, 8.0, 7.0], [8.0, 3.0, 8.0], [3.0, 3.0, 2.0]], 
[[3.0, 6.0, 4.0], [6.0, 3.0, 8.0], [1.0, 8.0, 8.0]]], [[[3.0, 7.0, 5.0], [5.0, 6.0, 2.0], [6.0, 2.0, 2.0]], [[1.0, 4.0, 1.0], 
[3.0, 8.0, 5.0], [4.0, 3.0, 1.0]], [[3.0, 5.0, 7.0], [5.0, 1.0, 3.0], [5.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CLCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 4.0], [6.0, 5.0, 4.0], [6.0, 8.0, 8.0]], [[7.0, 5.0, 5.0], [7.0, 5.0, 5.0], [6.0, 
1.0, 8.0]], [[7.0, 4.0, 2.0], [7.0, 3.0, 7.0], [7.0, 4.0, 4.0]]], [[[7.0, 6.0, 1.0], [8.0, 7.0, 5.0], [7.0, 7.0, 1.0]], [[2.0, 
3.0, 5.0], [6.0, 7.0, 3.0], [4.0, 8.0, 2.0]], [[3.0, 8.0, 2.0], [1.0, 8.0, 8.0], [7.0, 7.0, 3.0]]], [[[6.0, 1.0, 8.0], [4.0, 
6.0, 3.0], [8.0, 3.0, 1.0]], [[4.0, 2.0, 2.0], [8.0, 7.0, 4.0], [7.0, 3.0, 4.0]], [[6.0, 5.0, 2.0], [6.0, 8.0, 5.0], [2.0, 8.0, 
6.0]]], [[[7.0, 6.0, 3.0], [7.0, 4.0, 3.0], [1.0, 3.0, 7.0]], [[6.0, 2.0, 5.0], [8.0, 8.0, 5.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 
4.0], [1.0, 8.0, 5.0], [8.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 3.0, 7.0], [8.0, 3.0, 5.0], [3.0, 2.0, 1.0]], 
[[3.0, 7.0, 7.0], [8.0, 6.0, 8.0], [5.0, 2.0, 4.0]], [[6.0, 3.0, 5.0], [6.0, 8.0, 8.0], [5.0, 4.0, 7.0]]], [[[4.0, 2.0, 4.0], 
[2.0, 1.0, 7.0], [5.0, 4.0, 4.0]], [[8.0, 3.0, 4.0], [4.0, 4.0, 7.0], [6.0, 4.0, 3.0]], [[2.0, 3.0, 1.0], [7.0, 1.0, 7.0], 
[5.0, 4.0, 3.0]]], [[[4.0, 4.0, 7.0], [6.0, 4.0, 6.0], [4.0, 3.0, 3.0]], [[6.0, 8.0, 2.0], [8.0, 7.0, 7.0], [7.0, 4.0, 6.0]], 
[[5.0, 6.0, 2.0], [5.0, 6.0, 2.0], [4.0, 7.0, 7.0]]], [[[1.0, 8.0, 7.0], [2.0, 7.0, 2.0], [2.0, 1.0, 4.0]], [[1.0, 7.0, 3.0], 
[3.0, 3.0, 6.0], [6.0, 6.0, 7.0]], [[2.0, 4.0, 6.0], [8.0, 2.0, 2.0], [1.0, 8.0, 5.0]]]])
      val=Data(numpy.array([[[[2.0, 7.0, 2.0], [8.0, 8.0, 2.0], [6.0, 4.0, 5.0]]], [[[2.0, 2.0, 8.0], [3.0, 6.0, 5.0], [2.0, 
5.0, 8.0]]], [[[3.0, 2.0, 2.0], [3.0, 5.0, 7.0], [3.0, 2.0, 5.0]]], [[[4.0, 7.0, 1.0], [7.0, 2.0, 4.0], [5.0, 4.0, 
3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 7.0], [4.0, 1.0, 3.0], [1.0, 6.0, 7.0]]], [[[3.0, 7.0, 6.0], [5.0, 3.0, 
5.0], [5.0, 7.0, 3.0]]], [[[6.0, 8.0, 1.0], [1.0, 8.0, 6.0], [8.0, 5.0, 3.0]]], [[[1.0, 3.0, 3.0], [4.0, 5.0, 6.0], [5.0, 2.0, 
2.0]]]]))
      arg[:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 4.0], [6.0, 5.0, 4.0], [6.0, 8.0, 8.0]], [[7.0, 5.0, 5.0], [7.0, 5.0, 5.0], [6.0, 
1.0, 8.0]], [[2.0, 7.0, 2.0], [8.0, 8.0, 2.0], [6.0, 4.0, 5.0]]], [[[7.0, 6.0, 1.0], [8.0, 7.0, 5.0], [7.0, 7.0, 1.0]], [[2.0, 
3.0, 5.0], [6.0, 7.0, 3.0], [4.0, 8.0, 2.0]], [[2.0, 2.0, 8.0], [3.0, 6.0, 5.0], [2.0, 5.0, 8.0]]], [[[6.0, 1.0, 8.0], [4.0, 
6.0, 3.0], [8.0, 3.0, 1.0]], [[4.0, 2.0, 2.0], [8.0, 7.0, 4.0], [7.0, 3.0, 4.0]], [[3.0, 2.0, 2.0], [3.0, 5.0, 7.0], [3.0, 2.0, 
5.0]]], [[[7.0, 6.0, 3.0], [7.0, 4.0, 3.0], [1.0, 3.0, 7.0]], [[6.0, 2.0, 5.0], [8.0, 8.0, 5.0], [7.0, 6.0, 8.0]], [[4.0, 7.0, 
1.0], [7.0, 2.0, 4.0], [5.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 7.0], [8.0, 3.0, 5.0], [3.0, 2.0, 1.0]], 
[[3.0, 7.0, 7.0], [8.0, 6.0, 8.0], [5.0, 2.0, 4.0]], [[7.0, 7.0, 7.0], [4.0, 1.0, 3.0], [1.0, 6.0, 7.0]]], [[[4.0, 2.0, 4.0], 
[2.0, 1.0, 7.0], [5.0, 4.0, 4.0]], [[8.0, 3.0, 4.0], [4.0, 4.0, 7.0], [6.0, 4.0, 3.0]], [[3.0, 7.0, 6.0], [5.0, 3.0, 5.0], 
[5.0, 7.0, 3.0]]], [[[4.0, 4.0, 7.0], [6.0, 4.0, 6.0], [4.0, 3.0, 3.0]], [[6.0, 8.0, 2.0], [8.0, 7.0, 7.0], [7.0, 4.0, 6.0]], 
[[6.0, 8.0, 1.0], [1.0, 8.0, 6.0], [8.0, 5.0, 3.0]]], [[[1.0, 8.0, 7.0], [2.0, 7.0, 2.0], [2.0, 1.0, 4.0]], [[1.0, 7.0, 3.0], 
[3.0, 3.0, 6.0], [6.0, 6.0, 7.0]], [[1.0, 3.0, 3.0], [4.0, 5.0, 6.0], [5.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 1.0], [4.0, 8.0, 5.0], [4.0, 3.0, 1.0]], [[4.0, 3.0, 3.0], [7.0, 8.0, 1.0], [7.0, 
4.0, 3.0]], [[3.0, 3.0, 2.0], [2.0, 4.0, 6.0], [5.0, 4.0, 3.0]]], [[[6.0, 7.0, 4.0], [5.0, 8.0, 1.0], [3.0, 7.0, 2.0]], [[8.0, 
2.0, 7.0], [2.0, 2.0, 8.0], [3.0, 7.0, 4.0]], [[3.0, 3.0, 7.0], [8.0, 4.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 8.0, 2.0], [3.0, 
7.0, 7.0], [8.0, 1.0, 1.0]], [[6.0, 5.0, 2.0], [4.0, 1.0, 4.0], [4.0, 4.0, 1.0]], [[8.0, 3.0, 4.0], [6.0, 4.0, 6.0], [4.0, 3.0, 
1.0]]], [[[4.0, 4.0, 1.0], [4.0, 7.0, 5.0], [3.0, 4.0, 2.0]], [[7.0, 3.0, 2.0], [6.0, 1.0, 1.0], [6.0, 8.0, 8.0]], [[7.0, 7.0, 
2.0], [8.0, 6.0, 2.0], [2.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 5.0], [1.0, 7.0, 3.0], [4.0, 2.0, 5.0]], 
[[4.0, 8.0, 4.0], [7.0, 6.0, 3.0], [3.0, 7.0, 5.0]], [[2.0, 2.0, 8.0], [1.0, 8.0, 6.0], [2.0, 6.0, 4.0]]], [[[6.0, 4.0, 8.0], 
[5.0, 3.0, 5.0], [1.0, 5.0, 6.0]], [[6.0, 7.0, 8.0], [3.0, 6.0, 4.0], [8.0, 6.0, 7.0]], [[1.0, 3.0, 2.0], [7.0, 1.0, 4.0], 
[1.0, 2.0, 8.0]]], [[[7.0, 2.0, 1.0], [2.0, 8.0, 8.0], [2.0, 8.0, 2.0]], [[6.0, 1.0, 5.0], [5.0, 6.0, 7.0], [7.0, 6.0, 6.0]], 
[[8.0, 2.0, 7.0], [1.0, 1.0, 7.0], [2.0, 7.0, 8.0]]], [[[3.0, 8.0, 8.0], [5.0, 6.0, 2.0], [5.0, 2.0, 5.0]], [[4.0, 4.0, 6.0], 
[2.0, 8.0, 5.0], [6.0, 6.0, 6.0]], [[6.0, 5.0, 4.0], [3.0, 8.0, 7.0], [2.0, 8.0, 2.0]]]])
      val=Data(numpy.array([[[[2.0, 1.0, 5.0]]], [[[2.0, 4.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 2.0, 7.0]]], [[[7.0, 7.0, 8.0]]]]))
      arg[2:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 1.0], [4.0, 8.0, 5.0], [4.0, 3.0, 1.0]], [[4.0, 3.0, 3.0], [7.0, 8.0, 1.0], [7.0, 
4.0, 3.0]], [[3.0, 3.0, 2.0], [2.0, 4.0, 6.0], [5.0, 4.0, 3.0]]], [[[6.0, 7.0, 4.0], [5.0, 8.0, 1.0], [3.0, 7.0, 2.0]], [[8.0, 
2.0, 7.0], [2.0, 2.0, 8.0], [3.0, 7.0, 4.0]], [[3.0, 3.0, 7.0], [8.0, 4.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 8.0, 2.0], [3.0, 
7.0, 7.0], [8.0, 1.0, 1.0]], [[6.0, 5.0, 2.0], [4.0, 1.0, 4.0], [4.0, 4.0, 1.0]], [[8.0, 3.0, 4.0], [2.0, 1.0, 5.0], [4.0, 3.0, 
1.0]]], [[[4.0, 4.0, 1.0], [4.0, 7.0, 5.0], [3.0, 4.0, 2.0]], [[7.0, 3.0, 2.0], [6.0, 1.0, 1.0], [6.0, 8.0, 8.0]], [[7.0, 7.0, 
2.0], [2.0, 4.0, 8.0], [2.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 5.0], [1.0, 7.0, 3.0], [4.0, 2.0, 5.0]], 
[[4.0, 8.0, 4.0], [7.0, 6.0, 3.0], [3.0, 7.0, 5.0]], [[2.0, 2.0, 8.0], [1.0, 8.0, 6.0], [2.0, 6.0, 4.0]]], [[[6.0, 4.0, 8.0], 
[5.0, 3.0, 5.0], [1.0, 5.0, 6.0]], [[6.0, 7.0, 8.0], [3.0, 6.0, 4.0], [8.0, 6.0, 7.0]], [[1.0, 3.0, 2.0], [7.0, 1.0, 4.0], 
[1.0, 2.0, 8.0]]], [[[7.0, 2.0, 1.0], [2.0, 8.0, 8.0], [2.0, 8.0, 2.0]], [[6.0, 1.0, 5.0], [5.0, 6.0, 7.0], [7.0, 6.0, 6.0]], 
[[8.0, 2.0, 7.0], [2.0, 2.0, 7.0], [2.0, 7.0, 8.0]]], [[[3.0, 8.0, 8.0], [5.0, 6.0, 2.0], [5.0, 2.0, 5.0]], [[4.0, 4.0, 6.0], 
[2.0, 8.0, 5.0], [6.0, 6.0, 6.0]], [[6.0, 5.0, 4.0], [7.0, 7.0, 8.0], [2.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CUCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 6.0], [6.0, 3.0, 8.0], [2.0, 6.0, 1.0]], [[6.0, 5.0, 6.0], [1.0, 6.0, 7.0], [6.0, 
3.0, 7.0]], [[5.0, 7.0, 2.0], [4.0, 3.0, 8.0], [1.0, 7.0, 5.0]]], [[[6.0, 8.0, 1.0], [1.0, 6.0, 3.0], [7.0, 7.0, 2.0]], [[4.0, 
6.0, 6.0], [1.0, 4.0, 3.0], [1.0, 3.0, 5.0]], [[8.0, 1.0, 1.0], [4.0, 5.0, 4.0], [8.0, 6.0, 8.0]]], [[[5.0, 4.0, 5.0], [7.0, 
2.0, 4.0], [3.0, 3.0, 4.0]], [[8.0, 2.0, 6.0], [3.0, 7.0, 3.0], [1.0, 5.0, 1.0]], [[8.0, 6.0, 6.0], [7.0, 3.0, 5.0], [4.0, 8.0, 
1.0]]], [[[6.0, 6.0, 6.0], [5.0, 3.0, 3.0], [4.0, 6.0, 7.0]], [[8.0, 6.0, 4.0], [3.0, 1.0, 4.0], [8.0, 6.0, 5.0]], [[3.0, 5.0, 
6.0], [8.0, 6.0, 6.0], [2.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 4.0], [2.0, 5.0, 3.0], [5.0, 7.0, 6.0]], 
[[6.0, 8.0, 6.0], [8.0, 8.0, 3.0], [5.0, 3.0, 4.0]], [[6.0, 1.0, 6.0], [2.0, 8.0, 3.0], [4.0, 1.0, 7.0]]], [[[6.0, 5.0, 7.0], 
[4.0, 8.0, 1.0], [7.0, 4.0, 5.0]], [[3.0, 4.0, 6.0], [2.0, 1.0, 1.0], [3.0, 4.0, 3.0]], [[2.0, 2.0, 4.0], [8.0, 6.0, 8.0], 
[1.0, 2.0, 6.0]]], [[[5.0, 6.0, 6.0], [1.0, 5.0, 2.0], [5.0, 6.0, 3.0]], [[1.0, 7.0, 1.0], [6.0, 2.0, 3.0], [2.0, 3.0, 2.0]], 
[[6.0, 4.0, 5.0], [8.0, 7.0, 7.0], [5.0, 3.0, 2.0]]], [[[4.0, 5.0, 7.0], [3.0, 4.0, 8.0], [5.0, 7.0, 3.0]], [[7.0, 8.0, 1.0], 
[4.0, 8.0, 1.0], [8.0, 3.0, 4.0]], [[5.0, 3.0, 4.0], [4.0, 5.0, 6.0], [7.0, 7.0, 8.0]]]])
      val=Data(numpy.array([[[[7.0, 5.0, 2.0], [1.0, 4.0, 4.0], [6.0, 3.0, 1.0]], [[7.0, 2.0, 8.0], [7.0, 1.0, 4.0], [5.0, 8.0, 
1.0]]], [[[1.0, 5.0, 3.0], [8.0, 1.0, 4.0], [3.0, 2.0, 3.0]], [[6.0, 1.0, 7.0], [8.0, 3.0, 4.0], [2.0, 2.0, 4.0]]], [[[1.0, 
3.0, 5.0], [4.0, 4.0, 3.0], [2.0, 4.0, 8.0]], [[2.0, 2.0, 4.0], [4.0, 1.0, 6.0], [8.0, 6.0, 7.0]]], [[[3.0, 4.0, 8.0], [1.0, 
8.0, 3.0], [1.0, 2.0, 1.0]], [[2.0, 2.0, 4.0], [2.0, 3.0, 4.0], [7.0, 6.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 3.0], [5.0, 2.0, 7.0], [1.0, 8.0, 7.0]], [[7.0, 6.0, 6.0], [6.0, 7.0, 
8.0], [7.0, 2.0, 8.0]]], [[[8.0, 2.0, 4.0], [2.0, 6.0, 4.0], [7.0, 7.0, 3.0]], [[8.0, 4.0, 4.0], [5.0, 5.0, 5.0], [2.0, 2.0, 
1.0]]], [[[8.0, 8.0, 1.0], [6.0, 6.0, 5.0], [3.0, 2.0, 8.0]], [[4.0, 2.0, 3.0], [2.0, 3.0, 3.0], [7.0, 1.0, 2.0]]], [[[3.0, 
3.0, 5.0], [4.0, 3.0, 3.0], [7.0, 5.0, 3.0]], [[4.0, 2.0, 7.0], [3.0, 2.0, 2.0], [1.0, 3.0, 1.0]]]]))
      arg[:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 2.0], [1.0, 4.0, 4.0], [6.0, 3.0, 1.0]], [[7.0, 2.0, 8.0], [7.0, 1.0, 4.0], [5.0, 
8.0, 1.0]], [[5.0, 7.0, 2.0], [4.0, 3.0, 8.0], [1.0, 7.0, 5.0]]], [[[1.0, 5.0, 3.0], [8.0, 1.0, 4.0], [3.0, 2.0, 3.0]], [[6.0, 
1.0, 7.0], [8.0, 3.0, 4.0], [2.0, 2.0, 4.0]], [[8.0, 1.0, 1.0], [4.0, 5.0, 4.0], [8.0, 6.0, 8.0]]], [[[1.0, 3.0, 5.0], [4.0, 
4.0, 3.0], [2.0, 4.0, 8.0]], [[2.0, 2.0, 4.0], [4.0, 1.0, 6.0], [8.0, 6.0, 7.0]], [[8.0, 6.0, 6.0], [7.0, 3.0, 5.0], [4.0, 8.0, 
1.0]]], [[[3.0, 4.0, 8.0], [1.0, 8.0, 3.0], [1.0, 2.0, 1.0]], [[2.0, 2.0, 4.0], [2.0, 3.0, 4.0], [7.0, 6.0, 7.0]], [[3.0, 5.0, 
6.0], [8.0, 6.0, 6.0], [2.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 3.0], [5.0, 2.0, 7.0], [1.0, 8.0, 7.0]], 
[[7.0, 6.0, 6.0], [6.0, 7.0, 8.0], [7.0, 2.0, 8.0]], [[6.0, 1.0, 6.0], [2.0, 8.0, 3.0], [4.0, 1.0, 7.0]]], [[[8.0, 2.0, 4.0], 
[2.0, 6.0, 4.0], [7.0, 7.0, 3.0]], [[8.0, 4.0, 4.0], [5.0, 5.0, 5.0], [2.0, 2.0, 1.0]], [[2.0, 2.0, 4.0], [8.0, 6.0, 8.0], 
[1.0, 2.0, 6.0]]], [[[8.0, 8.0, 1.0], [6.0, 6.0, 5.0], [3.0, 2.0, 8.0]], [[4.0, 2.0, 3.0], [2.0, 3.0, 3.0], [7.0, 1.0, 2.0]], 
[[6.0, 4.0, 5.0], [8.0, 7.0, 7.0], [5.0, 3.0, 2.0]]], [[[3.0, 3.0, 5.0], [4.0, 3.0, 3.0], [7.0, 5.0, 3.0]], [[4.0, 2.0, 7.0], 
[3.0, 2.0, 2.0], [1.0, 3.0, 1.0]], [[5.0, 3.0, 4.0], [4.0, 5.0, 6.0], [7.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RPUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 3.0], [4.0, 5.0, 7.0], [4.0, 7.0, 4.0]], [[7.0, 6.0, 3.0], [1.0, 2.0, 8.0], [6.0, 
5.0, 1.0]], [[5.0, 4.0, 6.0], [6.0, 8.0, 2.0], [5.0, 4.0, 8.0]]], [[[3.0, 7.0, 8.0], [4.0, 5.0, 5.0], [7.0, 3.0, 6.0]], [[7.0, 
1.0, 1.0], [8.0, 2.0, 3.0], [1.0, 8.0, 8.0]], [[2.0, 8.0, 8.0], [2.0, 4.0, 7.0], [1.0, 2.0, 5.0]]], [[[4.0, 6.0, 5.0], [2.0, 
1.0, 1.0], [2.0, 4.0, 7.0]], [[6.0, 5.0, 2.0], [6.0, 6.0, 4.0], [5.0, 3.0, 5.0]], [[1.0, 3.0, 1.0], [3.0, 7.0, 4.0], [1.0, 5.0, 
3.0]]], [[[2.0, 4.0, 7.0], [2.0, 2.0, 5.0], [1.0, 1.0, 8.0]], [[2.0, 8.0, 1.0], [2.0, 8.0, 5.0], [1.0, 5.0, 6.0]], [[1.0, 8.0, 
4.0], [3.0, 6.0, 4.0], [3.0, 2.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 8.0], [5.0, 3.0, 1.0], [7.0, 3.0, 2.0]], 
[[6.0, 1.0, 2.0], [5.0, 8.0, 8.0], [5.0, 5.0, 5.0]], [[8.0, 1.0, 2.0], [1.0, 1.0, 2.0], [6.0, 3.0, 5.0]]], [[[7.0, 1.0, 5.0], 
[7.0, 4.0, 5.0], [6.0, 1.0, 6.0]], [[3.0, 3.0, 3.0], [4.0, 6.0, 8.0], [2.0, 3.0, 7.0]], [[4.0, 4.0, 1.0], [4.0, 3.0, 4.0], 
[8.0, 3.0, 7.0]]], [[[4.0, 7.0, 8.0], [2.0, 4.0, 4.0], [6.0, 8.0, 6.0]], [[6.0, 8.0, 6.0], [4.0, 4.0, 8.0], [7.0, 2.0, 7.0]], 
[[5.0, 4.0, 2.0], [5.0, 2.0, 4.0], [7.0, 2.0, 2.0]]], [[[2.0, 2.0, 6.0], [4.0, 6.0, 7.0], [7.0, 8.0, 8.0]], [[8.0, 2.0, 2.0], 
[3.0, 4.0, 2.0], [8.0, 6.0, 8.0]], [[8.0, 2.0, 3.0], [7.0, 7.0, 2.0], [3.0, 7.0, 1.0]]]])
      val=Data(numpy.array([[[3.0, 1.0, 1.0], [5.0, 3.0, 2.0]], [[1.0, 6.0, 1.0], [3.0, 3.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 1.0, 6.0], [5.0, 6.0, 7.0]], [[4.0, 1.0, 6.0], [2.0, 2.0, 8.0]]]))
      arg[1:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 3.0], [4.0, 5.0, 7.0], [4.0, 7.0, 4.0]], [[7.0, 6.0, 3.0], [1.0, 2.0, 8.0], [6.0, 
5.0, 1.0]], [[5.0, 4.0, 6.0], [6.0, 8.0, 2.0], [5.0, 4.0, 8.0]]], [[[3.0, 7.0, 8.0], [4.0, 5.0, 5.0], [7.0, 3.0, 6.0]], [[3.0, 
1.0, 1.0], [5.0, 3.0, 2.0], [1.0, 8.0, 8.0]], [[2.0, 8.0, 8.0], [2.0, 4.0, 7.0], [1.0, 2.0, 5.0]]], [[[4.0, 6.0, 5.0], [2.0, 
1.0, 1.0], [2.0, 4.0, 7.0]], [[1.0, 6.0, 1.0], [3.0, 3.0, 5.0], [5.0, 3.0, 5.0]], [[1.0, 3.0, 1.0], [3.0, 7.0, 4.0], [1.0, 5.0, 
3.0]]], [[[2.0, 4.0, 7.0], [2.0, 2.0, 5.0], [1.0, 1.0, 8.0]], [[2.0, 8.0, 1.0], [2.0, 8.0, 5.0], [1.0, 5.0, 6.0]], [[1.0, 8.0, 
4.0], [3.0, 6.0, 4.0], [3.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 8.0], [5.0, 3.0, 1.0], [7.0, 3.0, 2.0]], 
[[6.0, 1.0, 2.0], [5.0, 8.0, 8.0], [5.0, 5.0, 5.0]], [[8.0, 1.0, 2.0], [1.0, 1.0, 2.0], [6.0, 3.0, 5.0]]], [[[7.0, 1.0, 5.0], 
[7.0, 4.0, 5.0], [6.0, 1.0, 6.0]], [[1.0, 1.0, 6.0], [5.0, 6.0, 7.0], [2.0, 3.0, 7.0]], [[4.0, 4.0, 1.0], [4.0, 3.0, 4.0], 
[8.0, 3.0, 7.0]]], [[[4.0, 7.0, 8.0], [2.0, 4.0, 4.0], [6.0, 8.0, 6.0]], [[4.0, 1.0, 6.0], [2.0, 2.0, 8.0], [7.0, 2.0, 7.0]], 
[[5.0, 4.0, 2.0], [5.0, 2.0, 4.0], [7.0, 2.0, 2.0]]], [[[2.0, 2.0, 6.0], [4.0, 6.0, 7.0], [7.0, 8.0, 8.0]], [[8.0, 2.0, 2.0], 
[3.0, 4.0, 2.0], [8.0, 6.0, 8.0]], [[8.0, 2.0, 3.0], [7.0, 7.0, 2.0], [3.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PUUU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 1.0], [6.0, 6.0, 7.0], [5.0, 2.0, 2.0]], [[3.0, 8.0, 4.0], [6.0, 3.0, 4.0], [7.0, 
4.0, 1.0]], [[2.0, 6.0, 4.0], [4.0, 1.0, 4.0], [6.0, 4.0, 2.0]]], [[[2.0, 3.0, 6.0], [4.0, 7.0, 5.0], [7.0, 4.0, 1.0]], [[7.0, 
1.0, 6.0], [4.0, 6.0, 3.0], [2.0, 6.0, 7.0]], [[4.0, 7.0, 3.0], [5.0, 5.0, 8.0], [6.0, 2.0, 4.0]]], [[[7.0, 8.0, 2.0], [3.0, 
7.0, 3.0], [6.0, 2.0, 4.0]], [[7.0, 7.0, 8.0], [4.0, 2.0, 6.0], [1.0, 5.0, 4.0]], [[1.0, 5.0, 4.0], [6.0, 8.0, 5.0], [2.0, 8.0, 
2.0]]], [[[5.0, 2.0, 8.0], [2.0, 3.0, 4.0], [4.0, 7.0, 2.0]], [[6.0, 2.0, 6.0], [6.0, 4.0, 4.0], [1.0, 8.0, 8.0]], [[2.0, 2.0, 
7.0], [8.0, 6.0, 5.0], [4.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 2.0], [7.0, 2.0, 6.0], [6.0, 1.0, 5.0]], 
[[6.0, 1.0, 3.0], [2.0, 1.0, 4.0], [1.0, 2.0, 3.0]], [[8.0, 6.0, 7.0], [6.0, 1.0, 5.0], [3.0, 2.0, 5.0]]], [[[4.0, 2.0, 2.0], 
[1.0, 7.0, 6.0], [4.0, 5.0, 3.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 5.0], [2.0, 1.0, 8.0]], [[6.0, 2.0, 4.0], [4.0, 8.0, 8.0], 
[8.0, 4.0, 7.0]]], [[[8.0, 2.0, 8.0], [4.0, 5.0, 5.0], [2.0, 7.0, 4.0]], [[1.0, 3.0, 4.0], [1.0, 3.0, 4.0], [7.0, 6.0, 2.0]], 
[[4.0, 7.0, 7.0], [4.0, 4.0, 3.0], [3.0, 4.0, 8.0]]], [[[3.0, 7.0, 8.0], [3.0, 1.0, 7.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 4.0], 
[8.0, 4.0, 5.0], [1.0, 6.0, 6.0]], [[5.0, 5.0, 7.0], [6.0, 1.0, 8.0], [8.0, 3.0, 7.0]]]])
      val=Data(numpy.array([[[7.0, 3.0], [7.0, 4.0]], [[2.0, 3.0], [3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 3.0], [6.0, 5.0]], [[5.0, 6.0], [3.0, 1.0]]]))
      arg[2,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 1.0], [6.0, 6.0, 7.0], [5.0, 2.0, 2.0]], [[3.0, 8.0, 4.0], [6.0, 3.0, 4.0], [7.0, 
4.0, 1.0]], [[2.0, 6.0, 4.0], [4.0, 1.0, 4.0], [6.0, 4.0, 2.0]]], [[[2.0, 3.0, 6.0], [4.0, 7.0, 5.0], [7.0, 4.0, 1.0]], [[7.0, 
1.0, 6.0], [4.0, 6.0, 3.0], [2.0, 6.0, 7.0]], [[4.0, 7.0, 3.0], [5.0, 5.0, 8.0], [6.0, 2.0, 4.0]]], [[[7.0, 3.0, 2.0], [7.0, 
4.0, 3.0], [6.0, 2.0, 4.0]], [[2.0, 3.0, 8.0], [3.0, 1.0, 6.0], [1.0, 5.0, 4.0]], [[1.0, 5.0, 4.0], [6.0, 8.0, 5.0], [2.0, 8.0, 
2.0]]], [[[5.0, 2.0, 8.0], [2.0, 3.0, 4.0], [4.0, 7.0, 2.0]], [[6.0, 2.0, 6.0], [6.0, 4.0, 4.0], [1.0, 8.0, 8.0]], [[2.0, 2.0, 
7.0], [8.0, 6.0, 5.0], [4.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 2.0], [7.0, 2.0, 6.0], [6.0, 1.0, 5.0]], 
[[6.0, 1.0, 3.0], [2.0, 1.0, 4.0], [1.0, 2.0, 3.0]], [[8.0, 6.0, 7.0], [6.0, 1.0, 5.0], [3.0, 2.0, 5.0]]], [[[4.0, 2.0, 2.0], 
[1.0, 7.0, 6.0], [4.0, 5.0, 3.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 5.0], [2.0, 1.0, 8.0]], [[6.0, 2.0, 4.0], [4.0, 8.0, 8.0], 
[8.0, 4.0, 7.0]]], [[[5.0, 3.0, 8.0], [6.0, 5.0, 5.0], [2.0, 7.0, 4.0]], [[5.0, 6.0, 4.0], [3.0, 1.0, 4.0], [7.0, 6.0, 2.0]], 
[[4.0, 7.0, 7.0], [4.0, 4.0, 3.0], [3.0, 4.0, 8.0]]], [[[3.0, 7.0, 8.0], [3.0, 1.0, 7.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 4.0], 
[8.0, 4.0, 5.0], [1.0, 6.0, 6.0]], [[5.0, 5.0, 7.0], [6.0, 1.0, 8.0], [8.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLLC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 3.0, 3.0], [4.0, 4.0, 6.0], [2.0, 6.0, 7.0]], [[7.0, 1.0, 6.0], [6.0, 4.0, 6.0], [8.0, 
8.0, 5.0]], [[6.0, 2.0, 8.0], [6.0, 5.0, 6.0], [5.0, 5.0, 5.0]]], [[[5.0, 6.0, 7.0], [7.0, 4.0, 2.0], [4.0, 1.0, 1.0]], [[7.0, 
1.0, 3.0], [1.0, 8.0, 4.0], [2.0, 2.0, 5.0]], [[2.0, 1.0, 6.0], [7.0, 4.0, 4.0], [8.0, 5.0, 7.0]]], [[[5.0, 6.0, 1.0], [5.0, 
3.0, 3.0], [8.0, 2.0, 3.0]], [[7.0, 8.0, 2.0], [5.0, 5.0, 8.0], [8.0, 1.0, 6.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 4.0], [5.0, 2.0, 
2.0]]], [[[4.0, 6.0, 6.0], [4.0, 7.0, 6.0], [6.0, 1.0, 5.0]], [[2.0, 4.0, 2.0], [8.0, 1.0, 5.0], [2.0, 7.0, 2.0]], [[3.0, 7.0, 
4.0], [5.0, 6.0, 8.0], [6.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 7.0], [2.0, 2.0, 7.0], [3.0, 7.0, 2.0]], 
[[5.0, 2.0, 1.0], [3.0, 2.0, 7.0], [6.0, 8.0, 7.0]], [[4.0, 7.0, 6.0], [4.0, 3.0, 6.0], [8.0, 3.0, 7.0]]], [[[7.0, 2.0, 3.0], 
[7.0, 7.0, 2.0], [2.0, 3.0, 6.0]], [[3.0, 7.0, 2.0], [3.0, 1.0, 5.0], [7.0, 4.0, 7.0]], [[3.0, 8.0, 8.0], [1.0, 3.0, 2.0], 
[7.0, 6.0, 4.0]]], [[[1.0, 3.0, 7.0], [8.0, 7.0, 4.0], [8.0, 7.0, 7.0]], [[4.0, 3.0, 8.0], [3.0, 5.0, 1.0], [2.0, 1.0, 4.0]], 
[[6.0, 6.0, 4.0], [4.0, 5.0, 8.0], [7.0, 1.0, 5.0]]], [[[3.0, 8.0, 4.0], [4.0, 4.0, 5.0], [8.0, 2.0, 3.0]], [[7.0, 4.0, 4.0], 
[3.0, 5.0, 7.0], [1.0, 4.0, 7.0]], [[1.0, 6.0, 5.0], [6.0, 6.0, 1.0], [6.0, 8.0, 2.0]]]])
      val=Data(numpy.array([[[[6.0, 8.0, 4.0]]], [[[1.0, 7.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 6.0]]], [[[4.0, 2.0, 1.0]]]]))
      arg[2:,2:,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 3.0], [4.0, 4.0, 6.0], [2.0, 6.0, 7.0]], [[7.0, 1.0, 6.0], [6.0, 4.0, 6.0], [8.0, 
8.0, 5.0]], [[6.0, 2.0, 8.0], [6.0, 5.0, 6.0], [5.0, 5.0, 5.0]]], [[[5.0, 6.0, 7.0], [7.0, 4.0, 2.0], [4.0, 1.0, 1.0]], [[7.0, 
1.0, 3.0], [1.0, 8.0, 4.0], [2.0, 2.0, 5.0]], [[2.0, 1.0, 6.0], [7.0, 4.0, 4.0], [8.0, 5.0, 7.0]]], [[[5.0, 6.0, 1.0], [5.0, 
3.0, 3.0], [8.0, 2.0, 3.0]], [[7.0, 8.0, 2.0], [5.0, 5.0, 8.0], [8.0, 1.0, 6.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 4.0], [6.0, 8.0, 
4.0]]], [[[4.0, 6.0, 6.0], [4.0, 7.0, 6.0], [6.0, 1.0, 5.0]], [[2.0, 4.0, 2.0], [8.0, 1.0, 5.0], [2.0, 7.0, 2.0]], [[3.0, 7.0, 
4.0], [5.0, 6.0, 8.0], [1.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 7.0], [2.0, 2.0, 7.0], [3.0, 7.0, 2.0]], 
[[5.0, 2.0, 1.0], [3.0, 2.0, 7.0], [6.0, 8.0, 7.0]], [[4.0, 7.0, 6.0], [4.0, 3.0, 6.0], [8.0, 3.0, 7.0]]], [[[7.0, 2.0, 3.0], 
[7.0, 7.0, 2.0], [2.0, 3.0, 6.0]], [[3.0, 7.0, 2.0], [3.0, 1.0, 5.0], [7.0, 4.0, 7.0]], [[3.0, 8.0, 8.0], [1.0, 3.0, 2.0], 
[7.0, 6.0, 4.0]]], [[[1.0, 3.0, 7.0], [8.0, 7.0, 4.0], [8.0, 7.0, 7.0]], [[4.0, 3.0, 8.0], [3.0, 5.0, 1.0], [2.0, 1.0, 4.0]], 
[[6.0, 6.0, 4.0], [4.0, 5.0, 8.0], [6.0, 8.0, 6.0]]], [[[3.0, 8.0, 4.0], [4.0, 4.0, 5.0], [8.0, 2.0, 3.0]], [[7.0, 4.0, 4.0], 
[3.0, 5.0, 7.0], [1.0, 4.0, 7.0]], [[1.0, 6.0, 5.0], [6.0, 6.0, 1.0], [4.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LPPU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 8.0], [5.0, 1.0, 4.0], [8.0, 5.0, 4.0]], [[5.0, 6.0, 6.0], [2.0, 3.0, 2.0], [7.0, 
1.0, 4.0]], [[6.0, 6.0, 7.0], [1.0, 8.0, 7.0], [5.0, 8.0, 4.0]]], [[[5.0, 5.0, 2.0], [3.0, 1.0, 6.0], [2.0, 6.0, 6.0]], [[8.0, 
3.0, 3.0], [1.0, 7.0, 4.0], [1.0, 1.0, 6.0]], [[2.0, 3.0, 6.0], [8.0, 1.0, 2.0], [6.0, 2.0, 6.0]]], [[[2.0, 3.0, 3.0], [8.0, 
5.0, 1.0], [2.0, 4.0, 7.0]], [[4.0, 8.0, 2.0], [3.0, 2.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 1.0, 5.0], [7.0, 5.0, 2.0], [6.0, 1.0, 
8.0]]], [[[5.0, 3.0, 2.0], [2.0, 3.0, 5.0], [6.0, 2.0, 1.0]], [[4.0, 5.0, 6.0], [8.0, 6.0, 3.0], [7.0, 3.0, 8.0]], [[2.0, 6.0, 
8.0], [3.0, 3.0, 7.0], [8.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 1.0], [7.0, 8.0, 2.0], [7.0, 4.0, 5.0]], 
[[7.0, 6.0, 1.0], [1.0, 7.0, 6.0], [2.0, 4.0, 2.0]], [[5.0, 2.0, 3.0], [7.0, 1.0, 7.0], [4.0, 4.0, 4.0]]], [[[2.0, 2.0, 8.0], 
[6.0, 8.0, 6.0], [3.0, 2.0, 4.0]], [[5.0, 3.0, 2.0], [1.0, 4.0, 6.0], [7.0, 3.0, 2.0]], [[1.0, 6.0, 1.0], [8.0, 7.0, 4.0], 
[5.0, 6.0, 8.0]]], [[[7.0, 1.0, 4.0], [5.0, 2.0, 1.0], [1.0, 5.0, 2.0]], [[4.0, 3.0, 4.0], [2.0, 8.0, 6.0], [1.0, 4.0, 7.0]], 
[[4.0, 3.0, 7.0], [4.0, 1.0, 3.0], [5.0, 3.0, 1.0]]], [[[3.0, 8.0, 7.0], [5.0, 6.0, 7.0], [4.0, 3.0, 8.0]], [[7.0, 1.0, 1.0], 
[8.0, 2.0, 5.0], [5.0, 8.0, 3.0]], [[5.0, 5.0, 2.0], [6.0, 4.0, 6.0], [2.0, 2.0, 5.0]]]])
      val=Data(numpy.array([[8.0, 1.0], [5.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 1.0], [6.0, 8.0]]))
      arg[2:,1,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 8.0], [5.0, 1.0, 4.0], [8.0, 5.0, 4.0]], [[5.0, 6.0, 6.0], [2.0, 3.0, 2.0], [7.0, 
1.0, 4.0]], [[6.0, 6.0, 7.0], [1.0, 8.0, 7.0], [5.0, 8.0, 4.0]]], [[[5.0, 5.0, 2.0], [3.0, 1.0, 6.0], [2.0, 6.0, 6.0]], [[8.0, 
3.0, 3.0], [1.0, 7.0, 4.0], [1.0, 1.0, 6.0]], [[2.0, 3.0, 6.0], [8.0, 1.0, 2.0], [6.0, 2.0, 6.0]]], [[[2.0, 3.0, 3.0], [8.0, 
5.0, 1.0], [2.0, 4.0, 7.0]], [[4.0, 8.0, 2.0], [8.0, 1.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 1.0, 5.0], [7.0, 5.0, 2.0], [6.0, 1.0, 
8.0]]], [[[5.0, 3.0, 2.0], [2.0, 3.0, 5.0], [6.0, 2.0, 1.0]], [[4.0, 5.0, 6.0], [5.0, 1.0, 3.0], [7.0, 3.0, 8.0]], [[2.0, 6.0, 
8.0], [3.0, 3.0, 7.0], [8.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 1.0], [7.0, 8.0, 2.0], [7.0, 4.0, 5.0]], 
[[7.0, 6.0, 1.0], [1.0, 7.0, 6.0], [2.0, 4.0, 2.0]], [[5.0, 2.0, 3.0], [7.0, 1.0, 7.0], [4.0, 4.0, 4.0]]], [[[2.0, 2.0, 8.0], 
[6.0, 8.0, 6.0], [3.0, 2.0, 4.0]], [[5.0, 3.0, 2.0], [1.0, 4.0, 6.0], [7.0, 3.0, 2.0]], [[1.0, 6.0, 1.0], [8.0, 7.0, 4.0], 
[5.0, 6.0, 8.0]]], [[[7.0, 1.0, 4.0], [5.0, 2.0, 1.0], [1.0, 5.0, 2.0]], [[4.0, 3.0, 4.0], [6.0, 1.0, 6.0], [1.0, 4.0, 7.0]], 
[[4.0, 3.0, 7.0], [4.0, 1.0, 3.0], [5.0, 3.0, 1.0]]], [[[3.0, 8.0, 7.0], [5.0, 6.0, 7.0], [4.0, 3.0, 8.0]], [[7.0, 1.0, 1.0], 
[6.0, 8.0, 5.0], [5.0, 8.0, 3.0]], [[5.0, 5.0, 2.0], [6.0, 4.0, 6.0], [2.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 6.0], [6.0, 5.0, 2.0], [6.0, 1.0, 6.0]], [[7.0, 4.0, 4.0], [1.0, 7.0, 3.0], [4.0, 
5.0, 2.0]], [[4.0, 6.0, 2.0], [6.0, 1.0, 3.0], [8.0, 8.0, 1.0]]], [[[3.0, 2.0, 8.0], [1.0, 8.0, 2.0], [5.0, 1.0, 5.0]], [[2.0, 
1.0, 1.0], [7.0, 6.0, 8.0], [3.0, 2.0, 4.0]], [[1.0, 3.0, 1.0], [8.0, 4.0, 8.0], [7.0, 6.0, 4.0]]], [[[8.0, 2.0, 5.0], [2.0, 
8.0, 2.0], [3.0, 8.0, 8.0]], [[7.0, 8.0, 4.0], [1.0, 4.0, 3.0], [5.0, 5.0, 1.0]], [[6.0, 7.0, 3.0], [1.0, 3.0, 2.0], [6.0, 3.0, 
7.0]]], [[[6.0, 3.0, 7.0], [4.0, 7.0, 6.0], [6.0, 5.0, 1.0]], [[4.0, 8.0, 7.0], [8.0, 2.0, 7.0], [2.0, 7.0, 1.0]], [[8.0, 5.0, 
4.0], [3.0, 1.0, 3.0], [5.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 3.0], [8.0, 5.0, 2.0], [1.0, 3.0, 2.0]], 
[[8.0, 2.0, 4.0], [6.0, 1.0, 1.0], [1.0, 8.0, 7.0]], [[1.0, 1.0, 5.0], [7.0, 1.0, 1.0], [8.0, 4.0, 6.0]]], [[[4.0, 1.0, 2.0], 
[5.0, 8.0, 7.0], [4.0, 5.0, 8.0]], [[8.0, 7.0, 3.0], [4.0, 4.0, 7.0], [2.0, 8.0, 5.0]], [[6.0, 5.0, 6.0], [2.0, 5.0, 1.0], 
[7.0, 4.0, 8.0]]], [[[5.0, 2.0, 4.0], [7.0, 1.0, 6.0], [6.0, 3.0, 5.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 6.0], [3.0, 8.0, 4.0]], 
[[7.0, 1.0, 5.0], [3.0, 2.0, 1.0], [2.0, 4.0, 8.0]]], [[[2.0, 8.0, 5.0], [6.0, 4.0, 4.0], [6.0, 5.0, 8.0]], [[2.0, 1.0, 3.0], 
[4.0, 2.0, 7.0], [4.0, 8.0, 8.0]], [[8.0, 1.0, 4.0], [2.0, 1.0, 3.0], [7.0, 7.0, 6.0]]]])
      val=Data(numpy.array([[[[1.0]]], [[[1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0]]], [[[1.0]]]]))
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 6.0], [6.0, 5.0, 2.0], [6.0, 1.0, 6.0]], [[7.0, 4.0, 4.0], [1.0, 7.0, 3.0], [4.0, 
5.0, 2.0]], [[4.0, 6.0, 2.0], [6.0, 1.0, 3.0], [8.0, 8.0, 1.0]]], [[[3.0, 2.0, 8.0], [1.0, 8.0, 2.0], [5.0, 1.0, 5.0]], [[2.0, 
1.0, 1.0], [7.0, 1.0, 8.0], [3.0, 2.0, 4.0]], [[1.0, 3.0, 1.0], [8.0, 4.0, 8.0], [7.0, 6.0, 4.0]]], [[[8.0, 2.0, 5.0], [2.0, 
8.0, 2.0], [3.0, 8.0, 8.0]], [[7.0, 8.0, 4.0], [1.0, 1.0, 3.0], [5.0, 5.0, 1.0]], [[6.0, 7.0, 3.0], [1.0, 3.0, 2.0], [6.0, 3.0, 
7.0]]], [[[6.0, 3.0, 7.0], [4.0, 7.0, 6.0], [6.0, 5.0, 1.0]], [[4.0, 8.0, 7.0], [8.0, 2.0, 7.0], [2.0, 7.0, 1.0]], [[8.0, 5.0, 
4.0], [3.0, 1.0, 3.0], [5.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 3.0], [8.0, 5.0, 2.0], [1.0, 3.0, 2.0]], 
[[8.0, 2.0, 4.0], [6.0, 1.0, 1.0], [1.0, 8.0, 7.0]], [[1.0, 1.0, 5.0], [7.0, 1.0, 1.0], [8.0, 4.0, 6.0]]], [[[4.0, 1.0, 2.0], 
[5.0, 8.0, 7.0], [4.0, 5.0, 8.0]], [[8.0, 7.0, 3.0], [4.0, 7.0, 7.0], [2.0, 8.0, 5.0]], [[6.0, 5.0, 6.0], [2.0, 5.0, 1.0], 
[7.0, 4.0, 8.0]]], [[[5.0, 2.0, 4.0], [7.0, 1.0, 6.0], [6.0, 3.0, 5.0]], [[6.0, 8.0, 4.0], [7.0, 1.0, 6.0], [3.0, 8.0, 4.0]], 
[[7.0, 1.0, 5.0], [3.0, 2.0, 1.0], [2.0, 4.0, 8.0]]], [[[2.0, 8.0, 5.0], [6.0, 4.0, 4.0], [6.0, 5.0, 8.0]], [[2.0, 1.0, 3.0], 
[4.0, 2.0, 7.0], [4.0, 8.0, 8.0]], [[8.0, 1.0, 4.0], [2.0, 1.0, 3.0], [7.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PLPC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 5.0], [1.0, 8.0, 1.0], [6.0, 6.0, 7.0]], [[5.0, 1.0, 6.0], [4.0, 3.0, 2.0], [1.0, 
5.0, 4.0]], [[1.0, 7.0, 8.0], [2.0, 3.0, 3.0], [1.0, 3.0, 6.0]]], [[[7.0, 4.0, 1.0], [7.0, 8.0, 8.0], [7.0, 8.0, 4.0]], [[6.0, 
1.0, 7.0], [4.0, 8.0, 2.0], [5.0, 3.0, 8.0]], [[5.0, 5.0, 2.0], [5.0, 7.0, 6.0], [5.0, 6.0, 6.0]]], [[[1.0, 3.0, 2.0], [4.0, 
1.0, 3.0], [3.0, 5.0, 8.0]], [[8.0, 1.0, 4.0], [7.0, 1.0, 5.0], [7.0, 6.0, 3.0]], [[4.0, 4.0, 5.0], [7.0, 4.0, 5.0], [8.0, 6.0, 
1.0]]], [[[1.0, 4.0, 7.0], [5.0, 3.0, 4.0], [7.0, 8.0, 4.0]], [[8.0, 7.0, 4.0], [2.0, 1.0, 2.0], [4.0, 8.0, 1.0]], [[7.0, 3.0, 
5.0], [1.0, 5.0, 8.0], [6.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 1.0], [6.0, 8.0, 2.0], [4.0, 7.0, 6.0]], 
[[8.0, 5.0, 1.0], [8.0, 8.0, 8.0], [7.0, 1.0, 3.0]], [[2.0, 6.0, 1.0], [2.0, 8.0, 4.0], [5.0, 2.0, 5.0]]], [[[6.0, 8.0, 1.0], 
[4.0, 2.0, 5.0], [5.0, 3.0, 4.0]], [[2.0, 2.0, 6.0], [2.0, 1.0, 6.0], [1.0, 4.0, 7.0]], [[4.0, 8.0, 1.0], [8.0, 7.0, 4.0], 
[8.0, 6.0, 5.0]]], [[[2.0, 1.0, 8.0], [2.0, 1.0, 1.0], [5.0, 2.0, 6.0]], [[8.0, 2.0, 3.0], [7.0, 5.0, 4.0], [8.0, 5.0, 3.0]], 
[[4.0, 2.0, 8.0], [6.0, 4.0, 2.0], [5.0, 1.0, 7.0]]], [[[7.0, 5.0, 4.0], [4.0, 8.0, 2.0], [8.0, 8.0, 6.0]], [[7.0, 8.0, 6.0], 
[8.0, 3.0, 1.0], [8.0, 5.0, 5.0]], [[1.0, 5.0, 6.0], [7.0, 7.0, 2.0], [8.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[2.0, 2.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[1.0, 5.0, 8.0]]))
      arg[2,2:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 5.0], [1.0, 8.0, 1.0], [6.0, 6.0, 7.0]], [[5.0, 1.0, 6.0], [4.0, 3.0, 2.0], [1.0, 
5.0, 4.0]], [[1.0, 7.0, 8.0], [2.0, 3.0, 3.0], [1.0, 3.0, 6.0]]], [[[7.0, 4.0, 1.0], [7.0, 8.0, 8.0], [7.0, 8.0, 4.0]], [[6.0, 
1.0, 7.0], [4.0, 8.0, 2.0], [5.0, 3.0, 8.0]], [[5.0, 5.0, 2.0], [5.0, 7.0, 6.0], [5.0, 6.0, 6.0]]], [[[1.0, 3.0, 2.0], [4.0, 
1.0, 3.0], [3.0, 5.0, 8.0]], [[8.0, 1.0, 4.0], [7.0, 1.0, 5.0], [7.0, 6.0, 3.0]], [[4.0, 4.0, 5.0], [2.0, 2.0, 5.0], [8.0, 6.0, 
1.0]]], [[[1.0, 4.0, 7.0], [5.0, 3.0, 4.0], [7.0, 8.0, 4.0]], [[8.0, 7.0, 4.0], [2.0, 1.0, 2.0], [4.0, 8.0, 1.0]], [[7.0, 3.0, 
5.0], [1.0, 5.0, 8.0], [6.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 1.0], [6.0, 8.0, 2.0], [4.0, 7.0, 6.0]], 
[[8.0, 5.0, 1.0], [8.0, 8.0, 8.0], [7.0, 1.0, 3.0]], [[2.0, 6.0, 1.0], [2.0, 8.0, 4.0], [5.0, 2.0, 5.0]]], [[[6.0, 8.0, 1.0], 
[4.0, 2.0, 5.0], [5.0, 3.0, 4.0]], [[2.0, 2.0, 6.0], [2.0, 1.0, 6.0], [1.0, 4.0, 7.0]], [[4.0, 8.0, 1.0], [8.0, 7.0, 4.0], 
[8.0, 6.0, 5.0]]], [[[2.0, 1.0, 8.0], [2.0, 1.0, 1.0], [5.0, 2.0, 6.0]], [[8.0, 2.0, 3.0], [7.0, 5.0, 4.0], [8.0, 5.0, 3.0]], 
[[4.0, 2.0, 8.0], [1.0, 5.0, 8.0], [5.0, 1.0, 7.0]]], [[[7.0, 5.0, 4.0], [4.0, 8.0, 2.0], [8.0, 8.0, 6.0]], [[7.0, 8.0, 6.0], 
[8.0, 3.0, 1.0], [8.0, 5.0, 5.0]], [[1.0, 5.0, 6.0], [7.0, 7.0, 2.0], [8.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UULL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 8.0], [6.0, 8.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 7.0, 7.0], [6.0, 5.0, 5.0], [8.0, 
7.0, 6.0]], [[1.0, 1.0, 4.0], [4.0, 5.0, 4.0], [5.0, 6.0, 3.0]]], [[[5.0, 4.0, 2.0], [2.0, 7.0, 8.0], [2.0, 7.0, 5.0]], [[7.0, 
4.0, 2.0], [1.0, 4.0, 2.0], [2.0, 1.0, 6.0]], [[8.0, 3.0, 6.0], [7.0, 6.0, 2.0], [2.0, 8.0, 7.0]]], [[[1.0, 5.0, 6.0], [4.0, 
5.0, 5.0], [7.0, 7.0, 4.0]], [[8.0, 4.0, 1.0], [1.0, 5.0, 2.0], [8.0, 5.0, 2.0]], [[8.0, 4.0, 5.0], [8.0, 1.0, 6.0], [3.0, 1.0, 
8.0]]], [[[8.0, 2.0, 5.0], [7.0, 4.0, 3.0], [3.0, 3.0, 1.0]], [[5.0, 6.0, 2.0], [4.0, 6.0, 3.0], [4.0, 6.0, 7.0]], [[7.0, 5.0, 
4.0], [7.0, 7.0, 7.0], [2.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 1.0], [5.0, 1.0, 6.0], [4.0, 7.0, 1.0]], 
[[7.0, 7.0, 8.0], [8.0, 5.0, 7.0], [8.0, 7.0, 1.0]], [[7.0, 3.0, 8.0], [7.0, 6.0, 3.0], [6.0, 8.0, 8.0]]], [[[1.0, 4.0, 2.0], 
[2.0, 5.0, 8.0], [2.0, 7.0, 7.0]], [[4.0, 7.0, 4.0], [7.0, 7.0, 5.0], [6.0, 1.0, 1.0]], [[6.0, 7.0, 7.0], [6.0, 8.0, 4.0], 
[6.0, 3.0, 1.0]]], [[[2.0, 8.0, 7.0], [7.0, 1.0, 6.0], [6.0, 7.0, 4.0]], [[2.0, 2.0, 6.0], [3.0, 7.0, 4.0], [6.0, 2.0, 2.0]], 
[[1.0, 1.0, 6.0], [8.0, 6.0, 2.0], [3.0, 8.0, 8.0]]], [[[6.0, 1.0, 6.0], [7.0, 3.0, 1.0], [1.0, 8.0, 8.0]], [[3.0, 6.0, 7.0], 
[4.0, 1.0, 1.0], [7.0, 8.0, 2.0]], [[8.0, 2.0, 2.0], [1.0, 3.0, 8.0], [1.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[[5.0]], [[5.0]]], [[[1.0]], [[8.0]]], [[[2.0]], [[2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0]], [[5.0]]], [[[4.0]], [[1.0]]], [[[3.0]], [[8.0]]]]))
      arg[:3,:2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 8.0], [6.0, 8.0, 7.0], [1.0, 1.0, 5.0]], [[7.0, 7.0, 7.0], [6.0, 5.0, 5.0], [8.0, 
7.0, 5.0]], [[1.0, 1.0, 4.0], [4.0, 5.0, 4.0], [5.0, 6.0, 3.0]]], [[[5.0, 4.0, 2.0], [2.0, 7.0, 8.0], [2.0, 7.0, 1.0]], [[7.0, 
4.0, 2.0], [1.0, 4.0, 2.0], [2.0, 1.0, 8.0]], [[8.0, 3.0, 6.0], [7.0, 6.0, 2.0], [2.0, 8.0, 7.0]]], [[[1.0, 5.0, 6.0], [4.0, 
5.0, 5.0], [7.0, 7.0, 2.0]], [[8.0, 4.0, 1.0], [1.0, 5.0, 2.0], [8.0, 5.0, 2.0]], [[8.0, 4.0, 5.0], [8.0, 1.0, 6.0], [3.0, 1.0, 
8.0]]], [[[8.0, 2.0, 5.0], [7.0, 4.0, 3.0], [3.0, 3.0, 1.0]], [[5.0, 6.0, 2.0], [4.0, 6.0, 3.0], [4.0, 6.0, 7.0]], [[7.0, 5.0, 
4.0], [7.0, 7.0, 7.0], [2.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 1.0], [5.0, 1.0, 6.0], [4.0, 7.0, 1.0]], 
[[7.0, 7.0, 8.0], [8.0, 5.0, 7.0], [8.0, 7.0, 5.0]], [[7.0, 3.0, 8.0], [7.0, 6.0, 3.0], [6.0, 8.0, 8.0]]], [[[1.0, 4.0, 2.0], 
[2.0, 5.0, 8.0], [2.0, 7.0, 4.0]], [[4.0, 7.0, 4.0], [7.0, 7.0, 5.0], [6.0, 1.0, 1.0]], [[6.0, 7.0, 7.0], [6.0, 8.0, 4.0], 
[6.0, 3.0, 1.0]]], [[[2.0, 8.0, 7.0], [7.0, 1.0, 6.0], [6.0, 7.0, 3.0]], [[2.0, 2.0, 6.0], [3.0, 7.0, 4.0], [6.0, 2.0, 8.0]], 
[[1.0, 1.0, 6.0], [8.0, 6.0, 2.0], [3.0, 8.0, 8.0]]], [[[6.0, 1.0, 6.0], [7.0, 3.0, 1.0], [1.0, 8.0, 8.0]], [[3.0, 6.0, 7.0], 
[4.0, 1.0, 1.0], [7.0, 8.0, 2.0]], [[8.0, 2.0, 2.0], [1.0, 3.0, 8.0], [1.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PRRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 6.0], [3.0, 4.0, 7.0], [6.0, 8.0, 2.0]], [[4.0, 6.0, 7.0], [6.0, 5.0, 6.0], [2.0, 
8.0, 2.0]], [[4.0, 6.0, 2.0], [4.0, 1.0, 6.0], [5.0, 2.0, 8.0]]], [[[6.0, 2.0, 2.0], [7.0, 7.0, 8.0], [3.0, 7.0, 2.0]], [[6.0, 
1.0, 8.0], [7.0, 4.0, 5.0], [1.0, 7.0, 5.0]], [[3.0, 8.0, 5.0], [5.0, 4.0, 4.0], [4.0, 6.0, 5.0]]], [[[3.0, 5.0, 3.0], [7.0, 
3.0, 3.0], [4.0, 7.0, 6.0]], [[8.0, 1.0, 3.0], [7.0, 2.0, 6.0], [5.0, 7.0, 1.0]], [[8.0, 2.0, 2.0], [1.0, 1.0, 8.0], [4.0, 5.0, 
5.0]]], [[[3.0, 4.0, 7.0], [8.0, 4.0, 8.0], [1.0, 2.0, 4.0]], [[7.0, 1.0, 3.0], [7.0, 4.0, 3.0], [4.0, 4.0, 3.0]], [[4.0, 5.0, 
7.0], [1.0, 1.0, 7.0], [6.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 7.0], [4.0, 5.0, 2.0], [6.0, 3.0, 6.0]], 
[[1.0, 4.0, 4.0], [2.0, 4.0, 5.0], [8.0, 2.0, 6.0]], [[8.0, 6.0, 3.0], [7.0, 6.0, 2.0], [2.0, 5.0, 1.0]]], [[[7.0, 8.0, 4.0], 
[5.0, 8.0, 8.0], [2.0, 7.0, 6.0]], [[3.0, 4.0, 4.0], [3.0, 7.0, 5.0], [1.0, 8.0, 7.0]], [[4.0, 3.0, 5.0], [6.0, 7.0, 6.0], 
[3.0, 6.0, 7.0]]], [[[7.0, 1.0, 8.0], [2.0, 4.0, 2.0], [7.0, 6.0, 7.0]], [[2.0, 7.0, 4.0], [2.0, 7.0, 7.0], [7.0, 7.0, 2.0]], 
[[7.0, 2.0, 4.0], [5.0, 8.0, 5.0], [4.0, 7.0, 3.0]]], [[[8.0, 4.0, 7.0], [2.0, 7.0, 1.0], [8.0, 2.0, 1.0]], [[8.0, 1.0, 4.0], 
[3.0, 2.0, 1.0], [6.0, 8.0, 1.0]], [[2.0, 5.0, 2.0], [6.0, 2.0, 7.0], [1.0, 5.0, 3.0]]]])
      val=Data(numpy.array([[[1.0, 4.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 8.0, 7.0]]]))
      arg[2,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 6.0], [3.0, 4.0, 7.0], [6.0, 8.0, 2.0]], [[4.0, 6.0, 7.0], [6.0, 5.0, 6.0], [2.0, 
8.0, 2.0]], [[4.0, 6.0, 2.0], [4.0, 1.0, 6.0], [5.0, 2.0, 8.0]]], [[[6.0, 2.0, 2.0], [7.0, 7.0, 8.0], [3.0, 7.0, 2.0]], [[6.0, 
1.0, 8.0], [7.0, 4.0, 5.0], [1.0, 7.0, 5.0]], [[3.0, 8.0, 5.0], [5.0, 4.0, 4.0], [4.0, 6.0, 5.0]]], [[[3.0, 5.0, 3.0], [7.0, 
3.0, 3.0], [4.0, 7.0, 6.0]], [[8.0, 1.0, 3.0], [1.0, 4.0, 6.0], [5.0, 7.0, 1.0]], [[8.0, 2.0, 2.0], [1.0, 1.0, 8.0], [4.0, 5.0, 
5.0]]], [[[3.0, 4.0, 7.0], [8.0, 4.0, 8.0], [1.0, 2.0, 4.0]], [[7.0, 1.0, 3.0], [7.0, 4.0, 3.0], [4.0, 4.0, 3.0]], [[4.0, 5.0, 
7.0], [1.0, 1.0, 7.0], [6.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 7.0], [4.0, 5.0, 2.0], [6.0, 3.0, 6.0]], 
[[1.0, 4.0, 4.0], [2.0, 4.0, 5.0], [8.0, 2.0, 6.0]], [[8.0, 6.0, 3.0], [7.0, 6.0, 2.0], [2.0, 5.0, 1.0]]], [[[7.0, 8.0, 4.0], 
[5.0, 8.0, 8.0], [2.0, 7.0, 6.0]], [[3.0, 4.0, 4.0], [3.0, 7.0, 5.0], [1.0, 8.0, 7.0]], [[4.0, 3.0, 5.0], [6.0, 7.0, 6.0], 
[3.0, 6.0, 7.0]]], [[[7.0, 1.0, 8.0], [2.0, 4.0, 2.0], [7.0, 6.0, 7.0]], [[2.0, 7.0, 4.0], [7.0, 8.0, 7.0], [7.0, 7.0, 2.0]], 
[[7.0, 2.0, 4.0], [5.0, 8.0, 5.0], [4.0, 7.0, 3.0]]], [[[8.0, 4.0, 7.0], [2.0, 7.0, 1.0], [8.0, 2.0, 1.0]], [[8.0, 1.0, 4.0], 
[3.0, 2.0, 1.0], [6.0, 8.0, 1.0]], [[2.0, 5.0, 2.0], [6.0, 2.0, 7.0], [1.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RCPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 8.0], [4.0, 4.0, 3.0], [4.0, 3.0, 4.0]], [[4.0, 2.0, 4.0], [1.0, 6.0, 3.0], [6.0, 
7.0, 7.0]], [[6.0, 8.0, 8.0], [3.0, 8.0, 2.0], [5.0, 7.0, 1.0]]], [[[6.0, 8.0, 7.0], [2.0, 2.0, 6.0], [3.0, 6.0, 1.0]], [[8.0, 
1.0, 5.0], [8.0, 5.0, 5.0], [1.0, 7.0, 3.0]], [[1.0, 2.0, 1.0], [3.0, 5.0, 8.0], [1.0, 8.0, 1.0]]], [[[6.0, 4.0, 1.0], [1.0, 
5.0, 8.0], [6.0, 1.0, 7.0]], [[5.0, 1.0, 4.0], [8.0, 3.0, 8.0], [8.0, 8.0, 1.0]], [[4.0, 3.0, 3.0], [2.0, 5.0, 5.0], [5.0, 7.0, 
6.0]]], [[[8.0, 8.0, 4.0], [4.0, 6.0, 7.0], [4.0, 8.0, 6.0]], [[6.0, 5.0, 7.0], [5.0, 7.0, 5.0], [6.0, 4.0, 2.0]], [[1.0, 4.0, 
6.0], [2.0, 1.0, 6.0], [2.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 2.0], [5.0, 1.0, 6.0], [1.0, 3.0, 8.0]], 
[[6.0, 2.0, 4.0], [1.0, 8.0, 6.0], [7.0, 8.0, 2.0]], [[8.0, 3.0, 7.0], [7.0, 6.0, 6.0], [8.0, 1.0, 7.0]]], [[[2.0, 8.0, 1.0], 
[1.0, 1.0, 5.0], [5.0, 3.0, 5.0]], [[5.0, 1.0, 8.0], [3.0, 6.0, 5.0], [6.0, 2.0, 8.0]], [[8.0, 1.0, 8.0], [1.0, 3.0, 6.0], 
[6.0, 3.0, 7.0]]], [[[5.0, 6.0, 6.0], [2.0, 1.0, 3.0], [6.0, 6.0, 5.0]], [[6.0, 1.0, 1.0], [2.0, 3.0, 1.0], [1.0, 4.0, 5.0]], 
[[7.0, 1.0, 7.0], [1.0, 5.0, 6.0], [6.0, 5.0, 4.0]]], [[[7.0, 7.0, 1.0], [1.0, 4.0, 3.0], [7.0, 7.0, 2.0]], [[7.0, 6.0, 8.0], 
[7.0, 7.0, 7.0], [3.0, 4.0, 3.0]], [[2.0, 4.0, 8.0], [6.0, 7.0, 5.0], [5.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[3.0, 2.0, 8.0], [8.0, 2.0, 8.0], [1.0, 8.0, 7.0]], [[7.0, 2.0, 6.0], [4.0, 3.0, 3.0], [8.0, 1.0, 
1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 2.0, 2.0], [1.0, 8.0, 7.0], [1.0, 8.0, 8.0]], [[4.0, 2.0, 4.0], [2.0, 7.0, 3.0], 
[6.0, 6.0, 3.0]]]))
      arg[1:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 8.0], [4.0, 4.0, 3.0], [4.0, 3.0, 4.0]], [[4.0, 2.0, 4.0], [1.0, 6.0, 3.0], [6.0, 
7.0, 7.0]], [[6.0, 8.0, 8.0], [3.0, 8.0, 2.0], [5.0, 7.0, 1.0]]], [[[6.0, 8.0, 7.0], [3.0, 2.0, 8.0], [3.0, 6.0, 1.0]], [[8.0, 
1.0, 5.0], [8.0, 2.0, 8.0], [1.0, 7.0, 3.0]], [[1.0, 2.0, 1.0], [1.0, 8.0, 7.0], [1.0, 8.0, 1.0]]], [[[6.0, 4.0, 1.0], [7.0, 
2.0, 6.0], [6.0, 1.0, 7.0]], [[5.0, 1.0, 4.0], [4.0, 3.0, 3.0], [8.0, 8.0, 1.0]], [[4.0, 3.0, 3.0], [8.0, 1.0, 1.0], [5.0, 7.0, 
6.0]]], [[[8.0, 8.0, 4.0], [4.0, 6.0, 7.0], [4.0, 8.0, 6.0]], [[6.0, 5.0, 7.0], [5.0, 7.0, 5.0], [6.0, 4.0, 2.0]], [[1.0, 4.0, 
6.0], [2.0, 1.0, 6.0], [2.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 2.0], [5.0, 1.0, 6.0], [1.0, 3.0, 8.0]], 
[[6.0, 2.0, 4.0], [1.0, 8.0, 6.0], [7.0, 8.0, 2.0]], [[8.0, 3.0, 7.0], [7.0, 6.0, 6.0], [8.0, 1.0, 7.0]]], [[[2.0, 8.0, 1.0], 
[5.0, 2.0, 2.0], [5.0, 3.0, 5.0]], [[5.0, 1.0, 8.0], [1.0, 8.0, 7.0], [6.0, 2.0, 8.0]], [[8.0, 1.0, 8.0], [1.0, 8.0, 8.0], 
[6.0, 3.0, 7.0]]], [[[5.0, 6.0, 6.0], [4.0, 2.0, 4.0], [6.0, 6.0, 5.0]], [[6.0, 1.0, 1.0], [2.0, 7.0, 3.0], [1.0, 4.0, 5.0]], 
[[7.0, 1.0, 7.0], [6.0, 6.0, 3.0], [6.0, 5.0, 4.0]]], [[[7.0, 7.0, 1.0], [1.0, 4.0, 3.0], [7.0, 7.0, 2.0]], [[7.0, 6.0, 8.0], 
[7.0, 7.0, 7.0], [3.0, 4.0, 3.0]], [[2.0, 4.0, 8.0], [6.0, 7.0, 5.0], [5.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPLP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 8.0], [5.0, 6.0, 3.0], [5.0, 6.0, 5.0]], [[4.0, 5.0, 7.0], [6.0, 3.0, 1.0], [3.0, 
2.0, 3.0]], [[6.0, 1.0, 5.0], [2.0, 7.0, 4.0], [5.0, 6.0, 4.0]]], [[[3.0, 7.0, 8.0], [3.0, 2.0, 8.0], [8.0, 8.0, 2.0]], [[8.0, 
2.0, 6.0], [5.0, 8.0, 6.0], [8.0, 3.0, 5.0]], [[8.0, 7.0, 6.0], [6.0, 8.0, 5.0], [6.0, 3.0, 1.0]]], [[[7.0, 1.0, 5.0], [3.0, 
5.0, 3.0], [8.0, 8.0, 2.0]], [[8.0, 3.0, 2.0], [5.0, 7.0, 1.0], [5.0, 3.0, 5.0]], [[2.0, 3.0, 1.0], [2.0, 2.0, 8.0], [2.0, 3.0, 
3.0]]], [[[6.0, 6.0, 5.0], [5.0, 7.0, 8.0], [8.0, 8.0, 4.0]], [[4.0, 5.0, 5.0], [5.0, 4.0, 4.0], [2.0, 4.0, 2.0]], [[6.0, 4.0, 
5.0], [1.0, 2.0, 2.0], [7.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 1.0, 6.0], [4.0, 1.0, 4.0]], 
[[7.0, 1.0, 2.0], [4.0, 1.0, 3.0], [8.0, 3.0, 5.0]], [[4.0, 1.0, 4.0], [1.0, 3.0, 3.0], [6.0, 3.0, 4.0]]], [[[6.0, 2.0, 2.0], 
[1.0, 8.0, 6.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 2.0], [4.0, 4.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 7.0, 2.0], [7.0, 4.0, 5.0], 
[4.0, 4.0, 6.0]]], [[[7.0, 7.0, 1.0], [1.0, 2.0, 1.0], [7.0, 2.0, 7.0]], [[1.0, 2.0, 7.0], [6.0, 8.0, 4.0], [7.0, 4.0, 5.0]], 
[[6.0, 7.0, 6.0], [7.0, 5.0, 3.0], [3.0, 4.0, 1.0]]], [[[6.0, 1.0, 5.0], [1.0, 2.0, 3.0], [6.0, 7.0, 7.0]], [[8.0, 7.0, 7.0], 
[1.0, 5.0, 5.0], [2.0, 2.0, 8.0]], [[5.0, 2.0, 7.0], [6.0, 3.0, 2.0], [8.0, 5.0, 8.0]]]])
      val=Data(numpy.array([6.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([4.0]))
      arg[2,1,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 8.0], [5.0, 6.0, 3.0], [5.0, 6.0, 5.0]], [[4.0, 5.0, 7.0], [6.0, 3.0, 1.0], [3.0, 
2.0, 3.0]], [[6.0, 1.0, 5.0], [2.0, 7.0, 4.0], [5.0, 6.0, 4.0]]], [[[3.0, 7.0, 8.0], [3.0, 2.0, 8.0], [8.0, 8.0, 2.0]], [[8.0, 
2.0, 6.0], [5.0, 8.0, 6.0], [8.0, 3.0, 5.0]], [[8.0, 7.0, 6.0], [6.0, 8.0, 5.0], [6.0, 3.0, 1.0]]], [[[7.0, 1.0, 5.0], [3.0, 
5.0, 3.0], [8.0, 8.0, 2.0]], [[8.0, 3.0, 2.0], [5.0, 7.0, 1.0], [5.0, 6.0, 5.0]], [[2.0, 3.0, 1.0], [2.0, 2.0, 8.0], [2.0, 3.0, 
3.0]]], [[[6.0, 6.0, 5.0], [5.0, 7.0, 8.0], [8.0, 8.0, 4.0]], [[4.0, 5.0, 5.0], [5.0, 4.0, 4.0], [2.0, 4.0, 2.0]], [[6.0, 4.0, 
5.0], [1.0, 2.0, 2.0], [7.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 1.0, 6.0], [4.0, 1.0, 4.0]], 
[[7.0, 1.0, 2.0], [4.0, 1.0, 3.0], [8.0, 3.0, 5.0]], [[4.0, 1.0, 4.0], [1.0, 3.0, 3.0], [6.0, 3.0, 4.0]]], [[[6.0, 2.0, 2.0], 
[1.0, 8.0, 6.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 2.0], [4.0, 4.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 7.0, 2.0], [7.0, 4.0, 5.0], 
[4.0, 4.0, 6.0]]], [[[7.0, 7.0, 1.0], [1.0, 2.0, 1.0], [7.0, 2.0, 7.0]], [[1.0, 2.0, 7.0], [6.0, 8.0, 4.0], [7.0, 4.0, 5.0]], 
[[6.0, 7.0, 6.0], [7.0, 5.0, 3.0], [3.0, 4.0, 1.0]]], [[[6.0, 1.0, 5.0], [1.0, 2.0, 3.0], [6.0, 7.0, 7.0]], [[8.0, 7.0, 7.0], 
[1.0, 5.0, 5.0], [2.0, 2.0, 8.0]], [[5.0, 2.0, 7.0], [6.0, 3.0, 2.0], [8.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UCRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 7.0], [5.0, 1.0, 2.0], [1.0, 5.0, 6.0]], [[7.0, 8.0, 4.0], [4.0, 3.0, 1.0], [2.0, 
8.0, 3.0]], [[8.0, 6.0, 6.0], [3.0, 5.0, 4.0], [6.0, 8.0, 2.0]]], [[[5.0, 1.0, 1.0], [5.0, 6.0, 7.0], [7.0, 7.0, 6.0]], [[6.0, 
4.0, 7.0], [2.0, 2.0, 7.0], [4.0, 3.0, 1.0]], [[4.0, 3.0, 7.0], [4.0, 5.0, 8.0], [8.0, 5.0, 2.0]]], [[[5.0, 4.0, 1.0], [4.0, 
5.0, 6.0], [3.0, 5.0, 3.0]], [[2.0, 4.0, 3.0], [5.0, 6.0, 2.0], [5.0, 7.0, 4.0]], [[4.0, 1.0, 8.0], [6.0, 5.0, 8.0], [7.0, 1.0, 
1.0]]], [[[4.0, 3.0, 4.0], [5.0, 1.0, 2.0], [1.0, 6.0, 6.0]], [[3.0, 3.0, 3.0], [7.0, 3.0, 3.0], [7.0, 6.0, 4.0]], [[2.0, 7.0, 
3.0], [4.0, 2.0, 1.0], [6.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 1.0, 8.0], [3.0, 5.0, 8.0], [6.0, 8.0, 7.0]], 
[[6.0, 5.0, 3.0], [3.0, 5.0, 4.0], [4.0, 2.0, 7.0]], [[6.0, 8.0, 6.0], [2.0, 7.0, 3.0], [1.0, 2.0, 3.0]]], [[[1.0, 1.0, 2.0], 
[3.0, 6.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 6.0, 7.0], [7.0, 4.0, 8.0], [6.0, 8.0, 8.0]], [[3.0, 5.0, 4.0], [6.0, 6.0, 2.0], 
[1.0, 1.0, 1.0]]], [[[7.0, 1.0, 2.0], [5.0, 8.0, 1.0], [1.0, 4.0, 1.0]], [[2.0, 6.0, 1.0], [6.0, 2.0, 8.0], [4.0, 1.0, 5.0]], 
[[5.0, 5.0, 7.0], [3.0, 1.0, 7.0], [6.0, 7.0, 6.0]]], [[[6.0, 7.0, 8.0], [1.0, 6.0, 5.0], [5.0, 6.0, 4.0]], [[8.0, 7.0, 3.0], 
[8.0, 7.0, 7.0], [6.0, 1.0, 1.0]], [[2.0, 5.0, 3.0], [2.0, 3.0, 4.0], [8.0, 6.0, 7.0]]]])
      val=Data(numpy.array([[[[2.0, 6.0, 4.0]], [[8.0, 8.0, 3.0]], [[7.0, 8.0, 7.0]]], [[[4.0, 5.0, 3.0]], [[6.0, 4.0, 3.0]], 
[[6.0, 7.0, 6.0]]], [[[2.0, 8.0, 5.0]], [[8.0, 7.0, 4.0]], [[3.0, 8.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 4.0]], [[2.0, 2.0, 5.0]], [[8.0, 2.0, 3.0]]], [[[1.0, 7.0, 4.0]], [[7.0, 
5.0, 1.0]], [[5.0, 7.0, 8.0]]], [[[2.0, 5.0, 5.0]], [[6.0, 8.0, 2.0]], [[8.0, 5.0, 8.0]]]]))
      arg[:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 7.0], [2.0, 6.0, 4.0], [1.0, 5.0, 6.0]], [[7.0, 8.0, 4.0], [8.0, 8.0, 3.0], [2.0, 
8.0, 3.0]], [[8.0, 6.0, 6.0], [7.0, 8.0, 7.0], [6.0, 8.0, 2.0]]], [[[5.0, 1.0, 1.0], [4.0, 5.0, 3.0], [7.0, 7.0, 6.0]], [[6.0, 
4.0, 7.0], [6.0, 4.0, 3.0], [4.0, 3.0, 1.0]], [[4.0, 3.0, 7.0], [6.0, 7.0, 6.0], [8.0, 5.0, 2.0]]], [[[5.0, 4.0, 1.0], [2.0, 
8.0, 5.0], [3.0, 5.0, 3.0]], [[2.0, 4.0, 3.0], [8.0, 7.0, 4.0], [5.0, 7.0, 4.0]], [[4.0, 1.0, 8.0], [3.0, 8.0, 3.0], [7.0, 1.0, 
1.0]]], [[[4.0, 3.0, 4.0], [5.0, 1.0, 2.0], [1.0, 6.0, 6.0]], [[3.0, 3.0, 3.0], [7.0, 3.0, 3.0], [7.0, 6.0, 4.0]], [[2.0, 7.0, 
3.0], [4.0, 2.0, 1.0], [6.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 8.0], [1.0, 7.0, 4.0], [6.0, 8.0, 7.0]], 
[[6.0, 5.0, 3.0], [2.0, 2.0, 5.0], [4.0, 2.0, 7.0]], [[6.0, 8.0, 6.0], [8.0, 2.0, 3.0], [1.0, 2.0, 3.0]]], [[[1.0, 1.0, 2.0], 
[1.0, 7.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 6.0, 7.0], [7.0, 5.0, 1.0], [6.0, 8.0, 8.0]], [[3.0, 5.0, 4.0], [5.0, 7.0, 8.0], 
[1.0, 1.0, 1.0]]], [[[7.0, 1.0, 2.0], [2.0, 5.0, 5.0], [1.0, 4.0, 1.0]], [[2.0, 6.0, 1.0], [6.0, 8.0, 2.0], [4.0, 1.0, 5.0]], 
[[5.0, 5.0, 7.0], [8.0, 5.0, 8.0], [6.0, 7.0, 6.0]]], [[[6.0, 7.0, 8.0], [1.0, 6.0, 5.0], [5.0, 6.0, 4.0]], [[8.0, 7.0, 3.0], 
[8.0, 7.0, 7.0], [6.0, 1.0, 1.0]], [[2.0, 5.0, 3.0], [2.0, 3.0, 4.0], [8.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LPRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 8.0], [4.0, 8.0, 4.0], [3.0, 1.0, 7.0]], [[6.0, 7.0, 5.0], [7.0, 1.0, 8.0], [8.0, 
1.0, 2.0]], [[3.0, 6.0, 7.0], [7.0, 6.0, 8.0], [6.0, 6.0, 3.0]]], [[[1.0, 8.0, 7.0], [4.0, 5.0, 7.0], [4.0, 1.0, 6.0]], [[1.0, 
7.0, 6.0], [5.0, 5.0, 6.0], [4.0, 8.0, 3.0]], [[5.0, 3.0, 1.0], [8.0, 7.0, 3.0], [6.0, 1.0, 3.0]]], [[[8.0, 1.0, 3.0], [5.0, 
5.0, 3.0], [2.0, 4.0, 5.0]], [[2.0, 7.0, 8.0], [5.0, 4.0, 8.0], [6.0, 5.0, 7.0]], [[6.0, 6.0, 4.0], [7.0, 5.0, 4.0], [1.0, 2.0, 
6.0]]], [[[1.0, 3.0, 8.0], [3.0, 6.0, 4.0], [8.0, 5.0, 5.0]], [[3.0, 4.0, 3.0], [2.0, 5.0, 5.0], [4.0, 5.0, 8.0]], [[7.0, 2.0, 
6.0], [3.0, 8.0, 6.0], [8.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 5.0], [1.0, 1.0, 7.0], [8.0, 1.0, 3.0]], 
[[3.0, 1.0, 8.0], [3.0, 2.0, 2.0], [2.0, 5.0, 6.0]], [[2.0, 3.0, 1.0], [1.0, 8.0, 6.0], [1.0, 2.0, 2.0]]], [[[2.0, 5.0, 4.0], 
[8.0, 2.0, 2.0], [3.0, 1.0, 7.0]], [[5.0, 3.0, 5.0], [4.0, 2.0, 8.0], [7.0, 7.0, 8.0]], [[5.0, 4.0, 8.0], [7.0, 6.0, 8.0], 
[2.0, 5.0, 4.0]]], [[[2.0, 4.0, 4.0], [2.0, 7.0, 7.0], [5.0, 3.0, 4.0]], [[7.0, 6.0, 2.0], [8.0, 7.0, 5.0], [5.0, 3.0, 8.0]], 
[[3.0, 2.0, 8.0], [5.0, 8.0, 1.0], [7.0, 3.0, 6.0]]], [[[7.0, 7.0, 4.0], [2.0, 4.0, 3.0], [1.0, 8.0, 8.0]], [[1.0, 3.0, 2.0], 
[3.0, 2.0, 8.0], [7.0, 2.0, 6.0]], [[2.0, 5.0, 7.0], [1.0, 1.0, 4.0], [2.0, 6.0, 2.0]]]])
      val=Data(numpy.array([[[1.0, 8.0, 3.0]], [[6.0, 2.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 6.0, 1.0]], [[8.0, 2.0, 6.0]]]))
      arg[2:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 8.0], [4.0, 8.0, 4.0], [3.0, 1.0, 7.0]], [[6.0, 7.0, 5.0], [7.0, 1.0, 8.0], [8.0, 
1.0, 2.0]], [[3.0, 6.0, 7.0], [7.0, 6.0, 8.0], [6.0, 6.0, 3.0]]], [[[1.0, 8.0, 7.0], [4.0, 5.0, 7.0], [4.0, 1.0, 6.0]], [[1.0, 
7.0, 6.0], [5.0, 5.0, 6.0], [4.0, 8.0, 3.0]], [[5.0, 3.0, 1.0], [8.0, 7.0, 3.0], [6.0, 1.0, 3.0]]], [[[8.0, 1.0, 3.0], [5.0, 
5.0, 3.0], [2.0, 4.0, 5.0]], [[2.0, 7.0, 8.0], [1.0, 8.0, 3.0], [6.0, 5.0, 7.0]], [[6.0, 6.0, 4.0], [7.0, 5.0, 4.0], [1.0, 2.0, 
6.0]]], [[[1.0, 3.0, 8.0], [3.0, 6.0, 4.0], [8.0, 5.0, 5.0]], [[3.0, 4.0, 3.0], [6.0, 2.0, 2.0], [4.0, 5.0, 8.0]], [[7.0, 2.0, 
6.0], [3.0, 8.0, 6.0], [8.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 5.0], [1.0, 1.0, 7.0], [8.0, 1.0, 3.0]], 
[[3.0, 1.0, 8.0], [3.0, 2.0, 2.0], [2.0, 5.0, 6.0]], [[2.0, 3.0, 1.0], [1.0, 8.0, 6.0], [1.0, 2.0, 2.0]]], [[[2.0, 5.0, 4.0], 
[8.0, 2.0, 2.0], [3.0, 1.0, 7.0]], [[5.0, 3.0, 5.0], [4.0, 2.0, 8.0], [7.0, 7.0, 8.0]], [[5.0, 4.0, 8.0], [7.0, 6.0, 8.0], 
[2.0, 5.0, 4.0]]], [[[2.0, 4.0, 4.0], [2.0, 7.0, 7.0], [5.0, 3.0, 4.0]], [[7.0, 6.0, 2.0], [5.0, 6.0, 1.0], [5.0, 3.0, 8.0]], 
[[3.0, 2.0, 8.0], [5.0, 8.0, 1.0], [7.0, 3.0, 6.0]]], [[[7.0, 7.0, 4.0], [2.0, 4.0, 3.0], [1.0, 8.0, 8.0]], [[1.0, 3.0, 2.0], 
[8.0, 2.0, 6.0], [7.0, 2.0, 6.0]], [[2.0, 5.0, 7.0], [1.0, 1.0, 4.0], [2.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCLU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 5.0], [4.0, 6.0, 1.0], [2.0, 8.0, 5.0]], [[8.0, 1.0, 4.0], [4.0, 5.0, 3.0], [2.0, 
2.0, 8.0]], [[5.0, 6.0, 2.0], [5.0, 1.0, 7.0], [1.0, 8.0, 8.0]]], [[[3.0, 5.0, 6.0], [2.0, 3.0, 4.0], [7.0, 5.0, 8.0]], [[3.0, 
3.0, 5.0], [6.0, 7.0, 5.0], [7.0, 7.0, 4.0]], [[7.0, 7.0, 6.0], [5.0, 7.0, 6.0], [6.0, 3.0, 4.0]]], [[[4.0, 2.0, 7.0], [2.0, 
5.0, 4.0], [1.0, 2.0, 3.0]], [[4.0, 4.0, 1.0], [7.0, 5.0, 1.0], [4.0, 7.0, 2.0]], [[7.0, 3.0, 7.0], [8.0, 6.0, 1.0], [2.0, 2.0, 
1.0]]], [[[2.0, 1.0, 6.0], [6.0, 7.0, 3.0], [7.0, 5.0, 4.0]], [[8.0, 7.0, 6.0], [4.0, 2.0, 2.0], [1.0, 6.0, 6.0]], [[7.0, 2.0, 
4.0], [1.0, 6.0, 7.0], [5.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 2.0], [4.0, 7.0, 5.0], [8.0, 3.0, 8.0]], 
[[2.0, 3.0, 4.0], [3.0, 2.0, 3.0], [7.0, 3.0, 8.0]], [[5.0, 8.0, 6.0], [2.0, 7.0, 8.0], [5.0, 6.0, 5.0]]], [[[6.0, 7.0, 2.0], 
[7.0, 6.0, 7.0], [4.0, 2.0, 1.0]], [[2.0, 2.0, 7.0], [3.0, 1.0, 3.0], [8.0, 3.0, 2.0]], [[2.0, 4.0, 2.0], [8.0, 5.0, 6.0], 
[6.0, 1.0, 8.0]]], [[[3.0, 8.0, 8.0], [7.0, 3.0, 7.0], [7.0, 6.0, 5.0]], [[5.0, 5.0, 4.0], [7.0, 6.0, 1.0], [6.0, 2.0, 3.0]], 
[[2.0, 5.0, 6.0], [5.0, 4.0, 3.0], [4.0, 2.0, 6.0]]], [[[6.0, 1.0, 7.0], [2.0, 6.0, 1.0], [7.0, 2.0, 2.0]], [[2.0, 5.0, 6.0], 
[7.0, 1.0, 6.0], [6.0, 6.0, 7.0]], [[2.0, 4.0, 6.0], [8.0, 3.0, 4.0], [4.0, 2.0, 2.0]]]])
      val=Data(numpy.array([[[[1.0, 5.0]], [[2.0, 1.0]], [[1.0, 5.0]]], [[[8.0, 3.0]], [[1.0, 3.0]], [[7.0, 5.0]]], [[[2.0, 
6.0]], [[7.0, 8.0]], [[4.0, 2.0]]], [[[3.0, 4.0]], [[5.0, 5.0]], [[7.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 6.0]], [[6.0, 8.0]], [[6.0, 1.0]]], [[[7.0, 5.0]], [[4.0, 7.0]], [[6.0, 5.0]]], 
[[[6.0, 1.0]], [[4.0, 6.0]], [[1.0, 2.0]]], [[[8.0, 1.0]], [[8.0, 8.0]], [[5.0, 7.0]]]]))
      arg[:,:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [4.0, 6.0, 1.0], [1.0, 5.0, 5.0]], [[8.0, 1.0, 4.0], [4.0, 5.0, 3.0], [2.0, 
1.0, 8.0]], [[5.0, 6.0, 2.0], [5.0, 1.0, 7.0], [1.0, 5.0, 8.0]]], [[[3.0, 5.0, 6.0], [2.0, 3.0, 4.0], [8.0, 3.0, 8.0]], [[3.0, 
3.0, 5.0], [6.0, 7.0, 5.0], [1.0, 3.0, 4.0]], [[7.0, 7.0, 6.0], [5.0, 7.0, 6.0], [7.0, 5.0, 4.0]]], [[[4.0, 2.0, 7.0], [2.0, 
5.0, 4.0], [2.0, 6.0, 3.0]], [[4.0, 4.0, 1.0], [7.0, 5.0, 1.0], [7.0, 8.0, 2.0]], [[7.0, 3.0, 7.0], [8.0, 6.0, 1.0], [4.0, 2.0, 
1.0]]], [[[2.0, 1.0, 6.0], [6.0, 7.0, 3.0], [3.0, 4.0, 4.0]], [[8.0, 7.0, 6.0], [4.0, 2.0, 2.0], [5.0, 5.0, 6.0]], [[7.0, 2.0, 
4.0], [1.0, 6.0, 7.0], [7.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 2.0], [4.0, 7.0, 5.0], [6.0, 6.0, 8.0]], 
[[2.0, 3.0, 4.0], [3.0, 2.0, 3.0], [6.0, 8.0, 8.0]], [[5.0, 8.0, 6.0], [2.0, 7.0, 8.0], [6.0, 1.0, 5.0]]], [[[6.0, 7.0, 2.0], 
[7.0, 6.0, 7.0], [7.0, 5.0, 1.0]], [[2.0, 2.0, 7.0], [3.0, 1.0, 3.0], [4.0, 7.0, 2.0]], [[2.0, 4.0, 2.0], [8.0, 5.0, 6.0], 
[6.0, 5.0, 8.0]]], [[[3.0, 8.0, 8.0], [7.0, 3.0, 7.0], [6.0, 1.0, 5.0]], [[5.0, 5.0, 4.0], [7.0, 6.0, 1.0], [4.0, 6.0, 3.0]], 
[[2.0, 5.0, 6.0], [5.0, 4.0, 3.0], [1.0, 2.0, 6.0]]], [[[6.0, 1.0, 7.0], [2.0, 6.0, 1.0], [8.0, 1.0, 2.0]], [[2.0, 5.0, 6.0], 
[7.0, 1.0, 6.0], [8.0, 8.0, 7.0]], [[2.0, 4.0, 6.0], [8.0, 3.0, 4.0], [5.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PLLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 7.0], [4.0, 3.0, 8.0], [2.0, 1.0, 1.0]], [[1.0, 1.0, 4.0], [1.0, 1.0, 3.0], [3.0, 
1.0, 4.0]], [[2.0, 8.0, 2.0], [5.0, 6.0, 6.0], [7.0, 8.0, 1.0]]], [[[8.0, 4.0, 3.0], [7.0, 3.0, 2.0], [3.0, 3.0, 1.0]], [[1.0, 
1.0, 6.0], [6.0, 2.0, 1.0], [5.0, 1.0, 8.0]], [[3.0, 4.0, 3.0], [4.0, 8.0, 1.0], [7.0, 6.0, 4.0]]], [[[3.0, 5.0, 6.0], [4.0, 
6.0, 3.0], [3.0, 4.0, 5.0]], [[5.0, 2.0, 3.0], [5.0, 3.0, 4.0], [3.0, 3.0, 1.0]], [[4.0, 3.0, 5.0], [5.0, 6.0, 4.0], [2.0, 3.0, 
8.0]]], [[[5.0, 5.0, 1.0], [8.0, 8.0, 6.0], [3.0, 7.0, 7.0]], [[2.0, 2.0, 5.0], [8.0, 5.0, 5.0], [5.0, 4.0, 3.0]], [[1.0, 8.0, 
6.0], [2.0, 6.0, 4.0], [7.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 8.0], [6.0, 2.0, 8.0], [2.0, 3.0, 8.0]], 
[[6.0, 4.0, 3.0], [1.0, 5.0, 5.0], [8.0, 8.0, 4.0]], [[1.0, 1.0, 1.0], [3.0, 2.0, 2.0], [8.0, 8.0, 7.0]]], [[[4.0, 6.0, 7.0], 
[3.0, 7.0, 5.0], [5.0, 6.0, 2.0]], [[5.0, 7.0, 1.0], [8.0, 7.0, 5.0], [4.0, 2.0, 5.0]], [[3.0, 1.0, 3.0], [8.0, 8.0, 3.0], 
[2.0, 2.0, 2.0]]], [[[6.0, 4.0, 6.0], [6.0, 3.0, 2.0], [4.0, 7.0, 2.0]], [[3.0, 6.0, 8.0], [1.0, 7.0, 4.0], [5.0, 6.0, 8.0]], 
[[8.0, 2.0, 6.0], [5.0, 5.0, 5.0], [7.0, 7.0, 3.0]]], [[[4.0, 2.0, 2.0], [2.0, 2.0, 3.0], [3.0, 3.0, 8.0]], [[1.0, 1.0, 6.0], 
[2.0, 1.0, 8.0], [6.0, 5.0, 6.0]], [[7.0, 5.0, 3.0], [8.0, 1.0, 7.0], [8.0, 1.0, 3.0]]]])
      val=Data(numpy.array([[[1.0, 1.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 1.0, 6.0]]]))
      arg[2,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 7.0], [4.0, 3.0, 8.0], [2.0, 1.0, 1.0]], [[1.0, 1.0, 4.0], [1.0, 1.0, 3.0], [3.0, 
1.0, 4.0]], [[2.0, 8.0, 2.0], [5.0, 6.0, 6.0], [7.0, 8.0, 1.0]]], [[[8.0, 4.0, 3.0], [7.0, 3.0, 2.0], [3.0, 3.0, 1.0]], [[1.0, 
1.0, 6.0], [6.0, 2.0, 1.0], [5.0, 1.0, 8.0]], [[3.0, 4.0, 3.0], [4.0, 8.0, 1.0], [7.0, 6.0, 4.0]]], [[[3.0, 5.0, 6.0], [4.0, 
6.0, 3.0], [3.0, 4.0, 5.0]], [[5.0, 2.0, 3.0], [5.0, 3.0, 4.0], [3.0, 3.0, 1.0]], [[4.0, 3.0, 5.0], [5.0, 6.0, 4.0], [1.0, 1.0, 
7.0]]], [[[5.0, 5.0, 1.0], [8.0, 8.0, 6.0], [3.0, 7.0, 7.0]], [[2.0, 2.0, 5.0], [8.0, 5.0, 5.0], [5.0, 4.0, 3.0]], [[1.0, 8.0, 
6.0], [2.0, 6.0, 4.0], [7.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 8.0], [6.0, 2.0, 8.0], [2.0, 3.0, 8.0]], 
[[6.0, 4.0, 3.0], [1.0, 5.0, 5.0], [8.0, 8.0, 4.0]], [[1.0, 1.0, 1.0], [3.0, 2.0, 2.0], [8.0, 8.0, 7.0]]], [[[4.0, 6.0, 7.0], 
[3.0, 7.0, 5.0], [5.0, 6.0, 2.0]], [[5.0, 7.0, 1.0], [8.0, 7.0, 5.0], [4.0, 2.0, 5.0]], [[3.0, 1.0, 3.0], [8.0, 8.0, 3.0], 
[2.0, 2.0, 2.0]]], [[[6.0, 4.0, 6.0], [6.0, 3.0, 2.0], [4.0, 7.0, 2.0]], [[3.0, 6.0, 8.0], [1.0, 7.0, 4.0], [5.0, 6.0, 8.0]], 
[[8.0, 2.0, 6.0], [5.0, 5.0, 5.0], [5.0, 1.0, 6.0]]], [[[4.0, 2.0, 2.0], [2.0, 2.0, 3.0], [3.0, 3.0, 8.0]], [[1.0, 1.0, 6.0], 
[2.0, 1.0, 8.0], [6.0, 5.0, 6.0]], [[7.0, 5.0, 3.0], [8.0, 1.0, 7.0], [8.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UUCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 2.0], [2.0, 3.0, 4.0], [8.0, 4.0, 7.0]], [[3.0, 8.0, 5.0], [7.0, 1.0, 2.0], [2.0, 
2.0, 4.0]], [[6.0, 4.0, 6.0], [5.0, 2.0, 5.0], [3.0, 5.0, 2.0]]], [[[2.0, 6.0, 8.0], [1.0, 4.0, 3.0], [1.0, 1.0, 6.0]], [[7.0, 
2.0, 3.0], [4.0, 1.0, 2.0], [4.0, 1.0, 2.0]], [[3.0, 1.0, 7.0], [5.0, 4.0, 5.0], [6.0, 5.0, 1.0]]], [[[6.0, 8.0, 1.0], [2.0, 
8.0, 8.0], [4.0, 3.0, 3.0]], [[3.0, 5.0, 1.0], [4.0, 6.0, 4.0], [4.0, 3.0, 3.0]], [[6.0, 6.0, 3.0], [7.0, 5.0, 5.0], [5.0, 1.0, 
7.0]]], [[[5.0, 7.0, 2.0], [8.0, 1.0, 8.0], [4.0, 2.0, 8.0]], [[2.0, 3.0, 8.0], [6.0, 7.0, 4.0], [4.0, 8.0, 8.0]], [[5.0, 2.0, 
6.0], [1.0, 2.0, 2.0], [7.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 6.0], [5.0, 1.0, 6.0], [8.0, 4.0, 1.0]], 
[[8.0, 6.0, 5.0], [8.0, 7.0, 7.0], [7.0, 6.0, 4.0]], [[7.0, 6.0, 1.0], [7.0, 6.0, 6.0], [1.0, 4.0, 7.0]]], [[[3.0, 4.0, 5.0], 
[1.0, 4.0, 1.0], [6.0, 1.0, 8.0]], [[8.0, 5.0, 7.0], [2.0, 4.0, 6.0], [8.0, 1.0, 3.0]], [[3.0, 3.0, 6.0], [2.0, 4.0, 4.0], 
[8.0, 7.0, 4.0]]], [[[8.0, 2.0, 3.0], [6.0, 1.0, 4.0], [8.0, 6.0, 3.0]], [[5.0, 2.0, 2.0], [1.0, 1.0, 8.0], [2.0, 3.0, 8.0]], 
[[8.0, 7.0, 2.0], [4.0, 8.0, 3.0], [4.0, 8.0, 2.0]]], [[[5.0, 7.0, 5.0], [6.0, 6.0, 8.0], [4.0, 4.0, 3.0]], [[8.0, 6.0, 6.0], 
[3.0, 2.0, 2.0], [8.0, 4.0, 8.0]], [[6.0, 2.0, 5.0], [7.0, 6.0, 7.0], [3.0, 3.0, 5.0]]]])
      val=Data(numpy.array([[[[4.0, 2.0, 8.0], [4.0, 5.0, 5.0], [7.0, 7.0, 8.0]], [[6.0, 2.0, 2.0], [1.0, 1.0, 4.0], [1.0, 2.0, 
5.0]]], [[[3.0, 5.0, 5.0], [4.0, 3.0, 7.0], [6.0, 5.0, 6.0]], [[6.0, 2.0, 2.0], [4.0, 4.0, 5.0], [2.0, 3.0, 3.0]]], [[[4.0, 
6.0, 8.0], [3.0, 5.0, 5.0], [1.0, 1.0, 4.0]], [[7.0, 1.0, 5.0], [8.0, 6.0, 5.0], [5.0, 1.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 7.0], [3.0, 4.0, 3.0], [7.0, 1.0, 4.0]], [[6.0, 4.0, 4.0], [2.0, 2.0, 
8.0], [5.0, 6.0, 7.0]]], [[[1.0, 6.0, 6.0], [3.0, 7.0, 3.0], [2.0, 6.0, 2.0]], [[5.0, 7.0, 2.0], [2.0, 1.0, 4.0], [2.0, 2.0, 
8.0]]], [[[3.0, 2.0, 7.0], [6.0, 6.0, 7.0], [6.0, 5.0, 8.0]], [[6.0, 1.0, 1.0], [2.0, 4.0, 8.0], [7.0, 2.0, 3.0]]]]))
      arg[:3,:2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 8.0], [4.0, 5.0, 5.0], [7.0, 7.0, 8.0]], [[6.0, 2.0, 2.0], [1.0, 1.0, 4.0], [1.0, 
2.0, 5.0]], [[6.0, 4.0, 6.0], [5.0, 2.0, 5.0], [3.0, 5.0, 2.0]]], [[[3.0, 5.0, 5.0], [4.0, 3.0, 7.0], [6.0, 5.0, 6.0]], [[6.0, 
2.0, 2.0], [4.0, 4.0, 5.0], [2.0, 3.0, 3.0]], [[3.0, 1.0, 7.0], [5.0, 4.0, 5.0], [6.0, 5.0, 1.0]]], [[[4.0, 6.0, 8.0], [3.0, 
5.0, 5.0], [1.0, 1.0, 4.0]], [[7.0, 1.0, 5.0], [8.0, 6.0, 5.0], [5.0, 1.0, 2.0]], [[6.0, 6.0, 3.0], [7.0, 5.0, 5.0], [5.0, 1.0, 
7.0]]], [[[5.0, 7.0, 2.0], [8.0, 1.0, 8.0], [4.0, 2.0, 8.0]], [[2.0, 3.0, 8.0], [6.0, 7.0, 4.0], [4.0, 8.0, 8.0]], [[5.0, 2.0, 
6.0], [1.0, 2.0, 2.0], [7.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 7.0], [3.0, 4.0, 3.0], [7.0, 1.0, 4.0]], 
[[6.0, 4.0, 4.0], [2.0, 2.0, 8.0], [5.0, 6.0, 7.0]], [[7.0, 6.0, 1.0], [7.0, 6.0, 6.0], [1.0, 4.0, 7.0]]], [[[1.0, 6.0, 6.0], 
[3.0, 7.0, 3.0], [2.0, 6.0, 2.0]], [[5.0, 7.0, 2.0], [2.0, 1.0, 4.0], [2.0, 2.0, 8.0]], [[3.0, 3.0, 6.0], [2.0, 4.0, 4.0], 
[8.0, 7.0, 4.0]]], [[[3.0, 2.0, 7.0], [6.0, 6.0, 7.0], [6.0, 5.0, 8.0]], [[6.0, 1.0, 1.0], [2.0, 4.0, 8.0], [7.0, 2.0, 3.0]], 
[[8.0, 7.0, 2.0], [4.0, 8.0, 3.0], [4.0, 8.0, 2.0]]], [[[5.0, 7.0, 5.0], [6.0, 6.0, 8.0], [4.0, 4.0, 3.0]], [[8.0, 6.0, 6.0], 
[3.0, 2.0, 2.0], [8.0, 4.0, 8.0]], [[6.0, 2.0, 5.0], [7.0, 6.0, 7.0], [3.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 1.0], [2.0, 6.0, 3.0], [7.0, 8.0, 6.0]], [[6.0, 7.0, 6.0], [4.0, 4.0, 3.0], [3.0, 
8.0, 5.0]], [[6.0, 8.0, 7.0], [2.0, 4.0, 3.0], [4.0, 3.0, 1.0]]], [[[2.0, 1.0, 4.0], [5.0, 4.0, 1.0], [4.0, 4.0, 6.0]], [[5.0, 
7.0, 2.0], [7.0, 4.0, 8.0], [8.0, 5.0, 2.0]], [[3.0, 6.0, 5.0], [1.0, 1.0, 5.0], [2.0, 3.0, 4.0]]], [[[4.0, 4.0, 6.0], [7.0, 
5.0, 5.0], [5.0, 3.0, 6.0]], [[6.0, 3.0, 5.0], [1.0, 3.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 3.0, 5.0], [4.0, 5.0, 1.0], [2.0, 1.0, 
1.0]]], [[[5.0, 7.0, 7.0], [6.0, 5.0, 3.0], [1.0, 4.0, 7.0]], [[8.0, 7.0, 7.0], [4.0, 2.0, 8.0], [4.0, 3.0, 3.0]], [[2.0, 6.0, 
5.0], [7.0, 1.0, 5.0], [2.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 1.0], [7.0, 4.0, 7.0], [7.0, 7.0, 6.0]], 
[[5.0, 4.0, 8.0], [2.0, 1.0, 2.0], [1.0, 4.0, 6.0]], [[5.0, 5.0, 4.0], [4.0, 7.0, 1.0], [3.0, 1.0, 4.0]]], [[[5.0, 1.0, 7.0], 
[2.0, 5.0, 7.0], [4.0, 7.0, 4.0]], [[1.0, 7.0, 7.0], [3.0, 2.0, 4.0], [8.0, 4.0, 6.0]], [[1.0, 5.0, 2.0], [8.0, 6.0, 2.0], 
[8.0, 6.0, 3.0]]], [[[1.0, 2.0, 6.0], [8.0, 1.0, 1.0], [7.0, 5.0, 1.0]], [[5.0, 3.0, 3.0], [7.0, 2.0, 1.0], [4.0, 6.0, 8.0]], 
[[6.0, 2.0, 5.0], [5.0, 7.0, 4.0], [7.0, 7.0, 5.0]]], [[[8.0, 6.0, 6.0], [1.0, 4.0, 7.0], [6.0, 6.0, 1.0]], [[5.0, 5.0, 2.0], 
[6.0, 3.0, 4.0], [1.0, 5.0, 7.0]], [[3.0, 1.0, 5.0], [6.0, 1.0, 2.0], [3.0, 3.0, 8.0]]]])
      val=Data(numpy.array([[[4.0, 4.0, 3.0]], [[4.0, 1.0, 4.0]], [[3.0, 8.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 5.0, 8.0]], [[6.0, 1.0, 1.0]], [[6.0, 3.0, 3.0]]]))
      arg[:3,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 1.0], [2.0, 6.0, 3.0], [7.0, 8.0, 6.0]], [[6.0, 7.0, 6.0], [4.0, 4.0, 3.0], [3.0, 
8.0, 5.0]], [[6.0, 8.0, 7.0], [4.0, 4.0, 3.0], [4.0, 3.0, 1.0]]], [[[2.0, 1.0, 4.0], [5.0, 4.0, 1.0], [4.0, 4.0, 6.0]], [[5.0, 
7.0, 2.0], [7.0, 4.0, 8.0], [8.0, 5.0, 2.0]], [[3.0, 6.0, 5.0], [4.0, 1.0, 4.0], [2.0, 3.0, 4.0]]], [[[4.0, 4.0, 6.0], [7.0, 
5.0, 5.0], [5.0, 3.0, 6.0]], [[6.0, 3.0, 5.0], [1.0, 3.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 3.0, 5.0], [3.0, 8.0, 6.0], [2.0, 1.0, 
1.0]]], [[[5.0, 7.0, 7.0], [6.0, 5.0, 3.0], [1.0, 4.0, 7.0]], [[8.0, 7.0, 7.0], [4.0, 2.0, 8.0], [4.0, 3.0, 3.0]], [[2.0, 6.0, 
5.0], [7.0, 1.0, 5.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 1.0], [7.0, 4.0, 7.0], [7.0, 7.0, 6.0]], 
[[5.0, 4.0, 8.0], [2.0, 1.0, 2.0], [1.0, 4.0, 6.0]], [[5.0, 5.0, 4.0], [3.0, 5.0, 8.0], [3.0, 1.0, 4.0]]], [[[5.0, 1.0, 7.0], 
[2.0, 5.0, 7.0], [4.0, 7.0, 4.0]], [[1.0, 7.0, 7.0], [3.0, 2.0, 4.0], [8.0, 4.0, 6.0]], [[1.0, 5.0, 2.0], [6.0, 1.0, 1.0], 
[8.0, 6.0, 3.0]]], [[[1.0, 2.0, 6.0], [8.0, 1.0, 1.0], [7.0, 5.0, 1.0]], [[5.0, 3.0, 3.0], [7.0, 2.0, 1.0], [4.0, 6.0, 8.0]], 
[[6.0, 2.0, 5.0], [6.0, 3.0, 3.0], [7.0, 7.0, 5.0]]], [[[8.0, 6.0, 6.0], [1.0, 4.0, 7.0], [6.0, 6.0, 1.0]], [[5.0, 5.0, 2.0], 
[6.0, 3.0, 4.0], [1.0, 5.0, 7.0]], [[3.0, 1.0, 5.0], [6.0, 1.0, 2.0], [3.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPCP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 6.0], [6.0, 1.0, 2.0], [2.0, 7.0, 6.0]], [[3.0, 4.0, 8.0], [6.0, 1.0, 3.0], [8.0, 
6.0, 2.0]], [[2.0, 2.0, 2.0], [5.0, 7.0, 1.0], [1.0, 6.0, 4.0]]], [[[1.0, 4.0, 4.0], [4.0, 7.0, 4.0], [2.0, 3.0, 7.0]], [[4.0, 
5.0, 8.0], [7.0, 4.0, 1.0], [5.0, 6.0, 3.0]], [[8.0, 8.0, 7.0], [5.0, 2.0, 3.0], [2.0, 1.0, 7.0]]], [[[5.0, 7.0, 8.0], [6.0, 
2.0, 5.0], [8.0, 6.0, 6.0]], [[5.0, 8.0, 6.0], [8.0, 7.0, 1.0], [3.0, 1.0, 7.0]], [[1.0, 3.0, 3.0], [4.0, 7.0, 1.0], [3.0, 1.0, 
8.0]]], [[[5.0, 7.0, 2.0], [7.0, 6.0, 5.0], [8.0, 2.0, 6.0]], [[3.0, 4.0, 4.0], [3.0, 3.0, 2.0], [8.0, 3.0, 5.0]], [[6.0, 1.0, 
4.0], [2.0, 6.0, 5.0], [4.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 8.0, 3.0], [3.0, 8.0, 1.0], [6.0, 1.0, 8.0]], 
[[2.0, 5.0, 6.0], [6.0, 1.0, 4.0], [3.0, 8.0, 5.0]], [[7.0, 3.0, 8.0], [6.0, 4.0, 7.0], [7.0, 1.0, 2.0]]], [[[5.0, 4.0, 2.0], 
[7.0, 3.0, 3.0], [5.0, 7.0, 1.0]], [[7.0, 4.0, 6.0], [3.0, 8.0, 2.0], [1.0, 5.0, 4.0]], [[7.0, 2.0, 1.0], [2.0, 8.0, 6.0], 
[6.0, 8.0, 2.0]]], [[[8.0, 4.0, 1.0], [2.0, 3.0, 5.0], [6.0, 6.0, 1.0]], [[6.0, 4.0, 6.0], [7.0, 2.0, 8.0], [1.0, 2.0, 5.0]], 
[[5.0, 8.0, 3.0], [7.0, 4.0, 8.0], [4.0, 4.0, 1.0]]], [[[4.0, 7.0, 8.0], [5.0, 4.0, 8.0], [7.0, 7.0, 6.0]], [[7.0, 2.0, 3.0], 
[7.0, 1.0, 2.0], [7.0, 5.0, 2.0]], [[8.0, 6.0, 5.0], [4.0, 3.0, 8.0], [6.0, 5.0, 4.0]]]])
      val=Data(numpy.array([3.0, 1.0, 2.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([6.0, 7.0, 6.0]))
      arg[2,1,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 6.0], [6.0, 1.0, 2.0], [2.0, 7.0, 6.0]], [[3.0, 4.0, 8.0], [6.0, 1.0, 3.0], [8.0, 
6.0, 2.0]], [[2.0, 2.0, 2.0], [5.0, 7.0, 1.0], [1.0, 6.0, 4.0]]], [[[1.0, 4.0, 4.0], [4.0, 7.0, 4.0], [2.0, 3.0, 7.0]], [[4.0, 
5.0, 8.0], [7.0, 4.0, 1.0], [5.0, 6.0, 3.0]], [[8.0, 8.0, 7.0], [5.0, 2.0, 3.0], [2.0, 1.0, 7.0]]], [[[5.0, 7.0, 8.0], [6.0, 
2.0, 5.0], [8.0, 6.0, 6.0]], [[5.0, 3.0, 6.0], [8.0, 1.0, 1.0], [3.0, 2.0, 7.0]], [[1.0, 3.0, 3.0], [4.0, 7.0, 1.0], [3.0, 1.0, 
8.0]]], [[[5.0, 7.0, 2.0], [7.0, 6.0, 5.0], [8.0, 2.0, 6.0]], [[3.0, 4.0, 4.0], [3.0, 3.0, 2.0], [8.0, 3.0, 5.0]], [[6.0, 1.0, 
4.0], [2.0, 6.0, 5.0], [4.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 3.0], [3.0, 8.0, 1.0], [6.0, 1.0, 8.0]], 
[[2.0, 5.0, 6.0], [6.0, 1.0, 4.0], [3.0, 8.0, 5.0]], [[7.0, 3.0, 8.0], [6.0, 4.0, 7.0], [7.0, 1.0, 2.0]]], [[[5.0, 4.0, 2.0], 
[7.0, 3.0, 3.0], [5.0, 7.0, 1.0]], [[7.0, 4.0, 6.0], [3.0, 8.0, 2.0], [1.0, 5.0, 4.0]], [[7.0, 2.0, 1.0], [2.0, 8.0, 6.0], 
[6.0, 8.0, 2.0]]], [[[8.0, 4.0, 1.0], [2.0, 3.0, 5.0], [6.0, 6.0, 1.0]], [[6.0, 6.0, 6.0], [7.0, 7.0, 8.0], [1.0, 6.0, 5.0]], 
[[5.0, 8.0, 3.0], [7.0, 4.0, 8.0], [4.0, 4.0, 1.0]]], [[[4.0, 7.0, 8.0], [5.0, 4.0, 8.0], [7.0, 7.0, 6.0]], [[7.0, 2.0, 3.0], 
[7.0, 1.0, 2.0], [7.0, 5.0, 2.0]], [[8.0, 6.0, 5.0], [4.0, 3.0, 8.0], [6.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLLP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 7.0], [1.0, 2.0, 4.0], [7.0, 5.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 3.0, 6.0], [8.0, 
2.0, 6.0]], [[6.0, 7.0, 2.0], [5.0, 4.0, 2.0], [7.0, 6.0, 6.0]]], [[[6.0, 4.0, 7.0], [1.0, 4.0, 8.0], [2.0, 7.0, 6.0]], [[3.0, 
6.0, 1.0], [7.0, 6.0, 2.0], [2.0, 4.0, 7.0]], [[4.0, 5.0, 3.0], [7.0, 1.0, 2.0], [4.0, 6.0, 3.0]]], [[[7.0, 2.0, 3.0], [7.0, 
3.0, 5.0], [5.0, 3.0, 1.0]], [[7.0, 4.0, 3.0], [1.0, 8.0, 3.0], [4.0, 7.0, 8.0]], [[2.0, 3.0, 5.0], [8.0, 7.0, 7.0], [7.0, 2.0, 
3.0]]], [[[6.0, 7.0, 7.0], [8.0, 2.0, 2.0], [6.0, 6.0, 8.0]], [[1.0, 8.0, 2.0], [1.0, 1.0, 8.0], [7.0, 1.0, 2.0]], [[7.0, 8.0, 
8.0], [8.0, 5.0, 2.0], [1.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 7.0, 2.0], [2.0, 3.0, 5.0], [7.0, 2.0, 5.0]], 
[[8.0, 4.0, 8.0], [1.0, 1.0, 3.0], [5.0, 3.0, 3.0]], [[5.0, 5.0, 7.0], [6.0, 2.0, 8.0], [3.0, 6.0, 4.0]]], [[[1.0, 5.0, 5.0], 
[8.0, 2.0, 1.0], [4.0, 2.0, 4.0]], [[8.0, 6.0, 6.0], [5.0, 2.0, 1.0], [7.0, 2.0, 7.0]], [[7.0, 2.0, 8.0], [8.0, 7.0, 1.0], 
[6.0, 1.0, 8.0]]], [[[4.0, 6.0, 4.0], [7.0, 4.0, 6.0], [7.0, 8.0, 8.0]], [[6.0, 3.0, 7.0], [6.0, 5.0, 3.0], [3.0, 4.0, 4.0]], 
[[1.0, 2.0, 6.0], [2.0, 8.0, 8.0], [2.0, 6.0, 6.0]]], [[[6.0, 7.0, 2.0], [2.0, 7.0, 6.0], [2.0, 8.0, 6.0]], [[1.0, 4.0, 7.0], 
[4.0, 5.0, 5.0], [3.0, 8.0, 1.0]], [[5.0, 4.0, 4.0], [1.0, 4.0, 4.0], [4.0, 1.0, 6.0]]]])
      val=Data(numpy.array([[[5.0]], [[7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0]], [[2.0]]]))
      arg[2:,2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 7.0], [1.0, 2.0, 4.0], [7.0, 5.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 3.0, 6.0], [8.0, 
2.0, 6.0]], [[6.0, 7.0, 2.0], [5.0, 4.0, 2.0], [7.0, 6.0, 6.0]]], [[[6.0, 4.0, 7.0], [1.0, 4.0, 8.0], [2.0, 7.0, 6.0]], [[3.0, 
6.0, 1.0], [7.0, 6.0, 2.0], [2.0, 4.0, 7.0]], [[4.0, 5.0, 3.0], [7.0, 1.0, 2.0], [4.0, 6.0, 3.0]]], [[[7.0, 2.0, 3.0], [7.0, 
3.0, 5.0], [5.0, 3.0, 1.0]], [[7.0, 4.0, 3.0], [1.0, 8.0, 3.0], [4.0, 7.0, 8.0]], [[2.0, 3.0, 5.0], [8.0, 7.0, 7.0], [7.0, 5.0, 
3.0]]], [[[6.0, 7.0, 7.0], [8.0, 2.0, 2.0], [6.0, 6.0, 8.0]], [[1.0, 8.0, 2.0], [1.0, 1.0, 8.0], [7.0, 1.0, 2.0]], [[7.0, 8.0, 
8.0], [8.0, 5.0, 2.0], [1.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 2.0], [2.0, 3.0, 5.0], [7.0, 2.0, 5.0]], 
[[8.0, 4.0, 8.0], [1.0, 1.0, 3.0], [5.0, 3.0, 3.0]], [[5.0, 5.0, 7.0], [6.0, 2.0, 8.0], [3.0, 6.0, 4.0]]], [[[1.0, 5.0, 5.0], 
[8.0, 2.0, 1.0], [4.0, 2.0, 4.0]], [[8.0, 6.0, 6.0], [5.0, 2.0, 1.0], [7.0, 2.0, 7.0]], [[7.0, 2.0, 8.0], [8.0, 7.0, 1.0], 
[6.0, 1.0, 8.0]]], [[[4.0, 6.0, 4.0], [7.0, 4.0, 6.0], [7.0, 8.0, 8.0]], [[6.0, 3.0, 7.0], [6.0, 5.0, 3.0], [3.0, 4.0, 4.0]], 
[[1.0, 2.0, 6.0], [2.0, 8.0, 8.0], [2.0, 1.0, 6.0]]], [[[6.0, 7.0, 2.0], [2.0, 7.0, 6.0], [2.0, 8.0, 6.0]], [[1.0, 4.0, 7.0], 
[4.0, 5.0, 5.0], [3.0, 8.0, 1.0]], [[5.0, 4.0, 4.0], [1.0, 4.0, 4.0], [4.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 5.0], [2.0, 1.0, 7.0], [1.0, 4.0, 2.0]], [[7.0, 8.0, 5.0], [3.0, 3.0, 8.0], [2.0, 
1.0, 3.0]], [[4.0, 6.0, 7.0], [1.0, 2.0, 4.0], [1.0, 2.0, 4.0]]], [[[6.0, 3.0, 4.0], [7.0, 4.0, 4.0], [3.0, 4.0, 4.0]], [[7.0, 
1.0, 2.0], [8.0, 3.0, 3.0], [3.0, 1.0, 3.0]], [[3.0, 2.0, 8.0], [5.0, 4.0, 6.0], [2.0, 2.0, 6.0]]], [[[5.0, 1.0, 6.0], [3.0, 
4.0, 8.0], [8.0, 7.0, 3.0]], [[1.0, 6.0, 2.0], [6.0, 3.0, 8.0], [8.0, 6.0, 4.0]], [[8.0, 6.0, 8.0], [3.0, 5.0, 2.0], [8.0, 5.0, 
2.0]]], [[[2.0, 8.0, 7.0], [6.0, 4.0, 6.0], [2.0, 3.0, 6.0]], [[6.0, 5.0, 6.0], [2.0, 6.0, 6.0], [3.0, 7.0, 2.0]], [[3.0, 4.0, 
4.0], [7.0, 2.0, 6.0], [2.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 3.0], [2.0, 7.0, 8.0], [7.0, 2.0, 3.0]], 
[[5.0, 1.0, 3.0], [2.0, 1.0, 7.0], [6.0, 8.0, 7.0]], [[6.0, 8.0, 7.0], [3.0, 6.0, 6.0], [2.0, 5.0, 8.0]]], [[[8.0, 4.0, 3.0], 
[7.0, 6.0, 6.0], [6.0, 8.0, 4.0]], [[2.0, 7.0, 2.0], [4.0, 6.0, 2.0], [2.0, 1.0, 1.0]], [[6.0, 3.0, 5.0], [1.0, 6.0, 7.0], 
[2.0, 1.0, 7.0]]], [[[3.0, 6.0, 5.0], [1.0, 2.0, 5.0], [2.0, 1.0, 4.0]], [[1.0, 3.0, 5.0], [2.0, 7.0, 5.0], [5.0, 2.0, 3.0]], 
[[2.0, 7.0, 2.0], [5.0, 5.0, 1.0], [1.0, 5.0, 2.0]]], [[[5.0, 3.0, 6.0], [7.0, 6.0, 4.0], [2.0, 7.0, 6.0]], [[2.0, 7.0, 6.0], 
[7.0, 7.0, 8.0], [7.0, 1.0, 4.0]], [[7.0, 3.0, 1.0], [6.0, 3.0, 8.0], [3.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[[6.0, 4.0, 7.0]], [[8.0, 5.0, 4.0]], [[6.0, 3.0, 8.0]]], [[[6.0, 3.0, 6.0]], [[2.0, 3.0, 4.0]], 
[[5.0, 6.0, 1.0]]], [[[1.0, 4.0, 6.0]], [[5.0, 8.0, 8.0]], [[7.0, 2.0, 4.0]]], [[[3.0, 5.0, 3.0]], [[8.0, 5.0, 5.0]], [[6.0, 
5.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 7.0]], [[2.0, 7.0, 7.0]], [[7.0, 4.0, 3.0]]], [[[1.0, 3.0, 8.0]], [[6.0, 
7.0, 5.0]], [[1.0, 7.0, 2.0]]], [[[3.0, 5.0, 3.0]], [[6.0, 5.0, 4.0]], [[8.0, 8.0, 3.0]]], [[[8.0, 8.0, 8.0]], [[4.0, 6.0, 
2.0]], [[2.0, 6.0, 4.0]]]]))
      arg[:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 5.0], [2.0, 1.0, 7.0], [6.0, 4.0, 7.0]], [[7.0, 8.0, 5.0], [3.0, 3.0, 8.0], [8.0, 
5.0, 4.0]], [[4.0, 6.0, 7.0], [1.0, 2.0, 4.0], [6.0, 3.0, 8.0]]], [[[6.0, 3.0, 4.0], [7.0, 4.0, 4.0], [6.0, 3.0, 6.0]], [[7.0, 
1.0, 2.0], [8.0, 3.0, 3.0], [2.0, 3.0, 4.0]], [[3.0, 2.0, 8.0], [5.0, 4.0, 6.0], [5.0, 6.0, 1.0]]], [[[5.0, 1.0, 6.0], [3.0, 
4.0, 8.0], [1.0, 4.0, 6.0]], [[1.0, 6.0, 2.0], [6.0, 3.0, 8.0], [5.0, 8.0, 8.0]], [[8.0, 6.0, 8.0], [3.0, 5.0, 2.0], [7.0, 2.0, 
4.0]]], [[[2.0, 8.0, 7.0], [6.0, 4.0, 6.0], [3.0, 5.0, 3.0]], [[6.0, 5.0, 6.0], [2.0, 6.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 4.0, 
4.0], [7.0, 2.0, 6.0], [6.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 3.0], [2.0, 7.0, 8.0], [7.0, 4.0, 7.0]], 
[[5.0, 1.0, 3.0], [2.0, 1.0, 7.0], [2.0, 7.0, 7.0]], [[6.0, 8.0, 7.0], [3.0, 6.0, 6.0], [7.0, 4.0, 3.0]]], [[[8.0, 4.0, 3.0], 
[7.0, 6.0, 6.0], [1.0, 3.0, 8.0]], [[2.0, 7.0, 2.0], [4.0, 6.0, 2.0], [6.0, 7.0, 5.0]], [[6.0, 3.0, 5.0], [1.0, 6.0, 7.0], 
[1.0, 7.0, 2.0]]], [[[3.0, 6.0, 5.0], [1.0, 2.0, 5.0], [3.0, 5.0, 3.0]], [[1.0, 3.0, 5.0], [2.0, 7.0, 5.0], [6.0, 5.0, 4.0]], 
[[2.0, 7.0, 2.0], [5.0, 5.0, 1.0], [8.0, 8.0, 3.0]]], [[[5.0, 3.0, 6.0], [7.0, 6.0, 4.0], [8.0, 8.0, 8.0]], [[2.0, 7.0, 6.0], 
[7.0, 7.0, 8.0], [4.0, 6.0, 2.0]], [[7.0, 3.0, 1.0], [6.0, 3.0, 8.0], [2.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 1.0], [4.0, 6.0, 5.0], [7.0, 1.0, 8.0]], [[4.0, 4.0, 5.0], [6.0, 2.0, 1.0], [4.0, 
4.0, 5.0]], [[4.0, 6.0, 7.0], [2.0, 3.0, 7.0], [1.0, 8.0, 4.0]]], [[[8.0, 4.0, 1.0], [2.0, 8.0, 5.0], [6.0, 7.0, 4.0]], [[2.0, 
8.0, 3.0], [6.0, 5.0, 7.0], [1.0, 5.0, 1.0]], [[5.0, 7.0, 8.0], [4.0, 7.0, 6.0], [5.0, 3.0, 5.0]]], [[[7.0, 5.0, 7.0], [3.0, 
8.0, 6.0], [4.0, 5.0, 6.0]], [[2.0, 6.0, 1.0], [7.0, 4.0, 3.0], [2.0, 7.0, 1.0]], [[1.0, 4.0, 7.0], [6.0, 1.0, 6.0], [3.0, 5.0, 
7.0]]], [[[8.0, 1.0, 4.0], [1.0, 7.0, 8.0], [1.0, 1.0, 6.0]], [[3.0, 5.0, 8.0], [8.0, 3.0, 7.0], [8.0, 7.0, 2.0]], [[7.0, 8.0, 
6.0], [7.0, 6.0, 4.0], [4.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 8.0], [1.0, 4.0, 8.0], [5.0, 3.0, 8.0]], 
[[1.0, 2.0, 3.0], [3.0, 6.0, 5.0], [3.0, 6.0, 6.0]], [[1.0, 7.0, 8.0], [7.0, 6.0, 2.0], [7.0, 6.0, 1.0]]], [[[4.0, 4.0, 5.0], 
[6.0, 3.0, 7.0], [6.0, 2.0, 6.0]], [[6.0, 2.0, 6.0], [6.0, 4.0, 6.0], [7.0, 5.0, 7.0]], [[1.0, 6.0, 2.0], [1.0, 3.0, 5.0], 
[5.0, 6.0, 8.0]]], [[[1.0, 4.0, 7.0], [5.0, 3.0, 4.0], [2.0, 7.0, 5.0]], [[1.0, 5.0, 5.0], [3.0, 5.0, 5.0], [6.0, 6.0, 4.0]], 
[[7.0, 5.0, 3.0], [5.0, 4.0, 3.0], [3.0, 6.0, 3.0]]], [[[7.0, 3.0, 4.0], [7.0, 5.0, 3.0], [8.0, 2.0, 5.0]], [[6.0, 3.0, 7.0], 
[1.0, 3.0, 5.0], [2.0, 2.0, 3.0]], [[6.0, 7.0, 5.0], [3.0, 5.0, 2.0], [8.0, 3.0, 6.0]]]])
      val=Data(numpy.array([[[[8.0, 2.0, 8.0], [7.0, 4.0, 1.0]], [[6.0, 3.0, 2.0], [8.0, 5.0, 7.0]], [[8.0, 3.0, 4.0], [4.0, 
7.0, 6.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 2.0]], [[3.0, 4.0, 7.0], [8.0, 3.0, 4.0]], [[5.0, 7.0, 3.0], [4.0, 7.0, 5.0]]], 
[[[3.0, 6.0, 4.0], [3.0, 7.0, 1.0]], [[8.0, 1.0, 1.0], [6.0, 5.0, 7.0]], [[7.0, 4.0, 6.0], [4.0, 2.0, 2.0]]], [[[5.0, 3.0, 
5.0], [7.0, 3.0, 3.0]], [[7.0, 1.0, 4.0], [3.0, 7.0, 7.0]], [[4.0, 8.0, 1.0], [8.0, 4.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 5.0, 7.0], [4.0, 6.0, 7.0]], [[4.0, 5.0, 1.0], [7.0, 8.0, 2.0]], [[4.0, 8.0, 
8.0], [2.0, 7.0, 4.0]]], [[[1.0, 3.0, 2.0], [1.0, 3.0, 8.0]], [[3.0, 2.0, 2.0], [8.0, 7.0, 4.0]], [[4.0, 4.0, 1.0], [3.0, 1.0, 
4.0]]], [[[7.0, 1.0, 5.0], [3.0, 7.0, 4.0]], [[4.0, 2.0, 5.0], [1.0, 1.0, 5.0]], [[8.0, 3.0, 1.0], [5.0, 8.0, 3.0]]], [[[6.0, 
4.0, 6.0], [1.0, 2.0, 2.0]], [[5.0, 5.0, 4.0], [5.0, 5.0, 7.0]], [[4.0, 4.0, 4.0], [7.0, 3.0, 5.0]]]]))
      arg[:,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 8.0], [7.0, 4.0, 1.0], [7.0, 1.0, 8.0]], [[6.0, 3.0, 2.0], [8.0, 5.0, 7.0], [4.0, 
4.0, 5.0]], [[8.0, 3.0, 4.0], [4.0, 7.0, 6.0], [1.0, 8.0, 4.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 2.0], [6.0, 7.0, 4.0]], [[3.0, 
4.0, 7.0], [8.0, 3.0, 4.0], [1.0, 5.0, 1.0]], [[5.0, 7.0, 3.0], [4.0, 7.0, 5.0], [5.0, 3.0, 5.0]]], [[[3.0, 6.0, 4.0], [3.0, 
7.0, 1.0], [4.0, 5.0, 6.0]], [[8.0, 1.0, 1.0], [6.0, 5.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 6.0], [4.0, 2.0, 2.0], [3.0, 5.0, 
7.0]]], [[[5.0, 3.0, 5.0], [7.0, 3.0, 3.0], [1.0, 1.0, 6.0]], [[7.0, 1.0, 4.0], [3.0, 7.0, 7.0], [8.0, 7.0, 2.0]], [[4.0, 8.0, 
1.0], [8.0, 4.0, 7.0], [4.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 7.0], [4.0, 6.0, 7.0], [5.0, 3.0, 8.0]], 
[[4.0, 5.0, 1.0], [7.0, 8.0, 2.0], [3.0, 6.0, 6.0]], [[4.0, 8.0, 8.0], [2.0, 7.0, 4.0], [7.0, 6.0, 1.0]]], [[[1.0, 3.0, 2.0], 
[1.0, 3.0, 8.0], [6.0, 2.0, 6.0]], [[3.0, 2.0, 2.0], [8.0, 7.0, 4.0], [7.0, 5.0, 7.0]], [[4.0, 4.0, 1.0], [3.0, 1.0, 4.0], 
[5.0, 6.0, 8.0]]], [[[7.0, 1.0, 5.0], [3.0, 7.0, 4.0], [2.0, 7.0, 5.0]], [[4.0, 2.0, 5.0], [1.0, 1.0, 5.0], [6.0, 6.0, 4.0]], 
[[8.0, 3.0, 1.0], [5.0, 8.0, 3.0], [3.0, 6.0, 3.0]]], [[[6.0, 4.0, 6.0], [1.0, 2.0, 2.0], [8.0, 2.0, 5.0]], [[5.0, 5.0, 4.0], 
[5.0, 5.0, 7.0], [2.0, 2.0, 3.0]], [[4.0, 4.0, 4.0], [7.0, 3.0, 5.0], [8.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PRPP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 2.0], [1.0, 2.0, 2.0], [4.0, 3.0, 7.0]], [[1.0, 3.0, 7.0], [1.0, 3.0, 8.0], [2.0, 
2.0, 7.0]], [[3.0, 1.0, 4.0], [2.0, 6.0, 7.0], [8.0, 4.0, 3.0]]], [[[6.0, 4.0, 1.0], [7.0, 1.0, 8.0], [1.0, 5.0, 2.0]], [[5.0, 
6.0, 2.0], [3.0, 3.0, 6.0], [1.0, 3.0, 8.0]], [[6.0, 2.0, 2.0], [4.0, 1.0, 7.0], [4.0, 8.0, 1.0]]], [[[5.0, 3.0, 5.0], [3.0, 
2.0, 6.0], [6.0, 7.0, 8.0]], [[4.0, 7.0, 3.0], [2.0, 6.0, 8.0], [7.0, 5.0, 3.0]], [[1.0, 4.0, 8.0], [7.0, 2.0, 8.0], [7.0, 1.0, 
8.0]]], [[[8.0, 7.0, 3.0], [6.0, 8.0, 1.0], [6.0, 4.0, 6.0]], [[5.0, 3.0, 3.0], [6.0, 4.0, 6.0], [4.0, 3.0, 1.0]], [[2.0, 5.0, 
6.0], [6.0, 4.0, 7.0], [4.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 5.0], [6.0, 8.0, 1.0], [2.0, 8.0, 7.0]], 
[[1.0, 3.0, 4.0], [5.0, 5.0, 5.0], [7.0, 5.0, 5.0]], [[2.0, 6.0, 7.0], [3.0, 2.0, 2.0], [4.0, 4.0, 5.0]]], [[[4.0, 1.0, 3.0], 
[6.0, 3.0, 5.0], [4.0, 3.0, 8.0]], [[7.0, 8.0, 3.0], [7.0, 1.0, 3.0], [4.0, 8.0, 6.0]], [[3.0, 5.0, 1.0], [6.0, 4.0, 4.0], 
[5.0, 6.0, 8.0]]], [[[1.0, 7.0, 8.0], [4.0, 1.0, 5.0], [1.0, 1.0, 4.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 6.0], [1.0, 1.0, 8.0]], 
[[1.0, 5.0, 4.0], [1.0, 1.0, 1.0], [4.0, 7.0, 3.0]]], [[[1.0, 6.0, 8.0], [3.0, 6.0, 5.0], [1.0, 6.0, 4.0]], [[7.0, 2.0, 3.0], 
[8.0, 4.0, 4.0], [8.0, 7.0, 8.0]], [[3.0, 1.0, 4.0], [2.0, 2.0, 3.0], [8.0, 3.0, 2.0]]]])
      val=Data(numpy.array([3.0]),self.functionspace)
      val.setTaggedValue(1,numpy.array([6.0]))
      arg[2,1:2,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 2.0], [1.0, 2.0, 2.0], [4.0, 3.0, 7.0]], [[1.0, 3.0, 7.0], [1.0, 3.0, 8.0], [2.0, 
2.0, 7.0]], [[3.0, 1.0, 4.0], [2.0, 6.0, 7.0], [8.0, 4.0, 3.0]]], [[[6.0, 4.0, 1.0], [7.0, 1.0, 8.0], [1.0, 5.0, 2.0]], [[5.0, 
6.0, 2.0], [3.0, 3.0, 6.0], [1.0, 3.0, 8.0]], [[6.0, 2.0, 2.0], [4.0, 1.0, 7.0], [4.0, 8.0, 1.0]]], [[[5.0, 3.0, 5.0], [3.0, 
2.0, 6.0], [6.0, 7.0, 8.0]], [[4.0, 7.0, 3.0], [2.0, 3.0, 8.0], [7.0, 5.0, 3.0]], [[1.0, 4.0, 8.0], [7.0, 2.0, 8.0], [7.0, 1.0, 
8.0]]], [[[8.0, 7.0, 3.0], [6.0, 8.0, 1.0], [6.0, 4.0, 6.0]], [[5.0, 3.0, 3.0], [6.0, 4.0, 6.0], [4.0, 3.0, 1.0]], [[2.0, 5.0, 
6.0], [6.0, 4.0, 7.0], [4.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 5.0], [6.0, 8.0, 1.0], [2.0, 8.0, 7.0]], 
[[1.0, 3.0, 4.0], [5.0, 5.0, 5.0], [7.0, 5.0, 5.0]], [[2.0, 6.0, 7.0], [3.0, 2.0, 2.0], [4.0, 4.0, 5.0]]], [[[4.0, 1.0, 3.0], 
[6.0, 3.0, 5.0], [4.0, 3.0, 8.0]], [[7.0, 8.0, 3.0], [7.0, 1.0, 3.0], [4.0, 8.0, 6.0]], [[3.0, 5.0, 1.0], [6.0, 4.0, 4.0], 
[5.0, 6.0, 8.0]]], [[[1.0, 7.0, 8.0], [4.0, 1.0, 5.0], [1.0, 1.0, 4.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 6.0], [1.0, 1.0, 8.0]], 
[[1.0, 5.0, 4.0], [1.0, 1.0, 1.0], [4.0, 7.0, 3.0]]], [[[1.0, 6.0, 8.0], [3.0, 6.0, 5.0], [1.0, 6.0, 4.0]], [[7.0, 2.0, 3.0], 
[8.0, 4.0, 4.0], [8.0, 7.0, 8.0]], [[3.0, 1.0, 4.0], [2.0, 2.0, 3.0], [8.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UPRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 6.0], [6.0, 5.0, 1.0], [3.0, 2.0, 5.0]], [[7.0, 3.0, 4.0], [4.0, 3.0, 3.0], [2.0, 
7.0, 4.0]], [[5.0, 7.0, 2.0], [3.0, 7.0, 2.0], [8.0, 7.0, 4.0]]], [[[3.0, 6.0, 2.0], [1.0, 8.0, 3.0], [3.0, 4.0, 8.0]], [[7.0, 
7.0, 8.0], [2.0, 4.0, 4.0], [7.0, 5.0, 3.0]], [[6.0, 7.0, 3.0], [3.0, 7.0, 3.0], [6.0, 5.0, 2.0]]], [[[7.0, 6.0, 8.0], [8.0, 
4.0, 2.0], [8.0, 5.0, 4.0]], [[2.0, 4.0, 6.0], [1.0, 6.0, 2.0], [3.0, 4.0, 3.0]], [[5.0, 6.0, 5.0], [2.0, 1.0, 8.0], [5.0, 3.0, 
7.0]]], [[[7.0, 6.0, 7.0], [7.0, 7.0, 8.0], [3.0, 5.0, 8.0]], [[4.0, 8.0, 8.0], [3.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[4.0, 2.0, 
8.0], [2.0, 4.0, 1.0], [7.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 4.0], [5.0, 5.0, 3.0], [2.0, 6.0, 5.0]], 
[[2.0, 1.0, 4.0], [2.0, 7.0, 5.0], [6.0, 5.0, 4.0]], [[1.0, 7.0, 7.0], [3.0, 4.0, 3.0], [5.0, 1.0, 7.0]]], [[[4.0, 3.0, 6.0], 
[4.0, 7.0, 4.0], [3.0, 5.0, 8.0]], [[1.0, 6.0, 6.0], [1.0, 2.0, 6.0], [4.0, 1.0, 8.0]], [[7.0, 7.0, 4.0], [8.0, 3.0, 3.0], 
[1.0, 3.0, 4.0]]], [[[7.0, 5.0, 7.0], [3.0, 2.0, 8.0], [8.0, 5.0, 5.0]], [[7.0, 4.0, 8.0], [7.0, 4.0, 2.0], [6.0, 4.0, 2.0]], 
[[5.0, 3.0, 1.0], [5.0, 7.0, 5.0], [2.0, 7.0, 3.0]]], [[[4.0, 4.0, 8.0], [3.0, 1.0, 5.0], [3.0, 4.0, 2.0]], [[3.0, 4.0, 2.0], 
[6.0, 7.0, 4.0], [6.0, 2.0, 4.0]], [[1.0, 4.0, 7.0], [3.0, 6.0, 5.0], [4.0, 2.0, 7.0]]]])
      val=Data(numpy.array([[[7.0, 2.0, 3.0]], [[4.0, 3.0, 6.0]], [[8.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 5.0, 4.0]], [[5.0, 3.0, 4.0]], [[3.0, 2.0, 2.0]]]))
      arg[:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 6.0], [6.0, 5.0, 1.0], [3.0, 2.0, 5.0]], [[7.0, 3.0, 4.0], [7.0, 2.0, 3.0], [2.0, 
7.0, 4.0]], [[5.0, 7.0, 2.0], [3.0, 7.0, 2.0], [8.0, 7.0, 4.0]]], [[[3.0, 6.0, 2.0], [1.0, 8.0, 3.0], [3.0, 4.0, 8.0]], [[7.0, 
7.0, 8.0], [4.0, 3.0, 6.0], [7.0, 5.0, 3.0]], [[6.0, 7.0, 3.0], [3.0, 7.0, 3.0], [6.0, 5.0, 2.0]]], [[[7.0, 6.0, 8.0], [8.0, 
4.0, 2.0], [8.0, 5.0, 4.0]], [[2.0, 4.0, 6.0], [8.0, 3.0, 1.0], [3.0, 4.0, 3.0]], [[5.0, 6.0, 5.0], [2.0, 1.0, 8.0], [5.0, 3.0, 
7.0]]], [[[7.0, 6.0, 7.0], [7.0, 7.0, 8.0], [3.0, 5.0, 8.0]], [[4.0, 8.0, 8.0], [3.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[4.0, 2.0, 
8.0], [2.0, 4.0, 1.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 4.0], [5.0, 5.0, 3.0], [2.0, 6.0, 5.0]], 
[[2.0, 1.0, 4.0], [6.0, 5.0, 4.0], [6.0, 5.0, 4.0]], [[1.0, 7.0, 7.0], [3.0, 4.0, 3.0], [5.0, 1.0, 7.0]]], [[[4.0, 3.0, 6.0], 
[4.0, 7.0, 4.0], [3.0, 5.0, 8.0]], [[1.0, 6.0, 6.0], [5.0, 3.0, 4.0], [4.0, 1.0, 8.0]], [[7.0, 7.0, 4.0], [8.0, 3.0, 3.0], 
[1.0, 3.0, 4.0]]], [[[7.0, 5.0, 7.0], [3.0, 2.0, 8.0], [8.0, 5.0, 5.0]], [[7.0, 4.0, 8.0], [3.0, 2.0, 2.0], [6.0, 4.0, 2.0]], 
[[5.0, 3.0, 1.0], [5.0, 7.0, 5.0], [2.0, 7.0, 3.0]]], [[[4.0, 4.0, 8.0], [3.0, 1.0, 5.0], [3.0, 4.0, 2.0]], [[3.0, 4.0, 2.0], 
[6.0, 7.0, 4.0], [6.0, 2.0, 4.0]], [[1.0, 4.0, 7.0], [3.0, 6.0, 5.0], [4.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RPRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 2.0], [1.0, 6.0, 4.0], [2.0, 2.0, 7.0]], [[6.0, 4.0, 2.0], [3.0, 2.0, 6.0], [5.0, 
8.0, 6.0]], [[1.0, 1.0, 2.0], [5.0, 6.0, 2.0], [6.0, 5.0, 4.0]]], [[[1.0, 6.0, 8.0], [5.0, 5.0, 2.0], [8.0, 4.0, 1.0]], [[2.0, 
4.0, 5.0], [1.0, 4.0, 3.0], [6.0, 1.0, 4.0]], [[3.0, 2.0, 6.0], [5.0, 3.0, 7.0], [6.0, 1.0, 5.0]]], [[[8.0, 7.0, 3.0], [4.0, 
1.0, 3.0], [7.0, 3.0, 7.0]], [[5.0, 5.0, 5.0], [3.0, 5.0, 7.0], [3.0, 2.0, 6.0]], [[5.0, 8.0, 6.0], [6.0, 5.0, 6.0], [1.0, 3.0, 
5.0]]], [[[7.0, 3.0, 5.0], [6.0, 3.0, 2.0], [1.0, 5.0, 8.0]], [[5.0, 5.0, 8.0], [7.0, 8.0, 3.0], [4.0, 2.0, 5.0]], [[3.0, 5.0, 
6.0], [5.0, 4.0, 5.0], [1.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 7.0], [5.0, 2.0, 1.0], [7.0, 2.0, 8.0]], 
[[8.0, 7.0, 4.0], [1.0, 2.0, 2.0], [1.0, 1.0, 8.0]], [[5.0, 2.0, 2.0], [3.0, 8.0, 4.0], [8.0, 6.0, 8.0]]], [[[6.0, 6.0, 6.0], 
[6.0, 2.0, 3.0], [1.0, 3.0, 4.0]], [[7.0, 3.0, 5.0], [4.0, 8.0, 7.0], [5.0, 4.0, 8.0]], [[7.0, 6.0, 6.0], [7.0, 4.0, 8.0], 
[1.0, 8.0, 2.0]]], [[[2.0, 8.0, 5.0], [8.0, 6.0, 7.0], [5.0, 5.0, 5.0]], [[8.0, 3.0, 4.0], [4.0, 4.0, 4.0], [8.0, 6.0, 4.0]], 
[[7.0, 8.0, 8.0], [4.0, 5.0, 5.0], [5.0, 5.0, 4.0]]], [[[5.0, 2.0, 1.0], [4.0, 1.0, 4.0], [4.0, 6.0, 7.0]], [[5.0, 6.0, 5.0], 
[1.0, 6.0, 4.0], [3.0, 4.0, 6.0]], [[3.0, 7.0, 4.0], [2.0, 5.0, 8.0], [8.0, 1.0, 1.0]]]])
      val=Data(numpy.array([[[3.0, 8.0, 1.0]], [[6.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 7.0, 8.0]], [[3.0, 5.0, 8.0]]]))
      arg[1:3,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 2.0], [1.0, 6.0, 4.0], [2.0, 2.0, 7.0]], [[6.0, 4.0, 2.0], [3.0, 2.0, 6.0], [5.0, 
8.0, 6.0]], [[1.0, 1.0, 2.0], [5.0, 6.0, 2.0], [6.0, 5.0, 4.0]]], [[[1.0, 6.0, 8.0], [5.0, 5.0, 2.0], [8.0, 4.0, 1.0]], [[2.0, 
4.0, 5.0], [3.0, 8.0, 1.0], [6.0, 1.0, 4.0]], [[3.0, 2.0, 6.0], [5.0, 3.0, 7.0], [6.0, 1.0, 5.0]]], [[[8.0, 7.0, 3.0], [4.0, 
1.0, 3.0], [7.0, 3.0, 7.0]], [[5.0, 5.0, 5.0], [6.0, 3.0, 1.0], [3.0, 2.0, 6.0]], [[5.0, 8.0, 6.0], [6.0, 5.0, 6.0], [1.0, 3.0, 
5.0]]], [[[7.0, 3.0, 5.0], [6.0, 3.0, 2.0], [1.0, 5.0, 8.0]], [[5.0, 5.0, 8.0], [7.0, 8.0, 3.0], [4.0, 2.0, 5.0]], [[3.0, 5.0, 
6.0], [5.0, 4.0, 5.0], [1.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 7.0], [5.0, 2.0, 1.0], [7.0, 2.0, 8.0]], 
[[8.0, 7.0, 4.0], [1.0, 2.0, 2.0], [1.0, 1.0, 8.0]], [[5.0, 2.0, 2.0], [3.0, 8.0, 4.0], [8.0, 6.0, 8.0]]], [[[6.0, 6.0, 6.0], 
[6.0, 2.0, 3.0], [1.0, 3.0, 4.0]], [[7.0, 3.0, 5.0], [6.0, 7.0, 8.0], [5.0, 4.0, 8.0]], [[7.0, 6.0, 6.0], [7.0, 4.0, 8.0], 
[1.0, 8.0, 2.0]]], [[[2.0, 8.0, 5.0], [8.0, 6.0, 7.0], [5.0, 5.0, 5.0]], [[8.0, 3.0, 4.0], [3.0, 5.0, 8.0], [8.0, 6.0, 4.0]], 
[[7.0, 8.0, 8.0], [4.0, 5.0, 5.0], [5.0, 5.0, 4.0]]], [[[5.0, 2.0, 1.0], [4.0, 1.0, 4.0], [4.0, 6.0, 7.0]], [[5.0, 6.0, 5.0], 
[1.0, 6.0, 4.0], [3.0, 4.0, 6.0]], [[3.0, 7.0, 4.0], [2.0, 5.0, 8.0], [8.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LPPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 1.0], [3.0, 2.0, 4.0], [2.0, 2.0, 6.0]], [[3.0, 6.0, 8.0], [2.0, 5.0, 4.0], [2.0, 
5.0, 3.0]], [[8.0, 3.0, 7.0], [3.0, 3.0, 5.0], [2.0, 8.0, 7.0]]], [[[6.0, 6.0, 5.0], [4.0, 7.0, 4.0], [6.0, 4.0, 5.0]], [[6.0, 
8.0, 4.0], [4.0, 1.0, 6.0], [5.0, 2.0, 4.0]], [[1.0, 2.0, 8.0], [8.0, 2.0, 6.0], [5.0, 6.0, 4.0]]], [[[2.0, 5.0, 6.0], [6.0, 
3.0, 1.0], [1.0, 5.0, 5.0]], [[4.0, 4.0, 6.0], [2.0, 5.0, 2.0], [6.0, 4.0, 4.0]], [[3.0, 8.0, 4.0], [2.0, 6.0, 2.0], [7.0, 8.0, 
2.0]]], [[[1.0, 6.0, 7.0], [7.0, 3.0, 7.0], [3.0, 2.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 3.0, 1.0], [6.0, 6.0, 8.0]], [[5.0, 7.0, 
8.0], [5.0, 5.0, 8.0], [5.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 1.0], [2.0, 7.0, 5.0], [1.0, 1.0, 5.0]], 
[[8.0, 6.0, 5.0], [6.0, 7.0, 1.0], [8.0, 6.0, 6.0]], [[5.0, 3.0, 7.0], [2.0, 2.0, 1.0], [4.0, 8.0, 8.0]]], [[[2.0, 4.0, 5.0], 
[1.0, 6.0, 3.0], [1.0, 7.0, 8.0]], [[5.0, 4.0, 2.0], [5.0, 6.0, 1.0], [3.0, 4.0, 7.0]], [[4.0, 2.0, 4.0], [7.0, 5.0, 3.0], 
[1.0, 8.0, 7.0]]], [[[4.0, 6.0, 1.0], [5.0, 6.0, 5.0], [6.0, 7.0, 1.0]], [[2.0, 4.0, 8.0], [5.0, 7.0, 7.0], [4.0, 4.0, 2.0]], 
[[2.0, 6.0, 1.0], [8.0, 6.0, 4.0], [6.0, 6.0, 5.0]]], [[[1.0, 1.0, 2.0], [3.0, 2.0, 7.0], [2.0, 6.0, 7.0]], [[5.0, 5.0, 3.0], 
[4.0, 2.0, 8.0], [2.0, 6.0, 7.0]], [[2.0, 7.0, 2.0], [7.0, 5.0, 1.0], [6.0, 5.0, 2.0]]]])
      val=Data(numpy.array([[6.0, 3.0, 3.0], [2.0, 3.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 8.0, 8.0], [5.0, 1.0, 8.0]]))
      arg[2:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 1.0], [3.0, 2.0, 4.0], [2.0, 2.0, 6.0]], [[3.0, 6.0, 8.0], [2.0, 5.0, 4.0], [2.0, 
5.0, 3.0]], [[8.0, 3.0, 7.0], [3.0, 3.0, 5.0], [2.0, 8.0, 7.0]]], [[[6.0, 6.0, 5.0], [4.0, 7.0, 4.0], [6.0, 4.0, 5.0]], [[6.0, 
8.0, 4.0], [4.0, 1.0, 6.0], [5.0, 2.0, 4.0]], [[1.0, 2.0, 8.0], [8.0, 2.0, 6.0], [5.0, 6.0, 4.0]]], [[[2.0, 5.0, 6.0], [6.0, 
3.0, 1.0], [1.0, 5.0, 5.0]], [[4.0, 4.0, 6.0], [6.0, 3.0, 3.0], [6.0, 4.0, 4.0]], [[3.0, 8.0, 4.0], [2.0, 6.0, 2.0], [7.0, 8.0, 
2.0]]], [[[1.0, 6.0, 7.0], [7.0, 3.0, 7.0], [3.0, 2.0, 5.0]], [[7.0, 3.0, 2.0], [2.0, 3.0, 2.0], [6.0, 6.0, 8.0]], [[5.0, 7.0, 
8.0], [5.0, 5.0, 8.0], [5.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 1.0], [2.0, 7.0, 5.0], [1.0, 1.0, 5.0]], 
[[8.0, 6.0, 5.0], [6.0, 7.0, 1.0], [8.0, 6.0, 6.0]], [[5.0, 3.0, 7.0], [2.0, 2.0, 1.0], [4.0, 8.0, 8.0]]], [[[2.0, 4.0, 5.0], 
[1.0, 6.0, 3.0], [1.0, 7.0, 8.0]], [[5.0, 4.0, 2.0], [5.0, 6.0, 1.0], [3.0, 4.0, 7.0]], [[4.0, 2.0, 4.0], [7.0, 5.0, 3.0], 
[1.0, 8.0, 7.0]]], [[[4.0, 6.0, 1.0], [5.0, 6.0, 5.0], [6.0, 7.0, 1.0]], [[2.0, 4.0, 8.0], [8.0, 8.0, 8.0], [4.0, 4.0, 2.0]], 
[[2.0, 6.0, 1.0], [8.0, 6.0, 4.0], [6.0, 6.0, 5.0]]], [[[1.0, 1.0, 2.0], [3.0, 2.0, 7.0], [2.0, 6.0, 7.0]], [[5.0, 5.0, 3.0], 
[5.0, 1.0, 8.0], [2.0, 6.0, 7.0]], [[2.0, 7.0, 2.0], [7.0, 5.0, 1.0], [6.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UPCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 2.0, 2.0], [1.0, 2.0, 2.0], [6.0, 8.0, 6.0]], [[4.0, 2.0, 5.0], [4.0, 1.0, 3.0], [2.0, 
8.0, 3.0]], [[5.0, 5.0, 4.0], [4.0, 5.0, 8.0], [4.0, 6.0, 6.0]]], [[[2.0, 6.0, 1.0], [7.0, 5.0, 5.0], [5.0, 8.0, 2.0]], [[3.0, 
2.0, 3.0], [4.0, 7.0, 2.0], [8.0, 5.0, 5.0]], [[8.0, 5.0, 2.0], [2.0, 4.0, 3.0], [8.0, 5.0, 6.0]]], [[[5.0, 6.0, 8.0], [6.0, 
2.0, 6.0], [4.0, 5.0, 3.0]], [[4.0, 1.0, 5.0], [1.0, 4.0, 3.0], [5.0, 8.0, 3.0]], [[5.0, 5.0, 5.0], [8.0, 8.0, 8.0], [8.0, 6.0, 
4.0]]], [[[1.0, 4.0, 1.0], [3.0, 1.0, 3.0], [4.0, 6.0, 4.0]], [[1.0, 3.0, 4.0], [4.0, 4.0, 4.0], [1.0, 3.0, 7.0]], [[1.0, 2.0, 
7.0], [5.0, 1.0, 3.0], [8.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 2.0], [1.0, 2.0, 1.0], [3.0, 4.0, 1.0]], 
[[7.0, 5.0, 8.0], [3.0, 8.0, 3.0], [4.0, 4.0, 5.0]], [[2.0, 3.0, 5.0], [2.0, 4.0, 2.0], [6.0, 2.0, 7.0]]], [[[6.0, 3.0, 8.0], 
[8.0, 1.0, 6.0], [5.0, 8.0, 8.0]], [[2.0, 4.0, 4.0], [8.0, 5.0, 8.0], [3.0, 2.0, 2.0]], [[6.0, 7.0, 3.0], [5.0, 4.0, 7.0], 
[5.0, 6.0, 8.0]]], [[[5.0, 5.0, 1.0], [3.0, 2.0, 2.0], [2.0, 2.0, 5.0]], [[8.0, 4.0, 3.0], [1.0, 6.0, 6.0], [6.0, 4.0, 4.0]], 
[[4.0, 8.0, 2.0], [2.0, 4.0, 1.0], [3.0, 4.0, 7.0]]], [[[5.0, 8.0, 6.0], [6.0, 5.0, 2.0], [1.0, 2.0, 1.0]], [[7.0, 8.0, 8.0], 
[7.0, 6.0, 7.0], [4.0, 3.0, 1.0]], [[4.0, 2.0, 6.0], [4.0, 1.0, 8.0], [5.0, 1.0, 4.0]]]])
      val=Data(numpy.array([[[8.0, 4.0, 6.0], [3.0, 3.0, 1.0], [1.0, 6.0, 4.0]], [[1.0, 6.0, 8.0], [2.0, 5.0, 2.0], [8.0, 3.0, 
1.0]], [[3.0, 6.0, 4.0], [4.0, 6.0, 3.0], [1.0, 7.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 3.0, 7.0], [4.0, 1.0, 6.0], [4.0, 8.0, 7.0]], [[1.0, 5.0, 3.0], [3.0, 6.0, 8.0], 
[7.0, 8.0, 8.0]], [[2.0, 5.0, 5.0], [1.0, 7.0, 7.0], [6.0, 8.0, 2.0]]]))
      arg[:3,1,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 2.0], [1.0, 2.0, 2.0], [6.0, 8.0, 6.0]], [[8.0, 4.0, 6.0], [3.0, 3.0, 1.0], [1.0, 
6.0, 4.0]], [[5.0, 5.0, 4.0], [4.0, 5.0, 8.0], [4.0, 6.0, 6.0]]], [[[2.0, 6.0, 1.0], [7.0, 5.0, 5.0], [5.0, 8.0, 2.0]], [[1.0, 
6.0, 8.0], [2.0, 5.0, 2.0], [8.0, 3.0, 1.0]], [[8.0, 5.0, 2.0], [2.0, 4.0, 3.0], [8.0, 5.0, 6.0]]], [[[5.0, 6.0, 8.0], [6.0, 
2.0, 6.0], [4.0, 5.0, 3.0]], [[3.0, 6.0, 4.0], [4.0, 6.0, 3.0], [1.0, 7.0, 4.0]], [[5.0, 5.0, 5.0], [8.0, 8.0, 8.0], [8.0, 6.0, 
4.0]]], [[[1.0, 4.0, 1.0], [3.0, 1.0, 3.0], [4.0, 6.0, 4.0]], [[1.0, 3.0, 4.0], [4.0, 4.0, 4.0], [1.0, 3.0, 7.0]], [[1.0, 2.0, 
7.0], [5.0, 1.0, 3.0], [8.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 2.0], [1.0, 2.0, 1.0], [3.0, 4.0, 1.0]], 
[[7.0, 3.0, 7.0], [4.0, 1.0, 6.0], [4.0, 8.0, 7.0]], [[2.0, 3.0, 5.0], [2.0, 4.0, 2.0], [6.0, 2.0, 7.0]]], [[[6.0, 3.0, 8.0], 
[8.0, 1.0, 6.0], [5.0, 8.0, 8.0]], [[1.0, 5.0, 3.0], [3.0, 6.0, 8.0], [7.0, 8.0, 8.0]], [[6.0, 7.0, 3.0], [5.0, 4.0, 7.0], 
[5.0, 6.0, 8.0]]], [[[5.0, 5.0, 1.0], [3.0, 2.0, 2.0], [2.0, 2.0, 5.0]], [[2.0, 5.0, 5.0], [1.0, 7.0, 7.0], [6.0, 8.0, 2.0]], 
[[4.0, 8.0, 2.0], [2.0, 4.0, 1.0], [3.0, 4.0, 7.0]]], [[[5.0, 8.0, 6.0], [6.0, 5.0, 2.0], [1.0, 2.0, 1.0]], [[7.0, 8.0, 8.0], 
[7.0, 6.0, 7.0], [4.0, 3.0, 1.0]], [[4.0, 2.0, 6.0], [4.0, 1.0, 8.0], [5.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 3.0], [7.0, 4.0, 6.0], [2.0, 3.0, 7.0]], [[5.0, 3.0, 6.0], [5.0, 1.0, 4.0], [1.0, 
7.0, 4.0]], [[5.0, 4.0, 6.0], [1.0, 6.0, 3.0], [1.0, 3.0, 7.0]]], [[[8.0, 6.0, 1.0], [3.0, 7.0, 2.0], [5.0, 6.0, 1.0]], [[7.0, 
3.0, 1.0], [7.0, 8.0, 8.0], [2.0, 3.0, 6.0]], [[8.0, 8.0, 7.0], [5.0, 2.0, 3.0], [1.0, 8.0, 1.0]]], [[[4.0, 5.0, 1.0], [6.0, 
4.0, 4.0], [4.0, 3.0, 6.0]], [[7.0, 1.0, 8.0], [6.0, 7.0, 7.0], [4.0, 4.0, 6.0]], [[1.0, 7.0, 4.0], [8.0, 2.0, 1.0], [1.0, 1.0, 
3.0]]], [[[4.0, 3.0, 1.0], [7.0, 1.0, 4.0], [4.0, 3.0, 8.0]], [[6.0, 3.0, 4.0], [3.0, 7.0, 1.0], [2.0, 1.0, 2.0]], [[6.0, 5.0, 
4.0], [1.0, 8.0, 8.0], [3.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 4.0], [4.0, 3.0, 8.0], [7.0, 1.0, 4.0]], 
[[4.0, 5.0, 1.0], [8.0, 1.0, 7.0], [8.0, 1.0, 6.0]], [[5.0, 5.0, 4.0], [8.0, 2.0, 7.0], [4.0, 6.0, 5.0]]], [[[1.0, 5.0, 5.0], 
[1.0, 5.0, 1.0], [5.0, 5.0, 1.0]], [[7.0, 5.0, 8.0], [6.0, 3.0, 1.0], [5.0, 1.0, 2.0]], [[8.0, 1.0, 4.0], [4.0, 3.0, 3.0], 
[5.0, 2.0, 5.0]]], [[[4.0, 5.0, 4.0], [6.0, 3.0, 4.0], [8.0, 1.0, 3.0]], [[3.0, 2.0, 6.0], [7.0, 2.0, 4.0], [2.0, 1.0, 8.0]], 
[[1.0, 3.0, 1.0], [1.0, 6.0, 6.0], [8.0, 6.0, 1.0]]], [[[1.0, 6.0, 2.0], [5.0, 5.0, 5.0], [1.0, 2.0, 2.0]], [[6.0, 6.0, 4.0], 
[1.0, 3.0, 3.0], [5.0, 8.0, 3.0]], [[1.0, 5.0, 1.0], [8.0, 1.0, 6.0], [6.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[2.0, 2.0, 6.0], [5.0, 5.0, 7.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[4.0, 1.0, 5.0], [3.0, 8.0, 1.0]]))
      arg[2,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 3.0], [7.0, 4.0, 6.0], [2.0, 3.0, 7.0]], [[5.0, 3.0, 6.0], [5.0, 1.0, 4.0], [1.0, 
7.0, 4.0]], [[5.0, 4.0, 6.0], [1.0, 6.0, 3.0], [1.0, 3.0, 7.0]]], [[[8.0, 6.0, 1.0], [3.0, 7.0, 2.0], [5.0, 6.0, 1.0]], [[7.0, 
3.0, 1.0], [7.0, 8.0, 8.0], [2.0, 3.0, 6.0]], [[8.0, 8.0, 7.0], [5.0, 2.0, 3.0], [1.0, 8.0, 1.0]]], [[[4.0, 5.0, 1.0], [6.0, 
4.0, 4.0], [4.0, 3.0, 6.0]], [[2.0, 2.0, 6.0], [5.0, 5.0, 7.0], [4.0, 4.0, 6.0]], [[1.0, 7.0, 4.0], [8.0, 2.0, 1.0], [1.0, 1.0, 
3.0]]], [[[4.0, 3.0, 1.0], [7.0, 1.0, 4.0], [4.0, 3.0, 8.0]], [[6.0, 3.0, 4.0], [3.0, 7.0, 1.0], [2.0, 1.0, 2.0]], [[6.0, 5.0, 
4.0], [1.0, 8.0, 8.0], [3.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 4.0], [4.0, 3.0, 8.0], [7.0, 1.0, 4.0]], 
[[4.0, 5.0, 1.0], [8.0, 1.0, 7.0], [8.0, 1.0, 6.0]], [[5.0, 5.0, 4.0], [8.0, 2.0, 7.0], [4.0, 6.0, 5.0]]], [[[1.0, 5.0, 5.0], 
[1.0, 5.0, 1.0], [5.0, 5.0, 1.0]], [[7.0, 5.0, 8.0], [6.0, 3.0, 1.0], [5.0, 1.0, 2.0]], [[8.0, 1.0, 4.0], [4.0, 3.0, 3.0], 
[5.0, 2.0, 5.0]]], [[[4.0, 5.0, 4.0], [6.0, 3.0, 4.0], [8.0, 1.0, 3.0]], [[4.0, 1.0, 5.0], [3.0, 8.0, 1.0], [2.0, 1.0, 8.0]], 
[[1.0, 3.0, 1.0], [1.0, 6.0, 6.0], [8.0, 6.0, 1.0]]], [[[1.0, 6.0, 2.0], [5.0, 5.0, 5.0], [1.0, 2.0, 2.0]], [[6.0, 6.0, 4.0], 
[1.0, 3.0, 3.0], [5.0, 8.0, 3.0]], [[1.0, 5.0, 1.0], [8.0, 1.0, 6.0], [6.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UULN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 6.0], [7.0, 7.0, 8.0], [8.0, 4.0, 5.0]], [[8.0, 4.0, 6.0], [7.0, 6.0, 3.0], [3.0, 
1.0, 1.0]], [[3.0, 3.0, 3.0], [7.0, 5.0, 6.0], [7.0, 4.0, 2.0]]], [[[4.0, 6.0, 3.0], [1.0, 4.0, 8.0], [2.0, 1.0, 7.0]], [[5.0, 
5.0, 1.0], [4.0, 7.0, 7.0], [4.0, 1.0, 6.0]], [[3.0, 3.0, 2.0], [3.0, 2.0, 5.0], [8.0, 5.0, 3.0]]], [[[3.0, 7.0, 4.0], [3.0, 
7.0, 3.0], [8.0, 1.0, 7.0]], [[3.0, 3.0, 1.0], [3.0, 6.0, 3.0], [3.0, 5.0, 8.0]], [[4.0, 8.0, 2.0], [7.0, 5.0, 4.0], [7.0, 2.0, 
3.0]]], [[[5.0, 8.0, 7.0], [4.0, 1.0, 5.0], [3.0, 1.0, 4.0]], [[7.0, 7.0, 4.0], [5.0, 1.0, 4.0], [5.0, 8.0, 4.0]], [[3.0, 1.0, 
4.0], [3.0, 8.0, 3.0], [1.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 7.0], [6.0, 4.0, 2.0], [8.0, 8.0, 7.0]], 
[[4.0, 7.0, 3.0], [4.0, 3.0, 1.0], [4.0, 5.0, 7.0]], [[3.0, 3.0, 8.0], [2.0, 6.0, 7.0], [6.0, 5.0, 5.0]]], [[[3.0, 1.0, 8.0], 
[8.0, 4.0, 7.0], [8.0, 3.0, 8.0]], [[8.0, 8.0, 3.0], [7.0, 7.0, 6.0], [6.0, 8.0, 3.0]], [[6.0, 4.0, 8.0], [2.0, 6.0, 8.0], 
[2.0, 1.0, 3.0]]], [[[1.0, 7.0, 3.0], [1.0, 4.0, 7.0], [5.0, 8.0, 5.0]], [[6.0, 3.0, 4.0], [2.0, 6.0, 5.0], [5.0, 2.0, 6.0]], 
[[4.0, 1.0, 6.0], [3.0, 8.0, 1.0], [7.0, 2.0, 2.0]]], [[[4.0, 5.0, 8.0], [6.0, 2.0, 1.0], [6.0, 6.0, 3.0]], [[4.0, 5.0, 7.0], 
[2.0, 5.0, 3.0], [4.0, 5.0, 5.0]], [[8.0, 3.0, 3.0], [4.0, 1.0, 7.0], [8.0, 4.0, 7.0]]]])
      val=Data(numpy.array([[[[8.0, 7.0, 8.0]], [[4.0, 2.0, 5.0]]], [[[2.0, 1.0, 4.0]], [[6.0, 2.0, 5.0]]], [[[1.0, 3.0, 2.0]], 
[[7.0, 2.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 4.0, 8.0]], [[4.0, 8.0, 1.0]]], [[[8.0, 7.0, 8.0]], [[7.0, 8.0, 7.0]]], [[[1.0, 
8.0, 3.0]], [[6.0, 5.0, 6.0]]]]))
      arg[:3,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 6.0], [7.0, 7.0, 8.0], [8.0, 7.0, 8.0]], [[8.0, 4.0, 6.0], [7.0, 6.0, 3.0], [4.0, 
2.0, 5.0]], [[3.0, 3.0, 3.0], [7.0, 5.0, 6.0], [7.0, 4.0, 2.0]]], [[[4.0, 6.0, 3.0], [1.0, 4.0, 8.0], [2.0, 1.0, 4.0]], [[5.0, 
5.0, 1.0], [4.0, 7.0, 7.0], [6.0, 2.0, 5.0]], [[3.0, 3.0, 2.0], [3.0, 2.0, 5.0], [8.0, 5.0, 3.0]]], [[[3.0, 7.0, 4.0], [3.0, 
7.0, 3.0], [1.0, 3.0, 2.0]], [[3.0, 3.0, 1.0], [3.0, 6.0, 3.0], [7.0, 2.0, 8.0]], [[4.0, 8.0, 2.0], [7.0, 5.0, 4.0], [7.0, 2.0, 
3.0]]], [[[5.0, 8.0, 7.0], [4.0, 1.0, 5.0], [3.0, 1.0, 4.0]], [[7.0, 7.0, 4.0], [5.0, 1.0, 4.0], [5.0, 8.0, 4.0]], [[3.0, 1.0, 
4.0], [3.0, 8.0, 3.0], [1.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 7.0], [6.0, 4.0, 2.0], [5.0, 4.0, 8.0]], 
[[4.0, 7.0, 3.0], [4.0, 3.0, 1.0], [4.0, 8.0, 1.0]], [[3.0, 3.0, 8.0], [2.0, 6.0, 7.0], [6.0, 5.0, 5.0]]], [[[3.0, 1.0, 8.0], 
[8.0, 4.0, 7.0], [8.0, 7.0, 8.0]], [[8.0, 8.0, 3.0], [7.0, 7.0, 6.0], [7.0, 8.0, 7.0]], [[6.0, 4.0, 8.0], [2.0, 6.0, 8.0], 
[2.0, 1.0, 3.0]]], [[[1.0, 7.0, 3.0], [1.0, 4.0, 7.0], [1.0, 8.0, 3.0]], [[6.0, 3.0, 4.0], [2.0, 6.0, 5.0], [6.0, 5.0, 6.0]], 
[[4.0, 1.0, 6.0], [3.0, 8.0, 1.0], [7.0, 2.0, 2.0]]], [[[4.0, 5.0, 8.0], [6.0, 2.0, 1.0], [6.0, 6.0, 3.0]], [[4.0, 5.0, 7.0], 
[2.0, 5.0, 3.0], [4.0, 5.0, 5.0]], [[8.0, 3.0, 3.0], [4.0, 1.0, 7.0], [8.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 6.0], [4.0, 6.0, 4.0], [7.0, 7.0, 6.0]], [[6.0, 1.0, 1.0], [1.0, 4.0, 1.0], [3.0, 
3.0, 6.0]], [[2.0, 7.0, 1.0], [7.0, 7.0, 5.0], [7.0, 1.0, 8.0]]], [[[5.0, 5.0, 3.0], [7.0, 8.0, 5.0], [4.0, 7.0, 2.0]], [[6.0, 
7.0, 2.0], [4.0, 7.0, 3.0], [4.0, 7.0, 2.0]], [[4.0, 5.0, 6.0], [4.0, 7.0, 8.0], [6.0, 8.0, 5.0]]], [[[8.0, 2.0, 2.0], [7.0, 
6.0, 5.0], [2.0, 4.0, 5.0]], [[7.0, 7.0, 5.0], [1.0, 2.0, 8.0], [8.0, 8.0, 3.0]], [[8.0, 7.0, 6.0], [8.0, 1.0, 6.0], [7.0, 1.0, 
2.0]]], [[[5.0, 4.0, 1.0], [3.0, 5.0, 7.0], [1.0, 5.0, 1.0]], [[5.0, 6.0, 8.0], [3.0, 4.0, 3.0], [2.0, 8.0, 7.0]], [[2.0, 6.0, 
8.0], [3.0, 7.0, 7.0], [3.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 5.0], [1.0, 4.0, 4.0], [3.0, 5.0, 6.0]], 
[[8.0, 1.0, 6.0], [4.0, 2.0, 8.0], [1.0, 8.0, 4.0]], [[2.0, 3.0, 3.0], [5.0, 6.0, 5.0], [7.0, 5.0, 1.0]]], [[[7.0, 7.0, 5.0], 
[2.0, 4.0, 7.0], [6.0, 6.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 7.0, 8.0], [4.0, 2.0, 6.0]], [[1.0, 8.0, 4.0], [5.0, 1.0, 7.0], 
[1.0, 4.0, 8.0]]], [[[1.0, 4.0, 7.0], [7.0, 8.0, 3.0], [8.0, 8.0, 5.0]], [[7.0, 8.0, 8.0], [7.0, 5.0, 3.0], [7.0, 7.0, 6.0]], 
[[8.0, 1.0, 3.0], [8.0, 8.0, 7.0], [3.0, 4.0, 8.0]]], [[[8.0, 3.0, 8.0], [5.0, 6.0, 8.0], [7.0, 8.0, 6.0]], [[4.0, 5.0, 7.0], 
[2.0, 3.0, 6.0], [6.0, 5.0, 2.0]], [[2.0, 7.0, 1.0], [7.0, 5.0, 7.0], [4.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[7.0, 5.0, 8.0]], [[4.0, 4.0, 5.0]], [[2.0, 7.0, 4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 7.0, 2.0]], [[6.0, 4.0, 5.0]], [[8.0, 2.0, 4.0]]]))
      arg[:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 6.0], [4.0, 6.0, 4.0], [7.0, 7.0, 6.0]], [[6.0, 1.0, 1.0], [7.0, 5.0, 8.0], [3.0, 
3.0, 6.0]], [[2.0, 7.0, 1.0], [7.0, 7.0, 5.0], [7.0, 1.0, 8.0]]], [[[5.0, 5.0, 3.0], [7.0, 8.0, 5.0], [4.0, 7.0, 2.0]], [[6.0, 
7.0, 2.0], [4.0, 4.0, 5.0], [4.0, 7.0, 2.0]], [[4.0, 5.0, 6.0], [4.0, 7.0, 8.0], [6.0, 8.0, 5.0]]], [[[8.0, 2.0, 2.0], [7.0, 
6.0, 5.0], [2.0, 4.0, 5.0]], [[7.0, 7.0, 5.0], [2.0, 7.0, 4.0], [8.0, 8.0, 3.0]], [[8.0, 7.0, 6.0], [8.0, 1.0, 6.0], [7.0, 1.0, 
2.0]]], [[[5.0, 4.0, 1.0], [3.0, 5.0, 7.0], [1.0, 5.0, 1.0]], [[5.0, 6.0, 8.0], [3.0, 4.0, 3.0], [2.0, 8.0, 7.0]], [[2.0, 6.0, 
8.0], [3.0, 7.0, 7.0], [3.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 5.0], [1.0, 4.0, 4.0], [3.0, 5.0, 6.0]], 
[[8.0, 1.0, 6.0], [2.0, 7.0, 2.0], [1.0, 8.0, 4.0]], [[2.0, 3.0, 3.0], [5.0, 6.0, 5.0], [7.0, 5.0, 1.0]]], [[[7.0, 7.0, 5.0], 
[2.0, 4.0, 7.0], [6.0, 6.0, 3.0]], [[1.0, 1.0, 7.0], [6.0, 4.0, 5.0], [4.0, 2.0, 6.0]], [[1.0, 8.0, 4.0], [5.0, 1.0, 7.0], 
[1.0, 4.0, 8.0]]], [[[1.0, 4.0, 7.0], [7.0, 8.0, 3.0], [8.0, 8.0, 5.0]], [[7.0, 8.0, 8.0], [8.0, 2.0, 4.0], [7.0, 7.0, 6.0]], 
[[8.0, 1.0, 3.0], [8.0, 8.0, 7.0], [3.0, 4.0, 8.0]]], [[[8.0, 3.0, 8.0], [5.0, 6.0, 8.0], [7.0, 8.0, 6.0]], [[4.0, 5.0, 7.0], 
[2.0, 3.0, 6.0], [6.0, 5.0, 2.0]], [[2.0, 7.0, 1.0], [7.0, 5.0, 7.0], [4.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLLU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 1.0], [8.0, 2.0, 2.0], [2.0, 7.0, 7.0]], [[4.0, 7.0, 3.0], [8.0, 6.0, 7.0], [1.0, 
7.0, 4.0]], [[3.0, 3.0, 2.0], [8.0, 2.0, 2.0], [8.0, 5.0, 1.0]]], [[[1.0, 5.0, 1.0], [8.0, 5.0, 5.0], [4.0, 6.0, 3.0]], [[3.0, 
7.0, 4.0], [1.0, 8.0, 3.0], [4.0, 8.0, 8.0]], [[4.0, 2.0, 2.0], [4.0, 3.0, 1.0], [3.0, 5.0, 8.0]]], [[[4.0, 7.0, 2.0], [7.0, 
8.0, 7.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 1.0], [2.0, 1.0, 3.0], [6.0, 3.0, 8.0]], [[6.0, 4.0, 4.0], [3.0, 8.0, 7.0], [1.0, 6.0, 
3.0]]], [[[7.0, 5.0, 7.0], [3.0, 2.0, 4.0], [6.0, 2.0, 4.0]], [[8.0, 3.0, 4.0], [4.0, 6.0, 7.0], [1.0, 3.0, 5.0]], [[2.0, 7.0, 
1.0], [1.0, 8.0, 6.0], [3.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 1.0]], 
[[4.0, 1.0, 1.0], [3.0, 7.0, 8.0], [1.0, 8.0, 2.0]], [[5.0, 2.0, 3.0], [6.0, 1.0, 8.0], [1.0, 6.0, 4.0]]], [[[5.0, 4.0, 2.0], 
[7.0, 7.0, 7.0], [6.0, 3.0, 8.0]], [[8.0, 8.0, 7.0], [8.0, 6.0, 1.0], [7.0, 1.0, 3.0]], [[2.0, 4.0, 6.0], [4.0, 4.0, 5.0], 
[6.0, 6.0, 4.0]]], [[[2.0, 1.0, 7.0], [4.0, 6.0, 8.0], [8.0, 8.0, 6.0]], [[1.0, 7.0, 8.0], [6.0, 4.0, 4.0], [8.0, 6.0, 8.0]], 
[[6.0, 5.0, 8.0], [6.0, 8.0, 3.0], [4.0, 1.0, 4.0]]], [[[8.0, 3.0, 5.0], [2.0, 6.0, 3.0], [7.0, 6.0, 1.0]], [[1.0, 1.0, 7.0], 
[2.0, 7.0, 1.0], [7.0, 6.0, 7.0]], [[1.0, 4.0, 6.0], [6.0, 1.0, 5.0], [5.0, 8.0, 2.0]]]])
      val=Data(numpy.array([[[[3.0, 7.0]]], [[[4.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 8.0]]], [[[7.0, 6.0]]]]))
      arg[2:,2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 1.0], [8.0, 2.0, 2.0], [2.0, 7.0, 7.0]], [[4.0, 7.0, 3.0], [8.0, 6.0, 7.0], [1.0, 
7.0, 4.0]], [[3.0, 3.0, 2.0], [8.0, 2.0, 2.0], [8.0, 5.0, 1.0]]], [[[1.0, 5.0, 1.0], [8.0, 5.0, 5.0], [4.0, 6.0, 3.0]], [[3.0, 
7.0, 4.0], [1.0, 8.0, 3.0], [4.0, 8.0, 8.0]], [[4.0, 2.0, 2.0], [4.0, 3.0, 1.0], [3.0, 5.0, 8.0]]], [[[4.0, 7.0, 2.0], [7.0, 
8.0, 7.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 1.0], [2.0, 1.0, 3.0], [6.0, 3.0, 8.0]], [[6.0, 4.0, 4.0], [3.0, 8.0, 7.0], [3.0, 7.0, 
3.0]]], [[[7.0, 5.0, 7.0], [3.0, 2.0, 4.0], [6.0, 2.0, 4.0]], [[8.0, 3.0, 4.0], [4.0, 6.0, 7.0], [1.0, 3.0, 5.0]], [[2.0, 7.0, 
1.0], [1.0, 8.0, 6.0], [4.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 1.0]], 
[[4.0, 1.0, 1.0], [3.0, 7.0, 8.0], [1.0, 8.0, 2.0]], [[5.0, 2.0, 3.0], [6.0, 1.0, 8.0], [1.0, 6.0, 4.0]]], [[[5.0, 4.0, 2.0], 
[7.0, 7.0, 7.0], [6.0, 3.0, 8.0]], [[8.0, 8.0, 7.0], [8.0, 6.0, 1.0], [7.0, 1.0, 3.0]], [[2.0, 4.0, 6.0], [4.0, 4.0, 5.0], 
[6.0, 6.0, 4.0]]], [[[2.0, 1.0, 7.0], [4.0, 6.0, 8.0], [8.0, 8.0, 6.0]], [[1.0, 7.0, 8.0], [6.0, 4.0, 4.0], [8.0, 6.0, 8.0]], 
[[6.0, 5.0, 8.0], [6.0, 8.0, 3.0], [5.0, 8.0, 4.0]]], [[[8.0, 3.0, 5.0], [2.0, 6.0, 3.0], [7.0, 6.0, 1.0]], [[1.0, 1.0, 7.0], 
[2.0, 7.0, 1.0], [7.0, 6.0, 7.0]], [[1.0, 4.0, 6.0], [6.0, 1.0, 5.0], [7.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 1.0], [1.0, 1.0, 8.0], [5.0, 2.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 5.0, 2.0], [6.0, 
2.0, 3.0]], [[3.0, 3.0, 7.0], [7.0, 8.0, 3.0], [1.0, 6.0, 5.0]]], [[[1.0, 8.0, 1.0], [2.0, 8.0, 1.0], [3.0, 4.0, 1.0]], [[3.0, 
6.0, 5.0], [3.0, 3.0, 4.0], [3.0, 2.0, 8.0]], [[5.0, 8.0, 6.0], [3.0, 6.0, 7.0], [2.0, 1.0, 5.0]]], [[[3.0, 7.0, 1.0], [4.0, 
6.0, 4.0], [3.0, 1.0, 2.0]], [[5.0, 4.0, 4.0], [3.0, 6.0, 5.0], [8.0, 8.0, 8.0]], [[3.0, 3.0, 2.0], [8.0, 6.0, 5.0], [5.0, 1.0, 
3.0]]], [[[7.0, 8.0, 3.0], [8.0, 2.0, 4.0], [7.0, 5.0, 6.0]], [[5.0, 6.0, 5.0], [4.0, 4.0, 5.0], [1.0, 5.0, 8.0]], [[2.0, 8.0, 
3.0], [2.0, 5.0, 1.0], [3.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 7.0], [5.0, 4.0, 5.0], [3.0, 7.0, 3.0]], 
[[1.0, 2.0, 4.0], [1.0, 4.0, 5.0], [6.0, 6.0, 8.0]], [[1.0, 8.0, 1.0], [5.0, 2.0, 3.0], [5.0, 3.0, 4.0]]], [[[2.0, 3.0, 5.0], 
[8.0, 6.0, 6.0], [1.0, 6.0, 3.0]], [[6.0, 4.0, 4.0], [2.0, 7.0, 8.0], [3.0, 5.0, 6.0]], [[3.0, 2.0, 1.0], [2.0, 8.0, 5.0], 
[2.0, 3.0, 3.0]]], [[[6.0, 5.0, 5.0], [6.0, 1.0, 7.0], [7.0, 2.0, 8.0]], [[3.0, 2.0, 8.0], [8.0, 7.0, 3.0], [1.0, 4.0, 1.0]], 
[[1.0, 4.0, 7.0], [5.0, 1.0, 1.0], [3.0, 7.0, 1.0]]], [[[1.0, 8.0, 1.0], [7.0, 8.0, 5.0], [1.0, 5.0, 1.0]], [[7.0, 5.0, 2.0], 
[1.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[1.0, 3.0, 6.0], [6.0, 6.0, 8.0], [2.0, 3.0, 3.0]]]])
      val=Data(numpy.array([[[[6.0, 7.0, 3.0]]], [[[4.0, 7.0, 7.0]]], [[[7.0, 3.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 7.0, 6.0]]], [[[3.0, 4.0, 3.0]]], [[[2.0, 6.0, 1.0]]]]))
      arg[:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 1.0], [1.0, 1.0, 8.0], [5.0, 2.0, 7.0]], [[6.0, 3.0, 7.0], [6.0, 7.0, 3.0], [6.0, 
2.0, 3.0]], [[3.0, 3.0, 7.0], [7.0, 8.0, 3.0], [1.0, 6.0, 5.0]]], [[[1.0, 8.0, 1.0], [2.0, 8.0, 1.0], [3.0, 4.0, 1.0]], [[3.0, 
6.0, 5.0], [4.0, 7.0, 7.0], [3.0, 2.0, 8.0]], [[5.0, 8.0, 6.0], [3.0, 6.0, 7.0], [2.0, 1.0, 5.0]]], [[[3.0, 7.0, 1.0], [4.0, 
6.0, 4.0], [3.0, 1.0, 2.0]], [[5.0, 4.0, 4.0], [7.0, 3.0, 3.0], [8.0, 8.0, 8.0]], [[3.0, 3.0, 2.0], [8.0, 6.0, 5.0], [5.0, 1.0, 
3.0]]], [[[7.0, 8.0, 3.0], [8.0, 2.0, 4.0], [7.0, 5.0, 6.0]], [[5.0, 6.0, 5.0], [4.0, 4.0, 5.0], [1.0, 5.0, 8.0]], [[2.0, 8.0, 
3.0], [2.0, 5.0, 1.0], [3.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 7.0], [5.0, 4.0, 5.0], [3.0, 7.0, 3.0]], 
[[1.0, 2.0, 4.0], [3.0, 7.0, 6.0], [6.0, 6.0, 8.0]], [[1.0, 8.0, 1.0], [5.0, 2.0, 3.0], [5.0, 3.0, 4.0]]], [[[2.0, 3.0, 5.0], 
[8.0, 6.0, 6.0], [1.0, 6.0, 3.0]], [[6.0, 4.0, 4.0], [3.0, 4.0, 3.0], [3.0, 5.0, 6.0]], [[3.0, 2.0, 1.0], [2.0, 8.0, 5.0], 
[2.0, 3.0, 3.0]]], [[[6.0, 5.0, 5.0], [6.0, 1.0, 7.0], [7.0, 2.0, 8.0]], [[3.0, 2.0, 8.0], [2.0, 6.0, 1.0], [1.0, 4.0, 1.0]], 
[[1.0, 4.0, 7.0], [5.0, 1.0, 1.0], [3.0, 7.0, 1.0]]], [[[1.0, 8.0, 1.0], [7.0, 8.0, 5.0], [1.0, 5.0, 1.0]], [[7.0, 5.0, 2.0], 
[1.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[1.0, 3.0, 6.0], [6.0, 6.0, 8.0], [2.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PCCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 5.0, 5.0], [1.0, 2.0, 5.0], [1.0, 3.0, 6.0]], [[1.0, 2.0, 8.0], [6.0, 7.0, 4.0], [5.0, 
8.0, 8.0]], [[1.0, 6.0, 7.0], [3.0, 6.0, 6.0], [2.0, 7.0, 3.0]]], [[[8.0, 8.0, 3.0], [5.0, 6.0, 7.0], [2.0, 5.0, 1.0]], [[1.0, 
6.0, 2.0], [7.0, 1.0, 1.0], [7.0, 7.0, 8.0]], [[4.0, 5.0, 6.0], [6.0, 5.0, 2.0], [6.0, 7.0, 1.0]]], [[[3.0, 8.0, 5.0], [7.0, 
6.0, 7.0], [3.0, 3.0, 7.0]], [[8.0, 1.0, 1.0], [2.0, 4.0, 8.0], [8.0, 3.0, 3.0]], [[7.0, 4.0, 5.0], [1.0, 5.0, 8.0], [2.0, 2.0, 
2.0]]], [[[5.0, 6.0, 7.0], [7.0, 3.0, 1.0], [1.0, 3.0, 6.0]], [[2.0, 5.0, 3.0], [7.0, 5.0, 3.0], [8.0, 6.0, 1.0]], [[4.0, 8.0, 
4.0], [5.0, 7.0, 8.0], [1.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 7.0], [4.0, 1.0, 5.0], [5.0, 1.0, 7.0]], 
[[4.0, 8.0, 7.0], [6.0, 8.0, 5.0], [6.0, 1.0, 7.0]], [[4.0, 8.0, 7.0], [6.0, 6.0, 5.0], [7.0, 8.0, 5.0]]], [[[1.0, 5.0, 8.0], 
[4.0, 4.0, 4.0], [6.0, 7.0, 3.0]], [[2.0, 7.0, 6.0], [6.0, 7.0, 2.0], [6.0, 4.0, 1.0]], [[8.0, 2.0, 1.0], [1.0, 2.0, 4.0], 
[4.0, 6.0, 7.0]]], [[[1.0, 7.0, 3.0], [5.0, 5.0, 8.0], [4.0, 8.0, 4.0]], [[8.0, 7.0, 4.0], [7.0, 6.0, 4.0], [1.0, 3.0, 3.0]], 
[[1.0, 2.0, 5.0], [3.0, 5.0, 5.0], [7.0, 6.0, 5.0]]], [[[6.0, 5.0, 3.0], [1.0, 5.0, 1.0], [4.0, 6.0, 6.0]], [[7.0, 2.0, 8.0], 
[6.0, 2.0, 4.0], [6.0, 8.0, 6.0]], [[2.0, 7.0, 7.0], [1.0, 3.0, 5.0], [4.0, 4.0, 7.0]]]])
      val=Data(numpy.array([[[3.0, 7.0, 7.0], [3.0, 2.0, 6.0], [2.0, 7.0, 5.0]], [[7.0, 6.0, 1.0], [6.0, 8.0, 8.0], [5.0, 4.0, 
2.0]], [[4.0, 8.0, 5.0], [6.0, 7.0, 2.0], [1.0, 8.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 4.0, 3.0], [7.0, 5.0, 5.0], [7.0, 3.0, 6.0]], [[6.0, 3.0, 1.0], [3.0, 7.0, 1.0], 
[3.0, 6.0, 5.0]], [[1.0, 8.0, 3.0], [6.0, 3.0, 2.0], [1.0, 3.0, 2.0]]]))
      arg[2,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 5.0], [1.0, 2.0, 5.0], [1.0, 3.0, 6.0]], [[1.0, 2.0, 8.0], [6.0, 7.0, 4.0], [5.0, 
8.0, 8.0]], [[1.0, 6.0, 7.0], [3.0, 6.0, 6.0], [2.0, 7.0, 3.0]]], [[[8.0, 8.0, 3.0], [5.0, 6.0, 7.0], [2.0, 5.0, 1.0]], [[1.0, 
6.0, 2.0], [7.0, 1.0, 1.0], [7.0, 7.0, 8.0]], [[4.0, 5.0, 6.0], [6.0, 5.0, 2.0], [6.0, 7.0, 1.0]]], [[[3.0, 7.0, 7.0], [3.0, 
2.0, 6.0], [2.0, 7.0, 5.0]], [[7.0, 6.0, 1.0], [6.0, 8.0, 8.0], [5.0, 4.0, 2.0]], [[4.0, 8.0, 5.0], [6.0, 7.0, 2.0], [1.0, 8.0, 
8.0]]], [[[5.0, 6.0, 7.0], [7.0, 3.0, 1.0], [1.0, 3.0, 6.0]], [[2.0, 5.0, 3.0], [7.0, 5.0, 3.0], [8.0, 6.0, 1.0]], [[4.0, 8.0, 
4.0], [5.0, 7.0, 8.0], [1.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 7.0], [4.0, 1.0, 5.0], [5.0, 1.0, 7.0]], 
[[4.0, 8.0, 7.0], [6.0, 8.0, 5.0], [6.0, 1.0, 7.0]], [[4.0, 8.0, 7.0], [6.0, 6.0, 5.0], [7.0, 8.0, 5.0]]], [[[1.0, 5.0, 8.0], 
[4.0, 4.0, 4.0], [6.0, 7.0, 3.0]], [[2.0, 7.0, 6.0], [6.0, 7.0, 2.0], [6.0, 4.0, 1.0]], [[8.0, 2.0, 1.0], [1.0, 2.0, 4.0], 
[4.0, 6.0, 7.0]]], [[[3.0, 4.0, 3.0], [7.0, 5.0, 5.0], [7.0, 3.0, 6.0]], [[6.0, 3.0, 1.0], [3.0, 7.0, 1.0], [3.0, 6.0, 5.0]], 
[[1.0, 8.0, 3.0], [6.0, 3.0, 2.0], [1.0, 3.0, 2.0]]], [[[6.0, 5.0, 3.0], [1.0, 5.0, 1.0], [4.0, 6.0, 6.0]], [[7.0, 2.0, 8.0], 
[6.0, 2.0, 4.0], [6.0, 8.0, 6.0]], [[2.0, 7.0, 7.0], [1.0, 3.0, 5.0], [4.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RUPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 4.0], [2.0, 4.0, 7.0], [8.0, 1.0, 6.0]], [[7.0, 5.0, 1.0], [7.0, 7.0, 1.0], [8.0, 
6.0, 3.0]], [[4.0, 4.0, 1.0], [7.0, 6.0, 4.0], [2.0, 7.0, 4.0]]], [[[7.0, 1.0, 7.0], [7.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[1.0, 
2.0, 4.0], [8.0, 4.0, 1.0], [1.0, 7.0, 1.0]], [[1.0, 4.0, 2.0], [5.0, 5.0, 1.0], [8.0, 6.0, 7.0]]], [[[5.0, 1.0, 1.0], [2.0, 
2.0, 3.0], [6.0, 2.0, 7.0]], [[1.0, 4.0, 6.0], [7.0, 3.0, 7.0], [3.0, 4.0, 1.0]], [[1.0, 4.0, 3.0], [1.0, 6.0, 3.0], [1.0, 2.0, 
4.0]]], [[[5.0, 7.0, 4.0], [4.0, 2.0, 1.0], [7.0, 4.0, 1.0]], [[7.0, 1.0, 3.0], [5.0, 4.0, 3.0], [7.0, 4.0, 6.0]], [[6.0, 3.0, 
1.0], [1.0, 3.0, 6.0], [7.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 5.0], [4.0, 5.0, 1.0], [7.0, 7.0, 4.0]], 
[[6.0, 3.0, 7.0], [6.0, 8.0, 4.0], [7.0, 4.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 6.0, 7.0], [8.0, 4.0, 3.0]]], [[[5.0, 4.0, 6.0], 
[6.0, 2.0, 1.0], [8.0, 3.0, 4.0]], [[1.0, 6.0, 6.0], [5.0, 8.0, 3.0], [3.0, 2.0, 8.0]], [[2.0, 1.0, 7.0], [2.0, 5.0, 4.0], 
[1.0, 6.0, 7.0]]], [[[5.0, 1.0, 4.0], [8.0, 8.0, 1.0], [4.0, 5.0, 1.0]], [[3.0, 6.0, 5.0], [3.0, 1.0, 7.0], [1.0, 3.0, 2.0]], 
[[5.0, 7.0, 8.0], [7.0, 6.0, 1.0], [1.0, 8.0, 4.0]]], [[[4.0, 6.0, 4.0], [6.0, 4.0, 8.0], [7.0, 1.0, 2.0]], [[4.0, 5.0, 7.0], 
[6.0, 7.0, 5.0], [1.0, 1.0, 8.0]], [[7.0, 4.0, 8.0], [5.0, 6.0, 4.0], [7.0, 4.0, 4.0]]]])
      val=Data(numpy.array([[[7.0, 1.0, 8.0], [5.0, 2.0, 6.0]], [[8.0, 6.0, 3.0], [2.0, 5.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 7.0, 2.0], [7.0, 1.0, 3.0]], [[7.0, 2.0, 4.0], [3.0, 7.0, 6.0]]]))
      arg[1:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 4.0], [2.0, 4.0, 7.0], [8.0, 1.0, 6.0]], [[7.0, 5.0, 1.0], [7.0, 7.0, 1.0], [8.0, 
6.0, 3.0]], [[4.0, 4.0, 1.0], [7.0, 6.0, 4.0], [2.0, 7.0, 4.0]]], [[[7.0, 1.0, 7.0], [7.0, 1.0, 8.0], [1.0, 2.0, 1.0]], [[1.0, 
2.0, 4.0], [5.0, 2.0, 6.0], [1.0, 7.0, 1.0]], [[1.0, 4.0, 2.0], [5.0, 5.0, 1.0], [8.0, 6.0, 7.0]]], [[[5.0, 1.0, 1.0], [8.0, 
6.0, 3.0], [6.0, 2.0, 7.0]], [[1.0, 4.0, 6.0], [2.0, 5.0, 3.0], [3.0, 4.0, 1.0]], [[1.0, 4.0, 3.0], [1.0, 6.0, 3.0], [1.0, 2.0, 
4.0]]], [[[5.0, 7.0, 4.0], [4.0, 2.0, 1.0], [7.0, 4.0, 1.0]], [[7.0, 1.0, 3.0], [5.0, 4.0, 3.0], [7.0, 4.0, 6.0]], [[6.0, 3.0, 
1.0], [1.0, 3.0, 6.0], [7.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 5.0], [4.0, 5.0, 1.0], [7.0, 7.0, 4.0]], 
[[6.0, 3.0, 7.0], [6.0, 8.0, 4.0], [7.0, 4.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 6.0, 7.0], [8.0, 4.0, 3.0]]], [[[5.0, 4.0, 6.0], 
[8.0, 7.0, 2.0], [8.0, 3.0, 4.0]], [[1.0, 6.0, 6.0], [7.0, 1.0, 3.0], [3.0, 2.0, 8.0]], [[2.0, 1.0, 7.0], [2.0, 5.0, 4.0], 
[1.0, 6.0, 7.0]]], [[[5.0, 1.0, 4.0], [7.0, 2.0, 4.0], [4.0, 5.0, 1.0]], [[3.0, 6.0, 5.0], [3.0, 7.0, 6.0], [1.0, 3.0, 2.0]], 
[[5.0, 7.0, 8.0], [7.0, 6.0, 1.0], [1.0, 8.0, 4.0]]], [[[4.0, 6.0, 4.0], [6.0, 4.0, 8.0], [7.0, 1.0, 2.0]], [[4.0, 5.0, 7.0], 
[6.0, 7.0, 5.0], [1.0, 1.0, 8.0]], [[7.0, 4.0, 8.0], [5.0, 6.0, 4.0], [7.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CPCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 2.0], [4.0, 7.0, 1.0], [6.0, 7.0, 3.0]], [[6.0, 5.0, 5.0], [6.0, 3.0, 6.0], [4.0, 
5.0, 8.0]], [[1.0, 5.0, 6.0], [8.0, 6.0, 5.0], [4.0, 8.0, 6.0]]], [[[5.0, 5.0, 7.0], [1.0, 5.0, 1.0], [6.0, 2.0, 7.0]], [[3.0, 
5.0, 4.0], [5.0, 5.0, 2.0], [7.0, 7.0, 7.0]], [[7.0, 4.0, 8.0], [2.0, 6.0, 8.0], [8.0, 4.0, 7.0]]], [[[8.0, 3.0, 3.0], [3.0, 
1.0, 5.0], [1.0, 1.0, 6.0]], [[8.0, 6.0, 6.0], [2.0, 5.0, 1.0], [6.0, 7.0, 8.0]], [[8.0, 2.0, 7.0], [4.0, 2.0, 5.0], [1.0, 3.0, 
3.0]]], [[[2.0, 1.0, 1.0], [2.0, 4.0, 4.0], [6.0, 8.0, 4.0]], [[6.0, 4.0, 1.0], [6.0, 8.0, 5.0], [8.0, 6.0, 6.0]], [[6.0, 4.0, 
8.0], [6.0, 6.0, 7.0], [4.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 1.0], [7.0, 6.0, 5.0], [1.0, 6.0, 3.0]], 
[[6.0, 7.0, 6.0], [8.0, 3.0, 8.0], [6.0, 4.0, 3.0]], [[6.0, 6.0, 5.0], [5.0, 7.0, 8.0], [1.0, 7.0, 1.0]]], [[[3.0, 2.0, 8.0], 
[2.0, 3.0, 2.0], [7.0, 3.0, 8.0]], [[3.0, 1.0, 4.0], [8.0, 7.0, 7.0], [2.0, 3.0, 8.0]], [[4.0, 4.0, 7.0], [7.0, 2.0, 7.0], 
[4.0, 4.0, 6.0]]], [[[8.0, 4.0, 3.0], [1.0, 6.0, 8.0], [8.0, 7.0, 3.0]], [[6.0, 6.0, 8.0], [3.0, 3.0, 5.0], [4.0, 4.0, 1.0]], 
[[6.0, 6.0, 6.0], [6.0, 2.0, 5.0], [2.0, 5.0, 5.0]]], [[[1.0, 7.0, 5.0], [1.0, 6.0, 1.0], [2.0, 1.0, 1.0]], [[3.0, 7.0, 7.0], 
[3.0, 2.0, 7.0], [8.0, 2.0, 7.0]], [[3.0, 1.0, 7.0], [3.0, 3.0, 7.0], [7.0, 4.0, 7.0]]]])
      val=Data(numpy.array([[[4.0, 6.0, 5.0], [6.0, 7.0, 5.0], [2.0, 3.0, 5.0]], [[7.0, 1.0, 5.0], [5.0, 5.0, 4.0], [5.0, 7.0, 
3.0]], [[8.0, 8.0, 2.0], [1.0, 4.0, 5.0], [1.0, 2.0, 2.0]], [[6.0, 2.0, 1.0], [8.0, 6.0, 6.0], [5.0, 8.0, 
8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 2.0, 8.0], [8.0, 5.0, 1.0], [6.0, 8.0, 5.0]], [[4.0, 1.0, 1.0], [3.0, 6.0, 3.0], 
[7.0, 1.0, 7.0]], [[6.0, 1.0, 1.0], [2.0, 2.0, 8.0], [2.0, 6.0, 8.0]], [[1.0, 6.0, 3.0], [3.0, 2.0, 8.0], [5.0, 8.0, 2.0]]]))
      arg[:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 2.0], [4.0, 7.0, 1.0], [6.0, 7.0, 3.0]], [[4.0, 6.0, 5.0], [6.0, 7.0, 5.0], [2.0, 
3.0, 5.0]], [[1.0, 5.0, 6.0], [8.0, 6.0, 5.0], [4.0, 8.0, 6.0]]], [[[5.0, 5.0, 7.0], [1.0, 5.0, 1.0], [6.0, 2.0, 7.0]], [[7.0, 
1.0, 5.0], [5.0, 5.0, 4.0], [5.0, 7.0, 3.0]], [[7.0, 4.0, 8.0], [2.0, 6.0, 8.0], [8.0, 4.0, 7.0]]], [[[8.0, 3.0, 3.0], [3.0, 
1.0, 5.0], [1.0, 1.0, 6.0]], [[8.0, 8.0, 2.0], [1.0, 4.0, 5.0], [1.0, 2.0, 2.0]], [[8.0, 2.0, 7.0], [4.0, 2.0, 5.0], [1.0, 3.0, 
3.0]]], [[[2.0, 1.0, 1.0], [2.0, 4.0, 4.0], [6.0, 8.0, 4.0]], [[6.0, 2.0, 1.0], [8.0, 6.0, 6.0], [5.0, 8.0, 8.0]], [[6.0, 4.0, 
8.0], [6.0, 6.0, 7.0], [4.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 1.0], [7.0, 6.0, 5.0], [1.0, 6.0, 3.0]], 
[[3.0, 2.0, 8.0], [8.0, 5.0, 1.0], [6.0, 8.0, 5.0]], [[6.0, 6.0, 5.0], [5.0, 7.0, 8.0], [1.0, 7.0, 1.0]]], [[[3.0, 2.0, 8.0], 
[2.0, 3.0, 2.0], [7.0, 3.0, 8.0]], [[4.0, 1.0, 1.0], [3.0, 6.0, 3.0], [7.0, 1.0, 7.0]], [[4.0, 4.0, 7.0], [7.0, 2.0, 7.0], 
[4.0, 4.0, 6.0]]], [[[8.0, 4.0, 3.0], [1.0, 6.0, 8.0], [8.0, 7.0, 3.0]], [[6.0, 1.0, 1.0], [2.0, 2.0, 8.0], [2.0, 6.0, 8.0]], 
[[6.0, 6.0, 6.0], [6.0, 2.0, 5.0], [2.0, 5.0, 5.0]]], [[[1.0, 7.0, 5.0], [1.0, 6.0, 1.0], [2.0, 1.0, 1.0]], [[1.0, 6.0, 3.0], 
[3.0, 2.0, 8.0], [5.0, 8.0, 2.0]], [[3.0, 1.0, 7.0], [3.0, 3.0, 7.0], [7.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCCR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 2.0], [5.0, 6.0, 2.0], [2.0, 7.0, 8.0]], [[4.0, 2.0, 8.0], [8.0, 5.0, 5.0], [8.0, 
3.0, 7.0]], [[5.0, 6.0, 8.0], [8.0, 5.0, 7.0], [1.0, 1.0, 2.0]]], [[[8.0, 1.0, 7.0], [5.0, 8.0, 5.0], [4.0, 6.0, 1.0]], [[3.0, 
2.0, 1.0], [1.0, 7.0, 5.0], [5.0, 3.0, 4.0]], [[4.0, 3.0, 8.0], [4.0, 1.0, 7.0], [6.0, 1.0, 5.0]]], [[[1.0, 7.0, 2.0], [7.0, 
7.0, 7.0], [3.0, 4.0, 1.0]], [[4.0, 3.0, 5.0], [3.0, 7.0, 2.0], [6.0, 6.0, 5.0]], [[3.0, 7.0, 4.0], [5.0, 4.0, 3.0], [5.0, 4.0, 
6.0]]], [[[7.0, 7.0, 2.0], [7.0, 1.0, 2.0], [8.0, 6.0, 7.0]], [[8.0, 4.0, 5.0], [7.0, 4.0, 8.0], [1.0, 8.0, 5.0]], [[1.0, 7.0, 
7.0], [2.0, 5.0, 4.0], [4.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 2.0], [7.0, 6.0, 8.0], [6.0, 5.0, 6.0]], 
[[2.0, 6.0, 7.0], [1.0, 8.0, 1.0], [2.0, 8.0, 4.0]], [[8.0, 7.0, 2.0], [1.0, 2.0, 5.0], [5.0, 4.0, 5.0]]], [[[4.0, 2.0, 2.0], 
[7.0, 7.0, 7.0], [3.0, 4.0, 4.0]], [[1.0, 8.0, 2.0], [1.0, 6.0, 1.0], [2.0, 1.0, 8.0]], [[6.0, 3.0, 6.0], [7.0, 3.0, 8.0], 
[8.0, 7.0, 8.0]]], [[[2.0, 4.0, 7.0], [6.0, 1.0, 6.0], [7.0, 6.0, 4.0]], [[8.0, 1.0, 5.0], [5.0, 5.0, 4.0], [4.0, 7.0, 7.0]], 
[[4.0, 1.0, 2.0], [8.0, 1.0, 1.0], [1.0, 1.0, 7.0]]], [[[1.0, 8.0, 6.0], [4.0, 5.0, 1.0], [3.0, 2.0, 5.0]], [[2.0, 7.0, 2.0], 
[1.0, 4.0, 8.0], [4.0, 5.0, 7.0]], [[6.0, 6.0, 6.0], [4.0, 8.0, 3.0], [2.0, 3.0, 3.0]]]])
      val=Data(numpy.array([[[[1.0], [4.0], [5.0]], [[8.0], [1.0], [4.0]], [[5.0], [7.0], [2.0]]], [[[2.0], [6.0], [4.0]], 
[[4.0], [6.0], [5.0]], [[2.0], [1.0], [6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0], [1.0], [3.0]], [[4.0], [6.0], [7.0]], [[7.0], [5.0], [8.0]]], [[[4.0], [5.0], 
[1.0]], [[8.0], [4.0], [4.0]], [[6.0], [8.0], [3.0]]]]))
      arg[2:,:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 2.0], [5.0, 6.0, 2.0], [2.0, 7.0, 8.0]], [[4.0, 2.0, 8.0], [8.0, 5.0, 5.0], [8.0, 
3.0, 7.0]], [[5.0, 6.0, 8.0], [8.0, 5.0, 7.0], [1.0, 1.0, 2.0]]], [[[8.0, 1.0, 7.0], [5.0, 8.0, 5.0], [4.0, 6.0, 1.0]], [[3.0, 
2.0, 1.0], [1.0, 7.0, 5.0], [5.0, 3.0, 4.0]], [[4.0, 3.0, 8.0], [4.0, 1.0, 7.0], [6.0, 1.0, 5.0]]], [[[1.0, 1.0, 2.0], [7.0, 
4.0, 7.0], [3.0, 5.0, 1.0]], [[4.0, 8.0, 5.0], [3.0, 1.0, 2.0], [6.0, 4.0, 5.0]], [[3.0, 5.0, 4.0], [5.0, 7.0, 3.0], [5.0, 2.0, 
6.0]]], [[[7.0, 2.0, 2.0], [7.0, 6.0, 2.0], [8.0, 4.0, 7.0]], [[8.0, 4.0, 5.0], [7.0, 6.0, 8.0], [1.0, 5.0, 5.0]], [[1.0, 2.0, 
7.0], [2.0, 1.0, 4.0], [4.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 2.0], [7.0, 6.0, 8.0], [6.0, 5.0, 6.0]], 
[[2.0, 6.0, 7.0], [1.0, 8.0, 1.0], [2.0, 8.0, 4.0]], [[8.0, 7.0, 2.0], [1.0, 2.0, 5.0], [5.0, 4.0, 5.0]]], [[[4.0, 2.0, 2.0], 
[7.0, 7.0, 7.0], [3.0, 4.0, 4.0]], [[1.0, 8.0, 2.0], [1.0, 6.0, 1.0], [2.0, 1.0, 8.0]], [[6.0, 3.0, 6.0], [7.0, 3.0, 8.0], 
[8.0, 7.0, 8.0]]], [[[2.0, 5.0, 7.0], [6.0, 1.0, 6.0], [7.0, 3.0, 4.0]], [[8.0, 4.0, 5.0], [5.0, 6.0, 4.0], [4.0, 7.0, 7.0]], 
[[4.0, 7.0, 2.0], [8.0, 5.0, 1.0], [1.0, 8.0, 7.0]]], [[[1.0, 4.0, 6.0], [4.0, 5.0, 1.0], [3.0, 1.0, 5.0]], [[2.0, 8.0, 2.0], 
[1.0, 4.0, 8.0], [4.0, 4.0, 7.0]], [[6.0, 6.0, 6.0], [4.0, 8.0, 3.0], [2.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 3.0], [8.0, 6.0, 4.0], [2.0, 5.0, 5.0]], [[5.0, 8.0, 7.0], [6.0, 2.0, 2.0], [7.0, 
2.0, 8.0]], [[1.0, 8.0, 7.0], [6.0, 3.0, 4.0], [3.0, 1.0, 3.0]]], [[[4.0, 8.0, 7.0], [6.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 
4.0, 1.0], [3.0, 7.0, 4.0], [5.0, 3.0, 2.0]], [[7.0, 1.0, 1.0], [4.0, 7.0, 1.0], [1.0, 2.0, 4.0]]], [[[6.0, 6.0, 4.0], [2.0, 
2.0, 6.0], [2.0, 6.0, 6.0]], [[8.0, 6.0, 8.0], [2.0, 1.0, 1.0], [2.0, 6.0, 1.0]], [[5.0, 2.0, 6.0], [6.0, 6.0, 7.0], [6.0, 3.0, 
3.0]]], [[[8.0, 4.0, 6.0], [3.0, 4.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 1.0, 7.0], [1.0, 5.0, 8.0], [8.0, 5.0, 2.0]], [[6.0, 8.0, 
4.0], [8.0, 7.0, 3.0], [6.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 2.0], [4.0, 7.0, 6.0], [7.0, 7.0, 2.0]], 
[[2.0, 7.0, 4.0], [5.0, 5.0, 3.0], [1.0, 5.0, 8.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 6.0], [3.0, 6.0, 8.0]]], [[[4.0, 6.0, 4.0], 
[8.0, 4.0, 8.0], [4.0, 8.0, 3.0]], [[4.0, 3.0, 1.0], [8.0, 1.0, 1.0], [1.0, 7.0, 6.0]], [[7.0, 4.0, 7.0], [7.0, 5.0, 3.0], 
[8.0, 5.0, 6.0]]], [[[4.0, 7.0, 1.0], [3.0, 5.0, 8.0], [8.0, 3.0, 4.0]], [[3.0, 2.0, 4.0], [2.0, 3.0, 4.0], [8.0, 7.0, 1.0]], 
[[7.0, 3.0, 2.0], [3.0, 2.0, 3.0], [3.0, 6.0, 5.0]]], [[[4.0, 4.0, 2.0], [3.0, 8.0, 1.0], [7.0, 7.0, 8.0]], [[6.0, 3.0, 6.0], 
[3.0, 7.0, 3.0], [4.0, 6.0, 5.0]], [[3.0, 3.0, 3.0], [3.0, 3.0, 3.0], [7.0, 8.0, 5.0]]]])
      val=Data(numpy.array([[[[4.0, 1.0, 1.0]]], [[[4.0, 1.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 8.0, 2.0]]], [[[8.0, 8.0, 6.0]]]]))
      arg[1:3,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 3.0], [8.0, 6.0, 4.0], [2.0, 5.0, 5.0]], [[5.0, 8.0, 7.0], [6.0, 2.0, 2.0], [7.0, 
2.0, 8.0]], [[1.0, 8.0, 7.0], [6.0, 3.0, 4.0], [3.0, 1.0, 3.0]]], [[[4.0, 8.0, 7.0], [6.0, 4.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 
4.0, 1.0], [4.0, 1.0, 1.0], [5.0, 3.0, 2.0]], [[7.0, 1.0, 1.0], [4.0, 7.0, 1.0], [1.0, 2.0, 4.0]]], [[[6.0, 6.0, 4.0], [2.0, 
2.0, 6.0], [2.0, 6.0, 6.0]], [[8.0, 6.0, 8.0], [4.0, 1.0, 3.0], [2.0, 6.0, 1.0]], [[5.0, 2.0, 6.0], [6.0, 6.0, 7.0], [6.0, 3.0, 
3.0]]], [[[8.0, 4.0, 6.0], [3.0, 4.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 1.0, 7.0], [1.0, 5.0, 8.0], [8.0, 5.0, 2.0]], [[6.0, 8.0, 
4.0], [8.0, 7.0, 3.0], [6.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 2.0], [4.0, 7.0, 6.0], [7.0, 7.0, 2.0]], 
[[2.0, 7.0, 4.0], [5.0, 5.0, 3.0], [1.0, 5.0, 8.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 6.0], [3.0, 6.0, 8.0]]], [[[4.0, 6.0, 4.0], 
[8.0, 4.0, 8.0], [4.0, 8.0, 3.0]], [[4.0, 3.0, 1.0], [6.0, 8.0, 2.0], [1.0, 7.0, 6.0]], [[7.0, 4.0, 7.0], [7.0, 5.0, 3.0], 
[8.0, 5.0, 6.0]]], [[[4.0, 7.0, 1.0], [3.0, 5.0, 8.0], [8.0, 3.0, 4.0]], [[3.0, 2.0, 4.0], [8.0, 8.0, 6.0], [8.0, 7.0, 1.0]], 
[[7.0, 3.0, 2.0], [3.0, 2.0, 3.0], [3.0, 6.0, 5.0]]], [[[4.0, 4.0, 2.0], [3.0, 8.0, 1.0], [7.0, 7.0, 8.0]], [[6.0, 3.0, 6.0], 
[3.0, 7.0, 3.0], [4.0, 6.0, 5.0]], [[3.0, 3.0, 3.0], [3.0, 3.0, 3.0], [7.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CLLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 4.0], [3.0, 7.0, 6.0], [4.0, 4.0, 6.0]], [[4.0, 7.0, 1.0], [3.0, 5.0, 4.0], [6.0, 
7.0, 2.0]], [[5.0, 4.0, 2.0], [5.0, 2.0, 1.0], [8.0, 5.0, 4.0]]], [[[6.0, 8.0, 3.0], [5.0, 7.0, 5.0], [7.0, 6.0, 6.0]], [[7.0, 
7.0, 6.0], [5.0, 1.0, 7.0], [5.0, 2.0, 1.0]], [[8.0, 4.0, 7.0], [4.0, 7.0, 5.0], [7.0, 8.0, 8.0]]], [[[8.0, 1.0, 6.0], [1.0, 
3.0, 6.0], [8.0, 4.0, 1.0]], [[2.0, 7.0, 2.0], [8.0, 8.0, 6.0], [7.0, 4.0, 4.0]], [[4.0, 8.0, 6.0], [5.0, 5.0, 4.0], [8.0, 3.0, 
3.0]]], [[[3.0, 5.0, 1.0], [1.0, 3.0, 5.0], [1.0, 2.0, 3.0]], [[7.0, 5.0, 5.0], [3.0, 2.0, 2.0], [5.0, 2.0, 2.0]], [[3.0, 7.0, 
4.0], [8.0, 8.0, 6.0], [7.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 6.0], [2.0, 2.0, 8.0], [8.0, 7.0, 6.0]], 
[[2.0, 2.0, 8.0], [6.0, 6.0, 8.0], [5.0, 7.0, 7.0]], [[5.0, 1.0, 5.0], [8.0, 6.0, 7.0], [8.0, 6.0, 2.0]]], [[[1.0, 6.0, 3.0], 
[5.0, 3.0, 4.0], [5.0, 1.0, 2.0]], [[4.0, 5.0, 7.0], [1.0, 2.0, 1.0], [1.0, 7.0, 6.0]], [[5.0, 7.0, 1.0], [6.0, 2.0, 7.0], 
[8.0, 2.0, 8.0]]], [[[8.0, 6.0, 5.0], [6.0, 3.0, 4.0], [8.0, 8.0, 1.0]], [[3.0, 2.0, 2.0], [7.0, 1.0, 2.0], [7.0, 8.0, 8.0]], 
[[8.0, 4.0, 5.0], [2.0, 3.0, 7.0], [3.0, 5.0, 3.0]]], [[[8.0, 8.0, 8.0], [5.0, 6.0, 8.0], [8.0, 8.0, 3.0]], [[8.0, 2.0, 2.0], 
[5.0, 3.0, 4.0], [4.0, 4.0, 4.0]], [[1.0, 3.0, 4.0], [2.0, 3.0, 2.0], [7.0, 6.0, 7.0]]]])
      val=Data(numpy.array([[[[6.0, 7.0, 3.0]]], [[[7.0, 8.0, 8.0]]], [[[2.0, 2.0, 6.0]]], [[[5.0, 7.0, 
7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 6.0, 7.0]]], [[[4.0, 3.0, 3.0]]], [[[4.0, 4.0, 5.0]]], [[[5.0, 8.0, 6.0]]]]))
      arg[:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 4.0], [3.0, 7.0, 6.0], [4.0, 4.0, 6.0]], [[4.0, 7.0, 1.0], [3.0, 5.0, 4.0], [6.0, 
7.0, 2.0]], [[5.0, 4.0, 2.0], [5.0, 2.0, 1.0], [6.0, 7.0, 3.0]]], [[[6.0, 8.0, 3.0], [5.0, 7.0, 5.0], [7.0, 6.0, 6.0]], [[7.0, 
7.0, 6.0], [5.0, 1.0, 7.0], [5.0, 2.0, 1.0]], [[8.0, 4.0, 7.0], [4.0, 7.0, 5.0], [7.0, 8.0, 8.0]]], [[[8.0, 1.0, 6.0], [1.0, 
3.0, 6.0], [8.0, 4.0, 1.0]], [[2.0, 7.0, 2.0], [8.0, 8.0, 6.0], [7.0, 4.0, 4.0]], [[4.0, 8.0, 6.0], [5.0, 5.0, 4.0], [2.0, 2.0, 
6.0]]], [[[3.0, 5.0, 1.0], [1.0, 3.0, 5.0], [1.0, 2.0, 3.0]], [[7.0, 5.0, 5.0], [3.0, 2.0, 2.0], [5.0, 2.0, 2.0]], [[3.0, 7.0, 
4.0], [8.0, 8.0, 6.0], [5.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 6.0], [2.0, 2.0, 8.0], [8.0, 7.0, 6.0]], 
[[2.0, 2.0, 8.0], [6.0, 6.0, 8.0], [5.0, 7.0, 7.0]], [[5.0, 1.0, 5.0], [8.0, 6.0, 7.0], [6.0, 6.0, 7.0]]], [[[1.0, 6.0, 3.0], 
[5.0, 3.0, 4.0], [5.0, 1.0, 2.0]], [[4.0, 5.0, 7.0], [1.0, 2.0, 1.0], [1.0, 7.0, 6.0]], [[5.0, 7.0, 1.0], [6.0, 2.0, 7.0], 
[4.0, 3.0, 3.0]]], [[[8.0, 6.0, 5.0], [6.0, 3.0, 4.0], [8.0, 8.0, 1.0]], [[3.0, 2.0, 2.0], [7.0, 1.0, 2.0], [7.0, 8.0, 8.0]], 
[[8.0, 4.0, 5.0], [2.0, 3.0, 7.0], [4.0, 4.0, 5.0]]], [[[8.0, 8.0, 8.0], [5.0, 6.0, 8.0], [8.0, 8.0, 3.0]], [[8.0, 2.0, 2.0], 
[5.0, 3.0, 4.0], [4.0, 4.0, 4.0]], [[1.0, 3.0, 4.0], [2.0, 3.0, 2.0], [5.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RCCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 4.0], [5.0, 4.0, 4.0], [1.0, 4.0, 6.0]], [[4.0, 3.0, 8.0], [5.0, 5.0, 3.0], [1.0, 
5.0, 4.0]], [[7.0, 1.0, 4.0], [3.0, 8.0, 7.0], [2.0, 2.0, 2.0]]], [[[6.0, 8.0, 8.0], [8.0, 7.0, 5.0], [8.0, 7.0, 3.0]], [[5.0, 
2.0, 5.0], [4.0, 7.0, 2.0], [4.0, 3.0, 8.0]], [[2.0, 4.0, 1.0], [1.0, 3.0, 5.0], [2.0, 4.0, 1.0]]], [[[2.0, 5.0, 3.0], [1.0, 
6.0, 6.0], [4.0, 7.0, 2.0]], [[5.0, 1.0, 3.0], [4.0, 7.0, 2.0], [7.0, 5.0, 5.0]], [[1.0, 5.0, 7.0], [8.0, 2.0, 6.0], [4.0, 3.0, 
6.0]]], [[[7.0, 8.0, 5.0], [2.0, 5.0, 6.0], [2.0, 5.0, 4.0]], [[2.0, 2.0, 6.0], [3.0, 7.0, 7.0], [5.0, 8.0, 4.0]], [[7.0, 1.0, 
1.0], [1.0, 5.0, 6.0], [2.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 7.0], [6.0, 5.0, 8.0], [4.0, 2.0, 6.0]], 
[[6.0, 1.0, 6.0], [8.0, 8.0, 7.0], [2.0, 1.0, 1.0]], [[7.0, 3.0, 7.0], [2.0, 2.0, 3.0], [2.0, 1.0, 5.0]]], [[[7.0, 7.0, 6.0], 
[2.0, 4.0, 2.0], [6.0, 6.0, 1.0]], [[6.0, 7.0, 1.0], [2.0, 6.0, 6.0], [5.0, 6.0, 3.0]], [[1.0, 8.0, 5.0], [7.0, 7.0, 3.0], 
[1.0, 3.0, 7.0]]], [[[2.0, 6.0, 6.0], [5.0, 3.0, 2.0], [4.0, 1.0, 1.0]], [[1.0, 2.0, 5.0], [3.0, 7.0, 6.0], [1.0, 8.0, 6.0]], 
[[8.0, 2.0, 5.0], [7.0, 1.0, 3.0], [5.0, 3.0, 4.0]]], [[[3.0, 5.0, 7.0], [5.0, 2.0, 6.0], [2.0, 3.0, 4.0]], [[3.0, 4.0, 1.0], 
[1.0, 7.0, 1.0], [7.0, 4.0, 4.0]], [[5.0, 6.0, 5.0], [5.0, 3.0, 5.0], [8.0, 6.0, 4.0]]]])
      val=Data(numpy.array([[[[1.0, 4.0, 4.0], [4.0, 7.0, 4.0], [3.0, 8.0, 8.0]], [[1.0, 1.0, 1.0], [3.0, 4.0, 4.0], [1.0, 7.0, 
4.0]], [[6.0, 2.0, 1.0], [2.0, 7.0, 6.0], [1.0, 4.0, 8.0]]], [[[2.0, 1.0, 5.0], [6.0, 6.0, 3.0], [4.0, 7.0, 2.0]], [[4.0, 2.0, 
8.0], [5.0, 4.0, 8.0], [8.0, 1.0, 1.0]], [[8.0, 7.0, 7.0], [6.0, 5.0, 4.0], [5.0, 6.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 3.0], [8.0, 3.0, 6.0], [6.0, 4.0, 6.0]], [[5.0, 8.0, 1.0], [3.0, 2.0, 
4.0], [8.0, 7.0, 4.0]], [[1.0, 3.0, 6.0], [2.0, 6.0, 5.0], [3.0, 1.0, 6.0]]], [[[1.0, 5.0, 4.0], [5.0, 4.0, 4.0], [1.0, 5.0, 
3.0]], [[3.0, 4.0, 7.0], [5.0, 1.0, 2.0], [1.0, 6.0, 8.0]], [[6.0, 3.0, 8.0], [1.0, 8.0, 5.0], [2.0, 1.0, 2.0]]]]))
      arg[1:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 4.0], [5.0, 4.0, 4.0], [1.0, 4.0, 6.0]], [[4.0, 3.0, 8.0], [5.0, 5.0, 3.0], [1.0, 
5.0, 4.0]], [[7.0, 1.0, 4.0], [3.0, 8.0, 7.0], [2.0, 2.0, 2.0]]], [[[1.0, 4.0, 4.0], [4.0, 7.0, 4.0], [3.0, 8.0, 8.0]], [[1.0, 
1.0, 1.0], [3.0, 4.0, 4.0], [1.0, 7.0, 4.0]], [[6.0, 2.0, 1.0], [2.0, 7.0, 6.0], [1.0, 4.0, 8.0]]], [[[2.0, 1.0, 5.0], [6.0, 
6.0, 3.0], [4.0, 7.0, 2.0]], [[4.0, 2.0, 8.0], [5.0, 4.0, 8.0], [8.0, 1.0, 1.0]], [[8.0, 7.0, 7.0], [6.0, 5.0, 4.0], [5.0, 6.0, 
7.0]]], [[[7.0, 8.0, 5.0], [2.0, 5.0, 6.0], [2.0, 5.0, 4.0]], [[2.0, 2.0, 6.0], [3.0, 7.0, 7.0], [5.0, 8.0, 4.0]], [[7.0, 1.0, 
1.0], [1.0, 5.0, 6.0], [2.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 7.0], [6.0, 5.0, 8.0], [4.0, 2.0, 6.0]], 
[[6.0, 1.0, 6.0], [8.0, 8.0, 7.0], [2.0, 1.0, 1.0]], [[7.0, 3.0, 7.0], [2.0, 2.0, 3.0], [2.0, 1.0, 5.0]]], [[[5.0, 6.0, 3.0], 
[8.0, 3.0, 6.0], [6.0, 4.0, 6.0]], [[5.0, 8.0, 1.0], [3.0, 2.0, 4.0], [8.0, 7.0, 4.0]], [[1.0, 3.0, 6.0], [2.0, 6.0, 5.0], 
[3.0, 1.0, 6.0]]], [[[1.0, 5.0, 4.0], [5.0, 4.0, 4.0], [1.0, 5.0, 3.0]], [[3.0, 4.0, 7.0], [5.0, 1.0, 2.0], [1.0, 6.0, 8.0]], 
[[6.0, 3.0, 8.0], [1.0, 8.0, 5.0], [2.0, 1.0, 2.0]]], [[[3.0, 5.0, 7.0], [5.0, 2.0, 6.0], [2.0, 3.0, 4.0]], [[3.0, 4.0, 1.0], 
[1.0, 7.0, 1.0], [7.0, 4.0, 4.0]], [[5.0, 6.0, 5.0], [5.0, 3.0, 5.0], [8.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PUUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 4.0], [5.0, 4.0, 2.0], [5.0, 3.0, 5.0]], [[6.0, 4.0, 1.0], [6.0, 7.0, 8.0], [4.0, 
6.0, 4.0]], [[2.0, 6.0, 7.0], [3.0, 8.0, 2.0], [3.0, 5.0, 6.0]]], [[[5.0, 3.0, 4.0], [6.0, 2.0, 3.0], [6.0, 7.0, 5.0]], [[2.0, 
5.0, 1.0], [2.0, 7.0, 5.0], [2.0, 6.0, 6.0]], [[4.0, 8.0, 8.0], [4.0, 4.0, 6.0], [7.0, 5.0, 4.0]]], [[[1.0, 8.0, 2.0], [7.0, 
5.0, 2.0], [1.0, 6.0, 2.0]], [[2.0, 8.0, 6.0], [1.0, 3.0, 3.0], [6.0, 4.0, 1.0]], [[8.0, 5.0, 3.0], [6.0, 4.0, 5.0], [7.0, 8.0, 
5.0]]], [[[5.0, 8.0, 3.0], [1.0, 6.0, 1.0], [3.0, 6.0, 5.0]], [[6.0, 2.0, 5.0], [1.0, 3.0, 1.0], [3.0, 5.0, 3.0]], [[5.0, 7.0, 
5.0], [4.0, 5.0, 2.0], [7.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 7.0], [3.0, 8.0, 5.0], [7.0, 4.0, 1.0]], 
[[7.0, 4.0, 1.0], [6.0, 8.0, 1.0], [8.0, 4.0, 8.0]], [[6.0, 8.0, 1.0], [8.0, 2.0, 3.0], [6.0, 4.0, 8.0]]], [[[4.0, 3.0, 5.0], 
[2.0, 3.0, 6.0], [7.0, 3.0, 3.0]], [[2.0, 7.0, 7.0], [6.0, 8.0, 1.0], [6.0, 8.0, 2.0]], [[4.0, 2.0, 5.0], [8.0, 1.0, 5.0], 
[8.0, 5.0, 2.0]]], [[[7.0, 3.0, 5.0], [5.0, 3.0, 4.0], [7.0, 7.0, 7.0]], [[4.0, 3.0, 2.0], [5.0, 5.0, 6.0], [6.0, 6.0, 1.0]], 
[[3.0, 4.0, 4.0], [5.0, 7.0, 1.0], [8.0, 1.0, 4.0]]], [[[5.0, 7.0, 1.0], [2.0, 8.0, 5.0], [2.0, 6.0, 4.0]], [[2.0, 6.0, 3.0], 
[8.0, 2.0, 5.0], [8.0, 3.0, 5.0]], [[5.0, 7.0, 5.0], [2.0, 8.0, 6.0], [2.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[[2.0, 1.0, 7.0], [8.0, 4.0, 5.0]], [[7.0, 1.0, 5.0], [1.0, 8.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 1.0, 5.0], [6.0, 1.0, 2.0]], [[3.0, 2.0, 3.0], [6.0, 4.0, 8.0]]]))
      arg[2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 4.0], [5.0, 4.0, 2.0], [5.0, 3.0, 5.0]], [[6.0, 4.0, 1.0], [6.0, 7.0, 8.0], [4.0, 
6.0, 4.0]], [[2.0, 6.0, 7.0], [3.0, 8.0, 2.0], [3.0, 5.0, 6.0]]], [[[5.0, 3.0, 4.0], [6.0, 2.0, 3.0], [6.0, 7.0, 5.0]], [[2.0, 
5.0, 1.0], [2.0, 7.0, 5.0], [2.0, 6.0, 6.0]], [[4.0, 8.0, 8.0], [4.0, 4.0, 6.0], [7.0, 5.0, 4.0]]], [[[2.0, 1.0, 7.0], [8.0, 
4.0, 5.0], [1.0, 6.0, 2.0]], [[7.0, 1.0, 5.0], [1.0, 8.0, 3.0], [6.0, 4.0, 1.0]], [[8.0, 5.0, 3.0], [6.0, 4.0, 5.0], [7.0, 8.0, 
5.0]]], [[[5.0, 8.0, 3.0], [1.0, 6.0, 1.0], [3.0, 6.0, 5.0]], [[6.0, 2.0, 5.0], [1.0, 3.0, 1.0], [3.0, 5.0, 3.0]], [[5.0, 7.0, 
5.0], [4.0, 5.0, 2.0], [7.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 7.0], [3.0, 8.0, 5.0], [7.0, 4.0, 1.0]], 
[[7.0, 4.0, 1.0], [6.0, 8.0, 1.0], [8.0, 4.0, 8.0]], [[6.0, 8.0, 1.0], [8.0, 2.0, 3.0], [6.0, 4.0, 8.0]]], [[[4.0, 3.0, 5.0], 
[2.0, 3.0, 6.0], [7.0, 3.0, 3.0]], [[2.0, 7.0, 7.0], [6.0, 8.0, 1.0], [6.0, 8.0, 2.0]], [[4.0, 2.0, 5.0], [8.0, 1.0, 5.0], 
[8.0, 5.0, 2.0]]], [[[7.0, 1.0, 5.0], [6.0, 1.0, 2.0], [7.0, 7.0, 7.0]], [[3.0, 2.0, 3.0], [6.0, 4.0, 8.0], [6.0, 6.0, 1.0]], 
[[3.0, 4.0, 4.0], [5.0, 7.0, 1.0], [8.0, 1.0, 4.0]]], [[[5.0, 7.0, 1.0], [2.0, 8.0, 5.0], [2.0, 6.0, 4.0]], [[2.0, 6.0, 3.0], 
[8.0, 2.0, 5.0], [8.0, 3.0, 5.0]], [[5.0, 7.0, 5.0], [2.0, 8.0, 6.0], [2.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PULN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 2.0], [6.0, 3.0, 8.0], [1.0, 7.0, 8.0]], [[6.0, 3.0, 7.0], [8.0, 5.0, 7.0], [7.0, 
6.0, 4.0]], [[5.0, 7.0, 7.0], [6.0, 7.0, 1.0], [6.0, 5.0, 3.0]]], [[[2.0, 6.0, 4.0], [8.0, 8.0, 1.0], [3.0, 2.0, 3.0]], [[5.0, 
4.0, 4.0], [7.0, 2.0, 6.0], [7.0, 7.0, 6.0]], [[1.0, 2.0, 6.0], [5.0, 7.0, 1.0], [5.0, 1.0, 4.0]]], [[[8.0, 7.0, 2.0], [7.0, 
8.0, 6.0], [1.0, 3.0, 2.0]], [[7.0, 7.0, 8.0], [1.0, 8.0, 7.0], [3.0, 5.0, 4.0]], [[8.0, 6.0, 6.0], [2.0, 3.0, 4.0], [8.0, 3.0, 
6.0]]], [[[4.0, 2.0, 3.0], [2.0, 8.0, 5.0], [5.0, 2.0, 7.0]], [[8.0, 2.0, 1.0], [6.0, 4.0, 7.0], [4.0, 2.0, 5.0]], [[3.0, 6.0, 
5.0], [7.0, 1.0, 4.0], [2.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 4.0], [2.0, 2.0, 8.0], [4.0, 1.0, 5.0]], 
[[4.0, 1.0, 4.0], [8.0, 7.0, 8.0], [4.0, 8.0, 2.0]], [[5.0, 8.0, 4.0], [8.0, 5.0, 7.0], [1.0, 2.0, 5.0]]], [[[6.0, 8.0, 8.0], 
[6.0, 2.0, 1.0], [5.0, 2.0, 8.0]], [[3.0, 4.0, 5.0], [1.0, 8.0, 4.0], [5.0, 7.0, 7.0]], [[5.0, 2.0, 3.0], [2.0, 3.0, 4.0], 
[3.0, 7.0, 5.0]]], [[[1.0, 5.0, 6.0], [5.0, 6.0, 7.0], [4.0, 4.0, 1.0]], [[2.0, 1.0, 6.0], [1.0, 8.0, 7.0], [1.0, 5.0, 7.0]], 
[[3.0, 6.0, 2.0], [2.0, 5.0, 6.0], [8.0, 6.0, 4.0]]], [[[8.0, 5.0, 6.0], [5.0, 4.0, 1.0], [6.0, 7.0, 5.0]], [[7.0, 6.0, 8.0], 
[8.0, 1.0, 4.0], [5.0, 8.0, 5.0]], [[6.0, 2.0, 2.0], [3.0, 1.0, 5.0], [6.0, 1.0, 8.0]]]])
      val=Data(numpy.array([[[2.0, 7.0, 3.0]], [[8.0, 4.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 2.0, 4.0]], [[6.0, 5.0, 5.0]]]))
      arg[2,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 2.0], [6.0, 3.0, 8.0], [1.0, 7.0, 8.0]], [[6.0, 3.0, 7.0], [8.0, 5.0, 7.0], [7.0, 
6.0, 4.0]], [[5.0, 7.0, 7.0], [6.0, 7.0, 1.0], [6.0, 5.0, 3.0]]], [[[2.0, 6.0, 4.0], [8.0, 8.0, 1.0], [3.0, 2.0, 3.0]], [[5.0, 
4.0, 4.0], [7.0, 2.0, 6.0], [7.0, 7.0, 6.0]], [[1.0, 2.0, 6.0], [5.0, 7.0, 1.0], [5.0, 1.0, 4.0]]], [[[8.0, 7.0, 2.0], [7.0, 
8.0, 6.0], [2.0, 7.0, 3.0]], [[7.0, 7.0, 8.0], [1.0, 8.0, 7.0], [8.0, 4.0, 1.0]], [[8.0, 6.0, 6.0], [2.0, 3.0, 4.0], [8.0, 3.0, 
6.0]]], [[[4.0, 2.0, 3.0], [2.0, 8.0, 5.0], [5.0, 2.0, 7.0]], [[8.0, 2.0, 1.0], [6.0, 4.0, 7.0], [4.0, 2.0, 5.0]], [[3.0, 6.0, 
5.0], [7.0, 1.0, 4.0], [2.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 4.0], [2.0, 2.0, 8.0], [4.0, 1.0, 5.0]], 
[[4.0, 1.0, 4.0], [8.0, 7.0, 8.0], [4.0, 8.0, 2.0]], [[5.0, 8.0, 4.0], [8.0, 5.0, 7.0], [1.0, 2.0, 5.0]]], [[[6.0, 8.0, 8.0], 
[6.0, 2.0, 1.0], [5.0, 2.0, 8.0]], [[3.0, 4.0, 5.0], [1.0, 8.0, 4.0], [5.0, 7.0, 7.0]], [[5.0, 2.0, 3.0], [2.0, 3.0, 4.0], 
[3.0, 7.0, 5.0]]], [[[1.0, 5.0, 6.0], [5.0, 6.0, 7.0], [4.0, 2.0, 4.0]], [[2.0, 1.0, 6.0], [1.0, 8.0, 7.0], [6.0, 5.0, 5.0]], 
[[3.0, 6.0, 2.0], [2.0, 5.0, 6.0], [8.0, 6.0, 4.0]]], [[[8.0, 5.0, 6.0], [5.0, 4.0, 1.0], [6.0, 7.0, 5.0]], [[7.0, 6.0, 8.0], 
[8.0, 1.0, 4.0], [5.0, 8.0, 5.0]], [[6.0, 2.0, 2.0], [3.0, 1.0, 5.0], [6.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPRR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 6.0], [6.0, 5.0, 4.0], [6.0, 7.0, 6.0]], [[1.0, 5.0, 6.0], [3.0, 1.0, 1.0], [6.0, 
4.0, 7.0]], [[6.0, 5.0, 6.0], [3.0, 8.0, 3.0], [6.0, 1.0, 5.0]]], [[[5.0, 2.0, 1.0], [2.0, 2.0, 2.0], [3.0, 1.0, 1.0]], [[5.0, 
4.0, 7.0], [3.0, 3.0, 8.0], [6.0, 2.0, 8.0]], [[7.0, 4.0, 4.0], [8.0, 7.0, 7.0], [7.0, 8.0, 2.0]]], [[[3.0, 6.0, 8.0], [4.0, 
1.0, 1.0], [4.0, 4.0, 2.0]], [[2.0, 1.0, 8.0], [8.0, 1.0, 8.0], [5.0, 3.0, 7.0]], [[3.0, 3.0, 8.0], [7.0, 7.0, 1.0], [6.0, 7.0, 
3.0]]], [[[6.0, 7.0, 1.0], [1.0, 3.0, 2.0], [2.0, 2.0, 7.0]], [[4.0, 4.0, 8.0], [6.0, 1.0, 1.0], [1.0, 5.0, 2.0]], [[3.0, 8.0, 
2.0], [4.0, 7.0, 1.0], [3.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 2.0], [7.0, 8.0, 8.0], [3.0, 5.0, 3.0]], 
[[8.0, 8.0, 8.0], [2.0, 2.0, 2.0], [3.0, 6.0, 6.0]], [[3.0, 5.0, 8.0], [2.0, 7.0, 6.0], [1.0, 6.0, 7.0]]], [[[5.0, 3.0, 8.0], 
[5.0, 2.0, 1.0], [4.0, 6.0, 2.0]], [[1.0, 7.0, 3.0], [8.0, 2.0, 5.0], [4.0, 7.0, 3.0]], [[5.0, 5.0, 5.0], [3.0, 1.0, 5.0], 
[2.0, 1.0, 5.0]]], [[[2.0, 5.0, 1.0], [8.0, 7.0, 8.0], [1.0, 8.0, 5.0]], [[4.0, 1.0, 6.0], [7.0, 3.0, 8.0], [8.0, 3.0, 7.0]], 
[[8.0, 4.0, 6.0], [6.0, 3.0, 8.0], [1.0, 5.0, 1.0]]], [[[6.0, 2.0, 1.0], [5.0, 7.0, 7.0], [7.0, 1.0, 5.0]], [[6.0, 2.0, 2.0], 
[2.0, 6.0, 2.0], [1.0, 6.0, 4.0]], [[3.0, 5.0, 4.0], [4.0, 2.0, 6.0], [2.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0]]))
      arg[2,1,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 6.0], [6.0, 5.0, 4.0], [6.0, 7.0, 6.0]], [[1.0, 5.0, 6.0], [3.0, 1.0, 1.0], [6.0, 
4.0, 7.0]], [[6.0, 5.0, 6.0], [3.0, 8.0, 3.0], [6.0, 1.0, 5.0]]], [[[5.0, 2.0, 1.0], [2.0, 2.0, 2.0], [3.0, 1.0, 1.0]], [[5.0, 
4.0, 7.0], [3.0, 3.0, 8.0], [6.0, 2.0, 8.0]], [[7.0, 4.0, 4.0], [8.0, 7.0, 7.0], [7.0, 8.0, 2.0]]], [[[3.0, 6.0, 8.0], [4.0, 
1.0, 1.0], [4.0, 4.0, 2.0]], [[2.0, 1.0, 8.0], [8.0, 6.0, 8.0], [5.0, 3.0, 7.0]], [[3.0, 3.0, 8.0], [7.0, 7.0, 1.0], [6.0, 7.0, 
3.0]]], [[[6.0, 7.0, 1.0], [1.0, 3.0, 2.0], [2.0, 2.0, 7.0]], [[4.0, 4.0, 8.0], [6.0, 1.0, 1.0], [1.0, 5.0, 2.0]], [[3.0, 8.0, 
2.0], [4.0, 7.0, 1.0], [3.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 2.0], [7.0, 8.0, 8.0], [3.0, 5.0, 3.0]], 
[[8.0, 8.0, 8.0], [2.0, 2.0, 2.0], [3.0, 6.0, 6.0]], [[3.0, 5.0, 8.0], [2.0, 7.0, 6.0], [1.0, 6.0, 7.0]]], [[[5.0, 3.0, 8.0], 
[5.0, 2.0, 1.0], [4.0, 6.0, 2.0]], [[1.0, 7.0, 3.0], [8.0, 2.0, 5.0], [4.0, 7.0, 3.0]], [[5.0, 5.0, 5.0], [3.0, 1.0, 5.0], 
[2.0, 1.0, 5.0]]], [[[2.0, 5.0, 1.0], [8.0, 7.0, 8.0], [1.0, 8.0, 5.0]], [[4.0, 1.0, 6.0], [7.0, 8.0, 8.0], [8.0, 3.0, 7.0]], 
[[8.0, 4.0, 6.0], [6.0, 3.0, 8.0], [1.0, 5.0, 1.0]]], [[[6.0, 2.0, 1.0], [5.0, 7.0, 7.0], [7.0, 1.0, 5.0]], [[6.0, 2.0, 2.0], 
[2.0, 6.0, 2.0], [1.0, 6.0, 4.0]], [[3.0, 5.0, 4.0], [4.0, 2.0, 6.0], [2.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRUR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 7.0], [2.0, 5.0, 7.0], [2.0, 3.0, 8.0]], [[3.0, 4.0, 8.0], [4.0, 3.0, 2.0], [4.0, 
2.0, 1.0]], [[4.0, 3.0, 6.0], [4.0, 5.0, 8.0], [6.0, 5.0, 3.0]]], [[[5.0, 2.0, 8.0], [5.0, 2.0, 2.0], [8.0, 7.0, 4.0]], [[3.0, 
8.0, 1.0], [1.0, 1.0, 3.0], [1.0, 1.0, 1.0]], [[5.0, 6.0, 4.0], [5.0, 2.0, 5.0], [6.0, 8.0, 3.0]]], [[[3.0, 6.0, 1.0], [6.0, 
6.0, 4.0], [8.0, 5.0, 5.0]], [[2.0, 3.0, 1.0], [7.0, 8.0, 5.0], [6.0, 7.0, 7.0]], [[3.0, 4.0, 1.0], [6.0, 8.0, 2.0], [4.0, 1.0, 
5.0]]], [[[6.0, 3.0, 5.0], [6.0, 8.0, 1.0], [4.0, 8.0, 3.0]], [[7.0, 5.0, 3.0], [4.0, 7.0, 7.0], [6.0, 1.0, 8.0]], [[6.0, 2.0, 
7.0], [2.0, 2.0, 5.0], [1.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 8.0], [1.0, 6.0, 4.0], [7.0, 5.0, 4.0]], 
[[1.0, 1.0, 4.0], [2.0, 8.0, 8.0], [6.0, 6.0, 1.0]], [[2.0, 3.0, 5.0], [5.0, 7.0, 4.0], [5.0, 2.0, 7.0]]], [[[2.0, 5.0, 1.0], 
[6.0, 6.0, 1.0], [7.0, 4.0, 8.0]], [[8.0, 7.0, 5.0], [7.0, 8.0, 4.0], [5.0, 7.0, 1.0]], [[8.0, 7.0, 6.0], [2.0, 2.0, 7.0], 
[1.0, 8.0, 3.0]]], [[[4.0, 7.0, 7.0], [2.0, 1.0, 3.0], [5.0, 7.0, 4.0]], [[1.0, 2.0, 3.0], [1.0, 2.0, 1.0], [5.0, 6.0, 6.0]], 
[[4.0, 3.0, 3.0], [4.0, 4.0, 4.0], [6.0, 3.0, 1.0]]], [[[7.0, 5.0, 2.0], [5.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[6.0, 5.0, 4.0], 
[6.0, 2.0, 7.0], [4.0, 1.0, 2.0]], [[4.0, 4.0, 5.0], [5.0, 5.0, 6.0], [1.0, 4.0, 3.0]]]])
      val=Data(numpy.array([[[[7.0], [2.0]]], [[[7.0], [6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0], [3.0]]], [[[6.0], [1.0]]]]))
      arg[1:3,1:2,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 7.0], [2.0, 5.0, 7.0], [2.0, 3.0, 8.0]], [[3.0, 4.0, 8.0], [4.0, 3.0, 2.0], [4.0, 
2.0, 1.0]], [[4.0, 3.0, 6.0], [4.0, 5.0, 8.0], [6.0, 5.0, 3.0]]], [[[5.0, 2.0, 8.0], [5.0, 2.0, 2.0], [8.0, 7.0, 4.0]], [[3.0, 
7.0, 1.0], [1.0, 2.0, 3.0], [1.0, 1.0, 1.0]], [[5.0, 6.0, 4.0], [5.0, 2.0, 5.0], [6.0, 8.0, 3.0]]], [[[3.0, 6.0, 1.0], [6.0, 
6.0, 4.0], [8.0, 5.0, 5.0]], [[2.0, 7.0, 1.0], [7.0, 6.0, 5.0], [6.0, 7.0, 7.0]], [[3.0, 4.0, 1.0], [6.0, 8.0, 2.0], [4.0, 1.0, 
5.0]]], [[[6.0, 3.0, 5.0], [6.0, 8.0, 1.0], [4.0, 8.0, 3.0]], [[7.0, 5.0, 3.0], [4.0, 7.0, 7.0], [6.0, 1.0, 8.0]], [[6.0, 2.0, 
7.0], [2.0, 2.0, 5.0], [1.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 8.0], [1.0, 6.0, 4.0], [7.0, 5.0, 4.0]], 
[[1.0, 1.0, 4.0], [2.0, 8.0, 8.0], [6.0, 6.0, 1.0]], [[2.0, 3.0, 5.0], [5.0, 7.0, 4.0], [5.0, 2.0, 7.0]]], [[[2.0, 5.0, 1.0], 
[6.0, 6.0, 1.0], [7.0, 4.0, 8.0]], [[8.0, 8.0, 5.0], [7.0, 3.0, 4.0], [5.0, 7.0, 1.0]], [[8.0, 7.0, 6.0], [2.0, 2.0, 7.0], 
[1.0, 8.0, 3.0]]], [[[4.0, 7.0, 7.0], [2.0, 1.0, 3.0], [5.0, 7.0, 4.0]], [[1.0, 6.0, 3.0], [1.0, 1.0, 1.0], [5.0, 6.0, 6.0]], 
[[4.0, 3.0, 3.0], [4.0, 4.0, 4.0], [6.0, 3.0, 1.0]]], [[[7.0, 5.0, 2.0], [5.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[6.0, 5.0, 4.0], 
[6.0, 2.0, 7.0], [4.0, 1.0, 2.0]], [[4.0, 4.0, 5.0], [5.0, 5.0, 6.0], [1.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LUPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 6.0], [2.0, 4.0, 7.0], [6.0, 5.0, 2.0]], [[8.0, 3.0, 1.0], [1.0, 5.0, 1.0], [5.0, 
1.0, 7.0]], [[3.0, 3.0, 4.0], [7.0, 1.0, 6.0], [5.0, 5.0, 4.0]]], [[[2.0, 3.0, 2.0], [5.0, 3.0, 3.0], [1.0, 6.0, 6.0]], [[4.0, 
1.0, 7.0], [8.0, 6.0, 8.0], [3.0, 8.0, 6.0]], [[3.0, 6.0, 3.0], [2.0, 3.0, 3.0], [2.0, 6.0, 4.0]]], [[[6.0, 7.0, 1.0], [5.0, 
5.0, 1.0], [7.0, 8.0, 6.0]], [[7.0, 6.0, 5.0], [8.0, 3.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 2.0, 5.0], [6.0, 2.0, 2.0], [8.0, 4.0, 
4.0]]], [[[8.0, 4.0, 6.0], [3.0, 6.0, 1.0], [8.0, 4.0, 8.0]], [[8.0, 1.0, 3.0], [3.0, 4.0, 8.0], [4.0, 8.0, 8.0]], [[1.0, 4.0, 
2.0], [8.0, 3.0, 7.0], [2.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 2.0], [4.0, 2.0, 5.0], [6.0, 1.0, 8.0]], 
[[1.0, 6.0, 2.0], [1.0, 7.0, 7.0], [4.0, 6.0, 5.0]], [[4.0, 8.0, 6.0], [2.0, 4.0, 7.0], [5.0, 6.0, 7.0]]], [[[1.0, 6.0, 1.0], 
[6.0, 1.0, 1.0], [8.0, 3.0, 4.0]], [[5.0, 8.0, 2.0], [2.0, 8.0, 8.0], [3.0, 3.0, 2.0]], [[2.0, 3.0, 7.0], [1.0, 6.0, 6.0], 
[1.0, 7.0, 6.0]]], [[[7.0, 3.0, 5.0], [8.0, 6.0, 6.0], [8.0, 1.0, 3.0]], [[3.0, 6.0, 5.0], [6.0, 2.0, 2.0], [2.0, 5.0, 5.0]], 
[[2.0, 4.0, 5.0], [3.0, 6.0, 3.0], [4.0, 5.0, 1.0]]], [[[7.0, 6.0, 7.0], [5.0, 5.0, 7.0], [2.0, 4.0, 8.0]], [[6.0, 2.0, 7.0], 
[5.0, 6.0, 4.0], [8.0, 1.0, 5.0]], [[3.0, 7.0, 6.0], [1.0, 4.0, 3.0], [1.0, 3.0, 7.0]]]])
      val=Data(numpy.array([[[3.0, 7.0, 5.0], [1.0, 5.0, 6.0]], [[2.0, 5.0, 7.0], [2.0, 3.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 7.0, 4.0], [4.0, 1.0, 2.0]], [[8.0, 8.0, 2.0], [3.0, 2.0, 3.0]]]))
      arg[2:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 6.0], [2.0, 4.0, 7.0], [6.0, 5.0, 2.0]], [[8.0, 3.0, 1.0], [1.0, 5.0, 1.0], [5.0, 
1.0, 7.0]], [[3.0, 3.0, 4.0], [7.0, 1.0, 6.0], [5.0, 5.0, 4.0]]], [[[2.0, 3.0, 2.0], [5.0, 3.0, 3.0], [1.0, 6.0, 6.0]], [[4.0, 
1.0, 7.0], [8.0, 6.0, 8.0], [3.0, 8.0, 6.0]], [[3.0, 6.0, 3.0], [2.0, 3.0, 3.0], [2.0, 6.0, 4.0]]], [[[6.0, 7.0, 1.0], [3.0, 
7.0, 5.0], [7.0, 8.0, 6.0]], [[7.0, 6.0, 5.0], [1.0, 5.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 2.0, 5.0], [6.0, 2.0, 2.0], [8.0, 4.0, 
4.0]]], [[[8.0, 4.0, 6.0], [2.0, 5.0, 7.0], [8.0, 4.0, 8.0]], [[8.0, 1.0, 3.0], [2.0, 3.0, 3.0], [4.0, 8.0, 8.0]], [[1.0, 4.0, 
2.0], [8.0, 3.0, 7.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 2.0], [4.0, 2.0, 5.0], [6.0, 1.0, 8.0]], 
[[1.0, 6.0, 2.0], [1.0, 7.0, 7.0], [4.0, 6.0, 5.0]], [[4.0, 8.0, 6.0], [2.0, 4.0, 7.0], [5.0, 6.0, 7.0]]], [[[1.0, 6.0, 1.0], 
[6.0, 1.0, 1.0], [8.0, 3.0, 4.0]], [[5.0, 8.0, 2.0], [2.0, 8.0, 8.0], [3.0, 3.0, 2.0]], [[2.0, 3.0, 7.0], [1.0, 6.0, 6.0], 
[1.0, 7.0, 6.0]]], [[[7.0, 3.0, 5.0], [2.0, 7.0, 4.0], [8.0, 1.0, 3.0]], [[3.0, 6.0, 5.0], [4.0, 1.0, 2.0], [2.0, 5.0, 5.0]], 
[[2.0, 4.0, 5.0], [3.0, 6.0, 3.0], [4.0, 5.0, 1.0]]], [[[7.0, 6.0, 7.0], [8.0, 8.0, 2.0], [2.0, 4.0, 8.0]], [[6.0, 2.0, 7.0], 
[3.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[3.0, 7.0, 6.0], [1.0, 4.0, 3.0], [1.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UCPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 3.0], [4.0, 1.0, 5.0], [5.0, 7.0, 5.0]], [[8.0, 1.0, 6.0], [4.0, 8.0, 8.0], [8.0, 
1.0, 6.0]], [[8.0, 2.0, 8.0], [1.0, 1.0, 4.0], [4.0, 7.0, 2.0]]], [[[7.0, 2.0, 6.0], [2.0, 3.0, 8.0], [6.0, 3.0, 1.0]], [[2.0, 
6.0, 2.0], [6.0, 6.0, 7.0], [1.0, 7.0, 8.0]], [[6.0, 3.0, 3.0], [8.0, 2.0, 3.0], [1.0, 5.0, 3.0]]], [[[2.0, 6.0, 5.0], [1.0, 
7.0, 6.0], [5.0, 2.0, 1.0]], [[5.0, 1.0, 8.0], [2.0, 8.0, 2.0], [2.0, 6.0, 4.0]], [[2.0, 5.0, 7.0], [7.0, 6.0, 3.0], [3.0, 7.0, 
4.0]]], [[[7.0, 6.0, 1.0], [2.0, 6.0, 8.0], [4.0, 1.0, 1.0]], [[1.0, 3.0, 4.0], [6.0, 6.0, 2.0], [1.0, 4.0, 5.0]], [[2.0, 7.0, 
7.0], [2.0, 5.0, 8.0], [2.0, 7.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 1.0], [5.0, 8.0, 5.0], [7.0, 8.0, 1.0]], 
[[6.0, 8.0, 3.0], [5.0, 3.0, 6.0], [4.0, 6.0, 8.0]], [[1.0, 1.0, 2.0], [5.0, 5.0, 4.0], [6.0, 3.0, 8.0]]], [[[3.0, 1.0, 4.0], 
[5.0, 2.0, 6.0], [6.0, 7.0, 6.0]], [[4.0, 6.0, 5.0], [2.0, 8.0, 3.0], [1.0, 2.0, 7.0]], [[4.0, 5.0, 4.0], [1.0, 7.0, 1.0], 
[5.0, 4.0, 6.0]]], [[[2.0, 6.0, 3.0], [4.0, 7.0, 6.0], [3.0, 2.0, 5.0]], [[5.0, 1.0, 6.0], [2.0, 4.0, 4.0], [3.0, 1.0, 1.0]], 
[[1.0, 5.0, 1.0], [1.0, 6.0, 7.0], [8.0, 2.0, 5.0]]], [[[5.0, 2.0, 2.0], [2.0, 8.0, 5.0], [2.0, 7.0, 4.0]], [[5.0, 6.0, 5.0], 
[4.0, 8.0, 8.0], [6.0, 4.0, 7.0]], [[4.0, 6.0, 5.0], [5.0, 7.0, 7.0], [1.0, 4.0, 8.0]]]])
      val=Data(numpy.array([[[7.0, 5.0, 4.0], [1.0, 2.0, 2.0], [3.0, 5.0, 4.0]], [[3.0, 7.0, 7.0], [8.0, 5.0, 7.0], [3.0, 5.0, 
1.0]], [[8.0, 7.0, 7.0], [1.0, 5.0, 1.0], [3.0, 4.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 3.0, 3.0], [7.0, 8.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 7.0, 4.0], [2.0, 5.0, 2.0], 
[7.0, 7.0, 2.0]], [[5.0, 1.0, 2.0], [6.0, 1.0, 2.0], [7.0, 2.0, 4.0]]]))
      arg[:3,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [7.0, 5.0, 4.0], [5.0, 7.0, 5.0]], [[8.0, 1.0, 6.0], [1.0, 2.0, 2.0], [8.0, 
1.0, 6.0]], [[8.0, 2.0, 8.0], [3.0, 5.0, 4.0], [4.0, 7.0, 2.0]]], [[[7.0, 2.0, 6.0], [3.0, 7.0, 7.0], [6.0, 3.0, 1.0]], [[2.0, 
6.0, 2.0], [8.0, 5.0, 7.0], [1.0, 7.0, 8.0]], [[6.0, 3.0, 3.0], [3.0, 5.0, 1.0], [1.0, 5.0, 3.0]]], [[[2.0, 6.0, 5.0], [8.0, 
7.0, 7.0], [5.0, 2.0, 1.0]], [[5.0, 1.0, 8.0], [1.0, 5.0, 1.0], [2.0, 6.0, 4.0]], [[2.0, 5.0, 7.0], [3.0, 4.0, 8.0], [3.0, 7.0, 
4.0]]], [[[7.0, 6.0, 1.0], [2.0, 6.0, 8.0], [4.0, 1.0, 1.0]], [[1.0, 3.0, 4.0], [6.0, 6.0, 2.0], [1.0, 4.0, 5.0]], [[2.0, 7.0, 
7.0], [2.0, 5.0, 8.0], [2.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 1.0], [4.0, 3.0, 3.0], [7.0, 8.0, 1.0]], 
[[6.0, 8.0, 3.0], [7.0, 8.0, 7.0], [4.0, 6.0, 8.0]], [[1.0, 1.0, 2.0], [2.0, 7.0, 1.0], [6.0, 3.0, 8.0]]], [[[3.0, 1.0, 4.0], 
[7.0, 7.0, 4.0], [6.0, 7.0, 6.0]], [[4.0, 6.0, 5.0], [2.0, 5.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 5.0, 4.0], [7.0, 7.0, 2.0], 
[5.0, 4.0, 6.0]]], [[[2.0, 6.0, 3.0], [5.0, 1.0, 2.0], [3.0, 2.0, 5.0]], [[5.0, 1.0, 6.0], [6.0, 1.0, 2.0], [3.0, 1.0, 1.0]], 
[[1.0, 5.0, 1.0], [7.0, 2.0, 4.0], [8.0, 2.0, 5.0]]], [[[5.0, 2.0, 2.0], [2.0, 8.0, 5.0], [2.0, 7.0, 4.0]], [[5.0, 6.0, 5.0], 
[4.0, 8.0, 8.0], [6.0, 4.0, 7.0]], [[4.0, 6.0, 5.0], [5.0, 7.0, 7.0], [1.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CUPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 5.0], [5.0, 1.0, 4.0], [8.0, 5.0, 4.0]], [[2.0, 1.0, 4.0], [3.0, 1.0, 4.0], [7.0, 
4.0, 8.0]], [[4.0, 8.0, 4.0], [1.0, 8.0, 4.0], [7.0, 8.0, 4.0]]], [[[2.0, 3.0, 6.0], [4.0, 2.0, 2.0], [2.0, 6.0, 7.0]], [[4.0, 
4.0, 6.0], [3.0, 4.0, 8.0], [8.0, 7.0, 6.0]], [[6.0, 3.0, 4.0], [1.0, 4.0, 5.0], [3.0, 7.0, 1.0]]], [[[7.0, 7.0, 8.0], [1.0, 
2.0, 2.0], [7.0, 1.0, 8.0]], [[2.0, 1.0, 1.0], [3.0, 5.0, 6.0], [8.0, 8.0, 6.0]], [[5.0, 5.0, 3.0], [5.0, 1.0, 6.0], [8.0, 1.0, 
4.0]]], [[[8.0, 1.0, 1.0], [8.0, 5.0, 1.0], [8.0, 3.0, 4.0]], [[6.0, 4.0, 1.0], [6.0, 1.0, 2.0], [3.0, 2.0, 1.0]], [[4.0, 5.0, 
5.0], [7.0, 7.0, 3.0], [5.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 1.0], [3.0, 1.0, 1.0], [5.0, 4.0, 1.0]], 
[[6.0, 5.0, 1.0], [3.0, 8.0, 4.0], [4.0, 1.0, 3.0]], [[7.0, 7.0, 1.0], [2.0, 2.0, 7.0], [8.0, 3.0, 7.0]]], [[[6.0, 7.0, 2.0], 
[8.0, 2.0, 5.0], [5.0, 3.0, 4.0]], [[6.0, 2.0, 6.0], [2.0, 8.0, 8.0], [4.0, 3.0, 2.0]], [[1.0, 2.0, 2.0], [6.0, 3.0, 2.0], 
[2.0, 1.0, 1.0]]], [[[7.0, 3.0, 4.0], [3.0, 2.0, 7.0], [5.0, 8.0, 7.0]], [[6.0, 6.0, 2.0], [6.0, 2.0, 3.0], [7.0, 4.0, 4.0]], 
[[7.0, 1.0, 8.0], [4.0, 8.0, 3.0], [3.0, 7.0, 4.0]]], [[[3.0, 1.0, 5.0], [3.0, 2.0, 5.0], [4.0, 8.0, 1.0]], [[5.0, 1.0, 3.0], 
[2.0, 5.0, 8.0], [8.0, 6.0, 5.0]], [[1.0, 2.0, 4.0], [6.0, 4.0, 4.0], [2.0, 7.0, 3.0]]]])
      val=Data(numpy.array([[[1.0, 2.0, 2.0], [6.0, 4.0, 3.0]], [[3.0, 4.0, 7.0], [5.0, 6.0, 3.0]], [[2.0, 2.0, 3.0], [6.0, 
3.0, 6.0]], [[3.0, 7.0, 4.0], [5.0, 1.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 3.0, 8.0], [1.0, 3.0, 2.0]], [[4.0, 1.0, 8.0], [7.0, 7.0, 5.0]], [[1.0, 6.0, 
7.0], [4.0, 8.0, 7.0]], [[5.0, 8.0, 1.0], [7.0, 6.0, 3.0]]]))
      arg[:,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 5.0], [1.0, 2.0, 2.0], [8.0, 5.0, 4.0]], [[2.0, 1.0, 4.0], [6.0, 4.0, 3.0], [7.0, 
4.0, 8.0]], [[4.0, 8.0, 4.0], [1.0, 8.0, 4.0], [7.0, 8.0, 4.0]]], [[[2.0, 3.0, 6.0], [3.0, 4.0, 7.0], [2.0, 6.0, 7.0]], [[4.0, 
4.0, 6.0], [5.0, 6.0, 3.0], [8.0, 7.0, 6.0]], [[6.0, 3.0, 4.0], [1.0, 4.0, 5.0], [3.0, 7.0, 1.0]]], [[[7.0, 7.0, 8.0], [2.0, 
2.0, 3.0], [7.0, 1.0, 8.0]], [[2.0, 1.0, 1.0], [6.0, 3.0, 6.0], [8.0, 8.0, 6.0]], [[5.0, 5.0, 3.0], [5.0, 1.0, 6.0], [8.0, 1.0, 
4.0]]], [[[8.0, 1.0, 1.0], [3.0, 7.0, 4.0], [8.0, 3.0, 4.0]], [[6.0, 4.0, 1.0], [5.0, 1.0, 7.0], [3.0, 2.0, 1.0]], [[4.0, 5.0, 
5.0], [7.0, 7.0, 3.0], [5.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 1.0], [3.0, 3.0, 8.0], [5.0, 4.0, 1.0]], 
[[6.0, 5.0, 1.0], [1.0, 3.0, 2.0], [4.0, 1.0, 3.0]], [[7.0, 7.0, 1.0], [2.0, 2.0, 7.0], [8.0, 3.0, 7.0]]], [[[6.0, 7.0, 2.0], 
[4.0, 1.0, 8.0], [5.0, 3.0, 4.0]], [[6.0, 2.0, 6.0], [7.0, 7.0, 5.0], [4.0, 3.0, 2.0]], [[1.0, 2.0, 2.0], [6.0, 3.0, 2.0], 
[2.0, 1.0, 1.0]]], [[[7.0, 3.0, 4.0], [1.0, 6.0, 7.0], [5.0, 8.0, 7.0]], [[6.0, 6.0, 2.0], [4.0, 8.0, 7.0], [7.0, 4.0, 4.0]], 
[[7.0, 1.0, 8.0], [4.0, 8.0, 3.0], [3.0, 7.0, 4.0]]], [[[3.0, 1.0, 5.0], [5.0, 8.0, 1.0], [4.0, 8.0, 1.0]], [[5.0, 1.0, 3.0], 
[7.0, 6.0, 3.0], [8.0, 6.0, 5.0]], [[1.0, 2.0, 4.0], [6.0, 4.0, 4.0], [2.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRPR(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 3.0], [6.0, 1.0, 7.0], [3.0, 3.0, 4.0]], [[6.0, 8.0, 2.0], [3.0, 6.0, 2.0], [5.0, 
2.0, 4.0]], [[2.0, 6.0, 4.0], [4.0, 7.0, 4.0], [4.0, 3.0, 8.0]]], [[[5.0, 1.0, 5.0], [2.0, 6.0, 5.0], [4.0, 3.0, 5.0]], [[6.0, 
7.0, 3.0], [7.0, 6.0, 4.0], [5.0, 1.0, 1.0]], [[8.0, 6.0, 2.0], [6.0, 4.0, 3.0], [8.0, 4.0, 6.0]]], [[[7.0, 7.0, 1.0], [5.0, 
7.0, 7.0], [7.0, 7.0, 6.0]], [[3.0, 7.0, 5.0], [4.0, 4.0, 5.0], [4.0, 5.0, 8.0]], [[7.0, 3.0, 2.0], [5.0, 7.0, 6.0], [8.0, 3.0, 
5.0]]], [[[1.0, 2.0, 3.0], [4.0, 1.0, 3.0], [4.0, 8.0, 7.0]], [[2.0, 5.0, 5.0], [1.0, 5.0, 3.0], [4.0, 8.0, 5.0]], [[7.0, 2.0, 
6.0], [3.0, 2.0, 4.0], [2.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 8.0, 4.0], [6.0, 3.0, 7.0], [3.0, 4.0, 1.0]], 
[[3.0, 7.0, 2.0], [6.0, 4.0, 5.0], [4.0, 5.0, 7.0]], [[4.0, 3.0, 4.0], [8.0, 1.0, 1.0], [1.0, 3.0, 1.0]]], [[[1.0, 3.0, 4.0], 
[7.0, 8.0, 5.0], [4.0, 2.0, 1.0]], [[6.0, 4.0, 8.0], [8.0, 4.0, 1.0], [4.0, 8.0, 4.0]], [[6.0, 4.0, 3.0], [3.0, 7.0, 8.0], 
[6.0, 3.0, 6.0]]], [[[7.0, 6.0, 5.0], [7.0, 1.0, 1.0], [8.0, 5.0, 3.0]], [[2.0, 4.0, 1.0], [4.0, 7.0, 7.0], [8.0, 6.0, 4.0]], 
[[8.0, 4.0, 6.0], [8.0, 2.0, 6.0], [6.0, 1.0, 6.0]]], [[[1.0, 6.0, 7.0], [7.0, 8.0, 1.0], [3.0, 4.0, 2.0]], [[1.0, 6.0, 4.0], 
[4.0, 6.0, 7.0], [1.0, 2.0, 6.0]], [[3.0, 5.0, 4.0], [4.0, 1.0, 3.0], [3.0, 6.0, 5.0]]]])
      val=Data(numpy.array([[[4.0]], [[2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0]], [[7.0]]]))
      arg[1:3,1:2,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 3.0], [6.0, 1.0, 7.0], [3.0, 3.0, 4.0]], [[6.0, 8.0, 2.0], [3.0, 6.0, 2.0], [5.0, 
2.0, 4.0]], [[2.0, 6.0, 4.0], [4.0, 7.0, 4.0], [4.0, 3.0, 8.0]]], [[[5.0, 1.0, 5.0], [2.0, 6.0, 5.0], [4.0, 3.0, 5.0]], [[6.0, 
7.0, 3.0], [7.0, 4.0, 4.0], [5.0, 1.0, 1.0]], [[8.0, 6.0, 2.0], [6.0, 4.0, 3.0], [8.0, 4.0, 6.0]]], [[[7.0, 7.0, 1.0], [5.0, 
7.0, 7.0], [7.0, 7.0, 6.0]], [[3.0, 7.0, 5.0], [4.0, 2.0, 5.0], [4.0, 5.0, 8.0]], [[7.0, 3.0, 2.0], [5.0, 7.0, 6.0], [8.0, 3.0, 
5.0]]], [[[1.0, 2.0, 3.0], [4.0, 1.0, 3.0], [4.0, 8.0, 7.0]], [[2.0, 5.0, 5.0], [1.0, 5.0, 3.0], [4.0, 8.0, 5.0]], [[7.0, 2.0, 
6.0], [3.0, 2.0, 4.0], [2.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 4.0], [6.0, 3.0, 7.0], [3.0, 4.0, 1.0]], 
[[3.0, 7.0, 2.0], [6.0, 4.0, 5.0], [4.0, 5.0, 7.0]], [[4.0, 3.0, 4.0], [8.0, 1.0, 1.0], [1.0, 3.0, 1.0]]], [[[1.0, 3.0, 4.0], 
[7.0, 8.0, 5.0], [4.0, 2.0, 1.0]], [[6.0, 4.0, 8.0], [8.0, 8.0, 1.0], [4.0, 8.0, 4.0]], [[6.0, 4.0, 3.0], [3.0, 7.0, 8.0], 
[6.0, 3.0, 6.0]]], [[[7.0, 6.0, 5.0], [7.0, 1.0, 1.0], [8.0, 5.0, 3.0]], [[2.0, 4.0, 1.0], [4.0, 7.0, 7.0], [8.0, 6.0, 4.0]], 
[[8.0, 4.0, 6.0], [8.0, 2.0, 6.0], [6.0, 1.0, 6.0]]], [[[1.0, 6.0, 7.0], [7.0, 8.0, 1.0], [3.0, 4.0, 2.0]], [[1.0, 6.0, 4.0], 
[4.0, 6.0, 7.0], [1.0, 2.0, 6.0]], [[3.0, 5.0, 4.0], [4.0, 1.0, 3.0], [3.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CLPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 1.0], [6.0, 6.0, 5.0], [6.0, 5.0, 2.0]], [[3.0, 3.0, 7.0], [3.0, 7.0, 1.0], [7.0, 
7.0, 1.0]], [[3.0, 7.0, 5.0], [4.0, 6.0, 6.0], [5.0, 2.0, 4.0]]], [[[4.0, 7.0, 2.0], [7.0, 1.0, 6.0], [2.0, 5.0, 2.0]], [[5.0, 
5.0, 3.0], [1.0, 6.0, 3.0], [6.0, 6.0, 7.0]], [[3.0, 3.0, 7.0], [8.0, 4.0, 1.0], [2.0, 1.0, 7.0]]], [[[4.0, 7.0, 4.0], [5.0, 
5.0, 8.0], [5.0, 5.0, 6.0]], [[8.0, 6.0, 1.0], [2.0, 2.0, 5.0], [1.0, 8.0, 8.0]], [[5.0, 8.0, 7.0], [6.0, 5.0, 8.0], [1.0, 5.0, 
5.0]]], [[[4.0, 6.0, 2.0], [8.0, 3.0, 5.0], [7.0, 6.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 6.0, 6.0], [3.0, 6.0, 8.0]], [[1.0, 3.0, 
1.0], [2.0, 5.0, 8.0], [3.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 6.0], [3.0, 3.0, 4.0], [6.0, 7.0, 5.0]], 
[[4.0, 4.0, 2.0], [2.0, 7.0, 4.0], [7.0, 1.0, 4.0]], [[4.0, 7.0, 4.0], [6.0, 8.0, 2.0], [5.0, 6.0, 2.0]]], [[[7.0, 5.0, 5.0], 
[8.0, 6.0, 1.0], [4.0, 3.0, 5.0]], [[7.0, 2.0, 4.0], [5.0, 6.0, 3.0], [7.0, 5.0, 6.0]], [[1.0, 1.0, 8.0], [5.0, 3.0, 7.0], 
[1.0, 7.0, 5.0]]], [[[7.0, 1.0, 5.0], [8.0, 5.0, 1.0], [2.0, 5.0, 6.0]], [[1.0, 8.0, 1.0], [3.0, 2.0, 2.0], [2.0, 7.0, 7.0]], 
[[1.0, 5.0, 1.0], [8.0, 3.0, 4.0], [4.0, 4.0, 8.0]]], [[[1.0, 4.0, 8.0], [7.0, 5.0, 7.0], [3.0, 5.0, 8.0]], [[7.0, 7.0, 2.0], 
[6.0, 7.0, 2.0], [7.0, 8.0, 5.0]], [[7.0, 1.0, 5.0], [4.0, 3.0, 7.0], [4.0, 2.0, 2.0]]]])
      val=Data(numpy.array([[[3.0, 1.0, 7.0]], [[3.0, 2.0, 8.0]], [[1.0, 7.0, 2.0]], [[1.0, 7.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 6.0, 1.0]], [[2.0, 1.0, 3.0]], [[5.0, 1.0, 4.0]], [[2.0, 8.0, 5.0]]]))
      arg[:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 1.0], [6.0, 6.0, 5.0], [6.0, 5.0, 2.0]], [[3.0, 3.0, 7.0], [3.0, 7.0, 1.0], [7.0, 
7.0, 1.0]], [[3.0, 7.0, 5.0], [3.0, 1.0, 7.0], [5.0, 2.0, 4.0]]], [[[4.0, 7.0, 2.0], [7.0, 1.0, 6.0], [2.0, 5.0, 2.0]], [[5.0, 
5.0, 3.0], [1.0, 6.0, 3.0], [6.0, 6.0, 7.0]], [[3.0, 3.0, 7.0], [3.0, 2.0, 8.0], [2.0, 1.0, 7.0]]], [[[4.0, 7.0, 4.0], [5.0, 
5.0, 8.0], [5.0, 5.0, 6.0]], [[8.0, 6.0, 1.0], [2.0, 2.0, 5.0], [1.0, 8.0, 8.0]], [[5.0, 8.0, 7.0], [1.0, 7.0, 2.0], [1.0, 5.0, 
5.0]]], [[[4.0, 6.0, 2.0], [8.0, 3.0, 5.0], [7.0, 6.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 6.0, 6.0], [3.0, 6.0, 8.0]], [[1.0, 3.0, 
1.0], [1.0, 7.0, 1.0], [3.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 6.0], [3.0, 3.0, 4.0], [6.0, 7.0, 5.0]], 
[[4.0, 4.0, 2.0], [2.0, 7.0, 4.0], [7.0, 1.0, 4.0]], [[4.0, 7.0, 4.0], [4.0, 6.0, 1.0], [5.0, 6.0, 2.0]]], [[[7.0, 5.0, 5.0], 
[8.0, 6.0, 1.0], [4.0, 3.0, 5.0]], [[7.0, 2.0, 4.0], [5.0, 6.0, 3.0], [7.0, 5.0, 6.0]], [[1.0, 1.0, 8.0], [2.0, 1.0, 3.0], 
[1.0, 7.0, 5.0]]], [[[7.0, 1.0, 5.0], [8.0, 5.0, 1.0], [2.0, 5.0, 6.0]], [[1.0, 8.0, 1.0], [3.0, 2.0, 2.0], [2.0, 7.0, 7.0]], 
[[1.0, 5.0, 1.0], [5.0, 1.0, 4.0], [4.0, 4.0, 8.0]]], [[[1.0, 4.0, 8.0], [7.0, 5.0, 7.0], [3.0, 5.0, 8.0]], [[7.0, 7.0, 2.0], 
[6.0, 7.0, 2.0], [7.0, 8.0, 5.0]], [[7.0, 1.0, 5.0], [2.0, 8.0, 5.0], [4.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 4.0], [8.0, 3.0, 2.0], [3.0, 4.0, 5.0]], [[1.0, 1.0, 2.0], [3.0, 8.0, 7.0], [5.0, 
8.0, 3.0]], [[6.0, 1.0, 5.0], [4.0, 3.0, 8.0], [5.0, 2.0, 4.0]]], [[[1.0, 7.0, 1.0], [8.0, 8.0, 4.0], [1.0, 4.0, 5.0]], [[5.0, 
7.0, 5.0], [2.0, 4.0, 7.0], [5.0, 5.0, 4.0]], [[2.0, 7.0, 2.0], [2.0, 4.0, 5.0], [1.0, 3.0, 4.0]]], [[[5.0, 6.0, 4.0], [8.0, 
5.0, 6.0], [4.0, 5.0, 3.0]], [[5.0, 6.0, 5.0], [7.0, 1.0, 2.0], [3.0, 3.0, 5.0]], [[6.0, 6.0, 2.0], [1.0, 2.0, 2.0], [3.0, 6.0, 
2.0]]], [[[1.0, 1.0, 8.0], [7.0, 7.0, 8.0], [8.0, 2.0, 3.0]], [[1.0, 4.0, 4.0], [4.0, 2.0, 8.0], [6.0, 5.0, 4.0]], [[2.0, 7.0, 
8.0], [1.0, 3.0, 5.0], [2.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 7.0], [8.0, 8.0, 5.0], [3.0, 7.0, 4.0]], 
[[4.0, 3.0, 7.0], [8.0, 4.0, 7.0], [8.0, 2.0, 7.0]], [[1.0, 4.0, 3.0], [1.0, 7.0, 2.0], [6.0, 5.0, 8.0]]], [[[4.0, 3.0, 8.0], 
[8.0, 8.0, 5.0], [2.0, 2.0, 7.0]], [[2.0, 3.0, 7.0], [7.0, 5.0, 1.0], [2.0, 7.0, 3.0]], [[6.0, 3.0, 3.0], [2.0, 5.0, 8.0], 
[6.0, 5.0, 3.0]]], [[[2.0, 2.0, 2.0], [3.0, 8.0, 2.0], [7.0, 7.0, 1.0]], [[3.0, 8.0, 7.0], [5.0, 4.0, 4.0], [4.0, 4.0, 2.0]], 
[[3.0, 2.0, 3.0], [2.0, 2.0, 6.0], [7.0, 5.0, 8.0]]], [[[8.0, 1.0, 6.0], [1.0, 4.0, 1.0], [6.0, 2.0, 1.0]], [[5.0, 8.0, 2.0], 
[3.0, 4.0, 7.0], [5.0, 5.0, 6.0]], [[5.0, 8.0, 5.0], [6.0, 1.0, 7.0], [5.0, 3.0, 5.0]]]])
      val=Data(numpy.array([[[[5.0, 7.0, 2.0], [1.0, 5.0, 7.0], [8.0, 5.0, 2.0]]], [[[7.0, 1.0, 3.0], [3.0, 7.0, 6.0], [6.0, 
7.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 4.0, 2.0], [8.0, 8.0, 1.0], [1.0, 1.0, 6.0]]], [[[7.0, 2.0, 1.0], [7.0, 3.0, 
4.0], [6.0, 4.0, 1.0]]]]))
      arg[2:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 4.0], [8.0, 3.0, 2.0], [3.0, 4.0, 5.0]], [[1.0, 1.0, 2.0], [3.0, 8.0, 7.0], [5.0, 
8.0, 3.0]], [[6.0, 1.0, 5.0], [4.0, 3.0, 8.0], [5.0, 2.0, 4.0]]], [[[1.0, 7.0, 1.0], [8.0, 8.0, 4.0], [1.0, 4.0, 5.0]], [[5.0, 
7.0, 5.0], [2.0, 4.0, 7.0], [5.0, 5.0, 4.0]], [[2.0, 7.0, 2.0], [2.0, 4.0, 5.0], [1.0, 3.0, 4.0]]], [[[5.0, 6.0, 4.0], [8.0, 
5.0, 6.0], [4.0, 5.0, 3.0]], [[5.0, 7.0, 2.0], [1.0, 5.0, 7.0], [8.0, 5.0, 2.0]], [[6.0, 6.0, 2.0], [1.0, 2.0, 2.0], [3.0, 6.0, 
2.0]]], [[[1.0, 1.0, 8.0], [7.0, 7.0, 8.0], [8.0, 2.0, 3.0]], [[7.0, 1.0, 3.0], [3.0, 7.0, 6.0], [6.0, 7.0, 5.0]], [[2.0, 7.0, 
8.0], [1.0, 3.0, 5.0], [2.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 7.0], [8.0, 8.0, 5.0], [3.0, 7.0, 4.0]], 
[[4.0, 3.0, 7.0], [8.0, 4.0, 7.0], [8.0, 2.0, 7.0]], [[1.0, 4.0, 3.0], [1.0, 7.0, 2.0], [6.0, 5.0, 8.0]]], [[[4.0, 3.0, 8.0], 
[8.0, 8.0, 5.0], [2.0, 2.0, 7.0]], [[2.0, 3.0, 7.0], [7.0, 5.0, 1.0], [2.0, 7.0, 3.0]], [[6.0, 3.0, 3.0], [2.0, 5.0, 8.0], 
[6.0, 5.0, 3.0]]], [[[2.0, 2.0, 2.0], [3.0, 8.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 4.0, 2.0], [8.0, 8.0, 1.0], [1.0, 1.0, 6.0]], 
[[3.0, 2.0, 3.0], [2.0, 2.0, 6.0], [7.0, 5.0, 8.0]]], [[[8.0, 1.0, 6.0], [1.0, 4.0, 1.0], [6.0, 2.0, 1.0]], [[7.0, 2.0, 1.0], 
[7.0, 3.0, 4.0], [6.0, 4.0, 1.0]], [[5.0, 8.0, 5.0], [6.0, 1.0, 7.0], [5.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 2.0, 2.0], [8.0, 2.0, 1.0], [6.0, 3.0, 8.0]], [[7.0, 8.0, 8.0], [2.0, 2.0, 1.0], [5.0, 
6.0, 8.0]], [[3.0, 6.0, 5.0], [8.0, 7.0, 4.0], [6.0, 3.0, 2.0]]], [[[2.0, 3.0, 7.0], [8.0, 2.0, 4.0], [6.0, 7.0, 2.0]], [[8.0, 
4.0, 3.0], [3.0, 4.0, 4.0], [4.0, 4.0, 4.0]], [[8.0, 6.0, 1.0], [6.0, 5.0, 1.0], [2.0, 7.0, 4.0]]], [[[5.0, 3.0, 3.0], [8.0, 
2.0, 2.0], [2.0, 1.0, 6.0]], [[3.0, 7.0, 8.0], [7.0, 4.0, 6.0], [1.0, 8.0, 3.0]], [[7.0, 1.0, 7.0], [3.0, 2.0, 6.0], [3.0, 3.0, 
5.0]]], [[[3.0, 1.0, 4.0], [4.0, 2.0, 6.0], [6.0, 3.0, 5.0]], [[6.0, 3.0, 6.0], [3.0, 3.0, 5.0], [2.0, 4.0, 6.0]], [[6.0, 6.0, 
6.0], [5.0, 5.0, 2.0], [3.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 8.0], [8.0, 1.0, 8.0], [6.0, 2.0, 5.0]], 
[[7.0, 3.0, 6.0], [6.0, 7.0, 4.0], [3.0, 1.0, 2.0]], [[1.0, 2.0, 5.0], [3.0, 1.0, 7.0], [3.0, 6.0, 7.0]]], [[[8.0, 4.0, 7.0], 
[5.0, 2.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 8.0, 2.0], [3.0, 8.0, 8.0], [7.0, 5.0, 1.0]], [[6.0, 8.0, 2.0], [6.0, 2.0, 4.0], 
[8.0, 6.0, 4.0]]], [[[3.0, 5.0, 5.0], [3.0, 4.0, 6.0], [4.0, 3.0, 2.0]], [[6.0, 7.0, 7.0], [7.0, 6.0, 5.0], [4.0, 2.0, 5.0]], 
[[4.0, 5.0, 8.0], [6.0, 4.0, 8.0], [6.0, 3.0, 2.0]]], [[[1.0, 6.0, 3.0], [1.0, 5.0, 2.0], [4.0, 6.0, 7.0]], [[2.0, 5.0, 7.0], 
[2.0, 3.0, 2.0], [1.0, 8.0, 2.0]], [[5.0, 5.0, 8.0], [4.0, 5.0, 5.0], [1.0, 8.0, 3.0]]]])
      val=Data(numpy.array([[[[4.0, 4.0, 8.0]]], [[[8.0, 8.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 4.0]]], [[[1.0, 5.0, 4.0]]]]))
      arg[2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 2.0], [8.0, 2.0, 1.0], [6.0, 3.0, 8.0]], [[7.0, 8.0, 8.0], [2.0, 2.0, 1.0], [5.0, 
6.0, 8.0]], [[3.0, 6.0, 5.0], [8.0, 7.0, 4.0], [6.0, 3.0, 2.0]]], [[[2.0, 3.0, 7.0], [8.0, 2.0, 4.0], [6.0, 7.0, 2.0]], [[8.0, 
4.0, 3.0], [3.0, 4.0, 4.0], [4.0, 4.0, 4.0]], [[8.0, 6.0, 1.0], [6.0, 5.0, 1.0], [2.0, 7.0, 4.0]]], [[[5.0, 3.0, 3.0], [8.0, 
2.0, 2.0], [2.0, 1.0, 6.0]], [[3.0, 7.0, 8.0], [7.0, 4.0, 6.0], [1.0, 8.0, 3.0]], [[7.0, 1.0, 7.0], [3.0, 2.0, 6.0], [4.0, 4.0, 
8.0]]], [[[3.0, 1.0, 4.0], [4.0, 2.0, 6.0], [6.0, 3.0, 5.0]], [[6.0, 3.0, 6.0], [3.0, 3.0, 5.0], [2.0, 4.0, 6.0]], [[6.0, 6.0, 
6.0], [5.0, 5.0, 2.0], [8.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 8.0], [8.0, 1.0, 8.0], [6.0, 2.0, 5.0]], 
[[7.0, 3.0, 6.0], [6.0, 7.0, 4.0], [3.0, 1.0, 2.0]], [[1.0, 2.0, 5.0], [3.0, 1.0, 7.0], [3.0, 6.0, 7.0]]], [[[8.0, 4.0, 7.0], 
[5.0, 2.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 8.0, 2.0], [3.0, 8.0, 8.0], [7.0, 5.0, 1.0]], [[6.0, 8.0, 2.0], [6.0, 2.0, 4.0], 
[8.0, 6.0, 4.0]]], [[[3.0, 5.0, 5.0], [3.0, 4.0, 6.0], [4.0, 3.0, 2.0]], [[6.0, 7.0, 7.0], [7.0, 6.0, 5.0], [4.0, 2.0, 5.0]], 
[[4.0, 5.0, 8.0], [6.0, 4.0, 8.0], [6.0, 1.0, 4.0]]], [[[1.0, 6.0, 3.0], [1.0, 5.0, 2.0], [4.0, 6.0, 7.0]], [[2.0, 5.0, 7.0], 
[2.0, 3.0, 2.0], [1.0, 8.0, 2.0]], [[5.0, 5.0, 8.0], [4.0, 5.0, 5.0], [1.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LPUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 1.0], [2.0, 2.0, 6.0], [3.0, 2.0, 4.0]], [[2.0, 2.0, 4.0], [3.0, 5.0, 8.0], [5.0, 
7.0, 6.0]], [[2.0, 6.0, 3.0], [4.0, 6.0, 2.0], [1.0, 4.0, 2.0]]], [[[8.0, 1.0, 3.0], [3.0, 1.0, 2.0], [3.0, 6.0, 1.0]], [[6.0, 
7.0, 1.0], [8.0, 2.0, 8.0], [4.0, 1.0, 3.0]], [[5.0, 6.0, 6.0], [7.0, 3.0, 3.0], [6.0, 8.0, 2.0]]], [[[7.0, 6.0, 3.0], [4.0, 
4.0, 7.0], [2.0, 1.0, 8.0]], [[6.0, 4.0, 4.0], [2.0, 3.0, 3.0], [2.0, 2.0, 6.0]], [[5.0, 6.0, 5.0], [8.0, 8.0, 3.0], [6.0, 2.0, 
5.0]]], [[[4.0, 1.0, 2.0], [4.0, 5.0, 3.0], [8.0, 4.0, 3.0]], [[5.0, 6.0, 4.0], [6.0, 8.0, 8.0], [7.0, 2.0, 6.0]], [[2.0, 6.0, 
3.0], [5.0, 1.0, 5.0], [2.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 6.0], [3.0, 7.0, 5.0], [3.0, 7.0, 4.0]], 
[[6.0, 7.0, 5.0], [8.0, 7.0, 3.0], [5.0, 6.0, 8.0]], [[3.0, 7.0, 4.0], [3.0, 1.0, 4.0], [7.0, 8.0, 5.0]]], [[[7.0, 3.0, 2.0], 
[6.0, 6.0, 3.0], [2.0, 6.0, 5.0]], [[5.0, 4.0, 7.0], [2.0, 8.0, 1.0], [6.0, 5.0, 5.0]], [[7.0, 1.0, 6.0], [7.0, 5.0, 7.0], 
[3.0, 8.0, 5.0]]], [[[7.0, 3.0, 2.0], [4.0, 2.0, 8.0], [1.0, 5.0, 6.0]], [[4.0, 8.0, 3.0], [6.0, 7.0, 5.0], [7.0, 5.0, 5.0]], 
[[6.0, 7.0, 3.0], [4.0, 3.0, 7.0], [5.0, 7.0, 4.0]]], [[[6.0, 3.0, 5.0], [6.0, 6.0, 5.0], [4.0, 3.0, 5.0]], [[4.0, 3.0, 1.0], 
[5.0, 3.0, 7.0], [3.0, 1.0, 2.0]], [[2.0, 2.0, 5.0], [3.0, 2.0, 7.0], [4.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[3.0, 1.0, 6.0], [1.0, 2.0, 2.0]], [[7.0, 2.0, 8.0], [2.0, 6.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 1.0, 4.0], [4.0, 6.0, 4.0]], [[7.0, 8.0, 2.0], [8.0, 1.0, 3.0]]]))
      arg[2:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 1.0], [2.0, 2.0, 6.0], [3.0, 2.0, 4.0]], [[2.0, 2.0, 4.0], [3.0, 5.0, 8.0], [5.0, 
7.0, 6.0]], [[2.0, 6.0, 3.0], [4.0, 6.0, 2.0], [1.0, 4.0, 2.0]]], [[[8.0, 1.0, 3.0], [3.0, 1.0, 2.0], [3.0, 6.0, 1.0]], [[6.0, 
7.0, 1.0], [8.0, 2.0, 8.0], [4.0, 1.0, 3.0]], [[5.0, 6.0, 6.0], [7.0, 3.0, 3.0], [6.0, 8.0, 2.0]]], [[[7.0, 6.0, 3.0], [4.0, 
4.0, 7.0], [2.0, 1.0, 8.0]], [[3.0, 1.0, 6.0], [1.0, 2.0, 2.0], [2.0, 2.0, 6.0]], [[5.0, 6.0, 5.0], [8.0, 8.0, 3.0], [6.0, 2.0, 
5.0]]], [[[4.0, 1.0, 2.0], [4.0, 5.0, 3.0], [8.0, 4.0, 3.0]], [[7.0, 2.0, 8.0], [2.0, 6.0, 3.0], [7.0, 2.0, 6.0]], [[2.0, 6.0, 
3.0], [5.0, 1.0, 5.0], [2.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 6.0], [3.0, 7.0, 5.0], [3.0, 7.0, 4.0]], 
[[6.0, 7.0, 5.0], [8.0, 7.0, 3.0], [5.0, 6.0, 8.0]], [[3.0, 7.0, 4.0], [3.0, 1.0, 4.0], [7.0, 8.0, 5.0]]], [[[7.0, 3.0, 2.0], 
[6.0, 6.0, 3.0], [2.0, 6.0, 5.0]], [[5.0, 4.0, 7.0], [2.0, 8.0, 1.0], [6.0, 5.0, 5.0]], [[7.0, 1.0, 6.0], [7.0, 5.0, 7.0], 
[3.0, 8.0, 5.0]]], [[[7.0, 3.0, 2.0], [4.0, 2.0, 8.0], [1.0, 5.0, 6.0]], [[5.0, 1.0, 4.0], [4.0, 6.0, 4.0], [7.0, 5.0, 5.0]], 
[[6.0, 7.0, 3.0], [4.0, 3.0, 7.0], [5.0, 7.0, 4.0]]], [[[6.0, 3.0, 5.0], [6.0, 6.0, 5.0], [4.0, 3.0, 5.0]], [[7.0, 8.0, 2.0], 
[8.0, 1.0, 3.0], [3.0, 1.0, 2.0]], [[2.0, 2.0, 5.0], [3.0, 2.0, 7.0], [4.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RCLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 3.0], [5.0, 2.0, 2.0], [4.0, 5.0, 3.0]], [[4.0, 5.0, 5.0], [6.0, 1.0, 4.0], [2.0, 
6.0, 8.0]], [[8.0, 6.0, 5.0], [1.0, 8.0, 7.0], [7.0, 8.0, 4.0]]], [[[5.0, 7.0, 6.0], [1.0, 4.0, 6.0], [4.0, 3.0, 1.0]], [[6.0, 
8.0, 8.0], [5.0, 5.0, 4.0], [6.0, 7.0, 8.0]], [[4.0, 8.0, 7.0], [1.0, 7.0, 8.0], [8.0, 3.0, 5.0]]], [[[7.0, 4.0, 1.0], [8.0, 
2.0, 2.0], [1.0, 8.0, 8.0]], [[8.0, 1.0, 6.0], [8.0, 2.0, 5.0], [1.0, 5.0, 7.0]], [[8.0, 2.0, 3.0], [4.0, 5.0, 3.0], [5.0, 6.0, 
3.0]]], [[[5.0, 7.0, 4.0], [5.0, 3.0, 2.0], [6.0, 3.0, 7.0]], [[5.0, 1.0, 5.0], [6.0, 7.0, 4.0], [2.0, 5.0, 1.0]], [[2.0, 7.0, 
4.0], [1.0, 2.0, 2.0], [2.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 4.0], [4.0, 5.0, 7.0], [1.0, 1.0, 4.0]], 
[[7.0, 5.0, 7.0], [3.0, 8.0, 6.0], [2.0, 7.0, 7.0]], [[5.0, 4.0, 4.0], [6.0, 5.0, 2.0], [3.0, 4.0, 6.0]]], [[[1.0, 2.0, 8.0], 
[7.0, 4.0, 6.0], [2.0, 8.0, 8.0]], [[7.0, 3.0, 7.0], [4.0, 5.0, 2.0], [1.0, 6.0, 3.0]], [[3.0, 2.0, 3.0], [6.0, 4.0, 7.0], 
[2.0, 2.0, 3.0]]], [[[4.0, 4.0, 8.0], [5.0, 2.0, 1.0], [3.0, 5.0, 4.0]], [[4.0, 4.0, 1.0], [5.0, 4.0, 7.0], [4.0, 4.0, 3.0]], 
[[6.0, 3.0, 1.0], [4.0, 2.0, 2.0], [5.0, 4.0, 1.0]]], [[[1.0, 1.0, 6.0], [2.0, 2.0, 1.0], [6.0, 3.0, 7.0]], [[8.0, 6.0, 7.0], 
[1.0, 6.0, 7.0], [8.0, 6.0, 8.0]], [[5.0, 8.0, 4.0], [4.0, 2.0, 3.0], [2.0, 1.0, 8.0]]]])
      val=Data(numpy.array([[[[3.0, 5.0, 3.0]], [[1.0, 4.0, 1.0]], [[3.0, 2.0, 3.0]]], [[[3.0, 1.0, 3.0]], [[1.0, 5.0, 1.0]], 
[[5.0, 5.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[6.0, 1.0, 7.0]], [[1.0, 8.0, 1.0]], [[5.0, 6.0, 6.0]]], [[[2.0, 2.0, 1.0]], [[7.0, 
1.0, 4.0]], [[1.0, 5.0, 6.0]]]]))
      arg[1:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 3.0], [5.0, 2.0, 2.0], [4.0, 5.0, 3.0]], [[4.0, 5.0, 5.0], [6.0, 1.0, 4.0], [2.0, 
6.0, 8.0]], [[8.0, 6.0, 5.0], [1.0, 8.0, 7.0], [7.0, 8.0, 4.0]]], [[[5.0, 7.0, 6.0], [1.0, 4.0, 6.0], [3.0, 5.0, 3.0]], [[6.0, 
8.0, 8.0], [5.0, 5.0, 4.0], [1.0, 4.0, 1.0]], [[4.0, 8.0, 7.0], [1.0, 7.0, 8.0], [3.0, 2.0, 3.0]]], [[[7.0, 4.0, 1.0], [8.0, 
2.0, 2.0], [3.0, 1.0, 3.0]], [[8.0, 1.0, 6.0], [8.0, 2.0, 5.0], [1.0, 5.0, 1.0]], [[8.0, 2.0, 3.0], [4.0, 5.0, 3.0], [5.0, 5.0, 
1.0]]], [[[5.0, 7.0, 4.0], [5.0, 3.0, 2.0], [6.0, 3.0, 7.0]], [[5.0, 1.0, 5.0], [6.0, 7.0, 4.0], [2.0, 5.0, 1.0]], [[2.0, 7.0, 
4.0], [1.0, 2.0, 2.0], [2.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 4.0], [4.0, 5.0, 7.0], [1.0, 1.0, 4.0]], 
[[7.0, 5.0, 7.0], [3.0, 8.0, 6.0], [2.0, 7.0, 7.0]], [[5.0, 4.0, 4.0], [6.0, 5.0, 2.0], [3.0, 4.0, 6.0]]], [[[1.0, 2.0, 8.0], 
[7.0, 4.0, 6.0], [6.0, 1.0, 7.0]], [[7.0, 3.0, 7.0], [4.0, 5.0, 2.0], [1.0, 8.0, 1.0]], [[3.0, 2.0, 3.0], [6.0, 4.0, 7.0], 
[5.0, 6.0, 6.0]]], [[[4.0, 4.0, 8.0], [5.0, 2.0, 1.0], [2.0, 2.0, 1.0]], [[4.0, 4.0, 1.0], [5.0, 4.0, 7.0], [7.0, 1.0, 4.0]], 
[[6.0, 3.0, 1.0], [4.0, 2.0, 2.0], [1.0, 5.0, 6.0]]], [[[1.0, 1.0, 6.0], [2.0, 2.0, 1.0], [6.0, 3.0, 7.0]], [[8.0, 6.0, 7.0], 
[1.0, 6.0, 7.0], [8.0, 6.0, 8.0]], [[5.0, 8.0, 4.0], [4.0, 2.0, 3.0], [2.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 2.0], [2.0, 4.0, 4.0], [6.0, 2.0, 8.0]], [[1.0, 6.0, 4.0], [8.0, 1.0, 6.0], [4.0, 
7.0, 7.0]], [[3.0, 7.0, 3.0], [6.0, 8.0, 1.0], [8.0, 4.0, 1.0]]], [[[7.0, 6.0, 3.0], [2.0, 3.0, 2.0], [8.0, 5.0, 5.0]], [[7.0, 
7.0, 8.0], [5.0, 1.0, 1.0], [2.0, 6.0, 6.0]], [[3.0, 8.0, 5.0], [8.0, 3.0, 1.0], [3.0, 8.0, 8.0]]], [[[3.0, 7.0, 6.0], [3.0, 
5.0, 8.0], [3.0, 4.0, 8.0]], [[7.0, 8.0, 6.0], [2.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[4.0, 3.0, 4.0], [4.0, 1.0, 3.0], [5.0, 2.0, 
2.0]]], [[[4.0, 1.0, 1.0], [1.0, 7.0, 4.0], [5.0, 4.0, 7.0]], [[2.0, 8.0, 2.0], [6.0, 1.0, 6.0], [3.0, 3.0, 7.0]], [[6.0, 7.0, 
2.0], [6.0, 4.0, 7.0], [6.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 2.0], [4.0, 3.0, 3.0], [1.0, 4.0, 4.0]], 
[[4.0, 3.0, 7.0], [1.0, 5.0, 7.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 1.0], [4.0, 1.0, 5.0], [5.0, 5.0, 8.0]]], [[[1.0, 2.0, 1.0], 
[6.0, 5.0, 3.0], [1.0, 1.0, 3.0]], [[8.0, 3.0, 7.0], [8.0, 7.0, 3.0], [5.0, 5.0, 5.0]], [[5.0, 2.0, 8.0], [8.0, 2.0, 4.0], 
[6.0, 7.0, 2.0]]], [[[4.0, 2.0, 8.0], [7.0, 4.0, 1.0], [4.0, 1.0, 6.0]], [[6.0, 8.0, 7.0], [3.0, 1.0, 4.0], [3.0, 6.0, 4.0]], 
[[2.0, 8.0, 4.0], [7.0, 4.0, 4.0], [4.0, 8.0, 2.0]]], [[[6.0, 6.0, 2.0], [1.0, 7.0, 7.0], [4.0, 2.0, 3.0]], [[4.0, 4.0, 2.0], 
[7.0, 3.0, 3.0], [8.0, 1.0, 6.0]], [[4.0, 8.0, 2.0], [2.0, 3.0, 6.0], [2.0, 6.0, 8.0]]]])
      val=Data(numpy.array([[4.0, 4.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[7.0, 8.0, 2.0]]))
      arg[2,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 2.0], [2.0, 4.0, 4.0], [6.0, 2.0, 8.0]], [[1.0, 6.0, 4.0], [8.0, 1.0, 6.0], [4.0, 
7.0, 7.0]], [[3.0, 7.0, 3.0], [6.0, 8.0, 1.0], [8.0, 4.0, 1.0]]], [[[7.0, 6.0, 3.0], [2.0, 3.0, 2.0], [8.0, 5.0, 5.0]], [[7.0, 
7.0, 8.0], [5.0, 1.0, 1.0], [2.0, 6.0, 6.0]], [[3.0, 8.0, 5.0], [8.0, 3.0, 1.0], [3.0, 8.0, 8.0]]], [[[3.0, 7.0, 6.0], [3.0, 
5.0, 8.0], [3.0, 4.0, 8.0]], [[7.0, 8.0, 6.0], [2.0, 7.0, 1.0], [4.0, 4.0, 3.0]], [[4.0, 3.0, 4.0], [4.0, 1.0, 3.0], [5.0, 2.0, 
2.0]]], [[[4.0, 1.0, 1.0], [1.0, 7.0, 4.0], [5.0, 4.0, 7.0]], [[2.0, 8.0, 2.0], [6.0, 1.0, 6.0], [3.0, 3.0, 7.0]], [[6.0, 7.0, 
2.0], [6.0, 4.0, 7.0], [6.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 2.0], [4.0, 3.0, 3.0], [1.0, 4.0, 4.0]], 
[[4.0, 3.0, 7.0], [1.0, 5.0, 7.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 1.0], [4.0, 1.0, 5.0], [5.0, 5.0, 8.0]]], [[[1.0, 2.0, 1.0], 
[6.0, 5.0, 3.0], [1.0, 1.0, 3.0]], [[8.0, 3.0, 7.0], [8.0, 7.0, 3.0], [5.0, 5.0, 5.0]], [[5.0, 2.0, 8.0], [8.0, 2.0, 4.0], 
[6.0, 7.0, 2.0]]], [[[4.0, 2.0, 8.0], [7.0, 4.0, 1.0], [4.0, 1.0, 6.0]], [[6.0, 8.0, 7.0], [3.0, 1.0, 4.0], [7.0, 8.0, 2.0]], 
[[2.0, 8.0, 4.0], [7.0, 4.0, 4.0], [4.0, 8.0, 2.0]]], [[[6.0, 6.0, 2.0], [1.0, 7.0, 7.0], [4.0, 2.0, 3.0]], [[4.0, 4.0, 2.0], 
[7.0, 3.0, 3.0], [8.0, 1.0, 6.0]], [[4.0, 8.0, 2.0], [2.0, 3.0, 6.0], [2.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 7.0], [7.0, 7.0, 7.0], [5.0, 1.0, 4.0]], [[7.0, 2.0, 2.0], [8.0, 7.0, 5.0], [8.0, 
6.0, 7.0]], [[8.0, 3.0, 3.0], [1.0, 3.0, 5.0], [1.0, 7.0, 1.0]]], [[[7.0, 2.0, 5.0], [5.0, 1.0, 4.0], [6.0, 4.0, 3.0]], [[3.0, 
6.0, 5.0], [5.0, 8.0, 4.0], [5.0, 4.0, 2.0]], [[8.0, 8.0, 7.0], [1.0, 5.0, 3.0], [8.0, 7.0, 1.0]]], [[[1.0, 2.0, 6.0], [1.0, 
1.0, 2.0], [7.0, 7.0, 3.0]], [[7.0, 8.0, 8.0], [5.0, 3.0, 5.0], [8.0, 8.0, 5.0]], [[3.0, 5.0, 3.0], [8.0, 5.0, 4.0], [8.0, 6.0, 
4.0]]], [[[6.0, 7.0, 2.0], [1.0, 5.0, 6.0], [7.0, 5.0, 4.0]], [[1.0, 7.0, 4.0], [4.0, 7.0, 8.0], [6.0, 3.0, 3.0]], [[6.0, 3.0, 
6.0], [7.0, 8.0, 6.0], [8.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 2.0], [8.0, 1.0, 3.0], [4.0, 4.0, 1.0]], 
[[5.0, 2.0, 4.0], [3.0, 7.0, 1.0], [4.0, 6.0, 5.0]], [[5.0, 5.0, 8.0], [5.0, 1.0, 1.0], [1.0, 2.0, 6.0]]], [[[3.0, 3.0, 7.0], 
[8.0, 3.0, 3.0], [7.0, 6.0, 4.0]], [[2.0, 5.0, 6.0], [3.0, 8.0, 2.0], [5.0, 1.0, 6.0]], [[6.0, 7.0, 8.0], [6.0, 1.0, 1.0], 
[5.0, 3.0, 6.0]]], [[[5.0, 2.0, 2.0], [8.0, 6.0, 3.0], [4.0, 6.0, 2.0]], [[8.0, 1.0, 6.0], [7.0, 7.0, 4.0], [8.0, 4.0, 8.0]], 
[[1.0, 7.0, 7.0], [5.0, 3.0, 6.0], [3.0, 4.0, 4.0]]], [[[4.0, 6.0, 2.0], [3.0, 6.0, 6.0], [1.0, 1.0, 7.0]], [[3.0, 2.0, 2.0], 
[1.0, 7.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 8.0, 8.0], [5.0, 1.0, 2.0], [1.0, 7.0, 2.0]]]])
      val=Data(numpy.array([[[[3.0, 2.0, 8.0]], [[1.0, 2.0, 7.0]], [[3.0, 2.0, 2.0]]], [[[1.0, 7.0, 6.0]], [[2.0, 8.0, 2.0]], 
[[5.0, 7.0, 2.0]]], [[[6.0, 2.0, 7.0]], [[4.0, 6.0, 6.0]], [[5.0, 7.0, 4.0]]], [[[1.0, 7.0, 5.0]], [[7.0, 6.0, 3.0]], [[4.0, 
2.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 2.0]], [[2.0, 5.0, 1.0]], [[8.0, 4.0, 2.0]]], [[[1.0, 5.0, 8.0]], [[6.0, 
1.0, 4.0]], [[6.0, 3.0, 2.0]]], [[[3.0, 5.0, 3.0]], [[3.0, 5.0, 7.0]], [[1.0, 8.0, 6.0]]], [[[3.0, 1.0, 6.0]], [[6.0, 4.0, 
6.0]], [[1.0, 3.0, 6.0]]]]))
      arg[:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 7.0], [3.0, 2.0, 8.0], [5.0, 1.0, 4.0]], [[7.0, 2.0, 2.0], [1.0, 2.0, 7.0], [8.0, 
6.0, 7.0]], [[8.0, 3.0, 3.0], [3.0, 2.0, 2.0], [1.0, 7.0, 1.0]]], [[[7.0, 2.0, 5.0], [1.0, 7.0, 6.0], [6.0, 4.0, 3.0]], [[3.0, 
6.0, 5.0], [2.0, 8.0, 2.0], [5.0, 4.0, 2.0]], [[8.0, 8.0, 7.0], [5.0, 7.0, 2.0], [8.0, 7.0, 1.0]]], [[[1.0, 2.0, 6.0], [6.0, 
2.0, 7.0], [7.0, 7.0, 3.0]], [[7.0, 8.0, 8.0], [4.0, 6.0, 6.0], [8.0, 8.0, 5.0]], [[3.0, 5.0, 3.0], [5.0, 7.0, 4.0], [8.0, 6.0, 
4.0]]], [[[6.0, 7.0, 2.0], [1.0, 7.0, 5.0], [7.0, 5.0, 4.0]], [[1.0, 7.0, 4.0], [7.0, 6.0, 3.0], [6.0, 3.0, 3.0]], [[6.0, 3.0, 
6.0], [4.0, 2.0, 2.0], [8.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 2.0], [3.0, 6.0, 2.0], [4.0, 4.0, 1.0]], 
[[5.0, 2.0, 4.0], [2.0, 5.0, 1.0], [4.0, 6.0, 5.0]], [[5.0, 5.0, 8.0], [8.0, 4.0, 2.0], [1.0, 2.0, 6.0]]], [[[3.0, 3.0, 7.0], 
[1.0, 5.0, 8.0], [7.0, 6.0, 4.0]], [[2.0, 5.0, 6.0], [6.0, 1.0, 4.0], [5.0, 1.0, 6.0]], [[6.0, 7.0, 8.0], [6.0, 3.0, 2.0], 
[5.0, 3.0, 6.0]]], [[[5.0, 2.0, 2.0], [3.0, 5.0, 3.0], [4.0, 6.0, 2.0]], [[8.0, 1.0, 6.0], [3.0, 5.0, 7.0], [8.0, 4.0, 8.0]], 
[[1.0, 7.0, 7.0], [1.0, 8.0, 6.0], [3.0, 4.0, 4.0]]], [[[4.0, 6.0, 2.0], [3.0, 1.0, 6.0], [1.0, 1.0, 7.0]], [[3.0, 2.0, 2.0], 
[6.0, 4.0, 6.0], [2.0, 2.0, 4.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 6.0], [1.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 4.0], [8.0, 1.0, 3.0], [3.0, 5.0, 4.0]], [[6.0, 2.0, 4.0], [4.0, 5.0, 1.0], [6.0, 
7.0, 5.0]], [[2.0, 1.0, 3.0], [4.0, 5.0, 2.0], [5.0, 8.0, 5.0]]], [[[8.0, 4.0, 8.0], [4.0, 8.0, 8.0], [6.0, 4.0, 8.0]], [[1.0, 
2.0, 2.0], [6.0, 2.0, 5.0], [4.0, 7.0, 7.0]], [[1.0, 4.0, 7.0], [3.0, 4.0, 8.0], [8.0, 4.0, 2.0]]], [[[4.0, 7.0, 5.0], [8.0, 
8.0, 8.0], [2.0, 7.0, 7.0]], [[1.0, 5.0, 3.0], [1.0, 5.0, 7.0], [1.0, 3.0, 8.0]], [[4.0, 7.0, 5.0], [1.0, 4.0, 1.0], [4.0, 6.0, 
7.0]]], [[[8.0, 3.0, 6.0], [2.0, 5.0, 8.0], [4.0, 2.0, 1.0]], [[6.0, 6.0, 6.0], [2.0, 7.0, 2.0], [2.0, 6.0, 4.0]], [[3.0, 4.0, 
4.0], [1.0, 8.0, 8.0], [4.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 3.0], [3.0, 2.0, 8.0], [8.0, 5.0, 8.0]], 
[[4.0, 3.0, 5.0], [8.0, 4.0, 3.0], [8.0, 3.0, 8.0]], [[8.0, 3.0, 3.0], [4.0, 2.0, 1.0], [8.0, 8.0, 3.0]]], [[[3.0, 1.0, 8.0], 
[2.0, 1.0, 5.0], [8.0, 7.0, 5.0]], [[1.0, 6.0, 6.0], [4.0, 7.0, 8.0], [8.0, 8.0, 7.0]], [[7.0, 7.0, 5.0], [4.0, 3.0, 3.0], 
[1.0, 8.0, 4.0]]], [[[8.0, 2.0, 1.0], [3.0, 4.0, 8.0], [6.0, 5.0, 7.0]], [[4.0, 5.0, 3.0], [6.0, 4.0, 2.0], [6.0, 4.0, 2.0]], 
[[6.0, 8.0, 2.0], [8.0, 7.0, 1.0], [6.0, 1.0, 8.0]]], [[[8.0, 4.0, 4.0], [5.0, 3.0, 5.0], [7.0, 1.0, 8.0]], [[8.0, 7.0, 7.0], 
[5.0, 8.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 7.0, 4.0], [6.0, 1.0, 6.0], [4.0, 6.0, 4.0]]]])
      val=Data(numpy.array([[[[7.0, 3.0, 6.0]]], [[[5.0, 4.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 8.0, 4.0]]], [[[5.0, 1.0, 3.0]]]]))
      arg[2:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 4.0], [8.0, 1.0, 3.0], [3.0, 5.0, 4.0]], [[6.0, 2.0, 4.0], [4.0, 5.0, 1.0], [6.0, 
7.0, 5.0]], [[2.0, 1.0, 3.0], [4.0, 5.0, 2.0], [5.0, 8.0, 5.0]]], [[[8.0, 4.0, 8.0], [4.0, 8.0, 8.0], [6.0, 4.0, 8.0]], [[1.0, 
2.0, 2.0], [6.0, 2.0, 5.0], [4.0, 7.0, 7.0]], [[1.0, 4.0, 7.0], [3.0, 4.0, 8.0], [8.0, 4.0, 2.0]]], [[[4.0, 7.0, 5.0], [8.0, 
8.0, 8.0], [2.0, 7.0, 7.0]], [[1.0, 5.0, 3.0], [7.0, 3.0, 6.0], [1.0, 3.0, 8.0]], [[4.0, 7.0, 5.0], [1.0, 4.0, 1.0], [4.0, 6.0, 
7.0]]], [[[8.0, 3.0, 6.0], [2.0, 5.0, 8.0], [4.0, 2.0, 1.0]], [[6.0, 6.0, 6.0], [5.0, 4.0, 1.0], [2.0, 6.0, 4.0]], [[3.0, 4.0, 
4.0], [1.0, 8.0, 8.0], [4.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 3.0], [3.0, 2.0, 8.0], [8.0, 5.0, 8.0]], 
[[4.0, 3.0, 5.0], [8.0, 4.0, 3.0], [8.0, 3.0, 8.0]], [[8.0, 3.0, 3.0], [4.0, 2.0, 1.0], [8.0, 8.0, 3.0]]], [[[3.0, 1.0, 8.0], 
[2.0, 1.0, 5.0], [8.0, 7.0, 5.0]], [[1.0, 6.0, 6.0], [4.0, 7.0, 8.0], [8.0, 8.0, 7.0]], [[7.0, 7.0, 5.0], [4.0, 3.0, 3.0], 
[1.0, 8.0, 4.0]]], [[[8.0, 2.0, 1.0], [3.0, 4.0, 8.0], [6.0, 5.0, 7.0]], [[4.0, 5.0, 3.0], [2.0, 8.0, 4.0], [6.0, 4.0, 2.0]], 
[[6.0, 8.0, 2.0], [8.0, 7.0, 1.0], [6.0, 1.0, 8.0]]], [[[8.0, 4.0, 4.0], [5.0, 3.0, 5.0], [7.0, 1.0, 8.0]], [[8.0, 7.0, 7.0], 
[5.0, 1.0, 3.0], [6.0, 5.0, 6.0]], [[2.0, 7.0, 4.0], [6.0, 1.0, 6.0], [4.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PLUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 3.0], [2.0, 3.0, 6.0], [5.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [7.0, 6.0, 1.0], [3.0, 
5.0, 8.0]], [[5.0, 4.0, 8.0], [6.0, 4.0, 3.0], [7.0, 4.0, 4.0]]], [[[1.0, 4.0, 7.0], [5.0, 2.0, 1.0], [3.0, 2.0, 1.0]], [[1.0, 
6.0, 4.0], [1.0, 4.0, 6.0], [3.0, 8.0, 5.0]], [[8.0, 5.0, 1.0], [5.0, 4.0, 3.0], [3.0, 6.0, 4.0]]], [[[2.0, 7.0, 1.0], [2.0, 
8.0, 8.0], [7.0, 8.0, 2.0]], [[6.0, 1.0, 6.0], [4.0, 6.0, 6.0], [8.0, 2.0, 6.0]], [[6.0, 7.0, 2.0], [4.0, 5.0, 3.0], [5.0, 4.0, 
3.0]]], [[[7.0, 3.0, 2.0], [6.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[8.0, 7.0, 1.0], [7.0, 2.0, 6.0], [7.0, 8.0, 8.0]], [[1.0, 1.0, 
1.0], [3.0, 8.0, 3.0], [8.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 7.0], [6.0, 3.0, 3.0], [4.0, 2.0, 5.0]], 
[[2.0, 8.0, 4.0], [6.0, 7.0, 8.0], [8.0, 3.0, 1.0]], [[2.0, 3.0, 5.0], [8.0, 1.0, 1.0], [6.0, 6.0, 2.0]]], [[[2.0, 1.0, 3.0], 
[2.0, 6.0, 6.0], [2.0, 4.0, 3.0]], [[7.0, 5.0, 7.0], [1.0, 8.0, 5.0], [5.0, 3.0, 4.0]], [[8.0, 1.0, 2.0], [5.0, 7.0, 4.0], 
[6.0, 1.0, 2.0]]], [[[3.0, 1.0, 5.0], [7.0, 2.0, 2.0], [1.0, 8.0, 1.0]], [[6.0, 6.0, 6.0], [2.0, 3.0, 6.0], [3.0, 6.0, 3.0]], 
[[8.0, 8.0, 2.0], [3.0, 2.0, 4.0], [8.0, 2.0, 4.0]]], [[[1.0, 7.0, 1.0], [4.0, 2.0, 7.0], [7.0, 6.0, 3.0]], [[4.0, 8.0, 4.0], 
[1.0, 2.0, 5.0], [3.0, 5.0, 4.0]], [[6.0, 6.0, 6.0], [2.0, 5.0, 4.0], [3.0, 1.0, 7.0]]]])
      val=Data(numpy.array([[[1.0, 2.0, 8.0], [2.0, 2.0, 5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 7.0, 4.0], [5.0, 2.0, 8.0]]]))
      arg[2,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 3.0], [2.0, 3.0, 6.0], [5.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [7.0, 6.0, 1.0], [3.0, 
5.0, 8.0]], [[5.0, 4.0, 8.0], [6.0, 4.0, 3.0], [7.0, 4.0, 4.0]]], [[[1.0, 4.0, 7.0], [5.0, 2.0, 1.0], [3.0, 2.0, 1.0]], [[1.0, 
6.0, 4.0], [1.0, 4.0, 6.0], [3.0, 8.0, 5.0]], [[8.0, 5.0, 1.0], [5.0, 4.0, 3.0], [3.0, 6.0, 4.0]]], [[[2.0, 7.0, 1.0], [2.0, 
8.0, 8.0], [7.0, 8.0, 2.0]], [[6.0, 1.0, 6.0], [4.0, 6.0, 6.0], [8.0, 2.0, 6.0]], [[1.0, 2.0, 8.0], [2.0, 2.0, 5.0], [5.0, 4.0, 
3.0]]], [[[7.0, 3.0, 2.0], [6.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[8.0, 7.0, 1.0], [7.0, 2.0, 6.0], [7.0, 8.0, 8.0]], [[1.0, 1.0, 
1.0], [3.0, 8.0, 3.0], [8.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 7.0], [6.0, 3.0, 3.0], [4.0, 2.0, 5.0]], 
[[2.0, 8.0, 4.0], [6.0, 7.0, 8.0], [8.0, 3.0, 1.0]], [[2.0, 3.0, 5.0], [8.0, 1.0, 1.0], [6.0, 6.0, 2.0]]], [[[2.0, 1.0, 3.0], 
[2.0, 6.0, 6.0], [2.0, 4.0, 3.0]], [[7.0, 5.0, 7.0], [1.0, 8.0, 5.0], [5.0, 3.0, 4.0]], [[8.0, 1.0, 2.0], [5.0, 7.0, 4.0], 
[6.0, 1.0, 2.0]]], [[[3.0, 1.0, 5.0], [7.0, 2.0, 2.0], [1.0, 8.0, 1.0]], [[6.0, 6.0, 6.0], [2.0, 3.0, 6.0], [3.0, 6.0, 3.0]], 
[[8.0, 7.0, 4.0], [5.0, 2.0, 8.0], [8.0, 2.0, 4.0]]], [[[1.0, 7.0, 1.0], [4.0, 2.0, 7.0], [7.0, 6.0, 3.0]], [[4.0, 8.0, 4.0], 
[1.0, 2.0, 5.0], [3.0, 5.0, 4.0]], [[6.0, 6.0, 6.0], [2.0, 5.0, 4.0], [3.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RLRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 2.0], [6.0, 6.0, 6.0], [1.0, 7.0, 6.0]], [[2.0, 8.0, 5.0], [7.0, 1.0, 1.0], [2.0, 
7.0, 1.0]], [[3.0, 7.0, 4.0], [3.0, 7.0, 4.0], [1.0, 4.0, 4.0]]], [[[8.0, 4.0, 8.0], [7.0, 6.0, 1.0], [7.0, 7.0, 4.0]], [[5.0, 
6.0, 8.0], [3.0, 2.0, 6.0], [6.0, 7.0, 6.0]], [[7.0, 1.0, 2.0], [1.0, 5.0, 6.0], [5.0, 5.0, 3.0]]], [[[7.0, 2.0, 8.0], [1.0, 
6.0, 3.0], [3.0, 3.0, 4.0]], [[7.0, 8.0, 5.0], [6.0, 5.0, 4.0], [3.0, 8.0, 5.0]], [[6.0, 7.0, 3.0], [2.0, 6.0, 6.0], [7.0, 7.0, 
1.0]]], [[[5.0, 3.0, 8.0], [4.0, 5.0, 8.0], [7.0, 5.0, 2.0]], [[4.0, 1.0, 2.0], [1.0, 7.0, 2.0], [2.0, 5.0, 8.0]], [[2.0, 6.0, 
6.0], [3.0, 4.0, 8.0], [6.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 7.0], [6.0, 5.0, 8.0], [3.0, 4.0, 5.0]], 
[[5.0, 8.0, 7.0], [5.0, 2.0, 1.0], [6.0, 3.0, 4.0]], [[2.0, 2.0, 5.0], [6.0, 8.0, 7.0], [1.0, 5.0, 5.0]]], [[[2.0, 8.0, 6.0], 
[8.0, 6.0, 4.0], [6.0, 3.0, 2.0]], [[7.0, 6.0, 8.0], [3.0, 2.0, 1.0], [5.0, 2.0, 5.0]], [[6.0, 5.0, 8.0], [4.0, 3.0, 1.0], 
[1.0, 8.0, 8.0]]], [[[1.0, 1.0, 8.0], [8.0, 5.0, 2.0], [1.0, 8.0, 5.0]], [[1.0, 4.0, 8.0], [4.0, 5.0, 3.0], [5.0, 3.0, 7.0]], 
[[1.0, 3.0, 2.0], [2.0, 1.0, 5.0], [6.0, 1.0, 6.0]]], [[[1.0, 6.0, 7.0], [2.0, 7.0, 4.0], [1.0, 1.0, 4.0]], [[6.0, 7.0, 8.0], 
[2.0, 3.0, 7.0], [6.0, 5.0, 2.0]], [[4.0, 6.0, 3.0], [7.0, 5.0, 3.0], [5.0, 2.0, 4.0]]]])
      val=Data(numpy.array([[[[6.0, 8.0, 4.0]]], [[[8.0, 3.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 4.0]]], [[[2.0, 7.0, 3.0]]]]))
      arg[1:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 2.0], [6.0, 6.0, 6.0], [1.0, 7.0, 6.0]], [[2.0, 8.0, 5.0], [7.0, 1.0, 1.0], [2.0, 
7.0, 1.0]], [[3.0, 7.0, 4.0], [3.0, 7.0, 4.0], [1.0, 4.0, 4.0]]], [[[8.0, 4.0, 8.0], [7.0, 6.0, 1.0], [7.0, 7.0, 4.0]], [[5.0, 
6.0, 8.0], [3.0, 2.0, 6.0], [6.0, 7.0, 6.0]], [[7.0, 1.0, 2.0], [6.0, 8.0, 4.0], [5.0, 5.0, 3.0]]], [[[7.0, 2.0, 8.0], [1.0, 
6.0, 3.0], [3.0, 3.0, 4.0]], [[7.0, 8.0, 5.0], [6.0, 5.0, 4.0], [3.0, 8.0, 5.0]], [[6.0, 7.0, 3.0], [8.0, 3.0, 5.0], [7.0, 7.0, 
1.0]]], [[[5.0, 3.0, 8.0], [4.0, 5.0, 8.0], [7.0, 5.0, 2.0]], [[4.0, 1.0, 2.0], [1.0, 7.0, 2.0], [2.0, 5.0, 8.0]], [[2.0, 6.0, 
6.0], [3.0, 4.0, 8.0], [6.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 7.0], [6.0, 5.0, 8.0], [3.0, 4.0, 5.0]], 
[[5.0, 8.0, 7.0], [5.0, 2.0, 1.0], [6.0, 3.0, 4.0]], [[2.0, 2.0, 5.0], [6.0, 8.0, 7.0], [1.0, 5.0, 5.0]]], [[[2.0, 8.0, 6.0], 
[8.0, 6.0, 4.0], [6.0, 3.0, 2.0]], [[7.0, 6.0, 8.0], [3.0, 2.0, 1.0], [5.0, 2.0, 5.0]], [[6.0, 5.0, 8.0], [7.0, 6.0, 4.0], 
[1.0, 8.0, 8.0]]], [[[1.0, 1.0, 8.0], [8.0, 5.0, 2.0], [1.0, 8.0, 5.0]], [[1.0, 4.0, 8.0], [4.0, 5.0, 3.0], [5.0, 3.0, 7.0]], 
[[1.0, 3.0, 2.0], [2.0, 7.0, 3.0], [6.0, 1.0, 6.0]]], [[[1.0, 6.0, 7.0], [2.0, 7.0, 4.0], [1.0, 1.0, 4.0]], [[6.0, 7.0, 8.0], 
[2.0, 3.0, 7.0], [6.0, 5.0, 2.0]], [[4.0, 6.0, 3.0], [7.0, 5.0, 3.0], [5.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UUCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 1.0], [3.0, 6.0, 5.0], [2.0, 8.0, 2.0]], [[2.0, 6.0, 2.0], [3.0, 8.0, 4.0], [7.0, 
6.0, 5.0]], [[4.0, 5.0, 7.0], [5.0, 3.0, 8.0], [6.0, 5.0, 6.0]]], [[[3.0, 6.0, 1.0], [7.0, 6.0, 3.0], [5.0, 7.0, 2.0]], [[5.0, 
4.0, 8.0], [8.0, 7.0, 8.0], [4.0, 5.0, 2.0]], [[8.0, 7.0, 2.0], [8.0, 2.0, 3.0], [5.0, 6.0, 6.0]]], [[[6.0, 3.0, 2.0], [8.0, 
6.0, 8.0], [8.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [5.0, 5.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 8.0, 3.0], [8.0, 3.0, 2.0], [7.0, 3.0, 
7.0]]], [[[3.0, 2.0, 5.0], [4.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[6.0, 7.0, 3.0], [4.0, 1.0, 6.0], [8.0, 1.0, 7.0]], [[1.0, 4.0, 
3.0], [8.0, 5.0, 8.0], [5.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 5.0], [6.0, 3.0, 3.0], [5.0, 3.0, 6.0]], 
[[6.0, 3.0, 2.0], [1.0, 2.0, 1.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 5.0], [6.0, 1.0, 2.0], [6.0, 3.0, 3.0]]], [[[3.0, 5.0, 2.0], 
[3.0, 4.0, 6.0], [3.0, 5.0, 3.0]], [[8.0, 4.0, 8.0], [2.0, 8.0, 7.0], [5.0, 7.0, 1.0]], [[4.0, 3.0, 6.0], [3.0, 3.0, 5.0], 
[7.0, 5.0, 3.0]]], [[[5.0, 2.0, 6.0], [4.0, 6.0, 6.0], [6.0, 5.0, 8.0]], [[7.0, 5.0, 2.0], [2.0, 7.0, 5.0], [5.0, 8.0, 2.0]], 
[[2.0, 2.0, 2.0], [5.0, 2.0, 3.0], [3.0, 3.0, 6.0]]], [[[6.0, 7.0, 5.0], [2.0, 2.0, 1.0], [8.0, 4.0, 3.0]], [[5.0, 1.0, 1.0], 
[8.0, 2.0, 1.0], [4.0, 8.0, 5.0]], [[8.0, 5.0, 3.0], [1.0, 2.0, 7.0], [4.0, 6.0, 6.0]]]])
      val=Data(numpy.array([[[[6.0, 3.0, 6.0], [3.0, 7.0, 5.0], [4.0, 5.0, 2.0]], [[7.0, 8.0, 5.0], [3.0, 1.0, 3.0], [2.0, 3.0, 
7.0]]], [[[5.0, 1.0, 1.0], [2.0, 5.0, 3.0], [4.0, 6.0, 2.0]], [[1.0, 3.0, 1.0], [3.0, 8.0, 1.0], [3.0, 6.0, 1.0]]], [[[7.0, 
3.0, 2.0], [1.0, 8.0, 8.0], [6.0, 8.0, 8.0]], [[3.0, 2.0, 6.0], [1.0, 4.0, 3.0], [7.0, 8.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 4.0, 2.0], [5.0, 3.0, 6.0], [6.0, 3.0, 5.0]], [[8.0, 5.0, 5.0], [4.0, 2.0, 
5.0], [5.0, 6.0, 8.0]]], [[[7.0, 8.0, 7.0], [2.0, 8.0, 6.0], [5.0, 2.0, 2.0]], [[6.0, 7.0, 8.0], [3.0, 6.0, 7.0], [4.0, 1.0, 
3.0]]], [[[1.0, 7.0, 8.0], [4.0, 7.0, 6.0], [4.0, 7.0, 7.0]], [[5.0, 4.0, 4.0], [4.0, 1.0, 2.0], [1.0, 3.0, 1.0]]]]))
      arg[:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 6.0], [3.0, 7.0, 5.0], [4.0, 5.0, 2.0]], [[7.0, 8.0, 5.0], [3.0, 1.0, 3.0], [2.0, 
3.0, 7.0]], [[4.0, 5.0, 7.0], [5.0, 3.0, 8.0], [6.0, 5.0, 6.0]]], [[[5.0, 1.0, 1.0], [2.0, 5.0, 3.0], [4.0, 6.0, 2.0]], [[1.0, 
3.0, 1.0], [3.0, 8.0, 1.0], [3.0, 6.0, 1.0]], [[8.0, 7.0, 2.0], [8.0, 2.0, 3.0], [5.0, 6.0, 6.0]]], [[[7.0, 3.0, 2.0], [1.0, 
8.0, 8.0], [6.0, 8.0, 8.0]], [[3.0, 2.0, 6.0], [1.0, 4.0, 3.0], [7.0, 8.0, 3.0]], [[3.0, 8.0, 3.0], [8.0, 3.0, 2.0], [7.0, 3.0, 
7.0]]], [[[3.0, 2.0, 5.0], [4.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[6.0, 7.0, 3.0], [4.0, 1.0, 6.0], [8.0, 1.0, 7.0]], [[1.0, 4.0, 
3.0], [8.0, 5.0, 8.0], [5.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 2.0], [5.0, 3.0, 6.0], [6.0, 3.0, 5.0]], 
[[8.0, 5.0, 5.0], [4.0, 2.0, 5.0], [5.0, 6.0, 8.0]], [[3.0, 5.0, 5.0], [6.0, 1.0, 2.0], [6.0, 3.0, 3.0]]], [[[7.0, 8.0, 7.0], 
[2.0, 8.0, 6.0], [5.0, 2.0, 2.0]], [[6.0, 7.0, 8.0], [3.0, 6.0, 7.0], [4.0, 1.0, 3.0]], [[4.0, 3.0, 6.0], [3.0, 3.0, 5.0], 
[7.0, 5.0, 3.0]]], [[[1.0, 7.0, 8.0], [4.0, 7.0, 6.0], [4.0, 7.0, 7.0]], [[5.0, 4.0, 4.0], [4.0, 1.0, 2.0], [1.0, 3.0, 1.0]], 
[[2.0, 2.0, 2.0], [5.0, 2.0, 3.0], [3.0, 3.0, 6.0]]], [[[6.0, 7.0, 5.0], [2.0, 2.0, 1.0], [8.0, 4.0, 3.0]], [[5.0, 1.0, 1.0], 
[8.0, 2.0, 1.0], [4.0, 8.0, 5.0]], [[8.0, 5.0, 3.0], [1.0, 2.0, 7.0], [4.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UUUU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 1.0], [5.0, 5.0, 6.0], [8.0, 1.0, 7.0]], [[1.0, 4.0, 2.0], [5.0, 8.0, 5.0], [8.0, 
2.0, 2.0]], [[4.0, 6.0, 6.0], [1.0, 1.0, 8.0], [3.0, 6.0, 1.0]]], [[[1.0, 4.0, 4.0], [8.0, 6.0, 5.0], [3.0, 4.0, 4.0]], [[6.0, 
2.0, 8.0], [1.0, 6.0, 5.0], [6.0, 4.0, 8.0]], [[8.0, 7.0, 7.0], [2.0, 5.0, 8.0], [6.0, 3.0, 5.0]]], [[[2.0, 5.0, 3.0], [1.0, 
8.0, 5.0], [2.0, 2.0, 3.0]], [[7.0, 2.0, 5.0], [4.0, 3.0, 5.0], [2.0, 3.0, 2.0]], [[5.0, 8.0, 4.0], [8.0, 3.0, 3.0], [5.0, 5.0, 
1.0]]], [[[2.0, 1.0, 5.0], [7.0, 5.0, 3.0], [5.0, 1.0, 1.0]], [[6.0, 2.0, 1.0], [8.0, 6.0, 6.0], [8.0, 3.0, 3.0]], [[4.0, 1.0, 
6.0], [8.0, 1.0, 3.0], [8.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 3.0, 7.0], [1.0, 8.0, 7.0], [2.0, 8.0, 2.0]], 
[[4.0, 2.0, 7.0], [7.0, 6.0, 3.0], [1.0, 7.0, 1.0]], [[8.0, 8.0, 6.0], [5.0, 3.0, 2.0], [6.0, 5.0, 2.0]]], [[[4.0, 3.0, 8.0], 
[5.0, 1.0, 4.0], [7.0, 5.0, 7.0]], [[7.0, 8.0, 2.0], [2.0, 8.0, 7.0], [8.0, 5.0, 2.0]], [[2.0, 3.0, 2.0], [2.0, 2.0, 1.0], 
[7.0, 7.0, 5.0]]], [[[2.0, 6.0, 3.0], [1.0, 2.0, 1.0], [3.0, 6.0, 8.0]], [[2.0, 3.0, 6.0], [4.0, 3.0, 5.0], [2.0, 6.0, 2.0]], 
[[2.0, 8.0, 1.0], [6.0, 2.0, 4.0], [8.0, 1.0, 4.0]]], [[[7.0, 5.0, 4.0], [7.0, 2.0, 8.0], [6.0, 8.0, 6.0]], [[4.0, 6.0, 5.0], 
[2.0, 7.0, 1.0], [7.0, 6.0, 2.0]], [[3.0, 7.0, 5.0], [5.0, 8.0, 2.0], [4.0, 4.0, 5.0]]]])
      val=Data(numpy.array([[[[7.0, 7.0], [1.0, 3.0]], [[4.0, 7.0], [6.0, 8.0]]], [[[2.0, 1.0], [8.0, 6.0]], [[5.0, 8.0], [8.0, 
2.0]]], [[[5.0, 1.0], [6.0, 2.0]], [[3.0, 7.0], [1.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 7.0], [4.0, 8.0]], [[8.0, 2.0], [2.0, 5.0]]], [[[7.0, 1.0], [6.0, 5.0]], [[3.0, 
8.0], [6.0, 3.0]]], [[[3.0, 4.0], [5.0, 6.0]], [[3.0, 5.0], [1.0, 2.0]]]]))
      arg[:3,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 1.0], [1.0, 3.0, 6.0], [8.0, 1.0, 7.0]], [[4.0, 7.0, 2.0], [6.0, 8.0, 5.0], [8.0, 
2.0, 2.0]], [[4.0, 6.0, 6.0], [1.0, 1.0, 8.0], [3.0, 6.0, 1.0]]], [[[2.0, 1.0, 4.0], [8.0, 6.0, 5.0], [3.0, 4.0, 4.0]], [[5.0, 
8.0, 8.0], [8.0, 2.0, 5.0], [6.0, 4.0, 8.0]], [[8.0, 7.0, 7.0], [2.0, 5.0, 8.0], [6.0, 3.0, 5.0]]], [[[5.0, 1.0, 3.0], [6.0, 
2.0, 5.0], [2.0, 2.0, 3.0]], [[3.0, 7.0, 5.0], [1.0, 5.0, 5.0], [2.0, 3.0, 2.0]], [[5.0, 8.0, 4.0], [8.0, 3.0, 3.0], [5.0, 5.0, 
1.0]]], [[[2.0, 1.0, 5.0], [7.0, 5.0, 3.0], [5.0, 1.0, 1.0]], [[6.0, 2.0, 1.0], [8.0, 6.0, 6.0], [8.0, 3.0, 3.0]], [[4.0, 1.0, 
6.0], [8.0, 1.0, 3.0], [8.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 7.0], [4.0, 8.0, 7.0], [2.0, 8.0, 2.0]], 
[[8.0, 2.0, 7.0], [2.0, 5.0, 3.0], [1.0, 7.0, 1.0]], [[8.0, 8.0, 6.0], [5.0, 3.0, 2.0], [6.0, 5.0, 2.0]]], [[[7.0, 1.0, 8.0], 
[6.0, 5.0, 4.0], [7.0, 5.0, 7.0]], [[3.0, 8.0, 2.0], [6.0, 3.0, 7.0], [8.0, 5.0, 2.0]], [[2.0, 3.0, 2.0], [2.0, 2.0, 1.0], 
[7.0, 7.0, 5.0]]], [[[3.0, 4.0, 3.0], [5.0, 6.0, 1.0], [3.0, 6.0, 8.0]], [[3.0, 5.0, 6.0], [1.0, 2.0, 5.0], [2.0, 6.0, 2.0]], 
[[2.0, 8.0, 1.0], [6.0, 2.0, 4.0], [8.0, 1.0, 4.0]]], [[[7.0, 5.0, 4.0], [7.0, 2.0, 8.0], [6.0, 8.0, 6.0]], [[4.0, 6.0, 5.0], 
[2.0, 7.0, 1.0], [7.0, 6.0, 2.0]], [[3.0, 7.0, 5.0], [5.0, 8.0, 2.0], [4.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PRCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 4.0], [7.0, 7.0, 5.0], [7.0, 8.0, 7.0]], [[4.0, 3.0, 3.0], [7.0, 4.0, 7.0], [1.0, 
4.0, 7.0]], [[1.0, 4.0, 8.0], [3.0, 4.0, 6.0], [1.0, 1.0, 7.0]]], [[[3.0, 8.0, 1.0], [5.0, 7.0, 3.0], [3.0, 8.0, 6.0]], [[7.0, 
4.0, 8.0], [3.0, 1.0, 2.0], [7.0, 2.0, 1.0]], [[8.0, 6.0, 3.0], [5.0, 5.0, 6.0], [6.0, 3.0, 1.0]]], [[[2.0, 6.0, 1.0], [7.0, 
5.0, 4.0], [4.0, 1.0, 5.0]], [[1.0, 2.0, 5.0], [7.0, 8.0, 3.0], [7.0, 3.0, 2.0]], [[8.0, 4.0, 2.0], [7.0, 6.0, 5.0], [7.0, 3.0, 
5.0]]], [[[6.0, 1.0, 8.0], [4.0, 1.0, 5.0], [8.0, 1.0, 6.0]], [[2.0, 8.0, 8.0], [8.0, 5.0, 3.0], [3.0, 7.0, 1.0]], [[8.0, 5.0, 
8.0], [8.0, 5.0, 1.0], [8.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 4.0], [1.0, 5.0, 8.0], [2.0, 5.0, 6.0]], 
[[5.0, 4.0, 3.0], [2.0, 2.0, 4.0], [2.0, 2.0, 2.0]], [[3.0, 6.0, 6.0], [2.0, 4.0, 4.0], [6.0, 8.0, 3.0]]], [[[7.0, 8.0, 4.0], 
[7.0, 2.0, 5.0], [3.0, 2.0, 6.0]], [[1.0, 7.0, 8.0], [4.0, 3.0, 4.0], [2.0, 7.0, 4.0]], [[4.0, 6.0, 3.0], [2.0, 5.0, 3.0], 
[5.0, 2.0, 8.0]]], [[[5.0, 8.0, 5.0], [1.0, 2.0, 1.0], [3.0, 2.0, 4.0]], [[5.0, 4.0, 3.0], [2.0, 1.0, 7.0], [6.0, 5.0, 4.0]], 
[[7.0, 4.0, 8.0], [2.0, 3.0, 2.0], [1.0, 3.0, 8.0]]], [[[1.0, 6.0, 6.0], [5.0, 3.0, 6.0], [3.0, 8.0, 3.0]], [[7.0, 7.0, 8.0], 
[3.0, 7.0, 7.0], [4.0, 8.0, 5.0]], [[3.0, 2.0, 3.0], [2.0, 4.0, 3.0], [3.0, 7.0, 8.0]]]])
      val=Data(numpy.array([[[6.0, 5.0, 8.0], [2.0, 3.0, 3.0], [7.0, 4.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 7.0, 1.0], [7.0, 8.0, 6.0], [2.0, 5.0, 1.0]]]))
      arg[2,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 4.0], [7.0, 7.0, 5.0], [7.0, 8.0, 7.0]], [[4.0, 3.0, 3.0], [7.0, 4.0, 7.0], [1.0, 
4.0, 7.0]], [[1.0, 4.0, 8.0], [3.0, 4.0, 6.0], [1.0, 1.0, 7.0]]], [[[3.0, 8.0, 1.0], [5.0, 7.0, 3.0], [3.0, 8.0, 6.0]], [[7.0, 
4.0, 8.0], [3.0, 1.0, 2.0], [7.0, 2.0, 1.0]], [[8.0, 6.0, 3.0], [5.0, 5.0, 6.0], [6.0, 3.0, 1.0]]], [[[2.0, 6.0, 1.0], [7.0, 
5.0, 4.0], [4.0, 1.0, 5.0]], [[6.0, 5.0, 8.0], [2.0, 3.0, 3.0], [7.0, 4.0, 1.0]], [[8.0, 4.0, 2.0], [7.0, 6.0, 5.0], [7.0, 3.0, 
5.0]]], [[[6.0, 1.0, 8.0], [4.0, 1.0, 5.0], [8.0, 1.0, 6.0]], [[2.0, 8.0, 8.0], [8.0, 5.0, 3.0], [3.0, 7.0, 1.0]], [[8.0, 5.0, 
8.0], [8.0, 5.0, 1.0], [8.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 4.0], [1.0, 5.0, 8.0], [2.0, 5.0, 6.0]], 
[[5.0, 4.0, 3.0], [2.0, 2.0, 4.0], [2.0, 2.0, 2.0]], [[3.0, 6.0, 6.0], [2.0, 4.0, 4.0], [6.0, 8.0, 3.0]]], [[[7.0, 8.0, 4.0], 
[7.0, 2.0, 5.0], [3.0, 2.0, 6.0]], [[1.0, 7.0, 8.0], [4.0, 3.0, 4.0], [2.0, 7.0, 4.0]], [[4.0, 6.0, 3.0], [2.0, 5.0, 3.0], 
[5.0, 2.0, 8.0]]], [[[5.0, 8.0, 5.0], [1.0, 2.0, 1.0], [3.0, 2.0, 4.0]], [[7.0, 7.0, 1.0], [7.0, 8.0, 6.0], [2.0, 5.0, 1.0]], 
[[7.0, 4.0, 8.0], [2.0, 3.0, 2.0], [1.0, 3.0, 8.0]]], [[[1.0, 6.0, 6.0], [5.0, 3.0, 6.0], [3.0, 8.0, 3.0]], [[7.0, 7.0, 8.0], 
[3.0, 7.0, 7.0], [4.0, 8.0, 5.0]], [[3.0, 2.0, 3.0], [2.0, 4.0, 3.0], [3.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CULN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 2.0], [5.0, 8.0, 4.0], [1.0, 3.0, 1.0]], [[3.0, 1.0, 2.0], [6.0, 4.0, 3.0], [1.0, 
4.0, 1.0]], [[6.0, 1.0, 3.0], [3.0, 8.0, 1.0], [1.0, 4.0, 1.0]]], [[[6.0, 4.0, 6.0], [7.0, 4.0, 4.0], [7.0, 2.0, 2.0]], [[7.0, 
7.0, 8.0], [8.0, 7.0, 2.0], [8.0, 8.0, 3.0]], [[6.0, 5.0, 4.0], [1.0, 1.0, 8.0], [1.0, 6.0, 3.0]]], [[[3.0, 6.0, 8.0], [6.0, 
3.0, 7.0], [5.0, 1.0, 6.0]], [[2.0, 7.0, 2.0], [8.0, 4.0, 1.0], [3.0, 5.0, 4.0]], [[6.0, 5.0, 2.0], [5.0, 7.0, 1.0], [4.0, 7.0, 
4.0]]], [[[3.0, 3.0, 2.0], [5.0, 2.0, 4.0], [8.0, 3.0, 1.0]], [[1.0, 6.0, 3.0], [8.0, 2.0, 1.0], [1.0, 6.0, 8.0]], [[1.0, 8.0, 
2.0], [5.0, 8.0, 7.0], [2.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 4.0], [7.0, 6.0, 7.0], [2.0, 8.0, 7.0]], 
[[4.0, 2.0, 4.0], [3.0, 5.0, 5.0], [2.0, 8.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 4.0, 1.0], [6.0, 1.0, 4.0]]], [[[3.0, 5.0, 7.0], 
[5.0, 8.0, 4.0], [1.0, 2.0, 6.0]], [[2.0, 2.0, 4.0], [2.0, 2.0, 3.0], [1.0, 8.0, 1.0]], [[6.0, 1.0, 7.0], [7.0, 7.0, 6.0], 
[3.0, 7.0, 3.0]]], [[[4.0, 1.0, 4.0], [5.0, 6.0, 2.0], [6.0, 5.0, 4.0]], [[5.0, 8.0, 1.0], [3.0, 6.0, 4.0], [1.0, 2.0, 8.0]], 
[[8.0, 6.0, 5.0], [8.0, 1.0, 6.0], [1.0, 1.0, 8.0]]], [[[4.0, 8.0, 2.0], [7.0, 7.0, 8.0], [6.0, 3.0, 8.0]], [[3.0, 6.0, 4.0], 
[2.0, 6.0, 8.0], [7.0, 4.0, 6.0]], [[5.0, 2.0, 5.0], [3.0, 3.0, 4.0], [3.0, 3.0, 2.0]]]])
      val=Data(numpy.array([[[[3.0, 4.0, 8.0]], [[3.0, 4.0, 7.0]]], [[[2.0, 1.0, 6.0]], [[4.0, 8.0, 4.0]]], [[[3.0, 1.0, 4.0]], 
[[3.0, 8.0, 8.0]]], [[[2.0, 1.0, 1.0]], [[2.0, 7.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 1.0, 3.0]], [[6.0, 7.0, 7.0]]], [[[3.0, 6.0, 4.0]], [[6.0, 6.0, 2.0]]], [[[2.0, 
1.0, 2.0]], [[1.0, 4.0, 7.0]]], [[[4.0, 8.0, 4.0]], [[8.0, 4.0, 7.0]]]]))
      arg[:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 2.0], [5.0, 8.0, 4.0], [3.0, 4.0, 8.0]], [[3.0, 1.0, 2.0], [6.0, 4.0, 3.0], [3.0, 
4.0, 7.0]], [[6.0, 1.0, 3.0], [3.0, 8.0, 1.0], [1.0, 4.0, 1.0]]], [[[6.0, 4.0, 6.0], [7.0, 4.0, 4.0], [2.0, 1.0, 6.0]], [[7.0, 
7.0, 8.0], [8.0, 7.0, 2.0], [4.0, 8.0, 4.0]], [[6.0, 5.0, 4.0], [1.0, 1.0, 8.0], [1.0, 6.0, 3.0]]], [[[3.0, 6.0, 8.0], [6.0, 
3.0, 7.0], [3.0, 1.0, 4.0]], [[2.0, 7.0, 2.0], [8.0, 4.0, 1.0], [3.0, 8.0, 8.0]], [[6.0, 5.0, 2.0], [5.0, 7.0, 1.0], [4.0, 7.0, 
4.0]]], [[[3.0, 3.0, 2.0], [5.0, 2.0, 4.0], [2.0, 1.0, 1.0]], [[1.0, 6.0, 3.0], [8.0, 2.0, 1.0], [2.0, 7.0, 2.0]], [[1.0, 8.0, 
2.0], [5.0, 8.0, 7.0], [2.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 4.0], [7.0, 6.0, 7.0], [2.0, 1.0, 3.0]], 
[[4.0, 2.0, 4.0], [3.0, 5.0, 5.0], [6.0, 7.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 4.0, 1.0], [6.0, 1.0, 4.0]]], [[[3.0, 5.0, 7.0], 
[5.0, 8.0, 4.0], [3.0, 6.0, 4.0]], [[2.0, 2.0, 4.0], [2.0, 2.0, 3.0], [6.0, 6.0, 2.0]], [[6.0, 1.0, 7.0], [7.0, 7.0, 6.0], 
[3.0, 7.0, 3.0]]], [[[4.0, 1.0, 4.0], [5.0, 6.0, 2.0], [2.0, 1.0, 2.0]], [[5.0, 8.0, 1.0], [3.0, 6.0, 4.0], [1.0, 4.0, 7.0]], 
[[8.0, 6.0, 5.0], [8.0, 1.0, 6.0], [1.0, 1.0, 8.0]]], [[[4.0, 8.0, 2.0], [7.0, 7.0, 8.0], [4.0, 8.0, 4.0]], [[3.0, 6.0, 4.0], 
[2.0, 6.0, 8.0], [8.0, 4.0, 7.0]], [[5.0, 2.0, 5.0], [3.0, 3.0, 4.0], [3.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 6.0], [8.0, 6.0, 2.0], [1.0, 3.0, 7.0]], [[1.0, 5.0, 4.0], [8.0, 8.0, 7.0], [7.0, 
2.0, 5.0]], [[3.0, 2.0, 5.0], [2.0, 4.0, 2.0], [6.0, 3.0, 1.0]]], [[[2.0, 2.0, 5.0], [8.0, 1.0, 2.0], [5.0, 1.0, 3.0]], [[8.0, 
1.0, 1.0], [3.0, 3.0, 7.0], [4.0, 4.0, 5.0]], [[2.0, 2.0, 3.0], [1.0, 2.0, 2.0], [4.0, 4.0, 4.0]]], [[[7.0, 8.0, 4.0], [7.0, 
5.0, 7.0], [5.0, 3.0, 7.0]], [[5.0, 7.0, 5.0], [4.0, 5.0, 4.0], [5.0, 4.0, 2.0]], [[2.0, 6.0, 2.0], [3.0, 6.0, 4.0], [6.0, 8.0, 
4.0]]], [[[7.0, 3.0, 8.0], [8.0, 3.0, 1.0], [7.0, 1.0, 3.0]], [[1.0, 8.0, 2.0], [3.0, 7.0, 4.0], [7.0, 1.0, 3.0]], [[1.0, 1.0, 
6.0], [4.0, 6.0, 4.0], [8.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 6.0], [7.0, 7.0, 6.0], [3.0, 6.0, 5.0]], 
[[4.0, 5.0, 3.0], [6.0, 7.0, 4.0], [5.0, 4.0, 8.0]], [[4.0, 3.0, 5.0], [3.0, 3.0, 8.0], [3.0, 6.0, 3.0]]], [[[5.0, 8.0, 1.0], 
[3.0, 2.0, 6.0], [3.0, 1.0, 4.0]], [[4.0, 2.0, 3.0], [7.0, 8.0, 8.0], [7.0, 6.0, 7.0]], [[4.0, 1.0, 7.0], [4.0, 1.0, 7.0], 
[2.0, 6.0, 4.0]]], [[[4.0, 1.0, 5.0], [1.0, 5.0, 3.0], [5.0, 3.0, 4.0]], [[2.0, 3.0, 4.0], [3.0, 2.0, 6.0], [7.0, 3.0, 3.0]], 
[[2.0, 3.0, 7.0], [8.0, 2.0, 8.0], [4.0, 8.0, 7.0]]], [[[6.0, 4.0, 8.0], [5.0, 8.0, 5.0], [8.0, 3.0, 4.0]], [[5.0, 2.0, 4.0], 
[8.0, 8.0, 1.0], [5.0, 8.0, 8.0]], [[1.0, 6.0, 1.0], [6.0, 7.0, 8.0], [1.0, 8.0, 4.0]]]])
      val=Data(numpy.array([[2.0, 5.0, 5.0], [8.0, 3.0, 8.0], [3.0, 6.0, 2.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[5.0, 2.0, 6.0], [6.0, 5.0, 3.0], [6.0, 2.0, 3.0]]))
      arg[2,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 6.0], [8.0, 6.0, 2.0], [1.0, 3.0, 7.0]], [[1.0, 5.0, 4.0], [8.0, 8.0, 7.0], [7.0, 
2.0, 5.0]], [[3.0, 2.0, 5.0], [2.0, 4.0, 2.0], [6.0, 3.0, 1.0]]], [[[2.0, 2.0, 5.0], [8.0, 1.0, 2.0], [5.0, 1.0, 3.0]], [[8.0, 
1.0, 1.0], [3.0, 3.0, 7.0], [4.0, 4.0, 5.0]], [[2.0, 2.0, 3.0], [1.0, 2.0, 2.0], [4.0, 4.0, 4.0]]], [[[7.0, 8.0, 4.0], [7.0, 
5.0, 7.0], [5.0, 3.0, 7.0]], [[2.0, 5.0, 5.0], [8.0, 3.0, 8.0], [3.0, 6.0, 2.0]], [[2.0, 6.0, 2.0], [3.0, 6.0, 4.0], [6.0, 8.0, 
4.0]]], [[[7.0, 3.0, 8.0], [8.0, 3.0, 1.0], [7.0, 1.0, 3.0]], [[1.0, 8.0, 2.0], [3.0, 7.0, 4.0], [7.0, 1.0, 3.0]], [[1.0, 1.0, 
6.0], [4.0, 6.0, 4.0], [8.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 6.0], [7.0, 7.0, 6.0], [3.0, 6.0, 5.0]], 
[[4.0, 5.0, 3.0], [6.0, 7.0, 4.0], [5.0, 4.0, 8.0]], [[4.0, 3.0, 5.0], [3.0, 3.0, 8.0], [3.0, 6.0, 3.0]]], [[[5.0, 8.0, 1.0], 
[3.0, 2.0, 6.0], [3.0, 1.0, 4.0]], [[4.0, 2.0, 3.0], [7.0, 8.0, 8.0], [7.0, 6.0, 7.0]], [[4.0, 1.0, 7.0], [4.0, 1.0, 7.0], 
[2.0, 6.0, 4.0]]], [[[4.0, 1.0, 5.0], [1.0, 5.0, 3.0], [5.0, 3.0, 4.0]], [[5.0, 2.0, 6.0], [6.0, 5.0, 3.0], [6.0, 2.0, 3.0]], 
[[2.0, 3.0, 7.0], [8.0, 2.0, 8.0], [4.0, 8.0, 7.0]]], [[[6.0, 4.0, 8.0], [5.0, 8.0, 5.0], [8.0, 3.0, 4.0]], [[5.0, 2.0, 4.0], 
[8.0, 8.0, 1.0], [5.0, 8.0, 8.0]], [[1.0, 6.0, 1.0], [6.0, 7.0, 8.0], [1.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CURN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 2.0], [5.0, 5.0, 1.0], [5.0, 4.0, 3.0]], [[4.0, 1.0, 7.0], [2.0, 4.0, 3.0], [6.0, 
2.0, 1.0]], [[6.0, 6.0, 2.0], [5.0, 1.0, 4.0], [2.0, 1.0, 6.0]]], [[[7.0, 8.0, 2.0], [6.0, 6.0, 6.0], [4.0, 6.0, 3.0]], [[7.0, 
2.0, 4.0], [2.0, 2.0, 5.0], [6.0, 1.0, 2.0]], [[7.0, 5.0, 7.0], [6.0, 6.0, 3.0], [8.0, 7.0, 8.0]]], [[[2.0, 1.0, 1.0], [2.0, 
7.0, 2.0], [4.0, 2.0, 7.0]], [[8.0, 7.0, 5.0], [5.0, 4.0, 8.0], [3.0, 2.0, 4.0]], [[6.0, 5.0, 1.0], [7.0, 8.0, 1.0], [4.0, 8.0, 
2.0]]], [[[2.0, 3.0, 8.0], [8.0, 2.0, 6.0], [2.0, 7.0, 7.0]], [[3.0, 2.0, 3.0], [2.0, 4.0, 2.0], [4.0, 3.0, 5.0]], [[5.0, 4.0, 
6.0], [2.0, 5.0, 6.0], [2.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 3.0], [2.0, 2.0, 3.0], [5.0, 3.0, 2.0]], 
[[2.0, 1.0, 2.0], [4.0, 2.0, 5.0], [3.0, 6.0, 1.0]], [[1.0, 8.0, 3.0], [8.0, 5.0, 1.0], [7.0, 2.0, 6.0]]], [[[4.0, 7.0, 7.0], 
[4.0, 6.0, 3.0], [7.0, 5.0, 1.0]], [[7.0, 3.0, 4.0], [1.0, 7.0, 1.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 2.0], [3.0, 4.0, 1.0], 
[8.0, 1.0, 6.0]]], [[[8.0, 4.0, 8.0], [2.0, 4.0, 3.0], [5.0, 4.0, 3.0]], [[4.0, 2.0, 2.0], [1.0, 3.0, 6.0], [4.0, 6.0, 5.0]], 
[[1.0, 6.0, 6.0], [8.0, 4.0, 2.0], [2.0, 4.0, 5.0]]], [[[1.0, 2.0, 2.0], [4.0, 6.0, 5.0], [8.0, 6.0, 3.0]], [[2.0, 7.0, 7.0], 
[8.0, 3.0, 3.0], [3.0, 4.0, 2.0]], [[3.0, 7.0, 3.0], [2.0, 7.0, 3.0], [5.0, 5.0, 5.0]]]])
      val=Data(numpy.array([[[[6.0, 2.0, 5.0]], [[1.0, 2.0, 1.0]]], [[[3.0, 3.0, 3.0]], [[5.0, 4.0, 7.0]]], [[[4.0, 6.0, 3.0]], 
[[2.0, 7.0, 5.0]]], [[[5.0, 8.0, 7.0]], [[8.0, 3.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 1.0]], [[2.0, 7.0, 2.0]]], [[[8.0, 8.0, 8.0]], [[5.0, 4.0, 3.0]]], [[[6.0, 
7.0, 7.0]], [[2.0, 1.0, 4.0]]], [[[3.0, 2.0, 1.0]], [[2.0, 7.0, 8.0]]]]))
      arg[:,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 2.0], [6.0, 2.0, 5.0], [5.0, 4.0, 3.0]], [[4.0, 1.0, 7.0], [1.0, 2.0, 1.0], [6.0, 
2.0, 1.0]], [[6.0, 6.0, 2.0], [5.0, 1.0, 4.0], [2.0, 1.0, 6.0]]], [[[7.0, 8.0, 2.0], [3.0, 3.0, 3.0], [4.0, 6.0, 3.0]], [[7.0, 
2.0, 4.0], [5.0, 4.0, 7.0], [6.0, 1.0, 2.0]], [[7.0, 5.0, 7.0], [6.0, 6.0, 3.0], [8.0, 7.0, 8.0]]], [[[2.0, 1.0, 1.0], [4.0, 
6.0, 3.0], [4.0, 2.0, 7.0]], [[8.0, 7.0, 5.0], [2.0, 7.0, 5.0], [3.0, 2.0, 4.0]], [[6.0, 5.0, 1.0], [7.0, 8.0, 1.0], [4.0, 8.0, 
2.0]]], [[[2.0, 3.0, 8.0], [5.0, 8.0, 7.0], [2.0, 7.0, 7.0]], [[3.0, 2.0, 3.0], [8.0, 3.0, 4.0], [4.0, 3.0, 5.0]], [[5.0, 4.0, 
6.0], [2.0, 5.0, 6.0], [2.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 3.0], [5.0, 3.0, 1.0], [5.0, 3.0, 2.0]], 
[[2.0, 1.0, 2.0], [2.0, 7.0, 2.0], [3.0, 6.0, 1.0]], [[1.0, 8.0, 3.0], [8.0, 5.0, 1.0], [7.0, 2.0, 6.0]]], [[[4.0, 7.0, 7.0], 
[8.0, 8.0, 8.0], [7.0, 5.0, 1.0]], [[7.0, 3.0, 4.0], [5.0, 4.0, 3.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 2.0], [3.0, 4.0, 1.0], 
[8.0, 1.0, 6.0]]], [[[8.0, 4.0, 8.0], [6.0, 7.0, 7.0], [5.0, 4.0, 3.0]], [[4.0, 2.0, 2.0], [2.0, 1.0, 4.0], [4.0, 6.0, 5.0]], 
[[1.0, 6.0, 6.0], [8.0, 4.0, 2.0], [2.0, 4.0, 5.0]]], [[[1.0, 2.0, 2.0], [3.0, 2.0, 1.0], [8.0, 6.0, 3.0]], [[2.0, 7.0, 7.0], 
[2.0, 7.0, 8.0], [3.0, 4.0, 2.0]], [[3.0, 7.0, 3.0], [2.0, 7.0, 3.0], [5.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CUUU(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 3.0], [7.0, 5.0, 4.0], [5.0, 2.0, 7.0]], [[5.0, 5.0, 5.0], [1.0, 3.0, 5.0], [1.0, 
8.0, 5.0]], [[4.0, 1.0, 8.0], [2.0, 5.0, 7.0], [5.0, 2.0, 5.0]]], [[[2.0, 2.0, 4.0], [1.0, 1.0, 2.0], [5.0, 8.0, 7.0]], [[8.0, 
3.0, 4.0], [4.0, 3.0, 2.0], [8.0, 8.0, 6.0]], [[1.0, 8.0, 7.0], [2.0, 3.0, 5.0], [5.0, 6.0, 6.0]]], [[[6.0, 3.0, 6.0], [7.0, 
1.0, 5.0], [4.0, 7.0, 8.0]], [[6.0, 5.0, 4.0], [1.0, 3.0, 5.0], [6.0, 5.0, 2.0]], [[7.0, 4.0, 4.0], [6.0, 1.0, 6.0], [7.0, 2.0, 
2.0]]], [[[2.0, 1.0, 4.0], [2.0, 7.0, 3.0], [3.0, 6.0, 4.0]], [[2.0, 1.0, 6.0], [8.0, 6.0, 4.0], [2.0, 2.0, 5.0]], [[3.0, 1.0, 
5.0], [5.0, 8.0, 2.0], [8.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 7.0], [4.0, 5.0, 7.0], [3.0, 8.0, 3.0]], 
[[5.0, 4.0, 1.0], [4.0, 2.0, 2.0], [1.0, 6.0, 2.0]], [[8.0, 8.0, 5.0], [3.0, 5.0, 4.0], [4.0, 6.0, 3.0]]], [[[5.0, 8.0, 5.0], 
[7.0, 6.0, 5.0], [5.0, 1.0, 3.0]], [[1.0, 6.0, 8.0], [7.0, 4.0, 3.0], [6.0, 4.0, 8.0]], [[3.0, 8.0, 2.0], [1.0, 1.0, 6.0], 
[4.0, 5.0, 3.0]]], [[[6.0, 8.0, 1.0], [8.0, 8.0, 6.0], [8.0, 7.0, 5.0]], [[2.0, 3.0, 1.0], [4.0, 2.0, 5.0], [6.0, 7.0, 8.0]], 
[[3.0, 5.0, 4.0], [8.0, 5.0, 2.0], [2.0, 7.0, 1.0]]], [[[1.0, 7.0, 4.0], [5.0, 5.0, 7.0], [6.0, 6.0, 1.0]], [[6.0, 2.0, 4.0], 
[5.0, 3.0, 3.0], [4.0, 5.0, 7.0]], [[8.0, 3.0, 6.0], [4.0, 2.0, 6.0], [2.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[[[3.0, 8.0], [8.0, 7.0]], [[8.0, 2.0], [4.0, 6.0]]], [[[4.0, 2.0], [7.0, 4.0]], [[7.0, 8.0], [5.0, 
7.0]]], [[[7.0, 5.0], [8.0, 3.0]], [[8.0, 4.0], [4.0, 4.0]]], [[[6.0, 3.0], [3.0, 1.0]], [[4.0, 7.0], [6.0, 
5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 5.0], [2.0, 8.0]], [[3.0, 8.0], [6.0, 4.0]]], [[[8.0, 6.0], [4.0, 2.0]], [[1.0, 
5.0], [6.0, 1.0]]], [[[3.0, 4.0], [1.0, 3.0]], [[7.0, 8.0], [1.0, 7.0]]], [[[1.0, 1.0], [8.0, 2.0]], [[4.0, 8.0], [2.0, 
6.0]]]]))
      arg[:,:2,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 3.0], [8.0, 7.0, 4.0], [5.0, 2.0, 7.0]], [[8.0, 2.0, 5.0], [4.0, 6.0, 5.0], [1.0, 
8.0, 5.0]], [[4.0, 1.0, 8.0], [2.0, 5.0, 7.0], [5.0, 2.0, 5.0]]], [[[4.0, 2.0, 4.0], [7.0, 4.0, 2.0], [5.0, 8.0, 7.0]], [[7.0, 
8.0, 4.0], [5.0, 7.0, 2.0], [8.0, 8.0, 6.0]], [[1.0, 8.0, 7.0], [2.0, 3.0, 5.0], [5.0, 6.0, 6.0]]], [[[7.0, 5.0, 6.0], [8.0, 
3.0, 5.0], [4.0, 7.0, 8.0]], [[8.0, 4.0, 4.0], [4.0, 4.0, 5.0], [6.0, 5.0, 2.0]], [[7.0, 4.0, 4.0], [6.0, 1.0, 6.0], [7.0, 2.0, 
2.0]]], [[[6.0, 3.0, 4.0], [3.0, 1.0, 3.0], [3.0, 6.0, 4.0]], [[4.0, 7.0, 6.0], [6.0, 5.0, 4.0], [2.0, 2.0, 5.0]], [[3.0, 1.0, 
5.0], [5.0, 8.0, 2.0], [8.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 7.0], [2.0, 8.0, 7.0], [3.0, 8.0, 3.0]], 
[[3.0, 8.0, 1.0], [6.0, 4.0, 2.0], [1.0, 6.0, 2.0]], [[8.0, 8.0, 5.0], [3.0, 5.0, 4.0], [4.0, 6.0, 3.0]]], [[[8.0, 6.0, 5.0], 
[4.0, 2.0, 5.0], [5.0, 1.0, 3.0]], [[1.0, 5.0, 8.0], [6.0, 1.0, 3.0], [6.0, 4.0, 8.0]], [[3.0, 8.0, 2.0], [1.0, 1.0, 6.0], 
[4.0, 5.0, 3.0]]], [[[3.0, 4.0, 1.0], [1.0, 3.0, 6.0], [8.0, 7.0, 5.0]], [[7.0, 8.0, 1.0], [1.0, 7.0, 5.0], [6.0, 7.0, 8.0]], 
[[3.0, 5.0, 4.0], [8.0, 5.0, 2.0], [2.0, 7.0, 1.0]]], [[[1.0, 1.0, 4.0], [8.0, 2.0, 7.0], [6.0, 6.0, 1.0]], [[4.0, 8.0, 4.0], 
[2.0, 6.0, 3.0], [4.0, 5.0, 7.0]], [[8.0, 3.0, 6.0], [4.0, 2.0, 6.0], [2.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CPUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 2.0], [3.0, 8.0, 2.0], [5.0, 3.0, 2.0]], [[2.0, 4.0, 6.0], [4.0, 3.0, 6.0], [1.0, 
5.0, 2.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 4.0], [1.0, 4.0, 5.0]]], [[[2.0, 6.0, 7.0], [1.0, 7.0, 1.0], [5.0, 4.0, 1.0]], [[8.0, 
6.0, 5.0], [6.0, 1.0, 4.0], [1.0, 1.0, 1.0]], [[1.0, 8.0, 8.0], [1.0, 2.0, 6.0], [7.0, 2.0, 4.0]]], [[[2.0, 6.0, 6.0], [2.0, 
1.0, 8.0], [6.0, 4.0, 1.0]], [[2.0, 6.0, 5.0], [7.0, 5.0, 4.0], [6.0, 6.0, 8.0]], [[2.0, 4.0, 3.0], [5.0, 6.0, 7.0], [3.0, 6.0, 
7.0]]], [[[3.0, 3.0, 6.0], [5.0, 4.0, 7.0], [8.0, 8.0, 8.0]], [[2.0, 8.0, 3.0], [5.0, 2.0, 4.0], [2.0, 2.0, 4.0]], [[5.0, 2.0, 
4.0], [3.0, 7.0, 2.0], [3.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 4.0], [5.0, 1.0, 6.0], [4.0, 4.0, 6.0]], 
[[7.0, 5.0, 6.0], [8.0, 1.0, 1.0], [7.0, 4.0, 5.0]], [[4.0, 5.0, 3.0], [4.0, 4.0, 4.0], [6.0, 1.0, 7.0]]], [[[2.0, 1.0, 3.0], 
[4.0, 3.0, 4.0], [7.0, 1.0, 1.0]], [[7.0, 8.0, 6.0], [5.0, 7.0, 8.0], [6.0, 3.0, 8.0]], [[4.0, 6.0, 1.0], [5.0, 5.0, 2.0], 
[7.0, 3.0, 8.0]]], [[[8.0, 1.0, 7.0], [1.0, 7.0, 5.0], [4.0, 7.0, 6.0]], [[8.0, 5.0, 7.0], [4.0, 8.0, 2.0], [3.0, 1.0, 8.0]], 
[[8.0, 1.0, 7.0], [2.0, 6.0, 8.0], [4.0, 7.0, 7.0]]], [[[1.0, 1.0, 3.0], [4.0, 2.0, 8.0], [5.0, 8.0, 3.0]], [[5.0, 4.0, 7.0], 
[6.0, 5.0, 8.0], [1.0, 3.0, 6.0]], [[3.0, 7.0, 5.0], [2.0, 6.0, 7.0], [8.0, 8.0, 3.0]]]])
      val=Data(numpy.array([[[4.0, 2.0, 5.0], [3.0, 4.0, 4.0]], [[4.0, 7.0, 1.0], [6.0, 2.0, 1.0]], [[3.0, 8.0, 4.0], [8.0, 
8.0, 5.0]], [[3.0, 8.0, 4.0], [4.0, 8.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 2.0, 6.0], [7.0, 3.0, 1.0]], [[6.0, 2.0, 3.0], [6.0, 8.0, 2.0]], [[8.0, 4.0, 
6.0], [8.0, 1.0, 5.0]], [[3.0, 2.0, 6.0], [2.0, 7.0, 3.0]]]))
      arg[:,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 2.0], [3.0, 8.0, 2.0], [5.0, 3.0, 2.0]], [[4.0, 2.0, 5.0], [3.0, 4.0, 4.0], [1.0, 
5.0, 2.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 4.0], [1.0, 4.0, 5.0]]], [[[2.0, 6.0, 7.0], [1.0, 7.0, 1.0], [5.0, 4.0, 1.0]], [[4.0, 
7.0, 1.0], [6.0, 2.0, 1.0], [1.0, 1.0, 1.0]], [[1.0, 8.0, 8.0], [1.0, 2.0, 6.0], [7.0, 2.0, 4.0]]], [[[2.0, 6.0, 6.0], [2.0, 
1.0, 8.0], [6.0, 4.0, 1.0]], [[3.0, 8.0, 4.0], [8.0, 8.0, 5.0], [6.0, 6.0, 8.0]], [[2.0, 4.0, 3.0], [5.0, 6.0, 7.0], [3.0, 6.0, 
7.0]]], [[[3.0, 3.0, 6.0], [5.0, 4.0, 7.0], [8.0, 8.0, 8.0]], [[3.0, 8.0, 4.0], [4.0, 8.0, 6.0], [2.0, 2.0, 4.0]], [[5.0, 2.0, 
4.0], [3.0, 7.0, 2.0], [3.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 4.0], [5.0, 1.0, 6.0], [4.0, 4.0, 6.0]], 
[[2.0, 2.0, 6.0], [7.0, 3.0, 1.0], [7.0, 4.0, 5.0]], [[4.0, 5.0, 3.0], [4.0, 4.0, 4.0], [6.0, 1.0, 7.0]]], [[[2.0, 1.0, 3.0], 
[4.0, 3.0, 4.0], [7.0, 1.0, 1.0]], [[6.0, 2.0, 3.0], [6.0, 8.0, 2.0], [6.0, 3.0, 8.0]], [[4.0, 6.0, 1.0], [5.0, 5.0, 2.0], 
[7.0, 3.0, 8.0]]], [[[8.0, 1.0, 7.0], [1.0, 7.0, 5.0], [4.0, 7.0, 6.0]], [[8.0, 4.0, 6.0], [8.0, 1.0, 5.0], [3.0, 1.0, 8.0]], 
[[8.0, 1.0, 7.0], [2.0, 6.0, 8.0], [4.0, 7.0, 7.0]]], [[[1.0, 1.0, 3.0], [4.0, 2.0, 8.0], [5.0, 8.0, 3.0]], [[3.0, 2.0, 6.0], 
[2.0, 7.0, 3.0], [1.0, 3.0, 6.0]], [[3.0, 7.0, 5.0], [2.0, 6.0, 7.0], [8.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PRUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 4.0], [7.0, 7.0, 2.0], [1.0, 7.0, 3.0]], [[5.0, 5.0, 8.0], [6.0, 5.0, 3.0], [8.0, 
2.0, 4.0]], [[3.0, 8.0, 8.0], [4.0, 1.0, 6.0], [5.0, 6.0, 8.0]]], [[[7.0, 1.0, 6.0], [1.0, 5.0, 6.0], [8.0, 4.0, 8.0]], [[5.0, 
3.0, 1.0], [4.0, 7.0, 1.0], [4.0, 3.0, 3.0]], [[7.0, 3.0, 4.0], [4.0, 8.0, 7.0], [4.0, 4.0, 3.0]]], [[[4.0, 3.0, 1.0], [8.0, 
8.0, 1.0], [2.0, 1.0, 8.0]], [[1.0, 7.0, 1.0], [7.0, 6.0, 8.0], [7.0, 4.0, 3.0]], [[7.0, 8.0, 1.0], [2.0, 2.0, 7.0], [3.0, 4.0, 
1.0]]], [[[7.0, 5.0, 4.0], [5.0, 1.0, 6.0], [5.0, 2.0, 5.0]], [[1.0, 1.0, 5.0], [3.0, 8.0, 3.0], [3.0, 6.0, 3.0]], [[1.0, 5.0, 
7.0], [8.0, 6.0, 7.0], [7.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 8.0], [4.0, 6.0, 4.0], [1.0, 5.0, 2.0]], 
[[2.0, 1.0, 5.0], [2.0, 2.0, 2.0], [4.0, 6.0, 4.0]], [[2.0, 8.0, 3.0], [2.0, 1.0, 7.0], [7.0, 6.0, 8.0]]], [[[4.0, 4.0, 3.0], 
[3.0, 2.0, 5.0], [6.0, 3.0, 6.0]], [[8.0, 6.0, 2.0], [3.0, 7.0, 2.0], [3.0, 2.0, 8.0]], [[2.0, 5.0, 3.0], [3.0, 7.0, 3.0], 
[3.0, 4.0, 1.0]]], [[[1.0, 1.0, 8.0], [5.0, 7.0, 8.0], [8.0, 8.0, 8.0]], [[8.0, 8.0, 2.0], [6.0, 3.0, 3.0], [1.0, 3.0, 6.0]], 
[[3.0, 5.0, 2.0], [3.0, 2.0, 5.0], [2.0, 2.0, 1.0]]], [[[5.0, 1.0, 7.0], [5.0, 4.0, 1.0], [3.0, 7.0, 1.0]], [[5.0, 2.0, 6.0], 
[1.0, 7.0, 2.0], [5.0, 6.0, 4.0]], [[7.0, 2.0, 2.0], [4.0, 3.0, 5.0], [7.0, 2.0, 3.0]]]])
      val=Data(numpy.array([[[1.0, 5.0, 7.0], [5.0, 1.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 2.0, 6.0], [3.0, 8.0, 1.0]]]))
      arg[2,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 4.0], [7.0, 7.0, 2.0], [1.0, 7.0, 3.0]], [[5.0, 5.0, 8.0], [6.0, 5.0, 3.0], [8.0, 
2.0, 4.0]], [[3.0, 8.0, 8.0], [4.0, 1.0, 6.0], [5.0, 6.0, 8.0]]], [[[7.0, 1.0, 6.0], [1.0, 5.0, 6.0], [8.0, 4.0, 8.0]], [[5.0, 
3.0, 1.0], [4.0, 7.0, 1.0], [4.0, 3.0, 3.0]], [[7.0, 3.0, 4.0], [4.0, 8.0, 7.0], [4.0, 4.0, 3.0]]], [[[4.0, 3.0, 1.0], [8.0, 
8.0, 1.0], [2.0, 1.0, 8.0]], [[1.0, 5.0, 7.0], [5.0, 1.0, 6.0], [7.0, 4.0, 3.0]], [[7.0, 8.0, 1.0], [2.0, 2.0, 7.0], [3.0, 4.0, 
1.0]]], [[[7.0, 5.0, 4.0], [5.0, 1.0, 6.0], [5.0, 2.0, 5.0]], [[1.0, 1.0, 5.0], [3.0, 8.0, 3.0], [3.0, 6.0, 3.0]], [[1.0, 5.0, 
7.0], [8.0, 6.0, 7.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 8.0], [4.0, 6.0, 4.0], [1.0, 5.0, 2.0]], 
[[2.0, 1.0, 5.0], [2.0, 2.0, 2.0], [4.0, 6.0, 4.0]], [[2.0, 8.0, 3.0], [2.0, 1.0, 7.0], [7.0, 6.0, 8.0]]], [[[4.0, 4.0, 3.0], 
[3.0, 2.0, 5.0], [6.0, 3.0, 6.0]], [[8.0, 6.0, 2.0], [3.0, 7.0, 2.0], [3.0, 2.0, 8.0]], [[2.0, 5.0, 3.0], [3.0, 7.0, 3.0], 
[3.0, 4.0, 1.0]]], [[[1.0, 1.0, 8.0], [5.0, 7.0, 8.0], [8.0, 8.0, 8.0]], [[1.0, 2.0, 6.0], [3.0, 8.0, 1.0], [1.0, 3.0, 6.0]], 
[[3.0, 5.0, 2.0], [3.0, 2.0, 5.0], [2.0, 2.0, 1.0]]], [[[5.0, 1.0, 7.0], [5.0, 4.0, 1.0], [3.0, 7.0, 1.0]], [[5.0, 2.0, 6.0], 
[1.0, 7.0, 2.0], [5.0, 6.0, 4.0]], [[7.0, 2.0, 2.0], [4.0, 3.0, 5.0], [7.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RURN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 2.0], [5.0, 1.0, 7.0], [1.0, 6.0, 7.0]], [[6.0, 4.0, 3.0], [8.0, 1.0, 5.0], [5.0, 
2.0, 5.0]], [[1.0, 8.0, 8.0], [4.0, 4.0, 4.0], [2.0, 7.0, 7.0]]], [[[2.0, 5.0, 3.0], [8.0, 8.0, 7.0], [1.0, 6.0, 2.0]], [[2.0, 
1.0, 6.0], [4.0, 5.0, 3.0], [1.0, 8.0, 8.0]], [[8.0, 5.0, 5.0], [7.0, 8.0, 4.0], [5.0, 3.0, 6.0]]], [[[7.0, 2.0, 6.0], [8.0, 
3.0, 8.0], [8.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], [5.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[7.0, 7.0, 1.0], [5.0, 7.0, 1.0], [3.0, 2.0, 
8.0]]], [[[4.0, 2.0, 8.0], [8.0, 2.0, 3.0], [8.0, 7.0, 8.0]], [[4.0, 7.0, 2.0], [8.0, 7.0, 5.0], [2.0, 8.0, 3.0]], [[1.0, 3.0, 
6.0], [7.0, 5.0, 4.0], [6.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 1.0], [2.0, 6.0, 6.0], [3.0, 4.0, 3.0]], 
[[2.0, 1.0, 8.0], [3.0, 7.0, 8.0], [7.0, 4.0, 7.0]], [[2.0, 8.0, 3.0], [7.0, 4.0, 8.0], [6.0, 2.0, 6.0]]], [[[2.0, 1.0, 6.0], 
[3.0, 3.0, 8.0], [4.0, 6.0, 3.0]], [[3.0, 8.0, 6.0], [7.0, 8.0, 7.0], [2.0, 6.0, 3.0]], [[6.0, 4.0, 6.0], [4.0, 8.0, 1.0], 
[5.0, 4.0, 6.0]]], [[[5.0, 7.0, 8.0], [5.0, 4.0, 7.0], [7.0, 8.0, 1.0]], [[5.0, 6.0, 1.0], [7.0, 7.0, 4.0], [6.0, 1.0, 5.0]], 
[[2.0, 8.0, 3.0], [3.0, 7.0, 7.0], [1.0, 7.0, 5.0]]], [[[4.0, 1.0, 1.0], [7.0, 2.0, 3.0], [1.0, 1.0, 6.0]], [[3.0, 1.0, 5.0], 
[2.0, 8.0, 6.0], [3.0, 7.0, 8.0]], [[8.0, 1.0, 8.0], [3.0, 1.0, 3.0], [1.0, 7.0, 1.0]]]])
      val=Data(numpy.array([[[[6.0, 1.0, 2.0]], [[3.0, 4.0, 2.0]]], [[[8.0, 1.0, 8.0]], [[6.0, 7.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 3.0, 7.0]], [[8.0, 4.0, 2.0]]], [[[7.0, 6.0, 3.0]], [[7.0, 5.0, 2.0]]]]))
      arg[1:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 2.0], [5.0, 1.0, 7.0], [1.0, 6.0, 7.0]], [[6.0, 4.0, 3.0], [8.0, 1.0, 5.0], [5.0, 
2.0, 5.0]], [[1.0, 8.0, 8.0], [4.0, 4.0, 4.0], [2.0, 7.0, 7.0]]], [[[2.0, 5.0, 3.0], [6.0, 1.0, 2.0], [1.0, 6.0, 2.0]], [[2.0, 
1.0, 6.0], [3.0, 4.0, 2.0], [1.0, 8.0, 8.0]], [[8.0, 5.0, 5.0], [7.0, 8.0, 4.0], [5.0, 3.0, 6.0]]], [[[7.0, 2.0, 6.0], [8.0, 
1.0, 8.0], [8.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], [6.0, 7.0, 4.0], [3.0, 7.0, 8.0]], [[7.0, 7.0, 1.0], [5.0, 7.0, 1.0], [3.0, 2.0, 
8.0]]], [[[4.0, 2.0, 8.0], [8.0, 2.0, 3.0], [8.0, 7.0, 8.0]], [[4.0, 7.0, 2.0], [8.0, 7.0, 5.0], [2.0, 8.0, 3.0]], [[1.0, 3.0, 
6.0], [7.0, 5.0, 4.0], [6.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 1.0], [2.0, 6.0, 6.0], [3.0, 4.0, 3.0]], 
[[2.0, 1.0, 8.0], [3.0, 7.0, 8.0], [7.0, 4.0, 7.0]], [[2.0, 8.0, 3.0], [7.0, 4.0, 8.0], [6.0, 2.0, 6.0]]], [[[2.0, 1.0, 6.0], 
[2.0, 3.0, 7.0], [4.0, 6.0, 3.0]], [[3.0, 8.0, 6.0], [8.0, 4.0, 2.0], [2.0, 6.0, 3.0]], [[6.0, 4.0, 6.0], [4.0, 8.0, 1.0], 
[5.0, 4.0, 6.0]]], [[[5.0, 7.0, 8.0], [7.0, 6.0, 3.0], [7.0, 8.0, 1.0]], [[5.0, 6.0, 1.0], [7.0, 5.0, 2.0], [6.0, 1.0, 5.0]], 
[[2.0, 8.0, 3.0], [3.0, 7.0, 7.0], [1.0, 7.0, 5.0]]], [[[4.0, 1.0, 1.0], [7.0, 2.0, 3.0], [1.0, 1.0, 6.0]], [[3.0, 1.0, 5.0], 
[2.0, 8.0, 6.0], [3.0, 7.0, 8.0]], [[8.0, 1.0, 8.0], [3.0, 1.0, 3.0], [1.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 3.0, 1.0], [7.0, 8.0, 4.0], [6.0, 1.0, 4.0]], [[7.0, 4.0, 1.0], [4.0, 1.0, 1.0], [8.0, 
1.0, 8.0]], [[5.0, 5.0, 7.0], [7.0, 5.0, 4.0], [4.0, 4.0, 3.0]]], [[[1.0, 4.0, 5.0], [1.0, 7.0, 6.0], [3.0, 5.0, 4.0]], [[8.0, 
4.0, 3.0], [5.0, 7.0, 4.0], [1.0, 1.0, 3.0]], [[3.0, 6.0, 5.0], [8.0, 4.0, 7.0], [4.0, 1.0, 5.0]]], [[[3.0, 1.0, 2.0], [2.0, 
4.0, 8.0], [3.0, 7.0, 2.0]], [[1.0, 7.0, 7.0], [7.0, 2.0, 3.0], [2.0, 1.0, 3.0]], [[6.0, 2.0, 4.0], [4.0, 8.0, 1.0], [5.0, 4.0, 
7.0]]], [[[4.0, 1.0, 6.0], [5.0, 5.0, 1.0], [6.0, 5.0, 8.0]], [[2.0, 8.0, 6.0], [1.0, 1.0, 8.0], [2.0, 1.0, 6.0]], [[7.0, 7.0, 
5.0], [3.0, 4.0, 1.0], [1.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 4.0], [7.0, 7.0, 8.0], [2.0, 6.0, 7.0]], 
[[4.0, 2.0, 6.0], [8.0, 6.0, 7.0], [2.0, 8.0, 8.0]], [[5.0, 6.0, 1.0], [6.0, 3.0, 8.0], [1.0, 7.0, 7.0]]], [[[2.0, 4.0, 4.0], 
[5.0, 8.0, 8.0], [3.0, 8.0, 5.0]], [[7.0, 1.0, 1.0], [5.0, 7.0, 4.0], [7.0, 7.0, 8.0]], [[5.0, 7.0, 8.0], [5.0, 6.0, 5.0], 
[7.0, 1.0, 3.0]]], [[[6.0, 6.0, 1.0], [5.0, 3.0, 8.0], [4.0, 6.0, 1.0]], [[6.0, 3.0, 2.0], [1.0, 4.0, 2.0], [2.0, 7.0, 6.0]], 
[[5.0, 6.0, 5.0], [8.0, 1.0, 7.0], [6.0, 4.0, 1.0]]], [[[5.0, 1.0, 7.0], [5.0, 7.0, 1.0], [5.0, 4.0, 7.0]], [[7.0, 1.0, 3.0], 
[6.0, 4.0, 2.0], [7.0, 2.0, 3.0]], [[7.0, 6.0, 8.0], [3.0, 3.0, 7.0], [3.0, 8.0, 4.0]]]])
      val=Data(numpy.array([[[5.0, 3.0, 7.0]], [[8.0, 3.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 4.0, 7.0]], [[1.0, 8.0, 1.0]]]))
      arg[2:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 1.0], [7.0, 8.0, 4.0], [6.0, 1.0, 4.0]], [[7.0, 4.0, 1.0], [4.0, 1.0, 1.0], [8.0, 
1.0, 8.0]], [[5.0, 5.0, 7.0], [7.0, 5.0, 4.0], [4.0, 4.0, 3.0]]], [[[1.0, 4.0, 5.0], [1.0, 7.0, 6.0], [3.0, 5.0, 4.0]], [[8.0, 
4.0, 3.0], [5.0, 7.0, 4.0], [1.0, 1.0, 3.0]], [[3.0, 6.0, 5.0], [8.0, 4.0, 7.0], [4.0, 1.0, 5.0]]], [[[3.0, 1.0, 2.0], [2.0, 
4.0, 8.0], [3.0, 7.0, 2.0]], [[1.0, 7.0, 7.0], [5.0, 3.0, 7.0], [2.0, 1.0, 3.0]], [[6.0, 2.0, 4.0], [4.0, 8.0, 1.0], [5.0, 4.0, 
7.0]]], [[[4.0, 1.0, 6.0], [5.0, 5.0, 1.0], [6.0, 5.0, 8.0]], [[2.0, 8.0, 6.0], [8.0, 3.0, 6.0], [2.0, 1.0, 6.0]], [[7.0, 7.0, 
5.0], [3.0, 4.0, 1.0], [1.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 4.0], [7.0, 7.0, 8.0], [2.0, 6.0, 7.0]], 
[[4.0, 2.0, 6.0], [8.0, 6.0, 7.0], [2.0, 8.0, 8.0]], [[5.0, 6.0, 1.0], [6.0, 3.0, 8.0], [1.0, 7.0, 7.0]]], [[[2.0, 4.0, 4.0], 
[5.0, 8.0, 8.0], [3.0, 8.0, 5.0]], [[7.0, 1.0, 1.0], [5.0, 7.0, 4.0], [7.0, 7.0, 8.0]], [[5.0, 7.0, 8.0], [5.0, 6.0, 5.0], 
[7.0, 1.0, 3.0]]], [[[6.0, 6.0, 1.0], [5.0, 3.0, 8.0], [4.0, 6.0, 1.0]], [[6.0, 3.0, 2.0], [4.0, 4.0, 7.0], [2.0, 7.0, 6.0]], 
[[5.0, 6.0, 5.0], [8.0, 1.0, 7.0], [6.0, 4.0, 1.0]]], [[[5.0, 1.0, 7.0], [5.0, 7.0, 1.0], [5.0, 4.0, 7.0]], [[7.0, 1.0, 3.0], 
[1.0, 8.0, 1.0], [7.0, 2.0, 3.0]], [[7.0, 6.0, 8.0], [3.0, 3.0, 7.0], [3.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UPPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 6.0, 7.0], [3.0, 2.0, 7.0], [6.0, 5.0, 1.0]], [[6.0, 4.0, 6.0], [7.0, 8.0, 2.0], [5.0, 
3.0, 7.0]], [[8.0, 6.0, 3.0], [4.0, 4.0, 7.0], [5.0, 6.0, 5.0]]], [[[8.0, 1.0, 6.0], [5.0, 1.0, 8.0], [7.0, 8.0, 7.0]], [[4.0, 
3.0, 2.0], [2.0, 5.0, 7.0], [3.0, 1.0, 3.0]], [[3.0, 6.0, 4.0], [2.0, 4.0, 3.0], [1.0, 6.0, 6.0]]], [[[1.0, 8.0, 1.0], [2.0, 
1.0, 7.0], [8.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], [4.0, 8.0, 2.0], [3.0, 3.0, 6.0]], [[7.0, 1.0, 7.0], [1.0, 3.0, 6.0], [6.0, 7.0, 
5.0]]], [[[4.0, 1.0, 1.0], [6.0, 3.0, 1.0], [8.0, 7.0, 2.0]], [[1.0, 1.0, 5.0], [1.0, 6.0, 2.0], [2.0, 3.0, 5.0]], [[5.0, 5.0, 
5.0], [6.0, 3.0, 4.0], [2.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 7.0], [3.0, 2.0, 4.0], [1.0, 5.0, 3.0]], 
[[6.0, 7.0, 2.0], [6.0, 6.0, 5.0], [5.0, 4.0, 4.0]], [[6.0, 3.0, 8.0], [5.0, 6.0, 6.0], [6.0, 7.0, 4.0]]], [[[2.0, 8.0, 6.0], 
[2.0, 4.0, 7.0], [3.0, 4.0, 8.0]], [[7.0, 5.0, 1.0], [3.0, 5.0, 2.0], [2.0, 1.0, 5.0]], [[3.0, 5.0, 3.0], [7.0, 1.0, 2.0], 
[2.0, 6.0, 2.0]]], [[[2.0, 7.0, 1.0], [8.0, 1.0, 7.0], [7.0, 8.0, 6.0]], [[2.0, 1.0, 1.0], [1.0, 5.0, 2.0], [1.0, 2.0, 5.0]], 
[[6.0, 3.0, 8.0], [2.0, 8.0, 7.0], [5.0, 1.0, 2.0]]], [[[5.0, 6.0, 7.0], [1.0, 6.0, 2.0], [2.0, 2.0, 2.0]], [[7.0, 6.0, 7.0], 
[7.0, 1.0, 2.0], [8.0, 4.0, 8.0]], [[6.0, 5.0, 5.0], [1.0, 4.0, 6.0], [4.0, 1.0, 8.0]]]])
      val=Data(numpy.array([[7.0, 6.0, 3.0], [7.0, 5.0, 3.0], [7.0, 1.0, 4.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 4.0, 5.0], [2.0, 4.0, 8.0], [5.0, 6.0, 7.0]]))
      arg[:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 7.0], [3.0, 2.0, 7.0], [6.0, 5.0, 1.0]], [[6.0, 4.0, 6.0], [7.0, 6.0, 3.0], [5.0, 
3.0, 7.0]], [[8.0, 6.0, 3.0], [4.0, 4.0, 7.0], [5.0, 6.0, 5.0]]], [[[8.0, 1.0, 6.0], [5.0, 1.0, 8.0], [7.0, 8.0, 7.0]], [[4.0, 
3.0, 2.0], [7.0, 5.0, 3.0], [3.0, 1.0, 3.0]], [[3.0, 6.0, 4.0], [2.0, 4.0, 3.0], [1.0, 6.0, 6.0]]], [[[1.0, 8.0, 1.0], [2.0, 
1.0, 7.0], [8.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], [7.0, 1.0, 4.0], [3.0, 3.0, 6.0]], [[7.0, 1.0, 7.0], [1.0, 3.0, 6.0], [6.0, 7.0, 
5.0]]], [[[4.0, 1.0, 1.0], [6.0, 3.0, 1.0], [8.0, 7.0, 2.0]], [[1.0, 1.0, 5.0], [1.0, 6.0, 2.0], [2.0, 3.0, 5.0]], [[5.0, 5.0, 
5.0], [6.0, 3.0, 4.0], [2.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 7.0], [3.0, 2.0, 4.0], [1.0, 5.0, 3.0]], 
[[6.0, 7.0, 2.0], [8.0, 4.0, 5.0], [5.0, 4.0, 4.0]], [[6.0, 3.0, 8.0], [5.0, 6.0, 6.0], [6.0, 7.0, 4.0]]], [[[2.0, 8.0, 6.0], 
[2.0, 4.0, 7.0], [3.0, 4.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 4.0, 8.0], [2.0, 1.0, 5.0]], [[3.0, 5.0, 3.0], [7.0, 1.0, 2.0], 
[2.0, 6.0, 2.0]]], [[[2.0, 7.0, 1.0], [8.0, 1.0, 7.0], [7.0, 8.0, 6.0]], [[2.0, 1.0, 1.0], [5.0, 6.0, 7.0], [1.0, 2.0, 5.0]], 
[[6.0, 3.0, 8.0], [2.0, 8.0, 7.0], [5.0, 1.0, 2.0]]], [[[5.0, 6.0, 7.0], [1.0, 6.0, 2.0], [2.0, 2.0, 2.0]], [[7.0, 6.0, 7.0], 
[7.0, 1.0, 2.0], [8.0, 4.0, 8.0]], [[6.0, 5.0, 5.0], [1.0, 4.0, 6.0], [4.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UPLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 7.0], [3.0, 1.0, 6.0], [1.0, 7.0, 8.0]], [[8.0, 5.0, 8.0], [4.0, 8.0, 8.0], [5.0, 
6.0, 7.0]], [[4.0, 6.0, 1.0], [4.0, 6.0, 4.0], [6.0, 7.0, 3.0]]], [[[4.0, 6.0, 5.0], [5.0, 5.0, 4.0], [1.0, 6.0, 8.0]], [[7.0, 
1.0, 2.0], [6.0, 1.0, 5.0], [7.0, 5.0, 7.0]], [[1.0, 8.0, 1.0], [6.0, 6.0, 3.0], [6.0, 7.0, 4.0]]], [[[6.0, 3.0, 5.0], [7.0, 
1.0, 6.0], [1.0, 5.0, 4.0]], [[1.0, 8.0, 3.0], [5.0, 1.0, 5.0], [7.0, 8.0, 7.0]], [[1.0, 5.0, 2.0], [2.0, 7.0, 7.0], [6.0, 6.0, 
4.0]]], [[[2.0, 2.0, 6.0], [7.0, 5.0, 1.0], [7.0, 8.0, 2.0]], [[4.0, 2.0, 7.0], [2.0, 7.0, 1.0], [3.0, 3.0, 6.0]], [[3.0, 7.0, 
4.0], [2.0, 3.0, 3.0], [3.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 7.0], [1.0, 3.0, 5.0], [6.0, 1.0, 4.0]], 
[[1.0, 1.0, 7.0], [2.0, 7.0, 5.0], [5.0, 7.0, 1.0]], [[2.0, 8.0, 7.0], [6.0, 5.0, 8.0], [5.0, 1.0, 5.0]]], [[[5.0, 3.0, 8.0], 
[4.0, 5.0, 7.0], [2.0, 4.0, 7.0]], [[8.0, 3.0, 1.0], [8.0, 2.0, 3.0], [4.0, 2.0, 1.0]], [[1.0, 4.0, 8.0], [4.0, 1.0, 5.0], 
[2.0, 8.0, 2.0]]], [[[2.0, 1.0, 2.0], [6.0, 7.0, 8.0], [5.0, 2.0, 7.0]], [[5.0, 2.0, 6.0], [2.0, 6.0, 8.0], [7.0, 6.0, 7.0]], 
[[1.0, 3.0, 5.0], [6.0, 4.0, 2.0], [6.0, 3.0, 8.0]]], [[[5.0, 3.0, 8.0], [4.0, 4.0, 8.0], [1.0, 5.0, 7.0]], [[2.0, 6.0, 4.0], 
[7.0, 1.0, 2.0], [2.0, 2.0, 5.0]], [[4.0, 7.0, 5.0], [6.0, 6.0, 7.0], [3.0, 1.0, 6.0]]]])
      val=Data(numpy.array([[[6.0, 6.0, 3.0]], [[8.0, 6.0, 3.0]], [[1.0, 8.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 6.0, 6.0]], [[4.0, 4.0, 4.0]], [[4.0, 5.0, 5.0]]]))
      arg[:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 7.0], [3.0, 1.0, 6.0], [1.0, 7.0, 8.0]], [[8.0, 5.0, 8.0], [4.0, 8.0, 8.0], [6.0, 
6.0, 3.0]], [[4.0, 6.0, 1.0], [4.0, 6.0, 4.0], [6.0, 7.0, 3.0]]], [[[4.0, 6.0, 5.0], [5.0, 5.0, 4.0], [1.0, 6.0, 8.0]], [[7.0, 
1.0, 2.0], [6.0, 1.0, 5.0], [8.0, 6.0, 3.0]], [[1.0, 8.0, 1.0], [6.0, 6.0, 3.0], [6.0, 7.0, 4.0]]], [[[6.0, 3.0, 5.0], [7.0, 
1.0, 6.0], [1.0, 5.0, 4.0]], [[1.0, 8.0, 3.0], [5.0, 1.0, 5.0], [1.0, 8.0, 8.0]], [[1.0, 5.0, 2.0], [2.0, 7.0, 7.0], [6.0, 6.0, 
4.0]]], [[[2.0, 2.0, 6.0], [7.0, 5.0, 1.0], [7.0, 8.0, 2.0]], [[4.0, 2.0, 7.0], [2.0, 7.0, 1.0], [3.0, 3.0, 6.0]], [[3.0, 7.0, 
4.0], [2.0, 3.0, 3.0], [3.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 7.0], [1.0, 3.0, 5.0], [6.0, 1.0, 4.0]], 
[[1.0, 1.0, 7.0], [2.0, 7.0, 5.0], [2.0, 6.0, 6.0]], [[2.0, 8.0, 7.0], [6.0, 5.0, 8.0], [5.0, 1.0, 5.0]]], [[[5.0, 3.0, 8.0], 
[4.0, 5.0, 7.0], [2.0, 4.0, 7.0]], [[8.0, 3.0, 1.0], [8.0, 2.0, 3.0], [4.0, 4.0, 4.0]], [[1.0, 4.0, 8.0], [4.0, 1.0, 5.0], 
[2.0, 8.0, 2.0]]], [[[2.0, 1.0, 2.0], [6.0, 7.0, 8.0], [5.0, 2.0, 7.0]], [[5.0, 2.0, 6.0], [2.0, 6.0, 8.0], [4.0, 5.0, 5.0]], 
[[1.0, 3.0, 5.0], [6.0, 4.0, 2.0], [6.0, 3.0, 8.0]]], [[[5.0, 3.0, 8.0], [4.0, 4.0, 8.0], [1.0, 5.0, 7.0]], [[2.0, 6.0, 4.0], 
[7.0, 1.0, 2.0], [2.0, 2.0, 5.0]], [[4.0, 7.0, 5.0], [6.0, 6.0, 7.0], [3.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CPRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 5.0, 1.0], [7.0, 1.0, 6.0], [4.0, 1.0, 5.0]], [[2.0, 2.0, 2.0], [5.0, 7.0, 8.0], [2.0, 
1.0, 3.0]], [[1.0, 6.0, 8.0], [7.0, 3.0, 5.0], [8.0, 5.0, 4.0]]], [[[1.0, 8.0, 3.0], [5.0, 1.0, 8.0], [3.0, 3.0, 4.0]], [[3.0, 
2.0, 1.0], [8.0, 4.0, 2.0], [1.0, 4.0, 1.0]], [[5.0, 4.0, 7.0], [3.0, 8.0, 3.0], [6.0, 4.0, 4.0]]], [[[2.0, 7.0, 5.0], [6.0, 
3.0, 7.0], [3.0, 6.0, 2.0]], [[3.0, 2.0, 6.0], [3.0, 1.0, 1.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [6.0, 1.0, 8.0], [3.0, 5.0, 
3.0]]], [[[6.0, 8.0, 4.0], [7.0, 1.0, 5.0], [8.0, 4.0, 1.0]], [[5.0, 8.0, 6.0], [3.0, 1.0, 5.0], [4.0, 1.0, 6.0]], [[2.0, 3.0, 
3.0], [5.0, 8.0, 6.0], [6.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 1.0], [5.0, 8.0, 6.0], [1.0, 7.0, 5.0]], 
[[4.0, 3.0, 1.0], [8.0, 7.0, 3.0], [2.0, 3.0, 6.0]], [[2.0, 2.0, 1.0], [7.0, 5.0, 1.0], [1.0, 6.0, 7.0]]], [[[6.0, 5.0, 3.0], 
[4.0, 3.0, 4.0], [2.0, 8.0, 7.0]], [[2.0, 5.0, 6.0], [8.0, 1.0, 8.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 6.0], [6.0, 8.0, 7.0], 
[7.0, 5.0, 8.0]]], [[[4.0, 7.0, 3.0], [7.0, 6.0, 5.0], [5.0, 2.0, 4.0]], [[4.0, 7.0, 3.0], [6.0, 3.0, 5.0], [4.0, 3.0, 5.0]], 
[[3.0, 3.0, 1.0], [3.0, 1.0, 5.0], [4.0, 8.0, 5.0]]], [[[7.0, 4.0, 3.0], [6.0, 3.0, 3.0], [4.0, 2.0, 1.0]], [[7.0, 2.0, 2.0], 
[3.0, 6.0, 8.0], [5.0, 3.0, 8.0]], [[5.0, 5.0, 4.0], [7.0, 7.0, 1.0], [5.0, 3.0, 5.0]]]])
      val=Data(numpy.array([[[8.0, 2.0, 6.0]], [[6.0, 8.0, 6.0]], [[1.0, 7.0, 4.0]], [[6.0, 6.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 3.0, 8.0]], [[1.0, 2.0, 5.0]], [[6.0, 3.0, 1.0]], [[1.0, 4.0, 4.0]]]))
      arg[:,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 1.0], [7.0, 1.0, 6.0], [4.0, 1.0, 5.0]], [[2.0, 2.0, 2.0], [8.0, 2.0, 6.0], [2.0, 
1.0, 3.0]], [[1.0, 6.0, 8.0], [7.0, 3.0, 5.0], [8.0, 5.0, 4.0]]], [[[1.0, 8.0, 3.0], [5.0, 1.0, 8.0], [3.0, 3.0, 4.0]], [[3.0, 
2.0, 1.0], [6.0, 8.0, 6.0], [1.0, 4.0, 1.0]], [[5.0, 4.0, 7.0], [3.0, 8.0, 3.0], [6.0, 4.0, 4.0]]], [[[2.0, 7.0, 5.0], [6.0, 
3.0, 7.0], [3.0, 6.0, 2.0]], [[3.0, 2.0, 6.0], [1.0, 7.0, 4.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [6.0, 1.0, 8.0], [3.0, 5.0, 
3.0]]], [[[6.0, 8.0, 4.0], [7.0, 1.0, 5.0], [8.0, 4.0, 1.0]], [[5.0, 8.0, 6.0], [6.0, 6.0, 2.0], [4.0, 1.0, 6.0]], [[2.0, 3.0, 
3.0], [5.0, 8.0, 6.0], [6.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 1.0], [5.0, 8.0, 6.0], [1.0, 7.0, 5.0]], 
[[4.0, 3.0, 1.0], [6.0, 3.0, 8.0], [2.0, 3.0, 6.0]], [[2.0, 2.0, 1.0], [7.0, 5.0, 1.0], [1.0, 6.0, 7.0]]], [[[6.0, 5.0, 3.0], 
[4.0, 3.0, 4.0], [2.0, 8.0, 7.0]], [[2.0, 5.0, 6.0], [1.0, 2.0, 5.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 6.0], [6.0, 8.0, 7.0], 
[7.0, 5.0, 8.0]]], [[[4.0, 7.0, 3.0], [7.0, 6.0, 5.0], [5.0, 2.0, 4.0]], [[4.0, 7.0, 3.0], [6.0, 3.0, 1.0], [4.0, 3.0, 5.0]], 
[[3.0, 3.0, 1.0], [3.0, 1.0, 5.0], [4.0, 8.0, 5.0]]], [[[7.0, 4.0, 3.0], [6.0, 3.0, 3.0], [4.0, 2.0, 1.0]], [[7.0, 2.0, 2.0], 
[1.0, 4.0, 4.0], [5.0, 3.0, 8.0]], [[5.0, 5.0, 4.0], [7.0, 7.0, 1.0], [5.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RLLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 6.0], [6.0, 6.0, 8.0], [4.0, 6.0, 2.0]], [[6.0, 4.0, 8.0], [4.0, 5.0, 8.0], [7.0, 
1.0, 6.0]], [[3.0, 6.0, 7.0], [5.0, 2.0, 5.0], [5.0, 6.0, 6.0]]], [[[1.0, 4.0, 5.0], [1.0, 8.0, 6.0], [5.0, 7.0, 6.0]], [[2.0, 
2.0, 5.0], [6.0, 5.0, 5.0], [7.0, 1.0, 4.0]], [[8.0, 3.0, 6.0], [1.0, 5.0, 4.0], [7.0, 7.0, 2.0]]], [[[3.0, 5.0, 2.0], [8.0, 
3.0, 2.0], [8.0, 6.0, 6.0]], [[3.0, 4.0, 7.0], [5.0, 7.0, 6.0], [2.0, 1.0, 1.0]], [[3.0, 8.0, 3.0], [4.0, 6.0, 3.0], [5.0, 4.0, 
8.0]]], [[[8.0, 8.0, 2.0], [5.0, 3.0, 4.0], [2.0, 5.0, 2.0]], [[7.0, 4.0, 8.0], [6.0, 1.0, 7.0], [1.0, 8.0, 4.0]], [[4.0, 8.0, 
5.0], [2.0, 6.0, 6.0], [4.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 8.0], [2.0, 4.0, 8.0], [3.0, 2.0, 6.0]], 
[[3.0, 1.0, 5.0], [8.0, 6.0, 3.0], [8.0, 8.0, 7.0]], [[6.0, 7.0, 5.0], [5.0, 6.0, 3.0], [7.0, 8.0, 2.0]]], [[[7.0, 7.0, 4.0], 
[5.0, 7.0, 2.0], [6.0, 8.0, 2.0]], [[8.0, 6.0, 1.0], [1.0, 3.0, 7.0], [8.0, 3.0, 8.0]], [[5.0, 6.0, 7.0], [6.0, 2.0, 2.0], 
[6.0, 7.0, 7.0]]], [[[2.0, 7.0, 7.0], [7.0, 5.0, 6.0], [8.0, 2.0, 3.0]], [[2.0, 3.0, 5.0], [7.0, 6.0, 5.0], [7.0, 2.0, 2.0]], 
[[5.0, 6.0, 8.0], [8.0, 7.0, 1.0], [5.0, 2.0, 1.0]]], [[[1.0, 1.0, 1.0], [2.0, 5.0, 8.0], [7.0, 6.0, 3.0]], [[1.0, 2.0, 6.0], 
[1.0, 2.0, 5.0], [7.0, 3.0, 6.0]], [[7.0, 7.0, 5.0], [4.0, 7.0, 5.0], [8.0, 1.0, 5.0]]]])
      val=Data(numpy.array([[[[7.0, 5.0, 5.0]]], [[[4.0, 7.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 5.0]]], [[[5.0, 5.0, 7.0]]]]))
      arg[1:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 6.0], [6.0, 6.0, 8.0], [4.0, 6.0, 2.0]], [[6.0, 4.0, 8.0], [4.0, 5.0, 8.0], [7.0, 
1.0, 6.0]], [[3.0, 6.0, 7.0], [5.0, 2.0, 5.0], [5.0, 6.0, 6.0]]], [[[1.0, 4.0, 5.0], [1.0, 8.0, 6.0], [5.0, 7.0, 6.0]], [[2.0, 
2.0, 5.0], [6.0, 5.0, 5.0], [7.0, 1.0, 4.0]], [[8.0, 3.0, 6.0], [1.0, 5.0, 4.0], [7.0, 5.0, 5.0]]], [[[3.0, 5.0, 2.0], [8.0, 
3.0, 2.0], [8.0, 6.0, 6.0]], [[3.0, 4.0, 7.0], [5.0, 7.0, 6.0], [2.0, 1.0, 1.0]], [[3.0, 8.0, 3.0], [4.0, 6.0, 3.0], [4.0, 7.0, 
2.0]]], [[[8.0, 8.0, 2.0], [5.0, 3.0, 4.0], [2.0, 5.0, 2.0]], [[7.0, 4.0, 8.0], [6.0, 1.0, 7.0], [1.0, 8.0, 4.0]], [[4.0, 8.0, 
5.0], [2.0, 6.0, 6.0], [4.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 8.0], [2.0, 4.0, 8.0], [3.0, 2.0, 6.0]], 
[[3.0, 1.0, 5.0], [8.0, 6.0, 3.0], [8.0, 8.0, 7.0]], [[6.0, 7.0, 5.0], [5.0, 6.0, 3.0], [7.0, 8.0, 2.0]]], [[[7.0, 7.0, 4.0], 
[5.0, 7.0, 2.0], [6.0, 8.0, 2.0]], [[8.0, 6.0, 1.0], [1.0, 3.0, 7.0], [8.0, 3.0, 8.0]], [[5.0, 6.0, 7.0], [6.0, 2.0, 2.0], 
[4.0, 8.0, 5.0]]], [[[2.0, 7.0, 7.0], [7.0, 5.0, 6.0], [8.0, 2.0, 3.0]], [[2.0, 3.0, 5.0], [7.0, 6.0, 5.0], [7.0, 2.0, 2.0]], 
[[5.0, 6.0, 8.0], [8.0, 7.0, 1.0], [5.0, 5.0, 7.0]]], [[[1.0, 1.0, 1.0], [2.0, 5.0, 8.0], [7.0, 6.0, 3.0]], [[1.0, 2.0, 6.0], 
[1.0, 2.0, 5.0], [7.0, 3.0, 6.0]], [[7.0, 7.0, 5.0], [4.0, 7.0, 5.0], [8.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 7.0], [1.0, 4.0, 3.0], [3.0, 1.0, 4.0]], [[3.0, 1.0, 2.0], [5.0, 4.0, 4.0], [6.0, 
1.0, 2.0]], [[4.0, 3.0, 4.0], [1.0, 6.0, 4.0], [8.0, 4.0, 4.0]]], [[[6.0, 2.0, 8.0], [8.0, 8.0, 7.0], [5.0, 7.0, 1.0]], [[1.0, 
4.0, 3.0], [6.0, 1.0, 2.0], [4.0, 7.0, 2.0]], [[4.0, 7.0, 3.0], [2.0, 1.0, 7.0], [6.0, 6.0, 6.0]]], [[[3.0, 3.0, 5.0], [5.0, 
6.0, 5.0], [4.0, 8.0, 1.0]], [[2.0, 7.0, 5.0], [4.0, 6.0, 3.0], [5.0, 7.0, 1.0]], [[7.0, 8.0, 3.0], [1.0, 4.0, 8.0], [7.0, 5.0, 
1.0]]], [[[6.0, 3.0, 8.0], [2.0, 7.0, 4.0], [1.0, 8.0, 2.0]], [[2.0, 3.0, 5.0], [2.0, 8.0, 2.0], [7.0, 3.0, 3.0]], [[1.0, 3.0, 
6.0], [3.0, 7.0, 5.0], [7.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 6.0], [6.0, 7.0, 4.0], [4.0, 6.0, 8.0]], 
[[5.0, 1.0, 7.0], [4.0, 7.0, 6.0], [2.0, 3.0, 1.0]], [[5.0, 3.0, 7.0], [8.0, 5.0, 1.0], [5.0, 5.0, 3.0]]], [[[6.0, 2.0, 1.0], 
[4.0, 1.0, 8.0], [4.0, 6.0, 1.0]], [[5.0, 4.0, 4.0], [3.0, 4.0, 2.0], [4.0, 5.0, 5.0]], [[8.0, 4.0, 8.0], [4.0, 8.0, 3.0], 
[1.0, 4.0, 6.0]]], [[[7.0, 5.0, 5.0], [3.0, 1.0, 6.0], [5.0, 1.0, 6.0]], [[7.0, 2.0, 3.0], [7.0, 4.0, 8.0], [6.0, 2.0, 8.0]], 
[[6.0, 2.0, 8.0], [7.0, 2.0, 3.0], [6.0, 2.0, 3.0]]], [[[3.0, 6.0, 3.0], [2.0, 5.0, 1.0], [8.0, 8.0, 7.0]], [[5.0, 5.0, 4.0], 
[7.0, 6.0, 6.0], [4.0, 5.0, 3.0]], [[8.0, 5.0, 5.0], [5.0, 8.0, 4.0], [3.0, 2.0, 2.0]]]])
      val=Data(numpy.array([[[[4.0, 1.0, 2.0]]], [[[6.0, 5.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 5.0]]], [[[8.0, 7.0, 8.0]]]]))
      arg[1:3,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 7.0], [1.0, 4.0, 3.0], [3.0, 1.0, 4.0]], [[3.0, 1.0, 2.0], [5.0, 4.0, 4.0], [6.0, 
1.0, 2.0]], [[4.0, 3.0, 4.0], [1.0, 6.0, 4.0], [8.0, 4.0, 4.0]]], [[[6.0, 2.0, 8.0], [8.0, 8.0, 7.0], [5.0, 7.0, 1.0]], [[1.0, 
4.0, 3.0], [6.0, 1.0, 2.0], [4.0, 1.0, 2.0]], [[4.0, 7.0, 3.0], [2.0, 1.0, 7.0], [6.0, 6.0, 6.0]]], [[[3.0, 3.0, 5.0], [5.0, 
6.0, 5.0], [4.0, 8.0, 1.0]], [[2.0, 7.0, 5.0], [4.0, 6.0, 3.0], [6.0, 5.0, 3.0]], [[7.0, 8.0, 3.0], [1.0, 4.0, 8.0], [7.0, 5.0, 
1.0]]], [[[6.0, 3.0, 8.0], [2.0, 7.0, 4.0], [1.0, 8.0, 2.0]], [[2.0, 3.0, 5.0], [2.0, 8.0, 2.0], [7.0, 3.0, 3.0]], [[1.0, 3.0, 
6.0], [3.0, 7.0, 5.0], [7.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 6.0], [6.0, 7.0, 4.0], [4.0, 6.0, 8.0]], 
[[5.0, 1.0, 7.0], [4.0, 7.0, 6.0], [2.0, 3.0, 1.0]], [[5.0, 3.0, 7.0], [8.0, 5.0, 1.0], [5.0, 5.0, 3.0]]], [[[6.0, 2.0, 1.0], 
[4.0, 1.0, 8.0], [4.0, 6.0, 1.0]], [[5.0, 4.0, 4.0], [3.0, 4.0, 2.0], [7.0, 6.0, 5.0]], [[8.0, 4.0, 8.0], [4.0, 8.0, 3.0], 
[1.0, 4.0, 6.0]]], [[[7.0, 5.0, 5.0], [3.0, 1.0, 6.0], [5.0, 1.0, 6.0]], [[7.0, 2.0, 3.0], [7.0, 4.0, 8.0], [8.0, 7.0, 8.0]], 
[[6.0, 2.0, 8.0], [7.0, 2.0, 3.0], [6.0, 2.0, 3.0]]], [[[3.0, 6.0, 3.0], [2.0, 5.0, 1.0], [8.0, 8.0, 7.0]], [[5.0, 5.0, 4.0], 
[7.0, 6.0, 6.0], [4.0, 5.0, 3.0]], [[8.0, 5.0, 5.0], [5.0, 8.0, 4.0], [3.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UPCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 5.0], [1.0, 2.0, 4.0], [4.0, 5.0, 7.0]], [[3.0, 3.0, 4.0], [3.0, 6.0, 2.0], [3.0, 
6.0, 2.0]], [[5.0, 8.0, 1.0], [4.0, 4.0, 8.0], [3.0, 5.0, 4.0]]], [[[6.0, 7.0, 4.0], [6.0, 6.0, 6.0], [6.0, 8.0, 6.0]], [[7.0, 
3.0, 8.0], [3.0, 6.0, 3.0], [1.0, 4.0, 4.0]], [[3.0, 2.0, 4.0], [6.0, 2.0, 1.0], [7.0, 3.0, 6.0]]], [[[1.0, 2.0, 5.0], [7.0, 
7.0, 1.0], [6.0, 3.0, 6.0]], [[2.0, 6.0, 7.0], [6.0, 3.0, 8.0], [6.0, 3.0, 3.0]], [[5.0, 3.0, 6.0], [5.0, 2.0, 7.0], [3.0, 3.0, 
7.0]]], [[[5.0, 1.0, 6.0], [6.0, 5.0, 1.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 5.0], [1.0, 7.0, 3.0], [3.0, 1.0, 5.0]], [[4.0, 1.0, 
6.0], [3.0, 5.0, 2.0], [7.0, 5.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 4.0], [5.0, 6.0, 4.0], [6.0, 5.0, 1.0]], 
[[1.0, 2.0, 3.0], [8.0, 5.0, 2.0], [5.0, 8.0, 3.0]], [[4.0, 1.0, 2.0], [8.0, 6.0, 2.0], [7.0, 8.0, 3.0]]], [[[7.0, 2.0, 1.0], 
[6.0, 5.0, 3.0], [3.0, 6.0, 2.0]], [[4.0, 8.0, 1.0], [7.0, 7.0, 4.0], [2.0, 7.0, 4.0]], [[2.0, 1.0, 1.0], [4.0, 8.0, 5.0], 
[1.0, 7.0, 8.0]]], [[[8.0, 2.0, 4.0], [1.0, 4.0, 2.0], [1.0, 3.0, 2.0]], [[7.0, 2.0, 2.0], [3.0, 1.0, 4.0], [5.0, 5.0, 7.0]], 
[[3.0, 3.0, 8.0], [6.0, 4.0, 8.0], [7.0, 6.0, 3.0]]], [[[2.0, 3.0, 4.0], [4.0, 2.0, 5.0], [2.0, 2.0, 2.0]], [[8.0, 3.0, 5.0], 
[8.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[3.0, 7.0, 2.0], [5.0, 6.0, 3.0], [8.0, 4.0, 4.0]]]])
      val=Data(numpy.array([[[4.0, 2.0, 6.0], [2.0, 5.0, 6.0], [8.0, 3.0, 5.0]], [[5.0, 2.0, 8.0], [5.0, 2.0, 7.0], [7.0, 6.0, 
4.0]], [[2.0, 5.0, 6.0], [5.0, 4.0, 6.0], [4.0, 1.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 4.0, 5.0], [2.0, 2.0, 6.0], [4.0, 3.0, 4.0]], [[3.0, 1.0, 6.0], [5.0, 7.0, 8.0], 
[5.0, 4.0, 5.0]], [[5.0, 4.0, 3.0], [5.0, 4.0, 2.0], [3.0, 1.0, 5.0]]]))
      arg[:3,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 5.0], [1.0, 2.0, 4.0], [4.0, 5.0, 7.0]], [[4.0, 2.0, 6.0], [2.0, 5.0, 6.0], [8.0, 
3.0, 5.0]], [[5.0, 8.0, 1.0], [4.0, 4.0, 8.0], [3.0, 5.0, 4.0]]], [[[6.0, 7.0, 4.0], [6.0, 6.0, 6.0], [6.0, 8.0, 6.0]], [[5.0, 
2.0, 8.0], [5.0, 2.0, 7.0], [7.0, 6.0, 4.0]], [[3.0, 2.0, 4.0], [6.0, 2.0, 1.0], [7.0, 3.0, 6.0]]], [[[1.0, 2.0, 5.0], [7.0, 
7.0, 1.0], [6.0, 3.0, 6.0]], [[2.0, 5.0, 6.0], [5.0, 4.0, 6.0], [4.0, 1.0, 1.0]], [[5.0, 3.0, 6.0], [5.0, 2.0, 7.0], [3.0, 3.0, 
7.0]]], [[[5.0, 1.0, 6.0], [6.0, 5.0, 1.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 5.0], [1.0, 7.0, 3.0], [3.0, 1.0, 5.0]], [[4.0, 1.0, 
6.0], [3.0, 5.0, 2.0], [7.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 4.0], [5.0, 6.0, 4.0], [6.0, 5.0, 1.0]], 
[[3.0, 4.0, 5.0], [2.0, 2.0, 6.0], [4.0, 3.0, 4.0]], [[4.0, 1.0, 2.0], [8.0, 6.0, 2.0], [7.0, 8.0, 3.0]]], [[[7.0, 2.0, 1.0], 
[6.0, 5.0, 3.0], [3.0, 6.0, 2.0]], [[3.0, 1.0, 6.0], [5.0, 7.0, 8.0], [5.0, 4.0, 5.0]], [[2.0, 1.0, 1.0], [4.0, 8.0, 5.0], 
[1.0, 7.0, 8.0]]], [[[8.0, 2.0, 4.0], [1.0, 4.0, 2.0], [1.0, 3.0, 2.0]], [[5.0, 4.0, 3.0], [5.0, 4.0, 2.0], [3.0, 1.0, 5.0]], 
[[3.0, 3.0, 8.0], [6.0, 4.0, 8.0], [7.0, 6.0, 3.0]]], [[[2.0, 3.0, 4.0], [4.0, 2.0, 5.0], [2.0, 2.0, 2.0]], [[8.0, 3.0, 5.0], 
[8.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[3.0, 7.0, 2.0], [5.0, 6.0, 3.0], [8.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UURN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 1.0], [2.0, 7.0, 3.0], [6.0, 4.0, 2.0]], [[4.0, 2.0, 5.0], [7.0, 6.0, 8.0], [7.0, 
5.0, 1.0]], [[4.0, 2.0, 8.0], [4.0, 1.0, 8.0], [2.0, 3.0, 5.0]]], [[[3.0, 4.0, 1.0], [3.0, 3.0, 5.0], [5.0, 5.0, 2.0]], [[4.0, 
3.0, 5.0], [7.0, 1.0, 1.0], [4.0, 5.0, 8.0]], [[3.0, 8.0, 1.0], [7.0, 2.0, 5.0], [3.0, 5.0, 5.0]]], [[[4.0, 3.0, 1.0], [3.0, 
3.0, 6.0], [7.0, 8.0, 5.0]], [[7.0, 4.0, 4.0], [4.0, 1.0, 5.0], [4.0, 7.0, 6.0]], [[1.0, 2.0, 1.0], [5.0, 7.0, 2.0], [3.0, 7.0, 
6.0]]], [[[5.0, 2.0, 1.0], [1.0, 6.0, 5.0], [2.0, 1.0, 5.0]], [[6.0, 5.0, 6.0], [2.0, 4.0, 3.0], [4.0, 5.0, 4.0]], [[1.0, 3.0, 
2.0], [8.0, 1.0, 2.0], [3.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 5.0], [1.0, 2.0, 3.0], [5.0, 1.0, 4.0]], 
[[5.0, 2.0, 2.0], [1.0, 7.0, 4.0], [8.0, 4.0, 5.0]], [[7.0, 4.0, 2.0], [7.0, 4.0, 8.0], [8.0, 4.0, 4.0]]], [[[2.0, 3.0, 1.0], 
[1.0, 6.0, 8.0], [5.0, 2.0, 6.0]], [[1.0, 3.0, 4.0], [5.0, 7.0, 3.0], [3.0, 6.0, 5.0]], [[5.0, 2.0, 6.0], [6.0, 8.0, 6.0], 
[6.0, 1.0, 3.0]]], [[[7.0, 8.0, 3.0], [7.0, 5.0, 4.0], [5.0, 7.0, 5.0]], [[7.0, 4.0, 8.0], [8.0, 8.0, 3.0], [6.0, 1.0, 6.0]], 
[[5.0, 7.0, 8.0], [7.0, 8.0, 4.0], [3.0, 7.0, 3.0]]], [[[1.0, 5.0, 4.0], [6.0, 1.0, 4.0], [6.0, 3.0, 3.0]], [[7.0, 3.0, 2.0], 
[8.0, 1.0, 5.0], [7.0, 2.0, 2.0]], [[7.0, 6.0, 8.0], [1.0, 4.0, 5.0], [8.0, 1.0, 1.0]]]])
      val=Data(numpy.array([[[[2.0, 8.0, 3.0]], [[5.0, 2.0, 5.0]]], [[[6.0, 7.0, 4.0]], [[6.0, 6.0, 1.0]]], [[[3.0, 6.0, 5.0]], 
[[8.0, 4.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 7.0, 5.0]], [[7.0, 4.0, 8.0]]], [[[2.0, 8.0, 5.0]], [[2.0, 7.0, 7.0]]], [[[8.0, 
3.0, 5.0]], [[8.0, 2.0, 1.0]]]]))
      arg[:3,:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 1.0], [2.0, 8.0, 3.0], [6.0, 4.0, 2.0]], [[4.0, 2.0, 5.0], [5.0, 2.0, 5.0], [7.0, 
5.0, 1.0]], [[4.0, 2.0, 8.0], [4.0, 1.0, 8.0], [2.0, 3.0, 5.0]]], [[[3.0, 4.0, 1.0], [6.0, 7.0, 4.0], [5.0, 5.0, 2.0]], [[4.0, 
3.0, 5.0], [6.0, 6.0, 1.0], [4.0, 5.0, 8.0]], [[3.0, 8.0, 1.0], [7.0, 2.0, 5.0], [3.0, 5.0, 5.0]]], [[[4.0, 3.0, 1.0], [3.0, 
6.0, 5.0], [7.0, 8.0, 5.0]], [[7.0, 4.0, 4.0], [8.0, 4.0, 7.0], [4.0, 7.0, 6.0]], [[1.0, 2.0, 1.0], [5.0, 7.0, 2.0], [3.0, 7.0, 
6.0]]], [[[5.0, 2.0, 1.0], [1.0, 6.0, 5.0], [2.0, 1.0, 5.0]], [[6.0, 5.0, 6.0], [2.0, 4.0, 3.0], [4.0, 5.0, 4.0]], [[1.0, 3.0, 
2.0], [8.0, 1.0, 2.0], [3.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 5.0], [8.0, 7.0, 5.0], [5.0, 1.0, 4.0]], 
[[5.0, 2.0, 2.0], [7.0, 4.0, 8.0], [8.0, 4.0, 5.0]], [[7.0, 4.0, 2.0], [7.0, 4.0, 8.0], [8.0, 4.0, 4.0]]], [[[2.0, 3.0, 1.0], 
[2.0, 8.0, 5.0], [5.0, 2.0, 6.0]], [[1.0, 3.0, 4.0], [2.0, 7.0, 7.0], [3.0, 6.0, 5.0]], [[5.0, 2.0, 6.0], [6.0, 8.0, 6.0], 
[6.0, 1.0, 3.0]]], [[[7.0, 8.0, 3.0], [8.0, 3.0, 5.0], [5.0, 7.0, 5.0]], [[7.0, 4.0, 8.0], [8.0, 2.0, 1.0], [6.0, 1.0, 6.0]], 
[[5.0, 7.0, 8.0], [7.0, 8.0, 4.0], [3.0, 7.0, 3.0]]], [[[1.0, 5.0, 4.0], [6.0, 1.0, 4.0], [6.0, 3.0, 3.0]], [[7.0, 3.0, 2.0], 
[8.0, 1.0, 5.0], [7.0, 2.0, 2.0]], [[7.0, 6.0, 8.0], [1.0, 4.0, 5.0], [8.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 3.0], [2.0, 7.0, 6.0], [5.0, 4.0, 7.0]], [[4.0, 4.0, 3.0], [5.0, 8.0, 1.0], [8.0, 
2.0, 5.0]], [[2.0, 4.0, 5.0], [7.0, 1.0, 1.0], [1.0, 1.0, 7.0]]], [[[4.0, 7.0, 3.0], [2.0, 7.0, 3.0], [1.0, 7.0, 8.0]], [[1.0, 
5.0, 4.0], [5.0, 6.0, 6.0], [5.0, 4.0, 4.0]], [[6.0, 7.0, 2.0], [4.0, 8.0, 8.0], [8.0, 6.0, 1.0]]], [[[2.0, 7.0, 7.0], [7.0, 
8.0, 8.0], [6.0, 7.0, 5.0]], [[7.0, 7.0, 6.0], [3.0, 3.0, 6.0], [5.0, 6.0, 8.0]], [[7.0, 3.0, 1.0], [6.0, 4.0, 8.0], [5.0, 5.0, 
7.0]]], [[[1.0, 7.0, 7.0], [6.0, 6.0, 5.0], [7.0, 1.0, 7.0]], [[2.0, 5.0, 2.0], [5.0, 7.0, 7.0], [1.0, 6.0, 8.0]], [[2.0, 5.0, 
6.0], [1.0, 6.0, 6.0], [2.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 4.0], [8.0, 7.0, 6.0], [4.0, 1.0, 3.0]], 
[[1.0, 4.0, 7.0], [6.0, 7.0, 3.0], [8.0, 3.0, 2.0]], [[6.0, 2.0, 4.0], [4.0, 5.0, 5.0], [1.0, 4.0, 2.0]]], [[[4.0, 8.0, 8.0], 
[7.0, 5.0, 2.0], [5.0, 4.0, 6.0]], [[6.0, 4.0, 5.0], [3.0, 2.0, 8.0], [8.0, 8.0, 8.0]], [[6.0, 2.0, 3.0], [2.0, 5.0, 6.0], 
[8.0, 3.0, 3.0]]], [[[8.0, 4.0, 2.0], [2.0, 7.0, 6.0], [1.0, 7.0, 2.0]], [[3.0, 5.0, 5.0], [8.0, 8.0, 4.0], [3.0, 7.0, 6.0]], 
[[7.0, 1.0, 3.0], [8.0, 8.0, 6.0], [7.0, 3.0, 4.0]]], [[[5.0, 5.0, 1.0], [1.0, 8.0, 6.0], [7.0, 2.0, 3.0]], [[3.0, 5.0, 8.0], 
[6.0, 4.0, 1.0], [6.0, 6.0, 3.0]], [[8.0, 6.0, 6.0], [2.0, 7.0, 5.0], [5.0, 2.0, 4.0]]]])
      val=Data(numpy.array([[[[1.0, 3.0, 6.0], [1.0, 5.0, 2.0]]], [[[2.0, 5.0, 6.0], [5.0, 8.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 4.0, 3.0], [7.0, 6.0, 5.0]]], [[[2.0, 4.0, 2.0], [1.0, 7.0, 4.0]]]]))
      arg[2:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 3.0], [2.0, 7.0, 6.0], [5.0, 4.0, 7.0]], [[4.0, 4.0, 3.0], [5.0, 8.0, 1.0], [8.0, 
2.0, 5.0]], [[2.0, 4.0, 5.0], [7.0, 1.0, 1.0], [1.0, 1.0, 7.0]]], [[[4.0, 7.0, 3.0], [2.0, 7.0, 3.0], [1.0, 7.0, 8.0]], [[1.0, 
5.0, 4.0], [5.0, 6.0, 6.0], [5.0, 4.0, 4.0]], [[6.0, 7.0, 2.0], [4.0, 8.0, 8.0], [8.0, 6.0, 1.0]]], [[[2.0, 7.0, 7.0], [7.0, 
8.0, 8.0], [6.0, 7.0, 5.0]], [[7.0, 7.0, 6.0], [3.0, 3.0, 6.0], [5.0, 6.0, 8.0]], [[1.0, 3.0, 6.0], [1.0, 5.0, 2.0], [5.0, 5.0, 
7.0]]], [[[1.0, 7.0, 7.0], [6.0, 6.0, 5.0], [7.0, 1.0, 7.0]], [[2.0, 5.0, 2.0], [5.0, 7.0, 7.0], [1.0, 6.0, 8.0]], [[2.0, 5.0, 
6.0], [5.0, 8.0, 5.0], [2.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 4.0], [8.0, 7.0, 6.0], [4.0, 1.0, 3.0]], 
[[1.0, 4.0, 7.0], [6.0, 7.0, 3.0], [8.0, 3.0, 2.0]], [[6.0, 2.0, 4.0], [4.0, 5.0, 5.0], [1.0, 4.0, 2.0]]], [[[4.0, 8.0, 8.0], 
[7.0, 5.0, 2.0], [5.0, 4.0, 6.0]], [[6.0, 4.0, 5.0], [3.0, 2.0, 8.0], [8.0, 8.0, 8.0]], [[6.0, 2.0, 3.0], [2.0, 5.0, 6.0], 
[8.0, 3.0, 3.0]]], [[[8.0, 4.0, 2.0], [2.0, 7.0, 6.0], [1.0, 7.0, 2.0]], [[3.0, 5.0, 5.0], [8.0, 8.0, 4.0], [3.0, 7.0, 6.0]], 
[[7.0, 4.0, 3.0], [7.0, 6.0, 5.0], [7.0, 3.0, 4.0]]], [[[5.0, 5.0, 1.0], [1.0, 8.0, 6.0], [7.0, 2.0, 3.0]], [[3.0, 5.0, 8.0], 
[6.0, 4.0, 1.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 2.0], [1.0, 7.0, 4.0], [5.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LUCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 2.0], [3.0, 3.0, 1.0], [4.0, 2.0, 4.0]], [[3.0, 1.0, 1.0], [8.0, 6.0, 8.0], [3.0, 
2.0, 8.0]], [[1.0, 3.0, 8.0], [7.0, 7.0, 1.0], [3.0, 4.0, 7.0]]], [[[7.0, 2.0, 5.0], [1.0, 4.0, 1.0], [4.0, 4.0, 2.0]], [[6.0, 
5.0, 5.0], [3.0, 3.0, 8.0], [3.0, 7.0, 2.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 6.0], [2.0, 7.0, 1.0]]], [[[5.0, 7.0, 6.0], [8.0, 
6.0, 5.0], [3.0, 7.0, 2.0]], [[5.0, 6.0, 2.0], [4.0, 1.0, 3.0], [3.0, 8.0, 2.0]], [[7.0, 5.0, 1.0], [5.0, 4.0, 6.0], [4.0, 1.0, 
1.0]]], [[[8.0, 7.0, 5.0], [1.0, 6.0, 1.0], [5.0, 7.0, 2.0]], [[3.0, 2.0, 2.0], [8.0, 1.0, 5.0], [8.0, 6.0, 4.0]], [[4.0, 1.0, 
2.0], [7.0, 5.0, 5.0], [2.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 3.0], [1.0, 7.0, 7.0], [8.0, 2.0, 4.0]], 
[[7.0, 3.0, 6.0], [5.0, 8.0, 8.0], [6.0, 8.0, 4.0]], [[4.0, 7.0, 4.0], [2.0, 4.0, 1.0], [7.0, 3.0, 4.0]]], [[[4.0, 1.0, 6.0], 
[6.0, 4.0, 4.0], [5.0, 2.0, 6.0]], [[7.0, 1.0, 4.0], [4.0, 6.0, 5.0], [4.0, 7.0, 2.0]], [[4.0, 3.0, 6.0], [2.0, 7.0, 6.0], 
[4.0, 2.0, 8.0]]], [[[7.0, 3.0, 3.0], [6.0, 6.0, 8.0], [3.0, 2.0, 6.0]], [[6.0, 8.0, 1.0], [8.0, 7.0, 2.0], [2.0, 8.0, 7.0]], 
[[1.0, 5.0, 1.0], [8.0, 6.0, 5.0], [7.0, 7.0, 1.0]]], [[[2.0, 7.0, 1.0], [8.0, 3.0, 5.0], [4.0, 4.0, 4.0]], [[5.0, 6.0, 5.0], 
[2.0, 6.0, 2.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 3.0], [6.0, 2.0, 1.0], [4.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[[7.0, 5.0, 7.0], [6.0, 5.0, 6.0], [3.0, 5.0, 5.0]], [[6.0, 5.0, 6.0], [3.0, 6.0, 3.0], [6.0, 7.0, 
7.0]]], [[[6.0, 6.0, 5.0], [7.0, 3.0, 2.0], [6.0, 1.0, 3.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 1.0], [3.0, 1.0, 
4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 1.0, 5.0], [6.0, 6.0, 3.0], [4.0, 8.0, 1.0]], [[6.0, 8.0, 1.0], [2.0, 6.0, 
6.0], [4.0, 6.0, 7.0]]], [[[4.0, 4.0, 1.0], [3.0, 4.0, 2.0], [8.0, 3.0, 4.0]], [[7.0, 6.0, 1.0], [8.0, 2.0, 2.0], [1.0, 2.0, 
1.0]]]]))
      arg[2:,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 2.0], [3.0, 3.0, 1.0], [4.0, 2.0, 4.0]], [[3.0, 1.0, 1.0], [8.0, 6.0, 8.0], [3.0, 
2.0, 8.0]], [[1.0, 3.0, 8.0], [7.0, 7.0, 1.0], [3.0, 4.0, 7.0]]], [[[7.0, 2.0, 5.0], [1.0, 4.0, 1.0], [4.0, 4.0, 2.0]], [[6.0, 
5.0, 5.0], [3.0, 3.0, 8.0], [3.0, 7.0, 2.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 6.0], [2.0, 7.0, 1.0]]], [[[7.0, 5.0, 7.0], [6.0, 
5.0, 6.0], [3.0, 5.0, 5.0]], [[6.0, 5.0, 6.0], [3.0, 6.0, 3.0], [6.0, 7.0, 7.0]], [[7.0, 5.0, 1.0], [5.0, 4.0, 6.0], [4.0, 1.0, 
1.0]]], [[[6.0, 6.0, 5.0], [7.0, 3.0, 2.0], [6.0, 1.0, 3.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 1.0], [3.0, 1.0, 4.0]], [[4.0, 1.0, 
2.0], [7.0, 5.0, 5.0], [2.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 3.0], [1.0, 7.0, 7.0], [8.0, 2.0, 4.0]], 
[[7.0, 3.0, 6.0], [5.0, 8.0, 8.0], [6.0, 8.0, 4.0]], [[4.0, 7.0, 4.0], [2.0, 4.0, 1.0], [7.0, 3.0, 4.0]]], [[[4.0, 1.0, 6.0], 
[6.0, 4.0, 4.0], [5.0, 2.0, 6.0]], [[7.0, 1.0, 4.0], [4.0, 6.0, 5.0], [4.0, 7.0, 2.0]], [[4.0, 3.0, 6.0], [2.0, 7.0, 6.0], 
[4.0, 2.0, 8.0]]], [[[7.0, 1.0, 5.0], [6.0, 6.0, 3.0], [4.0, 8.0, 1.0]], [[6.0, 8.0, 1.0], [2.0, 6.0, 6.0], [4.0, 6.0, 7.0]], 
[[1.0, 5.0, 1.0], [8.0, 6.0, 5.0], [7.0, 7.0, 1.0]]], [[[4.0, 4.0, 1.0], [3.0, 4.0, 2.0], [8.0, 3.0, 4.0]], [[7.0, 6.0, 1.0], 
[8.0, 2.0, 2.0], [1.0, 2.0, 1.0]], [[5.0, 8.0, 3.0], [6.0, 2.0, 1.0], [4.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 4.0], [6.0, 4.0, 5.0], [2.0, 6.0, 6.0]], [[7.0, 3.0, 4.0], [7.0, 5.0, 3.0], [2.0, 
4.0, 7.0]], [[3.0, 3.0, 1.0], [2.0, 5.0, 1.0], [4.0, 4.0, 1.0]]], [[[6.0, 2.0, 6.0], [6.0, 4.0, 3.0], [4.0, 8.0, 1.0]], [[5.0, 
4.0, 2.0], [4.0, 2.0, 5.0], [5.0, 7.0, 7.0]], [[3.0, 3.0, 3.0], [1.0, 3.0, 2.0], [3.0, 2.0, 8.0]]], [[[8.0, 6.0, 7.0], [6.0, 
7.0, 1.0], [6.0, 4.0, 1.0]], [[1.0, 6.0, 8.0], [7.0, 4.0, 3.0], [6.0, 7.0, 5.0]], [[5.0, 1.0, 5.0], [7.0, 4.0, 4.0], [8.0, 3.0, 
2.0]]], [[[8.0, 2.0, 4.0], [8.0, 6.0, 5.0], [1.0, 7.0, 7.0]], [[7.0, 2.0, 7.0], [8.0, 8.0, 5.0], [3.0, 7.0, 8.0]], [[8.0, 3.0, 
3.0], [4.0, 3.0, 2.0], [3.0, 5.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 5.0], [5.0, 6.0, 8.0], [6.0, 1.0, 1.0]], 
[[6.0, 4.0, 7.0], [6.0, 4.0, 3.0], [2.0, 3.0, 7.0]], [[2.0, 8.0, 6.0], [1.0, 8.0, 6.0], [3.0, 1.0, 6.0]]], [[[4.0, 5.0, 3.0], 
[2.0, 3.0, 5.0], [3.0, 3.0, 2.0]], [[5.0, 5.0, 6.0], [3.0, 2.0, 4.0], [8.0, 5.0, 1.0]], [[4.0, 1.0, 6.0], [2.0, 8.0, 3.0], 
[3.0, 2.0, 2.0]]], [[[4.0, 3.0, 3.0], [6.0, 4.0, 5.0], [2.0, 3.0, 6.0]], [[1.0, 1.0, 8.0], [6.0, 2.0, 3.0], [8.0, 2.0, 6.0]], 
[[4.0, 8.0, 1.0], [7.0, 3.0, 1.0], [8.0, 8.0, 8.0]]], [[[4.0, 1.0, 6.0], [4.0, 4.0, 2.0], [3.0, 8.0, 7.0]], [[2.0, 1.0, 5.0], 
[2.0, 8.0, 7.0], [2.0, 8.0, 2.0]], [[4.0, 1.0, 5.0], [4.0, 5.0, 7.0], [8.0, 3.0, 2.0]]]])
      val=Data(numpy.array([[[[4.0, 4.0, 3.0]], [[8.0, 7.0, 6.0]], [[1.0, 1.0, 5.0]]], [[[2.0, 5.0, 3.0]], [[6.0, 2.0, 1.0]], 
[[5.0, 3.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 5.0, 2.0]], [[2.0, 7.0, 1.0]], [[8.0, 4.0, 7.0]]], [[[3.0, 6.0, 8.0]], [[8.0, 
6.0, 6.0]], [[1.0, 1.0, 3.0]]]]))
      arg[2:,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 4.0], [6.0, 4.0, 5.0], [2.0, 6.0, 6.0]], [[7.0, 3.0, 4.0], [7.0, 5.0, 3.0], [2.0, 
4.0, 7.0]], [[3.0, 3.0, 1.0], [2.0, 5.0, 1.0], [4.0, 4.0, 1.0]]], [[[6.0, 2.0, 6.0], [6.0, 4.0, 3.0], [4.0, 8.0, 1.0]], [[5.0, 
4.0, 2.0], [4.0, 2.0, 5.0], [5.0, 7.0, 7.0]], [[3.0, 3.0, 3.0], [1.0, 3.0, 2.0], [3.0, 2.0, 8.0]]], [[[8.0, 6.0, 7.0], [4.0, 
4.0, 3.0], [6.0, 4.0, 1.0]], [[1.0, 6.0, 8.0], [8.0, 7.0, 6.0], [6.0, 7.0, 5.0]], [[5.0, 1.0, 5.0], [1.0, 1.0, 5.0], [8.0, 3.0, 
2.0]]], [[[8.0, 2.0, 4.0], [2.0, 5.0, 3.0], [1.0, 7.0, 7.0]], [[7.0, 2.0, 7.0], [6.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[8.0, 3.0, 
3.0], [5.0, 3.0, 3.0], [3.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 5.0], [5.0, 6.0, 8.0], [6.0, 1.0, 1.0]], 
[[6.0, 4.0, 7.0], [6.0, 4.0, 3.0], [2.0, 3.0, 7.0]], [[2.0, 8.0, 6.0], [1.0, 8.0, 6.0], [3.0, 1.0, 6.0]]], [[[4.0, 5.0, 3.0], 
[2.0, 3.0, 5.0], [3.0, 3.0, 2.0]], [[5.0, 5.0, 6.0], [3.0, 2.0, 4.0], [8.0, 5.0, 1.0]], [[4.0, 1.0, 6.0], [2.0, 8.0, 3.0], 
[3.0, 2.0, 2.0]]], [[[4.0, 3.0, 3.0], [1.0, 5.0, 2.0], [2.0, 3.0, 6.0]], [[1.0, 1.0, 8.0], [2.0, 7.0, 1.0], [8.0, 2.0, 6.0]], 
[[4.0, 8.0, 1.0], [8.0, 4.0, 7.0], [8.0, 8.0, 8.0]]], [[[4.0, 1.0, 6.0], [3.0, 6.0, 8.0], [3.0, 8.0, 7.0]], [[2.0, 1.0, 5.0], 
[8.0, 6.0, 6.0], [2.0, 8.0, 2.0]], [[4.0, 1.0, 5.0], [1.0, 1.0, 3.0], [8.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CRLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 3.0], [8.0, 5.0, 7.0], [6.0, 3.0, 3.0]], [[7.0, 7.0, 6.0], [4.0, 7.0, 4.0], [5.0, 
1.0, 7.0]], [[3.0, 6.0, 5.0], [2.0, 3.0, 8.0], [7.0, 2.0, 8.0]]], [[[7.0, 1.0, 6.0], [8.0, 5.0, 5.0], [7.0, 4.0, 2.0]], [[1.0, 
5.0, 8.0], [5.0, 7.0, 3.0], [5.0, 8.0, 5.0]], [[3.0, 5.0, 2.0], [5.0, 5.0, 7.0], [7.0, 8.0, 2.0]]], [[[7.0, 2.0, 4.0], [7.0, 
4.0, 4.0], [8.0, 2.0, 1.0]], [[2.0, 5.0, 2.0], [2.0, 5.0, 7.0], [3.0, 3.0, 6.0]], [[8.0, 5.0, 1.0], [2.0, 2.0, 2.0], [2.0, 4.0, 
3.0]]], [[[8.0, 8.0, 3.0], [7.0, 4.0, 7.0], [3.0, 7.0, 7.0]], [[7.0, 6.0, 3.0], [7.0, 3.0, 6.0], [4.0, 8.0, 4.0]], [[2.0, 6.0, 
1.0], [7.0, 3.0, 2.0], [1.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 2.0], [7.0, 4.0, 6.0], [7.0, 7.0, 2.0]], 
[[8.0, 2.0, 2.0], [4.0, 8.0, 4.0], [6.0, 3.0, 2.0]], [[7.0, 2.0, 6.0], [5.0, 2.0, 2.0], [2.0, 6.0, 3.0]]], [[[4.0, 5.0, 3.0], 
[2.0, 6.0, 4.0], [3.0, 5.0, 3.0]], [[4.0, 3.0, 5.0], [8.0, 5.0, 3.0], [1.0, 7.0, 1.0]], [[1.0, 8.0, 7.0], [7.0, 6.0, 5.0], 
[4.0, 4.0, 1.0]]], [[[1.0, 6.0, 4.0], [1.0, 6.0, 1.0], [8.0, 1.0, 2.0]], [[7.0, 5.0, 3.0], [4.0, 6.0, 6.0], [6.0, 5.0, 4.0]], 
[[6.0, 7.0, 1.0], [5.0, 6.0, 2.0], [3.0, 3.0, 2.0]]], [[[8.0, 4.0, 7.0], [4.0, 2.0, 6.0], [3.0, 7.0, 6.0]], [[1.0, 3.0, 8.0], 
[8.0, 4.0, 5.0], [5.0, 8.0, 8.0]], [[7.0, 6.0, 2.0], [5.0, 8.0, 2.0], [8.0, 5.0, 3.0]]]])
      val=Data(numpy.array([[[[4.0, 7.0, 7.0]]], [[[6.0, 2.0, 4.0]]], [[[2.0, 5.0, 8.0]]], [[[4.0, 1.0, 
6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 3.0]]], [[[5.0, 5.0, 7.0]]], [[[3.0, 1.0, 6.0]]], [[[8.0, 5.0, 8.0]]]]))
      arg[:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [8.0, 5.0, 7.0], [6.0, 3.0, 3.0]], [[7.0, 7.0, 6.0], [4.0, 7.0, 4.0], [4.0, 
7.0, 7.0]], [[3.0, 6.0, 5.0], [2.0, 3.0, 8.0], [7.0, 2.0, 8.0]]], [[[7.0, 1.0, 6.0], [8.0, 5.0, 5.0], [7.0, 4.0, 2.0]], [[1.0, 
5.0, 8.0], [5.0, 7.0, 3.0], [6.0, 2.0, 4.0]], [[3.0, 5.0, 2.0], [5.0, 5.0, 7.0], [7.0, 8.0, 2.0]]], [[[7.0, 2.0, 4.0], [7.0, 
4.0, 4.0], [8.0, 2.0, 1.0]], [[2.0, 5.0, 2.0], [2.0, 5.0, 7.0], [2.0, 5.0, 8.0]], [[8.0, 5.0, 1.0], [2.0, 2.0, 2.0], [2.0, 4.0, 
3.0]]], [[[8.0, 8.0, 3.0], [7.0, 4.0, 7.0], [3.0, 7.0, 7.0]], [[7.0, 6.0, 3.0], [7.0, 3.0, 6.0], [4.0, 1.0, 6.0]], [[2.0, 6.0, 
1.0], [7.0, 3.0, 2.0], [1.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 2.0], [7.0, 4.0, 6.0], [7.0, 7.0, 2.0]], 
[[8.0, 2.0, 2.0], [4.0, 8.0, 4.0], [7.0, 5.0, 3.0]], [[7.0, 2.0, 6.0], [5.0, 2.0, 2.0], [2.0, 6.0, 3.0]]], [[[4.0, 5.0, 3.0], 
[2.0, 6.0, 4.0], [3.0, 5.0, 3.0]], [[4.0, 3.0, 5.0], [8.0, 5.0, 3.0], [5.0, 5.0, 7.0]], [[1.0, 8.0, 7.0], [7.0, 6.0, 5.0], 
[4.0, 4.0, 1.0]]], [[[1.0, 6.0, 4.0], [1.0, 6.0, 1.0], [8.0, 1.0, 2.0]], [[7.0, 5.0, 3.0], [4.0, 6.0, 6.0], [3.0, 1.0, 6.0]], 
[[6.0, 7.0, 1.0], [5.0, 6.0, 2.0], [3.0, 3.0, 2.0]]], [[[8.0, 4.0, 7.0], [4.0, 2.0, 6.0], [3.0, 7.0, 6.0]], [[1.0, 3.0, 8.0], 
[8.0, 4.0, 5.0], [8.0, 5.0, 8.0]], [[7.0, 6.0, 2.0], [5.0, 8.0, 2.0], [8.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CRCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 5.0], [6.0, 4.0, 7.0], [6.0, 3.0, 2.0]], [[1.0, 6.0, 6.0], [4.0, 1.0, 5.0], [4.0, 
7.0, 1.0]], [[3.0, 1.0, 7.0], [2.0, 1.0, 1.0], [8.0, 4.0, 4.0]]], [[[2.0, 1.0, 4.0], [2.0, 4.0, 6.0], [7.0, 6.0, 4.0]], [[8.0, 
4.0, 5.0], [1.0, 6.0, 1.0], [5.0, 1.0, 6.0]], [[3.0, 3.0, 2.0], [5.0, 8.0, 4.0], [6.0, 8.0, 4.0]]], [[[7.0, 7.0, 6.0], [1.0, 
7.0, 3.0], [7.0, 6.0, 3.0]], [[7.0, 2.0, 8.0], [1.0, 2.0, 1.0], [8.0, 8.0, 3.0]], [[7.0, 6.0, 4.0], [4.0, 1.0, 4.0], [8.0, 2.0, 
2.0]]], [[[1.0, 8.0, 8.0], [2.0, 6.0, 4.0], [6.0, 6.0, 7.0]], [[6.0, 7.0, 3.0], [5.0, 3.0, 2.0], [6.0, 3.0, 1.0]], [[4.0, 6.0, 
5.0], [1.0, 1.0, 1.0], [1.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 4.0], [4.0, 6.0, 5.0], [5.0, 3.0, 2.0]], 
[[5.0, 2.0, 1.0], [2.0, 1.0, 6.0], [7.0, 1.0, 1.0]], [[8.0, 1.0, 3.0], [1.0, 5.0, 4.0], [1.0, 6.0, 6.0]]], [[[1.0, 3.0, 2.0], 
[6.0, 6.0, 4.0], [1.0, 5.0, 5.0]], [[4.0, 6.0, 3.0], [1.0, 8.0, 8.0], [4.0, 4.0, 3.0]], [[7.0, 6.0, 3.0], [8.0, 7.0, 7.0], 
[8.0, 8.0, 7.0]]], [[[7.0, 5.0, 6.0], [7.0, 8.0, 3.0], [6.0, 4.0, 2.0]], [[5.0, 8.0, 1.0], [7.0, 1.0, 6.0], [4.0, 1.0, 1.0]], 
[[8.0, 2.0, 8.0], [6.0, 4.0, 6.0], [5.0, 4.0, 4.0]]], [[[2.0, 8.0, 6.0], [8.0, 3.0, 4.0], [7.0, 4.0, 4.0]], [[1.0, 6.0, 6.0], 
[2.0, 1.0, 7.0], [3.0, 3.0, 2.0]], [[6.0, 8.0, 3.0], [2.0, 5.0, 6.0], [6.0, 3.0, 6.0]]]])
      val=Data(numpy.array([[[[8.0, 5.0, 2.0], [3.0, 7.0, 3.0], [3.0, 4.0, 2.0]]], [[[2.0, 4.0, 1.0], [7.0, 7.0, 7.0], [1.0, 
4.0, 7.0]]], [[[4.0, 4.0, 2.0], [8.0, 7.0, 1.0], [5.0, 3.0, 4.0]]], [[[8.0, 3.0, 4.0], [8.0, 3.0, 1.0], [6.0, 1.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 4.0, 7.0], [6.0, 4.0, 6.0], [3.0, 6.0, 2.0]]], [[[3.0, 3.0, 8.0], [8.0, 8.0, 
1.0], [7.0, 1.0, 5.0]]], [[[4.0, 2.0, 1.0], [6.0, 5.0, 4.0], [1.0, 7.0, 3.0]]], [[[3.0, 3.0, 5.0], [2.0, 3.0, 4.0], [5.0, 4.0, 
4.0]]]]))
      arg[:,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 5.0], [6.0, 4.0, 7.0], [6.0, 3.0, 2.0]], [[8.0, 5.0, 2.0], [3.0, 7.0, 3.0], [3.0, 
4.0, 2.0]], [[3.0, 1.0, 7.0], [2.0, 1.0, 1.0], [8.0, 4.0, 4.0]]], [[[2.0, 1.0, 4.0], [2.0, 4.0, 6.0], [7.0, 6.0, 4.0]], [[2.0, 
4.0, 1.0], [7.0, 7.0, 7.0], [1.0, 4.0, 7.0]], [[3.0, 3.0, 2.0], [5.0, 8.0, 4.0], [6.0, 8.0, 4.0]]], [[[7.0, 7.0, 6.0], [1.0, 
7.0, 3.0], [7.0, 6.0, 3.0]], [[4.0, 4.0, 2.0], [8.0, 7.0, 1.0], [5.0, 3.0, 4.0]], [[7.0, 6.0, 4.0], [4.0, 1.0, 4.0], [8.0, 2.0, 
2.0]]], [[[1.0, 8.0, 8.0], [2.0, 6.0, 4.0], [6.0, 6.0, 7.0]], [[8.0, 3.0, 4.0], [8.0, 3.0, 1.0], [6.0, 1.0, 1.0]], [[4.0, 6.0, 
5.0], [1.0, 1.0, 1.0], [1.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 4.0], [4.0, 6.0, 5.0], [5.0, 3.0, 2.0]], 
[[8.0, 4.0, 7.0], [6.0, 4.0, 6.0], [3.0, 6.0, 2.0]], [[8.0, 1.0, 3.0], [1.0, 5.0, 4.0], [1.0, 6.0, 6.0]]], [[[1.0, 3.0, 2.0], 
[6.0, 6.0, 4.0], [1.0, 5.0, 5.0]], [[3.0, 3.0, 8.0], [8.0, 8.0, 1.0], [7.0, 1.0, 5.0]], [[7.0, 6.0, 3.0], [8.0, 7.0, 7.0], 
[8.0, 8.0, 7.0]]], [[[7.0, 5.0, 6.0], [7.0, 8.0, 3.0], [6.0, 4.0, 2.0]], [[4.0, 2.0, 1.0], [6.0, 5.0, 4.0], [1.0, 7.0, 3.0]], 
[[8.0, 2.0, 8.0], [6.0, 4.0, 6.0], [5.0, 4.0, 4.0]]], [[[2.0, 8.0, 6.0], [8.0, 3.0, 4.0], [7.0, 4.0, 4.0]], [[3.0, 3.0, 5.0], 
[2.0, 3.0, 4.0], [5.0, 4.0, 4.0]], [[6.0, 8.0, 3.0], [2.0, 5.0, 6.0], [6.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 5.0, 2.0], [6.0, 5.0, 6.0], [1.0, 8.0, 6.0]], [[8.0, 2.0, 5.0], [6.0, 8.0, 6.0], [2.0, 
7.0, 6.0]], [[4.0, 8.0, 1.0], [5.0, 1.0, 7.0], [6.0, 1.0, 8.0]]], [[[7.0, 4.0, 5.0], [5.0, 5.0, 6.0], [4.0, 2.0, 7.0]], [[8.0, 
3.0, 3.0], [6.0, 7.0, 2.0], [3.0, 5.0, 7.0]], [[4.0, 6.0, 3.0], [7.0, 1.0, 7.0], [6.0, 1.0, 2.0]]], [[[6.0, 5.0, 2.0], [3.0, 
1.0, 1.0], [1.0, 2.0, 2.0]], [[3.0, 6.0, 6.0], [4.0, 6.0, 7.0], [8.0, 3.0, 4.0]], [[2.0, 7.0, 8.0], [7.0, 8.0, 2.0], [4.0, 6.0, 
4.0]]], [[[5.0, 1.0, 5.0], [8.0, 7.0, 1.0], [3.0, 8.0, 4.0]], [[1.0, 5.0, 5.0], [2.0, 5.0, 3.0], [6.0, 2.0, 5.0]], [[1.0, 6.0, 
3.0], [6.0, 2.0, 4.0], [8.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 7.0], [8.0, 8.0, 3.0], [1.0, 7.0, 5.0]], 
[[5.0, 6.0, 7.0], [2.0, 1.0, 6.0], [7.0, 8.0, 4.0]], [[4.0, 4.0, 1.0], [7.0, 3.0, 2.0], [8.0, 2.0, 1.0]]], [[[8.0, 8.0, 2.0], 
[3.0, 3.0, 5.0], [2.0, 3.0, 3.0]], [[4.0, 3.0, 1.0], [8.0, 2.0, 4.0], [1.0, 7.0, 7.0]], [[3.0, 8.0, 2.0], [1.0, 5.0, 1.0], 
[6.0, 2.0, 1.0]]], [[[3.0, 5.0, 5.0], [1.0, 4.0, 3.0], [8.0, 6.0, 6.0]], [[8.0, 1.0, 5.0], [5.0, 4.0, 6.0], [1.0, 6.0, 2.0]], 
[[4.0, 7.0, 6.0], [1.0, 3.0, 8.0], [7.0, 2.0, 5.0]]], [[[4.0, 2.0, 4.0], [5.0, 3.0, 2.0], [5.0, 1.0, 8.0]], [[6.0, 5.0, 6.0], 
[7.0, 4.0, 3.0], [6.0, 1.0, 4.0]], [[3.0, 5.0, 2.0], [1.0, 5.0, 2.0], [8.0, 6.0, 1.0]]]])
      val=Data(numpy.array([[[2.0, 8.0, 2.0]], [[4.0, 7.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 7.0, 4.0]], [[1.0, 1.0, 8.0]]]))
      arg[1:3,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 2.0], [6.0, 5.0, 6.0], [1.0, 8.0, 6.0]], [[8.0, 2.0, 5.0], [6.0, 8.0, 6.0], [2.0, 
7.0, 6.0]], [[4.0, 8.0, 1.0], [5.0, 1.0, 7.0], [6.0, 1.0, 8.0]]], [[[7.0, 4.0, 5.0], [5.0, 5.0, 6.0], [4.0, 2.0, 7.0]], [[8.0, 
3.0, 3.0], [2.0, 8.0, 2.0], [3.0, 5.0, 7.0]], [[4.0, 6.0, 3.0], [7.0, 1.0, 7.0], [6.0, 1.0, 2.0]]], [[[6.0, 5.0, 2.0], [3.0, 
1.0, 1.0], [1.0, 2.0, 2.0]], [[3.0, 6.0, 6.0], [4.0, 7.0, 2.0], [8.0, 3.0, 4.0]], [[2.0, 7.0, 8.0], [7.0, 8.0, 2.0], [4.0, 6.0, 
4.0]]], [[[5.0, 1.0, 5.0], [8.0, 7.0, 1.0], [3.0, 8.0, 4.0]], [[1.0, 5.0, 5.0], [2.0, 5.0, 3.0], [6.0, 2.0, 5.0]], [[1.0, 6.0, 
3.0], [6.0, 2.0, 4.0], [8.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 7.0], [8.0, 8.0, 3.0], [1.0, 7.0, 5.0]], 
[[5.0, 6.0, 7.0], [2.0, 1.0, 6.0], [7.0, 8.0, 4.0]], [[4.0, 4.0, 1.0], [7.0, 3.0, 2.0], [8.0, 2.0, 1.0]]], [[[8.0, 8.0, 2.0], 
[3.0, 3.0, 5.0], [2.0, 3.0, 3.0]], [[4.0, 3.0, 1.0], [6.0, 7.0, 4.0], [1.0, 7.0, 7.0]], [[3.0, 8.0, 2.0], [1.0, 5.0, 1.0], 
[6.0, 2.0, 1.0]]], [[[3.0, 5.0, 5.0], [1.0, 4.0, 3.0], [8.0, 6.0, 6.0]], [[8.0, 1.0, 5.0], [1.0, 1.0, 8.0], [1.0, 6.0, 2.0]], 
[[4.0, 7.0, 6.0], [1.0, 3.0, 8.0], [7.0, 2.0, 5.0]]], [[[4.0, 2.0, 4.0], [5.0, 3.0, 2.0], [5.0, 1.0, 8.0]], [[6.0, 5.0, 6.0], 
[7.0, 4.0, 3.0], [6.0, 1.0, 4.0]], [[3.0, 5.0, 2.0], [1.0, 5.0, 2.0], [8.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LULN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 3.0], [2.0, 2.0, 3.0], [3.0, 3.0, 3.0]], [[3.0, 3.0, 5.0], [7.0, 8.0, 2.0], [5.0, 
1.0, 3.0]], [[2.0, 2.0, 7.0], [3.0, 7.0, 7.0], [6.0, 2.0, 5.0]]], [[[7.0, 4.0, 1.0], [4.0, 5.0, 7.0], [6.0, 2.0, 3.0]], [[6.0, 
8.0, 4.0], [4.0, 1.0, 2.0], [5.0, 8.0, 4.0]], [[2.0, 4.0, 8.0], [5.0, 3.0, 5.0], [4.0, 1.0, 6.0]]], [[[4.0, 8.0, 6.0], [1.0, 
7.0, 1.0], [8.0, 2.0, 4.0]], [[2.0, 4.0, 1.0], [5.0, 3.0, 3.0], [2.0, 3.0, 3.0]], [[4.0, 3.0, 4.0], [3.0, 6.0, 3.0], [7.0, 5.0, 
2.0]]], [[[2.0, 5.0, 5.0], [4.0, 8.0, 8.0], [4.0, 3.0, 5.0]], [[8.0, 4.0, 5.0], [6.0, 1.0, 6.0], [5.0, 5.0, 8.0]], [[3.0, 3.0, 
3.0], [1.0, 8.0, 3.0], [7.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 7.0], [2.0, 4.0, 1.0], [2.0, 8.0, 2.0]], 
[[7.0, 5.0, 4.0], [6.0, 6.0, 1.0], [5.0, 3.0, 7.0]], [[6.0, 5.0, 2.0], [2.0, 5.0, 3.0], [1.0, 7.0, 4.0]]], [[[8.0, 3.0, 3.0], 
[6.0, 1.0, 4.0], [2.0, 5.0, 3.0]], [[5.0, 8.0, 8.0], [7.0, 4.0, 1.0], [4.0, 5.0, 4.0]], [[8.0, 1.0, 8.0], [1.0, 1.0, 4.0], 
[7.0, 3.0, 3.0]]], [[[8.0, 4.0, 5.0], [3.0, 7.0, 2.0], [5.0, 7.0, 7.0]], [[8.0, 5.0, 4.0], [3.0, 4.0, 8.0], [1.0, 3.0, 3.0]], 
[[3.0, 2.0, 8.0], [1.0, 7.0, 5.0], [1.0, 6.0, 6.0]]], [[[8.0, 4.0, 3.0], [4.0, 5.0, 4.0], [3.0, 2.0, 7.0]], [[3.0, 7.0, 4.0], 
[7.0, 2.0, 7.0], [3.0, 6.0, 6.0]], [[5.0, 5.0, 7.0], [5.0, 3.0, 8.0], [1.0, 4.0, 7.0]]]])
      val=Data(numpy.array([[[[8.0, 7.0, 7.0]], [[1.0, 7.0, 6.0]]], [[[6.0, 8.0, 1.0]], [[8.0, 5.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 6.0]], [[1.0, 4.0, 8.0]]], [[[3.0, 3.0, 4.0]], [[6.0, 2.0, 5.0]]]]))
      arg[2:,:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [2.0, 2.0, 3.0], [3.0, 3.0, 3.0]], [[3.0, 3.0, 5.0], [7.0, 8.0, 2.0], [5.0, 
1.0, 3.0]], [[2.0, 2.0, 7.0], [3.0, 7.0, 7.0], [6.0, 2.0, 5.0]]], [[[7.0, 4.0, 1.0], [4.0, 5.0, 7.0], [6.0, 2.0, 3.0]], [[6.0, 
8.0, 4.0], [4.0, 1.0, 2.0], [5.0, 8.0, 4.0]], [[2.0, 4.0, 8.0], [5.0, 3.0, 5.0], [4.0, 1.0, 6.0]]], [[[4.0, 8.0, 6.0], [1.0, 
7.0, 1.0], [8.0, 7.0, 7.0]], [[2.0, 4.0, 1.0], [5.0, 3.0, 3.0], [1.0, 7.0, 6.0]], [[4.0, 3.0, 4.0], [3.0, 6.0, 3.0], [7.0, 5.0, 
2.0]]], [[[2.0, 5.0, 5.0], [4.0, 8.0, 8.0], [6.0, 8.0, 1.0]], [[8.0, 4.0, 5.0], [6.0, 1.0, 6.0], [8.0, 5.0, 8.0]], [[3.0, 3.0, 
3.0], [1.0, 8.0, 3.0], [7.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 7.0], [2.0, 4.0, 1.0], [2.0, 8.0, 2.0]], 
[[7.0, 5.0, 4.0], [6.0, 6.0, 1.0], [5.0, 3.0, 7.0]], [[6.0, 5.0, 2.0], [2.0, 5.0, 3.0], [1.0, 7.0, 4.0]]], [[[8.0, 3.0, 3.0], 
[6.0, 1.0, 4.0], [2.0, 5.0, 3.0]], [[5.0, 8.0, 8.0], [7.0, 4.0, 1.0], [4.0, 5.0, 4.0]], [[8.0, 1.0, 8.0], [1.0, 1.0, 4.0], 
[7.0, 3.0, 3.0]]], [[[8.0, 4.0, 5.0], [3.0, 7.0, 2.0], [8.0, 8.0, 6.0]], [[8.0, 5.0, 4.0], [3.0, 4.0, 8.0], [1.0, 4.0, 8.0]], 
[[3.0, 2.0, 8.0], [1.0, 7.0, 5.0], [1.0, 6.0, 6.0]]], [[[8.0, 4.0, 3.0], [4.0, 5.0, 4.0], [3.0, 3.0, 4.0]], [[3.0, 7.0, 4.0], 
[7.0, 2.0, 7.0], [6.0, 2.0, 5.0]], [[5.0, 5.0, 7.0], [5.0, 3.0, 8.0], [1.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_URUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 4.0], [8.0, 5.0, 4.0], [6.0, 4.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 4.0, 5.0], [3.0, 
2.0, 8.0]], [[2.0, 3.0, 6.0], [1.0, 7.0, 7.0], [4.0, 4.0, 1.0]]], [[[3.0, 6.0, 6.0], [6.0, 7.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 
7.0, 7.0], [8.0, 4.0, 8.0], [2.0, 7.0, 1.0]], [[4.0, 6.0, 1.0], [6.0, 4.0, 2.0], [1.0, 2.0, 1.0]]], [[[7.0, 7.0, 4.0], [3.0, 
3.0, 4.0], [2.0, 8.0, 8.0]], [[4.0, 1.0, 6.0], [1.0, 2.0, 3.0], [2.0, 6.0, 6.0]], [[2.0, 4.0, 6.0], [1.0, 4.0, 8.0], [5.0, 8.0, 
6.0]]], [[[5.0, 5.0, 2.0], [8.0, 3.0, 8.0], [8.0, 4.0, 4.0]], [[7.0, 7.0, 3.0], [5.0, 5.0, 1.0], [2.0, 6.0, 8.0]], [[8.0, 6.0, 
5.0], [2.0, 4.0, 6.0], [7.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 6.0], [7.0, 7.0, 3.0], [3.0, 6.0, 3.0]], 
[[1.0, 2.0, 2.0], [3.0, 2.0, 4.0], [4.0, 8.0, 8.0]], [[3.0, 4.0, 5.0], [3.0, 1.0, 8.0], [1.0, 7.0, 6.0]]], [[[7.0, 6.0, 5.0], 
[4.0, 3.0, 8.0], [6.0, 6.0, 7.0]], [[8.0, 8.0, 4.0], [6.0, 8.0, 5.0], [2.0, 8.0, 6.0]], [[7.0, 7.0, 7.0], [4.0, 2.0, 7.0], 
[3.0, 4.0, 2.0]]], [[[7.0, 5.0, 2.0], [1.0, 2.0, 3.0], [8.0, 2.0, 5.0]], [[5.0, 6.0, 7.0], [7.0, 3.0, 7.0], [8.0, 1.0, 5.0]], 
[[3.0, 5.0, 3.0], [7.0, 1.0, 2.0], [5.0, 3.0, 1.0]]], [[[5.0, 1.0, 7.0], [6.0, 6.0, 3.0], [2.0, 7.0, 7.0]], [[4.0, 2.0, 4.0], 
[4.0, 3.0, 6.0], [5.0, 3.0, 6.0]], [[5.0, 7.0, 5.0], [5.0, 8.0, 1.0], [2.0, 4.0, 4.0]]]])
      val=Data(numpy.array([[[[1.0, 8.0, 4.0], [4.0, 8.0, 3.0]]], [[[8.0, 7.0, 3.0], [3.0, 3.0, 2.0]]], [[[2.0, 8.0, 5.0], 
[3.0, 5.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 1.0, 6.0], [8.0, 5.0, 1.0]]], [[[4.0, 1.0, 6.0], [5.0, 6.0, 1.0]]], [[[6.0, 
5.0, 6.0], [4.0, 6.0, 6.0]]]]))
      arg[:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 4.0], [8.0, 5.0, 4.0], [6.0, 4.0, 1.0]], [[1.0, 8.0, 4.0], [4.0, 8.0, 3.0], [3.0, 
2.0, 8.0]], [[2.0, 3.0, 6.0], [1.0, 7.0, 7.0], [4.0, 4.0, 1.0]]], [[[3.0, 6.0, 6.0], [6.0, 7.0, 3.0], [2.0, 5.0, 5.0]], [[8.0, 
7.0, 3.0], [3.0, 3.0, 2.0], [2.0, 7.0, 1.0]], [[4.0, 6.0, 1.0], [6.0, 4.0, 2.0], [1.0, 2.0, 1.0]]], [[[7.0, 7.0, 4.0], [3.0, 
3.0, 4.0], [2.0, 8.0, 8.0]], [[2.0, 8.0, 5.0], [3.0, 5.0, 7.0], [2.0, 6.0, 6.0]], [[2.0, 4.0, 6.0], [1.0, 4.0, 8.0], [5.0, 8.0, 
6.0]]], [[[5.0, 5.0, 2.0], [8.0, 3.0, 8.0], [8.0, 4.0, 4.0]], [[7.0, 7.0, 3.0], [5.0, 5.0, 1.0], [2.0, 6.0, 8.0]], [[8.0, 6.0, 
5.0], [2.0, 4.0, 6.0], [7.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 6.0], [7.0, 7.0, 3.0], [3.0, 6.0, 3.0]], 
[[4.0, 1.0, 6.0], [8.0, 5.0, 1.0], [4.0, 8.0, 8.0]], [[3.0, 4.0, 5.0], [3.0, 1.0, 8.0], [1.0, 7.0, 6.0]]], [[[7.0, 6.0, 5.0], 
[4.0, 3.0, 8.0], [6.0, 6.0, 7.0]], [[4.0, 1.0, 6.0], [5.0, 6.0, 1.0], [2.0, 8.0, 6.0]], [[7.0, 7.0, 7.0], [4.0, 2.0, 7.0], 
[3.0, 4.0, 2.0]]], [[[7.0, 5.0, 2.0], [1.0, 2.0, 3.0], [8.0, 2.0, 5.0]], [[6.0, 5.0, 6.0], [4.0, 6.0, 6.0], [8.0, 1.0, 5.0]], 
[[3.0, 5.0, 3.0], [7.0, 1.0, 2.0], [5.0, 3.0, 1.0]]], [[[5.0, 1.0, 7.0], [6.0, 6.0, 3.0], [2.0, 7.0, 7.0]], [[4.0, 2.0, 4.0], 
[4.0, 3.0, 6.0], [5.0, 3.0, 6.0]], [[5.0, 7.0, 5.0], [5.0, 8.0, 1.0], [2.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 8.0], [5.0, 5.0, 7.0], [7.0, 7.0, 8.0]], [[1.0, 4.0, 6.0], [3.0, 5.0, 3.0], [8.0, 
6.0, 5.0]], [[6.0, 6.0, 2.0], [6.0, 7.0, 1.0], [5.0, 7.0, 4.0]]], [[[1.0, 1.0, 8.0], [3.0, 1.0, 3.0], [7.0, 2.0, 3.0]], [[1.0, 
2.0, 4.0], [8.0, 6.0, 6.0], [8.0, 2.0, 8.0]], [[7.0, 6.0, 6.0], [1.0, 7.0, 4.0], [5.0, 7.0, 1.0]]], [[[3.0, 1.0, 5.0], [6.0, 
6.0, 7.0], [7.0, 6.0, 5.0]], [[4.0, 8.0, 1.0], [2.0, 1.0, 8.0], [5.0, 7.0, 4.0]], [[6.0, 6.0, 4.0], [2.0, 7.0, 5.0], [4.0, 8.0, 
3.0]]], [[[5.0, 7.0, 5.0], [1.0, 4.0, 1.0], [2.0, 1.0, 1.0]], [[6.0, 5.0, 3.0], [2.0, 2.0, 6.0], [2.0, 3.0, 7.0]], [[5.0, 4.0, 
3.0], [2.0, 7.0, 3.0], [4.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 3.0, 6.0], [3.0, 4.0, 2.0], [3.0, 2.0, 4.0]], 
[[8.0, 8.0, 5.0], [4.0, 6.0, 3.0], [8.0, 3.0, 3.0]], [[4.0, 3.0, 2.0], [8.0, 3.0, 3.0], [4.0, 4.0, 2.0]]], [[[5.0, 2.0, 1.0], 
[5.0, 3.0, 1.0], [5.0, 6.0, 7.0]], [[6.0, 5.0, 8.0], [1.0, 7.0, 1.0], [1.0, 3.0, 8.0]], [[7.0, 6.0, 4.0], [6.0, 1.0, 8.0], 
[7.0, 7.0, 7.0]]], [[[6.0, 5.0, 5.0], [2.0, 3.0, 4.0], [1.0, 6.0, 4.0]], [[8.0, 5.0, 2.0], [4.0, 4.0, 5.0], [3.0, 6.0, 7.0]], 
[[2.0, 3.0, 7.0], [6.0, 1.0, 3.0], [6.0, 6.0, 3.0]]], [[[4.0, 3.0, 8.0], [5.0, 3.0, 6.0], [6.0, 8.0, 3.0]], [[4.0, 1.0, 7.0], 
[1.0, 7.0, 6.0], [5.0, 1.0, 7.0]], [[1.0, 6.0, 5.0], [8.0, 4.0, 3.0], [4.0, 7.0, 7.0]]]])
      val=Data(numpy.array([[[[8.0, 3.0, 4.0], [6.0, 8.0, 7.0]]], [[[7.0, 3.0, 3.0], [8.0, 2.0, 4.0]]], [[[6.0, 7.0, 1.0], 
[6.0, 2.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 6.0, 5.0], [7.0, 6.0, 7.0]]], [[[2.0, 1.0, 2.0], [4.0, 4.0, 1.0]]], [[[3.0, 
3.0, 4.0], [2.0, 7.0, 8.0]]]]))
      arg[:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 8.0], [5.0, 5.0, 7.0], [7.0, 7.0, 8.0]], [[1.0, 4.0, 6.0], [3.0, 5.0, 3.0], [8.0, 
6.0, 5.0]], [[8.0, 3.0, 4.0], [6.0, 8.0, 7.0], [5.0, 7.0, 4.0]]], [[[1.0, 1.0, 8.0], [3.0, 1.0, 3.0], [7.0, 2.0, 3.0]], [[1.0, 
2.0, 4.0], [8.0, 6.0, 6.0], [8.0, 2.0, 8.0]], [[7.0, 3.0, 3.0], [8.0, 2.0, 4.0], [5.0, 7.0, 1.0]]], [[[3.0, 1.0, 5.0], [6.0, 
6.0, 7.0], [7.0, 6.0, 5.0]], [[4.0, 8.0, 1.0], [2.0, 1.0, 8.0], [5.0, 7.0, 4.0]], [[6.0, 7.0, 1.0], [6.0, 2.0, 8.0], [4.0, 8.0, 
3.0]]], [[[5.0, 7.0, 5.0], [1.0, 4.0, 1.0], [2.0, 1.0, 1.0]], [[6.0, 5.0, 3.0], [2.0, 2.0, 6.0], [2.0, 3.0, 7.0]], [[5.0, 4.0, 
3.0], [2.0, 7.0, 3.0], [4.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 6.0], [3.0, 4.0, 2.0], [3.0, 2.0, 4.0]], 
[[8.0, 8.0, 5.0], [4.0, 6.0, 3.0], [8.0, 3.0, 3.0]], [[3.0, 6.0, 5.0], [7.0, 6.0, 7.0], [4.0, 4.0, 2.0]]], [[[5.0, 2.0, 1.0], 
[5.0, 3.0, 1.0], [5.0, 6.0, 7.0]], [[6.0, 5.0, 8.0], [1.0, 7.0, 1.0], [1.0, 3.0, 8.0]], [[2.0, 1.0, 2.0], [4.0, 4.0, 1.0], 
[7.0, 7.0, 7.0]]], [[[6.0, 5.0, 5.0], [2.0, 3.0, 4.0], [1.0, 6.0, 4.0]], [[8.0, 5.0, 2.0], [4.0, 4.0, 5.0], [3.0, 6.0, 7.0]], 
[[3.0, 3.0, 4.0], [2.0, 7.0, 8.0], [6.0, 6.0, 3.0]]], [[[4.0, 3.0, 8.0], [5.0, 3.0, 6.0], [6.0, 8.0, 3.0]], [[4.0, 1.0, 7.0], 
[1.0, 7.0, 6.0], [5.0, 1.0, 7.0]], [[1.0, 6.0, 5.0], [8.0, 4.0, 3.0], [4.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UCUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 7.0], [3.0, 1.0, 6.0], [8.0, 1.0, 7.0]], [[8.0, 4.0, 2.0], [2.0, 5.0, 5.0], [3.0, 
8.0, 1.0]], [[6.0, 2.0, 1.0], [3.0, 5.0, 1.0], [3.0, 5.0, 8.0]]], [[[1.0, 3.0, 1.0], [2.0, 8.0, 1.0], [3.0, 8.0, 4.0]], [[8.0, 
4.0, 6.0], [7.0, 2.0, 3.0], [1.0, 3.0, 3.0]], [[4.0, 4.0, 4.0], [5.0, 6.0, 2.0], [2.0, 8.0, 8.0]]], [[[2.0, 7.0, 5.0], [4.0, 
3.0, 7.0], [8.0, 3.0, 6.0]], [[7.0, 1.0, 1.0], [2.0, 1.0, 7.0], [5.0, 4.0, 8.0]], [[7.0, 1.0, 1.0], [8.0, 2.0, 7.0], [4.0, 8.0, 
8.0]]], [[[7.0, 6.0, 7.0], [7.0, 2.0, 6.0], [6.0, 8.0, 4.0]], [[3.0, 4.0, 6.0], [6.0, 8.0, 1.0], [5.0, 3.0, 7.0]], [[2.0, 7.0, 
1.0], [2.0, 1.0, 7.0], [7.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 8.0], [7.0, 8.0, 3.0], [4.0, 8.0, 5.0]], 
[[6.0, 7.0, 7.0], [6.0, 6.0, 8.0], [7.0, 2.0, 6.0]], [[3.0, 3.0, 7.0], [3.0, 1.0, 5.0], [4.0, 5.0, 2.0]]], [[[6.0, 5.0, 4.0], 
[6.0, 8.0, 7.0], [7.0, 2.0, 6.0]], [[7.0, 7.0, 2.0], [3.0, 8.0, 7.0], [7.0, 3.0, 6.0]], [[4.0, 1.0, 5.0], [6.0, 5.0, 7.0], 
[4.0, 3.0, 1.0]]], [[[1.0, 3.0, 7.0], [1.0, 3.0, 3.0], [2.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], [5.0, 5.0, 5.0], [2.0, 8.0, 6.0]], 
[[4.0, 3.0, 1.0], [2.0, 3.0, 6.0], [8.0, 2.0, 5.0]]], [[[2.0, 6.0, 7.0], [4.0, 6.0, 1.0], [8.0, 2.0, 1.0]], [[2.0, 4.0, 8.0], 
[2.0, 7.0, 5.0], [6.0, 4.0, 2.0]], [[7.0, 5.0, 4.0], [6.0, 1.0, 8.0], [6.0, 1.0, 5.0]]]])
      val=Data(numpy.array([[[[2.0, 1.0, 6.0], [2.0, 4.0, 4.0]], [[2.0, 6.0, 6.0], [2.0, 5.0, 7.0]], [[6.0, 5.0, 7.0], [5.0, 
5.0, 2.0]]], [[[6.0, 1.0, 3.0], [1.0, 3.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 4.0, 4.0]], [[3.0, 2.0, 7.0], [8.0, 8.0, 8.0]]], 
[[[4.0, 1.0, 1.0], [2.0, 6.0, 3.0]], [[8.0, 7.0, 2.0], [6.0, 8.0, 5.0]], [[1.0, 4.0, 7.0], [4.0, 3.0, 
7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 7.0, 6.0], [3.0, 5.0, 5.0]], [[4.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[5.0, 8.0, 
7.0], [3.0, 1.0, 7.0]]], [[[2.0, 7.0, 7.0], [1.0, 6.0, 3.0]], [[1.0, 2.0, 4.0], [2.0, 8.0, 7.0]], [[4.0, 5.0, 8.0], [7.0, 8.0, 
5.0]]], [[[6.0, 6.0, 2.0], [8.0, 7.0, 7.0]], [[3.0, 1.0, 5.0], [8.0, 8.0, 8.0]], [[3.0, 8.0, 3.0], [3.0, 6.0, 7.0]]]]))
      arg[:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 6.0], [2.0, 4.0, 4.0], [8.0, 1.0, 7.0]], [[2.0, 6.0, 6.0], [2.0, 5.0, 7.0], [3.0, 
8.0, 1.0]], [[6.0, 5.0, 7.0], [5.0, 5.0, 2.0], [3.0, 5.0, 8.0]]], [[[6.0, 1.0, 3.0], [1.0, 3.0, 6.0], [3.0, 8.0, 4.0]], [[4.0, 
1.0, 4.0], [1.0, 4.0, 4.0], [1.0, 3.0, 3.0]], [[3.0, 2.0, 7.0], [8.0, 8.0, 8.0], [2.0, 8.0, 8.0]]], [[[4.0, 1.0, 1.0], [2.0, 
6.0, 3.0], [8.0, 3.0, 6.0]], [[8.0, 7.0, 2.0], [6.0, 8.0, 5.0], [5.0, 4.0, 8.0]], [[1.0, 4.0, 7.0], [4.0, 3.0, 7.0], [4.0, 8.0, 
8.0]]], [[[7.0, 6.0, 7.0], [7.0, 2.0, 6.0], [6.0, 8.0, 4.0]], [[3.0, 4.0, 6.0], [6.0, 8.0, 1.0], [5.0, 3.0, 7.0]], [[2.0, 7.0, 
1.0], [2.0, 1.0, 7.0], [7.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 6.0], [3.0, 5.0, 5.0], [4.0, 8.0, 5.0]], 
[[4.0, 8.0, 4.0], [2.0, 7.0, 2.0], [7.0, 2.0, 6.0]], [[5.0, 8.0, 7.0], [3.0, 1.0, 7.0], [4.0, 5.0, 2.0]]], [[[2.0, 7.0, 7.0], 
[1.0, 6.0, 3.0], [7.0, 2.0, 6.0]], [[1.0, 2.0, 4.0], [2.0, 8.0, 7.0], [7.0, 3.0, 6.0]], [[4.0, 5.0, 8.0], [7.0, 8.0, 5.0], 
[4.0, 3.0, 1.0]]], [[[6.0, 6.0, 2.0], [8.0, 7.0, 7.0], [2.0, 5.0, 4.0]], [[3.0, 1.0, 5.0], [8.0, 8.0, 8.0], [2.0, 8.0, 6.0]], 
[[3.0, 8.0, 3.0], [3.0, 6.0, 7.0], [8.0, 2.0, 5.0]]], [[[2.0, 6.0, 7.0], [4.0, 6.0, 1.0], [8.0, 2.0, 1.0]], [[2.0, 4.0, 8.0], 
[2.0, 7.0, 5.0], [6.0, 4.0, 2.0]], [[7.0, 5.0, 4.0], [6.0, 1.0, 8.0], [6.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCCC(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 6.0], [5.0, 7.0, 3.0], [7.0, 3.0, 7.0]], [[8.0, 2.0, 1.0], [3.0, 6.0, 7.0], [5.0, 
8.0, 7.0]], [[4.0, 6.0, 1.0], [8.0, 6.0, 1.0], [1.0, 7.0, 5.0]]], [[[5.0, 6.0, 6.0], [2.0, 7.0, 3.0], [5.0, 7.0, 6.0]], [[5.0, 
5.0, 6.0], [6.0, 4.0, 6.0], [4.0, 1.0, 7.0]], [[1.0, 7.0, 1.0], [8.0, 8.0, 2.0], [3.0, 1.0, 8.0]]], [[[8.0, 5.0, 6.0], [5.0, 
3.0, 2.0], [1.0, 5.0, 7.0]], [[8.0, 3.0, 1.0], [7.0, 8.0, 7.0], [6.0, 4.0, 7.0]], [[2.0, 1.0, 1.0], [1.0, 3.0, 5.0], [5.0, 8.0, 
1.0]]], [[[3.0, 1.0, 3.0], [8.0, 3.0, 5.0], [2.0, 7.0, 4.0]], [[4.0, 5.0, 5.0], [4.0, 2.0, 3.0], [8.0, 6.0, 1.0]], [[6.0, 1.0, 
1.0], [2.0, 7.0, 5.0], [3.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 1.0], [4.0, 6.0, 4.0], [7.0, 2.0, 7.0]], 
[[3.0, 8.0, 7.0], [8.0, 5.0, 8.0], [2.0, 5.0, 3.0]], [[4.0, 6.0, 6.0], [8.0, 3.0, 6.0], [7.0, 2.0, 1.0]]], [[[4.0, 5.0, 5.0], 
[7.0, 2.0, 5.0], [4.0, 6.0, 6.0]], [[3.0, 7.0, 1.0], [5.0, 2.0, 8.0], [4.0, 3.0, 1.0]], [[2.0, 5.0, 5.0], [5.0, 2.0, 1.0], 
[3.0, 5.0, 5.0]]], [[[2.0, 4.0, 7.0], [7.0, 3.0, 7.0], [5.0, 4.0, 6.0]], [[5.0, 1.0, 5.0], [3.0, 1.0, 6.0], [2.0, 3.0, 6.0]], 
[[8.0, 4.0, 6.0], [3.0, 5.0, 4.0], [1.0, 2.0, 8.0]]], [[[5.0, 5.0, 4.0], [7.0, 4.0, 6.0], [6.0, 6.0, 3.0]], [[5.0, 1.0, 2.0], 
[7.0, 7.0, 2.0], [7.0, 6.0, 3.0]], [[8.0, 3.0, 1.0], [2.0, 6.0, 7.0], [2.0, 6.0, 4.0]]]])
      val=Data(numpy.array([[[[6.0, 3.0, 7.0], [5.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[2.0, 4.0, 1.0], [7.0, 8.0, 2.0], [4.0, 2.0, 
7.0]], [[4.0, 1.0, 8.0], [5.0, 8.0, 5.0], [6.0, 2.0, 3.0]]], [[[5.0, 3.0, 2.0], [3.0, 6.0, 3.0], [3.0, 6.0, 8.0]], [[6.0, 4.0, 
8.0], [7.0, 5.0, 1.0], [5.0, 5.0, 6.0]], [[7.0, 4.0, 6.0], [6.0, 5.0, 2.0], [3.0, 3.0, 4.0]]], [[[4.0, 4.0, 8.0], [8.0, 5.0, 
4.0], [6.0, 3.0, 6.0]], [[7.0, 4.0, 4.0], [1.0, 8.0, 7.0], [4.0, 3.0, 1.0]], [[3.0, 7.0, 4.0], [4.0, 4.0, 6.0], [3.0, 4.0, 
6.0]]], [[[5.0, 4.0, 6.0], [7.0, 3.0, 3.0], [7.0, 7.0, 6.0]], [[8.0, 6.0, 6.0], [3.0, 4.0, 5.0], [4.0, 4.0, 4.0]], [[6.0, 2.0, 
3.0], [3.0, 5.0, 2.0], [1.0, 4.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 8.0], [4.0, 5.0, 5.0], [5.0, 3.0, 8.0]], [[5.0, 1.0, 7.0], [8.0, 6.0, 
1.0], [2.0, 5.0, 1.0]], [[6.0, 4.0, 3.0], [7.0, 8.0, 1.0], [2.0, 5.0, 6.0]]], [[[8.0, 8.0, 7.0], [2.0, 8.0, 3.0], [6.0, 5.0, 
1.0]], [[8.0, 6.0, 8.0], [1.0, 8.0, 5.0], [2.0, 1.0, 5.0]], [[1.0, 3.0, 6.0], [8.0, 3.0, 2.0], [5.0, 6.0, 5.0]]], [[[5.0, 3.0, 
3.0], [6.0, 7.0, 2.0], [6.0, 7.0, 3.0]], [[3.0, 4.0, 8.0], [3.0, 7.0, 1.0], [3.0, 8.0, 2.0]], [[2.0, 2.0, 3.0], [2.0, 5.0, 
4.0], [2.0, 6.0, 3.0]]], [[[6.0, 5.0, 4.0], [8.0, 7.0, 8.0], [3.0, 3.0, 1.0]], [[1.0, 7.0, 5.0], [7.0, 4.0, 2.0], [5.0, 1.0, 
5.0]], [[4.0, 2.0, 1.0], [8.0, 2.0, 1.0], [4.0, 8.0, 6.0]]]]))
      arg[:,:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 7.0], [5.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[2.0, 4.0, 1.0], [7.0, 8.0, 2.0], [4.0, 
2.0, 7.0]], [[4.0, 1.0, 8.0], [5.0, 8.0, 5.0], [6.0, 2.0, 3.0]]], [[[5.0, 3.0, 2.0], [3.0, 6.0, 3.0], [3.0, 6.0, 8.0]], [[6.0, 
4.0, 8.0], [7.0, 5.0, 1.0], [5.0, 5.0, 6.0]], [[7.0, 4.0, 6.0], [6.0, 5.0, 2.0], [3.0, 3.0, 4.0]]], [[[4.0, 4.0, 8.0], [8.0, 
5.0, 4.0], [6.0, 3.0, 6.0]], [[7.0, 4.0, 4.0], [1.0, 8.0, 7.0], [4.0, 3.0, 1.0]], [[3.0, 7.0, 4.0], [4.0, 4.0, 6.0], [3.0, 4.0, 
6.0]]], [[[5.0, 4.0, 6.0], [7.0, 3.0, 3.0], [7.0, 7.0, 6.0]], [[8.0, 6.0, 6.0], [3.0, 4.0, 5.0], [4.0, 4.0, 4.0]], [[6.0, 2.0, 
3.0], [3.0, 5.0, 2.0], [1.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 8.0], [4.0, 5.0, 5.0], [5.0, 3.0, 8.0]], 
[[5.0, 1.0, 7.0], [8.0, 6.0, 1.0], [2.0, 5.0, 1.0]], [[6.0, 4.0, 3.0], [7.0, 8.0, 1.0], [2.0, 5.0, 6.0]]], [[[8.0, 8.0, 7.0], 
[2.0, 8.0, 3.0], [6.0, 5.0, 1.0]], [[8.0, 6.0, 8.0], [1.0, 8.0, 5.0], [2.0, 1.0, 5.0]], [[1.0, 3.0, 6.0], [8.0, 3.0, 2.0], 
[5.0, 6.0, 5.0]]], [[[5.0, 3.0, 3.0], [6.0, 7.0, 2.0], [6.0, 7.0, 3.0]], [[3.0, 4.0, 8.0], [3.0, 7.0, 1.0], [3.0, 8.0, 2.0]], 
[[2.0, 2.0, 3.0], [2.0, 5.0, 4.0], [2.0, 6.0, 3.0]]], [[[6.0, 5.0, 4.0], [8.0, 7.0, 8.0], [3.0, 3.0, 1.0]], [[1.0, 7.0, 5.0], 
[7.0, 4.0, 2.0], [5.0, 1.0, 5.0]], [[4.0, 2.0, 1.0], [8.0, 2.0, 1.0], [4.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CPPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 5.0], [5.0, 7.0, 4.0], [8.0, 3.0, 7.0]], [[3.0, 4.0, 6.0], [4.0, 2.0, 1.0], [4.0, 
2.0, 6.0]], [[6.0, 5.0, 2.0], [1.0, 4.0, 4.0], [5.0, 3.0, 4.0]]], [[[5.0, 4.0, 5.0], [4.0, 8.0, 7.0], [2.0, 5.0, 6.0]], [[3.0, 
3.0, 6.0], [5.0, 4.0, 8.0], [1.0, 7.0, 8.0]], [[4.0, 4.0, 6.0], [5.0, 6.0, 5.0], [1.0, 4.0, 6.0]]], [[[4.0, 2.0, 5.0], [2.0, 
2.0, 7.0], [7.0, 3.0, 5.0]], [[1.0, 3.0, 3.0], [5.0, 2.0, 8.0], [6.0, 2.0, 1.0]], [[8.0, 4.0, 4.0], [4.0, 2.0, 4.0], [2.0, 1.0, 
1.0]]], [[[8.0, 4.0, 8.0], [7.0, 1.0, 5.0], [5.0, 2.0, 2.0]], [[4.0, 8.0, 2.0], [1.0, 2.0, 6.0], [3.0, 3.0, 5.0]], [[6.0, 5.0, 
5.0], [7.0, 1.0, 2.0], [4.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 6.0], [1.0, 1.0, 6.0], [6.0, 5.0, 1.0]], 
[[1.0, 8.0, 6.0], [1.0, 3.0, 4.0], [2.0, 1.0, 6.0]], [[7.0, 8.0, 8.0], [7.0, 3.0, 8.0], [7.0, 5.0, 8.0]]], [[[7.0, 4.0, 5.0], 
[4.0, 2.0, 5.0], [5.0, 6.0, 6.0]], [[7.0, 8.0, 4.0], [4.0, 7.0, 6.0], [4.0, 6.0, 5.0]], [[1.0, 1.0, 6.0], [2.0, 4.0, 8.0], 
[8.0, 3.0, 1.0]]], [[[1.0, 1.0, 7.0], [2.0, 5.0, 8.0], [5.0, 6.0, 1.0]], [[3.0, 4.0, 5.0], [1.0, 6.0, 3.0], [2.0, 1.0, 4.0]], 
[[5.0, 4.0, 4.0], [3.0, 5.0, 7.0], [8.0, 2.0, 3.0]]], [[[2.0, 2.0, 1.0], [2.0, 8.0, 4.0], [1.0, 8.0, 4.0]], [[5.0, 5.0, 3.0], 
[3.0, 3.0, 3.0], [8.0, 5.0, 5.0]], [[7.0, 5.0, 8.0], [1.0, 2.0, 2.0], [4.0, 2.0, 2.0]]]])
      val=Data(numpy.array([[7.0, 1.0, 1.0], [6.0, 2.0, 2.0], [8.0, 5.0, 6.0], [4.0, 7.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 6.0, 4.0], [1.0, 5.0, 6.0], [8.0, 7.0, 4.0], [1.0, 7.0, 8.0]]))
      arg[:,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 5.0], [5.0, 7.0, 4.0], [8.0, 3.0, 7.0]], [[3.0, 4.0, 6.0], [7.0, 1.0, 1.0], [4.0, 
2.0, 6.0]], [[6.0, 5.0, 2.0], [1.0, 4.0, 4.0], [5.0, 3.0, 4.0]]], [[[5.0, 4.0, 5.0], [4.0, 8.0, 7.0], [2.0, 5.0, 6.0]], [[3.0, 
3.0, 6.0], [6.0, 2.0, 2.0], [1.0, 7.0, 8.0]], [[4.0, 4.0, 6.0], [5.0, 6.0, 5.0], [1.0, 4.0, 6.0]]], [[[4.0, 2.0, 5.0], [2.0, 
2.0, 7.0], [7.0, 3.0, 5.0]], [[1.0, 3.0, 3.0], [8.0, 5.0, 6.0], [6.0, 2.0, 1.0]], [[8.0, 4.0, 4.0], [4.0, 2.0, 4.0], [2.0, 1.0, 
1.0]]], [[[8.0, 4.0, 8.0], [7.0, 1.0, 5.0], [5.0, 2.0, 2.0]], [[4.0, 8.0, 2.0], [4.0, 7.0, 6.0], [3.0, 3.0, 5.0]], [[6.0, 5.0, 
5.0], [7.0, 1.0, 2.0], [4.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 6.0], [1.0, 1.0, 6.0], [6.0, 5.0, 1.0]], 
[[1.0, 8.0, 6.0], [8.0, 6.0, 4.0], [2.0, 1.0, 6.0]], [[7.0, 8.0, 8.0], [7.0, 3.0, 8.0], [7.0, 5.0, 8.0]]], [[[7.0, 4.0, 5.0], 
[4.0, 2.0, 5.0], [5.0, 6.0, 6.0]], [[7.0, 8.0, 4.0], [1.0, 5.0, 6.0], [4.0, 6.0, 5.0]], [[1.0, 1.0, 6.0], [2.0, 4.0, 8.0], 
[8.0, 3.0, 1.0]]], [[[1.0, 1.0, 7.0], [2.0, 5.0, 8.0], [5.0, 6.0, 1.0]], [[3.0, 4.0, 5.0], [8.0, 7.0, 4.0], [2.0, 1.0, 4.0]], 
[[5.0, 4.0, 4.0], [3.0, 5.0, 7.0], [8.0, 2.0, 3.0]]], [[[2.0, 2.0, 1.0], [2.0, 8.0, 4.0], [1.0, 8.0, 4.0]], [[5.0, 5.0, 3.0], 
[1.0, 7.0, 8.0], [8.0, 5.0, 5.0]], [[7.0, 5.0, 8.0], [1.0, 2.0, 2.0], [4.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 3.0, 1.0], [1.0, 1.0, 6.0], [5.0, 8.0, 7.0]], [[4.0, 5.0, 6.0], [1.0, 1.0, 8.0], [1.0, 
2.0, 3.0]], [[4.0, 3.0, 1.0], [3.0, 4.0, 5.0], [5.0, 3.0, 4.0]]], [[[8.0, 4.0, 2.0], [8.0, 5.0, 5.0], [6.0, 4.0, 1.0]], [[6.0, 
2.0, 5.0], [1.0, 4.0, 5.0], [3.0, 7.0, 5.0]], [[4.0, 6.0, 3.0], [2.0, 2.0, 7.0], [8.0, 5.0, 8.0]]], [[[8.0, 6.0, 8.0], [1.0, 
2.0, 3.0], [7.0, 5.0, 6.0]], [[1.0, 7.0, 4.0], [4.0, 5.0, 2.0], [2.0, 5.0, 5.0]], [[7.0, 1.0, 4.0], [3.0, 2.0, 5.0], [7.0, 7.0, 
5.0]]], [[[3.0, 5.0, 4.0], [3.0, 2.0, 3.0], [3.0, 5.0, 6.0]], [[3.0, 7.0, 1.0], [5.0, 6.0, 6.0], [3.0, 6.0, 2.0]], [[7.0, 8.0, 
8.0], [6.0, 4.0, 7.0], [7.0, 8.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 5.0, 7.0], [5.0, 7.0, 5.0], [1.0, 8.0, 5.0]], 
[[1.0, 8.0, 7.0], [2.0, 3.0, 2.0], [6.0, 5.0, 1.0]], [[1.0, 4.0, 1.0], [6.0, 5.0, 5.0], [4.0, 4.0, 2.0]]], [[[1.0, 6.0, 8.0], 
[2.0, 2.0, 6.0], [2.0, 3.0, 8.0]], [[3.0, 8.0, 2.0], [7.0, 5.0, 2.0], [1.0, 7.0, 8.0]], [[1.0, 5.0, 1.0], [7.0, 3.0, 6.0], 
[1.0, 2.0, 3.0]]], [[[2.0, 4.0, 4.0], [3.0, 6.0, 2.0], [2.0, 5.0, 1.0]], [[6.0, 8.0, 8.0], [5.0, 3.0, 7.0], [5.0, 5.0, 1.0]], 
[[3.0, 8.0, 2.0], [4.0, 2.0, 7.0], [6.0, 8.0, 8.0]]], [[[2.0, 1.0, 2.0], [4.0, 7.0, 6.0], [6.0, 6.0, 4.0]], [[1.0, 6.0, 6.0], 
[2.0, 1.0, 8.0], [1.0, 7.0, 1.0]], [[5.0, 8.0, 1.0], [5.0, 3.0, 4.0], [5.0, 1.0, 3.0]]]])
      val=Data(numpy.array([[[[6.0, 2.0, 2.0], [6.0, 3.0, 2.0]]], [[[2.0, 8.0, 5.0], [5.0, 3.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 7.0, 4.0], [8.0, 1.0, 5.0]]], [[[5.0, 8.0, 6.0], [2.0, 4.0, 8.0]]]]))
      arg[1:3,1:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 1.0], [1.0, 1.0, 6.0], [5.0, 8.0, 7.0]], [[4.0, 5.0, 6.0], [1.0, 1.0, 8.0], [1.0, 
2.0, 3.0]], [[4.0, 3.0, 1.0], [3.0, 4.0, 5.0], [5.0, 3.0, 4.0]]], [[[8.0, 4.0, 2.0], [8.0, 5.0, 5.0], [6.0, 4.0, 1.0]], [[6.0, 
2.0, 2.0], [6.0, 3.0, 2.0], [3.0, 7.0, 5.0]], [[4.0, 6.0, 3.0], [2.0, 2.0, 7.0], [8.0, 5.0, 8.0]]], [[[8.0, 6.0, 8.0], [1.0, 
2.0, 3.0], [7.0, 5.0, 6.0]], [[2.0, 8.0, 5.0], [5.0, 3.0, 1.0], [2.0, 5.0, 5.0]], [[7.0, 1.0, 4.0], [3.0, 2.0, 5.0], [7.0, 7.0, 
5.0]]], [[[3.0, 5.0, 4.0], [3.0, 2.0, 3.0], [3.0, 5.0, 6.0]], [[3.0, 7.0, 1.0], [5.0, 6.0, 6.0], [3.0, 6.0, 2.0]], [[7.0, 8.0, 
8.0], [6.0, 4.0, 7.0], [7.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 7.0], [5.0, 7.0, 5.0], [1.0, 8.0, 5.0]], 
[[1.0, 8.0, 7.0], [2.0, 3.0, 2.0], [6.0, 5.0, 1.0]], [[1.0, 4.0, 1.0], [6.0, 5.0, 5.0], [4.0, 4.0, 2.0]]], [[[1.0, 6.0, 8.0], 
[2.0, 2.0, 6.0], [2.0, 3.0, 8.0]], [[7.0, 7.0, 4.0], [8.0, 1.0, 5.0], [1.0, 7.0, 8.0]], [[1.0, 5.0, 1.0], [7.0, 3.0, 6.0], 
[1.0, 2.0, 3.0]]], [[[2.0, 4.0, 4.0], [3.0, 6.0, 2.0], [2.0, 5.0, 1.0]], [[5.0, 8.0, 6.0], [2.0, 4.0, 8.0], [5.0, 5.0, 1.0]], 
[[3.0, 8.0, 2.0], [4.0, 2.0, 7.0], [6.0, 8.0, 8.0]]], [[[2.0, 1.0, 2.0], [4.0, 7.0, 6.0], [6.0, 6.0, 4.0]], [[1.0, 6.0, 6.0], 
[2.0, 1.0, 8.0], [1.0, 7.0, 1.0]], [[5.0, 8.0, 1.0], [5.0, 3.0, 4.0], [5.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LUUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 5.0], [1.0, 7.0, 5.0], [6.0, 2.0, 3.0]], [[5.0, 8.0, 8.0], [8.0, 4.0, 4.0], [8.0, 
6.0, 1.0]], [[8.0, 4.0, 7.0], [5.0, 6.0, 5.0], [1.0, 8.0, 6.0]]], [[[8.0, 8.0, 1.0], [7.0, 5.0, 2.0], [4.0, 7.0, 5.0]], [[5.0, 
1.0, 5.0], [2.0, 4.0, 5.0], [8.0, 3.0, 4.0]], [[2.0, 8.0, 5.0], [6.0, 7.0, 7.0], [5.0, 5.0, 7.0]]], [[[5.0, 5.0, 4.0], [6.0, 
7.0, 3.0], [3.0, 5.0, 1.0]], [[8.0, 8.0, 5.0], [2.0, 8.0, 4.0], [1.0, 3.0, 4.0]], [[6.0, 2.0, 3.0], [7.0, 8.0, 2.0], [3.0, 8.0, 
3.0]]], [[[1.0, 3.0, 7.0], [6.0, 6.0, 3.0], [1.0, 8.0, 7.0]], [[7.0, 3.0, 8.0], [2.0, 5.0, 1.0], [2.0, 1.0, 2.0]], [[6.0, 6.0, 
2.0], [6.0, 4.0, 3.0], [1.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 5.0], [1.0, 6.0, 4.0], [1.0, 1.0, 7.0]], 
[[1.0, 1.0, 5.0], [3.0, 5.0, 1.0], [5.0, 6.0, 8.0]], [[5.0, 5.0, 4.0], [6.0, 1.0, 7.0], [3.0, 5.0, 6.0]]], [[[4.0, 7.0, 6.0], 
[4.0, 2.0, 2.0], [6.0, 6.0, 4.0]], [[3.0, 8.0, 8.0], [1.0, 5.0, 8.0], [7.0, 5.0, 4.0]], [[1.0, 6.0, 8.0], [6.0, 2.0, 2.0], 
[1.0, 6.0, 4.0]]], [[[5.0, 7.0, 1.0], [3.0, 6.0, 7.0], [7.0, 3.0, 8.0]], [[2.0, 1.0, 7.0], [4.0, 1.0, 6.0], [1.0, 3.0, 3.0]], 
[[2.0, 3.0, 1.0], [1.0, 3.0, 5.0], [4.0, 2.0, 4.0]]], [[[3.0, 6.0, 4.0], [8.0, 4.0, 6.0], [2.0, 3.0, 6.0]], [[1.0, 8.0, 8.0], 
[1.0, 6.0, 6.0], [1.0, 3.0, 2.0]], [[4.0, 1.0, 2.0], [1.0, 1.0, 5.0], [7.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[[[5.0, 1.0, 4.0], [6.0, 6.0, 6.0]], [[8.0, 6.0, 7.0], [5.0, 7.0, 8.0]]], [[[6.0, 7.0, 7.0], [5.0, 
6.0, 7.0]], [[3.0, 2.0, 7.0], [3.0, 2.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 5.0], [4.0, 5.0, 4.0]], [[6.0, 5.0, 1.0], [8.0, 7.0, 7.0]]], [[[3.0, 6.0, 
1.0], [6.0, 1.0, 5.0]], [[3.0, 3.0, 1.0], [2.0, 5.0, 6.0]]]]))
      arg[2:,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 5.0], [1.0, 7.0, 5.0], [6.0, 2.0, 3.0]], [[5.0, 8.0, 8.0], [8.0, 4.0, 4.0], [8.0, 
6.0, 1.0]], [[8.0, 4.0, 7.0], [5.0, 6.0, 5.0], [1.0, 8.0, 6.0]]], [[[8.0, 8.0, 1.0], [7.0, 5.0, 2.0], [4.0, 7.0, 5.0]], [[5.0, 
1.0, 5.0], [2.0, 4.0, 5.0], [8.0, 3.0, 4.0]], [[2.0, 8.0, 5.0], [6.0, 7.0, 7.0], [5.0, 5.0, 7.0]]], [[[5.0, 1.0, 4.0], [6.0, 
6.0, 6.0], [3.0, 5.0, 1.0]], [[8.0, 6.0, 7.0], [5.0, 7.0, 8.0], [1.0, 3.0, 4.0]], [[6.0, 2.0, 3.0], [7.0, 8.0, 2.0], [3.0, 8.0, 
3.0]]], [[[6.0, 7.0, 7.0], [5.0, 6.0, 7.0], [1.0, 8.0, 7.0]], [[3.0, 2.0, 7.0], [3.0, 2.0, 2.0], [2.0, 1.0, 2.0]], [[6.0, 6.0, 
2.0], [6.0, 4.0, 3.0], [1.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 5.0], [1.0, 6.0, 4.0], [1.0, 1.0, 7.0]], 
[[1.0, 1.0, 5.0], [3.0, 5.0, 1.0], [5.0, 6.0, 8.0]], [[5.0, 5.0, 4.0], [6.0, 1.0, 7.0], [3.0, 5.0, 6.0]]], [[[4.0, 7.0, 6.0], 
[4.0, 2.0, 2.0], [6.0, 6.0, 4.0]], [[3.0, 8.0, 8.0], [1.0, 5.0, 8.0], [7.0, 5.0, 4.0]], [[1.0, 6.0, 8.0], [6.0, 2.0, 2.0], 
[1.0, 6.0, 4.0]]], [[[8.0, 8.0, 5.0], [4.0, 5.0, 4.0], [7.0, 3.0, 8.0]], [[6.0, 5.0, 1.0], [8.0, 7.0, 7.0], [1.0, 3.0, 3.0]], 
[[2.0, 3.0, 1.0], [1.0, 3.0, 5.0], [4.0, 2.0, 4.0]]], [[[3.0, 6.0, 1.0], [6.0, 1.0, 5.0], [2.0, 3.0, 6.0]], [[3.0, 3.0, 1.0], 
[2.0, 5.0, 6.0], [1.0, 3.0, 2.0]], [[4.0, 1.0, 2.0], [1.0, 1.0, 5.0], [7.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PCUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 3.0], [8.0, 1.0, 1.0], [4.0, 6.0, 4.0]], [[5.0, 8.0, 5.0], [6.0, 6.0, 4.0], [5.0, 
3.0, 2.0]], [[5.0, 3.0, 6.0], [7.0, 2.0, 8.0], [1.0, 6.0, 5.0]]], [[[6.0, 3.0, 8.0], [2.0, 3.0, 4.0], [2.0, 7.0, 5.0]], [[3.0, 
1.0, 2.0], [5.0, 8.0, 4.0], [6.0, 2.0, 1.0]], [[6.0, 8.0, 7.0], [2.0, 2.0, 1.0], [1.0, 5.0, 7.0]]], [[[8.0, 5.0, 2.0], [5.0, 
7.0, 7.0], [3.0, 2.0, 8.0]], [[7.0, 5.0, 4.0], [7.0, 2.0, 1.0], [1.0, 7.0, 2.0]], [[8.0, 7.0, 8.0], [4.0, 6.0, 3.0], [4.0, 6.0, 
3.0]]], [[[6.0, 6.0, 8.0], [2.0, 3.0, 4.0], [1.0, 3.0, 2.0]], [[4.0, 2.0, 6.0], [4.0, 7.0, 6.0], [3.0, 8.0, 3.0]], [[3.0, 8.0, 
1.0], [5.0, 1.0, 8.0], [4.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 5.0], [4.0, 5.0, 8.0], [7.0, 2.0, 1.0]], 
[[8.0, 3.0, 6.0], [1.0, 5.0, 7.0], [5.0, 6.0, 4.0]], [[5.0, 2.0, 6.0], [1.0, 8.0, 3.0], [1.0, 2.0, 1.0]]], [[[4.0, 6.0, 1.0], 
[3.0, 8.0, 4.0], [3.0, 3.0, 7.0]], [[5.0, 8.0, 1.0], [2.0, 7.0, 6.0], [4.0, 1.0, 8.0]], [[1.0, 3.0, 1.0], [7.0, 7.0, 4.0], 
[2.0, 2.0, 3.0]]], [[[1.0, 8.0, 6.0], [6.0, 3.0, 7.0], [2.0, 1.0, 4.0]], [[7.0, 8.0, 4.0], [5.0, 7.0, 1.0], [8.0, 3.0, 1.0]], 
[[6.0, 6.0, 5.0], [6.0, 4.0, 2.0], [2.0, 1.0, 4.0]]], [[[5.0, 2.0, 4.0], [8.0, 3.0, 3.0], [3.0, 7.0, 7.0]], [[5.0, 5.0, 8.0], 
[7.0, 7.0, 8.0], [8.0, 3.0, 2.0]], [[4.0, 7.0, 5.0], [7.0, 3.0, 2.0], [7.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[[5.0, 1.0, 6.0], [4.0, 1.0, 1.0]], [[4.0, 3.0, 1.0], [4.0, 2.0, 5.0]], [[1.0, 4.0, 8.0], [8.0, 
5.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 6.0, 4.0], [4.0, 3.0, 5.0]], [[4.0, 4.0, 1.0], [2.0, 7.0, 7.0]], [[8.0, 6.0, 
2.0], [3.0, 1.0, 7.0]]]))
      arg[2,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 3.0], [8.0, 1.0, 1.0], [4.0, 6.0, 4.0]], [[5.0, 8.0, 5.0], [6.0, 6.0, 4.0], [5.0, 
3.0, 2.0]], [[5.0, 3.0, 6.0], [7.0, 2.0, 8.0], [1.0, 6.0, 5.0]]], [[[6.0, 3.0, 8.0], [2.0, 3.0, 4.0], [2.0, 7.0, 5.0]], [[3.0, 
1.0, 2.0], [5.0, 8.0, 4.0], [6.0, 2.0, 1.0]], [[6.0, 8.0, 7.0], [2.0, 2.0, 1.0], [1.0, 5.0, 7.0]]], [[[5.0, 1.0, 6.0], [4.0, 
1.0, 1.0], [3.0, 2.0, 8.0]], [[4.0, 3.0, 1.0], [4.0, 2.0, 5.0], [1.0, 7.0, 2.0]], [[1.0, 4.0, 8.0], [8.0, 5.0, 8.0], [4.0, 6.0, 
3.0]]], [[[6.0, 6.0, 8.0], [2.0, 3.0, 4.0], [1.0, 3.0, 2.0]], [[4.0, 2.0, 6.0], [4.0, 7.0, 6.0], [3.0, 8.0, 3.0]], [[3.0, 8.0, 
1.0], [5.0, 1.0, 8.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 5.0], [4.0, 5.0, 8.0], [7.0, 2.0, 1.0]], 
[[8.0, 3.0, 6.0], [1.0, 5.0, 7.0], [5.0, 6.0, 4.0]], [[5.0, 2.0, 6.0], [1.0, 8.0, 3.0], [1.0, 2.0, 1.0]]], [[[4.0, 6.0, 1.0], 
[3.0, 8.0, 4.0], [3.0, 3.0, 7.0]], [[5.0, 8.0, 1.0], [2.0, 7.0, 6.0], [4.0, 1.0, 8.0]], [[1.0, 3.0, 1.0], [7.0, 7.0, 4.0], 
[2.0, 2.0, 3.0]]], [[[7.0, 6.0, 4.0], [4.0, 3.0, 5.0], [2.0, 1.0, 4.0]], [[4.0, 4.0, 1.0], [2.0, 7.0, 7.0], [8.0, 3.0, 1.0]], 
[[8.0, 6.0, 2.0], [3.0, 1.0, 7.0], [2.0, 1.0, 4.0]]], [[[5.0, 2.0, 4.0], [8.0, 3.0, 3.0], [3.0, 7.0, 7.0]], [[5.0, 5.0, 8.0], 
[7.0, 7.0, 8.0], [8.0, 3.0, 2.0]], [[4.0, 7.0, 5.0], [7.0, 3.0, 2.0], [7.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UPUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 7.0], [1.0, 5.0, 3.0], [1.0, 5.0, 7.0]], [[5.0, 6.0, 5.0], [3.0, 4.0, 2.0], [4.0, 
2.0, 8.0]], [[1.0, 4.0, 6.0], [2.0, 5.0, 7.0], [6.0, 3.0, 5.0]]], [[[3.0, 6.0, 5.0], [8.0, 8.0, 7.0], [6.0, 8.0, 6.0]], [[3.0, 
2.0, 4.0], [8.0, 6.0, 6.0], [5.0, 3.0, 4.0]], [[3.0, 3.0, 4.0], [6.0, 4.0, 5.0], [7.0, 3.0, 8.0]]], [[[2.0, 4.0, 4.0], [5.0, 
5.0, 1.0], [1.0, 1.0, 2.0]], [[3.0, 5.0, 8.0], [5.0, 4.0, 5.0], [5.0, 5.0, 8.0]], [[4.0, 3.0, 6.0], [3.0, 6.0, 3.0], [2.0, 1.0, 
7.0]]], [[[7.0, 3.0, 5.0], [4.0, 6.0, 7.0], [5.0, 1.0, 1.0]], [[4.0, 6.0, 6.0], [4.0, 5.0, 2.0], [4.0, 5.0, 6.0]], [[1.0, 3.0, 
5.0], [6.0, 1.0, 1.0], [3.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 2.0], [4.0, 6.0, 2.0], [1.0, 6.0, 8.0]], 
[[3.0, 5.0, 3.0], [1.0, 7.0, 1.0], [8.0, 3.0, 1.0]], [[6.0, 4.0, 5.0], [8.0, 3.0, 4.0], [5.0, 2.0, 3.0]]], [[[5.0, 8.0, 6.0], 
[6.0, 5.0, 5.0], [1.0, 7.0, 2.0]], [[2.0, 6.0, 6.0], [2.0, 2.0, 2.0], [8.0, 6.0, 6.0]], [[7.0, 5.0, 2.0], [3.0, 2.0, 1.0], 
[6.0, 6.0, 4.0]]], [[[3.0, 8.0, 4.0], [3.0, 6.0, 2.0], [6.0, 4.0, 5.0]], [[4.0, 4.0, 5.0], [7.0, 6.0, 1.0], [2.0, 3.0, 4.0]], 
[[8.0, 7.0, 3.0], [8.0, 1.0, 5.0], [1.0, 4.0, 6.0]]], [[[7.0, 3.0, 1.0], [8.0, 2.0, 7.0], [5.0, 2.0, 8.0]], [[7.0, 7.0, 5.0], 
[5.0, 4.0, 8.0], [2.0, 1.0, 2.0]], [[5.0, 3.0, 4.0], [2.0, 5.0, 1.0], [2.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[[3.0, 1.0, 5.0], [3.0, 5.0, 3.0]], [[7.0, 1.0, 1.0], [5.0, 6.0, 3.0]], [[8.0, 8.0, 3.0], [2.0, 
6.0, 3.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[7.0, 1.0, 1.0], [6.0, 2.0, 1.0]], [[2.0, 8.0, 3.0], [5.0, 8.0, 5.0]], [[6.0, 7.0, 
4.0], [1.0, 1.0, 7.0]]]))
      arg[:3,1,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 7.0], [1.0, 5.0, 3.0], [1.0, 5.0, 7.0]], [[3.0, 1.0, 5.0], [3.0, 5.0, 3.0], [4.0, 
2.0, 8.0]], [[1.0, 4.0, 6.0], [2.0, 5.0, 7.0], [6.0, 3.0, 5.0]]], [[[3.0, 6.0, 5.0], [8.0, 8.0, 7.0], [6.0, 8.0, 6.0]], [[7.0, 
1.0, 1.0], [5.0, 6.0, 3.0], [5.0, 3.0, 4.0]], [[3.0, 3.0, 4.0], [6.0, 4.0, 5.0], [7.0, 3.0, 8.0]]], [[[2.0, 4.0, 4.0], [5.0, 
5.0, 1.0], [1.0, 1.0, 2.0]], [[8.0, 8.0, 3.0], [2.0, 6.0, 3.0], [5.0, 5.0, 8.0]], [[4.0, 3.0, 6.0], [3.0, 6.0, 3.0], [2.0, 1.0, 
7.0]]], [[[7.0, 3.0, 5.0], [4.0, 6.0, 7.0], [5.0, 1.0, 1.0]], [[4.0, 6.0, 6.0], [4.0, 5.0, 2.0], [4.0, 5.0, 6.0]], [[1.0, 3.0, 
5.0], [6.0, 1.0, 1.0], [3.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 2.0], [4.0, 6.0, 2.0], [1.0, 6.0, 8.0]], 
[[7.0, 1.0, 1.0], [6.0, 2.0, 1.0], [8.0, 3.0, 1.0]], [[6.0, 4.0, 5.0], [8.0, 3.0, 4.0], [5.0, 2.0, 3.0]]], [[[5.0, 8.0, 6.0], 
[6.0, 5.0, 5.0], [1.0, 7.0, 2.0]], [[2.0, 8.0, 3.0], [5.0, 8.0, 5.0], [8.0, 6.0, 6.0]], [[7.0, 5.0, 2.0], [3.0, 2.0, 1.0], 
[6.0, 6.0, 4.0]]], [[[3.0, 8.0, 4.0], [3.0, 6.0, 2.0], [6.0, 4.0, 5.0]], [[6.0, 7.0, 4.0], [1.0, 1.0, 7.0], [2.0, 3.0, 4.0]], 
[[8.0, 7.0, 3.0], [8.0, 1.0, 5.0], [1.0, 4.0, 6.0]]], [[[7.0, 3.0, 1.0], [8.0, 2.0, 7.0], [5.0, 2.0, 8.0]], [[7.0, 7.0, 5.0], 
[5.0, 4.0, 8.0], [2.0, 1.0, 2.0]], [[5.0, 3.0, 4.0], [2.0, 5.0, 1.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RLCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 8.0], [3.0, 8.0, 8.0], [4.0, 8.0, 3.0]], [[6.0, 1.0, 1.0], [5.0, 3.0, 1.0], [6.0, 
4.0, 3.0]], [[1.0, 4.0, 5.0], [3.0, 3.0, 3.0], [5.0, 3.0, 8.0]]], [[[1.0, 8.0, 3.0], [7.0, 4.0, 8.0], [6.0, 1.0, 8.0]], [[4.0, 
5.0, 6.0], [7.0, 5.0, 7.0], [1.0, 8.0, 2.0]], [[2.0, 5.0, 7.0], [4.0, 3.0, 8.0], [6.0, 7.0, 3.0]]], [[[3.0, 2.0, 3.0], [7.0, 
3.0, 2.0], [5.0, 4.0, 8.0]], [[4.0, 5.0, 7.0], [7.0, 2.0, 2.0], [2.0, 6.0, 7.0]], [[8.0, 1.0, 8.0], [6.0, 8.0, 6.0], [2.0, 3.0, 
1.0]]], [[[7.0, 7.0, 2.0], [7.0, 3.0, 1.0], [4.0, 6.0, 2.0]], [[6.0, 6.0, 2.0], [1.0, 4.0, 7.0], [6.0, 1.0, 4.0]], [[8.0, 3.0, 
5.0], [2.0, 8.0, 8.0], [4.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 5.0], [6.0, 7.0, 8.0], [1.0, 8.0, 5.0]], 
[[3.0, 1.0, 3.0], [5.0, 4.0, 1.0], [3.0, 6.0, 6.0]], [[3.0, 4.0, 2.0], [4.0, 8.0, 4.0], [8.0, 2.0, 5.0]]], [[[4.0, 1.0, 5.0], 
[5.0, 4.0, 4.0], [1.0, 1.0, 1.0]], [[8.0, 1.0, 8.0], [5.0, 2.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 6.0, 5.0], [5.0, 2.0, 2.0], 
[5.0, 4.0, 3.0]]], [[[3.0, 6.0, 7.0], [7.0, 1.0, 1.0], [7.0, 4.0, 5.0]], [[2.0, 1.0, 3.0], [6.0, 4.0, 5.0], [3.0, 3.0, 4.0]], 
[[3.0, 2.0, 7.0], [4.0, 2.0, 8.0], [1.0, 7.0, 2.0]]], [[[7.0, 6.0, 4.0], [6.0, 2.0, 6.0], [3.0, 6.0, 1.0]], [[7.0, 8.0, 3.0], 
[5.0, 2.0, 8.0], [6.0, 4.0, 7.0]], [[5.0, 7.0, 8.0], [6.0, 6.0, 1.0], [4.0, 3.0, 8.0]]]])
      val=Data(numpy.array([[[[5.0, 1.0, 7.0], [6.0, 2.0, 3.0], [1.0, 2.0, 7.0]]], [[[7.0, 2.0, 1.0], [7.0, 2.0, 1.0], [8.0, 
7.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 3.0, 7.0], [7.0, 4.0, 4.0], [7.0, 4.0, 5.0]]], [[[3.0, 1.0, 1.0], [5.0, 8.0, 
1.0], [8.0, 3.0, 7.0]]]]))
      arg[1:3,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 8.0], [3.0, 8.0, 8.0], [4.0, 8.0, 3.0]], [[6.0, 1.0, 1.0], [5.0, 3.0, 1.0], [6.0, 
4.0, 3.0]], [[1.0, 4.0, 5.0], [3.0, 3.0, 3.0], [5.0, 3.0, 8.0]]], [[[1.0, 8.0, 3.0], [7.0, 4.0, 8.0], [6.0, 1.0, 8.0]], [[4.0, 
5.0, 6.0], [7.0, 5.0, 7.0], [1.0, 8.0, 2.0]], [[5.0, 1.0, 7.0], [6.0, 2.0, 3.0], [1.0, 2.0, 7.0]]], [[[3.0, 2.0, 3.0], [7.0, 
3.0, 2.0], [5.0, 4.0, 8.0]], [[4.0, 5.0, 7.0], [7.0, 2.0, 2.0], [2.0, 6.0, 7.0]], [[7.0, 2.0, 1.0], [7.0, 2.0, 1.0], [8.0, 7.0, 
8.0]]], [[[7.0, 7.0, 2.0], [7.0, 3.0, 1.0], [4.0, 6.0, 2.0]], [[6.0, 6.0, 2.0], [1.0, 4.0, 7.0], [6.0, 1.0, 4.0]], [[8.0, 3.0, 
5.0], [2.0, 8.0, 8.0], [4.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 5.0], [6.0, 7.0, 8.0], [1.0, 8.0, 5.0]], 
[[3.0, 1.0, 3.0], [5.0, 4.0, 1.0], [3.0, 6.0, 6.0]], [[3.0, 4.0, 2.0], [4.0, 8.0, 4.0], [8.0, 2.0, 5.0]]], [[[4.0, 1.0, 5.0], 
[5.0, 4.0, 4.0], [1.0, 1.0, 1.0]], [[8.0, 1.0, 8.0], [5.0, 2.0, 4.0], [6.0, 1.0, 1.0]], [[1.0, 3.0, 7.0], [7.0, 4.0, 4.0], 
[7.0, 4.0, 5.0]]], [[[3.0, 6.0, 7.0], [7.0, 1.0, 1.0], [7.0, 4.0, 5.0]], [[2.0, 1.0, 3.0], [6.0, 4.0, 5.0], [3.0, 3.0, 4.0]], 
[[3.0, 1.0, 1.0], [5.0, 8.0, 1.0], [8.0, 3.0, 7.0]]], [[[7.0, 6.0, 4.0], [6.0, 2.0, 6.0], [3.0, 6.0, 1.0]], [[7.0, 8.0, 3.0], 
[5.0, 2.0, 8.0], [6.0, 4.0, 7.0]], [[5.0, 7.0, 8.0], [6.0, 6.0, 1.0], [4.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UUUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 7.0], [4.0, 1.0, 6.0], [2.0, 4.0, 6.0]], [[6.0, 7.0, 7.0], [7.0, 3.0, 2.0], [1.0, 
1.0, 4.0]], [[7.0, 3.0, 4.0], [8.0, 6.0, 4.0], [1.0, 3.0, 6.0]]], [[[4.0, 5.0, 3.0], [8.0, 3.0, 4.0], [8.0, 6.0, 8.0]], [[6.0, 
2.0, 2.0], [7.0, 1.0, 3.0], [3.0, 7.0, 7.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 2.0], [5.0, 5.0, 2.0]]], [[[8.0, 4.0, 3.0], [8.0, 
5.0, 5.0], [1.0, 7.0, 1.0]], [[1.0, 8.0, 1.0], [3.0, 8.0, 6.0], [5.0, 7.0, 7.0]], [[5.0, 4.0, 4.0], [4.0, 5.0, 8.0], [6.0, 2.0, 
1.0]]], [[[7.0, 8.0, 1.0], [7.0, 7.0, 6.0], [4.0, 8.0, 6.0]], [[5.0, 1.0, 5.0], [7.0, 8.0, 2.0], [1.0, 1.0, 4.0]], [[8.0, 3.0, 
2.0], [6.0, 8.0, 8.0], [4.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 6.0], [4.0, 3.0, 7.0], [3.0, 6.0, 3.0]], 
[[8.0, 4.0, 1.0], [8.0, 7.0, 1.0], [7.0, 1.0, 1.0]], [[6.0, 3.0, 3.0], [3.0, 5.0, 4.0], [8.0, 5.0, 6.0]]], [[[6.0, 2.0, 2.0], 
[1.0, 6.0, 4.0], [4.0, 8.0, 2.0]], [[3.0, 1.0, 3.0], [8.0, 6.0, 4.0], [6.0, 2.0, 6.0]], [[6.0, 6.0, 7.0], [2.0, 8.0, 6.0], 
[3.0, 1.0, 3.0]]], [[[5.0, 8.0, 4.0], [5.0, 2.0, 8.0], [6.0, 4.0, 5.0]], [[5.0, 1.0, 6.0], [4.0, 7.0, 6.0], [7.0, 7.0, 8.0]], 
[[4.0, 4.0, 7.0], [4.0, 1.0, 6.0], [6.0, 8.0, 5.0]]], [[[5.0, 5.0, 8.0], [4.0, 1.0, 4.0], [8.0, 4.0, 6.0]], [[1.0, 5.0, 4.0], 
[6.0, 2.0, 4.0], [6.0, 7.0, 8.0]], [[8.0, 3.0, 3.0], [1.0, 2.0, 8.0], [3.0, 2.0, 4.0]]]])
      val=Data(numpy.array([[[[4.0, 4.0, 5.0], [8.0, 8.0, 5.0]], [[8.0, 1.0, 1.0], [1.0, 3.0, 8.0]]], [[[1.0, 8.0, 7.0], [1.0, 
3.0, 2.0]], [[1.0, 3.0, 8.0], [1.0, 3.0, 4.0]]], [[[7.0, 6.0, 7.0], [4.0, 3.0, 8.0]], [[1.0, 2.0, 8.0], [1.0, 8.0, 
1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 4.0, 7.0], [3.0, 6.0, 5.0]], [[3.0, 3.0, 7.0], [5.0, 7.0, 1.0]]], [[[1.0, 5.0, 
4.0], [5.0, 2.0, 4.0]], [[3.0, 1.0, 8.0], [1.0, 8.0, 2.0]]], [[[7.0, 5.0, 4.0], [4.0, 6.0, 2.0]], [[5.0, 5.0, 8.0], [8.0, 5.0, 
5.0]]]]))
      arg[:3,:2,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 5.0], [8.0, 8.0, 5.0], [2.0, 4.0, 6.0]], [[8.0, 1.0, 1.0], [1.0, 3.0, 8.0], [1.0, 
1.0, 4.0]], [[7.0, 3.0, 4.0], [8.0, 6.0, 4.0], [1.0, 3.0, 6.0]]], [[[1.0, 8.0, 7.0], [1.0, 3.0, 2.0], [8.0, 6.0, 8.0]], [[1.0, 
3.0, 8.0], [1.0, 3.0, 4.0], [3.0, 7.0, 7.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 2.0], [5.0, 5.0, 2.0]]], [[[7.0, 6.0, 7.0], [4.0, 
3.0, 8.0], [1.0, 7.0, 1.0]], [[1.0, 2.0, 8.0], [1.0, 8.0, 1.0], [5.0, 7.0, 7.0]], [[5.0, 4.0, 4.0], [4.0, 5.0, 8.0], [6.0, 2.0, 
1.0]]], [[[7.0, 8.0, 1.0], [7.0, 7.0, 6.0], [4.0, 8.0, 6.0]], [[5.0, 1.0, 5.0], [7.0, 8.0, 2.0], [1.0, 1.0, 4.0]], [[8.0, 3.0, 
2.0], [6.0, 8.0, 8.0], [4.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 7.0], [3.0, 6.0, 5.0], [3.0, 6.0, 3.0]], 
[[3.0, 3.0, 7.0], [5.0, 7.0, 1.0], [7.0, 1.0, 1.0]], [[6.0, 3.0, 3.0], [3.0, 5.0, 4.0], [8.0, 5.0, 6.0]]], [[[1.0, 5.0, 4.0], 
[5.0, 2.0, 4.0], [4.0, 8.0, 2.0]], [[3.0, 1.0, 8.0], [1.0, 8.0, 2.0], [6.0, 2.0, 6.0]], [[6.0, 6.0, 7.0], [2.0, 8.0, 6.0], 
[3.0, 1.0, 3.0]]], [[[7.0, 5.0, 4.0], [4.0, 6.0, 2.0], [6.0, 4.0, 5.0]], [[5.0, 5.0, 8.0], [8.0, 5.0, 5.0], [7.0, 7.0, 8.0]], 
[[4.0, 4.0, 7.0], [4.0, 1.0, 6.0], [6.0, 8.0, 5.0]]], [[[5.0, 5.0, 8.0], [4.0, 1.0, 4.0], [8.0, 4.0, 6.0]], [[1.0, 5.0, 4.0], 
[6.0, 2.0, 4.0], [6.0, 7.0, 8.0]], [[8.0, 3.0, 3.0], [1.0, 2.0, 8.0], [3.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RPLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 1.0], [7.0, 3.0, 8.0], [3.0, 8.0, 8.0]], [[3.0, 3.0, 1.0], [6.0, 4.0, 5.0], [2.0, 
7.0, 4.0]], [[1.0, 7.0, 2.0], [6.0, 7.0, 3.0], [7.0, 3.0, 5.0]]], [[[5.0, 1.0, 6.0], [6.0, 7.0, 8.0], [3.0, 2.0, 8.0]], [[1.0, 
7.0, 1.0], [8.0, 7.0, 6.0], [3.0, 5.0, 2.0]], [[5.0, 1.0, 2.0], [1.0, 2.0, 3.0], [7.0, 2.0, 4.0]]], [[[6.0, 1.0, 4.0], [5.0, 
5.0, 6.0], [8.0, 2.0, 1.0]], [[4.0, 8.0, 8.0], [5.0, 8.0, 7.0], [6.0, 6.0, 8.0]], [[6.0, 1.0, 1.0], [7.0, 2.0, 1.0], [4.0, 2.0, 
8.0]]], [[[8.0, 8.0, 3.0], [5.0, 7.0, 7.0], [4.0, 6.0, 1.0]], [[3.0, 2.0, 3.0], [1.0, 2.0, 4.0], [7.0, 2.0, 7.0]], [[6.0, 8.0, 
5.0], [4.0, 5.0, 1.0], [4.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 1.0], [2.0, 6.0, 8.0], [4.0, 5.0, 5.0]], 
[[6.0, 6.0, 7.0], [4.0, 1.0, 4.0], [5.0, 8.0, 8.0]], [[8.0, 3.0, 7.0], [7.0, 6.0, 1.0], [3.0, 5.0, 7.0]]], [[[2.0, 8.0, 4.0], 
[4.0, 4.0, 8.0], [3.0, 6.0, 1.0]], [[5.0, 8.0, 3.0], [3.0, 8.0, 8.0], [6.0, 7.0, 4.0]], [[1.0, 8.0, 4.0], [3.0, 5.0, 4.0], 
[3.0, 6.0, 8.0]]], [[[3.0, 2.0, 4.0], [6.0, 2.0, 2.0], [8.0, 8.0, 5.0]], [[5.0, 8.0, 7.0], [4.0, 5.0, 6.0], [1.0, 2.0, 1.0]], 
[[3.0, 1.0, 1.0], [7.0, 6.0, 3.0], [6.0, 4.0, 3.0]]], [[[2.0, 1.0, 2.0], [2.0, 5.0, 2.0], [7.0, 5.0, 4.0]], [[4.0, 5.0, 4.0], 
[1.0, 1.0, 4.0], [3.0, 6.0, 4.0]], [[2.0, 8.0, 8.0], [8.0, 4.0, 4.0], [7.0, 6.0, 7.0]]]])
      val=Data(numpy.array([[[8.0, 4.0, 4.0]], [[8.0, 8.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[2.0, 8.0, 7.0]], [[8.0, 6.0, 6.0]]]))
      arg[1:3,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 1.0], [7.0, 3.0, 8.0], [3.0, 8.0, 8.0]], [[3.0, 3.0, 1.0], [6.0, 4.0, 5.0], [2.0, 
7.0, 4.0]], [[1.0, 7.0, 2.0], [6.0, 7.0, 3.0], [7.0, 3.0, 5.0]]], [[[5.0, 1.0, 6.0], [6.0, 7.0, 8.0], [3.0, 2.0, 8.0]], [[1.0, 
7.0, 1.0], [8.0, 7.0, 6.0], [8.0, 4.0, 4.0]], [[5.0, 1.0, 2.0], [1.0, 2.0, 3.0], [7.0, 2.0, 4.0]]], [[[6.0, 1.0, 4.0], [5.0, 
5.0, 6.0], [8.0, 2.0, 1.0]], [[4.0, 8.0, 8.0], [5.0, 8.0, 7.0], [8.0, 8.0, 2.0]], [[6.0, 1.0, 1.0], [7.0, 2.0, 1.0], [4.0, 2.0, 
8.0]]], [[[8.0, 8.0, 3.0], [5.0, 7.0, 7.0], [4.0, 6.0, 1.0]], [[3.0, 2.0, 3.0], [1.0, 2.0, 4.0], [7.0, 2.0, 7.0]], [[6.0, 8.0, 
5.0], [4.0, 5.0, 1.0], [4.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 1.0], [2.0, 6.0, 8.0], [4.0, 5.0, 5.0]], 
[[6.0, 6.0, 7.0], [4.0, 1.0, 4.0], [5.0, 8.0, 8.0]], [[8.0, 3.0, 7.0], [7.0, 6.0, 1.0], [3.0, 5.0, 7.0]]], [[[2.0, 8.0, 4.0], 
[4.0, 4.0, 8.0], [3.0, 6.0, 1.0]], [[5.0, 8.0, 3.0], [3.0, 8.0, 8.0], [2.0, 8.0, 7.0]], [[1.0, 8.0, 4.0], [3.0, 5.0, 4.0], 
[3.0, 6.0, 8.0]]], [[[3.0, 2.0, 4.0], [6.0, 2.0, 2.0], [8.0, 8.0, 5.0]], [[5.0, 8.0, 7.0], [4.0, 5.0, 6.0], [8.0, 6.0, 6.0]], 
[[3.0, 1.0, 1.0], [7.0, 6.0, 3.0], [6.0, 4.0, 3.0]]], [[[2.0, 1.0, 2.0], [2.0, 5.0, 2.0], [7.0, 5.0, 4.0]], [[4.0, 5.0, 4.0], 
[1.0, 1.0, 4.0], [3.0, 6.0, 4.0]], [[2.0, 8.0, 8.0], [8.0, 4.0, 4.0], [7.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PUPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 2.0], [7.0, 7.0, 7.0], [7.0, 4.0, 2.0]], [[6.0, 6.0, 4.0], [8.0, 2.0, 6.0], [8.0, 
7.0, 3.0]], [[6.0, 1.0, 1.0], [5.0, 8.0, 8.0], [8.0, 2.0, 5.0]]], [[[5.0, 4.0, 1.0], [3.0, 1.0, 6.0], [7.0, 4.0, 8.0]], [[2.0, 
3.0, 3.0], [1.0, 5.0, 8.0], [5.0, 5.0, 1.0]], [[2.0, 1.0, 3.0], [1.0, 4.0, 6.0], [1.0, 7.0, 8.0]]], [[[4.0, 3.0, 8.0], [1.0, 
7.0, 6.0], [8.0, 4.0, 7.0]], [[5.0, 7.0, 8.0], [7.0, 3.0, 3.0], [8.0, 7.0, 6.0]], [[8.0, 1.0, 5.0], [6.0, 5.0, 2.0], [2.0, 4.0, 
6.0]]], [[[6.0, 4.0, 6.0], [1.0, 2.0, 8.0], [6.0, 8.0, 6.0]], [[6.0, 6.0, 6.0], [5.0, 1.0, 2.0], [8.0, 1.0, 3.0]], [[8.0, 5.0, 
7.0], [6.0, 6.0, 4.0], [6.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 2.0], [6.0, 1.0, 2.0], [4.0, 3.0, 8.0]], 
[[2.0, 3.0, 5.0], [6.0, 5.0, 1.0], [2.0, 2.0, 5.0]], [[1.0, 5.0, 5.0], [7.0, 6.0, 5.0], [4.0, 1.0, 8.0]]], [[[2.0, 1.0, 4.0], 
[7.0, 6.0, 1.0], [4.0, 3.0, 1.0]], [[4.0, 3.0, 2.0], [1.0, 3.0, 5.0], [5.0, 5.0, 4.0]], [[2.0, 6.0, 2.0], [2.0, 5.0, 2.0], 
[3.0, 7.0, 1.0]]], [[[1.0, 7.0, 6.0], [3.0, 8.0, 3.0], [5.0, 8.0, 8.0]], [[3.0, 2.0, 1.0], [8.0, 1.0, 6.0], [1.0, 8.0, 1.0]], 
[[3.0, 1.0, 1.0], [2.0, 2.0, 7.0], [8.0, 4.0, 6.0]]], [[[7.0, 2.0, 5.0], [5.0, 5.0, 7.0], [6.0, 1.0, 7.0]], [[2.0, 7.0, 7.0], 
[6.0, 1.0, 2.0], [7.0, 6.0, 3.0]], [[6.0, 2.0, 4.0], [3.0, 6.0, 5.0], [2.0, 2.0, 8.0]]]])
      val=Data(numpy.array([[7.0, 7.0, 8.0], [6.0, 8.0, 5.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 7.0, 6.0], [2.0, 4.0, 2.0]]))
      arg[2,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 2.0], [7.0, 7.0, 7.0], [7.0, 4.0, 2.0]], [[6.0, 6.0, 4.0], [8.0, 2.0, 6.0], [8.0, 
7.0, 3.0]], [[6.0, 1.0, 1.0], [5.0, 8.0, 8.0], [8.0, 2.0, 5.0]]], [[[5.0, 4.0, 1.0], [3.0, 1.0, 6.0], [7.0, 4.0, 8.0]], [[2.0, 
3.0, 3.0], [1.0, 5.0, 8.0], [5.0, 5.0, 1.0]], [[2.0, 1.0, 3.0], [1.0, 4.0, 6.0], [1.0, 7.0, 8.0]]], [[[4.0, 3.0, 8.0], [7.0, 
7.0, 8.0], [8.0, 4.0, 7.0]], [[5.0, 7.0, 8.0], [6.0, 8.0, 5.0], [8.0, 7.0, 6.0]], [[8.0, 1.0, 5.0], [6.0, 5.0, 2.0], [2.0, 4.0, 
6.0]]], [[[6.0, 4.0, 6.0], [1.0, 2.0, 8.0], [6.0, 8.0, 6.0]], [[6.0, 6.0, 6.0], [5.0, 1.0, 2.0], [8.0, 1.0, 3.0]], [[8.0, 5.0, 
7.0], [6.0, 6.0, 4.0], [6.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 2.0], [6.0, 1.0, 2.0], [4.0, 3.0, 8.0]], 
[[2.0, 3.0, 5.0], [6.0, 5.0, 1.0], [2.0, 2.0, 5.0]], [[1.0, 5.0, 5.0], [7.0, 6.0, 5.0], [4.0, 1.0, 8.0]]], [[[2.0, 1.0, 4.0], 
[7.0, 6.0, 1.0], [4.0, 3.0, 1.0]], [[4.0, 3.0, 2.0], [1.0, 3.0, 5.0], [5.0, 5.0, 4.0]], [[2.0, 6.0, 2.0], [2.0, 5.0, 2.0], 
[3.0, 7.0, 1.0]]], [[[1.0, 7.0, 6.0], [3.0, 7.0, 6.0], [5.0, 8.0, 8.0]], [[3.0, 2.0, 1.0], [2.0, 4.0, 2.0], [1.0, 8.0, 1.0]], 
[[3.0, 1.0, 1.0], [2.0, 2.0, 7.0], [8.0, 4.0, 6.0]]], [[[7.0, 2.0, 5.0], [5.0, 5.0, 7.0], [6.0, 1.0, 7.0]], [[2.0, 7.0, 7.0], 
[6.0, 1.0, 2.0], [7.0, 6.0, 3.0]], [[6.0, 2.0, 4.0], [3.0, 6.0, 5.0], [2.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 2.0], [4.0, 3.0, 4.0], [7.0, 7.0, 5.0]], [[1.0, 7.0, 6.0], [2.0, 2.0, 6.0], [1.0, 
1.0, 5.0]], [[1.0, 1.0, 5.0], [6.0, 2.0, 1.0], [2.0, 8.0, 6.0]]], [[[7.0, 5.0, 3.0], [8.0, 2.0, 6.0], [3.0, 3.0, 2.0]], [[5.0, 
8.0, 8.0], [1.0, 8.0, 2.0], [5.0, 5.0, 1.0]], [[4.0, 7.0, 5.0], [6.0, 7.0, 5.0], [7.0, 1.0, 4.0]]], [[[1.0, 2.0, 4.0], [2.0, 
4.0, 6.0], [1.0, 8.0, 5.0]], [[4.0, 6.0, 8.0], [7.0, 8.0, 1.0], [8.0, 2.0, 1.0]], [[6.0, 4.0, 3.0], [2.0, 3.0, 1.0], [4.0, 2.0, 
2.0]]], [[[3.0, 2.0, 4.0], [2.0, 8.0, 2.0], [8.0, 2.0, 7.0]], [[4.0, 4.0, 8.0], [8.0, 2.0, 8.0], [2.0, 7.0, 8.0]], [[7.0, 5.0, 
2.0], [2.0, 1.0, 8.0], [8.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 6.0], [2.0, 4.0, 7.0], [7.0, 6.0, 8.0]], 
[[2.0, 3.0, 7.0], [6.0, 8.0, 7.0], [8.0, 8.0, 5.0]], [[2.0, 7.0, 7.0], [2.0, 7.0, 1.0], [4.0, 6.0, 2.0]]], [[[4.0, 7.0, 8.0], 
[3.0, 6.0, 8.0], [2.0, 3.0, 4.0]], [[8.0, 4.0, 4.0], [3.0, 8.0, 3.0], [3.0, 4.0, 6.0]], [[5.0, 7.0, 3.0], [4.0, 5.0, 6.0], 
[1.0, 5.0, 8.0]]], [[[8.0, 3.0, 1.0], [4.0, 4.0, 6.0], [3.0, 8.0, 2.0]], [[2.0, 1.0, 2.0], [5.0, 3.0, 1.0], [7.0, 3.0, 3.0]], 
[[7.0, 8.0, 5.0], [3.0, 8.0, 8.0], [5.0, 3.0, 6.0]]], [[[6.0, 8.0, 7.0], [5.0, 2.0, 8.0], [2.0, 7.0, 1.0]], [[5.0, 8.0, 1.0], 
[2.0, 5.0, 3.0], [5.0, 4.0, 4.0]], [[6.0, 2.0, 2.0], [7.0, 5.0, 7.0], [1.0, 1.0, 8.0]]]])
      val=Data(numpy.array([[[8.0, 4.0, 3.0]], [[5.0, 3.0, 8.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 6.0, 1.0]], [[3.0, 7.0, 7.0]]]))
      arg[2:,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [4.0, 3.0, 4.0], [7.0, 7.0, 5.0]], [[1.0, 7.0, 6.0], [2.0, 2.0, 6.0], [1.0, 
1.0, 5.0]], [[1.0, 1.0, 5.0], [6.0, 2.0, 1.0], [2.0, 8.0, 6.0]]], [[[7.0, 5.0, 3.0], [8.0, 2.0, 6.0], [3.0, 3.0, 2.0]], [[5.0, 
8.0, 8.0], [1.0, 8.0, 2.0], [5.0, 5.0, 1.0]], [[4.0, 7.0, 5.0], [6.0, 7.0, 5.0], [7.0, 1.0, 4.0]]], [[[1.0, 2.0, 4.0], [2.0, 
4.0, 6.0], [1.0, 8.0, 5.0]], [[4.0, 6.0, 8.0], [7.0, 8.0, 1.0], [8.0, 2.0, 1.0]], [[6.0, 4.0, 3.0], [8.0, 4.0, 3.0], [4.0, 2.0, 
2.0]]], [[[3.0, 2.0, 4.0], [2.0, 8.0, 2.0], [8.0, 2.0, 7.0]], [[4.0, 4.0, 8.0], [8.0, 2.0, 8.0], [2.0, 7.0, 8.0]], [[7.0, 5.0, 
2.0], [5.0, 3.0, 8.0], [8.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 6.0], [2.0, 4.0, 7.0], [7.0, 6.0, 8.0]], 
[[2.0, 3.0, 7.0], [6.0, 8.0, 7.0], [8.0, 8.0, 5.0]], [[2.0, 7.0, 7.0], [2.0, 7.0, 1.0], [4.0, 6.0, 2.0]]], [[[4.0, 7.0, 8.0], 
[3.0, 6.0, 8.0], [2.0, 3.0, 4.0]], [[8.0, 4.0, 4.0], [3.0, 8.0, 3.0], [3.0, 4.0, 6.0]], [[5.0, 7.0, 3.0], [4.0, 5.0, 6.0], 
[1.0, 5.0, 8.0]]], [[[8.0, 3.0, 1.0], [4.0, 4.0, 6.0], [3.0, 8.0, 2.0]], [[2.0, 1.0, 2.0], [5.0, 3.0, 1.0], [7.0, 3.0, 3.0]], 
[[7.0, 8.0, 5.0], [1.0, 6.0, 1.0], [5.0, 3.0, 6.0]]], [[[6.0, 8.0, 7.0], [5.0, 2.0, 8.0], [2.0, 7.0, 1.0]], [[5.0, 8.0, 1.0], 
[2.0, 5.0, 3.0], [5.0, 4.0, 4.0]], [[6.0, 2.0, 2.0], [3.0, 7.0, 7.0], [1.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CRPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 5.0], [3.0, 2.0, 4.0], [4.0, 4.0, 8.0]], [[4.0, 5.0, 5.0], [4.0, 3.0, 7.0], [5.0, 
3.0, 8.0]], [[8.0, 3.0, 4.0], [2.0, 5.0, 7.0], [4.0, 3.0, 1.0]]], [[[6.0, 1.0, 1.0], [8.0, 8.0, 4.0], [5.0, 5.0, 7.0]], [[5.0, 
2.0, 3.0], [2.0, 8.0, 2.0], [6.0, 6.0, 7.0]], [[4.0, 1.0, 3.0], [1.0, 6.0, 2.0], [4.0, 3.0, 3.0]]], [[[7.0, 2.0, 8.0], [2.0, 
1.0, 2.0], [3.0, 5.0, 6.0]], [[3.0, 6.0, 1.0], [7.0, 6.0, 7.0], [3.0, 6.0, 7.0]], [[5.0, 3.0, 2.0], [2.0, 5.0, 1.0], [5.0, 8.0, 
8.0]]], [[[2.0, 3.0, 8.0], [6.0, 8.0, 2.0], [7.0, 5.0, 2.0]], [[8.0, 5.0, 8.0], [1.0, 2.0, 8.0], [5.0, 3.0, 1.0]], [[3.0, 6.0, 
6.0], [3.0, 2.0, 7.0], [8.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 3.0], [7.0, 3.0, 5.0], [7.0, 6.0, 1.0]], 
[[4.0, 4.0, 4.0], [3.0, 3.0, 5.0], [2.0, 3.0, 8.0]], [[3.0, 7.0, 3.0], [1.0, 4.0, 5.0], [8.0, 4.0, 5.0]]], [[[3.0, 1.0, 3.0], 
[2.0, 1.0, 5.0], [7.0, 7.0, 2.0]], [[1.0, 6.0, 4.0], [1.0, 7.0, 8.0], [6.0, 1.0, 4.0]], [[1.0, 7.0, 2.0], [1.0, 5.0, 6.0], 
[8.0, 3.0, 5.0]]], [[[2.0, 1.0, 5.0], [8.0, 2.0, 4.0], [5.0, 2.0, 1.0]], [[4.0, 8.0, 2.0], [7.0, 8.0, 7.0], [6.0, 4.0, 4.0]], 
[[3.0, 8.0, 3.0], [6.0, 1.0, 1.0], [3.0, 2.0, 6.0]]], [[[3.0, 1.0, 3.0], [1.0, 1.0, 5.0], [7.0, 2.0, 4.0]], [[8.0, 5.0, 8.0], 
[6.0, 2.0, 3.0], [5.0, 7.0, 7.0]], [[4.0, 1.0, 4.0], [8.0, 3.0, 1.0], [4.0, 5.0, 1.0]]]])
      val=Data(numpy.array([[[4.0, 1.0, 1.0]], [[2.0, 1.0, 3.0]], [[3.0, 6.0, 4.0]], [[2.0, 5.0, 2.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 4.0, 3.0]], [[4.0, 7.0, 4.0]], [[3.0, 7.0, 3.0]], [[5.0, 3.0, 5.0]]]))
      arg[:,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 5.0], [3.0, 2.0, 4.0], [4.0, 4.0, 8.0]], [[4.0, 5.0, 5.0], [4.0, 1.0, 1.0], [5.0, 
3.0, 8.0]], [[8.0, 3.0, 4.0], [2.0, 5.0, 7.0], [4.0, 3.0, 1.0]]], [[[6.0, 1.0, 1.0], [8.0, 8.0, 4.0], [5.0, 5.0, 7.0]], [[5.0, 
2.0, 3.0], [2.0, 1.0, 3.0], [6.0, 6.0, 7.0]], [[4.0, 1.0, 3.0], [1.0, 6.0, 2.0], [4.0, 3.0, 3.0]]], [[[7.0, 2.0, 8.0], [2.0, 
1.0, 2.0], [3.0, 5.0, 6.0]], [[3.0, 6.0, 1.0], [3.0, 6.0, 4.0], [3.0, 6.0, 7.0]], [[5.0, 3.0, 2.0], [2.0, 5.0, 1.0], [5.0, 8.0, 
8.0]]], [[[2.0, 3.0, 8.0], [6.0, 8.0, 2.0], [7.0, 5.0, 2.0]], [[8.0, 5.0, 8.0], [2.0, 5.0, 2.0], [5.0, 3.0, 1.0]], [[3.0, 6.0, 
6.0], [3.0, 2.0, 7.0], [8.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 3.0], [7.0, 3.0, 5.0], [7.0, 6.0, 1.0]], 
[[4.0, 4.0, 4.0], [4.0, 4.0, 3.0], [2.0, 3.0, 8.0]], [[3.0, 7.0, 3.0], [1.0, 4.0, 5.0], [8.0, 4.0, 5.0]]], [[[3.0, 1.0, 3.0], 
[2.0, 1.0, 5.0], [7.0, 7.0, 2.0]], [[1.0, 6.0, 4.0], [4.0, 7.0, 4.0], [6.0, 1.0, 4.0]], [[1.0, 7.0, 2.0], [1.0, 5.0, 6.0], 
[8.0, 3.0, 5.0]]], [[[2.0, 1.0, 5.0], [8.0, 2.0, 4.0], [5.0, 2.0, 1.0]], [[4.0, 8.0, 2.0], [3.0, 7.0, 3.0], [6.0, 4.0, 4.0]], 
[[3.0, 8.0, 3.0], [6.0, 1.0, 1.0], [3.0, 2.0, 6.0]]], [[[3.0, 1.0, 3.0], [1.0, 1.0, 5.0], [7.0, 2.0, 4.0]], [[8.0, 5.0, 8.0], 
[5.0, 3.0, 5.0], [5.0, 7.0, 7.0]], [[4.0, 1.0, 4.0], [8.0, 3.0, 1.0], [4.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RCRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 1.0], [5.0, 6.0, 8.0], [1.0, 6.0, 1.0]], [[3.0, 8.0, 3.0], [7.0, 4.0, 2.0], [2.0, 
6.0, 2.0]], [[6.0, 6.0, 5.0], [5.0, 2.0, 6.0], [2.0, 6.0, 8.0]]], [[[3.0, 2.0, 2.0], [4.0, 8.0, 7.0], [8.0, 4.0, 4.0]], [[7.0, 
7.0, 4.0], [5.0, 6.0, 8.0], [6.0, 5.0, 3.0]], [[7.0, 7.0, 3.0], [4.0, 8.0, 6.0], [7.0, 6.0, 5.0]]], [[[2.0, 2.0, 3.0], [4.0, 
7.0, 7.0], [7.0, 6.0, 5.0]], [[2.0, 1.0, 2.0], [3.0, 3.0, 8.0], [6.0, 7.0, 4.0]], [[3.0, 1.0, 3.0], [5.0, 1.0, 1.0], [7.0, 4.0, 
2.0]]], [[[2.0, 8.0, 5.0], [4.0, 6.0, 2.0], [6.0, 6.0, 7.0]], [[7.0, 5.0, 6.0], [3.0, 3.0, 8.0], [3.0, 5.0, 1.0]], [[6.0, 8.0, 
1.0], [4.0, 8.0, 6.0], [6.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 6.0], [8.0, 2.0, 2.0], [2.0, 3.0, 4.0]], 
[[6.0, 3.0, 5.0], [2.0, 3.0, 8.0], [5.0, 1.0, 4.0]], [[8.0, 8.0, 8.0], [5.0, 3.0, 2.0], [6.0, 6.0, 2.0]]], [[[7.0, 6.0, 2.0], 
[1.0, 4.0, 3.0], [5.0, 6.0, 5.0]], [[3.0, 4.0, 7.0], [8.0, 6.0, 5.0], [1.0, 3.0, 4.0]], [[2.0, 7.0, 3.0], [3.0, 2.0, 7.0], 
[7.0, 6.0, 5.0]]], [[[6.0, 3.0, 2.0], [8.0, 4.0, 5.0], [4.0, 7.0, 3.0]], [[3.0, 8.0, 7.0], [8.0, 5.0, 5.0], [2.0, 2.0, 8.0]], 
[[1.0, 7.0, 5.0], [4.0, 8.0, 5.0], [8.0, 3.0, 5.0]]], [[[2.0, 7.0, 1.0], [2.0, 6.0, 4.0], [7.0, 5.0, 7.0]], [[3.0, 4.0, 4.0], 
[4.0, 2.0, 5.0], [2.0, 8.0, 1.0]], [[1.0, 3.0, 3.0], [2.0, 6.0, 7.0], [4.0, 5.0, 2.0]]]])
      val=Data(numpy.array([[[[4.0, 1.0, 5.0]], [[2.0, 3.0, 8.0]], [[2.0, 4.0, 2.0]]], [[[2.0, 5.0, 2.0]], [[4.0, 8.0, 2.0]], 
[[2.0, 2.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[4.0, 8.0, 6.0]], [[8.0, 8.0, 4.0]], [[7.0, 7.0, 7.0]]], [[[8.0, 7.0, 7.0]], [[3.0, 
7.0, 3.0]], [[6.0, 3.0, 6.0]]]]))
      arg[1:3,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 1.0], [5.0, 6.0, 8.0], [1.0, 6.0, 1.0]], [[3.0, 8.0, 3.0], [7.0, 4.0, 2.0], [2.0, 
6.0, 2.0]], [[6.0, 6.0, 5.0], [5.0, 2.0, 6.0], [2.0, 6.0, 8.0]]], [[[3.0, 2.0, 2.0], [4.0, 1.0, 5.0], [8.0, 4.0, 4.0]], [[7.0, 
7.0, 4.0], [2.0, 3.0, 8.0], [6.0, 5.0, 3.0]], [[7.0, 7.0, 3.0], [2.0, 4.0, 2.0], [7.0, 6.0, 5.0]]], [[[2.0, 2.0, 3.0], [2.0, 
5.0, 2.0], [7.0, 6.0, 5.0]], [[2.0, 1.0, 2.0], [4.0, 8.0, 2.0], [6.0, 7.0, 4.0]], [[3.0, 1.0, 3.0], [2.0, 2.0, 8.0], [7.0, 4.0, 
2.0]]], [[[2.0, 8.0, 5.0], [4.0, 6.0, 2.0], [6.0, 6.0, 7.0]], [[7.0, 5.0, 6.0], [3.0, 3.0, 8.0], [3.0, 5.0, 1.0]], [[6.0, 8.0, 
1.0], [4.0, 8.0, 6.0], [6.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 6.0], [8.0, 2.0, 2.0], [2.0, 3.0, 4.0]], 
[[6.0, 3.0, 5.0], [2.0, 3.0, 8.0], [5.0, 1.0, 4.0]], [[8.0, 8.0, 8.0], [5.0, 3.0, 2.0], [6.0, 6.0, 2.0]]], [[[7.0, 6.0, 2.0], 
[4.0, 8.0, 6.0], [5.0, 6.0, 5.0]], [[3.0, 4.0, 7.0], [8.0, 8.0, 4.0], [1.0, 3.0, 4.0]], [[2.0, 7.0, 3.0], [7.0, 7.0, 7.0], 
[7.0, 6.0, 5.0]]], [[[6.0, 3.0, 2.0], [8.0, 7.0, 7.0], [4.0, 7.0, 3.0]], [[3.0, 8.0, 7.0], [3.0, 7.0, 3.0], [2.0, 2.0, 8.0]], 
[[1.0, 7.0, 5.0], [6.0, 3.0, 6.0], [8.0, 3.0, 5.0]]], [[[2.0, 7.0, 1.0], [2.0, 6.0, 4.0], [7.0, 5.0, 7.0]], [[3.0, 4.0, 4.0], 
[4.0, 2.0, 5.0], [2.0, 8.0, 1.0]], [[1.0, 3.0, 3.0], [2.0, 6.0, 7.0], [4.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 6.0], [6.0, 1.0, 8.0], [7.0, 8.0, 2.0]], [[5.0, 2.0, 6.0], [4.0, 1.0, 7.0], [1.0, 
8.0, 1.0]], [[5.0, 1.0, 6.0], [8.0, 3.0, 1.0], [1.0, 4.0, 1.0]]], [[[6.0, 2.0, 8.0], [5.0, 6.0, 8.0], [7.0, 4.0, 6.0]], [[3.0, 
8.0, 6.0], [2.0, 3.0, 5.0], [5.0, 2.0, 6.0]], [[1.0, 3.0, 7.0], [6.0, 3.0, 2.0], [5.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], [1.0, 
6.0, 5.0], [1.0, 8.0, 2.0]], [[3.0, 7.0, 6.0], [7.0, 2.0, 8.0], [6.0, 4.0, 4.0]], [[8.0, 3.0, 4.0], [6.0, 1.0, 4.0], [1.0, 8.0, 
7.0]]], [[[4.0, 8.0, 2.0], [8.0, 7.0, 3.0], [3.0, 6.0, 2.0]], [[1.0, 1.0, 6.0], [5.0, 7.0, 5.0], [5.0, 8.0, 7.0]], [[3.0, 7.0, 
8.0], [7.0, 3.0, 4.0], [1.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 4.0], [8.0, 1.0, 1.0], [7.0, 2.0, 1.0]], 
[[6.0, 7.0, 8.0], [2.0, 3.0, 4.0], [1.0, 5.0, 2.0]], [[8.0, 2.0, 4.0], [6.0, 1.0, 1.0], [8.0, 5.0, 7.0]]], [[[2.0, 3.0, 8.0], 
[1.0, 7.0, 1.0], [5.0, 6.0, 8.0]], [[4.0, 4.0, 7.0], [4.0, 2.0, 4.0], [4.0, 2.0, 2.0]], [[1.0, 4.0, 8.0], [6.0, 8.0, 5.0], 
[1.0, 2.0, 4.0]]], [[[8.0, 5.0, 4.0], [4.0, 8.0, 3.0], [6.0, 5.0, 1.0]], [[7.0, 8.0, 4.0], [5.0, 1.0, 5.0], [1.0, 8.0, 1.0]], 
[[7.0, 1.0, 2.0], [5.0, 7.0, 1.0], [1.0, 2.0, 2.0]]], [[[5.0, 2.0, 3.0], [6.0, 6.0, 4.0], [6.0, 4.0, 6.0]], [[1.0, 5.0, 8.0], 
[7.0, 1.0, 3.0], [2.0, 2.0, 3.0]], [[1.0, 7.0, 1.0], [1.0, 7.0, 8.0], [4.0, 2.0, 7.0]]]])
      val=Data(numpy.array([[[6.0, 3.0, 1.0], [6.0, 6.0, 7.0], [5.0, 6.0, 4.0]], [[1.0, 5.0, 6.0], [4.0, 6.0, 3.0], [7.0, 3.0, 
2.0]], [[5.0, 4.0, 7.0], [1.0, 2.0, 5.0], [3.0, 1.0, 6.0]], [[2.0, 7.0, 6.0], [7.0, 5.0, 8.0], [3.0, 3.0, 
4.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 6.0, 6.0], [4.0, 8.0, 2.0], [5.0, 7.0, 8.0]], [[3.0, 4.0, 3.0], [8.0, 2.0, 3.0], 
[7.0, 1.0, 1.0]], [[7.0, 1.0, 5.0], [6.0, 4.0, 7.0], [4.0, 7.0, 1.0]], [[8.0, 4.0, 1.0], [8.0, 8.0, 8.0], [7.0, 2.0, 2.0]]]))
      arg[:,:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 6.0], [6.0, 3.0, 1.0], [7.0, 8.0, 2.0]], [[5.0, 2.0, 6.0], [6.0, 6.0, 7.0], [1.0, 
8.0, 1.0]], [[5.0, 1.0, 6.0], [5.0, 6.0, 4.0], [1.0, 4.0, 1.0]]], [[[6.0, 2.0, 8.0], [1.0, 5.0, 6.0], [7.0, 4.0, 6.0]], [[3.0, 
8.0, 6.0], [4.0, 6.0, 3.0], [5.0, 2.0, 6.0]], [[1.0, 3.0, 7.0], [7.0, 3.0, 2.0], [5.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], [5.0, 
4.0, 7.0], [1.0, 8.0, 2.0]], [[3.0, 7.0, 6.0], [1.0, 2.0, 5.0], [6.0, 4.0, 4.0]], [[8.0, 3.0, 4.0], [3.0, 1.0, 6.0], [1.0, 8.0, 
7.0]]], [[[4.0, 8.0, 2.0], [2.0, 7.0, 6.0], [3.0, 6.0, 2.0]], [[1.0, 1.0, 6.0], [7.0, 5.0, 8.0], [5.0, 8.0, 7.0]], [[3.0, 7.0, 
8.0], [3.0, 3.0, 4.0], [1.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 4.0], [1.0, 6.0, 6.0], [7.0, 2.0, 1.0]], 
[[6.0, 7.0, 8.0], [4.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[8.0, 2.0, 4.0], [5.0, 7.0, 8.0], [8.0, 5.0, 7.0]]], [[[2.0, 3.0, 8.0], 
[3.0, 4.0, 3.0], [5.0, 6.0, 8.0]], [[4.0, 4.0, 7.0], [8.0, 2.0, 3.0], [4.0, 2.0, 2.0]], [[1.0, 4.0, 8.0], [7.0, 1.0, 1.0], 
[1.0, 2.0, 4.0]]], [[[8.0, 5.0, 4.0], [7.0, 1.0, 5.0], [6.0, 5.0, 1.0]], [[7.0, 8.0, 4.0], [6.0, 4.0, 7.0], [1.0, 8.0, 1.0]], 
[[7.0, 1.0, 2.0], [4.0, 7.0, 1.0], [1.0, 2.0, 2.0]]], [[[5.0, 2.0, 3.0], [8.0, 4.0, 1.0], [6.0, 4.0, 6.0]], [[1.0, 5.0, 8.0], 
[8.0, 8.0, 8.0], [2.0, 2.0, 3.0]], [[1.0, 7.0, 1.0], [7.0, 2.0, 2.0], [4.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPPP(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 4.0], [5.0, 2.0, 1.0], [6.0, 1.0, 4.0]], [[3.0, 7.0, 4.0], [7.0, 4.0, 5.0], [8.0, 
6.0, 3.0]], [[6.0, 3.0, 8.0], [2.0, 3.0, 7.0], [5.0, 3.0, 2.0]]], [[[4.0, 6.0, 2.0], [2.0, 8.0, 8.0], [7.0, 5.0, 4.0]], [[1.0, 
8.0, 7.0], [6.0, 1.0, 2.0], [5.0, 6.0, 8.0]], [[5.0, 8.0, 3.0], [3.0, 7.0, 6.0], [1.0, 7.0, 4.0]]], [[[8.0, 2.0, 8.0], [5.0, 
1.0, 1.0], [2.0, 2.0, 2.0]], [[8.0, 8.0, 5.0], [1.0, 4.0, 4.0], [3.0, 6.0, 4.0]], [[3.0, 1.0, 1.0], [8.0, 5.0, 3.0], [3.0, 5.0, 
5.0]]], [[[7.0, 7.0, 6.0], [8.0, 2.0, 7.0], [6.0, 7.0, 7.0]], [[1.0, 1.0, 5.0], [5.0, 6.0, 3.0], [5.0, 7.0, 6.0]], [[5.0, 6.0, 
2.0], [1.0, 8.0, 3.0], [3.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 7.0], [6.0, 7.0, 5.0], [5.0, 4.0, 8.0]], 
[[5.0, 2.0, 6.0], [1.0, 5.0, 2.0], [8.0, 7.0, 7.0]], [[2.0, 6.0, 6.0], [8.0, 4.0, 4.0], [6.0, 8.0, 1.0]]], [[[5.0, 5.0, 3.0], 
[8.0, 6.0, 7.0], [5.0, 2.0, 4.0]], [[4.0, 6.0, 4.0], [2.0, 4.0, 8.0], [1.0, 1.0, 2.0]], [[6.0, 7.0, 3.0], [2.0, 1.0, 1.0], 
[4.0, 3.0, 6.0]]], [[[4.0, 3.0, 4.0], [6.0, 2.0, 3.0], [8.0, 1.0, 7.0]], [[7.0, 3.0, 8.0], [5.0, 6.0, 8.0], [7.0, 6.0, 5.0]], 
[[2.0, 4.0, 3.0], [6.0, 1.0, 4.0], [2.0, 4.0, 8.0]]], [[[4.0, 8.0, 2.0], [8.0, 2.0, 5.0], [2.0, 6.0, 7.0]], [[8.0, 2.0, 7.0], 
[6.0, 2.0, 6.0], [3.0, 7.0, 7.0]], [[4.0, 7.0, 4.0], [1.0, 2.0, 8.0], [2.0, 2.0, 5.0]]]])
      val=Data(1.0,self.functionspace)
      val.setTaggedValue(1,1.0)
      arg[2,1,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 4.0], [5.0, 2.0, 1.0], [6.0, 1.0, 4.0]], [[3.0, 7.0, 4.0], [7.0, 4.0, 5.0], [8.0, 
6.0, 3.0]], [[6.0, 3.0, 8.0], [2.0, 3.0, 7.0], [5.0, 3.0, 2.0]]], [[[4.0, 6.0, 2.0], [2.0, 8.0, 8.0], [7.0, 5.0, 4.0]], [[1.0, 
8.0, 7.0], [6.0, 1.0, 2.0], [5.0, 6.0, 8.0]], [[5.0, 8.0, 3.0], [3.0, 7.0, 6.0], [1.0, 7.0, 4.0]]], [[[8.0, 2.0, 8.0], [5.0, 
1.0, 1.0], [2.0, 2.0, 2.0]], [[8.0, 8.0, 5.0], [1.0, 1.0, 4.0], [3.0, 6.0, 4.0]], [[3.0, 1.0, 1.0], [8.0, 5.0, 3.0], [3.0, 5.0, 
5.0]]], [[[7.0, 7.0, 6.0], [8.0, 2.0, 7.0], [6.0, 7.0, 7.0]], [[1.0, 1.0, 5.0], [5.0, 6.0, 3.0], [5.0, 7.0, 6.0]], [[5.0, 6.0, 
2.0], [1.0, 8.0, 3.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 7.0], [6.0, 7.0, 5.0], [5.0, 4.0, 8.0]], 
[[5.0, 2.0, 6.0], [1.0, 5.0, 2.0], [8.0, 7.0, 7.0]], [[2.0, 6.0, 6.0], [8.0, 4.0, 4.0], [6.0, 8.0, 1.0]]], [[[5.0, 5.0, 3.0], 
[8.0, 6.0, 7.0], [5.0, 2.0, 4.0]], [[4.0, 6.0, 4.0], [2.0, 4.0, 8.0], [1.0, 1.0, 2.0]], [[6.0, 7.0, 3.0], [2.0, 1.0, 1.0], 
[4.0, 3.0, 6.0]]], [[[4.0, 3.0, 4.0], [6.0, 2.0, 3.0], [8.0, 1.0, 7.0]], [[7.0, 3.0, 8.0], [5.0, 1.0, 8.0], [7.0, 6.0, 5.0]], 
[[2.0, 4.0, 3.0], [6.0, 1.0, 4.0], [2.0, 4.0, 8.0]]], [[[4.0, 8.0, 2.0], [8.0, 2.0, 5.0], [2.0, 6.0, 7.0]], [[8.0, 2.0, 7.0], 
[6.0, 2.0, 6.0], [3.0, 7.0, 7.0]], [[4.0, 7.0, 4.0], [1.0, 2.0, 8.0], [2.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PLCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 1.0, 7.0], [4.0, 4.0, 6.0]], [[6.0, 7.0, 6.0], [8.0, 1.0, 7.0], [4.0, 
1.0, 7.0]], [[2.0, 4.0, 5.0], [7.0, 2.0, 1.0], [3.0, 2.0, 5.0]]], [[[4.0, 4.0, 2.0], [7.0, 1.0, 3.0], [2.0, 3.0, 7.0]], [[4.0, 
1.0, 1.0], [5.0, 4.0, 6.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 8.0], [4.0, 6.0, 1.0], [5.0, 8.0, 5.0]]], [[[5.0, 6.0, 8.0], [3.0, 
5.0, 3.0], [8.0, 7.0, 2.0]], [[4.0, 4.0, 4.0], [6.0, 6.0, 3.0], [6.0, 8.0, 8.0]], [[6.0, 6.0, 4.0], [8.0, 6.0, 5.0], [7.0, 2.0, 
5.0]]], [[[8.0, 1.0, 5.0], [7.0, 5.0, 4.0], [5.0, 6.0, 4.0]], [[5.0, 5.0, 5.0], [5.0, 5.0, 4.0], [8.0, 1.0, 5.0]], [[6.0, 6.0, 
2.0], [5.0, 1.0, 3.0], [8.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 2.0], [7.0, 2.0, 8.0], [3.0, 7.0, 8.0]], 
[[6.0, 5.0, 2.0], [7.0, 3.0, 2.0], [6.0, 1.0, 1.0]], [[6.0, 5.0, 8.0], [3.0, 6.0, 4.0], [4.0, 5.0, 1.0]]], [[[6.0, 8.0, 6.0], 
[7.0, 6.0, 7.0], [2.0, 7.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 2.0, 4.0], [4.0, 7.0, 6.0]], [[3.0, 2.0, 2.0], [1.0, 3.0, 6.0], 
[2.0, 5.0, 6.0]]], [[[5.0, 7.0, 8.0], [7.0, 6.0, 5.0], [2.0, 7.0, 1.0]], [[8.0, 4.0, 8.0], [2.0, 6.0, 1.0], [4.0, 2.0, 6.0]], 
[[6.0, 7.0, 7.0], [4.0, 1.0, 8.0], [8.0, 7.0, 2.0]]], [[[3.0, 1.0, 7.0], [7.0, 1.0, 3.0], [5.0, 8.0, 5.0]], [[5.0, 7.0, 4.0], 
[3.0, 4.0, 7.0], [5.0, 5.0, 8.0]], [[2.0, 3.0, 7.0], [5.0, 4.0, 2.0], [7.0, 1.0, 2.0]]]])
      val=Data(numpy.array([[[7.0, 6.0, 6.0], [2.0, 2.0, 6.0], [2.0, 3.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 5.0, 6.0], [7.0, 2.0, 4.0], [5.0, 3.0, 1.0]]]))
      arg[2,2:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 1.0, 7.0], [4.0, 4.0, 6.0]], [[6.0, 7.0, 6.0], [8.0, 1.0, 7.0], [4.0, 
1.0, 7.0]], [[2.0, 4.0, 5.0], [7.0, 2.0, 1.0], [3.0, 2.0, 5.0]]], [[[4.0, 4.0, 2.0], [7.0, 1.0, 3.0], [2.0, 3.0, 7.0]], [[4.0, 
1.0, 1.0], [5.0, 4.0, 6.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 8.0], [4.0, 6.0, 1.0], [5.0, 8.0, 5.0]]], [[[5.0, 6.0, 8.0], [3.0, 
5.0, 3.0], [8.0, 7.0, 2.0]], [[4.0, 4.0, 4.0], [6.0, 6.0, 3.0], [6.0, 8.0, 8.0]], [[7.0, 6.0, 6.0], [2.0, 2.0, 6.0], [2.0, 3.0, 
1.0]]], [[[8.0, 1.0, 5.0], [7.0, 5.0, 4.0], [5.0, 6.0, 4.0]], [[5.0, 5.0, 5.0], [5.0, 5.0, 4.0], [8.0, 1.0, 5.0]], [[6.0, 6.0, 
2.0], [5.0, 1.0, 3.0], [8.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 2.0], [7.0, 2.0, 8.0], [3.0, 7.0, 8.0]], 
[[6.0, 5.0, 2.0], [7.0, 3.0, 2.0], [6.0, 1.0, 1.0]], [[6.0, 5.0, 8.0], [3.0, 6.0, 4.0], [4.0, 5.0, 1.0]]], [[[6.0, 8.0, 6.0], 
[7.0, 6.0, 7.0], [2.0, 7.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 2.0, 4.0], [4.0, 7.0, 6.0]], [[3.0, 2.0, 2.0], [1.0, 3.0, 6.0], 
[2.0, 5.0, 6.0]]], [[[5.0, 7.0, 8.0], [7.0, 6.0, 5.0], [2.0, 7.0, 1.0]], [[8.0, 4.0, 8.0], [2.0, 6.0, 1.0], [4.0, 2.0, 6.0]], 
[[6.0, 5.0, 6.0], [7.0, 2.0, 4.0], [5.0, 3.0, 1.0]]], [[[3.0, 1.0, 7.0], [7.0, 1.0, 3.0], [5.0, 8.0, 5.0]], [[5.0, 7.0, 4.0], 
[3.0, 4.0, 7.0], [5.0, 5.0, 8.0]], [[2.0, 3.0, 7.0], [5.0, 4.0, 2.0], [7.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CCCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 7.0], [6.0, 3.0, 3.0], [8.0, 2.0, 2.0]], [[8.0, 4.0, 7.0], [3.0, 7.0, 4.0], [6.0, 
7.0, 2.0]], [[1.0, 6.0, 7.0], [7.0, 5.0, 3.0], [8.0, 8.0, 8.0]]], [[[4.0, 4.0, 3.0], [7.0, 4.0, 8.0], [1.0, 2.0, 8.0]], [[5.0, 
4.0, 6.0], [4.0, 5.0, 2.0], [2.0, 8.0, 5.0]], [[6.0, 2.0, 2.0], [7.0, 5.0, 3.0], [7.0, 4.0, 8.0]]], [[[4.0, 2.0, 4.0], [8.0, 
2.0, 8.0], [1.0, 7.0, 6.0]], [[8.0, 2.0, 3.0], [7.0, 2.0, 5.0], [1.0, 8.0, 4.0]], [[4.0, 3.0, 7.0], [4.0, 2.0, 4.0], [6.0, 5.0, 
7.0]]], [[[6.0, 7.0, 5.0], [5.0, 3.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 5.0, 7.0], [7.0, 3.0, 1.0], [5.0, 6.0, 4.0]], [[3.0, 7.0, 
2.0], [3.0, 7.0, 1.0], [7.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 7.0], [7.0, 2.0, 6.0], [5.0, 1.0, 3.0]], 
[[4.0, 7.0, 4.0], [2.0, 8.0, 8.0], [7.0, 2.0, 1.0]], [[3.0, 2.0, 4.0], [2.0, 5.0, 6.0], [1.0, 5.0, 4.0]]], [[[2.0, 5.0, 7.0], 
[8.0, 2.0, 2.0], [4.0, 6.0, 7.0]], [[3.0, 7.0, 5.0], [3.0, 8.0, 7.0], [4.0, 4.0, 8.0]], [[6.0, 4.0, 8.0], [3.0, 8.0, 4.0], 
[2.0, 6.0, 4.0]]], [[[3.0, 6.0, 1.0], [6.0, 6.0, 4.0], [2.0, 1.0, 5.0]], [[7.0, 1.0, 1.0], [7.0, 4.0, 4.0], [1.0, 2.0, 7.0]], 
[[3.0, 1.0, 4.0], [8.0, 5.0, 7.0], [2.0, 3.0, 8.0]]], [[[4.0, 6.0, 4.0], [4.0, 2.0, 1.0], [1.0, 7.0, 3.0]], [[4.0, 2.0, 2.0], 
[6.0, 5.0, 2.0], [3.0, 6.0, 2.0]], [[2.0, 2.0, 5.0], [4.0, 6.0, 4.0], [8.0, 7.0, 1.0]]]])
      val=Data(numpy.array([[[[8.0, 8.0, 6.0], [8.0, 2.0, 3.0], [7.0, 5.0, 2.0]], [[1.0, 3.0, 4.0], [1.0, 2.0, 3.0], [3.0, 4.0, 
4.0]], [[6.0, 8.0, 7.0], [2.0, 3.0, 1.0], [8.0, 5.0, 3.0]]], [[[7.0, 7.0, 1.0], [3.0, 8.0, 5.0], [2.0, 4.0, 1.0]], [[7.0, 3.0, 
3.0], [2.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[4.0, 1.0, 1.0], [2.0, 2.0, 7.0], [1.0, 6.0, 2.0]]], [[[2.0, 3.0, 7.0], [4.0, 4.0, 
3.0], [3.0, 4.0, 6.0]], [[5.0, 2.0, 3.0], [2.0, 4.0, 5.0], [1.0, 2.0, 5.0]], [[4.0, 4.0, 6.0], [6.0, 1.0, 4.0], [4.0, 4.0, 
6.0]]], [[[8.0, 7.0, 4.0], [1.0, 2.0, 8.0], [6.0, 1.0, 4.0]], [[6.0, 6.0, 6.0], [6.0, 8.0, 1.0], [4.0, 4.0, 1.0]], [[6.0, 8.0, 
4.0], [5.0, 7.0, 2.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 8.0, 4.0], [1.0, 4.0, 4.0], [7.0, 4.0, 6.0]], [[2.0, 5.0, 5.0], [7.0, 8.0, 
6.0], [1.0, 6.0, 2.0]], [[7.0, 3.0, 4.0], [1.0, 2.0, 4.0], [1.0, 1.0, 1.0]]], [[[6.0, 6.0, 7.0], [6.0, 5.0, 3.0], [8.0, 2.0, 
7.0]], [[1.0, 7.0, 3.0], [3.0, 6.0, 7.0], [2.0, 1.0, 4.0]], [[5.0, 6.0, 2.0], [4.0, 3.0, 4.0], [3.0, 4.0, 5.0]]], [[[2.0, 8.0, 
1.0], [8.0, 2.0, 2.0], [5.0, 8.0, 6.0]], [[3.0, 6.0, 2.0], [8.0, 6.0, 6.0], [7.0, 7.0, 3.0]], [[4.0, 8.0, 1.0], [4.0, 7.0, 
3.0], [2.0, 5.0, 3.0]]], [[[4.0, 2.0, 8.0], [1.0, 8.0, 6.0], [3.0, 6.0, 4.0]], [[2.0, 4.0, 5.0], [1.0, 6.0, 2.0], [7.0, 7.0, 
6.0]], [[1.0, 2.0, 4.0], [3.0, 8.0, 6.0], [2.0, 6.0, 6.0]]]]))
      arg[:,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 6.0], [8.0, 2.0, 3.0], [7.0, 5.0, 2.0]], [[1.0, 3.0, 4.0], [1.0, 2.0, 3.0], [3.0, 
4.0, 4.0]], [[6.0, 8.0, 7.0], [2.0, 3.0, 1.0], [8.0, 5.0, 3.0]]], [[[7.0, 7.0, 1.0], [3.0, 8.0, 5.0], [2.0, 4.0, 1.0]], [[7.0, 
3.0, 3.0], [2.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[4.0, 1.0, 1.0], [2.0, 2.0, 7.0], [1.0, 6.0, 2.0]]], [[[2.0, 3.0, 7.0], [4.0, 
4.0, 3.0], [3.0, 4.0, 6.0]], [[5.0, 2.0, 3.0], [2.0, 4.0, 5.0], [1.0, 2.0, 5.0]], [[4.0, 4.0, 6.0], [6.0, 1.0, 4.0], [4.0, 4.0, 
6.0]]], [[[8.0, 7.0, 4.0], [1.0, 2.0, 8.0], [6.0, 1.0, 4.0]], [[6.0, 6.0, 6.0], [6.0, 8.0, 1.0], [4.0, 4.0, 1.0]], [[6.0, 8.0, 
4.0], [5.0, 7.0, 2.0], [3.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 4.0], [1.0, 4.0, 4.0], [7.0, 4.0, 6.0]], 
[[2.0, 5.0, 5.0], [7.0, 8.0, 6.0], [1.0, 6.0, 2.0]], [[7.0, 3.0, 4.0], [1.0, 2.0, 4.0], [1.0, 1.0, 1.0]]], [[[6.0, 6.0, 7.0], 
[6.0, 5.0, 3.0], [8.0, 2.0, 7.0]], [[1.0, 7.0, 3.0], [3.0, 6.0, 7.0], [2.0, 1.0, 4.0]], [[5.0, 6.0, 2.0], [4.0, 3.0, 4.0], 
[3.0, 4.0, 5.0]]], [[[2.0, 8.0, 1.0], [8.0, 2.0, 2.0], [5.0, 8.0, 6.0]], [[3.0, 6.0, 2.0], [8.0, 6.0, 6.0], [7.0, 7.0, 3.0]], 
[[4.0, 8.0, 1.0], [4.0, 7.0, 3.0], [2.0, 5.0, 3.0]]], [[[4.0, 2.0, 8.0], [1.0, 8.0, 6.0], [3.0, 6.0, 4.0]], [[2.0, 4.0, 5.0], 
[1.0, 6.0, 2.0], [7.0, 7.0, 6.0]], [[1.0, 2.0, 4.0], [3.0, 8.0, 6.0], [2.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 7.0, 1.0], [8.0, 1.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 5.0, 7.0], [6.0, 8.0, 7.0], [4.0, 
6.0, 8.0]], [[3.0, 5.0, 3.0], [1.0, 2.0, 1.0], [6.0, 5.0, 2.0]]], [[[6.0, 8.0, 3.0], [3.0, 4.0, 1.0], [1.0, 4.0, 1.0]], [[4.0, 
7.0, 8.0], [4.0, 8.0, 7.0], [8.0, 4.0, 6.0]], [[6.0, 3.0, 6.0], [6.0, 6.0, 7.0], [2.0, 7.0, 1.0]]], [[[6.0, 7.0, 1.0], [8.0, 
2.0, 4.0], [4.0, 2.0, 7.0]], [[4.0, 7.0, 8.0], [8.0, 4.0, 1.0], [1.0, 3.0, 5.0]], [[3.0, 5.0, 5.0], [4.0, 3.0, 8.0], [8.0, 5.0, 
3.0]]], [[[5.0, 1.0, 7.0], [5.0, 3.0, 5.0], [3.0, 4.0, 3.0]], [[2.0, 7.0, 2.0], [4.0, 1.0, 3.0], [3.0, 2.0, 1.0]], [[6.0, 1.0, 
3.0], [8.0, 2.0, 4.0], [3.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 8.0, 6.0], [4.0, 3.0, 6.0], [3.0, 6.0, 7.0]], 
[[3.0, 3.0, 7.0], [4.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[3.0, 7.0, 5.0], [5.0, 1.0, 6.0], [6.0, 1.0, 7.0]]], [[[2.0, 7.0, 8.0], 
[7.0, 8.0, 4.0], [6.0, 8.0, 8.0]], [[8.0, 7.0, 2.0], [4.0, 6.0, 5.0], [4.0, 8.0, 1.0]], [[3.0, 7.0, 8.0], [6.0, 7.0, 4.0], 
[3.0, 8.0, 7.0]]], [[[3.0, 2.0, 5.0], [8.0, 4.0, 2.0], [4.0, 5.0, 2.0]], [[8.0, 6.0, 6.0], [4.0, 4.0, 8.0], [5.0, 5.0, 3.0]], 
[[3.0, 4.0, 5.0], [5.0, 2.0, 4.0], [1.0, 5.0, 7.0]]], [[[2.0, 2.0, 7.0], [6.0, 4.0, 8.0], [2.0, 3.0, 4.0]], [[5.0, 4.0, 2.0], 
[3.0, 8.0, 6.0], [6.0, 2.0, 1.0]], [[5.0, 7.0, 6.0], [6.0, 7.0, 1.0], [8.0, 4.0, 4.0]]]])
      val=Data(numpy.array([[[[1.0, 2.0, 8.0]]], [[[1.0, 7.0, 3.0]]], [[[5.0, 5.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[1.0, 7.0, 3.0]]], [[[3.0, 4.0, 1.0]]], [[[7.0, 5.0, 7.0]]]]))
      arg[:3,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 1.0], [8.0, 1.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 5.0, 7.0], [6.0, 8.0, 7.0], [4.0, 
6.0, 8.0]], [[3.0, 5.0, 3.0], [1.0, 2.0, 1.0], [1.0, 2.0, 8.0]]], [[[6.0, 8.0, 3.0], [3.0, 4.0, 1.0], [1.0, 4.0, 1.0]], [[4.0, 
7.0, 8.0], [4.0, 8.0, 7.0], [8.0, 4.0, 6.0]], [[6.0, 3.0, 6.0], [6.0, 6.0, 7.0], [1.0, 7.0, 3.0]]], [[[6.0, 7.0, 1.0], [8.0, 
2.0, 4.0], [4.0, 2.0, 7.0]], [[4.0, 7.0, 8.0], [8.0, 4.0, 1.0], [1.0, 3.0, 5.0]], [[3.0, 5.0, 5.0], [4.0, 3.0, 8.0], [5.0, 5.0, 
3.0]]], [[[5.0, 1.0, 7.0], [5.0, 3.0, 5.0], [3.0, 4.0, 3.0]], [[2.0, 7.0, 2.0], [4.0, 1.0, 3.0], [3.0, 2.0, 1.0]], [[6.0, 1.0, 
3.0], [8.0, 2.0, 4.0], [3.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 6.0], [4.0, 3.0, 6.0], [3.0, 6.0, 7.0]], 
[[3.0, 3.0, 7.0], [4.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[3.0, 7.0, 5.0], [5.0, 1.0, 6.0], [1.0, 7.0, 3.0]]], [[[2.0, 7.0, 8.0], 
[7.0, 8.0, 4.0], [6.0, 8.0, 8.0]], [[8.0, 7.0, 2.0], [4.0, 6.0, 5.0], [4.0, 8.0, 1.0]], [[3.0, 7.0, 8.0], [6.0, 7.0, 4.0], 
[3.0, 4.0, 1.0]]], [[[3.0, 2.0, 5.0], [8.0, 4.0, 2.0], [4.0, 5.0, 2.0]], [[8.0, 6.0, 6.0], [4.0, 4.0, 8.0], [5.0, 5.0, 3.0]], 
[[3.0, 4.0, 5.0], [5.0, 2.0, 4.0], [7.0, 5.0, 7.0]]], [[[2.0, 2.0, 7.0], [6.0, 4.0, 8.0], [2.0, 3.0, 4.0]], [[5.0, 4.0, 2.0], 
[3.0, 8.0, 6.0], [6.0, 2.0, 1.0]], [[5.0, 7.0, 6.0], [6.0, 7.0, 1.0], [8.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RPPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 6.0], [8.0, 1.0, 1.0], [7.0, 5.0, 1.0]], [[3.0, 1.0, 1.0], [3.0, 1.0, 2.0], [3.0, 
7.0, 8.0]], [[8.0, 3.0, 5.0], [1.0, 3.0, 1.0], [7.0, 5.0, 3.0]]], [[[8.0, 6.0, 8.0], [7.0, 2.0, 5.0], [3.0, 2.0, 3.0]], [[2.0, 
6.0, 4.0], [2.0, 7.0, 1.0], [3.0, 4.0, 6.0]], [[8.0, 2.0, 3.0], [2.0, 2.0, 8.0], [8.0, 8.0, 3.0]]], [[[2.0, 5.0, 7.0], [6.0, 
1.0, 1.0], [1.0, 4.0, 6.0]], [[1.0, 4.0, 3.0], [4.0, 5.0, 2.0], [3.0, 1.0, 3.0]], [[8.0, 3.0, 7.0], [8.0, 7.0, 6.0], [6.0, 5.0, 
8.0]]], [[[1.0, 5.0, 5.0], [3.0, 7.0, 4.0], [4.0, 6.0, 3.0]], [[2.0, 5.0, 7.0], [7.0, 2.0, 6.0], [1.0, 7.0, 6.0]], [[7.0, 7.0, 
1.0], [1.0, 2.0, 5.0], [4.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 4.0], [6.0, 1.0, 7.0], [4.0, 2.0, 6.0]], 
[[1.0, 3.0, 8.0], [4.0, 4.0, 3.0], [2.0, 2.0, 6.0]], [[5.0, 6.0, 7.0], [5.0, 7.0, 8.0], [2.0, 1.0, 8.0]]], [[[7.0, 5.0, 1.0], 
[8.0, 6.0, 4.0], [6.0, 2.0, 4.0]], [[7.0, 6.0, 7.0], [7.0, 7.0, 6.0], [6.0, 7.0, 5.0]], [[5.0, 1.0, 7.0], [5.0, 5.0, 6.0], 
[6.0, 7.0, 2.0]]], [[[3.0, 6.0, 6.0], [8.0, 2.0, 8.0], [1.0, 1.0, 1.0]], [[3.0, 7.0, 4.0], [8.0, 1.0, 8.0], [7.0, 7.0, 1.0]], 
[[3.0, 8.0, 4.0], [3.0, 3.0, 5.0], [1.0, 6.0, 2.0]]], [[[2.0, 4.0, 3.0], [6.0, 6.0, 5.0], [1.0, 7.0, 8.0]], [[4.0, 1.0, 4.0], 
[7.0, 6.0, 7.0], [5.0, 6.0, 6.0]], [[2.0, 2.0, 4.0], [8.0, 4.0, 2.0], [8.0, 5.0, 5.0]]]])
      val=Data(numpy.array([[8.0, 2.0, 4.0], [3.0, 3.0, 8.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[6.0, 6.0, 8.0], [6.0, 5.0, 4.0]]))
      arg[1:3,1,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 6.0], [8.0, 1.0, 1.0], [7.0, 5.0, 1.0]], [[3.0, 1.0, 1.0], [3.0, 1.0, 2.0], [3.0, 
7.0, 8.0]], [[8.0, 3.0, 5.0], [1.0, 3.0, 1.0], [7.0, 5.0, 3.0]]], [[[8.0, 6.0, 8.0], [7.0, 2.0, 5.0], [3.0, 2.0, 3.0]], [[2.0, 
6.0, 4.0], [8.0, 2.0, 4.0], [3.0, 4.0, 6.0]], [[8.0, 2.0, 3.0], [2.0, 2.0, 8.0], [8.0, 8.0, 3.0]]], [[[2.0, 5.0, 7.0], [6.0, 
1.0, 1.0], [1.0, 4.0, 6.0]], [[1.0, 4.0, 3.0], [3.0, 3.0, 8.0], [3.0, 1.0, 3.0]], [[8.0, 3.0, 7.0], [8.0, 7.0, 6.0], [6.0, 5.0, 
8.0]]], [[[1.0, 5.0, 5.0], [3.0, 7.0, 4.0], [4.0, 6.0, 3.0]], [[2.0, 5.0, 7.0], [7.0, 2.0, 6.0], [1.0, 7.0, 6.0]], [[7.0, 7.0, 
1.0], [1.0, 2.0, 5.0], [4.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 4.0], [6.0, 1.0, 7.0], [4.0, 2.0, 6.0]], 
[[1.0, 3.0, 8.0], [4.0, 4.0, 3.0], [2.0, 2.0, 6.0]], [[5.0, 6.0, 7.0], [5.0, 7.0, 8.0], [2.0, 1.0, 8.0]]], [[[7.0, 5.0, 1.0], 
[8.0, 6.0, 4.0], [6.0, 2.0, 4.0]], [[7.0, 6.0, 7.0], [6.0, 6.0, 8.0], [6.0, 7.0, 5.0]], [[5.0, 1.0, 7.0], [5.0, 5.0, 6.0], 
[6.0, 7.0, 2.0]]], [[[3.0, 6.0, 6.0], [8.0, 2.0, 8.0], [1.0, 1.0, 1.0]], [[3.0, 7.0, 4.0], [6.0, 5.0, 4.0], [7.0, 7.0, 1.0]], 
[[3.0, 8.0, 4.0], [3.0, 3.0, 5.0], [1.0, 6.0, 2.0]]], [[[2.0, 4.0, 3.0], [6.0, 6.0, 5.0], [1.0, 7.0, 8.0]], [[4.0, 1.0, 4.0], 
[7.0, 6.0, 7.0], [5.0, 6.0, 6.0]], [[2.0, 2.0, 4.0], [8.0, 4.0, 2.0], [8.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PCLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 4.0], [2.0, 8.0, 7.0], [7.0, 4.0, 8.0]], [[8.0, 3.0, 7.0], [1.0, 7.0, 1.0], [5.0, 
6.0, 2.0]], [[3.0, 6.0, 8.0], [1.0, 1.0, 4.0], [3.0, 2.0, 1.0]]], [[[8.0, 2.0, 5.0], [6.0, 8.0, 6.0], [2.0, 4.0, 1.0]], [[6.0, 
4.0, 7.0], [8.0, 7.0, 6.0], [4.0, 3.0, 4.0]], [[5.0, 2.0, 8.0], [3.0, 5.0, 8.0], [3.0, 1.0, 7.0]]], [[[5.0, 8.0, 4.0], [6.0, 
6.0, 4.0], [2.0, 5.0, 2.0]], [[2.0, 4.0, 1.0], [2.0, 2.0, 2.0], [4.0, 5.0, 6.0]], [[8.0, 6.0, 4.0], [2.0, 1.0, 3.0], [7.0, 8.0, 
5.0]]], [[[7.0, 3.0, 8.0], [5.0, 5.0, 5.0], [4.0, 6.0, 4.0]], [[3.0, 6.0, 2.0], [7.0, 8.0, 5.0], [2.0, 2.0, 5.0]], [[7.0, 8.0, 
7.0], [5.0, 2.0, 8.0], [2.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 4.0], [7.0, 2.0, 8.0], [2.0, 4.0, 8.0]], 
[[2.0, 8.0, 2.0], [4.0, 5.0, 6.0], [5.0, 7.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 7.0, 3.0], [5.0, 6.0, 4.0]]], [[[5.0, 1.0, 2.0], 
[8.0, 3.0, 4.0], [7.0, 5.0, 5.0]], [[1.0, 7.0, 3.0], [8.0, 3.0, 3.0], [3.0, 4.0, 4.0]], [[2.0, 7.0, 8.0], [2.0, 5.0, 1.0], 
[8.0, 5.0, 3.0]]], [[[1.0, 2.0, 2.0], [8.0, 7.0, 2.0], [3.0, 2.0, 3.0]], [[7.0, 7.0, 7.0], [3.0, 7.0, 1.0], [2.0, 7.0, 3.0]], 
[[1.0, 7.0, 1.0], [4.0, 8.0, 8.0], [6.0, 5.0, 2.0]]], [[[6.0, 8.0, 1.0], [7.0, 6.0, 1.0], [2.0, 3.0, 4.0]], [[8.0, 3.0, 2.0], 
[1.0, 7.0, 5.0], [6.0, 6.0, 3.0]], [[1.0, 4.0, 3.0], [2.0, 8.0, 5.0], [2.0, 4.0, 2.0]]]])
      val=Data(numpy.array([[[3.0, 6.0, 1.0]], [[4.0, 2.0, 1.0]], [[6.0, 2.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[6.0, 1.0, 2.0]], [[4.0, 7.0, 6.0]], [[3.0, 1.0, 4.0]]]))
      arg[2,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 4.0], [2.0, 8.0, 7.0], [7.0, 4.0, 8.0]], [[8.0, 3.0, 7.0], [1.0, 7.0, 1.0], [5.0, 
6.0, 2.0]], [[3.0, 6.0, 8.0], [1.0, 1.0, 4.0], [3.0, 2.0, 1.0]]], [[[8.0, 2.0, 5.0], [6.0, 8.0, 6.0], [2.0, 4.0, 1.0]], [[6.0, 
4.0, 7.0], [8.0, 7.0, 6.0], [4.0, 3.0, 4.0]], [[5.0, 2.0, 8.0], [3.0, 5.0, 8.0], [3.0, 1.0, 7.0]]], [[[5.0, 8.0, 4.0], [6.0, 
6.0, 4.0], [3.0, 6.0, 1.0]], [[2.0, 4.0, 1.0], [2.0, 2.0, 2.0], [4.0, 2.0, 1.0]], [[8.0, 6.0, 4.0], [2.0, 1.0, 3.0], [6.0, 2.0, 
7.0]]], [[[7.0, 3.0, 8.0], [5.0, 5.0, 5.0], [4.0, 6.0, 4.0]], [[3.0, 6.0, 2.0], [7.0, 8.0, 5.0], [2.0, 2.0, 5.0]], [[7.0, 8.0, 
7.0], [5.0, 2.0, 8.0], [2.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 4.0], [7.0, 2.0, 8.0], [2.0, 4.0, 8.0]], 
[[2.0, 8.0, 2.0], [4.0, 5.0, 6.0], [5.0, 7.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 7.0, 3.0], [5.0, 6.0, 4.0]]], [[[5.0, 1.0, 2.0], 
[8.0, 3.0, 4.0], [7.0, 5.0, 5.0]], [[1.0, 7.0, 3.0], [8.0, 3.0, 3.0], [3.0, 4.0, 4.0]], [[2.0, 7.0, 8.0], [2.0, 5.0, 1.0], 
[8.0, 5.0, 3.0]]], [[[1.0, 2.0, 2.0], [8.0, 7.0, 2.0], [6.0, 1.0, 2.0]], [[7.0, 7.0, 7.0], [3.0, 7.0, 1.0], [4.0, 7.0, 6.0]], 
[[1.0, 7.0, 1.0], [4.0, 8.0, 8.0], [3.0, 1.0, 4.0]]], [[[6.0, 8.0, 1.0], [7.0, 6.0, 1.0], [2.0, 3.0, 4.0]], [[8.0, 3.0, 2.0], 
[1.0, 7.0, 5.0], [6.0, 6.0, 3.0]], [[1.0, 4.0, 3.0], [2.0, 8.0, 5.0], [2.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LLLL(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 6.0], [7.0, 6.0, 6.0], [1.0, 6.0, 1.0]], [[2.0, 8.0, 1.0], [5.0, 5.0, 8.0], [8.0, 
7.0, 3.0]], [[7.0, 3.0, 2.0], [6.0, 4.0, 3.0], [5.0, 3.0, 1.0]]], [[[4.0, 6.0, 4.0], [4.0, 7.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 
7.0, 5.0], [5.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[6.0, 6.0, 7.0], [5.0, 2.0, 6.0], [6.0, 8.0, 6.0]]], [[[4.0, 5.0, 5.0], [2.0, 
3.0, 4.0], [4.0, 2.0, 1.0]], [[7.0, 7.0, 7.0], [2.0, 6.0, 3.0], [4.0, 4.0, 6.0]], [[2.0, 2.0, 6.0], [7.0, 7.0, 1.0], [7.0, 3.0, 
2.0]]], [[[8.0, 8.0, 1.0], [2.0, 3.0, 1.0], [3.0, 7.0, 6.0]], [[5.0, 1.0, 1.0], [7.0, 8.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 7.0, 
6.0], [4.0, 3.0, 1.0], [8.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 3.0], [2.0, 7.0, 2.0], [4.0, 7.0, 6.0]], 
[[7.0, 7.0, 7.0], [3.0, 2.0, 1.0], [7.0, 2.0, 2.0]], [[7.0, 3.0, 3.0], [5.0, 2.0, 1.0], [3.0, 5.0, 2.0]]], [[[8.0, 5.0, 4.0], 
[8.0, 5.0, 4.0], [5.0, 4.0, 6.0]], [[1.0, 4.0, 8.0], [6.0, 7.0, 7.0], [5.0, 8.0, 7.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 5.0], 
[5.0, 4.0, 7.0]]], [[[4.0, 7.0, 8.0], [2.0, 3.0, 2.0], [7.0, 1.0, 4.0]], [[5.0, 4.0, 6.0], [2.0, 2.0, 3.0], [7.0, 1.0, 8.0]], 
[[4.0, 5.0, 5.0], [6.0, 4.0, 6.0], [6.0, 3.0, 6.0]]], [[[1.0, 1.0, 4.0], [1.0, 6.0, 6.0], [5.0, 5.0, 4.0]], [[7.0, 4.0, 8.0], 
[3.0, 6.0, 8.0], [6.0, 1.0, 2.0]], [[5.0, 6.0, 7.0], [2.0, 7.0, 8.0], [6.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[[[1.0]]], [[[2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0]]], [[[4.0]]]]))
      arg[2:,2:,2:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 6.0], [7.0, 6.0, 6.0], [1.0, 6.0, 1.0]], [[2.0, 8.0, 1.0], [5.0, 5.0, 8.0], [8.0, 
7.0, 3.0]], [[7.0, 3.0, 2.0], [6.0, 4.0, 3.0], [5.0, 3.0, 1.0]]], [[[4.0, 6.0, 4.0], [4.0, 7.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 
7.0, 5.0], [5.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[6.0, 6.0, 7.0], [5.0, 2.0, 6.0], [6.0, 8.0, 6.0]]], [[[4.0, 5.0, 5.0], [2.0, 
3.0, 4.0], [4.0, 2.0, 1.0]], [[7.0, 7.0, 7.0], [2.0, 6.0, 3.0], [4.0, 4.0, 6.0]], [[2.0, 2.0, 6.0], [7.0, 7.0, 1.0], [7.0, 3.0, 
1.0]]], [[[8.0, 8.0, 1.0], [2.0, 3.0, 1.0], [3.0, 7.0, 6.0]], [[5.0, 1.0, 1.0], [7.0, 8.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 7.0, 
6.0], [4.0, 3.0, 1.0], [8.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 3.0], [2.0, 7.0, 2.0], [4.0, 7.0, 6.0]], 
[[7.0, 7.0, 7.0], [3.0, 2.0, 1.0], [7.0, 2.0, 2.0]], [[7.0, 3.0, 3.0], [5.0, 2.0, 1.0], [3.0, 5.0, 2.0]]], [[[8.0, 5.0, 4.0], 
[8.0, 5.0, 4.0], [5.0, 4.0, 6.0]], [[1.0, 4.0, 8.0], [6.0, 7.0, 7.0], [5.0, 8.0, 7.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 5.0], 
[5.0, 4.0, 7.0]]], [[[4.0, 7.0, 8.0], [2.0, 3.0, 2.0], [7.0, 1.0, 4.0]], [[5.0, 4.0, 6.0], [2.0, 2.0, 3.0], [7.0, 1.0, 8.0]], 
[[4.0, 5.0, 5.0], [6.0, 4.0, 6.0], [6.0, 3.0, 8.0]]], [[[1.0, 1.0, 4.0], [1.0, 6.0, 6.0], [5.0, 5.0, 4.0]], [[7.0, 4.0, 8.0], 
[3.0, 6.0, 8.0], [6.0, 1.0, 2.0]], [[5.0, 6.0, 7.0], [2.0, 7.0, 8.0], [6.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CLUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 1.0], [7.0, 8.0, 8.0], [5.0, 3.0, 2.0]], [[3.0, 2.0, 5.0], [4.0, 2.0, 7.0], [8.0, 
3.0, 1.0]], [[3.0, 2.0, 8.0], [4.0, 4.0, 4.0], [5.0, 5.0, 5.0]]], [[[3.0, 5.0, 3.0], [3.0, 4.0, 4.0], [2.0, 4.0, 6.0]], [[5.0, 
2.0, 1.0], [5.0, 5.0, 8.0], [6.0, 6.0, 1.0]], [[3.0, 7.0, 6.0], [5.0, 2.0, 8.0], [8.0, 5.0, 6.0]]], [[[7.0, 6.0, 1.0], [4.0, 
6.0, 2.0], [5.0, 8.0, 6.0]], [[6.0, 4.0, 3.0], [4.0, 1.0, 8.0], [1.0, 6.0, 8.0]], [[2.0, 3.0, 7.0], [3.0, 3.0, 1.0], [1.0, 5.0, 
7.0]]], [[[8.0, 3.0, 5.0], [7.0, 3.0, 5.0], [4.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [1.0, 2.0, 6.0], [7.0, 2.0, 6.0]], [[3.0, 4.0, 
7.0], [3.0, 6.0, 6.0], [2.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 4.0], [2.0, 7.0, 8.0], [6.0, 6.0, 2.0]], 
[[6.0, 1.0, 2.0], [8.0, 6.0, 1.0], [1.0, 1.0, 8.0]], [[8.0, 3.0, 8.0], [2.0, 6.0, 8.0], [8.0, 4.0, 4.0]]], [[[4.0, 2.0, 1.0], 
[2.0, 1.0, 3.0], [3.0, 5.0, 3.0]], [[2.0, 2.0, 7.0], [6.0, 5.0, 4.0], [1.0, 4.0, 1.0]], [[7.0, 5.0, 2.0], [2.0, 2.0, 2.0], 
[4.0, 5.0, 5.0]]], [[[6.0, 2.0, 7.0], [8.0, 5.0, 5.0], [7.0, 5.0, 6.0]], [[2.0, 6.0, 8.0], [4.0, 3.0, 6.0], [1.0, 8.0, 1.0]], 
[[1.0, 4.0, 1.0], [7.0, 1.0, 2.0], [1.0, 4.0, 3.0]]], [[[5.0, 6.0, 8.0], [8.0, 4.0, 5.0], [3.0, 7.0, 5.0]], [[8.0, 5.0, 3.0], 
[5.0, 8.0, 8.0], [4.0, 1.0, 8.0]], [[5.0, 5.0, 1.0], [8.0, 6.0, 2.0], [4.0, 5.0, 1.0]]]])
      val=Data(numpy.array([[[[4.0, 1.0, 4.0], [2.0, 4.0, 6.0]]], [[[5.0, 6.0, 2.0], [3.0, 3.0, 6.0]]], [[[5.0, 3.0, 1.0], 
[4.0, 1.0, 8.0]]], [[[1.0, 6.0, 3.0], [5.0, 6.0, 3.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 2.0], [7.0, 7.0, 2.0]]], [[[6.0, 4.0, 3.0], [7.0, 1.0, 6.0]]], [[[3.0, 
6.0, 8.0], [6.0, 3.0, 8.0]]], [[[2.0, 7.0, 2.0], [4.0, 7.0, 5.0]]]]))
      arg[:,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 1.0], [7.0, 8.0, 8.0], [5.0, 3.0, 2.0]], [[3.0, 2.0, 5.0], [4.0, 2.0, 7.0], [8.0, 
3.0, 1.0]], [[4.0, 1.0, 4.0], [2.0, 4.0, 6.0], [5.0, 5.0, 5.0]]], [[[3.0, 5.0, 3.0], [3.0, 4.0, 4.0], [2.0, 4.0, 6.0]], [[5.0, 
2.0, 1.0], [5.0, 5.0, 8.0], [6.0, 6.0, 1.0]], [[5.0, 6.0, 2.0], [3.0, 3.0, 6.0], [8.0, 5.0, 6.0]]], [[[7.0, 6.0, 1.0], [4.0, 
6.0, 2.0], [5.0, 8.0, 6.0]], [[6.0, 4.0, 3.0], [4.0, 1.0, 8.0], [1.0, 6.0, 8.0]], [[5.0, 3.0, 1.0], [4.0, 1.0, 8.0], [1.0, 5.0, 
7.0]]], [[[8.0, 3.0, 5.0], [7.0, 3.0, 5.0], [4.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [1.0, 2.0, 6.0], [7.0, 2.0, 6.0]], [[1.0, 6.0, 
3.0], [5.0, 6.0, 3.0], [2.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 4.0], [2.0, 7.0, 8.0], [6.0, 6.0, 2.0]], 
[[6.0, 1.0, 2.0], [8.0, 6.0, 1.0], [1.0, 1.0, 8.0]], [[5.0, 6.0, 2.0], [7.0, 7.0, 2.0], [8.0, 4.0, 4.0]]], [[[4.0, 2.0, 1.0], 
[2.0, 1.0, 3.0], [3.0, 5.0, 3.0]], [[2.0, 2.0, 7.0], [6.0, 5.0, 4.0], [1.0, 4.0, 1.0]], [[6.0, 4.0, 3.0], [7.0, 1.0, 6.0], 
[4.0, 5.0, 5.0]]], [[[6.0, 2.0, 7.0], [8.0, 5.0, 5.0], [7.0, 5.0, 6.0]], [[2.0, 6.0, 8.0], [4.0, 3.0, 6.0], [1.0, 8.0, 1.0]], 
[[3.0, 6.0, 8.0], [6.0, 3.0, 8.0], [1.0, 4.0, 3.0]]], [[[5.0, 6.0, 8.0], [8.0, 4.0, 5.0], [3.0, 7.0, 5.0]], [[8.0, 5.0, 3.0], 
[5.0, 8.0, 8.0], [4.0, 1.0, 8.0]], [[2.0, 7.0, 2.0], [4.0, 7.0, 5.0], [4.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_ULRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 5.0], [8.0, 2.0, 6.0], [1.0, 2.0, 1.0]], [[3.0, 1.0, 5.0], [8.0, 3.0, 7.0], [7.0, 
2.0, 3.0]], [[7.0, 3.0, 6.0], [5.0, 5.0, 7.0], [1.0, 4.0, 4.0]]], [[[6.0, 8.0, 7.0], [4.0, 5.0, 7.0], [6.0, 6.0, 1.0]], [[7.0, 
2.0, 7.0], [5.0, 8.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 3.0, 8.0], [1.0, 1.0, 6.0], [5.0, 8.0, 1.0]]], [[[1.0, 5.0, 5.0], [2.0, 
5.0, 4.0], [7.0, 2.0, 5.0]], [[1.0, 8.0, 6.0], [8.0, 5.0, 5.0], [3.0, 8.0, 4.0]], [[6.0, 4.0, 7.0], [6.0, 1.0, 2.0], [2.0, 8.0, 
7.0]]], [[[3.0, 5.0, 1.0], [3.0, 5.0, 8.0], [7.0, 1.0, 2.0]], [[7.0, 6.0, 8.0], [4.0, 6.0, 5.0], [4.0, 7.0, 5.0]], [[4.0, 3.0, 
3.0], [6.0, 6.0, 3.0], [3.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 6.0], [2.0, 3.0, 8.0], [4.0, 2.0, 3.0]], 
[[4.0, 2.0, 7.0], [8.0, 5.0, 1.0], [3.0, 7.0, 7.0]], [[7.0, 4.0, 3.0], [4.0, 3.0, 4.0], [6.0, 1.0, 2.0]]], [[[2.0, 5.0, 5.0], 
[2.0, 8.0, 1.0], [5.0, 1.0, 5.0]], [[1.0, 3.0, 3.0], [8.0, 2.0, 6.0], [2.0, 7.0, 3.0]], [[2.0, 6.0, 3.0], [3.0, 3.0, 8.0], 
[7.0, 8.0, 7.0]]], [[[5.0, 3.0, 1.0], [3.0, 3.0, 4.0], [5.0, 4.0, 1.0]], [[3.0, 8.0, 2.0], [3.0, 7.0, 8.0], [1.0, 1.0, 2.0]], 
[[6.0, 5.0, 5.0], [2.0, 7.0, 5.0], [4.0, 8.0, 1.0]]], [[[4.0, 6.0, 6.0], [1.0, 1.0, 5.0], [5.0, 2.0, 4.0]], [[1.0, 2.0, 5.0], 
[8.0, 3.0, 4.0], [3.0, 2.0, 4.0]], [[3.0, 3.0, 8.0], [2.0, 2.0, 5.0], [6.0, 5.0, 2.0]]]])
      val=Data(numpy.array([[[[2.0, 1.0, 2.0]]], [[[1.0, 5.0, 3.0]]], [[[6.0, 4.0, 8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 3.0, 5.0]]], [[[4.0, 7.0, 7.0]]], [[[6.0, 4.0, 5.0]]]]))
      arg[:3,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 5.0], [8.0, 2.0, 6.0], [1.0, 2.0, 1.0]], [[3.0, 1.0, 5.0], [8.0, 3.0, 7.0], [7.0, 
2.0, 3.0]], [[7.0, 3.0, 6.0], [2.0, 1.0, 2.0], [1.0, 4.0, 4.0]]], [[[6.0, 8.0, 7.0], [4.0, 5.0, 7.0], [6.0, 6.0, 1.0]], [[7.0, 
2.0, 7.0], [5.0, 8.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 3.0, 8.0], [1.0, 5.0, 3.0], [5.0, 8.0, 1.0]]], [[[1.0, 5.0, 5.0], [2.0, 
5.0, 4.0], [7.0, 2.0, 5.0]], [[1.0, 8.0, 6.0], [8.0, 5.0, 5.0], [3.0, 8.0, 4.0]], [[6.0, 4.0, 7.0], [6.0, 4.0, 8.0], [2.0, 8.0, 
7.0]]], [[[3.0, 5.0, 1.0], [3.0, 5.0, 8.0], [7.0, 1.0, 2.0]], [[7.0, 6.0, 8.0], [4.0, 6.0, 5.0], [4.0, 7.0, 5.0]], [[4.0, 3.0, 
3.0], [6.0, 6.0, 3.0], [3.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 6.0], [2.0, 3.0, 8.0], [4.0, 2.0, 3.0]], 
[[4.0, 2.0, 7.0], [8.0, 5.0, 1.0], [3.0, 7.0, 7.0]], [[7.0, 4.0, 3.0], [5.0, 3.0, 5.0], [6.0, 1.0, 2.0]]], [[[2.0, 5.0, 5.0], 
[2.0, 8.0, 1.0], [5.0, 1.0, 5.0]], [[1.0, 3.0, 3.0], [8.0, 2.0, 6.0], [2.0, 7.0, 3.0]], [[2.0, 6.0, 3.0], [4.0, 7.0, 7.0], 
[7.0, 8.0, 7.0]]], [[[5.0, 3.0, 1.0], [3.0, 3.0, 4.0], [5.0, 4.0, 1.0]], [[3.0, 8.0, 2.0], [3.0, 7.0, 8.0], [1.0, 1.0, 2.0]], 
[[6.0, 5.0, 5.0], [6.0, 4.0, 5.0], [4.0, 8.0, 1.0]]], [[[4.0, 6.0, 6.0], [1.0, 1.0, 5.0], [5.0, 2.0, 4.0]], [[1.0, 2.0, 5.0], 
[8.0, 3.0, 4.0], [3.0, 2.0, 4.0]], [[3.0, 3.0, 8.0], [2.0, 2.0, 5.0], [6.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RLUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 4.0], [6.0, 5.0, 4.0], [7.0, 7.0, 6.0]], [[1.0, 3.0, 7.0], [1.0, 5.0, 6.0], [5.0, 
8.0, 2.0]], [[8.0, 8.0, 8.0], [6.0, 1.0, 4.0], [4.0, 7.0, 8.0]]], [[[5.0, 4.0, 2.0], [5.0, 5.0, 5.0], [8.0, 6.0, 5.0]], [[5.0, 
7.0, 6.0], [7.0, 6.0, 6.0], [2.0, 2.0, 8.0]], [[7.0, 1.0, 5.0], [7.0, 3.0, 2.0], [6.0, 6.0, 4.0]]], [[[3.0, 2.0, 6.0], [6.0, 
6.0, 1.0], [7.0, 8.0, 8.0]], [[6.0, 7.0, 4.0], [3.0, 8.0, 4.0], [3.0, 6.0, 8.0]], [[8.0, 8.0, 2.0], [8.0, 7.0, 2.0], [5.0, 3.0, 
2.0]]], [[[6.0, 4.0, 6.0], [5.0, 1.0, 4.0], [1.0, 1.0, 6.0]], [[1.0, 1.0, 7.0], [5.0, 5.0, 2.0], [5.0, 6.0, 3.0]], [[4.0, 8.0, 
1.0], [2.0, 2.0, 6.0], [7.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 7.0], [5.0, 1.0, 1.0], [6.0, 7.0, 7.0]], 
[[7.0, 8.0, 2.0], [4.0, 5.0, 5.0], [3.0, 1.0, 4.0]], [[4.0, 1.0, 8.0], [1.0, 7.0, 5.0], [3.0, 1.0, 7.0]]], [[[5.0, 7.0, 1.0], 
[6.0, 3.0, 7.0], [5.0, 2.0, 5.0]], [[5.0, 4.0, 8.0], [2.0, 6.0, 4.0], [1.0, 8.0, 2.0]], [[1.0, 3.0, 7.0], [4.0, 3.0, 5.0], 
[5.0, 7.0, 4.0]]], [[[6.0, 6.0, 7.0], [4.0, 6.0, 2.0], [2.0, 1.0, 8.0]], [[7.0, 5.0, 2.0], [1.0, 4.0, 6.0], [2.0, 4.0, 4.0]], 
[[3.0, 1.0, 7.0], [5.0, 8.0, 2.0], [4.0, 4.0, 8.0]]], [[[3.0, 2.0, 3.0], [8.0, 3.0, 3.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 2.0], 
[3.0, 4.0, 3.0], [1.0, 4.0, 4.0]], [[8.0, 4.0, 2.0], [3.0, 2.0, 5.0], [5.0, 4.0, 2.0]]]])
      val=Data(numpy.array([[[[5.0, 8.0, 5.0], [8.0, 6.0, 1.0]]], [[[7.0, 3.0, 3.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[2.0, 6.0, 4.0], [5.0, 6.0, 8.0]]], [[[2.0, 5.0, 6.0], [7.0, 2.0, 3.0]]]]))
      arg[1:3,2:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 4.0], [6.0, 5.0, 4.0], [7.0, 7.0, 6.0]], [[1.0, 3.0, 7.0], [1.0, 5.0, 6.0], [5.0, 
8.0, 2.0]], [[8.0, 8.0, 8.0], [6.0, 1.0, 4.0], [4.0, 7.0, 8.0]]], [[[5.0, 4.0, 2.0], [5.0, 5.0, 5.0], [8.0, 6.0, 5.0]], [[5.0, 
7.0, 6.0], [7.0, 6.0, 6.0], [2.0, 2.0, 8.0]], [[5.0, 8.0, 5.0], [8.0, 6.0, 1.0], [6.0, 6.0, 4.0]]], [[[3.0, 2.0, 6.0], [6.0, 
6.0, 1.0], [7.0, 8.0, 8.0]], [[6.0, 7.0, 4.0], [3.0, 8.0, 4.0], [3.0, 6.0, 8.0]], [[7.0, 3.0, 3.0], [4.0, 8.0, 4.0], [5.0, 3.0, 
2.0]]], [[[6.0, 4.0, 6.0], [5.0, 1.0, 4.0], [1.0, 1.0, 6.0]], [[1.0, 1.0, 7.0], [5.0, 5.0, 2.0], [5.0, 6.0, 3.0]], [[4.0, 8.0, 
1.0], [2.0, 2.0, 6.0], [7.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 7.0], [5.0, 1.0, 1.0], [6.0, 7.0, 7.0]], 
[[7.0, 8.0, 2.0], [4.0, 5.0, 5.0], [3.0, 1.0, 4.0]], [[4.0, 1.0, 8.0], [1.0, 7.0, 5.0], [3.0, 1.0, 7.0]]], [[[5.0, 7.0, 1.0], 
[6.0, 3.0, 7.0], [5.0, 2.0, 5.0]], [[5.0, 4.0, 8.0], [2.0, 6.0, 4.0], [1.0, 8.0, 2.0]], [[2.0, 6.0, 4.0], [5.0, 6.0, 8.0], 
[5.0, 7.0, 4.0]]], [[[6.0, 6.0, 7.0], [4.0, 6.0, 2.0], [2.0, 1.0, 8.0]], [[7.0, 5.0, 2.0], [1.0, 4.0, 6.0], [2.0, 4.0, 4.0]], 
[[2.0, 5.0, 6.0], [7.0, 2.0, 3.0], [4.0, 4.0, 8.0]]], [[[3.0, 2.0, 3.0], [8.0, 3.0, 3.0], [7.0, 1.0, 7.0]], [[3.0, 4.0, 2.0], 
[3.0, 4.0, 3.0], [1.0, 4.0, 4.0]], [[8.0, 4.0, 2.0], [3.0, 2.0, 5.0], [5.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LRLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 5.0], [6.0, 1.0, 4.0], [7.0, 7.0, 4.0]], [[5.0, 4.0, 7.0], [8.0, 4.0, 3.0], [2.0, 
2.0, 5.0]], [[4.0, 3.0, 4.0], [2.0, 7.0, 3.0], [3.0, 3.0, 7.0]]], [[[8.0, 8.0, 2.0], [1.0, 7.0, 3.0], [6.0, 5.0, 2.0]], [[8.0, 
7.0, 5.0], [6.0, 6.0, 6.0], [6.0, 1.0, 8.0]], [[6.0, 3.0, 5.0], [7.0, 7.0, 4.0], [3.0, 8.0, 1.0]]], [[[8.0, 8.0, 4.0], [5.0, 
7.0, 1.0], [7.0, 3.0, 5.0]], [[3.0, 7.0, 5.0], [8.0, 7.0, 1.0], [4.0, 8.0, 2.0]], [[2.0, 4.0, 8.0], [8.0, 3.0, 5.0], [7.0, 3.0, 
5.0]]], [[[5.0, 3.0, 5.0], [7.0, 7.0, 2.0], [3.0, 7.0, 1.0]], [[8.0, 1.0, 7.0], [8.0, 8.0, 8.0], [8.0, 3.0, 8.0]], [[7.0, 6.0, 
6.0], [2.0, 7.0, 4.0], [5.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 6.0], [7.0, 4.0, 5.0], [8.0, 8.0, 6.0]], 
[[2.0, 7.0, 5.0], [7.0, 3.0, 4.0], [4.0, 6.0, 2.0]], [[8.0, 6.0, 8.0], [3.0, 7.0, 6.0], [7.0, 4.0, 4.0]]], [[[1.0, 1.0, 5.0], 
[7.0, 1.0, 6.0], [2.0, 1.0, 5.0]], [[6.0, 1.0, 3.0], [5.0, 7.0, 1.0], [7.0, 3.0, 3.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 5.0], 
[1.0, 7.0, 2.0]]], [[[7.0, 5.0, 8.0], [5.0, 6.0, 6.0], [7.0, 7.0, 4.0]], [[1.0, 6.0, 3.0], [6.0, 4.0, 7.0], [7.0, 6.0, 3.0]], 
[[5.0, 7.0, 1.0], [7.0, 6.0, 7.0], [7.0, 1.0, 1.0]]], [[[5.0, 4.0, 1.0], [8.0, 1.0, 7.0], [1.0, 1.0, 7.0]], [[5.0, 3.0, 7.0], 
[5.0, 7.0, 1.0], [2.0, 8.0, 3.0]], [[1.0, 7.0, 2.0], [5.0, 8.0, 2.0], [3.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[[7.0, 1.0, 3.0]]], [[[7.0, 6.0, 1.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 5.0]]], [[[4.0, 4.0, 4.0]]]]))
      arg[2:,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 5.0], [6.0, 1.0, 4.0], [7.0, 7.0, 4.0]], [[5.0, 4.0, 7.0], [8.0, 4.0, 3.0], [2.0, 
2.0, 5.0]], [[4.0, 3.0, 4.0], [2.0, 7.0, 3.0], [3.0, 3.0, 7.0]]], [[[8.0, 8.0, 2.0], [1.0, 7.0, 3.0], [6.0, 5.0, 2.0]], [[8.0, 
7.0, 5.0], [6.0, 6.0, 6.0], [6.0, 1.0, 8.0]], [[6.0, 3.0, 5.0], [7.0, 7.0, 4.0], [3.0, 8.0, 1.0]]], [[[8.0, 8.0, 4.0], [5.0, 
7.0, 1.0], [7.0, 3.0, 5.0]], [[3.0, 7.0, 5.0], [8.0, 7.0, 1.0], [7.0, 1.0, 3.0]], [[2.0, 4.0, 8.0], [8.0, 3.0, 5.0], [7.0, 3.0, 
5.0]]], [[[5.0, 3.0, 5.0], [7.0, 7.0, 2.0], [3.0, 7.0, 1.0]], [[8.0, 1.0, 7.0], [8.0, 8.0, 8.0], [7.0, 6.0, 1.0]], [[7.0, 6.0, 
6.0], [2.0, 7.0, 4.0], [5.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 6.0], [7.0, 4.0, 5.0], [8.0, 8.0, 6.0]], 
[[2.0, 7.0, 5.0], [7.0, 3.0, 4.0], [4.0, 6.0, 2.0]], [[8.0, 6.0, 8.0], [3.0, 7.0, 6.0], [7.0, 4.0, 4.0]]], [[[1.0, 1.0, 5.0], 
[7.0, 1.0, 6.0], [2.0, 1.0, 5.0]], [[6.0, 1.0, 3.0], [5.0, 7.0, 1.0], [7.0, 3.0, 3.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 5.0], 
[1.0, 7.0, 2.0]]], [[[7.0, 5.0, 8.0], [5.0, 6.0, 6.0], [7.0, 7.0, 4.0]], [[1.0, 6.0, 3.0], [6.0, 4.0, 7.0], [8.0, 8.0, 5.0]], 
[[5.0, 7.0, 1.0], [7.0, 6.0, 7.0], [7.0, 1.0, 1.0]]], [[[5.0, 4.0, 1.0], [8.0, 1.0, 7.0], [1.0, 1.0, 7.0]], [[5.0, 3.0, 7.0], 
[5.0, 7.0, 1.0], [4.0, 4.0, 4.0]], [[1.0, 7.0, 2.0], [5.0, 8.0, 2.0], [3.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PRLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 3.0], [2.0, 5.0, 7.0], [6.0, 3.0, 7.0]], [[8.0, 7.0, 5.0], [1.0, 2.0, 1.0], [5.0, 
5.0, 1.0]], [[1.0, 8.0, 5.0], [3.0, 3.0, 2.0], [8.0, 4.0, 4.0]]], [[[2.0, 1.0, 8.0], [2.0, 7.0, 7.0], [4.0, 5.0, 8.0]], [[2.0, 
8.0, 7.0], [6.0, 8.0, 4.0], [5.0, 3.0, 1.0]], [[3.0, 6.0, 7.0], [3.0, 6.0, 5.0], [2.0, 5.0, 7.0]]], [[[6.0, 1.0, 7.0], [7.0, 
1.0, 5.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 1.0], [2.0, 3.0, 7.0], [6.0, 5.0, 1.0]], [[1.0, 4.0, 8.0], [5.0, 4.0, 4.0], [6.0, 2.0, 
1.0]]], [[[5.0, 8.0, 7.0], [5.0, 8.0, 4.0], [1.0, 3.0, 7.0]], [[4.0, 2.0, 2.0], [5.0, 8.0, 5.0], [5.0, 2.0, 3.0]], [[1.0, 7.0, 
3.0], [5.0, 7.0, 3.0], [4.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 6.0], [3.0, 4.0, 2.0], [4.0, 1.0, 8.0]], 
[[3.0, 2.0, 2.0], [1.0, 2.0, 8.0], [7.0, 8.0, 3.0]], [[6.0, 1.0, 2.0], [3.0, 7.0, 6.0], [1.0, 4.0, 1.0]]], [[[6.0, 8.0, 4.0], 
[6.0, 7.0, 5.0], [3.0, 3.0, 4.0]], [[1.0, 5.0, 3.0], [8.0, 5.0, 8.0], [7.0, 8.0, 2.0]], [[7.0, 2.0, 1.0], [5.0, 2.0, 7.0], 
[7.0, 7.0, 5.0]]], [[[5.0, 1.0, 5.0], [1.0, 4.0, 7.0], [2.0, 1.0, 1.0]], [[5.0, 3.0, 5.0], [3.0, 7.0, 6.0], [1.0, 6.0, 7.0]], 
[[8.0, 6.0, 1.0], [2.0, 7.0, 1.0], [4.0, 1.0, 7.0]]], [[[1.0, 5.0, 1.0], [8.0, 4.0, 3.0], [4.0, 4.0, 6.0]], [[2.0, 8.0, 8.0], 
[5.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[5.0, 7.0, 7.0], [5.0, 5.0, 3.0], [1.0, 1.0, 1.0]]]])
      val=Data(numpy.array([[[5.0, 1.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 7.0, 4.0]]]))
      arg[2,1:2,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 3.0], [2.0, 5.0, 7.0], [6.0, 3.0, 7.0]], [[8.0, 7.0, 5.0], [1.0, 2.0, 1.0], [5.0, 
5.0, 1.0]], [[1.0, 8.0, 5.0], [3.0, 3.0, 2.0], [8.0, 4.0, 4.0]]], [[[2.0, 1.0, 8.0], [2.0, 7.0, 7.0], [4.0, 5.0, 8.0]], [[2.0, 
8.0, 7.0], [6.0, 8.0, 4.0], [5.0, 3.0, 1.0]], [[3.0, 6.0, 7.0], [3.0, 6.0, 5.0], [2.0, 5.0, 7.0]]], [[[6.0, 1.0, 7.0], [7.0, 
1.0, 5.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 1.0], [2.0, 3.0, 7.0], [5.0, 1.0, 6.0]], [[1.0, 4.0, 8.0], [5.0, 4.0, 4.0], [6.0, 2.0, 
1.0]]], [[[5.0, 8.0, 7.0], [5.0, 8.0, 4.0], [1.0, 3.0, 7.0]], [[4.0, 2.0, 2.0], [5.0, 8.0, 5.0], [5.0, 2.0, 3.0]], [[1.0, 7.0, 
3.0], [5.0, 7.0, 3.0], [4.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 6.0], [3.0, 4.0, 2.0], [4.0, 1.0, 8.0]], 
[[3.0, 2.0, 2.0], [1.0, 2.0, 8.0], [7.0, 8.0, 3.0]], [[6.0, 1.0, 2.0], [3.0, 7.0, 6.0], [1.0, 4.0, 1.0]]], [[[6.0, 8.0, 4.0], 
[6.0, 7.0, 5.0], [3.0, 3.0, 4.0]], [[1.0, 5.0, 3.0], [8.0, 5.0, 8.0], [7.0, 8.0, 2.0]], [[7.0, 2.0, 1.0], [5.0, 2.0, 7.0], 
[7.0, 7.0, 5.0]]], [[[5.0, 1.0, 5.0], [1.0, 4.0, 7.0], [2.0, 1.0, 1.0]], [[5.0, 3.0, 5.0], [3.0, 7.0, 6.0], [5.0, 7.0, 4.0]], 
[[8.0, 6.0, 1.0], [2.0, 7.0, 1.0], [4.0, 1.0, 7.0]]], [[[1.0, 5.0, 1.0], [8.0, 4.0, 3.0], [4.0, 4.0, 6.0]], [[2.0, 8.0, 8.0], 
[5.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[5.0, 7.0, 7.0], [5.0, 5.0, 3.0], [1.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PUCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 6.0], [8.0, 1.0, 2.0], [6.0, 2.0, 5.0]], [[8.0, 5.0, 6.0], [2.0, 4.0, 7.0], [5.0, 
2.0, 7.0]], [[8.0, 8.0, 6.0], [4.0, 4.0, 6.0], [1.0, 3.0, 8.0]]], [[[1.0, 5.0, 1.0], [4.0, 8.0, 7.0], [2.0, 2.0, 3.0]], [[5.0, 
2.0, 1.0], [1.0, 1.0, 5.0], [7.0, 3.0, 2.0]], [[7.0, 8.0, 6.0], [7.0, 5.0, 1.0], [4.0, 8.0, 6.0]]], [[[8.0, 6.0, 5.0], [4.0, 
5.0, 7.0], [5.0, 7.0, 1.0]], [[8.0, 6.0, 3.0], [2.0, 1.0, 7.0], [2.0, 8.0, 2.0]], [[6.0, 2.0, 4.0], [2.0, 2.0, 3.0], [7.0, 7.0, 
1.0]]], [[[1.0, 7.0, 5.0], [5.0, 2.0, 4.0], [6.0, 4.0, 6.0]], [[1.0, 6.0, 8.0], [4.0, 6.0, 2.0], [2.0, 8.0, 6.0]], [[1.0, 8.0, 
4.0], [3.0, 3.0, 1.0], [8.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 6.0], [8.0, 3.0, 8.0], [8.0, 8.0, 7.0]], 
[[6.0, 7.0, 7.0], [2.0, 3.0, 8.0], [2.0, 7.0, 6.0]], [[6.0, 4.0, 7.0], [6.0, 2.0, 3.0], [2.0, 8.0, 6.0]]], [[[7.0, 1.0, 7.0], 
[5.0, 8.0, 2.0], [3.0, 1.0, 7.0]], [[4.0, 8.0, 1.0], [2.0, 4.0, 2.0], [7.0, 4.0, 8.0]], [[2.0, 7.0, 2.0], [4.0, 6.0, 8.0], 
[1.0, 8.0, 3.0]]], [[[6.0, 8.0, 5.0], [4.0, 1.0, 2.0], [3.0, 4.0, 8.0]], [[6.0, 2.0, 3.0], [3.0, 1.0, 7.0], [3.0, 4.0, 8.0]], 
[[4.0, 4.0, 3.0], [7.0, 7.0, 5.0], [4.0, 3.0, 4.0]]], [[[1.0, 5.0, 4.0], [1.0, 1.0, 5.0], [8.0, 5.0, 8.0]], [[8.0, 8.0, 7.0], 
[8.0, 5.0, 1.0], [5.0, 8.0, 5.0]], [[3.0, 1.0, 7.0], [2.0, 3.0, 7.0], [6.0, 5.0, 5.0]]]])
      val=Data(numpy.array([[[2.0, 3.0, 7.0], [6.0, 5.0, 2.0], [6.0, 3.0, 1.0]], [[8.0, 3.0, 7.0], [4.0, 8.0, 4.0], [3.0, 6.0, 
5.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[5.0, 3.0, 6.0], [4.0, 1.0, 2.0], [3.0, 7.0, 5.0]], [[3.0, 8.0, 3.0], [1.0, 6.0, 7.0], 
[2.0, 5.0, 7.0]]]))
      arg[2,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 6.0], [8.0, 1.0, 2.0], [6.0, 2.0, 5.0]], [[8.0, 5.0, 6.0], [2.0, 4.0, 7.0], [5.0, 
2.0, 7.0]], [[8.0, 8.0, 6.0], [4.0, 4.0, 6.0], [1.0, 3.0, 8.0]]], [[[1.0, 5.0, 1.0], [4.0, 8.0, 7.0], [2.0, 2.0, 3.0]], [[5.0, 
2.0, 1.0], [1.0, 1.0, 5.0], [7.0, 3.0, 2.0]], [[7.0, 8.0, 6.0], [7.0, 5.0, 1.0], [4.0, 8.0, 6.0]]], [[[2.0, 3.0, 7.0], [6.0, 
5.0, 2.0], [6.0, 3.0, 1.0]], [[8.0, 3.0, 7.0], [4.0, 8.0, 4.0], [3.0, 6.0, 5.0]], [[6.0, 2.0, 4.0], [2.0, 2.0, 3.0], [7.0, 7.0, 
1.0]]], [[[1.0, 7.0, 5.0], [5.0, 2.0, 4.0], [6.0, 4.0, 6.0]], [[1.0, 6.0, 8.0], [4.0, 6.0, 2.0], [2.0, 8.0, 6.0]], [[1.0, 8.0, 
4.0], [3.0, 3.0, 1.0], [8.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 6.0], [8.0, 3.0, 8.0], [8.0, 8.0, 7.0]], 
[[6.0, 7.0, 7.0], [2.0, 3.0, 8.0], [2.0, 7.0, 6.0]], [[6.0, 4.0, 7.0], [6.0, 2.0, 3.0], [2.0, 8.0, 6.0]]], [[[7.0, 1.0, 7.0], 
[5.0, 8.0, 2.0], [3.0, 1.0, 7.0]], [[4.0, 8.0, 1.0], [2.0, 4.0, 2.0], [7.0, 4.0, 8.0]], [[2.0, 7.0, 2.0], [4.0, 6.0, 8.0], 
[1.0, 8.0, 3.0]]], [[[5.0, 3.0, 6.0], [4.0, 1.0, 2.0], [3.0, 7.0, 5.0]], [[3.0, 8.0, 3.0], [1.0, 6.0, 7.0], [2.0, 5.0, 7.0]], 
[[4.0, 4.0, 3.0], [7.0, 7.0, 5.0], [4.0, 3.0, 4.0]]], [[[1.0, 5.0, 4.0], [1.0, 1.0, 5.0], [8.0, 5.0, 8.0]], [[8.0, 8.0, 7.0], 
[8.0, 5.0, 1.0], [5.0, 8.0, 5.0]], [[3.0, 1.0, 7.0], [2.0, 3.0, 7.0], [6.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PLPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 3.0], [1.0, 6.0, 2.0], [7.0, 3.0, 4.0]], [[2.0, 3.0, 3.0], [3.0, 2.0, 7.0], [1.0, 
8.0, 3.0]], [[6.0, 6.0, 8.0], [3.0, 7.0, 5.0], [8.0, 3.0, 8.0]]], [[[5.0, 3.0, 6.0], [2.0, 2.0, 1.0], [1.0, 6.0, 1.0]], [[5.0, 
8.0, 6.0], [4.0, 4.0, 6.0], [6.0, 6.0, 4.0]], [[7.0, 1.0, 4.0], [1.0, 7.0, 5.0], [4.0, 8.0, 4.0]]], [[[6.0, 5.0, 7.0], [6.0, 
2.0, 4.0], [4.0, 2.0, 4.0]], [[6.0, 3.0, 2.0], [8.0, 3.0, 3.0], [7.0, 4.0, 6.0]], [[1.0, 2.0, 8.0], [5.0, 2.0, 1.0], [2.0, 1.0, 
2.0]]], [[[4.0, 3.0, 8.0], [3.0, 2.0, 8.0], [7.0, 3.0, 5.0]], [[8.0, 6.0, 1.0], [8.0, 6.0, 6.0], [2.0, 3.0, 1.0]], [[8.0, 8.0, 
4.0], [6.0, 7.0, 4.0], [7.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 7.0], [2.0, 5.0, 5.0], [7.0, 5.0, 6.0]], 
[[3.0, 6.0, 8.0], [1.0, 7.0, 5.0], [7.0, 7.0, 5.0]], [[1.0, 3.0, 6.0], [3.0, 5.0, 6.0], [4.0, 6.0, 8.0]]], [[[2.0, 3.0, 6.0], 
[1.0, 8.0, 4.0], [1.0, 7.0, 8.0]], [[6.0, 3.0, 2.0], [8.0, 1.0, 5.0], [7.0, 8.0, 3.0]], [[1.0, 1.0, 1.0], [8.0, 2.0, 2.0], 
[4.0, 1.0, 4.0]]], [[[8.0, 1.0, 7.0], [5.0, 6.0, 1.0], [1.0, 2.0, 4.0]], [[3.0, 7.0, 7.0], [5.0, 7.0, 4.0], [5.0, 7.0, 8.0]], 
[[4.0, 2.0, 2.0], [6.0, 5.0, 1.0], [2.0, 3.0, 2.0]]], [[[7.0, 8.0, 8.0], [2.0, 1.0, 5.0], [4.0, 5.0, 6.0]], [[7.0, 1.0, 5.0], 
[8.0, 2.0, 4.0], [3.0, 1.0, 3.0]], [[5.0, 3.0, 3.0], [5.0, 7.0, 7.0], [6.0, 6.0, 7.0]]]])
      val=Data(numpy.array([[4.0, 5.0, 3.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[2.0, 7.0, 5.0]]))
      arg[2,2:,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 3.0], [1.0, 6.0, 2.0], [7.0, 3.0, 4.0]], [[2.0, 3.0, 3.0], [3.0, 2.0, 7.0], [1.0, 
8.0, 3.0]], [[6.0, 6.0, 8.0], [3.0, 7.0, 5.0], [8.0, 3.0, 8.0]]], [[[5.0, 3.0, 6.0], [2.0, 2.0, 1.0], [1.0, 6.0, 1.0]], [[5.0, 
8.0, 6.0], [4.0, 4.0, 6.0], [6.0, 6.0, 4.0]], [[7.0, 1.0, 4.0], [1.0, 7.0, 5.0], [4.0, 8.0, 4.0]]], [[[6.0, 5.0, 7.0], [6.0, 
2.0, 4.0], [4.0, 2.0, 4.0]], [[6.0, 3.0, 2.0], [8.0, 3.0, 3.0], [7.0, 4.0, 6.0]], [[1.0, 2.0, 8.0], [4.0, 5.0, 3.0], [2.0, 1.0, 
2.0]]], [[[4.0, 3.0, 8.0], [3.0, 2.0, 8.0], [7.0, 3.0, 5.0]], [[8.0, 6.0, 1.0], [8.0, 6.0, 6.0], [2.0, 3.0, 1.0]], [[8.0, 8.0, 
4.0], [6.0, 7.0, 4.0], [7.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 7.0], [2.0, 5.0, 5.0], [7.0, 5.0, 6.0]], 
[[3.0, 6.0, 8.0], [1.0, 7.0, 5.0], [7.0, 7.0, 5.0]], [[1.0, 3.0, 6.0], [3.0, 5.0, 6.0], [4.0, 6.0, 8.0]]], [[[2.0, 3.0, 6.0], 
[1.0, 8.0, 4.0], [1.0, 7.0, 8.0]], [[6.0, 3.0, 2.0], [8.0, 1.0, 5.0], [7.0, 8.0, 3.0]], [[1.0, 1.0, 1.0], [8.0, 2.0, 2.0], 
[4.0, 1.0, 4.0]]], [[[8.0, 1.0, 7.0], [5.0, 6.0, 1.0], [1.0, 2.0, 4.0]], [[3.0, 7.0, 7.0], [5.0, 7.0, 4.0], [5.0, 7.0, 8.0]], 
[[4.0, 2.0, 2.0], [2.0, 7.0, 5.0], [2.0, 3.0, 2.0]]], [[[7.0, 8.0, 8.0], [2.0, 1.0, 5.0], [4.0, 5.0, 6.0]], [[7.0, 1.0, 5.0], 
[8.0, 2.0, 4.0], [3.0, 1.0, 3.0]], [[5.0, 3.0, 3.0], [5.0, 7.0, 7.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RCUN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 2.0], [8.0, 1.0, 2.0], [3.0, 1.0, 5.0]], [[2.0, 5.0, 8.0], [2.0, 1.0, 7.0], [1.0, 
2.0, 3.0]], [[4.0, 3.0, 5.0], [6.0, 7.0, 1.0], [7.0, 3.0, 7.0]]], [[[4.0, 4.0, 5.0], [2.0, 2.0, 6.0], [3.0, 6.0, 8.0]], [[7.0, 
2.0, 5.0], [1.0, 3.0, 6.0], [3.0, 3.0, 3.0]], [[2.0, 5.0, 7.0], [7.0, 7.0, 5.0], [5.0, 8.0, 6.0]]], [[[4.0, 2.0, 5.0], [2.0, 
2.0, 3.0], [3.0, 4.0, 3.0]], [[1.0, 5.0, 5.0], [5.0, 3.0, 7.0], [4.0, 6.0, 5.0]], [[5.0, 3.0, 2.0], [5.0, 8.0, 7.0], [5.0, 3.0, 
1.0]]], [[[2.0, 8.0, 3.0], [2.0, 6.0, 7.0], [8.0, 1.0, 8.0]], [[8.0, 2.0, 1.0], [5.0, 3.0, 6.0], [2.0, 8.0, 5.0]], [[2.0, 4.0, 
8.0], [2.0, 7.0, 8.0], [1.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 7.0], [8.0, 4.0, 6.0], [4.0, 7.0, 4.0]], 
[[8.0, 5.0, 1.0], [1.0, 4.0, 4.0], [1.0, 2.0, 5.0]], [[4.0, 1.0, 5.0], [7.0, 2.0, 7.0], [3.0, 6.0, 3.0]]], [[[8.0, 4.0, 8.0], 
[1.0, 1.0, 5.0], [8.0, 4.0, 2.0]], [[2.0, 8.0, 1.0], [1.0, 7.0, 4.0], [3.0, 1.0, 8.0]], [[6.0, 3.0, 3.0], [2.0, 6.0, 8.0], 
[1.0, 3.0, 8.0]]], [[[3.0, 6.0, 5.0], [6.0, 6.0, 5.0], [7.0, 7.0, 5.0]], [[7.0, 5.0, 1.0], [7.0, 6.0, 5.0], [3.0, 4.0, 2.0]], 
[[2.0, 1.0, 2.0], [1.0, 6.0, 1.0], [7.0, 1.0, 7.0]]], [[[6.0, 4.0, 8.0], [2.0, 8.0, 2.0], [3.0, 2.0, 7.0]], [[3.0, 5.0, 7.0], 
[3.0, 1.0, 3.0], [5.0, 3.0, 7.0]], [[5.0, 2.0, 2.0], [4.0, 6.0, 5.0], [3.0, 7.0, 5.0]]]])
      val=Data(numpy.array([[[[6.0, 1.0, 4.0], [2.0, 6.0, 3.0]], [[7.0, 6.0, 2.0], [5.0, 1.0, 4.0]], [[3.0, 7.0, 7.0], [8.0, 
8.0, 7.0]]], [[[7.0, 7.0, 6.0], [1.0, 2.0, 5.0]], [[4.0, 7.0, 8.0], [5.0, 6.0, 6.0]], [[4.0, 3.0, 1.0], [2.0, 7.0, 
4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 6.0, 4.0], [7.0, 4.0, 4.0]], [[2.0, 3.0, 5.0], [8.0, 8.0, 1.0]], [[7.0, 5.0, 
7.0], [8.0, 8.0, 3.0]]], [[[7.0, 3.0, 1.0], [8.0, 1.0, 6.0]], [[2.0, 3.0, 1.0], [5.0, 1.0, 8.0]], [[7.0, 6.0, 4.0], [3.0, 5.0, 
8.0]]]]))
      arg[1:3,:,:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 2.0], [8.0, 1.0, 2.0], [3.0, 1.0, 5.0]], [[2.0, 5.0, 8.0], [2.0, 1.0, 7.0], [1.0, 
2.0, 3.0]], [[4.0, 3.0, 5.0], [6.0, 7.0, 1.0], [7.0, 3.0, 7.0]]], [[[6.0, 1.0, 4.0], [2.0, 6.0, 3.0], [3.0, 6.0, 8.0]], [[7.0, 
6.0, 2.0], [5.0, 1.0, 4.0], [3.0, 3.0, 3.0]], [[3.0, 7.0, 7.0], [8.0, 8.0, 7.0], [5.0, 8.0, 6.0]]], [[[7.0, 7.0, 6.0], [1.0, 
2.0, 5.0], [3.0, 4.0, 3.0]], [[4.0, 7.0, 8.0], [5.0, 6.0, 6.0], [4.0, 6.0, 5.0]], [[4.0, 3.0, 1.0], [2.0, 7.0, 4.0], [5.0, 3.0, 
1.0]]], [[[2.0, 8.0, 3.0], [2.0, 6.0, 7.0], [8.0, 1.0, 8.0]], [[8.0, 2.0, 1.0], [5.0, 3.0, 6.0], [2.0, 8.0, 5.0]], [[2.0, 4.0, 
8.0], [2.0, 7.0, 8.0], [1.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 7.0], [8.0, 4.0, 6.0], [4.0, 7.0, 4.0]], 
[[8.0, 5.0, 1.0], [1.0, 4.0, 4.0], [1.0, 2.0, 5.0]], [[4.0, 1.0, 5.0], [7.0, 2.0, 7.0], [3.0, 6.0, 3.0]]], [[[5.0, 6.0, 4.0], 
[7.0, 4.0, 4.0], [8.0, 4.0, 2.0]], [[2.0, 3.0, 5.0], [8.0, 8.0, 1.0], [3.0, 1.0, 8.0]], [[7.0, 5.0, 7.0], [8.0, 8.0, 3.0], 
[1.0, 3.0, 8.0]]], [[[7.0, 3.0, 1.0], [8.0, 1.0, 6.0], [7.0, 7.0, 5.0]], [[2.0, 3.0, 1.0], [5.0, 1.0, 8.0], [3.0, 4.0, 2.0]], 
[[7.0, 6.0, 4.0], [3.0, 5.0, 8.0], [7.0, 1.0, 7.0]]], [[[6.0, 4.0, 8.0], [2.0, 8.0, 2.0], [3.0, 2.0, 7.0]], [[3.0, 5.0, 7.0], 
[3.0, 1.0, 3.0], [5.0, 3.0, 7.0]], [[5.0, 2.0, 2.0], [4.0, 6.0, 5.0], [3.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LPCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 8.0], [1.0, 8.0, 1.0], [6.0, 4.0, 6.0]], [[2.0, 7.0, 2.0], [8.0, 7.0, 3.0], [6.0, 
7.0, 6.0]], [[8.0, 7.0, 2.0], [6.0, 7.0, 2.0], [4.0, 7.0, 8.0]]], [[[7.0, 5.0, 2.0], [1.0, 6.0, 7.0], [7.0, 3.0, 3.0]], [[8.0, 
1.0, 2.0], [7.0, 5.0, 1.0], [2.0, 6.0, 8.0]], [[4.0, 5.0, 6.0], [3.0, 8.0, 7.0], [3.0, 7.0, 7.0]]], [[[3.0, 3.0, 2.0], [8.0, 
8.0, 3.0], [5.0, 2.0, 6.0]], [[8.0, 6.0, 8.0], [1.0, 7.0, 5.0], [7.0, 4.0, 1.0]], [[6.0, 7.0, 8.0], [3.0, 4.0, 6.0], [8.0, 1.0, 
7.0]]], [[[1.0, 8.0, 3.0], [3.0, 2.0, 3.0], [4.0, 8.0, 5.0]], [[8.0, 6.0, 6.0], [6.0, 2.0, 3.0], [1.0, 2.0, 5.0]], [[6.0, 5.0, 
7.0], [6.0, 4.0, 2.0], [5.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 1.0], [5.0, 5.0, 6.0], [5.0, 2.0, 4.0]], 
[[7.0, 8.0, 3.0], [6.0, 4.0, 5.0], [2.0, 6.0, 4.0]], [[3.0, 8.0, 2.0], [2.0, 3.0, 8.0], [7.0, 4.0, 2.0]]], [[[5.0, 8.0, 6.0], 
[5.0, 1.0, 7.0], [7.0, 3.0, 8.0]], [[6.0, 4.0, 7.0], [4.0, 5.0, 2.0], [8.0, 8.0, 1.0]], [[4.0, 4.0, 2.0], [8.0, 4.0, 6.0], 
[7.0, 3.0, 2.0]]], [[[2.0, 5.0, 6.0], [7.0, 7.0, 1.0], [7.0, 2.0, 6.0]], [[3.0, 6.0, 6.0], [7.0, 2.0, 3.0], [1.0, 4.0, 1.0]], 
[[4.0, 6.0, 1.0], [6.0, 6.0, 1.0], [2.0, 8.0, 4.0]]], [[[4.0, 8.0, 3.0], [6.0, 7.0, 4.0], [2.0, 8.0, 5.0]], [[1.0, 3.0, 8.0], 
[3.0, 4.0, 3.0], [5.0, 2.0, 6.0]], [[3.0, 8.0, 8.0], [3.0, 7.0, 3.0], [6.0, 6.0, 6.0]]]])
      val=Data(numpy.array([[[5.0, 7.0, 1.0], [3.0, 1.0, 7.0], [6.0, 1.0, 8.0]], [[5.0, 1.0, 8.0], [7.0, 6.0, 8.0], [6.0, 3.0, 
6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[8.0, 1.0, 3.0], [7.0, 1.0, 1.0], [8.0, 6.0, 6.0]], [[5.0, 8.0, 3.0], [7.0, 6.0, 5.0], 
[4.0, 4.0, 8.0]]]))
      arg[2:,1,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 8.0], [1.0, 8.0, 1.0], [6.0, 4.0, 6.0]], [[2.0, 7.0, 2.0], [8.0, 7.0, 3.0], [6.0, 
7.0, 6.0]], [[8.0, 7.0, 2.0], [6.0, 7.0, 2.0], [4.0, 7.0, 8.0]]], [[[7.0, 5.0, 2.0], [1.0, 6.0, 7.0], [7.0, 3.0, 3.0]], [[8.0, 
1.0, 2.0], [7.0, 5.0, 1.0], [2.0, 6.0, 8.0]], [[4.0, 5.0, 6.0], [3.0, 8.0, 7.0], [3.0, 7.0, 7.0]]], [[[3.0, 3.0, 2.0], [8.0, 
8.0, 3.0], [5.0, 2.0, 6.0]], [[5.0, 7.0, 1.0], [3.0, 1.0, 7.0], [6.0, 1.0, 8.0]], [[6.0, 7.0, 8.0], [3.0, 4.0, 6.0], [8.0, 1.0, 
7.0]]], [[[1.0, 8.0, 3.0], [3.0, 2.0, 3.0], [4.0, 8.0, 5.0]], [[5.0, 1.0, 8.0], [7.0, 6.0, 8.0], [6.0, 3.0, 6.0]], [[6.0, 5.0, 
7.0], [6.0, 4.0, 2.0], [5.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 1.0], [5.0, 5.0, 6.0], [5.0, 2.0, 4.0]], 
[[7.0, 8.0, 3.0], [6.0, 4.0, 5.0], [2.0, 6.0, 4.0]], [[3.0, 8.0, 2.0], [2.0, 3.0, 8.0], [7.0, 4.0, 2.0]]], [[[5.0, 8.0, 6.0], 
[5.0, 1.0, 7.0], [7.0, 3.0, 8.0]], [[6.0, 4.0, 7.0], [4.0, 5.0, 2.0], [8.0, 8.0, 1.0]], [[4.0, 4.0, 2.0], [8.0, 4.0, 6.0], 
[7.0, 3.0, 2.0]]], [[[2.0, 5.0, 6.0], [7.0, 7.0, 1.0], [7.0, 2.0, 6.0]], [[8.0, 1.0, 3.0], [7.0, 1.0, 1.0], [8.0, 6.0, 6.0]], 
[[4.0, 6.0, 1.0], [6.0, 6.0, 1.0], [2.0, 8.0, 4.0]]], [[[4.0, 8.0, 3.0], [6.0, 7.0, 4.0], [2.0, 8.0, 5.0]], [[5.0, 8.0, 3.0], 
[7.0, 6.0, 5.0], [4.0, 4.0, 8.0]], [[3.0, 8.0, 8.0], [3.0, 7.0, 3.0], [6.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_LCLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 4.0], [6.0, 6.0, 7.0], [5.0, 3.0, 7.0]], [[4.0, 6.0, 5.0], [8.0, 8.0, 3.0], [4.0, 
5.0, 7.0]], [[7.0, 5.0, 3.0], [5.0, 3.0, 1.0], [4.0, 7.0, 7.0]]], [[[5.0, 1.0, 1.0], [6.0, 7.0, 7.0], [7.0, 6.0, 8.0]], [[6.0, 
6.0, 4.0], [3.0, 5.0, 3.0], [1.0, 6.0, 6.0]], [[2.0, 2.0, 3.0], [1.0, 4.0, 7.0], [5.0, 7.0, 8.0]]], [[[6.0, 1.0, 8.0], [8.0, 
6.0, 2.0], [3.0, 8.0, 7.0]], [[8.0, 7.0, 6.0], [6.0, 3.0, 7.0], [5.0, 4.0, 6.0]], [[8.0, 4.0, 7.0], [7.0, 5.0, 1.0], [5.0, 7.0, 
4.0]]], [[[8.0, 4.0, 4.0], [8.0, 5.0, 1.0], [4.0, 7.0, 5.0]], [[2.0, 3.0, 2.0], [2.0, 1.0, 5.0], [3.0, 1.0, 4.0]], [[7.0, 2.0, 
8.0], [6.0, 5.0, 2.0], [7.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 1.0, 3.0], [8.0, 2.0, 5.0], [2.0, 6.0, 3.0]], 
[[8.0, 3.0, 7.0], [5.0, 3.0, 5.0], [7.0, 7.0, 3.0]], [[1.0, 1.0, 5.0], [8.0, 6.0, 8.0], [1.0, 3.0, 6.0]]], [[[1.0, 5.0, 6.0], 
[8.0, 4.0, 3.0], [5.0, 6.0, 3.0]], [[5.0, 8.0, 8.0], [4.0, 5.0, 3.0], [8.0, 5.0, 6.0]], [[1.0, 8.0, 3.0], [8.0, 6.0, 8.0], 
[8.0, 6.0, 8.0]]], [[[2.0, 1.0, 8.0], [3.0, 6.0, 5.0], [4.0, 5.0, 5.0]], [[6.0, 7.0, 6.0], [4.0, 6.0, 4.0], [3.0, 5.0, 6.0]], 
[[4.0, 1.0, 5.0], [3.0, 8.0, 1.0], [3.0, 8.0, 1.0]]], [[[7.0, 6.0, 4.0], [6.0, 8.0, 4.0], [7.0, 8.0, 5.0]], [[7.0, 1.0, 2.0], 
[3.0, 2.0, 2.0], [5.0, 5.0, 8.0]], [[1.0, 2.0, 7.0], [5.0, 7.0, 2.0], [7.0, 2.0, 3.0]]]])
      val=Data(numpy.array([[[[5.0, 2.0, 7.0]], [[7.0, 8.0, 2.0]], [[3.0, 7.0, 3.0]]], [[[1.0, 6.0, 2.0]], [[4.0, 8.0, 1.0]], 
[[7.0, 5.0, 7.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 8.0]], [[6.0, 3.0, 7.0]], [[8.0, 3.0, 3.0]]], [[[7.0, 1.0, 8.0]], [[6.0, 
8.0, 5.0]], [[3.0, 8.0, 6.0]]]]))
      arg[2:,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 4.0], [6.0, 6.0, 7.0], [5.0, 3.0, 7.0]], [[4.0, 6.0, 5.0], [8.0, 8.0, 3.0], [4.0, 
5.0, 7.0]], [[7.0, 5.0, 3.0], [5.0, 3.0, 1.0], [4.0, 7.0, 7.0]]], [[[5.0, 1.0, 1.0], [6.0, 7.0, 7.0], [7.0, 6.0, 8.0]], [[6.0, 
6.0, 4.0], [3.0, 5.0, 3.0], [1.0, 6.0, 6.0]], [[2.0, 2.0, 3.0], [1.0, 4.0, 7.0], [5.0, 7.0, 8.0]]], [[[6.0, 1.0, 8.0], [8.0, 
6.0, 2.0], [5.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], [6.0, 3.0, 7.0], [7.0, 8.0, 2.0]], [[8.0, 4.0, 7.0], [7.0, 5.0, 1.0], [3.0, 7.0, 
3.0]]], [[[8.0, 4.0, 4.0], [8.0, 5.0, 1.0], [1.0, 6.0, 2.0]], [[2.0, 3.0, 2.0], [2.0, 1.0, 5.0], [4.0, 8.0, 1.0]], [[7.0, 2.0, 
8.0], [6.0, 5.0, 2.0], [7.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 3.0], [8.0, 2.0, 5.0], [2.0, 6.0, 3.0]], 
[[8.0, 3.0, 7.0], [5.0, 3.0, 5.0], [7.0, 7.0, 3.0]], [[1.0, 1.0, 5.0], [8.0, 6.0, 8.0], [1.0, 3.0, 6.0]]], [[[1.0, 5.0, 6.0], 
[8.0, 4.0, 3.0], [5.0, 6.0, 3.0]], [[5.0, 8.0, 8.0], [4.0, 5.0, 3.0], [8.0, 5.0, 6.0]], [[1.0, 8.0, 3.0], [8.0, 6.0, 8.0], 
[8.0, 6.0, 8.0]]], [[[2.0, 1.0, 8.0], [3.0, 6.0, 5.0], [8.0, 2.0, 8.0]], [[6.0, 7.0, 6.0], [4.0, 6.0, 4.0], [6.0, 3.0, 7.0]], 
[[4.0, 1.0, 5.0], [3.0, 8.0, 1.0], [8.0, 3.0, 3.0]]], [[[7.0, 6.0, 4.0], [6.0, 8.0, 4.0], [7.0, 1.0, 8.0]], [[7.0, 1.0, 2.0], 
[3.0, 2.0, 2.0], [6.0, 8.0, 5.0]], [[1.0, 2.0, 7.0], [5.0, 7.0, 2.0], [3.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CLRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 7.0], [7.0, 8.0, 2.0], [7.0, 3.0, 7.0]], [[3.0, 4.0, 7.0], [4.0, 2.0, 3.0], [7.0, 
4.0, 6.0]], [[6.0, 1.0, 1.0], [1.0, 7.0, 2.0], [7.0, 3.0, 3.0]]], [[[2.0, 7.0, 1.0], [7.0, 2.0, 7.0], [6.0, 5.0, 5.0]], [[3.0, 
6.0, 3.0], [7.0, 7.0, 6.0], [2.0, 7.0, 8.0]], [[7.0, 2.0, 5.0], [6.0, 8.0, 5.0], [3.0, 2.0, 2.0]]], [[[4.0, 8.0, 1.0], [5.0, 
2.0, 1.0], [8.0, 5.0, 2.0]], [[2.0, 1.0, 7.0], [3.0, 5.0, 4.0], [3.0, 4.0, 8.0]], [[4.0, 3.0, 2.0], [4.0, 5.0, 6.0], [2.0, 1.0, 
5.0]]], [[[6.0, 4.0, 8.0], [3.0, 3.0, 1.0], [4.0, 8.0, 2.0]], [[1.0, 8.0, 4.0], [6.0, 4.0, 4.0], [5.0, 7.0, 4.0]], [[6.0, 6.0, 
1.0], [5.0, 8.0, 3.0], [2.0, 7.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 4.0], [8.0, 1.0, 1.0], [3.0, 1.0, 6.0]], 
[[7.0, 3.0, 3.0], [7.0, 2.0, 5.0], [5.0, 1.0, 5.0]], [[6.0, 2.0, 5.0], [6.0, 6.0, 6.0], [4.0, 6.0, 6.0]]], [[[1.0, 2.0, 8.0], 
[8.0, 1.0, 3.0], [4.0, 1.0, 5.0]], [[7.0, 3.0, 7.0], [5.0, 5.0, 3.0], [4.0, 6.0, 7.0]], [[8.0, 4.0, 8.0], [7.0, 3.0, 1.0], 
[2.0, 6.0, 4.0]]], [[[4.0, 2.0, 4.0], [5.0, 2.0, 5.0], [8.0, 7.0, 1.0]], [[7.0, 8.0, 4.0], [5.0, 6.0, 5.0], [2.0, 6.0, 7.0]], 
[[5.0, 7.0, 7.0], [4.0, 7.0, 5.0], [6.0, 3.0, 1.0]]], [[[6.0, 1.0, 6.0], [7.0, 8.0, 2.0], [5.0, 4.0, 4.0]], [[8.0, 5.0, 5.0], 
[1.0, 3.0, 6.0], [6.0, 8.0, 6.0]], [[3.0, 4.0, 5.0], [4.0, 1.0, 6.0], [7.0, 2.0, 8.0]]]])
      val=Data(numpy.array([[[[3.0, 6.0, 6.0]]], [[[1.0, 4.0, 3.0]]], [[[7.0, 4.0, 5.0]]], [[[7.0, 7.0, 
6.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 5.0, 4.0]]], [[[3.0, 4.0, 8.0]]], [[[6.0, 8.0, 8.0]]], [[[6.0, 1.0, 7.0]]]]))
      arg[:,2:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 7.0], [7.0, 8.0, 2.0], [7.0, 3.0, 7.0]], [[3.0, 4.0, 7.0], [4.0, 2.0, 3.0], [7.0, 
4.0, 6.0]], [[6.0, 1.0, 1.0], [3.0, 6.0, 6.0], [7.0, 3.0, 3.0]]], [[[2.0, 7.0, 1.0], [7.0, 2.0, 7.0], [6.0, 5.0, 5.0]], [[3.0, 
6.0, 3.0], [7.0, 7.0, 6.0], [2.0, 7.0, 8.0]], [[7.0, 2.0, 5.0], [1.0, 4.0, 3.0], [3.0, 2.0, 2.0]]], [[[4.0, 8.0, 1.0], [5.0, 
2.0, 1.0], [8.0, 5.0, 2.0]], [[2.0, 1.0, 7.0], [3.0, 5.0, 4.0], [3.0, 4.0, 8.0]], [[4.0, 3.0, 2.0], [7.0, 4.0, 5.0], [2.0, 1.0, 
5.0]]], [[[6.0, 4.0, 8.0], [3.0, 3.0, 1.0], [4.0, 8.0, 2.0]], [[1.0, 8.0, 4.0], [6.0, 4.0, 4.0], [5.0, 7.0, 4.0]], [[6.0, 6.0, 
1.0], [7.0, 7.0, 6.0], [2.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 4.0], [8.0, 1.0, 1.0], [3.0, 1.0, 6.0]], 
[[7.0, 3.0, 3.0], [7.0, 2.0, 5.0], [5.0, 1.0, 5.0]], [[6.0, 2.0, 5.0], [7.0, 5.0, 4.0], [4.0, 6.0, 6.0]]], [[[1.0, 2.0, 8.0], 
[8.0, 1.0, 3.0], [4.0, 1.0, 5.0]], [[7.0, 3.0, 7.0], [5.0, 5.0, 3.0], [4.0, 6.0, 7.0]], [[8.0, 4.0, 8.0], [3.0, 4.0, 8.0], 
[2.0, 6.0, 4.0]]], [[[4.0, 2.0, 4.0], [5.0, 2.0, 5.0], [8.0, 7.0, 1.0]], [[7.0, 8.0, 4.0], [5.0, 6.0, 5.0], [2.0, 6.0, 7.0]], 
[[5.0, 7.0, 7.0], [6.0, 8.0, 8.0], [6.0, 3.0, 1.0]]], [[[6.0, 1.0, 6.0], [7.0, 8.0, 2.0], [5.0, 4.0, 4.0]], [[8.0, 5.0, 5.0], 
[1.0, 3.0, 6.0], [6.0, 8.0, 6.0]], [[3.0, 4.0, 5.0], [6.0, 1.0, 7.0], [7.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RRCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 3.0], [1.0, 6.0, 4.0], [2.0, 5.0, 8.0]], [[8.0, 1.0, 3.0], [2.0, 4.0, 3.0], [3.0, 
2.0, 5.0]], [[3.0, 2.0, 2.0], [7.0, 8.0, 1.0], [6.0, 7.0, 7.0]]], [[[8.0, 7.0, 4.0], [2.0, 1.0, 8.0], [3.0, 3.0, 2.0]], [[7.0, 
3.0, 7.0], [5.0, 7.0, 3.0], [8.0, 4.0, 5.0]], [[6.0, 5.0, 4.0], [4.0, 7.0, 7.0], [5.0, 5.0, 8.0]]], [[[7.0, 7.0, 4.0], [1.0, 
4.0, 6.0], [7.0, 3.0, 5.0]], [[6.0, 6.0, 3.0], [6.0, 1.0, 8.0], [5.0, 1.0, 8.0]], [[1.0, 1.0, 1.0], [1.0, 3.0, 7.0], [8.0, 2.0, 
1.0]]], [[[7.0, 4.0, 2.0], [4.0, 1.0, 3.0], [8.0, 5.0, 5.0]], [[6.0, 3.0, 4.0], [6.0, 8.0, 4.0], [7.0, 3.0, 7.0]], [[2.0, 3.0, 
4.0], [1.0, 4.0, 4.0], [3.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 3.0], [2.0, 8.0, 4.0], [8.0, 1.0, 5.0]], 
[[1.0, 3.0, 6.0], [5.0, 3.0, 5.0], [4.0, 7.0, 5.0]], [[1.0, 5.0, 8.0], [8.0, 4.0, 1.0], [4.0, 7.0, 7.0]]], [[[1.0, 3.0, 8.0], 
[5.0, 8.0, 3.0], [4.0, 2.0, 3.0]], [[2.0, 8.0, 5.0], [4.0, 1.0, 5.0], [2.0, 1.0, 5.0]], [[8.0, 2.0, 6.0], [7.0, 3.0, 3.0], 
[4.0, 6.0, 6.0]]], [[[3.0, 1.0, 8.0], [7.0, 1.0, 4.0], [2.0, 5.0, 4.0]], [[3.0, 5.0, 4.0], [8.0, 5.0, 5.0], [8.0, 1.0, 5.0]], 
[[8.0, 7.0, 2.0], [8.0, 1.0, 7.0], [8.0, 8.0, 5.0]]], [[[5.0, 6.0, 8.0], [4.0, 5.0, 2.0], [2.0, 8.0, 5.0]], [[6.0, 7.0, 6.0], 
[5.0, 4.0, 1.0], [7.0, 6.0, 6.0]], [[2.0, 3.0, 6.0], [4.0, 1.0, 3.0], [5.0, 7.0, 2.0]]]])
      val=Data(numpy.array([[[[1.0, 6.0, 3.0], [7.0, 4.0, 1.0], [2.0, 1.0, 7.0]]], [[[7.0, 8.0, 2.0], [2.0, 6.0, 4.0], [2.0, 
3.0, 2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[5.0, 7.0, 3.0], [5.0, 5.0, 2.0], [6.0, 3.0, 7.0]]], [[[2.0, 1.0, 6.0], [6.0, 6.0, 
5.0], [7.0, 3.0, 1.0]]]]))
      arg[1:3,1:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 3.0], [1.0, 6.0, 4.0], [2.0, 5.0, 8.0]], [[8.0, 1.0, 3.0], [2.0, 4.0, 3.0], [3.0, 
2.0, 5.0]], [[3.0, 2.0, 2.0], [7.0, 8.0, 1.0], [6.0, 7.0, 7.0]]], [[[8.0, 7.0, 4.0], [2.0, 1.0, 8.0], [3.0, 3.0, 2.0]], [[1.0, 
6.0, 3.0], [7.0, 4.0, 1.0], [2.0, 1.0, 7.0]], [[6.0, 5.0, 4.0], [4.0, 7.0, 7.0], [5.0, 5.0, 8.0]]], [[[7.0, 7.0, 4.0], [1.0, 
4.0, 6.0], [7.0, 3.0, 5.0]], [[7.0, 8.0, 2.0], [2.0, 6.0, 4.0], [2.0, 3.0, 2.0]], [[1.0, 1.0, 1.0], [1.0, 3.0, 7.0], [8.0, 2.0, 
1.0]]], [[[7.0, 4.0, 2.0], [4.0, 1.0, 3.0], [8.0, 5.0, 5.0]], [[6.0, 3.0, 4.0], [6.0, 8.0, 4.0], [7.0, 3.0, 7.0]], [[2.0, 3.0, 
4.0], [1.0, 4.0, 4.0], [3.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 3.0], [2.0, 8.0, 4.0], [8.0, 1.0, 5.0]], 
[[1.0, 3.0, 6.0], [5.0, 3.0, 5.0], [4.0, 7.0, 5.0]], [[1.0, 5.0, 8.0], [8.0, 4.0, 1.0], [4.0, 7.0, 7.0]]], [[[1.0, 3.0, 8.0], 
[5.0, 8.0, 3.0], [4.0, 2.0, 3.0]], [[5.0, 7.0, 3.0], [5.0, 5.0, 2.0], [6.0, 3.0, 7.0]], [[8.0, 2.0, 6.0], [7.0, 3.0, 3.0], 
[4.0, 6.0, 6.0]]], [[[3.0, 1.0, 8.0], [7.0, 1.0, 4.0], [2.0, 5.0, 4.0]], [[2.0, 1.0, 6.0], [6.0, 6.0, 5.0], [7.0, 3.0, 1.0]], 
[[8.0, 7.0, 2.0], [8.0, 1.0, 7.0], [8.0, 8.0, 5.0]]], [[[5.0, 6.0, 8.0], [4.0, 5.0, 2.0], [2.0, 8.0, 5.0]], [[6.0, 7.0, 6.0], 
[5.0, 4.0, 1.0], [7.0, 6.0, 6.0]], [[2.0, 3.0, 6.0], [4.0, 1.0, 3.0], [5.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UUPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 1.0], [5.0, 6.0, 5.0], [6.0, 1.0, 1.0]], [[8.0, 3.0, 3.0], [6.0, 5.0, 6.0], [5.0, 
2.0, 4.0]], [[1.0, 2.0, 5.0], [5.0, 1.0, 7.0], [3.0, 4.0, 3.0]]], [[[2.0, 2.0, 1.0], [7.0, 3.0, 5.0], [3.0, 7.0, 2.0]], [[1.0, 
7.0, 2.0], [1.0, 4.0, 4.0], [6.0, 6.0, 7.0]], [[7.0, 6.0, 8.0], [5.0, 6.0, 1.0], [8.0, 3.0, 7.0]]], [[[8.0, 7.0, 7.0], [6.0, 
6.0, 7.0], [4.0, 4.0, 3.0]], [[7.0, 2.0, 3.0], [6.0, 7.0, 1.0], [5.0, 2.0, 3.0]], [[4.0, 7.0, 6.0], [2.0, 5.0, 6.0], [5.0, 2.0, 
4.0]]], [[[7.0, 7.0, 4.0], [5.0, 3.0, 8.0], [6.0, 6.0, 3.0]], [[7.0, 4.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 1.0]], [[2.0, 7.0, 
7.0], [7.0, 6.0, 6.0], [7.0, 6.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 3.0], [6.0, 3.0, 5.0], [1.0, 1.0, 5.0]], 
[[7.0, 7.0, 4.0], [6.0, 4.0, 6.0], [4.0, 4.0, 4.0]], [[4.0, 2.0, 4.0], [4.0, 2.0, 4.0], [2.0, 7.0, 8.0]]], [[[4.0, 7.0, 5.0], 
[7.0, 1.0, 4.0], [8.0, 2.0, 5.0]], [[7.0, 1.0, 2.0], [1.0, 2.0, 4.0], [2.0, 5.0, 4.0]], [[7.0, 8.0, 5.0], [8.0, 8.0, 4.0], 
[2.0, 8.0, 2.0]]], [[[8.0, 6.0, 5.0], [7.0, 3.0, 7.0], [8.0, 7.0, 2.0]], [[2.0, 1.0, 6.0], [7.0, 2.0, 2.0], [4.0, 2.0, 1.0]], 
[[1.0, 6.0, 3.0], [8.0, 5.0, 1.0], [2.0, 4.0, 4.0]]], [[[1.0, 2.0, 2.0], [1.0, 7.0, 2.0], [5.0, 1.0, 3.0]], [[5.0, 3.0, 7.0], 
[4.0, 8.0, 1.0], [1.0, 5.0, 8.0]], [[1.0, 3.0, 5.0], [5.0, 2.0, 7.0], [6.0, 1.0, 2.0]]]])
      val=Data(numpy.array([[[1.0, 2.0, 4.0], [7.0, 5.0, 4.0]], [[5.0, 7.0, 1.0], [4.0, 5.0, 3.0]], [[5.0, 5.0, 6.0], [4.0, 
4.0, 1.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[3.0, 2.0, 4.0], [3.0, 1.0, 4.0]], [[2.0, 6.0, 6.0], [5.0, 7.0, 8.0]], [[2.0, 1.0, 
6.0], [4.0, 7.0, 3.0]]]))
      arg[:3,:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 1.0], [1.0, 2.0, 4.0], [6.0, 1.0, 1.0]], [[8.0, 3.0, 3.0], [7.0, 5.0, 4.0], [5.0, 
2.0, 4.0]], [[1.0, 2.0, 5.0], [5.0, 1.0, 7.0], [3.0, 4.0, 3.0]]], [[[2.0, 2.0, 1.0], [5.0, 7.0, 1.0], [3.0, 7.0, 2.0]], [[1.0, 
7.0, 2.0], [4.0, 5.0, 3.0], [6.0, 6.0, 7.0]], [[7.0, 6.0, 8.0], [5.0, 6.0, 1.0], [8.0, 3.0, 7.0]]], [[[8.0, 7.0, 7.0], [5.0, 
5.0, 6.0], [4.0, 4.0, 3.0]], [[7.0, 2.0, 3.0], [4.0, 4.0, 1.0], [5.0, 2.0, 3.0]], [[4.0, 7.0, 6.0], [2.0, 5.0, 6.0], [5.0, 2.0, 
4.0]]], [[[7.0, 7.0, 4.0], [5.0, 3.0, 8.0], [6.0, 6.0, 3.0]], [[7.0, 4.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 1.0]], [[2.0, 7.0, 
7.0], [7.0, 6.0, 6.0], [7.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 3.0], [3.0, 2.0, 4.0], [1.0, 1.0, 5.0]], 
[[7.0, 7.0, 4.0], [3.0, 1.0, 4.0], [4.0, 4.0, 4.0]], [[4.0, 2.0, 4.0], [4.0, 2.0, 4.0], [2.0, 7.0, 8.0]]], [[[4.0, 7.0, 5.0], 
[2.0, 6.0, 6.0], [8.0, 2.0, 5.0]], [[7.0, 1.0, 2.0], [5.0, 7.0, 8.0], [2.0, 5.0, 4.0]], [[7.0, 8.0, 5.0], [8.0, 8.0, 4.0], 
[2.0, 8.0, 2.0]]], [[[8.0, 6.0, 5.0], [2.0, 1.0, 6.0], [8.0, 7.0, 2.0]], [[2.0, 1.0, 6.0], [4.0, 7.0, 3.0], [4.0, 2.0, 1.0]], 
[[1.0, 6.0, 3.0], [8.0, 5.0, 1.0], [2.0, 4.0, 4.0]]], [[[1.0, 2.0, 2.0], [1.0, 7.0, 2.0], [5.0, 1.0, 3.0]], [[5.0, 3.0, 7.0], 
[4.0, 8.0, 1.0], [1.0, 5.0, 8.0]], [[1.0, 3.0, 5.0], [5.0, 2.0, 7.0], [6.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_RUCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[1.0, 5.0, 2.0], [8.0, 7.0, 5.0], [1.0, 7.0, 6.0]], [[1.0, 5.0, 3.0], [2.0, 7.0, 4.0], [6.0, 
1.0, 4.0]], [[6.0, 2.0, 4.0], [3.0, 7.0, 1.0], [1.0, 6.0, 2.0]]], [[[6.0, 5.0, 7.0], [4.0, 1.0, 8.0], [6.0, 7.0, 1.0]], [[6.0, 
5.0, 8.0], [5.0, 3.0, 3.0], [1.0, 2.0, 6.0]], [[2.0, 5.0, 5.0], [2.0, 3.0, 8.0], [5.0, 1.0, 7.0]]], [[[1.0, 1.0, 6.0], [4.0, 
2.0, 6.0], [4.0, 2.0, 4.0]], [[2.0, 3.0, 1.0], [3.0, 1.0, 4.0], [8.0, 2.0, 6.0]], [[7.0, 3.0, 7.0], [7.0, 2.0, 5.0], [1.0, 1.0, 
6.0]]], [[[6.0, 8.0, 4.0], [6.0, 6.0, 6.0], [6.0, 7.0, 1.0]], [[6.0, 8.0, 5.0], [2.0, 2.0, 1.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 
2.0], [4.0, 5.0, 8.0], [3.0, 7.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 6.0], [6.0, 8.0, 7.0], [2.0, 7.0, 7.0]], 
[[6.0, 7.0, 8.0], [4.0, 8.0, 6.0], [3.0, 8.0, 3.0]], [[1.0, 8.0, 7.0], [1.0, 8.0, 3.0], [4.0, 6.0, 5.0]]], [[[2.0, 2.0, 1.0], 
[1.0, 2.0, 7.0], [3.0, 2.0, 7.0]], [[7.0, 8.0, 7.0], [3.0, 7.0, 2.0], [6.0, 3.0, 5.0]], [[2.0, 2.0, 7.0], [8.0, 2.0, 6.0], 
[7.0, 6.0, 1.0]]], [[[1.0, 4.0, 3.0], [7.0, 5.0, 5.0], [4.0, 8.0, 7.0]], [[8.0, 4.0, 3.0], [4.0, 1.0, 5.0], [8.0, 1.0, 8.0]], 
[[5.0, 1.0, 5.0], [4.0, 1.0, 2.0], [4.0, 5.0, 8.0]]], [[[5.0, 1.0, 1.0], [1.0, 5.0, 4.0], [4.0, 1.0, 5.0]], [[8.0, 7.0, 2.0], 
[5.0, 2.0, 6.0], [6.0, 5.0, 2.0]], [[3.0, 8.0, 7.0], [3.0, 5.0, 7.0], [2.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[[4.0, 6.0, 7.0], [6.0, 6.0, 4.0], [6.0, 2.0, 7.0]], [[1.0, 2.0, 2.0], [1.0, 7.0, 1.0], [8.0, 6.0, 
8.0]]], [[[8.0, 4.0, 2.0], [7.0, 8.0, 4.0], [2.0, 2.0, 6.0]], [[3.0, 7.0, 7.0], [2.0, 4.0, 7.0], [3.0, 4.0, 
2.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[7.0, 6.0, 8.0], [2.0, 8.0, 4.0], [1.0, 3.0, 4.0]], [[3.0, 8.0, 6.0], [2.0, 5.0, 
2.0], [1.0, 8.0, 4.0]]], [[[5.0, 4.0, 3.0], [3.0, 1.0, 3.0], [1.0, 5.0, 5.0]], [[1.0, 3.0, 8.0], [6.0, 8.0, 7.0], [1.0, 8.0, 
1.0]]]]))
      arg[1:3,:2,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 2.0], [8.0, 7.0, 5.0], [1.0, 7.0, 6.0]], [[1.0, 5.0, 3.0], [2.0, 7.0, 4.0], [6.0, 
1.0, 4.0]], [[6.0, 2.0, 4.0], [3.0, 7.0, 1.0], [1.0, 6.0, 2.0]]], [[[4.0, 6.0, 7.0], [6.0, 6.0, 4.0], [6.0, 2.0, 7.0]], [[1.0, 
2.0, 2.0], [1.0, 7.0, 1.0], [8.0, 6.0, 8.0]], [[2.0, 5.0, 5.0], [2.0, 3.0, 8.0], [5.0, 1.0, 7.0]]], [[[8.0, 4.0, 2.0], [7.0, 
8.0, 4.0], [2.0, 2.0, 6.0]], [[3.0, 7.0, 7.0], [2.0, 4.0, 7.0], [3.0, 4.0, 2.0]], [[7.0, 3.0, 7.0], [7.0, 2.0, 5.0], [1.0, 1.0, 
6.0]]], [[[6.0, 8.0, 4.0], [6.0, 6.0, 6.0], [6.0, 7.0, 1.0]], [[6.0, 8.0, 5.0], [2.0, 2.0, 1.0], [5.0, 8.0, 3.0]], [[8.0, 1.0, 
2.0], [4.0, 5.0, 8.0], [3.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 6.0], [6.0, 8.0, 7.0], [2.0, 7.0, 7.0]], 
[[6.0, 7.0, 8.0], [4.0, 8.0, 6.0], [3.0, 8.0, 3.0]], [[1.0, 8.0, 7.0], [1.0, 8.0, 3.0], [4.0, 6.0, 5.0]]], [[[7.0, 6.0, 8.0], 
[2.0, 8.0, 4.0], [1.0, 3.0, 4.0]], [[3.0, 8.0, 6.0], [2.0, 5.0, 2.0], [1.0, 8.0, 4.0]], [[2.0, 2.0, 7.0], [8.0, 2.0, 6.0], 
[7.0, 6.0, 1.0]]], [[[5.0, 4.0, 3.0], [3.0, 1.0, 3.0], [1.0, 5.0, 5.0]], [[1.0, 3.0, 8.0], [6.0, 8.0, 7.0], [1.0, 8.0, 1.0]], 
[[5.0, 1.0, 5.0], [4.0, 1.0, 2.0], [4.0, 5.0, 8.0]]], [[[5.0, 1.0, 1.0], [1.0, 5.0, 4.0], [4.0, 1.0, 5.0]], [[8.0, 7.0, 2.0], 
[5.0, 2.0, 6.0], [6.0, 5.0, 2.0]], [[3.0, 8.0, 7.0], [3.0, 5.0, 7.0], [2.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CRRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 7.0], [7.0, 2.0, 3.0], [4.0, 3.0, 4.0]], [[1.0, 4.0, 4.0], [5.0, 2.0, 7.0], [7.0, 
3.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 8.0, 4.0], [8.0, 6.0, 6.0]]], [[[4.0, 1.0, 6.0], [8.0, 4.0, 4.0], [4.0, 8.0, 5.0]], [[2.0, 
6.0, 3.0], [1.0, 1.0, 8.0], [7.0, 6.0, 7.0]], [[7.0, 5.0, 1.0], [6.0, 1.0, 4.0], [8.0, 7.0, 7.0]]], [[[1.0, 4.0, 6.0], [8.0, 
4.0, 5.0], [2.0, 8.0, 7.0]], [[3.0, 7.0, 1.0], [4.0, 1.0, 7.0], [2.0, 5.0, 8.0]], [[3.0, 5.0, 4.0], [6.0, 4.0, 3.0], [3.0, 6.0, 
2.0]]], [[[2.0, 8.0, 8.0], [3.0, 1.0, 6.0], [3.0, 1.0, 3.0]], [[7.0, 8.0, 4.0], [5.0, 8.0, 1.0], [8.0, 7.0, 2.0]], [[7.0, 2.0, 
2.0], [2.0, 1.0, 2.0], [7.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 6.0], [4.0, 4.0, 4.0], [2.0, 8.0, 8.0]], 
[[1.0, 1.0, 1.0], [7.0, 7.0, 5.0], [8.0, 3.0, 1.0]], [[6.0, 1.0, 1.0], [1.0, 1.0, 2.0], [4.0, 1.0, 2.0]]], [[[5.0, 7.0, 5.0], 
[7.0, 7.0, 1.0], [3.0, 2.0, 4.0]], [[6.0, 8.0, 6.0], [6.0, 8.0, 1.0], [2.0, 1.0, 6.0]], [[1.0, 4.0, 3.0], [3.0, 8.0, 2.0], 
[6.0, 8.0, 8.0]]], [[[3.0, 1.0, 3.0], [6.0, 5.0, 4.0], [7.0, 1.0, 1.0]], [[3.0, 1.0, 4.0], [1.0, 8.0, 7.0], [6.0, 4.0, 4.0]], 
[[3.0, 1.0, 5.0], [3.0, 4.0, 8.0], [1.0, 8.0, 5.0]]], [[[4.0, 1.0, 4.0], [4.0, 6.0, 6.0], [8.0, 8.0, 5.0]], [[7.0, 6.0, 5.0], 
[2.0, 7.0, 7.0], [7.0, 1.0, 4.0]], [[5.0, 4.0, 8.0], [2.0, 7.0, 5.0], [3.0, 4.0, 6.0]]]])
      val=Data(numpy.array([[[[5.0, 8.0, 2.0]]], [[[3.0, 8.0, 4.0]]], [[[6.0, 2.0, 2.0]]], [[[1.0, 2.0, 
8.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[3.0, 8.0, 4.0]]], [[[5.0, 6.0, 6.0]]], [[[7.0, 1.0, 6.0]]], [[[2.0, 2.0, 6.0]]]]))
      arg[:,1:2,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 7.0], [7.0, 2.0, 3.0], [4.0, 3.0, 4.0]], [[1.0, 4.0, 4.0], [5.0, 8.0, 2.0], [7.0, 
3.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 8.0, 4.0], [8.0, 6.0, 6.0]]], [[[4.0, 1.0, 6.0], [8.0, 4.0, 4.0], [4.0, 8.0, 5.0]], [[2.0, 
6.0, 3.0], [3.0, 8.0, 4.0], [7.0, 6.0, 7.0]], [[7.0, 5.0, 1.0], [6.0, 1.0, 4.0], [8.0, 7.0, 7.0]]], [[[1.0, 4.0, 6.0], [8.0, 
4.0, 5.0], [2.0, 8.0, 7.0]], [[3.0, 7.0, 1.0], [6.0, 2.0, 2.0], [2.0, 5.0, 8.0]], [[3.0, 5.0, 4.0], [6.0, 4.0, 3.0], [3.0, 6.0, 
2.0]]], [[[2.0, 8.0, 8.0], [3.0, 1.0, 6.0], [3.0, 1.0, 3.0]], [[7.0, 8.0, 4.0], [1.0, 2.0, 8.0], [8.0, 7.0, 2.0]], [[7.0, 2.0, 
2.0], [2.0, 1.0, 2.0], [7.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 6.0], [4.0, 4.0, 4.0], [2.0, 8.0, 8.0]], 
[[1.0, 1.0, 1.0], [3.0, 8.0, 4.0], [8.0, 3.0, 1.0]], [[6.0, 1.0, 1.0], [1.0, 1.0, 2.0], [4.0, 1.0, 2.0]]], [[[5.0, 7.0, 5.0], 
[7.0, 7.0, 1.0], [3.0, 2.0, 4.0]], [[6.0, 8.0, 6.0], [5.0, 6.0, 6.0], [2.0, 1.0, 6.0]], [[1.0, 4.0, 3.0], [3.0, 8.0, 2.0], 
[6.0, 8.0, 8.0]]], [[[3.0, 1.0, 3.0], [6.0, 5.0, 4.0], [7.0, 1.0, 1.0]], [[3.0, 1.0, 4.0], [7.0, 1.0, 6.0], [6.0, 4.0, 4.0]], 
[[3.0, 1.0, 5.0], [3.0, 4.0, 8.0], [1.0, 8.0, 5.0]]], [[[4.0, 1.0, 4.0], [4.0, 6.0, 6.0], [8.0, 8.0, 5.0]], [[7.0, 6.0, 5.0], 
[2.0, 2.0, 6.0], [7.0, 1.0, 4.0]], [[5.0, 4.0, 8.0], [2.0, 7.0, 5.0], [3.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UCLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 1.0], [7.0, 3.0, 6.0], [8.0, 6.0, 2.0]], [[7.0, 3.0, 8.0], [1.0, 4.0, 8.0], [2.0, 
3.0, 2.0]], [[1.0, 2.0, 1.0], [6.0, 4.0, 8.0], [7.0, 1.0, 6.0]]], [[[4.0, 1.0, 8.0], [3.0, 3.0, 6.0], [4.0, 7.0, 7.0]], [[2.0, 
3.0, 7.0], [4.0, 7.0, 7.0], [4.0, 7.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 2.0, 8.0], [7.0, 2.0, 6.0]]], [[[7.0, 6.0, 2.0], [7.0, 
8.0, 3.0], [6.0, 7.0, 4.0]], [[8.0, 4.0, 1.0], [7.0, 8.0, 5.0], [8.0, 1.0, 5.0]], [[3.0, 5.0, 4.0], [8.0, 8.0, 2.0], [6.0, 8.0, 
2.0]]], [[[5.0, 6.0, 8.0], [1.0, 8.0, 2.0], [7.0, 3.0, 3.0]], [[1.0, 3.0, 6.0], [3.0, 7.0, 7.0], [5.0, 5.0, 4.0]], [[2.0, 8.0, 
3.0], [5.0, 5.0, 1.0], [2.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 2.0], [7.0, 6.0, 8.0], [2.0, 8.0, 8.0]], 
[[4.0, 2.0, 7.0], [2.0, 4.0, 7.0], [2.0, 4.0, 3.0]], [[8.0, 8.0, 2.0], [4.0, 8.0, 6.0], [6.0, 1.0, 8.0]]], [[[4.0, 7.0, 6.0], 
[2.0, 4.0, 8.0], [1.0, 2.0, 8.0]], [[7.0, 5.0, 7.0], [7.0, 6.0, 7.0], [6.0, 6.0, 8.0]], [[5.0, 2.0, 3.0], [8.0, 5.0, 8.0], 
[6.0, 3.0, 6.0]]], [[[6.0, 8.0, 8.0], [8.0, 8.0, 1.0], [4.0, 8.0, 5.0]], [[5.0, 2.0, 2.0], [1.0, 1.0, 6.0], [5.0, 6.0, 6.0]], 
[[3.0, 2.0, 1.0], [3.0, 6.0, 4.0], [1.0, 5.0, 6.0]]], [[[5.0, 5.0, 1.0], [1.0, 3.0, 1.0], [8.0, 3.0, 8.0]], [[3.0, 2.0, 1.0], 
[6.0, 7.0, 2.0], [8.0, 4.0, 8.0]], [[2.0, 1.0, 7.0], [2.0, 8.0, 6.0], [4.0, 3.0, 3.0]]]])
      val=Data(numpy.array([[[[3.0, 7.0, 8.0]], [[7.0, 7.0, 8.0]], [[3.0, 3.0, 4.0]]], [[[2.0, 5.0, 8.0]], [[7.0, 1.0, 3.0]], 
[[7.0, 3.0, 5.0]]], [[[7.0, 1.0, 5.0]], [[5.0, 4.0, 5.0]], [[5.0, 8.0, 5.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 8.0, 4.0]], [[4.0, 4.0, 1.0]], [[2.0, 6.0, 3.0]]], [[[7.0, 2.0, 2.0]], [[6.0, 
2.0, 3.0]], [[1.0, 2.0, 6.0]]], [[[7.0, 2.0, 2.0]], [[7.0, 4.0, 1.0]], [[5.0, 8.0, 4.0]]]]))
      arg[:3,:,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 1.0], [7.0, 3.0, 6.0], [3.0, 7.0, 8.0]], [[7.0, 3.0, 8.0], [1.0, 4.0, 8.0], [7.0, 
7.0, 8.0]], [[1.0, 2.0, 1.0], [6.0, 4.0, 8.0], [3.0, 3.0, 4.0]]], [[[4.0, 1.0, 8.0], [3.0, 3.0, 6.0], [2.0, 5.0, 8.0]], [[2.0, 
3.0, 7.0], [4.0, 7.0, 7.0], [7.0, 1.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 2.0, 8.0], [7.0, 3.0, 5.0]]], [[[7.0, 6.0, 2.0], [7.0, 
8.0, 3.0], [7.0, 1.0, 5.0]], [[8.0, 4.0, 1.0], [7.0, 8.0, 5.0], [5.0, 4.0, 5.0]], [[3.0, 5.0, 4.0], [8.0, 8.0, 2.0], [5.0, 8.0, 
5.0]]], [[[5.0, 6.0, 8.0], [1.0, 8.0, 2.0], [7.0, 3.0, 3.0]], [[1.0, 3.0, 6.0], [3.0, 7.0, 7.0], [5.0, 5.0, 4.0]], [[2.0, 8.0, 
3.0], [5.0, 5.0, 1.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 2.0], [7.0, 6.0, 8.0], [8.0, 8.0, 4.0]], 
[[4.0, 2.0, 7.0], [2.0, 4.0, 7.0], [4.0, 4.0, 1.0]], [[8.0, 8.0, 2.0], [4.0, 8.0, 6.0], [2.0, 6.0, 3.0]]], [[[4.0, 7.0, 6.0], 
[2.0, 4.0, 8.0], [7.0, 2.0, 2.0]], [[7.0, 5.0, 7.0], [7.0, 6.0, 7.0], [6.0, 2.0, 3.0]], [[5.0, 2.0, 3.0], [8.0, 5.0, 8.0], 
[1.0, 2.0, 6.0]]], [[[6.0, 8.0, 8.0], [8.0, 8.0, 1.0], [7.0, 2.0, 2.0]], [[5.0, 2.0, 2.0], [1.0, 1.0, 6.0], [7.0, 4.0, 1.0]], 
[[3.0, 2.0, 1.0], [3.0, 6.0, 4.0], [5.0, 8.0, 4.0]]], [[[5.0, 5.0, 1.0], [1.0, 3.0, 1.0], [8.0, 3.0, 8.0]], [[3.0, 2.0, 1.0], 
[6.0, 7.0, 2.0], [8.0, 4.0, 8.0]], [[2.0, 1.0, 7.0], [2.0, 8.0, 6.0], [4.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PRPN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 7.0], [3.0, 8.0, 5.0], [5.0, 2.0, 2.0]], [[5.0, 8.0, 2.0], [2.0, 8.0, 6.0], [1.0, 
2.0, 7.0]], [[6.0, 5.0, 7.0], [5.0, 1.0, 3.0], [3.0, 4.0, 7.0]]], [[[8.0, 6.0, 6.0], [4.0, 7.0, 2.0], [3.0, 5.0, 4.0]], [[1.0, 
3.0, 7.0], [6.0, 8.0, 3.0], [4.0, 5.0, 5.0]], [[2.0, 4.0, 4.0], [4.0, 2.0, 5.0], [5.0, 8.0, 3.0]]], [[[8.0, 8.0, 4.0], [5.0, 
6.0, 2.0], [8.0, 3.0, 5.0]], [[3.0, 6.0, 7.0], [1.0, 5.0, 4.0], [5.0, 7.0, 4.0]], [[8.0, 2.0, 6.0], [6.0, 4.0, 5.0], [2.0, 4.0, 
3.0]]], [[[5.0, 3.0, 8.0], [3.0, 6.0, 1.0], [2.0, 4.0, 8.0]], [[8.0, 5.0, 6.0], [2.0, 5.0, 3.0], [7.0, 3.0, 3.0]], [[4.0, 4.0, 
3.0], [5.0, 8.0, 7.0], [2.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 3.0], [7.0, 2.0, 1.0], [1.0, 4.0, 6.0]], 
[[1.0, 1.0, 1.0], [7.0, 2.0, 7.0], [7.0, 3.0, 1.0]], [[3.0, 8.0, 8.0], [4.0, 8.0, 3.0], [3.0, 3.0, 6.0]]], [[[5.0, 1.0, 8.0], 
[5.0, 2.0, 5.0], [2.0, 3.0, 2.0]], [[8.0, 5.0, 4.0], [8.0, 3.0, 5.0], [2.0, 4.0, 3.0]], [[6.0, 6.0, 1.0], [2.0, 6.0, 3.0], 
[1.0, 2.0, 1.0]]], [[[5.0, 1.0, 4.0], [3.0, 2.0, 8.0], [3.0, 5.0, 8.0]], [[2.0, 1.0, 7.0], [4.0, 5.0, 8.0], [5.0, 3.0, 5.0]], 
[[7.0, 8.0, 7.0], [4.0, 6.0, 8.0], [3.0, 8.0, 8.0]]], [[[6.0, 3.0, 8.0], [8.0, 7.0, 7.0], [4.0, 5.0, 4.0]], [[8.0, 3.0, 8.0], 
[7.0, 7.0, 6.0], [6.0, 1.0, 1.0]], [[8.0, 5.0, 3.0], [3.0, 7.0, 7.0], [3.0, 4.0, 7.0]]]])
      val=Data(numpy.array([[1.0, 6.0, 1.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[8.0, 1.0, 5.0]]))
      arg[2,1:2,1]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 7.0], [3.0, 8.0, 5.0], [5.0, 2.0, 2.0]], [[5.0, 8.0, 2.0], [2.0, 8.0, 6.0], [1.0, 
2.0, 7.0]], [[6.0, 5.0, 7.0], [5.0, 1.0, 3.0], [3.0, 4.0, 7.0]]], [[[8.0, 6.0, 6.0], [4.0, 7.0, 2.0], [3.0, 5.0, 4.0]], [[1.0, 
3.0, 7.0], [6.0, 8.0, 3.0], [4.0, 5.0, 5.0]], [[2.0, 4.0, 4.0], [4.0, 2.0, 5.0], [5.0, 8.0, 3.0]]], [[[8.0, 8.0, 4.0], [5.0, 
6.0, 2.0], [8.0, 3.0, 5.0]], [[3.0, 6.0, 7.0], [1.0, 6.0, 1.0], [5.0, 7.0, 4.0]], [[8.0, 2.0, 6.0], [6.0, 4.0, 5.0], [2.0, 4.0, 
3.0]]], [[[5.0, 3.0, 8.0], [3.0, 6.0, 1.0], [2.0, 4.0, 8.0]], [[8.0, 5.0, 6.0], [2.0, 5.0, 3.0], [7.0, 3.0, 3.0]], [[4.0, 4.0, 
3.0], [5.0, 8.0, 7.0], [2.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 3.0], [7.0, 2.0, 1.0], [1.0, 4.0, 6.0]], 
[[1.0, 1.0, 1.0], [7.0, 2.0, 7.0], [7.0, 3.0, 1.0]], [[3.0, 8.0, 8.0], [4.0, 8.0, 3.0], [3.0, 3.0, 6.0]]], [[[5.0, 1.0, 8.0], 
[5.0, 2.0, 5.0], [2.0, 3.0, 2.0]], [[8.0, 5.0, 4.0], [8.0, 3.0, 5.0], [2.0, 4.0, 3.0]], [[6.0, 6.0, 1.0], [2.0, 6.0, 3.0], 
[1.0, 2.0, 1.0]]], [[[5.0, 1.0, 4.0], [3.0, 2.0, 8.0], [3.0, 5.0, 8.0]], [[2.0, 1.0, 7.0], [8.0, 1.0, 5.0], [5.0, 3.0, 5.0]], 
[[7.0, 8.0, 7.0], [4.0, 6.0, 8.0], [3.0, 8.0, 8.0]]], [[[6.0, 3.0, 8.0], [8.0, 7.0, 7.0], [4.0, 5.0, 4.0]], [[8.0, 3.0, 8.0], 
[7.0, 7.0, 6.0], [6.0, 1.0, 1.0]], [[8.0, 5.0, 3.0], [3.0, 7.0, 7.0], [3.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_CPLN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 8.0], [2.0, 1.0, 5.0], [4.0, 4.0, 6.0]], [[4.0, 8.0, 7.0], [1.0, 3.0, 6.0], [2.0, 
8.0, 5.0]], [[4.0, 3.0, 3.0], [7.0, 4.0, 5.0], [6.0, 2.0, 8.0]]], [[[4.0, 1.0, 3.0], [4.0, 4.0, 2.0], [7.0, 8.0, 8.0]], [[5.0, 
6.0, 1.0], [6.0, 4.0, 6.0], [5.0, 7.0, 5.0]], [[8.0, 4.0, 6.0], [8.0, 4.0, 8.0], [5.0, 6.0, 7.0]]], [[[4.0, 2.0, 3.0], [5.0, 
8.0, 1.0], [1.0, 2.0, 1.0]], [[7.0, 2.0, 6.0], [1.0, 5.0, 8.0], [8.0, 2.0, 6.0]], [[3.0, 1.0, 6.0], [3.0, 3.0, 7.0], [8.0, 5.0, 
3.0]]], [[[5.0, 4.0, 6.0], [5.0, 3.0, 7.0], [3.0, 4.0, 7.0]], [[5.0, 2.0, 1.0], [1.0, 7.0, 7.0], [2.0, 6.0, 1.0]], [[7.0, 4.0, 
8.0], [4.0, 1.0, 7.0], [3.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 3.0], [1.0, 7.0, 7.0], [2.0, 4.0, 7.0]], 
[[1.0, 3.0, 4.0], [5.0, 8.0, 2.0], [7.0, 7.0, 8.0]], [[8.0, 7.0, 3.0], [5.0, 1.0, 7.0], [4.0, 8.0, 2.0]]], [[[7.0, 5.0, 5.0], 
[5.0, 3.0, 5.0], [7.0, 3.0, 1.0]], [[6.0, 2.0, 7.0], [4.0, 8.0, 8.0], [4.0, 6.0, 4.0]], [[6.0, 4.0, 2.0], [8.0, 4.0, 5.0], 
[2.0, 6.0, 7.0]]], [[[7.0, 1.0, 6.0], [6.0, 8.0, 7.0], [4.0, 3.0, 7.0]], [[7.0, 5.0, 6.0], [5.0, 8.0, 1.0], [7.0, 4.0, 1.0]], 
[[6.0, 7.0, 1.0], [8.0, 5.0, 3.0], [7.0, 6.0, 6.0]]], [[[5.0, 8.0, 6.0], [3.0, 4.0, 6.0], [7.0, 4.0, 5.0]], [[7.0, 2.0, 1.0], 
[3.0, 2.0, 5.0], [6.0, 4.0, 6.0]], [[6.0, 4.0, 1.0], [1.0, 3.0, 8.0], [7.0, 3.0, 7.0]]]])
      val=Data(numpy.array([[[7.0, 3.0, 4.0]], [[8.0, 5.0, 8.0]], [[6.0, 1.0, 8.0]], [[5.0, 5.0, 6.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[4.0, 5.0, 5.0]], [[5.0, 4.0, 8.0]], [[6.0, 5.0, 8.0]], [[6.0, 5.0, 8.0]]]))
      arg[:,1,2:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 8.0], [2.0, 1.0, 5.0], [4.0, 4.0, 6.0]], [[4.0, 8.0, 7.0], [1.0, 3.0, 6.0], [7.0, 
3.0, 4.0]], [[4.0, 3.0, 3.0], [7.0, 4.0, 5.0], [6.0, 2.0, 8.0]]], [[[4.0, 1.0, 3.0], [4.0, 4.0, 2.0], [7.0, 8.0, 8.0]], [[5.0, 
6.0, 1.0], [6.0, 4.0, 6.0], [8.0, 5.0, 8.0]], [[8.0, 4.0, 6.0], [8.0, 4.0, 8.0], [5.0, 6.0, 7.0]]], [[[4.0, 2.0, 3.0], [5.0, 
8.0, 1.0], [1.0, 2.0, 1.0]], [[7.0, 2.0, 6.0], [1.0, 5.0, 8.0], [6.0, 1.0, 8.0]], [[3.0, 1.0, 6.0], [3.0, 3.0, 7.0], [8.0, 5.0, 
3.0]]], [[[5.0, 4.0, 6.0], [5.0, 3.0, 7.0], [3.0, 4.0, 7.0]], [[5.0, 2.0, 1.0], [1.0, 7.0, 7.0], [5.0, 5.0, 6.0]], [[7.0, 4.0, 
8.0], [4.0, 1.0, 7.0], [3.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 3.0], [1.0, 7.0, 7.0], [2.0, 4.0, 7.0]], 
[[1.0, 3.0, 4.0], [5.0, 8.0, 2.0], [4.0, 5.0, 5.0]], [[8.0, 7.0, 3.0], [5.0, 1.0, 7.0], [4.0, 8.0, 2.0]]], [[[7.0, 5.0, 5.0], 
[5.0, 3.0, 5.0], [7.0, 3.0, 1.0]], [[6.0, 2.0, 7.0], [4.0, 8.0, 8.0], [5.0, 4.0, 8.0]], [[6.0, 4.0, 2.0], [8.0, 4.0, 5.0], 
[2.0, 6.0, 7.0]]], [[[7.0, 1.0, 6.0], [6.0, 8.0, 7.0], [4.0, 3.0, 7.0]], [[7.0, 5.0, 6.0], [5.0, 8.0, 1.0], [6.0, 5.0, 8.0]], 
[[6.0, 7.0, 1.0], [8.0, 5.0, 3.0], [7.0, 6.0, 6.0]]], [[[5.0, 8.0, 6.0], [3.0, 4.0, 6.0], [7.0, 4.0, 5.0]], [[7.0, 2.0, 1.0], 
[3.0, 2.0, 5.0], [6.0, 5.0, 8.0]], [[6.0, 4.0, 1.0], [1.0, 3.0, 8.0], [7.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_UCCN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 2.0], [6.0, 3.0, 1.0], [2.0, 3.0, 1.0]], [[5.0, 7.0, 3.0], [6.0, 7.0, 7.0], [1.0, 
3.0, 7.0]], [[3.0, 8.0, 4.0], [4.0, 7.0, 8.0], [6.0, 6.0, 2.0]]], [[[2.0, 5.0, 5.0], [2.0, 7.0, 1.0], [8.0, 1.0, 4.0]], [[4.0, 
6.0, 6.0], [4.0, 6.0, 1.0], [7.0, 3.0, 5.0]], [[4.0, 5.0, 4.0], [1.0, 2.0, 1.0], [4.0, 7.0, 1.0]]], [[[6.0, 6.0, 1.0], [3.0, 
3.0, 2.0], [8.0, 8.0, 2.0]], [[5.0, 2.0, 3.0], [6.0, 6.0, 2.0], [8.0, 1.0, 6.0]], [[7.0, 2.0, 2.0], [5.0, 4.0, 1.0], [1.0, 6.0, 
3.0]]], [[[6.0, 1.0, 3.0], [1.0, 1.0, 2.0], [2.0, 8.0, 5.0]], [[4.0, 5.0, 4.0], [4.0, 6.0, 7.0], [2.0, 3.0, 2.0]], [[7.0, 3.0, 
1.0], [1.0, 8.0, 5.0], [3.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 1.0], [7.0, 2.0, 8.0], [2.0, 4.0, 8.0]], 
[[1.0, 2.0, 6.0], [2.0, 5.0, 2.0], [6.0, 8.0, 6.0]], [[3.0, 7.0, 3.0], [7.0, 2.0, 6.0], [2.0, 2.0, 2.0]]], [[[7.0, 8.0, 3.0], 
[5.0, 6.0, 2.0], [6.0, 8.0, 6.0]], [[2.0, 2.0, 6.0], [8.0, 5.0, 3.0], [3.0, 5.0, 4.0]], [[8.0, 5.0, 8.0], [8.0, 8.0, 1.0], 
[6.0, 1.0, 4.0]]], [[[8.0, 7.0, 7.0], [6.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[3.0, 1.0, 4.0], [5.0, 4.0, 1.0], [2.0, 3.0, 3.0]], 
[[8.0, 8.0, 2.0], [5.0, 4.0, 2.0], [7.0, 4.0, 8.0]]], [[[3.0, 8.0, 5.0], [1.0, 6.0, 2.0], [8.0, 5.0, 8.0]], [[7.0, 7.0, 5.0], 
[3.0, 7.0, 3.0], [3.0, 5.0, 6.0]], [[4.0, 1.0, 7.0], [3.0, 6.0, 5.0], [8.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[[[7.0, 4.0, 7.0], [1.0, 2.0, 4.0], [8.0, 6.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 4.0, 2.0], [3.0, 3.0, 
6.0]], [[4.0, 5.0, 1.0], [6.0, 7.0, 5.0], [1.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], [4.0, 1.0, 4.0], [1.0, 6.0, 6.0]], [[5.0, 8.0, 
4.0], [3.0, 3.0, 8.0], [6.0, 5.0, 8.0]], [[2.0, 5.0, 8.0], [8.0, 6.0, 3.0], [4.0, 8.0, 5.0]]], [[[2.0, 1.0, 3.0], [8.0, 5.0, 
7.0], [3.0, 8.0, 6.0]], [[3.0, 4.0, 3.0], [1.0, 7.0, 8.0], [7.0, 1.0, 4.0]], [[6.0, 1.0, 3.0], [7.0, 1.0, 2.0], [8.0, 6.0, 
4.0]]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[[8.0, 2.0, 8.0], [8.0, 4.0, 2.0], [5.0, 2.0, 5.0]], [[5.0, 3.0, 1.0], [2.0, 3.0, 
7.0], [7.0, 3.0, 1.0]], [[5.0, 7.0, 1.0], [2.0, 1.0, 2.0], [1.0, 1.0, 1.0]]], [[[8.0, 8.0, 5.0], [8.0, 7.0, 7.0], [3.0, 5.0, 
2.0]], [[5.0, 3.0, 4.0], [4.0, 7.0, 5.0], [4.0, 6.0, 1.0]], [[8.0, 3.0, 5.0], [3.0, 7.0, 4.0], [1.0, 3.0, 1.0]]], [[[6.0, 8.0, 
4.0], [6.0, 5.0, 3.0], [2.0, 7.0, 6.0]], [[8.0, 8.0, 5.0], [3.0, 4.0, 6.0], [1.0, 2.0, 1.0]], [[8.0, 1.0, 6.0], [4.0, 2.0, 
2.0], [3.0, 2.0, 4.0]]]]))
      arg[:3,:,:]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 7.0], [1.0, 2.0, 4.0], [8.0, 6.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 4.0, 2.0], [3.0, 
3.0, 6.0]], [[4.0, 5.0, 1.0], [6.0, 7.0, 5.0], [1.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], [4.0, 1.0, 4.0], [1.0, 6.0, 6.0]], [[5.0, 
8.0, 4.0], [3.0, 3.0, 8.0], [6.0, 5.0, 8.0]], [[2.0, 5.0, 8.0], [8.0, 6.0, 3.0], [4.0, 8.0, 5.0]]], [[[2.0, 1.0, 3.0], [8.0, 
5.0, 7.0], [3.0, 8.0, 6.0]], [[3.0, 4.0, 3.0], [1.0, 7.0, 8.0], [7.0, 1.0, 4.0]], [[6.0, 1.0, 3.0], [7.0, 1.0, 2.0], [8.0, 6.0, 
4.0]]], [[[6.0, 1.0, 3.0], [1.0, 1.0, 2.0], [2.0, 8.0, 5.0]], [[4.0, 5.0, 4.0], [4.0, 6.0, 7.0], [2.0, 3.0, 2.0]], [[7.0, 3.0, 
1.0], [1.0, 8.0, 5.0], [3.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 8.0], [8.0, 4.0, 2.0], [5.0, 2.0, 5.0]], 
[[5.0, 3.0, 1.0], [2.0, 3.0, 7.0], [7.0, 3.0, 1.0]], [[5.0, 7.0, 1.0], [2.0, 1.0, 2.0], [1.0, 1.0, 1.0]]], [[[8.0, 8.0, 5.0], 
[8.0, 7.0, 7.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 4.0], [4.0, 7.0, 5.0], [4.0, 6.0, 1.0]], [[8.0, 3.0, 5.0], [3.0, 7.0, 4.0], 
[1.0, 3.0, 1.0]]], [[[6.0, 8.0, 4.0], [6.0, 5.0, 3.0], [2.0, 7.0, 6.0]], [[8.0, 8.0, 5.0], [3.0, 4.0, 6.0], [1.0, 2.0, 1.0]], 
[[8.0, 1.0, 6.0], [4.0, 2.0, 2.0], [3.0, 2.0, 4.0]]], [[[3.0, 8.0, 5.0], [1.0, 6.0, 2.0], [8.0, 5.0, 8.0]], [[7.0, 7.0, 5.0], 
[3.0, 7.0, 3.0], [3.0, 5.0, 6.0]], [[4.0, 1.0, 7.0], [3.0, 6.0, 5.0], [8.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PCRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 7.0, 2.0], [8.0, 8.0, 2.0], [7.0, 7.0, 7.0]], [[1.0, 7.0, 1.0], [6.0, 8.0, 6.0], [1.0, 
5.0, 5.0]], [[7.0, 5.0, 5.0], [4.0, 5.0, 5.0], [7.0, 7.0, 7.0]]], [[[4.0, 2.0, 3.0], [1.0, 8.0, 2.0], [6.0, 4.0, 6.0]], [[3.0, 
4.0, 7.0], [7.0, 5.0, 5.0], [8.0, 6.0, 8.0]], [[2.0, 3.0, 2.0], [4.0, 3.0, 6.0], [3.0, 4.0, 2.0]]], [[[8.0, 2.0, 8.0], [2.0, 
4.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 2.0, 7.0], [3.0, 2.0, 4.0], [4.0, 7.0, 1.0]], [[6.0, 4.0, 5.0], [4.0, 6.0, 1.0], [6.0, 6.0, 
4.0]]], [[[8.0, 8.0, 2.0], [6.0, 6.0, 6.0], [5.0, 7.0, 6.0]], [[8.0, 2.0, 1.0], [8.0, 7.0, 6.0], [8.0, 8.0, 1.0]], [[4.0, 8.0, 
2.0], [7.0, 4.0, 4.0], [4.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 7.0], [2.0, 1.0, 3.0], [2.0, 8.0, 6.0]], 
[[4.0, 1.0, 1.0], [4.0, 5.0, 7.0], [8.0, 6.0, 1.0]], [[7.0, 3.0, 2.0], [2.0, 5.0, 7.0], [2.0, 8.0, 5.0]]], [[[7.0, 3.0, 2.0], 
[8.0, 3.0, 4.0], [3.0, 6.0, 6.0]], [[8.0, 7.0, 8.0], [8.0, 7.0, 5.0], [5.0, 1.0, 6.0]], [[6.0, 5.0, 5.0], [2.0, 8.0, 1.0], 
[1.0, 2.0, 8.0]]], [[[8.0, 8.0, 4.0], [4.0, 3.0, 7.0], [6.0, 6.0, 4.0]], [[8.0, 3.0, 4.0], [3.0, 1.0, 3.0], [7.0, 7.0, 4.0]], 
[[1.0, 4.0, 8.0], [7.0, 4.0, 3.0], [4.0, 5.0, 1.0]]], [[[3.0, 2.0, 5.0], [5.0, 4.0, 1.0], [2.0, 5.0, 3.0]], [[4.0, 3.0, 5.0], 
[7.0, 1.0, 2.0], [8.0, 1.0, 6.0]], [[4.0, 8.0, 5.0], [4.0, 3.0, 1.0], [8.0, 1.0, 5.0]]]])
      val=Data(numpy.array([[[2.0, 6.0, 6.0]], [[2.0, 4.0, 7.0]], [[6.0, 8.0, 7.0]]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[[1.0, 8.0, 4.0]], [[4.0, 1.0, 7.0]], [[2.0, 6.0, 6.0]]]))
      arg[2,:,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 2.0], [8.0, 8.0, 2.0], [7.0, 7.0, 7.0]], [[1.0, 7.0, 1.0], [6.0, 8.0, 6.0], [1.0, 
5.0, 5.0]], [[7.0, 5.0, 5.0], [4.0, 5.0, 5.0], [7.0, 7.0, 7.0]]], [[[4.0, 2.0, 3.0], [1.0, 8.0, 2.0], [6.0, 4.0, 6.0]], [[3.0, 
4.0, 7.0], [7.0, 5.0, 5.0], [8.0, 6.0, 8.0]], [[2.0, 3.0, 2.0], [4.0, 3.0, 6.0], [3.0, 4.0, 2.0]]], [[[8.0, 2.0, 8.0], [2.0, 
6.0, 6.0], [7.0, 7.0, 1.0]], [[4.0, 2.0, 7.0], [2.0, 4.0, 7.0], [4.0, 7.0, 1.0]], [[6.0, 4.0, 5.0], [6.0, 8.0, 7.0], [6.0, 6.0, 
4.0]]], [[[8.0, 8.0, 2.0], [6.0, 6.0, 6.0], [5.0, 7.0, 6.0]], [[8.0, 2.0, 1.0], [8.0, 7.0, 6.0], [8.0, 8.0, 1.0]], [[4.0, 8.0, 
2.0], [7.0, 4.0, 4.0], [4.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 7.0], [2.0, 1.0, 3.0], [2.0, 8.0, 6.0]], 
[[4.0, 1.0, 1.0], [4.0, 5.0, 7.0], [8.0, 6.0, 1.0]], [[7.0, 3.0, 2.0], [2.0, 5.0, 7.0], [2.0, 8.0, 5.0]]], [[[7.0, 3.0, 2.0], 
[8.0, 3.0, 4.0], [3.0, 6.0, 6.0]], [[8.0, 7.0, 8.0], [8.0, 7.0, 5.0], [5.0, 1.0, 6.0]], [[6.0, 5.0, 5.0], [2.0, 8.0, 1.0], 
[1.0, 2.0, 8.0]]], [[[8.0, 8.0, 4.0], [1.0, 8.0, 4.0], [6.0, 6.0, 4.0]], [[8.0, 3.0, 4.0], [4.0, 1.0, 7.0], [7.0, 7.0, 4.0]], 
[[1.0, 4.0, 8.0], [2.0, 6.0, 6.0], [4.0, 5.0, 1.0]]], [[[3.0, 2.0, 5.0], [5.0, 4.0, 1.0], [2.0, 5.0, 3.0]], [[4.0, 3.0, 5.0], 
[7.0, 1.0, 2.0], [8.0, 1.0, 6.0]], [[4.0, 8.0, 5.0], [4.0, 3.0, 1.0], [8.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_taggedData_PPRN(self):
      msk_arg=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 7.0], [5.0, 3.0, 2.0], [5.0, 2.0, 5.0]], [[8.0, 1.0, 7.0], [6.0, 8.0, 8.0], [7.0, 
7.0, 8.0]], [[7.0, 8.0, 7.0], [8.0, 1.0, 1.0], [6.0, 7.0, 6.0]]], [[[5.0, 8.0, 3.0], [1.0, 8.0, 4.0], [5.0, 3.0, 8.0]], [[8.0, 
1.0, 7.0], [8.0, 8.0, 6.0], [2.0, 2.0, 5.0]], [[5.0, 4.0, 6.0], [4.0, 1.0, 2.0], [4.0, 7.0, 2.0]]], [[[2.0, 4.0, 4.0], [5.0, 
4.0, 2.0], [4.0, 1.0, 1.0]], [[8.0, 5.0, 1.0], [7.0, 1.0, 4.0], [3.0, 1.0, 4.0]], [[2.0, 5.0, 2.0], [5.0, 6.0, 6.0], [3.0, 5.0, 
8.0]]], [[[6.0, 4.0, 2.0], [1.0, 8.0, 2.0], [4.0, 4.0, 6.0]], [[1.0, 4.0, 4.0], [6.0, 7.0, 5.0], [6.0, 8.0, 2.0]], [[8.0, 1.0, 
4.0], [5.0, 3.0, 5.0], [8.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 1.0], [2.0, 4.0, 5.0], [3.0, 5.0, 5.0]], 
[[4.0, 7.0, 4.0], [2.0, 3.0, 1.0], [3.0, 4.0, 1.0]], [[5.0, 3.0, 3.0], [7.0, 3.0, 7.0], [4.0, 2.0, 3.0]]], [[[7.0, 3.0, 1.0], 
[3.0, 7.0, 5.0], [5.0, 2.0, 8.0]], [[7.0, 8.0, 8.0], [5.0, 2.0, 3.0], [5.0, 7.0, 4.0]], [[3.0, 1.0, 8.0], [6.0, 7.0, 4.0], 
[7.0, 6.0, 8.0]]], [[[1.0, 1.0, 6.0], [2.0, 8.0, 7.0], [4.0, 1.0, 2.0]], [[5.0, 1.0, 4.0], [4.0, 6.0, 2.0], [2.0, 4.0, 7.0]], 
[[4.0, 8.0, 2.0], [4.0, 8.0, 2.0], [3.0, 3.0, 6.0]]], [[[4.0, 2.0, 8.0], [6.0, 4.0, 5.0], [8.0, 8.0, 2.0]], [[6.0, 4.0, 1.0], 
[2.0, 5.0, 5.0], [1.0, 5.0, 8.0]], [[4.0, 7.0, 5.0], [6.0, 4.0, 8.0], [5.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[2.0, 8.0, 6.0]]),self.functionspace)
      val.setTaggedValue(1,numpy.array([[3.0, 2.0, 1.0]]))
      arg[2,1,1:2]=val
      msk_ref=1.-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 7.0], [5.0, 3.0, 2.0], [5.0, 2.0, 5.0]], [[8.0, 1.0, 7.0], [6.0, 8.0, 8.0], [7.0, 
7.0, 8.0]], [[7.0, 8.0, 7.0], [8.0, 1.0, 1.0], [6.0, 7.0, 6.0]]], [[[5.0, 8.0, 3.0], [1.0, 8.0, 4.0], [5.0, 3.0, 8.0]], [[8.0, 
1.0, 7.0], [8.0, 8.0, 6.0], [2.0, 2.0, 5.0]], [[5.0, 4.0, 6.0], [4.0, 1.0, 2.0], [4.0, 7.0, 2.0]]], [[[2.0, 4.0, 4.0], [5.0, 
4.0, 2.0], [4.0, 1.0, 1.0]], [[8.0, 5.0, 1.0], [2.0, 8.0, 6.0], [3.0, 1.0, 4.0]], [[2.0, 5.0, 2.0], [5.0, 6.0, 6.0], [3.0, 5.0, 
8.0]]], [[[6.0, 4.0, 2.0], [1.0, 8.0, 2.0], [4.0, 4.0, 6.0]], [[1.0, 4.0, 4.0], [6.0, 7.0, 5.0], [6.0, 8.0, 2.0]], [[8.0, 1.0, 
4.0], [5.0, 3.0, 5.0], [8.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 1.0], [2.0, 4.0, 5.0], [3.0, 5.0, 5.0]], 
[[4.0, 7.0, 4.0], [2.0, 3.0, 1.0], [3.0, 4.0, 1.0]], [[5.0, 3.0, 3.0], [7.0, 3.0, 7.0], [4.0, 2.0, 3.0]]], [[[7.0, 3.0, 1.0], 
[3.0, 7.0, 5.0], [5.0, 2.0, 8.0]], [[7.0, 8.0, 8.0], [5.0, 2.0, 3.0], [5.0, 7.0, 4.0]], [[3.0, 1.0, 8.0], [6.0, 7.0, 4.0], 
[7.0, 6.0, 8.0]]], [[[1.0, 1.0, 6.0], [2.0, 8.0, 7.0], [4.0, 1.0, 2.0]], [[5.0, 1.0, 4.0], [3.0, 2.0, 1.0], [2.0, 4.0, 7.0]], 
[[4.0, 8.0, 2.0], [4.0, 8.0, 2.0], [3.0, 3.0, 6.0]]], [[[4.0, 2.0, 8.0], [6.0, 4.0, 5.0], [8.0, 8.0, 2.0]], [[6.0, 4.0, 1.0], 
[2.0, 5.0, 5.0], [1.0, 5.0, 8.0]], [[4.0, 7.0, 5.0], [6.0, 4.0, 8.0], [5.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
